# Format is: [Number, 'Name', ['Aliases','List',...], 'SymbolCatagory', 'DirectReplacement']
unicodeConversionTable = [
  [ 0x0, 'UnicodeCharacter0x0', [], 'Ascii', None],
  [ 0x1, 'UnicodeCharacter0x1', [], 'Ascii', None],
  [ 0x2, 'UnicodeCharacter0x2', [], 'Ascii', None],
  [ 0x3, 'UnicodeCharacter0x3', [], 'Ascii', None],
  [ 0x4, 'UnicodeCharacter0x4', [], 'Ascii', None],
  [ 0x5, 'UnicodeCharacter0x5', [], 'Ascii', None],
  [ 0x6, 'UnicodeCharacter0x6', [], 'Ascii', None],
  [ 0x7, 'UnicodeCharacter0x7', [], 'Ascii', None],
  [ 0x8, 'UnicodeCharacter0x8', [], 'Ascii', None],
  [ 0x9, 'UnicodeCharacter0x9', [], 'Ascii', None],
  [ 0xa, 'UnicodeCharacter0xa', [], 'Ascii', None],
  [ 0xb, 'UnicodeCharacter0xb', [], 'Ascii', None],
  [ 0xc, 'UnicodeCharacter0xc', [], 'Ascii', None],
  [ 0xd, 'UnicodeCharacter0xd', [], 'Ascii', None],
  [ 0xe, 'UnicodeCharacter0xe', [], 'Ascii', None],
  [ 0xf, 'UnicodeCharacter0xf', [], 'Ascii', None],
  [ 0x10, 'UnicodeCharacter0x10', [], 'Ascii', None],
  [ 0x11, 'UnicodeCharacter0x11', [], 'Ascii', None],
  [ 0x12, 'UnicodeCharacter0x12', [], 'Ascii', None],
  [ 0x13, 'UnicodeCharacter0x13', [], 'Ascii', None],
  [ 0x14, 'UnicodeCharacter0x14', [], 'Ascii', None],
  [ 0x15, 'UnicodeCharacter0x15', [], 'Ascii', None],
  [ 0x16, 'UnicodeCharacter0x16', [], 'Ascii', None],
  [ 0x17, 'UnicodeCharacter0x17', [], 'Ascii', None],
  [ 0x18, 'UnicodeCharacter0x18', [], 'Ascii', None],
  [ 0x19, 'UnicodeCharacter0x19', [], 'Ascii', None],
  [ 0x1a, 'UnicodeCharacter0x1a', [], 'Ascii', None],
  [ 0x1b, 'UnicodeCharacter0x1b', [], 'Ascii', None],
  [ 0x1c, 'UnicodeCharacter0x1c', [], 'Ascii', None],
  [ 0x1d, 'UnicodeCharacter0x1d', [], 'Ascii', None],
  [ 0x1e, 'UnicodeCharacter0x1e', [], 'Ascii', None],
  [ 0x1f, 'UnicodeCharacter0x1f', [], 'Ascii', None],
  [ 0x20, 'Space', [], 'Ascii', None],
  [ 0x21, 'ExclamationMark', [], 'Ascii', None],
  [ 0x22, 'QuotationMark', [], 'Ascii', None],
  [ 0x23, 'NumberSign', [], 'Ascii', None],
  [ 0x24, 'DollarSign', [], 'Ascii', None],
  [ 0x25, 'PercentSign', [], 'Ascii', None],
  [ 0x26, 'Ampersand', [], 'Ascii', None],
  [ 0x27, 'Apostrophe', [], 'Ascii', None],
  [ 0x28, 'LeftParenthesis', [], 'Ascii', None],
  [ 0x29, 'RightParenthesis', [], 'Ascii', None],
  [ 0x2a, 'Asterisk', [], 'Ascii', None],
  [ 0x2b, 'PlusSign', [], 'Ascii', None],
  [ 0x2c, 'Comma', [], 'Ascii', None],
  [ 0x2d, 'Hyphenminus', [], 'Ascii', None],
  [ 0x2e, 'FullStop', [], 'Ascii', None],
  [ 0x2f, 'Solidus', [], 'Ascii', None],
  [ 0x30, 'DigitZero', [], 'Ascii', None],
  [ 0x31, 'DigitOne', [], 'Ascii', None],
  [ 0x32, 'DigitTwo', [], 'Ascii', None],
  [ 0x33, 'DigitThree', [], 'Ascii', None],
  [ 0x34, 'DigitFour', [], 'Ascii', None],
  [ 0x35, 'DigitFive', [], 'Ascii', None],
  [ 0x36, 'DigitSix', [], 'Ascii', None],
  [ 0x37, 'DigitSeven', [], 'Ascii', None],
  [ 0x38, 'DigitEight', [], 'Ascii', None],
  [ 0x39, 'DigitNine', [], 'Ascii', None],
  [ 0x3a, 'Colon', [], 'Ascii', None],
  [ 0x3b, 'Semicolon', [], 'Ascii', None],
  [ 0x3c, 'LessthanSign', [], 'Ascii', None],
  [ 0x3d, 'EqualsSign', [], 'Ascii', None],
  [ 0x3e, 'GreaterthanSign', [], 'Ascii', None],
  [ 0x3f, 'QuestionMark', [], 'Ascii', None],
  [ 0x40, 'CommercialAt', [], 'Ascii', None],
  [ 0x41, 'LatinCapitalLetterA', [], 'Ascii', None],
  [ 0x42, 'LatinCapitalLetterB', [], 'Ascii', None],
  [ 0x43, 'LatinCapitalLetterC', [], 'Ascii', None],
  [ 0x44, 'LatinCapitalLetterD', [], 'Ascii', None],
  [ 0x45, 'LatinCapitalLetterE', [], 'Ascii', None],
  [ 0x46, 'LatinCapitalLetterF', [], 'Ascii', None],
  [ 0x47, 'LatinCapitalLetterG', [], 'Ascii', None],
  [ 0x48, 'LatinCapitalLetterH', [], 'Ascii', None],
  [ 0x49, 'LatinCapitalLetterI', [], 'Ascii', None],
  [ 0x4a, 'LatinCapitalLetterJ', [], 'Ascii', None],
  [ 0x4b, 'LatinCapitalLetterK', [], 'Ascii', None],
  [ 0x4c, 'LatinCapitalLetterL', [], 'Ascii', None],
  [ 0x4d, 'LatinCapitalLetterM', [], 'Ascii', None],
  [ 0x4e, 'LatinCapitalLetterN', [], 'Ascii', None],
  [ 0x4f, 'LatinCapitalLetterO', [], 'Ascii', None],
  [ 0x50, 'LatinCapitalLetterP', [], 'Ascii', None],
  [ 0x51, 'LatinCapitalLetterQ', [], 'Ascii', None],
  [ 0x52, 'LatinCapitalLetterR', [], 'Ascii', None],
  [ 0x53, 'LatinCapitalLetterS', [], 'Ascii', None],
  [ 0x54, 'LatinCapitalLetterT', [], 'Ascii', None],
  [ 0x55, 'LatinCapitalLetterU', [], 'Ascii', None],
  [ 0x56, 'LatinCapitalLetterV', [], 'Ascii', None],
  [ 0x57, 'LatinCapitalLetterW', [], 'Ascii', None],
  [ 0x58, 'LatinCapitalLetterX', [], 'Ascii', None],
  [ 0x59, 'LatinCapitalLetterY', [], 'Ascii', None],
  [ 0x5a, 'LatinCapitalLetterZ', [], 'Ascii', None],
  [ 0x5b, 'LeftSquareBracket', [], 'Ascii', None],
  [ 0x5c, 'ReverseSolidus', [], 'Ascii', None],
  [ 0x5d, 'RightSquareBracket', [], 'Ascii', None],
  [ 0x5e, 'CircumflexAccent', [], 'Ascii', None],
  [ 0x5f, 'LowLine', [], 'Ascii', None],
  [ 0x60, 'GraveAccent', [], 'Ascii', None],
  [ 0x61, 'LatinSmallLetterA', [], 'Ascii', None],
  [ 0x62, 'LatinSmallLetterB', [], 'Ascii', None],
  [ 0x63, 'LatinSmallLetterC', [], 'Ascii', None],
  [ 0x64, 'LatinSmallLetterD', [], 'Ascii', None],
  [ 0x65, 'LatinSmallLetterE', [], 'Ascii', None],
  [ 0x66, 'LatinSmallLetterF', [], 'Ascii', None],
  [ 0x67, 'LatinSmallLetterG', [], 'Ascii', None],
  [ 0x68, 'LatinSmallLetterH', [], 'Ascii', None],
  [ 0x69, 'LatinSmallLetterI', [], 'Ascii', None],
  [ 0x6a, 'LatinSmallLetterJ', [], 'Ascii', None],
  [ 0x6b, 'LatinSmallLetterK', [], 'Ascii', None],
  [ 0x6c, 'LatinSmallLetterL', [], 'Ascii', None],
  [ 0x6d, 'LatinSmallLetterM', [], 'Ascii', None],
  [ 0x6e, 'LatinSmallLetterN', [], 'Ascii', None],
  [ 0x6f, 'LatinSmallLetterO', [], 'Ascii', None],
  [ 0x70, 'LatinSmallLetterP', [], 'Ascii', None],
  [ 0x71, 'LatinSmallLetterQ', [], 'Ascii', None],
  [ 0x72, 'LatinSmallLetterR', [], 'Ascii', None],
  [ 0x73, 'LatinSmallLetterS', [], 'Ascii', None],
  [ 0x74, 'LatinSmallLetterT', [], 'Ascii', None],
  [ 0x75, 'LatinSmallLetterU', [], 'Ascii', None],
  [ 0x76, 'LatinSmallLetterV', [], 'Ascii', None],
  [ 0x77, 'LatinSmallLetterW', [], 'Ascii', None],
  [ 0x78, 'LatinSmallLetterX', [], 'Ascii', None],
  [ 0x79, 'LatinSmallLetterY', [], 'Ascii', None],
  [ 0x7a, 'LatinSmallLetterZ', [], 'Ascii', None],
  [ 0x7b, 'LeftCurlyBracket', [], 'Ascii', None],
  [ 0x7c, 'VerticalLine', [], 'Ascii', None],
  [ 0x7d, 'RightCurlyBracket', [], 'Ascii', None],
  [ 0x7e, 'Tilde', [], 'Ascii', None],
  [ 0x7f, 'UnicodeCharacter0x7f', [], 'Ascii', None],
  [ 0x80, 'UnicodeCharacter0x80', [], 'Display', None],
  [ 0x81, 'UnicodeCharacter0x81', [], 'Display', None],
  [ 0x82, 'UnicodeCharacter0x82', [], 'Display', None],
  [ 0x83, 'UnicodeCharacter0x83', [], 'Display', None],
  [ 0x84, 'UnicodeCharacter0x84', [], 'Display', None],
  [ 0x85, 'UnicodeCharacter0x85', [], 'Display', None],
  [ 0x86, 'UnicodeCharacter0x86', [], 'Display', None],
  [ 0x87, 'UnicodeCharacter0x87', [], 'Display', None],
  [ 0x88, 'UnicodeCharacter0x88', [], 'Display', None],
  [ 0x89, 'UnicodeCharacter0x89', [], 'Display', None],
  [ 0x8a, 'UnicodeCharacter0x8a', [], 'Display', None],
  [ 0x8b, 'UnicodeCharacter0x8b', [], 'Display', None],
  [ 0x8c, 'UnicodeCharacter0x8c', [], 'Display', None],
  [ 0x8d, 'UnicodeCharacter0x8d', [], 'Display', None],
  [ 0x8e, 'UnicodeCharacter0x8e', [], 'Display', None],
  [ 0x8f, 'UnicodeCharacter0x8f', [], 'Display', None],
  [ 0x90, 'UnicodeCharacter0x90', [], 'Display', None],
  [ 0x91, 'UnicodeCharacter0x91', [], 'Display', None],
  [ 0x92, 'UnicodeCharacter0x92', [], 'Display', None],
  [ 0x93, 'UnicodeCharacter0x93', [], 'Display', None],
  [ 0x94, 'UnicodeCharacter0x94', [], 'Display', None],
  [ 0x95, 'UnicodeCharacter0x95', [], 'Display', None],
  [ 0x96, 'UnicodeCharacter0x96', [], 'Display', None],
  [ 0x97, 'UnicodeCharacter0x97', [], 'Display', None],
  [ 0x98, 'UnicodeCharacter0x98', [], 'Display', None],
  [ 0x99, 'UnicodeCharacter0x99', [], 'Display', None],
  [ 0x9a, 'UnicodeCharacter0x9a', [], 'Display', None],
  [ 0x9b, 'UnicodeCharacter0x9b', [], 'Display', None],
  [ 0x9c, 'UnicodeCharacter0x9c', [], 'Display', None],
  [ 0x9d, 'UnicodeCharacter0x9d', [], 'Display', None],
  [ 0x9e, 'UnicodeCharacter0x9e', [], 'Display', None],
  [ 0x9f, 'UnicodeCharacter0x9f', [], 'Display', None],
  [ 0xa0, 'NobreakSpace', [], 'Display', None],
  [ 0xa1, 'InvertedExclamationMark', [], 'Display', None],
  [ 0xa2, 'CentSign', [], 'Display', None],
  [ 0xa3, 'PoundSign', [], 'Display', None],
  [ 0xa4, 'CurrencySign', [], 'Display', None],
  [ 0xa5, 'YenSign', [], 'Display', None],
  [ 0xa6, 'BrokenBar', [], 'Display', None],
  [ 0xa7, 'SectionSign', [], 'Display', None],
  [ 0xa8, 'Diaeresis', [], 'Display', None],
  [ 0xa9, 'CopyrightSign', [], 'Display', None],
  [ 0xaa, 'FeminineOrdinalIndicator', [], 'Display', None],
  [ 0xab, 'LeftpointingDoubleAngleQuotationMark', [], 'Display', None],
  [ 0xac, 'NotSign', ['not','logicalnot','negation','neg'], 'SimpleSubstitution', ' not '],
  [ 0xad, 'SoftHyphen', [], 'Display', None],
  [ 0xae, 'RegisteredSign', [], 'Display', None],
  [ 0xaf, 'Macron', [], 'Display', None],
  [ 0xb0, 'DegreeSign', ['deg','degree'], 'Display', None],
  [ 0xb1, 'PlusminusSign', ['+-','+/-','plusminus'], 'Display', None],
  [ 0xb2, 'SuperscriptTwo', ['squared','^2','**2'], 'SimpleSubstitution', '**2'],
  [ 0xb3, 'SuperscriptThree', ['cubed','^3','**3'], 'SimpleSubstitution', '**3'],
  [ 0xb4, 'AcuteAccent', [], 'Display', None],
  [ 0xb5, 'MicroSign', [], 'Display', None],
  [ 0xb6, 'PilcrowSign', [], 'Display', None],
  [ 0xb7, 'MiddleDot', [], 'Display', None],
  [ 0xb8, 'Cedilla', [], 'Display', None],
  [ 0xb9, 'SuperscriptOne', ['^1','**1'], 'SimpleSubstitution', '**1'],
  [ 0xba, 'MasculineOrdinalIndicator', [], 'Display', None],
  [ 0xbb, 'RightpointingDoubleAngleQuotationMark', [], 'Display', None],
  [ 0xbc, 'VulgarFractionOneQuarter', [], 'Display', None],
  [ 0xbd, 'VulgarFractionOneHalf', [], 'Display', None],
  [ 0xbe, 'VulgarFractionThreeQuarters', [], 'Display', None],
  [ 0xbf, 'InvertedQuestionMark', [], 'Display', None],
  [ 0xc0, 'LatinCapitalLetterAWithGrave', [], 'Display', None],
  [ 0xc1, 'LatinCapitalLetterAWithAcute', [], 'Display', None],
  [ 0xc2, 'LatinCapitalLetterAWithCircumflex', [], 'Display', None],
  [ 0xc3, 'LatinCapitalLetterAWithTilde', [], 'Display', None],
  [ 0xc4, 'LatinCapitalLetterAWithDiaeresis', [], 'Display', None],
  [ 0xc5, 'LatinCapitalLetterAWithRingAbove', [], 'Display', None],
  [ 0xc6, 'LatinCapitalLetterAe', [], 'Display', None],
  [ 0xc7, 'LatinCapitalLetterCWithCedilla', [], 'Display', None],
  [ 0xc8, 'LatinCapitalLetterEWithGrave', [], 'Display', None],
  [ 0xc9, 'LatinCapitalLetterEWithAcute', [], 'Display', None],
  [ 0xca, 'LatinCapitalLetterEWithCircumflex', [], 'Display', None],
  [ 0xcb, 'LatinCapitalLetterEWithDiaeresis', [], 'Display', None],
  [ 0xcc, 'LatinCapitalLetterIWithGrave', [], 'Display', None],
  [ 0xcd, 'LatinCapitalLetterIWithAcute', [], 'Display', None],
  [ 0xce, 'LatinCapitalLetterIWithCircumflex', [], 'Display', None],
  [ 0xcf, 'LatinCapitalLetterIWithDiaeresis', [], 'Display', None],
  [ 0xd0, 'LatinCapitalLetterEth', [], 'Display', None],
  [ 0xd1, 'LatinCapitalLetterNWithTilde', [], 'Display', None],
  [ 0xd2, 'LatinCapitalLetterOWithGrave', [], 'Display', None],
  [ 0xd3, 'LatinCapitalLetterOWithAcute', [], 'Display', None],
  [ 0xd4, 'LatinCapitalLetterOWithCircumflex', [], 'Display', None],
  [ 0xd5, 'LatinCapitalLetterOWithTilde', [], 'Display', None],
  [ 0xd6, 'LatinCapitalLetterOWithDiaeresis', [], 'Display', None],
  [ 0xd7, 'MultiplicationSign', ['mult','multiply'], 'InfixOperator', '*'],
  [ 0xd8, 'LatinCapitalLetterOWithStroke', [], 'Display', None],
  [ 0xd9, 'LatinCapitalLetterUWithGrave', [], 'Display', None],
  [ 0xda, 'LatinCapitalLetterUWithAcute', [], 'Display', None],
  [ 0xdb, 'LatinCapitalLetterUWithCircumflex', [], 'Display', None],
  [ 0xdc, 'LatinCapitalLetterUWithDiaeresis', [], 'Display', None],
  [ 0xdd, 'LatinCapitalLetterYWithAcute', [], 'Display', None],
  [ 0xde, 'LatinCapitalLetterThorn', [], 'Display', None],
  [ 0xdf, 'LatinSmallLetterSharpS', [], 'Display', None],
  [ 0xe0, 'LatinSmallLetterAWithGrave', [], 'Display', None],
  [ 0xe1, 'LatinSmallLetterAWithAcute', [], 'Display', None],
  [ 0xe2, 'LatinSmallLetterAWithCircumflex', [], 'Display', None],
  [ 0xe3, 'LatinSmallLetterAWithTilde', [], 'Display', None],
  [ 0xe4, 'LatinSmallLetterAWithDiaeresis', [], 'Display', None],
  [ 0xe5, 'LatinSmallLetterAWithRingAbove', [], 'Display', None],
  [ 0xe6, 'LatinSmallLetterAe', [], 'Display', None],
  [ 0xe7, 'LatinSmallLetterCWithCedilla', [], 'Display', None],
  [ 0xe8, 'LatinSmallLetterEWithGrave', [], 'Display', None],
  [ 0xe9, 'LatinSmallLetterEWithAcute', [], 'Display', None],
  [ 0xea, 'LatinSmallLetterEWithCircumflex', [], 'Display', None],
  [ 0xeb, 'LatinSmallLetterEWithDiaeresis', [], 'Display', None],
  [ 0xec, 'LatinSmallLetterIWithGrave', [], 'Display', None],
  [ 0xed, 'LatinSmallLetterIWithAcute', [], 'Display', None],
  [ 0xee, 'LatinSmallLetterIWithCircumflex', [], 'Display', None],
  [ 0xef, 'LatinSmallLetterIWithDiaeresis', [], 'Display', None],
  [ 0xf0, 'LatinSmallLetterEth', [], 'Display', None],
  [ 0xf1, 'LatinSmallLetterNWithTilde', [], 'Display', None],
  [ 0xf2, 'LatinSmallLetterOWithGrave', [], 'Display', None],
  [ 0xf3, 'LatinSmallLetterOWithAcute', [], 'Display', None],
  [ 0xf4, 'LatinSmallLetterOWithCircumflex', [], 'Display', None],
  [ 0xf5, 'LatinSmallLetterOWithTilde', [], 'Display', None],
  [ 0xf6, 'LatinSmallLetterOWithDiaeresis', [], 'Display', None],
  [ 0xf7, 'DivisionSign', ['div','division','divisionsign'], 'InfixOperator', '*'],
  [ 0xf8, 'LatinSmallLetterOWithStroke', [], 'Display', None],
  [ 0xf9, 'LatinSmallLetterUWithGrave', [], 'Display', None],
  [ 0xfa, 'LatinSmallLetterUWithAcute', [], 'Display', None],
  [ 0xfb, 'LatinSmallLetterUWithCircumflex', [], 'Display', None],
  [ 0xfc, 'LatinSmallLetterUWithDiaeresis', [], 'Display', None],
  [ 0xfd, 'LatinSmallLetterYWithAcute', [], 'Display', None],
  [ 0xfe, 'LatinSmallLetterThorn', [], 'Display', None],
  [ 0xff, 'LatinSmallLetterYWithDiaeresis', [], 'Display', None],
  [ 0x100, 'LatinCapitalLetterAWithMacron', [], 'Display', None],
  [ 0x101, 'LatinSmallLetterAWithMacron', [], 'Display', None],
  [ 0x102, 'LatinCapitalLetterAWithBreve', [], 'Display', None],
  [ 0x103, 'LatinSmallLetterAWithBreve', [], 'Display', None],
  [ 0x104, 'LatinCapitalLetterAWithOgonek', [], 'Display', None],
  [ 0x105, 'LatinSmallLetterAWithOgonek', [], 'Display', None],
  [ 0x106, 'LatinCapitalLetterCWithAcute', [], 'Display', None],
  [ 0x107, 'LatinSmallLetterCWithAcute', [], 'Display', None],
  [ 0x108, 'LatinCapitalLetterCWithCircumflex', [], 'Display', None],
  [ 0x109, 'LatinSmallLetterCWithCircumflex', [], 'Display', None],
  [ 0x10a, 'LatinCapitalLetterCWithDotAbove', [], 'Display', None],
  [ 0x10b, 'LatinSmallLetterCWithDotAbove', [], 'Display', None],
  [ 0x10c, 'LatinCapitalLetterCWithCaron', [], 'Display', None],
  [ 0x10d, 'LatinSmallLetterCWithCaron', [], 'Display', None],
  [ 0x10e, 'LatinCapitalLetterDWithCaron', [], 'Display', None],
  [ 0x10f, 'LatinSmallLetterDWithCaron', [], 'Display', None],
  [ 0x110, 'LatinCapitalLetterDWithStroke', [], 'Display', None],
  [ 0x111, 'LatinSmallLetterDWithStroke', [], 'Display', None],
  [ 0x112, 'LatinCapitalLetterEWithMacron', [], 'Display', None],
  [ 0x113, 'LatinSmallLetterEWithMacron', [], 'Display', None],
  [ 0x114, 'LatinCapitalLetterEWithBreve', [], 'Display', None],
  [ 0x115, 'LatinSmallLetterEWithBreve', [], 'Display', None],
  [ 0x116, 'LatinCapitalLetterEWithDotAbove', [], 'Display', None],
  [ 0x117, 'LatinSmallLetterEWithDotAbove', [], 'Display', None],
  [ 0x118, 'LatinCapitalLetterEWithOgonek', [], 'Display', None],
  [ 0x119, 'LatinSmallLetterEWithOgonek', [], 'Display', None],
  [ 0x11a, 'LatinCapitalLetterEWithCaron', [], 'Display', None],
  [ 0x11b, 'LatinSmallLetterEWithCaron', [], 'Display', None],
  [ 0x11c, 'LatinCapitalLetterGWithCircumflex', [], 'Display', None],
  [ 0x11d, 'LatinSmallLetterGWithCircumflex', [], 'Display', None],
  [ 0x11e, 'LatinCapitalLetterGWithBreve', [], 'Display', None],
  [ 0x11f, 'LatinSmallLetterGWithBreve', [], 'Display', None],
  [ 0x120, 'LatinCapitalLetterGWithDotAbove', [], 'Display', None],
  [ 0x121, 'LatinSmallLetterGWithDotAbove', [], 'Display', None],
  [ 0x122, 'LatinCapitalLetterGWithCedilla', [], 'Display', None],
  [ 0x123, 'LatinSmallLetterGWithCedilla', [], 'Display', None],
  [ 0x124, 'LatinCapitalLetterHWithCircumflex', [], 'Display', None],
  [ 0x125, 'LatinSmallLetterHWithCircumflex', [], 'Display', None],
  [ 0x126, 'LatinCapitalLetterHWithStroke', [], 'Display', None],
  [ 0x127, 'LatinSmallLetterHWithStroke', [], 'Display', None],
  [ 0x128, 'LatinCapitalLetterIWithTilde', [], 'Display', None],
  [ 0x129, 'LatinSmallLetterIWithTilde', [], 'Display', None],
  [ 0x12a, 'LatinCapitalLetterIWithMacron', [], 'Display', None],
  [ 0x12b, 'LatinSmallLetterIWithMacron', [], 'Display', None],
  [ 0x12c, 'LatinCapitalLetterIWithBreve', [], 'Display', None],
  [ 0x12d, 'LatinSmallLetterIWithBreve', [], 'Display', None],
  [ 0x12e, 'LatinCapitalLetterIWithOgonek', [], 'Display', None],
  [ 0x12f, 'LatinSmallLetterIWithOgonek', [], 'Display', None],
  [ 0x130, 'LatinCapitalLetterIWithDotAbove', [], 'Display', None],
  [ 0x131, 'LatinSmallLetterDotlessI', [], 'Display', None],
  [ 0x132, 'LatinCapitalLigatureIj', [], 'Display', None],
  [ 0x133, 'LatinSmallLigatureIj', [], 'Display', None],
  [ 0x134, 'LatinCapitalLetterJWithCircumflex', [], 'Display', None],
  [ 0x135, 'LatinSmallLetterJWithCircumflex', [], 'Display', None],
  [ 0x136, 'LatinCapitalLetterKWithCedilla', [], 'Display', None],
  [ 0x137, 'LatinSmallLetterKWithCedilla', [], 'Display', None],
  [ 0x138, 'LatinSmallLetterKra', [], 'Display', None],
  [ 0x139, 'LatinCapitalLetterLWithAcute', [], 'Display', None],
  [ 0x13a, 'LatinSmallLetterLWithAcute', [], 'Display', None],
  [ 0x13b, 'LatinCapitalLetterLWithCedilla', [], 'Display', None],
  [ 0x13c, 'LatinSmallLetterLWithCedilla', [], 'Display', None],
  [ 0x13d, 'LatinCapitalLetterLWithCaron', [], 'Display', None],
  [ 0x13e, 'LatinSmallLetterLWithCaron', [], 'Display', None],
  [ 0x13f, 'LatinCapitalLetterLWithMiddleDot', [], 'Display', None],
  [ 0x140, 'LatinSmallLetterLWithMiddleDot', [], 'Display', None],
  [ 0x141, 'LatinCapitalLetterLWithStroke', [], 'Display', None],
  [ 0x142, 'LatinSmallLetterLWithStroke', [], 'Display', None],
  [ 0x143, 'LatinCapitalLetterNWithAcute', [], 'Display', None],
  [ 0x144, 'LatinSmallLetterNWithAcute', [], 'Display', None],
  [ 0x145, 'LatinCapitalLetterNWithCedilla', [], 'Display', None],
  [ 0x146, 'LatinSmallLetterNWithCedilla', [], 'Display', None],
  [ 0x147, 'LatinCapitalLetterNWithCaron', [], 'Display', None],
  [ 0x148, 'LatinSmallLetterNWithCaron', [], 'Display', None],
  [ 0x149, 'LatinSmallLetterNPrecededByApostrophe', [], 'Display', None],
  [ 0x14a, 'LatinCapitalLetterEng', [], 'Display', None],
  [ 0x14b, 'LatinSmallLetterEng', [], 'Display', None],
  [ 0x14c, 'LatinCapitalLetterOWithMacron', [], 'Display', None],
  [ 0x14d, 'LatinSmallLetterOWithMacron', [], 'Display', None],
  [ 0x14e, 'LatinCapitalLetterOWithBreve', [], 'Display', None],
  [ 0x14f, 'LatinSmallLetterOWithBreve', [], 'Display', None],
  [ 0x150, 'LatinCapitalLetterOWithDoubleAcute', [], 'Display', None],
  [ 0x151, 'LatinSmallLetterOWithDoubleAcute', [], 'Display', None],
  [ 0x152, 'LatinCapitalLigatureOe', [], 'Display', None],
  [ 0x153, 'LatinSmallLigatureOe', [], 'Display', None],
  [ 0x154, 'LatinCapitalLetterRWithAcute', [], 'Display', None],
  [ 0x155, 'LatinSmallLetterRWithAcute', [], 'Display', None],
  [ 0x156, 'LatinCapitalLetterRWithCedilla', [], 'Display', None],
  [ 0x157, 'LatinSmallLetterRWithCedilla', [], 'Display', None],
  [ 0x158, 'LatinCapitalLetterRWithCaron', [], 'Display', None],
  [ 0x159, 'LatinSmallLetterRWithCaron', [], 'Display', None],
  [ 0x15a, 'LatinCapitalLetterSWithAcute', [], 'Display', None],
  [ 0x15b, 'LatinSmallLetterSWithAcute', [], 'Display', None],
  [ 0x15c, 'LatinCapitalLetterSWithCircumflex', [], 'Display', None],
  [ 0x15d, 'LatinSmallLetterSWithCircumflex', [], 'Display', None],
  [ 0x15e, 'LatinCapitalLetterSWithCedilla', [], 'Display', None],
  [ 0x15f, 'LatinSmallLetterSWithCedilla', [], 'Display', None],
  [ 0x160, 'LatinCapitalLetterSWithCaron', [], 'Display', None],
  [ 0x161, 'LatinSmallLetterSWithCaron', [], 'Display', None],
  [ 0x162, 'LatinCapitalLetterTWithCedilla', [], 'Display', None],
  [ 0x163, 'LatinSmallLetterTWithCedilla', [], 'Display', None],
  [ 0x164, 'LatinCapitalLetterTWithCaron', [], 'Display', None],
  [ 0x165, 'LatinSmallLetterTWithCaron', [], 'Display', None],
  [ 0x166, 'LatinCapitalLetterTWithStroke', [], 'Display', None],
  [ 0x167, 'LatinSmallLetterTWithStroke', [], 'Display', None],
  [ 0x168, 'LatinCapitalLetterUWithTilde', [], 'Display', None],
  [ 0x169, 'LatinSmallLetterUWithTilde', [], 'Display', None],
  [ 0x16a, 'LatinCapitalLetterUWithMacron', [], 'Display', None],
  [ 0x16b, 'LatinSmallLetterUWithMacron', [], 'Display', None],
  [ 0x16c, 'LatinCapitalLetterUWithBreve', [], 'Display', None],
  [ 0x16d, 'LatinSmallLetterUWithBreve', [], 'Display', None],
  [ 0x16e, 'LatinCapitalLetterUWithRingAbove', [], 'Display', None],
  [ 0x16f, 'LatinSmallLetterUWithRingAbove', [], 'Display', None],
  [ 0x170, 'LatinCapitalLetterUWithDoubleAcute', [], 'Display', None],
  [ 0x171, 'LatinSmallLetterUWithDoubleAcute', [], 'Display', None],
  [ 0x172, 'LatinCapitalLetterUWithOgonek', [], 'Display', None],
  [ 0x173, 'LatinSmallLetterUWithOgonek', [], 'Display', None],
  [ 0x174, 'LatinCapitalLetterWWithCircumflex', [], 'Display', None],
  [ 0x175, 'LatinSmallLetterWWithCircumflex', [], 'Display', None],
  [ 0x176, 'LatinCapitalLetterYWithCircumflex', [], 'Display', None],
  [ 0x177, 'LatinSmallLetterYWithCircumflex', [], 'Display', None],
  [ 0x178, 'LatinCapitalLetterYWithDiaeresis', [], 'Display', None],
  [ 0x179, 'LatinCapitalLetterZWithAcute', [], 'Display', None],
  [ 0x17a, 'LatinSmallLetterZWithAcute', [], 'Display', None],
  [ 0x17b, 'LatinCapitalLetterZWithDotAbove', [], 'Display', None],
  [ 0x17c, 'LatinSmallLetterZWithDotAbove', [], 'Display', None],
  [ 0x17d, 'LatinCapitalLetterZWithCaron', [], 'Display', None],
  [ 0x17e, 'LatinSmallLetterZWithCaron', [], 'Display', None],
  [ 0x17f, 'LatinSmallLetterLongS', [], 'Display', None],
  [ 0x180, 'LatinSmallLetterBWithStroke', [], 'Display', None],
  [ 0x181, 'LatinCapitalLetterBWithHook', [], 'Display', None],
  [ 0x182, 'LatinCapitalLetterBWithTopbar', [], 'Display', None],
  [ 0x183, 'LatinSmallLetterBWithTopbar', [], 'Display', None],
  [ 0x184, 'LatinCapitalLetterToneSix', [], 'Display', None],
  [ 0x185, 'LatinSmallLetterToneSix', [], 'Display', None],
  [ 0x186, 'LatinCapitalLetterOpenO', [], 'Display', None],
  [ 0x187, 'LatinCapitalLetterCWithHook', [], 'Display', None],
  [ 0x188, 'LatinSmallLetterCWithHook', [], 'Display', None],
  [ 0x189, 'LatinCapitalLetterAfricanD', [], 'Display', None],
  [ 0x18a, 'LatinCapitalLetterDWithHook', [], 'Display', None],
  [ 0x18b, 'LatinCapitalLetterDWithTopbar', [], 'Display', None],
  [ 0x18c, 'LatinSmallLetterDWithTopbar', [], 'Display', None],
  [ 0x18d, 'LatinSmallLetterTurnedDelta', [], 'Display', None],
  [ 0x18e, 'LatinCapitalLetterReversedE', [], 'Display', None],
  [ 0x18f, 'LatinCapitalLetterSchwa', [], 'Display', None],
  [ 0x190, 'LatinCapitalLetterOpenE', [], 'Display', None],
  [ 0x191, 'LatinCapitalLetterFWithHook', [], 'Display', None],
  [ 0x192, 'LatinSmallLetterFWithHook', [], 'Display', None],
  [ 0x193, 'LatinCapitalLetterGWithHook', [], 'Display', None],
  [ 0x194, 'LatinCapitalLetterGamma', [], 'Display', None],
  [ 0x195, 'LatinSmallLetterHv', [], 'Display', None],
  [ 0x196, 'LatinCapitalLetterIota', [], 'Display', None],
  [ 0x197, 'LatinCapitalLetterIWithStroke', [], 'Display', None],
  [ 0x198, 'LatinCapitalLetterKWithHook', [], 'Display', None],
  [ 0x199, 'LatinSmallLetterKWithHook', [], 'Display', None],
  [ 0x19a, 'LatinSmallLetterLWithBar', [], 'Display', None],
  [ 0x19b, 'LatinSmallLetterLambdaWithStroke', [], 'Display', None],
  [ 0x19c, 'LatinCapitalLetterTurnedM', [], 'Display', None],
  [ 0x19d, 'LatinCapitalLetterNWithLeftHook', [], 'Display', None],
  [ 0x19e, 'LatinSmallLetterNWithLongRightLeg', [], 'Display', None],
  [ 0x19f, 'LatinCapitalLetterOWithMiddleTilde', [], 'Display', None],
  [ 0x1a0, 'LatinCapitalLetterOWithHorn', [], 'Display', None],
  [ 0x1a1, 'LatinSmallLetterOWithHorn', [], 'Display', None],
  [ 0x1a2, 'LatinCapitalLetterOi', [], 'Display', None],
  [ 0x1a3, 'LatinSmallLetterOi', [], 'Display', None],
  [ 0x1a4, 'LatinCapitalLetterPWithHook', [], 'Display', None],
  [ 0x1a5, 'LatinSmallLetterPWithHook', [], 'Display', None],
  [ 0x1a6, 'LatinLetterYr', [], 'Display', None],
  [ 0x1a7, 'LatinCapitalLetterToneTwo', [], 'Display', None],
  [ 0x1a8, 'LatinSmallLetterToneTwo', [], 'Display', None],
  [ 0x1a9, 'LatinCapitalLetterEsh', [], 'Display', None],
  [ 0x1aa, 'LatinLetterReversedEshLoop', [], 'Display', None],
  [ 0x1ab, 'LatinSmallLetterTWithPalatalHook', [], 'Display', None],
  [ 0x1ac, 'LatinCapitalLetterTWithHook', [], 'Display', None],
  [ 0x1ad, 'LatinSmallLetterTWithHook', [], 'Display', None],
  [ 0x1ae, 'LatinCapitalLetterTWithRetroflexHook', [], 'Display', None],
  [ 0x1af, 'LatinCapitalLetterUWithHorn', [], 'Display', None],
  [ 0x1b0, 'LatinSmallLetterUWithHorn', [], 'Display', None],
  [ 0x1b1, 'LatinCapitalLetterUpsilon', [], 'Display', None],
  [ 0x1b2, 'LatinCapitalLetterVWithHook', [], 'Display', None],
  [ 0x1b3, 'LatinCapitalLetterYWithHook', [], 'Display', None],
  [ 0x1b4, 'LatinSmallLetterYWithHook', [], 'Display', None],
  [ 0x1b5, 'LatinCapitalLetterZWithStroke', [], 'Display', None],
  [ 0x1b6, 'LatinSmallLetterZWithStroke', [], 'Display', None],
  [ 0x1b7, 'LatinCapitalLetterEzh', [], 'Display', None],
  [ 0x1b8, 'LatinCapitalLetterEzhReversed', [], 'Display', None],
  [ 0x1b9, 'LatinSmallLetterEzhReversed', [], 'Display', None],
  [ 0x1ba, 'LatinSmallLetterEzhWithTail', [], 'Display', None],
  [ 0x1bb, 'LatinLetterTwoWithStroke', [], 'Display', None],
  [ 0x1bc, 'LatinCapitalLetterToneFive', [], 'Display', None],
  [ 0x1bd, 'LatinSmallLetterToneFive', [], 'Display', None],
  [ 0x1be, 'LatinLetterInvertedGlottalStopWithStroke', [], 'Display', None],
  [ 0x1bf, 'LatinLetterWynn', [], 'Display', None],
  [ 0x1c0, 'LatinLetterDentalClick', [], 'Display', None],
  [ 0x1c1, 'LatinLetterLateralClick', [], 'Display', None],
  [ 0x1c2, 'LatinLetterAlveolarClick', [], 'Display', None],
  [ 0x1c3, 'LatinLetterRetroflexClick', [], 'Display', None],
  [ 0x1c4, 'LatinCapitalLetterDzWithCaron', [], 'Display', None],
  [ 0x1c5, 'LatinCapitalLetterDWithSmallLetterZWithCaron', [], 'Display', None],
  [ 0x1c6, 'LatinSmallLetterDzWithCaron', [], 'Display', None],
  [ 0x1c7, 'LatinCapitalLetterLj', [], 'Display', None],
  [ 0x1c8, 'LatinCapitalLetterLWithSmallLetterJ', [], 'Display', None],
  [ 0x1c9, 'LatinSmallLetterLj', [], 'Display', None],
  [ 0x1ca, 'LatinCapitalLetterNj', [], 'Display', None],
  [ 0x1cb, 'LatinCapitalLetterNWithSmallLetterJ', [], 'Display', None],
  [ 0x1cc, 'LatinSmallLetterNj', [], 'Display', None],
  [ 0x1cd, 'LatinCapitalLetterAWithCaron', [], 'Display', None],
  [ 0x1ce, 'LatinSmallLetterAWithCaron', [], 'Display', None],
  [ 0x1cf, 'LatinCapitalLetterIWithCaron', [], 'Display', None],
  [ 0x1d0, 'LatinSmallLetterIWithCaron', [], 'Display', None],
  [ 0x1d1, 'LatinCapitalLetterOWithCaron', [], 'Display', None],
  [ 0x1d2, 'LatinSmallLetterOWithCaron', [], 'Display', None],
  [ 0x1d3, 'LatinCapitalLetterUWithCaron', [], 'Display', None],
  [ 0x1d4, 'LatinSmallLetterUWithCaron', [], 'Display', None],
  [ 0x1d5, 'LatinCapitalLetterUWithDiaeresisAndMacron', [], 'Display', None],
  [ 0x1d6, 'LatinSmallLetterUWithDiaeresisAndMacron', [], 'Display', None],
  [ 0x1d7, 'LatinCapitalLetterUWithDiaeresisAndAcute', [], 'Display', None],
  [ 0x1d8, 'LatinSmallLetterUWithDiaeresisAndAcute', [], 'Display', None],
  [ 0x1d9, 'LatinCapitalLetterUWithDiaeresisAndCaron', [], 'Display', None],
  [ 0x1da, 'LatinSmallLetterUWithDiaeresisAndCaron', [], 'Display', None],
  [ 0x1db, 'LatinCapitalLetterUWithDiaeresisAndGrave', [], 'Display', None],
  [ 0x1dc, 'LatinSmallLetterUWithDiaeresisAndGrave', [], 'Display', None],
  [ 0x1dd, 'LatinSmallLetterTurnedE', [], 'Display', None],
  [ 0x1de, 'LatinCapitalLetterAWithDiaeresisAndMacron', [], 'Display', None],
  [ 0x1df, 'LatinSmallLetterAWithDiaeresisAndMacron', [], 'Display', None],
  [ 0x1e0, 'LatinCapitalLetterAWithDotAboveAndMacron', [], 'Display', None],
  [ 0x1e1, 'LatinSmallLetterAWithDotAboveAndMacron', [], 'Display', None],
  [ 0x1e2, 'LatinCapitalLetterAeWithMacron', [], 'Display', None],
  [ 0x1e3, 'LatinSmallLetterAeWithMacron', [], 'Display', None],
  [ 0x1e4, 'LatinCapitalLetterGWithStroke', [], 'Display', None],
  [ 0x1e5, 'LatinSmallLetterGWithStroke', [], 'Display', None],
  [ 0x1e6, 'LatinCapitalLetterGWithCaron', [], 'Display', None],
  [ 0x1e7, 'LatinSmallLetterGWithCaron', [], 'Display', None],
  [ 0x1e8, 'LatinCapitalLetterKWithCaron', [], 'Display', None],
  [ 0x1e9, 'LatinSmallLetterKWithCaron', [], 'Display', None],
  [ 0x1ea, 'LatinCapitalLetterOWithOgonek', [], 'Display', None],
  [ 0x1eb, 'LatinSmallLetterOWithOgonek', [], 'Display', None],
  [ 0x1ec, 'LatinCapitalLetterOWithOgonekAndMacron', [], 'Display', None],
  [ 0x1ed, 'LatinSmallLetterOWithOgonekAndMacron', [], 'Display', None],
  [ 0x1ee, 'LatinCapitalLetterEzhWithCaron', [], 'Display', None],
  [ 0x1ef, 'LatinSmallLetterEzhWithCaron', [], 'Display', None],
  [ 0x1f0, 'LatinSmallLetterJWithCaron', [], 'Display', None],
  [ 0x1f1, 'LatinCapitalLetterDz', [], 'Display', None],
  [ 0x1f2, 'LatinCapitalLetterDWithSmallLetterZ', [], 'Display', None],
  [ 0x1f3, 'LatinSmallLetterDz', [], 'Display', None],
  [ 0x1f4, 'LatinCapitalLetterGWithAcute', [], 'Display', None],
  [ 0x1f5, 'LatinSmallLetterGWithAcute', [], 'Display', None],
  [ 0x1f6, 'LatinCapitalLetterHwair', [], 'Display', None],
  [ 0x1f7, 'LatinCapitalLetterWynn', [], 'Display', None],
  [ 0x1f8, 'LatinCapitalLetterNWithGrave', [], 'Display', None],
  [ 0x1f9, 'LatinSmallLetterNWithGrave', [], 'Display', None],
  [ 0x1fa, 'LatinCapitalLetterAWithRingAboveAndAcute', [], 'Display', None],
  [ 0x1fb, 'LatinSmallLetterAWithRingAboveAndAcute', [], 'Display', None],
  [ 0x1fc, 'LatinCapitalLetterAeWithAcute', [], 'Display', None],
  [ 0x1fd, 'LatinSmallLetterAeWithAcute', [], 'Display', None],
  [ 0x1fe, 'LatinCapitalLetterOWithStrokeAndAcute', [], 'Display', None],
  [ 0x1ff, 'LatinSmallLetterOWithStrokeAndAcute', [], 'Display', None],
  [ 0x200, 'LatinCapitalLetterAWithDoubleGrave', [], 'Display', None],
  [ 0x201, 'LatinSmallLetterAWithDoubleGrave', [], 'Display', None],
  [ 0x202, 'LatinCapitalLetterAWithInvertedBreve', [], 'Display', None],
  [ 0x203, 'LatinSmallLetterAWithInvertedBreve', [], 'Display', None],
  [ 0x204, 'LatinCapitalLetterEWithDoubleGrave', [], 'Display', None],
  [ 0x205, 'LatinSmallLetterEWithDoubleGrave', [], 'Display', None],
  [ 0x206, 'LatinCapitalLetterEWithInvertedBreve', [], 'Display', None],
  [ 0x207, 'LatinSmallLetterEWithInvertedBreve', [], 'Display', None],
  [ 0x208, 'LatinCapitalLetterIWithDoubleGrave', [], 'Display', None],
  [ 0x209, 'LatinSmallLetterIWithDoubleGrave', [], 'Display', None],
  [ 0x20a, 'LatinCapitalLetterIWithInvertedBreve', [], 'Display', None],
  [ 0x20b, 'LatinSmallLetterIWithInvertedBreve', [], 'Display', None],
  [ 0x20c, 'LatinCapitalLetterOWithDoubleGrave', [], 'Display', None],
  [ 0x20d, 'LatinSmallLetterOWithDoubleGrave', [], 'Display', None],
  [ 0x20e, 'LatinCapitalLetterOWithInvertedBreve', [], 'Display', None],
  [ 0x20f, 'LatinSmallLetterOWithInvertedBreve', [], 'Display', None],
  [ 0x210, 'LatinCapitalLetterRWithDoubleGrave', [], 'Display', None],
  [ 0x211, 'LatinSmallLetterRWithDoubleGrave', [], 'Display', None],
  [ 0x212, 'LatinCapitalLetterRWithInvertedBreve', [], 'Display', None],
  [ 0x213, 'LatinSmallLetterRWithInvertedBreve', [], 'Display', None],
  [ 0x214, 'LatinCapitalLetterUWithDoubleGrave', [], 'Display', None],
  [ 0x215, 'LatinSmallLetterUWithDoubleGrave', [], 'Display', None],
  [ 0x216, 'LatinCapitalLetterUWithInvertedBreve', [], 'Display', None],
  [ 0x217, 'LatinSmallLetterUWithInvertedBreve', [], 'Display', None],
  [ 0x218, 'LatinCapitalLetterSWithCommaBelow', [], 'Display', None],
  [ 0x219, 'LatinSmallLetterSWithCommaBelow', [], 'Display', None],
  [ 0x21a, 'LatinCapitalLetterTWithCommaBelow', [], 'Display', None],
  [ 0x21b, 'LatinSmallLetterTWithCommaBelow', [], 'Display', None],
  [ 0x21c, 'LatinCapitalLetterYogh', [], 'Display', None],
  [ 0x21d, 'LatinSmallLetterYogh', [], 'Display', None],
  [ 0x21e, 'LatinCapitalLetterHWithCaron', [], 'Display', None],
  [ 0x21f, 'LatinSmallLetterHWithCaron', [], 'Display', None],
  [ 0x220, 'LatinCapitalLetterNWithLongRightLeg', [], 'Display', None],
  [ 0x221, 'LatinSmallLetterDWithCurl', [], 'Display', None],
  [ 0x222, 'LatinCapitalLetterOu', [], 'Display', None],
  [ 0x223, 'LatinSmallLetterOu', [], 'Display', None],
  [ 0x224, 'LatinCapitalLetterZWithHook', [], 'Display', None],
  [ 0x225, 'LatinSmallLetterZWithHook', [], 'Display', None],
  [ 0x226, 'LatinCapitalLetterAWithDotAbove', [], 'Display', None],
  [ 0x227, 'LatinSmallLetterAWithDotAbove', [], 'Display', None],
  [ 0x228, 'LatinCapitalLetterEWithCedilla', [], 'Display', None],
  [ 0x229, 'LatinSmallLetterEWithCedilla', [], 'Display', None],
  [ 0x22a, 'LatinCapitalLetterOWithDiaeresisAndMacron', [], 'Display', None],
  [ 0x22b, 'LatinSmallLetterOWithDiaeresisAndMacron', [], 'Display', None],
  [ 0x22c, 'LatinCapitalLetterOWithTildeAndMacron', [], 'Display', None],
  [ 0x22d, 'LatinSmallLetterOWithTildeAndMacron', [], 'Display', None],
  [ 0x22e, 'LatinCapitalLetterOWithDotAbove', [], 'Display', None],
  [ 0x22f, 'LatinSmallLetterOWithDotAbove', [], 'Display', None],
  [ 0x230, 'LatinCapitalLetterOWithDotAboveAndMacron', [], 'Display', None],
  [ 0x231, 'LatinSmallLetterOWithDotAboveAndMacron', [], 'Display', None],
  [ 0x232, 'LatinCapitalLetterYWithMacron', [], 'Display', None],
  [ 0x233, 'LatinSmallLetterYWithMacron', [], 'Display', None],
  [ 0x234, 'LatinSmallLetterLWithCurl', [], 'Display', None],
  [ 0x235, 'LatinSmallLetterNWithCurl', [], 'Display', None],
  [ 0x236, 'LatinSmallLetterTWithCurl', [], 'Display', None],
  [ 0x237, 'LatinSmallLetterDotlessJ', [], 'Display', None],
  [ 0x238, 'LatinSmallLetterDbDigraph', [], 'Display', None],
  [ 0x239, 'LatinSmallLetterQpDigraph', [], 'Display', None],
  [ 0x23a, 'LatinCapitalLetterAWithStroke', [], 'Display', None],
  [ 0x23b, 'LatinCapitalLetterCWithStroke', [], 'Display', None],
  [ 0x23c, 'LatinSmallLetterCWithStroke', [], 'Display', None],
  [ 0x23d, 'LatinCapitalLetterLWithBar', [], 'Display', None],
  [ 0x23e, 'LatinCapitalLetterTWithDiagonalStroke', [], 'Display', None],
  [ 0x23f, 'LatinSmallLetterSWithSwashTail', [], 'Display', None],
  [ 0x240, 'LatinSmallLetterZWithSwashTail', [], 'Display', None],
  [ 0x241, 'LatinCapitalLetterGlottalStop', [], 'Display', None],
  [ 0x242, 'LatinSmallLetterGlottalStop', [], 'Display', None],
  [ 0x243, 'LatinCapitalLetterBWithStroke', [], 'Display', None],
  [ 0x244, 'LatinCapitalLetterUBar', [], 'Display', None],
  [ 0x245, 'LatinCapitalLetterTurnedV', [], 'Display', None],
  [ 0x246, 'LatinCapitalLetterEWithStroke', [], 'Display', None],
  [ 0x247, 'LatinSmallLetterEWithStroke', [], 'Display', None],
  [ 0x248, 'LatinCapitalLetterJWithStroke', [], 'Display', None],
  [ 0x249, 'LatinSmallLetterJWithStroke', [], 'Display', None],
  [ 0x24a, 'LatinCapitalLetterSmallQWithHookTail', [], 'Display', None],
  [ 0x24b, 'LatinSmallLetterQWithHookTail', [], 'Display', None],
  [ 0x24c, 'LatinCapitalLetterRWithStroke', [], 'Display', None],
  [ 0x24d, 'LatinSmallLetterRWithStroke', [], 'Display', None],
  [ 0x24e, 'LatinCapitalLetterYWithStroke', [], 'Display', None],
  [ 0x24f, 'LatinSmallLetterYWithStroke', [], 'Display', None],
  [ 0x250, 'LatinSmallLetterTurnedA', [], 'Display', None],
  [ 0x251, 'LatinSmallLetterAlpha', [], 'Display', None],
  [ 0x252, 'LatinSmallLetterTurnedAlpha', [], 'Display', None],
  [ 0x253, 'LatinSmallLetterBWithHook', [], 'Display', None],
  [ 0x254, 'LatinSmallLetterOpenO', [], 'Display', None],
  [ 0x255, 'LatinSmallLetterCWithCurl', [], 'Display', None],
  [ 0x256, 'LatinSmallLetterDWithTail', [], 'Display', None],
  [ 0x257, 'LatinSmallLetterDWithHook', [], 'Display', None],
  [ 0x258, 'LatinSmallLetterReversedE', [], 'Display', None],
  [ 0x259, 'LatinSmallLetterSchwa', [], 'Display', None],
  [ 0x25a, 'LatinSmallLetterSchwaWithHook', [], 'Display', None],
  [ 0x25b, 'LatinSmallLetterOpenE', [], 'Display', None],
  [ 0x25c, 'LatinSmallLetterReversedOpenE', [], 'Display', None],
  [ 0x25d, 'LatinSmallLetterReversedOpenEWithHook', [], 'Display', None],
  [ 0x25e, 'LatinSmallLetterClosedReversedOpenE', [], 'Display', None],
  [ 0x25f, 'LatinSmallLetterDotlessJWithStroke', [], 'Display', None],
  [ 0x260, 'LatinSmallLetterGWithHook', [], 'Display', None],
  [ 0x261, 'LatinSmallLetterScriptG', [], 'Display', None],
  [ 0x262, 'LatinLetterSmallCapitalG', [], 'Display', None],
  [ 0x263, 'LatinSmallLetterGamma', [], 'Display', None],
  [ 0x264, 'LatinSmallLetterRamsHorn', [], 'Display', None],
  [ 0x265, 'LatinSmallLetterTurnedH', [], 'Display', None],
  [ 0x266, 'LatinSmallLetterHWithHook', [], 'Display', None],
  [ 0x267, 'LatinSmallLetterHengWithHook', [], 'Display', None],
  [ 0x268, 'LatinSmallLetterIWithStroke', [], 'Display', None],
  [ 0x269, 'LatinSmallLetterIota', [], 'Display', None],
  [ 0x26a, 'LatinLetterSmallCapitalI', [], 'Display', None],
  [ 0x26b, 'LatinSmallLetterLWithMiddleTilde', [], 'Display', None],
  [ 0x26c, 'LatinSmallLetterLWithBelt', [], 'Display', None],
  [ 0x26d, 'LatinSmallLetterLWithRetroflexHook', [], 'Display', None],
  [ 0x26e, 'LatinSmallLetterLezh', [], 'Display', None],
  [ 0x26f, 'LatinSmallLetterTurnedM', [], 'Display', None],
  [ 0x270, 'LatinSmallLetterTurnedMWithLongLeg', [], 'Display', None],
  [ 0x271, 'LatinSmallLetterMWithHook', [], 'Display', None],
  [ 0x272, 'LatinSmallLetterNWithLeftHook', [], 'Display', None],
  [ 0x273, 'LatinSmallLetterNWithRetroflexHook', [], 'Display', None],
  [ 0x274, 'LatinLetterSmallCapitalN', [], 'Display', None],
  [ 0x275, 'LatinSmallLetterBarredO', [], 'Display', None],
  [ 0x276, 'LatinLetterSmallCapitalOe', [], 'Display', None],
  [ 0x277, 'LatinSmallLetterClosedOmega', [], 'Display', None],
  [ 0x278, 'LatinSmallLetterPhi', [], 'Display', None],
  [ 0x279, 'LatinSmallLetterTurnedR', [], 'Display', None],
  [ 0x27a, 'LatinSmallLetterTurnedRWithLongLeg', [], 'Display', None],
  [ 0x27b, 'LatinSmallLetterTurnedRWithHook', [], 'Display', None],
  [ 0x27c, 'LatinSmallLetterRWithLongLeg', [], 'Display', None],
  [ 0x27d, 'LatinSmallLetterRWithTail', [], 'Display', None],
  [ 0x27e, 'LatinSmallLetterRWithFishhook', [], 'Display', None],
  [ 0x27f, 'LatinSmallLetterReversedRWithFishhook', [], 'Display', None],
  [ 0x280, 'LatinLetterSmallCapitalR', [], 'Display', None],
  [ 0x281, 'LatinLetterSmallCapitalInvertedR', [], 'Display', None],
  [ 0x282, 'LatinSmallLetterSWithHook', [], 'Display', None],
  [ 0x283, 'LatinSmallLetterEsh', [], 'Display', None],
  [ 0x284, 'LatinSmallLetterDotlessJWithStrokeAndHook', [], 'Display', None],
  [ 0x285, 'LatinSmallLetterSquatReversedEsh', [], 'Display', None],
  [ 0x286, 'LatinSmallLetterEshWithCurl', [], 'Display', None],
  [ 0x287, 'LatinSmallLetterTurnedT', [], 'Display', None],
  [ 0x288, 'LatinSmallLetterTWithRetroflexHook', [], 'Display', None],
  [ 0x289, 'LatinSmallLetterUBar', [], 'Display', None],
  [ 0x28a, 'LatinSmallLetterUpsilon', [], 'Display', None],
  [ 0x28b, 'LatinSmallLetterVWithHook', [], 'Display', None],
  [ 0x28c, 'LatinSmallLetterTurnedV', [], 'Display', None],
  [ 0x28d, 'LatinSmallLetterTurnedW', [], 'Display', None],
  [ 0x28e, 'LatinSmallLetterTurnedY', [], 'Display', None],
  [ 0x28f, 'LatinLetterSmallCapitalY', [], 'Display', None],
  [ 0x290, 'LatinSmallLetterZWithRetroflexHook', [], 'Display', None],
  [ 0x291, 'LatinSmallLetterZWithCurl', [], 'Display', None],
  [ 0x292, 'LatinSmallLetterEzh', [], 'Display', None],
  [ 0x293, 'LatinSmallLetterEzhWithCurl', [], 'Display', None],
  [ 0x294, 'LatinLetterGlottalStop', [], 'Display', None],
  [ 0x295, 'LatinLetterPharyngealVoicedFricative', [], 'Display', None],
  [ 0x296, 'LatinLetterInvertedGlottalStop', [], 'Display', None],
  [ 0x297, 'LatinLetterStretchedC', [], 'Display', None],
  [ 0x298, 'LatinLetterBilabialClick', [], 'Display', None],
  [ 0x299, 'LatinLetterSmallCapitalB', [], 'Display', None],
  [ 0x29a, 'LatinSmallLetterClosedOpenE', [], 'Display', None],
  [ 0x29b, 'LatinLetterSmallCapitalGWithHook', [], 'Display', None],
  [ 0x29c, 'LatinLetterSmallCapitalH', [], 'Display', None],
  [ 0x29d, 'LatinSmallLetterJWithCrossedtail', [], 'Display', None],
  [ 0x29e, 'LatinSmallLetterTurnedK', [], 'Display', None],
  [ 0x29f, 'LatinLetterSmallCapitalL', [], 'Display', None],
  [ 0x2a0, 'LatinSmallLetterQWithHook', [], 'Display', None],
  [ 0x2a1, 'LatinLetterGlottalStopWithStroke', [], 'Display', None],
  [ 0x2a2, 'LatinLetterReversedGlottalStopWithStroke', [], 'Display', None],
  [ 0x2a3, 'LatinSmallLetterDzDigraph', [], 'Display', None],
  [ 0x2a4, 'LatinSmallLetterDezhDigraph', [], 'Display', None],
  [ 0x2a5, 'LatinSmallLetterDzDigraphWithCurl', [], 'Display', None],
  [ 0x2a6, 'LatinSmallLetterTsDigraph', [], 'Display', None],
  [ 0x2a7, 'LatinSmallLetterTeshDigraph', [], 'Display', None],
  [ 0x2a8, 'LatinSmallLetterTcDigraphWithCurl', [], 'Display', None],
  [ 0x2a9, 'LatinSmallLetterFengDigraph', [], 'Display', None],
  [ 0x2aa, 'LatinSmallLetterLsDigraph', [], 'Display', None],
  [ 0x2ab, 'LatinSmallLetterLzDigraph', [], 'Display', None],
  [ 0x2ac, 'LatinLetterBilabialPercussive', [], 'Display', None],
  [ 0x2ad, 'LatinLetterBidentalPercussive', [], 'Display', None],
  [ 0x2ae, 'LatinSmallLetterTurnedHWithFishhook', [], 'Display', None],
  [ 0x2af, 'LatinSmallLetterTurnedHWithFishhookAndTail', [], 'Display', None],
  [ 0x2b0, 'ModifierLetterSmallH', [], 'Display', None],
  [ 0x2b1, 'ModifierLetterSmallHWithHook', [], 'Display', None],
  [ 0x2b2, 'ModifierLetterSmallJ', [], 'Display', None],
  [ 0x2b3, 'ModifierLetterSmallR', [], 'Display', None],
  [ 0x2b4, 'ModifierLetterSmallTurnedR', [], 'Display', None],
  [ 0x2b5, 'ModifierLetterSmallTurnedRWithHook', [], 'Display', None],
  [ 0x2b6, 'ModifierLetterSmallCapitalInvertedR', [], 'Display', None],
  [ 0x2b7, 'ModifierLetterSmallW', [], 'Display', None],
  [ 0x2b8, 'ModifierLetterSmallY', [], 'Display', None],
  [ 0x2b9, 'ModifierLetterPrime', [], 'Display', None],
  [ 0x2ba, 'ModifierLetterDoublePrime', [], 'Display', None],
  [ 0x2bb, 'ModifierLetterTurnedComma', [], 'Display', None],
  [ 0x2bc, 'ModifierLetterApostrophe', [], 'Display', None],
  [ 0x2bd, 'ModifierLetterReversedComma', [], 'Display', None],
  [ 0x2be, 'ModifierLetterRightHalfRing', [], 'Display', None],
  [ 0x2bf, 'ModifierLetterLeftHalfRing', [], 'Display', None],
  [ 0x2c0, 'ModifierLetterGlottalStop', [], 'Display', None],
  [ 0x2c1, 'ModifierLetterReversedGlottalStop', [], 'Display', None],
  [ 0x2c2, 'ModifierLetterLeftArrowhead', [], 'Display', None],
  [ 0x2c3, 'ModifierLetterRightArrowhead', [], 'Display', None],
  [ 0x2c4, 'ModifierLetterUpArrowhead', [], 'Display', None],
  [ 0x2c5, 'ModifierLetterDownArrowhead', [], 'Display', None],
  [ 0x2c6, 'ModifierLetterCircumflexAccent', [], 'Display', None],
  [ 0x2c7, 'Caron', [], 'Display', None],
  [ 0x2c8, 'ModifierLetterVerticalLine', [], 'Display', None],
  [ 0x2c9, 'ModifierLetterMacron', [], 'Display', None],
  [ 0x2ca, 'ModifierLetterAcuteAccent', [], 'Display', None],
  [ 0x2cb, 'ModifierLetterGraveAccent', [], 'Display', None],
  [ 0x2cc, 'ModifierLetterLowVerticalLine', [], 'Display', None],
  [ 0x2cd, 'ModifierLetterLowMacron', [], 'Display', None],
  [ 0x2ce, 'ModifierLetterLowGraveAccent', [], 'Display', None],
  [ 0x2cf, 'ModifierLetterLowAcuteAccent', [], 'Display', None],
  [ 0x2d0, 'ModifierLetterTriangularColon', [], 'Display', None],
  [ 0x2d1, 'ModifierLetterHalfTriangularColon', [], 'Display', None],
  [ 0x2d2, 'ModifierLetterCentredRightHalfRing', [], 'Display', None],
  [ 0x2d3, 'ModifierLetterCentredLeftHalfRing', [], 'Display', None],
  [ 0x2d4, 'ModifierLetterUpTack', [], 'Display', None],
  [ 0x2d5, 'ModifierLetterDownTack', [], 'Display', None],
  [ 0x2d6, 'ModifierLetterPlusSign', [], 'Display', None],
  [ 0x2d7, 'ModifierLetterMinusSign', [], 'Display', None],
  [ 0x2d8, 'Breve', [], 'Display', None],
  [ 0x2d9, 'DotAbove', [], 'Display', None],
  [ 0x2da, 'RingAbove', [], 'Display', None],
  [ 0x2db, 'Ogonek', [], 'Display', None],
  [ 0x2dc, 'SmallTilde', [], 'Display', None],
  [ 0x2dd, 'DoubleAcuteAccent', [], 'Display', None],
  [ 0x2de, 'ModifierLetterRhoticHook', [], 'Display', None],
  [ 0x2df, 'ModifierLetterCrossAccent', [], 'Display', None],
  [ 0x2e0, 'ModifierLetterSmallGamma', [], 'Display', None],
  [ 0x2e1, 'ModifierLetterSmallL', [], 'Display', None],
  [ 0x2e2, 'ModifierLetterSmallS', [], 'Display', None],
  [ 0x2e3, 'ModifierLetterSmallX', [], 'Display', None],
  [ 0x2e4, 'ModifierLetterSmallReversedGlottalStop', [], 'Display', None],
  [ 0x2e5, 'ModifierLetterExtrahighToneBar', [], 'Display', None],
  [ 0x2e6, 'ModifierLetterHighToneBar', [], 'Display', None],
  [ 0x2e7, 'ModifierLetterMidToneBar', [], 'Display', None],
  [ 0x2e8, 'ModifierLetterLowToneBar', [], 'Display', None],
  [ 0x2e9, 'ModifierLetterExtralowToneBar', [], 'Display', None],
  [ 0x2ea, 'ModifierLetterYinDepartingToneMark', [], 'Display', None],
  [ 0x2eb, 'ModifierLetterYangDepartingToneMark', [], 'Display', None],
  [ 0x2ec, 'ModifierLetterVoicing', [], 'Display', None],
  [ 0x2ed, 'ModifierLetterUnaspirated', [], 'Display', None],
  [ 0x2ee, 'ModifierLetterDoubleApostrophe', [], 'Display', None],
  [ 0x2ef, 'ModifierLetterLowDownArrowhead', [], 'Display', None],
  [ 0x2f0, 'ModifierLetterLowUpArrowhead', [], 'Display', None],
  [ 0x2f1, 'ModifierLetterLowLeftArrowhead', [], 'Display', None],
  [ 0x2f2, 'ModifierLetterLowRightArrowhead', [], 'Display', None],
  [ 0x2f3, 'ModifierLetterLowRing', [], 'Display', None],
  [ 0x2f4, 'ModifierLetterMiddleGraveAccent', [], 'Display', None],
  [ 0x2f5, 'ModifierLetterMiddleDoubleGraveAccent', [], 'Display', None],
  [ 0x2f6, 'ModifierLetterMiddleDoubleAcuteAccent', [], 'Display', None],
  [ 0x2f7, 'ModifierLetterLowTilde', [], 'Display', None],
  [ 0x2f8, 'ModifierLetterRaisedColon', [], 'Display', None],
  [ 0x2f9, 'ModifierLetterBeginHighTone', [], 'Display', None],
  [ 0x2fa, 'ModifierLetterEndHighTone', [], 'Display', None],
  [ 0x2fb, 'ModifierLetterBeginLowTone', [], 'Display', None],
  [ 0x2fc, 'ModifierLetterEndLowTone', [], 'Display', None],
  [ 0x2fd, 'ModifierLetterShelf', [], 'Display', None],
  [ 0x2fe, 'ModifierLetterOpenShelf', [], 'Display', None],
  [ 0x2ff, 'ModifierLetterLowLeftArrow', [], 'Display', None],
  [ 0x300, 'CombiningGraveAccent', [], 'Display', None],
  [ 0x301, 'CombiningAcuteAccent', [], 'Display', None],
  [ 0x302, 'CombiningCircumflexAccent', [], 'Display', None],
  [ 0x303, 'CombiningTilde', [], 'Display', None],
  [ 0x304, 'CombiningMacron', [], 'Display', None],
  [ 0x305, 'CombiningOverline', [], 'Display', None],
  [ 0x306, 'CombiningBreve', [], 'Display', None],
  [ 0x307, 'CombiningDotAbove', [], 'Display', None],
  [ 0x308, 'CombiningDiaeresis', [], 'Display', None],
  [ 0x309, 'CombiningHookAbove', [], 'Display', None],
  [ 0x30a, 'CombiningRingAbove', [], 'Display', None],
  [ 0x30b, 'CombiningDoubleAcuteAccent', [], 'Display', None],
  [ 0x30c, 'CombiningCaron', [], 'Display', None],
  [ 0x30d, 'CombiningVerticalLineAbove', [], 'Display', None],
  [ 0x30e, 'CombiningDoubleVerticalLineAbove', [], 'Display', None],
  [ 0x30f, 'CombiningDoubleGraveAccent', [], 'Display', None],
  [ 0x310, 'CombiningCandrabindu', [], 'Display', None],
  [ 0x311, 'CombiningInvertedBreve', [], 'Display', None],
  [ 0x312, 'CombiningTurnedCommaAbove', [], 'Display', None],
  [ 0x313, 'CombiningCommaAbove', [], 'Display', None],
  [ 0x314, 'CombiningReversedCommaAbove', [], 'Display', None],
  [ 0x315, 'CombiningCommaAboveRight', [], 'Display', None],
  [ 0x316, 'CombiningGraveAccentBelow', [], 'Display', None],
  [ 0x317, 'CombiningAcuteAccentBelow', [], 'Display', None],
  [ 0x318, 'CombiningLeftTackBelow', [], 'Display', None],
  [ 0x319, 'CombiningRightTackBelow', [], 'Display', None],
  [ 0x31a, 'CombiningLeftAngleAbove', [], 'Display', None],
  [ 0x31b, 'CombiningHorn', [], 'Display', None],
  [ 0x31c, 'CombiningLeftHalfRingBelow', [], 'Display', None],
  [ 0x31d, 'CombiningUpTackBelow', [], 'Display', None],
  [ 0x31e, 'CombiningDownTackBelow', [], 'Display', None],
  [ 0x31f, 'CombiningPlusSignBelow', [], 'Display', None],
  [ 0x320, 'CombiningMinusSignBelow', [], 'Display', None],
  [ 0x321, 'CombiningPalatalizedHookBelow', [], 'Display', None],
  [ 0x322, 'CombiningRetroflexHookBelow', [], 'Display', None],
  [ 0x323, 'CombiningDotBelow', [], 'Display', None],
  [ 0x324, 'CombiningDiaeresisBelow', [], 'Display', None],
  [ 0x325, 'CombiningRingBelow', [], 'Display', None],
  [ 0x326, 'CombiningCommaBelow', [], 'Display', None],
  [ 0x327, 'CombiningCedilla', [], 'Display', None],
  [ 0x328, 'CombiningOgonek', [], 'Display', None],
  [ 0x329, 'CombiningVerticalLineBelow', [], 'Display', None],
  [ 0x32a, 'CombiningBridgeBelow', [], 'Display', None],
  [ 0x32b, 'CombiningInvertedDoubleArchBelow', [], 'Display', None],
  [ 0x32c, 'CombiningCaronBelow', [], 'Display', None],
  [ 0x32d, 'CombiningCircumflexAccentBelow', [], 'Display', None],
  [ 0x32e, 'CombiningBreveBelow', [], 'Display', None],
  [ 0x32f, 'CombiningInvertedBreveBelow', [], 'Display', None],
  [ 0x330, 'CombiningTildeBelow', [], 'Display', None],
  [ 0x331, 'CombiningMacronBelow', [], 'Display', None],
  [ 0x332, 'CombiningLowLine', [], 'Display', None],
  [ 0x333, 'CombiningDoubleLowLine', [], 'Display', None],
  [ 0x334, 'CombiningTildeOverlay', [], 'Display', None],
  [ 0x335, 'CombiningShortStrokeOverlay', [], 'Display', None],
  [ 0x336, 'CombiningLongStrokeOverlay', [], 'Display', None],
  [ 0x337, 'CombiningShortSolidusOverlay', [], 'Display', None],
  [ 0x338, 'CombiningLongSolidusOverlay', [], 'Display', None],
  [ 0x339, 'CombiningRightHalfRingBelow', [], 'Display', None],
  [ 0x33a, 'CombiningInvertedBridgeBelow', [], 'Display', None],
  [ 0x33b, 'CombiningSquareBelow', [], 'Display', None],
  [ 0x33c, 'CombiningSeagullBelow', [], 'Display', None],
  [ 0x33d, 'CombiningXAbove', [], 'Display', None],
  [ 0x33e, 'CombiningVerticalTilde', [], 'Display', None],
  [ 0x33f, 'CombiningDoubleOverline', [], 'Display', None],
  [ 0x340, 'CombiningGraveToneMark', [], 'Display', None],
  [ 0x341, 'CombiningAcuteToneMark', [], 'Display', None],
  [ 0x342, 'CombiningGreekPerispomeni', [], 'Display', None],
  [ 0x343, 'CombiningGreekKoronis', [], 'Display', None],
  [ 0x344, 'CombiningGreekDialytikaTonos', [], 'Display', None],
  [ 0x345, 'CombiningGreekYpogegrammeni', [], 'Display', None],
  [ 0x346, 'CombiningBridgeAbove', [], 'Display', None],
  [ 0x347, 'CombiningEqualsSignBelow', [], 'Display', None],
  [ 0x348, 'CombiningDoubleVerticalLineBelow', [], 'Display', None],
  [ 0x349, 'CombiningLeftAngleBelow', [], 'Display', None],
  [ 0x34a, 'CombiningNotTildeAbove', [], 'Display', None],
  [ 0x34b, 'CombiningHomotheticAbove', [], 'Display', None],
  [ 0x34c, 'CombiningAlmostEqualToAbove', [], 'Display', None],
  [ 0x34d, 'CombiningLeftRightArrowBelow', [], 'Display', None],
  [ 0x34e, 'CombiningUpwardsArrowBelow', [], 'Display', None],
  [ 0x34f, 'CombiningGraphemeJoiner', [], 'Display', None],
  [ 0x350, 'CombiningRightArrowheadAbove', [], 'Display', None],
  [ 0x351, 'CombiningLeftHalfRingAbove', [], 'Display', None],
  [ 0x352, 'CombiningFermata', [], 'Display', None],
  [ 0x353, 'CombiningXBelow', [], 'Display', None],
  [ 0x354, 'CombiningLeftArrowheadBelow', [], 'Display', None],
  [ 0x355, 'CombiningRightArrowheadBelow', [], 'Display', None],
  [ 0x356, 'CombiningRightArrowheadAndUpArrowheadBelow', [], 'Display', None],
  [ 0x357, 'CombiningRightHalfRingAbove', [], 'Display', None],
  [ 0x358, 'CombiningDotAboveRight', [], 'Display', None],
  [ 0x359, 'CombiningAsteriskBelow', [], 'Display', None],
  [ 0x35a, 'CombiningDoubleRingBelow', [], 'Display', None],
  [ 0x35b, 'CombiningZigzagAbove', [], 'Display', None],
  [ 0x35c, 'CombiningDoubleBreveBelow', [], 'Display', None],
  [ 0x35d, 'CombiningDoubleBreve', [], 'Display', None],
  [ 0x35e, 'CombiningDoubleMacron', [], 'Display', None],
  [ 0x35f, 'CombiningDoubleMacronBelow', [], 'Display', None],
  [ 0x360, 'CombiningDoubleTilde', [], 'Display', None],
  [ 0x361, 'CombiningDoubleInvertedBreve', [], 'Display', None],
  [ 0x362, 'CombiningDoubleRightwardsArrowBelow', [], 'Display', None],
  [ 0x363, 'CombiningLatinSmallLetterA', [], 'Display', None],
  [ 0x364, 'CombiningLatinSmallLetterE', [], 'Display', None],
  [ 0x365, 'CombiningLatinSmallLetterI', [], 'Display', None],
  [ 0x366, 'CombiningLatinSmallLetterO', [], 'Display', None],
  [ 0x367, 'CombiningLatinSmallLetterU', [], 'Display', None],
  [ 0x368, 'CombiningLatinSmallLetterC', [], 'Display', None],
  [ 0x369, 'CombiningLatinSmallLetterD', [], 'Display', None],
  [ 0x36a, 'CombiningLatinSmallLetterH', [], 'Display', None],
  [ 0x36b, 'CombiningLatinSmallLetterM', [], 'Display', None],
  [ 0x36c, 'CombiningLatinSmallLetterR', [], 'Display', None],
  [ 0x36d, 'CombiningLatinSmallLetterT', [], 'Display', None],
  [ 0x36e, 'CombiningLatinSmallLetterV', [], 'Display', None],
  [ 0x36f, 'CombiningLatinSmallLetterX', [], 'Display', None],
  [ 0x370, 'GreekCapitalLetterHeta', ['Heta'], 'Symbol', None],
  [ 0x371, 'GreekSmallLetterHeta', ['heta'], 'Symbol', None],
  [ 0x372, 'GreekCapitalLetterArchaicSampi', ['ArchaicSampi'], 'Symbol', None],
  [ 0x373, 'GreekSmallLetterArchaicSampi', ['archaicsampi'], 'Symbol', None],
  [ 0x374, 'GreekNumeralSign', [], 'Display', None],
  [ 0x375, 'GreekLowerNumeralSign', [], 'Display', None],
  [ 0x376, 'GreekCapitalLetterPamphylianDigamma', [], 'Symbol', None],
  [ 0x377, 'GreekSmallLetterPamphylianDigamma', [], 'Symbol', None],
  [ 0x378, 'UnicodeCharacter0x378', [], 'Display', None],
  [ 0x379, 'UnicodeCharacter0x379', [], 'Display', None],
  [ 0x37a, 'GreekYpogegrammeni', [], 'Display', None],
  [ 0x37b, 'GreekSmallReversedLunateSigmaSymbol', [], 'Symbol', None],
  [ 0x37c, 'GreekSmallDottedLunateSigmaSymbol', [], 'Symbol', None],
  [ 0x37d, 'GreekSmallReversedDottedLunateSigmaSymbol', [], 'Symbol', None],
  [ 0x37e, 'GreekQuestionMark', [], 'Display', None],
  [ 0x37f, 'UnicodeCharacter0x37f', [], 'Display', None],
  [ 0x380, 'UnicodeCharacter0x380', [], 'Display', None],
  [ 0x381, 'UnicodeCharacter0x381', [], 'Display', None],
  [ 0x382, 'UnicodeCharacter0x382', [], 'Display', None],
  [ 0x383, 'UnicodeCharacter0x383', [], 'Display', None],
  [ 0x384, 'GreekTonos', [], 'Display', None],
  [ 0x385, 'GreekDialytikaTonos', [], 'Display', None],
  [ 0x386, 'GreekCapitalLetterAlphaWithTonos', [], 'Display', None],
  [ 0x387, 'GreekAnoTeleia', [], 'Display', None],
  [ 0x388, 'GreekCapitalLetterEpsilonWithTonos', [], 'Display', None],
  [ 0x389, 'GreekCapitalLetterEtaWithTonos', [], 'Display', None],
  [ 0x38a, 'GreekCapitalLetterIotaWithTonos', [], 'Display', None],
  [ 0x38b, 'UnicodeCharacter0x38b', [], 'Display', None],
  [ 0x38c, 'GreekCapitalLetterOmicronWithTonos', [], 'Display', None],
  [ 0x38d, 'UnicodeCharacter0x38d', [], 'Display', None],
  [ 0x38e, 'GreekCapitalLetterUpsilonWithTonos', [], 'Display', None],
  [ 0x38f, 'GreekCapitalLetterOmegaWithTonos', [], 'Display', None],
  [ 0x390, 'GreekSmallLetterIotaWithDialytikaAndTonos', [], 'Display', None],
  [ 0x391, 'GreekCapitalLetterAlpha', ['A','Alpha'], 'Symbol', None],
  [ 0x392, 'GreekCapitalLetterBeta', ['B','Beta'], 'Symbol', None],
  [ 0x393, 'GreekCapitalLetterGamma', ['G','Gamma'], 'Symbol', None],
  [ 0x394, 'GreekCapitalLetterDelta', ['D','Delta'], 'Symbol', None],
  [ 0x395, 'GreekCapitalLetterEpsilon', ['E','Epsilon'], 'Symbol', None],
  [ 0x396, 'GreekCapitalLetterZeta', ['Z','Zeta'], 'Symbol', None],
  [ 0x397, 'GreekCapitalLetterEta', ['H','Et','Eta'], 'Symbol', None],
  [ 0x398, 'GreekCapitalLetterTheta', ['Q','Th','Theta'], 'Symbol', None],
  [ 0x399, 'GreekCapitalLetterIota', ['I','Iota'], 'Symbol', None],
  [ 0x39a, 'GreekCapitalLetterKappa', ['K','Kappa'], 'Symbol', None],
  [ 0x39b, 'GreekCapitalLetterLamda', ['L','Lambda'], 'Symbol', None],
  [ 0x39c, 'GreekCapitalLetterMu', ['M','Mu'], 'Symbol', None],
  [ 0x39d, 'GreekCapitalLetterNu', ['V','N','Nu'], 'Symbol', None],
  [ 0x39e, 'GreekCapitalLetterXi', ['X','Xi'], 'Symbol', None],
  [ 0x39f, 'GreekCapitalLetterOmicron', ['O','Omicron'], 'Symbol', None],
  [ 0x3a0, 'GreekCapitalLetterPi', ['P','Pi'], 'Symbol', None],
  [ 0x3a1, 'GreekCapitalLetterRho', ['R','Rho'], 'Symbol', None],
  [ 0x3a2, 'UnicodeCharacter0x3a2', [], 'Display', None],
  [ 0x3a3, 'GreekCapitalLetterSigma', ['S','Sigma'], 'Symbol', None],
  [ 0x3a4, 'GreekCapitalLetterTau', ['T','Tau'], 'Symbol', None],
  [ 0x3a5, 'GreekCapitalLetterUpsilon', ['U','Upsilon'], 'Symbol', None],
  [ 0x3a6, 'GreekCapitalLetterPhi', ['F','J','Ph','Phi'], 'Symbol', None],
  [ 0x3a7, 'GreekCapitalLetterChi', ['C','Ch','Chi'], 'Symbol', None],
  [ 0x3a8, 'GreekCapitalLetterPsi', ['Y','Ps','Psi'], 'Symbol', None],
  [ 0x3a9, 'GreekCapitalLetterOmega', ['W','Om','Omega'], 'Symbol', None],
  [ 0x3aa, 'GreekCapitalLetterIotaWithDialytika', [], 'Display', None],
  [ 0x3ab, 'GreekCapitalLetterUpsilonWithDialytika', [], 'Display', None],
  [ 0x3ac, 'GreekSmallLetterAlphaWithTonos', [], 'Display', None],
  [ 0x3ad, 'GreekSmallLetterEpsilonWithTonos', [], 'Display', None],
  [ 0x3ae, 'GreekSmallLetterEtaWithTonos', [], 'Display', None],
  [ 0x3af, 'GreekSmallLetterIotaWithTonos', [], 'Display', None],
  [ 0x3b0, 'GreekSmallLetterUpsilonWithDialytikaAndTonos', [], 'Display', None],
  [ 0x3b1, 'GreekSmallLetterAlpha', ['a','alpha'], 'Symbol', None],
  [ 0x3b2, 'GreekSmallLetterBeta', ['b','beta'], 'Symbol', None],
  [ 0x3b3, 'GreekSmallLetterGamma', ['g','gamma'], 'Symbol', None],
  [ 0x3b4, 'GreekSmallLetterDelta', ['d','delta'], 'Symbol', None],
  [ 0x3b5, 'GreekSmallLetterEpsilon', ['ce','cepsilon','varepsilon'], 'Symbol', None],
  [ 0x3b6, 'GreekSmallLetterZeta', ['z','zeta'], 'Symbol', None],
  [ 0x3b7, 'GreekSmallLetterEta', ['h','et','eta'], 'Symbol', None],
  [ 0x3b8, 'GreekSmallLetterTheta', ['q','th','theta'], 'Symbol', None],
  [ 0x3b9, 'GreekSmallLetterIota', ['i','iota'], 'Symbol', None],
  [ 0x3ba, 'GreekSmallLetterKappa', ['k','kappa'], 'Symbol', None],
  [ 0x3bb, 'GreekSmallLetterLamda', ['l','lambda'], 'Symbol', None],
  [ 0x3bc, 'GreekSmallLetterMu', ['m','mu'], 'Symbol', None],
  [ 0x3bd, 'GreekSmallLetterNu', ['v','n','nu'], 'Symbol', None],
  [ 0x3be, 'GreekSmallLetterXi', ['x','xi'], 'Symbol', None],
  [ 0x3bf, 'GreekSmallLetterOmicron', ['o','omicron'], 'Symbol', None],
  [ 0x3c0, 'GreekSmallLetterPi', ['p','pi'], 'Symbol', None],
  [ 0x3c1, 'GreekSmallLetterRho', ['r','rho'], 'Symbol', None],
  [ 0x3c2, 'GreekSmallLetterFinalSigma', ['cs','csigma','varsigma','finalsigma'], 'Symbol', None],
  [ 0x3c3, 'GreekSmallLetterSigma', ['s','sigma'], 'Symbol', None],
  [ 0x3c4, 'GreekSmallLetterTau', ['t','tau'], 'Symbol', None],
  [ 0x3c5, 'GreekSmallLetterUpsilon', ['u','upsilon'], 'Symbol', None],
  [ 0x3c6, 'GreekSmallLetterPhi', ['j','cf','cph','cphi','varphi'], 'Symbol', None],
  [ 0x3c7, 'GreekSmallLetterChi', ['c','ch','chi'], 'Symbol', None],
  [ 0x3c8, 'GreekSmallLetterPsi', ['y','ps','psi'], 'Symbol', None],
  [ 0x3c9, 'GreekSmallLetterOmega', ['w','om','omega'], 'Symbol', None],
  [ 0x3ca, 'GreekSmallLetterIotaWithDialytika', [], 'Display', None],
  [ 0x3cb, 'GreekSmallLetterUpsilonWithDialytika', [], 'Display', None],
  [ 0x3cc, 'GreekSmallLetterOmicronWithTonos', [], 'Display', None],
  [ 0x3cd, 'GreekSmallLetterUpsilonWithTonos', [], 'Display', None],
  [ 0x3ce, 'GreekSmallLetterOmegaWithTonos', [], 'Display', None],
  [ 0x3cf, 'GreekCapitalKaiSymbol', ['Kai'], 'Symbol', None],
  [ 0x3d0, 'GreekBetaSymbol', ['cb','cbeta','varbeta'], 'Symbol', None],
  [ 0x3d1, 'GreekThetaSymbol', ['cth','ctheta','vartheta'], 'Symbol', None],
  [ 0x3d2, 'GreekUpsilonWithHookSymbol', [], 'Display', None],
  [ 0x3d3, 'GreekUpsilonWithAcuteAndHookSymbol', [], 'Display', None],
  [ 0x3d4, 'GreekUpsilonWithDiaeresisAndHookSymbol', [], 'Display', None],
  [ 0x3d5, 'GreekPhiSymbol', ['f','ph','phi'], 'Symbol', None],
  [ 0x3d6, 'GreekPiSymbol', ['cp','cpi','varpi'], 'Symbol', None],
  [ 0x3d7, 'GreekKaiSymbol', ['kai'], 'Symbol', None],
  [ 0x3d8, 'GreekLetterArchaicKoppa', ['ArchaicKoppa'], 'Symbol', None],
  [ 0x3d9, 'GreekSmallLetterArchaicKoppa', ['archaickoppa'], 'Symbol', None],
  [ 0x3da, 'GreekLetterStigma', ['Stigma'], 'Symbol', None],
  [ 0x3db, 'GreekSmallLetterStigma', ['stigma'], 'Symbol', None],
  [ 0x3dc, 'GreekLetterDigamma', ['Digamma'], 'Symbol', None],
  [ 0x3dd, 'GreekSmallLetterDigamma', ['digamma'], 'Symbol', None],
  [ 0x3de, 'GreekLetterKoppa', ['Koppa'], 'Symbol', None],
  [ 0x3df, 'GreekSmallLetterKoppa', ['koppa'], 'Symbol', None],
  [ 0x3e0, 'GreekLetterSampi', ['Sampi'], 'Symbol', None],
  [ 0x3e1, 'GreekSmallLetterSampi', ['sampi'], 'Symbol', None],
  [ 0x3e2, 'CopticCapitalLetterShei', [], 'Display', None],
  [ 0x3e3, 'CopticSmallLetterShei', [], 'Display', None],
  [ 0x3e4, 'CopticCapitalLetterFei', [], 'Display', None],
  [ 0x3e5, 'CopticSmallLetterFei', [], 'Display', None],
  [ 0x3e6, 'CopticCapitalLetterKhei', [], 'Display', None],
  [ 0x3e7, 'CopticSmallLetterKhei', [], 'Display', None],
  [ 0x3e8, 'CopticCapitalLetterHori', [], 'Display', None],
  [ 0x3e9, 'CopticSmallLetterHori', [], 'Display', None],
  [ 0x3ea, 'CopticCapitalLetterGangia', [], 'Display', None],
  [ 0x3eb, 'CopticSmallLetterGangia', [], 'Display', None],
  [ 0x3ec, 'CopticCapitalLetterShima', [], 'Display', None],
  [ 0x3ed, 'CopticSmallLetterShima', [], 'Display', None],
  [ 0x3ee, 'CopticCapitalLetterDei', [], 'Display', None],
  [ 0x3ef, 'CopticSmallLetterDei', [], 'Display', None],
  [ 0x3f0, 'GreekKappaSymbol', ['ck','ckappa','varkappa'], 'Symbol', None],
  [ 0x3f1, 'GreekRhoSymbol', ['cr','crho','varrho'], 'Symbol', None],
  [ 0x3f2, 'GreekLunateSigmaSymbol', [], 'Display', None],
  [ 0x3f3, 'GreekLetterYot', ['Yot'], 'Symbol', None],
  [ 0x3f4, 'GreekCapitalThetaSymbol', ['cTh','cTheta','varTheta'], 'Display', None],
  [ 0x3f5, 'GreekLunateEpsilonSymbol', ['e','epsilon'], 'Symbol', None],
  [ 0x3f6, 'GreekReversedLunateEpsilonSymbol', ['backepsilon'], 'Symbol', None],
  [ 0x3f7, 'GreekCapitalLetterSho', ['Sho'], 'Symbol', None],
  [ 0x3f8, 'GreekSmallLetterSho', ['sho'], 'Symbol', None],
  [ 0x3f9, 'GreekCapitalLunateSigmaSymbol', ['backsigma'], 'Symbol', None],
  [ 0x3fa, 'GreekCapitalLetterSan', ['San'], 'Symbol', None],
  [ 0x3fb, 'GreekSmallLetterSan', ['san'], 'Symbol', None],
  [ 0x3fc, 'GreekRhoWithStrokeSymbol', [], 'Display', None],
  [ 0x3fd, 'GreekCapitalReversedLunateSigmaSymbol', [], 'Display', None],
  [ 0x3fe, 'GreekCapitalDottedLunateSigmaSymbol', [], 'Display', None],
  [ 0x3ff, 'GreekCapitalReversedDottedLunateSigmaSymbol', [], 'Display', None],
  [ 0x400, 'CyrillicCapitalLetterIeWithGrave', [], 'Display', None],
  [ 0x401, 'CyrillicCapitalLetterIo', [], 'Display', None],
  [ 0x402, 'CyrillicCapitalLetterDje', [], 'Display', None],
  [ 0x403, 'CyrillicCapitalLetterGje', [], 'Display', None],
  [ 0x404, 'CyrillicCapitalLetterUkrainianIe', [], 'Display', None],
  [ 0x405, 'CyrillicCapitalLetterDze', [], 'Display', None],
  [ 0x406, 'CyrillicCapitalLetterByelorussianukrainianI', [], 'Display', None],
  [ 0x407, 'CyrillicCapitalLetterYi', [], 'Display', None],
  [ 0x408, 'CyrillicCapitalLetterJe', [], 'Display', None],
  [ 0x409, 'CyrillicCapitalLetterLje', [], 'Display', None],
  [ 0x40a, 'CyrillicCapitalLetterNje', [], 'Display', None],
  [ 0x40b, 'CyrillicCapitalLetterTshe', [], 'Display', None],
  [ 0x40c, 'CyrillicCapitalLetterKje', [], 'Display', None],
  [ 0x40d, 'CyrillicCapitalLetterIWithGrave', [], 'Display', None],
  [ 0x40e, 'CyrillicCapitalLetterShortU', [], 'Display', None],
  [ 0x40f, 'CyrillicCapitalLetterDzhe', [], 'Display', None],
  [ 0x410, 'CyrillicCapitalLetterA', [], 'Display', None],
  [ 0x411, 'CyrillicCapitalLetterBe', [], 'Display', None],
  [ 0x412, 'CyrillicCapitalLetterVe', [], 'Display', None],
  [ 0x413, 'CyrillicCapitalLetterGhe', [], 'Display', None],
  [ 0x414, 'CyrillicCapitalLetterDe', [], 'Display', None],
  [ 0x415, 'CyrillicCapitalLetterIe', [], 'Display', None],
  [ 0x416, 'CyrillicCapitalLetterZhe', [], 'Display', None],
  [ 0x417, 'CyrillicCapitalLetterZe', [], 'Display', None],
  [ 0x418, 'CyrillicCapitalLetterI', [], 'Display', None],
  [ 0x419, 'CyrillicCapitalLetterShortI', [], 'Display', None],
  [ 0x41a, 'CyrillicCapitalLetterKa', [], 'Display', None],
  [ 0x41b, 'CyrillicCapitalLetterEl', [], 'Display', None],
  [ 0x41c, 'CyrillicCapitalLetterEm', [], 'Display', None],
  [ 0x41d, 'CyrillicCapitalLetterEn', [], 'Display', None],
  [ 0x41e, 'CyrillicCapitalLetterO', [], 'Display', None],
  [ 0x41f, 'CyrillicCapitalLetterPe', [], 'Display', None],
  [ 0x420, 'CyrillicCapitalLetterEr', [], 'Display', None],
  [ 0x421, 'CyrillicCapitalLetterEs', [], 'Display', None],
  [ 0x422, 'CyrillicCapitalLetterTe', [], 'Display', None],
  [ 0x423, 'CyrillicCapitalLetterU', [], 'Display', None],
  [ 0x424, 'CyrillicCapitalLetterEf', [], 'Display', None],
  [ 0x425, 'CyrillicCapitalLetterHa', [], 'Display', None],
  [ 0x426, 'CyrillicCapitalLetterTse', [], 'Display', None],
  [ 0x427, 'CyrillicCapitalLetterChe', [], 'Display', None],
  [ 0x428, 'CyrillicCapitalLetterSha', [], 'Display', None],
  [ 0x429, 'CyrillicCapitalLetterShcha', [], 'Display', None],
  [ 0x42a, 'CyrillicCapitalLetterHardSign', [], 'Display', None],
  [ 0x42b, 'CyrillicCapitalLetterYeru', [], 'Display', None],
  [ 0x42c, 'CyrillicCapitalLetterSoftSign', [], 'Display', None],
  [ 0x42d, 'CyrillicCapitalLetterE', [], 'Display', None],
  [ 0x42e, 'CyrillicCapitalLetterYu', [], 'Display', None],
  [ 0x42f, 'CyrillicCapitalLetterYa', [], 'Display', None],
  [ 0x430, 'CyrillicSmallLetterA', [], 'Display', None],
  [ 0x431, 'CyrillicSmallLetterBe', [], 'Display', None],
  [ 0x432, 'CyrillicSmallLetterVe', [], 'Display', None],
  [ 0x433, 'CyrillicSmallLetterGhe', [], 'Display', None],
  [ 0x434, 'CyrillicSmallLetterDe', [], 'Display', None],
  [ 0x435, 'CyrillicSmallLetterIe', [], 'Display', None],
  [ 0x436, 'CyrillicSmallLetterZhe', [], 'Display', None],
  [ 0x437, 'CyrillicSmallLetterZe', [], 'Display', None],
  [ 0x438, 'CyrillicSmallLetterI', [], 'Display', None],
  [ 0x439, 'CyrillicSmallLetterShortI', [], 'Display', None],
  [ 0x43a, 'CyrillicSmallLetterKa', [], 'Display', None],
  [ 0x43b, 'CyrillicSmallLetterEl', [], 'Display', None],
  [ 0x43c, 'CyrillicSmallLetterEm', [], 'Display', None],
  [ 0x43d, 'CyrillicSmallLetterEn', [], 'Display', None],
  [ 0x43e, 'CyrillicSmallLetterO', [], 'Display', None],
  [ 0x43f, 'CyrillicSmallLetterPe', [], 'Display', None],
  [ 0x440, 'CyrillicSmallLetterEr', [], 'Display', None],
  [ 0x441, 'CyrillicSmallLetterEs', [], 'Display', None],
  [ 0x442, 'CyrillicSmallLetterTe', [], 'Display', None],
  [ 0x443, 'CyrillicSmallLetterU', [], 'Display', None],
  [ 0x444, 'CyrillicSmallLetterEf', [], 'Display', None],
  [ 0x445, 'CyrillicSmallLetterHa', [], 'Display', None],
  [ 0x446, 'CyrillicSmallLetterTse', [], 'Display', None],
  [ 0x447, 'CyrillicSmallLetterChe', [], 'Display', None],
  [ 0x448, 'CyrillicSmallLetterSha', [], 'Display', None],
  [ 0x449, 'CyrillicSmallLetterShcha', [], 'Display', None],
  [ 0x44a, 'CyrillicSmallLetterHardSign', [], 'Display', None],
  [ 0x44b, 'CyrillicSmallLetterYeru', [], 'Display', None],
  [ 0x44c, 'CyrillicSmallLetterSoftSign', [], 'Display', None],
  [ 0x44d, 'CyrillicSmallLetterE', [], 'Display', None],
  [ 0x44e, 'CyrillicSmallLetterYu', [], 'Display', None],
  [ 0x44f, 'CyrillicSmallLetterYa', [], 'Display', None],
  [ 0x450, 'CyrillicSmallLetterIeWithGrave', [], 'Display', None],
  [ 0x451, 'CyrillicSmallLetterIo', [], 'Display', None],
  [ 0x452, 'CyrillicSmallLetterDje', [], 'Display', None],
  [ 0x453, 'CyrillicSmallLetterGje', [], 'Display', None],
  [ 0x454, 'CyrillicSmallLetterUkrainianIe', [], 'Display', None],
  [ 0x455, 'CyrillicSmallLetterDze', [], 'Display', None],
  [ 0x456, 'CyrillicSmallLetterByelorussianukrainianI', [], 'Display', None],
  [ 0x457, 'CyrillicSmallLetterYi', [], 'Display', None],
  [ 0x458, 'CyrillicSmallLetterJe', [], 'Display', None],
  [ 0x459, 'CyrillicSmallLetterLje', [], 'Display', None],
  [ 0x45a, 'CyrillicSmallLetterNje', [], 'Display', None],
  [ 0x45b, 'CyrillicSmallLetterTshe', [], 'Display', None],
  [ 0x45c, 'CyrillicSmallLetterKje', [], 'Display', None],
  [ 0x45d, 'CyrillicSmallLetterIWithGrave', [], 'Display', None],
  [ 0x45e, 'CyrillicSmallLetterShortU', [], 'Display', None],
  [ 0x45f, 'CyrillicSmallLetterDzhe', [], 'Display', None],
  [ 0x460, 'CyrillicCapitalLetterOmega', [], 'Display', None],
  [ 0x461, 'CyrillicSmallLetterOmega', [], 'Display', None],
  [ 0x462, 'CyrillicCapitalLetterYat', [], 'Display', None],
  [ 0x463, 'CyrillicSmallLetterYat', [], 'Display', None],
  [ 0x464, 'CyrillicCapitalLetterIotifiedE', [], 'Display', None],
  [ 0x465, 'CyrillicSmallLetterIotifiedE', [], 'Display', None],
  [ 0x466, 'CyrillicCapitalLetterLittleYus', [], 'Display', None],
  [ 0x467, 'CyrillicSmallLetterLittleYus', [], 'Display', None],
  [ 0x468, 'CyrillicCapitalLetterIotifiedLittleYus', [], 'Display', None],
  [ 0x469, 'CyrillicSmallLetterIotifiedLittleYus', [], 'Display', None],
  [ 0x46a, 'CyrillicCapitalLetterBigYus', [], 'Display', None],
  [ 0x46b, 'CyrillicSmallLetterBigYus', [], 'Display', None],
  [ 0x46c, 'CyrillicCapitalLetterIotifiedBigYus', [], 'Display', None],
  [ 0x46d, 'CyrillicSmallLetterIotifiedBigYus', [], 'Display', None],
  [ 0x46e, 'CyrillicCapitalLetterKsi', [], 'Display', None],
  [ 0x46f, 'CyrillicSmallLetterKsi', [], 'Display', None],
  [ 0x470, 'CyrillicCapitalLetterPsi', [], 'Display', None],
  [ 0x471, 'CyrillicSmallLetterPsi', [], 'Display', None],
  [ 0x472, 'CyrillicCapitalLetterFita', [], 'Display', None],
  [ 0x473, 'CyrillicSmallLetterFita', [], 'Display', None],
  [ 0x474, 'CyrillicCapitalLetterIzhitsa', [], 'Display', None],
  [ 0x475, 'CyrillicSmallLetterIzhitsa', [], 'Display', None],
  [ 0x476, 'CyrillicCapitalLetterIzhitsaWithDoubleGraveAccent', [], 'Display', None],
  [ 0x477, 'CyrillicSmallLetterIzhitsaWithDoubleGraveAccent', [], 'Display', None],
  [ 0x478, 'CyrillicCapitalLetterUk', [], 'Display', None],
  [ 0x479, 'CyrillicSmallLetterUk', [], 'Display', None],
  [ 0x47a, 'CyrillicCapitalLetterRoundOmega', [], 'Display', None],
  [ 0x47b, 'CyrillicSmallLetterRoundOmega', [], 'Display', None],
  [ 0x47c, 'CyrillicCapitalLetterOmegaWithTitlo', [], 'Display', None],
  [ 0x47d, 'CyrillicSmallLetterOmegaWithTitlo', [], 'Display', None],
  [ 0x47e, 'CyrillicCapitalLetterOt', [], 'Display', None],
  [ 0x47f, 'CyrillicSmallLetterOt', [], 'Display', None],
  [ 0x480, 'CyrillicCapitalLetterKoppa', [], 'Display', None],
  [ 0x481, 'CyrillicSmallLetterKoppa', [], 'Display', None],
  [ 0x482, 'CyrillicThousandsSign', [], 'Display', None],
  [ 0x483, 'CombiningCyrillicTitlo', [], 'Display', None],
  [ 0x484, 'CombiningCyrillicPalatalization', [], 'Display', None],
  [ 0x485, 'CombiningCyrillicDasiaPneumata', [], 'Display', None],
  [ 0x486, 'CombiningCyrillicPsiliPneumata', [], 'Display', None],
  [ 0x487, 'CombiningCyrillicPokrytie', [], 'Display', None],
  [ 0x488, 'CombiningCyrillicHundredThousandsSign', [], 'Display', None],
  [ 0x489, 'CombiningCyrillicMillionsSign', [], 'Display', None],
  [ 0x48a, 'CyrillicCapitalLetterShortIWithTail', [], 'Display', None],
  [ 0x48b, 'CyrillicSmallLetterShortIWithTail', [], 'Display', None],
  [ 0x48c, 'CyrillicCapitalLetterSemisoftSign', [], 'Display', None],
  [ 0x48d, 'CyrillicSmallLetterSemisoftSign', [], 'Display', None],
  [ 0x48e, 'CyrillicCapitalLetterErWithTick', [], 'Display', None],
  [ 0x48f, 'CyrillicSmallLetterErWithTick', [], 'Display', None],
  [ 0x490, 'CyrillicCapitalLetterGheWithUpturn', [], 'Display', None],
  [ 0x491, 'CyrillicSmallLetterGheWithUpturn', [], 'Display', None],
  [ 0x492, 'CyrillicCapitalLetterGheWithStroke', [], 'Display', None],
  [ 0x493, 'CyrillicSmallLetterGheWithStroke', [], 'Display', None],
  [ 0x494, 'CyrillicCapitalLetterGheWithMiddleHook', [], 'Display', None],
  [ 0x495, 'CyrillicSmallLetterGheWithMiddleHook', [], 'Display', None],
  [ 0x496, 'CyrillicCapitalLetterZheWithDescender', [], 'Display', None],
  [ 0x497, 'CyrillicSmallLetterZheWithDescender', [], 'Display', None],
  [ 0x498, 'CyrillicCapitalLetterZeWithDescender', [], 'Display', None],
  [ 0x499, 'CyrillicSmallLetterZeWithDescender', [], 'Display', None],
  [ 0x49a, 'CyrillicCapitalLetterKaWithDescender', [], 'Display', None],
  [ 0x49b, 'CyrillicSmallLetterKaWithDescender', [], 'Display', None],
  [ 0x49c, 'CyrillicCapitalLetterKaWithVerticalStroke', [], 'Display', None],
  [ 0x49d, 'CyrillicSmallLetterKaWithVerticalStroke', [], 'Display', None],
  [ 0x49e, 'CyrillicCapitalLetterKaWithStroke', [], 'Display', None],
  [ 0x49f, 'CyrillicSmallLetterKaWithStroke', [], 'Display', None],
  [ 0x4a0, 'CyrillicCapitalLetterBashkirKa', [], 'Display', None],
  [ 0x4a1, 'CyrillicSmallLetterBashkirKa', [], 'Display', None],
  [ 0x4a2, 'CyrillicCapitalLetterEnWithDescender', [], 'Display', None],
  [ 0x4a3, 'CyrillicSmallLetterEnWithDescender', [], 'Display', None],
  [ 0x4a4, 'CyrillicCapitalLigatureEnGhe', [], 'Display', None],
  [ 0x4a5, 'CyrillicSmallLigatureEnGhe', [], 'Display', None],
  [ 0x4a6, 'CyrillicCapitalLetterPeWithMiddleHook', [], 'Display', None],
  [ 0x4a7, 'CyrillicSmallLetterPeWithMiddleHook', [], 'Display', None],
  [ 0x4a8, 'CyrillicCapitalLetterAbkhasianHa', [], 'Display', None],
  [ 0x4a9, 'CyrillicSmallLetterAbkhasianHa', [], 'Display', None],
  [ 0x4aa, 'CyrillicCapitalLetterEsWithDescender', [], 'Display', None],
  [ 0x4ab, 'CyrillicSmallLetterEsWithDescender', [], 'Display', None],
  [ 0x4ac, 'CyrillicCapitalLetterTeWithDescender', [], 'Display', None],
  [ 0x4ad, 'CyrillicSmallLetterTeWithDescender', [], 'Display', None],
  [ 0x4ae, 'CyrillicCapitalLetterStraightU', [], 'Display', None],
  [ 0x4af, 'CyrillicSmallLetterStraightU', [], 'Display', None],
  [ 0x4b0, 'CyrillicCapitalLetterStraightUWithStroke', [], 'Display', None],
  [ 0x4b1, 'CyrillicSmallLetterStraightUWithStroke', [], 'Display', None],
  [ 0x4b2, 'CyrillicCapitalLetterHaWithDescender', [], 'Display', None],
  [ 0x4b3, 'CyrillicSmallLetterHaWithDescender', [], 'Display', None],
  [ 0x4b4, 'CyrillicCapitalLigatureTeTse', [], 'Display', None],
  [ 0x4b5, 'CyrillicSmallLigatureTeTse', [], 'Display', None],
  [ 0x4b6, 'CyrillicCapitalLetterCheWithDescender', [], 'Display', None],
  [ 0x4b7, 'CyrillicSmallLetterCheWithDescender', [], 'Display', None],
  [ 0x4b8, 'CyrillicCapitalLetterCheWithVerticalStroke', [], 'Display', None],
  [ 0x4b9, 'CyrillicSmallLetterCheWithVerticalStroke', [], 'Display', None],
  [ 0x4ba, 'CyrillicCapitalLetterShha', [], 'Display', None],
  [ 0x4bb, 'CyrillicSmallLetterShha', [], 'Display', None],
  [ 0x4bc, 'CyrillicCapitalLetterAbkhasianChe', [], 'Display', None],
  [ 0x4bd, 'CyrillicSmallLetterAbkhasianChe', [], 'Display', None],
  [ 0x4be, 'CyrillicCapitalLetterAbkhasianCheWithDescender', [], 'Display', None],
  [ 0x4bf, 'CyrillicSmallLetterAbkhasianCheWithDescender', [], 'Display', None],
  [ 0x4c0, 'CyrillicLetterPalochka', [], 'Display', None],
  [ 0x4c1, 'CyrillicCapitalLetterZheWithBreve', [], 'Display', None],
  [ 0x4c2, 'CyrillicSmallLetterZheWithBreve', [], 'Display', None],
  [ 0x4c3, 'CyrillicCapitalLetterKaWithHook', [], 'Display', None],
  [ 0x4c4, 'CyrillicSmallLetterKaWithHook', [], 'Display', None],
  [ 0x4c5, 'CyrillicCapitalLetterElWithTail', [], 'Display', None],
  [ 0x4c6, 'CyrillicSmallLetterElWithTail', [], 'Display', None],
  [ 0x4c7, 'CyrillicCapitalLetterEnWithHook', [], 'Display', None],
  [ 0x4c8, 'CyrillicSmallLetterEnWithHook', [], 'Display', None],
  [ 0x4c9, 'CyrillicCapitalLetterEnWithTail', [], 'Display', None],
  [ 0x4ca, 'CyrillicSmallLetterEnWithTail', [], 'Display', None],
  [ 0x4cb, 'CyrillicCapitalLetterKhakassianChe', [], 'Display', None],
  [ 0x4cc, 'CyrillicSmallLetterKhakassianChe', [], 'Display', None],
  [ 0x4cd, 'CyrillicCapitalLetterEmWithTail', [], 'Display', None],
  [ 0x4ce, 'CyrillicSmallLetterEmWithTail', [], 'Display', None],
  [ 0x4cf, 'CyrillicSmallLetterPalochka', [], 'Display', None],
  [ 0x4d0, 'CyrillicCapitalLetterAWithBreve', [], 'Display', None],
  [ 0x4d1, 'CyrillicSmallLetterAWithBreve', [], 'Display', None],
  [ 0x4d2, 'CyrillicCapitalLetterAWithDiaeresis', [], 'Display', None],
  [ 0x4d3, 'CyrillicSmallLetterAWithDiaeresis', [], 'Display', None],
  [ 0x4d4, 'CyrillicCapitalLigatureAIe', [], 'Display', None],
  [ 0x4d5, 'CyrillicSmallLigatureAIe', [], 'Display', None],
  [ 0x4d6, 'CyrillicCapitalLetterIeWithBreve', [], 'Display', None],
  [ 0x4d7, 'CyrillicSmallLetterIeWithBreve', [], 'Display', None],
  [ 0x4d8, 'CyrillicCapitalLetterSchwa', [], 'Display', None],
  [ 0x4d9, 'CyrillicSmallLetterSchwa', [], 'Display', None],
  [ 0x4da, 'CyrillicCapitalLetterSchwaWithDiaeresis', [], 'Display', None],
  [ 0x4db, 'CyrillicSmallLetterSchwaWithDiaeresis', [], 'Display', None],
  [ 0x4dc, 'CyrillicCapitalLetterZheWithDiaeresis', [], 'Display', None],
  [ 0x4dd, 'CyrillicSmallLetterZheWithDiaeresis', [], 'Display', None],
  [ 0x4de, 'CyrillicCapitalLetterZeWithDiaeresis', [], 'Display', None],
  [ 0x4df, 'CyrillicSmallLetterZeWithDiaeresis', [], 'Display', None],
  [ 0x4e0, 'CyrillicCapitalLetterAbkhasianDze', [], 'Display', None],
  [ 0x4e1, 'CyrillicSmallLetterAbkhasianDze', [], 'Display', None],
  [ 0x4e2, 'CyrillicCapitalLetterIWithMacron', [], 'Display', None],
  [ 0x4e3, 'CyrillicSmallLetterIWithMacron', [], 'Display', None],
  [ 0x4e4, 'CyrillicCapitalLetterIWithDiaeresis', [], 'Display', None],
  [ 0x4e5, 'CyrillicSmallLetterIWithDiaeresis', [], 'Display', None],
  [ 0x4e6, 'CyrillicCapitalLetterOWithDiaeresis', [], 'Display', None],
  [ 0x4e7, 'CyrillicSmallLetterOWithDiaeresis', [], 'Display', None],
  [ 0x4e8, 'CyrillicCapitalLetterBarredO', [], 'Display', None],
  [ 0x4e9, 'CyrillicSmallLetterBarredO', [], 'Display', None],
  [ 0x4ea, 'CyrillicCapitalLetterBarredOWithDiaeresis', [], 'Display', None],
  [ 0x4eb, 'CyrillicSmallLetterBarredOWithDiaeresis', [], 'Display', None],
  [ 0x4ec, 'CyrillicCapitalLetterEWithDiaeresis', [], 'Display', None],
  [ 0x4ed, 'CyrillicSmallLetterEWithDiaeresis', [], 'Display', None],
  [ 0x4ee, 'CyrillicCapitalLetterUWithMacron', [], 'Display', None],
  [ 0x4ef, 'CyrillicSmallLetterUWithMacron', [], 'Display', None],
  [ 0x4f0, 'CyrillicCapitalLetterUWithDiaeresis', [], 'Display', None],
  [ 0x4f1, 'CyrillicSmallLetterUWithDiaeresis', [], 'Display', None],
  [ 0x4f2, 'CyrillicCapitalLetterUWithDoubleAcute', [], 'Display', None],
  [ 0x4f3, 'CyrillicSmallLetterUWithDoubleAcute', [], 'Display', None],
  [ 0x4f4, 'CyrillicCapitalLetterCheWithDiaeresis', [], 'Display', None],
  [ 0x4f5, 'CyrillicSmallLetterCheWithDiaeresis', [], 'Display', None],
  [ 0x4f6, 'CyrillicCapitalLetterGheWithDescender', [], 'Display', None],
  [ 0x4f7, 'CyrillicSmallLetterGheWithDescender', [], 'Display', None],
  [ 0x4f8, 'CyrillicCapitalLetterYeruWithDiaeresis', [], 'Display', None],
  [ 0x4f9, 'CyrillicSmallLetterYeruWithDiaeresis', [], 'Display', None],
  [ 0x4fa, 'CyrillicCapitalLetterGheWithStrokeAndHook', [], 'Display', None],
  [ 0x4fb, 'CyrillicSmallLetterGheWithStrokeAndHook', [], 'Display', None],
  [ 0x4fc, 'CyrillicCapitalLetterHaWithHook', [], 'Display', None],
  [ 0x4fd, 'CyrillicSmallLetterHaWithHook', [], 'Display', None],
  [ 0x4fe, 'CyrillicCapitalLetterHaWithStroke', [], 'Display', None],
  [ 0x4ff, 'CyrillicSmallLetterHaWithStroke', [], 'Display', None],
  [ 0x500, 'CyrillicCapitalLetterKomiDe', [], 'Display', None],
  [ 0x501, 'CyrillicSmallLetterKomiDe', [], 'Display', None],
  [ 0x502, 'CyrillicCapitalLetterKomiDje', [], 'Display', None],
  [ 0x503, 'CyrillicSmallLetterKomiDje', [], 'Display', None],
  [ 0x504, 'CyrillicCapitalLetterKomiZje', [], 'Display', None],
  [ 0x505, 'CyrillicSmallLetterKomiZje', [], 'Display', None],
  [ 0x506, 'CyrillicCapitalLetterKomiDzje', [], 'Display', None],
  [ 0x507, 'CyrillicSmallLetterKomiDzje', [], 'Display', None],
  [ 0x508, 'CyrillicCapitalLetterKomiLje', [], 'Display', None],
  [ 0x509, 'CyrillicSmallLetterKomiLje', [], 'Display', None],
  [ 0x50a, 'CyrillicCapitalLetterKomiNje', [], 'Display', None],
  [ 0x50b, 'CyrillicSmallLetterKomiNje', [], 'Display', None],
  [ 0x50c, 'CyrillicCapitalLetterKomiSje', [], 'Display', None],
  [ 0x50d, 'CyrillicSmallLetterKomiSje', [], 'Display', None],
  [ 0x50e, 'CyrillicCapitalLetterKomiTje', [], 'Display', None],
  [ 0x50f, 'CyrillicSmallLetterKomiTje', [], 'Display', None],
  [ 0x510, 'CyrillicCapitalLetterReversedZe', [], 'Display', None],
  [ 0x511, 'CyrillicSmallLetterReversedZe', [], 'Display', None],
  [ 0x512, 'CyrillicCapitalLetterElWithHook', [], 'Display', None],
  [ 0x513, 'CyrillicSmallLetterElWithHook', [], 'Display', None],
  [ 0x514, 'CyrillicCapitalLetterLha', [], 'Display', None],
  [ 0x515, 'CyrillicSmallLetterLha', [], 'Display', None],
  [ 0x516, 'CyrillicCapitalLetterRha', [], 'Display', None],
  [ 0x517, 'CyrillicSmallLetterRha', [], 'Display', None],
  [ 0x518, 'CyrillicCapitalLetterYae', [], 'Display', None],
  [ 0x519, 'CyrillicSmallLetterYae', [], 'Display', None],
  [ 0x51a, 'CyrillicCapitalLetterQa', [], 'Display', None],
  [ 0x51b, 'CyrillicSmallLetterQa', [], 'Display', None],
  [ 0x51c, 'CyrillicCapitalLetterWe', [], 'Display', None],
  [ 0x51d, 'CyrillicSmallLetterWe', [], 'Display', None],
  [ 0x51e, 'CyrillicCapitalLetterAleutKa', [], 'Display', None],
  [ 0x51f, 'CyrillicSmallLetterAleutKa', [], 'Display', None],
  [ 0x520, 'CyrillicCapitalLetterElWithMiddleHook', [], 'Display', None],
  [ 0x521, 'CyrillicSmallLetterElWithMiddleHook', [], 'Display', None],
  [ 0x522, 'CyrillicCapitalLetterEnWithMiddleHook', [], 'Display', None],
  [ 0x523, 'CyrillicSmallLetterEnWithMiddleHook', [], 'Display', None],
  [ 0x524, 'UnicodeCharacter0x524', [], 'Display', None],
  [ 0x525, 'UnicodeCharacter0x525', [], 'Display', None],
  [ 0x526, 'UnicodeCharacter0x526', [], 'Display', None],
  [ 0x527, 'UnicodeCharacter0x527', [], 'Display', None],
  [ 0x528, 'UnicodeCharacter0x528', [], 'Display', None],
  [ 0x529, 'UnicodeCharacter0x529', [], 'Display', None],
  [ 0x52a, 'UnicodeCharacter0x52a', [], 'Display', None],
  [ 0x52b, 'UnicodeCharacter0x52b', [], 'Display', None],
  [ 0x52c, 'UnicodeCharacter0x52c', [], 'Display', None],
  [ 0x52d, 'UnicodeCharacter0x52d', [], 'Display', None],
  [ 0x52e, 'UnicodeCharacter0x52e', [], 'Display', None],
  [ 0x52f, 'UnicodeCharacter0x52f', [], 'Display', None],
  [ 0x530, 'UnicodeCharacter0x530', [], 'Display', None],
  [ 0x531, 'ArmenianCapitalLetterAyb', [], 'Display', None],
  [ 0x532, 'ArmenianCapitalLetterBen', [], 'Display', None],
  [ 0x533, 'ArmenianCapitalLetterGim', [], 'Display', None],
  [ 0x534, 'ArmenianCapitalLetterDa', [], 'Display', None],
  [ 0x535, 'ArmenianCapitalLetterEch', [], 'Display', None],
  [ 0x536, 'ArmenianCapitalLetterZa', [], 'Display', None],
  [ 0x537, 'ArmenianCapitalLetterEh', [], 'Display', None],
  [ 0x538, 'ArmenianCapitalLetterEt', [], 'Display', None],
  [ 0x539, 'ArmenianCapitalLetterTo', [], 'Display', None],
  [ 0x53a, 'ArmenianCapitalLetterZhe', [], 'Display', None],
  [ 0x53b, 'ArmenianCapitalLetterIni', [], 'Display', None],
  [ 0x53c, 'ArmenianCapitalLetterLiwn', [], 'Display', None],
  [ 0x53d, 'ArmenianCapitalLetterXeh', [], 'Display', None],
  [ 0x53e, 'ArmenianCapitalLetterCa', [], 'Display', None],
  [ 0x53f, 'ArmenianCapitalLetterKen', [], 'Display', None],
  [ 0x540, 'ArmenianCapitalLetterHo', [], 'Display', None],
  [ 0x541, 'ArmenianCapitalLetterJa', [], 'Display', None],
  [ 0x542, 'ArmenianCapitalLetterGhad', [], 'Display', None],
  [ 0x543, 'ArmenianCapitalLetterCheh', [], 'Display', None],
  [ 0x544, 'ArmenianCapitalLetterMen', [], 'Display', None],
  [ 0x545, 'ArmenianCapitalLetterYi', [], 'Display', None],
  [ 0x546, 'ArmenianCapitalLetterNow', [], 'Display', None],
  [ 0x547, 'ArmenianCapitalLetterSha', [], 'Display', None],
  [ 0x548, 'ArmenianCapitalLetterVo', [], 'Display', None],
  [ 0x549, 'ArmenianCapitalLetterCha', [], 'Display', None],
  [ 0x54a, 'ArmenianCapitalLetterPeh', [], 'Display', None],
  [ 0x54b, 'ArmenianCapitalLetterJheh', [], 'Display', None],
  [ 0x54c, 'ArmenianCapitalLetterRa', [], 'Display', None],
  [ 0x54d, 'ArmenianCapitalLetterSeh', [], 'Display', None],
  [ 0x54e, 'ArmenianCapitalLetterVew', [], 'Display', None],
  [ 0x54f, 'ArmenianCapitalLetterTiwn', [], 'Display', None],
  [ 0x550, 'ArmenianCapitalLetterReh', [], 'Display', None],
  [ 0x551, 'ArmenianCapitalLetterCo', [], 'Display', None],
  [ 0x552, 'ArmenianCapitalLetterYiwn', [], 'Display', None],
  [ 0x553, 'ArmenianCapitalLetterPiwr', [], 'Display', None],
  [ 0x554, 'ArmenianCapitalLetterKeh', [], 'Display', None],
  [ 0x555, 'ArmenianCapitalLetterOh', [], 'Display', None],
  [ 0x556, 'ArmenianCapitalLetterFeh', [], 'Display', None],
  [ 0x557, 'UnicodeCharacter0x557', [], 'Display', None],
  [ 0x558, 'UnicodeCharacter0x558', [], 'Display', None],
  [ 0x559, 'ArmenianModifierLetterLeftHalfRing', [], 'Display', None],
  [ 0x55a, 'ArmenianApostrophe', [], 'Display', None],
  [ 0x55b, 'ArmenianEmphasisMark', [], 'Display', None],
  [ 0x55c, 'ArmenianExclamationMark', [], 'Display', None],
  [ 0x55d, 'ArmenianComma', [], 'Display', None],
  [ 0x55e, 'ArmenianQuestionMark', [], 'Display', None],
  [ 0x55f, 'ArmenianAbbreviationMark', [], 'Display', None],
  [ 0x560, 'UnicodeCharacter0x560', [], 'Display', None],
  [ 0x561, 'ArmenianSmallLetterAyb', [], 'Display', None],
  [ 0x562, 'ArmenianSmallLetterBen', [], 'Display', None],
  [ 0x563, 'ArmenianSmallLetterGim', [], 'Display', None],
  [ 0x564, 'ArmenianSmallLetterDa', [], 'Display', None],
  [ 0x565, 'ArmenianSmallLetterEch', [], 'Display', None],
  [ 0x566, 'ArmenianSmallLetterZa', [], 'Display', None],
  [ 0x567, 'ArmenianSmallLetterEh', [], 'Display', None],
  [ 0x568, 'ArmenianSmallLetterEt', [], 'Display', None],
  [ 0x569, 'ArmenianSmallLetterTo', [], 'Display', None],
  [ 0x56a, 'ArmenianSmallLetterZhe', [], 'Display', None],
  [ 0x56b, 'ArmenianSmallLetterIni', [], 'Display', None],
  [ 0x56c, 'ArmenianSmallLetterLiwn', [], 'Display', None],
  [ 0x56d, 'ArmenianSmallLetterXeh', [], 'Display', None],
  [ 0x56e, 'ArmenianSmallLetterCa', [], 'Display', None],
  [ 0x56f, 'ArmenianSmallLetterKen', [], 'Display', None],
  [ 0x570, 'ArmenianSmallLetterHo', [], 'Display', None],
  [ 0x571, 'ArmenianSmallLetterJa', [], 'Display', None],
  [ 0x572, 'ArmenianSmallLetterGhad', [], 'Display', None],
  [ 0x573, 'ArmenianSmallLetterCheh', [], 'Display', None],
  [ 0x574, 'ArmenianSmallLetterMen', [], 'Display', None],
  [ 0x575, 'ArmenianSmallLetterYi', [], 'Display', None],
  [ 0x576, 'ArmenianSmallLetterNow', [], 'Display', None],
  [ 0x577, 'ArmenianSmallLetterSha', [], 'Display', None],
  [ 0x578, 'ArmenianSmallLetterVo', [], 'Display', None],
  [ 0x579, 'ArmenianSmallLetterCha', [], 'Display', None],
  [ 0x57a, 'ArmenianSmallLetterPeh', [], 'Display', None],
  [ 0x57b, 'ArmenianSmallLetterJheh', [], 'Display', None],
  [ 0x57c, 'ArmenianSmallLetterRa', [], 'Display', None],
  [ 0x57d, 'ArmenianSmallLetterSeh', [], 'Display', None],
  [ 0x57e, 'ArmenianSmallLetterVew', [], 'Display', None],
  [ 0x57f, 'ArmenianSmallLetterTiwn', [], 'Display', None],
  [ 0x580, 'ArmenianSmallLetterReh', [], 'Display', None],
  [ 0x581, 'ArmenianSmallLetterCo', [], 'Display', None],
  [ 0x582, 'ArmenianSmallLetterYiwn', [], 'Display', None],
  [ 0x583, 'ArmenianSmallLetterPiwr', [], 'Display', None],
  [ 0x584, 'ArmenianSmallLetterKeh', [], 'Display', None],
  [ 0x585, 'ArmenianSmallLetterOh', [], 'Display', None],
  [ 0x586, 'ArmenianSmallLetterFeh', [], 'Display', None],
  [ 0x587, 'ArmenianSmallLigatureEchYiwn', [], 'Display', None],
  [ 0x588, 'UnicodeCharacter0x588', [], 'Display', None],
  [ 0x589, 'ArmenianFullStop', [], 'Display', None],
  [ 0x58a, 'ArmenianHyphen', [], 'Display', None],
  [ 0x58b, 'UnicodeCharacter0x58b', [], 'Display', None],
  [ 0x58c, 'UnicodeCharacter0x58c', [], 'Display', None],
  [ 0x58d, 'UnicodeCharacter0x58d', [], 'Display', None],
  [ 0x58e, 'UnicodeCharacter0x58e', [], 'Display', None],
  [ 0x58f, 'UnicodeCharacter0x58f', [], 'Display', None],
  [ 0x590, 'UnicodeCharacter0x590', [], 'Display', None],
  [ 0x591, 'HebrewAccentEtnahta', [], 'Display', None],
  [ 0x592, 'HebrewAccentSegol', [], 'Display', None],
  [ 0x593, 'HebrewAccentShalshelet', [], 'Display', None],
  [ 0x594, 'HebrewAccentZaqefQatan', [], 'Display', None],
  [ 0x595, 'HebrewAccentZaqefGadol', [], 'Display', None],
  [ 0x596, 'HebrewAccentTipeha', [], 'Display', None],
  [ 0x597, 'HebrewAccentRevia', [], 'Display', None],
  [ 0x598, 'HebrewAccentZarqa', [], 'Display', None],
  [ 0x599, 'HebrewAccentPashta', [], 'Display', None],
  [ 0x59a, 'HebrewAccentYetiv', [], 'Display', None],
  [ 0x59b, 'HebrewAccentTevir', [], 'Display', None],
  [ 0x59c, 'HebrewAccentGeresh', [], 'Display', None],
  [ 0x59d, 'HebrewAccentGereshMuqdam', [], 'Display', None],
  [ 0x59e, 'HebrewAccentGershayim', [], 'Display', None],
  [ 0x59f, 'HebrewAccentQarneyPara', [], 'Display', None],
  [ 0x5a0, 'HebrewAccentTelishaGedola', [], 'Display', None],
  [ 0x5a1, 'HebrewAccentPazer', [], 'Display', None],
  [ 0x5a2, 'HebrewAccentAtnahHafukh', [], 'Display', None],
  [ 0x5a3, 'HebrewAccentMunah', [], 'Display', None],
  [ 0x5a4, 'HebrewAccentMahapakh', [], 'Display', None],
  [ 0x5a5, 'HebrewAccentMerkha', [], 'Display', None],
  [ 0x5a6, 'HebrewAccentMerkhaKefula', [], 'Display', None],
  [ 0x5a7, 'HebrewAccentDarga', [], 'Display', None],
  [ 0x5a8, 'HebrewAccentQadma', [], 'Display', None],
  [ 0x5a9, 'HebrewAccentTelishaQetana', [], 'Display', None],
  [ 0x5aa, 'HebrewAccentYerahBenYomo', [], 'Display', None],
  [ 0x5ab, 'HebrewAccentOle', [], 'Display', None],
  [ 0x5ac, 'HebrewAccentIluy', [], 'Display', None],
  [ 0x5ad, 'HebrewAccentDehi', [], 'Display', None],
  [ 0x5ae, 'HebrewAccentZinor', [], 'Display', None],
  [ 0x5af, 'HebrewMarkMasoraCircle', [], 'Display', None],
  [ 0x5b0, 'HebrewPointSheva', [], 'Display', None],
  [ 0x5b1, 'HebrewPointHatafSegol', [], 'Display', None],
  [ 0x5b2, 'HebrewPointHatafPatah', [], 'Display', None],
  [ 0x5b3, 'HebrewPointHatafQamats', [], 'Display', None],
  [ 0x5b4, 'HebrewPointHiriq', [], 'Display', None],
  [ 0x5b5, 'HebrewPointTsere', [], 'Display', None],
  [ 0x5b6, 'HebrewPointSegol', [], 'Display', None],
  [ 0x5b7, 'HebrewPointPatah', [], 'Display', None],
  [ 0x5b8, 'HebrewPointQamats', [], 'Display', None],
  [ 0x5b9, 'HebrewPointHolam', [], 'Display', None],
  [ 0x5ba, 'HebrewPointHolamHaserForVav', [], 'Display', None],
  [ 0x5bb, 'HebrewPointQubuts', [], 'Display', None],
  [ 0x5bc, 'HebrewPointDageshOrMapiq', [], 'Display', None],
  [ 0x5bd, 'HebrewPointMeteg', [], 'Display', None],
  [ 0x5be, 'HebrewPunctuationMaqaf', [], 'Display', None],
  [ 0x5bf, 'HebrewPointRafe', [], 'Display', None],
  [ 0x5c0, 'HebrewPunctuationPaseq', [], 'Display', None],
  [ 0x5c1, 'HebrewPointShinDot', [], 'Display', None],
  [ 0x5c2, 'HebrewPointSinDot', [], 'Display', None],
  [ 0x5c3, 'HebrewPunctuationSofPasuq', [], 'Display', None],
  [ 0x5c4, 'HebrewMarkUpperDot', [], 'Display', None],
  [ 0x5c5, 'HebrewMarkLowerDot', [], 'Display', None],
  [ 0x5c6, 'HebrewPunctuationNunHafukha', [], 'Display', None],
  [ 0x5c7, 'HebrewPointQamatsQatan', [], 'Display', None],
  [ 0x5c8, 'UnicodeCharacter0x5c8', [], 'Display', None],
  [ 0x5c9, 'UnicodeCharacter0x5c9', [], 'Display', None],
  [ 0x5ca, 'UnicodeCharacter0x5ca', [], 'Display', None],
  [ 0x5cb, 'UnicodeCharacter0x5cb', [], 'Display', None],
  [ 0x5cc, 'UnicodeCharacter0x5cc', [], 'Display', None],
  [ 0x5cd, 'UnicodeCharacter0x5cd', [], 'Display', None],
  [ 0x5ce, 'UnicodeCharacter0x5ce', [], 'Display', None],
  [ 0x5cf, 'UnicodeCharacter0x5cf', [], 'Display', None],
  [ 0x5d0, 'HebrewLetterAlef', [], 'Display', None],
  [ 0x5d1, 'HebrewLetterBet', [], 'Display', None],
  [ 0x5d2, 'HebrewLetterGimel', [], 'Display', None],
  [ 0x5d3, 'HebrewLetterDalet', [], 'Display', None],
  [ 0x5d4, 'HebrewLetterHe', [], 'Display', None],
  [ 0x5d5, 'HebrewLetterVav', [], 'Display', None],
  [ 0x5d6, 'HebrewLetterZayin', [], 'Display', None],
  [ 0x5d7, 'HebrewLetterHet', [], 'Display', None],
  [ 0x5d8, 'HebrewLetterTet', [], 'Display', None],
  [ 0x5d9, 'HebrewLetterYod', [], 'Display', None],
  [ 0x5da, 'HebrewLetterFinalKaf', [], 'Display', None],
  [ 0x5db, 'HebrewLetterKaf', [], 'Display', None],
  [ 0x5dc, 'HebrewLetterLamed', [], 'Display', None],
  [ 0x5dd, 'HebrewLetterFinalMem', [], 'Display', None],
  [ 0x5de, 'HebrewLetterMem', [], 'Display', None],
  [ 0x5df, 'HebrewLetterFinalNun', [], 'Display', None],
  [ 0x5e0, 'HebrewLetterNun', [], 'Display', None],
  [ 0x5e1, 'HebrewLetterSamekh', [], 'Display', None],
  [ 0x5e2, 'HebrewLetterAyin', [], 'Display', None],
  [ 0x5e3, 'HebrewLetterFinalPe', [], 'Display', None],
  [ 0x5e4, 'HebrewLetterPe', [], 'Display', None],
  [ 0x5e5, 'HebrewLetterFinalTsadi', [], 'Display', None],
  [ 0x5e6, 'HebrewLetterTsadi', [], 'Display', None],
  [ 0x5e7, 'HebrewLetterQof', [], 'Display', None],
  [ 0x5e8, 'HebrewLetterResh', [], 'Display', None],
  [ 0x5e9, 'HebrewLetterShin', [], 'Display', None],
  [ 0x5ea, 'HebrewLetterTav', [], 'Display', None],
  [ 0x5eb, 'UnicodeCharacter0x5eb', [], 'Display', None],
  [ 0x5ec, 'UnicodeCharacter0x5ec', [], 'Display', None],
  [ 0x5ed, 'UnicodeCharacter0x5ed', [], 'Display', None],
  [ 0x5ee, 'UnicodeCharacter0x5ee', [], 'Display', None],
  [ 0x5ef, 'UnicodeCharacter0x5ef', [], 'Display', None],
  [ 0x5f0, 'HebrewLigatureYiddishDoubleVav', [], 'Display', None],
  [ 0x5f1, 'HebrewLigatureYiddishVavYod', [], 'Display', None],
  [ 0x5f2, 'HebrewLigatureYiddishDoubleYod', [], 'Display', None],
  [ 0x5f3, 'HebrewPunctuationGeresh', [], 'Display', None],
  [ 0x5f4, 'HebrewPunctuationGershayim', [], 'Display', None],
  [ 0x5f5, 'UnicodeCharacter0x5f5', [], 'Display', None],
  [ 0x5f6, 'UnicodeCharacter0x5f6', [], 'Display', None],
  [ 0x5f7, 'UnicodeCharacter0x5f7', [], 'Display', None],
  [ 0x5f8, 'UnicodeCharacter0x5f8', [], 'Display', None],
  [ 0x5f9, 'UnicodeCharacter0x5f9', [], 'Display', None],
  [ 0x5fa, 'UnicodeCharacter0x5fa', [], 'Display', None],
  [ 0x5fb, 'UnicodeCharacter0x5fb', [], 'Display', None],
  [ 0x5fc, 'UnicodeCharacter0x5fc', [], 'Display', None],
  [ 0x5fd, 'UnicodeCharacter0x5fd', [], 'Display', None],
  [ 0x5fe, 'UnicodeCharacter0x5fe', [], 'Display', None],
  [ 0x5ff, 'UnicodeCharacter0x5ff', [], 'Display', None],
  [ 0x600, 'ArabicNumberSign', [], 'Display', None],
  [ 0x601, 'ArabicSignSanah', [], 'Display', None],
  [ 0x602, 'ArabicFootnoteMarker', [], 'Display', None],
  [ 0x603, 'ArabicSignSafha', [], 'Display', None],
  [ 0x604, 'UnicodeCharacter0x604', [], 'Display', None],
  [ 0x605, 'UnicodeCharacter0x605', [], 'Display', None],
  [ 0x606, 'ArabicindicCubeRoot', [], 'Display', None],
  [ 0x607, 'ArabicindicFourthRoot', [], 'Display', None],
  [ 0x608, 'ArabicRay', [], 'Display', None],
  [ 0x609, 'ArabicindicPerMilleSign', [], 'Display', None],
  [ 0x60a, 'ArabicindicPerTenThousandSign', [], 'Display', None],
  [ 0x60b, 'AfghaniSign', [], 'Display', None],
  [ 0x60c, 'ArabicComma', [], 'Display', None],
  [ 0x60d, 'ArabicDateSeparator', [], 'Display', None],
  [ 0x60e, 'ArabicPoeticVerseSign', [], 'Display', None],
  [ 0x60f, 'ArabicSignMisra', [], 'Display', None],
  [ 0x610, 'ArabicSignSallallahouAlayheWassallam', [], 'Display', None],
  [ 0x611, 'ArabicSignAlayheAssallam', [], 'Display', None],
  [ 0x612, 'ArabicSignRahmatullahAlayhe', [], 'Display', None],
  [ 0x613, 'ArabicSignRadiAllahouAnhu', [], 'Display', None],
  [ 0x614, 'ArabicSignTakhallus', [], 'Display', None],
  [ 0x615, 'ArabicSmallHighTah', [], 'Display', None],
  [ 0x616, 'ArabicSmallHighLigatureAlefWithLamWithYeh', [], 'Display', None],
  [ 0x617, 'ArabicSmallHighZain', [], 'Display', None],
  [ 0x618, 'ArabicSmallFatha', [], 'Display', None],
  [ 0x619, 'ArabicSmallDamma', [], 'Display', None],
  [ 0x61a, 'ArabicSmallKasra', [], 'Display', None],
  [ 0x61b, 'ArabicSemicolon', [], 'Display', None],
  [ 0x61c, 'UnicodeCharacter0x61c', [], 'Display', None],
  [ 0x61d, 'UnicodeCharacter0x61d', [], 'Display', None],
  [ 0x61e, 'ArabicTripleDotPunctuationMark', [], 'Display', None],
  [ 0x61f, 'ArabicQuestionMark', [], 'Display', None],
  [ 0x620, 'UnicodeCharacter0x620', [], 'Display', None],
  [ 0x621, 'ArabicLetterHamza', [], 'Display', None],
  [ 0x622, 'ArabicLetterAlefWithMaddaAbove', [], 'Display', None],
  [ 0x623, 'ArabicLetterAlefWithHamzaAbove', [], 'Display', None],
  [ 0x624, 'ArabicLetterWawWithHamzaAbove', [], 'Display', None],
  [ 0x625, 'ArabicLetterAlefWithHamzaBelow', [], 'Display', None],
  [ 0x626, 'ArabicLetterYehWithHamzaAbove', [], 'Display', None],
  [ 0x627, 'ArabicLetterAlef', [], 'Display', None],
  [ 0x628, 'ArabicLetterBeh', [], 'Display', None],
  [ 0x629, 'ArabicLetterTehMarbuta', [], 'Display', None],
  [ 0x62a, 'ArabicLetterTeh', [], 'Display', None],
  [ 0x62b, 'ArabicLetterTheh', [], 'Display', None],
  [ 0x62c, 'ArabicLetterJeem', [], 'Display', None],
  [ 0x62d, 'ArabicLetterHah', [], 'Display', None],
  [ 0x62e, 'ArabicLetterKhah', [], 'Display', None],
  [ 0x62f, 'ArabicLetterDal', [], 'Display', None],
  [ 0x630, 'ArabicLetterThal', [], 'Display', None],
  [ 0x631, 'ArabicLetterReh', [], 'Display', None],
  [ 0x632, 'ArabicLetterZain', [], 'Display', None],
  [ 0x633, 'ArabicLetterSeen', [], 'Display', None],
  [ 0x634, 'ArabicLetterSheen', [], 'Display', None],
  [ 0x635, 'ArabicLetterSad', [], 'Display', None],
  [ 0x636, 'ArabicLetterDad', [], 'Display', None],
  [ 0x637, 'ArabicLetterTah', [], 'Display', None],
  [ 0x638, 'ArabicLetterZah', [], 'Display', None],
  [ 0x639, 'ArabicLetterAin', [], 'Display', None],
  [ 0x63a, 'ArabicLetterGhain', [], 'Display', None],
  [ 0x63b, 'ArabicLetterKehehWithTwoDotsAbove', [], 'Display', None],
  [ 0x63c, 'ArabicLetterKehehWithThreeDotsBelow', [], 'Display', None],
  [ 0x63d, 'ArabicLetterFarsiYehWithInvertedV', [], 'Display', None],
  [ 0x63e, 'ArabicLetterFarsiYehWithTwoDotsAbove', [], 'Display', None],
  [ 0x63f, 'ArabicLetterFarsiYehWithThreeDotsAbove', [], 'Display', None],
  [ 0x640, 'ArabicTatweel', [], 'Display', None],
  [ 0x641, 'ArabicLetterFeh', [], 'Display', None],
  [ 0x642, 'ArabicLetterQaf', [], 'Display', None],
  [ 0x643, 'ArabicLetterKaf', [], 'Display', None],
  [ 0x644, 'ArabicLetterLam', [], 'Display', None],
  [ 0x645, 'ArabicLetterMeem', [], 'Display', None],
  [ 0x646, 'ArabicLetterNoon', [], 'Display', None],
  [ 0x647, 'ArabicLetterHeh', [], 'Display', None],
  [ 0x648, 'ArabicLetterWaw', [], 'Display', None],
  [ 0x649, 'ArabicLetterAlefMaksura', [], 'Display', None],
  [ 0x64a, 'ArabicLetterYeh', [], 'Display', None],
  [ 0x64b, 'ArabicFathatan', [], 'Display', None],
  [ 0x64c, 'ArabicDammatan', [], 'Display', None],
  [ 0x64d, 'ArabicKasratan', [], 'Display', None],
  [ 0x64e, 'ArabicFatha', [], 'Display', None],
  [ 0x64f, 'ArabicDamma', [], 'Display', None],
  [ 0x650, 'ArabicKasra', [], 'Display', None],
  [ 0x651, 'ArabicShadda', [], 'Display', None],
  [ 0x652, 'ArabicSukun', [], 'Display', None],
  [ 0x653, 'ArabicMaddahAbove', [], 'Display', None],
  [ 0x654, 'ArabicHamzaAbove', [], 'Display', None],
  [ 0x655, 'ArabicHamzaBelow', [], 'Display', None],
  [ 0x656, 'ArabicSubscriptAlef', [], 'Display', None],
  [ 0x657, 'ArabicInvertedDamma', [], 'Display', None],
  [ 0x658, 'ArabicMarkNoonGhunna', [], 'Display', None],
  [ 0x659, 'ArabicZwarakay', [], 'Display', None],
  [ 0x65a, 'ArabicVowelSignSmallVAbove', [], 'Display', None],
  [ 0x65b, 'ArabicVowelSignInvertedSmallVAbove', [], 'Display', None],
  [ 0x65c, 'ArabicVowelSignDotBelow', [], 'Display', None],
  [ 0x65d, 'ArabicReversedDamma', [], 'Display', None],
  [ 0x65e, 'ArabicFathaWithTwoDots', [], 'Display', None],
  [ 0x65f, 'UnicodeCharacter0x65f', [], 'Display', None],
  [ 0x660, 'ArabicindicDigitZero', [], 'Display', None],
  [ 0x661, 'ArabicindicDigitOne', [], 'Display', None],
  [ 0x662, 'ArabicindicDigitTwo', [], 'Display', None],
  [ 0x663, 'ArabicindicDigitThree', [], 'Display', None],
  [ 0x664, 'ArabicindicDigitFour', [], 'Display', None],
  [ 0x665, 'ArabicindicDigitFive', [], 'Display', None],
  [ 0x666, 'ArabicindicDigitSix', [], 'Display', None],
  [ 0x667, 'ArabicindicDigitSeven', [], 'Display', None],
  [ 0x668, 'ArabicindicDigitEight', [], 'Display', None],
  [ 0x669, 'ArabicindicDigitNine', [], 'Display', None],
  [ 0x66a, 'ArabicPercentSign', [], 'Display', None],
  [ 0x66b, 'ArabicDecimalSeparator', [], 'Display', None],
  [ 0x66c, 'ArabicThousandsSeparator', [], 'Display', None],
  [ 0x66d, 'ArabicFivePointedStar', [], 'Display', None],
  [ 0x66e, 'ArabicLetterDotlessBeh', [], 'Display', None],
  [ 0x66f, 'ArabicLetterDotlessQaf', [], 'Display', None],
  [ 0x670, 'ArabicLetterSuperscriptAlef', [], 'Display', None],
  [ 0x671, 'ArabicLetterAlefWasla', [], 'Display', None],
  [ 0x672, 'ArabicLetterAlefWithWavyHamzaAbove', [], 'Display', None],
  [ 0x673, 'ArabicLetterAlefWithWavyHamzaBelow', [], 'Display', None],
  [ 0x674, 'ArabicLetterHighHamza', [], 'Display', None],
  [ 0x675, 'ArabicLetterHighHamzaAlef', [], 'Display', None],
  [ 0x676, 'ArabicLetterHighHamzaWaw', [], 'Display', None],
  [ 0x677, 'ArabicLetterUWithHamzaAbove', [], 'Display', None],
  [ 0x678, 'ArabicLetterHighHamzaYeh', [], 'Display', None],
  [ 0x679, 'ArabicLetterTteh', [], 'Display', None],
  [ 0x67a, 'ArabicLetterTteheh', [], 'Display', None],
  [ 0x67b, 'ArabicLetterBeeh', [], 'Display', None],
  [ 0x67c, 'ArabicLetterTehWithRing', [], 'Display', None],
  [ 0x67d, 'ArabicLetterTehWithThreeDotsAboveDownwards', [], 'Display', None],
  [ 0x67e, 'ArabicLetterPeh', [], 'Display', None],
  [ 0x67f, 'ArabicLetterTeheh', [], 'Display', None],
  [ 0x680, 'ArabicLetterBeheh', [], 'Display', None],
  [ 0x681, 'ArabicLetterHahWithHamzaAbove', [], 'Display', None],
  [ 0x682, 'ArabicLetterHahWithTwoDotsVerticalAbove', [], 'Display', None],
  [ 0x683, 'ArabicLetterNyeh', [], 'Display', None],
  [ 0x684, 'ArabicLetterDyeh', [], 'Display', None],
  [ 0x685, 'ArabicLetterHahWithThreeDotsAbove', [], 'Display', None],
  [ 0x686, 'ArabicLetterTcheh', [], 'Display', None],
  [ 0x687, 'ArabicLetterTcheheh', [], 'Display', None],
  [ 0x688, 'ArabicLetterDdal', [], 'Display', None],
  [ 0x689, 'ArabicLetterDalWithRing', [], 'Display', None],
  [ 0x68a, 'ArabicLetterDalWithDotBelow', [], 'Display', None],
  [ 0x68b, 'ArabicLetterDalWithDotBelowAndSmallTah', [], 'Display', None],
  [ 0x68c, 'ArabicLetterDahal', [], 'Display', None],
  [ 0x68d, 'ArabicLetterDdahal', [], 'Display', None],
  [ 0x68e, 'ArabicLetterDul', [], 'Display', None],
  [ 0x68f, 'ArabicLetterDalWithThreeDotsAboveDownwards', [], 'Display', None],
  [ 0x690, 'ArabicLetterDalWithFourDotsAbove', [], 'Display', None],
  [ 0x691, 'ArabicLetterRreh', [], 'Display', None],
  [ 0x692, 'ArabicLetterRehWithSmallV', [], 'Display', None],
  [ 0x693, 'ArabicLetterRehWithRing', [], 'Display', None],
  [ 0x694, 'ArabicLetterRehWithDotBelow', [], 'Display', None],
  [ 0x695, 'ArabicLetterRehWithSmallVBelow', [], 'Display', None],
  [ 0x696, 'ArabicLetterRehWithDotBelowAndDotAbove', [], 'Display', None],
  [ 0x697, 'ArabicLetterRehWithTwoDotsAbove', [], 'Display', None],
  [ 0x698, 'ArabicLetterJeh', [], 'Display', None],
  [ 0x699, 'ArabicLetterRehWithFourDotsAbove', [], 'Display', None],
  [ 0x69a, 'ArabicLetterSeenWithDotBelowAndDotAbove', [], 'Display', None],
  [ 0x69b, 'ArabicLetterSeenWithThreeDotsBelow', [], 'Display', None],
  [ 0x69c, 'ArabicLetterSeenWithThreeDotsBelowAndThreeDotsAbove', [], 'Display', None],
  [ 0x69d, 'ArabicLetterSadWithTwoDotsBelow', [], 'Display', None],
  [ 0x69e, 'ArabicLetterSadWithThreeDotsAbove', [], 'Display', None],
  [ 0x69f, 'ArabicLetterTahWithThreeDotsAbove', [], 'Display', None],
  [ 0x6a0, 'ArabicLetterAinWithThreeDotsAbove', [], 'Display', None],
  [ 0x6a1, 'ArabicLetterDotlessFeh', [], 'Display', None],
  [ 0x6a2, 'ArabicLetterFehWithDotMovedBelow', [], 'Display', None],
  [ 0x6a3, 'ArabicLetterFehWithDotBelow', [], 'Display', None],
  [ 0x6a4, 'ArabicLetterVeh', [], 'Display', None],
  [ 0x6a5, 'ArabicLetterFehWithThreeDotsBelow', [], 'Display', None],
  [ 0x6a6, 'ArabicLetterPeheh', [], 'Display', None],
  [ 0x6a7, 'ArabicLetterQafWithDotAbove', [], 'Display', None],
  [ 0x6a8, 'ArabicLetterQafWithThreeDotsAbove', [], 'Display', None],
  [ 0x6a9, 'ArabicLetterKeheh', [], 'Display', None],
  [ 0x6aa, 'ArabicLetterSwashKaf', [], 'Display', None],
  [ 0x6ab, 'ArabicLetterKafWithRing', [], 'Display', None],
  [ 0x6ac, 'ArabicLetterKafWithDotAbove', [], 'Display', None],
  [ 0x6ad, 'ArabicLetterNg', [], 'Display', None],
  [ 0x6ae, 'ArabicLetterKafWithThreeDotsBelow', [], 'Display', None],
  [ 0x6af, 'ArabicLetterGaf', [], 'Display', None],
  [ 0x6b0, 'ArabicLetterGafWithRing', [], 'Display', None],
  [ 0x6b1, 'ArabicLetterNgoeh', [], 'Display', None],
  [ 0x6b2, 'ArabicLetterGafWithTwoDotsBelow', [], 'Display', None],
  [ 0x6b3, 'ArabicLetterGueh', [], 'Display', None],
  [ 0x6b4, 'ArabicLetterGafWithThreeDotsAbove', [], 'Display', None],
  [ 0x6b5, 'ArabicLetterLamWithSmallV', [], 'Display', None],
  [ 0x6b6, 'ArabicLetterLamWithDotAbove', [], 'Display', None],
  [ 0x6b7, 'ArabicLetterLamWithThreeDotsAbove', [], 'Display', None],
  [ 0x6b8, 'ArabicLetterLamWithThreeDotsBelow', [], 'Display', None],
  [ 0x6b9, 'ArabicLetterNoonWithDotBelow', [], 'Display', None],
  [ 0x6ba, 'ArabicLetterNoonGhunna', [], 'Display', None],
  [ 0x6bb, 'ArabicLetterRnoon', [], 'Display', None],
  [ 0x6bc, 'ArabicLetterNoonWithRing', [], 'Display', None],
  [ 0x6bd, 'ArabicLetterNoonWithThreeDotsAbove', [], 'Display', None],
  [ 0x6be, 'ArabicLetterHehDoachashmee', [], 'Display', None],
  [ 0x6bf, 'ArabicLetterTchehWithDotAbove', [], 'Display', None],
  [ 0x6c0, 'ArabicLetterHehWithYehAbove', [], 'Display', None],
  [ 0x6c1, 'ArabicLetterHehGoal', [], 'Display', None],
  [ 0x6c2, 'ArabicLetterHehGoalWithHamzaAbove', [], 'Display', None],
  [ 0x6c3, 'ArabicLetterTehMarbutaGoal', [], 'Display', None],
  [ 0x6c4, 'ArabicLetterWawWithRing', [], 'Display', None],
  [ 0x6c5, 'ArabicLetterKirghizOe', [], 'Display', None],
  [ 0x6c6, 'ArabicLetterOe', [], 'Display', None],
  [ 0x6c7, 'ArabicLetterU', [], 'Display', None],
  [ 0x6c8, 'ArabicLetterYu', [], 'Display', None],
  [ 0x6c9, 'ArabicLetterKirghizYu', [], 'Display', None],
  [ 0x6ca, 'ArabicLetterWawWithTwoDotsAbove', [], 'Display', None],
  [ 0x6cb, 'ArabicLetterVe', [], 'Display', None],
  [ 0x6cc, 'ArabicLetterFarsiYeh', [], 'Display', None],
  [ 0x6cd, 'ArabicLetterYehWithTail', [], 'Display', None],
  [ 0x6ce, 'ArabicLetterYehWithSmallV', [], 'Display', None],
  [ 0x6cf, 'ArabicLetterWawWithDotAbove', [], 'Display', None],
  [ 0x6d0, 'ArabicLetterE', [], 'Display', None],
  [ 0x6d1, 'ArabicLetterYehWithThreeDotsBelow', [], 'Display', None],
  [ 0x6d2, 'ArabicLetterYehBarree', [], 'Display', None],
  [ 0x6d3, 'ArabicLetterYehBarreeWithHamzaAbove', [], 'Display', None],
  [ 0x6d4, 'ArabicFullStop', [], 'Display', None],
  [ 0x6d5, 'ArabicLetterAe', [], 'Display', None],
  [ 0x6d6, 'ArabicSmallHighLigatureSadWithLamWithAlefMaksura', [], 'Display', None],
  [ 0x6d7, 'ArabicSmallHighLigatureQafWithLamWithAlefMaksura', [], 'Display', None],
  [ 0x6d8, 'ArabicSmallHighMeemInitialForm', [], 'Display', None],
  [ 0x6d9, 'ArabicSmallHighLamAlef', [], 'Display', None],
  [ 0x6da, 'ArabicSmallHighJeem', [], 'Display', None],
  [ 0x6db, 'ArabicSmallHighThreeDots', [], 'Display', None],
  [ 0x6dc, 'ArabicSmallHighSeen', [], 'Display', None],
  [ 0x6dd, 'ArabicEndOfAyah', [], 'Display', None],
  [ 0x6de, 'ArabicStartOfRubElHizb', [], 'Display', None],
  [ 0x6df, 'ArabicSmallHighRoundedZero', [], 'Display', None],
  [ 0x6e0, 'ArabicSmallHighUprightRectangularZero', [], 'Display', None],
  [ 0x6e1, 'ArabicSmallHighDotlessHeadOfKhah', [], 'Display', None],
  [ 0x6e2, 'ArabicSmallHighMeemIsolatedForm', [], 'Display', None],
  [ 0x6e3, 'ArabicSmallLowSeen', [], 'Display', None],
  [ 0x6e4, 'ArabicSmallHighMadda', [], 'Display', None],
  [ 0x6e5, 'ArabicSmallWaw', [], 'Display', None],
  [ 0x6e6, 'ArabicSmallYeh', [], 'Display', None],
  [ 0x6e7, 'ArabicSmallHighYeh', [], 'Display', None],
  [ 0x6e8, 'ArabicSmallHighNoon', [], 'Display', None],
  [ 0x6e9, 'ArabicPlaceOfSajdah', [], 'Display', None],
  [ 0x6ea, 'ArabicEmptyCentreLowStop', [], 'Display', None],
  [ 0x6eb, 'ArabicEmptyCentreHighStop', [], 'Display', None],
  [ 0x6ec, 'ArabicRoundedHighStopWithFilledCentre', [], 'Display', None],
  [ 0x6ed, 'ArabicSmallLowMeem', [], 'Display', None],
  [ 0x6ee, 'ArabicLetterDalWithInvertedV', [], 'Display', None],
  [ 0x6ef, 'ArabicLetterRehWithInvertedV', [], 'Display', None],
  [ 0x6f0, 'ExtendedArabicindicDigitZero', [], 'Display', None],
  [ 0x6f1, 'ExtendedArabicindicDigitOne', [], 'Display', None],
  [ 0x6f2, 'ExtendedArabicindicDigitTwo', [], 'Display', None],
  [ 0x6f3, 'ExtendedArabicindicDigitThree', [], 'Display', None],
  [ 0x6f4, 'ExtendedArabicindicDigitFour', [], 'Display', None],
  [ 0x6f5, 'ExtendedArabicindicDigitFive', [], 'Display', None],
  [ 0x6f6, 'ExtendedArabicindicDigitSix', [], 'Display', None],
  [ 0x6f7, 'ExtendedArabicindicDigitSeven', [], 'Display', None],
  [ 0x6f8, 'ExtendedArabicindicDigitEight', [], 'Display', None],
  [ 0x6f9, 'ExtendedArabicindicDigitNine', [], 'Display', None],
  [ 0x6fa, 'ArabicLetterSheenWithDotBelow', [], 'Display', None],
  [ 0x6fb, 'ArabicLetterDadWithDotBelow', [], 'Display', None],
  [ 0x6fc, 'ArabicLetterGhainWithDotBelow', [], 'Display', None],
  [ 0x6fd, 'ArabicSignSindhiAmpersand', [], 'Display', None],
  [ 0x6fe, 'ArabicSignSindhiPostpositionMen', [], 'Display', None],
  [ 0x6ff, 'ArabicLetterHehWithInvertedV', [], 'Display', None],
  [ 0x700, 'SyriacEndOfParagraph', [], 'Display', None],
  [ 0x701, 'SyriacSupralinearFullStop', [], 'Display', None],
  [ 0x702, 'SyriacSublinearFullStop', [], 'Display', None],
  [ 0x703, 'SyriacSupralinearColon', [], 'Display', None],
  [ 0x704, 'SyriacSublinearColon', [], 'Display', None],
  [ 0x705, 'SyriacHorizontalColon', [], 'Display', None],
  [ 0x706, 'SyriacColonSkewedLeft', [], 'Display', None],
  [ 0x707, 'SyriacColonSkewedRight', [], 'Display', None],
  [ 0x708, 'SyriacSupralinearColonSkewedLeft', [], 'Display', None],
  [ 0x709, 'SyriacSublinearColonSkewedRight', [], 'Display', None],
  [ 0x70a, 'SyriacContraction', [], 'Display', None],
  [ 0x70b, 'SyriacHarkleanObelus', [], 'Display', None],
  [ 0x70c, 'SyriacHarkleanMetobelus', [], 'Display', None],
  [ 0x70d, 'SyriacHarkleanAsteriscus', [], 'Display', None],
  [ 0x70e, 'UnicodeCharacter0x70e', [], 'Display', None],
  [ 0x70f, 'SyriacAbbreviationMark', [], 'Display', None],
  [ 0x710, 'SyriacLetterAlaph', [], 'Display', None],
  [ 0x711, 'SyriacLetterSuperscriptAlaph', [], 'Display', None],
  [ 0x712, 'SyriacLetterBeth', [], 'Display', None],
  [ 0x713, 'SyriacLetterGamal', [], 'Display', None],
  [ 0x714, 'SyriacLetterGamalGarshuni', [], 'Display', None],
  [ 0x715, 'SyriacLetterDalath', [], 'Display', None],
  [ 0x716, 'SyriacLetterDotlessDalathRish', [], 'Display', None],
  [ 0x717, 'SyriacLetterHe', [], 'Display', None],
  [ 0x718, 'SyriacLetterWaw', [], 'Display', None],
  [ 0x719, 'SyriacLetterZain', [], 'Display', None],
  [ 0x71a, 'SyriacLetterHeth', [], 'Display', None],
  [ 0x71b, 'SyriacLetterTeth', [], 'Display', None],
  [ 0x71c, 'SyriacLetterTethGarshuni', [], 'Display', None],
  [ 0x71d, 'SyriacLetterYudh', [], 'Display', None],
  [ 0x71e, 'SyriacLetterYudhHe', [], 'Display', None],
  [ 0x71f, 'SyriacLetterKaph', [], 'Display', None],
  [ 0x720, 'SyriacLetterLamadh', [], 'Display', None],
  [ 0x721, 'SyriacLetterMim', [], 'Display', None],
  [ 0x722, 'SyriacLetterNun', [], 'Display', None],
  [ 0x723, 'SyriacLetterSemkath', [], 'Display', None],
  [ 0x724, 'SyriacLetterFinalSemkath', [], 'Display', None],
  [ 0x725, 'SyriacLetterE', [], 'Display', None],
  [ 0x726, 'SyriacLetterPe', [], 'Display', None],
  [ 0x727, 'SyriacLetterReversedPe', [], 'Display', None],
  [ 0x728, 'SyriacLetterSadhe', [], 'Display', None],
  [ 0x729, 'SyriacLetterQaph', [], 'Display', None],
  [ 0x72a, 'SyriacLetterRish', [], 'Display', None],
  [ 0x72b, 'SyriacLetterShin', [], 'Display', None],
  [ 0x72c, 'SyriacLetterTaw', [], 'Display', None],
  [ 0x72d, 'SyriacLetterPersianBheth', [], 'Display', None],
  [ 0x72e, 'SyriacLetterPersianGhamal', [], 'Display', None],
  [ 0x72f, 'SyriacLetterPersianDhalath', [], 'Display', None],
  [ 0x730, 'SyriacPthahaAbove', [], 'Display', None],
  [ 0x731, 'SyriacPthahaBelow', [], 'Display', None],
  [ 0x732, 'SyriacPthahaDotted', [], 'Display', None],
  [ 0x733, 'SyriacZqaphaAbove', [], 'Display', None],
  [ 0x734, 'SyriacZqaphaBelow', [], 'Display', None],
  [ 0x735, 'SyriacZqaphaDotted', [], 'Display', None],
  [ 0x736, 'SyriacRbasaAbove', [], 'Display', None],
  [ 0x737, 'SyriacRbasaBelow', [], 'Display', None],
  [ 0x738, 'SyriacDottedZlamaHorizontal', [], 'Display', None],
  [ 0x739, 'SyriacDottedZlamaAngular', [], 'Display', None],
  [ 0x73a, 'SyriacHbasaAbove', [], 'Display', None],
  [ 0x73b, 'SyriacHbasaBelow', [], 'Display', None],
  [ 0x73c, 'SyriacHbasaesasaDotted', [], 'Display', None],
  [ 0x73d, 'SyriacEsasaAbove', [], 'Display', None],
  [ 0x73e, 'SyriacEsasaBelow', [], 'Display', None],
  [ 0x73f, 'SyriacRwaha', [], 'Display', None],
  [ 0x740, 'SyriacFeminineDot', [], 'Display', None],
  [ 0x741, 'SyriacQushshaya', [], 'Display', None],
  [ 0x742, 'SyriacRukkakha', [], 'Display', None],
  [ 0x743, 'SyriacTwoVerticalDotsAbove', [], 'Display', None],
  [ 0x744, 'SyriacTwoVerticalDotsBelow', [], 'Display', None],
  [ 0x745, 'SyriacThreeDotsAbove', [], 'Display', None],
  [ 0x746, 'SyriacThreeDotsBelow', [], 'Display', None],
  [ 0x747, 'SyriacObliqueLineAbove', [], 'Display', None],
  [ 0x748, 'SyriacObliqueLineBelow', [], 'Display', None],
  [ 0x749, 'SyriacMusic', [], 'Display', None],
  [ 0x74a, 'SyriacBarrekh', [], 'Display', None],
  [ 0x74b, 'UnicodeCharacter0x74b', [], 'Display', None],
  [ 0x74c, 'UnicodeCharacter0x74c', [], 'Display', None],
  [ 0x74d, 'SyriacLetterSogdianZhain', [], 'Display', None],
  [ 0x74e, 'SyriacLetterSogdianKhaph', [], 'Display', None],
  [ 0x74f, 'SyriacLetterSogdianFe', [], 'Display', None],
  [ 0x750, 'ArabicLetterBehWithThreeDotsHorizontallyBelow', [], 'Display', None],
  [ 0x751, 'ArabicLetterBehWithDotBelowAndThreeDotsAbove', [], 'Display', None],
  [ 0x752, 'ArabicLetterBehWithThreeDotsPointingUpwardsBelow', [], 'Display', None],
  [ 0x753, 'ArabicLetterBehWithThreeDotsPointingUpwardsBelowAndTwoDotsAbove', [], 'Display', None],
  [ 0x754, 'ArabicLetterBehWithTwoDotsBelowAndDotAbove', [], 'Display', None],
  [ 0x755, 'ArabicLetterBehWithInvertedSmallVBelow', [], 'Display', None],
  [ 0x756, 'ArabicLetterBehWithSmallV', [], 'Display', None],
  [ 0x757, 'ArabicLetterHahWithTwoDotsAbove', [], 'Display', None],
  [ 0x758, 'ArabicLetterHahWithThreeDotsPointingUpwardsBelow', [], 'Display', None],
  [ 0x759, 'ArabicLetterDalWithTwoDotsVerticallyBelowAndSmallTah', [], 'Display', None],
  [ 0x75a, 'ArabicLetterDalWithInvertedSmallVBelow', [], 'Display', None],
  [ 0x75b, 'ArabicLetterRehWithStroke', [], 'Display', None],
  [ 0x75c, 'ArabicLetterSeenWithFourDotsAbove', [], 'Display', None],
  [ 0x75d, 'ArabicLetterAinWithTwoDotsAbove', [], 'Display', None],
  [ 0x75e, 'ArabicLetterAinWithThreeDotsPointingDownwardsAbove', [], 'Display', None],
  [ 0x75f, 'ArabicLetterAinWithTwoDotsVerticallyAbove', [], 'Display', None],
  [ 0x760, 'ArabicLetterFehWithTwoDotsBelow', [], 'Display', None],
  [ 0x761, 'ArabicLetterFehWithThreeDotsPointingUpwardsBelow', [], 'Display', None],
  [ 0x762, 'ArabicLetterKehehWithDotAbove', [], 'Display', None],
  [ 0x763, 'ArabicLetterKehehWithThreeDotsAbove', [], 'Display', None],
  [ 0x764, 'ArabicLetterKehehWithThreeDotsPointingUpwardsBelow', [], 'Display', None],
  [ 0x765, 'ArabicLetterMeemWithDotAbove', [], 'Display', None],
  [ 0x766, 'ArabicLetterMeemWithDotBelow', [], 'Display', None],
  [ 0x767, 'ArabicLetterNoonWithTwoDotsBelow', [], 'Display', None],
  [ 0x768, 'ArabicLetterNoonWithSmallTah', [], 'Display', None],
  [ 0x769, 'ArabicLetterNoonWithSmallV', [], 'Display', None],
  [ 0x76a, 'ArabicLetterLamWithBar', [], 'Display', None],
  [ 0x76b, 'ArabicLetterRehWithTwoDotsVerticallyAbove', [], 'Display', None],
  [ 0x76c, 'ArabicLetterRehWithHamzaAbove', [], 'Display', None],
  [ 0x76d, 'ArabicLetterSeenWithTwoDotsVerticallyAbove', [], 'Display', None],
  [ 0x76e, 'ArabicLetterHahWithSmallArabicLetterTahBelow', [], 'Display', None],
  [ 0x76f, 'ArabicLetterHahWithSmallArabicLetterTahAndTwoDots', [], 'Display', None],
  [ 0x770, 'ArabicLetterSeenWithSmallArabicLetterTahAndTwoDots', [], 'Display', None],
  [ 0x771, 'ArabicLetterRehWithSmallArabicLetterTahAndTwoDots', [], 'Display', None],
  [ 0x772, 'ArabicLetterHahWithSmallArabicLetterTahAbove', [], 'Display', None],
  [ 0x773, 'ArabicLetterAlefWithExtendedArabicindicDigitTwoAbove', [], 'Display', None],
  [ 0x774, 'ArabicLetterAlefWithExtendedArabicindicDigitThreeAbove', [], 'Display', None],
  [ 0x775, 'ArabicLetterFarsiYehWithExtendedArabicindicDigitTwoAbove', [], 'Display', None],
  [ 0x776, 'ArabicLetterFarsiYehWithExtendedArabicindicDigitThreeAbove', [], 'Display', None],
  [ 0x777, 'ArabicLetterFarsiYehWithExtendedArabicindicDigitFourBelow', [], 'Display', None],
  [ 0x778, 'ArabicLetterWawWithExtendedArabicindicDigitTwoAbove', [], 'Display', None],
  [ 0x779, 'ArabicLetterWawWithExtendedArabicindicDigitThreeAbove', [], 'Display', None],
  [ 0x77a, 'ArabicLetterYehBarreeWithExtendedArabicindicDigitTwoAbove', [], 'Display', None],
  [ 0x77b, 'ArabicLetterYehBarreeWithExtendedArabicindicDigitThreeAbove', [], 'Display', None],
  [ 0x77c, 'ArabicLetterHahWithExtendedArabicindicDigitFourBelow', [], 'Display', None],
  [ 0x77d, 'ArabicLetterSeenWithExtendedArabicindicDigitFourAbove', [], 'Display', None],
  [ 0x77e, 'ArabicLetterSeenWithInvertedV', [], 'Display', None],
  [ 0x77f, 'ArabicLetterKafWithTwoDotsAbove', [], 'Display', None],
  [ 0x780, 'ThaanaLetterHaa', [], 'Display', None],
  [ 0x781, 'ThaanaLetterShaviyani', [], 'Display', None],
  [ 0x782, 'ThaanaLetterNoonu', [], 'Display', None],
  [ 0x783, 'ThaanaLetterRaa', [], 'Display', None],
  [ 0x784, 'ThaanaLetterBaa', [], 'Display', None],
  [ 0x785, 'ThaanaLetterLhaviyani', [], 'Display', None],
  [ 0x786, 'ThaanaLetterKaafu', [], 'Display', None],
  [ 0x787, 'ThaanaLetterAlifu', [], 'Display', None],
  [ 0x788, 'ThaanaLetterVaavu', [], 'Display', None],
  [ 0x789, 'ThaanaLetterMeemu', [], 'Display', None],
  [ 0x78a, 'ThaanaLetterFaafu', [], 'Display', None],
  [ 0x78b, 'ThaanaLetterDhaalu', [], 'Display', None],
  [ 0x78c, 'ThaanaLetterThaa', [], 'Display', None],
  [ 0x78d, 'ThaanaLetterLaamu', [], 'Display', None],
  [ 0x78e, 'ThaanaLetterGaafu', [], 'Display', None],
  [ 0x78f, 'ThaanaLetterGnaviyani', [], 'Display', None],
  [ 0x790, 'ThaanaLetterSeenu', [], 'Display', None],
  [ 0x791, 'ThaanaLetterDaviyani', [], 'Display', None],
  [ 0x792, 'ThaanaLetterZaviyani', [], 'Display', None],
  [ 0x793, 'ThaanaLetterTaviyani', [], 'Display', None],
  [ 0x794, 'ThaanaLetterYaa', [], 'Display', None],
  [ 0x795, 'ThaanaLetterPaviyani', [], 'Display', None],
  [ 0x796, 'ThaanaLetterJaviyani', [], 'Display', None],
  [ 0x797, 'ThaanaLetterChaviyani', [], 'Display', None],
  [ 0x798, 'ThaanaLetterTtaa', [], 'Display', None],
  [ 0x799, 'ThaanaLetterHhaa', [], 'Display', None],
  [ 0x79a, 'ThaanaLetterKhaa', [], 'Display', None],
  [ 0x79b, 'ThaanaLetterThaalu', [], 'Display', None],
  [ 0x79c, 'ThaanaLetterZaa', [], 'Display', None],
  [ 0x79d, 'ThaanaLetterSheenu', [], 'Display', None],
  [ 0x79e, 'ThaanaLetterSaadhu', [], 'Display', None],
  [ 0x79f, 'ThaanaLetterDaadhu', [], 'Display', None],
  [ 0x7a0, 'ThaanaLetterTo', [], 'Display', None],
  [ 0x7a1, 'ThaanaLetterZo', [], 'Display', None],
  [ 0x7a2, 'ThaanaLetterAinu', [], 'Display', None],
  [ 0x7a3, 'ThaanaLetterGhainu', [], 'Display', None],
  [ 0x7a4, 'ThaanaLetterQaafu', [], 'Display', None],
  [ 0x7a5, 'ThaanaLetterWaavu', [], 'Display', None],
  [ 0x7a6, 'ThaanaAbafili', [], 'Display', None],
  [ 0x7a7, 'ThaanaAabaafili', [], 'Display', None],
  [ 0x7a8, 'ThaanaIbifili', [], 'Display', None],
  [ 0x7a9, 'ThaanaEebeefili', [], 'Display', None],
  [ 0x7aa, 'ThaanaUbufili', [], 'Display', None],
  [ 0x7ab, 'ThaanaOoboofili', [], 'Display', None],
  [ 0x7ac, 'ThaanaEbefili', [], 'Display', None],
  [ 0x7ad, 'ThaanaEybeyfili', [], 'Display', None],
  [ 0x7ae, 'ThaanaObofili', [], 'Display', None],
  [ 0x7af, 'ThaanaOaboafili', [], 'Display', None],
  [ 0x7b0, 'ThaanaSukun', [], 'Display', None],
  [ 0x7b1, 'ThaanaLetterNaa', [], 'Display', None],
  [ 0x7b2, 'UnicodeCharacter0x7b2', [], 'Display', None],
  [ 0x7b3, 'UnicodeCharacter0x7b3', [], 'Display', None],
  [ 0x7b4, 'UnicodeCharacter0x7b4', [], 'Display', None],
  [ 0x7b5, 'UnicodeCharacter0x7b5', [], 'Display', None],
  [ 0x7b6, 'UnicodeCharacter0x7b6', [], 'Display', None],
  [ 0x7b7, 'UnicodeCharacter0x7b7', [], 'Display', None],
  [ 0x7b8, 'UnicodeCharacter0x7b8', [], 'Display', None],
  [ 0x7b9, 'UnicodeCharacter0x7b9', [], 'Display', None],
  [ 0x7ba, 'UnicodeCharacter0x7ba', [], 'Display', None],
  [ 0x7bb, 'UnicodeCharacter0x7bb', [], 'Display', None],
  [ 0x7bc, 'UnicodeCharacter0x7bc', [], 'Display', None],
  [ 0x7bd, 'UnicodeCharacter0x7bd', [], 'Display', None],
  [ 0x7be, 'UnicodeCharacter0x7be', [], 'Display', None],
  [ 0x7bf, 'UnicodeCharacter0x7bf', [], 'Display', None],
  [ 0x7c0, 'NkoDigitZero', [], 'Display', None],
  [ 0x7c1, 'NkoDigitOne', [], 'Display', None],
  [ 0x7c2, 'NkoDigitTwo', [], 'Display', None],
  [ 0x7c3, 'NkoDigitThree', [], 'Display', None],
  [ 0x7c4, 'NkoDigitFour', [], 'Display', None],
  [ 0x7c5, 'NkoDigitFive', [], 'Display', None],
  [ 0x7c6, 'NkoDigitSix', [], 'Display', None],
  [ 0x7c7, 'NkoDigitSeven', [], 'Display', None],
  [ 0x7c8, 'NkoDigitEight', [], 'Display', None],
  [ 0x7c9, 'NkoDigitNine', [], 'Display', None],
  [ 0x7ca, 'NkoLetterA', [], 'Display', None],
  [ 0x7cb, 'NkoLetterEe', [], 'Display', None],
  [ 0x7cc, 'NkoLetterI', [], 'Display', None],
  [ 0x7cd, 'NkoLetterE', [], 'Display', None],
  [ 0x7ce, 'NkoLetterU', [], 'Display', None],
  [ 0x7cf, 'NkoLetterOo', [], 'Display', None],
  [ 0x7d0, 'NkoLetterO', [], 'Display', None],
  [ 0x7d1, 'NkoLetterDagbasinna', [], 'Display', None],
  [ 0x7d2, 'NkoLetterN', [], 'Display', None],
  [ 0x7d3, 'NkoLetterBa', [], 'Display', None],
  [ 0x7d4, 'NkoLetterPa', [], 'Display', None],
  [ 0x7d5, 'NkoLetterTa', [], 'Display', None],
  [ 0x7d6, 'NkoLetterJa', [], 'Display', None],
  [ 0x7d7, 'NkoLetterCha', [], 'Display', None],
  [ 0x7d8, 'NkoLetterDa', [], 'Display', None],
  [ 0x7d9, 'NkoLetterRa', [], 'Display', None],
  [ 0x7da, 'NkoLetterRra', [], 'Display', None],
  [ 0x7db, 'NkoLetterSa', [], 'Display', None],
  [ 0x7dc, 'NkoLetterGba', [], 'Display', None],
  [ 0x7dd, 'NkoLetterFa', [], 'Display', None],
  [ 0x7de, 'NkoLetterKa', [], 'Display', None],
  [ 0x7df, 'NkoLetterLa', [], 'Display', None],
  [ 0x7e0, 'NkoLetterNaWoloso', [], 'Display', None],
  [ 0x7e1, 'NkoLetterMa', [], 'Display', None],
  [ 0x7e2, 'NkoLetterNya', [], 'Display', None],
  [ 0x7e3, 'NkoLetterNa', [], 'Display', None],
  [ 0x7e4, 'NkoLetterHa', [], 'Display', None],
  [ 0x7e5, 'NkoLetterWa', [], 'Display', None],
  [ 0x7e6, 'NkoLetterYa', [], 'Display', None],
  [ 0x7e7, 'NkoLetterNyaWoloso', [], 'Display', None],
  [ 0x7e8, 'NkoLetterJonaJa', [], 'Display', None],
  [ 0x7e9, 'NkoLetterJonaCha', [], 'Display', None],
  [ 0x7ea, 'NkoLetterJonaRa', [], 'Display', None],
  [ 0x7eb, 'NkoCombiningShortHighTone', [], 'Display', None],
  [ 0x7ec, 'NkoCombiningShortLowTone', [], 'Display', None],
  [ 0x7ed, 'NkoCombiningShortRisingTone', [], 'Display', None],
  [ 0x7ee, 'NkoCombiningLongDescendingTone', [], 'Display', None],
  [ 0x7ef, 'NkoCombiningLongHighTone', [], 'Display', None],
  [ 0x7f0, 'NkoCombiningLongLowTone', [], 'Display', None],
  [ 0x7f1, 'NkoCombiningLongRisingTone', [], 'Display', None],
  [ 0x7f2, 'NkoCombiningNasalizationMark', [], 'Display', None],
  [ 0x7f3, 'NkoCombiningDoubleDotAbove', [], 'Display', None],
  [ 0x7f4, 'NkoHighToneApostrophe', [], 'Display', None],
  [ 0x7f5, 'NkoLowToneApostrophe', [], 'Display', None],
  [ 0x7f6, 'NkoSymbolOoDennen', [], 'Display', None],
  [ 0x7f7, 'NkoSymbolGbakurunen', [], 'Display', None],
  [ 0x7f8, 'NkoComma', [], 'Display', None],
  [ 0x7f9, 'NkoExclamationMark', [], 'Display', None],
  [ 0x7fa, 'NkoLajanyalan', [], 'Display', None],
  [ 0x7fb, 'UnicodeCharacter0x7fb', [], 'Display', None],
  [ 0x7fc, 'UnicodeCharacter0x7fc', [], 'Display', None],
  [ 0x7fd, 'UnicodeCharacter0x7fd', [], 'Display', None],
  [ 0x7fe, 'UnicodeCharacter0x7fe', [], 'Display', None],
  [ 0x7ff, 'UnicodeCharacter0x7ff', [], 'Display', None],
  [ 0x800, 'UnicodeCharacter0x800', [], 'Display', None],
  [ 0x801, 'UnicodeCharacter0x801', [], 'Display', None],
  [ 0x802, 'UnicodeCharacter0x802', [], 'Display', None],
  [ 0x803, 'UnicodeCharacter0x803', [], 'Display', None],
  [ 0x804, 'UnicodeCharacter0x804', [], 'Display', None],
  [ 0x805, 'UnicodeCharacter0x805', [], 'Display', None],
  [ 0x806, 'UnicodeCharacter0x806', [], 'Display', None],
  [ 0x807, 'UnicodeCharacter0x807', [], 'Display', None],
  [ 0x808, 'UnicodeCharacter0x808', [], 'Display', None],
  [ 0x809, 'UnicodeCharacter0x809', [], 'Display', None],
  [ 0x80a, 'UnicodeCharacter0x80a', [], 'Display', None],
  [ 0x80b, 'UnicodeCharacter0x80b', [], 'Display', None],
  [ 0x80c, 'UnicodeCharacter0x80c', [], 'Display', None],
  [ 0x80d, 'UnicodeCharacter0x80d', [], 'Display', None],
  [ 0x80e, 'UnicodeCharacter0x80e', [], 'Display', None],
  [ 0x80f, 'UnicodeCharacter0x80f', [], 'Display', None],
  [ 0x810, 'UnicodeCharacter0x810', [], 'Display', None],
  [ 0x811, 'UnicodeCharacter0x811', [], 'Display', None],
  [ 0x812, 'UnicodeCharacter0x812', [], 'Display', None],
  [ 0x813, 'UnicodeCharacter0x813', [], 'Display', None],
  [ 0x814, 'UnicodeCharacter0x814', [], 'Display', None],
  [ 0x815, 'UnicodeCharacter0x815', [], 'Display', None],
  [ 0x816, 'UnicodeCharacter0x816', [], 'Display', None],
  [ 0x817, 'UnicodeCharacter0x817', [], 'Display', None],
  [ 0x818, 'UnicodeCharacter0x818', [], 'Display', None],
  [ 0x819, 'UnicodeCharacter0x819', [], 'Display', None],
  [ 0x81a, 'UnicodeCharacter0x81a', [], 'Display', None],
  [ 0x81b, 'UnicodeCharacter0x81b', [], 'Display', None],
  [ 0x81c, 'UnicodeCharacter0x81c', [], 'Display', None],
  [ 0x81d, 'UnicodeCharacter0x81d', [], 'Display', None],
  [ 0x81e, 'UnicodeCharacter0x81e', [], 'Display', None],
  [ 0x81f, 'UnicodeCharacter0x81f', [], 'Display', None],
  [ 0x820, 'UnicodeCharacter0x820', [], 'Display', None],
  [ 0x821, 'UnicodeCharacter0x821', [], 'Display', None],
  [ 0x822, 'UnicodeCharacter0x822', [], 'Display', None],
  [ 0x823, 'UnicodeCharacter0x823', [], 'Display', None],
  [ 0x824, 'UnicodeCharacter0x824', [], 'Display', None],
  [ 0x825, 'UnicodeCharacter0x825', [], 'Display', None],
  [ 0x826, 'UnicodeCharacter0x826', [], 'Display', None],
  [ 0x827, 'UnicodeCharacter0x827', [], 'Display', None],
  [ 0x828, 'UnicodeCharacter0x828', [], 'Display', None],
  [ 0x829, 'UnicodeCharacter0x829', [], 'Display', None],
  [ 0x82a, 'UnicodeCharacter0x82a', [], 'Display', None],
  [ 0x82b, 'UnicodeCharacter0x82b', [], 'Display', None],
  [ 0x82c, 'UnicodeCharacter0x82c', [], 'Display', None],
  [ 0x82d, 'UnicodeCharacter0x82d', [], 'Display', None],
  [ 0x82e, 'UnicodeCharacter0x82e', [], 'Display', None],
  [ 0x82f, 'UnicodeCharacter0x82f', [], 'Display', None],
  [ 0x830, 'UnicodeCharacter0x830', [], 'Display', None],
  [ 0x831, 'UnicodeCharacter0x831', [], 'Display', None],
  [ 0x832, 'UnicodeCharacter0x832', [], 'Display', None],
  [ 0x833, 'UnicodeCharacter0x833', [], 'Display', None],
  [ 0x834, 'UnicodeCharacter0x834', [], 'Display', None],
  [ 0x835, 'UnicodeCharacter0x835', [], 'Display', None],
  [ 0x836, 'UnicodeCharacter0x836', [], 'Display', None],
  [ 0x837, 'UnicodeCharacter0x837', [], 'Display', None],
  [ 0x838, 'UnicodeCharacter0x838', [], 'Display', None],
  [ 0x839, 'UnicodeCharacter0x839', [], 'Display', None],
  [ 0x83a, 'UnicodeCharacter0x83a', [], 'Display', None],
  [ 0x83b, 'UnicodeCharacter0x83b', [], 'Display', None],
  [ 0x83c, 'UnicodeCharacter0x83c', [], 'Display', None],
  [ 0x83d, 'UnicodeCharacter0x83d', [], 'Display', None],
  [ 0x83e, 'UnicodeCharacter0x83e', [], 'Display', None],
  [ 0x83f, 'UnicodeCharacter0x83f', [], 'Display', None],
  [ 0x840, 'UnicodeCharacter0x840', [], 'Display', None],
  [ 0x841, 'UnicodeCharacter0x841', [], 'Display', None],
  [ 0x842, 'UnicodeCharacter0x842', [], 'Display', None],
  [ 0x843, 'UnicodeCharacter0x843', [], 'Display', None],
  [ 0x844, 'UnicodeCharacter0x844', [], 'Display', None],
  [ 0x845, 'UnicodeCharacter0x845', [], 'Display', None],
  [ 0x846, 'UnicodeCharacter0x846', [], 'Display', None],
  [ 0x847, 'UnicodeCharacter0x847', [], 'Display', None],
  [ 0x848, 'UnicodeCharacter0x848', [], 'Display', None],
  [ 0x849, 'UnicodeCharacter0x849', [], 'Display', None],
  [ 0x84a, 'UnicodeCharacter0x84a', [], 'Display', None],
  [ 0x84b, 'UnicodeCharacter0x84b', [], 'Display', None],
  [ 0x84c, 'UnicodeCharacter0x84c', [], 'Display', None],
  [ 0x84d, 'UnicodeCharacter0x84d', [], 'Display', None],
  [ 0x84e, 'UnicodeCharacter0x84e', [], 'Display', None],
  [ 0x84f, 'UnicodeCharacter0x84f', [], 'Display', None],
  [ 0x850, 'UnicodeCharacter0x850', [], 'Display', None],
  [ 0x851, 'UnicodeCharacter0x851', [], 'Display', None],
  [ 0x852, 'UnicodeCharacter0x852', [], 'Display', None],
  [ 0x853, 'UnicodeCharacter0x853', [], 'Display', None],
  [ 0x854, 'UnicodeCharacter0x854', [], 'Display', None],
  [ 0x855, 'UnicodeCharacter0x855', [], 'Display', None],
  [ 0x856, 'UnicodeCharacter0x856', [], 'Display', None],
  [ 0x857, 'UnicodeCharacter0x857', [], 'Display', None],
  [ 0x858, 'UnicodeCharacter0x858', [], 'Display', None],
  [ 0x859, 'UnicodeCharacter0x859', [], 'Display', None],
  [ 0x85a, 'UnicodeCharacter0x85a', [], 'Display', None],
  [ 0x85b, 'UnicodeCharacter0x85b', [], 'Display', None],
  [ 0x85c, 'UnicodeCharacter0x85c', [], 'Display', None],
  [ 0x85d, 'UnicodeCharacter0x85d', [], 'Display', None],
  [ 0x85e, 'UnicodeCharacter0x85e', [], 'Display', None],
  [ 0x85f, 'UnicodeCharacter0x85f', [], 'Display', None],
  [ 0x860, 'UnicodeCharacter0x860', [], 'Display', None],
  [ 0x861, 'UnicodeCharacter0x861', [], 'Display', None],
  [ 0x862, 'UnicodeCharacter0x862', [], 'Display', None],
  [ 0x863, 'UnicodeCharacter0x863', [], 'Display', None],
  [ 0x864, 'UnicodeCharacter0x864', [], 'Display', None],
  [ 0x865, 'UnicodeCharacter0x865', [], 'Display', None],
  [ 0x866, 'UnicodeCharacter0x866', [], 'Display', None],
  [ 0x867, 'UnicodeCharacter0x867', [], 'Display', None],
  [ 0x868, 'UnicodeCharacter0x868', [], 'Display', None],
  [ 0x869, 'UnicodeCharacter0x869', [], 'Display', None],
  [ 0x86a, 'UnicodeCharacter0x86a', [], 'Display', None],
  [ 0x86b, 'UnicodeCharacter0x86b', [], 'Display', None],
  [ 0x86c, 'UnicodeCharacter0x86c', [], 'Display', None],
  [ 0x86d, 'UnicodeCharacter0x86d', [], 'Display', None],
  [ 0x86e, 'UnicodeCharacter0x86e', [], 'Display', None],
  [ 0x86f, 'UnicodeCharacter0x86f', [], 'Display', None],
  [ 0x870, 'UnicodeCharacter0x870', [], 'Display', None],
  [ 0x871, 'UnicodeCharacter0x871', [], 'Display', None],
  [ 0x872, 'UnicodeCharacter0x872', [], 'Display', None],
  [ 0x873, 'UnicodeCharacter0x873', [], 'Display', None],
  [ 0x874, 'UnicodeCharacter0x874', [], 'Display', None],
  [ 0x875, 'UnicodeCharacter0x875', [], 'Display', None],
  [ 0x876, 'UnicodeCharacter0x876', [], 'Display', None],
  [ 0x877, 'UnicodeCharacter0x877', [], 'Display', None],
  [ 0x878, 'UnicodeCharacter0x878', [], 'Display', None],
  [ 0x879, 'UnicodeCharacter0x879', [], 'Display', None],
  [ 0x87a, 'UnicodeCharacter0x87a', [], 'Display', None],
  [ 0x87b, 'UnicodeCharacter0x87b', [], 'Display', None],
  [ 0x87c, 'UnicodeCharacter0x87c', [], 'Display', None],
  [ 0x87d, 'UnicodeCharacter0x87d', [], 'Display', None],
  [ 0x87e, 'UnicodeCharacter0x87e', [], 'Display', None],
  [ 0x87f, 'UnicodeCharacter0x87f', [], 'Display', None],
  [ 0x880, 'UnicodeCharacter0x880', [], 'Display', None],
  [ 0x881, 'UnicodeCharacter0x881', [], 'Display', None],
  [ 0x882, 'UnicodeCharacter0x882', [], 'Display', None],
  [ 0x883, 'UnicodeCharacter0x883', [], 'Display', None],
  [ 0x884, 'UnicodeCharacter0x884', [], 'Display', None],
  [ 0x885, 'UnicodeCharacter0x885', [], 'Display', None],
  [ 0x886, 'UnicodeCharacter0x886', [], 'Display', None],
  [ 0x887, 'UnicodeCharacter0x887', [], 'Display', None],
  [ 0x888, 'UnicodeCharacter0x888', [], 'Display', None],
  [ 0x889, 'UnicodeCharacter0x889', [], 'Display', None],
  [ 0x88a, 'UnicodeCharacter0x88a', [], 'Display', None],
  [ 0x88b, 'UnicodeCharacter0x88b', [], 'Display', None],
  [ 0x88c, 'UnicodeCharacter0x88c', [], 'Display', None],
  [ 0x88d, 'UnicodeCharacter0x88d', [], 'Display', None],
  [ 0x88e, 'UnicodeCharacter0x88e', [], 'Display', None],
  [ 0x88f, 'UnicodeCharacter0x88f', [], 'Display', None],
  [ 0x890, 'UnicodeCharacter0x890', [], 'Display', None],
  [ 0x891, 'UnicodeCharacter0x891', [], 'Display', None],
  [ 0x892, 'UnicodeCharacter0x892', [], 'Display', None],
  [ 0x893, 'UnicodeCharacter0x893', [], 'Display', None],
  [ 0x894, 'UnicodeCharacter0x894', [], 'Display', None],
  [ 0x895, 'UnicodeCharacter0x895', [], 'Display', None],
  [ 0x896, 'UnicodeCharacter0x896', [], 'Display', None],
  [ 0x897, 'UnicodeCharacter0x897', [], 'Display', None],
  [ 0x898, 'UnicodeCharacter0x898', [], 'Display', None],
  [ 0x899, 'UnicodeCharacter0x899', [], 'Display', None],
  [ 0x89a, 'UnicodeCharacter0x89a', [], 'Display', None],
  [ 0x89b, 'UnicodeCharacter0x89b', [], 'Display', None],
  [ 0x89c, 'UnicodeCharacter0x89c', [], 'Display', None],
  [ 0x89d, 'UnicodeCharacter0x89d', [], 'Display', None],
  [ 0x89e, 'UnicodeCharacter0x89e', [], 'Display', None],
  [ 0x89f, 'UnicodeCharacter0x89f', [], 'Display', None],
  [ 0x8a0, 'UnicodeCharacter0x8a0', [], 'Display', None],
  [ 0x8a1, 'UnicodeCharacter0x8a1', [], 'Display', None],
  [ 0x8a2, 'UnicodeCharacter0x8a2', [], 'Display', None],
  [ 0x8a3, 'UnicodeCharacter0x8a3', [], 'Display', None],
  [ 0x8a4, 'UnicodeCharacter0x8a4', [], 'Display', None],
  [ 0x8a5, 'UnicodeCharacter0x8a5', [], 'Display', None],
  [ 0x8a6, 'UnicodeCharacter0x8a6', [], 'Display', None],
  [ 0x8a7, 'UnicodeCharacter0x8a7', [], 'Display', None],
  [ 0x8a8, 'UnicodeCharacter0x8a8', [], 'Display', None],
  [ 0x8a9, 'UnicodeCharacter0x8a9', [], 'Display', None],
  [ 0x8aa, 'UnicodeCharacter0x8aa', [], 'Display', None],
  [ 0x8ab, 'UnicodeCharacter0x8ab', [], 'Display', None],
  [ 0x8ac, 'UnicodeCharacter0x8ac', [], 'Display', None],
  [ 0x8ad, 'UnicodeCharacter0x8ad', [], 'Display', None],
  [ 0x8ae, 'UnicodeCharacter0x8ae', [], 'Display', None],
  [ 0x8af, 'UnicodeCharacter0x8af', [], 'Display', None],
  [ 0x8b0, 'UnicodeCharacter0x8b0', [], 'Display', None],
  [ 0x8b1, 'UnicodeCharacter0x8b1', [], 'Display', None],
  [ 0x8b2, 'UnicodeCharacter0x8b2', [], 'Display', None],
  [ 0x8b3, 'UnicodeCharacter0x8b3', [], 'Display', None],
  [ 0x8b4, 'UnicodeCharacter0x8b4', [], 'Display', None],
  [ 0x8b5, 'UnicodeCharacter0x8b5', [], 'Display', None],
  [ 0x8b6, 'UnicodeCharacter0x8b6', [], 'Display', None],
  [ 0x8b7, 'UnicodeCharacter0x8b7', [], 'Display', None],
  [ 0x8b8, 'UnicodeCharacter0x8b8', [], 'Display', None],
  [ 0x8b9, 'UnicodeCharacter0x8b9', [], 'Display', None],
  [ 0x8ba, 'UnicodeCharacter0x8ba', [], 'Display', None],
  [ 0x8bb, 'UnicodeCharacter0x8bb', [], 'Display', None],
  [ 0x8bc, 'UnicodeCharacter0x8bc', [], 'Display', None],
  [ 0x8bd, 'UnicodeCharacter0x8bd', [], 'Display', None],
  [ 0x8be, 'UnicodeCharacter0x8be', [], 'Display', None],
  [ 0x8bf, 'UnicodeCharacter0x8bf', [], 'Display', None],
  [ 0x8c0, 'UnicodeCharacter0x8c0', [], 'Display', None],
  [ 0x8c1, 'UnicodeCharacter0x8c1', [], 'Display', None],
  [ 0x8c2, 'UnicodeCharacter0x8c2', [], 'Display', None],
  [ 0x8c3, 'UnicodeCharacter0x8c3', [], 'Display', None],
  [ 0x8c4, 'UnicodeCharacter0x8c4', [], 'Display', None],
  [ 0x8c5, 'UnicodeCharacter0x8c5', [], 'Display', None],
  [ 0x8c6, 'UnicodeCharacter0x8c6', [], 'Display', None],
  [ 0x8c7, 'UnicodeCharacter0x8c7', [], 'Display', None],
  [ 0x8c8, 'UnicodeCharacter0x8c8', [], 'Display', None],
  [ 0x8c9, 'UnicodeCharacter0x8c9', [], 'Display', None],
  [ 0x8ca, 'UnicodeCharacter0x8ca', [], 'Display', None],
  [ 0x8cb, 'UnicodeCharacter0x8cb', [], 'Display', None],
  [ 0x8cc, 'UnicodeCharacter0x8cc', [], 'Display', None],
  [ 0x8cd, 'UnicodeCharacter0x8cd', [], 'Display', None],
  [ 0x8ce, 'UnicodeCharacter0x8ce', [], 'Display', None],
  [ 0x8cf, 'UnicodeCharacter0x8cf', [], 'Display', None],
  [ 0x8d0, 'UnicodeCharacter0x8d0', [], 'Display', None],
  [ 0x8d1, 'UnicodeCharacter0x8d1', [], 'Display', None],
  [ 0x8d2, 'UnicodeCharacter0x8d2', [], 'Display', None],
  [ 0x8d3, 'UnicodeCharacter0x8d3', [], 'Display', None],
  [ 0x8d4, 'UnicodeCharacter0x8d4', [], 'Display', None],
  [ 0x8d5, 'UnicodeCharacter0x8d5', [], 'Display', None],
  [ 0x8d6, 'UnicodeCharacter0x8d6', [], 'Display', None],
  [ 0x8d7, 'UnicodeCharacter0x8d7', [], 'Display', None],
  [ 0x8d8, 'UnicodeCharacter0x8d8', [], 'Display', None],
  [ 0x8d9, 'UnicodeCharacter0x8d9', [], 'Display', None],
  [ 0x8da, 'UnicodeCharacter0x8da', [], 'Display', None],
  [ 0x8db, 'UnicodeCharacter0x8db', [], 'Display', None],
  [ 0x8dc, 'UnicodeCharacter0x8dc', [], 'Display', None],
  [ 0x8dd, 'UnicodeCharacter0x8dd', [], 'Display', None],
  [ 0x8de, 'UnicodeCharacter0x8de', [], 'Display', None],
  [ 0x8df, 'UnicodeCharacter0x8df', [], 'Display', None],
  [ 0x8e0, 'UnicodeCharacter0x8e0', [], 'Display', None],
  [ 0x8e1, 'UnicodeCharacter0x8e1', [], 'Display', None],
  [ 0x8e2, 'UnicodeCharacter0x8e2', [], 'Display', None],
  [ 0x8e3, 'UnicodeCharacter0x8e3', [], 'Display', None],
  [ 0x8e4, 'UnicodeCharacter0x8e4', [], 'Display', None],
  [ 0x8e5, 'UnicodeCharacter0x8e5', [], 'Display', None],
  [ 0x8e6, 'UnicodeCharacter0x8e6', [], 'Display', None],
  [ 0x8e7, 'UnicodeCharacter0x8e7', [], 'Display', None],
  [ 0x8e8, 'UnicodeCharacter0x8e8', [], 'Display', None],
  [ 0x8e9, 'UnicodeCharacter0x8e9', [], 'Display', None],
  [ 0x8ea, 'UnicodeCharacter0x8ea', [], 'Display', None],
  [ 0x8eb, 'UnicodeCharacter0x8eb', [], 'Display', None],
  [ 0x8ec, 'UnicodeCharacter0x8ec', [], 'Display', None],
  [ 0x8ed, 'UnicodeCharacter0x8ed', [], 'Display', None],
  [ 0x8ee, 'UnicodeCharacter0x8ee', [], 'Display', None],
  [ 0x8ef, 'UnicodeCharacter0x8ef', [], 'Display', None],
  [ 0x8f0, 'UnicodeCharacter0x8f0', [], 'Display', None],
  [ 0x8f1, 'UnicodeCharacter0x8f1', [], 'Display', None],
  [ 0x8f2, 'UnicodeCharacter0x8f2', [], 'Display', None],
  [ 0x8f3, 'UnicodeCharacter0x8f3', [], 'Display', None],
  [ 0x8f4, 'UnicodeCharacter0x8f4', [], 'Display', None],
  [ 0x8f5, 'UnicodeCharacter0x8f5', [], 'Display', None],
  [ 0x8f6, 'UnicodeCharacter0x8f6', [], 'Display', None],
  [ 0x8f7, 'UnicodeCharacter0x8f7', [], 'Display', None],
  [ 0x8f8, 'UnicodeCharacter0x8f8', [], 'Display', None],
  [ 0x8f9, 'UnicodeCharacter0x8f9', [], 'Display', None],
  [ 0x8fa, 'UnicodeCharacter0x8fa', [], 'Display', None],
  [ 0x8fb, 'UnicodeCharacter0x8fb', [], 'Display', None],
  [ 0x8fc, 'UnicodeCharacter0x8fc', [], 'Display', None],
  [ 0x8fd, 'UnicodeCharacter0x8fd', [], 'Display', None],
  [ 0x8fe, 'UnicodeCharacter0x8fe', [], 'Display', None],
  [ 0x8ff, 'UnicodeCharacter0x8ff', [], 'Display', None],
  [ 0x900, 'UnicodeCharacter0x900', [], 'Display', None],
  [ 0x901, 'DevanagariSignCandrabindu', [], 'Display', None],
  [ 0x902, 'DevanagariSignAnusvara', [], 'Display', None],
  [ 0x903, 'DevanagariSignVisarga', [], 'Display', None],
  [ 0x904, 'DevanagariLetterShortA', [], 'Display', None],
  [ 0x905, 'DevanagariLetterA', [], 'Display', None],
  [ 0x906, 'DevanagariLetterAa', [], 'Display', None],
  [ 0x907, 'DevanagariLetterI', [], 'Display', None],
  [ 0x908, 'DevanagariLetterIi', [], 'Display', None],
  [ 0x909, 'DevanagariLetterU', [], 'Display', None],
  [ 0x90a, 'DevanagariLetterUu', [], 'Display', None],
  [ 0x90b, 'DevanagariLetterVocalicR', [], 'Display', None],
  [ 0x90c, 'DevanagariLetterVocalicL', [], 'Display', None],
  [ 0x90d, 'DevanagariLetterCandraE', [], 'Display', None],
  [ 0x90e, 'DevanagariLetterShortE', [], 'Display', None],
  [ 0x90f, 'DevanagariLetterE', [], 'Display', None],
  [ 0x910, 'DevanagariLetterAi', [], 'Display', None],
  [ 0x911, 'DevanagariLetterCandraO', [], 'Display', None],
  [ 0x912, 'DevanagariLetterShortO', [], 'Display', None],
  [ 0x913, 'DevanagariLetterO', [], 'Display', None],
  [ 0x914, 'DevanagariLetterAu', [], 'Display', None],
  [ 0x915, 'DevanagariLetterKa', [], 'Display', None],
  [ 0x916, 'DevanagariLetterKha', [], 'Display', None],
  [ 0x917, 'DevanagariLetterGa', [], 'Display', None],
  [ 0x918, 'DevanagariLetterGha', [], 'Display', None],
  [ 0x919, 'DevanagariLetterNga', [], 'Display', None],
  [ 0x91a, 'DevanagariLetterCa', [], 'Display', None],
  [ 0x91b, 'DevanagariLetterCha', [], 'Display', None],
  [ 0x91c, 'DevanagariLetterJa', [], 'Display', None],
  [ 0x91d, 'DevanagariLetterJha', [], 'Display', None],
  [ 0x91e, 'DevanagariLetterNya', [], 'Display', None],
  [ 0x91f, 'DevanagariLetterTta', [], 'Display', None],
  [ 0x920, 'DevanagariLetterTtha', [], 'Display', None],
  [ 0x921, 'DevanagariLetterDda', [], 'Display', None],
  [ 0x922, 'DevanagariLetterDdha', [], 'Display', None],
  [ 0x923, 'DevanagariLetterNna', [], 'Display', None],
  [ 0x924, 'DevanagariLetterTa', [], 'Display', None],
  [ 0x925, 'DevanagariLetterTha', [], 'Display', None],
  [ 0x926, 'DevanagariLetterDa', [], 'Display', None],
  [ 0x927, 'DevanagariLetterDha', [], 'Display', None],
  [ 0x928, 'DevanagariLetterNa', [], 'Display', None],
  [ 0x929, 'DevanagariLetterNnna', [], 'Display', None],
  [ 0x92a, 'DevanagariLetterPa', [], 'Display', None],
  [ 0x92b, 'DevanagariLetterPha', [], 'Display', None],
  [ 0x92c, 'DevanagariLetterBa', [], 'Display', None],
  [ 0x92d, 'DevanagariLetterBha', [], 'Display', None],
  [ 0x92e, 'DevanagariLetterMa', [], 'Display', None],
  [ 0x92f, 'DevanagariLetterYa', [], 'Display', None],
  [ 0x930, 'DevanagariLetterRa', [], 'Display', None],
  [ 0x931, 'DevanagariLetterRra', [], 'Display', None],
  [ 0x932, 'DevanagariLetterLa', [], 'Display', None],
  [ 0x933, 'DevanagariLetterLla', [], 'Display', None],
  [ 0x934, 'DevanagariLetterLlla', [], 'Display', None],
  [ 0x935, 'DevanagariLetterVa', [], 'Display', None],
  [ 0x936, 'DevanagariLetterSha', [], 'Display', None],
  [ 0x937, 'DevanagariLetterSsa', [], 'Display', None],
  [ 0x938, 'DevanagariLetterSa', [], 'Display', None],
  [ 0x939, 'DevanagariLetterHa', [], 'Display', None],
  [ 0x93a, 'UnicodeCharacter0x93a', [], 'Display', None],
  [ 0x93b, 'UnicodeCharacter0x93b', [], 'Display', None],
  [ 0x93c, 'DevanagariSignNukta', [], 'Display', None],
  [ 0x93d, 'DevanagariSignAvagraha', [], 'Display', None],
  [ 0x93e, 'DevanagariVowelSignAa', [], 'Display', None],
  [ 0x93f, 'DevanagariVowelSignI', [], 'Display', None],
  [ 0x940, 'DevanagariVowelSignIi', [], 'Display', None],
  [ 0x941, 'DevanagariVowelSignU', [], 'Display', None],
  [ 0x942, 'DevanagariVowelSignUu', [], 'Display', None],
  [ 0x943, 'DevanagariVowelSignVocalicR', [], 'Display', None],
  [ 0x944, 'DevanagariVowelSignVocalicRr', [], 'Display', None],
  [ 0x945, 'DevanagariVowelSignCandraE', [], 'Display', None],
  [ 0x946, 'DevanagariVowelSignShortE', [], 'Display', None],
  [ 0x947, 'DevanagariVowelSignE', [], 'Display', None],
  [ 0x948, 'DevanagariVowelSignAi', [], 'Display', None],
  [ 0x949, 'DevanagariVowelSignCandraO', [], 'Display', None],
  [ 0x94a, 'DevanagariVowelSignShortO', [], 'Display', None],
  [ 0x94b, 'DevanagariVowelSignO', [], 'Display', None],
  [ 0x94c, 'DevanagariVowelSignAu', [], 'Display', None],
  [ 0x94d, 'DevanagariSignVirama', [], 'Display', None],
  [ 0x94e, 'UnicodeCharacter0x94e', [], 'Display', None],
  [ 0x94f, 'UnicodeCharacter0x94f', [], 'Display', None],
  [ 0x950, 'DevanagariOm', [], 'Display', None],
  [ 0x951, 'DevanagariStressSignUdatta', [], 'Display', None],
  [ 0x952, 'DevanagariStressSignAnudatta', [], 'Display', None],
  [ 0x953, 'DevanagariGraveAccent', [], 'Display', None],
  [ 0x954, 'DevanagariAcuteAccent', [], 'Display', None],
  [ 0x955, 'UnicodeCharacter0x955', [], 'Display', None],
  [ 0x956, 'UnicodeCharacter0x956', [], 'Display', None],
  [ 0x957, 'UnicodeCharacter0x957', [], 'Display', None],
  [ 0x958, 'DevanagariLetterQa', [], 'Display', None],
  [ 0x959, 'DevanagariLetterKhha', [], 'Display', None],
  [ 0x95a, 'DevanagariLetterGhha', [], 'Display', None],
  [ 0x95b, 'DevanagariLetterZa', [], 'Display', None],
  [ 0x95c, 'DevanagariLetterDddha', [], 'Display', None],
  [ 0x95d, 'DevanagariLetterRha', [], 'Display', None],
  [ 0x95e, 'DevanagariLetterFa', [], 'Display', None],
  [ 0x95f, 'DevanagariLetterYya', [], 'Display', None],
  [ 0x960, 'DevanagariLetterVocalicRr', [], 'Display', None],
  [ 0x961, 'DevanagariLetterVocalicLl', [], 'Display', None],
  [ 0x962, 'DevanagariVowelSignVocalicL', [], 'Display', None],
  [ 0x963, 'DevanagariVowelSignVocalicLl', [], 'Display', None],
  [ 0x964, 'DevanagariDanda', [], 'Display', None],
  [ 0x965, 'DevanagariDoubleDanda', [], 'Display', None],
  [ 0x966, 'DevanagariDigitZero', [], 'Display', None],
  [ 0x967, 'DevanagariDigitOne', [], 'Display', None],
  [ 0x968, 'DevanagariDigitTwo', [], 'Display', None],
  [ 0x969, 'DevanagariDigitThree', [], 'Display', None],
  [ 0x96a, 'DevanagariDigitFour', [], 'Display', None],
  [ 0x96b, 'DevanagariDigitFive', [], 'Display', None],
  [ 0x96c, 'DevanagariDigitSix', [], 'Display', None],
  [ 0x96d, 'DevanagariDigitSeven', [], 'Display', None],
  [ 0x96e, 'DevanagariDigitEight', [], 'Display', None],
  [ 0x96f, 'DevanagariDigitNine', [], 'Display', None],
  [ 0x970, 'DevanagariAbbreviationSign', [], 'Display', None],
  [ 0x971, 'DevanagariSignHighSpacingDot', [], 'Display', None],
  [ 0x972, 'DevanagariLetterCandraA', [], 'Display', None],
  [ 0x973, 'UnicodeCharacter0x973', [], 'Display', None],
  [ 0x974, 'UnicodeCharacter0x974', [], 'Display', None],
  [ 0x975, 'UnicodeCharacter0x975', [], 'Display', None],
  [ 0x976, 'UnicodeCharacter0x976', [], 'Display', None],
  [ 0x977, 'UnicodeCharacter0x977', [], 'Display', None],
  [ 0x978, 'UnicodeCharacter0x978', [], 'Display', None],
  [ 0x979, 'UnicodeCharacter0x979', [], 'Display', None],
  [ 0x97a, 'UnicodeCharacter0x97a', [], 'Display', None],
  [ 0x97b, 'DevanagariLetterGga', [], 'Display', None],
  [ 0x97c, 'DevanagariLetterJja', [], 'Display', None],
  [ 0x97d, 'DevanagariLetterGlottalStop', [], 'Display', None],
  [ 0x97e, 'DevanagariLetterDdda', [], 'Display', None],
  [ 0x97f, 'DevanagariLetterBba', [], 'Display', None],
  [ 0x980, 'UnicodeCharacter0x980', [], 'Display', None],
  [ 0x981, 'BengaliSignCandrabindu', [], 'Display', None],
  [ 0x982, 'BengaliSignAnusvara', [], 'Display', None],
  [ 0x983, 'BengaliSignVisarga', [], 'Display', None],
  [ 0x984, 'UnicodeCharacter0x984', [], 'Display', None],
  [ 0x985, 'BengaliLetterA', [], 'Display', None],
  [ 0x986, 'BengaliLetterAa', [], 'Display', None],
  [ 0x987, 'BengaliLetterI', [], 'Display', None],
  [ 0x988, 'BengaliLetterIi', [], 'Display', None],
  [ 0x989, 'BengaliLetterU', [], 'Display', None],
  [ 0x98a, 'BengaliLetterUu', [], 'Display', None],
  [ 0x98b, 'BengaliLetterVocalicR', [], 'Display', None],
  [ 0x98c, 'BengaliLetterVocalicL', [], 'Display', None],
  [ 0x98d, 'UnicodeCharacter0x98d', [], 'Display', None],
  [ 0x98e, 'UnicodeCharacter0x98e', [], 'Display', None],
  [ 0x98f, 'BengaliLetterE', [], 'Display', None],
  [ 0x990, 'BengaliLetterAi', [], 'Display', None],
  [ 0x991, 'UnicodeCharacter0x991', [], 'Display', None],
  [ 0x992, 'UnicodeCharacter0x992', [], 'Display', None],
  [ 0x993, 'BengaliLetterO', [], 'Display', None],
  [ 0x994, 'BengaliLetterAu', [], 'Display', None],
  [ 0x995, 'BengaliLetterKa', [], 'Display', None],
  [ 0x996, 'BengaliLetterKha', [], 'Display', None],
  [ 0x997, 'BengaliLetterGa', [], 'Display', None],
  [ 0x998, 'BengaliLetterGha', [], 'Display', None],
  [ 0x999, 'BengaliLetterNga', [], 'Display', None],
  [ 0x99a, 'BengaliLetterCa', [], 'Display', None],
  [ 0x99b, 'BengaliLetterCha', [], 'Display', None],
  [ 0x99c, 'BengaliLetterJa', [], 'Display', None],
  [ 0x99d, 'BengaliLetterJha', [], 'Display', None],
  [ 0x99e, 'BengaliLetterNya', [], 'Display', None],
  [ 0x99f, 'BengaliLetterTta', [], 'Display', None],
  [ 0x9a0, 'BengaliLetterTtha', [], 'Display', None],
  [ 0x9a1, 'BengaliLetterDda', [], 'Display', None],
  [ 0x9a2, 'BengaliLetterDdha', [], 'Display', None],
  [ 0x9a3, 'BengaliLetterNna', [], 'Display', None],
  [ 0x9a4, 'BengaliLetterTa', [], 'Display', None],
  [ 0x9a5, 'BengaliLetterTha', [], 'Display', None],
  [ 0x9a6, 'BengaliLetterDa', [], 'Display', None],
  [ 0x9a7, 'BengaliLetterDha', [], 'Display', None],
  [ 0x9a8, 'BengaliLetterNa', [], 'Display', None],
  [ 0x9a9, 'UnicodeCharacter0x9a9', [], 'Display', None],
  [ 0x9aa, 'BengaliLetterPa', [], 'Display', None],
  [ 0x9ab, 'BengaliLetterPha', [], 'Display', None],
  [ 0x9ac, 'BengaliLetterBa', [], 'Display', None],
  [ 0x9ad, 'BengaliLetterBha', [], 'Display', None],
  [ 0x9ae, 'BengaliLetterMa', [], 'Display', None],
  [ 0x9af, 'BengaliLetterYa', [], 'Display', None],
  [ 0x9b0, 'BengaliLetterRa', [], 'Display', None],
  [ 0x9b1, 'UnicodeCharacter0x9b1', [], 'Display', None],
  [ 0x9b2, 'BengaliLetterLa', [], 'Display', None],
  [ 0x9b3, 'UnicodeCharacter0x9b3', [], 'Display', None],
  [ 0x9b4, 'UnicodeCharacter0x9b4', [], 'Display', None],
  [ 0x9b5, 'UnicodeCharacter0x9b5', [], 'Display', None],
  [ 0x9b6, 'BengaliLetterSha', [], 'Display', None],
  [ 0x9b7, 'BengaliLetterSsa', [], 'Display', None],
  [ 0x9b8, 'BengaliLetterSa', [], 'Display', None],
  [ 0x9b9, 'BengaliLetterHa', [], 'Display', None],
  [ 0x9ba, 'UnicodeCharacter0x9ba', [], 'Display', None],
  [ 0x9bb, 'UnicodeCharacter0x9bb', [], 'Display', None],
  [ 0x9bc, 'BengaliSignNukta', [], 'Display', None],
  [ 0x9bd, 'BengaliSignAvagraha', [], 'Display', None],
  [ 0x9be, 'BengaliVowelSignAa', [], 'Display', None],
  [ 0x9bf, 'BengaliVowelSignI', [], 'Display', None],
  [ 0x9c0, 'BengaliVowelSignIi', [], 'Display', None],
  [ 0x9c1, 'BengaliVowelSignU', [], 'Display', None],
  [ 0x9c2, 'BengaliVowelSignUu', [], 'Display', None],
  [ 0x9c3, 'BengaliVowelSignVocalicR', [], 'Display', None],
  [ 0x9c4, 'BengaliVowelSignVocalicRr', [], 'Display', None],
  [ 0x9c5, 'UnicodeCharacter0x9c5', [], 'Display', None],
  [ 0x9c6, 'UnicodeCharacter0x9c6', [], 'Display', None],
  [ 0x9c7, 'BengaliVowelSignE', [], 'Display', None],
  [ 0x9c8, 'BengaliVowelSignAi', [], 'Display', None],
  [ 0x9c9, 'UnicodeCharacter0x9c9', [], 'Display', None],
  [ 0x9ca, 'UnicodeCharacter0x9ca', [], 'Display', None],
  [ 0x9cb, 'BengaliVowelSignO', [], 'Display', None],
  [ 0x9cc, 'BengaliVowelSignAu', [], 'Display', None],
  [ 0x9cd, 'BengaliSignVirama', [], 'Display', None],
  [ 0x9ce, 'BengaliLetterKhandaTa', [], 'Display', None],
  [ 0x9cf, 'UnicodeCharacter0x9cf', [], 'Display', None],
  [ 0x9d0, 'UnicodeCharacter0x9d0', [], 'Display', None],
  [ 0x9d1, 'UnicodeCharacter0x9d1', [], 'Display', None],
  [ 0x9d2, 'UnicodeCharacter0x9d2', [], 'Display', None],
  [ 0x9d3, 'UnicodeCharacter0x9d3', [], 'Display', None],
  [ 0x9d4, 'UnicodeCharacter0x9d4', [], 'Display', None],
  [ 0x9d5, 'UnicodeCharacter0x9d5', [], 'Display', None],
  [ 0x9d6, 'UnicodeCharacter0x9d6', [], 'Display', None],
  [ 0x9d7, 'BengaliAuLengthMark', [], 'Display', None],
  [ 0x9d8, 'UnicodeCharacter0x9d8', [], 'Display', None],
  [ 0x9d9, 'UnicodeCharacter0x9d9', [], 'Display', None],
  [ 0x9da, 'UnicodeCharacter0x9da', [], 'Display', None],
  [ 0x9db, 'UnicodeCharacter0x9db', [], 'Display', None],
  [ 0x9dc, 'BengaliLetterRra', [], 'Display', None],
  [ 0x9dd, 'BengaliLetterRha', [], 'Display', None],
  [ 0x9de, 'UnicodeCharacter0x9de', [], 'Display', None],
  [ 0x9df, 'BengaliLetterYya', [], 'Display', None],
  [ 0x9e0, 'BengaliLetterVocalicRr', [], 'Display', None],
  [ 0x9e1, 'BengaliLetterVocalicLl', [], 'Display', None],
  [ 0x9e2, 'BengaliVowelSignVocalicL', [], 'Display', None],
  [ 0x9e3, 'BengaliVowelSignVocalicLl', [], 'Display', None],
  [ 0x9e4, 'UnicodeCharacter0x9e4', [], 'Display', None],
  [ 0x9e5, 'UnicodeCharacter0x9e5', [], 'Display', None],
  [ 0x9e6, 'BengaliDigitZero', [], 'Display', None],
  [ 0x9e7, 'BengaliDigitOne', [], 'Display', None],
  [ 0x9e8, 'BengaliDigitTwo', [], 'Display', None],
  [ 0x9e9, 'BengaliDigitThree', [], 'Display', None],
  [ 0x9ea, 'BengaliDigitFour', [], 'Display', None],
  [ 0x9eb, 'BengaliDigitFive', [], 'Display', None],
  [ 0x9ec, 'BengaliDigitSix', [], 'Display', None],
  [ 0x9ed, 'BengaliDigitSeven', [], 'Display', None],
  [ 0x9ee, 'BengaliDigitEight', [], 'Display', None],
  [ 0x9ef, 'BengaliDigitNine', [], 'Display', None],
  [ 0x9f0, 'BengaliLetterRaWithMiddleDiagonal', [], 'Display', None],
  [ 0x9f1, 'BengaliLetterRaWithLowerDiagonal', [], 'Display', None],
  [ 0x9f2, 'BengaliRupeeMark', [], 'Display', None],
  [ 0x9f3, 'BengaliRupeeSign', [], 'Display', None],
  [ 0x9f4, 'BengaliCurrencyNumeratorOne', [], 'Display', None],
  [ 0x9f5, 'BengaliCurrencyNumeratorTwo', [], 'Display', None],
  [ 0x9f6, 'BengaliCurrencyNumeratorThree', [], 'Display', None],
  [ 0x9f7, 'BengaliCurrencyNumeratorFour', [], 'Display', None],
  [ 0x9f8, 'BengaliCurrencyNumeratorOneLessThanTheDenominator', [], 'Display', None],
  [ 0x9f9, 'BengaliCurrencyDenominatorSixteen', [], 'Display', None],
  [ 0x9fa, 'BengaliIsshar', [], 'Display', None],
  [ 0x9fb, 'UnicodeCharacter0x9fb', [], 'Display', None],
  [ 0x9fc, 'UnicodeCharacter0x9fc', [], 'Display', None],
  [ 0x9fd, 'UnicodeCharacter0x9fd', [], 'Display', None],
  [ 0x9fe, 'UnicodeCharacter0x9fe', [], 'Display', None],
  [ 0x9ff, 'UnicodeCharacter0x9ff', [], 'Display', None],
  [ 0xa00, 'UnicodeCharacter0xa00', [], 'Display', None],
  [ 0xa01, 'GurmukhiSignAdakBindi', [], 'Display', None],
  [ 0xa02, 'GurmukhiSignBindi', [], 'Display', None],
  [ 0xa03, 'GurmukhiSignVisarga', [], 'Display', None],
  [ 0xa04, 'UnicodeCharacter0xa04', [], 'Display', None],
  [ 0xa05, 'GurmukhiLetterA', [], 'Display', None],
  [ 0xa06, 'GurmukhiLetterAa', [], 'Display', None],
  [ 0xa07, 'GurmukhiLetterI', [], 'Display', None],
  [ 0xa08, 'GurmukhiLetterIi', [], 'Display', None],
  [ 0xa09, 'GurmukhiLetterU', [], 'Display', None],
  [ 0xa0a, 'GurmukhiLetterUu', [], 'Display', None],
  [ 0xa0b, 'UnicodeCharacter0xa0b', [], 'Display', None],
  [ 0xa0c, 'UnicodeCharacter0xa0c', [], 'Display', None],
  [ 0xa0d, 'UnicodeCharacter0xa0d', [], 'Display', None],
  [ 0xa0e, 'UnicodeCharacter0xa0e', [], 'Display', None],
  [ 0xa0f, 'GurmukhiLetterEe', [], 'Display', None],
  [ 0xa10, 'GurmukhiLetterAi', [], 'Display', None],
  [ 0xa11, 'UnicodeCharacter0xa11', [], 'Display', None],
  [ 0xa12, 'UnicodeCharacter0xa12', [], 'Display', None],
  [ 0xa13, 'GurmukhiLetterOo', [], 'Display', None],
  [ 0xa14, 'GurmukhiLetterAu', [], 'Display', None],
  [ 0xa15, 'GurmukhiLetterKa', [], 'Display', None],
  [ 0xa16, 'GurmukhiLetterKha', [], 'Display', None],
  [ 0xa17, 'GurmukhiLetterGa', [], 'Display', None],
  [ 0xa18, 'GurmukhiLetterGha', [], 'Display', None],
  [ 0xa19, 'GurmukhiLetterNga', [], 'Display', None],
  [ 0xa1a, 'GurmukhiLetterCa', [], 'Display', None],
  [ 0xa1b, 'GurmukhiLetterCha', [], 'Display', None],
  [ 0xa1c, 'GurmukhiLetterJa', [], 'Display', None],
  [ 0xa1d, 'GurmukhiLetterJha', [], 'Display', None],
  [ 0xa1e, 'GurmukhiLetterNya', [], 'Display', None],
  [ 0xa1f, 'GurmukhiLetterTta', [], 'Display', None],
  [ 0xa20, 'GurmukhiLetterTtha', [], 'Display', None],
  [ 0xa21, 'GurmukhiLetterDda', [], 'Display', None],
  [ 0xa22, 'GurmukhiLetterDdha', [], 'Display', None],
  [ 0xa23, 'GurmukhiLetterNna', [], 'Display', None],
  [ 0xa24, 'GurmukhiLetterTa', [], 'Display', None],
  [ 0xa25, 'GurmukhiLetterTha', [], 'Display', None],
  [ 0xa26, 'GurmukhiLetterDa', [], 'Display', None],
  [ 0xa27, 'GurmukhiLetterDha', [], 'Display', None],
  [ 0xa28, 'GurmukhiLetterNa', [], 'Display', None],
  [ 0xa29, 'UnicodeCharacter0xa29', [], 'Display', None],
  [ 0xa2a, 'GurmukhiLetterPa', [], 'Display', None],
  [ 0xa2b, 'GurmukhiLetterPha', [], 'Display', None],
  [ 0xa2c, 'GurmukhiLetterBa', [], 'Display', None],
  [ 0xa2d, 'GurmukhiLetterBha', [], 'Display', None],
  [ 0xa2e, 'GurmukhiLetterMa', [], 'Display', None],
  [ 0xa2f, 'GurmukhiLetterYa', [], 'Display', None],
  [ 0xa30, 'GurmukhiLetterRa', [], 'Display', None],
  [ 0xa31, 'UnicodeCharacter0xa31', [], 'Display', None],
  [ 0xa32, 'GurmukhiLetterLa', [], 'Display', None],
  [ 0xa33, 'GurmukhiLetterLla', [], 'Display', None],
  [ 0xa34, 'UnicodeCharacter0xa34', [], 'Display', None],
  [ 0xa35, 'GurmukhiLetterVa', [], 'Display', None],
  [ 0xa36, 'GurmukhiLetterSha', [], 'Display', None],
  [ 0xa37, 'UnicodeCharacter0xa37', [], 'Display', None],
  [ 0xa38, 'GurmukhiLetterSa', [], 'Display', None],
  [ 0xa39, 'GurmukhiLetterHa', [], 'Display', None],
  [ 0xa3a, 'UnicodeCharacter0xa3a', [], 'Display', None],
  [ 0xa3b, 'UnicodeCharacter0xa3b', [], 'Display', None],
  [ 0xa3c, 'GurmukhiSignNukta', [], 'Display', None],
  [ 0xa3d, 'UnicodeCharacter0xa3d', [], 'Display', None],
  [ 0xa3e, 'GurmukhiVowelSignAa', [], 'Display', None],
  [ 0xa3f, 'GurmukhiVowelSignI', [], 'Display', None],
  [ 0xa40, 'GurmukhiVowelSignIi', [], 'Display', None],
  [ 0xa41, 'GurmukhiVowelSignU', [], 'Display', None],
  [ 0xa42, 'GurmukhiVowelSignUu', [], 'Display', None],
  [ 0xa43, 'UnicodeCharacter0xa43', [], 'Display', None],
  [ 0xa44, 'UnicodeCharacter0xa44', [], 'Display', None],
  [ 0xa45, 'UnicodeCharacter0xa45', [], 'Display', None],
  [ 0xa46, 'UnicodeCharacter0xa46', [], 'Display', None],
  [ 0xa47, 'GurmukhiVowelSignEe', [], 'Display', None],
  [ 0xa48, 'GurmukhiVowelSignAi', [], 'Display', None],
  [ 0xa49, 'UnicodeCharacter0xa49', [], 'Display', None],
  [ 0xa4a, 'UnicodeCharacter0xa4a', [], 'Display', None],
  [ 0xa4b, 'GurmukhiVowelSignOo', [], 'Display', None],
  [ 0xa4c, 'GurmukhiVowelSignAu', [], 'Display', None],
  [ 0xa4d, 'GurmukhiSignVirama', [], 'Display', None],
  [ 0xa4e, 'UnicodeCharacter0xa4e', [], 'Display', None],
  [ 0xa4f, 'UnicodeCharacter0xa4f', [], 'Display', None],
  [ 0xa50, 'UnicodeCharacter0xa50', [], 'Display', None],
  [ 0xa51, 'GurmukhiSignUdaat', [], 'Display', None],
  [ 0xa52, 'UnicodeCharacter0xa52', [], 'Display', None],
  [ 0xa53, 'UnicodeCharacter0xa53', [], 'Display', None],
  [ 0xa54, 'UnicodeCharacter0xa54', [], 'Display', None],
  [ 0xa55, 'UnicodeCharacter0xa55', [], 'Display', None],
  [ 0xa56, 'UnicodeCharacter0xa56', [], 'Display', None],
  [ 0xa57, 'UnicodeCharacter0xa57', [], 'Display', None],
  [ 0xa58, 'UnicodeCharacter0xa58', [], 'Display', None],
  [ 0xa59, 'GurmukhiLetterKhha', [], 'Display', None],
  [ 0xa5a, 'GurmukhiLetterGhha', [], 'Display', None],
  [ 0xa5b, 'GurmukhiLetterZa', [], 'Display', None],
  [ 0xa5c, 'GurmukhiLetterRra', [], 'Display', None],
  [ 0xa5d, 'UnicodeCharacter0xa5d', [], 'Display', None],
  [ 0xa5e, 'GurmukhiLetterFa', [], 'Display', None],
  [ 0xa5f, 'UnicodeCharacter0xa5f', [], 'Display', None],
  [ 0xa60, 'UnicodeCharacter0xa60', [], 'Display', None],
  [ 0xa61, 'UnicodeCharacter0xa61', [], 'Display', None],
  [ 0xa62, 'UnicodeCharacter0xa62', [], 'Display', None],
  [ 0xa63, 'UnicodeCharacter0xa63', [], 'Display', None],
  [ 0xa64, 'UnicodeCharacter0xa64', [], 'Display', None],
  [ 0xa65, 'UnicodeCharacter0xa65', [], 'Display', None],
  [ 0xa66, 'GurmukhiDigitZero', [], 'Display', None],
  [ 0xa67, 'GurmukhiDigitOne', [], 'Display', None],
  [ 0xa68, 'GurmukhiDigitTwo', [], 'Display', None],
  [ 0xa69, 'GurmukhiDigitThree', [], 'Display', None],
  [ 0xa6a, 'GurmukhiDigitFour', [], 'Display', None],
  [ 0xa6b, 'GurmukhiDigitFive', [], 'Display', None],
  [ 0xa6c, 'GurmukhiDigitSix', [], 'Display', None],
  [ 0xa6d, 'GurmukhiDigitSeven', [], 'Display', None],
  [ 0xa6e, 'GurmukhiDigitEight', [], 'Display', None],
  [ 0xa6f, 'GurmukhiDigitNine', [], 'Display', None],
  [ 0xa70, 'GurmukhiTippi', [], 'Display', None],
  [ 0xa71, 'GurmukhiAddak', [], 'Display', None],
  [ 0xa72, 'GurmukhiIri', [], 'Display', None],
  [ 0xa73, 'GurmukhiUra', [], 'Display', None],
  [ 0xa74, 'GurmukhiEkOnkar', [], 'Display', None],
  [ 0xa75, 'GurmukhiSignYakash', [], 'Display', None],
  [ 0xa76, 'UnicodeCharacter0xa76', [], 'Display', None],
  [ 0xa77, 'UnicodeCharacter0xa77', [], 'Display', None],
  [ 0xa78, 'UnicodeCharacter0xa78', [], 'Display', None],
  [ 0xa79, 'UnicodeCharacter0xa79', [], 'Display', None],
  [ 0xa7a, 'UnicodeCharacter0xa7a', [], 'Display', None],
  [ 0xa7b, 'UnicodeCharacter0xa7b', [], 'Display', None],
  [ 0xa7c, 'UnicodeCharacter0xa7c', [], 'Display', None],
  [ 0xa7d, 'UnicodeCharacter0xa7d', [], 'Display', None],
  [ 0xa7e, 'UnicodeCharacter0xa7e', [], 'Display', None],
  [ 0xa7f, 'UnicodeCharacter0xa7f', [], 'Display', None],
  [ 0xa80, 'UnicodeCharacter0xa80', [], 'Display', None],
  [ 0xa81, 'GujaratiSignCandrabindu', [], 'Display', None],
  [ 0xa82, 'GujaratiSignAnusvara', [], 'Display', None],
  [ 0xa83, 'GujaratiSignVisarga', [], 'Display', None],
  [ 0xa84, 'UnicodeCharacter0xa84', [], 'Display', None],
  [ 0xa85, 'GujaratiLetterA', [], 'Display', None],
  [ 0xa86, 'GujaratiLetterAa', [], 'Display', None],
  [ 0xa87, 'GujaratiLetterI', [], 'Display', None],
  [ 0xa88, 'GujaratiLetterIi', [], 'Display', None],
  [ 0xa89, 'GujaratiLetterU', [], 'Display', None],
  [ 0xa8a, 'GujaratiLetterUu', [], 'Display', None],
  [ 0xa8b, 'GujaratiLetterVocalicR', [], 'Display', None],
  [ 0xa8c, 'GujaratiLetterVocalicL', [], 'Display', None],
  [ 0xa8d, 'GujaratiVowelCandraE', [], 'Display', None],
  [ 0xa8e, 'UnicodeCharacter0xa8e', [], 'Display', None],
  [ 0xa8f, 'GujaratiLetterE', [], 'Display', None],
  [ 0xa90, 'GujaratiLetterAi', [], 'Display', None],
  [ 0xa91, 'GujaratiVowelCandraO', [], 'Display', None],
  [ 0xa92, 'UnicodeCharacter0xa92', [], 'Display', None],
  [ 0xa93, 'GujaratiLetterO', [], 'Display', None],
  [ 0xa94, 'GujaratiLetterAu', [], 'Display', None],
  [ 0xa95, 'GujaratiLetterKa', [], 'Display', None],
  [ 0xa96, 'GujaratiLetterKha', [], 'Display', None],
  [ 0xa97, 'GujaratiLetterGa', [], 'Display', None],
  [ 0xa98, 'GujaratiLetterGha', [], 'Display', None],
  [ 0xa99, 'GujaratiLetterNga', [], 'Display', None],
  [ 0xa9a, 'GujaratiLetterCa', [], 'Display', None],
  [ 0xa9b, 'GujaratiLetterCha', [], 'Display', None],
  [ 0xa9c, 'GujaratiLetterJa', [], 'Display', None],
  [ 0xa9d, 'GujaratiLetterJha', [], 'Display', None],
  [ 0xa9e, 'GujaratiLetterNya', [], 'Display', None],
  [ 0xa9f, 'GujaratiLetterTta', [], 'Display', None],
  [ 0xaa0, 'GujaratiLetterTtha', [], 'Display', None],
  [ 0xaa1, 'GujaratiLetterDda', [], 'Display', None],
  [ 0xaa2, 'GujaratiLetterDdha', [], 'Display', None],
  [ 0xaa3, 'GujaratiLetterNna', [], 'Display', None],
  [ 0xaa4, 'GujaratiLetterTa', [], 'Display', None],
  [ 0xaa5, 'GujaratiLetterTha', [], 'Display', None],
  [ 0xaa6, 'GujaratiLetterDa', [], 'Display', None],
  [ 0xaa7, 'GujaratiLetterDha', [], 'Display', None],
  [ 0xaa8, 'GujaratiLetterNa', [], 'Display', None],
  [ 0xaa9, 'UnicodeCharacter0xaa9', [], 'Display', None],
  [ 0xaaa, 'GujaratiLetterPa', [], 'Display', None],
  [ 0xaab, 'GujaratiLetterPha', [], 'Display', None],
  [ 0xaac, 'GujaratiLetterBa', [], 'Display', None],
  [ 0xaad, 'GujaratiLetterBha', [], 'Display', None],
  [ 0xaae, 'GujaratiLetterMa', [], 'Display', None],
  [ 0xaaf, 'GujaratiLetterYa', [], 'Display', None],
  [ 0xab0, 'GujaratiLetterRa', [], 'Display', None],
  [ 0xab1, 'UnicodeCharacter0xab1', [], 'Display', None],
  [ 0xab2, 'GujaratiLetterLa', [], 'Display', None],
  [ 0xab3, 'GujaratiLetterLla', [], 'Display', None],
  [ 0xab4, 'UnicodeCharacter0xab4', [], 'Display', None],
  [ 0xab5, 'GujaratiLetterVa', [], 'Display', None],
  [ 0xab6, 'GujaratiLetterSha', [], 'Display', None],
  [ 0xab7, 'GujaratiLetterSsa', [], 'Display', None],
  [ 0xab8, 'GujaratiLetterSa', [], 'Display', None],
  [ 0xab9, 'GujaratiLetterHa', [], 'Display', None],
  [ 0xaba, 'UnicodeCharacter0xaba', [], 'Display', None],
  [ 0xabb, 'UnicodeCharacter0xabb', [], 'Display', None],
  [ 0xabc, 'GujaratiSignNukta', [], 'Display', None],
  [ 0xabd, 'GujaratiSignAvagraha', [], 'Display', None],
  [ 0xabe, 'GujaratiVowelSignAa', [], 'Display', None],
  [ 0xabf, 'GujaratiVowelSignI', [], 'Display', None],
  [ 0xac0, 'GujaratiVowelSignIi', [], 'Display', None],
  [ 0xac1, 'GujaratiVowelSignU', [], 'Display', None],
  [ 0xac2, 'GujaratiVowelSignUu', [], 'Display', None],
  [ 0xac3, 'GujaratiVowelSignVocalicR', [], 'Display', None],
  [ 0xac4, 'GujaratiVowelSignVocalicRr', [], 'Display', None],
  [ 0xac5, 'GujaratiVowelSignCandraE', [], 'Display', None],
  [ 0xac6, 'UnicodeCharacter0xac6', [], 'Display', None],
  [ 0xac7, 'GujaratiVowelSignE', [], 'Display', None],
  [ 0xac8, 'GujaratiVowelSignAi', [], 'Display', None],
  [ 0xac9, 'GujaratiVowelSignCandraO', [], 'Display', None],
  [ 0xaca, 'UnicodeCharacter0xaca', [], 'Display', None],
  [ 0xacb, 'GujaratiVowelSignO', [], 'Display', None],
  [ 0xacc, 'GujaratiVowelSignAu', [], 'Display', None],
  [ 0xacd, 'GujaratiSignVirama', [], 'Display', None],
  [ 0xace, 'UnicodeCharacter0xace', [], 'Display', None],
  [ 0xacf, 'UnicodeCharacter0xacf', [], 'Display', None],
  [ 0xad0, 'GujaratiOm', [], 'Display', None],
  [ 0xad1, 'UnicodeCharacter0xad1', [], 'Display', None],
  [ 0xad2, 'UnicodeCharacter0xad2', [], 'Display', None],
  [ 0xad3, 'UnicodeCharacter0xad3', [], 'Display', None],
  [ 0xad4, 'UnicodeCharacter0xad4', [], 'Display', None],
  [ 0xad5, 'UnicodeCharacter0xad5', [], 'Display', None],
  [ 0xad6, 'UnicodeCharacter0xad6', [], 'Display', None],
  [ 0xad7, 'UnicodeCharacter0xad7', [], 'Display', None],
  [ 0xad8, 'UnicodeCharacter0xad8', [], 'Display', None],
  [ 0xad9, 'UnicodeCharacter0xad9', [], 'Display', None],
  [ 0xada, 'UnicodeCharacter0xada', [], 'Display', None],
  [ 0xadb, 'UnicodeCharacter0xadb', [], 'Display', None],
  [ 0xadc, 'UnicodeCharacter0xadc', [], 'Display', None],
  [ 0xadd, 'UnicodeCharacter0xadd', [], 'Display', None],
  [ 0xade, 'UnicodeCharacter0xade', [], 'Display', None],
  [ 0xadf, 'UnicodeCharacter0xadf', [], 'Display', None],
  [ 0xae0, 'GujaratiLetterVocalicRr', [], 'Display', None],
  [ 0xae1, 'GujaratiLetterVocalicLl', [], 'Display', None],
  [ 0xae2, 'GujaratiVowelSignVocalicL', [], 'Display', None],
  [ 0xae3, 'GujaratiVowelSignVocalicLl', [], 'Display', None],
  [ 0xae4, 'UnicodeCharacter0xae4', [], 'Display', None],
  [ 0xae5, 'UnicodeCharacter0xae5', [], 'Display', None],
  [ 0xae6, 'GujaratiDigitZero', [], 'Display', None],
  [ 0xae7, 'GujaratiDigitOne', [], 'Display', None],
  [ 0xae8, 'GujaratiDigitTwo', [], 'Display', None],
  [ 0xae9, 'GujaratiDigitThree', [], 'Display', None],
  [ 0xaea, 'GujaratiDigitFour', [], 'Display', None],
  [ 0xaeb, 'GujaratiDigitFive', [], 'Display', None],
  [ 0xaec, 'GujaratiDigitSix', [], 'Display', None],
  [ 0xaed, 'GujaratiDigitSeven', [], 'Display', None],
  [ 0xaee, 'GujaratiDigitEight', [], 'Display', None],
  [ 0xaef, 'GujaratiDigitNine', [], 'Display', None],
  [ 0xaf0, 'UnicodeCharacter0xaf0', [], 'Display', None],
  [ 0xaf1, 'GujaratiRupeeSign', [], 'Display', None],
  [ 0xaf2, 'UnicodeCharacter0xaf2', [], 'Display', None],
  [ 0xaf3, 'UnicodeCharacter0xaf3', [], 'Display', None],
  [ 0xaf4, 'UnicodeCharacter0xaf4', [], 'Display', None],
  [ 0xaf5, 'UnicodeCharacter0xaf5', [], 'Display', None],
  [ 0xaf6, 'UnicodeCharacter0xaf6', [], 'Display', None],
  [ 0xaf7, 'UnicodeCharacter0xaf7', [], 'Display', None],
  [ 0xaf8, 'UnicodeCharacter0xaf8', [], 'Display', None],
  [ 0xaf9, 'UnicodeCharacter0xaf9', [], 'Display', None],
  [ 0xafa, 'UnicodeCharacter0xafa', [], 'Display', None],
  [ 0xafb, 'UnicodeCharacter0xafb', [], 'Display', None],
  [ 0xafc, 'UnicodeCharacter0xafc', [], 'Display', None],
  [ 0xafd, 'UnicodeCharacter0xafd', [], 'Display', None],
  [ 0xafe, 'UnicodeCharacter0xafe', [], 'Display', None],
  [ 0xaff, 'UnicodeCharacter0xaff', [], 'Display', None],
  [ 0xb00, 'UnicodeCharacter0xb00', [], 'Display', None],
  [ 0xb01, 'OriyaSignCandrabindu', [], 'Display', None],
  [ 0xb02, 'OriyaSignAnusvara', [], 'Display', None],
  [ 0xb03, 'OriyaSignVisarga', [], 'Display', None],
  [ 0xb04, 'UnicodeCharacter0xb04', [], 'Display', None],
  [ 0xb05, 'OriyaLetterA', [], 'Display', None],
  [ 0xb06, 'OriyaLetterAa', [], 'Display', None],
  [ 0xb07, 'OriyaLetterI', [], 'Display', None],
  [ 0xb08, 'OriyaLetterIi', [], 'Display', None],
  [ 0xb09, 'OriyaLetterU', [], 'Display', None],
  [ 0xb0a, 'OriyaLetterUu', [], 'Display', None],
  [ 0xb0b, 'OriyaLetterVocalicR', [], 'Display', None],
  [ 0xb0c, 'OriyaLetterVocalicL', [], 'Display', None],
  [ 0xb0d, 'UnicodeCharacter0xb0d', [], 'Display', None],
  [ 0xb0e, 'UnicodeCharacter0xb0e', [], 'Display', None],
  [ 0xb0f, 'OriyaLetterE', [], 'Display', None],
  [ 0xb10, 'OriyaLetterAi', [], 'Display', None],
  [ 0xb11, 'UnicodeCharacter0xb11', [], 'Display', None],
  [ 0xb12, 'UnicodeCharacter0xb12', [], 'Display', None],
  [ 0xb13, 'OriyaLetterO', [], 'Display', None],
  [ 0xb14, 'OriyaLetterAu', [], 'Display', None],
  [ 0xb15, 'OriyaLetterKa', [], 'Display', None],
  [ 0xb16, 'OriyaLetterKha', [], 'Display', None],
  [ 0xb17, 'OriyaLetterGa', [], 'Display', None],
  [ 0xb18, 'OriyaLetterGha', [], 'Display', None],
  [ 0xb19, 'OriyaLetterNga', [], 'Display', None],
  [ 0xb1a, 'OriyaLetterCa', [], 'Display', None],
  [ 0xb1b, 'OriyaLetterCha', [], 'Display', None],
  [ 0xb1c, 'OriyaLetterJa', [], 'Display', None],
  [ 0xb1d, 'OriyaLetterJha', [], 'Display', None],
  [ 0xb1e, 'OriyaLetterNya', [], 'Display', None],
  [ 0xb1f, 'OriyaLetterTta', [], 'Display', None],
  [ 0xb20, 'OriyaLetterTtha', [], 'Display', None],
  [ 0xb21, 'OriyaLetterDda', [], 'Display', None],
  [ 0xb22, 'OriyaLetterDdha', [], 'Display', None],
  [ 0xb23, 'OriyaLetterNna', [], 'Display', None],
  [ 0xb24, 'OriyaLetterTa', [], 'Display', None],
  [ 0xb25, 'OriyaLetterTha', [], 'Display', None],
  [ 0xb26, 'OriyaLetterDa', [], 'Display', None],
  [ 0xb27, 'OriyaLetterDha', [], 'Display', None],
  [ 0xb28, 'OriyaLetterNa', [], 'Display', None],
  [ 0xb29, 'UnicodeCharacter0xb29', [], 'Display', None],
  [ 0xb2a, 'OriyaLetterPa', [], 'Display', None],
  [ 0xb2b, 'OriyaLetterPha', [], 'Display', None],
  [ 0xb2c, 'OriyaLetterBa', [], 'Display', None],
  [ 0xb2d, 'OriyaLetterBha', [], 'Display', None],
  [ 0xb2e, 'OriyaLetterMa', [], 'Display', None],
  [ 0xb2f, 'OriyaLetterYa', [], 'Display', None],
  [ 0xb30, 'OriyaLetterRa', [], 'Display', None],
  [ 0xb31, 'UnicodeCharacter0xb31', [], 'Display', None],
  [ 0xb32, 'OriyaLetterLa', [], 'Display', None],
  [ 0xb33, 'OriyaLetterLla', [], 'Display', None],
  [ 0xb34, 'UnicodeCharacter0xb34', [], 'Display', None],
  [ 0xb35, 'OriyaLetterVa', [], 'Display', None],
  [ 0xb36, 'OriyaLetterSha', [], 'Display', None],
  [ 0xb37, 'OriyaLetterSsa', [], 'Display', None],
  [ 0xb38, 'OriyaLetterSa', [], 'Display', None],
  [ 0xb39, 'OriyaLetterHa', [], 'Display', None],
  [ 0xb3a, 'UnicodeCharacter0xb3a', [], 'Display', None],
  [ 0xb3b, 'UnicodeCharacter0xb3b', [], 'Display', None],
  [ 0xb3c, 'OriyaSignNukta', [], 'Display', None],
  [ 0xb3d, 'OriyaSignAvagraha', [], 'Display', None],
  [ 0xb3e, 'OriyaVowelSignAa', [], 'Display', None],
  [ 0xb3f, 'OriyaVowelSignI', [], 'Display', None],
  [ 0xb40, 'OriyaVowelSignIi', [], 'Display', None],
  [ 0xb41, 'OriyaVowelSignU', [], 'Display', None],
  [ 0xb42, 'OriyaVowelSignUu', [], 'Display', None],
  [ 0xb43, 'OriyaVowelSignVocalicR', [], 'Display', None],
  [ 0xb44, 'OriyaVowelSignVocalicRr', [], 'Display', None],
  [ 0xb45, 'UnicodeCharacter0xb45', [], 'Display', None],
  [ 0xb46, 'UnicodeCharacter0xb46', [], 'Display', None],
  [ 0xb47, 'OriyaVowelSignE', [], 'Display', None],
  [ 0xb48, 'OriyaVowelSignAi', [], 'Display', None],
  [ 0xb49, 'UnicodeCharacter0xb49', [], 'Display', None],
  [ 0xb4a, 'UnicodeCharacter0xb4a', [], 'Display', None],
  [ 0xb4b, 'OriyaVowelSignO', [], 'Display', None],
  [ 0xb4c, 'OriyaVowelSignAu', [], 'Display', None],
  [ 0xb4d, 'OriyaSignVirama', [], 'Display', None],
  [ 0xb4e, 'UnicodeCharacter0xb4e', [], 'Display', None],
  [ 0xb4f, 'UnicodeCharacter0xb4f', [], 'Display', None],
  [ 0xb50, 'UnicodeCharacter0xb50', [], 'Display', None],
  [ 0xb51, 'UnicodeCharacter0xb51', [], 'Display', None],
  [ 0xb52, 'UnicodeCharacter0xb52', [], 'Display', None],
  [ 0xb53, 'UnicodeCharacter0xb53', [], 'Display', None],
  [ 0xb54, 'UnicodeCharacter0xb54', [], 'Display', None],
  [ 0xb55, 'UnicodeCharacter0xb55', [], 'Display', None],
  [ 0xb56, 'OriyaAiLengthMark', [], 'Display', None],
  [ 0xb57, 'OriyaAuLengthMark', [], 'Display', None],
  [ 0xb58, 'UnicodeCharacter0xb58', [], 'Display', None],
  [ 0xb59, 'UnicodeCharacter0xb59', [], 'Display', None],
  [ 0xb5a, 'UnicodeCharacter0xb5a', [], 'Display', None],
  [ 0xb5b, 'UnicodeCharacter0xb5b', [], 'Display', None],
  [ 0xb5c, 'OriyaLetterRra', [], 'Display', None],
  [ 0xb5d, 'OriyaLetterRha', [], 'Display', None],
  [ 0xb5e, 'UnicodeCharacter0xb5e', [], 'Display', None],
  [ 0xb5f, 'OriyaLetterYya', [], 'Display', None],
  [ 0xb60, 'OriyaLetterVocalicRr', [], 'Display', None],
  [ 0xb61, 'OriyaLetterVocalicLl', [], 'Display', None],
  [ 0xb62, 'OriyaVowelSignVocalicL', [], 'Display', None],
  [ 0xb63, 'OriyaVowelSignVocalicLl', [], 'Display', None],
  [ 0xb64, 'UnicodeCharacter0xb64', [], 'Display', None],
  [ 0xb65, 'UnicodeCharacter0xb65', [], 'Display', None],
  [ 0xb66, 'OriyaDigitZero', [], 'Display', None],
  [ 0xb67, 'OriyaDigitOne', [], 'Display', None],
  [ 0xb68, 'OriyaDigitTwo', [], 'Display', None],
  [ 0xb69, 'OriyaDigitThree', [], 'Display', None],
  [ 0xb6a, 'OriyaDigitFour', [], 'Display', None],
  [ 0xb6b, 'OriyaDigitFive', [], 'Display', None],
  [ 0xb6c, 'OriyaDigitSix', [], 'Display', None],
  [ 0xb6d, 'OriyaDigitSeven', [], 'Display', None],
  [ 0xb6e, 'OriyaDigitEight', [], 'Display', None],
  [ 0xb6f, 'OriyaDigitNine', [], 'Display', None],
  [ 0xb70, 'OriyaIsshar', [], 'Display', None],
  [ 0xb71, 'OriyaLetterWa', [], 'Display', None],
  [ 0xb72, 'UnicodeCharacter0xb72', [], 'Display', None],
  [ 0xb73, 'UnicodeCharacter0xb73', [], 'Display', None],
  [ 0xb74, 'UnicodeCharacter0xb74', [], 'Display', None],
  [ 0xb75, 'UnicodeCharacter0xb75', [], 'Display', None],
  [ 0xb76, 'UnicodeCharacter0xb76', [], 'Display', None],
  [ 0xb77, 'UnicodeCharacter0xb77', [], 'Display', None],
  [ 0xb78, 'UnicodeCharacter0xb78', [], 'Display', None],
  [ 0xb79, 'UnicodeCharacter0xb79', [], 'Display', None],
  [ 0xb7a, 'UnicodeCharacter0xb7a', [], 'Display', None],
  [ 0xb7b, 'UnicodeCharacter0xb7b', [], 'Display', None],
  [ 0xb7c, 'UnicodeCharacter0xb7c', [], 'Display', None],
  [ 0xb7d, 'UnicodeCharacter0xb7d', [], 'Display', None],
  [ 0xb7e, 'UnicodeCharacter0xb7e', [], 'Display', None],
  [ 0xb7f, 'UnicodeCharacter0xb7f', [], 'Display', None],
  [ 0xb80, 'UnicodeCharacter0xb80', [], 'Display', None],
  [ 0xb81, 'UnicodeCharacter0xb81', [], 'Display', None],
  [ 0xb82, 'TamilSignAnusvara', [], 'Display', None],
  [ 0xb83, 'TamilSignVisarga', [], 'Display', None],
  [ 0xb84, 'UnicodeCharacter0xb84', [], 'Display', None],
  [ 0xb85, 'TamilLetterA', [], 'Display', None],
  [ 0xb86, 'TamilLetterAa', [], 'Display', None],
  [ 0xb87, 'TamilLetterI', [], 'Display', None],
  [ 0xb88, 'TamilLetterIi', [], 'Display', None],
  [ 0xb89, 'TamilLetterU', [], 'Display', None],
  [ 0xb8a, 'TamilLetterUu', [], 'Display', None],
  [ 0xb8b, 'UnicodeCharacter0xb8b', [], 'Display', None],
  [ 0xb8c, 'UnicodeCharacter0xb8c', [], 'Display', None],
  [ 0xb8d, 'UnicodeCharacter0xb8d', [], 'Display', None],
  [ 0xb8e, 'TamilLetterE', [], 'Display', None],
  [ 0xb8f, 'TamilLetterEe', [], 'Display', None],
  [ 0xb90, 'TamilLetterAi', [], 'Display', None],
  [ 0xb91, 'UnicodeCharacter0xb91', [], 'Display', None],
  [ 0xb92, 'TamilLetterO', [], 'Display', None],
  [ 0xb93, 'TamilLetterOo', [], 'Display', None],
  [ 0xb94, 'TamilLetterAu', [], 'Display', None],
  [ 0xb95, 'TamilLetterKa', [], 'Display', None],
  [ 0xb96, 'UnicodeCharacter0xb96', [], 'Display', None],
  [ 0xb97, 'UnicodeCharacter0xb97', [], 'Display', None],
  [ 0xb98, 'UnicodeCharacter0xb98', [], 'Display', None],
  [ 0xb99, 'TamilLetterNga', [], 'Display', None],
  [ 0xb9a, 'TamilLetterCa', [], 'Display', None],
  [ 0xb9b, 'UnicodeCharacter0xb9b', [], 'Display', None],
  [ 0xb9c, 'TamilLetterJa', [], 'Display', None],
  [ 0xb9d, 'UnicodeCharacter0xb9d', [], 'Display', None],
  [ 0xb9e, 'TamilLetterNya', [], 'Display', None],
  [ 0xb9f, 'TamilLetterTta', [], 'Display', None],
  [ 0xba0, 'UnicodeCharacter0xba0', [], 'Display', None],
  [ 0xba1, 'UnicodeCharacter0xba1', [], 'Display', None],
  [ 0xba2, 'UnicodeCharacter0xba2', [], 'Display', None],
  [ 0xba3, 'TamilLetterNna', [], 'Display', None],
  [ 0xba4, 'TamilLetterTa', [], 'Display', None],
  [ 0xba5, 'UnicodeCharacter0xba5', [], 'Display', None],
  [ 0xba6, 'UnicodeCharacter0xba6', [], 'Display', None],
  [ 0xba7, 'UnicodeCharacter0xba7', [], 'Display', None],
  [ 0xba8, 'TamilLetterNa', [], 'Display', None],
  [ 0xba9, 'TamilLetterNnna', [], 'Display', None],
  [ 0xbaa, 'TamilLetterPa', [], 'Display', None],
  [ 0xbab, 'UnicodeCharacter0xbab', [], 'Display', None],
  [ 0xbac, 'UnicodeCharacter0xbac', [], 'Display', None],
  [ 0xbad, 'UnicodeCharacter0xbad', [], 'Display', None],
  [ 0xbae, 'TamilLetterMa', [], 'Display', None],
  [ 0xbaf, 'TamilLetterYa', [], 'Display', None],
  [ 0xbb0, 'TamilLetterRa', [], 'Display', None],
  [ 0xbb1, 'TamilLetterRra', [], 'Display', None],
  [ 0xbb2, 'TamilLetterLa', [], 'Display', None],
  [ 0xbb3, 'TamilLetterLla', [], 'Display', None],
  [ 0xbb4, 'TamilLetterLlla', [], 'Display', None],
  [ 0xbb5, 'TamilLetterVa', [], 'Display', None],
  [ 0xbb6, 'TamilLetterSha', [], 'Display', None],
  [ 0xbb7, 'TamilLetterSsa', [], 'Display', None],
  [ 0xbb8, 'TamilLetterSa', [], 'Display', None],
  [ 0xbb9, 'TamilLetterHa', [], 'Display', None],
  [ 0xbba, 'UnicodeCharacter0xbba', [], 'Display', None],
  [ 0xbbb, 'UnicodeCharacter0xbbb', [], 'Display', None],
  [ 0xbbc, 'UnicodeCharacter0xbbc', [], 'Display', None],
  [ 0xbbd, 'UnicodeCharacter0xbbd', [], 'Display', None],
  [ 0xbbe, 'TamilVowelSignAa', [], 'Display', None],
  [ 0xbbf, 'TamilVowelSignI', [], 'Display', None],
  [ 0xbc0, 'TamilVowelSignIi', [], 'Display', None],
  [ 0xbc1, 'TamilVowelSignU', [], 'Display', None],
  [ 0xbc2, 'TamilVowelSignUu', [], 'Display', None],
  [ 0xbc3, 'UnicodeCharacter0xbc3', [], 'Display', None],
  [ 0xbc4, 'UnicodeCharacter0xbc4', [], 'Display', None],
  [ 0xbc5, 'UnicodeCharacter0xbc5', [], 'Display', None],
  [ 0xbc6, 'TamilVowelSignE', [], 'Display', None],
  [ 0xbc7, 'TamilVowelSignEe', [], 'Display', None],
  [ 0xbc8, 'TamilVowelSignAi', [], 'Display', None],
  [ 0xbc9, 'UnicodeCharacter0xbc9', [], 'Display', None],
  [ 0xbca, 'TamilVowelSignO', [], 'Display', None],
  [ 0xbcb, 'TamilVowelSignOo', [], 'Display', None],
  [ 0xbcc, 'TamilVowelSignAu', [], 'Display', None],
  [ 0xbcd, 'TamilSignVirama', [], 'Display', None],
  [ 0xbce, 'UnicodeCharacter0xbce', [], 'Display', None],
  [ 0xbcf, 'UnicodeCharacter0xbcf', [], 'Display', None],
  [ 0xbd0, 'TamilOm', [], 'Display', None],
  [ 0xbd1, 'UnicodeCharacter0xbd1', [], 'Display', None],
  [ 0xbd2, 'UnicodeCharacter0xbd2', [], 'Display', None],
  [ 0xbd3, 'UnicodeCharacter0xbd3', [], 'Display', None],
  [ 0xbd4, 'UnicodeCharacter0xbd4', [], 'Display', None],
  [ 0xbd5, 'UnicodeCharacter0xbd5', [], 'Display', None],
  [ 0xbd6, 'UnicodeCharacter0xbd6', [], 'Display', None],
  [ 0xbd7, 'TamilAuLengthMark', [], 'Display', None],
  [ 0xbd8, 'UnicodeCharacter0xbd8', [], 'Display', None],
  [ 0xbd9, 'UnicodeCharacter0xbd9', [], 'Display', None],
  [ 0xbda, 'UnicodeCharacter0xbda', [], 'Display', None],
  [ 0xbdb, 'UnicodeCharacter0xbdb', [], 'Display', None],
  [ 0xbdc, 'UnicodeCharacter0xbdc', [], 'Display', None],
  [ 0xbdd, 'UnicodeCharacter0xbdd', [], 'Display', None],
  [ 0xbde, 'UnicodeCharacter0xbde', [], 'Display', None],
  [ 0xbdf, 'UnicodeCharacter0xbdf', [], 'Display', None],
  [ 0xbe0, 'UnicodeCharacter0xbe0', [], 'Display', None],
  [ 0xbe1, 'UnicodeCharacter0xbe1', [], 'Display', None],
  [ 0xbe2, 'UnicodeCharacter0xbe2', [], 'Display', None],
  [ 0xbe3, 'UnicodeCharacter0xbe3', [], 'Display', None],
  [ 0xbe4, 'UnicodeCharacter0xbe4', [], 'Display', None],
  [ 0xbe5, 'UnicodeCharacter0xbe5', [], 'Display', None],
  [ 0xbe6, 'TamilDigitZero', [], 'Display', None],
  [ 0xbe7, 'TamilDigitOne', [], 'Display', None],
  [ 0xbe8, 'TamilDigitTwo', [], 'Display', None],
  [ 0xbe9, 'TamilDigitThree', [], 'Display', None],
  [ 0xbea, 'TamilDigitFour', [], 'Display', None],
  [ 0xbeb, 'TamilDigitFive', [], 'Display', None],
  [ 0xbec, 'TamilDigitSix', [], 'Display', None],
  [ 0xbed, 'TamilDigitSeven', [], 'Display', None],
  [ 0xbee, 'TamilDigitEight', [], 'Display', None],
  [ 0xbef, 'TamilDigitNine', [], 'Display', None],
  [ 0xbf0, 'TamilNumberTen', [], 'Display', None],
  [ 0xbf1, 'TamilNumberOneHundred', [], 'Display', None],
  [ 0xbf2, 'TamilNumberOneThousand', [], 'Display', None],
  [ 0xbf3, 'TamilDaySign', [], 'Display', None],
  [ 0xbf4, 'TamilMonthSign', [], 'Display', None],
  [ 0xbf5, 'TamilYearSign', [], 'Display', None],
  [ 0xbf6, 'TamilDebitSign', [], 'Display', None],
  [ 0xbf7, 'TamilCreditSign', [], 'Display', None],
  [ 0xbf8, 'TamilAsAboveSign', [], 'Display', None],
  [ 0xbf9, 'TamilRupeeSign', [], 'Display', None],
  [ 0xbfa, 'TamilNumberSign', [], 'Display', None],
  [ 0xbfb, 'UnicodeCharacter0xbfb', [], 'Display', None],
  [ 0xbfc, 'UnicodeCharacter0xbfc', [], 'Display', None],
  [ 0xbfd, 'UnicodeCharacter0xbfd', [], 'Display', None],
  [ 0xbfe, 'UnicodeCharacter0xbfe', [], 'Display', None],
  [ 0xbff, 'UnicodeCharacter0xbff', [], 'Display', None],
  [ 0xc00, 'UnicodeCharacter0xc00', [], 'Display', None],
  [ 0xc01, 'TeluguSignCandrabindu', [], 'Display', None],
  [ 0xc02, 'TeluguSignAnusvara', [], 'Display', None],
  [ 0xc03, 'TeluguSignVisarga', [], 'Display', None],
  [ 0xc04, 'UnicodeCharacter0xc04', [], 'Display', None],
  [ 0xc05, 'TeluguLetterA', [], 'Display', None],
  [ 0xc06, 'TeluguLetterAa', [], 'Display', None],
  [ 0xc07, 'TeluguLetterI', [], 'Display', None],
  [ 0xc08, 'TeluguLetterIi', [], 'Display', None],
  [ 0xc09, 'TeluguLetterU', [], 'Display', None],
  [ 0xc0a, 'TeluguLetterUu', [], 'Display', None],
  [ 0xc0b, 'TeluguLetterVocalicR', [], 'Display', None],
  [ 0xc0c, 'TeluguLetterVocalicL', [], 'Display', None],
  [ 0xc0d, 'UnicodeCharacter0xc0d', [], 'Display', None],
  [ 0xc0e, 'TeluguLetterE', [], 'Display', None],
  [ 0xc0f, 'TeluguLetterEe', [], 'Display', None],
  [ 0xc10, 'TeluguLetterAi', [], 'Display', None],
  [ 0xc11, 'UnicodeCharacter0xc11', [], 'Display', None],
  [ 0xc12, 'TeluguLetterO', [], 'Display', None],
  [ 0xc13, 'TeluguLetterOo', [], 'Display', None],
  [ 0xc14, 'TeluguLetterAu', [], 'Display', None],
  [ 0xc15, 'TeluguLetterKa', [], 'Display', None],
  [ 0xc16, 'TeluguLetterKha', [], 'Display', None],
  [ 0xc17, 'TeluguLetterGa', [], 'Display', None],
  [ 0xc18, 'TeluguLetterGha', [], 'Display', None],
  [ 0xc19, 'TeluguLetterNga', [], 'Display', None],
  [ 0xc1a, 'TeluguLetterCa', [], 'Display', None],
  [ 0xc1b, 'TeluguLetterCha', [], 'Display', None],
  [ 0xc1c, 'TeluguLetterJa', [], 'Display', None],
  [ 0xc1d, 'TeluguLetterJha', [], 'Display', None],
  [ 0xc1e, 'TeluguLetterNya', [], 'Display', None],
  [ 0xc1f, 'TeluguLetterTta', [], 'Display', None],
  [ 0xc20, 'TeluguLetterTtha', [], 'Display', None],
  [ 0xc21, 'TeluguLetterDda', [], 'Display', None],
  [ 0xc22, 'TeluguLetterDdha', [], 'Display', None],
  [ 0xc23, 'TeluguLetterNna', [], 'Display', None],
  [ 0xc24, 'TeluguLetterTa', [], 'Display', None],
  [ 0xc25, 'TeluguLetterTha', [], 'Display', None],
  [ 0xc26, 'TeluguLetterDa', [], 'Display', None],
  [ 0xc27, 'TeluguLetterDha', [], 'Display', None],
  [ 0xc28, 'TeluguLetterNa', [], 'Display', None],
  [ 0xc29, 'UnicodeCharacter0xc29', [], 'Display', None],
  [ 0xc2a, 'TeluguLetterPa', [], 'Display', None],
  [ 0xc2b, 'TeluguLetterPha', [], 'Display', None],
  [ 0xc2c, 'TeluguLetterBa', [], 'Display', None],
  [ 0xc2d, 'TeluguLetterBha', [], 'Display', None],
  [ 0xc2e, 'TeluguLetterMa', [], 'Display', None],
  [ 0xc2f, 'TeluguLetterYa', [], 'Display', None],
  [ 0xc30, 'TeluguLetterRa', [], 'Display', None],
  [ 0xc31, 'TeluguLetterRra', [], 'Display', None],
  [ 0xc32, 'TeluguLetterLa', [], 'Display', None],
  [ 0xc33, 'TeluguLetterLla', [], 'Display', None],
  [ 0xc34, 'UnicodeCharacter0xc34', [], 'Display', None],
  [ 0xc35, 'TeluguLetterVa', [], 'Display', None],
  [ 0xc36, 'TeluguLetterSha', [], 'Display', None],
  [ 0xc37, 'TeluguLetterSsa', [], 'Display', None],
  [ 0xc38, 'TeluguLetterSa', [], 'Display', None],
  [ 0xc39, 'TeluguLetterHa', [], 'Display', None],
  [ 0xc3a, 'UnicodeCharacter0xc3a', [], 'Display', None],
  [ 0xc3b, 'UnicodeCharacter0xc3b', [], 'Display', None],
  [ 0xc3c, 'UnicodeCharacter0xc3c', [], 'Display', None],
  [ 0xc3d, 'TeluguSignAvagraha', [], 'Display', None],
  [ 0xc3e, 'TeluguVowelSignAa', [], 'Display', None],
  [ 0xc3f, 'TeluguVowelSignI', [], 'Display', None],
  [ 0xc40, 'TeluguVowelSignIi', [], 'Display', None],
  [ 0xc41, 'TeluguVowelSignU', [], 'Display', None],
  [ 0xc42, 'TeluguVowelSignUu', [], 'Display', None],
  [ 0xc43, 'TeluguVowelSignVocalicR', [], 'Display', None],
  [ 0xc44, 'TeluguVowelSignVocalicRr', [], 'Display', None],
  [ 0xc45, 'UnicodeCharacter0xc45', [], 'Display', None],
  [ 0xc46, 'TeluguVowelSignE', [], 'Display', None],
  [ 0xc47, 'TeluguVowelSignEe', [], 'Display', None],
  [ 0xc48, 'TeluguVowelSignAi', [], 'Display', None],
  [ 0xc49, 'UnicodeCharacter0xc49', [], 'Display', None],
  [ 0xc4a, 'TeluguVowelSignO', [], 'Display', None],
  [ 0xc4b, 'TeluguVowelSignOo', [], 'Display', None],
  [ 0xc4c, 'TeluguVowelSignAu', [], 'Display', None],
  [ 0xc4d, 'TeluguSignVirama', [], 'Display', None],
  [ 0xc4e, 'UnicodeCharacter0xc4e', [], 'Display', None],
  [ 0xc4f, 'UnicodeCharacter0xc4f', [], 'Display', None],
  [ 0xc50, 'UnicodeCharacter0xc50', [], 'Display', None],
  [ 0xc51, 'UnicodeCharacter0xc51', [], 'Display', None],
  [ 0xc52, 'UnicodeCharacter0xc52', [], 'Display', None],
  [ 0xc53, 'UnicodeCharacter0xc53', [], 'Display', None],
  [ 0xc54, 'UnicodeCharacter0xc54', [], 'Display', None],
  [ 0xc55, 'TeluguLengthMark', [], 'Display', None],
  [ 0xc56, 'TeluguAiLengthMark', [], 'Display', None],
  [ 0xc57, 'UnicodeCharacter0xc57', [], 'Display', None],
  [ 0xc58, 'TeluguLetterTsa', [], 'Display', None],
  [ 0xc59, 'TeluguLetterDza', [], 'Display', None],
  [ 0xc5a, 'UnicodeCharacter0xc5a', [], 'Display', None],
  [ 0xc5b, 'UnicodeCharacter0xc5b', [], 'Display', None],
  [ 0xc5c, 'UnicodeCharacter0xc5c', [], 'Display', None],
  [ 0xc5d, 'UnicodeCharacter0xc5d', [], 'Display', None],
  [ 0xc5e, 'UnicodeCharacter0xc5e', [], 'Display', None],
  [ 0xc5f, 'UnicodeCharacter0xc5f', [], 'Display', None],
  [ 0xc60, 'TeluguLetterVocalicRr', [], 'Display', None],
  [ 0xc61, 'TeluguLetterVocalicLl', [], 'Display', None],
  [ 0xc62, 'TeluguVowelSignVocalicL', [], 'Display', None],
  [ 0xc63, 'TeluguVowelSignVocalicLl', [], 'Display', None],
  [ 0xc64, 'UnicodeCharacter0xc64', [], 'Display', None],
  [ 0xc65, 'UnicodeCharacter0xc65', [], 'Display', None],
  [ 0xc66, 'TeluguDigitZero', [], 'Display', None],
  [ 0xc67, 'TeluguDigitOne', [], 'Display', None],
  [ 0xc68, 'TeluguDigitTwo', [], 'Display', None],
  [ 0xc69, 'TeluguDigitThree', [], 'Display', None],
  [ 0xc6a, 'TeluguDigitFour', [], 'Display', None],
  [ 0xc6b, 'TeluguDigitFive', [], 'Display', None],
  [ 0xc6c, 'TeluguDigitSix', [], 'Display', None],
  [ 0xc6d, 'TeluguDigitSeven', [], 'Display', None],
  [ 0xc6e, 'TeluguDigitEight', [], 'Display', None],
  [ 0xc6f, 'TeluguDigitNine', [], 'Display', None],
  [ 0xc70, 'UnicodeCharacter0xc70', [], 'Display', None],
  [ 0xc71, 'UnicodeCharacter0xc71', [], 'Display', None],
  [ 0xc72, 'UnicodeCharacter0xc72', [], 'Display', None],
  [ 0xc73, 'UnicodeCharacter0xc73', [], 'Display', None],
  [ 0xc74, 'UnicodeCharacter0xc74', [], 'Display', None],
  [ 0xc75, 'UnicodeCharacter0xc75', [], 'Display', None],
  [ 0xc76, 'UnicodeCharacter0xc76', [], 'Display', None],
  [ 0xc77, 'UnicodeCharacter0xc77', [], 'Display', None],
  [ 0xc78, 'TeluguFractionDigitZeroForOddPowersOfFour', [], 'Display', None],
  [ 0xc79, 'TeluguFractionDigitOneForOddPowersOfFour', [], 'Display', None],
  [ 0xc7a, 'TeluguFractionDigitTwoForOddPowersOfFour', [], 'Display', None],
  [ 0xc7b, 'TeluguFractionDigitThreeForOddPowersOfFour', [], 'Display', None],
  [ 0xc7c, 'TeluguFractionDigitOneForEvenPowersOfFour', [], 'Display', None],
  [ 0xc7d, 'TeluguFractionDigitTwoForEvenPowersOfFour', [], 'Display', None],
  [ 0xc7e, 'TeluguFractionDigitThreeForEvenPowersOfFour', [], 'Display', None],
  [ 0xc7f, 'TeluguSignTuumu', [], 'Display', None],
  [ 0xc80, 'UnicodeCharacter0xc80', [], 'Display', None],
  [ 0xc81, 'UnicodeCharacter0xc81', [], 'Display', None],
  [ 0xc82, 'KannadaSignAnusvara', [], 'Display', None],
  [ 0xc83, 'KannadaSignVisarga', [], 'Display', None],
  [ 0xc84, 'UnicodeCharacter0xc84', [], 'Display', None],
  [ 0xc85, 'KannadaLetterA', [], 'Display', None],
  [ 0xc86, 'KannadaLetterAa', [], 'Display', None],
  [ 0xc87, 'KannadaLetterI', [], 'Display', None],
  [ 0xc88, 'KannadaLetterIi', [], 'Display', None],
  [ 0xc89, 'KannadaLetterU', [], 'Display', None],
  [ 0xc8a, 'KannadaLetterUu', [], 'Display', None],
  [ 0xc8b, 'KannadaLetterVocalicR', [], 'Display', None],
  [ 0xc8c, 'KannadaLetterVocalicL', [], 'Display', None],
  [ 0xc8d, 'UnicodeCharacter0xc8d', [], 'Display', None],
  [ 0xc8e, 'KannadaLetterE', [], 'Display', None],
  [ 0xc8f, 'KannadaLetterEe', [], 'Display', None],
  [ 0xc90, 'KannadaLetterAi', [], 'Display', None],
  [ 0xc91, 'UnicodeCharacter0xc91', [], 'Display', None],
  [ 0xc92, 'KannadaLetterO', [], 'Display', None],
  [ 0xc93, 'KannadaLetterOo', [], 'Display', None],
  [ 0xc94, 'KannadaLetterAu', [], 'Display', None],
  [ 0xc95, 'KannadaLetterKa', [], 'Display', None],
  [ 0xc96, 'KannadaLetterKha', [], 'Display', None],
  [ 0xc97, 'KannadaLetterGa', [], 'Display', None],
  [ 0xc98, 'KannadaLetterGha', [], 'Display', None],
  [ 0xc99, 'KannadaLetterNga', [], 'Display', None],
  [ 0xc9a, 'KannadaLetterCa', [], 'Display', None],
  [ 0xc9b, 'KannadaLetterCha', [], 'Display', None],
  [ 0xc9c, 'KannadaLetterJa', [], 'Display', None],
  [ 0xc9d, 'KannadaLetterJha', [], 'Display', None],
  [ 0xc9e, 'KannadaLetterNya', [], 'Display', None],
  [ 0xc9f, 'KannadaLetterTta', [], 'Display', None],
  [ 0xca0, 'KannadaLetterTtha', [], 'Display', None],
  [ 0xca1, 'KannadaLetterDda', [], 'Display', None],
  [ 0xca2, 'KannadaLetterDdha', [], 'Display', None],
  [ 0xca3, 'KannadaLetterNna', [], 'Display', None],
  [ 0xca4, 'KannadaLetterTa', [], 'Display', None],
  [ 0xca5, 'KannadaLetterTha', [], 'Display', None],
  [ 0xca6, 'KannadaLetterDa', [], 'Display', None],
  [ 0xca7, 'KannadaLetterDha', [], 'Display', None],
  [ 0xca8, 'KannadaLetterNa', [], 'Display', None],
  [ 0xca9, 'UnicodeCharacter0xca9', [], 'Display', None],
  [ 0xcaa, 'KannadaLetterPa', [], 'Display', None],
  [ 0xcab, 'KannadaLetterPha', [], 'Display', None],
  [ 0xcac, 'KannadaLetterBa', [], 'Display', None],
  [ 0xcad, 'KannadaLetterBha', [], 'Display', None],
  [ 0xcae, 'KannadaLetterMa', [], 'Display', None],
  [ 0xcaf, 'KannadaLetterYa', [], 'Display', None],
  [ 0xcb0, 'KannadaLetterRa', [], 'Display', None],
  [ 0xcb1, 'KannadaLetterRra', [], 'Display', None],
  [ 0xcb2, 'KannadaLetterLa', [], 'Display', None],
  [ 0xcb3, 'KannadaLetterLla', [], 'Display', None],
  [ 0xcb4, 'UnicodeCharacter0xcb4', [], 'Display', None],
  [ 0xcb5, 'KannadaLetterVa', [], 'Display', None],
  [ 0xcb6, 'KannadaLetterSha', [], 'Display', None],
  [ 0xcb7, 'KannadaLetterSsa', [], 'Display', None],
  [ 0xcb8, 'KannadaLetterSa', [], 'Display', None],
  [ 0xcb9, 'KannadaLetterHa', [], 'Display', None],
  [ 0xcba, 'UnicodeCharacter0xcba', [], 'Display', None],
  [ 0xcbb, 'UnicodeCharacter0xcbb', [], 'Display', None],
  [ 0xcbc, 'KannadaSignNukta', [], 'Display', None],
  [ 0xcbd, 'KannadaSignAvagraha', [], 'Display', None],
  [ 0xcbe, 'KannadaVowelSignAa', [], 'Display', None],
  [ 0xcbf, 'KannadaVowelSignI', [], 'Display', None],
  [ 0xcc0, 'KannadaVowelSignIi', [], 'Display', None],
  [ 0xcc1, 'KannadaVowelSignU', [], 'Display', None],
  [ 0xcc2, 'KannadaVowelSignUu', [], 'Display', None],
  [ 0xcc3, 'KannadaVowelSignVocalicR', [], 'Display', None],
  [ 0xcc4, 'KannadaVowelSignVocalicRr', [], 'Display', None],
  [ 0xcc5, 'UnicodeCharacter0xcc5', [], 'Display', None],
  [ 0xcc6, 'KannadaVowelSignE', [], 'Display', None],
  [ 0xcc7, 'KannadaVowelSignEe', [], 'Display', None],
  [ 0xcc8, 'KannadaVowelSignAi', [], 'Display', None],
  [ 0xcc9, 'UnicodeCharacter0xcc9', [], 'Display', None],
  [ 0xcca, 'KannadaVowelSignO', [], 'Display', None],
  [ 0xccb, 'KannadaVowelSignOo', [], 'Display', None],
  [ 0xccc, 'KannadaVowelSignAu', [], 'Display', None],
  [ 0xccd, 'KannadaSignVirama', [], 'Display', None],
  [ 0xcce, 'UnicodeCharacter0xcce', [], 'Display', None],
  [ 0xccf, 'UnicodeCharacter0xccf', [], 'Display', None],
  [ 0xcd0, 'UnicodeCharacter0xcd0', [], 'Display', None],
  [ 0xcd1, 'UnicodeCharacter0xcd1', [], 'Display', None],
  [ 0xcd2, 'UnicodeCharacter0xcd2', [], 'Display', None],
  [ 0xcd3, 'UnicodeCharacter0xcd3', [], 'Display', None],
  [ 0xcd4, 'UnicodeCharacter0xcd4', [], 'Display', None],
  [ 0xcd5, 'KannadaLengthMark', [], 'Display', None],
  [ 0xcd6, 'KannadaAiLengthMark', [], 'Display', None],
  [ 0xcd7, 'UnicodeCharacter0xcd7', [], 'Display', None],
  [ 0xcd8, 'UnicodeCharacter0xcd8', [], 'Display', None],
  [ 0xcd9, 'UnicodeCharacter0xcd9', [], 'Display', None],
  [ 0xcda, 'UnicodeCharacter0xcda', [], 'Display', None],
  [ 0xcdb, 'UnicodeCharacter0xcdb', [], 'Display', None],
  [ 0xcdc, 'UnicodeCharacter0xcdc', [], 'Display', None],
  [ 0xcdd, 'UnicodeCharacter0xcdd', [], 'Display', None],
  [ 0xcde, 'KannadaLetterFa', [], 'Display', None],
  [ 0xcdf, 'UnicodeCharacter0xcdf', [], 'Display', None],
  [ 0xce0, 'KannadaLetterVocalicRr', [], 'Display', None],
  [ 0xce1, 'KannadaLetterVocalicLl', [], 'Display', None],
  [ 0xce2, 'KannadaVowelSignVocalicL', [], 'Display', None],
  [ 0xce3, 'KannadaVowelSignVocalicLl', [], 'Display', None],
  [ 0xce4, 'UnicodeCharacter0xce4', [], 'Display', None],
  [ 0xce5, 'UnicodeCharacter0xce5', [], 'Display', None],
  [ 0xce6, 'KannadaDigitZero', [], 'Display', None],
  [ 0xce7, 'KannadaDigitOne', [], 'Display', None],
  [ 0xce8, 'KannadaDigitTwo', [], 'Display', None],
  [ 0xce9, 'KannadaDigitThree', [], 'Display', None],
  [ 0xcea, 'KannadaDigitFour', [], 'Display', None],
  [ 0xceb, 'KannadaDigitFive', [], 'Display', None],
  [ 0xcec, 'KannadaDigitSix', [], 'Display', None],
  [ 0xced, 'KannadaDigitSeven', [], 'Display', None],
  [ 0xcee, 'KannadaDigitEight', [], 'Display', None],
  [ 0xcef, 'KannadaDigitNine', [], 'Display', None],
  [ 0xcf0, 'UnicodeCharacter0xcf0', [], 'Display', None],
  [ 0xcf1, 'KannadaSignJihvamuliya', [], 'Display', None],
  [ 0xcf2, 'KannadaSignUpadhmaniya', [], 'Display', None],
  [ 0xcf3, 'UnicodeCharacter0xcf3', [], 'Display', None],
  [ 0xcf4, 'UnicodeCharacter0xcf4', [], 'Display', None],
  [ 0xcf5, 'UnicodeCharacter0xcf5', [], 'Display', None],
  [ 0xcf6, 'UnicodeCharacter0xcf6', [], 'Display', None],
  [ 0xcf7, 'UnicodeCharacter0xcf7', [], 'Display', None],
  [ 0xcf8, 'UnicodeCharacter0xcf8', [], 'Display', None],
  [ 0xcf9, 'UnicodeCharacter0xcf9', [], 'Display', None],
  [ 0xcfa, 'UnicodeCharacter0xcfa', [], 'Display', None],
  [ 0xcfb, 'UnicodeCharacter0xcfb', [], 'Display', None],
  [ 0xcfc, 'UnicodeCharacter0xcfc', [], 'Display', None],
  [ 0xcfd, 'UnicodeCharacter0xcfd', [], 'Display', None],
  [ 0xcfe, 'UnicodeCharacter0xcfe', [], 'Display', None],
  [ 0xcff, 'UnicodeCharacter0xcff', [], 'Display', None],
  [ 0xd00, 'UnicodeCharacter0xd00', [], 'Display', None],
  [ 0xd01, 'UnicodeCharacter0xd01', [], 'Display', None],
  [ 0xd02, 'MalayalamSignAnusvara', [], 'Display', None],
  [ 0xd03, 'MalayalamSignVisarga', [], 'Display', None],
  [ 0xd04, 'UnicodeCharacter0xd04', [], 'Display', None],
  [ 0xd05, 'MalayalamLetterA', [], 'Display', None],
  [ 0xd06, 'MalayalamLetterAa', [], 'Display', None],
  [ 0xd07, 'MalayalamLetterI', [], 'Display', None],
  [ 0xd08, 'MalayalamLetterIi', [], 'Display', None],
  [ 0xd09, 'MalayalamLetterU', [], 'Display', None],
  [ 0xd0a, 'MalayalamLetterUu', [], 'Display', None],
  [ 0xd0b, 'MalayalamLetterVocalicR', [], 'Display', None],
  [ 0xd0c, 'MalayalamLetterVocalicL', [], 'Display', None],
  [ 0xd0d, 'UnicodeCharacter0xd0d', [], 'Display', None],
  [ 0xd0e, 'MalayalamLetterE', [], 'Display', None],
  [ 0xd0f, 'MalayalamLetterEe', [], 'Display', None],
  [ 0xd10, 'MalayalamLetterAi', [], 'Display', None],
  [ 0xd11, 'UnicodeCharacter0xd11', [], 'Display', None],
  [ 0xd12, 'MalayalamLetterO', [], 'Display', None],
  [ 0xd13, 'MalayalamLetterOo', [], 'Display', None],
  [ 0xd14, 'MalayalamLetterAu', [], 'Display', None],
  [ 0xd15, 'MalayalamLetterKa', [], 'Display', None],
  [ 0xd16, 'MalayalamLetterKha', [], 'Display', None],
  [ 0xd17, 'MalayalamLetterGa', [], 'Display', None],
  [ 0xd18, 'MalayalamLetterGha', [], 'Display', None],
  [ 0xd19, 'MalayalamLetterNga', [], 'Display', None],
  [ 0xd1a, 'MalayalamLetterCa', [], 'Display', None],
  [ 0xd1b, 'MalayalamLetterCha', [], 'Display', None],
  [ 0xd1c, 'MalayalamLetterJa', [], 'Display', None],
  [ 0xd1d, 'MalayalamLetterJha', [], 'Display', None],
  [ 0xd1e, 'MalayalamLetterNya', [], 'Display', None],
  [ 0xd1f, 'MalayalamLetterTta', [], 'Display', None],
  [ 0xd20, 'MalayalamLetterTtha', [], 'Display', None],
  [ 0xd21, 'MalayalamLetterDda', [], 'Display', None],
  [ 0xd22, 'MalayalamLetterDdha', [], 'Display', None],
  [ 0xd23, 'MalayalamLetterNna', [], 'Display', None],
  [ 0xd24, 'MalayalamLetterTa', [], 'Display', None],
  [ 0xd25, 'MalayalamLetterTha', [], 'Display', None],
  [ 0xd26, 'MalayalamLetterDa', [], 'Display', None],
  [ 0xd27, 'MalayalamLetterDha', [], 'Display', None],
  [ 0xd28, 'MalayalamLetterNa', [], 'Display', None],
  [ 0xd29, 'UnicodeCharacter0xd29', [], 'Display', None],
  [ 0xd2a, 'MalayalamLetterPa', [], 'Display', None],
  [ 0xd2b, 'MalayalamLetterPha', [], 'Display', None],
  [ 0xd2c, 'MalayalamLetterBa', [], 'Display', None],
  [ 0xd2d, 'MalayalamLetterBha', [], 'Display', None],
  [ 0xd2e, 'MalayalamLetterMa', [], 'Display', None],
  [ 0xd2f, 'MalayalamLetterYa', [], 'Display', None],
  [ 0xd30, 'MalayalamLetterRa', [], 'Display', None],
  [ 0xd31, 'MalayalamLetterRra', [], 'Display', None],
  [ 0xd32, 'MalayalamLetterLa', [], 'Display', None],
  [ 0xd33, 'MalayalamLetterLla', [], 'Display', None],
  [ 0xd34, 'MalayalamLetterLlla', [], 'Display', None],
  [ 0xd35, 'MalayalamLetterVa', [], 'Display', None],
  [ 0xd36, 'MalayalamLetterSha', [], 'Display', None],
  [ 0xd37, 'MalayalamLetterSsa', [], 'Display', None],
  [ 0xd38, 'MalayalamLetterSa', [], 'Display', None],
  [ 0xd39, 'MalayalamLetterHa', [], 'Display', None],
  [ 0xd3a, 'UnicodeCharacter0xd3a', [], 'Display', None],
  [ 0xd3b, 'UnicodeCharacter0xd3b', [], 'Display', None],
  [ 0xd3c, 'UnicodeCharacter0xd3c', [], 'Display', None],
  [ 0xd3d, 'MalayalamSignAvagraha', [], 'Display', None],
  [ 0xd3e, 'MalayalamVowelSignAa', [], 'Display', None],
  [ 0xd3f, 'MalayalamVowelSignI', [], 'Display', None],
  [ 0xd40, 'MalayalamVowelSignIi', [], 'Display', None],
  [ 0xd41, 'MalayalamVowelSignU', [], 'Display', None],
  [ 0xd42, 'MalayalamVowelSignUu', [], 'Display', None],
  [ 0xd43, 'MalayalamVowelSignVocalicR', [], 'Display', None],
  [ 0xd44, 'MalayalamVowelSignVocalicRr', [], 'Display', None],
  [ 0xd45, 'UnicodeCharacter0xd45', [], 'Display', None],
  [ 0xd46, 'MalayalamVowelSignE', [], 'Display', None],
  [ 0xd47, 'MalayalamVowelSignEe', [], 'Display', None],
  [ 0xd48, 'MalayalamVowelSignAi', [], 'Display', None],
  [ 0xd49, 'UnicodeCharacter0xd49', [], 'Display', None],
  [ 0xd4a, 'MalayalamVowelSignO', [], 'Display', None],
  [ 0xd4b, 'MalayalamVowelSignOo', [], 'Display', None],
  [ 0xd4c, 'MalayalamVowelSignAu', [], 'Display', None],
  [ 0xd4d, 'MalayalamSignVirama', [], 'Display', None],
  [ 0xd4e, 'UnicodeCharacter0xd4e', [], 'Display', None],
  [ 0xd4f, 'UnicodeCharacter0xd4f', [], 'Display', None],
  [ 0xd50, 'UnicodeCharacter0xd50', [], 'Display', None],
  [ 0xd51, 'UnicodeCharacter0xd51', [], 'Display', None],
  [ 0xd52, 'UnicodeCharacter0xd52', [], 'Display', None],
  [ 0xd53, 'UnicodeCharacter0xd53', [], 'Display', None],
  [ 0xd54, 'UnicodeCharacter0xd54', [], 'Display', None],
  [ 0xd55, 'UnicodeCharacter0xd55', [], 'Display', None],
  [ 0xd56, 'UnicodeCharacter0xd56', [], 'Display', None],
  [ 0xd57, 'MalayalamAuLengthMark', [], 'Display', None],
  [ 0xd58, 'UnicodeCharacter0xd58', [], 'Display', None],
  [ 0xd59, 'UnicodeCharacter0xd59', [], 'Display', None],
  [ 0xd5a, 'UnicodeCharacter0xd5a', [], 'Display', None],
  [ 0xd5b, 'UnicodeCharacter0xd5b', [], 'Display', None],
  [ 0xd5c, 'UnicodeCharacter0xd5c', [], 'Display', None],
  [ 0xd5d, 'UnicodeCharacter0xd5d', [], 'Display', None],
  [ 0xd5e, 'UnicodeCharacter0xd5e', [], 'Display', None],
  [ 0xd5f, 'UnicodeCharacter0xd5f', [], 'Display', None],
  [ 0xd60, 'MalayalamLetterVocalicRr', [], 'Display', None],
  [ 0xd61, 'MalayalamLetterVocalicLl', [], 'Display', None],
  [ 0xd62, 'MalayalamVowelSignVocalicL', [], 'Display', None],
  [ 0xd63, 'MalayalamVowelSignVocalicLl', [], 'Display', None],
  [ 0xd64, 'UnicodeCharacter0xd64', [], 'Display', None],
  [ 0xd65, 'UnicodeCharacter0xd65', [], 'Display', None],
  [ 0xd66, 'MalayalamDigitZero', [], 'Display', None],
  [ 0xd67, 'MalayalamDigitOne', [], 'Display', None],
  [ 0xd68, 'MalayalamDigitTwo', [], 'Display', None],
  [ 0xd69, 'MalayalamDigitThree', [], 'Display', None],
  [ 0xd6a, 'MalayalamDigitFour', [], 'Display', None],
  [ 0xd6b, 'MalayalamDigitFive', [], 'Display', None],
  [ 0xd6c, 'MalayalamDigitSix', [], 'Display', None],
  [ 0xd6d, 'MalayalamDigitSeven', [], 'Display', None],
  [ 0xd6e, 'MalayalamDigitEight', [], 'Display', None],
  [ 0xd6f, 'MalayalamDigitNine', [], 'Display', None],
  [ 0xd70, 'MalayalamNumberTen', [], 'Display', None],
  [ 0xd71, 'MalayalamNumberOneHundred', [], 'Display', None],
  [ 0xd72, 'MalayalamNumberOneThousand', [], 'Display', None],
  [ 0xd73, 'MalayalamFractionOneQuarter', [], 'Display', None],
  [ 0xd74, 'MalayalamFractionOneHalf', [], 'Display', None],
  [ 0xd75, 'MalayalamFractionThreeQuarters', [], 'Display', None],
  [ 0xd76, 'UnicodeCharacter0xd76', [], 'Display', None],
  [ 0xd77, 'UnicodeCharacter0xd77', [], 'Display', None],
  [ 0xd78, 'UnicodeCharacter0xd78', [], 'Display', None],
  [ 0xd79, 'MalayalamDateMark', [], 'Display', None],
  [ 0xd7a, 'MalayalamLetterChilluNn', [], 'Display', None],
  [ 0xd7b, 'MalayalamLetterChilluN', [], 'Display', None],
  [ 0xd7c, 'MalayalamLetterChilluRr', [], 'Display', None],
  [ 0xd7d, 'MalayalamLetterChilluL', [], 'Display', None],
  [ 0xd7e, 'MalayalamLetterChilluLl', [], 'Display', None],
  [ 0xd7f, 'MalayalamLetterChilluK', [], 'Display', None],
  [ 0xd80, 'UnicodeCharacter0xd80', [], 'Display', None],
  [ 0xd81, 'UnicodeCharacter0xd81', [], 'Display', None],
  [ 0xd82, 'SinhalaSignAnusvaraya', [], 'Display', None],
  [ 0xd83, 'SinhalaSignVisargaya', [], 'Display', None],
  [ 0xd84, 'UnicodeCharacter0xd84', [], 'Display', None],
  [ 0xd85, 'SinhalaLetterAyanna', [], 'Display', None],
  [ 0xd86, 'SinhalaLetterAayanna', [], 'Display', None],
  [ 0xd87, 'SinhalaLetterAeyanna', [], 'Display', None],
  [ 0xd88, 'SinhalaLetterAeeyanna', [], 'Display', None],
  [ 0xd89, 'SinhalaLetterIyanna', [], 'Display', None],
  [ 0xd8a, 'SinhalaLetterIiyanna', [], 'Display', None],
  [ 0xd8b, 'SinhalaLetterUyanna', [], 'Display', None],
  [ 0xd8c, 'SinhalaLetterUuyanna', [], 'Display', None],
  [ 0xd8d, 'SinhalaLetterIruyanna', [], 'Display', None],
  [ 0xd8e, 'SinhalaLetterIruuyanna', [], 'Display', None],
  [ 0xd8f, 'SinhalaLetterIluyanna', [], 'Display', None],
  [ 0xd90, 'SinhalaLetterIluuyanna', [], 'Display', None],
  [ 0xd91, 'SinhalaLetterEyanna', [], 'Display', None],
  [ 0xd92, 'SinhalaLetterEeyanna', [], 'Display', None],
  [ 0xd93, 'SinhalaLetterAiyanna', [], 'Display', None],
  [ 0xd94, 'SinhalaLetterOyanna', [], 'Display', None],
  [ 0xd95, 'SinhalaLetterOoyanna', [], 'Display', None],
  [ 0xd96, 'SinhalaLetterAuyanna', [], 'Display', None],
  [ 0xd97, 'UnicodeCharacter0xd97', [], 'Display', None],
  [ 0xd98, 'UnicodeCharacter0xd98', [], 'Display', None],
  [ 0xd99, 'UnicodeCharacter0xd99', [], 'Display', None],
  [ 0xd9a, 'SinhalaLetterAlpapraanaKayanna', [], 'Display', None],
  [ 0xd9b, 'SinhalaLetterMahaapraanaKayanna', [], 'Display', None],
  [ 0xd9c, 'SinhalaLetterAlpapraanaGayanna', [], 'Display', None],
  [ 0xd9d, 'SinhalaLetterMahaapraanaGayanna', [], 'Display', None],
  [ 0xd9e, 'SinhalaLetterKantajaNaasikyaya', [], 'Display', None],
  [ 0xd9f, 'SinhalaLetterSanyakaGayanna', [], 'Display', None],
  [ 0xda0, 'SinhalaLetterAlpapraanaCayanna', [], 'Display', None],
  [ 0xda1, 'SinhalaLetterMahaapraanaCayanna', [], 'Display', None],
  [ 0xda2, 'SinhalaLetterAlpapraanaJayanna', [], 'Display', None],
  [ 0xda3, 'SinhalaLetterMahaapraanaJayanna', [], 'Display', None],
  [ 0xda4, 'SinhalaLetterTaalujaNaasikyaya', [], 'Display', None],
  [ 0xda5, 'SinhalaLetterTaalujaSanyoogaNaaksikyaya', [], 'Display', None],
  [ 0xda6, 'SinhalaLetterSanyakaJayanna', [], 'Display', None],
  [ 0xda7, 'SinhalaLetterAlpapraanaTtayanna', [], 'Display', None],
  [ 0xda8, 'SinhalaLetterMahaapraanaTtayanna', [], 'Display', None],
  [ 0xda9, 'SinhalaLetterAlpapraanaDdayanna', [], 'Display', None],
  [ 0xdaa, 'SinhalaLetterMahaapraanaDdayanna', [], 'Display', None],
  [ 0xdab, 'SinhalaLetterMuurdhajaNayanna', [], 'Display', None],
  [ 0xdac, 'SinhalaLetterSanyakaDdayanna', [], 'Display', None],
  [ 0xdad, 'SinhalaLetterAlpapraanaTayanna', [], 'Display', None],
  [ 0xdae, 'SinhalaLetterMahaapraanaTayanna', [], 'Display', None],
  [ 0xdaf, 'SinhalaLetterAlpapraanaDayanna', [], 'Display', None],
  [ 0xdb0, 'SinhalaLetterMahaapraanaDayanna', [], 'Display', None],
  [ 0xdb1, 'SinhalaLetterDantajaNayanna', [], 'Display', None],
  [ 0xdb2, 'UnicodeCharacter0xdb2', [], 'Display', None],
  [ 0xdb3, 'SinhalaLetterSanyakaDayanna', [], 'Display', None],
  [ 0xdb4, 'SinhalaLetterAlpapraanaPayanna', [], 'Display', None],
  [ 0xdb5, 'SinhalaLetterMahaapraanaPayanna', [], 'Display', None],
  [ 0xdb6, 'SinhalaLetterAlpapraanaBayanna', [], 'Display', None],
  [ 0xdb7, 'SinhalaLetterMahaapraanaBayanna', [], 'Display', None],
  [ 0xdb8, 'SinhalaLetterMayanna', [], 'Display', None],
  [ 0xdb9, 'SinhalaLetterAmbaBayanna', [], 'Display', None],
  [ 0xdba, 'SinhalaLetterYayanna', [], 'Display', None],
  [ 0xdbb, 'SinhalaLetterRayanna', [], 'Display', None],
  [ 0xdbc, 'UnicodeCharacter0xdbc', [], 'Display', None],
  [ 0xdbd, 'SinhalaLetterDantajaLayanna', [], 'Display', None],
  [ 0xdbe, 'UnicodeCharacter0xdbe', [], 'Display', None],
  [ 0xdbf, 'UnicodeCharacter0xdbf', [], 'Display', None],
  [ 0xdc0, 'SinhalaLetterVayanna', [], 'Display', None],
  [ 0xdc1, 'SinhalaLetterTaalujaSayanna', [], 'Display', None],
  [ 0xdc2, 'SinhalaLetterMuurdhajaSayanna', [], 'Display', None],
  [ 0xdc3, 'SinhalaLetterDantajaSayanna', [], 'Display', None],
  [ 0xdc4, 'SinhalaLetterHayanna', [], 'Display', None],
  [ 0xdc5, 'SinhalaLetterMuurdhajaLayanna', [], 'Display', None],
  [ 0xdc6, 'SinhalaLetterFayanna', [], 'Display', None],
  [ 0xdc7, 'UnicodeCharacter0xdc7', [], 'Display', None],
  [ 0xdc8, 'UnicodeCharacter0xdc8', [], 'Display', None],
  [ 0xdc9, 'UnicodeCharacter0xdc9', [], 'Display', None],
  [ 0xdca, 'SinhalaSignAllakuna', [], 'Display', None],
  [ 0xdcb, 'UnicodeCharacter0xdcb', [], 'Display', None],
  [ 0xdcc, 'UnicodeCharacter0xdcc', [], 'Display', None],
  [ 0xdcd, 'UnicodeCharacter0xdcd', [], 'Display', None],
  [ 0xdce, 'UnicodeCharacter0xdce', [], 'Display', None],
  [ 0xdcf, 'SinhalaVowelSignAelapilla', [], 'Display', None],
  [ 0xdd0, 'SinhalaVowelSignKettiAedapilla', [], 'Display', None],
  [ 0xdd1, 'SinhalaVowelSignDigaAedapilla', [], 'Display', None],
  [ 0xdd2, 'SinhalaVowelSignKettiIspilla', [], 'Display', None],
  [ 0xdd3, 'SinhalaVowelSignDigaIspilla', [], 'Display', None],
  [ 0xdd4, 'SinhalaVowelSignKettiPaapilla', [], 'Display', None],
  [ 0xdd5, 'UnicodeCharacter0xdd5', [], 'Display', None],
  [ 0xdd6, 'SinhalaVowelSignDigaPaapilla', [], 'Display', None],
  [ 0xdd7, 'UnicodeCharacter0xdd7', [], 'Display', None],
  [ 0xdd8, 'SinhalaVowelSignGaettapilla', [], 'Display', None],
  [ 0xdd9, 'SinhalaVowelSignKombuva', [], 'Display', None],
  [ 0xdda, 'SinhalaVowelSignDigaKombuva', [], 'Display', None],
  [ 0xddb, 'SinhalaVowelSignKombuDeka', [], 'Display', None],
  [ 0xddc, 'SinhalaVowelSignKombuvaHaaAelapilla', [], 'Display', None],
  [ 0xddd, 'SinhalaVowelSignKombuvaHaaDigaAelapilla', [], 'Display', None],
  [ 0xdde, 'SinhalaVowelSignKombuvaHaaGayanukitta', [], 'Display', None],
  [ 0xddf, 'SinhalaVowelSignGayanukitta', [], 'Display', None],
  [ 0xde0, 'UnicodeCharacter0xde0', [], 'Display', None],
  [ 0xde1, 'UnicodeCharacter0xde1', [], 'Display', None],
  [ 0xde2, 'UnicodeCharacter0xde2', [], 'Display', None],
  [ 0xde3, 'UnicodeCharacter0xde3', [], 'Display', None],
  [ 0xde4, 'UnicodeCharacter0xde4', [], 'Display', None],
  [ 0xde5, 'UnicodeCharacter0xde5', [], 'Display', None],
  [ 0xde6, 'UnicodeCharacter0xde6', [], 'Display', None],
  [ 0xde7, 'UnicodeCharacter0xde7', [], 'Display', None],
  [ 0xde8, 'UnicodeCharacter0xde8', [], 'Display', None],
  [ 0xde9, 'UnicodeCharacter0xde9', [], 'Display', None],
  [ 0xdea, 'UnicodeCharacter0xdea', [], 'Display', None],
  [ 0xdeb, 'UnicodeCharacter0xdeb', [], 'Display', None],
  [ 0xdec, 'UnicodeCharacter0xdec', [], 'Display', None],
  [ 0xded, 'UnicodeCharacter0xded', [], 'Display', None],
  [ 0xdee, 'UnicodeCharacter0xdee', [], 'Display', None],
  [ 0xdef, 'UnicodeCharacter0xdef', [], 'Display', None],
  [ 0xdf0, 'UnicodeCharacter0xdf0', [], 'Display', None],
  [ 0xdf1, 'UnicodeCharacter0xdf1', [], 'Display', None],
  [ 0xdf2, 'SinhalaVowelSignDigaGaettapilla', [], 'Display', None],
  [ 0xdf3, 'SinhalaVowelSignDigaGayanukitta', [], 'Display', None],
  [ 0xdf4, 'SinhalaPunctuationKunddaliya', [], 'Display', None],
  [ 0xdf5, 'UnicodeCharacter0xdf5', [], 'Display', None],
  [ 0xdf6, 'UnicodeCharacter0xdf6', [], 'Display', None],
  [ 0xdf7, 'UnicodeCharacter0xdf7', [], 'Display', None],
  [ 0xdf8, 'UnicodeCharacter0xdf8', [], 'Display', None],
  [ 0xdf9, 'UnicodeCharacter0xdf9', [], 'Display', None],
  [ 0xdfa, 'UnicodeCharacter0xdfa', [], 'Display', None],
  [ 0xdfb, 'UnicodeCharacter0xdfb', [], 'Display', None],
  [ 0xdfc, 'UnicodeCharacter0xdfc', [], 'Display', None],
  [ 0xdfd, 'UnicodeCharacter0xdfd', [], 'Display', None],
  [ 0xdfe, 'UnicodeCharacter0xdfe', [], 'Display', None],
  [ 0xdff, 'UnicodeCharacter0xdff', [], 'Display', None],
  [ 0xe00, 'UnicodeCharacter0xe00', [], 'Display', None],
  [ 0xe01, 'ThaiCharacterKoKai', [], 'Display', None],
  [ 0xe02, 'ThaiCharacterKhoKhai', [], 'Display', None],
  [ 0xe03, 'ThaiCharacterKhoKhuat', [], 'Display', None],
  [ 0xe04, 'ThaiCharacterKhoKhwai', [], 'Display', None],
  [ 0xe05, 'ThaiCharacterKhoKhon', [], 'Display', None],
  [ 0xe06, 'ThaiCharacterKhoRakhang', [], 'Display', None],
  [ 0xe07, 'ThaiCharacterNgoNgu', [], 'Display', None],
  [ 0xe08, 'ThaiCharacterChoChan', [], 'Display', None],
  [ 0xe09, 'ThaiCharacterChoChing', [], 'Display', None],
  [ 0xe0a, 'ThaiCharacterChoChang', [], 'Display', None],
  [ 0xe0b, 'ThaiCharacterSoSo', [], 'Display', None],
  [ 0xe0c, 'ThaiCharacterChoChoe', [], 'Display', None],
  [ 0xe0d, 'ThaiCharacterYoYing', [], 'Display', None],
  [ 0xe0e, 'ThaiCharacterDoChada', [], 'Display', None],
  [ 0xe0f, 'ThaiCharacterToPatak', [], 'Display', None],
  [ 0xe10, 'ThaiCharacterThoThan', [], 'Display', None],
  [ 0xe11, 'ThaiCharacterThoNangmontho', [], 'Display', None],
  [ 0xe12, 'ThaiCharacterThoPhuthao', [], 'Display', None],
  [ 0xe13, 'ThaiCharacterNoNen', [], 'Display', None],
  [ 0xe14, 'ThaiCharacterDoDek', [], 'Display', None],
  [ 0xe15, 'ThaiCharacterToTao', [], 'Display', None],
  [ 0xe16, 'ThaiCharacterThoThung', [], 'Display', None],
  [ 0xe17, 'ThaiCharacterThoThahan', [], 'Display', None],
  [ 0xe18, 'ThaiCharacterThoThong', [], 'Display', None],
  [ 0xe19, 'ThaiCharacterNoNu', [], 'Display', None],
  [ 0xe1a, 'ThaiCharacterBoBaimai', [], 'Display', None],
  [ 0xe1b, 'ThaiCharacterPoPla', [], 'Display', None],
  [ 0xe1c, 'ThaiCharacterPhoPhung', [], 'Display', None],
  [ 0xe1d, 'ThaiCharacterFoFa', [], 'Display', None],
  [ 0xe1e, 'ThaiCharacterPhoPhan', [], 'Display', None],
  [ 0xe1f, 'ThaiCharacterFoFan', [], 'Display', None],
  [ 0xe20, 'ThaiCharacterPhoSamphao', [], 'Display', None],
  [ 0xe21, 'ThaiCharacterMoMa', [], 'Display', None],
  [ 0xe22, 'ThaiCharacterYoYak', [], 'Display', None],
  [ 0xe23, 'ThaiCharacterRoRua', [], 'Display', None],
  [ 0xe24, 'ThaiCharacterRu', [], 'Display', None],
  [ 0xe25, 'ThaiCharacterLoLing', [], 'Display', None],
  [ 0xe26, 'ThaiCharacterLu', [], 'Display', None],
  [ 0xe27, 'ThaiCharacterWoWaen', [], 'Display', None],
  [ 0xe28, 'ThaiCharacterSoSala', [], 'Display', None],
  [ 0xe29, 'ThaiCharacterSoRusi', [], 'Display', None],
  [ 0xe2a, 'ThaiCharacterSoSua', [], 'Display', None],
  [ 0xe2b, 'ThaiCharacterHoHip', [], 'Display', None],
  [ 0xe2c, 'ThaiCharacterLoChula', [], 'Display', None],
  [ 0xe2d, 'ThaiCharacterOAng', [], 'Display', None],
  [ 0xe2e, 'ThaiCharacterHoNokhuk', [], 'Display', None],
  [ 0xe2f, 'ThaiCharacterPaiyannoi', [], 'Display', None],
  [ 0xe30, 'ThaiCharacterSaraA', [], 'Display', None],
  [ 0xe31, 'ThaiCharacterMaiHanakat', [], 'Display', None],
  [ 0xe32, 'ThaiCharacterSaraAa', [], 'Display', None],
  [ 0xe33, 'ThaiCharacterSaraAm', [], 'Display', None],
  [ 0xe34, 'ThaiCharacterSaraI', [], 'Display', None],
  [ 0xe35, 'ThaiCharacterSaraIi', [], 'Display', None],
  [ 0xe36, 'ThaiCharacterSaraUe', [], 'Display', None],
  [ 0xe37, 'ThaiCharacterSaraUee', [], 'Display', None],
  [ 0xe38, 'ThaiCharacterSaraU', [], 'Display', None],
  [ 0xe39, 'ThaiCharacterSaraUu', [], 'Display', None],
  [ 0xe3a, 'ThaiCharacterPhinthu', [], 'Display', None],
  [ 0xe3b, 'UnicodeCharacter0xe3b', [], 'Display', None],
  [ 0xe3c, 'UnicodeCharacter0xe3c', [], 'Display', None],
  [ 0xe3d, 'UnicodeCharacter0xe3d', [], 'Display', None],
  [ 0xe3e, 'UnicodeCharacter0xe3e', [], 'Display', None],
  [ 0xe3f, 'ThaiCurrencySymbolBaht', [], 'Display', None],
  [ 0xe40, 'ThaiCharacterSaraE', [], 'Display', None],
  [ 0xe41, 'ThaiCharacterSaraAe', [], 'Display', None],
  [ 0xe42, 'ThaiCharacterSaraO', [], 'Display', None],
  [ 0xe43, 'ThaiCharacterSaraAiMaimuan', [], 'Display', None],
  [ 0xe44, 'ThaiCharacterSaraAiMaimalai', [], 'Display', None],
  [ 0xe45, 'ThaiCharacterLakkhangyao', [], 'Display', None],
  [ 0xe46, 'ThaiCharacterMaiyamok', [], 'Display', None],
  [ 0xe47, 'ThaiCharacterMaitaikhu', [], 'Display', None],
  [ 0xe48, 'ThaiCharacterMaiEk', [], 'Display', None],
  [ 0xe49, 'ThaiCharacterMaiTho', [], 'Display', None],
  [ 0xe4a, 'ThaiCharacterMaiTri', [], 'Display', None],
  [ 0xe4b, 'ThaiCharacterMaiChattawa', [], 'Display', None],
  [ 0xe4c, 'ThaiCharacterThanthakhat', [], 'Display', None],
  [ 0xe4d, 'ThaiCharacterNikhahit', [], 'Display', None],
  [ 0xe4e, 'ThaiCharacterYamakkan', [], 'Display', None],
  [ 0xe4f, 'ThaiCharacterFongman', [], 'Display', None],
  [ 0xe50, 'ThaiDigitZero', [], 'Display', None],
  [ 0xe51, 'ThaiDigitOne', [], 'Display', None],
  [ 0xe52, 'ThaiDigitTwo', [], 'Display', None],
  [ 0xe53, 'ThaiDigitThree', [], 'Display', None],
  [ 0xe54, 'ThaiDigitFour', [], 'Display', None],
  [ 0xe55, 'ThaiDigitFive', [], 'Display', None],
  [ 0xe56, 'ThaiDigitSix', [], 'Display', None],
  [ 0xe57, 'ThaiDigitSeven', [], 'Display', None],
  [ 0xe58, 'ThaiDigitEight', [], 'Display', None],
  [ 0xe59, 'ThaiDigitNine', [], 'Display', None],
  [ 0xe5a, 'ThaiCharacterAngkhankhu', [], 'Display', None],
  [ 0xe5b, 'ThaiCharacterKhomut', [], 'Display', None],
  [ 0xe5c, 'UnicodeCharacter0xe5c', [], 'Display', None],
  [ 0xe5d, 'UnicodeCharacter0xe5d', [], 'Display', None],
  [ 0xe5e, 'UnicodeCharacter0xe5e', [], 'Display', None],
  [ 0xe5f, 'UnicodeCharacter0xe5f', [], 'Display', None],
  [ 0xe60, 'UnicodeCharacter0xe60', [], 'Display', None],
  [ 0xe61, 'UnicodeCharacter0xe61', [], 'Display', None],
  [ 0xe62, 'UnicodeCharacter0xe62', [], 'Display', None],
  [ 0xe63, 'UnicodeCharacter0xe63', [], 'Display', None],
  [ 0xe64, 'UnicodeCharacter0xe64', [], 'Display', None],
  [ 0xe65, 'UnicodeCharacter0xe65', [], 'Display', None],
  [ 0xe66, 'UnicodeCharacter0xe66', [], 'Display', None],
  [ 0xe67, 'UnicodeCharacter0xe67', [], 'Display', None],
  [ 0xe68, 'UnicodeCharacter0xe68', [], 'Display', None],
  [ 0xe69, 'UnicodeCharacter0xe69', [], 'Display', None],
  [ 0xe6a, 'UnicodeCharacter0xe6a', [], 'Display', None],
  [ 0xe6b, 'UnicodeCharacter0xe6b', [], 'Display', None],
  [ 0xe6c, 'UnicodeCharacter0xe6c', [], 'Display', None],
  [ 0xe6d, 'UnicodeCharacter0xe6d', [], 'Display', None],
  [ 0xe6e, 'UnicodeCharacter0xe6e', [], 'Display', None],
  [ 0xe6f, 'UnicodeCharacter0xe6f', [], 'Display', None],
  [ 0xe70, 'UnicodeCharacter0xe70', [], 'Display', None],
  [ 0xe71, 'UnicodeCharacter0xe71', [], 'Display', None],
  [ 0xe72, 'UnicodeCharacter0xe72', [], 'Display', None],
  [ 0xe73, 'UnicodeCharacter0xe73', [], 'Display', None],
  [ 0xe74, 'UnicodeCharacter0xe74', [], 'Display', None],
  [ 0xe75, 'UnicodeCharacter0xe75', [], 'Display', None],
  [ 0xe76, 'UnicodeCharacter0xe76', [], 'Display', None],
  [ 0xe77, 'UnicodeCharacter0xe77', [], 'Display', None],
  [ 0xe78, 'UnicodeCharacter0xe78', [], 'Display', None],
  [ 0xe79, 'UnicodeCharacter0xe79', [], 'Display', None],
  [ 0xe7a, 'UnicodeCharacter0xe7a', [], 'Display', None],
  [ 0xe7b, 'UnicodeCharacter0xe7b', [], 'Display', None],
  [ 0xe7c, 'UnicodeCharacter0xe7c', [], 'Display', None],
  [ 0xe7d, 'UnicodeCharacter0xe7d', [], 'Display', None],
  [ 0xe7e, 'UnicodeCharacter0xe7e', [], 'Display', None],
  [ 0xe7f, 'UnicodeCharacter0xe7f', [], 'Display', None],
  [ 0xe80, 'UnicodeCharacter0xe80', [], 'Display', None],
  [ 0xe81, 'LaoLetterKo', [], 'Display', None],
  [ 0xe82, 'LaoLetterKhoSung', [], 'Display', None],
  [ 0xe83, 'UnicodeCharacter0xe83', [], 'Display', None],
  [ 0xe84, 'LaoLetterKhoTam', [], 'Display', None],
  [ 0xe85, 'UnicodeCharacter0xe85', [], 'Display', None],
  [ 0xe86, 'UnicodeCharacter0xe86', [], 'Display', None],
  [ 0xe87, 'LaoLetterNgo', [], 'Display', None],
  [ 0xe88, 'LaoLetterCo', [], 'Display', None],
  [ 0xe89, 'UnicodeCharacter0xe89', [], 'Display', None],
  [ 0xe8a, 'LaoLetterSoTam', [], 'Display', None],
  [ 0xe8b, 'UnicodeCharacter0xe8b', [], 'Display', None],
  [ 0xe8c, 'UnicodeCharacter0xe8c', [], 'Display', None],
  [ 0xe8d, 'LaoLetterNyo', [], 'Display', None],
  [ 0xe8e, 'UnicodeCharacter0xe8e', [], 'Display', None],
  [ 0xe8f, 'UnicodeCharacter0xe8f', [], 'Display', None],
  [ 0xe90, 'UnicodeCharacter0xe90', [], 'Display', None],
  [ 0xe91, 'UnicodeCharacter0xe91', [], 'Display', None],
  [ 0xe92, 'UnicodeCharacter0xe92', [], 'Display', None],
  [ 0xe93, 'UnicodeCharacter0xe93', [], 'Display', None],
  [ 0xe94, 'LaoLetterDo', [], 'Display', None],
  [ 0xe95, 'LaoLetterTo', [], 'Display', None],
  [ 0xe96, 'LaoLetterThoSung', [], 'Display', None],
  [ 0xe97, 'LaoLetterThoTam', [], 'Display', None],
  [ 0xe98, 'UnicodeCharacter0xe98', [], 'Display', None],
  [ 0xe99, 'LaoLetterNo', [], 'Display', None],
  [ 0xe9a, 'LaoLetterBo', [], 'Display', None],
  [ 0xe9b, 'LaoLetterPo', [], 'Display', None],
  [ 0xe9c, 'LaoLetterPhoSung', [], 'Display', None],
  [ 0xe9d, 'LaoLetterFoTam', [], 'Display', None],
  [ 0xe9e, 'LaoLetterPhoTam', [], 'Display', None],
  [ 0xe9f, 'LaoLetterFoSung', [], 'Display', None],
  [ 0xea0, 'UnicodeCharacter0xea0', [], 'Display', None],
  [ 0xea1, 'LaoLetterMo', [], 'Display', None],
  [ 0xea2, 'LaoLetterYo', [], 'Display', None],
  [ 0xea3, 'LaoLetterLoLing', [], 'Display', None],
  [ 0xea4, 'UnicodeCharacter0xea4', [], 'Display', None],
  [ 0xea5, 'LaoLetterLoLoot', [], 'Display', None],
  [ 0xea6, 'UnicodeCharacter0xea6', [], 'Display', None],
  [ 0xea7, 'LaoLetterWo', [], 'Display', None],
  [ 0xea8, 'UnicodeCharacter0xea8', [], 'Display', None],
  [ 0xea9, 'UnicodeCharacter0xea9', [], 'Display', None],
  [ 0xeaa, 'LaoLetterSoSung', [], 'Display', None],
  [ 0xeab, 'LaoLetterHoSung', [], 'Display', None],
  [ 0xeac, 'UnicodeCharacter0xeac', [], 'Display', None],
  [ 0xead, 'LaoLetterO', [], 'Display', None],
  [ 0xeae, 'LaoLetterHoTam', [], 'Display', None],
  [ 0xeaf, 'LaoEllipsis', [], 'Display', None],
  [ 0xeb0, 'LaoVowelSignA', [], 'Display', None],
  [ 0xeb1, 'LaoVowelSignMaiKan', [], 'Display', None],
  [ 0xeb2, 'LaoVowelSignAa', [], 'Display', None],
  [ 0xeb3, 'LaoVowelSignAm', [], 'Display', None],
  [ 0xeb4, 'LaoVowelSignI', [], 'Display', None],
  [ 0xeb5, 'LaoVowelSignIi', [], 'Display', None],
  [ 0xeb6, 'LaoVowelSignY', [], 'Display', None],
  [ 0xeb7, 'LaoVowelSignYy', [], 'Display', None],
  [ 0xeb8, 'LaoVowelSignU', [], 'Display', None],
  [ 0xeb9, 'LaoVowelSignUu', [], 'Display', None],
  [ 0xeba, 'UnicodeCharacter0xeba', [], 'Display', None],
  [ 0xebb, 'LaoVowelSignMaiKon', [], 'Display', None],
  [ 0xebc, 'LaoSemivowelSignLo', [], 'Display', None],
  [ 0xebd, 'LaoSemivowelSignNyo', [], 'Display', None],
  [ 0xebe, 'UnicodeCharacter0xebe', [], 'Display', None],
  [ 0xebf, 'UnicodeCharacter0xebf', [], 'Display', None],
  [ 0xec0, 'LaoVowelSignE', [], 'Display', None],
  [ 0xec1, 'LaoVowelSignEi', [], 'Display', None],
  [ 0xec2, 'LaoVowelSignO', [], 'Display', None],
  [ 0xec3, 'LaoVowelSignAy', [], 'Display', None],
  [ 0xec4, 'LaoVowelSignAi', [], 'Display', None],
  [ 0xec5, 'UnicodeCharacter0xec5', [], 'Display', None],
  [ 0xec6, 'LaoKoLa', [], 'Display', None],
  [ 0xec7, 'UnicodeCharacter0xec7', [], 'Display', None],
  [ 0xec8, 'LaoToneMaiEk', [], 'Display', None],
  [ 0xec9, 'LaoToneMaiTho', [], 'Display', None],
  [ 0xeca, 'LaoToneMaiTi', [], 'Display', None],
  [ 0xecb, 'LaoToneMaiCatawa', [], 'Display', None],
  [ 0xecc, 'LaoCancellationMark', [], 'Display', None],
  [ 0xecd, 'LaoNiggahita', [], 'Display', None],
  [ 0xece, 'UnicodeCharacter0xece', [], 'Display', None],
  [ 0xecf, 'UnicodeCharacter0xecf', [], 'Display', None],
  [ 0xed0, 'LaoDigitZero', [], 'Display', None],
  [ 0xed1, 'LaoDigitOne', [], 'Display', None],
  [ 0xed2, 'LaoDigitTwo', [], 'Display', None],
  [ 0xed3, 'LaoDigitThree', [], 'Display', None],
  [ 0xed4, 'LaoDigitFour', [], 'Display', None],
  [ 0xed5, 'LaoDigitFive', [], 'Display', None],
  [ 0xed6, 'LaoDigitSix', [], 'Display', None],
  [ 0xed7, 'LaoDigitSeven', [], 'Display', None],
  [ 0xed8, 'LaoDigitEight', [], 'Display', None],
  [ 0xed9, 'LaoDigitNine', [], 'Display', None],
  [ 0xeda, 'UnicodeCharacter0xeda', [], 'Display', None],
  [ 0xedb, 'UnicodeCharacter0xedb', [], 'Display', None],
  [ 0xedc, 'LaoHoNo', [], 'Display', None],
  [ 0xedd, 'LaoHoMo', [], 'Display', None],
  [ 0xede, 'UnicodeCharacter0xede', [], 'Display', None],
  [ 0xedf, 'UnicodeCharacter0xedf', [], 'Display', None],
  [ 0xee0, 'UnicodeCharacter0xee0', [], 'Display', None],
  [ 0xee1, 'UnicodeCharacter0xee1', [], 'Display', None],
  [ 0xee2, 'UnicodeCharacter0xee2', [], 'Display', None],
  [ 0xee3, 'UnicodeCharacter0xee3', [], 'Display', None],
  [ 0xee4, 'UnicodeCharacter0xee4', [], 'Display', None],
  [ 0xee5, 'UnicodeCharacter0xee5', [], 'Display', None],
  [ 0xee6, 'UnicodeCharacter0xee6', [], 'Display', None],
  [ 0xee7, 'UnicodeCharacter0xee7', [], 'Display', None],
  [ 0xee8, 'UnicodeCharacter0xee8', [], 'Display', None],
  [ 0xee9, 'UnicodeCharacter0xee9', [], 'Display', None],
  [ 0xeea, 'UnicodeCharacter0xeea', [], 'Display', None],
  [ 0xeeb, 'UnicodeCharacter0xeeb', [], 'Display', None],
  [ 0xeec, 'UnicodeCharacter0xeec', [], 'Display', None],
  [ 0xeed, 'UnicodeCharacter0xeed', [], 'Display', None],
  [ 0xeee, 'UnicodeCharacter0xeee', [], 'Display', None],
  [ 0xeef, 'UnicodeCharacter0xeef', [], 'Display', None],
  [ 0xef0, 'UnicodeCharacter0xef0', [], 'Display', None],
  [ 0xef1, 'UnicodeCharacter0xef1', [], 'Display', None],
  [ 0xef2, 'UnicodeCharacter0xef2', [], 'Display', None],
  [ 0xef3, 'UnicodeCharacter0xef3', [], 'Display', None],
  [ 0xef4, 'UnicodeCharacter0xef4', [], 'Display', None],
  [ 0xef5, 'UnicodeCharacter0xef5', [], 'Display', None],
  [ 0xef6, 'UnicodeCharacter0xef6', [], 'Display', None],
  [ 0xef7, 'UnicodeCharacter0xef7', [], 'Display', None],
  [ 0xef8, 'UnicodeCharacter0xef8', [], 'Display', None],
  [ 0xef9, 'UnicodeCharacter0xef9', [], 'Display', None],
  [ 0xefa, 'UnicodeCharacter0xefa', [], 'Display', None],
  [ 0xefb, 'UnicodeCharacter0xefb', [], 'Display', None],
  [ 0xefc, 'UnicodeCharacter0xefc', [], 'Display', None],
  [ 0xefd, 'UnicodeCharacter0xefd', [], 'Display', None],
  [ 0xefe, 'UnicodeCharacter0xefe', [], 'Display', None],
  [ 0xeff, 'UnicodeCharacter0xeff', [], 'Display', None],
  [ 0xf00, 'TibetanSyllableOm', [], 'Display', None],
  [ 0xf01, 'TibetanMarkGterYigMgoTruncatedA', [], 'Display', None],
  [ 0xf02, 'TibetanMarkGterYigMgoUmRnamBcadMa', [], 'Display', None],
  [ 0xf03, 'TibetanMarkGterYigMgoUmGterTshegMa', [], 'Display', None],
  [ 0xf04, 'TibetanMarkInitialYigMgoMdunMa', [], 'Display', None],
  [ 0xf05, 'TibetanMarkClosingYigMgoSgabMa', [], 'Display', None],
  [ 0xf06, 'TibetanMarkCaretYigMgoPhurShadMa', [], 'Display', None],
  [ 0xf07, 'TibetanMarkYigMgoTshegShadMa', [], 'Display', None],
  [ 0xf08, 'TibetanMarkSbrulShad', [], 'Display', None],
  [ 0xf09, 'TibetanMarkBskurYigMgo', [], 'Display', None],
  [ 0xf0a, 'TibetanMarkBkaShogYigMgo', [], 'Display', None],
  [ 0xf0b, 'TibetanMarkIntersyllabicTsheg', [], 'Display', None],
  [ 0xf0c, 'TibetanMarkDelimiterTshegBstar', [], 'Display', None],
  [ 0xf0d, 'TibetanMarkShad', [], 'Display', None],
  [ 0xf0e, 'TibetanMarkNyisShad', [], 'Display', None],
  [ 0xf0f, 'TibetanMarkTshegShad', [], 'Display', None],
  [ 0xf10, 'TibetanMarkNyisTshegShad', [], 'Display', None],
  [ 0xf11, 'TibetanMarkRinChenSpungsShad', [], 'Display', None],
  [ 0xf12, 'TibetanMarkRgyaGramShad', [], 'Display', None],
  [ 0xf13, 'TibetanMarkCaretDzudRtagsMeLongCan', [], 'Display', None],
  [ 0xf14, 'TibetanMarkGterTsheg', [], 'Display', None],
  [ 0xf15, 'TibetanLogotypeSignChadRtags', [], 'Display', None],
  [ 0xf16, 'TibetanLogotypeSignLhagRtags', [], 'Display', None],
  [ 0xf17, 'TibetanAstrologicalSignSgraGcanCharRtags', [], 'Display', None],
  [ 0xf18, 'TibetanAstrologicalSignKhyudPa', [], 'Display', None],
  [ 0xf19, 'TibetanAstrologicalSignSdongTshugs', [], 'Display', None],
  [ 0xf1a, 'TibetanSignRdelDkarGcig', [], 'Display', None],
  [ 0xf1b, 'TibetanSignRdelDkarGnyis', [], 'Display', None],
  [ 0xf1c, 'TibetanSignRdelDkarGsum', [], 'Display', None],
  [ 0xf1d, 'TibetanSignRdelNagGcig', [], 'Display', None],
  [ 0xf1e, 'TibetanSignRdelNagGnyis', [], 'Display', None],
  [ 0xf1f, 'TibetanSignRdelDkarRdelNag', [], 'Display', None],
  [ 0xf20, 'TibetanDigitZero', [], 'Display', None],
  [ 0xf21, 'TibetanDigitOne', [], 'Display', None],
  [ 0xf22, 'TibetanDigitTwo', [], 'Display', None],
  [ 0xf23, 'TibetanDigitThree', [], 'Display', None],
  [ 0xf24, 'TibetanDigitFour', [], 'Display', None],
  [ 0xf25, 'TibetanDigitFive', [], 'Display', None],
  [ 0xf26, 'TibetanDigitSix', [], 'Display', None],
  [ 0xf27, 'TibetanDigitSeven', [], 'Display', None],
  [ 0xf28, 'TibetanDigitEight', [], 'Display', None],
  [ 0xf29, 'TibetanDigitNine', [], 'Display', None],
  [ 0xf2a, 'TibetanDigitHalfOne', [], 'Display', None],
  [ 0xf2b, 'TibetanDigitHalfTwo', [], 'Display', None],
  [ 0xf2c, 'TibetanDigitHalfThree', [], 'Display', None],
  [ 0xf2d, 'TibetanDigitHalfFour', [], 'Display', None],
  [ 0xf2e, 'TibetanDigitHalfFive', [], 'Display', None],
  [ 0xf2f, 'TibetanDigitHalfSix', [], 'Display', None],
  [ 0xf30, 'TibetanDigitHalfSeven', [], 'Display', None],
  [ 0xf31, 'TibetanDigitHalfEight', [], 'Display', None],
  [ 0xf32, 'TibetanDigitHalfNine', [], 'Display', None],
  [ 0xf33, 'TibetanDigitHalfZero', [], 'Display', None],
  [ 0xf34, 'TibetanMarkBsdusRtags', [], 'Display', None],
  [ 0xf35, 'TibetanMarkNgasBzungNyiZla', [], 'Display', None],
  [ 0xf36, 'TibetanMarkCaretDzudRtagsBzhiMigCan', [], 'Display', None],
  [ 0xf37, 'TibetanMarkNgasBzungSgorRtags', [], 'Display', None],
  [ 0xf38, 'TibetanMarkCheMgo', [], 'Display', None],
  [ 0xf39, 'TibetanMarkTsaPhru', [], 'Display', None],
  [ 0xf3a, 'TibetanMarkGugRtagsGyon', [], 'Display', None],
  [ 0xf3b, 'TibetanMarkGugRtagsGyas', [], 'Display', None],
  [ 0xf3c, 'TibetanMarkAngKhangGyon', [], 'Display', None],
  [ 0xf3d, 'TibetanMarkAngKhangGyas', [], 'Display', None],
  [ 0xf3e, 'TibetanSignYarTshes', [], 'Display', None],
  [ 0xf3f, 'TibetanSignMarTshes', [], 'Display', None],
  [ 0xf40, 'TibetanLetterKa', [], 'Display', None],
  [ 0xf41, 'TibetanLetterKha', [], 'Display', None],
  [ 0xf42, 'TibetanLetterGa', [], 'Display', None],
  [ 0xf43, 'TibetanLetterGha', [], 'Display', None],
  [ 0xf44, 'TibetanLetterNga', [], 'Display', None],
  [ 0xf45, 'TibetanLetterCa', [], 'Display', None],
  [ 0xf46, 'TibetanLetterCha', [], 'Display', None],
  [ 0xf47, 'TibetanLetterJa', [], 'Display', None],
  [ 0xf48, 'UnicodeCharacter0xf48', [], 'Display', None],
  [ 0xf49, 'TibetanLetterNya', [], 'Display', None],
  [ 0xf4a, 'TibetanLetterTta', [], 'Display', None],
  [ 0xf4b, 'TibetanLetterTtha', [], 'Display', None],
  [ 0xf4c, 'TibetanLetterDda', [], 'Display', None],
  [ 0xf4d, 'TibetanLetterDdha', [], 'Display', None],
  [ 0xf4e, 'TibetanLetterNna', [], 'Display', None],
  [ 0xf4f, 'TibetanLetterTa', [], 'Display', None],
  [ 0xf50, 'TibetanLetterTha', [], 'Display', None],
  [ 0xf51, 'TibetanLetterDa', [], 'Display', None],
  [ 0xf52, 'TibetanLetterDha', [], 'Display', None],
  [ 0xf53, 'TibetanLetterNa', [], 'Display', None],
  [ 0xf54, 'TibetanLetterPa', [], 'Display', None],
  [ 0xf55, 'TibetanLetterPha', [], 'Display', None],
  [ 0xf56, 'TibetanLetterBa', [], 'Display', None],
  [ 0xf57, 'TibetanLetterBha', [], 'Display', None],
  [ 0xf58, 'TibetanLetterMa', [], 'Display', None],
  [ 0xf59, 'TibetanLetterTsa', [], 'Display', None],
  [ 0xf5a, 'TibetanLetterTsha', [], 'Display', None],
  [ 0xf5b, 'TibetanLetterDza', [], 'Display', None],
  [ 0xf5c, 'TibetanLetterDzha', [], 'Display', None],
  [ 0xf5d, 'TibetanLetterWa', [], 'Display', None],
  [ 0xf5e, 'TibetanLetterZha', [], 'Display', None],
  [ 0xf5f, 'TibetanLetterZa', [], 'Display', None],
  [ 0xf60, 'TibetanLetterA', [], 'Display', None],
  [ 0xf61, 'TibetanLetterYa', [], 'Display', None],
  [ 0xf62, 'TibetanLetterRa', [], 'Display', None],
  [ 0xf63, 'TibetanLetterLa', [], 'Display', None],
  [ 0xf64, 'TibetanLetterSha', [], 'Display', None],
  [ 0xf65, 'TibetanLetterSsa', [], 'Display', None],
  [ 0xf66, 'TibetanLetterSa', [], 'Display', None],
  [ 0xf67, 'TibetanLetterHa', [], 'Display', None],
  [ 0xf68, 'TibetanLetterA', [], 'Display', None],
  [ 0xf69, 'TibetanLetterKssa', [], 'Display', None],
  [ 0xf6a, 'TibetanLetterFixedformRa', [], 'Display', None],
  [ 0xf6b, 'TibetanLetterKka', [], 'Display', None],
  [ 0xf6c, 'TibetanLetterRra', [], 'Display', None],
  [ 0xf6d, 'UnicodeCharacter0xf6d', [], 'Display', None],
  [ 0xf6e, 'UnicodeCharacter0xf6e', [], 'Display', None],
  [ 0xf6f, 'UnicodeCharacter0xf6f', [], 'Display', None],
  [ 0xf70, 'UnicodeCharacter0xf70', [], 'Display', None],
  [ 0xf71, 'TibetanVowelSignAa', [], 'Display', None],
  [ 0xf72, 'TibetanVowelSignI', [], 'Display', None],
  [ 0xf73, 'TibetanVowelSignIi', [], 'Display', None],
  [ 0xf74, 'TibetanVowelSignU', [], 'Display', None],
  [ 0xf75, 'TibetanVowelSignUu', [], 'Display', None],
  [ 0xf76, 'TibetanVowelSignVocalicR', [], 'Display', None],
  [ 0xf77, 'TibetanVowelSignVocalicRr', [], 'Display', None],
  [ 0xf78, 'TibetanVowelSignVocalicL', [], 'Display', None],
  [ 0xf79, 'TibetanVowelSignVocalicLl', [], 'Display', None],
  [ 0xf7a, 'TibetanVowelSignE', [], 'Display', None],
  [ 0xf7b, 'TibetanVowelSignEe', [], 'Display', None],
  [ 0xf7c, 'TibetanVowelSignO', [], 'Display', None],
  [ 0xf7d, 'TibetanVowelSignOo', [], 'Display', None],
  [ 0xf7e, 'TibetanSignRjesSuNgaRo', [], 'Display', None],
  [ 0xf7f, 'TibetanSignRnamBcad', [], 'Display', None],
  [ 0xf80, 'TibetanVowelSignReversedI', [], 'Display', None],
  [ 0xf81, 'TibetanVowelSignReversedIi', [], 'Display', None],
  [ 0xf82, 'TibetanSignNyiZlaNaaDa', [], 'Display', None],
  [ 0xf83, 'TibetanSignSnaLdan', [], 'Display', None],
  [ 0xf84, 'TibetanMarkHalanta', [], 'Display', None],
  [ 0xf85, 'TibetanMarkPaluta', [], 'Display', None],
  [ 0xf86, 'TibetanSignLciRtags', [], 'Display', None],
  [ 0xf87, 'TibetanSignYangRtags', [], 'Display', None],
  [ 0xf88, 'TibetanSignLceTsaCan', [], 'Display', None],
  [ 0xf89, 'TibetanSignMchuCan', [], 'Display', None],
  [ 0xf8a, 'TibetanSignGruCanRgyings', [], 'Display', None],
  [ 0xf8b, 'TibetanSignGruMedRgyings', [], 'Display', None],
  [ 0xf8c, 'UnicodeCharacter0xf8c', [], 'Display', None],
  [ 0xf8d, 'UnicodeCharacter0xf8d', [], 'Display', None],
  [ 0xf8e, 'UnicodeCharacter0xf8e', [], 'Display', None],
  [ 0xf8f, 'UnicodeCharacter0xf8f', [], 'Display', None],
  [ 0xf90, 'TibetanSubjoinedLetterKa', [], 'Display', None],
  [ 0xf91, 'TibetanSubjoinedLetterKha', [], 'Display', None],
  [ 0xf92, 'TibetanSubjoinedLetterGa', [], 'Display', None],
  [ 0xf93, 'TibetanSubjoinedLetterGha', [], 'Display', None],
  [ 0xf94, 'TibetanSubjoinedLetterNga', [], 'Display', None],
  [ 0xf95, 'TibetanSubjoinedLetterCa', [], 'Display', None],
  [ 0xf96, 'TibetanSubjoinedLetterCha', [], 'Display', None],
  [ 0xf97, 'TibetanSubjoinedLetterJa', [], 'Display', None],
  [ 0xf98, 'UnicodeCharacter0xf98', [], 'Display', None],
  [ 0xf99, 'TibetanSubjoinedLetterNya', [], 'Display', None],
  [ 0xf9a, 'TibetanSubjoinedLetterTta', [], 'Display', None],
  [ 0xf9b, 'TibetanSubjoinedLetterTtha', [], 'Display', None],
  [ 0xf9c, 'TibetanSubjoinedLetterDda', [], 'Display', None],
  [ 0xf9d, 'TibetanSubjoinedLetterDdha', [], 'Display', None],
  [ 0xf9e, 'TibetanSubjoinedLetterNna', [], 'Display', None],
  [ 0xf9f, 'TibetanSubjoinedLetterTa', [], 'Display', None],
  [ 0xfa0, 'TibetanSubjoinedLetterTha', [], 'Display', None],
  [ 0xfa1, 'TibetanSubjoinedLetterDa', [], 'Display', None],
  [ 0xfa2, 'TibetanSubjoinedLetterDha', [], 'Display', None],
  [ 0xfa3, 'TibetanSubjoinedLetterNa', [], 'Display', None],
  [ 0xfa4, 'TibetanSubjoinedLetterPa', [], 'Display', None],
  [ 0xfa5, 'TibetanSubjoinedLetterPha', [], 'Display', None],
  [ 0xfa6, 'TibetanSubjoinedLetterBa', [], 'Display', None],
  [ 0xfa7, 'TibetanSubjoinedLetterBha', [], 'Display', None],
  [ 0xfa8, 'TibetanSubjoinedLetterMa', [], 'Display', None],
  [ 0xfa9, 'TibetanSubjoinedLetterTsa', [], 'Display', None],
  [ 0xfaa, 'TibetanSubjoinedLetterTsha', [], 'Display', None],
  [ 0xfab, 'TibetanSubjoinedLetterDza', [], 'Display', None],
  [ 0xfac, 'TibetanSubjoinedLetterDzha', [], 'Display', None],
  [ 0xfad, 'TibetanSubjoinedLetterWa', [], 'Display', None],
  [ 0xfae, 'TibetanSubjoinedLetterZha', [], 'Display', None],
  [ 0xfaf, 'TibetanSubjoinedLetterZa', [], 'Display', None],
  [ 0xfb0, 'TibetanSubjoinedLetterA', [], 'Display', None],
  [ 0xfb1, 'TibetanSubjoinedLetterYa', [], 'Display', None],
  [ 0xfb2, 'TibetanSubjoinedLetterRa', [], 'Display', None],
  [ 0xfb3, 'TibetanSubjoinedLetterLa', [], 'Display', None],
  [ 0xfb4, 'TibetanSubjoinedLetterSha', [], 'Display', None],
  [ 0xfb5, 'TibetanSubjoinedLetterSsa', [], 'Display', None],
  [ 0xfb6, 'TibetanSubjoinedLetterSa', [], 'Display', None],
  [ 0xfb7, 'TibetanSubjoinedLetterHa', [], 'Display', None],
  [ 0xfb8, 'TibetanSubjoinedLetterA', [], 'Display', None],
  [ 0xfb9, 'TibetanSubjoinedLetterKssa', [], 'Display', None],
  [ 0xfba, 'TibetanSubjoinedLetterFixedformWa', [], 'Display', None],
  [ 0xfbb, 'TibetanSubjoinedLetterFixedformYa', [], 'Display', None],
  [ 0xfbc, 'TibetanSubjoinedLetterFixedformRa', [], 'Display', None],
  [ 0xfbd, 'UnicodeCharacter0xfbd', [], 'Display', None],
  [ 0xfbe, 'TibetanKuRuKha', [], 'Display', None],
  [ 0xfbf, 'TibetanKuRuKhaBzhiMigCan', [], 'Display', None],
  [ 0xfc0, 'TibetanCantillationSignHeavyBeat', [], 'Display', None],
  [ 0xfc1, 'TibetanCantillationSignLightBeat', [], 'Display', None],
  [ 0xfc2, 'TibetanCantillationSignCangTeu', [], 'Display', None],
  [ 0xfc3, 'TibetanCantillationSignSbubChal', [], 'Display', None],
  [ 0xfc4, 'TibetanSymbolDrilBu', [], 'Display', None],
  [ 0xfc5, 'TibetanSymbolRdoRje', [], 'Display', None],
  [ 0xfc6, 'TibetanSymbolPadmaGdan', [], 'Display', None],
  [ 0xfc7, 'TibetanSymbolRdoRjeRgyaGram', [], 'Display', None],
  [ 0xfc8, 'TibetanSymbolPhurPa', [], 'Display', None],
  [ 0xfc9, 'TibetanSymbolNorBu', [], 'Display', None],
  [ 0xfca, 'TibetanSymbolNorBuNyisKhyil', [], 'Display', None],
  [ 0xfcb, 'TibetanSymbolNorBuGsumKhyil', [], 'Display', None],
  [ 0xfcc, 'TibetanSymbolNorBuBzhiKhyil', [], 'Display', None],
  [ 0xfcd, 'UnicodeCharacter0xfcd', [], 'Display', None],
  [ 0xfce, 'TibetanSignRdelNagRdelDkar', [], 'Display', None],
  [ 0xfcf, 'TibetanSignRdelNagGsum', [], 'Display', None],
  [ 0xfd0, 'TibetanMarkBskaShogGiMgoRgyan', [], 'Display', None],
  [ 0xfd1, 'TibetanMarkMnyamYigGiMgoRgyan', [], 'Display', None],
  [ 0xfd2, 'TibetanMarkNyisTsheg', [], 'Display', None],
  [ 0xfd3, 'TibetanMarkInitialBrdaRnyingYigMgoMdunMa', [], 'Display', None],
  [ 0xfd4, 'TibetanMarkClosingBrdaRnyingYigMgoSgabMa', [], 'Display', None],
  [ 0xfd5, 'UnicodeCharacter0xfd5', [], 'Display', None],
  [ 0xfd6, 'UnicodeCharacter0xfd6', [], 'Display', None],
  [ 0xfd7, 'UnicodeCharacter0xfd7', [], 'Display', None],
  [ 0xfd8, 'UnicodeCharacter0xfd8', [], 'Display', None],
  [ 0xfd9, 'UnicodeCharacter0xfd9', [], 'Display', None],
  [ 0xfda, 'UnicodeCharacter0xfda', [], 'Display', None],
  [ 0xfdb, 'UnicodeCharacter0xfdb', [], 'Display', None],
  [ 0xfdc, 'UnicodeCharacter0xfdc', [], 'Display', None],
  [ 0xfdd, 'UnicodeCharacter0xfdd', [], 'Display', None],
  [ 0xfde, 'UnicodeCharacter0xfde', [], 'Display', None],
  [ 0xfdf, 'UnicodeCharacter0xfdf', [], 'Display', None],
  [ 0xfe0, 'UnicodeCharacter0xfe0', [], 'Display', None],
  [ 0xfe1, 'UnicodeCharacter0xfe1', [], 'Display', None],
  [ 0xfe2, 'UnicodeCharacter0xfe2', [], 'Display', None],
  [ 0xfe3, 'UnicodeCharacter0xfe3', [], 'Display', None],
  [ 0xfe4, 'UnicodeCharacter0xfe4', [], 'Display', None],
  [ 0xfe5, 'UnicodeCharacter0xfe5', [], 'Display', None],
  [ 0xfe6, 'UnicodeCharacter0xfe6', [], 'Display', None],
  [ 0xfe7, 'UnicodeCharacter0xfe7', [], 'Display', None],
  [ 0xfe8, 'UnicodeCharacter0xfe8', [], 'Display', None],
  [ 0xfe9, 'UnicodeCharacter0xfe9', [], 'Display', None],
  [ 0xfea, 'UnicodeCharacter0xfea', [], 'Display', None],
  [ 0xfeb, 'UnicodeCharacter0xfeb', [], 'Display', None],
  [ 0xfec, 'UnicodeCharacter0xfec', [], 'Display', None],
  [ 0xfed, 'UnicodeCharacter0xfed', [], 'Display', None],
  [ 0xfee, 'UnicodeCharacter0xfee', [], 'Display', None],
  [ 0xfef, 'UnicodeCharacter0xfef', [], 'Display', None],
  [ 0xff0, 'UnicodeCharacter0xff0', [], 'Display', None],
  [ 0xff1, 'UnicodeCharacter0xff1', [], 'Display', None],
  [ 0xff2, 'UnicodeCharacter0xff2', [], 'Display', None],
  [ 0xff3, 'UnicodeCharacter0xff3', [], 'Display', None],
  [ 0xff4, 'UnicodeCharacter0xff4', [], 'Display', None],
  [ 0xff5, 'UnicodeCharacter0xff5', [], 'Display', None],
  [ 0xff6, 'UnicodeCharacter0xff6', [], 'Display', None],
  [ 0xff7, 'UnicodeCharacter0xff7', [], 'Display', None],
  [ 0xff8, 'UnicodeCharacter0xff8', [], 'Display', None],
  [ 0xff9, 'UnicodeCharacter0xff9', [], 'Display', None],
  [ 0xffa, 'UnicodeCharacter0xffa', [], 'Display', None],
  [ 0xffb, 'UnicodeCharacter0xffb', [], 'Display', None],
  [ 0xffc, 'UnicodeCharacter0xffc', [], 'Display', None],
  [ 0xffd, 'UnicodeCharacter0xffd', [], 'Display', None],
  [ 0xffe, 'UnicodeCharacter0xffe', [], 'Display', None],
  [ 0xfff, 'UnicodeCharacter0xfff', [], 'Display', None],
  [ 0x1000, 'MyanmarLetterKa', [], 'Display', None],
  [ 0x1001, 'MyanmarLetterKha', [], 'Display', None],
  [ 0x1002, 'MyanmarLetterGa', [], 'Display', None],
  [ 0x1003, 'MyanmarLetterGha', [], 'Display', None],
  [ 0x1004, 'MyanmarLetterNga', [], 'Display', None],
  [ 0x1005, 'MyanmarLetterCa', [], 'Display', None],
  [ 0x1006, 'MyanmarLetterCha', [], 'Display', None],
  [ 0x1007, 'MyanmarLetterJa', [], 'Display', None],
  [ 0x1008, 'MyanmarLetterJha', [], 'Display', None],
  [ 0x1009, 'MyanmarLetterNya', [], 'Display', None],
  [ 0x100a, 'MyanmarLetterNnya', [], 'Display', None],
  [ 0x100b, 'MyanmarLetterTta', [], 'Display', None],
  [ 0x100c, 'MyanmarLetterTtha', [], 'Display', None],
  [ 0x100d, 'MyanmarLetterDda', [], 'Display', None],
  [ 0x100e, 'MyanmarLetterDdha', [], 'Display', None],
  [ 0x100f, 'MyanmarLetterNna', [], 'Display', None],
  [ 0x1010, 'MyanmarLetterTa', [], 'Display', None],
  [ 0x1011, 'MyanmarLetterTha', [], 'Display', None],
  [ 0x1012, 'MyanmarLetterDa', [], 'Display', None],
  [ 0x1013, 'MyanmarLetterDha', [], 'Display', None],
  [ 0x1014, 'MyanmarLetterNa', [], 'Display', None],
  [ 0x1015, 'MyanmarLetterPa', [], 'Display', None],
  [ 0x1016, 'MyanmarLetterPha', [], 'Display', None],
  [ 0x1017, 'MyanmarLetterBa', [], 'Display', None],
  [ 0x1018, 'MyanmarLetterBha', [], 'Display', None],
  [ 0x1019, 'MyanmarLetterMa', [], 'Display', None],
  [ 0x101a, 'MyanmarLetterYa', [], 'Display', None],
  [ 0x101b, 'MyanmarLetterRa', [], 'Display', None],
  [ 0x101c, 'MyanmarLetterLa', [], 'Display', None],
  [ 0x101d, 'MyanmarLetterWa', [], 'Display', None],
  [ 0x101e, 'MyanmarLetterSa', [], 'Display', None],
  [ 0x101f, 'MyanmarLetterHa', [], 'Display', None],
  [ 0x1020, 'MyanmarLetterLla', [], 'Display', None],
  [ 0x1021, 'MyanmarLetterA', [], 'Display', None],
  [ 0x1022, 'MyanmarLetterShanA', [], 'Display', None],
  [ 0x1023, 'MyanmarLetterI', [], 'Display', None],
  [ 0x1024, 'MyanmarLetterIi', [], 'Display', None],
  [ 0x1025, 'MyanmarLetterU', [], 'Display', None],
  [ 0x1026, 'MyanmarLetterUu', [], 'Display', None],
  [ 0x1027, 'MyanmarLetterE', [], 'Display', None],
  [ 0x1028, 'MyanmarLetterMonE', [], 'Display', None],
  [ 0x1029, 'MyanmarLetterO', [], 'Display', None],
  [ 0x102a, 'MyanmarLetterAu', [], 'Display', None],
  [ 0x102b, 'MyanmarVowelSignTallAa', [], 'Display', None],
  [ 0x102c, 'MyanmarVowelSignAa', [], 'Display', None],
  [ 0x102d, 'MyanmarVowelSignI', [], 'Display', None],
  [ 0x102e, 'MyanmarVowelSignIi', [], 'Display', None],
  [ 0x102f, 'MyanmarVowelSignU', [], 'Display', None],
  [ 0x1030, 'MyanmarVowelSignUu', [], 'Display', None],
  [ 0x1031, 'MyanmarVowelSignE', [], 'Display', None],
  [ 0x1032, 'MyanmarVowelSignAi', [], 'Display', None],
  [ 0x1033, 'MyanmarVowelSignMonIi', [], 'Display', None],
  [ 0x1034, 'MyanmarVowelSignMonO', [], 'Display', None],
  [ 0x1035, 'MyanmarVowelSignEAbove', [], 'Display', None],
  [ 0x1036, 'MyanmarSignAnusvara', [], 'Display', None],
  [ 0x1037, 'MyanmarSignDotBelow', [], 'Display', None],
  [ 0x1038, 'MyanmarSignVisarga', [], 'Display', None],
  [ 0x1039, 'MyanmarSignVirama', [], 'Display', None],
  [ 0x103a, 'MyanmarSignAsat', [], 'Display', None],
  [ 0x103b, 'MyanmarConsonantSignMedialYa', [], 'Display', None],
  [ 0x103c, 'MyanmarConsonantSignMedialRa', [], 'Display', None],
  [ 0x103d, 'MyanmarConsonantSignMedialWa', [], 'Display', None],
  [ 0x103e, 'MyanmarConsonantSignMedialHa', [], 'Display', None],
  [ 0x103f, 'MyanmarLetterGreatSa', [], 'Display', None],
  [ 0x1040, 'MyanmarDigitZero', [], 'Display', None],
  [ 0x1041, 'MyanmarDigitOne', [], 'Display', None],
  [ 0x1042, 'MyanmarDigitTwo', [], 'Display', None],
  [ 0x1043, 'MyanmarDigitThree', [], 'Display', None],
  [ 0x1044, 'MyanmarDigitFour', [], 'Display', None],
  [ 0x1045, 'MyanmarDigitFive', [], 'Display', None],
  [ 0x1046, 'MyanmarDigitSix', [], 'Display', None],
  [ 0x1047, 'MyanmarDigitSeven', [], 'Display', None],
  [ 0x1048, 'MyanmarDigitEight', [], 'Display', None],
  [ 0x1049, 'MyanmarDigitNine', [], 'Display', None],
  [ 0x104a, 'MyanmarSignLittleSection', [], 'Display', None],
  [ 0x104b, 'MyanmarSignSection', [], 'Display', None],
  [ 0x104c, 'MyanmarSymbolLocative', [], 'Display', None],
  [ 0x104d, 'MyanmarSymbolCompleted', [], 'Display', None],
  [ 0x104e, 'MyanmarSymbolAforementioned', [], 'Display', None],
  [ 0x104f, 'MyanmarSymbolGenitive', [], 'Display', None],
  [ 0x1050, 'MyanmarLetterSha', [], 'Display', None],
  [ 0x1051, 'MyanmarLetterSsa', [], 'Display', None],
  [ 0x1052, 'MyanmarLetterVocalicR', [], 'Display', None],
  [ 0x1053, 'MyanmarLetterVocalicRr', [], 'Display', None],
  [ 0x1054, 'MyanmarLetterVocalicL', [], 'Display', None],
  [ 0x1055, 'MyanmarLetterVocalicLl', [], 'Display', None],
  [ 0x1056, 'MyanmarVowelSignVocalicR', [], 'Display', None],
  [ 0x1057, 'MyanmarVowelSignVocalicRr', [], 'Display', None],
  [ 0x1058, 'MyanmarVowelSignVocalicL', [], 'Display', None],
  [ 0x1059, 'MyanmarVowelSignVocalicLl', [], 'Display', None],
  [ 0x105a, 'MyanmarLetterMonNga', [], 'Display', None],
  [ 0x105b, 'MyanmarLetterMonJha', [], 'Display', None],
  [ 0x105c, 'MyanmarLetterMonBba', [], 'Display', None],
  [ 0x105d, 'MyanmarLetterMonBbe', [], 'Display', None],
  [ 0x105e, 'MyanmarConsonantSignMonMedialNa', [], 'Display', None],
  [ 0x105f, 'MyanmarConsonantSignMonMedialMa', [], 'Display', None],
  [ 0x1060, 'MyanmarConsonantSignMonMedialLa', [], 'Display', None],
  [ 0x1061, 'MyanmarLetterSgawKarenSha', [], 'Display', None],
  [ 0x1062, 'MyanmarVowelSignSgawKarenEu', [], 'Display', None],
  [ 0x1063, 'MyanmarToneMarkSgawKarenHathi', [], 'Display', None],
  [ 0x1064, 'MyanmarToneMarkSgawKarenKePho', [], 'Display', None],
  [ 0x1065, 'MyanmarLetterWesternPwoKarenTha', [], 'Display', None],
  [ 0x1066, 'MyanmarLetterWesternPwoKarenPwa', [], 'Display', None],
  [ 0x1067, 'MyanmarVowelSignWesternPwoKarenEu', [], 'Display', None],
  [ 0x1068, 'MyanmarVowelSignWesternPwoKarenUe', [], 'Display', None],
  [ 0x1069, 'MyanmarSignWesternPwoKarenTone1', [], 'Display', None],
  [ 0x106a, 'MyanmarSignWesternPwoKarenTone2', [], 'Display', None],
  [ 0x106b, 'MyanmarSignWesternPwoKarenTone3', [], 'Display', None],
  [ 0x106c, 'MyanmarSignWesternPwoKarenTone4', [], 'Display', None],
  [ 0x106d, 'MyanmarSignWesternPwoKarenTone5', [], 'Display', None],
  [ 0x106e, 'MyanmarLetterEasternPwoKarenNna', [], 'Display', None],
  [ 0x106f, 'MyanmarLetterEasternPwoKarenYwa', [], 'Display', None],
  [ 0x1070, 'MyanmarLetterEasternPwoKarenGhwa', [], 'Display', None],
  [ 0x1071, 'MyanmarVowelSignGebaKarenI', [], 'Display', None],
  [ 0x1072, 'MyanmarVowelSignKayahOe', [], 'Display', None],
  [ 0x1073, 'MyanmarVowelSignKayahU', [], 'Display', None],
  [ 0x1074, 'MyanmarVowelSignKayahEe', [], 'Display', None],
  [ 0x1075, 'MyanmarLetterShanKa', [], 'Display', None],
  [ 0x1076, 'MyanmarLetterShanKha', [], 'Display', None],
  [ 0x1077, 'MyanmarLetterShanGa', [], 'Display', None],
  [ 0x1078, 'MyanmarLetterShanCa', [], 'Display', None],
  [ 0x1079, 'MyanmarLetterShanZa', [], 'Display', None],
  [ 0x107a, 'MyanmarLetterShanNya', [], 'Display', None],
  [ 0x107b, 'MyanmarLetterShanDa', [], 'Display', None],
  [ 0x107c, 'MyanmarLetterShanNa', [], 'Display', None],
  [ 0x107d, 'MyanmarLetterShanPha', [], 'Display', None],
  [ 0x107e, 'MyanmarLetterShanFa', [], 'Display', None],
  [ 0x107f, 'MyanmarLetterShanBa', [], 'Display', None],
  [ 0x1080, 'MyanmarLetterShanTha', [], 'Display', None],
  [ 0x1081, 'MyanmarLetterShanHa', [], 'Display', None],
  [ 0x1082, 'MyanmarConsonantSignShanMedialWa', [], 'Display', None],
  [ 0x1083, 'MyanmarVowelSignShanAa', [], 'Display', None],
  [ 0x1084, 'MyanmarVowelSignShanE', [], 'Display', None],
  [ 0x1085, 'MyanmarVowelSignShanEAbove', [], 'Display', None],
  [ 0x1086, 'MyanmarVowelSignShanFinalY', [], 'Display', None],
  [ 0x1087, 'MyanmarSignShanTone2', [], 'Display', None],
  [ 0x1088, 'MyanmarSignShanTone3', [], 'Display', None],
  [ 0x1089, 'MyanmarSignShanTone5', [], 'Display', None],
  [ 0x108a, 'MyanmarSignShanTone6', [], 'Display', None],
  [ 0x108b, 'MyanmarSignShanCouncilTone2', [], 'Display', None],
  [ 0x108c, 'MyanmarSignShanCouncilTone3', [], 'Display', None],
  [ 0x108d, 'MyanmarSignShanCouncilEmphaticTone', [], 'Display', None],
  [ 0x108e, 'MyanmarLetterRumaiPalaungFa', [], 'Display', None],
  [ 0x108f, 'MyanmarSignRumaiPalaungTone5', [], 'Display', None],
  [ 0x1090, 'MyanmarShanDigitZero', [], 'Display', None],
  [ 0x1091, 'MyanmarShanDigitOne', [], 'Display', None],
  [ 0x1092, 'MyanmarShanDigitTwo', [], 'Display', None],
  [ 0x1093, 'MyanmarShanDigitThree', [], 'Display', None],
  [ 0x1094, 'MyanmarShanDigitFour', [], 'Display', None],
  [ 0x1095, 'MyanmarShanDigitFive', [], 'Display', None],
  [ 0x1096, 'MyanmarShanDigitSix', [], 'Display', None],
  [ 0x1097, 'MyanmarShanDigitSeven', [], 'Display', None],
  [ 0x1098, 'MyanmarShanDigitEight', [], 'Display', None],
  [ 0x1099, 'MyanmarShanDigitNine', [], 'Display', None],
  [ 0x109a, 'UnicodeCharacter0x109a', [], 'Display', None],
  [ 0x109b, 'UnicodeCharacter0x109b', [], 'Display', None],
  [ 0x109c, 'UnicodeCharacter0x109c', [], 'Display', None],
  [ 0x109d, 'UnicodeCharacter0x109d', [], 'Display', None],
  [ 0x109e, 'MyanmarSymbolShanOne', [], 'Display', None],
  [ 0x109f, 'MyanmarSymbolShanExclamation', [], 'Display', None],
  [ 0x10a0, 'GeorgianCapitalLetterAn', [], 'Display', None],
  [ 0x10a1, 'GeorgianCapitalLetterBan', [], 'Display', None],
  [ 0x10a2, 'GeorgianCapitalLetterGan', [], 'Display', None],
  [ 0x10a3, 'GeorgianCapitalLetterDon', [], 'Display', None],
  [ 0x10a4, 'GeorgianCapitalLetterEn', [], 'Display', None],
  [ 0x10a5, 'GeorgianCapitalLetterVin', [], 'Display', None],
  [ 0x10a6, 'GeorgianCapitalLetterZen', [], 'Display', None],
  [ 0x10a7, 'GeorgianCapitalLetterTan', [], 'Display', None],
  [ 0x10a8, 'GeorgianCapitalLetterIn', [], 'Display', None],
  [ 0x10a9, 'GeorgianCapitalLetterKan', [], 'Display', None],
  [ 0x10aa, 'GeorgianCapitalLetterLas', [], 'Display', None],
  [ 0x10ab, 'GeorgianCapitalLetterMan', [], 'Display', None],
  [ 0x10ac, 'GeorgianCapitalLetterNar', [], 'Display', None],
  [ 0x10ad, 'GeorgianCapitalLetterOn', [], 'Display', None],
  [ 0x10ae, 'GeorgianCapitalLetterPar', [], 'Display', None],
  [ 0x10af, 'GeorgianCapitalLetterZhar', [], 'Display', None],
  [ 0x10b0, 'GeorgianCapitalLetterRae', [], 'Display', None],
  [ 0x10b1, 'GeorgianCapitalLetterSan', [], 'Display', None],
  [ 0x10b2, 'GeorgianCapitalLetterTar', [], 'Display', None],
  [ 0x10b3, 'GeorgianCapitalLetterUn', [], 'Display', None],
  [ 0x10b4, 'GeorgianCapitalLetterPhar', [], 'Display', None],
  [ 0x10b5, 'GeorgianCapitalLetterKhar', [], 'Display', None],
  [ 0x10b6, 'GeorgianCapitalLetterGhan', [], 'Display', None],
  [ 0x10b7, 'GeorgianCapitalLetterQar', [], 'Display', None],
  [ 0x10b8, 'GeorgianCapitalLetterShin', [], 'Display', None],
  [ 0x10b9, 'GeorgianCapitalLetterChin', [], 'Display', None],
  [ 0x10ba, 'GeorgianCapitalLetterCan', [], 'Display', None],
  [ 0x10bb, 'GeorgianCapitalLetterJil', [], 'Display', None],
  [ 0x10bc, 'GeorgianCapitalLetterCil', [], 'Display', None],
  [ 0x10bd, 'GeorgianCapitalLetterChar', [], 'Display', None],
  [ 0x10be, 'GeorgianCapitalLetterXan', [], 'Display', None],
  [ 0x10bf, 'GeorgianCapitalLetterJhan', [], 'Display', None],
  [ 0x10c0, 'GeorgianCapitalLetterHae', [], 'Display', None],
  [ 0x10c1, 'GeorgianCapitalLetterHe', [], 'Display', None],
  [ 0x10c2, 'GeorgianCapitalLetterHie', [], 'Display', None],
  [ 0x10c3, 'GeorgianCapitalLetterWe', [], 'Display', None],
  [ 0x10c4, 'GeorgianCapitalLetterHar', [], 'Display', None],
  [ 0x10c5, 'GeorgianCapitalLetterHoe', [], 'Display', None],
  [ 0x10c6, 'UnicodeCharacter0x10c6', [], 'Display', None],
  [ 0x10c7, 'UnicodeCharacter0x10c7', [], 'Display', None],
  [ 0x10c8, 'UnicodeCharacter0x10c8', [], 'Display', None],
  [ 0x10c9, 'UnicodeCharacter0x10c9', [], 'Display', None],
  [ 0x10ca, 'UnicodeCharacter0x10ca', [], 'Display', None],
  [ 0x10cb, 'UnicodeCharacter0x10cb', [], 'Display', None],
  [ 0x10cc, 'UnicodeCharacter0x10cc', [], 'Display', None],
  [ 0x10cd, 'UnicodeCharacter0x10cd', [], 'Display', None],
  [ 0x10ce, 'UnicodeCharacter0x10ce', [], 'Display', None],
  [ 0x10cf, 'UnicodeCharacter0x10cf', [], 'Display', None],
  [ 0x10d0, 'GeorgianLetterAn', [], 'Display', None],
  [ 0x10d1, 'GeorgianLetterBan', [], 'Display', None],
  [ 0x10d2, 'GeorgianLetterGan', [], 'Display', None],
  [ 0x10d3, 'GeorgianLetterDon', [], 'Display', None],
  [ 0x10d4, 'GeorgianLetterEn', [], 'Display', None],
  [ 0x10d5, 'GeorgianLetterVin', [], 'Display', None],
  [ 0x10d6, 'GeorgianLetterZen', [], 'Display', None],
  [ 0x10d7, 'GeorgianLetterTan', [], 'Display', None],
  [ 0x10d8, 'GeorgianLetterIn', [], 'Display', None],
  [ 0x10d9, 'GeorgianLetterKan', [], 'Display', None],
  [ 0x10da, 'GeorgianLetterLas', [], 'Display', None],
  [ 0x10db, 'GeorgianLetterMan', [], 'Display', None],
  [ 0x10dc, 'GeorgianLetterNar', [], 'Display', None],
  [ 0x10dd, 'GeorgianLetterOn', [], 'Display', None],
  [ 0x10de, 'GeorgianLetterPar', [], 'Display', None],
  [ 0x10df, 'GeorgianLetterZhar', [], 'Display', None],
  [ 0x10e0, 'GeorgianLetterRae', [], 'Display', None],
  [ 0x10e1, 'GeorgianLetterSan', [], 'Display', None],
  [ 0x10e2, 'GeorgianLetterTar', [], 'Display', None],
  [ 0x10e3, 'GeorgianLetterUn', [], 'Display', None],
  [ 0x10e4, 'GeorgianLetterPhar', [], 'Display', None],
  [ 0x10e5, 'GeorgianLetterKhar', [], 'Display', None],
  [ 0x10e6, 'GeorgianLetterGhan', [], 'Display', None],
  [ 0x10e7, 'GeorgianLetterQar', [], 'Display', None],
  [ 0x10e8, 'GeorgianLetterShin', [], 'Display', None],
  [ 0x10e9, 'GeorgianLetterChin', [], 'Display', None],
  [ 0x10ea, 'GeorgianLetterCan', [], 'Display', None],
  [ 0x10eb, 'GeorgianLetterJil', [], 'Display', None],
  [ 0x10ec, 'GeorgianLetterCil', [], 'Display', None],
  [ 0x10ed, 'GeorgianLetterChar', [], 'Display', None],
  [ 0x10ee, 'GeorgianLetterXan', [], 'Display', None],
  [ 0x10ef, 'GeorgianLetterJhan', [], 'Display', None],
  [ 0x10f0, 'GeorgianLetterHae', [], 'Display', None],
  [ 0x10f1, 'GeorgianLetterHe', [], 'Display', None],
  [ 0x10f2, 'GeorgianLetterHie', [], 'Display', None],
  [ 0x10f3, 'GeorgianLetterWe', [], 'Display', None],
  [ 0x10f4, 'GeorgianLetterHar', [], 'Display', None],
  [ 0x10f5, 'GeorgianLetterHoe', [], 'Display', None],
  [ 0x10f6, 'GeorgianLetterFi', [], 'Display', None],
  [ 0x10f7, 'GeorgianLetterYn', [], 'Display', None],
  [ 0x10f8, 'GeorgianLetterElifi', [], 'Display', None],
  [ 0x10f9, 'GeorgianLetterTurnedGan', [], 'Display', None],
  [ 0x10fa, 'GeorgianLetterAin', [], 'Display', None],
  [ 0x10fb, 'GeorgianParagraphSeparator', [], 'Display', None],
  [ 0x10fc, 'ModifierLetterGeorgianNar', [], 'Display', None],
  [ 0x10fd, 'UnicodeCharacter0x10fd', [], 'Display', None],
  [ 0x10fe, 'UnicodeCharacter0x10fe', [], 'Display', None],
  [ 0x10ff, 'UnicodeCharacter0x10ff', [], 'Display', None],
  [ 0x1100, 'HangulChoseongKiyeok', [], 'Display', None],
  [ 0x1101, 'HangulChoseongSsangkiyeok', [], 'Display', None],
  [ 0x1102, 'HangulChoseongNieun', [], 'Display', None],
  [ 0x1103, 'HangulChoseongTikeut', [], 'Display', None],
  [ 0x1104, 'HangulChoseongSsangtikeut', [], 'Display', None],
  [ 0x1105, 'HangulChoseongRieul', [], 'Display', None],
  [ 0x1106, 'HangulChoseongMieum', [], 'Display', None],
  [ 0x1107, 'HangulChoseongPieup', [], 'Display', None],
  [ 0x1108, 'HangulChoseongSsangpieup', [], 'Display', None],
  [ 0x1109, 'HangulChoseongSios', [], 'Display', None],
  [ 0x110a, 'HangulChoseongSsangsios', [], 'Display', None],
  [ 0x110b, 'HangulChoseongIeung', [], 'Display', None],
  [ 0x110c, 'HangulChoseongCieuc', [], 'Display', None],
  [ 0x110d, 'HangulChoseongSsangcieuc', [], 'Display', None],
  [ 0x110e, 'HangulChoseongChieuch', [], 'Display', None],
  [ 0x110f, 'HangulChoseongKhieukh', [], 'Display', None],
  [ 0x1110, 'HangulChoseongThieuth', [], 'Display', None],
  [ 0x1111, 'HangulChoseongPhieuph', [], 'Display', None],
  [ 0x1112, 'HangulChoseongHieuh', [], 'Display', None],
  [ 0x1113, 'HangulChoseongNieunkiyeok', [], 'Display', None],
  [ 0x1114, 'HangulChoseongSsangnieun', [], 'Display', None],
  [ 0x1115, 'HangulChoseongNieuntikeut', [], 'Display', None],
  [ 0x1116, 'HangulChoseongNieunpieup', [], 'Display', None],
  [ 0x1117, 'HangulChoseongTikeutkiyeok', [], 'Display', None],
  [ 0x1118, 'HangulChoseongRieulnieun', [], 'Display', None],
  [ 0x1119, 'HangulChoseongSsangrieul', [], 'Display', None],
  [ 0x111a, 'HangulChoseongRieulhieuh', [], 'Display', None],
  [ 0x111b, 'HangulChoseongKapyeounrieul', [], 'Display', None],
  [ 0x111c, 'HangulChoseongMieumpieup', [], 'Display', None],
  [ 0x111d, 'HangulChoseongKapyeounmieum', [], 'Display', None],
  [ 0x111e, 'HangulChoseongPieupkiyeok', [], 'Display', None],
  [ 0x111f, 'HangulChoseongPieupnieun', [], 'Display', None],
  [ 0x1120, 'HangulChoseongPieuptikeut', [], 'Display', None],
  [ 0x1121, 'HangulChoseongPieupsios', [], 'Display', None],
  [ 0x1122, 'HangulChoseongPieupsioskiyeok', [], 'Display', None],
  [ 0x1123, 'HangulChoseongPieupsiostikeut', [], 'Display', None],
  [ 0x1124, 'HangulChoseongPieupsiospieup', [], 'Display', None],
  [ 0x1125, 'HangulChoseongPieupssangsios', [], 'Display', None],
  [ 0x1126, 'HangulChoseongPieupsioscieuc', [], 'Display', None],
  [ 0x1127, 'HangulChoseongPieupcieuc', [], 'Display', None],
  [ 0x1128, 'HangulChoseongPieupchieuch', [], 'Display', None],
  [ 0x1129, 'HangulChoseongPieupthieuth', [], 'Display', None],
  [ 0x112a, 'HangulChoseongPieupphieuph', [], 'Display', None],
  [ 0x112b, 'HangulChoseongKapyeounpieup', [], 'Display', None],
  [ 0x112c, 'HangulChoseongKapyeounssangpieup', [], 'Display', None],
  [ 0x112d, 'HangulChoseongSioskiyeok', [], 'Display', None],
  [ 0x112e, 'HangulChoseongSiosnieun', [], 'Display', None],
  [ 0x112f, 'HangulChoseongSiostikeut', [], 'Display', None],
  [ 0x1130, 'HangulChoseongSiosrieul', [], 'Display', None],
  [ 0x1131, 'HangulChoseongSiosmieum', [], 'Display', None],
  [ 0x1132, 'HangulChoseongSiospieup', [], 'Display', None],
  [ 0x1133, 'HangulChoseongSiospieupkiyeok', [], 'Display', None],
  [ 0x1134, 'HangulChoseongSiosssangsios', [], 'Display', None],
  [ 0x1135, 'HangulChoseongSiosieung', [], 'Display', None],
  [ 0x1136, 'HangulChoseongSioscieuc', [], 'Display', None],
  [ 0x1137, 'HangulChoseongSioschieuch', [], 'Display', None],
  [ 0x1138, 'HangulChoseongSioskhieukh', [], 'Display', None],
  [ 0x1139, 'HangulChoseongSiosthieuth', [], 'Display', None],
  [ 0x113a, 'HangulChoseongSiosphieuph', [], 'Display', None],
  [ 0x113b, 'HangulChoseongSioshieuh', [], 'Display', None],
  [ 0x113c, 'HangulChoseongChitueumsios', [], 'Display', None],
  [ 0x113d, 'HangulChoseongChitueumssangsios', [], 'Display', None],
  [ 0x113e, 'HangulChoseongCeongchieumsios', [], 'Display', None],
  [ 0x113f, 'HangulChoseongCeongchieumssangsios', [], 'Display', None],
  [ 0x1140, 'HangulChoseongPansios', [], 'Display', None],
  [ 0x1141, 'HangulChoseongIeungkiyeok', [], 'Display', None],
  [ 0x1142, 'HangulChoseongIeungtikeut', [], 'Display', None],
  [ 0x1143, 'HangulChoseongIeungmieum', [], 'Display', None],
  [ 0x1144, 'HangulChoseongIeungpieup', [], 'Display', None],
  [ 0x1145, 'HangulChoseongIeungsios', [], 'Display', None],
  [ 0x1146, 'HangulChoseongIeungpansios', [], 'Display', None],
  [ 0x1147, 'HangulChoseongSsangieung', [], 'Display', None],
  [ 0x1148, 'HangulChoseongIeungcieuc', [], 'Display', None],
  [ 0x1149, 'HangulChoseongIeungchieuch', [], 'Display', None],
  [ 0x114a, 'HangulChoseongIeungthieuth', [], 'Display', None],
  [ 0x114b, 'HangulChoseongIeungphieuph', [], 'Display', None],
  [ 0x114c, 'HangulChoseongYesieung', [], 'Display', None],
  [ 0x114d, 'HangulChoseongCieucieung', [], 'Display', None],
  [ 0x114e, 'HangulChoseongChitueumcieuc', [], 'Display', None],
  [ 0x114f, 'HangulChoseongChitueumssangcieuc', [], 'Display', None],
  [ 0x1150, 'HangulChoseongCeongchieumcieuc', [], 'Display', None],
  [ 0x1151, 'HangulChoseongCeongchieumssangcieuc', [], 'Display', None],
  [ 0x1152, 'HangulChoseongChieuchkhieukh', [], 'Display', None],
  [ 0x1153, 'HangulChoseongChieuchhieuh', [], 'Display', None],
  [ 0x1154, 'HangulChoseongChitueumchieuch', [], 'Display', None],
  [ 0x1155, 'HangulChoseongCeongchieumchieuch', [], 'Display', None],
  [ 0x1156, 'HangulChoseongPhieuphpieup', [], 'Display', None],
  [ 0x1157, 'HangulChoseongKapyeounphieuph', [], 'Display', None],
  [ 0x1158, 'HangulChoseongSsanghieuh', [], 'Display', None],
  [ 0x1159, 'HangulChoseongYeorinhieuh', [], 'Display', None],
  [ 0x115a, 'UnicodeCharacter0x115a', [], 'Display', None],
  [ 0x115b, 'UnicodeCharacter0x115b', [], 'Display', None],
  [ 0x115c, 'UnicodeCharacter0x115c', [], 'Display', None],
  [ 0x115d, 'UnicodeCharacter0x115d', [], 'Display', None],
  [ 0x115e, 'UnicodeCharacter0x115e', [], 'Display', None],
  [ 0x115f, 'HangulChoseongFiller', [], 'Display', None],
  [ 0x1160, 'HangulJungseongFiller', [], 'Display', None],
  [ 0x1161, 'HangulJungseongA', [], 'Display', None],
  [ 0x1162, 'HangulJungseongAe', [], 'Display', None],
  [ 0x1163, 'HangulJungseongYa', [], 'Display', None],
  [ 0x1164, 'HangulJungseongYae', [], 'Display', None],
  [ 0x1165, 'HangulJungseongEo', [], 'Display', None],
  [ 0x1166, 'HangulJungseongE', [], 'Display', None],
  [ 0x1167, 'HangulJungseongYeo', [], 'Display', None],
  [ 0x1168, 'HangulJungseongYe', [], 'Display', None],
  [ 0x1169, 'HangulJungseongO', [], 'Display', None],
  [ 0x116a, 'HangulJungseongWa', [], 'Display', None],
  [ 0x116b, 'HangulJungseongWae', [], 'Display', None],
  [ 0x116c, 'HangulJungseongOe', [], 'Display', None],
  [ 0x116d, 'HangulJungseongYo', [], 'Display', None],
  [ 0x116e, 'HangulJungseongU', [], 'Display', None],
  [ 0x116f, 'HangulJungseongWeo', [], 'Display', None],
  [ 0x1170, 'HangulJungseongWe', [], 'Display', None],
  [ 0x1171, 'HangulJungseongWi', [], 'Display', None],
  [ 0x1172, 'HangulJungseongYu', [], 'Display', None],
  [ 0x1173, 'HangulJungseongEu', [], 'Display', None],
  [ 0x1174, 'HangulJungseongYi', [], 'Display', None],
  [ 0x1175, 'HangulJungseongI', [], 'Display', None],
  [ 0x1176, 'HangulJungseongAo', [], 'Display', None],
  [ 0x1177, 'HangulJungseongAu', [], 'Display', None],
  [ 0x1178, 'HangulJungseongYao', [], 'Display', None],
  [ 0x1179, 'HangulJungseongYayo', [], 'Display', None],
  [ 0x117a, 'HangulJungseongEoo', [], 'Display', None],
  [ 0x117b, 'HangulJungseongEou', [], 'Display', None],
  [ 0x117c, 'HangulJungseongEoeu', [], 'Display', None],
  [ 0x117d, 'HangulJungseongYeoo', [], 'Display', None],
  [ 0x117e, 'HangulJungseongYeou', [], 'Display', None],
  [ 0x117f, 'HangulJungseongOeo', [], 'Display', None],
  [ 0x1180, 'HangulJungseongOe', [], 'Display', None],
  [ 0x1181, 'HangulJungseongOye', [], 'Display', None],
  [ 0x1182, 'HangulJungseongOo', [], 'Display', None],
  [ 0x1183, 'HangulJungseongOu', [], 'Display', None],
  [ 0x1184, 'HangulJungseongYoya', [], 'Display', None],
  [ 0x1185, 'HangulJungseongYoyae', [], 'Display', None],
  [ 0x1186, 'HangulJungseongYoyeo', [], 'Display', None],
  [ 0x1187, 'HangulJungseongYoo', [], 'Display', None],
  [ 0x1188, 'HangulJungseongYoi', [], 'Display', None],
  [ 0x1189, 'HangulJungseongUa', [], 'Display', None],
  [ 0x118a, 'HangulJungseongUae', [], 'Display', None],
  [ 0x118b, 'HangulJungseongUeoeu', [], 'Display', None],
  [ 0x118c, 'HangulJungseongUye', [], 'Display', None],
  [ 0x118d, 'HangulJungseongUu', [], 'Display', None],
  [ 0x118e, 'HangulJungseongYua', [], 'Display', None],
  [ 0x118f, 'HangulJungseongYueo', [], 'Display', None],
  [ 0x1190, 'HangulJungseongYue', [], 'Display', None],
  [ 0x1191, 'HangulJungseongYuyeo', [], 'Display', None],
  [ 0x1192, 'HangulJungseongYuye', [], 'Display', None],
  [ 0x1193, 'HangulJungseongYuu', [], 'Display', None],
  [ 0x1194, 'HangulJungseongYui', [], 'Display', None],
  [ 0x1195, 'HangulJungseongEuu', [], 'Display', None],
  [ 0x1196, 'HangulJungseongEueu', [], 'Display', None],
  [ 0x1197, 'HangulJungseongYiu', [], 'Display', None],
  [ 0x1198, 'HangulJungseongIa', [], 'Display', None],
  [ 0x1199, 'HangulJungseongIya', [], 'Display', None],
  [ 0x119a, 'HangulJungseongIo', [], 'Display', None],
  [ 0x119b, 'HangulJungseongIu', [], 'Display', None],
  [ 0x119c, 'HangulJungseongIeu', [], 'Display', None],
  [ 0x119d, 'HangulJungseongIaraea', [], 'Display', None],
  [ 0x119e, 'HangulJungseongAraea', [], 'Display', None],
  [ 0x119f, 'HangulJungseongAraeaeo', [], 'Display', None],
  [ 0x11a0, 'HangulJungseongAraeau', [], 'Display', None],
  [ 0x11a1, 'HangulJungseongAraeai', [], 'Display', None],
  [ 0x11a2, 'HangulJungseongSsangaraea', [], 'Display', None],
  [ 0x11a3, 'UnicodeCharacter0x11a3', [], 'Display', None],
  [ 0x11a4, 'UnicodeCharacter0x11a4', [], 'Display', None],
  [ 0x11a5, 'UnicodeCharacter0x11a5', [], 'Display', None],
  [ 0x11a6, 'UnicodeCharacter0x11a6', [], 'Display', None],
  [ 0x11a7, 'UnicodeCharacter0x11a7', [], 'Display', None],
  [ 0x11a8, 'HangulJongseongKiyeok', [], 'Display', None],
  [ 0x11a9, 'HangulJongseongSsangkiyeok', [], 'Display', None],
  [ 0x11aa, 'HangulJongseongKiyeoksios', [], 'Display', None],
  [ 0x11ab, 'HangulJongseongNieun', [], 'Display', None],
  [ 0x11ac, 'HangulJongseongNieuncieuc', [], 'Display', None],
  [ 0x11ad, 'HangulJongseongNieunhieuh', [], 'Display', None],
  [ 0x11ae, 'HangulJongseongTikeut', [], 'Display', None],
  [ 0x11af, 'HangulJongseongRieul', [], 'Display', None],
  [ 0x11b0, 'HangulJongseongRieulkiyeok', [], 'Display', None],
  [ 0x11b1, 'HangulJongseongRieulmieum', [], 'Display', None],
  [ 0x11b2, 'HangulJongseongRieulpieup', [], 'Display', None],
  [ 0x11b3, 'HangulJongseongRieulsios', [], 'Display', None],
  [ 0x11b4, 'HangulJongseongRieulthieuth', [], 'Display', None],
  [ 0x11b5, 'HangulJongseongRieulphieuph', [], 'Display', None],
  [ 0x11b6, 'HangulJongseongRieulhieuh', [], 'Display', None],
  [ 0x11b7, 'HangulJongseongMieum', [], 'Display', None],
  [ 0x11b8, 'HangulJongseongPieup', [], 'Display', None],
  [ 0x11b9, 'HangulJongseongPieupsios', [], 'Display', None],
  [ 0x11ba, 'HangulJongseongSios', [], 'Display', None],
  [ 0x11bb, 'HangulJongseongSsangsios', [], 'Display', None],
  [ 0x11bc, 'HangulJongseongIeung', [], 'Display', None],
  [ 0x11bd, 'HangulJongseongCieuc', [], 'Display', None],
  [ 0x11be, 'HangulJongseongChieuch', [], 'Display', None],
  [ 0x11bf, 'HangulJongseongKhieukh', [], 'Display', None],
  [ 0x11c0, 'HangulJongseongThieuth', [], 'Display', None],
  [ 0x11c1, 'HangulJongseongPhieuph', [], 'Display', None],
  [ 0x11c2, 'HangulJongseongHieuh', [], 'Display', None],
  [ 0x11c3, 'HangulJongseongKiyeokrieul', [], 'Display', None],
  [ 0x11c4, 'HangulJongseongKiyeoksioskiyeok', [], 'Display', None],
  [ 0x11c5, 'HangulJongseongNieunkiyeok', [], 'Display', None],
  [ 0x11c6, 'HangulJongseongNieuntikeut', [], 'Display', None],
  [ 0x11c7, 'HangulJongseongNieunsios', [], 'Display', None],
  [ 0x11c8, 'HangulJongseongNieunpansios', [], 'Display', None],
  [ 0x11c9, 'HangulJongseongNieunthieuth', [], 'Display', None],
  [ 0x11ca, 'HangulJongseongTikeutkiyeok', [], 'Display', None],
  [ 0x11cb, 'HangulJongseongTikeutrieul', [], 'Display', None],
  [ 0x11cc, 'HangulJongseongRieulkiyeoksios', [], 'Display', None],
  [ 0x11cd, 'HangulJongseongRieulnieun', [], 'Display', None],
  [ 0x11ce, 'HangulJongseongRieultikeut', [], 'Display', None],
  [ 0x11cf, 'HangulJongseongRieultikeuthieuh', [], 'Display', None],
  [ 0x11d0, 'HangulJongseongSsangrieul', [], 'Display', None],
  [ 0x11d1, 'HangulJongseongRieulmieumkiyeok', [], 'Display', None],
  [ 0x11d2, 'HangulJongseongRieulmieumsios', [], 'Display', None],
  [ 0x11d3, 'HangulJongseongRieulpieupsios', [], 'Display', None],
  [ 0x11d4, 'HangulJongseongRieulpieuphieuh', [], 'Display', None],
  [ 0x11d5, 'HangulJongseongRieulkapyeounpieup', [], 'Display', None],
  [ 0x11d6, 'HangulJongseongRieulssangsios', [], 'Display', None],
  [ 0x11d7, 'HangulJongseongRieulpansios', [], 'Display', None],
  [ 0x11d8, 'HangulJongseongRieulkhieukh', [], 'Display', None],
  [ 0x11d9, 'HangulJongseongRieulyeorinhieuh', [], 'Display', None],
  [ 0x11da, 'HangulJongseongMieumkiyeok', [], 'Display', None],
  [ 0x11db, 'HangulJongseongMieumrieul', [], 'Display', None],
  [ 0x11dc, 'HangulJongseongMieumpieup', [], 'Display', None],
  [ 0x11dd, 'HangulJongseongMieumsios', [], 'Display', None],
  [ 0x11de, 'HangulJongseongMieumssangsios', [], 'Display', None],
  [ 0x11df, 'HangulJongseongMieumpansios', [], 'Display', None],
  [ 0x11e0, 'HangulJongseongMieumchieuch', [], 'Display', None],
  [ 0x11e1, 'HangulJongseongMieumhieuh', [], 'Display', None],
  [ 0x11e2, 'HangulJongseongKapyeounmieum', [], 'Display', None],
  [ 0x11e3, 'HangulJongseongPieuprieul', [], 'Display', None],
  [ 0x11e4, 'HangulJongseongPieupphieuph', [], 'Display', None],
  [ 0x11e5, 'HangulJongseongPieuphieuh', [], 'Display', None],
  [ 0x11e6, 'HangulJongseongKapyeounpieup', [], 'Display', None],
  [ 0x11e7, 'HangulJongseongSioskiyeok', [], 'Display', None],
  [ 0x11e8, 'HangulJongseongSiostikeut', [], 'Display', None],
  [ 0x11e9, 'HangulJongseongSiosrieul', [], 'Display', None],
  [ 0x11ea, 'HangulJongseongSiospieup', [], 'Display', None],
  [ 0x11eb, 'HangulJongseongPansios', [], 'Display', None],
  [ 0x11ec, 'HangulJongseongIeungkiyeok', [], 'Display', None],
  [ 0x11ed, 'HangulJongseongIeungssangkiyeok', [], 'Display', None],
  [ 0x11ee, 'HangulJongseongSsangieung', [], 'Display', None],
  [ 0x11ef, 'HangulJongseongIeungkhieukh', [], 'Display', None],
  [ 0x11f0, 'HangulJongseongYesieung', [], 'Display', None],
  [ 0x11f1, 'HangulJongseongYesieungsios', [], 'Display', None],
  [ 0x11f2, 'HangulJongseongYesieungpansios', [], 'Display', None],
  [ 0x11f3, 'HangulJongseongPhieuphpieup', [], 'Display', None],
  [ 0x11f4, 'HangulJongseongKapyeounphieuph', [], 'Display', None],
  [ 0x11f5, 'HangulJongseongHieuhnieun', [], 'Display', None],
  [ 0x11f6, 'HangulJongseongHieuhrieul', [], 'Display', None],
  [ 0x11f7, 'HangulJongseongHieuhmieum', [], 'Display', None],
  [ 0x11f8, 'HangulJongseongHieuhpieup', [], 'Display', None],
  [ 0x11f9, 'HangulJongseongYeorinhieuh', [], 'Display', None],
  [ 0x11fa, 'UnicodeCharacter0x11fa', [], 'Display', None],
  [ 0x11fb, 'UnicodeCharacter0x11fb', [], 'Display', None],
  [ 0x11fc, 'UnicodeCharacter0x11fc', [], 'Display', None],
  [ 0x11fd, 'UnicodeCharacter0x11fd', [], 'Display', None],
  [ 0x11fe, 'UnicodeCharacter0x11fe', [], 'Display', None],
  [ 0x11ff, 'UnicodeCharacter0x11ff', [], 'Display', None],
  [ 0x1200, 'EthiopicSyllableHa', [], 'Display', None],
  [ 0x1201, 'EthiopicSyllableHu', [], 'Display', None],
  [ 0x1202, 'EthiopicSyllableHi', [], 'Display', None],
  [ 0x1203, 'EthiopicSyllableHaa', [], 'Display', None],
  [ 0x1204, 'EthiopicSyllableHee', [], 'Display', None],
  [ 0x1205, 'EthiopicSyllableHe', [], 'Display', None],
  [ 0x1206, 'EthiopicSyllableHo', [], 'Display', None],
  [ 0x1207, 'EthiopicSyllableHoa', [], 'Display', None],
  [ 0x1208, 'EthiopicSyllableLa', [], 'Display', None],
  [ 0x1209, 'EthiopicSyllableLu', [], 'Display', None],
  [ 0x120a, 'EthiopicSyllableLi', [], 'Display', None],
  [ 0x120b, 'EthiopicSyllableLaa', [], 'Display', None],
  [ 0x120c, 'EthiopicSyllableLee', [], 'Display', None],
  [ 0x120d, 'EthiopicSyllableLe', [], 'Display', None],
  [ 0x120e, 'EthiopicSyllableLo', [], 'Display', None],
  [ 0x120f, 'EthiopicSyllableLwa', [], 'Display', None],
  [ 0x1210, 'EthiopicSyllableHha', [], 'Display', None],
  [ 0x1211, 'EthiopicSyllableHhu', [], 'Display', None],
  [ 0x1212, 'EthiopicSyllableHhi', [], 'Display', None],
  [ 0x1213, 'EthiopicSyllableHhaa', [], 'Display', None],
  [ 0x1214, 'EthiopicSyllableHhee', [], 'Display', None],
  [ 0x1215, 'EthiopicSyllableHhe', [], 'Display', None],
  [ 0x1216, 'EthiopicSyllableHho', [], 'Display', None],
  [ 0x1217, 'EthiopicSyllableHhwa', [], 'Display', None],
  [ 0x1218, 'EthiopicSyllableMa', [], 'Display', None],
  [ 0x1219, 'EthiopicSyllableMu', [], 'Display', None],
  [ 0x121a, 'EthiopicSyllableMi', [], 'Display', None],
  [ 0x121b, 'EthiopicSyllableMaa', [], 'Display', None],
  [ 0x121c, 'EthiopicSyllableMee', [], 'Display', None],
  [ 0x121d, 'EthiopicSyllableMe', [], 'Display', None],
  [ 0x121e, 'EthiopicSyllableMo', [], 'Display', None],
  [ 0x121f, 'EthiopicSyllableMwa', [], 'Display', None],
  [ 0x1220, 'EthiopicSyllableSza', [], 'Display', None],
  [ 0x1221, 'EthiopicSyllableSzu', [], 'Display', None],
  [ 0x1222, 'EthiopicSyllableSzi', [], 'Display', None],
  [ 0x1223, 'EthiopicSyllableSzaa', [], 'Display', None],
  [ 0x1224, 'EthiopicSyllableSzee', [], 'Display', None],
  [ 0x1225, 'EthiopicSyllableSze', [], 'Display', None],
  [ 0x1226, 'EthiopicSyllableSzo', [], 'Display', None],
  [ 0x1227, 'EthiopicSyllableSzwa', [], 'Display', None],
  [ 0x1228, 'EthiopicSyllableRa', [], 'Display', None],
  [ 0x1229, 'EthiopicSyllableRu', [], 'Display', None],
  [ 0x122a, 'EthiopicSyllableRi', [], 'Display', None],
  [ 0x122b, 'EthiopicSyllableRaa', [], 'Display', None],
  [ 0x122c, 'EthiopicSyllableRee', [], 'Display', None],
  [ 0x122d, 'EthiopicSyllableRe', [], 'Display', None],
  [ 0x122e, 'EthiopicSyllableRo', [], 'Display', None],
  [ 0x122f, 'EthiopicSyllableRwa', [], 'Display', None],
  [ 0x1230, 'EthiopicSyllableSa', [], 'Display', None],
  [ 0x1231, 'EthiopicSyllableSu', [], 'Display', None],
  [ 0x1232, 'EthiopicSyllableSi', [], 'Display', None],
  [ 0x1233, 'EthiopicSyllableSaa', [], 'Display', None],
  [ 0x1234, 'EthiopicSyllableSee', [], 'Display', None],
  [ 0x1235, 'EthiopicSyllableSe', [], 'Display', None],
  [ 0x1236, 'EthiopicSyllableSo', [], 'Display', None],
  [ 0x1237, 'EthiopicSyllableSwa', [], 'Display', None],
  [ 0x1238, 'EthiopicSyllableSha', [], 'Display', None],
  [ 0x1239, 'EthiopicSyllableShu', [], 'Display', None],
  [ 0x123a, 'EthiopicSyllableShi', [], 'Display', None],
  [ 0x123b, 'EthiopicSyllableShaa', [], 'Display', None],
  [ 0x123c, 'EthiopicSyllableShee', [], 'Display', None],
  [ 0x123d, 'EthiopicSyllableShe', [], 'Display', None],
  [ 0x123e, 'EthiopicSyllableSho', [], 'Display', None],
  [ 0x123f, 'EthiopicSyllableShwa', [], 'Display', None],
  [ 0x1240, 'EthiopicSyllableQa', [], 'Display', None],
  [ 0x1241, 'EthiopicSyllableQu', [], 'Display', None],
  [ 0x1242, 'EthiopicSyllableQi', [], 'Display', None],
  [ 0x1243, 'EthiopicSyllableQaa', [], 'Display', None],
  [ 0x1244, 'EthiopicSyllableQee', [], 'Display', None],
  [ 0x1245, 'EthiopicSyllableQe', [], 'Display', None],
  [ 0x1246, 'EthiopicSyllableQo', [], 'Display', None],
  [ 0x1247, 'EthiopicSyllableQoa', [], 'Display', None],
  [ 0x1248, 'EthiopicSyllableQwa', [], 'Display', None],
  [ 0x1249, 'UnicodeCharacter0x1249', [], 'Display', None],
  [ 0x124a, 'EthiopicSyllableQwi', [], 'Display', None],
  [ 0x124b, 'EthiopicSyllableQwaa', [], 'Display', None],
  [ 0x124c, 'EthiopicSyllableQwee', [], 'Display', None],
  [ 0x124d, 'EthiopicSyllableQwe', [], 'Display', None],
  [ 0x124e, 'UnicodeCharacter0x124e', [], 'Display', None],
  [ 0x124f, 'UnicodeCharacter0x124f', [], 'Display', None],
  [ 0x1250, 'EthiopicSyllableQha', [], 'Display', None],
  [ 0x1251, 'EthiopicSyllableQhu', [], 'Display', None],
  [ 0x1252, 'EthiopicSyllableQhi', [], 'Display', None],
  [ 0x1253, 'EthiopicSyllableQhaa', [], 'Display', None],
  [ 0x1254, 'EthiopicSyllableQhee', [], 'Display', None],
  [ 0x1255, 'EthiopicSyllableQhe', [], 'Display', None],
  [ 0x1256, 'EthiopicSyllableQho', [], 'Display', None],
  [ 0x1257, 'UnicodeCharacter0x1257', [], 'Display', None],
  [ 0x1258, 'EthiopicSyllableQhwa', [], 'Display', None],
  [ 0x1259, 'UnicodeCharacter0x1259', [], 'Display', None],
  [ 0x125a, 'EthiopicSyllableQhwi', [], 'Display', None],
  [ 0x125b, 'EthiopicSyllableQhwaa', [], 'Display', None],
  [ 0x125c, 'EthiopicSyllableQhwee', [], 'Display', None],
  [ 0x125d, 'EthiopicSyllableQhwe', [], 'Display', None],
  [ 0x125e, 'UnicodeCharacter0x125e', [], 'Display', None],
  [ 0x125f, 'UnicodeCharacter0x125f', [], 'Display', None],
  [ 0x1260, 'EthiopicSyllableBa', [], 'Display', None],
  [ 0x1261, 'EthiopicSyllableBu', [], 'Display', None],
  [ 0x1262, 'EthiopicSyllableBi', [], 'Display', None],
  [ 0x1263, 'EthiopicSyllableBaa', [], 'Display', None],
  [ 0x1264, 'EthiopicSyllableBee', [], 'Display', None],
  [ 0x1265, 'EthiopicSyllableBe', [], 'Display', None],
  [ 0x1266, 'EthiopicSyllableBo', [], 'Display', None],
  [ 0x1267, 'EthiopicSyllableBwa', [], 'Display', None],
  [ 0x1268, 'EthiopicSyllableVa', [], 'Display', None],
  [ 0x1269, 'EthiopicSyllableVu', [], 'Display', None],
  [ 0x126a, 'EthiopicSyllableVi', [], 'Display', None],
  [ 0x126b, 'EthiopicSyllableVaa', [], 'Display', None],
  [ 0x126c, 'EthiopicSyllableVee', [], 'Display', None],
  [ 0x126d, 'EthiopicSyllableVe', [], 'Display', None],
  [ 0x126e, 'EthiopicSyllableVo', [], 'Display', None],
  [ 0x126f, 'EthiopicSyllableVwa', [], 'Display', None],
  [ 0x1270, 'EthiopicSyllableTa', [], 'Display', None],
  [ 0x1271, 'EthiopicSyllableTu', [], 'Display', None],
  [ 0x1272, 'EthiopicSyllableTi', [], 'Display', None],
  [ 0x1273, 'EthiopicSyllableTaa', [], 'Display', None],
  [ 0x1274, 'EthiopicSyllableTee', [], 'Display', None],
  [ 0x1275, 'EthiopicSyllableTe', [], 'Display', None],
  [ 0x1276, 'EthiopicSyllableTo', [], 'Display', None],
  [ 0x1277, 'EthiopicSyllableTwa', [], 'Display', None],
  [ 0x1278, 'EthiopicSyllableCa', [], 'Display', None],
  [ 0x1279, 'EthiopicSyllableCu', [], 'Display', None],
  [ 0x127a, 'EthiopicSyllableCi', [], 'Display', None],
  [ 0x127b, 'EthiopicSyllableCaa', [], 'Display', None],
  [ 0x127c, 'EthiopicSyllableCee', [], 'Display', None],
  [ 0x127d, 'EthiopicSyllableCe', [], 'Display', None],
  [ 0x127e, 'EthiopicSyllableCo', [], 'Display', None],
  [ 0x127f, 'EthiopicSyllableCwa', [], 'Display', None],
  [ 0x1280, 'EthiopicSyllableXa', [], 'Display', None],
  [ 0x1281, 'EthiopicSyllableXu', [], 'Display', None],
  [ 0x1282, 'EthiopicSyllableXi', [], 'Display', None],
  [ 0x1283, 'EthiopicSyllableXaa', [], 'Display', None],
  [ 0x1284, 'EthiopicSyllableXee', [], 'Display', None],
  [ 0x1285, 'EthiopicSyllableXe', [], 'Display', None],
  [ 0x1286, 'EthiopicSyllableXo', [], 'Display', None],
  [ 0x1287, 'EthiopicSyllableXoa', [], 'Display', None],
  [ 0x1288, 'EthiopicSyllableXwa', [], 'Display', None],
  [ 0x1289, 'UnicodeCharacter0x1289', [], 'Display', None],
  [ 0x128a, 'EthiopicSyllableXwi', [], 'Display', None],
  [ 0x128b, 'EthiopicSyllableXwaa', [], 'Display', None],
  [ 0x128c, 'EthiopicSyllableXwee', [], 'Display', None],
  [ 0x128d, 'EthiopicSyllableXwe', [], 'Display', None],
  [ 0x128e, 'UnicodeCharacter0x128e', [], 'Display', None],
  [ 0x128f, 'UnicodeCharacter0x128f', [], 'Display', None],
  [ 0x1290, 'EthiopicSyllableNa', [], 'Display', None],
  [ 0x1291, 'EthiopicSyllableNu', [], 'Display', None],
  [ 0x1292, 'EthiopicSyllableNi', [], 'Display', None],
  [ 0x1293, 'EthiopicSyllableNaa', [], 'Display', None],
  [ 0x1294, 'EthiopicSyllableNee', [], 'Display', None],
  [ 0x1295, 'EthiopicSyllableNe', [], 'Display', None],
  [ 0x1296, 'EthiopicSyllableNo', [], 'Display', None],
  [ 0x1297, 'EthiopicSyllableNwa', [], 'Display', None],
  [ 0x1298, 'EthiopicSyllableNya', [], 'Display', None],
  [ 0x1299, 'EthiopicSyllableNyu', [], 'Display', None],
  [ 0x129a, 'EthiopicSyllableNyi', [], 'Display', None],
  [ 0x129b, 'EthiopicSyllableNyaa', [], 'Display', None],
  [ 0x129c, 'EthiopicSyllableNyee', [], 'Display', None],
  [ 0x129d, 'EthiopicSyllableNye', [], 'Display', None],
  [ 0x129e, 'EthiopicSyllableNyo', [], 'Display', None],
  [ 0x129f, 'EthiopicSyllableNywa', [], 'Display', None],
  [ 0x12a0, 'EthiopicSyllableGlottalA', [], 'Display', None],
  [ 0x12a1, 'EthiopicSyllableGlottalU', [], 'Display', None],
  [ 0x12a2, 'EthiopicSyllableGlottalI', [], 'Display', None],
  [ 0x12a3, 'EthiopicSyllableGlottalAa', [], 'Display', None],
  [ 0x12a4, 'EthiopicSyllableGlottalEe', [], 'Display', None],
  [ 0x12a5, 'EthiopicSyllableGlottalE', [], 'Display', None],
  [ 0x12a6, 'EthiopicSyllableGlottalO', [], 'Display', None],
  [ 0x12a7, 'EthiopicSyllableGlottalWa', [], 'Display', None],
  [ 0x12a8, 'EthiopicSyllableKa', [], 'Display', None],
  [ 0x12a9, 'EthiopicSyllableKu', [], 'Display', None],
  [ 0x12aa, 'EthiopicSyllableKi', [], 'Display', None],
  [ 0x12ab, 'EthiopicSyllableKaa', [], 'Display', None],
  [ 0x12ac, 'EthiopicSyllableKee', [], 'Display', None],
  [ 0x12ad, 'EthiopicSyllableKe', [], 'Display', None],
  [ 0x12ae, 'EthiopicSyllableKo', [], 'Display', None],
  [ 0x12af, 'EthiopicSyllableKoa', [], 'Display', None],
  [ 0x12b0, 'EthiopicSyllableKwa', [], 'Display', None],
  [ 0x12b1, 'UnicodeCharacter0x12b1', [], 'Display', None],
  [ 0x12b2, 'EthiopicSyllableKwi', [], 'Display', None],
  [ 0x12b3, 'EthiopicSyllableKwaa', [], 'Display', None],
  [ 0x12b4, 'EthiopicSyllableKwee', [], 'Display', None],
  [ 0x12b5, 'EthiopicSyllableKwe', [], 'Display', None],
  [ 0x12b6, 'UnicodeCharacter0x12b6', [], 'Display', None],
  [ 0x12b7, 'UnicodeCharacter0x12b7', [], 'Display', None],
  [ 0x12b8, 'EthiopicSyllableKxa', [], 'Display', None],
  [ 0x12b9, 'EthiopicSyllableKxu', [], 'Display', None],
  [ 0x12ba, 'EthiopicSyllableKxi', [], 'Display', None],
  [ 0x12bb, 'EthiopicSyllableKxaa', [], 'Display', None],
  [ 0x12bc, 'EthiopicSyllableKxee', [], 'Display', None],
  [ 0x12bd, 'EthiopicSyllableKxe', [], 'Display', None],
  [ 0x12be, 'EthiopicSyllableKxo', [], 'Display', None],
  [ 0x12bf, 'UnicodeCharacter0x12bf', [], 'Display', None],
  [ 0x12c0, 'EthiopicSyllableKxwa', [], 'Display', None],
  [ 0x12c1, 'UnicodeCharacter0x12c1', [], 'Display', None],
  [ 0x12c2, 'EthiopicSyllableKxwi', [], 'Display', None],
  [ 0x12c3, 'EthiopicSyllableKxwaa', [], 'Display', None],
  [ 0x12c4, 'EthiopicSyllableKxwee', [], 'Display', None],
  [ 0x12c5, 'EthiopicSyllableKxwe', [], 'Display', None],
  [ 0x12c6, 'UnicodeCharacter0x12c6', [], 'Display', None],
  [ 0x12c7, 'UnicodeCharacter0x12c7', [], 'Display', None],
  [ 0x12c8, 'EthiopicSyllableWa', [], 'Display', None],
  [ 0x12c9, 'EthiopicSyllableWu', [], 'Display', None],
  [ 0x12ca, 'EthiopicSyllableWi', [], 'Display', None],
  [ 0x12cb, 'EthiopicSyllableWaa', [], 'Display', None],
  [ 0x12cc, 'EthiopicSyllableWee', [], 'Display', None],
  [ 0x12cd, 'EthiopicSyllableWe', [], 'Display', None],
  [ 0x12ce, 'EthiopicSyllableWo', [], 'Display', None],
  [ 0x12cf, 'EthiopicSyllableWoa', [], 'Display', None],
  [ 0x12d0, 'EthiopicSyllablePharyngealA', [], 'Display', None],
  [ 0x12d1, 'EthiopicSyllablePharyngealU', [], 'Display', None],
  [ 0x12d2, 'EthiopicSyllablePharyngealI', [], 'Display', None],
  [ 0x12d3, 'EthiopicSyllablePharyngealAa', [], 'Display', None],
  [ 0x12d4, 'EthiopicSyllablePharyngealEe', [], 'Display', None],
  [ 0x12d5, 'EthiopicSyllablePharyngealE', [], 'Display', None],
  [ 0x12d6, 'EthiopicSyllablePharyngealO', [], 'Display', None],
  [ 0x12d7, 'UnicodeCharacter0x12d7', [], 'Display', None],
  [ 0x12d8, 'EthiopicSyllableZa', [], 'Display', None],
  [ 0x12d9, 'EthiopicSyllableZu', [], 'Display', None],
  [ 0x12da, 'EthiopicSyllableZi', [], 'Display', None],
  [ 0x12db, 'EthiopicSyllableZaa', [], 'Display', None],
  [ 0x12dc, 'EthiopicSyllableZee', [], 'Display', None],
  [ 0x12dd, 'EthiopicSyllableZe', [], 'Display', None],
  [ 0x12de, 'EthiopicSyllableZo', [], 'Display', None],
  [ 0x12df, 'EthiopicSyllableZwa', [], 'Display', None],
  [ 0x12e0, 'EthiopicSyllableZha', [], 'Display', None],
  [ 0x12e1, 'EthiopicSyllableZhu', [], 'Display', None],
  [ 0x12e2, 'EthiopicSyllableZhi', [], 'Display', None],
  [ 0x12e3, 'EthiopicSyllableZhaa', [], 'Display', None],
  [ 0x12e4, 'EthiopicSyllableZhee', [], 'Display', None],
  [ 0x12e5, 'EthiopicSyllableZhe', [], 'Display', None],
  [ 0x12e6, 'EthiopicSyllableZho', [], 'Display', None],
  [ 0x12e7, 'EthiopicSyllableZhwa', [], 'Display', None],
  [ 0x12e8, 'EthiopicSyllableYa', [], 'Display', None],
  [ 0x12e9, 'EthiopicSyllableYu', [], 'Display', None],
  [ 0x12ea, 'EthiopicSyllableYi', [], 'Display', None],
  [ 0x12eb, 'EthiopicSyllableYaa', [], 'Display', None],
  [ 0x12ec, 'EthiopicSyllableYee', [], 'Display', None],
  [ 0x12ed, 'EthiopicSyllableYe', [], 'Display', None],
  [ 0x12ee, 'EthiopicSyllableYo', [], 'Display', None],
  [ 0x12ef, 'EthiopicSyllableYoa', [], 'Display', None],
  [ 0x12f0, 'EthiopicSyllableDa', [], 'Display', None],
  [ 0x12f1, 'EthiopicSyllableDu', [], 'Display', None],
  [ 0x12f2, 'EthiopicSyllableDi', [], 'Display', None],
  [ 0x12f3, 'EthiopicSyllableDaa', [], 'Display', None],
  [ 0x12f4, 'EthiopicSyllableDee', [], 'Display', None],
  [ 0x12f5, 'EthiopicSyllableDe', [], 'Display', None],
  [ 0x12f6, 'EthiopicSyllableDo', [], 'Display', None],
  [ 0x12f7, 'EthiopicSyllableDwa', [], 'Display', None],
  [ 0x12f8, 'EthiopicSyllableDda', [], 'Display', None],
  [ 0x12f9, 'EthiopicSyllableDdu', [], 'Display', None],
  [ 0x12fa, 'EthiopicSyllableDdi', [], 'Display', None],
  [ 0x12fb, 'EthiopicSyllableDdaa', [], 'Display', None],
  [ 0x12fc, 'EthiopicSyllableDdee', [], 'Display', None],
  [ 0x12fd, 'EthiopicSyllableDde', [], 'Display', None],
  [ 0x12fe, 'EthiopicSyllableDdo', [], 'Display', None],
  [ 0x12ff, 'EthiopicSyllableDdwa', [], 'Display', None],
  [ 0x1300, 'EthiopicSyllableJa', [], 'Display', None],
  [ 0x1301, 'EthiopicSyllableJu', [], 'Display', None],
  [ 0x1302, 'EthiopicSyllableJi', [], 'Display', None],
  [ 0x1303, 'EthiopicSyllableJaa', [], 'Display', None],
  [ 0x1304, 'EthiopicSyllableJee', [], 'Display', None],
  [ 0x1305, 'EthiopicSyllableJe', [], 'Display', None],
  [ 0x1306, 'EthiopicSyllableJo', [], 'Display', None],
  [ 0x1307, 'EthiopicSyllableJwa', [], 'Display', None],
  [ 0x1308, 'EthiopicSyllableGa', [], 'Display', None],
  [ 0x1309, 'EthiopicSyllableGu', [], 'Display', None],
  [ 0x130a, 'EthiopicSyllableGi', [], 'Display', None],
  [ 0x130b, 'EthiopicSyllableGaa', [], 'Display', None],
  [ 0x130c, 'EthiopicSyllableGee', [], 'Display', None],
  [ 0x130d, 'EthiopicSyllableGe', [], 'Display', None],
  [ 0x130e, 'EthiopicSyllableGo', [], 'Display', None],
  [ 0x130f, 'EthiopicSyllableGoa', [], 'Display', None],
  [ 0x1310, 'EthiopicSyllableGwa', [], 'Display', None],
  [ 0x1311, 'UnicodeCharacter0x1311', [], 'Display', None],
  [ 0x1312, 'EthiopicSyllableGwi', [], 'Display', None],
  [ 0x1313, 'EthiopicSyllableGwaa', [], 'Display', None],
  [ 0x1314, 'EthiopicSyllableGwee', [], 'Display', None],
  [ 0x1315, 'EthiopicSyllableGwe', [], 'Display', None],
  [ 0x1316, 'UnicodeCharacter0x1316', [], 'Display', None],
  [ 0x1317, 'UnicodeCharacter0x1317', [], 'Display', None],
  [ 0x1318, 'EthiopicSyllableGga', [], 'Display', None],
  [ 0x1319, 'EthiopicSyllableGgu', [], 'Display', None],
  [ 0x131a, 'EthiopicSyllableGgi', [], 'Display', None],
  [ 0x131b, 'EthiopicSyllableGgaa', [], 'Display', None],
  [ 0x131c, 'EthiopicSyllableGgee', [], 'Display', None],
  [ 0x131d, 'EthiopicSyllableGge', [], 'Display', None],
  [ 0x131e, 'EthiopicSyllableGgo', [], 'Display', None],
  [ 0x131f, 'EthiopicSyllableGgwaa', [], 'Display', None],
  [ 0x1320, 'EthiopicSyllableTha', [], 'Display', None],
  [ 0x1321, 'EthiopicSyllableThu', [], 'Display', None],
  [ 0x1322, 'EthiopicSyllableThi', [], 'Display', None],
  [ 0x1323, 'EthiopicSyllableThaa', [], 'Display', None],
  [ 0x1324, 'EthiopicSyllableThee', [], 'Display', None],
  [ 0x1325, 'EthiopicSyllableThe', [], 'Display', None],
  [ 0x1326, 'EthiopicSyllableTho', [], 'Display', None],
  [ 0x1327, 'EthiopicSyllableThwa', [], 'Display', None],
  [ 0x1328, 'EthiopicSyllableCha', [], 'Display', None],
  [ 0x1329, 'EthiopicSyllableChu', [], 'Display', None],
  [ 0x132a, 'EthiopicSyllableChi', [], 'Display', None],
  [ 0x132b, 'EthiopicSyllableChaa', [], 'Display', None],
  [ 0x132c, 'EthiopicSyllableChee', [], 'Display', None],
  [ 0x132d, 'EthiopicSyllableChe', [], 'Display', None],
  [ 0x132e, 'EthiopicSyllableCho', [], 'Display', None],
  [ 0x132f, 'EthiopicSyllableChwa', [], 'Display', None],
  [ 0x1330, 'EthiopicSyllablePha', [], 'Display', None],
  [ 0x1331, 'EthiopicSyllablePhu', [], 'Display', None],
  [ 0x1332, 'EthiopicSyllablePhi', [], 'Display', None],
  [ 0x1333, 'EthiopicSyllablePhaa', [], 'Display', None],
  [ 0x1334, 'EthiopicSyllablePhee', [], 'Display', None],
  [ 0x1335, 'EthiopicSyllablePhe', [], 'Display', None],
  [ 0x1336, 'EthiopicSyllablePho', [], 'Display', None],
  [ 0x1337, 'EthiopicSyllablePhwa', [], 'Display', None],
  [ 0x1338, 'EthiopicSyllableTsa', [], 'Display', None],
  [ 0x1339, 'EthiopicSyllableTsu', [], 'Display', None],
  [ 0x133a, 'EthiopicSyllableTsi', [], 'Display', None],
  [ 0x133b, 'EthiopicSyllableTsaa', [], 'Display', None],
  [ 0x133c, 'EthiopicSyllableTsee', [], 'Display', None],
  [ 0x133d, 'EthiopicSyllableTse', [], 'Display', None],
  [ 0x133e, 'EthiopicSyllableTso', [], 'Display', None],
  [ 0x133f, 'EthiopicSyllableTswa', [], 'Display', None],
  [ 0x1340, 'EthiopicSyllableTza', [], 'Display', None],
  [ 0x1341, 'EthiopicSyllableTzu', [], 'Display', None],
  [ 0x1342, 'EthiopicSyllableTzi', [], 'Display', None],
  [ 0x1343, 'EthiopicSyllableTzaa', [], 'Display', None],
  [ 0x1344, 'EthiopicSyllableTzee', [], 'Display', None],
  [ 0x1345, 'EthiopicSyllableTze', [], 'Display', None],
  [ 0x1346, 'EthiopicSyllableTzo', [], 'Display', None],
  [ 0x1347, 'EthiopicSyllableTzoa', [], 'Display', None],
  [ 0x1348, 'EthiopicSyllableFa', [], 'Display', None],
  [ 0x1349, 'EthiopicSyllableFu', [], 'Display', None],
  [ 0x134a, 'EthiopicSyllableFi', [], 'Display', None],
  [ 0x134b, 'EthiopicSyllableFaa', [], 'Display', None],
  [ 0x134c, 'EthiopicSyllableFee', [], 'Display', None],
  [ 0x134d, 'EthiopicSyllableFe', [], 'Display', None],
  [ 0x134e, 'EthiopicSyllableFo', [], 'Display', None],
  [ 0x134f, 'EthiopicSyllableFwa', [], 'Display', None],
  [ 0x1350, 'EthiopicSyllablePa', [], 'Display', None],
  [ 0x1351, 'EthiopicSyllablePu', [], 'Display', None],
  [ 0x1352, 'EthiopicSyllablePi', [], 'Display', None],
  [ 0x1353, 'EthiopicSyllablePaa', [], 'Display', None],
  [ 0x1354, 'EthiopicSyllablePee', [], 'Display', None],
  [ 0x1355, 'EthiopicSyllablePe', [], 'Display', None],
  [ 0x1356, 'EthiopicSyllablePo', [], 'Display', None],
  [ 0x1357, 'EthiopicSyllablePwa', [], 'Display', None],
  [ 0x1358, 'EthiopicSyllableRya', [], 'Display', None],
  [ 0x1359, 'EthiopicSyllableMya', [], 'Display', None],
  [ 0x135a, 'EthiopicSyllableFya', [], 'Display', None],
  [ 0x135b, 'UnicodeCharacter0x135b', [], 'Display', None],
  [ 0x135c, 'UnicodeCharacter0x135c', [], 'Display', None],
  [ 0x135d, 'UnicodeCharacter0x135d', [], 'Display', None],
  [ 0x135e, 'UnicodeCharacter0x135e', [], 'Display', None],
  [ 0x135f, 'EthiopicCombiningGeminationMark', [], 'Display', None],
  [ 0x1360, 'EthiopicSectionMark', [], 'Display', None],
  [ 0x1361, 'EthiopicWordspace', [], 'Display', None],
  [ 0x1362, 'EthiopicFullStop', [], 'Display', None],
  [ 0x1363, 'EthiopicComma', [], 'Display', None],
  [ 0x1364, 'EthiopicSemicolon', [], 'Display', None],
  [ 0x1365, 'EthiopicColon', [], 'Display', None],
  [ 0x1366, 'EthiopicPrefaceColon', [], 'Display', None],
  [ 0x1367, 'EthiopicQuestionMark', [], 'Display', None],
  [ 0x1368, 'EthiopicParagraphSeparator', [], 'Display', None],
  [ 0x1369, 'EthiopicDigitOne', [], 'Display', None],
  [ 0x136a, 'EthiopicDigitTwo', [], 'Display', None],
  [ 0x136b, 'EthiopicDigitThree', [], 'Display', None],
  [ 0x136c, 'EthiopicDigitFour', [], 'Display', None],
  [ 0x136d, 'EthiopicDigitFive', [], 'Display', None],
  [ 0x136e, 'EthiopicDigitSix', [], 'Display', None],
  [ 0x136f, 'EthiopicDigitSeven', [], 'Display', None],
  [ 0x1370, 'EthiopicDigitEight', [], 'Display', None],
  [ 0x1371, 'EthiopicDigitNine', [], 'Display', None],
  [ 0x1372, 'EthiopicNumberTen', [], 'Display', None],
  [ 0x1373, 'EthiopicNumberTwenty', [], 'Display', None],
  [ 0x1374, 'EthiopicNumberThirty', [], 'Display', None],
  [ 0x1375, 'EthiopicNumberForty', [], 'Display', None],
  [ 0x1376, 'EthiopicNumberFifty', [], 'Display', None],
  [ 0x1377, 'EthiopicNumberSixty', [], 'Display', None],
  [ 0x1378, 'EthiopicNumberSeventy', [], 'Display', None],
  [ 0x1379, 'EthiopicNumberEighty', [], 'Display', None],
  [ 0x137a, 'EthiopicNumberNinety', [], 'Display', None],
  [ 0x137b, 'EthiopicNumberHundred', [], 'Display', None],
  [ 0x137c, 'EthiopicNumberTenThousand', [], 'Display', None],
  [ 0x137d, 'UnicodeCharacter0x137d', [], 'Display', None],
  [ 0x137e, 'UnicodeCharacter0x137e', [], 'Display', None],
  [ 0x137f, 'UnicodeCharacter0x137f', [], 'Display', None],
  [ 0x1380, 'EthiopicSyllableSebatbeitMwa', [], 'Display', None],
  [ 0x1381, 'EthiopicSyllableMwi', [], 'Display', None],
  [ 0x1382, 'EthiopicSyllableMwee', [], 'Display', None],
  [ 0x1383, 'EthiopicSyllableMwe', [], 'Display', None],
  [ 0x1384, 'EthiopicSyllableSebatbeitBwa', [], 'Display', None],
  [ 0x1385, 'EthiopicSyllableBwi', [], 'Display', None],
  [ 0x1386, 'EthiopicSyllableBwee', [], 'Display', None],
  [ 0x1387, 'EthiopicSyllableBwe', [], 'Display', None],
  [ 0x1388, 'EthiopicSyllableSebatbeitFwa', [], 'Display', None],
  [ 0x1389, 'EthiopicSyllableFwi', [], 'Display', None],
  [ 0x138a, 'EthiopicSyllableFwee', [], 'Display', None],
  [ 0x138b, 'EthiopicSyllableFwe', [], 'Display', None],
  [ 0x138c, 'EthiopicSyllableSebatbeitPwa', [], 'Display', None],
  [ 0x138d, 'EthiopicSyllablePwi', [], 'Display', None],
  [ 0x138e, 'EthiopicSyllablePwee', [], 'Display', None],
  [ 0x138f, 'EthiopicSyllablePwe', [], 'Display', None],
  [ 0x1390, 'EthiopicTonalMarkYizet', [], 'Display', None],
  [ 0x1391, 'EthiopicTonalMarkDeret', [], 'Display', None],
  [ 0x1392, 'EthiopicTonalMarkRikrik', [], 'Display', None],
  [ 0x1393, 'EthiopicTonalMarkShortRikrik', [], 'Display', None],
  [ 0x1394, 'EthiopicTonalMarkDifat', [], 'Display', None],
  [ 0x1395, 'EthiopicTonalMarkKenat', [], 'Display', None],
  [ 0x1396, 'EthiopicTonalMarkChiret', [], 'Display', None],
  [ 0x1397, 'EthiopicTonalMarkHidet', [], 'Display', None],
  [ 0x1398, 'EthiopicTonalMarkDerethidet', [], 'Display', None],
  [ 0x1399, 'EthiopicTonalMarkKurt', [], 'Display', None],
  [ 0x139a, 'UnicodeCharacter0x139a', [], 'Display', None],
  [ 0x139b, 'UnicodeCharacter0x139b', [], 'Display', None],
  [ 0x139c, 'UnicodeCharacter0x139c', [], 'Display', None],
  [ 0x139d, 'UnicodeCharacter0x139d', [], 'Display', None],
  [ 0x139e, 'UnicodeCharacter0x139e', [], 'Display', None],
  [ 0x139f, 'UnicodeCharacter0x139f', [], 'Display', None],
  [ 0x13a0, 'CherokeeLetterA', [], 'Display', None],
  [ 0x13a1, 'CherokeeLetterE', [], 'Display', None],
  [ 0x13a2, 'CherokeeLetterI', [], 'Display', None],
  [ 0x13a3, 'CherokeeLetterO', [], 'Display', None],
  [ 0x13a4, 'CherokeeLetterU', [], 'Display', None],
  [ 0x13a5, 'CherokeeLetterV', [], 'Display', None],
  [ 0x13a6, 'CherokeeLetterGa', [], 'Display', None],
  [ 0x13a7, 'CherokeeLetterKa', [], 'Display', None],
  [ 0x13a8, 'CherokeeLetterGe', [], 'Display', None],
  [ 0x13a9, 'CherokeeLetterGi', [], 'Display', None],
  [ 0x13aa, 'CherokeeLetterGo', [], 'Display', None],
  [ 0x13ab, 'CherokeeLetterGu', [], 'Display', None],
  [ 0x13ac, 'CherokeeLetterGv', [], 'Display', None],
  [ 0x13ad, 'CherokeeLetterHa', [], 'Display', None],
  [ 0x13ae, 'CherokeeLetterHe', [], 'Display', None],
  [ 0x13af, 'CherokeeLetterHi', [], 'Display', None],
  [ 0x13b0, 'CherokeeLetterHo', [], 'Display', None],
  [ 0x13b1, 'CherokeeLetterHu', [], 'Display', None],
  [ 0x13b2, 'CherokeeLetterHv', [], 'Display', None],
  [ 0x13b3, 'CherokeeLetterLa', [], 'Display', None],
  [ 0x13b4, 'CherokeeLetterLe', [], 'Display', None],
  [ 0x13b5, 'CherokeeLetterLi', [], 'Display', None],
  [ 0x13b6, 'CherokeeLetterLo', [], 'Display', None],
  [ 0x13b7, 'CherokeeLetterLu', [], 'Display', None],
  [ 0x13b8, 'CherokeeLetterLv', [], 'Display', None],
  [ 0x13b9, 'CherokeeLetterMa', [], 'Display', None],
  [ 0x13ba, 'CherokeeLetterMe', [], 'Display', None],
  [ 0x13bb, 'CherokeeLetterMi', [], 'Display', None],
  [ 0x13bc, 'CherokeeLetterMo', [], 'Display', None],
  [ 0x13bd, 'CherokeeLetterMu', [], 'Display', None],
  [ 0x13be, 'CherokeeLetterNa', [], 'Display', None],
  [ 0x13bf, 'CherokeeLetterHna', [], 'Display', None],
  [ 0x13c0, 'CherokeeLetterNah', [], 'Display', None],
  [ 0x13c1, 'CherokeeLetterNe', [], 'Display', None],
  [ 0x13c2, 'CherokeeLetterNi', [], 'Display', None],
  [ 0x13c3, 'CherokeeLetterNo', [], 'Display', None],
  [ 0x13c4, 'CherokeeLetterNu', [], 'Display', None],
  [ 0x13c5, 'CherokeeLetterNv', [], 'Display', None],
  [ 0x13c6, 'CherokeeLetterQua', [], 'Display', None],
  [ 0x13c7, 'CherokeeLetterQue', [], 'Display', None],
  [ 0x13c8, 'CherokeeLetterQui', [], 'Display', None],
  [ 0x13c9, 'CherokeeLetterQuo', [], 'Display', None],
  [ 0x13ca, 'CherokeeLetterQuu', [], 'Display', None],
  [ 0x13cb, 'CherokeeLetterQuv', [], 'Display', None],
  [ 0x13cc, 'CherokeeLetterSa', [], 'Display', None],
  [ 0x13cd, 'CherokeeLetterS', [], 'Display', None],
  [ 0x13ce, 'CherokeeLetterSe', [], 'Display', None],
  [ 0x13cf, 'CherokeeLetterSi', [], 'Display', None],
  [ 0x13d0, 'CherokeeLetterSo', [], 'Display', None],
  [ 0x13d1, 'CherokeeLetterSu', [], 'Display', None],
  [ 0x13d2, 'CherokeeLetterSv', [], 'Display', None],
  [ 0x13d3, 'CherokeeLetterDa', [], 'Display', None],
  [ 0x13d4, 'CherokeeLetterTa', [], 'Display', None],
  [ 0x13d5, 'CherokeeLetterDe', [], 'Display', None],
  [ 0x13d6, 'CherokeeLetterTe', [], 'Display', None],
  [ 0x13d7, 'CherokeeLetterDi', [], 'Display', None],
  [ 0x13d8, 'CherokeeLetterTi', [], 'Display', None],
  [ 0x13d9, 'CherokeeLetterDo', [], 'Display', None],
  [ 0x13da, 'CherokeeLetterDu', [], 'Display', None],
  [ 0x13db, 'CherokeeLetterDv', [], 'Display', None],
  [ 0x13dc, 'CherokeeLetterDla', [], 'Display', None],
  [ 0x13dd, 'CherokeeLetterTla', [], 'Display', None],
  [ 0x13de, 'CherokeeLetterTle', [], 'Display', None],
  [ 0x13df, 'CherokeeLetterTli', [], 'Display', None],
  [ 0x13e0, 'CherokeeLetterTlo', [], 'Display', None],
  [ 0x13e1, 'CherokeeLetterTlu', [], 'Display', None],
  [ 0x13e2, 'CherokeeLetterTlv', [], 'Display', None],
  [ 0x13e3, 'CherokeeLetterTsa', [], 'Display', None],
  [ 0x13e4, 'CherokeeLetterTse', [], 'Display', None],
  [ 0x13e5, 'CherokeeLetterTsi', [], 'Display', None],
  [ 0x13e6, 'CherokeeLetterTso', [], 'Display', None],
  [ 0x13e7, 'CherokeeLetterTsu', [], 'Display', None],
  [ 0x13e8, 'CherokeeLetterTsv', [], 'Display', None],
  [ 0x13e9, 'CherokeeLetterWa', [], 'Display', None],
  [ 0x13ea, 'CherokeeLetterWe', [], 'Display', None],
  [ 0x13eb, 'CherokeeLetterWi', [], 'Display', None],
  [ 0x13ec, 'CherokeeLetterWo', [], 'Display', None],
  [ 0x13ed, 'CherokeeLetterWu', [], 'Display', None],
  [ 0x13ee, 'CherokeeLetterWv', [], 'Display', None],
  [ 0x13ef, 'CherokeeLetterYa', [], 'Display', None],
  [ 0x13f0, 'CherokeeLetterYe', [], 'Display', None],
  [ 0x13f1, 'CherokeeLetterYi', [], 'Display', None],
  [ 0x13f2, 'CherokeeLetterYo', [], 'Display', None],
  [ 0x13f3, 'CherokeeLetterYu', [], 'Display', None],
  [ 0x13f4, 'CherokeeLetterYv', [], 'Display', None],
  [ 0x13f5, 'UnicodeCharacter0x13f5', [], 'Display', None],
  [ 0x13f6, 'UnicodeCharacter0x13f6', [], 'Display', None],
  [ 0x13f7, 'UnicodeCharacter0x13f7', [], 'Display', None],
  [ 0x13f8, 'UnicodeCharacter0x13f8', [], 'Display', None],
  [ 0x13f9, 'UnicodeCharacter0x13f9', [], 'Display', None],
  [ 0x13fa, 'UnicodeCharacter0x13fa', [], 'Display', None],
  [ 0x13fb, 'UnicodeCharacter0x13fb', [], 'Display', None],
  [ 0x13fc, 'UnicodeCharacter0x13fc', [], 'Display', None],
  [ 0x13fd, 'UnicodeCharacter0x13fd', [], 'Display', None],
  [ 0x13fe, 'UnicodeCharacter0x13fe', [], 'Display', None],
  [ 0x13ff, 'UnicodeCharacter0x13ff', [], 'Display', None],
  [ 0x1400, 'UnicodeCharacter0x1400', [], 'Display', None],
  [ 0x1401, 'CanadianSyllabicsE', [], 'Display', None],
  [ 0x1402, 'CanadianSyllabicsAai', [], 'Display', None],
  [ 0x1403, 'CanadianSyllabicsI', [], 'Display', None],
  [ 0x1404, 'CanadianSyllabicsIi', [], 'Display', None],
  [ 0x1405, 'CanadianSyllabicsO', [], 'Display', None],
  [ 0x1406, 'CanadianSyllabicsOo', [], 'Display', None],
  [ 0x1407, 'CanadianSyllabicsYcreeOo', [], 'Display', None],
  [ 0x1408, 'CanadianSyllabicsCarrierEe', [], 'Display', None],
  [ 0x1409, 'CanadianSyllabicsCarrierI', [], 'Display', None],
  [ 0x140a, 'CanadianSyllabicsA', [], 'Display', None],
  [ 0x140b, 'CanadianSyllabicsAa', [], 'Display', None],
  [ 0x140c, 'CanadianSyllabicsWe', [], 'Display', None],
  [ 0x140d, 'CanadianSyllabicsWestcreeWe', [], 'Display', None],
  [ 0x140e, 'CanadianSyllabicsWi', [], 'Display', None],
  [ 0x140f, 'CanadianSyllabicsWestcreeWi', [], 'Display', None],
  [ 0x1410, 'CanadianSyllabicsWii', [], 'Display', None],
  [ 0x1411, 'CanadianSyllabicsWestcreeWii', [], 'Display', None],
  [ 0x1412, 'CanadianSyllabicsWo', [], 'Display', None],
  [ 0x1413, 'CanadianSyllabicsWestcreeWo', [], 'Display', None],
  [ 0x1414, 'CanadianSyllabicsWoo', [], 'Display', None],
  [ 0x1415, 'CanadianSyllabicsWestcreeWoo', [], 'Display', None],
  [ 0x1416, 'CanadianSyllabicsNaskapiWoo', [], 'Display', None],
  [ 0x1417, 'CanadianSyllabicsWa', [], 'Display', None],
  [ 0x1418, 'CanadianSyllabicsWestcreeWa', [], 'Display', None],
  [ 0x1419, 'CanadianSyllabicsWaa', [], 'Display', None],
  [ 0x141a, 'CanadianSyllabicsWestcreeWaa', [], 'Display', None],
  [ 0x141b, 'CanadianSyllabicsNaskapiWaa', [], 'Display', None],
  [ 0x141c, 'CanadianSyllabicsAi', [], 'Display', None],
  [ 0x141d, 'CanadianSyllabicsYcreeW', [], 'Display', None],
  [ 0x141e, 'CanadianSyllabicsGlottalStop', [], 'Display', None],
  [ 0x141f, 'CanadianSyllabicsFinalAcute', [], 'Display', None],
  [ 0x1420, 'CanadianSyllabicsFinalGrave', [], 'Display', None],
  [ 0x1421, 'CanadianSyllabicsFinalBottomHalfRing', [], 'Display', None],
  [ 0x1422, 'CanadianSyllabicsFinalTopHalfRing', [], 'Display', None],
  [ 0x1423, 'CanadianSyllabicsFinalRightHalfRing', [], 'Display', None],
  [ 0x1424, 'CanadianSyllabicsFinalRing', [], 'Display', None],
  [ 0x1425, 'CanadianSyllabicsFinalDoubleAcute', [], 'Display', None],
  [ 0x1426, 'CanadianSyllabicsFinalDoubleShortVerticalStrokes', [], 'Display', None],
  [ 0x1427, 'CanadianSyllabicsFinalMiddleDot', [], 'Display', None],
  [ 0x1428, 'CanadianSyllabicsFinalShortHorizontalStroke', [], 'Display', None],
  [ 0x1429, 'CanadianSyllabicsFinalPlus', [], 'Display', None],
  [ 0x142a, 'CanadianSyllabicsFinalDownTack', [], 'Display', None],
  [ 0x142b, 'CanadianSyllabicsEn', [], 'Display', None],
  [ 0x142c, 'CanadianSyllabicsIn', [], 'Display', None],
  [ 0x142d, 'CanadianSyllabicsOn', [], 'Display', None],
  [ 0x142e, 'CanadianSyllabicsAn', [], 'Display', None],
  [ 0x142f, 'CanadianSyllabicsPe', [], 'Display', None],
  [ 0x1430, 'CanadianSyllabicsPaai', [], 'Display', None],
  [ 0x1431, 'CanadianSyllabicsPi', [], 'Display', None],
  [ 0x1432, 'CanadianSyllabicsPii', [], 'Display', None],
  [ 0x1433, 'CanadianSyllabicsPo', [], 'Display', None],
  [ 0x1434, 'CanadianSyllabicsPoo', [], 'Display', None],
  [ 0x1435, 'CanadianSyllabicsYcreePoo', [], 'Display', None],
  [ 0x1436, 'CanadianSyllabicsCarrierHee', [], 'Display', None],
  [ 0x1437, 'CanadianSyllabicsCarrierHi', [], 'Display', None],
  [ 0x1438, 'CanadianSyllabicsPa', [], 'Display', None],
  [ 0x1439, 'CanadianSyllabicsPaa', [], 'Display', None],
  [ 0x143a, 'CanadianSyllabicsPwe', [], 'Display', None],
  [ 0x143b, 'CanadianSyllabicsWestcreePwe', [], 'Display', None],
  [ 0x143c, 'CanadianSyllabicsPwi', [], 'Display', None],
  [ 0x143d, 'CanadianSyllabicsWestcreePwi', [], 'Display', None],
  [ 0x143e, 'CanadianSyllabicsPwii', [], 'Display', None],
  [ 0x143f, 'CanadianSyllabicsWestcreePwii', [], 'Display', None],
  [ 0x1440, 'CanadianSyllabicsPwo', [], 'Display', None],
  [ 0x1441, 'CanadianSyllabicsWestcreePwo', [], 'Display', None],
  [ 0x1442, 'CanadianSyllabicsPwoo', [], 'Display', None],
  [ 0x1443, 'CanadianSyllabicsWestcreePwoo', [], 'Display', None],
  [ 0x1444, 'CanadianSyllabicsPwa', [], 'Display', None],
  [ 0x1445, 'CanadianSyllabicsWestcreePwa', [], 'Display', None],
  [ 0x1446, 'CanadianSyllabicsPwaa', [], 'Display', None],
  [ 0x1447, 'CanadianSyllabicsWestcreePwaa', [], 'Display', None],
  [ 0x1448, 'CanadianSyllabicsYcreePwaa', [], 'Display', None],
  [ 0x1449, 'CanadianSyllabicsP', [], 'Display', None],
  [ 0x144a, 'CanadianSyllabicsWestcreeP', [], 'Display', None],
  [ 0x144b, 'CanadianSyllabicsCarrierH', [], 'Display', None],
  [ 0x144c, 'CanadianSyllabicsTe', [], 'Display', None],
  [ 0x144d, 'CanadianSyllabicsTaai', [], 'Display', None],
  [ 0x144e, 'CanadianSyllabicsTi', [], 'Display', None],
  [ 0x144f, 'CanadianSyllabicsTii', [], 'Display', None],
  [ 0x1450, 'CanadianSyllabicsTo', [], 'Display', None],
  [ 0x1451, 'CanadianSyllabicsToo', [], 'Display', None],
  [ 0x1452, 'CanadianSyllabicsYcreeToo', [], 'Display', None],
  [ 0x1453, 'CanadianSyllabicsCarrierDee', [], 'Display', None],
  [ 0x1454, 'CanadianSyllabicsCarrierDi', [], 'Display', None],
  [ 0x1455, 'CanadianSyllabicsTa', [], 'Display', None],
  [ 0x1456, 'CanadianSyllabicsTaa', [], 'Display', None],
  [ 0x1457, 'CanadianSyllabicsTwe', [], 'Display', None],
  [ 0x1458, 'CanadianSyllabicsWestcreeTwe', [], 'Display', None],
  [ 0x1459, 'CanadianSyllabicsTwi', [], 'Display', None],
  [ 0x145a, 'CanadianSyllabicsWestcreeTwi', [], 'Display', None],
  [ 0x145b, 'CanadianSyllabicsTwii', [], 'Display', None],
  [ 0x145c, 'CanadianSyllabicsWestcreeTwii', [], 'Display', None],
  [ 0x145d, 'CanadianSyllabicsTwo', [], 'Display', None],
  [ 0x145e, 'CanadianSyllabicsWestcreeTwo', [], 'Display', None],
  [ 0x145f, 'CanadianSyllabicsTwoo', [], 'Display', None],
  [ 0x1460, 'CanadianSyllabicsWestcreeTwoo', [], 'Display', None],
  [ 0x1461, 'CanadianSyllabicsTwa', [], 'Display', None],
  [ 0x1462, 'CanadianSyllabicsWestcreeTwa', [], 'Display', None],
  [ 0x1463, 'CanadianSyllabicsTwaa', [], 'Display', None],
  [ 0x1464, 'CanadianSyllabicsWestcreeTwaa', [], 'Display', None],
  [ 0x1465, 'CanadianSyllabicsNaskapiTwaa', [], 'Display', None],
  [ 0x1466, 'CanadianSyllabicsT', [], 'Display', None],
  [ 0x1467, 'CanadianSyllabicsTte', [], 'Display', None],
  [ 0x1468, 'CanadianSyllabicsTti', [], 'Display', None],
  [ 0x1469, 'CanadianSyllabicsTto', [], 'Display', None],
  [ 0x146a, 'CanadianSyllabicsTta', [], 'Display', None],
  [ 0x146b, 'CanadianSyllabicsKe', [], 'Display', None],
  [ 0x146c, 'CanadianSyllabicsKaai', [], 'Display', None],
  [ 0x146d, 'CanadianSyllabicsKi', [], 'Display', None],
  [ 0x146e, 'CanadianSyllabicsKii', [], 'Display', None],
  [ 0x146f, 'CanadianSyllabicsKo', [], 'Display', None],
  [ 0x1470, 'CanadianSyllabicsKoo', [], 'Display', None],
  [ 0x1471, 'CanadianSyllabicsYcreeKoo', [], 'Display', None],
  [ 0x1472, 'CanadianSyllabicsKa', [], 'Display', None],
  [ 0x1473, 'CanadianSyllabicsKaa', [], 'Display', None],
  [ 0x1474, 'CanadianSyllabicsKwe', [], 'Display', None],
  [ 0x1475, 'CanadianSyllabicsWestcreeKwe', [], 'Display', None],
  [ 0x1476, 'CanadianSyllabicsKwi', [], 'Display', None],
  [ 0x1477, 'CanadianSyllabicsWestcreeKwi', [], 'Display', None],
  [ 0x1478, 'CanadianSyllabicsKwii', [], 'Display', None],
  [ 0x1479, 'CanadianSyllabicsWestcreeKwii', [], 'Display', None],
  [ 0x147a, 'CanadianSyllabicsKwo', [], 'Display', None],
  [ 0x147b, 'CanadianSyllabicsWestcreeKwo', [], 'Display', None],
  [ 0x147c, 'CanadianSyllabicsKwoo', [], 'Display', None],
  [ 0x147d, 'CanadianSyllabicsWestcreeKwoo', [], 'Display', None],
  [ 0x147e, 'CanadianSyllabicsKwa', [], 'Display', None],
  [ 0x147f, 'CanadianSyllabicsWestcreeKwa', [], 'Display', None],
  [ 0x1480, 'CanadianSyllabicsKwaa', [], 'Display', None],
  [ 0x1481, 'CanadianSyllabicsWestcreeKwaa', [], 'Display', None],
  [ 0x1482, 'CanadianSyllabicsNaskapiKwaa', [], 'Display', None],
  [ 0x1483, 'CanadianSyllabicsK', [], 'Display', None],
  [ 0x1484, 'CanadianSyllabicsKw', [], 'Display', None],
  [ 0x1485, 'CanadianSyllabicsSouthslaveyKeh', [], 'Display', None],
  [ 0x1486, 'CanadianSyllabicsSouthslaveyKih', [], 'Display', None],
  [ 0x1487, 'CanadianSyllabicsSouthslaveyKoh', [], 'Display', None],
  [ 0x1488, 'CanadianSyllabicsSouthslaveyKah', [], 'Display', None],
  [ 0x1489, 'CanadianSyllabicsCe', [], 'Display', None],
  [ 0x148a, 'CanadianSyllabicsCaai', [], 'Display', None],
  [ 0x148b, 'CanadianSyllabicsCi', [], 'Display', None],
  [ 0x148c, 'CanadianSyllabicsCii', [], 'Display', None],
  [ 0x148d, 'CanadianSyllabicsCo', [], 'Display', None],
  [ 0x148e, 'CanadianSyllabicsCoo', [], 'Display', None],
  [ 0x148f, 'CanadianSyllabicsYcreeCoo', [], 'Display', None],
  [ 0x1490, 'CanadianSyllabicsCa', [], 'Display', None],
  [ 0x1491, 'CanadianSyllabicsCaa', [], 'Display', None],
  [ 0x1492, 'CanadianSyllabicsCwe', [], 'Display', None],
  [ 0x1493, 'CanadianSyllabicsWestcreeCwe', [], 'Display', None],
  [ 0x1494, 'CanadianSyllabicsCwi', [], 'Display', None],
  [ 0x1495, 'CanadianSyllabicsWestcreeCwi', [], 'Display', None],
  [ 0x1496, 'CanadianSyllabicsCwii', [], 'Display', None],
  [ 0x1497, 'CanadianSyllabicsWestcreeCwii', [], 'Display', None],
  [ 0x1498, 'CanadianSyllabicsCwo', [], 'Display', None],
  [ 0x1499, 'CanadianSyllabicsWestcreeCwo', [], 'Display', None],
  [ 0x149a, 'CanadianSyllabicsCwoo', [], 'Display', None],
  [ 0x149b, 'CanadianSyllabicsWestcreeCwoo', [], 'Display', None],
  [ 0x149c, 'CanadianSyllabicsCwa', [], 'Display', None],
  [ 0x149d, 'CanadianSyllabicsWestcreeCwa', [], 'Display', None],
  [ 0x149e, 'CanadianSyllabicsCwaa', [], 'Display', None],
  [ 0x149f, 'CanadianSyllabicsWestcreeCwaa', [], 'Display', None],
  [ 0x14a0, 'CanadianSyllabicsNaskapiCwaa', [], 'Display', None],
  [ 0x14a1, 'CanadianSyllabicsC', [], 'Display', None],
  [ 0x14a2, 'CanadianSyllabicsSayisiTh', [], 'Display', None],
  [ 0x14a3, 'CanadianSyllabicsMe', [], 'Display', None],
  [ 0x14a4, 'CanadianSyllabicsMaai', [], 'Display', None],
  [ 0x14a5, 'CanadianSyllabicsMi', [], 'Display', None],
  [ 0x14a6, 'CanadianSyllabicsMii', [], 'Display', None],
  [ 0x14a7, 'CanadianSyllabicsMo', [], 'Display', None],
  [ 0x14a8, 'CanadianSyllabicsMoo', [], 'Display', None],
  [ 0x14a9, 'CanadianSyllabicsYcreeMoo', [], 'Display', None],
  [ 0x14aa, 'CanadianSyllabicsMa', [], 'Display', None],
  [ 0x14ab, 'CanadianSyllabicsMaa', [], 'Display', None],
  [ 0x14ac, 'CanadianSyllabicsMwe', [], 'Display', None],
  [ 0x14ad, 'CanadianSyllabicsWestcreeMwe', [], 'Display', None],
  [ 0x14ae, 'CanadianSyllabicsMwi', [], 'Display', None],
  [ 0x14af, 'CanadianSyllabicsWestcreeMwi', [], 'Display', None],
  [ 0x14b0, 'CanadianSyllabicsMwii', [], 'Display', None],
  [ 0x14b1, 'CanadianSyllabicsWestcreeMwii', [], 'Display', None],
  [ 0x14b2, 'CanadianSyllabicsMwo', [], 'Display', None],
  [ 0x14b3, 'CanadianSyllabicsWestcreeMwo', [], 'Display', None],
  [ 0x14b4, 'CanadianSyllabicsMwoo', [], 'Display', None],
  [ 0x14b5, 'CanadianSyllabicsWestcreeMwoo', [], 'Display', None],
  [ 0x14b6, 'CanadianSyllabicsMwa', [], 'Display', None],
  [ 0x14b7, 'CanadianSyllabicsWestcreeMwa', [], 'Display', None],
  [ 0x14b8, 'CanadianSyllabicsMwaa', [], 'Display', None],
  [ 0x14b9, 'CanadianSyllabicsWestcreeMwaa', [], 'Display', None],
  [ 0x14ba, 'CanadianSyllabicsNaskapiMwaa', [], 'Display', None],
  [ 0x14bb, 'CanadianSyllabicsM', [], 'Display', None],
  [ 0x14bc, 'CanadianSyllabicsWestcreeM', [], 'Display', None],
  [ 0x14bd, 'CanadianSyllabicsMh', [], 'Display', None],
  [ 0x14be, 'CanadianSyllabicsAthapascanM', [], 'Display', None],
  [ 0x14bf, 'CanadianSyllabicsSayisiM', [], 'Display', None],
  [ 0x14c0, 'CanadianSyllabicsNe', [], 'Display', None],
  [ 0x14c1, 'CanadianSyllabicsNaai', [], 'Display', None],
  [ 0x14c2, 'CanadianSyllabicsNi', [], 'Display', None],
  [ 0x14c3, 'CanadianSyllabicsNii', [], 'Display', None],
  [ 0x14c4, 'CanadianSyllabicsNo', [], 'Display', None],
  [ 0x14c5, 'CanadianSyllabicsNoo', [], 'Display', None],
  [ 0x14c6, 'CanadianSyllabicsYcreeNoo', [], 'Display', None],
  [ 0x14c7, 'CanadianSyllabicsNa', [], 'Display', None],
  [ 0x14c8, 'CanadianSyllabicsNaa', [], 'Display', None],
  [ 0x14c9, 'CanadianSyllabicsNwe', [], 'Display', None],
  [ 0x14ca, 'CanadianSyllabicsWestcreeNwe', [], 'Display', None],
  [ 0x14cb, 'CanadianSyllabicsNwa', [], 'Display', None],
  [ 0x14cc, 'CanadianSyllabicsWestcreeNwa', [], 'Display', None],
  [ 0x14cd, 'CanadianSyllabicsNwaa', [], 'Display', None],
  [ 0x14ce, 'CanadianSyllabicsWestcreeNwaa', [], 'Display', None],
  [ 0x14cf, 'CanadianSyllabicsNaskapiNwaa', [], 'Display', None],
  [ 0x14d0, 'CanadianSyllabicsN', [], 'Display', None],
  [ 0x14d1, 'CanadianSyllabicsCarrierNg', [], 'Display', None],
  [ 0x14d2, 'CanadianSyllabicsNh', [], 'Display', None],
  [ 0x14d3, 'CanadianSyllabicsLe', [], 'Display', None],
  [ 0x14d4, 'CanadianSyllabicsLaai', [], 'Display', None],
  [ 0x14d5, 'CanadianSyllabicsLi', [], 'Display', None],
  [ 0x14d6, 'CanadianSyllabicsLii', [], 'Display', None],
  [ 0x14d7, 'CanadianSyllabicsLo', [], 'Display', None],
  [ 0x14d8, 'CanadianSyllabicsLoo', [], 'Display', None],
  [ 0x14d9, 'CanadianSyllabicsYcreeLoo', [], 'Display', None],
  [ 0x14da, 'CanadianSyllabicsLa', [], 'Display', None],
  [ 0x14db, 'CanadianSyllabicsLaa', [], 'Display', None],
  [ 0x14dc, 'CanadianSyllabicsLwe', [], 'Display', None],
  [ 0x14dd, 'CanadianSyllabicsWestcreeLwe', [], 'Display', None],
  [ 0x14de, 'CanadianSyllabicsLwi', [], 'Display', None],
  [ 0x14df, 'CanadianSyllabicsWestcreeLwi', [], 'Display', None],
  [ 0x14e0, 'CanadianSyllabicsLwii', [], 'Display', None],
  [ 0x14e1, 'CanadianSyllabicsWestcreeLwii', [], 'Display', None],
  [ 0x14e2, 'CanadianSyllabicsLwo', [], 'Display', None],
  [ 0x14e3, 'CanadianSyllabicsWestcreeLwo', [], 'Display', None],
  [ 0x14e4, 'CanadianSyllabicsLwoo', [], 'Display', None],
  [ 0x14e5, 'CanadianSyllabicsWestcreeLwoo', [], 'Display', None],
  [ 0x14e6, 'CanadianSyllabicsLwa', [], 'Display', None],
  [ 0x14e7, 'CanadianSyllabicsWestcreeLwa', [], 'Display', None],
  [ 0x14e8, 'CanadianSyllabicsLwaa', [], 'Display', None],
  [ 0x14e9, 'CanadianSyllabicsWestcreeLwaa', [], 'Display', None],
  [ 0x14ea, 'CanadianSyllabicsL', [], 'Display', None],
  [ 0x14eb, 'CanadianSyllabicsWestcreeL', [], 'Display', None],
  [ 0x14ec, 'CanadianSyllabicsMedialL', [], 'Display', None],
  [ 0x14ed, 'CanadianSyllabicsSe', [], 'Display', None],
  [ 0x14ee, 'CanadianSyllabicsSaai', [], 'Display', None],
  [ 0x14ef, 'CanadianSyllabicsSi', [], 'Display', None],
  [ 0x14f0, 'CanadianSyllabicsSii', [], 'Display', None],
  [ 0x14f1, 'CanadianSyllabicsSo', [], 'Display', None],
  [ 0x14f2, 'CanadianSyllabicsSoo', [], 'Display', None],
  [ 0x14f3, 'CanadianSyllabicsYcreeSoo', [], 'Display', None],
  [ 0x14f4, 'CanadianSyllabicsSa', [], 'Display', None],
  [ 0x14f5, 'CanadianSyllabicsSaa', [], 'Display', None],
  [ 0x14f6, 'CanadianSyllabicsSwe', [], 'Display', None],
  [ 0x14f7, 'CanadianSyllabicsWestcreeSwe', [], 'Display', None],
  [ 0x14f8, 'CanadianSyllabicsSwi', [], 'Display', None],
  [ 0x14f9, 'CanadianSyllabicsWestcreeSwi', [], 'Display', None],
  [ 0x14fa, 'CanadianSyllabicsSwii', [], 'Display', None],
  [ 0x14fb, 'CanadianSyllabicsWestcreeSwii', [], 'Display', None],
  [ 0x14fc, 'CanadianSyllabicsSwo', [], 'Display', None],
  [ 0x14fd, 'CanadianSyllabicsWestcreeSwo', [], 'Display', None],
  [ 0x14fe, 'CanadianSyllabicsSwoo', [], 'Display', None],
  [ 0x14ff, 'CanadianSyllabicsWestcreeSwoo', [], 'Display', None],
  [ 0x1500, 'CanadianSyllabicsSwa', [], 'Display', None],
  [ 0x1501, 'CanadianSyllabicsWestcreeSwa', [], 'Display', None],
  [ 0x1502, 'CanadianSyllabicsSwaa', [], 'Display', None],
  [ 0x1503, 'CanadianSyllabicsWestcreeSwaa', [], 'Display', None],
  [ 0x1504, 'CanadianSyllabicsNaskapiSwaa', [], 'Display', None],
  [ 0x1505, 'CanadianSyllabicsS', [], 'Display', None],
  [ 0x1506, 'CanadianSyllabicsAthapascanS', [], 'Display', None],
  [ 0x1507, 'CanadianSyllabicsSw', [], 'Display', None],
  [ 0x1508, 'CanadianSyllabicsBlackfootS', [], 'Display', None],
  [ 0x1509, 'CanadianSyllabicsMoosecreeSk', [], 'Display', None],
  [ 0x150a, 'CanadianSyllabicsNaskapiSkw', [], 'Display', None],
  [ 0x150b, 'CanadianSyllabicsNaskapiSw', [], 'Display', None],
  [ 0x150c, 'CanadianSyllabicsNaskapiSpwa', [], 'Display', None],
  [ 0x150d, 'CanadianSyllabicsNaskapiStwa', [], 'Display', None],
  [ 0x150e, 'CanadianSyllabicsNaskapiSkwa', [], 'Display', None],
  [ 0x150f, 'CanadianSyllabicsNaskapiScwa', [], 'Display', None],
  [ 0x1510, 'CanadianSyllabicsShe', [], 'Display', None],
  [ 0x1511, 'CanadianSyllabicsShi', [], 'Display', None],
  [ 0x1512, 'CanadianSyllabicsShii', [], 'Display', None],
  [ 0x1513, 'CanadianSyllabicsSho', [], 'Display', None],
  [ 0x1514, 'CanadianSyllabicsShoo', [], 'Display', None],
  [ 0x1515, 'CanadianSyllabicsSha', [], 'Display', None],
  [ 0x1516, 'CanadianSyllabicsShaa', [], 'Display', None],
  [ 0x1517, 'CanadianSyllabicsShwe', [], 'Display', None],
  [ 0x1518, 'CanadianSyllabicsWestcreeShwe', [], 'Display', None],
  [ 0x1519, 'CanadianSyllabicsShwi', [], 'Display', None],
  [ 0x151a, 'CanadianSyllabicsWestcreeShwi', [], 'Display', None],
  [ 0x151b, 'CanadianSyllabicsShwii', [], 'Display', None],
  [ 0x151c, 'CanadianSyllabicsWestcreeShwii', [], 'Display', None],
  [ 0x151d, 'CanadianSyllabicsShwo', [], 'Display', None],
  [ 0x151e, 'CanadianSyllabicsWestcreeShwo', [], 'Display', None],
  [ 0x151f, 'CanadianSyllabicsShwoo', [], 'Display', None],
  [ 0x1520, 'CanadianSyllabicsWestcreeShwoo', [], 'Display', None],
  [ 0x1521, 'CanadianSyllabicsShwa', [], 'Display', None],
  [ 0x1522, 'CanadianSyllabicsWestcreeShwa', [], 'Display', None],
  [ 0x1523, 'CanadianSyllabicsShwaa', [], 'Display', None],
  [ 0x1524, 'CanadianSyllabicsWestcreeShwaa', [], 'Display', None],
  [ 0x1525, 'CanadianSyllabicsSh', [], 'Display', None],
  [ 0x1526, 'CanadianSyllabicsYe', [], 'Display', None],
  [ 0x1527, 'CanadianSyllabicsYaai', [], 'Display', None],
  [ 0x1528, 'CanadianSyllabicsYi', [], 'Display', None],
  [ 0x1529, 'CanadianSyllabicsYii', [], 'Display', None],
  [ 0x152a, 'CanadianSyllabicsYo', [], 'Display', None],
  [ 0x152b, 'CanadianSyllabicsYoo', [], 'Display', None],
  [ 0x152c, 'CanadianSyllabicsYcreeYoo', [], 'Display', None],
  [ 0x152d, 'CanadianSyllabicsYa', [], 'Display', None],
  [ 0x152e, 'CanadianSyllabicsYaa', [], 'Display', None],
  [ 0x152f, 'CanadianSyllabicsYwe', [], 'Display', None],
  [ 0x1530, 'CanadianSyllabicsWestcreeYwe', [], 'Display', None],
  [ 0x1531, 'CanadianSyllabicsYwi', [], 'Display', None],
  [ 0x1532, 'CanadianSyllabicsWestcreeYwi', [], 'Display', None],
  [ 0x1533, 'CanadianSyllabicsYwii', [], 'Display', None],
  [ 0x1534, 'CanadianSyllabicsWestcreeYwii', [], 'Display', None],
  [ 0x1535, 'CanadianSyllabicsYwo', [], 'Display', None],
  [ 0x1536, 'CanadianSyllabicsWestcreeYwo', [], 'Display', None],
  [ 0x1537, 'CanadianSyllabicsYwoo', [], 'Display', None],
  [ 0x1538, 'CanadianSyllabicsWestcreeYwoo', [], 'Display', None],
  [ 0x1539, 'CanadianSyllabicsYwa', [], 'Display', None],
  [ 0x153a, 'CanadianSyllabicsWestcreeYwa', [], 'Display', None],
  [ 0x153b, 'CanadianSyllabicsYwaa', [], 'Display', None],
  [ 0x153c, 'CanadianSyllabicsWestcreeYwaa', [], 'Display', None],
  [ 0x153d, 'CanadianSyllabicsNaskapiYwaa', [], 'Display', None],
  [ 0x153e, 'CanadianSyllabicsY', [], 'Display', None],
  [ 0x153f, 'CanadianSyllabicsBiblecreeY', [], 'Display', None],
  [ 0x1540, 'CanadianSyllabicsWestcreeY', [], 'Display', None],
  [ 0x1541, 'CanadianSyllabicsSayisiYi', [], 'Display', None],
  [ 0x1542, 'CanadianSyllabicsRe', [], 'Display', None],
  [ 0x1543, 'CanadianSyllabicsRcreeRe', [], 'Display', None],
  [ 0x1544, 'CanadianSyllabicsWestcreeLe', [], 'Display', None],
  [ 0x1545, 'CanadianSyllabicsRaai', [], 'Display', None],
  [ 0x1546, 'CanadianSyllabicsRi', [], 'Display', None],
  [ 0x1547, 'CanadianSyllabicsRii', [], 'Display', None],
  [ 0x1548, 'CanadianSyllabicsRo', [], 'Display', None],
  [ 0x1549, 'CanadianSyllabicsRoo', [], 'Display', None],
  [ 0x154a, 'CanadianSyllabicsWestcreeLo', [], 'Display', None],
  [ 0x154b, 'CanadianSyllabicsRa', [], 'Display', None],
  [ 0x154c, 'CanadianSyllabicsRaa', [], 'Display', None],
  [ 0x154d, 'CanadianSyllabicsWestcreeLa', [], 'Display', None],
  [ 0x154e, 'CanadianSyllabicsRwaa', [], 'Display', None],
  [ 0x154f, 'CanadianSyllabicsWestcreeRwaa', [], 'Display', None],
  [ 0x1550, 'CanadianSyllabicsR', [], 'Display', None],
  [ 0x1551, 'CanadianSyllabicsWestcreeR', [], 'Display', None],
  [ 0x1552, 'CanadianSyllabicsMedialR', [], 'Display', None],
  [ 0x1553, 'CanadianSyllabicsFe', [], 'Display', None],
  [ 0x1554, 'CanadianSyllabicsFaai', [], 'Display', None],
  [ 0x1555, 'CanadianSyllabicsFi', [], 'Display', None],
  [ 0x1556, 'CanadianSyllabicsFii', [], 'Display', None],
  [ 0x1557, 'CanadianSyllabicsFo', [], 'Display', None],
  [ 0x1558, 'CanadianSyllabicsFoo', [], 'Display', None],
  [ 0x1559, 'CanadianSyllabicsFa', [], 'Display', None],
  [ 0x155a, 'CanadianSyllabicsFaa', [], 'Display', None],
  [ 0x155b, 'CanadianSyllabicsFwaa', [], 'Display', None],
  [ 0x155c, 'CanadianSyllabicsWestcreeFwaa', [], 'Display', None],
  [ 0x155d, 'CanadianSyllabicsF', [], 'Display', None],
  [ 0x155e, 'CanadianSyllabicsThe', [], 'Display', None],
  [ 0x155f, 'CanadianSyllabicsNcreeThe', [], 'Display', None],
  [ 0x1560, 'CanadianSyllabicsThi', [], 'Display', None],
  [ 0x1561, 'CanadianSyllabicsNcreeThi', [], 'Display', None],
  [ 0x1562, 'CanadianSyllabicsThii', [], 'Display', None],
  [ 0x1563, 'CanadianSyllabicsNcreeThii', [], 'Display', None],
  [ 0x1564, 'CanadianSyllabicsTho', [], 'Display', None],
  [ 0x1565, 'CanadianSyllabicsThoo', [], 'Display', None],
  [ 0x1566, 'CanadianSyllabicsTha', [], 'Display', None],
  [ 0x1567, 'CanadianSyllabicsThaa', [], 'Display', None],
  [ 0x1568, 'CanadianSyllabicsThwaa', [], 'Display', None],
  [ 0x1569, 'CanadianSyllabicsWestcreeThwaa', [], 'Display', None],
  [ 0x156a, 'CanadianSyllabicsTh', [], 'Display', None],
  [ 0x156b, 'CanadianSyllabicsTthe', [], 'Display', None],
  [ 0x156c, 'CanadianSyllabicsTthi', [], 'Display', None],
  [ 0x156d, 'CanadianSyllabicsTtho', [], 'Display', None],
  [ 0x156e, 'CanadianSyllabicsTtha', [], 'Display', None],
  [ 0x156f, 'CanadianSyllabicsTth', [], 'Display', None],
  [ 0x1570, 'CanadianSyllabicsTye', [], 'Display', None],
  [ 0x1571, 'CanadianSyllabicsTyi', [], 'Display', None],
  [ 0x1572, 'CanadianSyllabicsTyo', [], 'Display', None],
  [ 0x1573, 'CanadianSyllabicsTya', [], 'Display', None],
  [ 0x1574, 'CanadianSyllabicsNunavikHe', [], 'Display', None],
  [ 0x1575, 'CanadianSyllabicsNunavikHi', [], 'Display', None],
  [ 0x1576, 'CanadianSyllabicsNunavikHii', [], 'Display', None],
  [ 0x1577, 'CanadianSyllabicsNunavikHo', [], 'Display', None],
  [ 0x1578, 'CanadianSyllabicsNunavikHoo', [], 'Display', None],
  [ 0x1579, 'CanadianSyllabicsNunavikHa', [], 'Display', None],
  [ 0x157a, 'CanadianSyllabicsNunavikHaa', [], 'Display', None],
  [ 0x157b, 'CanadianSyllabicsNunavikH', [], 'Display', None],
  [ 0x157c, 'CanadianSyllabicsNunavutH', [], 'Display', None],
  [ 0x157d, 'CanadianSyllabicsHk', [], 'Display', None],
  [ 0x157e, 'CanadianSyllabicsQaai', [], 'Display', None],
  [ 0x157f, 'CanadianSyllabicsQi', [], 'Display', None],
  [ 0x1580, 'CanadianSyllabicsQii', [], 'Display', None],
  [ 0x1581, 'CanadianSyllabicsQo', [], 'Display', None],
  [ 0x1582, 'CanadianSyllabicsQoo', [], 'Display', None],
  [ 0x1583, 'CanadianSyllabicsQa', [], 'Display', None],
  [ 0x1584, 'CanadianSyllabicsQaa', [], 'Display', None],
  [ 0x1585, 'CanadianSyllabicsQ', [], 'Display', None],
  [ 0x1586, 'CanadianSyllabicsTlhe', [], 'Display', None],
  [ 0x1587, 'CanadianSyllabicsTlhi', [], 'Display', None],
  [ 0x1588, 'CanadianSyllabicsTlho', [], 'Display', None],
  [ 0x1589, 'CanadianSyllabicsTlha', [], 'Display', None],
  [ 0x158a, 'CanadianSyllabicsWestcreeRe', [], 'Display', None],
  [ 0x158b, 'CanadianSyllabicsWestcreeRi', [], 'Display', None],
  [ 0x158c, 'CanadianSyllabicsWestcreeRo', [], 'Display', None],
  [ 0x158d, 'CanadianSyllabicsWestcreeRa', [], 'Display', None],
  [ 0x158e, 'CanadianSyllabicsNgaai', [], 'Display', None],
  [ 0x158f, 'CanadianSyllabicsNgi', [], 'Display', None],
  [ 0x1590, 'CanadianSyllabicsNgii', [], 'Display', None],
  [ 0x1591, 'CanadianSyllabicsNgo', [], 'Display', None],
  [ 0x1592, 'CanadianSyllabicsNgoo', [], 'Display', None],
  [ 0x1593, 'CanadianSyllabicsNga', [], 'Display', None],
  [ 0x1594, 'CanadianSyllabicsNgaa', [], 'Display', None],
  [ 0x1595, 'CanadianSyllabicsNg', [], 'Display', None],
  [ 0x1596, 'CanadianSyllabicsNng', [], 'Display', None],
  [ 0x1597, 'CanadianSyllabicsSayisiShe', [], 'Display', None],
  [ 0x1598, 'CanadianSyllabicsSayisiShi', [], 'Display', None],
  [ 0x1599, 'CanadianSyllabicsSayisiSho', [], 'Display', None],
  [ 0x159a, 'CanadianSyllabicsSayisiSha', [], 'Display', None],
  [ 0x159b, 'CanadianSyllabicsWoodscreeThe', [], 'Display', None],
  [ 0x159c, 'CanadianSyllabicsWoodscreeThi', [], 'Display', None],
  [ 0x159d, 'CanadianSyllabicsWoodscreeTho', [], 'Display', None],
  [ 0x159e, 'CanadianSyllabicsWoodscreeTha', [], 'Display', None],
  [ 0x159f, 'CanadianSyllabicsWoodscreeTh', [], 'Display', None],
  [ 0x15a0, 'CanadianSyllabicsLhi', [], 'Display', None],
  [ 0x15a1, 'CanadianSyllabicsLhii', [], 'Display', None],
  [ 0x15a2, 'CanadianSyllabicsLho', [], 'Display', None],
  [ 0x15a3, 'CanadianSyllabicsLhoo', [], 'Display', None],
  [ 0x15a4, 'CanadianSyllabicsLha', [], 'Display', None],
  [ 0x15a5, 'CanadianSyllabicsLhaa', [], 'Display', None],
  [ 0x15a6, 'CanadianSyllabicsLh', [], 'Display', None],
  [ 0x15a7, 'CanadianSyllabicsThcreeThe', [], 'Display', None],
  [ 0x15a8, 'CanadianSyllabicsThcreeThi', [], 'Display', None],
  [ 0x15a9, 'CanadianSyllabicsThcreeThii', [], 'Display', None],
  [ 0x15aa, 'CanadianSyllabicsThcreeTho', [], 'Display', None],
  [ 0x15ab, 'CanadianSyllabicsThcreeThoo', [], 'Display', None],
  [ 0x15ac, 'CanadianSyllabicsThcreeTha', [], 'Display', None],
  [ 0x15ad, 'CanadianSyllabicsThcreeThaa', [], 'Display', None],
  [ 0x15ae, 'CanadianSyllabicsThcreeTh', [], 'Display', None],
  [ 0x15af, 'CanadianSyllabicsAivilikB', [], 'Display', None],
  [ 0x15b0, 'CanadianSyllabicsBlackfootE', [], 'Display', None],
  [ 0x15b1, 'CanadianSyllabicsBlackfootI', [], 'Display', None],
  [ 0x15b2, 'CanadianSyllabicsBlackfootO', [], 'Display', None],
  [ 0x15b3, 'CanadianSyllabicsBlackfootA', [], 'Display', None],
  [ 0x15b4, 'CanadianSyllabicsBlackfootWe', [], 'Display', None],
  [ 0x15b5, 'CanadianSyllabicsBlackfootWi', [], 'Display', None],
  [ 0x15b6, 'CanadianSyllabicsBlackfootWo', [], 'Display', None],
  [ 0x15b7, 'CanadianSyllabicsBlackfootWa', [], 'Display', None],
  [ 0x15b8, 'CanadianSyllabicsBlackfootNe', [], 'Display', None],
  [ 0x15b9, 'CanadianSyllabicsBlackfootNi', [], 'Display', None],
  [ 0x15ba, 'CanadianSyllabicsBlackfootNo', [], 'Display', None],
  [ 0x15bb, 'CanadianSyllabicsBlackfootNa', [], 'Display', None],
  [ 0x15bc, 'CanadianSyllabicsBlackfootKe', [], 'Display', None],
  [ 0x15bd, 'CanadianSyllabicsBlackfootKi', [], 'Display', None],
  [ 0x15be, 'CanadianSyllabicsBlackfootKo', [], 'Display', None],
  [ 0x15bf, 'CanadianSyllabicsBlackfootKa', [], 'Display', None],
  [ 0x15c0, 'CanadianSyllabicsSayisiHe', [], 'Display', None],
  [ 0x15c1, 'CanadianSyllabicsSayisiHi', [], 'Display', None],
  [ 0x15c2, 'CanadianSyllabicsSayisiHo', [], 'Display', None],
  [ 0x15c3, 'CanadianSyllabicsSayisiHa', [], 'Display', None],
  [ 0x15c4, 'CanadianSyllabicsCarrierGhu', [], 'Display', None],
  [ 0x15c5, 'CanadianSyllabicsCarrierGho', [], 'Display', None],
  [ 0x15c6, 'CanadianSyllabicsCarrierGhe', [], 'Display', None],
  [ 0x15c7, 'CanadianSyllabicsCarrierGhee', [], 'Display', None],
  [ 0x15c8, 'CanadianSyllabicsCarrierGhi', [], 'Display', None],
  [ 0x15c9, 'CanadianSyllabicsCarrierGha', [], 'Display', None],
  [ 0x15ca, 'CanadianSyllabicsCarrierRu', [], 'Display', None],
  [ 0x15cb, 'CanadianSyllabicsCarrierRo', [], 'Display', None],
  [ 0x15cc, 'CanadianSyllabicsCarrierRe', [], 'Display', None],
  [ 0x15cd, 'CanadianSyllabicsCarrierRee', [], 'Display', None],
  [ 0x15ce, 'CanadianSyllabicsCarrierRi', [], 'Display', None],
  [ 0x15cf, 'CanadianSyllabicsCarrierRa', [], 'Display', None],
  [ 0x15d0, 'CanadianSyllabicsCarrierWu', [], 'Display', None],
  [ 0x15d1, 'CanadianSyllabicsCarrierWo', [], 'Display', None],
  [ 0x15d2, 'CanadianSyllabicsCarrierWe', [], 'Display', None],
  [ 0x15d3, 'CanadianSyllabicsCarrierWee', [], 'Display', None],
  [ 0x15d4, 'CanadianSyllabicsCarrierWi', [], 'Display', None],
  [ 0x15d5, 'CanadianSyllabicsCarrierWa', [], 'Display', None],
  [ 0x15d6, 'CanadianSyllabicsCarrierHwu', [], 'Display', None],
  [ 0x15d7, 'CanadianSyllabicsCarrierHwo', [], 'Display', None],
  [ 0x15d8, 'CanadianSyllabicsCarrierHwe', [], 'Display', None],
  [ 0x15d9, 'CanadianSyllabicsCarrierHwee', [], 'Display', None],
  [ 0x15da, 'CanadianSyllabicsCarrierHwi', [], 'Display', None],
  [ 0x15db, 'CanadianSyllabicsCarrierHwa', [], 'Display', None],
  [ 0x15dc, 'CanadianSyllabicsCarrierThu', [], 'Display', None],
  [ 0x15dd, 'CanadianSyllabicsCarrierTho', [], 'Display', None],
  [ 0x15de, 'CanadianSyllabicsCarrierThe', [], 'Display', None],
  [ 0x15df, 'CanadianSyllabicsCarrierThee', [], 'Display', None],
  [ 0x15e0, 'CanadianSyllabicsCarrierThi', [], 'Display', None],
  [ 0x15e1, 'CanadianSyllabicsCarrierTha', [], 'Display', None],
  [ 0x15e2, 'CanadianSyllabicsCarrierTtu', [], 'Display', None],
  [ 0x15e3, 'CanadianSyllabicsCarrierTto', [], 'Display', None],
  [ 0x15e4, 'CanadianSyllabicsCarrierTte', [], 'Display', None],
  [ 0x15e5, 'CanadianSyllabicsCarrierTtee', [], 'Display', None],
  [ 0x15e6, 'CanadianSyllabicsCarrierTti', [], 'Display', None],
  [ 0x15e7, 'CanadianSyllabicsCarrierTta', [], 'Display', None],
  [ 0x15e8, 'CanadianSyllabicsCarrierPu', [], 'Display', None],
  [ 0x15e9, 'CanadianSyllabicsCarrierPo', [], 'Display', None],
  [ 0x15ea, 'CanadianSyllabicsCarrierPe', [], 'Display', None],
  [ 0x15eb, 'CanadianSyllabicsCarrierPee', [], 'Display', None],
  [ 0x15ec, 'CanadianSyllabicsCarrierPi', [], 'Display', None],
  [ 0x15ed, 'CanadianSyllabicsCarrierPa', [], 'Display', None],
  [ 0x15ee, 'CanadianSyllabicsCarrierP', [], 'Display', None],
  [ 0x15ef, 'CanadianSyllabicsCarrierGu', [], 'Display', None],
  [ 0x15f0, 'CanadianSyllabicsCarrierGo', [], 'Display', None],
  [ 0x15f1, 'CanadianSyllabicsCarrierGe', [], 'Display', None],
  [ 0x15f2, 'CanadianSyllabicsCarrierGee', [], 'Display', None],
  [ 0x15f3, 'CanadianSyllabicsCarrierGi', [], 'Display', None],
  [ 0x15f4, 'CanadianSyllabicsCarrierGa', [], 'Display', None],
  [ 0x15f5, 'CanadianSyllabicsCarrierKhu', [], 'Display', None],
  [ 0x15f6, 'CanadianSyllabicsCarrierKho', [], 'Display', None],
  [ 0x15f7, 'CanadianSyllabicsCarrierKhe', [], 'Display', None],
  [ 0x15f8, 'CanadianSyllabicsCarrierKhee', [], 'Display', None],
  [ 0x15f9, 'CanadianSyllabicsCarrierKhi', [], 'Display', None],
  [ 0x15fa, 'CanadianSyllabicsCarrierKha', [], 'Display', None],
  [ 0x15fb, 'CanadianSyllabicsCarrierKku', [], 'Display', None],
  [ 0x15fc, 'CanadianSyllabicsCarrierKko', [], 'Display', None],
  [ 0x15fd, 'CanadianSyllabicsCarrierKke', [], 'Display', None],
  [ 0x15fe, 'CanadianSyllabicsCarrierKkee', [], 'Display', None],
  [ 0x15ff, 'CanadianSyllabicsCarrierKki', [], 'Display', None],
  [ 0x1600, 'CanadianSyllabicsCarrierKka', [], 'Display', None],
  [ 0x1601, 'CanadianSyllabicsCarrierKk', [], 'Display', None],
  [ 0x1602, 'CanadianSyllabicsCarrierNu', [], 'Display', None],
  [ 0x1603, 'CanadianSyllabicsCarrierNo', [], 'Display', None],
  [ 0x1604, 'CanadianSyllabicsCarrierNe', [], 'Display', None],
  [ 0x1605, 'CanadianSyllabicsCarrierNee', [], 'Display', None],
  [ 0x1606, 'CanadianSyllabicsCarrierNi', [], 'Display', None],
  [ 0x1607, 'CanadianSyllabicsCarrierNa', [], 'Display', None],
  [ 0x1608, 'CanadianSyllabicsCarrierMu', [], 'Display', None],
  [ 0x1609, 'CanadianSyllabicsCarrierMo', [], 'Display', None],
  [ 0x160a, 'CanadianSyllabicsCarrierMe', [], 'Display', None],
  [ 0x160b, 'CanadianSyllabicsCarrierMee', [], 'Display', None],
  [ 0x160c, 'CanadianSyllabicsCarrierMi', [], 'Display', None],
  [ 0x160d, 'CanadianSyllabicsCarrierMa', [], 'Display', None],
  [ 0x160e, 'CanadianSyllabicsCarrierYu', [], 'Display', None],
  [ 0x160f, 'CanadianSyllabicsCarrierYo', [], 'Display', None],
  [ 0x1610, 'CanadianSyllabicsCarrierYe', [], 'Display', None],
  [ 0x1611, 'CanadianSyllabicsCarrierYee', [], 'Display', None],
  [ 0x1612, 'CanadianSyllabicsCarrierYi', [], 'Display', None],
  [ 0x1613, 'CanadianSyllabicsCarrierYa', [], 'Display', None],
  [ 0x1614, 'CanadianSyllabicsCarrierJu', [], 'Display', None],
  [ 0x1615, 'CanadianSyllabicsSayisiJu', [], 'Display', None],
  [ 0x1616, 'CanadianSyllabicsCarrierJo', [], 'Display', None],
  [ 0x1617, 'CanadianSyllabicsCarrierJe', [], 'Display', None],
  [ 0x1618, 'CanadianSyllabicsCarrierJee', [], 'Display', None],
  [ 0x1619, 'CanadianSyllabicsCarrierJi', [], 'Display', None],
  [ 0x161a, 'CanadianSyllabicsSayisiJi', [], 'Display', None],
  [ 0x161b, 'CanadianSyllabicsCarrierJa', [], 'Display', None],
  [ 0x161c, 'CanadianSyllabicsCarrierJju', [], 'Display', None],
  [ 0x161d, 'CanadianSyllabicsCarrierJjo', [], 'Display', None],
  [ 0x161e, 'CanadianSyllabicsCarrierJje', [], 'Display', None],
  [ 0x161f, 'CanadianSyllabicsCarrierJjee', [], 'Display', None],
  [ 0x1620, 'CanadianSyllabicsCarrierJji', [], 'Display', None],
  [ 0x1621, 'CanadianSyllabicsCarrierJja', [], 'Display', None],
  [ 0x1622, 'CanadianSyllabicsCarrierLu', [], 'Display', None],
  [ 0x1623, 'CanadianSyllabicsCarrierLo', [], 'Display', None],
  [ 0x1624, 'CanadianSyllabicsCarrierLe', [], 'Display', None],
  [ 0x1625, 'CanadianSyllabicsCarrierLee', [], 'Display', None],
  [ 0x1626, 'CanadianSyllabicsCarrierLi', [], 'Display', None],
  [ 0x1627, 'CanadianSyllabicsCarrierLa', [], 'Display', None],
  [ 0x1628, 'CanadianSyllabicsCarrierDlu', [], 'Display', None],
  [ 0x1629, 'CanadianSyllabicsCarrierDlo', [], 'Display', None],
  [ 0x162a, 'CanadianSyllabicsCarrierDle', [], 'Display', None],
  [ 0x162b, 'CanadianSyllabicsCarrierDlee', [], 'Display', None],
  [ 0x162c, 'CanadianSyllabicsCarrierDli', [], 'Display', None],
  [ 0x162d, 'CanadianSyllabicsCarrierDla', [], 'Display', None],
  [ 0x162e, 'CanadianSyllabicsCarrierLhu', [], 'Display', None],
  [ 0x162f, 'CanadianSyllabicsCarrierLho', [], 'Display', None],
  [ 0x1630, 'CanadianSyllabicsCarrierLhe', [], 'Display', None],
  [ 0x1631, 'CanadianSyllabicsCarrierLhee', [], 'Display', None],
  [ 0x1632, 'CanadianSyllabicsCarrierLhi', [], 'Display', None],
  [ 0x1633, 'CanadianSyllabicsCarrierLha', [], 'Display', None],
  [ 0x1634, 'CanadianSyllabicsCarrierTlhu', [], 'Display', None],
  [ 0x1635, 'CanadianSyllabicsCarrierTlho', [], 'Display', None],
  [ 0x1636, 'CanadianSyllabicsCarrierTlhe', [], 'Display', None],
  [ 0x1637, 'CanadianSyllabicsCarrierTlhee', [], 'Display', None],
  [ 0x1638, 'CanadianSyllabicsCarrierTlhi', [], 'Display', None],
  [ 0x1639, 'CanadianSyllabicsCarrierTlha', [], 'Display', None],
  [ 0x163a, 'CanadianSyllabicsCarrierTlu', [], 'Display', None],
  [ 0x163b, 'CanadianSyllabicsCarrierTlo', [], 'Display', None],
  [ 0x163c, 'CanadianSyllabicsCarrierTle', [], 'Display', None],
  [ 0x163d, 'CanadianSyllabicsCarrierTlee', [], 'Display', None],
  [ 0x163e, 'CanadianSyllabicsCarrierTli', [], 'Display', None],
  [ 0x163f, 'CanadianSyllabicsCarrierTla', [], 'Display', None],
  [ 0x1640, 'CanadianSyllabicsCarrierZu', [], 'Display', None],
  [ 0x1641, 'CanadianSyllabicsCarrierZo', [], 'Display', None],
  [ 0x1642, 'CanadianSyllabicsCarrierZe', [], 'Display', None],
  [ 0x1643, 'CanadianSyllabicsCarrierZee', [], 'Display', None],
  [ 0x1644, 'CanadianSyllabicsCarrierZi', [], 'Display', None],
  [ 0x1645, 'CanadianSyllabicsCarrierZa', [], 'Display', None],
  [ 0x1646, 'CanadianSyllabicsCarrierZ', [], 'Display', None],
  [ 0x1647, 'CanadianSyllabicsCarrierInitialZ', [], 'Display', None],
  [ 0x1648, 'CanadianSyllabicsCarrierDzu', [], 'Display', None],
  [ 0x1649, 'CanadianSyllabicsCarrierDzo', [], 'Display', None],
  [ 0x164a, 'CanadianSyllabicsCarrierDze', [], 'Display', None],
  [ 0x164b, 'CanadianSyllabicsCarrierDzee', [], 'Display', None],
  [ 0x164c, 'CanadianSyllabicsCarrierDzi', [], 'Display', None],
  [ 0x164d, 'CanadianSyllabicsCarrierDza', [], 'Display', None],
  [ 0x164e, 'CanadianSyllabicsCarrierSu', [], 'Display', None],
  [ 0x164f, 'CanadianSyllabicsCarrierSo', [], 'Display', None],
  [ 0x1650, 'CanadianSyllabicsCarrierSe', [], 'Display', None],
  [ 0x1651, 'CanadianSyllabicsCarrierSee', [], 'Display', None],
  [ 0x1652, 'CanadianSyllabicsCarrierSi', [], 'Display', None],
  [ 0x1653, 'CanadianSyllabicsCarrierSa', [], 'Display', None],
  [ 0x1654, 'CanadianSyllabicsCarrierShu', [], 'Display', None],
  [ 0x1655, 'CanadianSyllabicsCarrierSho', [], 'Display', None],
  [ 0x1656, 'CanadianSyllabicsCarrierShe', [], 'Display', None],
  [ 0x1657, 'CanadianSyllabicsCarrierShee', [], 'Display', None],
  [ 0x1658, 'CanadianSyllabicsCarrierShi', [], 'Display', None],
  [ 0x1659, 'CanadianSyllabicsCarrierSha', [], 'Display', None],
  [ 0x165a, 'CanadianSyllabicsCarrierSh', [], 'Display', None],
  [ 0x165b, 'CanadianSyllabicsCarrierTsu', [], 'Display', None],
  [ 0x165c, 'CanadianSyllabicsCarrierTso', [], 'Display', None],
  [ 0x165d, 'CanadianSyllabicsCarrierTse', [], 'Display', None],
  [ 0x165e, 'CanadianSyllabicsCarrierTsee', [], 'Display', None],
  [ 0x165f, 'CanadianSyllabicsCarrierTsi', [], 'Display', None],
  [ 0x1660, 'CanadianSyllabicsCarrierTsa', [], 'Display', None],
  [ 0x1661, 'CanadianSyllabicsCarrierChu', [], 'Display', None],
  [ 0x1662, 'CanadianSyllabicsCarrierCho', [], 'Display', None],
  [ 0x1663, 'CanadianSyllabicsCarrierChe', [], 'Display', None],
  [ 0x1664, 'CanadianSyllabicsCarrierChee', [], 'Display', None],
  [ 0x1665, 'CanadianSyllabicsCarrierChi', [], 'Display', None],
  [ 0x1666, 'CanadianSyllabicsCarrierCha', [], 'Display', None],
  [ 0x1667, 'CanadianSyllabicsCarrierTtsu', [], 'Display', None],
  [ 0x1668, 'CanadianSyllabicsCarrierTtso', [], 'Display', None],
  [ 0x1669, 'CanadianSyllabicsCarrierTtse', [], 'Display', None],
  [ 0x166a, 'CanadianSyllabicsCarrierTtsee', [], 'Display', None],
  [ 0x166b, 'CanadianSyllabicsCarrierTtsi', [], 'Display', None],
  [ 0x166c, 'CanadianSyllabicsCarrierTtsa', [], 'Display', None],
  [ 0x166d, 'CanadianSyllabicsChiSign', [], 'Display', None],
  [ 0x166e, 'CanadianSyllabicsFullStop', [], 'Display', None],
  [ 0x166f, 'CanadianSyllabicsQai', [], 'Display', None],
  [ 0x1670, 'CanadianSyllabicsNgai', [], 'Display', None],
  [ 0x1671, 'CanadianSyllabicsNngi', [], 'Display', None],
  [ 0x1672, 'CanadianSyllabicsNngii', [], 'Display', None],
  [ 0x1673, 'CanadianSyllabicsNngo', [], 'Display', None],
  [ 0x1674, 'CanadianSyllabicsNngoo', [], 'Display', None],
  [ 0x1675, 'CanadianSyllabicsNnga', [], 'Display', None],
  [ 0x1676, 'CanadianSyllabicsNngaa', [], 'Display', None],
  [ 0x1677, 'UnicodeCharacter0x1677', [], 'Display', None],
  [ 0x1678, 'UnicodeCharacter0x1678', [], 'Display', None],
  [ 0x1679, 'UnicodeCharacter0x1679', [], 'Display', None],
  [ 0x167a, 'UnicodeCharacter0x167a', [], 'Display', None],
  [ 0x167b, 'UnicodeCharacter0x167b', [], 'Display', None],
  [ 0x167c, 'UnicodeCharacter0x167c', [], 'Display', None],
  [ 0x167d, 'UnicodeCharacter0x167d', [], 'Display', None],
  [ 0x167e, 'UnicodeCharacter0x167e', [], 'Display', None],
  [ 0x167f, 'UnicodeCharacter0x167f', [], 'Display', None],
  [ 0x1680, 'OghamSpaceMark', [], 'Display', None],
  [ 0x1681, 'OghamLetterBeith', [], 'Display', None],
  [ 0x1682, 'OghamLetterLuis', [], 'Display', None],
  [ 0x1683, 'OghamLetterFearn', [], 'Display', None],
  [ 0x1684, 'OghamLetterSail', [], 'Display', None],
  [ 0x1685, 'OghamLetterNion', [], 'Display', None],
  [ 0x1686, 'OghamLetterUath', [], 'Display', None],
  [ 0x1687, 'OghamLetterDair', [], 'Display', None],
  [ 0x1688, 'OghamLetterTinne', [], 'Display', None],
  [ 0x1689, 'OghamLetterColl', [], 'Display', None],
  [ 0x168a, 'OghamLetterCeirt', [], 'Display', None],
  [ 0x168b, 'OghamLetterMuin', [], 'Display', None],
  [ 0x168c, 'OghamLetterGort', [], 'Display', None],
  [ 0x168d, 'OghamLetterNgeadal', [], 'Display', None],
  [ 0x168e, 'OghamLetterStraif', [], 'Display', None],
  [ 0x168f, 'OghamLetterRuis', [], 'Display', None],
  [ 0x1690, 'OghamLetterAilm', [], 'Display', None],
  [ 0x1691, 'OghamLetterOnn', [], 'Display', None],
  [ 0x1692, 'OghamLetterUr', [], 'Display', None],
  [ 0x1693, 'OghamLetterEadhadh', [], 'Display', None],
  [ 0x1694, 'OghamLetterIodhadh', [], 'Display', None],
  [ 0x1695, 'OghamLetterEabhadh', [], 'Display', None],
  [ 0x1696, 'OghamLetterOr', [], 'Display', None],
  [ 0x1697, 'OghamLetterUilleann', [], 'Display', None],
  [ 0x1698, 'OghamLetterIfin', [], 'Display', None],
  [ 0x1699, 'OghamLetterEamhancholl', [], 'Display', None],
  [ 0x169a, 'OghamLetterPeith', [], 'Display', None],
  [ 0x169b, 'OghamFeatherMark', [], 'Display', None],
  [ 0x169c, 'OghamReversedFeatherMark', [], 'Display', None],
  [ 0x169d, 'UnicodeCharacter0x169d', [], 'Display', None],
  [ 0x169e, 'UnicodeCharacter0x169e', [], 'Display', None],
  [ 0x169f, 'UnicodeCharacter0x169f', [], 'Display', None],
  [ 0x16a0, 'RunicLetterFehuFeohFeF', [], 'Display', None],
  [ 0x16a1, 'RunicLetterV', [], 'Display', None],
  [ 0x16a2, 'RunicLetterUruzUrU', [], 'Display', None],
  [ 0x16a3, 'RunicLetterYr', [], 'Display', None],
  [ 0x16a4, 'RunicLetterY', [], 'Display', None],
  [ 0x16a5, 'RunicLetterW', [], 'Display', None],
  [ 0x16a6, 'RunicLetterThurisazThursThorn', [], 'Display', None],
  [ 0x16a7, 'RunicLetterEth', [], 'Display', None],
  [ 0x16a8, 'RunicLetterAnsuzA', [], 'Display', None],
  [ 0x16a9, 'RunicLetterOsO', [], 'Display', None],
  [ 0x16aa, 'RunicLetterAcA', [], 'Display', None],
  [ 0x16ab, 'RunicLetterAesc', [], 'Display', None],
  [ 0x16ac, 'RunicLetterLongbranchossO', [], 'Display', None],
  [ 0x16ad, 'RunicLetterShorttwigossO', [], 'Display', None],
  [ 0x16ae, 'RunicLetterO', [], 'Display', None],
  [ 0x16af, 'RunicLetterOe', [], 'Display', None],
  [ 0x16b0, 'RunicLetterOn', [], 'Display', None],
  [ 0x16b1, 'RunicLetterRaidoRadReidR', [], 'Display', None],
  [ 0x16b2, 'RunicLetterKauna', [], 'Display', None],
  [ 0x16b3, 'RunicLetterCen', [], 'Display', None],
  [ 0x16b4, 'RunicLetterKaunK', [], 'Display', None],
  [ 0x16b5, 'RunicLetterG', [], 'Display', None],
  [ 0x16b6, 'RunicLetterEng', [], 'Display', None],
  [ 0x16b7, 'RunicLetterGeboGyfuG', [], 'Display', None],
  [ 0x16b8, 'RunicLetterGar', [], 'Display', None],
  [ 0x16b9, 'RunicLetterWunjoWynnW', [], 'Display', None],
  [ 0x16ba, 'RunicLetterHaglazH', [], 'Display', None],
  [ 0x16bb, 'RunicLetterHaeglH', [], 'Display', None],
  [ 0x16bc, 'RunicLetterLongbranchhagallH', [], 'Display', None],
  [ 0x16bd, 'RunicLetterShorttwighagallH', [], 'Display', None],
  [ 0x16be, 'RunicLetterNaudizNydNaudN', [], 'Display', None],
  [ 0x16bf, 'RunicLetterShorttwignaudN', [], 'Display', None],
  [ 0x16c0, 'RunicLetterDottedn', [], 'Display', None],
  [ 0x16c1, 'RunicLetterIsazIsIssI', [], 'Display', None],
  [ 0x16c2, 'RunicLetterE', [], 'Display', None],
  [ 0x16c3, 'RunicLetterJeranJ', [], 'Display', None],
  [ 0x16c4, 'RunicLetterGer', [], 'Display', None],
  [ 0x16c5, 'RunicLetterLongbrancharAe', [], 'Display', None],
  [ 0x16c6, 'RunicLetterShorttwigarA', [], 'Display', None],
  [ 0x16c7, 'RunicLetterIwazEoh', [], 'Display', None],
  [ 0x16c8, 'RunicLetterPerthoPeorthP', [], 'Display', None],
  [ 0x16c9, 'RunicLetterAlgizEolhx', [], 'Display', None],
  [ 0x16ca, 'RunicLetterSowiloS', [], 'Display', None],
  [ 0x16cb, 'RunicLetterSigelLongbranchsolS', [], 'Display', None],
  [ 0x16cc, 'RunicLetterShorttwigsolS', [], 'Display', None],
  [ 0x16cd, 'RunicLetterC', [], 'Display', None],
  [ 0x16ce, 'RunicLetterZ', [], 'Display', None],
  [ 0x16cf, 'RunicLetterTiwazTirTyrT', [], 'Display', None],
  [ 0x16d0, 'RunicLetterShorttwigtyrT', [], 'Display', None],
  [ 0x16d1, 'RunicLetterD', [], 'Display', None],
  [ 0x16d2, 'RunicLetterBerkananBeorcBjarkanB', [], 'Display', None],
  [ 0x16d3, 'RunicLetterShorttwigbjarkanB', [], 'Display', None],
  [ 0x16d4, 'RunicLetterDottedp', [], 'Display', None],
  [ 0x16d5, 'RunicLetterOpenp', [], 'Display', None],
  [ 0x16d6, 'RunicLetterEhwazEhE', [], 'Display', None],
  [ 0x16d7, 'RunicLetterMannazManM', [], 'Display', None],
  [ 0x16d8, 'RunicLetterLongbranchmadrM', [], 'Display', None],
  [ 0x16d9, 'RunicLetterShorttwigmadrM', [], 'Display', None],
  [ 0x16da, 'RunicLetterLaukazLaguLogrL', [], 'Display', None],
  [ 0x16db, 'RunicLetterDottedl', [], 'Display', None],
  [ 0x16dc, 'RunicLetterIngwaz', [], 'Display', None],
  [ 0x16dd, 'RunicLetterIng', [], 'Display', None],
  [ 0x16de, 'RunicLetterDagazDaegD', [], 'Display', None],
  [ 0x16df, 'RunicLetterOthalanEthelO', [], 'Display', None],
  [ 0x16e0, 'RunicLetterEar', [], 'Display', None],
  [ 0x16e1, 'RunicLetterIor', [], 'Display', None],
  [ 0x16e2, 'RunicLetterCweorth', [], 'Display', None],
  [ 0x16e3, 'RunicLetterCalc', [], 'Display', None],
  [ 0x16e4, 'RunicLetterCealc', [], 'Display', None],
  [ 0x16e5, 'RunicLetterStan', [], 'Display', None],
  [ 0x16e6, 'RunicLetterLongbranchyr', [], 'Display', None],
  [ 0x16e7, 'RunicLetterShorttwigyr', [], 'Display', None],
  [ 0x16e8, 'RunicLetterIcelandicyr', [], 'Display', None],
  [ 0x16e9, 'RunicLetterQ', [], 'Display', None],
  [ 0x16ea, 'RunicLetterX', [], 'Display', None],
  [ 0x16eb, 'RunicSinglePunctuation', [], 'Display', None],
  [ 0x16ec, 'RunicMultiplePunctuation', [], 'Display', None],
  [ 0x16ed, 'RunicCrossPunctuation', [], 'Display', None],
  [ 0x16ee, 'RunicArlaugSymbol', [], 'Display', None],
  [ 0x16ef, 'RunicTvimadurSymbol', [], 'Display', None],
  [ 0x16f0, 'RunicBelgthorSymbol', [], 'Display', None],
  [ 0x16f1, 'UnicodeCharacter0x16f1', [], 'Display', None],
  [ 0x16f2, 'UnicodeCharacter0x16f2', [], 'Display', None],
  [ 0x16f3, 'UnicodeCharacter0x16f3', [], 'Display', None],
  [ 0x16f4, 'UnicodeCharacter0x16f4', [], 'Display', None],
  [ 0x16f5, 'UnicodeCharacter0x16f5', [], 'Display', None],
  [ 0x16f6, 'UnicodeCharacter0x16f6', [], 'Display', None],
  [ 0x16f7, 'UnicodeCharacter0x16f7', [], 'Display', None],
  [ 0x16f8, 'UnicodeCharacter0x16f8', [], 'Display', None],
  [ 0x16f9, 'UnicodeCharacter0x16f9', [], 'Display', None],
  [ 0x16fa, 'UnicodeCharacter0x16fa', [], 'Display', None],
  [ 0x16fb, 'UnicodeCharacter0x16fb', [], 'Display', None],
  [ 0x16fc, 'UnicodeCharacter0x16fc', [], 'Display', None],
  [ 0x16fd, 'UnicodeCharacter0x16fd', [], 'Display', None],
  [ 0x16fe, 'UnicodeCharacter0x16fe', [], 'Display', None],
  [ 0x16ff, 'UnicodeCharacter0x16ff', [], 'Display', None],
  [ 0x1700, 'TagalogLetterA', [], 'Display', None],
  [ 0x1701, 'TagalogLetterI', [], 'Display', None],
  [ 0x1702, 'TagalogLetterU', [], 'Display', None],
  [ 0x1703, 'TagalogLetterKa', [], 'Display', None],
  [ 0x1704, 'TagalogLetterGa', [], 'Display', None],
  [ 0x1705, 'TagalogLetterNga', [], 'Display', None],
  [ 0x1706, 'TagalogLetterTa', [], 'Display', None],
  [ 0x1707, 'TagalogLetterDa', [], 'Display', None],
  [ 0x1708, 'TagalogLetterNa', [], 'Display', None],
  [ 0x1709, 'TagalogLetterPa', [], 'Display', None],
  [ 0x170a, 'TagalogLetterBa', [], 'Display', None],
  [ 0x170b, 'TagalogLetterMa', [], 'Display', None],
  [ 0x170c, 'TagalogLetterYa', [], 'Display', None],
  [ 0x170d, 'UnicodeCharacter0x170d', [], 'Display', None],
  [ 0x170e, 'TagalogLetterLa', [], 'Display', None],
  [ 0x170f, 'TagalogLetterWa', [], 'Display', None],
  [ 0x1710, 'TagalogLetterSa', [], 'Display', None],
  [ 0x1711, 'TagalogLetterHa', [], 'Display', None],
  [ 0x1712, 'TagalogVowelSignI', [], 'Display', None],
  [ 0x1713, 'TagalogVowelSignU', [], 'Display', None],
  [ 0x1714, 'TagalogSignVirama', [], 'Display', None],
  [ 0x1715, 'UnicodeCharacter0x1715', [], 'Display', None],
  [ 0x1716, 'UnicodeCharacter0x1716', [], 'Display', None],
  [ 0x1717, 'UnicodeCharacter0x1717', [], 'Display', None],
  [ 0x1718, 'UnicodeCharacter0x1718', [], 'Display', None],
  [ 0x1719, 'UnicodeCharacter0x1719', [], 'Display', None],
  [ 0x171a, 'UnicodeCharacter0x171a', [], 'Display', None],
  [ 0x171b, 'UnicodeCharacter0x171b', [], 'Display', None],
  [ 0x171c, 'UnicodeCharacter0x171c', [], 'Display', None],
  [ 0x171d, 'UnicodeCharacter0x171d', [], 'Display', None],
  [ 0x171e, 'UnicodeCharacter0x171e', [], 'Display', None],
  [ 0x171f, 'UnicodeCharacter0x171f', [], 'Display', None],
  [ 0x1720, 'HanunooLetterA', [], 'Display', None],
  [ 0x1721, 'HanunooLetterI', [], 'Display', None],
  [ 0x1722, 'HanunooLetterU', [], 'Display', None],
  [ 0x1723, 'HanunooLetterKa', [], 'Display', None],
  [ 0x1724, 'HanunooLetterGa', [], 'Display', None],
  [ 0x1725, 'HanunooLetterNga', [], 'Display', None],
  [ 0x1726, 'HanunooLetterTa', [], 'Display', None],
  [ 0x1727, 'HanunooLetterDa', [], 'Display', None],
  [ 0x1728, 'HanunooLetterNa', [], 'Display', None],
  [ 0x1729, 'HanunooLetterPa', [], 'Display', None],
  [ 0x172a, 'HanunooLetterBa', [], 'Display', None],
  [ 0x172b, 'HanunooLetterMa', [], 'Display', None],
  [ 0x172c, 'HanunooLetterYa', [], 'Display', None],
  [ 0x172d, 'HanunooLetterRa', [], 'Display', None],
  [ 0x172e, 'HanunooLetterLa', [], 'Display', None],
  [ 0x172f, 'HanunooLetterWa', [], 'Display', None],
  [ 0x1730, 'HanunooLetterSa', [], 'Display', None],
  [ 0x1731, 'HanunooLetterHa', [], 'Display', None],
  [ 0x1732, 'HanunooVowelSignI', [], 'Display', None],
  [ 0x1733, 'HanunooVowelSignU', [], 'Display', None],
  [ 0x1734, 'HanunooSignPamudpod', [], 'Display', None],
  [ 0x1735, 'PhilippineSinglePunctuation', [], 'Display', None],
  [ 0x1736, 'PhilippineDoublePunctuation', [], 'Display', None],
  [ 0x1737, 'UnicodeCharacter0x1737', [], 'Display', None],
  [ 0x1738, 'UnicodeCharacter0x1738', [], 'Display', None],
  [ 0x1739, 'UnicodeCharacter0x1739', [], 'Display', None],
  [ 0x173a, 'UnicodeCharacter0x173a', [], 'Display', None],
  [ 0x173b, 'UnicodeCharacter0x173b', [], 'Display', None],
  [ 0x173c, 'UnicodeCharacter0x173c', [], 'Display', None],
  [ 0x173d, 'UnicodeCharacter0x173d', [], 'Display', None],
  [ 0x173e, 'UnicodeCharacter0x173e', [], 'Display', None],
  [ 0x173f, 'UnicodeCharacter0x173f', [], 'Display', None],
  [ 0x1740, 'BuhidLetterA', [], 'Display', None],
  [ 0x1741, 'BuhidLetterI', [], 'Display', None],
  [ 0x1742, 'BuhidLetterU', [], 'Display', None],
  [ 0x1743, 'BuhidLetterKa', [], 'Display', None],
  [ 0x1744, 'BuhidLetterGa', [], 'Display', None],
  [ 0x1745, 'BuhidLetterNga', [], 'Display', None],
  [ 0x1746, 'BuhidLetterTa', [], 'Display', None],
  [ 0x1747, 'BuhidLetterDa', [], 'Display', None],
  [ 0x1748, 'BuhidLetterNa', [], 'Display', None],
  [ 0x1749, 'BuhidLetterPa', [], 'Display', None],
  [ 0x174a, 'BuhidLetterBa', [], 'Display', None],
  [ 0x174b, 'BuhidLetterMa', [], 'Display', None],
  [ 0x174c, 'BuhidLetterYa', [], 'Display', None],
  [ 0x174d, 'BuhidLetterRa', [], 'Display', None],
  [ 0x174e, 'BuhidLetterLa', [], 'Display', None],
  [ 0x174f, 'BuhidLetterWa', [], 'Display', None],
  [ 0x1750, 'BuhidLetterSa', [], 'Display', None],
  [ 0x1751, 'BuhidLetterHa', [], 'Display', None],
  [ 0x1752, 'BuhidVowelSignI', [], 'Display', None],
  [ 0x1753, 'BuhidVowelSignU', [], 'Display', None],
  [ 0x1754, 'UnicodeCharacter0x1754', [], 'Display', None],
  [ 0x1755, 'UnicodeCharacter0x1755', [], 'Display', None],
  [ 0x1756, 'UnicodeCharacter0x1756', [], 'Display', None],
  [ 0x1757, 'UnicodeCharacter0x1757', [], 'Display', None],
  [ 0x1758, 'UnicodeCharacter0x1758', [], 'Display', None],
  [ 0x1759, 'UnicodeCharacter0x1759', [], 'Display', None],
  [ 0x175a, 'UnicodeCharacter0x175a', [], 'Display', None],
  [ 0x175b, 'UnicodeCharacter0x175b', [], 'Display', None],
  [ 0x175c, 'UnicodeCharacter0x175c', [], 'Display', None],
  [ 0x175d, 'UnicodeCharacter0x175d', [], 'Display', None],
  [ 0x175e, 'UnicodeCharacter0x175e', [], 'Display', None],
  [ 0x175f, 'UnicodeCharacter0x175f', [], 'Display', None],
  [ 0x1760, 'TagbanwaLetterA', [], 'Display', None],
  [ 0x1761, 'TagbanwaLetterI', [], 'Display', None],
  [ 0x1762, 'TagbanwaLetterU', [], 'Display', None],
  [ 0x1763, 'TagbanwaLetterKa', [], 'Display', None],
  [ 0x1764, 'TagbanwaLetterGa', [], 'Display', None],
  [ 0x1765, 'TagbanwaLetterNga', [], 'Display', None],
  [ 0x1766, 'TagbanwaLetterTa', [], 'Display', None],
  [ 0x1767, 'TagbanwaLetterDa', [], 'Display', None],
  [ 0x1768, 'TagbanwaLetterNa', [], 'Display', None],
  [ 0x1769, 'TagbanwaLetterPa', [], 'Display', None],
  [ 0x176a, 'TagbanwaLetterBa', [], 'Display', None],
  [ 0x176b, 'TagbanwaLetterMa', [], 'Display', None],
  [ 0x176c, 'TagbanwaLetterYa', [], 'Display', None],
  [ 0x176d, 'UnicodeCharacter0x176d', [], 'Display', None],
  [ 0x176e, 'TagbanwaLetterLa', [], 'Display', None],
  [ 0x176f, 'TagbanwaLetterWa', [], 'Display', None],
  [ 0x1770, 'TagbanwaLetterSa', [], 'Display', None],
  [ 0x1771, 'UnicodeCharacter0x1771', [], 'Display', None],
  [ 0x1772, 'TagbanwaVowelSignI', [], 'Display', None],
  [ 0x1773, 'TagbanwaVowelSignU', [], 'Display', None],
  [ 0x1774, 'UnicodeCharacter0x1774', [], 'Display', None],
  [ 0x1775, 'UnicodeCharacter0x1775', [], 'Display', None],
  [ 0x1776, 'UnicodeCharacter0x1776', [], 'Display', None],
  [ 0x1777, 'UnicodeCharacter0x1777', [], 'Display', None],
  [ 0x1778, 'UnicodeCharacter0x1778', [], 'Display', None],
  [ 0x1779, 'UnicodeCharacter0x1779', [], 'Display', None],
  [ 0x177a, 'UnicodeCharacter0x177a', [], 'Display', None],
  [ 0x177b, 'UnicodeCharacter0x177b', [], 'Display', None],
  [ 0x177c, 'UnicodeCharacter0x177c', [], 'Display', None],
  [ 0x177d, 'UnicodeCharacter0x177d', [], 'Display', None],
  [ 0x177e, 'UnicodeCharacter0x177e', [], 'Display', None],
  [ 0x177f, 'UnicodeCharacter0x177f', [], 'Display', None],
  [ 0x1780, 'KhmerLetterKa', [], 'Display', None],
  [ 0x1781, 'KhmerLetterKha', [], 'Display', None],
  [ 0x1782, 'KhmerLetterKo', [], 'Display', None],
  [ 0x1783, 'KhmerLetterKho', [], 'Display', None],
  [ 0x1784, 'KhmerLetterNgo', [], 'Display', None],
  [ 0x1785, 'KhmerLetterCa', [], 'Display', None],
  [ 0x1786, 'KhmerLetterCha', [], 'Display', None],
  [ 0x1787, 'KhmerLetterCo', [], 'Display', None],
  [ 0x1788, 'KhmerLetterCho', [], 'Display', None],
  [ 0x1789, 'KhmerLetterNyo', [], 'Display', None],
  [ 0x178a, 'KhmerLetterDa', [], 'Display', None],
  [ 0x178b, 'KhmerLetterTtha', [], 'Display', None],
  [ 0x178c, 'KhmerLetterDo', [], 'Display', None],
  [ 0x178d, 'KhmerLetterTtho', [], 'Display', None],
  [ 0x178e, 'KhmerLetterNno', [], 'Display', None],
  [ 0x178f, 'KhmerLetterTa', [], 'Display', None],
  [ 0x1790, 'KhmerLetterTha', [], 'Display', None],
  [ 0x1791, 'KhmerLetterTo', [], 'Display', None],
  [ 0x1792, 'KhmerLetterTho', [], 'Display', None],
  [ 0x1793, 'KhmerLetterNo', [], 'Display', None],
  [ 0x1794, 'KhmerLetterBa', [], 'Display', None],
  [ 0x1795, 'KhmerLetterPha', [], 'Display', None],
  [ 0x1796, 'KhmerLetterPo', [], 'Display', None],
  [ 0x1797, 'KhmerLetterPho', [], 'Display', None],
  [ 0x1798, 'KhmerLetterMo', [], 'Display', None],
  [ 0x1799, 'KhmerLetterYo', [], 'Display', None],
  [ 0x179a, 'KhmerLetterRo', [], 'Display', None],
  [ 0x179b, 'KhmerLetterLo', [], 'Display', None],
  [ 0x179c, 'KhmerLetterVo', [], 'Display', None],
  [ 0x179d, 'KhmerLetterSha', [], 'Display', None],
  [ 0x179e, 'KhmerLetterSso', [], 'Display', None],
  [ 0x179f, 'KhmerLetterSa', [], 'Display', None],
  [ 0x17a0, 'KhmerLetterHa', [], 'Display', None],
  [ 0x17a1, 'KhmerLetterLa', [], 'Display', None],
  [ 0x17a2, 'KhmerLetterQa', [], 'Display', None],
  [ 0x17a3, 'KhmerIndependentVowelQaq', [], 'Display', None],
  [ 0x17a4, 'KhmerIndependentVowelQaa', [], 'Display', None],
  [ 0x17a5, 'KhmerIndependentVowelQi', [], 'Display', None],
  [ 0x17a6, 'KhmerIndependentVowelQii', [], 'Display', None],
  [ 0x17a7, 'KhmerIndependentVowelQu', [], 'Display', None],
  [ 0x17a8, 'KhmerIndependentVowelQuk', [], 'Display', None],
  [ 0x17a9, 'KhmerIndependentVowelQuu', [], 'Display', None],
  [ 0x17aa, 'KhmerIndependentVowelQuuv', [], 'Display', None],
  [ 0x17ab, 'KhmerIndependentVowelRy', [], 'Display', None],
  [ 0x17ac, 'KhmerIndependentVowelRyy', [], 'Display', None],
  [ 0x17ad, 'KhmerIndependentVowelLy', [], 'Display', None],
  [ 0x17ae, 'KhmerIndependentVowelLyy', [], 'Display', None],
  [ 0x17af, 'KhmerIndependentVowelQe', [], 'Display', None],
  [ 0x17b0, 'KhmerIndependentVowelQai', [], 'Display', None],
  [ 0x17b1, 'KhmerIndependentVowelQooTypeOne', [], 'Display', None],
  [ 0x17b2, 'KhmerIndependentVowelQooTypeTwo', [], 'Display', None],
  [ 0x17b3, 'KhmerIndependentVowelQau', [], 'Display', None],
  [ 0x17b4, 'KhmerVowelInherentAq', [], 'Display', None],
  [ 0x17b5, 'KhmerVowelInherentAa', [], 'Display', None],
  [ 0x17b6, 'KhmerVowelSignAa', [], 'Display', None],
  [ 0x17b7, 'KhmerVowelSignI', [], 'Display', None],
  [ 0x17b8, 'KhmerVowelSignIi', [], 'Display', None],
  [ 0x17b9, 'KhmerVowelSignY', [], 'Display', None],
  [ 0x17ba, 'KhmerVowelSignYy', [], 'Display', None],
  [ 0x17bb, 'KhmerVowelSignU', [], 'Display', None],
  [ 0x17bc, 'KhmerVowelSignUu', [], 'Display', None],
  [ 0x17bd, 'KhmerVowelSignUa', [], 'Display', None],
  [ 0x17be, 'KhmerVowelSignOe', [], 'Display', None],
  [ 0x17bf, 'KhmerVowelSignYa', [], 'Display', None],
  [ 0x17c0, 'KhmerVowelSignIe', [], 'Display', None],
  [ 0x17c1, 'KhmerVowelSignE', [], 'Display', None],
  [ 0x17c2, 'KhmerVowelSignAe', [], 'Display', None],
  [ 0x17c3, 'KhmerVowelSignAi', [], 'Display', None],
  [ 0x17c4, 'KhmerVowelSignOo', [], 'Display', None],
  [ 0x17c5, 'KhmerVowelSignAu', [], 'Display', None],
  [ 0x17c6, 'KhmerSignNikahit', [], 'Display', None],
  [ 0x17c7, 'KhmerSignReahmuk', [], 'Display', None],
  [ 0x17c8, 'KhmerSignYuukaleapintu', [], 'Display', None],
  [ 0x17c9, 'KhmerSignMuusikatoan', [], 'Display', None],
  [ 0x17ca, 'KhmerSignTriisap', [], 'Display', None],
  [ 0x17cb, 'KhmerSignBantoc', [], 'Display', None],
  [ 0x17cc, 'KhmerSignRobat', [], 'Display', None],
  [ 0x17cd, 'KhmerSignToandakhiat', [], 'Display', None],
  [ 0x17ce, 'KhmerSignKakabat', [], 'Display', None],
  [ 0x17cf, 'KhmerSignAhsda', [], 'Display', None],
  [ 0x17d0, 'KhmerSignSamyokSannya', [], 'Display', None],
  [ 0x17d1, 'KhmerSignViriam', [], 'Display', None],
  [ 0x17d2, 'KhmerSignCoeng', [], 'Display', None],
  [ 0x17d3, 'KhmerSignBathamasat', [], 'Display', None],
  [ 0x17d4, 'KhmerSignKhan', [], 'Display', None],
  [ 0x17d5, 'KhmerSignBariyoosan', [], 'Display', None],
  [ 0x17d6, 'KhmerSignCamnucPiiKuuh', [], 'Display', None],
  [ 0x17d7, 'KhmerSignLekToo', [], 'Display', None],
  [ 0x17d8, 'KhmerSignBeyyal', [], 'Display', None],
  [ 0x17d9, 'KhmerSignPhnaekMuan', [], 'Display', None],
  [ 0x17da, 'KhmerSignKoomuut', [], 'Display', None],
  [ 0x17db, 'KhmerCurrencySymbolRiel', [], 'Display', None],
  [ 0x17dc, 'KhmerSignAvakrahasanya', [], 'Display', None],
  [ 0x17dd, 'KhmerSignAtthacan', [], 'Display', None],
  [ 0x17de, 'UnicodeCharacter0x17de', [], 'Display', None],
  [ 0x17df, 'UnicodeCharacter0x17df', [], 'Display', None],
  [ 0x17e0, 'KhmerDigitZero', [], 'Display', None],
  [ 0x17e1, 'KhmerDigitOne', [], 'Display', None],
  [ 0x17e2, 'KhmerDigitTwo', [], 'Display', None],
  [ 0x17e3, 'KhmerDigitThree', [], 'Display', None],
  [ 0x17e4, 'KhmerDigitFour', [], 'Display', None],
  [ 0x17e5, 'KhmerDigitFive', [], 'Display', None],
  [ 0x17e6, 'KhmerDigitSix', [], 'Display', None],
  [ 0x17e7, 'KhmerDigitSeven', [], 'Display', None],
  [ 0x17e8, 'KhmerDigitEight', [], 'Display', None],
  [ 0x17e9, 'KhmerDigitNine', [], 'Display', None],
  [ 0x17ea, 'UnicodeCharacter0x17ea', [], 'Display', None],
  [ 0x17eb, 'UnicodeCharacter0x17eb', [], 'Display', None],
  [ 0x17ec, 'UnicodeCharacter0x17ec', [], 'Display', None],
  [ 0x17ed, 'UnicodeCharacter0x17ed', [], 'Display', None],
  [ 0x17ee, 'UnicodeCharacter0x17ee', [], 'Display', None],
  [ 0x17ef, 'UnicodeCharacter0x17ef', [], 'Display', None],
  [ 0x17f0, 'KhmerSymbolLekAttakSon', [], 'Display', None],
  [ 0x17f1, 'KhmerSymbolLekAttakMuoy', [], 'Display', None],
  [ 0x17f2, 'KhmerSymbolLekAttakPii', [], 'Display', None],
  [ 0x17f3, 'KhmerSymbolLekAttakBei', [], 'Display', None],
  [ 0x17f4, 'KhmerSymbolLekAttakBuon', [], 'Display', None],
  [ 0x17f5, 'KhmerSymbolLekAttakPram', [], 'Display', None],
  [ 0x17f6, 'KhmerSymbolLekAttakPrammuoy', [], 'Display', None],
  [ 0x17f7, 'KhmerSymbolLekAttakPrampii', [], 'Display', None],
  [ 0x17f8, 'KhmerSymbolLekAttakPrambei', [], 'Display', None],
  [ 0x17f9, 'KhmerSymbolLekAttakPrambuon', [], 'Display', None],
  [ 0x17fa, 'UnicodeCharacter0x17fa', [], 'Display', None],
  [ 0x17fb, 'UnicodeCharacter0x17fb', [], 'Display', None],
  [ 0x17fc, 'UnicodeCharacter0x17fc', [], 'Display', None],
  [ 0x17fd, 'UnicodeCharacter0x17fd', [], 'Display', None],
  [ 0x17fe, 'UnicodeCharacter0x17fe', [], 'Display', None],
  [ 0x17ff, 'UnicodeCharacter0x17ff', [], 'Display', None],
  [ 0x1800, 'MongolianBirga', [], 'Display', None],
  [ 0x1801, 'MongolianEllipsis', [], 'Display', None],
  [ 0x1802, 'MongolianComma', [], 'Display', None],
  [ 0x1803, 'MongolianFullStop', [], 'Display', None],
  [ 0x1804, 'MongolianColon', [], 'Display', None],
  [ 0x1805, 'MongolianFourDots', [], 'Display', None],
  [ 0x1806, 'MongolianTodoSoftHyphen', [], 'Display', None],
  [ 0x1807, 'MongolianSibeSyllableBoundaryMarker', [], 'Display', None],
  [ 0x1808, 'MongolianManchuComma', [], 'Display', None],
  [ 0x1809, 'MongolianManchuFullStop', [], 'Display', None],
  [ 0x180a, 'MongolianNirugu', [], 'Display', None],
  [ 0x180b, 'MongolianFreeVariationSelectorOne', [], 'Display', None],
  [ 0x180c, 'MongolianFreeVariationSelectorTwo', [], 'Display', None],
  [ 0x180d, 'MongolianFreeVariationSelectorThree', [], 'Display', None],
  [ 0x180e, 'MongolianVowelSeparator', [], 'Display', None],
  [ 0x180f, 'UnicodeCharacter0x180f', [], 'Display', None],
  [ 0x1810, 'MongolianDigitZero', [], 'Display', None],
  [ 0x1811, 'MongolianDigitOne', [], 'Display', None],
  [ 0x1812, 'MongolianDigitTwo', [], 'Display', None],
  [ 0x1813, 'MongolianDigitThree', [], 'Display', None],
  [ 0x1814, 'MongolianDigitFour', [], 'Display', None],
  [ 0x1815, 'MongolianDigitFive', [], 'Display', None],
  [ 0x1816, 'MongolianDigitSix', [], 'Display', None],
  [ 0x1817, 'MongolianDigitSeven', [], 'Display', None],
  [ 0x1818, 'MongolianDigitEight', [], 'Display', None],
  [ 0x1819, 'MongolianDigitNine', [], 'Display', None],
  [ 0x181a, 'UnicodeCharacter0x181a', [], 'Display', None],
  [ 0x181b, 'UnicodeCharacter0x181b', [], 'Display', None],
  [ 0x181c, 'UnicodeCharacter0x181c', [], 'Display', None],
  [ 0x181d, 'UnicodeCharacter0x181d', [], 'Display', None],
  [ 0x181e, 'UnicodeCharacter0x181e', [], 'Display', None],
  [ 0x181f, 'UnicodeCharacter0x181f', [], 'Display', None],
  [ 0x1820, 'MongolianLetterA', [], 'Display', None],
  [ 0x1821, 'MongolianLetterE', [], 'Display', None],
  [ 0x1822, 'MongolianLetterI', [], 'Display', None],
  [ 0x1823, 'MongolianLetterO', [], 'Display', None],
  [ 0x1824, 'MongolianLetterU', [], 'Display', None],
  [ 0x1825, 'MongolianLetterOe', [], 'Display', None],
  [ 0x1826, 'MongolianLetterUe', [], 'Display', None],
  [ 0x1827, 'MongolianLetterEe', [], 'Display', None],
  [ 0x1828, 'MongolianLetterNa', [], 'Display', None],
  [ 0x1829, 'MongolianLetterAng', [], 'Display', None],
  [ 0x182a, 'MongolianLetterBa', [], 'Display', None],
  [ 0x182b, 'MongolianLetterPa', [], 'Display', None],
  [ 0x182c, 'MongolianLetterQa', [], 'Display', None],
  [ 0x182d, 'MongolianLetterGa', [], 'Display', None],
  [ 0x182e, 'MongolianLetterMa', [], 'Display', None],
  [ 0x182f, 'MongolianLetterLa', [], 'Display', None],
  [ 0x1830, 'MongolianLetterSa', [], 'Display', None],
  [ 0x1831, 'MongolianLetterSha', [], 'Display', None],
  [ 0x1832, 'MongolianLetterTa', [], 'Display', None],
  [ 0x1833, 'MongolianLetterDa', [], 'Display', None],
  [ 0x1834, 'MongolianLetterCha', [], 'Display', None],
  [ 0x1835, 'MongolianLetterJa', [], 'Display', None],
  [ 0x1836, 'MongolianLetterYa', [], 'Display', None],
  [ 0x1837, 'MongolianLetterRa', [], 'Display', None],
  [ 0x1838, 'MongolianLetterWa', [], 'Display', None],
  [ 0x1839, 'MongolianLetterFa', [], 'Display', None],
  [ 0x183a, 'MongolianLetterKa', [], 'Display', None],
  [ 0x183b, 'MongolianLetterKha', [], 'Display', None],
  [ 0x183c, 'MongolianLetterTsa', [], 'Display', None],
  [ 0x183d, 'MongolianLetterZa', [], 'Display', None],
  [ 0x183e, 'MongolianLetterHaa', [], 'Display', None],
  [ 0x183f, 'MongolianLetterZra', [], 'Display', None],
  [ 0x1840, 'MongolianLetterLha', [], 'Display', None],
  [ 0x1841, 'MongolianLetterZhi', [], 'Display', None],
  [ 0x1842, 'MongolianLetterChi', [], 'Display', None],
  [ 0x1843, 'MongolianLetterTodoLongVowelSign', [], 'Display', None],
  [ 0x1844, 'MongolianLetterTodoE', [], 'Display', None],
  [ 0x1845, 'MongolianLetterTodoI', [], 'Display', None],
  [ 0x1846, 'MongolianLetterTodoO', [], 'Display', None],
  [ 0x1847, 'MongolianLetterTodoU', [], 'Display', None],
  [ 0x1848, 'MongolianLetterTodoOe', [], 'Display', None],
  [ 0x1849, 'MongolianLetterTodoUe', [], 'Display', None],
  [ 0x184a, 'MongolianLetterTodoAng', [], 'Display', None],
  [ 0x184b, 'MongolianLetterTodoBa', [], 'Display', None],
  [ 0x184c, 'MongolianLetterTodoPa', [], 'Display', None],
  [ 0x184d, 'MongolianLetterTodoQa', [], 'Display', None],
  [ 0x184e, 'MongolianLetterTodoGa', [], 'Display', None],
  [ 0x184f, 'MongolianLetterTodoMa', [], 'Display', None],
  [ 0x1850, 'MongolianLetterTodoTa', [], 'Display', None],
  [ 0x1851, 'MongolianLetterTodoDa', [], 'Display', None],
  [ 0x1852, 'MongolianLetterTodoCha', [], 'Display', None],
  [ 0x1853, 'MongolianLetterTodoJa', [], 'Display', None],
  [ 0x1854, 'MongolianLetterTodoTsa', [], 'Display', None],
  [ 0x1855, 'MongolianLetterTodoYa', [], 'Display', None],
  [ 0x1856, 'MongolianLetterTodoWa', [], 'Display', None],
  [ 0x1857, 'MongolianLetterTodoKa', [], 'Display', None],
  [ 0x1858, 'MongolianLetterTodoGaa', [], 'Display', None],
  [ 0x1859, 'MongolianLetterTodoHaa', [], 'Display', None],
  [ 0x185a, 'MongolianLetterTodoJia', [], 'Display', None],
  [ 0x185b, 'MongolianLetterTodoNia', [], 'Display', None],
  [ 0x185c, 'MongolianLetterTodoDza', [], 'Display', None],
  [ 0x185d, 'MongolianLetterSibeE', [], 'Display', None],
  [ 0x185e, 'MongolianLetterSibeI', [], 'Display', None],
  [ 0x185f, 'MongolianLetterSibeIy', [], 'Display', None],
  [ 0x1860, 'MongolianLetterSibeUe', [], 'Display', None],
  [ 0x1861, 'MongolianLetterSibeU', [], 'Display', None],
  [ 0x1862, 'MongolianLetterSibeAng', [], 'Display', None],
  [ 0x1863, 'MongolianLetterSibeKa', [], 'Display', None],
  [ 0x1864, 'MongolianLetterSibeGa', [], 'Display', None],
  [ 0x1865, 'MongolianLetterSibeHa', [], 'Display', None],
  [ 0x1866, 'MongolianLetterSibePa', [], 'Display', None],
  [ 0x1867, 'MongolianLetterSibeSha', [], 'Display', None],
  [ 0x1868, 'MongolianLetterSibeTa', [], 'Display', None],
  [ 0x1869, 'MongolianLetterSibeDa', [], 'Display', None],
  [ 0x186a, 'MongolianLetterSibeJa', [], 'Display', None],
  [ 0x186b, 'MongolianLetterSibeFa', [], 'Display', None],
  [ 0x186c, 'MongolianLetterSibeGaa', [], 'Display', None],
  [ 0x186d, 'MongolianLetterSibeHaa', [], 'Display', None],
  [ 0x186e, 'MongolianLetterSibeTsa', [], 'Display', None],
  [ 0x186f, 'MongolianLetterSibeZa', [], 'Display', None],
  [ 0x1870, 'MongolianLetterSibeRaa', [], 'Display', None],
  [ 0x1871, 'MongolianLetterSibeCha', [], 'Display', None],
  [ 0x1872, 'MongolianLetterSibeZha', [], 'Display', None],
  [ 0x1873, 'MongolianLetterManchuI', [], 'Display', None],
  [ 0x1874, 'MongolianLetterManchuKa', [], 'Display', None],
  [ 0x1875, 'MongolianLetterManchuRa', [], 'Display', None],
  [ 0x1876, 'MongolianLetterManchuFa', [], 'Display', None],
  [ 0x1877, 'MongolianLetterManchuZha', [], 'Display', None],
  [ 0x1878, 'UnicodeCharacter0x1878', [], 'Display', None],
  [ 0x1879, 'UnicodeCharacter0x1879', [], 'Display', None],
  [ 0x187a, 'UnicodeCharacter0x187a', [], 'Display', None],
  [ 0x187b, 'UnicodeCharacter0x187b', [], 'Display', None],
  [ 0x187c, 'UnicodeCharacter0x187c', [], 'Display', None],
  [ 0x187d, 'UnicodeCharacter0x187d', [], 'Display', None],
  [ 0x187e, 'UnicodeCharacter0x187e', [], 'Display', None],
  [ 0x187f, 'UnicodeCharacter0x187f', [], 'Display', None],
  [ 0x1880, 'MongolianLetterAliGaliAnusvaraOne', [], 'Display', None],
  [ 0x1881, 'MongolianLetterAliGaliVisargaOne', [], 'Display', None],
  [ 0x1882, 'MongolianLetterAliGaliDamaru', [], 'Display', None],
  [ 0x1883, 'MongolianLetterAliGaliUbadama', [], 'Display', None],
  [ 0x1884, 'MongolianLetterAliGaliInvertedUbadama', [], 'Display', None],
  [ 0x1885, 'MongolianLetterAliGaliBaluda', [], 'Display', None],
  [ 0x1886, 'MongolianLetterAliGaliThreeBaluda', [], 'Display', None],
  [ 0x1887, 'MongolianLetterAliGaliA', [], 'Display', None],
  [ 0x1888, 'MongolianLetterAliGaliI', [], 'Display', None],
  [ 0x1889, 'MongolianLetterAliGaliKa', [], 'Display', None],
  [ 0x188a, 'MongolianLetterAliGaliNga', [], 'Display', None],
  [ 0x188b, 'MongolianLetterAliGaliCa', [], 'Display', None],
  [ 0x188c, 'MongolianLetterAliGaliTta', [], 'Display', None],
  [ 0x188d, 'MongolianLetterAliGaliTtha', [], 'Display', None],
  [ 0x188e, 'MongolianLetterAliGaliDda', [], 'Display', None],
  [ 0x188f, 'MongolianLetterAliGaliNna', [], 'Display', None],
  [ 0x1890, 'MongolianLetterAliGaliTa', [], 'Display', None],
  [ 0x1891, 'MongolianLetterAliGaliDa', [], 'Display', None],
  [ 0x1892, 'MongolianLetterAliGaliPa', [], 'Display', None],
  [ 0x1893, 'MongolianLetterAliGaliPha', [], 'Display', None],
  [ 0x1894, 'MongolianLetterAliGaliSsa', [], 'Display', None],
  [ 0x1895, 'MongolianLetterAliGaliZha', [], 'Display', None],
  [ 0x1896, 'MongolianLetterAliGaliZa', [], 'Display', None],
  [ 0x1897, 'MongolianLetterAliGaliAh', [], 'Display', None],
  [ 0x1898, 'MongolianLetterTodoAliGaliTa', [], 'Display', None],
  [ 0x1899, 'MongolianLetterTodoAliGaliZha', [], 'Display', None],
  [ 0x189a, 'MongolianLetterManchuAliGaliGha', [], 'Display', None],
  [ 0x189b, 'MongolianLetterManchuAliGaliNga', [], 'Display', None],
  [ 0x189c, 'MongolianLetterManchuAliGaliCa', [], 'Display', None],
  [ 0x189d, 'MongolianLetterManchuAliGaliJha', [], 'Display', None],
  [ 0x189e, 'MongolianLetterManchuAliGaliTta', [], 'Display', None],
  [ 0x189f, 'MongolianLetterManchuAliGaliDdha', [], 'Display', None],
  [ 0x18a0, 'MongolianLetterManchuAliGaliTa', [], 'Display', None],
  [ 0x18a1, 'MongolianLetterManchuAliGaliDha', [], 'Display', None],
  [ 0x18a2, 'MongolianLetterManchuAliGaliSsa', [], 'Display', None],
  [ 0x18a3, 'MongolianLetterManchuAliGaliCya', [], 'Display', None],
  [ 0x18a4, 'MongolianLetterManchuAliGaliZha', [], 'Display', None],
  [ 0x18a5, 'MongolianLetterManchuAliGaliZa', [], 'Display', None],
  [ 0x18a6, 'MongolianLetterAliGaliHalfU', [], 'Display', None],
  [ 0x18a7, 'MongolianLetterAliGaliHalfYa', [], 'Display', None],
  [ 0x18a8, 'MongolianLetterManchuAliGaliBha', [], 'Display', None],
  [ 0x18a9, 'MongolianLetterAliGaliDagalga', [], 'Display', None],
  [ 0x18aa, 'MongolianLetterManchuAliGaliLha', [], 'Display', None],
  [ 0x18ab, 'UnicodeCharacter0x18ab', [], 'Display', None],
  [ 0x18ac, 'UnicodeCharacter0x18ac', [], 'Display', None],
  [ 0x18ad, 'UnicodeCharacter0x18ad', [], 'Display', None],
  [ 0x18ae, 'UnicodeCharacter0x18ae', [], 'Display', None],
  [ 0x18af, 'UnicodeCharacter0x18af', [], 'Display', None],
  [ 0x18b0, 'UnicodeCharacter0x18b0', [], 'Display', None],
  [ 0x18b1, 'UnicodeCharacter0x18b1', [], 'Display', None],
  [ 0x18b2, 'UnicodeCharacter0x18b2', [], 'Display', None],
  [ 0x18b3, 'UnicodeCharacter0x18b3', [], 'Display', None],
  [ 0x18b4, 'UnicodeCharacter0x18b4', [], 'Display', None],
  [ 0x18b5, 'UnicodeCharacter0x18b5', [], 'Display', None],
  [ 0x18b6, 'UnicodeCharacter0x18b6', [], 'Display', None],
  [ 0x18b7, 'UnicodeCharacter0x18b7', [], 'Display', None],
  [ 0x18b8, 'UnicodeCharacter0x18b8', [], 'Display', None],
  [ 0x18b9, 'UnicodeCharacter0x18b9', [], 'Display', None],
  [ 0x18ba, 'UnicodeCharacter0x18ba', [], 'Display', None],
  [ 0x18bb, 'UnicodeCharacter0x18bb', [], 'Display', None],
  [ 0x18bc, 'UnicodeCharacter0x18bc', [], 'Display', None],
  [ 0x18bd, 'UnicodeCharacter0x18bd', [], 'Display', None],
  [ 0x18be, 'UnicodeCharacter0x18be', [], 'Display', None],
  [ 0x18bf, 'UnicodeCharacter0x18bf', [], 'Display', None],
  [ 0x18c0, 'UnicodeCharacter0x18c0', [], 'Display', None],
  [ 0x18c1, 'UnicodeCharacter0x18c1', [], 'Display', None],
  [ 0x18c2, 'UnicodeCharacter0x18c2', [], 'Display', None],
  [ 0x18c3, 'UnicodeCharacter0x18c3', [], 'Display', None],
  [ 0x18c4, 'UnicodeCharacter0x18c4', [], 'Display', None],
  [ 0x18c5, 'UnicodeCharacter0x18c5', [], 'Display', None],
  [ 0x18c6, 'UnicodeCharacter0x18c6', [], 'Display', None],
  [ 0x18c7, 'UnicodeCharacter0x18c7', [], 'Display', None],
  [ 0x18c8, 'UnicodeCharacter0x18c8', [], 'Display', None],
  [ 0x18c9, 'UnicodeCharacter0x18c9', [], 'Display', None],
  [ 0x18ca, 'UnicodeCharacter0x18ca', [], 'Display', None],
  [ 0x18cb, 'UnicodeCharacter0x18cb', [], 'Display', None],
  [ 0x18cc, 'UnicodeCharacter0x18cc', [], 'Display', None],
  [ 0x18cd, 'UnicodeCharacter0x18cd', [], 'Display', None],
  [ 0x18ce, 'UnicodeCharacter0x18ce', [], 'Display', None],
  [ 0x18cf, 'UnicodeCharacter0x18cf', [], 'Display', None],
  [ 0x18d0, 'UnicodeCharacter0x18d0', [], 'Display', None],
  [ 0x18d1, 'UnicodeCharacter0x18d1', [], 'Display', None],
  [ 0x18d2, 'UnicodeCharacter0x18d2', [], 'Display', None],
  [ 0x18d3, 'UnicodeCharacter0x18d3', [], 'Display', None],
  [ 0x18d4, 'UnicodeCharacter0x18d4', [], 'Display', None],
  [ 0x18d5, 'UnicodeCharacter0x18d5', [], 'Display', None],
  [ 0x18d6, 'UnicodeCharacter0x18d6', [], 'Display', None],
  [ 0x18d7, 'UnicodeCharacter0x18d7', [], 'Display', None],
  [ 0x18d8, 'UnicodeCharacter0x18d8', [], 'Display', None],
  [ 0x18d9, 'UnicodeCharacter0x18d9', [], 'Display', None],
  [ 0x18da, 'UnicodeCharacter0x18da', [], 'Display', None],
  [ 0x18db, 'UnicodeCharacter0x18db', [], 'Display', None],
  [ 0x18dc, 'UnicodeCharacter0x18dc', [], 'Display', None],
  [ 0x18dd, 'UnicodeCharacter0x18dd', [], 'Display', None],
  [ 0x18de, 'UnicodeCharacter0x18de', [], 'Display', None],
  [ 0x18df, 'UnicodeCharacter0x18df', [], 'Display', None],
  [ 0x18e0, 'UnicodeCharacter0x18e0', [], 'Display', None],
  [ 0x18e1, 'UnicodeCharacter0x18e1', [], 'Display', None],
  [ 0x18e2, 'UnicodeCharacter0x18e2', [], 'Display', None],
  [ 0x18e3, 'UnicodeCharacter0x18e3', [], 'Display', None],
  [ 0x18e4, 'UnicodeCharacter0x18e4', [], 'Display', None],
  [ 0x18e5, 'UnicodeCharacter0x18e5', [], 'Display', None],
  [ 0x18e6, 'UnicodeCharacter0x18e6', [], 'Display', None],
  [ 0x18e7, 'UnicodeCharacter0x18e7', [], 'Display', None],
  [ 0x18e8, 'UnicodeCharacter0x18e8', [], 'Display', None],
  [ 0x18e9, 'UnicodeCharacter0x18e9', [], 'Display', None],
  [ 0x18ea, 'UnicodeCharacter0x18ea', [], 'Display', None],
  [ 0x18eb, 'UnicodeCharacter0x18eb', [], 'Display', None],
  [ 0x18ec, 'UnicodeCharacter0x18ec', [], 'Display', None],
  [ 0x18ed, 'UnicodeCharacter0x18ed', [], 'Display', None],
  [ 0x18ee, 'UnicodeCharacter0x18ee', [], 'Display', None],
  [ 0x18ef, 'UnicodeCharacter0x18ef', [], 'Display', None],
  [ 0x18f0, 'UnicodeCharacter0x18f0', [], 'Display', None],
  [ 0x18f1, 'UnicodeCharacter0x18f1', [], 'Display', None],
  [ 0x18f2, 'UnicodeCharacter0x18f2', [], 'Display', None],
  [ 0x18f3, 'UnicodeCharacter0x18f3', [], 'Display', None],
  [ 0x18f4, 'UnicodeCharacter0x18f4', [], 'Display', None],
  [ 0x18f5, 'UnicodeCharacter0x18f5', [], 'Display', None],
  [ 0x18f6, 'UnicodeCharacter0x18f6', [], 'Display', None],
  [ 0x18f7, 'UnicodeCharacter0x18f7', [], 'Display', None],
  [ 0x18f8, 'UnicodeCharacter0x18f8', [], 'Display', None],
  [ 0x18f9, 'UnicodeCharacter0x18f9', [], 'Display', None],
  [ 0x18fa, 'UnicodeCharacter0x18fa', [], 'Display', None],
  [ 0x18fb, 'UnicodeCharacter0x18fb', [], 'Display', None],
  [ 0x18fc, 'UnicodeCharacter0x18fc', [], 'Display', None],
  [ 0x18fd, 'UnicodeCharacter0x18fd', [], 'Display', None],
  [ 0x18fe, 'UnicodeCharacter0x18fe', [], 'Display', None],
  [ 0x18ff, 'UnicodeCharacter0x18ff', [], 'Display', None],
  [ 0x1900, 'LimbuVowelcarrierLetter', [], 'Display', None],
  [ 0x1901, 'LimbuLetterKa', [], 'Display', None],
  [ 0x1902, 'LimbuLetterKha', [], 'Display', None],
  [ 0x1903, 'LimbuLetterGa', [], 'Display', None],
  [ 0x1904, 'LimbuLetterGha', [], 'Display', None],
  [ 0x1905, 'LimbuLetterNga', [], 'Display', None],
  [ 0x1906, 'LimbuLetterCa', [], 'Display', None],
  [ 0x1907, 'LimbuLetterCha', [], 'Display', None],
  [ 0x1908, 'LimbuLetterJa', [], 'Display', None],
  [ 0x1909, 'LimbuLetterJha', [], 'Display', None],
  [ 0x190a, 'LimbuLetterYan', [], 'Display', None],
  [ 0x190b, 'LimbuLetterTa', [], 'Display', None],
  [ 0x190c, 'LimbuLetterTha', [], 'Display', None],
  [ 0x190d, 'LimbuLetterDa', [], 'Display', None],
  [ 0x190e, 'LimbuLetterDha', [], 'Display', None],
  [ 0x190f, 'LimbuLetterNa', [], 'Display', None],
  [ 0x1910, 'LimbuLetterPa', [], 'Display', None],
  [ 0x1911, 'LimbuLetterPha', [], 'Display', None],
  [ 0x1912, 'LimbuLetterBa', [], 'Display', None],
  [ 0x1913, 'LimbuLetterBha', [], 'Display', None],
  [ 0x1914, 'LimbuLetterMa', [], 'Display', None],
  [ 0x1915, 'LimbuLetterYa', [], 'Display', None],
  [ 0x1916, 'LimbuLetterRa', [], 'Display', None],
  [ 0x1917, 'LimbuLetterLa', [], 'Display', None],
  [ 0x1918, 'LimbuLetterWa', [], 'Display', None],
  [ 0x1919, 'LimbuLetterSha', [], 'Display', None],
  [ 0x191a, 'LimbuLetterSsa', [], 'Display', None],
  [ 0x191b, 'LimbuLetterSa', [], 'Display', None],
  [ 0x191c, 'LimbuLetterHa', [], 'Display', None],
  [ 0x191d, 'UnicodeCharacter0x191d', [], 'Display', None],
  [ 0x191e, 'UnicodeCharacter0x191e', [], 'Display', None],
  [ 0x191f, 'UnicodeCharacter0x191f', [], 'Display', None],
  [ 0x1920, 'LimbuVowelSignA', [], 'Display', None],
  [ 0x1921, 'LimbuVowelSignI', [], 'Display', None],
  [ 0x1922, 'LimbuVowelSignU', [], 'Display', None],
  [ 0x1923, 'LimbuVowelSignEe', [], 'Display', None],
  [ 0x1924, 'LimbuVowelSignAi', [], 'Display', None],
  [ 0x1925, 'LimbuVowelSignOo', [], 'Display', None],
  [ 0x1926, 'LimbuVowelSignAu', [], 'Display', None],
  [ 0x1927, 'LimbuVowelSignE', [], 'Display', None],
  [ 0x1928, 'LimbuVowelSignO', [], 'Display', None],
  [ 0x1929, 'LimbuSubjoinedLetterYa', [], 'Display', None],
  [ 0x192a, 'LimbuSubjoinedLetterRa', [], 'Display', None],
  [ 0x192b, 'LimbuSubjoinedLetterWa', [], 'Display', None],
  [ 0x192c, 'UnicodeCharacter0x192c', [], 'Display', None],
  [ 0x192d, 'UnicodeCharacter0x192d', [], 'Display', None],
  [ 0x192e, 'UnicodeCharacter0x192e', [], 'Display', None],
  [ 0x192f, 'UnicodeCharacter0x192f', [], 'Display', None],
  [ 0x1930, 'LimbuSmallLetterKa', [], 'Display', None],
  [ 0x1931, 'LimbuSmallLetterNga', [], 'Display', None],
  [ 0x1932, 'LimbuSmallLetterAnusvara', [], 'Display', None],
  [ 0x1933, 'LimbuSmallLetterTa', [], 'Display', None],
  [ 0x1934, 'LimbuSmallLetterNa', [], 'Display', None],
  [ 0x1935, 'LimbuSmallLetterPa', [], 'Display', None],
  [ 0x1936, 'LimbuSmallLetterMa', [], 'Display', None],
  [ 0x1937, 'LimbuSmallLetterRa', [], 'Display', None],
  [ 0x1938, 'LimbuSmallLetterLa', [], 'Display', None],
  [ 0x1939, 'LimbuSignMukphreng', [], 'Display', None],
  [ 0x193a, 'LimbuSignKemphreng', [], 'Display', None],
  [ 0x193b, 'LimbuSignSai', [], 'Display', None],
  [ 0x193c, 'UnicodeCharacter0x193c', [], 'Display', None],
  [ 0x193d, 'UnicodeCharacter0x193d', [], 'Display', None],
  [ 0x193e, 'UnicodeCharacter0x193e', [], 'Display', None],
  [ 0x193f, 'UnicodeCharacter0x193f', [], 'Display', None],
  [ 0x1940, 'LimbuSignLoo', [], 'Display', None],
  [ 0x1941, 'UnicodeCharacter0x1941', [], 'Display', None],
  [ 0x1942, 'UnicodeCharacter0x1942', [], 'Display', None],
  [ 0x1943, 'UnicodeCharacter0x1943', [], 'Display', None],
  [ 0x1944, 'LimbuExclamationMark', [], 'Display', None],
  [ 0x1945, 'LimbuQuestionMark', [], 'Display', None],
  [ 0x1946, 'LimbuDigitZero', [], 'Display', None],
  [ 0x1947, 'LimbuDigitOne', [], 'Display', None],
  [ 0x1948, 'LimbuDigitTwo', [], 'Display', None],
  [ 0x1949, 'LimbuDigitThree', [], 'Display', None],
  [ 0x194a, 'LimbuDigitFour', [], 'Display', None],
  [ 0x194b, 'LimbuDigitFive', [], 'Display', None],
  [ 0x194c, 'LimbuDigitSix', [], 'Display', None],
  [ 0x194d, 'LimbuDigitSeven', [], 'Display', None],
  [ 0x194e, 'LimbuDigitEight', [], 'Display', None],
  [ 0x194f, 'LimbuDigitNine', [], 'Display', None],
  [ 0x1950, 'TaiLeLetterKa', [], 'Display', None],
  [ 0x1951, 'TaiLeLetterXa', [], 'Display', None],
  [ 0x1952, 'TaiLeLetterNga', [], 'Display', None],
  [ 0x1953, 'TaiLeLetterTsa', [], 'Display', None],
  [ 0x1954, 'TaiLeLetterSa', [], 'Display', None],
  [ 0x1955, 'TaiLeLetterYa', [], 'Display', None],
  [ 0x1956, 'TaiLeLetterTa', [], 'Display', None],
  [ 0x1957, 'TaiLeLetterTha', [], 'Display', None],
  [ 0x1958, 'TaiLeLetterLa', [], 'Display', None],
  [ 0x1959, 'TaiLeLetterPa', [], 'Display', None],
  [ 0x195a, 'TaiLeLetterPha', [], 'Display', None],
  [ 0x195b, 'TaiLeLetterMa', [], 'Display', None],
  [ 0x195c, 'TaiLeLetterFa', [], 'Display', None],
  [ 0x195d, 'TaiLeLetterVa', [], 'Display', None],
  [ 0x195e, 'TaiLeLetterHa', [], 'Display', None],
  [ 0x195f, 'TaiLeLetterQa', [], 'Display', None],
  [ 0x1960, 'TaiLeLetterKha', [], 'Display', None],
  [ 0x1961, 'TaiLeLetterTsha', [], 'Display', None],
  [ 0x1962, 'TaiLeLetterNa', [], 'Display', None],
  [ 0x1963, 'TaiLeLetterA', [], 'Display', None],
  [ 0x1964, 'TaiLeLetterI', [], 'Display', None],
  [ 0x1965, 'TaiLeLetterEe', [], 'Display', None],
  [ 0x1966, 'TaiLeLetterEh', [], 'Display', None],
  [ 0x1967, 'TaiLeLetterU', [], 'Display', None],
  [ 0x1968, 'TaiLeLetterOo', [], 'Display', None],
  [ 0x1969, 'TaiLeLetterO', [], 'Display', None],
  [ 0x196a, 'TaiLeLetterUe', [], 'Display', None],
  [ 0x196b, 'TaiLeLetterE', [], 'Display', None],
  [ 0x196c, 'TaiLeLetterAue', [], 'Display', None],
  [ 0x196d, 'TaiLeLetterAi', [], 'Display', None],
  [ 0x196e, 'UnicodeCharacter0x196e', [], 'Display', None],
  [ 0x196f, 'UnicodeCharacter0x196f', [], 'Display', None],
  [ 0x1970, 'TaiLeLetterTone2', [], 'Display', None],
  [ 0x1971, 'TaiLeLetterTone3', [], 'Display', None],
  [ 0x1972, 'TaiLeLetterTone4', [], 'Display', None],
  [ 0x1973, 'TaiLeLetterTone5', [], 'Display', None],
  [ 0x1974, 'TaiLeLetterTone6', [], 'Display', None],
  [ 0x1975, 'UnicodeCharacter0x1975', [], 'Display', None],
  [ 0x1976, 'UnicodeCharacter0x1976', [], 'Display', None],
  [ 0x1977, 'UnicodeCharacter0x1977', [], 'Display', None],
  [ 0x1978, 'UnicodeCharacter0x1978', [], 'Display', None],
  [ 0x1979, 'UnicodeCharacter0x1979', [], 'Display', None],
  [ 0x197a, 'UnicodeCharacter0x197a', [], 'Display', None],
  [ 0x197b, 'UnicodeCharacter0x197b', [], 'Display', None],
  [ 0x197c, 'UnicodeCharacter0x197c', [], 'Display', None],
  [ 0x197d, 'UnicodeCharacter0x197d', [], 'Display', None],
  [ 0x197e, 'UnicodeCharacter0x197e', [], 'Display', None],
  [ 0x197f, 'UnicodeCharacter0x197f', [], 'Display', None],
  [ 0x1980, 'NewTaiLueLetterHighQa', [], 'Display', None],
  [ 0x1981, 'NewTaiLueLetterLowQa', [], 'Display', None],
  [ 0x1982, 'NewTaiLueLetterHighKa', [], 'Display', None],
  [ 0x1983, 'NewTaiLueLetterHighXa', [], 'Display', None],
  [ 0x1984, 'NewTaiLueLetterHighNga', [], 'Display', None],
  [ 0x1985, 'NewTaiLueLetterLowKa', [], 'Display', None],
  [ 0x1986, 'NewTaiLueLetterLowXa', [], 'Display', None],
  [ 0x1987, 'NewTaiLueLetterLowNga', [], 'Display', None],
  [ 0x1988, 'NewTaiLueLetterHighTsa', [], 'Display', None],
  [ 0x1989, 'NewTaiLueLetterHighSa', [], 'Display', None],
  [ 0x198a, 'NewTaiLueLetterHighYa', [], 'Display', None],
  [ 0x198b, 'NewTaiLueLetterLowTsa', [], 'Display', None],
  [ 0x198c, 'NewTaiLueLetterLowSa', [], 'Display', None],
  [ 0x198d, 'NewTaiLueLetterLowYa', [], 'Display', None],
  [ 0x198e, 'NewTaiLueLetterHighTa', [], 'Display', None],
  [ 0x198f, 'NewTaiLueLetterHighTha', [], 'Display', None],
  [ 0x1990, 'NewTaiLueLetterHighNa', [], 'Display', None],
  [ 0x1991, 'NewTaiLueLetterLowTa', [], 'Display', None],
  [ 0x1992, 'NewTaiLueLetterLowTha', [], 'Display', None],
  [ 0x1993, 'NewTaiLueLetterLowNa', [], 'Display', None],
  [ 0x1994, 'NewTaiLueLetterHighPa', [], 'Display', None],
  [ 0x1995, 'NewTaiLueLetterHighPha', [], 'Display', None],
  [ 0x1996, 'NewTaiLueLetterHighMa', [], 'Display', None],
  [ 0x1997, 'NewTaiLueLetterLowPa', [], 'Display', None],
  [ 0x1998, 'NewTaiLueLetterLowPha', [], 'Display', None],
  [ 0x1999, 'NewTaiLueLetterLowMa', [], 'Display', None],
  [ 0x199a, 'NewTaiLueLetterHighFa', [], 'Display', None],
  [ 0x199b, 'NewTaiLueLetterHighVa', [], 'Display', None],
  [ 0x199c, 'NewTaiLueLetterHighLa', [], 'Display', None],
  [ 0x199d, 'NewTaiLueLetterLowFa', [], 'Display', None],
  [ 0x199e, 'NewTaiLueLetterLowVa', [], 'Display', None],
  [ 0x199f, 'NewTaiLueLetterLowLa', [], 'Display', None],
  [ 0x19a0, 'NewTaiLueLetterHighHa', [], 'Display', None],
  [ 0x19a1, 'NewTaiLueLetterHighDa', [], 'Display', None],
  [ 0x19a2, 'NewTaiLueLetterHighBa', [], 'Display', None],
  [ 0x19a3, 'NewTaiLueLetterLowHa', [], 'Display', None],
  [ 0x19a4, 'NewTaiLueLetterLowDa', [], 'Display', None],
  [ 0x19a5, 'NewTaiLueLetterLowBa', [], 'Display', None],
  [ 0x19a6, 'NewTaiLueLetterHighKva', [], 'Display', None],
  [ 0x19a7, 'NewTaiLueLetterHighXva', [], 'Display', None],
  [ 0x19a8, 'NewTaiLueLetterLowKva', [], 'Display', None],
  [ 0x19a9, 'NewTaiLueLetterLowXva', [], 'Display', None],
  [ 0x19aa, 'UnicodeCharacter0x19aa', [], 'Display', None],
  [ 0x19ab, 'UnicodeCharacter0x19ab', [], 'Display', None],
  [ 0x19ac, 'UnicodeCharacter0x19ac', [], 'Display', None],
  [ 0x19ad, 'UnicodeCharacter0x19ad', [], 'Display', None],
  [ 0x19ae, 'UnicodeCharacter0x19ae', [], 'Display', None],
  [ 0x19af, 'UnicodeCharacter0x19af', [], 'Display', None],
  [ 0x19b0, 'NewTaiLueVowelSignVowelShortener', [], 'Display', None],
  [ 0x19b1, 'NewTaiLueVowelSignAa', [], 'Display', None],
  [ 0x19b2, 'NewTaiLueVowelSignIi', [], 'Display', None],
  [ 0x19b3, 'NewTaiLueVowelSignU', [], 'Display', None],
  [ 0x19b4, 'NewTaiLueVowelSignUu', [], 'Display', None],
  [ 0x19b5, 'NewTaiLueVowelSignE', [], 'Display', None],
  [ 0x19b6, 'NewTaiLueVowelSignAe', [], 'Display', None],
  [ 0x19b7, 'NewTaiLueVowelSignO', [], 'Display', None],
  [ 0x19b8, 'NewTaiLueVowelSignOa', [], 'Display', None],
  [ 0x19b9, 'NewTaiLueVowelSignUe', [], 'Display', None],
  [ 0x19ba, 'NewTaiLueVowelSignAy', [], 'Display', None],
  [ 0x19bb, 'NewTaiLueVowelSignAay', [], 'Display', None],
  [ 0x19bc, 'NewTaiLueVowelSignUy', [], 'Display', None],
  [ 0x19bd, 'NewTaiLueVowelSignOy', [], 'Display', None],
  [ 0x19be, 'NewTaiLueVowelSignOay', [], 'Display', None],
  [ 0x19bf, 'NewTaiLueVowelSignUey', [], 'Display', None],
  [ 0x19c0, 'NewTaiLueVowelSignIy', [], 'Display', None],
  [ 0x19c1, 'NewTaiLueLetterFinalV', [], 'Display', None],
  [ 0x19c2, 'NewTaiLueLetterFinalNg', [], 'Display', None],
  [ 0x19c3, 'NewTaiLueLetterFinalN', [], 'Display', None],
  [ 0x19c4, 'NewTaiLueLetterFinalM', [], 'Display', None],
  [ 0x19c5, 'NewTaiLueLetterFinalK', [], 'Display', None],
  [ 0x19c6, 'NewTaiLueLetterFinalD', [], 'Display', None],
  [ 0x19c7, 'NewTaiLueLetterFinalB', [], 'Display', None],
  [ 0x19c8, 'NewTaiLueToneMark1', [], 'Display', None],
  [ 0x19c9, 'NewTaiLueToneMark2', [], 'Display', None],
  [ 0x19ca, 'UnicodeCharacter0x19ca', [], 'Display', None],
  [ 0x19cb, 'UnicodeCharacter0x19cb', [], 'Display', None],
  [ 0x19cc, 'UnicodeCharacter0x19cc', [], 'Display', None],
  [ 0x19cd, 'UnicodeCharacter0x19cd', [], 'Display', None],
  [ 0x19ce, 'UnicodeCharacter0x19ce', [], 'Display', None],
  [ 0x19cf, 'UnicodeCharacter0x19cf', [], 'Display', None],
  [ 0x19d0, 'NewTaiLueDigitZero', [], 'Display', None],
  [ 0x19d1, 'NewTaiLueDigitOne', [], 'Display', None],
  [ 0x19d2, 'NewTaiLueDigitTwo', [], 'Display', None],
  [ 0x19d3, 'NewTaiLueDigitThree', [], 'Display', None],
  [ 0x19d4, 'NewTaiLueDigitFour', [], 'Display', None],
  [ 0x19d5, 'NewTaiLueDigitFive', [], 'Display', None],
  [ 0x19d6, 'NewTaiLueDigitSix', [], 'Display', None],
  [ 0x19d7, 'NewTaiLueDigitSeven', [], 'Display', None],
  [ 0x19d8, 'NewTaiLueDigitEight', [], 'Display', None],
  [ 0x19d9, 'NewTaiLueDigitNine', [], 'Display', None],
  [ 0x19da, 'UnicodeCharacter0x19da', [], 'Display', None],
  [ 0x19db, 'UnicodeCharacter0x19db', [], 'Display', None],
  [ 0x19dc, 'UnicodeCharacter0x19dc', [], 'Display', None],
  [ 0x19dd, 'UnicodeCharacter0x19dd', [], 'Display', None],
  [ 0x19de, 'NewTaiLueSignLae', [], 'Display', None],
  [ 0x19df, 'NewTaiLueSignLaev', [], 'Display', None],
  [ 0x19e0, 'KhmerSymbolPathamasat', [], 'Display', None],
  [ 0x19e1, 'KhmerSymbolMuoyKoet', [], 'Display', None],
  [ 0x19e2, 'KhmerSymbolPiiKoet', [], 'Display', None],
  [ 0x19e3, 'KhmerSymbolBeiKoet', [], 'Display', None],
  [ 0x19e4, 'KhmerSymbolBuonKoet', [], 'Display', None],
  [ 0x19e5, 'KhmerSymbolPramKoet', [], 'Display', None],
  [ 0x19e6, 'KhmerSymbolPrammuoyKoet', [], 'Display', None],
  [ 0x19e7, 'KhmerSymbolPrampiiKoet', [], 'Display', None],
  [ 0x19e8, 'KhmerSymbolPrambeiKoet', [], 'Display', None],
  [ 0x19e9, 'KhmerSymbolPrambuonKoet', [], 'Display', None],
  [ 0x19ea, 'KhmerSymbolDapKoet', [], 'Display', None],
  [ 0x19eb, 'KhmerSymbolDapmuoyKoet', [], 'Display', None],
  [ 0x19ec, 'KhmerSymbolDappiiKoet', [], 'Display', None],
  [ 0x19ed, 'KhmerSymbolDapbeiKoet', [], 'Display', None],
  [ 0x19ee, 'KhmerSymbolDapbuonKoet', [], 'Display', None],
  [ 0x19ef, 'KhmerSymbolDappramKoet', [], 'Display', None],
  [ 0x19f0, 'KhmerSymbolTuteyasat', [], 'Display', None],
  [ 0x19f1, 'KhmerSymbolMuoyRoc', [], 'Display', None],
  [ 0x19f2, 'KhmerSymbolPiiRoc', [], 'Display', None],
  [ 0x19f3, 'KhmerSymbolBeiRoc', [], 'Display', None],
  [ 0x19f4, 'KhmerSymbolBuonRoc', [], 'Display', None],
  [ 0x19f5, 'KhmerSymbolPramRoc', [], 'Display', None],
  [ 0x19f6, 'KhmerSymbolPrammuoyRoc', [], 'Display', None],
  [ 0x19f7, 'KhmerSymbolPrampiiRoc', [], 'Display', None],
  [ 0x19f8, 'KhmerSymbolPrambeiRoc', [], 'Display', None],
  [ 0x19f9, 'KhmerSymbolPrambuonRoc', [], 'Display', None],
  [ 0x19fa, 'KhmerSymbolDapRoc', [], 'Display', None],
  [ 0x19fb, 'KhmerSymbolDapmuoyRoc', [], 'Display', None],
  [ 0x19fc, 'KhmerSymbolDappiiRoc', [], 'Display', None],
  [ 0x19fd, 'KhmerSymbolDapbeiRoc', [], 'Display', None],
  [ 0x19fe, 'KhmerSymbolDapbuonRoc', [], 'Display', None],
  [ 0x19ff, 'KhmerSymbolDappramRoc', [], 'Display', None],
  [ 0x1a00, 'BugineseLetterKa', [], 'Display', None],
  [ 0x1a01, 'BugineseLetterGa', [], 'Display', None],
  [ 0x1a02, 'BugineseLetterNga', [], 'Display', None],
  [ 0x1a03, 'BugineseLetterNgka', [], 'Display', None],
  [ 0x1a04, 'BugineseLetterPa', [], 'Display', None],
  [ 0x1a05, 'BugineseLetterBa', [], 'Display', None],
  [ 0x1a06, 'BugineseLetterMa', [], 'Display', None],
  [ 0x1a07, 'BugineseLetterMpa', [], 'Display', None],
  [ 0x1a08, 'BugineseLetterTa', [], 'Display', None],
  [ 0x1a09, 'BugineseLetterDa', [], 'Display', None],
  [ 0x1a0a, 'BugineseLetterNa', [], 'Display', None],
  [ 0x1a0b, 'BugineseLetterNra', [], 'Display', None],
  [ 0x1a0c, 'BugineseLetterCa', [], 'Display', None],
  [ 0x1a0d, 'BugineseLetterJa', [], 'Display', None],
  [ 0x1a0e, 'BugineseLetterNya', [], 'Display', None],
  [ 0x1a0f, 'BugineseLetterNyca', [], 'Display', None],
  [ 0x1a10, 'BugineseLetterYa', [], 'Display', None],
  [ 0x1a11, 'BugineseLetterRa', [], 'Display', None],
  [ 0x1a12, 'BugineseLetterLa', [], 'Display', None],
  [ 0x1a13, 'BugineseLetterVa', [], 'Display', None],
  [ 0x1a14, 'BugineseLetterSa', [], 'Display', None],
  [ 0x1a15, 'BugineseLetterA', [], 'Display', None],
  [ 0x1a16, 'BugineseLetterHa', [], 'Display', None],
  [ 0x1a17, 'BugineseVowelSignI', [], 'Display', None],
  [ 0x1a18, 'BugineseVowelSignU', [], 'Display', None],
  [ 0x1a19, 'BugineseVowelSignE', [], 'Display', None],
  [ 0x1a1a, 'BugineseVowelSignO', [], 'Display', None],
  [ 0x1a1b, 'BugineseVowelSignAe', [], 'Display', None],
  [ 0x1a1c, 'UnicodeCharacter0x1a1c', [], 'Display', None],
  [ 0x1a1d, 'UnicodeCharacter0x1a1d', [], 'Display', None],
  [ 0x1a1e, 'BuginesePallawa', [], 'Display', None],
  [ 0x1a1f, 'BugineseEndOfSection', [], 'Display', None],
  [ 0x1a20, 'UnicodeCharacter0x1a20', [], 'Display', None],
  [ 0x1a21, 'UnicodeCharacter0x1a21', [], 'Display', None],
  [ 0x1a22, 'UnicodeCharacter0x1a22', [], 'Display', None],
  [ 0x1a23, 'UnicodeCharacter0x1a23', [], 'Display', None],
  [ 0x1a24, 'UnicodeCharacter0x1a24', [], 'Display', None],
  [ 0x1a25, 'UnicodeCharacter0x1a25', [], 'Display', None],
  [ 0x1a26, 'UnicodeCharacter0x1a26', [], 'Display', None],
  [ 0x1a27, 'UnicodeCharacter0x1a27', [], 'Display', None],
  [ 0x1a28, 'UnicodeCharacter0x1a28', [], 'Display', None],
  [ 0x1a29, 'UnicodeCharacter0x1a29', [], 'Display', None],
  [ 0x1a2a, 'UnicodeCharacter0x1a2a', [], 'Display', None],
  [ 0x1a2b, 'UnicodeCharacter0x1a2b', [], 'Display', None],
  [ 0x1a2c, 'UnicodeCharacter0x1a2c', [], 'Display', None],
  [ 0x1a2d, 'UnicodeCharacter0x1a2d', [], 'Display', None],
  [ 0x1a2e, 'UnicodeCharacter0x1a2e', [], 'Display', None],
  [ 0x1a2f, 'UnicodeCharacter0x1a2f', [], 'Display', None],
  [ 0x1a30, 'UnicodeCharacter0x1a30', [], 'Display', None],
  [ 0x1a31, 'UnicodeCharacter0x1a31', [], 'Display', None],
  [ 0x1a32, 'UnicodeCharacter0x1a32', [], 'Display', None],
  [ 0x1a33, 'UnicodeCharacter0x1a33', [], 'Display', None],
  [ 0x1a34, 'UnicodeCharacter0x1a34', [], 'Display', None],
  [ 0x1a35, 'UnicodeCharacter0x1a35', [], 'Display', None],
  [ 0x1a36, 'UnicodeCharacter0x1a36', [], 'Display', None],
  [ 0x1a37, 'UnicodeCharacter0x1a37', [], 'Display', None],
  [ 0x1a38, 'UnicodeCharacter0x1a38', [], 'Display', None],
  [ 0x1a39, 'UnicodeCharacter0x1a39', [], 'Display', None],
  [ 0x1a3a, 'UnicodeCharacter0x1a3a', [], 'Display', None],
  [ 0x1a3b, 'UnicodeCharacter0x1a3b', [], 'Display', None],
  [ 0x1a3c, 'UnicodeCharacter0x1a3c', [], 'Display', None],
  [ 0x1a3d, 'UnicodeCharacter0x1a3d', [], 'Display', None],
  [ 0x1a3e, 'UnicodeCharacter0x1a3e', [], 'Display', None],
  [ 0x1a3f, 'UnicodeCharacter0x1a3f', [], 'Display', None],
  [ 0x1a40, 'UnicodeCharacter0x1a40', [], 'Display', None],
  [ 0x1a41, 'UnicodeCharacter0x1a41', [], 'Display', None],
  [ 0x1a42, 'UnicodeCharacter0x1a42', [], 'Display', None],
  [ 0x1a43, 'UnicodeCharacter0x1a43', [], 'Display', None],
  [ 0x1a44, 'UnicodeCharacter0x1a44', [], 'Display', None],
  [ 0x1a45, 'UnicodeCharacter0x1a45', [], 'Display', None],
  [ 0x1a46, 'UnicodeCharacter0x1a46', [], 'Display', None],
  [ 0x1a47, 'UnicodeCharacter0x1a47', [], 'Display', None],
  [ 0x1a48, 'UnicodeCharacter0x1a48', [], 'Display', None],
  [ 0x1a49, 'UnicodeCharacter0x1a49', [], 'Display', None],
  [ 0x1a4a, 'UnicodeCharacter0x1a4a', [], 'Display', None],
  [ 0x1a4b, 'UnicodeCharacter0x1a4b', [], 'Display', None],
  [ 0x1a4c, 'UnicodeCharacter0x1a4c', [], 'Display', None],
  [ 0x1a4d, 'UnicodeCharacter0x1a4d', [], 'Display', None],
  [ 0x1a4e, 'UnicodeCharacter0x1a4e', [], 'Display', None],
  [ 0x1a4f, 'UnicodeCharacter0x1a4f', [], 'Display', None],
  [ 0x1a50, 'UnicodeCharacter0x1a50', [], 'Display', None],
  [ 0x1a51, 'UnicodeCharacter0x1a51', [], 'Display', None],
  [ 0x1a52, 'UnicodeCharacter0x1a52', [], 'Display', None],
  [ 0x1a53, 'UnicodeCharacter0x1a53', [], 'Display', None],
  [ 0x1a54, 'UnicodeCharacter0x1a54', [], 'Display', None],
  [ 0x1a55, 'UnicodeCharacter0x1a55', [], 'Display', None],
  [ 0x1a56, 'UnicodeCharacter0x1a56', [], 'Display', None],
  [ 0x1a57, 'UnicodeCharacter0x1a57', [], 'Display', None],
  [ 0x1a58, 'UnicodeCharacter0x1a58', [], 'Display', None],
  [ 0x1a59, 'UnicodeCharacter0x1a59', [], 'Display', None],
  [ 0x1a5a, 'UnicodeCharacter0x1a5a', [], 'Display', None],
  [ 0x1a5b, 'UnicodeCharacter0x1a5b', [], 'Display', None],
  [ 0x1a5c, 'UnicodeCharacter0x1a5c', [], 'Display', None],
  [ 0x1a5d, 'UnicodeCharacter0x1a5d', [], 'Display', None],
  [ 0x1a5e, 'UnicodeCharacter0x1a5e', [], 'Display', None],
  [ 0x1a5f, 'UnicodeCharacter0x1a5f', [], 'Display', None],
  [ 0x1a60, 'UnicodeCharacter0x1a60', [], 'Display', None],
  [ 0x1a61, 'UnicodeCharacter0x1a61', [], 'Display', None],
  [ 0x1a62, 'UnicodeCharacter0x1a62', [], 'Display', None],
  [ 0x1a63, 'UnicodeCharacter0x1a63', [], 'Display', None],
  [ 0x1a64, 'UnicodeCharacter0x1a64', [], 'Display', None],
  [ 0x1a65, 'UnicodeCharacter0x1a65', [], 'Display', None],
  [ 0x1a66, 'UnicodeCharacter0x1a66', [], 'Display', None],
  [ 0x1a67, 'UnicodeCharacter0x1a67', [], 'Display', None],
  [ 0x1a68, 'UnicodeCharacter0x1a68', [], 'Display', None],
  [ 0x1a69, 'UnicodeCharacter0x1a69', [], 'Display', None],
  [ 0x1a6a, 'UnicodeCharacter0x1a6a', [], 'Display', None],
  [ 0x1a6b, 'UnicodeCharacter0x1a6b', [], 'Display', None],
  [ 0x1a6c, 'UnicodeCharacter0x1a6c', [], 'Display', None],
  [ 0x1a6d, 'UnicodeCharacter0x1a6d', [], 'Display', None],
  [ 0x1a6e, 'UnicodeCharacter0x1a6e', [], 'Display', None],
  [ 0x1a6f, 'UnicodeCharacter0x1a6f', [], 'Display', None],
  [ 0x1a70, 'UnicodeCharacter0x1a70', [], 'Display', None],
  [ 0x1a71, 'UnicodeCharacter0x1a71', [], 'Display', None],
  [ 0x1a72, 'UnicodeCharacter0x1a72', [], 'Display', None],
  [ 0x1a73, 'UnicodeCharacter0x1a73', [], 'Display', None],
  [ 0x1a74, 'UnicodeCharacter0x1a74', [], 'Display', None],
  [ 0x1a75, 'UnicodeCharacter0x1a75', [], 'Display', None],
  [ 0x1a76, 'UnicodeCharacter0x1a76', [], 'Display', None],
  [ 0x1a77, 'UnicodeCharacter0x1a77', [], 'Display', None],
  [ 0x1a78, 'UnicodeCharacter0x1a78', [], 'Display', None],
  [ 0x1a79, 'UnicodeCharacter0x1a79', [], 'Display', None],
  [ 0x1a7a, 'UnicodeCharacter0x1a7a', [], 'Display', None],
  [ 0x1a7b, 'UnicodeCharacter0x1a7b', [], 'Display', None],
  [ 0x1a7c, 'UnicodeCharacter0x1a7c', [], 'Display', None],
  [ 0x1a7d, 'UnicodeCharacter0x1a7d', [], 'Display', None],
  [ 0x1a7e, 'UnicodeCharacter0x1a7e', [], 'Display', None],
  [ 0x1a7f, 'UnicodeCharacter0x1a7f', [], 'Display', None],
  [ 0x1a80, 'UnicodeCharacter0x1a80', [], 'Display', None],
  [ 0x1a81, 'UnicodeCharacter0x1a81', [], 'Display', None],
  [ 0x1a82, 'UnicodeCharacter0x1a82', [], 'Display', None],
  [ 0x1a83, 'UnicodeCharacter0x1a83', [], 'Display', None],
  [ 0x1a84, 'UnicodeCharacter0x1a84', [], 'Display', None],
  [ 0x1a85, 'UnicodeCharacter0x1a85', [], 'Display', None],
  [ 0x1a86, 'UnicodeCharacter0x1a86', [], 'Display', None],
  [ 0x1a87, 'UnicodeCharacter0x1a87', [], 'Display', None],
  [ 0x1a88, 'UnicodeCharacter0x1a88', [], 'Display', None],
  [ 0x1a89, 'UnicodeCharacter0x1a89', [], 'Display', None],
  [ 0x1a8a, 'UnicodeCharacter0x1a8a', [], 'Display', None],
  [ 0x1a8b, 'UnicodeCharacter0x1a8b', [], 'Display', None],
  [ 0x1a8c, 'UnicodeCharacter0x1a8c', [], 'Display', None],
  [ 0x1a8d, 'UnicodeCharacter0x1a8d', [], 'Display', None],
  [ 0x1a8e, 'UnicodeCharacter0x1a8e', [], 'Display', None],
  [ 0x1a8f, 'UnicodeCharacter0x1a8f', [], 'Display', None],
  [ 0x1a90, 'UnicodeCharacter0x1a90', [], 'Display', None],
  [ 0x1a91, 'UnicodeCharacter0x1a91', [], 'Display', None],
  [ 0x1a92, 'UnicodeCharacter0x1a92', [], 'Display', None],
  [ 0x1a93, 'UnicodeCharacter0x1a93', [], 'Display', None],
  [ 0x1a94, 'UnicodeCharacter0x1a94', [], 'Display', None],
  [ 0x1a95, 'UnicodeCharacter0x1a95', [], 'Display', None],
  [ 0x1a96, 'UnicodeCharacter0x1a96', [], 'Display', None],
  [ 0x1a97, 'UnicodeCharacter0x1a97', [], 'Display', None],
  [ 0x1a98, 'UnicodeCharacter0x1a98', [], 'Display', None],
  [ 0x1a99, 'UnicodeCharacter0x1a99', [], 'Display', None],
  [ 0x1a9a, 'UnicodeCharacter0x1a9a', [], 'Display', None],
  [ 0x1a9b, 'UnicodeCharacter0x1a9b', [], 'Display', None],
  [ 0x1a9c, 'UnicodeCharacter0x1a9c', [], 'Display', None],
  [ 0x1a9d, 'UnicodeCharacter0x1a9d', [], 'Display', None],
  [ 0x1a9e, 'UnicodeCharacter0x1a9e', [], 'Display', None],
  [ 0x1a9f, 'UnicodeCharacter0x1a9f', [], 'Display', None],
  [ 0x1aa0, 'UnicodeCharacter0x1aa0', [], 'Display', None],
  [ 0x1aa1, 'UnicodeCharacter0x1aa1', [], 'Display', None],
  [ 0x1aa2, 'UnicodeCharacter0x1aa2', [], 'Display', None],
  [ 0x1aa3, 'UnicodeCharacter0x1aa3', [], 'Display', None],
  [ 0x1aa4, 'UnicodeCharacter0x1aa4', [], 'Display', None],
  [ 0x1aa5, 'UnicodeCharacter0x1aa5', [], 'Display', None],
  [ 0x1aa6, 'UnicodeCharacter0x1aa6', [], 'Display', None],
  [ 0x1aa7, 'UnicodeCharacter0x1aa7', [], 'Display', None],
  [ 0x1aa8, 'UnicodeCharacter0x1aa8', [], 'Display', None],
  [ 0x1aa9, 'UnicodeCharacter0x1aa9', [], 'Display', None],
  [ 0x1aaa, 'UnicodeCharacter0x1aaa', [], 'Display', None],
  [ 0x1aab, 'UnicodeCharacter0x1aab', [], 'Display', None],
  [ 0x1aac, 'UnicodeCharacter0x1aac', [], 'Display', None],
  [ 0x1aad, 'UnicodeCharacter0x1aad', [], 'Display', None],
  [ 0x1aae, 'UnicodeCharacter0x1aae', [], 'Display', None],
  [ 0x1aaf, 'UnicodeCharacter0x1aaf', [], 'Display', None],
  [ 0x1ab0, 'UnicodeCharacter0x1ab0', [], 'Display', None],
  [ 0x1ab1, 'UnicodeCharacter0x1ab1', [], 'Display', None],
  [ 0x1ab2, 'UnicodeCharacter0x1ab2', [], 'Display', None],
  [ 0x1ab3, 'UnicodeCharacter0x1ab3', [], 'Display', None],
  [ 0x1ab4, 'UnicodeCharacter0x1ab4', [], 'Display', None],
  [ 0x1ab5, 'UnicodeCharacter0x1ab5', [], 'Display', None],
  [ 0x1ab6, 'UnicodeCharacter0x1ab6', [], 'Display', None],
  [ 0x1ab7, 'UnicodeCharacter0x1ab7', [], 'Display', None],
  [ 0x1ab8, 'UnicodeCharacter0x1ab8', [], 'Display', None],
  [ 0x1ab9, 'UnicodeCharacter0x1ab9', [], 'Display', None],
  [ 0x1aba, 'UnicodeCharacter0x1aba', [], 'Display', None],
  [ 0x1abb, 'UnicodeCharacter0x1abb', [], 'Display', None],
  [ 0x1abc, 'UnicodeCharacter0x1abc', [], 'Display', None],
  [ 0x1abd, 'UnicodeCharacter0x1abd', [], 'Display', None],
  [ 0x1abe, 'UnicodeCharacter0x1abe', [], 'Display', None],
  [ 0x1abf, 'UnicodeCharacter0x1abf', [], 'Display', None],
  [ 0x1ac0, 'UnicodeCharacter0x1ac0', [], 'Display', None],
  [ 0x1ac1, 'UnicodeCharacter0x1ac1', [], 'Display', None],
  [ 0x1ac2, 'UnicodeCharacter0x1ac2', [], 'Display', None],
  [ 0x1ac3, 'UnicodeCharacter0x1ac3', [], 'Display', None],
  [ 0x1ac4, 'UnicodeCharacter0x1ac4', [], 'Display', None],
  [ 0x1ac5, 'UnicodeCharacter0x1ac5', [], 'Display', None],
  [ 0x1ac6, 'UnicodeCharacter0x1ac6', [], 'Display', None],
  [ 0x1ac7, 'UnicodeCharacter0x1ac7', [], 'Display', None],
  [ 0x1ac8, 'UnicodeCharacter0x1ac8', [], 'Display', None],
  [ 0x1ac9, 'UnicodeCharacter0x1ac9', [], 'Display', None],
  [ 0x1aca, 'UnicodeCharacter0x1aca', [], 'Display', None],
  [ 0x1acb, 'UnicodeCharacter0x1acb', [], 'Display', None],
  [ 0x1acc, 'UnicodeCharacter0x1acc', [], 'Display', None],
  [ 0x1acd, 'UnicodeCharacter0x1acd', [], 'Display', None],
  [ 0x1ace, 'UnicodeCharacter0x1ace', [], 'Display', None],
  [ 0x1acf, 'UnicodeCharacter0x1acf', [], 'Display', None],
  [ 0x1ad0, 'UnicodeCharacter0x1ad0', [], 'Display', None],
  [ 0x1ad1, 'UnicodeCharacter0x1ad1', [], 'Display', None],
  [ 0x1ad2, 'UnicodeCharacter0x1ad2', [], 'Display', None],
  [ 0x1ad3, 'UnicodeCharacter0x1ad3', [], 'Display', None],
  [ 0x1ad4, 'UnicodeCharacter0x1ad4', [], 'Display', None],
  [ 0x1ad5, 'UnicodeCharacter0x1ad5', [], 'Display', None],
  [ 0x1ad6, 'UnicodeCharacter0x1ad6', [], 'Display', None],
  [ 0x1ad7, 'UnicodeCharacter0x1ad7', [], 'Display', None],
  [ 0x1ad8, 'UnicodeCharacter0x1ad8', [], 'Display', None],
  [ 0x1ad9, 'UnicodeCharacter0x1ad9', [], 'Display', None],
  [ 0x1ada, 'UnicodeCharacter0x1ada', [], 'Display', None],
  [ 0x1adb, 'UnicodeCharacter0x1adb', [], 'Display', None],
  [ 0x1adc, 'UnicodeCharacter0x1adc', [], 'Display', None],
  [ 0x1add, 'UnicodeCharacter0x1add', [], 'Display', None],
  [ 0x1ade, 'UnicodeCharacter0x1ade', [], 'Display', None],
  [ 0x1adf, 'UnicodeCharacter0x1adf', [], 'Display', None],
  [ 0x1ae0, 'UnicodeCharacter0x1ae0', [], 'Display', None],
  [ 0x1ae1, 'UnicodeCharacter0x1ae1', [], 'Display', None],
  [ 0x1ae2, 'UnicodeCharacter0x1ae2', [], 'Display', None],
  [ 0x1ae3, 'UnicodeCharacter0x1ae3', [], 'Display', None],
  [ 0x1ae4, 'UnicodeCharacter0x1ae4', [], 'Display', None],
  [ 0x1ae5, 'UnicodeCharacter0x1ae5', [], 'Display', None],
  [ 0x1ae6, 'UnicodeCharacter0x1ae6', [], 'Display', None],
  [ 0x1ae7, 'UnicodeCharacter0x1ae7', [], 'Display', None],
  [ 0x1ae8, 'UnicodeCharacter0x1ae8', [], 'Display', None],
  [ 0x1ae9, 'UnicodeCharacter0x1ae9', [], 'Display', None],
  [ 0x1aea, 'UnicodeCharacter0x1aea', [], 'Display', None],
  [ 0x1aeb, 'UnicodeCharacter0x1aeb', [], 'Display', None],
  [ 0x1aec, 'UnicodeCharacter0x1aec', [], 'Display', None],
  [ 0x1aed, 'UnicodeCharacter0x1aed', [], 'Display', None],
  [ 0x1aee, 'UnicodeCharacter0x1aee', [], 'Display', None],
  [ 0x1aef, 'UnicodeCharacter0x1aef', [], 'Display', None],
  [ 0x1af0, 'UnicodeCharacter0x1af0', [], 'Display', None],
  [ 0x1af1, 'UnicodeCharacter0x1af1', [], 'Display', None],
  [ 0x1af2, 'UnicodeCharacter0x1af2', [], 'Display', None],
  [ 0x1af3, 'UnicodeCharacter0x1af3', [], 'Display', None],
  [ 0x1af4, 'UnicodeCharacter0x1af4', [], 'Display', None],
  [ 0x1af5, 'UnicodeCharacter0x1af5', [], 'Display', None],
  [ 0x1af6, 'UnicodeCharacter0x1af6', [], 'Display', None],
  [ 0x1af7, 'UnicodeCharacter0x1af7', [], 'Display', None],
  [ 0x1af8, 'UnicodeCharacter0x1af8', [], 'Display', None],
  [ 0x1af9, 'UnicodeCharacter0x1af9', [], 'Display', None],
  [ 0x1afa, 'UnicodeCharacter0x1afa', [], 'Display', None],
  [ 0x1afb, 'UnicodeCharacter0x1afb', [], 'Display', None],
  [ 0x1afc, 'UnicodeCharacter0x1afc', [], 'Display', None],
  [ 0x1afd, 'UnicodeCharacter0x1afd', [], 'Display', None],
  [ 0x1afe, 'UnicodeCharacter0x1afe', [], 'Display', None],
  [ 0x1aff, 'UnicodeCharacter0x1aff', [], 'Display', None],
  [ 0x1b00, 'BalineseSignUluRicem', [], 'Display', None],
  [ 0x1b01, 'BalineseSignUluCandra', [], 'Display', None],
  [ 0x1b02, 'BalineseSignCecek', [], 'Display', None],
  [ 0x1b03, 'BalineseSignSurang', [], 'Display', None],
  [ 0x1b04, 'BalineseSignBisah', [], 'Display', None],
  [ 0x1b05, 'BalineseLetterAkara', [], 'Display', None],
  [ 0x1b06, 'BalineseLetterAkaraTedung', [], 'Display', None],
  [ 0x1b07, 'BalineseLetterIkara', [], 'Display', None],
  [ 0x1b08, 'BalineseLetterIkaraTedung', [], 'Display', None],
  [ 0x1b09, 'BalineseLetterUkara', [], 'Display', None],
  [ 0x1b0a, 'BalineseLetterUkaraTedung', [], 'Display', None],
  [ 0x1b0b, 'BalineseLetterRaRepa', [], 'Display', None],
  [ 0x1b0c, 'BalineseLetterRaRepaTedung', [], 'Display', None],
  [ 0x1b0d, 'BalineseLetterLaLenga', [], 'Display', None],
  [ 0x1b0e, 'BalineseLetterLaLengaTedung', [], 'Display', None],
  [ 0x1b0f, 'BalineseLetterEkara', [], 'Display', None],
  [ 0x1b10, 'BalineseLetterAikara', [], 'Display', None],
  [ 0x1b11, 'BalineseLetterOkara', [], 'Display', None],
  [ 0x1b12, 'BalineseLetterOkaraTedung', [], 'Display', None],
  [ 0x1b13, 'BalineseLetterKa', [], 'Display', None],
  [ 0x1b14, 'BalineseLetterKaMahaprana', [], 'Display', None],
  [ 0x1b15, 'BalineseLetterGa', [], 'Display', None],
  [ 0x1b16, 'BalineseLetterGaGora', [], 'Display', None],
  [ 0x1b17, 'BalineseLetterNga', [], 'Display', None],
  [ 0x1b18, 'BalineseLetterCa', [], 'Display', None],
  [ 0x1b19, 'BalineseLetterCaLaca', [], 'Display', None],
  [ 0x1b1a, 'BalineseLetterJa', [], 'Display', None],
  [ 0x1b1b, 'BalineseLetterJaJera', [], 'Display', None],
  [ 0x1b1c, 'BalineseLetterNya', [], 'Display', None],
  [ 0x1b1d, 'BalineseLetterTaLatik', [], 'Display', None],
  [ 0x1b1e, 'BalineseLetterTaMurdaMahaprana', [], 'Display', None],
  [ 0x1b1f, 'BalineseLetterDaMurdaAlpaprana', [], 'Display', None],
  [ 0x1b20, 'BalineseLetterDaMurdaMahaprana', [], 'Display', None],
  [ 0x1b21, 'BalineseLetterNaRambat', [], 'Display', None],
  [ 0x1b22, 'BalineseLetterTa', [], 'Display', None],
  [ 0x1b23, 'BalineseLetterTaTawa', [], 'Display', None],
  [ 0x1b24, 'BalineseLetterDa', [], 'Display', None],
  [ 0x1b25, 'BalineseLetterDaMadu', [], 'Display', None],
  [ 0x1b26, 'BalineseLetterNa', [], 'Display', None],
  [ 0x1b27, 'BalineseLetterPa', [], 'Display', None],
  [ 0x1b28, 'BalineseLetterPaKapal', [], 'Display', None],
  [ 0x1b29, 'BalineseLetterBa', [], 'Display', None],
  [ 0x1b2a, 'BalineseLetterBaKembang', [], 'Display', None],
  [ 0x1b2b, 'BalineseLetterMa', [], 'Display', None],
  [ 0x1b2c, 'BalineseLetterYa', [], 'Display', None],
  [ 0x1b2d, 'BalineseLetterRa', [], 'Display', None],
  [ 0x1b2e, 'BalineseLetterLa', [], 'Display', None],
  [ 0x1b2f, 'BalineseLetterWa', [], 'Display', None],
  [ 0x1b30, 'BalineseLetterSaSaga', [], 'Display', None],
  [ 0x1b31, 'BalineseLetterSaSapa', [], 'Display', None],
  [ 0x1b32, 'BalineseLetterSa', [], 'Display', None],
  [ 0x1b33, 'BalineseLetterHa', [], 'Display', None],
  [ 0x1b34, 'BalineseSignRerekan', [], 'Display', None],
  [ 0x1b35, 'BalineseVowelSignTedung', [], 'Display', None],
  [ 0x1b36, 'BalineseVowelSignUlu', [], 'Display', None],
  [ 0x1b37, 'BalineseVowelSignUluSari', [], 'Display', None],
  [ 0x1b38, 'BalineseVowelSignSuku', [], 'Display', None],
  [ 0x1b39, 'BalineseVowelSignSukuIlut', [], 'Display', None],
  [ 0x1b3a, 'BalineseVowelSignRaRepa', [], 'Display', None],
  [ 0x1b3b, 'BalineseVowelSignRaRepaTedung', [], 'Display', None],
  [ 0x1b3c, 'BalineseVowelSignLaLenga', [], 'Display', None],
  [ 0x1b3d, 'BalineseVowelSignLaLengaTedung', [], 'Display', None],
  [ 0x1b3e, 'BalineseVowelSignTaling', [], 'Display', None],
  [ 0x1b3f, 'BalineseVowelSignTalingRepa', [], 'Display', None],
  [ 0x1b40, 'BalineseVowelSignTalingTedung', [], 'Display', None],
  [ 0x1b41, 'BalineseVowelSignTalingRepaTedung', [], 'Display', None],
  [ 0x1b42, 'BalineseVowelSignPepet', [], 'Display', None],
  [ 0x1b43, 'BalineseVowelSignPepetTedung', [], 'Display', None],
  [ 0x1b44, 'BalineseAdegAdeg', [], 'Display', None],
  [ 0x1b45, 'BalineseLetterKafSasak', [], 'Display', None],
  [ 0x1b46, 'BalineseLetterKhotSasak', [], 'Display', None],
  [ 0x1b47, 'BalineseLetterTzirSasak', [], 'Display', None],
  [ 0x1b48, 'BalineseLetterEfSasak', [], 'Display', None],
  [ 0x1b49, 'BalineseLetterVeSasak', [], 'Display', None],
  [ 0x1b4a, 'BalineseLetterZalSasak', [], 'Display', None],
  [ 0x1b4b, 'BalineseLetterAsyuraSasak', [], 'Display', None],
  [ 0x1b4c, 'UnicodeCharacter0x1b4c', [], 'Display', None],
  [ 0x1b4d, 'UnicodeCharacter0x1b4d', [], 'Display', None],
  [ 0x1b4e, 'UnicodeCharacter0x1b4e', [], 'Display', None],
  [ 0x1b4f, 'UnicodeCharacter0x1b4f', [], 'Display', None],
  [ 0x1b50, 'BalineseDigitZero', [], 'Display', None],
  [ 0x1b51, 'BalineseDigitOne', [], 'Display', None],
  [ 0x1b52, 'BalineseDigitTwo', [], 'Display', None],
  [ 0x1b53, 'BalineseDigitThree', [], 'Display', None],
  [ 0x1b54, 'BalineseDigitFour', [], 'Display', None],
  [ 0x1b55, 'BalineseDigitFive', [], 'Display', None],
  [ 0x1b56, 'BalineseDigitSix', [], 'Display', None],
  [ 0x1b57, 'BalineseDigitSeven', [], 'Display', None],
  [ 0x1b58, 'BalineseDigitEight', [], 'Display', None],
  [ 0x1b59, 'BalineseDigitNine', [], 'Display', None],
  [ 0x1b5a, 'BalinesePanti', [], 'Display', None],
  [ 0x1b5b, 'BalinesePamada', [], 'Display', None],
  [ 0x1b5c, 'BalineseWindu', [], 'Display', None],
  [ 0x1b5d, 'BalineseCarikPamungkah', [], 'Display', None],
  [ 0x1b5e, 'BalineseCarikSiki', [], 'Display', None],
  [ 0x1b5f, 'BalineseCarikPareren', [], 'Display', None],
  [ 0x1b60, 'BalinesePameneng', [], 'Display', None],
  [ 0x1b61, 'BalineseMusicalSymbolDong', [], 'Display', None],
  [ 0x1b62, 'BalineseMusicalSymbolDeng', [], 'Display', None],
  [ 0x1b63, 'BalineseMusicalSymbolDung', [], 'Display', None],
  [ 0x1b64, 'BalineseMusicalSymbolDang', [], 'Display', None],
  [ 0x1b65, 'BalineseMusicalSymbolDangSurang', [], 'Display', None],
  [ 0x1b66, 'BalineseMusicalSymbolDing', [], 'Display', None],
  [ 0x1b67, 'BalineseMusicalSymbolDaeng', [], 'Display', None],
  [ 0x1b68, 'BalineseMusicalSymbolDeung', [], 'Display', None],
  [ 0x1b69, 'BalineseMusicalSymbolDaing', [], 'Display', None],
  [ 0x1b6a, 'BalineseMusicalSymbolDangGede', [], 'Display', None],
  [ 0x1b6b, 'BalineseMusicalSymbolCombiningTegeh', [], 'Display', None],
  [ 0x1b6c, 'BalineseMusicalSymbolCombiningEndep', [], 'Display', None],
  [ 0x1b6d, 'BalineseMusicalSymbolCombiningKempul', [], 'Display', None],
  [ 0x1b6e, 'BalineseMusicalSymbolCombiningKempli', [], 'Display', None],
  [ 0x1b6f, 'BalineseMusicalSymbolCombiningJegogan', [], 'Display', None],
  [ 0x1b70, 'BalineseMusicalSymbolCombiningKempulWithJegogan', [], 'Display', None],
  [ 0x1b71, 'BalineseMusicalSymbolCombiningKempliWithJegogan', [], 'Display', None],
  [ 0x1b72, 'BalineseMusicalSymbolCombiningBende', [], 'Display', None],
  [ 0x1b73, 'BalineseMusicalSymbolCombiningGong', [], 'Display', None],
  [ 0x1b74, 'BalineseMusicalSymbolRighthandOpenDug', [], 'Display', None],
  [ 0x1b75, 'BalineseMusicalSymbolRighthandOpenDag', [], 'Display', None],
  [ 0x1b76, 'BalineseMusicalSymbolRighthandClosedTuk', [], 'Display', None],
  [ 0x1b77, 'BalineseMusicalSymbolRighthandClosedTak', [], 'Display', None],
  [ 0x1b78, 'BalineseMusicalSymbolLefthandOpenPang', [], 'Display', None],
  [ 0x1b79, 'BalineseMusicalSymbolLefthandOpenPung', [], 'Display', None],
  [ 0x1b7a, 'BalineseMusicalSymbolLefthandClosedPlak', [], 'Display', None],
  [ 0x1b7b, 'BalineseMusicalSymbolLefthandClosedPluk', [], 'Display', None],
  [ 0x1b7c, 'BalineseMusicalSymbolLefthandOpenPing', [], 'Display', None],
  [ 0x1b7d, 'UnicodeCharacter0x1b7d', [], 'Display', None],
  [ 0x1b7e, 'UnicodeCharacter0x1b7e', [], 'Display', None],
  [ 0x1b7f, 'UnicodeCharacter0x1b7f', [], 'Display', None],
  [ 0x1b80, 'SundaneseSignPanyecek', [], 'Display', None],
  [ 0x1b81, 'SundaneseSignPanglayar', [], 'Display', None],
  [ 0x1b82, 'SundaneseSignPangwisad', [], 'Display', None],
  [ 0x1b83, 'SundaneseLetterA', [], 'Display', None],
  [ 0x1b84, 'SundaneseLetterI', [], 'Display', None],
  [ 0x1b85, 'SundaneseLetterU', [], 'Display', None],
  [ 0x1b86, 'SundaneseLetterAe', [], 'Display', None],
  [ 0x1b87, 'SundaneseLetterO', [], 'Display', None],
  [ 0x1b88, 'SundaneseLetterE', [], 'Display', None],
  [ 0x1b89, 'SundaneseLetterEu', [], 'Display', None],
  [ 0x1b8a, 'SundaneseLetterKa', [], 'Display', None],
  [ 0x1b8b, 'SundaneseLetterQa', [], 'Display', None],
  [ 0x1b8c, 'SundaneseLetterGa', [], 'Display', None],
  [ 0x1b8d, 'SundaneseLetterNga', [], 'Display', None],
  [ 0x1b8e, 'SundaneseLetterCa', [], 'Display', None],
  [ 0x1b8f, 'SundaneseLetterJa', [], 'Display', None],
  [ 0x1b90, 'SundaneseLetterZa', [], 'Display', None],
  [ 0x1b91, 'SundaneseLetterNya', [], 'Display', None],
  [ 0x1b92, 'SundaneseLetterTa', [], 'Display', None],
  [ 0x1b93, 'SundaneseLetterDa', [], 'Display', None],
  [ 0x1b94, 'SundaneseLetterNa', [], 'Display', None],
  [ 0x1b95, 'SundaneseLetterPa', [], 'Display', None],
  [ 0x1b96, 'SundaneseLetterFa', [], 'Display', None],
  [ 0x1b97, 'SundaneseLetterVa', [], 'Display', None],
  [ 0x1b98, 'SundaneseLetterBa', [], 'Display', None],
  [ 0x1b99, 'SundaneseLetterMa', [], 'Display', None],
  [ 0x1b9a, 'SundaneseLetterYa', [], 'Display', None],
  [ 0x1b9b, 'SundaneseLetterRa', [], 'Display', None],
  [ 0x1b9c, 'SundaneseLetterLa', [], 'Display', None],
  [ 0x1b9d, 'SundaneseLetterWa', [], 'Display', None],
  [ 0x1b9e, 'SundaneseLetterSa', [], 'Display', None],
  [ 0x1b9f, 'SundaneseLetterXa', [], 'Display', None],
  [ 0x1ba0, 'SundaneseLetterHa', [], 'Display', None],
  [ 0x1ba1, 'SundaneseConsonantSignPamingkal', [], 'Display', None],
  [ 0x1ba2, 'SundaneseConsonantSignPanyakra', [], 'Display', None],
  [ 0x1ba3, 'SundaneseConsonantSignPanyiku', [], 'Display', None],
  [ 0x1ba4, 'SundaneseVowelSignPanghulu', [], 'Display', None],
  [ 0x1ba5, 'SundaneseVowelSignPanyuku', [], 'Display', None],
  [ 0x1ba6, 'SundaneseVowelSignPanaelaeng', [], 'Display', None],
  [ 0x1ba7, 'SundaneseVowelSignPanolong', [], 'Display', None],
  [ 0x1ba8, 'SundaneseVowelSignPamepet', [], 'Display', None],
  [ 0x1ba9, 'SundaneseVowelSignPaneuleung', [], 'Display', None],
  [ 0x1baa, 'SundaneseSignPamaaeh', [], 'Display', None],
  [ 0x1bab, 'UnicodeCharacter0x1bab', [], 'Display', None],
  [ 0x1bac, 'UnicodeCharacter0x1bac', [], 'Display', None],
  [ 0x1bad, 'UnicodeCharacter0x1bad', [], 'Display', None],
  [ 0x1bae, 'SundaneseLetterKha', [], 'Display', None],
  [ 0x1baf, 'SundaneseLetterSya', [], 'Display', None],
  [ 0x1bb0, 'SundaneseDigitZero', [], 'Display', None],
  [ 0x1bb1, 'SundaneseDigitOne', [], 'Display', None],
  [ 0x1bb2, 'SundaneseDigitTwo', [], 'Display', None],
  [ 0x1bb3, 'SundaneseDigitThree', [], 'Display', None],
  [ 0x1bb4, 'SundaneseDigitFour', [], 'Display', None],
  [ 0x1bb5, 'SundaneseDigitFive', [], 'Display', None],
  [ 0x1bb6, 'SundaneseDigitSix', [], 'Display', None],
  [ 0x1bb7, 'SundaneseDigitSeven', [], 'Display', None],
  [ 0x1bb8, 'SundaneseDigitEight', [], 'Display', None],
  [ 0x1bb9, 'SundaneseDigitNine', [], 'Display', None],
  [ 0x1bba, 'UnicodeCharacter0x1bba', [], 'Display', None],
  [ 0x1bbb, 'UnicodeCharacter0x1bbb', [], 'Display', None],
  [ 0x1bbc, 'UnicodeCharacter0x1bbc', [], 'Display', None],
  [ 0x1bbd, 'UnicodeCharacter0x1bbd', [], 'Display', None],
  [ 0x1bbe, 'UnicodeCharacter0x1bbe', [], 'Display', None],
  [ 0x1bbf, 'UnicodeCharacter0x1bbf', [], 'Display', None],
  [ 0x1bc0, 'UnicodeCharacter0x1bc0', [], 'Display', None],
  [ 0x1bc1, 'UnicodeCharacter0x1bc1', [], 'Display', None],
  [ 0x1bc2, 'UnicodeCharacter0x1bc2', [], 'Display', None],
  [ 0x1bc3, 'UnicodeCharacter0x1bc3', [], 'Display', None],
  [ 0x1bc4, 'UnicodeCharacter0x1bc4', [], 'Display', None],
  [ 0x1bc5, 'UnicodeCharacter0x1bc5', [], 'Display', None],
  [ 0x1bc6, 'UnicodeCharacter0x1bc6', [], 'Display', None],
  [ 0x1bc7, 'UnicodeCharacter0x1bc7', [], 'Display', None],
  [ 0x1bc8, 'UnicodeCharacter0x1bc8', [], 'Display', None],
  [ 0x1bc9, 'UnicodeCharacter0x1bc9', [], 'Display', None],
  [ 0x1bca, 'UnicodeCharacter0x1bca', [], 'Display', None],
  [ 0x1bcb, 'UnicodeCharacter0x1bcb', [], 'Display', None],
  [ 0x1bcc, 'UnicodeCharacter0x1bcc', [], 'Display', None],
  [ 0x1bcd, 'UnicodeCharacter0x1bcd', [], 'Display', None],
  [ 0x1bce, 'UnicodeCharacter0x1bce', [], 'Display', None],
  [ 0x1bcf, 'UnicodeCharacter0x1bcf', [], 'Display', None],
  [ 0x1bd0, 'UnicodeCharacter0x1bd0', [], 'Display', None],
  [ 0x1bd1, 'UnicodeCharacter0x1bd1', [], 'Display', None],
  [ 0x1bd2, 'UnicodeCharacter0x1bd2', [], 'Display', None],
  [ 0x1bd3, 'UnicodeCharacter0x1bd3', [], 'Display', None],
  [ 0x1bd4, 'UnicodeCharacter0x1bd4', [], 'Display', None],
  [ 0x1bd5, 'UnicodeCharacter0x1bd5', [], 'Display', None],
  [ 0x1bd6, 'UnicodeCharacter0x1bd6', [], 'Display', None],
  [ 0x1bd7, 'UnicodeCharacter0x1bd7', [], 'Display', None],
  [ 0x1bd8, 'UnicodeCharacter0x1bd8', [], 'Display', None],
  [ 0x1bd9, 'UnicodeCharacter0x1bd9', [], 'Display', None],
  [ 0x1bda, 'UnicodeCharacter0x1bda', [], 'Display', None],
  [ 0x1bdb, 'UnicodeCharacter0x1bdb', [], 'Display', None],
  [ 0x1bdc, 'UnicodeCharacter0x1bdc', [], 'Display', None],
  [ 0x1bdd, 'UnicodeCharacter0x1bdd', [], 'Display', None],
  [ 0x1bde, 'UnicodeCharacter0x1bde', [], 'Display', None],
  [ 0x1bdf, 'UnicodeCharacter0x1bdf', [], 'Display', None],
  [ 0x1be0, 'UnicodeCharacter0x1be0', [], 'Display', None],
  [ 0x1be1, 'UnicodeCharacter0x1be1', [], 'Display', None],
  [ 0x1be2, 'UnicodeCharacter0x1be2', [], 'Display', None],
  [ 0x1be3, 'UnicodeCharacter0x1be3', [], 'Display', None],
  [ 0x1be4, 'UnicodeCharacter0x1be4', [], 'Display', None],
  [ 0x1be5, 'UnicodeCharacter0x1be5', [], 'Display', None],
  [ 0x1be6, 'UnicodeCharacter0x1be6', [], 'Display', None],
  [ 0x1be7, 'UnicodeCharacter0x1be7', [], 'Display', None],
  [ 0x1be8, 'UnicodeCharacter0x1be8', [], 'Display', None],
  [ 0x1be9, 'UnicodeCharacter0x1be9', [], 'Display', None],
  [ 0x1bea, 'UnicodeCharacter0x1bea', [], 'Display', None],
  [ 0x1beb, 'UnicodeCharacter0x1beb', [], 'Display', None],
  [ 0x1bec, 'UnicodeCharacter0x1bec', [], 'Display', None],
  [ 0x1bed, 'UnicodeCharacter0x1bed', [], 'Display', None],
  [ 0x1bee, 'UnicodeCharacter0x1bee', [], 'Display', None],
  [ 0x1bef, 'UnicodeCharacter0x1bef', [], 'Display', None],
  [ 0x1bf0, 'UnicodeCharacter0x1bf0', [], 'Display', None],
  [ 0x1bf1, 'UnicodeCharacter0x1bf1', [], 'Display', None],
  [ 0x1bf2, 'UnicodeCharacter0x1bf2', [], 'Display', None],
  [ 0x1bf3, 'UnicodeCharacter0x1bf3', [], 'Display', None],
  [ 0x1bf4, 'UnicodeCharacter0x1bf4', [], 'Display', None],
  [ 0x1bf5, 'UnicodeCharacter0x1bf5', [], 'Display', None],
  [ 0x1bf6, 'UnicodeCharacter0x1bf6', [], 'Display', None],
  [ 0x1bf7, 'UnicodeCharacter0x1bf7', [], 'Display', None],
  [ 0x1bf8, 'UnicodeCharacter0x1bf8', [], 'Display', None],
  [ 0x1bf9, 'UnicodeCharacter0x1bf9', [], 'Display', None],
  [ 0x1bfa, 'UnicodeCharacter0x1bfa', [], 'Display', None],
  [ 0x1bfb, 'UnicodeCharacter0x1bfb', [], 'Display', None],
  [ 0x1bfc, 'UnicodeCharacter0x1bfc', [], 'Display', None],
  [ 0x1bfd, 'UnicodeCharacter0x1bfd', [], 'Display', None],
  [ 0x1bfe, 'UnicodeCharacter0x1bfe', [], 'Display', None],
  [ 0x1bff, 'UnicodeCharacter0x1bff', [], 'Display', None],
  [ 0x1c00, 'LepchaLetterKa', [], 'Display', None],
  [ 0x1c01, 'LepchaLetterKla', [], 'Display', None],
  [ 0x1c02, 'LepchaLetterKha', [], 'Display', None],
  [ 0x1c03, 'LepchaLetterGa', [], 'Display', None],
  [ 0x1c04, 'LepchaLetterGla', [], 'Display', None],
  [ 0x1c05, 'LepchaLetterNga', [], 'Display', None],
  [ 0x1c06, 'LepchaLetterCa', [], 'Display', None],
  [ 0x1c07, 'LepchaLetterCha', [], 'Display', None],
  [ 0x1c08, 'LepchaLetterJa', [], 'Display', None],
  [ 0x1c09, 'LepchaLetterNya', [], 'Display', None],
  [ 0x1c0a, 'LepchaLetterTa', [], 'Display', None],
  [ 0x1c0b, 'LepchaLetterTha', [], 'Display', None],
  [ 0x1c0c, 'LepchaLetterDa', [], 'Display', None],
  [ 0x1c0d, 'LepchaLetterNa', [], 'Display', None],
  [ 0x1c0e, 'LepchaLetterPa', [], 'Display', None],
  [ 0x1c0f, 'LepchaLetterPla', [], 'Display', None],
  [ 0x1c10, 'LepchaLetterPha', [], 'Display', None],
  [ 0x1c11, 'LepchaLetterFa', [], 'Display', None],
  [ 0x1c12, 'LepchaLetterFla', [], 'Display', None],
  [ 0x1c13, 'LepchaLetterBa', [], 'Display', None],
  [ 0x1c14, 'LepchaLetterBla', [], 'Display', None],
  [ 0x1c15, 'LepchaLetterMa', [], 'Display', None],
  [ 0x1c16, 'LepchaLetterMla', [], 'Display', None],
  [ 0x1c17, 'LepchaLetterTsa', [], 'Display', None],
  [ 0x1c18, 'LepchaLetterTsha', [], 'Display', None],
  [ 0x1c19, 'LepchaLetterDza', [], 'Display', None],
  [ 0x1c1a, 'LepchaLetterYa', [], 'Display', None],
  [ 0x1c1b, 'LepchaLetterRa', [], 'Display', None],
  [ 0x1c1c, 'LepchaLetterLa', [], 'Display', None],
  [ 0x1c1d, 'LepchaLetterHa', [], 'Display', None],
  [ 0x1c1e, 'LepchaLetterHla', [], 'Display', None],
  [ 0x1c1f, 'LepchaLetterVa', [], 'Display', None],
  [ 0x1c20, 'LepchaLetterSa', [], 'Display', None],
  [ 0x1c21, 'LepchaLetterSha', [], 'Display', None],
  [ 0x1c22, 'LepchaLetterWa', [], 'Display', None],
  [ 0x1c23, 'LepchaLetterA', [], 'Display', None],
  [ 0x1c24, 'LepchaSubjoinedLetterYa', [], 'Display', None],
  [ 0x1c25, 'LepchaSubjoinedLetterRa', [], 'Display', None],
  [ 0x1c26, 'LepchaVowelSignAa', [], 'Display', None],
  [ 0x1c27, 'LepchaVowelSignI', [], 'Display', None],
  [ 0x1c28, 'LepchaVowelSignO', [], 'Display', None],
  [ 0x1c29, 'LepchaVowelSignOo', [], 'Display', None],
  [ 0x1c2a, 'LepchaVowelSignU', [], 'Display', None],
  [ 0x1c2b, 'LepchaVowelSignUu', [], 'Display', None],
  [ 0x1c2c, 'LepchaVowelSignE', [], 'Display', None],
  [ 0x1c2d, 'LepchaConsonantSignK', [], 'Display', None],
  [ 0x1c2e, 'LepchaConsonantSignM', [], 'Display', None],
  [ 0x1c2f, 'LepchaConsonantSignL', [], 'Display', None],
  [ 0x1c30, 'LepchaConsonantSignN', [], 'Display', None],
  [ 0x1c31, 'LepchaConsonantSignP', [], 'Display', None],
  [ 0x1c32, 'LepchaConsonantSignR', [], 'Display', None],
  [ 0x1c33, 'LepchaConsonantSignT', [], 'Display', None],
  [ 0x1c34, 'LepchaConsonantSignNyindo', [], 'Display', None],
  [ 0x1c35, 'LepchaConsonantSignKang', [], 'Display', None],
  [ 0x1c36, 'LepchaSignRan', [], 'Display', None],
  [ 0x1c37, 'LepchaSignNukta', [], 'Display', None],
  [ 0x1c38, 'UnicodeCharacter0x1c38', [], 'Display', None],
  [ 0x1c39, 'UnicodeCharacter0x1c39', [], 'Display', None],
  [ 0x1c3a, 'UnicodeCharacter0x1c3a', [], 'Display', None],
  [ 0x1c3b, 'LepchaPunctuationTarol', [], 'Display', None],
  [ 0x1c3c, 'LepchaPunctuationNyetThyoomTarol', [], 'Display', None],
  [ 0x1c3d, 'LepchaPunctuationCerwa', [], 'Display', None],
  [ 0x1c3e, 'LepchaPunctuationTshookCerwa', [], 'Display', None],
  [ 0x1c3f, 'LepchaPunctuationTshook', [], 'Display', None],
  [ 0x1c40, 'LepchaDigitZero', [], 'Display', None],
  [ 0x1c41, 'LepchaDigitOne', [], 'Display', None],
  [ 0x1c42, 'LepchaDigitTwo', [], 'Display', None],
  [ 0x1c43, 'LepchaDigitThree', [], 'Display', None],
  [ 0x1c44, 'LepchaDigitFour', [], 'Display', None],
  [ 0x1c45, 'LepchaDigitFive', [], 'Display', None],
  [ 0x1c46, 'LepchaDigitSix', [], 'Display', None],
  [ 0x1c47, 'LepchaDigitSeven', [], 'Display', None],
  [ 0x1c48, 'LepchaDigitEight', [], 'Display', None],
  [ 0x1c49, 'LepchaDigitNine', [], 'Display', None],
  [ 0x1c4a, 'UnicodeCharacter0x1c4a', [], 'Display', None],
  [ 0x1c4b, 'UnicodeCharacter0x1c4b', [], 'Display', None],
  [ 0x1c4c, 'UnicodeCharacter0x1c4c', [], 'Display', None],
  [ 0x1c4d, 'LepchaLetterTta', [], 'Display', None],
  [ 0x1c4e, 'LepchaLetterTtha', [], 'Display', None],
  [ 0x1c4f, 'LepchaLetterDda', [], 'Display', None],
  [ 0x1c50, 'OlChikiDigitZero', [], 'Display', None],
  [ 0x1c51, 'OlChikiDigitOne', [], 'Display', None],
  [ 0x1c52, 'OlChikiDigitTwo', [], 'Display', None],
  [ 0x1c53, 'OlChikiDigitThree', [], 'Display', None],
  [ 0x1c54, 'OlChikiDigitFour', [], 'Display', None],
  [ 0x1c55, 'OlChikiDigitFive', [], 'Display', None],
  [ 0x1c56, 'OlChikiDigitSix', [], 'Display', None],
  [ 0x1c57, 'OlChikiDigitSeven', [], 'Display', None],
  [ 0x1c58, 'OlChikiDigitEight', [], 'Display', None],
  [ 0x1c59, 'OlChikiDigitNine', [], 'Display', None],
  [ 0x1c5a, 'OlChikiLetterLa', [], 'Display', None],
  [ 0x1c5b, 'OlChikiLetterAt', [], 'Display', None],
  [ 0x1c5c, 'OlChikiLetterAg', [], 'Display', None],
  [ 0x1c5d, 'OlChikiLetterAng', [], 'Display', None],
  [ 0x1c5e, 'OlChikiLetterAl', [], 'Display', None],
  [ 0x1c5f, 'OlChikiLetterLaa', [], 'Display', None],
  [ 0x1c60, 'OlChikiLetterAak', [], 'Display', None],
  [ 0x1c61, 'OlChikiLetterAaj', [], 'Display', None],
  [ 0x1c62, 'OlChikiLetterAam', [], 'Display', None],
  [ 0x1c63, 'OlChikiLetterAaw', [], 'Display', None],
  [ 0x1c64, 'OlChikiLetterLi', [], 'Display', None],
  [ 0x1c65, 'OlChikiLetterIs', [], 'Display', None],
  [ 0x1c66, 'OlChikiLetterIh', [], 'Display', None],
  [ 0x1c67, 'OlChikiLetterIny', [], 'Display', None],
  [ 0x1c68, 'OlChikiLetterIr', [], 'Display', None],
  [ 0x1c69, 'OlChikiLetterLu', [], 'Display', None],
  [ 0x1c6a, 'OlChikiLetterUc', [], 'Display', None],
  [ 0x1c6b, 'OlChikiLetterUd', [], 'Display', None],
  [ 0x1c6c, 'OlChikiLetterUnn', [], 'Display', None],
  [ 0x1c6d, 'OlChikiLetterUy', [], 'Display', None],
  [ 0x1c6e, 'OlChikiLetterLe', [], 'Display', None],
  [ 0x1c6f, 'OlChikiLetterEp', [], 'Display', None],
  [ 0x1c70, 'OlChikiLetterEdd', [], 'Display', None],
  [ 0x1c71, 'OlChikiLetterEn', [], 'Display', None],
  [ 0x1c72, 'OlChikiLetterErr', [], 'Display', None],
  [ 0x1c73, 'OlChikiLetterLo', [], 'Display', None],
  [ 0x1c74, 'OlChikiLetterOtt', [], 'Display', None],
  [ 0x1c75, 'OlChikiLetterOb', [], 'Display', None],
  [ 0x1c76, 'OlChikiLetterOv', [], 'Display', None],
  [ 0x1c77, 'OlChikiLetterOh', [], 'Display', None],
  [ 0x1c78, 'OlChikiMuTtuddag', [], 'Display', None],
  [ 0x1c79, 'OlChikiGaahlaaTtuddaag', [], 'Display', None],
  [ 0x1c7a, 'OlChikiMugaahlaaTtuddaag', [], 'Display', None],
  [ 0x1c7b, 'OlChikiRelaa', [], 'Display', None],
  [ 0x1c7c, 'OlChikiPhaarkaa', [], 'Display', None],
  [ 0x1c7d, 'OlChikiAhad', [], 'Display', None],
  [ 0x1c7e, 'OlChikiPunctuationMucaad', [], 'Display', None],
  [ 0x1c7f, 'OlChikiPunctuationDoubleMucaad', [], 'Display', None],
  [ 0x1c80, 'UnicodeCharacter0x1c80', [], 'Display', None],
  [ 0x1c81, 'UnicodeCharacter0x1c81', [], 'Display', None],
  [ 0x1c82, 'UnicodeCharacter0x1c82', [], 'Display', None],
  [ 0x1c83, 'UnicodeCharacter0x1c83', [], 'Display', None],
  [ 0x1c84, 'UnicodeCharacter0x1c84', [], 'Display', None],
  [ 0x1c85, 'UnicodeCharacter0x1c85', [], 'Display', None],
  [ 0x1c86, 'UnicodeCharacter0x1c86', [], 'Display', None],
  [ 0x1c87, 'UnicodeCharacter0x1c87', [], 'Display', None],
  [ 0x1c88, 'UnicodeCharacter0x1c88', [], 'Display', None],
  [ 0x1c89, 'UnicodeCharacter0x1c89', [], 'Display', None],
  [ 0x1c8a, 'UnicodeCharacter0x1c8a', [], 'Display', None],
  [ 0x1c8b, 'UnicodeCharacter0x1c8b', [], 'Display', None],
  [ 0x1c8c, 'UnicodeCharacter0x1c8c', [], 'Display', None],
  [ 0x1c8d, 'UnicodeCharacter0x1c8d', [], 'Display', None],
  [ 0x1c8e, 'UnicodeCharacter0x1c8e', [], 'Display', None],
  [ 0x1c8f, 'UnicodeCharacter0x1c8f', [], 'Display', None],
  [ 0x1c90, 'UnicodeCharacter0x1c90', [], 'Display', None],
  [ 0x1c91, 'UnicodeCharacter0x1c91', [], 'Display', None],
  [ 0x1c92, 'UnicodeCharacter0x1c92', [], 'Display', None],
  [ 0x1c93, 'UnicodeCharacter0x1c93', [], 'Display', None],
  [ 0x1c94, 'UnicodeCharacter0x1c94', [], 'Display', None],
  [ 0x1c95, 'UnicodeCharacter0x1c95', [], 'Display', None],
  [ 0x1c96, 'UnicodeCharacter0x1c96', [], 'Display', None],
  [ 0x1c97, 'UnicodeCharacter0x1c97', [], 'Display', None],
  [ 0x1c98, 'UnicodeCharacter0x1c98', [], 'Display', None],
  [ 0x1c99, 'UnicodeCharacter0x1c99', [], 'Display', None],
  [ 0x1c9a, 'UnicodeCharacter0x1c9a', [], 'Display', None],
  [ 0x1c9b, 'UnicodeCharacter0x1c9b', [], 'Display', None],
  [ 0x1c9c, 'UnicodeCharacter0x1c9c', [], 'Display', None],
  [ 0x1c9d, 'UnicodeCharacter0x1c9d', [], 'Display', None],
  [ 0x1c9e, 'UnicodeCharacter0x1c9e', [], 'Display', None],
  [ 0x1c9f, 'UnicodeCharacter0x1c9f', [], 'Display', None],
  [ 0x1ca0, 'UnicodeCharacter0x1ca0', [], 'Display', None],
  [ 0x1ca1, 'UnicodeCharacter0x1ca1', [], 'Display', None],
  [ 0x1ca2, 'UnicodeCharacter0x1ca2', [], 'Display', None],
  [ 0x1ca3, 'UnicodeCharacter0x1ca3', [], 'Display', None],
  [ 0x1ca4, 'UnicodeCharacter0x1ca4', [], 'Display', None],
  [ 0x1ca5, 'UnicodeCharacter0x1ca5', [], 'Display', None],
  [ 0x1ca6, 'UnicodeCharacter0x1ca6', [], 'Display', None],
  [ 0x1ca7, 'UnicodeCharacter0x1ca7', [], 'Display', None],
  [ 0x1ca8, 'UnicodeCharacter0x1ca8', [], 'Display', None],
  [ 0x1ca9, 'UnicodeCharacter0x1ca9', [], 'Display', None],
  [ 0x1caa, 'UnicodeCharacter0x1caa', [], 'Display', None],
  [ 0x1cab, 'UnicodeCharacter0x1cab', [], 'Display', None],
  [ 0x1cac, 'UnicodeCharacter0x1cac', [], 'Display', None],
  [ 0x1cad, 'UnicodeCharacter0x1cad', [], 'Display', None],
  [ 0x1cae, 'UnicodeCharacter0x1cae', [], 'Display', None],
  [ 0x1caf, 'UnicodeCharacter0x1caf', [], 'Display', None],
  [ 0x1cb0, 'UnicodeCharacter0x1cb0', [], 'Display', None],
  [ 0x1cb1, 'UnicodeCharacter0x1cb1', [], 'Display', None],
  [ 0x1cb2, 'UnicodeCharacter0x1cb2', [], 'Display', None],
  [ 0x1cb3, 'UnicodeCharacter0x1cb3', [], 'Display', None],
  [ 0x1cb4, 'UnicodeCharacter0x1cb4', [], 'Display', None],
  [ 0x1cb5, 'UnicodeCharacter0x1cb5', [], 'Display', None],
  [ 0x1cb6, 'UnicodeCharacter0x1cb6', [], 'Display', None],
  [ 0x1cb7, 'UnicodeCharacter0x1cb7', [], 'Display', None],
  [ 0x1cb8, 'UnicodeCharacter0x1cb8', [], 'Display', None],
  [ 0x1cb9, 'UnicodeCharacter0x1cb9', [], 'Display', None],
  [ 0x1cba, 'UnicodeCharacter0x1cba', [], 'Display', None],
  [ 0x1cbb, 'UnicodeCharacter0x1cbb', [], 'Display', None],
  [ 0x1cbc, 'UnicodeCharacter0x1cbc', [], 'Display', None],
  [ 0x1cbd, 'UnicodeCharacter0x1cbd', [], 'Display', None],
  [ 0x1cbe, 'UnicodeCharacter0x1cbe', [], 'Display', None],
  [ 0x1cbf, 'UnicodeCharacter0x1cbf', [], 'Display', None],
  [ 0x1cc0, 'UnicodeCharacter0x1cc0', [], 'Display', None],
  [ 0x1cc1, 'UnicodeCharacter0x1cc1', [], 'Display', None],
  [ 0x1cc2, 'UnicodeCharacter0x1cc2', [], 'Display', None],
  [ 0x1cc3, 'UnicodeCharacter0x1cc3', [], 'Display', None],
  [ 0x1cc4, 'UnicodeCharacter0x1cc4', [], 'Display', None],
  [ 0x1cc5, 'UnicodeCharacter0x1cc5', [], 'Display', None],
  [ 0x1cc6, 'UnicodeCharacter0x1cc6', [], 'Display', None],
  [ 0x1cc7, 'UnicodeCharacter0x1cc7', [], 'Display', None],
  [ 0x1cc8, 'UnicodeCharacter0x1cc8', [], 'Display', None],
  [ 0x1cc9, 'UnicodeCharacter0x1cc9', [], 'Display', None],
  [ 0x1cca, 'UnicodeCharacter0x1cca', [], 'Display', None],
  [ 0x1ccb, 'UnicodeCharacter0x1ccb', [], 'Display', None],
  [ 0x1ccc, 'UnicodeCharacter0x1ccc', [], 'Display', None],
  [ 0x1ccd, 'UnicodeCharacter0x1ccd', [], 'Display', None],
  [ 0x1cce, 'UnicodeCharacter0x1cce', [], 'Display', None],
  [ 0x1ccf, 'UnicodeCharacter0x1ccf', [], 'Display', None],
  [ 0x1cd0, 'UnicodeCharacter0x1cd0', [], 'Display', None],
  [ 0x1cd1, 'UnicodeCharacter0x1cd1', [], 'Display', None],
  [ 0x1cd2, 'UnicodeCharacter0x1cd2', [], 'Display', None],
  [ 0x1cd3, 'UnicodeCharacter0x1cd3', [], 'Display', None],
  [ 0x1cd4, 'UnicodeCharacter0x1cd4', [], 'Display', None],
  [ 0x1cd5, 'UnicodeCharacter0x1cd5', [], 'Display', None],
  [ 0x1cd6, 'UnicodeCharacter0x1cd6', [], 'Display', None],
  [ 0x1cd7, 'UnicodeCharacter0x1cd7', [], 'Display', None],
  [ 0x1cd8, 'UnicodeCharacter0x1cd8', [], 'Display', None],
  [ 0x1cd9, 'UnicodeCharacter0x1cd9', [], 'Display', None],
  [ 0x1cda, 'UnicodeCharacter0x1cda', [], 'Display', None],
  [ 0x1cdb, 'UnicodeCharacter0x1cdb', [], 'Display', None],
  [ 0x1cdc, 'UnicodeCharacter0x1cdc', [], 'Display', None],
  [ 0x1cdd, 'UnicodeCharacter0x1cdd', [], 'Display', None],
  [ 0x1cde, 'UnicodeCharacter0x1cde', [], 'Display', None],
  [ 0x1cdf, 'UnicodeCharacter0x1cdf', [], 'Display', None],
  [ 0x1ce0, 'UnicodeCharacter0x1ce0', [], 'Display', None],
  [ 0x1ce1, 'UnicodeCharacter0x1ce1', [], 'Display', None],
  [ 0x1ce2, 'UnicodeCharacter0x1ce2', [], 'Display', None],
  [ 0x1ce3, 'UnicodeCharacter0x1ce3', [], 'Display', None],
  [ 0x1ce4, 'UnicodeCharacter0x1ce4', [], 'Display', None],
  [ 0x1ce5, 'UnicodeCharacter0x1ce5', [], 'Display', None],
  [ 0x1ce6, 'UnicodeCharacter0x1ce6', [], 'Display', None],
  [ 0x1ce7, 'UnicodeCharacter0x1ce7', [], 'Display', None],
  [ 0x1ce8, 'UnicodeCharacter0x1ce8', [], 'Display', None],
  [ 0x1ce9, 'UnicodeCharacter0x1ce9', [], 'Display', None],
  [ 0x1cea, 'UnicodeCharacter0x1cea', [], 'Display', None],
  [ 0x1ceb, 'UnicodeCharacter0x1ceb', [], 'Display', None],
  [ 0x1cec, 'UnicodeCharacter0x1cec', [], 'Display', None],
  [ 0x1ced, 'UnicodeCharacter0x1ced', [], 'Display', None],
  [ 0x1cee, 'UnicodeCharacter0x1cee', [], 'Display', None],
  [ 0x1cef, 'UnicodeCharacter0x1cef', [], 'Display', None],
  [ 0x1cf0, 'UnicodeCharacter0x1cf0', [], 'Display', None],
  [ 0x1cf1, 'UnicodeCharacter0x1cf1', [], 'Display', None],
  [ 0x1cf2, 'UnicodeCharacter0x1cf2', [], 'Display', None],
  [ 0x1cf3, 'UnicodeCharacter0x1cf3', [], 'Display', None],
  [ 0x1cf4, 'UnicodeCharacter0x1cf4', [], 'Display', None],
  [ 0x1cf5, 'UnicodeCharacter0x1cf5', [], 'Display', None],
  [ 0x1cf6, 'UnicodeCharacter0x1cf6', [], 'Display', None],
  [ 0x1cf7, 'UnicodeCharacter0x1cf7', [], 'Display', None],
  [ 0x1cf8, 'UnicodeCharacter0x1cf8', [], 'Display', None],
  [ 0x1cf9, 'UnicodeCharacter0x1cf9', [], 'Display', None],
  [ 0x1cfa, 'UnicodeCharacter0x1cfa', [], 'Display', None],
  [ 0x1cfb, 'UnicodeCharacter0x1cfb', [], 'Display', None],
  [ 0x1cfc, 'UnicodeCharacter0x1cfc', [], 'Display', None],
  [ 0x1cfd, 'UnicodeCharacter0x1cfd', [], 'Display', None],
  [ 0x1cfe, 'UnicodeCharacter0x1cfe', [], 'Display', None],
  [ 0x1cff, 'UnicodeCharacter0x1cff', [], 'Display', None],
  [ 0x1d00, 'LatinLetterSmallCapitalA', [], 'Display', None],
  [ 0x1d01, 'LatinLetterSmallCapitalAe', [], 'Display', None],
  [ 0x1d02, 'LatinSmallLetterTurnedAe', [], 'Display', None],
  [ 0x1d03, 'LatinLetterSmallCapitalBarredB', [], 'Display', None],
  [ 0x1d04, 'LatinLetterSmallCapitalC', [], 'Display', None],
  [ 0x1d05, 'LatinLetterSmallCapitalD', [], 'Display', None],
  [ 0x1d06, 'LatinLetterSmallCapitalEth', [], 'Display', None],
  [ 0x1d07, 'LatinLetterSmallCapitalE', [], 'Display', None],
  [ 0x1d08, 'LatinSmallLetterTurnedOpenE', [], 'Display', None],
  [ 0x1d09, 'LatinSmallLetterTurnedI', [], 'Display', None],
  [ 0x1d0a, 'LatinLetterSmallCapitalJ', [], 'Display', None],
  [ 0x1d0b, 'LatinLetterSmallCapitalK', [], 'Display', None],
  [ 0x1d0c, 'LatinLetterSmallCapitalLWithStroke', [], 'Display', None],
  [ 0x1d0d, 'LatinLetterSmallCapitalM', [], 'Display', None],
  [ 0x1d0e, 'LatinLetterSmallCapitalReversedN', [], 'Display', None],
  [ 0x1d0f, 'LatinLetterSmallCapitalO', [], 'Display', None],
  [ 0x1d10, 'LatinLetterSmallCapitalOpenO', [], 'Display', None],
  [ 0x1d11, 'LatinSmallLetterSidewaysO', [], 'Display', None],
  [ 0x1d12, 'LatinSmallLetterSidewaysOpenO', [], 'Display', None],
  [ 0x1d13, 'LatinSmallLetterSidewaysOWithStroke', [], 'Display', None],
  [ 0x1d14, 'LatinSmallLetterTurnedOe', [], 'Display', None],
  [ 0x1d15, 'LatinLetterSmallCapitalOu', [], 'Display', None],
  [ 0x1d16, 'LatinSmallLetterTopHalfO', [], 'Display', None],
  [ 0x1d17, 'LatinSmallLetterBottomHalfO', [], 'Display', None],
  [ 0x1d18, 'LatinLetterSmallCapitalP', [], 'Display', None],
  [ 0x1d19, 'LatinLetterSmallCapitalReversedR', [], 'Display', None],
  [ 0x1d1a, 'LatinLetterSmallCapitalTurnedR', [], 'Display', None],
  [ 0x1d1b, 'LatinLetterSmallCapitalT', [], 'Display', None],
  [ 0x1d1c, 'LatinLetterSmallCapitalU', [], 'Display', None],
  [ 0x1d1d, 'LatinSmallLetterSidewaysU', [], 'Display', None],
  [ 0x1d1e, 'LatinSmallLetterSidewaysDiaeresizedU', [], 'Display', None],
  [ 0x1d1f, 'LatinSmallLetterSidewaysTurnedM', [], 'Display', None],
  [ 0x1d20, 'LatinLetterSmallCapitalV', [], 'Display', None],
  [ 0x1d21, 'LatinLetterSmallCapitalW', [], 'Display', None],
  [ 0x1d22, 'LatinLetterSmallCapitalZ', [], 'Display', None],
  [ 0x1d23, 'LatinLetterSmallCapitalEzh', [], 'Display', None],
  [ 0x1d24, 'LatinLetterVoicedLaryngealSpirant', [], 'Display', None],
  [ 0x1d25, 'LatinLetterAin', [], 'Display', None],
  [ 0x1d26, 'GreekLetterSmallCapitalGamma', [], 'Display', None],
  [ 0x1d27, 'GreekLetterSmallCapitalLamda', [], 'Display', None],
  [ 0x1d28, 'GreekLetterSmallCapitalPi', [], 'Display', None],
  [ 0x1d29, 'GreekLetterSmallCapitalRho', [], 'Display', None],
  [ 0x1d2a, 'GreekLetterSmallCapitalPsi', [], 'Display', None],
  [ 0x1d2b, 'CyrillicLetterSmallCapitalEl', [], 'Display', None],
  [ 0x1d2c, 'ModifierLetterCapitalA', [], 'Display', None],
  [ 0x1d2d, 'ModifierLetterCapitalAe', [], 'Display', None],
  [ 0x1d2e, 'ModifierLetterCapitalB', [], 'Display', None],
  [ 0x1d2f, 'ModifierLetterCapitalBarredB', [], 'Display', None],
  [ 0x1d30, 'ModifierLetterCapitalD', [], 'Display', None],
  [ 0x1d31, 'ModifierLetterCapitalE', [], 'Display', None],
  [ 0x1d32, 'ModifierLetterCapitalReversedE', [], 'Display', None],
  [ 0x1d33, 'ModifierLetterCapitalG', [], 'Display', None],
  [ 0x1d34, 'ModifierLetterCapitalH', [], 'Display', None],
  [ 0x1d35, 'ModifierLetterCapitalI', [], 'Display', None],
  [ 0x1d36, 'ModifierLetterCapitalJ', [], 'Display', None],
  [ 0x1d37, 'ModifierLetterCapitalK', [], 'Display', None],
  [ 0x1d38, 'ModifierLetterCapitalL', [], 'Display', None],
  [ 0x1d39, 'ModifierLetterCapitalM', [], 'Display', None],
  [ 0x1d3a, 'ModifierLetterCapitalN', [], 'Display', None],
  [ 0x1d3b, 'ModifierLetterCapitalReversedN', [], 'Display', None],
  [ 0x1d3c, 'ModifierLetterCapitalO', [], 'Display', None],
  [ 0x1d3d, 'ModifierLetterCapitalOu', [], 'Display', None],
  [ 0x1d3e, 'ModifierLetterCapitalP', [], 'Display', None],
  [ 0x1d3f, 'ModifierLetterCapitalR', [], 'Display', None],
  [ 0x1d40, 'ModifierLetterCapitalT', [], 'Display', None],
  [ 0x1d41, 'ModifierLetterCapitalU', [], 'Display', None],
  [ 0x1d42, 'ModifierLetterCapitalW', [], 'Display', None],
  [ 0x1d43, 'ModifierLetterSmallA', [], 'Display', None],
  [ 0x1d44, 'ModifierLetterSmallTurnedA', [], 'Display', None],
  [ 0x1d45, 'ModifierLetterSmallAlpha', [], 'Display', None],
  [ 0x1d46, 'ModifierLetterSmallTurnedAe', [], 'Display', None],
  [ 0x1d47, 'ModifierLetterSmallB', [], 'Display', None],
  [ 0x1d48, 'ModifierLetterSmallD', [], 'Display', None],
  [ 0x1d49, 'ModifierLetterSmallE', [], 'Display', None],
  [ 0x1d4a, 'ModifierLetterSmallSchwa', [], 'Display', None],
  [ 0x1d4b, 'ModifierLetterSmallOpenE', [], 'Display', None],
  [ 0x1d4c, 'ModifierLetterSmallTurnedOpenE', [], 'Display', None],
  [ 0x1d4d, 'ModifierLetterSmallG', [], 'Display', None],
  [ 0x1d4e, 'ModifierLetterSmallTurnedI', [], 'Display', None],
  [ 0x1d4f, 'ModifierLetterSmallK', [], 'Display', None],
  [ 0x1d50, 'ModifierLetterSmallM', [], 'Display', None],
  [ 0x1d51, 'ModifierLetterSmallEng', [], 'Display', None],
  [ 0x1d52, 'ModifierLetterSmallO', [], 'Display', None],
  [ 0x1d53, 'ModifierLetterSmallOpenO', [], 'Display', None],
  [ 0x1d54, 'ModifierLetterSmallTopHalfO', [], 'Display', None],
  [ 0x1d55, 'ModifierLetterSmallBottomHalfO', [], 'Display', None],
  [ 0x1d56, 'ModifierLetterSmallP', [], 'Display', None],
  [ 0x1d57, 'ModifierLetterSmallT', [], 'Display', None],
  [ 0x1d58, 'ModifierLetterSmallU', [], 'Display', None],
  [ 0x1d59, 'ModifierLetterSmallSidewaysU', [], 'Display', None],
  [ 0x1d5a, 'ModifierLetterSmallTurnedM', [], 'Display', None],
  [ 0x1d5b, 'ModifierLetterSmallV', [], 'Display', None],
  [ 0x1d5c, 'ModifierLetterSmallAin', [], 'Display', None],
  [ 0x1d5d, 'ModifierLetterSmallBeta', [], 'Display', None],
  [ 0x1d5e, 'ModifierLetterSmallGreekGamma', [], 'Display', None],
  [ 0x1d5f, 'ModifierLetterSmallDelta', [], 'Display', None],
  [ 0x1d60, 'ModifierLetterSmallGreekPhi', [], 'Display', None],
  [ 0x1d61, 'ModifierLetterSmallChi', [], 'Display', None],
  [ 0x1d62, 'LatinSubscriptSmallLetterI', ['subi','_i'], 'Symbol', None],
  [ 0x1d63, 'LatinSubscriptSmallLetterR', ['subr','_r'], 'Symbol', None],
  [ 0x1d64, 'LatinSubscriptSmallLetterU', ['subu','_u'], 'Symbol', None],
  [ 0x1d65, 'LatinSubscriptSmallLetterV', ['subv','_v'], 'Symbol', None],
  [ 0x1d66, 'GreekSubscriptSmallLetterBeta', ['subbeta','_beta'], 'Symbol', None],
  [ 0x1d67, 'GreekSubscriptSmallLetterGamma', ['subgamma','_gamma'], 'Symbol', None],
  [ 0x1d68, 'GreekSubscriptSmallLetterRho', ['subrho','_rho'], 'Symbol', None],
  [ 0x1d69, 'GreekSubscriptSmallLetterPhi', ['subphi','_phi'], 'Symbol', None],
  [ 0x1d6a, 'GreekSubscriptSmallLetterChi', ['subchi','_chi'], 'Symbol', None],
  [ 0x1d6b, 'LatinSmallLetterUe', [], 'Display', None],
  [ 0x1d6c, 'LatinSmallLetterBWithMiddleTilde', [], 'Display', None],
  [ 0x1d6d, 'LatinSmallLetterDWithMiddleTilde', [], 'Display', None],
  [ 0x1d6e, 'LatinSmallLetterFWithMiddleTilde', [], 'Display', None],
  [ 0x1d6f, 'LatinSmallLetterMWithMiddleTilde', [], 'Display', None],
  [ 0x1d70, 'LatinSmallLetterNWithMiddleTilde', [], 'Display', None],
  [ 0x1d71, 'LatinSmallLetterPWithMiddleTilde', [], 'Display', None],
  [ 0x1d72, 'LatinSmallLetterRWithMiddleTilde', [], 'Display', None],
  [ 0x1d73, 'LatinSmallLetterRWithFishhookAndMiddleTilde', [], 'Display', None],
  [ 0x1d74, 'LatinSmallLetterSWithMiddleTilde', [], 'Display', None],
  [ 0x1d75, 'LatinSmallLetterTWithMiddleTilde', [], 'Display', None],
  [ 0x1d76, 'LatinSmallLetterZWithMiddleTilde', [], 'Display', None],
  [ 0x1d77, 'LatinSmallLetterTurnedG', [], 'Display', None],
  [ 0x1d78, 'ModifierLetterCyrillicEn', [], 'Display', None],
  [ 0x1d79, 'LatinSmallLetterInsularG', [], 'Display', None],
  [ 0x1d7a, 'LatinSmallLetterThWithStrikethrough', [], 'Display', None],
  [ 0x1d7b, 'LatinSmallCapitalLetterIWithStroke', [], 'Display', None],
  [ 0x1d7c, 'LatinSmallLetterIotaWithStroke', [], 'Display', None],
  [ 0x1d7d, 'LatinSmallLetterPWithStroke', [], 'Display', None],
  [ 0x1d7e, 'LatinSmallCapitalLetterUWithStroke', [], 'Display', None],
  [ 0x1d7f, 'LatinSmallLetterUpsilonWithStroke', [], 'Display', None],
  [ 0x1d80, 'LatinSmallLetterBWithPalatalHook', [], 'Display', None],
  [ 0x1d81, 'LatinSmallLetterDWithPalatalHook', [], 'Display', None],
  [ 0x1d82, 'LatinSmallLetterFWithPalatalHook', [], 'Display', None],
  [ 0x1d83, 'LatinSmallLetterGWithPalatalHook', [], 'Display', None],
  [ 0x1d84, 'LatinSmallLetterKWithPalatalHook', [], 'Display', None],
  [ 0x1d85, 'LatinSmallLetterLWithPalatalHook', [], 'Display', None],
  [ 0x1d86, 'LatinSmallLetterMWithPalatalHook', [], 'Display', None],
  [ 0x1d87, 'LatinSmallLetterNWithPalatalHook', [], 'Display', None],
  [ 0x1d88, 'LatinSmallLetterPWithPalatalHook', [], 'Display', None],
  [ 0x1d89, 'LatinSmallLetterRWithPalatalHook', [], 'Display', None],
  [ 0x1d8a, 'LatinSmallLetterSWithPalatalHook', [], 'Display', None],
  [ 0x1d8b, 'LatinSmallLetterEshWithPalatalHook', [], 'Display', None],
  [ 0x1d8c, 'LatinSmallLetterVWithPalatalHook', [], 'Display', None],
  [ 0x1d8d, 'LatinSmallLetterXWithPalatalHook', [], 'Display', None],
  [ 0x1d8e, 'LatinSmallLetterZWithPalatalHook', [], 'Display', None],
  [ 0x1d8f, 'LatinSmallLetterAWithRetroflexHook', [], 'Display', None],
  [ 0x1d90, 'LatinSmallLetterAlphaWithRetroflexHook', [], 'Display', None],
  [ 0x1d91, 'LatinSmallLetterDWithHookAndTail', [], 'Display', None],
  [ 0x1d92, 'LatinSmallLetterEWithRetroflexHook', [], 'Display', None],
  [ 0x1d93, 'LatinSmallLetterOpenEWithRetroflexHook', [], 'Display', None],
  [ 0x1d94, 'LatinSmallLetterReversedOpenEWithRetroflexHook', [], 'Display', None],
  [ 0x1d95, 'LatinSmallLetterSchwaWithRetroflexHook', [], 'Display', None],
  [ 0x1d96, 'LatinSmallLetterIWithRetroflexHook', [], 'Display', None],
  [ 0x1d97, 'LatinSmallLetterOpenOWithRetroflexHook', [], 'Display', None],
  [ 0x1d98, 'LatinSmallLetterEshWithRetroflexHook', [], 'Display', None],
  [ 0x1d99, 'LatinSmallLetterUWithRetroflexHook', [], 'Display', None],
  [ 0x1d9a, 'LatinSmallLetterEzhWithRetroflexHook', [], 'Display', None],
  [ 0x1d9b, 'ModifierLetterSmallTurnedAlpha', [], 'Display', None],
  [ 0x1d9c, 'ModifierLetterSmallC', [], 'Display', None],
  [ 0x1d9d, 'ModifierLetterSmallCWithCurl', [], 'Display', None],
  [ 0x1d9e, 'ModifierLetterSmallEth', [], 'Display', None],
  [ 0x1d9f, 'ModifierLetterSmallReversedOpenE', [], 'Display', None],
  [ 0x1da0, 'ModifierLetterSmallF', [], 'Display', None],
  [ 0x1da1, 'ModifierLetterSmallDotlessJWithStroke', [], 'Display', None],
  [ 0x1da2, 'ModifierLetterSmallScriptG', [], 'Display', None],
  [ 0x1da3, 'ModifierLetterSmallTurnedH', [], 'Display', None],
  [ 0x1da4, 'ModifierLetterSmallIWithStroke', [], 'Display', None],
  [ 0x1da5, 'ModifierLetterSmallIota', [], 'Display', None],
  [ 0x1da6, 'ModifierLetterSmallCapitalI', [], 'Display', None],
  [ 0x1da7, 'ModifierLetterSmallCapitalIWithStroke', [], 'Display', None],
  [ 0x1da8, 'ModifierLetterSmallJWithCrossedtail', [], 'Display', None],
  [ 0x1da9, 'ModifierLetterSmallLWithRetroflexHook', [], 'Display', None],
  [ 0x1daa, 'ModifierLetterSmallLWithPalatalHook', [], 'Display', None],
  [ 0x1dab, 'ModifierLetterSmallCapitalL', [], 'Display', None],
  [ 0x1dac, 'ModifierLetterSmallMWithHook', [], 'Display', None],
  [ 0x1dad, 'ModifierLetterSmallTurnedMWithLongLeg', [], 'Display', None],
  [ 0x1dae, 'ModifierLetterSmallNWithLeftHook', [], 'Display', None],
  [ 0x1daf, 'ModifierLetterSmallNWithRetroflexHook', [], 'Display', None],
  [ 0x1db0, 'ModifierLetterSmallCapitalN', [], 'Display', None],
  [ 0x1db1, 'ModifierLetterSmallBarredO', [], 'Display', None],
  [ 0x1db2, 'ModifierLetterSmallPhi', [], 'Display', None],
  [ 0x1db3, 'ModifierLetterSmallSWithHook', [], 'Display', None],
  [ 0x1db4, 'ModifierLetterSmallEsh', [], 'Display', None],
  [ 0x1db5, 'ModifierLetterSmallTWithPalatalHook', [], 'Display', None],
  [ 0x1db6, 'ModifierLetterSmallUBar', [], 'Display', None],
  [ 0x1db7, 'ModifierLetterSmallUpsilon', [], 'Display', None],
  [ 0x1db8, 'ModifierLetterSmallCapitalU', [], 'Display', None],
  [ 0x1db9, 'ModifierLetterSmallVWithHook', [], 'Display', None],
  [ 0x1dba, 'ModifierLetterSmallTurnedV', [], 'Display', None],
  [ 0x1dbb, 'ModifierLetterSmallZ', [], 'Display', None],
  [ 0x1dbc, 'ModifierLetterSmallZWithRetroflexHook', [], 'Display', None],
  [ 0x1dbd, 'ModifierLetterSmallZWithCurl', [], 'Display', None],
  [ 0x1dbe, 'ModifierLetterSmallEzh', [], 'Display', None],
  [ 0x1dbf, 'ModifierLetterSmallTheta', [], 'Display', None],
  [ 0x1dc0, 'CombiningDottedGraveAccent', [], 'Display', None],
  [ 0x1dc1, 'CombiningDottedAcuteAccent', [], 'Display', None],
  [ 0x1dc2, 'CombiningSnakeBelow', [], 'Display', None],
  [ 0x1dc3, 'CombiningSuspensionMark', [], 'Display', None],
  [ 0x1dc4, 'CombiningMacronacute', [], 'Display', None],
  [ 0x1dc5, 'CombiningGravemacron', [], 'Display', None],
  [ 0x1dc6, 'CombiningMacrongrave', [], 'Display', None],
  [ 0x1dc7, 'CombiningAcutemacron', [], 'Display', None],
  [ 0x1dc8, 'CombiningGraveacutegrave', [], 'Display', None],
  [ 0x1dc9, 'CombiningAcutegraveacute', [], 'Display', None],
  [ 0x1dca, 'CombiningLatinSmallLetterRBelow', [], 'Display', None],
  [ 0x1dcb, 'CombiningBrevemacron', [], 'Display', None],
  [ 0x1dcc, 'CombiningMacronbreve', [], 'Display', None],
  [ 0x1dcd, 'CombiningDoubleCircumflexAbove', [], 'Display', None],
  [ 0x1dce, 'CombiningOgonekAbove', [], 'Display', None],
  [ 0x1dcf, 'CombiningZigzagBelow', [], 'Display', None],
  [ 0x1dd0, 'CombiningIsBelow', [], 'Display', None],
  [ 0x1dd1, 'CombiningUrAbove', [], 'Display', None],
  [ 0x1dd2, 'CombiningUsAbove', [], 'Display', None],
  [ 0x1dd3, 'CombiningLatinSmallLetterFlattenedOpenAAbove', [], 'Display', None],
  [ 0x1dd4, 'CombiningLatinSmallLetterAe', [], 'Display', None],
  [ 0x1dd5, 'CombiningLatinSmallLetterAo', [], 'Display', None],
  [ 0x1dd6, 'CombiningLatinSmallLetterAv', [], 'Display', None],
  [ 0x1dd7, 'CombiningLatinSmallLetterCCedilla', [], 'Display', None],
  [ 0x1dd8, 'CombiningLatinSmallLetterInsularD', [], 'Display', None],
  [ 0x1dd9, 'CombiningLatinSmallLetterEth', [], 'Display', None],
  [ 0x1dda, 'CombiningLatinSmallLetterG', [], 'Display', None],
  [ 0x1ddb, 'CombiningLatinLetterSmallCapitalG', [], 'Display', None],
  [ 0x1ddc, 'CombiningLatinSmallLetterK', [], 'Display', None],
  [ 0x1ddd, 'CombiningLatinSmallLetterL', [], 'Display', None],
  [ 0x1dde, 'CombiningLatinLetterSmallCapitalL', [], 'Display', None],
  [ 0x1ddf, 'CombiningLatinLetterSmallCapitalM', [], 'Display', None],
  [ 0x1de0, 'CombiningLatinSmallLetterN', [], 'Display', None],
  [ 0x1de1, 'CombiningLatinLetterSmallCapitalN', [], 'Display', None],
  [ 0x1de2, 'CombiningLatinLetterSmallCapitalR', [], 'Display', None],
  [ 0x1de3, 'CombiningLatinSmallLetterRRotunda', [], 'Display', None],
  [ 0x1de4, 'CombiningLatinSmallLetterS', [], 'Display', None],
  [ 0x1de5, 'CombiningLatinSmallLetterLongS', [], 'Display', None],
  [ 0x1de6, 'CombiningLatinSmallLetterZ', [], 'Display', None],
  [ 0x1de7, 'UnicodeCharacter0x1de7', [], 'Display', None],
  [ 0x1de8, 'UnicodeCharacter0x1de8', [], 'Display', None],
  [ 0x1de9, 'UnicodeCharacter0x1de9', [], 'Display', None],
  [ 0x1dea, 'UnicodeCharacter0x1dea', [], 'Display', None],
  [ 0x1deb, 'UnicodeCharacter0x1deb', [], 'Display', None],
  [ 0x1dec, 'UnicodeCharacter0x1dec', [], 'Display', None],
  [ 0x1ded, 'UnicodeCharacter0x1ded', [], 'Display', None],
  [ 0x1dee, 'UnicodeCharacter0x1dee', [], 'Display', None],
  [ 0x1def, 'UnicodeCharacter0x1def', [], 'Display', None],
  [ 0x1df0, 'UnicodeCharacter0x1df0', [], 'Display', None],
  [ 0x1df1, 'UnicodeCharacter0x1df1', [], 'Display', None],
  [ 0x1df2, 'UnicodeCharacter0x1df2', [], 'Display', None],
  [ 0x1df3, 'UnicodeCharacter0x1df3', [], 'Display', None],
  [ 0x1df4, 'UnicodeCharacter0x1df4', [], 'Display', None],
  [ 0x1df5, 'UnicodeCharacter0x1df5', [], 'Display', None],
  [ 0x1df6, 'UnicodeCharacter0x1df6', [], 'Display', None],
  [ 0x1df7, 'UnicodeCharacter0x1df7', [], 'Display', None],
  [ 0x1df8, 'UnicodeCharacter0x1df8', [], 'Display', None],
  [ 0x1df9, 'UnicodeCharacter0x1df9', [], 'Display', None],
  [ 0x1dfa, 'UnicodeCharacter0x1dfa', [], 'Display', None],
  [ 0x1dfb, 'UnicodeCharacter0x1dfb', [], 'Display', None],
  [ 0x1dfc, 'UnicodeCharacter0x1dfc', [], 'Display', None],
  [ 0x1dfd, 'UnicodeCharacter0x1dfd', [], 'Display', None],
  [ 0x1dfe, 'CombiningLeftArrowheadAbove', [], 'Display', None],
  [ 0x1dff, 'CombiningRightArrowheadAndDownArrowheadBelow', [], 'Display', None],
  [ 0x1e00, 'LatinCapitalLetterAWithRingBelow', [], 'Display', None],
  [ 0x1e01, 'LatinSmallLetterAWithRingBelow', [], 'Display', None],
  [ 0x1e02, 'LatinCapitalLetterBWithDotAbove', [], 'Display', None],
  [ 0x1e03, 'LatinSmallLetterBWithDotAbove', [], 'Display', None],
  [ 0x1e04, 'LatinCapitalLetterBWithDotBelow', [], 'Display', None],
  [ 0x1e05, 'LatinSmallLetterBWithDotBelow', [], 'Display', None],
  [ 0x1e06, 'LatinCapitalLetterBWithLineBelow', [], 'Display', None],
  [ 0x1e07, 'LatinSmallLetterBWithLineBelow', [], 'Display', None],
  [ 0x1e08, 'LatinCapitalLetterCWithCedillaAndAcute', [], 'Display', None],
  [ 0x1e09, 'LatinSmallLetterCWithCedillaAndAcute', [], 'Display', None],
  [ 0x1e0a, 'LatinCapitalLetterDWithDotAbove', [], 'Display', None],
  [ 0x1e0b, 'LatinSmallLetterDWithDotAbove', [], 'Display', None],
  [ 0x1e0c, 'LatinCapitalLetterDWithDotBelow', [], 'Display', None],
  [ 0x1e0d, 'LatinSmallLetterDWithDotBelow', [], 'Display', None],
  [ 0x1e0e, 'LatinCapitalLetterDWithLineBelow', [], 'Display', None],
  [ 0x1e0f, 'LatinSmallLetterDWithLineBelow', [], 'Display', None],
  [ 0x1e10, 'LatinCapitalLetterDWithCedilla', [], 'Display', None],
  [ 0x1e11, 'LatinSmallLetterDWithCedilla', [], 'Display', None],
  [ 0x1e12, 'LatinCapitalLetterDWithCircumflexBelow', [], 'Display', None],
  [ 0x1e13, 'LatinSmallLetterDWithCircumflexBelow', [], 'Display', None],
  [ 0x1e14, 'LatinCapitalLetterEWithMacronAndGrave', [], 'Display', None],
  [ 0x1e15, 'LatinSmallLetterEWithMacronAndGrave', [], 'Display', None],
  [ 0x1e16, 'LatinCapitalLetterEWithMacronAndAcute', [], 'Display', None],
  [ 0x1e17, 'LatinSmallLetterEWithMacronAndAcute', [], 'Display', None],
  [ 0x1e18, 'LatinCapitalLetterEWithCircumflexBelow', [], 'Display', None],
  [ 0x1e19, 'LatinSmallLetterEWithCircumflexBelow', [], 'Display', None],
  [ 0x1e1a, 'LatinCapitalLetterEWithTildeBelow', [], 'Display', None],
  [ 0x1e1b, 'LatinSmallLetterEWithTildeBelow', [], 'Display', None],
  [ 0x1e1c, 'LatinCapitalLetterEWithCedillaAndBreve', [], 'Display', None],
  [ 0x1e1d, 'LatinSmallLetterEWithCedillaAndBreve', [], 'Display', None],
  [ 0x1e1e, 'LatinCapitalLetterFWithDotAbove', [], 'Display', None],
  [ 0x1e1f, 'LatinSmallLetterFWithDotAbove', [], 'Display', None],
  [ 0x1e20, 'LatinCapitalLetterGWithMacron', [], 'Display', None],
  [ 0x1e21, 'LatinSmallLetterGWithMacron', [], 'Display', None],
  [ 0x1e22, 'LatinCapitalLetterHWithDotAbove', [], 'Display', None],
  [ 0x1e23, 'LatinSmallLetterHWithDotAbove', [], 'Display', None],
  [ 0x1e24, 'LatinCapitalLetterHWithDotBelow', [], 'Display', None],
  [ 0x1e25, 'LatinSmallLetterHWithDotBelow', [], 'Display', None],
  [ 0x1e26, 'LatinCapitalLetterHWithDiaeresis', [], 'Display', None],
  [ 0x1e27, 'LatinSmallLetterHWithDiaeresis', [], 'Display', None],
  [ 0x1e28, 'LatinCapitalLetterHWithCedilla', [], 'Display', None],
  [ 0x1e29, 'LatinSmallLetterHWithCedilla', [], 'Display', None],
  [ 0x1e2a, 'LatinCapitalLetterHWithBreveBelow', [], 'Display', None],
  [ 0x1e2b, 'LatinSmallLetterHWithBreveBelow', [], 'Display', None],
  [ 0x1e2c, 'LatinCapitalLetterIWithTildeBelow', [], 'Display', None],
  [ 0x1e2d, 'LatinSmallLetterIWithTildeBelow', [], 'Display', None],
  [ 0x1e2e, 'LatinCapitalLetterIWithDiaeresisAndAcute', [], 'Display', None],
  [ 0x1e2f, 'LatinSmallLetterIWithDiaeresisAndAcute', [], 'Display', None],
  [ 0x1e30, 'LatinCapitalLetterKWithAcute', [], 'Display', None],
  [ 0x1e31, 'LatinSmallLetterKWithAcute', [], 'Display', None],
  [ 0x1e32, 'LatinCapitalLetterKWithDotBelow', [], 'Display', None],
  [ 0x1e33, 'LatinSmallLetterKWithDotBelow', [], 'Display', None],
  [ 0x1e34, 'LatinCapitalLetterKWithLineBelow', [], 'Display', None],
  [ 0x1e35, 'LatinSmallLetterKWithLineBelow', [], 'Display', None],
  [ 0x1e36, 'LatinCapitalLetterLWithDotBelow', [], 'Display', None],
  [ 0x1e37, 'LatinSmallLetterLWithDotBelow', [], 'Display', None],
  [ 0x1e38, 'LatinCapitalLetterLWithDotBelowAndMacron', [], 'Display', None],
  [ 0x1e39, 'LatinSmallLetterLWithDotBelowAndMacron', [], 'Display', None],
  [ 0x1e3a, 'LatinCapitalLetterLWithLineBelow', [], 'Display', None],
  [ 0x1e3b, 'LatinSmallLetterLWithLineBelow', [], 'Display', None],
  [ 0x1e3c, 'LatinCapitalLetterLWithCircumflexBelow', [], 'Display', None],
  [ 0x1e3d, 'LatinSmallLetterLWithCircumflexBelow', [], 'Display', None],
  [ 0x1e3e, 'LatinCapitalLetterMWithAcute', [], 'Display', None],
  [ 0x1e3f, 'LatinSmallLetterMWithAcute', [], 'Display', None],
  [ 0x1e40, 'LatinCapitalLetterMWithDotAbove', [], 'Display', None],
  [ 0x1e41, 'LatinSmallLetterMWithDotAbove', [], 'Display', None],
  [ 0x1e42, 'LatinCapitalLetterMWithDotBelow', [], 'Display', None],
  [ 0x1e43, 'LatinSmallLetterMWithDotBelow', [], 'Display', None],
  [ 0x1e44, 'LatinCapitalLetterNWithDotAbove', [], 'Display', None],
  [ 0x1e45, 'LatinSmallLetterNWithDotAbove', [], 'Display', None],
  [ 0x1e46, 'LatinCapitalLetterNWithDotBelow', [], 'Display', None],
  [ 0x1e47, 'LatinSmallLetterNWithDotBelow', [], 'Display', None],
  [ 0x1e48, 'LatinCapitalLetterNWithLineBelow', [], 'Display', None],
  [ 0x1e49, 'LatinSmallLetterNWithLineBelow', [], 'Display', None],
  [ 0x1e4a, 'LatinCapitalLetterNWithCircumflexBelow', [], 'Display', None],
  [ 0x1e4b, 'LatinSmallLetterNWithCircumflexBelow', [], 'Display', None],
  [ 0x1e4c, 'LatinCapitalLetterOWithTildeAndAcute', [], 'Display', None],
  [ 0x1e4d, 'LatinSmallLetterOWithTildeAndAcute', [], 'Display', None],
  [ 0x1e4e, 'LatinCapitalLetterOWithTildeAndDiaeresis', [], 'Display', None],
  [ 0x1e4f, 'LatinSmallLetterOWithTildeAndDiaeresis', [], 'Display', None],
  [ 0x1e50, 'LatinCapitalLetterOWithMacronAndGrave', [], 'Display', None],
  [ 0x1e51, 'LatinSmallLetterOWithMacronAndGrave', [], 'Display', None],
  [ 0x1e52, 'LatinCapitalLetterOWithMacronAndAcute', [], 'Display', None],
  [ 0x1e53, 'LatinSmallLetterOWithMacronAndAcute', [], 'Display', None],
  [ 0x1e54, 'LatinCapitalLetterPWithAcute', [], 'Display', None],
  [ 0x1e55, 'LatinSmallLetterPWithAcute', [], 'Display', None],
  [ 0x1e56, 'LatinCapitalLetterPWithDotAbove', [], 'Display', None],
  [ 0x1e57, 'LatinSmallLetterPWithDotAbove', [], 'Display', None],
  [ 0x1e58, 'LatinCapitalLetterRWithDotAbove', [], 'Display', None],
  [ 0x1e59, 'LatinSmallLetterRWithDotAbove', [], 'Display', None],
  [ 0x1e5a, 'LatinCapitalLetterRWithDotBelow', [], 'Display', None],
  [ 0x1e5b, 'LatinSmallLetterRWithDotBelow', [], 'Display', None],
  [ 0x1e5c, 'LatinCapitalLetterRWithDotBelowAndMacron', [], 'Display', None],
  [ 0x1e5d, 'LatinSmallLetterRWithDotBelowAndMacron', [], 'Display', None],
  [ 0x1e5e, 'LatinCapitalLetterRWithLineBelow', [], 'Display', None],
  [ 0x1e5f, 'LatinSmallLetterRWithLineBelow', [], 'Display', None],
  [ 0x1e60, 'LatinCapitalLetterSWithDotAbove', [], 'Display', None],
  [ 0x1e61, 'LatinSmallLetterSWithDotAbove', [], 'Display', None],
  [ 0x1e62, 'LatinCapitalLetterSWithDotBelow', [], 'Display', None],
  [ 0x1e63, 'LatinSmallLetterSWithDotBelow', [], 'Display', None],
  [ 0x1e64, 'LatinCapitalLetterSWithAcuteAndDotAbove', [], 'Display', None],
  [ 0x1e65, 'LatinSmallLetterSWithAcuteAndDotAbove', [], 'Display', None],
  [ 0x1e66, 'LatinCapitalLetterSWithCaronAndDotAbove', [], 'Display', None],
  [ 0x1e67, 'LatinSmallLetterSWithCaronAndDotAbove', [], 'Display', None],
  [ 0x1e68, 'LatinCapitalLetterSWithDotBelowAndDotAbove', [], 'Display', None],
  [ 0x1e69, 'LatinSmallLetterSWithDotBelowAndDotAbove', [], 'Display', None],
  [ 0x1e6a, 'LatinCapitalLetterTWithDotAbove', [], 'Display', None],
  [ 0x1e6b, 'LatinSmallLetterTWithDotAbove', [], 'Display', None],
  [ 0x1e6c, 'LatinCapitalLetterTWithDotBelow', [], 'Display', None],
  [ 0x1e6d, 'LatinSmallLetterTWithDotBelow', [], 'Display', None],
  [ 0x1e6e, 'LatinCapitalLetterTWithLineBelow', [], 'Display', None],
  [ 0x1e6f, 'LatinSmallLetterTWithLineBelow', [], 'Display', None],
  [ 0x1e70, 'LatinCapitalLetterTWithCircumflexBelow', [], 'Display', None],
  [ 0x1e71, 'LatinSmallLetterTWithCircumflexBelow', [], 'Display', None],
  [ 0x1e72, 'LatinCapitalLetterUWithDiaeresisBelow', [], 'Display', None],
  [ 0x1e73, 'LatinSmallLetterUWithDiaeresisBelow', [], 'Display', None],
  [ 0x1e74, 'LatinCapitalLetterUWithTildeBelow', [], 'Display', None],
  [ 0x1e75, 'LatinSmallLetterUWithTildeBelow', [], 'Display', None],
  [ 0x1e76, 'LatinCapitalLetterUWithCircumflexBelow', [], 'Display', None],
  [ 0x1e77, 'LatinSmallLetterUWithCircumflexBelow', [], 'Display', None],
  [ 0x1e78, 'LatinCapitalLetterUWithTildeAndAcute', [], 'Display', None],
  [ 0x1e79, 'LatinSmallLetterUWithTildeAndAcute', [], 'Display', None],
  [ 0x1e7a, 'LatinCapitalLetterUWithMacronAndDiaeresis', [], 'Display', None],
  [ 0x1e7b, 'LatinSmallLetterUWithMacronAndDiaeresis', [], 'Display', None],
  [ 0x1e7c, 'LatinCapitalLetterVWithTilde', [], 'Display', None],
  [ 0x1e7d, 'LatinSmallLetterVWithTilde', [], 'Display', None],
  [ 0x1e7e, 'LatinCapitalLetterVWithDotBelow', [], 'Display', None],
  [ 0x1e7f, 'LatinSmallLetterVWithDotBelow', [], 'Display', None],
  [ 0x1e80, 'LatinCapitalLetterWWithGrave', [], 'Display', None],
  [ 0x1e81, 'LatinSmallLetterWWithGrave', [], 'Display', None],
  [ 0x1e82, 'LatinCapitalLetterWWithAcute', [], 'Display', None],
  [ 0x1e83, 'LatinSmallLetterWWithAcute', [], 'Display', None],
  [ 0x1e84, 'LatinCapitalLetterWWithDiaeresis', [], 'Display', None],
  [ 0x1e85, 'LatinSmallLetterWWithDiaeresis', [], 'Display', None],
  [ 0x1e86, 'LatinCapitalLetterWWithDotAbove', [], 'Display', None],
  [ 0x1e87, 'LatinSmallLetterWWithDotAbove', [], 'Display', None],
  [ 0x1e88, 'LatinCapitalLetterWWithDotBelow', [], 'Display', None],
  [ 0x1e89, 'LatinSmallLetterWWithDotBelow', [], 'Display', None],
  [ 0x1e8a, 'LatinCapitalLetterXWithDotAbove', [], 'Display', None],
  [ 0x1e8b, 'LatinSmallLetterXWithDotAbove', [], 'Display', None],
  [ 0x1e8c, 'LatinCapitalLetterXWithDiaeresis', [], 'Display', None],
  [ 0x1e8d, 'LatinSmallLetterXWithDiaeresis', [], 'Display', None],
  [ 0x1e8e, 'LatinCapitalLetterYWithDotAbove', [], 'Display', None],
  [ 0x1e8f, 'LatinSmallLetterYWithDotAbove', [], 'Display', None],
  [ 0x1e90, 'LatinCapitalLetterZWithCircumflex', [], 'Display', None],
  [ 0x1e91, 'LatinSmallLetterZWithCircumflex', [], 'Display', None],
  [ 0x1e92, 'LatinCapitalLetterZWithDotBelow', [], 'Display', None],
  [ 0x1e93, 'LatinSmallLetterZWithDotBelow', [], 'Display', None],
  [ 0x1e94, 'LatinCapitalLetterZWithLineBelow', [], 'Display', None],
  [ 0x1e95, 'LatinSmallLetterZWithLineBelow', [], 'Display', None],
  [ 0x1e96, 'LatinSmallLetterHWithLineBelow', [], 'Display', None],
  [ 0x1e97, 'LatinSmallLetterTWithDiaeresis', [], 'Display', None],
  [ 0x1e98, 'LatinSmallLetterWWithRingAbove', [], 'Display', None],
  [ 0x1e99, 'LatinSmallLetterYWithRingAbove', [], 'Display', None],
  [ 0x1e9a, 'LatinSmallLetterAWithRightHalfRing', [], 'Display', None],
  [ 0x1e9b, 'LatinSmallLetterLongSWithDotAbove', [], 'Display', None],
  [ 0x1e9c, 'LatinSmallLetterLongSWithDiagonalStroke', [], 'Display', None],
  [ 0x1e9d, 'LatinSmallLetterLongSWithHighStroke', [], 'Display', None],
  [ 0x1e9e, 'LatinCapitalLetterSharpS', [], 'Display', None],
  [ 0x1e9f, 'LatinSmallLetterDelta', [], 'Display', None],
  [ 0x1ea0, 'LatinCapitalLetterAWithDotBelow', [], 'Display', None],
  [ 0x1ea1, 'LatinSmallLetterAWithDotBelow', [], 'Display', None],
  [ 0x1ea2, 'LatinCapitalLetterAWithHookAbove', [], 'Display', None],
  [ 0x1ea3, 'LatinSmallLetterAWithHookAbove', [], 'Display', None],
  [ 0x1ea4, 'LatinCapitalLetterAWithCircumflexAndAcute', [], 'Display', None],
  [ 0x1ea5, 'LatinSmallLetterAWithCircumflexAndAcute', [], 'Display', None],
  [ 0x1ea6, 'LatinCapitalLetterAWithCircumflexAndGrave', [], 'Display', None],
  [ 0x1ea7, 'LatinSmallLetterAWithCircumflexAndGrave', [], 'Display', None],
  [ 0x1ea8, 'LatinCapitalLetterAWithCircumflexAndHookAbove', [], 'Display', None],
  [ 0x1ea9, 'LatinSmallLetterAWithCircumflexAndHookAbove', [], 'Display', None],
  [ 0x1eaa, 'LatinCapitalLetterAWithCircumflexAndTilde', [], 'Display', None],
  [ 0x1eab, 'LatinSmallLetterAWithCircumflexAndTilde', [], 'Display', None],
  [ 0x1eac, 'LatinCapitalLetterAWithCircumflexAndDotBelow', [], 'Display', None],
  [ 0x1ead, 'LatinSmallLetterAWithCircumflexAndDotBelow', [], 'Display', None],
  [ 0x1eae, 'LatinCapitalLetterAWithBreveAndAcute', [], 'Display', None],
  [ 0x1eaf, 'LatinSmallLetterAWithBreveAndAcute', [], 'Display', None],
  [ 0x1eb0, 'LatinCapitalLetterAWithBreveAndGrave', [], 'Display', None],
  [ 0x1eb1, 'LatinSmallLetterAWithBreveAndGrave', [], 'Display', None],
  [ 0x1eb2, 'LatinCapitalLetterAWithBreveAndHookAbove', [], 'Display', None],
  [ 0x1eb3, 'LatinSmallLetterAWithBreveAndHookAbove', [], 'Display', None],
  [ 0x1eb4, 'LatinCapitalLetterAWithBreveAndTilde', [], 'Display', None],
  [ 0x1eb5, 'LatinSmallLetterAWithBreveAndTilde', [], 'Display', None],
  [ 0x1eb6, 'LatinCapitalLetterAWithBreveAndDotBelow', [], 'Display', None],
  [ 0x1eb7, 'LatinSmallLetterAWithBreveAndDotBelow', [], 'Display', None],
  [ 0x1eb8, 'LatinCapitalLetterEWithDotBelow', [], 'Display', None],
  [ 0x1eb9, 'LatinSmallLetterEWithDotBelow', [], 'Display', None],
  [ 0x1eba, 'LatinCapitalLetterEWithHookAbove', [], 'Display', None],
  [ 0x1ebb, 'LatinSmallLetterEWithHookAbove', [], 'Display', None],
  [ 0x1ebc, 'LatinCapitalLetterEWithTilde', [], 'Display', None],
  [ 0x1ebd, 'LatinSmallLetterEWithTilde', [], 'Display', None],
  [ 0x1ebe, 'LatinCapitalLetterEWithCircumflexAndAcute', [], 'Display', None],
  [ 0x1ebf, 'LatinSmallLetterEWithCircumflexAndAcute', [], 'Display', None],
  [ 0x1ec0, 'LatinCapitalLetterEWithCircumflexAndGrave', [], 'Display', None],
  [ 0x1ec1, 'LatinSmallLetterEWithCircumflexAndGrave', [], 'Display', None],
  [ 0x1ec2, 'LatinCapitalLetterEWithCircumflexAndHookAbove', [], 'Display', None],
  [ 0x1ec3, 'LatinSmallLetterEWithCircumflexAndHookAbove', [], 'Display', None],
  [ 0x1ec4, 'LatinCapitalLetterEWithCircumflexAndTilde', [], 'Display', None],
  [ 0x1ec5, 'LatinSmallLetterEWithCircumflexAndTilde', [], 'Display', None],
  [ 0x1ec6, 'LatinCapitalLetterEWithCircumflexAndDotBelow', [], 'Display', None],
  [ 0x1ec7, 'LatinSmallLetterEWithCircumflexAndDotBelow', [], 'Display', None],
  [ 0x1ec8, 'LatinCapitalLetterIWithHookAbove', [], 'Display', None],
  [ 0x1ec9, 'LatinSmallLetterIWithHookAbove', [], 'Display', None],
  [ 0x1eca, 'LatinCapitalLetterIWithDotBelow', [], 'Display', None],
  [ 0x1ecb, 'LatinSmallLetterIWithDotBelow', [], 'Display', None],
  [ 0x1ecc, 'LatinCapitalLetterOWithDotBelow', [], 'Display', None],
  [ 0x1ecd, 'LatinSmallLetterOWithDotBelow', [], 'Display', None],
  [ 0x1ece, 'LatinCapitalLetterOWithHookAbove', [], 'Display', None],
  [ 0x1ecf, 'LatinSmallLetterOWithHookAbove', [], 'Display', None],
  [ 0x1ed0, 'LatinCapitalLetterOWithCircumflexAndAcute', [], 'Display', None],
  [ 0x1ed1, 'LatinSmallLetterOWithCircumflexAndAcute', [], 'Display', None],
  [ 0x1ed2, 'LatinCapitalLetterOWithCircumflexAndGrave', [], 'Display', None],
  [ 0x1ed3, 'LatinSmallLetterOWithCircumflexAndGrave', [], 'Display', None],
  [ 0x1ed4, 'LatinCapitalLetterOWithCircumflexAndHookAbove', [], 'Display', None],
  [ 0x1ed5, 'LatinSmallLetterOWithCircumflexAndHookAbove', [], 'Display', None],
  [ 0x1ed6, 'LatinCapitalLetterOWithCircumflexAndTilde', [], 'Display', None],
  [ 0x1ed7, 'LatinSmallLetterOWithCircumflexAndTilde', [], 'Display', None],
  [ 0x1ed8, 'LatinCapitalLetterOWithCircumflexAndDotBelow', [], 'Display', None],
  [ 0x1ed9, 'LatinSmallLetterOWithCircumflexAndDotBelow', [], 'Display', None],
  [ 0x1eda, 'LatinCapitalLetterOWithHornAndAcute', [], 'Display', None],
  [ 0x1edb, 'LatinSmallLetterOWithHornAndAcute', [], 'Display', None],
  [ 0x1edc, 'LatinCapitalLetterOWithHornAndGrave', [], 'Display', None],
  [ 0x1edd, 'LatinSmallLetterOWithHornAndGrave', [], 'Display', None],
  [ 0x1ede, 'LatinCapitalLetterOWithHornAndHookAbove', [], 'Display', None],
  [ 0x1edf, 'LatinSmallLetterOWithHornAndHookAbove', [], 'Display', None],
  [ 0x1ee0, 'LatinCapitalLetterOWithHornAndTilde', [], 'Display', None],
  [ 0x1ee1, 'LatinSmallLetterOWithHornAndTilde', [], 'Display', None],
  [ 0x1ee2, 'LatinCapitalLetterOWithHornAndDotBelow', [], 'Display', None],
  [ 0x1ee3, 'LatinSmallLetterOWithHornAndDotBelow', [], 'Display', None],
  [ 0x1ee4, 'LatinCapitalLetterUWithDotBelow', [], 'Display', None],
  [ 0x1ee5, 'LatinSmallLetterUWithDotBelow', [], 'Display', None],
  [ 0x1ee6, 'LatinCapitalLetterUWithHookAbove', [], 'Display', None],
  [ 0x1ee7, 'LatinSmallLetterUWithHookAbove', [], 'Display', None],
  [ 0x1ee8, 'LatinCapitalLetterUWithHornAndAcute', [], 'Display', None],
  [ 0x1ee9, 'LatinSmallLetterUWithHornAndAcute', [], 'Display', None],
  [ 0x1eea, 'LatinCapitalLetterUWithHornAndGrave', [], 'Display', None],
  [ 0x1eeb, 'LatinSmallLetterUWithHornAndGrave', [], 'Display', None],
  [ 0x1eec, 'LatinCapitalLetterUWithHornAndHookAbove', [], 'Display', None],
  [ 0x1eed, 'LatinSmallLetterUWithHornAndHookAbove', [], 'Display', None],
  [ 0x1eee, 'LatinCapitalLetterUWithHornAndTilde', [], 'Display', None],
  [ 0x1eef, 'LatinSmallLetterUWithHornAndTilde', [], 'Display', None],
  [ 0x1ef0, 'LatinCapitalLetterUWithHornAndDotBelow', [], 'Display', None],
  [ 0x1ef1, 'LatinSmallLetterUWithHornAndDotBelow', [], 'Display', None],
  [ 0x1ef2, 'LatinCapitalLetterYWithGrave', [], 'Display', None],
  [ 0x1ef3, 'LatinSmallLetterYWithGrave', [], 'Display', None],
  [ 0x1ef4, 'LatinCapitalLetterYWithDotBelow', [], 'Display', None],
  [ 0x1ef5, 'LatinSmallLetterYWithDotBelow', [], 'Display', None],
  [ 0x1ef6, 'LatinCapitalLetterYWithHookAbove', [], 'Display', None],
  [ 0x1ef7, 'LatinSmallLetterYWithHookAbove', [], 'Display', None],
  [ 0x1ef8, 'LatinCapitalLetterYWithTilde', [], 'Display', None],
  [ 0x1ef9, 'LatinSmallLetterYWithTilde', [], 'Display', None],
  [ 0x1efa, 'LatinCapitalLetterMiddlewelshLl', [], 'Display', None],
  [ 0x1efb, 'LatinSmallLetterMiddlewelshLl', [], 'Display', None],
  [ 0x1efc, 'LatinCapitalLetterMiddlewelshV', [], 'Display', None],
  [ 0x1efd, 'LatinSmallLetterMiddlewelshV', [], 'Display', None],
  [ 0x1efe, 'LatinCapitalLetterYWithLoop', [], 'Display', None],
  [ 0x1eff, 'LatinSmallLetterYWithLoop', [], 'Display', None],
  [ 0x1f00, 'GreekSmallLetterAlphaWithPsili', [], 'Display', None],
  [ 0x1f01, 'GreekSmallLetterAlphaWithDasia', [], 'Display', None],
  [ 0x1f02, 'GreekSmallLetterAlphaWithPsiliAndVaria', [], 'Display', None],
  [ 0x1f03, 'GreekSmallLetterAlphaWithDasiaAndVaria', [], 'Display', None],
  [ 0x1f04, 'GreekSmallLetterAlphaWithPsiliAndOxia', [], 'Display', None],
  [ 0x1f05, 'GreekSmallLetterAlphaWithDasiaAndOxia', [], 'Display', None],
  [ 0x1f06, 'GreekSmallLetterAlphaWithPsiliAndPerispomeni', [], 'Display', None],
  [ 0x1f07, 'GreekSmallLetterAlphaWithDasiaAndPerispomeni', [], 'Display', None],
  [ 0x1f08, 'GreekCapitalLetterAlphaWithPsili', [], 'Display', None],
  [ 0x1f09, 'GreekCapitalLetterAlphaWithDasia', [], 'Display', None],
  [ 0x1f0a, 'GreekCapitalLetterAlphaWithPsiliAndVaria', [], 'Display', None],
  [ 0x1f0b, 'GreekCapitalLetterAlphaWithDasiaAndVaria', [], 'Display', None],
  [ 0x1f0c, 'GreekCapitalLetterAlphaWithPsiliAndOxia', [], 'Display', None],
  [ 0x1f0d, 'GreekCapitalLetterAlphaWithDasiaAndOxia', [], 'Display', None],
  [ 0x1f0e, 'GreekCapitalLetterAlphaWithPsiliAndPerispomeni', [], 'Display', None],
  [ 0x1f0f, 'GreekCapitalLetterAlphaWithDasiaAndPerispomeni', [], 'Display', None],
  [ 0x1f10, 'GreekSmallLetterEpsilonWithPsili', [], 'Display', None],
  [ 0x1f11, 'GreekSmallLetterEpsilonWithDasia', [], 'Display', None],
  [ 0x1f12, 'GreekSmallLetterEpsilonWithPsiliAndVaria', [], 'Display', None],
  [ 0x1f13, 'GreekSmallLetterEpsilonWithDasiaAndVaria', [], 'Display', None],
  [ 0x1f14, 'GreekSmallLetterEpsilonWithPsiliAndOxia', [], 'Display', None],
  [ 0x1f15, 'GreekSmallLetterEpsilonWithDasiaAndOxia', [], 'Display', None],
  [ 0x1f16, 'UnicodeCharacter0x1f16', [], 'Display', None],
  [ 0x1f17, 'UnicodeCharacter0x1f17', [], 'Display', None],
  [ 0x1f18, 'GreekCapitalLetterEpsilonWithPsili', [], 'Display', None],
  [ 0x1f19, 'GreekCapitalLetterEpsilonWithDasia', [], 'Display', None],
  [ 0x1f1a, 'GreekCapitalLetterEpsilonWithPsiliAndVaria', [], 'Display', None],
  [ 0x1f1b, 'GreekCapitalLetterEpsilonWithDasiaAndVaria', [], 'Display', None],
  [ 0x1f1c, 'GreekCapitalLetterEpsilonWithPsiliAndOxia', [], 'Display', None],
  [ 0x1f1d, 'GreekCapitalLetterEpsilonWithDasiaAndOxia', [], 'Display', None],
  [ 0x1f1e, 'UnicodeCharacter0x1f1e', [], 'Display', None],
  [ 0x1f1f, 'UnicodeCharacter0x1f1f', [], 'Display', None],
  [ 0x1f20, 'GreekSmallLetterEtaWithPsili', [], 'Display', None],
  [ 0x1f21, 'GreekSmallLetterEtaWithDasia', [], 'Display', None],
  [ 0x1f22, 'GreekSmallLetterEtaWithPsiliAndVaria', [], 'Display', None],
  [ 0x1f23, 'GreekSmallLetterEtaWithDasiaAndVaria', [], 'Display', None],
  [ 0x1f24, 'GreekSmallLetterEtaWithPsiliAndOxia', [], 'Display', None],
  [ 0x1f25, 'GreekSmallLetterEtaWithDasiaAndOxia', [], 'Display', None],
  [ 0x1f26, 'GreekSmallLetterEtaWithPsiliAndPerispomeni', [], 'Display', None],
  [ 0x1f27, 'GreekSmallLetterEtaWithDasiaAndPerispomeni', [], 'Display', None],
  [ 0x1f28, 'GreekCapitalLetterEtaWithPsili', [], 'Display', None],
  [ 0x1f29, 'GreekCapitalLetterEtaWithDasia', [], 'Display', None],
  [ 0x1f2a, 'GreekCapitalLetterEtaWithPsiliAndVaria', [], 'Display', None],
  [ 0x1f2b, 'GreekCapitalLetterEtaWithDasiaAndVaria', [], 'Display', None],
  [ 0x1f2c, 'GreekCapitalLetterEtaWithPsiliAndOxia', [], 'Display', None],
  [ 0x1f2d, 'GreekCapitalLetterEtaWithDasiaAndOxia', [], 'Display', None],
  [ 0x1f2e, 'GreekCapitalLetterEtaWithPsiliAndPerispomeni', [], 'Display', None],
  [ 0x1f2f, 'GreekCapitalLetterEtaWithDasiaAndPerispomeni', [], 'Display', None],
  [ 0x1f30, 'GreekSmallLetterIotaWithPsili', [], 'Display', None],
  [ 0x1f31, 'GreekSmallLetterIotaWithDasia', [], 'Display', None],
  [ 0x1f32, 'GreekSmallLetterIotaWithPsiliAndVaria', [], 'Display', None],
  [ 0x1f33, 'GreekSmallLetterIotaWithDasiaAndVaria', [], 'Display', None],
  [ 0x1f34, 'GreekSmallLetterIotaWithPsiliAndOxia', [], 'Display', None],
  [ 0x1f35, 'GreekSmallLetterIotaWithDasiaAndOxia', [], 'Display', None],
  [ 0x1f36, 'GreekSmallLetterIotaWithPsiliAndPerispomeni', [], 'Display', None],
  [ 0x1f37, 'GreekSmallLetterIotaWithDasiaAndPerispomeni', [], 'Display', None],
  [ 0x1f38, 'GreekCapitalLetterIotaWithPsili', [], 'Display', None],
  [ 0x1f39, 'GreekCapitalLetterIotaWithDasia', [], 'Display', None],
  [ 0x1f3a, 'GreekCapitalLetterIotaWithPsiliAndVaria', [], 'Display', None],
  [ 0x1f3b, 'GreekCapitalLetterIotaWithDasiaAndVaria', [], 'Display', None],
  [ 0x1f3c, 'GreekCapitalLetterIotaWithPsiliAndOxia', [], 'Display', None],
  [ 0x1f3d, 'GreekCapitalLetterIotaWithDasiaAndOxia', [], 'Display', None],
  [ 0x1f3e, 'GreekCapitalLetterIotaWithPsiliAndPerispomeni', [], 'Display', None],
  [ 0x1f3f, 'GreekCapitalLetterIotaWithDasiaAndPerispomeni', [], 'Display', None],
  [ 0x1f40, 'GreekSmallLetterOmicronWithPsili', [], 'Display', None],
  [ 0x1f41, 'GreekSmallLetterOmicronWithDasia', [], 'Display', None],
  [ 0x1f42, 'GreekSmallLetterOmicronWithPsiliAndVaria', [], 'Display', None],
  [ 0x1f43, 'GreekSmallLetterOmicronWithDasiaAndVaria', [], 'Display', None],
  [ 0x1f44, 'GreekSmallLetterOmicronWithPsiliAndOxia', [], 'Display', None],
  [ 0x1f45, 'GreekSmallLetterOmicronWithDasiaAndOxia', [], 'Display', None],
  [ 0x1f46, 'UnicodeCharacter0x1f46', [], 'Display', None],
  [ 0x1f47, 'UnicodeCharacter0x1f47', [], 'Display', None],
  [ 0x1f48, 'GreekCapitalLetterOmicronWithPsili', [], 'Display', None],
  [ 0x1f49, 'GreekCapitalLetterOmicronWithDasia', [], 'Display', None],
  [ 0x1f4a, 'GreekCapitalLetterOmicronWithPsiliAndVaria', [], 'Display', None],
  [ 0x1f4b, 'GreekCapitalLetterOmicronWithDasiaAndVaria', [], 'Display', None],
  [ 0x1f4c, 'GreekCapitalLetterOmicronWithPsiliAndOxia', [], 'Display', None],
  [ 0x1f4d, 'GreekCapitalLetterOmicronWithDasiaAndOxia', [], 'Display', None],
  [ 0x1f4e, 'UnicodeCharacter0x1f4e', [], 'Display', None],
  [ 0x1f4f, 'UnicodeCharacter0x1f4f', [], 'Display', None],
  [ 0x1f50, 'GreekSmallLetterUpsilonWithPsili', [], 'Display', None],
  [ 0x1f51, 'GreekSmallLetterUpsilonWithDasia', [], 'Display', None],
  [ 0x1f52, 'GreekSmallLetterUpsilonWithPsiliAndVaria', [], 'Display', None],
  [ 0x1f53, 'GreekSmallLetterUpsilonWithDasiaAndVaria', [], 'Display', None],
  [ 0x1f54, 'GreekSmallLetterUpsilonWithPsiliAndOxia', [], 'Display', None],
  [ 0x1f55, 'GreekSmallLetterUpsilonWithDasiaAndOxia', [], 'Display', None],
  [ 0x1f56, 'GreekSmallLetterUpsilonWithPsiliAndPerispomeni', [], 'Display', None],
  [ 0x1f57, 'GreekSmallLetterUpsilonWithDasiaAndPerispomeni', [], 'Display', None],
  [ 0x1f58, 'UnicodeCharacter0x1f58', [], 'Display', None],
  [ 0x1f59, 'GreekCapitalLetterUpsilonWithDasia', [], 'Display', None],
  [ 0x1f5a, 'UnicodeCharacter0x1f5a', [], 'Display', None],
  [ 0x1f5b, 'GreekCapitalLetterUpsilonWithDasiaAndVaria', [], 'Display', None],
  [ 0x1f5c, 'UnicodeCharacter0x1f5c', [], 'Display', None],
  [ 0x1f5d, 'GreekCapitalLetterUpsilonWithDasiaAndOxia', [], 'Display', None],
  [ 0x1f5e, 'UnicodeCharacter0x1f5e', [], 'Display', None],
  [ 0x1f5f, 'GreekCapitalLetterUpsilonWithDasiaAndPerispomeni', [], 'Display', None],
  [ 0x1f60, 'GreekSmallLetterOmegaWithPsili', [], 'Display', None],
  [ 0x1f61, 'GreekSmallLetterOmegaWithDasia', [], 'Display', None],
  [ 0x1f62, 'GreekSmallLetterOmegaWithPsiliAndVaria', [], 'Display', None],
  [ 0x1f63, 'GreekSmallLetterOmegaWithDasiaAndVaria', [], 'Display', None],
  [ 0x1f64, 'GreekSmallLetterOmegaWithPsiliAndOxia', [], 'Display', None],
  [ 0x1f65, 'GreekSmallLetterOmegaWithDasiaAndOxia', [], 'Display', None],
  [ 0x1f66, 'GreekSmallLetterOmegaWithPsiliAndPerispomeni', [], 'Display', None],
  [ 0x1f67, 'GreekSmallLetterOmegaWithDasiaAndPerispomeni', [], 'Display', None],
  [ 0x1f68, 'GreekCapitalLetterOmegaWithPsili', [], 'Display', None],
  [ 0x1f69, 'GreekCapitalLetterOmegaWithDasia', [], 'Display', None],
  [ 0x1f6a, 'GreekCapitalLetterOmegaWithPsiliAndVaria', [], 'Display', None],
  [ 0x1f6b, 'GreekCapitalLetterOmegaWithDasiaAndVaria', [], 'Display', None],
  [ 0x1f6c, 'GreekCapitalLetterOmegaWithPsiliAndOxia', [], 'Display', None],
  [ 0x1f6d, 'GreekCapitalLetterOmegaWithDasiaAndOxia', [], 'Display', None],
  [ 0x1f6e, 'GreekCapitalLetterOmegaWithPsiliAndPerispomeni', [], 'Display', None],
  [ 0x1f6f, 'GreekCapitalLetterOmegaWithDasiaAndPerispomeni', [], 'Display', None],
  [ 0x1f70, 'GreekSmallLetterAlphaWithVaria', [], 'Display', None],
  [ 0x1f71, 'GreekSmallLetterAlphaWithOxia', [], 'Display', None],
  [ 0x1f72, 'GreekSmallLetterEpsilonWithVaria', [], 'Display', None],
  [ 0x1f73, 'GreekSmallLetterEpsilonWithOxia', [], 'Display', None],
  [ 0x1f74, 'GreekSmallLetterEtaWithVaria', [], 'Display', None],
  [ 0x1f75, 'GreekSmallLetterEtaWithOxia', [], 'Display', None],
  [ 0x1f76, 'GreekSmallLetterIotaWithVaria', [], 'Display', None],
  [ 0x1f77, 'GreekSmallLetterIotaWithOxia', [], 'Display', None],
  [ 0x1f78, 'GreekSmallLetterOmicronWithVaria', [], 'Display', None],
  [ 0x1f79, 'GreekSmallLetterOmicronWithOxia', [], 'Display', None],
  [ 0x1f7a, 'GreekSmallLetterUpsilonWithVaria', [], 'Display', None],
  [ 0x1f7b, 'GreekSmallLetterUpsilonWithOxia', [], 'Display', None],
  [ 0x1f7c, 'GreekSmallLetterOmegaWithVaria', [], 'Display', None],
  [ 0x1f7d, 'GreekSmallLetterOmegaWithOxia', [], 'Display', None],
  [ 0x1f7e, 'UnicodeCharacter0x1f7e', [], 'Display', None],
  [ 0x1f7f, 'UnicodeCharacter0x1f7f', [], 'Display', None],
  [ 0x1f80, 'GreekSmallLetterAlphaWithPsiliAndYpogegrammeni', [], 'Display', None],
  [ 0x1f81, 'GreekSmallLetterAlphaWithDasiaAndYpogegrammeni', [], 'Display', None],
  [ 0x1f82, 'GreekSmallLetterAlphaWithPsiliAndVariaAndYpogegrammeni', [], 'Display', None],
  [ 0x1f83, 'GreekSmallLetterAlphaWithDasiaAndVariaAndYpogegrammeni', [], 'Display', None],
  [ 0x1f84, 'GreekSmallLetterAlphaWithPsiliAndOxiaAndYpogegrammeni', [], 'Display', None],
  [ 0x1f85, 'GreekSmallLetterAlphaWithDasiaAndOxiaAndYpogegrammeni', [], 'Display', None],
  [ 0x1f86, 'GreekSmallLetterAlphaWithPsiliAndPerispomeniAndYpogegrammeni', [], 'Display', None],
  [ 0x1f87, 'GreekSmallLetterAlphaWithDasiaAndPerispomeniAndYpogegrammeni', [], 'Display', None],
  [ 0x1f88, 'GreekCapitalLetterAlphaWithPsiliAndProsgegrammeni', [], 'Display', None],
  [ 0x1f89, 'GreekCapitalLetterAlphaWithDasiaAndProsgegrammeni', [], 'Display', None],
  [ 0x1f8a, 'GreekCapitalLetterAlphaWithPsiliAndVariaAndProsgegrammeni', [], 'Display', None],
  [ 0x1f8b, 'GreekCapitalLetterAlphaWithDasiaAndVariaAndProsgegrammeni', [], 'Display', None],
  [ 0x1f8c, 'GreekCapitalLetterAlphaWithPsiliAndOxiaAndProsgegrammeni', [], 'Display', None],
  [ 0x1f8d, 'GreekCapitalLetterAlphaWithDasiaAndOxiaAndProsgegrammeni', [], 'Display', None],
  [ 0x1f8e, 'GreekCapitalLetterAlphaWithPsiliAndPerispomeniAndProsgegrammeni', [], 'Display', None],
  [ 0x1f8f, 'GreekCapitalLetterAlphaWithDasiaAndPerispomeniAndProsgegrammeni', [], 'Display', None],
  [ 0x1f90, 'GreekSmallLetterEtaWithPsiliAndYpogegrammeni', [], 'Display', None],
  [ 0x1f91, 'GreekSmallLetterEtaWithDasiaAndYpogegrammeni', [], 'Display', None],
  [ 0x1f92, 'GreekSmallLetterEtaWithPsiliAndVariaAndYpogegrammeni', [], 'Display', None],
  [ 0x1f93, 'GreekSmallLetterEtaWithDasiaAndVariaAndYpogegrammeni', [], 'Display', None],
  [ 0x1f94, 'GreekSmallLetterEtaWithPsiliAndOxiaAndYpogegrammeni', [], 'Display', None],
  [ 0x1f95, 'GreekSmallLetterEtaWithDasiaAndOxiaAndYpogegrammeni', [], 'Display', None],
  [ 0x1f96, 'GreekSmallLetterEtaWithPsiliAndPerispomeniAndYpogegrammeni', [], 'Display', None],
  [ 0x1f97, 'GreekSmallLetterEtaWithDasiaAndPerispomeniAndYpogegrammeni', [], 'Display', None],
  [ 0x1f98, 'GreekCapitalLetterEtaWithPsiliAndProsgegrammeni', [], 'Display', None],
  [ 0x1f99, 'GreekCapitalLetterEtaWithDasiaAndProsgegrammeni', [], 'Display', None],
  [ 0x1f9a, 'GreekCapitalLetterEtaWithPsiliAndVariaAndProsgegrammeni', [], 'Display', None],
  [ 0x1f9b, 'GreekCapitalLetterEtaWithDasiaAndVariaAndProsgegrammeni', [], 'Display', None],
  [ 0x1f9c, 'GreekCapitalLetterEtaWithPsiliAndOxiaAndProsgegrammeni', [], 'Display', None],
  [ 0x1f9d, 'GreekCapitalLetterEtaWithDasiaAndOxiaAndProsgegrammeni', [], 'Display', None],
  [ 0x1f9e, 'GreekCapitalLetterEtaWithPsiliAndPerispomeniAndProsgegrammeni', [], 'Display', None],
  [ 0x1f9f, 'GreekCapitalLetterEtaWithDasiaAndPerispomeniAndProsgegrammeni', [], 'Display', None],
  [ 0x1fa0, 'GreekSmallLetterOmegaWithPsiliAndYpogegrammeni', [], 'Display', None],
  [ 0x1fa1, 'GreekSmallLetterOmegaWithDasiaAndYpogegrammeni', [], 'Display', None],
  [ 0x1fa2, 'GreekSmallLetterOmegaWithPsiliAndVariaAndYpogegrammeni', [], 'Display', None],
  [ 0x1fa3, 'GreekSmallLetterOmegaWithDasiaAndVariaAndYpogegrammeni', [], 'Display', None],
  [ 0x1fa4, 'GreekSmallLetterOmegaWithPsiliAndOxiaAndYpogegrammeni', [], 'Display', None],
  [ 0x1fa5, 'GreekSmallLetterOmegaWithDasiaAndOxiaAndYpogegrammeni', [], 'Display', None],
  [ 0x1fa6, 'GreekSmallLetterOmegaWithPsiliAndPerispomeniAndYpogegrammeni', [], 'Display', None],
  [ 0x1fa7, 'GreekSmallLetterOmegaWithDasiaAndPerispomeniAndYpogegrammeni', [], 'Display', None],
  [ 0x1fa8, 'GreekCapitalLetterOmegaWithPsiliAndProsgegrammeni', [], 'Display', None],
  [ 0x1fa9, 'GreekCapitalLetterOmegaWithDasiaAndProsgegrammeni', [], 'Display', None],
  [ 0x1faa, 'GreekCapitalLetterOmegaWithPsiliAndVariaAndProsgegrammeni', [], 'Display', None],
  [ 0x1fab, 'GreekCapitalLetterOmegaWithDasiaAndVariaAndProsgegrammeni', [], 'Display', None],
  [ 0x1fac, 'GreekCapitalLetterOmegaWithPsiliAndOxiaAndProsgegrammeni', [], 'Display', None],
  [ 0x1fad, 'GreekCapitalLetterOmegaWithDasiaAndOxiaAndProsgegrammeni', [], 'Display', None],
  [ 0x1fae, 'GreekCapitalLetterOmegaWithPsiliAndPerispomeniAndProsgegrammeni', [], 'Display', None],
  [ 0x1faf, 'GreekCapitalLetterOmegaWithDasiaAndPerispomeniAndProsgegrammeni', [], 'Display', None],
  [ 0x1fb0, 'GreekSmallLetterAlphaWithVrachy', [], 'Display', None],
  [ 0x1fb1, 'GreekSmallLetterAlphaWithMacron', [], 'Display', None],
  [ 0x1fb2, 'GreekSmallLetterAlphaWithVariaAndYpogegrammeni', [], 'Display', None],
  [ 0x1fb3, 'GreekSmallLetterAlphaWithYpogegrammeni', [], 'Display', None],
  [ 0x1fb4, 'GreekSmallLetterAlphaWithOxiaAndYpogegrammeni', [], 'Display', None],
  [ 0x1fb5, 'UnicodeCharacter0x1fb5', [], 'Display', None],
  [ 0x1fb6, 'GreekSmallLetterAlphaWithPerispomeni', [], 'Display', None],
  [ 0x1fb7, 'GreekSmallLetterAlphaWithPerispomeniAndYpogegrammeni', [], 'Display', None],
  [ 0x1fb8, 'GreekCapitalLetterAlphaWithVrachy', [], 'Display', None],
  [ 0x1fb9, 'GreekCapitalLetterAlphaWithMacron', [], 'Display', None],
  [ 0x1fba, 'GreekCapitalLetterAlphaWithVaria', [], 'Display', None],
  [ 0x1fbb, 'GreekCapitalLetterAlphaWithOxia', [], 'Display', None],
  [ 0x1fbc, 'GreekCapitalLetterAlphaWithProsgegrammeni', [], 'Display', None],
  [ 0x1fbd, 'GreekKoronis', [], 'Display', None],
  [ 0x1fbe, 'GreekProsgegrammeni', [], 'Display', None],
  [ 0x1fbf, 'GreekPsili', [], 'Display', None],
  [ 0x1fc0, 'GreekPerispomeni', [], 'Display', None],
  [ 0x1fc1, 'GreekDialytikaAndPerispomeni', [], 'Display', None],
  [ 0x1fc2, 'GreekSmallLetterEtaWithVariaAndYpogegrammeni', [], 'Display', None],
  [ 0x1fc3, 'GreekSmallLetterEtaWithYpogegrammeni', [], 'Display', None],
  [ 0x1fc4, 'GreekSmallLetterEtaWithOxiaAndYpogegrammeni', [], 'Display', None],
  [ 0x1fc5, 'UnicodeCharacter0x1fc5', [], 'Display', None],
  [ 0x1fc6, 'GreekSmallLetterEtaWithPerispomeni', [], 'Display', None],
  [ 0x1fc7, 'GreekSmallLetterEtaWithPerispomeniAndYpogegrammeni', [], 'Display', None],
  [ 0x1fc8, 'GreekCapitalLetterEpsilonWithVaria', [], 'Display', None],
  [ 0x1fc9, 'GreekCapitalLetterEpsilonWithOxia', [], 'Display', None],
  [ 0x1fca, 'GreekCapitalLetterEtaWithVaria', [], 'Display', None],
  [ 0x1fcb, 'GreekCapitalLetterEtaWithOxia', [], 'Display', None],
  [ 0x1fcc, 'GreekCapitalLetterEtaWithProsgegrammeni', [], 'Display', None],
  [ 0x1fcd, 'GreekPsiliAndVaria', [], 'Display', None],
  [ 0x1fce, 'GreekPsiliAndOxia', [], 'Display', None],
  [ 0x1fcf, 'GreekPsiliAndPerispomeni', [], 'Display', None],
  [ 0x1fd0, 'GreekSmallLetterIotaWithVrachy', [], 'Display', None],
  [ 0x1fd1, 'GreekSmallLetterIotaWithMacron', [], 'Display', None],
  [ 0x1fd2, 'GreekSmallLetterIotaWithDialytikaAndVaria', [], 'Display', None],
  [ 0x1fd3, 'GreekSmallLetterIotaWithDialytikaAndOxia', [], 'Display', None],
  [ 0x1fd4, 'UnicodeCharacter0x1fd4', [], 'Display', None],
  [ 0x1fd5, 'UnicodeCharacter0x1fd5', [], 'Display', None],
  [ 0x1fd6, 'GreekSmallLetterIotaWithPerispomeni', [], 'Display', None],
  [ 0x1fd7, 'GreekSmallLetterIotaWithDialytikaAndPerispomeni', [], 'Display', None],
  [ 0x1fd8, 'GreekCapitalLetterIotaWithVrachy', [], 'Display', None],
  [ 0x1fd9, 'GreekCapitalLetterIotaWithMacron', [], 'Display', None],
  [ 0x1fda, 'GreekCapitalLetterIotaWithVaria', [], 'Display', None],
  [ 0x1fdb, 'GreekCapitalLetterIotaWithOxia', [], 'Display', None],
  [ 0x1fdc, 'UnicodeCharacter0x1fdc', [], 'Display', None],
  [ 0x1fdd, 'GreekDasiaAndVaria', [], 'Display', None],
  [ 0x1fde, 'GreekDasiaAndOxia', [], 'Display', None],
  [ 0x1fdf, 'GreekDasiaAndPerispomeni', [], 'Display', None],
  [ 0x1fe0, 'GreekSmallLetterUpsilonWithVrachy', [], 'Display', None],
  [ 0x1fe1, 'GreekSmallLetterUpsilonWithMacron', [], 'Display', None],
  [ 0x1fe2, 'GreekSmallLetterUpsilonWithDialytikaAndVaria', [], 'Display', None],
  [ 0x1fe3, 'GreekSmallLetterUpsilonWithDialytikaAndOxia', [], 'Display', None],
  [ 0x1fe4, 'GreekSmallLetterRhoWithPsili', [], 'Display', None],
  [ 0x1fe5, 'GreekSmallLetterRhoWithDasia', [], 'Display', None],
  [ 0x1fe6, 'GreekSmallLetterUpsilonWithPerispomeni', [], 'Display', None],
  [ 0x1fe7, 'GreekSmallLetterUpsilonWithDialytikaAndPerispomeni', [], 'Display', None],
  [ 0x1fe8, 'GreekCapitalLetterUpsilonWithVrachy', [], 'Display', None],
  [ 0x1fe9, 'GreekCapitalLetterUpsilonWithMacron', [], 'Display', None],
  [ 0x1fea, 'GreekCapitalLetterUpsilonWithVaria', [], 'Display', None],
  [ 0x1feb, 'GreekCapitalLetterUpsilonWithOxia', [], 'Display', None],
  [ 0x1fec, 'GreekCapitalLetterRhoWithDasia', [], 'Display', None],
  [ 0x1fed, 'GreekDialytikaAndVaria', [], 'Display', None],
  [ 0x1fee, 'GreekDialytikaAndOxia', [], 'Display', None],
  [ 0x1fef, 'GreekVaria', [], 'Display', None],
  [ 0x1ff0, 'UnicodeCharacter0x1ff0', [], 'Display', None],
  [ 0x1ff1, 'UnicodeCharacter0x1ff1', [], 'Display', None],
  [ 0x1ff2, 'GreekSmallLetterOmegaWithVariaAndYpogegrammeni', [], 'Display', None],
  [ 0x1ff3, 'GreekSmallLetterOmegaWithYpogegrammeni', [], 'Display', None],
  [ 0x1ff4, 'GreekSmallLetterOmegaWithOxiaAndYpogegrammeni', [], 'Display', None],
  [ 0x1ff5, 'UnicodeCharacter0x1ff5', [], 'Display', None],
  [ 0x1ff6, 'GreekSmallLetterOmegaWithPerispomeni', [], 'Display', None],
  [ 0x1ff7, 'GreekSmallLetterOmegaWithPerispomeniAndYpogegrammeni', [], 'Display', None],
  [ 0x1ff8, 'GreekCapitalLetterOmicronWithVaria', [], 'Display', None],
  [ 0x1ff9, 'GreekCapitalLetterOmicronWithOxia', [], 'Display', None],
  [ 0x1ffa, 'GreekCapitalLetterOmegaWithVaria', [], 'Display', None],
  [ 0x1ffb, 'GreekCapitalLetterOmegaWithOxia', [], 'Display', None],
  [ 0x1ffc, 'GreekCapitalLetterOmegaWithProsgegrammeni', [], 'Display', None],
  [ 0x1ffd, 'GreekOxia', [], 'Display', None],
  [ 0x1ffe, 'GreekDasia', [], 'Display', None],
  [ 0x1fff, 'UnicodeCharacter0x1fff', [], 'Display', None],
  [ 0x2000, 'EnQuad', [], 'Display', None],
  [ 0x2001, 'EmQuad', [], 'Display', None],
  [ 0x2002, 'EnSpace', [], 'Display', None],
  [ 0x2003, 'EmSpace', [], 'Display', None],
  [ 0x2004, 'ThreeperemSpace', [], 'Display', None],
  [ 0x2005, 'FourperemSpace', [], 'Display', None],
  [ 0x2006, 'SixperemSpace', [], 'Display', None],
  [ 0x2007, 'FigureSpace', [], 'Display', None],
  [ 0x2008, 'PunctuationSpace', [], 'Display', None],
  [ 0x2009, 'ThinSpace', [], 'Display', None],
  [ 0x200a, 'HairSpace', [], 'Display', None],
  [ 0x200b, 'ZeroWidthSpace', [], 'Display', None],
  [ 0x200c, 'ZeroWidthNonjoiner', [], 'Display', None],
  [ 0x200d, 'ZeroWidthJoiner', [], 'Display', None],
  [ 0x200e, 'LefttorightMark', [], 'Display', None],
  [ 0x200f, 'RighttoleftMark', [], 'Display', None],
  [ 0x2010, 'Hyphen', [], 'Display', None],
  [ 0x2011, 'NonbreakingHyphen', [], 'Display', None],
  [ 0x2012, 'FigureDash', [], 'Display', None],
  [ 0x2013, 'EnDash', [], 'Display', None],
  [ 0x2014, 'EmDash', [], 'Display', None],
  [ 0x2015, 'HorizontalBar', [], 'Display', None],
  [ 0x2016, 'DoubleVerticalLine', [], 'Display', None],
  [ 0x2017, 'DoubleLowLine', [], 'Display', None],
  [ 0x2018, 'LeftSingleQuotationMark', [], 'Display', None],
  [ 0x2019, 'RightSingleQuotationMark', [], 'Display', None],
  [ 0x201a, 'SingleLow9QuotationMark', [], 'Display', None],
  [ 0x201b, 'SingleHighreversed9QuotationMark', [], 'Display', None],
  [ 0x201c, 'LeftDoubleQuotationMark', [], 'Display', None],
  [ 0x201d, 'RightDoubleQuotationMark', [], 'Display', None],
  [ 0x201e, 'DoubleLow9QuotationMark', [], 'Display', None],
  [ 0x201f, 'DoubleHighreversed9QuotationMark', [], 'Display', None],
  [ 0x2020, 'Dagger', [], 'Display', None],
  [ 0x2021, 'DoubleDagger', [], 'Display', None],
  [ 0x2022, 'Bullet', [], 'Display', None],
  [ 0x2023, 'TriangularBullet', [], 'Display', None],
  [ 0x2024, 'OneDotLeader', [], 'Display', None],
  [ 0x2025, 'TwoDotLeader', [], 'Display', None],
  [ 0x2026, 'HorizontalEllipsis', [], 'Display', None],
  [ 0x2027, 'HyphenationPoint', [], 'Display', None],
  [ 0x2028, 'LineSeparator', [], 'Display', None],
  [ 0x2029, 'ParagraphSeparator', [], 'Display', None],
  [ 0x202a, 'LefttorightEmbedding', [], 'Display', None],
  [ 0x202b, 'RighttoleftEmbedding', [], 'Display', None],
  [ 0x202c, 'PopDirectionalFormatting', [], 'Display', None],
  [ 0x202d, 'LefttorightOverride', [], 'Display', None],
  [ 0x202e, 'RighttoleftOverride', [], 'Display', None],
  [ 0x202f, 'NarrowNobreakSpace', [], 'Display', None],
  [ 0x2030, 'PerMilleSign', [], 'Display', None],
  [ 0x2031, 'PerTenThousandSign', [], 'Display', None],
  [ 0x2032, 'Prime', [], 'Display', None],
  [ 0x2033, 'DoublePrime', [], 'Display', None],
  [ 0x2034, 'TriplePrime', [], 'Display', None],
  [ 0x2035, 'ReversedPrime', [], 'Display', None],
  [ 0x2036, 'ReversedDoublePrime', [], 'Display', None],
  [ 0x2037, 'ReversedTriplePrime', [], 'Display', None],
  [ 0x2038, 'Caret', [], 'Display', None],
  [ 0x2039, 'SingleLeftpointingAngleQuotationMark', [], 'Display', None],
  [ 0x203a, 'SingleRightpointingAngleQuotationMark', [], 'Display', None],
  [ 0x203b, 'ReferenceMark', [], 'Display', None],
  [ 0x203c, 'DoubleExclamationMark', [], 'Display', None],
  [ 0x203d, 'Interrobang', [], 'Display', None],
  [ 0x203e, 'Overline', [], 'Display', None],
  [ 0x203f, 'Undertie', [], 'Display', None],
  [ 0x2040, 'CharacterTie', [], 'Display', None],
  [ 0x2041, 'CaretInsertionPoint', [], 'Display', None],
  [ 0x2042, 'Asterism', [], 'Display', None],
  [ 0x2043, 'HyphenBullet', [], 'Display', None],
  [ 0x2044, 'FractionSlash', ['frac','/'], 'InfixOperator', '*'],
  [ 0x2045, 'LeftSquareBracketWithQuill', [], 'Display', None],
  [ 0x2046, 'RightSquareBracketWithQuill', [], 'Display', None],
  [ 0x2047, 'DoubleQuestionMark', [], 'Display', None],
  [ 0x2048, 'QuestionExclamationMark', [], 'Display', None],
  [ 0x2049, 'ExclamationQuestionMark', [], 'Display', None],
  [ 0x204a, 'TironianSignEt', [], 'Display', None],
  [ 0x204b, 'ReversedPilcrowSign', [], 'Display', None],
  [ 0x204c, 'BlackLeftwardsBullet', [], 'Display', None],
  [ 0x204d, 'BlackRightwardsBullet', [], 'Display', None],
  [ 0x204e, 'LowAsterisk', [], 'Display', None],
  [ 0x204f, 'ReversedSemicolon', [], 'Display', None],
  [ 0x2050, 'CloseUp', [], 'Display', None],
  [ 0x2051, 'TwoAsterisksAlignedVertically', [], 'Display', None],
  [ 0x2052, 'CommercialMinusSign', [], 'Display', None],
  [ 0x2053, 'SwungDash', [], 'Display', None],
  [ 0x2054, 'InvertedUndertie', [], 'Display', None],
  [ 0x2055, 'FlowerPunctuationMark', [], 'Display', None],
  [ 0x2056, 'ThreeDotPunctuation', [], 'Display', None],
  [ 0x2057, 'QuadruplePrime', [], 'Display', None],
  [ 0x2058, 'FourDotPunctuation', [], 'Display', None],
  [ 0x2059, 'FiveDotPunctuation', [], 'Display', None],
  [ 0x205a, 'TwoDotPunctuation', [], 'Display', None],
  [ 0x205b, 'FourDotMark', [], 'Display', None],
  [ 0x205c, 'DottedCross', [], 'Display', None],
  [ 0x205d, 'Tricolon', [], 'Display', None],
  [ 0x205e, 'VerticalFourDots', [], 'Display', None],
  [ 0x205f, 'MediumMathematicalSpace', [], 'Display', None],
  [ 0x2060, 'WordJoiner', [], 'Display', None],
  [ 0x2061, 'FunctionApplication', [], 'Display', None],
  [ 0x2062, 'InvisibleTimes', [], 'Display', None],
  [ 0x2063, 'InvisibleSeparator', [], 'Display', None],
  [ 0x2064, 'InvisiblePlus', [], 'Display', None],
  [ 0x2065, 'UnicodeCharacter0x2065', [], 'Display', None],
  [ 0x2066, 'UnicodeCharacter0x2066', [], 'Display', None],
  [ 0x2067, 'UnicodeCharacter0x2067', [], 'Display', None],
  [ 0x2068, 'UnicodeCharacter0x2068', [], 'Display', None],
  [ 0x2069, 'UnicodeCharacter0x2069', [], 'Display', None],
  [ 0x206a, 'InhibitSymmetricSwapping', [], 'Display', None],
  [ 0x206b, 'ActivateSymmetricSwapping', [], 'Display', None],
  [ 0x206c, 'InhibitArabicFormShaping', [], 'Display', None],
  [ 0x206d, 'ActivateArabicFormShaping', [], 'Display', None],
  [ 0x206e, 'NationalDigitShapes', [], 'Display', None],
  [ 0x206f, 'NominalDigitShapes', [], 'Display', None],
  [ 0x2070, 'SuperscriptZero', ['^0','**0'], 'SimpleSubstitution', '**0'],
  [ 0x2071, 'SuperscriptLatinSmallLetterI', ['^i','**i','superi'], 'Display', None],
  [ 0x2072, 'UnicodeCharacter0x2072', [], 'Display', None],
  [ 0x2073, 'UnicodeCharacter0x2073', [], 'Display', None],
  [ 0x2074, 'SuperscriptFour', ['^4','**4'], 'SimpleSubstitution', '**4'],
  [ 0x2075, 'SuperscriptFive', ['^5','**5'], 'SimpleSubstitution', '**5'],
  [ 0x2076, 'SuperscriptSix', ['^6','**6'], 'SimpleSubstitution', '**6'],
  [ 0x2077, 'SuperscriptSeven', ['^7','**7'], 'SimpleSubstitution', '**7'],
  [ 0x2078, 'SuperscriptEight', ['^8','**8'], 'SimpleSubstitution', '**8'],
  [ 0x2079, 'SuperscriptNine', ['^9','**9'], 'SimpleSubstitution', '**9'],
  [ 0x207a, 'SuperscriptPlusSign', ['^+','**+'], 'Display', None],
  [ 0x207b, 'SuperscriptMinus', ['^-','**-'], 'Display', None],
  [ 0x207c, 'SuperscriptEqualsSign', ['^=','**='], 'Display', None],
  [ 0x207d, 'SuperscriptLeftParenthesis', ['^(','**('], 'Display', None],
  [ 0x207e, 'SuperscriptRightParenthesis', ['^)','**)'], 'Display', None],
  [ 0x207f, 'SuperscriptLatinSmallLetterN', ['^n','**n'], 'Display', None],
  [ 0x2080, 'SubscriptZero', ['_0','[0]'], 'Symbol', None],
  [ 0x2081, 'SubscriptOne', ['_1','[1]'], 'Symbol', None],
  [ 0x2082, 'SubscriptTwo', ['_2','[2]'], 'Symbol', None],
  [ 0x2083, 'SubscriptThree', ['_3','[3]'], 'Symbol', None],
  [ 0x2084, 'SubscriptFour', ['_4','[4]'], 'Symbol', None],
  [ 0x2085, 'SubscriptFive', ['_5','[5]'], 'Symbol', None],
  [ 0x2086, 'SubscriptSix', ['_6','[6]'], 'Symbol', None],
  [ 0x2087, 'SubscriptSeven', ['_7','[7]'], 'Symbol', None],
  [ 0x2088, 'SubscriptEight', ['_8','[8]'], 'Symbol', None],
  [ 0x2089, 'SubscriptNine', ['_9','[9]'], 'Symbol', None],
  [ 0x208a, 'SubscriptPlusSign', ['_+'], 'Display', None],
  [ 0x208b, 'SubscriptMinus', ['_-'], 'Display', None],
  [ 0x208c, 'SubscriptEqualsSign', ['_='], 'Display', None],
  [ 0x208d, 'SubscriptLeftParenthesis', ['_('], 'Display', None],
  [ 0x208e, 'SubscriptRightParenthesis', ['_)'], 'Display', None],
  [ 0x208f, 'UnicodeCharacter0x208f', [], 'Display', None],
  [ 0x2090, 'LatinSubscriptSmallLetterA', ['suba','_a'], 'Symbol', None],
  [ 0x2091, 'LatinSubscriptSmallLetterE', ['sube','_e'], 'Symbol', None],
  [ 0x2092, 'LatinSubscriptSmallLetterO', ['subo','_o'], 'Symbol', None],
  [ 0x2093, 'LatinSubscriptSmallLetterX', ['subx','_x'], 'Symbol', None],
  [ 0x2094, 'LatinSubscriptSmallLetterSchwa', [], 'Display', None],
  [ 0x2095, 'UnicodeCharacter0x2095', [], 'Display', None],
  [ 0x2096, 'UnicodeCharacter0x2096', [], 'Display', None],
  [ 0x2097, 'UnicodeCharacter0x2097', [], 'Display', None],
  [ 0x2098, 'UnicodeCharacter0x2098', [], 'Display', None],
  [ 0x2099, 'UnicodeCharacter0x2099', [], 'Display', None],
  [ 0x209a, 'UnicodeCharacter0x209a', [], 'Display', None],
  [ 0x209b, 'UnicodeCharacter0x209b', [], 'Display', None],
  [ 0x209c, 'UnicodeCharacter0x209c', [], 'Display', None],
  [ 0x209d, 'UnicodeCharacter0x209d', [], 'Display', None],
  [ 0x209e, 'UnicodeCharacter0x209e', [], 'Display', None],
  [ 0x209f, 'UnicodeCharacter0x209f', [], 'Display', None],
  [ 0x20a0, 'EurocurrencySign', [], 'Display', None],
  [ 0x20a1, 'ColonSign', [], 'Display', None],
  [ 0x20a2, 'CruzeiroSign', [], 'Display', None],
  [ 0x20a3, 'FrenchFrancSign', [], 'Display', None],
  [ 0x20a4, 'LiraSign', [], 'Display', None],
  [ 0x20a5, 'MillSign', [], 'Display', None],
  [ 0x20a6, 'NairaSign', [], 'Display', None],
  [ 0x20a7, 'PesetaSign', [], 'Display', None],
  [ 0x20a8, 'RupeeSign', [], 'Display', None],
  [ 0x20a9, 'WonSign', [], 'Display', None],
  [ 0x20aa, 'NewSheqelSign', [], 'Display', None],
  [ 0x20ab, 'DongSign', [], 'Display', None],
  [ 0x20ac, 'EuroSign', [], 'Display', None],
  [ 0x20ad, 'KipSign', [], 'Display', None],
  [ 0x20ae, 'TugrikSign', [], 'Display', None],
  [ 0x20af, 'DrachmaSign', [], 'Display', None],
  [ 0x20b0, 'GermanPennySign', [], 'Display', None],
  [ 0x20b1, 'PesoSign', [], 'Display', None],
  [ 0x20b2, 'GuaraniSign', [], 'Display', None],
  [ 0x20b3, 'AustralSign', [], 'Display', None],
  [ 0x20b4, 'HryvniaSign', [], 'Display', None],
  [ 0x20b5, 'CediSign', [], 'Display', None],
  [ 0x20b6, 'UnicodeCharacter0x20b6', [], 'Display', None],
  [ 0x20b7, 'UnicodeCharacter0x20b7', [], 'Display', None],
  [ 0x20b8, 'UnicodeCharacter0x20b8', [], 'Display', None],
  [ 0x20b9, 'UnicodeCharacter0x20b9', [], 'Display', None],
  [ 0x20ba, 'UnicodeCharacter0x20ba', [], 'Display', None],
  [ 0x20bb, 'UnicodeCharacter0x20bb', [], 'Display', None],
  [ 0x20bc, 'UnicodeCharacter0x20bc', [], 'Display', None],
  [ 0x20bd, 'UnicodeCharacter0x20bd', [], 'Display', None],
  [ 0x20be, 'UnicodeCharacter0x20be', [], 'Display', None],
  [ 0x20bf, 'UnicodeCharacter0x20bf', [], 'Display', None],
  [ 0x20c0, 'UnicodeCharacter0x20c0', [], 'Display', None],
  [ 0x20c1, 'UnicodeCharacter0x20c1', [], 'Display', None],
  [ 0x20c2, 'UnicodeCharacter0x20c2', [], 'Display', None],
  [ 0x20c3, 'UnicodeCharacter0x20c3', [], 'Display', None],
  [ 0x20c4, 'UnicodeCharacter0x20c4', [], 'Display', None],
  [ 0x20c5, 'UnicodeCharacter0x20c5', [], 'Display', None],
  [ 0x20c6, 'UnicodeCharacter0x20c6', [], 'Display', None],
  [ 0x20c7, 'UnicodeCharacter0x20c7', [], 'Display', None],
  [ 0x20c8, 'UnicodeCharacter0x20c8', [], 'Display', None],
  [ 0x20c9, 'UnicodeCharacter0x20c9', [], 'Display', None],
  [ 0x20ca, 'UnicodeCharacter0x20ca', [], 'Display', None],
  [ 0x20cb, 'UnicodeCharacter0x20cb', [], 'Display', None],
  [ 0x20cc, 'UnicodeCharacter0x20cc', [], 'Display', None],
  [ 0x20cd, 'UnicodeCharacter0x20cd', [], 'Display', None],
  [ 0x20ce, 'UnicodeCharacter0x20ce', [], 'Display', None],
  [ 0x20cf, 'UnicodeCharacter0x20cf', [], 'Display', None],
  [ 0x20d0, 'CombiningLeftHarpoonAbove', [], 'Display', None],
  [ 0x20d1, 'CombiningRightHarpoonAbove', [], 'Display', None],
  [ 0x20d2, 'CombiningLongVerticalLineOverlay', [], 'Display', None],
  [ 0x20d3, 'CombiningShortVerticalLineOverlay', [], 'Display', None],
  [ 0x20d4, 'CombiningAnticlockwiseArrowAbove', [], 'Display', None],
  [ 0x20d5, 'CombiningClockwiseArrowAbove', [], 'Display', None],
  [ 0x20d6, 'CombiningLeftArrowAbove', [], 'Display', None],
  [ 0x20d7, 'CombiningRightArrowAbove', [], 'Display', None],
  [ 0x20d8, 'CombiningRingOverlay', [], 'Display', None],
  [ 0x20d9, 'CombiningClockwiseRingOverlay', [], 'Display', None],
  [ 0x20da, 'CombiningAnticlockwiseRingOverlay', [], 'Display', None],
  [ 0x20db, 'CombiningThreeDotsAbove', [], 'Display', None],
  [ 0x20dc, 'CombiningFourDotsAbove', [], 'Display', None],
  [ 0x20dd, 'CombiningEnclosingCircle', [], 'Display', None],
  [ 0x20de, 'CombiningEnclosingSquare', [], 'Display', None],
  [ 0x20df, 'CombiningEnclosingDiamond', [], 'Display', None],
  [ 0x20e0, 'CombiningEnclosingCircleBackslash', [], 'Display', None],
  [ 0x20e1, 'CombiningLeftRightArrowAbove', [], 'Display', None],
  [ 0x20e2, 'CombiningEnclosingScreen', [], 'Display', None],
  [ 0x20e3, 'CombiningEnclosingKeycap', [], 'Display', None],
  [ 0x20e4, 'CombiningEnclosingUpwardPointingTriangle', [], 'Display', None],
  [ 0x20e5, 'CombiningReverseSolidusOverlay', [], 'Display', None],
  [ 0x20e6, 'CombiningDoubleVerticalStrokeOverlay', [], 'Display', None],
  [ 0x20e7, 'CombiningAnnuitySymbol', [], 'Display', None],
  [ 0x20e8, 'CombiningTripleUnderdot', [], 'Display', None],
  [ 0x20e9, 'CombiningWideBridgeAbove', [], 'Display', None],
  [ 0x20ea, 'CombiningLeftwardsArrowOverlay', [], 'Display', None],
  [ 0x20eb, 'CombiningLongDoubleSolidusOverlay', [], 'Display', None],
  [ 0x20ec, 'CombiningRightwardsHarpoonWithBarbDownwards', [], 'Display', None],
  [ 0x20ed, 'CombiningLeftwardsHarpoonWithBarbDownwards', [], 'Display', None],
  [ 0x20ee, 'CombiningLeftArrowBelow', [], 'Display', None],
  [ 0x20ef, 'CombiningRightArrowBelow', [], 'Display', None],
  [ 0x20f0, 'CombiningAsteriskAbove', [], 'Display', None],
  [ 0x20f1, 'UnicodeCharacter0x20f1', [], 'Display', None],
  [ 0x20f2, 'UnicodeCharacter0x20f2', [], 'Display', None],
  [ 0x20f3, 'UnicodeCharacter0x20f3', [], 'Display', None],
  [ 0x20f4, 'UnicodeCharacter0x20f4', [], 'Display', None],
  [ 0x20f5, 'UnicodeCharacter0x20f5', [], 'Display', None],
  [ 0x20f6, 'UnicodeCharacter0x20f6', [], 'Display', None],
  [ 0x20f7, 'UnicodeCharacter0x20f7', [], 'Display', None],
  [ 0x20f8, 'UnicodeCharacter0x20f8', [], 'Display', None],
  [ 0x20f9, 'UnicodeCharacter0x20f9', [], 'Display', None],
  [ 0x20fa, 'UnicodeCharacter0x20fa', [], 'Display', None],
  [ 0x20fb, 'UnicodeCharacter0x20fb', [], 'Display', None],
  [ 0x20fc, 'UnicodeCharacter0x20fc', [], 'Display', None],
  [ 0x20fd, 'UnicodeCharacter0x20fd', [], 'Display', None],
  [ 0x20fe, 'UnicodeCharacter0x20fe', [], 'Display', None],
  [ 0x20ff, 'UnicodeCharacter0x20ff', [], 'Display', None],
  [ 0x2100, 'AccountOf', [], 'Display', None],
  [ 0x2101, 'AddressedToTheSubject', [], 'Display', None],
  [ 0x2102, 'DoublestruckCapitalC', [], 'Display', None],
  [ 0x2103, 'DegreeCelsius', [], 'Display', None],
  [ 0x2104, 'CentreLineSymbol', [], 'Display', None],
  [ 0x2105, 'CareOf', [], 'Display', None],
  [ 0x2106, 'CadaUna', [], 'Display', None],
  [ 0x2107, 'EulerConstant', [], 'Display', None],
  [ 0x2108, 'Scruple', [], 'Display', None],
  [ 0x2109, 'DegreeFahrenheit', [], 'Display', None],
  [ 0x210a, 'ScriptSmallG', [], 'Display', None],
  [ 0x210b, 'ScriptCapitalH', [], 'Display', None],
  [ 0x210c, 'BlackletterCapitalH', [], 'Display', None],
  [ 0x210d, 'DoublestruckCapitalH', [], 'Display', None],
  [ 0x210e, 'PlanckConstant', [], 'Display', None],
  [ 0x210f, 'PlanckConstantOverTwoPi', [], 'Display', None],
  [ 0x2110, 'ScriptCapitalI', [], 'Display', None],
  [ 0x2111, 'BlackletterCapitalI', [], 'Display', None],
  [ 0x2112, 'ScriptCapitalL', [], 'Display', None],
  [ 0x2113, 'ScriptSmallL', [], 'Display', None],
  [ 0x2114, 'LBBarSymbol', [], 'Display', None],
  [ 0x2115, 'DoublestruckCapitalN', [], 'Display', None],
  [ 0x2116, 'NumeroSign', [], 'Display', None],
  [ 0x2117, 'SoundRecordingCopyright', [], 'Display', None],
  [ 0x2118, 'ScriptCapitalP', [], 'Display', None],
  [ 0x2119, 'DoublestruckCapitalP', [], 'Display', None],
  [ 0x211a, 'DoublestruckCapitalQ', [], 'Display', None],
  [ 0x211b, 'ScriptCapitalR', [], 'Display', None],
  [ 0x211c, 'BlackletterCapitalR', [], 'Display', None],
  [ 0x211d, 'DoublestruckCapitalR', [], 'Display', None],
  [ 0x211e, 'PrescriptionTake', [], 'Display', None],
  [ 0x211f, 'Response', [], 'Display', None],
  [ 0x2120, 'ServiceMark', [], 'Display', None],
  [ 0x2121, 'TelephoneSign', [], 'Display', None],
  [ 0x2122, 'TradeMarkSign', [], 'Display', None],
  [ 0x2123, 'Versicle', [], 'Display', None],
  [ 0x2124, 'DoublestruckCapitalZ', [], 'Display', None],
  [ 0x2125, 'OunceSign', [], 'Display', None],
  [ 0x2126, 'OhmSign', [], 'Display', None],
  [ 0x2127, 'InvertedOhmSign', [], 'Display', None],
  [ 0x2128, 'BlackletterCapitalZ', [], 'Display', None],
  [ 0x2129, 'TurnedGreekSmallLetterIota', [], 'Display', None],
  [ 0x212a, 'KelvinSign', [], 'Display', None],
  [ 0x212b, 'AngstromSign', [], 'Display', None],
  [ 0x212c, 'ScriptCapitalB', [], 'Display', None],
  [ 0x212d, 'BlackletterCapitalC', [], 'Display', None],
  [ 0x212e, 'EstimatedSymbol', [], 'Display', None],
  [ 0x212f, 'ScriptSmallE', [], 'Display', None],
  [ 0x2130, 'ScriptCapitalE', [], 'Display', None],
  [ 0x2131, 'ScriptCapitalF', [], 'Display', None],
  [ 0x2132, 'TurnedCapitalF', [], 'Display', None],
  [ 0x2133, 'ScriptCapitalM', [], 'Display', None],
  [ 0x2134, 'ScriptSmallO', [], 'Display', None],
  [ 0x2135, 'AlefSymbol', [], 'Display', None],
  [ 0x2136, 'BetSymbol', [], 'Display', None],
  [ 0x2137, 'GimelSymbol', [], 'Display', None],
  [ 0x2138, 'DaletSymbol', [], 'Display', None],
  [ 0x2139, 'InformationSource', [], 'Display', None],
  [ 0x213a, 'RotatedCapitalQ', [], 'Display', None],
  [ 0x213b, 'FacsimileSign', [], 'Display', None],
  [ 0x213c, 'DoublestruckSmallPi', ['pipi'], 'Display', None],
  [ 0x213d, 'DoublestruckSmallGamma', ['gammagamma'], 'Display', None],
  [ 0x213e, 'DoublestruckCapitalGamma', ['GammaGamma'], 'Display', None],
  [ 0x213f, 'DoublestruckCapitalPi', ['PiPi'], 'Display', None],
  [ 0x2140, 'DoublestruckNarySummation', ['SumSum'], 'Display', None],
  [ 0x2141, 'TurnedSansserifCapitalG', ['GG'], 'Display', None],
  [ 0x2142, 'TurnedSansserifCapitalL', ['LL'], 'Display', None],
  [ 0x2143, 'ReversedSansserifCapitalL', [], 'Display', None],
  [ 0x2144, 'TurnedSansserifCapitalY', [], 'Display', None],
  [ 0x2145, 'DoublestruckItalicCapitalD', ['differentialD','diffD','DD'], 'Symbol', None],
  [ 0x2146, 'DoublestruckItalicSmallD', ['differentiald','diffd','dd'], 'Symbol', None],
  [ 0x2147, 'DoublestruckItalicSmallE', ['naturalExponent','natE','ee','natexp'], 'Symbol', None],
  [ 0x2148, 'DoublestruckItalicSmallI', ['imaginaryI','imag','im','ii'], 'Symbol', None],
  [ 0x2149, 'DoublestruckItalicSmallJ', ['imaginaryJ','imagj','imj','jj'], 'Symbol', None],
  [ 0x214a, 'PropertyLine', [], 'Display', None],
  [ 0x214b, 'TurnedAmpersand', [], 'Display', None],
  [ 0x214c, 'PerSign', [], 'Display', None],
  [ 0x214d, 'Aktieselskab', [], 'Display', None],
  [ 0x214e, 'TurnedSmallF', [], 'Display', None],
  [ 0x214f, 'SymbolForSamaritanSource', [], 'Display', None],
  [ 0x2150, 'UnicodeCharacter0x2150', [], 'Display', None],
  [ 0x2151, 'UnicodeCharacter0x2151', [], 'Display', None],
  [ 0x2152, 'UnicodeCharacter0x2152', [], 'Display', None],
  [ 0x2153, 'VulgarFractionOneThird', [], 'Display', None],
  [ 0x2154, 'VulgarFractionTwoThirds', [], 'Display', None],
  [ 0x2155, 'VulgarFractionOneFifth', [], 'Display', None],
  [ 0x2156, 'VulgarFractionTwoFifths', [], 'Display', None],
  [ 0x2157, 'VulgarFractionThreeFifths', [], 'Display', None],
  [ 0x2158, 'VulgarFractionFourFifths', [], 'Display', None],
  [ 0x2159, 'VulgarFractionOneSixth', [], 'Display', None],
  [ 0x215a, 'VulgarFractionFiveSixths', [], 'Display', None],
  [ 0x215b, 'VulgarFractionOneEighth', [], 'Display', None],
  [ 0x215c, 'VulgarFractionThreeEighths', [], 'Display', None],
  [ 0x215d, 'VulgarFractionFiveEighths', [], 'Display', None],
  [ 0x215e, 'VulgarFractionSevenEighths', [], 'Display', None],
  [ 0x215f, 'FractionNumeratorOne', [], 'Display', None],
  [ 0x2160, 'RomanNumeralOne', [], 'Display', None],
  [ 0x2161, 'RomanNumeralTwo', [], 'Display', None],
  [ 0x2162, 'RomanNumeralThree', [], 'Display', None],
  [ 0x2163, 'RomanNumeralFour', [], 'Display', None],
  [ 0x2164, 'RomanNumeralFive', [], 'Display', None],
  [ 0x2165, 'RomanNumeralSix', [], 'Display', None],
  [ 0x2166, 'RomanNumeralSeven', [], 'Display', None],
  [ 0x2167, 'RomanNumeralEight', [], 'Display', None],
  [ 0x2168, 'RomanNumeralNine', [], 'Display', None],
  [ 0x2169, 'RomanNumeralTen', [], 'Display', None],
  [ 0x216a, 'RomanNumeralEleven', [], 'Display', None],
  [ 0x216b, 'RomanNumeralTwelve', [], 'Display', None],
  [ 0x216c, 'RomanNumeralFifty', [], 'Display', None],
  [ 0x216d, 'RomanNumeralOneHundred', [], 'Display', None],
  [ 0x216e, 'RomanNumeralFiveHundred', [], 'Display', None],
  [ 0x216f, 'RomanNumeralOneThousand', [], 'Display', None],
  [ 0x2170, 'SmallRomanNumeralOne', [], 'Display', None],
  [ 0x2171, 'SmallRomanNumeralTwo', [], 'Display', None],
  [ 0x2172, 'SmallRomanNumeralThree', [], 'Display', None],
  [ 0x2173, 'SmallRomanNumeralFour', [], 'Display', None],
  [ 0x2174, 'SmallRomanNumeralFive', [], 'Display', None],
  [ 0x2175, 'SmallRomanNumeralSix', [], 'Display', None],
  [ 0x2176, 'SmallRomanNumeralSeven', [], 'Display', None],
  [ 0x2177, 'SmallRomanNumeralEight', [], 'Display', None],
  [ 0x2178, 'SmallRomanNumeralNine', [], 'Display', None],
  [ 0x2179, 'SmallRomanNumeralTen', [], 'Display', None],
  [ 0x217a, 'SmallRomanNumeralEleven', [], 'Display', None],
  [ 0x217b, 'SmallRomanNumeralTwelve', [], 'Display', None],
  [ 0x217c, 'SmallRomanNumeralFifty', [], 'Display', None],
  [ 0x217d, 'SmallRomanNumeralOneHundred', [], 'Display', None],
  [ 0x217e, 'SmallRomanNumeralFiveHundred', [], 'Display', None],
  [ 0x217f, 'SmallRomanNumeralOneThousand', [], 'Display', None],
  [ 0x2180, 'RomanNumeralOneThousandCD', [], 'Display', None],
  [ 0x2181, 'RomanNumeralFiveThousand', [], 'Display', None],
  [ 0x2182, 'RomanNumeralTenThousand', [], 'Display', None],
  [ 0x2183, 'RomanNumeralReversedOneHundred', [], 'Display', None],
  [ 0x2184, 'LatinSmallLetterReversedC', [], 'Display', None],
  [ 0x2185, 'RomanNumeralSixLateForm', [], 'Display', None],
  [ 0x2186, 'RomanNumeralFiftyEarlyForm', [], 'Display', None],
  [ 0x2187, 'RomanNumeralFiftyThousand', [], 'Display', None],
  [ 0x2188, 'RomanNumeralOneHundredThousand', [], 'Display', None],
  [ 0x2189, 'UnicodeCharacter0x2189', [], 'Display', None],
  [ 0x218a, 'UnicodeCharacter0x218a', [], 'Display', None],
  [ 0x218b, 'UnicodeCharacter0x218b', [], 'Display', None],
  [ 0x218c, 'UnicodeCharacter0x218c', [], 'Display', None],
  [ 0x218d, 'UnicodeCharacter0x218d', [], 'Display', None],
  [ 0x218e, 'UnicodeCharacter0x218e', [], 'Display', None],
  [ 0x218f, 'UnicodeCharacter0x218f', [], 'Display', None],
  [ 0x2190, 'LeftwardsArrow', ['<-','leftarrow'], 'Display', None],
  [ 0x2191, 'UpwardsArrow', ['|/\\','uparrow'], 'Display', None],
  [ 0x2192, 'RightwardsArrow', ['->','rightarrow'], 'Display', None],
  [ 0x2193, 'DownwardsArrow', ['|\\/','downarrow'], 'Display', None],
  [ 0x2194, 'LeftRightArrow', ['<->','leftrightarrow'], 'Display', None],
  [ 0x2195, 'UpDownArrow', ['/\\|\\/','updownarrow'], 'Display', None],
  [ 0x2196, 'NorthWestArrow', ['nwarrow'], 'Display', None],
  [ 0x2197, 'NorthEastArrow', ['nearrow'], 'Display', None],
  [ 0x2198, 'SouthEastArrow', ['searrow'], 'Display', None],
  [ 0x2199, 'SouthWestArrow', ['swarrow'], 'Display', None],
  [ 0x219a, 'LeftwardsArrowWithStroke', ['!<-'], 'Display', None],
  [ 0x219b, 'RightwardsArrowWithStroke', ['!->'], 'Display', None],
  [ 0x219c, 'LeftwardsWaveArrow', ['<~'], 'Display', None],
  [ 0x219d, 'RightwardsWaveArrow', ['~>'], 'Display', None],
  [ 0x219e, 'LeftwardsTwoHeadedArrow', ['<<-'], 'Display', None],
  [ 0x219f, 'UpwardsTwoHeadedArrow', [], 'Display', None],
  [ 0x21a0, 'RightwardsTwoHeadedArrow', ['->>'], 'Display', None],
  [ 0x21a1, 'DownwardsTwoHeadedArrow', [], 'Display', None],
  [ 0x21a2, 'LeftwardsArrowWithTail', ['<-<'], 'Display', None],
  [ 0x21a3, 'RightwardsArrowWithTail', ['>->'], 'Display', None],
  [ 0x21a4, 'LeftwardsArrowFromBar', ['<-|'], 'Display', None],
  [ 0x21a5, 'UpwardsArrowFromBar', [], 'Display', None],
  [ 0x21a6, 'RightwardsArrowFromBar', ['|->'], 'Display', None],
  [ 0x21a7, 'DownwardsArrowFromBar', [], 'Display', None],
  [ 0x21a8, 'UpDownArrowWithBase', [], 'Display', None],
  [ 0x21a9, 'LeftwardsArrowWithHook', [], 'Display', None],
  [ 0x21aa, 'RightwardsArrowWithHook', [], 'Display', None],
  [ 0x21ab, 'LeftwardsArrowWithLoop', [], 'Display', None],
  [ 0x21ac, 'RightwardsArrowWithLoop', [], 'Display', None],
  [ 0x21ad, 'LeftRightWaveArrow', [], 'Display', None],
  [ 0x21ae, 'LeftRightArrowWithStroke', [], 'Display', None],
  [ 0x21af, 'DownwardsZigzagArrow', [], 'Display', None],
  [ 0x21b0, 'UpwardsArrowWithTipLeftwards', [], 'Display', None],
  [ 0x21b1, 'UpwardsArrowWithTipRightwards', [], 'Display', None],
  [ 0x21b2, 'DownwardsArrowWithTipLeftwards', [], 'Display', None],
  [ 0x21b3, 'DownwardsArrowWithTipRightwards', [], 'Display', None],
  [ 0x21b4, 'RightwardsArrowWithCornerDownwards', [], 'Display', None],
  [ 0x21b5, 'DownwardsArrowWithCornerLeftwards', [], 'Display', None],
  [ 0x21b6, 'AnticlockwiseTopSemicircleArrow', [], 'Display', None],
  [ 0x21b7, 'ClockwiseTopSemicircleArrow', [], 'Display', None],
  [ 0x21b8, 'NorthWestArrowToLongBar', [], 'Display', None],
  [ 0x21b9, 'LeftwardsArrowToBarOverRightwardsArrowToBar', [], 'Display', None],
  [ 0x21ba, 'AnticlockwiseOpenCircleArrow', [], 'Display', None],
  [ 0x21bb, 'ClockwiseOpenCircleArrow', [], 'Display', None],
  [ 0x21bc, 'LeftwardsHarpoonWithBarbUpwards', [], 'Display', None],
  [ 0x21bd, 'LeftwardsHarpoonWithBarbDownwards', [], 'Display', None],
  [ 0x21be, 'UpwardsHarpoonWithBarbRightwards', [], 'Display', None],
  [ 0x21bf, 'UpwardsHarpoonWithBarbLeftwards', [], 'Display', None],
  [ 0x21c0, 'RightwardsHarpoonWithBarbUpwards', [], 'Display', None],
  [ 0x21c1, 'RightwardsHarpoonWithBarbDownwards', [], 'Display', None],
  [ 0x21c2, 'DownwardsHarpoonWithBarbRightwards', [], 'Display', None],
  [ 0x21c3, 'DownwardsHarpoonWithBarbLeftwards', [], 'Display', None],
  [ 0x21c4, 'RightwardsArrowOverLeftwardsArrow', [], 'Display', None],
  [ 0x21c5, 'UpwardsArrowLeftwardsOfDownwardsArrow', [], 'Display', None],
  [ 0x21c6, 'LeftwardsArrowOverRightwardsArrow', [], 'Display', None],
  [ 0x21c7, 'LeftwardsPairedArrows', [], 'Display', None],
  [ 0x21c8, 'UpwardsPairedArrows', [], 'Display', None],
  [ 0x21c9, 'RightwardsPairedArrows', [], 'Display', None],
  [ 0x21ca, 'DownwardsPairedArrows', [], 'Display', None],
  [ 0x21cb, 'LeftwardsHarpoonOverRightwardsHarpoon', [], 'Display', None],
  [ 0x21cc, 'RightwardsHarpoonOverLeftwardsHarpoon', [], 'Display', None],
  [ 0x21cd, 'LeftwardsDoubleArrowWithStroke', ['!<<-'], 'Display', None],
  [ 0x21ce, 'LeftRightDoubleArrowWithStroke', ['!<<->>'], 'Display', None],
  [ 0x21cf, 'RightwardsDoubleArrowWithStroke', ['!->>'], 'Display', None],
  [ 0x21d0, 'LeftwardsDoubleArrow', ['<='], 'Display', None],
  [ 0x21d1, 'UpwardsDoubleArrow', [], 'Display', None],
  [ 0x21d2, 'RightwardsDoubleArrow', ['=>'], 'Display', None],
  [ 0x21d3, 'DownwardsDoubleArrow', [], 'Display', None],
  [ 0x21d4, 'LeftRightDoubleArrow', ['<=>'], 'Display', None],
  [ 0x21d5, 'UpDownDoubleArrow', [], 'Display', None],
  [ 0x21d6, 'NorthWestDoubleArrow', [], 'Display', None],
  [ 0x21d7, 'NorthEastDoubleArrow', [], 'Display', None],
  [ 0x21d8, 'SouthEastDoubleArrow', [], 'Display', None],
  [ 0x21d9, 'SouthWestDoubleArrow', [], 'Display', None],
  [ 0x21da, 'LeftwardsTripleArrow', ['<==='], 'Display', None],
  [ 0x21db, 'RightwardsTripleArrow', ['===>'], 'Display', None],
  [ 0x21dc, 'LeftwardsSquiggleArrow', [], 'Display', None],
  [ 0x21dd, 'RightwardsSquiggleArrow', [], 'Display', None],
  [ 0x21de, 'UpwardsArrowWithDoubleStroke', [], 'Display', None],
  [ 0x21df, 'DownwardsArrowWithDoubleStroke', [], 'Display', None],
  [ 0x21e0, 'LeftwardsDashedArrow', [], 'Display', None],
  [ 0x21e1, 'UpwardsDashedArrow', [], 'Display', None],
  [ 0x21e2, 'RightwardsDashedArrow', [], 'Display', None],
  [ 0x21e3, 'DownwardsDashedArrow', [], 'Display', None],
  [ 0x21e4, 'LeftwardsArrowToBar', [], 'Display', None],
  [ 0x21e5, 'RightwardsArrowToBar', [], 'Display', None],
  [ 0x21e6, 'LeftwardsWhiteArrow', [], 'Display', None],
  [ 0x21e7, 'UpwardsWhiteArrow', [], 'Display', None],
  [ 0x21e8, 'RightwardsWhiteArrow', [], 'Display', None],
  [ 0x21e9, 'DownwardsWhiteArrow', [], 'Display', None],
  [ 0x21ea, 'UpwardsWhiteArrowFromBar', [], 'Display', None],
  [ 0x21eb, 'UpwardsWhiteArrowOnPedestal', [], 'Display', None],
  [ 0x21ec, 'UpwardsWhiteArrowOnPedestalWithHorizontalBar', [], 'Display', None],
  [ 0x21ed, 'UpwardsWhiteArrowOnPedestalWithVerticalBar', [], 'Display', None],
  [ 0x21ee, 'UpwardsWhiteDoubleArrow', [], 'Display', None],
  [ 0x21ef, 'UpwardsWhiteDoubleArrowOnPedestal', [], 'Display', None],
  [ 0x21f0, 'RightwardsWhiteArrowFromWall', [], 'Display', None],
  [ 0x21f1, 'NorthWestArrowToCorner', [], 'Display', None],
  [ 0x21f2, 'SouthEastArrowToCorner', [], 'Display', None],
  [ 0x21f3, 'UpDownWhiteArrow', [], 'Display', None],
  [ 0x21f4, 'RightArrowWithSmallCircle', [], 'Display', None],
  [ 0x21f5, 'DownwardsArrowLeftwardsOfUpwardsArrow', [], 'Display', None],
  [ 0x21f6, 'ThreeRightwardsArrows', [], 'Display', None],
  [ 0x21f7, 'LeftwardsArrowWithVerticalStroke', [], 'Display', None],
  [ 0x21f8, 'RightwardsArrowWithVerticalStroke', [], 'Display', None],
  [ 0x21f9, 'LeftRightArrowWithVerticalStroke', [], 'Display', None],
  [ 0x21fa, 'LeftwardsArrowWithDoubleVerticalStroke', [], 'Display', None],
  [ 0x21fb, 'RightwardsArrowWithDoubleVerticalStroke', [], 'Display', None],
  [ 0x21fc, 'LeftRightArrowWithDoubleVerticalStroke', [], 'Display', None],
  [ 0x21fd, 'LeftwardsOpenheadedArrow', [], 'Display', None],
  [ 0x21fe, 'RightwardsOpenheadedArrow', [], 'Display', None],
  [ 0x21ff, 'LeftRightOpenheadedArrow', [], 'Display', None],
  [ 0x2200, 'ForAll', ['fa','forall'], 'Display', None],
  [ 0x2201, 'Complement', ['complement'], 'Symbol', None],
  [ 0x2202, 'PartialDifferential', ['pd','partial','diff'], 'Symbol', None],
  [ 0x2203, 'ThereExists', ['exists'], 'Symbol', None],
  [ 0x2204, 'ThereDoesNotExist', ['nexists','notexists'], 'Symbol', None],
  [ 0x2205, 'EmptySet', ['emptyset','varnothing'], 'Symbol', None],
  [ 0x2206, 'Increment', ['increment'], 'Symbol', None],
  [ 0x2207, 'Nabla', ['del','nabla'], 'Symbol', None],
  [ 0x2208, 'ElementOf', ['elementof','elem','el'], 'InfixOperator', '|'],
  [ 0x2209, 'NotAnElementOf', ['notelementof','notelem'], 'InfixOperator', '|'],
  [ 0x220a, 'SmallElementOf', [], 'Display', None],
  [ 0x220b, 'ContainsAsMember', [], 'Display', None],
  [ 0x220c, 'DoesNotContainAsMember', [], 'Display', None],
  [ 0x220d, 'SmallContainsAsMember', [], 'Display', None],
  [ 0x220e, 'EndOfProof', [], 'Display', None],
  [ 0x220f, 'NaryProduct', ['prod','product'], 'Symbol', None],
  [ 0x2210, 'NaryCoproduct', ['coprod','coproduct'], 'Symbol', None],
  [ 0x2211, 'NarySummation', ['sum','summation'], 'Symbol', None],
  [ 0x2212, 'MinusSign', [], 'Display', None],
  [ 0x2213, 'MinusorplusSign', ['mp','-+','-/+'], 'Display', None],
  [ 0x2214, 'DotPlus', ['.+','dotplus'], 'InfixOperator', '+'],
  [ 0x2215, 'DivisionSlash', [], 'Display', None],
  [ 0x2216, 'SetMinus', [], 'Display', None],
  [ 0x2217, 'AsteriskOperator', [], 'Display', None],
  [ 0x2218, 'RingOperator', ['ring'], 'InfixOperator', '*'],
  [ 0x2219, 'BulletOperator', ['bullet'], 'InfixOperator', '*'],
  [ 0x221a, 'SquareRoot', ['squareroot','sqrt'], 'Symbol', None],
  [ 0x221b, 'CubeRoot', ['cuberoot','3rt'], 'Symbol', None],
  [ 0x221c, 'FourthRoot', ['fourthroot','4rt'], 'Symbol', None],
  [ 0x221d, 'ProportionalTo', [], 'Display', None],
  [ 0x221e, 'Infinity', ['infty','inf','infinity','oo'], 'Symbol', None],
  [ 0x221f, 'RightAngle', [], 'Display', None],
  [ 0x2220, 'Angle', [], 'Display', None],
  [ 0x2221, 'MeasuredAngle', [], 'Display', None],
  [ 0x2222, 'SphericalAngle', [], 'Display', None],
  [ 0x2223, 'Divides', [], 'Display', None],
  [ 0x2224, 'DoesNotDivide', [], 'Display', None],
  [ 0x2225, 'ParallelTo', [], 'Display', None],
  [ 0x2226, 'NotParallelTo', [], 'Display', None],
  [ 0x2227, 'LogicalAnd', ['/\\','and'], 'SimpleSubstitution', ' and '],
  [ 0x2228, 'LogicalOr', ['\\/','or'], 'SimpleSubstitution', ' or '],
  [ 0x2229, 'Intersection', ['intersect','inter'], 'InfixOperator', '|'],
  [ 0x222a, 'Union', ['union','un'], 'InfixOperator', '|'],
  [ 0x222b, 'Integral', ['int','integral'], 'Symbol', None],
  [ 0x222c, 'DoubleIntegral', ['iint','doubleintegral'], 'Symbol', None],
  [ 0x222d, 'TripleIntegral', ['iiint','tripleintegral'], 'Symbol', None],
  [ 0x222e, 'ContourIntegral', ['oint','contourintegral'], 'Symbol', None],
  [ 0x222f, 'SurfaceIntegral', ['oiint','surfaceintegral'], 'Symbol', None],
  [ 0x2230, 'VolumeIntegral', ['oiiint','volumeintegral'], 'Symbol', None],
  [ 0x2231, 'ClockwiseIntegral', ['clockwiseintegral','cwint'], 'Symbol', None],
  [ 0x2232, 'ClockwiseContourIntegral', ['counterclockwiseintegral','ccwint'], 'Symbol', None],
  [ 0x2233, 'AnticlockwiseContourIntegral', [], 'Symbol', None],
  [ 0x2234, 'Therefore', [], 'Display', None],
  [ 0x2235, 'Because', [], 'Display', None],
  [ 0x2236, 'Ratio', [], 'Display', None],
  [ 0x2237, 'Proportion', [], 'Display', None],
  [ 0x2238, 'DotMinus', ['.-','dotminus'], 'InfixOperator', '+'],
  [ 0x2239, 'Excess', [], 'Display', None],
  [ 0x223a, 'GeometricProportion', [], 'Display', None],
  [ 0x223b, 'Homothetic', [], 'Display', None],
  [ 0x223c, 'TildeOperator', [], 'Display', None],
  [ 0x223d, 'ReversedTilde', [], 'Display', None],
  [ 0x223e, 'InvertedLazyS', [], 'Display', None],
  [ 0x223f, 'SineWave', [], 'Display', None],
  [ 0x2240, 'WreathProduct', [], 'Display', None],
  [ 0x2241, 'NotTilde', [], 'Display', None],
  [ 0x2242, 'MinusTilde', [], 'Display', None],
  [ 0x2243, 'AsymptoticallyEqualTo', ['~='], 'Display', None],
  [ 0x2244, 'NotAsymptoticallyEqualTo', ['!~='], 'Display', None],
  [ 0x2245, 'ApproximatelyEqualTo', ['~=='], 'Display', None],
  [ 0x2246, 'ApproximatelyButNotActuallyEqualTo', ['~!=='], 'Display', None],
  [ 0x2247, 'NeitherApproximatelyNorActuallyEqualTo', ['!~=='], 'Display', None],
  [ 0x2248, 'AlmostEqualTo', ['~~'], 'Display', None],
  [ 0x2249, 'NotAlmostEqualTo', ['!~~'], 'Display', None],
  [ 0x224a, 'AlmostEqualOrEqualTo', ['~~='], 'Display', None],
  [ 0x224b, 'TripleTilde', ['~~~'], 'Display', None],
  [ 0x224c, 'AllEqualTo', [], 'Display', None],
  [ 0x224d, 'EquivalentTo', [], 'Display', None],
  [ 0x224e, 'GeometricallyEquivalentTo', [], 'Display', None],
  [ 0x224f, 'DifferenceBetween', [], 'Display', None],
  [ 0x2250, 'ApproachesTheLimit', [], 'Display', None],
  [ 0x2251, 'GeometricallyEqualTo', [], 'Display', None],
  [ 0x2252, 'ApproximatelyEqualToOrTheImageOf', [], 'Display', None],
  [ 0x2253, 'ImageOfOrApproximatelyEqualTo', [], 'Display', None],
  [ 0x2254, 'ColonEquals', [':='], 'Display', None],
  [ 0x2255, 'EqualsColon', ['=:'], 'Display', None],
  [ 0x2256, 'RingInEqualTo', [], 'Display', None],
  [ 0x2257, 'RingEqualTo', [], 'Display', None],
  [ 0x2258, 'CorrespondsTo', [], 'Display', None],
  [ 0x2259, 'Estimates', [], 'Display', None],
  [ 0x225a, 'EquiangularTo', [], 'Display', None],
  [ 0x225b, 'StarEquals', [], 'Display', None],
  [ 0x225c, 'DeltaEqualTo', [], 'Display', None],
  [ 0x225d, 'EqualToByDefinition', [], 'Display', None],
  [ 0x225e, 'MeasuredBy', [], 'Display', None],
  [ 0x225f, 'QuestionedEqualTo', [], 'Display', None],
  [ 0x2260, 'NotEqualTo', ['notequal','!='], 'SimpleSubstitution', '!='],
  [ 0x2261, 'IdenticalTo', [], 'Display', None],
  [ 0x2262, 'NotIdenticalTo', [], 'Display', None],
  [ 0x2263, 'StrictlyEquivalentTo', [], 'Display', None],
  [ 0x2264, 'LessthanOrEqualTo', ['lessthanequal', '<='], 'SimpleSubstitution', '<='],
  [ 0x2265, 'GreaterthanOrEqualTo', ['greaterthanequal', '>=' ], 'SimpleSubstitution', '>='],
  [ 0x2266, 'LessthanOverEqualTo', [], 'Display', None],
  [ 0x2267, 'GreaterthanOverEqualTo', [], 'Display', None],
  [ 0x2268, 'LessthanButNotEqualTo', [], 'Display', None],
  [ 0x2269, 'GreaterthanButNotEqualTo', [], 'Display', None],
  [ 0x226a, 'MuchLessthan', [], 'Display', None],
  [ 0x226b, 'MuchGreaterthan', [], 'Display', None],
  [ 0x226c, 'Between', [], 'Display', None],
  [ 0x226d, 'NotEquivalentTo', [], 'Display', None],
  [ 0x226e, 'NotLessthan', [], 'Display', None],
  [ 0x226f, 'NotGreaterthan', [], 'Display', None],
  [ 0x2270, 'NeitherLessthanNorEqualTo', [], 'Display', None],
  [ 0x2271, 'NeitherGreaterthanNorEqualTo', [], 'Display', None],
  [ 0x2272, 'LessthanOrEquivalentTo', [], 'Display', None],
  [ 0x2273, 'GreaterthanOrEquivalentTo', [], 'Display', None],
  [ 0x2274, 'NeitherLessthanNorEquivalentTo', [], 'Display', None],
  [ 0x2275, 'NeitherGreaterthanNorEquivalentTo', [], 'Display', None],
  [ 0x2276, 'LessthanOrGreaterthan', [], 'Display', None],
  [ 0x2277, 'GreaterthanOrLessthan', [], 'Display', None],
  [ 0x2278, 'NeitherLessthanNorGreaterthan', [], 'Display', None],
  [ 0x2279, 'NeitherGreaterthanNorLessthan', [], 'Display', None],
  [ 0x227a, 'Precedes', [], 'Display', None],
  [ 0x227b, 'Succeeds', [], 'Display', None],
  [ 0x227c, 'PrecedesOrEqualTo', [], 'Display', None],
  [ 0x227d, 'SucceedsOrEqualTo', [], 'Display', None],
  [ 0x227e, 'PrecedesOrEquivalentTo', [], 'Display', None],
  [ 0x227f, 'SucceedsOrEquivalentTo', [], 'Display', None],
  [ 0x2280, 'DoesNotPrecede', [], 'Display', None],
  [ 0x2281, 'DoesNotSucceed', [], 'Display', None],
  [ 0x2282, 'SubsetOf', [], 'Display', None],
  [ 0x2283, 'SupersetOf', [], 'Display', None],
  [ 0x2284, 'NotASubsetOf', [], 'Display', None],
  [ 0x2285, 'NotASupersetOf', [], 'Display', None],
  [ 0x2286, 'SubsetOfOrEqualTo', [], 'Display', None],
  [ 0x2287, 'SupersetOfOrEqualTo', [], 'Display', None],
  [ 0x2288, 'NeitherASubsetOfNorEqualTo', [], 'Display', None],
  [ 0x2289, 'NeitherASupersetOfNorEqualTo', [], 'Display', None],
  [ 0x228a, 'SubsetOfWithNotEqualTo', [], 'Display', None],
  [ 0x228b, 'SupersetOfWithNotEqualTo', [], 'Display', None],
  [ 0x228c, 'Multiset', [], 'Display', None],
  [ 0x228d, 'MultisetMultiplication', [], 'Display', None],
  [ 0x228e, 'MultisetUnion', [], 'Display', None],
  [ 0x228f, 'SquareImageOf', [], 'Display', None],
  [ 0x2290, 'SquareOriginalOf', [], 'Display', None],
  [ 0x2291, 'SquareImageOfOrEqualTo', [], 'Display', None],
  [ 0x2292, 'SquareOriginalOfOrEqualTo', [], 'Display', None],
  [ 0x2293, 'SquareCap', [], 'Display', None],
  [ 0x2294, 'SquareCup', [], 'Display', None],
  [ 0x2295, 'CircledPlus', ['c+'], 'InfixOperator', '+'],
  [ 0x2296, 'CircledMinus', ['c-'], 'InfixOperator', '+'],
  [ 0x2297, 'CircledTimes', ['c*'], 'InfixOperator', '*'],
  [ 0x2298, 'CircledDivisionSlash', ['c/'], 'InfixOperator', '*'],
  [ 0x2299, 'CircledDotOperator', ['c.'], 'InfixOperator', '*'],
  [ 0x229a, 'CircledRingOperator', [], 'Display', None],
  [ 0x229b, 'CircledAsteriskOperator', [], 'Display', None],
  [ 0x229c, 'CircledEquals', [], 'Display', None],
  [ 0x229d, 'CircledDash', [], 'Display', None],
  [ 0x229e, 'SquaredPlus', [], 'Display', None],
  [ 0x229f, 'SquaredMinus', [], 'Display', None],
  [ 0x22a0, 'SquaredTimes', [], 'Display', None],
  [ 0x22a1, 'SquaredDotOperator', [], 'Display', None],
  [ 0x22a2, 'RightTack', [], 'Display', None],
  [ 0x22a3, 'LeftTack', [], 'Display', None],
  [ 0x22a4, 'DownTack', [], 'Display', None],
  [ 0x22a5, 'UpTack', [], 'Display', None],
  [ 0x22a6, 'Assertion', [], 'Display', None],
  [ 0x22a7, 'Models', [], 'Display', None],
  [ 0x22a8, 'True', [], 'Display', None],
  [ 0x22a9, 'Forces', [], 'Display', None],
  [ 0x22aa, 'TripleVerticalBarRightTurnstile', [], 'Display', None],
  [ 0x22ab, 'DoubleVerticalBarDoubleRightTurnstile', [], 'Display', None],
  [ 0x22ac, 'DoesNotProve', [], 'Display', None],
  [ 0x22ad, 'NotTrue', [], 'Display', None],
  [ 0x22ae, 'DoesNotForce', [], 'Display', None],
  [ 0x22af, 'NegatedDoubleVerticalBarDoubleRightTurnstile', [], 'Display', None],
  [ 0x22b0, 'PrecedesUnderRelation', [], 'Display', None],
  [ 0x22b1, 'SucceedsUnderRelation', [], 'Display', None],
  [ 0x22b2, 'NormalSubgroupOf', [], 'Display', None],
  [ 0x22b3, 'ContainsAsNormalSubgroup', [], 'Display', None],
  [ 0x22b4, 'NormalSubgroupOfOrEqualTo', [], 'Display', None],
  [ 0x22b5, 'ContainsAsNormalSubgroupOrEqualTo', [], 'Display', None],
  [ 0x22b6, 'OriginalOf', [], 'Display', None],
  [ 0x22b7, 'ImageOf', [], 'Display', None],
  [ 0x22b8, 'Multimap', [], 'Display', None],
  [ 0x22b9, 'HermitianConjugateMatrix', [], 'Display', None],
  [ 0x22ba, 'Intercalate', [], 'Display', None],
  [ 0x22bb, 'Xor', [], 'Display', None],
  [ 0x22bc, 'Nand', [], 'Display', None],
  [ 0x22bd, 'Nor', [], 'Display', None],
  [ 0x22be, 'RightAngleWithArc', [], 'Display', None],
  [ 0x22bf, 'RightTriangle', [], 'Display', None],
  [ 0x22c0, 'NaryLogicalAnd', [], 'Display', None],
  [ 0x22c1, 'NaryLogicalOr', [], 'Display', None],
  [ 0x22c2, 'NaryIntersection', [], 'Display', None],
  [ 0x22c3, 'NaryUnion', [], 'Display', None],
  [ 0x22c4, 'DiamondOperator', [], 'Display', None],
  [ 0x22c5, 'DotOperator', ['dot','.'], 'InfixOperator', '*'],
  [ 0x22c6, 'StarOperator', ['star'], 'InfixOperator', '*'],
  [ 0x22c7, 'DivisionTimes', ['/*'], 'InfixOperator', '*'],
  [ 0x22c8, 'Bowtie', [], 'Display', None],
  [ 0x22c9, 'LeftNormalFactorSemidirectProduct', [], 'Display', None],
  [ 0x22ca, 'RightNormalFactorSemidirectProduct', [], 'Display', None],
  [ 0x22cb, 'LeftSemidirectProduct', [], 'Display', None],
  [ 0x22cc, 'RightSemidirectProduct', [], 'Display', None],
  [ 0x22cd, 'ReversedTildeEquals', [], 'Display', None],
  [ 0x22ce, 'CurlyLogicalOr', [], 'Display', None],
  [ 0x22cf, 'CurlyLogicalAnd', [], 'Display', None],
  [ 0x22d0, 'DoubleSubset', [], 'Display', None],
  [ 0x22d1, 'DoubleSuperset', [], 'Display', None],
  [ 0x22d2, 'DoubleIntersection', [], 'Display', None],
  [ 0x22d3, 'DoubleUnion', [], 'Display', None],
  [ 0x22d4, 'Pitchfork', [], 'Display', None],
  [ 0x22d5, 'EqualAndParallelTo', [], 'Display', None],
  [ 0x22d6, 'LessthanWithDot', [], 'Display', None],
  [ 0x22d7, 'GreaterthanWithDot', [], 'Display', None],
  [ 0x22d8, 'VeryMuchLessthan', [], 'Display', None],
  [ 0x22d9, 'VeryMuchGreaterthan', [], 'Display', None],
  [ 0x22da, 'LessthanEqualToOrGreaterthan', [], 'Display', None],
  [ 0x22db, 'GreaterthanEqualToOrLessthan', [], 'Display', None],
  [ 0x22dc, 'EqualToOrLessthan', [], 'Display', None],
  [ 0x22dd, 'EqualToOrGreaterthan', [], 'Display', None],
  [ 0x22de, 'EqualToOrPrecedes', [], 'Display', None],
  [ 0x22df, 'EqualToOrSucceeds', [], 'Display', None],
  [ 0x22e0, 'DoesNotPrecedeOrEqual', [], 'Display', None],
  [ 0x22e1, 'DoesNotSucceedOrEqual', [], 'Display', None],
  [ 0x22e2, 'NotSquareImageOfOrEqualTo', [], 'Display', None],
  [ 0x22e3, 'NotSquareOriginalOfOrEqualTo', [], 'Display', None],
  [ 0x22e4, 'SquareImageOfOrNotEqualTo', [], 'Display', None],
  [ 0x22e5, 'SquareOriginalOfOrNotEqualTo', [], 'Display', None],
  [ 0x22e6, 'LessthanButNotEquivalentTo', [], 'Display', None],
  [ 0x22e7, 'GreaterthanButNotEquivalentTo', [], 'Display', None],
  [ 0x22e8, 'PrecedesButNotEquivalentTo', [], 'Display', None],
  [ 0x22e9, 'SucceedsButNotEquivalentTo', [], 'Display', None],
  [ 0x22ea, 'NotNormalSubgroupOf', [], 'Display', None],
  [ 0x22eb, 'DoesNotContainAsNormalSubgroup', [], 'Display', None],
  [ 0x22ec, 'NotNormalSubgroupOfOrEqualTo', [], 'Display', None],
  [ 0x22ed, 'DoesNotContainAsNormalSubgroupOrEqual', [], 'Display', None],
  [ 0x22ee, 'VerticalEllipsis', [], 'SpecialEscapeCharacter', None],
  [ 0x22ef, 'MidlineHorizontalEllipsis', [], 'Display', None],
  [ 0x22f0, 'UpRightDiagonalEllipsis', [], 'Display', None],
  [ 0x22f1, 'DownRightDiagonalEllipsis', [], 'Display', None],
  [ 0x22f2, 'ElementOfWithLongHorizontalStroke', [], 'Display', None],
  [ 0x22f3, 'ElementOfWithVerticalBarAtEndOfHorizontalStroke', [], 'Display', None],
  [ 0x22f4, 'SmallElementOfWithVerticalBarAtEndOfHorizontalStroke', [], 'Display', None],
  [ 0x22f5, 'ElementOfWithDotAbove', [], 'Display', None],
  [ 0x22f6, 'ElementOfWithOverbar', [], 'Display', None],
  [ 0x22f7, 'SmallElementOfWithOverbar', [], 'Display', None],
  [ 0x22f8, 'ElementOfWithUnderbar', [], 'Display', None],
  [ 0x22f9, 'ElementOfWithTwoHorizontalStrokes', [], 'Display', None],
  [ 0x22fa, 'ContainsWithLongHorizontalStroke', [], 'Display', None],
  [ 0x22fb, 'ContainsWithVerticalBarAtEndOfHorizontalStroke', [], 'Display', None],
  [ 0x22fc, 'SmallContainsWithVerticalBarAtEndOfHorizontalStroke', [], 'Display', None],
  [ 0x22fd, 'ContainsWithOverbar', [], 'Display', None],
  [ 0x22fe, 'SmallContainsWithOverbar', [], 'Display', None],
  [ 0x22ff, 'ZNotationBagMembership', [], 'Display', None],
  [ 0x2300, 'DiameterSign', [], 'Display', None],
  [ 0x2301, 'ElectricArrow', [], 'Display', None],
  [ 0x2302, 'House', [], 'Display', None],
  [ 0x2303, 'UpArrowhead', [], 'Display', None],
  [ 0x2304, 'DownArrowhead', [], 'Display', None],
  [ 0x2305, 'Projective', [], 'Display', None],
  [ 0x2306, 'Perspective', [], 'Display', None],
  [ 0x2307, 'WavyLine', [], 'Display', None],
  [ 0x2308, 'LeftCeiling', [], 'Display', None],
  [ 0x2309, 'RightCeiling', [], 'Display', None],
  [ 0x230a, 'LeftFloor', [], 'Display', None],
  [ 0x230b, 'RightFloor', [], 'Display', None],
  [ 0x230c, 'BottomRightCrop', [], 'Display', None],
  [ 0x230d, 'BottomLeftCrop', [], 'Display', None],
  [ 0x230e, 'TopRightCrop', [], 'Display', None],
  [ 0x230f, 'TopLeftCrop', [], 'Display', None],
  [ 0x2310, 'ReversedNotSign', ['invnot'], 'Symbol', None],
  [ 0x2311, 'SquareLozenge', [], 'Display', None],
  [ 0x2312, 'Arc', [], 'Display', None],
  [ 0x2313, 'Segment', [], 'Display', None],
  [ 0x2314, 'Sector', [], 'Display', None],
  [ 0x2315, 'TelephoneRecorder', [], 'Display', None],
  [ 0x2316, 'PositionIndicator', [], 'Display', None],
  [ 0x2317, 'ViewdataSquare', [], 'Display', None],
  [ 0x2318, 'PlaceOfInterestSign', [], 'Display', None],
  [ 0x2319, 'TurnedNotSign', ['turnednot'], 'Symbol', None],
  [ 0x231a, 'Watch', [], 'Display', None],
  [ 0x231b, 'Hourglass', [], 'Display', None],
  [ 0x231c, 'TopLeftCorner', [], 'Display', None],
  [ 0x231d, 'TopRightCorner', [], 'Display', None],
  [ 0x231e, 'BottomLeftCorner', [], 'Display', None],
  [ 0x231f, 'BottomRightCorner', [], 'Display', None],
  [ 0x2320, 'TopHalfIntegral', [], 'Display', None],
  [ 0x2321, 'BottomHalfIntegral', [], 'Display', None],
  [ 0x2322, 'Frown', [':(',':-('], 'Display', None],
  [ 0x2323, 'Smile', [':)',':-)'], 'Display', None],
  [ 0x2324, 'UpArrowheadBetweenTwoHorizontalBars', [], 'Display', None],
  [ 0x2325, 'OptionKey', [], 'Display', None],
  [ 0x2326, 'EraseToTheRight', [], 'Display', None],
  [ 0x2327, 'XInARectangleBox', [], 'Display', None],
  [ 0x2328, 'Keyboard', [], 'Display', None],
  [ 0x2329, 'LeftpointingAngleBracket', [], 'Display', None],
  [ 0x232a, 'RightpointingAngleBracket', [], 'Display', None],
  [ 0x232b, 'EraseToTheLeft', [], 'Display', None],
  [ 0x232c, 'BenzeneRing', [], 'Display', None],
  [ 0x232d, 'Cylindricity', [], 'Display', None],
  [ 0x232e, 'AllAroundprofile', [], 'Display', None],
  [ 0x232f, 'Symmetry', [], 'Display', None],
  [ 0x2330, 'TotalRunout', [], 'Display', None],
  [ 0x2331, 'DimensionOrigin', [], 'Display', None],
  [ 0x2332, 'ConicalTaper', [], 'Display', None],
  [ 0x2333, 'Slope', [], 'Display', None],
  [ 0x2334, 'Counterbore', [], 'Display', None],
  [ 0x2335, 'Countersink', [], 'Display', None],
  [ 0x2336, 'AplFunctionalSymbolIbeam', [], 'Display', None],
  [ 0x2337, 'AplFunctionalSymbolSquishQuad', [], 'Display', None],
  [ 0x2338, 'AplFunctionalSymbolQuadEqual', [], 'Display', None],
  [ 0x2339, 'AplFunctionalSymbolQuadDivide', [], 'Display', None],
  [ 0x233a, 'AplFunctionalSymbolQuadDiamond', [], 'Display', None],
  [ 0x233b, 'AplFunctionalSymbolQuadJot', [], 'Display', None],
  [ 0x233c, 'AplFunctionalSymbolQuadCircle', [], 'Display', None],
  [ 0x233d, 'AplFunctionalSymbolCircleStile', [], 'Display', None],
  [ 0x233e, 'AplFunctionalSymbolCircleJot', [], 'Display', None],
  [ 0x233f, 'AplFunctionalSymbolSlashBar', [], 'Display', None],
  [ 0x2340, 'AplFunctionalSymbolBackslashBar', [], 'Display', None],
  [ 0x2341, 'AplFunctionalSymbolQuadSlash', [], 'Display', None],
  [ 0x2342, 'AplFunctionalSymbolQuadBackslash', [], 'Display', None],
  [ 0x2343, 'AplFunctionalSymbolQuadLessthan', [], 'Display', None],
  [ 0x2344, 'AplFunctionalSymbolQuadGreaterthan', [], 'Display', None],
  [ 0x2345, 'AplFunctionalSymbolLeftwardsVane', [], 'Display', None],
  [ 0x2346, 'AplFunctionalSymbolRightwardsVane', [], 'Display', None],
  [ 0x2347, 'AplFunctionalSymbolQuadLeftwardsArrow', [], 'Display', None],
  [ 0x2348, 'AplFunctionalSymbolQuadRightwardsArrow', [], 'Display', None],
  [ 0x2349, 'AplFunctionalSymbolCircleBackslash', [], 'Display', None],
  [ 0x234a, 'AplFunctionalSymbolDownTackUnderbar', [], 'Display', None],
  [ 0x234b, 'AplFunctionalSymbolDeltaStile', [], 'Display', None],
  [ 0x234c, 'AplFunctionalSymbolQuadDownCaret', [], 'Display', None],
  [ 0x234d, 'AplFunctionalSymbolQuadDelta', [], 'Display', None],
  [ 0x234e, 'AplFunctionalSymbolDownTackJot', [], 'Display', None],
  [ 0x234f, 'AplFunctionalSymbolUpwardsVane', [], 'Display', None],
  [ 0x2350, 'AplFunctionalSymbolQuadUpwardsArrow', [], 'Display', None],
  [ 0x2351, 'AplFunctionalSymbolUpTackOverbar', [], 'Display', None],
  [ 0x2352, 'AplFunctionalSymbolDelStile', [], 'Display', None],
  [ 0x2353, 'AplFunctionalSymbolQuadUpCaret', [], 'Display', None],
  [ 0x2354, 'AplFunctionalSymbolQuadDel', [], 'Display', None],
  [ 0x2355, 'AplFunctionalSymbolUpTackJot', [], 'Display', None],
  [ 0x2356, 'AplFunctionalSymbolDownwardsVane', [], 'Display', None],
  [ 0x2357, 'AplFunctionalSymbolQuadDownwardsArrow', [], 'Display', None],
  [ 0x2358, 'AplFunctionalSymbolQuoteUnderbar', [], 'Display', None],
  [ 0x2359, 'AplFunctionalSymbolDeltaUnderbar', [], 'Display', None],
  [ 0x235a, 'AplFunctionalSymbolDiamondUnderbar', [], 'Display', None],
  [ 0x235b, 'AplFunctionalSymbolJotUnderbar', [], 'Display', None],
  [ 0x235c, 'AplFunctionalSymbolCircleUnderbar', [], 'Display', None],
  [ 0x235d, 'AplFunctionalSymbolUpShoeJot', [], 'Display', None],
  [ 0x235e, 'AplFunctionalSymbolQuoteQuad', [], 'Display', None],
  [ 0x235f, 'AplFunctionalSymbolCircleStar', [], 'Display', None],
  [ 0x2360, 'AplFunctionalSymbolQuadColon', [], 'Display', None],
  [ 0x2361, 'AplFunctionalSymbolUpTackDiaeresis', [], 'Display', None],
  [ 0x2362, 'AplFunctionalSymbolDelDiaeresis', [], 'Display', None],
  [ 0x2363, 'AplFunctionalSymbolStarDiaeresis', [], 'Display', None],
  [ 0x2364, 'AplFunctionalSymbolJotDiaeresis', [], 'Display', None],
  [ 0x2365, 'AplFunctionalSymbolCircleDiaeresis', [], 'Display', None],
  [ 0x2366, 'AplFunctionalSymbolDownShoeStile', [], 'Display', None],
  [ 0x2367, 'AplFunctionalSymbolLeftShoeStile', [], 'Display', None],
  [ 0x2368, 'AplFunctionalSymbolTildeDiaeresis', [], 'Display', None],
  [ 0x2369, 'AplFunctionalSymbolGreaterthanDiaeresis', [], 'Display', None],
  [ 0x236a, 'AplFunctionalSymbolCommaBar', [], 'Display', None],
  [ 0x236b, 'AplFunctionalSymbolDelTilde', [], 'Display', None],
  [ 0x236c, 'AplFunctionalSymbolZilde', [], 'Display', None],
  [ 0x236d, 'AplFunctionalSymbolStileTilde', [], 'Display', None],
  [ 0x236e, 'AplFunctionalSymbolSemicolonUnderbar', [], 'Display', None],
  [ 0x236f, 'AplFunctionalSymbolQuadNotEqual', [], 'Display', None],
  [ 0x2370, 'AplFunctionalSymbolQuadQuestion', [], 'Display', None],
  [ 0x2371, 'AplFunctionalSymbolDownCaretTilde', [], 'Display', None],
  [ 0x2372, 'AplFunctionalSymbolUpCaretTilde', [], 'Display', None],
  [ 0x2373, 'AplFunctionalSymbolIota', [], 'Display', None],
  [ 0x2374, 'AplFunctionalSymbolRho', [], 'Display', None],
  [ 0x2375, 'AplFunctionalSymbolOmega', [], 'Display', None],
  [ 0x2376, 'AplFunctionalSymbolAlphaUnderbar', [], 'Display', None],
  [ 0x2377, 'AplFunctionalSymbolEpsilonUnderbar', [], 'Display', None],
  [ 0x2378, 'AplFunctionalSymbolIotaUnderbar', [], 'Display', None],
  [ 0x2379, 'AplFunctionalSymbolOmegaUnderbar', [], 'Display', None],
  [ 0x237a, 'AplFunctionalSymbolAlpha', [], 'Display', None],
  [ 0x237b, 'NotCheckMark', [], 'Display', None],
  [ 0x237c, 'RightAngleWithDownwardsZigzagArrow', [], 'Display', None],
  [ 0x237d, 'ShoulderedOpenBox', [], 'Display', None],
  [ 0x237e, 'BellSymbol', [], 'Display', None],
  [ 0x237f, 'VerticalLineWithMiddleDot', [], 'Display', None],
  [ 0x2380, 'InsertionSymbol', [], 'Display', None],
  [ 0x2381, 'ContinuousUnderlineSymbol', [], 'Display', None],
  [ 0x2382, 'DiscontinuousUnderlineSymbol', [], 'Display', None],
  [ 0x2383, 'EmphasisSymbol', [], 'Display', None],
  [ 0x2384, 'CompositionSymbol', [], 'Display', None],
  [ 0x2385, 'WhiteSquareWithCentreVerticalLine', [], 'Display', None],
  [ 0x2386, 'EnterSymbol', [], 'Display', None],
  [ 0x2387, 'AlternativeKeySymbol', [], 'Display', None],
  [ 0x2388, 'HelmSymbol', [], 'Display', None],
  [ 0x2389, 'CircledHorizontalBarWithNotch', [], 'Display', None],
  [ 0x238a, 'CircledTriangleDown', [], 'Display', None],
  [ 0x238b, 'BrokenCircleWithNorthwestArrow', [], 'Display', None],
  [ 0x238c, 'UndoSymbol', [], 'Display', None],
  [ 0x238d, 'MonostableSymbol', [], 'Display', None],
  [ 0x238e, 'HysteresisSymbol', [], 'Display', None],
  [ 0x238f, 'OpencircuitoutputHtypeSymbol', [], 'Display', None],
  [ 0x2390, 'OpencircuitoutputLtypeSymbol', [], 'Display', None],
  [ 0x2391, 'PassivepulldownoutputSymbol', [], 'Display', None],
  [ 0x2392, 'PassivepullupoutputSymbol', [], 'Display', None],
  [ 0x2393, 'DirectCurrentSymbolFormTwo', [], 'Display', None],
  [ 0x2394, 'SoftwarefunctionSymbol', [], 'Display', None],
  [ 0x2395, 'AplFunctionalSymbolQuad', [], 'Display', None],
  [ 0x2396, 'DecimalSeparatorKeySymbol', [], 'Display', None],
  [ 0x2397, 'PreviousPage', [], 'Display', None],
  [ 0x2398, 'NextPage', [], 'Display', None],
  [ 0x2399, 'PrintScreenSymbol', [], 'Display', None],
  [ 0x239a, 'ClearScreenSymbol', [], 'Display', None],
  [ 0x239b, 'LeftParenthesisUpperHook', [], 'Display', None],
  [ 0x239c, 'LeftParenthesisExtension', [], 'Display', None],
  [ 0x239d, 'LeftParenthesisLowerHook', [], 'Display', None],
  [ 0x239e, 'RightParenthesisUpperHook', [], 'Display', None],
  [ 0x239f, 'RightParenthesisExtension', [], 'Display', None],
  [ 0x23a0, 'RightParenthesisLowerHook', [], 'Display', None],
  [ 0x23a1, 'LeftSquareBracketUpperCorner', [], 'Display', None],
  [ 0x23a2, 'LeftSquareBracketExtension', [], 'Display', None],
  [ 0x23a3, 'LeftSquareBracketLowerCorner', [], 'Display', None],
  [ 0x23a4, 'RightSquareBracketUpperCorner', [], 'Display', None],
  [ 0x23a5, 'RightSquareBracketExtension', [], 'Display', None],
  [ 0x23a6, 'RightSquareBracketLowerCorner', [], 'Display', None],
  [ 0x23a7, 'LeftCurlyBracketUpperHook', [], 'Display', None],
  [ 0x23a8, 'LeftCurlyBracketMiddlePiece', [], 'Display', None],
  [ 0x23a9, 'LeftCurlyBracketLowerHook', [], 'Display', None],
  [ 0x23aa, 'CurlyBracketExtension', [], 'Display', None],
  [ 0x23ab, 'RightCurlyBracketUpperHook', [], 'Display', None],
  [ 0x23ac, 'RightCurlyBracketMiddlePiece', [], 'Display', None],
  [ 0x23ad, 'RightCurlyBracketLowerHook', [], 'Display', None],
  [ 0x23ae, 'IntegralExtension', [], 'Display', None],
  [ 0x23af, 'HorizontalLineExtension', [], 'Display', None],
  [ 0x23b0, 'UpperLeftOrLowerRightCurlyBracketSection', [], 'Display', None],
  [ 0x23b1, 'UpperRightOrLowerLeftCurlyBracketSection', [], 'Display', None],
  [ 0x23b2, 'SummationTop', [], 'Display', None],
  [ 0x23b3, 'SummationBottom', [], 'Display', None],
  [ 0x23b4, 'TopSquareBracket', [], 'Display', None],
  [ 0x23b5, 'BottomSquareBracket', [], 'Display', None],
  [ 0x23b6, 'BottomSquareBracketOverTopSquareBracket', [], 'Display', None],
  [ 0x23b7, 'RadicalSymbolBottom', [], 'Display', None],
  [ 0x23b8, 'LeftVerticalBoxLine', [], 'Display', None],
  [ 0x23b9, 'RightVerticalBoxLine', [], 'Display', None],
  [ 0x23ba, 'HorizontalScanLine1', [], 'Display', None],
  [ 0x23bb, 'HorizontalScanLine3', [], 'Display', None],
  [ 0x23bc, 'HorizontalScanLine7', [], 'Display', None],
  [ 0x23bd, 'HorizontalScanLine9', [], 'Display', None],
  [ 0x23be, 'DentistrySymbolLightVerticalAndTopRight', [], 'Display', None],
  [ 0x23bf, 'DentistrySymbolLightVerticalAndBottomRight', [], 'Display', None],
  [ 0x23c0, 'DentistrySymbolLightVerticalWithCircle', [], 'Display', None],
  [ 0x23c1, 'DentistrySymbolLightDownAndHorizontalWithCircle', [], 'Display', None],
  [ 0x23c2, 'DentistrySymbolLightUpAndHorizontalWithCircle', [], 'Display', None],
  [ 0x23c3, 'DentistrySymbolLightVerticalWithTriangle', [], 'Display', None],
  [ 0x23c4, 'DentistrySymbolLightDownAndHorizontalWithTriangle', [], 'Display', None],
  [ 0x23c5, 'DentistrySymbolLightUpAndHorizontalWithTriangle', [], 'Display', None],
  [ 0x23c6, 'DentistrySymbolLightVerticalAndWave', [], 'Display', None],
  [ 0x23c7, 'DentistrySymbolLightDownAndHorizontalWithWave', [], 'Display', None],
  [ 0x23c8, 'DentistrySymbolLightUpAndHorizontalWithWave', [], 'Display', None],
  [ 0x23c9, 'DentistrySymbolLightDownAndHorizontal', [], 'Display', None],
  [ 0x23ca, 'DentistrySymbolLightUpAndHorizontal', [], 'Display', None],
  [ 0x23cb, 'DentistrySymbolLightVerticalAndTopLeft', [], 'Display', None],
  [ 0x23cc, 'DentistrySymbolLightVerticalAndBottomLeft', [], 'Display', None],
  [ 0x23cd, 'SquareFoot', [], 'Display', None],
  [ 0x23ce, 'ReturnSymbol', [], 'Display', None],
  [ 0x23cf, 'EjectSymbol', [], 'Display', None],
  [ 0x23d0, 'VerticalLineExtension', [], 'Display', None],
  [ 0x23d1, 'MetricalBreve', [], 'Display', None],
  [ 0x23d2, 'MetricalLongOverShort', [], 'Display', None],
  [ 0x23d3, 'MetricalShortOverLong', [], 'Display', None],
  [ 0x23d4, 'MetricalLongOverTwoShorts', [], 'Display', None],
  [ 0x23d5, 'MetricalTwoShortsOverLong', [], 'Display', None],
  [ 0x23d6, 'MetricalTwoShortsJoined', [], 'Display', None],
  [ 0x23d7, 'MetricalTriseme', [], 'Display', None],
  [ 0x23d8, 'MetricalTetraseme', [], 'Display', None],
  [ 0x23d9, 'MetricalPentaseme', [], 'Display', None],
  [ 0x23da, 'EarthGround', [], 'Display', None],
  [ 0x23db, 'Fuse', [], 'Display', None],
  [ 0x23dc, 'TopParenthesis', [], 'Display', None],
  [ 0x23dd, 'BottomParenthesis', [], 'Display', None],
  [ 0x23de, 'TopCurlyBracket', [], 'Display', None],
  [ 0x23df, 'BottomCurlyBracket', [], 'Display', None],
  [ 0x23e0, 'TopTortoiseShellBracket', [], 'Display', None],
  [ 0x23e1, 'BottomTortoiseShellBracket', [], 'Display', None],
  [ 0x23e2, 'WhiteTrapezium', [], 'Display', None],
  [ 0x23e3, 'BenzeneRingWithCircle', [], 'Display', None],
  [ 0x23e4, 'Straightness', [], 'Display', None],
  [ 0x23e5, 'Flatness', [], 'Display', None],
  [ 0x23e6, 'AcCurrent', [], 'Display', None],
  [ 0x23e7, 'ElectricalIntersection', [], 'Display', None],
  [ 0x23e8, 'UnicodeCharacter0x23e8', [], 'Display', None],
  [ 0x23e9, 'UnicodeCharacter0x23e9', [], 'Display', None],
  [ 0x23ea, 'UnicodeCharacter0x23ea', [], 'Display', None],
  [ 0x23eb, 'UnicodeCharacter0x23eb', [], 'Display', None],
  [ 0x23ec, 'UnicodeCharacter0x23ec', [], 'Display', None],
  [ 0x23ed, 'UnicodeCharacter0x23ed', [], 'Display', None],
  [ 0x23ee, 'UnicodeCharacter0x23ee', [], 'Display', None],
  [ 0x23ef, 'UnicodeCharacter0x23ef', [], 'Display', None],
  [ 0x23f0, 'UnicodeCharacter0x23f0', [], 'Display', None],
  [ 0x23f1, 'UnicodeCharacter0x23f1', [], 'Display', None],
  [ 0x23f2, 'UnicodeCharacter0x23f2', [], 'Display', None],
  [ 0x23f3, 'UnicodeCharacter0x23f3', [], 'Display', None],
  [ 0x23f4, 'UnicodeCharacter0x23f4', [], 'Display', None],
  [ 0x23f5, 'UnicodeCharacter0x23f5', [], 'Display', None],
  [ 0x23f6, 'UnicodeCharacter0x23f6', [], 'Display', None],
  [ 0x23f7, 'UnicodeCharacter0x23f7', [], 'Display', None],
  [ 0x23f8, 'UnicodeCharacter0x23f8', [], 'Display', None],
  [ 0x23f9, 'UnicodeCharacter0x23f9', [], 'Display', None],
  [ 0x23fa, 'UnicodeCharacter0x23fa', [], 'Display', None],
  [ 0x23fb, 'UnicodeCharacter0x23fb', [], 'Display', None],
  [ 0x23fc, 'UnicodeCharacter0x23fc', [], 'Display', None],
  [ 0x23fd, 'UnicodeCharacter0x23fd', [], 'Display', None],
  [ 0x23fe, 'UnicodeCharacter0x23fe', [], 'Display', None],
  [ 0x23ff, 'UnicodeCharacter0x23ff', [], 'Display', None],
  [ 0x2400, 'SymbolForNull', [], 'Display', None],
  [ 0x2401, 'SymbolForStartOfHeading', [], 'Display', None],
  [ 0x2402, 'SymbolForStartOfText', [], 'Display', None],
  [ 0x2403, 'SymbolForEndOfText', [], 'Display', None],
  [ 0x2404, 'SymbolForEndOfTransmission', [], 'Display', None],
  [ 0x2405, 'SymbolForEnquiry', [], 'Display', None],
  [ 0x2406, 'SymbolForAcknowledge', [], 'Display', None],
  [ 0x2407, 'SymbolForBell', [], 'Display', None],
  [ 0x2408, 'SymbolForBackspace', [], 'Display', None],
  [ 0x2409, 'SymbolForHorizontalTabulation', [], 'Display', None],
  [ 0x240a, 'SymbolForLineFeed', [], 'Display', None],
  [ 0x240b, 'SymbolForVerticalTabulation', [], 'Display', None],
  [ 0x240c, 'SymbolForFormFeed', [], 'Display', None],
  [ 0x240d, 'SymbolForCarriageReturn', [], 'Display', None],
  [ 0x240e, 'SymbolForShiftOut', [], 'Display', None],
  [ 0x240f, 'SymbolForShiftIn', [], 'Display', None],
  [ 0x2410, 'SymbolForDataLinkEscape', [], 'Display', None],
  [ 0x2411, 'SymbolForDeviceControlOne', [], 'Display', None],
  [ 0x2412, 'SymbolForDeviceControlTwo', [], 'Display', None],
  [ 0x2413, 'SymbolForDeviceControlThree', [], 'Display', None],
  [ 0x2414, 'SymbolForDeviceControlFour', [], 'Display', None],
  [ 0x2415, 'SymbolForNegativeAcknowledge', [], 'Display', None],
  [ 0x2416, 'SymbolForSynchronousIdle', [], 'Display', None],
  [ 0x2417, 'SymbolForEndOfTransmissionBlock', [], 'Display', None],
  [ 0x2418, 'SymbolForCancel', [], 'Display', None],
  [ 0x2419, 'SymbolForEndOfMedium', [], 'Display', None],
  [ 0x241a, 'SymbolForSubstitute', [], 'Display', None],
  [ 0x241b, 'SymbolForEscape', [], 'Display', None],
  [ 0x241c, 'SymbolForFileSeparator', [], 'Display', None],
  [ 0x241d, 'SymbolForGroupSeparator', [], 'Display', None],
  [ 0x241e, 'SymbolForRecordSeparator', [], 'Display', None],
  [ 0x241f, 'SymbolForUnitSeparator', [], 'Display', None],
  [ 0x2420, 'SymbolForSpace', [], 'Display', None],
  [ 0x2421, 'SymbolForDelete', [], 'Display', None],
  [ 0x2422, 'BlankSymbol', [], 'Display', None],
  [ 0x2423, 'OpenBox', [], 'Display', None],
  [ 0x2424, 'SymbolForNewline', [], 'Display', None],
  [ 0x2425, 'SymbolForDeleteFormTwo', [], 'Display', None],
  [ 0x2426, 'SymbolForSubstituteFormTwo', [], 'Display', None],
  [ 0x2427, 'UnicodeCharacter0x2427', [], 'Display', None],
  [ 0x2428, 'UnicodeCharacter0x2428', [], 'Display', None],
  [ 0x2429, 'UnicodeCharacter0x2429', [], 'Display', None],
  [ 0x242a, 'UnicodeCharacter0x242a', [], 'Display', None],
  [ 0x242b, 'UnicodeCharacter0x242b', [], 'Display', None],
  [ 0x242c, 'UnicodeCharacter0x242c', [], 'Display', None],
  [ 0x242d, 'UnicodeCharacter0x242d', [], 'Display', None],
  [ 0x242e, 'UnicodeCharacter0x242e', [], 'Display', None],
  [ 0x242f, 'UnicodeCharacter0x242f', [], 'Display', None],
  [ 0x2430, 'UnicodeCharacter0x2430', [], 'Display', None],
  [ 0x2431, 'UnicodeCharacter0x2431', [], 'Display', None],
  [ 0x2432, 'UnicodeCharacter0x2432', [], 'Display', None],
  [ 0x2433, 'UnicodeCharacter0x2433', [], 'Display', None],
  [ 0x2434, 'UnicodeCharacter0x2434', [], 'Display', None],
  [ 0x2435, 'UnicodeCharacter0x2435', [], 'Display', None],
  [ 0x2436, 'UnicodeCharacter0x2436', [], 'Display', None],
  [ 0x2437, 'UnicodeCharacter0x2437', [], 'Display', None],
  [ 0x2438, 'UnicodeCharacter0x2438', [], 'Display', None],
  [ 0x2439, 'UnicodeCharacter0x2439', [], 'Display', None],
  [ 0x243a, 'UnicodeCharacter0x243a', [], 'Display', None],
  [ 0x243b, 'UnicodeCharacter0x243b', [], 'Display', None],
  [ 0x243c, 'UnicodeCharacter0x243c', [], 'Display', None],
  [ 0x243d, 'UnicodeCharacter0x243d', [], 'Display', None],
  [ 0x243e, 'UnicodeCharacter0x243e', [], 'Display', None],
  [ 0x243f, 'UnicodeCharacter0x243f', [], 'Display', None],
  [ 0x2440, 'OcrHook', [], 'Display', None],
  [ 0x2441, 'OcrChair', [], 'Display', None],
  [ 0x2442, 'OcrFork', [], 'Display', None],
  [ 0x2443, 'OcrInvertedFork', [], 'Display', None],
  [ 0x2444, 'OcrBeltBuckle', [], 'Display', None],
  [ 0x2445, 'OcrBowTie', [], 'Display', None],
  [ 0x2446, 'OcrBranchBankIdentification', [], 'Display', None],
  [ 0x2447, 'OcrAmountOfCheck', [], 'Display', None],
  [ 0x2448, 'OcrDash', [], 'Display', None],
  [ 0x2449, 'OcrCustomerAccountNumber', [], 'Display', None],
  [ 0x244a, 'OcrDoubleBackslash', [], 'Display', None],
  [ 0x244b, 'UnicodeCharacter0x244b', [], 'Display', None],
  [ 0x244c, 'UnicodeCharacter0x244c', [], 'Display', None],
  [ 0x244d, 'UnicodeCharacter0x244d', [], 'Display', None],
  [ 0x244e, 'UnicodeCharacter0x244e', [], 'Display', None],
  [ 0x244f, 'UnicodeCharacter0x244f', [], 'Display', None],
  [ 0x2450, 'UnicodeCharacter0x2450', [], 'Display', None],
  [ 0x2451, 'UnicodeCharacter0x2451', [], 'Display', None],
  [ 0x2452, 'UnicodeCharacter0x2452', [], 'Display', None],
  [ 0x2453, 'UnicodeCharacter0x2453', [], 'Display', None],
  [ 0x2454, 'UnicodeCharacter0x2454', [], 'Display', None],
  [ 0x2455, 'UnicodeCharacter0x2455', [], 'Display', None],
  [ 0x2456, 'UnicodeCharacter0x2456', [], 'Display', None],
  [ 0x2457, 'UnicodeCharacter0x2457', [], 'Display', None],
  [ 0x2458, 'UnicodeCharacter0x2458', [], 'Display', None],
  [ 0x2459, 'UnicodeCharacter0x2459', [], 'Display', None],
  [ 0x245a, 'UnicodeCharacter0x245a', [], 'Display', None],
  [ 0x245b, 'UnicodeCharacter0x245b', [], 'Display', None],
  [ 0x245c, 'UnicodeCharacter0x245c', [], 'Display', None],
  [ 0x245d, 'UnicodeCharacter0x245d', [], 'Display', None],
  [ 0x245e, 'UnicodeCharacter0x245e', [], 'Display', None],
  [ 0x245f, 'UnicodeCharacter0x245f', [], 'Display', None],
  [ 0x2460, 'CircledDigitOne', [], 'Display', None],
  [ 0x2461, 'CircledDigitTwo', [], 'Display', None],
  [ 0x2462, 'CircledDigitThree', [], 'Display', None],
  [ 0x2463, 'CircledDigitFour', [], 'Display', None],
  [ 0x2464, 'CircledDigitFive', [], 'Display', None],
  [ 0x2465, 'CircledDigitSix', [], 'Display', None],
  [ 0x2466, 'CircledDigitSeven', [], 'Display', None],
  [ 0x2467, 'CircledDigitEight', [], 'Display', None],
  [ 0x2468, 'CircledDigitNine', [], 'Display', None],
  [ 0x2469, 'CircledNumberTen', [], 'Display', None],
  [ 0x246a, 'CircledNumberEleven', [], 'Display', None],
  [ 0x246b, 'CircledNumberTwelve', [], 'Display', None],
  [ 0x246c, 'CircledNumberThirteen', [], 'Display', None],
  [ 0x246d, 'CircledNumberFourteen', [], 'Display', None],
  [ 0x246e, 'CircledNumberFifteen', [], 'Display', None],
  [ 0x246f, 'CircledNumberSixteen', [], 'Display', None],
  [ 0x2470, 'CircledNumberSeventeen', [], 'Display', None],
  [ 0x2471, 'CircledNumberEighteen', [], 'Display', None],
  [ 0x2472, 'CircledNumberNineteen', [], 'Display', None],
  [ 0x2473, 'CircledNumberTwenty', [], 'Display', None],
  [ 0x2474, 'ParenthesizedDigitOne', [], 'Display', None],
  [ 0x2475, 'ParenthesizedDigitTwo', [], 'Display', None],
  [ 0x2476, 'ParenthesizedDigitThree', [], 'Display', None],
  [ 0x2477, 'ParenthesizedDigitFour', [], 'Display', None],
  [ 0x2478, 'ParenthesizedDigitFive', [], 'Display', None],
  [ 0x2479, 'ParenthesizedDigitSix', [], 'Display', None],
  [ 0x247a, 'ParenthesizedDigitSeven', [], 'Display', None],
  [ 0x247b, 'ParenthesizedDigitEight', [], 'Display', None],
  [ 0x247c, 'ParenthesizedDigitNine', [], 'Display', None],
  [ 0x247d, 'ParenthesizedNumberTen', [], 'Display', None],
  [ 0x247e, 'ParenthesizedNumberEleven', [], 'Display', None],
  [ 0x247f, 'ParenthesizedNumberTwelve', [], 'Display', None],
  [ 0x2480, 'ParenthesizedNumberThirteen', [], 'Display', None],
  [ 0x2481, 'ParenthesizedNumberFourteen', [], 'Display', None],
  [ 0x2482, 'ParenthesizedNumberFifteen', [], 'Display', None],
  [ 0x2483, 'ParenthesizedNumberSixteen', [], 'Display', None],
  [ 0x2484, 'ParenthesizedNumberSeventeen', [], 'Display', None],
  [ 0x2485, 'ParenthesizedNumberEighteen', [], 'Display', None],
  [ 0x2486, 'ParenthesizedNumberNineteen', [], 'Display', None],
  [ 0x2487, 'ParenthesizedNumberTwenty', [], 'Display', None],
  [ 0x2488, 'DigitOneFullStop', [], 'Display', None],
  [ 0x2489, 'DigitTwoFullStop', [], 'Display', None],
  [ 0x248a, 'DigitThreeFullStop', [], 'Display', None],
  [ 0x248b, 'DigitFourFullStop', [], 'Display', None],
  [ 0x248c, 'DigitFiveFullStop', [], 'Display', None],
  [ 0x248d, 'DigitSixFullStop', [], 'Display', None],
  [ 0x248e, 'DigitSevenFullStop', [], 'Display', None],
  [ 0x248f, 'DigitEightFullStop', [], 'Display', None],
  [ 0x2490, 'DigitNineFullStop', [], 'Display', None],
  [ 0x2491, 'NumberTenFullStop', [], 'Display', None],
  [ 0x2492, 'NumberElevenFullStop', [], 'Display', None],
  [ 0x2493, 'NumberTwelveFullStop', [], 'Display', None],
  [ 0x2494, 'NumberThirteenFullStop', [], 'Display', None],
  [ 0x2495, 'NumberFourteenFullStop', [], 'Display', None],
  [ 0x2496, 'NumberFifteenFullStop', [], 'Display', None],
  [ 0x2497, 'NumberSixteenFullStop', [], 'Display', None],
  [ 0x2498, 'NumberSeventeenFullStop', [], 'Display', None],
  [ 0x2499, 'NumberEighteenFullStop', [], 'Display', None],
  [ 0x249a, 'NumberNineteenFullStop', [], 'Display', None],
  [ 0x249b, 'NumberTwentyFullStop', [], 'Display', None],
  [ 0x249c, 'ParenthesizedLatinSmallLetterA', [], 'Display', None],
  [ 0x249d, 'ParenthesizedLatinSmallLetterB', [], 'Display', None],
  [ 0x249e, 'ParenthesizedLatinSmallLetterC', [], 'Display', None],
  [ 0x249f, 'ParenthesizedLatinSmallLetterD', [], 'Display', None],
  [ 0x24a0, 'ParenthesizedLatinSmallLetterE', [], 'Display', None],
  [ 0x24a1, 'ParenthesizedLatinSmallLetterF', [], 'Display', None],
  [ 0x24a2, 'ParenthesizedLatinSmallLetterG', [], 'Display', None],
  [ 0x24a3, 'ParenthesizedLatinSmallLetterH', [], 'Display', None],
  [ 0x24a4, 'ParenthesizedLatinSmallLetterI', [], 'Display', None],
  [ 0x24a5, 'ParenthesizedLatinSmallLetterJ', [], 'Display', None],
  [ 0x24a6, 'ParenthesizedLatinSmallLetterK', [], 'Display', None],
  [ 0x24a7, 'ParenthesizedLatinSmallLetterL', [], 'Display', None],
  [ 0x24a8, 'ParenthesizedLatinSmallLetterM', [], 'Display', None],
  [ 0x24a9, 'ParenthesizedLatinSmallLetterN', [], 'Display', None],
  [ 0x24aa, 'ParenthesizedLatinSmallLetterO', [], 'Display', None],
  [ 0x24ab, 'ParenthesizedLatinSmallLetterP', [], 'Display', None],
  [ 0x24ac, 'ParenthesizedLatinSmallLetterQ', [], 'Display', None],
  [ 0x24ad, 'ParenthesizedLatinSmallLetterR', [], 'Display', None],
  [ 0x24ae, 'ParenthesizedLatinSmallLetterS', [], 'Display', None],
  [ 0x24af, 'ParenthesizedLatinSmallLetterT', [], 'Display', None],
  [ 0x24b0, 'ParenthesizedLatinSmallLetterU', [], 'Display', None],
  [ 0x24b1, 'ParenthesizedLatinSmallLetterV', [], 'Display', None],
  [ 0x24b2, 'ParenthesizedLatinSmallLetterW', [], 'Display', None],
  [ 0x24b3, 'ParenthesizedLatinSmallLetterX', [], 'Display', None],
  [ 0x24b4, 'ParenthesizedLatinSmallLetterY', [], 'Display', None],
  [ 0x24b5, 'ParenthesizedLatinSmallLetterZ', [], 'Display', None],
  [ 0x24b6, 'CircledLatinCapitalLetterA', [], 'Display', None],
  [ 0x24b7, 'CircledLatinCapitalLetterB', [], 'Display', None],
  [ 0x24b8, 'CircledLatinCapitalLetterC', [], 'Display', None],
  [ 0x24b9, 'CircledLatinCapitalLetterD', [], 'Display', None],
  [ 0x24ba, 'CircledLatinCapitalLetterE', [], 'Display', None],
  [ 0x24bb, 'CircledLatinCapitalLetterF', [], 'Display', None],
  [ 0x24bc, 'CircledLatinCapitalLetterG', [], 'Display', None],
  [ 0x24bd, 'CircledLatinCapitalLetterH', [], 'Display', None],
  [ 0x24be, 'CircledLatinCapitalLetterI', [], 'Display', None],
  [ 0x24bf, 'CircledLatinCapitalLetterJ', [], 'Display', None],
  [ 0x24c0, 'CircledLatinCapitalLetterK', [], 'Display', None],
  [ 0x24c1, 'CircledLatinCapitalLetterL', [], 'Display', None],
  [ 0x24c2, 'CircledLatinCapitalLetterM', [], 'Display', None],
  [ 0x24c3, 'CircledLatinCapitalLetterN', [], 'Display', None],
  [ 0x24c4, 'CircledLatinCapitalLetterO', [], 'Display', None],
  [ 0x24c5, 'CircledLatinCapitalLetterP', [], 'Display', None],
  [ 0x24c6, 'CircledLatinCapitalLetterQ', [], 'Display', None],
  [ 0x24c7, 'CircledLatinCapitalLetterR', [], 'Display', None],
  [ 0x24c8, 'CircledLatinCapitalLetterS', [], 'Display', None],
  [ 0x24c9, 'CircledLatinCapitalLetterT', [], 'Display', None],
  [ 0x24ca, 'CircledLatinCapitalLetterU', [], 'Display', None],
  [ 0x24cb, 'CircledLatinCapitalLetterV', [], 'Display', None],
  [ 0x24cc, 'CircledLatinCapitalLetterW', [], 'Display', None],
  [ 0x24cd, 'CircledLatinCapitalLetterX', [], 'Display', None],
  [ 0x24ce, 'CircledLatinCapitalLetterY', [], 'Display', None],
  [ 0x24cf, 'CircledLatinCapitalLetterZ', [], 'Display', None],
  [ 0x24d0, 'CircledLatinSmallLetterA', [], 'Display', None],
  [ 0x24d1, 'CircledLatinSmallLetterB', [], 'Display', None],
  [ 0x24d2, 'CircledLatinSmallLetterC', [], 'Display', None],
  [ 0x24d3, 'CircledLatinSmallLetterD', [], 'Display', None],
  [ 0x24d4, 'CircledLatinSmallLetterE', [], 'Display', None],
  [ 0x24d5, 'CircledLatinSmallLetterF', [], 'Display', None],
  [ 0x24d6, 'CircledLatinSmallLetterG', [], 'Display', None],
  [ 0x24d7, 'CircledLatinSmallLetterH', [], 'Display', None],
  [ 0x24d8, 'CircledLatinSmallLetterI', [], 'Display', None],
  [ 0x24d9, 'CircledLatinSmallLetterJ', [], 'Display', None],
  [ 0x24da, 'CircledLatinSmallLetterK', [], 'Display', None],
  [ 0x24db, 'CircledLatinSmallLetterL', [], 'Display', None],
  [ 0x24dc, 'CircledLatinSmallLetterM', [], 'Display', None],
  [ 0x24dd, 'CircledLatinSmallLetterN', [], 'Display', None],
  [ 0x24de, 'CircledLatinSmallLetterO', [], 'Display', None],
  [ 0x24df, 'CircledLatinSmallLetterP', [], 'Display', None],
  [ 0x24e0, 'CircledLatinSmallLetterQ', [], 'Display', None],
  [ 0x24e1, 'CircledLatinSmallLetterR', [], 'Display', None],
  [ 0x24e2, 'CircledLatinSmallLetterS', [], 'Display', None],
  [ 0x24e3, 'CircledLatinSmallLetterT', [], 'Display', None],
  [ 0x24e4, 'CircledLatinSmallLetterU', [], 'Display', None],
  [ 0x24e5, 'CircledLatinSmallLetterV', [], 'Display', None],
  [ 0x24e6, 'CircledLatinSmallLetterW', [], 'Display', None],
  [ 0x24e7, 'CircledLatinSmallLetterX', [], 'Display', None],
  [ 0x24e8, 'CircledLatinSmallLetterY', [], 'Display', None],
  [ 0x24e9, 'CircledLatinSmallLetterZ', [], 'Display', None],
  [ 0x24ea, 'CircledDigitZero', [], 'Display', None],
  [ 0x24eb, 'NegativeCircledNumberEleven', [], 'Display', None],
  [ 0x24ec, 'NegativeCircledNumberTwelve', [], 'Display', None],
  [ 0x24ed, 'NegativeCircledNumberThirteen', [], 'Display', None],
  [ 0x24ee, 'NegativeCircledNumberFourteen', [], 'Display', None],
  [ 0x24ef, 'NegativeCircledNumberFifteen', [], 'Display', None],
  [ 0x24f0, 'NegativeCircledNumberSixteen', [], 'Display', None],
  [ 0x24f1, 'NegativeCircledNumberSeventeen', [], 'Display', None],
  [ 0x24f2, 'NegativeCircledNumberEighteen', [], 'Display', None],
  [ 0x24f3, 'NegativeCircledNumberNineteen', [], 'Display', None],
  [ 0x24f4, 'NegativeCircledNumberTwenty', [], 'Display', None],
  [ 0x24f5, 'DoubleCircledDigitOne', [], 'Display', None],
  [ 0x24f6, 'DoubleCircledDigitTwo', [], 'Display', None],
  [ 0x24f7, 'DoubleCircledDigitThree', [], 'Display', None],
  [ 0x24f8, 'DoubleCircledDigitFour', [], 'Display', None],
  [ 0x24f9, 'DoubleCircledDigitFive', [], 'Display', None],
  [ 0x24fa, 'DoubleCircledDigitSix', [], 'Display', None],
  [ 0x24fb, 'DoubleCircledDigitSeven', [], 'Display', None],
  [ 0x24fc, 'DoubleCircledDigitEight', [], 'Display', None],
  [ 0x24fd, 'DoubleCircledDigitNine', [], 'Display', None],
  [ 0x24fe, 'DoubleCircledNumberTen', [], 'Display', None],
  [ 0x24ff, 'NegativeCircledDigitZero', [], 'Display', None],
  [ 0x2500, 'BoxDrawingsLightHorizontal', [], 'Display', None],
  [ 0x2501, 'BoxDrawingsHeavyHorizontal', [], 'Display', None],
  [ 0x2502, 'BoxDrawingsLightVertical', [], 'Display', None],
  [ 0x2503, 'BoxDrawingsHeavyVertical', [], 'Display', None],
  [ 0x2504, 'BoxDrawingsLightTripleDashHorizontal', [], 'Display', None],
  [ 0x2505, 'BoxDrawingsHeavyTripleDashHorizontal', [], 'Display', None],
  [ 0x2506, 'BoxDrawingsLightTripleDashVertical', [], 'Display', None],
  [ 0x2507, 'BoxDrawingsHeavyTripleDashVertical', [], 'Display', None],
  [ 0x2508, 'BoxDrawingsLightQuadrupleDashHorizontal', [], 'Display', None],
  [ 0x2509, 'BoxDrawingsHeavyQuadrupleDashHorizontal', [], 'Display', None],
  [ 0x250a, 'BoxDrawingsLightQuadrupleDashVertical', [], 'Display', None],
  [ 0x250b, 'BoxDrawingsHeavyQuadrupleDashVertical', [], 'Display', None],
  [ 0x250c, 'BoxDrawingsLightDownAndRight', [], 'Display', None],
  [ 0x250d, 'BoxDrawingsDownLightAndRightHeavy', [], 'Display', None],
  [ 0x250e, 'BoxDrawingsDownHeavyAndRightLight', [], 'Display', None],
  [ 0x250f, 'BoxDrawingsHeavyDownAndRight', [], 'Display', None],
  [ 0x2510, 'BoxDrawingsLightDownAndLeft', [], 'Display', None],
  [ 0x2511, 'BoxDrawingsDownLightAndLeftHeavy', [], 'Display', None],
  [ 0x2512, 'BoxDrawingsDownHeavyAndLeftLight', [], 'Display', None],
  [ 0x2513, 'BoxDrawingsHeavyDownAndLeft', [], 'Display', None],
  [ 0x2514, 'BoxDrawingsLightUpAndRight', [], 'Display', None],
  [ 0x2515, 'BoxDrawingsUpLightAndRightHeavy', [], 'Display', None],
  [ 0x2516, 'BoxDrawingsUpHeavyAndRightLight', [], 'Display', None],
  [ 0x2517, 'BoxDrawingsHeavyUpAndRight', [], 'Display', None],
  [ 0x2518, 'BoxDrawingsLightUpAndLeft', [], 'Display', None],
  [ 0x2519, 'BoxDrawingsUpLightAndLeftHeavy', [], 'Display', None],
  [ 0x251a, 'BoxDrawingsUpHeavyAndLeftLight', [], 'Display', None],
  [ 0x251b, 'BoxDrawingsHeavyUpAndLeft', [], 'Display', None],
  [ 0x251c, 'BoxDrawingsLightVerticalAndRight', [], 'Display', None],
  [ 0x251d, 'BoxDrawingsVerticalLightAndRightHeavy', [], 'Display', None],
  [ 0x251e, 'BoxDrawingsUpHeavyAndRightDownLight', [], 'Display', None],
  [ 0x251f, 'BoxDrawingsDownHeavyAndRightUpLight', [], 'Display', None],
  [ 0x2520, 'BoxDrawingsVerticalHeavyAndRightLight', [], 'Display', None],
  [ 0x2521, 'BoxDrawingsDownLightAndRightUpHeavy', [], 'Display', None],
  [ 0x2522, 'BoxDrawingsUpLightAndRightDownHeavy', [], 'Display', None],
  [ 0x2523, 'BoxDrawingsHeavyVerticalAndRight', [], 'Display', None],
  [ 0x2524, 'BoxDrawingsLightVerticalAndLeft', [], 'Display', None],
  [ 0x2525, 'BoxDrawingsVerticalLightAndLeftHeavy', [], 'Display', None],
  [ 0x2526, 'BoxDrawingsUpHeavyAndLeftDownLight', [], 'Display', None],
  [ 0x2527, 'BoxDrawingsDownHeavyAndLeftUpLight', [], 'Display', None],
  [ 0x2528, 'BoxDrawingsVerticalHeavyAndLeftLight', [], 'Display', None],
  [ 0x2529, 'BoxDrawingsDownLightAndLeftUpHeavy', [], 'Display', None],
  [ 0x252a, 'BoxDrawingsUpLightAndLeftDownHeavy', [], 'Display', None],
  [ 0x252b, 'BoxDrawingsHeavyVerticalAndLeft', [], 'Display', None],
  [ 0x252c, 'BoxDrawingsLightDownAndHorizontal', [], 'Display', None],
  [ 0x252d, 'BoxDrawingsLeftHeavyAndRightDownLight', [], 'Display', None],
  [ 0x252e, 'BoxDrawingsRightHeavyAndLeftDownLight', [], 'Display', None],
  [ 0x252f, 'BoxDrawingsDownLightAndHorizontalHeavy', [], 'Display', None],
  [ 0x2530, 'BoxDrawingsDownHeavyAndHorizontalLight', [], 'Display', None],
  [ 0x2531, 'BoxDrawingsRightLightAndLeftDownHeavy', [], 'Display', None],
  [ 0x2532, 'BoxDrawingsLeftLightAndRightDownHeavy', [], 'Display', None],
  [ 0x2533, 'BoxDrawingsHeavyDownAndHorizontal', [], 'Display', None],
  [ 0x2534, 'BoxDrawingsLightUpAndHorizontal', [], 'Display', None],
  [ 0x2535, 'BoxDrawingsLeftHeavyAndRightUpLight', [], 'Display', None],
  [ 0x2536, 'BoxDrawingsRightHeavyAndLeftUpLight', [], 'Display', None],
  [ 0x2537, 'BoxDrawingsUpLightAndHorizontalHeavy', [], 'Display', None],
  [ 0x2538, 'BoxDrawingsUpHeavyAndHorizontalLight', [], 'Display', None],
  [ 0x2539, 'BoxDrawingsRightLightAndLeftUpHeavy', [], 'Display', None],
  [ 0x253a, 'BoxDrawingsLeftLightAndRightUpHeavy', [], 'Display', None],
  [ 0x253b, 'BoxDrawingsHeavyUpAndHorizontal', [], 'Display', None],
  [ 0x253c, 'BoxDrawingsLightVerticalAndHorizontal', [], 'Display', None],
  [ 0x253d, 'BoxDrawingsLeftHeavyAndRightVerticalLight', [], 'Display', None],
  [ 0x253e, 'BoxDrawingsRightHeavyAndLeftVerticalLight', [], 'Display', None],
  [ 0x253f, 'BoxDrawingsVerticalLightAndHorizontalHeavy', [], 'Display', None],
  [ 0x2540, 'BoxDrawingsUpHeavyAndDownHorizontalLight', [], 'Display', None],
  [ 0x2541, 'BoxDrawingsDownHeavyAndUpHorizontalLight', [], 'Display', None],
  [ 0x2542, 'BoxDrawingsVerticalHeavyAndHorizontalLight', [], 'Display', None],
  [ 0x2543, 'BoxDrawingsLeftUpHeavyAndRightDownLight', [], 'Display', None],
  [ 0x2544, 'BoxDrawingsRightUpHeavyAndLeftDownLight', [], 'Display', None],
  [ 0x2545, 'BoxDrawingsLeftDownHeavyAndRightUpLight', [], 'Display', None],
  [ 0x2546, 'BoxDrawingsRightDownHeavyAndLeftUpLight', [], 'Display', None],
  [ 0x2547, 'BoxDrawingsDownLightAndUpHorizontalHeavy', [], 'Display', None],
  [ 0x2548, 'BoxDrawingsUpLightAndDownHorizontalHeavy', [], 'Display', None],
  [ 0x2549, 'BoxDrawingsRightLightAndLeftVerticalHeavy', [], 'Display', None],
  [ 0x254a, 'BoxDrawingsLeftLightAndRightVerticalHeavy', [], 'Display', None],
  [ 0x254b, 'BoxDrawingsHeavyVerticalAndHorizontal', [], 'Display', None],
  [ 0x254c, 'BoxDrawingsLightDoubleDashHorizontal', [], 'Display', None],
  [ 0x254d, 'BoxDrawingsHeavyDoubleDashHorizontal', [], 'Display', None],
  [ 0x254e, 'BoxDrawingsLightDoubleDashVertical', [], 'Display', None],
  [ 0x254f, 'BoxDrawingsHeavyDoubleDashVertical', [], 'Display', None],
  [ 0x2550, 'BoxDrawingsDoubleHorizontal', [], 'Display', None],
  [ 0x2551, 'BoxDrawingsDoubleVertical', [], 'Display', None],
  [ 0x2552, 'BoxDrawingsDownSingleAndRightDouble', [], 'Display', None],
  [ 0x2553, 'BoxDrawingsDownDoubleAndRightSingle', [], 'Display', None],
  [ 0x2554, 'BoxDrawingsDoubleDownAndRight', [], 'Display', None],
  [ 0x2555, 'BoxDrawingsDownSingleAndLeftDouble', [], 'Display', None],
  [ 0x2556, 'BoxDrawingsDownDoubleAndLeftSingle', [], 'Display', None],
  [ 0x2557, 'BoxDrawingsDoubleDownAndLeft', [], 'Display', None],
  [ 0x2558, 'BoxDrawingsUpSingleAndRightDouble', [], 'Display', None],
  [ 0x2559, 'BoxDrawingsUpDoubleAndRightSingle', [], 'Display', None],
  [ 0x255a, 'BoxDrawingsDoubleUpAndRight', [], 'Display', None],
  [ 0x255b, 'BoxDrawingsUpSingleAndLeftDouble', [], 'Display', None],
  [ 0x255c, 'BoxDrawingsUpDoubleAndLeftSingle', [], 'Display', None],
  [ 0x255d, 'BoxDrawingsDoubleUpAndLeft', [], 'Display', None],
  [ 0x255e, 'BoxDrawingsVerticalSingleAndRightDouble', [], 'Display', None],
  [ 0x255f, 'BoxDrawingsVerticalDoubleAndRightSingle', [], 'Display', None],
  [ 0x2560, 'BoxDrawingsDoubleVerticalAndRight', [], 'Display', None],
  [ 0x2561, 'BoxDrawingsVerticalSingleAndLeftDouble', [], 'Display', None],
  [ 0x2562, 'BoxDrawingsVerticalDoubleAndLeftSingle', [], 'Display', None],
  [ 0x2563, 'BoxDrawingsDoubleVerticalAndLeft', [], 'Display', None],
  [ 0x2564, 'BoxDrawingsDownSingleAndHorizontalDouble', [], 'Display', None],
  [ 0x2565, 'BoxDrawingsDownDoubleAndHorizontalSingle', [], 'Display', None],
  [ 0x2566, 'BoxDrawingsDoubleDownAndHorizontal', [], 'Display', None],
  [ 0x2567, 'BoxDrawingsUpSingleAndHorizontalDouble', [], 'Display', None],
  [ 0x2568, 'BoxDrawingsUpDoubleAndHorizontalSingle', [], 'Display', None],
  [ 0x2569, 'BoxDrawingsDoubleUpAndHorizontal', [], 'Display', None],
  [ 0x256a, 'BoxDrawingsVerticalSingleAndHorizontalDouble', [], 'Display', None],
  [ 0x256b, 'BoxDrawingsVerticalDoubleAndHorizontalSingle', [], 'Display', None],
  [ 0x256c, 'BoxDrawingsDoubleVerticalAndHorizontal', [], 'Display', None],
  [ 0x256d, 'BoxDrawingsLightArcDownAndRight', [], 'Display', None],
  [ 0x256e, 'BoxDrawingsLightArcDownAndLeft', [], 'Display', None],
  [ 0x256f, 'BoxDrawingsLightArcUpAndLeft', [], 'Display', None],
  [ 0x2570, 'BoxDrawingsLightArcUpAndRight', [], 'Display', None],
  [ 0x2571, 'BoxDrawingsLightDiagonalUpperRightToLowerLeft', [], 'Display', None],
  [ 0x2572, 'BoxDrawingsLightDiagonalUpperLeftToLowerRight', [], 'Display', None],
  [ 0x2573, 'BoxDrawingsLightDiagonalCross', [], 'Display', None],
  [ 0x2574, 'BoxDrawingsLightLeft', [], 'Display', None],
  [ 0x2575, 'BoxDrawingsLightUp', [], 'Display', None],
  [ 0x2576, 'BoxDrawingsLightRight', [], 'Display', None],
  [ 0x2577, 'BoxDrawingsLightDown', [], 'Display', None],
  [ 0x2578, 'BoxDrawingsHeavyLeft', [], 'Display', None],
  [ 0x2579, 'BoxDrawingsHeavyUp', [], 'Display', None],
  [ 0x257a, 'BoxDrawingsHeavyRight', [], 'Display', None],
  [ 0x257b, 'BoxDrawingsHeavyDown', [], 'Display', None],
  [ 0x257c, 'BoxDrawingsLightLeftAndHeavyRight', [], 'Display', None],
  [ 0x257d, 'BoxDrawingsLightUpAndHeavyDown', [], 'Display', None],
  [ 0x257e, 'BoxDrawingsHeavyLeftAndLightRight', [], 'Display', None],
  [ 0x257f, 'BoxDrawingsHeavyUpAndLightDown', [], 'Display', None],
  [ 0x2580, 'UpperHalfBlock', [], 'Display', None],
  [ 0x2581, 'LowerOneEighthBlock', [], 'Display', None],
  [ 0x2582, 'LowerOneQuarterBlock', [], 'Display', None],
  [ 0x2583, 'LowerThreeEighthsBlock', [], 'Display', None],
  [ 0x2584, 'LowerHalfBlock', [], 'Display', None],
  [ 0x2585, 'LowerFiveEighthsBlock', [], 'Display', None],
  [ 0x2586, 'LowerThreeQuartersBlock', [], 'Display', None],
  [ 0x2587, 'LowerSevenEighthsBlock', [], 'Display', None],
  [ 0x2588, 'FullBlock', [], 'Display', None],
  [ 0x2589, 'LeftSevenEighthsBlock', [], 'Display', None],
  [ 0x258a, 'LeftThreeQuartersBlock', [], 'Display', None],
  [ 0x258b, 'LeftFiveEighthsBlock', [], 'Display', None],
  [ 0x258c, 'LeftHalfBlock', [], 'Display', None],
  [ 0x258d, 'LeftThreeEighthsBlock', [], 'Display', None],
  [ 0x258e, 'LeftOneQuarterBlock', [], 'Display', None],
  [ 0x258f, 'LeftOneEighthBlock', [], 'Display', None],
  [ 0x2590, 'RightHalfBlock', [], 'Display', None],
  [ 0x2591, 'LightShade', [], 'Display', None],
  [ 0x2592, 'MediumShade', [], 'Display', None],
  [ 0x2593, 'DarkShade', [], 'Display', None],
  [ 0x2594, 'UpperOneEighthBlock', [], 'Display', None],
  [ 0x2595, 'RightOneEighthBlock', [], 'Display', None],
  [ 0x2596, 'QuadrantLowerLeft', [], 'Display', None],
  [ 0x2597, 'QuadrantLowerRight', [], 'Display', None],
  [ 0x2598, 'QuadrantUpperLeft', [], 'Display', None],
  [ 0x2599, 'QuadrantUpperLeftAndLowerLeftAndLowerRight', [], 'Display', None],
  [ 0x259a, 'QuadrantUpperLeftAndLowerRight', [], 'Display', None],
  [ 0x259b, 'QuadrantUpperLeftAndUpperRightAndLowerLeft', [], 'Display', None],
  [ 0x259c, 'QuadrantUpperLeftAndUpperRightAndLowerRight', [], 'Display', None],
  [ 0x259d, 'QuadrantUpperRight', [], 'Display', None],
  [ 0x259e, 'QuadrantUpperRightAndLowerLeft', [], 'Display', None],
  [ 0x259f, 'QuadrantUpperRightAndLowerLeftAndLowerRight', [], 'Display', None],
  [ 0x25a0, 'BlackSquare', [], 'Display', None],
  [ 0x25a1, 'WhiteSquare', [], 'Display', None],
  [ 0x25a2, 'WhiteSquareWithRoundedCorners', [], 'Display', None],
  [ 0x25a3, 'WhiteSquareContainingBlackSmallSquare', [], 'Display', None],
  [ 0x25a4, 'SquareWithHorizontalFill', [], 'Display', None],
  [ 0x25a5, 'SquareWithVerticalFill', [], 'Display', None],
  [ 0x25a6, 'SquareWithOrthogonalCrosshatchFill', [], 'Display', None],
  [ 0x25a7, 'SquareWithUpperLeftToLowerRightFill', [], 'Display', None],
  [ 0x25a8, 'SquareWithUpperRightToLowerLeftFill', [], 'Display', None],
  [ 0x25a9, 'SquareWithDiagonalCrosshatchFill', [], 'Display', None],
  [ 0x25aa, 'BlackSmallSquare', [], 'Display', None],
  [ 0x25ab, 'WhiteSmallSquare', [], 'Display', None],
  [ 0x25ac, 'BlackRectangle', [], 'Display', None],
  [ 0x25ad, 'WhiteRectangle', [], 'Display', None],
  [ 0x25ae, 'BlackVerticalRectangle', [], 'Display', None],
  [ 0x25af, 'WhiteVerticalRectangle', [], 'Display', None],
  [ 0x25b0, 'BlackParallelogram', [], 'Display', None],
  [ 0x25b1, 'WhiteParallelogram', [], 'Display', None],
  [ 0x25b2, 'BlackUppointingTriangle', [], 'Display', None],
  [ 0x25b3, 'WhiteUppointingTriangle', [], 'Display', None],
  [ 0x25b4, 'BlackUppointingSmallTriangle', [], 'Display', None],
  [ 0x25b5, 'WhiteUppointingSmallTriangle', [], 'Display', None],
  [ 0x25b6, 'BlackRightpointingTriangle', [], 'Display', None],
  [ 0x25b7, 'WhiteRightpointingTriangle', [], 'Display', None],
  [ 0x25b8, 'BlackRightpointingSmallTriangle', [], 'Display', None],
  [ 0x25b9, 'WhiteRightpointingSmallTriangle', [], 'Display', None],
  [ 0x25ba, 'BlackRightpointingPointer', [], 'Display', None],
  [ 0x25bb, 'WhiteRightpointingPointer', [], 'Display', None],
  [ 0x25bc, 'BlackDownpointingTriangle', [], 'Display', None],
  [ 0x25bd, 'WhiteDownpointingTriangle', [], 'Display', None],
  [ 0x25be, 'BlackDownpointingSmallTriangle', [], 'Display', None],
  [ 0x25bf, 'WhiteDownpointingSmallTriangle', [], 'Display', None],
  [ 0x25c0, 'BlackLeftpointingTriangle', [], 'Display', None],
  [ 0x25c1, 'WhiteLeftpointingTriangle', [], 'Display', None],
  [ 0x25c2, 'BlackLeftpointingSmallTriangle', [], 'Display', None],
  [ 0x25c3, 'WhiteLeftpointingSmallTriangle', [], 'Display', None],
  [ 0x25c4, 'BlackLeftpointingPointer', [], 'Display', None],
  [ 0x25c5, 'WhiteLeftpointingPointer', [], 'Display', None],
  [ 0x25c6, 'BlackDiamond', [], 'Display', None],
  [ 0x25c7, 'WhiteDiamond', [], 'Display', None],
  [ 0x25c8, 'WhiteDiamondContainingBlackSmallDiamond', [], 'Display', None],
  [ 0x25c9, 'Fisheye', [], 'Display', None],
  [ 0x25ca, 'Lozenge', [], 'Display', None],
  [ 0x25cb, 'WhiteCircle', [], 'Display', None],
  [ 0x25cc, 'DottedCircle', [], 'Display', None],
  [ 0x25cd, 'CircleWithVerticalFill', [], 'Display', None],
  [ 0x25ce, 'Bullseye', [], 'Display', None],
  [ 0x25cf, 'BlackCircle', [], 'Display', None],
  [ 0x25d0, 'CircleWithLeftHalfBlack', [], 'Display', None],
  [ 0x25d1, 'CircleWithRightHalfBlack', [], 'Display', None],
  [ 0x25d2, 'CircleWithLowerHalfBlack', [], 'Display', None],
  [ 0x25d3, 'CircleWithUpperHalfBlack', [], 'Display', None],
  [ 0x25d4, 'CircleWithUpperRightQuadrantBlack', [], 'Display', None],
  [ 0x25d5, 'CircleWithAllButUpperLeftQuadrantBlack', [], 'Display', None],
  [ 0x25d6, 'LeftHalfBlackCircle', [], 'Display', None],
  [ 0x25d7, 'RightHalfBlackCircle', [], 'Display', None],
  [ 0x25d8, 'InverseBullet', [], 'Display', None],
  [ 0x25d9, 'InverseWhiteCircle', [], 'Display', None],
  [ 0x25da, 'UpperHalfInverseWhiteCircle', [], 'Display', None],
  [ 0x25db, 'LowerHalfInverseWhiteCircle', [], 'Display', None],
  [ 0x25dc, 'UpperLeftQuadrantCircularArc', [], 'Display', None],
  [ 0x25dd, 'UpperRightQuadrantCircularArc', [], 'Display', None],
  [ 0x25de, 'LowerRightQuadrantCircularArc', [], 'Display', None],
  [ 0x25df, 'LowerLeftQuadrantCircularArc', [], 'Display', None],
  [ 0x25e0, 'UpperHalfCircle', [], 'Display', None],
  [ 0x25e1, 'LowerHalfCircle', [], 'Display', None],
  [ 0x25e2, 'BlackLowerRightTriangle', [], 'Display', None],
  [ 0x25e3, 'BlackLowerLeftTriangle', [], 'Display', None],
  [ 0x25e4, 'BlackUpperLeftTriangle', [], 'Display', None],
  [ 0x25e5, 'BlackUpperRightTriangle', [], 'Display', None],
  [ 0x25e6, 'WhiteBullet', [], 'Display', None],
  [ 0x25e7, 'SquareWithLeftHalfBlack', [], 'Display', None],
  [ 0x25e8, 'SquareWithRightHalfBlack', [], 'Display', None],
  [ 0x25e9, 'SquareWithUpperLeftDiagonalHalfBlack', [], 'Display', None],
  [ 0x25ea, 'SquareWithLowerRightDiagonalHalfBlack', [], 'Display', None],
  [ 0x25eb, 'WhiteSquareWithVerticalBisectingLine', [], 'Display', None],
  [ 0x25ec, 'WhiteUppointingTriangleWithDot', [], 'Display', None],
  [ 0x25ed, 'UppointingTriangleWithLeftHalfBlack', [], 'Display', None],
  [ 0x25ee, 'UppointingTriangleWithRightHalfBlack', [], 'Display', None],
  [ 0x25ef, 'LargeCircle', [], 'Display', None],
  [ 0x25f0, 'WhiteSquareWithUpperLeftQuadrant', [], 'Display', None],
  [ 0x25f1, 'WhiteSquareWithLowerLeftQuadrant', [], 'Display', None],
  [ 0x25f2, 'WhiteSquareWithLowerRightQuadrant', [], 'Display', None],
  [ 0x25f3, 'WhiteSquareWithUpperRightQuadrant', [], 'Display', None],
  [ 0x25f4, 'WhiteCircleWithUpperLeftQuadrant', [], 'Display', None],
  [ 0x25f5, 'WhiteCircleWithLowerLeftQuadrant', [], 'Display', None],
  [ 0x25f6, 'WhiteCircleWithLowerRightQuadrant', [], 'Display', None],
  [ 0x25f7, 'WhiteCircleWithUpperRightQuadrant', [], 'Display', None],
  [ 0x25f8, 'UpperLeftTriangle', [], 'Display', None],
  [ 0x25f9, 'UpperRightTriangle', [], 'Display', None],
  [ 0x25fa, 'LowerLeftTriangle', [], 'Display', None],
  [ 0x25fb, 'WhiteMediumSquare', [], 'Display', None],
  [ 0x25fc, 'BlackMediumSquare', [], 'Display', None],
  [ 0x25fd, 'WhiteMediumSmallSquare', [], 'Display', None],
  [ 0x25fe, 'BlackMediumSmallSquare', [], 'Display', None],
  [ 0x25ff, 'LowerRightTriangle', [], 'Display', None],
  [ 0x2600, 'BlackSunWithRays', [], 'Display', None],
  [ 0x2601, 'Cloud', [], 'Display', None],
  [ 0x2602, 'Umbrella', [], 'Display', None],
  [ 0x2603, 'Snowman', [], 'Display', None],
  [ 0x2604, 'Comet', [], 'Display', None],
  [ 0x2605, 'BlackStar', [], 'Display', None],
  [ 0x2606, 'WhiteStar', [], 'Display', None],
  [ 0x2607, 'Lightning', [], 'Display', None],
  [ 0x2608, 'Thunderstorm', [], 'Display', None],
  [ 0x2609, 'Sun', [], 'Display', None],
  [ 0x260a, 'AscendingNode', [], 'Display', None],
  [ 0x260b, 'DescendingNode', [], 'Display', None],
  [ 0x260c, 'Conjunction', [], 'Display', None],
  [ 0x260d, 'Opposition', [], 'Display', None],
  [ 0x260e, 'BlackTelephone', [], 'Display', None],
  [ 0x260f, 'WhiteTelephone', [], 'Display', None],
  [ 0x2610, 'BallotBox', [], 'Display', None],
  [ 0x2611, 'BallotBoxWithCheck', [], 'Display', None],
  [ 0x2612, 'BallotBoxWithX', [], 'Display', None],
  [ 0x2613, 'Saltire', [], 'Display', None],
  [ 0x2614, 'UmbrellaWithRainDrops', [], 'Display', None],
  [ 0x2615, 'HotBeverage', [], 'Display', None],
  [ 0x2616, 'WhiteShogiPiece', [], 'Display', None],
  [ 0x2617, 'BlackShogiPiece', [], 'Display', None],
  [ 0x2618, 'Shamrock', [], 'Display', None],
  [ 0x2619, 'ReversedRotatedFloralHeartBullet', [], 'Display', None],
  [ 0x261a, 'BlackLeftPointingIndex', [], 'Display', None],
  [ 0x261b, 'BlackRightPointingIndex', [], 'Display', None],
  [ 0x261c, 'WhiteLeftPointingIndex', [], 'Display', None],
  [ 0x261d, 'WhiteUpPointingIndex', [], 'Display', None],
  [ 0x261e, 'WhiteRightPointingIndex', [], 'Display', None],
  [ 0x261f, 'WhiteDownPointingIndex', [], 'Display', None],
  [ 0x2620, 'SkullAndCrossbones', [], 'Display', None],
  [ 0x2621, 'CautionSign', [], 'Display', None],
  [ 0x2622, 'RadioactiveSign', [], 'Display', None],
  [ 0x2623, 'BiohazardSign', [], 'Display', None],
  [ 0x2624, 'Caduceus', [], 'Display', None],
  [ 0x2625, 'Ankh', [], 'Display', None],
  [ 0x2626, 'OrthodoxCross', [], 'Display', None],
  [ 0x2627, 'ChiRho', [], 'Display', None],
  [ 0x2628, 'CrossOfLorraine', [], 'Display', None],
  [ 0x2629, 'CrossOfJerusalem', [], 'Display', None],
  [ 0x262a, 'StarAndCrescent', [], 'Display', None],
  [ 0x262b, 'FarsiSymbol', [], 'Display', None],
  [ 0x262c, 'AdiShakti', [], 'Display', None],
  [ 0x262d, 'HammerAndSickle', [], 'Display', None],
  [ 0x262e, 'PeaceSymbol', [], 'Display', None],
  [ 0x262f, 'YinYang', [], 'Display', None],
  [ 0x2630, 'TrigramForHeaven', [], 'Display', None],
  [ 0x2631, 'TrigramForLake', [], 'Display', None],
  [ 0x2632, 'TrigramForFire', [], 'Display', None],
  [ 0x2633, 'TrigramForThunder', [], 'Display', None],
  [ 0x2634, 'TrigramForWind', [], 'Display', None],
  [ 0x2635, 'TrigramForWater', [], 'Display', None],
  [ 0x2636, 'TrigramForMountain', [], 'Display', None],
  [ 0x2637, 'TrigramForEarth', [], 'Display', None],
  [ 0x2638, 'WheelOfDharma', [], 'Display', None],
  [ 0x2639, 'WhiteFrowningFace', [], 'Display', None],
  [ 0x263a, 'WhiteSmilingFace', [], 'Display', None],
  [ 0x263b, 'BlackSmilingFace', [], 'Display', None],
  [ 0x263c, 'WhiteSunWithRays', [], 'Display', None],
  [ 0x263d, 'FirstQuarterMoon', [], 'Display', None],
  [ 0x263e, 'LastQuarterMoon', [], 'Display', None],
  [ 0x263f, 'Mercury', [], 'Display', None],
  [ 0x2640, 'FemaleSign', [], 'Display', None],
  [ 0x2641, 'Earth', [], 'Display', None],
  [ 0x2642, 'MaleSign', [], 'Display', None],
  [ 0x2643, 'Jupiter', [], 'Display', None],
  [ 0x2644, 'Saturn', [], 'Display', None],
  [ 0x2645, 'Uranus', [], 'Display', None],
  [ 0x2646, 'Neptune', [], 'Display', None],
  [ 0x2647, 'Pluto', [], 'Display', None],
  [ 0x2648, 'Aries', [], 'Display', None],
  [ 0x2649, 'Taurus', [], 'Display', None],
  [ 0x264a, 'Gemini', [], 'Display', None],
  [ 0x264b, 'Cancer', [], 'Display', None],
  [ 0x264c, 'Leo', [], 'Display', None],
  [ 0x264d, 'Virgo', [], 'Display', None],
  [ 0x264e, 'Libra', [], 'Display', None],
  [ 0x264f, 'Scorpius', [], 'Display', None],
  [ 0x2650, 'Sagittarius', [], 'Display', None],
  [ 0x2651, 'Capricorn', [], 'Display', None],
  [ 0x2652, 'Aquarius', [], 'Display', None],
  [ 0x2653, 'Pisces', [], 'Display', None],
  [ 0x2654, 'WhiteChessKing', [], 'Display', None],
  [ 0x2655, 'WhiteChessQueen', [], 'Display', None],
  [ 0x2656, 'WhiteChessRook', [], 'Display', None],
  [ 0x2657, 'WhiteChessBishop', [], 'Display', None],
  [ 0x2658, 'WhiteChessKnight', [], 'Display', None],
  [ 0x2659, 'WhiteChessPawn', [], 'Display', None],
  [ 0x265a, 'BlackChessKing', [], 'Display', None],
  [ 0x265b, 'BlackChessQueen', [], 'Display', None],
  [ 0x265c, 'BlackChessRook', [], 'Display', None],
  [ 0x265d, 'BlackChessBishop', [], 'Display', None],
  [ 0x265e, 'BlackChessKnight', [], 'Display', None],
  [ 0x265f, 'BlackChessPawn', [], 'Display', None],
  [ 0x2660, 'BlackSpadeSuit', [], 'Display', None],
  [ 0x2661, 'WhiteHeartSuit', [], 'Display', None],
  [ 0x2662, 'WhiteDiamondSuit', [], 'Display', None],
  [ 0x2663, 'BlackClubSuit', [], 'Display', None],
  [ 0x2664, 'WhiteSpadeSuit', [], 'Display', None],
  [ 0x2665, 'BlackHeartSuit', [], 'Display', None],
  [ 0x2666, 'BlackDiamondSuit', [], 'Display', None],
  [ 0x2667, 'WhiteClubSuit', [], 'Display', None],
  [ 0x2668, 'HotSprings', [], 'Display', None],
  [ 0x2669, 'QuarterNote', [], 'Display', None],
  [ 0x266a, 'EighthNote', [], 'Display', None],
  [ 0x266b, 'BeamedEighthNotes', [], 'Display', None],
  [ 0x266c, 'BeamedSixteenthNotes', [], 'Display', None],
  [ 0x266d, 'MusicFlatSign', [], 'Display', None],
  [ 0x266e, 'MusicNaturalSign', [], 'Display', None],
  [ 0x266f, 'MusicSharpSign', [], 'Display', None],
  [ 0x2670, 'WestSyriacCross', [], 'Display', None],
  [ 0x2671, 'EastSyriacCross', [], 'Display', None],
  [ 0x2672, 'UniversalRecyclingSymbol', [], 'Display', None],
  [ 0x2673, 'RecyclingSymbolForType1Plastics', [], 'Display', None],
  [ 0x2674, 'RecyclingSymbolForType2Plastics', [], 'Display', None],
  [ 0x2675, 'RecyclingSymbolForType3Plastics', [], 'Display', None],
  [ 0x2676, 'RecyclingSymbolForType4Plastics', [], 'Display', None],
  [ 0x2677, 'RecyclingSymbolForType5Plastics', [], 'Display', None],
  [ 0x2678, 'RecyclingSymbolForType6Plastics', [], 'Display', None],
  [ 0x2679, 'RecyclingSymbolForType7Plastics', [], 'Display', None],
  [ 0x267a, 'RecyclingSymbolForGenericMaterials', [], 'Display', None],
  [ 0x267b, 'BlackUniversalRecyclingSymbol', [], 'Display', None],
  [ 0x267c, 'RecycledPaperSymbol', [], 'Display', None],
  [ 0x267d, 'PartiallyrecycledPaperSymbol', [], 'Display', None],
  [ 0x267e, 'PermanentPaperSign', [], 'Display', None],
  [ 0x267f, 'WheelchairSymbol', [], 'Display', None],
  [ 0x2680, 'DieFace1', [], 'Display', None],
  [ 0x2681, 'DieFace2', [], 'Display', None],
  [ 0x2682, 'DieFace3', [], 'Display', None],
  [ 0x2683, 'DieFace4', [], 'Display', None],
  [ 0x2684, 'DieFace5', [], 'Display', None],
  [ 0x2685, 'DieFace6', [], 'Display', None],
  [ 0x2686, 'WhiteCircleWithDotRight', [], 'Display', None],
  [ 0x2687, 'WhiteCircleWithTwoDots', [], 'Display', None],
  [ 0x2688, 'BlackCircleWithWhiteDotRight', [], 'Display', None],
  [ 0x2689, 'BlackCircleWithTwoWhiteDots', [], 'Display', None],
  [ 0x268a, 'MonogramForYang', [], 'Display', None],
  [ 0x268b, 'MonogramForYin', [], 'Display', None],
  [ 0x268c, 'DigramForGreaterYang', [], 'Display', None],
  [ 0x268d, 'DigramForLesserYin', [], 'Display', None],
  [ 0x268e, 'DigramForLesserYang', [], 'Display', None],
  [ 0x268f, 'DigramForGreaterYin', [], 'Display', None],
  [ 0x2690, 'WhiteFlag', [], 'Display', None],
  [ 0x2691, 'BlackFlag', [], 'Display', None],
  [ 0x2692, 'HammerAndPick', [], 'Display', None],
  [ 0x2693, 'Anchor', [], 'Display', None],
  [ 0x2694, 'CrossedSwords', [], 'Display', None],
  [ 0x2695, 'StaffOfAesculapius', [], 'Display', None],
  [ 0x2696, 'Scales', [], 'Display', None],
  [ 0x2697, 'Alembic', [], 'Display', None],
  [ 0x2698, 'Flower', [], 'Display', None],
  [ 0x2699, 'Gear', [], 'Display', None],
  [ 0x269a, 'StaffOfHermes', [], 'Display', None],
  [ 0x269b, 'AtomSymbol', [], 'Display', None],
  [ 0x269c, 'Fleurdelis', [], 'Display', None],
  [ 0x269d, 'OutlinedWhiteStar', [], 'Display', None],
  [ 0x269e, 'UnicodeCharacter0x269e', [], 'Display', None],
  [ 0x269f, 'UnicodeCharacter0x269f', [], 'Display', None],
  [ 0x26a0, 'WarningSign', [], 'Display', None],
  [ 0x26a1, 'HighVoltageSign', [], 'Display', None],
  [ 0x26a2, 'DoubledFemaleSign', [], 'Display', None],
  [ 0x26a3, 'DoubledMaleSign', [], 'Display', None],
  [ 0x26a4, 'InterlockedFemaleAndMaleSign', [], 'Display', None],
  [ 0x26a5, 'MaleAndFemaleSign', [], 'Display', None],
  [ 0x26a6, 'MaleWithStrokeSign', [], 'Display', None],
  [ 0x26a7, 'MaleWithStrokeAndMaleAndFemaleSign', [], 'Display', None],
  [ 0x26a8, 'VerticalMaleWithStrokeSign', [], 'Display', None],
  [ 0x26a9, 'HorizontalMaleWithStrokeSign', [], 'Display', None],
  [ 0x26aa, 'MediumWhiteCircle', [], 'Display', None],
  [ 0x26ab, 'MediumBlackCircle', [], 'Display', None],
  [ 0x26ac, 'MediumSmallWhiteCircle', [], 'Display', None],
  [ 0x26ad, 'MarriageSymbol', [], 'Display', None],
  [ 0x26ae, 'DivorceSymbol', [], 'Display', None],
  [ 0x26af, 'UnmarriedPartnershipSymbol', [], 'Display', None],
  [ 0x26b0, 'Coffin', [], 'Display', None],
  [ 0x26b1, 'FuneralUrn', [], 'Display', None],
  [ 0x26b2, 'Neuter', [], 'Display', None],
  [ 0x26b3, 'Ceres', [], 'Display', None],
  [ 0x26b4, 'Pallas', [], 'Display', None],
  [ 0x26b5, 'Juno', [], 'Display', None],
  [ 0x26b6, 'Vesta', [], 'Display', None],
  [ 0x26b7, 'Chiron', [], 'Display', None],
  [ 0x26b8, 'BlackMoonLilith', [], 'Display', None],
  [ 0x26b9, 'Sextile', [], 'Display', None],
  [ 0x26ba, 'Semisextile', [], 'Display', None],
  [ 0x26bb, 'Quincunx', [], 'Display', None],
  [ 0x26bc, 'Sesquiquadrate', [], 'Display', None],
  [ 0x26bd, 'UnicodeCharacter0x26bd', [], 'Display', None],
  [ 0x26be, 'UnicodeCharacter0x26be', [], 'Display', None],
  [ 0x26bf, 'UnicodeCharacter0x26bf', [], 'Display', None],
  [ 0x26c0, 'WhiteDraughtsMan', [], 'Display', None],
  [ 0x26c1, 'WhiteDraughtsKing', [], 'Display', None],
  [ 0x26c2, 'BlackDraughtsMan', [], 'Display', None],
  [ 0x26c3, 'BlackDraughtsKing', [], 'Display', None],
  [ 0x26c4, 'UnicodeCharacter0x26c4', [], 'Display', None],
  [ 0x26c5, 'UnicodeCharacter0x26c5', [], 'Display', None],
  [ 0x26c6, 'UnicodeCharacter0x26c6', [], 'Display', None],
  [ 0x26c7, 'UnicodeCharacter0x26c7', [], 'Display', None],
  [ 0x26c8, 'UnicodeCharacter0x26c8', [], 'Display', None],
  [ 0x26c9, 'UnicodeCharacter0x26c9', [], 'Display', None],
  [ 0x26ca, 'UnicodeCharacter0x26ca', [], 'Display', None],
  [ 0x26cb, 'UnicodeCharacter0x26cb', [], 'Display', None],
  [ 0x26cc, 'UnicodeCharacter0x26cc', [], 'Display', None],
  [ 0x26cd, 'UnicodeCharacter0x26cd', [], 'Display', None],
  [ 0x26ce, 'UnicodeCharacter0x26ce', [], 'Display', None],
  [ 0x26cf, 'UnicodeCharacter0x26cf', [], 'Display', None],
  [ 0x26d0, 'UnicodeCharacter0x26d0', [], 'Display', None],
  [ 0x26d1, 'UnicodeCharacter0x26d1', [], 'Display', None],
  [ 0x26d2, 'UnicodeCharacter0x26d2', [], 'Display', None],
  [ 0x26d3, 'UnicodeCharacter0x26d3', [], 'Display', None],
  [ 0x26d4, 'UnicodeCharacter0x26d4', [], 'Display', None],
  [ 0x26d5, 'UnicodeCharacter0x26d5', [], 'Display', None],
  [ 0x26d6, 'UnicodeCharacter0x26d6', [], 'Display', None],
  [ 0x26d7, 'UnicodeCharacter0x26d7', [], 'Display', None],
  [ 0x26d8, 'UnicodeCharacter0x26d8', [], 'Display', None],
  [ 0x26d9, 'UnicodeCharacter0x26d9', [], 'Display', None],
  [ 0x26da, 'UnicodeCharacter0x26da', [], 'Display', None],
  [ 0x26db, 'UnicodeCharacter0x26db', [], 'Display', None],
  [ 0x26dc, 'UnicodeCharacter0x26dc', [], 'Display', None],
  [ 0x26dd, 'UnicodeCharacter0x26dd', [], 'Display', None],
  [ 0x26de, 'UnicodeCharacter0x26de', [], 'Display', None],
  [ 0x26df, 'UnicodeCharacter0x26df', [], 'Display', None],
  [ 0x26e0, 'UnicodeCharacter0x26e0', [], 'Display', None],
  [ 0x26e1, 'UnicodeCharacter0x26e1', [], 'Display', None],
  [ 0x26e2, 'UnicodeCharacter0x26e2', [], 'Display', None],
  [ 0x26e3, 'UnicodeCharacter0x26e3', [], 'Display', None],
  [ 0x26e4, 'UnicodeCharacter0x26e4', [], 'Display', None],
  [ 0x26e5, 'UnicodeCharacter0x26e5', [], 'Display', None],
  [ 0x26e6, 'UnicodeCharacter0x26e6', [], 'Display', None],
  [ 0x26e7, 'UnicodeCharacter0x26e7', [], 'Display', None],
  [ 0x26e8, 'UnicodeCharacter0x26e8', [], 'Display', None],
  [ 0x26e9, 'UnicodeCharacter0x26e9', [], 'Display', None],
  [ 0x26ea, 'UnicodeCharacter0x26ea', [], 'Display', None],
  [ 0x26eb, 'UnicodeCharacter0x26eb', [], 'Display', None],
  [ 0x26ec, 'UnicodeCharacter0x26ec', [], 'Display', None],
  [ 0x26ed, 'UnicodeCharacter0x26ed', [], 'Display', None],
  [ 0x26ee, 'UnicodeCharacter0x26ee', [], 'Display', None],
  [ 0x26ef, 'UnicodeCharacter0x26ef', [], 'Display', None],
  [ 0x26f0, 'UnicodeCharacter0x26f0', [], 'Display', None],
  [ 0x26f1, 'UnicodeCharacter0x26f1', [], 'Display', None],
  [ 0x26f2, 'UnicodeCharacter0x26f2', [], 'Display', None],
  [ 0x26f3, 'UnicodeCharacter0x26f3', [], 'Display', None],
  [ 0x26f4, 'UnicodeCharacter0x26f4', [], 'Display', None],
  [ 0x26f5, 'UnicodeCharacter0x26f5', [], 'Display', None],
  [ 0x26f6, 'UnicodeCharacter0x26f6', [], 'Display', None],
  [ 0x26f7, 'UnicodeCharacter0x26f7', [], 'Display', None],
  [ 0x26f8, 'UnicodeCharacter0x26f8', [], 'Display', None],
  [ 0x26f9, 'UnicodeCharacter0x26f9', [], 'Display', None],
  [ 0x26fa, 'UnicodeCharacter0x26fa', [], 'Display', None],
  [ 0x26fb, 'UnicodeCharacter0x26fb', [], 'Display', None],
  [ 0x26fc, 'UnicodeCharacter0x26fc', [], 'Display', None],
  [ 0x26fd, 'UnicodeCharacter0x26fd', [], 'Display', None],
  [ 0x26fe, 'UnicodeCharacter0x26fe', [], 'Display', None],
  [ 0x26ff, 'UnicodeCharacter0x26ff', [], 'Display', None],
  [ 0x2700, 'UnicodeCharacter0x2700', [], 'Display', None],
  [ 0x2701, 'UpperBladeScissors', [], 'Display', None],
  [ 0x2702, 'BlackScissors', [], 'Display', None],
  [ 0x2703, 'LowerBladeScissors', [], 'Display', None],
  [ 0x2704, 'WhiteScissors', [], 'Display', None],
  [ 0x2705, 'UnicodeCharacter0x2705', [], 'Display', None],
  [ 0x2706, 'TelephoneLocationSign', [], 'Display', None],
  [ 0x2707, 'TapeDrive', [], 'Display', None],
  [ 0x2708, 'Airplane', [], 'Display', None],
  [ 0x2709, 'Envelope', [], 'Display', None],
  [ 0x270a, 'UnicodeCharacter0x270a', [], 'Display', None],
  [ 0x270b, 'UnicodeCharacter0x270b', [], 'Display', None],
  [ 0x270c, 'VictoryHand', [], 'Display', None],
  [ 0x270d, 'WritingHand', [], 'Display', None],
  [ 0x270e, 'LowerRightPencil', [], 'Display', None],
  [ 0x270f, 'Pencil', [], 'Display', None],
  [ 0x2710, 'UpperRightPencil', [], 'Display', None],
  [ 0x2711, 'WhiteNib', [], 'Display', None],
  [ 0x2712, 'BlackNib', [], 'Display', None],
  [ 0x2713, 'CheckMark', [], 'Display', None],
  [ 0x2714, 'HeavyCheckMark', [], 'Display', None],
  [ 0x2715, 'MultiplicationX', [], 'Display', None],
  [ 0x2716, 'HeavyMultiplicationX', [], 'Display', None],
  [ 0x2717, 'BallotX', [], 'Display', None],
  [ 0x2718, 'HeavyBallotX', [], 'Display', None],
  [ 0x2719, 'OutlinedGreekCross', [], 'Display', None],
  [ 0x271a, 'HeavyGreekCross', [], 'Display', None],
  [ 0x271b, 'OpenCentreCross', [], 'Display', None],
  [ 0x271c, 'HeavyOpenCentreCross', [], 'Display', None],
  [ 0x271d, 'LatinCross', [], 'Display', None],
  [ 0x271e, 'ShadowedWhiteLatinCross', [], 'Display', None],
  [ 0x271f, 'OutlinedLatinCross', [], 'Display', None],
  [ 0x2720, 'MalteseCross', [], 'Display', None],
  [ 0x2721, 'StarOfDavid', [], 'Display', None],
  [ 0x2722, 'FourTeardropspokedAsterisk', [], 'Display', None],
  [ 0x2723, 'FourBalloonspokedAsterisk', [], 'Display', None],
  [ 0x2724, 'HeavyFourBalloonspokedAsterisk', [], 'Display', None],
  [ 0x2725, 'FourClubspokedAsterisk', [], 'Display', None],
  [ 0x2726, 'BlackFourPointedStar', [], 'Display', None],
  [ 0x2727, 'WhiteFourPointedStar', [], 'Display', None],
  [ 0x2728, 'UnicodeCharacter0x2728', [], 'Display', None],
  [ 0x2729, 'StressOutlinedWhiteStar', [], 'Display', None],
  [ 0x272a, 'CircledWhiteStar', [], 'Display', None],
  [ 0x272b, 'OpenCentreBlackStar', [], 'Display', None],
  [ 0x272c, 'BlackCentreWhiteStar', [], 'Display', None],
  [ 0x272d, 'OutlinedBlackStar', [], 'Display', None],
  [ 0x272e, 'HeavyOutlinedBlackStar', [], 'Display', None],
  [ 0x272f, 'PinwheelStar', [], 'Display', None],
  [ 0x2730, 'ShadowedWhiteStar', [], 'Display', None],
  [ 0x2731, 'HeavyAsterisk', [], 'Display', None],
  [ 0x2732, 'OpenCentreAsterisk', [], 'Display', None],
  [ 0x2733, 'EightSpokedAsterisk', [], 'Display', None],
  [ 0x2734, 'EightPointedBlackStar', [], 'Display', None],
  [ 0x2735, 'EightPointedPinwheelStar', [], 'Display', None],
  [ 0x2736, 'SixPointedBlackStar', [], 'Display', None],
  [ 0x2737, 'EightPointedRectilinearBlackStar', [], 'Display', None],
  [ 0x2738, 'HeavyEightPointedRectilinearBlackStar', [], 'Display', None],
  [ 0x2739, 'TwelvePointedBlackStar', [], 'Display', None],
  [ 0x273a, 'SixteenPointedAsterisk', [], 'Display', None],
  [ 0x273b, 'TeardropspokedAsterisk', [], 'Display', None],
  [ 0x273c, 'OpenCentreTeardropspokedAsterisk', [], 'Display', None],
  [ 0x273d, 'HeavyTeardropspokedAsterisk', [], 'Display', None],
  [ 0x273e, 'SixPetalledBlackAndWhiteFlorette', [], 'Display', None],
  [ 0x273f, 'BlackFlorette', [], 'Display', None],
  [ 0x2740, 'WhiteFlorette', [], 'Display', None],
  [ 0x2741, 'EightPetalledOutlinedBlackFlorette', [], 'Display', None],
  [ 0x2742, 'CircledOpenCentreEightPointedStar', [], 'Display', None],
  [ 0x2743, 'HeavyTeardropspokedPinwheelAsterisk', [], 'Display', None],
  [ 0x2744, 'Snowflake', [], 'Display', None],
  [ 0x2745, 'TightTrifoliateSnowflake', [], 'Display', None],
  [ 0x2746, 'HeavyChevronSnowflake', [], 'Display', None],
  [ 0x2747, 'Sparkle', [], 'Display', None],
  [ 0x2748, 'HeavySparkle', [], 'Display', None],
  [ 0x2749, 'BalloonspokedAsterisk', [], 'Display', None],
  [ 0x274a, 'EightTeardropspokedPropellerAsterisk', [], 'Display', None],
  [ 0x274b, 'HeavyEightTeardropspokedPropellerAsterisk', [], 'Display', None],
  [ 0x274c, 'UnicodeCharacter0x274c', [], 'Display', None],
  [ 0x274d, 'ShadowedWhiteCircle', [], 'Display', None],
  [ 0x274e, 'UnicodeCharacter0x274e', [], 'Display', None],
  [ 0x274f, 'LowerRightDropshadowedWhiteSquare', [], 'Display', None],
  [ 0x2750, 'UpperRightDropshadowedWhiteSquare', [], 'Display', None],
  [ 0x2751, 'LowerRightShadowedWhiteSquare', [], 'Display', None],
  [ 0x2752, 'UpperRightShadowedWhiteSquare', [], 'Display', None],
  [ 0x2753, 'UnicodeCharacter0x2753', [], 'Display', None],
  [ 0x2754, 'UnicodeCharacter0x2754', [], 'Display', None],
  [ 0x2755, 'UnicodeCharacter0x2755', [], 'Display', None],
  [ 0x2756, 'BlackDiamondMinusWhiteX', [], 'Display', None],
  [ 0x2757, 'UnicodeCharacter0x2757', [], 'Display', None],
  [ 0x2758, 'LightVerticalBar', [], 'Display', None],
  [ 0x2759, 'MediumVerticalBar', [], 'Display', None],
  [ 0x275a, 'HeavyVerticalBar', [], 'Display', None],
  [ 0x275b, 'HeavySingleTurnedCommaQuotationMarkOrnament', [], 'Display', None],
  [ 0x275c, 'HeavySingleCommaQuotationMarkOrnament', [], 'Display', None],
  [ 0x275d, 'HeavyDoubleTurnedCommaQuotationMarkOrnament', [], 'Display', None],
  [ 0x275e, 'HeavyDoubleCommaQuotationMarkOrnament', [], 'Display', None],
  [ 0x275f, 'UnicodeCharacter0x275f', [], 'Display', None],
  [ 0x2760, 'UnicodeCharacter0x2760', [], 'Display', None],
  [ 0x2761, 'CurvedStemParagraphSignOrnament', [], 'Display', None],
  [ 0x2762, 'HeavyExclamationMarkOrnament', [], 'Display', None],
  [ 0x2763, 'HeavyHeartExclamationMarkOrnament', [], 'Display', None],
  [ 0x2764, 'HeavyBlackHeart', [], 'Display', None],
  [ 0x2765, 'RotatedHeavyBlackHeartBullet', [], 'Display', None],
  [ 0x2766, 'FloralHeart', [], 'Display', None],
  [ 0x2767, 'RotatedFloralHeartBullet', [], 'Display', None],
  [ 0x2768, 'MediumLeftParenthesisOrnament', [], 'Display', None],
  [ 0x2769, 'MediumRightParenthesisOrnament', [], 'Display', None],
  [ 0x276a, 'MediumFlattenedLeftParenthesisOrnament', [], 'Display', None],
  [ 0x276b, 'MediumFlattenedRightParenthesisOrnament', [], 'Display', None],
  [ 0x276c, 'MediumLeftpointingAngleBracketOrnament', [], 'Display', None],
  [ 0x276d, 'MediumRightpointingAngleBracketOrnament', [], 'Display', None],
  [ 0x276e, 'HeavyLeftpointingAngleQuotationMarkOrnament', [], 'Display', None],
  [ 0x276f, 'HeavyRightpointingAngleQuotationMarkOrnament', [], 'Display', None],
  [ 0x2770, 'HeavyLeftpointingAngleBracketOrnament', [], 'Display', None],
  [ 0x2771, 'HeavyRightpointingAngleBracketOrnament', [], 'Display', None],
  [ 0x2772, 'LightLeftTortoiseShellBracketOrnament', [], 'Display', None],
  [ 0x2773, 'LightRightTortoiseShellBracketOrnament', [], 'Display', None],
  [ 0x2774, 'MediumLeftCurlyBracketOrnament', [], 'Display', None],
  [ 0x2775, 'MediumRightCurlyBracketOrnament', [], 'Display', None],
  [ 0x2776, 'DingbatNegativeCircledDigitOne', [], 'Display', None],
  [ 0x2777, 'DingbatNegativeCircledDigitTwo', [], 'Display', None],
  [ 0x2778, 'DingbatNegativeCircledDigitThree', [], 'Display', None],
  [ 0x2779, 'DingbatNegativeCircledDigitFour', [], 'Display', None],
  [ 0x277a, 'DingbatNegativeCircledDigitFive', [], 'Display', None],
  [ 0x277b, 'DingbatNegativeCircledDigitSix', [], 'Display', None],
  [ 0x277c, 'DingbatNegativeCircledDigitSeven', [], 'Display', None],
  [ 0x277d, 'DingbatNegativeCircledDigitEight', [], 'Display', None],
  [ 0x277e, 'DingbatNegativeCircledDigitNine', [], 'Display', None],
  [ 0x277f, 'DingbatNegativeCircledNumberTen', [], 'Display', None],
  [ 0x2780, 'DingbatCircledSansserifDigitOne', [], 'Display', None],
  [ 0x2781, 'DingbatCircledSansserifDigitTwo', [], 'Display', None],
  [ 0x2782, 'DingbatCircledSansserifDigitThree', [], 'Display', None],
  [ 0x2783, 'DingbatCircledSansserifDigitFour', [], 'Display', None],
  [ 0x2784, 'DingbatCircledSansserifDigitFive', [], 'Display', None],
  [ 0x2785, 'DingbatCircledSansserifDigitSix', [], 'Display', None],
  [ 0x2786, 'DingbatCircledSansserifDigitSeven', [], 'Display', None],
  [ 0x2787, 'DingbatCircledSansserifDigitEight', [], 'Display', None],
  [ 0x2788, 'DingbatCircledSansserifDigitNine', [], 'Display', None],
  [ 0x2789, 'DingbatCircledSansserifNumberTen', [], 'Display', None],
  [ 0x278a, 'DingbatNegativeCircledSansserifDigitOne', [], 'Display', None],
  [ 0x278b, 'DingbatNegativeCircledSansserifDigitTwo', [], 'Display', None],
  [ 0x278c, 'DingbatNegativeCircledSansserifDigitThree', [], 'Display', None],
  [ 0x278d, 'DingbatNegativeCircledSansserifDigitFour', [], 'Display', None],
  [ 0x278e, 'DingbatNegativeCircledSansserifDigitFive', [], 'Display', None],
  [ 0x278f, 'DingbatNegativeCircledSansserifDigitSix', [], 'Display', None],
  [ 0x2790, 'DingbatNegativeCircledSansserifDigitSeven', [], 'Display', None],
  [ 0x2791, 'DingbatNegativeCircledSansserifDigitEight', [], 'Display', None],
  [ 0x2792, 'DingbatNegativeCircledSansserifDigitNine', [], 'Display', None],
  [ 0x2793, 'DingbatNegativeCircledSansserifNumberTen', [], 'Display', None],
  [ 0x2794, 'HeavyWideheadedRightwardsArrow', [], 'Display', None],
  [ 0x2795, 'UnicodeCharacter0x2795', [], 'Display', None],
  [ 0x2796, 'UnicodeCharacter0x2796', [], 'Display', None],
  [ 0x2797, 'UnicodeCharacter0x2797', [], 'Display', None],
  [ 0x2798, 'HeavySouthEastArrow', [], 'Display', None],
  [ 0x2799, 'HeavyRightwardsArrow', [], 'Display', None],
  [ 0x279a, 'HeavyNorthEastArrow', [], 'Display', None],
  [ 0x279b, 'DraftingPointRightwardsArrow', [], 'Display', None],
  [ 0x279c, 'HeavyRoundtippedRightwardsArrow', [], 'Display', None],
  [ 0x279d, 'TriangleheadedRightwardsArrow', [], 'Display', None],
  [ 0x279e, 'HeavyTriangleheadedRightwardsArrow', [], 'Display', None],
  [ 0x279f, 'DashedTriangleheadedRightwardsArrow', [], 'Display', None],
  [ 0x27a0, 'HeavyDashedTriangleheadedRightwardsArrow', [], 'Display', None],
  [ 0x27a1, 'BlackRightwardsArrow', [], 'Display', None],
  [ 0x27a2, 'ThreedToplightedRightwardsArrowhead', [], 'Display', None],
  [ 0x27a3, 'ThreedBottomlightedRightwardsArrowhead', [], 'Display', None],
  [ 0x27a4, 'BlackRightwardsArrowhead', [], 'Display', None],
  [ 0x27a5, 'HeavyBlackCurvedDownwardsAndRightwardsArrow', [], 'Display', None],
  [ 0x27a6, 'HeavyBlackCurvedUpwardsAndRightwardsArrow', [], 'Display', None],
  [ 0x27a7, 'SquatBlackRightwardsArrow', [], 'Display', None],
  [ 0x27a8, 'HeavyConcavepointedBlackRightwardsArrow', [], 'Display', None],
  [ 0x27a9, 'RightshadedWhiteRightwardsArrow', [], 'Display', None],
  [ 0x27aa, 'LeftshadedWhiteRightwardsArrow', [], 'Display', None],
  [ 0x27ab, 'BacktiltedShadowedWhiteRightwardsArrow', [], 'Display', None],
  [ 0x27ac, 'FronttiltedShadowedWhiteRightwardsArrow', [], 'Display', None],
  [ 0x27ad, 'HeavyLowerRightshadowedWhiteRightwardsArrow', [], 'Display', None],
  [ 0x27ae, 'HeavyUpperRightshadowedWhiteRightwardsArrow', [], 'Display', None],
  [ 0x27af, 'NotchedLowerRightshadowedWhiteRightwardsArrow', [], 'Display', None],
  [ 0x27b0, 'UnicodeCharacter0x27b0', [], 'Display', None],
  [ 0x27b1, 'NotchedUpperRightshadowedWhiteRightwardsArrow', [], 'Display', None],
  [ 0x27b2, 'CircledHeavyWhiteRightwardsArrow', [], 'Display', None],
  [ 0x27b3, 'WhitefeatheredRightwardsArrow', [], 'Display', None],
  [ 0x27b4, 'BlackfeatheredSouthEastArrow', [], 'Display', None],
  [ 0x27b5, 'BlackfeatheredRightwardsArrow', [], 'Display', None],
  [ 0x27b6, 'BlackfeatheredNorthEastArrow', [], 'Display', None],
  [ 0x27b7, 'HeavyBlackfeatheredSouthEastArrow', [], 'Display', None],
  [ 0x27b8, 'HeavyBlackfeatheredRightwardsArrow', [], 'Display', None],
  [ 0x27b9, 'HeavyBlackfeatheredNorthEastArrow', [], 'Display', None],
  [ 0x27ba, 'TeardropbarbedRightwardsArrow', [], 'Display', None],
  [ 0x27bb, 'HeavyTeardropshankedRightwardsArrow', [], 'Display', None],
  [ 0x27bc, 'WedgetailedRightwardsArrow', [], 'Display', None],
  [ 0x27bd, 'HeavyWedgetailedRightwardsArrow', [], 'Display', None],
  [ 0x27be, 'OpenoutlinedRightwardsArrow', [], 'Display', None],
  [ 0x27bf, 'UnicodeCharacter0x27bf', [], 'Display', None],
  [ 0x27c0, 'ThreeDimensionalAngle', [], 'Display', None],
  [ 0x27c1, 'WhiteTriangleContainingSmallWhiteTriangle', [], 'Display', None],
  [ 0x27c2, 'Perpendicular', [], 'Display', None],
  [ 0x27c3, 'OpenSubset', [], 'Display', None],
  [ 0x27c4, 'OpenSuperset', [], 'Display', None],
  [ 0x27c5, 'LeftSshapedBagDelimiter', [], 'Display', None],
  [ 0x27c6, 'RightSshapedBagDelimiter', [], 'Display', None],
  [ 0x27c7, 'OrWithDotInside', [], 'Display', None],
  [ 0x27c8, 'ReverseSolidusPrecedingSubset', [], 'Display', None],
  [ 0x27c9, 'SupersetPrecedingSolidus', [], 'Display', None],
  [ 0x27ca, 'VerticalBarWithHorizontalStroke', [], 'Display', None],
  [ 0x27cb, 'UnicodeCharacter0x27cb', [], 'Display', None],
  [ 0x27cc, 'LongDivision', [], 'Display', None],
  [ 0x27cd, 'UnicodeCharacter0x27cd', [], 'Display', None],
  [ 0x27ce, 'UnicodeCharacter0x27ce', [], 'Display', None],
  [ 0x27cf, 'UnicodeCharacter0x27cf', [], 'Display', None],
  [ 0x27d0, 'WhiteDiamondWithCentredDot', [], 'Display', None],
  [ 0x27d1, 'AndWithDot', [], 'Display', None],
  [ 0x27d2, 'ElementOfOpeningUpwards', [], 'Display', None],
  [ 0x27d3, 'LowerRightCornerWithDot', [], 'Display', None],
  [ 0x27d4, 'UpperLeftCornerWithDot', [], 'Display', None],
  [ 0x27d5, 'LeftOuterJoin', [], 'Display', None],
  [ 0x27d6, 'RightOuterJoin', [], 'Display', None],
  [ 0x27d7, 'FullOuterJoin', [], 'Display', None],
  [ 0x27d8, 'LargeUpTack', [], 'Display', None],
  [ 0x27d9, 'LargeDownTack', [], 'Display', None],
  [ 0x27da, 'LeftAndRightDoubleTurnstile', [], 'Display', None],
  [ 0x27db, 'LeftAndRightTack', [], 'Display', None],
  [ 0x27dc, 'LeftMultimap', [], 'Display', None],
  [ 0x27dd, 'LongRightTack', [], 'Display', None],
  [ 0x27de, 'LongLeftTack', [], 'Display', None],
  [ 0x27df, 'UpTackWithCircleAbove', [], 'Display', None],
  [ 0x27e0, 'LozengeDividedByHorizontalRule', [], 'Display', None],
  [ 0x27e1, 'WhiteConcavesidedDiamond', [], 'Display', None],
  [ 0x27e2, 'WhiteConcavesidedDiamondWithLeftwardsTick', [], 'Display', None],
  [ 0x27e3, 'WhiteConcavesidedDiamondWithRightwardsTick', [], 'Display', None],
  [ 0x27e4, 'WhiteSquareWithLeftwardsTick', [], 'Display', None],
  [ 0x27e5, 'WhiteSquareWithRightwardsTick', [], 'Display', None],
  [ 0x27e6, 'MathematicalLeftWhiteSquareBracket', [], 'Display', None],
  [ 0x27e7, 'MathematicalRightWhiteSquareBracket', [], 'Display', None],
  [ 0x27e8, 'MathematicalLeftAngleBracket', [], 'Display', None],
  [ 0x27e9, 'MathematicalRightAngleBracket', [], 'Display', None],
  [ 0x27ea, 'MathematicalLeftDoubleAngleBracket', [], 'Display', None],
  [ 0x27eb, 'MathematicalRightDoubleAngleBracket', [], 'Display', None],
  [ 0x27ec, 'MathematicalLeftWhiteTortoiseShellBracket', [], 'Display', None],
  [ 0x27ed, 'MathematicalRightWhiteTortoiseShellBracket', [], 'Display', None],
  [ 0x27ee, 'MathematicalLeftFlattenedParenthesis', [], 'Display', None],
  [ 0x27ef, 'MathematicalRightFlattenedParenthesis', [], 'Display', None],
  [ 0x27f0, 'UpwardsQuadrupleArrow', [], 'Display', None],
  [ 0x27f1, 'DownwardsQuadrupleArrow', [], 'Display', None],
  [ 0x27f2, 'AnticlockwiseGappedCircleArrow', [], 'Display', None],
  [ 0x27f3, 'ClockwiseGappedCircleArrow', [], 'Display', None],
  [ 0x27f4, 'RightArrowWithCircledPlus', [], 'Display', None],
  [ 0x27f5, 'LongLeftwardsArrow', [], 'Display', None],
  [ 0x27f6, 'LongRightwardsArrow', [], 'Display', None],
  [ 0x27f7, 'LongLeftRightArrow', [], 'Display', None],
  [ 0x27f8, 'LongLeftwardsDoubleArrow', [], 'Display', None],
  [ 0x27f9, 'LongRightwardsDoubleArrow', [], 'Display', None],
  [ 0x27fa, 'LongLeftRightDoubleArrow', [], 'Display', None],
  [ 0x27fb, 'LongLeftwardsArrowFromBar', [], 'Display', None],
  [ 0x27fc, 'LongRightwardsArrowFromBar', [], 'Display', None],
  [ 0x27fd, 'LongLeftwardsDoubleArrowFromBar', [], 'Display', None],
  [ 0x27fe, 'LongRightwardsDoubleArrowFromBar', [], 'Display', None],
  [ 0x27ff, 'LongRightwardsSquiggleArrow', [], 'Display', None],
  [ 0x2800, 'BraillePatternBlank', [], 'Display', None],
  [ 0x2801, 'BraillePatternDots1', [], 'Display', None],
  [ 0x2802, 'BraillePatternDots2', [], 'Display', None],
  [ 0x2803, 'BraillePatternDots12', [], 'Display', None],
  [ 0x2804, 'BraillePatternDots3', [], 'Display', None],
  [ 0x2805, 'BraillePatternDots13', [], 'Display', None],
  [ 0x2806, 'BraillePatternDots23', [], 'Display', None],
  [ 0x2807, 'BraillePatternDots123', [], 'Display', None],
  [ 0x2808, 'BraillePatternDots4', [], 'Display', None],
  [ 0x2809, 'BraillePatternDots14', [], 'Display', None],
  [ 0x280a, 'BraillePatternDots24', [], 'Display', None],
  [ 0x280b, 'BraillePatternDots124', [], 'Display', None],
  [ 0x280c, 'BraillePatternDots34', [], 'Display', None],
  [ 0x280d, 'BraillePatternDots134', [], 'Display', None],
  [ 0x280e, 'BraillePatternDots234', [], 'Display', None],
  [ 0x280f, 'BraillePatternDots1234', [], 'Display', None],
  [ 0x2810, 'BraillePatternDots5', [], 'Display', None],
  [ 0x2811, 'BraillePatternDots15', [], 'Display', None],
  [ 0x2812, 'BraillePatternDots25', [], 'Display', None],
  [ 0x2813, 'BraillePatternDots125', [], 'Display', None],
  [ 0x2814, 'BraillePatternDots35', [], 'Display', None],
  [ 0x2815, 'BraillePatternDots135', [], 'Display', None],
  [ 0x2816, 'BraillePatternDots235', [], 'Display', None],
  [ 0x2817, 'BraillePatternDots1235', [], 'Display', None],
  [ 0x2818, 'BraillePatternDots45', [], 'Display', None],
  [ 0x2819, 'BraillePatternDots145', [], 'Display', None],
  [ 0x281a, 'BraillePatternDots245', [], 'Display', None],
  [ 0x281b, 'BraillePatternDots1245', [], 'Display', None],
  [ 0x281c, 'BraillePatternDots345', [], 'Display', None],
  [ 0x281d, 'BraillePatternDots1345', [], 'Display', None],
  [ 0x281e, 'BraillePatternDots2345', [], 'Display', None],
  [ 0x281f, 'BraillePatternDots12345', [], 'Display', None],
  [ 0x2820, 'BraillePatternDots6', [], 'Display', None],
  [ 0x2821, 'BraillePatternDots16', [], 'Display', None],
  [ 0x2822, 'BraillePatternDots26', [], 'Display', None],
  [ 0x2823, 'BraillePatternDots126', [], 'Display', None],
  [ 0x2824, 'BraillePatternDots36', [], 'Display', None],
  [ 0x2825, 'BraillePatternDots136', [], 'Display', None],
  [ 0x2826, 'BraillePatternDots236', [], 'Display', None],
  [ 0x2827, 'BraillePatternDots1236', [], 'Display', None],
  [ 0x2828, 'BraillePatternDots46', [], 'Display', None],
  [ 0x2829, 'BraillePatternDots146', [], 'Display', None],
  [ 0x282a, 'BraillePatternDots246', [], 'Display', None],
  [ 0x282b, 'BraillePatternDots1246', [], 'Display', None],
  [ 0x282c, 'BraillePatternDots346', [], 'Display', None],
  [ 0x282d, 'BraillePatternDots1346', [], 'Display', None],
  [ 0x282e, 'BraillePatternDots2346', [], 'Display', None],
  [ 0x282f, 'BraillePatternDots12346', [], 'Display', None],
  [ 0x2830, 'BraillePatternDots56', [], 'Display', None],
  [ 0x2831, 'BraillePatternDots156', [], 'Display', None],
  [ 0x2832, 'BraillePatternDots256', [], 'Display', None],
  [ 0x2833, 'BraillePatternDots1256', [], 'Display', None],
  [ 0x2834, 'BraillePatternDots356', [], 'Display', None],
  [ 0x2835, 'BraillePatternDots1356', [], 'Display', None],
  [ 0x2836, 'BraillePatternDots2356', [], 'Display', None],
  [ 0x2837, 'BraillePatternDots12356', [], 'Display', None],
  [ 0x2838, 'BraillePatternDots456', [], 'Display', None],
  [ 0x2839, 'BraillePatternDots1456', [], 'Display', None],
  [ 0x283a, 'BraillePatternDots2456', [], 'Display', None],
  [ 0x283b, 'BraillePatternDots12456', [], 'Display', None],
  [ 0x283c, 'BraillePatternDots3456', [], 'Display', None],
  [ 0x283d, 'BraillePatternDots13456', [], 'Display', None],
  [ 0x283e, 'BraillePatternDots23456', [], 'Display', None],
  [ 0x283f, 'BraillePatternDots123456', [], 'Display', None],
  [ 0x2840, 'BraillePatternDots7', [], 'Display', None],
  [ 0x2841, 'BraillePatternDots17', [], 'Display', None],
  [ 0x2842, 'BraillePatternDots27', [], 'Display', None],
  [ 0x2843, 'BraillePatternDots127', [], 'Display', None],
  [ 0x2844, 'BraillePatternDots37', [], 'Display', None],
  [ 0x2845, 'BraillePatternDots137', [], 'Display', None],
  [ 0x2846, 'BraillePatternDots237', [], 'Display', None],
  [ 0x2847, 'BraillePatternDots1237', [], 'Display', None],
  [ 0x2848, 'BraillePatternDots47', [], 'Display', None],
  [ 0x2849, 'BraillePatternDots147', [], 'Display', None],
  [ 0x284a, 'BraillePatternDots247', [], 'Display', None],
  [ 0x284b, 'BraillePatternDots1247', [], 'Display', None],
  [ 0x284c, 'BraillePatternDots347', [], 'Display', None],
  [ 0x284d, 'BraillePatternDots1347', [], 'Display', None],
  [ 0x284e, 'BraillePatternDots2347', [], 'Display', None],
  [ 0x284f, 'BraillePatternDots12347', [], 'Display', None],
  [ 0x2850, 'BraillePatternDots57', [], 'Display', None],
  [ 0x2851, 'BraillePatternDots157', [], 'Display', None],
  [ 0x2852, 'BraillePatternDots257', [], 'Display', None],
  [ 0x2853, 'BraillePatternDots1257', [], 'Display', None],
  [ 0x2854, 'BraillePatternDots357', [], 'Display', None],
  [ 0x2855, 'BraillePatternDots1357', [], 'Display', None],
  [ 0x2856, 'BraillePatternDots2357', [], 'Display', None],
  [ 0x2857, 'BraillePatternDots12357', [], 'Display', None],
  [ 0x2858, 'BraillePatternDots457', [], 'Display', None],
  [ 0x2859, 'BraillePatternDots1457', [], 'Display', None],
  [ 0x285a, 'BraillePatternDots2457', [], 'Display', None],
  [ 0x285b, 'BraillePatternDots12457', [], 'Display', None],
  [ 0x285c, 'BraillePatternDots3457', [], 'Display', None],
  [ 0x285d, 'BraillePatternDots13457', [], 'Display', None],
  [ 0x285e, 'BraillePatternDots23457', [], 'Display', None],
  [ 0x285f, 'BraillePatternDots123457', [], 'Display', None],
  [ 0x2860, 'BraillePatternDots67', [], 'Display', None],
  [ 0x2861, 'BraillePatternDots167', [], 'Display', None],
  [ 0x2862, 'BraillePatternDots267', [], 'Display', None],
  [ 0x2863, 'BraillePatternDots1267', [], 'Display', None],
  [ 0x2864, 'BraillePatternDots367', [], 'Display', None],
  [ 0x2865, 'BraillePatternDots1367', [], 'Display', None],
  [ 0x2866, 'BraillePatternDots2367', [], 'Display', None],
  [ 0x2867, 'BraillePatternDots12367', [], 'Display', None],
  [ 0x2868, 'BraillePatternDots467', [], 'Display', None],
  [ 0x2869, 'BraillePatternDots1467', [], 'Display', None],
  [ 0x286a, 'BraillePatternDots2467', [], 'Display', None],
  [ 0x286b, 'BraillePatternDots12467', [], 'Display', None],
  [ 0x286c, 'BraillePatternDots3467', [], 'Display', None],
  [ 0x286d, 'BraillePatternDots13467', [], 'Display', None],
  [ 0x286e, 'BraillePatternDots23467', [], 'Display', None],
  [ 0x286f, 'BraillePatternDots123467', [], 'Display', None],
  [ 0x2870, 'BraillePatternDots567', [], 'Display', None],
  [ 0x2871, 'BraillePatternDots1567', [], 'Display', None],
  [ 0x2872, 'BraillePatternDots2567', [], 'Display', None],
  [ 0x2873, 'BraillePatternDots12567', [], 'Display', None],
  [ 0x2874, 'BraillePatternDots3567', [], 'Display', None],
  [ 0x2875, 'BraillePatternDots13567', [], 'Display', None],
  [ 0x2876, 'BraillePatternDots23567', [], 'Display', None],
  [ 0x2877, 'BraillePatternDots123567', [], 'Display', None],
  [ 0x2878, 'BraillePatternDots4567', [], 'Display', None],
  [ 0x2879, 'BraillePatternDots14567', [], 'Display', None],
  [ 0x287a, 'BraillePatternDots24567', [], 'Display', None],
  [ 0x287b, 'BraillePatternDots124567', [], 'Display', None],
  [ 0x287c, 'BraillePatternDots34567', [], 'Display', None],
  [ 0x287d, 'BraillePatternDots134567', [], 'Display', None],
  [ 0x287e, 'BraillePatternDots234567', [], 'Display', None],
  [ 0x287f, 'BraillePatternDots1234567', [], 'Display', None],
  [ 0x2880, 'BraillePatternDots8', [], 'Display', None],
  [ 0x2881, 'BraillePatternDots18', [], 'Display', None],
  [ 0x2882, 'BraillePatternDots28', [], 'Display', None],
  [ 0x2883, 'BraillePatternDots128', [], 'Display', None],
  [ 0x2884, 'BraillePatternDots38', [], 'Display', None],
  [ 0x2885, 'BraillePatternDots138', [], 'Display', None],
  [ 0x2886, 'BraillePatternDots238', [], 'Display', None],
  [ 0x2887, 'BraillePatternDots1238', [], 'Display', None],
  [ 0x2888, 'BraillePatternDots48', [], 'Display', None],
  [ 0x2889, 'BraillePatternDots148', [], 'Display', None],
  [ 0x288a, 'BraillePatternDots248', [], 'Display', None],
  [ 0x288b, 'BraillePatternDots1248', [], 'Display', None],
  [ 0x288c, 'BraillePatternDots348', [], 'Display', None],
  [ 0x288d, 'BraillePatternDots1348', [], 'Display', None],
  [ 0x288e, 'BraillePatternDots2348', [], 'Display', None],
  [ 0x288f, 'BraillePatternDots12348', [], 'Display', None],
  [ 0x2890, 'BraillePatternDots58', [], 'Display', None],
  [ 0x2891, 'BraillePatternDots158', [], 'Display', None],
  [ 0x2892, 'BraillePatternDots258', [], 'Display', None],
  [ 0x2893, 'BraillePatternDots1258', [], 'Display', None],
  [ 0x2894, 'BraillePatternDots358', [], 'Display', None],
  [ 0x2895, 'BraillePatternDots1358', [], 'Display', None],
  [ 0x2896, 'BraillePatternDots2358', [], 'Display', None],
  [ 0x2897, 'BraillePatternDots12358', [], 'Display', None],
  [ 0x2898, 'BraillePatternDots458', [], 'Display', None],
  [ 0x2899, 'BraillePatternDots1458', [], 'Display', None],
  [ 0x289a, 'BraillePatternDots2458', [], 'Display', None],
  [ 0x289b, 'BraillePatternDots12458', [], 'Display', None],
  [ 0x289c, 'BraillePatternDots3458', [], 'Display', None],
  [ 0x289d, 'BraillePatternDots13458', [], 'Display', None],
  [ 0x289e, 'BraillePatternDots23458', [], 'Display', None],
  [ 0x289f, 'BraillePatternDots123458', [], 'Display', None],
  [ 0x28a0, 'BraillePatternDots68', [], 'Display', None],
  [ 0x28a1, 'BraillePatternDots168', [], 'Display', None],
  [ 0x28a2, 'BraillePatternDots268', [], 'Display', None],
  [ 0x28a3, 'BraillePatternDots1268', [], 'Display', None],
  [ 0x28a4, 'BraillePatternDots368', [], 'Display', None],
  [ 0x28a5, 'BraillePatternDots1368', [], 'Display', None],
  [ 0x28a6, 'BraillePatternDots2368', [], 'Display', None],
  [ 0x28a7, 'BraillePatternDots12368', [], 'Display', None],
  [ 0x28a8, 'BraillePatternDots468', [], 'Display', None],
  [ 0x28a9, 'BraillePatternDots1468', [], 'Display', None],
  [ 0x28aa, 'BraillePatternDots2468', [], 'Display', None],
  [ 0x28ab, 'BraillePatternDots12468', [], 'Display', None],
  [ 0x28ac, 'BraillePatternDots3468', [], 'Display', None],
  [ 0x28ad, 'BraillePatternDots13468', [], 'Display', None],
  [ 0x28ae, 'BraillePatternDots23468', [], 'Display', None],
  [ 0x28af, 'BraillePatternDots123468', [], 'Display', None],
  [ 0x28b0, 'BraillePatternDots568', [], 'Display', None],
  [ 0x28b1, 'BraillePatternDots1568', [], 'Display', None],
  [ 0x28b2, 'BraillePatternDots2568', [], 'Display', None],
  [ 0x28b3, 'BraillePatternDots12568', [], 'Display', None],
  [ 0x28b4, 'BraillePatternDots3568', [], 'Display', None],
  [ 0x28b5, 'BraillePatternDots13568', [], 'Display', None],
  [ 0x28b6, 'BraillePatternDots23568', [], 'Display', None],
  [ 0x28b7, 'BraillePatternDots123568', [], 'Display', None],
  [ 0x28b8, 'BraillePatternDots4568', [], 'Display', None],
  [ 0x28b9, 'BraillePatternDots14568', [], 'Display', None],
  [ 0x28ba, 'BraillePatternDots24568', [], 'Display', None],
  [ 0x28bb, 'BraillePatternDots124568', [], 'Display', None],
  [ 0x28bc, 'BraillePatternDots34568', [], 'Display', None],
  [ 0x28bd, 'BraillePatternDots134568', [], 'Display', None],
  [ 0x28be, 'BraillePatternDots234568', [], 'Display', None],
  [ 0x28bf, 'BraillePatternDots1234568', [], 'Display', None],
  [ 0x28c0, 'BraillePatternDots78', [], 'Display', None],
  [ 0x28c1, 'BraillePatternDots178', [], 'Display', None],
  [ 0x28c2, 'BraillePatternDots278', [], 'Display', None],
  [ 0x28c3, 'BraillePatternDots1278', [], 'Display', None],
  [ 0x28c4, 'BraillePatternDots378', [], 'Display', None],
  [ 0x28c5, 'BraillePatternDots1378', [], 'Display', None],
  [ 0x28c6, 'BraillePatternDots2378', [], 'Display', None],
  [ 0x28c7, 'BraillePatternDots12378', [], 'Display', None],
  [ 0x28c8, 'BraillePatternDots478', [], 'Display', None],
  [ 0x28c9, 'BraillePatternDots1478', [], 'Display', None],
  [ 0x28ca, 'BraillePatternDots2478', [], 'Display', None],
  [ 0x28cb, 'BraillePatternDots12478', [], 'Display', None],
  [ 0x28cc, 'BraillePatternDots3478', [], 'Display', None],
  [ 0x28cd, 'BraillePatternDots13478', [], 'Display', None],
  [ 0x28ce, 'BraillePatternDots23478', [], 'Display', None],
  [ 0x28cf, 'BraillePatternDots123478', [], 'Display', None],
  [ 0x28d0, 'BraillePatternDots578', [], 'Display', None],
  [ 0x28d1, 'BraillePatternDots1578', [], 'Display', None],
  [ 0x28d2, 'BraillePatternDots2578', [], 'Display', None],
  [ 0x28d3, 'BraillePatternDots12578', [], 'Display', None],
  [ 0x28d4, 'BraillePatternDots3578', [], 'Display', None],
  [ 0x28d5, 'BraillePatternDots13578', [], 'Display', None],
  [ 0x28d6, 'BraillePatternDots23578', [], 'Display', None],
  [ 0x28d7, 'BraillePatternDots123578', [], 'Display', None],
  [ 0x28d8, 'BraillePatternDots4578', [], 'Display', None],
  [ 0x28d9, 'BraillePatternDots14578', [], 'Display', None],
  [ 0x28da, 'BraillePatternDots24578', [], 'Display', None],
  [ 0x28db, 'BraillePatternDots124578', [], 'Display', None],
  [ 0x28dc, 'BraillePatternDots34578', [], 'Display', None],
  [ 0x28dd, 'BraillePatternDots134578', [], 'Display', None],
  [ 0x28de, 'BraillePatternDots234578', [], 'Display', None],
  [ 0x28df, 'BraillePatternDots1234578', [], 'Display', None],
  [ 0x28e0, 'BraillePatternDots678', [], 'Display', None],
  [ 0x28e1, 'BraillePatternDots1678', [], 'Display', None],
  [ 0x28e2, 'BraillePatternDots2678', [], 'Display', None],
  [ 0x28e3, 'BraillePatternDots12678', [], 'Display', None],
  [ 0x28e4, 'BraillePatternDots3678', [], 'Display', None],
  [ 0x28e5, 'BraillePatternDots13678', [], 'Display', None],
  [ 0x28e6, 'BraillePatternDots23678', [], 'Display', None],
  [ 0x28e7, 'BraillePatternDots123678', [], 'Display', None],
  [ 0x28e8, 'BraillePatternDots4678', [], 'Display', None],
  [ 0x28e9, 'BraillePatternDots14678', [], 'Display', None],
  [ 0x28ea, 'BraillePatternDots24678', [], 'Display', None],
  [ 0x28eb, 'BraillePatternDots124678', [], 'Display', None],
  [ 0x28ec, 'BraillePatternDots34678', [], 'Display', None],
  [ 0x28ed, 'BraillePatternDots134678', [], 'Display', None],
  [ 0x28ee, 'BraillePatternDots234678', [], 'Display', None],
  [ 0x28ef, 'BraillePatternDots1234678', [], 'Display', None],
  [ 0x28f0, 'BraillePatternDots5678', [], 'Display', None],
  [ 0x28f1, 'BraillePatternDots15678', [], 'Display', None],
  [ 0x28f2, 'BraillePatternDots25678', [], 'Display', None],
  [ 0x28f3, 'BraillePatternDots125678', [], 'Display', None],
  [ 0x28f4, 'BraillePatternDots35678', [], 'Display', None],
  [ 0x28f5, 'BraillePatternDots135678', [], 'Display', None],
  [ 0x28f6, 'BraillePatternDots235678', [], 'Display', None],
  [ 0x28f7, 'BraillePatternDots1235678', [], 'Display', None],
  [ 0x28f8, 'BraillePatternDots45678', [], 'Display', None],
  [ 0x28f9, 'BraillePatternDots145678', [], 'Display', None],
  [ 0x28fa, 'BraillePatternDots245678', [], 'Display', None],
  [ 0x28fb, 'BraillePatternDots1245678', [], 'Display', None],
  [ 0x28fc, 'BraillePatternDots345678', [], 'Display', None],
  [ 0x28fd, 'BraillePatternDots1345678', [], 'Display', None],
  [ 0x28fe, 'BraillePatternDots2345678', [], 'Display', None],
  [ 0x28ff, 'BraillePatternDots12345678', [], 'Display', None],
  [ 0x2900, 'RightwardsTwoheadedArrowWithVerticalStroke', [], 'Display', None],
  [ 0x2901, 'RightwardsTwoheadedArrowWithDoubleVerticalStroke', [], 'Display', None],
  [ 0x2902, 'LeftwardsDoubleArrowWithVerticalStroke', [], 'Display', None],
  [ 0x2903, 'RightwardsDoubleArrowWithVerticalStroke', [], 'Display', None],
  [ 0x2904, 'LeftRightDoubleArrowWithVerticalStroke', [], 'Display', None],
  [ 0x2905, 'RightwardsTwoheadedArrowFromBar', [], 'Display', None],
  [ 0x2906, 'LeftwardsDoubleArrowFromBar', [], 'Display', None],
  [ 0x2907, 'RightwardsDoubleArrowFromBar', [], 'Display', None],
  [ 0x2908, 'DownwardsArrowWithHorizontalStroke', [], 'Display', None],
  [ 0x2909, 'UpwardsArrowWithHorizontalStroke', [], 'Display', None],
  [ 0x290a, 'UpwardsTripleArrow', [], 'Display', None],
  [ 0x290b, 'DownwardsTripleArrow', [], 'Display', None],
  [ 0x290c, 'LeftwardsDoubleDashArrow', [], 'Display', None],
  [ 0x290d, 'RightwardsDoubleDashArrow', [], 'Display', None],
  [ 0x290e, 'LeftwardsTripleDashArrow', [], 'Display', None],
  [ 0x290f, 'RightwardsTripleDashArrow', [], 'Display', None],
  [ 0x2910, 'RightwardsTwoheadedTripleDashArrow', [], 'Display', None],
  [ 0x2911, 'RightwardsArrowWithDottedStem', [], 'Display', None],
  [ 0x2912, 'UpwardsArrowToBar', [], 'Display', None],
  [ 0x2913, 'DownwardsArrowToBar', [], 'Display', None],
  [ 0x2914, 'RightwardsArrowWithTailWithVerticalStroke', [], 'Display', None],
  [ 0x2915, 'RightwardsArrowWithTailWithDoubleVerticalStroke', [], 'Display', None],
  [ 0x2916, 'RightwardsTwoheadedArrowWithTail', [], 'Display', None],
  [ 0x2917, 'RightwardsTwoheadedArrowWithTailWithVerticalStroke', [], 'Display', None],
  [ 0x2918, 'RightwardsTwoheadedArrowWithTailWithDoubleVerticalStroke', [], 'Display', None],
  [ 0x2919, 'LeftwardsArrowtail', [], 'Display', None],
  [ 0x291a, 'RightwardsArrowtail', [], 'Display', None],
  [ 0x291b, 'LeftwardsDoubleArrowtail', [], 'Display', None],
  [ 0x291c, 'RightwardsDoubleArrowtail', [], 'Display', None],
  [ 0x291d, 'LeftwardsArrowToBlackDiamond', [], 'Display', None],
  [ 0x291e, 'RightwardsArrowToBlackDiamond', [], 'Display', None],
  [ 0x291f, 'LeftwardsArrowFromBarToBlackDiamond', [], 'Display', None],
  [ 0x2920, 'RightwardsArrowFromBarToBlackDiamond', [], 'Display', None],
  [ 0x2921, 'NorthWestAndSouthEastArrow', [], 'Display', None],
  [ 0x2922, 'NorthEastAndSouthWestArrow', [], 'Display', None],
  [ 0x2923, 'NorthWestArrowWithHook', [], 'Display', None],
  [ 0x2924, 'NorthEastArrowWithHook', [], 'Display', None],
  [ 0x2925, 'SouthEastArrowWithHook', [], 'Display', None],
  [ 0x2926, 'SouthWestArrowWithHook', [], 'Display', None],
  [ 0x2927, 'NorthWestArrowAndNorthEastArrow', [], 'Display', None],
  [ 0x2928, 'NorthEastArrowAndSouthEastArrow', [], 'Display', None],
  [ 0x2929, 'SouthEastArrowAndSouthWestArrow', [], 'Display', None],
  [ 0x292a, 'SouthWestArrowAndNorthWestArrow', [], 'Display', None],
  [ 0x292b, 'RisingDiagonalCrossingFallingDiagonal', [], 'Display', None],
  [ 0x292c, 'FallingDiagonalCrossingRisingDiagonal', [], 'Display', None],
  [ 0x292d, 'SouthEastArrowCrossingNorthEastArrow', [], 'Display', None],
  [ 0x292e, 'NorthEastArrowCrossingSouthEastArrow', [], 'Display', None],
  [ 0x292f, 'FallingDiagonalCrossingNorthEastArrow', [], 'Display', None],
  [ 0x2930, 'RisingDiagonalCrossingSouthEastArrow', [], 'Display', None],
  [ 0x2931, 'NorthEastArrowCrossingNorthWestArrow', [], 'Display', None],
  [ 0x2932, 'NorthWestArrowCrossingNorthEastArrow', [], 'Display', None],
  [ 0x2933, 'WaveArrowPointingDirectlyRight', [], 'Display', None],
  [ 0x2934, 'ArrowPointingRightwardsThenCurvingUpwards', [], 'Display', None],
  [ 0x2935, 'ArrowPointingRightwardsThenCurvingDownwards', [], 'Display', None],
  [ 0x2936, 'ArrowPointingDownwardsThenCurvingLeftwards', [], 'Display', None],
  [ 0x2937, 'ArrowPointingDownwardsThenCurvingRightwards', [], 'Display', None],
  [ 0x2938, 'RightsideArcClockwiseArrow', [], 'Display', None],
  [ 0x2939, 'LeftsideArcAnticlockwiseArrow', [], 'Display', None],
  [ 0x293a, 'TopArcAnticlockwiseArrow', [], 'Display', None],
  [ 0x293b, 'BottomArcAnticlockwiseArrow', [], 'Display', None],
  [ 0x293c, 'TopArcClockwiseArrowWithMinus', [], 'Display', None],
  [ 0x293d, 'TopArcAnticlockwiseArrowWithPlus', [], 'Display', None],
  [ 0x293e, 'LowerRightSemicircularClockwiseArrow', [], 'Display', None],
  [ 0x293f, 'LowerLeftSemicircularAnticlockwiseArrow', [], 'Display', None],
  [ 0x2940, 'AnticlockwiseClosedCircleArrow', [], 'Display', None],
  [ 0x2941, 'ClockwiseClosedCircleArrow', [], 'Display', None],
  [ 0x2942, 'RightwardsArrowAboveShortLeftwardsArrow', [], 'Display', None],
  [ 0x2943, 'LeftwardsArrowAboveShortRightwardsArrow', [], 'Display', None],
  [ 0x2944, 'ShortRightwardsArrowAboveLeftwardsArrow', [], 'Display', None],
  [ 0x2945, 'RightwardsArrowWithPlusBelow', [], 'Display', None],
  [ 0x2946, 'LeftwardsArrowWithPlusBelow', [], 'Display', None],
  [ 0x2947, 'RightwardsArrowThroughX', [], 'Display', None],
  [ 0x2948, 'LeftRightArrowThroughSmallCircle', [], 'Display', None],
  [ 0x2949, 'UpwardsTwoheadedArrowFromSmallCircle', [], 'Display', None],
  [ 0x294a, 'LeftBarbUpRightBarbDownHarpoon', [], 'Display', None],
  [ 0x294b, 'LeftBarbDownRightBarbUpHarpoon', [], 'Display', None],
  [ 0x294c, 'UpBarbRightDownBarbLeftHarpoon', [], 'Display', None],
  [ 0x294d, 'UpBarbLeftDownBarbRightHarpoon', [], 'Display', None],
  [ 0x294e, 'LeftBarbUpRightBarbUpHarpoon', [], 'Display', None],
  [ 0x294f, 'UpBarbRightDownBarbRightHarpoon', [], 'Display', None],
  [ 0x2950, 'LeftBarbDownRightBarbDownHarpoon', [], 'Display', None],
  [ 0x2951, 'UpBarbLeftDownBarbLeftHarpoon', [], 'Display', None],
  [ 0x2952, 'LeftwardsHarpoonWithBarbUpToBar', [], 'Display', None],
  [ 0x2953, 'RightwardsHarpoonWithBarbUpToBar', [], 'Display', None],
  [ 0x2954, 'UpwardsHarpoonWithBarbRightToBar', [], 'Display', None],
  [ 0x2955, 'DownwardsHarpoonWithBarbRightToBar', [], 'Display', None],
  [ 0x2956, 'LeftwardsHarpoonWithBarbDownToBar', [], 'Display', None],
  [ 0x2957, 'RightwardsHarpoonWithBarbDownToBar', [], 'Display', None],
  [ 0x2958, 'UpwardsHarpoonWithBarbLeftToBar', [], 'Display', None],
  [ 0x2959, 'DownwardsHarpoonWithBarbLeftToBar', [], 'Display', None],
  [ 0x295a, 'LeftwardsHarpoonWithBarbUpFromBar', [], 'Display', None],
  [ 0x295b, 'RightwardsHarpoonWithBarbUpFromBar', [], 'Display', None],
  [ 0x295c, 'UpwardsHarpoonWithBarbRightFromBar', [], 'Display', None],
  [ 0x295d, 'DownwardsHarpoonWithBarbRightFromBar', [], 'Display', None],
  [ 0x295e, 'LeftwardsHarpoonWithBarbDownFromBar', [], 'Display', None],
  [ 0x295f, 'RightwardsHarpoonWithBarbDownFromBar', [], 'Display', None],
  [ 0x2960, 'UpwardsHarpoonWithBarbLeftFromBar', [], 'Display', None],
  [ 0x2961, 'DownwardsHarpoonWithBarbLeftFromBar', [], 'Display', None],
  [ 0x2962, 'LeftwardsHarpoonWithBarbUpAboveLeftwardsHarpoonWithBarbDown', [], 'Display', None],
  [ 0x2963, 'UpwardsHarpoonWithBarbLeftBesideUpwardsHarpoonWithBarbRight', [], 'Display', None],
  [ 0x2964, 'RightwardsHarpoonWithBarbUpAboveRightwardsHarpoonWithBarbDown', [], 'Display', None],
  [ 0x2965, 'DownwardsHarpoonWithBarbLeftBesideDownwardsHarpoonWithBarbRight', [], 'Display', None],
  [ 0x2966, 'LeftwardsHarpoonWithBarbUpAboveRightwardsHarpoonWithBarbUp', [], 'Display', None],
  [ 0x2967, 'LeftwardsHarpoonWithBarbDownAboveRightwardsHarpoonWithBarbDown', [], 'Display', None],
  [ 0x2968, 'RightwardsHarpoonWithBarbUpAboveLeftwardsHarpoonWithBarbUp', [], 'Display', None],
  [ 0x2969, 'RightwardsHarpoonWithBarbDownAboveLeftwardsHarpoonWithBarbDown', [], 'Display', None],
  [ 0x296a, 'LeftwardsHarpoonWithBarbUpAboveLongDash', [], 'Display', None],
  [ 0x296b, 'LeftwardsHarpoonWithBarbDownBelowLongDash', [], 'Display', None],
  [ 0x296c, 'RightwardsHarpoonWithBarbUpAboveLongDash', [], 'Display', None],
  [ 0x296d, 'RightwardsHarpoonWithBarbDownBelowLongDash', [], 'Display', None],
  [ 0x296e, 'UpwardsHarpoonWithBarbLeftBesideDownwardsHarpoonWithBarbRight', [], 'Display', None],
  [ 0x296f, 'DownwardsHarpoonWithBarbLeftBesideUpwardsHarpoonWithBarbRight', [], 'Display', None],
  [ 0x2970, 'RightDoubleArrowWithRoundedHead', [], 'Display', None],
  [ 0x2971, 'EqualsSignAboveRightwardsArrow', [], 'Display', None],
  [ 0x2972, 'TildeOperatorAboveRightwardsArrow', [], 'Display', None],
  [ 0x2973, 'LeftwardsArrowAboveTildeOperator', [], 'Display', None],
  [ 0x2974, 'RightwardsArrowAboveTildeOperator', [], 'Display', None],
  [ 0x2975, 'RightwardsArrowAboveAlmostEqualTo', [], 'Display', None],
  [ 0x2976, 'LessthanAboveLeftwardsArrow', [], 'Display', None],
  [ 0x2977, 'LeftwardsArrowThroughLessthan', [], 'Display', None],
  [ 0x2978, 'GreaterthanAboveRightwardsArrow', [], 'Display', None],
  [ 0x2979, 'SubsetAboveRightwardsArrow', [], 'Display', None],
  [ 0x297a, 'LeftwardsArrowThroughSubset', [], 'Display', None],
  [ 0x297b, 'SupersetAboveLeftwardsArrow', [], 'Display', None],
  [ 0x297c, 'LeftFishTail', [], 'Display', None],
  [ 0x297d, 'RightFishTail', [], 'Display', None],
  [ 0x297e, 'UpFishTail', [], 'Display', None],
  [ 0x297f, 'DownFishTail', [], 'Display', None],
  [ 0x2980, 'TripleVerticalBarDelimiter', [], 'Display', None],
  [ 0x2981, 'ZNotationSpot', [], 'Display', None],
  [ 0x2982, 'ZNotationTypeColon', [], 'Display', None],
  [ 0x2983, 'LeftWhiteCurlyBracket', [], 'Display', None],
  [ 0x2984, 'RightWhiteCurlyBracket', [], 'Display', None],
  [ 0x2985, 'LeftWhiteParenthesis', [], 'Display', None],
  [ 0x2986, 'RightWhiteParenthesis', [], 'Display', None],
  [ 0x2987, 'ZNotationLeftImageBracket', [], 'Display', None],
  [ 0x2988, 'ZNotationRightImageBracket', [], 'Display', None],
  [ 0x2989, 'ZNotationLeftBindingBracket', [], 'Display', None],
  [ 0x298a, 'ZNotationRightBindingBracket', [], 'Display', None],
  [ 0x298b, 'LeftSquareBracketWithUnderbar', [], 'Display', None],
  [ 0x298c, 'RightSquareBracketWithUnderbar', [], 'Display', None],
  [ 0x298d, 'LeftSquareBracketWithTickInTopCorner', [], 'Display', None],
  [ 0x298e, 'RightSquareBracketWithTickInBottomCorner', [], 'Display', None],
  [ 0x298f, 'LeftSquareBracketWithTickInBottomCorner', [], 'Display', None],
  [ 0x2990, 'RightSquareBracketWithTickInTopCorner', [], 'Display', None],
  [ 0x2991, 'LeftAngleBracketWithDot', [], 'Display', None],
  [ 0x2992, 'RightAngleBracketWithDot', [], 'Display', None],
  [ 0x2993, 'LeftArcLessthanBracket', [], 'Display', None],
  [ 0x2994, 'RightArcGreaterthanBracket', [], 'Display', None],
  [ 0x2995, 'DoubleLeftArcGreaterthanBracket', [], 'Display', None],
  [ 0x2996, 'DoubleRightArcLessthanBracket', [], 'Display', None],
  [ 0x2997, 'LeftBlackTortoiseShellBracket', [], 'Display', None],
  [ 0x2998, 'RightBlackTortoiseShellBracket', [], 'Display', None],
  [ 0x2999, 'DottedFence', [], 'Display', None],
  [ 0x299a, 'VerticalZigzagLine', [], 'Display', None],
  [ 0x299b, 'MeasuredAngleOpeningLeft', [], 'Display', None],
  [ 0x299c, 'RightAngleVariantWithSquare', [], 'Display', None],
  [ 0x299d, 'MeasuredRightAngleWithDot', [], 'Display', None],
  [ 0x299e, 'AngleWithSInside', [], 'Display', None],
  [ 0x299f, 'AcuteAngle', [], 'Display', None],
  [ 0x29a0, 'SphericalAngleOpeningLeft', [], 'Display', None],
  [ 0x29a1, 'SphericalAngleOpeningUp', [], 'Display', None],
  [ 0x29a2, 'TurnedAngle', [], 'Display', None],
  [ 0x29a3, 'ReversedAngle', [], 'Display', None],
  [ 0x29a4, 'AngleWithUnderbar', [], 'Display', None],
  [ 0x29a5, 'ReversedAngleWithUnderbar', [], 'Display', None],
  [ 0x29a6, 'ObliqueAngleOpeningUp', [], 'Display', None],
  [ 0x29a7, 'ObliqueAngleOpeningDown', [], 'Display', None],
  [ 0x29a8, 'MeasuredAngleWithOpenArmEndingInArrowPointingUpAndRight', [], 'Display', None],
  [ 0x29a9, 'MeasuredAngleWithOpenArmEndingInArrowPointingUpAndLeft', [], 'Display', None],
  [ 0x29aa, 'MeasuredAngleWithOpenArmEndingInArrowPointingDownAndRight', [], 'Display', None],
  [ 0x29ab, 'MeasuredAngleWithOpenArmEndingInArrowPointingDownAndLeft', [], 'Display', None],
  [ 0x29ac, 'MeasuredAngleWithOpenArmEndingInArrowPointingRightAndUp', [], 'Display', None],
  [ 0x29ad, 'MeasuredAngleWithOpenArmEndingInArrowPointingLeftAndUp', [], 'Display', None],
  [ 0x29ae, 'MeasuredAngleWithOpenArmEndingInArrowPointingRightAndDown', [], 'Display', None],
  [ 0x29af, 'MeasuredAngleWithOpenArmEndingInArrowPointingLeftAndDown', [], 'Display', None],
  [ 0x29b0, 'ReversedEmptySet', [], 'Display', None],
  [ 0x29b1, 'EmptySetWithOverbar', [], 'Display', None],
  [ 0x29b2, 'EmptySetWithSmallCircleAbove', [], 'Display', None],
  [ 0x29b3, 'EmptySetWithRightArrowAbove', [], 'Display', None],
  [ 0x29b4, 'EmptySetWithLeftArrowAbove', [], 'Display', None],
  [ 0x29b5, 'CircleWithHorizontalBar', [], 'Display', None],
  [ 0x29b6, 'CircledVerticalBar', [], 'Display', None],
  [ 0x29b7, 'CircledParallel', [], 'Display', None],
  [ 0x29b8, 'CircledReverseSolidus', [], 'Display', None],
  [ 0x29b9, 'CircledPerpendicular', [], 'Display', None],
  [ 0x29ba, 'CircleDividedByHorizontalBarAndTopHalfDividedByVerticalBar', [], 'Display', None],
  [ 0x29bb, 'CircleWithSuperimposedX', [], 'Display', None],
  [ 0x29bc, 'CircledAnticlockwiserotatedDivisionSign', [], 'Display', None],
  [ 0x29bd, 'UpArrowThroughCircle', [], 'Display', None],
  [ 0x29be, 'CircledWhiteBullet', [], 'Display', None],
  [ 0x29bf, 'CircledBullet', [], 'Display', None],
  [ 0x29c0, 'CircledLessthan', [], 'Display', None],
  [ 0x29c1, 'CircledGreaterthan', [], 'Display', None],
  [ 0x29c2, 'CircleWithSmallCircleToTheRight', [], 'Display', None],
  [ 0x29c3, 'CircleWithTwoHorizontalStrokesToTheRight', [], 'Display', None],
  [ 0x29c4, 'SquaredRisingDiagonalSlash', [], 'Display', None],
  [ 0x29c5, 'SquaredFallingDiagonalSlash', [], 'Display', None],
  [ 0x29c6, 'SquaredAsterisk', [], 'Display', None],
  [ 0x29c7, 'SquaredSmallCircle', [], 'Display', None],
  [ 0x29c8, 'SquaredSquare', [], 'Display', None],
  [ 0x29c9, 'TwoJoinedSquares', [], 'Display', None],
  [ 0x29ca, 'TriangleWithDotAbove', [], 'Display', None],
  [ 0x29cb, 'TriangleWithUnderbar', [], 'Display', None],
  [ 0x29cc, 'SInTriangle', [], 'Display', None],
  [ 0x29cd, 'TriangleWithSerifsAtBottom', [], 'Display', None],
  [ 0x29ce, 'RightTriangleAboveLeftTriangle', [], 'Display', None],
  [ 0x29cf, 'LeftTriangleBesideVerticalBar', [], 'Display', None],
  [ 0x29d0, 'VerticalBarBesideRightTriangle', [], 'Display', None],
  [ 0x29d1, 'BowtieWithLeftHalfBlack', [], 'Display', None],
  [ 0x29d2, 'BowtieWithRightHalfBlack', [], 'Display', None],
  [ 0x29d3, 'BlackBowtie', [], 'Display', None],
  [ 0x29d4, 'TimesWithLeftHalfBlack', [], 'Display', None],
  [ 0x29d5, 'TimesWithRightHalfBlack', [], 'Display', None],
  [ 0x29d6, 'WhiteHourglass', [], 'Display', None],
  [ 0x29d7, 'BlackHourglass', [], 'Display', None],
  [ 0x29d8, 'LeftWigglyFence', [], 'Display', None],
  [ 0x29d9, 'RightWigglyFence', [], 'Display', None],
  [ 0x29da, 'LeftDoubleWigglyFence', [], 'Display', None],
  [ 0x29db, 'RightDoubleWigglyFence', [], 'Display', None],
  [ 0x29dc, 'IncompleteInfinity', [], 'Display', None],
  [ 0x29dd, 'TieOverInfinity', [], 'Display', None],
  [ 0x29de, 'InfinityNegatedWithVerticalBar', [], 'Display', None],
  [ 0x29df, 'DoubleendedMultimap', [], 'Display', None],
  [ 0x29e0, 'SquareWithContouredOutline', [], 'Display', None],
  [ 0x29e1, 'IncreasesAs', [], 'Display', None],
  [ 0x29e2, 'ShuffleProduct', [], 'Display', None],
  [ 0x29e3, 'EqualsSignAndSlantedParallel', [], 'Display', None],
  [ 0x29e4, 'EqualsSignAndSlantedParallelWithTildeAbove', [], 'Display', None],
  [ 0x29e5, 'IdenticalToAndSlantedParallel', [], 'Display', None],
  [ 0x29e6, 'GleichStark', [], 'Display', None],
  [ 0x29e7, 'Thermodynamic', [], 'Display', None],
  [ 0x29e8, 'DownpointingTriangleWithLeftHalfBlack', [], 'Display', None],
  [ 0x29e9, 'DownpointingTriangleWithRightHalfBlack', [], 'Display', None],
  [ 0x29ea, 'BlackDiamondWithDownArrow', [], 'Display', None],
  [ 0x29eb, 'BlackLozenge', [], 'Display', None],
  [ 0x29ec, 'WhiteCircleWithDownArrow', [], 'Display', None],
  [ 0x29ed, 'BlackCircleWithDownArrow', [], 'Display', None],
  [ 0x29ee, 'ErrorbarredWhiteSquare', [], 'Display', None],
  [ 0x29ef, 'ErrorbarredBlackSquare', [], 'Display', None],
  [ 0x29f0, 'ErrorbarredWhiteDiamond', [], 'Display', None],
  [ 0x29f1, 'ErrorbarredBlackDiamond', [], 'Display', None],
  [ 0x29f2, 'ErrorbarredWhiteCircle', [], 'Display', None],
  [ 0x29f3, 'ErrorbarredBlackCircle', [], 'Display', None],
  [ 0x29f4, 'Ruledelayed', [], 'Display', None],
  [ 0x29f5, 'ReverseSolidusOperator', [], 'Display', None],
  [ 0x29f6, 'SolidusWithOverbar', [], 'Display', None],
  [ 0x29f7, 'ReverseSolidusWithHorizontalStroke', [], 'Display', None],
  [ 0x29f8, 'BigSolidus', [], 'Display', None],
  [ 0x29f9, 'BigReverseSolidus', [], 'Display', None],
  [ 0x29fa, 'DoublePlus', [], 'Display', None],
  [ 0x29fb, 'TriplePlus', [], 'Display', None],
  [ 0x29fc, 'LeftpointingCurvedAngleBracket', [], 'Display', None],
  [ 0x29fd, 'RightpointingCurvedAngleBracket', [], 'Display', None],
  [ 0x29fe, 'Tiny', [], 'Display', None],
  [ 0x29ff, 'Miny', [], 'Display', None],
  [ 0x2a00, 'NaryCircledDotOperator', [], 'Display', None],
  [ 0x2a01, 'NaryCircledPlusOperator', [], 'Display', None],
  [ 0x2a02, 'NaryCircledTimesOperator', [], 'Display', None],
  [ 0x2a03, 'NaryUnionOperatorWithDot', [], 'Display', None],
  [ 0x2a04, 'NaryUnionOperatorWithPlus', [], 'Display', None],
  [ 0x2a05, 'NarySquareIntersectionOperator', [], 'Display', None],
  [ 0x2a06, 'NarySquareUnionOperator', [], 'Display', None],
  [ 0x2a07, 'TwoLogicalAndOperator', [], 'Display', None],
  [ 0x2a08, 'TwoLogicalOrOperator', [], 'Display', None],
  [ 0x2a09, 'NaryTimesOperator', [], 'Display', None],
  [ 0x2a0a, 'ModuloTwoSum', [], 'Display', None],
  [ 0x2a0b, 'SummationWithIntegral', [], 'Display', None],
  [ 0x2a0c, 'QuadrupleIntegralOperator', [], 'Display', None],
  [ 0x2a0d, 'FinitePartIntegral', [], 'Display', None],
  [ 0x2a0e, 'IntegralWithDoubleStroke', [], 'Display', None],
  [ 0x2a0f, 'IntegralAverageWithSlash', [], 'Display', None],
  [ 0x2a10, 'CirculationFunction', [], 'Display', None],
  [ 0x2a11, 'AnticlockwiseIntegration', [], 'Display', None],
  [ 0x2a12, 'LineIntegrationWithRectangularPathAroundPole', [], 'Display', None],
  [ 0x2a13, 'LineIntegrationWithSemicircularPathAroundPole', [], 'Display', None],
  [ 0x2a14, 'LineIntegrationNotIncludingThePole', [], 'Display', None],
  [ 0x2a15, 'IntegralAroundAPointOperator', [], 'Display', None],
  [ 0x2a16, 'QuaternionIntegralOperator', [], 'Display', None],
  [ 0x2a17, 'IntegralWithLeftwardsArrowWithHook', [], 'Display', None],
  [ 0x2a18, 'IntegralWithTimesSign', [], 'Display', None],
  [ 0x2a19, 'IntegralWithIntersection', [], 'Display', None],
  [ 0x2a1a, 'IntegralWithUnion', [], 'Display', None],
  [ 0x2a1b, 'IntegralWithOverbar', [], 'Display', None],
  [ 0x2a1c, 'IntegralWithUnderbar', [], 'Display', None],
  [ 0x2a1d, 'Join', [], 'Display', None],
  [ 0x2a1e, 'LargeLeftTriangleOperator', [], 'Display', None],
  [ 0x2a1f, 'ZNotationSchemaComposition', [], 'Display', None],
  [ 0x2a20, 'ZNotationSchemaPiping', [], 'Display', None],
  [ 0x2a21, 'ZNotationSchemaProjection', [], 'Display', None],
  [ 0x2a22, 'PlusSignWithSmallCircleAbove', [], 'Display', None],
  [ 0x2a23, 'PlusSignWithCircumflexAccentAbove', [], 'Display', None],
  [ 0x2a24, 'PlusSignWithTildeAbove', [], 'Display', None],
  [ 0x2a25, 'PlusSignWithDotBelow', [], 'Display', None],
  [ 0x2a26, 'PlusSignWithTildeBelow', [], 'Display', None],
  [ 0x2a27, 'PlusSignWithSubscriptTwo', [], 'Display', None],
  [ 0x2a28, 'PlusSignWithBlackTriangle', [], 'Display', None],
  [ 0x2a29, 'MinusSignWithCommaAbove', [], 'Display', None],
  [ 0x2a2a, 'MinusSignWithDotBelow', [], 'Display', None],
  [ 0x2a2b, 'MinusSignWithFallingDots', [], 'Display', None],
  [ 0x2a2c, 'MinusSignWithRisingDots', [], 'Display', None],
  [ 0x2a2d, 'PlusSignInLeftHalfCircle', [], 'Display', None],
  [ 0x2a2e, 'PlusSignInRightHalfCircle', [], 'Display', None],
  [ 0x2a2f, 'VectorOrCrossProduct', [], 'Display', None],
  [ 0x2a30, 'MultiplicationSignWithDotAbove', [], 'Display', None],
  [ 0x2a31, 'MultiplicationSignWithUnderbar', [], 'Display', None],
  [ 0x2a32, 'SemidirectProductWithBottomClosed', [], 'Display', None],
  [ 0x2a33, 'SmashProduct', [], 'Display', None],
  [ 0x2a34, 'MultiplicationSignInLeftHalfCircle', [], 'Display', None],
  [ 0x2a35, 'MultiplicationSignInRightHalfCircle', [], 'Display', None],
  [ 0x2a36, 'CircledMultiplicationSignWithCircumflexAccent', [], 'Display', None],
  [ 0x2a37, 'MultiplicationSignInDoubleCircle', [], 'Display', None],
  [ 0x2a38, 'CircledDivisionSign', [], 'Display', None],
  [ 0x2a39, 'PlusSignInTriangle', [], 'Display', None],
  [ 0x2a3a, 'MinusSignInTriangle', [], 'Display', None],
  [ 0x2a3b, 'MultiplicationSignInTriangle', [], 'Display', None],
  [ 0x2a3c, 'InteriorProduct', [], 'Display', None],
  [ 0x2a3d, 'RighthandInteriorProduct', [], 'Display', None],
  [ 0x2a3e, 'ZNotationRelationalComposition', [], 'Display', None],
  [ 0x2a3f, 'AmalgamationOrCoproduct', [], 'Display', None],
  [ 0x2a40, 'IntersectionWithDot', [], 'Display', None],
  [ 0x2a41, 'UnionWithMinusSign', [], 'Display', None],
  [ 0x2a42, 'UnionWithOverbar', [], 'Display', None],
  [ 0x2a43, 'IntersectionWithOverbar', [], 'Display', None],
  [ 0x2a44, 'IntersectionWithLogicalAnd', [], 'Display', None],
  [ 0x2a45, 'UnionWithLogicalOr', [], 'Display', None],
  [ 0x2a46, 'UnionAboveIntersection', [], 'Display', None],
  [ 0x2a47, 'IntersectionAboveUnion', [], 'Display', None],
  [ 0x2a48, 'UnionAboveBarAboveIntersection', [], 'Display', None],
  [ 0x2a49, 'IntersectionAboveBarAboveUnion', [], 'Display', None],
  [ 0x2a4a, 'UnionBesideAndJoinedWithUnion', [], 'Display', None],
  [ 0x2a4b, 'IntersectionBesideAndJoinedWithIntersection', [], 'Display', None],
  [ 0x2a4c, 'ClosedUnionWithSerifs', [], 'Display', None],
  [ 0x2a4d, 'ClosedIntersectionWithSerifs', [], 'Display', None],
  [ 0x2a4e, 'DoubleSquareIntersection', [], 'Display', None],
  [ 0x2a4f, 'DoubleSquareUnion', [], 'Display', None],
  [ 0x2a50, 'ClosedUnionWithSerifsAndSmashProduct', [], 'Display', None],
  [ 0x2a51, 'LogicalAndWithDotAbove', [], 'Display', None],
  [ 0x2a52, 'LogicalOrWithDotAbove', [], 'Display', None],
  [ 0x2a53, 'DoubleLogicalAnd', [], 'Display', None],
  [ 0x2a54, 'DoubleLogicalOr', [], 'Display', None],
  [ 0x2a55, 'TwoIntersectingLogicalAnd', [], 'Display', None],
  [ 0x2a56, 'TwoIntersectingLogicalOr', [], 'Display', None],
  [ 0x2a57, 'SlopingLargeOr', [], 'Display', None],
  [ 0x2a58, 'SlopingLargeAnd', [], 'Display', None],
  [ 0x2a59, 'LogicalOrOverlappingLogicalAnd', [], 'Display', None],
  [ 0x2a5a, 'LogicalAndWithMiddleStem', [], 'Display', None],
  [ 0x2a5b, 'LogicalOrWithMiddleStem', [], 'Display', None],
  [ 0x2a5c, 'LogicalAndWithHorizontalDash', [], 'Display', None],
  [ 0x2a5d, 'LogicalOrWithHorizontalDash', [], 'Display', None],
  [ 0x2a5e, 'LogicalAndWithDoubleOverbar', [], 'Display', None],
  [ 0x2a5f, 'LogicalAndWithUnderbar', [], 'Display', None],
  [ 0x2a60, 'LogicalAndWithDoubleUnderbar', [], 'Display', None],
  [ 0x2a61, 'SmallVeeWithUnderbar', [], 'Display', None],
  [ 0x2a62, 'LogicalOrWithDoubleOverbar', [], 'Display', None],
  [ 0x2a63, 'LogicalOrWithDoubleUnderbar', [], 'Display', None],
  [ 0x2a64, 'ZNotationDomainAntirestriction', [], 'Display', None],
  [ 0x2a65, 'ZNotationRangeAntirestriction', [], 'Display', None],
  [ 0x2a66, 'EqualsSignWithDotBelow', [], 'Display', None],
  [ 0x2a67, 'IdenticalWithDotAbove', [], 'Display', None],
  [ 0x2a68, 'TripleHorizontalBarWithDoubleVerticalStroke', [], 'Display', None],
  [ 0x2a69, 'TripleHorizontalBarWithTripleVerticalStroke', [], 'Display', None],
  [ 0x2a6a, 'TildeOperatorWithDotAbove', [], 'Display', None],
  [ 0x2a6b, 'TildeOperatorWithRisingDots', [], 'Display', None],
  [ 0x2a6c, 'SimilarMinusSimilar', [], 'Display', None],
  [ 0x2a6d, 'CongruentWithDotAbove', [], 'Display', None],
  [ 0x2a6e, 'EqualsWithAsterisk', [], 'Display', None],
  [ 0x2a6f, 'AlmostEqualToWithCircumflexAccent', [], 'Display', None],
  [ 0x2a70, 'ApproximatelyEqualOrEqualTo', [], 'Display', None],
  [ 0x2a71, 'EqualsSignAbovePlusSign', [], 'Display', None],
  [ 0x2a72, 'PlusSignAboveEqualsSign', [], 'Display', None],
  [ 0x2a73, 'EqualsSignAboveTildeOperator', [], 'Display', None],
  [ 0x2a74, 'DoubleColonEqual', [], 'Display', None],
  [ 0x2a75, 'TwoConsecutiveEqualsSigns', [], 'Display', None],
  [ 0x2a76, 'ThreeConsecutiveEqualsSigns', [], 'Display', None],
  [ 0x2a77, 'EqualsSignWithTwoDotsAboveAndTwoDotsBelow', [], 'Display', None],
  [ 0x2a78, 'EquivalentWithFourDotsAbove', [], 'Display', None],
  [ 0x2a79, 'LessthanWithCircleInside', [], 'Display', None],
  [ 0x2a7a, 'GreaterthanWithCircleInside', [], 'Display', None],
  [ 0x2a7b, 'LessthanWithQuestionMarkAbove', [], 'Display', None],
  [ 0x2a7c, 'GreaterthanWithQuestionMarkAbove', [], 'Display', None],
  [ 0x2a7d, 'LessthanOrSlantedEqualTo', [], 'Display', None],
  [ 0x2a7e, 'GreaterthanOrSlantedEqualTo', [], 'Display', None],
  [ 0x2a7f, 'LessthanOrSlantedEqualToWithDotInside', [], 'Display', None],
  [ 0x2a80, 'GreaterthanOrSlantedEqualToWithDotInside', [], 'Display', None],
  [ 0x2a81, 'LessthanOrSlantedEqualToWithDotAbove', [], 'Display', None],
  [ 0x2a82, 'GreaterthanOrSlantedEqualToWithDotAbove', [], 'Display', None],
  [ 0x2a83, 'LessthanOrSlantedEqualToWithDotAboveRight', [], 'Display', None],
  [ 0x2a84, 'GreaterthanOrSlantedEqualToWithDotAboveLeft', [], 'Display', None],
  [ 0x2a85, 'LessthanOrApproximate', [], 'Display', None],
  [ 0x2a86, 'GreaterthanOrApproximate', [], 'Display', None],
  [ 0x2a87, 'LessthanAndSinglelineNotEqualTo', [], 'Display', None],
  [ 0x2a88, 'GreaterthanAndSinglelineNotEqualTo', [], 'Display', None],
  [ 0x2a89, 'LessthanAndNotApproximate', [], 'Display', None],
  [ 0x2a8a, 'GreaterthanAndNotApproximate', [], 'Display', None],
  [ 0x2a8b, 'LessthanAboveDoublelineEqualAboveGreaterthan', [], 'Display', None],
  [ 0x2a8c, 'GreaterthanAboveDoublelineEqualAboveLessthan', [], 'Display', None],
  [ 0x2a8d, 'LessthanAboveSimilarOrEqual', [], 'Display', None],
  [ 0x2a8e, 'GreaterthanAboveSimilarOrEqual', [], 'Display', None],
  [ 0x2a8f, 'LessthanAboveSimilarAboveGreaterthan', [], 'Display', None],
  [ 0x2a90, 'GreaterthanAboveSimilarAboveLessthan', [], 'Display', None],
  [ 0x2a91, 'LessthanAboveGreaterthanAboveDoublelineEqual', [], 'Display', None],
  [ 0x2a92, 'GreaterthanAboveLessthanAboveDoublelineEqual', [], 'Display', None],
  [ 0x2a93, 'LessthanAboveSlantedEqualAboveGreaterthanAboveSlantedEqual', [], 'Display', None],
  [ 0x2a94, 'GreaterthanAboveSlantedEqualAboveLessthanAboveSlantedEqual', [], 'Display', None],
  [ 0x2a95, 'SlantedEqualToOrLessthan', [], 'Display', None],
  [ 0x2a96, 'SlantedEqualToOrGreaterthan', [], 'Display', None],
  [ 0x2a97, 'SlantedEqualToOrLessthanWithDotInside', [], 'Display', None],
  [ 0x2a98, 'SlantedEqualToOrGreaterthanWithDotInside', [], 'Display', None],
  [ 0x2a99, 'DoublelineEqualToOrLessthan', [], 'Display', None],
  [ 0x2a9a, 'DoublelineEqualToOrGreaterthan', [], 'Display', None],
  [ 0x2a9b, 'DoublelineSlantedEqualToOrLessthan', [], 'Display', None],
  [ 0x2a9c, 'DoublelineSlantedEqualToOrGreaterthan', [], 'Display', None],
  [ 0x2a9d, 'SimilarOrLessthan', [], 'Display', None],
  [ 0x2a9e, 'SimilarOrGreaterthan', [], 'Display', None],
  [ 0x2a9f, 'SimilarAboveLessthanAboveEqualsSign', [], 'Display', None],
  [ 0x2aa0, 'SimilarAboveGreaterthanAboveEqualsSign', [], 'Display', None],
  [ 0x2aa1, 'DoubleNestedLessthan', [], 'Display', None],
  [ 0x2aa2, 'DoubleNestedGreaterthan', [], 'Display', None],
  [ 0x2aa3, 'DoubleNestedLessthanWithUnderbar', [], 'Display', None],
  [ 0x2aa4, 'GreaterthanOverlappingLessthan', [], 'Display', None],
  [ 0x2aa5, 'GreaterthanBesideLessthan', [], 'Display', None],
  [ 0x2aa6, 'LessthanClosedByCurve', [], 'Display', None],
  [ 0x2aa7, 'GreaterthanClosedByCurve', [], 'Display', None],
  [ 0x2aa8, 'LessthanClosedByCurveAboveSlantedEqual', [], 'Display', None],
  [ 0x2aa9, 'GreaterthanClosedByCurveAboveSlantedEqual', [], 'Display', None],
  [ 0x2aaa, 'SmallerThan', [], 'Display', None],
  [ 0x2aab, 'LargerThan', [], 'Display', None],
  [ 0x2aac, 'SmallerThanOrEqualTo', [], 'Display', None],
  [ 0x2aad, 'LargerThanOrEqualTo', [], 'Display', None],
  [ 0x2aae, 'EqualsSignWithBumpyAbove', [], 'Display', None],
  [ 0x2aaf, 'PrecedesAboveSinglelineEqualsSign', [], 'Display', None],
  [ 0x2ab0, 'SucceedsAboveSinglelineEqualsSign', [], 'Display', None],
  [ 0x2ab1, 'PrecedesAboveSinglelineNotEqualTo', [], 'Display', None],
  [ 0x2ab2, 'SucceedsAboveSinglelineNotEqualTo', [], 'Display', None],
  [ 0x2ab3, 'PrecedesAboveEqualsSign', [], 'Display', None],
  [ 0x2ab4, 'SucceedsAboveEqualsSign', [], 'Display', None],
  [ 0x2ab5, 'PrecedesAboveNotEqualTo', [], 'Display', None],
  [ 0x2ab6, 'SucceedsAboveNotEqualTo', [], 'Display', None],
  [ 0x2ab7, 'PrecedesAboveAlmostEqualTo', [], 'Display', None],
  [ 0x2ab8, 'SucceedsAboveAlmostEqualTo', [], 'Display', None],
  [ 0x2ab9, 'PrecedesAboveNotAlmostEqualTo', [], 'Display', None],
  [ 0x2aba, 'SucceedsAboveNotAlmostEqualTo', [], 'Display', None],
  [ 0x2abb, 'DoublePrecedes', [], 'Display', None],
  [ 0x2abc, 'DoubleSucceeds', [], 'Display', None],
  [ 0x2abd, 'SubsetWithDot', [], 'Display', None],
  [ 0x2abe, 'SupersetWithDot', [], 'Display', None],
  [ 0x2abf, 'SubsetWithPlusSignBelow', [], 'Display', None],
  [ 0x2ac0, 'SupersetWithPlusSignBelow', [], 'Display', None],
  [ 0x2ac1, 'SubsetWithMultiplicationSignBelow', [], 'Display', None],
  [ 0x2ac2, 'SupersetWithMultiplicationSignBelow', [], 'Display', None],
  [ 0x2ac3, 'SubsetOfOrEqualToWithDotAbove', [], 'Display', None],
  [ 0x2ac4, 'SupersetOfOrEqualToWithDotAbove', [], 'Display', None],
  [ 0x2ac5, 'SubsetOfAboveEqualsSign', [], 'Display', None],
  [ 0x2ac6, 'SupersetOfAboveEqualsSign', [], 'Display', None],
  [ 0x2ac7, 'SubsetOfAboveTildeOperator', [], 'Display', None],
  [ 0x2ac8, 'SupersetOfAboveTildeOperator', [], 'Display', None],
  [ 0x2ac9, 'SubsetOfAboveAlmostEqualTo', [], 'Display', None],
  [ 0x2aca, 'SupersetOfAboveAlmostEqualTo', [], 'Display', None],
  [ 0x2acb, 'SubsetOfAboveNotEqualTo', [], 'Display', None],
  [ 0x2acc, 'SupersetOfAboveNotEqualTo', [], 'Display', None],
  [ 0x2acd, 'SquareLeftOpenBoxOperator', [], 'Display', None],
  [ 0x2ace, 'SquareRightOpenBoxOperator', [], 'Display', None],
  [ 0x2acf, 'ClosedSubset', [], 'Display', None],
  [ 0x2ad0, 'ClosedSuperset', [], 'Display', None],
  [ 0x2ad1, 'ClosedSubsetOrEqualTo', [], 'Display', None],
  [ 0x2ad2, 'ClosedSupersetOrEqualTo', [], 'Display', None],
  [ 0x2ad3, 'SubsetAboveSuperset', [], 'Display', None],
  [ 0x2ad4, 'SupersetAboveSubset', [], 'Display', None],
  [ 0x2ad5, 'SubsetAboveSubset', [], 'Display', None],
  [ 0x2ad6, 'SupersetAboveSuperset', [], 'Display', None],
  [ 0x2ad7, 'SupersetBesideSubset', [], 'Display', None],
  [ 0x2ad8, 'SupersetBesideAndJoinedByDashWithSubset', [], 'Display', None],
  [ 0x2ad9, 'ElementOfOpeningDownwards', [], 'Display', None],
  [ 0x2ada, 'PitchforkWithTeeTop', [], 'Display', None],
  [ 0x2adb, 'TransversalIntersection', [], 'Display', None],
  [ 0x2adc, 'Forking', [], 'Display', None],
  [ 0x2add, 'Nonforking', [], 'Display', None],
  [ 0x2ade, 'ShortLeftTack', [], 'Display', None],
  [ 0x2adf, 'ShortDownTack', [], 'Display', None],
  [ 0x2ae0, 'ShortUpTack', [], 'Display', None],
  [ 0x2ae1, 'PerpendicularWithS', [], 'Display', None],
  [ 0x2ae2, 'VerticalBarTripleRightTurnstile', [], 'Display', None],
  [ 0x2ae3, 'DoubleVerticalBarLeftTurnstile', [], 'Display', None],
  [ 0x2ae4, 'VerticalBarDoubleLeftTurnstile', [], 'Display', None],
  [ 0x2ae5, 'DoubleVerticalBarDoubleLeftTurnstile', [], 'Display', None],
  [ 0x2ae6, 'LongDashFromLeftMemberOfDoubleVertical', [], 'Display', None],
  [ 0x2ae7, 'ShortDownTackWithOverbar', [], 'Display', None],
  [ 0x2ae8, 'ShortUpTackWithUnderbar', [], 'Display', None],
  [ 0x2ae9, 'ShortUpTackAboveShortDownTack', [], 'Display', None],
  [ 0x2aea, 'DoubleDownTack', [], 'Display', None],
  [ 0x2aeb, 'DoubleUpTack', [], 'Display', None],
  [ 0x2aec, 'DoubleStrokeNotSign', [], 'Display', None],
  [ 0x2aed, 'ReversedDoubleStrokeNotSign', [], 'Display', None],
  [ 0x2aee, 'DoesNotDivideWithReversedNegationSlash', [], 'Display', None],
  [ 0x2aef, 'VerticalLineWithCircleAbove', [], 'Display', None],
  [ 0x2af0, 'VerticalLineWithCircleBelow', [], 'Display', None],
  [ 0x2af1, 'DownTackWithCircleBelow', [], 'Display', None],
  [ 0x2af2, 'ParallelWithHorizontalStroke', [], 'Display', None],
  [ 0x2af3, 'ParallelWithTildeOperator', [], 'Display', None],
  [ 0x2af4, 'TripleVerticalBarBinaryRelation', [], 'Display', None],
  [ 0x2af5, 'TripleVerticalBarWithHorizontalStroke', [], 'Display', None],
  [ 0x2af6, 'TripleColonOperator', [], 'Display', None],
  [ 0x2af7, 'TripleNestedLessthan', [], 'Display', None],
  [ 0x2af8, 'TripleNestedGreaterthan', [], 'Display', None],
  [ 0x2af9, 'DoublelineSlantedLessthanOrEqualTo', [], 'Display', None],
  [ 0x2afa, 'DoublelineSlantedGreaterthanOrEqualTo', [], 'Display', None],
  [ 0x2afb, 'TripleSolidusBinaryRelation', [], 'Display', None],
  [ 0x2afc, 'LargeTripleVerticalBarOperator', [], 'Display', None],
  [ 0x2afd, 'DoubleSolidusOperator', [], 'Display', None],
  [ 0x2afe, 'WhiteVerticalBar', [], 'Display', None],
  [ 0x2aff, 'NaryWhiteVerticalBar', [], 'Display', None],
  [ 0x2b00, 'NorthEastWhiteArrow', [], 'Display', None],
  [ 0x2b01, 'NorthWestWhiteArrow', [], 'Display', None],
  [ 0x2b02, 'SouthEastWhiteArrow', [], 'Display', None],
  [ 0x2b03, 'SouthWestWhiteArrow', [], 'Display', None],
  [ 0x2b04, 'LeftRightWhiteArrow', [], 'Display', None],
  [ 0x2b05, 'LeftwardsBlackArrow', [], 'Display', None],
  [ 0x2b06, 'UpwardsBlackArrow', [], 'Display', None],
  [ 0x2b07, 'DownwardsBlackArrow', [], 'Display', None],
  [ 0x2b08, 'NorthEastBlackArrow', [], 'Display', None],
  [ 0x2b09, 'NorthWestBlackArrow', [], 'Display', None],
  [ 0x2b0a, 'SouthEastBlackArrow', [], 'Display', None],
  [ 0x2b0b, 'SouthWestBlackArrow', [], 'Display', None],
  [ 0x2b0c, 'LeftRightBlackArrow', [], 'Display', None],
  [ 0x2b0d, 'UpDownBlackArrow', [], 'Display', None],
  [ 0x2b0e, 'RightwardsArrowWithTipDownwards', [], 'Display', None],
  [ 0x2b0f, 'RightwardsArrowWithTipUpwards', [], 'Display', None],
  [ 0x2b10, 'LeftwardsArrowWithTipDownwards', [], 'Display', None],
  [ 0x2b11, 'LeftwardsArrowWithTipUpwards', [], 'Display', None],
  [ 0x2b12, 'SquareWithTopHalfBlack', [], 'Display', None],
  [ 0x2b13, 'SquareWithBottomHalfBlack', [], 'Display', None],
  [ 0x2b14, 'SquareWithUpperRightDiagonalHalfBlack', [], 'Display', None],
  [ 0x2b15, 'SquareWithLowerLeftDiagonalHalfBlack', [], 'Display', None],
  [ 0x2b16, 'DiamondWithLeftHalfBlack', [], 'Display', None],
  [ 0x2b17, 'DiamondWithRightHalfBlack', [], 'Display', None],
  [ 0x2b18, 'DiamondWithTopHalfBlack', [], 'Display', None],
  [ 0x2b19, 'DiamondWithBottomHalfBlack', [], 'Display', None],
  [ 0x2b1a, 'DottedSquare', [], 'Display', None],
  [ 0x2b1b, 'BlackLargeSquare', [], 'Display', None],
  [ 0x2b1c, 'WhiteLargeSquare', [], 'Display', None],
  [ 0x2b1d, 'BlackVerySmallSquare', [], 'Display', None],
  [ 0x2b1e, 'WhiteVerySmallSquare', [], 'Display', None],
  [ 0x2b1f, 'BlackPentagon', [], 'Display', None],
  [ 0x2b20, 'WhitePentagon', [], 'Display', None],
  [ 0x2b21, 'WhiteHexagon', [], 'Display', None],
  [ 0x2b22, 'BlackHexagon', [], 'Display', None],
  [ 0x2b23, 'HorizontalBlackHexagon', [], 'Display', None],
  [ 0x2b24, 'BlackLargeCircle', [], 'Display', None],
  [ 0x2b25, 'BlackMediumDiamond', [], 'Display', None],
  [ 0x2b26, 'WhiteMediumDiamond', [], 'Display', None],
  [ 0x2b27, 'BlackMediumLozenge', [], 'Display', None],
  [ 0x2b28, 'WhiteMediumLozenge', [], 'Display', None],
  [ 0x2b29, 'BlackSmallDiamond', [], 'Display', None],
  [ 0x2b2a, 'BlackSmallLozenge', [], 'Display', None],
  [ 0x2b2b, 'WhiteSmallLozenge', [], 'Display', None],
  [ 0x2b2c, 'BlackHorizontalEllipse', [], 'Display', None],
  [ 0x2b2d, 'WhiteHorizontalEllipse', [], 'Display', None],
  [ 0x2b2e, 'BlackVerticalEllipse', [], 'Display', None],
  [ 0x2b2f, 'WhiteVerticalEllipse', [], 'Display', None],
  [ 0x2b30, 'LeftArrowWithSmallCircle', [], 'Display', None],
  [ 0x2b31, 'ThreeLeftwardsArrows', [], 'Display', None],
  [ 0x2b32, 'LeftArrowWithCircledPlus', [], 'Display', None],
  [ 0x2b33, 'LongLeftwardsSquiggleArrow', [], 'Display', None],
  [ 0x2b34, 'LeftwardsTwoheadedArrowWithVerticalStroke', [], 'Display', None],
  [ 0x2b35, 'LeftwardsTwoheadedArrowWithDoubleVerticalStroke', [], 'Display', None],
  [ 0x2b36, 'LeftwardsTwoheadedArrowFromBar', [], 'Display', None],
  [ 0x2b37, 'LeftwardsTwoheadedTripleDashArrow', [], 'Display', None],
  [ 0x2b38, 'LeftwardsArrowWithDottedStem', [], 'Display', None],
  [ 0x2b39, 'LeftwardsArrowWithTailWithVerticalStroke', [], 'Display', None],
  [ 0x2b3a, 'LeftwardsArrowWithTailWithDoubleVerticalStroke', [], 'Display', None],
  [ 0x2b3b, 'LeftwardsTwoheadedArrowWithTail', [], 'Display', None],
  [ 0x2b3c, 'LeftwardsTwoheadedArrowWithTailWithVerticalStroke', [], 'Display', None],
  [ 0x2b3d, 'LeftwardsTwoheadedArrowWithTailWithDoubleVerticalStroke', [], 'Display', None],
  [ 0x2b3e, 'LeftwardsArrowThroughX', [], 'Display', None],
  [ 0x2b3f, 'WaveArrowPointingDirectlyLeft', [], 'Display', None],
  [ 0x2b40, 'EqualsSignAboveLeftwardsArrow', [], 'Display', None],
  [ 0x2b41, 'ReverseTildeOperatorAboveLeftwardsArrow', [], 'Display', None],
  [ 0x2b42, 'LeftwardsArrowAboveReverseAlmostEqualTo', [], 'Display', None],
  [ 0x2b43, 'RightwardsArrowThroughGreaterthan', [], 'Display', None],
  [ 0x2b44, 'RightwardsArrowThroughSuperset', [], 'Display', None],
  [ 0x2b45, 'LeftwardsQuadrupleArrow', [], 'Display', None],
  [ 0x2b46, 'RightwardsQuadrupleArrow', [], 'Display', None],
  [ 0x2b47, 'ReverseTildeOperatorAboveRightwardsArrow', [], 'Display', None],
  [ 0x2b48, 'RightwardsArrowAboveReverseAlmostEqualTo', [], 'Display', None],
  [ 0x2b49, 'TildeOperatorAboveLeftwardsArrow', [], 'Display', None],
  [ 0x2b4a, 'LeftwardsArrowAboveAlmostEqualTo', [], 'Display', None],
  [ 0x2b4b, 'LeftwardsArrowAboveReverseTildeOperator', [], 'Display', None],
  [ 0x2b4c, 'RightwardsArrowAboveReverseTildeOperator', [], 'Display', None],
  [ 0x2b4d, 'UnicodeCharacter0x2b4d', [], 'Display', None],
  [ 0x2b4e, 'UnicodeCharacter0x2b4e', [], 'Display', None],
  [ 0x2b4f, 'UnicodeCharacter0x2b4f', [], 'Display', None],
  [ 0x2b50, 'WhiteMediumStar', [], 'Display', None],
  [ 0x2b51, 'BlackSmallStar', [], 'Display', None],
  [ 0x2b52, 'WhiteSmallStar', [], 'Display', None],
  [ 0x2b53, 'BlackRightpointingPentagon', [], 'Display', None],
  [ 0x2b54, 'WhiteRightpointingPentagon', [], 'Display', None],
  [ 0x2b55, 'UnicodeCharacter0x2b55', [], 'Display', None],
  [ 0x2b56, 'UnicodeCharacter0x2b56', [], 'Display', None],
  [ 0x2b57, 'UnicodeCharacter0x2b57', [], 'Display', None],
  [ 0x2b58, 'UnicodeCharacter0x2b58', [], 'Display', None],
  [ 0x2b59, 'UnicodeCharacter0x2b59', [], 'Display', None],
  [ 0x2b5a, 'UnicodeCharacter0x2b5a', [], 'Display', None],
  [ 0x2b5b, 'UnicodeCharacter0x2b5b', [], 'Display', None],
  [ 0x2b5c, 'UnicodeCharacter0x2b5c', [], 'Display', None],
  [ 0x2b5d, 'UnicodeCharacter0x2b5d', [], 'Display', None],
  [ 0x2b5e, 'UnicodeCharacter0x2b5e', [], 'Display', None],
  [ 0x2b5f, 'UnicodeCharacter0x2b5f', [], 'Display', None],
  [ 0x2b60, 'UnicodeCharacter0x2b60', [], 'Display', None],
  [ 0x2b61, 'UnicodeCharacter0x2b61', [], 'Display', None],
  [ 0x2b62, 'UnicodeCharacter0x2b62', [], 'Display', None],
  [ 0x2b63, 'UnicodeCharacter0x2b63', [], 'Display', None],
  [ 0x2b64, 'UnicodeCharacter0x2b64', [], 'Display', None],
  [ 0x2b65, 'UnicodeCharacter0x2b65', [], 'Display', None],
  [ 0x2b66, 'UnicodeCharacter0x2b66', [], 'Display', None],
  [ 0x2b67, 'UnicodeCharacter0x2b67', [], 'Display', None],
  [ 0x2b68, 'UnicodeCharacter0x2b68', [], 'Display', None],
  [ 0x2b69, 'UnicodeCharacter0x2b69', [], 'Display', None],
  [ 0x2b6a, 'UnicodeCharacter0x2b6a', [], 'Display', None],
  [ 0x2b6b, 'UnicodeCharacter0x2b6b', [], 'Display', None],
  [ 0x2b6c, 'UnicodeCharacter0x2b6c', [], 'Display', None],
  [ 0x2b6d, 'UnicodeCharacter0x2b6d', [], 'Display', None],
  [ 0x2b6e, 'UnicodeCharacter0x2b6e', [], 'Display', None],
  [ 0x2b6f, 'UnicodeCharacter0x2b6f', [], 'Display', None],
  [ 0x2b70, 'UnicodeCharacter0x2b70', [], 'Display', None],
  [ 0x2b71, 'UnicodeCharacter0x2b71', [], 'Display', None],
  [ 0x2b72, 'UnicodeCharacter0x2b72', [], 'Display', None],
  [ 0x2b73, 'UnicodeCharacter0x2b73', [], 'Display', None],
  [ 0x2b74, 'UnicodeCharacter0x2b74', [], 'Display', None],
  [ 0x2b75, 'UnicodeCharacter0x2b75', [], 'Display', None],
  [ 0x2b76, 'UnicodeCharacter0x2b76', [], 'Display', None],
  [ 0x2b77, 'UnicodeCharacter0x2b77', [], 'Display', None],
  [ 0x2b78, 'UnicodeCharacter0x2b78', [], 'Display', None],
  [ 0x2b79, 'UnicodeCharacter0x2b79', [], 'Display', None],
  [ 0x2b7a, 'UnicodeCharacter0x2b7a', [], 'Display', None],
  [ 0x2b7b, 'UnicodeCharacter0x2b7b', [], 'Display', None],
  [ 0x2b7c, 'UnicodeCharacter0x2b7c', [], 'Display', None],
  [ 0x2b7d, 'UnicodeCharacter0x2b7d', [], 'Display', None],
  [ 0x2b7e, 'UnicodeCharacter0x2b7e', [], 'Display', None],
  [ 0x2b7f, 'UnicodeCharacter0x2b7f', [], 'Display', None],
  [ 0x2b80, 'UnicodeCharacter0x2b80', [], 'Display', None],
  [ 0x2b81, 'UnicodeCharacter0x2b81', [], 'Display', None],
  [ 0x2b82, 'UnicodeCharacter0x2b82', [], 'Display', None],
  [ 0x2b83, 'UnicodeCharacter0x2b83', [], 'Display', None],
  [ 0x2b84, 'UnicodeCharacter0x2b84', [], 'Display', None],
  [ 0x2b85, 'UnicodeCharacter0x2b85', [], 'Display', None],
  [ 0x2b86, 'UnicodeCharacter0x2b86', [], 'Display', None],
  [ 0x2b87, 'UnicodeCharacter0x2b87', [], 'Display', None],
  [ 0x2b88, 'UnicodeCharacter0x2b88', [], 'Display', None],
  [ 0x2b89, 'UnicodeCharacter0x2b89', [], 'Display', None],
  [ 0x2b8a, 'UnicodeCharacter0x2b8a', [], 'Display', None],
  [ 0x2b8b, 'UnicodeCharacter0x2b8b', [], 'Display', None],
  [ 0x2b8c, 'UnicodeCharacter0x2b8c', [], 'Display', None],
  [ 0x2b8d, 'UnicodeCharacter0x2b8d', [], 'Display', None],
  [ 0x2b8e, 'UnicodeCharacter0x2b8e', [], 'Display', None],
  [ 0x2b8f, 'UnicodeCharacter0x2b8f', [], 'Display', None],
  [ 0x2b90, 'UnicodeCharacter0x2b90', [], 'Display', None],
  [ 0x2b91, 'UnicodeCharacter0x2b91', [], 'Display', None],
  [ 0x2b92, 'UnicodeCharacter0x2b92', [], 'Display', None],
  [ 0x2b93, 'UnicodeCharacter0x2b93', [], 'Display', None],
  [ 0x2b94, 'UnicodeCharacter0x2b94', [], 'Display', None],
  [ 0x2b95, 'UnicodeCharacter0x2b95', [], 'Display', None],
  [ 0x2b96, 'UnicodeCharacter0x2b96', [], 'Display', None],
  [ 0x2b97, 'UnicodeCharacter0x2b97', [], 'Display', None],
  [ 0x2b98, 'UnicodeCharacter0x2b98', [], 'Display', None],
  [ 0x2b99, 'UnicodeCharacter0x2b99', [], 'Display', None],
  [ 0x2b9a, 'UnicodeCharacter0x2b9a', [], 'Display', None],
  [ 0x2b9b, 'UnicodeCharacter0x2b9b', [], 'Display', None],
  [ 0x2b9c, 'UnicodeCharacter0x2b9c', [], 'Display', None],
  [ 0x2b9d, 'UnicodeCharacter0x2b9d', [], 'Display', None],
  [ 0x2b9e, 'UnicodeCharacter0x2b9e', [], 'Display', None],
  [ 0x2b9f, 'UnicodeCharacter0x2b9f', [], 'Display', None],
  [ 0x2ba0, 'UnicodeCharacter0x2ba0', [], 'Display', None],
  [ 0x2ba1, 'UnicodeCharacter0x2ba1', [], 'Display', None],
  [ 0x2ba2, 'UnicodeCharacter0x2ba2', [], 'Display', None],
  [ 0x2ba3, 'UnicodeCharacter0x2ba3', [], 'Display', None],
  [ 0x2ba4, 'UnicodeCharacter0x2ba4', [], 'Display', None],
  [ 0x2ba5, 'UnicodeCharacter0x2ba5', [], 'Display', None],
  [ 0x2ba6, 'UnicodeCharacter0x2ba6', [], 'Display', None],
  [ 0x2ba7, 'UnicodeCharacter0x2ba7', [], 'Display', None],
  [ 0x2ba8, 'UnicodeCharacter0x2ba8', [], 'Display', None],
  [ 0x2ba9, 'UnicodeCharacter0x2ba9', [], 'Display', None],
  [ 0x2baa, 'UnicodeCharacter0x2baa', [], 'Display', None],
  [ 0x2bab, 'UnicodeCharacter0x2bab', [], 'Display', None],
  [ 0x2bac, 'UnicodeCharacter0x2bac', [], 'Display', None],
  [ 0x2bad, 'UnicodeCharacter0x2bad', [], 'Display', None],
  [ 0x2bae, 'UnicodeCharacter0x2bae', [], 'Display', None],
  [ 0x2baf, 'UnicodeCharacter0x2baf', [], 'Display', None],
  [ 0x2bb0, 'UnicodeCharacter0x2bb0', [], 'Display', None],
  [ 0x2bb1, 'UnicodeCharacter0x2bb1', [], 'Display', None],
  [ 0x2bb2, 'UnicodeCharacter0x2bb2', [], 'Display', None],
  [ 0x2bb3, 'UnicodeCharacter0x2bb3', [], 'Display', None],
  [ 0x2bb4, 'UnicodeCharacter0x2bb4', [], 'Display', None],
  [ 0x2bb5, 'UnicodeCharacter0x2bb5', [], 'Display', None],
  [ 0x2bb6, 'UnicodeCharacter0x2bb6', [], 'Display', None],
  [ 0x2bb7, 'UnicodeCharacter0x2bb7', [], 'Display', None],
  [ 0x2bb8, 'UnicodeCharacter0x2bb8', [], 'Display', None],
  [ 0x2bb9, 'UnicodeCharacter0x2bb9', [], 'Display', None],
  [ 0x2bba, 'UnicodeCharacter0x2bba', [], 'Display', None],
  [ 0x2bbb, 'UnicodeCharacter0x2bbb', [], 'Display', None],
  [ 0x2bbc, 'UnicodeCharacter0x2bbc', [], 'Display', None],
  [ 0x2bbd, 'UnicodeCharacter0x2bbd', [], 'Display', None],
  [ 0x2bbe, 'UnicodeCharacter0x2bbe', [], 'Display', None],
  [ 0x2bbf, 'UnicodeCharacter0x2bbf', [], 'Display', None],
  [ 0x2bc0, 'UnicodeCharacter0x2bc0', [], 'Display', None],
  [ 0x2bc1, 'UnicodeCharacter0x2bc1', [], 'Display', None],
  [ 0x2bc2, 'UnicodeCharacter0x2bc2', [], 'Display', None],
  [ 0x2bc3, 'UnicodeCharacter0x2bc3', [], 'Display', None],
  [ 0x2bc4, 'UnicodeCharacter0x2bc4', [], 'Display', None],
  [ 0x2bc5, 'UnicodeCharacter0x2bc5', [], 'Display', None],
  [ 0x2bc6, 'UnicodeCharacter0x2bc6', [], 'Display', None],
  [ 0x2bc7, 'UnicodeCharacter0x2bc7', [], 'Display', None],
  [ 0x2bc8, 'UnicodeCharacter0x2bc8', [], 'Display', None],
  [ 0x2bc9, 'UnicodeCharacter0x2bc9', [], 'Display', None],
  [ 0x2bca, 'UnicodeCharacter0x2bca', [], 'Display', None],
  [ 0x2bcb, 'UnicodeCharacter0x2bcb', [], 'Display', None],
  [ 0x2bcc, 'UnicodeCharacter0x2bcc', [], 'Display', None],
  [ 0x2bcd, 'UnicodeCharacter0x2bcd', [], 'Display', None],
  [ 0x2bce, 'UnicodeCharacter0x2bce', [], 'Display', None],
  [ 0x2bcf, 'UnicodeCharacter0x2bcf', [], 'Display', None],
  [ 0x2bd0, 'UnicodeCharacter0x2bd0', [], 'Display', None],
  [ 0x2bd1, 'UnicodeCharacter0x2bd1', [], 'Display', None],
  [ 0x2bd2, 'UnicodeCharacter0x2bd2', [], 'Display', None],
  [ 0x2bd3, 'UnicodeCharacter0x2bd3', [], 'Display', None],
  [ 0x2bd4, 'UnicodeCharacter0x2bd4', [], 'Display', None],
  [ 0x2bd5, 'UnicodeCharacter0x2bd5', [], 'Display', None],
  [ 0x2bd6, 'UnicodeCharacter0x2bd6', [], 'Display', None],
  [ 0x2bd7, 'UnicodeCharacter0x2bd7', [], 'Display', None],
  [ 0x2bd8, 'UnicodeCharacter0x2bd8', [], 'Display', None],
  [ 0x2bd9, 'UnicodeCharacter0x2bd9', [], 'Display', None],
  [ 0x2bda, 'UnicodeCharacter0x2bda', [], 'Display', None],
  [ 0x2bdb, 'UnicodeCharacter0x2bdb', [], 'Display', None],
  [ 0x2bdc, 'UnicodeCharacter0x2bdc', [], 'Display', None],
  [ 0x2bdd, 'UnicodeCharacter0x2bdd', [], 'Display', None],
  [ 0x2bde, 'UnicodeCharacter0x2bde', [], 'Display', None],
  [ 0x2bdf, 'UnicodeCharacter0x2bdf', [], 'Display', None],
  [ 0x2be0, 'UnicodeCharacter0x2be0', [], 'Display', None],
  [ 0x2be1, 'UnicodeCharacter0x2be1', [], 'Display', None],
  [ 0x2be2, 'UnicodeCharacter0x2be2', [], 'Display', None],
  [ 0x2be3, 'UnicodeCharacter0x2be3', [], 'Display', None],
  [ 0x2be4, 'UnicodeCharacter0x2be4', [], 'Display', None],
  [ 0x2be5, 'UnicodeCharacter0x2be5', [], 'Display', None],
  [ 0x2be6, 'UnicodeCharacter0x2be6', [], 'Display', None],
  [ 0x2be7, 'UnicodeCharacter0x2be7', [], 'Display', None],
  [ 0x2be8, 'UnicodeCharacter0x2be8', [], 'Display', None],
  [ 0x2be9, 'UnicodeCharacter0x2be9', [], 'Display', None],
  [ 0x2bea, 'UnicodeCharacter0x2bea', [], 'Display', None],
  [ 0x2beb, 'UnicodeCharacter0x2beb', [], 'Display', None],
  [ 0x2bec, 'UnicodeCharacter0x2bec', [], 'Display', None],
  [ 0x2bed, 'UnicodeCharacter0x2bed', [], 'Display', None],
  [ 0x2bee, 'UnicodeCharacter0x2bee', [], 'Display', None],
  [ 0x2bef, 'UnicodeCharacter0x2bef', [], 'Display', None],
  [ 0x2bf0, 'UnicodeCharacter0x2bf0', [], 'Display', None],
  [ 0x2bf1, 'UnicodeCharacter0x2bf1', [], 'Display', None],
  [ 0x2bf2, 'UnicodeCharacter0x2bf2', [], 'Display', None],
  [ 0x2bf3, 'UnicodeCharacter0x2bf3', [], 'Display', None],
  [ 0x2bf4, 'UnicodeCharacter0x2bf4', [], 'Display', None],
  [ 0x2bf5, 'UnicodeCharacter0x2bf5', [], 'Display', None],
  [ 0x2bf6, 'UnicodeCharacter0x2bf6', [], 'Display', None],
  [ 0x2bf7, 'UnicodeCharacter0x2bf7', [], 'Display', None],
  [ 0x2bf8, 'UnicodeCharacter0x2bf8', [], 'Display', None],
  [ 0x2bf9, 'UnicodeCharacter0x2bf9', [], 'Display', None],
  [ 0x2bfa, 'UnicodeCharacter0x2bfa', [], 'Display', None],
  [ 0x2bfb, 'UnicodeCharacter0x2bfb', [], 'Display', None],
  [ 0x2bfc, 'UnicodeCharacter0x2bfc', [], 'Display', None],
  [ 0x2bfd, 'UnicodeCharacter0x2bfd', [], 'Display', None],
  [ 0x2bfe, 'UnicodeCharacter0x2bfe', [], 'Display', None],
  [ 0x2bff, 'UnicodeCharacter0x2bff', [], 'Display', None],
  [ 0x2c00, 'GlagoliticCapitalLetterAzu', [], 'Display', None],
  [ 0x2c01, 'GlagoliticCapitalLetterBuky', [], 'Display', None],
  [ 0x2c02, 'GlagoliticCapitalLetterVede', [], 'Display', None],
  [ 0x2c03, 'GlagoliticCapitalLetterGlagoli', [], 'Display', None],
  [ 0x2c04, 'GlagoliticCapitalLetterDobro', [], 'Display', None],
  [ 0x2c05, 'GlagoliticCapitalLetterYestu', [], 'Display', None],
  [ 0x2c06, 'GlagoliticCapitalLetterZhivete', [], 'Display', None],
  [ 0x2c07, 'GlagoliticCapitalLetterDzelo', [], 'Display', None],
  [ 0x2c08, 'GlagoliticCapitalLetterZemlja', [], 'Display', None],
  [ 0x2c09, 'GlagoliticCapitalLetterIzhe', [], 'Display', None],
  [ 0x2c0a, 'GlagoliticCapitalLetterInitialIzhe', [], 'Display', None],
  [ 0x2c0b, 'GlagoliticCapitalLetterI', [], 'Display', None],
  [ 0x2c0c, 'GlagoliticCapitalLetterDjervi', [], 'Display', None],
  [ 0x2c0d, 'GlagoliticCapitalLetterKako', [], 'Display', None],
  [ 0x2c0e, 'GlagoliticCapitalLetterLjudije', [], 'Display', None],
  [ 0x2c0f, 'GlagoliticCapitalLetterMyslite', [], 'Display', None],
  [ 0x2c10, 'GlagoliticCapitalLetterNashi', [], 'Display', None],
  [ 0x2c11, 'GlagoliticCapitalLetterOnu', [], 'Display', None],
  [ 0x2c12, 'GlagoliticCapitalLetterPokoji', [], 'Display', None],
  [ 0x2c13, 'GlagoliticCapitalLetterRitsi', [], 'Display', None],
  [ 0x2c14, 'GlagoliticCapitalLetterSlovo', [], 'Display', None],
  [ 0x2c15, 'GlagoliticCapitalLetterTvrido', [], 'Display', None],
  [ 0x2c16, 'GlagoliticCapitalLetterUku', [], 'Display', None],
  [ 0x2c17, 'GlagoliticCapitalLetterFritu', [], 'Display', None],
  [ 0x2c18, 'GlagoliticCapitalLetterHeru', [], 'Display', None],
  [ 0x2c19, 'GlagoliticCapitalLetterOtu', [], 'Display', None],
  [ 0x2c1a, 'GlagoliticCapitalLetterPe', [], 'Display', None],
  [ 0x2c1b, 'GlagoliticCapitalLetterShta', [], 'Display', None],
  [ 0x2c1c, 'GlagoliticCapitalLetterTsi', [], 'Display', None],
  [ 0x2c1d, 'GlagoliticCapitalLetterChrivi', [], 'Display', None],
  [ 0x2c1e, 'GlagoliticCapitalLetterSha', [], 'Display', None],
  [ 0x2c1f, 'GlagoliticCapitalLetterYeru', [], 'Display', None],
  [ 0x2c20, 'GlagoliticCapitalLetterYeri', [], 'Display', None],
  [ 0x2c21, 'GlagoliticCapitalLetterYati', [], 'Display', None],
  [ 0x2c22, 'GlagoliticCapitalLetterSpideryHa', [], 'Display', None],
  [ 0x2c23, 'GlagoliticCapitalLetterYu', [], 'Display', None],
  [ 0x2c24, 'GlagoliticCapitalLetterSmallYus', [], 'Display', None],
  [ 0x2c25, 'GlagoliticCapitalLetterSmallYusWithTail', [], 'Display', None],
  [ 0x2c26, 'GlagoliticCapitalLetterYo', [], 'Display', None],
  [ 0x2c27, 'GlagoliticCapitalLetterIotatedSmallYus', [], 'Display', None],
  [ 0x2c28, 'GlagoliticCapitalLetterBigYus', [], 'Display', None],
  [ 0x2c29, 'GlagoliticCapitalLetterIotatedBigYus', [], 'Display', None],
  [ 0x2c2a, 'GlagoliticCapitalLetterFita', [], 'Display', None],
  [ 0x2c2b, 'GlagoliticCapitalLetterIzhitsa', [], 'Display', None],
  [ 0x2c2c, 'GlagoliticCapitalLetterShtapic', [], 'Display', None],
  [ 0x2c2d, 'GlagoliticCapitalLetterTrokutastiA', [], 'Display', None],
  [ 0x2c2e, 'GlagoliticCapitalLetterLatinateMyslite', [], 'Display', None],
  [ 0x2c2f, 'UnicodeCharacter0x2c2f', [], 'Display', None],
  [ 0x2c30, 'GlagoliticSmallLetterAzu', [], 'Display', None],
  [ 0x2c31, 'GlagoliticSmallLetterBuky', [], 'Display', None],
  [ 0x2c32, 'GlagoliticSmallLetterVede', [], 'Display', None],
  [ 0x2c33, 'GlagoliticSmallLetterGlagoli', [], 'Display', None],
  [ 0x2c34, 'GlagoliticSmallLetterDobro', [], 'Display', None],
  [ 0x2c35, 'GlagoliticSmallLetterYestu', [], 'Display', None],
  [ 0x2c36, 'GlagoliticSmallLetterZhivete', [], 'Display', None],
  [ 0x2c37, 'GlagoliticSmallLetterDzelo', [], 'Display', None],
  [ 0x2c38, 'GlagoliticSmallLetterZemlja', [], 'Display', None],
  [ 0x2c39, 'GlagoliticSmallLetterIzhe', [], 'Display', None],
  [ 0x2c3a, 'GlagoliticSmallLetterInitialIzhe', [], 'Display', None],
  [ 0x2c3b, 'GlagoliticSmallLetterI', [], 'Display', None],
  [ 0x2c3c, 'GlagoliticSmallLetterDjervi', [], 'Display', None],
  [ 0x2c3d, 'GlagoliticSmallLetterKako', [], 'Display', None],
  [ 0x2c3e, 'GlagoliticSmallLetterLjudije', [], 'Display', None],
  [ 0x2c3f, 'GlagoliticSmallLetterMyslite', [], 'Display', None],
  [ 0x2c40, 'GlagoliticSmallLetterNashi', [], 'Display', None],
  [ 0x2c41, 'GlagoliticSmallLetterOnu', [], 'Display', None],
  [ 0x2c42, 'GlagoliticSmallLetterPokoji', [], 'Display', None],
  [ 0x2c43, 'GlagoliticSmallLetterRitsi', [], 'Display', None],
  [ 0x2c44, 'GlagoliticSmallLetterSlovo', [], 'Display', None],
  [ 0x2c45, 'GlagoliticSmallLetterTvrido', [], 'Display', None],
  [ 0x2c46, 'GlagoliticSmallLetterUku', [], 'Display', None],
  [ 0x2c47, 'GlagoliticSmallLetterFritu', [], 'Display', None],
  [ 0x2c48, 'GlagoliticSmallLetterHeru', [], 'Display', None],
  [ 0x2c49, 'GlagoliticSmallLetterOtu', [], 'Display', None],
  [ 0x2c4a, 'GlagoliticSmallLetterPe', [], 'Display', None],
  [ 0x2c4b, 'GlagoliticSmallLetterShta', [], 'Display', None],
  [ 0x2c4c, 'GlagoliticSmallLetterTsi', [], 'Display', None],
  [ 0x2c4d, 'GlagoliticSmallLetterChrivi', [], 'Display', None],
  [ 0x2c4e, 'GlagoliticSmallLetterSha', [], 'Display', None],
  [ 0x2c4f, 'GlagoliticSmallLetterYeru', [], 'Display', None],
  [ 0x2c50, 'GlagoliticSmallLetterYeri', [], 'Display', None],
  [ 0x2c51, 'GlagoliticSmallLetterYati', [], 'Display', None],
  [ 0x2c52, 'GlagoliticSmallLetterSpideryHa', [], 'Display', None],
  [ 0x2c53, 'GlagoliticSmallLetterYu', [], 'Display', None],
  [ 0x2c54, 'GlagoliticSmallLetterSmallYus', [], 'Display', None],
  [ 0x2c55, 'GlagoliticSmallLetterSmallYusWithTail', [], 'Display', None],
  [ 0x2c56, 'GlagoliticSmallLetterYo', [], 'Display', None],
  [ 0x2c57, 'GlagoliticSmallLetterIotatedSmallYus', [], 'Display', None],
  [ 0x2c58, 'GlagoliticSmallLetterBigYus', [], 'Display', None],
  [ 0x2c59, 'GlagoliticSmallLetterIotatedBigYus', [], 'Display', None],
  [ 0x2c5a, 'GlagoliticSmallLetterFita', [], 'Display', None],
  [ 0x2c5b, 'GlagoliticSmallLetterIzhitsa', [], 'Display', None],
  [ 0x2c5c, 'GlagoliticSmallLetterShtapic', [], 'Display', None],
  [ 0x2c5d, 'GlagoliticSmallLetterTrokutastiA', [], 'Display', None],
  [ 0x2c5e, 'GlagoliticSmallLetterLatinateMyslite', [], 'Display', None],
  [ 0x2c5f, 'UnicodeCharacter0x2c5f', [], 'Display', None],
  [ 0x2c60, 'LatinCapitalLetterLWithDoubleBar', [], 'Display', None],
  [ 0x2c61, 'LatinSmallLetterLWithDoubleBar', [], 'Display', None],
  [ 0x2c62, 'LatinCapitalLetterLWithMiddleTilde', [], 'Display', None],
  [ 0x2c63, 'LatinCapitalLetterPWithStroke', [], 'Display', None],
  [ 0x2c64, 'LatinCapitalLetterRWithTail', [], 'Display', None],
  [ 0x2c65, 'LatinSmallLetterAWithStroke', [], 'Display', None],
  [ 0x2c66, 'LatinSmallLetterTWithDiagonalStroke', [], 'Display', None],
  [ 0x2c67, 'LatinCapitalLetterHWithDescender', [], 'Display', None],
  [ 0x2c68, 'LatinSmallLetterHWithDescender', [], 'Display', None],
  [ 0x2c69, 'LatinCapitalLetterKWithDescender', [], 'Display', None],
  [ 0x2c6a, 'LatinSmallLetterKWithDescender', [], 'Display', None],
  [ 0x2c6b, 'LatinCapitalLetterZWithDescender', [], 'Display', None],
  [ 0x2c6c, 'LatinSmallLetterZWithDescender', [], 'Display', None],
  [ 0x2c6d, 'LatinCapitalLetterAlpha', [], 'Display', None],
  [ 0x2c6e, 'LatinCapitalLetterMWithHook', [], 'Display', None],
  [ 0x2c6f, 'LatinCapitalLetterTurnedA', [], 'Display', None],
  [ 0x2c70, 'UnicodeCharacter0x2c70', [], 'Display', None],
  [ 0x2c71, 'LatinSmallLetterVWithRightHook', [], 'Display', None],
  [ 0x2c72, 'LatinCapitalLetterWWithHook', [], 'Display', None],
  [ 0x2c73, 'LatinSmallLetterWWithHook', [], 'Display', None],
  [ 0x2c74, 'LatinSmallLetterVWithCurl', [], 'Display', None],
  [ 0x2c75, 'LatinCapitalLetterHalfH', [], 'Display', None],
  [ 0x2c76, 'LatinSmallLetterHalfH', [], 'Display', None],
  [ 0x2c77, 'LatinSmallLetterTaillessPhi', [], 'Display', None],
  [ 0x2c78, 'LatinSmallLetterEWithNotch', [], 'Display', None],
  [ 0x2c79, 'LatinSmallLetterTurnedRWithTail', [], 'Display', None],
  [ 0x2c7a, 'LatinSmallLetterOWithLowRingInside', [], 'Display', None],
  [ 0x2c7b, 'LatinLetterSmallCapitalTurnedE', [], 'Display', None],
  [ 0x2c7c, 'LatinSubscriptSmallLetterJ', ['subj','_j'], 'Symbol', None],
  [ 0x2c7d, 'ModifierLetterCapitalV', [], 'Display', None],
  [ 0x2c7e, 'UnicodeCharacter0x2c7e', [], 'Display', None],
  [ 0x2c7f, 'UnicodeCharacter0x2c7f', [], 'Display', None],
  [ 0x2c80, 'CopticCapitalLetterAlfa', [], 'Display', None],
  [ 0x2c81, 'CopticSmallLetterAlfa', [], 'Display', None],
  [ 0x2c82, 'CopticCapitalLetterVida', [], 'Display', None],
  [ 0x2c83, 'CopticSmallLetterVida', [], 'Display', None],
  [ 0x2c84, 'CopticCapitalLetterGamma', [], 'Display', None],
  [ 0x2c85, 'CopticSmallLetterGamma', [], 'Display', None],
  [ 0x2c86, 'CopticCapitalLetterDalda', [], 'Display', None],
  [ 0x2c87, 'CopticSmallLetterDalda', [], 'Display', None],
  [ 0x2c88, 'CopticCapitalLetterEie', [], 'Display', None],
  [ 0x2c89, 'CopticSmallLetterEie', [], 'Display', None],
  [ 0x2c8a, 'CopticCapitalLetterSou', [], 'Display', None],
  [ 0x2c8b, 'CopticSmallLetterSou', [], 'Display', None],
  [ 0x2c8c, 'CopticCapitalLetterZata', [], 'Display', None],
  [ 0x2c8d, 'CopticSmallLetterZata', [], 'Display', None],
  [ 0x2c8e, 'CopticCapitalLetterHate', [], 'Display', None],
  [ 0x2c8f, 'CopticSmallLetterHate', [], 'Display', None],
  [ 0x2c90, 'CopticCapitalLetterThethe', [], 'Display', None],
  [ 0x2c91, 'CopticSmallLetterThethe', [], 'Display', None],
  [ 0x2c92, 'CopticCapitalLetterIauda', [], 'Display', None],
  [ 0x2c93, 'CopticSmallLetterIauda', [], 'Display', None],
  [ 0x2c94, 'CopticCapitalLetterKapa', [], 'Display', None],
  [ 0x2c95, 'CopticSmallLetterKapa', [], 'Display', None],
  [ 0x2c96, 'CopticCapitalLetterLaula', [], 'Display', None],
  [ 0x2c97, 'CopticSmallLetterLaula', [], 'Display', None],
  [ 0x2c98, 'CopticCapitalLetterMi', [], 'Display', None],
  [ 0x2c99, 'CopticSmallLetterMi', [], 'Display', None],
  [ 0x2c9a, 'CopticCapitalLetterNi', [], 'Display', None],
  [ 0x2c9b, 'CopticSmallLetterNi', [], 'Display', None],
  [ 0x2c9c, 'CopticCapitalLetterKsi', [], 'Display', None],
  [ 0x2c9d, 'CopticSmallLetterKsi', [], 'Display', None],
  [ 0x2c9e, 'CopticCapitalLetterO', [], 'Display', None],
  [ 0x2c9f, 'CopticSmallLetterO', [], 'Display', None],
  [ 0x2ca0, 'CopticCapitalLetterPi', [], 'Display', None],
  [ 0x2ca1, 'CopticSmallLetterPi', [], 'Display', None],
  [ 0x2ca2, 'CopticCapitalLetterRo', [], 'Display', None],
  [ 0x2ca3, 'CopticSmallLetterRo', [], 'Display', None],
  [ 0x2ca4, 'CopticCapitalLetterSima', [], 'Display', None],
  [ 0x2ca5, 'CopticSmallLetterSima', [], 'Display', None],
  [ 0x2ca6, 'CopticCapitalLetterTau', [], 'Display', None],
  [ 0x2ca7, 'CopticSmallLetterTau', [], 'Display', None],
  [ 0x2ca8, 'CopticCapitalLetterUa', [], 'Display', None],
  [ 0x2ca9, 'CopticSmallLetterUa', [], 'Display', None],
  [ 0x2caa, 'CopticCapitalLetterFi', [], 'Display', None],
  [ 0x2cab, 'CopticSmallLetterFi', [], 'Display', None],
  [ 0x2cac, 'CopticCapitalLetterKhi', [], 'Display', None],
  [ 0x2cad, 'CopticSmallLetterKhi', [], 'Display', None],
  [ 0x2cae, 'CopticCapitalLetterPsi', [], 'Display', None],
  [ 0x2caf, 'CopticSmallLetterPsi', [], 'Display', None],
  [ 0x2cb0, 'CopticCapitalLetterOou', [], 'Display', None],
  [ 0x2cb1, 'CopticSmallLetterOou', [], 'Display', None],
  [ 0x2cb2, 'CopticCapitalLetterDialectpAlef', [], 'Display', None],
  [ 0x2cb3, 'CopticSmallLetterDialectpAlef', [], 'Display', None],
  [ 0x2cb4, 'CopticCapitalLetterOldCopticAin', [], 'Display', None],
  [ 0x2cb5, 'CopticSmallLetterOldCopticAin', [], 'Display', None],
  [ 0x2cb6, 'CopticCapitalLetterCryptogrammicEie', [], 'Display', None],
  [ 0x2cb7, 'CopticSmallLetterCryptogrammicEie', [], 'Display', None],
  [ 0x2cb8, 'CopticCapitalLetterDialectpKapa', [], 'Display', None],
  [ 0x2cb9, 'CopticSmallLetterDialectpKapa', [], 'Display', None],
  [ 0x2cba, 'CopticCapitalLetterDialectpNi', [], 'Display', None],
  [ 0x2cbb, 'CopticSmallLetterDialectpNi', [], 'Display', None],
  [ 0x2cbc, 'CopticCapitalLetterCryptogrammicNi', [], 'Display', None],
  [ 0x2cbd, 'CopticSmallLetterCryptogrammicNi', [], 'Display', None],
  [ 0x2cbe, 'CopticCapitalLetterOldCopticOou', [], 'Display', None],
  [ 0x2cbf, 'CopticSmallLetterOldCopticOou', [], 'Display', None],
  [ 0x2cc0, 'CopticCapitalLetterSampi', [], 'Display', None],
  [ 0x2cc1, 'CopticSmallLetterSampi', [], 'Display', None],
  [ 0x2cc2, 'CopticCapitalLetterCrossedShei', [], 'Display', None],
  [ 0x2cc3, 'CopticSmallLetterCrossedShei', [], 'Display', None],
  [ 0x2cc4, 'CopticCapitalLetterOldCopticShei', [], 'Display', None],
  [ 0x2cc5, 'CopticSmallLetterOldCopticShei', [], 'Display', None],
  [ 0x2cc6, 'CopticCapitalLetterOldCopticEsh', [], 'Display', None],
  [ 0x2cc7, 'CopticSmallLetterOldCopticEsh', [], 'Display', None],
  [ 0x2cc8, 'CopticCapitalLetterAkhmimicKhei', [], 'Display', None],
  [ 0x2cc9, 'CopticSmallLetterAkhmimicKhei', [], 'Display', None],
  [ 0x2cca, 'CopticCapitalLetterDialectpHori', [], 'Display', None],
  [ 0x2ccb, 'CopticSmallLetterDialectpHori', [], 'Display', None],
  [ 0x2ccc, 'CopticCapitalLetterOldCopticHori', [], 'Display', None],
  [ 0x2ccd, 'CopticSmallLetterOldCopticHori', [], 'Display', None],
  [ 0x2cce, 'CopticCapitalLetterOldCopticHa', [], 'Display', None],
  [ 0x2ccf, 'CopticSmallLetterOldCopticHa', [], 'Display', None],
  [ 0x2cd0, 'CopticCapitalLetterLshapedHa', [], 'Display', None],
  [ 0x2cd1, 'CopticSmallLetterLshapedHa', [], 'Display', None],
  [ 0x2cd2, 'CopticCapitalLetterOldCopticHei', [], 'Display', None],
  [ 0x2cd3, 'CopticSmallLetterOldCopticHei', [], 'Display', None],
  [ 0x2cd4, 'CopticCapitalLetterOldCopticHat', [], 'Display', None],
  [ 0x2cd5, 'CopticSmallLetterOldCopticHat', [], 'Display', None],
  [ 0x2cd6, 'CopticCapitalLetterOldCopticGangia', [], 'Display', None],
  [ 0x2cd7, 'CopticSmallLetterOldCopticGangia', [], 'Display', None],
  [ 0x2cd8, 'CopticCapitalLetterOldCopticDja', [], 'Display', None],
  [ 0x2cd9, 'CopticSmallLetterOldCopticDja', [], 'Display', None],
  [ 0x2cda, 'CopticCapitalLetterOldCopticShima', [], 'Display', None],
  [ 0x2cdb, 'CopticSmallLetterOldCopticShima', [], 'Display', None],
  [ 0x2cdc, 'CopticCapitalLetterOldNubianShima', [], 'Display', None],
  [ 0x2cdd, 'CopticSmallLetterOldNubianShima', [], 'Display', None],
  [ 0x2cde, 'CopticCapitalLetterOldNubianNgi', [], 'Display', None],
  [ 0x2cdf, 'CopticSmallLetterOldNubianNgi', [], 'Display', None],
  [ 0x2ce0, 'CopticCapitalLetterOldNubianNyi', [], 'Display', None],
  [ 0x2ce1, 'CopticSmallLetterOldNubianNyi', [], 'Display', None],
  [ 0x2ce2, 'CopticCapitalLetterOldNubianWau', [], 'Display', None],
  [ 0x2ce3, 'CopticSmallLetterOldNubianWau', [], 'Display', None],
  [ 0x2ce4, 'CopticSymbolKai', [], 'Display', None],
  [ 0x2ce5, 'CopticSymbolMiRo', [], 'Display', None],
  [ 0x2ce6, 'CopticSymbolPiRo', [], 'Display', None],
  [ 0x2ce7, 'CopticSymbolStauros', [], 'Display', None],
  [ 0x2ce8, 'CopticSymbolTauRo', [], 'Display', None],
  [ 0x2ce9, 'CopticSymbolKhiRo', [], 'Display', None],
  [ 0x2cea, 'CopticSymbolShimaSima', [], 'Display', None],
  [ 0x2ceb, 'UnicodeCharacter0x2ceb', [], 'Display', None],
  [ 0x2cec, 'UnicodeCharacter0x2cec', [], 'Display', None],
  [ 0x2ced, 'UnicodeCharacter0x2ced', [], 'Display', None],
  [ 0x2cee, 'UnicodeCharacter0x2cee', [], 'Display', None],
  [ 0x2cef, 'UnicodeCharacter0x2cef', [], 'Display', None],
  [ 0x2cf0, 'UnicodeCharacter0x2cf0', [], 'Display', None],
  [ 0x2cf1, 'UnicodeCharacter0x2cf1', [], 'Display', None],
  [ 0x2cf2, 'UnicodeCharacter0x2cf2', [], 'Display', None],
  [ 0x2cf3, 'UnicodeCharacter0x2cf3', [], 'Display', None],
  [ 0x2cf4, 'UnicodeCharacter0x2cf4', [], 'Display', None],
  [ 0x2cf5, 'UnicodeCharacter0x2cf5', [], 'Display', None],
  [ 0x2cf6, 'UnicodeCharacter0x2cf6', [], 'Display', None],
  [ 0x2cf7, 'UnicodeCharacter0x2cf7', [], 'Display', None],
  [ 0x2cf8, 'UnicodeCharacter0x2cf8', [], 'Display', None],
  [ 0x2cf9, 'CopticOldNubianFullStop', [], 'Display', None],
  [ 0x2cfa, 'CopticOldNubianDirectQuestionMark', [], 'Display', None],
  [ 0x2cfb, 'CopticOldNubianIndirectQuestionMark', [], 'Display', None],
  [ 0x2cfc, 'CopticOldNubianVerseDivider', [], 'Display', None],
  [ 0x2cfd, 'CopticFractionOneHalf', [], 'Display', None],
  [ 0x2cfe, 'CopticFullStop', [], 'Display', None],
  [ 0x2cff, 'CopticMorphologicalDivider', [], 'Display', None],
  [ 0x2d00, 'GeorgianSmallLetterAn', [], 'Display', None],
  [ 0x2d01, 'GeorgianSmallLetterBan', [], 'Display', None],
  [ 0x2d02, 'GeorgianSmallLetterGan', [], 'Display', None],
  [ 0x2d03, 'GeorgianSmallLetterDon', [], 'Display', None],
  [ 0x2d04, 'GeorgianSmallLetterEn', [], 'Display', None],
  [ 0x2d05, 'GeorgianSmallLetterVin', [], 'Display', None],
  [ 0x2d06, 'GeorgianSmallLetterZen', [], 'Display', None],
  [ 0x2d07, 'GeorgianSmallLetterTan', [], 'Display', None],
  [ 0x2d08, 'GeorgianSmallLetterIn', [], 'Display', None],
  [ 0x2d09, 'GeorgianSmallLetterKan', [], 'Display', None],
  [ 0x2d0a, 'GeorgianSmallLetterLas', [], 'Display', None],
  [ 0x2d0b, 'GeorgianSmallLetterMan', [], 'Display', None],
  [ 0x2d0c, 'GeorgianSmallLetterNar', [], 'Display', None],
  [ 0x2d0d, 'GeorgianSmallLetterOn', [], 'Display', None],
  [ 0x2d0e, 'GeorgianSmallLetterPar', [], 'Display', None],
  [ 0x2d0f, 'GeorgianSmallLetterZhar', [], 'Display', None],
  [ 0x2d10, 'GeorgianSmallLetterRae', [], 'Display', None],
  [ 0x2d11, 'GeorgianSmallLetterSan', [], 'Display', None],
  [ 0x2d12, 'GeorgianSmallLetterTar', [], 'Display', None],
  [ 0x2d13, 'GeorgianSmallLetterUn', [], 'Display', None],
  [ 0x2d14, 'GeorgianSmallLetterPhar', [], 'Display', None],
  [ 0x2d15, 'GeorgianSmallLetterKhar', [], 'Display', None],
  [ 0x2d16, 'GeorgianSmallLetterGhan', [], 'Display', None],
  [ 0x2d17, 'GeorgianSmallLetterQar', [], 'Display', None],
  [ 0x2d18, 'GeorgianSmallLetterShin', [], 'Display', None],
  [ 0x2d19, 'GeorgianSmallLetterChin', [], 'Display', None],
  [ 0x2d1a, 'GeorgianSmallLetterCan', [], 'Display', None],
  [ 0x2d1b, 'GeorgianSmallLetterJil', [], 'Display', None],
  [ 0x2d1c, 'GeorgianSmallLetterCil', [], 'Display', None],
  [ 0x2d1d, 'GeorgianSmallLetterChar', [], 'Display', None],
  [ 0x2d1e, 'GeorgianSmallLetterXan', [], 'Display', None],
  [ 0x2d1f, 'GeorgianSmallLetterJhan', [], 'Display', None],
  [ 0x2d20, 'GeorgianSmallLetterHae', [], 'Display', None],
  [ 0x2d21, 'GeorgianSmallLetterHe', [], 'Display', None],
  [ 0x2d22, 'GeorgianSmallLetterHie', [], 'Display', None],
  [ 0x2d23, 'GeorgianSmallLetterWe', [], 'Display', None],
  [ 0x2d24, 'GeorgianSmallLetterHar', [], 'Display', None],
  [ 0x2d25, 'GeorgianSmallLetterHoe', [], 'Display', None],
  [ 0x2d26, 'UnicodeCharacter0x2d26', [], 'Display', None],
  [ 0x2d27, 'UnicodeCharacter0x2d27', [], 'Display', None],
  [ 0x2d28, 'UnicodeCharacter0x2d28', [], 'Display', None],
  [ 0x2d29, 'UnicodeCharacter0x2d29', [], 'Display', None],
  [ 0x2d2a, 'UnicodeCharacter0x2d2a', [], 'Display', None],
  [ 0x2d2b, 'UnicodeCharacter0x2d2b', [], 'Display', None],
  [ 0x2d2c, 'UnicodeCharacter0x2d2c', [], 'Display', None],
  [ 0x2d2d, 'UnicodeCharacter0x2d2d', [], 'Display', None],
  [ 0x2d2e, 'UnicodeCharacter0x2d2e', [], 'Display', None],
  [ 0x2d2f, 'UnicodeCharacter0x2d2f', [], 'Display', None],
  [ 0x2d30, 'TifinaghLetterYa', [], 'Display', None],
  [ 0x2d31, 'TifinaghLetterYab', [], 'Display', None],
  [ 0x2d32, 'TifinaghLetterYabh', [], 'Display', None],
  [ 0x2d33, 'TifinaghLetterYag', [], 'Display', None],
  [ 0x2d34, 'TifinaghLetterYaghh', [], 'Display', None],
  [ 0x2d35, 'TifinaghLetterBerberAcademyYaj', [], 'Display', None],
  [ 0x2d36, 'TifinaghLetterYaj', [], 'Display', None],
  [ 0x2d37, 'TifinaghLetterYad', [], 'Display', None],
  [ 0x2d38, 'TifinaghLetterYadh', [], 'Display', None],
  [ 0x2d39, 'TifinaghLetterYadd', [], 'Display', None],
  [ 0x2d3a, 'TifinaghLetterYaddh', [], 'Display', None],
  [ 0x2d3b, 'TifinaghLetterYey', [], 'Display', None],
  [ 0x2d3c, 'TifinaghLetterYaf', [], 'Display', None],
  [ 0x2d3d, 'TifinaghLetterYak', [], 'Display', None],
  [ 0x2d3e, 'TifinaghLetterTuaregYak', [], 'Display', None],
  [ 0x2d3f, 'TifinaghLetterYakhh', [], 'Display', None],
  [ 0x2d40, 'TifinaghLetterYah', [], 'Display', None],
  [ 0x2d41, 'TifinaghLetterBerberAcademyYah', [], 'Display', None],
  [ 0x2d42, 'TifinaghLetterTuaregYah', [], 'Display', None],
  [ 0x2d43, 'TifinaghLetterYahh', [], 'Display', None],
  [ 0x2d44, 'TifinaghLetterYaa', [], 'Display', None],
  [ 0x2d45, 'TifinaghLetterYakh', [], 'Display', None],
  [ 0x2d46, 'TifinaghLetterTuaregYakh', [], 'Display', None],
  [ 0x2d47, 'TifinaghLetterYaq', [], 'Display', None],
  [ 0x2d48, 'TifinaghLetterTuaregYaq', [], 'Display', None],
  [ 0x2d49, 'TifinaghLetterYi', [], 'Display', None],
  [ 0x2d4a, 'TifinaghLetterYazh', [], 'Display', None],
  [ 0x2d4b, 'TifinaghLetterAhaggarYazh', [], 'Display', None],
  [ 0x2d4c, 'TifinaghLetterTuaregYazh', [], 'Display', None],
  [ 0x2d4d, 'TifinaghLetterYal', [], 'Display', None],
  [ 0x2d4e, 'TifinaghLetterYam', [], 'Display', None],
  [ 0x2d4f, 'TifinaghLetterYan', [], 'Display', None],
  [ 0x2d50, 'TifinaghLetterTuaregYagn', [], 'Display', None],
  [ 0x2d51, 'TifinaghLetterTuaregYang', [], 'Display', None],
  [ 0x2d52, 'TifinaghLetterYap', [], 'Display', None],
  [ 0x2d53, 'TifinaghLetterYu', [], 'Display', None],
  [ 0x2d54, 'TifinaghLetterYar', [], 'Display', None],
  [ 0x2d55, 'TifinaghLetterYarr', [], 'Display', None],
  [ 0x2d56, 'TifinaghLetterYagh', [], 'Display', None],
  [ 0x2d57, 'TifinaghLetterTuaregYagh', [], 'Display', None],
  [ 0x2d58, 'TifinaghLetterAyerYagh', [], 'Display', None],
  [ 0x2d59, 'TifinaghLetterYas', [], 'Display', None],
  [ 0x2d5a, 'TifinaghLetterYass', [], 'Display', None],
  [ 0x2d5b, 'TifinaghLetterYash', [], 'Display', None],
  [ 0x2d5c, 'TifinaghLetterYat', [], 'Display', None],
  [ 0x2d5d, 'TifinaghLetterYath', [], 'Display', None],
  [ 0x2d5e, 'TifinaghLetterYach', [], 'Display', None],
  [ 0x2d5f, 'TifinaghLetterYatt', [], 'Display', None],
  [ 0x2d60, 'TifinaghLetterYav', [], 'Display', None],
  [ 0x2d61, 'TifinaghLetterYaw', [], 'Display', None],
  [ 0x2d62, 'TifinaghLetterYay', [], 'Display', None],
  [ 0x2d63, 'TifinaghLetterYaz', [], 'Display', None],
  [ 0x2d64, 'TifinaghLetterTawellemetYaz', [], 'Display', None],
  [ 0x2d65, 'TifinaghLetterYazz', [], 'Display', None],
  [ 0x2d66, 'UnicodeCharacter0x2d66', [], 'Display', None],
  [ 0x2d67, 'UnicodeCharacter0x2d67', [], 'Display', None],
  [ 0x2d68, 'UnicodeCharacter0x2d68', [], 'Display', None],
  [ 0x2d69, 'UnicodeCharacter0x2d69', [], 'Display', None],
  [ 0x2d6a, 'UnicodeCharacter0x2d6a', [], 'Display', None],
  [ 0x2d6b, 'UnicodeCharacter0x2d6b', [], 'Display', None],
  [ 0x2d6c, 'UnicodeCharacter0x2d6c', [], 'Display', None],
  [ 0x2d6d, 'UnicodeCharacter0x2d6d', [], 'Display', None],
  [ 0x2d6e, 'UnicodeCharacter0x2d6e', [], 'Display', None],
  [ 0x2d6f, 'TifinaghModifierLetterLabializationMark', [], 'Display', None],
  [ 0x2d70, 'UnicodeCharacter0x2d70', [], 'Display', None],
  [ 0x2d71, 'UnicodeCharacter0x2d71', [], 'Display', None],
  [ 0x2d72, 'UnicodeCharacter0x2d72', [], 'Display', None],
  [ 0x2d73, 'UnicodeCharacter0x2d73', [], 'Display', None],
  [ 0x2d74, 'UnicodeCharacter0x2d74', [], 'Display', None],
  [ 0x2d75, 'UnicodeCharacter0x2d75', [], 'Display', None],
  [ 0x2d76, 'UnicodeCharacter0x2d76', [], 'Display', None],
  [ 0x2d77, 'UnicodeCharacter0x2d77', [], 'Display', None],
  [ 0x2d78, 'UnicodeCharacter0x2d78', [], 'Display', None],
  [ 0x2d79, 'UnicodeCharacter0x2d79', [], 'Display', None],
  [ 0x2d7a, 'UnicodeCharacter0x2d7a', [], 'Display', None],
  [ 0x2d7b, 'UnicodeCharacter0x2d7b', [], 'Display', None],
  [ 0x2d7c, 'UnicodeCharacter0x2d7c', [], 'Display', None],
  [ 0x2d7d, 'UnicodeCharacter0x2d7d', [], 'Display', None],
  [ 0x2d7e, 'UnicodeCharacter0x2d7e', [], 'Display', None],
  [ 0x2d7f, 'UnicodeCharacter0x2d7f', [], 'Display', None],
  [ 0x2d80, 'EthiopicSyllableLoa', [], 'Display', None],
  [ 0x2d81, 'EthiopicSyllableMoa', [], 'Display', None],
  [ 0x2d82, 'EthiopicSyllableRoa', [], 'Display', None],
  [ 0x2d83, 'EthiopicSyllableSoa', [], 'Display', None],
  [ 0x2d84, 'EthiopicSyllableShoa', [], 'Display', None],
  [ 0x2d85, 'EthiopicSyllableBoa', [], 'Display', None],
  [ 0x2d86, 'EthiopicSyllableToa', [], 'Display', None],
  [ 0x2d87, 'EthiopicSyllableCoa', [], 'Display', None],
  [ 0x2d88, 'EthiopicSyllableNoa', [], 'Display', None],
  [ 0x2d89, 'EthiopicSyllableNyoa', [], 'Display', None],
  [ 0x2d8a, 'EthiopicSyllableGlottalOa', [], 'Display', None],
  [ 0x2d8b, 'EthiopicSyllableZoa', [], 'Display', None],
  [ 0x2d8c, 'EthiopicSyllableDoa', [], 'Display', None],
  [ 0x2d8d, 'EthiopicSyllableDdoa', [], 'Display', None],
  [ 0x2d8e, 'EthiopicSyllableJoa', [], 'Display', None],
  [ 0x2d8f, 'EthiopicSyllableThoa', [], 'Display', None],
  [ 0x2d90, 'EthiopicSyllableChoa', [], 'Display', None],
  [ 0x2d91, 'EthiopicSyllablePhoa', [], 'Display', None],
  [ 0x2d92, 'EthiopicSyllablePoa', [], 'Display', None],
  [ 0x2d93, 'EthiopicSyllableGgwa', [], 'Display', None],
  [ 0x2d94, 'EthiopicSyllableGgwi', [], 'Display', None],
  [ 0x2d95, 'EthiopicSyllableGgwee', [], 'Display', None],
  [ 0x2d96, 'EthiopicSyllableGgwe', [], 'Display', None],
  [ 0x2d97, 'UnicodeCharacter0x2d97', [], 'Display', None],
  [ 0x2d98, 'UnicodeCharacter0x2d98', [], 'Display', None],
  [ 0x2d99, 'UnicodeCharacter0x2d99', [], 'Display', None],
  [ 0x2d9a, 'UnicodeCharacter0x2d9a', [], 'Display', None],
  [ 0x2d9b, 'UnicodeCharacter0x2d9b', [], 'Display', None],
  [ 0x2d9c, 'UnicodeCharacter0x2d9c', [], 'Display', None],
  [ 0x2d9d, 'UnicodeCharacter0x2d9d', [], 'Display', None],
  [ 0x2d9e, 'UnicodeCharacter0x2d9e', [], 'Display', None],
  [ 0x2d9f, 'UnicodeCharacter0x2d9f', [], 'Display', None],
  [ 0x2da0, 'EthiopicSyllableSsa', [], 'Display', None],
  [ 0x2da1, 'EthiopicSyllableSsu', [], 'Display', None],
  [ 0x2da2, 'EthiopicSyllableSsi', [], 'Display', None],
  [ 0x2da3, 'EthiopicSyllableSsaa', [], 'Display', None],
  [ 0x2da4, 'EthiopicSyllableSsee', [], 'Display', None],
  [ 0x2da5, 'EthiopicSyllableSse', [], 'Display', None],
  [ 0x2da6, 'EthiopicSyllableSso', [], 'Display', None],
  [ 0x2da7, 'UnicodeCharacter0x2da7', [], 'Display', None],
  [ 0x2da8, 'EthiopicSyllableCca', [], 'Display', None],
  [ 0x2da9, 'EthiopicSyllableCcu', [], 'Display', None],
  [ 0x2daa, 'EthiopicSyllableCci', [], 'Display', None],
  [ 0x2dab, 'EthiopicSyllableCcaa', [], 'Display', None],
  [ 0x2dac, 'EthiopicSyllableCcee', [], 'Display', None],
  [ 0x2dad, 'EthiopicSyllableCce', [], 'Display', None],
  [ 0x2dae, 'EthiopicSyllableCco', [], 'Display', None],
  [ 0x2daf, 'UnicodeCharacter0x2daf', [], 'Display', None],
  [ 0x2db0, 'EthiopicSyllableZza', [], 'Display', None],
  [ 0x2db1, 'EthiopicSyllableZzu', [], 'Display', None],
  [ 0x2db2, 'EthiopicSyllableZzi', [], 'Display', None],
  [ 0x2db3, 'EthiopicSyllableZzaa', [], 'Display', None],
  [ 0x2db4, 'EthiopicSyllableZzee', [], 'Display', None],
  [ 0x2db5, 'EthiopicSyllableZze', [], 'Display', None],
  [ 0x2db6, 'EthiopicSyllableZzo', [], 'Display', None],
  [ 0x2db7, 'UnicodeCharacter0x2db7', [], 'Display', None],
  [ 0x2db8, 'EthiopicSyllableCcha', [], 'Display', None],
  [ 0x2db9, 'EthiopicSyllableCchu', [], 'Display', None],
  [ 0x2dba, 'EthiopicSyllableCchi', [], 'Display', None],
  [ 0x2dbb, 'EthiopicSyllableCchaa', [], 'Display', None],
  [ 0x2dbc, 'EthiopicSyllableCchee', [], 'Display', None],
  [ 0x2dbd, 'EthiopicSyllableCche', [], 'Display', None],
  [ 0x2dbe, 'EthiopicSyllableCcho', [], 'Display', None],
  [ 0x2dbf, 'UnicodeCharacter0x2dbf', [], 'Display', None],
  [ 0x2dc0, 'EthiopicSyllableQya', [], 'Display', None],
  [ 0x2dc1, 'EthiopicSyllableQyu', [], 'Display', None],
  [ 0x2dc2, 'EthiopicSyllableQyi', [], 'Display', None],
  [ 0x2dc3, 'EthiopicSyllableQyaa', [], 'Display', None],
  [ 0x2dc4, 'EthiopicSyllableQyee', [], 'Display', None],
  [ 0x2dc5, 'EthiopicSyllableQye', [], 'Display', None],
  [ 0x2dc6, 'EthiopicSyllableQyo', [], 'Display', None],
  [ 0x2dc7, 'UnicodeCharacter0x2dc7', [], 'Display', None],
  [ 0x2dc8, 'EthiopicSyllableKya', [], 'Display', None],
  [ 0x2dc9, 'EthiopicSyllableKyu', [], 'Display', None],
  [ 0x2dca, 'EthiopicSyllableKyi', [], 'Display', None],
  [ 0x2dcb, 'EthiopicSyllableKyaa', [], 'Display', None],
  [ 0x2dcc, 'EthiopicSyllableKyee', [], 'Display', None],
  [ 0x2dcd, 'EthiopicSyllableKye', [], 'Display', None],
  [ 0x2dce, 'EthiopicSyllableKyo', [], 'Display', None],
  [ 0x2dcf, 'UnicodeCharacter0x2dcf', [], 'Display', None],
  [ 0x2dd0, 'EthiopicSyllableXya', [], 'Display', None],
  [ 0x2dd1, 'EthiopicSyllableXyu', [], 'Display', None],
  [ 0x2dd2, 'EthiopicSyllableXyi', [], 'Display', None],
  [ 0x2dd3, 'EthiopicSyllableXyaa', [], 'Display', None],
  [ 0x2dd4, 'EthiopicSyllableXyee', [], 'Display', None],
  [ 0x2dd5, 'EthiopicSyllableXye', [], 'Display', None],
  [ 0x2dd6, 'EthiopicSyllableXyo', [], 'Display', None],
  [ 0x2dd7, 'UnicodeCharacter0x2dd7', [], 'Display', None],
  [ 0x2dd8, 'EthiopicSyllableGya', [], 'Display', None],
  [ 0x2dd9, 'EthiopicSyllableGyu', [], 'Display', None],
  [ 0x2dda, 'EthiopicSyllableGyi', [], 'Display', None],
  [ 0x2ddb, 'EthiopicSyllableGyaa', [], 'Display', None],
  [ 0x2ddc, 'EthiopicSyllableGyee', [], 'Display', None],
  [ 0x2ddd, 'EthiopicSyllableGye', [], 'Display', None],
  [ 0x2dde, 'EthiopicSyllableGyo', [], 'Display', None],
  [ 0x2ddf, 'UnicodeCharacter0x2ddf', [], 'Display', None],
  [ 0x2de0, 'CombiningCyrillicLetterBe', [], 'Display', None],
  [ 0x2de1, 'CombiningCyrillicLetterVe', [], 'Display', None],
  [ 0x2de2, 'CombiningCyrillicLetterGhe', [], 'Display', None],
  [ 0x2de3, 'CombiningCyrillicLetterDe', [], 'Display', None],
  [ 0x2de4, 'CombiningCyrillicLetterZhe', [], 'Display', None],
  [ 0x2de5, 'CombiningCyrillicLetterZe', [], 'Display', None],
  [ 0x2de6, 'CombiningCyrillicLetterKa', [], 'Display', None],
  [ 0x2de7, 'CombiningCyrillicLetterEl', [], 'Display', None],
  [ 0x2de8, 'CombiningCyrillicLetterEm', [], 'Display', None],
  [ 0x2de9, 'CombiningCyrillicLetterEn', [], 'Display', None],
  [ 0x2dea, 'CombiningCyrillicLetterO', [], 'Display', None],
  [ 0x2deb, 'CombiningCyrillicLetterPe', [], 'Display', None],
  [ 0x2dec, 'CombiningCyrillicLetterEr', [], 'Display', None],
  [ 0x2ded, 'CombiningCyrillicLetterEs', [], 'Display', None],
  [ 0x2dee, 'CombiningCyrillicLetterTe', [], 'Display', None],
  [ 0x2def, 'CombiningCyrillicLetterHa', [], 'Display', None],
  [ 0x2df0, 'CombiningCyrillicLetterTse', [], 'Display', None],
  [ 0x2df1, 'CombiningCyrillicLetterChe', [], 'Display', None],
  [ 0x2df2, 'CombiningCyrillicLetterSha', [], 'Display', None],
  [ 0x2df3, 'CombiningCyrillicLetterShcha', [], 'Display', None],
  [ 0x2df4, 'CombiningCyrillicLetterFita', [], 'Display', None],
  [ 0x2df5, 'CombiningCyrillicLetterEste', [], 'Display', None],
  [ 0x2df6, 'CombiningCyrillicLetterA', [], 'Display', None],
  [ 0x2df7, 'CombiningCyrillicLetterIe', [], 'Display', None],
  [ 0x2df8, 'CombiningCyrillicLetterDjerv', [], 'Display', None],
  [ 0x2df9, 'CombiningCyrillicLetterMonographUk', [], 'Display', None],
  [ 0x2dfa, 'CombiningCyrillicLetterYat', [], 'Display', None],
  [ 0x2dfb, 'CombiningCyrillicLetterYu', [], 'Display', None],
  [ 0x2dfc, 'CombiningCyrillicLetterIotifiedA', [], 'Display', None],
  [ 0x2dfd, 'CombiningCyrillicLetterLittleYus', [], 'Display', None],
  [ 0x2dfe, 'CombiningCyrillicLetterBigYus', [], 'Display', None],
  [ 0x2dff, 'CombiningCyrillicLetterIotifiedBigYus', [], 'Display', None],
  [ 0x2e00, 'RightAngleSubstitutionMarker', [], 'Display', None],
  [ 0x2e01, 'RightAngleDottedSubstitutionMarker', [], 'Display', None],
  [ 0x2e02, 'LeftSubstitutionBracket', [], 'Display', None],
  [ 0x2e03, 'RightSubstitutionBracket', [], 'Display', None],
  [ 0x2e04, 'LeftDottedSubstitutionBracket', [], 'Display', None],
  [ 0x2e05, 'RightDottedSubstitutionBracket', [], 'Display', None],
  [ 0x2e06, 'RaisedInterpolationMarker', [], 'Display', None],
  [ 0x2e07, 'RaisedDottedInterpolationMarker', [], 'Display', None],
  [ 0x2e08, 'DottedTranspositionMarker', [], 'Display', None],
  [ 0x2e09, 'LeftTranspositionBracket', [], 'Display', None],
  [ 0x2e0a, 'RightTranspositionBracket', [], 'Display', None],
  [ 0x2e0b, 'RaisedSquare', [], 'Display', None],
  [ 0x2e0c, 'LeftRaisedOmissionBracket', [], 'Display', None],
  [ 0x2e0d, 'RightRaisedOmissionBracket', [], 'Display', None],
  [ 0x2e0e, 'EditorialCoronis', [], 'Display', None],
  [ 0x2e0f, 'Paragraphos', [], 'Display', None],
  [ 0x2e10, 'ForkedParagraphos', [], 'Display', None],
  [ 0x2e11, 'ReversedForkedParagraphos', [], 'Display', None],
  [ 0x2e12, 'Hypodiastole', [], 'Display', None],
  [ 0x2e13, 'DottedObelos', [], 'Display', None],
  [ 0x2e14, 'DownwardsAncora', [], 'Display', None],
  [ 0x2e15, 'UpwardsAncora', [], 'Display', None],
  [ 0x2e16, 'DottedRightpointingAngle', [], 'Display', None],
  [ 0x2e17, 'DoubleObliqueHyphen', [], 'Display', None],
  [ 0x2e18, 'InvertedInterrobang', [], 'Display', None],
  [ 0x2e19, 'PalmBranch', [], 'Display', None],
  [ 0x2e1a, 'HyphenWithDiaeresis', [], 'Display', None],
  [ 0x2e1b, 'TildeWithRingAbove', [], 'Display', None],
  [ 0x2e1c, 'LeftLowParaphraseBracket', [], 'Display', None],
  [ 0x2e1d, 'RightLowParaphraseBracket', [], 'Display', None],
  [ 0x2e1e, 'TildeWithDotAbove', [], 'Display', None],
  [ 0x2e1f, 'TildeWithDotBelow', [], 'Display', None],
  [ 0x2e20, 'LeftVerticalBarWithQuill', [], 'Display', None],
  [ 0x2e21, 'RightVerticalBarWithQuill', [], 'Display', None],
  [ 0x2e22, 'TopLeftHalfBracket', [], 'Display', None],
  [ 0x2e23, 'TopRightHalfBracket', [], 'Display', None],
  [ 0x2e24, 'BottomLeftHalfBracket', [], 'Display', None],
  [ 0x2e25, 'BottomRightHalfBracket', [], 'Display', None],
  [ 0x2e26, 'LeftSidewaysUBracket', [], 'Display', None],
  [ 0x2e27, 'RightSidewaysUBracket', [], 'Display', None],
  [ 0x2e28, 'LeftDoubleParenthesis', [], 'Display', None],
  [ 0x2e29, 'RightDoubleParenthesis', [], 'Display', None],
  [ 0x2e2a, 'TwoDotsOverOneDotPunctuation', [], 'Display', None],
  [ 0x2e2b, 'OneDotOverTwoDotsPunctuation', [], 'Display', None],
  [ 0x2e2c, 'SquaredFourDotPunctuation', [], 'Display', None],
  [ 0x2e2d, 'FiveDotMark', [], 'Display', None],
  [ 0x2e2e, 'ReversedQuestionMark', [], 'Display', None],
  [ 0x2e2f, 'VerticalTilde', [], 'Display', None],
  [ 0x2e30, 'RingPoint', [], 'Display', None],
  [ 0x2e31, 'UnicodeCharacter0x2e31', [], 'Display', None],
  [ 0x2e32, 'UnicodeCharacter0x2e32', [], 'Display', None],
  [ 0x2e33, 'UnicodeCharacter0x2e33', [], 'Display', None],
  [ 0x2e34, 'UnicodeCharacter0x2e34', [], 'Display', None],
  [ 0x2e35, 'UnicodeCharacter0x2e35', [], 'Display', None],
  [ 0x2e36, 'UnicodeCharacter0x2e36', [], 'Display', None],
  [ 0x2e37, 'UnicodeCharacter0x2e37', [], 'Display', None],
  [ 0x2e38, 'UnicodeCharacter0x2e38', [], 'Display', None],
  [ 0x2e39, 'UnicodeCharacter0x2e39', [], 'Display', None],
  [ 0x2e3a, 'UnicodeCharacter0x2e3a', [], 'Display', None],
  [ 0x2e3b, 'UnicodeCharacter0x2e3b', [], 'Display', None],
  [ 0x2e3c, 'UnicodeCharacter0x2e3c', [], 'Display', None],
  [ 0x2e3d, 'UnicodeCharacter0x2e3d', [], 'Display', None],
  [ 0x2e3e, 'UnicodeCharacter0x2e3e', [], 'Display', None],
  [ 0x2e3f, 'UnicodeCharacter0x2e3f', [], 'Display', None],
  [ 0x2e40, 'UnicodeCharacter0x2e40', [], 'Display', None],
  [ 0x2e41, 'UnicodeCharacter0x2e41', [], 'Display', None],
  [ 0x2e42, 'UnicodeCharacter0x2e42', [], 'Display', None],
  [ 0x2e43, 'UnicodeCharacter0x2e43', [], 'Display', None],
  [ 0x2e44, 'UnicodeCharacter0x2e44', [], 'Display', None],
  [ 0x2e45, 'UnicodeCharacter0x2e45', [], 'Display', None],
  [ 0x2e46, 'UnicodeCharacter0x2e46', [], 'Display', None],
  [ 0x2e47, 'UnicodeCharacter0x2e47', [], 'Display', None],
  [ 0x2e48, 'UnicodeCharacter0x2e48', [], 'Display', None],
  [ 0x2e49, 'UnicodeCharacter0x2e49', [], 'Display', None],
  [ 0x2e4a, 'UnicodeCharacter0x2e4a', [], 'Display', None],
  [ 0x2e4b, 'UnicodeCharacter0x2e4b', [], 'Display', None],
  [ 0x2e4c, 'UnicodeCharacter0x2e4c', [], 'Display', None],
  [ 0x2e4d, 'UnicodeCharacter0x2e4d', [], 'Display', None],
  [ 0x2e4e, 'UnicodeCharacter0x2e4e', [], 'Display', None],
  [ 0x2e4f, 'UnicodeCharacter0x2e4f', [], 'Display', None],
  [ 0x2e50, 'UnicodeCharacter0x2e50', [], 'Display', None],
  [ 0x2e51, 'UnicodeCharacter0x2e51', [], 'Display', None],
  [ 0x2e52, 'UnicodeCharacter0x2e52', [], 'Display', None],
  [ 0x2e53, 'UnicodeCharacter0x2e53', [], 'Display', None],
  [ 0x2e54, 'UnicodeCharacter0x2e54', [], 'Display', None],
  [ 0x2e55, 'UnicodeCharacter0x2e55', [], 'Display', None],
  [ 0x2e56, 'UnicodeCharacter0x2e56', [], 'Display', None],
  [ 0x2e57, 'UnicodeCharacter0x2e57', [], 'Display', None],
  [ 0x2e58, 'UnicodeCharacter0x2e58', [], 'Display', None],
  [ 0x2e59, 'UnicodeCharacter0x2e59', [], 'Display', None],
  [ 0x2e5a, 'UnicodeCharacter0x2e5a', [], 'Display', None],
  [ 0x2e5b, 'UnicodeCharacter0x2e5b', [], 'Display', None],
  [ 0x2e5c, 'UnicodeCharacter0x2e5c', [], 'Display', None],
  [ 0x2e5d, 'UnicodeCharacter0x2e5d', [], 'Display', None],
  [ 0x2e5e, 'UnicodeCharacter0x2e5e', [], 'Display', None],
  [ 0x2e5f, 'UnicodeCharacter0x2e5f', [], 'Display', None],
  [ 0x2e60, 'UnicodeCharacter0x2e60', [], 'Display', None],
  [ 0x2e61, 'UnicodeCharacter0x2e61', [], 'Display', None],
  [ 0x2e62, 'UnicodeCharacter0x2e62', [], 'Display', None],
  [ 0x2e63, 'UnicodeCharacter0x2e63', [], 'Display', None],
  [ 0x2e64, 'UnicodeCharacter0x2e64', [], 'Display', None],
  [ 0x2e65, 'UnicodeCharacter0x2e65', [], 'Display', None],
  [ 0x2e66, 'UnicodeCharacter0x2e66', [], 'Display', None],
  [ 0x2e67, 'UnicodeCharacter0x2e67', [], 'Display', None],
  [ 0x2e68, 'UnicodeCharacter0x2e68', [], 'Display', None],
  [ 0x2e69, 'UnicodeCharacter0x2e69', [], 'Display', None],
  [ 0x2e6a, 'UnicodeCharacter0x2e6a', [], 'Display', None],
  [ 0x2e6b, 'UnicodeCharacter0x2e6b', [], 'Display', None],
  [ 0x2e6c, 'UnicodeCharacter0x2e6c', [], 'Display', None],
  [ 0x2e6d, 'UnicodeCharacter0x2e6d', [], 'Display', None],
  [ 0x2e6e, 'UnicodeCharacter0x2e6e', [], 'Display', None],
  [ 0x2e6f, 'UnicodeCharacter0x2e6f', [], 'Display', None],
  [ 0x2e70, 'UnicodeCharacter0x2e70', [], 'Display', None],
  [ 0x2e71, 'UnicodeCharacter0x2e71', [], 'Display', None],
  [ 0x2e72, 'UnicodeCharacter0x2e72', [], 'Display', None],
  [ 0x2e73, 'UnicodeCharacter0x2e73', [], 'Display', None],
  [ 0x2e74, 'UnicodeCharacter0x2e74', [], 'Display', None],
  [ 0x2e75, 'UnicodeCharacter0x2e75', [], 'Display', None],
  [ 0x2e76, 'UnicodeCharacter0x2e76', [], 'Display', None],
  [ 0x2e77, 'UnicodeCharacter0x2e77', [], 'Display', None],
  [ 0x2e78, 'UnicodeCharacter0x2e78', [], 'Display', None],
  [ 0x2e79, 'UnicodeCharacter0x2e79', [], 'Display', None],
  [ 0x2e7a, 'UnicodeCharacter0x2e7a', [], 'Display', None],
  [ 0x2e7b, 'UnicodeCharacter0x2e7b', [], 'Display', None],
  [ 0x2e7c, 'UnicodeCharacter0x2e7c', [], 'Display', None],
  [ 0x2e7d, 'UnicodeCharacter0x2e7d', [], 'Display', None],
  [ 0x2e7e, 'UnicodeCharacter0x2e7e', [], 'Display', None],
  [ 0x2e7f, 'UnicodeCharacter0x2e7f', [], 'Display', None],
  [ 0x2e80, 'CjkRadicalRepeat', [], 'Display', None],
  [ 0x2e81, 'CjkRadicalCliff', [], 'Display', None],
  [ 0x2e82, 'CjkRadicalSecondOne', [], 'Display', None],
  [ 0x2e83, 'CjkRadicalSecondTwo', [], 'Display', None],
  [ 0x2e84, 'CjkRadicalSecondThree', [], 'Display', None],
  [ 0x2e85, 'CjkRadicalPerson', [], 'Display', None],
  [ 0x2e86, 'CjkRadicalBox', [], 'Display', None],
  [ 0x2e87, 'CjkRadicalTable', [], 'Display', None],
  [ 0x2e88, 'CjkRadicalKnifeOne', [], 'Display', None],
  [ 0x2e89, 'CjkRadicalKnifeTwo', [], 'Display', None],
  [ 0x2e8a, 'CjkRadicalDivination', [], 'Display', None],
  [ 0x2e8b, 'CjkRadicalSeal', [], 'Display', None],
  [ 0x2e8c, 'CjkRadicalSmallOne', [], 'Display', None],
  [ 0x2e8d, 'CjkRadicalSmallTwo', [], 'Display', None],
  [ 0x2e8e, 'CjkRadicalLameOne', [], 'Display', None],
  [ 0x2e8f, 'CjkRadicalLameTwo', [], 'Display', None],
  [ 0x2e90, 'CjkRadicalLameThree', [], 'Display', None],
  [ 0x2e91, 'CjkRadicalLameFour', [], 'Display', None],
  [ 0x2e92, 'CjkRadicalSnake', [], 'Display', None],
  [ 0x2e93, 'CjkRadicalThread', [], 'Display', None],
  [ 0x2e94, 'CjkRadicalSnoutOne', [], 'Display', None],
  [ 0x2e95, 'CjkRadicalSnoutTwo', [], 'Display', None],
  [ 0x2e96, 'CjkRadicalHeartOne', [], 'Display', None],
  [ 0x2e97, 'CjkRadicalHeartTwo', [], 'Display', None],
  [ 0x2e98, 'CjkRadicalHand', [], 'Display', None],
  [ 0x2e99, 'CjkRadicalRap', [], 'Display', None],
  [ 0x2e9a, 'UnicodeCharacter0x2e9a', [], 'Display', None],
  [ 0x2e9b, 'CjkRadicalChoke', [], 'Display', None],
  [ 0x2e9c, 'CjkRadicalSun', [], 'Display', None],
  [ 0x2e9d, 'CjkRadicalMoon', [], 'Display', None],
  [ 0x2e9e, 'CjkRadicalDeath', [], 'Display', None],
  [ 0x2e9f, 'CjkRadicalMother', [], 'Display', None],
  [ 0x2ea0, 'CjkRadicalCivilian', [], 'Display', None],
  [ 0x2ea1, 'CjkRadicalWaterOne', [], 'Display', None],
  [ 0x2ea2, 'CjkRadicalWaterTwo', [], 'Display', None],
  [ 0x2ea3, 'CjkRadicalFire', [], 'Display', None],
  [ 0x2ea4, 'CjkRadicalPawOne', [], 'Display', None],
  [ 0x2ea5, 'CjkRadicalPawTwo', [], 'Display', None],
  [ 0x2ea6, 'CjkRadicalSimplifiedHalfTreeTrunk', [], 'Display', None],
  [ 0x2ea7, 'CjkRadicalCow', [], 'Display', None],
  [ 0x2ea8, 'CjkRadicalDog', [], 'Display', None],
  [ 0x2ea9, 'CjkRadicalJade', [], 'Display', None],
  [ 0x2eaa, 'CjkRadicalBoltOfCloth', [], 'Display', None],
  [ 0x2eab, 'CjkRadicalEye', [], 'Display', None],
  [ 0x2eac, 'CjkRadicalSpiritOne', [], 'Display', None],
  [ 0x2ead, 'CjkRadicalSpiritTwo', [], 'Display', None],
  [ 0x2eae, 'CjkRadicalBamboo', [], 'Display', None],
  [ 0x2eaf, 'CjkRadicalSilk', [], 'Display', None],
  [ 0x2eb0, 'CjkRadicalCsimplifiedSilk', [], 'Display', None],
  [ 0x2eb1, 'CjkRadicalNetOne', [], 'Display', None],
  [ 0x2eb2, 'CjkRadicalNetTwo', [], 'Display', None],
  [ 0x2eb3, 'CjkRadicalNetThree', [], 'Display', None],
  [ 0x2eb4, 'CjkRadicalNetFour', [], 'Display', None],
  [ 0x2eb5, 'CjkRadicalMesh', [], 'Display', None],
  [ 0x2eb6, 'CjkRadicalSheep', [], 'Display', None],
  [ 0x2eb7, 'CjkRadicalRam', [], 'Display', None],
  [ 0x2eb8, 'CjkRadicalEwe', [], 'Display', None],
  [ 0x2eb9, 'CjkRadicalOld', [], 'Display', None],
  [ 0x2eba, 'CjkRadicalBrushOne', [], 'Display', None],
  [ 0x2ebb, 'CjkRadicalBrushTwo', [], 'Display', None],
  [ 0x2ebc, 'CjkRadicalMeat', [], 'Display', None],
  [ 0x2ebd, 'CjkRadicalMortar', [], 'Display', None],
  [ 0x2ebe, 'CjkRadicalGrassOne', [], 'Display', None],
  [ 0x2ebf, 'CjkRadicalGrassTwo', [], 'Display', None],
  [ 0x2ec0, 'CjkRadicalGrassThree', [], 'Display', None],
  [ 0x2ec1, 'CjkRadicalTiger', [], 'Display', None],
  [ 0x2ec2, 'CjkRadicalClothes', [], 'Display', None],
  [ 0x2ec3, 'CjkRadicalWestOne', [], 'Display', None],
  [ 0x2ec4, 'CjkRadicalWestTwo', [], 'Display', None],
  [ 0x2ec5, 'CjkRadicalCsimplifiedSee', [], 'Display', None],
  [ 0x2ec6, 'CjkRadicalSimplifiedHorn', [], 'Display', None],
  [ 0x2ec7, 'CjkRadicalHorn', [], 'Display', None],
  [ 0x2ec8, 'CjkRadicalCsimplifiedSpeech', [], 'Display', None],
  [ 0x2ec9, 'CjkRadicalCsimplifiedShell', [], 'Display', None],
  [ 0x2eca, 'CjkRadicalFoot', [], 'Display', None],
  [ 0x2ecb, 'CjkRadicalCsimplifiedCart', [], 'Display', None],
  [ 0x2ecc, 'CjkRadicalSimplifiedWalk', [], 'Display', None],
  [ 0x2ecd, 'CjkRadicalWalkOne', [], 'Display', None],
  [ 0x2ece, 'CjkRadicalWalkTwo', [], 'Display', None],
  [ 0x2ecf, 'CjkRadicalCity', [], 'Display', None],
  [ 0x2ed0, 'CjkRadicalCsimplifiedGold', [], 'Display', None],
  [ 0x2ed1, 'CjkRadicalLongOne', [], 'Display', None],
  [ 0x2ed2, 'CjkRadicalLongTwo', [], 'Display', None],
  [ 0x2ed3, 'CjkRadicalCsimplifiedLong', [], 'Display', None],
  [ 0x2ed4, 'CjkRadicalCsimplifiedGate', [], 'Display', None],
  [ 0x2ed5, 'CjkRadicalMoundOne', [], 'Display', None],
  [ 0x2ed6, 'CjkRadicalMoundTwo', [], 'Display', None],
  [ 0x2ed7, 'CjkRadicalRain', [], 'Display', None],
  [ 0x2ed8, 'CjkRadicalBlue', [], 'Display', None],
  [ 0x2ed9, 'CjkRadicalCsimplifiedTannedLeather', [], 'Display', None],
  [ 0x2eda, 'CjkRadicalCsimplifiedLeaf', [], 'Display', None],
  [ 0x2edb, 'CjkRadicalCsimplifiedWind', [], 'Display', None],
  [ 0x2edc, 'CjkRadicalCsimplifiedFly', [], 'Display', None],
  [ 0x2edd, 'CjkRadicalEatOne', [], 'Display', None],
  [ 0x2ede, 'CjkRadicalEatTwo', [], 'Display', None],
  [ 0x2edf, 'CjkRadicalEatThree', [], 'Display', None],
  [ 0x2ee0, 'CjkRadicalCsimplifiedEat', [], 'Display', None],
  [ 0x2ee1, 'CjkRadicalHead', [], 'Display', None],
  [ 0x2ee2, 'CjkRadicalCsimplifiedHorse', [], 'Display', None],
  [ 0x2ee3, 'CjkRadicalBone', [], 'Display', None],
  [ 0x2ee4, 'CjkRadicalGhost', [], 'Display', None],
  [ 0x2ee5, 'CjkRadicalCsimplifiedFish', [], 'Display', None],
  [ 0x2ee6, 'CjkRadicalCsimplifiedBird', [], 'Display', None],
  [ 0x2ee7, 'CjkRadicalCsimplifiedSalt', [], 'Display', None],
  [ 0x2ee8, 'CjkRadicalSimplifiedWheat', [], 'Display', None],
  [ 0x2ee9, 'CjkRadicalSimplifiedYellow', [], 'Display', None],
  [ 0x2eea, 'CjkRadicalCsimplifiedFrog', [], 'Display', None],
  [ 0x2eeb, 'CjkRadicalJsimplifiedEven', [], 'Display', None],
  [ 0x2eec, 'CjkRadicalCsimplifiedEven', [], 'Display', None],
  [ 0x2eed, 'CjkRadicalJsimplifiedTooth', [], 'Display', None],
  [ 0x2eee, 'CjkRadicalCsimplifiedTooth', [], 'Display', None],
  [ 0x2eef, 'CjkRadicalJsimplifiedDragon', [], 'Display', None],
  [ 0x2ef0, 'CjkRadicalCsimplifiedDragon', [], 'Display', None],
  [ 0x2ef1, 'CjkRadicalTurtle', [], 'Display', None],
  [ 0x2ef2, 'CjkRadicalJsimplifiedTurtle', [], 'Display', None],
  [ 0x2ef3, 'CjkRadicalCsimplifiedTurtle', [], 'Display', None],
  [ 0x2ef4, 'UnicodeCharacter0x2ef4', [], 'Display', None],
  [ 0x2ef5, 'UnicodeCharacter0x2ef5', [], 'Display', None],
  [ 0x2ef6, 'UnicodeCharacter0x2ef6', [], 'Display', None],
  [ 0x2ef7, 'UnicodeCharacter0x2ef7', [], 'Display', None],
  [ 0x2ef8, 'UnicodeCharacter0x2ef8', [], 'Display', None],
  [ 0x2ef9, 'UnicodeCharacter0x2ef9', [], 'Display', None],
  [ 0x2efa, 'UnicodeCharacter0x2efa', [], 'Display', None],
  [ 0x2efb, 'UnicodeCharacter0x2efb', [], 'Display', None],
  [ 0x2efc, 'UnicodeCharacter0x2efc', [], 'Display', None],
  [ 0x2efd, 'UnicodeCharacter0x2efd', [], 'Display', None],
  [ 0x2efe, 'UnicodeCharacter0x2efe', [], 'Display', None],
  [ 0x2eff, 'UnicodeCharacter0x2eff', [], 'Display', None],
  [ 0x2f00, 'KangxiRadicalOne', [], 'Display', None],
  [ 0x2f01, 'KangxiRadicalLine', [], 'Display', None],
  [ 0x2f02, 'KangxiRadicalDot', [], 'Display', None],
  [ 0x2f03, 'KangxiRadicalSlash', [], 'Display', None],
  [ 0x2f04, 'KangxiRadicalSecond', [], 'Display', None],
  [ 0x2f05, 'KangxiRadicalHook', [], 'Display', None],
  [ 0x2f06, 'KangxiRadicalTwo', [], 'Display', None],
  [ 0x2f07, 'KangxiRadicalLid', [], 'Display', None],
  [ 0x2f08, 'KangxiRadicalMan', [], 'Display', None],
  [ 0x2f09, 'KangxiRadicalLegs', [], 'Display', None],
  [ 0x2f0a, 'KangxiRadicalEnter', [], 'Display', None],
  [ 0x2f0b, 'KangxiRadicalEight', [], 'Display', None],
  [ 0x2f0c, 'KangxiRadicalDownBox', [], 'Display', None],
  [ 0x2f0d, 'KangxiRadicalCover', [], 'Display', None],
  [ 0x2f0e, 'KangxiRadicalIce', [], 'Display', None],
  [ 0x2f0f, 'KangxiRadicalTable', [], 'Display', None],
  [ 0x2f10, 'KangxiRadicalOpenBox', [], 'Display', None],
  [ 0x2f11, 'KangxiRadicalKnife', [], 'Display', None],
  [ 0x2f12, 'KangxiRadicalPower', [], 'Display', None],
  [ 0x2f13, 'KangxiRadicalWrap', [], 'Display', None],
  [ 0x2f14, 'KangxiRadicalSpoon', [], 'Display', None],
  [ 0x2f15, 'KangxiRadicalRightOpenBox', [], 'Display', None],
  [ 0x2f16, 'KangxiRadicalHidingEnclosure', [], 'Display', None],
  [ 0x2f17, 'KangxiRadicalTen', [], 'Display', None],
  [ 0x2f18, 'KangxiRadicalDivination', [], 'Display', None],
  [ 0x2f19, 'KangxiRadicalSeal', [], 'Display', None],
  [ 0x2f1a, 'KangxiRadicalCliff', [], 'Display', None],
  [ 0x2f1b, 'KangxiRadicalPrivate', [], 'Display', None],
  [ 0x2f1c, 'KangxiRadicalAgain', [], 'Display', None],
  [ 0x2f1d, 'KangxiRadicalMouth', [], 'Display', None],
  [ 0x2f1e, 'KangxiRadicalEnclosure', [], 'Display', None],
  [ 0x2f1f, 'KangxiRadicalEarth', [], 'Display', None],
  [ 0x2f20, 'KangxiRadicalScholar', [], 'Display', None],
  [ 0x2f21, 'KangxiRadicalGo', [], 'Display', None],
  [ 0x2f22, 'KangxiRadicalGoSlowly', [], 'Display', None],
  [ 0x2f23, 'KangxiRadicalEvening', [], 'Display', None],
  [ 0x2f24, 'KangxiRadicalBig', [], 'Display', None],
  [ 0x2f25, 'KangxiRadicalWoman', [], 'Display', None],
  [ 0x2f26, 'KangxiRadicalChild', [], 'Display', None],
  [ 0x2f27, 'KangxiRadicalRoof', [], 'Display', None],
  [ 0x2f28, 'KangxiRadicalInch', [], 'Display', None],
  [ 0x2f29, 'KangxiRadicalSmall', [], 'Display', None],
  [ 0x2f2a, 'KangxiRadicalLame', [], 'Display', None],
  [ 0x2f2b, 'KangxiRadicalCorpse', [], 'Display', None],
  [ 0x2f2c, 'KangxiRadicalSprout', [], 'Display', None],
  [ 0x2f2d, 'KangxiRadicalMountain', [], 'Display', None],
  [ 0x2f2e, 'KangxiRadicalRiver', [], 'Display', None],
  [ 0x2f2f, 'KangxiRadicalWork', [], 'Display', None],
  [ 0x2f30, 'KangxiRadicalOneself', [], 'Display', None],
  [ 0x2f31, 'KangxiRadicalTurban', [], 'Display', None],
  [ 0x2f32, 'KangxiRadicalDry', [], 'Display', None],
  [ 0x2f33, 'KangxiRadicalShortThread', [], 'Display', None],
  [ 0x2f34, 'KangxiRadicalDottedCliff', [], 'Display', None],
  [ 0x2f35, 'KangxiRadicalLongStride', [], 'Display', None],
  [ 0x2f36, 'KangxiRadicalTwoHands', [], 'Display', None],
  [ 0x2f37, 'KangxiRadicalShoot', [], 'Display', None],
  [ 0x2f38, 'KangxiRadicalBow', [], 'Display', None],
  [ 0x2f39, 'KangxiRadicalSnout', [], 'Display', None],
  [ 0x2f3a, 'KangxiRadicalBristle', [], 'Display', None],
  [ 0x2f3b, 'KangxiRadicalStep', [], 'Display', None],
  [ 0x2f3c, 'KangxiRadicalHeart', [], 'Display', None],
  [ 0x2f3d, 'KangxiRadicalHalberd', [], 'Display', None],
  [ 0x2f3e, 'KangxiRadicalDoor', [], 'Display', None],
  [ 0x2f3f, 'KangxiRadicalHand', [], 'Display', None],
  [ 0x2f40, 'KangxiRadicalBranch', [], 'Display', None],
  [ 0x2f41, 'KangxiRadicalRap', [], 'Display', None],
  [ 0x2f42, 'KangxiRadicalScript', [], 'Display', None],
  [ 0x2f43, 'KangxiRadicalDipper', [], 'Display', None],
  [ 0x2f44, 'KangxiRadicalAxe', [], 'Display', None],
  [ 0x2f45, 'KangxiRadicalSquare', [], 'Display', None],
  [ 0x2f46, 'KangxiRadicalNot', [], 'Display', None],
  [ 0x2f47, 'KangxiRadicalSun', [], 'Display', None],
  [ 0x2f48, 'KangxiRadicalSay', [], 'Display', None],
  [ 0x2f49, 'KangxiRadicalMoon', [], 'Display', None],
  [ 0x2f4a, 'KangxiRadicalTree', [], 'Display', None],
  [ 0x2f4b, 'KangxiRadicalLack', [], 'Display', None],
  [ 0x2f4c, 'KangxiRadicalStop', [], 'Display', None],
  [ 0x2f4d, 'KangxiRadicalDeath', [], 'Display', None],
  [ 0x2f4e, 'KangxiRadicalWeapon', [], 'Display', None],
  [ 0x2f4f, 'KangxiRadicalDoNot', [], 'Display', None],
  [ 0x2f50, 'KangxiRadicalCompare', [], 'Display', None],
  [ 0x2f51, 'KangxiRadicalFur', [], 'Display', None],
  [ 0x2f52, 'KangxiRadicalClan', [], 'Display', None],
  [ 0x2f53, 'KangxiRadicalSteam', [], 'Display', None],
  [ 0x2f54, 'KangxiRadicalWater', [], 'Display', None],
  [ 0x2f55, 'KangxiRadicalFire', [], 'Display', None],
  [ 0x2f56, 'KangxiRadicalClaw', [], 'Display', None],
  [ 0x2f57, 'KangxiRadicalFather', [], 'Display', None],
  [ 0x2f58, 'KangxiRadicalDoubleX', [], 'Display', None],
  [ 0x2f59, 'KangxiRadicalHalfTreeTrunk', [], 'Display', None],
  [ 0x2f5a, 'KangxiRadicalSlice', [], 'Display', None],
  [ 0x2f5b, 'KangxiRadicalFang', [], 'Display', None],
  [ 0x2f5c, 'KangxiRadicalCow', [], 'Display', None],
  [ 0x2f5d, 'KangxiRadicalDog', [], 'Display', None],
  [ 0x2f5e, 'KangxiRadicalProfound', [], 'Display', None],
  [ 0x2f5f, 'KangxiRadicalJade', [], 'Display', None],
  [ 0x2f60, 'KangxiRadicalMelon', [], 'Display', None],
  [ 0x2f61, 'KangxiRadicalTile', [], 'Display', None],
  [ 0x2f62, 'KangxiRadicalSweet', [], 'Display', None],
  [ 0x2f63, 'KangxiRadicalLife', [], 'Display', None],
  [ 0x2f64, 'KangxiRadicalUse', [], 'Display', None],
  [ 0x2f65, 'KangxiRadicalField', [], 'Display', None],
  [ 0x2f66, 'KangxiRadicalBoltOfCloth', [], 'Display', None],
  [ 0x2f67, 'KangxiRadicalSickness', [], 'Display', None],
  [ 0x2f68, 'KangxiRadicalDottedTent', [], 'Display', None],
  [ 0x2f69, 'KangxiRadicalWhite', [], 'Display', None],
  [ 0x2f6a, 'KangxiRadicalSkin', [], 'Display', None],
  [ 0x2f6b, 'KangxiRadicalDish', [], 'Display', None],
  [ 0x2f6c, 'KangxiRadicalEye', [], 'Display', None],
  [ 0x2f6d, 'KangxiRadicalSpear', [], 'Display', None],
  [ 0x2f6e, 'KangxiRadicalArrow', [], 'Display', None],
  [ 0x2f6f, 'KangxiRadicalStone', [], 'Display', None],
  [ 0x2f70, 'KangxiRadicalSpirit', [], 'Display', None],
  [ 0x2f71, 'KangxiRadicalTrack', [], 'Display', None],
  [ 0x2f72, 'KangxiRadicalGrain', [], 'Display', None],
  [ 0x2f73, 'KangxiRadicalCave', [], 'Display', None],
  [ 0x2f74, 'KangxiRadicalStand', [], 'Display', None],
  [ 0x2f75, 'KangxiRadicalBamboo', [], 'Display', None],
  [ 0x2f76, 'KangxiRadicalRice', [], 'Display', None],
  [ 0x2f77, 'KangxiRadicalSilk', [], 'Display', None],
  [ 0x2f78, 'KangxiRadicalJar', [], 'Display', None],
  [ 0x2f79, 'KangxiRadicalNet', [], 'Display', None],
  [ 0x2f7a, 'KangxiRadicalSheep', [], 'Display', None],
  [ 0x2f7b, 'KangxiRadicalFeather', [], 'Display', None],
  [ 0x2f7c, 'KangxiRadicalOld', [], 'Display', None],
  [ 0x2f7d, 'KangxiRadicalAnd', [], 'Display', None],
  [ 0x2f7e, 'KangxiRadicalPlow', [], 'Display', None],
  [ 0x2f7f, 'KangxiRadicalEar', [], 'Display', None],
  [ 0x2f80, 'KangxiRadicalBrush', [], 'Display', None],
  [ 0x2f81, 'KangxiRadicalMeat', [], 'Display', None],
  [ 0x2f82, 'KangxiRadicalMinister', [], 'Display', None],
  [ 0x2f83, 'KangxiRadicalSelf', [], 'Display', None],
  [ 0x2f84, 'KangxiRadicalArrive', [], 'Display', None],
  [ 0x2f85, 'KangxiRadicalMortar', [], 'Display', None],
  [ 0x2f86, 'KangxiRadicalTongue', [], 'Display', None],
  [ 0x2f87, 'KangxiRadicalOppose', [], 'Display', None],
  [ 0x2f88, 'KangxiRadicalBoat', [], 'Display', None],
  [ 0x2f89, 'KangxiRadicalStopping', [], 'Display', None],
  [ 0x2f8a, 'KangxiRadicalColor', [], 'Display', None],
  [ 0x2f8b, 'KangxiRadicalGrass', [], 'Display', None],
  [ 0x2f8c, 'KangxiRadicalTiger', [], 'Display', None],
  [ 0x2f8d, 'KangxiRadicalInsect', [], 'Display', None],
  [ 0x2f8e, 'KangxiRadicalBlood', [], 'Display', None],
  [ 0x2f8f, 'KangxiRadicalWalkEnclosure', [], 'Display', None],
  [ 0x2f90, 'KangxiRadicalClothes', [], 'Display', None],
  [ 0x2f91, 'KangxiRadicalWest', [], 'Display', None],
  [ 0x2f92, 'KangxiRadicalSee', [], 'Display', None],
  [ 0x2f93, 'KangxiRadicalHorn', [], 'Display', None],
  [ 0x2f94, 'KangxiRadicalSpeech', [], 'Display', None],
  [ 0x2f95, 'KangxiRadicalValley', [], 'Display', None],
  [ 0x2f96, 'KangxiRadicalBean', [], 'Display', None],
  [ 0x2f97, 'KangxiRadicalPig', [], 'Display', None],
  [ 0x2f98, 'KangxiRadicalBadger', [], 'Display', None],
  [ 0x2f99, 'KangxiRadicalShell', [], 'Display', None],
  [ 0x2f9a, 'KangxiRadicalRed', [], 'Display', None],
  [ 0x2f9b, 'KangxiRadicalRun', [], 'Display', None],
  [ 0x2f9c, 'KangxiRadicalFoot', [], 'Display', None],
  [ 0x2f9d, 'KangxiRadicalBody', [], 'Display', None],
  [ 0x2f9e, 'KangxiRadicalCart', [], 'Display', None],
  [ 0x2f9f, 'KangxiRadicalBitter', [], 'Display', None],
  [ 0x2fa0, 'KangxiRadicalMorning', [], 'Display', None],
  [ 0x2fa1, 'KangxiRadicalWalk', [], 'Display', None],
  [ 0x2fa2, 'KangxiRadicalCity', [], 'Display', None],
  [ 0x2fa3, 'KangxiRadicalWine', [], 'Display', None],
  [ 0x2fa4, 'KangxiRadicalDistinguish', [], 'Display', None],
  [ 0x2fa5, 'KangxiRadicalVillage', [], 'Display', None],
  [ 0x2fa6, 'KangxiRadicalGold', [], 'Display', None],
  [ 0x2fa7, 'KangxiRadicalLong', [], 'Display', None],
  [ 0x2fa8, 'KangxiRadicalGate', [], 'Display', None],
  [ 0x2fa9, 'KangxiRadicalMound', [], 'Display', None],
  [ 0x2faa, 'KangxiRadicalSlave', [], 'Display', None],
  [ 0x2fab, 'KangxiRadicalShortTailedBird', [], 'Display', None],
  [ 0x2fac, 'KangxiRadicalRain', [], 'Display', None],
  [ 0x2fad, 'KangxiRadicalBlue', [], 'Display', None],
  [ 0x2fae, 'KangxiRadicalWrong', [], 'Display', None],
  [ 0x2faf, 'KangxiRadicalFace', [], 'Display', None],
  [ 0x2fb0, 'KangxiRadicalLeather', [], 'Display', None],
  [ 0x2fb1, 'KangxiRadicalTannedLeather', [], 'Display', None],
  [ 0x2fb2, 'KangxiRadicalLeek', [], 'Display', None],
  [ 0x2fb3, 'KangxiRadicalSound', [], 'Display', None],
  [ 0x2fb4, 'KangxiRadicalLeaf', [], 'Display', None],
  [ 0x2fb5, 'KangxiRadicalWind', [], 'Display', None],
  [ 0x2fb6, 'KangxiRadicalFly', [], 'Display', None],
  [ 0x2fb7, 'KangxiRadicalEat', [], 'Display', None],
  [ 0x2fb8, 'KangxiRadicalHead', [], 'Display', None],
  [ 0x2fb9, 'KangxiRadicalFragrant', [], 'Display', None],
  [ 0x2fba, 'KangxiRadicalHorse', [], 'Display', None],
  [ 0x2fbb, 'KangxiRadicalBone', [], 'Display', None],
  [ 0x2fbc, 'KangxiRadicalTall', [], 'Display', None],
  [ 0x2fbd, 'KangxiRadicalHair', [], 'Display', None],
  [ 0x2fbe, 'KangxiRadicalFight', [], 'Display', None],
  [ 0x2fbf, 'KangxiRadicalSacrificialWine', [], 'Display', None],
  [ 0x2fc0, 'KangxiRadicalCauldron', [], 'Display', None],
  [ 0x2fc1, 'KangxiRadicalGhost', [], 'Display', None],
  [ 0x2fc2, 'KangxiRadicalFish', [], 'Display', None],
  [ 0x2fc3, 'KangxiRadicalBird', [], 'Display', None],
  [ 0x2fc4, 'KangxiRadicalSalt', [], 'Display', None],
  [ 0x2fc5, 'KangxiRadicalDeer', [], 'Display', None],
  [ 0x2fc6, 'KangxiRadicalWheat', [], 'Display', None],
  [ 0x2fc7, 'KangxiRadicalHemp', [], 'Display', None],
  [ 0x2fc8, 'KangxiRadicalYellow', [], 'Display', None],
  [ 0x2fc9, 'KangxiRadicalMillet', [], 'Display', None],
  [ 0x2fca, 'KangxiRadicalBlack', [], 'Display', None],
  [ 0x2fcb, 'KangxiRadicalEmbroidery', [], 'Display', None],
  [ 0x2fcc, 'KangxiRadicalFrog', [], 'Display', None],
  [ 0x2fcd, 'KangxiRadicalTripod', [], 'Display', None],
  [ 0x2fce, 'KangxiRadicalDrum', [], 'Display', None],
  [ 0x2fcf, 'KangxiRadicalRat', [], 'Display', None],
  [ 0x2fd0, 'KangxiRadicalNose', [], 'Display', None],
  [ 0x2fd1, 'KangxiRadicalEven', [], 'Display', None],
  [ 0x2fd2, 'KangxiRadicalTooth', [], 'Display', None],
  [ 0x2fd3, 'KangxiRadicalDragon', [], 'Display', None],
  [ 0x2fd4, 'KangxiRadicalTurtle', [], 'Display', None],
  [ 0x2fd5, 'KangxiRadicalFlute', [], 'Display', None],
  [ 0x2fd6, 'UnicodeCharacter0x2fd6', [], 'Display', None],
  [ 0x2fd7, 'UnicodeCharacter0x2fd7', [], 'Display', None],
  [ 0x2fd8, 'UnicodeCharacter0x2fd8', [], 'Display', None],
  [ 0x2fd9, 'UnicodeCharacter0x2fd9', [], 'Display', None],
  [ 0x2fda, 'UnicodeCharacter0x2fda', [], 'Display', None],
  [ 0x2fdb, 'UnicodeCharacter0x2fdb', [], 'Display', None],
  [ 0x2fdc, 'UnicodeCharacter0x2fdc', [], 'Display', None],
  [ 0x2fdd, 'UnicodeCharacter0x2fdd', [], 'Display', None],
  [ 0x2fde, 'UnicodeCharacter0x2fde', [], 'Display', None],
  [ 0x2fdf, 'UnicodeCharacter0x2fdf', [], 'Display', None],
  [ 0x2fe0, 'UnicodeCharacter0x2fe0', [], 'Display', None],
  [ 0x2fe1, 'UnicodeCharacter0x2fe1', [], 'Display', None],
  [ 0x2fe2, 'UnicodeCharacter0x2fe2', [], 'Display', None],
  [ 0x2fe3, 'UnicodeCharacter0x2fe3', [], 'Display', None],
  [ 0x2fe4, 'UnicodeCharacter0x2fe4', [], 'Display', None],
  [ 0x2fe5, 'UnicodeCharacter0x2fe5', [], 'Display', None],
  [ 0x2fe6, 'UnicodeCharacter0x2fe6', [], 'Display', None],
  [ 0x2fe7, 'UnicodeCharacter0x2fe7', [], 'Display', None],
  [ 0x2fe8, 'UnicodeCharacter0x2fe8', [], 'Display', None],
  [ 0x2fe9, 'UnicodeCharacter0x2fe9', [], 'Display', None],
  [ 0x2fea, 'UnicodeCharacter0x2fea', [], 'Display', None],
  [ 0x2feb, 'UnicodeCharacter0x2feb', [], 'Display', None],
  [ 0x2fec, 'UnicodeCharacter0x2fec', [], 'Display', None],
  [ 0x2fed, 'UnicodeCharacter0x2fed', [], 'Display', None],
  [ 0x2fee, 'UnicodeCharacter0x2fee', [], 'Display', None],
  [ 0x2fef, 'UnicodeCharacter0x2fef', [], 'Display', None],
  [ 0x2ff0, 'IdeographicDescriptionCharacterLeftToRight', [], 'Display', None],
  [ 0x2ff1, 'IdeographicDescriptionCharacterAboveToBelow', [], 'Display', None],
  [ 0x2ff2, 'IdeographicDescriptionCharacterLeftToMiddleAndRight', [], 'Display', None],
  [ 0x2ff3, 'IdeographicDescriptionCharacterAboveToMiddleAndBelow', [], 'Display', None],
  [ 0x2ff4, 'IdeographicDescriptionCharacterFullSurround', [], 'Display', None],
  [ 0x2ff5, 'IdeographicDescriptionCharacterSurroundFromAbove', [], 'Display', None],
  [ 0x2ff6, 'IdeographicDescriptionCharacterSurroundFromBelow', [], 'Display', None],
  [ 0x2ff7, 'IdeographicDescriptionCharacterSurroundFromLeft', [], 'Display', None],
  [ 0x2ff8, 'IdeographicDescriptionCharacterSurroundFromUpperLeft', [], 'Display', None],
  [ 0x2ff9, 'IdeographicDescriptionCharacterSurroundFromUpperRight', [], 'Display', None],
  [ 0x2ffa, 'IdeographicDescriptionCharacterSurroundFromLowerLeft', [], 'Display', None],
  [ 0x2ffb, 'IdeographicDescriptionCharacterOverlaid', [], 'Display', None],
  [ 0x2ffc, 'UnicodeCharacter0x2ffc', [], 'Display', None],
  [ 0x2ffd, 'UnicodeCharacter0x2ffd', [], 'Display', None],
  [ 0x2ffe, 'UnicodeCharacter0x2ffe', [], 'Display', None],
  [ 0x2fff, 'UnicodeCharacter0x2fff', [], 'Display', None],
  [ 0x3000, 'IdeographicSpace', [], 'Display', None],
  [ 0x3001, 'IdeographicComma', [], 'Display', None],
  [ 0x3002, 'IdeographicFullStop', [], 'Display', None],
  [ 0x3003, 'DittoMark', [], 'Display', None],
  [ 0x3004, 'JapaneseIndustrialStandardSymbol', [], 'Display', None],
  [ 0x3005, 'IdeographicIterationMark', [], 'Display', None],
  [ 0x3006, 'IdeographicClosingMark', [], 'Display', None],
  [ 0x3007, 'IdeographicNumberZero', [], 'Display', None],
  [ 0x3008, 'LeftAngleBracket', [], 'Display', None],
  [ 0x3009, 'RightAngleBracket', [], 'Display', None],
  [ 0x300a, 'LeftDoubleAngleBracket', [], 'Display', None],
  [ 0x300b, 'RightDoubleAngleBracket', [], 'Display', None],
  [ 0x300c, 'LeftCornerBracket', [], 'Display', None],
  [ 0x300d, 'RightCornerBracket', [], 'Display', None],
  [ 0x300e, 'LeftWhiteCornerBracket', [], 'Display', None],
  [ 0x300f, 'RightWhiteCornerBracket', [], 'Display', None],
  [ 0x3010, 'LeftBlackLenticularBracket', [], 'Display', None],
  [ 0x3011, 'RightBlackLenticularBracket', [], 'Display', None],
  [ 0x3012, 'PostalMark', [], 'Display', None],
  [ 0x3013, 'GetaMark', [], 'Display', None],
  [ 0x3014, 'LeftTortoiseShellBracket', [], 'Display', None],
  [ 0x3015, 'RightTortoiseShellBracket', [], 'Display', None],
  [ 0x3016, 'LeftWhiteLenticularBracket', [], 'Display', None],
  [ 0x3017, 'RightWhiteLenticularBracket', [], 'Display', None],
  [ 0x3018, 'LeftWhiteTortoiseShellBracket', [], 'Display', None],
  [ 0x3019, 'RightWhiteTortoiseShellBracket', [], 'Display', None],
  [ 0x301a, 'LeftWhiteSquareBracket', [], 'Display', None],
  [ 0x301b, 'RightWhiteSquareBracket', [], 'Display', None],
  [ 0x301c, 'WaveDash', [], 'Display', None],
  [ 0x301d, 'ReversedDoublePrimeQuotationMark', [], 'Display', None],
  [ 0x301e, 'DoublePrimeQuotationMark', [], 'Display', None],
  [ 0x301f, 'LowDoublePrimeQuotationMark', [], 'Display', None],
  [ 0x3020, 'PostalMarkFace', [], 'Display', None],
  [ 0x3021, 'HangzhouNumeralOne', [], 'Display', None],
  [ 0x3022, 'HangzhouNumeralTwo', [], 'Display', None],
  [ 0x3023, 'HangzhouNumeralThree', [], 'Display', None],
  [ 0x3024, 'HangzhouNumeralFour', [], 'Display', None],
  [ 0x3025, 'HangzhouNumeralFive', [], 'Display', None],
  [ 0x3026, 'HangzhouNumeralSix', [], 'Display', None],
  [ 0x3027, 'HangzhouNumeralSeven', [], 'Display', None],
  [ 0x3028, 'HangzhouNumeralEight', [], 'Display', None],
  [ 0x3029, 'HangzhouNumeralNine', [], 'Display', None],
  [ 0x302a, 'IdeographicLevelToneMark', [], 'Display', None],
  [ 0x302b, 'IdeographicRisingToneMark', [], 'Display', None],
  [ 0x302c, 'IdeographicDepartingToneMark', [], 'Display', None],
  [ 0x302d, 'IdeographicEnteringToneMark', [], 'Display', None],
  [ 0x302e, 'HangulSingleDotToneMark', [], 'Display', None],
  [ 0x302f, 'HangulDoubleDotToneMark', [], 'Display', None],
  [ 0x3030, 'WavyDash', [], 'Display', None],
  [ 0x3031, 'VerticalKanaRepeatMark', [], 'Display', None],
  [ 0x3032, 'VerticalKanaRepeatWithVoicedSoundMark', [], 'Display', None],
  [ 0x3033, 'VerticalKanaRepeatMarkUpperHalf', [], 'Display', None],
  [ 0x3034, 'VerticalKanaRepeatWithVoicedSoundMarkUpperHalf', [], 'Display', None],
  [ 0x3035, 'VerticalKanaRepeatMarkLowerHalf', [], 'Display', None],
  [ 0x3036, 'CircledPostalMark', [], 'Display', None],
  [ 0x3037, 'IdeographicTelegraphLineFeedSeparatorSymbol', [], 'Display', None],
  [ 0x3038, 'HangzhouNumeralTen', [], 'Display', None],
  [ 0x3039, 'HangzhouNumeralTwenty', [], 'Display', None],
  [ 0x303a, 'HangzhouNumeralThirty', [], 'Display', None],
  [ 0x303b, 'VerticalIdeographicIterationMark', [], 'Display', None],
  [ 0x303c, 'MasuMark', [], 'Display', None],
  [ 0x303d, 'PartAlternationMark', [], 'Display', None],
  [ 0x303e, 'IdeographicVariationIndicator', [], 'Display', None],
  [ 0x303f, 'IdeographicHalfFillSpace', [], 'Display', None],
  [ 0x3040, 'UnicodeCharacter0x3040', [], 'Display', None],
  [ 0x3041, 'HiraganaLetterSmallA', [], 'Display', None],
  [ 0x3042, 'HiraganaLetterA', [], 'Display', None],
  [ 0x3043, 'HiraganaLetterSmallI', [], 'Display', None],
  [ 0x3044, 'HiraganaLetterI', [], 'Display', None],
  [ 0x3045, 'HiraganaLetterSmallU', [], 'Display', None],
  [ 0x3046, 'HiraganaLetterU', [], 'Display', None],
  [ 0x3047, 'HiraganaLetterSmallE', [], 'Display', None],
  [ 0x3048, 'HiraganaLetterE', [], 'Display', None],
  [ 0x3049, 'HiraganaLetterSmallO', [], 'Display', None],
  [ 0x304a, 'HiraganaLetterO', [], 'Display', None],
  [ 0x304b, 'HiraganaLetterKa', [], 'Display', None],
  [ 0x304c, 'HiraganaLetterGa', [], 'Display', None],
  [ 0x304d, 'HiraganaLetterKi', [], 'Display', None],
  [ 0x304e, 'HiraganaLetterGi', [], 'Display', None],
  [ 0x304f, 'HiraganaLetterKu', [], 'Display', None],
  [ 0x3050, 'HiraganaLetterGu', [], 'Display', None],
  [ 0x3051, 'HiraganaLetterKe', [], 'Display', None],
  [ 0x3052, 'HiraganaLetterGe', [], 'Display', None],
  [ 0x3053, 'HiraganaLetterKo', [], 'Display', None],
  [ 0x3054, 'HiraganaLetterGo', [], 'Display', None],
  [ 0x3055, 'HiraganaLetterSa', [], 'Display', None],
  [ 0x3056, 'HiraganaLetterZa', [], 'Display', None],
  [ 0x3057, 'HiraganaLetterSi', [], 'Display', None],
  [ 0x3058, 'HiraganaLetterZi', [], 'Display', None],
  [ 0x3059, 'HiraganaLetterSu', [], 'Display', None],
  [ 0x305a, 'HiraganaLetterZu', [], 'Display', None],
  [ 0x305b, 'HiraganaLetterSe', [], 'Display', None],
  [ 0x305c, 'HiraganaLetterZe', [], 'Display', None],
  [ 0x305d, 'HiraganaLetterSo', [], 'Display', None],
  [ 0x305e, 'HiraganaLetterZo', [], 'Display', None],
  [ 0x305f, 'HiraganaLetterTa', [], 'Display', None],
  [ 0x3060, 'HiraganaLetterDa', [], 'Display', None],
  [ 0x3061, 'HiraganaLetterTi', [], 'Display', None],
  [ 0x3062, 'HiraganaLetterDi', [], 'Display', None],
  [ 0x3063, 'HiraganaLetterSmallTu', [], 'Display', None],
  [ 0x3064, 'HiraganaLetterTu', [], 'Display', None],
  [ 0x3065, 'HiraganaLetterDu', [], 'Display', None],
  [ 0x3066, 'HiraganaLetterTe', [], 'Display', None],
  [ 0x3067, 'HiraganaLetterDe', [], 'Display', None],
  [ 0x3068, 'HiraganaLetterTo', [], 'Display', None],
  [ 0x3069, 'HiraganaLetterDo', [], 'Display', None],
  [ 0x306a, 'HiraganaLetterNa', [], 'Display', None],
  [ 0x306b, 'HiraganaLetterNi', [], 'Display', None],
  [ 0x306c, 'HiraganaLetterNu', [], 'Display', None],
  [ 0x306d, 'HiraganaLetterNe', [], 'Display', None],
  [ 0x306e, 'HiraganaLetterNo', [], 'Display', None],
  [ 0x306f, 'HiraganaLetterHa', [], 'Display', None],
  [ 0x3070, 'HiraganaLetterBa', [], 'Display', None],
  [ 0x3071, 'HiraganaLetterPa', [], 'Display', None],
  [ 0x3072, 'HiraganaLetterHi', [], 'Display', None],
  [ 0x3073, 'HiraganaLetterBi', [], 'Display', None],
  [ 0x3074, 'HiraganaLetterPi', [], 'Display', None],
  [ 0x3075, 'HiraganaLetterHu', [], 'Display', None],
  [ 0x3076, 'HiraganaLetterBu', [], 'Display', None],
  [ 0x3077, 'HiraganaLetterPu', [], 'Display', None],
  [ 0x3078, 'HiraganaLetterHe', [], 'Display', None],
  [ 0x3079, 'HiraganaLetterBe', [], 'Display', None],
  [ 0x307a, 'HiraganaLetterPe', [], 'Display', None],
  [ 0x307b, 'HiraganaLetterHo', [], 'Display', None],
  [ 0x307c, 'HiraganaLetterBo', [], 'Display', None],
  [ 0x307d, 'HiraganaLetterPo', [], 'Display', None],
  [ 0x307e, 'HiraganaLetterMa', [], 'Display', None],
  [ 0x307f, 'HiraganaLetterMi', [], 'Display', None],
  [ 0x3080, 'HiraganaLetterMu', [], 'Display', None],
  [ 0x3081, 'HiraganaLetterMe', [], 'Display', None],
  [ 0x3082, 'HiraganaLetterMo', [], 'Display', None],
  [ 0x3083, 'HiraganaLetterSmallYa', [], 'Display', None],
  [ 0x3084, 'HiraganaLetterYa', [], 'Display', None],
  [ 0x3085, 'HiraganaLetterSmallYu', [], 'Display', None],
  [ 0x3086, 'HiraganaLetterYu', [], 'Display', None],
  [ 0x3087, 'HiraganaLetterSmallYo', [], 'Display', None],
  [ 0x3088, 'HiraganaLetterYo', [], 'Display', None],
  [ 0x3089, 'HiraganaLetterRa', [], 'Display', None],
  [ 0x308a, 'HiraganaLetterRi', [], 'Display', None],
  [ 0x308b, 'HiraganaLetterRu', [], 'Display', None],
  [ 0x308c, 'HiraganaLetterRe', [], 'Display', None],
  [ 0x308d, 'HiraganaLetterRo', [], 'Display', None],
  [ 0x308e, 'HiraganaLetterSmallWa', [], 'Display', None],
  [ 0x308f, 'HiraganaLetterWa', [], 'Display', None],
  [ 0x3090, 'HiraganaLetterWi', [], 'Display', None],
  [ 0x3091, 'HiraganaLetterWe', [], 'Display', None],
  [ 0x3092, 'HiraganaLetterWo', [], 'Display', None],
  [ 0x3093, 'HiraganaLetterN', [], 'Display', None],
  [ 0x3094, 'HiraganaLetterVu', [], 'Display', None],
  [ 0x3095, 'HiraganaLetterSmallKa', [], 'Display', None],
  [ 0x3096, 'HiraganaLetterSmallKe', [], 'Display', None],
  [ 0x3097, 'UnicodeCharacter0x3097', [], 'Display', None],
  [ 0x3098, 'UnicodeCharacter0x3098', [], 'Display', None],
  [ 0x3099, 'CombiningKatakanahiraganaVoicedSoundMark', [], 'Display', None],
  [ 0x309a, 'CombiningKatakanahiraganaSemivoicedSoundMark', [], 'Display', None],
  [ 0x309b, 'KatakanahiraganaVoicedSoundMark', [], 'Display', None],
  [ 0x309c, 'KatakanahiraganaSemivoicedSoundMark', [], 'Display', None],
  [ 0x309d, 'HiraganaIterationMark', [], 'Display', None],
  [ 0x309e, 'HiraganaVoicedIterationMark', [], 'Display', None],
  [ 0x309f, 'HiraganaDigraphYori', [], 'Display', None],
  [ 0x30a0, 'KatakanahiraganaDoubleHyphen', [], 'Display', None],
  [ 0x30a1, 'KatakanaLetterSmallA', [], 'Display', None],
  [ 0x30a2, 'KatakanaLetterA', [], 'Display', None],
  [ 0x30a3, 'KatakanaLetterSmallI', [], 'Display', None],
  [ 0x30a4, 'KatakanaLetterI', [], 'Display', None],
  [ 0x30a5, 'KatakanaLetterSmallU', [], 'Display', None],
  [ 0x30a6, 'KatakanaLetterU', [], 'Display', None],
  [ 0x30a7, 'KatakanaLetterSmallE', [], 'Display', None],
  [ 0x30a8, 'KatakanaLetterE', [], 'Display', None],
  [ 0x30a9, 'KatakanaLetterSmallO', [], 'Display', None],
  [ 0x30aa, 'KatakanaLetterO', [], 'Display', None],
  [ 0x30ab, 'KatakanaLetterKa', [], 'Display', None],
  [ 0x30ac, 'KatakanaLetterGa', [], 'Display', None],
  [ 0x30ad, 'KatakanaLetterKi', [], 'Display', None],
  [ 0x30ae, 'KatakanaLetterGi', [], 'Display', None],
  [ 0x30af, 'KatakanaLetterKu', [], 'Display', None],
  [ 0x30b0, 'KatakanaLetterGu', [], 'Display', None],
  [ 0x30b1, 'KatakanaLetterKe', [], 'Display', None],
  [ 0x30b2, 'KatakanaLetterGe', [], 'Display', None],
  [ 0x30b3, 'KatakanaLetterKo', [], 'Display', None],
  [ 0x30b4, 'KatakanaLetterGo', [], 'Display', None],
  [ 0x30b5, 'KatakanaLetterSa', [], 'Display', None],
  [ 0x30b6, 'KatakanaLetterZa', [], 'Display', None],
  [ 0x30b7, 'KatakanaLetterSi', [], 'Display', None],
  [ 0x30b8, 'KatakanaLetterZi', [], 'Display', None],
  [ 0x30b9, 'KatakanaLetterSu', [], 'Display', None],
  [ 0x30ba, 'KatakanaLetterZu', [], 'Display', None],
  [ 0x30bb, 'KatakanaLetterSe', [], 'Display', None],
  [ 0x30bc, 'KatakanaLetterZe', [], 'Display', None],
  [ 0x30bd, 'KatakanaLetterSo', [], 'Display', None],
  [ 0x30be, 'KatakanaLetterZo', [], 'Display', None],
  [ 0x30bf, 'KatakanaLetterTa', [], 'Display', None],
  [ 0x30c0, 'KatakanaLetterDa', [], 'Display', None],
  [ 0x30c1, 'KatakanaLetterTi', [], 'Display', None],
  [ 0x30c2, 'KatakanaLetterDi', [], 'Display', None],
  [ 0x30c3, 'KatakanaLetterSmallTu', [], 'Display', None],
  [ 0x30c4, 'KatakanaLetterTu', [], 'Display', None],
  [ 0x30c5, 'KatakanaLetterDu', [], 'Display', None],
  [ 0x30c6, 'KatakanaLetterTe', [], 'Display', None],
  [ 0x30c7, 'KatakanaLetterDe', [], 'Display', None],
  [ 0x30c8, 'KatakanaLetterTo', [], 'Display', None],
  [ 0x30c9, 'KatakanaLetterDo', [], 'Display', None],
  [ 0x30ca, 'KatakanaLetterNa', [], 'Display', None],
  [ 0x30cb, 'KatakanaLetterNi', [], 'Display', None],
  [ 0x30cc, 'KatakanaLetterNu', [], 'Display', None],
  [ 0x30cd, 'KatakanaLetterNe', [], 'Display', None],
  [ 0x30ce, 'KatakanaLetterNo', [], 'Display', None],
  [ 0x30cf, 'KatakanaLetterHa', [], 'Display', None],
  [ 0x30d0, 'KatakanaLetterBa', [], 'Display', None],
  [ 0x30d1, 'KatakanaLetterPa', [], 'Display', None],
  [ 0x30d2, 'KatakanaLetterHi', [], 'Display', None],
  [ 0x30d3, 'KatakanaLetterBi', [], 'Display', None],
  [ 0x30d4, 'KatakanaLetterPi', [], 'Display', None],
  [ 0x30d5, 'KatakanaLetterHu', [], 'Display', None],
  [ 0x30d6, 'KatakanaLetterBu', [], 'Display', None],
  [ 0x30d7, 'KatakanaLetterPu', [], 'Display', None],
  [ 0x30d8, 'KatakanaLetterHe', [], 'Display', None],
  [ 0x30d9, 'KatakanaLetterBe', [], 'Display', None],
  [ 0x30da, 'KatakanaLetterPe', [], 'Display', None],
  [ 0x30db, 'KatakanaLetterHo', [], 'Display', None],
  [ 0x30dc, 'KatakanaLetterBo', [], 'Display', None],
  [ 0x30dd, 'KatakanaLetterPo', [], 'Display', None],
  [ 0x30de, 'KatakanaLetterMa', [], 'Display', None],
  [ 0x30df, 'KatakanaLetterMi', [], 'Display', None],
  [ 0x30e0, 'KatakanaLetterMu', [], 'Display', None],
  [ 0x30e1, 'KatakanaLetterMe', [], 'Display', None],
  [ 0x30e2, 'KatakanaLetterMo', [], 'Display', None],
  [ 0x30e3, 'KatakanaLetterSmallYa', [], 'Display', None],
  [ 0x30e4, 'KatakanaLetterYa', [], 'Display', None],
  [ 0x30e5, 'KatakanaLetterSmallYu', [], 'Display', None],
  [ 0x30e6, 'KatakanaLetterYu', [], 'Display', None],
  [ 0x30e7, 'KatakanaLetterSmallYo', [], 'Display', None],
  [ 0x30e8, 'KatakanaLetterYo', [], 'Display', None],
  [ 0x30e9, 'KatakanaLetterRa', [], 'Display', None],
  [ 0x30ea, 'KatakanaLetterRi', [], 'Display', None],
  [ 0x30eb, 'KatakanaLetterRu', [], 'Display', None],
  [ 0x30ec, 'KatakanaLetterRe', [], 'Display', None],
  [ 0x30ed, 'KatakanaLetterRo', [], 'Display', None],
  [ 0x30ee, 'KatakanaLetterSmallWa', [], 'Display', None],
  [ 0x30ef, 'KatakanaLetterWa', [], 'Display', None],
  [ 0x30f0, 'KatakanaLetterWi', [], 'Display', None],
  [ 0x30f1, 'KatakanaLetterWe', [], 'Display', None],
  [ 0x30f2, 'KatakanaLetterWo', [], 'Display', None],
  [ 0x30f3, 'KatakanaLetterN', [], 'Display', None],
  [ 0x30f4, 'KatakanaLetterVu', [], 'Display', None],
  [ 0x30f5, 'KatakanaLetterSmallKa', [], 'Display', None],
  [ 0x30f6, 'KatakanaLetterSmallKe', [], 'Display', None],
  [ 0x30f7, 'KatakanaLetterVa', [], 'Display', None],
  [ 0x30f8, 'KatakanaLetterVi', [], 'Display', None],
  [ 0x30f9, 'KatakanaLetterVe', [], 'Display', None],
  [ 0x30fa, 'KatakanaLetterVo', [], 'Display', None],
  [ 0x30fb, 'KatakanaMiddleDot', [], 'Display', None],
  [ 0x30fc, 'KatakanahiraganaProlongedSoundMark', [], 'Display', None],
  [ 0x30fd, 'KatakanaIterationMark', [], 'Display', None],
  [ 0x30fe, 'KatakanaVoicedIterationMark', [], 'Display', None],
  [ 0x30ff, 'KatakanaDigraphKoto', [], 'Display', None],
  [ 0x3100, 'UnicodeCharacter0x3100', [], 'Display', None],
  [ 0x3101, 'UnicodeCharacter0x3101', [], 'Display', None],
  [ 0x3102, 'UnicodeCharacter0x3102', [], 'Display', None],
  [ 0x3103, 'UnicodeCharacter0x3103', [], 'Display', None],
  [ 0x3104, 'UnicodeCharacter0x3104', [], 'Display', None],
  [ 0x3105, 'BopomofoLetterB', [], 'Display', None],
  [ 0x3106, 'BopomofoLetterP', [], 'Display', None],
  [ 0x3107, 'BopomofoLetterM', [], 'Display', None],
  [ 0x3108, 'BopomofoLetterF', [], 'Display', None],
  [ 0x3109, 'BopomofoLetterD', [], 'Display', None],
  [ 0x310a, 'BopomofoLetterT', [], 'Display', None],
  [ 0x310b, 'BopomofoLetterN', [], 'Display', None],
  [ 0x310c, 'BopomofoLetterL', [], 'Display', None],
  [ 0x310d, 'BopomofoLetterG', [], 'Display', None],
  [ 0x310e, 'BopomofoLetterK', [], 'Display', None],
  [ 0x310f, 'BopomofoLetterH', [], 'Display', None],
  [ 0x3110, 'BopomofoLetterJ', [], 'Display', None],
  [ 0x3111, 'BopomofoLetterQ', [], 'Display', None],
  [ 0x3112, 'BopomofoLetterX', [], 'Display', None],
  [ 0x3113, 'BopomofoLetterZh', [], 'Display', None],
  [ 0x3114, 'BopomofoLetterCh', [], 'Display', None],
  [ 0x3115, 'BopomofoLetterSh', [], 'Display', None],
  [ 0x3116, 'BopomofoLetterR', [], 'Display', None],
  [ 0x3117, 'BopomofoLetterZ', [], 'Display', None],
  [ 0x3118, 'BopomofoLetterC', [], 'Display', None],
  [ 0x3119, 'BopomofoLetterS', [], 'Display', None],
  [ 0x311a, 'BopomofoLetterA', [], 'Display', None],
  [ 0x311b, 'BopomofoLetterO', [], 'Display', None],
  [ 0x311c, 'BopomofoLetterE', [], 'Display', None],
  [ 0x311d, 'BopomofoLetterEh', [], 'Display', None],
  [ 0x311e, 'BopomofoLetterAi', [], 'Display', None],
  [ 0x311f, 'BopomofoLetterEi', [], 'Display', None],
  [ 0x3120, 'BopomofoLetterAu', [], 'Display', None],
  [ 0x3121, 'BopomofoLetterOu', [], 'Display', None],
  [ 0x3122, 'BopomofoLetterAn', [], 'Display', None],
  [ 0x3123, 'BopomofoLetterEn', [], 'Display', None],
  [ 0x3124, 'BopomofoLetterAng', [], 'Display', None],
  [ 0x3125, 'BopomofoLetterEng', [], 'Display', None],
  [ 0x3126, 'BopomofoLetterEr', [], 'Display', None],
  [ 0x3127, 'BopomofoLetterI', [], 'Display', None],
  [ 0x3128, 'BopomofoLetterU', [], 'Display', None],
  [ 0x3129, 'BopomofoLetterIu', [], 'Display', None],
  [ 0x312a, 'BopomofoLetterV', [], 'Display', None],
  [ 0x312b, 'BopomofoLetterNg', [], 'Display', None],
  [ 0x312c, 'BopomofoLetterGn', [], 'Display', None],
  [ 0x312d, 'BopomofoLetterIh', [], 'Display', None],
  [ 0x312e, 'UnicodeCharacter0x312e', [], 'Display', None],
  [ 0x312f, 'UnicodeCharacter0x312f', [], 'Display', None],
  [ 0x3130, 'UnicodeCharacter0x3130', [], 'Display', None],
  [ 0x3131, 'HangulLetterKiyeok', [], 'Display', None],
  [ 0x3132, 'HangulLetterSsangkiyeok', [], 'Display', None],
  [ 0x3133, 'HangulLetterKiyeoksios', [], 'Display', None],
  [ 0x3134, 'HangulLetterNieun', [], 'Display', None],
  [ 0x3135, 'HangulLetterNieuncieuc', [], 'Display', None],
  [ 0x3136, 'HangulLetterNieunhieuh', [], 'Display', None],
  [ 0x3137, 'HangulLetterTikeut', [], 'Display', None],
  [ 0x3138, 'HangulLetterSsangtikeut', [], 'Display', None],
  [ 0x3139, 'HangulLetterRieul', [], 'Display', None],
  [ 0x313a, 'HangulLetterRieulkiyeok', [], 'Display', None],
  [ 0x313b, 'HangulLetterRieulmieum', [], 'Display', None],
  [ 0x313c, 'HangulLetterRieulpieup', [], 'Display', None],
  [ 0x313d, 'HangulLetterRieulsios', [], 'Display', None],
  [ 0x313e, 'HangulLetterRieulthieuth', [], 'Display', None],
  [ 0x313f, 'HangulLetterRieulphieuph', [], 'Display', None],
  [ 0x3140, 'HangulLetterRieulhieuh', [], 'Display', None],
  [ 0x3141, 'HangulLetterMieum', [], 'Display', None],
  [ 0x3142, 'HangulLetterPieup', [], 'Display', None],
  [ 0x3143, 'HangulLetterSsangpieup', [], 'Display', None],
  [ 0x3144, 'HangulLetterPieupsios', [], 'Display', None],
  [ 0x3145, 'HangulLetterSios', [], 'Display', None],
  [ 0x3146, 'HangulLetterSsangsios', [], 'Display', None],
  [ 0x3147, 'HangulLetterIeung', [], 'Display', None],
  [ 0x3148, 'HangulLetterCieuc', [], 'Display', None],
  [ 0x3149, 'HangulLetterSsangcieuc', [], 'Display', None],
  [ 0x314a, 'HangulLetterChieuch', [], 'Display', None],
  [ 0x314b, 'HangulLetterKhieukh', [], 'Display', None],
  [ 0x314c, 'HangulLetterThieuth', [], 'Display', None],
  [ 0x314d, 'HangulLetterPhieuph', [], 'Display', None],
  [ 0x314e, 'HangulLetterHieuh', [], 'Display', None],
  [ 0x314f, 'HangulLetterA', [], 'Display', None],
  [ 0x3150, 'HangulLetterAe', [], 'Display', None],
  [ 0x3151, 'HangulLetterYa', [], 'Display', None],
  [ 0x3152, 'HangulLetterYae', [], 'Display', None],
  [ 0x3153, 'HangulLetterEo', [], 'Display', None],
  [ 0x3154, 'HangulLetterE', [], 'Display', None],
  [ 0x3155, 'HangulLetterYeo', [], 'Display', None],
  [ 0x3156, 'HangulLetterYe', [], 'Display', None],
  [ 0x3157, 'HangulLetterO', [], 'Display', None],
  [ 0x3158, 'HangulLetterWa', [], 'Display', None],
  [ 0x3159, 'HangulLetterWae', [], 'Display', None],
  [ 0x315a, 'HangulLetterOe', [], 'Display', None],
  [ 0x315b, 'HangulLetterYo', [], 'Display', None],
  [ 0x315c, 'HangulLetterU', [], 'Display', None],
  [ 0x315d, 'HangulLetterWeo', [], 'Display', None],
  [ 0x315e, 'HangulLetterWe', [], 'Display', None],
  [ 0x315f, 'HangulLetterWi', [], 'Display', None],
  [ 0x3160, 'HangulLetterYu', [], 'Display', None],
  [ 0x3161, 'HangulLetterEu', [], 'Display', None],
  [ 0x3162, 'HangulLetterYi', [], 'Display', None],
  [ 0x3163, 'HangulLetterI', [], 'Display', None],
  [ 0x3164, 'HangulFiller', [], 'Display', None],
  [ 0x3165, 'HangulLetterSsangnieun', [], 'Display', None],
  [ 0x3166, 'HangulLetterNieuntikeut', [], 'Display', None],
  [ 0x3167, 'HangulLetterNieunsios', [], 'Display', None],
  [ 0x3168, 'HangulLetterNieunpansios', [], 'Display', None],
  [ 0x3169, 'HangulLetterRieulkiyeoksios', [], 'Display', None],
  [ 0x316a, 'HangulLetterRieultikeut', [], 'Display', None],
  [ 0x316b, 'HangulLetterRieulpieupsios', [], 'Display', None],
  [ 0x316c, 'HangulLetterRieulpansios', [], 'Display', None],
  [ 0x316d, 'HangulLetterRieulyeorinhieuh', [], 'Display', None],
  [ 0x316e, 'HangulLetterMieumpieup', [], 'Display', None],
  [ 0x316f, 'HangulLetterMieumsios', [], 'Display', None],
  [ 0x3170, 'HangulLetterMieumpansios', [], 'Display', None],
  [ 0x3171, 'HangulLetterKapyeounmieum', [], 'Display', None],
  [ 0x3172, 'HangulLetterPieupkiyeok', [], 'Display', None],
  [ 0x3173, 'HangulLetterPieuptikeut', [], 'Display', None],
  [ 0x3174, 'HangulLetterPieupsioskiyeok', [], 'Display', None],
  [ 0x3175, 'HangulLetterPieupsiostikeut', [], 'Display', None],
  [ 0x3176, 'HangulLetterPieupcieuc', [], 'Display', None],
  [ 0x3177, 'HangulLetterPieupthieuth', [], 'Display', None],
  [ 0x3178, 'HangulLetterKapyeounpieup', [], 'Display', None],
  [ 0x3179, 'HangulLetterKapyeounssangpieup', [], 'Display', None],
  [ 0x317a, 'HangulLetterSioskiyeok', [], 'Display', None],
  [ 0x317b, 'HangulLetterSiosnieun', [], 'Display', None],
  [ 0x317c, 'HangulLetterSiostikeut', [], 'Display', None],
  [ 0x317d, 'HangulLetterSiospieup', [], 'Display', None],
  [ 0x317e, 'HangulLetterSioscieuc', [], 'Display', None],
  [ 0x317f, 'HangulLetterPansios', [], 'Display', None],
  [ 0x3180, 'HangulLetterSsangieung', [], 'Display', None],
  [ 0x3181, 'HangulLetterYesieung', [], 'Display', None],
  [ 0x3182, 'HangulLetterYesieungsios', [], 'Display', None],
  [ 0x3183, 'HangulLetterYesieungpansios', [], 'Display', None],
  [ 0x3184, 'HangulLetterKapyeounphieuph', [], 'Display', None],
  [ 0x3185, 'HangulLetterSsanghieuh', [], 'Display', None],
  [ 0x3186, 'HangulLetterYeorinhieuh', [], 'Display', None],
  [ 0x3187, 'HangulLetterYoya', [], 'Display', None],
  [ 0x3188, 'HangulLetterYoyae', [], 'Display', None],
  [ 0x3189, 'HangulLetterYoi', [], 'Display', None],
  [ 0x318a, 'HangulLetterYuyeo', [], 'Display', None],
  [ 0x318b, 'HangulLetterYuye', [], 'Display', None],
  [ 0x318c, 'HangulLetterYui', [], 'Display', None],
  [ 0x318d, 'HangulLetterAraea', [], 'Display', None],
  [ 0x318e, 'HangulLetterAraeae', [], 'Display', None],
  [ 0x318f, 'UnicodeCharacter0x318f', [], 'Display', None],
  [ 0x3190, 'IdeographicAnnotationLinkingMark', [], 'Display', None],
  [ 0x3191, 'IdeographicAnnotationReverseMark', [], 'Display', None],
  [ 0x3192, 'IdeographicAnnotationOneMark', [], 'Display', None],
  [ 0x3193, 'IdeographicAnnotationTwoMark', [], 'Display', None],
  [ 0x3194, 'IdeographicAnnotationThreeMark', [], 'Display', None],
  [ 0x3195, 'IdeographicAnnotationFourMark', [], 'Display', None],
  [ 0x3196, 'IdeographicAnnotationTopMark', [], 'Display', None],
  [ 0x3197, 'IdeographicAnnotationMiddleMark', [], 'Display', None],
  [ 0x3198, 'IdeographicAnnotationBottomMark', [], 'Display', None],
  [ 0x3199, 'IdeographicAnnotationFirstMark', [], 'Display', None],
  [ 0x319a, 'IdeographicAnnotationSecondMark', [], 'Display', None],
  [ 0x319b, 'IdeographicAnnotationThirdMark', [], 'Display', None],
  [ 0x319c, 'IdeographicAnnotationFourthMark', [], 'Display', None],
  [ 0x319d, 'IdeographicAnnotationHeavenMark', [], 'Display', None],
  [ 0x319e, 'IdeographicAnnotationEarthMark', [], 'Display', None],
  [ 0x319f, 'IdeographicAnnotationManMark', [], 'Display', None],
  [ 0x31a0, 'BopomofoLetterBu', [], 'Display', None],
  [ 0x31a1, 'BopomofoLetterZi', [], 'Display', None],
  [ 0x31a2, 'BopomofoLetterJi', [], 'Display', None],
  [ 0x31a3, 'BopomofoLetterGu', [], 'Display', None],
  [ 0x31a4, 'BopomofoLetterEe', [], 'Display', None],
  [ 0x31a5, 'BopomofoLetterEnn', [], 'Display', None],
  [ 0x31a6, 'BopomofoLetterOo', [], 'Display', None],
  [ 0x31a7, 'BopomofoLetterOnn', [], 'Display', None],
  [ 0x31a8, 'BopomofoLetterIr', [], 'Display', None],
  [ 0x31a9, 'BopomofoLetterAnn', [], 'Display', None],
  [ 0x31aa, 'BopomofoLetterInn', [], 'Display', None],
  [ 0x31ab, 'BopomofoLetterUnn', [], 'Display', None],
  [ 0x31ac, 'BopomofoLetterIm', [], 'Display', None],
  [ 0x31ad, 'BopomofoLetterNgg', [], 'Display', None],
  [ 0x31ae, 'BopomofoLetterAinn', [], 'Display', None],
  [ 0x31af, 'BopomofoLetterAunn', [], 'Display', None],
  [ 0x31b0, 'BopomofoLetterAm', [], 'Display', None],
  [ 0x31b1, 'BopomofoLetterOm', [], 'Display', None],
  [ 0x31b2, 'BopomofoLetterOng', [], 'Display', None],
  [ 0x31b3, 'BopomofoLetterInnn', [], 'Display', None],
  [ 0x31b4, 'BopomofoFinalLetterP', [], 'Display', None],
  [ 0x31b5, 'BopomofoFinalLetterT', [], 'Display', None],
  [ 0x31b6, 'BopomofoFinalLetterK', [], 'Display', None],
  [ 0x31b7, 'BopomofoFinalLetterH', [], 'Display', None],
  [ 0x31b8, 'UnicodeCharacter0x31b8', [], 'Display', None],
  [ 0x31b9, 'UnicodeCharacter0x31b9', [], 'Display', None],
  [ 0x31ba, 'UnicodeCharacter0x31ba', [], 'Display', None],
  [ 0x31bb, 'UnicodeCharacter0x31bb', [], 'Display', None],
  [ 0x31bc, 'UnicodeCharacter0x31bc', [], 'Display', None],
  [ 0x31bd, 'UnicodeCharacter0x31bd', [], 'Display', None],
  [ 0x31be, 'UnicodeCharacter0x31be', [], 'Display', None],
  [ 0x31bf, 'UnicodeCharacter0x31bf', [], 'Display', None],
  [ 0x31c0, 'CjkStrokeT', [], 'Display', None],
  [ 0x31c1, 'CjkStrokeWg', [], 'Display', None],
  [ 0x31c2, 'CjkStrokeXg', [], 'Display', None],
  [ 0x31c3, 'CjkStrokeBxg', [], 'Display', None],
  [ 0x31c4, 'CjkStrokeSw', [], 'Display', None],
  [ 0x31c5, 'CjkStrokeHzz', [], 'Display', None],
  [ 0x31c6, 'CjkStrokeHzg', [], 'Display', None],
  [ 0x31c7, 'CjkStrokeHp', [], 'Display', None],
  [ 0x31c8, 'CjkStrokeHzwg', [], 'Display', None],
  [ 0x31c9, 'CjkStrokeSzwg', [], 'Display', None],
  [ 0x31ca, 'CjkStrokeHzt', [], 'Display', None],
  [ 0x31cb, 'CjkStrokeHzzp', [], 'Display', None],
  [ 0x31cc, 'CjkStrokeHpwg', [], 'Display', None],
  [ 0x31cd, 'CjkStrokeHzw', [], 'Display', None],
  [ 0x31ce, 'CjkStrokeHzzz', [], 'Display', None],
  [ 0x31cf, 'CjkStrokeN', [], 'Display', None],
  [ 0x31d0, 'CjkStrokeH', [], 'Display', None],
  [ 0x31d1, 'CjkStrokeS', [], 'Display', None],
  [ 0x31d2, 'CjkStrokeP', [], 'Display', None],
  [ 0x31d3, 'CjkStrokeSp', [], 'Display', None],
  [ 0x31d4, 'CjkStrokeD', [], 'Display', None],
  [ 0x31d5, 'CjkStrokeHz', [], 'Display', None],
  [ 0x31d6, 'CjkStrokeHg', [], 'Display', None],
  [ 0x31d7, 'CjkStrokeSz', [], 'Display', None],
  [ 0x31d8, 'CjkStrokeSwz', [], 'Display', None],
  [ 0x31d9, 'CjkStrokeSt', [], 'Display', None],
  [ 0x31da, 'CjkStrokeSg', [], 'Display', None],
  [ 0x31db, 'CjkStrokePd', [], 'Display', None],
  [ 0x31dc, 'CjkStrokePz', [], 'Display', None],
  [ 0x31dd, 'CjkStrokeTn', [], 'Display', None],
  [ 0x31de, 'CjkStrokeSzz', [], 'Display', None],
  [ 0x31df, 'CjkStrokeSwg', [], 'Display', None],
  [ 0x31e0, 'CjkStrokeHxwg', [], 'Display', None],
  [ 0x31e1, 'CjkStrokeHzzzg', [], 'Display', None],
  [ 0x31e2, 'CjkStrokePg', [], 'Display', None],
  [ 0x31e3, 'CjkStrokeQ', [], 'Display', None],
  [ 0x31e4, 'UnicodeCharacter0x31e4', [], 'Display', None],
  [ 0x31e5, 'UnicodeCharacter0x31e5', [], 'Display', None],
  [ 0x31e6, 'UnicodeCharacter0x31e6', [], 'Display', None],
  [ 0x31e7, 'UnicodeCharacter0x31e7', [], 'Display', None],
  [ 0x31e8, 'UnicodeCharacter0x31e8', [], 'Display', None],
  [ 0x31e9, 'UnicodeCharacter0x31e9', [], 'Display', None],
  [ 0x31ea, 'UnicodeCharacter0x31ea', [], 'Display', None],
  [ 0x31eb, 'UnicodeCharacter0x31eb', [], 'Display', None],
  [ 0x31ec, 'UnicodeCharacter0x31ec', [], 'Display', None],
  [ 0x31ed, 'UnicodeCharacter0x31ed', [], 'Display', None],
  [ 0x31ee, 'UnicodeCharacter0x31ee', [], 'Display', None],
  [ 0x31ef, 'UnicodeCharacter0x31ef', [], 'Display', None],
  [ 0x31f0, 'KatakanaLetterSmallKu', [], 'Display', None],
  [ 0x31f1, 'KatakanaLetterSmallSi', [], 'Display', None],
  [ 0x31f2, 'KatakanaLetterSmallSu', [], 'Display', None],
  [ 0x31f3, 'KatakanaLetterSmallTo', [], 'Display', None],
  [ 0x31f4, 'KatakanaLetterSmallNu', [], 'Display', None],
  [ 0x31f5, 'KatakanaLetterSmallHa', [], 'Display', None],
  [ 0x31f6, 'KatakanaLetterSmallHi', [], 'Display', None],
  [ 0x31f7, 'KatakanaLetterSmallHu', [], 'Display', None],
  [ 0x31f8, 'KatakanaLetterSmallHe', [], 'Display', None],
  [ 0x31f9, 'KatakanaLetterSmallHo', [], 'Display', None],
  [ 0x31fa, 'KatakanaLetterSmallMu', [], 'Display', None],
  [ 0x31fb, 'KatakanaLetterSmallRa', [], 'Display', None],
  [ 0x31fc, 'KatakanaLetterSmallRi', [], 'Display', None],
  [ 0x31fd, 'KatakanaLetterSmallRu', [], 'Display', None],
  [ 0x31fe, 'KatakanaLetterSmallRe', [], 'Display', None],
  [ 0x31ff, 'KatakanaLetterSmallRo', [], 'Display', None],
  [ 0x3200, 'ParenthesizedHangulKiyeok', [], 'Display', None],
  [ 0x3201, 'ParenthesizedHangulNieun', [], 'Display', None],
  [ 0x3202, 'ParenthesizedHangulTikeut', [], 'Display', None],
  [ 0x3203, 'ParenthesizedHangulRieul', [], 'Display', None],
  [ 0x3204, 'ParenthesizedHangulMieum', [], 'Display', None],
  [ 0x3205, 'ParenthesizedHangulPieup', [], 'Display', None],
  [ 0x3206, 'ParenthesizedHangulSios', [], 'Display', None],
  [ 0x3207, 'ParenthesizedHangulIeung', [], 'Display', None],
  [ 0x3208, 'ParenthesizedHangulCieuc', [], 'Display', None],
  [ 0x3209, 'ParenthesizedHangulChieuch', [], 'Display', None],
  [ 0x320a, 'ParenthesizedHangulKhieukh', [], 'Display', None],
  [ 0x320b, 'ParenthesizedHangulThieuth', [], 'Display', None],
  [ 0x320c, 'ParenthesizedHangulPhieuph', [], 'Display', None],
  [ 0x320d, 'ParenthesizedHangulHieuh', [], 'Display', None],
  [ 0x320e, 'ParenthesizedHangulKiyeokA', [], 'Display', None],
  [ 0x320f, 'ParenthesizedHangulNieunA', [], 'Display', None],
  [ 0x3210, 'ParenthesizedHangulTikeutA', [], 'Display', None],
  [ 0x3211, 'ParenthesizedHangulRieulA', [], 'Display', None],
  [ 0x3212, 'ParenthesizedHangulMieumA', [], 'Display', None],
  [ 0x3213, 'ParenthesizedHangulPieupA', [], 'Display', None],
  [ 0x3214, 'ParenthesizedHangulSiosA', [], 'Display', None],
  [ 0x3215, 'ParenthesizedHangulIeungA', [], 'Display', None],
  [ 0x3216, 'ParenthesizedHangulCieucA', [], 'Display', None],
  [ 0x3217, 'ParenthesizedHangulChieuchA', [], 'Display', None],
  [ 0x3218, 'ParenthesizedHangulKhieukhA', [], 'Display', None],
  [ 0x3219, 'ParenthesizedHangulThieuthA', [], 'Display', None],
  [ 0x321a, 'ParenthesizedHangulPhieuphA', [], 'Display', None],
  [ 0x321b, 'ParenthesizedHangulHieuhA', [], 'Display', None],
  [ 0x321c, 'ParenthesizedHangulCieucU', [], 'Display', None],
  [ 0x321d, 'ParenthesizedKoreanCharacterOjeon', [], 'Display', None],
  [ 0x321e, 'ParenthesizedKoreanCharacterOHu', [], 'Display', None],
  [ 0x321f, 'UnicodeCharacter0x321f', [], 'Display', None],
  [ 0x3220, 'ParenthesizedIdeographOne', [], 'Display', None],
  [ 0x3221, 'ParenthesizedIdeographTwo', [], 'Display', None],
  [ 0x3222, 'ParenthesizedIdeographThree', [], 'Display', None],
  [ 0x3223, 'ParenthesizedIdeographFour', [], 'Display', None],
  [ 0x3224, 'ParenthesizedIdeographFive', [], 'Display', None],
  [ 0x3225, 'ParenthesizedIdeographSix', [], 'Display', None],
  [ 0x3226, 'ParenthesizedIdeographSeven', [], 'Display', None],
  [ 0x3227, 'ParenthesizedIdeographEight', [], 'Display', None],
  [ 0x3228, 'ParenthesizedIdeographNine', [], 'Display', None],
  [ 0x3229, 'ParenthesizedIdeographTen', [], 'Display', None],
  [ 0x322a, 'ParenthesizedIdeographMoon', [], 'Display', None],
  [ 0x322b, 'ParenthesizedIdeographFire', [], 'Display', None],
  [ 0x322c, 'ParenthesizedIdeographWater', [], 'Display', None],
  [ 0x322d, 'ParenthesizedIdeographWood', [], 'Display', None],
  [ 0x322e, 'ParenthesizedIdeographMetal', [], 'Display', None],
  [ 0x322f, 'ParenthesizedIdeographEarth', [], 'Display', None],
  [ 0x3230, 'ParenthesizedIdeographSun', [], 'Display', None],
  [ 0x3231, 'ParenthesizedIdeographStock', [], 'Display', None],
  [ 0x3232, 'ParenthesizedIdeographHave', [], 'Display', None],
  [ 0x3233, 'ParenthesizedIdeographSociety', [], 'Display', None],
  [ 0x3234, 'ParenthesizedIdeographName', [], 'Display', None],
  [ 0x3235, 'ParenthesizedIdeographSpecial', [], 'Display', None],
  [ 0x3236, 'ParenthesizedIdeographFinancial', [], 'Display', None],
  [ 0x3237, 'ParenthesizedIdeographCongratulation', [], 'Display', None],
  [ 0x3238, 'ParenthesizedIdeographLabor', [], 'Display', None],
  [ 0x3239, 'ParenthesizedIdeographRepresent', [], 'Display', None],
  [ 0x323a, 'ParenthesizedIdeographCall', [], 'Display', None],
  [ 0x323b, 'ParenthesizedIdeographStudy', [], 'Display', None],
  [ 0x323c, 'ParenthesizedIdeographSupervise', [], 'Display', None],
  [ 0x323d, 'ParenthesizedIdeographEnterprise', [], 'Display', None],
  [ 0x323e, 'ParenthesizedIdeographResource', [], 'Display', None],
  [ 0x323f, 'ParenthesizedIdeographAlliance', [], 'Display', None],
  [ 0x3240, 'ParenthesizedIdeographFestival', [], 'Display', None],
  [ 0x3241, 'ParenthesizedIdeographRest', [], 'Display', None],
  [ 0x3242, 'ParenthesizedIdeographSelf', [], 'Display', None],
  [ 0x3243, 'ParenthesizedIdeographReach', [], 'Display', None],
  [ 0x3244, 'UnicodeCharacter0x3244', [], 'Display', None],
  [ 0x3245, 'UnicodeCharacter0x3245', [], 'Display', None],
  [ 0x3246, 'UnicodeCharacter0x3246', [], 'Display', None],
  [ 0x3247, 'UnicodeCharacter0x3247', [], 'Display', None],
  [ 0x3248, 'UnicodeCharacter0x3248', [], 'Display', None],
  [ 0x3249, 'UnicodeCharacter0x3249', [], 'Display', None],
  [ 0x324a, 'UnicodeCharacter0x324a', [], 'Display', None],
  [ 0x324b, 'UnicodeCharacter0x324b', [], 'Display', None],
  [ 0x324c, 'UnicodeCharacter0x324c', [], 'Display', None],
  [ 0x324d, 'UnicodeCharacter0x324d', [], 'Display', None],
  [ 0x324e, 'UnicodeCharacter0x324e', [], 'Display', None],
  [ 0x324f, 'UnicodeCharacter0x324f', [], 'Display', None],
  [ 0x3250, 'PartnershipSign', [], 'Display', None],
  [ 0x3251, 'CircledNumberTwentyOne', [], 'Display', None],
  [ 0x3252, 'CircledNumberTwentyTwo', [], 'Display', None],
  [ 0x3253, 'CircledNumberTwentyThree', [], 'Display', None],
  [ 0x3254, 'CircledNumberTwentyFour', [], 'Display', None],
  [ 0x3255, 'CircledNumberTwentyFive', [], 'Display', None],
  [ 0x3256, 'CircledNumberTwentySix', [], 'Display', None],
  [ 0x3257, 'CircledNumberTwentySeven', [], 'Display', None],
  [ 0x3258, 'CircledNumberTwentyEight', [], 'Display', None],
  [ 0x3259, 'CircledNumberTwentyNine', [], 'Display', None],
  [ 0x325a, 'CircledNumberThirty', [], 'Display', None],
  [ 0x325b, 'CircledNumberThirtyOne', [], 'Display', None],
  [ 0x325c, 'CircledNumberThirtyTwo', [], 'Display', None],
  [ 0x325d, 'CircledNumberThirtyThree', [], 'Display', None],
  [ 0x325e, 'CircledNumberThirtyFour', [], 'Display', None],
  [ 0x325f, 'CircledNumberThirtyFive', [], 'Display', None],
  [ 0x3260, 'CircledHangulKiyeok', [], 'Display', None],
  [ 0x3261, 'CircledHangulNieun', [], 'Display', None],
  [ 0x3262, 'CircledHangulTikeut', [], 'Display', None],
  [ 0x3263, 'CircledHangulRieul', [], 'Display', None],
  [ 0x3264, 'CircledHangulMieum', [], 'Display', None],
  [ 0x3265, 'CircledHangulPieup', [], 'Display', None],
  [ 0x3266, 'CircledHangulSios', [], 'Display', None],
  [ 0x3267, 'CircledHangulIeung', [], 'Display', None],
  [ 0x3268, 'CircledHangulCieuc', [], 'Display', None],
  [ 0x3269, 'CircledHangulChieuch', [], 'Display', None],
  [ 0x326a, 'CircledHangulKhieukh', [], 'Display', None],
  [ 0x326b, 'CircledHangulThieuth', [], 'Display', None],
  [ 0x326c, 'CircledHangulPhieuph', [], 'Display', None],
  [ 0x326d, 'CircledHangulHieuh', [], 'Display', None],
  [ 0x326e, 'CircledHangulKiyeokA', [], 'Display', None],
  [ 0x326f, 'CircledHangulNieunA', [], 'Display', None],
  [ 0x3270, 'CircledHangulTikeutA', [], 'Display', None],
  [ 0x3271, 'CircledHangulRieulA', [], 'Display', None],
  [ 0x3272, 'CircledHangulMieumA', [], 'Display', None],
  [ 0x3273, 'CircledHangulPieupA', [], 'Display', None],
  [ 0x3274, 'CircledHangulSiosA', [], 'Display', None],
  [ 0x3275, 'CircledHangulIeungA', [], 'Display', None],
  [ 0x3276, 'CircledHangulCieucA', [], 'Display', None],
  [ 0x3277, 'CircledHangulChieuchA', [], 'Display', None],
  [ 0x3278, 'CircledHangulKhieukhA', [], 'Display', None],
  [ 0x3279, 'CircledHangulThieuthA', [], 'Display', None],
  [ 0x327a, 'CircledHangulPhieuphA', [], 'Display', None],
  [ 0x327b, 'CircledHangulHieuhA', [], 'Display', None],
  [ 0x327c, 'CircledKoreanCharacterChamko', [], 'Display', None],
  [ 0x327d, 'CircledKoreanCharacterJueui', [], 'Display', None],
  [ 0x327e, 'CircledHangulIeungU', [], 'Display', None],
  [ 0x327f, 'KoreanStandardSymbol', [], 'Display', None],
  [ 0x3280, 'CircledIdeographOne', [], 'Display', None],
  [ 0x3281, 'CircledIdeographTwo', [], 'Display', None],
  [ 0x3282, 'CircledIdeographThree', [], 'Display', None],
  [ 0x3283, 'CircledIdeographFour', [], 'Display', None],
  [ 0x3284, 'CircledIdeographFive', [], 'Display', None],
  [ 0x3285, 'CircledIdeographSix', [], 'Display', None],
  [ 0x3286, 'CircledIdeographSeven', [], 'Display', None],
  [ 0x3287, 'CircledIdeographEight', [], 'Display', None],
  [ 0x3288, 'CircledIdeographNine', [], 'Display', None],
  [ 0x3289, 'CircledIdeographTen', [], 'Display', None],
  [ 0x328a, 'CircledIdeographMoon', [], 'Display', None],
  [ 0x328b, 'CircledIdeographFire', [], 'Display', None],
  [ 0x328c, 'CircledIdeographWater', [], 'Display', None],
  [ 0x328d, 'CircledIdeographWood', [], 'Display', None],
  [ 0x328e, 'CircledIdeographMetal', [], 'Display', None],
  [ 0x328f, 'CircledIdeographEarth', [], 'Display', None],
  [ 0x3290, 'CircledIdeographSun', [], 'Display', None],
  [ 0x3291, 'CircledIdeographStock', [], 'Display', None],
  [ 0x3292, 'CircledIdeographHave', [], 'Display', None],
  [ 0x3293, 'CircledIdeographSociety', [], 'Display', None],
  [ 0x3294, 'CircledIdeographName', [], 'Display', None],
  [ 0x3295, 'CircledIdeographSpecial', [], 'Display', None],
  [ 0x3296, 'CircledIdeographFinancial', [], 'Display', None],
  [ 0x3297, 'CircledIdeographCongratulation', [], 'Display', None],
  [ 0x3298, 'CircledIdeographLabor', [], 'Display', None],
  [ 0x3299, 'CircledIdeographSecret', [], 'Display', None],
  [ 0x329a, 'CircledIdeographMale', [], 'Display', None],
  [ 0x329b, 'CircledIdeographFemale', [], 'Display', None],
  [ 0x329c, 'CircledIdeographSuitable', [], 'Display', None],
  [ 0x329d, 'CircledIdeographExcellent', [], 'Display', None],
  [ 0x329e, 'CircledIdeographPrint', [], 'Display', None],
  [ 0x329f, 'CircledIdeographAttention', [], 'Display', None],
  [ 0x32a0, 'CircledIdeographItem', [], 'Display', None],
  [ 0x32a1, 'CircledIdeographRest', [], 'Display', None],
  [ 0x32a2, 'CircledIdeographCopy', [], 'Display', None],
  [ 0x32a3, 'CircledIdeographCorrect', [], 'Display', None],
  [ 0x32a4, 'CircledIdeographHigh', [], 'Display', None],
  [ 0x32a5, 'CircledIdeographCentre', [], 'Display', None],
  [ 0x32a6, 'CircledIdeographLow', [], 'Display', None],
  [ 0x32a7, 'CircledIdeographLeft', [], 'Display', None],
  [ 0x32a8, 'CircledIdeographRight', [], 'Display', None],
  [ 0x32a9, 'CircledIdeographMedicine', [], 'Display', None],
  [ 0x32aa, 'CircledIdeographReligion', [], 'Display', None],
  [ 0x32ab, 'CircledIdeographStudy', [], 'Display', None],
  [ 0x32ac, 'CircledIdeographSupervise', [], 'Display', None],
  [ 0x32ad, 'CircledIdeographEnterprise', [], 'Display', None],
  [ 0x32ae, 'CircledIdeographResource', [], 'Display', None],
  [ 0x32af, 'CircledIdeographAlliance', [], 'Display', None],
  [ 0x32b0, 'CircledIdeographNight', [], 'Display', None],
  [ 0x32b1, 'CircledNumberThirtySix', [], 'Display', None],
  [ 0x32b2, 'CircledNumberThirtySeven', [], 'Display', None],
  [ 0x32b3, 'CircledNumberThirtyEight', [], 'Display', None],
  [ 0x32b4, 'CircledNumberThirtyNine', [], 'Display', None],
  [ 0x32b5, 'CircledNumberForty', [], 'Display', None],
  [ 0x32b6, 'CircledNumberFortyOne', [], 'Display', None],
  [ 0x32b7, 'CircledNumberFortyTwo', [], 'Display', None],
  [ 0x32b8, 'CircledNumberFortyThree', [], 'Display', None],
  [ 0x32b9, 'CircledNumberFortyFour', [], 'Display', None],
  [ 0x32ba, 'CircledNumberFortyFive', [], 'Display', None],
  [ 0x32bb, 'CircledNumberFortySix', [], 'Display', None],
  [ 0x32bc, 'CircledNumberFortySeven', [], 'Display', None],
  [ 0x32bd, 'CircledNumberFortyEight', [], 'Display', None],
  [ 0x32be, 'CircledNumberFortyNine', [], 'Display', None],
  [ 0x32bf, 'CircledNumberFifty', [], 'Display', None],
  [ 0x32c0, 'IdeographicTelegraphSymbolForJanuary', [], 'Display', None],
  [ 0x32c1, 'IdeographicTelegraphSymbolForFebruary', [], 'Display', None],
  [ 0x32c2, 'IdeographicTelegraphSymbolForMarch', [], 'Display', None],
  [ 0x32c3, 'IdeographicTelegraphSymbolForApril', [], 'Display', None],
  [ 0x32c4, 'IdeographicTelegraphSymbolForMay', [], 'Display', None],
  [ 0x32c5, 'IdeographicTelegraphSymbolForJune', [], 'Display', None],
  [ 0x32c6, 'IdeographicTelegraphSymbolForJuly', [], 'Display', None],
  [ 0x32c7, 'IdeographicTelegraphSymbolForAugust', [], 'Display', None],
  [ 0x32c8, 'IdeographicTelegraphSymbolForSeptember', [], 'Display', None],
  [ 0x32c9, 'IdeographicTelegraphSymbolForOctober', [], 'Display', None],
  [ 0x32ca, 'IdeographicTelegraphSymbolForNovember', [], 'Display', None],
  [ 0x32cb, 'IdeographicTelegraphSymbolForDecember', [], 'Display', None],
  [ 0x32cc, 'SquareHg', [], 'Display', None],
  [ 0x32cd, 'SquareErg', [], 'Display', None],
  [ 0x32ce, 'SquareEv', [], 'Display', None],
  [ 0x32cf, 'LimitedLiabilitySign', [], 'Display', None],
  [ 0x32d0, 'CircledKatakanaA', [], 'Display', None],
  [ 0x32d1, 'CircledKatakanaI', [], 'Display', None],
  [ 0x32d2, 'CircledKatakanaU', [], 'Display', None],
  [ 0x32d3, 'CircledKatakanaE', [], 'Display', None],
  [ 0x32d4, 'CircledKatakanaO', [], 'Display', None],
  [ 0x32d5, 'CircledKatakanaKa', [], 'Display', None],
  [ 0x32d6, 'CircledKatakanaKi', [], 'Display', None],
  [ 0x32d7, 'CircledKatakanaKu', [], 'Display', None],
  [ 0x32d8, 'CircledKatakanaKe', [], 'Display', None],
  [ 0x32d9, 'CircledKatakanaKo', [], 'Display', None],
  [ 0x32da, 'CircledKatakanaSa', [], 'Display', None],
  [ 0x32db, 'CircledKatakanaSi', [], 'Display', None],
  [ 0x32dc, 'CircledKatakanaSu', [], 'Display', None],
  [ 0x32dd, 'CircledKatakanaSe', [], 'Display', None],
  [ 0x32de, 'CircledKatakanaSo', [], 'Display', None],
  [ 0x32df, 'CircledKatakanaTa', [], 'Display', None],
  [ 0x32e0, 'CircledKatakanaTi', [], 'Display', None],
  [ 0x32e1, 'CircledKatakanaTu', [], 'Display', None],
  [ 0x32e2, 'CircledKatakanaTe', [], 'Display', None],
  [ 0x32e3, 'CircledKatakanaTo', [], 'Display', None],
  [ 0x32e4, 'CircledKatakanaNa', [], 'Display', None],
  [ 0x32e5, 'CircledKatakanaNi', [], 'Display', None],
  [ 0x32e6, 'CircledKatakanaNu', [], 'Display', None],
  [ 0x32e7, 'CircledKatakanaNe', [], 'Display', None],
  [ 0x32e8, 'CircledKatakanaNo', [], 'Display', None],
  [ 0x32e9, 'CircledKatakanaHa', [], 'Display', None],
  [ 0x32ea, 'CircledKatakanaHi', [], 'Display', None],
  [ 0x32eb, 'CircledKatakanaHu', [], 'Display', None],
  [ 0x32ec, 'CircledKatakanaHe', [], 'Display', None],
  [ 0x32ed, 'CircledKatakanaHo', [], 'Display', None],
  [ 0x32ee, 'CircledKatakanaMa', [], 'Display', None],
  [ 0x32ef, 'CircledKatakanaMi', [], 'Display', None],
  [ 0x32f0, 'CircledKatakanaMu', [], 'Display', None],
  [ 0x32f1, 'CircledKatakanaMe', [], 'Display', None],
  [ 0x32f2, 'CircledKatakanaMo', [], 'Display', None],
  [ 0x32f3, 'CircledKatakanaYa', [], 'Display', None],
  [ 0x32f4, 'CircledKatakanaYu', [], 'Display', None],
  [ 0x32f5, 'CircledKatakanaYo', [], 'Display', None],
  [ 0x32f6, 'CircledKatakanaRa', [], 'Display', None],
  [ 0x32f7, 'CircledKatakanaRi', [], 'Display', None],
  [ 0x32f8, 'CircledKatakanaRu', [], 'Display', None],
  [ 0x32f9, 'CircledKatakanaRe', [], 'Display', None],
  [ 0x32fa, 'CircledKatakanaRo', [], 'Display', None],
  [ 0x32fb, 'CircledKatakanaWa', [], 'Display', None],
  [ 0x32fc, 'CircledKatakanaWi', [], 'Display', None],
  [ 0x32fd, 'CircledKatakanaWe', [], 'Display', None],
  [ 0x32fe, 'CircledKatakanaWo', [], 'Display', None],
  [ 0x32ff, 'UnicodeCharacter0x32ff', [], 'Display', None],
  [ 0x3300, 'SquareApaato', [], 'Display', None],
  [ 0x3301, 'SquareAruhua', [], 'Display', None],
  [ 0x3302, 'SquareAnpea', [], 'Display', None],
  [ 0x3303, 'SquareAaru', [], 'Display', None],
  [ 0x3304, 'SquareIningu', [], 'Display', None],
  [ 0x3305, 'SquareInti', [], 'Display', None],
  [ 0x3306, 'SquareUon', [], 'Display', None],
  [ 0x3307, 'SquareEsukuudo', [], 'Display', None],
  [ 0x3308, 'SquareEekaa', [], 'Display', None],
  [ 0x3309, 'SquareOnsu', [], 'Display', None],
  [ 0x330a, 'SquareOomu', [], 'Display', None],
  [ 0x330b, 'SquareKairi', [], 'Display', None],
  [ 0x330c, 'SquareKaratto', [], 'Display', None],
  [ 0x330d, 'SquareKarorii', [], 'Display', None],
  [ 0x330e, 'SquareGaron', [], 'Display', None],
  [ 0x330f, 'SquareGanma', [], 'Display', None],
  [ 0x3310, 'SquareGiga', [], 'Display', None],
  [ 0x3311, 'SquareGinii', [], 'Display', None],
  [ 0x3312, 'SquareKyurii', [], 'Display', None],
  [ 0x3313, 'SquareGirudaa', [], 'Display', None],
  [ 0x3314, 'SquareKiro', [], 'Display', None],
  [ 0x3315, 'SquareKiroguramu', [], 'Display', None],
  [ 0x3316, 'SquareKiromeetoru', [], 'Display', None],
  [ 0x3317, 'SquareKirowatto', [], 'Display', None],
  [ 0x3318, 'SquareGuramu', [], 'Display', None],
  [ 0x3319, 'SquareGuramuton', [], 'Display', None],
  [ 0x331a, 'SquareKuruzeiro', [], 'Display', None],
  [ 0x331b, 'SquareKuroone', [], 'Display', None],
  [ 0x331c, 'SquareKeesu', [], 'Display', None],
  [ 0x331d, 'SquareKoruna', [], 'Display', None],
  [ 0x331e, 'SquareKoopo', [], 'Display', None],
  [ 0x331f, 'SquareSaikuru', [], 'Display', None],
  [ 0x3320, 'SquareSantiimu', [], 'Display', None],
  [ 0x3321, 'SquareSiringu', [], 'Display', None],
  [ 0x3322, 'SquareSenti', [], 'Display', None],
  [ 0x3323, 'SquareSento', [], 'Display', None],
  [ 0x3324, 'SquareDaasu', [], 'Display', None],
  [ 0x3325, 'SquareDesi', [], 'Display', None],
  [ 0x3326, 'SquareDoru', [], 'Display', None],
  [ 0x3327, 'SquareTon', [], 'Display', None],
  [ 0x3328, 'SquareNano', [], 'Display', None],
  [ 0x3329, 'SquareNotto', [], 'Display', None],
  [ 0x332a, 'SquareHaitu', [], 'Display', None],
  [ 0x332b, 'SquarePaasento', [], 'Display', None],
  [ 0x332c, 'SquarePaatu', [], 'Display', None],
  [ 0x332d, 'SquareBaareru', [], 'Display', None],
  [ 0x332e, 'SquarePiasutoru', [], 'Display', None],
  [ 0x332f, 'SquarePikuru', [], 'Display', None],
  [ 0x3330, 'SquarePiko', [], 'Display', None],
  [ 0x3331, 'SquareBiru', [], 'Display', None],
  [ 0x3332, 'SquareHuaraddo', [], 'Display', None],
  [ 0x3333, 'SquareHuiito', [], 'Display', None],
  [ 0x3334, 'SquareBussyeru', [], 'Display', None],
  [ 0x3335, 'SquareHuran', [], 'Display', None],
  [ 0x3336, 'SquareHekutaaru', [], 'Display', None],
  [ 0x3337, 'SquarePeso', [], 'Display', None],
  [ 0x3338, 'SquarePenihi', [], 'Display', None],
  [ 0x3339, 'SquareHerutu', [], 'Display', None],
  [ 0x333a, 'SquarePensu', [], 'Display', None],
  [ 0x333b, 'SquarePeezi', [], 'Display', None],
  [ 0x333c, 'SquareBeeta', [], 'Display', None],
  [ 0x333d, 'SquarePointo', [], 'Display', None],
  [ 0x333e, 'SquareBoruto', [], 'Display', None],
  [ 0x333f, 'SquareHon', [], 'Display', None],
  [ 0x3340, 'SquarePondo', [], 'Display', None],
  [ 0x3341, 'SquareHooru', [], 'Display', None],
  [ 0x3342, 'SquareHoon', [], 'Display', None],
  [ 0x3343, 'SquareMaikuro', [], 'Display', None],
  [ 0x3344, 'SquareMairu', [], 'Display', None],
  [ 0x3345, 'SquareMahha', [], 'Display', None],
  [ 0x3346, 'SquareMaruku', [], 'Display', None],
  [ 0x3347, 'SquareMansyon', [], 'Display', None],
  [ 0x3348, 'SquareMikuron', [], 'Display', None],
  [ 0x3349, 'SquareMiri', [], 'Display', None],
  [ 0x334a, 'SquareMiribaaru', [], 'Display', None],
  [ 0x334b, 'SquareMega', [], 'Display', None],
  [ 0x334c, 'SquareMegaton', [], 'Display', None],
  [ 0x334d, 'SquareMeetoru', [], 'Display', None],
  [ 0x334e, 'SquareYaado', [], 'Display', None],
  [ 0x334f, 'SquareYaaru', [], 'Display', None],
  [ 0x3350, 'SquareYuan', [], 'Display', None],
  [ 0x3351, 'SquareRittoru', [], 'Display', None],
  [ 0x3352, 'SquareRira', [], 'Display', None],
  [ 0x3353, 'SquareRupii', [], 'Display', None],
  [ 0x3354, 'SquareRuuburu', [], 'Display', None],
  [ 0x3355, 'SquareRemu', [], 'Display', None],
  [ 0x3356, 'SquareRentogen', [], 'Display', None],
  [ 0x3357, 'SquareWatto', [], 'Display', None],
  [ 0x3358, 'IdeographicTelegraphSymbolForHourZero', [], 'Display', None],
  [ 0x3359, 'IdeographicTelegraphSymbolForHourOne', [], 'Display', None],
  [ 0x335a, 'IdeographicTelegraphSymbolForHourTwo', [], 'Display', None],
  [ 0x335b, 'IdeographicTelegraphSymbolForHourThree', [], 'Display', None],
  [ 0x335c, 'IdeographicTelegraphSymbolForHourFour', [], 'Display', None],
  [ 0x335d, 'IdeographicTelegraphSymbolForHourFive', [], 'Display', None],
  [ 0x335e, 'IdeographicTelegraphSymbolForHourSix', [], 'Display', None],
  [ 0x335f, 'IdeographicTelegraphSymbolForHourSeven', [], 'Display', None],
  [ 0x3360, 'IdeographicTelegraphSymbolForHourEight', [], 'Display', None],
  [ 0x3361, 'IdeographicTelegraphSymbolForHourNine', [], 'Display', None],
  [ 0x3362, 'IdeographicTelegraphSymbolForHourTen', [], 'Display', None],
  [ 0x3363, 'IdeographicTelegraphSymbolForHourEleven', [], 'Display', None],
  [ 0x3364, 'IdeographicTelegraphSymbolForHourTwelve', [], 'Display', None],
  [ 0x3365, 'IdeographicTelegraphSymbolForHourThirteen', [], 'Display', None],
  [ 0x3366, 'IdeographicTelegraphSymbolForHourFourteen', [], 'Display', None],
  [ 0x3367, 'IdeographicTelegraphSymbolForHourFifteen', [], 'Display', None],
  [ 0x3368, 'IdeographicTelegraphSymbolForHourSixteen', [], 'Display', None],
  [ 0x3369, 'IdeographicTelegraphSymbolForHourSeventeen', [], 'Display', None],
  [ 0x336a, 'IdeographicTelegraphSymbolForHourEighteen', [], 'Display', None],
  [ 0x336b, 'IdeographicTelegraphSymbolForHourNineteen', [], 'Display', None],
  [ 0x336c, 'IdeographicTelegraphSymbolForHourTwenty', [], 'Display', None],
  [ 0x336d, 'IdeographicTelegraphSymbolForHourTwentyone', [], 'Display', None],
  [ 0x336e, 'IdeographicTelegraphSymbolForHourTwentytwo', [], 'Display', None],
  [ 0x336f, 'IdeographicTelegraphSymbolForHourTwentythree', [], 'Display', None],
  [ 0x3370, 'IdeographicTelegraphSymbolForHourTwentyfour', [], 'Display', None],
  [ 0x3371, 'SquareHpa', [], 'Display', None],
  [ 0x3372, 'SquareDa', [], 'Display', None],
  [ 0x3373, 'SquareAu', [], 'Display', None],
  [ 0x3374, 'SquareBar', [], 'Display', None],
  [ 0x3375, 'SquareOv', [], 'Display', None],
  [ 0x3376, 'SquarePc', [], 'Display', None],
  [ 0x3377, 'SquareDm', [], 'Display', None],
  [ 0x3378, 'SquareDmSquared', [], 'Display', None],
  [ 0x3379, 'SquareDmCubed', [], 'Display', None],
  [ 0x337a, 'SquareIu', [], 'Display', None],
  [ 0x337b, 'SquareEraNameHeisei', [], 'Display', None],
  [ 0x337c, 'SquareEraNameSyouwa', [], 'Display', None],
  [ 0x337d, 'SquareEraNameTaisyou', [], 'Display', None],
  [ 0x337e, 'SquareEraNameMeizi', [], 'Display', None],
  [ 0x337f, 'SquareCorporation', [], 'Display', None],
  [ 0x3380, 'SquarePaAmps', [], 'Display', None],
  [ 0x3381, 'SquareNa', [], 'Display', None],
  [ 0x3382, 'SquareMuA', [], 'Display', None],
  [ 0x3383, 'SquareMa', [], 'Display', None],
  [ 0x3384, 'SquareKa', [], 'Display', None],
  [ 0x3385, 'SquareKb', [], 'Display', None],
  [ 0x3386, 'SquareMb', [], 'Display', None],
  [ 0x3387, 'SquareGb', [], 'Display', None],
  [ 0x3388, 'SquareCal', [], 'Display', None],
  [ 0x3389, 'SquareKcal', [], 'Display', None],
  [ 0x338a, 'SquarePf', [], 'Display', None],
  [ 0x338b, 'SquareNf', [], 'Display', None],
  [ 0x338c, 'SquareMuF', [], 'Display', None],
  [ 0x338d, 'SquareMuG', [], 'Display', None],
  [ 0x338e, 'SquareMg', [], 'Display', None],
  [ 0x338f, 'SquareKg', [], 'Display', None],
  [ 0x3390, 'SquareHz', [], 'Display', None],
  [ 0x3391, 'SquareKhz', [], 'Display', None],
  [ 0x3392, 'SquareMhz', [], 'Display', None],
  [ 0x3393, 'SquareGhz', [], 'Display', None],
  [ 0x3394, 'SquareThz', [], 'Display', None],
  [ 0x3395, 'SquareMuL', [], 'Display', None],
  [ 0x3396, 'SquareMl', [], 'Display', None],
  [ 0x3397, 'SquareDl', [], 'Display', None],
  [ 0x3398, 'SquareKl', [], 'Display', None],
  [ 0x3399, 'SquareFm', [], 'Display', None],
  [ 0x339a, 'SquareNm', [], 'Display', None],
  [ 0x339b, 'SquareMuM', [], 'Display', None],
  [ 0x339c, 'SquareMm', [], 'Display', None],
  [ 0x339d, 'SquareCm', [], 'Display', None],
  [ 0x339e, 'SquareKm', [], 'Display', None],
  [ 0x339f, 'SquareMmSquared', [], 'Display', None],
  [ 0x33a0, 'SquareCmSquared', [], 'Display', None],
  [ 0x33a1, 'SquareMSquared', [], 'Display', None],
  [ 0x33a2, 'SquareKmSquared', [], 'Display', None],
  [ 0x33a3, 'SquareMmCubed', [], 'Display', None],
  [ 0x33a4, 'SquareCmCubed', [], 'Display', None],
  [ 0x33a5, 'SquareMCubed', [], 'Display', None],
  [ 0x33a6, 'SquareKmCubed', [], 'Display', None],
  [ 0x33a7, 'SquareMOverS', [], 'Display', None],
  [ 0x33a8, 'SquareMOverSSquared', [], 'Display', None],
  [ 0x33a9, 'SquarePa', [], 'Display', None],
  [ 0x33aa, 'SquareKpa', [], 'Display', None],
  [ 0x33ab, 'SquareMpa', [], 'Display', None],
  [ 0x33ac, 'SquareGpa', [], 'Display', None],
  [ 0x33ad, 'SquareRad', [], 'Display', None],
  [ 0x33ae, 'SquareRadOverS', [], 'Display', None],
  [ 0x33af, 'SquareRadOverSSquared', [], 'Display', None],
  [ 0x33b0, 'SquarePs', [], 'Display', None],
  [ 0x33b1, 'SquareNs', [], 'Display', None],
  [ 0x33b2, 'SquareMuS', [], 'Display', None],
  [ 0x33b3, 'SquareMs', [], 'Display', None],
  [ 0x33b4, 'SquarePv', [], 'Display', None],
  [ 0x33b5, 'SquareNv', [], 'Display', None],
  [ 0x33b6, 'SquareMuV', [], 'Display', None],
  [ 0x33b7, 'SquareMv', [], 'Display', None],
  [ 0x33b8, 'SquareKv', [], 'Display', None],
  [ 0x33b9, 'SquareMvMega', [], 'Display', None],
  [ 0x33ba, 'SquarePw', [], 'Display', None],
  [ 0x33bb, 'SquareNw', [], 'Display', None],
  [ 0x33bc, 'SquareMuW', [], 'Display', None],
  [ 0x33bd, 'SquareMw', [], 'Display', None],
  [ 0x33be, 'SquareKw', [], 'Display', None],
  [ 0x33bf, 'SquareMwMega', [], 'Display', None],
  [ 0x33c0, 'SquareKOhm', [], 'Display', None],
  [ 0x33c1, 'SquareMOhm', [], 'Display', None],
  [ 0x33c2, 'SquareAm', [], 'Display', None],
  [ 0x33c3, 'SquareBq', [], 'Display', None],
  [ 0x33c4, 'SquareCc', [], 'Display', None],
  [ 0x33c5, 'SquareCd', [], 'Display', None],
  [ 0x33c6, 'SquareCOverKg', [], 'Display', None],
  [ 0x33c7, 'SquareCo', [], 'Display', None],
  [ 0x33c8, 'SquareDb', [], 'Display', None],
  [ 0x33c9, 'SquareGy', [], 'Display', None],
  [ 0x33ca, 'SquareHa', [], 'Display', None],
  [ 0x33cb, 'SquareHp', [], 'Display', None],
  [ 0x33cc, 'SquareIn', [], 'Display', None],
  [ 0x33cd, 'SquareKk', [], 'Display', None],
  [ 0x33ce, 'SquareKmCapital', [], 'Display', None],
  [ 0x33cf, 'SquareKt', [], 'Display', None],
  [ 0x33d0, 'SquareLm', [], 'Display', None],
  [ 0x33d1, 'SquareLn', [], 'Display', None],
  [ 0x33d2, 'SquareLog', [], 'Display', None],
  [ 0x33d3, 'SquareLx', [], 'Display', None],
  [ 0x33d4, 'SquareMbSmall', [], 'Display', None],
  [ 0x33d5, 'SquareMil', [], 'Display', None],
  [ 0x33d6, 'SquareMol', [], 'Display', None],
  [ 0x33d7, 'SquarePh', [], 'Display', None],
  [ 0x33d8, 'SquarePm', [], 'Display', None],
  [ 0x33d9, 'SquarePpm', [], 'Display', None],
  [ 0x33da, 'SquarePr', [], 'Display', None],
  [ 0x33db, 'SquareSr', [], 'Display', None],
  [ 0x33dc, 'SquareSv', [], 'Display', None],
  [ 0x33dd, 'SquareWb', [], 'Display', None],
  [ 0x33de, 'SquareVOverM', [], 'Display', None],
  [ 0x33df, 'SquareAOverM', [], 'Display', None],
  [ 0x33e0, 'IdeographicTelegraphSymbolForDayOne', [], 'Display', None],
  [ 0x33e1, 'IdeographicTelegraphSymbolForDayTwo', [], 'Display', None],
  [ 0x33e2, 'IdeographicTelegraphSymbolForDayThree', [], 'Display', None],
  [ 0x33e3, 'IdeographicTelegraphSymbolForDayFour', [], 'Display', None],
  [ 0x33e4, 'IdeographicTelegraphSymbolForDayFive', [], 'Display', None],
  [ 0x33e5, 'IdeographicTelegraphSymbolForDaySix', [], 'Display', None],
  [ 0x33e6, 'IdeographicTelegraphSymbolForDaySeven', [], 'Display', None],
  [ 0x33e7, 'IdeographicTelegraphSymbolForDayEight', [], 'Display', None],
  [ 0x33e8, 'IdeographicTelegraphSymbolForDayNine', [], 'Display', None],
  [ 0x33e9, 'IdeographicTelegraphSymbolForDayTen', [], 'Display', None],
  [ 0x33ea, 'IdeographicTelegraphSymbolForDayEleven', [], 'Display', None],
  [ 0x33eb, 'IdeographicTelegraphSymbolForDayTwelve', [], 'Display', None],
  [ 0x33ec, 'IdeographicTelegraphSymbolForDayThirteen', [], 'Display', None],
  [ 0x33ed, 'IdeographicTelegraphSymbolForDayFourteen', [], 'Display', None],
  [ 0x33ee, 'IdeographicTelegraphSymbolForDayFifteen', [], 'Display', None],
  [ 0x33ef, 'IdeographicTelegraphSymbolForDaySixteen', [], 'Display', None],
  [ 0x33f0, 'IdeographicTelegraphSymbolForDaySeventeen', [], 'Display', None],
  [ 0x33f1, 'IdeographicTelegraphSymbolForDayEighteen', [], 'Display', None],
  [ 0x33f2, 'IdeographicTelegraphSymbolForDayNineteen', [], 'Display', None],
  [ 0x33f3, 'IdeographicTelegraphSymbolForDayTwenty', [], 'Display', None],
  [ 0x33f4, 'IdeographicTelegraphSymbolForDayTwentyone', [], 'Display', None],
  [ 0x33f5, 'IdeographicTelegraphSymbolForDayTwentytwo', [], 'Display', None],
  [ 0x33f6, 'IdeographicTelegraphSymbolForDayTwentythree', [], 'Display', None],
  [ 0x33f7, 'IdeographicTelegraphSymbolForDayTwentyfour', [], 'Display', None],
  [ 0x33f8, 'IdeographicTelegraphSymbolForDayTwentyfive', [], 'Display', None],
  [ 0x33f9, 'IdeographicTelegraphSymbolForDayTwentysix', [], 'Display', None],
  [ 0x33fa, 'IdeographicTelegraphSymbolForDayTwentyseven', [], 'Display', None],
  [ 0x33fb, 'IdeographicTelegraphSymbolForDayTwentyeight', [], 'Display', None],
  [ 0x33fc, 'IdeographicTelegraphSymbolForDayTwentynine', [], 'Display', None],
  [ 0x33fd, 'IdeographicTelegraphSymbolForDayThirty', [], 'Display', None],
  [ 0x33fe, 'IdeographicTelegraphSymbolForDayThirtyone', [], 'Display', None],
  [ 0x33ff, 'SquareGal', [], 'Display', None],
  [ 0x3400, 'CjkUnifiedIdeograph3400', [], 'Display', None],
  [ 0x3401, 'CjkUnifiedIdeograph3401', [], 'Display', None],
  [ 0x3402, 'CjkUnifiedIdeograph3402', [], 'Display', None],
  [ 0x3403, 'CjkUnifiedIdeograph3403', [], 'Display', None],
  [ 0x3404, 'CjkUnifiedIdeograph3404', [], 'Display', None],
  [ 0x3405, 'CjkUnifiedIdeograph3405', [], 'Display', None],
  [ 0x3406, 'CjkUnifiedIdeograph3406', [], 'Display', None],
  [ 0x3407, 'CjkUnifiedIdeograph3407', [], 'Display', None],
  [ 0x3408, 'CjkUnifiedIdeograph3408', [], 'Display', None],
  [ 0x3409, 'CjkUnifiedIdeograph3409', [], 'Display', None],
  [ 0x340a, 'CjkUnifiedIdeograph340a', [], 'Display', None],
  [ 0x340b, 'CjkUnifiedIdeograph340b', [], 'Display', None],
  [ 0x340c, 'CjkUnifiedIdeograph340c', [], 'Display', None],
  [ 0x340d, 'CjkUnifiedIdeograph340d', [], 'Display', None],
  [ 0x340e, 'CjkUnifiedIdeograph340e', [], 'Display', None],
  [ 0x340f, 'CjkUnifiedIdeograph340f', [], 'Display', None],
  [ 0x3410, 'CjkUnifiedIdeograph3410', [], 'Display', None],
  [ 0x3411, 'CjkUnifiedIdeograph3411', [], 'Display', None],
  [ 0x3412, 'CjkUnifiedIdeograph3412', [], 'Display', None],
  [ 0x3413, 'CjkUnifiedIdeograph3413', [], 'Display', None],
  [ 0x3414, 'CjkUnifiedIdeograph3414', [], 'Display', None],
  [ 0x3415, 'CjkUnifiedIdeograph3415', [], 'Display', None],
  [ 0x3416, 'CjkUnifiedIdeograph3416', [], 'Display', None],
  [ 0x3417, 'CjkUnifiedIdeograph3417', [], 'Display', None],
  [ 0x3418, 'CjkUnifiedIdeograph3418', [], 'Display', None],
  [ 0x3419, 'CjkUnifiedIdeograph3419', [], 'Display', None],
  [ 0x341a, 'CjkUnifiedIdeograph341a', [], 'Display', None],
  [ 0x341b, 'CjkUnifiedIdeograph341b', [], 'Display', None],
  [ 0x341c, 'CjkUnifiedIdeograph341c', [], 'Display', None],
  [ 0x341d, 'CjkUnifiedIdeograph341d', [], 'Display', None],
  [ 0x341e, 'CjkUnifiedIdeograph341e', [], 'Display', None],
  [ 0x341f, 'CjkUnifiedIdeograph341f', [], 'Display', None],
  [ 0x3420, 'CjkUnifiedIdeograph3420', [], 'Display', None],
  [ 0x3421, 'CjkUnifiedIdeograph3421', [], 'Display', None],
  [ 0x3422, 'CjkUnifiedIdeograph3422', [], 'Display', None],
  [ 0x3423, 'CjkUnifiedIdeograph3423', [], 'Display', None],
  [ 0x3424, 'CjkUnifiedIdeograph3424', [], 'Display', None],
  [ 0x3425, 'CjkUnifiedIdeograph3425', [], 'Display', None],
  [ 0x3426, 'CjkUnifiedIdeograph3426', [], 'Display', None],
  [ 0x3427, 'CjkUnifiedIdeograph3427', [], 'Display', None],
  [ 0x3428, 'CjkUnifiedIdeograph3428', [], 'Display', None],
  [ 0x3429, 'CjkUnifiedIdeograph3429', [], 'Display', None],
  [ 0x342a, 'CjkUnifiedIdeograph342a', [], 'Display', None],
  [ 0x342b, 'CjkUnifiedIdeograph342b', [], 'Display', None],
  [ 0x342c, 'CjkUnifiedIdeograph342c', [], 'Display', None],
  [ 0x342d, 'CjkUnifiedIdeograph342d', [], 'Display', None],
  [ 0x342e, 'CjkUnifiedIdeograph342e', [], 'Display', None],
  [ 0x342f, 'CjkUnifiedIdeograph342f', [], 'Display', None],
  [ 0x3430, 'CjkUnifiedIdeograph3430', [], 'Display', None],
  [ 0x3431, 'CjkUnifiedIdeograph3431', [], 'Display', None],
  [ 0x3432, 'CjkUnifiedIdeograph3432', [], 'Display', None],
  [ 0x3433, 'CjkUnifiedIdeograph3433', [], 'Display', None],
  [ 0x3434, 'CjkUnifiedIdeograph3434', [], 'Display', None],
  [ 0x3435, 'CjkUnifiedIdeograph3435', [], 'Display', None],
  [ 0x3436, 'CjkUnifiedIdeograph3436', [], 'Display', None],
  [ 0x3437, 'CjkUnifiedIdeograph3437', [], 'Display', None],
  [ 0x3438, 'CjkUnifiedIdeograph3438', [], 'Display', None],
  [ 0x3439, 'CjkUnifiedIdeograph3439', [], 'Display', None],
  [ 0x343a, 'CjkUnifiedIdeograph343a', [], 'Display', None],
  [ 0x343b, 'CjkUnifiedIdeograph343b', [], 'Display', None],
  [ 0x343c, 'CjkUnifiedIdeograph343c', [], 'Display', None],
  [ 0x343d, 'CjkUnifiedIdeograph343d', [], 'Display', None],
  [ 0x343e, 'CjkUnifiedIdeograph343e', [], 'Display', None],
  [ 0x343f, 'CjkUnifiedIdeograph343f', [], 'Display', None],
  [ 0x3440, 'CjkUnifiedIdeograph3440', [], 'Display', None],
  [ 0x3441, 'CjkUnifiedIdeograph3441', [], 'Display', None],
  [ 0x3442, 'CjkUnifiedIdeograph3442', [], 'Display', None],
  [ 0x3443, 'CjkUnifiedIdeograph3443', [], 'Display', None],
  [ 0x3444, 'CjkUnifiedIdeograph3444', [], 'Display', None],
  [ 0x3445, 'CjkUnifiedIdeograph3445', [], 'Display', None],
  [ 0x3446, 'CjkUnifiedIdeograph3446', [], 'Display', None],
  [ 0x3447, 'CjkUnifiedIdeograph3447', [], 'Display', None],
  [ 0x3448, 'CjkUnifiedIdeograph3448', [], 'Display', None],
  [ 0x3449, 'CjkUnifiedIdeograph3449', [], 'Display', None],
  [ 0x344a, 'CjkUnifiedIdeograph344a', [], 'Display', None],
  [ 0x344b, 'CjkUnifiedIdeograph344b', [], 'Display', None],
  [ 0x344c, 'CjkUnifiedIdeograph344c', [], 'Display', None],
  [ 0x344d, 'CjkUnifiedIdeograph344d', [], 'Display', None],
  [ 0x344e, 'CjkUnifiedIdeograph344e', [], 'Display', None],
  [ 0x344f, 'CjkUnifiedIdeograph344f', [], 'Display', None],
  [ 0x3450, 'CjkUnifiedIdeograph3450', [], 'Display', None],
  [ 0x3451, 'CjkUnifiedIdeograph3451', [], 'Display', None],
  [ 0x3452, 'CjkUnifiedIdeograph3452', [], 'Display', None],
  [ 0x3453, 'CjkUnifiedIdeograph3453', [], 'Display', None],
  [ 0x3454, 'CjkUnifiedIdeograph3454', [], 'Display', None],
  [ 0x3455, 'CjkUnifiedIdeograph3455', [], 'Display', None],
  [ 0x3456, 'CjkUnifiedIdeograph3456', [], 'Display', None],
  [ 0x3457, 'CjkUnifiedIdeograph3457', [], 'Display', None],
  [ 0x3458, 'CjkUnifiedIdeograph3458', [], 'Display', None],
  [ 0x3459, 'CjkUnifiedIdeograph3459', [], 'Display', None],
  [ 0x345a, 'CjkUnifiedIdeograph345a', [], 'Display', None],
  [ 0x345b, 'CjkUnifiedIdeograph345b', [], 'Display', None],
  [ 0x345c, 'CjkUnifiedIdeograph345c', [], 'Display', None],
  [ 0x345d, 'CjkUnifiedIdeograph345d', [], 'Display', None],
  [ 0x345e, 'CjkUnifiedIdeograph345e', [], 'Display', None],
  [ 0x345f, 'CjkUnifiedIdeograph345f', [], 'Display', None],
  [ 0x3460, 'CjkUnifiedIdeograph3460', [], 'Display', None],
  [ 0x3461, 'CjkUnifiedIdeograph3461', [], 'Display', None],
  [ 0x3462, 'CjkUnifiedIdeograph3462', [], 'Display', None],
  [ 0x3463, 'CjkUnifiedIdeograph3463', [], 'Display', None],
  [ 0x3464, 'CjkUnifiedIdeograph3464', [], 'Display', None],
  [ 0x3465, 'CjkUnifiedIdeograph3465', [], 'Display', None],
  [ 0x3466, 'CjkUnifiedIdeograph3466', [], 'Display', None],
  [ 0x3467, 'CjkUnifiedIdeograph3467', [], 'Display', None],
  [ 0x3468, 'CjkUnifiedIdeograph3468', [], 'Display', None],
  [ 0x3469, 'CjkUnifiedIdeograph3469', [], 'Display', None],
  [ 0x346a, 'CjkUnifiedIdeograph346a', [], 'Display', None],
  [ 0x346b, 'CjkUnifiedIdeograph346b', [], 'Display', None],
  [ 0x346c, 'CjkUnifiedIdeograph346c', [], 'Display', None],
  [ 0x346d, 'CjkUnifiedIdeograph346d', [], 'Display', None],
  [ 0x346e, 'CjkUnifiedIdeograph346e', [], 'Display', None],
  [ 0x346f, 'CjkUnifiedIdeograph346f', [], 'Display', None],
  [ 0x3470, 'CjkUnifiedIdeograph3470', [], 'Display', None],
  [ 0x3471, 'CjkUnifiedIdeograph3471', [], 'Display', None],
  [ 0x3472, 'CjkUnifiedIdeograph3472', [], 'Display', None],
  [ 0x3473, 'CjkUnifiedIdeograph3473', [], 'Display', None],
  [ 0x3474, 'CjkUnifiedIdeograph3474', [], 'Display', None],
  [ 0x3475, 'CjkUnifiedIdeograph3475', [], 'Display', None],
  [ 0x3476, 'CjkUnifiedIdeograph3476', [], 'Display', None],
  [ 0x3477, 'CjkUnifiedIdeograph3477', [], 'Display', None],
  [ 0x3478, 'CjkUnifiedIdeograph3478', [], 'Display', None],
  [ 0x3479, 'CjkUnifiedIdeograph3479', [], 'Display', None],
  [ 0x347a, 'CjkUnifiedIdeograph347a', [], 'Display', None],
  [ 0x347b, 'CjkUnifiedIdeograph347b', [], 'Display', None],
  [ 0x347c, 'CjkUnifiedIdeograph347c', [], 'Display', None],
  [ 0x347d, 'CjkUnifiedIdeograph347d', [], 'Display', None],
  [ 0x347e, 'CjkUnifiedIdeograph347e', [], 'Display', None],
  [ 0x347f, 'CjkUnifiedIdeograph347f', [], 'Display', None],
  [ 0x3480, 'CjkUnifiedIdeograph3480', [], 'Display', None],
  [ 0x3481, 'CjkUnifiedIdeograph3481', [], 'Display', None],
  [ 0x3482, 'CjkUnifiedIdeograph3482', [], 'Display', None],
  [ 0x3483, 'CjkUnifiedIdeograph3483', [], 'Display', None],
  [ 0x3484, 'CjkUnifiedIdeograph3484', [], 'Display', None],
  [ 0x3485, 'CjkUnifiedIdeograph3485', [], 'Display', None],
  [ 0x3486, 'CjkUnifiedIdeograph3486', [], 'Display', None],
  [ 0x3487, 'CjkUnifiedIdeograph3487', [], 'Display', None],
  [ 0x3488, 'CjkUnifiedIdeograph3488', [], 'Display', None],
  [ 0x3489, 'CjkUnifiedIdeograph3489', [], 'Display', None],
  [ 0x348a, 'CjkUnifiedIdeograph348a', [], 'Display', None],
  [ 0x348b, 'CjkUnifiedIdeograph348b', [], 'Display', None],
  [ 0x348c, 'CjkUnifiedIdeograph348c', [], 'Display', None],
  [ 0x348d, 'CjkUnifiedIdeograph348d', [], 'Display', None],
  [ 0x348e, 'CjkUnifiedIdeograph348e', [], 'Display', None],
  [ 0x348f, 'CjkUnifiedIdeograph348f', [], 'Display', None],
  [ 0x3490, 'CjkUnifiedIdeograph3490', [], 'Display', None],
  [ 0x3491, 'CjkUnifiedIdeograph3491', [], 'Display', None],
  [ 0x3492, 'CjkUnifiedIdeograph3492', [], 'Display', None],
  [ 0x3493, 'CjkUnifiedIdeograph3493', [], 'Display', None],
  [ 0x3494, 'CjkUnifiedIdeograph3494', [], 'Display', None],
  [ 0x3495, 'CjkUnifiedIdeograph3495', [], 'Display', None],
  [ 0x3496, 'CjkUnifiedIdeograph3496', [], 'Display', None],
  [ 0x3497, 'CjkUnifiedIdeograph3497', [], 'Display', None],
  [ 0x3498, 'CjkUnifiedIdeograph3498', [], 'Display', None],
  [ 0x3499, 'CjkUnifiedIdeograph3499', [], 'Display', None],
  [ 0x349a, 'CjkUnifiedIdeograph349a', [], 'Display', None],
  [ 0x349b, 'CjkUnifiedIdeograph349b', [], 'Display', None],
  [ 0x349c, 'CjkUnifiedIdeograph349c', [], 'Display', None],
  [ 0x349d, 'CjkUnifiedIdeograph349d', [], 'Display', None],
  [ 0x349e, 'CjkUnifiedIdeograph349e', [], 'Display', None],
  [ 0x349f, 'CjkUnifiedIdeograph349f', [], 'Display', None],
  [ 0x34a0, 'CjkUnifiedIdeograph34a0', [], 'Display', None],
  [ 0x34a1, 'CjkUnifiedIdeograph34a1', [], 'Display', None],
  [ 0x34a2, 'CjkUnifiedIdeograph34a2', [], 'Display', None],
  [ 0x34a3, 'CjkUnifiedIdeograph34a3', [], 'Display', None],
  [ 0x34a4, 'CjkUnifiedIdeograph34a4', [], 'Display', None],
  [ 0x34a5, 'CjkUnifiedIdeograph34a5', [], 'Display', None],
  [ 0x34a6, 'CjkUnifiedIdeograph34a6', [], 'Display', None],
  [ 0x34a7, 'CjkUnifiedIdeograph34a7', [], 'Display', None],
  [ 0x34a8, 'CjkUnifiedIdeograph34a8', [], 'Display', None],
  [ 0x34a9, 'CjkUnifiedIdeograph34a9', [], 'Display', None],
  [ 0x34aa, 'CjkUnifiedIdeograph34aa', [], 'Display', None],
  [ 0x34ab, 'CjkUnifiedIdeograph34ab', [], 'Display', None],
  [ 0x34ac, 'CjkUnifiedIdeograph34ac', [], 'Display', None],
  [ 0x34ad, 'CjkUnifiedIdeograph34ad', [], 'Display', None],
  [ 0x34ae, 'CjkUnifiedIdeograph34ae', [], 'Display', None],
  [ 0x34af, 'CjkUnifiedIdeograph34af', [], 'Display', None],
  [ 0x34b0, 'CjkUnifiedIdeograph34b0', [], 'Display', None],
  [ 0x34b1, 'CjkUnifiedIdeograph34b1', [], 'Display', None],
  [ 0x34b2, 'CjkUnifiedIdeograph34b2', [], 'Display', None],
  [ 0x34b3, 'CjkUnifiedIdeograph34b3', [], 'Display', None],
  [ 0x34b4, 'CjkUnifiedIdeograph34b4', [], 'Display', None],
  [ 0x34b5, 'CjkUnifiedIdeograph34b5', [], 'Display', None],
  [ 0x34b6, 'CjkUnifiedIdeograph34b6', [], 'Display', None],
  [ 0x34b7, 'CjkUnifiedIdeograph34b7', [], 'Display', None],
  [ 0x34b8, 'CjkUnifiedIdeograph34b8', [], 'Display', None],
  [ 0x34b9, 'CjkUnifiedIdeograph34b9', [], 'Display', None],
  [ 0x34ba, 'CjkUnifiedIdeograph34ba', [], 'Display', None],
  [ 0x34bb, 'CjkUnifiedIdeograph34bb', [], 'Display', None],
  [ 0x34bc, 'CjkUnifiedIdeograph34bc', [], 'Display', None],
  [ 0x34bd, 'CjkUnifiedIdeograph34bd', [], 'Display', None],
  [ 0x34be, 'CjkUnifiedIdeograph34be', [], 'Display', None],
  [ 0x34bf, 'CjkUnifiedIdeograph34bf', [], 'Display', None],
  [ 0x34c0, 'CjkUnifiedIdeograph34c0', [], 'Display', None],
  [ 0x34c1, 'CjkUnifiedIdeograph34c1', [], 'Display', None],
  [ 0x34c2, 'CjkUnifiedIdeograph34c2', [], 'Display', None],
  [ 0x34c3, 'CjkUnifiedIdeograph34c3', [], 'Display', None],
  [ 0x34c4, 'CjkUnifiedIdeograph34c4', [], 'Display', None],
  [ 0x34c5, 'CjkUnifiedIdeograph34c5', [], 'Display', None],
  [ 0x34c6, 'CjkUnifiedIdeograph34c6', [], 'Display', None],
  [ 0x34c7, 'CjkUnifiedIdeograph34c7', [], 'Display', None],
  [ 0x34c8, 'CjkUnifiedIdeograph34c8', [], 'Display', None],
  [ 0x34c9, 'CjkUnifiedIdeograph34c9', [], 'Display', None],
  [ 0x34ca, 'CjkUnifiedIdeograph34ca', [], 'Display', None],
  [ 0x34cb, 'CjkUnifiedIdeograph34cb', [], 'Display', None],
  [ 0x34cc, 'CjkUnifiedIdeograph34cc', [], 'Display', None],
  [ 0x34cd, 'CjkUnifiedIdeograph34cd', [], 'Display', None],
  [ 0x34ce, 'CjkUnifiedIdeograph34ce', [], 'Display', None],
  [ 0x34cf, 'CjkUnifiedIdeograph34cf', [], 'Display', None],
  [ 0x34d0, 'CjkUnifiedIdeograph34d0', [], 'Display', None],
  [ 0x34d1, 'CjkUnifiedIdeograph34d1', [], 'Display', None],
  [ 0x34d2, 'CjkUnifiedIdeograph34d2', [], 'Display', None],
  [ 0x34d3, 'CjkUnifiedIdeograph34d3', [], 'Display', None],
  [ 0x34d4, 'CjkUnifiedIdeograph34d4', [], 'Display', None],
  [ 0x34d5, 'CjkUnifiedIdeograph34d5', [], 'Display', None],
  [ 0x34d6, 'CjkUnifiedIdeograph34d6', [], 'Display', None],
  [ 0x34d7, 'CjkUnifiedIdeograph34d7', [], 'Display', None],
  [ 0x34d8, 'CjkUnifiedIdeograph34d8', [], 'Display', None],
  [ 0x34d9, 'CjkUnifiedIdeograph34d9', [], 'Display', None],
  [ 0x34da, 'CjkUnifiedIdeograph34da', [], 'Display', None],
  [ 0x34db, 'CjkUnifiedIdeograph34db', [], 'Display', None],
  [ 0x34dc, 'CjkUnifiedIdeograph34dc', [], 'Display', None],
  [ 0x34dd, 'CjkUnifiedIdeograph34dd', [], 'Display', None],
  [ 0x34de, 'CjkUnifiedIdeograph34de', [], 'Display', None],
  [ 0x34df, 'CjkUnifiedIdeograph34df', [], 'Display', None],
  [ 0x34e0, 'CjkUnifiedIdeograph34e0', [], 'Display', None],
  [ 0x34e1, 'CjkUnifiedIdeograph34e1', [], 'Display', None],
  [ 0x34e2, 'CjkUnifiedIdeograph34e2', [], 'Display', None],
  [ 0x34e3, 'CjkUnifiedIdeograph34e3', [], 'Display', None],
  [ 0x34e4, 'CjkUnifiedIdeograph34e4', [], 'Display', None],
  [ 0x34e5, 'CjkUnifiedIdeograph34e5', [], 'Display', None],
  [ 0x34e6, 'CjkUnifiedIdeograph34e6', [], 'Display', None],
  [ 0x34e7, 'CjkUnifiedIdeograph34e7', [], 'Display', None],
  [ 0x34e8, 'CjkUnifiedIdeograph34e8', [], 'Display', None],
  [ 0x34e9, 'CjkUnifiedIdeograph34e9', [], 'Display', None],
  [ 0x34ea, 'CjkUnifiedIdeograph34ea', [], 'Display', None],
  [ 0x34eb, 'CjkUnifiedIdeograph34eb', [], 'Display', None],
  [ 0x34ec, 'CjkUnifiedIdeograph34ec', [], 'Display', None],
  [ 0x34ed, 'CjkUnifiedIdeograph34ed', [], 'Display', None],
  [ 0x34ee, 'CjkUnifiedIdeograph34ee', [], 'Display', None],
  [ 0x34ef, 'CjkUnifiedIdeograph34ef', [], 'Display', None],
  [ 0x34f0, 'CjkUnifiedIdeograph34f0', [], 'Display', None],
  [ 0x34f1, 'CjkUnifiedIdeograph34f1', [], 'Display', None],
  [ 0x34f2, 'CjkUnifiedIdeograph34f2', [], 'Display', None],
  [ 0x34f3, 'CjkUnifiedIdeograph34f3', [], 'Display', None],
  [ 0x34f4, 'CjkUnifiedIdeograph34f4', [], 'Display', None],
  [ 0x34f5, 'CjkUnifiedIdeograph34f5', [], 'Display', None],
  [ 0x34f6, 'CjkUnifiedIdeograph34f6', [], 'Display', None],
  [ 0x34f7, 'CjkUnifiedIdeograph34f7', [], 'Display', None],
  [ 0x34f8, 'CjkUnifiedIdeograph34f8', [], 'Display', None],
  [ 0x34f9, 'CjkUnifiedIdeograph34f9', [], 'Display', None],
  [ 0x34fa, 'CjkUnifiedIdeograph34fa', [], 'Display', None],
  [ 0x34fb, 'CjkUnifiedIdeograph34fb', [], 'Display', None],
  [ 0x34fc, 'CjkUnifiedIdeograph34fc', [], 'Display', None],
  [ 0x34fd, 'CjkUnifiedIdeograph34fd', [], 'Display', None],
  [ 0x34fe, 'CjkUnifiedIdeograph34fe', [], 'Display', None],
  [ 0x34ff, 'CjkUnifiedIdeograph34ff', [], 'Display', None],
  [ 0x3500, 'CjkUnifiedIdeograph3500', [], 'Display', None],
  [ 0x3501, 'CjkUnifiedIdeograph3501', [], 'Display', None],
  [ 0x3502, 'CjkUnifiedIdeograph3502', [], 'Display', None],
  [ 0x3503, 'CjkUnifiedIdeograph3503', [], 'Display', None],
  [ 0x3504, 'CjkUnifiedIdeograph3504', [], 'Display', None],
  [ 0x3505, 'CjkUnifiedIdeograph3505', [], 'Display', None],
  [ 0x3506, 'CjkUnifiedIdeograph3506', [], 'Display', None],
  [ 0x3507, 'CjkUnifiedIdeograph3507', [], 'Display', None],
  [ 0x3508, 'CjkUnifiedIdeograph3508', [], 'Display', None],
  [ 0x3509, 'CjkUnifiedIdeograph3509', [], 'Display', None],
  [ 0x350a, 'CjkUnifiedIdeograph350a', [], 'Display', None],
  [ 0x350b, 'CjkUnifiedIdeograph350b', [], 'Display', None],
  [ 0x350c, 'CjkUnifiedIdeograph350c', [], 'Display', None],
  [ 0x350d, 'CjkUnifiedIdeograph350d', [], 'Display', None],
  [ 0x350e, 'CjkUnifiedIdeograph350e', [], 'Display', None],
  [ 0x350f, 'CjkUnifiedIdeograph350f', [], 'Display', None],
  [ 0x3510, 'CjkUnifiedIdeograph3510', [], 'Display', None],
  [ 0x3511, 'CjkUnifiedIdeograph3511', [], 'Display', None],
  [ 0x3512, 'CjkUnifiedIdeograph3512', [], 'Display', None],
  [ 0x3513, 'CjkUnifiedIdeograph3513', [], 'Display', None],
  [ 0x3514, 'CjkUnifiedIdeograph3514', [], 'Display', None],
  [ 0x3515, 'CjkUnifiedIdeograph3515', [], 'Display', None],
  [ 0x3516, 'CjkUnifiedIdeograph3516', [], 'Display', None],
  [ 0x3517, 'CjkUnifiedIdeograph3517', [], 'Display', None],
  [ 0x3518, 'CjkUnifiedIdeograph3518', [], 'Display', None],
  [ 0x3519, 'CjkUnifiedIdeograph3519', [], 'Display', None],
  [ 0x351a, 'CjkUnifiedIdeograph351a', [], 'Display', None],
  [ 0x351b, 'CjkUnifiedIdeograph351b', [], 'Display', None],
  [ 0x351c, 'CjkUnifiedIdeograph351c', [], 'Display', None],
  [ 0x351d, 'CjkUnifiedIdeograph351d', [], 'Display', None],
  [ 0x351e, 'CjkUnifiedIdeograph351e', [], 'Display', None],
  [ 0x351f, 'CjkUnifiedIdeograph351f', [], 'Display', None],
  [ 0x3520, 'CjkUnifiedIdeograph3520', [], 'Display', None],
  [ 0x3521, 'CjkUnifiedIdeograph3521', [], 'Display', None],
  [ 0x3522, 'CjkUnifiedIdeograph3522', [], 'Display', None],
  [ 0x3523, 'CjkUnifiedIdeograph3523', [], 'Display', None],
  [ 0x3524, 'CjkUnifiedIdeograph3524', [], 'Display', None],
  [ 0x3525, 'CjkUnifiedIdeograph3525', [], 'Display', None],
  [ 0x3526, 'CjkUnifiedIdeograph3526', [], 'Display', None],
  [ 0x3527, 'CjkUnifiedIdeograph3527', [], 'Display', None],
  [ 0x3528, 'CjkUnifiedIdeograph3528', [], 'Display', None],
  [ 0x3529, 'CjkUnifiedIdeograph3529', [], 'Display', None],
  [ 0x352a, 'CjkUnifiedIdeograph352a', [], 'Display', None],
  [ 0x352b, 'CjkUnifiedIdeograph352b', [], 'Display', None],
  [ 0x352c, 'CjkUnifiedIdeograph352c', [], 'Display', None],
  [ 0x352d, 'CjkUnifiedIdeograph352d', [], 'Display', None],
  [ 0x352e, 'CjkUnifiedIdeograph352e', [], 'Display', None],
  [ 0x352f, 'CjkUnifiedIdeograph352f', [], 'Display', None],
  [ 0x3530, 'CjkUnifiedIdeograph3530', [], 'Display', None],
  [ 0x3531, 'CjkUnifiedIdeograph3531', [], 'Display', None],
  [ 0x3532, 'CjkUnifiedIdeograph3532', [], 'Display', None],
  [ 0x3533, 'CjkUnifiedIdeograph3533', [], 'Display', None],
  [ 0x3534, 'CjkUnifiedIdeograph3534', [], 'Display', None],
  [ 0x3535, 'CjkUnifiedIdeograph3535', [], 'Display', None],
  [ 0x3536, 'CjkUnifiedIdeograph3536', [], 'Display', None],
  [ 0x3537, 'CjkUnifiedIdeograph3537', [], 'Display', None],
  [ 0x3538, 'CjkUnifiedIdeograph3538', [], 'Display', None],
  [ 0x3539, 'CjkUnifiedIdeograph3539', [], 'Display', None],
  [ 0x353a, 'CjkUnifiedIdeograph353a', [], 'Display', None],
  [ 0x353b, 'CjkUnifiedIdeograph353b', [], 'Display', None],
  [ 0x353c, 'CjkUnifiedIdeograph353c', [], 'Display', None],
  [ 0x353d, 'CjkUnifiedIdeograph353d', [], 'Display', None],
  [ 0x353e, 'CjkUnifiedIdeograph353e', [], 'Display', None],
  [ 0x353f, 'CjkUnifiedIdeograph353f', [], 'Display', None],
  [ 0x3540, 'CjkUnifiedIdeograph3540', [], 'Display', None],
  [ 0x3541, 'CjkUnifiedIdeograph3541', [], 'Display', None],
  [ 0x3542, 'CjkUnifiedIdeograph3542', [], 'Display', None],
  [ 0x3543, 'CjkUnifiedIdeograph3543', [], 'Display', None],
  [ 0x3544, 'CjkUnifiedIdeograph3544', [], 'Display', None],
  [ 0x3545, 'CjkUnifiedIdeograph3545', [], 'Display', None],
  [ 0x3546, 'CjkUnifiedIdeograph3546', [], 'Display', None],
  [ 0x3547, 'CjkUnifiedIdeograph3547', [], 'Display', None],
  [ 0x3548, 'CjkUnifiedIdeograph3548', [], 'Display', None],
  [ 0x3549, 'CjkUnifiedIdeograph3549', [], 'Display', None],
  [ 0x354a, 'CjkUnifiedIdeograph354a', [], 'Display', None],
  [ 0x354b, 'CjkUnifiedIdeograph354b', [], 'Display', None],
  [ 0x354c, 'CjkUnifiedIdeograph354c', [], 'Display', None],
  [ 0x354d, 'CjkUnifiedIdeograph354d', [], 'Display', None],
  [ 0x354e, 'CjkUnifiedIdeograph354e', [], 'Display', None],
  [ 0x354f, 'CjkUnifiedIdeograph354f', [], 'Display', None],
  [ 0x3550, 'CjkUnifiedIdeograph3550', [], 'Display', None],
  [ 0x3551, 'CjkUnifiedIdeograph3551', [], 'Display', None],
  [ 0x3552, 'CjkUnifiedIdeograph3552', [], 'Display', None],
  [ 0x3553, 'CjkUnifiedIdeograph3553', [], 'Display', None],
  [ 0x3554, 'CjkUnifiedIdeograph3554', [], 'Display', None],
  [ 0x3555, 'CjkUnifiedIdeograph3555', [], 'Display', None],
  [ 0x3556, 'CjkUnifiedIdeograph3556', [], 'Display', None],
  [ 0x3557, 'CjkUnifiedIdeograph3557', [], 'Display', None],
  [ 0x3558, 'CjkUnifiedIdeograph3558', [], 'Display', None],
  [ 0x3559, 'CjkUnifiedIdeograph3559', [], 'Display', None],
  [ 0x355a, 'CjkUnifiedIdeograph355a', [], 'Display', None],
  [ 0x355b, 'CjkUnifiedIdeograph355b', [], 'Display', None],
  [ 0x355c, 'CjkUnifiedIdeograph355c', [], 'Display', None],
  [ 0x355d, 'CjkUnifiedIdeograph355d', [], 'Display', None],
  [ 0x355e, 'CjkUnifiedIdeograph355e', [], 'Display', None],
  [ 0x355f, 'CjkUnifiedIdeograph355f', [], 'Display', None],
  [ 0x3560, 'CjkUnifiedIdeograph3560', [], 'Display', None],
  [ 0x3561, 'CjkUnifiedIdeograph3561', [], 'Display', None],
  [ 0x3562, 'CjkUnifiedIdeograph3562', [], 'Display', None],
  [ 0x3563, 'CjkUnifiedIdeograph3563', [], 'Display', None],
  [ 0x3564, 'CjkUnifiedIdeograph3564', [], 'Display', None],
  [ 0x3565, 'CjkUnifiedIdeograph3565', [], 'Display', None],
  [ 0x3566, 'CjkUnifiedIdeograph3566', [], 'Display', None],
  [ 0x3567, 'CjkUnifiedIdeograph3567', [], 'Display', None],
  [ 0x3568, 'CjkUnifiedIdeograph3568', [], 'Display', None],
  [ 0x3569, 'CjkUnifiedIdeograph3569', [], 'Display', None],
  [ 0x356a, 'CjkUnifiedIdeograph356a', [], 'Display', None],
  [ 0x356b, 'CjkUnifiedIdeograph356b', [], 'Display', None],
  [ 0x356c, 'CjkUnifiedIdeograph356c', [], 'Display', None],
  [ 0x356d, 'CjkUnifiedIdeograph356d', [], 'Display', None],
  [ 0x356e, 'CjkUnifiedIdeograph356e', [], 'Display', None],
  [ 0x356f, 'CjkUnifiedIdeograph356f', [], 'Display', None],
  [ 0x3570, 'CjkUnifiedIdeograph3570', [], 'Display', None],
  [ 0x3571, 'CjkUnifiedIdeograph3571', [], 'Display', None],
  [ 0x3572, 'CjkUnifiedIdeograph3572', [], 'Display', None],
  [ 0x3573, 'CjkUnifiedIdeograph3573', [], 'Display', None],
  [ 0x3574, 'CjkUnifiedIdeograph3574', [], 'Display', None],
  [ 0x3575, 'CjkUnifiedIdeograph3575', [], 'Display', None],
  [ 0x3576, 'CjkUnifiedIdeograph3576', [], 'Display', None],
  [ 0x3577, 'CjkUnifiedIdeograph3577', [], 'Display', None],
  [ 0x3578, 'CjkUnifiedIdeograph3578', [], 'Display', None],
  [ 0x3579, 'CjkUnifiedIdeograph3579', [], 'Display', None],
  [ 0x357a, 'CjkUnifiedIdeograph357a', [], 'Display', None],
  [ 0x357b, 'CjkUnifiedIdeograph357b', [], 'Display', None],
  [ 0x357c, 'CjkUnifiedIdeograph357c', [], 'Display', None],
  [ 0x357d, 'CjkUnifiedIdeograph357d', [], 'Display', None],
  [ 0x357e, 'CjkUnifiedIdeograph357e', [], 'Display', None],
  [ 0x357f, 'CjkUnifiedIdeograph357f', [], 'Display', None],
  [ 0x3580, 'CjkUnifiedIdeograph3580', [], 'Display', None],
  [ 0x3581, 'CjkUnifiedIdeograph3581', [], 'Display', None],
  [ 0x3582, 'CjkUnifiedIdeograph3582', [], 'Display', None],
  [ 0x3583, 'CjkUnifiedIdeograph3583', [], 'Display', None],
  [ 0x3584, 'CjkUnifiedIdeograph3584', [], 'Display', None],
  [ 0x3585, 'CjkUnifiedIdeograph3585', [], 'Display', None],
  [ 0x3586, 'CjkUnifiedIdeograph3586', [], 'Display', None],
  [ 0x3587, 'CjkUnifiedIdeograph3587', [], 'Display', None],
  [ 0x3588, 'CjkUnifiedIdeograph3588', [], 'Display', None],
  [ 0x3589, 'CjkUnifiedIdeograph3589', [], 'Display', None],
  [ 0x358a, 'CjkUnifiedIdeograph358a', [], 'Display', None],
  [ 0x358b, 'CjkUnifiedIdeograph358b', [], 'Display', None],
  [ 0x358c, 'CjkUnifiedIdeograph358c', [], 'Display', None],
  [ 0x358d, 'CjkUnifiedIdeograph358d', [], 'Display', None],
  [ 0x358e, 'CjkUnifiedIdeograph358e', [], 'Display', None],
  [ 0x358f, 'CjkUnifiedIdeograph358f', [], 'Display', None],
  [ 0x3590, 'CjkUnifiedIdeograph3590', [], 'Display', None],
  [ 0x3591, 'CjkUnifiedIdeograph3591', [], 'Display', None],
  [ 0x3592, 'CjkUnifiedIdeograph3592', [], 'Display', None],
  [ 0x3593, 'CjkUnifiedIdeograph3593', [], 'Display', None],
  [ 0x3594, 'CjkUnifiedIdeograph3594', [], 'Display', None],
  [ 0x3595, 'CjkUnifiedIdeograph3595', [], 'Display', None],
  [ 0x3596, 'CjkUnifiedIdeograph3596', [], 'Display', None],
  [ 0x3597, 'CjkUnifiedIdeograph3597', [], 'Display', None],
  [ 0x3598, 'CjkUnifiedIdeograph3598', [], 'Display', None],
  [ 0x3599, 'CjkUnifiedIdeograph3599', [], 'Display', None],
  [ 0x359a, 'CjkUnifiedIdeograph359a', [], 'Display', None],
  [ 0x359b, 'CjkUnifiedIdeograph359b', [], 'Display', None],
  [ 0x359c, 'CjkUnifiedIdeograph359c', [], 'Display', None],
  [ 0x359d, 'CjkUnifiedIdeograph359d', [], 'Display', None],
  [ 0x359e, 'CjkUnifiedIdeograph359e', [], 'Display', None],
  [ 0x359f, 'CjkUnifiedIdeograph359f', [], 'Display', None],
  [ 0x35a0, 'CjkUnifiedIdeograph35a0', [], 'Display', None],
  [ 0x35a1, 'CjkUnifiedIdeograph35a1', [], 'Display', None],
  [ 0x35a2, 'CjkUnifiedIdeograph35a2', [], 'Display', None],
  [ 0x35a3, 'CjkUnifiedIdeograph35a3', [], 'Display', None],
  [ 0x35a4, 'CjkUnifiedIdeograph35a4', [], 'Display', None],
  [ 0x35a5, 'CjkUnifiedIdeograph35a5', [], 'Display', None],
  [ 0x35a6, 'CjkUnifiedIdeograph35a6', [], 'Display', None],
  [ 0x35a7, 'CjkUnifiedIdeograph35a7', [], 'Display', None],
  [ 0x35a8, 'CjkUnifiedIdeograph35a8', [], 'Display', None],
  [ 0x35a9, 'CjkUnifiedIdeograph35a9', [], 'Display', None],
  [ 0x35aa, 'CjkUnifiedIdeograph35aa', [], 'Display', None],
  [ 0x35ab, 'CjkUnifiedIdeograph35ab', [], 'Display', None],
  [ 0x35ac, 'CjkUnifiedIdeograph35ac', [], 'Display', None],
  [ 0x35ad, 'CjkUnifiedIdeograph35ad', [], 'Display', None],
  [ 0x35ae, 'CjkUnifiedIdeograph35ae', [], 'Display', None],
  [ 0x35af, 'CjkUnifiedIdeograph35af', [], 'Display', None],
  [ 0x35b0, 'CjkUnifiedIdeograph35b0', [], 'Display', None],
  [ 0x35b1, 'CjkUnifiedIdeograph35b1', [], 'Display', None],
  [ 0x35b2, 'CjkUnifiedIdeograph35b2', [], 'Display', None],
  [ 0x35b3, 'CjkUnifiedIdeograph35b3', [], 'Display', None],
  [ 0x35b4, 'CjkUnifiedIdeograph35b4', [], 'Display', None],
  [ 0x35b5, 'CjkUnifiedIdeograph35b5', [], 'Display', None],
  [ 0x35b6, 'CjkUnifiedIdeograph35b6', [], 'Display', None],
  [ 0x35b7, 'CjkUnifiedIdeograph35b7', [], 'Display', None],
  [ 0x35b8, 'CjkUnifiedIdeograph35b8', [], 'Display', None],
  [ 0x35b9, 'CjkUnifiedIdeograph35b9', [], 'Display', None],
  [ 0x35ba, 'CjkUnifiedIdeograph35ba', [], 'Display', None],
  [ 0x35bb, 'CjkUnifiedIdeograph35bb', [], 'Display', None],
  [ 0x35bc, 'CjkUnifiedIdeograph35bc', [], 'Display', None],
  [ 0x35bd, 'CjkUnifiedIdeograph35bd', [], 'Display', None],
  [ 0x35be, 'CjkUnifiedIdeograph35be', [], 'Display', None],
  [ 0x35bf, 'CjkUnifiedIdeograph35bf', [], 'Display', None],
  [ 0x35c0, 'CjkUnifiedIdeograph35c0', [], 'Display', None],
  [ 0x35c1, 'CjkUnifiedIdeograph35c1', [], 'Display', None],
  [ 0x35c2, 'CjkUnifiedIdeograph35c2', [], 'Display', None],
  [ 0x35c3, 'CjkUnifiedIdeograph35c3', [], 'Display', None],
  [ 0x35c4, 'CjkUnifiedIdeograph35c4', [], 'Display', None],
  [ 0x35c5, 'CjkUnifiedIdeograph35c5', [], 'Display', None],
  [ 0x35c6, 'CjkUnifiedIdeograph35c6', [], 'Display', None],
  [ 0x35c7, 'CjkUnifiedIdeograph35c7', [], 'Display', None],
  [ 0x35c8, 'CjkUnifiedIdeograph35c8', [], 'Display', None],
  [ 0x35c9, 'CjkUnifiedIdeograph35c9', [], 'Display', None],
  [ 0x35ca, 'CjkUnifiedIdeograph35ca', [], 'Display', None],
  [ 0x35cb, 'CjkUnifiedIdeograph35cb', [], 'Display', None],
  [ 0x35cc, 'CjkUnifiedIdeograph35cc', [], 'Display', None],
  [ 0x35cd, 'CjkUnifiedIdeograph35cd', [], 'Display', None],
  [ 0x35ce, 'CjkUnifiedIdeograph35ce', [], 'Display', None],
  [ 0x35cf, 'CjkUnifiedIdeograph35cf', [], 'Display', None],
  [ 0x35d0, 'CjkUnifiedIdeograph35d0', [], 'Display', None],
  [ 0x35d1, 'CjkUnifiedIdeograph35d1', [], 'Display', None],
  [ 0x35d2, 'CjkUnifiedIdeograph35d2', [], 'Display', None],
  [ 0x35d3, 'CjkUnifiedIdeograph35d3', [], 'Display', None],
  [ 0x35d4, 'CjkUnifiedIdeograph35d4', [], 'Display', None],
  [ 0x35d5, 'CjkUnifiedIdeograph35d5', [], 'Display', None],
  [ 0x35d6, 'CjkUnifiedIdeograph35d6', [], 'Display', None],
  [ 0x35d7, 'CjkUnifiedIdeograph35d7', [], 'Display', None],
  [ 0x35d8, 'CjkUnifiedIdeograph35d8', [], 'Display', None],
  [ 0x35d9, 'CjkUnifiedIdeograph35d9', [], 'Display', None],
  [ 0x35da, 'CjkUnifiedIdeograph35da', [], 'Display', None],
  [ 0x35db, 'CjkUnifiedIdeograph35db', [], 'Display', None],
  [ 0x35dc, 'CjkUnifiedIdeograph35dc', [], 'Display', None],
  [ 0x35dd, 'CjkUnifiedIdeograph35dd', [], 'Display', None],
  [ 0x35de, 'CjkUnifiedIdeograph35de', [], 'Display', None],
  [ 0x35df, 'CjkUnifiedIdeograph35df', [], 'Display', None],
  [ 0x35e0, 'CjkUnifiedIdeograph35e0', [], 'Display', None],
  [ 0x35e1, 'CjkUnifiedIdeograph35e1', [], 'Display', None],
  [ 0x35e2, 'CjkUnifiedIdeograph35e2', [], 'Display', None],
  [ 0x35e3, 'CjkUnifiedIdeograph35e3', [], 'Display', None],
  [ 0x35e4, 'CjkUnifiedIdeograph35e4', [], 'Display', None],
  [ 0x35e5, 'CjkUnifiedIdeograph35e5', [], 'Display', None],
  [ 0x35e6, 'CjkUnifiedIdeograph35e6', [], 'Display', None],
  [ 0x35e7, 'CjkUnifiedIdeograph35e7', [], 'Display', None],
  [ 0x35e8, 'CjkUnifiedIdeograph35e8', [], 'Display', None],
  [ 0x35e9, 'CjkUnifiedIdeograph35e9', [], 'Display', None],
  [ 0x35ea, 'CjkUnifiedIdeograph35ea', [], 'Display', None],
  [ 0x35eb, 'CjkUnifiedIdeograph35eb', [], 'Display', None],
  [ 0x35ec, 'CjkUnifiedIdeograph35ec', [], 'Display', None],
  [ 0x35ed, 'CjkUnifiedIdeograph35ed', [], 'Display', None],
  [ 0x35ee, 'CjkUnifiedIdeograph35ee', [], 'Display', None],
  [ 0x35ef, 'CjkUnifiedIdeograph35ef', [], 'Display', None],
  [ 0x35f0, 'CjkUnifiedIdeograph35f0', [], 'Display', None],
  [ 0x35f1, 'CjkUnifiedIdeograph35f1', [], 'Display', None],
  [ 0x35f2, 'CjkUnifiedIdeograph35f2', [], 'Display', None],
  [ 0x35f3, 'CjkUnifiedIdeograph35f3', [], 'Display', None],
  [ 0x35f4, 'CjkUnifiedIdeograph35f4', [], 'Display', None],
  [ 0x35f5, 'CjkUnifiedIdeograph35f5', [], 'Display', None],
  [ 0x35f6, 'CjkUnifiedIdeograph35f6', [], 'Display', None],
  [ 0x35f7, 'CjkUnifiedIdeograph35f7', [], 'Display', None],
  [ 0x35f8, 'CjkUnifiedIdeograph35f8', [], 'Display', None],
  [ 0x35f9, 'CjkUnifiedIdeograph35f9', [], 'Display', None],
  [ 0x35fa, 'CjkUnifiedIdeograph35fa', [], 'Display', None],
  [ 0x35fb, 'CjkUnifiedIdeograph35fb', [], 'Display', None],
  [ 0x35fc, 'CjkUnifiedIdeograph35fc', [], 'Display', None],
  [ 0x35fd, 'CjkUnifiedIdeograph35fd', [], 'Display', None],
  [ 0x35fe, 'CjkUnifiedIdeograph35fe', [], 'Display', None],
  [ 0x35ff, 'CjkUnifiedIdeograph35ff', [], 'Display', None],
  [ 0x3600, 'CjkUnifiedIdeograph3600', [], 'Display', None],
  [ 0x3601, 'CjkUnifiedIdeograph3601', [], 'Display', None],
  [ 0x3602, 'CjkUnifiedIdeograph3602', [], 'Display', None],
  [ 0x3603, 'CjkUnifiedIdeograph3603', [], 'Display', None],
  [ 0x3604, 'CjkUnifiedIdeograph3604', [], 'Display', None],
  [ 0x3605, 'CjkUnifiedIdeograph3605', [], 'Display', None],
  [ 0x3606, 'CjkUnifiedIdeograph3606', [], 'Display', None],
  [ 0x3607, 'CjkUnifiedIdeograph3607', [], 'Display', None],
  [ 0x3608, 'CjkUnifiedIdeograph3608', [], 'Display', None],
  [ 0x3609, 'CjkUnifiedIdeograph3609', [], 'Display', None],
  [ 0x360a, 'CjkUnifiedIdeograph360a', [], 'Display', None],
  [ 0x360b, 'CjkUnifiedIdeograph360b', [], 'Display', None],
  [ 0x360c, 'CjkUnifiedIdeograph360c', [], 'Display', None],
  [ 0x360d, 'CjkUnifiedIdeograph360d', [], 'Display', None],
  [ 0x360e, 'CjkUnifiedIdeograph360e', [], 'Display', None],
  [ 0x360f, 'CjkUnifiedIdeograph360f', [], 'Display', None],
  [ 0x3610, 'CjkUnifiedIdeograph3610', [], 'Display', None],
  [ 0x3611, 'CjkUnifiedIdeograph3611', [], 'Display', None],
  [ 0x3612, 'CjkUnifiedIdeograph3612', [], 'Display', None],
  [ 0x3613, 'CjkUnifiedIdeograph3613', [], 'Display', None],
  [ 0x3614, 'CjkUnifiedIdeograph3614', [], 'Display', None],
  [ 0x3615, 'CjkUnifiedIdeograph3615', [], 'Display', None],
  [ 0x3616, 'CjkUnifiedIdeograph3616', [], 'Display', None],
  [ 0x3617, 'CjkUnifiedIdeograph3617', [], 'Display', None],
  [ 0x3618, 'CjkUnifiedIdeograph3618', [], 'Display', None],
  [ 0x3619, 'CjkUnifiedIdeograph3619', [], 'Display', None],
  [ 0x361a, 'CjkUnifiedIdeograph361a', [], 'Display', None],
  [ 0x361b, 'CjkUnifiedIdeograph361b', [], 'Display', None],
  [ 0x361c, 'CjkUnifiedIdeograph361c', [], 'Display', None],
  [ 0x361d, 'CjkUnifiedIdeograph361d', [], 'Display', None],
  [ 0x361e, 'CjkUnifiedIdeograph361e', [], 'Display', None],
  [ 0x361f, 'CjkUnifiedIdeograph361f', [], 'Display', None],
  [ 0x3620, 'CjkUnifiedIdeograph3620', [], 'Display', None],
  [ 0x3621, 'CjkUnifiedIdeograph3621', [], 'Display', None],
  [ 0x3622, 'CjkUnifiedIdeograph3622', [], 'Display', None],
  [ 0x3623, 'CjkUnifiedIdeograph3623', [], 'Display', None],
  [ 0x3624, 'CjkUnifiedIdeograph3624', [], 'Display', None],
  [ 0x3625, 'CjkUnifiedIdeograph3625', [], 'Display', None],
  [ 0x3626, 'CjkUnifiedIdeograph3626', [], 'Display', None],
  [ 0x3627, 'CjkUnifiedIdeograph3627', [], 'Display', None],
  [ 0x3628, 'CjkUnifiedIdeograph3628', [], 'Display', None],
  [ 0x3629, 'CjkUnifiedIdeograph3629', [], 'Display', None],
  [ 0x362a, 'CjkUnifiedIdeograph362a', [], 'Display', None],
  [ 0x362b, 'CjkUnifiedIdeograph362b', [], 'Display', None],
  [ 0x362c, 'CjkUnifiedIdeograph362c', [], 'Display', None],
  [ 0x362d, 'CjkUnifiedIdeograph362d', [], 'Display', None],
  [ 0x362e, 'CjkUnifiedIdeograph362e', [], 'Display', None],
  [ 0x362f, 'CjkUnifiedIdeograph362f', [], 'Display', None],
  [ 0x3630, 'CjkUnifiedIdeograph3630', [], 'Display', None],
  [ 0x3631, 'CjkUnifiedIdeograph3631', [], 'Display', None],
  [ 0x3632, 'CjkUnifiedIdeograph3632', [], 'Display', None],
  [ 0x3633, 'CjkUnifiedIdeograph3633', [], 'Display', None],
  [ 0x3634, 'CjkUnifiedIdeograph3634', [], 'Display', None],
  [ 0x3635, 'CjkUnifiedIdeograph3635', [], 'Display', None],
  [ 0x3636, 'CjkUnifiedIdeograph3636', [], 'Display', None],
  [ 0x3637, 'CjkUnifiedIdeograph3637', [], 'Display', None],
  [ 0x3638, 'CjkUnifiedIdeograph3638', [], 'Display', None],
  [ 0x3639, 'CjkUnifiedIdeograph3639', [], 'Display', None],
  [ 0x363a, 'CjkUnifiedIdeograph363a', [], 'Display', None],
  [ 0x363b, 'CjkUnifiedIdeograph363b', [], 'Display', None],
  [ 0x363c, 'CjkUnifiedIdeograph363c', [], 'Display', None],
  [ 0x363d, 'CjkUnifiedIdeograph363d', [], 'Display', None],
  [ 0x363e, 'CjkUnifiedIdeograph363e', [], 'Display', None],
  [ 0x363f, 'CjkUnifiedIdeograph363f', [], 'Display', None],
  [ 0x3640, 'CjkUnifiedIdeograph3640', [], 'Display', None],
  [ 0x3641, 'CjkUnifiedIdeograph3641', [], 'Display', None],
  [ 0x3642, 'CjkUnifiedIdeograph3642', [], 'Display', None],
  [ 0x3643, 'CjkUnifiedIdeograph3643', [], 'Display', None],
  [ 0x3644, 'CjkUnifiedIdeograph3644', [], 'Display', None],
  [ 0x3645, 'CjkUnifiedIdeograph3645', [], 'Display', None],
  [ 0x3646, 'CjkUnifiedIdeograph3646', [], 'Display', None],
  [ 0x3647, 'CjkUnifiedIdeograph3647', [], 'Display', None],
  [ 0x3648, 'CjkUnifiedIdeograph3648', [], 'Display', None],
  [ 0x3649, 'CjkUnifiedIdeograph3649', [], 'Display', None],
  [ 0x364a, 'CjkUnifiedIdeograph364a', [], 'Display', None],
  [ 0x364b, 'CjkUnifiedIdeograph364b', [], 'Display', None],
  [ 0x364c, 'CjkUnifiedIdeograph364c', [], 'Display', None],
  [ 0x364d, 'CjkUnifiedIdeograph364d', [], 'Display', None],
  [ 0x364e, 'CjkUnifiedIdeograph364e', [], 'Display', None],
  [ 0x364f, 'CjkUnifiedIdeograph364f', [], 'Display', None],
  [ 0x3650, 'CjkUnifiedIdeograph3650', [], 'Display', None],
  [ 0x3651, 'CjkUnifiedIdeograph3651', [], 'Display', None],
  [ 0x3652, 'CjkUnifiedIdeograph3652', [], 'Display', None],
  [ 0x3653, 'CjkUnifiedIdeograph3653', [], 'Display', None],
  [ 0x3654, 'CjkUnifiedIdeograph3654', [], 'Display', None],
  [ 0x3655, 'CjkUnifiedIdeograph3655', [], 'Display', None],
  [ 0x3656, 'CjkUnifiedIdeograph3656', [], 'Display', None],
  [ 0x3657, 'CjkUnifiedIdeograph3657', [], 'Display', None],
  [ 0x3658, 'CjkUnifiedIdeograph3658', [], 'Display', None],
  [ 0x3659, 'CjkUnifiedIdeograph3659', [], 'Display', None],
  [ 0x365a, 'CjkUnifiedIdeograph365a', [], 'Display', None],
  [ 0x365b, 'CjkUnifiedIdeograph365b', [], 'Display', None],
  [ 0x365c, 'CjkUnifiedIdeograph365c', [], 'Display', None],
  [ 0x365d, 'CjkUnifiedIdeograph365d', [], 'Display', None],
  [ 0x365e, 'CjkUnifiedIdeograph365e', [], 'Display', None],
  [ 0x365f, 'CjkUnifiedIdeograph365f', [], 'Display', None],
  [ 0x3660, 'CjkUnifiedIdeograph3660', [], 'Display', None],
  [ 0x3661, 'CjkUnifiedIdeograph3661', [], 'Display', None],
  [ 0x3662, 'CjkUnifiedIdeograph3662', [], 'Display', None],
  [ 0x3663, 'CjkUnifiedIdeograph3663', [], 'Display', None],
  [ 0x3664, 'CjkUnifiedIdeograph3664', [], 'Display', None],
  [ 0x3665, 'CjkUnifiedIdeograph3665', [], 'Display', None],
  [ 0x3666, 'CjkUnifiedIdeograph3666', [], 'Display', None],
  [ 0x3667, 'CjkUnifiedIdeograph3667', [], 'Display', None],
  [ 0x3668, 'CjkUnifiedIdeograph3668', [], 'Display', None],
  [ 0x3669, 'CjkUnifiedIdeograph3669', [], 'Display', None],
  [ 0x366a, 'CjkUnifiedIdeograph366a', [], 'Display', None],
  [ 0x366b, 'CjkUnifiedIdeograph366b', [], 'Display', None],
  [ 0x366c, 'CjkUnifiedIdeograph366c', [], 'Display', None],
  [ 0x366d, 'CjkUnifiedIdeograph366d', [], 'Display', None],
  [ 0x366e, 'CjkUnifiedIdeograph366e', [], 'Display', None],
  [ 0x366f, 'CjkUnifiedIdeograph366f', [], 'Display', None],
  [ 0x3670, 'CjkUnifiedIdeograph3670', [], 'Display', None],
  [ 0x3671, 'CjkUnifiedIdeograph3671', [], 'Display', None],
  [ 0x3672, 'CjkUnifiedIdeograph3672', [], 'Display', None],
  [ 0x3673, 'CjkUnifiedIdeograph3673', [], 'Display', None],
  [ 0x3674, 'CjkUnifiedIdeograph3674', [], 'Display', None],
  [ 0x3675, 'CjkUnifiedIdeograph3675', [], 'Display', None],
  [ 0x3676, 'CjkUnifiedIdeograph3676', [], 'Display', None],
  [ 0x3677, 'CjkUnifiedIdeograph3677', [], 'Display', None],
  [ 0x3678, 'CjkUnifiedIdeograph3678', [], 'Display', None],
  [ 0x3679, 'CjkUnifiedIdeograph3679', [], 'Display', None],
  [ 0x367a, 'CjkUnifiedIdeograph367a', [], 'Display', None],
  [ 0x367b, 'CjkUnifiedIdeograph367b', [], 'Display', None],
  [ 0x367c, 'CjkUnifiedIdeograph367c', [], 'Display', None],
  [ 0x367d, 'CjkUnifiedIdeograph367d', [], 'Display', None],
  [ 0x367e, 'CjkUnifiedIdeograph367e', [], 'Display', None],
  [ 0x367f, 'CjkUnifiedIdeograph367f', [], 'Display', None],
  [ 0x3680, 'CjkUnifiedIdeograph3680', [], 'Display', None],
  [ 0x3681, 'CjkUnifiedIdeograph3681', [], 'Display', None],
  [ 0x3682, 'CjkUnifiedIdeograph3682', [], 'Display', None],
  [ 0x3683, 'CjkUnifiedIdeograph3683', [], 'Display', None],
  [ 0x3684, 'CjkUnifiedIdeograph3684', [], 'Display', None],
  [ 0x3685, 'CjkUnifiedIdeograph3685', [], 'Display', None],
  [ 0x3686, 'CjkUnifiedIdeograph3686', [], 'Display', None],
  [ 0x3687, 'CjkUnifiedIdeograph3687', [], 'Display', None],
  [ 0x3688, 'CjkUnifiedIdeograph3688', [], 'Display', None],
  [ 0x3689, 'CjkUnifiedIdeograph3689', [], 'Display', None],
  [ 0x368a, 'CjkUnifiedIdeograph368a', [], 'Display', None],
  [ 0x368b, 'CjkUnifiedIdeograph368b', [], 'Display', None],
  [ 0x368c, 'CjkUnifiedIdeograph368c', [], 'Display', None],
  [ 0x368d, 'CjkUnifiedIdeograph368d', [], 'Display', None],
  [ 0x368e, 'CjkUnifiedIdeograph368e', [], 'Display', None],
  [ 0x368f, 'CjkUnifiedIdeograph368f', [], 'Display', None],
  [ 0x3690, 'CjkUnifiedIdeograph3690', [], 'Display', None],
  [ 0x3691, 'CjkUnifiedIdeograph3691', [], 'Display', None],
  [ 0x3692, 'CjkUnifiedIdeograph3692', [], 'Display', None],
  [ 0x3693, 'CjkUnifiedIdeograph3693', [], 'Display', None],
  [ 0x3694, 'CjkUnifiedIdeograph3694', [], 'Display', None],
  [ 0x3695, 'CjkUnifiedIdeograph3695', [], 'Display', None],
  [ 0x3696, 'CjkUnifiedIdeograph3696', [], 'Display', None],
  [ 0x3697, 'CjkUnifiedIdeograph3697', [], 'Display', None],
  [ 0x3698, 'CjkUnifiedIdeograph3698', [], 'Display', None],
  [ 0x3699, 'CjkUnifiedIdeograph3699', [], 'Display', None],
  [ 0x369a, 'CjkUnifiedIdeograph369a', [], 'Display', None],
  [ 0x369b, 'CjkUnifiedIdeograph369b', [], 'Display', None],
  [ 0x369c, 'CjkUnifiedIdeograph369c', [], 'Display', None],
  [ 0x369d, 'CjkUnifiedIdeograph369d', [], 'Display', None],
  [ 0x369e, 'CjkUnifiedIdeograph369e', [], 'Display', None],
  [ 0x369f, 'CjkUnifiedIdeograph369f', [], 'Display', None],
  [ 0x36a0, 'CjkUnifiedIdeograph36a0', [], 'Display', None],
  [ 0x36a1, 'CjkUnifiedIdeograph36a1', [], 'Display', None],
  [ 0x36a2, 'CjkUnifiedIdeograph36a2', [], 'Display', None],
  [ 0x36a3, 'CjkUnifiedIdeograph36a3', [], 'Display', None],
  [ 0x36a4, 'CjkUnifiedIdeograph36a4', [], 'Display', None],
  [ 0x36a5, 'CjkUnifiedIdeograph36a5', [], 'Display', None],
  [ 0x36a6, 'CjkUnifiedIdeograph36a6', [], 'Display', None],
  [ 0x36a7, 'CjkUnifiedIdeograph36a7', [], 'Display', None],
  [ 0x36a8, 'CjkUnifiedIdeograph36a8', [], 'Display', None],
  [ 0x36a9, 'CjkUnifiedIdeograph36a9', [], 'Display', None],
  [ 0x36aa, 'CjkUnifiedIdeograph36aa', [], 'Display', None],
  [ 0x36ab, 'CjkUnifiedIdeograph36ab', [], 'Display', None],
  [ 0x36ac, 'CjkUnifiedIdeograph36ac', [], 'Display', None],
  [ 0x36ad, 'CjkUnifiedIdeograph36ad', [], 'Display', None],
  [ 0x36ae, 'CjkUnifiedIdeograph36ae', [], 'Display', None],
  [ 0x36af, 'CjkUnifiedIdeograph36af', [], 'Display', None],
  [ 0x36b0, 'CjkUnifiedIdeograph36b0', [], 'Display', None],
  [ 0x36b1, 'CjkUnifiedIdeograph36b1', [], 'Display', None],
  [ 0x36b2, 'CjkUnifiedIdeograph36b2', [], 'Display', None],
  [ 0x36b3, 'CjkUnifiedIdeograph36b3', [], 'Display', None],
  [ 0x36b4, 'CjkUnifiedIdeograph36b4', [], 'Display', None],
  [ 0x36b5, 'CjkUnifiedIdeograph36b5', [], 'Display', None],
  [ 0x36b6, 'CjkUnifiedIdeograph36b6', [], 'Display', None],
  [ 0x36b7, 'CjkUnifiedIdeograph36b7', [], 'Display', None],
  [ 0x36b8, 'CjkUnifiedIdeograph36b8', [], 'Display', None],
  [ 0x36b9, 'CjkUnifiedIdeograph36b9', [], 'Display', None],
  [ 0x36ba, 'CjkUnifiedIdeograph36ba', [], 'Display', None],
  [ 0x36bb, 'CjkUnifiedIdeograph36bb', [], 'Display', None],
  [ 0x36bc, 'CjkUnifiedIdeograph36bc', [], 'Display', None],
  [ 0x36bd, 'CjkUnifiedIdeograph36bd', [], 'Display', None],
  [ 0x36be, 'CjkUnifiedIdeograph36be', [], 'Display', None],
  [ 0x36bf, 'CjkUnifiedIdeograph36bf', [], 'Display', None],
  [ 0x36c0, 'CjkUnifiedIdeograph36c0', [], 'Display', None],
  [ 0x36c1, 'CjkUnifiedIdeograph36c1', [], 'Display', None],
  [ 0x36c2, 'CjkUnifiedIdeograph36c2', [], 'Display', None],
  [ 0x36c3, 'CjkUnifiedIdeograph36c3', [], 'Display', None],
  [ 0x36c4, 'CjkUnifiedIdeograph36c4', [], 'Display', None],
  [ 0x36c5, 'CjkUnifiedIdeograph36c5', [], 'Display', None],
  [ 0x36c6, 'CjkUnifiedIdeograph36c6', [], 'Display', None],
  [ 0x36c7, 'CjkUnifiedIdeograph36c7', [], 'Display', None],
  [ 0x36c8, 'CjkUnifiedIdeograph36c8', [], 'Display', None],
  [ 0x36c9, 'CjkUnifiedIdeograph36c9', [], 'Display', None],
  [ 0x36ca, 'CjkUnifiedIdeograph36ca', [], 'Display', None],
  [ 0x36cb, 'CjkUnifiedIdeograph36cb', [], 'Display', None],
  [ 0x36cc, 'CjkUnifiedIdeograph36cc', [], 'Display', None],
  [ 0x36cd, 'CjkUnifiedIdeograph36cd', [], 'Display', None],
  [ 0x36ce, 'CjkUnifiedIdeograph36ce', [], 'Display', None],
  [ 0x36cf, 'CjkUnifiedIdeograph36cf', [], 'Display', None],
  [ 0x36d0, 'CjkUnifiedIdeograph36d0', [], 'Display', None],
  [ 0x36d1, 'CjkUnifiedIdeograph36d1', [], 'Display', None],
  [ 0x36d2, 'CjkUnifiedIdeograph36d2', [], 'Display', None],
  [ 0x36d3, 'CjkUnifiedIdeograph36d3', [], 'Display', None],
  [ 0x36d4, 'CjkUnifiedIdeograph36d4', [], 'Display', None],
  [ 0x36d5, 'CjkUnifiedIdeograph36d5', [], 'Display', None],
  [ 0x36d6, 'CjkUnifiedIdeograph36d6', [], 'Display', None],
  [ 0x36d7, 'CjkUnifiedIdeograph36d7', [], 'Display', None],
  [ 0x36d8, 'CjkUnifiedIdeograph36d8', [], 'Display', None],
  [ 0x36d9, 'CjkUnifiedIdeograph36d9', [], 'Display', None],
  [ 0x36da, 'CjkUnifiedIdeograph36da', [], 'Display', None],
  [ 0x36db, 'CjkUnifiedIdeograph36db', [], 'Display', None],
  [ 0x36dc, 'CjkUnifiedIdeograph36dc', [], 'Display', None],
  [ 0x36dd, 'CjkUnifiedIdeograph36dd', [], 'Display', None],
  [ 0x36de, 'CjkUnifiedIdeograph36de', [], 'Display', None],
  [ 0x36df, 'CjkUnifiedIdeograph36df', [], 'Display', None],
  [ 0x36e0, 'CjkUnifiedIdeograph36e0', [], 'Display', None],
  [ 0x36e1, 'CjkUnifiedIdeograph36e1', [], 'Display', None],
  [ 0x36e2, 'CjkUnifiedIdeograph36e2', [], 'Display', None],
  [ 0x36e3, 'CjkUnifiedIdeograph36e3', [], 'Display', None],
  [ 0x36e4, 'CjkUnifiedIdeograph36e4', [], 'Display', None],
  [ 0x36e5, 'CjkUnifiedIdeograph36e5', [], 'Display', None],
  [ 0x36e6, 'CjkUnifiedIdeograph36e6', [], 'Display', None],
  [ 0x36e7, 'CjkUnifiedIdeograph36e7', [], 'Display', None],
  [ 0x36e8, 'CjkUnifiedIdeograph36e8', [], 'Display', None],
  [ 0x36e9, 'CjkUnifiedIdeograph36e9', [], 'Display', None],
  [ 0x36ea, 'CjkUnifiedIdeograph36ea', [], 'Display', None],
  [ 0x36eb, 'CjkUnifiedIdeograph36eb', [], 'Display', None],
  [ 0x36ec, 'CjkUnifiedIdeograph36ec', [], 'Display', None],
  [ 0x36ed, 'CjkUnifiedIdeograph36ed', [], 'Display', None],
  [ 0x36ee, 'CjkUnifiedIdeograph36ee', [], 'Display', None],
  [ 0x36ef, 'CjkUnifiedIdeograph36ef', [], 'Display', None],
  [ 0x36f0, 'CjkUnifiedIdeograph36f0', [], 'Display', None],
  [ 0x36f1, 'CjkUnifiedIdeograph36f1', [], 'Display', None],
  [ 0x36f2, 'CjkUnifiedIdeograph36f2', [], 'Display', None],
  [ 0x36f3, 'CjkUnifiedIdeograph36f3', [], 'Display', None],
  [ 0x36f4, 'CjkUnifiedIdeograph36f4', [], 'Display', None],
  [ 0x36f5, 'CjkUnifiedIdeograph36f5', [], 'Display', None],
  [ 0x36f6, 'CjkUnifiedIdeograph36f6', [], 'Display', None],
  [ 0x36f7, 'CjkUnifiedIdeograph36f7', [], 'Display', None],
  [ 0x36f8, 'CjkUnifiedIdeograph36f8', [], 'Display', None],
  [ 0x36f9, 'CjkUnifiedIdeograph36f9', [], 'Display', None],
  [ 0x36fa, 'CjkUnifiedIdeograph36fa', [], 'Display', None],
  [ 0x36fb, 'CjkUnifiedIdeograph36fb', [], 'Display', None],
  [ 0x36fc, 'CjkUnifiedIdeograph36fc', [], 'Display', None],
  [ 0x36fd, 'CjkUnifiedIdeograph36fd', [], 'Display', None],
  [ 0x36fe, 'CjkUnifiedIdeograph36fe', [], 'Display', None],
  [ 0x36ff, 'CjkUnifiedIdeograph36ff', [], 'Display', None],
  [ 0x3700, 'CjkUnifiedIdeograph3700', [], 'Display', None],
  [ 0x3701, 'CjkUnifiedIdeograph3701', [], 'Display', None],
  [ 0x3702, 'CjkUnifiedIdeograph3702', [], 'Display', None],
  [ 0x3703, 'CjkUnifiedIdeograph3703', [], 'Display', None],
  [ 0x3704, 'CjkUnifiedIdeograph3704', [], 'Display', None],
  [ 0x3705, 'CjkUnifiedIdeograph3705', [], 'Display', None],
  [ 0x3706, 'CjkUnifiedIdeograph3706', [], 'Display', None],
  [ 0x3707, 'CjkUnifiedIdeograph3707', [], 'Display', None],
  [ 0x3708, 'CjkUnifiedIdeograph3708', [], 'Display', None],
  [ 0x3709, 'CjkUnifiedIdeograph3709', [], 'Display', None],
  [ 0x370a, 'CjkUnifiedIdeograph370a', [], 'Display', None],
  [ 0x370b, 'CjkUnifiedIdeograph370b', [], 'Display', None],
  [ 0x370c, 'CjkUnifiedIdeograph370c', [], 'Display', None],
  [ 0x370d, 'CjkUnifiedIdeograph370d', [], 'Display', None],
  [ 0x370e, 'CjkUnifiedIdeograph370e', [], 'Display', None],
  [ 0x370f, 'CjkUnifiedIdeograph370f', [], 'Display', None],
  [ 0x3710, 'CjkUnifiedIdeograph3710', [], 'Display', None],
  [ 0x3711, 'CjkUnifiedIdeograph3711', [], 'Display', None],
  [ 0x3712, 'CjkUnifiedIdeograph3712', [], 'Display', None],
  [ 0x3713, 'CjkUnifiedIdeograph3713', [], 'Display', None],
  [ 0x3714, 'CjkUnifiedIdeograph3714', [], 'Display', None],
  [ 0x3715, 'CjkUnifiedIdeograph3715', [], 'Display', None],
  [ 0x3716, 'CjkUnifiedIdeograph3716', [], 'Display', None],
  [ 0x3717, 'CjkUnifiedIdeograph3717', [], 'Display', None],
  [ 0x3718, 'CjkUnifiedIdeograph3718', [], 'Display', None],
  [ 0x3719, 'CjkUnifiedIdeograph3719', [], 'Display', None],
  [ 0x371a, 'CjkUnifiedIdeograph371a', [], 'Display', None],
  [ 0x371b, 'CjkUnifiedIdeograph371b', [], 'Display', None],
  [ 0x371c, 'CjkUnifiedIdeograph371c', [], 'Display', None],
  [ 0x371d, 'CjkUnifiedIdeograph371d', [], 'Display', None],
  [ 0x371e, 'CjkUnifiedIdeograph371e', [], 'Display', None],
  [ 0x371f, 'CjkUnifiedIdeograph371f', [], 'Display', None],
  [ 0x3720, 'CjkUnifiedIdeograph3720', [], 'Display', None],
  [ 0x3721, 'CjkUnifiedIdeograph3721', [], 'Display', None],
  [ 0x3722, 'CjkUnifiedIdeograph3722', [], 'Display', None],
  [ 0x3723, 'CjkUnifiedIdeograph3723', [], 'Display', None],
  [ 0x3724, 'CjkUnifiedIdeograph3724', [], 'Display', None],
  [ 0x3725, 'CjkUnifiedIdeograph3725', [], 'Display', None],
  [ 0x3726, 'CjkUnifiedIdeograph3726', [], 'Display', None],
  [ 0x3727, 'CjkUnifiedIdeograph3727', [], 'Display', None],
  [ 0x3728, 'CjkUnifiedIdeograph3728', [], 'Display', None],
  [ 0x3729, 'CjkUnifiedIdeograph3729', [], 'Display', None],
  [ 0x372a, 'CjkUnifiedIdeograph372a', [], 'Display', None],
  [ 0x372b, 'CjkUnifiedIdeograph372b', [], 'Display', None],
  [ 0x372c, 'CjkUnifiedIdeograph372c', [], 'Display', None],
  [ 0x372d, 'CjkUnifiedIdeograph372d', [], 'Display', None],
  [ 0x372e, 'CjkUnifiedIdeograph372e', [], 'Display', None],
  [ 0x372f, 'CjkUnifiedIdeograph372f', [], 'Display', None],
  [ 0x3730, 'CjkUnifiedIdeograph3730', [], 'Display', None],
  [ 0x3731, 'CjkUnifiedIdeograph3731', [], 'Display', None],
  [ 0x3732, 'CjkUnifiedIdeograph3732', [], 'Display', None],
  [ 0x3733, 'CjkUnifiedIdeograph3733', [], 'Display', None],
  [ 0x3734, 'CjkUnifiedIdeograph3734', [], 'Display', None],
  [ 0x3735, 'CjkUnifiedIdeograph3735', [], 'Display', None],
  [ 0x3736, 'CjkUnifiedIdeograph3736', [], 'Display', None],
  [ 0x3737, 'CjkUnifiedIdeograph3737', [], 'Display', None],
  [ 0x3738, 'CjkUnifiedIdeograph3738', [], 'Display', None],
  [ 0x3739, 'CjkUnifiedIdeograph3739', [], 'Display', None],
  [ 0x373a, 'CjkUnifiedIdeograph373a', [], 'Display', None],
  [ 0x373b, 'CjkUnifiedIdeograph373b', [], 'Display', None],
  [ 0x373c, 'CjkUnifiedIdeograph373c', [], 'Display', None],
  [ 0x373d, 'CjkUnifiedIdeograph373d', [], 'Display', None],
  [ 0x373e, 'CjkUnifiedIdeograph373e', [], 'Display', None],
  [ 0x373f, 'CjkUnifiedIdeograph373f', [], 'Display', None],
  [ 0x3740, 'CjkUnifiedIdeograph3740', [], 'Display', None],
  [ 0x3741, 'CjkUnifiedIdeograph3741', [], 'Display', None],
  [ 0x3742, 'CjkUnifiedIdeograph3742', [], 'Display', None],
  [ 0x3743, 'CjkUnifiedIdeograph3743', [], 'Display', None],
  [ 0x3744, 'CjkUnifiedIdeograph3744', [], 'Display', None],
  [ 0x3745, 'CjkUnifiedIdeograph3745', [], 'Display', None],
  [ 0x3746, 'CjkUnifiedIdeograph3746', [], 'Display', None],
  [ 0x3747, 'CjkUnifiedIdeograph3747', [], 'Display', None],
  [ 0x3748, 'CjkUnifiedIdeograph3748', [], 'Display', None],
  [ 0x3749, 'CjkUnifiedIdeograph3749', [], 'Display', None],
  [ 0x374a, 'CjkUnifiedIdeograph374a', [], 'Display', None],
  [ 0x374b, 'CjkUnifiedIdeograph374b', [], 'Display', None],
  [ 0x374c, 'CjkUnifiedIdeograph374c', [], 'Display', None],
  [ 0x374d, 'CjkUnifiedIdeograph374d', [], 'Display', None],
  [ 0x374e, 'CjkUnifiedIdeograph374e', [], 'Display', None],
  [ 0x374f, 'CjkUnifiedIdeograph374f', [], 'Display', None],
  [ 0x3750, 'CjkUnifiedIdeograph3750', [], 'Display', None],
  [ 0x3751, 'CjkUnifiedIdeograph3751', [], 'Display', None],
  [ 0x3752, 'CjkUnifiedIdeograph3752', [], 'Display', None],
  [ 0x3753, 'CjkUnifiedIdeograph3753', [], 'Display', None],
  [ 0x3754, 'CjkUnifiedIdeograph3754', [], 'Display', None],
  [ 0x3755, 'CjkUnifiedIdeograph3755', [], 'Display', None],
  [ 0x3756, 'CjkUnifiedIdeograph3756', [], 'Display', None],
  [ 0x3757, 'CjkUnifiedIdeograph3757', [], 'Display', None],
  [ 0x3758, 'CjkUnifiedIdeograph3758', [], 'Display', None],
  [ 0x3759, 'CjkUnifiedIdeograph3759', [], 'Display', None],
  [ 0x375a, 'CjkUnifiedIdeograph375a', [], 'Display', None],
  [ 0x375b, 'CjkUnifiedIdeograph375b', [], 'Display', None],
  [ 0x375c, 'CjkUnifiedIdeograph375c', [], 'Display', None],
  [ 0x375d, 'CjkUnifiedIdeograph375d', [], 'Display', None],
  [ 0x375e, 'CjkUnifiedIdeograph375e', [], 'Display', None],
  [ 0x375f, 'CjkUnifiedIdeograph375f', [], 'Display', None],
  [ 0x3760, 'CjkUnifiedIdeograph3760', [], 'Display', None],
  [ 0x3761, 'CjkUnifiedIdeograph3761', [], 'Display', None],
  [ 0x3762, 'CjkUnifiedIdeograph3762', [], 'Display', None],
  [ 0x3763, 'CjkUnifiedIdeograph3763', [], 'Display', None],
  [ 0x3764, 'CjkUnifiedIdeograph3764', [], 'Display', None],
  [ 0x3765, 'CjkUnifiedIdeograph3765', [], 'Display', None],
  [ 0x3766, 'CjkUnifiedIdeograph3766', [], 'Display', None],
  [ 0x3767, 'CjkUnifiedIdeograph3767', [], 'Display', None],
  [ 0x3768, 'CjkUnifiedIdeograph3768', [], 'Display', None],
  [ 0x3769, 'CjkUnifiedIdeograph3769', [], 'Display', None],
  [ 0x376a, 'CjkUnifiedIdeograph376a', [], 'Display', None],
  [ 0x376b, 'CjkUnifiedIdeograph376b', [], 'Display', None],
  [ 0x376c, 'CjkUnifiedIdeograph376c', [], 'Display', None],
  [ 0x376d, 'CjkUnifiedIdeograph376d', [], 'Display', None],
  [ 0x376e, 'CjkUnifiedIdeograph376e', [], 'Display', None],
  [ 0x376f, 'CjkUnifiedIdeograph376f', [], 'Display', None],
  [ 0x3770, 'CjkUnifiedIdeograph3770', [], 'Display', None],
  [ 0x3771, 'CjkUnifiedIdeograph3771', [], 'Display', None],
  [ 0x3772, 'CjkUnifiedIdeograph3772', [], 'Display', None],
  [ 0x3773, 'CjkUnifiedIdeograph3773', [], 'Display', None],
  [ 0x3774, 'CjkUnifiedIdeograph3774', [], 'Display', None],
  [ 0x3775, 'CjkUnifiedIdeograph3775', [], 'Display', None],
  [ 0x3776, 'CjkUnifiedIdeograph3776', [], 'Display', None],
  [ 0x3777, 'CjkUnifiedIdeograph3777', [], 'Display', None],
  [ 0x3778, 'CjkUnifiedIdeograph3778', [], 'Display', None],
  [ 0x3779, 'CjkUnifiedIdeograph3779', [], 'Display', None],
  [ 0x377a, 'CjkUnifiedIdeograph377a', [], 'Display', None],
  [ 0x377b, 'CjkUnifiedIdeograph377b', [], 'Display', None],
  [ 0x377c, 'CjkUnifiedIdeograph377c', [], 'Display', None],
  [ 0x377d, 'CjkUnifiedIdeograph377d', [], 'Display', None],
  [ 0x377e, 'CjkUnifiedIdeograph377e', [], 'Display', None],
  [ 0x377f, 'CjkUnifiedIdeograph377f', [], 'Display', None],
  [ 0x3780, 'CjkUnifiedIdeograph3780', [], 'Display', None],
  [ 0x3781, 'CjkUnifiedIdeograph3781', [], 'Display', None],
  [ 0x3782, 'CjkUnifiedIdeograph3782', [], 'Display', None],
  [ 0x3783, 'CjkUnifiedIdeograph3783', [], 'Display', None],
  [ 0x3784, 'CjkUnifiedIdeograph3784', [], 'Display', None],
  [ 0x3785, 'CjkUnifiedIdeograph3785', [], 'Display', None],
  [ 0x3786, 'CjkUnifiedIdeograph3786', [], 'Display', None],
  [ 0x3787, 'CjkUnifiedIdeograph3787', [], 'Display', None],
  [ 0x3788, 'CjkUnifiedIdeograph3788', [], 'Display', None],
  [ 0x3789, 'CjkUnifiedIdeograph3789', [], 'Display', None],
  [ 0x378a, 'CjkUnifiedIdeograph378a', [], 'Display', None],
  [ 0x378b, 'CjkUnifiedIdeograph378b', [], 'Display', None],
  [ 0x378c, 'CjkUnifiedIdeograph378c', [], 'Display', None],
  [ 0x378d, 'CjkUnifiedIdeograph378d', [], 'Display', None],
  [ 0x378e, 'CjkUnifiedIdeograph378e', [], 'Display', None],
  [ 0x378f, 'CjkUnifiedIdeograph378f', [], 'Display', None],
  [ 0x3790, 'CjkUnifiedIdeograph3790', [], 'Display', None],
  [ 0x3791, 'CjkUnifiedIdeograph3791', [], 'Display', None],
  [ 0x3792, 'CjkUnifiedIdeograph3792', [], 'Display', None],
  [ 0x3793, 'CjkUnifiedIdeograph3793', [], 'Display', None],
  [ 0x3794, 'CjkUnifiedIdeograph3794', [], 'Display', None],
  [ 0x3795, 'CjkUnifiedIdeograph3795', [], 'Display', None],
  [ 0x3796, 'CjkUnifiedIdeograph3796', [], 'Display', None],
  [ 0x3797, 'CjkUnifiedIdeograph3797', [], 'Display', None],
  [ 0x3798, 'CjkUnifiedIdeograph3798', [], 'Display', None],
  [ 0x3799, 'CjkUnifiedIdeograph3799', [], 'Display', None],
  [ 0x379a, 'CjkUnifiedIdeograph379a', [], 'Display', None],
  [ 0x379b, 'CjkUnifiedIdeograph379b', [], 'Display', None],
  [ 0x379c, 'CjkUnifiedIdeograph379c', [], 'Display', None],
  [ 0x379d, 'CjkUnifiedIdeograph379d', [], 'Display', None],
  [ 0x379e, 'CjkUnifiedIdeograph379e', [], 'Display', None],
  [ 0x379f, 'CjkUnifiedIdeograph379f', [], 'Display', None],
  [ 0x37a0, 'CjkUnifiedIdeograph37a0', [], 'Display', None],
  [ 0x37a1, 'CjkUnifiedIdeograph37a1', [], 'Display', None],
  [ 0x37a2, 'CjkUnifiedIdeograph37a2', [], 'Display', None],
  [ 0x37a3, 'CjkUnifiedIdeograph37a3', [], 'Display', None],
  [ 0x37a4, 'CjkUnifiedIdeograph37a4', [], 'Display', None],
  [ 0x37a5, 'CjkUnifiedIdeograph37a5', [], 'Display', None],
  [ 0x37a6, 'CjkUnifiedIdeograph37a6', [], 'Display', None],
  [ 0x37a7, 'CjkUnifiedIdeograph37a7', [], 'Display', None],
  [ 0x37a8, 'CjkUnifiedIdeograph37a8', [], 'Display', None],
  [ 0x37a9, 'CjkUnifiedIdeograph37a9', [], 'Display', None],
  [ 0x37aa, 'CjkUnifiedIdeograph37aa', [], 'Display', None],
  [ 0x37ab, 'CjkUnifiedIdeograph37ab', [], 'Display', None],
  [ 0x37ac, 'CjkUnifiedIdeograph37ac', [], 'Display', None],
  [ 0x37ad, 'CjkUnifiedIdeograph37ad', [], 'Display', None],
  [ 0x37ae, 'CjkUnifiedIdeograph37ae', [], 'Display', None],
  [ 0x37af, 'CjkUnifiedIdeograph37af', [], 'Display', None],
  [ 0x37b0, 'CjkUnifiedIdeograph37b0', [], 'Display', None],
  [ 0x37b1, 'CjkUnifiedIdeograph37b1', [], 'Display', None],
  [ 0x37b2, 'CjkUnifiedIdeograph37b2', [], 'Display', None],
  [ 0x37b3, 'CjkUnifiedIdeograph37b3', [], 'Display', None],
  [ 0x37b4, 'CjkUnifiedIdeograph37b4', [], 'Display', None],
  [ 0x37b5, 'CjkUnifiedIdeograph37b5', [], 'Display', None],
  [ 0x37b6, 'CjkUnifiedIdeograph37b6', [], 'Display', None],
  [ 0x37b7, 'CjkUnifiedIdeograph37b7', [], 'Display', None],
  [ 0x37b8, 'CjkUnifiedIdeograph37b8', [], 'Display', None],
  [ 0x37b9, 'CjkUnifiedIdeograph37b9', [], 'Display', None],
  [ 0x37ba, 'CjkUnifiedIdeograph37ba', [], 'Display', None],
  [ 0x37bb, 'CjkUnifiedIdeograph37bb', [], 'Display', None],
  [ 0x37bc, 'CjkUnifiedIdeograph37bc', [], 'Display', None],
  [ 0x37bd, 'CjkUnifiedIdeograph37bd', [], 'Display', None],
  [ 0x37be, 'CjkUnifiedIdeograph37be', [], 'Display', None],
  [ 0x37bf, 'CjkUnifiedIdeograph37bf', [], 'Display', None],
  [ 0x37c0, 'CjkUnifiedIdeograph37c0', [], 'Display', None],
  [ 0x37c1, 'CjkUnifiedIdeograph37c1', [], 'Display', None],
  [ 0x37c2, 'CjkUnifiedIdeograph37c2', [], 'Display', None],
  [ 0x37c3, 'CjkUnifiedIdeograph37c3', [], 'Display', None],
  [ 0x37c4, 'CjkUnifiedIdeograph37c4', [], 'Display', None],
  [ 0x37c5, 'CjkUnifiedIdeograph37c5', [], 'Display', None],
  [ 0x37c6, 'CjkUnifiedIdeograph37c6', [], 'Display', None],
  [ 0x37c7, 'CjkUnifiedIdeograph37c7', [], 'Display', None],
  [ 0x37c8, 'CjkUnifiedIdeograph37c8', [], 'Display', None],
  [ 0x37c9, 'CjkUnifiedIdeograph37c9', [], 'Display', None],
  [ 0x37ca, 'CjkUnifiedIdeograph37ca', [], 'Display', None],
  [ 0x37cb, 'CjkUnifiedIdeograph37cb', [], 'Display', None],
  [ 0x37cc, 'CjkUnifiedIdeograph37cc', [], 'Display', None],
  [ 0x37cd, 'CjkUnifiedIdeograph37cd', [], 'Display', None],
  [ 0x37ce, 'CjkUnifiedIdeograph37ce', [], 'Display', None],
  [ 0x37cf, 'CjkUnifiedIdeograph37cf', [], 'Display', None],
  [ 0x37d0, 'CjkUnifiedIdeograph37d0', [], 'Display', None],
  [ 0x37d1, 'CjkUnifiedIdeograph37d1', [], 'Display', None],
  [ 0x37d2, 'CjkUnifiedIdeograph37d2', [], 'Display', None],
  [ 0x37d3, 'CjkUnifiedIdeograph37d3', [], 'Display', None],
  [ 0x37d4, 'CjkUnifiedIdeograph37d4', [], 'Display', None],
  [ 0x37d5, 'CjkUnifiedIdeograph37d5', [], 'Display', None],
  [ 0x37d6, 'CjkUnifiedIdeograph37d6', [], 'Display', None],
  [ 0x37d7, 'CjkUnifiedIdeograph37d7', [], 'Display', None],
  [ 0x37d8, 'CjkUnifiedIdeograph37d8', [], 'Display', None],
  [ 0x37d9, 'CjkUnifiedIdeograph37d9', [], 'Display', None],
  [ 0x37da, 'CjkUnifiedIdeograph37da', [], 'Display', None],
  [ 0x37db, 'CjkUnifiedIdeograph37db', [], 'Display', None],
  [ 0x37dc, 'CjkUnifiedIdeograph37dc', [], 'Display', None],
  [ 0x37dd, 'CjkUnifiedIdeograph37dd', [], 'Display', None],
  [ 0x37de, 'CjkUnifiedIdeograph37de', [], 'Display', None],
  [ 0x37df, 'CjkUnifiedIdeograph37df', [], 'Display', None],
  [ 0x37e0, 'CjkUnifiedIdeograph37e0', [], 'Display', None],
  [ 0x37e1, 'CjkUnifiedIdeograph37e1', [], 'Display', None],
  [ 0x37e2, 'CjkUnifiedIdeograph37e2', [], 'Display', None],
  [ 0x37e3, 'CjkUnifiedIdeograph37e3', [], 'Display', None],
  [ 0x37e4, 'CjkUnifiedIdeograph37e4', [], 'Display', None],
  [ 0x37e5, 'CjkUnifiedIdeograph37e5', [], 'Display', None],
  [ 0x37e6, 'CjkUnifiedIdeograph37e6', [], 'Display', None],
  [ 0x37e7, 'CjkUnifiedIdeograph37e7', [], 'Display', None],
  [ 0x37e8, 'CjkUnifiedIdeograph37e8', [], 'Display', None],
  [ 0x37e9, 'CjkUnifiedIdeograph37e9', [], 'Display', None],
  [ 0x37ea, 'CjkUnifiedIdeograph37ea', [], 'Display', None],
  [ 0x37eb, 'CjkUnifiedIdeograph37eb', [], 'Display', None],
  [ 0x37ec, 'CjkUnifiedIdeograph37ec', [], 'Display', None],
  [ 0x37ed, 'CjkUnifiedIdeograph37ed', [], 'Display', None],
  [ 0x37ee, 'CjkUnifiedIdeograph37ee', [], 'Display', None],
  [ 0x37ef, 'CjkUnifiedIdeograph37ef', [], 'Display', None],
  [ 0x37f0, 'CjkUnifiedIdeograph37f0', [], 'Display', None],
  [ 0x37f1, 'CjkUnifiedIdeograph37f1', [], 'Display', None],
  [ 0x37f2, 'CjkUnifiedIdeograph37f2', [], 'Display', None],
  [ 0x37f3, 'CjkUnifiedIdeograph37f3', [], 'Display', None],
  [ 0x37f4, 'CjkUnifiedIdeograph37f4', [], 'Display', None],
  [ 0x37f5, 'CjkUnifiedIdeograph37f5', [], 'Display', None],
  [ 0x37f6, 'CjkUnifiedIdeograph37f6', [], 'Display', None],
  [ 0x37f7, 'CjkUnifiedIdeograph37f7', [], 'Display', None],
  [ 0x37f8, 'CjkUnifiedIdeograph37f8', [], 'Display', None],
  [ 0x37f9, 'CjkUnifiedIdeograph37f9', [], 'Display', None],
  [ 0x37fa, 'CjkUnifiedIdeograph37fa', [], 'Display', None],
  [ 0x37fb, 'CjkUnifiedIdeograph37fb', [], 'Display', None],
  [ 0x37fc, 'CjkUnifiedIdeograph37fc', [], 'Display', None],
  [ 0x37fd, 'CjkUnifiedIdeograph37fd', [], 'Display', None],
  [ 0x37fe, 'CjkUnifiedIdeograph37fe', [], 'Display', None],
  [ 0x37ff, 'CjkUnifiedIdeograph37ff', [], 'Display', None],
  [ 0x3800, 'CjkUnifiedIdeograph3800', [], 'Display', None],
  [ 0x3801, 'CjkUnifiedIdeograph3801', [], 'Display', None],
  [ 0x3802, 'CjkUnifiedIdeograph3802', [], 'Display', None],
  [ 0x3803, 'CjkUnifiedIdeograph3803', [], 'Display', None],
  [ 0x3804, 'CjkUnifiedIdeograph3804', [], 'Display', None],
  [ 0x3805, 'CjkUnifiedIdeograph3805', [], 'Display', None],
  [ 0x3806, 'CjkUnifiedIdeograph3806', [], 'Display', None],
  [ 0x3807, 'CjkUnifiedIdeograph3807', [], 'Display', None],
  [ 0x3808, 'CjkUnifiedIdeograph3808', [], 'Display', None],
  [ 0x3809, 'CjkUnifiedIdeograph3809', [], 'Display', None],
  [ 0x380a, 'CjkUnifiedIdeograph380a', [], 'Display', None],
  [ 0x380b, 'CjkUnifiedIdeograph380b', [], 'Display', None],
  [ 0x380c, 'CjkUnifiedIdeograph380c', [], 'Display', None],
  [ 0x380d, 'CjkUnifiedIdeograph380d', [], 'Display', None],
  [ 0x380e, 'CjkUnifiedIdeograph380e', [], 'Display', None],
  [ 0x380f, 'CjkUnifiedIdeograph380f', [], 'Display', None],
  [ 0x3810, 'CjkUnifiedIdeograph3810', [], 'Display', None],
  [ 0x3811, 'CjkUnifiedIdeograph3811', [], 'Display', None],
  [ 0x3812, 'CjkUnifiedIdeograph3812', [], 'Display', None],
  [ 0x3813, 'CjkUnifiedIdeograph3813', [], 'Display', None],
  [ 0x3814, 'CjkUnifiedIdeograph3814', [], 'Display', None],
  [ 0x3815, 'CjkUnifiedIdeograph3815', [], 'Display', None],
  [ 0x3816, 'CjkUnifiedIdeograph3816', [], 'Display', None],
  [ 0x3817, 'CjkUnifiedIdeograph3817', [], 'Display', None],
  [ 0x3818, 'CjkUnifiedIdeograph3818', [], 'Display', None],
  [ 0x3819, 'CjkUnifiedIdeograph3819', [], 'Display', None],
  [ 0x381a, 'CjkUnifiedIdeograph381a', [], 'Display', None],
  [ 0x381b, 'CjkUnifiedIdeograph381b', [], 'Display', None],
  [ 0x381c, 'CjkUnifiedIdeograph381c', [], 'Display', None],
  [ 0x381d, 'CjkUnifiedIdeograph381d', [], 'Display', None],
  [ 0x381e, 'CjkUnifiedIdeograph381e', [], 'Display', None],
  [ 0x381f, 'CjkUnifiedIdeograph381f', [], 'Display', None],
  [ 0x3820, 'CjkUnifiedIdeograph3820', [], 'Display', None],
  [ 0x3821, 'CjkUnifiedIdeograph3821', [], 'Display', None],
  [ 0x3822, 'CjkUnifiedIdeograph3822', [], 'Display', None],
  [ 0x3823, 'CjkUnifiedIdeograph3823', [], 'Display', None],
  [ 0x3824, 'CjkUnifiedIdeograph3824', [], 'Display', None],
  [ 0x3825, 'CjkUnifiedIdeograph3825', [], 'Display', None],
  [ 0x3826, 'CjkUnifiedIdeograph3826', [], 'Display', None],
  [ 0x3827, 'CjkUnifiedIdeograph3827', [], 'Display', None],
  [ 0x3828, 'CjkUnifiedIdeograph3828', [], 'Display', None],
  [ 0x3829, 'CjkUnifiedIdeograph3829', [], 'Display', None],
  [ 0x382a, 'CjkUnifiedIdeograph382a', [], 'Display', None],
  [ 0x382b, 'CjkUnifiedIdeograph382b', [], 'Display', None],
  [ 0x382c, 'CjkUnifiedIdeograph382c', [], 'Display', None],
  [ 0x382d, 'CjkUnifiedIdeograph382d', [], 'Display', None],
  [ 0x382e, 'CjkUnifiedIdeograph382e', [], 'Display', None],
  [ 0x382f, 'CjkUnifiedIdeograph382f', [], 'Display', None],
  [ 0x3830, 'CjkUnifiedIdeograph3830', [], 'Display', None],
  [ 0x3831, 'CjkUnifiedIdeograph3831', [], 'Display', None],
  [ 0x3832, 'CjkUnifiedIdeograph3832', [], 'Display', None],
  [ 0x3833, 'CjkUnifiedIdeograph3833', [], 'Display', None],
  [ 0x3834, 'CjkUnifiedIdeograph3834', [], 'Display', None],
  [ 0x3835, 'CjkUnifiedIdeograph3835', [], 'Display', None],
  [ 0x3836, 'CjkUnifiedIdeograph3836', [], 'Display', None],
  [ 0x3837, 'CjkUnifiedIdeograph3837', [], 'Display', None],
  [ 0x3838, 'CjkUnifiedIdeograph3838', [], 'Display', None],
  [ 0x3839, 'CjkUnifiedIdeograph3839', [], 'Display', None],
  [ 0x383a, 'CjkUnifiedIdeograph383a', [], 'Display', None],
  [ 0x383b, 'CjkUnifiedIdeograph383b', [], 'Display', None],
  [ 0x383c, 'CjkUnifiedIdeograph383c', [], 'Display', None],
  [ 0x383d, 'CjkUnifiedIdeograph383d', [], 'Display', None],
  [ 0x383e, 'CjkUnifiedIdeograph383e', [], 'Display', None],
  [ 0x383f, 'CjkUnifiedIdeograph383f', [], 'Display', None],
  [ 0x3840, 'CjkUnifiedIdeograph3840', [], 'Display', None],
  [ 0x3841, 'CjkUnifiedIdeograph3841', [], 'Display', None],
  [ 0x3842, 'CjkUnifiedIdeograph3842', [], 'Display', None],
  [ 0x3843, 'CjkUnifiedIdeograph3843', [], 'Display', None],
  [ 0x3844, 'CjkUnifiedIdeograph3844', [], 'Display', None],
  [ 0x3845, 'CjkUnifiedIdeograph3845', [], 'Display', None],
  [ 0x3846, 'CjkUnifiedIdeograph3846', [], 'Display', None],
  [ 0x3847, 'CjkUnifiedIdeograph3847', [], 'Display', None],
  [ 0x3848, 'CjkUnifiedIdeograph3848', [], 'Display', None],
  [ 0x3849, 'CjkUnifiedIdeograph3849', [], 'Display', None],
  [ 0x384a, 'CjkUnifiedIdeograph384a', [], 'Display', None],
  [ 0x384b, 'CjkUnifiedIdeograph384b', [], 'Display', None],
  [ 0x384c, 'CjkUnifiedIdeograph384c', [], 'Display', None],
  [ 0x384d, 'CjkUnifiedIdeograph384d', [], 'Display', None],
  [ 0x384e, 'CjkUnifiedIdeograph384e', [], 'Display', None],
  [ 0x384f, 'CjkUnifiedIdeograph384f', [], 'Display', None],
  [ 0x3850, 'CjkUnifiedIdeograph3850', [], 'Display', None],
  [ 0x3851, 'CjkUnifiedIdeograph3851', [], 'Display', None],
  [ 0x3852, 'CjkUnifiedIdeograph3852', [], 'Display', None],
  [ 0x3853, 'CjkUnifiedIdeograph3853', [], 'Display', None],
  [ 0x3854, 'CjkUnifiedIdeograph3854', [], 'Display', None],
  [ 0x3855, 'CjkUnifiedIdeograph3855', [], 'Display', None],
  [ 0x3856, 'CjkUnifiedIdeograph3856', [], 'Display', None],
  [ 0x3857, 'CjkUnifiedIdeograph3857', [], 'Display', None],
  [ 0x3858, 'CjkUnifiedIdeograph3858', [], 'Display', None],
  [ 0x3859, 'CjkUnifiedIdeograph3859', [], 'Display', None],
  [ 0x385a, 'CjkUnifiedIdeograph385a', [], 'Display', None],
  [ 0x385b, 'CjkUnifiedIdeograph385b', [], 'Display', None],
  [ 0x385c, 'CjkUnifiedIdeograph385c', [], 'Display', None],
  [ 0x385d, 'CjkUnifiedIdeograph385d', [], 'Display', None],
  [ 0x385e, 'CjkUnifiedIdeograph385e', [], 'Display', None],
  [ 0x385f, 'CjkUnifiedIdeograph385f', [], 'Display', None],
  [ 0x3860, 'CjkUnifiedIdeograph3860', [], 'Display', None],
  [ 0x3861, 'CjkUnifiedIdeograph3861', [], 'Display', None],
  [ 0x3862, 'CjkUnifiedIdeograph3862', [], 'Display', None],
  [ 0x3863, 'CjkUnifiedIdeograph3863', [], 'Display', None],
  [ 0x3864, 'CjkUnifiedIdeograph3864', [], 'Display', None],
  [ 0x3865, 'CjkUnifiedIdeograph3865', [], 'Display', None],
  [ 0x3866, 'CjkUnifiedIdeograph3866', [], 'Display', None],
  [ 0x3867, 'CjkUnifiedIdeograph3867', [], 'Display', None],
  [ 0x3868, 'CjkUnifiedIdeograph3868', [], 'Display', None],
  [ 0x3869, 'CjkUnifiedIdeograph3869', [], 'Display', None],
  [ 0x386a, 'CjkUnifiedIdeograph386a', [], 'Display', None],
  [ 0x386b, 'CjkUnifiedIdeograph386b', [], 'Display', None],
  [ 0x386c, 'CjkUnifiedIdeograph386c', [], 'Display', None],
  [ 0x386d, 'CjkUnifiedIdeograph386d', [], 'Display', None],
  [ 0x386e, 'CjkUnifiedIdeograph386e', [], 'Display', None],
  [ 0x386f, 'CjkUnifiedIdeograph386f', [], 'Display', None],
  [ 0x3870, 'CjkUnifiedIdeograph3870', [], 'Display', None],
  [ 0x3871, 'CjkUnifiedIdeograph3871', [], 'Display', None],
  [ 0x3872, 'CjkUnifiedIdeograph3872', [], 'Display', None],
  [ 0x3873, 'CjkUnifiedIdeograph3873', [], 'Display', None],
  [ 0x3874, 'CjkUnifiedIdeograph3874', [], 'Display', None],
  [ 0x3875, 'CjkUnifiedIdeograph3875', [], 'Display', None],
  [ 0x3876, 'CjkUnifiedIdeograph3876', [], 'Display', None],
  [ 0x3877, 'CjkUnifiedIdeograph3877', [], 'Display', None],
  [ 0x3878, 'CjkUnifiedIdeograph3878', [], 'Display', None],
  [ 0x3879, 'CjkUnifiedIdeograph3879', [], 'Display', None],
  [ 0x387a, 'CjkUnifiedIdeograph387a', [], 'Display', None],
  [ 0x387b, 'CjkUnifiedIdeograph387b', [], 'Display', None],
  [ 0x387c, 'CjkUnifiedIdeograph387c', [], 'Display', None],
  [ 0x387d, 'CjkUnifiedIdeograph387d', [], 'Display', None],
  [ 0x387e, 'CjkUnifiedIdeograph387e', [], 'Display', None],
  [ 0x387f, 'CjkUnifiedIdeograph387f', [], 'Display', None],
  [ 0x3880, 'CjkUnifiedIdeograph3880', [], 'Display', None],
  [ 0x3881, 'CjkUnifiedIdeograph3881', [], 'Display', None],
  [ 0x3882, 'CjkUnifiedIdeograph3882', [], 'Display', None],
  [ 0x3883, 'CjkUnifiedIdeograph3883', [], 'Display', None],
  [ 0x3884, 'CjkUnifiedIdeograph3884', [], 'Display', None],
  [ 0x3885, 'CjkUnifiedIdeograph3885', [], 'Display', None],
  [ 0x3886, 'CjkUnifiedIdeograph3886', [], 'Display', None],
  [ 0x3887, 'CjkUnifiedIdeograph3887', [], 'Display', None],
  [ 0x3888, 'CjkUnifiedIdeograph3888', [], 'Display', None],
  [ 0x3889, 'CjkUnifiedIdeograph3889', [], 'Display', None],
  [ 0x388a, 'CjkUnifiedIdeograph388a', [], 'Display', None],
  [ 0x388b, 'CjkUnifiedIdeograph388b', [], 'Display', None],
  [ 0x388c, 'CjkUnifiedIdeograph388c', [], 'Display', None],
  [ 0x388d, 'CjkUnifiedIdeograph388d', [], 'Display', None],
  [ 0x388e, 'CjkUnifiedIdeograph388e', [], 'Display', None],
  [ 0x388f, 'CjkUnifiedIdeograph388f', [], 'Display', None],
  [ 0x3890, 'CjkUnifiedIdeograph3890', [], 'Display', None],
  [ 0x3891, 'CjkUnifiedIdeograph3891', [], 'Display', None],
  [ 0x3892, 'CjkUnifiedIdeograph3892', [], 'Display', None],
  [ 0x3893, 'CjkUnifiedIdeograph3893', [], 'Display', None],
  [ 0x3894, 'CjkUnifiedIdeograph3894', [], 'Display', None],
  [ 0x3895, 'CjkUnifiedIdeograph3895', [], 'Display', None],
  [ 0x3896, 'CjkUnifiedIdeograph3896', [], 'Display', None],
  [ 0x3897, 'CjkUnifiedIdeograph3897', [], 'Display', None],
  [ 0x3898, 'CjkUnifiedIdeograph3898', [], 'Display', None],
  [ 0x3899, 'CjkUnifiedIdeograph3899', [], 'Display', None],
  [ 0x389a, 'CjkUnifiedIdeograph389a', [], 'Display', None],
  [ 0x389b, 'CjkUnifiedIdeograph389b', [], 'Display', None],
  [ 0x389c, 'CjkUnifiedIdeograph389c', [], 'Display', None],
  [ 0x389d, 'CjkUnifiedIdeograph389d', [], 'Display', None],
  [ 0x389e, 'CjkUnifiedIdeograph389e', [], 'Display', None],
  [ 0x389f, 'CjkUnifiedIdeograph389f', [], 'Display', None],
  [ 0x38a0, 'CjkUnifiedIdeograph38a0', [], 'Display', None],
  [ 0x38a1, 'CjkUnifiedIdeograph38a1', [], 'Display', None],
  [ 0x38a2, 'CjkUnifiedIdeograph38a2', [], 'Display', None],
  [ 0x38a3, 'CjkUnifiedIdeograph38a3', [], 'Display', None],
  [ 0x38a4, 'CjkUnifiedIdeograph38a4', [], 'Display', None],
  [ 0x38a5, 'CjkUnifiedIdeograph38a5', [], 'Display', None],
  [ 0x38a6, 'CjkUnifiedIdeograph38a6', [], 'Display', None],
  [ 0x38a7, 'CjkUnifiedIdeograph38a7', [], 'Display', None],
  [ 0x38a8, 'CjkUnifiedIdeograph38a8', [], 'Display', None],
  [ 0x38a9, 'CjkUnifiedIdeograph38a9', [], 'Display', None],
  [ 0x38aa, 'CjkUnifiedIdeograph38aa', [], 'Display', None],
  [ 0x38ab, 'CjkUnifiedIdeograph38ab', [], 'Display', None],
  [ 0x38ac, 'CjkUnifiedIdeograph38ac', [], 'Display', None],
  [ 0x38ad, 'CjkUnifiedIdeograph38ad', [], 'Display', None],
  [ 0x38ae, 'CjkUnifiedIdeograph38ae', [], 'Display', None],
  [ 0x38af, 'CjkUnifiedIdeograph38af', [], 'Display', None],
  [ 0x38b0, 'CjkUnifiedIdeograph38b0', [], 'Display', None],
  [ 0x38b1, 'CjkUnifiedIdeograph38b1', [], 'Display', None],
  [ 0x38b2, 'CjkUnifiedIdeograph38b2', [], 'Display', None],
  [ 0x38b3, 'CjkUnifiedIdeograph38b3', [], 'Display', None],
  [ 0x38b4, 'CjkUnifiedIdeograph38b4', [], 'Display', None],
  [ 0x38b5, 'CjkUnifiedIdeograph38b5', [], 'Display', None],
  [ 0x38b6, 'CjkUnifiedIdeograph38b6', [], 'Display', None],
  [ 0x38b7, 'CjkUnifiedIdeograph38b7', [], 'Display', None],
  [ 0x38b8, 'CjkUnifiedIdeograph38b8', [], 'Display', None],
  [ 0x38b9, 'CjkUnifiedIdeograph38b9', [], 'Display', None],
  [ 0x38ba, 'CjkUnifiedIdeograph38ba', [], 'Display', None],
  [ 0x38bb, 'CjkUnifiedIdeograph38bb', [], 'Display', None],
  [ 0x38bc, 'CjkUnifiedIdeograph38bc', [], 'Display', None],
  [ 0x38bd, 'CjkUnifiedIdeograph38bd', [], 'Display', None],
  [ 0x38be, 'CjkUnifiedIdeograph38be', [], 'Display', None],
  [ 0x38bf, 'CjkUnifiedIdeograph38bf', [], 'Display', None],
  [ 0x38c0, 'CjkUnifiedIdeograph38c0', [], 'Display', None],
  [ 0x38c1, 'CjkUnifiedIdeograph38c1', [], 'Display', None],
  [ 0x38c2, 'CjkUnifiedIdeograph38c2', [], 'Display', None],
  [ 0x38c3, 'CjkUnifiedIdeograph38c3', [], 'Display', None],
  [ 0x38c4, 'CjkUnifiedIdeograph38c4', [], 'Display', None],
  [ 0x38c5, 'CjkUnifiedIdeograph38c5', [], 'Display', None],
  [ 0x38c6, 'CjkUnifiedIdeograph38c6', [], 'Display', None],
  [ 0x38c7, 'CjkUnifiedIdeograph38c7', [], 'Display', None],
  [ 0x38c8, 'CjkUnifiedIdeograph38c8', [], 'Display', None],
  [ 0x38c9, 'CjkUnifiedIdeograph38c9', [], 'Display', None],
  [ 0x38ca, 'CjkUnifiedIdeograph38ca', [], 'Display', None],
  [ 0x38cb, 'CjkUnifiedIdeograph38cb', [], 'Display', None],
  [ 0x38cc, 'CjkUnifiedIdeograph38cc', [], 'Display', None],
  [ 0x38cd, 'CjkUnifiedIdeograph38cd', [], 'Display', None],
  [ 0x38ce, 'CjkUnifiedIdeograph38ce', [], 'Display', None],
  [ 0x38cf, 'CjkUnifiedIdeograph38cf', [], 'Display', None],
  [ 0x38d0, 'CjkUnifiedIdeograph38d0', [], 'Display', None],
  [ 0x38d1, 'CjkUnifiedIdeograph38d1', [], 'Display', None],
  [ 0x38d2, 'CjkUnifiedIdeograph38d2', [], 'Display', None],
  [ 0x38d3, 'CjkUnifiedIdeograph38d3', [], 'Display', None],
  [ 0x38d4, 'CjkUnifiedIdeograph38d4', [], 'Display', None],
  [ 0x38d5, 'CjkUnifiedIdeograph38d5', [], 'Display', None],
  [ 0x38d6, 'CjkUnifiedIdeograph38d6', [], 'Display', None],
  [ 0x38d7, 'CjkUnifiedIdeograph38d7', [], 'Display', None],
  [ 0x38d8, 'CjkUnifiedIdeograph38d8', [], 'Display', None],
  [ 0x38d9, 'CjkUnifiedIdeograph38d9', [], 'Display', None],
  [ 0x38da, 'CjkUnifiedIdeograph38da', [], 'Display', None],
  [ 0x38db, 'CjkUnifiedIdeograph38db', [], 'Display', None],
  [ 0x38dc, 'CjkUnifiedIdeograph38dc', [], 'Display', None],
  [ 0x38dd, 'CjkUnifiedIdeograph38dd', [], 'Display', None],
  [ 0x38de, 'CjkUnifiedIdeograph38de', [], 'Display', None],
  [ 0x38df, 'CjkUnifiedIdeograph38df', [], 'Display', None],
  [ 0x38e0, 'CjkUnifiedIdeograph38e0', [], 'Display', None],
  [ 0x38e1, 'CjkUnifiedIdeograph38e1', [], 'Display', None],
  [ 0x38e2, 'CjkUnifiedIdeograph38e2', [], 'Display', None],
  [ 0x38e3, 'CjkUnifiedIdeograph38e3', [], 'Display', None],
  [ 0x38e4, 'CjkUnifiedIdeograph38e4', [], 'Display', None],
  [ 0x38e5, 'CjkUnifiedIdeograph38e5', [], 'Display', None],
  [ 0x38e6, 'CjkUnifiedIdeograph38e6', [], 'Display', None],
  [ 0x38e7, 'CjkUnifiedIdeograph38e7', [], 'Display', None],
  [ 0x38e8, 'CjkUnifiedIdeograph38e8', [], 'Display', None],
  [ 0x38e9, 'CjkUnifiedIdeograph38e9', [], 'Display', None],
  [ 0x38ea, 'CjkUnifiedIdeograph38ea', [], 'Display', None],
  [ 0x38eb, 'CjkUnifiedIdeograph38eb', [], 'Display', None],
  [ 0x38ec, 'CjkUnifiedIdeograph38ec', [], 'Display', None],
  [ 0x38ed, 'CjkUnifiedIdeograph38ed', [], 'Display', None],
  [ 0x38ee, 'CjkUnifiedIdeograph38ee', [], 'Display', None],
  [ 0x38ef, 'CjkUnifiedIdeograph38ef', [], 'Display', None],
  [ 0x38f0, 'CjkUnifiedIdeograph38f0', [], 'Display', None],
  [ 0x38f1, 'CjkUnifiedIdeograph38f1', [], 'Display', None],
  [ 0x38f2, 'CjkUnifiedIdeograph38f2', [], 'Display', None],
  [ 0x38f3, 'CjkUnifiedIdeograph38f3', [], 'Display', None],
  [ 0x38f4, 'CjkUnifiedIdeograph38f4', [], 'Display', None],
  [ 0x38f5, 'CjkUnifiedIdeograph38f5', [], 'Display', None],
  [ 0x38f6, 'CjkUnifiedIdeograph38f6', [], 'Display', None],
  [ 0x38f7, 'CjkUnifiedIdeograph38f7', [], 'Display', None],
  [ 0x38f8, 'CjkUnifiedIdeograph38f8', [], 'Display', None],
  [ 0x38f9, 'CjkUnifiedIdeograph38f9', [], 'Display', None],
  [ 0x38fa, 'CjkUnifiedIdeograph38fa', [], 'Display', None],
  [ 0x38fb, 'CjkUnifiedIdeograph38fb', [], 'Display', None],
  [ 0x38fc, 'CjkUnifiedIdeograph38fc', [], 'Display', None],
  [ 0x38fd, 'CjkUnifiedIdeograph38fd', [], 'Display', None],
  [ 0x38fe, 'CjkUnifiedIdeograph38fe', [], 'Display', None],
  [ 0x38ff, 'CjkUnifiedIdeograph38ff', [], 'Display', None],
  [ 0x3900, 'CjkUnifiedIdeograph3900', [], 'Display', None],
  [ 0x3901, 'CjkUnifiedIdeograph3901', [], 'Display', None],
  [ 0x3902, 'CjkUnifiedIdeograph3902', [], 'Display', None],
  [ 0x3903, 'CjkUnifiedIdeograph3903', [], 'Display', None],
  [ 0x3904, 'CjkUnifiedIdeograph3904', [], 'Display', None],
  [ 0x3905, 'CjkUnifiedIdeograph3905', [], 'Display', None],
  [ 0x3906, 'CjkUnifiedIdeograph3906', [], 'Display', None],
  [ 0x3907, 'CjkUnifiedIdeograph3907', [], 'Display', None],
  [ 0x3908, 'CjkUnifiedIdeograph3908', [], 'Display', None],
  [ 0x3909, 'CjkUnifiedIdeograph3909', [], 'Display', None],
  [ 0x390a, 'CjkUnifiedIdeograph390a', [], 'Display', None],
  [ 0x390b, 'CjkUnifiedIdeograph390b', [], 'Display', None],
  [ 0x390c, 'CjkUnifiedIdeograph390c', [], 'Display', None],
  [ 0x390d, 'CjkUnifiedIdeograph390d', [], 'Display', None],
  [ 0x390e, 'CjkUnifiedIdeograph390e', [], 'Display', None],
  [ 0x390f, 'CjkUnifiedIdeograph390f', [], 'Display', None],
  [ 0x3910, 'CjkUnifiedIdeograph3910', [], 'Display', None],
  [ 0x3911, 'CjkUnifiedIdeograph3911', [], 'Display', None],
  [ 0x3912, 'CjkUnifiedIdeograph3912', [], 'Display', None],
  [ 0x3913, 'CjkUnifiedIdeograph3913', [], 'Display', None],
  [ 0x3914, 'CjkUnifiedIdeograph3914', [], 'Display', None],
  [ 0x3915, 'CjkUnifiedIdeograph3915', [], 'Display', None],
  [ 0x3916, 'CjkUnifiedIdeograph3916', [], 'Display', None],
  [ 0x3917, 'CjkUnifiedIdeograph3917', [], 'Display', None],
  [ 0x3918, 'CjkUnifiedIdeograph3918', [], 'Display', None],
  [ 0x3919, 'CjkUnifiedIdeograph3919', [], 'Display', None],
  [ 0x391a, 'CjkUnifiedIdeograph391a', [], 'Display', None],
  [ 0x391b, 'CjkUnifiedIdeograph391b', [], 'Display', None],
  [ 0x391c, 'CjkUnifiedIdeograph391c', [], 'Display', None],
  [ 0x391d, 'CjkUnifiedIdeograph391d', [], 'Display', None],
  [ 0x391e, 'CjkUnifiedIdeograph391e', [], 'Display', None],
  [ 0x391f, 'CjkUnifiedIdeograph391f', [], 'Display', None],
  [ 0x3920, 'CjkUnifiedIdeograph3920', [], 'Display', None],
  [ 0x3921, 'CjkUnifiedIdeograph3921', [], 'Display', None],
  [ 0x3922, 'CjkUnifiedIdeograph3922', [], 'Display', None],
  [ 0x3923, 'CjkUnifiedIdeograph3923', [], 'Display', None],
  [ 0x3924, 'CjkUnifiedIdeograph3924', [], 'Display', None],
  [ 0x3925, 'CjkUnifiedIdeograph3925', [], 'Display', None],
  [ 0x3926, 'CjkUnifiedIdeograph3926', [], 'Display', None],
  [ 0x3927, 'CjkUnifiedIdeograph3927', [], 'Display', None],
  [ 0x3928, 'CjkUnifiedIdeograph3928', [], 'Display', None],
  [ 0x3929, 'CjkUnifiedIdeograph3929', [], 'Display', None],
  [ 0x392a, 'CjkUnifiedIdeograph392a', [], 'Display', None],
  [ 0x392b, 'CjkUnifiedIdeograph392b', [], 'Display', None],
  [ 0x392c, 'CjkUnifiedIdeograph392c', [], 'Display', None],
  [ 0x392d, 'CjkUnifiedIdeograph392d', [], 'Display', None],
  [ 0x392e, 'CjkUnifiedIdeograph392e', [], 'Display', None],
  [ 0x392f, 'CjkUnifiedIdeograph392f', [], 'Display', None],
  [ 0x3930, 'CjkUnifiedIdeograph3930', [], 'Display', None],
  [ 0x3931, 'CjkUnifiedIdeograph3931', [], 'Display', None],
  [ 0x3932, 'CjkUnifiedIdeograph3932', [], 'Display', None],
  [ 0x3933, 'CjkUnifiedIdeograph3933', [], 'Display', None],
  [ 0x3934, 'CjkUnifiedIdeograph3934', [], 'Display', None],
  [ 0x3935, 'CjkUnifiedIdeograph3935', [], 'Display', None],
  [ 0x3936, 'CjkUnifiedIdeograph3936', [], 'Display', None],
  [ 0x3937, 'CjkUnifiedIdeograph3937', [], 'Display', None],
  [ 0x3938, 'CjkUnifiedIdeograph3938', [], 'Display', None],
  [ 0x3939, 'CjkUnifiedIdeograph3939', [], 'Display', None],
  [ 0x393a, 'CjkUnifiedIdeograph393a', [], 'Display', None],
  [ 0x393b, 'CjkUnifiedIdeograph393b', [], 'Display', None],
  [ 0x393c, 'CjkUnifiedIdeograph393c', [], 'Display', None],
  [ 0x393d, 'CjkUnifiedIdeograph393d', [], 'Display', None],
  [ 0x393e, 'CjkUnifiedIdeograph393e', [], 'Display', None],
  [ 0x393f, 'CjkUnifiedIdeograph393f', [], 'Display', None],
  [ 0x3940, 'CjkUnifiedIdeograph3940', [], 'Display', None],
  [ 0x3941, 'CjkUnifiedIdeograph3941', [], 'Display', None],
  [ 0x3942, 'CjkUnifiedIdeograph3942', [], 'Display', None],
  [ 0x3943, 'CjkUnifiedIdeograph3943', [], 'Display', None],
  [ 0x3944, 'CjkUnifiedIdeograph3944', [], 'Display', None],
  [ 0x3945, 'CjkUnifiedIdeograph3945', [], 'Display', None],
  [ 0x3946, 'CjkUnifiedIdeograph3946', [], 'Display', None],
  [ 0x3947, 'CjkUnifiedIdeograph3947', [], 'Display', None],
  [ 0x3948, 'CjkUnifiedIdeograph3948', [], 'Display', None],
  [ 0x3949, 'CjkUnifiedIdeograph3949', [], 'Display', None],
  [ 0x394a, 'CjkUnifiedIdeograph394a', [], 'Display', None],
  [ 0x394b, 'CjkUnifiedIdeograph394b', [], 'Display', None],
  [ 0x394c, 'CjkUnifiedIdeograph394c', [], 'Display', None],
  [ 0x394d, 'CjkUnifiedIdeograph394d', [], 'Display', None],
  [ 0x394e, 'CjkUnifiedIdeograph394e', [], 'Display', None],
  [ 0x394f, 'CjkUnifiedIdeograph394f', [], 'Display', None],
  [ 0x3950, 'CjkUnifiedIdeograph3950', [], 'Display', None],
  [ 0x3951, 'CjkUnifiedIdeograph3951', [], 'Display', None],
  [ 0x3952, 'CjkUnifiedIdeograph3952', [], 'Display', None],
  [ 0x3953, 'CjkUnifiedIdeograph3953', [], 'Display', None],
  [ 0x3954, 'CjkUnifiedIdeograph3954', [], 'Display', None],
  [ 0x3955, 'CjkUnifiedIdeograph3955', [], 'Display', None],
  [ 0x3956, 'CjkUnifiedIdeograph3956', [], 'Display', None],
  [ 0x3957, 'CjkUnifiedIdeograph3957', [], 'Display', None],
  [ 0x3958, 'CjkUnifiedIdeograph3958', [], 'Display', None],
  [ 0x3959, 'CjkUnifiedIdeograph3959', [], 'Display', None],
  [ 0x395a, 'CjkUnifiedIdeograph395a', [], 'Display', None],
  [ 0x395b, 'CjkUnifiedIdeograph395b', [], 'Display', None],
  [ 0x395c, 'CjkUnifiedIdeograph395c', [], 'Display', None],
  [ 0x395d, 'CjkUnifiedIdeograph395d', [], 'Display', None],
  [ 0x395e, 'CjkUnifiedIdeograph395e', [], 'Display', None],
  [ 0x395f, 'CjkUnifiedIdeograph395f', [], 'Display', None],
  [ 0x3960, 'CjkUnifiedIdeograph3960', [], 'Display', None],
  [ 0x3961, 'CjkUnifiedIdeograph3961', [], 'Display', None],
  [ 0x3962, 'CjkUnifiedIdeograph3962', [], 'Display', None],
  [ 0x3963, 'CjkUnifiedIdeograph3963', [], 'Display', None],
  [ 0x3964, 'CjkUnifiedIdeograph3964', [], 'Display', None],
  [ 0x3965, 'CjkUnifiedIdeograph3965', [], 'Display', None],
  [ 0x3966, 'CjkUnifiedIdeograph3966', [], 'Display', None],
  [ 0x3967, 'CjkUnifiedIdeograph3967', [], 'Display', None],
  [ 0x3968, 'CjkUnifiedIdeograph3968', [], 'Display', None],
  [ 0x3969, 'CjkUnifiedIdeograph3969', [], 'Display', None],
  [ 0x396a, 'CjkUnifiedIdeograph396a', [], 'Display', None],
  [ 0x396b, 'CjkUnifiedIdeograph396b', [], 'Display', None],
  [ 0x396c, 'CjkUnifiedIdeograph396c', [], 'Display', None],
  [ 0x396d, 'CjkUnifiedIdeograph396d', [], 'Display', None],
  [ 0x396e, 'CjkUnifiedIdeograph396e', [], 'Display', None],
  [ 0x396f, 'CjkUnifiedIdeograph396f', [], 'Display', None],
  [ 0x3970, 'CjkUnifiedIdeograph3970', [], 'Display', None],
  [ 0x3971, 'CjkUnifiedIdeograph3971', [], 'Display', None],
  [ 0x3972, 'CjkUnifiedIdeograph3972', [], 'Display', None],
  [ 0x3973, 'CjkUnifiedIdeograph3973', [], 'Display', None],
  [ 0x3974, 'CjkUnifiedIdeograph3974', [], 'Display', None],
  [ 0x3975, 'CjkUnifiedIdeograph3975', [], 'Display', None],
  [ 0x3976, 'CjkUnifiedIdeograph3976', [], 'Display', None],
  [ 0x3977, 'CjkUnifiedIdeograph3977', [], 'Display', None],
  [ 0x3978, 'CjkUnifiedIdeograph3978', [], 'Display', None],
  [ 0x3979, 'CjkUnifiedIdeograph3979', [], 'Display', None],
  [ 0x397a, 'CjkUnifiedIdeograph397a', [], 'Display', None],
  [ 0x397b, 'CjkUnifiedIdeograph397b', [], 'Display', None],
  [ 0x397c, 'CjkUnifiedIdeograph397c', [], 'Display', None],
  [ 0x397d, 'CjkUnifiedIdeograph397d', [], 'Display', None],
  [ 0x397e, 'CjkUnifiedIdeograph397e', [], 'Display', None],
  [ 0x397f, 'CjkUnifiedIdeograph397f', [], 'Display', None],
  [ 0x3980, 'CjkUnifiedIdeograph3980', [], 'Display', None],
  [ 0x3981, 'CjkUnifiedIdeograph3981', [], 'Display', None],
  [ 0x3982, 'CjkUnifiedIdeograph3982', [], 'Display', None],
  [ 0x3983, 'CjkUnifiedIdeograph3983', [], 'Display', None],
  [ 0x3984, 'CjkUnifiedIdeograph3984', [], 'Display', None],
  [ 0x3985, 'CjkUnifiedIdeograph3985', [], 'Display', None],
  [ 0x3986, 'CjkUnifiedIdeograph3986', [], 'Display', None],
  [ 0x3987, 'CjkUnifiedIdeograph3987', [], 'Display', None],
  [ 0x3988, 'CjkUnifiedIdeograph3988', [], 'Display', None],
  [ 0x3989, 'CjkUnifiedIdeograph3989', [], 'Display', None],
  [ 0x398a, 'CjkUnifiedIdeograph398a', [], 'Display', None],
  [ 0x398b, 'CjkUnifiedIdeograph398b', [], 'Display', None],
  [ 0x398c, 'CjkUnifiedIdeograph398c', [], 'Display', None],
  [ 0x398d, 'CjkUnifiedIdeograph398d', [], 'Display', None],
  [ 0x398e, 'CjkUnifiedIdeograph398e', [], 'Display', None],
  [ 0x398f, 'CjkUnifiedIdeograph398f', [], 'Display', None],
  [ 0x3990, 'CjkUnifiedIdeograph3990', [], 'Display', None],
  [ 0x3991, 'CjkUnifiedIdeograph3991', [], 'Display', None],
  [ 0x3992, 'CjkUnifiedIdeograph3992', [], 'Display', None],
  [ 0x3993, 'CjkUnifiedIdeograph3993', [], 'Display', None],
  [ 0x3994, 'CjkUnifiedIdeograph3994', [], 'Display', None],
  [ 0x3995, 'CjkUnifiedIdeograph3995', [], 'Display', None],
  [ 0x3996, 'CjkUnifiedIdeograph3996', [], 'Display', None],
  [ 0x3997, 'CjkUnifiedIdeograph3997', [], 'Display', None],
  [ 0x3998, 'CjkUnifiedIdeograph3998', [], 'Display', None],
  [ 0x3999, 'CjkUnifiedIdeograph3999', [], 'Display', None],
  [ 0x399a, 'CjkUnifiedIdeograph399a', [], 'Display', None],
  [ 0x399b, 'CjkUnifiedIdeograph399b', [], 'Display', None],
  [ 0x399c, 'CjkUnifiedIdeograph399c', [], 'Display', None],
  [ 0x399d, 'CjkUnifiedIdeograph399d', [], 'Display', None],
  [ 0x399e, 'CjkUnifiedIdeograph399e', [], 'Display', None],
  [ 0x399f, 'CjkUnifiedIdeograph399f', [], 'Display', None],
  [ 0x39a0, 'CjkUnifiedIdeograph39a0', [], 'Display', None],
  [ 0x39a1, 'CjkUnifiedIdeograph39a1', [], 'Display', None],
  [ 0x39a2, 'CjkUnifiedIdeograph39a2', [], 'Display', None],
  [ 0x39a3, 'CjkUnifiedIdeograph39a3', [], 'Display', None],
  [ 0x39a4, 'CjkUnifiedIdeograph39a4', [], 'Display', None],
  [ 0x39a5, 'CjkUnifiedIdeograph39a5', [], 'Display', None],
  [ 0x39a6, 'CjkUnifiedIdeograph39a6', [], 'Display', None],
  [ 0x39a7, 'CjkUnifiedIdeograph39a7', [], 'Display', None],
  [ 0x39a8, 'CjkUnifiedIdeograph39a8', [], 'Display', None],
  [ 0x39a9, 'CjkUnifiedIdeograph39a9', [], 'Display', None],
  [ 0x39aa, 'CjkUnifiedIdeograph39aa', [], 'Display', None],
  [ 0x39ab, 'CjkUnifiedIdeograph39ab', [], 'Display', None],
  [ 0x39ac, 'CjkUnifiedIdeograph39ac', [], 'Display', None],
  [ 0x39ad, 'CjkUnifiedIdeograph39ad', [], 'Display', None],
  [ 0x39ae, 'CjkUnifiedIdeograph39ae', [], 'Display', None],
  [ 0x39af, 'CjkUnifiedIdeograph39af', [], 'Display', None],
  [ 0x39b0, 'CjkUnifiedIdeograph39b0', [], 'Display', None],
  [ 0x39b1, 'CjkUnifiedIdeograph39b1', [], 'Display', None],
  [ 0x39b2, 'CjkUnifiedIdeograph39b2', [], 'Display', None],
  [ 0x39b3, 'CjkUnifiedIdeograph39b3', [], 'Display', None],
  [ 0x39b4, 'CjkUnifiedIdeograph39b4', [], 'Display', None],
  [ 0x39b5, 'CjkUnifiedIdeograph39b5', [], 'Display', None],
  [ 0x39b6, 'CjkUnifiedIdeograph39b6', [], 'Display', None],
  [ 0x39b7, 'CjkUnifiedIdeograph39b7', [], 'Display', None],
  [ 0x39b8, 'CjkUnifiedIdeograph39b8', [], 'Display', None],
  [ 0x39b9, 'CjkUnifiedIdeograph39b9', [], 'Display', None],
  [ 0x39ba, 'CjkUnifiedIdeograph39ba', [], 'Display', None],
  [ 0x39bb, 'CjkUnifiedIdeograph39bb', [], 'Display', None],
  [ 0x39bc, 'CjkUnifiedIdeograph39bc', [], 'Display', None],
  [ 0x39bd, 'CjkUnifiedIdeograph39bd', [], 'Display', None],
  [ 0x39be, 'CjkUnifiedIdeograph39be', [], 'Display', None],
  [ 0x39bf, 'CjkUnifiedIdeograph39bf', [], 'Display', None],
  [ 0x39c0, 'CjkUnifiedIdeograph39c0', [], 'Display', None],
  [ 0x39c1, 'CjkUnifiedIdeograph39c1', [], 'Display', None],
  [ 0x39c2, 'CjkUnifiedIdeograph39c2', [], 'Display', None],
  [ 0x39c3, 'CjkUnifiedIdeograph39c3', [], 'Display', None],
  [ 0x39c4, 'CjkUnifiedIdeograph39c4', [], 'Display', None],
  [ 0x39c5, 'CjkUnifiedIdeograph39c5', [], 'Display', None],
  [ 0x39c6, 'CjkUnifiedIdeograph39c6', [], 'Display', None],
  [ 0x39c7, 'CjkUnifiedIdeograph39c7', [], 'Display', None],
  [ 0x39c8, 'CjkUnifiedIdeograph39c8', [], 'Display', None],
  [ 0x39c9, 'CjkUnifiedIdeograph39c9', [], 'Display', None],
  [ 0x39ca, 'CjkUnifiedIdeograph39ca', [], 'Display', None],
  [ 0x39cb, 'CjkUnifiedIdeograph39cb', [], 'Display', None],
  [ 0x39cc, 'CjkUnifiedIdeograph39cc', [], 'Display', None],
  [ 0x39cd, 'CjkUnifiedIdeograph39cd', [], 'Display', None],
  [ 0x39ce, 'CjkUnifiedIdeograph39ce', [], 'Display', None],
  [ 0x39cf, 'CjkUnifiedIdeograph39cf', [], 'Display', None],
  [ 0x39d0, 'CjkUnifiedIdeograph39d0', [], 'Display', None],
  [ 0x39d1, 'CjkUnifiedIdeograph39d1', [], 'Display', None],
  [ 0x39d2, 'CjkUnifiedIdeograph39d2', [], 'Display', None],
  [ 0x39d3, 'CjkUnifiedIdeograph39d3', [], 'Display', None],
  [ 0x39d4, 'CjkUnifiedIdeograph39d4', [], 'Display', None],
  [ 0x39d5, 'CjkUnifiedIdeograph39d5', [], 'Display', None],
  [ 0x39d6, 'CjkUnifiedIdeograph39d6', [], 'Display', None],
  [ 0x39d7, 'CjkUnifiedIdeograph39d7', [], 'Display', None],
  [ 0x39d8, 'CjkUnifiedIdeograph39d8', [], 'Display', None],
  [ 0x39d9, 'CjkUnifiedIdeograph39d9', [], 'Display', None],
  [ 0x39da, 'CjkUnifiedIdeograph39da', [], 'Display', None],
  [ 0x39db, 'CjkUnifiedIdeograph39db', [], 'Display', None],
  [ 0x39dc, 'CjkUnifiedIdeograph39dc', [], 'Display', None],
  [ 0x39dd, 'CjkUnifiedIdeograph39dd', [], 'Display', None],
  [ 0x39de, 'CjkUnifiedIdeograph39de', [], 'Display', None],
  [ 0x39df, 'CjkUnifiedIdeograph39df', [], 'Display', None],
  [ 0x39e0, 'CjkUnifiedIdeograph39e0', [], 'Display', None],
  [ 0x39e1, 'CjkUnifiedIdeograph39e1', [], 'Display', None],
  [ 0x39e2, 'CjkUnifiedIdeograph39e2', [], 'Display', None],
  [ 0x39e3, 'CjkUnifiedIdeograph39e3', [], 'Display', None],
  [ 0x39e4, 'CjkUnifiedIdeograph39e4', [], 'Display', None],
  [ 0x39e5, 'CjkUnifiedIdeograph39e5', [], 'Display', None],
  [ 0x39e6, 'CjkUnifiedIdeograph39e6', [], 'Display', None],
  [ 0x39e7, 'CjkUnifiedIdeograph39e7', [], 'Display', None],
  [ 0x39e8, 'CjkUnifiedIdeograph39e8', [], 'Display', None],
  [ 0x39e9, 'CjkUnifiedIdeograph39e9', [], 'Display', None],
  [ 0x39ea, 'CjkUnifiedIdeograph39ea', [], 'Display', None],
  [ 0x39eb, 'CjkUnifiedIdeograph39eb', [], 'Display', None],
  [ 0x39ec, 'CjkUnifiedIdeograph39ec', [], 'Display', None],
  [ 0x39ed, 'CjkUnifiedIdeograph39ed', [], 'Display', None],
  [ 0x39ee, 'CjkUnifiedIdeograph39ee', [], 'Display', None],
  [ 0x39ef, 'CjkUnifiedIdeograph39ef', [], 'Display', None],
  [ 0x39f0, 'CjkUnifiedIdeograph39f0', [], 'Display', None],
  [ 0x39f1, 'CjkUnifiedIdeograph39f1', [], 'Display', None],
  [ 0x39f2, 'CjkUnifiedIdeograph39f2', [], 'Display', None],
  [ 0x39f3, 'CjkUnifiedIdeograph39f3', [], 'Display', None],
  [ 0x39f4, 'CjkUnifiedIdeograph39f4', [], 'Display', None],
  [ 0x39f5, 'CjkUnifiedIdeograph39f5', [], 'Display', None],
  [ 0x39f6, 'CjkUnifiedIdeograph39f6', [], 'Display', None],
  [ 0x39f7, 'CjkUnifiedIdeograph39f7', [], 'Display', None],
  [ 0x39f8, 'CjkUnifiedIdeograph39f8', [], 'Display', None],
  [ 0x39f9, 'CjkUnifiedIdeograph39f9', [], 'Display', None],
  [ 0x39fa, 'CjkUnifiedIdeograph39fa', [], 'Display', None],
  [ 0x39fb, 'CjkUnifiedIdeograph39fb', [], 'Display', None],
  [ 0x39fc, 'CjkUnifiedIdeograph39fc', [], 'Display', None],
  [ 0x39fd, 'CjkUnifiedIdeograph39fd', [], 'Display', None],
  [ 0x39fe, 'CjkUnifiedIdeograph39fe', [], 'Display', None],
  [ 0x39ff, 'CjkUnifiedIdeograph39ff', [], 'Display', None],
  [ 0x3a00, 'CjkUnifiedIdeograph3a00', [], 'Display', None],
  [ 0x3a01, 'CjkUnifiedIdeograph3a01', [], 'Display', None],
  [ 0x3a02, 'CjkUnifiedIdeograph3a02', [], 'Display', None],
  [ 0x3a03, 'CjkUnifiedIdeograph3a03', [], 'Display', None],
  [ 0x3a04, 'CjkUnifiedIdeograph3a04', [], 'Display', None],
  [ 0x3a05, 'CjkUnifiedIdeograph3a05', [], 'Display', None],
  [ 0x3a06, 'CjkUnifiedIdeograph3a06', [], 'Display', None],
  [ 0x3a07, 'CjkUnifiedIdeograph3a07', [], 'Display', None],
  [ 0x3a08, 'CjkUnifiedIdeograph3a08', [], 'Display', None],
  [ 0x3a09, 'CjkUnifiedIdeograph3a09', [], 'Display', None],
  [ 0x3a0a, 'CjkUnifiedIdeograph3a0a', [], 'Display', None],
  [ 0x3a0b, 'CjkUnifiedIdeograph3a0b', [], 'Display', None],
  [ 0x3a0c, 'CjkUnifiedIdeograph3a0c', [], 'Display', None],
  [ 0x3a0d, 'CjkUnifiedIdeograph3a0d', [], 'Display', None],
  [ 0x3a0e, 'CjkUnifiedIdeograph3a0e', [], 'Display', None],
  [ 0x3a0f, 'CjkUnifiedIdeograph3a0f', [], 'Display', None],
  [ 0x3a10, 'CjkUnifiedIdeograph3a10', [], 'Display', None],
  [ 0x3a11, 'CjkUnifiedIdeograph3a11', [], 'Display', None],
  [ 0x3a12, 'CjkUnifiedIdeograph3a12', [], 'Display', None],
  [ 0x3a13, 'CjkUnifiedIdeograph3a13', [], 'Display', None],
  [ 0x3a14, 'CjkUnifiedIdeograph3a14', [], 'Display', None],
  [ 0x3a15, 'CjkUnifiedIdeograph3a15', [], 'Display', None],
  [ 0x3a16, 'CjkUnifiedIdeograph3a16', [], 'Display', None],
  [ 0x3a17, 'CjkUnifiedIdeograph3a17', [], 'Display', None],
  [ 0x3a18, 'CjkUnifiedIdeograph3a18', [], 'Display', None],
  [ 0x3a19, 'CjkUnifiedIdeograph3a19', [], 'Display', None],
  [ 0x3a1a, 'CjkUnifiedIdeograph3a1a', [], 'Display', None],
  [ 0x3a1b, 'CjkUnifiedIdeograph3a1b', [], 'Display', None],
  [ 0x3a1c, 'CjkUnifiedIdeograph3a1c', [], 'Display', None],
  [ 0x3a1d, 'CjkUnifiedIdeograph3a1d', [], 'Display', None],
  [ 0x3a1e, 'CjkUnifiedIdeograph3a1e', [], 'Display', None],
  [ 0x3a1f, 'CjkUnifiedIdeograph3a1f', [], 'Display', None],
  [ 0x3a20, 'CjkUnifiedIdeograph3a20', [], 'Display', None],
  [ 0x3a21, 'CjkUnifiedIdeograph3a21', [], 'Display', None],
  [ 0x3a22, 'CjkUnifiedIdeograph3a22', [], 'Display', None],
  [ 0x3a23, 'CjkUnifiedIdeograph3a23', [], 'Display', None],
  [ 0x3a24, 'CjkUnifiedIdeograph3a24', [], 'Display', None],
  [ 0x3a25, 'CjkUnifiedIdeograph3a25', [], 'Display', None],
  [ 0x3a26, 'CjkUnifiedIdeograph3a26', [], 'Display', None],
  [ 0x3a27, 'CjkUnifiedIdeograph3a27', [], 'Display', None],
  [ 0x3a28, 'CjkUnifiedIdeograph3a28', [], 'Display', None],
  [ 0x3a29, 'CjkUnifiedIdeograph3a29', [], 'Display', None],
  [ 0x3a2a, 'CjkUnifiedIdeograph3a2a', [], 'Display', None],
  [ 0x3a2b, 'CjkUnifiedIdeograph3a2b', [], 'Display', None],
  [ 0x3a2c, 'CjkUnifiedIdeograph3a2c', [], 'Display', None],
  [ 0x3a2d, 'CjkUnifiedIdeograph3a2d', [], 'Display', None],
  [ 0x3a2e, 'CjkUnifiedIdeograph3a2e', [], 'Display', None],
  [ 0x3a2f, 'CjkUnifiedIdeograph3a2f', [], 'Display', None],
  [ 0x3a30, 'CjkUnifiedIdeograph3a30', [], 'Display', None],
  [ 0x3a31, 'CjkUnifiedIdeograph3a31', [], 'Display', None],
  [ 0x3a32, 'CjkUnifiedIdeograph3a32', [], 'Display', None],
  [ 0x3a33, 'CjkUnifiedIdeograph3a33', [], 'Display', None],
  [ 0x3a34, 'CjkUnifiedIdeograph3a34', [], 'Display', None],
  [ 0x3a35, 'CjkUnifiedIdeograph3a35', [], 'Display', None],
  [ 0x3a36, 'CjkUnifiedIdeograph3a36', [], 'Display', None],
  [ 0x3a37, 'CjkUnifiedIdeograph3a37', [], 'Display', None],
  [ 0x3a38, 'CjkUnifiedIdeograph3a38', [], 'Display', None],
  [ 0x3a39, 'CjkUnifiedIdeograph3a39', [], 'Display', None],
  [ 0x3a3a, 'CjkUnifiedIdeograph3a3a', [], 'Display', None],
  [ 0x3a3b, 'CjkUnifiedIdeograph3a3b', [], 'Display', None],
  [ 0x3a3c, 'CjkUnifiedIdeograph3a3c', [], 'Display', None],
  [ 0x3a3d, 'CjkUnifiedIdeograph3a3d', [], 'Display', None],
  [ 0x3a3e, 'CjkUnifiedIdeograph3a3e', [], 'Display', None],
  [ 0x3a3f, 'CjkUnifiedIdeograph3a3f', [], 'Display', None],
  [ 0x3a40, 'CjkUnifiedIdeograph3a40', [], 'Display', None],
  [ 0x3a41, 'CjkUnifiedIdeograph3a41', [], 'Display', None],
  [ 0x3a42, 'CjkUnifiedIdeograph3a42', [], 'Display', None],
  [ 0x3a43, 'CjkUnifiedIdeograph3a43', [], 'Display', None],
  [ 0x3a44, 'CjkUnifiedIdeograph3a44', [], 'Display', None],
  [ 0x3a45, 'CjkUnifiedIdeograph3a45', [], 'Display', None],
  [ 0x3a46, 'CjkUnifiedIdeograph3a46', [], 'Display', None],
  [ 0x3a47, 'CjkUnifiedIdeograph3a47', [], 'Display', None],
  [ 0x3a48, 'CjkUnifiedIdeograph3a48', [], 'Display', None],
  [ 0x3a49, 'CjkUnifiedIdeograph3a49', [], 'Display', None],
  [ 0x3a4a, 'CjkUnifiedIdeograph3a4a', [], 'Display', None],
  [ 0x3a4b, 'CjkUnifiedIdeograph3a4b', [], 'Display', None],
  [ 0x3a4c, 'CjkUnifiedIdeograph3a4c', [], 'Display', None],
  [ 0x3a4d, 'CjkUnifiedIdeograph3a4d', [], 'Display', None],
  [ 0x3a4e, 'CjkUnifiedIdeograph3a4e', [], 'Display', None],
  [ 0x3a4f, 'CjkUnifiedIdeograph3a4f', [], 'Display', None],
  [ 0x3a50, 'CjkUnifiedIdeograph3a50', [], 'Display', None],
  [ 0x3a51, 'CjkUnifiedIdeograph3a51', [], 'Display', None],
  [ 0x3a52, 'CjkUnifiedIdeograph3a52', [], 'Display', None],
  [ 0x3a53, 'CjkUnifiedIdeograph3a53', [], 'Display', None],
  [ 0x3a54, 'CjkUnifiedIdeograph3a54', [], 'Display', None],
  [ 0x3a55, 'CjkUnifiedIdeograph3a55', [], 'Display', None],
  [ 0x3a56, 'CjkUnifiedIdeograph3a56', [], 'Display', None],
  [ 0x3a57, 'CjkUnifiedIdeograph3a57', [], 'Display', None],
  [ 0x3a58, 'CjkUnifiedIdeograph3a58', [], 'Display', None],
  [ 0x3a59, 'CjkUnifiedIdeograph3a59', [], 'Display', None],
  [ 0x3a5a, 'CjkUnifiedIdeograph3a5a', [], 'Display', None],
  [ 0x3a5b, 'CjkUnifiedIdeograph3a5b', [], 'Display', None],
  [ 0x3a5c, 'CjkUnifiedIdeograph3a5c', [], 'Display', None],
  [ 0x3a5d, 'CjkUnifiedIdeograph3a5d', [], 'Display', None],
  [ 0x3a5e, 'CjkUnifiedIdeograph3a5e', [], 'Display', None],
  [ 0x3a5f, 'CjkUnifiedIdeograph3a5f', [], 'Display', None],
  [ 0x3a60, 'CjkUnifiedIdeograph3a60', [], 'Display', None],
  [ 0x3a61, 'CjkUnifiedIdeograph3a61', [], 'Display', None],
  [ 0x3a62, 'CjkUnifiedIdeograph3a62', [], 'Display', None],
  [ 0x3a63, 'CjkUnifiedIdeograph3a63', [], 'Display', None],
  [ 0x3a64, 'CjkUnifiedIdeograph3a64', [], 'Display', None],
  [ 0x3a65, 'CjkUnifiedIdeograph3a65', [], 'Display', None],
  [ 0x3a66, 'CjkUnifiedIdeograph3a66', [], 'Display', None],
  [ 0x3a67, 'CjkUnifiedIdeograph3a67', [], 'Display', None],
  [ 0x3a68, 'CjkUnifiedIdeograph3a68', [], 'Display', None],
  [ 0x3a69, 'CjkUnifiedIdeograph3a69', [], 'Display', None],
  [ 0x3a6a, 'CjkUnifiedIdeograph3a6a', [], 'Display', None],
  [ 0x3a6b, 'CjkUnifiedIdeograph3a6b', [], 'Display', None],
  [ 0x3a6c, 'CjkUnifiedIdeograph3a6c', [], 'Display', None],
  [ 0x3a6d, 'CjkUnifiedIdeograph3a6d', [], 'Display', None],
  [ 0x3a6e, 'CjkUnifiedIdeograph3a6e', [], 'Display', None],
  [ 0x3a6f, 'CjkUnifiedIdeograph3a6f', [], 'Display', None],
  [ 0x3a70, 'CjkUnifiedIdeograph3a70', [], 'Display', None],
  [ 0x3a71, 'CjkUnifiedIdeograph3a71', [], 'Display', None],
  [ 0x3a72, 'CjkUnifiedIdeograph3a72', [], 'Display', None],
  [ 0x3a73, 'CjkUnifiedIdeograph3a73', [], 'Display', None],
  [ 0x3a74, 'CjkUnifiedIdeograph3a74', [], 'Display', None],
  [ 0x3a75, 'CjkUnifiedIdeograph3a75', [], 'Display', None],
  [ 0x3a76, 'CjkUnifiedIdeograph3a76', [], 'Display', None],
  [ 0x3a77, 'CjkUnifiedIdeograph3a77', [], 'Display', None],
  [ 0x3a78, 'CjkUnifiedIdeograph3a78', [], 'Display', None],
  [ 0x3a79, 'CjkUnifiedIdeograph3a79', [], 'Display', None],
  [ 0x3a7a, 'CjkUnifiedIdeograph3a7a', [], 'Display', None],
  [ 0x3a7b, 'CjkUnifiedIdeograph3a7b', [], 'Display', None],
  [ 0x3a7c, 'CjkUnifiedIdeograph3a7c', [], 'Display', None],
  [ 0x3a7d, 'CjkUnifiedIdeograph3a7d', [], 'Display', None],
  [ 0x3a7e, 'CjkUnifiedIdeograph3a7e', [], 'Display', None],
  [ 0x3a7f, 'CjkUnifiedIdeograph3a7f', [], 'Display', None],
  [ 0x3a80, 'CjkUnifiedIdeograph3a80', [], 'Display', None],
  [ 0x3a81, 'CjkUnifiedIdeograph3a81', [], 'Display', None],
  [ 0x3a82, 'CjkUnifiedIdeograph3a82', [], 'Display', None],
  [ 0x3a83, 'CjkUnifiedIdeograph3a83', [], 'Display', None],
  [ 0x3a84, 'CjkUnifiedIdeograph3a84', [], 'Display', None],
  [ 0x3a85, 'CjkUnifiedIdeograph3a85', [], 'Display', None],
  [ 0x3a86, 'CjkUnifiedIdeograph3a86', [], 'Display', None],
  [ 0x3a87, 'CjkUnifiedIdeograph3a87', [], 'Display', None],
  [ 0x3a88, 'CjkUnifiedIdeograph3a88', [], 'Display', None],
  [ 0x3a89, 'CjkUnifiedIdeograph3a89', [], 'Display', None],
  [ 0x3a8a, 'CjkUnifiedIdeograph3a8a', [], 'Display', None],
  [ 0x3a8b, 'CjkUnifiedIdeograph3a8b', [], 'Display', None],
  [ 0x3a8c, 'CjkUnifiedIdeograph3a8c', [], 'Display', None],
  [ 0x3a8d, 'CjkUnifiedIdeograph3a8d', [], 'Display', None],
  [ 0x3a8e, 'CjkUnifiedIdeograph3a8e', [], 'Display', None],
  [ 0x3a8f, 'CjkUnifiedIdeograph3a8f', [], 'Display', None],
  [ 0x3a90, 'CjkUnifiedIdeograph3a90', [], 'Display', None],
  [ 0x3a91, 'CjkUnifiedIdeograph3a91', [], 'Display', None],
  [ 0x3a92, 'CjkUnifiedIdeograph3a92', [], 'Display', None],
  [ 0x3a93, 'CjkUnifiedIdeograph3a93', [], 'Display', None],
  [ 0x3a94, 'CjkUnifiedIdeograph3a94', [], 'Display', None],
  [ 0x3a95, 'CjkUnifiedIdeograph3a95', [], 'Display', None],
  [ 0x3a96, 'CjkUnifiedIdeograph3a96', [], 'Display', None],
  [ 0x3a97, 'CjkUnifiedIdeograph3a97', [], 'Display', None],
  [ 0x3a98, 'CjkUnifiedIdeograph3a98', [], 'Display', None],
  [ 0x3a99, 'CjkUnifiedIdeograph3a99', [], 'Display', None],
  [ 0x3a9a, 'CjkUnifiedIdeograph3a9a', [], 'Display', None],
  [ 0x3a9b, 'CjkUnifiedIdeograph3a9b', [], 'Display', None],
  [ 0x3a9c, 'CjkUnifiedIdeograph3a9c', [], 'Display', None],
  [ 0x3a9d, 'CjkUnifiedIdeograph3a9d', [], 'Display', None],
  [ 0x3a9e, 'CjkUnifiedIdeograph3a9e', [], 'Display', None],
  [ 0x3a9f, 'CjkUnifiedIdeograph3a9f', [], 'Display', None],
  [ 0x3aa0, 'CjkUnifiedIdeograph3aa0', [], 'Display', None],
  [ 0x3aa1, 'CjkUnifiedIdeograph3aa1', [], 'Display', None],
  [ 0x3aa2, 'CjkUnifiedIdeograph3aa2', [], 'Display', None],
  [ 0x3aa3, 'CjkUnifiedIdeograph3aa3', [], 'Display', None],
  [ 0x3aa4, 'CjkUnifiedIdeograph3aa4', [], 'Display', None],
  [ 0x3aa5, 'CjkUnifiedIdeograph3aa5', [], 'Display', None],
  [ 0x3aa6, 'CjkUnifiedIdeograph3aa6', [], 'Display', None],
  [ 0x3aa7, 'CjkUnifiedIdeograph3aa7', [], 'Display', None],
  [ 0x3aa8, 'CjkUnifiedIdeograph3aa8', [], 'Display', None],
  [ 0x3aa9, 'CjkUnifiedIdeograph3aa9', [], 'Display', None],
  [ 0x3aaa, 'CjkUnifiedIdeograph3aaa', [], 'Display', None],
  [ 0x3aab, 'CjkUnifiedIdeograph3aab', [], 'Display', None],
  [ 0x3aac, 'CjkUnifiedIdeograph3aac', [], 'Display', None],
  [ 0x3aad, 'CjkUnifiedIdeograph3aad', [], 'Display', None],
  [ 0x3aae, 'CjkUnifiedIdeograph3aae', [], 'Display', None],
  [ 0x3aaf, 'CjkUnifiedIdeograph3aaf', [], 'Display', None],
  [ 0x3ab0, 'CjkUnifiedIdeograph3ab0', [], 'Display', None],
  [ 0x3ab1, 'CjkUnifiedIdeograph3ab1', [], 'Display', None],
  [ 0x3ab2, 'CjkUnifiedIdeograph3ab2', [], 'Display', None],
  [ 0x3ab3, 'CjkUnifiedIdeograph3ab3', [], 'Display', None],
  [ 0x3ab4, 'CjkUnifiedIdeograph3ab4', [], 'Display', None],
  [ 0x3ab5, 'CjkUnifiedIdeograph3ab5', [], 'Display', None],
  [ 0x3ab6, 'CjkUnifiedIdeograph3ab6', [], 'Display', None],
  [ 0x3ab7, 'CjkUnifiedIdeograph3ab7', [], 'Display', None],
  [ 0x3ab8, 'CjkUnifiedIdeograph3ab8', [], 'Display', None],
  [ 0x3ab9, 'CjkUnifiedIdeograph3ab9', [], 'Display', None],
  [ 0x3aba, 'CjkUnifiedIdeograph3aba', [], 'Display', None],
  [ 0x3abb, 'CjkUnifiedIdeograph3abb', [], 'Display', None],
  [ 0x3abc, 'CjkUnifiedIdeograph3abc', [], 'Display', None],
  [ 0x3abd, 'CjkUnifiedIdeograph3abd', [], 'Display', None],
  [ 0x3abe, 'CjkUnifiedIdeograph3abe', [], 'Display', None],
  [ 0x3abf, 'CjkUnifiedIdeograph3abf', [], 'Display', None],
  [ 0x3ac0, 'CjkUnifiedIdeograph3ac0', [], 'Display', None],
  [ 0x3ac1, 'CjkUnifiedIdeograph3ac1', [], 'Display', None],
  [ 0x3ac2, 'CjkUnifiedIdeograph3ac2', [], 'Display', None],
  [ 0x3ac3, 'CjkUnifiedIdeograph3ac3', [], 'Display', None],
  [ 0x3ac4, 'CjkUnifiedIdeograph3ac4', [], 'Display', None],
  [ 0x3ac5, 'CjkUnifiedIdeograph3ac5', [], 'Display', None],
  [ 0x3ac6, 'CjkUnifiedIdeograph3ac6', [], 'Display', None],
  [ 0x3ac7, 'CjkUnifiedIdeograph3ac7', [], 'Display', None],
  [ 0x3ac8, 'CjkUnifiedIdeograph3ac8', [], 'Display', None],
  [ 0x3ac9, 'CjkUnifiedIdeograph3ac9', [], 'Display', None],
  [ 0x3aca, 'CjkUnifiedIdeograph3aca', [], 'Display', None],
  [ 0x3acb, 'CjkUnifiedIdeograph3acb', [], 'Display', None],
  [ 0x3acc, 'CjkUnifiedIdeograph3acc', [], 'Display', None],
  [ 0x3acd, 'CjkUnifiedIdeograph3acd', [], 'Display', None],
  [ 0x3ace, 'CjkUnifiedIdeograph3ace', [], 'Display', None],
  [ 0x3acf, 'CjkUnifiedIdeograph3acf', [], 'Display', None],
  [ 0x3ad0, 'CjkUnifiedIdeograph3ad0', [], 'Display', None],
  [ 0x3ad1, 'CjkUnifiedIdeograph3ad1', [], 'Display', None],
  [ 0x3ad2, 'CjkUnifiedIdeograph3ad2', [], 'Display', None],
  [ 0x3ad3, 'CjkUnifiedIdeograph3ad3', [], 'Display', None],
  [ 0x3ad4, 'CjkUnifiedIdeograph3ad4', [], 'Display', None],
  [ 0x3ad5, 'CjkUnifiedIdeograph3ad5', [], 'Display', None],
  [ 0x3ad6, 'CjkUnifiedIdeograph3ad6', [], 'Display', None],
  [ 0x3ad7, 'CjkUnifiedIdeograph3ad7', [], 'Display', None],
  [ 0x3ad8, 'CjkUnifiedIdeograph3ad8', [], 'Display', None],
  [ 0x3ad9, 'CjkUnifiedIdeograph3ad9', [], 'Display', None],
  [ 0x3ada, 'CjkUnifiedIdeograph3ada', [], 'Display', None],
  [ 0x3adb, 'CjkUnifiedIdeograph3adb', [], 'Display', None],
  [ 0x3adc, 'CjkUnifiedIdeograph3adc', [], 'Display', None],
  [ 0x3add, 'CjkUnifiedIdeograph3add', [], 'Display', None],
  [ 0x3ade, 'CjkUnifiedIdeograph3ade', [], 'Display', None],
  [ 0x3adf, 'CjkUnifiedIdeograph3adf', [], 'Display', None],
  [ 0x3ae0, 'CjkUnifiedIdeograph3ae0', [], 'Display', None],
  [ 0x3ae1, 'CjkUnifiedIdeograph3ae1', [], 'Display', None],
  [ 0x3ae2, 'CjkUnifiedIdeograph3ae2', [], 'Display', None],
  [ 0x3ae3, 'CjkUnifiedIdeograph3ae3', [], 'Display', None],
  [ 0x3ae4, 'CjkUnifiedIdeograph3ae4', [], 'Display', None],
  [ 0x3ae5, 'CjkUnifiedIdeograph3ae5', [], 'Display', None],
  [ 0x3ae6, 'CjkUnifiedIdeograph3ae6', [], 'Display', None],
  [ 0x3ae7, 'CjkUnifiedIdeograph3ae7', [], 'Display', None],
  [ 0x3ae8, 'CjkUnifiedIdeograph3ae8', [], 'Display', None],
  [ 0x3ae9, 'CjkUnifiedIdeograph3ae9', [], 'Display', None],
  [ 0x3aea, 'CjkUnifiedIdeograph3aea', [], 'Display', None],
  [ 0x3aeb, 'CjkUnifiedIdeograph3aeb', [], 'Display', None],
  [ 0x3aec, 'CjkUnifiedIdeograph3aec', [], 'Display', None],
  [ 0x3aed, 'CjkUnifiedIdeograph3aed', [], 'Display', None],
  [ 0x3aee, 'CjkUnifiedIdeograph3aee', [], 'Display', None],
  [ 0x3aef, 'CjkUnifiedIdeograph3aef', [], 'Display', None],
  [ 0x3af0, 'CjkUnifiedIdeograph3af0', [], 'Display', None],
  [ 0x3af1, 'CjkUnifiedIdeograph3af1', [], 'Display', None],
  [ 0x3af2, 'CjkUnifiedIdeograph3af2', [], 'Display', None],
  [ 0x3af3, 'CjkUnifiedIdeograph3af3', [], 'Display', None],
  [ 0x3af4, 'CjkUnifiedIdeograph3af4', [], 'Display', None],
  [ 0x3af5, 'CjkUnifiedIdeograph3af5', [], 'Display', None],
  [ 0x3af6, 'CjkUnifiedIdeograph3af6', [], 'Display', None],
  [ 0x3af7, 'CjkUnifiedIdeograph3af7', [], 'Display', None],
  [ 0x3af8, 'CjkUnifiedIdeograph3af8', [], 'Display', None],
  [ 0x3af9, 'CjkUnifiedIdeograph3af9', [], 'Display', None],
  [ 0x3afa, 'CjkUnifiedIdeograph3afa', [], 'Display', None],
  [ 0x3afb, 'CjkUnifiedIdeograph3afb', [], 'Display', None],
  [ 0x3afc, 'CjkUnifiedIdeograph3afc', [], 'Display', None],
  [ 0x3afd, 'CjkUnifiedIdeograph3afd', [], 'Display', None],
  [ 0x3afe, 'CjkUnifiedIdeograph3afe', [], 'Display', None],
  [ 0x3aff, 'CjkUnifiedIdeograph3aff', [], 'Display', None],
  [ 0x3b00, 'CjkUnifiedIdeograph3b00', [], 'Display', None],
  [ 0x3b01, 'CjkUnifiedIdeograph3b01', [], 'Display', None],
  [ 0x3b02, 'CjkUnifiedIdeograph3b02', [], 'Display', None],
  [ 0x3b03, 'CjkUnifiedIdeograph3b03', [], 'Display', None],
  [ 0x3b04, 'CjkUnifiedIdeograph3b04', [], 'Display', None],
  [ 0x3b05, 'CjkUnifiedIdeograph3b05', [], 'Display', None],
  [ 0x3b06, 'CjkUnifiedIdeograph3b06', [], 'Display', None],
  [ 0x3b07, 'CjkUnifiedIdeograph3b07', [], 'Display', None],
  [ 0x3b08, 'CjkUnifiedIdeograph3b08', [], 'Display', None],
  [ 0x3b09, 'CjkUnifiedIdeograph3b09', [], 'Display', None],
  [ 0x3b0a, 'CjkUnifiedIdeograph3b0a', [], 'Display', None],
  [ 0x3b0b, 'CjkUnifiedIdeograph3b0b', [], 'Display', None],
  [ 0x3b0c, 'CjkUnifiedIdeograph3b0c', [], 'Display', None],
  [ 0x3b0d, 'CjkUnifiedIdeograph3b0d', [], 'Display', None],
  [ 0x3b0e, 'CjkUnifiedIdeograph3b0e', [], 'Display', None],
  [ 0x3b0f, 'CjkUnifiedIdeograph3b0f', [], 'Display', None],
  [ 0x3b10, 'CjkUnifiedIdeograph3b10', [], 'Display', None],
  [ 0x3b11, 'CjkUnifiedIdeograph3b11', [], 'Display', None],
  [ 0x3b12, 'CjkUnifiedIdeograph3b12', [], 'Display', None],
  [ 0x3b13, 'CjkUnifiedIdeograph3b13', [], 'Display', None],
  [ 0x3b14, 'CjkUnifiedIdeograph3b14', [], 'Display', None],
  [ 0x3b15, 'CjkUnifiedIdeograph3b15', [], 'Display', None],
  [ 0x3b16, 'CjkUnifiedIdeograph3b16', [], 'Display', None],
  [ 0x3b17, 'CjkUnifiedIdeograph3b17', [], 'Display', None],
  [ 0x3b18, 'CjkUnifiedIdeograph3b18', [], 'Display', None],
  [ 0x3b19, 'CjkUnifiedIdeograph3b19', [], 'Display', None],
  [ 0x3b1a, 'CjkUnifiedIdeograph3b1a', [], 'Display', None],
  [ 0x3b1b, 'CjkUnifiedIdeograph3b1b', [], 'Display', None],
  [ 0x3b1c, 'CjkUnifiedIdeograph3b1c', [], 'Display', None],
  [ 0x3b1d, 'CjkUnifiedIdeograph3b1d', [], 'Display', None],
  [ 0x3b1e, 'CjkUnifiedIdeograph3b1e', [], 'Display', None],
  [ 0x3b1f, 'CjkUnifiedIdeograph3b1f', [], 'Display', None],
  [ 0x3b20, 'CjkUnifiedIdeograph3b20', [], 'Display', None],
  [ 0x3b21, 'CjkUnifiedIdeograph3b21', [], 'Display', None],
  [ 0x3b22, 'CjkUnifiedIdeograph3b22', [], 'Display', None],
  [ 0x3b23, 'CjkUnifiedIdeograph3b23', [], 'Display', None],
  [ 0x3b24, 'CjkUnifiedIdeograph3b24', [], 'Display', None],
  [ 0x3b25, 'CjkUnifiedIdeograph3b25', [], 'Display', None],
  [ 0x3b26, 'CjkUnifiedIdeograph3b26', [], 'Display', None],
  [ 0x3b27, 'CjkUnifiedIdeograph3b27', [], 'Display', None],
  [ 0x3b28, 'CjkUnifiedIdeograph3b28', [], 'Display', None],
  [ 0x3b29, 'CjkUnifiedIdeograph3b29', [], 'Display', None],
  [ 0x3b2a, 'CjkUnifiedIdeograph3b2a', [], 'Display', None],
  [ 0x3b2b, 'CjkUnifiedIdeograph3b2b', [], 'Display', None],
  [ 0x3b2c, 'CjkUnifiedIdeograph3b2c', [], 'Display', None],
  [ 0x3b2d, 'CjkUnifiedIdeograph3b2d', [], 'Display', None],
  [ 0x3b2e, 'CjkUnifiedIdeograph3b2e', [], 'Display', None],
  [ 0x3b2f, 'CjkUnifiedIdeograph3b2f', [], 'Display', None],
  [ 0x3b30, 'CjkUnifiedIdeograph3b30', [], 'Display', None],
  [ 0x3b31, 'CjkUnifiedIdeograph3b31', [], 'Display', None],
  [ 0x3b32, 'CjkUnifiedIdeograph3b32', [], 'Display', None],
  [ 0x3b33, 'CjkUnifiedIdeograph3b33', [], 'Display', None],
  [ 0x3b34, 'CjkUnifiedIdeograph3b34', [], 'Display', None],
  [ 0x3b35, 'CjkUnifiedIdeograph3b35', [], 'Display', None],
  [ 0x3b36, 'CjkUnifiedIdeograph3b36', [], 'Display', None],
  [ 0x3b37, 'CjkUnifiedIdeograph3b37', [], 'Display', None],
  [ 0x3b38, 'CjkUnifiedIdeograph3b38', [], 'Display', None],
  [ 0x3b39, 'CjkUnifiedIdeograph3b39', [], 'Display', None],
  [ 0x3b3a, 'CjkUnifiedIdeograph3b3a', [], 'Display', None],
  [ 0x3b3b, 'CjkUnifiedIdeograph3b3b', [], 'Display', None],
  [ 0x3b3c, 'CjkUnifiedIdeograph3b3c', [], 'Display', None],
  [ 0x3b3d, 'CjkUnifiedIdeograph3b3d', [], 'Display', None],
  [ 0x3b3e, 'CjkUnifiedIdeograph3b3e', [], 'Display', None],
  [ 0x3b3f, 'CjkUnifiedIdeograph3b3f', [], 'Display', None],
  [ 0x3b40, 'CjkUnifiedIdeograph3b40', [], 'Display', None],
  [ 0x3b41, 'CjkUnifiedIdeograph3b41', [], 'Display', None],
  [ 0x3b42, 'CjkUnifiedIdeograph3b42', [], 'Display', None],
  [ 0x3b43, 'CjkUnifiedIdeograph3b43', [], 'Display', None],
  [ 0x3b44, 'CjkUnifiedIdeograph3b44', [], 'Display', None],
  [ 0x3b45, 'CjkUnifiedIdeograph3b45', [], 'Display', None],
  [ 0x3b46, 'CjkUnifiedIdeograph3b46', [], 'Display', None],
  [ 0x3b47, 'CjkUnifiedIdeograph3b47', [], 'Display', None],
  [ 0x3b48, 'CjkUnifiedIdeograph3b48', [], 'Display', None],
  [ 0x3b49, 'CjkUnifiedIdeograph3b49', [], 'Display', None],
  [ 0x3b4a, 'CjkUnifiedIdeograph3b4a', [], 'Display', None],
  [ 0x3b4b, 'CjkUnifiedIdeograph3b4b', [], 'Display', None],
  [ 0x3b4c, 'CjkUnifiedIdeograph3b4c', [], 'Display', None],
  [ 0x3b4d, 'CjkUnifiedIdeograph3b4d', [], 'Display', None],
  [ 0x3b4e, 'CjkUnifiedIdeograph3b4e', [], 'Display', None],
  [ 0x3b4f, 'CjkUnifiedIdeograph3b4f', [], 'Display', None],
  [ 0x3b50, 'CjkUnifiedIdeograph3b50', [], 'Display', None],
  [ 0x3b51, 'CjkUnifiedIdeograph3b51', [], 'Display', None],
  [ 0x3b52, 'CjkUnifiedIdeograph3b52', [], 'Display', None],
  [ 0x3b53, 'CjkUnifiedIdeograph3b53', [], 'Display', None],
  [ 0x3b54, 'CjkUnifiedIdeograph3b54', [], 'Display', None],
  [ 0x3b55, 'CjkUnifiedIdeograph3b55', [], 'Display', None],
  [ 0x3b56, 'CjkUnifiedIdeograph3b56', [], 'Display', None],
  [ 0x3b57, 'CjkUnifiedIdeograph3b57', [], 'Display', None],
  [ 0x3b58, 'CjkUnifiedIdeograph3b58', [], 'Display', None],
  [ 0x3b59, 'CjkUnifiedIdeograph3b59', [], 'Display', None],
  [ 0x3b5a, 'CjkUnifiedIdeograph3b5a', [], 'Display', None],
  [ 0x3b5b, 'CjkUnifiedIdeograph3b5b', [], 'Display', None],
  [ 0x3b5c, 'CjkUnifiedIdeograph3b5c', [], 'Display', None],
  [ 0x3b5d, 'CjkUnifiedIdeograph3b5d', [], 'Display', None],
  [ 0x3b5e, 'CjkUnifiedIdeograph3b5e', [], 'Display', None],
  [ 0x3b5f, 'CjkUnifiedIdeograph3b5f', [], 'Display', None],
  [ 0x3b60, 'CjkUnifiedIdeograph3b60', [], 'Display', None],
  [ 0x3b61, 'CjkUnifiedIdeograph3b61', [], 'Display', None],
  [ 0x3b62, 'CjkUnifiedIdeograph3b62', [], 'Display', None],
  [ 0x3b63, 'CjkUnifiedIdeograph3b63', [], 'Display', None],
  [ 0x3b64, 'CjkUnifiedIdeograph3b64', [], 'Display', None],
  [ 0x3b65, 'CjkUnifiedIdeograph3b65', [], 'Display', None],
  [ 0x3b66, 'CjkUnifiedIdeograph3b66', [], 'Display', None],
  [ 0x3b67, 'CjkUnifiedIdeograph3b67', [], 'Display', None],
  [ 0x3b68, 'CjkUnifiedIdeograph3b68', [], 'Display', None],
  [ 0x3b69, 'CjkUnifiedIdeograph3b69', [], 'Display', None],
  [ 0x3b6a, 'CjkUnifiedIdeograph3b6a', [], 'Display', None],
  [ 0x3b6b, 'CjkUnifiedIdeograph3b6b', [], 'Display', None],
  [ 0x3b6c, 'CjkUnifiedIdeograph3b6c', [], 'Display', None],
  [ 0x3b6d, 'CjkUnifiedIdeograph3b6d', [], 'Display', None],
  [ 0x3b6e, 'CjkUnifiedIdeograph3b6e', [], 'Display', None],
  [ 0x3b6f, 'CjkUnifiedIdeograph3b6f', [], 'Display', None],
  [ 0x3b70, 'CjkUnifiedIdeograph3b70', [], 'Display', None],
  [ 0x3b71, 'CjkUnifiedIdeograph3b71', [], 'Display', None],
  [ 0x3b72, 'CjkUnifiedIdeograph3b72', [], 'Display', None],
  [ 0x3b73, 'CjkUnifiedIdeograph3b73', [], 'Display', None],
  [ 0x3b74, 'CjkUnifiedIdeograph3b74', [], 'Display', None],
  [ 0x3b75, 'CjkUnifiedIdeograph3b75', [], 'Display', None],
  [ 0x3b76, 'CjkUnifiedIdeograph3b76', [], 'Display', None],
  [ 0x3b77, 'CjkUnifiedIdeograph3b77', [], 'Display', None],
  [ 0x3b78, 'CjkUnifiedIdeograph3b78', [], 'Display', None],
  [ 0x3b79, 'CjkUnifiedIdeograph3b79', [], 'Display', None],
  [ 0x3b7a, 'CjkUnifiedIdeograph3b7a', [], 'Display', None],
  [ 0x3b7b, 'CjkUnifiedIdeograph3b7b', [], 'Display', None],
  [ 0x3b7c, 'CjkUnifiedIdeograph3b7c', [], 'Display', None],
  [ 0x3b7d, 'CjkUnifiedIdeograph3b7d', [], 'Display', None],
  [ 0x3b7e, 'CjkUnifiedIdeograph3b7e', [], 'Display', None],
  [ 0x3b7f, 'CjkUnifiedIdeograph3b7f', [], 'Display', None],
  [ 0x3b80, 'CjkUnifiedIdeograph3b80', [], 'Display', None],
  [ 0x3b81, 'CjkUnifiedIdeograph3b81', [], 'Display', None],
  [ 0x3b82, 'CjkUnifiedIdeograph3b82', [], 'Display', None],
  [ 0x3b83, 'CjkUnifiedIdeograph3b83', [], 'Display', None],
  [ 0x3b84, 'CjkUnifiedIdeograph3b84', [], 'Display', None],
  [ 0x3b85, 'CjkUnifiedIdeograph3b85', [], 'Display', None],
  [ 0x3b86, 'CjkUnifiedIdeograph3b86', [], 'Display', None],
  [ 0x3b87, 'CjkUnifiedIdeograph3b87', [], 'Display', None],
  [ 0x3b88, 'CjkUnifiedIdeograph3b88', [], 'Display', None],
  [ 0x3b89, 'CjkUnifiedIdeograph3b89', [], 'Display', None],
  [ 0x3b8a, 'CjkUnifiedIdeograph3b8a', [], 'Display', None],
  [ 0x3b8b, 'CjkUnifiedIdeograph3b8b', [], 'Display', None],
  [ 0x3b8c, 'CjkUnifiedIdeograph3b8c', [], 'Display', None],
  [ 0x3b8d, 'CjkUnifiedIdeograph3b8d', [], 'Display', None],
  [ 0x3b8e, 'CjkUnifiedIdeograph3b8e', [], 'Display', None],
  [ 0x3b8f, 'CjkUnifiedIdeograph3b8f', [], 'Display', None],
  [ 0x3b90, 'CjkUnifiedIdeograph3b90', [], 'Display', None],
  [ 0x3b91, 'CjkUnifiedIdeograph3b91', [], 'Display', None],
  [ 0x3b92, 'CjkUnifiedIdeograph3b92', [], 'Display', None],
  [ 0x3b93, 'CjkUnifiedIdeograph3b93', [], 'Display', None],
  [ 0x3b94, 'CjkUnifiedIdeograph3b94', [], 'Display', None],
  [ 0x3b95, 'CjkUnifiedIdeograph3b95', [], 'Display', None],
  [ 0x3b96, 'CjkUnifiedIdeograph3b96', [], 'Display', None],
  [ 0x3b97, 'CjkUnifiedIdeograph3b97', [], 'Display', None],
  [ 0x3b98, 'CjkUnifiedIdeograph3b98', [], 'Display', None],
  [ 0x3b99, 'CjkUnifiedIdeograph3b99', [], 'Display', None],
  [ 0x3b9a, 'CjkUnifiedIdeograph3b9a', [], 'Display', None],
  [ 0x3b9b, 'CjkUnifiedIdeograph3b9b', [], 'Display', None],
  [ 0x3b9c, 'CjkUnifiedIdeograph3b9c', [], 'Display', None],
  [ 0x3b9d, 'CjkUnifiedIdeograph3b9d', [], 'Display', None],
  [ 0x3b9e, 'CjkUnifiedIdeograph3b9e', [], 'Display', None],
  [ 0x3b9f, 'CjkUnifiedIdeograph3b9f', [], 'Display', None],
  [ 0x3ba0, 'CjkUnifiedIdeograph3ba0', [], 'Display', None],
  [ 0x3ba1, 'CjkUnifiedIdeograph3ba1', [], 'Display', None],
  [ 0x3ba2, 'CjkUnifiedIdeograph3ba2', [], 'Display', None],
  [ 0x3ba3, 'CjkUnifiedIdeograph3ba3', [], 'Display', None],
  [ 0x3ba4, 'CjkUnifiedIdeograph3ba4', [], 'Display', None],
  [ 0x3ba5, 'CjkUnifiedIdeograph3ba5', [], 'Display', None],
  [ 0x3ba6, 'CjkUnifiedIdeograph3ba6', [], 'Display', None],
  [ 0x3ba7, 'CjkUnifiedIdeograph3ba7', [], 'Display', None],
  [ 0x3ba8, 'CjkUnifiedIdeograph3ba8', [], 'Display', None],
  [ 0x3ba9, 'CjkUnifiedIdeograph3ba9', [], 'Display', None],
  [ 0x3baa, 'CjkUnifiedIdeograph3baa', [], 'Display', None],
  [ 0x3bab, 'CjkUnifiedIdeograph3bab', [], 'Display', None],
  [ 0x3bac, 'CjkUnifiedIdeograph3bac', [], 'Display', None],
  [ 0x3bad, 'CjkUnifiedIdeograph3bad', [], 'Display', None],
  [ 0x3bae, 'CjkUnifiedIdeograph3bae', [], 'Display', None],
  [ 0x3baf, 'CjkUnifiedIdeograph3baf', [], 'Display', None],
  [ 0x3bb0, 'CjkUnifiedIdeograph3bb0', [], 'Display', None],
  [ 0x3bb1, 'CjkUnifiedIdeograph3bb1', [], 'Display', None],
  [ 0x3bb2, 'CjkUnifiedIdeograph3bb2', [], 'Display', None],
  [ 0x3bb3, 'CjkUnifiedIdeograph3bb3', [], 'Display', None],
  [ 0x3bb4, 'CjkUnifiedIdeograph3bb4', [], 'Display', None],
  [ 0x3bb5, 'CjkUnifiedIdeograph3bb5', [], 'Display', None],
  [ 0x3bb6, 'CjkUnifiedIdeograph3bb6', [], 'Display', None],
  [ 0x3bb7, 'CjkUnifiedIdeograph3bb7', [], 'Display', None],
  [ 0x3bb8, 'CjkUnifiedIdeograph3bb8', [], 'Display', None],
  [ 0x3bb9, 'CjkUnifiedIdeograph3bb9', [], 'Display', None],
  [ 0x3bba, 'CjkUnifiedIdeograph3bba', [], 'Display', None],
  [ 0x3bbb, 'CjkUnifiedIdeograph3bbb', [], 'Display', None],
  [ 0x3bbc, 'CjkUnifiedIdeograph3bbc', [], 'Display', None],
  [ 0x3bbd, 'CjkUnifiedIdeograph3bbd', [], 'Display', None],
  [ 0x3bbe, 'CjkUnifiedIdeograph3bbe', [], 'Display', None],
  [ 0x3bbf, 'CjkUnifiedIdeograph3bbf', [], 'Display', None],
  [ 0x3bc0, 'CjkUnifiedIdeograph3bc0', [], 'Display', None],
  [ 0x3bc1, 'CjkUnifiedIdeograph3bc1', [], 'Display', None],
  [ 0x3bc2, 'CjkUnifiedIdeograph3bc2', [], 'Display', None],
  [ 0x3bc3, 'CjkUnifiedIdeograph3bc3', [], 'Display', None],
  [ 0x3bc4, 'CjkUnifiedIdeograph3bc4', [], 'Display', None],
  [ 0x3bc5, 'CjkUnifiedIdeograph3bc5', [], 'Display', None],
  [ 0x3bc6, 'CjkUnifiedIdeograph3bc6', [], 'Display', None],
  [ 0x3bc7, 'CjkUnifiedIdeograph3bc7', [], 'Display', None],
  [ 0x3bc8, 'CjkUnifiedIdeograph3bc8', [], 'Display', None],
  [ 0x3bc9, 'CjkUnifiedIdeograph3bc9', [], 'Display', None],
  [ 0x3bca, 'CjkUnifiedIdeograph3bca', [], 'Display', None],
  [ 0x3bcb, 'CjkUnifiedIdeograph3bcb', [], 'Display', None],
  [ 0x3bcc, 'CjkUnifiedIdeograph3bcc', [], 'Display', None],
  [ 0x3bcd, 'CjkUnifiedIdeograph3bcd', [], 'Display', None],
  [ 0x3bce, 'CjkUnifiedIdeograph3bce', [], 'Display', None],
  [ 0x3bcf, 'CjkUnifiedIdeograph3bcf', [], 'Display', None],
  [ 0x3bd0, 'CjkUnifiedIdeograph3bd0', [], 'Display', None],
  [ 0x3bd1, 'CjkUnifiedIdeograph3bd1', [], 'Display', None],
  [ 0x3bd2, 'CjkUnifiedIdeograph3bd2', [], 'Display', None],
  [ 0x3bd3, 'CjkUnifiedIdeograph3bd3', [], 'Display', None],
  [ 0x3bd4, 'CjkUnifiedIdeograph3bd4', [], 'Display', None],
  [ 0x3bd5, 'CjkUnifiedIdeograph3bd5', [], 'Display', None],
  [ 0x3bd6, 'CjkUnifiedIdeograph3bd6', [], 'Display', None],
  [ 0x3bd7, 'CjkUnifiedIdeograph3bd7', [], 'Display', None],
  [ 0x3bd8, 'CjkUnifiedIdeograph3bd8', [], 'Display', None],
  [ 0x3bd9, 'CjkUnifiedIdeograph3bd9', [], 'Display', None],
  [ 0x3bda, 'CjkUnifiedIdeograph3bda', [], 'Display', None],
  [ 0x3bdb, 'CjkUnifiedIdeograph3bdb', [], 'Display', None],
  [ 0x3bdc, 'CjkUnifiedIdeograph3bdc', [], 'Display', None],
  [ 0x3bdd, 'CjkUnifiedIdeograph3bdd', [], 'Display', None],
  [ 0x3bde, 'CjkUnifiedIdeograph3bde', [], 'Display', None],
  [ 0x3bdf, 'CjkUnifiedIdeograph3bdf', [], 'Display', None],
  [ 0x3be0, 'CjkUnifiedIdeograph3be0', [], 'Display', None],
  [ 0x3be1, 'CjkUnifiedIdeograph3be1', [], 'Display', None],
  [ 0x3be2, 'CjkUnifiedIdeograph3be2', [], 'Display', None],
  [ 0x3be3, 'CjkUnifiedIdeograph3be3', [], 'Display', None],
  [ 0x3be4, 'CjkUnifiedIdeograph3be4', [], 'Display', None],
  [ 0x3be5, 'CjkUnifiedIdeograph3be5', [], 'Display', None],
  [ 0x3be6, 'CjkUnifiedIdeograph3be6', [], 'Display', None],
  [ 0x3be7, 'CjkUnifiedIdeograph3be7', [], 'Display', None],
  [ 0x3be8, 'CjkUnifiedIdeograph3be8', [], 'Display', None],
  [ 0x3be9, 'CjkUnifiedIdeograph3be9', [], 'Display', None],
  [ 0x3bea, 'CjkUnifiedIdeograph3bea', [], 'Display', None],
  [ 0x3beb, 'CjkUnifiedIdeograph3beb', [], 'Display', None],
  [ 0x3bec, 'CjkUnifiedIdeograph3bec', [], 'Display', None],
  [ 0x3bed, 'CjkUnifiedIdeograph3bed', [], 'Display', None],
  [ 0x3bee, 'CjkUnifiedIdeograph3bee', [], 'Display', None],
  [ 0x3bef, 'CjkUnifiedIdeograph3bef', [], 'Display', None],
  [ 0x3bf0, 'CjkUnifiedIdeograph3bf0', [], 'Display', None],
  [ 0x3bf1, 'CjkUnifiedIdeograph3bf1', [], 'Display', None],
  [ 0x3bf2, 'CjkUnifiedIdeograph3bf2', [], 'Display', None],
  [ 0x3bf3, 'CjkUnifiedIdeograph3bf3', [], 'Display', None],
  [ 0x3bf4, 'CjkUnifiedIdeograph3bf4', [], 'Display', None],
  [ 0x3bf5, 'CjkUnifiedIdeograph3bf5', [], 'Display', None],
  [ 0x3bf6, 'CjkUnifiedIdeograph3bf6', [], 'Display', None],
  [ 0x3bf7, 'CjkUnifiedIdeograph3bf7', [], 'Display', None],
  [ 0x3bf8, 'CjkUnifiedIdeograph3bf8', [], 'Display', None],
  [ 0x3bf9, 'CjkUnifiedIdeograph3bf9', [], 'Display', None],
  [ 0x3bfa, 'CjkUnifiedIdeograph3bfa', [], 'Display', None],
  [ 0x3bfb, 'CjkUnifiedIdeograph3bfb', [], 'Display', None],
  [ 0x3bfc, 'CjkUnifiedIdeograph3bfc', [], 'Display', None],
  [ 0x3bfd, 'CjkUnifiedIdeograph3bfd', [], 'Display', None],
  [ 0x3bfe, 'CjkUnifiedIdeograph3bfe', [], 'Display', None],
  [ 0x3bff, 'CjkUnifiedIdeograph3bff', [], 'Display', None],
  [ 0x3c00, 'CjkUnifiedIdeograph3c00', [], 'Display', None],
  [ 0x3c01, 'CjkUnifiedIdeograph3c01', [], 'Display', None],
  [ 0x3c02, 'CjkUnifiedIdeograph3c02', [], 'Display', None],
  [ 0x3c03, 'CjkUnifiedIdeograph3c03', [], 'Display', None],
  [ 0x3c04, 'CjkUnifiedIdeograph3c04', [], 'Display', None],
  [ 0x3c05, 'CjkUnifiedIdeograph3c05', [], 'Display', None],
  [ 0x3c06, 'CjkUnifiedIdeograph3c06', [], 'Display', None],
  [ 0x3c07, 'CjkUnifiedIdeograph3c07', [], 'Display', None],
  [ 0x3c08, 'CjkUnifiedIdeograph3c08', [], 'Display', None],
  [ 0x3c09, 'CjkUnifiedIdeograph3c09', [], 'Display', None],
  [ 0x3c0a, 'CjkUnifiedIdeograph3c0a', [], 'Display', None],
  [ 0x3c0b, 'CjkUnifiedIdeograph3c0b', [], 'Display', None],
  [ 0x3c0c, 'CjkUnifiedIdeograph3c0c', [], 'Display', None],
  [ 0x3c0d, 'CjkUnifiedIdeograph3c0d', [], 'Display', None],
  [ 0x3c0e, 'CjkUnifiedIdeograph3c0e', [], 'Display', None],
  [ 0x3c0f, 'CjkUnifiedIdeograph3c0f', [], 'Display', None],
  [ 0x3c10, 'CjkUnifiedIdeograph3c10', [], 'Display', None],
  [ 0x3c11, 'CjkUnifiedIdeograph3c11', [], 'Display', None],
  [ 0x3c12, 'CjkUnifiedIdeograph3c12', [], 'Display', None],
  [ 0x3c13, 'CjkUnifiedIdeograph3c13', [], 'Display', None],
  [ 0x3c14, 'CjkUnifiedIdeograph3c14', [], 'Display', None],
  [ 0x3c15, 'CjkUnifiedIdeograph3c15', [], 'Display', None],
  [ 0x3c16, 'CjkUnifiedIdeograph3c16', [], 'Display', None],
  [ 0x3c17, 'CjkUnifiedIdeograph3c17', [], 'Display', None],
  [ 0x3c18, 'CjkUnifiedIdeograph3c18', [], 'Display', None],
  [ 0x3c19, 'CjkUnifiedIdeograph3c19', [], 'Display', None],
  [ 0x3c1a, 'CjkUnifiedIdeograph3c1a', [], 'Display', None],
  [ 0x3c1b, 'CjkUnifiedIdeograph3c1b', [], 'Display', None],
  [ 0x3c1c, 'CjkUnifiedIdeograph3c1c', [], 'Display', None],
  [ 0x3c1d, 'CjkUnifiedIdeograph3c1d', [], 'Display', None],
  [ 0x3c1e, 'CjkUnifiedIdeograph3c1e', [], 'Display', None],
  [ 0x3c1f, 'CjkUnifiedIdeograph3c1f', [], 'Display', None],
  [ 0x3c20, 'CjkUnifiedIdeograph3c20', [], 'Display', None],
  [ 0x3c21, 'CjkUnifiedIdeograph3c21', [], 'Display', None],
  [ 0x3c22, 'CjkUnifiedIdeograph3c22', [], 'Display', None],
  [ 0x3c23, 'CjkUnifiedIdeograph3c23', [], 'Display', None],
  [ 0x3c24, 'CjkUnifiedIdeograph3c24', [], 'Display', None],
  [ 0x3c25, 'CjkUnifiedIdeograph3c25', [], 'Display', None],
  [ 0x3c26, 'CjkUnifiedIdeograph3c26', [], 'Display', None],
  [ 0x3c27, 'CjkUnifiedIdeograph3c27', [], 'Display', None],
  [ 0x3c28, 'CjkUnifiedIdeograph3c28', [], 'Display', None],
  [ 0x3c29, 'CjkUnifiedIdeograph3c29', [], 'Display', None],
  [ 0x3c2a, 'CjkUnifiedIdeograph3c2a', [], 'Display', None],
  [ 0x3c2b, 'CjkUnifiedIdeograph3c2b', [], 'Display', None],
  [ 0x3c2c, 'CjkUnifiedIdeograph3c2c', [], 'Display', None],
  [ 0x3c2d, 'CjkUnifiedIdeograph3c2d', [], 'Display', None],
  [ 0x3c2e, 'CjkUnifiedIdeograph3c2e', [], 'Display', None],
  [ 0x3c2f, 'CjkUnifiedIdeograph3c2f', [], 'Display', None],
  [ 0x3c30, 'CjkUnifiedIdeograph3c30', [], 'Display', None],
  [ 0x3c31, 'CjkUnifiedIdeograph3c31', [], 'Display', None],
  [ 0x3c32, 'CjkUnifiedIdeograph3c32', [], 'Display', None],
  [ 0x3c33, 'CjkUnifiedIdeograph3c33', [], 'Display', None],
  [ 0x3c34, 'CjkUnifiedIdeograph3c34', [], 'Display', None],
  [ 0x3c35, 'CjkUnifiedIdeograph3c35', [], 'Display', None],
  [ 0x3c36, 'CjkUnifiedIdeograph3c36', [], 'Display', None],
  [ 0x3c37, 'CjkUnifiedIdeograph3c37', [], 'Display', None],
  [ 0x3c38, 'CjkUnifiedIdeograph3c38', [], 'Display', None],
  [ 0x3c39, 'CjkUnifiedIdeograph3c39', [], 'Display', None],
  [ 0x3c3a, 'CjkUnifiedIdeograph3c3a', [], 'Display', None],
  [ 0x3c3b, 'CjkUnifiedIdeograph3c3b', [], 'Display', None],
  [ 0x3c3c, 'CjkUnifiedIdeograph3c3c', [], 'Display', None],
  [ 0x3c3d, 'CjkUnifiedIdeograph3c3d', [], 'Display', None],
  [ 0x3c3e, 'CjkUnifiedIdeograph3c3e', [], 'Display', None],
  [ 0x3c3f, 'CjkUnifiedIdeograph3c3f', [], 'Display', None],
  [ 0x3c40, 'CjkUnifiedIdeograph3c40', [], 'Display', None],
  [ 0x3c41, 'CjkUnifiedIdeograph3c41', [], 'Display', None],
  [ 0x3c42, 'CjkUnifiedIdeograph3c42', [], 'Display', None],
  [ 0x3c43, 'CjkUnifiedIdeograph3c43', [], 'Display', None],
  [ 0x3c44, 'CjkUnifiedIdeograph3c44', [], 'Display', None],
  [ 0x3c45, 'CjkUnifiedIdeograph3c45', [], 'Display', None],
  [ 0x3c46, 'CjkUnifiedIdeograph3c46', [], 'Display', None],
  [ 0x3c47, 'CjkUnifiedIdeograph3c47', [], 'Display', None],
  [ 0x3c48, 'CjkUnifiedIdeograph3c48', [], 'Display', None],
  [ 0x3c49, 'CjkUnifiedIdeograph3c49', [], 'Display', None],
  [ 0x3c4a, 'CjkUnifiedIdeograph3c4a', [], 'Display', None],
  [ 0x3c4b, 'CjkUnifiedIdeograph3c4b', [], 'Display', None],
  [ 0x3c4c, 'CjkUnifiedIdeograph3c4c', [], 'Display', None],
  [ 0x3c4d, 'CjkUnifiedIdeograph3c4d', [], 'Display', None],
  [ 0x3c4e, 'CjkUnifiedIdeograph3c4e', [], 'Display', None],
  [ 0x3c4f, 'CjkUnifiedIdeograph3c4f', [], 'Display', None],
  [ 0x3c50, 'CjkUnifiedIdeograph3c50', [], 'Display', None],
  [ 0x3c51, 'CjkUnifiedIdeograph3c51', [], 'Display', None],
  [ 0x3c52, 'CjkUnifiedIdeograph3c52', [], 'Display', None],
  [ 0x3c53, 'CjkUnifiedIdeograph3c53', [], 'Display', None],
  [ 0x3c54, 'CjkUnifiedIdeograph3c54', [], 'Display', None],
  [ 0x3c55, 'CjkUnifiedIdeograph3c55', [], 'Display', None],
  [ 0x3c56, 'CjkUnifiedIdeograph3c56', [], 'Display', None],
  [ 0x3c57, 'CjkUnifiedIdeograph3c57', [], 'Display', None],
  [ 0x3c58, 'CjkUnifiedIdeograph3c58', [], 'Display', None],
  [ 0x3c59, 'CjkUnifiedIdeograph3c59', [], 'Display', None],
  [ 0x3c5a, 'CjkUnifiedIdeograph3c5a', [], 'Display', None],
  [ 0x3c5b, 'CjkUnifiedIdeograph3c5b', [], 'Display', None],
  [ 0x3c5c, 'CjkUnifiedIdeograph3c5c', [], 'Display', None],
  [ 0x3c5d, 'CjkUnifiedIdeograph3c5d', [], 'Display', None],
  [ 0x3c5e, 'CjkUnifiedIdeograph3c5e', [], 'Display', None],
  [ 0x3c5f, 'CjkUnifiedIdeograph3c5f', [], 'Display', None],
  [ 0x3c60, 'CjkUnifiedIdeograph3c60', [], 'Display', None],
  [ 0x3c61, 'CjkUnifiedIdeograph3c61', [], 'Display', None],
  [ 0x3c62, 'CjkUnifiedIdeograph3c62', [], 'Display', None],
  [ 0x3c63, 'CjkUnifiedIdeograph3c63', [], 'Display', None],
  [ 0x3c64, 'CjkUnifiedIdeograph3c64', [], 'Display', None],
  [ 0x3c65, 'CjkUnifiedIdeograph3c65', [], 'Display', None],
  [ 0x3c66, 'CjkUnifiedIdeograph3c66', [], 'Display', None],
  [ 0x3c67, 'CjkUnifiedIdeograph3c67', [], 'Display', None],
  [ 0x3c68, 'CjkUnifiedIdeograph3c68', [], 'Display', None],
  [ 0x3c69, 'CjkUnifiedIdeograph3c69', [], 'Display', None],
  [ 0x3c6a, 'CjkUnifiedIdeograph3c6a', [], 'Display', None],
  [ 0x3c6b, 'CjkUnifiedIdeograph3c6b', [], 'Display', None],
  [ 0x3c6c, 'CjkUnifiedIdeograph3c6c', [], 'Display', None],
  [ 0x3c6d, 'CjkUnifiedIdeograph3c6d', [], 'Display', None],
  [ 0x3c6e, 'CjkUnifiedIdeograph3c6e', [], 'Display', None],
  [ 0x3c6f, 'CjkUnifiedIdeograph3c6f', [], 'Display', None],
  [ 0x3c70, 'CjkUnifiedIdeograph3c70', [], 'Display', None],
  [ 0x3c71, 'CjkUnifiedIdeograph3c71', [], 'Display', None],
  [ 0x3c72, 'CjkUnifiedIdeograph3c72', [], 'Display', None],
  [ 0x3c73, 'CjkUnifiedIdeograph3c73', [], 'Display', None],
  [ 0x3c74, 'CjkUnifiedIdeograph3c74', [], 'Display', None],
  [ 0x3c75, 'CjkUnifiedIdeograph3c75', [], 'Display', None],
  [ 0x3c76, 'CjkUnifiedIdeograph3c76', [], 'Display', None],
  [ 0x3c77, 'CjkUnifiedIdeograph3c77', [], 'Display', None],
  [ 0x3c78, 'CjkUnifiedIdeograph3c78', [], 'Display', None],
  [ 0x3c79, 'CjkUnifiedIdeograph3c79', [], 'Display', None],
  [ 0x3c7a, 'CjkUnifiedIdeograph3c7a', [], 'Display', None],
  [ 0x3c7b, 'CjkUnifiedIdeograph3c7b', [], 'Display', None],
  [ 0x3c7c, 'CjkUnifiedIdeograph3c7c', [], 'Display', None],
  [ 0x3c7d, 'CjkUnifiedIdeograph3c7d', [], 'Display', None],
  [ 0x3c7e, 'CjkUnifiedIdeograph3c7e', [], 'Display', None],
  [ 0x3c7f, 'CjkUnifiedIdeograph3c7f', [], 'Display', None],
  [ 0x3c80, 'CjkUnifiedIdeograph3c80', [], 'Display', None],
  [ 0x3c81, 'CjkUnifiedIdeograph3c81', [], 'Display', None],
  [ 0x3c82, 'CjkUnifiedIdeograph3c82', [], 'Display', None],
  [ 0x3c83, 'CjkUnifiedIdeograph3c83', [], 'Display', None],
  [ 0x3c84, 'CjkUnifiedIdeograph3c84', [], 'Display', None],
  [ 0x3c85, 'CjkUnifiedIdeograph3c85', [], 'Display', None],
  [ 0x3c86, 'CjkUnifiedIdeograph3c86', [], 'Display', None],
  [ 0x3c87, 'CjkUnifiedIdeograph3c87', [], 'Display', None],
  [ 0x3c88, 'CjkUnifiedIdeograph3c88', [], 'Display', None],
  [ 0x3c89, 'CjkUnifiedIdeograph3c89', [], 'Display', None],
  [ 0x3c8a, 'CjkUnifiedIdeograph3c8a', [], 'Display', None],
  [ 0x3c8b, 'CjkUnifiedIdeograph3c8b', [], 'Display', None],
  [ 0x3c8c, 'CjkUnifiedIdeograph3c8c', [], 'Display', None],
  [ 0x3c8d, 'CjkUnifiedIdeograph3c8d', [], 'Display', None],
  [ 0x3c8e, 'CjkUnifiedIdeograph3c8e', [], 'Display', None],
  [ 0x3c8f, 'CjkUnifiedIdeograph3c8f', [], 'Display', None],
  [ 0x3c90, 'CjkUnifiedIdeograph3c90', [], 'Display', None],
  [ 0x3c91, 'CjkUnifiedIdeograph3c91', [], 'Display', None],
  [ 0x3c92, 'CjkUnifiedIdeograph3c92', [], 'Display', None],
  [ 0x3c93, 'CjkUnifiedIdeograph3c93', [], 'Display', None],
  [ 0x3c94, 'CjkUnifiedIdeograph3c94', [], 'Display', None],
  [ 0x3c95, 'CjkUnifiedIdeograph3c95', [], 'Display', None],
  [ 0x3c96, 'CjkUnifiedIdeograph3c96', [], 'Display', None],
  [ 0x3c97, 'CjkUnifiedIdeograph3c97', [], 'Display', None],
  [ 0x3c98, 'CjkUnifiedIdeograph3c98', [], 'Display', None],
  [ 0x3c99, 'CjkUnifiedIdeograph3c99', [], 'Display', None],
  [ 0x3c9a, 'CjkUnifiedIdeograph3c9a', [], 'Display', None],
  [ 0x3c9b, 'CjkUnifiedIdeograph3c9b', [], 'Display', None],
  [ 0x3c9c, 'CjkUnifiedIdeograph3c9c', [], 'Display', None],
  [ 0x3c9d, 'CjkUnifiedIdeograph3c9d', [], 'Display', None],
  [ 0x3c9e, 'CjkUnifiedIdeograph3c9e', [], 'Display', None],
  [ 0x3c9f, 'CjkUnifiedIdeograph3c9f', [], 'Display', None],
  [ 0x3ca0, 'CjkUnifiedIdeograph3ca0', [], 'Display', None],
  [ 0x3ca1, 'CjkUnifiedIdeograph3ca1', [], 'Display', None],
  [ 0x3ca2, 'CjkUnifiedIdeograph3ca2', [], 'Display', None],
  [ 0x3ca3, 'CjkUnifiedIdeograph3ca3', [], 'Display', None],
  [ 0x3ca4, 'CjkUnifiedIdeograph3ca4', [], 'Display', None],
  [ 0x3ca5, 'CjkUnifiedIdeograph3ca5', [], 'Display', None],
  [ 0x3ca6, 'CjkUnifiedIdeograph3ca6', [], 'Display', None],
  [ 0x3ca7, 'CjkUnifiedIdeograph3ca7', [], 'Display', None],
  [ 0x3ca8, 'CjkUnifiedIdeograph3ca8', [], 'Display', None],
  [ 0x3ca9, 'CjkUnifiedIdeograph3ca9', [], 'Display', None],
  [ 0x3caa, 'CjkUnifiedIdeograph3caa', [], 'Display', None],
  [ 0x3cab, 'CjkUnifiedIdeograph3cab', [], 'Display', None],
  [ 0x3cac, 'CjkUnifiedIdeograph3cac', [], 'Display', None],
  [ 0x3cad, 'CjkUnifiedIdeograph3cad', [], 'Display', None],
  [ 0x3cae, 'CjkUnifiedIdeograph3cae', [], 'Display', None],
  [ 0x3caf, 'CjkUnifiedIdeograph3caf', [], 'Display', None],
  [ 0x3cb0, 'CjkUnifiedIdeograph3cb0', [], 'Display', None],
  [ 0x3cb1, 'CjkUnifiedIdeograph3cb1', [], 'Display', None],
  [ 0x3cb2, 'CjkUnifiedIdeograph3cb2', [], 'Display', None],
  [ 0x3cb3, 'CjkUnifiedIdeograph3cb3', [], 'Display', None],
  [ 0x3cb4, 'CjkUnifiedIdeograph3cb4', [], 'Display', None],
  [ 0x3cb5, 'CjkUnifiedIdeograph3cb5', [], 'Display', None],
  [ 0x3cb6, 'CjkUnifiedIdeograph3cb6', [], 'Display', None],
  [ 0x3cb7, 'CjkUnifiedIdeograph3cb7', [], 'Display', None],
  [ 0x3cb8, 'CjkUnifiedIdeograph3cb8', [], 'Display', None],
  [ 0x3cb9, 'CjkUnifiedIdeograph3cb9', [], 'Display', None],
  [ 0x3cba, 'CjkUnifiedIdeograph3cba', [], 'Display', None],
  [ 0x3cbb, 'CjkUnifiedIdeograph3cbb', [], 'Display', None],
  [ 0x3cbc, 'CjkUnifiedIdeograph3cbc', [], 'Display', None],
  [ 0x3cbd, 'CjkUnifiedIdeograph3cbd', [], 'Display', None],
  [ 0x3cbe, 'CjkUnifiedIdeograph3cbe', [], 'Display', None],
  [ 0x3cbf, 'CjkUnifiedIdeograph3cbf', [], 'Display', None],
  [ 0x3cc0, 'CjkUnifiedIdeograph3cc0', [], 'Display', None],
  [ 0x3cc1, 'CjkUnifiedIdeograph3cc1', [], 'Display', None],
  [ 0x3cc2, 'CjkUnifiedIdeograph3cc2', [], 'Display', None],
  [ 0x3cc3, 'CjkUnifiedIdeograph3cc3', [], 'Display', None],
  [ 0x3cc4, 'CjkUnifiedIdeograph3cc4', [], 'Display', None],
  [ 0x3cc5, 'CjkUnifiedIdeograph3cc5', [], 'Display', None],
  [ 0x3cc6, 'CjkUnifiedIdeograph3cc6', [], 'Display', None],
  [ 0x3cc7, 'CjkUnifiedIdeograph3cc7', [], 'Display', None],
  [ 0x3cc8, 'CjkUnifiedIdeograph3cc8', [], 'Display', None],
  [ 0x3cc9, 'CjkUnifiedIdeograph3cc9', [], 'Display', None],
  [ 0x3cca, 'CjkUnifiedIdeograph3cca', [], 'Display', None],
  [ 0x3ccb, 'CjkUnifiedIdeograph3ccb', [], 'Display', None],
  [ 0x3ccc, 'CjkUnifiedIdeograph3ccc', [], 'Display', None],
  [ 0x3ccd, 'CjkUnifiedIdeograph3ccd', [], 'Display', None],
  [ 0x3cce, 'CjkUnifiedIdeograph3cce', [], 'Display', None],
  [ 0x3ccf, 'CjkUnifiedIdeograph3ccf', [], 'Display', None],
  [ 0x3cd0, 'CjkUnifiedIdeograph3cd0', [], 'Display', None],
  [ 0x3cd1, 'CjkUnifiedIdeograph3cd1', [], 'Display', None],
  [ 0x3cd2, 'CjkUnifiedIdeograph3cd2', [], 'Display', None],
  [ 0x3cd3, 'CjkUnifiedIdeograph3cd3', [], 'Display', None],
  [ 0x3cd4, 'CjkUnifiedIdeograph3cd4', [], 'Display', None],
  [ 0x3cd5, 'CjkUnifiedIdeograph3cd5', [], 'Display', None],
  [ 0x3cd6, 'CjkUnifiedIdeograph3cd6', [], 'Display', None],
  [ 0x3cd7, 'CjkUnifiedIdeograph3cd7', [], 'Display', None],
  [ 0x3cd8, 'CjkUnifiedIdeograph3cd8', [], 'Display', None],
  [ 0x3cd9, 'CjkUnifiedIdeograph3cd9', [], 'Display', None],
  [ 0x3cda, 'CjkUnifiedIdeograph3cda', [], 'Display', None],
  [ 0x3cdb, 'CjkUnifiedIdeograph3cdb', [], 'Display', None],
  [ 0x3cdc, 'CjkUnifiedIdeograph3cdc', [], 'Display', None],
  [ 0x3cdd, 'CjkUnifiedIdeograph3cdd', [], 'Display', None],
  [ 0x3cde, 'CjkUnifiedIdeograph3cde', [], 'Display', None],
  [ 0x3cdf, 'CjkUnifiedIdeograph3cdf', [], 'Display', None],
  [ 0x3ce0, 'CjkUnifiedIdeograph3ce0', [], 'Display', None],
  [ 0x3ce1, 'CjkUnifiedIdeograph3ce1', [], 'Display', None],
  [ 0x3ce2, 'CjkUnifiedIdeograph3ce2', [], 'Display', None],
  [ 0x3ce3, 'CjkUnifiedIdeograph3ce3', [], 'Display', None],
  [ 0x3ce4, 'CjkUnifiedIdeograph3ce4', [], 'Display', None],
  [ 0x3ce5, 'CjkUnifiedIdeograph3ce5', [], 'Display', None],
  [ 0x3ce6, 'CjkUnifiedIdeograph3ce6', [], 'Display', None],
  [ 0x3ce7, 'CjkUnifiedIdeograph3ce7', [], 'Display', None],
  [ 0x3ce8, 'CjkUnifiedIdeograph3ce8', [], 'Display', None],
  [ 0x3ce9, 'CjkUnifiedIdeograph3ce9', [], 'Display', None],
  [ 0x3cea, 'CjkUnifiedIdeograph3cea', [], 'Display', None],
  [ 0x3ceb, 'CjkUnifiedIdeograph3ceb', [], 'Display', None],
  [ 0x3cec, 'CjkUnifiedIdeograph3cec', [], 'Display', None],
  [ 0x3ced, 'CjkUnifiedIdeograph3ced', [], 'Display', None],
  [ 0x3cee, 'CjkUnifiedIdeograph3cee', [], 'Display', None],
  [ 0x3cef, 'CjkUnifiedIdeograph3cef', [], 'Display', None],
  [ 0x3cf0, 'CjkUnifiedIdeograph3cf0', [], 'Display', None],
  [ 0x3cf1, 'CjkUnifiedIdeograph3cf1', [], 'Display', None],
  [ 0x3cf2, 'CjkUnifiedIdeograph3cf2', [], 'Display', None],
  [ 0x3cf3, 'CjkUnifiedIdeograph3cf3', [], 'Display', None],
  [ 0x3cf4, 'CjkUnifiedIdeograph3cf4', [], 'Display', None],
  [ 0x3cf5, 'CjkUnifiedIdeograph3cf5', [], 'Display', None],
  [ 0x3cf6, 'CjkUnifiedIdeograph3cf6', [], 'Display', None],
  [ 0x3cf7, 'CjkUnifiedIdeograph3cf7', [], 'Display', None],
  [ 0x3cf8, 'CjkUnifiedIdeograph3cf8', [], 'Display', None],
  [ 0x3cf9, 'CjkUnifiedIdeograph3cf9', [], 'Display', None],
  [ 0x3cfa, 'CjkUnifiedIdeograph3cfa', [], 'Display', None],
  [ 0x3cfb, 'CjkUnifiedIdeograph3cfb', [], 'Display', None],
  [ 0x3cfc, 'CjkUnifiedIdeograph3cfc', [], 'Display', None],
  [ 0x3cfd, 'CjkUnifiedIdeograph3cfd', [], 'Display', None],
  [ 0x3cfe, 'CjkUnifiedIdeograph3cfe', [], 'Display', None],
  [ 0x3cff, 'CjkUnifiedIdeograph3cff', [], 'Display', None],
  [ 0x3d00, 'CjkUnifiedIdeograph3d00', [], 'Display', None],
  [ 0x3d01, 'CjkUnifiedIdeograph3d01', [], 'Display', None],
  [ 0x3d02, 'CjkUnifiedIdeograph3d02', [], 'Display', None],
  [ 0x3d03, 'CjkUnifiedIdeograph3d03', [], 'Display', None],
  [ 0x3d04, 'CjkUnifiedIdeograph3d04', [], 'Display', None],
  [ 0x3d05, 'CjkUnifiedIdeograph3d05', [], 'Display', None],
  [ 0x3d06, 'CjkUnifiedIdeograph3d06', [], 'Display', None],
  [ 0x3d07, 'CjkUnifiedIdeograph3d07', [], 'Display', None],
  [ 0x3d08, 'CjkUnifiedIdeograph3d08', [], 'Display', None],
  [ 0x3d09, 'CjkUnifiedIdeograph3d09', [], 'Display', None],
  [ 0x3d0a, 'CjkUnifiedIdeograph3d0a', [], 'Display', None],
  [ 0x3d0b, 'CjkUnifiedIdeograph3d0b', [], 'Display', None],
  [ 0x3d0c, 'CjkUnifiedIdeograph3d0c', [], 'Display', None],
  [ 0x3d0d, 'CjkUnifiedIdeograph3d0d', [], 'Display', None],
  [ 0x3d0e, 'CjkUnifiedIdeograph3d0e', [], 'Display', None],
  [ 0x3d0f, 'CjkUnifiedIdeograph3d0f', [], 'Display', None],
  [ 0x3d10, 'CjkUnifiedIdeograph3d10', [], 'Display', None],
  [ 0x3d11, 'CjkUnifiedIdeograph3d11', [], 'Display', None],
  [ 0x3d12, 'CjkUnifiedIdeograph3d12', [], 'Display', None],
  [ 0x3d13, 'CjkUnifiedIdeograph3d13', [], 'Display', None],
  [ 0x3d14, 'CjkUnifiedIdeograph3d14', [], 'Display', None],
  [ 0x3d15, 'CjkUnifiedIdeograph3d15', [], 'Display', None],
  [ 0x3d16, 'CjkUnifiedIdeograph3d16', [], 'Display', None],
  [ 0x3d17, 'CjkUnifiedIdeograph3d17', [], 'Display', None],
  [ 0x3d18, 'CjkUnifiedIdeograph3d18', [], 'Display', None],
  [ 0x3d19, 'CjkUnifiedIdeograph3d19', [], 'Display', None],
  [ 0x3d1a, 'CjkUnifiedIdeograph3d1a', [], 'Display', None],
  [ 0x3d1b, 'CjkUnifiedIdeograph3d1b', [], 'Display', None],
  [ 0x3d1c, 'CjkUnifiedIdeograph3d1c', [], 'Display', None],
  [ 0x3d1d, 'CjkUnifiedIdeograph3d1d', [], 'Display', None],
  [ 0x3d1e, 'CjkUnifiedIdeograph3d1e', [], 'Display', None],
  [ 0x3d1f, 'CjkUnifiedIdeograph3d1f', [], 'Display', None],
  [ 0x3d20, 'CjkUnifiedIdeograph3d20', [], 'Display', None],
  [ 0x3d21, 'CjkUnifiedIdeograph3d21', [], 'Display', None],
  [ 0x3d22, 'CjkUnifiedIdeograph3d22', [], 'Display', None],
  [ 0x3d23, 'CjkUnifiedIdeograph3d23', [], 'Display', None],
  [ 0x3d24, 'CjkUnifiedIdeograph3d24', [], 'Display', None],
  [ 0x3d25, 'CjkUnifiedIdeograph3d25', [], 'Display', None],
  [ 0x3d26, 'CjkUnifiedIdeograph3d26', [], 'Display', None],
  [ 0x3d27, 'CjkUnifiedIdeograph3d27', [], 'Display', None],
  [ 0x3d28, 'CjkUnifiedIdeograph3d28', [], 'Display', None],
  [ 0x3d29, 'CjkUnifiedIdeograph3d29', [], 'Display', None],
  [ 0x3d2a, 'CjkUnifiedIdeograph3d2a', [], 'Display', None],
  [ 0x3d2b, 'CjkUnifiedIdeograph3d2b', [], 'Display', None],
  [ 0x3d2c, 'CjkUnifiedIdeograph3d2c', [], 'Display', None],
  [ 0x3d2d, 'CjkUnifiedIdeograph3d2d', [], 'Display', None],
  [ 0x3d2e, 'CjkUnifiedIdeograph3d2e', [], 'Display', None],
  [ 0x3d2f, 'CjkUnifiedIdeograph3d2f', [], 'Display', None],
  [ 0x3d30, 'CjkUnifiedIdeograph3d30', [], 'Display', None],
  [ 0x3d31, 'CjkUnifiedIdeograph3d31', [], 'Display', None],
  [ 0x3d32, 'CjkUnifiedIdeograph3d32', [], 'Display', None],
  [ 0x3d33, 'CjkUnifiedIdeograph3d33', [], 'Display', None],
  [ 0x3d34, 'CjkUnifiedIdeograph3d34', [], 'Display', None],
  [ 0x3d35, 'CjkUnifiedIdeograph3d35', [], 'Display', None],
  [ 0x3d36, 'CjkUnifiedIdeograph3d36', [], 'Display', None],
  [ 0x3d37, 'CjkUnifiedIdeograph3d37', [], 'Display', None],
  [ 0x3d38, 'CjkUnifiedIdeograph3d38', [], 'Display', None],
  [ 0x3d39, 'CjkUnifiedIdeograph3d39', [], 'Display', None],
  [ 0x3d3a, 'CjkUnifiedIdeograph3d3a', [], 'Display', None],
  [ 0x3d3b, 'CjkUnifiedIdeograph3d3b', [], 'Display', None],
  [ 0x3d3c, 'CjkUnifiedIdeograph3d3c', [], 'Display', None],
  [ 0x3d3d, 'CjkUnifiedIdeograph3d3d', [], 'Display', None],
  [ 0x3d3e, 'CjkUnifiedIdeograph3d3e', [], 'Display', None],
  [ 0x3d3f, 'CjkUnifiedIdeograph3d3f', [], 'Display', None],
  [ 0x3d40, 'CjkUnifiedIdeograph3d40', [], 'Display', None],
  [ 0x3d41, 'CjkUnifiedIdeograph3d41', [], 'Display', None],
  [ 0x3d42, 'CjkUnifiedIdeograph3d42', [], 'Display', None],
  [ 0x3d43, 'CjkUnifiedIdeograph3d43', [], 'Display', None],
  [ 0x3d44, 'CjkUnifiedIdeograph3d44', [], 'Display', None],
  [ 0x3d45, 'CjkUnifiedIdeograph3d45', [], 'Display', None],
  [ 0x3d46, 'CjkUnifiedIdeograph3d46', [], 'Display', None],
  [ 0x3d47, 'CjkUnifiedIdeograph3d47', [], 'Display', None],
  [ 0x3d48, 'CjkUnifiedIdeograph3d48', [], 'Display', None],
  [ 0x3d49, 'CjkUnifiedIdeograph3d49', [], 'Display', None],
  [ 0x3d4a, 'CjkUnifiedIdeograph3d4a', [], 'Display', None],
  [ 0x3d4b, 'CjkUnifiedIdeograph3d4b', [], 'Display', None],
  [ 0x3d4c, 'CjkUnifiedIdeograph3d4c', [], 'Display', None],
  [ 0x3d4d, 'CjkUnifiedIdeograph3d4d', [], 'Display', None],
  [ 0x3d4e, 'CjkUnifiedIdeograph3d4e', [], 'Display', None],
  [ 0x3d4f, 'CjkUnifiedIdeograph3d4f', [], 'Display', None],
  [ 0x3d50, 'CjkUnifiedIdeograph3d50', [], 'Display', None],
  [ 0x3d51, 'CjkUnifiedIdeograph3d51', [], 'Display', None],
  [ 0x3d52, 'CjkUnifiedIdeograph3d52', [], 'Display', None],
  [ 0x3d53, 'CjkUnifiedIdeograph3d53', [], 'Display', None],
  [ 0x3d54, 'CjkUnifiedIdeograph3d54', [], 'Display', None],
  [ 0x3d55, 'CjkUnifiedIdeograph3d55', [], 'Display', None],
  [ 0x3d56, 'CjkUnifiedIdeograph3d56', [], 'Display', None],
  [ 0x3d57, 'CjkUnifiedIdeograph3d57', [], 'Display', None],
  [ 0x3d58, 'CjkUnifiedIdeograph3d58', [], 'Display', None],
  [ 0x3d59, 'CjkUnifiedIdeograph3d59', [], 'Display', None],
  [ 0x3d5a, 'CjkUnifiedIdeograph3d5a', [], 'Display', None],
  [ 0x3d5b, 'CjkUnifiedIdeograph3d5b', [], 'Display', None],
  [ 0x3d5c, 'CjkUnifiedIdeograph3d5c', [], 'Display', None],
  [ 0x3d5d, 'CjkUnifiedIdeograph3d5d', [], 'Display', None],
  [ 0x3d5e, 'CjkUnifiedIdeograph3d5e', [], 'Display', None],
  [ 0x3d5f, 'CjkUnifiedIdeograph3d5f', [], 'Display', None],
  [ 0x3d60, 'CjkUnifiedIdeograph3d60', [], 'Display', None],
  [ 0x3d61, 'CjkUnifiedIdeograph3d61', [], 'Display', None],
  [ 0x3d62, 'CjkUnifiedIdeograph3d62', [], 'Display', None],
  [ 0x3d63, 'CjkUnifiedIdeograph3d63', [], 'Display', None],
  [ 0x3d64, 'CjkUnifiedIdeograph3d64', [], 'Display', None],
  [ 0x3d65, 'CjkUnifiedIdeograph3d65', [], 'Display', None],
  [ 0x3d66, 'CjkUnifiedIdeograph3d66', [], 'Display', None],
  [ 0x3d67, 'CjkUnifiedIdeograph3d67', [], 'Display', None],
  [ 0x3d68, 'CjkUnifiedIdeograph3d68', [], 'Display', None],
  [ 0x3d69, 'CjkUnifiedIdeograph3d69', [], 'Display', None],
  [ 0x3d6a, 'CjkUnifiedIdeograph3d6a', [], 'Display', None],
  [ 0x3d6b, 'CjkUnifiedIdeograph3d6b', [], 'Display', None],
  [ 0x3d6c, 'CjkUnifiedIdeograph3d6c', [], 'Display', None],
  [ 0x3d6d, 'CjkUnifiedIdeograph3d6d', [], 'Display', None],
  [ 0x3d6e, 'CjkUnifiedIdeograph3d6e', [], 'Display', None],
  [ 0x3d6f, 'CjkUnifiedIdeograph3d6f', [], 'Display', None],
  [ 0x3d70, 'CjkUnifiedIdeograph3d70', [], 'Display', None],
  [ 0x3d71, 'CjkUnifiedIdeograph3d71', [], 'Display', None],
  [ 0x3d72, 'CjkUnifiedIdeograph3d72', [], 'Display', None],
  [ 0x3d73, 'CjkUnifiedIdeograph3d73', [], 'Display', None],
  [ 0x3d74, 'CjkUnifiedIdeograph3d74', [], 'Display', None],
  [ 0x3d75, 'CjkUnifiedIdeograph3d75', [], 'Display', None],
  [ 0x3d76, 'CjkUnifiedIdeograph3d76', [], 'Display', None],
  [ 0x3d77, 'CjkUnifiedIdeograph3d77', [], 'Display', None],
  [ 0x3d78, 'CjkUnifiedIdeograph3d78', [], 'Display', None],
  [ 0x3d79, 'CjkUnifiedIdeograph3d79', [], 'Display', None],
  [ 0x3d7a, 'CjkUnifiedIdeograph3d7a', [], 'Display', None],
  [ 0x3d7b, 'CjkUnifiedIdeograph3d7b', [], 'Display', None],
  [ 0x3d7c, 'CjkUnifiedIdeograph3d7c', [], 'Display', None],
  [ 0x3d7d, 'CjkUnifiedIdeograph3d7d', [], 'Display', None],
  [ 0x3d7e, 'CjkUnifiedIdeograph3d7e', [], 'Display', None],
  [ 0x3d7f, 'CjkUnifiedIdeograph3d7f', [], 'Display', None],
  [ 0x3d80, 'CjkUnifiedIdeograph3d80', [], 'Display', None],
  [ 0x3d81, 'CjkUnifiedIdeograph3d81', [], 'Display', None],
  [ 0x3d82, 'CjkUnifiedIdeograph3d82', [], 'Display', None],
  [ 0x3d83, 'CjkUnifiedIdeograph3d83', [], 'Display', None],
  [ 0x3d84, 'CjkUnifiedIdeograph3d84', [], 'Display', None],
  [ 0x3d85, 'CjkUnifiedIdeograph3d85', [], 'Display', None],
  [ 0x3d86, 'CjkUnifiedIdeograph3d86', [], 'Display', None],
  [ 0x3d87, 'CjkUnifiedIdeograph3d87', [], 'Display', None],
  [ 0x3d88, 'CjkUnifiedIdeograph3d88', [], 'Display', None],
  [ 0x3d89, 'CjkUnifiedIdeograph3d89', [], 'Display', None],
  [ 0x3d8a, 'CjkUnifiedIdeograph3d8a', [], 'Display', None],
  [ 0x3d8b, 'CjkUnifiedIdeograph3d8b', [], 'Display', None],
  [ 0x3d8c, 'CjkUnifiedIdeograph3d8c', [], 'Display', None],
  [ 0x3d8d, 'CjkUnifiedIdeograph3d8d', [], 'Display', None],
  [ 0x3d8e, 'CjkUnifiedIdeograph3d8e', [], 'Display', None],
  [ 0x3d8f, 'CjkUnifiedIdeograph3d8f', [], 'Display', None],
  [ 0x3d90, 'CjkUnifiedIdeograph3d90', [], 'Display', None],
  [ 0x3d91, 'CjkUnifiedIdeograph3d91', [], 'Display', None],
  [ 0x3d92, 'CjkUnifiedIdeograph3d92', [], 'Display', None],
  [ 0x3d93, 'CjkUnifiedIdeograph3d93', [], 'Display', None],
  [ 0x3d94, 'CjkUnifiedIdeograph3d94', [], 'Display', None],
  [ 0x3d95, 'CjkUnifiedIdeograph3d95', [], 'Display', None],
  [ 0x3d96, 'CjkUnifiedIdeograph3d96', [], 'Display', None],
  [ 0x3d97, 'CjkUnifiedIdeograph3d97', [], 'Display', None],
  [ 0x3d98, 'CjkUnifiedIdeograph3d98', [], 'Display', None],
  [ 0x3d99, 'CjkUnifiedIdeograph3d99', [], 'Display', None],
  [ 0x3d9a, 'CjkUnifiedIdeograph3d9a', [], 'Display', None],
  [ 0x3d9b, 'CjkUnifiedIdeograph3d9b', [], 'Display', None],
  [ 0x3d9c, 'CjkUnifiedIdeograph3d9c', [], 'Display', None],
  [ 0x3d9d, 'CjkUnifiedIdeograph3d9d', [], 'Display', None],
  [ 0x3d9e, 'CjkUnifiedIdeograph3d9e', [], 'Display', None],
  [ 0x3d9f, 'CjkUnifiedIdeograph3d9f', [], 'Display', None],
  [ 0x3da0, 'CjkUnifiedIdeograph3da0', [], 'Display', None],
  [ 0x3da1, 'CjkUnifiedIdeograph3da1', [], 'Display', None],
  [ 0x3da2, 'CjkUnifiedIdeograph3da2', [], 'Display', None],
  [ 0x3da3, 'CjkUnifiedIdeograph3da3', [], 'Display', None],
  [ 0x3da4, 'CjkUnifiedIdeograph3da4', [], 'Display', None],
  [ 0x3da5, 'CjkUnifiedIdeograph3da5', [], 'Display', None],
  [ 0x3da6, 'CjkUnifiedIdeograph3da6', [], 'Display', None],
  [ 0x3da7, 'CjkUnifiedIdeograph3da7', [], 'Display', None],
  [ 0x3da8, 'CjkUnifiedIdeograph3da8', [], 'Display', None],
  [ 0x3da9, 'CjkUnifiedIdeograph3da9', [], 'Display', None],
  [ 0x3daa, 'CjkUnifiedIdeograph3daa', [], 'Display', None],
  [ 0x3dab, 'CjkUnifiedIdeograph3dab', [], 'Display', None],
  [ 0x3dac, 'CjkUnifiedIdeograph3dac', [], 'Display', None],
  [ 0x3dad, 'CjkUnifiedIdeograph3dad', [], 'Display', None],
  [ 0x3dae, 'CjkUnifiedIdeograph3dae', [], 'Display', None],
  [ 0x3daf, 'CjkUnifiedIdeograph3daf', [], 'Display', None],
  [ 0x3db0, 'CjkUnifiedIdeograph3db0', [], 'Display', None],
  [ 0x3db1, 'CjkUnifiedIdeograph3db1', [], 'Display', None],
  [ 0x3db2, 'CjkUnifiedIdeograph3db2', [], 'Display', None],
  [ 0x3db3, 'CjkUnifiedIdeograph3db3', [], 'Display', None],
  [ 0x3db4, 'CjkUnifiedIdeograph3db4', [], 'Display', None],
  [ 0x3db5, 'CjkUnifiedIdeograph3db5', [], 'Display', None],
  [ 0x3db6, 'CjkUnifiedIdeograph3db6', [], 'Display', None],
  [ 0x3db7, 'CjkUnifiedIdeograph3db7', [], 'Display', None],
  [ 0x3db8, 'CjkUnifiedIdeograph3db8', [], 'Display', None],
  [ 0x3db9, 'CjkUnifiedIdeograph3db9', [], 'Display', None],
  [ 0x3dba, 'CjkUnifiedIdeograph3dba', [], 'Display', None],
  [ 0x3dbb, 'CjkUnifiedIdeograph3dbb', [], 'Display', None],
  [ 0x3dbc, 'CjkUnifiedIdeograph3dbc', [], 'Display', None],
  [ 0x3dbd, 'CjkUnifiedIdeograph3dbd', [], 'Display', None],
  [ 0x3dbe, 'CjkUnifiedIdeograph3dbe', [], 'Display', None],
  [ 0x3dbf, 'CjkUnifiedIdeograph3dbf', [], 'Display', None],
  [ 0x3dc0, 'CjkUnifiedIdeograph3dc0', [], 'Display', None],
  [ 0x3dc1, 'CjkUnifiedIdeograph3dc1', [], 'Display', None],
  [ 0x3dc2, 'CjkUnifiedIdeograph3dc2', [], 'Display', None],
  [ 0x3dc3, 'CjkUnifiedIdeograph3dc3', [], 'Display', None],
  [ 0x3dc4, 'CjkUnifiedIdeograph3dc4', [], 'Display', None],
  [ 0x3dc5, 'CjkUnifiedIdeograph3dc5', [], 'Display', None],
  [ 0x3dc6, 'CjkUnifiedIdeograph3dc6', [], 'Display', None],
  [ 0x3dc7, 'CjkUnifiedIdeograph3dc7', [], 'Display', None],
  [ 0x3dc8, 'CjkUnifiedIdeograph3dc8', [], 'Display', None],
  [ 0x3dc9, 'CjkUnifiedIdeograph3dc9', [], 'Display', None],
  [ 0x3dca, 'CjkUnifiedIdeograph3dca', [], 'Display', None],
  [ 0x3dcb, 'CjkUnifiedIdeograph3dcb', [], 'Display', None],
  [ 0x3dcc, 'CjkUnifiedIdeograph3dcc', [], 'Display', None],
  [ 0x3dcd, 'CjkUnifiedIdeograph3dcd', [], 'Display', None],
  [ 0x3dce, 'CjkUnifiedIdeograph3dce', [], 'Display', None],
  [ 0x3dcf, 'CjkUnifiedIdeograph3dcf', [], 'Display', None],
  [ 0x3dd0, 'CjkUnifiedIdeograph3dd0', [], 'Display', None],
  [ 0x3dd1, 'CjkUnifiedIdeograph3dd1', [], 'Display', None],
  [ 0x3dd2, 'CjkUnifiedIdeograph3dd2', [], 'Display', None],
  [ 0x3dd3, 'CjkUnifiedIdeograph3dd3', [], 'Display', None],
  [ 0x3dd4, 'CjkUnifiedIdeograph3dd4', [], 'Display', None],
  [ 0x3dd5, 'CjkUnifiedIdeograph3dd5', [], 'Display', None],
  [ 0x3dd6, 'CjkUnifiedIdeograph3dd6', [], 'Display', None],
  [ 0x3dd7, 'CjkUnifiedIdeograph3dd7', [], 'Display', None],
  [ 0x3dd8, 'CjkUnifiedIdeograph3dd8', [], 'Display', None],
  [ 0x3dd9, 'CjkUnifiedIdeograph3dd9', [], 'Display', None],
  [ 0x3dda, 'CjkUnifiedIdeograph3dda', [], 'Display', None],
  [ 0x3ddb, 'CjkUnifiedIdeograph3ddb', [], 'Display', None],
  [ 0x3ddc, 'CjkUnifiedIdeograph3ddc', [], 'Display', None],
  [ 0x3ddd, 'CjkUnifiedIdeograph3ddd', [], 'Display', None],
  [ 0x3dde, 'CjkUnifiedIdeograph3dde', [], 'Display', None],
  [ 0x3ddf, 'CjkUnifiedIdeograph3ddf', [], 'Display', None],
  [ 0x3de0, 'CjkUnifiedIdeograph3de0', [], 'Display', None],
  [ 0x3de1, 'CjkUnifiedIdeograph3de1', [], 'Display', None],
  [ 0x3de2, 'CjkUnifiedIdeograph3de2', [], 'Display', None],
  [ 0x3de3, 'CjkUnifiedIdeograph3de3', [], 'Display', None],
  [ 0x3de4, 'CjkUnifiedIdeograph3de4', [], 'Display', None],
  [ 0x3de5, 'CjkUnifiedIdeograph3de5', [], 'Display', None],
  [ 0x3de6, 'CjkUnifiedIdeograph3de6', [], 'Display', None],
  [ 0x3de7, 'CjkUnifiedIdeograph3de7', [], 'Display', None],
  [ 0x3de8, 'CjkUnifiedIdeograph3de8', [], 'Display', None],
  [ 0x3de9, 'CjkUnifiedIdeograph3de9', [], 'Display', None],
  [ 0x3dea, 'CjkUnifiedIdeograph3dea', [], 'Display', None],
  [ 0x3deb, 'CjkUnifiedIdeograph3deb', [], 'Display', None],
  [ 0x3dec, 'CjkUnifiedIdeograph3dec', [], 'Display', None],
  [ 0x3ded, 'CjkUnifiedIdeograph3ded', [], 'Display', None],
  [ 0x3dee, 'CjkUnifiedIdeograph3dee', [], 'Display', None],
  [ 0x3def, 'CjkUnifiedIdeograph3def', [], 'Display', None],
  [ 0x3df0, 'CjkUnifiedIdeograph3df0', [], 'Display', None],
  [ 0x3df1, 'CjkUnifiedIdeograph3df1', [], 'Display', None],
  [ 0x3df2, 'CjkUnifiedIdeograph3df2', [], 'Display', None],
  [ 0x3df3, 'CjkUnifiedIdeograph3df3', [], 'Display', None],
  [ 0x3df4, 'CjkUnifiedIdeograph3df4', [], 'Display', None],
  [ 0x3df5, 'CjkUnifiedIdeograph3df5', [], 'Display', None],
  [ 0x3df6, 'CjkUnifiedIdeograph3df6', [], 'Display', None],
  [ 0x3df7, 'CjkUnifiedIdeograph3df7', [], 'Display', None],
  [ 0x3df8, 'CjkUnifiedIdeograph3df8', [], 'Display', None],
  [ 0x3df9, 'CjkUnifiedIdeograph3df9', [], 'Display', None],
  [ 0x3dfa, 'CjkUnifiedIdeograph3dfa', [], 'Display', None],
  [ 0x3dfb, 'CjkUnifiedIdeograph3dfb', [], 'Display', None],
  [ 0x3dfc, 'CjkUnifiedIdeograph3dfc', [], 'Display', None],
  [ 0x3dfd, 'CjkUnifiedIdeograph3dfd', [], 'Display', None],
  [ 0x3dfe, 'CjkUnifiedIdeograph3dfe', [], 'Display', None],
  [ 0x3dff, 'CjkUnifiedIdeograph3dff', [], 'Display', None],
  [ 0x3e00, 'CjkUnifiedIdeograph3e00', [], 'Display', None],
  [ 0x3e01, 'CjkUnifiedIdeograph3e01', [], 'Display', None],
  [ 0x3e02, 'CjkUnifiedIdeograph3e02', [], 'Display', None],
  [ 0x3e03, 'CjkUnifiedIdeograph3e03', [], 'Display', None],
  [ 0x3e04, 'CjkUnifiedIdeograph3e04', [], 'Display', None],
  [ 0x3e05, 'CjkUnifiedIdeograph3e05', [], 'Display', None],
  [ 0x3e06, 'CjkUnifiedIdeograph3e06', [], 'Display', None],
  [ 0x3e07, 'CjkUnifiedIdeograph3e07', [], 'Display', None],
  [ 0x3e08, 'CjkUnifiedIdeograph3e08', [], 'Display', None],
  [ 0x3e09, 'CjkUnifiedIdeograph3e09', [], 'Display', None],
  [ 0x3e0a, 'CjkUnifiedIdeograph3e0a', [], 'Display', None],
  [ 0x3e0b, 'CjkUnifiedIdeograph3e0b', [], 'Display', None],
  [ 0x3e0c, 'CjkUnifiedIdeograph3e0c', [], 'Display', None],
  [ 0x3e0d, 'CjkUnifiedIdeograph3e0d', [], 'Display', None],
  [ 0x3e0e, 'CjkUnifiedIdeograph3e0e', [], 'Display', None],
  [ 0x3e0f, 'CjkUnifiedIdeograph3e0f', [], 'Display', None],
  [ 0x3e10, 'CjkUnifiedIdeograph3e10', [], 'Display', None],
  [ 0x3e11, 'CjkUnifiedIdeograph3e11', [], 'Display', None],
  [ 0x3e12, 'CjkUnifiedIdeograph3e12', [], 'Display', None],
  [ 0x3e13, 'CjkUnifiedIdeograph3e13', [], 'Display', None],
  [ 0x3e14, 'CjkUnifiedIdeograph3e14', [], 'Display', None],
  [ 0x3e15, 'CjkUnifiedIdeograph3e15', [], 'Display', None],
  [ 0x3e16, 'CjkUnifiedIdeograph3e16', [], 'Display', None],
  [ 0x3e17, 'CjkUnifiedIdeograph3e17', [], 'Display', None],
  [ 0x3e18, 'CjkUnifiedIdeograph3e18', [], 'Display', None],
  [ 0x3e19, 'CjkUnifiedIdeograph3e19', [], 'Display', None],
  [ 0x3e1a, 'CjkUnifiedIdeograph3e1a', [], 'Display', None],
  [ 0x3e1b, 'CjkUnifiedIdeograph3e1b', [], 'Display', None],
  [ 0x3e1c, 'CjkUnifiedIdeograph3e1c', [], 'Display', None],
  [ 0x3e1d, 'CjkUnifiedIdeograph3e1d', [], 'Display', None],
  [ 0x3e1e, 'CjkUnifiedIdeograph3e1e', [], 'Display', None],
  [ 0x3e1f, 'CjkUnifiedIdeograph3e1f', [], 'Display', None],
  [ 0x3e20, 'CjkUnifiedIdeograph3e20', [], 'Display', None],
  [ 0x3e21, 'CjkUnifiedIdeograph3e21', [], 'Display', None],
  [ 0x3e22, 'CjkUnifiedIdeograph3e22', [], 'Display', None],
  [ 0x3e23, 'CjkUnifiedIdeograph3e23', [], 'Display', None],
  [ 0x3e24, 'CjkUnifiedIdeograph3e24', [], 'Display', None],
  [ 0x3e25, 'CjkUnifiedIdeograph3e25', [], 'Display', None],
  [ 0x3e26, 'CjkUnifiedIdeograph3e26', [], 'Display', None],
  [ 0x3e27, 'CjkUnifiedIdeograph3e27', [], 'Display', None],
  [ 0x3e28, 'CjkUnifiedIdeograph3e28', [], 'Display', None],
  [ 0x3e29, 'CjkUnifiedIdeograph3e29', [], 'Display', None],
  [ 0x3e2a, 'CjkUnifiedIdeograph3e2a', [], 'Display', None],
  [ 0x3e2b, 'CjkUnifiedIdeograph3e2b', [], 'Display', None],
  [ 0x3e2c, 'CjkUnifiedIdeograph3e2c', [], 'Display', None],
  [ 0x3e2d, 'CjkUnifiedIdeograph3e2d', [], 'Display', None],
  [ 0x3e2e, 'CjkUnifiedIdeograph3e2e', [], 'Display', None],
  [ 0x3e2f, 'CjkUnifiedIdeograph3e2f', [], 'Display', None],
  [ 0x3e30, 'CjkUnifiedIdeograph3e30', [], 'Display', None],
  [ 0x3e31, 'CjkUnifiedIdeograph3e31', [], 'Display', None],
  [ 0x3e32, 'CjkUnifiedIdeograph3e32', [], 'Display', None],
  [ 0x3e33, 'CjkUnifiedIdeograph3e33', [], 'Display', None],
  [ 0x3e34, 'CjkUnifiedIdeograph3e34', [], 'Display', None],
  [ 0x3e35, 'CjkUnifiedIdeograph3e35', [], 'Display', None],
  [ 0x3e36, 'CjkUnifiedIdeograph3e36', [], 'Display', None],
  [ 0x3e37, 'CjkUnifiedIdeograph3e37', [], 'Display', None],
  [ 0x3e38, 'CjkUnifiedIdeograph3e38', [], 'Display', None],
  [ 0x3e39, 'CjkUnifiedIdeograph3e39', [], 'Display', None],
  [ 0x3e3a, 'CjkUnifiedIdeograph3e3a', [], 'Display', None],
  [ 0x3e3b, 'CjkUnifiedIdeograph3e3b', [], 'Display', None],
  [ 0x3e3c, 'CjkUnifiedIdeograph3e3c', [], 'Display', None],
  [ 0x3e3d, 'CjkUnifiedIdeograph3e3d', [], 'Display', None],
  [ 0x3e3e, 'CjkUnifiedIdeograph3e3e', [], 'Display', None],
  [ 0x3e3f, 'CjkUnifiedIdeograph3e3f', [], 'Display', None],
  [ 0x3e40, 'CjkUnifiedIdeograph3e40', [], 'Display', None],
  [ 0x3e41, 'CjkUnifiedIdeograph3e41', [], 'Display', None],
  [ 0x3e42, 'CjkUnifiedIdeograph3e42', [], 'Display', None],
  [ 0x3e43, 'CjkUnifiedIdeograph3e43', [], 'Display', None],
  [ 0x3e44, 'CjkUnifiedIdeograph3e44', [], 'Display', None],
  [ 0x3e45, 'CjkUnifiedIdeograph3e45', [], 'Display', None],
  [ 0x3e46, 'CjkUnifiedIdeograph3e46', [], 'Display', None],
  [ 0x3e47, 'CjkUnifiedIdeograph3e47', [], 'Display', None],
  [ 0x3e48, 'CjkUnifiedIdeograph3e48', [], 'Display', None],
  [ 0x3e49, 'CjkUnifiedIdeograph3e49', [], 'Display', None],
  [ 0x3e4a, 'CjkUnifiedIdeograph3e4a', [], 'Display', None],
  [ 0x3e4b, 'CjkUnifiedIdeograph3e4b', [], 'Display', None],
  [ 0x3e4c, 'CjkUnifiedIdeograph3e4c', [], 'Display', None],
  [ 0x3e4d, 'CjkUnifiedIdeograph3e4d', [], 'Display', None],
  [ 0x3e4e, 'CjkUnifiedIdeograph3e4e', [], 'Display', None],
  [ 0x3e4f, 'CjkUnifiedIdeograph3e4f', [], 'Display', None],
  [ 0x3e50, 'CjkUnifiedIdeograph3e50', [], 'Display', None],
  [ 0x3e51, 'CjkUnifiedIdeograph3e51', [], 'Display', None],
  [ 0x3e52, 'CjkUnifiedIdeograph3e52', [], 'Display', None],
  [ 0x3e53, 'CjkUnifiedIdeograph3e53', [], 'Display', None],
  [ 0x3e54, 'CjkUnifiedIdeograph3e54', [], 'Display', None],
  [ 0x3e55, 'CjkUnifiedIdeograph3e55', [], 'Display', None],
  [ 0x3e56, 'CjkUnifiedIdeograph3e56', [], 'Display', None],
  [ 0x3e57, 'CjkUnifiedIdeograph3e57', [], 'Display', None],
  [ 0x3e58, 'CjkUnifiedIdeograph3e58', [], 'Display', None],
  [ 0x3e59, 'CjkUnifiedIdeograph3e59', [], 'Display', None],
  [ 0x3e5a, 'CjkUnifiedIdeograph3e5a', [], 'Display', None],
  [ 0x3e5b, 'CjkUnifiedIdeograph3e5b', [], 'Display', None],
  [ 0x3e5c, 'CjkUnifiedIdeograph3e5c', [], 'Display', None],
  [ 0x3e5d, 'CjkUnifiedIdeograph3e5d', [], 'Display', None],
  [ 0x3e5e, 'CjkUnifiedIdeograph3e5e', [], 'Display', None],
  [ 0x3e5f, 'CjkUnifiedIdeograph3e5f', [], 'Display', None],
  [ 0x3e60, 'CjkUnifiedIdeograph3e60', [], 'Display', None],
  [ 0x3e61, 'CjkUnifiedIdeograph3e61', [], 'Display', None],
  [ 0x3e62, 'CjkUnifiedIdeograph3e62', [], 'Display', None],
  [ 0x3e63, 'CjkUnifiedIdeograph3e63', [], 'Display', None],
  [ 0x3e64, 'CjkUnifiedIdeograph3e64', [], 'Display', None],
  [ 0x3e65, 'CjkUnifiedIdeograph3e65', [], 'Display', None],
  [ 0x3e66, 'CjkUnifiedIdeograph3e66', [], 'Display', None],
  [ 0x3e67, 'CjkUnifiedIdeograph3e67', [], 'Display', None],
  [ 0x3e68, 'CjkUnifiedIdeograph3e68', [], 'Display', None],
  [ 0x3e69, 'CjkUnifiedIdeograph3e69', [], 'Display', None],
  [ 0x3e6a, 'CjkUnifiedIdeograph3e6a', [], 'Display', None],
  [ 0x3e6b, 'CjkUnifiedIdeograph3e6b', [], 'Display', None],
  [ 0x3e6c, 'CjkUnifiedIdeograph3e6c', [], 'Display', None],
  [ 0x3e6d, 'CjkUnifiedIdeograph3e6d', [], 'Display', None],
  [ 0x3e6e, 'CjkUnifiedIdeograph3e6e', [], 'Display', None],
  [ 0x3e6f, 'CjkUnifiedIdeograph3e6f', [], 'Display', None],
  [ 0x3e70, 'CjkUnifiedIdeograph3e70', [], 'Display', None],
  [ 0x3e71, 'CjkUnifiedIdeograph3e71', [], 'Display', None],
  [ 0x3e72, 'CjkUnifiedIdeograph3e72', [], 'Display', None],
  [ 0x3e73, 'CjkUnifiedIdeograph3e73', [], 'Display', None],
  [ 0x3e74, 'CjkUnifiedIdeograph3e74', [], 'Display', None],
  [ 0x3e75, 'CjkUnifiedIdeograph3e75', [], 'Display', None],
  [ 0x3e76, 'CjkUnifiedIdeograph3e76', [], 'Display', None],
  [ 0x3e77, 'CjkUnifiedIdeograph3e77', [], 'Display', None],
  [ 0x3e78, 'CjkUnifiedIdeograph3e78', [], 'Display', None],
  [ 0x3e79, 'CjkUnifiedIdeograph3e79', [], 'Display', None],
  [ 0x3e7a, 'CjkUnifiedIdeograph3e7a', [], 'Display', None],
  [ 0x3e7b, 'CjkUnifiedIdeograph3e7b', [], 'Display', None],
  [ 0x3e7c, 'CjkUnifiedIdeograph3e7c', [], 'Display', None],
  [ 0x3e7d, 'CjkUnifiedIdeograph3e7d', [], 'Display', None],
  [ 0x3e7e, 'CjkUnifiedIdeograph3e7e', [], 'Display', None],
  [ 0x3e7f, 'CjkUnifiedIdeograph3e7f', [], 'Display', None],
  [ 0x3e80, 'CjkUnifiedIdeograph3e80', [], 'Display', None],
  [ 0x3e81, 'CjkUnifiedIdeograph3e81', [], 'Display', None],
  [ 0x3e82, 'CjkUnifiedIdeograph3e82', [], 'Display', None],
  [ 0x3e83, 'CjkUnifiedIdeograph3e83', [], 'Display', None],
  [ 0x3e84, 'CjkUnifiedIdeograph3e84', [], 'Display', None],
  [ 0x3e85, 'CjkUnifiedIdeograph3e85', [], 'Display', None],
  [ 0x3e86, 'CjkUnifiedIdeograph3e86', [], 'Display', None],
  [ 0x3e87, 'CjkUnifiedIdeograph3e87', [], 'Display', None],
  [ 0x3e88, 'CjkUnifiedIdeograph3e88', [], 'Display', None],
  [ 0x3e89, 'CjkUnifiedIdeograph3e89', [], 'Display', None],
  [ 0x3e8a, 'CjkUnifiedIdeograph3e8a', [], 'Display', None],
  [ 0x3e8b, 'CjkUnifiedIdeograph3e8b', [], 'Display', None],
  [ 0x3e8c, 'CjkUnifiedIdeograph3e8c', [], 'Display', None],
  [ 0x3e8d, 'CjkUnifiedIdeograph3e8d', [], 'Display', None],
  [ 0x3e8e, 'CjkUnifiedIdeograph3e8e', [], 'Display', None],
  [ 0x3e8f, 'CjkUnifiedIdeograph3e8f', [], 'Display', None],
  [ 0x3e90, 'CjkUnifiedIdeograph3e90', [], 'Display', None],
  [ 0x3e91, 'CjkUnifiedIdeograph3e91', [], 'Display', None],
  [ 0x3e92, 'CjkUnifiedIdeograph3e92', [], 'Display', None],
  [ 0x3e93, 'CjkUnifiedIdeograph3e93', [], 'Display', None],
  [ 0x3e94, 'CjkUnifiedIdeograph3e94', [], 'Display', None],
  [ 0x3e95, 'CjkUnifiedIdeograph3e95', [], 'Display', None],
  [ 0x3e96, 'CjkUnifiedIdeograph3e96', [], 'Display', None],
  [ 0x3e97, 'CjkUnifiedIdeograph3e97', [], 'Display', None],
  [ 0x3e98, 'CjkUnifiedIdeograph3e98', [], 'Display', None],
  [ 0x3e99, 'CjkUnifiedIdeograph3e99', [], 'Display', None],
  [ 0x3e9a, 'CjkUnifiedIdeograph3e9a', [], 'Display', None],
  [ 0x3e9b, 'CjkUnifiedIdeograph3e9b', [], 'Display', None],
  [ 0x3e9c, 'CjkUnifiedIdeograph3e9c', [], 'Display', None],
  [ 0x3e9d, 'CjkUnifiedIdeograph3e9d', [], 'Display', None],
  [ 0x3e9e, 'CjkUnifiedIdeograph3e9e', [], 'Display', None],
  [ 0x3e9f, 'CjkUnifiedIdeograph3e9f', [], 'Display', None],
  [ 0x3ea0, 'CjkUnifiedIdeograph3ea0', [], 'Display', None],
  [ 0x3ea1, 'CjkUnifiedIdeograph3ea1', [], 'Display', None],
  [ 0x3ea2, 'CjkUnifiedIdeograph3ea2', [], 'Display', None],
  [ 0x3ea3, 'CjkUnifiedIdeograph3ea3', [], 'Display', None],
  [ 0x3ea4, 'CjkUnifiedIdeograph3ea4', [], 'Display', None],
  [ 0x3ea5, 'CjkUnifiedIdeograph3ea5', [], 'Display', None],
  [ 0x3ea6, 'CjkUnifiedIdeograph3ea6', [], 'Display', None],
  [ 0x3ea7, 'CjkUnifiedIdeograph3ea7', [], 'Display', None],
  [ 0x3ea8, 'CjkUnifiedIdeograph3ea8', [], 'Display', None],
  [ 0x3ea9, 'CjkUnifiedIdeograph3ea9', [], 'Display', None],
  [ 0x3eaa, 'CjkUnifiedIdeograph3eaa', [], 'Display', None],
  [ 0x3eab, 'CjkUnifiedIdeograph3eab', [], 'Display', None],
  [ 0x3eac, 'CjkUnifiedIdeograph3eac', [], 'Display', None],
  [ 0x3ead, 'CjkUnifiedIdeograph3ead', [], 'Display', None],
  [ 0x3eae, 'CjkUnifiedIdeograph3eae', [], 'Display', None],
  [ 0x3eaf, 'CjkUnifiedIdeograph3eaf', [], 'Display', None],
  [ 0x3eb0, 'CjkUnifiedIdeograph3eb0', [], 'Display', None],
  [ 0x3eb1, 'CjkUnifiedIdeograph3eb1', [], 'Display', None],
  [ 0x3eb2, 'CjkUnifiedIdeograph3eb2', [], 'Display', None],
  [ 0x3eb3, 'CjkUnifiedIdeograph3eb3', [], 'Display', None],
  [ 0x3eb4, 'CjkUnifiedIdeograph3eb4', [], 'Display', None],
  [ 0x3eb5, 'CjkUnifiedIdeograph3eb5', [], 'Display', None],
  [ 0x3eb6, 'CjkUnifiedIdeograph3eb6', [], 'Display', None],
  [ 0x3eb7, 'CjkUnifiedIdeograph3eb7', [], 'Display', None],
  [ 0x3eb8, 'CjkUnifiedIdeograph3eb8', [], 'Display', None],
  [ 0x3eb9, 'CjkUnifiedIdeograph3eb9', [], 'Display', None],
  [ 0x3eba, 'CjkUnifiedIdeograph3eba', [], 'Display', None],
  [ 0x3ebb, 'CjkUnifiedIdeograph3ebb', [], 'Display', None],
  [ 0x3ebc, 'CjkUnifiedIdeograph3ebc', [], 'Display', None],
  [ 0x3ebd, 'CjkUnifiedIdeograph3ebd', [], 'Display', None],
  [ 0x3ebe, 'CjkUnifiedIdeograph3ebe', [], 'Display', None],
  [ 0x3ebf, 'CjkUnifiedIdeograph3ebf', [], 'Display', None],
  [ 0x3ec0, 'CjkUnifiedIdeograph3ec0', [], 'Display', None],
  [ 0x3ec1, 'CjkUnifiedIdeograph3ec1', [], 'Display', None],
  [ 0x3ec2, 'CjkUnifiedIdeograph3ec2', [], 'Display', None],
  [ 0x3ec3, 'CjkUnifiedIdeograph3ec3', [], 'Display', None],
  [ 0x3ec4, 'CjkUnifiedIdeograph3ec4', [], 'Display', None],
  [ 0x3ec5, 'CjkUnifiedIdeograph3ec5', [], 'Display', None],
  [ 0x3ec6, 'CjkUnifiedIdeograph3ec6', [], 'Display', None],
  [ 0x3ec7, 'CjkUnifiedIdeograph3ec7', [], 'Display', None],
  [ 0x3ec8, 'CjkUnifiedIdeograph3ec8', [], 'Display', None],
  [ 0x3ec9, 'CjkUnifiedIdeograph3ec9', [], 'Display', None],
  [ 0x3eca, 'CjkUnifiedIdeograph3eca', [], 'Display', None],
  [ 0x3ecb, 'CjkUnifiedIdeograph3ecb', [], 'Display', None],
  [ 0x3ecc, 'CjkUnifiedIdeograph3ecc', [], 'Display', None],
  [ 0x3ecd, 'CjkUnifiedIdeograph3ecd', [], 'Display', None],
  [ 0x3ece, 'CjkUnifiedIdeograph3ece', [], 'Display', None],
  [ 0x3ecf, 'CjkUnifiedIdeograph3ecf', [], 'Display', None],
  [ 0x3ed0, 'CjkUnifiedIdeograph3ed0', [], 'Display', None],
  [ 0x3ed1, 'CjkUnifiedIdeograph3ed1', [], 'Display', None],
  [ 0x3ed2, 'CjkUnifiedIdeograph3ed2', [], 'Display', None],
  [ 0x3ed3, 'CjkUnifiedIdeograph3ed3', [], 'Display', None],
  [ 0x3ed4, 'CjkUnifiedIdeograph3ed4', [], 'Display', None],
  [ 0x3ed5, 'CjkUnifiedIdeograph3ed5', [], 'Display', None],
  [ 0x3ed6, 'CjkUnifiedIdeograph3ed6', [], 'Display', None],
  [ 0x3ed7, 'CjkUnifiedIdeograph3ed7', [], 'Display', None],
  [ 0x3ed8, 'CjkUnifiedIdeograph3ed8', [], 'Display', None],
  [ 0x3ed9, 'CjkUnifiedIdeograph3ed9', [], 'Display', None],
  [ 0x3eda, 'CjkUnifiedIdeograph3eda', [], 'Display', None],
  [ 0x3edb, 'CjkUnifiedIdeograph3edb', [], 'Display', None],
  [ 0x3edc, 'CjkUnifiedIdeograph3edc', [], 'Display', None],
  [ 0x3edd, 'CjkUnifiedIdeograph3edd', [], 'Display', None],
  [ 0x3ede, 'CjkUnifiedIdeograph3ede', [], 'Display', None],
  [ 0x3edf, 'CjkUnifiedIdeograph3edf', [], 'Display', None],
  [ 0x3ee0, 'CjkUnifiedIdeograph3ee0', [], 'Display', None],
  [ 0x3ee1, 'CjkUnifiedIdeograph3ee1', [], 'Display', None],
  [ 0x3ee2, 'CjkUnifiedIdeograph3ee2', [], 'Display', None],
  [ 0x3ee3, 'CjkUnifiedIdeograph3ee3', [], 'Display', None],
  [ 0x3ee4, 'CjkUnifiedIdeograph3ee4', [], 'Display', None],
  [ 0x3ee5, 'CjkUnifiedIdeograph3ee5', [], 'Display', None],
  [ 0x3ee6, 'CjkUnifiedIdeograph3ee6', [], 'Display', None],
  [ 0x3ee7, 'CjkUnifiedIdeograph3ee7', [], 'Display', None],
  [ 0x3ee8, 'CjkUnifiedIdeograph3ee8', [], 'Display', None],
  [ 0x3ee9, 'CjkUnifiedIdeograph3ee9', [], 'Display', None],
  [ 0x3eea, 'CjkUnifiedIdeograph3eea', [], 'Display', None],
  [ 0x3eeb, 'CjkUnifiedIdeograph3eeb', [], 'Display', None],
  [ 0x3eec, 'CjkUnifiedIdeograph3eec', [], 'Display', None],
  [ 0x3eed, 'CjkUnifiedIdeograph3eed', [], 'Display', None],
  [ 0x3eee, 'CjkUnifiedIdeograph3eee', [], 'Display', None],
  [ 0x3eef, 'CjkUnifiedIdeograph3eef', [], 'Display', None],
  [ 0x3ef0, 'CjkUnifiedIdeograph3ef0', [], 'Display', None],
  [ 0x3ef1, 'CjkUnifiedIdeograph3ef1', [], 'Display', None],
  [ 0x3ef2, 'CjkUnifiedIdeograph3ef2', [], 'Display', None],
  [ 0x3ef3, 'CjkUnifiedIdeograph3ef3', [], 'Display', None],
  [ 0x3ef4, 'CjkUnifiedIdeograph3ef4', [], 'Display', None],
  [ 0x3ef5, 'CjkUnifiedIdeograph3ef5', [], 'Display', None],
  [ 0x3ef6, 'CjkUnifiedIdeograph3ef6', [], 'Display', None],
  [ 0x3ef7, 'CjkUnifiedIdeograph3ef7', [], 'Display', None],
  [ 0x3ef8, 'CjkUnifiedIdeograph3ef8', [], 'Display', None],
  [ 0x3ef9, 'CjkUnifiedIdeograph3ef9', [], 'Display', None],
  [ 0x3efa, 'CjkUnifiedIdeograph3efa', [], 'Display', None],
  [ 0x3efb, 'CjkUnifiedIdeograph3efb', [], 'Display', None],
  [ 0x3efc, 'CjkUnifiedIdeograph3efc', [], 'Display', None],
  [ 0x3efd, 'CjkUnifiedIdeograph3efd', [], 'Display', None],
  [ 0x3efe, 'CjkUnifiedIdeograph3efe', [], 'Display', None],
  [ 0x3eff, 'CjkUnifiedIdeograph3eff', [], 'Display', None],
  [ 0x3f00, 'CjkUnifiedIdeograph3f00', [], 'Display', None],
  [ 0x3f01, 'CjkUnifiedIdeograph3f01', [], 'Display', None],
  [ 0x3f02, 'CjkUnifiedIdeograph3f02', [], 'Display', None],
  [ 0x3f03, 'CjkUnifiedIdeograph3f03', [], 'Display', None],
  [ 0x3f04, 'CjkUnifiedIdeograph3f04', [], 'Display', None],
  [ 0x3f05, 'CjkUnifiedIdeograph3f05', [], 'Display', None],
  [ 0x3f06, 'CjkUnifiedIdeograph3f06', [], 'Display', None],
  [ 0x3f07, 'CjkUnifiedIdeograph3f07', [], 'Display', None],
  [ 0x3f08, 'CjkUnifiedIdeograph3f08', [], 'Display', None],
  [ 0x3f09, 'CjkUnifiedIdeograph3f09', [], 'Display', None],
  [ 0x3f0a, 'CjkUnifiedIdeograph3f0a', [], 'Display', None],
  [ 0x3f0b, 'CjkUnifiedIdeograph3f0b', [], 'Display', None],
  [ 0x3f0c, 'CjkUnifiedIdeograph3f0c', [], 'Display', None],
  [ 0x3f0d, 'CjkUnifiedIdeograph3f0d', [], 'Display', None],
  [ 0x3f0e, 'CjkUnifiedIdeograph3f0e', [], 'Display', None],
  [ 0x3f0f, 'CjkUnifiedIdeograph3f0f', [], 'Display', None],
  [ 0x3f10, 'CjkUnifiedIdeograph3f10', [], 'Display', None],
  [ 0x3f11, 'CjkUnifiedIdeograph3f11', [], 'Display', None],
  [ 0x3f12, 'CjkUnifiedIdeograph3f12', [], 'Display', None],
  [ 0x3f13, 'CjkUnifiedIdeograph3f13', [], 'Display', None],
  [ 0x3f14, 'CjkUnifiedIdeograph3f14', [], 'Display', None],
  [ 0x3f15, 'CjkUnifiedIdeograph3f15', [], 'Display', None],
  [ 0x3f16, 'CjkUnifiedIdeograph3f16', [], 'Display', None],
  [ 0x3f17, 'CjkUnifiedIdeograph3f17', [], 'Display', None],
  [ 0x3f18, 'CjkUnifiedIdeograph3f18', [], 'Display', None],
  [ 0x3f19, 'CjkUnifiedIdeograph3f19', [], 'Display', None],
  [ 0x3f1a, 'CjkUnifiedIdeograph3f1a', [], 'Display', None],
  [ 0x3f1b, 'CjkUnifiedIdeograph3f1b', [], 'Display', None],
  [ 0x3f1c, 'CjkUnifiedIdeograph3f1c', [], 'Display', None],
  [ 0x3f1d, 'CjkUnifiedIdeograph3f1d', [], 'Display', None],
  [ 0x3f1e, 'CjkUnifiedIdeograph3f1e', [], 'Display', None],
  [ 0x3f1f, 'CjkUnifiedIdeograph3f1f', [], 'Display', None],
  [ 0x3f20, 'CjkUnifiedIdeograph3f20', [], 'Display', None],
  [ 0x3f21, 'CjkUnifiedIdeograph3f21', [], 'Display', None],
  [ 0x3f22, 'CjkUnifiedIdeograph3f22', [], 'Display', None],
  [ 0x3f23, 'CjkUnifiedIdeograph3f23', [], 'Display', None],
  [ 0x3f24, 'CjkUnifiedIdeograph3f24', [], 'Display', None],
  [ 0x3f25, 'CjkUnifiedIdeograph3f25', [], 'Display', None],
  [ 0x3f26, 'CjkUnifiedIdeograph3f26', [], 'Display', None],
  [ 0x3f27, 'CjkUnifiedIdeograph3f27', [], 'Display', None],
  [ 0x3f28, 'CjkUnifiedIdeograph3f28', [], 'Display', None],
  [ 0x3f29, 'CjkUnifiedIdeograph3f29', [], 'Display', None],
  [ 0x3f2a, 'CjkUnifiedIdeograph3f2a', [], 'Display', None],
  [ 0x3f2b, 'CjkUnifiedIdeograph3f2b', [], 'Display', None],
  [ 0x3f2c, 'CjkUnifiedIdeograph3f2c', [], 'Display', None],
  [ 0x3f2d, 'CjkUnifiedIdeograph3f2d', [], 'Display', None],
  [ 0x3f2e, 'CjkUnifiedIdeograph3f2e', [], 'Display', None],
  [ 0x3f2f, 'CjkUnifiedIdeograph3f2f', [], 'Display', None],
  [ 0x3f30, 'CjkUnifiedIdeograph3f30', [], 'Display', None],
  [ 0x3f31, 'CjkUnifiedIdeograph3f31', [], 'Display', None],
  [ 0x3f32, 'CjkUnifiedIdeograph3f32', [], 'Display', None],
  [ 0x3f33, 'CjkUnifiedIdeograph3f33', [], 'Display', None],
  [ 0x3f34, 'CjkUnifiedIdeograph3f34', [], 'Display', None],
  [ 0x3f35, 'CjkUnifiedIdeograph3f35', [], 'Display', None],
  [ 0x3f36, 'CjkUnifiedIdeograph3f36', [], 'Display', None],
  [ 0x3f37, 'CjkUnifiedIdeograph3f37', [], 'Display', None],
  [ 0x3f38, 'CjkUnifiedIdeograph3f38', [], 'Display', None],
  [ 0x3f39, 'CjkUnifiedIdeograph3f39', [], 'Display', None],
  [ 0x3f3a, 'CjkUnifiedIdeograph3f3a', [], 'Display', None],
  [ 0x3f3b, 'CjkUnifiedIdeograph3f3b', [], 'Display', None],
  [ 0x3f3c, 'CjkUnifiedIdeograph3f3c', [], 'Display', None],
  [ 0x3f3d, 'CjkUnifiedIdeograph3f3d', [], 'Display', None],
  [ 0x3f3e, 'CjkUnifiedIdeograph3f3e', [], 'Display', None],
  [ 0x3f3f, 'CjkUnifiedIdeograph3f3f', [], 'Display', None],
  [ 0x3f40, 'CjkUnifiedIdeograph3f40', [], 'Display', None],
  [ 0x3f41, 'CjkUnifiedIdeograph3f41', [], 'Display', None],
  [ 0x3f42, 'CjkUnifiedIdeograph3f42', [], 'Display', None],
  [ 0x3f43, 'CjkUnifiedIdeograph3f43', [], 'Display', None],
  [ 0x3f44, 'CjkUnifiedIdeograph3f44', [], 'Display', None],
  [ 0x3f45, 'CjkUnifiedIdeograph3f45', [], 'Display', None],
  [ 0x3f46, 'CjkUnifiedIdeograph3f46', [], 'Display', None],
  [ 0x3f47, 'CjkUnifiedIdeograph3f47', [], 'Display', None],
  [ 0x3f48, 'CjkUnifiedIdeograph3f48', [], 'Display', None],
  [ 0x3f49, 'CjkUnifiedIdeograph3f49', [], 'Display', None],
  [ 0x3f4a, 'CjkUnifiedIdeograph3f4a', [], 'Display', None],
  [ 0x3f4b, 'CjkUnifiedIdeograph3f4b', [], 'Display', None],
  [ 0x3f4c, 'CjkUnifiedIdeograph3f4c', [], 'Display', None],
  [ 0x3f4d, 'CjkUnifiedIdeograph3f4d', [], 'Display', None],
  [ 0x3f4e, 'CjkUnifiedIdeograph3f4e', [], 'Display', None],
  [ 0x3f4f, 'CjkUnifiedIdeograph3f4f', [], 'Display', None],
  [ 0x3f50, 'CjkUnifiedIdeograph3f50', [], 'Display', None],
  [ 0x3f51, 'CjkUnifiedIdeograph3f51', [], 'Display', None],
  [ 0x3f52, 'CjkUnifiedIdeograph3f52', [], 'Display', None],
  [ 0x3f53, 'CjkUnifiedIdeograph3f53', [], 'Display', None],
  [ 0x3f54, 'CjkUnifiedIdeograph3f54', [], 'Display', None],
  [ 0x3f55, 'CjkUnifiedIdeograph3f55', [], 'Display', None],
  [ 0x3f56, 'CjkUnifiedIdeograph3f56', [], 'Display', None],
  [ 0x3f57, 'CjkUnifiedIdeograph3f57', [], 'Display', None],
  [ 0x3f58, 'CjkUnifiedIdeograph3f58', [], 'Display', None],
  [ 0x3f59, 'CjkUnifiedIdeograph3f59', [], 'Display', None],
  [ 0x3f5a, 'CjkUnifiedIdeograph3f5a', [], 'Display', None],
  [ 0x3f5b, 'CjkUnifiedIdeograph3f5b', [], 'Display', None],
  [ 0x3f5c, 'CjkUnifiedIdeograph3f5c', [], 'Display', None],
  [ 0x3f5d, 'CjkUnifiedIdeograph3f5d', [], 'Display', None],
  [ 0x3f5e, 'CjkUnifiedIdeograph3f5e', [], 'Display', None],
  [ 0x3f5f, 'CjkUnifiedIdeograph3f5f', [], 'Display', None],
  [ 0x3f60, 'CjkUnifiedIdeograph3f60', [], 'Display', None],
  [ 0x3f61, 'CjkUnifiedIdeograph3f61', [], 'Display', None],
  [ 0x3f62, 'CjkUnifiedIdeograph3f62', [], 'Display', None],
  [ 0x3f63, 'CjkUnifiedIdeograph3f63', [], 'Display', None],
  [ 0x3f64, 'CjkUnifiedIdeograph3f64', [], 'Display', None],
  [ 0x3f65, 'CjkUnifiedIdeograph3f65', [], 'Display', None],
  [ 0x3f66, 'CjkUnifiedIdeograph3f66', [], 'Display', None],
  [ 0x3f67, 'CjkUnifiedIdeograph3f67', [], 'Display', None],
  [ 0x3f68, 'CjkUnifiedIdeograph3f68', [], 'Display', None],
  [ 0x3f69, 'CjkUnifiedIdeograph3f69', [], 'Display', None],
  [ 0x3f6a, 'CjkUnifiedIdeograph3f6a', [], 'Display', None],
  [ 0x3f6b, 'CjkUnifiedIdeograph3f6b', [], 'Display', None],
  [ 0x3f6c, 'CjkUnifiedIdeograph3f6c', [], 'Display', None],
  [ 0x3f6d, 'CjkUnifiedIdeograph3f6d', [], 'Display', None],
  [ 0x3f6e, 'CjkUnifiedIdeograph3f6e', [], 'Display', None],
  [ 0x3f6f, 'CjkUnifiedIdeograph3f6f', [], 'Display', None],
  [ 0x3f70, 'CjkUnifiedIdeograph3f70', [], 'Display', None],
  [ 0x3f71, 'CjkUnifiedIdeograph3f71', [], 'Display', None],
  [ 0x3f72, 'CjkUnifiedIdeograph3f72', [], 'Display', None],
  [ 0x3f73, 'CjkUnifiedIdeograph3f73', [], 'Display', None],
  [ 0x3f74, 'CjkUnifiedIdeograph3f74', [], 'Display', None],
  [ 0x3f75, 'CjkUnifiedIdeograph3f75', [], 'Display', None],
  [ 0x3f76, 'CjkUnifiedIdeograph3f76', [], 'Display', None],
  [ 0x3f77, 'CjkUnifiedIdeograph3f77', [], 'Display', None],
  [ 0x3f78, 'CjkUnifiedIdeograph3f78', [], 'Display', None],
  [ 0x3f79, 'CjkUnifiedIdeograph3f79', [], 'Display', None],
  [ 0x3f7a, 'CjkUnifiedIdeograph3f7a', [], 'Display', None],
  [ 0x3f7b, 'CjkUnifiedIdeograph3f7b', [], 'Display', None],
  [ 0x3f7c, 'CjkUnifiedIdeograph3f7c', [], 'Display', None],
  [ 0x3f7d, 'CjkUnifiedIdeograph3f7d', [], 'Display', None],
  [ 0x3f7e, 'CjkUnifiedIdeograph3f7e', [], 'Display', None],
  [ 0x3f7f, 'CjkUnifiedIdeograph3f7f', [], 'Display', None],
  [ 0x3f80, 'CjkUnifiedIdeograph3f80', [], 'Display', None],
  [ 0x3f81, 'CjkUnifiedIdeograph3f81', [], 'Display', None],
  [ 0x3f82, 'CjkUnifiedIdeograph3f82', [], 'Display', None],
  [ 0x3f83, 'CjkUnifiedIdeograph3f83', [], 'Display', None],
  [ 0x3f84, 'CjkUnifiedIdeograph3f84', [], 'Display', None],
  [ 0x3f85, 'CjkUnifiedIdeograph3f85', [], 'Display', None],
  [ 0x3f86, 'CjkUnifiedIdeograph3f86', [], 'Display', None],
  [ 0x3f87, 'CjkUnifiedIdeograph3f87', [], 'Display', None],
  [ 0x3f88, 'CjkUnifiedIdeograph3f88', [], 'Display', None],
  [ 0x3f89, 'CjkUnifiedIdeograph3f89', [], 'Display', None],
  [ 0x3f8a, 'CjkUnifiedIdeograph3f8a', [], 'Display', None],
  [ 0x3f8b, 'CjkUnifiedIdeograph3f8b', [], 'Display', None],
  [ 0x3f8c, 'CjkUnifiedIdeograph3f8c', [], 'Display', None],
  [ 0x3f8d, 'CjkUnifiedIdeograph3f8d', [], 'Display', None],
  [ 0x3f8e, 'CjkUnifiedIdeograph3f8e', [], 'Display', None],
  [ 0x3f8f, 'CjkUnifiedIdeograph3f8f', [], 'Display', None],
  [ 0x3f90, 'CjkUnifiedIdeograph3f90', [], 'Display', None],
  [ 0x3f91, 'CjkUnifiedIdeograph3f91', [], 'Display', None],
  [ 0x3f92, 'CjkUnifiedIdeograph3f92', [], 'Display', None],
  [ 0x3f93, 'CjkUnifiedIdeograph3f93', [], 'Display', None],
  [ 0x3f94, 'CjkUnifiedIdeograph3f94', [], 'Display', None],
  [ 0x3f95, 'CjkUnifiedIdeograph3f95', [], 'Display', None],
  [ 0x3f96, 'CjkUnifiedIdeograph3f96', [], 'Display', None],
  [ 0x3f97, 'CjkUnifiedIdeograph3f97', [], 'Display', None],
  [ 0x3f98, 'CjkUnifiedIdeograph3f98', [], 'Display', None],
  [ 0x3f99, 'CjkUnifiedIdeograph3f99', [], 'Display', None],
  [ 0x3f9a, 'CjkUnifiedIdeograph3f9a', [], 'Display', None],
  [ 0x3f9b, 'CjkUnifiedIdeograph3f9b', [], 'Display', None],
  [ 0x3f9c, 'CjkUnifiedIdeograph3f9c', [], 'Display', None],
  [ 0x3f9d, 'CjkUnifiedIdeograph3f9d', [], 'Display', None],
  [ 0x3f9e, 'CjkUnifiedIdeograph3f9e', [], 'Display', None],
  [ 0x3f9f, 'CjkUnifiedIdeograph3f9f', [], 'Display', None],
  [ 0x3fa0, 'CjkUnifiedIdeograph3fa0', [], 'Display', None],
  [ 0x3fa1, 'CjkUnifiedIdeograph3fa1', [], 'Display', None],
  [ 0x3fa2, 'CjkUnifiedIdeograph3fa2', [], 'Display', None],
  [ 0x3fa3, 'CjkUnifiedIdeograph3fa3', [], 'Display', None],
  [ 0x3fa4, 'CjkUnifiedIdeograph3fa4', [], 'Display', None],
  [ 0x3fa5, 'CjkUnifiedIdeograph3fa5', [], 'Display', None],
  [ 0x3fa6, 'CjkUnifiedIdeograph3fa6', [], 'Display', None],
  [ 0x3fa7, 'CjkUnifiedIdeograph3fa7', [], 'Display', None],
  [ 0x3fa8, 'CjkUnifiedIdeograph3fa8', [], 'Display', None],
  [ 0x3fa9, 'CjkUnifiedIdeograph3fa9', [], 'Display', None],
  [ 0x3faa, 'CjkUnifiedIdeograph3faa', [], 'Display', None],
  [ 0x3fab, 'CjkUnifiedIdeograph3fab', [], 'Display', None],
  [ 0x3fac, 'CjkUnifiedIdeograph3fac', [], 'Display', None],
  [ 0x3fad, 'CjkUnifiedIdeograph3fad', [], 'Display', None],
  [ 0x3fae, 'CjkUnifiedIdeograph3fae', [], 'Display', None],
  [ 0x3faf, 'CjkUnifiedIdeograph3faf', [], 'Display', None],
  [ 0x3fb0, 'CjkUnifiedIdeograph3fb0', [], 'Display', None],
  [ 0x3fb1, 'CjkUnifiedIdeograph3fb1', [], 'Display', None],
  [ 0x3fb2, 'CjkUnifiedIdeograph3fb2', [], 'Display', None],
  [ 0x3fb3, 'CjkUnifiedIdeograph3fb3', [], 'Display', None],
  [ 0x3fb4, 'CjkUnifiedIdeograph3fb4', [], 'Display', None],
  [ 0x3fb5, 'CjkUnifiedIdeograph3fb5', [], 'Display', None],
  [ 0x3fb6, 'CjkUnifiedIdeograph3fb6', [], 'Display', None],
  [ 0x3fb7, 'CjkUnifiedIdeograph3fb7', [], 'Display', None],
  [ 0x3fb8, 'CjkUnifiedIdeograph3fb8', [], 'Display', None],
  [ 0x3fb9, 'CjkUnifiedIdeograph3fb9', [], 'Display', None],
  [ 0x3fba, 'CjkUnifiedIdeograph3fba', [], 'Display', None],
  [ 0x3fbb, 'CjkUnifiedIdeograph3fbb', [], 'Display', None],
  [ 0x3fbc, 'CjkUnifiedIdeograph3fbc', [], 'Display', None],
  [ 0x3fbd, 'CjkUnifiedIdeograph3fbd', [], 'Display', None],
  [ 0x3fbe, 'CjkUnifiedIdeograph3fbe', [], 'Display', None],
  [ 0x3fbf, 'CjkUnifiedIdeograph3fbf', [], 'Display', None],
  [ 0x3fc0, 'CjkUnifiedIdeograph3fc0', [], 'Display', None],
  [ 0x3fc1, 'CjkUnifiedIdeograph3fc1', [], 'Display', None],
  [ 0x3fc2, 'CjkUnifiedIdeograph3fc2', [], 'Display', None],
  [ 0x3fc3, 'CjkUnifiedIdeograph3fc3', [], 'Display', None],
  [ 0x3fc4, 'CjkUnifiedIdeograph3fc4', [], 'Display', None],
  [ 0x3fc5, 'CjkUnifiedIdeograph3fc5', [], 'Display', None],
  [ 0x3fc6, 'CjkUnifiedIdeograph3fc6', [], 'Display', None],
  [ 0x3fc7, 'CjkUnifiedIdeograph3fc7', [], 'Display', None],
  [ 0x3fc8, 'CjkUnifiedIdeograph3fc8', [], 'Display', None],
  [ 0x3fc9, 'CjkUnifiedIdeograph3fc9', [], 'Display', None],
  [ 0x3fca, 'CjkUnifiedIdeograph3fca', [], 'Display', None],
  [ 0x3fcb, 'CjkUnifiedIdeograph3fcb', [], 'Display', None],
  [ 0x3fcc, 'CjkUnifiedIdeograph3fcc', [], 'Display', None],
  [ 0x3fcd, 'CjkUnifiedIdeograph3fcd', [], 'Display', None],
  [ 0x3fce, 'CjkUnifiedIdeograph3fce', [], 'Display', None],
  [ 0x3fcf, 'CjkUnifiedIdeograph3fcf', [], 'Display', None],
  [ 0x3fd0, 'CjkUnifiedIdeograph3fd0', [], 'Display', None],
  [ 0x3fd1, 'CjkUnifiedIdeograph3fd1', [], 'Display', None],
  [ 0x3fd2, 'CjkUnifiedIdeograph3fd2', [], 'Display', None],
  [ 0x3fd3, 'CjkUnifiedIdeograph3fd3', [], 'Display', None],
  [ 0x3fd4, 'CjkUnifiedIdeograph3fd4', [], 'Display', None],
  [ 0x3fd5, 'CjkUnifiedIdeograph3fd5', [], 'Display', None],
  [ 0x3fd6, 'CjkUnifiedIdeograph3fd6', [], 'Display', None],
  [ 0x3fd7, 'CjkUnifiedIdeograph3fd7', [], 'Display', None],
  [ 0x3fd8, 'CjkUnifiedIdeograph3fd8', [], 'Display', None],
  [ 0x3fd9, 'CjkUnifiedIdeograph3fd9', [], 'Display', None],
  [ 0x3fda, 'CjkUnifiedIdeograph3fda', [], 'Display', None],
  [ 0x3fdb, 'CjkUnifiedIdeograph3fdb', [], 'Display', None],
  [ 0x3fdc, 'CjkUnifiedIdeograph3fdc', [], 'Display', None],
  [ 0x3fdd, 'CjkUnifiedIdeograph3fdd', [], 'Display', None],
  [ 0x3fde, 'CjkUnifiedIdeograph3fde', [], 'Display', None],
  [ 0x3fdf, 'CjkUnifiedIdeograph3fdf', [], 'Display', None],
  [ 0x3fe0, 'CjkUnifiedIdeograph3fe0', [], 'Display', None],
  [ 0x3fe1, 'CjkUnifiedIdeograph3fe1', [], 'Display', None],
  [ 0x3fe2, 'CjkUnifiedIdeograph3fe2', [], 'Display', None],
  [ 0x3fe3, 'CjkUnifiedIdeograph3fe3', [], 'Display', None],
  [ 0x3fe4, 'CjkUnifiedIdeograph3fe4', [], 'Display', None],
  [ 0x3fe5, 'CjkUnifiedIdeograph3fe5', [], 'Display', None],
  [ 0x3fe6, 'CjkUnifiedIdeograph3fe6', [], 'Display', None],
  [ 0x3fe7, 'CjkUnifiedIdeograph3fe7', [], 'Display', None],
  [ 0x3fe8, 'CjkUnifiedIdeograph3fe8', [], 'Display', None],
  [ 0x3fe9, 'CjkUnifiedIdeograph3fe9', [], 'Display', None],
  [ 0x3fea, 'CjkUnifiedIdeograph3fea', [], 'Display', None],
  [ 0x3feb, 'CjkUnifiedIdeograph3feb', [], 'Display', None],
  [ 0x3fec, 'CjkUnifiedIdeograph3fec', [], 'Display', None],
  [ 0x3fed, 'CjkUnifiedIdeograph3fed', [], 'Display', None],
  [ 0x3fee, 'CjkUnifiedIdeograph3fee', [], 'Display', None],
  [ 0x3fef, 'CjkUnifiedIdeograph3fef', [], 'Display', None],
  [ 0x3ff0, 'CjkUnifiedIdeograph3ff0', [], 'Display', None],
  [ 0x3ff1, 'CjkUnifiedIdeograph3ff1', [], 'Display', None],
  [ 0x3ff2, 'CjkUnifiedIdeograph3ff2', [], 'Display', None],
  [ 0x3ff3, 'CjkUnifiedIdeograph3ff3', [], 'Display', None],
  [ 0x3ff4, 'CjkUnifiedIdeograph3ff4', [], 'Display', None],
  [ 0x3ff5, 'CjkUnifiedIdeograph3ff5', [], 'Display', None],
  [ 0x3ff6, 'CjkUnifiedIdeograph3ff6', [], 'Display', None],
  [ 0x3ff7, 'CjkUnifiedIdeograph3ff7', [], 'Display', None],
  [ 0x3ff8, 'CjkUnifiedIdeograph3ff8', [], 'Display', None],
  [ 0x3ff9, 'CjkUnifiedIdeograph3ff9', [], 'Display', None],
  [ 0x3ffa, 'CjkUnifiedIdeograph3ffa', [], 'Display', None],
  [ 0x3ffb, 'CjkUnifiedIdeograph3ffb', [], 'Display', None],
  [ 0x3ffc, 'CjkUnifiedIdeograph3ffc', [], 'Display', None],
  [ 0x3ffd, 'CjkUnifiedIdeograph3ffd', [], 'Display', None],
  [ 0x3ffe, 'CjkUnifiedIdeograph3ffe', [], 'Display', None],
  [ 0x3fff, 'CjkUnifiedIdeograph3fff', [], 'Display', None],
  [ 0x4000, 'CjkUnifiedIdeograph4000', [], 'Display', None],
  [ 0x4001, 'CjkUnifiedIdeograph4001', [], 'Display', None],
  [ 0x4002, 'CjkUnifiedIdeograph4002', [], 'Display', None],
  [ 0x4003, 'CjkUnifiedIdeograph4003', [], 'Display', None],
  [ 0x4004, 'CjkUnifiedIdeograph4004', [], 'Display', None],
  [ 0x4005, 'CjkUnifiedIdeograph4005', [], 'Display', None],
  [ 0x4006, 'CjkUnifiedIdeograph4006', [], 'Display', None],
  [ 0x4007, 'CjkUnifiedIdeograph4007', [], 'Display', None],
  [ 0x4008, 'CjkUnifiedIdeograph4008', [], 'Display', None],
  [ 0x4009, 'CjkUnifiedIdeograph4009', [], 'Display', None],
  [ 0x400a, 'CjkUnifiedIdeograph400a', [], 'Display', None],
  [ 0x400b, 'CjkUnifiedIdeograph400b', [], 'Display', None],
  [ 0x400c, 'CjkUnifiedIdeograph400c', [], 'Display', None],
  [ 0x400d, 'CjkUnifiedIdeograph400d', [], 'Display', None],
  [ 0x400e, 'CjkUnifiedIdeograph400e', [], 'Display', None],
  [ 0x400f, 'CjkUnifiedIdeograph400f', [], 'Display', None],
  [ 0x4010, 'CjkUnifiedIdeograph4010', [], 'Display', None],
  [ 0x4011, 'CjkUnifiedIdeograph4011', [], 'Display', None],
  [ 0x4012, 'CjkUnifiedIdeograph4012', [], 'Display', None],
  [ 0x4013, 'CjkUnifiedIdeograph4013', [], 'Display', None],
  [ 0x4014, 'CjkUnifiedIdeograph4014', [], 'Display', None],
  [ 0x4015, 'CjkUnifiedIdeograph4015', [], 'Display', None],
  [ 0x4016, 'CjkUnifiedIdeograph4016', [], 'Display', None],
  [ 0x4017, 'CjkUnifiedIdeograph4017', [], 'Display', None],
  [ 0x4018, 'CjkUnifiedIdeograph4018', [], 'Display', None],
  [ 0x4019, 'CjkUnifiedIdeograph4019', [], 'Display', None],
  [ 0x401a, 'CjkUnifiedIdeograph401a', [], 'Display', None],
  [ 0x401b, 'CjkUnifiedIdeograph401b', [], 'Display', None],
  [ 0x401c, 'CjkUnifiedIdeograph401c', [], 'Display', None],
  [ 0x401d, 'CjkUnifiedIdeograph401d', [], 'Display', None],
  [ 0x401e, 'CjkUnifiedIdeograph401e', [], 'Display', None],
  [ 0x401f, 'CjkUnifiedIdeograph401f', [], 'Display', None],
  [ 0x4020, 'CjkUnifiedIdeograph4020', [], 'Display', None],
  [ 0x4021, 'CjkUnifiedIdeograph4021', [], 'Display', None],
  [ 0x4022, 'CjkUnifiedIdeograph4022', [], 'Display', None],
  [ 0x4023, 'CjkUnifiedIdeograph4023', [], 'Display', None],
  [ 0x4024, 'CjkUnifiedIdeograph4024', [], 'Display', None],
  [ 0x4025, 'CjkUnifiedIdeograph4025', [], 'Display', None],
  [ 0x4026, 'CjkUnifiedIdeograph4026', [], 'Display', None],
  [ 0x4027, 'CjkUnifiedIdeograph4027', [], 'Display', None],
  [ 0x4028, 'CjkUnifiedIdeograph4028', [], 'Display', None],
  [ 0x4029, 'CjkUnifiedIdeograph4029', [], 'Display', None],
  [ 0x402a, 'CjkUnifiedIdeograph402a', [], 'Display', None],
  [ 0x402b, 'CjkUnifiedIdeograph402b', [], 'Display', None],
  [ 0x402c, 'CjkUnifiedIdeograph402c', [], 'Display', None],
  [ 0x402d, 'CjkUnifiedIdeograph402d', [], 'Display', None],
  [ 0x402e, 'CjkUnifiedIdeograph402e', [], 'Display', None],
  [ 0x402f, 'CjkUnifiedIdeograph402f', [], 'Display', None],
  [ 0x4030, 'CjkUnifiedIdeograph4030', [], 'Display', None],
  [ 0x4031, 'CjkUnifiedIdeograph4031', [], 'Display', None],
  [ 0x4032, 'CjkUnifiedIdeograph4032', [], 'Display', None],
  [ 0x4033, 'CjkUnifiedIdeograph4033', [], 'Display', None],
  [ 0x4034, 'CjkUnifiedIdeograph4034', [], 'Display', None],
  [ 0x4035, 'CjkUnifiedIdeograph4035', [], 'Display', None],
  [ 0x4036, 'CjkUnifiedIdeograph4036', [], 'Display', None],
  [ 0x4037, 'CjkUnifiedIdeograph4037', [], 'Display', None],
  [ 0x4038, 'CjkUnifiedIdeograph4038', [], 'Display', None],
  [ 0x4039, 'CjkUnifiedIdeograph4039', [], 'Display', None],
  [ 0x403a, 'CjkUnifiedIdeograph403a', [], 'Display', None],
  [ 0x403b, 'CjkUnifiedIdeograph403b', [], 'Display', None],
  [ 0x403c, 'CjkUnifiedIdeograph403c', [], 'Display', None],
  [ 0x403d, 'CjkUnifiedIdeograph403d', [], 'Display', None],
  [ 0x403e, 'CjkUnifiedIdeograph403e', [], 'Display', None],
  [ 0x403f, 'CjkUnifiedIdeograph403f', [], 'Display', None],
  [ 0x4040, 'CjkUnifiedIdeograph4040', [], 'Display', None],
  [ 0x4041, 'CjkUnifiedIdeograph4041', [], 'Display', None],
  [ 0x4042, 'CjkUnifiedIdeograph4042', [], 'Display', None],
  [ 0x4043, 'CjkUnifiedIdeograph4043', [], 'Display', None],
  [ 0x4044, 'CjkUnifiedIdeograph4044', [], 'Display', None],
  [ 0x4045, 'CjkUnifiedIdeograph4045', [], 'Display', None],
  [ 0x4046, 'CjkUnifiedIdeograph4046', [], 'Display', None],
  [ 0x4047, 'CjkUnifiedIdeograph4047', [], 'Display', None],
  [ 0x4048, 'CjkUnifiedIdeograph4048', [], 'Display', None],
  [ 0x4049, 'CjkUnifiedIdeograph4049', [], 'Display', None],
  [ 0x404a, 'CjkUnifiedIdeograph404a', [], 'Display', None],
  [ 0x404b, 'CjkUnifiedIdeograph404b', [], 'Display', None],
  [ 0x404c, 'CjkUnifiedIdeograph404c', [], 'Display', None],
  [ 0x404d, 'CjkUnifiedIdeograph404d', [], 'Display', None],
  [ 0x404e, 'CjkUnifiedIdeograph404e', [], 'Display', None],
  [ 0x404f, 'CjkUnifiedIdeograph404f', [], 'Display', None],
  [ 0x4050, 'CjkUnifiedIdeograph4050', [], 'Display', None],
  [ 0x4051, 'CjkUnifiedIdeograph4051', [], 'Display', None],
  [ 0x4052, 'CjkUnifiedIdeograph4052', [], 'Display', None],
  [ 0x4053, 'CjkUnifiedIdeograph4053', [], 'Display', None],
  [ 0x4054, 'CjkUnifiedIdeograph4054', [], 'Display', None],
  [ 0x4055, 'CjkUnifiedIdeograph4055', [], 'Display', None],
  [ 0x4056, 'CjkUnifiedIdeograph4056', [], 'Display', None],
  [ 0x4057, 'CjkUnifiedIdeograph4057', [], 'Display', None],
  [ 0x4058, 'CjkUnifiedIdeograph4058', [], 'Display', None],
  [ 0x4059, 'CjkUnifiedIdeograph4059', [], 'Display', None],
  [ 0x405a, 'CjkUnifiedIdeograph405a', [], 'Display', None],
  [ 0x405b, 'CjkUnifiedIdeograph405b', [], 'Display', None],
  [ 0x405c, 'CjkUnifiedIdeograph405c', [], 'Display', None],
  [ 0x405d, 'CjkUnifiedIdeograph405d', [], 'Display', None],
  [ 0x405e, 'CjkUnifiedIdeograph405e', [], 'Display', None],
  [ 0x405f, 'CjkUnifiedIdeograph405f', [], 'Display', None],
  [ 0x4060, 'CjkUnifiedIdeograph4060', [], 'Display', None],
  [ 0x4061, 'CjkUnifiedIdeograph4061', [], 'Display', None],
  [ 0x4062, 'CjkUnifiedIdeograph4062', [], 'Display', None],
  [ 0x4063, 'CjkUnifiedIdeograph4063', [], 'Display', None],
  [ 0x4064, 'CjkUnifiedIdeograph4064', [], 'Display', None],
  [ 0x4065, 'CjkUnifiedIdeograph4065', [], 'Display', None],
  [ 0x4066, 'CjkUnifiedIdeograph4066', [], 'Display', None],
  [ 0x4067, 'CjkUnifiedIdeograph4067', [], 'Display', None],
  [ 0x4068, 'CjkUnifiedIdeograph4068', [], 'Display', None],
  [ 0x4069, 'CjkUnifiedIdeograph4069', [], 'Display', None],
  [ 0x406a, 'CjkUnifiedIdeograph406a', [], 'Display', None],
  [ 0x406b, 'CjkUnifiedIdeograph406b', [], 'Display', None],
  [ 0x406c, 'CjkUnifiedIdeograph406c', [], 'Display', None],
  [ 0x406d, 'CjkUnifiedIdeograph406d', [], 'Display', None],
  [ 0x406e, 'CjkUnifiedIdeograph406e', [], 'Display', None],
  [ 0x406f, 'CjkUnifiedIdeograph406f', [], 'Display', None],
  [ 0x4070, 'CjkUnifiedIdeograph4070', [], 'Display', None],
  [ 0x4071, 'CjkUnifiedIdeograph4071', [], 'Display', None],
  [ 0x4072, 'CjkUnifiedIdeograph4072', [], 'Display', None],
  [ 0x4073, 'CjkUnifiedIdeograph4073', [], 'Display', None],
  [ 0x4074, 'CjkUnifiedIdeograph4074', [], 'Display', None],
  [ 0x4075, 'CjkUnifiedIdeograph4075', [], 'Display', None],
  [ 0x4076, 'CjkUnifiedIdeograph4076', [], 'Display', None],
  [ 0x4077, 'CjkUnifiedIdeograph4077', [], 'Display', None],
  [ 0x4078, 'CjkUnifiedIdeograph4078', [], 'Display', None],
  [ 0x4079, 'CjkUnifiedIdeograph4079', [], 'Display', None],
  [ 0x407a, 'CjkUnifiedIdeograph407a', [], 'Display', None],
  [ 0x407b, 'CjkUnifiedIdeograph407b', [], 'Display', None],
  [ 0x407c, 'CjkUnifiedIdeograph407c', [], 'Display', None],
  [ 0x407d, 'CjkUnifiedIdeograph407d', [], 'Display', None],
  [ 0x407e, 'CjkUnifiedIdeograph407e', [], 'Display', None],
  [ 0x407f, 'CjkUnifiedIdeograph407f', [], 'Display', None],
  [ 0x4080, 'CjkUnifiedIdeograph4080', [], 'Display', None],
  [ 0x4081, 'CjkUnifiedIdeograph4081', [], 'Display', None],
  [ 0x4082, 'CjkUnifiedIdeograph4082', [], 'Display', None],
  [ 0x4083, 'CjkUnifiedIdeograph4083', [], 'Display', None],
  [ 0x4084, 'CjkUnifiedIdeograph4084', [], 'Display', None],
  [ 0x4085, 'CjkUnifiedIdeograph4085', [], 'Display', None],
  [ 0x4086, 'CjkUnifiedIdeograph4086', [], 'Display', None],
  [ 0x4087, 'CjkUnifiedIdeograph4087', [], 'Display', None],
  [ 0x4088, 'CjkUnifiedIdeograph4088', [], 'Display', None],
  [ 0x4089, 'CjkUnifiedIdeograph4089', [], 'Display', None],
  [ 0x408a, 'CjkUnifiedIdeograph408a', [], 'Display', None],
  [ 0x408b, 'CjkUnifiedIdeograph408b', [], 'Display', None],
  [ 0x408c, 'CjkUnifiedIdeograph408c', [], 'Display', None],
  [ 0x408d, 'CjkUnifiedIdeograph408d', [], 'Display', None],
  [ 0x408e, 'CjkUnifiedIdeograph408e', [], 'Display', None],
  [ 0x408f, 'CjkUnifiedIdeograph408f', [], 'Display', None],
  [ 0x4090, 'CjkUnifiedIdeograph4090', [], 'Display', None],
  [ 0x4091, 'CjkUnifiedIdeograph4091', [], 'Display', None],
  [ 0x4092, 'CjkUnifiedIdeograph4092', [], 'Display', None],
  [ 0x4093, 'CjkUnifiedIdeograph4093', [], 'Display', None],
  [ 0x4094, 'CjkUnifiedIdeograph4094', [], 'Display', None],
  [ 0x4095, 'CjkUnifiedIdeograph4095', [], 'Display', None],
  [ 0x4096, 'CjkUnifiedIdeograph4096', [], 'Display', None],
  [ 0x4097, 'CjkUnifiedIdeograph4097', [], 'Display', None],
  [ 0x4098, 'CjkUnifiedIdeograph4098', [], 'Display', None],
  [ 0x4099, 'CjkUnifiedIdeograph4099', [], 'Display', None],
  [ 0x409a, 'CjkUnifiedIdeograph409a', [], 'Display', None],
  [ 0x409b, 'CjkUnifiedIdeograph409b', [], 'Display', None],
  [ 0x409c, 'CjkUnifiedIdeograph409c', [], 'Display', None],
  [ 0x409d, 'CjkUnifiedIdeograph409d', [], 'Display', None],
  [ 0x409e, 'CjkUnifiedIdeograph409e', [], 'Display', None],
  [ 0x409f, 'CjkUnifiedIdeograph409f', [], 'Display', None],
  [ 0x40a0, 'CjkUnifiedIdeograph40a0', [], 'Display', None],
  [ 0x40a1, 'CjkUnifiedIdeograph40a1', [], 'Display', None],
  [ 0x40a2, 'CjkUnifiedIdeograph40a2', [], 'Display', None],
  [ 0x40a3, 'CjkUnifiedIdeograph40a3', [], 'Display', None],
  [ 0x40a4, 'CjkUnifiedIdeograph40a4', [], 'Display', None],
  [ 0x40a5, 'CjkUnifiedIdeograph40a5', [], 'Display', None],
  [ 0x40a6, 'CjkUnifiedIdeograph40a6', [], 'Display', None],
  [ 0x40a7, 'CjkUnifiedIdeograph40a7', [], 'Display', None],
  [ 0x40a8, 'CjkUnifiedIdeograph40a8', [], 'Display', None],
  [ 0x40a9, 'CjkUnifiedIdeograph40a9', [], 'Display', None],
  [ 0x40aa, 'CjkUnifiedIdeograph40aa', [], 'Display', None],
  [ 0x40ab, 'CjkUnifiedIdeograph40ab', [], 'Display', None],
  [ 0x40ac, 'CjkUnifiedIdeograph40ac', [], 'Display', None],
  [ 0x40ad, 'CjkUnifiedIdeograph40ad', [], 'Display', None],
  [ 0x40ae, 'CjkUnifiedIdeograph40ae', [], 'Display', None],
  [ 0x40af, 'CjkUnifiedIdeograph40af', [], 'Display', None],
  [ 0x40b0, 'CjkUnifiedIdeograph40b0', [], 'Display', None],
  [ 0x40b1, 'CjkUnifiedIdeograph40b1', [], 'Display', None],
  [ 0x40b2, 'CjkUnifiedIdeograph40b2', [], 'Display', None],
  [ 0x40b3, 'CjkUnifiedIdeograph40b3', [], 'Display', None],
  [ 0x40b4, 'CjkUnifiedIdeograph40b4', [], 'Display', None],
  [ 0x40b5, 'CjkUnifiedIdeograph40b5', [], 'Display', None],
  [ 0x40b6, 'CjkUnifiedIdeograph40b6', [], 'Display', None],
  [ 0x40b7, 'CjkUnifiedIdeograph40b7', [], 'Display', None],
  [ 0x40b8, 'CjkUnifiedIdeograph40b8', [], 'Display', None],
  [ 0x40b9, 'CjkUnifiedIdeograph40b9', [], 'Display', None],
  [ 0x40ba, 'CjkUnifiedIdeograph40ba', [], 'Display', None],
  [ 0x40bb, 'CjkUnifiedIdeograph40bb', [], 'Display', None],
  [ 0x40bc, 'CjkUnifiedIdeograph40bc', [], 'Display', None],
  [ 0x40bd, 'CjkUnifiedIdeograph40bd', [], 'Display', None],
  [ 0x40be, 'CjkUnifiedIdeograph40be', [], 'Display', None],
  [ 0x40bf, 'CjkUnifiedIdeograph40bf', [], 'Display', None],
  [ 0x40c0, 'CjkUnifiedIdeograph40c0', [], 'Display', None],
  [ 0x40c1, 'CjkUnifiedIdeograph40c1', [], 'Display', None],
  [ 0x40c2, 'CjkUnifiedIdeograph40c2', [], 'Display', None],
  [ 0x40c3, 'CjkUnifiedIdeograph40c3', [], 'Display', None],
  [ 0x40c4, 'CjkUnifiedIdeograph40c4', [], 'Display', None],
  [ 0x40c5, 'CjkUnifiedIdeograph40c5', [], 'Display', None],
  [ 0x40c6, 'CjkUnifiedIdeograph40c6', [], 'Display', None],
  [ 0x40c7, 'CjkUnifiedIdeograph40c7', [], 'Display', None],
  [ 0x40c8, 'CjkUnifiedIdeograph40c8', [], 'Display', None],
  [ 0x40c9, 'CjkUnifiedIdeograph40c9', [], 'Display', None],
  [ 0x40ca, 'CjkUnifiedIdeograph40ca', [], 'Display', None],
  [ 0x40cb, 'CjkUnifiedIdeograph40cb', [], 'Display', None],
  [ 0x40cc, 'CjkUnifiedIdeograph40cc', [], 'Display', None],
  [ 0x40cd, 'CjkUnifiedIdeograph40cd', [], 'Display', None],
  [ 0x40ce, 'CjkUnifiedIdeograph40ce', [], 'Display', None],
  [ 0x40cf, 'CjkUnifiedIdeograph40cf', [], 'Display', None],
  [ 0x40d0, 'CjkUnifiedIdeograph40d0', [], 'Display', None],
  [ 0x40d1, 'CjkUnifiedIdeograph40d1', [], 'Display', None],
  [ 0x40d2, 'CjkUnifiedIdeograph40d2', [], 'Display', None],
  [ 0x40d3, 'CjkUnifiedIdeograph40d3', [], 'Display', None],
  [ 0x40d4, 'CjkUnifiedIdeograph40d4', [], 'Display', None],
  [ 0x40d5, 'CjkUnifiedIdeograph40d5', [], 'Display', None],
  [ 0x40d6, 'CjkUnifiedIdeograph40d6', [], 'Display', None],
  [ 0x40d7, 'CjkUnifiedIdeograph40d7', [], 'Display', None],
  [ 0x40d8, 'CjkUnifiedIdeograph40d8', [], 'Display', None],
  [ 0x40d9, 'CjkUnifiedIdeograph40d9', [], 'Display', None],
  [ 0x40da, 'CjkUnifiedIdeograph40da', [], 'Display', None],
  [ 0x40db, 'CjkUnifiedIdeograph40db', [], 'Display', None],
  [ 0x40dc, 'CjkUnifiedIdeograph40dc', [], 'Display', None],
  [ 0x40dd, 'CjkUnifiedIdeograph40dd', [], 'Display', None],
  [ 0x40de, 'CjkUnifiedIdeograph40de', [], 'Display', None],
  [ 0x40df, 'CjkUnifiedIdeograph40df', [], 'Display', None],
  [ 0x40e0, 'CjkUnifiedIdeograph40e0', [], 'Display', None],
  [ 0x40e1, 'CjkUnifiedIdeograph40e1', [], 'Display', None],
  [ 0x40e2, 'CjkUnifiedIdeograph40e2', [], 'Display', None],
  [ 0x40e3, 'CjkUnifiedIdeograph40e3', [], 'Display', None],
  [ 0x40e4, 'CjkUnifiedIdeograph40e4', [], 'Display', None],
  [ 0x40e5, 'CjkUnifiedIdeograph40e5', [], 'Display', None],
  [ 0x40e6, 'CjkUnifiedIdeograph40e6', [], 'Display', None],
  [ 0x40e7, 'CjkUnifiedIdeograph40e7', [], 'Display', None],
  [ 0x40e8, 'CjkUnifiedIdeograph40e8', [], 'Display', None],
  [ 0x40e9, 'CjkUnifiedIdeograph40e9', [], 'Display', None],
  [ 0x40ea, 'CjkUnifiedIdeograph40ea', [], 'Display', None],
  [ 0x40eb, 'CjkUnifiedIdeograph40eb', [], 'Display', None],
  [ 0x40ec, 'CjkUnifiedIdeograph40ec', [], 'Display', None],
  [ 0x40ed, 'CjkUnifiedIdeograph40ed', [], 'Display', None],
  [ 0x40ee, 'CjkUnifiedIdeograph40ee', [], 'Display', None],
  [ 0x40ef, 'CjkUnifiedIdeograph40ef', [], 'Display', None],
  [ 0x40f0, 'CjkUnifiedIdeograph40f0', [], 'Display', None],
  [ 0x40f1, 'CjkUnifiedIdeograph40f1', [], 'Display', None],
  [ 0x40f2, 'CjkUnifiedIdeograph40f2', [], 'Display', None],
  [ 0x40f3, 'CjkUnifiedIdeograph40f3', [], 'Display', None],
  [ 0x40f4, 'CjkUnifiedIdeograph40f4', [], 'Display', None],
  [ 0x40f5, 'CjkUnifiedIdeograph40f5', [], 'Display', None],
  [ 0x40f6, 'CjkUnifiedIdeograph40f6', [], 'Display', None],
  [ 0x40f7, 'CjkUnifiedIdeograph40f7', [], 'Display', None],
  [ 0x40f8, 'CjkUnifiedIdeograph40f8', [], 'Display', None],
  [ 0x40f9, 'CjkUnifiedIdeograph40f9', [], 'Display', None],
  [ 0x40fa, 'CjkUnifiedIdeograph40fa', [], 'Display', None],
  [ 0x40fb, 'CjkUnifiedIdeograph40fb', [], 'Display', None],
  [ 0x40fc, 'CjkUnifiedIdeograph40fc', [], 'Display', None],
  [ 0x40fd, 'CjkUnifiedIdeograph40fd', [], 'Display', None],
  [ 0x40fe, 'CjkUnifiedIdeograph40fe', [], 'Display', None],
  [ 0x40ff, 'CjkUnifiedIdeograph40ff', [], 'Display', None],
  [ 0x4100, 'CjkUnifiedIdeograph4100', [], 'Display', None],
  [ 0x4101, 'CjkUnifiedIdeograph4101', [], 'Display', None],
  [ 0x4102, 'CjkUnifiedIdeograph4102', [], 'Display', None],
  [ 0x4103, 'CjkUnifiedIdeograph4103', [], 'Display', None],
  [ 0x4104, 'CjkUnifiedIdeograph4104', [], 'Display', None],
  [ 0x4105, 'CjkUnifiedIdeograph4105', [], 'Display', None],
  [ 0x4106, 'CjkUnifiedIdeograph4106', [], 'Display', None],
  [ 0x4107, 'CjkUnifiedIdeograph4107', [], 'Display', None],
  [ 0x4108, 'CjkUnifiedIdeograph4108', [], 'Display', None],
  [ 0x4109, 'CjkUnifiedIdeograph4109', [], 'Display', None],
  [ 0x410a, 'CjkUnifiedIdeograph410a', [], 'Display', None],
  [ 0x410b, 'CjkUnifiedIdeograph410b', [], 'Display', None],
  [ 0x410c, 'CjkUnifiedIdeograph410c', [], 'Display', None],
  [ 0x410d, 'CjkUnifiedIdeograph410d', [], 'Display', None],
  [ 0x410e, 'CjkUnifiedIdeograph410e', [], 'Display', None],
  [ 0x410f, 'CjkUnifiedIdeograph410f', [], 'Display', None],
  [ 0x4110, 'CjkUnifiedIdeograph4110', [], 'Display', None],
  [ 0x4111, 'CjkUnifiedIdeograph4111', [], 'Display', None],
  [ 0x4112, 'CjkUnifiedIdeograph4112', [], 'Display', None],
  [ 0x4113, 'CjkUnifiedIdeograph4113', [], 'Display', None],
  [ 0x4114, 'CjkUnifiedIdeograph4114', [], 'Display', None],
  [ 0x4115, 'CjkUnifiedIdeograph4115', [], 'Display', None],
  [ 0x4116, 'CjkUnifiedIdeograph4116', [], 'Display', None],
  [ 0x4117, 'CjkUnifiedIdeograph4117', [], 'Display', None],
  [ 0x4118, 'CjkUnifiedIdeograph4118', [], 'Display', None],
  [ 0x4119, 'CjkUnifiedIdeograph4119', [], 'Display', None],
  [ 0x411a, 'CjkUnifiedIdeograph411a', [], 'Display', None],
  [ 0x411b, 'CjkUnifiedIdeograph411b', [], 'Display', None],
  [ 0x411c, 'CjkUnifiedIdeograph411c', [], 'Display', None],
  [ 0x411d, 'CjkUnifiedIdeograph411d', [], 'Display', None],
  [ 0x411e, 'CjkUnifiedIdeograph411e', [], 'Display', None],
  [ 0x411f, 'CjkUnifiedIdeograph411f', [], 'Display', None],
  [ 0x4120, 'CjkUnifiedIdeograph4120', [], 'Display', None],
  [ 0x4121, 'CjkUnifiedIdeograph4121', [], 'Display', None],
  [ 0x4122, 'CjkUnifiedIdeograph4122', [], 'Display', None],
  [ 0x4123, 'CjkUnifiedIdeograph4123', [], 'Display', None],
  [ 0x4124, 'CjkUnifiedIdeograph4124', [], 'Display', None],
  [ 0x4125, 'CjkUnifiedIdeograph4125', [], 'Display', None],
  [ 0x4126, 'CjkUnifiedIdeograph4126', [], 'Display', None],
  [ 0x4127, 'CjkUnifiedIdeograph4127', [], 'Display', None],
  [ 0x4128, 'CjkUnifiedIdeograph4128', [], 'Display', None],
  [ 0x4129, 'CjkUnifiedIdeograph4129', [], 'Display', None],
  [ 0x412a, 'CjkUnifiedIdeograph412a', [], 'Display', None],
  [ 0x412b, 'CjkUnifiedIdeograph412b', [], 'Display', None],
  [ 0x412c, 'CjkUnifiedIdeograph412c', [], 'Display', None],
  [ 0x412d, 'CjkUnifiedIdeograph412d', [], 'Display', None],
  [ 0x412e, 'CjkUnifiedIdeograph412e', [], 'Display', None],
  [ 0x412f, 'CjkUnifiedIdeograph412f', [], 'Display', None],
  [ 0x4130, 'CjkUnifiedIdeograph4130', [], 'Display', None],
  [ 0x4131, 'CjkUnifiedIdeograph4131', [], 'Display', None],
  [ 0x4132, 'CjkUnifiedIdeograph4132', [], 'Display', None],
  [ 0x4133, 'CjkUnifiedIdeograph4133', [], 'Display', None],
  [ 0x4134, 'CjkUnifiedIdeograph4134', [], 'Display', None],
  [ 0x4135, 'CjkUnifiedIdeograph4135', [], 'Display', None],
  [ 0x4136, 'CjkUnifiedIdeograph4136', [], 'Display', None],
  [ 0x4137, 'CjkUnifiedIdeograph4137', [], 'Display', None],
  [ 0x4138, 'CjkUnifiedIdeograph4138', [], 'Display', None],
  [ 0x4139, 'CjkUnifiedIdeograph4139', [], 'Display', None],
  [ 0x413a, 'CjkUnifiedIdeograph413a', [], 'Display', None],
  [ 0x413b, 'CjkUnifiedIdeograph413b', [], 'Display', None],
  [ 0x413c, 'CjkUnifiedIdeograph413c', [], 'Display', None],
  [ 0x413d, 'CjkUnifiedIdeograph413d', [], 'Display', None],
  [ 0x413e, 'CjkUnifiedIdeograph413e', [], 'Display', None],
  [ 0x413f, 'CjkUnifiedIdeograph413f', [], 'Display', None],
  [ 0x4140, 'CjkUnifiedIdeograph4140', [], 'Display', None],
  [ 0x4141, 'CjkUnifiedIdeograph4141', [], 'Display', None],
  [ 0x4142, 'CjkUnifiedIdeograph4142', [], 'Display', None],
  [ 0x4143, 'CjkUnifiedIdeograph4143', [], 'Display', None],
  [ 0x4144, 'CjkUnifiedIdeograph4144', [], 'Display', None],
  [ 0x4145, 'CjkUnifiedIdeograph4145', [], 'Display', None],
  [ 0x4146, 'CjkUnifiedIdeograph4146', [], 'Display', None],
  [ 0x4147, 'CjkUnifiedIdeograph4147', [], 'Display', None],
  [ 0x4148, 'CjkUnifiedIdeograph4148', [], 'Display', None],
  [ 0x4149, 'CjkUnifiedIdeograph4149', [], 'Display', None],
  [ 0x414a, 'CjkUnifiedIdeograph414a', [], 'Display', None],
  [ 0x414b, 'CjkUnifiedIdeograph414b', [], 'Display', None],
  [ 0x414c, 'CjkUnifiedIdeograph414c', [], 'Display', None],
  [ 0x414d, 'CjkUnifiedIdeograph414d', [], 'Display', None],
  [ 0x414e, 'CjkUnifiedIdeograph414e', [], 'Display', None],
  [ 0x414f, 'CjkUnifiedIdeograph414f', [], 'Display', None],
  [ 0x4150, 'CjkUnifiedIdeograph4150', [], 'Display', None],
  [ 0x4151, 'CjkUnifiedIdeograph4151', [], 'Display', None],
  [ 0x4152, 'CjkUnifiedIdeograph4152', [], 'Display', None],
  [ 0x4153, 'CjkUnifiedIdeograph4153', [], 'Display', None],
  [ 0x4154, 'CjkUnifiedIdeograph4154', [], 'Display', None],
  [ 0x4155, 'CjkUnifiedIdeograph4155', [], 'Display', None],
  [ 0x4156, 'CjkUnifiedIdeograph4156', [], 'Display', None],
  [ 0x4157, 'CjkUnifiedIdeograph4157', [], 'Display', None],
  [ 0x4158, 'CjkUnifiedIdeograph4158', [], 'Display', None],
  [ 0x4159, 'CjkUnifiedIdeograph4159', [], 'Display', None],
  [ 0x415a, 'CjkUnifiedIdeograph415a', [], 'Display', None],
  [ 0x415b, 'CjkUnifiedIdeograph415b', [], 'Display', None],
  [ 0x415c, 'CjkUnifiedIdeograph415c', [], 'Display', None],
  [ 0x415d, 'CjkUnifiedIdeograph415d', [], 'Display', None],
  [ 0x415e, 'CjkUnifiedIdeograph415e', [], 'Display', None],
  [ 0x415f, 'CjkUnifiedIdeograph415f', [], 'Display', None],
  [ 0x4160, 'CjkUnifiedIdeograph4160', [], 'Display', None],
  [ 0x4161, 'CjkUnifiedIdeograph4161', [], 'Display', None],
  [ 0x4162, 'CjkUnifiedIdeograph4162', [], 'Display', None],
  [ 0x4163, 'CjkUnifiedIdeograph4163', [], 'Display', None],
  [ 0x4164, 'CjkUnifiedIdeograph4164', [], 'Display', None],
  [ 0x4165, 'CjkUnifiedIdeograph4165', [], 'Display', None],
  [ 0x4166, 'CjkUnifiedIdeograph4166', [], 'Display', None],
  [ 0x4167, 'CjkUnifiedIdeograph4167', [], 'Display', None],
  [ 0x4168, 'CjkUnifiedIdeograph4168', [], 'Display', None],
  [ 0x4169, 'CjkUnifiedIdeograph4169', [], 'Display', None],
  [ 0x416a, 'CjkUnifiedIdeograph416a', [], 'Display', None],
  [ 0x416b, 'CjkUnifiedIdeograph416b', [], 'Display', None],
  [ 0x416c, 'CjkUnifiedIdeograph416c', [], 'Display', None],
  [ 0x416d, 'CjkUnifiedIdeograph416d', [], 'Display', None],
  [ 0x416e, 'CjkUnifiedIdeograph416e', [], 'Display', None],
  [ 0x416f, 'CjkUnifiedIdeograph416f', [], 'Display', None],
  [ 0x4170, 'CjkUnifiedIdeograph4170', [], 'Display', None],
  [ 0x4171, 'CjkUnifiedIdeograph4171', [], 'Display', None],
  [ 0x4172, 'CjkUnifiedIdeograph4172', [], 'Display', None],
  [ 0x4173, 'CjkUnifiedIdeograph4173', [], 'Display', None],
  [ 0x4174, 'CjkUnifiedIdeograph4174', [], 'Display', None],
  [ 0x4175, 'CjkUnifiedIdeograph4175', [], 'Display', None],
  [ 0x4176, 'CjkUnifiedIdeograph4176', [], 'Display', None],
  [ 0x4177, 'CjkUnifiedIdeograph4177', [], 'Display', None],
  [ 0x4178, 'CjkUnifiedIdeograph4178', [], 'Display', None],
  [ 0x4179, 'CjkUnifiedIdeograph4179', [], 'Display', None],
  [ 0x417a, 'CjkUnifiedIdeograph417a', [], 'Display', None],
  [ 0x417b, 'CjkUnifiedIdeograph417b', [], 'Display', None],
  [ 0x417c, 'CjkUnifiedIdeograph417c', [], 'Display', None],
  [ 0x417d, 'CjkUnifiedIdeograph417d', [], 'Display', None],
  [ 0x417e, 'CjkUnifiedIdeograph417e', [], 'Display', None],
  [ 0x417f, 'CjkUnifiedIdeograph417f', [], 'Display', None],
  [ 0x4180, 'CjkUnifiedIdeograph4180', [], 'Display', None],
  [ 0x4181, 'CjkUnifiedIdeograph4181', [], 'Display', None],
  [ 0x4182, 'CjkUnifiedIdeograph4182', [], 'Display', None],
  [ 0x4183, 'CjkUnifiedIdeograph4183', [], 'Display', None],
  [ 0x4184, 'CjkUnifiedIdeograph4184', [], 'Display', None],
  [ 0x4185, 'CjkUnifiedIdeograph4185', [], 'Display', None],
  [ 0x4186, 'CjkUnifiedIdeograph4186', [], 'Display', None],
  [ 0x4187, 'CjkUnifiedIdeograph4187', [], 'Display', None],
  [ 0x4188, 'CjkUnifiedIdeograph4188', [], 'Display', None],
  [ 0x4189, 'CjkUnifiedIdeograph4189', [], 'Display', None],
  [ 0x418a, 'CjkUnifiedIdeograph418a', [], 'Display', None],
  [ 0x418b, 'CjkUnifiedIdeograph418b', [], 'Display', None],
  [ 0x418c, 'CjkUnifiedIdeograph418c', [], 'Display', None],
  [ 0x418d, 'CjkUnifiedIdeograph418d', [], 'Display', None],
  [ 0x418e, 'CjkUnifiedIdeograph418e', [], 'Display', None],
  [ 0x418f, 'CjkUnifiedIdeograph418f', [], 'Display', None],
  [ 0x4190, 'CjkUnifiedIdeograph4190', [], 'Display', None],
  [ 0x4191, 'CjkUnifiedIdeograph4191', [], 'Display', None],
  [ 0x4192, 'CjkUnifiedIdeograph4192', [], 'Display', None],
  [ 0x4193, 'CjkUnifiedIdeograph4193', [], 'Display', None],
  [ 0x4194, 'CjkUnifiedIdeograph4194', [], 'Display', None],
  [ 0x4195, 'CjkUnifiedIdeograph4195', [], 'Display', None],
  [ 0x4196, 'CjkUnifiedIdeograph4196', [], 'Display', None],
  [ 0x4197, 'CjkUnifiedIdeograph4197', [], 'Display', None],
  [ 0x4198, 'CjkUnifiedIdeograph4198', [], 'Display', None],
  [ 0x4199, 'CjkUnifiedIdeograph4199', [], 'Display', None],
  [ 0x419a, 'CjkUnifiedIdeograph419a', [], 'Display', None],
  [ 0x419b, 'CjkUnifiedIdeograph419b', [], 'Display', None],
  [ 0x419c, 'CjkUnifiedIdeograph419c', [], 'Display', None],
  [ 0x419d, 'CjkUnifiedIdeograph419d', [], 'Display', None],
  [ 0x419e, 'CjkUnifiedIdeograph419e', [], 'Display', None],
  [ 0x419f, 'CjkUnifiedIdeograph419f', [], 'Display', None],
  [ 0x41a0, 'CjkUnifiedIdeograph41a0', [], 'Display', None],
  [ 0x41a1, 'CjkUnifiedIdeograph41a1', [], 'Display', None],
  [ 0x41a2, 'CjkUnifiedIdeograph41a2', [], 'Display', None],
  [ 0x41a3, 'CjkUnifiedIdeograph41a3', [], 'Display', None],
  [ 0x41a4, 'CjkUnifiedIdeograph41a4', [], 'Display', None],
  [ 0x41a5, 'CjkUnifiedIdeograph41a5', [], 'Display', None],
  [ 0x41a6, 'CjkUnifiedIdeograph41a6', [], 'Display', None],
  [ 0x41a7, 'CjkUnifiedIdeograph41a7', [], 'Display', None],
  [ 0x41a8, 'CjkUnifiedIdeograph41a8', [], 'Display', None],
  [ 0x41a9, 'CjkUnifiedIdeograph41a9', [], 'Display', None],
  [ 0x41aa, 'CjkUnifiedIdeograph41aa', [], 'Display', None],
  [ 0x41ab, 'CjkUnifiedIdeograph41ab', [], 'Display', None],
  [ 0x41ac, 'CjkUnifiedIdeograph41ac', [], 'Display', None],
  [ 0x41ad, 'CjkUnifiedIdeograph41ad', [], 'Display', None],
  [ 0x41ae, 'CjkUnifiedIdeograph41ae', [], 'Display', None],
  [ 0x41af, 'CjkUnifiedIdeograph41af', [], 'Display', None],
  [ 0x41b0, 'CjkUnifiedIdeograph41b0', [], 'Display', None],
  [ 0x41b1, 'CjkUnifiedIdeograph41b1', [], 'Display', None],
  [ 0x41b2, 'CjkUnifiedIdeograph41b2', [], 'Display', None],
  [ 0x41b3, 'CjkUnifiedIdeograph41b3', [], 'Display', None],
  [ 0x41b4, 'CjkUnifiedIdeograph41b4', [], 'Display', None],
  [ 0x41b5, 'CjkUnifiedIdeograph41b5', [], 'Display', None],
  [ 0x41b6, 'CjkUnifiedIdeograph41b6', [], 'Display', None],
  [ 0x41b7, 'CjkUnifiedIdeograph41b7', [], 'Display', None],
  [ 0x41b8, 'CjkUnifiedIdeograph41b8', [], 'Display', None],
  [ 0x41b9, 'CjkUnifiedIdeograph41b9', [], 'Display', None],
  [ 0x41ba, 'CjkUnifiedIdeograph41ba', [], 'Display', None],
  [ 0x41bb, 'CjkUnifiedIdeograph41bb', [], 'Display', None],
  [ 0x41bc, 'CjkUnifiedIdeograph41bc', [], 'Display', None],
  [ 0x41bd, 'CjkUnifiedIdeograph41bd', [], 'Display', None],
  [ 0x41be, 'CjkUnifiedIdeograph41be', [], 'Display', None],
  [ 0x41bf, 'CjkUnifiedIdeograph41bf', [], 'Display', None],
  [ 0x41c0, 'CjkUnifiedIdeograph41c0', [], 'Display', None],
  [ 0x41c1, 'CjkUnifiedIdeograph41c1', [], 'Display', None],
  [ 0x41c2, 'CjkUnifiedIdeograph41c2', [], 'Display', None],
  [ 0x41c3, 'CjkUnifiedIdeograph41c3', [], 'Display', None],
  [ 0x41c4, 'CjkUnifiedIdeograph41c4', [], 'Display', None],
  [ 0x41c5, 'CjkUnifiedIdeograph41c5', [], 'Display', None],
  [ 0x41c6, 'CjkUnifiedIdeograph41c6', [], 'Display', None],
  [ 0x41c7, 'CjkUnifiedIdeograph41c7', [], 'Display', None],
  [ 0x41c8, 'CjkUnifiedIdeograph41c8', [], 'Display', None],
  [ 0x41c9, 'CjkUnifiedIdeograph41c9', [], 'Display', None],
  [ 0x41ca, 'CjkUnifiedIdeograph41ca', [], 'Display', None],
  [ 0x41cb, 'CjkUnifiedIdeograph41cb', [], 'Display', None],
  [ 0x41cc, 'CjkUnifiedIdeograph41cc', [], 'Display', None],
  [ 0x41cd, 'CjkUnifiedIdeograph41cd', [], 'Display', None],
  [ 0x41ce, 'CjkUnifiedIdeograph41ce', [], 'Display', None],
  [ 0x41cf, 'CjkUnifiedIdeograph41cf', [], 'Display', None],
  [ 0x41d0, 'CjkUnifiedIdeograph41d0', [], 'Display', None],
  [ 0x41d1, 'CjkUnifiedIdeograph41d1', [], 'Display', None],
  [ 0x41d2, 'CjkUnifiedIdeograph41d2', [], 'Display', None],
  [ 0x41d3, 'CjkUnifiedIdeograph41d3', [], 'Display', None],
  [ 0x41d4, 'CjkUnifiedIdeograph41d4', [], 'Display', None],
  [ 0x41d5, 'CjkUnifiedIdeograph41d5', [], 'Display', None],
  [ 0x41d6, 'CjkUnifiedIdeograph41d6', [], 'Display', None],
  [ 0x41d7, 'CjkUnifiedIdeograph41d7', [], 'Display', None],
  [ 0x41d8, 'CjkUnifiedIdeograph41d8', [], 'Display', None],
  [ 0x41d9, 'CjkUnifiedIdeograph41d9', [], 'Display', None],
  [ 0x41da, 'CjkUnifiedIdeograph41da', [], 'Display', None],
  [ 0x41db, 'CjkUnifiedIdeograph41db', [], 'Display', None],
  [ 0x41dc, 'CjkUnifiedIdeograph41dc', [], 'Display', None],
  [ 0x41dd, 'CjkUnifiedIdeograph41dd', [], 'Display', None],
  [ 0x41de, 'CjkUnifiedIdeograph41de', [], 'Display', None],
  [ 0x41df, 'CjkUnifiedIdeograph41df', [], 'Display', None],
  [ 0x41e0, 'CjkUnifiedIdeograph41e0', [], 'Display', None],
  [ 0x41e1, 'CjkUnifiedIdeograph41e1', [], 'Display', None],
  [ 0x41e2, 'CjkUnifiedIdeograph41e2', [], 'Display', None],
  [ 0x41e3, 'CjkUnifiedIdeograph41e3', [], 'Display', None],
  [ 0x41e4, 'CjkUnifiedIdeograph41e4', [], 'Display', None],
  [ 0x41e5, 'CjkUnifiedIdeograph41e5', [], 'Display', None],
  [ 0x41e6, 'CjkUnifiedIdeograph41e6', [], 'Display', None],
  [ 0x41e7, 'CjkUnifiedIdeograph41e7', [], 'Display', None],
  [ 0x41e8, 'CjkUnifiedIdeograph41e8', [], 'Display', None],
  [ 0x41e9, 'CjkUnifiedIdeograph41e9', [], 'Display', None],
  [ 0x41ea, 'CjkUnifiedIdeograph41ea', [], 'Display', None],
  [ 0x41eb, 'CjkUnifiedIdeograph41eb', [], 'Display', None],
  [ 0x41ec, 'CjkUnifiedIdeograph41ec', [], 'Display', None],
  [ 0x41ed, 'CjkUnifiedIdeograph41ed', [], 'Display', None],
  [ 0x41ee, 'CjkUnifiedIdeograph41ee', [], 'Display', None],
  [ 0x41ef, 'CjkUnifiedIdeograph41ef', [], 'Display', None],
  [ 0x41f0, 'CjkUnifiedIdeograph41f0', [], 'Display', None],
  [ 0x41f1, 'CjkUnifiedIdeograph41f1', [], 'Display', None],
  [ 0x41f2, 'CjkUnifiedIdeograph41f2', [], 'Display', None],
  [ 0x41f3, 'CjkUnifiedIdeograph41f3', [], 'Display', None],
  [ 0x41f4, 'CjkUnifiedIdeograph41f4', [], 'Display', None],
  [ 0x41f5, 'CjkUnifiedIdeograph41f5', [], 'Display', None],
  [ 0x41f6, 'CjkUnifiedIdeograph41f6', [], 'Display', None],
  [ 0x41f7, 'CjkUnifiedIdeograph41f7', [], 'Display', None],
  [ 0x41f8, 'CjkUnifiedIdeograph41f8', [], 'Display', None],
  [ 0x41f9, 'CjkUnifiedIdeograph41f9', [], 'Display', None],
  [ 0x41fa, 'CjkUnifiedIdeograph41fa', [], 'Display', None],
  [ 0x41fb, 'CjkUnifiedIdeograph41fb', [], 'Display', None],
  [ 0x41fc, 'CjkUnifiedIdeograph41fc', [], 'Display', None],
  [ 0x41fd, 'CjkUnifiedIdeograph41fd', [], 'Display', None],
  [ 0x41fe, 'CjkUnifiedIdeograph41fe', [], 'Display', None],
  [ 0x41ff, 'CjkUnifiedIdeograph41ff', [], 'Display', None],
  [ 0x4200, 'CjkUnifiedIdeograph4200', [], 'Display', None],
  [ 0x4201, 'CjkUnifiedIdeograph4201', [], 'Display', None],
  [ 0x4202, 'CjkUnifiedIdeograph4202', [], 'Display', None],
  [ 0x4203, 'CjkUnifiedIdeograph4203', [], 'Display', None],
  [ 0x4204, 'CjkUnifiedIdeograph4204', [], 'Display', None],
  [ 0x4205, 'CjkUnifiedIdeograph4205', [], 'Display', None],
  [ 0x4206, 'CjkUnifiedIdeograph4206', [], 'Display', None],
  [ 0x4207, 'CjkUnifiedIdeograph4207', [], 'Display', None],
  [ 0x4208, 'CjkUnifiedIdeograph4208', [], 'Display', None],
  [ 0x4209, 'CjkUnifiedIdeograph4209', [], 'Display', None],
  [ 0x420a, 'CjkUnifiedIdeograph420a', [], 'Display', None],
  [ 0x420b, 'CjkUnifiedIdeograph420b', [], 'Display', None],
  [ 0x420c, 'CjkUnifiedIdeograph420c', [], 'Display', None],
  [ 0x420d, 'CjkUnifiedIdeograph420d', [], 'Display', None],
  [ 0x420e, 'CjkUnifiedIdeograph420e', [], 'Display', None],
  [ 0x420f, 'CjkUnifiedIdeograph420f', [], 'Display', None],
  [ 0x4210, 'CjkUnifiedIdeograph4210', [], 'Display', None],
  [ 0x4211, 'CjkUnifiedIdeograph4211', [], 'Display', None],
  [ 0x4212, 'CjkUnifiedIdeograph4212', [], 'Display', None],
  [ 0x4213, 'CjkUnifiedIdeograph4213', [], 'Display', None],
  [ 0x4214, 'CjkUnifiedIdeograph4214', [], 'Display', None],
  [ 0x4215, 'CjkUnifiedIdeograph4215', [], 'Display', None],
  [ 0x4216, 'CjkUnifiedIdeograph4216', [], 'Display', None],
  [ 0x4217, 'CjkUnifiedIdeograph4217', [], 'Display', None],
  [ 0x4218, 'CjkUnifiedIdeograph4218', [], 'Display', None],
  [ 0x4219, 'CjkUnifiedIdeograph4219', [], 'Display', None],
  [ 0x421a, 'CjkUnifiedIdeograph421a', [], 'Display', None],
  [ 0x421b, 'CjkUnifiedIdeograph421b', [], 'Display', None],
  [ 0x421c, 'CjkUnifiedIdeograph421c', [], 'Display', None],
  [ 0x421d, 'CjkUnifiedIdeograph421d', [], 'Display', None],
  [ 0x421e, 'CjkUnifiedIdeograph421e', [], 'Display', None],
  [ 0x421f, 'CjkUnifiedIdeograph421f', [], 'Display', None],
  [ 0x4220, 'CjkUnifiedIdeograph4220', [], 'Display', None],
  [ 0x4221, 'CjkUnifiedIdeograph4221', [], 'Display', None],
  [ 0x4222, 'CjkUnifiedIdeograph4222', [], 'Display', None],
  [ 0x4223, 'CjkUnifiedIdeograph4223', [], 'Display', None],
  [ 0x4224, 'CjkUnifiedIdeograph4224', [], 'Display', None],
  [ 0x4225, 'CjkUnifiedIdeograph4225', [], 'Display', None],
  [ 0x4226, 'CjkUnifiedIdeograph4226', [], 'Display', None],
  [ 0x4227, 'CjkUnifiedIdeograph4227', [], 'Display', None],
  [ 0x4228, 'CjkUnifiedIdeograph4228', [], 'Display', None],
  [ 0x4229, 'CjkUnifiedIdeograph4229', [], 'Display', None],
  [ 0x422a, 'CjkUnifiedIdeograph422a', [], 'Display', None],
  [ 0x422b, 'CjkUnifiedIdeograph422b', [], 'Display', None],
  [ 0x422c, 'CjkUnifiedIdeograph422c', [], 'Display', None],
  [ 0x422d, 'CjkUnifiedIdeograph422d', [], 'Display', None],
  [ 0x422e, 'CjkUnifiedIdeograph422e', [], 'Display', None],
  [ 0x422f, 'CjkUnifiedIdeograph422f', [], 'Display', None],
  [ 0x4230, 'CjkUnifiedIdeograph4230', [], 'Display', None],
  [ 0x4231, 'CjkUnifiedIdeograph4231', [], 'Display', None],
  [ 0x4232, 'CjkUnifiedIdeograph4232', [], 'Display', None],
  [ 0x4233, 'CjkUnifiedIdeograph4233', [], 'Display', None],
  [ 0x4234, 'CjkUnifiedIdeograph4234', [], 'Display', None],
  [ 0x4235, 'CjkUnifiedIdeograph4235', [], 'Display', None],
  [ 0x4236, 'CjkUnifiedIdeograph4236', [], 'Display', None],
  [ 0x4237, 'CjkUnifiedIdeograph4237', [], 'Display', None],
  [ 0x4238, 'CjkUnifiedIdeograph4238', [], 'Display', None],
  [ 0x4239, 'CjkUnifiedIdeograph4239', [], 'Display', None],
  [ 0x423a, 'CjkUnifiedIdeograph423a', [], 'Display', None],
  [ 0x423b, 'CjkUnifiedIdeograph423b', [], 'Display', None],
  [ 0x423c, 'CjkUnifiedIdeograph423c', [], 'Display', None],
  [ 0x423d, 'CjkUnifiedIdeograph423d', [], 'Display', None],
  [ 0x423e, 'CjkUnifiedIdeograph423e', [], 'Display', None],
  [ 0x423f, 'CjkUnifiedIdeograph423f', [], 'Display', None],
  [ 0x4240, 'CjkUnifiedIdeograph4240', [], 'Display', None],
  [ 0x4241, 'CjkUnifiedIdeograph4241', [], 'Display', None],
  [ 0x4242, 'CjkUnifiedIdeograph4242', [], 'Display', None],
  [ 0x4243, 'CjkUnifiedIdeograph4243', [], 'Display', None],
  [ 0x4244, 'CjkUnifiedIdeograph4244', [], 'Display', None],
  [ 0x4245, 'CjkUnifiedIdeograph4245', [], 'Display', None],
  [ 0x4246, 'CjkUnifiedIdeograph4246', [], 'Display', None],
  [ 0x4247, 'CjkUnifiedIdeograph4247', [], 'Display', None],
  [ 0x4248, 'CjkUnifiedIdeograph4248', [], 'Display', None],
  [ 0x4249, 'CjkUnifiedIdeograph4249', [], 'Display', None],
  [ 0x424a, 'CjkUnifiedIdeograph424a', [], 'Display', None],
  [ 0x424b, 'CjkUnifiedIdeograph424b', [], 'Display', None],
  [ 0x424c, 'CjkUnifiedIdeograph424c', [], 'Display', None],
  [ 0x424d, 'CjkUnifiedIdeograph424d', [], 'Display', None],
  [ 0x424e, 'CjkUnifiedIdeograph424e', [], 'Display', None],
  [ 0x424f, 'CjkUnifiedIdeograph424f', [], 'Display', None],
  [ 0x4250, 'CjkUnifiedIdeograph4250', [], 'Display', None],
  [ 0x4251, 'CjkUnifiedIdeograph4251', [], 'Display', None],
  [ 0x4252, 'CjkUnifiedIdeograph4252', [], 'Display', None],
  [ 0x4253, 'CjkUnifiedIdeograph4253', [], 'Display', None],
  [ 0x4254, 'CjkUnifiedIdeograph4254', [], 'Display', None],
  [ 0x4255, 'CjkUnifiedIdeograph4255', [], 'Display', None],
  [ 0x4256, 'CjkUnifiedIdeograph4256', [], 'Display', None],
  [ 0x4257, 'CjkUnifiedIdeograph4257', [], 'Display', None],
  [ 0x4258, 'CjkUnifiedIdeograph4258', [], 'Display', None],
  [ 0x4259, 'CjkUnifiedIdeograph4259', [], 'Display', None],
  [ 0x425a, 'CjkUnifiedIdeograph425a', [], 'Display', None],
  [ 0x425b, 'CjkUnifiedIdeograph425b', [], 'Display', None],
  [ 0x425c, 'CjkUnifiedIdeograph425c', [], 'Display', None],
  [ 0x425d, 'CjkUnifiedIdeograph425d', [], 'Display', None],
  [ 0x425e, 'CjkUnifiedIdeograph425e', [], 'Display', None],
  [ 0x425f, 'CjkUnifiedIdeograph425f', [], 'Display', None],
  [ 0x4260, 'CjkUnifiedIdeograph4260', [], 'Display', None],
  [ 0x4261, 'CjkUnifiedIdeograph4261', [], 'Display', None],
  [ 0x4262, 'CjkUnifiedIdeograph4262', [], 'Display', None],
  [ 0x4263, 'CjkUnifiedIdeograph4263', [], 'Display', None],
  [ 0x4264, 'CjkUnifiedIdeograph4264', [], 'Display', None],
  [ 0x4265, 'CjkUnifiedIdeograph4265', [], 'Display', None],
  [ 0x4266, 'CjkUnifiedIdeograph4266', [], 'Display', None],
  [ 0x4267, 'CjkUnifiedIdeograph4267', [], 'Display', None],
  [ 0x4268, 'CjkUnifiedIdeograph4268', [], 'Display', None],
  [ 0x4269, 'CjkUnifiedIdeograph4269', [], 'Display', None],
  [ 0x426a, 'CjkUnifiedIdeograph426a', [], 'Display', None],
  [ 0x426b, 'CjkUnifiedIdeograph426b', [], 'Display', None],
  [ 0x426c, 'CjkUnifiedIdeograph426c', [], 'Display', None],
  [ 0x426d, 'CjkUnifiedIdeograph426d', [], 'Display', None],
  [ 0x426e, 'CjkUnifiedIdeograph426e', [], 'Display', None],
  [ 0x426f, 'CjkUnifiedIdeograph426f', [], 'Display', None],
  [ 0x4270, 'CjkUnifiedIdeograph4270', [], 'Display', None],
  [ 0x4271, 'CjkUnifiedIdeograph4271', [], 'Display', None],
  [ 0x4272, 'CjkUnifiedIdeograph4272', [], 'Display', None],
  [ 0x4273, 'CjkUnifiedIdeograph4273', [], 'Display', None],
  [ 0x4274, 'CjkUnifiedIdeograph4274', [], 'Display', None],
  [ 0x4275, 'CjkUnifiedIdeograph4275', [], 'Display', None],
  [ 0x4276, 'CjkUnifiedIdeograph4276', [], 'Display', None],
  [ 0x4277, 'CjkUnifiedIdeograph4277', [], 'Display', None],
  [ 0x4278, 'CjkUnifiedIdeograph4278', [], 'Display', None],
  [ 0x4279, 'CjkUnifiedIdeograph4279', [], 'Display', None],
  [ 0x427a, 'CjkUnifiedIdeograph427a', [], 'Display', None],
  [ 0x427b, 'CjkUnifiedIdeograph427b', [], 'Display', None],
  [ 0x427c, 'CjkUnifiedIdeograph427c', [], 'Display', None],
  [ 0x427d, 'CjkUnifiedIdeograph427d', [], 'Display', None],
  [ 0x427e, 'CjkUnifiedIdeograph427e', [], 'Display', None],
  [ 0x427f, 'CjkUnifiedIdeograph427f', [], 'Display', None],
  [ 0x4280, 'CjkUnifiedIdeograph4280', [], 'Display', None],
  [ 0x4281, 'CjkUnifiedIdeograph4281', [], 'Display', None],
  [ 0x4282, 'CjkUnifiedIdeograph4282', [], 'Display', None],
  [ 0x4283, 'CjkUnifiedIdeograph4283', [], 'Display', None],
  [ 0x4284, 'CjkUnifiedIdeograph4284', [], 'Display', None],
  [ 0x4285, 'CjkUnifiedIdeograph4285', [], 'Display', None],
  [ 0x4286, 'CjkUnifiedIdeograph4286', [], 'Display', None],
  [ 0x4287, 'CjkUnifiedIdeograph4287', [], 'Display', None],
  [ 0x4288, 'CjkUnifiedIdeograph4288', [], 'Display', None],
  [ 0x4289, 'CjkUnifiedIdeograph4289', [], 'Display', None],
  [ 0x428a, 'CjkUnifiedIdeograph428a', [], 'Display', None],
  [ 0x428b, 'CjkUnifiedIdeograph428b', [], 'Display', None],
  [ 0x428c, 'CjkUnifiedIdeograph428c', [], 'Display', None],
  [ 0x428d, 'CjkUnifiedIdeograph428d', [], 'Display', None],
  [ 0x428e, 'CjkUnifiedIdeograph428e', [], 'Display', None],
  [ 0x428f, 'CjkUnifiedIdeograph428f', [], 'Display', None],
  [ 0x4290, 'CjkUnifiedIdeograph4290', [], 'Display', None],
  [ 0x4291, 'CjkUnifiedIdeograph4291', [], 'Display', None],
  [ 0x4292, 'CjkUnifiedIdeograph4292', [], 'Display', None],
  [ 0x4293, 'CjkUnifiedIdeograph4293', [], 'Display', None],
  [ 0x4294, 'CjkUnifiedIdeograph4294', [], 'Display', None],
  [ 0x4295, 'CjkUnifiedIdeograph4295', [], 'Display', None],
  [ 0x4296, 'CjkUnifiedIdeograph4296', [], 'Display', None],
  [ 0x4297, 'CjkUnifiedIdeograph4297', [], 'Display', None],
  [ 0x4298, 'CjkUnifiedIdeograph4298', [], 'Display', None],
  [ 0x4299, 'CjkUnifiedIdeograph4299', [], 'Display', None],
  [ 0x429a, 'CjkUnifiedIdeograph429a', [], 'Display', None],
  [ 0x429b, 'CjkUnifiedIdeograph429b', [], 'Display', None],
  [ 0x429c, 'CjkUnifiedIdeograph429c', [], 'Display', None],
  [ 0x429d, 'CjkUnifiedIdeograph429d', [], 'Display', None],
  [ 0x429e, 'CjkUnifiedIdeograph429e', [], 'Display', None],
  [ 0x429f, 'CjkUnifiedIdeograph429f', [], 'Display', None],
  [ 0x42a0, 'CjkUnifiedIdeograph42a0', [], 'Display', None],
  [ 0x42a1, 'CjkUnifiedIdeograph42a1', [], 'Display', None],
  [ 0x42a2, 'CjkUnifiedIdeograph42a2', [], 'Display', None],
  [ 0x42a3, 'CjkUnifiedIdeograph42a3', [], 'Display', None],
  [ 0x42a4, 'CjkUnifiedIdeograph42a4', [], 'Display', None],
  [ 0x42a5, 'CjkUnifiedIdeograph42a5', [], 'Display', None],
  [ 0x42a6, 'CjkUnifiedIdeograph42a6', [], 'Display', None],
  [ 0x42a7, 'CjkUnifiedIdeograph42a7', [], 'Display', None],
  [ 0x42a8, 'CjkUnifiedIdeograph42a8', [], 'Display', None],
  [ 0x42a9, 'CjkUnifiedIdeograph42a9', [], 'Display', None],
  [ 0x42aa, 'CjkUnifiedIdeograph42aa', [], 'Display', None],
  [ 0x42ab, 'CjkUnifiedIdeograph42ab', [], 'Display', None],
  [ 0x42ac, 'CjkUnifiedIdeograph42ac', [], 'Display', None],
  [ 0x42ad, 'CjkUnifiedIdeograph42ad', [], 'Display', None],
  [ 0x42ae, 'CjkUnifiedIdeograph42ae', [], 'Display', None],
  [ 0x42af, 'CjkUnifiedIdeograph42af', [], 'Display', None],
  [ 0x42b0, 'CjkUnifiedIdeograph42b0', [], 'Display', None],
  [ 0x42b1, 'CjkUnifiedIdeograph42b1', [], 'Display', None],
  [ 0x42b2, 'CjkUnifiedIdeograph42b2', [], 'Display', None],
  [ 0x42b3, 'CjkUnifiedIdeograph42b3', [], 'Display', None],
  [ 0x42b4, 'CjkUnifiedIdeograph42b4', [], 'Display', None],
  [ 0x42b5, 'CjkUnifiedIdeograph42b5', [], 'Display', None],
  [ 0x42b6, 'CjkUnifiedIdeograph42b6', [], 'Display', None],
  [ 0x42b7, 'CjkUnifiedIdeograph42b7', [], 'Display', None],
  [ 0x42b8, 'CjkUnifiedIdeograph42b8', [], 'Display', None],
  [ 0x42b9, 'CjkUnifiedIdeograph42b9', [], 'Display', None],
  [ 0x42ba, 'CjkUnifiedIdeograph42ba', [], 'Display', None],
  [ 0x42bb, 'CjkUnifiedIdeograph42bb', [], 'Display', None],
  [ 0x42bc, 'CjkUnifiedIdeograph42bc', [], 'Display', None],
  [ 0x42bd, 'CjkUnifiedIdeograph42bd', [], 'Display', None],
  [ 0x42be, 'CjkUnifiedIdeograph42be', [], 'Display', None],
  [ 0x42bf, 'CjkUnifiedIdeograph42bf', [], 'Display', None],
  [ 0x42c0, 'CjkUnifiedIdeograph42c0', [], 'Display', None],
  [ 0x42c1, 'CjkUnifiedIdeograph42c1', [], 'Display', None],
  [ 0x42c2, 'CjkUnifiedIdeograph42c2', [], 'Display', None],
  [ 0x42c3, 'CjkUnifiedIdeograph42c3', [], 'Display', None],
  [ 0x42c4, 'CjkUnifiedIdeograph42c4', [], 'Display', None],
  [ 0x42c5, 'CjkUnifiedIdeograph42c5', [], 'Display', None],
  [ 0x42c6, 'CjkUnifiedIdeograph42c6', [], 'Display', None],
  [ 0x42c7, 'CjkUnifiedIdeograph42c7', [], 'Display', None],
  [ 0x42c8, 'CjkUnifiedIdeograph42c8', [], 'Display', None],
  [ 0x42c9, 'CjkUnifiedIdeograph42c9', [], 'Display', None],
  [ 0x42ca, 'CjkUnifiedIdeograph42ca', [], 'Display', None],
  [ 0x42cb, 'CjkUnifiedIdeograph42cb', [], 'Display', None],
  [ 0x42cc, 'CjkUnifiedIdeograph42cc', [], 'Display', None],
  [ 0x42cd, 'CjkUnifiedIdeograph42cd', [], 'Display', None],
  [ 0x42ce, 'CjkUnifiedIdeograph42ce', [], 'Display', None],
  [ 0x42cf, 'CjkUnifiedIdeograph42cf', [], 'Display', None],
  [ 0x42d0, 'CjkUnifiedIdeograph42d0', [], 'Display', None],
  [ 0x42d1, 'CjkUnifiedIdeograph42d1', [], 'Display', None],
  [ 0x42d2, 'CjkUnifiedIdeograph42d2', [], 'Display', None],
  [ 0x42d3, 'CjkUnifiedIdeograph42d3', [], 'Display', None],
  [ 0x42d4, 'CjkUnifiedIdeograph42d4', [], 'Display', None],
  [ 0x42d5, 'CjkUnifiedIdeograph42d5', [], 'Display', None],
  [ 0x42d6, 'CjkUnifiedIdeograph42d6', [], 'Display', None],
  [ 0x42d7, 'CjkUnifiedIdeograph42d7', [], 'Display', None],
  [ 0x42d8, 'CjkUnifiedIdeograph42d8', [], 'Display', None],
  [ 0x42d9, 'CjkUnifiedIdeograph42d9', [], 'Display', None],
  [ 0x42da, 'CjkUnifiedIdeograph42da', [], 'Display', None],
  [ 0x42db, 'CjkUnifiedIdeograph42db', [], 'Display', None],
  [ 0x42dc, 'CjkUnifiedIdeograph42dc', [], 'Display', None],
  [ 0x42dd, 'CjkUnifiedIdeograph42dd', [], 'Display', None],
  [ 0x42de, 'CjkUnifiedIdeograph42de', [], 'Display', None],
  [ 0x42df, 'CjkUnifiedIdeograph42df', [], 'Display', None],
  [ 0x42e0, 'CjkUnifiedIdeograph42e0', [], 'Display', None],
  [ 0x42e1, 'CjkUnifiedIdeograph42e1', [], 'Display', None],
  [ 0x42e2, 'CjkUnifiedIdeograph42e2', [], 'Display', None],
  [ 0x42e3, 'CjkUnifiedIdeograph42e3', [], 'Display', None],
  [ 0x42e4, 'CjkUnifiedIdeograph42e4', [], 'Display', None],
  [ 0x42e5, 'CjkUnifiedIdeograph42e5', [], 'Display', None],
  [ 0x42e6, 'CjkUnifiedIdeograph42e6', [], 'Display', None],
  [ 0x42e7, 'CjkUnifiedIdeograph42e7', [], 'Display', None],
  [ 0x42e8, 'CjkUnifiedIdeograph42e8', [], 'Display', None],
  [ 0x42e9, 'CjkUnifiedIdeograph42e9', [], 'Display', None],
  [ 0x42ea, 'CjkUnifiedIdeograph42ea', [], 'Display', None],
  [ 0x42eb, 'CjkUnifiedIdeograph42eb', [], 'Display', None],
  [ 0x42ec, 'CjkUnifiedIdeograph42ec', [], 'Display', None],
  [ 0x42ed, 'CjkUnifiedIdeograph42ed', [], 'Display', None],
  [ 0x42ee, 'CjkUnifiedIdeograph42ee', [], 'Display', None],
  [ 0x42ef, 'CjkUnifiedIdeograph42ef', [], 'Display', None],
  [ 0x42f0, 'CjkUnifiedIdeograph42f0', [], 'Display', None],
  [ 0x42f1, 'CjkUnifiedIdeograph42f1', [], 'Display', None],
  [ 0x42f2, 'CjkUnifiedIdeograph42f2', [], 'Display', None],
  [ 0x42f3, 'CjkUnifiedIdeograph42f3', [], 'Display', None],
  [ 0x42f4, 'CjkUnifiedIdeograph42f4', [], 'Display', None],
  [ 0x42f5, 'CjkUnifiedIdeograph42f5', [], 'Display', None],
  [ 0x42f6, 'CjkUnifiedIdeograph42f6', [], 'Display', None],
  [ 0x42f7, 'CjkUnifiedIdeograph42f7', [], 'Display', None],
  [ 0x42f8, 'CjkUnifiedIdeograph42f8', [], 'Display', None],
  [ 0x42f9, 'CjkUnifiedIdeograph42f9', [], 'Display', None],
  [ 0x42fa, 'CjkUnifiedIdeograph42fa', [], 'Display', None],
  [ 0x42fb, 'CjkUnifiedIdeograph42fb', [], 'Display', None],
  [ 0x42fc, 'CjkUnifiedIdeograph42fc', [], 'Display', None],
  [ 0x42fd, 'CjkUnifiedIdeograph42fd', [], 'Display', None],
  [ 0x42fe, 'CjkUnifiedIdeograph42fe', [], 'Display', None],
  [ 0x42ff, 'CjkUnifiedIdeograph42ff', [], 'Display', None],
  [ 0x4300, 'CjkUnifiedIdeograph4300', [], 'Display', None],
  [ 0x4301, 'CjkUnifiedIdeograph4301', [], 'Display', None],
  [ 0x4302, 'CjkUnifiedIdeograph4302', [], 'Display', None],
  [ 0x4303, 'CjkUnifiedIdeograph4303', [], 'Display', None],
  [ 0x4304, 'CjkUnifiedIdeograph4304', [], 'Display', None],
  [ 0x4305, 'CjkUnifiedIdeograph4305', [], 'Display', None],
  [ 0x4306, 'CjkUnifiedIdeograph4306', [], 'Display', None],
  [ 0x4307, 'CjkUnifiedIdeograph4307', [], 'Display', None],
  [ 0x4308, 'CjkUnifiedIdeograph4308', [], 'Display', None],
  [ 0x4309, 'CjkUnifiedIdeograph4309', [], 'Display', None],
  [ 0x430a, 'CjkUnifiedIdeograph430a', [], 'Display', None],
  [ 0x430b, 'CjkUnifiedIdeograph430b', [], 'Display', None],
  [ 0x430c, 'CjkUnifiedIdeograph430c', [], 'Display', None],
  [ 0x430d, 'CjkUnifiedIdeograph430d', [], 'Display', None],
  [ 0x430e, 'CjkUnifiedIdeograph430e', [], 'Display', None],
  [ 0x430f, 'CjkUnifiedIdeograph430f', [], 'Display', None],
  [ 0x4310, 'CjkUnifiedIdeograph4310', [], 'Display', None],
  [ 0x4311, 'CjkUnifiedIdeograph4311', [], 'Display', None],
  [ 0x4312, 'CjkUnifiedIdeograph4312', [], 'Display', None],
  [ 0x4313, 'CjkUnifiedIdeograph4313', [], 'Display', None],
  [ 0x4314, 'CjkUnifiedIdeograph4314', [], 'Display', None],
  [ 0x4315, 'CjkUnifiedIdeograph4315', [], 'Display', None],
  [ 0x4316, 'CjkUnifiedIdeograph4316', [], 'Display', None],
  [ 0x4317, 'CjkUnifiedIdeograph4317', [], 'Display', None],
  [ 0x4318, 'CjkUnifiedIdeograph4318', [], 'Display', None],
  [ 0x4319, 'CjkUnifiedIdeograph4319', [], 'Display', None],
  [ 0x431a, 'CjkUnifiedIdeograph431a', [], 'Display', None],
  [ 0x431b, 'CjkUnifiedIdeograph431b', [], 'Display', None],
  [ 0x431c, 'CjkUnifiedIdeograph431c', [], 'Display', None],
  [ 0x431d, 'CjkUnifiedIdeograph431d', [], 'Display', None],
  [ 0x431e, 'CjkUnifiedIdeograph431e', [], 'Display', None],
  [ 0x431f, 'CjkUnifiedIdeograph431f', [], 'Display', None],
  [ 0x4320, 'CjkUnifiedIdeograph4320', [], 'Display', None],
  [ 0x4321, 'CjkUnifiedIdeograph4321', [], 'Display', None],
  [ 0x4322, 'CjkUnifiedIdeograph4322', [], 'Display', None],
  [ 0x4323, 'CjkUnifiedIdeograph4323', [], 'Display', None],
  [ 0x4324, 'CjkUnifiedIdeograph4324', [], 'Display', None],
  [ 0x4325, 'CjkUnifiedIdeograph4325', [], 'Display', None],
  [ 0x4326, 'CjkUnifiedIdeograph4326', [], 'Display', None],
  [ 0x4327, 'CjkUnifiedIdeograph4327', [], 'Display', None],
  [ 0x4328, 'CjkUnifiedIdeograph4328', [], 'Display', None],
  [ 0x4329, 'CjkUnifiedIdeograph4329', [], 'Display', None],
  [ 0x432a, 'CjkUnifiedIdeograph432a', [], 'Display', None],
  [ 0x432b, 'CjkUnifiedIdeograph432b', [], 'Display', None],
  [ 0x432c, 'CjkUnifiedIdeograph432c', [], 'Display', None],
  [ 0x432d, 'CjkUnifiedIdeograph432d', [], 'Display', None],
  [ 0x432e, 'CjkUnifiedIdeograph432e', [], 'Display', None],
  [ 0x432f, 'CjkUnifiedIdeograph432f', [], 'Display', None],
  [ 0x4330, 'CjkUnifiedIdeograph4330', [], 'Display', None],
  [ 0x4331, 'CjkUnifiedIdeograph4331', [], 'Display', None],
  [ 0x4332, 'CjkUnifiedIdeograph4332', [], 'Display', None],
  [ 0x4333, 'CjkUnifiedIdeograph4333', [], 'Display', None],
  [ 0x4334, 'CjkUnifiedIdeograph4334', [], 'Display', None],
  [ 0x4335, 'CjkUnifiedIdeograph4335', [], 'Display', None],
  [ 0x4336, 'CjkUnifiedIdeograph4336', [], 'Display', None],
  [ 0x4337, 'CjkUnifiedIdeograph4337', [], 'Display', None],
  [ 0x4338, 'CjkUnifiedIdeograph4338', [], 'Display', None],
  [ 0x4339, 'CjkUnifiedIdeograph4339', [], 'Display', None],
  [ 0x433a, 'CjkUnifiedIdeograph433a', [], 'Display', None],
  [ 0x433b, 'CjkUnifiedIdeograph433b', [], 'Display', None],
  [ 0x433c, 'CjkUnifiedIdeograph433c', [], 'Display', None],
  [ 0x433d, 'CjkUnifiedIdeograph433d', [], 'Display', None],
  [ 0x433e, 'CjkUnifiedIdeograph433e', [], 'Display', None],
  [ 0x433f, 'CjkUnifiedIdeograph433f', [], 'Display', None],
  [ 0x4340, 'CjkUnifiedIdeograph4340', [], 'Display', None],
  [ 0x4341, 'CjkUnifiedIdeograph4341', [], 'Display', None],
  [ 0x4342, 'CjkUnifiedIdeograph4342', [], 'Display', None],
  [ 0x4343, 'CjkUnifiedIdeograph4343', [], 'Display', None],
  [ 0x4344, 'CjkUnifiedIdeograph4344', [], 'Display', None],
  [ 0x4345, 'CjkUnifiedIdeograph4345', [], 'Display', None],
  [ 0x4346, 'CjkUnifiedIdeograph4346', [], 'Display', None],
  [ 0x4347, 'CjkUnifiedIdeograph4347', [], 'Display', None],
  [ 0x4348, 'CjkUnifiedIdeograph4348', [], 'Display', None],
  [ 0x4349, 'CjkUnifiedIdeograph4349', [], 'Display', None],
  [ 0x434a, 'CjkUnifiedIdeograph434a', [], 'Display', None],
  [ 0x434b, 'CjkUnifiedIdeograph434b', [], 'Display', None],
  [ 0x434c, 'CjkUnifiedIdeograph434c', [], 'Display', None],
  [ 0x434d, 'CjkUnifiedIdeograph434d', [], 'Display', None],
  [ 0x434e, 'CjkUnifiedIdeograph434e', [], 'Display', None],
  [ 0x434f, 'CjkUnifiedIdeograph434f', [], 'Display', None],
  [ 0x4350, 'CjkUnifiedIdeograph4350', [], 'Display', None],
  [ 0x4351, 'CjkUnifiedIdeograph4351', [], 'Display', None],
  [ 0x4352, 'CjkUnifiedIdeograph4352', [], 'Display', None],
  [ 0x4353, 'CjkUnifiedIdeograph4353', [], 'Display', None],
  [ 0x4354, 'CjkUnifiedIdeograph4354', [], 'Display', None],
  [ 0x4355, 'CjkUnifiedIdeograph4355', [], 'Display', None],
  [ 0x4356, 'CjkUnifiedIdeograph4356', [], 'Display', None],
  [ 0x4357, 'CjkUnifiedIdeograph4357', [], 'Display', None],
  [ 0x4358, 'CjkUnifiedIdeograph4358', [], 'Display', None],
  [ 0x4359, 'CjkUnifiedIdeograph4359', [], 'Display', None],
  [ 0x435a, 'CjkUnifiedIdeograph435a', [], 'Display', None],
  [ 0x435b, 'CjkUnifiedIdeograph435b', [], 'Display', None],
  [ 0x435c, 'CjkUnifiedIdeograph435c', [], 'Display', None],
  [ 0x435d, 'CjkUnifiedIdeograph435d', [], 'Display', None],
  [ 0x435e, 'CjkUnifiedIdeograph435e', [], 'Display', None],
  [ 0x435f, 'CjkUnifiedIdeograph435f', [], 'Display', None],
  [ 0x4360, 'CjkUnifiedIdeograph4360', [], 'Display', None],
  [ 0x4361, 'CjkUnifiedIdeograph4361', [], 'Display', None],
  [ 0x4362, 'CjkUnifiedIdeograph4362', [], 'Display', None],
  [ 0x4363, 'CjkUnifiedIdeograph4363', [], 'Display', None],
  [ 0x4364, 'CjkUnifiedIdeograph4364', [], 'Display', None],
  [ 0x4365, 'CjkUnifiedIdeograph4365', [], 'Display', None],
  [ 0x4366, 'CjkUnifiedIdeograph4366', [], 'Display', None],
  [ 0x4367, 'CjkUnifiedIdeograph4367', [], 'Display', None],
  [ 0x4368, 'CjkUnifiedIdeograph4368', [], 'Display', None],
  [ 0x4369, 'CjkUnifiedIdeograph4369', [], 'Display', None],
  [ 0x436a, 'CjkUnifiedIdeograph436a', [], 'Display', None],
  [ 0x436b, 'CjkUnifiedIdeograph436b', [], 'Display', None],
  [ 0x436c, 'CjkUnifiedIdeograph436c', [], 'Display', None],
  [ 0x436d, 'CjkUnifiedIdeograph436d', [], 'Display', None],
  [ 0x436e, 'CjkUnifiedIdeograph436e', [], 'Display', None],
  [ 0x436f, 'CjkUnifiedIdeograph436f', [], 'Display', None],
  [ 0x4370, 'CjkUnifiedIdeograph4370', [], 'Display', None],
  [ 0x4371, 'CjkUnifiedIdeograph4371', [], 'Display', None],
  [ 0x4372, 'CjkUnifiedIdeograph4372', [], 'Display', None],
  [ 0x4373, 'CjkUnifiedIdeograph4373', [], 'Display', None],
  [ 0x4374, 'CjkUnifiedIdeograph4374', [], 'Display', None],
  [ 0x4375, 'CjkUnifiedIdeograph4375', [], 'Display', None],
  [ 0x4376, 'CjkUnifiedIdeograph4376', [], 'Display', None],
  [ 0x4377, 'CjkUnifiedIdeograph4377', [], 'Display', None],
  [ 0x4378, 'CjkUnifiedIdeograph4378', [], 'Display', None],
  [ 0x4379, 'CjkUnifiedIdeograph4379', [], 'Display', None],
  [ 0x437a, 'CjkUnifiedIdeograph437a', [], 'Display', None],
  [ 0x437b, 'CjkUnifiedIdeograph437b', [], 'Display', None],
  [ 0x437c, 'CjkUnifiedIdeograph437c', [], 'Display', None],
  [ 0x437d, 'CjkUnifiedIdeograph437d', [], 'Display', None],
  [ 0x437e, 'CjkUnifiedIdeograph437e', [], 'Display', None],
  [ 0x437f, 'CjkUnifiedIdeograph437f', [], 'Display', None],
  [ 0x4380, 'CjkUnifiedIdeograph4380', [], 'Display', None],
  [ 0x4381, 'CjkUnifiedIdeograph4381', [], 'Display', None],
  [ 0x4382, 'CjkUnifiedIdeograph4382', [], 'Display', None],
  [ 0x4383, 'CjkUnifiedIdeograph4383', [], 'Display', None],
  [ 0x4384, 'CjkUnifiedIdeograph4384', [], 'Display', None],
  [ 0x4385, 'CjkUnifiedIdeograph4385', [], 'Display', None],
  [ 0x4386, 'CjkUnifiedIdeograph4386', [], 'Display', None],
  [ 0x4387, 'CjkUnifiedIdeograph4387', [], 'Display', None],
  [ 0x4388, 'CjkUnifiedIdeograph4388', [], 'Display', None],
  [ 0x4389, 'CjkUnifiedIdeograph4389', [], 'Display', None],
  [ 0x438a, 'CjkUnifiedIdeograph438a', [], 'Display', None],
  [ 0x438b, 'CjkUnifiedIdeograph438b', [], 'Display', None],
  [ 0x438c, 'CjkUnifiedIdeograph438c', [], 'Display', None],
  [ 0x438d, 'CjkUnifiedIdeograph438d', [], 'Display', None],
  [ 0x438e, 'CjkUnifiedIdeograph438e', [], 'Display', None],
  [ 0x438f, 'CjkUnifiedIdeograph438f', [], 'Display', None],
  [ 0x4390, 'CjkUnifiedIdeograph4390', [], 'Display', None],
  [ 0x4391, 'CjkUnifiedIdeograph4391', [], 'Display', None],
  [ 0x4392, 'CjkUnifiedIdeograph4392', [], 'Display', None],
  [ 0x4393, 'CjkUnifiedIdeograph4393', [], 'Display', None],
  [ 0x4394, 'CjkUnifiedIdeograph4394', [], 'Display', None],
  [ 0x4395, 'CjkUnifiedIdeograph4395', [], 'Display', None],
  [ 0x4396, 'CjkUnifiedIdeograph4396', [], 'Display', None],
  [ 0x4397, 'CjkUnifiedIdeograph4397', [], 'Display', None],
  [ 0x4398, 'CjkUnifiedIdeograph4398', [], 'Display', None],
  [ 0x4399, 'CjkUnifiedIdeograph4399', [], 'Display', None],
  [ 0x439a, 'CjkUnifiedIdeograph439a', [], 'Display', None],
  [ 0x439b, 'CjkUnifiedIdeograph439b', [], 'Display', None],
  [ 0x439c, 'CjkUnifiedIdeograph439c', [], 'Display', None],
  [ 0x439d, 'CjkUnifiedIdeograph439d', [], 'Display', None],
  [ 0x439e, 'CjkUnifiedIdeograph439e', [], 'Display', None],
  [ 0x439f, 'CjkUnifiedIdeograph439f', [], 'Display', None],
  [ 0x43a0, 'CjkUnifiedIdeograph43a0', [], 'Display', None],
  [ 0x43a1, 'CjkUnifiedIdeograph43a1', [], 'Display', None],
  [ 0x43a2, 'CjkUnifiedIdeograph43a2', [], 'Display', None],
  [ 0x43a3, 'CjkUnifiedIdeograph43a3', [], 'Display', None],
  [ 0x43a4, 'CjkUnifiedIdeograph43a4', [], 'Display', None],
  [ 0x43a5, 'CjkUnifiedIdeograph43a5', [], 'Display', None],
  [ 0x43a6, 'CjkUnifiedIdeograph43a6', [], 'Display', None],
  [ 0x43a7, 'CjkUnifiedIdeograph43a7', [], 'Display', None],
  [ 0x43a8, 'CjkUnifiedIdeograph43a8', [], 'Display', None],
  [ 0x43a9, 'CjkUnifiedIdeograph43a9', [], 'Display', None],
  [ 0x43aa, 'CjkUnifiedIdeograph43aa', [], 'Display', None],
  [ 0x43ab, 'CjkUnifiedIdeograph43ab', [], 'Display', None],
  [ 0x43ac, 'CjkUnifiedIdeograph43ac', [], 'Display', None],
  [ 0x43ad, 'CjkUnifiedIdeograph43ad', [], 'Display', None],
  [ 0x43ae, 'CjkUnifiedIdeograph43ae', [], 'Display', None],
  [ 0x43af, 'CjkUnifiedIdeograph43af', [], 'Display', None],
  [ 0x43b0, 'CjkUnifiedIdeograph43b0', [], 'Display', None],
  [ 0x43b1, 'CjkUnifiedIdeograph43b1', [], 'Display', None],
  [ 0x43b2, 'CjkUnifiedIdeograph43b2', [], 'Display', None],
  [ 0x43b3, 'CjkUnifiedIdeograph43b3', [], 'Display', None],
  [ 0x43b4, 'CjkUnifiedIdeograph43b4', [], 'Display', None],
  [ 0x43b5, 'CjkUnifiedIdeograph43b5', [], 'Display', None],
  [ 0x43b6, 'CjkUnifiedIdeograph43b6', [], 'Display', None],
  [ 0x43b7, 'CjkUnifiedIdeograph43b7', [], 'Display', None],
  [ 0x43b8, 'CjkUnifiedIdeograph43b8', [], 'Display', None],
  [ 0x43b9, 'CjkUnifiedIdeograph43b9', [], 'Display', None],
  [ 0x43ba, 'CjkUnifiedIdeograph43ba', [], 'Display', None],
  [ 0x43bb, 'CjkUnifiedIdeograph43bb', [], 'Display', None],
  [ 0x43bc, 'CjkUnifiedIdeograph43bc', [], 'Display', None],
  [ 0x43bd, 'CjkUnifiedIdeograph43bd', [], 'Display', None],
  [ 0x43be, 'CjkUnifiedIdeograph43be', [], 'Display', None],
  [ 0x43bf, 'CjkUnifiedIdeograph43bf', [], 'Display', None],
  [ 0x43c0, 'CjkUnifiedIdeograph43c0', [], 'Display', None],
  [ 0x43c1, 'CjkUnifiedIdeograph43c1', [], 'Display', None],
  [ 0x43c2, 'CjkUnifiedIdeograph43c2', [], 'Display', None],
  [ 0x43c3, 'CjkUnifiedIdeograph43c3', [], 'Display', None],
  [ 0x43c4, 'CjkUnifiedIdeograph43c4', [], 'Display', None],
  [ 0x43c5, 'CjkUnifiedIdeograph43c5', [], 'Display', None],
  [ 0x43c6, 'CjkUnifiedIdeograph43c6', [], 'Display', None],
  [ 0x43c7, 'CjkUnifiedIdeograph43c7', [], 'Display', None],
  [ 0x43c8, 'CjkUnifiedIdeograph43c8', [], 'Display', None],
  [ 0x43c9, 'CjkUnifiedIdeograph43c9', [], 'Display', None],
  [ 0x43ca, 'CjkUnifiedIdeograph43ca', [], 'Display', None],
  [ 0x43cb, 'CjkUnifiedIdeograph43cb', [], 'Display', None],
  [ 0x43cc, 'CjkUnifiedIdeograph43cc', [], 'Display', None],
  [ 0x43cd, 'CjkUnifiedIdeograph43cd', [], 'Display', None],
  [ 0x43ce, 'CjkUnifiedIdeograph43ce', [], 'Display', None],
  [ 0x43cf, 'CjkUnifiedIdeograph43cf', [], 'Display', None],
  [ 0x43d0, 'CjkUnifiedIdeograph43d0', [], 'Display', None],
  [ 0x43d1, 'CjkUnifiedIdeograph43d1', [], 'Display', None],
  [ 0x43d2, 'CjkUnifiedIdeograph43d2', [], 'Display', None],
  [ 0x43d3, 'CjkUnifiedIdeograph43d3', [], 'Display', None],
  [ 0x43d4, 'CjkUnifiedIdeograph43d4', [], 'Display', None],
  [ 0x43d5, 'CjkUnifiedIdeograph43d5', [], 'Display', None],
  [ 0x43d6, 'CjkUnifiedIdeograph43d6', [], 'Display', None],
  [ 0x43d7, 'CjkUnifiedIdeograph43d7', [], 'Display', None],
  [ 0x43d8, 'CjkUnifiedIdeograph43d8', [], 'Display', None],
  [ 0x43d9, 'CjkUnifiedIdeograph43d9', [], 'Display', None],
  [ 0x43da, 'CjkUnifiedIdeograph43da', [], 'Display', None],
  [ 0x43db, 'CjkUnifiedIdeograph43db', [], 'Display', None],
  [ 0x43dc, 'CjkUnifiedIdeograph43dc', [], 'Display', None],
  [ 0x43dd, 'CjkUnifiedIdeograph43dd', [], 'Display', None],
  [ 0x43de, 'CjkUnifiedIdeograph43de', [], 'Display', None],
  [ 0x43df, 'CjkUnifiedIdeograph43df', [], 'Display', None],
  [ 0x43e0, 'CjkUnifiedIdeograph43e0', [], 'Display', None],
  [ 0x43e1, 'CjkUnifiedIdeograph43e1', [], 'Display', None],
  [ 0x43e2, 'CjkUnifiedIdeograph43e2', [], 'Display', None],
  [ 0x43e3, 'CjkUnifiedIdeograph43e3', [], 'Display', None],
  [ 0x43e4, 'CjkUnifiedIdeograph43e4', [], 'Display', None],
  [ 0x43e5, 'CjkUnifiedIdeograph43e5', [], 'Display', None],
  [ 0x43e6, 'CjkUnifiedIdeograph43e6', [], 'Display', None],
  [ 0x43e7, 'CjkUnifiedIdeograph43e7', [], 'Display', None],
  [ 0x43e8, 'CjkUnifiedIdeograph43e8', [], 'Display', None],
  [ 0x43e9, 'CjkUnifiedIdeograph43e9', [], 'Display', None],
  [ 0x43ea, 'CjkUnifiedIdeograph43ea', [], 'Display', None],
  [ 0x43eb, 'CjkUnifiedIdeograph43eb', [], 'Display', None],
  [ 0x43ec, 'CjkUnifiedIdeograph43ec', [], 'Display', None],
  [ 0x43ed, 'CjkUnifiedIdeograph43ed', [], 'Display', None],
  [ 0x43ee, 'CjkUnifiedIdeograph43ee', [], 'Display', None],
  [ 0x43ef, 'CjkUnifiedIdeograph43ef', [], 'Display', None],
  [ 0x43f0, 'CjkUnifiedIdeograph43f0', [], 'Display', None],
  [ 0x43f1, 'CjkUnifiedIdeograph43f1', [], 'Display', None],
  [ 0x43f2, 'CjkUnifiedIdeograph43f2', [], 'Display', None],
  [ 0x43f3, 'CjkUnifiedIdeograph43f3', [], 'Display', None],
  [ 0x43f4, 'CjkUnifiedIdeograph43f4', [], 'Display', None],
  [ 0x43f5, 'CjkUnifiedIdeograph43f5', [], 'Display', None],
  [ 0x43f6, 'CjkUnifiedIdeograph43f6', [], 'Display', None],
  [ 0x43f7, 'CjkUnifiedIdeograph43f7', [], 'Display', None],
  [ 0x43f8, 'CjkUnifiedIdeograph43f8', [], 'Display', None],
  [ 0x43f9, 'CjkUnifiedIdeograph43f9', [], 'Display', None],
  [ 0x43fa, 'CjkUnifiedIdeograph43fa', [], 'Display', None],
  [ 0x43fb, 'CjkUnifiedIdeograph43fb', [], 'Display', None],
  [ 0x43fc, 'CjkUnifiedIdeograph43fc', [], 'Display', None],
  [ 0x43fd, 'CjkUnifiedIdeograph43fd', [], 'Display', None],
  [ 0x43fe, 'CjkUnifiedIdeograph43fe', [], 'Display', None],
  [ 0x43ff, 'CjkUnifiedIdeograph43ff', [], 'Display', None],
  [ 0x4400, 'CjkUnifiedIdeograph4400', [], 'Display', None],
  [ 0x4401, 'CjkUnifiedIdeograph4401', [], 'Display', None],
  [ 0x4402, 'CjkUnifiedIdeograph4402', [], 'Display', None],
  [ 0x4403, 'CjkUnifiedIdeograph4403', [], 'Display', None],
  [ 0x4404, 'CjkUnifiedIdeograph4404', [], 'Display', None],
  [ 0x4405, 'CjkUnifiedIdeograph4405', [], 'Display', None],
  [ 0x4406, 'CjkUnifiedIdeograph4406', [], 'Display', None],
  [ 0x4407, 'CjkUnifiedIdeograph4407', [], 'Display', None],
  [ 0x4408, 'CjkUnifiedIdeograph4408', [], 'Display', None],
  [ 0x4409, 'CjkUnifiedIdeograph4409', [], 'Display', None],
  [ 0x440a, 'CjkUnifiedIdeograph440a', [], 'Display', None],
  [ 0x440b, 'CjkUnifiedIdeograph440b', [], 'Display', None],
  [ 0x440c, 'CjkUnifiedIdeograph440c', [], 'Display', None],
  [ 0x440d, 'CjkUnifiedIdeograph440d', [], 'Display', None],
  [ 0x440e, 'CjkUnifiedIdeograph440e', [], 'Display', None],
  [ 0x440f, 'CjkUnifiedIdeograph440f', [], 'Display', None],
  [ 0x4410, 'CjkUnifiedIdeograph4410', [], 'Display', None],
  [ 0x4411, 'CjkUnifiedIdeograph4411', [], 'Display', None],
  [ 0x4412, 'CjkUnifiedIdeograph4412', [], 'Display', None],
  [ 0x4413, 'CjkUnifiedIdeograph4413', [], 'Display', None],
  [ 0x4414, 'CjkUnifiedIdeograph4414', [], 'Display', None],
  [ 0x4415, 'CjkUnifiedIdeograph4415', [], 'Display', None],
  [ 0x4416, 'CjkUnifiedIdeograph4416', [], 'Display', None],
  [ 0x4417, 'CjkUnifiedIdeograph4417', [], 'Display', None],
  [ 0x4418, 'CjkUnifiedIdeograph4418', [], 'Display', None],
  [ 0x4419, 'CjkUnifiedIdeograph4419', [], 'Display', None],
  [ 0x441a, 'CjkUnifiedIdeograph441a', [], 'Display', None],
  [ 0x441b, 'CjkUnifiedIdeograph441b', [], 'Display', None],
  [ 0x441c, 'CjkUnifiedIdeograph441c', [], 'Display', None],
  [ 0x441d, 'CjkUnifiedIdeograph441d', [], 'Display', None],
  [ 0x441e, 'CjkUnifiedIdeograph441e', [], 'Display', None],
  [ 0x441f, 'CjkUnifiedIdeograph441f', [], 'Display', None],
  [ 0x4420, 'CjkUnifiedIdeograph4420', [], 'Display', None],
  [ 0x4421, 'CjkUnifiedIdeograph4421', [], 'Display', None],
  [ 0x4422, 'CjkUnifiedIdeograph4422', [], 'Display', None],
  [ 0x4423, 'CjkUnifiedIdeograph4423', [], 'Display', None],
  [ 0x4424, 'CjkUnifiedIdeograph4424', [], 'Display', None],
  [ 0x4425, 'CjkUnifiedIdeograph4425', [], 'Display', None],
  [ 0x4426, 'CjkUnifiedIdeograph4426', [], 'Display', None],
  [ 0x4427, 'CjkUnifiedIdeograph4427', [], 'Display', None],
  [ 0x4428, 'CjkUnifiedIdeograph4428', [], 'Display', None],
  [ 0x4429, 'CjkUnifiedIdeograph4429', [], 'Display', None],
  [ 0x442a, 'CjkUnifiedIdeograph442a', [], 'Display', None],
  [ 0x442b, 'CjkUnifiedIdeograph442b', [], 'Display', None],
  [ 0x442c, 'CjkUnifiedIdeograph442c', [], 'Display', None],
  [ 0x442d, 'CjkUnifiedIdeograph442d', [], 'Display', None],
  [ 0x442e, 'CjkUnifiedIdeograph442e', [], 'Display', None],
  [ 0x442f, 'CjkUnifiedIdeograph442f', [], 'Display', None],
  [ 0x4430, 'CjkUnifiedIdeograph4430', [], 'Display', None],
  [ 0x4431, 'CjkUnifiedIdeograph4431', [], 'Display', None],
  [ 0x4432, 'CjkUnifiedIdeograph4432', [], 'Display', None],
  [ 0x4433, 'CjkUnifiedIdeograph4433', [], 'Display', None],
  [ 0x4434, 'CjkUnifiedIdeograph4434', [], 'Display', None],
  [ 0x4435, 'CjkUnifiedIdeograph4435', [], 'Display', None],
  [ 0x4436, 'CjkUnifiedIdeograph4436', [], 'Display', None],
  [ 0x4437, 'CjkUnifiedIdeograph4437', [], 'Display', None],
  [ 0x4438, 'CjkUnifiedIdeograph4438', [], 'Display', None],
  [ 0x4439, 'CjkUnifiedIdeograph4439', [], 'Display', None],
  [ 0x443a, 'CjkUnifiedIdeograph443a', [], 'Display', None],
  [ 0x443b, 'CjkUnifiedIdeograph443b', [], 'Display', None],
  [ 0x443c, 'CjkUnifiedIdeograph443c', [], 'Display', None],
  [ 0x443d, 'CjkUnifiedIdeograph443d', [], 'Display', None],
  [ 0x443e, 'CjkUnifiedIdeograph443e', [], 'Display', None],
  [ 0x443f, 'CjkUnifiedIdeograph443f', [], 'Display', None],
  [ 0x4440, 'CjkUnifiedIdeograph4440', [], 'Display', None],
  [ 0x4441, 'CjkUnifiedIdeograph4441', [], 'Display', None],
  [ 0x4442, 'CjkUnifiedIdeograph4442', [], 'Display', None],
  [ 0x4443, 'CjkUnifiedIdeograph4443', [], 'Display', None],
  [ 0x4444, 'CjkUnifiedIdeograph4444', [], 'Display', None],
  [ 0x4445, 'CjkUnifiedIdeograph4445', [], 'Display', None],
  [ 0x4446, 'CjkUnifiedIdeograph4446', [], 'Display', None],
  [ 0x4447, 'CjkUnifiedIdeograph4447', [], 'Display', None],
  [ 0x4448, 'CjkUnifiedIdeograph4448', [], 'Display', None],
  [ 0x4449, 'CjkUnifiedIdeograph4449', [], 'Display', None],
  [ 0x444a, 'CjkUnifiedIdeograph444a', [], 'Display', None],
  [ 0x444b, 'CjkUnifiedIdeograph444b', [], 'Display', None],
  [ 0x444c, 'CjkUnifiedIdeograph444c', [], 'Display', None],
  [ 0x444d, 'CjkUnifiedIdeograph444d', [], 'Display', None],
  [ 0x444e, 'CjkUnifiedIdeograph444e', [], 'Display', None],
  [ 0x444f, 'CjkUnifiedIdeograph444f', [], 'Display', None],
  [ 0x4450, 'CjkUnifiedIdeograph4450', [], 'Display', None],
  [ 0x4451, 'CjkUnifiedIdeograph4451', [], 'Display', None],
  [ 0x4452, 'CjkUnifiedIdeograph4452', [], 'Display', None],
  [ 0x4453, 'CjkUnifiedIdeograph4453', [], 'Display', None],
  [ 0x4454, 'CjkUnifiedIdeograph4454', [], 'Display', None],
  [ 0x4455, 'CjkUnifiedIdeograph4455', [], 'Display', None],
  [ 0x4456, 'CjkUnifiedIdeograph4456', [], 'Display', None],
  [ 0x4457, 'CjkUnifiedIdeograph4457', [], 'Display', None],
  [ 0x4458, 'CjkUnifiedIdeograph4458', [], 'Display', None],
  [ 0x4459, 'CjkUnifiedIdeograph4459', [], 'Display', None],
  [ 0x445a, 'CjkUnifiedIdeograph445a', [], 'Display', None],
  [ 0x445b, 'CjkUnifiedIdeograph445b', [], 'Display', None],
  [ 0x445c, 'CjkUnifiedIdeograph445c', [], 'Display', None],
  [ 0x445d, 'CjkUnifiedIdeograph445d', [], 'Display', None],
  [ 0x445e, 'CjkUnifiedIdeograph445e', [], 'Display', None],
  [ 0x445f, 'CjkUnifiedIdeograph445f', [], 'Display', None],
  [ 0x4460, 'CjkUnifiedIdeograph4460', [], 'Display', None],
  [ 0x4461, 'CjkUnifiedIdeograph4461', [], 'Display', None],
  [ 0x4462, 'CjkUnifiedIdeograph4462', [], 'Display', None],
  [ 0x4463, 'CjkUnifiedIdeograph4463', [], 'Display', None],
  [ 0x4464, 'CjkUnifiedIdeograph4464', [], 'Display', None],
  [ 0x4465, 'CjkUnifiedIdeograph4465', [], 'Display', None],
  [ 0x4466, 'CjkUnifiedIdeograph4466', [], 'Display', None],
  [ 0x4467, 'CjkUnifiedIdeograph4467', [], 'Display', None],
  [ 0x4468, 'CjkUnifiedIdeograph4468', [], 'Display', None],
  [ 0x4469, 'CjkUnifiedIdeograph4469', [], 'Display', None],
  [ 0x446a, 'CjkUnifiedIdeograph446a', [], 'Display', None],
  [ 0x446b, 'CjkUnifiedIdeograph446b', [], 'Display', None],
  [ 0x446c, 'CjkUnifiedIdeograph446c', [], 'Display', None],
  [ 0x446d, 'CjkUnifiedIdeograph446d', [], 'Display', None],
  [ 0x446e, 'CjkUnifiedIdeograph446e', [], 'Display', None],
  [ 0x446f, 'CjkUnifiedIdeograph446f', [], 'Display', None],
  [ 0x4470, 'CjkUnifiedIdeograph4470', [], 'Display', None],
  [ 0x4471, 'CjkUnifiedIdeograph4471', [], 'Display', None],
  [ 0x4472, 'CjkUnifiedIdeograph4472', [], 'Display', None],
  [ 0x4473, 'CjkUnifiedIdeograph4473', [], 'Display', None],
  [ 0x4474, 'CjkUnifiedIdeograph4474', [], 'Display', None],
  [ 0x4475, 'CjkUnifiedIdeograph4475', [], 'Display', None],
  [ 0x4476, 'CjkUnifiedIdeograph4476', [], 'Display', None],
  [ 0x4477, 'CjkUnifiedIdeograph4477', [], 'Display', None],
  [ 0x4478, 'CjkUnifiedIdeograph4478', [], 'Display', None],
  [ 0x4479, 'CjkUnifiedIdeograph4479', [], 'Display', None],
  [ 0x447a, 'CjkUnifiedIdeograph447a', [], 'Display', None],
  [ 0x447b, 'CjkUnifiedIdeograph447b', [], 'Display', None],
  [ 0x447c, 'CjkUnifiedIdeograph447c', [], 'Display', None],
  [ 0x447d, 'CjkUnifiedIdeograph447d', [], 'Display', None],
  [ 0x447e, 'CjkUnifiedIdeograph447e', [], 'Display', None],
  [ 0x447f, 'CjkUnifiedIdeograph447f', [], 'Display', None],
  [ 0x4480, 'CjkUnifiedIdeograph4480', [], 'Display', None],
  [ 0x4481, 'CjkUnifiedIdeograph4481', [], 'Display', None],
  [ 0x4482, 'CjkUnifiedIdeograph4482', [], 'Display', None],
  [ 0x4483, 'CjkUnifiedIdeograph4483', [], 'Display', None],
  [ 0x4484, 'CjkUnifiedIdeograph4484', [], 'Display', None],
  [ 0x4485, 'CjkUnifiedIdeograph4485', [], 'Display', None],
  [ 0x4486, 'CjkUnifiedIdeograph4486', [], 'Display', None],
  [ 0x4487, 'CjkUnifiedIdeograph4487', [], 'Display', None],
  [ 0x4488, 'CjkUnifiedIdeograph4488', [], 'Display', None],
  [ 0x4489, 'CjkUnifiedIdeograph4489', [], 'Display', None],
  [ 0x448a, 'CjkUnifiedIdeograph448a', [], 'Display', None],
  [ 0x448b, 'CjkUnifiedIdeograph448b', [], 'Display', None],
  [ 0x448c, 'CjkUnifiedIdeograph448c', [], 'Display', None],
  [ 0x448d, 'CjkUnifiedIdeograph448d', [], 'Display', None],
  [ 0x448e, 'CjkUnifiedIdeograph448e', [], 'Display', None],
  [ 0x448f, 'CjkUnifiedIdeograph448f', [], 'Display', None],
  [ 0x4490, 'CjkUnifiedIdeograph4490', [], 'Display', None],
  [ 0x4491, 'CjkUnifiedIdeograph4491', [], 'Display', None],
  [ 0x4492, 'CjkUnifiedIdeograph4492', [], 'Display', None],
  [ 0x4493, 'CjkUnifiedIdeograph4493', [], 'Display', None],
  [ 0x4494, 'CjkUnifiedIdeograph4494', [], 'Display', None],
  [ 0x4495, 'CjkUnifiedIdeograph4495', [], 'Display', None],
  [ 0x4496, 'CjkUnifiedIdeograph4496', [], 'Display', None],
  [ 0x4497, 'CjkUnifiedIdeograph4497', [], 'Display', None],
  [ 0x4498, 'CjkUnifiedIdeograph4498', [], 'Display', None],
  [ 0x4499, 'CjkUnifiedIdeograph4499', [], 'Display', None],
  [ 0x449a, 'CjkUnifiedIdeograph449a', [], 'Display', None],
  [ 0x449b, 'CjkUnifiedIdeograph449b', [], 'Display', None],
  [ 0x449c, 'CjkUnifiedIdeograph449c', [], 'Display', None],
  [ 0x449d, 'CjkUnifiedIdeograph449d', [], 'Display', None],
  [ 0x449e, 'CjkUnifiedIdeograph449e', [], 'Display', None],
  [ 0x449f, 'CjkUnifiedIdeograph449f', [], 'Display', None],
  [ 0x44a0, 'CjkUnifiedIdeograph44a0', [], 'Display', None],
  [ 0x44a1, 'CjkUnifiedIdeograph44a1', [], 'Display', None],
  [ 0x44a2, 'CjkUnifiedIdeograph44a2', [], 'Display', None],
  [ 0x44a3, 'CjkUnifiedIdeograph44a3', [], 'Display', None],
  [ 0x44a4, 'CjkUnifiedIdeograph44a4', [], 'Display', None],
  [ 0x44a5, 'CjkUnifiedIdeograph44a5', [], 'Display', None],
  [ 0x44a6, 'CjkUnifiedIdeograph44a6', [], 'Display', None],
  [ 0x44a7, 'CjkUnifiedIdeograph44a7', [], 'Display', None],
  [ 0x44a8, 'CjkUnifiedIdeograph44a8', [], 'Display', None],
  [ 0x44a9, 'CjkUnifiedIdeograph44a9', [], 'Display', None],
  [ 0x44aa, 'CjkUnifiedIdeograph44aa', [], 'Display', None],
  [ 0x44ab, 'CjkUnifiedIdeograph44ab', [], 'Display', None],
  [ 0x44ac, 'CjkUnifiedIdeograph44ac', [], 'Display', None],
  [ 0x44ad, 'CjkUnifiedIdeograph44ad', [], 'Display', None],
  [ 0x44ae, 'CjkUnifiedIdeograph44ae', [], 'Display', None],
  [ 0x44af, 'CjkUnifiedIdeograph44af', [], 'Display', None],
  [ 0x44b0, 'CjkUnifiedIdeograph44b0', [], 'Display', None],
  [ 0x44b1, 'CjkUnifiedIdeograph44b1', [], 'Display', None],
  [ 0x44b2, 'CjkUnifiedIdeograph44b2', [], 'Display', None],
  [ 0x44b3, 'CjkUnifiedIdeograph44b3', [], 'Display', None],
  [ 0x44b4, 'CjkUnifiedIdeograph44b4', [], 'Display', None],
  [ 0x44b5, 'CjkUnifiedIdeograph44b5', [], 'Display', None],
  [ 0x44b6, 'CjkUnifiedIdeograph44b6', [], 'Display', None],
  [ 0x44b7, 'CjkUnifiedIdeograph44b7', [], 'Display', None],
  [ 0x44b8, 'CjkUnifiedIdeograph44b8', [], 'Display', None],
  [ 0x44b9, 'CjkUnifiedIdeograph44b9', [], 'Display', None],
  [ 0x44ba, 'CjkUnifiedIdeograph44ba', [], 'Display', None],
  [ 0x44bb, 'CjkUnifiedIdeograph44bb', [], 'Display', None],
  [ 0x44bc, 'CjkUnifiedIdeograph44bc', [], 'Display', None],
  [ 0x44bd, 'CjkUnifiedIdeograph44bd', [], 'Display', None],
  [ 0x44be, 'CjkUnifiedIdeograph44be', [], 'Display', None],
  [ 0x44bf, 'CjkUnifiedIdeograph44bf', [], 'Display', None],
  [ 0x44c0, 'CjkUnifiedIdeograph44c0', [], 'Display', None],
  [ 0x44c1, 'CjkUnifiedIdeograph44c1', [], 'Display', None],
  [ 0x44c2, 'CjkUnifiedIdeograph44c2', [], 'Display', None],
  [ 0x44c3, 'CjkUnifiedIdeograph44c3', [], 'Display', None],
  [ 0x44c4, 'CjkUnifiedIdeograph44c4', [], 'Display', None],
  [ 0x44c5, 'CjkUnifiedIdeograph44c5', [], 'Display', None],
  [ 0x44c6, 'CjkUnifiedIdeograph44c6', [], 'Display', None],
  [ 0x44c7, 'CjkUnifiedIdeograph44c7', [], 'Display', None],
  [ 0x44c8, 'CjkUnifiedIdeograph44c8', [], 'Display', None],
  [ 0x44c9, 'CjkUnifiedIdeograph44c9', [], 'Display', None],
  [ 0x44ca, 'CjkUnifiedIdeograph44ca', [], 'Display', None],
  [ 0x44cb, 'CjkUnifiedIdeograph44cb', [], 'Display', None],
  [ 0x44cc, 'CjkUnifiedIdeograph44cc', [], 'Display', None],
  [ 0x44cd, 'CjkUnifiedIdeograph44cd', [], 'Display', None],
  [ 0x44ce, 'CjkUnifiedIdeograph44ce', [], 'Display', None],
  [ 0x44cf, 'CjkUnifiedIdeograph44cf', [], 'Display', None],
  [ 0x44d0, 'CjkUnifiedIdeograph44d0', [], 'Display', None],
  [ 0x44d1, 'CjkUnifiedIdeograph44d1', [], 'Display', None],
  [ 0x44d2, 'CjkUnifiedIdeograph44d2', [], 'Display', None],
  [ 0x44d3, 'CjkUnifiedIdeograph44d3', [], 'Display', None],
  [ 0x44d4, 'CjkUnifiedIdeograph44d4', [], 'Display', None],
  [ 0x44d5, 'CjkUnifiedIdeograph44d5', [], 'Display', None],
  [ 0x44d6, 'CjkUnifiedIdeograph44d6', [], 'Display', None],
  [ 0x44d7, 'CjkUnifiedIdeograph44d7', [], 'Display', None],
  [ 0x44d8, 'CjkUnifiedIdeograph44d8', [], 'Display', None],
  [ 0x44d9, 'CjkUnifiedIdeograph44d9', [], 'Display', None],
  [ 0x44da, 'CjkUnifiedIdeograph44da', [], 'Display', None],
  [ 0x44db, 'CjkUnifiedIdeograph44db', [], 'Display', None],
  [ 0x44dc, 'CjkUnifiedIdeograph44dc', [], 'Display', None],
  [ 0x44dd, 'CjkUnifiedIdeograph44dd', [], 'Display', None],
  [ 0x44de, 'CjkUnifiedIdeograph44de', [], 'Display', None],
  [ 0x44df, 'CjkUnifiedIdeograph44df', [], 'Display', None],
  [ 0x44e0, 'CjkUnifiedIdeograph44e0', [], 'Display', None],
  [ 0x44e1, 'CjkUnifiedIdeograph44e1', [], 'Display', None],
  [ 0x44e2, 'CjkUnifiedIdeograph44e2', [], 'Display', None],
  [ 0x44e3, 'CjkUnifiedIdeograph44e3', [], 'Display', None],
  [ 0x44e4, 'CjkUnifiedIdeograph44e4', [], 'Display', None],
  [ 0x44e5, 'CjkUnifiedIdeograph44e5', [], 'Display', None],
  [ 0x44e6, 'CjkUnifiedIdeograph44e6', [], 'Display', None],
  [ 0x44e7, 'CjkUnifiedIdeograph44e7', [], 'Display', None],
  [ 0x44e8, 'CjkUnifiedIdeograph44e8', [], 'Display', None],
  [ 0x44e9, 'CjkUnifiedIdeograph44e9', [], 'Display', None],
  [ 0x44ea, 'CjkUnifiedIdeograph44ea', [], 'Display', None],
  [ 0x44eb, 'CjkUnifiedIdeograph44eb', [], 'Display', None],
  [ 0x44ec, 'CjkUnifiedIdeograph44ec', [], 'Display', None],
  [ 0x44ed, 'CjkUnifiedIdeograph44ed', [], 'Display', None],
  [ 0x44ee, 'CjkUnifiedIdeograph44ee', [], 'Display', None],
  [ 0x44ef, 'CjkUnifiedIdeograph44ef', [], 'Display', None],
  [ 0x44f0, 'CjkUnifiedIdeograph44f0', [], 'Display', None],
  [ 0x44f1, 'CjkUnifiedIdeograph44f1', [], 'Display', None],
  [ 0x44f2, 'CjkUnifiedIdeograph44f2', [], 'Display', None],
  [ 0x44f3, 'CjkUnifiedIdeograph44f3', [], 'Display', None],
  [ 0x44f4, 'CjkUnifiedIdeograph44f4', [], 'Display', None],
  [ 0x44f5, 'CjkUnifiedIdeograph44f5', [], 'Display', None],
  [ 0x44f6, 'CjkUnifiedIdeograph44f6', [], 'Display', None],
  [ 0x44f7, 'CjkUnifiedIdeograph44f7', [], 'Display', None],
  [ 0x44f8, 'CjkUnifiedIdeograph44f8', [], 'Display', None],
  [ 0x44f9, 'CjkUnifiedIdeograph44f9', [], 'Display', None],
  [ 0x44fa, 'CjkUnifiedIdeograph44fa', [], 'Display', None],
  [ 0x44fb, 'CjkUnifiedIdeograph44fb', [], 'Display', None],
  [ 0x44fc, 'CjkUnifiedIdeograph44fc', [], 'Display', None],
  [ 0x44fd, 'CjkUnifiedIdeograph44fd', [], 'Display', None],
  [ 0x44fe, 'CjkUnifiedIdeograph44fe', [], 'Display', None],
  [ 0x44ff, 'CjkUnifiedIdeograph44ff', [], 'Display', None],
  [ 0x4500, 'CjkUnifiedIdeograph4500', [], 'Display', None],
  [ 0x4501, 'CjkUnifiedIdeograph4501', [], 'Display', None],
  [ 0x4502, 'CjkUnifiedIdeograph4502', [], 'Display', None],
  [ 0x4503, 'CjkUnifiedIdeograph4503', [], 'Display', None],
  [ 0x4504, 'CjkUnifiedIdeograph4504', [], 'Display', None],
  [ 0x4505, 'CjkUnifiedIdeograph4505', [], 'Display', None],
  [ 0x4506, 'CjkUnifiedIdeograph4506', [], 'Display', None],
  [ 0x4507, 'CjkUnifiedIdeograph4507', [], 'Display', None],
  [ 0x4508, 'CjkUnifiedIdeograph4508', [], 'Display', None],
  [ 0x4509, 'CjkUnifiedIdeograph4509', [], 'Display', None],
  [ 0x450a, 'CjkUnifiedIdeograph450a', [], 'Display', None],
  [ 0x450b, 'CjkUnifiedIdeograph450b', [], 'Display', None],
  [ 0x450c, 'CjkUnifiedIdeograph450c', [], 'Display', None],
  [ 0x450d, 'CjkUnifiedIdeograph450d', [], 'Display', None],
  [ 0x450e, 'CjkUnifiedIdeograph450e', [], 'Display', None],
  [ 0x450f, 'CjkUnifiedIdeograph450f', [], 'Display', None],
  [ 0x4510, 'CjkUnifiedIdeograph4510', [], 'Display', None],
  [ 0x4511, 'CjkUnifiedIdeograph4511', [], 'Display', None],
  [ 0x4512, 'CjkUnifiedIdeograph4512', [], 'Display', None],
  [ 0x4513, 'CjkUnifiedIdeograph4513', [], 'Display', None],
  [ 0x4514, 'CjkUnifiedIdeograph4514', [], 'Display', None],
  [ 0x4515, 'CjkUnifiedIdeograph4515', [], 'Display', None],
  [ 0x4516, 'CjkUnifiedIdeograph4516', [], 'Display', None],
  [ 0x4517, 'CjkUnifiedIdeograph4517', [], 'Display', None],
  [ 0x4518, 'CjkUnifiedIdeograph4518', [], 'Display', None],
  [ 0x4519, 'CjkUnifiedIdeograph4519', [], 'Display', None],
  [ 0x451a, 'CjkUnifiedIdeograph451a', [], 'Display', None],
  [ 0x451b, 'CjkUnifiedIdeograph451b', [], 'Display', None],
  [ 0x451c, 'CjkUnifiedIdeograph451c', [], 'Display', None],
  [ 0x451d, 'CjkUnifiedIdeograph451d', [], 'Display', None],
  [ 0x451e, 'CjkUnifiedIdeograph451e', [], 'Display', None],
  [ 0x451f, 'CjkUnifiedIdeograph451f', [], 'Display', None],
  [ 0x4520, 'CjkUnifiedIdeograph4520', [], 'Display', None],
  [ 0x4521, 'CjkUnifiedIdeograph4521', [], 'Display', None],
  [ 0x4522, 'CjkUnifiedIdeograph4522', [], 'Display', None],
  [ 0x4523, 'CjkUnifiedIdeograph4523', [], 'Display', None],
  [ 0x4524, 'CjkUnifiedIdeograph4524', [], 'Display', None],
  [ 0x4525, 'CjkUnifiedIdeograph4525', [], 'Display', None],
  [ 0x4526, 'CjkUnifiedIdeograph4526', [], 'Display', None],
  [ 0x4527, 'CjkUnifiedIdeograph4527', [], 'Display', None],
  [ 0x4528, 'CjkUnifiedIdeograph4528', [], 'Display', None],
  [ 0x4529, 'CjkUnifiedIdeograph4529', [], 'Display', None],
  [ 0x452a, 'CjkUnifiedIdeograph452a', [], 'Display', None],
  [ 0x452b, 'CjkUnifiedIdeograph452b', [], 'Display', None],
  [ 0x452c, 'CjkUnifiedIdeograph452c', [], 'Display', None],
  [ 0x452d, 'CjkUnifiedIdeograph452d', [], 'Display', None],
  [ 0x452e, 'CjkUnifiedIdeograph452e', [], 'Display', None],
  [ 0x452f, 'CjkUnifiedIdeograph452f', [], 'Display', None],
  [ 0x4530, 'CjkUnifiedIdeograph4530', [], 'Display', None],
  [ 0x4531, 'CjkUnifiedIdeograph4531', [], 'Display', None],
  [ 0x4532, 'CjkUnifiedIdeograph4532', [], 'Display', None],
  [ 0x4533, 'CjkUnifiedIdeograph4533', [], 'Display', None],
  [ 0x4534, 'CjkUnifiedIdeograph4534', [], 'Display', None],
  [ 0x4535, 'CjkUnifiedIdeograph4535', [], 'Display', None],
  [ 0x4536, 'CjkUnifiedIdeograph4536', [], 'Display', None],
  [ 0x4537, 'CjkUnifiedIdeograph4537', [], 'Display', None],
  [ 0x4538, 'CjkUnifiedIdeograph4538', [], 'Display', None],
  [ 0x4539, 'CjkUnifiedIdeograph4539', [], 'Display', None],
  [ 0x453a, 'CjkUnifiedIdeograph453a', [], 'Display', None],
  [ 0x453b, 'CjkUnifiedIdeograph453b', [], 'Display', None],
  [ 0x453c, 'CjkUnifiedIdeograph453c', [], 'Display', None],
  [ 0x453d, 'CjkUnifiedIdeograph453d', [], 'Display', None],
  [ 0x453e, 'CjkUnifiedIdeograph453e', [], 'Display', None],
  [ 0x453f, 'CjkUnifiedIdeograph453f', [], 'Display', None],
  [ 0x4540, 'CjkUnifiedIdeograph4540', [], 'Display', None],
  [ 0x4541, 'CjkUnifiedIdeograph4541', [], 'Display', None],
  [ 0x4542, 'CjkUnifiedIdeograph4542', [], 'Display', None],
  [ 0x4543, 'CjkUnifiedIdeograph4543', [], 'Display', None],
  [ 0x4544, 'CjkUnifiedIdeograph4544', [], 'Display', None],
  [ 0x4545, 'CjkUnifiedIdeograph4545', [], 'Display', None],
  [ 0x4546, 'CjkUnifiedIdeograph4546', [], 'Display', None],
  [ 0x4547, 'CjkUnifiedIdeograph4547', [], 'Display', None],
  [ 0x4548, 'CjkUnifiedIdeograph4548', [], 'Display', None],
  [ 0x4549, 'CjkUnifiedIdeograph4549', [], 'Display', None],
  [ 0x454a, 'CjkUnifiedIdeograph454a', [], 'Display', None],
  [ 0x454b, 'CjkUnifiedIdeograph454b', [], 'Display', None],
  [ 0x454c, 'CjkUnifiedIdeograph454c', [], 'Display', None],
  [ 0x454d, 'CjkUnifiedIdeograph454d', [], 'Display', None],
  [ 0x454e, 'CjkUnifiedIdeograph454e', [], 'Display', None],
  [ 0x454f, 'CjkUnifiedIdeograph454f', [], 'Display', None],
  [ 0x4550, 'CjkUnifiedIdeograph4550', [], 'Display', None],
  [ 0x4551, 'CjkUnifiedIdeograph4551', [], 'Display', None],
  [ 0x4552, 'CjkUnifiedIdeograph4552', [], 'Display', None],
  [ 0x4553, 'CjkUnifiedIdeograph4553', [], 'Display', None],
  [ 0x4554, 'CjkUnifiedIdeograph4554', [], 'Display', None],
  [ 0x4555, 'CjkUnifiedIdeograph4555', [], 'Display', None],
  [ 0x4556, 'CjkUnifiedIdeograph4556', [], 'Display', None],
  [ 0x4557, 'CjkUnifiedIdeograph4557', [], 'Display', None],
  [ 0x4558, 'CjkUnifiedIdeograph4558', [], 'Display', None],
  [ 0x4559, 'CjkUnifiedIdeograph4559', [], 'Display', None],
  [ 0x455a, 'CjkUnifiedIdeograph455a', [], 'Display', None],
  [ 0x455b, 'CjkUnifiedIdeograph455b', [], 'Display', None],
  [ 0x455c, 'CjkUnifiedIdeograph455c', [], 'Display', None],
  [ 0x455d, 'CjkUnifiedIdeograph455d', [], 'Display', None],
  [ 0x455e, 'CjkUnifiedIdeograph455e', [], 'Display', None],
  [ 0x455f, 'CjkUnifiedIdeograph455f', [], 'Display', None],
  [ 0x4560, 'CjkUnifiedIdeograph4560', [], 'Display', None],
  [ 0x4561, 'CjkUnifiedIdeograph4561', [], 'Display', None],
  [ 0x4562, 'CjkUnifiedIdeograph4562', [], 'Display', None],
  [ 0x4563, 'CjkUnifiedIdeograph4563', [], 'Display', None],
  [ 0x4564, 'CjkUnifiedIdeograph4564', [], 'Display', None],
  [ 0x4565, 'CjkUnifiedIdeograph4565', [], 'Display', None],
  [ 0x4566, 'CjkUnifiedIdeograph4566', [], 'Display', None],
  [ 0x4567, 'CjkUnifiedIdeograph4567', [], 'Display', None],
  [ 0x4568, 'CjkUnifiedIdeograph4568', [], 'Display', None],
  [ 0x4569, 'CjkUnifiedIdeograph4569', [], 'Display', None],
  [ 0x456a, 'CjkUnifiedIdeograph456a', [], 'Display', None],
  [ 0x456b, 'CjkUnifiedIdeograph456b', [], 'Display', None],
  [ 0x456c, 'CjkUnifiedIdeograph456c', [], 'Display', None],
  [ 0x456d, 'CjkUnifiedIdeograph456d', [], 'Display', None],
  [ 0x456e, 'CjkUnifiedIdeograph456e', [], 'Display', None],
  [ 0x456f, 'CjkUnifiedIdeograph456f', [], 'Display', None],
  [ 0x4570, 'CjkUnifiedIdeograph4570', [], 'Display', None],
  [ 0x4571, 'CjkUnifiedIdeograph4571', [], 'Display', None],
  [ 0x4572, 'CjkUnifiedIdeograph4572', [], 'Display', None],
  [ 0x4573, 'CjkUnifiedIdeograph4573', [], 'Display', None],
  [ 0x4574, 'CjkUnifiedIdeograph4574', [], 'Display', None],
  [ 0x4575, 'CjkUnifiedIdeograph4575', [], 'Display', None],
  [ 0x4576, 'CjkUnifiedIdeograph4576', [], 'Display', None],
  [ 0x4577, 'CjkUnifiedIdeograph4577', [], 'Display', None],
  [ 0x4578, 'CjkUnifiedIdeograph4578', [], 'Display', None],
  [ 0x4579, 'CjkUnifiedIdeograph4579', [], 'Display', None],
  [ 0x457a, 'CjkUnifiedIdeograph457a', [], 'Display', None],
  [ 0x457b, 'CjkUnifiedIdeograph457b', [], 'Display', None],
  [ 0x457c, 'CjkUnifiedIdeograph457c', [], 'Display', None],
  [ 0x457d, 'CjkUnifiedIdeograph457d', [], 'Display', None],
  [ 0x457e, 'CjkUnifiedIdeograph457e', [], 'Display', None],
  [ 0x457f, 'CjkUnifiedIdeograph457f', [], 'Display', None],
  [ 0x4580, 'CjkUnifiedIdeograph4580', [], 'Display', None],
  [ 0x4581, 'CjkUnifiedIdeograph4581', [], 'Display', None],
  [ 0x4582, 'CjkUnifiedIdeograph4582', [], 'Display', None],
  [ 0x4583, 'CjkUnifiedIdeograph4583', [], 'Display', None],
  [ 0x4584, 'CjkUnifiedIdeograph4584', [], 'Display', None],
  [ 0x4585, 'CjkUnifiedIdeograph4585', [], 'Display', None],
  [ 0x4586, 'CjkUnifiedIdeograph4586', [], 'Display', None],
  [ 0x4587, 'CjkUnifiedIdeograph4587', [], 'Display', None],
  [ 0x4588, 'CjkUnifiedIdeograph4588', [], 'Display', None],
  [ 0x4589, 'CjkUnifiedIdeograph4589', [], 'Display', None],
  [ 0x458a, 'CjkUnifiedIdeograph458a', [], 'Display', None],
  [ 0x458b, 'CjkUnifiedIdeograph458b', [], 'Display', None],
  [ 0x458c, 'CjkUnifiedIdeograph458c', [], 'Display', None],
  [ 0x458d, 'CjkUnifiedIdeograph458d', [], 'Display', None],
  [ 0x458e, 'CjkUnifiedIdeograph458e', [], 'Display', None],
  [ 0x458f, 'CjkUnifiedIdeograph458f', [], 'Display', None],
  [ 0x4590, 'CjkUnifiedIdeograph4590', [], 'Display', None],
  [ 0x4591, 'CjkUnifiedIdeograph4591', [], 'Display', None],
  [ 0x4592, 'CjkUnifiedIdeograph4592', [], 'Display', None],
  [ 0x4593, 'CjkUnifiedIdeograph4593', [], 'Display', None],
  [ 0x4594, 'CjkUnifiedIdeograph4594', [], 'Display', None],
  [ 0x4595, 'CjkUnifiedIdeograph4595', [], 'Display', None],
  [ 0x4596, 'CjkUnifiedIdeograph4596', [], 'Display', None],
  [ 0x4597, 'CjkUnifiedIdeograph4597', [], 'Display', None],
  [ 0x4598, 'CjkUnifiedIdeograph4598', [], 'Display', None],
  [ 0x4599, 'CjkUnifiedIdeograph4599', [], 'Display', None],
  [ 0x459a, 'CjkUnifiedIdeograph459a', [], 'Display', None],
  [ 0x459b, 'CjkUnifiedIdeograph459b', [], 'Display', None],
  [ 0x459c, 'CjkUnifiedIdeograph459c', [], 'Display', None],
  [ 0x459d, 'CjkUnifiedIdeograph459d', [], 'Display', None],
  [ 0x459e, 'CjkUnifiedIdeograph459e', [], 'Display', None],
  [ 0x459f, 'CjkUnifiedIdeograph459f', [], 'Display', None],
  [ 0x45a0, 'CjkUnifiedIdeograph45a0', [], 'Display', None],
  [ 0x45a1, 'CjkUnifiedIdeograph45a1', [], 'Display', None],
  [ 0x45a2, 'CjkUnifiedIdeograph45a2', [], 'Display', None],
  [ 0x45a3, 'CjkUnifiedIdeograph45a3', [], 'Display', None],
  [ 0x45a4, 'CjkUnifiedIdeograph45a4', [], 'Display', None],
  [ 0x45a5, 'CjkUnifiedIdeograph45a5', [], 'Display', None],
  [ 0x45a6, 'CjkUnifiedIdeograph45a6', [], 'Display', None],
  [ 0x45a7, 'CjkUnifiedIdeograph45a7', [], 'Display', None],
  [ 0x45a8, 'CjkUnifiedIdeograph45a8', [], 'Display', None],
  [ 0x45a9, 'CjkUnifiedIdeograph45a9', [], 'Display', None],
  [ 0x45aa, 'CjkUnifiedIdeograph45aa', [], 'Display', None],
  [ 0x45ab, 'CjkUnifiedIdeograph45ab', [], 'Display', None],
  [ 0x45ac, 'CjkUnifiedIdeograph45ac', [], 'Display', None],
  [ 0x45ad, 'CjkUnifiedIdeograph45ad', [], 'Display', None],
  [ 0x45ae, 'CjkUnifiedIdeograph45ae', [], 'Display', None],
  [ 0x45af, 'CjkUnifiedIdeograph45af', [], 'Display', None],
  [ 0x45b0, 'CjkUnifiedIdeograph45b0', [], 'Display', None],
  [ 0x45b1, 'CjkUnifiedIdeograph45b1', [], 'Display', None],
  [ 0x45b2, 'CjkUnifiedIdeograph45b2', [], 'Display', None],
  [ 0x45b3, 'CjkUnifiedIdeograph45b3', [], 'Display', None],
  [ 0x45b4, 'CjkUnifiedIdeograph45b4', [], 'Display', None],
  [ 0x45b5, 'CjkUnifiedIdeograph45b5', [], 'Display', None],
  [ 0x45b6, 'CjkUnifiedIdeograph45b6', [], 'Display', None],
  [ 0x45b7, 'CjkUnifiedIdeograph45b7', [], 'Display', None],
  [ 0x45b8, 'CjkUnifiedIdeograph45b8', [], 'Display', None],
  [ 0x45b9, 'CjkUnifiedIdeograph45b9', [], 'Display', None],
  [ 0x45ba, 'CjkUnifiedIdeograph45ba', [], 'Display', None],
  [ 0x45bb, 'CjkUnifiedIdeograph45bb', [], 'Display', None],
  [ 0x45bc, 'CjkUnifiedIdeograph45bc', [], 'Display', None],
  [ 0x45bd, 'CjkUnifiedIdeograph45bd', [], 'Display', None],
  [ 0x45be, 'CjkUnifiedIdeograph45be', [], 'Display', None],
  [ 0x45bf, 'CjkUnifiedIdeograph45bf', [], 'Display', None],
  [ 0x45c0, 'CjkUnifiedIdeograph45c0', [], 'Display', None],
  [ 0x45c1, 'CjkUnifiedIdeograph45c1', [], 'Display', None],
  [ 0x45c2, 'CjkUnifiedIdeograph45c2', [], 'Display', None],
  [ 0x45c3, 'CjkUnifiedIdeograph45c3', [], 'Display', None],
  [ 0x45c4, 'CjkUnifiedIdeograph45c4', [], 'Display', None],
  [ 0x45c5, 'CjkUnifiedIdeograph45c5', [], 'Display', None],
  [ 0x45c6, 'CjkUnifiedIdeograph45c6', [], 'Display', None],
  [ 0x45c7, 'CjkUnifiedIdeograph45c7', [], 'Display', None],
  [ 0x45c8, 'CjkUnifiedIdeograph45c8', [], 'Display', None],
  [ 0x45c9, 'CjkUnifiedIdeograph45c9', [], 'Display', None],
  [ 0x45ca, 'CjkUnifiedIdeograph45ca', [], 'Display', None],
  [ 0x45cb, 'CjkUnifiedIdeograph45cb', [], 'Display', None],
  [ 0x45cc, 'CjkUnifiedIdeograph45cc', [], 'Display', None],
  [ 0x45cd, 'CjkUnifiedIdeograph45cd', [], 'Display', None],
  [ 0x45ce, 'CjkUnifiedIdeograph45ce', [], 'Display', None],
  [ 0x45cf, 'CjkUnifiedIdeograph45cf', [], 'Display', None],
  [ 0x45d0, 'CjkUnifiedIdeograph45d0', [], 'Display', None],
  [ 0x45d1, 'CjkUnifiedIdeograph45d1', [], 'Display', None],
  [ 0x45d2, 'CjkUnifiedIdeograph45d2', [], 'Display', None],
  [ 0x45d3, 'CjkUnifiedIdeograph45d3', [], 'Display', None],
  [ 0x45d4, 'CjkUnifiedIdeograph45d4', [], 'Display', None],
  [ 0x45d5, 'CjkUnifiedIdeograph45d5', [], 'Display', None],
  [ 0x45d6, 'CjkUnifiedIdeograph45d6', [], 'Display', None],
  [ 0x45d7, 'CjkUnifiedIdeograph45d7', [], 'Display', None],
  [ 0x45d8, 'CjkUnifiedIdeograph45d8', [], 'Display', None],
  [ 0x45d9, 'CjkUnifiedIdeograph45d9', [], 'Display', None],
  [ 0x45da, 'CjkUnifiedIdeograph45da', [], 'Display', None],
  [ 0x45db, 'CjkUnifiedIdeograph45db', [], 'Display', None],
  [ 0x45dc, 'CjkUnifiedIdeograph45dc', [], 'Display', None],
  [ 0x45dd, 'CjkUnifiedIdeograph45dd', [], 'Display', None],
  [ 0x45de, 'CjkUnifiedIdeograph45de', [], 'Display', None],
  [ 0x45df, 'CjkUnifiedIdeograph45df', [], 'Display', None],
  [ 0x45e0, 'CjkUnifiedIdeograph45e0', [], 'Display', None],
  [ 0x45e1, 'CjkUnifiedIdeograph45e1', [], 'Display', None],
  [ 0x45e2, 'CjkUnifiedIdeograph45e2', [], 'Display', None],
  [ 0x45e3, 'CjkUnifiedIdeograph45e3', [], 'Display', None],
  [ 0x45e4, 'CjkUnifiedIdeograph45e4', [], 'Display', None],
  [ 0x45e5, 'CjkUnifiedIdeograph45e5', [], 'Display', None],
  [ 0x45e6, 'CjkUnifiedIdeograph45e6', [], 'Display', None],
  [ 0x45e7, 'CjkUnifiedIdeograph45e7', [], 'Display', None],
  [ 0x45e8, 'CjkUnifiedIdeograph45e8', [], 'Display', None],
  [ 0x45e9, 'CjkUnifiedIdeograph45e9', [], 'Display', None],
  [ 0x45ea, 'CjkUnifiedIdeograph45ea', [], 'Display', None],
  [ 0x45eb, 'CjkUnifiedIdeograph45eb', [], 'Display', None],
  [ 0x45ec, 'CjkUnifiedIdeograph45ec', [], 'Display', None],
  [ 0x45ed, 'CjkUnifiedIdeograph45ed', [], 'Display', None],
  [ 0x45ee, 'CjkUnifiedIdeograph45ee', [], 'Display', None],
  [ 0x45ef, 'CjkUnifiedIdeograph45ef', [], 'Display', None],
  [ 0x45f0, 'CjkUnifiedIdeograph45f0', [], 'Display', None],
  [ 0x45f1, 'CjkUnifiedIdeograph45f1', [], 'Display', None],
  [ 0x45f2, 'CjkUnifiedIdeograph45f2', [], 'Display', None],
  [ 0x45f3, 'CjkUnifiedIdeograph45f3', [], 'Display', None],
  [ 0x45f4, 'CjkUnifiedIdeograph45f4', [], 'Display', None],
  [ 0x45f5, 'CjkUnifiedIdeograph45f5', [], 'Display', None],
  [ 0x45f6, 'CjkUnifiedIdeograph45f6', [], 'Display', None],
  [ 0x45f7, 'CjkUnifiedIdeograph45f7', [], 'Display', None],
  [ 0x45f8, 'CjkUnifiedIdeograph45f8', [], 'Display', None],
  [ 0x45f9, 'CjkUnifiedIdeograph45f9', [], 'Display', None],
  [ 0x45fa, 'CjkUnifiedIdeograph45fa', [], 'Display', None],
  [ 0x45fb, 'CjkUnifiedIdeograph45fb', [], 'Display', None],
  [ 0x45fc, 'CjkUnifiedIdeograph45fc', [], 'Display', None],
  [ 0x45fd, 'CjkUnifiedIdeograph45fd', [], 'Display', None],
  [ 0x45fe, 'CjkUnifiedIdeograph45fe', [], 'Display', None],
  [ 0x45ff, 'CjkUnifiedIdeograph45ff', [], 'Display', None],
  [ 0x4600, 'CjkUnifiedIdeograph4600', [], 'Display', None],
  [ 0x4601, 'CjkUnifiedIdeograph4601', [], 'Display', None],
  [ 0x4602, 'CjkUnifiedIdeograph4602', [], 'Display', None],
  [ 0x4603, 'CjkUnifiedIdeograph4603', [], 'Display', None],
  [ 0x4604, 'CjkUnifiedIdeograph4604', [], 'Display', None],
  [ 0x4605, 'CjkUnifiedIdeograph4605', [], 'Display', None],
  [ 0x4606, 'CjkUnifiedIdeograph4606', [], 'Display', None],
  [ 0x4607, 'CjkUnifiedIdeograph4607', [], 'Display', None],
  [ 0x4608, 'CjkUnifiedIdeograph4608', [], 'Display', None],
  [ 0x4609, 'CjkUnifiedIdeograph4609', [], 'Display', None],
  [ 0x460a, 'CjkUnifiedIdeograph460a', [], 'Display', None],
  [ 0x460b, 'CjkUnifiedIdeograph460b', [], 'Display', None],
  [ 0x460c, 'CjkUnifiedIdeograph460c', [], 'Display', None],
  [ 0x460d, 'CjkUnifiedIdeograph460d', [], 'Display', None],
  [ 0x460e, 'CjkUnifiedIdeograph460e', [], 'Display', None],
  [ 0x460f, 'CjkUnifiedIdeograph460f', [], 'Display', None],
  [ 0x4610, 'CjkUnifiedIdeograph4610', [], 'Display', None],
  [ 0x4611, 'CjkUnifiedIdeograph4611', [], 'Display', None],
  [ 0x4612, 'CjkUnifiedIdeograph4612', [], 'Display', None],
  [ 0x4613, 'CjkUnifiedIdeograph4613', [], 'Display', None],
  [ 0x4614, 'CjkUnifiedIdeograph4614', [], 'Display', None],
  [ 0x4615, 'CjkUnifiedIdeograph4615', [], 'Display', None],
  [ 0x4616, 'CjkUnifiedIdeograph4616', [], 'Display', None],
  [ 0x4617, 'CjkUnifiedIdeograph4617', [], 'Display', None],
  [ 0x4618, 'CjkUnifiedIdeograph4618', [], 'Display', None],
  [ 0x4619, 'CjkUnifiedIdeograph4619', [], 'Display', None],
  [ 0x461a, 'CjkUnifiedIdeograph461a', [], 'Display', None],
  [ 0x461b, 'CjkUnifiedIdeograph461b', [], 'Display', None],
  [ 0x461c, 'CjkUnifiedIdeograph461c', [], 'Display', None],
  [ 0x461d, 'CjkUnifiedIdeograph461d', [], 'Display', None],
  [ 0x461e, 'CjkUnifiedIdeograph461e', [], 'Display', None],
  [ 0x461f, 'CjkUnifiedIdeograph461f', [], 'Display', None],
  [ 0x4620, 'CjkUnifiedIdeograph4620', [], 'Display', None],
  [ 0x4621, 'CjkUnifiedIdeograph4621', [], 'Display', None],
  [ 0x4622, 'CjkUnifiedIdeograph4622', [], 'Display', None],
  [ 0x4623, 'CjkUnifiedIdeograph4623', [], 'Display', None],
  [ 0x4624, 'CjkUnifiedIdeograph4624', [], 'Display', None],
  [ 0x4625, 'CjkUnifiedIdeograph4625', [], 'Display', None],
  [ 0x4626, 'CjkUnifiedIdeograph4626', [], 'Display', None],
  [ 0x4627, 'CjkUnifiedIdeograph4627', [], 'Display', None],
  [ 0x4628, 'CjkUnifiedIdeograph4628', [], 'Display', None],
  [ 0x4629, 'CjkUnifiedIdeograph4629', [], 'Display', None],
  [ 0x462a, 'CjkUnifiedIdeograph462a', [], 'Display', None],
  [ 0x462b, 'CjkUnifiedIdeograph462b', [], 'Display', None],
  [ 0x462c, 'CjkUnifiedIdeograph462c', [], 'Display', None],
  [ 0x462d, 'CjkUnifiedIdeograph462d', [], 'Display', None],
  [ 0x462e, 'CjkUnifiedIdeograph462e', [], 'Display', None],
  [ 0x462f, 'CjkUnifiedIdeograph462f', [], 'Display', None],
  [ 0x4630, 'CjkUnifiedIdeograph4630', [], 'Display', None],
  [ 0x4631, 'CjkUnifiedIdeograph4631', [], 'Display', None],
  [ 0x4632, 'CjkUnifiedIdeograph4632', [], 'Display', None],
  [ 0x4633, 'CjkUnifiedIdeograph4633', [], 'Display', None],
  [ 0x4634, 'CjkUnifiedIdeograph4634', [], 'Display', None],
  [ 0x4635, 'CjkUnifiedIdeograph4635', [], 'Display', None],
  [ 0x4636, 'CjkUnifiedIdeograph4636', [], 'Display', None],
  [ 0x4637, 'CjkUnifiedIdeograph4637', [], 'Display', None],
  [ 0x4638, 'CjkUnifiedIdeograph4638', [], 'Display', None],
  [ 0x4639, 'CjkUnifiedIdeograph4639', [], 'Display', None],
  [ 0x463a, 'CjkUnifiedIdeograph463a', [], 'Display', None],
  [ 0x463b, 'CjkUnifiedIdeograph463b', [], 'Display', None],
  [ 0x463c, 'CjkUnifiedIdeograph463c', [], 'Display', None],
  [ 0x463d, 'CjkUnifiedIdeograph463d', [], 'Display', None],
  [ 0x463e, 'CjkUnifiedIdeograph463e', [], 'Display', None],
  [ 0x463f, 'CjkUnifiedIdeograph463f', [], 'Display', None],
  [ 0x4640, 'CjkUnifiedIdeograph4640', [], 'Display', None],
  [ 0x4641, 'CjkUnifiedIdeograph4641', [], 'Display', None],
  [ 0x4642, 'CjkUnifiedIdeograph4642', [], 'Display', None],
  [ 0x4643, 'CjkUnifiedIdeograph4643', [], 'Display', None],
  [ 0x4644, 'CjkUnifiedIdeograph4644', [], 'Display', None],
  [ 0x4645, 'CjkUnifiedIdeograph4645', [], 'Display', None],
  [ 0x4646, 'CjkUnifiedIdeograph4646', [], 'Display', None],
  [ 0x4647, 'CjkUnifiedIdeograph4647', [], 'Display', None],
  [ 0x4648, 'CjkUnifiedIdeograph4648', [], 'Display', None],
  [ 0x4649, 'CjkUnifiedIdeograph4649', [], 'Display', None],
  [ 0x464a, 'CjkUnifiedIdeograph464a', [], 'Display', None],
  [ 0x464b, 'CjkUnifiedIdeograph464b', [], 'Display', None],
  [ 0x464c, 'CjkUnifiedIdeograph464c', [], 'Display', None],
  [ 0x464d, 'CjkUnifiedIdeograph464d', [], 'Display', None],
  [ 0x464e, 'CjkUnifiedIdeograph464e', [], 'Display', None],
  [ 0x464f, 'CjkUnifiedIdeograph464f', [], 'Display', None],
  [ 0x4650, 'CjkUnifiedIdeograph4650', [], 'Display', None],
  [ 0x4651, 'CjkUnifiedIdeograph4651', [], 'Display', None],
  [ 0x4652, 'CjkUnifiedIdeograph4652', [], 'Display', None],
  [ 0x4653, 'CjkUnifiedIdeograph4653', [], 'Display', None],
  [ 0x4654, 'CjkUnifiedIdeograph4654', [], 'Display', None],
  [ 0x4655, 'CjkUnifiedIdeograph4655', [], 'Display', None],
  [ 0x4656, 'CjkUnifiedIdeograph4656', [], 'Display', None],
  [ 0x4657, 'CjkUnifiedIdeograph4657', [], 'Display', None],
  [ 0x4658, 'CjkUnifiedIdeograph4658', [], 'Display', None],
  [ 0x4659, 'CjkUnifiedIdeograph4659', [], 'Display', None],
  [ 0x465a, 'CjkUnifiedIdeograph465a', [], 'Display', None],
  [ 0x465b, 'CjkUnifiedIdeograph465b', [], 'Display', None],
  [ 0x465c, 'CjkUnifiedIdeograph465c', [], 'Display', None],
  [ 0x465d, 'CjkUnifiedIdeograph465d', [], 'Display', None],
  [ 0x465e, 'CjkUnifiedIdeograph465e', [], 'Display', None],
  [ 0x465f, 'CjkUnifiedIdeograph465f', [], 'Display', None],
  [ 0x4660, 'CjkUnifiedIdeograph4660', [], 'Display', None],
  [ 0x4661, 'CjkUnifiedIdeograph4661', [], 'Display', None],
  [ 0x4662, 'CjkUnifiedIdeograph4662', [], 'Display', None],
  [ 0x4663, 'CjkUnifiedIdeograph4663', [], 'Display', None],
  [ 0x4664, 'CjkUnifiedIdeograph4664', [], 'Display', None],
  [ 0x4665, 'CjkUnifiedIdeograph4665', [], 'Display', None],
  [ 0x4666, 'CjkUnifiedIdeograph4666', [], 'Display', None],
  [ 0x4667, 'CjkUnifiedIdeograph4667', [], 'Display', None],
  [ 0x4668, 'CjkUnifiedIdeograph4668', [], 'Display', None],
  [ 0x4669, 'CjkUnifiedIdeograph4669', [], 'Display', None],
  [ 0x466a, 'CjkUnifiedIdeograph466a', [], 'Display', None],
  [ 0x466b, 'CjkUnifiedIdeograph466b', [], 'Display', None],
  [ 0x466c, 'CjkUnifiedIdeograph466c', [], 'Display', None],
  [ 0x466d, 'CjkUnifiedIdeograph466d', [], 'Display', None],
  [ 0x466e, 'CjkUnifiedIdeograph466e', [], 'Display', None],
  [ 0x466f, 'CjkUnifiedIdeograph466f', [], 'Display', None],
  [ 0x4670, 'CjkUnifiedIdeograph4670', [], 'Display', None],
  [ 0x4671, 'CjkUnifiedIdeograph4671', [], 'Display', None],
  [ 0x4672, 'CjkUnifiedIdeograph4672', [], 'Display', None],
  [ 0x4673, 'CjkUnifiedIdeograph4673', [], 'Display', None],
  [ 0x4674, 'CjkUnifiedIdeograph4674', [], 'Display', None],
  [ 0x4675, 'CjkUnifiedIdeograph4675', [], 'Display', None],
  [ 0x4676, 'CjkUnifiedIdeograph4676', [], 'Display', None],
  [ 0x4677, 'CjkUnifiedIdeograph4677', [], 'Display', None],
  [ 0x4678, 'CjkUnifiedIdeograph4678', [], 'Display', None],
  [ 0x4679, 'CjkUnifiedIdeograph4679', [], 'Display', None],
  [ 0x467a, 'CjkUnifiedIdeograph467a', [], 'Display', None],
  [ 0x467b, 'CjkUnifiedIdeograph467b', [], 'Display', None],
  [ 0x467c, 'CjkUnifiedIdeograph467c', [], 'Display', None],
  [ 0x467d, 'CjkUnifiedIdeograph467d', [], 'Display', None],
  [ 0x467e, 'CjkUnifiedIdeograph467e', [], 'Display', None],
  [ 0x467f, 'CjkUnifiedIdeograph467f', [], 'Display', None],
  [ 0x4680, 'CjkUnifiedIdeograph4680', [], 'Display', None],
  [ 0x4681, 'CjkUnifiedIdeograph4681', [], 'Display', None],
  [ 0x4682, 'CjkUnifiedIdeograph4682', [], 'Display', None],
  [ 0x4683, 'CjkUnifiedIdeograph4683', [], 'Display', None],
  [ 0x4684, 'CjkUnifiedIdeograph4684', [], 'Display', None],
  [ 0x4685, 'CjkUnifiedIdeograph4685', [], 'Display', None],
  [ 0x4686, 'CjkUnifiedIdeograph4686', [], 'Display', None],
  [ 0x4687, 'CjkUnifiedIdeograph4687', [], 'Display', None],
  [ 0x4688, 'CjkUnifiedIdeograph4688', [], 'Display', None],
  [ 0x4689, 'CjkUnifiedIdeograph4689', [], 'Display', None],
  [ 0x468a, 'CjkUnifiedIdeograph468a', [], 'Display', None],
  [ 0x468b, 'CjkUnifiedIdeograph468b', [], 'Display', None],
  [ 0x468c, 'CjkUnifiedIdeograph468c', [], 'Display', None],
  [ 0x468d, 'CjkUnifiedIdeograph468d', [], 'Display', None],
  [ 0x468e, 'CjkUnifiedIdeograph468e', [], 'Display', None],
  [ 0x468f, 'CjkUnifiedIdeograph468f', [], 'Display', None],
  [ 0x4690, 'CjkUnifiedIdeograph4690', [], 'Display', None],
  [ 0x4691, 'CjkUnifiedIdeograph4691', [], 'Display', None],
  [ 0x4692, 'CjkUnifiedIdeograph4692', [], 'Display', None],
  [ 0x4693, 'CjkUnifiedIdeograph4693', [], 'Display', None],
  [ 0x4694, 'CjkUnifiedIdeograph4694', [], 'Display', None],
  [ 0x4695, 'CjkUnifiedIdeograph4695', [], 'Display', None],
  [ 0x4696, 'CjkUnifiedIdeograph4696', [], 'Display', None],
  [ 0x4697, 'CjkUnifiedIdeograph4697', [], 'Display', None],
  [ 0x4698, 'CjkUnifiedIdeograph4698', [], 'Display', None],
  [ 0x4699, 'CjkUnifiedIdeograph4699', [], 'Display', None],
  [ 0x469a, 'CjkUnifiedIdeograph469a', [], 'Display', None],
  [ 0x469b, 'CjkUnifiedIdeograph469b', [], 'Display', None],
  [ 0x469c, 'CjkUnifiedIdeograph469c', [], 'Display', None],
  [ 0x469d, 'CjkUnifiedIdeograph469d', [], 'Display', None],
  [ 0x469e, 'CjkUnifiedIdeograph469e', [], 'Display', None],
  [ 0x469f, 'CjkUnifiedIdeograph469f', [], 'Display', None],
  [ 0x46a0, 'CjkUnifiedIdeograph46a0', [], 'Display', None],
  [ 0x46a1, 'CjkUnifiedIdeograph46a1', [], 'Display', None],
  [ 0x46a2, 'CjkUnifiedIdeograph46a2', [], 'Display', None],
  [ 0x46a3, 'CjkUnifiedIdeograph46a3', [], 'Display', None],
  [ 0x46a4, 'CjkUnifiedIdeograph46a4', [], 'Display', None],
  [ 0x46a5, 'CjkUnifiedIdeograph46a5', [], 'Display', None],
  [ 0x46a6, 'CjkUnifiedIdeograph46a6', [], 'Display', None],
  [ 0x46a7, 'CjkUnifiedIdeograph46a7', [], 'Display', None],
  [ 0x46a8, 'CjkUnifiedIdeograph46a8', [], 'Display', None],
  [ 0x46a9, 'CjkUnifiedIdeograph46a9', [], 'Display', None],
  [ 0x46aa, 'CjkUnifiedIdeograph46aa', [], 'Display', None],
  [ 0x46ab, 'CjkUnifiedIdeograph46ab', [], 'Display', None],
  [ 0x46ac, 'CjkUnifiedIdeograph46ac', [], 'Display', None],
  [ 0x46ad, 'CjkUnifiedIdeograph46ad', [], 'Display', None],
  [ 0x46ae, 'CjkUnifiedIdeograph46ae', [], 'Display', None],
  [ 0x46af, 'CjkUnifiedIdeograph46af', [], 'Display', None],
  [ 0x46b0, 'CjkUnifiedIdeograph46b0', [], 'Display', None],
  [ 0x46b1, 'CjkUnifiedIdeograph46b1', [], 'Display', None],
  [ 0x46b2, 'CjkUnifiedIdeograph46b2', [], 'Display', None],
  [ 0x46b3, 'CjkUnifiedIdeograph46b3', [], 'Display', None],
  [ 0x46b4, 'CjkUnifiedIdeograph46b4', [], 'Display', None],
  [ 0x46b5, 'CjkUnifiedIdeograph46b5', [], 'Display', None],
  [ 0x46b6, 'CjkUnifiedIdeograph46b6', [], 'Display', None],
  [ 0x46b7, 'CjkUnifiedIdeograph46b7', [], 'Display', None],
  [ 0x46b8, 'CjkUnifiedIdeograph46b8', [], 'Display', None],
  [ 0x46b9, 'CjkUnifiedIdeograph46b9', [], 'Display', None],
  [ 0x46ba, 'CjkUnifiedIdeograph46ba', [], 'Display', None],
  [ 0x46bb, 'CjkUnifiedIdeograph46bb', [], 'Display', None],
  [ 0x46bc, 'CjkUnifiedIdeograph46bc', [], 'Display', None],
  [ 0x46bd, 'CjkUnifiedIdeograph46bd', [], 'Display', None],
  [ 0x46be, 'CjkUnifiedIdeograph46be', [], 'Display', None],
  [ 0x46bf, 'CjkUnifiedIdeograph46bf', [], 'Display', None],
  [ 0x46c0, 'CjkUnifiedIdeograph46c0', [], 'Display', None],
  [ 0x46c1, 'CjkUnifiedIdeograph46c1', [], 'Display', None],
  [ 0x46c2, 'CjkUnifiedIdeograph46c2', [], 'Display', None],
  [ 0x46c3, 'CjkUnifiedIdeograph46c3', [], 'Display', None],
  [ 0x46c4, 'CjkUnifiedIdeograph46c4', [], 'Display', None],
  [ 0x46c5, 'CjkUnifiedIdeograph46c5', [], 'Display', None],
  [ 0x46c6, 'CjkUnifiedIdeograph46c6', [], 'Display', None],
  [ 0x46c7, 'CjkUnifiedIdeograph46c7', [], 'Display', None],
  [ 0x46c8, 'CjkUnifiedIdeograph46c8', [], 'Display', None],
  [ 0x46c9, 'CjkUnifiedIdeograph46c9', [], 'Display', None],
  [ 0x46ca, 'CjkUnifiedIdeograph46ca', [], 'Display', None],
  [ 0x46cb, 'CjkUnifiedIdeograph46cb', [], 'Display', None],
  [ 0x46cc, 'CjkUnifiedIdeograph46cc', [], 'Display', None],
  [ 0x46cd, 'CjkUnifiedIdeograph46cd', [], 'Display', None],
  [ 0x46ce, 'CjkUnifiedIdeograph46ce', [], 'Display', None],
  [ 0x46cf, 'CjkUnifiedIdeograph46cf', [], 'Display', None],
  [ 0x46d0, 'CjkUnifiedIdeograph46d0', [], 'Display', None],
  [ 0x46d1, 'CjkUnifiedIdeograph46d1', [], 'Display', None],
  [ 0x46d2, 'CjkUnifiedIdeograph46d2', [], 'Display', None],
  [ 0x46d3, 'CjkUnifiedIdeograph46d3', [], 'Display', None],
  [ 0x46d4, 'CjkUnifiedIdeograph46d4', [], 'Display', None],
  [ 0x46d5, 'CjkUnifiedIdeograph46d5', [], 'Display', None],
  [ 0x46d6, 'CjkUnifiedIdeograph46d6', [], 'Display', None],
  [ 0x46d7, 'CjkUnifiedIdeograph46d7', [], 'Display', None],
  [ 0x46d8, 'CjkUnifiedIdeograph46d8', [], 'Display', None],
  [ 0x46d9, 'CjkUnifiedIdeograph46d9', [], 'Display', None],
  [ 0x46da, 'CjkUnifiedIdeograph46da', [], 'Display', None],
  [ 0x46db, 'CjkUnifiedIdeograph46db', [], 'Display', None],
  [ 0x46dc, 'CjkUnifiedIdeograph46dc', [], 'Display', None],
  [ 0x46dd, 'CjkUnifiedIdeograph46dd', [], 'Display', None],
  [ 0x46de, 'CjkUnifiedIdeograph46de', [], 'Display', None],
  [ 0x46df, 'CjkUnifiedIdeograph46df', [], 'Display', None],
  [ 0x46e0, 'CjkUnifiedIdeograph46e0', [], 'Display', None],
  [ 0x46e1, 'CjkUnifiedIdeograph46e1', [], 'Display', None],
  [ 0x46e2, 'CjkUnifiedIdeograph46e2', [], 'Display', None],
  [ 0x46e3, 'CjkUnifiedIdeograph46e3', [], 'Display', None],
  [ 0x46e4, 'CjkUnifiedIdeograph46e4', [], 'Display', None],
  [ 0x46e5, 'CjkUnifiedIdeograph46e5', [], 'Display', None],
  [ 0x46e6, 'CjkUnifiedIdeograph46e6', [], 'Display', None],
  [ 0x46e7, 'CjkUnifiedIdeograph46e7', [], 'Display', None],
  [ 0x46e8, 'CjkUnifiedIdeograph46e8', [], 'Display', None],
  [ 0x46e9, 'CjkUnifiedIdeograph46e9', [], 'Display', None],
  [ 0x46ea, 'CjkUnifiedIdeograph46ea', [], 'Display', None],
  [ 0x46eb, 'CjkUnifiedIdeograph46eb', [], 'Display', None],
  [ 0x46ec, 'CjkUnifiedIdeograph46ec', [], 'Display', None],
  [ 0x46ed, 'CjkUnifiedIdeograph46ed', [], 'Display', None],
  [ 0x46ee, 'CjkUnifiedIdeograph46ee', [], 'Display', None],
  [ 0x46ef, 'CjkUnifiedIdeograph46ef', [], 'Display', None],
  [ 0x46f0, 'CjkUnifiedIdeograph46f0', [], 'Display', None],
  [ 0x46f1, 'CjkUnifiedIdeograph46f1', [], 'Display', None],
  [ 0x46f2, 'CjkUnifiedIdeograph46f2', [], 'Display', None],
  [ 0x46f3, 'CjkUnifiedIdeograph46f3', [], 'Display', None],
  [ 0x46f4, 'CjkUnifiedIdeograph46f4', [], 'Display', None],
  [ 0x46f5, 'CjkUnifiedIdeograph46f5', [], 'Display', None],
  [ 0x46f6, 'CjkUnifiedIdeograph46f6', [], 'Display', None],
  [ 0x46f7, 'CjkUnifiedIdeograph46f7', [], 'Display', None],
  [ 0x46f8, 'CjkUnifiedIdeograph46f8', [], 'Display', None],
  [ 0x46f9, 'CjkUnifiedIdeograph46f9', [], 'Display', None],
  [ 0x46fa, 'CjkUnifiedIdeograph46fa', [], 'Display', None],
  [ 0x46fb, 'CjkUnifiedIdeograph46fb', [], 'Display', None],
  [ 0x46fc, 'CjkUnifiedIdeograph46fc', [], 'Display', None],
  [ 0x46fd, 'CjkUnifiedIdeograph46fd', [], 'Display', None],
  [ 0x46fe, 'CjkUnifiedIdeograph46fe', [], 'Display', None],
  [ 0x46ff, 'CjkUnifiedIdeograph46ff', [], 'Display', None],
  [ 0x4700, 'CjkUnifiedIdeograph4700', [], 'Display', None],
  [ 0x4701, 'CjkUnifiedIdeograph4701', [], 'Display', None],
  [ 0x4702, 'CjkUnifiedIdeograph4702', [], 'Display', None],
  [ 0x4703, 'CjkUnifiedIdeograph4703', [], 'Display', None],
  [ 0x4704, 'CjkUnifiedIdeograph4704', [], 'Display', None],
  [ 0x4705, 'CjkUnifiedIdeograph4705', [], 'Display', None],
  [ 0x4706, 'CjkUnifiedIdeograph4706', [], 'Display', None],
  [ 0x4707, 'CjkUnifiedIdeograph4707', [], 'Display', None],
  [ 0x4708, 'CjkUnifiedIdeograph4708', [], 'Display', None],
  [ 0x4709, 'CjkUnifiedIdeograph4709', [], 'Display', None],
  [ 0x470a, 'CjkUnifiedIdeograph470a', [], 'Display', None],
  [ 0x470b, 'CjkUnifiedIdeograph470b', [], 'Display', None],
  [ 0x470c, 'CjkUnifiedIdeograph470c', [], 'Display', None],
  [ 0x470d, 'CjkUnifiedIdeograph470d', [], 'Display', None],
  [ 0x470e, 'CjkUnifiedIdeograph470e', [], 'Display', None],
  [ 0x470f, 'CjkUnifiedIdeograph470f', [], 'Display', None],
  [ 0x4710, 'CjkUnifiedIdeograph4710', [], 'Display', None],
  [ 0x4711, 'CjkUnifiedIdeograph4711', [], 'Display', None],
  [ 0x4712, 'CjkUnifiedIdeograph4712', [], 'Display', None],
  [ 0x4713, 'CjkUnifiedIdeograph4713', [], 'Display', None],
  [ 0x4714, 'CjkUnifiedIdeograph4714', [], 'Display', None],
  [ 0x4715, 'CjkUnifiedIdeograph4715', [], 'Display', None],
  [ 0x4716, 'CjkUnifiedIdeograph4716', [], 'Display', None],
  [ 0x4717, 'CjkUnifiedIdeograph4717', [], 'Display', None],
  [ 0x4718, 'CjkUnifiedIdeograph4718', [], 'Display', None],
  [ 0x4719, 'CjkUnifiedIdeograph4719', [], 'Display', None],
  [ 0x471a, 'CjkUnifiedIdeograph471a', [], 'Display', None],
  [ 0x471b, 'CjkUnifiedIdeograph471b', [], 'Display', None],
  [ 0x471c, 'CjkUnifiedIdeograph471c', [], 'Display', None],
  [ 0x471d, 'CjkUnifiedIdeograph471d', [], 'Display', None],
  [ 0x471e, 'CjkUnifiedIdeograph471e', [], 'Display', None],
  [ 0x471f, 'CjkUnifiedIdeograph471f', [], 'Display', None],
  [ 0x4720, 'CjkUnifiedIdeograph4720', [], 'Display', None],
  [ 0x4721, 'CjkUnifiedIdeograph4721', [], 'Display', None],
  [ 0x4722, 'CjkUnifiedIdeograph4722', [], 'Display', None],
  [ 0x4723, 'CjkUnifiedIdeograph4723', [], 'Display', None],
  [ 0x4724, 'CjkUnifiedIdeograph4724', [], 'Display', None],
  [ 0x4725, 'CjkUnifiedIdeograph4725', [], 'Display', None],
  [ 0x4726, 'CjkUnifiedIdeograph4726', [], 'Display', None],
  [ 0x4727, 'CjkUnifiedIdeograph4727', [], 'Display', None],
  [ 0x4728, 'CjkUnifiedIdeograph4728', [], 'Display', None],
  [ 0x4729, 'CjkUnifiedIdeograph4729', [], 'Display', None],
  [ 0x472a, 'CjkUnifiedIdeograph472a', [], 'Display', None],
  [ 0x472b, 'CjkUnifiedIdeograph472b', [], 'Display', None],
  [ 0x472c, 'CjkUnifiedIdeograph472c', [], 'Display', None],
  [ 0x472d, 'CjkUnifiedIdeograph472d', [], 'Display', None],
  [ 0x472e, 'CjkUnifiedIdeograph472e', [], 'Display', None],
  [ 0x472f, 'CjkUnifiedIdeograph472f', [], 'Display', None],
  [ 0x4730, 'CjkUnifiedIdeograph4730', [], 'Display', None],
  [ 0x4731, 'CjkUnifiedIdeograph4731', [], 'Display', None],
  [ 0x4732, 'CjkUnifiedIdeograph4732', [], 'Display', None],
  [ 0x4733, 'CjkUnifiedIdeograph4733', [], 'Display', None],
  [ 0x4734, 'CjkUnifiedIdeograph4734', [], 'Display', None],
  [ 0x4735, 'CjkUnifiedIdeograph4735', [], 'Display', None],
  [ 0x4736, 'CjkUnifiedIdeograph4736', [], 'Display', None],
  [ 0x4737, 'CjkUnifiedIdeograph4737', [], 'Display', None],
  [ 0x4738, 'CjkUnifiedIdeograph4738', [], 'Display', None],
  [ 0x4739, 'CjkUnifiedIdeograph4739', [], 'Display', None],
  [ 0x473a, 'CjkUnifiedIdeograph473a', [], 'Display', None],
  [ 0x473b, 'CjkUnifiedIdeograph473b', [], 'Display', None],
  [ 0x473c, 'CjkUnifiedIdeograph473c', [], 'Display', None],
  [ 0x473d, 'CjkUnifiedIdeograph473d', [], 'Display', None],
  [ 0x473e, 'CjkUnifiedIdeograph473e', [], 'Display', None],
  [ 0x473f, 'CjkUnifiedIdeograph473f', [], 'Display', None],
  [ 0x4740, 'CjkUnifiedIdeograph4740', [], 'Display', None],
  [ 0x4741, 'CjkUnifiedIdeograph4741', [], 'Display', None],
  [ 0x4742, 'CjkUnifiedIdeograph4742', [], 'Display', None],
  [ 0x4743, 'CjkUnifiedIdeograph4743', [], 'Display', None],
  [ 0x4744, 'CjkUnifiedIdeograph4744', [], 'Display', None],
  [ 0x4745, 'CjkUnifiedIdeograph4745', [], 'Display', None],
  [ 0x4746, 'CjkUnifiedIdeograph4746', [], 'Display', None],
  [ 0x4747, 'CjkUnifiedIdeograph4747', [], 'Display', None],
  [ 0x4748, 'CjkUnifiedIdeograph4748', [], 'Display', None],
  [ 0x4749, 'CjkUnifiedIdeograph4749', [], 'Display', None],
  [ 0x474a, 'CjkUnifiedIdeograph474a', [], 'Display', None],
  [ 0x474b, 'CjkUnifiedIdeograph474b', [], 'Display', None],
  [ 0x474c, 'CjkUnifiedIdeograph474c', [], 'Display', None],
  [ 0x474d, 'CjkUnifiedIdeograph474d', [], 'Display', None],
  [ 0x474e, 'CjkUnifiedIdeograph474e', [], 'Display', None],
  [ 0x474f, 'CjkUnifiedIdeograph474f', [], 'Display', None],
  [ 0x4750, 'CjkUnifiedIdeograph4750', [], 'Display', None],
  [ 0x4751, 'CjkUnifiedIdeograph4751', [], 'Display', None],
  [ 0x4752, 'CjkUnifiedIdeograph4752', [], 'Display', None],
  [ 0x4753, 'CjkUnifiedIdeograph4753', [], 'Display', None],
  [ 0x4754, 'CjkUnifiedIdeograph4754', [], 'Display', None],
  [ 0x4755, 'CjkUnifiedIdeograph4755', [], 'Display', None],
  [ 0x4756, 'CjkUnifiedIdeograph4756', [], 'Display', None],
  [ 0x4757, 'CjkUnifiedIdeograph4757', [], 'Display', None],
  [ 0x4758, 'CjkUnifiedIdeograph4758', [], 'Display', None],
  [ 0x4759, 'CjkUnifiedIdeograph4759', [], 'Display', None],
  [ 0x475a, 'CjkUnifiedIdeograph475a', [], 'Display', None],
  [ 0x475b, 'CjkUnifiedIdeograph475b', [], 'Display', None],
  [ 0x475c, 'CjkUnifiedIdeograph475c', [], 'Display', None],
  [ 0x475d, 'CjkUnifiedIdeograph475d', [], 'Display', None],
  [ 0x475e, 'CjkUnifiedIdeograph475e', [], 'Display', None],
  [ 0x475f, 'CjkUnifiedIdeograph475f', [], 'Display', None],
  [ 0x4760, 'CjkUnifiedIdeograph4760', [], 'Display', None],
  [ 0x4761, 'CjkUnifiedIdeograph4761', [], 'Display', None],
  [ 0x4762, 'CjkUnifiedIdeograph4762', [], 'Display', None],
  [ 0x4763, 'CjkUnifiedIdeograph4763', [], 'Display', None],
  [ 0x4764, 'CjkUnifiedIdeograph4764', [], 'Display', None],
  [ 0x4765, 'CjkUnifiedIdeograph4765', [], 'Display', None],
  [ 0x4766, 'CjkUnifiedIdeograph4766', [], 'Display', None],
  [ 0x4767, 'CjkUnifiedIdeograph4767', [], 'Display', None],
  [ 0x4768, 'CjkUnifiedIdeograph4768', [], 'Display', None],
  [ 0x4769, 'CjkUnifiedIdeograph4769', [], 'Display', None],
  [ 0x476a, 'CjkUnifiedIdeograph476a', [], 'Display', None],
  [ 0x476b, 'CjkUnifiedIdeograph476b', [], 'Display', None],
  [ 0x476c, 'CjkUnifiedIdeograph476c', [], 'Display', None],
  [ 0x476d, 'CjkUnifiedIdeograph476d', [], 'Display', None],
  [ 0x476e, 'CjkUnifiedIdeograph476e', [], 'Display', None],
  [ 0x476f, 'CjkUnifiedIdeograph476f', [], 'Display', None],
  [ 0x4770, 'CjkUnifiedIdeograph4770', [], 'Display', None],
  [ 0x4771, 'CjkUnifiedIdeograph4771', [], 'Display', None],
  [ 0x4772, 'CjkUnifiedIdeograph4772', [], 'Display', None],
  [ 0x4773, 'CjkUnifiedIdeograph4773', [], 'Display', None],
  [ 0x4774, 'CjkUnifiedIdeograph4774', [], 'Display', None],
  [ 0x4775, 'CjkUnifiedIdeograph4775', [], 'Display', None],
  [ 0x4776, 'CjkUnifiedIdeograph4776', [], 'Display', None],
  [ 0x4777, 'CjkUnifiedIdeograph4777', [], 'Display', None],
  [ 0x4778, 'CjkUnifiedIdeograph4778', [], 'Display', None],
  [ 0x4779, 'CjkUnifiedIdeograph4779', [], 'Display', None],
  [ 0x477a, 'CjkUnifiedIdeograph477a', [], 'Display', None],
  [ 0x477b, 'CjkUnifiedIdeograph477b', [], 'Display', None],
  [ 0x477c, 'CjkUnifiedIdeograph477c', [], 'Display', None],
  [ 0x477d, 'CjkUnifiedIdeograph477d', [], 'Display', None],
  [ 0x477e, 'CjkUnifiedIdeograph477e', [], 'Display', None],
  [ 0x477f, 'CjkUnifiedIdeograph477f', [], 'Display', None],
  [ 0x4780, 'CjkUnifiedIdeograph4780', [], 'Display', None],
  [ 0x4781, 'CjkUnifiedIdeograph4781', [], 'Display', None],
  [ 0x4782, 'CjkUnifiedIdeograph4782', [], 'Display', None],
  [ 0x4783, 'CjkUnifiedIdeograph4783', [], 'Display', None],
  [ 0x4784, 'CjkUnifiedIdeograph4784', [], 'Display', None],
  [ 0x4785, 'CjkUnifiedIdeograph4785', [], 'Display', None],
  [ 0x4786, 'CjkUnifiedIdeograph4786', [], 'Display', None],
  [ 0x4787, 'CjkUnifiedIdeograph4787', [], 'Display', None],
  [ 0x4788, 'CjkUnifiedIdeograph4788', [], 'Display', None],
  [ 0x4789, 'CjkUnifiedIdeograph4789', [], 'Display', None],
  [ 0x478a, 'CjkUnifiedIdeograph478a', [], 'Display', None],
  [ 0x478b, 'CjkUnifiedIdeograph478b', [], 'Display', None],
  [ 0x478c, 'CjkUnifiedIdeograph478c', [], 'Display', None],
  [ 0x478d, 'CjkUnifiedIdeograph478d', [], 'Display', None],
  [ 0x478e, 'CjkUnifiedIdeograph478e', [], 'Display', None],
  [ 0x478f, 'CjkUnifiedIdeograph478f', [], 'Display', None],
  [ 0x4790, 'CjkUnifiedIdeograph4790', [], 'Display', None],
  [ 0x4791, 'CjkUnifiedIdeograph4791', [], 'Display', None],
  [ 0x4792, 'CjkUnifiedIdeograph4792', [], 'Display', None],
  [ 0x4793, 'CjkUnifiedIdeograph4793', [], 'Display', None],
  [ 0x4794, 'CjkUnifiedIdeograph4794', [], 'Display', None],
  [ 0x4795, 'CjkUnifiedIdeograph4795', [], 'Display', None],
  [ 0x4796, 'CjkUnifiedIdeograph4796', [], 'Display', None],
  [ 0x4797, 'CjkUnifiedIdeograph4797', [], 'Display', None],
  [ 0x4798, 'CjkUnifiedIdeograph4798', [], 'Display', None],
  [ 0x4799, 'CjkUnifiedIdeograph4799', [], 'Display', None],
  [ 0x479a, 'CjkUnifiedIdeograph479a', [], 'Display', None],
  [ 0x479b, 'CjkUnifiedIdeograph479b', [], 'Display', None],
  [ 0x479c, 'CjkUnifiedIdeograph479c', [], 'Display', None],
  [ 0x479d, 'CjkUnifiedIdeograph479d', [], 'Display', None],
  [ 0x479e, 'CjkUnifiedIdeograph479e', [], 'Display', None],
  [ 0x479f, 'CjkUnifiedIdeograph479f', [], 'Display', None],
  [ 0x47a0, 'CjkUnifiedIdeograph47a0', [], 'Display', None],
  [ 0x47a1, 'CjkUnifiedIdeograph47a1', [], 'Display', None],
  [ 0x47a2, 'CjkUnifiedIdeograph47a2', [], 'Display', None],
  [ 0x47a3, 'CjkUnifiedIdeograph47a3', [], 'Display', None],
  [ 0x47a4, 'CjkUnifiedIdeograph47a4', [], 'Display', None],
  [ 0x47a5, 'CjkUnifiedIdeograph47a5', [], 'Display', None],
  [ 0x47a6, 'CjkUnifiedIdeograph47a6', [], 'Display', None],
  [ 0x47a7, 'CjkUnifiedIdeograph47a7', [], 'Display', None],
  [ 0x47a8, 'CjkUnifiedIdeograph47a8', [], 'Display', None],
  [ 0x47a9, 'CjkUnifiedIdeograph47a9', [], 'Display', None],
  [ 0x47aa, 'CjkUnifiedIdeograph47aa', [], 'Display', None],
  [ 0x47ab, 'CjkUnifiedIdeograph47ab', [], 'Display', None],
  [ 0x47ac, 'CjkUnifiedIdeograph47ac', [], 'Display', None],
  [ 0x47ad, 'CjkUnifiedIdeograph47ad', [], 'Display', None],
  [ 0x47ae, 'CjkUnifiedIdeograph47ae', [], 'Display', None],
  [ 0x47af, 'CjkUnifiedIdeograph47af', [], 'Display', None],
  [ 0x47b0, 'CjkUnifiedIdeograph47b0', [], 'Display', None],
  [ 0x47b1, 'CjkUnifiedIdeograph47b1', [], 'Display', None],
  [ 0x47b2, 'CjkUnifiedIdeograph47b2', [], 'Display', None],
  [ 0x47b3, 'CjkUnifiedIdeograph47b3', [], 'Display', None],
  [ 0x47b4, 'CjkUnifiedIdeograph47b4', [], 'Display', None],
  [ 0x47b5, 'CjkUnifiedIdeograph47b5', [], 'Display', None],
  [ 0x47b6, 'CjkUnifiedIdeograph47b6', [], 'Display', None],
  [ 0x47b7, 'CjkUnifiedIdeograph47b7', [], 'Display', None],
  [ 0x47b8, 'CjkUnifiedIdeograph47b8', [], 'Display', None],
  [ 0x47b9, 'CjkUnifiedIdeograph47b9', [], 'Display', None],
  [ 0x47ba, 'CjkUnifiedIdeograph47ba', [], 'Display', None],
  [ 0x47bb, 'CjkUnifiedIdeograph47bb', [], 'Display', None],
  [ 0x47bc, 'CjkUnifiedIdeograph47bc', [], 'Display', None],
  [ 0x47bd, 'CjkUnifiedIdeograph47bd', [], 'Display', None],
  [ 0x47be, 'CjkUnifiedIdeograph47be', [], 'Display', None],
  [ 0x47bf, 'CjkUnifiedIdeograph47bf', [], 'Display', None],
  [ 0x47c0, 'CjkUnifiedIdeograph47c0', [], 'Display', None],
  [ 0x47c1, 'CjkUnifiedIdeograph47c1', [], 'Display', None],
  [ 0x47c2, 'CjkUnifiedIdeograph47c2', [], 'Display', None],
  [ 0x47c3, 'CjkUnifiedIdeograph47c3', [], 'Display', None],
  [ 0x47c4, 'CjkUnifiedIdeograph47c4', [], 'Display', None],
  [ 0x47c5, 'CjkUnifiedIdeograph47c5', [], 'Display', None],
  [ 0x47c6, 'CjkUnifiedIdeograph47c6', [], 'Display', None],
  [ 0x47c7, 'CjkUnifiedIdeograph47c7', [], 'Display', None],
  [ 0x47c8, 'CjkUnifiedIdeograph47c8', [], 'Display', None],
  [ 0x47c9, 'CjkUnifiedIdeograph47c9', [], 'Display', None],
  [ 0x47ca, 'CjkUnifiedIdeograph47ca', [], 'Display', None],
  [ 0x47cb, 'CjkUnifiedIdeograph47cb', [], 'Display', None],
  [ 0x47cc, 'CjkUnifiedIdeograph47cc', [], 'Display', None],
  [ 0x47cd, 'CjkUnifiedIdeograph47cd', [], 'Display', None],
  [ 0x47ce, 'CjkUnifiedIdeograph47ce', [], 'Display', None],
  [ 0x47cf, 'CjkUnifiedIdeograph47cf', [], 'Display', None],
  [ 0x47d0, 'CjkUnifiedIdeograph47d0', [], 'Display', None],
  [ 0x47d1, 'CjkUnifiedIdeograph47d1', [], 'Display', None],
  [ 0x47d2, 'CjkUnifiedIdeograph47d2', [], 'Display', None],
  [ 0x47d3, 'CjkUnifiedIdeograph47d3', [], 'Display', None],
  [ 0x47d4, 'CjkUnifiedIdeograph47d4', [], 'Display', None],
  [ 0x47d5, 'CjkUnifiedIdeograph47d5', [], 'Display', None],
  [ 0x47d6, 'CjkUnifiedIdeograph47d6', [], 'Display', None],
  [ 0x47d7, 'CjkUnifiedIdeograph47d7', [], 'Display', None],
  [ 0x47d8, 'CjkUnifiedIdeograph47d8', [], 'Display', None],
  [ 0x47d9, 'CjkUnifiedIdeograph47d9', [], 'Display', None],
  [ 0x47da, 'CjkUnifiedIdeograph47da', [], 'Display', None],
  [ 0x47db, 'CjkUnifiedIdeograph47db', [], 'Display', None],
  [ 0x47dc, 'CjkUnifiedIdeograph47dc', [], 'Display', None],
  [ 0x47dd, 'CjkUnifiedIdeograph47dd', [], 'Display', None],
  [ 0x47de, 'CjkUnifiedIdeograph47de', [], 'Display', None],
  [ 0x47df, 'CjkUnifiedIdeograph47df', [], 'Display', None],
  [ 0x47e0, 'CjkUnifiedIdeograph47e0', [], 'Display', None],
  [ 0x47e1, 'CjkUnifiedIdeograph47e1', [], 'Display', None],
  [ 0x47e2, 'CjkUnifiedIdeograph47e2', [], 'Display', None],
  [ 0x47e3, 'CjkUnifiedIdeograph47e3', [], 'Display', None],
  [ 0x47e4, 'CjkUnifiedIdeograph47e4', [], 'Display', None],
  [ 0x47e5, 'CjkUnifiedIdeograph47e5', [], 'Display', None],
  [ 0x47e6, 'CjkUnifiedIdeograph47e6', [], 'Display', None],
  [ 0x47e7, 'CjkUnifiedIdeograph47e7', [], 'Display', None],
  [ 0x47e8, 'CjkUnifiedIdeograph47e8', [], 'Display', None],
  [ 0x47e9, 'CjkUnifiedIdeograph47e9', [], 'Display', None],
  [ 0x47ea, 'CjkUnifiedIdeograph47ea', [], 'Display', None],
  [ 0x47eb, 'CjkUnifiedIdeograph47eb', [], 'Display', None],
  [ 0x47ec, 'CjkUnifiedIdeograph47ec', [], 'Display', None],
  [ 0x47ed, 'CjkUnifiedIdeograph47ed', [], 'Display', None],
  [ 0x47ee, 'CjkUnifiedIdeograph47ee', [], 'Display', None],
  [ 0x47ef, 'CjkUnifiedIdeograph47ef', [], 'Display', None],
  [ 0x47f0, 'CjkUnifiedIdeograph47f0', [], 'Display', None],
  [ 0x47f1, 'CjkUnifiedIdeograph47f1', [], 'Display', None],
  [ 0x47f2, 'CjkUnifiedIdeograph47f2', [], 'Display', None],
  [ 0x47f3, 'CjkUnifiedIdeograph47f3', [], 'Display', None],
  [ 0x47f4, 'CjkUnifiedIdeograph47f4', [], 'Display', None],
  [ 0x47f5, 'CjkUnifiedIdeograph47f5', [], 'Display', None],
  [ 0x47f6, 'CjkUnifiedIdeograph47f6', [], 'Display', None],
  [ 0x47f7, 'CjkUnifiedIdeograph47f7', [], 'Display', None],
  [ 0x47f8, 'CjkUnifiedIdeograph47f8', [], 'Display', None],
  [ 0x47f9, 'CjkUnifiedIdeograph47f9', [], 'Display', None],
  [ 0x47fa, 'CjkUnifiedIdeograph47fa', [], 'Display', None],
  [ 0x47fb, 'CjkUnifiedIdeograph47fb', [], 'Display', None],
  [ 0x47fc, 'CjkUnifiedIdeograph47fc', [], 'Display', None],
  [ 0x47fd, 'CjkUnifiedIdeograph47fd', [], 'Display', None],
  [ 0x47fe, 'CjkUnifiedIdeograph47fe', [], 'Display', None],
  [ 0x47ff, 'CjkUnifiedIdeograph47ff', [], 'Display', None],
  [ 0x4800, 'CjkUnifiedIdeograph4800', [], 'Display', None],
  [ 0x4801, 'CjkUnifiedIdeograph4801', [], 'Display', None],
  [ 0x4802, 'CjkUnifiedIdeograph4802', [], 'Display', None],
  [ 0x4803, 'CjkUnifiedIdeograph4803', [], 'Display', None],
  [ 0x4804, 'CjkUnifiedIdeograph4804', [], 'Display', None],
  [ 0x4805, 'CjkUnifiedIdeograph4805', [], 'Display', None],
  [ 0x4806, 'CjkUnifiedIdeograph4806', [], 'Display', None],
  [ 0x4807, 'CjkUnifiedIdeograph4807', [], 'Display', None],
  [ 0x4808, 'CjkUnifiedIdeograph4808', [], 'Display', None],
  [ 0x4809, 'CjkUnifiedIdeograph4809', [], 'Display', None],
  [ 0x480a, 'CjkUnifiedIdeograph480a', [], 'Display', None],
  [ 0x480b, 'CjkUnifiedIdeograph480b', [], 'Display', None],
  [ 0x480c, 'CjkUnifiedIdeograph480c', [], 'Display', None],
  [ 0x480d, 'CjkUnifiedIdeograph480d', [], 'Display', None],
  [ 0x480e, 'CjkUnifiedIdeograph480e', [], 'Display', None],
  [ 0x480f, 'CjkUnifiedIdeograph480f', [], 'Display', None],
  [ 0x4810, 'CjkUnifiedIdeograph4810', [], 'Display', None],
  [ 0x4811, 'CjkUnifiedIdeograph4811', [], 'Display', None],
  [ 0x4812, 'CjkUnifiedIdeograph4812', [], 'Display', None],
  [ 0x4813, 'CjkUnifiedIdeograph4813', [], 'Display', None],
  [ 0x4814, 'CjkUnifiedIdeograph4814', [], 'Display', None],
  [ 0x4815, 'CjkUnifiedIdeograph4815', [], 'Display', None],
  [ 0x4816, 'CjkUnifiedIdeograph4816', [], 'Display', None],
  [ 0x4817, 'CjkUnifiedIdeograph4817', [], 'Display', None],
  [ 0x4818, 'CjkUnifiedIdeograph4818', [], 'Display', None],
  [ 0x4819, 'CjkUnifiedIdeograph4819', [], 'Display', None],
  [ 0x481a, 'CjkUnifiedIdeograph481a', [], 'Display', None],
  [ 0x481b, 'CjkUnifiedIdeograph481b', [], 'Display', None],
  [ 0x481c, 'CjkUnifiedIdeograph481c', [], 'Display', None],
  [ 0x481d, 'CjkUnifiedIdeograph481d', [], 'Display', None],
  [ 0x481e, 'CjkUnifiedIdeograph481e', [], 'Display', None],
  [ 0x481f, 'CjkUnifiedIdeograph481f', [], 'Display', None],
  [ 0x4820, 'CjkUnifiedIdeograph4820', [], 'Display', None],
  [ 0x4821, 'CjkUnifiedIdeograph4821', [], 'Display', None],
  [ 0x4822, 'CjkUnifiedIdeograph4822', [], 'Display', None],
  [ 0x4823, 'CjkUnifiedIdeograph4823', [], 'Display', None],
  [ 0x4824, 'CjkUnifiedIdeograph4824', [], 'Display', None],
  [ 0x4825, 'CjkUnifiedIdeograph4825', [], 'Display', None],
  [ 0x4826, 'CjkUnifiedIdeograph4826', [], 'Display', None],
  [ 0x4827, 'CjkUnifiedIdeograph4827', [], 'Display', None],
  [ 0x4828, 'CjkUnifiedIdeograph4828', [], 'Display', None],
  [ 0x4829, 'CjkUnifiedIdeograph4829', [], 'Display', None],
  [ 0x482a, 'CjkUnifiedIdeograph482a', [], 'Display', None],
  [ 0x482b, 'CjkUnifiedIdeograph482b', [], 'Display', None],
  [ 0x482c, 'CjkUnifiedIdeograph482c', [], 'Display', None],
  [ 0x482d, 'CjkUnifiedIdeograph482d', [], 'Display', None],
  [ 0x482e, 'CjkUnifiedIdeograph482e', [], 'Display', None],
  [ 0x482f, 'CjkUnifiedIdeograph482f', [], 'Display', None],
  [ 0x4830, 'CjkUnifiedIdeograph4830', [], 'Display', None],
  [ 0x4831, 'CjkUnifiedIdeograph4831', [], 'Display', None],
  [ 0x4832, 'CjkUnifiedIdeograph4832', [], 'Display', None],
  [ 0x4833, 'CjkUnifiedIdeograph4833', [], 'Display', None],
  [ 0x4834, 'CjkUnifiedIdeograph4834', [], 'Display', None],
  [ 0x4835, 'CjkUnifiedIdeograph4835', [], 'Display', None],
  [ 0x4836, 'CjkUnifiedIdeograph4836', [], 'Display', None],
  [ 0x4837, 'CjkUnifiedIdeograph4837', [], 'Display', None],
  [ 0x4838, 'CjkUnifiedIdeograph4838', [], 'Display', None],
  [ 0x4839, 'CjkUnifiedIdeograph4839', [], 'Display', None],
  [ 0x483a, 'CjkUnifiedIdeograph483a', [], 'Display', None],
  [ 0x483b, 'CjkUnifiedIdeograph483b', [], 'Display', None],
  [ 0x483c, 'CjkUnifiedIdeograph483c', [], 'Display', None],
  [ 0x483d, 'CjkUnifiedIdeograph483d', [], 'Display', None],
  [ 0x483e, 'CjkUnifiedIdeograph483e', [], 'Display', None],
  [ 0x483f, 'CjkUnifiedIdeograph483f', [], 'Display', None],
  [ 0x4840, 'CjkUnifiedIdeograph4840', [], 'Display', None],
  [ 0x4841, 'CjkUnifiedIdeograph4841', [], 'Display', None],
  [ 0x4842, 'CjkUnifiedIdeograph4842', [], 'Display', None],
  [ 0x4843, 'CjkUnifiedIdeograph4843', [], 'Display', None],
  [ 0x4844, 'CjkUnifiedIdeograph4844', [], 'Display', None],
  [ 0x4845, 'CjkUnifiedIdeograph4845', [], 'Display', None],
  [ 0x4846, 'CjkUnifiedIdeograph4846', [], 'Display', None],
  [ 0x4847, 'CjkUnifiedIdeograph4847', [], 'Display', None],
  [ 0x4848, 'CjkUnifiedIdeograph4848', [], 'Display', None],
  [ 0x4849, 'CjkUnifiedIdeograph4849', [], 'Display', None],
  [ 0x484a, 'CjkUnifiedIdeograph484a', [], 'Display', None],
  [ 0x484b, 'CjkUnifiedIdeograph484b', [], 'Display', None],
  [ 0x484c, 'CjkUnifiedIdeograph484c', [], 'Display', None],
  [ 0x484d, 'CjkUnifiedIdeograph484d', [], 'Display', None],
  [ 0x484e, 'CjkUnifiedIdeograph484e', [], 'Display', None],
  [ 0x484f, 'CjkUnifiedIdeograph484f', [], 'Display', None],
  [ 0x4850, 'CjkUnifiedIdeograph4850', [], 'Display', None],
  [ 0x4851, 'CjkUnifiedIdeograph4851', [], 'Display', None],
  [ 0x4852, 'CjkUnifiedIdeograph4852', [], 'Display', None],
  [ 0x4853, 'CjkUnifiedIdeograph4853', [], 'Display', None],
  [ 0x4854, 'CjkUnifiedIdeograph4854', [], 'Display', None],
  [ 0x4855, 'CjkUnifiedIdeograph4855', [], 'Display', None],
  [ 0x4856, 'CjkUnifiedIdeograph4856', [], 'Display', None],
  [ 0x4857, 'CjkUnifiedIdeograph4857', [], 'Display', None],
  [ 0x4858, 'CjkUnifiedIdeograph4858', [], 'Display', None],
  [ 0x4859, 'CjkUnifiedIdeograph4859', [], 'Display', None],
  [ 0x485a, 'CjkUnifiedIdeograph485a', [], 'Display', None],
  [ 0x485b, 'CjkUnifiedIdeograph485b', [], 'Display', None],
  [ 0x485c, 'CjkUnifiedIdeograph485c', [], 'Display', None],
  [ 0x485d, 'CjkUnifiedIdeograph485d', [], 'Display', None],
  [ 0x485e, 'CjkUnifiedIdeograph485e', [], 'Display', None],
  [ 0x485f, 'CjkUnifiedIdeograph485f', [], 'Display', None],
  [ 0x4860, 'CjkUnifiedIdeograph4860', [], 'Display', None],
  [ 0x4861, 'CjkUnifiedIdeograph4861', [], 'Display', None],
  [ 0x4862, 'CjkUnifiedIdeograph4862', [], 'Display', None],
  [ 0x4863, 'CjkUnifiedIdeograph4863', [], 'Display', None],
  [ 0x4864, 'CjkUnifiedIdeograph4864', [], 'Display', None],
  [ 0x4865, 'CjkUnifiedIdeograph4865', [], 'Display', None],
  [ 0x4866, 'CjkUnifiedIdeograph4866', [], 'Display', None],
  [ 0x4867, 'CjkUnifiedIdeograph4867', [], 'Display', None],
  [ 0x4868, 'CjkUnifiedIdeograph4868', [], 'Display', None],
  [ 0x4869, 'CjkUnifiedIdeograph4869', [], 'Display', None],
  [ 0x486a, 'CjkUnifiedIdeograph486a', [], 'Display', None],
  [ 0x486b, 'CjkUnifiedIdeograph486b', [], 'Display', None],
  [ 0x486c, 'CjkUnifiedIdeograph486c', [], 'Display', None],
  [ 0x486d, 'CjkUnifiedIdeograph486d', [], 'Display', None],
  [ 0x486e, 'CjkUnifiedIdeograph486e', [], 'Display', None],
  [ 0x486f, 'CjkUnifiedIdeograph486f', [], 'Display', None],
  [ 0x4870, 'CjkUnifiedIdeograph4870', [], 'Display', None],
  [ 0x4871, 'CjkUnifiedIdeograph4871', [], 'Display', None],
  [ 0x4872, 'CjkUnifiedIdeograph4872', [], 'Display', None],
  [ 0x4873, 'CjkUnifiedIdeograph4873', [], 'Display', None],
  [ 0x4874, 'CjkUnifiedIdeograph4874', [], 'Display', None],
  [ 0x4875, 'CjkUnifiedIdeograph4875', [], 'Display', None],
  [ 0x4876, 'CjkUnifiedIdeograph4876', [], 'Display', None],
  [ 0x4877, 'CjkUnifiedIdeograph4877', [], 'Display', None],
  [ 0x4878, 'CjkUnifiedIdeograph4878', [], 'Display', None],
  [ 0x4879, 'CjkUnifiedIdeograph4879', [], 'Display', None],
  [ 0x487a, 'CjkUnifiedIdeograph487a', [], 'Display', None],
  [ 0x487b, 'CjkUnifiedIdeograph487b', [], 'Display', None],
  [ 0x487c, 'CjkUnifiedIdeograph487c', [], 'Display', None],
  [ 0x487d, 'CjkUnifiedIdeograph487d', [], 'Display', None],
  [ 0x487e, 'CjkUnifiedIdeograph487e', [], 'Display', None],
  [ 0x487f, 'CjkUnifiedIdeograph487f', [], 'Display', None],
  [ 0x4880, 'CjkUnifiedIdeograph4880', [], 'Display', None],
  [ 0x4881, 'CjkUnifiedIdeograph4881', [], 'Display', None],
  [ 0x4882, 'CjkUnifiedIdeograph4882', [], 'Display', None],
  [ 0x4883, 'CjkUnifiedIdeograph4883', [], 'Display', None],
  [ 0x4884, 'CjkUnifiedIdeograph4884', [], 'Display', None],
  [ 0x4885, 'CjkUnifiedIdeograph4885', [], 'Display', None],
  [ 0x4886, 'CjkUnifiedIdeograph4886', [], 'Display', None],
  [ 0x4887, 'CjkUnifiedIdeograph4887', [], 'Display', None],
  [ 0x4888, 'CjkUnifiedIdeograph4888', [], 'Display', None],
  [ 0x4889, 'CjkUnifiedIdeograph4889', [], 'Display', None],
  [ 0x488a, 'CjkUnifiedIdeograph488a', [], 'Display', None],
  [ 0x488b, 'CjkUnifiedIdeograph488b', [], 'Display', None],
  [ 0x488c, 'CjkUnifiedIdeograph488c', [], 'Display', None],
  [ 0x488d, 'CjkUnifiedIdeograph488d', [], 'Display', None],
  [ 0x488e, 'CjkUnifiedIdeograph488e', [], 'Display', None],
  [ 0x488f, 'CjkUnifiedIdeograph488f', [], 'Display', None],
  [ 0x4890, 'CjkUnifiedIdeograph4890', [], 'Display', None],
  [ 0x4891, 'CjkUnifiedIdeograph4891', [], 'Display', None],
  [ 0x4892, 'CjkUnifiedIdeograph4892', [], 'Display', None],
  [ 0x4893, 'CjkUnifiedIdeograph4893', [], 'Display', None],
  [ 0x4894, 'CjkUnifiedIdeograph4894', [], 'Display', None],
  [ 0x4895, 'CjkUnifiedIdeograph4895', [], 'Display', None],
  [ 0x4896, 'CjkUnifiedIdeograph4896', [], 'Display', None],
  [ 0x4897, 'CjkUnifiedIdeograph4897', [], 'Display', None],
  [ 0x4898, 'CjkUnifiedIdeograph4898', [], 'Display', None],
  [ 0x4899, 'CjkUnifiedIdeograph4899', [], 'Display', None],
  [ 0x489a, 'CjkUnifiedIdeograph489a', [], 'Display', None],
  [ 0x489b, 'CjkUnifiedIdeograph489b', [], 'Display', None],
  [ 0x489c, 'CjkUnifiedIdeograph489c', [], 'Display', None],
  [ 0x489d, 'CjkUnifiedIdeograph489d', [], 'Display', None],
  [ 0x489e, 'CjkUnifiedIdeograph489e', [], 'Display', None],
  [ 0x489f, 'CjkUnifiedIdeograph489f', [], 'Display', None],
  [ 0x48a0, 'CjkUnifiedIdeograph48a0', [], 'Display', None],
  [ 0x48a1, 'CjkUnifiedIdeograph48a1', [], 'Display', None],
  [ 0x48a2, 'CjkUnifiedIdeograph48a2', [], 'Display', None],
  [ 0x48a3, 'CjkUnifiedIdeograph48a3', [], 'Display', None],
  [ 0x48a4, 'CjkUnifiedIdeograph48a4', [], 'Display', None],
  [ 0x48a5, 'CjkUnifiedIdeograph48a5', [], 'Display', None],
  [ 0x48a6, 'CjkUnifiedIdeograph48a6', [], 'Display', None],
  [ 0x48a7, 'CjkUnifiedIdeograph48a7', [], 'Display', None],
  [ 0x48a8, 'CjkUnifiedIdeograph48a8', [], 'Display', None],
  [ 0x48a9, 'CjkUnifiedIdeograph48a9', [], 'Display', None],
  [ 0x48aa, 'CjkUnifiedIdeograph48aa', [], 'Display', None],
  [ 0x48ab, 'CjkUnifiedIdeograph48ab', [], 'Display', None],
  [ 0x48ac, 'CjkUnifiedIdeograph48ac', [], 'Display', None],
  [ 0x48ad, 'CjkUnifiedIdeograph48ad', [], 'Display', None],
  [ 0x48ae, 'CjkUnifiedIdeograph48ae', [], 'Display', None],
  [ 0x48af, 'CjkUnifiedIdeograph48af', [], 'Display', None],
  [ 0x48b0, 'CjkUnifiedIdeograph48b0', [], 'Display', None],
  [ 0x48b1, 'CjkUnifiedIdeograph48b1', [], 'Display', None],
  [ 0x48b2, 'CjkUnifiedIdeograph48b2', [], 'Display', None],
  [ 0x48b3, 'CjkUnifiedIdeograph48b3', [], 'Display', None],
  [ 0x48b4, 'CjkUnifiedIdeograph48b4', [], 'Display', None],
  [ 0x48b5, 'CjkUnifiedIdeograph48b5', [], 'Display', None],
  [ 0x48b6, 'CjkUnifiedIdeograph48b6', [], 'Display', None],
  [ 0x48b7, 'CjkUnifiedIdeograph48b7', [], 'Display', None],
  [ 0x48b8, 'CjkUnifiedIdeograph48b8', [], 'Display', None],
  [ 0x48b9, 'CjkUnifiedIdeograph48b9', [], 'Display', None],
  [ 0x48ba, 'CjkUnifiedIdeograph48ba', [], 'Display', None],
  [ 0x48bb, 'CjkUnifiedIdeograph48bb', [], 'Display', None],
  [ 0x48bc, 'CjkUnifiedIdeograph48bc', [], 'Display', None],
  [ 0x48bd, 'CjkUnifiedIdeograph48bd', [], 'Display', None],
  [ 0x48be, 'CjkUnifiedIdeograph48be', [], 'Display', None],
  [ 0x48bf, 'CjkUnifiedIdeograph48bf', [], 'Display', None],
  [ 0x48c0, 'CjkUnifiedIdeograph48c0', [], 'Display', None],
  [ 0x48c1, 'CjkUnifiedIdeograph48c1', [], 'Display', None],
  [ 0x48c2, 'CjkUnifiedIdeograph48c2', [], 'Display', None],
  [ 0x48c3, 'CjkUnifiedIdeograph48c3', [], 'Display', None],
  [ 0x48c4, 'CjkUnifiedIdeograph48c4', [], 'Display', None],
  [ 0x48c5, 'CjkUnifiedIdeograph48c5', [], 'Display', None],
  [ 0x48c6, 'CjkUnifiedIdeograph48c6', [], 'Display', None],
  [ 0x48c7, 'CjkUnifiedIdeograph48c7', [], 'Display', None],
  [ 0x48c8, 'CjkUnifiedIdeograph48c8', [], 'Display', None],
  [ 0x48c9, 'CjkUnifiedIdeograph48c9', [], 'Display', None],
  [ 0x48ca, 'CjkUnifiedIdeograph48ca', [], 'Display', None],
  [ 0x48cb, 'CjkUnifiedIdeograph48cb', [], 'Display', None],
  [ 0x48cc, 'CjkUnifiedIdeograph48cc', [], 'Display', None],
  [ 0x48cd, 'CjkUnifiedIdeograph48cd', [], 'Display', None],
  [ 0x48ce, 'CjkUnifiedIdeograph48ce', [], 'Display', None],
  [ 0x48cf, 'CjkUnifiedIdeograph48cf', [], 'Display', None],
  [ 0x48d0, 'CjkUnifiedIdeograph48d0', [], 'Display', None],
  [ 0x48d1, 'CjkUnifiedIdeograph48d1', [], 'Display', None],
  [ 0x48d2, 'CjkUnifiedIdeograph48d2', [], 'Display', None],
  [ 0x48d3, 'CjkUnifiedIdeograph48d3', [], 'Display', None],
  [ 0x48d4, 'CjkUnifiedIdeograph48d4', [], 'Display', None],
  [ 0x48d5, 'CjkUnifiedIdeograph48d5', [], 'Display', None],
  [ 0x48d6, 'CjkUnifiedIdeograph48d6', [], 'Display', None],
  [ 0x48d7, 'CjkUnifiedIdeograph48d7', [], 'Display', None],
  [ 0x48d8, 'CjkUnifiedIdeograph48d8', [], 'Display', None],
  [ 0x48d9, 'CjkUnifiedIdeograph48d9', [], 'Display', None],
  [ 0x48da, 'CjkUnifiedIdeograph48da', [], 'Display', None],
  [ 0x48db, 'CjkUnifiedIdeograph48db', [], 'Display', None],
  [ 0x48dc, 'CjkUnifiedIdeograph48dc', [], 'Display', None],
  [ 0x48dd, 'CjkUnifiedIdeograph48dd', [], 'Display', None],
  [ 0x48de, 'CjkUnifiedIdeograph48de', [], 'Display', None],
  [ 0x48df, 'CjkUnifiedIdeograph48df', [], 'Display', None],
  [ 0x48e0, 'CjkUnifiedIdeograph48e0', [], 'Display', None],
  [ 0x48e1, 'CjkUnifiedIdeograph48e1', [], 'Display', None],
  [ 0x48e2, 'CjkUnifiedIdeograph48e2', [], 'Display', None],
  [ 0x48e3, 'CjkUnifiedIdeograph48e3', [], 'Display', None],
  [ 0x48e4, 'CjkUnifiedIdeograph48e4', [], 'Display', None],
  [ 0x48e5, 'CjkUnifiedIdeograph48e5', [], 'Display', None],
  [ 0x48e6, 'CjkUnifiedIdeograph48e6', [], 'Display', None],
  [ 0x48e7, 'CjkUnifiedIdeograph48e7', [], 'Display', None],
  [ 0x48e8, 'CjkUnifiedIdeograph48e8', [], 'Display', None],
  [ 0x48e9, 'CjkUnifiedIdeograph48e9', [], 'Display', None],
  [ 0x48ea, 'CjkUnifiedIdeograph48ea', [], 'Display', None],
  [ 0x48eb, 'CjkUnifiedIdeograph48eb', [], 'Display', None],
  [ 0x48ec, 'CjkUnifiedIdeograph48ec', [], 'Display', None],
  [ 0x48ed, 'CjkUnifiedIdeograph48ed', [], 'Display', None],
  [ 0x48ee, 'CjkUnifiedIdeograph48ee', [], 'Display', None],
  [ 0x48ef, 'CjkUnifiedIdeograph48ef', [], 'Display', None],
  [ 0x48f0, 'CjkUnifiedIdeograph48f0', [], 'Display', None],
  [ 0x48f1, 'CjkUnifiedIdeograph48f1', [], 'Display', None],
  [ 0x48f2, 'CjkUnifiedIdeograph48f2', [], 'Display', None],
  [ 0x48f3, 'CjkUnifiedIdeograph48f3', [], 'Display', None],
  [ 0x48f4, 'CjkUnifiedIdeograph48f4', [], 'Display', None],
  [ 0x48f5, 'CjkUnifiedIdeograph48f5', [], 'Display', None],
  [ 0x48f6, 'CjkUnifiedIdeograph48f6', [], 'Display', None],
  [ 0x48f7, 'CjkUnifiedIdeograph48f7', [], 'Display', None],
  [ 0x48f8, 'CjkUnifiedIdeograph48f8', [], 'Display', None],
  [ 0x48f9, 'CjkUnifiedIdeograph48f9', [], 'Display', None],
  [ 0x48fa, 'CjkUnifiedIdeograph48fa', [], 'Display', None],
  [ 0x48fb, 'CjkUnifiedIdeograph48fb', [], 'Display', None],
  [ 0x48fc, 'CjkUnifiedIdeograph48fc', [], 'Display', None],
  [ 0x48fd, 'CjkUnifiedIdeograph48fd', [], 'Display', None],
  [ 0x48fe, 'CjkUnifiedIdeograph48fe', [], 'Display', None],
  [ 0x48ff, 'CjkUnifiedIdeograph48ff', [], 'Display', None],
  [ 0x4900, 'CjkUnifiedIdeograph4900', [], 'Display', None],
  [ 0x4901, 'CjkUnifiedIdeograph4901', [], 'Display', None],
  [ 0x4902, 'CjkUnifiedIdeograph4902', [], 'Display', None],
  [ 0x4903, 'CjkUnifiedIdeograph4903', [], 'Display', None],
  [ 0x4904, 'CjkUnifiedIdeograph4904', [], 'Display', None],
  [ 0x4905, 'CjkUnifiedIdeograph4905', [], 'Display', None],
  [ 0x4906, 'CjkUnifiedIdeograph4906', [], 'Display', None],
  [ 0x4907, 'CjkUnifiedIdeograph4907', [], 'Display', None],
  [ 0x4908, 'CjkUnifiedIdeograph4908', [], 'Display', None],
  [ 0x4909, 'CjkUnifiedIdeograph4909', [], 'Display', None],
  [ 0x490a, 'CjkUnifiedIdeograph490a', [], 'Display', None],
  [ 0x490b, 'CjkUnifiedIdeograph490b', [], 'Display', None],
  [ 0x490c, 'CjkUnifiedIdeograph490c', [], 'Display', None],
  [ 0x490d, 'CjkUnifiedIdeograph490d', [], 'Display', None],
  [ 0x490e, 'CjkUnifiedIdeograph490e', [], 'Display', None],
  [ 0x490f, 'CjkUnifiedIdeograph490f', [], 'Display', None],
  [ 0x4910, 'CjkUnifiedIdeograph4910', [], 'Display', None],
  [ 0x4911, 'CjkUnifiedIdeograph4911', [], 'Display', None],
  [ 0x4912, 'CjkUnifiedIdeograph4912', [], 'Display', None],
  [ 0x4913, 'CjkUnifiedIdeograph4913', [], 'Display', None],
  [ 0x4914, 'CjkUnifiedIdeograph4914', [], 'Display', None],
  [ 0x4915, 'CjkUnifiedIdeograph4915', [], 'Display', None],
  [ 0x4916, 'CjkUnifiedIdeograph4916', [], 'Display', None],
  [ 0x4917, 'CjkUnifiedIdeograph4917', [], 'Display', None],
  [ 0x4918, 'CjkUnifiedIdeograph4918', [], 'Display', None],
  [ 0x4919, 'CjkUnifiedIdeograph4919', [], 'Display', None],
  [ 0x491a, 'CjkUnifiedIdeograph491a', [], 'Display', None],
  [ 0x491b, 'CjkUnifiedIdeograph491b', [], 'Display', None],
  [ 0x491c, 'CjkUnifiedIdeograph491c', [], 'Display', None],
  [ 0x491d, 'CjkUnifiedIdeograph491d', [], 'Display', None],
  [ 0x491e, 'CjkUnifiedIdeograph491e', [], 'Display', None],
  [ 0x491f, 'CjkUnifiedIdeograph491f', [], 'Display', None],
  [ 0x4920, 'CjkUnifiedIdeograph4920', [], 'Display', None],
  [ 0x4921, 'CjkUnifiedIdeograph4921', [], 'Display', None],
  [ 0x4922, 'CjkUnifiedIdeograph4922', [], 'Display', None],
  [ 0x4923, 'CjkUnifiedIdeograph4923', [], 'Display', None],
  [ 0x4924, 'CjkUnifiedIdeograph4924', [], 'Display', None],
  [ 0x4925, 'CjkUnifiedIdeograph4925', [], 'Display', None],
  [ 0x4926, 'CjkUnifiedIdeograph4926', [], 'Display', None],
  [ 0x4927, 'CjkUnifiedIdeograph4927', [], 'Display', None],
  [ 0x4928, 'CjkUnifiedIdeograph4928', [], 'Display', None],
  [ 0x4929, 'CjkUnifiedIdeograph4929', [], 'Display', None],
  [ 0x492a, 'CjkUnifiedIdeograph492a', [], 'Display', None],
  [ 0x492b, 'CjkUnifiedIdeograph492b', [], 'Display', None],
  [ 0x492c, 'CjkUnifiedIdeograph492c', [], 'Display', None],
  [ 0x492d, 'CjkUnifiedIdeograph492d', [], 'Display', None],
  [ 0x492e, 'CjkUnifiedIdeograph492e', [], 'Display', None],
  [ 0x492f, 'CjkUnifiedIdeograph492f', [], 'Display', None],
  [ 0x4930, 'CjkUnifiedIdeograph4930', [], 'Display', None],
  [ 0x4931, 'CjkUnifiedIdeograph4931', [], 'Display', None],
  [ 0x4932, 'CjkUnifiedIdeograph4932', [], 'Display', None],
  [ 0x4933, 'CjkUnifiedIdeograph4933', [], 'Display', None],
  [ 0x4934, 'CjkUnifiedIdeograph4934', [], 'Display', None],
  [ 0x4935, 'CjkUnifiedIdeograph4935', [], 'Display', None],
  [ 0x4936, 'CjkUnifiedIdeograph4936', [], 'Display', None],
  [ 0x4937, 'CjkUnifiedIdeograph4937', [], 'Display', None],
  [ 0x4938, 'CjkUnifiedIdeograph4938', [], 'Display', None],
  [ 0x4939, 'CjkUnifiedIdeograph4939', [], 'Display', None],
  [ 0x493a, 'CjkUnifiedIdeograph493a', [], 'Display', None],
  [ 0x493b, 'CjkUnifiedIdeograph493b', [], 'Display', None],
  [ 0x493c, 'CjkUnifiedIdeograph493c', [], 'Display', None],
  [ 0x493d, 'CjkUnifiedIdeograph493d', [], 'Display', None],
  [ 0x493e, 'CjkUnifiedIdeograph493e', [], 'Display', None],
  [ 0x493f, 'CjkUnifiedIdeograph493f', [], 'Display', None],
  [ 0x4940, 'CjkUnifiedIdeograph4940', [], 'Display', None],
  [ 0x4941, 'CjkUnifiedIdeograph4941', [], 'Display', None],
  [ 0x4942, 'CjkUnifiedIdeograph4942', [], 'Display', None],
  [ 0x4943, 'CjkUnifiedIdeograph4943', [], 'Display', None],
  [ 0x4944, 'CjkUnifiedIdeograph4944', [], 'Display', None],
  [ 0x4945, 'CjkUnifiedIdeograph4945', [], 'Display', None],
  [ 0x4946, 'CjkUnifiedIdeograph4946', [], 'Display', None],
  [ 0x4947, 'CjkUnifiedIdeograph4947', [], 'Display', None],
  [ 0x4948, 'CjkUnifiedIdeograph4948', [], 'Display', None],
  [ 0x4949, 'CjkUnifiedIdeograph4949', [], 'Display', None],
  [ 0x494a, 'CjkUnifiedIdeograph494a', [], 'Display', None],
  [ 0x494b, 'CjkUnifiedIdeograph494b', [], 'Display', None],
  [ 0x494c, 'CjkUnifiedIdeograph494c', [], 'Display', None],
  [ 0x494d, 'CjkUnifiedIdeograph494d', [], 'Display', None],
  [ 0x494e, 'CjkUnifiedIdeograph494e', [], 'Display', None],
  [ 0x494f, 'CjkUnifiedIdeograph494f', [], 'Display', None],
  [ 0x4950, 'CjkUnifiedIdeograph4950', [], 'Display', None],
  [ 0x4951, 'CjkUnifiedIdeograph4951', [], 'Display', None],
  [ 0x4952, 'CjkUnifiedIdeograph4952', [], 'Display', None],
  [ 0x4953, 'CjkUnifiedIdeograph4953', [], 'Display', None],
  [ 0x4954, 'CjkUnifiedIdeograph4954', [], 'Display', None],
  [ 0x4955, 'CjkUnifiedIdeograph4955', [], 'Display', None],
  [ 0x4956, 'CjkUnifiedIdeograph4956', [], 'Display', None],
  [ 0x4957, 'CjkUnifiedIdeograph4957', [], 'Display', None],
  [ 0x4958, 'CjkUnifiedIdeograph4958', [], 'Display', None],
  [ 0x4959, 'CjkUnifiedIdeograph4959', [], 'Display', None],
  [ 0x495a, 'CjkUnifiedIdeograph495a', [], 'Display', None],
  [ 0x495b, 'CjkUnifiedIdeograph495b', [], 'Display', None],
  [ 0x495c, 'CjkUnifiedIdeograph495c', [], 'Display', None],
  [ 0x495d, 'CjkUnifiedIdeograph495d', [], 'Display', None],
  [ 0x495e, 'CjkUnifiedIdeograph495e', [], 'Display', None],
  [ 0x495f, 'CjkUnifiedIdeograph495f', [], 'Display', None],
  [ 0x4960, 'CjkUnifiedIdeograph4960', [], 'Display', None],
  [ 0x4961, 'CjkUnifiedIdeograph4961', [], 'Display', None],
  [ 0x4962, 'CjkUnifiedIdeograph4962', [], 'Display', None],
  [ 0x4963, 'CjkUnifiedIdeograph4963', [], 'Display', None],
  [ 0x4964, 'CjkUnifiedIdeograph4964', [], 'Display', None],
  [ 0x4965, 'CjkUnifiedIdeograph4965', [], 'Display', None],
  [ 0x4966, 'CjkUnifiedIdeograph4966', [], 'Display', None],
  [ 0x4967, 'CjkUnifiedIdeograph4967', [], 'Display', None],
  [ 0x4968, 'CjkUnifiedIdeograph4968', [], 'Display', None],
  [ 0x4969, 'CjkUnifiedIdeograph4969', [], 'Display', None],
  [ 0x496a, 'CjkUnifiedIdeograph496a', [], 'Display', None],
  [ 0x496b, 'CjkUnifiedIdeograph496b', [], 'Display', None],
  [ 0x496c, 'CjkUnifiedIdeograph496c', [], 'Display', None],
  [ 0x496d, 'CjkUnifiedIdeograph496d', [], 'Display', None],
  [ 0x496e, 'CjkUnifiedIdeograph496e', [], 'Display', None],
  [ 0x496f, 'CjkUnifiedIdeograph496f', [], 'Display', None],
  [ 0x4970, 'CjkUnifiedIdeograph4970', [], 'Display', None],
  [ 0x4971, 'CjkUnifiedIdeograph4971', [], 'Display', None],
  [ 0x4972, 'CjkUnifiedIdeograph4972', [], 'Display', None],
  [ 0x4973, 'CjkUnifiedIdeograph4973', [], 'Display', None],
  [ 0x4974, 'CjkUnifiedIdeograph4974', [], 'Display', None],
  [ 0x4975, 'CjkUnifiedIdeograph4975', [], 'Display', None],
  [ 0x4976, 'CjkUnifiedIdeograph4976', [], 'Display', None],
  [ 0x4977, 'CjkUnifiedIdeograph4977', [], 'Display', None],
  [ 0x4978, 'CjkUnifiedIdeograph4978', [], 'Display', None],
  [ 0x4979, 'CjkUnifiedIdeograph4979', [], 'Display', None],
  [ 0x497a, 'CjkUnifiedIdeograph497a', [], 'Display', None],
  [ 0x497b, 'CjkUnifiedIdeograph497b', [], 'Display', None],
  [ 0x497c, 'CjkUnifiedIdeograph497c', [], 'Display', None],
  [ 0x497d, 'CjkUnifiedIdeograph497d', [], 'Display', None],
  [ 0x497e, 'CjkUnifiedIdeograph497e', [], 'Display', None],
  [ 0x497f, 'CjkUnifiedIdeograph497f', [], 'Display', None],
  [ 0x4980, 'CjkUnifiedIdeograph4980', [], 'Display', None],
  [ 0x4981, 'CjkUnifiedIdeograph4981', [], 'Display', None],
  [ 0x4982, 'CjkUnifiedIdeograph4982', [], 'Display', None],
  [ 0x4983, 'CjkUnifiedIdeograph4983', [], 'Display', None],
  [ 0x4984, 'CjkUnifiedIdeograph4984', [], 'Display', None],
  [ 0x4985, 'CjkUnifiedIdeograph4985', [], 'Display', None],
  [ 0x4986, 'CjkUnifiedIdeograph4986', [], 'Display', None],
  [ 0x4987, 'CjkUnifiedIdeograph4987', [], 'Display', None],
  [ 0x4988, 'CjkUnifiedIdeograph4988', [], 'Display', None],
  [ 0x4989, 'CjkUnifiedIdeograph4989', [], 'Display', None],
  [ 0x498a, 'CjkUnifiedIdeograph498a', [], 'Display', None],
  [ 0x498b, 'CjkUnifiedIdeograph498b', [], 'Display', None],
  [ 0x498c, 'CjkUnifiedIdeograph498c', [], 'Display', None],
  [ 0x498d, 'CjkUnifiedIdeograph498d', [], 'Display', None],
  [ 0x498e, 'CjkUnifiedIdeograph498e', [], 'Display', None],
  [ 0x498f, 'CjkUnifiedIdeograph498f', [], 'Display', None],
  [ 0x4990, 'CjkUnifiedIdeograph4990', [], 'Display', None],
  [ 0x4991, 'CjkUnifiedIdeograph4991', [], 'Display', None],
  [ 0x4992, 'CjkUnifiedIdeograph4992', [], 'Display', None],
  [ 0x4993, 'CjkUnifiedIdeograph4993', [], 'Display', None],
  [ 0x4994, 'CjkUnifiedIdeograph4994', [], 'Display', None],
  [ 0x4995, 'CjkUnifiedIdeograph4995', [], 'Display', None],
  [ 0x4996, 'CjkUnifiedIdeograph4996', [], 'Display', None],
  [ 0x4997, 'CjkUnifiedIdeograph4997', [], 'Display', None],
  [ 0x4998, 'CjkUnifiedIdeograph4998', [], 'Display', None],
  [ 0x4999, 'CjkUnifiedIdeograph4999', [], 'Display', None],
  [ 0x499a, 'CjkUnifiedIdeograph499a', [], 'Display', None],
  [ 0x499b, 'CjkUnifiedIdeograph499b', [], 'Display', None],
  [ 0x499c, 'CjkUnifiedIdeograph499c', [], 'Display', None],
  [ 0x499d, 'CjkUnifiedIdeograph499d', [], 'Display', None],
  [ 0x499e, 'CjkUnifiedIdeograph499e', [], 'Display', None],
  [ 0x499f, 'CjkUnifiedIdeograph499f', [], 'Display', None],
  [ 0x49a0, 'CjkUnifiedIdeograph49a0', [], 'Display', None],
  [ 0x49a1, 'CjkUnifiedIdeograph49a1', [], 'Display', None],
  [ 0x49a2, 'CjkUnifiedIdeograph49a2', [], 'Display', None],
  [ 0x49a3, 'CjkUnifiedIdeograph49a3', [], 'Display', None],
  [ 0x49a4, 'CjkUnifiedIdeograph49a4', [], 'Display', None],
  [ 0x49a5, 'CjkUnifiedIdeograph49a5', [], 'Display', None],
  [ 0x49a6, 'CjkUnifiedIdeograph49a6', [], 'Display', None],
  [ 0x49a7, 'CjkUnifiedIdeograph49a7', [], 'Display', None],
  [ 0x49a8, 'CjkUnifiedIdeograph49a8', [], 'Display', None],
  [ 0x49a9, 'CjkUnifiedIdeograph49a9', [], 'Display', None],
  [ 0x49aa, 'CjkUnifiedIdeograph49aa', [], 'Display', None],
  [ 0x49ab, 'CjkUnifiedIdeograph49ab', [], 'Display', None],
  [ 0x49ac, 'CjkUnifiedIdeograph49ac', [], 'Display', None],
  [ 0x49ad, 'CjkUnifiedIdeograph49ad', [], 'Display', None],
  [ 0x49ae, 'CjkUnifiedIdeograph49ae', [], 'Display', None],
  [ 0x49af, 'CjkUnifiedIdeograph49af', [], 'Display', None],
  [ 0x49b0, 'CjkUnifiedIdeograph49b0', [], 'Display', None],
  [ 0x49b1, 'CjkUnifiedIdeograph49b1', [], 'Display', None],
  [ 0x49b2, 'CjkUnifiedIdeograph49b2', [], 'Display', None],
  [ 0x49b3, 'CjkUnifiedIdeograph49b3', [], 'Display', None],
  [ 0x49b4, 'CjkUnifiedIdeograph49b4', [], 'Display', None],
  [ 0x49b5, 'CjkUnifiedIdeograph49b5', [], 'Display', None],
  [ 0x49b6, 'CjkUnifiedIdeograph49b6', [], 'Display', None],
  [ 0x49b7, 'CjkUnifiedIdeograph49b7', [], 'Display', None],
  [ 0x49b8, 'CjkUnifiedIdeograph49b8', [], 'Display', None],
  [ 0x49b9, 'CjkUnifiedIdeograph49b9', [], 'Display', None],
  [ 0x49ba, 'CjkUnifiedIdeograph49ba', [], 'Display', None],
  [ 0x49bb, 'CjkUnifiedIdeograph49bb', [], 'Display', None],
  [ 0x49bc, 'CjkUnifiedIdeograph49bc', [], 'Display', None],
  [ 0x49bd, 'CjkUnifiedIdeograph49bd', [], 'Display', None],
  [ 0x49be, 'CjkUnifiedIdeograph49be', [], 'Display', None],
  [ 0x49bf, 'CjkUnifiedIdeograph49bf', [], 'Display', None],
  [ 0x49c0, 'CjkUnifiedIdeograph49c0', [], 'Display', None],
  [ 0x49c1, 'CjkUnifiedIdeograph49c1', [], 'Display', None],
  [ 0x49c2, 'CjkUnifiedIdeograph49c2', [], 'Display', None],
  [ 0x49c3, 'CjkUnifiedIdeograph49c3', [], 'Display', None],
  [ 0x49c4, 'CjkUnifiedIdeograph49c4', [], 'Display', None],
  [ 0x49c5, 'CjkUnifiedIdeograph49c5', [], 'Display', None],
  [ 0x49c6, 'CjkUnifiedIdeograph49c6', [], 'Display', None],
  [ 0x49c7, 'CjkUnifiedIdeograph49c7', [], 'Display', None],
  [ 0x49c8, 'CjkUnifiedIdeograph49c8', [], 'Display', None],
  [ 0x49c9, 'CjkUnifiedIdeograph49c9', [], 'Display', None],
  [ 0x49ca, 'CjkUnifiedIdeograph49ca', [], 'Display', None],
  [ 0x49cb, 'CjkUnifiedIdeograph49cb', [], 'Display', None],
  [ 0x49cc, 'CjkUnifiedIdeograph49cc', [], 'Display', None],
  [ 0x49cd, 'CjkUnifiedIdeograph49cd', [], 'Display', None],
  [ 0x49ce, 'CjkUnifiedIdeograph49ce', [], 'Display', None],
  [ 0x49cf, 'CjkUnifiedIdeograph49cf', [], 'Display', None],
  [ 0x49d0, 'CjkUnifiedIdeograph49d0', [], 'Display', None],
  [ 0x49d1, 'CjkUnifiedIdeograph49d1', [], 'Display', None],
  [ 0x49d2, 'CjkUnifiedIdeograph49d2', [], 'Display', None],
  [ 0x49d3, 'CjkUnifiedIdeograph49d3', [], 'Display', None],
  [ 0x49d4, 'CjkUnifiedIdeograph49d4', [], 'Display', None],
  [ 0x49d5, 'CjkUnifiedIdeograph49d5', [], 'Display', None],
  [ 0x49d6, 'CjkUnifiedIdeograph49d6', [], 'Display', None],
  [ 0x49d7, 'CjkUnifiedIdeograph49d7', [], 'Display', None],
  [ 0x49d8, 'CjkUnifiedIdeograph49d8', [], 'Display', None],
  [ 0x49d9, 'CjkUnifiedIdeograph49d9', [], 'Display', None],
  [ 0x49da, 'CjkUnifiedIdeograph49da', [], 'Display', None],
  [ 0x49db, 'CjkUnifiedIdeograph49db', [], 'Display', None],
  [ 0x49dc, 'CjkUnifiedIdeograph49dc', [], 'Display', None],
  [ 0x49dd, 'CjkUnifiedIdeograph49dd', [], 'Display', None],
  [ 0x49de, 'CjkUnifiedIdeograph49de', [], 'Display', None],
  [ 0x49df, 'CjkUnifiedIdeograph49df', [], 'Display', None],
  [ 0x49e0, 'CjkUnifiedIdeograph49e0', [], 'Display', None],
  [ 0x49e1, 'CjkUnifiedIdeograph49e1', [], 'Display', None],
  [ 0x49e2, 'CjkUnifiedIdeograph49e2', [], 'Display', None],
  [ 0x49e3, 'CjkUnifiedIdeograph49e3', [], 'Display', None],
  [ 0x49e4, 'CjkUnifiedIdeograph49e4', [], 'Display', None],
  [ 0x49e5, 'CjkUnifiedIdeograph49e5', [], 'Display', None],
  [ 0x49e6, 'CjkUnifiedIdeograph49e6', [], 'Display', None],
  [ 0x49e7, 'CjkUnifiedIdeograph49e7', [], 'Display', None],
  [ 0x49e8, 'CjkUnifiedIdeograph49e8', [], 'Display', None],
  [ 0x49e9, 'CjkUnifiedIdeograph49e9', [], 'Display', None],
  [ 0x49ea, 'CjkUnifiedIdeograph49ea', [], 'Display', None],
  [ 0x49eb, 'CjkUnifiedIdeograph49eb', [], 'Display', None],
  [ 0x49ec, 'CjkUnifiedIdeograph49ec', [], 'Display', None],
  [ 0x49ed, 'CjkUnifiedIdeograph49ed', [], 'Display', None],
  [ 0x49ee, 'CjkUnifiedIdeograph49ee', [], 'Display', None],
  [ 0x49ef, 'CjkUnifiedIdeograph49ef', [], 'Display', None],
  [ 0x49f0, 'CjkUnifiedIdeograph49f0', [], 'Display', None],
  [ 0x49f1, 'CjkUnifiedIdeograph49f1', [], 'Display', None],
  [ 0x49f2, 'CjkUnifiedIdeograph49f2', [], 'Display', None],
  [ 0x49f3, 'CjkUnifiedIdeograph49f3', [], 'Display', None],
  [ 0x49f4, 'CjkUnifiedIdeograph49f4', [], 'Display', None],
  [ 0x49f5, 'CjkUnifiedIdeograph49f5', [], 'Display', None],
  [ 0x49f6, 'CjkUnifiedIdeograph49f6', [], 'Display', None],
  [ 0x49f7, 'CjkUnifiedIdeograph49f7', [], 'Display', None],
  [ 0x49f8, 'CjkUnifiedIdeograph49f8', [], 'Display', None],
  [ 0x49f9, 'CjkUnifiedIdeograph49f9', [], 'Display', None],
  [ 0x49fa, 'CjkUnifiedIdeograph49fa', [], 'Display', None],
  [ 0x49fb, 'CjkUnifiedIdeograph49fb', [], 'Display', None],
  [ 0x49fc, 'CjkUnifiedIdeograph49fc', [], 'Display', None],
  [ 0x49fd, 'CjkUnifiedIdeograph49fd', [], 'Display', None],
  [ 0x49fe, 'CjkUnifiedIdeograph49fe', [], 'Display', None],
  [ 0x49ff, 'CjkUnifiedIdeograph49ff', [], 'Display', None],
  [ 0x4a00, 'CjkUnifiedIdeograph4a00', [], 'Display', None],
  [ 0x4a01, 'CjkUnifiedIdeograph4a01', [], 'Display', None],
  [ 0x4a02, 'CjkUnifiedIdeograph4a02', [], 'Display', None],
  [ 0x4a03, 'CjkUnifiedIdeograph4a03', [], 'Display', None],
  [ 0x4a04, 'CjkUnifiedIdeograph4a04', [], 'Display', None],
  [ 0x4a05, 'CjkUnifiedIdeograph4a05', [], 'Display', None],
  [ 0x4a06, 'CjkUnifiedIdeograph4a06', [], 'Display', None],
  [ 0x4a07, 'CjkUnifiedIdeograph4a07', [], 'Display', None],
  [ 0x4a08, 'CjkUnifiedIdeograph4a08', [], 'Display', None],
  [ 0x4a09, 'CjkUnifiedIdeograph4a09', [], 'Display', None],
  [ 0x4a0a, 'CjkUnifiedIdeograph4a0a', [], 'Display', None],
  [ 0x4a0b, 'CjkUnifiedIdeograph4a0b', [], 'Display', None],
  [ 0x4a0c, 'CjkUnifiedIdeograph4a0c', [], 'Display', None],
  [ 0x4a0d, 'CjkUnifiedIdeograph4a0d', [], 'Display', None],
  [ 0x4a0e, 'CjkUnifiedIdeograph4a0e', [], 'Display', None],
  [ 0x4a0f, 'CjkUnifiedIdeograph4a0f', [], 'Display', None],
  [ 0x4a10, 'CjkUnifiedIdeograph4a10', [], 'Display', None],
  [ 0x4a11, 'CjkUnifiedIdeograph4a11', [], 'Display', None],
  [ 0x4a12, 'CjkUnifiedIdeograph4a12', [], 'Display', None],
  [ 0x4a13, 'CjkUnifiedIdeograph4a13', [], 'Display', None],
  [ 0x4a14, 'CjkUnifiedIdeograph4a14', [], 'Display', None],
  [ 0x4a15, 'CjkUnifiedIdeograph4a15', [], 'Display', None],
  [ 0x4a16, 'CjkUnifiedIdeograph4a16', [], 'Display', None],
  [ 0x4a17, 'CjkUnifiedIdeograph4a17', [], 'Display', None],
  [ 0x4a18, 'CjkUnifiedIdeograph4a18', [], 'Display', None],
  [ 0x4a19, 'CjkUnifiedIdeograph4a19', [], 'Display', None],
  [ 0x4a1a, 'CjkUnifiedIdeograph4a1a', [], 'Display', None],
  [ 0x4a1b, 'CjkUnifiedIdeograph4a1b', [], 'Display', None],
  [ 0x4a1c, 'CjkUnifiedIdeograph4a1c', [], 'Display', None],
  [ 0x4a1d, 'CjkUnifiedIdeograph4a1d', [], 'Display', None],
  [ 0x4a1e, 'CjkUnifiedIdeograph4a1e', [], 'Display', None],
  [ 0x4a1f, 'CjkUnifiedIdeograph4a1f', [], 'Display', None],
  [ 0x4a20, 'CjkUnifiedIdeograph4a20', [], 'Display', None],
  [ 0x4a21, 'CjkUnifiedIdeograph4a21', [], 'Display', None],
  [ 0x4a22, 'CjkUnifiedIdeograph4a22', [], 'Display', None],
  [ 0x4a23, 'CjkUnifiedIdeograph4a23', [], 'Display', None],
  [ 0x4a24, 'CjkUnifiedIdeograph4a24', [], 'Display', None],
  [ 0x4a25, 'CjkUnifiedIdeograph4a25', [], 'Display', None],
  [ 0x4a26, 'CjkUnifiedIdeograph4a26', [], 'Display', None],
  [ 0x4a27, 'CjkUnifiedIdeograph4a27', [], 'Display', None],
  [ 0x4a28, 'CjkUnifiedIdeograph4a28', [], 'Display', None],
  [ 0x4a29, 'CjkUnifiedIdeograph4a29', [], 'Display', None],
  [ 0x4a2a, 'CjkUnifiedIdeograph4a2a', [], 'Display', None],
  [ 0x4a2b, 'CjkUnifiedIdeograph4a2b', [], 'Display', None],
  [ 0x4a2c, 'CjkUnifiedIdeograph4a2c', [], 'Display', None],
  [ 0x4a2d, 'CjkUnifiedIdeograph4a2d', [], 'Display', None],
  [ 0x4a2e, 'CjkUnifiedIdeograph4a2e', [], 'Display', None],
  [ 0x4a2f, 'CjkUnifiedIdeograph4a2f', [], 'Display', None],
  [ 0x4a30, 'CjkUnifiedIdeograph4a30', [], 'Display', None],
  [ 0x4a31, 'CjkUnifiedIdeograph4a31', [], 'Display', None],
  [ 0x4a32, 'CjkUnifiedIdeograph4a32', [], 'Display', None],
  [ 0x4a33, 'CjkUnifiedIdeograph4a33', [], 'Display', None],
  [ 0x4a34, 'CjkUnifiedIdeograph4a34', [], 'Display', None],
  [ 0x4a35, 'CjkUnifiedIdeograph4a35', [], 'Display', None],
  [ 0x4a36, 'CjkUnifiedIdeograph4a36', [], 'Display', None],
  [ 0x4a37, 'CjkUnifiedIdeograph4a37', [], 'Display', None],
  [ 0x4a38, 'CjkUnifiedIdeograph4a38', [], 'Display', None],
  [ 0x4a39, 'CjkUnifiedIdeograph4a39', [], 'Display', None],
  [ 0x4a3a, 'CjkUnifiedIdeograph4a3a', [], 'Display', None],
  [ 0x4a3b, 'CjkUnifiedIdeograph4a3b', [], 'Display', None],
  [ 0x4a3c, 'CjkUnifiedIdeograph4a3c', [], 'Display', None],
  [ 0x4a3d, 'CjkUnifiedIdeograph4a3d', [], 'Display', None],
  [ 0x4a3e, 'CjkUnifiedIdeograph4a3e', [], 'Display', None],
  [ 0x4a3f, 'CjkUnifiedIdeograph4a3f', [], 'Display', None],
  [ 0x4a40, 'CjkUnifiedIdeograph4a40', [], 'Display', None],
  [ 0x4a41, 'CjkUnifiedIdeograph4a41', [], 'Display', None],
  [ 0x4a42, 'CjkUnifiedIdeograph4a42', [], 'Display', None],
  [ 0x4a43, 'CjkUnifiedIdeograph4a43', [], 'Display', None],
  [ 0x4a44, 'CjkUnifiedIdeograph4a44', [], 'Display', None],
  [ 0x4a45, 'CjkUnifiedIdeograph4a45', [], 'Display', None],
  [ 0x4a46, 'CjkUnifiedIdeograph4a46', [], 'Display', None],
  [ 0x4a47, 'CjkUnifiedIdeograph4a47', [], 'Display', None],
  [ 0x4a48, 'CjkUnifiedIdeograph4a48', [], 'Display', None],
  [ 0x4a49, 'CjkUnifiedIdeograph4a49', [], 'Display', None],
  [ 0x4a4a, 'CjkUnifiedIdeograph4a4a', [], 'Display', None],
  [ 0x4a4b, 'CjkUnifiedIdeograph4a4b', [], 'Display', None],
  [ 0x4a4c, 'CjkUnifiedIdeograph4a4c', [], 'Display', None],
  [ 0x4a4d, 'CjkUnifiedIdeograph4a4d', [], 'Display', None],
  [ 0x4a4e, 'CjkUnifiedIdeograph4a4e', [], 'Display', None],
  [ 0x4a4f, 'CjkUnifiedIdeograph4a4f', [], 'Display', None],
  [ 0x4a50, 'CjkUnifiedIdeograph4a50', [], 'Display', None],
  [ 0x4a51, 'CjkUnifiedIdeograph4a51', [], 'Display', None],
  [ 0x4a52, 'CjkUnifiedIdeograph4a52', [], 'Display', None],
  [ 0x4a53, 'CjkUnifiedIdeograph4a53', [], 'Display', None],
  [ 0x4a54, 'CjkUnifiedIdeograph4a54', [], 'Display', None],
  [ 0x4a55, 'CjkUnifiedIdeograph4a55', [], 'Display', None],
  [ 0x4a56, 'CjkUnifiedIdeograph4a56', [], 'Display', None],
  [ 0x4a57, 'CjkUnifiedIdeograph4a57', [], 'Display', None],
  [ 0x4a58, 'CjkUnifiedIdeograph4a58', [], 'Display', None],
  [ 0x4a59, 'CjkUnifiedIdeograph4a59', [], 'Display', None],
  [ 0x4a5a, 'CjkUnifiedIdeograph4a5a', [], 'Display', None],
  [ 0x4a5b, 'CjkUnifiedIdeograph4a5b', [], 'Display', None],
  [ 0x4a5c, 'CjkUnifiedIdeograph4a5c', [], 'Display', None],
  [ 0x4a5d, 'CjkUnifiedIdeograph4a5d', [], 'Display', None],
  [ 0x4a5e, 'CjkUnifiedIdeograph4a5e', [], 'Display', None],
  [ 0x4a5f, 'CjkUnifiedIdeograph4a5f', [], 'Display', None],
  [ 0x4a60, 'CjkUnifiedIdeograph4a60', [], 'Display', None],
  [ 0x4a61, 'CjkUnifiedIdeograph4a61', [], 'Display', None],
  [ 0x4a62, 'CjkUnifiedIdeograph4a62', [], 'Display', None],
  [ 0x4a63, 'CjkUnifiedIdeograph4a63', [], 'Display', None],
  [ 0x4a64, 'CjkUnifiedIdeograph4a64', [], 'Display', None],
  [ 0x4a65, 'CjkUnifiedIdeograph4a65', [], 'Display', None],
  [ 0x4a66, 'CjkUnifiedIdeograph4a66', [], 'Display', None],
  [ 0x4a67, 'CjkUnifiedIdeograph4a67', [], 'Display', None],
  [ 0x4a68, 'CjkUnifiedIdeograph4a68', [], 'Display', None],
  [ 0x4a69, 'CjkUnifiedIdeograph4a69', [], 'Display', None],
  [ 0x4a6a, 'CjkUnifiedIdeograph4a6a', [], 'Display', None],
  [ 0x4a6b, 'CjkUnifiedIdeograph4a6b', [], 'Display', None],
  [ 0x4a6c, 'CjkUnifiedIdeograph4a6c', [], 'Display', None],
  [ 0x4a6d, 'CjkUnifiedIdeograph4a6d', [], 'Display', None],
  [ 0x4a6e, 'CjkUnifiedIdeograph4a6e', [], 'Display', None],
  [ 0x4a6f, 'CjkUnifiedIdeograph4a6f', [], 'Display', None],
  [ 0x4a70, 'CjkUnifiedIdeograph4a70', [], 'Display', None],
  [ 0x4a71, 'CjkUnifiedIdeograph4a71', [], 'Display', None],
  [ 0x4a72, 'CjkUnifiedIdeograph4a72', [], 'Display', None],
  [ 0x4a73, 'CjkUnifiedIdeograph4a73', [], 'Display', None],
  [ 0x4a74, 'CjkUnifiedIdeograph4a74', [], 'Display', None],
  [ 0x4a75, 'CjkUnifiedIdeograph4a75', [], 'Display', None],
  [ 0x4a76, 'CjkUnifiedIdeograph4a76', [], 'Display', None],
  [ 0x4a77, 'CjkUnifiedIdeograph4a77', [], 'Display', None],
  [ 0x4a78, 'CjkUnifiedIdeograph4a78', [], 'Display', None],
  [ 0x4a79, 'CjkUnifiedIdeograph4a79', [], 'Display', None],
  [ 0x4a7a, 'CjkUnifiedIdeograph4a7a', [], 'Display', None],
  [ 0x4a7b, 'CjkUnifiedIdeograph4a7b', [], 'Display', None],
  [ 0x4a7c, 'CjkUnifiedIdeograph4a7c', [], 'Display', None],
  [ 0x4a7d, 'CjkUnifiedIdeograph4a7d', [], 'Display', None],
  [ 0x4a7e, 'CjkUnifiedIdeograph4a7e', [], 'Display', None],
  [ 0x4a7f, 'CjkUnifiedIdeograph4a7f', [], 'Display', None],
  [ 0x4a80, 'CjkUnifiedIdeograph4a80', [], 'Display', None],
  [ 0x4a81, 'CjkUnifiedIdeograph4a81', [], 'Display', None],
  [ 0x4a82, 'CjkUnifiedIdeograph4a82', [], 'Display', None],
  [ 0x4a83, 'CjkUnifiedIdeograph4a83', [], 'Display', None],
  [ 0x4a84, 'CjkUnifiedIdeograph4a84', [], 'Display', None],
  [ 0x4a85, 'CjkUnifiedIdeograph4a85', [], 'Display', None],
  [ 0x4a86, 'CjkUnifiedIdeograph4a86', [], 'Display', None],
  [ 0x4a87, 'CjkUnifiedIdeograph4a87', [], 'Display', None],
  [ 0x4a88, 'CjkUnifiedIdeograph4a88', [], 'Display', None],
  [ 0x4a89, 'CjkUnifiedIdeograph4a89', [], 'Display', None],
  [ 0x4a8a, 'CjkUnifiedIdeograph4a8a', [], 'Display', None],
  [ 0x4a8b, 'CjkUnifiedIdeograph4a8b', [], 'Display', None],
  [ 0x4a8c, 'CjkUnifiedIdeograph4a8c', [], 'Display', None],
  [ 0x4a8d, 'CjkUnifiedIdeograph4a8d', [], 'Display', None],
  [ 0x4a8e, 'CjkUnifiedIdeograph4a8e', [], 'Display', None],
  [ 0x4a8f, 'CjkUnifiedIdeograph4a8f', [], 'Display', None],
  [ 0x4a90, 'CjkUnifiedIdeograph4a90', [], 'Display', None],
  [ 0x4a91, 'CjkUnifiedIdeograph4a91', [], 'Display', None],
  [ 0x4a92, 'CjkUnifiedIdeograph4a92', [], 'Display', None],
  [ 0x4a93, 'CjkUnifiedIdeograph4a93', [], 'Display', None],
  [ 0x4a94, 'CjkUnifiedIdeograph4a94', [], 'Display', None],
  [ 0x4a95, 'CjkUnifiedIdeograph4a95', [], 'Display', None],
  [ 0x4a96, 'CjkUnifiedIdeograph4a96', [], 'Display', None],
  [ 0x4a97, 'CjkUnifiedIdeograph4a97', [], 'Display', None],
  [ 0x4a98, 'CjkUnifiedIdeograph4a98', [], 'Display', None],
  [ 0x4a99, 'CjkUnifiedIdeograph4a99', [], 'Display', None],
  [ 0x4a9a, 'CjkUnifiedIdeograph4a9a', [], 'Display', None],
  [ 0x4a9b, 'CjkUnifiedIdeograph4a9b', [], 'Display', None],
  [ 0x4a9c, 'CjkUnifiedIdeograph4a9c', [], 'Display', None],
  [ 0x4a9d, 'CjkUnifiedIdeograph4a9d', [], 'Display', None],
  [ 0x4a9e, 'CjkUnifiedIdeograph4a9e', [], 'Display', None],
  [ 0x4a9f, 'CjkUnifiedIdeograph4a9f', [], 'Display', None],
  [ 0x4aa0, 'CjkUnifiedIdeograph4aa0', [], 'Display', None],
  [ 0x4aa1, 'CjkUnifiedIdeograph4aa1', [], 'Display', None],
  [ 0x4aa2, 'CjkUnifiedIdeograph4aa2', [], 'Display', None],
  [ 0x4aa3, 'CjkUnifiedIdeograph4aa3', [], 'Display', None],
  [ 0x4aa4, 'CjkUnifiedIdeograph4aa4', [], 'Display', None],
  [ 0x4aa5, 'CjkUnifiedIdeograph4aa5', [], 'Display', None],
  [ 0x4aa6, 'CjkUnifiedIdeograph4aa6', [], 'Display', None],
  [ 0x4aa7, 'CjkUnifiedIdeograph4aa7', [], 'Display', None],
  [ 0x4aa8, 'CjkUnifiedIdeograph4aa8', [], 'Display', None],
  [ 0x4aa9, 'CjkUnifiedIdeograph4aa9', [], 'Display', None],
  [ 0x4aaa, 'CjkUnifiedIdeograph4aaa', [], 'Display', None],
  [ 0x4aab, 'CjkUnifiedIdeograph4aab', [], 'Display', None],
  [ 0x4aac, 'CjkUnifiedIdeograph4aac', [], 'Display', None],
  [ 0x4aad, 'CjkUnifiedIdeograph4aad', [], 'Display', None],
  [ 0x4aae, 'CjkUnifiedIdeograph4aae', [], 'Display', None],
  [ 0x4aaf, 'CjkUnifiedIdeograph4aaf', [], 'Display', None],
  [ 0x4ab0, 'CjkUnifiedIdeograph4ab0', [], 'Display', None],
  [ 0x4ab1, 'CjkUnifiedIdeograph4ab1', [], 'Display', None],
  [ 0x4ab2, 'CjkUnifiedIdeograph4ab2', [], 'Display', None],
  [ 0x4ab3, 'CjkUnifiedIdeograph4ab3', [], 'Display', None],
  [ 0x4ab4, 'CjkUnifiedIdeograph4ab4', [], 'Display', None],
  [ 0x4ab5, 'CjkUnifiedIdeograph4ab5', [], 'Display', None],
  [ 0x4ab6, 'CjkUnifiedIdeograph4ab6', [], 'Display', None],
  [ 0x4ab7, 'CjkUnifiedIdeograph4ab7', [], 'Display', None],
  [ 0x4ab8, 'CjkUnifiedIdeograph4ab8', [], 'Display', None],
  [ 0x4ab9, 'CjkUnifiedIdeograph4ab9', [], 'Display', None],
  [ 0x4aba, 'CjkUnifiedIdeograph4aba', [], 'Display', None],
  [ 0x4abb, 'CjkUnifiedIdeograph4abb', [], 'Display', None],
  [ 0x4abc, 'CjkUnifiedIdeograph4abc', [], 'Display', None],
  [ 0x4abd, 'CjkUnifiedIdeograph4abd', [], 'Display', None],
  [ 0x4abe, 'CjkUnifiedIdeograph4abe', [], 'Display', None],
  [ 0x4abf, 'CjkUnifiedIdeograph4abf', [], 'Display', None],
  [ 0x4ac0, 'CjkUnifiedIdeograph4ac0', [], 'Display', None],
  [ 0x4ac1, 'CjkUnifiedIdeograph4ac1', [], 'Display', None],
  [ 0x4ac2, 'CjkUnifiedIdeograph4ac2', [], 'Display', None],
  [ 0x4ac3, 'CjkUnifiedIdeograph4ac3', [], 'Display', None],
  [ 0x4ac4, 'CjkUnifiedIdeograph4ac4', [], 'Display', None],
  [ 0x4ac5, 'CjkUnifiedIdeograph4ac5', [], 'Display', None],
  [ 0x4ac6, 'CjkUnifiedIdeograph4ac6', [], 'Display', None],
  [ 0x4ac7, 'CjkUnifiedIdeograph4ac7', [], 'Display', None],
  [ 0x4ac8, 'CjkUnifiedIdeograph4ac8', [], 'Display', None],
  [ 0x4ac9, 'CjkUnifiedIdeograph4ac9', [], 'Display', None],
  [ 0x4aca, 'CjkUnifiedIdeograph4aca', [], 'Display', None],
  [ 0x4acb, 'CjkUnifiedIdeograph4acb', [], 'Display', None],
  [ 0x4acc, 'CjkUnifiedIdeograph4acc', [], 'Display', None],
  [ 0x4acd, 'CjkUnifiedIdeograph4acd', [], 'Display', None],
  [ 0x4ace, 'CjkUnifiedIdeograph4ace', [], 'Display', None],
  [ 0x4acf, 'CjkUnifiedIdeograph4acf', [], 'Display', None],
  [ 0x4ad0, 'CjkUnifiedIdeograph4ad0', [], 'Display', None],
  [ 0x4ad1, 'CjkUnifiedIdeograph4ad1', [], 'Display', None],
  [ 0x4ad2, 'CjkUnifiedIdeograph4ad2', [], 'Display', None],
  [ 0x4ad3, 'CjkUnifiedIdeograph4ad3', [], 'Display', None],
  [ 0x4ad4, 'CjkUnifiedIdeograph4ad4', [], 'Display', None],
  [ 0x4ad5, 'CjkUnifiedIdeograph4ad5', [], 'Display', None],
  [ 0x4ad6, 'CjkUnifiedIdeograph4ad6', [], 'Display', None],
  [ 0x4ad7, 'CjkUnifiedIdeograph4ad7', [], 'Display', None],
  [ 0x4ad8, 'CjkUnifiedIdeograph4ad8', [], 'Display', None],
  [ 0x4ad9, 'CjkUnifiedIdeograph4ad9', [], 'Display', None],
  [ 0x4ada, 'CjkUnifiedIdeograph4ada', [], 'Display', None],
  [ 0x4adb, 'CjkUnifiedIdeograph4adb', [], 'Display', None],
  [ 0x4adc, 'CjkUnifiedIdeograph4adc', [], 'Display', None],
  [ 0x4add, 'CjkUnifiedIdeograph4add', [], 'Display', None],
  [ 0x4ade, 'CjkUnifiedIdeograph4ade', [], 'Display', None],
  [ 0x4adf, 'CjkUnifiedIdeograph4adf', [], 'Display', None],
  [ 0x4ae0, 'CjkUnifiedIdeograph4ae0', [], 'Display', None],
  [ 0x4ae1, 'CjkUnifiedIdeograph4ae1', [], 'Display', None],
  [ 0x4ae2, 'CjkUnifiedIdeograph4ae2', [], 'Display', None],
  [ 0x4ae3, 'CjkUnifiedIdeograph4ae3', [], 'Display', None],
  [ 0x4ae4, 'CjkUnifiedIdeograph4ae4', [], 'Display', None],
  [ 0x4ae5, 'CjkUnifiedIdeograph4ae5', [], 'Display', None],
  [ 0x4ae6, 'CjkUnifiedIdeograph4ae6', [], 'Display', None],
  [ 0x4ae7, 'CjkUnifiedIdeograph4ae7', [], 'Display', None],
  [ 0x4ae8, 'CjkUnifiedIdeograph4ae8', [], 'Display', None],
  [ 0x4ae9, 'CjkUnifiedIdeograph4ae9', [], 'Display', None],
  [ 0x4aea, 'CjkUnifiedIdeograph4aea', [], 'Display', None],
  [ 0x4aeb, 'CjkUnifiedIdeograph4aeb', [], 'Display', None],
  [ 0x4aec, 'CjkUnifiedIdeograph4aec', [], 'Display', None],
  [ 0x4aed, 'CjkUnifiedIdeograph4aed', [], 'Display', None],
  [ 0x4aee, 'CjkUnifiedIdeograph4aee', [], 'Display', None],
  [ 0x4aef, 'CjkUnifiedIdeograph4aef', [], 'Display', None],
  [ 0x4af0, 'CjkUnifiedIdeograph4af0', [], 'Display', None],
  [ 0x4af1, 'CjkUnifiedIdeograph4af1', [], 'Display', None],
  [ 0x4af2, 'CjkUnifiedIdeograph4af2', [], 'Display', None],
  [ 0x4af3, 'CjkUnifiedIdeograph4af3', [], 'Display', None],
  [ 0x4af4, 'CjkUnifiedIdeograph4af4', [], 'Display', None],
  [ 0x4af5, 'CjkUnifiedIdeograph4af5', [], 'Display', None],
  [ 0x4af6, 'CjkUnifiedIdeograph4af6', [], 'Display', None],
  [ 0x4af7, 'CjkUnifiedIdeograph4af7', [], 'Display', None],
  [ 0x4af8, 'CjkUnifiedIdeograph4af8', [], 'Display', None],
  [ 0x4af9, 'CjkUnifiedIdeograph4af9', [], 'Display', None],
  [ 0x4afa, 'CjkUnifiedIdeograph4afa', [], 'Display', None],
  [ 0x4afb, 'CjkUnifiedIdeograph4afb', [], 'Display', None],
  [ 0x4afc, 'CjkUnifiedIdeograph4afc', [], 'Display', None],
  [ 0x4afd, 'CjkUnifiedIdeograph4afd', [], 'Display', None],
  [ 0x4afe, 'CjkUnifiedIdeograph4afe', [], 'Display', None],
  [ 0x4aff, 'CjkUnifiedIdeograph4aff', [], 'Display', None],
  [ 0x4b00, 'CjkUnifiedIdeograph4b00', [], 'Display', None],
  [ 0x4b01, 'CjkUnifiedIdeograph4b01', [], 'Display', None],
  [ 0x4b02, 'CjkUnifiedIdeograph4b02', [], 'Display', None],
  [ 0x4b03, 'CjkUnifiedIdeograph4b03', [], 'Display', None],
  [ 0x4b04, 'CjkUnifiedIdeograph4b04', [], 'Display', None],
  [ 0x4b05, 'CjkUnifiedIdeograph4b05', [], 'Display', None],
  [ 0x4b06, 'CjkUnifiedIdeograph4b06', [], 'Display', None],
  [ 0x4b07, 'CjkUnifiedIdeograph4b07', [], 'Display', None],
  [ 0x4b08, 'CjkUnifiedIdeograph4b08', [], 'Display', None],
  [ 0x4b09, 'CjkUnifiedIdeograph4b09', [], 'Display', None],
  [ 0x4b0a, 'CjkUnifiedIdeograph4b0a', [], 'Display', None],
  [ 0x4b0b, 'CjkUnifiedIdeograph4b0b', [], 'Display', None],
  [ 0x4b0c, 'CjkUnifiedIdeograph4b0c', [], 'Display', None],
  [ 0x4b0d, 'CjkUnifiedIdeograph4b0d', [], 'Display', None],
  [ 0x4b0e, 'CjkUnifiedIdeograph4b0e', [], 'Display', None],
  [ 0x4b0f, 'CjkUnifiedIdeograph4b0f', [], 'Display', None],
  [ 0x4b10, 'CjkUnifiedIdeograph4b10', [], 'Display', None],
  [ 0x4b11, 'CjkUnifiedIdeograph4b11', [], 'Display', None],
  [ 0x4b12, 'CjkUnifiedIdeograph4b12', [], 'Display', None],
  [ 0x4b13, 'CjkUnifiedIdeograph4b13', [], 'Display', None],
  [ 0x4b14, 'CjkUnifiedIdeograph4b14', [], 'Display', None],
  [ 0x4b15, 'CjkUnifiedIdeograph4b15', [], 'Display', None],
  [ 0x4b16, 'CjkUnifiedIdeograph4b16', [], 'Display', None],
  [ 0x4b17, 'CjkUnifiedIdeograph4b17', [], 'Display', None],
  [ 0x4b18, 'CjkUnifiedIdeograph4b18', [], 'Display', None],
  [ 0x4b19, 'CjkUnifiedIdeograph4b19', [], 'Display', None],
  [ 0x4b1a, 'CjkUnifiedIdeograph4b1a', [], 'Display', None],
  [ 0x4b1b, 'CjkUnifiedIdeograph4b1b', [], 'Display', None],
  [ 0x4b1c, 'CjkUnifiedIdeograph4b1c', [], 'Display', None],
  [ 0x4b1d, 'CjkUnifiedIdeograph4b1d', [], 'Display', None],
  [ 0x4b1e, 'CjkUnifiedIdeograph4b1e', [], 'Display', None],
  [ 0x4b1f, 'CjkUnifiedIdeograph4b1f', [], 'Display', None],
  [ 0x4b20, 'CjkUnifiedIdeograph4b20', [], 'Display', None],
  [ 0x4b21, 'CjkUnifiedIdeograph4b21', [], 'Display', None],
  [ 0x4b22, 'CjkUnifiedIdeograph4b22', [], 'Display', None],
  [ 0x4b23, 'CjkUnifiedIdeograph4b23', [], 'Display', None],
  [ 0x4b24, 'CjkUnifiedIdeograph4b24', [], 'Display', None],
  [ 0x4b25, 'CjkUnifiedIdeograph4b25', [], 'Display', None],
  [ 0x4b26, 'CjkUnifiedIdeograph4b26', [], 'Display', None],
  [ 0x4b27, 'CjkUnifiedIdeograph4b27', [], 'Display', None],
  [ 0x4b28, 'CjkUnifiedIdeograph4b28', [], 'Display', None],
  [ 0x4b29, 'CjkUnifiedIdeograph4b29', [], 'Display', None],
  [ 0x4b2a, 'CjkUnifiedIdeograph4b2a', [], 'Display', None],
  [ 0x4b2b, 'CjkUnifiedIdeograph4b2b', [], 'Display', None],
  [ 0x4b2c, 'CjkUnifiedIdeograph4b2c', [], 'Display', None],
  [ 0x4b2d, 'CjkUnifiedIdeograph4b2d', [], 'Display', None],
  [ 0x4b2e, 'CjkUnifiedIdeograph4b2e', [], 'Display', None],
  [ 0x4b2f, 'CjkUnifiedIdeograph4b2f', [], 'Display', None],
  [ 0x4b30, 'CjkUnifiedIdeograph4b30', [], 'Display', None],
  [ 0x4b31, 'CjkUnifiedIdeograph4b31', [], 'Display', None],
  [ 0x4b32, 'CjkUnifiedIdeograph4b32', [], 'Display', None],
  [ 0x4b33, 'CjkUnifiedIdeograph4b33', [], 'Display', None],
  [ 0x4b34, 'CjkUnifiedIdeograph4b34', [], 'Display', None],
  [ 0x4b35, 'CjkUnifiedIdeograph4b35', [], 'Display', None],
  [ 0x4b36, 'CjkUnifiedIdeograph4b36', [], 'Display', None],
  [ 0x4b37, 'CjkUnifiedIdeograph4b37', [], 'Display', None],
  [ 0x4b38, 'CjkUnifiedIdeograph4b38', [], 'Display', None],
  [ 0x4b39, 'CjkUnifiedIdeograph4b39', [], 'Display', None],
  [ 0x4b3a, 'CjkUnifiedIdeograph4b3a', [], 'Display', None],
  [ 0x4b3b, 'CjkUnifiedIdeograph4b3b', [], 'Display', None],
  [ 0x4b3c, 'CjkUnifiedIdeograph4b3c', [], 'Display', None],
  [ 0x4b3d, 'CjkUnifiedIdeograph4b3d', [], 'Display', None],
  [ 0x4b3e, 'CjkUnifiedIdeograph4b3e', [], 'Display', None],
  [ 0x4b3f, 'CjkUnifiedIdeograph4b3f', [], 'Display', None],
  [ 0x4b40, 'CjkUnifiedIdeograph4b40', [], 'Display', None],
  [ 0x4b41, 'CjkUnifiedIdeograph4b41', [], 'Display', None],
  [ 0x4b42, 'CjkUnifiedIdeograph4b42', [], 'Display', None],
  [ 0x4b43, 'CjkUnifiedIdeograph4b43', [], 'Display', None],
  [ 0x4b44, 'CjkUnifiedIdeograph4b44', [], 'Display', None],
  [ 0x4b45, 'CjkUnifiedIdeograph4b45', [], 'Display', None],
  [ 0x4b46, 'CjkUnifiedIdeograph4b46', [], 'Display', None],
  [ 0x4b47, 'CjkUnifiedIdeograph4b47', [], 'Display', None],
  [ 0x4b48, 'CjkUnifiedIdeograph4b48', [], 'Display', None],
  [ 0x4b49, 'CjkUnifiedIdeograph4b49', [], 'Display', None],
  [ 0x4b4a, 'CjkUnifiedIdeograph4b4a', [], 'Display', None],
  [ 0x4b4b, 'CjkUnifiedIdeograph4b4b', [], 'Display', None],
  [ 0x4b4c, 'CjkUnifiedIdeograph4b4c', [], 'Display', None],
  [ 0x4b4d, 'CjkUnifiedIdeograph4b4d', [], 'Display', None],
  [ 0x4b4e, 'CjkUnifiedIdeograph4b4e', [], 'Display', None],
  [ 0x4b4f, 'CjkUnifiedIdeograph4b4f', [], 'Display', None],
  [ 0x4b50, 'CjkUnifiedIdeograph4b50', [], 'Display', None],
  [ 0x4b51, 'CjkUnifiedIdeograph4b51', [], 'Display', None],
  [ 0x4b52, 'CjkUnifiedIdeograph4b52', [], 'Display', None],
  [ 0x4b53, 'CjkUnifiedIdeograph4b53', [], 'Display', None],
  [ 0x4b54, 'CjkUnifiedIdeograph4b54', [], 'Display', None],
  [ 0x4b55, 'CjkUnifiedIdeograph4b55', [], 'Display', None],
  [ 0x4b56, 'CjkUnifiedIdeograph4b56', [], 'Display', None],
  [ 0x4b57, 'CjkUnifiedIdeograph4b57', [], 'Display', None],
  [ 0x4b58, 'CjkUnifiedIdeograph4b58', [], 'Display', None],
  [ 0x4b59, 'CjkUnifiedIdeograph4b59', [], 'Display', None],
  [ 0x4b5a, 'CjkUnifiedIdeograph4b5a', [], 'Display', None],
  [ 0x4b5b, 'CjkUnifiedIdeograph4b5b', [], 'Display', None],
  [ 0x4b5c, 'CjkUnifiedIdeograph4b5c', [], 'Display', None],
  [ 0x4b5d, 'CjkUnifiedIdeograph4b5d', [], 'Display', None],
  [ 0x4b5e, 'CjkUnifiedIdeograph4b5e', [], 'Display', None],
  [ 0x4b5f, 'CjkUnifiedIdeograph4b5f', [], 'Display', None],
  [ 0x4b60, 'CjkUnifiedIdeograph4b60', [], 'Display', None],
  [ 0x4b61, 'CjkUnifiedIdeograph4b61', [], 'Display', None],
  [ 0x4b62, 'CjkUnifiedIdeograph4b62', [], 'Display', None],
  [ 0x4b63, 'CjkUnifiedIdeograph4b63', [], 'Display', None],
  [ 0x4b64, 'CjkUnifiedIdeograph4b64', [], 'Display', None],
  [ 0x4b65, 'CjkUnifiedIdeograph4b65', [], 'Display', None],
  [ 0x4b66, 'CjkUnifiedIdeograph4b66', [], 'Display', None],
  [ 0x4b67, 'CjkUnifiedIdeograph4b67', [], 'Display', None],
  [ 0x4b68, 'CjkUnifiedIdeograph4b68', [], 'Display', None],
  [ 0x4b69, 'CjkUnifiedIdeograph4b69', [], 'Display', None],
  [ 0x4b6a, 'CjkUnifiedIdeograph4b6a', [], 'Display', None],
  [ 0x4b6b, 'CjkUnifiedIdeograph4b6b', [], 'Display', None],
  [ 0x4b6c, 'CjkUnifiedIdeograph4b6c', [], 'Display', None],
  [ 0x4b6d, 'CjkUnifiedIdeograph4b6d', [], 'Display', None],
  [ 0x4b6e, 'CjkUnifiedIdeograph4b6e', [], 'Display', None],
  [ 0x4b6f, 'CjkUnifiedIdeograph4b6f', [], 'Display', None],
  [ 0x4b70, 'CjkUnifiedIdeograph4b70', [], 'Display', None],
  [ 0x4b71, 'CjkUnifiedIdeograph4b71', [], 'Display', None],
  [ 0x4b72, 'CjkUnifiedIdeograph4b72', [], 'Display', None],
  [ 0x4b73, 'CjkUnifiedIdeograph4b73', [], 'Display', None],
  [ 0x4b74, 'CjkUnifiedIdeograph4b74', [], 'Display', None],
  [ 0x4b75, 'CjkUnifiedIdeograph4b75', [], 'Display', None],
  [ 0x4b76, 'CjkUnifiedIdeograph4b76', [], 'Display', None],
  [ 0x4b77, 'CjkUnifiedIdeograph4b77', [], 'Display', None],
  [ 0x4b78, 'CjkUnifiedIdeograph4b78', [], 'Display', None],
  [ 0x4b79, 'CjkUnifiedIdeograph4b79', [], 'Display', None],
  [ 0x4b7a, 'CjkUnifiedIdeograph4b7a', [], 'Display', None],
  [ 0x4b7b, 'CjkUnifiedIdeograph4b7b', [], 'Display', None],
  [ 0x4b7c, 'CjkUnifiedIdeograph4b7c', [], 'Display', None],
  [ 0x4b7d, 'CjkUnifiedIdeograph4b7d', [], 'Display', None],
  [ 0x4b7e, 'CjkUnifiedIdeograph4b7e', [], 'Display', None],
  [ 0x4b7f, 'CjkUnifiedIdeograph4b7f', [], 'Display', None],
  [ 0x4b80, 'CjkUnifiedIdeograph4b80', [], 'Display', None],
  [ 0x4b81, 'CjkUnifiedIdeograph4b81', [], 'Display', None],
  [ 0x4b82, 'CjkUnifiedIdeograph4b82', [], 'Display', None],
  [ 0x4b83, 'CjkUnifiedIdeograph4b83', [], 'Display', None],
  [ 0x4b84, 'CjkUnifiedIdeograph4b84', [], 'Display', None],
  [ 0x4b85, 'CjkUnifiedIdeograph4b85', [], 'Display', None],
  [ 0x4b86, 'CjkUnifiedIdeograph4b86', [], 'Display', None],
  [ 0x4b87, 'CjkUnifiedIdeograph4b87', [], 'Display', None],
  [ 0x4b88, 'CjkUnifiedIdeograph4b88', [], 'Display', None],
  [ 0x4b89, 'CjkUnifiedIdeograph4b89', [], 'Display', None],
  [ 0x4b8a, 'CjkUnifiedIdeograph4b8a', [], 'Display', None],
  [ 0x4b8b, 'CjkUnifiedIdeograph4b8b', [], 'Display', None],
  [ 0x4b8c, 'CjkUnifiedIdeograph4b8c', [], 'Display', None],
  [ 0x4b8d, 'CjkUnifiedIdeograph4b8d', [], 'Display', None],
  [ 0x4b8e, 'CjkUnifiedIdeograph4b8e', [], 'Display', None],
  [ 0x4b8f, 'CjkUnifiedIdeograph4b8f', [], 'Display', None],
  [ 0x4b90, 'CjkUnifiedIdeograph4b90', [], 'Display', None],
  [ 0x4b91, 'CjkUnifiedIdeograph4b91', [], 'Display', None],
  [ 0x4b92, 'CjkUnifiedIdeograph4b92', [], 'Display', None],
  [ 0x4b93, 'CjkUnifiedIdeograph4b93', [], 'Display', None],
  [ 0x4b94, 'CjkUnifiedIdeograph4b94', [], 'Display', None],
  [ 0x4b95, 'CjkUnifiedIdeograph4b95', [], 'Display', None],
  [ 0x4b96, 'CjkUnifiedIdeograph4b96', [], 'Display', None],
  [ 0x4b97, 'CjkUnifiedIdeograph4b97', [], 'Display', None],
  [ 0x4b98, 'CjkUnifiedIdeograph4b98', [], 'Display', None],
  [ 0x4b99, 'CjkUnifiedIdeograph4b99', [], 'Display', None],
  [ 0x4b9a, 'CjkUnifiedIdeograph4b9a', [], 'Display', None],
  [ 0x4b9b, 'CjkUnifiedIdeograph4b9b', [], 'Display', None],
  [ 0x4b9c, 'CjkUnifiedIdeograph4b9c', [], 'Display', None],
  [ 0x4b9d, 'CjkUnifiedIdeograph4b9d', [], 'Display', None],
  [ 0x4b9e, 'CjkUnifiedIdeograph4b9e', [], 'Display', None],
  [ 0x4b9f, 'CjkUnifiedIdeograph4b9f', [], 'Display', None],
  [ 0x4ba0, 'CjkUnifiedIdeograph4ba0', [], 'Display', None],
  [ 0x4ba1, 'CjkUnifiedIdeograph4ba1', [], 'Display', None],
  [ 0x4ba2, 'CjkUnifiedIdeograph4ba2', [], 'Display', None],
  [ 0x4ba3, 'CjkUnifiedIdeograph4ba3', [], 'Display', None],
  [ 0x4ba4, 'CjkUnifiedIdeograph4ba4', [], 'Display', None],
  [ 0x4ba5, 'CjkUnifiedIdeograph4ba5', [], 'Display', None],
  [ 0x4ba6, 'CjkUnifiedIdeograph4ba6', [], 'Display', None],
  [ 0x4ba7, 'CjkUnifiedIdeograph4ba7', [], 'Display', None],
  [ 0x4ba8, 'CjkUnifiedIdeograph4ba8', [], 'Display', None],
  [ 0x4ba9, 'CjkUnifiedIdeograph4ba9', [], 'Display', None],
  [ 0x4baa, 'CjkUnifiedIdeograph4baa', [], 'Display', None],
  [ 0x4bab, 'CjkUnifiedIdeograph4bab', [], 'Display', None],
  [ 0x4bac, 'CjkUnifiedIdeograph4bac', [], 'Display', None],
  [ 0x4bad, 'CjkUnifiedIdeograph4bad', [], 'Display', None],
  [ 0x4bae, 'CjkUnifiedIdeograph4bae', [], 'Display', None],
  [ 0x4baf, 'CjkUnifiedIdeograph4baf', [], 'Display', None],
  [ 0x4bb0, 'CjkUnifiedIdeograph4bb0', [], 'Display', None],
  [ 0x4bb1, 'CjkUnifiedIdeograph4bb1', [], 'Display', None],
  [ 0x4bb2, 'CjkUnifiedIdeograph4bb2', [], 'Display', None],
  [ 0x4bb3, 'CjkUnifiedIdeograph4bb3', [], 'Display', None],
  [ 0x4bb4, 'CjkUnifiedIdeograph4bb4', [], 'Display', None],
  [ 0x4bb5, 'CjkUnifiedIdeograph4bb5', [], 'Display', None],
  [ 0x4bb6, 'CjkUnifiedIdeograph4bb6', [], 'Display', None],
  [ 0x4bb7, 'CjkUnifiedIdeograph4bb7', [], 'Display', None],
  [ 0x4bb8, 'CjkUnifiedIdeograph4bb8', [], 'Display', None],
  [ 0x4bb9, 'CjkUnifiedIdeograph4bb9', [], 'Display', None],
  [ 0x4bba, 'CjkUnifiedIdeograph4bba', [], 'Display', None],
  [ 0x4bbb, 'CjkUnifiedIdeograph4bbb', [], 'Display', None],
  [ 0x4bbc, 'CjkUnifiedIdeograph4bbc', [], 'Display', None],
  [ 0x4bbd, 'CjkUnifiedIdeograph4bbd', [], 'Display', None],
  [ 0x4bbe, 'CjkUnifiedIdeograph4bbe', [], 'Display', None],
  [ 0x4bbf, 'CjkUnifiedIdeograph4bbf', [], 'Display', None],
  [ 0x4bc0, 'CjkUnifiedIdeograph4bc0', [], 'Display', None],
  [ 0x4bc1, 'CjkUnifiedIdeograph4bc1', [], 'Display', None],
  [ 0x4bc2, 'CjkUnifiedIdeograph4bc2', [], 'Display', None],
  [ 0x4bc3, 'CjkUnifiedIdeograph4bc3', [], 'Display', None],
  [ 0x4bc4, 'CjkUnifiedIdeograph4bc4', [], 'Display', None],
  [ 0x4bc5, 'CjkUnifiedIdeograph4bc5', [], 'Display', None],
  [ 0x4bc6, 'CjkUnifiedIdeograph4bc6', [], 'Display', None],
  [ 0x4bc7, 'CjkUnifiedIdeograph4bc7', [], 'Display', None],
  [ 0x4bc8, 'CjkUnifiedIdeograph4bc8', [], 'Display', None],
  [ 0x4bc9, 'CjkUnifiedIdeograph4bc9', [], 'Display', None],
  [ 0x4bca, 'CjkUnifiedIdeograph4bca', [], 'Display', None],
  [ 0x4bcb, 'CjkUnifiedIdeograph4bcb', [], 'Display', None],
  [ 0x4bcc, 'CjkUnifiedIdeograph4bcc', [], 'Display', None],
  [ 0x4bcd, 'CjkUnifiedIdeograph4bcd', [], 'Display', None],
  [ 0x4bce, 'CjkUnifiedIdeograph4bce', [], 'Display', None],
  [ 0x4bcf, 'CjkUnifiedIdeograph4bcf', [], 'Display', None],
  [ 0x4bd0, 'CjkUnifiedIdeograph4bd0', [], 'Display', None],
  [ 0x4bd1, 'CjkUnifiedIdeograph4bd1', [], 'Display', None],
  [ 0x4bd2, 'CjkUnifiedIdeograph4bd2', [], 'Display', None],
  [ 0x4bd3, 'CjkUnifiedIdeograph4bd3', [], 'Display', None],
  [ 0x4bd4, 'CjkUnifiedIdeograph4bd4', [], 'Display', None],
  [ 0x4bd5, 'CjkUnifiedIdeograph4bd5', [], 'Display', None],
  [ 0x4bd6, 'CjkUnifiedIdeograph4bd6', [], 'Display', None],
  [ 0x4bd7, 'CjkUnifiedIdeograph4bd7', [], 'Display', None],
  [ 0x4bd8, 'CjkUnifiedIdeograph4bd8', [], 'Display', None],
  [ 0x4bd9, 'CjkUnifiedIdeograph4bd9', [], 'Display', None],
  [ 0x4bda, 'CjkUnifiedIdeograph4bda', [], 'Display', None],
  [ 0x4bdb, 'CjkUnifiedIdeograph4bdb', [], 'Display', None],
  [ 0x4bdc, 'CjkUnifiedIdeograph4bdc', [], 'Display', None],
  [ 0x4bdd, 'CjkUnifiedIdeograph4bdd', [], 'Display', None],
  [ 0x4bde, 'CjkUnifiedIdeograph4bde', [], 'Display', None],
  [ 0x4bdf, 'CjkUnifiedIdeograph4bdf', [], 'Display', None],
  [ 0x4be0, 'CjkUnifiedIdeograph4be0', [], 'Display', None],
  [ 0x4be1, 'CjkUnifiedIdeograph4be1', [], 'Display', None],
  [ 0x4be2, 'CjkUnifiedIdeograph4be2', [], 'Display', None],
  [ 0x4be3, 'CjkUnifiedIdeograph4be3', [], 'Display', None],
  [ 0x4be4, 'CjkUnifiedIdeograph4be4', [], 'Display', None],
  [ 0x4be5, 'CjkUnifiedIdeograph4be5', [], 'Display', None],
  [ 0x4be6, 'CjkUnifiedIdeograph4be6', [], 'Display', None],
  [ 0x4be7, 'CjkUnifiedIdeograph4be7', [], 'Display', None],
  [ 0x4be8, 'CjkUnifiedIdeograph4be8', [], 'Display', None],
  [ 0x4be9, 'CjkUnifiedIdeograph4be9', [], 'Display', None],
  [ 0x4bea, 'CjkUnifiedIdeograph4bea', [], 'Display', None],
  [ 0x4beb, 'CjkUnifiedIdeograph4beb', [], 'Display', None],
  [ 0x4bec, 'CjkUnifiedIdeograph4bec', [], 'Display', None],
  [ 0x4bed, 'CjkUnifiedIdeograph4bed', [], 'Display', None],
  [ 0x4bee, 'CjkUnifiedIdeograph4bee', [], 'Display', None],
  [ 0x4bef, 'CjkUnifiedIdeograph4bef', [], 'Display', None],
  [ 0x4bf0, 'CjkUnifiedIdeograph4bf0', [], 'Display', None],
  [ 0x4bf1, 'CjkUnifiedIdeograph4bf1', [], 'Display', None],
  [ 0x4bf2, 'CjkUnifiedIdeograph4bf2', [], 'Display', None],
  [ 0x4bf3, 'CjkUnifiedIdeograph4bf3', [], 'Display', None],
  [ 0x4bf4, 'CjkUnifiedIdeograph4bf4', [], 'Display', None],
  [ 0x4bf5, 'CjkUnifiedIdeograph4bf5', [], 'Display', None],
  [ 0x4bf6, 'CjkUnifiedIdeograph4bf6', [], 'Display', None],
  [ 0x4bf7, 'CjkUnifiedIdeograph4bf7', [], 'Display', None],
  [ 0x4bf8, 'CjkUnifiedIdeograph4bf8', [], 'Display', None],
  [ 0x4bf9, 'CjkUnifiedIdeograph4bf9', [], 'Display', None],
  [ 0x4bfa, 'CjkUnifiedIdeograph4bfa', [], 'Display', None],
  [ 0x4bfb, 'CjkUnifiedIdeograph4bfb', [], 'Display', None],
  [ 0x4bfc, 'CjkUnifiedIdeograph4bfc', [], 'Display', None],
  [ 0x4bfd, 'CjkUnifiedIdeograph4bfd', [], 'Display', None],
  [ 0x4bfe, 'CjkUnifiedIdeograph4bfe', [], 'Display', None],
  [ 0x4bff, 'CjkUnifiedIdeograph4bff', [], 'Display', None],
  [ 0x4c00, 'CjkUnifiedIdeograph4c00', [], 'Display', None],
  [ 0x4c01, 'CjkUnifiedIdeograph4c01', [], 'Display', None],
  [ 0x4c02, 'CjkUnifiedIdeograph4c02', [], 'Display', None],
  [ 0x4c03, 'CjkUnifiedIdeograph4c03', [], 'Display', None],
  [ 0x4c04, 'CjkUnifiedIdeograph4c04', [], 'Display', None],
  [ 0x4c05, 'CjkUnifiedIdeograph4c05', [], 'Display', None],
  [ 0x4c06, 'CjkUnifiedIdeograph4c06', [], 'Display', None],
  [ 0x4c07, 'CjkUnifiedIdeograph4c07', [], 'Display', None],
  [ 0x4c08, 'CjkUnifiedIdeograph4c08', [], 'Display', None],
  [ 0x4c09, 'CjkUnifiedIdeograph4c09', [], 'Display', None],
  [ 0x4c0a, 'CjkUnifiedIdeograph4c0a', [], 'Display', None],
  [ 0x4c0b, 'CjkUnifiedIdeograph4c0b', [], 'Display', None],
  [ 0x4c0c, 'CjkUnifiedIdeograph4c0c', [], 'Display', None],
  [ 0x4c0d, 'CjkUnifiedIdeograph4c0d', [], 'Display', None],
  [ 0x4c0e, 'CjkUnifiedIdeograph4c0e', [], 'Display', None],
  [ 0x4c0f, 'CjkUnifiedIdeograph4c0f', [], 'Display', None],
  [ 0x4c10, 'CjkUnifiedIdeograph4c10', [], 'Display', None],
  [ 0x4c11, 'CjkUnifiedIdeograph4c11', [], 'Display', None],
  [ 0x4c12, 'CjkUnifiedIdeograph4c12', [], 'Display', None],
  [ 0x4c13, 'CjkUnifiedIdeograph4c13', [], 'Display', None],
  [ 0x4c14, 'CjkUnifiedIdeograph4c14', [], 'Display', None],
  [ 0x4c15, 'CjkUnifiedIdeograph4c15', [], 'Display', None],
  [ 0x4c16, 'CjkUnifiedIdeograph4c16', [], 'Display', None],
  [ 0x4c17, 'CjkUnifiedIdeograph4c17', [], 'Display', None],
  [ 0x4c18, 'CjkUnifiedIdeograph4c18', [], 'Display', None],
  [ 0x4c19, 'CjkUnifiedIdeograph4c19', [], 'Display', None],
  [ 0x4c1a, 'CjkUnifiedIdeograph4c1a', [], 'Display', None],
  [ 0x4c1b, 'CjkUnifiedIdeograph4c1b', [], 'Display', None],
  [ 0x4c1c, 'CjkUnifiedIdeograph4c1c', [], 'Display', None],
  [ 0x4c1d, 'CjkUnifiedIdeograph4c1d', [], 'Display', None],
  [ 0x4c1e, 'CjkUnifiedIdeograph4c1e', [], 'Display', None],
  [ 0x4c1f, 'CjkUnifiedIdeograph4c1f', [], 'Display', None],
  [ 0x4c20, 'CjkUnifiedIdeograph4c20', [], 'Display', None],
  [ 0x4c21, 'CjkUnifiedIdeograph4c21', [], 'Display', None],
  [ 0x4c22, 'CjkUnifiedIdeograph4c22', [], 'Display', None],
  [ 0x4c23, 'CjkUnifiedIdeograph4c23', [], 'Display', None],
  [ 0x4c24, 'CjkUnifiedIdeograph4c24', [], 'Display', None],
  [ 0x4c25, 'CjkUnifiedIdeograph4c25', [], 'Display', None],
  [ 0x4c26, 'CjkUnifiedIdeograph4c26', [], 'Display', None],
  [ 0x4c27, 'CjkUnifiedIdeograph4c27', [], 'Display', None],
  [ 0x4c28, 'CjkUnifiedIdeograph4c28', [], 'Display', None],
  [ 0x4c29, 'CjkUnifiedIdeograph4c29', [], 'Display', None],
  [ 0x4c2a, 'CjkUnifiedIdeograph4c2a', [], 'Display', None],
  [ 0x4c2b, 'CjkUnifiedIdeograph4c2b', [], 'Display', None],
  [ 0x4c2c, 'CjkUnifiedIdeograph4c2c', [], 'Display', None],
  [ 0x4c2d, 'CjkUnifiedIdeograph4c2d', [], 'Display', None],
  [ 0x4c2e, 'CjkUnifiedIdeograph4c2e', [], 'Display', None],
  [ 0x4c2f, 'CjkUnifiedIdeograph4c2f', [], 'Display', None],
  [ 0x4c30, 'CjkUnifiedIdeograph4c30', [], 'Display', None],
  [ 0x4c31, 'CjkUnifiedIdeograph4c31', [], 'Display', None],
  [ 0x4c32, 'CjkUnifiedIdeograph4c32', [], 'Display', None],
  [ 0x4c33, 'CjkUnifiedIdeograph4c33', [], 'Display', None],
  [ 0x4c34, 'CjkUnifiedIdeograph4c34', [], 'Display', None],
  [ 0x4c35, 'CjkUnifiedIdeograph4c35', [], 'Display', None],
  [ 0x4c36, 'CjkUnifiedIdeograph4c36', [], 'Display', None],
  [ 0x4c37, 'CjkUnifiedIdeograph4c37', [], 'Display', None],
  [ 0x4c38, 'CjkUnifiedIdeograph4c38', [], 'Display', None],
  [ 0x4c39, 'CjkUnifiedIdeograph4c39', [], 'Display', None],
  [ 0x4c3a, 'CjkUnifiedIdeograph4c3a', [], 'Display', None],
  [ 0x4c3b, 'CjkUnifiedIdeograph4c3b', [], 'Display', None],
  [ 0x4c3c, 'CjkUnifiedIdeograph4c3c', [], 'Display', None],
  [ 0x4c3d, 'CjkUnifiedIdeograph4c3d', [], 'Display', None],
  [ 0x4c3e, 'CjkUnifiedIdeograph4c3e', [], 'Display', None],
  [ 0x4c3f, 'CjkUnifiedIdeograph4c3f', [], 'Display', None],
  [ 0x4c40, 'CjkUnifiedIdeograph4c40', [], 'Display', None],
  [ 0x4c41, 'CjkUnifiedIdeograph4c41', [], 'Display', None],
  [ 0x4c42, 'CjkUnifiedIdeograph4c42', [], 'Display', None],
  [ 0x4c43, 'CjkUnifiedIdeograph4c43', [], 'Display', None],
  [ 0x4c44, 'CjkUnifiedIdeograph4c44', [], 'Display', None],
  [ 0x4c45, 'CjkUnifiedIdeograph4c45', [], 'Display', None],
  [ 0x4c46, 'CjkUnifiedIdeograph4c46', [], 'Display', None],
  [ 0x4c47, 'CjkUnifiedIdeograph4c47', [], 'Display', None],
  [ 0x4c48, 'CjkUnifiedIdeograph4c48', [], 'Display', None],
  [ 0x4c49, 'CjkUnifiedIdeograph4c49', [], 'Display', None],
  [ 0x4c4a, 'CjkUnifiedIdeograph4c4a', [], 'Display', None],
  [ 0x4c4b, 'CjkUnifiedIdeograph4c4b', [], 'Display', None],
  [ 0x4c4c, 'CjkUnifiedIdeograph4c4c', [], 'Display', None],
  [ 0x4c4d, 'CjkUnifiedIdeograph4c4d', [], 'Display', None],
  [ 0x4c4e, 'CjkUnifiedIdeograph4c4e', [], 'Display', None],
  [ 0x4c4f, 'CjkUnifiedIdeograph4c4f', [], 'Display', None],
  [ 0x4c50, 'CjkUnifiedIdeograph4c50', [], 'Display', None],
  [ 0x4c51, 'CjkUnifiedIdeograph4c51', [], 'Display', None],
  [ 0x4c52, 'CjkUnifiedIdeograph4c52', [], 'Display', None],
  [ 0x4c53, 'CjkUnifiedIdeograph4c53', [], 'Display', None],
  [ 0x4c54, 'CjkUnifiedIdeograph4c54', [], 'Display', None],
  [ 0x4c55, 'CjkUnifiedIdeograph4c55', [], 'Display', None],
  [ 0x4c56, 'CjkUnifiedIdeograph4c56', [], 'Display', None],
  [ 0x4c57, 'CjkUnifiedIdeograph4c57', [], 'Display', None],
  [ 0x4c58, 'CjkUnifiedIdeograph4c58', [], 'Display', None],
  [ 0x4c59, 'CjkUnifiedIdeograph4c59', [], 'Display', None],
  [ 0x4c5a, 'CjkUnifiedIdeograph4c5a', [], 'Display', None],
  [ 0x4c5b, 'CjkUnifiedIdeograph4c5b', [], 'Display', None],
  [ 0x4c5c, 'CjkUnifiedIdeograph4c5c', [], 'Display', None],
  [ 0x4c5d, 'CjkUnifiedIdeograph4c5d', [], 'Display', None],
  [ 0x4c5e, 'CjkUnifiedIdeograph4c5e', [], 'Display', None],
  [ 0x4c5f, 'CjkUnifiedIdeograph4c5f', [], 'Display', None],
  [ 0x4c60, 'CjkUnifiedIdeograph4c60', [], 'Display', None],
  [ 0x4c61, 'CjkUnifiedIdeograph4c61', [], 'Display', None],
  [ 0x4c62, 'CjkUnifiedIdeograph4c62', [], 'Display', None],
  [ 0x4c63, 'CjkUnifiedIdeograph4c63', [], 'Display', None],
  [ 0x4c64, 'CjkUnifiedIdeograph4c64', [], 'Display', None],
  [ 0x4c65, 'CjkUnifiedIdeograph4c65', [], 'Display', None],
  [ 0x4c66, 'CjkUnifiedIdeograph4c66', [], 'Display', None],
  [ 0x4c67, 'CjkUnifiedIdeograph4c67', [], 'Display', None],
  [ 0x4c68, 'CjkUnifiedIdeograph4c68', [], 'Display', None],
  [ 0x4c69, 'CjkUnifiedIdeograph4c69', [], 'Display', None],
  [ 0x4c6a, 'CjkUnifiedIdeograph4c6a', [], 'Display', None],
  [ 0x4c6b, 'CjkUnifiedIdeograph4c6b', [], 'Display', None],
  [ 0x4c6c, 'CjkUnifiedIdeograph4c6c', [], 'Display', None],
  [ 0x4c6d, 'CjkUnifiedIdeograph4c6d', [], 'Display', None],
  [ 0x4c6e, 'CjkUnifiedIdeograph4c6e', [], 'Display', None],
  [ 0x4c6f, 'CjkUnifiedIdeograph4c6f', [], 'Display', None],
  [ 0x4c70, 'CjkUnifiedIdeograph4c70', [], 'Display', None],
  [ 0x4c71, 'CjkUnifiedIdeograph4c71', [], 'Display', None],
  [ 0x4c72, 'CjkUnifiedIdeograph4c72', [], 'Display', None],
  [ 0x4c73, 'CjkUnifiedIdeograph4c73', [], 'Display', None],
  [ 0x4c74, 'CjkUnifiedIdeograph4c74', [], 'Display', None],
  [ 0x4c75, 'CjkUnifiedIdeograph4c75', [], 'Display', None],
  [ 0x4c76, 'CjkUnifiedIdeograph4c76', [], 'Display', None],
  [ 0x4c77, 'CjkUnifiedIdeograph4c77', [], 'Display', None],
  [ 0x4c78, 'CjkUnifiedIdeograph4c78', [], 'Display', None],
  [ 0x4c79, 'CjkUnifiedIdeograph4c79', [], 'Display', None],
  [ 0x4c7a, 'CjkUnifiedIdeograph4c7a', [], 'Display', None],
  [ 0x4c7b, 'CjkUnifiedIdeograph4c7b', [], 'Display', None],
  [ 0x4c7c, 'CjkUnifiedIdeograph4c7c', [], 'Display', None],
  [ 0x4c7d, 'CjkUnifiedIdeograph4c7d', [], 'Display', None],
  [ 0x4c7e, 'CjkUnifiedIdeograph4c7e', [], 'Display', None],
  [ 0x4c7f, 'CjkUnifiedIdeograph4c7f', [], 'Display', None],
  [ 0x4c80, 'CjkUnifiedIdeograph4c80', [], 'Display', None],
  [ 0x4c81, 'CjkUnifiedIdeograph4c81', [], 'Display', None],
  [ 0x4c82, 'CjkUnifiedIdeograph4c82', [], 'Display', None],
  [ 0x4c83, 'CjkUnifiedIdeograph4c83', [], 'Display', None],
  [ 0x4c84, 'CjkUnifiedIdeograph4c84', [], 'Display', None],
  [ 0x4c85, 'CjkUnifiedIdeograph4c85', [], 'Display', None],
  [ 0x4c86, 'CjkUnifiedIdeograph4c86', [], 'Display', None],
  [ 0x4c87, 'CjkUnifiedIdeograph4c87', [], 'Display', None],
  [ 0x4c88, 'CjkUnifiedIdeograph4c88', [], 'Display', None],
  [ 0x4c89, 'CjkUnifiedIdeograph4c89', [], 'Display', None],
  [ 0x4c8a, 'CjkUnifiedIdeograph4c8a', [], 'Display', None],
  [ 0x4c8b, 'CjkUnifiedIdeograph4c8b', [], 'Display', None],
  [ 0x4c8c, 'CjkUnifiedIdeograph4c8c', [], 'Display', None],
  [ 0x4c8d, 'CjkUnifiedIdeograph4c8d', [], 'Display', None],
  [ 0x4c8e, 'CjkUnifiedIdeograph4c8e', [], 'Display', None],
  [ 0x4c8f, 'CjkUnifiedIdeograph4c8f', [], 'Display', None],
  [ 0x4c90, 'CjkUnifiedIdeograph4c90', [], 'Display', None],
  [ 0x4c91, 'CjkUnifiedIdeograph4c91', [], 'Display', None],
  [ 0x4c92, 'CjkUnifiedIdeograph4c92', [], 'Display', None],
  [ 0x4c93, 'CjkUnifiedIdeograph4c93', [], 'Display', None],
  [ 0x4c94, 'CjkUnifiedIdeograph4c94', [], 'Display', None],
  [ 0x4c95, 'CjkUnifiedIdeograph4c95', [], 'Display', None],
  [ 0x4c96, 'CjkUnifiedIdeograph4c96', [], 'Display', None],
  [ 0x4c97, 'CjkUnifiedIdeograph4c97', [], 'Display', None],
  [ 0x4c98, 'CjkUnifiedIdeograph4c98', [], 'Display', None],
  [ 0x4c99, 'CjkUnifiedIdeograph4c99', [], 'Display', None],
  [ 0x4c9a, 'CjkUnifiedIdeograph4c9a', [], 'Display', None],
  [ 0x4c9b, 'CjkUnifiedIdeograph4c9b', [], 'Display', None],
  [ 0x4c9c, 'CjkUnifiedIdeograph4c9c', [], 'Display', None],
  [ 0x4c9d, 'CjkUnifiedIdeograph4c9d', [], 'Display', None],
  [ 0x4c9e, 'CjkUnifiedIdeograph4c9e', [], 'Display', None],
  [ 0x4c9f, 'CjkUnifiedIdeograph4c9f', [], 'Display', None],
  [ 0x4ca0, 'CjkUnifiedIdeograph4ca0', [], 'Display', None],
  [ 0x4ca1, 'CjkUnifiedIdeograph4ca1', [], 'Display', None],
  [ 0x4ca2, 'CjkUnifiedIdeograph4ca2', [], 'Display', None],
  [ 0x4ca3, 'CjkUnifiedIdeograph4ca3', [], 'Display', None],
  [ 0x4ca4, 'CjkUnifiedIdeograph4ca4', [], 'Display', None],
  [ 0x4ca5, 'CjkUnifiedIdeograph4ca5', [], 'Display', None],
  [ 0x4ca6, 'CjkUnifiedIdeograph4ca6', [], 'Display', None],
  [ 0x4ca7, 'CjkUnifiedIdeograph4ca7', [], 'Display', None],
  [ 0x4ca8, 'CjkUnifiedIdeograph4ca8', [], 'Display', None],
  [ 0x4ca9, 'CjkUnifiedIdeograph4ca9', [], 'Display', None],
  [ 0x4caa, 'CjkUnifiedIdeograph4caa', [], 'Display', None],
  [ 0x4cab, 'CjkUnifiedIdeograph4cab', [], 'Display', None],
  [ 0x4cac, 'CjkUnifiedIdeograph4cac', [], 'Display', None],
  [ 0x4cad, 'CjkUnifiedIdeograph4cad', [], 'Display', None],
  [ 0x4cae, 'CjkUnifiedIdeograph4cae', [], 'Display', None],
  [ 0x4caf, 'CjkUnifiedIdeograph4caf', [], 'Display', None],
  [ 0x4cb0, 'CjkUnifiedIdeograph4cb0', [], 'Display', None],
  [ 0x4cb1, 'CjkUnifiedIdeograph4cb1', [], 'Display', None],
  [ 0x4cb2, 'CjkUnifiedIdeograph4cb2', [], 'Display', None],
  [ 0x4cb3, 'CjkUnifiedIdeograph4cb3', [], 'Display', None],
  [ 0x4cb4, 'CjkUnifiedIdeograph4cb4', [], 'Display', None],
  [ 0x4cb5, 'CjkUnifiedIdeograph4cb5', [], 'Display', None],
  [ 0x4cb6, 'CjkUnifiedIdeograph4cb6', [], 'Display', None],
  [ 0x4cb7, 'CjkUnifiedIdeograph4cb7', [], 'Display', None],
  [ 0x4cb8, 'CjkUnifiedIdeograph4cb8', [], 'Display', None],
  [ 0x4cb9, 'CjkUnifiedIdeograph4cb9', [], 'Display', None],
  [ 0x4cba, 'CjkUnifiedIdeograph4cba', [], 'Display', None],
  [ 0x4cbb, 'CjkUnifiedIdeograph4cbb', [], 'Display', None],
  [ 0x4cbc, 'CjkUnifiedIdeograph4cbc', [], 'Display', None],
  [ 0x4cbd, 'CjkUnifiedIdeograph4cbd', [], 'Display', None],
  [ 0x4cbe, 'CjkUnifiedIdeograph4cbe', [], 'Display', None],
  [ 0x4cbf, 'CjkUnifiedIdeograph4cbf', [], 'Display', None],
  [ 0x4cc0, 'CjkUnifiedIdeograph4cc0', [], 'Display', None],
  [ 0x4cc1, 'CjkUnifiedIdeograph4cc1', [], 'Display', None],
  [ 0x4cc2, 'CjkUnifiedIdeograph4cc2', [], 'Display', None],
  [ 0x4cc3, 'CjkUnifiedIdeograph4cc3', [], 'Display', None],
  [ 0x4cc4, 'CjkUnifiedIdeograph4cc4', [], 'Display', None],
  [ 0x4cc5, 'CjkUnifiedIdeograph4cc5', [], 'Display', None],
  [ 0x4cc6, 'CjkUnifiedIdeograph4cc6', [], 'Display', None],
  [ 0x4cc7, 'CjkUnifiedIdeograph4cc7', [], 'Display', None],
  [ 0x4cc8, 'CjkUnifiedIdeograph4cc8', [], 'Display', None],
  [ 0x4cc9, 'CjkUnifiedIdeograph4cc9', [], 'Display', None],
  [ 0x4cca, 'CjkUnifiedIdeograph4cca', [], 'Display', None],
  [ 0x4ccb, 'CjkUnifiedIdeograph4ccb', [], 'Display', None],
  [ 0x4ccc, 'CjkUnifiedIdeograph4ccc', [], 'Display', None],
  [ 0x4ccd, 'CjkUnifiedIdeograph4ccd', [], 'Display', None],
  [ 0x4cce, 'CjkUnifiedIdeograph4cce', [], 'Display', None],
  [ 0x4ccf, 'CjkUnifiedIdeograph4ccf', [], 'Display', None],
  [ 0x4cd0, 'CjkUnifiedIdeograph4cd0', [], 'Display', None],
  [ 0x4cd1, 'CjkUnifiedIdeograph4cd1', [], 'Display', None],
  [ 0x4cd2, 'CjkUnifiedIdeograph4cd2', [], 'Display', None],
  [ 0x4cd3, 'CjkUnifiedIdeograph4cd3', [], 'Display', None],
  [ 0x4cd4, 'CjkUnifiedIdeograph4cd4', [], 'Display', None],
  [ 0x4cd5, 'CjkUnifiedIdeograph4cd5', [], 'Display', None],
  [ 0x4cd6, 'CjkUnifiedIdeograph4cd6', [], 'Display', None],
  [ 0x4cd7, 'CjkUnifiedIdeograph4cd7', [], 'Display', None],
  [ 0x4cd8, 'CjkUnifiedIdeograph4cd8', [], 'Display', None],
  [ 0x4cd9, 'CjkUnifiedIdeograph4cd9', [], 'Display', None],
  [ 0x4cda, 'CjkUnifiedIdeograph4cda', [], 'Display', None],
  [ 0x4cdb, 'CjkUnifiedIdeograph4cdb', [], 'Display', None],
  [ 0x4cdc, 'CjkUnifiedIdeograph4cdc', [], 'Display', None],
  [ 0x4cdd, 'CjkUnifiedIdeograph4cdd', [], 'Display', None],
  [ 0x4cde, 'CjkUnifiedIdeograph4cde', [], 'Display', None],
  [ 0x4cdf, 'CjkUnifiedIdeograph4cdf', [], 'Display', None],
  [ 0x4ce0, 'CjkUnifiedIdeograph4ce0', [], 'Display', None],
  [ 0x4ce1, 'CjkUnifiedIdeograph4ce1', [], 'Display', None],
  [ 0x4ce2, 'CjkUnifiedIdeograph4ce2', [], 'Display', None],
  [ 0x4ce3, 'CjkUnifiedIdeograph4ce3', [], 'Display', None],
  [ 0x4ce4, 'CjkUnifiedIdeograph4ce4', [], 'Display', None],
  [ 0x4ce5, 'CjkUnifiedIdeograph4ce5', [], 'Display', None],
  [ 0x4ce6, 'CjkUnifiedIdeograph4ce6', [], 'Display', None],
  [ 0x4ce7, 'CjkUnifiedIdeograph4ce7', [], 'Display', None],
  [ 0x4ce8, 'CjkUnifiedIdeograph4ce8', [], 'Display', None],
  [ 0x4ce9, 'CjkUnifiedIdeograph4ce9', [], 'Display', None],
  [ 0x4cea, 'CjkUnifiedIdeograph4cea', [], 'Display', None],
  [ 0x4ceb, 'CjkUnifiedIdeograph4ceb', [], 'Display', None],
  [ 0x4cec, 'CjkUnifiedIdeograph4cec', [], 'Display', None],
  [ 0x4ced, 'CjkUnifiedIdeograph4ced', [], 'Display', None],
  [ 0x4cee, 'CjkUnifiedIdeograph4cee', [], 'Display', None],
  [ 0x4cef, 'CjkUnifiedIdeograph4cef', [], 'Display', None],
  [ 0x4cf0, 'CjkUnifiedIdeograph4cf0', [], 'Display', None],
  [ 0x4cf1, 'CjkUnifiedIdeograph4cf1', [], 'Display', None],
  [ 0x4cf2, 'CjkUnifiedIdeograph4cf2', [], 'Display', None],
  [ 0x4cf3, 'CjkUnifiedIdeograph4cf3', [], 'Display', None],
  [ 0x4cf4, 'CjkUnifiedIdeograph4cf4', [], 'Display', None],
  [ 0x4cf5, 'CjkUnifiedIdeograph4cf5', [], 'Display', None],
  [ 0x4cf6, 'CjkUnifiedIdeograph4cf6', [], 'Display', None],
  [ 0x4cf7, 'CjkUnifiedIdeograph4cf7', [], 'Display', None],
  [ 0x4cf8, 'CjkUnifiedIdeograph4cf8', [], 'Display', None],
  [ 0x4cf9, 'CjkUnifiedIdeograph4cf9', [], 'Display', None],
  [ 0x4cfa, 'CjkUnifiedIdeograph4cfa', [], 'Display', None],
  [ 0x4cfb, 'CjkUnifiedIdeograph4cfb', [], 'Display', None],
  [ 0x4cfc, 'CjkUnifiedIdeograph4cfc', [], 'Display', None],
  [ 0x4cfd, 'CjkUnifiedIdeograph4cfd', [], 'Display', None],
  [ 0x4cfe, 'CjkUnifiedIdeograph4cfe', [], 'Display', None],
  [ 0x4cff, 'CjkUnifiedIdeograph4cff', [], 'Display', None],
  [ 0x4d00, 'CjkUnifiedIdeograph4d00', [], 'Display', None],
  [ 0x4d01, 'CjkUnifiedIdeograph4d01', [], 'Display', None],
  [ 0x4d02, 'CjkUnifiedIdeograph4d02', [], 'Display', None],
  [ 0x4d03, 'CjkUnifiedIdeograph4d03', [], 'Display', None],
  [ 0x4d04, 'CjkUnifiedIdeograph4d04', [], 'Display', None],
  [ 0x4d05, 'CjkUnifiedIdeograph4d05', [], 'Display', None],
  [ 0x4d06, 'CjkUnifiedIdeograph4d06', [], 'Display', None],
  [ 0x4d07, 'CjkUnifiedIdeograph4d07', [], 'Display', None],
  [ 0x4d08, 'CjkUnifiedIdeograph4d08', [], 'Display', None],
  [ 0x4d09, 'CjkUnifiedIdeograph4d09', [], 'Display', None],
  [ 0x4d0a, 'CjkUnifiedIdeograph4d0a', [], 'Display', None],
  [ 0x4d0b, 'CjkUnifiedIdeograph4d0b', [], 'Display', None],
  [ 0x4d0c, 'CjkUnifiedIdeograph4d0c', [], 'Display', None],
  [ 0x4d0d, 'CjkUnifiedIdeograph4d0d', [], 'Display', None],
  [ 0x4d0e, 'CjkUnifiedIdeograph4d0e', [], 'Display', None],
  [ 0x4d0f, 'CjkUnifiedIdeograph4d0f', [], 'Display', None],
  [ 0x4d10, 'CjkUnifiedIdeograph4d10', [], 'Display', None],
  [ 0x4d11, 'CjkUnifiedIdeograph4d11', [], 'Display', None],
  [ 0x4d12, 'CjkUnifiedIdeograph4d12', [], 'Display', None],
  [ 0x4d13, 'CjkUnifiedIdeograph4d13', [], 'Display', None],
  [ 0x4d14, 'CjkUnifiedIdeograph4d14', [], 'Display', None],
  [ 0x4d15, 'CjkUnifiedIdeograph4d15', [], 'Display', None],
  [ 0x4d16, 'CjkUnifiedIdeograph4d16', [], 'Display', None],
  [ 0x4d17, 'CjkUnifiedIdeograph4d17', [], 'Display', None],
  [ 0x4d18, 'CjkUnifiedIdeograph4d18', [], 'Display', None],
  [ 0x4d19, 'CjkUnifiedIdeograph4d19', [], 'Display', None],
  [ 0x4d1a, 'CjkUnifiedIdeograph4d1a', [], 'Display', None],
  [ 0x4d1b, 'CjkUnifiedIdeograph4d1b', [], 'Display', None],
  [ 0x4d1c, 'CjkUnifiedIdeograph4d1c', [], 'Display', None],
  [ 0x4d1d, 'CjkUnifiedIdeograph4d1d', [], 'Display', None],
  [ 0x4d1e, 'CjkUnifiedIdeograph4d1e', [], 'Display', None],
  [ 0x4d1f, 'CjkUnifiedIdeograph4d1f', [], 'Display', None],
  [ 0x4d20, 'CjkUnifiedIdeograph4d20', [], 'Display', None],
  [ 0x4d21, 'CjkUnifiedIdeograph4d21', [], 'Display', None],
  [ 0x4d22, 'CjkUnifiedIdeograph4d22', [], 'Display', None],
  [ 0x4d23, 'CjkUnifiedIdeograph4d23', [], 'Display', None],
  [ 0x4d24, 'CjkUnifiedIdeograph4d24', [], 'Display', None],
  [ 0x4d25, 'CjkUnifiedIdeograph4d25', [], 'Display', None],
  [ 0x4d26, 'CjkUnifiedIdeograph4d26', [], 'Display', None],
  [ 0x4d27, 'CjkUnifiedIdeograph4d27', [], 'Display', None],
  [ 0x4d28, 'CjkUnifiedIdeograph4d28', [], 'Display', None],
  [ 0x4d29, 'CjkUnifiedIdeograph4d29', [], 'Display', None],
  [ 0x4d2a, 'CjkUnifiedIdeograph4d2a', [], 'Display', None],
  [ 0x4d2b, 'CjkUnifiedIdeograph4d2b', [], 'Display', None],
  [ 0x4d2c, 'CjkUnifiedIdeograph4d2c', [], 'Display', None],
  [ 0x4d2d, 'CjkUnifiedIdeograph4d2d', [], 'Display', None],
  [ 0x4d2e, 'CjkUnifiedIdeograph4d2e', [], 'Display', None],
  [ 0x4d2f, 'CjkUnifiedIdeograph4d2f', [], 'Display', None],
  [ 0x4d30, 'CjkUnifiedIdeograph4d30', [], 'Display', None],
  [ 0x4d31, 'CjkUnifiedIdeograph4d31', [], 'Display', None],
  [ 0x4d32, 'CjkUnifiedIdeograph4d32', [], 'Display', None],
  [ 0x4d33, 'CjkUnifiedIdeograph4d33', [], 'Display', None],
  [ 0x4d34, 'CjkUnifiedIdeograph4d34', [], 'Display', None],
  [ 0x4d35, 'CjkUnifiedIdeograph4d35', [], 'Display', None],
  [ 0x4d36, 'CjkUnifiedIdeograph4d36', [], 'Display', None],
  [ 0x4d37, 'CjkUnifiedIdeograph4d37', [], 'Display', None],
  [ 0x4d38, 'CjkUnifiedIdeograph4d38', [], 'Display', None],
  [ 0x4d39, 'CjkUnifiedIdeograph4d39', [], 'Display', None],
  [ 0x4d3a, 'CjkUnifiedIdeograph4d3a', [], 'Display', None],
  [ 0x4d3b, 'CjkUnifiedIdeograph4d3b', [], 'Display', None],
  [ 0x4d3c, 'CjkUnifiedIdeograph4d3c', [], 'Display', None],
  [ 0x4d3d, 'CjkUnifiedIdeograph4d3d', [], 'Display', None],
  [ 0x4d3e, 'CjkUnifiedIdeograph4d3e', [], 'Display', None],
  [ 0x4d3f, 'CjkUnifiedIdeograph4d3f', [], 'Display', None],
  [ 0x4d40, 'CjkUnifiedIdeograph4d40', [], 'Display', None],
  [ 0x4d41, 'CjkUnifiedIdeograph4d41', [], 'Display', None],
  [ 0x4d42, 'CjkUnifiedIdeograph4d42', [], 'Display', None],
  [ 0x4d43, 'CjkUnifiedIdeograph4d43', [], 'Display', None],
  [ 0x4d44, 'CjkUnifiedIdeograph4d44', [], 'Display', None],
  [ 0x4d45, 'CjkUnifiedIdeograph4d45', [], 'Display', None],
  [ 0x4d46, 'CjkUnifiedIdeograph4d46', [], 'Display', None],
  [ 0x4d47, 'CjkUnifiedIdeograph4d47', [], 'Display', None],
  [ 0x4d48, 'CjkUnifiedIdeograph4d48', [], 'Display', None],
  [ 0x4d49, 'CjkUnifiedIdeograph4d49', [], 'Display', None],
  [ 0x4d4a, 'CjkUnifiedIdeograph4d4a', [], 'Display', None],
  [ 0x4d4b, 'CjkUnifiedIdeograph4d4b', [], 'Display', None],
  [ 0x4d4c, 'CjkUnifiedIdeograph4d4c', [], 'Display', None],
  [ 0x4d4d, 'CjkUnifiedIdeograph4d4d', [], 'Display', None],
  [ 0x4d4e, 'CjkUnifiedIdeograph4d4e', [], 'Display', None],
  [ 0x4d4f, 'CjkUnifiedIdeograph4d4f', [], 'Display', None],
  [ 0x4d50, 'CjkUnifiedIdeograph4d50', [], 'Display', None],
  [ 0x4d51, 'CjkUnifiedIdeograph4d51', [], 'Display', None],
  [ 0x4d52, 'CjkUnifiedIdeograph4d52', [], 'Display', None],
  [ 0x4d53, 'CjkUnifiedIdeograph4d53', [], 'Display', None],
  [ 0x4d54, 'CjkUnifiedIdeograph4d54', [], 'Display', None],
  [ 0x4d55, 'CjkUnifiedIdeograph4d55', [], 'Display', None],
  [ 0x4d56, 'CjkUnifiedIdeograph4d56', [], 'Display', None],
  [ 0x4d57, 'CjkUnifiedIdeograph4d57', [], 'Display', None],
  [ 0x4d58, 'CjkUnifiedIdeograph4d58', [], 'Display', None],
  [ 0x4d59, 'CjkUnifiedIdeograph4d59', [], 'Display', None],
  [ 0x4d5a, 'CjkUnifiedIdeograph4d5a', [], 'Display', None],
  [ 0x4d5b, 'CjkUnifiedIdeograph4d5b', [], 'Display', None],
  [ 0x4d5c, 'CjkUnifiedIdeograph4d5c', [], 'Display', None],
  [ 0x4d5d, 'CjkUnifiedIdeograph4d5d', [], 'Display', None],
  [ 0x4d5e, 'CjkUnifiedIdeograph4d5e', [], 'Display', None],
  [ 0x4d5f, 'CjkUnifiedIdeograph4d5f', [], 'Display', None],
  [ 0x4d60, 'CjkUnifiedIdeograph4d60', [], 'Display', None],
  [ 0x4d61, 'CjkUnifiedIdeograph4d61', [], 'Display', None],
  [ 0x4d62, 'CjkUnifiedIdeograph4d62', [], 'Display', None],
  [ 0x4d63, 'CjkUnifiedIdeograph4d63', [], 'Display', None],
  [ 0x4d64, 'CjkUnifiedIdeograph4d64', [], 'Display', None],
  [ 0x4d65, 'CjkUnifiedIdeograph4d65', [], 'Display', None],
  [ 0x4d66, 'CjkUnifiedIdeograph4d66', [], 'Display', None],
  [ 0x4d67, 'CjkUnifiedIdeograph4d67', [], 'Display', None],
  [ 0x4d68, 'CjkUnifiedIdeograph4d68', [], 'Display', None],
  [ 0x4d69, 'CjkUnifiedIdeograph4d69', [], 'Display', None],
  [ 0x4d6a, 'CjkUnifiedIdeograph4d6a', [], 'Display', None],
  [ 0x4d6b, 'CjkUnifiedIdeograph4d6b', [], 'Display', None],
  [ 0x4d6c, 'CjkUnifiedIdeograph4d6c', [], 'Display', None],
  [ 0x4d6d, 'CjkUnifiedIdeograph4d6d', [], 'Display', None],
  [ 0x4d6e, 'CjkUnifiedIdeograph4d6e', [], 'Display', None],
  [ 0x4d6f, 'CjkUnifiedIdeograph4d6f', [], 'Display', None],
  [ 0x4d70, 'CjkUnifiedIdeograph4d70', [], 'Display', None],
  [ 0x4d71, 'CjkUnifiedIdeograph4d71', [], 'Display', None],
  [ 0x4d72, 'CjkUnifiedIdeograph4d72', [], 'Display', None],
  [ 0x4d73, 'CjkUnifiedIdeograph4d73', [], 'Display', None],
  [ 0x4d74, 'CjkUnifiedIdeograph4d74', [], 'Display', None],
  [ 0x4d75, 'CjkUnifiedIdeograph4d75', [], 'Display', None],
  [ 0x4d76, 'CjkUnifiedIdeograph4d76', [], 'Display', None],
  [ 0x4d77, 'CjkUnifiedIdeograph4d77', [], 'Display', None],
  [ 0x4d78, 'CjkUnifiedIdeograph4d78', [], 'Display', None],
  [ 0x4d79, 'CjkUnifiedIdeograph4d79', [], 'Display', None],
  [ 0x4d7a, 'CjkUnifiedIdeograph4d7a', [], 'Display', None],
  [ 0x4d7b, 'CjkUnifiedIdeograph4d7b', [], 'Display', None],
  [ 0x4d7c, 'CjkUnifiedIdeograph4d7c', [], 'Display', None],
  [ 0x4d7d, 'CjkUnifiedIdeograph4d7d', [], 'Display', None],
  [ 0x4d7e, 'CjkUnifiedIdeograph4d7e', [], 'Display', None],
  [ 0x4d7f, 'CjkUnifiedIdeograph4d7f', [], 'Display', None],
  [ 0x4d80, 'CjkUnifiedIdeograph4d80', [], 'Display', None],
  [ 0x4d81, 'CjkUnifiedIdeograph4d81', [], 'Display', None],
  [ 0x4d82, 'CjkUnifiedIdeograph4d82', [], 'Display', None],
  [ 0x4d83, 'CjkUnifiedIdeograph4d83', [], 'Display', None],
  [ 0x4d84, 'CjkUnifiedIdeograph4d84', [], 'Display', None],
  [ 0x4d85, 'CjkUnifiedIdeograph4d85', [], 'Display', None],
  [ 0x4d86, 'CjkUnifiedIdeograph4d86', [], 'Display', None],
  [ 0x4d87, 'CjkUnifiedIdeograph4d87', [], 'Display', None],
  [ 0x4d88, 'CjkUnifiedIdeograph4d88', [], 'Display', None],
  [ 0x4d89, 'CjkUnifiedIdeograph4d89', [], 'Display', None],
  [ 0x4d8a, 'CjkUnifiedIdeograph4d8a', [], 'Display', None],
  [ 0x4d8b, 'CjkUnifiedIdeograph4d8b', [], 'Display', None],
  [ 0x4d8c, 'CjkUnifiedIdeograph4d8c', [], 'Display', None],
  [ 0x4d8d, 'CjkUnifiedIdeograph4d8d', [], 'Display', None],
  [ 0x4d8e, 'CjkUnifiedIdeograph4d8e', [], 'Display', None],
  [ 0x4d8f, 'CjkUnifiedIdeograph4d8f', [], 'Display', None],
  [ 0x4d90, 'CjkUnifiedIdeograph4d90', [], 'Display', None],
  [ 0x4d91, 'CjkUnifiedIdeograph4d91', [], 'Display', None],
  [ 0x4d92, 'CjkUnifiedIdeograph4d92', [], 'Display', None],
  [ 0x4d93, 'CjkUnifiedIdeograph4d93', [], 'Display', None],
  [ 0x4d94, 'CjkUnifiedIdeograph4d94', [], 'Display', None],
  [ 0x4d95, 'CjkUnifiedIdeograph4d95', [], 'Display', None],
  [ 0x4d96, 'CjkUnifiedIdeograph4d96', [], 'Display', None],
  [ 0x4d97, 'CjkUnifiedIdeograph4d97', [], 'Display', None],
  [ 0x4d98, 'CjkUnifiedIdeograph4d98', [], 'Display', None],
  [ 0x4d99, 'CjkUnifiedIdeograph4d99', [], 'Display', None],
  [ 0x4d9a, 'CjkUnifiedIdeograph4d9a', [], 'Display', None],
  [ 0x4d9b, 'CjkUnifiedIdeograph4d9b', [], 'Display', None],
  [ 0x4d9c, 'CjkUnifiedIdeograph4d9c', [], 'Display', None],
  [ 0x4d9d, 'CjkUnifiedIdeograph4d9d', [], 'Display', None],
  [ 0x4d9e, 'CjkUnifiedIdeograph4d9e', [], 'Display', None],
  [ 0x4d9f, 'CjkUnifiedIdeograph4d9f', [], 'Display', None],
  [ 0x4da0, 'CjkUnifiedIdeograph4da0', [], 'Display', None],
  [ 0x4da1, 'CjkUnifiedIdeograph4da1', [], 'Display', None],
  [ 0x4da2, 'CjkUnifiedIdeograph4da2', [], 'Display', None],
  [ 0x4da3, 'CjkUnifiedIdeograph4da3', [], 'Display', None],
  [ 0x4da4, 'CjkUnifiedIdeograph4da4', [], 'Display', None],
  [ 0x4da5, 'CjkUnifiedIdeograph4da5', [], 'Display', None],
  [ 0x4da6, 'CjkUnifiedIdeograph4da6', [], 'Display', None],
  [ 0x4da7, 'CjkUnifiedIdeograph4da7', [], 'Display', None],
  [ 0x4da8, 'CjkUnifiedIdeograph4da8', [], 'Display', None],
  [ 0x4da9, 'CjkUnifiedIdeograph4da9', [], 'Display', None],
  [ 0x4daa, 'CjkUnifiedIdeograph4daa', [], 'Display', None],
  [ 0x4dab, 'CjkUnifiedIdeograph4dab', [], 'Display', None],
  [ 0x4dac, 'CjkUnifiedIdeograph4dac', [], 'Display', None],
  [ 0x4dad, 'CjkUnifiedIdeograph4dad', [], 'Display', None],
  [ 0x4dae, 'CjkUnifiedIdeograph4dae', [], 'Display', None],
  [ 0x4daf, 'CjkUnifiedIdeograph4daf', [], 'Display', None],
  [ 0x4db0, 'CjkUnifiedIdeograph4db0', [], 'Display', None],
  [ 0x4db1, 'CjkUnifiedIdeograph4db1', [], 'Display', None],
  [ 0x4db2, 'CjkUnifiedIdeograph4db2', [], 'Display', None],
  [ 0x4db3, 'CjkUnifiedIdeograph4db3', [], 'Display', None],
  [ 0x4db4, 'CjkUnifiedIdeograph4db4', [], 'Display', None],
  [ 0x4db5, 'CjkUnifiedIdeograph4db5', [], 'Display', None],
  [ 0x4db6, 'UnicodeCharacter0x4db6', [], 'Display', None],
  [ 0x4db7, 'UnicodeCharacter0x4db7', [], 'Display', None],
  [ 0x4db8, 'UnicodeCharacter0x4db8', [], 'Display', None],
  [ 0x4db9, 'UnicodeCharacter0x4db9', [], 'Display', None],
  [ 0x4dba, 'UnicodeCharacter0x4dba', [], 'Display', None],
  [ 0x4dbb, 'UnicodeCharacter0x4dbb', [], 'Display', None],
  [ 0x4dbc, 'UnicodeCharacter0x4dbc', [], 'Display', None],
  [ 0x4dbd, 'UnicodeCharacter0x4dbd', [], 'Display', None],
  [ 0x4dbe, 'UnicodeCharacter0x4dbe', [], 'Display', None],
  [ 0x4dbf, 'UnicodeCharacter0x4dbf', [], 'Display', None],
  [ 0x4dc0, 'HexagramForTheCreativeHeaven', [], 'Display', None],
  [ 0x4dc1, 'HexagramForTheReceptiveEarth', [], 'Display', None],
  [ 0x4dc2, 'HexagramForDifficultyAtTheBeginning', [], 'Display', None],
  [ 0x4dc3, 'HexagramForYouthfulFolly', [], 'Display', None],
  [ 0x4dc4, 'HexagramForWaiting', [], 'Display', None],
  [ 0x4dc5, 'HexagramForConflict', [], 'Display', None],
  [ 0x4dc6, 'HexagramForTheArmy', [], 'Display', None],
  [ 0x4dc7, 'HexagramForHoldingTogether', [], 'Display', None],
  [ 0x4dc8, 'HexagramForSmallTaming', [], 'Display', None],
  [ 0x4dc9, 'HexagramForTreading', [], 'Display', None],
  [ 0x4dca, 'HexagramForPeace', [], 'Display', None],
  [ 0x4dcb, 'HexagramForStandstill', [], 'Display', None],
  [ 0x4dcc, 'HexagramForFellowship', [], 'Display', None],
  [ 0x4dcd, 'HexagramForGreatPossession', [], 'Display', None],
  [ 0x4dce, 'HexagramForModesty', [], 'Display', None],
  [ 0x4dcf, 'HexagramForEnthusiasm', [], 'Display', None],
  [ 0x4dd0, 'HexagramForFollowing', [], 'Display', None],
  [ 0x4dd1, 'HexagramForWorkOnTheDecayed', [], 'Display', None],
  [ 0x4dd2, 'HexagramForApproach', [], 'Display', None],
  [ 0x4dd3, 'HexagramForContemplation', [], 'Display', None],
  [ 0x4dd4, 'HexagramForBitingThrough', [], 'Display', None],
  [ 0x4dd5, 'HexagramForGrace', [], 'Display', None],
  [ 0x4dd6, 'HexagramForSplittingApart', [], 'Display', None],
  [ 0x4dd7, 'HexagramForReturn', [], 'Display', None],
  [ 0x4dd8, 'HexagramForInnocence', [], 'Display', None],
  [ 0x4dd9, 'HexagramForGreatTaming', [], 'Display', None],
  [ 0x4dda, 'HexagramForMouthCorners', [], 'Display', None],
  [ 0x4ddb, 'HexagramForGreatPreponderance', [], 'Display', None],
  [ 0x4ddc, 'HexagramForTheAbysmalWater', [], 'Display', None],
  [ 0x4ddd, 'HexagramForTheClingingFire', [], 'Display', None],
  [ 0x4dde, 'HexagramForInfluence', [], 'Display', None],
  [ 0x4ddf, 'HexagramForDuration', [], 'Display', None],
  [ 0x4de0, 'HexagramForRetreat', [], 'Display', None],
  [ 0x4de1, 'HexagramForGreatPower', [], 'Display', None],
  [ 0x4de2, 'HexagramForProgress', [], 'Display', None],
  [ 0x4de3, 'HexagramForDarkeningOfTheLight', [], 'Display', None],
  [ 0x4de4, 'HexagramForTheFamily', [], 'Display', None],
  [ 0x4de5, 'HexagramForOpposition', [], 'Display', None],
  [ 0x4de6, 'HexagramForObstruction', [], 'Display', None],
  [ 0x4de7, 'HexagramForDeliverance', [], 'Display', None],
  [ 0x4de8, 'HexagramForDecrease', [], 'Display', None],
  [ 0x4de9, 'HexagramForIncrease', [], 'Display', None],
  [ 0x4dea, 'HexagramForBreakthrough', [], 'Display', None],
  [ 0x4deb, 'HexagramForComingToMeet', [], 'Display', None],
  [ 0x4dec, 'HexagramForGatheringTogether', [], 'Display', None],
  [ 0x4ded, 'HexagramForPushingUpward', [], 'Display', None],
  [ 0x4dee, 'HexagramForOppression', [], 'Display', None],
  [ 0x4def, 'HexagramForTheWell', [], 'Display', None],
  [ 0x4df0, 'HexagramForRevolution', [], 'Display', None],
  [ 0x4df1, 'HexagramForTheCauldron', [], 'Display', None],
  [ 0x4df2, 'HexagramForTheArousingThunder', [], 'Display', None],
  [ 0x4df3, 'HexagramForTheKeepingStillMountain', [], 'Display', None],
  [ 0x4df4, 'HexagramForDevelopment', [], 'Display', None],
  [ 0x4df5, 'HexagramForTheMarryingMaiden', [], 'Display', None],
  [ 0x4df6, 'HexagramForAbundance', [], 'Display', None],
  [ 0x4df7, 'HexagramForTheWanderer', [], 'Display', None],
  [ 0x4df8, 'HexagramForTheGentleWind', [], 'Display', None],
  [ 0x4df9, 'HexagramForTheJoyousLake', [], 'Display', None],
  [ 0x4dfa, 'HexagramForDispersion', [], 'Display', None],
  [ 0x4dfb, 'HexagramForLimitation', [], 'Display', None],
  [ 0x4dfc, 'HexagramForInnerTruth', [], 'Display', None],
  [ 0x4dfd, 'HexagramForSmallPreponderance', [], 'Display', None],
  [ 0x4dfe, 'HexagramForAfterCompletion', [], 'Display', None],
  [ 0x4dff, 'HexagramForBeforeCompletion', [], 'Display', None],
  [ 0x4e00, 'CjkUnifiedIdeograph4e00', [], 'Display', None],
  [ 0x4e01, 'CjkUnifiedIdeograph4e01', [], 'Display', None],
  [ 0x4e02, 'CjkUnifiedIdeograph4e02', [], 'Display', None],
  [ 0x4e03, 'CjkUnifiedIdeograph4e03', [], 'Display', None],
  [ 0x4e04, 'CjkUnifiedIdeograph4e04', [], 'Display', None],
  [ 0x4e05, 'CjkUnifiedIdeograph4e05', [], 'Display', None],
  [ 0x4e06, 'CjkUnifiedIdeograph4e06', [], 'Display', None],
  [ 0x4e07, 'CjkUnifiedIdeograph4e07', [], 'Display', None],
  [ 0x4e08, 'CjkUnifiedIdeograph4e08', [], 'Display', None],
  [ 0x4e09, 'CjkUnifiedIdeograph4e09', [], 'Display', None],
  [ 0x4e0a, 'CjkUnifiedIdeograph4e0a', [], 'Display', None],
  [ 0x4e0b, 'CjkUnifiedIdeograph4e0b', [], 'Display', None],
  [ 0x4e0c, 'CjkUnifiedIdeograph4e0c', [], 'Display', None],
  [ 0x4e0d, 'CjkUnifiedIdeograph4e0d', [], 'Display', None],
  [ 0x4e0e, 'CjkUnifiedIdeograph4e0e', [], 'Display', None],
  [ 0x4e0f, 'CjkUnifiedIdeograph4e0f', [], 'Display', None],
  [ 0x4e10, 'CjkUnifiedIdeograph4e10', [], 'Display', None],
  [ 0x4e11, 'CjkUnifiedIdeograph4e11', [], 'Display', None],
  [ 0x4e12, 'CjkUnifiedIdeograph4e12', [], 'Display', None],
  [ 0x4e13, 'CjkUnifiedIdeograph4e13', [], 'Display', None],
  [ 0x4e14, 'CjkUnifiedIdeograph4e14', [], 'Display', None],
  [ 0x4e15, 'CjkUnifiedIdeograph4e15', [], 'Display', None],
  [ 0x4e16, 'CjkUnifiedIdeograph4e16', [], 'Display', None],
  [ 0x4e17, 'CjkUnifiedIdeograph4e17', [], 'Display', None],
  [ 0x4e18, 'CjkUnifiedIdeograph4e18', [], 'Display', None],
  [ 0x4e19, 'CjkUnifiedIdeograph4e19', [], 'Display', None],
  [ 0x4e1a, 'CjkUnifiedIdeograph4e1a', [], 'Display', None],
  [ 0x4e1b, 'CjkUnifiedIdeograph4e1b', [], 'Display', None],
  [ 0x4e1c, 'CjkUnifiedIdeograph4e1c', [], 'Display', None],
  [ 0x4e1d, 'CjkUnifiedIdeograph4e1d', [], 'Display', None],
  [ 0x4e1e, 'CjkUnifiedIdeograph4e1e', [], 'Display', None],
  [ 0x4e1f, 'CjkUnifiedIdeograph4e1f', [], 'Display', None],
  [ 0x4e20, 'CjkUnifiedIdeograph4e20', [], 'Display', None],
  [ 0x4e21, 'CjkUnifiedIdeograph4e21', [], 'Display', None],
  [ 0x4e22, 'CjkUnifiedIdeograph4e22', [], 'Display', None],
  [ 0x4e23, 'CjkUnifiedIdeograph4e23', [], 'Display', None],
  [ 0x4e24, 'CjkUnifiedIdeograph4e24', [], 'Display', None],
  [ 0x4e25, 'CjkUnifiedIdeograph4e25', [], 'Display', None],
  [ 0x4e26, 'CjkUnifiedIdeograph4e26', [], 'Display', None],
  [ 0x4e27, 'CjkUnifiedIdeograph4e27', [], 'Display', None],
  [ 0x4e28, 'CjkUnifiedIdeograph4e28', [], 'Display', None],
  [ 0x4e29, 'CjkUnifiedIdeograph4e29', [], 'Display', None],
  [ 0x4e2a, 'CjkUnifiedIdeograph4e2a', [], 'Display', None],
  [ 0x4e2b, 'CjkUnifiedIdeograph4e2b', [], 'Display', None],
  [ 0x4e2c, 'CjkUnifiedIdeograph4e2c', [], 'Display', None],
  [ 0x4e2d, 'CjkUnifiedIdeograph4e2d', [], 'Display', None],
  [ 0x4e2e, 'CjkUnifiedIdeograph4e2e', [], 'Display', None],
  [ 0x4e2f, 'CjkUnifiedIdeograph4e2f', [], 'Display', None],
  [ 0x4e30, 'CjkUnifiedIdeograph4e30', [], 'Display', None],
  [ 0x4e31, 'CjkUnifiedIdeograph4e31', [], 'Display', None],
  [ 0x4e32, 'CjkUnifiedIdeograph4e32', [], 'Display', None],
  [ 0x4e33, 'CjkUnifiedIdeograph4e33', [], 'Display', None],
  [ 0x4e34, 'CjkUnifiedIdeograph4e34', [], 'Display', None],
  [ 0x4e35, 'CjkUnifiedIdeograph4e35', [], 'Display', None],
  [ 0x4e36, 'CjkUnifiedIdeograph4e36', [], 'Display', None],
  [ 0x4e37, 'CjkUnifiedIdeograph4e37', [], 'Display', None],
  [ 0x4e38, 'CjkUnifiedIdeograph4e38', [], 'Display', None],
  [ 0x4e39, 'CjkUnifiedIdeograph4e39', [], 'Display', None],
  [ 0x4e3a, 'CjkUnifiedIdeograph4e3a', [], 'Display', None],
  [ 0x4e3b, 'CjkUnifiedIdeograph4e3b', [], 'Display', None],
  [ 0x4e3c, 'CjkUnifiedIdeograph4e3c', [], 'Display', None],
  [ 0x4e3d, 'CjkUnifiedIdeograph4e3d', [], 'Display', None],
  [ 0x4e3e, 'CjkUnifiedIdeograph4e3e', [], 'Display', None],
  [ 0x4e3f, 'CjkUnifiedIdeograph4e3f', [], 'Display', None],
  [ 0x4e40, 'CjkUnifiedIdeograph4e40', [], 'Display', None],
  [ 0x4e41, 'CjkUnifiedIdeograph4e41', [], 'Display', None],
  [ 0x4e42, 'CjkUnifiedIdeograph4e42', [], 'Display', None],
  [ 0x4e43, 'CjkUnifiedIdeograph4e43', [], 'Display', None],
  [ 0x4e44, 'CjkUnifiedIdeograph4e44', [], 'Display', None],
  [ 0x4e45, 'CjkUnifiedIdeograph4e45', [], 'Display', None],
  [ 0x4e46, 'CjkUnifiedIdeograph4e46', [], 'Display', None],
  [ 0x4e47, 'CjkUnifiedIdeograph4e47', [], 'Display', None],
  [ 0x4e48, 'CjkUnifiedIdeograph4e48', [], 'Display', None],
  [ 0x4e49, 'CjkUnifiedIdeograph4e49', [], 'Display', None],
  [ 0x4e4a, 'CjkUnifiedIdeograph4e4a', [], 'Display', None],
  [ 0x4e4b, 'CjkUnifiedIdeograph4e4b', [], 'Display', None],
  [ 0x4e4c, 'CjkUnifiedIdeograph4e4c', [], 'Display', None],
  [ 0x4e4d, 'CjkUnifiedIdeograph4e4d', [], 'Display', None],
  [ 0x4e4e, 'CjkUnifiedIdeograph4e4e', [], 'Display', None],
  [ 0x4e4f, 'CjkUnifiedIdeograph4e4f', [], 'Display', None],
  [ 0x4e50, 'CjkUnifiedIdeograph4e50', [], 'Display', None],
  [ 0x4e51, 'CjkUnifiedIdeograph4e51', [], 'Display', None],
  [ 0x4e52, 'CjkUnifiedIdeograph4e52', [], 'Display', None],
  [ 0x4e53, 'CjkUnifiedIdeograph4e53', [], 'Display', None],
  [ 0x4e54, 'CjkUnifiedIdeograph4e54', [], 'Display', None],
  [ 0x4e55, 'CjkUnifiedIdeograph4e55', [], 'Display', None],
  [ 0x4e56, 'CjkUnifiedIdeograph4e56', [], 'Display', None],
  [ 0x4e57, 'CjkUnifiedIdeograph4e57', [], 'Display', None],
  [ 0x4e58, 'CjkUnifiedIdeograph4e58', [], 'Display', None],
  [ 0x4e59, 'CjkUnifiedIdeograph4e59', [], 'Display', None],
  [ 0x4e5a, 'CjkUnifiedIdeograph4e5a', [], 'Display', None],
  [ 0x4e5b, 'CjkUnifiedIdeograph4e5b', [], 'Display', None],
  [ 0x4e5c, 'CjkUnifiedIdeograph4e5c', [], 'Display', None],
  [ 0x4e5d, 'CjkUnifiedIdeograph4e5d', [], 'Display', None],
  [ 0x4e5e, 'CjkUnifiedIdeograph4e5e', [], 'Display', None],
  [ 0x4e5f, 'CjkUnifiedIdeograph4e5f', [], 'Display', None],
  [ 0x4e60, 'CjkUnifiedIdeograph4e60', [], 'Display', None],
  [ 0x4e61, 'CjkUnifiedIdeograph4e61', [], 'Display', None],
  [ 0x4e62, 'CjkUnifiedIdeograph4e62', [], 'Display', None],
  [ 0x4e63, 'CjkUnifiedIdeograph4e63', [], 'Display', None],
  [ 0x4e64, 'CjkUnifiedIdeograph4e64', [], 'Display', None],
  [ 0x4e65, 'CjkUnifiedIdeograph4e65', [], 'Display', None],
  [ 0x4e66, 'CjkUnifiedIdeograph4e66', [], 'Display', None],
  [ 0x4e67, 'CjkUnifiedIdeograph4e67', [], 'Display', None],
  [ 0x4e68, 'CjkUnifiedIdeograph4e68', [], 'Display', None],
  [ 0x4e69, 'CjkUnifiedIdeograph4e69', [], 'Display', None],
  [ 0x4e6a, 'CjkUnifiedIdeograph4e6a', [], 'Display', None],
  [ 0x4e6b, 'CjkUnifiedIdeograph4e6b', [], 'Display', None],
  [ 0x4e6c, 'CjkUnifiedIdeograph4e6c', [], 'Display', None],
  [ 0x4e6d, 'CjkUnifiedIdeograph4e6d', [], 'Display', None],
  [ 0x4e6e, 'CjkUnifiedIdeograph4e6e', [], 'Display', None],
  [ 0x4e6f, 'CjkUnifiedIdeograph4e6f', [], 'Display', None],
  [ 0x4e70, 'CjkUnifiedIdeograph4e70', [], 'Display', None],
  [ 0x4e71, 'CjkUnifiedIdeograph4e71', [], 'Display', None],
  [ 0x4e72, 'CjkUnifiedIdeograph4e72', [], 'Display', None],
  [ 0x4e73, 'CjkUnifiedIdeograph4e73', [], 'Display', None],
  [ 0x4e74, 'CjkUnifiedIdeograph4e74', [], 'Display', None],
  [ 0x4e75, 'CjkUnifiedIdeograph4e75', [], 'Display', None],
  [ 0x4e76, 'CjkUnifiedIdeograph4e76', [], 'Display', None],
  [ 0x4e77, 'CjkUnifiedIdeograph4e77', [], 'Display', None],
  [ 0x4e78, 'CjkUnifiedIdeograph4e78', [], 'Display', None],
  [ 0x4e79, 'CjkUnifiedIdeograph4e79', [], 'Display', None],
  [ 0x4e7a, 'CjkUnifiedIdeograph4e7a', [], 'Display', None],
  [ 0x4e7b, 'CjkUnifiedIdeograph4e7b', [], 'Display', None],
  [ 0x4e7c, 'CjkUnifiedIdeograph4e7c', [], 'Display', None],
  [ 0x4e7d, 'CjkUnifiedIdeograph4e7d', [], 'Display', None],
  [ 0x4e7e, 'CjkUnifiedIdeograph4e7e', [], 'Display', None],
  [ 0x4e7f, 'CjkUnifiedIdeograph4e7f', [], 'Display', None],
  [ 0x4e80, 'CjkUnifiedIdeograph4e80', [], 'Display', None],
  [ 0x4e81, 'CjkUnifiedIdeograph4e81', [], 'Display', None],
  [ 0x4e82, 'CjkUnifiedIdeograph4e82', [], 'Display', None],
  [ 0x4e83, 'CjkUnifiedIdeograph4e83', [], 'Display', None],
  [ 0x4e84, 'CjkUnifiedIdeograph4e84', [], 'Display', None],
  [ 0x4e85, 'CjkUnifiedIdeograph4e85', [], 'Display', None],
  [ 0x4e86, 'CjkUnifiedIdeograph4e86', [], 'Display', None],
  [ 0x4e87, 'CjkUnifiedIdeograph4e87', [], 'Display', None],
  [ 0x4e88, 'CjkUnifiedIdeograph4e88', [], 'Display', None],
  [ 0x4e89, 'CjkUnifiedIdeograph4e89', [], 'Display', None],
  [ 0x4e8a, 'CjkUnifiedIdeograph4e8a', [], 'Display', None],
  [ 0x4e8b, 'CjkUnifiedIdeograph4e8b', [], 'Display', None],
  [ 0x4e8c, 'CjkUnifiedIdeograph4e8c', [], 'Display', None],
  [ 0x4e8d, 'CjkUnifiedIdeograph4e8d', [], 'Display', None],
  [ 0x4e8e, 'CjkUnifiedIdeograph4e8e', [], 'Display', None],
  [ 0x4e8f, 'CjkUnifiedIdeograph4e8f', [], 'Display', None],
  [ 0x4e90, 'CjkUnifiedIdeograph4e90', [], 'Display', None],
  [ 0x4e91, 'CjkUnifiedIdeograph4e91', [], 'Display', None],
  [ 0x4e92, 'CjkUnifiedIdeograph4e92', [], 'Display', None],
  [ 0x4e93, 'CjkUnifiedIdeograph4e93', [], 'Display', None],
  [ 0x4e94, 'CjkUnifiedIdeograph4e94', [], 'Display', None],
  [ 0x4e95, 'CjkUnifiedIdeograph4e95', [], 'Display', None],
  [ 0x4e96, 'CjkUnifiedIdeograph4e96', [], 'Display', None],
  [ 0x4e97, 'CjkUnifiedIdeograph4e97', [], 'Display', None],
  [ 0x4e98, 'CjkUnifiedIdeograph4e98', [], 'Display', None],
  [ 0x4e99, 'CjkUnifiedIdeograph4e99', [], 'Display', None],
  [ 0x4e9a, 'CjkUnifiedIdeograph4e9a', [], 'Display', None],
  [ 0x4e9b, 'CjkUnifiedIdeograph4e9b', [], 'Display', None],
  [ 0x4e9c, 'CjkUnifiedIdeograph4e9c', [], 'Display', None],
  [ 0x4e9d, 'CjkUnifiedIdeograph4e9d', [], 'Display', None],
  [ 0x4e9e, 'CjkUnifiedIdeograph4e9e', [], 'Display', None],
  [ 0x4e9f, 'CjkUnifiedIdeograph4e9f', [], 'Display', None],
  [ 0x4ea0, 'CjkUnifiedIdeograph4ea0', [], 'Display', None],
  [ 0x4ea1, 'CjkUnifiedIdeograph4ea1', [], 'Display', None],
  [ 0x4ea2, 'CjkUnifiedIdeograph4ea2', [], 'Display', None],
  [ 0x4ea3, 'CjkUnifiedIdeograph4ea3', [], 'Display', None],
  [ 0x4ea4, 'CjkUnifiedIdeograph4ea4', [], 'Display', None],
  [ 0x4ea5, 'CjkUnifiedIdeograph4ea5', [], 'Display', None],
  [ 0x4ea6, 'CjkUnifiedIdeograph4ea6', [], 'Display', None],
  [ 0x4ea7, 'CjkUnifiedIdeograph4ea7', [], 'Display', None],
  [ 0x4ea8, 'CjkUnifiedIdeograph4ea8', [], 'Display', None],
  [ 0x4ea9, 'CjkUnifiedIdeograph4ea9', [], 'Display', None],
  [ 0x4eaa, 'CjkUnifiedIdeograph4eaa', [], 'Display', None],
  [ 0x4eab, 'CjkUnifiedIdeograph4eab', [], 'Display', None],
  [ 0x4eac, 'CjkUnifiedIdeograph4eac', [], 'Display', None],
  [ 0x4ead, 'CjkUnifiedIdeograph4ead', [], 'Display', None],
  [ 0x4eae, 'CjkUnifiedIdeograph4eae', [], 'Display', None],
  [ 0x4eaf, 'CjkUnifiedIdeograph4eaf', [], 'Display', None],
  [ 0x4eb0, 'CjkUnifiedIdeograph4eb0', [], 'Display', None],
  [ 0x4eb1, 'CjkUnifiedIdeograph4eb1', [], 'Display', None],
  [ 0x4eb2, 'CjkUnifiedIdeograph4eb2', [], 'Display', None],
  [ 0x4eb3, 'CjkUnifiedIdeograph4eb3', [], 'Display', None],
  [ 0x4eb4, 'CjkUnifiedIdeograph4eb4', [], 'Display', None],
  [ 0x4eb5, 'CjkUnifiedIdeograph4eb5', [], 'Display', None],
  [ 0x4eb6, 'CjkUnifiedIdeograph4eb6', [], 'Display', None],
  [ 0x4eb7, 'CjkUnifiedIdeograph4eb7', [], 'Display', None],
  [ 0x4eb8, 'CjkUnifiedIdeograph4eb8', [], 'Display', None],
  [ 0x4eb9, 'CjkUnifiedIdeograph4eb9', [], 'Display', None],
  [ 0x4eba, 'CjkUnifiedIdeograph4eba', [], 'Display', None],
  [ 0x4ebb, 'CjkUnifiedIdeograph4ebb', [], 'Display', None],
  [ 0x4ebc, 'CjkUnifiedIdeograph4ebc', [], 'Display', None],
  [ 0x4ebd, 'CjkUnifiedIdeograph4ebd', [], 'Display', None],
  [ 0x4ebe, 'CjkUnifiedIdeograph4ebe', [], 'Display', None],
  [ 0x4ebf, 'CjkUnifiedIdeograph4ebf', [], 'Display', None],
  [ 0x4ec0, 'CjkUnifiedIdeograph4ec0', [], 'Display', None],
  [ 0x4ec1, 'CjkUnifiedIdeograph4ec1', [], 'Display', None],
  [ 0x4ec2, 'CjkUnifiedIdeograph4ec2', [], 'Display', None],
  [ 0x4ec3, 'CjkUnifiedIdeograph4ec3', [], 'Display', None],
  [ 0x4ec4, 'CjkUnifiedIdeograph4ec4', [], 'Display', None],
  [ 0x4ec5, 'CjkUnifiedIdeograph4ec5', [], 'Display', None],
  [ 0x4ec6, 'CjkUnifiedIdeograph4ec6', [], 'Display', None],
  [ 0x4ec7, 'CjkUnifiedIdeograph4ec7', [], 'Display', None],
  [ 0x4ec8, 'CjkUnifiedIdeograph4ec8', [], 'Display', None],
  [ 0x4ec9, 'CjkUnifiedIdeograph4ec9', [], 'Display', None],
  [ 0x4eca, 'CjkUnifiedIdeograph4eca', [], 'Display', None],
  [ 0x4ecb, 'CjkUnifiedIdeograph4ecb', [], 'Display', None],
  [ 0x4ecc, 'CjkUnifiedIdeograph4ecc', [], 'Display', None],
  [ 0x4ecd, 'CjkUnifiedIdeograph4ecd', [], 'Display', None],
  [ 0x4ece, 'CjkUnifiedIdeograph4ece', [], 'Display', None],
  [ 0x4ecf, 'CjkUnifiedIdeograph4ecf', [], 'Display', None],
  [ 0x4ed0, 'CjkUnifiedIdeograph4ed0', [], 'Display', None],
  [ 0x4ed1, 'CjkUnifiedIdeograph4ed1', [], 'Display', None],
  [ 0x4ed2, 'CjkUnifiedIdeograph4ed2', [], 'Display', None],
  [ 0x4ed3, 'CjkUnifiedIdeograph4ed3', [], 'Display', None],
  [ 0x4ed4, 'CjkUnifiedIdeograph4ed4', [], 'Display', None],
  [ 0x4ed5, 'CjkUnifiedIdeograph4ed5', [], 'Display', None],
  [ 0x4ed6, 'CjkUnifiedIdeograph4ed6', [], 'Display', None],
  [ 0x4ed7, 'CjkUnifiedIdeograph4ed7', [], 'Display', None],
  [ 0x4ed8, 'CjkUnifiedIdeograph4ed8', [], 'Display', None],
  [ 0x4ed9, 'CjkUnifiedIdeograph4ed9', [], 'Display', None],
  [ 0x4eda, 'CjkUnifiedIdeograph4eda', [], 'Display', None],
  [ 0x4edb, 'CjkUnifiedIdeograph4edb', [], 'Display', None],
  [ 0x4edc, 'CjkUnifiedIdeograph4edc', [], 'Display', None],
  [ 0x4edd, 'CjkUnifiedIdeograph4edd', [], 'Display', None],
  [ 0x4ede, 'CjkUnifiedIdeograph4ede', [], 'Display', None],
  [ 0x4edf, 'CjkUnifiedIdeograph4edf', [], 'Display', None],
  [ 0x4ee0, 'CjkUnifiedIdeograph4ee0', [], 'Display', None],
  [ 0x4ee1, 'CjkUnifiedIdeograph4ee1', [], 'Display', None],
  [ 0x4ee2, 'CjkUnifiedIdeograph4ee2', [], 'Display', None],
  [ 0x4ee3, 'CjkUnifiedIdeograph4ee3', [], 'Display', None],
  [ 0x4ee4, 'CjkUnifiedIdeograph4ee4', [], 'Display', None],
  [ 0x4ee5, 'CjkUnifiedIdeograph4ee5', [], 'Display', None],
  [ 0x4ee6, 'CjkUnifiedIdeograph4ee6', [], 'Display', None],
  [ 0x4ee7, 'CjkUnifiedIdeograph4ee7', [], 'Display', None],
  [ 0x4ee8, 'CjkUnifiedIdeograph4ee8', [], 'Display', None],
  [ 0x4ee9, 'CjkUnifiedIdeograph4ee9', [], 'Display', None],
  [ 0x4eea, 'CjkUnifiedIdeograph4eea', [], 'Display', None],
  [ 0x4eeb, 'CjkUnifiedIdeograph4eeb', [], 'Display', None],
  [ 0x4eec, 'CjkUnifiedIdeograph4eec', [], 'Display', None],
  [ 0x4eed, 'CjkUnifiedIdeograph4eed', [], 'Display', None],
  [ 0x4eee, 'CjkUnifiedIdeograph4eee', [], 'Display', None],
  [ 0x4eef, 'CjkUnifiedIdeograph4eef', [], 'Display', None],
  [ 0x4ef0, 'CjkUnifiedIdeograph4ef0', [], 'Display', None],
  [ 0x4ef1, 'CjkUnifiedIdeograph4ef1', [], 'Display', None],
  [ 0x4ef2, 'CjkUnifiedIdeograph4ef2', [], 'Display', None],
  [ 0x4ef3, 'CjkUnifiedIdeograph4ef3', [], 'Display', None],
  [ 0x4ef4, 'CjkUnifiedIdeograph4ef4', [], 'Display', None],
  [ 0x4ef5, 'CjkUnifiedIdeograph4ef5', [], 'Display', None],
  [ 0x4ef6, 'CjkUnifiedIdeograph4ef6', [], 'Display', None],
  [ 0x4ef7, 'CjkUnifiedIdeograph4ef7', [], 'Display', None],
  [ 0x4ef8, 'CjkUnifiedIdeograph4ef8', [], 'Display', None],
  [ 0x4ef9, 'CjkUnifiedIdeograph4ef9', [], 'Display', None],
  [ 0x4efa, 'CjkUnifiedIdeograph4efa', [], 'Display', None],
  [ 0x4efb, 'CjkUnifiedIdeograph4efb', [], 'Display', None],
  [ 0x4efc, 'CjkUnifiedIdeograph4efc', [], 'Display', None],
  [ 0x4efd, 'CjkUnifiedIdeograph4efd', [], 'Display', None],
  [ 0x4efe, 'CjkUnifiedIdeograph4efe', [], 'Display', None],
  [ 0x4eff, 'CjkUnifiedIdeograph4eff', [], 'Display', None],
  [ 0x4f00, 'CjkUnifiedIdeograph4f00', [], 'Display', None],
  [ 0x4f01, 'CjkUnifiedIdeograph4f01', [], 'Display', None],
  [ 0x4f02, 'CjkUnifiedIdeograph4f02', [], 'Display', None],
  [ 0x4f03, 'CjkUnifiedIdeograph4f03', [], 'Display', None],
  [ 0x4f04, 'CjkUnifiedIdeograph4f04', [], 'Display', None],
  [ 0x4f05, 'CjkUnifiedIdeograph4f05', [], 'Display', None],
  [ 0x4f06, 'CjkUnifiedIdeograph4f06', [], 'Display', None],
  [ 0x4f07, 'CjkUnifiedIdeograph4f07', [], 'Display', None],
  [ 0x4f08, 'CjkUnifiedIdeograph4f08', [], 'Display', None],
  [ 0x4f09, 'CjkUnifiedIdeograph4f09', [], 'Display', None],
  [ 0x4f0a, 'CjkUnifiedIdeograph4f0a', [], 'Display', None],
  [ 0x4f0b, 'CjkUnifiedIdeograph4f0b', [], 'Display', None],
  [ 0x4f0c, 'CjkUnifiedIdeograph4f0c', [], 'Display', None],
  [ 0x4f0d, 'CjkUnifiedIdeograph4f0d', [], 'Display', None],
  [ 0x4f0e, 'CjkUnifiedIdeograph4f0e', [], 'Display', None],
  [ 0x4f0f, 'CjkUnifiedIdeograph4f0f', [], 'Display', None],
  [ 0x4f10, 'CjkUnifiedIdeograph4f10', [], 'Display', None],
  [ 0x4f11, 'CjkUnifiedIdeograph4f11', [], 'Display', None],
  [ 0x4f12, 'CjkUnifiedIdeograph4f12', [], 'Display', None],
  [ 0x4f13, 'CjkUnifiedIdeograph4f13', [], 'Display', None],
  [ 0x4f14, 'CjkUnifiedIdeograph4f14', [], 'Display', None],
  [ 0x4f15, 'CjkUnifiedIdeograph4f15', [], 'Display', None],
  [ 0x4f16, 'CjkUnifiedIdeograph4f16', [], 'Display', None],
  [ 0x4f17, 'CjkUnifiedIdeograph4f17', [], 'Display', None],
  [ 0x4f18, 'CjkUnifiedIdeograph4f18', [], 'Display', None],
  [ 0x4f19, 'CjkUnifiedIdeograph4f19', [], 'Display', None],
  [ 0x4f1a, 'CjkUnifiedIdeograph4f1a', [], 'Display', None],
  [ 0x4f1b, 'CjkUnifiedIdeograph4f1b', [], 'Display', None],
  [ 0x4f1c, 'CjkUnifiedIdeograph4f1c', [], 'Display', None],
  [ 0x4f1d, 'CjkUnifiedIdeograph4f1d', [], 'Display', None],
  [ 0x4f1e, 'CjkUnifiedIdeograph4f1e', [], 'Display', None],
  [ 0x4f1f, 'CjkUnifiedIdeograph4f1f', [], 'Display', None],
  [ 0x4f20, 'CjkUnifiedIdeograph4f20', [], 'Display', None],
  [ 0x4f21, 'CjkUnifiedIdeograph4f21', [], 'Display', None],
  [ 0x4f22, 'CjkUnifiedIdeograph4f22', [], 'Display', None],
  [ 0x4f23, 'CjkUnifiedIdeograph4f23', [], 'Display', None],
  [ 0x4f24, 'CjkUnifiedIdeograph4f24', [], 'Display', None],
  [ 0x4f25, 'CjkUnifiedIdeograph4f25', [], 'Display', None],
  [ 0x4f26, 'CjkUnifiedIdeograph4f26', [], 'Display', None],
  [ 0x4f27, 'CjkUnifiedIdeograph4f27', [], 'Display', None],
  [ 0x4f28, 'CjkUnifiedIdeograph4f28', [], 'Display', None],
  [ 0x4f29, 'CjkUnifiedIdeograph4f29', [], 'Display', None],
  [ 0x4f2a, 'CjkUnifiedIdeograph4f2a', [], 'Display', None],
  [ 0x4f2b, 'CjkUnifiedIdeograph4f2b', [], 'Display', None],
  [ 0x4f2c, 'CjkUnifiedIdeograph4f2c', [], 'Display', None],
  [ 0x4f2d, 'CjkUnifiedIdeograph4f2d', [], 'Display', None],
  [ 0x4f2e, 'CjkUnifiedIdeograph4f2e', [], 'Display', None],
  [ 0x4f2f, 'CjkUnifiedIdeograph4f2f', [], 'Display', None],
  [ 0x4f30, 'CjkUnifiedIdeograph4f30', [], 'Display', None],
  [ 0x4f31, 'CjkUnifiedIdeograph4f31', [], 'Display', None],
  [ 0x4f32, 'CjkUnifiedIdeograph4f32', [], 'Display', None],
  [ 0x4f33, 'CjkUnifiedIdeograph4f33', [], 'Display', None],
  [ 0x4f34, 'CjkUnifiedIdeograph4f34', [], 'Display', None],
  [ 0x4f35, 'CjkUnifiedIdeograph4f35', [], 'Display', None],
  [ 0x4f36, 'CjkUnifiedIdeograph4f36', [], 'Display', None],
  [ 0x4f37, 'CjkUnifiedIdeograph4f37', [], 'Display', None],
  [ 0x4f38, 'CjkUnifiedIdeograph4f38', [], 'Display', None],
  [ 0x4f39, 'CjkUnifiedIdeograph4f39', [], 'Display', None],
  [ 0x4f3a, 'CjkUnifiedIdeograph4f3a', [], 'Display', None],
  [ 0x4f3b, 'CjkUnifiedIdeograph4f3b', [], 'Display', None],
  [ 0x4f3c, 'CjkUnifiedIdeograph4f3c', [], 'Display', None],
  [ 0x4f3d, 'CjkUnifiedIdeograph4f3d', [], 'Display', None],
  [ 0x4f3e, 'CjkUnifiedIdeograph4f3e', [], 'Display', None],
  [ 0x4f3f, 'CjkUnifiedIdeograph4f3f', [], 'Display', None],
  [ 0x4f40, 'CjkUnifiedIdeograph4f40', [], 'Display', None],
  [ 0x4f41, 'CjkUnifiedIdeograph4f41', [], 'Display', None],
  [ 0x4f42, 'CjkUnifiedIdeograph4f42', [], 'Display', None],
  [ 0x4f43, 'CjkUnifiedIdeograph4f43', [], 'Display', None],
  [ 0x4f44, 'CjkUnifiedIdeograph4f44', [], 'Display', None],
  [ 0x4f45, 'CjkUnifiedIdeograph4f45', [], 'Display', None],
  [ 0x4f46, 'CjkUnifiedIdeograph4f46', [], 'Display', None],
  [ 0x4f47, 'CjkUnifiedIdeograph4f47', [], 'Display', None],
  [ 0x4f48, 'CjkUnifiedIdeograph4f48', [], 'Display', None],
  [ 0x4f49, 'CjkUnifiedIdeograph4f49', [], 'Display', None],
  [ 0x4f4a, 'CjkUnifiedIdeograph4f4a', [], 'Display', None],
  [ 0x4f4b, 'CjkUnifiedIdeograph4f4b', [], 'Display', None],
  [ 0x4f4c, 'CjkUnifiedIdeograph4f4c', [], 'Display', None],
  [ 0x4f4d, 'CjkUnifiedIdeograph4f4d', [], 'Display', None],
  [ 0x4f4e, 'CjkUnifiedIdeograph4f4e', [], 'Display', None],
  [ 0x4f4f, 'CjkUnifiedIdeograph4f4f', [], 'Display', None],
  [ 0x4f50, 'CjkUnifiedIdeograph4f50', [], 'Display', None],
  [ 0x4f51, 'CjkUnifiedIdeograph4f51', [], 'Display', None],
  [ 0x4f52, 'CjkUnifiedIdeograph4f52', [], 'Display', None],
  [ 0x4f53, 'CjkUnifiedIdeograph4f53', [], 'Display', None],
  [ 0x4f54, 'CjkUnifiedIdeograph4f54', [], 'Display', None],
  [ 0x4f55, 'CjkUnifiedIdeograph4f55', [], 'Display', None],
  [ 0x4f56, 'CjkUnifiedIdeograph4f56', [], 'Display', None],
  [ 0x4f57, 'CjkUnifiedIdeograph4f57', [], 'Display', None],
  [ 0x4f58, 'CjkUnifiedIdeograph4f58', [], 'Display', None],
  [ 0x4f59, 'CjkUnifiedIdeograph4f59', [], 'Display', None],
  [ 0x4f5a, 'CjkUnifiedIdeograph4f5a', [], 'Display', None],
  [ 0x4f5b, 'CjkUnifiedIdeograph4f5b', [], 'Display', None],
  [ 0x4f5c, 'CjkUnifiedIdeograph4f5c', [], 'Display', None],
  [ 0x4f5d, 'CjkUnifiedIdeograph4f5d', [], 'Display', None],
  [ 0x4f5e, 'CjkUnifiedIdeograph4f5e', [], 'Display', None],
  [ 0x4f5f, 'CjkUnifiedIdeograph4f5f', [], 'Display', None],
  [ 0x4f60, 'CjkUnifiedIdeograph4f60', [], 'Display', None],
  [ 0x4f61, 'CjkUnifiedIdeograph4f61', [], 'Display', None],
  [ 0x4f62, 'CjkUnifiedIdeograph4f62', [], 'Display', None],
  [ 0x4f63, 'CjkUnifiedIdeograph4f63', [], 'Display', None],
  [ 0x4f64, 'CjkUnifiedIdeograph4f64', [], 'Display', None],
  [ 0x4f65, 'CjkUnifiedIdeograph4f65', [], 'Display', None],
  [ 0x4f66, 'CjkUnifiedIdeograph4f66', [], 'Display', None],
  [ 0x4f67, 'CjkUnifiedIdeograph4f67', [], 'Display', None],
  [ 0x4f68, 'CjkUnifiedIdeograph4f68', [], 'Display', None],
  [ 0x4f69, 'CjkUnifiedIdeograph4f69', [], 'Display', None],
  [ 0x4f6a, 'CjkUnifiedIdeograph4f6a', [], 'Display', None],
  [ 0x4f6b, 'CjkUnifiedIdeograph4f6b', [], 'Display', None],
  [ 0x4f6c, 'CjkUnifiedIdeograph4f6c', [], 'Display', None],
  [ 0x4f6d, 'CjkUnifiedIdeograph4f6d', [], 'Display', None],
  [ 0x4f6e, 'CjkUnifiedIdeograph4f6e', [], 'Display', None],
  [ 0x4f6f, 'CjkUnifiedIdeograph4f6f', [], 'Display', None],
  [ 0x4f70, 'CjkUnifiedIdeograph4f70', [], 'Display', None],
  [ 0x4f71, 'CjkUnifiedIdeograph4f71', [], 'Display', None],
  [ 0x4f72, 'CjkUnifiedIdeograph4f72', [], 'Display', None],
  [ 0x4f73, 'CjkUnifiedIdeograph4f73', [], 'Display', None],
  [ 0x4f74, 'CjkUnifiedIdeograph4f74', [], 'Display', None],
  [ 0x4f75, 'CjkUnifiedIdeograph4f75', [], 'Display', None],
  [ 0x4f76, 'CjkUnifiedIdeograph4f76', [], 'Display', None],
  [ 0x4f77, 'CjkUnifiedIdeograph4f77', [], 'Display', None],
  [ 0x4f78, 'CjkUnifiedIdeograph4f78', [], 'Display', None],
  [ 0x4f79, 'CjkUnifiedIdeograph4f79', [], 'Display', None],
  [ 0x4f7a, 'CjkUnifiedIdeograph4f7a', [], 'Display', None],
  [ 0x4f7b, 'CjkUnifiedIdeograph4f7b', [], 'Display', None],
  [ 0x4f7c, 'CjkUnifiedIdeograph4f7c', [], 'Display', None],
  [ 0x4f7d, 'CjkUnifiedIdeograph4f7d', [], 'Display', None],
  [ 0x4f7e, 'CjkUnifiedIdeograph4f7e', [], 'Display', None],
  [ 0x4f7f, 'CjkUnifiedIdeograph4f7f', [], 'Display', None],
  [ 0x4f80, 'CjkUnifiedIdeograph4f80', [], 'Display', None],
  [ 0x4f81, 'CjkUnifiedIdeograph4f81', [], 'Display', None],
  [ 0x4f82, 'CjkUnifiedIdeograph4f82', [], 'Display', None],
  [ 0x4f83, 'CjkUnifiedIdeograph4f83', [], 'Display', None],
  [ 0x4f84, 'CjkUnifiedIdeograph4f84', [], 'Display', None],
  [ 0x4f85, 'CjkUnifiedIdeograph4f85', [], 'Display', None],
  [ 0x4f86, 'CjkUnifiedIdeograph4f86', [], 'Display', None],
  [ 0x4f87, 'CjkUnifiedIdeograph4f87', [], 'Display', None],
  [ 0x4f88, 'CjkUnifiedIdeograph4f88', [], 'Display', None],
  [ 0x4f89, 'CjkUnifiedIdeograph4f89', [], 'Display', None],
  [ 0x4f8a, 'CjkUnifiedIdeograph4f8a', [], 'Display', None],
  [ 0x4f8b, 'CjkUnifiedIdeograph4f8b', [], 'Display', None],
  [ 0x4f8c, 'CjkUnifiedIdeograph4f8c', [], 'Display', None],
  [ 0x4f8d, 'CjkUnifiedIdeograph4f8d', [], 'Display', None],
  [ 0x4f8e, 'CjkUnifiedIdeograph4f8e', [], 'Display', None],
  [ 0x4f8f, 'CjkUnifiedIdeograph4f8f', [], 'Display', None],
  [ 0x4f90, 'CjkUnifiedIdeograph4f90', [], 'Display', None],
  [ 0x4f91, 'CjkUnifiedIdeograph4f91', [], 'Display', None],
  [ 0x4f92, 'CjkUnifiedIdeograph4f92', [], 'Display', None],
  [ 0x4f93, 'CjkUnifiedIdeograph4f93', [], 'Display', None],
  [ 0x4f94, 'CjkUnifiedIdeograph4f94', [], 'Display', None],
  [ 0x4f95, 'CjkUnifiedIdeograph4f95', [], 'Display', None],
  [ 0x4f96, 'CjkUnifiedIdeograph4f96', [], 'Display', None],
  [ 0x4f97, 'CjkUnifiedIdeograph4f97', [], 'Display', None],
  [ 0x4f98, 'CjkUnifiedIdeograph4f98', [], 'Display', None],
  [ 0x4f99, 'CjkUnifiedIdeograph4f99', [], 'Display', None],
  [ 0x4f9a, 'CjkUnifiedIdeograph4f9a', [], 'Display', None],
  [ 0x4f9b, 'CjkUnifiedIdeograph4f9b', [], 'Display', None],
  [ 0x4f9c, 'CjkUnifiedIdeograph4f9c', [], 'Display', None],
  [ 0x4f9d, 'CjkUnifiedIdeograph4f9d', [], 'Display', None],
  [ 0x4f9e, 'CjkUnifiedIdeograph4f9e', [], 'Display', None],
  [ 0x4f9f, 'CjkUnifiedIdeograph4f9f', [], 'Display', None],
  [ 0x4fa0, 'CjkUnifiedIdeograph4fa0', [], 'Display', None],
  [ 0x4fa1, 'CjkUnifiedIdeograph4fa1', [], 'Display', None],
  [ 0x4fa2, 'CjkUnifiedIdeograph4fa2', [], 'Display', None],
  [ 0x4fa3, 'CjkUnifiedIdeograph4fa3', [], 'Display', None],
  [ 0x4fa4, 'CjkUnifiedIdeograph4fa4', [], 'Display', None],
  [ 0x4fa5, 'CjkUnifiedIdeograph4fa5', [], 'Display', None],
  [ 0x4fa6, 'CjkUnifiedIdeograph4fa6', [], 'Display', None],
  [ 0x4fa7, 'CjkUnifiedIdeograph4fa7', [], 'Display', None],
  [ 0x4fa8, 'CjkUnifiedIdeograph4fa8', [], 'Display', None],
  [ 0x4fa9, 'CjkUnifiedIdeograph4fa9', [], 'Display', None],
  [ 0x4faa, 'CjkUnifiedIdeograph4faa', [], 'Display', None],
  [ 0x4fab, 'CjkUnifiedIdeograph4fab', [], 'Display', None],
  [ 0x4fac, 'CjkUnifiedIdeograph4fac', [], 'Display', None],
  [ 0x4fad, 'CjkUnifiedIdeograph4fad', [], 'Display', None],
  [ 0x4fae, 'CjkUnifiedIdeograph4fae', [], 'Display', None],
  [ 0x4faf, 'CjkUnifiedIdeograph4faf', [], 'Display', None],
  [ 0x4fb0, 'CjkUnifiedIdeograph4fb0', [], 'Display', None],
  [ 0x4fb1, 'CjkUnifiedIdeograph4fb1', [], 'Display', None],
  [ 0x4fb2, 'CjkUnifiedIdeograph4fb2', [], 'Display', None],
  [ 0x4fb3, 'CjkUnifiedIdeograph4fb3', [], 'Display', None],
  [ 0x4fb4, 'CjkUnifiedIdeograph4fb4', [], 'Display', None],
  [ 0x4fb5, 'CjkUnifiedIdeograph4fb5', [], 'Display', None],
  [ 0x4fb6, 'CjkUnifiedIdeograph4fb6', [], 'Display', None],
  [ 0x4fb7, 'CjkUnifiedIdeograph4fb7', [], 'Display', None],
  [ 0x4fb8, 'CjkUnifiedIdeograph4fb8', [], 'Display', None],
  [ 0x4fb9, 'CjkUnifiedIdeograph4fb9', [], 'Display', None],
  [ 0x4fba, 'CjkUnifiedIdeograph4fba', [], 'Display', None],
  [ 0x4fbb, 'CjkUnifiedIdeograph4fbb', [], 'Display', None],
  [ 0x4fbc, 'CjkUnifiedIdeograph4fbc', [], 'Display', None],
  [ 0x4fbd, 'CjkUnifiedIdeograph4fbd', [], 'Display', None],
  [ 0x4fbe, 'CjkUnifiedIdeograph4fbe', [], 'Display', None],
  [ 0x4fbf, 'CjkUnifiedIdeograph4fbf', [], 'Display', None],
  [ 0x4fc0, 'CjkUnifiedIdeograph4fc0', [], 'Display', None],
  [ 0x4fc1, 'CjkUnifiedIdeograph4fc1', [], 'Display', None],
  [ 0x4fc2, 'CjkUnifiedIdeograph4fc2', [], 'Display', None],
  [ 0x4fc3, 'CjkUnifiedIdeograph4fc3', [], 'Display', None],
  [ 0x4fc4, 'CjkUnifiedIdeograph4fc4', [], 'Display', None],
  [ 0x4fc5, 'CjkUnifiedIdeograph4fc5', [], 'Display', None],
  [ 0x4fc6, 'CjkUnifiedIdeograph4fc6', [], 'Display', None],
  [ 0x4fc7, 'CjkUnifiedIdeograph4fc7', [], 'Display', None],
  [ 0x4fc8, 'CjkUnifiedIdeograph4fc8', [], 'Display', None],
  [ 0x4fc9, 'CjkUnifiedIdeograph4fc9', [], 'Display', None],
  [ 0x4fca, 'CjkUnifiedIdeograph4fca', [], 'Display', None],
  [ 0x4fcb, 'CjkUnifiedIdeograph4fcb', [], 'Display', None],
  [ 0x4fcc, 'CjkUnifiedIdeograph4fcc', [], 'Display', None],
  [ 0x4fcd, 'CjkUnifiedIdeograph4fcd', [], 'Display', None],
  [ 0x4fce, 'CjkUnifiedIdeograph4fce', [], 'Display', None],
  [ 0x4fcf, 'CjkUnifiedIdeograph4fcf', [], 'Display', None],
  [ 0x4fd0, 'CjkUnifiedIdeograph4fd0', [], 'Display', None],
  [ 0x4fd1, 'CjkUnifiedIdeograph4fd1', [], 'Display', None],
  [ 0x4fd2, 'CjkUnifiedIdeograph4fd2', [], 'Display', None],
  [ 0x4fd3, 'CjkUnifiedIdeograph4fd3', [], 'Display', None],
  [ 0x4fd4, 'CjkUnifiedIdeograph4fd4', [], 'Display', None],
  [ 0x4fd5, 'CjkUnifiedIdeograph4fd5', [], 'Display', None],
  [ 0x4fd6, 'CjkUnifiedIdeograph4fd6', [], 'Display', None],
  [ 0x4fd7, 'CjkUnifiedIdeograph4fd7', [], 'Display', None],
  [ 0x4fd8, 'CjkUnifiedIdeograph4fd8', [], 'Display', None],
  [ 0x4fd9, 'CjkUnifiedIdeograph4fd9', [], 'Display', None],
  [ 0x4fda, 'CjkUnifiedIdeograph4fda', [], 'Display', None],
  [ 0x4fdb, 'CjkUnifiedIdeograph4fdb', [], 'Display', None],
  [ 0x4fdc, 'CjkUnifiedIdeograph4fdc', [], 'Display', None],
  [ 0x4fdd, 'CjkUnifiedIdeograph4fdd', [], 'Display', None],
  [ 0x4fde, 'CjkUnifiedIdeograph4fde', [], 'Display', None],
  [ 0x4fdf, 'CjkUnifiedIdeograph4fdf', [], 'Display', None],
  [ 0x4fe0, 'CjkUnifiedIdeograph4fe0', [], 'Display', None],
  [ 0x4fe1, 'CjkUnifiedIdeograph4fe1', [], 'Display', None],
  [ 0x4fe2, 'CjkUnifiedIdeograph4fe2', [], 'Display', None],
  [ 0x4fe3, 'CjkUnifiedIdeograph4fe3', [], 'Display', None],
  [ 0x4fe4, 'CjkUnifiedIdeograph4fe4', [], 'Display', None],
  [ 0x4fe5, 'CjkUnifiedIdeograph4fe5', [], 'Display', None],
  [ 0x4fe6, 'CjkUnifiedIdeograph4fe6', [], 'Display', None],
  [ 0x4fe7, 'CjkUnifiedIdeograph4fe7', [], 'Display', None],
  [ 0x4fe8, 'CjkUnifiedIdeograph4fe8', [], 'Display', None],
  [ 0x4fe9, 'CjkUnifiedIdeograph4fe9', [], 'Display', None],
  [ 0x4fea, 'CjkUnifiedIdeograph4fea', [], 'Display', None],
  [ 0x4feb, 'CjkUnifiedIdeograph4feb', [], 'Display', None],
  [ 0x4fec, 'CjkUnifiedIdeograph4fec', [], 'Display', None],
  [ 0x4fed, 'CjkUnifiedIdeograph4fed', [], 'Display', None],
  [ 0x4fee, 'CjkUnifiedIdeograph4fee', [], 'Display', None],
  [ 0x4fef, 'CjkUnifiedIdeograph4fef', [], 'Display', None],
  [ 0x4ff0, 'CjkUnifiedIdeograph4ff0', [], 'Display', None],
  [ 0x4ff1, 'CjkUnifiedIdeograph4ff1', [], 'Display', None],
  [ 0x4ff2, 'CjkUnifiedIdeograph4ff2', [], 'Display', None],
  [ 0x4ff3, 'CjkUnifiedIdeograph4ff3', [], 'Display', None],
  [ 0x4ff4, 'CjkUnifiedIdeograph4ff4', [], 'Display', None],
  [ 0x4ff5, 'CjkUnifiedIdeograph4ff5', [], 'Display', None],
  [ 0x4ff6, 'CjkUnifiedIdeograph4ff6', [], 'Display', None],
  [ 0x4ff7, 'CjkUnifiedIdeograph4ff7', [], 'Display', None],
  [ 0x4ff8, 'CjkUnifiedIdeograph4ff8', [], 'Display', None],
  [ 0x4ff9, 'CjkUnifiedIdeograph4ff9', [], 'Display', None],
  [ 0x4ffa, 'CjkUnifiedIdeograph4ffa', [], 'Display', None],
  [ 0x4ffb, 'CjkUnifiedIdeograph4ffb', [], 'Display', None],
  [ 0x4ffc, 'CjkUnifiedIdeograph4ffc', [], 'Display', None],
  [ 0x4ffd, 'CjkUnifiedIdeograph4ffd', [], 'Display', None],
  [ 0x4ffe, 'CjkUnifiedIdeograph4ffe', [], 'Display', None],
  [ 0x4fff, 'CjkUnifiedIdeograph4fff', [], 'Display', None],
  [ 0x5000, 'CjkUnifiedIdeograph5000', [], 'Display', None],
  [ 0x5001, 'CjkUnifiedIdeograph5001', [], 'Display', None],
  [ 0x5002, 'CjkUnifiedIdeograph5002', [], 'Display', None],
  [ 0x5003, 'CjkUnifiedIdeograph5003', [], 'Display', None],
  [ 0x5004, 'CjkUnifiedIdeograph5004', [], 'Display', None],
  [ 0x5005, 'CjkUnifiedIdeograph5005', [], 'Display', None],
  [ 0x5006, 'CjkUnifiedIdeograph5006', [], 'Display', None],
  [ 0x5007, 'CjkUnifiedIdeograph5007', [], 'Display', None],
  [ 0x5008, 'CjkUnifiedIdeograph5008', [], 'Display', None],
  [ 0x5009, 'CjkUnifiedIdeograph5009', [], 'Display', None],
  [ 0x500a, 'CjkUnifiedIdeograph500a', [], 'Display', None],
  [ 0x500b, 'CjkUnifiedIdeograph500b', [], 'Display', None],
  [ 0x500c, 'CjkUnifiedIdeograph500c', [], 'Display', None],
  [ 0x500d, 'CjkUnifiedIdeograph500d', [], 'Display', None],
  [ 0x500e, 'CjkUnifiedIdeograph500e', [], 'Display', None],
  [ 0x500f, 'CjkUnifiedIdeograph500f', [], 'Display', None],
  [ 0x5010, 'CjkUnifiedIdeograph5010', [], 'Display', None],
  [ 0x5011, 'CjkUnifiedIdeograph5011', [], 'Display', None],
  [ 0x5012, 'CjkUnifiedIdeograph5012', [], 'Display', None],
  [ 0x5013, 'CjkUnifiedIdeograph5013', [], 'Display', None],
  [ 0x5014, 'CjkUnifiedIdeograph5014', [], 'Display', None],
  [ 0x5015, 'CjkUnifiedIdeograph5015', [], 'Display', None],
  [ 0x5016, 'CjkUnifiedIdeograph5016', [], 'Display', None],
  [ 0x5017, 'CjkUnifiedIdeograph5017', [], 'Display', None],
  [ 0x5018, 'CjkUnifiedIdeograph5018', [], 'Display', None],
  [ 0x5019, 'CjkUnifiedIdeograph5019', [], 'Display', None],
  [ 0x501a, 'CjkUnifiedIdeograph501a', [], 'Display', None],
  [ 0x501b, 'CjkUnifiedIdeograph501b', [], 'Display', None],
  [ 0x501c, 'CjkUnifiedIdeograph501c', [], 'Display', None],
  [ 0x501d, 'CjkUnifiedIdeograph501d', [], 'Display', None],
  [ 0x501e, 'CjkUnifiedIdeograph501e', [], 'Display', None],
  [ 0x501f, 'CjkUnifiedIdeograph501f', [], 'Display', None],
  [ 0x5020, 'CjkUnifiedIdeograph5020', [], 'Display', None],
  [ 0x5021, 'CjkUnifiedIdeograph5021', [], 'Display', None],
  [ 0x5022, 'CjkUnifiedIdeograph5022', [], 'Display', None],
  [ 0x5023, 'CjkUnifiedIdeograph5023', [], 'Display', None],
  [ 0x5024, 'CjkUnifiedIdeograph5024', [], 'Display', None],
  [ 0x5025, 'CjkUnifiedIdeograph5025', [], 'Display', None],
  [ 0x5026, 'CjkUnifiedIdeograph5026', [], 'Display', None],
  [ 0x5027, 'CjkUnifiedIdeograph5027', [], 'Display', None],
  [ 0x5028, 'CjkUnifiedIdeograph5028', [], 'Display', None],
  [ 0x5029, 'CjkUnifiedIdeograph5029', [], 'Display', None],
  [ 0x502a, 'CjkUnifiedIdeograph502a', [], 'Display', None],
  [ 0x502b, 'CjkUnifiedIdeograph502b', [], 'Display', None],
  [ 0x502c, 'CjkUnifiedIdeograph502c', [], 'Display', None],
  [ 0x502d, 'CjkUnifiedIdeograph502d', [], 'Display', None],
  [ 0x502e, 'CjkUnifiedIdeograph502e', [], 'Display', None],
  [ 0x502f, 'CjkUnifiedIdeograph502f', [], 'Display', None],
  [ 0x5030, 'CjkUnifiedIdeograph5030', [], 'Display', None],
  [ 0x5031, 'CjkUnifiedIdeograph5031', [], 'Display', None],
  [ 0x5032, 'CjkUnifiedIdeograph5032', [], 'Display', None],
  [ 0x5033, 'CjkUnifiedIdeograph5033', [], 'Display', None],
  [ 0x5034, 'CjkUnifiedIdeograph5034', [], 'Display', None],
  [ 0x5035, 'CjkUnifiedIdeograph5035', [], 'Display', None],
  [ 0x5036, 'CjkUnifiedIdeograph5036', [], 'Display', None],
  [ 0x5037, 'CjkUnifiedIdeograph5037', [], 'Display', None],
  [ 0x5038, 'CjkUnifiedIdeograph5038', [], 'Display', None],
  [ 0x5039, 'CjkUnifiedIdeograph5039', [], 'Display', None],
  [ 0x503a, 'CjkUnifiedIdeograph503a', [], 'Display', None],
  [ 0x503b, 'CjkUnifiedIdeograph503b', [], 'Display', None],
  [ 0x503c, 'CjkUnifiedIdeograph503c', [], 'Display', None],
  [ 0x503d, 'CjkUnifiedIdeograph503d', [], 'Display', None],
  [ 0x503e, 'CjkUnifiedIdeograph503e', [], 'Display', None],
  [ 0x503f, 'CjkUnifiedIdeograph503f', [], 'Display', None],
  [ 0x5040, 'CjkUnifiedIdeograph5040', [], 'Display', None],
  [ 0x5041, 'CjkUnifiedIdeograph5041', [], 'Display', None],
  [ 0x5042, 'CjkUnifiedIdeograph5042', [], 'Display', None],
  [ 0x5043, 'CjkUnifiedIdeograph5043', [], 'Display', None],
  [ 0x5044, 'CjkUnifiedIdeograph5044', [], 'Display', None],
  [ 0x5045, 'CjkUnifiedIdeograph5045', [], 'Display', None],
  [ 0x5046, 'CjkUnifiedIdeograph5046', [], 'Display', None],
  [ 0x5047, 'CjkUnifiedIdeograph5047', [], 'Display', None],
  [ 0x5048, 'CjkUnifiedIdeograph5048', [], 'Display', None],
  [ 0x5049, 'CjkUnifiedIdeograph5049', [], 'Display', None],
  [ 0x504a, 'CjkUnifiedIdeograph504a', [], 'Display', None],
  [ 0x504b, 'CjkUnifiedIdeograph504b', [], 'Display', None],
  [ 0x504c, 'CjkUnifiedIdeograph504c', [], 'Display', None],
  [ 0x504d, 'CjkUnifiedIdeograph504d', [], 'Display', None],
  [ 0x504e, 'CjkUnifiedIdeograph504e', [], 'Display', None],
  [ 0x504f, 'CjkUnifiedIdeograph504f', [], 'Display', None],
  [ 0x5050, 'CjkUnifiedIdeograph5050', [], 'Display', None],
  [ 0x5051, 'CjkUnifiedIdeograph5051', [], 'Display', None],
  [ 0x5052, 'CjkUnifiedIdeograph5052', [], 'Display', None],
  [ 0x5053, 'CjkUnifiedIdeograph5053', [], 'Display', None],
  [ 0x5054, 'CjkUnifiedIdeograph5054', [], 'Display', None],
  [ 0x5055, 'CjkUnifiedIdeograph5055', [], 'Display', None],
  [ 0x5056, 'CjkUnifiedIdeograph5056', [], 'Display', None],
  [ 0x5057, 'CjkUnifiedIdeograph5057', [], 'Display', None],
  [ 0x5058, 'CjkUnifiedIdeograph5058', [], 'Display', None],
  [ 0x5059, 'CjkUnifiedIdeograph5059', [], 'Display', None],
  [ 0x505a, 'CjkUnifiedIdeograph505a', [], 'Display', None],
  [ 0x505b, 'CjkUnifiedIdeograph505b', [], 'Display', None],
  [ 0x505c, 'CjkUnifiedIdeograph505c', [], 'Display', None],
  [ 0x505d, 'CjkUnifiedIdeograph505d', [], 'Display', None],
  [ 0x505e, 'CjkUnifiedIdeograph505e', [], 'Display', None],
  [ 0x505f, 'CjkUnifiedIdeograph505f', [], 'Display', None],
  [ 0x5060, 'CjkUnifiedIdeograph5060', [], 'Display', None],
  [ 0x5061, 'CjkUnifiedIdeograph5061', [], 'Display', None],
  [ 0x5062, 'CjkUnifiedIdeograph5062', [], 'Display', None],
  [ 0x5063, 'CjkUnifiedIdeograph5063', [], 'Display', None],
  [ 0x5064, 'CjkUnifiedIdeograph5064', [], 'Display', None],
  [ 0x5065, 'CjkUnifiedIdeograph5065', [], 'Display', None],
  [ 0x5066, 'CjkUnifiedIdeograph5066', [], 'Display', None],
  [ 0x5067, 'CjkUnifiedIdeograph5067', [], 'Display', None],
  [ 0x5068, 'CjkUnifiedIdeograph5068', [], 'Display', None],
  [ 0x5069, 'CjkUnifiedIdeograph5069', [], 'Display', None],
  [ 0x506a, 'CjkUnifiedIdeograph506a', [], 'Display', None],
  [ 0x506b, 'CjkUnifiedIdeograph506b', [], 'Display', None],
  [ 0x506c, 'CjkUnifiedIdeograph506c', [], 'Display', None],
  [ 0x506d, 'CjkUnifiedIdeograph506d', [], 'Display', None],
  [ 0x506e, 'CjkUnifiedIdeograph506e', [], 'Display', None],
  [ 0x506f, 'CjkUnifiedIdeograph506f', [], 'Display', None],
  [ 0x5070, 'CjkUnifiedIdeograph5070', [], 'Display', None],
  [ 0x5071, 'CjkUnifiedIdeograph5071', [], 'Display', None],
  [ 0x5072, 'CjkUnifiedIdeograph5072', [], 'Display', None],
  [ 0x5073, 'CjkUnifiedIdeograph5073', [], 'Display', None],
  [ 0x5074, 'CjkUnifiedIdeograph5074', [], 'Display', None],
  [ 0x5075, 'CjkUnifiedIdeograph5075', [], 'Display', None],
  [ 0x5076, 'CjkUnifiedIdeograph5076', [], 'Display', None],
  [ 0x5077, 'CjkUnifiedIdeograph5077', [], 'Display', None],
  [ 0x5078, 'CjkUnifiedIdeograph5078', [], 'Display', None],
  [ 0x5079, 'CjkUnifiedIdeograph5079', [], 'Display', None],
  [ 0x507a, 'CjkUnifiedIdeograph507a', [], 'Display', None],
  [ 0x507b, 'CjkUnifiedIdeograph507b', [], 'Display', None],
  [ 0x507c, 'CjkUnifiedIdeograph507c', [], 'Display', None],
  [ 0x507d, 'CjkUnifiedIdeograph507d', [], 'Display', None],
  [ 0x507e, 'CjkUnifiedIdeograph507e', [], 'Display', None],
  [ 0x507f, 'CjkUnifiedIdeograph507f', [], 'Display', None],
  [ 0x5080, 'CjkUnifiedIdeograph5080', [], 'Display', None],
  [ 0x5081, 'CjkUnifiedIdeograph5081', [], 'Display', None],
  [ 0x5082, 'CjkUnifiedIdeograph5082', [], 'Display', None],
  [ 0x5083, 'CjkUnifiedIdeograph5083', [], 'Display', None],
  [ 0x5084, 'CjkUnifiedIdeograph5084', [], 'Display', None],
  [ 0x5085, 'CjkUnifiedIdeograph5085', [], 'Display', None],
  [ 0x5086, 'CjkUnifiedIdeograph5086', [], 'Display', None],
  [ 0x5087, 'CjkUnifiedIdeograph5087', [], 'Display', None],
  [ 0x5088, 'CjkUnifiedIdeograph5088', [], 'Display', None],
  [ 0x5089, 'CjkUnifiedIdeograph5089', [], 'Display', None],
  [ 0x508a, 'CjkUnifiedIdeograph508a', [], 'Display', None],
  [ 0x508b, 'CjkUnifiedIdeograph508b', [], 'Display', None],
  [ 0x508c, 'CjkUnifiedIdeograph508c', [], 'Display', None],
  [ 0x508d, 'CjkUnifiedIdeograph508d', [], 'Display', None],
  [ 0x508e, 'CjkUnifiedIdeograph508e', [], 'Display', None],
  [ 0x508f, 'CjkUnifiedIdeograph508f', [], 'Display', None],
  [ 0x5090, 'CjkUnifiedIdeograph5090', [], 'Display', None],
  [ 0x5091, 'CjkUnifiedIdeograph5091', [], 'Display', None],
  [ 0x5092, 'CjkUnifiedIdeograph5092', [], 'Display', None],
  [ 0x5093, 'CjkUnifiedIdeograph5093', [], 'Display', None],
  [ 0x5094, 'CjkUnifiedIdeograph5094', [], 'Display', None],
  [ 0x5095, 'CjkUnifiedIdeograph5095', [], 'Display', None],
  [ 0x5096, 'CjkUnifiedIdeograph5096', [], 'Display', None],
  [ 0x5097, 'CjkUnifiedIdeograph5097', [], 'Display', None],
  [ 0x5098, 'CjkUnifiedIdeograph5098', [], 'Display', None],
  [ 0x5099, 'CjkUnifiedIdeograph5099', [], 'Display', None],
  [ 0x509a, 'CjkUnifiedIdeograph509a', [], 'Display', None],
  [ 0x509b, 'CjkUnifiedIdeograph509b', [], 'Display', None],
  [ 0x509c, 'CjkUnifiedIdeograph509c', [], 'Display', None],
  [ 0x509d, 'CjkUnifiedIdeograph509d', [], 'Display', None],
  [ 0x509e, 'CjkUnifiedIdeograph509e', [], 'Display', None],
  [ 0x509f, 'CjkUnifiedIdeograph509f', [], 'Display', None],
  [ 0x50a0, 'CjkUnifiedIdeograph50a0', [], 'Display', None],
  [ 0x50a1, 'CjkUnifiedIdeograph50a1', [], 'Display', None],
  [ 0x50a2, 'CjkUnifiedIdeograph50a2', [], 'Display', None],
  [ 0x50a3, 'CjkUnifiedIdeograph50a3', [], 'Display', None],
  [ 0x50a4, 'CjkUnifiedIdeograph50a4', [], 'Display', None],
  [ 0x50a5, 'CjkUnifiedIdeograph50a5', [], 'Display', None],
  [ 0x50a6, 'CjkUnifiedIdeograph50a6', [], 'Display', None],
  [ 0x50a7, 'CjkUnifiedIdeograph50a7', [], 'Display', None],
  [ 0x50a8, 'CjkUnifiedIdeograph50a8', [], 'Display', None],
  [ 0x50a9, 'CjkUnifiedIdeograph50a9', [], 'Display', None],
  [ 0x50aa, 'CjkUnifiedIdeograph50aa', [], 'Display', None],
  [ 0x50ab, 'CjkUnifiedIdeograph50ab', [], 'Display', None],
  [ 0x50ac, 'CjkUnifiedIdeograph50ac', [], 'Display', None],
  [ 0x50ad, 'CjkUnifiedIdeograph50ad', [], 'Display', None],
  [ 0x50ae, 'CjkUnifiedIdeograph50ae', [], 'Display', None],
  [ 0x50af, 'CjkUnifiedIdeograph50af', [], 'Display', None],
  [ 0x50b0, 'CjkUnifiedIdeograph50b0', [], 'Display', None],
  [ 0x50b1, 'CjkUnifiedIdeograph50b1', [], 'Display', None],
  [ 0x50b2, 'CjkUnifiedIdeograph50b2', [], 'Display', None],
  [ 0x50b3, 'CjkUnifiedIdeograph50b3', [], 'Display', None],
  [ 0x50b4, 'CjkUnifiedIdeograph50b4', [], 'Display', None],
  [ 0x50b5, 'CjkUnifiedIdeograph50b5', [], 'Display', None],
  [ 0x50b6, 'CjkUnifiedIdeograph50b6', [], 'Display', None],
  [ 0x50b7, 'CjkUnifiedIdeograph50b7', [], 'Display', None],
  [ 0x50b8, 'CjkUnifiedIdeograph50b8', [], 'Display', None],
  [ 0x50b9, 'CjkUnifiedIdeograph50b9', [], 'Display', None],
  [ 0x50ba, 'CjkUnifiedIdeograph50ba', [], 'Display', None],
  [ 0x50bb, 'CjkUnifiedIdeograph50bb', [], 'Display', None],
  [ 0x50bc, 'CjkUnifiedIdeograph50bc', [], 'Display', None],
  [ 0x50bd, 'CjkUnifiedIdeograph50bd', [], 'Display', None],
  [ 0x50be, 'CjkUnifiedIdeograph50be', [], 'Display', None],
  [ 0x50bf, 'CjkUnifiedIdeograph50bf', [], 'Display', None],
  [ 0x50c0, 'CjkUnifiedIdeograph50c0', [], 'Display', None],
  [ 0x50c1, 'CjkUnifiedIdeograph50c1', [], 'Display', None],
  [ 0x50c2, 'CjkUnifiedIdeograph50c2', [], 'Display', None],
  [ 0x50c3, 'CjkUnifiedIdeograph50c3', [], 'Display', None],
  [ 0x50c4, 'CjkUnifiedIdeograph50c4', [], 'Display', None],
  [ 0x50c5, 'CjkUnifiedIdeograph50c5', [], 'Display', None],
  [ 0x50c6, 'CjkUnifiedIdeograph50c6', [], 'Display', None],
  [ 0x50c7, 'CjkUnifiedIdeograph50c7', [], 'Display', None],
  [ 0x50c8, 'CjkUnifiedIdeograph50c8', [], 'Display', None],
  [ 0x50c9, 'CjkUnifiedIdeograph50c9', [], 'Display', None],
  [ 0x50ca, 'CjkUnifiedIdeograph50ca', [], 'Display', None],
  [ 0x50cb, 'CjkUnifiedIdeograph50cb', [], 'Display', None],
  [ 0x50cc, 'CjkUnifiedIdeograph50cc', [], 'Display', None],
  [ 0x50cd, 'CjkUnifiedIdeograph50cd', [], 'Display', None],
  [ 0x50ce, 'CjkUnifiedIdeograph50ce', [], 'Display', None],
  [ 0x50cf, 'CjkUnifiedIdeograph50cf', [], 'Display', None],
  [ 0x50d0, 'CjkUnifiedIdeograph50d0', [], 'Display', None],
  [ 0x50d1, 'CjkUnifiedIdeograph50d1', [], 'Display', None],
  [ 0x50d2, 'CjkUnifiedIdeograph50d2', [], 'Display', None],
  [ 0x50d3, 'CjkUnifiedIdeograph50d3', [], 'Display', None],
  [ 0x50d4, 'CjkUnifiedIdeograph50d4', [], 'Display', None],
  [ 0x50d5, 'CjkUnifiedIdeograph50d5', [], 'Display', None],
  [ 0x50d6, 'CjkUnifiedIdeograph50d6', [], 'Display', None],
  [ 0x50d7, 'CjkUnifiedIdeograph50d7', [], 'Display', None],
  [ 0x50d8, 'CjkUnifiedIdeograph50d8', [], 'Display', None],
  [ 0x50d9, 'CjkUnifiedIdeograph50d9', [], 'Display', None],
  [ 0x50da, 'CjkUnifiedIdeograph50da', [], 'Display', None],
  [ 0x50db, 'CjkUnifiedIdeograph50db', [], 'Display', None],
  [ 0x50dc, 'CjkUnifiedIdeograph50dc', [], 'Display', None],
  [ 0x50dd, 'CjkUnifiedIdeograph50dd', [], 'Display', None],
  [ 0x50de, 'CjkUnifiedIdeograph50de', [], 'Display', None],
  [ 0x50df, 'CjkUnifiedIdeograph50df', [], 'Display', None],
  [ 0x50e0, 'CjkUnifiedIdeograph50e0', [], 'Display', None],
  [ 0x50e1, 'CjkUnifiedIdeograph50e1', [], 'Display', None],
  [ 0x50e2, 'CjkUnifiedIdeograph50e2', [], 'Display', None],
  [ 0x50e3, 'CjkUnifiedIdeograph50e3', [], 'Display', None],
  [ 0x50e4, 'CjkUnifiedIdeograph50e4', [], 'Display', None],
  [ 0x50e5, 'CjkUnifiedIdeograph50e5', [], 'Display', None],
  [ 0x50e6, 'CjkUnifiedIdeograph50e6', [], 'Display', None],
  [ 0x50e7, 'CjkUnifiedIdeograph50e7', [], 'Display', None],
  [ 0x50e8, 'CjkUnifiedIdeograph50e8', [], 'Display', None],
  [ 0x50e9, 'CjkUnifiedIdeograph50e9', [], 'Display', None],
  [ 0x50ea, 'CjkUnifiedIdeograph50ea', [], 'Display', None],
  [ 0x50eb, 'CjkUnifiedIdeograph50eb', [], 'Display', None],
  [ 0x50ec, 'CjkUnifiedIdeograph50ec', [], 'Display', None],
  [ 0x50ed, 'CjkUnifiedIdeograph50ed', [], 'Display', None],
  [ 0x50ee, 'CjkUnifiedIdeograph50ee', [], 'Display', None],
  [ 0x50ef, 'CjkUnifiedIdeograph50ef', [], 'Display', None],
  [ 0x50f0, 'CjkUnifiedIdeograph50f0', [], 'Display', None],
  [ 0x50f1, 'CjkUnifiedIdeograph50f1', [], 'Display', None],
  [ 0x50f2, 'CjkUnifiedIdeograph50f2', [], 'Display', None],
  [ 0x50f3, 'CjkUnifiedIdeograph50f3', [], 'Display', None],
  [ 0x50f4, 'CjkUnifiedIdeograph50f4', [], 'Display', None],
  [ 0x50f5, 'CjkUnifiedIdeograph50f5', [], 'Display', None],
  [ 0x50f6, 'CjkUnifiedIdeograph50f6', [], 'Display', None],
  [ 0x50f7, 'CjkUnifiedIdeograph50f7', [], 'Display', None],
  [ 0x50f8, 'CjkUnifiedIdeograph50f8', [], 'Display', None],
  [ 0x50f9, 'CjkUnifiedIdeograph50f9', [], 'Display', None],
  [ 0x50fa, 'CjkUnifiedIdeograph50fa', [], 'Display', None],
  [ 0x50fb, 'CjkUnifiedIdeograph50fb', [], 'Display', None],
  [ 0x50fc, 'CjkUnifiedIdeograph50fc', [], 'Display', None],
  [ 0x50fd, 'CjkUnifiedIdeograph50fd', [], 'Display', None],
  [ 0x50fe, 'CjkUnifiedIdeograph50fe', [], 'Display', None],
  [ 0x50ff, 'CjkUnifiedIdeograph50ff', [], 'Display', None],
  [ 0x5100, 'CjkUnifiedIdeograph5100', [], 'Display', None],
  [ 0x5101, 'CjkUnifiedIdeograph5101', [], 'Display', None],
  [ 0x5102, 'CjkUnifiedIdeograph5102', [], 'Display', None],
  [ 0x5103, 'CjkUnifiedIdeograph5103', [], 'Display', None],
  [ 0x5104, 'CjkUnifiedIdeograph5104', [], 'Display', None],
  [ 0x5105, 'CjkUnifiedIdeograph5105', [], 'Display', None],
  [ 0x5106, 'CjkUnifiedIdeograph5106', [], 'Display', None],
  [ 0x5107, 'CjkUnifiedIdeograph5107', [], 'Display', None],
  [ 0x5108, 'CjkUnifiedIdeograph5108', [], 'Display', None],
  [ 0x5109, 'CjkUnifiedIdeograph5109', [], 'Display', None],
  [ 0x510a, 'CjkUnifiedIdeograph510a', [], 'Display', None],
  [ 0x510b, 'CjkUnifiedIdeograph510b', [], 'Display', None],
  [ 0x510c, 'CjkUnifiedIdeograph510c', [], 'Display', None],
  [ 0x510d, 'CjkUnifiedIdeograph510d', [], 'Display', None],
  [ 0x510e, 'CjkUnifiedIdeograph510e', [], 'Display', None],
  [ 0x510f, 'CjkUnifiedIdeograph510f', [], 'Display', None],
  [ 0x5110, 'CjkUnifiedIdeograph5110', [], 'Display', None],
  [ 0x5111, 'CjkUnifiedIdeograph5111', [], 'Display', None],
  [ 0x5112, 'CjkUnifiedIdeograph5112', [], 'Display', None],
  [ 0x5113, 'CjkUnifiedIdeograph5113', [], 'Display', None],
  [ 0x5114, 'CjkUnifiedIdeograph5114', [], 'Display', None],
  [ 0x5115, 'CjkUnifiedIdeograph5115', [], 'Display', None],
  [ 0x5116, 'CjkUnifiedIdeograph5116', [], 'Display', None],
  [ 0x5117, 'CjkUnifiedIdeograph5117', [], 'Display', None],
  [ 0x5118, 'CjkUnifiedIdeograph5118', [], 'Display', None],
  [ 0x5119, 'CjkUnifiedIdeograph5119', [], 'Display', None],
  [ 0x511a, 'CjkUnifiedIdeograph511a', [], 'Display', None],
  [ 0x511b, 'CjkUnifiedIdeograph511b', [], 'Display', None],
  [ 0x511c, 'CjkUnifiedIdeograph511c', [], 'Display', None],
  [ 0x511d, 'CjkUnifiedIdeograph511d', [], 'Display', None],
  [ 0x511e, 'CjkUnifiedIdeograph511e', [], 'Display', None],
  [ 0x511f, 'CjkUnifiedIdeograph511f', [], 'Display', None],
  [ 0x5120, 'CjkUnifiedIdeograph5120', [], 'Display', None],
  [ 0x5121, 'CjkUnifiedIdeograph5121', [], 'Display', None],
  [ 0x5122, 'CjkUnifiedIdeograph5122', [], 'Display', None],
  [ 0x5123, 'CjkUnifiedIdeograph5123', [], 'Display', None],
  [ 0x5124, 'CjkUnifiedIdeograph5124', [], 'Display', None],
  [ 0x5125, 'CjkUnifiedIdeograph5125', [], 'Display', None],
  [ 0x5126, 'CjkUnifiedIdeograph5126', [], 'Display', None],
  [ 0x5127, 'CjkUnifiedIdeograph5127', [], 'Display', None],
  [ 0x5128, 'CjkUnifiedIdeograph5128', [], 'Display', None],
  [ 0x5129, 'CjkUnifiedIdeograph5129', [], 'Display', None],
  [ 0x512a, 'CjkUnifiedIdeograph512a', [], 'Display', None],
  [ 0x512b, 'CjkUnifiedIdeograph512b', [], 'Display', None],
  [ 0x512c, 'CjkUnifiedIdeograph512c', [], 'Display', None],
  [ 0x512d, 'CjkUnifiedIdeograph512d', [], 'Display', None],
  [ 0x512e, 'CjkUnifiedIdeograph512e', [], 'Display', None],
  [ 0x512f, 'CjkUnifiedIdeograph512f', [], 'Display', None],
  [ 0x5130, 'CjkUnifiedIdeograph5130', [], 'Display', None],
  [ 0x5131, 'CjkUnifiedIdeograph5131', [], 'Display', None],
  [ 0x5132, 'CjkUnifiedIdeograph5132', [], 'Display', None],
  [ 0x5133, 'CjkUnifiedIdeograph5133', [], 'Display', None],
  [ 0x5134, 'CjkUnifiedIdeograph5134', [], 'Display', None],
  [ 0x5135, 'CjkUnifiedIdeograph5135', [], 'Display', None],
  [ 0x5136, 'CjkUnifiedIdeograph5136', [], 'Display', None],
  [ 0x5137, 'CjkUnifiedIdeograph5137', [], 'Display', None],
  [ 0x5138, 'CjkUnifiedIdeograph5138', [], 'Display', None],
  [ 0x5139, 'CjkUnifiedIdeograph5139', [], 'Display', None],
  [ 0x513a, 'CjkUnifiedIdeograph513a', [], 'Display', None],
  [ 0x513b, 'CjkUnifiedIdeograph513b', [], 'Display', None],
  [ 0x513c, 'CjkUnifiedIdeograph513c', [], 'Display', None],
  [ 0x513d, 'CjkUnifiedIdeograph513d', [], 'Display', None],
  [ 0x513e, 'CjkUnifiedIdeograph513e', [], 'Display', None],
  [ 0x513f, 'CjkUnifiedIdeograph513f', [], 'Display', None],
  [ 0x5140, 'CjkUnifiedIdeograph5140', [], 'Display', None],
  [ 0x5141, 'CjkUnifiedIdeograph5141', [], 'Display', None],
  [ 0x5142, 'CjkUnifiedIdeograph5142', [], 'Display', None],
  [ 0x5143, 'CjkUnifiedIdeograph5143', [], 'Display', None],
  [ 0x5144, 'CjkUnifiedIdeograph5144', [], 'Display', None],
  [ 0x5145, 'CjkUnifiedIdeograph5145', [], 'Display', None],
  [ 0x5146, 'CjkUnifiedIdeograph5146', [], 'Display', None],
  [ 0x5147, 'CjkUnifiedIdeograph5147', [], 'Display', None],
  [ 0x5148, 'CjkUnifiedIdeograph5148', [], 'Display', None],
  [ 0x5149, 'CjkUnifiedIdeograph5149', [], 'Display', None],
  [ 0x514a, 'CjkUnifiedIdeograph514a', [], 'Display', None],
  [ 0x514b, 'CjkUnifiedIdeograph514b', [], 'Display', None],
  [ 0x514c, 'CjkUnifiedIdeograph514c', [], 'Display', None],
  [ 0x514d, 'CjkUnifiedIdeograph514d', [], 'Display', None],
  [ 0x514e, 'CjkUnifiedIdeograph514e', [], 'Display', None],
  [ 0x514f, 'CjkUnifiedIdeograph514f', [], 'Display', None],
  [ 0x5150, 'CjkUnifiedIdeograph5150', [], 'Display', None],
  [ 0x5151, 'CjkUnifiedIdeograph5151', [], 'Display', None],
  [ 0x5152, 'CjkUnifiedIdeograph5152', [], 'Display', None],
  [ 0x5153, 'CjkUnifiedIdeograph5153', [], 'Display', None],
  [ 0x5154, 'CjkUnifiedIdeograph5154', [], 'Display', None],
  [ 0x5155, 'CjkUnifiedIdeograph5155', [], 'Display', None],
  [ 0x5156, 'CjkUnifiedIdeograph5156', [], 'Display', None],
  [ 0x5157, 'CjkUnifiedIdeograph5157', [], 'Display', None],
  [ 0x5158, 'CjkUnifiedIdeograph5158', [], 'Display', None],
  [ 0x5159, 'CjkUnifiedIdeograph5159', [], 'Display', None],
  [ 0x515a, 'CjkUnifiedIdeograph515a', [], 'Display', None],
  [ 0x515b, 'CjkUnifiedIdeograph515b', [], 'Display', None],
  [ 0x515c, 'CjkUnifiedIdeograph515c', [], 'Display', None],
  [ 0x515d, 'CjkUnifiedIdeograph515d', [], 'Display', None],
  [ 0x515e, 'CjkUnifiedIdeograph515e', [], 'Display', None],
  [ 0x515f, 'CjkUnifiedIdeograph515f', [], 'Display', None],
  [ 0x5160, 'CjkUnifiedIdeograph5160', [], 'Display', None],
  [ 0x5161, 'CjkUnifiedIdeograph5161', [], 'Display', None],
  [ 0x5162, 'CjkUnifiedIdeograph5162', [], 'Display', None],
  [ 0x5163, 'CjkUnifiedIdeograph5163', [], 'Display', None],
  [ 0x5164, 'CjkUnifiedIdeograph5164', [], 'Display', None],
  [ 0x5165, 'CjkUnifiedIdeograph5165', [], 'Display', None],
  [ 0x5166, 'CjkUnifiedIdeograph5166', [], 'Display', None],
  [ 0x5167, 'CjkUnifiedIdeograph5167', [], 'Display', None],
  [ 0x5168, 'CjkUnifiedIdeograph5168', [], 'Display', None],
  [ 0x5169, 'CjkUnifiedIdeograph5169', [], 'Display', None],
  [ 0x516a, 'CjkUnifiedIdeograph516a', [], 'Display', None],
  [ 0x516b, 'CjkUnifiedIdeograph516b', [], 'Display', None],
  [ 0x516c, 'CjkUnifiedIdeograph516c', [], 'Display', None],
  [ 0x516d, 'CjkUnifiedIdeograph516d', [], 'Display', None],
  [ 0x516e, 'CjkUnifiedIdeograph516e', [], 'Display', None],
  [ 0x516f, 'CjkUnifiedIdeograph516f', [], 'Display', None],
  [ 0x5170, 'CjkUnifiedIdeograph5170', [], 'Display', None],
  [ 0x5171, 'CjkUnifiedIdeograph5171', [], 'Display', None],
  [ 0x5172, 'CjkUnifiedIdeograph5172', [], 'Display', None],
  [ 0x5173, 'CjkUnifiedIdeograph5173', [], 'Display', None],
  [ 0x5174, 'CjkUnifiedIdeograph5174', [], 'Display', None],
  [ 0x5175, 'CjkUnifiedIdeograph5175', [], 'Display', None],
  [ 0x5176, 'CjkUnifiedIdeograph5176', [], 'Display', None],
  [ 0x5177, 'CjkUnifiedIdeograph5177', [], 'Display', None],
  [ 0x5178, 'CjkUnifiedIdeograph5178', [], 'Display', None],
  [ 0x5179, 'CjkUnifiedIdeograph5179', [], 'Display', None],
  [ 0x517a, 'CjkUnifiedIdeograph517a', [], 'Display', None],
  [ 0x517b, 'CjkUnifiedIdeograph517b', [], 'Display', None],
  [ 0x517c, 'CjkUnifiedIdeograph517c', [], 'Display', None],
  [ 0x517d, 'CjkUnifiedIdeograph517d', [], 'Display', None],
  [ 0x517e, 'CjkUnifiedIdeograph517e', [], 'Display', None],
  [ 0x517f, 'CjkUnifiedIdeograph517f', [], 'Display', None],
  [ 0x5180, 'CjkUnifiedIdeograph5180', [], 'Display', None],
  [ 0x5181, 'CjkUnifiedIdeograph5181', [], 'Display', None],
  [ 0x5182, 'CjkUnifiedIdeograph5182', [], 'Display', None],
  [ 0x5183, 'CjkUnifiedIdeograph5183', [], 'Display', None],
  [ 0x5184, 'CjkUnifiedIdeograph5184', [], 'Display', None],
  [ 0x5185, 'CjkUnifiedIdeograph5185', [], 'Display', None],
  [ 0x5186, 'CjkUnifiedIdeograph5186', [], 'Display', None],
  [ 0x5187, 'CjkUnifiedIdeograph5187', [], 'Display', None],
  [ 0x5188, 'CjkUnifiedIdeograph5188', [], 'Display', None],
  [ 0x5189, 'CjkUnifiedIdeograph5189', [], 'Display', None],
  [ 0x518a, 'CjkUnifiedIdeograph518a', [], 'Display', None],
  [ 0x518b, 'CjkUnifiedIdeograph518b', [], 'Display', None],
  [ 0x518c, 'CjkUnifiedIdeograph518c', [], 'Display', None],
  [ 0x518d, 'CjkUnifiedIdeograph518d', [], 'Display', None],
  [ 0x518e, 'CjkUnifiedIdeograph518e', [], 'Display', None],
  [ 0x518f, 'CjkUnifiedIdeograph518f', [], 'Display', None],
  [ 0x5190, 'CjkUnifiedIdeograph5190', [], 'Display', None],
  [ 0x5191, 'CjkUnifiedIdeograph5191', [], 'Display', None],
  [ 0x5192, 'CjkUnifiedIdeograph5192', [], 'Display', None],
  [ 0x5193, 'CjkUnifiedIdeograph5193', [], 'Display', None],
  [ 0x5194, 'CjkUnifiedIdeograph5194', [], 'Display', None],
  [ 0x5195, 'CjkUnifiedIdeograph5195', [], 'Display', None],
  [ 0x5196, 'CjkUnifiedIdeograph5196', [], 'Display', None],
  [ 0x5197, 'CjkUnifiedIdeograph5197', [], 'Display', None],
  [ 0x5198, 'CjkUnifiedIdeograph5198', [], 'Display', None],
  [ 0x5199, 'CjkUnifiedIdeograph5199', [], 'Display', None],
  [ 0x519a, 'CjkUnifiedIdeograph519a', [], 'Display', None],
  [ 0x519b, 'CjkUnifiedIdeograph519b', [], 'Display', None],
  [ 0x519c, 'CjkUnifiedIdeograph519c', [], 'Display', None],
  [ 0x519d, 'CjkUnifiedIdeograph519d', [], 'Display', None],
  [ 0x519e, 'CjkUnifiedIdeograph519e', [], 'Display', None],
  [ 0x519f, 'CjkUnifiedIdeograph519f', [], 'Display', None],
  [ 0x51a0, 'CjkUnifiedIdeograph51a0', [], 'Display', None],
  [ 0x51a1, 'CjkUnifiedIdeograph51a1', [], 'Display', None],
  [ 0x51a2, 'CjkUnifiedIdeograph51a2', [], 'Display', None],
  [ 0x51a3, 'CjkUnifiedIdeograph51a3', [], 'Display', None],
  [ 0x51a4, 'CjkUnifiedIdeograph51a4', [], 'Display', None],
  [ 0x51a5, 'CjkUnifiedIdeograph51a5', [], 'Display', None],
  [ 0x51a6, 'CjkUnifiedIdeograph51a6', [], 'Display', None],
  [ 0x51a7, 'CjkUnifiedIdeograph51a7', [], 'Display', None],
  [ 0x51a8, 'CjkUnifiedIdeograph51a8', [], 'Display', None],
  [ 0x51a9, 'CjkUnifiedIdeograph51a9', [], 'Display', None],
  [ 0x51aa, 'CjkUnifiedIdeograph51aa', [], 'Display', None],
  [ 0x51ab, 'CjkUnifiedIdeograph51ab', [], 'Display', None],
  [ 0x51ac, 'CjkUnifiedIdeograph51ac', [], 'Display', None],
  [ 0x51ad, 'CjkUnifiedIdeograph51ad', [], 'Display', None],
  [ 0x51ae, 'CjkUnifiedIdeograph51ae', [], 'Display', None],
  [ 0x51af, 'CjkUnifiedIdeograph51af', [], 'Display', None],
  [ 0x51b0, 'CjkUnifiedIdeograph51b0', [], 'Display', None],
  [ 0x51b1, 'CjkUnifiedIdeograph51b1', [], 'Display', None],
  [ 0x51b2, 'CjkUnifiedIdeograph51b2', [], 'Display', None],
  [ 0x51b3, 'CjkUnifiedIdeograph51b3', [], 'Display', None],
  [ 0x51b4, 'CjkUnifiedIdeograph51b4', [], 'Display', None],
  [ 0x51b5, 'CjkUnifiedIdeograph51b5', [], 'Display', None],
  [ 0x51b6, 'CjkUnifiedIdeograph51b6', [], 'Display', None],
  [ 0x51b7, 'CjkUnifiedIdeograph51b7', [], 'Display', None],
  [ 0x51b8, 'CjkUnifiedIdeograph51b8', [], 'Display', None],
  [ 0x51b9, 'CjkUnifiedIdeograph51b9', [], 'Display', None],
  [ 0x51ba, 'CjkUnifiedIdeograph51ba', [], 'Display', None],
  [ 0x51bb, 'CjkUnifiedIdeograph51bb', [], 'Display', None],
  [ 0x51bc, 'CjkUnifiedIdeograph51bc', [], 'Display', None],
  [ 0x51bd, 'CjkUnifiedIdeograph51bd', [], 'Display', None],
  [ 0x51be, 'CjkUnifiedIdeograph51be', [], 'Display', None],
  [ 0x51bf, 'CjkUnifiedIdeograph51bf', [], 'Display', None],
  [ 0x51c0, 'CjkUnifiedIdeograph51c0', [], 'Display', None],
  [ 0x51c1, 'CjkUnifiedIdeograph51c1', [], 'Display', None],
  [ 0x51c2, 'CjkUnifiedIdeograph51c2', [], 'Display', None],
  [ 0x51c3, 'CjkUnifiedIdeograph51c3', [], 'Display', None],
  [ 0x51c4, 'CjkUnifiedIdeograph51c4', [], 'Display', None],
  [ 0x51c5, 'CjkUnifiedIdeograph51c5', [], 'Display', None],
  [ 0x51c6, 'CjkUnifiedIdeograph51c6', [], 'Display', None],
  [ 0x51c7, 'CjkUnifiedIdeograph51c7', [], 'Display', None],
  [ 0x51c8, 'CjkUnifiedIdeograph51c8', [], 'Display', None],
  [ 0x51c9, 'CjkUnifiedIdeograph51c9', [], 'Display', None],
  [ 0x51ca, 'CjkUnifiedIdeograph51ca', [], 'Display', None],
  [ 0x51cb, 'CjkUnifiedIdeograph51cb', [], 'Display', None],
  [ 0x51cc, 'CjkUnifiedIdeograph51cc', [], 'Display', None],
  [ 0x51cd, 'CjkUnifiedIdeograph51cd', [], 'Display', None],
  [ 0x51ce, 'CjkUnifiedIdeograph51ce', [], 'Display', None],
  [ 0x51cf, 'CjkUnifiedIdeograph51cf', [], 'Display', None],
  [ 0x51d0, 'CjkUnifiedIdeograph51d0', [], 'Display', None],
  [ 0x51d1, 'CjkUnifiedIdeograph51d1', [], 'Display', None],
  [ 0x51d2, 'CjkUnifiedIdeograph51d2', [], 'Display', None],
  [ 0x51d3, 'CjkUnifiedIdeograph51d3', [], 'Display', None],
  [ 0x51d4, 'CjkUnifiedIdeograph51d4', [], 'Display', None],
  [ 0x51d5, 'CjkUnifiedIdeograph51d5', [], 'Display', None],
  [ 0x51d6, 'CjkUnifiedIdeograph51d6', [], 'Display', None],
  [ 0x51d7, 'CjkUnifiedIdeograph51d7', [], 'Display', None],
  [ 0x51d8, 'CjkUnifiedIdeograph51d8', [], 'Display', None],
  [ 0x51d9, 'CjkUnifiedIdeograph51d9', [], 'Display', None],
  [ 0x51da, 'CjkUnifiedIdeograph51da', [], 'Display', None],
  [ 0x51db, 'CjkUnifiedIdeograph51db', [], 'Display', None],
  [ 0x51dc, 'CjkUnifiedIdeograph51dc', [], 'Display', None],
  [ 0x51dd, 'CjkUnifiedIdeograph51dd', [], 'Display', None],
  [ 0x51de, 'CjkUnifiedIdeograph51de', [], 'Display', None],
  [ 0x51df, 'CjkUnifiedIdeograph51df', [], 'Display', None],
  [ 0x51e0, 'CjkUnifiedIdeograph51e0', [], 'Display', None],
  [ 0x51e1, 'CjkUnifiedIdeograph51e1', [], 'Display', None],
  [ 0x51e2, 'CjkUnifiedIdeograph51e2', [], 'Display', None],
  [ 0x51e3, 'CjkUnifiedIdeograph51e3', [], 'Display', None],
  [ 0x51e4, 'CjkUnifiedIdeograph51e4', [], 'Display', None],
  [ 0x51e5, 'CjkUnifiedIdeograph51e5', [], 'Display', None],
  [ 0x51e6, 'CjkUnifiedIdeograph51e6', [], 'Display', None],
  [ 0x51e7, 'CjkUnifiedIdeograph51e7', [], 'Display', None],
  [ 0x51e8, 'CjkUnifiedIdeograph51e8', [], 'Display', None],
  [ 0x51e9, 'CjkUnifiedIdeograph51e9', [], 'Display', None],
  [ 0x51ea, 'CjkUnifiedIdeograph51ea', [], 'Display', None],
  [ 0x51eb, 'CjkUnifiedIdeograph51eb', [], 'Display', None],
  [ 0x51ec, 'CjkUnifiedIdeograph51ec', [], 'Display', None],
  [ 0x51ed, 'CjkUnifiedIdeograph51ed', [], 'Display', None],
  [ 0x51ee, 'CjkUnifiedIdeograph51ee', [], 'Display', None],
  [ 0x51ef, 'CjkUnifiedIdeograph51ef', [], 'Display', None],
  [ 0x51f0, 'CjkUnifiedIdeograph51f0', [], 'Display', None],
  [ 0x51f1, 'CjkUnifiedIdeograph51f1', [], 'Display', None],
  [ 0x51f2, 'CjkUnifiedIdeograph51f2', [], 'Display', None],
  [ 0x51f3, 'CjkUnifiedIdeograph51f3', [], 'Display', None],
  [ 0x51f4, 'CjkUnifiedIdeograph51f4', [], 'Display', None],
  [ 0x51f5, 'CjkUnifiedIdeograph51f5', [], 'Display', None],
  [ 0x51f6, 'CjkUnifiedIdeograph51f6', [], 'Display', None],
  [ 0x51f7, 'CjkUnifiedIdeograph51f7', [], 'Display', None],
  [ 0x51f8, 'CjkUnifiedIdeograph51f8', [], 'Display', None],
  [ 0x51f9, 'CjkUnifiedIdeograph51f9', [], 'Display', None],
  [ 0x51fa, 'CjkUnifiedIdeograph51fa', [], 'Display', None],
  [ 0x51fb, 'CjkUnifiedIdeograph51fb', [], 'Display', None],
  [ 0x51fc, 'CjkUnifiedIdeograph51fc', [], 'Display', None],
  [ 0x51fd, 'CjkUnifiedIdeograph51fd', [], 'Display', None],
  [ 0x51fe, 'CjkUnifiedIdeograph51fe', [], 'Display', None],
  [ 0x51ff, 'CjkUnifiedIdeograph51ff', [], 'Display', None],
  [ 0x5200, 'CjkUnifiedIdeograph5200', [], 'Display', None],
  [ 0x5201, 'CjkUnifiedIdeograph5201', [], 'Display', None],
  [ 0x5202, 'CjkUnifiedIdeograph5202', [], 'Display', None],
  [ 0x5203, 'CjkUnifiedIdeograph5203', [], 'Display', None],
  [ 0x5204, 'CjkUnifiedIdeograph5204', [], 'Display', None],
  [ 0x5205, 'CjkUnifiedIdeograph5205', [], 'Display', None],
  [ 0x5206, 'CjkUnifiedIdeograph5206', [], 'Display', None],
  [ 0x5207, 'CjkUnifiedIdeograph5207', [], 'Display', None],
  [ 0x5208, 'CjkUnifiedIdeograph5208', [], 'Display', None],
  [ 0x5209, 'CjkUnifiedIdeograph5209', [], 'Display', None],
  [ 0x520a, 'CjkUnifiedIdeograph520a', [], 'Display', None],
  [ 0x520b, 'CjkUnifiedIdeograph520b', [], 'Display', None],
  [ 0x520c, 'CjkUnifiedIdeograph520c', [], 'Display', None],
  [ 0x520d, 'CjkUnifiedIdeograph520d', [], 'Display', None],
  [ 0x520e, 'CjkUnifiedIdeograph520e', [], 'Display', None],
  [ 0x520f, 'CjkUnifiedIdeograph520f', [], 'Display', None],
  [ 0x5210, 'CjkUnifiedIdeograph5210', [], 'Display', None],
  [ 0x5211, 'CjkUnifiedIdeograph5211', [], 'Display', None],
  [ 0x5212, 'CjkUnifiedIdeograph5212', [], 'Display', None],
  [ 0x5213, 'CjkUnifiedIdeograph5213', [], 'Display', None],
  [ 0x5214, 'CjkUnifiedIdeograph5214', [], 'Display', None],
  [ 0x5215, 'CjkUnifiedIdeograph5215', [], 'Display', None],
  [ 0x5216, 'CjkUnifiedIdeograph5216', [], 'Display', None],
  [ 0x5217, 'CjkUnifiedIdeograph5217', [], 'Display', None],
  [ 0x5218, 'CjkUnifiedIdeograph5218', [], 'Display', None],
  [ 0x5219, 'CjkUnifiedIdeograph5219', [], 'Display', None],
  [ 0x521a, 'CjkUnifiedIdeograph521a', [], 'Display', None],
  [ 0x521b, 'CjkUnifiedIdeograph521b', [], 'Display', None],
  [ 0x521c, 'CjkUnifiedIdeograph521c', [], 'Display', None],
  [ 0x521d, 'CjkUnifiedIdeograph521d', [], 'Display', None],
  [ 0x521e, 'CjkUnifiedIdeograph521e', [], 'Display', None],
  [ 0x521f, 'CjkUnifiedIdeograph521f', [], 'Display', None],
  [ 0x5220, 'CjkUnifiedIdeograph5220', [], 'Display', None],
  [ 0x5221, 'CjkUnifiedIdeograph5221', [], 'Display', None],
  [ 0x5222, 'CjkUnifiedIdeograph5222', [], 'Display', None],
  [ 0x5223, 'CjkUnifiedIdeograph5223', [], 'Display', None],
  [ 0x5224, 'CjkUnifiedIdeograph5224', [], 'Display', None],
  [ 0x5225, 'CjkUnifiedIdeograph5225', [], 'Display', None],
  [ 0x5226, 'CjkUnifiedIdeograph5226', [], 'Display', None],
  [ 0x5227, 'CjkUnifiedIdeograph5227', [], 'Display', None],
  [ 0x5228, 'CjkUnifiedIdeograph5228', [], 'Display', None],
  [ 0x5229, 'CjkUnifiedIdeograph5229', [], 'Display', None],
  [ 0x522a, 'CjkUnifiedIdeograph522a', [], 'Display', None],
  [ 0x522b, 'CjkUnifiedIdeograph522b', [], 'Display', None],
  [ 0x522c, 'CjkUnifiedIdeograph522c', [], 'Display', None],
  [ 0x522d, 'CjkUnifiedIdeograph522d', [], 'Display', None],
  [ 0x522e, 'CjkUnifiedIdeograph522e', [], 'Display', None],
  [ 0x522f, 'CjkUnifiedIdeograph522f', [], 'Display', None],
  [ 0x5230, 'CjkUnifiedIdeograph5230', [], 'Display', None],
  [ 0x5231, 'CjkUnifiedIdeograph5231', [], 'Display', None],
  [ 0x5232, 'CjkUnifiedIdeograph5232', [], 'Display', None],
  [ 0x5233, 'CjkUnifiedIdeograph5233', [], 'Display', None],
  [ 0x5234, 'CjkUnifiedIdeograph5234', [], 'Display', None],
  [ 0x5235, 'CjkUnifiedIdeograph5235', [], 'Display', None],
  [ 0x5236, 'CjkUnifiedIdeograph5236', [], 'Display', None],
  [ 0x5237, 'CjkUnifiedIdeograph5237', [], 'Display', None],
  [ 0x5238, 'CjkUnifiedIdeograph5238', [], 'Display', None],
  [ 0x5239, 'CjkUnifiedIdeograph5239', [], 'Display', None],
  [ 0x523a, 'CjkUnifiedIdeograph523a', [], 'Display', None],
  [ 0x523b, 'CjkUnifiedIdeograph523b', [], 'Display', None],
  [ 0x523c, 'CjkUnifiedIdeograph523c', [], 'Display', None],
  [ 0x523d, 'CjkUnifiedIdeograph523d', [], 'Display', None],
  [ 0x523e, 'CjkUnifiedIdeograph523e', [], 'Display', None],
  [ 0x523f, 'CjkUnifiedIdeograph523f', [], 'Display', None],
  [ 0x5240, 'CjkUnifiedIdeograph5240', [], 'Display', None],
  [ 0x5241, 'CjkUnifiedIdeograph5241', [], 'Display', None],
  [ 0x5242, 'CjkUnifiedIdeograph5242', [], 'Display', None],
  [ 0x5243, 'CjkUnifiedIdeograph5243', [], 'Display', None],
  [ 0x5244, 'CjkUnifiedIdeograph5244', [], 'Display', None],
  [ 0x5245, 'CjkUnifiedIdeograph5245', [], 'Display', None],
  [ 0x5246, 'CjkUnifiedIdeograph5246', [], 'Display', None],
  [ 0x5247, 'CjkUnifiedIdeograph5247', [], 'Display', None],
  [ 0x5248, 'CjkUnifiedIdeograph5248', [], 'Display', None],
  [ 0x5249, 'CjkUnifiedIdeograph5249', [], 'Display', None],
  [ 0x524a, 'CjkUnifiedIdeograph524a', [], 'Display', None],
  [ 0x524b, 'CjkUnifiedIdeograph524b', [], 'Display', None],
  [ 0x524c, 'CjkUnifiedIdeograph524c', [], 'Display', None],
  [ 0x524d, 'CjkUnifiedIdeograph524d', [], 'Display', None],
  [ 0x524e, 'CjkUnifiedIdeograph524e', [], 'Display', None],
  [ 0x524f, 'CjkUnifiedIdeograph524f', [], 'Display', None],
  [ 0x5250, 'CjkUnifiedIdeograph5250', [], 'Display', None],
  [ 0x5251, 'CjkUnifiedIdeograph5251', [], 'Display', None],
  [ 0x5252, 'CjkUnifiedIdeograph5252', [], 'Display', None],
  [ 0x5253, 'CjkUnifiedIdeograph5253', [], 'Display', None],
  [ 0x5254, 'CjkUnifiedIdeograph5254', [], 'Display', None],
  [ 0x5255, 'CjkUnifiedIdeograph5255', [], 'Display', None],
  [ 0x5256, 'CjkUnifiedIdeograph5256', [], 'Display', None],
  [ 0x5257, 'CjkUnifiedIdeograph5257', [], 'Display', None],
  [ 0x5258, 'CjkUnifiedIdeograph5258', [], 'Display', None],
  [ 0x5259, 'CjkUnifiedIdeograph5259', [], 'Display', None],
  [ 0x525a, 'CjkUnifiedIdeograph525a', [], 'Display', None],
  [ 0x525b, 'CjkUnifiedIdeograph525b', [], 'Display', None],
  [ 0x525c, 'CjkUnifiedIdeograph525c', [], 'Display', None],
  [ 0x525d, 'CjkUnifiedIdeograph525d', [], 'Display', None],
  [ 0x525e, 'CjkUnifiedIdeograph525e', [], 'Display', None],
  [ 0x525f, 'CjkUnifiedIdeograph525f', [], 'Display', None],
  [ 0x5260, 'CjkUnifiedIdeograph5260', [], 'Display', None],
  [ 0x5261, 'CjkUnifiedIdeograph5261', [], 'Display', None],
  [ 0x5262, 'CjkUnifiedIdeograph5262', [], 'Display', None],
  [ 0x5263, 'CjkUnifiedIdeograph5263', [], 'Display', None],
  [ 0x5264, 'CjkUnifiedIdeograph5264', [], 'Display', None],
  [ 0x5265, 'CjkUnifiedIdeograph5265', [], 'Display', None],
  [ 0x5266, 'CjkUnifiedIdeograph5266', [], 'Display', None],
  [ 0x5267, 'CjkUnifiedIdeograph5267', [], 'Display', None],
  [ 0x5268, 'CjkUnifiedIdeograph5268', [], 'Display', None],
  [ 0x5269, 'CjkUnifiedIdeograph5269', [], 'Display', None],
  [ 0x526a, 'CjkUnifiedIdeograph526a', [], 'Display', None],
  [ 0x526b, 'CjkUnifiedIdeograph526b', [], 'Display', None],
  [ 0x526c, 'CjkUnifiedIdeograph526c', [], 'Display', None],
  [ 0x526d, 'CjkUnifiedIdeograph526d', [], 'Display', None],
  [ 0x526e, 'CjkUnifiedIdeograph526e', [], 'Display', None],
  [ 0x526f, 'CjkUnifiedIdeograph526f', [], 'Display', None],
  [ 0x5270, 'CjkUnifiedIdeograph5270', [], 'Display', None],
  [ 0x5271, 'CjkUnifiedIdeograph5271', [], 'Display', None],
  [ 0x5272, 'CjkUnifiedIdeograph5272', [], 'Display', None],
  [ 0x5273, 'CjkUnifiedIdeograph5273', [], 'Display', None],
  [ 0x5274, 'CjkUnifiedIdeograph5274', [], 'Display', None],
  [ 0x5275, 'CjkUnifiedIdeograph5275', [], 'Display', None],
  [ 0x5276, 'CjkUnifiedIdeograph5276', [], 'Display', None],
  [ 0x5277, 'CjkUnifiedIdeograph5277', [], 'Display', None],
  [ 0x5278, 'CjkUnifiedIdeograph5278', [], 'Display', None],
  [ 0x5279, 'CjkUnifiedIdeograph5279', [], 'Display', None],
  [ 0x527a, 'CjkUnifiedIdeograph527a', [], 'Display', None],
  [ 0x527b, 'CjkUnifiedIdeograph527b', [], 'Display', None],
  [ 0x527c, 'CjkUnifiedIdeograph527c', [], 'Display', None],
  [ 0x527d, 'CjkUnifiedIdeograph527d', [], 'Display', None],
  [ 0x527e, 'CjkUnifiedIdeograph527e', [], 'Display', None],
  [ 0x527f, 'CjkUnifiedIdeograph527f', [], 'Display', None],
  [ 0x5280, 'CjkUnifiedIdeograph5280', [], 'Display', None],
  [ 0x5281, 'CjkUnifiedIdeograph5281', [], 'Display', None],
  [ 0x5282, 'CjkUnifiedIdeograph5282', [], 'Display', None],
  [ 0x5283, 'CjkUnifiedIdeograph5283', [], 'Display', None],
  [ 0x5284, 'CjkUnifiedIdeograph5284', [], 'Display', None],
  [ 0x5285, 'CjkUnifiedIdeograph5285', [], 'Display', None],
  [ 0x5286, 'CjkUnifiedIdeograph5286', [], 'Display', None],
  [ 0x5287, 'CjkUnifiedIdeograph5287', [], 'Display', None],
  [ 0x5288, 'CjkUnifiedIdeograph5288', [], 'Display', None],
  [ 0x5289, 'CjkUnifiedIdeograph5289', [], 'Display', None],
  [ 0x528a, 'CjkUnifiedIdeograph528a', [], 'Display', None],
  [ 0x528b, 'CjkUnifiedIdeograph528b', [], 'Display', None],
  [ 0x528c, 'CjkUnifiedIdeograph528c', [], 'Display', None],
  [ 0x528d, 'CjkUnifiedIdeograph528d', [], 'Display', None],
  [ 0x528e, 'CjkUnifiedIdeograph528e', [], 'Display', None],
  [ 0x528f, 'CjkUnifiedIdeograph528f', [], 'Display', None],
  [ 0x5290, 'CjkUnifiedIdeograph5290', [], 'Display', None],
  [ 0x5291, 'CjkUnifiedIdeograph5291', [], 'Display', None],
  [ 0x5292, 'CjkUnifiedIdeograph5292', [], 'Display', None],
  [ 0x5293, 'CjkUnifiedIdeograph5293', [], 'Display', None],
  [ 0x5294, 'CjkUnifiedIdeograph5294', [], 'Display', None],
  [ 0x5295, 'CjkUnifiedIdeograph5295', [], 'Display', None],
  [ 0x5296, 'CjkUnifiedIdeograph5296', [], 'Display', None],
  [ 0x5297, 'CjkUnifiedIdeograph5297', [], 'Display', None],
  [ 0x5298, 'CjkUnifiedIdeograph5298', [], 'Display', None],
  [ 0x5299, 'CjkUnifiedIdeograph5299', [], 'Display', None],
  [ 0x529a, 'CjkUnifiedIdeograph529a', [], 'Display', None],
  [ 0x529b, 'CjkUnifiedIdeograph529b', [], 'Display', None],
  [ 0x529c, 'CjkUnifiedIdeograph529c', [], 'Display', None],
  [ 0x529d, 'CjkUnifiedIdeograph529d', [], 'Display', None],
  [ 0x529e, 'CjkUnifiedIdeograph529e', [], 'Display', None],
  [ 0x529f, 'CjkUnifiedIdeograph529f', [], 'Display', None],
  [ 0x52a0, 'CjkUnifiedIdeograph52a0', [], 'Display', None],
  [ 0x52a1, 'CjkUnifiedIdeograph52a1', [], 'Display', None],
  [ 0x52a2, 'CjkUnifiedIdeograph52a2', [], 'Display', None],
  [ 0x52a3, 'CjkUnifiedIdeograph52a3', [], 'Display', None],
  [ 0x52a4, 'CjkUnifiedIdeograph52a4', [], 'Display', None],
  [ 0x52a5, 'CjkUnifiedIdeograph52a5', [], 'Display', None],
  [ 0x52a6, 'CjkUnifiedIdeograph52a6', [], 'Display', None],
  [ 0x52a7, 'CjkUnifiedIdeograph52a7', [], 'Display', None],
  [ 0x52a8, 'CjkUnifiedIdeograph52a8', [], 'Display', None],
  [ 0x52a9, 'CjkUnifiedIdeograph52a9', [], 'Display', None],
  [ 0x52aa, 'CjkUnifiedIdeograph52aa', [], 'Display', None],
  [ 0x52ab, 'CjkUnifiedIdeograph52ab', [], 'Display', None],
  [ 0x52ac, 'CjkUnifiedIdeograph52ac', [], 'Display', None],
  [ 0x52ad, 'CjkUnifiedIdeograph52ad', [], 'Display', None],
  [ 0x52ae, 'CjkUnifiedIdeograph52ae', [], 'Display', None],
  [ 0x52af, 'CjkUnifiedIdeograph52af', [], 'Display', None],
  [ 0x52b0, 'CjkUnifiedIdeograph52b0', [], 'Display', None],
  [ 0x52b1, 'CjkUnifiedIdeograph52b1', [], 'Display', None],
  [ 0x52b2, 'CjkUnifiedIdeograph52b2', [], 'Display', None],
  [ 0x52b3, 'CjkUnifiedIdeograph52b3', [], 'Display', None],
  [ 0x52b4, 'CjkUnifiedIdeograph52b4', [], 'Display', None],
  [ 0x52b5, 'CjkUnifiedIdeograph52b5', [], 'Display', None],
  [ 0x52b6, 'CjkUnifiedIdeograph52b6', [], 'Display', None],
  [ 0x52b7, 'CjkUnifiedIdeograph52b7', [], 'Display', None],
  [ 0x52b8, 'CjkUnifiedIdeograph52b8', [], 'Display', None],
  [ 0x52b9, 'CjkUnifiedIdeograph52b9', [], 'Display', None],
  [ 0x52ba, 'CjkUnifiedIdeograph52ba', [], 'Display', None],
  [ 0x52bb, 'CjkUnifiedIdeograph52bb', [], 'Display', None],
  [ 0x52bc, 'CjkUnifiedIdeograph52bc', [], 'Display', None],
  [ 0x52bd, 'CjkUnifiedIdeograph52bd', [], 'Display', None],
  [ 0x52be, 'CjkUnifiedIdeograph52be', [], 'Display', None],
  [ 0x52bf, 'CjkUnifiedIdeograph52bf', [], 'Display', None],
  [ 0x52c0, 'CjkUnifiedIdeograph52c0', [], 'Display', None],
  [ 0x52c1, 'CjkUnifiedIdeograph52c1', [], 'Display', None],
  [ 0x52c2, 'CjkUnifiedIdeograph52c2', [], 'Display', None],
  [ 0x52c3, 'CjkUnifiedIdeograph52c3', [], 'Display', None],
  [ 0x52c4, 'CjkUnifiedIdeograph52c4', [], 'Display', None],
  [ 0x52c5, 'CjkUnifiedIdeograph52c5', [], 'Display', None],
  [ 0x52c6, 'CjkUnifiedIdeograph52c6', [], 'Display', None],
  [ 0x52c7, 'CjkUnifiedIdeograph52c7', [], 'Display', None],
  [ 0x52c8, 'CjkUnifiedIdeograph52c8', [], 'Display', None],
  [ 0x52c9, 'CjkUnifiedIdeograph52c9', [], 'Display', None],
  [ 0x52ca, 'CjkUnifiedIdeograph52ca', [], 'Display', None],
  [ 0x52cb, 'CjkUnifiedIdeograph52cb', [], 'Display', None],
  [ 0x52cc, 'CjkUnifiedIdeograph52cc', [], 'Display', None],
  [ 0x52cd, 'CjkUnifiedIdeograph52cd', [], 'Display', None],
  [ 0x52ce, 'CjkUnifiedIdeograph52ce', [], 'Display', None],
  [ 0x52cf, 'CjkUnifiedIdeograph52cf', [], 'Display', None],
  [ 0x52d0, 'CjkUnifiedIdeograph52d0', [], 'Display', None],
  [ 0x52d1, 'CjkUnifiedIdeograph52d1', [], 'Display', None],
  [ 0x52d2, 'CjkUnifiedIdeograph52d2', [], 'Display', None],
  [ 0x52d3, 'CjkUnifiedIdeograph52d3', [], 'Display', None],
  [ 0x52d4, 'CjkUnifiedIdeograph52d4', [], 'Display', None],
  [ 0x52d5, 'CjkUnifiedIdeograph52d5', [], 'Display', None],
  [ 0x52d6, 'CjkUnifiedIdeograph52d6', [], 'Display', None],
  [ 0x52d7, 'CjkUnifiedIdeograph52d7', [], 'Display', None],
  [ 0x52d8, 'CjkUnifiedIdeograph52d8', [], 'Display', None],
  [ 0x52d9, 'CjkUnifiedIdeograph52d9', [], 'Display', None],
  [ 0x52da, 'CjkUnifiedIdeograph52da', [], 'Display', None],
  [ 0x52db, 'CjkUnifiedIdeograph52db', [], 'Display', None],
  [ 0x52dc, 'CjkUnifiedIdeograph52dc', [], 'Display', None],
  [ 0x52dd, 'CjkUnifiedIdeograph52dd', [], 'Display', None],
  [ 0x52de, 'CjkUnifiedIdeograph52de', [], 'Display', None],
  [ 0x52df, 'CjkUnifiedIdeograph52df', [], 'Display', None],
  [ 0x52e0, 'CjkUnifiedIdeograph52e0', [], 'Display', None],
  [ 0x52e1, 'CjkUnifiedIdeograph52e1', [], 'Display', None],
  [ 0x52e2, 'CjkUnifiedIdeograph52e2', [], 'Display', None],
  [ 0x52e3, 'CjkUnifiedIdeograph52e3', [], 'Display', None],
  [ 0x52e4, 'CjkUnifiedIdeograph52e4', [], 'Display', None],
  [ 0x52e5, 'CjkUnifiedIdeograph52e5', [], 'Display', None],
  [ 0x52e6, 'CjkUnifiedIdeograph52e6', [], 'Display', None],
  [ 0x52e7, 'CjkUnifiedIdeograph52e7', [], 'Display', None],
  [ 0x52e8, 'CjkUnifiedIdeograph52e8', [], 'Display', None],
  [ 0x52e9, 'CjkUnifiedIdeograph52e9', [], 'Display', None],
  [ 0x52ea, 'CjkUnifiedIdeograph52ea', [], 'Display', None],
  [ 0x52eb, 'CjkUnifiedIdeograph52eb', [], 'Display', None],
  [ 0x52ec, 'CjkUnifiedIdeograph52ec', [], 'Display', None],
  [ 0x52ed, 'CjkUnifiedIdeograph52ed', [], 'Display', None],
  [ 0x52ee, 'CjkUnifiedIdeograph52ee', [], 'Display', None],
  [ 0x52ef, 'CjkUnifiedIdeograph52ef', [], 'Display', None],
  [ 0x52f0, 'CjkUnifiedIdeograph52f0', [], 'Display', None],
  [ 0x52f1, 'CjkUnifiedIdeograph52f1', [], 'Display', None],
  [ 0x52f2, 'CjkUnifiedIdeograph52f2', [], 'Display', None],
  [ 0x52f3, 'CjkUnifiedIdeograph52f3', [], 'Display', None],
  [ 0x52f4, 'CjkUnifiedIdeograph52f4', [], 'Display', None],
  [ 0x52f5, 'CjkUnifiedIdeograph52f5', [], 'Display', None],
  [ 0x52f6, 'CjkUnifiedIdeograph52f6', [], 'Display', None],
  [ 0x52f7, 'CjkUnifiedIdeograph52f7', [], 'Display', None],
  [ 0x52f8, 'CjkUnifiedIdeograph52f8', [], 'Display', None],
  [ 0x52f9, 'CjkUnifiedIdeograph52f9', [], 'Display', None],
  [ 0x52fa, 'CjkUnifiedIdeograph52fa', [], 'Display', None],
  [ 0x52fb, 'CjkUnifiedIdeograph52fb', [], 'Display', None],
  [ 0x52fc, 'CjkUnifiedIdeograph52fc', [], 'Display', None],
  [ 0x52fd, 'CjkUnifiedIdeograph52fd', [], 'Display', None],
  [ 0x52fe, 'CjkUnifiedIdeograph52fe', [], 'Display', None],
  [ 0x52ff, 'CjkUnifiedIdeograph52ff', [], 'Display', None],
  [ 0x5300, 'CjkUnifiedIdeograph5300', [], 'Display', None],
  [ 0x5301, 'CjkUnifiedIdeograph5301', [], 'Display', None],
  [ 0x5302, 'CjkUnifiedIdeograph5302', [], 'Display', None],
  [ 0x5303, 'CjkUnifiedIdeograph5303', [], 'Display', None],
  [ 0x5304, 'CjkUnifiedIdeograph5304', [], 'Display', None],
  [ 0x5305, 'CjkUnifiedIdeograph5305', [], 'Display', None],
  [ 0x5306, 'CjkUnifiedIdeograph5306', [], 'Display', None],
  [ 0x5307, 'CjkUnifiedIdeograph5307', [], 'Display', None],
  [ 0x5308, 'CjkUnifiedIdeograph5308', [], 'Display', None],
  [ 0x5309, 'CjkUnifiedIdeograph5309', [], 'Display', None],
  [ 0x530a, 'CjkUnifiedIdeograph530a', [], 'Display', None],
  [ 0x530b, 'CjkUnifiedIdeograph530b', [], 'Display', None],
  [ 0x530c, 'CjkUnifiedIdeograph530c', [], 'Display', None],
  [ 0x530d, 'CjkUnifiedIdeograph530d', [], 'Display', None],
  [ 0x530e, 'CjkUnifiedIdeograph530e', [], 'Display', None],
  [ 0x530f, 'CjkUnifiedIdeograph530f', [], 'Display', None],
  [ 0x5310, 'CjkUnifiedIdeograph5310', [], 'Display', None],
  [ 0x5311, 'CjkUnifiedIdeograph5311', [], 'Display', None],
  [ 0x5312, 'CjkUnifiedIdeograph5312', [], 'Display', None],
  [ 0x5313, 'CjkUnifiedIdeograph5313', [], 'Display', None],
  [ 0x5314, 'CjkUnifiedIdeograph5314', [], 'Display', None],
  [ 0x5315, 'CjkUnifiedIdeograph5315', [], 'Display', None],
  [ 0x5316, 'CjkUnifiedIdeograph5316', [], 'Display', None],
  [ 0x5317, 'CjkUnifiedIdeograph5317', [], 'Display', None],
  [ 0x5318, 'CjkUnifiedIdeograph5318', [], 'Display', None],
  [ 0x5319, 'CjkUnifiedIdeograph5319', [], 'Display', None],
  [ 0x531a, 'CjkUnifiedIdeograph531a', [], 'Display', None],
  [ 0x531b, 'CjkUnifiedIdeograph531b', [], 'Display', None],
  [ 0x531c, 'CjkUnifiedIdeograph531c', [], 'Display', None],
  [ 0x531d, 'CjkUnifiedIdeograph531d', [], 'Display', None],
  [ 0x531e, 'CjkUnifiedIdeograph531e', [], 'Display', None],
  [ 0x531f, 'CjkUnifiedIdeograph531f', [], 'Display', None],
  [ 0x5320, 'CjkUnifiedIdeograph5320', [], 'Display', None],
  [ 0x5321, 'CjkUnifiedIdeograph5321', [], 'Display', None],
  [ 0x5322, 'CjkUnifiedIdeograph5322', [], 'Display', None],
  [ 0x5323, 'CjkUnifiedIdeograph5323', [], 'Display', None],
  [ 0x5324, 'CjkUnifiedIdeograph5324', [], 'Display', None],
  [ 0x5325, 'CjkUnifiedIdeograph5325', [], 'Display', None],
  [ 0x5326, 'CjkUnifiedIdeograph5326', [], 'Display', None],
  [ 0x5327, 'CjkUnifiedIdeograph5327', [], 'Display', None],
  [ 0x5328, 'CjkUnifiedIdeograph5328', [], 'Display', None],
  [ 0x5329, 'CjkUnifiedIdeograph5329', [], 'Display', None],
  [ 0x532a, 'CjkUnifiedIdeograph532a', [], 'Display', None],
  [ 0x532b, 'CjkUnifiedIdeograph532b', [], 'Display', None],
  [ 0x532c, 'CjkUnifiedIdeograph532c', [], 'Display', None],
  [ 0x532d, 'CjkUnifiedIdeograph532d', [], 'Display', None],
  [ 0x532e, 'CjkUnifiedIdeograph532e', [], 'Display', None],
  [ 0x532f, 'CjkUnifiedIdeograph532f', [], 'Display', None],
  [ 0x5330, 'CjkUnifiedIdeograph5330', [], 'Display', None],
  [ 0x5331, 'CjkUnifiedIdeograph5331', [], 'Display', None],
  [ 0x5332, 'CjkUnifiedIdeograph5332', [], 'Display', None],
  [ 0x5333, 'CjkUnifiedIdeograph5333', [], 'Display', None],
  [ 0x5334, 'CjkUnifiedIdeograph5334', [], 'Display', None],
  [ 0x5335, 'CjkUnifiedIdeograph5335', [], 'Display', None],
  [ 0x5336, 'CjkUnifiedIdeograph5336', [], 'Display', None],
  [ 0x5337, 'CjkUnifiedIdeograph5337', [], 'Display', None],
  [ 0x5338, 'CjkUnifiedIdeograph5338', [], 'Display', None],
  [ 0x5339, 'CjkUnifiedIdeograph5339', [], 'Display', None],
  [ 0x533a, 'CjkUnifiedIdeograph533a', [], 'Display', None],
  [ 0x533b, 'CjkUnifiedIdeograph533b', [], 'Display', None],
  [ 0x533c, 'CjkUnifiedIdeograph533c', [], 'Display', None],
  [ 0x533d, 'CjkUnifiedIdeograph533d', [], 'Display', None],
  [ 0x533e, 'CjkUnifiedIdeograph533e', [], 'Display', None],
  [ 0x533f, 'CjkUnifiedIdeograph533f', [], 'Display', None],
  [ 0x5340, 'CjkUnifiedIdeograph5340', [], 'Display', None],
  [ 0x5341, 'CjkUnifiedIdeograph5341', [], 'Display', None],
  [ 0x5342, 'CjkUnifiedIdeograph5342', [], 'Display', None],
  [ 0x5343, 'CjkUnifiedIdeograph5343', [], 'Display', None],
  [ 0x5344, 'CjkUnifiedIdeograph5344', [], 'Display', None],
  [ 0x5345, 'CjkUnifiedIdeograph5345', [], 'Display', None],
  [ 0x5346, 'CjkUnifiedIdeograph5346', [], 'Display', None],
  [ 0x5347, 'CjkUnifiedIdeograph5347', [], 'Display', None],
  [ 0x5348, 'CjkUnifiedIdeograph5348', [], 'Display', None],
  [ 0x5349, 'CjkUnifiedIdeograph5349', [], 'Display', None],
  [ 0x534a, 'CjkUnifiedIdeograph534a', [], 'Display', None],
  [ 0x534b, 'CjkUnifiedIdeograph534b', [], 'Display', None],
  [ 0x534c, 'CjkUnifiedIdeograph534c', [], 'Display', None],
  [ 0x534d, 'CjkUnifiedIdeograph534d', [], 'Display', None],
  [ 0x534e, 'CjkUnifiedIdeograph534e', [], 'Display', None],
  [ 0x534f, 'CjkUnifiedIdeograph534f', [], 'Display', None],
  [ 0x5350, 'CjkUnifiedIdeograph5350', [], 'Display', None],
  [ 0x5351, 'CjkUnifiedIdeograph5351', [], 'Display', None],
  [ 0x5352, 'CjkUnifiedIdeograph5352', [], 'Display', None],
  [ 0x5353, 'CjkUnifiedIdeograph5353', [], 'Display', None],
  [ 0x5354, 'CjkUnifiedIdeograph5354', [], 'Display', None],
  [ 0x5355, 'CjkUnifiedIdeograph5355', [], 'Display', None],
  [ 0x5356, 'CjkUnifiedIdeograph5356', [], 'Display', None],
  [ 0x5357, 'CjkUnifiedIdeograph5357', [], 'Display', None],
  [ 0x5358, 'CjkUnifiedIdeograph5358', [], 'Display', None],
  [ 0x5359, 'CjkUnifiedIdeograph5359', [], 'Display', None],
  [ 0x535a, 'CjkUnifiedIdeograph535a', [], 'Display', None],
  [ 0x535b, 'CjkUnifiedIdeograph535b', [], 'Display', None],
  [ 0x535c, 'CjkUnifiedIdeograph535c', [], 'Display', None],
  [ 0x535d, 'CjkUnifiedIdeograph535d', [], 'Display', None],
  [ 0x535e, 'CjkUnifiedIdeograph535e', [], 'Display', None],
  [ 0x535f, 'CjkUnifiedIdeograph535f', [], 'Display', None],
  [ 0x5360, 'CjkUnifiedIdeograph5360', [], 'Display', None],
  [ 0x5361, 'CjkUnifiedIdeograph5361', [], 'Display', None],
  [ 0x5362, 'CjkUnifiedIdeograph5362', [], 'Display', None],
  [ 0x5363, 'CjkUnifiedIdeograph5363', [], 'Display', None],
  [ 0x5364, 'CjkUnifiedIdeograph5364', [], 'Display', None],
  [ 0x5365, 'CjkUnifiedIdeograph5365', [], 'Display', None],
  [ 0x5366, 'CjkUnifiedIdeograph5366', [], 'Display', None],
  [ 0x5367, 'CjkUnifiedIdeograph5367', [], 'Display', None],
  [ 0x5368, 'CjkUnifiedIdeograph5368', [], 'Display', None],
  [ 0x5369, 'CjkUnifiedIdeograph5369', [], 'Display', None],
  [ 0x536a, 'CjkUnifiedIdeograph536a', [], 'Display', None],
  [ 0x536b, 'CjkUnifiedIdeograph536b', [], 'Display', None],
  [ 0x536c, 'CjkUnifiedIdeograph536c', [], 'Display', None],
  [ 0x536d, 'CjkUnifiedIdeograph536d', [], 'Display', None],
  [ 0x536e, 'CjkUnifiedIdeograph536e', [], 'Display', None],
  [ 0x536f, 'CjkUnifiedIdeograph536f', [], 'Display', None],
  [ 0x5370, 'CjkUnifiedIdeograph5370', [], 'Display', None],
  [ 0x5371, 'CjkUnifiedIdeograph5371', [], 'Display', None],
  [ 0x5372, 'CjkUnifiedIdeograph5372', [], 'Display', None],
  [ 0x5373, 'CjkUnifiedIdeograph5373', [], 'Display', None],
  [ 0x5374, 'CjkUnifiedIdeograph5374', [], 'Display', None],
  [ 0x5375, 'CjkUnifiedIdeograph5375', [], 'Display', None],
  [ 0x5376, 'CjkUnifiedIdeograph5376', [], 'Display', None],
  [ 0x5377, 'CjkUnifiedIdeograph5377', [], 'Display', None],
  [ 0x5378, 'CjkUnifiedIdeograph5378', [], 'Display', None],
  [ 0x5379, 'CjkUnifiedIdeograph5379', [], 'Display', None],
  [ 0x537a, 'CjkUnifiedIdeograph537a', [], 'Display', None],
  [ 0x537b, 'CjkUnifiedIdeograph537b', [], 'Display', None],
  [ 0x537c, 'CjkUnifiedIdeograph537c', [], 'Display', None],
  [ 0x537d, 'CjkUnifiedIdeograph537d', [], 'Display', None],
  [ 0x537e, 'CjkUnifiedIdeograph537e', [], 'Display', None],
  [ 0x537f, 'CjkUnifiedIdeograph537f', [], 'Display', None],
  [ 0x5380, 'CjkUnifiedIdeograph5380', [], 'Display', None],
  [ 0x5381, 'CjkUnifiedIdeograph5381', [], 'Display', None],
  [ 0x5382, 'CjkUnifiedIdeograph5382', [], 'Display', None],
  [ 0x5383, 'CjkUnifiedIdeograph5383', [], 'Display', None],
  [ 0x5384, 'CjkUnifiedIdeograph5384', [], 'Display', None],
  [ 0x5385, 'CjkUnifiedIdeograph5385', [], 'Display', None],
  [ 0x5386, 'CjkUnifiedIdeograph5386', [], 'Display', None],
  [ 0x5387, 'CjkUnifiedIdeograph5387', [], 'Display', None],
  [ 0x5388, 'CjkUnifiedIdeograph5388', [], 'Display', None],
  [ 0x5389, 'CjkUnifiedIdeograph5389', [], 'Display', None],
  [ 0x538a, 'CjkUnifiedIdeograph538a', [], 'Display', None],
  [ 0x538b, 'CjkUnifiedIdeograph538b', [], 'Display', None],
  [ 0x538c, 'CjkUnifiedIdeograph538c', [], 'Display', None],
  [ 0x538d, 'CjkUnifiedIdeograph538d', [], 'Display', None],
  [ 0x538e, 'CjkUnifiedIdeograph538e', [], 'Display', None],
  [ 0x538f, 'CjkUnifiedIdeograph538f', [], 'Display', None],
  [ 0x5390, 'CjkUnifiedIdeograph5390', [], 'Display', None],
  [ 0x5391, 'CjkUnifiedIdeograph5391', [], 'Display', None],
  [ 0x5392, 'CjkUnifiedIdeograph5392', [], 'Display', None],
  [ 0x5393, 'CjkUnifiedIdeograph5393', [], 'Display', None],
  [ 0x5394, 'CjkUnifiedIdeograph5394', [], 'Display', None],
  [ 0x5395, 'CjkUnifiedIdeograph5395', [], 'Display', None],
  [ 0x5396, 'CjkUnifiedIdeograph5396', [], 'Display', None],
  [ 0x5397, 'CjkUnifiedIdeograph5397', [], 'Display', None],
  [ 0x5398, 'CjkUnifiedIdeograph5398', [], 'Display', None],
  [ 0x5399, 'CjkUnifiedIdeograph5399', [], 'Display', None],
  [ 0x539a, 'CjkUnifiedIdeograph539a', [], 'Display', None],
  [ 0x539b, 'CjkUnifiedIdeograph539b', [], 'Display', None],
  [ 0x539c, 'CjkUnifiedIdeograph539c', [], 'Display', None],
  [ 0x539d, 'CjkUnifiedIdeograph539d', [], 'Display', None],
  [ 0x539e, 'CjkUnifiedIdeograph539e', [], 'Display', None],
  [ 0x539f, 'CjkUnifiedIdeograph539f', [], 'Display', None],
  [ 0x53a0, 'CjkUnifiedIdeograph53a0', [], 'Display', None],
  [ 0x53a1, 'CjkUnifiedIdeograph53a1', [], 'Display', None],
  [ 0x53a2, 'CjkUnifiedIdeograph53a2', [], 'Display', None],
  [ 0x53a3, 'CjkUnifiedIdeograph53a3', [], 'Display', None],
  [ 0x53a4, 'CjkUnifiedIdeograph53a4', [], 'Display', None],
  [ 0x53a5, 'CjkUnifiedIdeograph53a5', [], 'Display', None],
  [ 0x53a6, 'CjkUnifiedIdeograph53a6', [], 'Display', None],
  [ 0x53a7, 'CjkUnifiedIdeograph53a7', [], 'Display', None],
  [ 0x53a8, 'CjkUnifiedIdeograph53a8', [], 'Display', None],
  [ 0x53a9, 'CjkUnifiedIdeograph53a9', [], 'Display', None],
  [ 0x53aa, 'CjkUnifiedIdeograph53aa', [], 'Display', None],
  [ 0x53ab, 'CjkUnifiedIdeograph53ab', [], 'Display', None],
  [ 0x53ac, 'CjkUnifiedIdeograph53ac', [], 'Display', None],
  [ 0x53ad, 'CjkUnifiedIdeograph53ad', [], 'Display', None],
  [ 0x53ae, 'CjkUnifiedIdeograph53ae', [], 'Display', None],
  [ 0x53af, 'CjkUnifiedIdeograph53af', [], 'Display', None],
  [ 0x53b0, 'CjkUnifiedIdeograph53b0', [], 'Display', None],
  [ 0x53b1, 'CjkUnifiedIdeograph53b1', [], 'Display', None],
  [ 0x53b2, 'CjkUnifiedIdeograph53b2', [], 'Display', None],
  [ 0x53b3, 'CjkUnifiedIdeograph53b3', [], 'Display', None],
  [ 0x53b4, 'CjkUnifiedIdeograph53b4', [], 'Display', None],
  [ 0x53b5, 'CjkUnifiedIdeograph53b5', [], 'Display', None],
  [ 0x53b6, 'CjkUnifiedIdeograph53b6', [], 'Display', None],
  [ 0x53b7, 'CjkUnifiedIdeograph53b7', [], 'Display', None],
  [ 0x53b8, 'CjkUnifiedIdeograph53b8', [], 'Display', None],
  [ 0x53b9, 'CjkUnifiedIdeograph53b9', [], 'Display', None],
  [ 0x53ba, 'CjkUnifiedIdeograph53ba', [], 'Display', None],
  [ 0x53bb, 'CjkUnifiedIdeograph53bb', [], 'Display', None],
  [ 0x53bc, 'CjkUnifiedIdeograph53bc', [], 'Display', None],
  [ 0x53bd, 'CjkUnifiedIdeograph53bd', [], 'Display', None],
  [ 0x53be, 'CjkUnifiedIdeograph53be', [], 'Display', None],
  [ 0x53bf, 'CjkUnifiedIdeograph53bf', [], 'Display', None],
  [ 0x53c0, 'CjkUnifiedIdeograph53c0', [], 'Display', None],
  [ 0x53c1, 'CjkUnifiedIdeograph53c1', [], 'Display', None],
  [ 0x53c2, 'CjkUnifiedIdeograph53c2', [], 'Display', None],
  [ 0x53c3, 'CjkUnifiedIdeograph53c3', [], 'Display', None],
  [ 0x53c4, 'CjkUnifiedIdeograph53c4', [], 'Display', None],
  [ 0x53c5, 'CjkUnifiedIdeograph53c5', [], 'Display', None],
  [ 0x53c6, 'CjkUnifiedIdeograph53c6', [], 'Display', None],
  [ 0x53c7, 'CjkUnifiedIdeograph53c7', [], 'Display', None],
  [ 0x53c8, 'CjkUnifiedIdeograph53c8', [], 'Display', None],
  [ 0x53c9, 'CjkUnifiedIdeograph53c9', [], 'Display', None],
  [ 0x53ca, 'CjkUnifiedIdeograph53ca', [], 'Display', None],
  [ 0x53cb, 'CjkUnifiedIdeograph53cb', [], 'Display', None],
  [ 0x53cc, 'CjkUnifiedIdeograph53cc', [], 'Display', None],
  [ 0x53cd, 'CjkUnifiedIdeograph53cd', [], 'Display', None],
  [ 0x53ce, 'CjkUnifiedIdeograph53ce', [], 'Display', None],
  [ 0x53cf, 'CjkUnifiedIdeograph53cf', [], 'Display', None],
  [ 0x53d0, 'CjkUnifiedIdeograph53d0', [], 'Display', None],
  [ 0x53d1, 'CjkUnifiedIdeograph53d1', [], 'Display', None],
  [ 0x53d2, 'CjkUnifiedIdeograph53d2', [], 'Display', None],
  [ 0x53d3, 'CjkUnifiedIdeograph53d3', [], 'Display', None],
  [ 0x53d4, 'CjkUnifiedIdeograph53d4', [], 'Display', None],
  [ 0x53d5, 'CjkUnifiedIdeograph53d5', [], 'Display', None],
  [ 0x53d6, 'CjkUnifiedIdeograph53d6', [], 'Display', None],
  [ 0x53d7, 'CjkUnifiedIdeograph53d7', [], 'Display', None],
  [ 0x53d8, 'CjkUnifiedIdeograph53d8', [], 'Display', None],
  [ 0x53d9, 'CjkUnifiedIdeograph53d9', [], 'Display', None],
  [ 0x53da, 'CjkUnifiedIdeograph53da', [], 'Display', None],
  [ 0x53db, 'CjkUnifiedIdeograph53db', [], 'Display', None],
  [ 0x53dc, 'CjkUnifiedIdeograph53dc', [], 'Display', None],
  [ 0x53dd, 'CjkUnifiedIdeograph53dd', [], 'Display', None],
  [ 0x53de, 'CjkUnifiedIdeograph53de', [], 'Display', None],
  [ 0x53df, 'CjkUnifiedIdeograph53df', [], 'Display', None],
  [ 0x53e0, 'CjkUnifiedIdeograph53e0', [], 'Display', None],
  [ 0x53e1, 'CjkUnifiedIdeograph53e1', [], 'Display', None],
  [ 0x53e2, 'CjkUnifiedIdeograph53e2', [], 'Display', None],
  [ 0x53e3, 'CjkUnifiedIdeograph53e3', [], 'Display', None],
  [ 0x53e4, 'CjkUnifiedIdeograph53e4', [], 'Display', None],
  [ 0x53e5, 'CjkUnifiedIdeograph53e5', [], 'Display', None],
  [ 0x53e6, 'CjkUnifiedIdeograph53e6', [], 'Display', None],
  [ 0x53e7, 'CjkUnifiedIdeograph53e7', [], 'Display', None],
  [ 0x53e8, 'CjkUnifiedIdeograph53e8', [], 'Display', None],
  [ 0x53e9, 'CjkUnifiedIdeograph53e9', [], 'Display', None],
  [ 0x53ea, 'CjkUnifiedIdeograph53ea', [], 'Display', None],
  [ 0x53eb, 'CjkUnifiedIdeograph53eb', [], 'Display', None],
  [ 0x53ec, 'CjkUnifiedIdeograph53ec', [], 'Display', None],
  [ 0x53ed, 'CjkUnifiedIdeograph53ed', [], 'Display', None],
  [ 0x53ee, 'CjkUnifiedIdeograph53ee', [], 'Display', None],
  [ 0x53ef, 'CjkUnifiedIdeograph53ef', [], 'Display', None],
  [ 0x53f0, 'CjkUnifiedIdeograph53f0', [], 'Display', None],
  [ 0x53f1, 'CjkUnifiedIdeograph53f1', [], 'Display', None],
  [ 0x53f2, 'CjkUnifiedIdeograph53f2', [], 'Display', None],
  [ 0x53f3, 'CjkUnifiedIdeograph53f3', [], 'Display', None],
  [ 0x53f4, 'CjkUnifiedIdeograph53f4', [], 'Display', None],
  [ 0x53f5, 'CjkUnifiedIdeograph53f5', [], 'Display', None],
  [ 0x53f6, 'CjkUnifiedIdeograph53f6', [], 'Display', None],
  [ 0x53f7, 'CjkUnifiedIdeograph53f7', [], 'Display', None],
  [ 0x53f8, 'CjkUnifiedIdeograph53f8', [], 'Display', None],
  [ 0x53f9, 'CjkUnifiedIdeograph53f9', [], 'Display', None],
  [ 0x53fa, 'CjkUnifiedIdeograph53fa', [], 'Display', None],
  [ 0x53fb, 'CjkUnifiedIdeograph53fb', [], 'Display', None],
  [ 0x53fc, 'CjkUnifiedIdeograph53fc', [], 'Display', None],
  [ 0x53fd, 'CjkUnifiedIdeograph53fd', [], 'Display', None],
  [ 0x53fe, 'CjkUnifiedIdeograph53fe', [], 'Display', None],
  [ 0x53ff, 'CjkUnifiedIdeograph53ff', [], 'Display', None],
  [ 0x5400, 'CjkUnifiedIdeograph5400', [], 'Display', None],
  [ 0x5401, 'CjkUnifiedIdeograph5401', [], 'Display', None],
  [ 0x5402, 'CjkUnifiedIdeograph5402', [], 'Display', None],
  [ 0x5403, 'CjkUnifiedIdeograph5403', [], 'Display', None],
  [ 0x5404, 'CjkUnifiedIdeograph5404', [], 'Display', None],
  [ 0x5405, 'CjkUnifiedIdeograph5405', [], 'Display', None],
  [ 0x5406, 'CjkUnifiedIdeograph5406', [], 'Display', None],
  [ 0x5407, 'CjkUnifiedIdeograph5407', [], 'Display', None],
  [ 0x5408, 'CjkUnifiedIdeograph5408', [], 'Display', None],
  [ 0x5409, 'CjkUnifiedIdeograph5409', [], 'Display', None],
  [ 0x540a, 'CjkUnifiedIdeograph540a', [], 'Display', None],
  [ 0x540b, 'CjkUnifiedIdeograph540b', [], 'Display', None],
  [ 0x540c, 'CjkUnifiedIdeograph540c', [], 'Display', None],
  [ 0x540d, 'CjkUnifiedIdeograph540d', [], 'Display', None],
  [ 0x540e, 'CjkUnifiedIdeograph540e', [], 'Display', None],
  [ 0x540f, 'CjkUnifiedIdeograph540f', [], 'Display', None],
  [ 0x5410, 'CjkUnifiedIdeograph5410', [], 'Display', None],
  [ 0x5411, 'CjkUnifiedIdeograph5411', [], 'Display', None],
  [ 0x5412, 'CjkUnifiedIdeograph5412', [], 'Display', None],
  [ 0x5413, 'CjkUnifiedIdeograph5413', [], 'Display', None],
  [ 0x5414, 'CjkUnifiedIdeograph5414', [], 'Display', None],
  [ 0x5415, 'CjkUnifiedIdeograph5415', [], 'Display', None],
  [ 0x5416, 'CjkUnifiedIdeograph5416', [], 'Display', None],
  [ 0x5417, 'CjkUnifiedIdeograph5417', [], 'Display', None],
  [ 0x5418, 'CjkUnifiedIdeograph5418', [], 'Display', None],
  [ 0x5419, 'CjkUnifiedIdeograph5419', [], 'Display', None],
  [ 0x541a, 'CjkUnifiedIdeograph541a', [], 'Display', None],
  [ 0x541b, 'CjkUnifiedIdeograph541b', [], 'Display', None],
  [ 0x541c, 'CjkUnifiedIdeograph541c', [], 'Display', None],
  [ 0x541d, 'CjkUnifiedIdeograph541d', [], 'Display', None],
  [ 0x541e, 'CjkUnifiedIdeograph541e', [], 'Display', None],
  [ 0x541f, 'CjkUnifiedIdeograph541f', [], 'Display', None],
  [ 0x5420, 'CjkUnifiedIdeograph5420', [], 'Display', None],
  [ 0x5421, 'CjkUnifiedIdeograph5421', [], 'Display', None],
  [ 0x5422, 'CjkUnifiedIdeograph5422', [], 'Display', None],
  [ 0x5423, 'CjkUnifiedIdeograph5423', [], 'Display', None],
  [ 0x5424, 'CjkUnifiedIdeograph5424', [], 'Display', None],
  [ 0x5425, 'CjkUnifiedIdeograph5425', [], 'Display', None],
  [ 0x5426, 'CjkUnifiedIdeograph5426', [], 'Display', None],
  [ 0x5427, 'CjkUnifiedIdeograph5427', [], 'Display', None],
  [ 0x5428, 'CjkUnifiedIdeograph5428', [], 'Display', None],
  [ 0x5429, 'CjkUnifiedIdeograph5429', [], 'Display', None],
  [ 0x542a, 'CjkUnifiedIdeograph542a', [], 'Display', None],
  [ 0x542b, 'CjkUnifiedIdeograph542b', [], 'Display', None],
  [ 0x542c, 'CjkUnifiedIdeograph542c', [], 'Display', None],
  [ 0x542d, 'CjkUnifiedIdeograph542d', [], 'Display', None],
  [ 0x542e, 'CjkUnifiedIdeograph542e', [], 'Display', None],
  [ 0x542f, 'CjkUnifiedIdeograph542f', [], 'Display', None],
  [ 0x5430, 'CjkUnifiedIdeograph5430', [], 'Display', None],
  [ 0x5431, 'CjkUnifiedIdeograph5431', [], 'Display', None],
  [ 0x5432, 'CjkUnifiedIdeograph5432', [], 'Display', None],
  [ 0x5433, 'CjkUnifiedIdeograph5433', [], 'Display', None],
  [ 0x5434, 'CjkUnifiedIdeograph5434', [], 'Display', None],
  [ 0x5435, 'CjkUnifiedIdeograph5435', [], 'Display', None],
  [ 0x5436, 'CjkUnifiedIdeograph5436', [], 'Display', None],
  [ 0x5437, 'CjkUnifiedIdeograph5437', [], 'Display', None],
  [ 0x5438, 'CjkUnifiedIdeograph5438', [], 'Display', None],
  [ 0x5439, 'CjkUnifiedIdeograph5439', [], 'Display', None],
  [ 0x543a, 'CjkUnifiedIdeograph543a', [], 'Display', None],
  [ 0x543b, 'CjkUnifiedIdeograph543b', [], 'Display', None],
  [ 0x543c, 'CjkUnifiedIdeograph543c', [], 'Display', None],
  [ 0x543d, 'CjkUnifiedIdeograph543d', [], 'Display', None],
  [ 0x543e, 'CjkUnifiedIdeograph543e', [], 'Display', None],
  [ 0x543f, 'CjkUnifiedIdeograph543f', [], 'Display', None],
  [ 0x5440, 'CjkUnifiedIdeograph5440', [], 'Display', None],
  [ 0x5441, 'CjkUnifiedIdeograph5441', [], 'Display', None],
  [ 0x5442, 'CjkUnifiedIdeograph5442', [], 'Display', None],
  [ 0x5443, 'CjkUnifiedIdeograph5443', [], 'Display', None],
  [ 0x5444, 'CjkUnifiedIdeograph5444', [], 'Display', None],
  [ 0x5445, 'CjkUnifiedIdeograph5445', [], 'Display', None],
  [ 0x5446, 'CjkUnifiedIdeograph5446', [], 'Display', None],
  [ 0x5447, 'CjkUnifiedIdeograph5447', [], 'Display', None],
  [ 0x5448, 'CjkUnifiedIdeograph5448', [], 'Display', None],
  [ 0x5449, 'CjkUnifiedIdeograph5449', [], 'Display', None],
  [ 0x544a, 'CjkUnifiedIdeograph544a', [], 'Display', None],
  [ 0x544b, 'CjkUnifiedIdeograph544b', [], 'Display', None],
  [ 0x544c, 'CjkUnifiedIdeograph544c', [], 'Display', None],
  [ 0x544d, 'CjkUnifiedIdeograph544d', [], 'Display', None],
  [ 0x544e, 'CjkUnifiedIdeograph544e', [], 'Display', None],
  [ 0x544f, 'CjkUnifiedIdeograph544f', [], 'Display', None],
  [ 0x5450, 'CjkUnifiedIdeograph5450', [], 'Display', None],
  [ 0x5451, 'CjkUnifiedIdeograph5451', [], 'Display', None],
  [ 0x5452, 'CjkUnifiedIdeograph5452', [], 'Display', None],
  [ 0x5453, 'CjkUnifiedIdeograph5453', [], 'Display', None],
  [ 0x5454, 'CjkUnifiedIdeograph5454', [], 'Display', None],
  [ 0x5455, 'CjkUnifiedIdeograph5455', [], 'Display', None],
  [ 0x5456, 'CjkUnifiedIdeograph5456', [], 'Display', None],
  [ 0x5457, 'CjkUnifiedIdeograph5457', [], 'Display', None],
  [ 0x5458, 'CjkUnifiedIdeograph5458', [], 'Display', None],
  [ 0x5459, 'CjkUnifiedIdeograph5459', [], 'Display', None],
  [ 0x545a, 'CjkUnifiedIdeograph545a', [], 'Display', None],
  [ 0x545b, 'CjkUnifiedIdeograph545b', [], 'Display', None],
  [ 0x545c, 'CjkUnifiedIdeograph545c', [], 'Display', None],
  [ 0x545d, 'CjkUnifiedIdeograph545d', [], 'Display', None],
  [ 0x545e, 'CjkUnifiedIdeograph545e', [], 'Display', None],
  [ 0x545f, 'CjkUnifiedIdeograph545f', [], 'Display', None],
  [ 0x5460, 'CjkUnifiedIdeograph5460', [], 'Display', None],
  [ 0x5461, 'CjkUnifiedIdeograph5461', [], 'Display', None],
  [ 0x5462, 'CjkUnifiedIdeograph5462', [], 'Display', None],
  [ 0x5463, 'CjkUnifiedIdeograph5463', [], 'Display', None],
  [ 0x5464, 'CjkUnifiedIdeograph5464', [], 'Display', None],
  [ 0x5465, 'CjkUnifiedIdeograph5465', [], 'Display', None],
  [ 0x5466, 'CjkUnifiedIdeograph5466', [], 'Display', None],
  [ 0x5467, 'CjkUnifiedIdeograph5467', [], 'Display', None],
  [ 0x5468, 'CjkUnifiedIdeograph5468', [], 'Display', None],
  [ 0x5469, 'CjkUnifiedIdeograph5469', [], 'Display', None],
  [ 0x546a, 'CjkUnifiedIdeograph546a', [], 'Display', None],
  [ 0x546b, 'CjkUnifiedIdeograph546b', [], 'Display', None],
  [ 0x546c, 'CjkUnifiedIdeograph546c', [], 'Display', None],
  [ 0x546d, 'CjkUnifiedIdeograph546d', [], 'Display', None],
  [ 0x546e, 'CjkUnifiedIdeograph546e', [], 'Display', None],
  [ 0x546f, 'CjkUnifiedIdeograph546f', [], 'Display', None],
  [ 0x5470, 'CjkUnifiedIdeograph5470', [], 'Display', None],
  [ 0x5471, 'CjkUnifiedIdeograph5471', [], 'Display', None],
  [ 0x5472, 'CjkUnifiedIdeograph5472', [], 'Display', None],
  [ 0x5473, 'CjkUnifiedIdeograph5473', [], 'Display', None],
  [ 0x5474, 'CjkUnifiedIdeograph5474', [], 'Display', None],
  [ 0x5475, 'CjkUnifiedIdeograph5475', [], 'Display', None],
  [ 0x5476, 'CjkUnifiedIdeograph5476', [], 'Display', None],
  [ 0x5477, 'CjkUnifiedIdeograph5477', [], 'Display', None],
  [ 0x5478, 'CjkUnifiedIdeograph5478', [], 'Display', None],
  [ 0x5479, 'CjkUnifiedIdeograph5479', [], 'Display', None],
  [ 0x547a, 'CjkUnifiedIdeograph547a', [], 'Display', None],
  [ 0x547b, 'CjkUnifiedIdeograph547b', [], 'Display', None],
  [ 0x547c, 'CjkUnifiedIdeograph547c', [], 'Display', None],
  [ 0x547d, 'CjkUnifiedIdeograph547d', [], 'Display', None],
  [ 0x547e, 'CjkUnifiedIdeograph547e', [], 'Display', None],
  [ 0x547f, 'CjkUnifiedIdeograph547f', [], 'Display', None],
  [ 0x5480, 'CjkUnifiedIdeograph5480', [], 'Display', None],
  [ 0x5481, 'CjkUnifiedIdeograph5481', [], 'Display', None],
  [ 0x5482, 'CjkUnifiedIdeograph5482', [], 'Display', None],
  [ 0x5483, 'CjkUnifiedIdeograph5483', [], 'Display', None],
  [ 0x5484, 'CjkUnifiedIdeograph5484', [], 'Display', None],
  [ 0x5485, 'CjkUnifiedIdeograph5485', [], 'Display', None],
  [ 0x5486, 'CjkUnifiedIdeograph5486', [], 'Display', None],
  [ 0x5487, 'CjkUnifiedIdeograph5487', [], 'Display', None],
  [ 0x5488, 'CjkUnifiedIdeograph5488', [], 'Display', None],
  [ 0x5489, 'CjkUnifiedIdeograph5489', [], 'Display', None],
  [ 0x548a, 'CjkUnifiedIdeograph548a', [], 'Display', None],
  [ 0x548b, 'CjkUnifiedIdeograph548b', [], 'Display', None],
  [ 0x548c, 'CjkUnifiedIdeograph548c', [], 'Display', None],
  [ 0x548d, 'CjkUnifiedIdeograph548d', [], 'Display', None],
  [ 0x548e, 'CjkUnifiedIdeograph548e', [], 'Display', None],
  [ 0x548f, 'CjkUnifiedIdeograph548f', [], 'Display', None],
  [ 0x5490, 'CjkUnifiedIdeograph5490', [], 'Display', None],
  [ 0x5491, 'CjkUnifiedIdeograph5491', [], 'Display', None],
  [ 0x5492, 'CjkUnifiedIdeograph5492', [], 'Display', None],
  [ 0x5493, 'CjkUnifiedIdeograph5493', [], 'Display', None],
  [ 0x5494, 'CjkUnifiedIdeograph5494', [], 'Display', None],
  [ 0x5495, 'CjkUnifiedIdeograph5495', [], 'Display', None],
  [ 0x5496, 'CjkUnifiedIdeograph5496', [], 'Display', None],
  [ 0x5497, 'CjkUnifiedIdeograph5497', [], 'Display', None],
  [ 0x5498, 'CjkUnifiedIdeograph5498', [], 'Display', None],
  [ 0x5499, 'CjkUnifiedIdeograph5499', [], 'Display', None],
  [ 0x549a, 'CjkUnifiedIdeograph549a', [], 'Display', None],
  [ 0x549b, 'CjkUnifiedIdeograph549b', [], 'Display', None],
  [ 0x549c, 'CjkUnifiedIdeograph549c', [], 'Display', None],
  [ 0x549d, 'CjkUnifiedIdeograph549d', [], 'Display', None],
  [ 0x549e, 'CjkUnifiedIdeograph549e', [], 'Display', None],
  [ 0x549f, 'CjkUnifiedIdeograph549f', [], 'Display', None],
  [ 0x54a0, 'CjkUnifiedIdeograph54a0', [], 'Display', None],
  [ 0x54a1, 'CjkUnifiedIdeograph54a1', [], 'Display', None],
  [ 0x54a2, 'CjkUnifiedIdeograph54a2', [], 'Display', None],
  [ 0x54a3, 'CjkUnifiedIdeograph54a3', [], 'Display', None],
  [ 0x54a4, 'CjkUnifiedIdeograph54a4', [], 'Display', None],
  [ 0x54a5, 'CjkUnifiedIdeograph54a5', [], 'Display', None],
  [ 0x54a6, 'CjkUnifiedIdeograph54a6', [], 'Display', None],
  [ 0x54a7, 'CjkUnifiedIdeograph54a7', [], 'Display', None],
  [ 0x54a8, 'CjkUnifiedIdeograph54a8', [], 'Display', None],
  [ 0x54a9, 'CjkUnifiedIdeograph54a9', [], 'Display', None],
  [ 0x54aa, 'CjkUnifiedIdeograph54aa', [], 'Display', None],
  [ 0x54ab, 'CjkUnifiedIdeograph54ab', [], 'Display', None],
  [ 0x54ac, 'CjkUnifiedIdeograph54ac', [], 'Display', None],
  [ 0x54ad, 'CjkUnifiedIdeograph54ad', [], 'Display', None],
  [ 0x54ae, 'CjkUnifiedIdeograph54ae', [], 'Display', None],
  [ 0x54af, 'CjkUnifiedIdeograph54af', [], 'Display', None],
  [ 0x54b0, 'CjkUnifiedIdeograph54b0', [], 'Display', None],
  [ 0x54b1, 'CjkUnifiedIdeograph54b1', [], 'Display', None],
  [ 0x54b2, 'CjkUnifiedIdeograph54b2', [], 'Display', None],
  [ 0x54b3, 'CjkUnifiedIdeograph54b3', [], 'Display', None],
  [ 0x54b4, 'CjkUnifiedIdeograph54b4', [], 'Display', None],
  [ 0x54b5, 'CjkUnifiedIdeograph54b5', [], 'Display', None],
  [ 0x54b6, 'CjkUnifiedIdeograph54b6', [], 'Display', None],
  [ 0x54b7, 'CjkUnifiedIdeograph54b7', [], 'Display', None],
  [ 0x54b8, 'CjkUnifiedIdeograph54b8', [], 'Display', None],
  [ 0x54b9, 'CjkUnifiedIdeograph54b9', [], 'Display', None],
  [ 0x54ba, 'CjkUnifiedIdeograph54ba', [], 'Display', None],
  [ 0x54bb, 'CjkUnifiedIdeograph54bb', [], 'Display', None],
  [ 0x54bc, 'CjkUnifiedIdeograph54bc', [], 'Display', None],
  [ 0x54bd, 'CjkUnifiedIdeograph54bd', [], 'Display', None],
  [ 0x54be, 'CjkUnifiedIdeograph54be', [], 'Display', None],
  [ 0x54bf, 'CjkUnifiedIdeograph54bf', [], 'Display', None],
  [ 0x54c0, 'CjkUnifiedIdeograph54c0', [], 'Display', None],
  [ 0x54c1, 'CjkUnifiedIdeograph54c1', [], 'Display', None],
  [ 0x54c2, 'CjkUnifiedIdeograph54c2', [], 'Display', None],
  [ 0x54c3, 'CjkUnifiedIdeograph54c3', [], 'Display', None],
  [ 0x54c4, 'CjkUnifiedIdeograph54c4', [], 'Display', None],
  [ 0x54c5, 'CjkUnifiedIdeograph54c5', [], 'Display', None],
  [ 0x54c6, 'CjkUnifiedIdeograph54c6', [], 'Display', None],
  [ 0x54c7, 'CjkUnifiedIdeograph54c7', [], 'Display', None],
  [ 0x54c8, 'CjkUnifiedIdeograph54c8', [], 'Display', None],
  [ 0x54c9, 'CjkUnifiedIdeograph54c9', [], 'Display', None],
  [ 0x54ca, 'CjkUnifiedIdeograph54ca', [], 'Display', None],
  [ 0x54cb, 'CjkUnifiedIdeograph54cb', [], 'Display', None],
  [ 0x54cc, 'CjkUnifiedIdeograph54cc', [], 'Display', None],
  [ 0x54cd, 'CjkUnifiedIdeograph54cd', [], 'Display', None],
  [ 0x54ce, 'CjkUnifiedIdeograph54ce', [], 'Display', None],
  [ 0x54cf, 'CjkUnifiedIdeograph54cf', [], 'Display', None],
  [ 0x54d0, 'CjkUnifiedIdeograph54d0', [], 'Display', None],
  [ 0x54d1, 'CjkUnifiedIdeograph54d1', [], 'Display', None],
  [ 0x54d2, 'CjkUnifiedIdeograph54d2', [], 'Display', None],
  [ 0x54d3, 'CjkUnifiedIdeograph54d3', [], 'Display', None],
  [ 0x54d4, 'CjkUnifiedIdeograph54d4', [], 'Display', None],
  [ 0x54d5, 'CjkUnifiedIdeograph54d5', [], 'Display', None],
  [ 0x54d6, 'CjkUnifiedIdeograph54d6', [], 'Display', None],
  [ 0x54d7, 'CjkUnifiedIdeograph54d7', [], 'Display', None],
  [ 0x54d8, 'CjkUnifiedIdeograph54d8', [], 'Display', None],
  [ 0x54d9, 'CjkUnifiedIdeograph54d9', [], 'Display', None],
  [ 0x54da, 'CjkUnifiedIdeograph54da', [], 'Display', None],
  [ 0x54db, 'CjkUnifiedIdeograph54db', [], 'Display', None],
  [ 0x54dc, 'CjkUnifiedIdeograph54dc', [], 'Display', None],
  [ 0x54dd, 'CjkUnifiedIdeograph54dd', [], 'Display', None],
  [ 0x54de, 'CjkUnifiedIdeograph54de', [], 'Display', None],
  [ 0x54df, 'CjkUnifiedIdeograph54df', [], 'Display', None],
  [ 0x54e0, 'CjkUnifiedIdeograph54e0', [], 'Display', None],
  [ 0x54e1, 'CjkUnifiedIdeograph54e1', [], 'Display', None],
  [ 0x54e2, 'CjkUnifiedIdeograph54e2', [], 'Display', None],
  [ 0x54e3, 'CjkUnifiedIdeograph54e3', [], 'Display', None],
  [ 0x54e4, 'CjkUnifiedIdeograph54e4', [], 'Display', None],
  [ 0x54e5, 'CjkUnifiedIdeograph54e5', [], 'Display', None],
  [ 0x54e6, 'CjkUnifiedIdeograph54e6', [], 'Display', None],
  [ 0x54e7, 'CjkUnifiedIdeograph54e7', [], 'Display', None],
  [ 0x54e8, 'CjkUnifiedIdeograph54e8', [], 'Display', None],
  [ 0x54e9, 'CjkUnifiedIdeograph54e9', [], 'Display', None],
  [ 0x54ea, 'CjkUnifiedIdeograph54ea', [], 'Display', None],
  [ 0x54eb, 'CjkUnifiedIdeograph54eb', [], 'Display', None],
  [ 0x54ec, 'CjkUnifiedIdeograph54ec', [], 'Display', None],
  [ 0x54ed, 'CjkUnifiedIdeograph54ed', [], 'Display', None],
  [ 0x54ee, 'CjkUnifiedIdeograph54ee', [], 'Display', None],
  [ 0x54ef, 'CjkUnifiedIdeograph54ef', [], 'Display', None],
  [ 0x54f0, 'CjkUnifiedIdeograph54f0', [], 'Display', None],
  [ 0x54f1, 'CjkUnifiedIdeograph54f1', [], 'Display', None],
  [ 0x54f2, 'CjkUnifiedIdeograph54f2', [], 'Display', None],
  [ 0x54f3, 'CjkUnifiedIdeograph54f3', [], 'Display', None],
  [ 0x54f4, 'CjkUnifiedIdeograph54f4', [], 'Display', None],
  [ 0x54f5, 'CjkUnifiedIdeograph54f5', [], 'Display', None],
  [ 0x54f6, 'CjkUnifiedIdeograph54f6', [], 'Display', None],
  [ 0x54f7, 'CjkUnifiedIdeograph54f7', [], 'Display', None],
  [ 0x54f8, 'CjkUnifiedIdeograph54f8', [], 'Display', None],
  [ 0x54f9, 'CjkUnifiedIdeograph54f9', [], 'Display', None],
  [ 0x54fa, 'CjkUnifiedIdeograph54fa', [], 'Display', None],
  [ 0x54fb, 'CjkUnifiedIdeograph54fb', [], 'Display', None],
  [ 0x54fc, 'CjkUnifiedIdeograph54fc', [], 'Display', None],
  [ 0x54fd, 'CjkUnifiedIdeograph54fd', [], 'Display', None],
  [ 0x54fe, 'CjkUnifiedIdeograph54fe', [], 'Display', None],
  [ 0x54ff, 'CjkUnifiedIdeograph54ff', [], 'Display', None],
  [ 0x5500, 'CjkUnifiedIdeograph5500', [], 'Display', None],
  [ 0x5501, 'CjkUnifiedIdeograph5501', [], 'Display', None],
  [ 0x5502, 'CjkUnifiedIdeograph5502', [], 'Display', None],
  [ 0x5503, 'CjkUnifiedIdeograph5503', [], 'Display', None],
  [ 0x5504, 'CjkUnifiedIdeograph5504', [], 'Display', None],
  [ 0x5505, 'CjkUnifiedIdeograph5505', [], 'Display', None],
  [ 0x5506, 'CjkUnifiedIdeograph5506', [], 'Display', None],
  [ 0x5507, 'CjkUnifiedIdeograph5507', [], 'Display', None],
  [ 0x5508, 'CjkUnifiedIdeograph5508', [], 'Display', None],
  [ 0x5509, 'CjkUnifiedIdeograph5509', [], 'Display', None],
  [ 0x550a, 'CjkUnifiedIdeograph550a', [], 'Display', None],
  [ 0x550b, 'CjkUnifiedIdeograph550b', [], 'Display', None],
  [ 0x550c, 'CjkUnifiedIdeograph550c', [], 'Display', None],
  [ 0x550d, 'CjkUnifiedIdeograph550d', [], 'Display', None],
  [ 0x550e, 'CjkUnifiedIdeograph550e', [], 'Display', None],
  [ 0x550f, 'CjkUnifiedIdeograph550f', [], 'Display', None],
  [ 0x5510, 'CjkUnifiedIdeograph5510', [], 'Display', None],
  [ 0x5511, 'CjkUnifiedIdeograph5511', [], 'Display', None],
  [ 0x5512, 'CjkUnifiedIdeograph5512', [], 'Display', None],
  [ 0x5513, 'CjkUnifiedIdeograph5513', [], 'Display', None],
  [ 0x5514, 'CjkUnifiedIdeograph5514', [], 'Display', None],
  [ 0x5515, 'CjkUnifiedIdeograph5515', [], 'Display', None],
  [ 0x5516, 'CjkUnifiedIdeograph5516', [], 'Display', None],
  [ 0x5517, 'CjkUnifiedIdeograph5517', [], 'Display', None],
  [ 0x5518, 'CjkUnifiedIdeograph5518', [], 'Display', None],
  [ 0x5519, 'CjkUnifiedIdeograph5519', [], 'Display', None],
  [ 0x551a, 'CjkUnifiedIdeograph551a', [], 'Display', None],
  [ 0x551b, 'CjkUnifiedIdeograph551b', [], 'Display', None],
  [ 0x551c, 'CjkUnifiedIdeograph551c', [], 'Display', None],
  [ 0x551d, 'CjkUnifiedIdeograph551d', [], 'Display', None],
  [ 0x551e, 'CjkUnifiedIdeograph551e', [], 'Display', None],
  [ 0x551f, 'CjkUnifiedIdeograph551f', [], 'Display', None],
  [ 0x5520, 'CjkUnifiedIdeograph5520', [], 'Display', None],
  [ 0x5521, 'CjkUnifiedIdeograph5521', [], 'Display', None],
  [ 0x5522, 'CjkUnifiedIdeograph5522', [], 'Display', None],
  [ 0x5523, 'CjkUnifiedIdeograph5523', [], 'Display', None],
  [ 0x5524, 'CjkUnifiedIdeograph5524', [], 'Display', None],
  [ 0x5525, 'CjkUnifiedIdeograph5525', [], 'Display', None],
  [ 0x5526, 'CjkUnifiedIdeograph5526', [], 'Display', None],
  [ 0x5527, 'CjkUnifiedIdeograph5527', [], 'Display', None],
  [ 0x5528, 'CjkUnifiedIdeograph5528', [], 'Display', None],
  [ 0x5529, 'CjkUnifiedIdeograph5529', [], 'Display', None],
  [ 0x552a, 'CjkUnifiedIdeograph552a', [], 'Display', None],
  [ 0x552b, 'CjkUnifiedIdeograph552b', [], 'Display', None],
  [ 0x552c, 'CjkUnifiedIdeograph552c', [], 'Display', None],
  [ 0x552d, 'CjkUnifiedIdeograph552d', [], 'Display', None],
  [ 0x552e, 'CjkUnifiedIdeograph552e', [], 'Display', None],
  [ 0x552f, 'CjkUnifiedIdeograph552f', [], 'Display', None],
  [ 0x5530, 'CjkUnifiedIdeograph5530', [], 'Display', None],
  [ 0x5531, 'CjkUnifiedIdeograph5531', [], 'Display', None],
  [ 0x5532, 'CjkUnifiedIdeograph5532', [], 'Display', None],
  [ 0x5533, 'CjkUnifiedIdeograph5533', [], 'Display', None],
  [ 0x5534, 'CjkUnifiedIdeograph5534', [], 'Display', None],
  [ 0x5535, 'CjkUnifiedIdeograph5535', [], 'Display', None],
  [ 0x5536, 'CjkUnifiedIdeograph5536', [], 'Display', None],
  [ 0x5537, 'CjkUnifiedIdeograph5537', [], 'Display', None],
  [ 0x5538, 'CjkUnifiedIdeograph5538', [], 'Display', None],
  [ 0x5539, 'CjkUnifiedIdeograph5539', [], 'Display', None],
  [ 0x553a, 'CjkUnifiedIdeograph553a', [], 'Display', None],
  [ 0x553b, 'CjkUnifiedIdeograph553b', [], 'Display', None],
  [ 0x553c, 'CjkUnifiedIdeograph553c', [], 'Display', None],
  [ 0x553d, 'CjkUnifiedIdeograph553d', [], 'Display', None],
  [ 0x553e, 'CjkUnifiedIdeograph553e', [], 'Display', None],
  [ 0x553f, 'CjkUnifiedIdeograph553f', [], 'Display', None],
  [ 0x5540, 'CjkUnifiedIdeograph5540', [], 'Display', None],
  [ 0x5541, 'CjkUnifiedIdeograph5541', [], 'Display', None],
  [ 0x5542, 'CjkUnifiedIdeograph5542', [], 'Display', None],
  [ 0x5543, 'CjkUnifiedIdeograph5543', [], 'Display', None],
  [ 0x5544, 'CjkUnifiedIdeograph5544', [], 'Display', None],
  [ 0x5545, 'CjkUnifiedIdeograph5545', [], 'Display', None],
  [ 0x5546, 'CjkUnifiedIdeograph5546', [], 'Display', None],
  [ 0x5547, 'CjkUnifiedIdeograph5547', [], 'Display', None],
  [ 0x5548, 'CjkUnifiedIdeograph5548', [], 'Display', None],
  [ 0x5549, 'CjkUnifiedIdeograph5549', [], 'Display', None],
  [ 0x554a, 'CjkUnifiedIdeograph554a', [], 'Display', None],
  [ 0x554b, 'CjkUnifiedIdeograph554b', [], 'Display', None],
  [ 0x554c, 'CjkUnifiedIdeograph554c', [], 'Display', None],
  [ 0x554d, 'CjkUnifiedIdeograph554d', [], 'Display', None],
  [ 0x554e, 'CjkUnifiedIdeograph554e', [], 'Display', None],
  [ 0x554f, 'CjkUnifiedIdeograph554f', [], 'Display', None],
  [ 0x5550, 'CjkUnifiedIdeograph5550', [], 'Display', None],
  [ 0x5551, 'CjkUnifiedIdeograph5551', [], 'Display', None],
  [ 0x5552, 'CjkUnifiedIdeograph5552', [], 'Display', None],
  [ 0x5553, 'CjkUnifiedIdeograph5553', [], 'Display', None],
  [ 0x5554, 'CjkUnifiedIdeograph5554', [], 'Display', None],
  [ 0x5555, 'CjkUnifiedIdeograph5555', [], 'Display', None],
  [ 0x5556, 'CjkUnifiedIdeograph5556', [], 'Display', None],
  [ 0x5557, 'CjkUnifiedIdeograph5557', [], 'Display', None],
  [ 0x5558, 'CjkUnifiedIdeograph5558', [], 'Display', None],
  [ 0x5559, 'CjkUnifiedIdeograph5559', [], 'Display', None],
  [ 0x555a, 'CjkUnifiedIdeograph555a', [], 'Display', None],
  [ 0x555b, 'CjkUnifiedIdeograph555b', [], 'Display', None],
  [ 0x555c, 'CjkUnifiedIdeograph555c', [], 'Display', None],
  [ 0x555d, 'CjkUnifiedIdeograph555d', [], 'Display', None],
  [ 0x555e, 'CjkUnifiedIdeograph555e', [], 'Display', None],
  [ 0x555f, 'CjkUnifiedIdeograph555f', [], 'Display', None],
  [ 0x5560, 'CjkUnifiedIdeograph5560', [], 'Display', None],
  [ 0x5561, 'CjkUnifiedIdeograph5561', [], 'Display', None],
  [ 0x5562, 'CjkUnifiedIdeograph5562', [], 'Display', None],
  [ 0x5563, 'CjkUnifiedIdeograph5563', [], 'Display', None],
  [ 0x5564, 'CjkUnifiedIdeograph5564', [], 'Display', None],
  [ 0x5565, 'CjkUnifiedIdeograph5565', [], 'Display', None],
  [ 0x5566, 'CjkUnifiedIdeograph5566', [], 'Display', None],
  [ 0x5567, 'CjkUnifiedIdeograph5567', [], 'Display', None],
  [ 0x5568, 'CjkUnifiedIdeograph5568', [], 'Display', None],
  [ 0x5569, 'CjkUnifiedIdeograph5569', [], 'Display', None],
  [ 0x556a, 'CjkUnifiedIdeograph556a', [], 'Display', None],
  [ 0x556b, 'CjkUnifiedIdeograph556b', [], 'Display', None],
  [ 0x556c, 'CjkUnifiedIdeograph556c', [], 'Display', None],
  [ 0x556d, 'CjkUnifiedIdeograph556d', [], 'Display', None],
  [ 0x556e, 'CjkUnifiedIdeograph556e', [], 'Display', None],
  [ 0x556f, 'CjkUnifiedIdeograph556f', [], 'Display', None],
  [ 0x5570, 'CjkUnifiedIdeograph5570', [], 'Display', None],
  [ 0x5571, 'CjkUnifiedIdeograph5571', [], 'Display', None],
  [ 0x5572, 'CjkUnifiedIdeograph5572', [], 'Display', None],
  [ 0x5573, 'CjkUnifiedIdeograph5573', [], 'Display', None],
  [ 0x5574, 'CjkUnifiedIdeograph5574', [], 'Display', None],
  [ 0x5575, 'CjkUnifiedIdeograph5575', [], 'Display', None],
  [ 0x5576, 'CjkUnifiedIdeograph5576', [], 'Display', None],
  [ 0x5577, 'CjkUnifiedIdeograph5577', [], 'Display', None],
  [ 0x5578, 'CjkUnifiedIdeograph5578', [], 'Display', None],
  [ 0x5579, 'CjkUnifiedIdeograph5579', [], 'Display', None],
  [ 0x557a, 'CjkUnifiedIdeograph557a', [], 'Display', None],
  [ 0x557b, 'CjkUnifiedIdeograph557b', [], 'Display', None],
  [ 0x557c, 'CjkUnifiedIdeograph557c', [], 'Display', None],
  [ 0x557d, 'CjkUnifiedIdeograph557d', [], 'Display', None],
  [ 0x557e, 'CjkUnifiedIdeograph557e', [], 'Display', None],
  [ 0x557f, 'CjkUnifiedIdeograph557f', [], 'Display', None],
  [ 0x5580, 'CjkUnifiedIdeograph5580', [], 'Display', None],
  [ 0x5581, 'CjkUnifiedIdeograph5581', [], 'Display', None],
  [ 0x5582, 'CjkUnifiedIdeograph5582', [], 'Display', None],
  [ 0x5583, 'CjkUnifiedIdeograph5583', [], 'Display', None],
  [ 0x5584, 'CjkUnifiedIdeograph5584', [], 'Display', None],
  [ 0x5585, 'CjkUnifiedIdeograph5585', [], 'Display', None],
  [ 0x5586, 'CjkUnifiedIdeograph5586', [], 'Display', None],
  [ 0x5587, 'CjkUnifiedIdeograph5587', [], 'Display', None],
  [ 0x5588, 'CjkUnifiedIdeograph5588', [], 'Display', None],
  [ 0x5589, 'CjkUnifiedIdeograph5589', [], 'Display', None],
  [ 0x558a, 'CjkUnifiedIdeograph558a', [], 'Display', None],
  [ 0x558b, 'CjkUnifiedIdeograph558b', [], 'Display', None],
  [ 0x558c, 'CjkUnifiedIdeograph558c', [], 'Display', None],
  [ 0x558d, 'CjkUnifiedIdeograph558d', [], 'Display', None],
  [ 0x558e, 'CjkUnifiedIdeograph558e', [], 'Display', None],
  [ 0x558f, 'CjkUnifiedIdeograph558f', [], 'Display', None],
  [ 0x5590, 'CjkUnifiedIdeograph5590', [], 'Display', None],
  [ 0x5591, 'CjkUnifiedIdeograph5591', [], 'Display', None],
  [ 0x5592, 'CjkUnifiedIdeograph5592', [], 'Display', None],
  [ 0x5593, 'CjkUnifiedIdeograph5593', [], 'Display', None],
  [ 0x5594, 'CjkUnifiedIdeograph5594', [], 'Display', None],
  [ 0x5595, 'CjkUnifiedIdeograph5595', [], 'Display', None],
  [ 0x5596, 'CjkUnifiedIdeograph5596', [], 'Display', None],
  [ 0x5597, 'CjkUnifiedIdeograph5597', [], 'Display', None],
  [ 0x5598, 'CjkUnifiedIdeograph5598', [], 'Display', None],
  [ 0x5599, 'CjkUnifiedIdeograph5599', [], 'Display', None],
  [ 0x559a, 'CjkUnifiedIdeograph559a', [], 'Display', None],
  [ 0x559b, 'CjkUnifiedIdeograph559b', [], 'Display', None],
  [ 0x559c, 'CjkUnifiedIdeograph559c', [], 'Display', None],
  [ 0x559d, 'CjkUnifiedIdeograph559d', [], 'Display', None],
  [ 0x559e, 'CjkUnifiedIdeograph559e', [], 'Display', None],
  [ 0x559f, 'CjkUnifiedIdeograph559f', [], 'Display', None],
  [ 0x55a0, 'CjkUnifiedIdeograph55a0', [], 'Display', None],
  [ 0x55a1, 'CjkUnifiedIdeograph55a1', [], 'Display', None],
  [ 0x55a2, 'CjkUnifiedIdeograph55a2', [], 'Display', None],
  [ 0x55a3, 'CjkUnifiedIdeograph55a3', [], 'Display', None],
  [ 0x55a4, 'CjkUnifiedIdeograph55a4', [], 'Display', None],
  [ 0x55a5, 'CjkUnifiedIdeograph55a5', [], 'Display', None],
  [ 0x55a6, 'CjkUnifiedIdeograph55a6', [], 'Display', None],
  [ 0x55a7, 'CjkUnifiedIdeograph55a7', [], 'Display', None],
  [ 0x55a8, 'CjkUnifiedIdeograph55a8', [], 'Display', None],
  [ 0x55a9, 'CjkUnifiedIdeograph55a9', [], 'Display', None],
  [ 0x55aa, 'CjkUnifiedIdeograph55aa', [], 'Display', None],
  [ 0x55ab, 'CjkUnifiedIdeograph55ab', [], 'Display', None],
  [ 0x55ac, 'CjkUnifiedIdeograph55ac', [], 'Display', None],
  [ 0x55ad, 'CjkUnifiedIdeograph55ad', [], 'Display', None],
  [ 0x55ae, 'CjkUnifiedIdeograph55ae', [], 'Display', None],
  [ 0x55af, 'CjkUnifiedIdeograph55af', [], 'Display', None],
  [ 0x55b0, 'CjkUnifiedIdeograph55b0', [], 'Display', None],
  [ 0x55b1, 'CjkUnifiedIdeograph55b1', [], 'Display', None],
  [ 0x55b2, 'CjkUnifiedIdeograph55b2', [], 'Display', None],
  [ 0x55b3, 'CjkUnifiedIdeograph55b3', [], 'Display', None],
  [ 0x55b4, 'CjkUnifiedIdeograph55b4', [], 'Display', None],
  [ 0x55b5, 'CjkUnifiedIdeograph55b5', [], 'Display', None],
  [ 0x55b6, 'CjkUnifiedIdeograph55b6', [], 'Display', None],
  [ 0x55b7, 'CjkUnifiedIdeograph55b7', [], 'Display', None],
  [ 0x55b8, 'CjkUnifiedIdeograph55b8', [], 'Display', None],
  [ 0x55b9, 'CjkUnifiedIdeograph55b9', [], 'Display', None],
  [ 0x55ba, 'CjkUnifiedIdeograph55ba', [], 'Display', None],
  [ 0x55bb, 'CjkUnifiedIdeograph55bb', [], 'Display', None],
  [ 0x55bc, 'CjkUnifiedIdeograph55bc', [], 'Display', None],
  [ 0x55bd, 'CjkUnifiedIdeograph55bd', [], 'Display', None],
  [ 0x55be, 'CjkUnifiedIdeograph55be', [], 'Display', None],
  [ 0x55bf, 'CjkUnifiedIdeograph55bf', [], 'Display', None],
  [ 0x55c0, 'CjkUnifiedIdeograph55c0', [], 'Display', None],
  [ 0x55c1, 'CjkUnifiedIdeograph55c1', [], 'Display', None],
  [ 0x55c2, 'CjkUnifiedIdeograph55c2', [], 'Display', None],
  [ 0x55c3, 'CjkUnifiedIdeograph55c3', [], 'Display', None],
  [ 0x55c4, 'CjkUnifiedIdeograph55c4', [], 'Display', None],
  [ 0x55c5, 'CjkUnifiedIdeograph55c5', [], 'Display', None],
  [ 0x55c6, 'CjkUnifiedIdeograph55c6', [], 'Display', None],
  [ 0x55c7, 'CjkUnifiedIdeograph55c7', [], 'Display', None],
  [ 0x55c8, 'CjkUnifiedIdeograph55c8', [], 'Display', None],
  [ 0x55c9, 'CjkUnifiedIdeograph55c9', [], 'Display', None],
  [ 0x55ca, 'CjkUnifiedIdeograph55ca', [], 'Display', None],
  [ 0x55cb, 'CjkUnifiedIdeograph55cb', [], 'Display', None],
  [ 0x55cc, 'CjkUnifiedIdeograph55cc', [], 'Display', None],
  [ 0x55cd, 'CjkUnifiedIdeograph55cd', [], 'Display', None],
  [ 0x55ce, 'CjkUnifiedIdeograph55ce', [], 'Display', None],
  [ 0x55cf, 'CjkUnifiedIdeograph55cf', [], 'Display', None],
  [ 0x55d0, 'CjkUnifiedIdeograph55d0', [], 'Display', None],
  [ 0x55d1, 'CjkUnifiedIdeograph55d1', [], 'Display', None],
  [ 0x55d2, 'CjkUnifiedIdeograph55d2', [], 'Display', None],
  [ 0x55d3, 'CjkUnifiedIdeograph55d3', [], 'Display', None],
  [ 0x55d4, 'CjkUnifiedIdeograph55d4', [], 'Display', None],
  [ 0x55d5, 'CjkUnifiedIdeograph55d5', [], 'Display', None],
  [ 0x55d6, 'CjkUnifiedIdeograph55d6', [], 'Display', None],
  [ 0x55d7, 'CjkUnifiedIdeograph55d7', [], 'Display', None],
  [ 0x55d8, 'CjkUnifiedIdeograph55d8', [], 'Display', None],
  [ 0x55d9, 'CjkUnifiedIdeograph55d9', [], 'Display', None],
  [ 0x55da, 'CjkUnifiedIdeograph55da', [], 'Display', None],
  [ 0x55db, 'CjkUnifiedIdeograph55db', [], 'Display', None],
  [ 0x55dc, 'CjkUnifiedIdeograph55dc', [], 'Display', None],
  [ 0x55dd, 'CjkUnifiedIdeograph55dd', [], 'Display', None],
  [ 0x55de, 'CjkUnifiedIdeograph55de', [], 'Display', None],
  [ 0x55df, 'CjkUnifiedIdeograph55df', [], 'Display', None],
  [ 0x55e0, 'CjkUnifiedIdeograph55e0', [], 'Display', None],
  [ 0x55e1, 'CjkUnifiedIdeograph55e1', [], 'Display', None],
  [ 0x55e2, 'CjkUnifiedIdeograph55e2', [], 'Display', None],
  [ 0x55e3, 'CjkUnifiedIdeograph55e3', [], 'Display', None],
  [ 0x55e4, 'CjkUnifiedIdeograph55e4', [], 'Display', None],
  [ 0x55e5, 'CjkUnifiedIdeograph55e5', [], 'Display', None],
  [ 0x55e6, 'CjkUnifiedIdeograph55e6', [], 'Display', None],
  [ 0x55e7, 'CjkUnifiedIdeograph55e7', [], 'Display', None],
  [ 0x55e8, 'CjkUnifiedIdeograph55e8', [], 'Display', None],
  [ 0x55e9, 'CjkUnifiedIdeograph55e9', [], 'Display', None],
  [ 0x55ea, 'CjkUnifiedIdeograph55ea', [], 'Display', None],
  [ 0x55eb, 'CjkUnifiedIdeograph55eb', [], 'Display', None],
  [ 0x55ec, 'CjkUnifiedIdeograph55ec', [], 'Display', None],
  [ 0x55ed, 'CjkUnifiedIdeograph55ed', [], 'Display', None],
  [ 0x55ee, 'CjkUnifiedIdeograph55ee', [], 'Display', None],
  [ 0x55ef, 'CjkUnifiedIdeograph55ef', [], 'Display', None],
  [ 0x55f0, 'CjkUnifiedIdeograph55f0', [], 'Display', None],
  [ 0x55f1, 'CjkUnifiedIdeograph55f1', [], 'Display', None],
  [ 0x55f2, 'CjkUnifiedIdeograph55f2', [], 'Display', None],
  [ 0x55f3, 'CjkUnifiedIdeograph55f3', [], 'Display', None],
  [ 0x55f4, 'CjkUnifiedIdeograph55f4', [], 'Display', None],
  [ 0x55f5, 'CjkUnifiedIdeograph55f5', [], 'Display', None],
  [ 0x55f6, 'CjkUnifiedIdeograph55f6', [], 'Display', None],
  [ 0x55f7, 'CjkUnifiedIdeograph55f7', [], 'Display', None],
  [ 0x55f8, 'CjkUnifiedIdeograph55f8', [], 'Display', None],
  [ 0x55f9, 'CjkUnifiedIdeograph55f9', [], 'Display', None],
  [ 0x55fa, 'CjkUnifiedIdeograph55fa', [], 'Display', None],
  [ 0x55fb, 'CjkUnifiedIdeograph55fb', [], 'Display', None],
  [ 0x55fc, 'CjkUnifiedIdeograph55fc', [], 'Display', None],
  [ 0x55fd, 'CjkUnifiedIdeograph55fd', [], 'Display', None],
  [ 0x55fe, 'CjkUnifiedIdeograph55fe', [], 'Display', None],
  [ 0x55ff, 'CjkUnifiedIdeograph55ff', [], 'Display', None],
  [ 0x5600, 'CjkUnifiedIdeograph5600', [], 'Display', None],
  [ 0x5601, 'CjkUnifiedIdeograph5601', [], 'Display', None],
  [ 0x5602, 'CjkUnifiedIdeograph5602', [], 'Display', None],
  [ 0x5603, 'CjkUnifiedIdeograph5603', [], 'Display', None],
  [ 0x5604, 'CjkUnifiedIdeograph5604', [], 'Display', None],
  [ 0x5605, 'CjkUnifiedIdeograph5605', [], 'Display', None],
  [ 0x5606, 'CjkUnifiedIdeograph5606', [], 'Display', None],
  [ 0x5607, 'CjkUnifiedIdeograph5607', [], 'Display', None],
  [ 0x5608, 'CjkUnifiedIdeograph5608', [], 'Display', None],
  [ 0x5609, 'CjkUnifiedIdeograph5609', [], 'Display', None],
  [ 0x560a, 'CjkUnifiedIdeograph560a', [], 'Display', None],
  [ 0x560b, 'CjkUnifiedIdeograph560b', [], 'Display', None],
  [ 0x560c, 'CjkUnifiedIdeograph560c', [], 'Display', None],
  [ 0x560d, 'CjkUnifiedIdeograph560d', [], 'Display', None],
  [ 0x560e, 'CjkUnifiedIdeograph560e', [], 'Display', None],
  [ 0x560f, 'CjkUnifiedIdeograph560f', [], 'Display', None],
  [ 0x5610, 'CjkUnifiedIdeograph5610', [], 'Display', None],
  [ 0x5611, 'CjkUnifiedIdeograph5611', [], 'Display', None],
  [ 0x5612, 'CjkUnifiedIdeograph5612', [], 'Display', None],
  [ 0x5613, 'CjkUnifiedIdeograph5613', [], 'Display', None],
  [ 0x5614, 'CjkUnifiedIdeograph5614', [], 'Display', None],
  [ 0x5615, 'CjkUnifiedIdeograph5615', [], 'Display', None],
  [ 0x5616, 'CjkUnifiedIdeograph5616', [], 'Display', None],
  [ 0x5617, 'CjkUnifiedIdeograph5617', [], 'Display', None],
  [ 0x5618, 'CjkUnifiedIdeograph5618', [], 'Display', None],
  [ 0x5619, 'CjkUnifiedIdeograph5619', [], 'Display', None],
  [ 0x561a, 'CjkUnifiedIdeograph561a', [], 'Display', None],
  [ 0x561b, 'CjkUnifiedIdeograph561b', [], 'Display', None],
  [ 0x561c, 'CjkUnifiedIdeograph561c', [], 'Display', None],
  [ 0x561d, 'CjkUnifiedIdeograph561d', [], 'Display', None],
  [ 0x561e, 'CjkUnifiedIdeograph561e', [], 'Display', None],
  [ 0x561f, 'CjkUnifiedIdeograph561f', [], 'Display', None],
  [ 0x5620, 'CjkUnifiedIdeograph5620', [], 'Display', None],
  [ 0x5621, 'CjkUnifiedIdeograph5621', [], 'Display', None],
  [ 0x5622, 'CjkUnifiedIdeograph5622', [], 'Display', None],
  [ 0x5623, 'CjkUnifiedIdeograph5623', [], 'Display', None],
  [ 0x5624, 'CjkUnifiedIdeograph5624', [], 'Display', None],
  [ 0x5625, 'CjkUnifiedIdeograph5625', [], 'Display', None],
  [ 0x5626, 'CjkUnifiedIdeograph5626', [], 'Display', None],
  [ 0x5627, 'CjkUnifiedIdeograph5627', [], 'Display', None],
  [ 0x5628, 'CjkUnifiedIdeograph5628', [], 'Display', None],
  [ 0x5629, 'CjkUnifiedIdeograph5629', [], 'Display', None],
  [ 0x562a, 'CjkUnifiedIdeograph562a', [], 'Display', None],
  [ 0x562b, 'CjkUnifiedIdeograph562b', [], 'Display', None],
  [ 0x562c, 'CjkUnifiedIdeograph562c', [], 'Display', None],
  [ 0x562d, 'CjkUnifiedIdeograph562d', [], 'Display', None],
  [ 0x562e, 'CjkUnifiedIdeograph562e', [], 'Display', None],
  [ 0x562f, 'CjkUnifiedIdeograph562f', [], 'Display', None],
  [ 0x5630, 'CjkUnifiedIdeograph5630', [], 'Display', None],
  [ 0x5631, 'CjkUnifiedIdeograph5631', [], 'Display', None],
  [ 0x5632, 'CjkUnifiedIdeograph5632', [], 'Display', None],
  [ 0x5633, 'CjkUnifiedIdeograph5633', [], 'Display', None],
  [ 0x5634, 'CjkUnifiedIdeograph5634', [], 'Display', None],
  [ 0x5635, 'CjkUnifiedIdeograph5635', [], 'Display', None],
  [ 0x5636, 'CjkUnifiedIdeograph5636', [], 'Display', None],
  [ 0x5637, 'CjkUnifiedIdeograph5637', [], 'Display', None],
  [ 0x5638, 'CjkUnifiedIdeograph5638', [], 'Display', None],
  [ 0x5639, 'CjkUnifiedIdeograph5639', [], 'Display', None],
  [ 0x563a, 'CjkUnifiedIdeograph563a', [], 'Display', None],
  [ 0x563b, 'CjkUnifiedIdeograph563b', [], 'Display', None],
  [ 0x563c, 'CjkUnifiedIdeograph563c', [], 'Display', None],
  [ 0x563d, 'CjkUnifiedIdeograph563d', [], 'Display', None],
  [ 0x563e, 'CjkUnifiedIdeograph563e', [], 'Display', None],
  [ 0x563f, 'CjkUnifiedIdeograph563f', [], 'Display', None],
  [ 0x5640, 'CjkUnifiedIdeograph5640', [], 'Display', None],
  [ 0x5641, 'CjkUnifiedIdeograph5641', [], 'Display', None],
  [ 0x5642, 'CjkUnifiedIdeograph5642', [], 'Display', None],
  [ 0x5643, 'CjkUnifiedIdeograph5643', [], 'Display', None],
  [ 0x5644, 'CjkUnifiedIdeograph5644', [], 'Display', None],
  [ 0x5645, 'CjkUnifiedIdeograph5645', [], 'Display', None],
  [ 0x5646, 'CjkUnifiedIdeograph5646', [], 'Display', None],
  [ 0x5647, 'CjkUnifiedIdeograph5647', [], 'Display', None],
  [ 0x5648, 'CjkUnifiedIdeograph5648', [], 'Display', None],
  [ 0x5649, 'CjkUnifiedIdeograph5649', [], 'Display', None],
  [ 0x564a, 'CjkUnifiedIdeograph564a', [], 'Display', None],
  [ 0x564b, 'CjkUnifiedIdeograph564b', [], 'Display', None],
  [ 0x564c, 'CjkUnifiedIdeograph564c', [], 'Display', None],
  [ 0x564d, 'CjkUnifiedIdeograph564d', [], 'Display', None],
  [ 0x564e, 'CjkUnifiedIdeograph564e', [], 'Display', None],
  [ 0x564f, 'CjkUnifiedIdeograph564f', [], 'Display', None],
  [ 0x5650, 'CjkUnifiedIdeograph5650', [], 'Display', None],
  [ 0x5651, 'CjkUnifiedIdeograph5651', [], 'Display', None],
  [ 0x5652, 'CjkUnifiedIdeograph5652', [], 'Display', None],
  [ 0x5653, 'CjkUnifiedIdeograph5653', [], 'Display', None],
  [ 0x5654, 'CjkUnifiedIdeograph5654', [], 'Display', None],
  [ 0x5655, 'CjkUnifiedIdeograph5655', [], 'Display', None],
  [ 0x5656, 'CjkUnifiedIdeograph5656', [], 'Display', None],
  [ 0x5657, 'CjkUnifiedIdeograph5657', [], 'Display', None],
  [ 0x5658, 'CjkUnifiedIdeograph5658', [], 'Display', None],
  [ 0x5659, 'CjkUnifiedIdeograph5659', [], 'Display', None],
  [ 0x565a, 'CjkUnifiedIdeograph565a', [], 'Display', None],
  [ 0x565b, 'CjkUnifiedIdeograph565b', [], 'Display', None],
  [ 0x565c, 'CjkUnifiedIdeograph565c', [], 'Display', None],
  [ 0x565d, 'CjkUnifiedIdeograph565d', [], 'Display', None],
  [ 0x565e, 'CjkUnifiedIdeograph565e', [], 'Display', None],
  [ 0x565f, 'CjkUnifiedIdeograph565f', [], 'Display', None],
  [ 0x5660, 'CjkUnifiedIdeograph5660', [], 'Display', None],
  [ 0x5661, 'CjkUnifiedIdeograph5661', [], 'Display', None],
  [ 0x5662, 'CjkUnifiedIdeograph5662', [], 'Display', None],
  [ 0x5663, 'CjkUnifiedIdeograph5663', [], 'Display', None],
  [ 0x5664, 'CjkUnifiedIdeograph5664', [], 'Display', None],
  [ 0x5665, 'CjkUnifiedIdeograph5665', [], 'Display', None],
  [ 0x5666, 'CjkUnifiedIdeograph5666', [], 'Display', None],
  [ 0x5667, 'CjkUnifiedIdeograph5667', [], 'Display', None],
  [ 0x5668, 'CjkUnifiedIdeograph5668', [], 'Display', None],
  [ 0x5669, 'CjkUnifiedIdeograph5669', [], 'Display', None],
  [ 0x566a, 'CjkUnifiedIdeograph566a', [], 'Display', None],
  [ 0x566b, 'CjkUnifiedIdeograph566b', [], 'Display', None],
  [ 0x566c, 'CjkUnifiedIdeograph566c', [], 'Display', None],
  [ 0x566d, 'CjkUnifiedIdeograph566d', [], 'Display', None],
  [ 0x566e, 'CjkUnifiedIdeograph566e', [], 'Display', None],
  [ 0x566f, 'CjkUnifiedIdeograph566f', [], 'Display', None],
  [ 0x5670, 'CjkUnifiedIdeograph5670', [], 'Display', None],
  [ 0x5671, 'CjkUnifiedIdeograph5671', [], 'Display', None],
  [ 0x5672, 'CjkUnifiedIdeograph5672', [], 'Display', None],
  [ 0x5673, 'CjkUnifiedIdeograph5673', [], 'Display', None],
  [ 0x5674, 'CjkUnifiedIdeograph5674', [], 'Display', None],
  [ 0x5675, 'CjkUnifiedIdeograph5675', [], 'Display', None],
  [ 0x5676, 'CjkUnifiedIdeograph5676', [], 'Display', None],
  [ 0x5677, 'CjkUnifiedIdeograph5677', [], 'Display', None],
  [ 0x5678, 'CjkUnifiedIdeograph5678', [], 'Display', None],
  [ 0x5679, 'CjkUnifiedIdeograph5679', [], 'Display', None],
  [ 0x567a, 'CjkUnifiedIdeograph567a', [], 'Display', None],
  [ 0x567b, 'CjkUnifiedIdeograph567b', [], 'Display', None],
  [ 0x567c, 'CjkUnifiedIdeograph567c', [], 'Display', None],
  [ 0x567d, 'CjkUnifiedIdeograph567d', [], 'Display', None],
  [ 0x567e, 'CjkUnifiedIdeograph567e', [], 'Display', None],
  [ 0x567f, 'CjkUnifiedIdeograph567f', [], 'Display', None],
  [ 0x5680, 'CjkUnifiedIdeograph5680', [], 'Display', None],
  [ 0x5681, 'CjkUnifiedIdeograph5681', [], 'Display', None],
  [ 0x5682, 'CjkUnifiedIdeograph5682', [], 'Display', None],
  [ 0x5683, 'CjkUnifiedIdeograph5683', [], 'Display', None],
  [ 0x5684, 'CjkUnifiedIdeograph5684', [], 'Display', None],
  [ 0x5685, 'CjkUnifiedIdeograph5685', [], 'Display', None],
  [ 0x5686, 'CjkUnifiedIdeograph5686', [], 'Display', None],
  [ 0x5687, 'CjkUnifiedIdeograph5687', [], 'Display', None],
  [ 0x5688, 'CjkUnifiedIdeograph5688', [], 'Display', None],
  [ 0x5689, 'CjkUnifiedIdeograph5689', [], 'Display', None],
  [ 0x568a, 'CjkUnifiedIdeograph568a', [], 'Display', None],
  [ 0x568b, 'CjkUnifiedIdeograph568b', [], 'Display', None],
  [ 0x568c, 'CjkUnifiedIdeograph568c', [], 'Display', None],
  [ 0x568d, 'CjkUnifiedIdeograph568d', [], 'Display', None],
  [ 0x568e, 'CjkUnifiedIdeograph568e', [], 'Display', None],
  [ 0x568f, 'CjkUnifiedIdeograph568f', [], 'Display', None],
  [ 0x5690, 'CjkUnifiedIdeograph5690', [], 'Display', None],
  [ 0x5691, 'CjkUnifiedIdeograph5691', [], 'Display', None],
  [ 0x5692, 'CjkUnifiedIdeograph5692', [], 'Display', None],
  [ 0x5693, 'CjkUnifiedIdeograph5693', [], 'Display', None],
  [ 0x5694, 'CjkUnifiedIdeograph5694', [], 'Display', None],
  [ 0x5695, 'CjkUnifiedIdeograph5695', [], 'Display', None],
  [ 0x5696, 'CjkUnifiedIdeograph5696', [], 'Display', None],
  [ 0x5697, 'CjkUnifiedIdeograph5697', [], 'Display', None],
  [ 0x5698, 'CjkUnifiedIdeograph5698', [], 'Display', None],
  [ 0x5699, 'CjkUnifiedIdeograph5699', [], 'Display', None],
  [ 0x569a, 'CjkUnifiedIdeograph569a', [], 'Display', None],
  [ 0x569b, 'CjkUnifiedIdeograph569b', [], 'Display', None],
  [ 0x569c, 'CjkUnifiedIdeograph569c', [], 'Display', None],
  [ 0x569d, 'CjkUnifiedIdeograph569d', [], 'Display', None],
  [ 0x569e, 'CjkUnifiedIdeograph569e', [], 'Display', None],
  [ 0x569f, 'CjkUnifiedIdeograph569f', [], 'Display', None],
  [ 0x56a0, 'CjkUnifiedIdeograph56a0', [], 'Display', None],
  [ 0x56a1, 'CjkUnifiedIdeograph56a1', [], 'Display', None],
  [ 0x56a2, 'CjkUnifiedIdeograph56a2', [], 'Display', None],
  [ 0x56a3, 'CjkUnifiedIdeograph56a3', [], 'Display', None],
  [ 0x56a4, 'CjkUnifiedIdeograph56a4', [], 'Display', None],
  [ 0x56a5, 'CjkUnifiedIdeograph56a5', [], 'Display', None],
  [ 0x56a6, 'CjkUnifiedIdeograph56a6', [], 'Display', None],
  [ 0x56a7, 'CjkUnifiedIdeograph56a7', [], 'Display', None],
  [ 0x56a8, 'CjkUnifiedIdeograph56a8', [], 'Display', None],
  [ 0x56a9, 'CjkUnifiedIdeograph56a9', [], 'Display', None],
  [ 0x56aa, 'CjkUnifiedIdeograph56aa', [], 'Display', None],
  [ 0x56ab, 'CjkUnifiedIdeograph56ab', [], 'Display', None],
  [ 0x56ac, 'CjkUnifiedIdeograph56ac', [], 'Display', None],
  [ 0x56ad, 'CjkUnifiedIdeograph56ad', [], 'Display', None],
  [ 0x56ae, 'CjkUnifiedIdeograph56ae', [], 'Display', None],
  [ 0x56af, 'CjkUnifiedIdeograph56af', [], 'Display', None],
  [ 0x56b0, 'CjkUnifiedIdeograph56b0', [], 'Display', None],
  [ 0x56b1, 'CjkUnifiedIdeograph56b1', [], 'Display', None],
  [ 0x56b2, 'CjkUnifiedIdeograph56b2', [], 'Display', None],
  [ 0x56b3, 'CjkUnifiedIdeograph56b3', [], 'Display', None],
  [ 0x56b4, 'CjkUnifiedIdeograph56b4', [], 'Display', None],
  [ 0x56b5, 'CjkUnifiedIdeograph56b5', [], 'Display', None],
  [ 0x56b6, 'CjkUnifiedIdeograph56b6', [], 'Display', None],
  [ 0x56b7, 'CjkUnifiedIdeograph56b7', [], 'Display', None],
  [ 0x56b8, 'CjkUnifiedIdeograph56b8', [], 'Display', None],
  [ 0x56b9, 'CjkUnifiedIdeograph56b9', [], 'Display', None],
  [ 0x56ba, 'CjkUnifiedIdeograph56ba', [], 'Display', None],
  [ 0x56bb, 'CjkUnifiedIdeograph56bb', [], 'Display', None],
  [ 0x56bc, 'CjkUnifiedIdeograph56bc', [], 'Display', None],
  [ 0x56bd, 'CjkUnifiedIdeograph56bd', [], 'Display', None],
  [ 0x56be, 'CjkUnifiedIdeograph56be', [], 'Display', None],
  [ 0x56bf, 'CjkUnifiedIdeograph56bf', [], 'Display', None],
  [ 0x56c0, 'CjkUnifiedIdeograph56c0', [], 'Display', None],
  [ 0x56c1, 'CjkUnifiedIdeograph56c1', [], 'Display', None],
  [ 0x56c2, 'CjkUnifiedIdeograph56c2', [], 'Display', None],
  [ 0x56c3, 'CjkUnifiedIdeograph56c3', [], 'Display', None],
  [ 0x56c4, 'CjkUnifiedIdeograph56c4', [], 'Display', None],
  [ 0x56c5, 'CjkUnifiedIdeograph56c5', [], 'Display', None],
  [ 0x56c6, 'CjkUnifiedIdeograph56c6', [], 'Display', None],
  [ 0x56c7, 'CjkUnifiedIdeograph56c7', [], 'Display', None],
  [ 0x56c8, 'CjkUnifiedIdeograph56c8', [], 'Display', None],
  [ 0x56c9, 'CjkUnifiedIdeograph56c9', [], 'Display', None],
  [ 0x56ca, 'CjkUnifiedIdeograph56ca', [], 'Display', None],
  [ 0x56cb, 'CjkUnifiedIdeograph56cb', [], 'Display', None],
  [ 0x56cc, 'CjkUnifiedIdeograph56cc', [], 'Display', None],
  [ 0x56cd, 'CjkUnifiedIdeograph56cd', [], 'Display', None],
  [ 0x56ce, 'CjkUnifiedIdeograph56ce', [], 'Display', None],
  [ 0x56cf, 'CjkUnifiedIdeograph56cf', [], 'Display', None],
  [ 0x56d0, 'CjkUnifiedIdeograph56d0', [], 'Display', None],
  [ 0x56d1, 'CjkUnifiedIdeograph56d1', [], 'Display', None],
  [ 0x56d2, 'CjkUnifiedIdeograph56d2', [], 'Display', None],
  [ 0x56d3, 'CjkUnifiedIdeograph56d3', [], 'Display', None],
  [ 0x56d4, 'CjkUnifiedIdeograph56d4', [], 'Display', None],
  [ 0x56d5, 'CjkUnifiedIdeograph56d5', [], 'Display', None],
  [ 0x56d6, 'CjkUnifiedIdeograph56d6', [], 'Display', None],
  [ 0x56d7, 'CjkUnifiedIdeograph56d7', [], 'Display', None],
  [ 0x56d8, 'CjkUnifiedIdeograph56d8', [], 'Display', None],
  [ 0x56d9, 'CjkUnifiedIdeograph56d9', [], 'Display', None],
  [ 0x56da, 'CjkUnifiedIdeograph56da', [], 'Display', None],
  [ 0x56db, 'CjkUnifiedIdeograph56db', [], 'Display', None],
  [ 0x56dc, 'CjkUnifiedIdeograph56dc', [], 'Display', None],
  [ 0x56dd, 'CjkUnifiedIdeograph56dd', [], 'Display', None],
  [ 0x56de, 'CjkUnifiedIdeograph56de', [], 'Display', None],
  [ 0x56df, 'CjkUnifiedIdeograph56df', [], 'Display', None],
  [ 0x56e0, 'CjkUnifiedIdeograph56e0', [], 'Display', None],
  [ 0x56e1, 'CjkUnifiedIdeograph56e1', [], 'Display', None],
  [ 0x56e2, 'CjkUnifiedIdeograph56e2', [], 'Display', None],
  [ 0x56e3, 'CjkUnifiedIdeograph56e3', [], 'Display', None],
  [ 0x56e4, 'CjkUnifiedIdeograph56e4', [], 'Display', None],
  [ 0x56e5, 'CjkUnifiedIdeograph56e5', [], 'Display', None],
  [ 0x56e6, 'CjkUnifiedIdeograph56e6', [], 'Display', None],
  [ 0x56e7, 'CjkUnifiedIdeograph56e7', [], 'Display', None],
  [ 0x56e8, 'CjkUnifiedIdeograph56e8', [], 'Display', None],
  [ 0x56e9, 'CjkUnifiedIdeograph56e9', [], 'Display', None],
  [ 0x56ea, 'CjkUnifiedIdeograph56ea', [], 'Display', None],
  [ 0x56eb, 'CjkUnifiedIdeograph56eb', [], 'Display', None],
  [ 0x56ec, 'CjkUnifiedIdeograph56ec', [], 'Display', None],
  [ 0x56ed, 'CjkUnifiedIdeograph56ed', [], 'Display', None],
  [ 0x56ee, 'CjkUnifiedIdeograph56ee', [], 'Display', None],
  [ 0x56ef, 'CjkUnifiedIdeograph56ef', [], 'Display', None],
  [ 0x56f0, 'CjkUnifiedIdeograph56f0', [], 'Display', None],
  [ 0x56f1, 'CjkUnifiedIdeograph56f1', [], 'Display', None],
  [ 0x56f2, 'CjkUnifiedIdeograph56f2', [], 'Display', None],
  [ 0x56f3, 'CjkUnifiedIdeograph56f3', [], 'Display', None],
  [ 0x56f4, 'CjkUnifiedIdeograph56f4', [], 'Display', None],
  [ 0x56f5, 'CjkUnifiedIdeograph56f5', [], 'Display', None],
  [ 0x56f6, 'CjkUnifiedIdeograph56f6', [], 'Display', None],
  [ 0x56f7, 'CjkUnifiedIdeograph56f7', [], 'Display', None],
  [ 0x56f8, 'CjkUnifiedIdeograph56f8', [], 'Display', None],
  [ 0x56f9, 'CjkUnifiedIdeograph56f9', [], 'Display', None],
  [ 0x56fa, 'CjkUnifiedIdeograph56fa', [], 'Display', None],
  [ 0x56fb, 'CjkUnifiedIdeograph56fb', [], 'Display', None],
  [ 0x56fc, 'CjkUnifiedIdeograph56fc', [], 'Display', None],
  [ 0x56fd, 'CjkUnifiedIdeograph56fd', [], 'Display', None],
  [ 0x56fe, 'CjkUnifiedIdeograph56fe', [], 'Display', None],
  [ 0x56ff, 'CjkUnifiedIdeograph56ff', [], 'Display', None],
  [ 0x5700, 'CjkUnifiedIdeograph5700', [], 'Display', None],
  [ 0x5701, 'CjkUnifiedIdeograph5701', [], 'Display', None],
  [ 0x5702, 'CjkUnifiedIdeograph5702', [], 'Display', None],
  [ 0x5703, 'CjkUnifiedIdeograph5703', [], 'Display', None],
  [ 0x5704, 'CjkUnifiedIdeograph5704', [], 'Display', None],
  [ 0x5705, 'CjkUnifiedIdeograph5705', [], 'Display', None],
  [ 0x5706, 'CjkUnifiedIdeograph5706', [], 'Display', None],
  [ 0x5707, 'CjkUnifiedIdeograph5707', [], 'Display', None],
  [ 0x5708, 'CjkUnifiedIdeograph5708', [], 'Display', None],
  [ 0x5709, 'CjkUnifiedIdeograph5709', [], 'Display', None],
  [ 0x570a, 'CjkUnifiedIdeograph570a', [], 'Display', None],
  [ 0x570b, 'CjkUnifiedIdeograph570b', [], 'Display', None],
  [ 0x570c, 'CjkUnifiedIdeograph570c', [], 'Display', None],
  [ 0x570d, 'CjkUnifiedIdeograph570d', [], 'Display', None],
  [ 0x570e, 'CjkUnifiedIdeograph570e', [], 'Display', None],
  [ 0x570f, 'CjkUnifiedIdeograph570f', [], 'Display', None],
  [ 0x5710, 'CjkUnifiedIdeograph5710', [], 'Display', None],
  [ 0x5711, 'CjkUnifiedIdeograph5711', [], 'Display', None],
  [ 0x5712, 'CjkUnifiedIdeograph5712', [], 'Display', None],
  [ 0x5713, 'CjkUnifiedIdeograph5713', [], 'Display', None],
  [ 0x5714, 'CjkUnifiedIdeograph5714', [], 'Display', None],
  [ 0x5715, 'CjkUnifiedIdeograph5715', [], 'Display', None],
  [ 0x5716, 'CjkUnifiedIdeograph5716', [], 'Display', None],
  [ 0x5717, 'CjkUnifiedIdeograph5717', [], 'Display', None],
  [ 0x5718, 'CjkUnifiedIdeograph5718', [], 'Display', None],
  [ 0x5719, 'CjkUnifiedIdeograph5719', [], 'Display', None],
  [ 0x571a, 'CjkUnifiedIdeograph571a', [], 'Display', None],
  [ 0x571b, 'CjkUnifiedIdeograph571b', [], 'Display', None],
  [ 0x571c, 'CjkUnifiedIdeograph571c', [], 'Display', None],
  [ 0x571d, 'CjkUnifiedIdeograph571d', [], 'Display', None],
  [ 0x571e, 'CjkUnifiedIdeograph571e', [], 'Display', None],
  [ 0x571f, 'CjkUnifiedIdeograph571f', [], 'Display', None],
  [ 0x5720, 'CjkUnifiedIdeograph5720', [], 'Display', None],
  [ 0x5721, 'CjkUnifiedIdeograph5721', [], 'Display', None],
  [ 0x5722, 'CjkUnifiedIdeograph5722', [], 'Display', None],
  [ 0x5723, 'CjkUnifiedIdeograph5723', [], 'Display', None],
  [ 0x5724, 'CjkUnifiedIdeograph5724', [], 'Display', None],
  [ 0x5725, 'CjkUnifiedIdeograph5725', [], 'Display', None],
  [ 0x5726, 'CjkUnifiedIdeograph5726', [], 'Display', None],
  [ 0x5727, 'CjkUnifiedIdeograph5727', [], 'Display', None],
  [ 0x5728, 'CjkUnifiedIdeograph5728', [], 'Display', None],
  [ 0x5729, 'CjkUnifiedIdeograph5729', [], 'Display', None],
  [ 0x572a, 'CjkUnifiedIdeograph572a', [], 'Display', None],
  [ 0x572b, 'CjkUnifiedIdeograph572b', [], 'Display', None],
  [ 0x572c, 'CjkUnifiedIdeograph572c', [], 'Display', None],
  [ 0x572d, 'CjkUnifiedIdeograph572d', [], 'Display', None],
  [ 0x572e, 'CjkUnifiedIdeograph572e', [], 'Display', None],
  [ 0x572f, 'CjkUnifiedIdeograph572f', [], 'Display', None],
  [ 0x5730, 'CjkUnifiedIdeograph5730', [], 'Display', None],
  [ 0x5731, 'CjkUnifiedIdeograph5731', [], 'Display', None],
  [ 0x5732, 'CjkUnifiedIdeograph5732', [], 'Display', None],
  [ 0x5733, 'CjkUnifiedIdeograph5733', [], 'Display', None],
  [ 0x5734, 'CjkUnifiedIdeograph5734', [], 'Display', None],
  [ 0x5735, 'CjkUnifiedIdeograph5735', [], 'Display', None],
  [ 0x5736, 'CjkUnifiedIdeograph5736', [], 'Display', None],
  [ 0x5737, 'CjkUnifiedIdeograph5737', [], 'Display', None],
  [ 0x5738, 'CjkUnifiedIdeograph5738', [], 'Display', None],
  [ 0x5739, 'CjkUnifiedIdeograph5739', [], 'Display', None],
  [ 0x573a, 'CjkUnifiedIdeograph573a', [], 'Display', None],
  [ 0x573b, 'CjkUnifiedIdeograph573b', [], 'Display', None],
  [ 0x573c, 'CjkUnifiedIdeograph573c', [], 'Display', None],
  [ 0x573d, 'CjkUnifiedIdeograph573d', [], 'Display', None],
  [ 0x573e, 'CjkUnifiedIdeograph573e', [], 'Display', None],
  [ 0x573f, 'CjkUnifiedIdeograph573f', [], 'Display', None],
  [ 0x5740, 'CjkUnifiedIdeograph5740', [], 'Display', None],
  [ 0x5741, 'CjkUnifiedIdeograph5741', [], 'Display', None],
  [ 0x5742, 'CjkUnifiedIdeograph5742', [], 'Display', None],
  [ 0x5743, 'CjkUnifiedIdeograph5743', [], 'Display', None],
  [ 0x5744, 'CjkUnifiedIdeograph5744', [], 'Display', None],
  [ 0x5745, 'CjkUnifiedIdeograph5745', [], 'Display', None],
  [ 0x5746, 'CjkUnifiedIdeograph5746', [], 'Display', None],
  [ 0x5747, 'CjkUnifiedIdeograph5747', [], 'Display', None],
  [ 0x5748, 'CjkUnifiedIdeograph5748', [], 'Display', None],
  [ 0x5749, 'CjkUnifiedIdeograph5749', [], 'Display', None],
  [ 0x574a, 'CjkUnifiedIdeograph574a', [], 'Display', None],
  [ 0x574b, 'CjkUnifiedIdeograph574b', [], 'Display', None],
  [ 0x574c, 'CjkUnifiedIdeograph574c', [], 'Display', None],
  [ 0x574d, 'CjkUnifiedIdeograph574d', [], 'Display', None],
  [ 0x574e, 'CjkUnifiedIdeograph574e', [], 'Display', None],
  [ 0x574f, 'CjkUnifiedIdeograph574f', [], 'Display', None],
  [ 0x5750, 'CjkUnifiedIdeograph5750', [], 'Display', None],
  [ 0x5751, 'CjkUnifiedIdeograph5751', [], 'Display', None],
  [ 0x5752, 'CjkUnifiedIdeograph5752', [], 'Display', None],
  [ 0x5753, 'CjkUnifiedIdeograph5753', [], 'Display', None],
  [ 0x5754, 'CjkUnifiedIdeograph5754', [], 'Display', None],
  [ 0x5755, 'CjkUnifiedIdeograph5755', [], 'Display', None],
  [ 0x5756, 'CjkUnifiedIdeograph5756', [], 'Display', None],
  [ 0x5757, 'CjkUnifiedIdeograph5757', [], 'Display', None],
  [ 0x5758, 'CjkUnifiedIdeograph5758', [], 'Display', None],
  [ 0x5759, 'CjkUnifiedIdeograph5759', [], 'Display', None],
  [ 0x575a, 'CjkUnifiedIdeograph575a', [], 'Display', None],
  [ 0x575b, 'CjkUnifiedIdeograph575b', [], 'Display', None],
  [ 0x575c, 'CjkUnifiedIdeograph575c', [], 'Display', None],
  [ 0x575d, 'CjkUnifiedIdeograph575d', [], 'Display', None],
  [ 0x575e, 'CjkUnifiedIdeograph575e', [], 'Display', None],
  [ 0x575f, 'CjkUnifiedIdeograph575f', [], 'Display', None],
  [ 0x5760, 'CjkUnifiedIdeograph5760', [], 'Display', None],
  [ 0x5761, 'CjkUnifiedIdeograph5761', [], 'Display', None],
  [ 0x5762, 'CjkUnifiedIdeograph5762', [], 'Display', None],
  [ 0x5763, 'CjkUnifiedIdeograph5763', [], 'Display', None],
  [ 0x5764, 'CjkUnifiedIdeograph5764', [], 'Display', None],
  [ 0x5765, 'CjkUnifiedIdeograph5765', [], 'Display', None],
  [ 0x5766, 'CjkUnifiedIdeograph5766', [], 'Display', None],
  [ 0x5767, 'CjkUnifiedIdeograph5767', [], 'Display', None],
  [ 0x5768, 'CjkUnifiedIdeograph5768', [], 'Display', None],
  [ 0x5769, 'CjkUnifiedIdeograph5769', [], 'Display', None],
  [ 0x576a, 'CjkUnifiedIdeograph576a', [], 'Display', None],
  [ 0x576b, 'CjkUnifiedIdeograph576b', [], 'Display', None],
  [ 0x576c, 'CjkUnifiedIdeograph576c', [], 'Display', None],
  [ 0x576d, 'CjkUnifiedIdeograph576d', [], 'Display', None],
  [ 0x576e, 'CjkUnifiedIdeograph576e', [], 'Display', None],
  [ 0x576f, 'CjkUnifiedIdeograph576f', [], 'Display', None],
  [ 0x5770, 'CjkUnifiedIdeograph5770', [], 'Display', None],
  [ 0x5771, 'CjkUnifiedIdeograph5771', [], 'Display', None],
  [ 0x5772, 'CjkUnifiedIdeograph5772', [], 'Display', None],
  [ 0x5773, 'CjkUnifiedIdeograph5773', [], 'Display', None],
  [ 0x5774, 'CjkUnifiedIdeograph5774', [], 'Display', None],
  [ 0x5775, 'CjkUnifiedIdeograph5775', [], 'Display', None],
  [ 0x5776, 'CjkUnifiedIdeograph5776', [], 'Display', None],
  [ 0x5777, 'CjkUnifiedIdeograph5777', [], 'Display', None],
  [ 0x5778, 'CjkUnifiedIdeograph5778', [], 'Display', None],
  [ 0x5779, 'CjkUnifiedIdeograph5779', [], 'Display', None],
  [ 0x577a, 'CjkUnifiedIdeograph577a', [], 'Display', None],
  [ 0x577b, 'CjkUnifiedIdeograph577b', [], 'Display', None],
  [ 0x577c, 'CjkUnifiedIdeograph577c', [], 'Display', None],
  [ 0x577d, 'CjkUnifiedIdeograph577d', [], 'Display', None],
  [ 0x577e, 'CjkUnifiedIdeograph577e', [], 'Display', None],
  [ 0x577f, 'CjkUnifiedIdeograph577f', [], 'Display', None],
  [ 0x5780, 'CjkUnifiedIdeograph5780', [], 'Display', None],
  [ 0x5781, 'CjkUnifiedIdeograph5781', [], 'Display', None],
  [ 0x5782, 'CjkUnifiedIdeograph5782', [], 'Display', None],
  [ 0x5783, 'CjkUnifiedIdeograph5783', [], 'Display', None],
  [ 0x5784, 'CjkUnifiedIdeograph5784', [], 'Display', None],
  [ 0x5785, 'CjkUnifiedIdeograph5785', [], 'Display', None],
  [ 0x5786, 'CjkUnifiedIdeograph5786', [], 'Display', None],
  [ 0x5787, 'CjkUnifiedIdeograph5787', [], 'Display', None],
  [ 0x5788, 'CjkUnifiedIdeograph5788', [], 'Display', None],
  [ 0x5789, 'CjkUnifiedIdeograph5789', [], 'Display', None],
  [ 0x578a, 'CjkUnifiedIdeograph578a', [], 'Display', None],
  [ 0x578b, 'CjkUnifiedIdeograph578b', [], 'Display', None],
  [ 0x578c, 'CjkUnifiedIdeograph578c', [], 'Display', None],
  [ 0x578d, 'CjkUnifiedIdeograph578d', [], 'Display', None],
  [ 0x578e, 'CjkUnifiedIdeograph578e', [], 'Display', None],
  [ 0x578f, 'CjkUnifiedIdeograph578f', [], 'Display', None],
  [ 0x5790, 'CjkUnifiedIdeograph5790', [], 'Display', None],
  [ 0x5791, 'CjkUnifiedIdeograph5791', [], 'Display', None],
  [ 0x5792, 'CjkUnifiedIdeograph5792', [], 'Display', None],
  [ 0x5793, 'CjkUnifiedIdeograph5793', [], 'Display', None],
  [ 0x5794, 'CjkUnifiedIdeograph5794', [], 'Display', None],
  [ 0x5795, 'CjkUnifiedIdeograph5795', [], 'Display', None],
  [ 0x5796, 'CjkUnifiedIdeograph5796', [], 'Display', None],
  [ 0x5797, 'CjkUnifiedIdeograph5797', [], 'Display', None],
  [ 0x5798, 'CjkUnifiedIdeograph5798', [], 'Display', None],
  [ 0x5799, 'CjkUnifiedIdeograph5799', [], 'Display', None],
  [ 0x579a, 'CjkUnifiedIdeograph579a', [], 'Display', None],
  [ 0x579b, 'CjkUnifiedIdeograph579b', [], 'Display', None],
  [ 0x579c, 'CjkUnifiedIdeograph579c', [], 'Display', None],
  [ 0x579d, 'CjkUnifiedIdeograph579d', [], 'Display', None],
  [ 0x579e, 'CjkUnifiedIdeograph579e', [], 'Display', None],
  [ 0x579f, 'CjkUnifiedIdeograph579f', [], 'Display', None],
  [ 0x57a0, 'CjkUnifiedIdeograph57a0', [], 'Display', None],
  [ 0x57a1, 'CjkUnifiedIdeograph57a1', [], 'Display', None],
  [ 0x57a2, 'CjkUnifiedIdeograph57a2', [], 'Display', None],
  [ 0x57a3, 'CjkUnifiedIdeograph57a3', [], 'Display', None],
  [ 0x57a4, 'CjkUnifiedIdeograph57a4', [], 'Display', None],
  [ 0x57a5, 'CjkUnifiedIdeograph57a5', [], 'Display', None],
  [ 0x57a6, 'CjkUnifiedIdeograph57a6', [], 'Display', None],
  [ 0x57a7, 'CjkUnifiedIdeograph57a7', [], 'Display', None],
  [ 0x57a8, 'CjkUnifiedIdeograph57a8', [], 'Display', None],
  [ 0x57a9, 'CjkUnifiedIdeograph57a9', [], 'Display', None],
  [ 0x57aa, 'CjkUnifiedIdeograph57aa', [], 'Display', None],
  [ 0x57ab, 'CjkUnifiedIdeograph57ab', [], 'Display', None],
  [ 0x57ac, 'CjkUnifiedIdeograph57ac', [], 'Display', None],
  [ 0x57ad, 'CjkUnifiedIdeograph57ad', [], 'Display', None],
  [ 0x57ae, 'CjkUnifiedIdeograph57ae', [], 'Display', None],
  [ 0x57af, 'CjkUnifiedIdeograph57af', [], 'Display', None],
  [ 0x57b0, 'CjkUnifiedIdeograph57b0', [], 'Display', None],
  [ 0x57b1, 'CjkUnifiedIdeograph57b1', [], 'Display', None],
  [ 0x57b2, 'CjkUnifiedIdeograph57b2', [], 'Display', None],
  [ 0x57b3, 'CjkUnifiedIdeograph57b3', [], 'Display', None],
  [ 0x57b4, 'CjkUnifiedIdeograph57b4', [], 'Display', None],
  [ 0x57b5, 'CjkUnifiedIdeograph57b5', [], 'Display', None],
  [ 0x57b6, 'CjkUnifiedIdeograph57b6', [], 'Display', None],
  [ 0x57b7, 'CjkUnifiedIdeograph57b7', [], 'Display', None],
  [ 0x57b8, 'CjkUnifiedIdeograph57b8', [], 'Display', None],
  [ 0x57b9, 'CjkUnifiedIdeograph57b9', [], 'Display', None],
  [ 0x57ba, 'CjkUnifiedIdeograph57ba', [], 'Display', None],
  [ 0x57bb, 'CjkUnifiedIdeograph57bb', [], 'Display', None],
  [ 0x57bc, 'CjkUnifiedIdeograph57bc', [], 'Display', None],
  [ 0x57bd, 'CjkUnifiedIdeograph57bd', [], 'Display', None],
  [ 0x57be, 'CjkUnifiedIdeograph57be', [], 'Display', None],
  [ 0x57bf, 'CjkUnifiedIdeograph57bf', [], 'Display', None],
  [ 0x57c0, 'CjkUnifiedIdeograph57c0', [], 'Display', None],
  [ 0x57c1, 'CjkUnifiedIdeograph57c1', [], 'Display', None],
  [ 0x57c2, 'CjkUnifiedIdeograph57c2', [], 'Display', None],
  [ 0x57c3, 'CjkUnifiedIdeograph57c3', [], 'Display', None],
  [ 0x57c4, 'CjkUnifiedIdeograph57c4', [], 'Display', None],
  [ 0x57c5, 'CjkUnifiedIdeograph57c5', [], 'Display', None],
  [ 0x57c6, 'CjkUnifiedIdeograph57c6', [], 'Display', None],
  [ 0x57c7, 'CjkUnifiedIdeograph57c7', [], 'Display', None],
  [ 0x57c8, 'CjkUnifiedIdeograph57c8', [], 'Display', None],
  [ 0x57c9, 'CjkUnifiedIdeograph57c9', [], 'Display', None],
  [ 0x57ca, 'CjkUnifiedIdeograph57ca', [], 'Display', None],
  [ 0x57cb, 'CjkUnifiedIdeograph57cb', [], 'Display', None],
  [ 0x57cc, 'CjkUnifiedIdeograph57cc', [], 'Display', None],
  [ 0x57cd, 'CjkUnifiedIdeograph57cd', [], 'Display', None],
  [ 0x57ce, 'CjkUnifiedIdeograph57ce', [], 'Display', None],
  [ 0x57cf, 'CjkUnifiedIdeograph57cf', [], 'Display', None],
  [ 0x57d0, 'CjkUnifiedIdeograph57d0', [], 'Display', None],
  [ 0x57d1, 'CjkUnifiedIdeograph57d1', [], 'Display', None],
  [ 0x57d2, 'CjkUnifiedIdeograph57d2', [], 'Display', None],
  [ 0x57d3, 'CjkUnifiedIdeograph57d3', [], 'Display', None],
  [ 0x57d4, 'CjkUnifiedIdeograph57d4', [], 'Display', None],
  [ 0x57d5, 'CjkUnifiedIdeograph57d5', [], 'Display', None],
  [ 0x57d6, 'CjkUnifiedIdeograph57d6', [], 'Display', None],
  [ 0x57d7, 'CjkUnifiedIdeograph57d7', [], 'Display', None],
  [ 0x57d8, 'CjkUnifiedIdeograph57d8', [], 'Display', None],
  [ 0x57d9, 'CjkUnifiedIdeograph57d9', [], 'Display', None],
  [ 0x57da, 'CjkUnifiedIdeograph57da', [], 'Display', None],
  [ 0x57db, 'CjkUnifiedIdeograph57db', [], 'Display', None],
  [ 0x57dc, 'CjkUnifiedIdeograph57dc', [], 'Display', None],
  [ 0x57dd, 'CjkUnifiedIdeograph57dd', [], 'Display', None],
  [ 0x57de, 'CjkUnifiedIdeograph57de', [], 'Display', None],
  [ 0x57df, 'CjkUnifiedIdeograph57df', [], 'Display', None],
  [ 0x57e0, 'CjkUnifiedIdeograph57e0', [], 'Display', None],
  [ 0x57e1, 'CjkUnifiedIdeograph57e1', [], 'Display', None],
  [ 0x57e2, 'CjkUnifiedIdeograph57e2', [], 'Display', None],
  [ 0x57e3, 'CjkUnifiedIdeograph57e3', [], 'Display', None],
  [ 0x57e4, 'CjkUnifiedIdeograph57e4', [], 'Display', None],
  [ 0x57e5, 'CjkUnifiedIdeograph57e5', [], 'Display', None],
  [ 0x57e6, 'CjkUnifiedIdeograph57e6', [], 'Display', None],
  [ 0x57e7, 'CjkUnifiedIdeograph57e7', [], 'Display', None],
  [ 0x57e8, 'CjkUnifiedIdeograph57e8', [], 'Display', None],
  [ 0x57e9, 'CjkUnifiedIdeograph57e9', [], 'Display', None],
  [ 0x57ea, 'CjkUnifiedIdeograph57ea', [], 'Display', None],
  [ 0x57eb, 'CjkUnifiedIdeograph57eb', [], 'Display', None],
  [ 0x57ec, 'CjkUnifiedIdeograph57ec', [], 'Display', None],
  [ 0x57ed, 'CjkUnifiedIdeograph57ed', [], 'Display', None],
  [ 0x57ee, 'CjkUnifiedIdeograph57ee', [], 'Display', None],
  [ 0x57ef, 'CjkUnifiedIdeograph57ef', [], 'Display', None],
  [ 0x57f0, 'CjkUnifiedIdeograph57f0', [], 'Display', None],
  [ 0x57f1, 'CjkUnifiedIdeograph57f1', [], 'Display', None],
  [ 0x57f2, 'CjkUnifiedIdeograph57f2', [], 'Display', None],
  [ 0x57f3, 'CjkUnifiedIdeograph57f3', [], 'Display', None],
  [ 0x57f4, 'CjkUnifiedIdeograph57f4', [], 'Display', None],
  [ 0x57f5, 'CjkUnifiedIdeograph57f5', [], 'Display', None],
  [ 0x57f6, 'CjkUnifiedIdeograph57f6', [], 'Display', None],
  [ 0x57f7, 'CjkUnifiedIdeograph57f7', [], 'Display', None],
  [ 0x57f8, 'CjkUnifiedIdeograph57f8', [], 'Display', None],
  [ 0x57f9, 'CjkUnifiedIdeograph57f9', [], 'Display', None],
  [ 0x57fa, 'CjkUnifiedIdeograph57fa', [], 'Display', None],
  [ 0x57fb, 'CjkUnifiedIdeograph57fb', [], 'Display', None],
  [ 0x57fc, 'CjkUnifiedIdeograph57fc', [], 'Display', None],
  [ 0x57fd, 'CjkUnifiedIdeograph57fd', [], 'Display', None],
  [ 0x57fe, 'CjkUnifiedIdeograph57fe', [], 'Display', None],
  [ 0x57ff, 'CjkUnifiedIdeograph57ff', [], 'Display', None],
  [ 0x5800, 'CjkUnifiedIdeograph5800', [], 'Display', None],
  [ 0x5801, 'CjkUnifiedIdeograph5801', [], 'Display', None],
  [ 0x5802, 'CjkUnifiedIdeograph5802', [], 'Display', None],
  [ 0x5803, 'CjkUnifiedIdeograph5803', [], 'Display', None],
  [ 0x5804, 'CjkUnifiedIdeograph5804', [], 'Display', None],
  [ 0x5805, 'CjkUnifiedIdeograph5805', [], 'Display', None],
  [ 0x5806, 'CjkUnifiedIdeograph5806', [], 'Display', None],
  [ 0x5807, 'CjkUnifiedIdeograph5807', [], 'Display', None],
  [ 0x5808, 'CjkUnifiedIdeograph5808', [], 'Display', None],
  [ 0x5809, 'CjkUnifiedIdeograph5809', [], 'Display', None],
  [ 0x580a, 'CjkUnifiedIdeograph580a', [], 'Display', None],
  [ 0x580b, 'CjkUnifiedIdeograph580b', [], 'Display', None],
  [ 0x580c, 'CjkUnifiedIdeograph580c', [], 'Display', None],
  [ 0x580d, 'CjkUnifiedIdeograph580d', [], 'Display', None],
  [ 0x580e, 'CjkUnifiedIdeograph580e', [], 'Display', None],
  [ 0x580f, 'CjkUnifiedIdeograph580f', [], 'Display', None],
  [ 0x5810, 'CjkUnifiedIdeograph5810', [], 'Display', None],
  [ 0x5811, 'CjkUnifiedIdeograph5811', [], 'Display', None],
  [ 0x5812, 'CjkUnifiedIdeograph5812', [], 'Display', None],
  [ 0x5813, 'CjkUnifiedIdeograph5813', [], 'Display', None],
  [ 0x5814, 'CjkUnifiedIdeograph5814', [], 'Display', None],
  [ 0x5815, 'CjkUnifiedIdeograph5815', [], 'Display', None],
  [ 0x5816, 'CjkUnifiedIdeograph5816', [], 'Display', None],
  [ 0x5817, 'CjkUnifiedIdeograph5817', [], 'Display', None],
  [ 0x5818, 'CjkUnifiedIdeograph5818', [], 'Display', None],
  [ 0x5819, 'CjkUnifiedIdeograph5819', [], 'Display', None],
  [ 0x581a, 'CjkUnifiedIdeograph581a', [], 'Display', None],
  [ 0x581b, 'CjkUnifiedIdeograph581b', [], 'Display', None],
  [ 0x581c, 'CjkUnifiedIdeograph581c', [], 'Display', None],
  [ 0x581d, 'CjkUnifiedIdeograph581d', [], 'Display', None],
  [ 0x581e, 'CjkUnifiedIdeograph581e', [], 'Display', None],
  [ 0x581f, 'CjkUnifiedIdeograph581f', [], 'Display', None],
  [ 0x5820, 'CjkUnifiedIdeograph5820', [], 'Display', None],
  [ 0x5821, 'CjkUnifiedIdeograph5821', [], 'Display', None],
  [ 0x5822, 'CjkUnifiedIdeograph5822', [], 'Display', None],
  [ 0x5823, 'CjkUnifiedIdeograph5823', [], 'Display', None],
  [ 0x5824, 'CjkUnifiedIdeograph5824', [], 'Display', None],
  [ 0x5825, 'CjkUnifiedIdeograph5825', [], 'Display', None],
  [ 0x5826, 'CjkUnifiedIdeograph5826', [], 'Display', None],
  [ 0x5827, 'CjkUnifiedIdeograph5827', [], 'Display', None],
  [ 0x5828, 'CjkUnifiedIdeograph5828', [], 'Display', None],
  [ 0x5829, 'CjkUnifiedIdeograph5829', [], 'Display', None],
  [ 0x582a, 'CjkUnifiedIdeograph582a', [], 'Display', None],
  [ 0x582b, 'CjkUnifiedIdeograph582b', [], 'Display', None],
  [ 0x582c, 'CjkUnifiedIdeograph582c', [], 'Display', None],
  [ 0x582d, 'CjkUnifiedIdeograph582d', [], 'Display', None],
  [ 0x582e, 'CjkUnifiedIdeograph582e', [], 'Display', None],
  [ 0x582f, 'CjkUnifiedIdeograph582f', [], 'Display', None],
  [ 0x5830, 'CjkUnifiedIdeograph5830', [], 'Display', None],
  [ 0x5831, 'CjkUnifiedIdeograph5831', [], 'Display', None],
  [ 0x5832, 'CjkUnifiedIdeograph5832', [], 'Display', None],
  [ 0x5833, 'CjkUnifiedIdeograph5833', [], 'Display', None],
  [ 0x5834, 'CjkUnifiedIdeograph5834', [], 'Display', None],
  [ 0x5835, 'CjkUnifiedIdeograph5835', [], 'Display', None],
  [ 0x5836, 'CjkUnifiedIdeograph5836', [], 'Display', None],
  [ 0x5837, 'CjkUnifiedIdeograph5837', [], 'Display', None],
  [ 0x5838, 'CjkUnifiedIdeograph5838', [], 'Display', None],
  [ 0x5839, 'CjkUnifiedIdeograph5839', [], 'Display', None],
  [ 0x583a, 'CjkUnifiedIdeograph583a', [], 'Display', None],
  [ 0x583b, 'CjkUnifiedIdeograph583b', [], 'Display', None],
  [ 0x583c, 'CjkUnifiedIdeograph583c', [], 'Display', None],
  [ 0x583d, 'CjkUnifiedIdeograph583d', [], 'Display', None],
  [ 0x583e, 'CjkUnifiedIdeograph583e', [], 'Display', None],
  [ 0x583f, 'CjkUnifiedIdeograph583f', [], 'Display', None],
  [ 0x5840, 'CjkUnifiedIdeograph5840', [], 'Display', None],
  [ 0x5841, 'CjkUnifiedIdeograph5841', [], 'Display', None],
  [ 0x5842, 'CjkUnifiedIdeograph5842', [], 'Display', None],
  [ 0x5843, 'CjkUnifiedIdeograph5843', [], 'Display', None],
  [ 0x5844, 'CjkUnifiedIdeograph5844', [], 'Display', None],
  [ 0x5845, 'CjkUnifiedIdeograph5845', [], 'Display', None],
  [ 0x5846, 'CjkUnifiedIdeograph5846', [], 'Display', None],
  [ 0x5847, 'CjkUnifiedIdeograph5847', [], 'Display', None],
  [ 0x5848, 'CjkUnifiedIdeograph5848', [], 'Display', None],
  [ 0x5849, 'CjkUnifiedIdeograph5849', [], 'Display', None],
  [ 0x584a, 'CjkUnifiedIdeograph584a', [], 'Display', None],
  [ 0x584b, 'CjkUnifiedIdeograph584b', [], 'Display', None],
  [ 0x584c, 'CjkUnifiedIdeograph584c', [], 'Display', None],
  [ 0x584d, 'CjkUnifiedIdeograph584d', [], 'Display', None],
  [ 0x584e, 'CjkUnifiedIdeograph584e', [], 'Display', None],
  [ 0x584f, 'CjkUnifiedIdeograph584f', [], 'Display', None],
  [ 0x5850, 'CjkUnifiedIdeograph5850', [], 'Display', None],
  [ 0x5851, 'CjkUnifiedIdeograph5851', [], 'Display', None],
  [ 0x5852, 'CjkUnifiedIdeograph5852', [], 'Display', None],
  [ 0x5853, 'CjkUnifiedIdeograph5853', [], 'Display', None],
  [ 0x5854, 'CjkUnifiedIdeograph5854', [], 'Display', None],
  [ 0x5855, 'CjkUnifiedIdeograph5855', [], 'Display', None],
  [ 0x5856, 'CjkUnifiedIdeograph5856', [], 'Display', None],
  [ 0x5857, 'CjkUnifiedIdeograph5857', [], 'Display', None],
  [ 0x5858, 'CjkUnifiedIdeograph5858', [], 'Display', None],
  [ 0x5859, 'CjkUnifiedIdeograph5859', [], 'Display', None],
  [ 0x585a, 'CjkUnifiedIdeograph585a', [], 'Display', None],
  [ 0x585b, 'CjkUnifiedIdeograph585b', [], 'Display', None],
  [ 0x585c, 'CjkUnifiedIdeograph585c', [], 'Display', None],
  [ 0x585d, 'CjkUnifiedIdeograph585d', [], 'Display', None],
  [ 0x585e, 'CjkUnifiedIdeograph585e', [], 'Display', None],
  [ 0x585f, 'CjkUnifiedIdeograph585f', [], 'Display', None],
  [ 0x5860, 'CjkUnifiedIdeograph5860', [], 'Display', None],
  [ 0x5861, 'CjkUnifiedIdeograph5861', [], 'Display', None],
  [ 0x5862, 'CjkUnifiedIdeograph5862', [], 'Display', None],
  [ 0x5863, 'CjkUnifiedIdeograph5863', [], 'Display', None],
  [ 0x5864, 'CjkUnifiedIdeograph5864', [], 'Display', None],
  [ 0x5865, 'CjkUnifiedIdeograph5865', [], 'Display', None],
  [ 0x5866, 'CjkUnifiedIdeograph5866', [], 'Display', None],
  [ 0x5867, 'CjkUnifiedIdeograph5867', [], 'Display', None],
  [ 0x5868, 'CjkUnifiedIdeograph5868', [], 'Display', None],
  [ 0x5869, 'CjkUnifiedIdeograph5869', [], 'Display', None],
  [ 0x586a, 'CjkUnifiedIdeograph586a', [], 'Display', None],
  [ 0x586b, 'CjkUnifiedIdeograph586b', [], 'Display', None],
  [ 0x586c, 'CjkUnifiedIdeograph586c', [], 'Display', None],
  [ 0x586d, 'CjkUnifiedIdeograph586d', [], 'Display', None],
  [ 0x586e, 'CjkUnifiedIdeograph586e', [], 'Display', None],
  [ 0x586f, 'CjkUnifiedIdeograph586f', [], 'Display', None],
  [ 0x5870, 'CjkUnifiedIdeograph5870', [], 'Display', None],
  [ 0x5871, 'CjkUnifiedIdeograph5871', [], 'Display', None],
  [ 0x5872, 'CjkUnifiedIdeograph5872', [], 'Display', None],
  [ 0x5873, 'CjkUnifiedIdeograph5873', [], 'Display', None],
  [ 0x5874, 'CjkUnifiedIdeograph5874', [], 'Display', None],
  [ 0x5875, 'CjkUnifiedIdeograph5875', [], 'Display', None],
  [ 0x5876, 'CjkUnifiedIdeograph5876', [], 'Display', None],
  [ 0x5877, 'CjkUnifiedIdeograph5877', [], 'Display', None],
  [ 0x5878, 'CjkUnifiedIdeograph5878', [], 'Display', None],
  [ 0x5879, 'CjkUnifiedIdeograph5879', [], 'Display', None],
  [ 0x587a, 'CjkUnifiedIdeograph587a', [], 'Display', None],
  [ 0x587b, 'CjkUnifiedIdeograph587b', [], 'Display', None],
  [ 0x587c, 'CjkUnifiedIdeograph587c', [], 'Display', None],
  [ 0x587d, 'CjkUnifiedIdeograph587d', [], 'Display', None],
  [ 0x587e, 'CjkUnifiedIdeograph587e', [], 'Display', None],
  [ 0x587f, 'CjkUnifiedIdeograph587f', [], 'Display', None],
  [ 0x5880, 'CjkUnifiedIdeograph5880', [], 'Display', None],
  [ 0x5881, 'CjkUnifiedIdeograph5881', [], 'Display', None],
  [ 0x5882, 'CjkUnifiedIdeograph5882', [], 'Display', None],
  [ 0x5883, 'CjkUnifiedIdeograph5883', [], 'Display', None],
  [ 0x5884, 'CjkUnifiedIdeograph5884', [], 'Display', None],
  [ 0x5885, 'CjkUnifiedIdeograph5885', [], 'Display', None],
  [ 0x5886, 'CjkUnifiedIdeograph5886', [], 'Display', None],
  [ 0x5887, 'CjkUnifiedIdeograph5887', [], 'Display', None],
  [ 0x5888, 'CjkUnifiedIdeograph5888', [], 'Display', None],
  [ 0x5889, 'CjkUnifiedIdeograph5889', [], 'Display', None],
  [ 0x588a, 'CjkUnifiedIdeograph588a', [], 'Display', None],
  [ 0x588b, 'CjkUnifiedIdeograph588b', [], 'Display', None],
  [ 0x588c, 'CjkUnifiedIdeograph588c', [], 'Display', None],
  [ 0x588d, 'CjkUnifiedIdeograph588d', [], 'Display', None],
  [ 0x588e, 'CjkUnifiedIdeograph588e', [], 'Display', None],
  [ 0x588f, 'CjkUnifiedIdeograph588f', [], 'Display', None],
  [ 0x5890, 'CjkUnifiedIdeograph5890', [], 'Display', None],
  [ 0x5891, 'CjkUnifiedIdeograph5891', [], 'Display', None],
  [ 0x5892, 'CjkUnifiedIdeograph5892', [], 'Display', None],
  [ 0x5893, 'CjkUnifiedIdeograph5893', [], 'Display', None],
  [ 0x5894, 'CjkUnifiedIdeograph5894', [], 'Display', None],
  [ 0x5895, 'CjkUnifiedIdeograph5895', [], 'Display', None],
  [ 0x5896, 'CjkUnifiedIdeograph5896', [], 'Display', None],
  [ 0x5897, 'CjkUnifiedIdeograph5897', [], 'Display', None],
  [ 0x5898, 'CjkUnifiedIdeograph5898', [], 'Display', None],
  [ 0x5899, 'CjkUnifiedIdeograph5899', [], 'Display', None],
  [ 0x589a, 'CjkUnifiedIdeograph589a', [], 'Display', None],
  [ 0x589b, 'CjkUnifiedIdeograph589b', [], 'Display', None],
  [ 0x589c, 'CjkUnifiedIdeograph589c', [], 'Display', None],
  [ 0x589d, 'CjkUnifiedIdeograph589d', [], 'Display', None],
  [ 0x589e, 'CjkUnifiedIdeograph589e', [], 'Display', None],
  [ 0x589f, 'CjkUnifiedIdeograph589f', [], 'Display', None],
  [ 0x58a0, 'CjkUnifiedIdeograph58a0', [], 'Display', None],
  [ 0x58a1, 'CjkUnifiedIdeograph58a1', [], 'Display', None],
  [ 0x58a2, 'CjkUnifiedIdeograph58a2', [], 'Display', None],
  [ 0x58a3, 'CjkUnifiedIdeograph58a3', [], 'Display', None],
  [ 0x58a4, 'CjkUnifiedIdeograph58a4', [], 'Display', None],
  [ 0x58a5, 'CjkUnifiedIdeograph58a5', [], 'Display', None],
  [ 0x58a6, 'CjkUnifiedIdeograph58a6', [], 'Display', None],
  [ 0x58a7, 'CjkUnifiedIdeograph58a7', [], 'Display', None],
  [ 0x58a8, 'CjkUnifiedIdeograph58a8', [], 'Display', None],
  [ 0x58a9, 'CjkUnifiedIdeograph58a9', [], 'Display', None],
  [ 0x58aa, 'CjkUnifiedIdeograph58aa', [], 'Display', None],
  [ 0x58ab, 'CjkUnifiedIdeograph58ab', [], 'Display', None],
  [ 0x58ac, 'CjkUnifiedIdeograph58ac', [], 'Display', None],
  [ 0x58ad, 'CjkUnifiedIdeograph58ad', [], 'Display', None],
  [ 0x58ae, 'CjkUnifiedIdeograph58ae', [], 'Display', None],
  [ 0x58af, 'CjkUnifiedIdeograph58af', [], 'Display', None],
  [ 0x58b0, 'CjkUnifiedIdeograph58b0', [], 'Display', None],
  [ 0x58b1, 'CjkUnifiedIdeograph58b1', [], 'Display', None],
  [ 0x58b2, 'CjkUnifiedIdeograph58b2', [], 'Display', None],
  [ 0x58b3, 'CjkUnifiedIdeograph58b3', [], 'Display', None],
  [ 0x58b4, 'CjkUnifiedIdeograph58b4', [], 'Display', None],
  [ 0x58b5, 'CjkUnifiedIdeograph58b5', [], 'Display', None],
  [ 0x58b6, 'CjkUnifiedIdeograph58b6', [], 'Display', None],
  [ 0x58b7, 'CjkUnifiedIdeograph58b7', [], 'Display', None],
  [ 0x58b8, 'CjkUnifiedIdeograph58b8', [], 'Display', None],
  [ 0x58b9, 'CjkUnifiedIdeograph58b9', [], 'Display', None],
  [ 0x58ba, 'CjkUnifiedIdeograph58ba', [], 'Display', None],
  [ 0x58bb, 'CjkUnifiedIdeograph58bb', [], 'Display', None],
  [ 0x58bc, 'CjkUnifiedIdeograph58bc', [], 'Display', None],
  [ 0x58bd, 'CjkUnifiedIdeograph58bd', [], 'Display', None],
  [ 0x58be, 'CjkUnifiedIdeograph58be', [], 'Display', None],
  [ 0x58bf, 'CjkUnifiedIdeograph58bf', [], 'Display', None],
  [ 0x58c0, 'CjkUnifiedIdeograph58c0', [], 'Display', None],
  [ 0x58c1, 'CjkUnifiedIdeograph58c1', [], 'Display', None],
  [ 0x58c2, 'CjkUnifiedIdeograph58c2', [], 'Display', None],
  [ 0x58c3, 'CjkUnifiedIdeograph58c3', [], 'Display', None],
  [ 0x58c4, 'CjkUnifiedIdeograph58c4', [], 'Display', None],
  [ 0x58c5, 'CjkUnifiedIdeograph58c5', [], 'Display', None],
  [ 0x58c6, 'CjkUnifiedIdeograph58c6', [], 'Display', None],
  [ 0x58c7, 'CjkUnifiedIdeograph58c7', [], 'Display', None],
  [ 0x58c8, 'CjkUnifiedIdeograph58c8', [], 'Display', None],
  [ 0x58c9, 'CjkUnifiedIdeograph58c9', [], 'Display', None],
  [ 0x58ca, 'CjkUnifiedIdeograph58ca', [], 'Display', None],
  [ 0x58cb, 'CjkUnifiedIdeograph58cb', [], 'Display', None],
  [ 0x58cc, 'CjkUnifiedIdeograph58cc', [], 'Display', None],
  [ 0x58cd, 'CjkUnifiedIdeograph58cd', [], 'Display', None],
  [ 0x58ce, 'CjkUnifiedIdeograph58ce', [], 'Display', None],
  [ 0x58cf, 'CjkUnifiedIdeograph58cf', [], 'Display', None],
  [ 0x58d0, 'CjkUnifiedIdeograph58d0', [], 'Display', None],
  [ 0x58d1, 'CjkUnifiedIdeograph58d1', [], 'Display', None],
  [ 0x58d2, 'CjkUnifiedIdeograph58d2', [], 'Display', None],
  [ 0x58d3, 'CjkUnifiedIdeograph58d3', [], 'Display', None],
  [ 0x58d4, 'CjkUnifiedIdeograph58d4', [], 'Display', None],
  [ 0x58d5, 'CjkUnifiedIdeograph58d5', [], 'Display', None],
  [ 0x58d6, 'CjkUnifiedIdeograph58d6', [], 'Display', None],
  [ 0x58d7, 'CjkUnifiedIdeograph58d7', [], 'Display', None],
  [ 0x58d8, 'CjkUnifiedIdeograph58d8', [], 'Display', None],
  [ 0x58d9, 'CjkUnifiedIdeograph58d9', [], 'Display', None],
  [ 0x58da, 'CjkUnifiedIdeograph58da', [], 'Display', None],
  [ 0x58db, 'CjkUnifiedIdeograph58db', [], 'Display', None],
  [ 0x58dc, 'CjkUnifiedIdeograph58dc', [], 'Display', None],
  [ 0x58dd, 'CjkUnifiedIdeograph58dd', [], 'Display', None],
  [ 0x58de, 'CjkUnifiedIdeograph58de', [], 'Display', None],
  [ 0x58df, 'CjkUnifiedIdeograph58df', [], 'Display', None],
  [ 0x58e0, 'CjkUnifiedIdeograph58e0', [], 'Display', None],
  [ 0x58e1, 'CjkUnifiedIdeograph58e1', [], 'Display', None],
  [ 0x58e2, 'CjkUnifiedIdeograph58e2', [], 'Display', None],
  [ 0x58e3, 'CjkUnifiedIdeograph58e3', [], 'Display', None],
  [ 0x58e4, 'CjkUnifiedIdeograph58e4', [], 'Display', None],
  [ 0x58e5, 'CjkUnifiedIdeograph58e5', [], 'Display', None],
  [ 0x58e6, 'CjkUnifiedIdeograph58e6', [], 'Display', None],
  [ 0x58e7, 'CjkUnifiedIdeograph58e7', [], 'Display', None],
  [ 0x58e8, 'CjkUnifiedIdeograph58e8', [], 'Display', None],
  [ 0x58e9, 'CjkUnifiedIdeograph58e9', [], 'Display', None],
  [ 0x58ea, 'CjkUnifiedIdeograph58ea', [], 'Display', None],
  [ 0x58eb, 'CjkUnifiedIdeograph58eb', [], 'Display', None],
  [ 0x58ec, 'CjkUnifiedIdeograph58ec', [], 'Display', None],
  [ 0x58ed, 'CjkUnifiedIdeograph58ed', [], 'Display', None],
  [ 0x58ee, 'CjkUnifiedIdeograph58ee', [], 'Display', None],
  [ 0x58ef, 'CjkUnifiedIdeograph58ef', [], 'Display', None],
  [ 0x58f0, 'CjkUnifiedIdeograph58f0', [], 'Display', None],
  [ 0x58f1, 'CjkUnifiedIdeograph58f1', [], 'Display', None],
  [ 0x58f2, 'CjkUnifiedIdeograph58f2', [], 'Display', None],
  [ 0x58f3, 'CjkUnifiedIdeograph58f3', [], 'Display', None],
  [ 0x58f4, 'CjkUnifiedIdeograph58f4', [], 'Display', None],
  [ 0x58f5, 'CjkUnifiedIdeograph58f5', [], 'Display', None],
  [ 0x58f6, 'CjkUnifiedIdeograph58f6', [], 'Display', None],
  [ 0x58f7, 'CjkUnifiedIdeograph58f7', [], 'Display', None],
  [ 0x58f8, 'CjkUnifiedIdeograph58f8', [], 'Display', None],
  [ 0x58f9, 'CjkUnifiedIdeograph58f9', [], 'Display', None],
  [ 0x58fa, 'CjkUnifiedIdeograph58fa', [], 'Display', None],
  [ 0x58fb, 'CjkUnifiedIdeograph58fb', [], 'Display', None],
  [ 0x58fc, 'CjkUnifiedIdeograph58fc', [], 'Display', None],
  [ 0x58fd, 'CjkUnifiedIdeograph58fd', [], 'Display', None],
  [ 0x58fe, 'CjkUnifiedIdeograph58fe', [], 'Display', None],
  [ 0x58ff, 'CjkUnifiedIdeograph58ff', [], 'Display', None],
  [ 0x5900, 'CjkUnifiedIdeograph5900', [], 'Display', None],
  [ 0x5901, 'CjkUnifiedIdeograph5901', [], 'Display', None],
  [ 0x5902, 'CjkUnifiedIdeograph5902', [], 'Display', None],
  [ 0x5903, 'CjkUnifiedIdeograph5903', [], 'Display', None],
  [ 0x5904, 'CjkUnifiedIdeograph5904', [], 'Display', None],
  [ 0x5905, 'CjkUnifiedIdeograph5905', [], 'Display', None],
  [ 0x5906, 'CjkUnifiedIdeograph5906', [], 'Display', None],
  [ 0x5907, 'CjkUnifiedIdeograph5907', [], 'Display', None],
  [ 0x5908, 'CjkUnifiedIdeograph5908', [], 'Display', None],
  [ 0x5909, 'CjkUnifiedIdeograph5909', [], 'Display', None],
  [ 0x590a, 'CjkUnifiedIdeograph590a', [], 'Display', None],
  [ 0x590b, 'CjkUnifiedIdeograph590b', [], 'Display', None],
  [ 0x590c, 'CjkUnifiedIdeograph590c', [], 'Display', None],
  [ 0x590d, 'CjkUnifiedIdeograph590d', [], 'Display', None],
  [ 0x590e, 'CjkUnifiedIdeograph590e', [], 'Display', None],
  [ 0x590f, 'CjkUnifiedIdeograph590f', [], 'Display', None],
  [ 0x5910, 'CjkUnifiedIdeograph5910', [], 'Display', None],
  [ 0x5911, 'CjkUnifiedIdeograph5911', [], 'Display', None],
  [ 0x5912, 'CjkUnifiedIdeograph5912', [], 'Display', None],
  [ 0x5913, 'CjkUnifiedIdeograph5913', [], 'Display', None],
  [ 0x5914, 'CjkUnifiedIdeograph5914', [], 'Display', None],
  [ 0x5915, 'CjkUnifiedIdeograph5915', [], 'Display', None],
  [ 0x5916, 'CjkUnifiedIdeograph5916', [], 'Display', None],
  [ 0x5917, 'CjkUnifiedIdeograph5917', [], 'Display', None],
  [ 0x5918, 'CjkUnifiedIdeograph5918', [], 'Display', None],
  [ 0x5919, 'CjkUnifiedIdeograph5919', [], 'Display', None],
  [ 0x591a, 'CjkUnifiedIdeograph591a', [], 'Display', None],
  [ 0x591b, 'CjkUnifiedIdeograph591b', [], 'Display', None],
  [ 0x591c, 'CjkUnifiedIdeograph591c', [], 'Display', None],
  [ 0x591d, 'CjkUnifiedIdeograph591d', [], 'Display', None],
  [ 0x591e, 'CjkUnifiedIdeograph591e', [], 'Display', None],
  [ 0x591f, 'CjkUnifiedIdeograph591f', [], 'Display', None],
  [ 0x5920, 'CjkUnifiedIdeograph5920', [], 'Display', None],
  [ 0x5921, 'CjkUnifiedIdeograph5921', [], 'Display', None],
  [ 0x5922, 'CjkUnifiedIdeograph5922', [], 'Display', None],
  [ 0x5923, 'CjkUnifiedIdeograph5923', [], 'Display', None],
  [ 0x5924, 'CjkUnifiedIdeograph5924', [], 'Display', None],
  [ 0x5925, 'CjkUnifiedIdeograph5925', [], 'Display', None],
  [ 0x5926, 'CjkUnifiedIdeograph5926', [], 'Display', None],
  [ 0x5927, 'CjkUnifiedIdeograph5927', [], 'Display', None],
  [ 0x5928, 'CjkUnifiedIdeograph5928', [], 'Display', None],
  [ 0x5929, 'CjkUnifiedIdeograph5929', [], 'Display', None],
  [ 0x592a, 'CjkUnifiedIdeograph592a', [], 'Display', None],
  [ 0x592b, 'CjkUnifiedIdeograph592b', [], 'Display', None],
  [ 0x592c, 'CjkUnifiedIdeograph592c', [], 'Display', None],
  [ 0x592d, 'CjkUnifiedIdeograph592d', [], 'Display', None],
  [ 0x592e, 'CjkUnifiedIdeograph592e', [], 'Display', None],
  [ 0x592f, 'CjkUnifiedIdeograph592f', [], 'Display', None],
  [ 0x5930, 'CjkUnifiedIdeograph5930', [], 'Display', None],
  [ 0x5931, 'CjkUnifiedIdeograph5931', [], 'Display', None],
  [ 0x5932, 'CjkUnifiedIdeograph5932', [], 'Display', None],
  [ 0x5933, 'CjkUnifiedIdeograph5933', [], 'Display', None],
  [ 0x5934, 'CjkUnifiedIdeograph5934', [], 'Display', None],
  [ 0x5935, 'CjkUnifiedIdeograph5935', [], 'Display', None],
  [ 0x5936, 'CjkUnifiedIdeograph5936', [], 'Display', None],
  [ 0x5937, 'CjkUnifiedIdeograph5937', [], 'Display', None],
  [ 0x5938, 'CjkUnifiedIdeograph5938', [], 'Display', None],
  [ 0x5939, 'CjkUnifiedIdeograph5939', [], 'Display', None],
  [ 0x593a, 'CjkUnifiedIdeograph593a', [], 'Display', None],
  [ 0x593b, 'CjkUnifiedIdeograph593b', [], 'Display', None],
  [ 0x593c, 'CjkUnifiedIdeograph593c', [], 'Display', None],
  [ 0x593d, 'CjkUnifiedIdeograph593d', [], 'Display', None],
  [ 0x593e, 'CjkUnifiedIdeograph593e', [], 'Display', None],
  [ 0x593f, 'CjkUnifiedIdeograph593f', [], 'Display', None],
  [ 0x5940, 'CjkUnifiedIdeograph5940', [], 'Display', None],
  [ 0x5941, 'CjkUnifiedIdeograph5941', [], 'Display', None],
  [ 0x5942, 'CjkUnifiedIdeograph5942', [], 'Display', None],
  [ 0x5943, 'CjkUnifiedIdeograph5943', [], 'Display', None],
  [ 0x5944, 'CjkUnifiedIdeograph5944', [], 'Display', None],
  [ 0x5945, 'CjkUnifiedIdeograph5945', [], 'Display', None],
  [ 0x5946, 'CjkUnifiedIdeograph5946', [], 'Display', None],
  [ 0x5947, 'CjkUnifiedIdeograph5947', [], 'Display', None],
  [ 0x5948, 'CjkUnifiedIdeograph5948', [], 'Display', None],
  [ 0x5949, 'CjkUnifiedIdeograph5949', [], 'Display', None],
  [ 0x594a, 'CjkUnifiedIdeograph594a', [], 'Display', None],
  [ 0x594b, 'CjkUnifiedIdeograph594b', [], 'Display', None],
  [ 0x594c, 'CjkUnifiedIdeograph594c', [], 'Display', None],
  [ 0x594d, 'CjkUnifiedIdeograph594d', [], 'Display', None],
  [ 0x594e, 'CjkUnifiedIdeograph594e', [], 'Display', None],
  [ 0x594f, 'CjkUnifiedIdeograph594f', [], 'Display', None],
  [ 0x5950, 'CjkUnifiedIdeograph5950', [], 'Display', None],
  [ 0x5951, 'CjkUnifiedIdeograph5951', [], 'Display', None],
  [ 0x5952, 'CjkUnifiedIdeograph5952', [], 'Display', None],
  [ 0x5953, 'CjkUnifiedIdeograph5953', [], 'Display', None],
  [ 0x5954, 'CjkUnifiedIdeograph5954', [], 'Display', None],
  [ 0x5955, 'CjkUnifiedIdeograph5955', [], 'Display', None],
  [ 0x5956, 'CjkUnifiedIdeograph5956', [], 'Display', None],
  [ 0x5957, 'CjkUnifiedIdeograph5957', [], 'Display', None],
  [ 0x5958, 'CjkUnifiedIdeograph5958', [], 'Display', None],
  [ 0x5959, 'CjkUnifiedIdeograph5959', [], 'Display', None],
  [ 0x595a, 'CjkUnifiedIdeograph595a', [], 'Display', None],
  [ 0x595b, 'CjkUnifiedIdeograph595b', [], 'Display', None],
  [ 0x595c, 'CjkUnifiedIdeograph595c', [], 'Display', None],
  [ 0x595d, 'CjkUnifiedIdeograph595d', [], 'Display', None],
  [ 0x595e, 'CjkUnifiedIdeograph595e', [], 'Display', None],
  [ 0x595f, 'CjkUnifiedIdeograph595f', [], 'Display', None],
  [ 0x5960, 'CjkUnifiedIdeograph5960', [], 'Display', None],
  [ 0x5961, 'CjkUnifiedIdeograph5961', [], 'Display', None],
  [ 0x5962, 'CjkUnifiedIdeograph5962', [], 'Display', None],
  [ 0x5963, 'CjkUnifiedIdeograph5963', [], 'Display', None],
  [ 0x5964, 'CjkUnifiedIdeograph5964', [], 'Display', None],
  [ 0x5965, 'CjkUnifiedIdeograph5965', [], 'Display', None],
  [ 0x5966, 'CjkUnifiedIdeograph5966', [], 'Display', None],
  [ 0x5967, 'CjkUnifiedIdeograph5967', [], 'Display', None],
  [ 0x5968, 'CjkUnifiedIdeograph5968', [], 'Display', None],
  [ 0x5969, 'CjkUnifiedIdeograph5969', [], 'Display', None],
  [ 0x596a, 'CjkUnifiedIdeograph596a', [], 'Display', None],
  [ 0x596b, 'CjkUnifiedIdeograph596b', [], 'Display', None],
  [ 0x596c, 'CjkUnifiedIdeograph596c', [], 'Display', None],
  [ 0x596d, 'CjkUnifiedIdeograph596d', [], 'Display', None],
  [ 0x596e, 'CjkUnifiedIdeograph596e', [], 'Display', None],
  [ 0x596f, 'CjkUnifiedIdeograph596f', [], 'Display', None],
  [ 0x5970, 'CjkUnifiedIdeograph5970', [], 'Display', None],
  [ 0x5971, 'CjkUnifiedIdeograph5971', [], 'Display', None],
  [ 0x5972, 'CjkUnifiedIdeograph5972', [], 'Display', None],
  [ 0x5973, 'CjkUnifiedIdeograph5973', [], 'Display', None],
  [ 0x5974, 'CjkUnifiedIdeograph5974', [], 'Display', None],
  [ 0x5975, 'CjkUnifiedIdeograph5975', [], 'Display', None],
  [ 0x5976, 'CjkUnifiedIdeograph5976', [], 'Display', None],
  [ 0x5977, 'CjkUnifiedIdeograph5977', [], 'Display', None],
  [ 0x5978, 'CjkUnifiedIdeograph5978', [], 'Display', None],
  [ 0x5979, 'CjkUnifiedIdeograph5979', [], 'Display', None],
  [ 0x597a, 'CjkUnifiedIdeograph597a', [], 'Display', None],
  [ 0x597b, 'CjkUnifiedIdeograph597b', [], 'Display', None],
  [ 0x597c, 'CjkUnifiedIdeograph597c', [], 'Display', None],
  [ 0x597d, 'CjkUnifiedIdeograph597d', [], 'Display', None],
  [ 0x597e, 'CjkUnifiedIdeograph597e', [], 'Display', None],
  [ 0x597f, 'CjkUnifiedIdeograph597f', [], 'Display', None],
  [ 0x5980, 'CjkUnifiedIdeograph5980', [], 'Display', None],
  [ 0x5981, 'CjkUnifiedIdeograph5981', [], 'Display', None],
  [ 0x5982, 'CjkUnifiedIdeograph5982', [], 'Display', None],
  [ 0x5983, 'CjkUnifiedIdeograph5983', [], 'Display', None],
  [ 0x5984, 'CjkUnifiedIdeograph5984', [], 'Display', None],
  [ 0x5985, 'CjkUnifiedIdeograph5985', [], 'Display', None],
  [ 0x5986, 'CjkUnifiedIdeograph5986', [], 'Display', None],
  [ 0x5987, 'CjkUnifiedIdeograph5987', [], 'Display', None],
  [ 0x5988, 'CjkUnifiedIdeograph5988', [], 'Display', None],
  [ 0x5989, 'CjkUnifiedIdeograph5989', [], 'Display', None],
  [ 0x598a, 'CjkUnifiedIdeograph598a', [], 'Display', None],
  [ 0x598b, 'CjkUnifiedIdeograph598b', [], 'Display', None],
  [ 0x598c, 'CjkUnifiedIdeograph598c', [], 'Display', None],
  [ 0x598d, 'CjkUnifiedIdeograph598d', [], 'Display', None],
  [ 0x598e, 'CjkUnifiedIdeograph598e', [], 'Display', None],
  [ 0x598f, 'CjkUnifiedIdeograph598f', [], 'Display', None],
  [ 0x5990, 'CjkUnifiedIdeograph5990', [], 'Display', None],
  [ 0x5991, 'CjkUnifiedIdeograph5991', [], 'Display', None],
  [ 0x5992, 'CjkUnifiedIdeograph5992', [], 'Display', None],
  [ 0x5993, 'CjkUnifiedIdeograph5993', [], 'Display', None],
  [ 0x5994, 'CjkUnifiedIdeograph5994', [], 'Display', None],
  [ 0x5995, 'CjkUnifiedIdeograph5995', [], 'Display', None],
  [ 0x5996, 'CjkUnifiedIdeograph5996', [], 'Display', None],
  [ 0x5997, 'CjkUnifiedIdeograph5997', [], 'Display', None],
  [ 0x5998, 'CjkUnifiedIdeograph5998', [], 'Display', None],
  [ 0x5999, 'CjkUnifiedIdeograph5999', [], 'Display', None],
  [ 0x599a, 'CjkUnifiedIdeograph599a', [], 'Display', None],
  [ 0x599b, 'CjkUnifiedIdeograph599b', [], 'Display', None],
  [ 0x599c, 'CjkUnifiedIdeograph599c', [], 'Display', None],
  [ 0x599d, 'CjkUnifiedIdeograph599d', [], 'Display', None],
  [ 0x599e, 'CjkUnifiedIdeograph599e', [], 'Display', None],
  [ 0x599f, 'CjkUnifiedIdeograph599f', [], 'Display', None],
  [ 0x59a0, 'CjkUnifiedIdeograph59a0', [], 'Display', None],
  [ 0x59a1, 'CjkUnifiedIdeograph59a1', [], 'Display', None],
  [ 0x59a2, 'CjkUnifiedIdeograph59a2', [], 'Display', None],
  [ 0x59a3, 'CjkUnifiedIdeograph59a3', [], 'Display', None],
  [ 0x59a4, 'CjkUnifiedIdeograph59a4', [], 'Display', None],
  [ 0x59a5, 'CjkUnifiedIdeograph59a5', [], 'Display', None],
  [ 0x59a6, 'CjkUnifiedIdeograph59a6', [], 'Display', None],
  [ 0x59a7, 'CjkUnifiedIdeograph59a7', [], 'Display', None],
  [ 0x59a8, 'CjkUnifiedIdeograph59a8', [], 'Display', None],
  [ 0x59a9, 'CjkUnifiedIdeograph59a9', [], 'Display', None],
  [ 0x59aa, 'CjkUnifiedIdeograph59aa', [], 'Display', None],
  [ 0x59ab, 'CjkUnifiedIdeograph59ab', [], 'Display', None],
  [ 0x59ac, 'CjkUnifiedIdeograph59ac', [], 'Display', None],
  [ 0x59ad, 'CjkUnifiedIdeograph59ad', [], 'Display', None],
  [ 0x59ae, 'CjkUnifiedIdeograph59ae', [], 'Display', None],
  [ 0x59af, 'CjkUnifiedIdeograph59af', [], 'Display', None],
  [ 0x59b0, 'CjkUnifiedIdeograph59b0', [], 'Display', None],
  [ 0x59b1, 'CjkUnifiedIdeograph59b1', [], 'Display', None],
  [ 0x59b2, 'CjkUnifiedIdeograph59b2', [], 'Display', None],
  [ 0x59b3, 'CjkUnifiedIdeograph59b3', [], 'Display', None],
  [ 0x59b4, 'CjkUnifiedIdeograph59b4', [], 'Display', None],
  [ 0x59b5, 'CjkUnifiedIdeograph59b5', [], 'Display', None],
  [ 0x59b6, 'CjkUnifiedIdeograph59b6', [], 'Display', None],
  [ 0x59b7, 'CjkUnifiedIdeograph59b7', [], 'Display', None],
  [ 0x59b8, 'CjkUnifiedIdeograph59b8', [], 'Display', None],
  [ 0x59b9, 'CjkUnifiedIdeograph59b9', [], 'Display', None],
  [ 0x59ba, 'CjkUnifiedIdeograph59ba', [], 'Display', None],
  [ 0x59bb, 'CjkUnifiedIdeograph59bb', [], 'Display', None],
  [ 0x59bc, 'CjkUnifiedIdeograph59bc', [], 'Display', None],
  [ 0x59bd, 'CjkUnifiedIdeograph59bd', [], 'Display', None],
  [ 0x59be, 'CjkUnifiedIdeograph59be', [], 'Display', None],
  [ 0x59bf, 'CjkUnifiedIdeograph59bf', [], 'Display', None],
  [ 0x59c0, 'CjkUnifiedIdeograph59c0', [], 'Display', None],
  [ 0x59c1, 'CjkUnifiedIdeograph59c1', [], 'Display', None],
  [ 0x59c2, 'CjkUnifiedIdeograph59c2', [], 'Display', None],
  [ 0x59c3, 'CjkUnifiedIdeograph59c3', [], 'Display', None],
  [ 0x59c4, 'CjkUnifiedIdeograph59c4', [], 'Display', None],
  [ 0x59c5, 'CjkUnifiedIdeograph59c5', [], 'Display', None],
  [ 0x59c6, 'CjkUnifiedIdeograph59c6', [], 'Display', None],
  [ 0x59c7, 'CjkUnifiedIdeograph59c7', [], 'Display', None],
  [ 0x59c8, 'CjkUnifiedIdeograph59c8', [], 'Display', None],
  [ 0x59c9, 'CjkUnifiedIdeograph59c9', [], 'Display', None],
  [ 0x59ca, 'CjkUnifiedIdeograph59ca', [], 'Display', None],
  [ 0x59cb, 'CjkUnifiedIdeograph59cb', [], 'Display', None],
  [ 0x59cc, 'CjkUnifiedIdeograph59cc', [], 'Display', None],
  [ 0x59cd, 'CjkUnifiedIdeograph59cd', [], 'Display', None],
  [ 0x59ce, 'CjkUnifiedIdeograph59ce', [], 'Display', None],
  [ 0x59cf, 'CjkUnifiedIdeograph59cf', [], 'Display', None],
  [ 0x59d0, 'CjkUnifiedIdeograph59d0', [], 'Display', None],
  [ 0x59d1, 'CjkUnifiedIdeograph59d1', [], 'Display', None],
  [ 0x59d2, 'CjkUnifiedIdeograph59d2', [], 'Display', None],
  [ 0x59d3, 'CjkUnifiedIdeograph59d3', [], 'Display', None],
  [ 0x59d4, 'CjkUnifiedIdeograph59d4', [], 'Display', None],
  [ 0x59d5, 'CjkUnifiedIdeograph59d5', [], 'Display', None],
  [ 0x59d6, 'CjkUnifiedIdeograph59d6', [], 'Display', None],
  [ 0x59d7, 'CjkUnifiedIdeograph59d7', [], 'Display', None],
  [ 0x59d8, 'CjkUnifiedIdeograph59d8', [], 'Display', None],
  [ 0x59d9, 'CjkUnifiedIdeograph59d9', [], 'Display', None],
  [ 0x59da, 'CjkUnifiedIdeograph59da', [], 'Display', None],
  [ 0x59db, 'CjkUnifiedIdeograph59db', [], 'Display', None],
  [ 0x59dc, 'CjkUnifiedIdeograph59dc', [], 'Display', None],
  [ 0x59dd, 'CjkUnifiedIdeograph59dd', [], 'Display', None],
  [ 0x59de, 'CjkUnifiedIdeograph59de', [], 'Display', None],
  [ 0x59df, 'CjkUnifiedIdeograph59df', [], 'Display', None],
  [ 0x59e0, 'CjkUnifiedIdeograph59e0', [], 'Display', None],
  [ 0x59e1, 'CjkUnifiedIdeograph59e1', [], 'Display', None],
  [ 0x59e2, 'CjkUnifiedIdeograph59e2', [], 'Display', None],
  [ 0x59e3, 'CjkUnifiedIdeograph59e3', [], 'Display', None],
  [ 0x59e4, 'CjkUnifiedIdeograph59e4', [], 'Display', None],
  [ 0x59e5, 'CjkUnifiedIdeograph59e5', [], 'Display', None],
  [ 0x59e6, 'CjkUnifiedIdeograph59e6', [], 'Display', None],
  [ 0x59e7, 'CjkUnifiedIdeograph59e7', [], 'Display', None],
  [ 0x59e8, 'CjkUnifiedIdeograph59e8', [], 'Display', None],
  [ 0x59e9, 'CjkUnifiedIdeograph59e9', [], 'Display', None],
  [ 0x59ea, 'CjkUnifiedIdeograph59ea', [], 'Display', None],
  [ 0x59eb, 'CjkUnifiedIdeograph59eb', [], 'Display', None],
  [ 0x59ec, 'CjkUnifiedIdeograph59ec', [], 'Display', None],
  [ 0x59ed, 'CjkUnifiedIdeograph59ed', [], 'Display', None],
  [ 0x59ee, 'CjkUnifiedIdeograph59ee', [], 'Display', None],
  [ 0x59ef, 'CjkUnifiedIdeograph59ef', [], 'Display', None],
  [ 0x59f0, 'CjkUnifiedIdeograph59f0', [], 'Display', None],
  [ 0x59f1, 'CjkUnifiedIdeograph59f1', [], 'Display', None],
  [ 0x59f2, 'CjkUnifiedIdeograph59f2', [], 'Display', None],
  [ 0x59f3, 'CjkUnifiedIdeograph59f3', [], 'Display', None],
  [ 0x59f4, 'CjkUnifiedIdeograph59f4', [], 'Display', None],
  [ 0x59f5, 'CjkUnifiedIdeograph59f5', [], 'Display', None],
  [ 0x59f6, 'CjkUnifiedIdeograph59f6', [], 'Display', None],
  [ 0x59f7, 'CjkUnifiedIdeograph59f7', [], 'Display', None],
  [ 0x59f8, 'CjkUnifiedIdeograph59f8', [], 'Display', None],
  [ 0x59f9, 'CjkUnifiedIdeograph59f9', [], 'Display', None],
  [ 0x59fa, 'CjkUnifiedIdeograph59fa', [], 'Display', None],
  [ 0x59fb, 'CjkUnifiedIdeograph59fb', [], 'Display', None],
  [ 0x59fc, 'CjkUnifiedIdeograph59fc', [], 'Display', None],
  [ 0x59fd, 'CjkUnifiedIdeograph59fd', [], 'Display', None],
  [ 0x59fe, 'CjkUnifiedIdeograph59fe', [], 'Display', None],
  [ 0x59ff, 'CjkUnifiedIdeograph59ff', [], 'Display', None],
  [ 0x5a00, 'CjkUnifiedIdeograph5a00', [], 'Display', None],
  [ 0x5a01, 'CjkUnifiedIdeograph5a01', [], 'Display', None],
  [ 0x5a02, 'CjkUnifiedIdeograph5a02', [], 'Display', None],
  [ 0x5a03, 'CjkUnifiedIdeograph5a03', [], 'Display', None],
  [ 0x5a04, 'CjkUnifiedIdeograph5a04', [], 'Display', None],
  [ 0x5a05, 'CjkUnifiedIdeograph5a05', [], 'Display', None],
  [ 0x5a06, 'CjkUnifiedIdeograph5a06', [], 'Display', None],
  [ 0x5a07, 'CjkUnifiedIdeograph5a07', [], 'Display', None],
  [ 0x5a08, 'CjkUnifiedIdeograph5a08', [], 'Display', None],
  [ 0x5a09, 'CjkUnifiedIdeograph5a09', [], 'Display', None],
  [ 0x5a0a, 'CjkUnifiedIdeograph5a0a', [], 'Display', None],
  [ 0x5a0b, 'CjkUnifiedIdeograph5a0b', [], 'Display', None],
  [ 0x5a0c, 'CjkUnifiedIdeograph5a0c', [], 'Display', None],
  [ 0x5a0d, 'CjkUnifiedIdeograph5a0d', [], 'Display', None],
  [ 0x5a0e, 'CjkUnifiedIdeograph5a0e', [], 'Display', None],
  [ 0x5a0f, 'CjkUnifiedIdeograph5a0f', [], 'Display', None],
  [ 0x5a10, 'CjkUnifiedIdeograph5a10', [], 'Display', None],
  [ 0x5a11, 'CjkUnifiedIdeograph5a11', [], 'Display', None],
  [ 0x5a12, 'CjkUnifiedIdeograph5a12', [], 'Display', None],
  [ 0x5a13, 'CjkUnifiedIdeograph5a13', [], 'Display', None],
  [ 0x5a14, 'CjkUnifiedIdeograph5a14', [], 'Display', None],
  [ 0x5a15, 'CjkUnifiedIdeograph5a15', [], 'Display', None],
  [ 0x5a16, 'CjkUnifiedIdeograph5a16', [], 'Display', None],
  [ 0x5a17, 'CjkUnifiedIdeograph5a17', [], 'Display', None],
  [ 0x5a18, 'CjkUnifiedIdeograph5a18', [], 'Display', None],
  [ 0x5a19, 'CjkUnifiedIdeograph5a19', [], 'Display', None],
  [ 0x5a1a, 'CjkUnifiedIdeograph5a1a', [], 'Display', None],
  [ 0x5a1b, 'CjkUnifiedIdeograph5a1b', [], 'Display', None],
  [ 0x5a1c, 'CjkUnifiedIdeograph5a1c', [], 'Display', None],
  [ 0x5a1d, 'CjkUnifiedIdeograph5a1d', [], 'Display', None],
  [ 0x5a1e, 'CjkUnifiedIdeograph5a1e', [], 'Display', None],
  [ 0x5a1f, 'CjkUnifiedIdeograph5a1f', [], 'Display', None],
  [ 0x5a20, 'CjkUnifiedIdeograph5a20', [], 'Display', None],
  [ 0x5a21, 'CjkUnifiedIdeograph5a21', [], 'Display', None],
  [ 0x5a22, 'CjkUnifiedIdeograph5a22', [], 'Display', None],
  [ 0x5a23, 'CjkUnifiedIdeograph5a23', [], 'Display', None],
  [ 0x5a24, 'CjkUnifiedIdeograph5a24', [], 'Display', None],
  [ 0x5a25, 'CjkUnifiedIdeograph5a25', [], 'Display', None],
  [ 0x5a26, 'CjkUnifiedIdeograph5a26', [], 'Display', None],
  [ 0x5a27, 'CjkUnifiedIdeograph5a27', [], 'Display', None],
  [ 0x5a28, 'CjkUnifiedIdeograph5a28', [], 'Display', None],
  [ 0x5a29, 'CjkUnifiedIdeograph5a29', [], 'Display', None],
  [ 0x5a2a, 'CjkUnifiedIdeograph5a2a', [], 'Display', None],
  [ 0x5a2b, 'CjkUnifiedIdeograph5a2b', [], 'Display', None],
  [ 0x5a2c, 'CjkUnifiedIdeograph5a2c', [], 'Display', None],
  [ 0x5a2d, 'CjkUnifiedIdeograph5a2d', [], 'Display', None],
  [ 0x5a2e, 'CjkUnifiedIdeograph5a2e', [], 'Display', None],
  [ 0x5a2f, 'CjkUnifiedIdeograph5a2f', [], 'Display', None],
  [ 0x5a30, 'CjkUnifiedIdeograph5a30', [], 'Display', None],
  [ 0x5a31, 'CjkUnifiedIdeograph5a31', [], 'Display', None],
  [ 0x5a32, 'CjkUnifiedIdeograph5a32', [], 'Display', None],
  [ 0x5a33, 'CjkUnifiedIdeograph5a33', [], 'Display', None],
  [ 0x5a34, 'CjkUnifiedIdeograph5a34', [], 'Display', None],
  [ 0x5a35, 'CjkUnifiedIdeograph5a35', [], 'Display', None],
  [ 0x5a36, 'CjkUnifiedIdeograph5a36', [], 'Display', None],
  [ 0x5a37, 'CjkUnifiedIdeograph5a37', [], 'Display', None],
  [ 0x5a38, 'CjkUnifiedIdeograph5a38', [], 'Display', None],
  [ 0x5a39, 'CjkUnifiedIdeograph5a39', [], 'Display', None],
  [ 0x5a3a, 'CjkUnifiedIdeograph5a3a', [], 'Display', None],
  [ 0x5a3b, 'CjkUnifiedIdeograph5a3b', [], 'Display', None],
  [ 0x5a3c, 'CjkUnifiedIdeograph5a3c', [], 'Display', None],
  [ 0x5a3d, 'CjkUnifiedIdeograph5a3d', [], 'Display', None],
  [ 0x5a3e, 'CjkUnifiedIdeograph5a3e', [], 'Display', None],
  [ 0x5a3f, 'CjkUnifiedIdeograph5a3f', [], 'Display', None],
  [ 0x5a40, 'CjkUnifiedIdeograph5a40', [], 'Display', None],
  [ 0x5a41, 'CjkUnifiedIdeograph5a41', [], 'Display', None],
  [ 0x5a42, 'CjkUnifiedIdeograph5a42', [], 'Display', None],
  [ 0x5a43, 'CjkUnifiedIdeograph5a43', [], 'Display', None],
  [ 0x5a44, 'CjkUnifiedIdeograph5a44', [], 'Display', None],
  [ 0x5a45, 'CjkUnifiedIdeograph5a45', [], 'Display', None],
  [ 0x5a46, 'CjkUnifiedIdeograph5a46', [], 'Display', None],
  [ 0x5a47, 'CjkUnifiedIdeograph5a47', [], 'Display', None],
  [ 0x5a48, 'CjkUnifiedIdeograph5a48', [], 'Display', None],
  [ 0x5a49, 'CjkUnifiedIdeograph5a49', [], 'Display', None],
  [ 0x5a4a, 'CjkUnifiedIdeograph5a4a', [], 'Display', None],
  [ 0x5a4b, 'CjkUnifiedIdeograph5a4b', [], 'Display', None],
  [ 0x5a4c, 'CjkUnifiedIdeograph5a4c', [], 'Display', None],
  [ 0x5a4d, 'CjkUnifiedIdeograph5a4d', [], 'Display', None],
  [ 0x5a4e, 'CjkUnifiedIdeograph5a4e', [], 'Display', None],
  [ 0x5a4f, 'CjkUnifiedIdeograph5a4f', [], 'Display', None],
  [ 0x5a50, 'CjkUnifiedIdeograph5a50', [], 'Display', None],
  [ 0x5a51, 'CjkUnifiedIdeograph5a51', [], 'Display', None],
  [ 0x5a52, 'CjkUnifiedIdeograph5a52', [], 'Display', None],
  [ 0x5a53, 'CjkUnifiedIdeograph5a53', [], 'Display', None],
  [ 0x5a54, 'CjkUnifiedIdeograph5a54', [], 'Display', None],
  [ 0x5a55, 'CjkUnifiedIdeograph5a55', [], 'Display', None],
  [ 0x5a56, 'CjkUnifiedIdeograph5a56', [], 'Display', None],
  [ 0x5a57, 'CjkUnifiedIdeograph5a57', [], 'Display', None],
  [ 0x5a58, 'CjkUnifiedIdeograph5a58', [], 'Display', None],
  [ 0x5a59, 'CjkUnifiedIdeograph5a59', [], 'Display', None],
  [ 0x5a5a, 'CjkUnifiedIdeograph5a5a', [], 'Display', None],
  [ 0x5a5b, 'CjkUnifiedIdeograph5a5b', [], 'Display', None],
  [ 0x5a5c, 'CjkUnifiedIdeograph5a5c', [], 'Display', None],
  [ 0x5a5d, 'CjkUnifiedIdeograph5a5d', [], 'Display', None],
  [ 0x5a5e, 'CjkUnifiedIdeograph5a5e', [], 'Display', None],
  [ 0x5a5f, 'CjkUnifiedIdeograph5a5f', [], 'Display', None],
  [ 0x5a60, 'CjkUnifiedIdeograph5a60', [], 'Display', None],
  [ 0x5a61, 'CjkUnifiedIdeograph5a61', [], 'Display', None],
  [ 0x5a62, 'CjkUnifiedIdeograph5a62', [], 'Display', None],
  [ 0x5a63, 'CjkUnifiedIdeograph5a63', [], 'Display', None],
  [ 0x5a64, 'CjkUnifiedIdeograph5a64', [], 'Display', None],
  [ 0x5a65, 'CjkUnifiedIdeograph5a65', [], 'Display', None],
  [ 0x5a66, 'CjkUnifiedIdeograph5a66', [], 'Display', None],
  [ 0x5a67, 'CjkUnifiedIdeograph5a67', [], 'Display', None],
  [ 0x5a68, 'CjkUnifiedIdeograph5a68', [], 'Display', None],
  [ 0x5a69, 'CjkUnifiedIdeograph5a69', [], 'Display', None],
  [ 0x5a6a, 'CjkUnifiedIdeograph5a6a', [], 'Display', None],
  [ 0x5a6b, 'CjkUnifiedIdeograph5a6b', [], 'Display', None],
  [ 0x5a6c, 'CjkUnifiedIdeograph5a6c', [], 'Display', None],
  [ 0x5a6d, 'CjkUnifiedIdeograph5a6d', [], 'Display', None],
  [ 0x5a6e, 'CjkUnifiedIdeograph5a6e', [], 'Display', None],
  [ 0x5a6f, 'CjkUnifiedIdeograph5a6f', [], 'Display', None],
  [ 0x5a70, 'CjkUnifiedIdeograph5a70', [], 'Display', None],
  [ 0x5a71, 'CjkUnifiedIdeograph5a71', [], 'Display', None],
  [ 0x5a72, 'CjkUnifiedIdeograph5a72', [], 'Display', None],
  [ 0x5a73, 'CjkUnifiedIdeograph5a73', [], 'Display', None],
  [ 0x5a74, 'CjkUnifiedIdeograph5a74', [], 'Display', None],
  [ 0x5a75, 'CjkUnifiedIdeograph5a75', [], 'Display', None],
  [ 0x5a76, 'CjkUnifiedIdeograph5a76', [], 'Display', None],
  [ 0x5a77, 'CjkUnifiedIdeograph5a77', [], 'Display', None],
  [ 0x5a78, 'CjkUnifiedIdeograph5a78', [], 'Display', None],
  [ 0x5a79, 'CjkUnifiedIdeograph5a79', [], 'Display', None],
  [ 0x5a7a, 'CjkUnifiedIdeograph5a7a', [], 'Display', None],
  [ 0x5a7b, 'CjkUnifiedIdeograph5a7b', [], 'Display', None],
  [ 0x5a7c, 'CjkUnifiedIdeograph5a7c', [], 'Display', None],
  [ 0x5a7d, 'CjkUnifiedIdeograph5a7d', [], 'Display', None],
  [ 0x5a7e, 'CjkUnifiedIdeograph5a7e', [], 'Display', None],
  [ 0x5a7f, 'CjkUnifiedIdeograph5a7f', [], 'Display', None],
  [ 0x5a80, 'CjkUnifiedIdeograph5a80', [], 'Display', None],
  [ 0x5a81, 'CjkUnifiedIdeograph5a81', [], 'Display', None],
  [ 0x5a82, 'CjkUnifiedIdeograph5a82', [], 'Display', None],
  [ 0x5a83, 'CjkUnifiedIdeograph5a83', [], 'Display', None],
  [ 0x5a84, 'CjkUnifiedIdeograph5a84', [], 'Display', None],
  [ 0x5a85, 'CjkUnifiedIdeograph5a85', [], 'Display', None],
  [ 0x5a86, 'CjkUnifiedIdeograph5a86', [], 'Display', None],
  [ 0x5a87, 'CjkUnifiedIdeograph5a87', [], 'Display', None],
  [ 0x5a88, 'CjkUnifiedIdeograph5a88', [], 'Display', None],
  [ 0x5a89, 'CjkUnifiedIdeograph5a89', [], 'Display', None],
  [ 0x5a8a, 'CjkUnifiedIdeograph5a8a', [], 'Display', None],
  [ 0x5a8b, 'CjkUnifiedIdeograph5a8b', [], 'Display', None],
  [ 0x5a8c, 'CjkUnifiedIdeograph5a8c', [], 'Display', None],
  [ 0x5a8d, 'CjkUnifiedIdeograph5a8d', [], 'Display', None],
  [ 0x5a8e, 'CjkUnifiedIdeograph5a8e', [], 'Display', None],
  [ 0x5a8f, 'CjkUnifiedIdeograph5a8f', [], 'Display', None],
  [ 0x5a90, 'CjkUnifiedIdeograph5a90', [], 'Display', None],
  [ 0x5a91, 'CjkUnifiedIdeograph5a91', [], 'Display', None],
  [ 0x5a92, 'CjkUnifiedIdeograph5a92', [], 'Display', None],
  [ 0x5a93, 'CjkUnifiedIdeograph5a93', [], 'Display', None],
  [ 0x5a94, 'CjkUnifiedIdeograph5a94', [], 'Display', None],
  [ 0x5a95, 'CjkUnifiedIdeograph5a95', [], 'Display', None],
  [ 0x5a96, 'CjkUnifiedIdeograph5a96', [], 'Display', None],
  [ 0x5a97, 'CjkUnifiedIdeograph5a97', [], 'Display', None],
  [ 0x5a98, 'CjkUnifiedIdeograph5a98', [], 'Display', None],
  [ 0x5a99, 'CjkUnifiedIdeograph5a99', [], 'Display', None],
  [ 0x5a9a, 'CjkUnifiedIdeograph5a9a', [], 'Display', None],
  [ 0x5a9b, 'CjkUnifiedIdeograph5a9b', [], 'Display', None],
  [ 0x5a9c, 'CjkUnifiedIdeograph5a9c', [], 'Display', None],
  [ 0x5a9d, 'CjkUnifiedIdeograph5a9d', [], 'Display', None],
  [ 0x5a9e, 'CjkUnifiedIdeograph5a9e', [], 'Display', None],
  [ 0x5a9f, 'CjkUnifiedIdeograph5a9f', [], 'Display', None],
  [ 0x5aa0, 'CjkUnifiedIdeograph5aa0', [], 'Display', None],
  [ 0x5aa1, 'CjkUnifiedIdeograph5aa1', [], 'Display', None],
  [ 0x5aa2, 'CjkUnifiedIdeograph5aa2', [], 'Display', None],
  [ 0x5aa3, 'CjkUnifiedIdeograph5aa3', [], 'Display', None],
  [ 0x5aa4, 'CjkUnifiedIdeograph5aa4', [], 'Display', None],
  [ 0x5aa5, 'CjkUnifiedIdeograph5aa5', [], 'Display', None],
  [ 0x5aa6, 'CjkUnifiedIdeograph5aa6', [], 'Display', None],
  [ 0x5aa7, 'CjkUnifiedIdeograph5aa7', [], 'Display', None],
  [ 0x5aa8, 'CjkUnifiedIdeograph5aa8', [], 'Display', None],
  [ 0x5aa9, 'CjkUnifiedIdeograph5aa9', [], 'Display', None],
  [ 0x5aaa, 'CjkUnifiedIdeograph5aaa', [], 'Display', None],
  [ 0x5aab, 'CjkUnifiedIdeograph5aab', [], 'Display', None],
  [ 0x5aac, 'CjkUnifiedIdeograph5aac', [], 'Display', None],
  [ 0x5aad, 'CjkUnifiedIdeograph5aad', [], 'Display', None],
  [ 0x5aae, 'CjkUnifiedIdeograph5aae', [], 'Display', None],
  [ 0x5aaf, 'CjkUnifiedIdeograph5aaf', [], 'Display', None],
  [ 0x5ab0, 'CjkUnifiedIdeograph5ab0', [], 'Display', None],
  [ 0x5ab1, 'CjkUnifiedIdeograph5ab1', [], 'Display', None],
  [ 0x5ab2, 'CjkUnifiedIdeograph5ab2', [], 'Display', None],
  [ 0x5ab3, 'CjkUnifiedIdeograph5ab3', [], 'Display', None],
  [ 0x5ab4, 'CjkUnifiedIdeograph5ab4', [], 'Display', None],
  [ 0x5ab5, 'CjkUnifiedIdeograph5ab5', [], 'Display', None],
  [ 0x5ab6, 'CjkUnifiedIdeograph5ab6', [], 'Display', None],
  [ 0x5ab7, 'CjkUnifiedIdeograph5ab7', [], 'Display', None],
  [ 0x5ab8, 'CjkUnifiedIdeograph5ab8', [], 'Display', None],
  [ 0x5ab9, 'CjkUnifiedIdeograph5ab9', [], 'Display', None],
  [ 0x5aba, 'CjkUnifiedIdeograph5aba', [], 'Display', None],
  [ 0x5abb, 'CjkUnifiedIdeograph5abb', [], 'Display', None],
  [ 0x5abc, 'CjkUnifiedIdeograph5abc', [], 'Display', None],
  [ 0x5abd, 'CjkUnifiedIdeograph5abd', [], 'Display', None],
  [ 0x5abe, 'CjkUnifiedIdeograph5abe', [], 'Display', None],
  [ 0x5abf, 'CjkUnifiedIdeograph5abf', [], 'Display', None],
  [ 0x5ac0, 'CjkUnifiedIdeograph5ac0', [], 'Display', None],
  [ 0x5ac1, 'CjkUnifiedIdeograph5ac1', [], 'Display', None],
  [ 0x5ac2, 'CjkUnifiedIdeograph5ac2', [], 'Display', None],
  [ 0x5ac3, 'CjkUnifiedIdeograph5ac3', [], 'Display', None],
  [ 0x5ac4, 'CjkUnifiedIdeograph5ac4', [], 'Display', None],
  [ 0x5ac5, 'CjkUnifiedIdeograph5ac5', [], 'Display', None],
  [ 0x5ac6, 'CjkUnifiedIdeograph5ac6', [], 'Display', None],
  [ 0x5ac7, 'CjkUnifiedIdeograph5ac7', [], 'Display', None],
  [ 0x5ac8, 'CjkUnifiedIdeograph5ac8', [], 'Display', None],
  [ 0x5ac9, 'CjkUnifiedIdeograph5ac9', [], 'Display', None],
  [ 0x5aca, 'CjkUnifiedIdeograph5aca', [], 'Display', None],
  [ 0x5acb, 'CjkUnifiedIdeograph5acb', [], 'Display', None],
  [ 0x5acc, 'CjkUnifiedIdeograph5acc', [], 'Display', None],
  [ 0x5acd, 'CjkUnifiedIdeograph5acd', [], 'Display', None],
  [ 0x5ace, 'CjkUnifiedIdeograph5ace', [], 'Display', None],
  [ 0x5acf, 'CjkUnifiedIdeograph5acf', [], 'Display', None],
  [ 0x5ad0, 'CjkUnifiedIdeograph5ad0', [], 'Display', None],
  [ 0x5ad1, 'CjkUnifiedIdeograph5ad1', [], 'Display', None],
  [ 0x5ad2, 'CjkUnifiedIdeograph5ad2', [], 'Display', None],
  [ 0x5ad3, 'CjkUnifiedIdeograph5ad3', [], 'Display', None],
  [ 0x5ad4, 'CjkUnifiedIdeograph5ad4', [], 'Display', None],
  [ 0x5ad5, 'CjkUnifiedIdeograph5ad5', [], 'Display', None],
  [ 0x5ad6, 'CjkUnifiedIdeograph5ad6', [], 'Display', None],
  [ 0x5ad7, 'CjkUnifiedIdeograph5ad7', [], 'Display', None],
  [ 0x5ad8, 'CjkUnifiedIdeograph5ad8', [], 'Display', None],
  [ 0x5ad9, 'CjkUnifiedIdeograph5ad9', [], 'Display', None],
  [ 0x5ada, 'CjkUnifiedIdeograph5ada', [], 'Display', None],
  [ 0x5adb, 'CjkUnifiedIdeograph5adb', [], 'Display', None],
  [ 0x5adc, 'CjkUnifiedIdeograph5adc', [], 'Display', None],
  [ 0x5add, 'CjkUnifiedIdeograph5add', [], 'Display', None],
  [ 0x5ade, 'CjkUnifiedIdeograph5ade', [], 'Display', None],
  [ 0x5adf, 'CjkUnifiedIdeograph5adf', [], 'Display', None],
  [ 0x5ae0, 'CjkUnifiedIdeograph5ae0', [], 'Display', None],
  [ 0x5ae1, 'CjkUnifiedIdeograph5ae1', [], 'Display', None],
  [ 0x5ae2, 'CjkUnifiedIdeograph5ae2', [], 'Display', None],
  [ 0x5ae3, 'CjkUnifiedIdeograph5ae3', [], 'Display', None],
  [ 0x5ae4, 'CjkUnifiedIdeograph5ae4', [], 'Display', None],
  [ 0x5ae5, 'CjkUnifiedIdeograph5ae5', [], 'Display', None],
  [ 0x5ae6, 'CjkUnifiedIdeograph5ae6', [], 'Display', None],
  [ 0x5ae7, 'CjkUnifiedIdeograph5ae7', [], 'Display', None],
  [ 0x5ae8, 'CjkUnifiedIdeograph5ae8', [], 'Display', None],
  [ 0x5ae9, 'CjkUnifiedIdeograph5ae9', [], 'Display', None],
  [ 0x5aea, 'CjkUnifiedIdeograph5aea', [], 'Display', None],
  [ 0x5aeb, 'CjkUnifiedIdeograph5aeb', [], 'Display', None],
  [ 0x5aec, 'CjkUnifiedIdeograph5aec', [], 'Display', None],
  [ 0x5aed, 'CjkUnifiedIdeograph5aed', [], 'Display', None],
  [ 0x5aee, 'CjkUnifiedIdeograph5aee', [], 'Display', None],
  [ 0x5aef, 'CjkUnifiedIdeograph5aef', [], 'Display', None],
  [ 0x5af0, 'CjkUnifiedIdeograph5af0', [], 'Display', None],
  [ 0x5af1, 'CjkUnifiedIdeograph5af1', [], 'Display', None],
  [ 0x5af2, 'CjkUnifiedIdeograph5af2', [], 'Display', None],
  [ 0x5af3, 'CjkUnifiedIdeograph5af3', [], 'Display', None],
  [ 0x5af4, 'CjkUnifiedIdeograph5af4', [], 'Display', None],
  [ 0x5af5, 'CjkUnifiedIdeograph5af5', [], 'Display', None],
  [ 0x5af6, 'CjkUnifiedIdeograph5af6', [], 'Display', None],
  [ 0x5af7, 'CjkUnifiedIdeograph5af7', [], 'Display', None],
  [ 0x5af8, 'CjkUnifiedIdeograph5af8', [], 'Display', None],
  [ 0x5af9, 'CjkUnifiedIdeograph5af9', [], 'Display', None],
  [ 0x5afa, 'CjkUnifiedIdeograph5afa', [], 'Display', None],
  [ 0x5afb, 'CjkUnifiedIdeograph5afb', [], 'Display', None],
  [ 0x5afc, 'CjkUnifiedIdeograph5afc', [], 'Display', None],
  [ 0x5afd, 'CjkUnifiedIdeograph5afd', [], 'Display', None],
  [ 0x5afe, 'CjkUnifiedIdeograph5afe', [], 'Display', None],
  [ 0x5aff, 'CjkUnifiedIdeograph5aff', [], 'Display', None],
  [ 0x5b00, 'CjkUnifiedIdeograph5b00', [], 'Display', None],
  [ 0x5b01, 'CjkUnifiedIdeograph5b01', [], 'Display', None],
  [ 0x5b02, 'CjkUnifiedIdeograph5b02', [], 'Display', None],
  [ 0x5b03, 'CjkUnifiedIdeograph5b03', [], 'Display', None],
  [ 0x5b04, 'CjkUnifiedIdeograph5b04', [], 'Display', None],
  [ 0x5b05, 'CjkUnifiedIdeograph5b05', [], 'Display', None],
  [ 0x5b06, 'CjkUnifiedIdeograph5b06', [], 'Display', None],
  [ 0x5b07, 'CjkUnifiedIdeograph5b07', [], 'Display', None],
  [ 0x5b08, 'CjkUnifiedIdeograph5b08', [], 'Display', None],
  [ 0x5b09, 'CjkUnifiedIdeograph5b09', [], 'Display', None],
  [ 0x5b0a, 'CjkUnifiedIdeograph5b0a', [], 'Display', None],
  [ 0x5b0b, 'CjkUnifiedIdeograph5b0b', [], 'Display', None],
  [ 0x5b0c, 'CjkUnifiedIdeograph5b0c', [], 'Display', None],
  [ 0x5b0d, 'CjkUnifiedIdeograph5b0d', [], 'Display', None],
  [ 0x5b0e, 'CjkUnifiedIdeograph5b0e', [], 'Display', None],
  [ 0x5b0f, 'CjkUnifiedIdeograph5b0f', [], 'Display', None],
  [ 0x5b10, 'CjkUnifiedIdeograph5b10', [], 'Display', None],
  [ 0x5b11, 'CjkUnifiedIdeograph5b11', [], 'Display', None],
  [ 0x5b12, 'CjkUnifiedIdeograph5b12', [], 'Display', None],
  [ 0x5b13, 'CjkUnifiedIdeograph5b13', [], 'Display', None],
  [ 0x5b14, 'CjkUnifiedIdeograph5b14', [], 'Display', None],
  [ 0x5b15, 'CjkUnifiedIdeograph5b15', [], 'Display', None],
  [ 0x5b16, 'CjkUnifiedIdeograph5b16', [], 'Display', None],
  [ 0x5b17, 'CjkUnifiedIdeograph5b17', [], 'Display', None],
  [ 0x5b18, 'CjkUnifiedIdeograph5b18', [], 'Display', None],
  [ 0x5b19, 'CjkUnifiedIdeograph5b19', [], 'Display', None],
  [ 0x5b1a, 'CjkUnifiedIdeograph5b1a', [], 'Display', None],
  [ 0x5b1b, 'CjkUnifiedIdeograph5b1b', [], 'Display', None],
  [ 0x5b1c, 'CjkUnifiedIdeograph5b1c', [], 'Display', None],
  [ 0x5b1d, 'CjkUnifiedIdeograph5b1d', [], 'Display', None],
  [ 0x5b1e, 'CjkUnifiedIdeograph5b1e', [], 'Display', None],
  [ 0x5b1f, 'CjkUnifiedIdeograph5b1f', [], 'Display', None],
  [ 0x5b20, 'CjkUnifiedIdeograph5b20', [], 'Display', None],
  [ 0x5b21, 'CjkUnifiedIdeograph5b21', [], 'Display', None],
  [ 0x5b22, 'CjkUnifiedIdeograph5b22', [], 'Display', None],
  [ 0x5b23, 'CjkUnifiedIdeograph5b23', [], 'Display', None],
  [ 0x5b24, 'CjkUnifiedIdeograph5b24', [], 'Display', None],
  [ 0x5b25, 'CjkUnifiedIdeograph5b25', [], 'Display', None],
  [ 0x5b26, 'CjkUnifiedIdeograph5b26', [], 'Display', None],
  [ 0x5b27, 'CjkUnifiedIdeograph5b27', [], 'Display', None],
  [ 0x5b28, 'CjkUnifiedIdeograph5b28', [], 'Display', None],
  [ 0x5b29, 'CjkUnifiedIdeograph5b29', [], 'Display', None],
  [ 0x5b2a, 'CjkUnifiedIdeograph5b2a', [], 'Display', None],
  [ 0x5b2b, 'CjkUnifiedIdeograph5b2b', [], 'Display', None],
  [ 0x5b2c, 'CjkUnifiedIdeograph5b2c', [], 'Display', None],
  [ 0x5b2d, 'CjkUnifiedIdeograph5b2d', [], 'Display', None],
  [ 0x5b2e, 'CjkUnifiedIdeograph5b2e', [], 'Display', None],
  [ 0x5b2f, 'CjkUnifiedIdeograph5b2f', [], 'Display', None],
  [ 0x5b30, 'CjkUnifiedIdeograph5b30', [], 'Display', None],
  [ 0x5b31, 'CjkUnifiedIdeograph5b31', [], 'Display', None],
  [ 0x5b32, 'CjkUnifiedIdeograph5b32', [], 'Display', None],
  [ 0x5b33, 'CjkUnifiedIdeograph5b33', [], 'Display', None],
  [ 0x5b34, 'CjkUnifiedIdeograph5b34', [], 'Display', None],
  [ 0x5b35, 'CjkUnifiedIdeograph5b35', [], 'Display', None],
  [ 0x5b36, 'CjkUnifiedIdeograph5b36', [], 'Display', None],
  [ 0x5b37, 'CjkUnifiedIdeograph5b37', [], 'Display', None],
  [ 0x5b38, 'CjkUnifiedIdeograph5b38', [], 'Display', None],
  [ 0x5b39, 'CjkUnifiedIdeograph5b39', [], 'Display', None],
  [ 0x5b3a, 'CjkUnifiedIdeograph5b3a', [], 'Display', None],
  [ 0x5b3b, 'CjkUnifiedIdeograph5b3b', [], 'Display', None],
  [ 0x5b3c, 'CjkUnifiedIdeograph5b3c', [], 'Display', None],
  [ 0x5b3d, 'CjkUnifiedIdeograph5b3d', [], 'Display', None],
  [ 0x5b3e, 'CjkUnifiedIdeograph5b3e', [], 'Display', None],
  [ 0x5b3f, 'CjkUnifiedIdeograph5b3f', [], 'Display', None],
  [ 0x5b40, 'CjkUnifiedIdeograph5b40', [], 'Display', None],
  [ 0x5b41, 'CjkUnifiedIdeograph5b41', [], 'Display', None],
  [ 0x5b42, 'CjkUnifiedIdeograph5b42', [], 'Display', None],
  [ 0x5b43, 'CjkUnifiedIdeograph5b43', [], 'Display', None],
  [ 0x5b44, 'CjkUnifiedIdeograph5b44', [], 'Display', None],
  [ 0x5b45, 'CjkUnifiedIdeograph5b45', [], 'Display', None],
  [ 0x5b46, 'CjkUnifiedIdeograph5b46', [], 'Display', None],
  [ 0x5b47, 'CjkUnifiedIdeograph5b47', [], 'Display', None],
  [ 0x5b48, 'CjkUnifiedIdeograph5b48', [], 'Display', None],
  [ 0x5b49, 'CjkUnifiedIdeograph5b49', [], 'Display', None],
  [ 0x5b4a, 'CjkUnifiedIdeograph5b4a', [], 'Display', None],
  [ 0x5b4b, 'CjkUnifiedIdeograph5b4b', [], 'Display', None],
  [ 0x5b4c, 'CjkUnifiedIdeograph5b4c', [], 'Display', None],
  [ 0x5b4d, 'CjkUnifiedIdeograph5b4d', [], 'Display', None],
  [ 0x5b4e, 'CjkUnifiedIdeograph5b4e', [], 'Display', None],
  [ 0x5b4f, 'CjkUnifiedIdeograph5b4f', [], 'Display', None],
  [ 0x5b50, 'CjkUnifiedIdeograph5b50', [], 'Display', None],
  [ 0x5b51, 'CjkUnifiedIdeograph5b51', [], 'Display', None],
  [ 0x5b52, 'CjkUnifiedIdeograph5b52', [], 'Display', None],
  [ 0x5b53, 'CjkUnifiedIdeograph5b53', [], 'Display', None],
  [ 0x5b54, 'CjkUnifiedIdeograph5b54', [], 'Display', None],
  [ 0x5b55, 'CjkUnifiedIdeograph5b55', [], 'Display', None],
  [ 0x5b56, 'CjkUnifiedIdeograph5b56', [], 'Display', None],
  [ 0x5b57, 'CjkUnifiedIdeograph5b57', [], 'Display', None],
  [ 0x5b58, 'CjkUnifiedIdeograph5b58', [], 'Display', None],
  [ 0x5b59, 'CjkUnifiedIdeograph5b59', [], 'Display', None],
  [ 0x5b5a, 'CjkUnifiedIdeograph5b5a', [], 'Display', None],
  [ 0x5b5b, 'CjkUnifiedIdeograph5b5b', [], 'Display', None],
  [ 0x5b5c, 'CjkUnifiedIdeograph5b5c', [], 'Display', None],
  [ 0x5b5d, 'CjkUnifiedIdeograph5b5d', [], 'Display', None],
  [ 0x5b5e, 'CjkUnifiedIdeograph5b5e', [], 'Display', None],
  [ 0x5b5f, 'CjkUnifiedIdeograph5b5f', [], 'Display', None],
  [ 0x5b60, 'CjkUnifiedIdeograph5b60', [], 'Display', None],
  [ 0x5b61, 'CjkUnifiedIdeograph5b61', [], 'Display', None],
  [ 0x5b62, 'CjkUnifiedIdeograph5b62', [], 'Display', None],
  [ 0x5b63, 'CjkUnifiedIdeograph5b63', [], 'Display', None],
  [ 0x5b64, 'CjkUnifiedIdeograph5b64', [], 'Display', None],
  [ 0x5b65, 'CjkUnifiedIdeograph5b65', [], 'Display', None],
  [ 0x5b66, 'CjkUnifiedIdeograph5b66', [], 'Display', None],
  [ 0x5b67, 'CjkUnifiedIdeograph5b67', [], 'Display', None],
  [ 0x5b68, 'CjkUnifiedIdeograph5b68', [], 'Display', None],
  [ 0x5b69, 'CjkUnifiedIdeograph5b69', [], 'Display', None],
  [ 0x5b6a, 'CjkUnifiedIdeograph5b6a', [], 'Display', None],
  [ 0x5b6b, 'CjkUnifiedIdeograph5b6b', [], 'Display', None],
  [ 0x5b6c, 'CjkUnifiedIdeograph5b6c', [], 'Display', None],
  [ 0x5b6d, 'CjkUnifiedIdeograph5b6d', [], 'Display', None],
  [ 0x5b6e, 'CjkUnifiedIdeograph5b6e', [], 'Display', None],
  [ 0x5b6f, 'CjkUnifiedIdeograph5b6f', [], 'Display', None],
  [ 0x5b70, 'CjkUnifiedIdeograph5b70', [], 'Display', None],
  [ 0x5b71, 'CjkUnifiedIdeograph5b71', [], 'Display', None],
  [ 0x5b72, 'CjkUnifiedIdeograph5b72', [], 'Display', None],
  [ 0x5b73, 'CjkUnifiedIdeograph5b73', [], 'Display', None],
  [ 0x5b74, 'CjkUnifiedIdeograph5b74', [], 'Display', None],
  [ 0x5b75, 'CjkUnifiedIdeograph5b75', [], 'Display', None],
  [ 0x5b76, 'CjkUnifiedIdeograph5b76', [], 'Display', None],
  [ 0x5b77, 'CjkUnifiedIdeograph5b77', [], 'Display', None],
  [ 0x5b78, 'CjkUnifiedIdeograph5b78', [], 'Display', None],
  [ 0x5b79, 'CjkUnifiedIdeograph5b79', [], 'Display', None],
  [ 0x5b7a, 'CjkUnifiedIdeograph5b7a', [], 'Display', None],
  [ 0x5b7b, 'CjkUnifiedIdeograph5b7b', [], 'Display', None],
  [ 0x5b7c, 'CjkUnifiedIdeograph5b7c', [], 'Display', None],
  [ 0x5b7d, 'CjkUnifiedIdeograph5b7d', [], 'Display', None],
  [ 0x5b7e, 'CjkUnifiedIdeograph5b7e', [], 'Display', None],
  [ 0x5b7f, 'CjkUnifiedIdeograph5b7f', [], 'Display', None],
  [ 0x5b80, 'CjkUnifiedIdeograph5b80', [], 'Display', None],
  [ 0x5b81, 'CjkUnifiedIdeograph5b81', [], 'Display', None],
  [ 0x5b82, 'CjkUnifiedIdeograph5b82', [], 'Display', None],
  [ 0x5b83, 'CjkUnifiedIdeograph5b83', [], 'Display', None],
  [ 0x5b84, 'CjkUnifiedIdeograph5b84', [], 'Display', None],
  [ 0x5b85, 'CjkUnifiedIdeograph5b85', [], 'Display', None],
  [ 0x5b86, 'CjkUnifiedIdeograph5b86', [], 'Display', None],
  [ 0x5b87, 'CjkUnifiedIdeograph5b87', [], 'Display', None],
  [ 0x5b88, 'CjkUnifiedIdeograph5b88', [], 'Display', None],
  [ 0x5b89, 'CjkUnifiedIdeograph5b89', [], 'Display', None],
  [ 0x5b8a, 'CjkUnifiedIdeograph5b8a', [], 'Display', None],
  [ 0x5b8b, 'CjkUnifiedIdeograph5b8b', [], 'Display', None],
  [ 0x5b8c, 'CjkUnifiedIdeograph5b8c', [], 'Display', None],
  [ 0x5b8d, 'CjkUnifiedIdeograph5b8d', [], 'Display', None],
  [ 0x5b8e, 'CjkUnifiedIdeograph5b8e', [], 'Display', None],
  [ 0x5b8f, 'CjkUnifiedIdeograph5b8f', [], 'Display', None],
  [ 0x5b90, 'CjkUnifiedIdeograph5b90', [], 'Display', None],
  [ 0x5b91, 'CjkUnifiedIdeograph5b91', [], 'Display', None],
  [ 0x5b92, 'CjkUnifiedIdeograph5b92', [], 'Display', None],
  [ 0x5b93, 'CjkUnifiedIdeograph5b93', [], 'Display', None],
  [ 0x5b94, 'CjkUnifiedIdeograph5b94', [], 'Display', None],
  [ 0x5b95, 'CjkUnifiedIdeograph5b95', [], 'Display', None],
  [ 0x5b96, 'CjkUnifiedIdeograph5b96', [], 'Display', None],
  [ 0x5b97, 'CjkUnifiedIdeograph5b97', [], 'Display', None],
  [ 0x5b98, 'CjkUnifiedIdeograph5b98', [], 'Display', None],
  [ 0x5b99, 'CjkUnifiedIdeograph5b99', [], 'Display', None],
  [ 0x5b9a, 'CjkUnifiedIdeograph5b9a', [], 'Display', None],
  [ 0x5b9b, 'CjkUnifiedIdeograph5b9b', [], 'Display', None],
  [ 0x5b9c, 'CjkUnifiedIdeograph5b9c', [], 'Display', None],
  [ 0x5b9d, 'CjkUnifiedIdeograph5b9d', [], 'Display', None],
  [ 0x5b9e, 'CjkUnifiedIdeograph5b9e', [], 'Display', None],
  [ 0x5b9f, 'CjkUnifiedIdeograph5b9f', [], 'Display', None],
  [ 0x5ba0, 'CjkUnifiedIdeograph5ba0', [], 'Display', None],
  [ 0x5ba1, 'CjkUnifiedIdeograph5ba1', [], 'Display', None],
  [ 0x5ba2, 'CjkUnifiedIdeograph5ba2', [], 'Display', None],
  [ 0x5ba3, 'CjkUnifiedIdeograph5ba3', [], 'Display', None],
  [ 0x5ba4, 'CjkUnifiedIdeograph5ba4', [], 'Display', None],
  [ 0x5ba5, 'CjkUnifiedIdeograph5ba5', [], 'Display', None],
  [ 0x5ba6, 'CjkUnifiedIdeograph5ba6', [], 'Display', None],
  [ 0x5ba7, 'CjkUnifiedIdeograph5ba7', [], 'Display', None],
  [ 0x5ba8, 'CjkUnifiedIdeograph5ba8', [], 'Display', None],
  [ 0x5ba9, 'CjkUnifiedIdeograph5ba9', [], 'Display', None],
  [ 0x5baa, 'CjkUnifiedIdeograph5baa', [], 'Display', None],
  [ 0x5bab, 'CjkUnifiedIdeograph5bab', [], 'Display', None],
  [ 0x5bac, 'CjkUnifiedIdeograph5bac', [], 'Display', None],
  [ 0x5bad, 'CjkUnifiedIdeograph5bad', [], 'Display', None],
  [ 0x5bae, 'CjkUnifiedIdeograph5bae', [], 'Display', None],
  [ 0x5baf, 'CjkUnifiedIdeograph5baf', [], 'Display', None],
  [ 0x5bb0, 'CjkUnifiedIdeograph5bb0', [], 'Display', None],
  [ 0x5bb1, 'CjkUnifiedIdeograph5bb1', [], 'Display', None],
  [ 0x5bb2, 'CjkUnifiedIdeograph5bb2', [], 'Display', None],
  [ 0x5bb3, 'CjkUnifiedIdeograph5bb3', [], 'Display', None],
  [ 0x5bb4, 'CjkUnifiedIdeograph5bb4', [], 'Display', None],
  [ 0x5bb5, 'CjkUnifiedIdeograph5bb5', [], 'Display', None],
  [ 0x5bb6, 'CjkUnifiedIdeograph5bb6', [], 'Display', None],
  [ 0x5bb7, 'CjkUnifiedIdeograph5bb7', [], 'Display', None],
  [ 0x5bb8, 'CjkUnifiedIdeograph5bb8', [], 'Display', None],
  [ 0x5bb9, 'CjkUnifiedIdeograph5bb9', [], 'Display', None],
  [ 0x5bba, 'CjkUnifiedIdeograph5bba', [], 'Display', None],
  [ 0x5bbb, 'CjkUnifiedIdeograph5bbb', [], 'Display', None],
  [ 0x5bbc, 'CjkUnifiedIdeograph5bbc', [], 'Display', None],
  [ 0x5bbd, 'CjkUnifiedIdeograph5bbd', [], 'Display', None],
  [ 0x5bbe, 'CjkUnifiedIdeograph5bbe', [], 'Display', None],
  [ 0x5bbf, 'CjkUnifiedIdeograph5bbf', [], 'Display', None],
  [ 0x5bc0, 'CjkUnifiedIdeograph5bc0', [], 'Display', None],
  [ 0x5bc1, 'CjkUnifiedIdeograph5bc1', [], 'Display', None],
  [ 0x5bc2, 'CjkUnifiedIdeograph5bc2', [], 'Display', None],
  [ 0x5bc3, 'CjkUnifiedIdeograph5bc3', [], 'Display', None],
  [ 0x5bc4, 'CjkUnifiedIdeograph5bc4', [], 'Display', None],
  [ 0x5bc5, 'CjkUnifiedIdeograph5bc5', [], 'Display', None],
  [ 0x5bc6, 'CjkUnifiedIdeograph5bc6', [], 'Display', None],
  [ 0x5bc7, 'CjkUnifiedIdeograph5bc7', [], 'Display', None],
  [ 0x5bc8, 'CjkUnifiedIdeograph5bc8', [], 'Display', None],
  [ 0x5bc9, 'CjkUnifiedIdeograph5bc9', [], 'Display', None],
  [ 0x5bca, 'CjkUnifiedIdeograph5bca', [], 'Display', None],
  [ 0x5bcb, 'CjkUnifiedIdeograph5bcb', [], 'Display', None],
  [ 0x5bcc, 'CjkUnifiedIdeograph5bcc', [], 'Display', None],
  [ 0x5bcd, 'CjkUnifiedIdeograph5bcd', [], 'Display', None],
  [ 0x5bce, 'CjkUnifiedIdeograph5bce', [], 'Display', None],
  [ 0x5bcf, 'CjkUnifiedIdeograph5bcf', [], 'Display', None],
  [ 0x5bd0, 'CjkUnifiedIdeograph5bd0', [], 'Display', None],
  [ 0x5bd1, 'CjkUnifiedIdeograph5bd1', [], 'Display', None],
  [ 0x5bd2, 'CjkUnifiedIdeograph5bd2', [], 'Display', None],
  [ 0x5bd3, 'CjkUnifiedIdeograph5bd3', [], 'Display', None],
  [ 0x5bd4, 'CjkUnifiedIdeograph5bd4', [], 'Display', None],
  [ 0x5bd5, 'CjkUnifiedIdeograph5bd5', [], 'Display', None],
  [ 0x5bd6, 'CjkUnifiedIdeograph5bd6', [], 'Display', None],
  [ 0x5bd7, 'CjkUnifiedIdeograph5bd7', [], 'Display', None],
  [ 0x5bd8, 'CjkUnifiedIdeograph5bd8', [], 'Display', None],
  [ 0x5bd9, 'CjkUnifiedIdeograph5bd9', [], 'Display', None],
  [ 0x5bda, 'CjkUnifiedIdeograph5bda', [], 'Display', None],
  [ 0x5bdb, 'CjkUnifiedIdeograph5bdb', [], 'Display', None],
  [ 0x5bdc, 'CjkUnifiedIdeograph5bdc', [], 'Display', None],
  [ 0x5bdd, 'CjkUnifiedIdeograph5bdd', [], 'Display', None],
  [ 0x5bde, 'CjkUnifiedIdeograph5bde', [], 'Display', None],
  [ 0x5bdf, 'CjkUnifiedIdeograph5bdf', [], 'Display', None],
  [ 0x5be0, 'CjkUnifiedIdeograph5be0', [], 'Display', None],
  [ 0x5be1, 'CjkUnifiedIdeograph5be1', [], 'Display', None],
  [ 0x5be2, 'CjkUnifiedIdeograph5be2', [], 'Display', None],
  [ 0x5be3, 'CjkUnifiedIdeograph5be3', [], 'Display', None],
  [ 0x5be4, 'CjkUnifiedIdeograph5be4', [], 'Display', None],
  [ 0x5be5, 'CjkUnifiedIdeograph5be5', [], 'Display', None],
  [ 0x5be6, 'CjkUnifiedIdeograph5be6', [], 'Display', None],
  [ 0x5be7, 'CjkUnifiedIdeograph5be7', [], 'Display', None],
  [ 0x5be8, 'CjkUnifiedIdeograph5be8', [], 'Display', None],
  [ 0x5be9, 'CjkUnifiedIdeograph5be9', [], 'Display', None],
  [ 0x5bea, 'CjkUnifiedIdeograph5bea', [], 'Display', None],
  [ 0x5beb, 'CjkUnifiedIdeograph5beb', [], 'Display', None],
  [ 0x5bec, 'CjkUnifiedIdeograph5bec', [], 'Display', None],
  [ 0x5bed, 'CjkUnifiedIdeograph5bed', [], 'Display', None],
  [ 0x5bee, 'CjkUnifiedIdeograph5bee', [], 'Display', None],
  [ 0x5bef, 'CjkUnifiedIdeograph5bef', [], 'Display', None],
  [ 0x5bf0, 'CjkUnifiedIdeograph5bf0', [], 'Display', None],
  [ 0x5bf1, 'CjkUnifiedIdeograph5bf1', [], 'Display', None],
  [ 0x5bf2, 'CjkUnifiedIdeograph5bf2', [], 'Display', None],
  [ 0x5bf3, 'CjkUnifiedIdeograph5bf3', [], 'Display', None],
  [ 0x5bf4, 'CjkUnifiedIdeograph5bf4', [], 'Display', None],
  [ 0x5bf5, 'CjkUnifiedIdeograph5bf5', [], 'Display', None],
  [ 0x5bf6, 'CjkUnifiedIdeograph5bf6', [], 'Display', None],
  [ 0x5bf7, 'CjkUnifiedIdeograph5bf7', [], 'Display', None],
  [ 0x5bf8, 'CjkUnifiedIdeograph5bf8', [], 'Display', None],
  [ 0x5bf9, 'CjkUnifiedIdeograph5bf9', [], 'Display', None],
  [ 0x5bfa, 'CjkUnifiedIdeograph5bfa', [], 'Display', None],
  [ 0x5bfb, 'CjkUnifiedIdeograph5bfb', [], 'Display', None],
  [ 0x5bfc, 'CjkUnifiedIdeograph5bfc', [], 'Display', None],
  [ 0x5bfd, 'CjkUnifiedIdeograph5bfd', [], 'Display', None],
  [ 0x5bfe, 'CjkUnifiedIdeograph5bfe', [], 'Display', None],
  [ 0x5bff, 'CjkUnifiedIdeograph5bff', [], 'Display', None],
  [ 0x5c00, 'CjkUnifiedIdeograph5c00', [], 'Display', None],
  [ 0x5c01, 'CjkUnifiedIdeograph5c01', [], 'Display', None],
  [ 0x5c02, 'CjkUnifiedIdeograph5c02', [], 'Display', None],
  [ 0x5c03, 'CjkUnifiedIdeograph5c03', [], 'Display', None],
  [ 0x5c04, 'CjkUnifiedIdeograph5c04', [], 'Display', None],
  [ 0x5c05, 'CjkUnifiedIdeograph5c05', [], 'Display', None],
  [ 0x5c06, 'CjkUnifiedIdeograph5c06', [], 'Display', None],
  [ 0x5c07, 'CjkUnifiedIdeograph5c07', [], 'Display', None],
  [ 0x5c08, 'CjkUnifiedIdeograph5c08', [], 'Display', None],
  [ 0x5c09, 'CjkUnifiedIdeograph5c09', [], 'Display', None],
  [ 0x5c0a, 'CjkUnifiedIdeograph5c0a', [], 'Display', None],
  [ 0x5c0b, 'CjkUnifiedIdeograph5c0b', [], 'Display', None],
  [ 0x5c0c, 'CjkUnifiedIdeograph5c0c', [], 'Display', None],
  [ 0x5c0d, 'CjkUnifiedIdeograph5c0d', [], 'Display', None],
  [ 0x5c0e, 'CjkUnifiedIdeograph5c0e', [], 'Display', None],
  [ 0x5c0f, 'CjkUnifiedIdeograph5c0f', [], 'Display', None],
  [ 0x5c10, 'CjkUnifiedIdeograph5c10', [], 'Display', None],
  [ 0x5c11, 'CjkUnifiedIdeograph5c11', [], 'Display', None],
  [ 0x5c12, 'CjkUnifiedIdeograph5c12', [], 'Display', None],
  [ 0x5c13, 'CjkUnifiedIdeograph5c13', [], 'Display', None],
  [ 0x5c14, 'CjkUnifiedIdeograph5c14', [], 'Display', None],
  [ 0x5c15, 'CjkUnifiedIdeograph5c15', [], 'Display', None],
  [ 0x5c16, 'CjkUnifiedIdeograph5c16', [], 'Display', None],
  [ 0x5c17, 'CjkUnifiedIdeograph5c17', [], 'Display', None],
  [ 0x5c18, 'CjkUnifiedIdeograph5c18', [], 'Display', None],
  [ 0x5c19, 'CjkUnifiedIdeograph5c19', [], 'Display', None],
  [ 0x5c1a, 'CjkUnifiedIdeograph5c1a', [], 'Display', None],
  [ 0x5c1b, 'CjkUnifiedIdeograph5c1b', [], 'Display', None],
  [ 0x5c1c, 'CjkUnifiedIdeograph5c1c', [], 'Display', None],
  [ 0x5c1d, 'CjkUnifiedIdeograph5c1d', [], 'Display', None],
  [ 0x5c1e, 'CjkUnifiedIdeograph5c1e', [], 'Display', None],
  [ 0x5c1f, 'CjkUnifiedIdeograph5c1f', [], 'Display', None],
  [ 0x5c20, 'CjkUnifiedIdeograph5c20', [], 'Display', None],
  [ 0x5c21, 'CjkUnifiedIdeograph5c21', [], 'Display', None],
  [ 0x5c22, 'CjkUnifiedIdeograph5c22', [], 'Display', None],
  [ 0x5c23, 'CjkUnifiedIdeograph5c23', [], 'Display', None],
  [ 0x5c24, 'CjkUnifiedIdeograph5c24', [], 'Display', None],
  [ 0x5c25, 'CjkUnifiedIdeograph5c25', [], 'Display', None],
  [ 0x5c26, 'CjkUnifiedIdeograph5c26', [], 'Display', None],
  [ 0x5c27, 'CjkUnifiedIdeograph5c27', [], 'Display', None],
  [ 0x5c28, 'CjkUnifiedIdeograph5c28', [], 'Display', None],
  [ 0x5c29, 'CjkUnifiedIdeograph5c29', [], 'Display', None],
  [ 0x5c2a, 'CjkUnifiedIdeograph5c2a', [], 'Display', None],
  [ 0x5c2b, 'CjkUnifiedIdeograph5c2b', [], 'Display', None],
  [ 0x5c2c, 'CjkUnifiedIdeograph5c2c', [], 'Display', None],
  [ 0x5c2d, 'CjkUnifiedIdeograph5c2d', [], 'Display', None],
  [ 0x5c2e, 'CjkUnifiedIdeograph5c2e', [], 'Display', None],
  [ 0x5c2f, 'CjkUnifiedIdeograph5c2f', [], 'Display', None],
  [ 0x5c30, 'CjkUnifiedIdeograph5c30', [], 'Display', None],
  [ 0x5c31, 'CjkUnifiedIdeograph5c31', [], 'Display', None],
  [ 0x5c32, 'CjkUnifiedIdeograph5c32', [], 'Display', None],
  [ 0x5c33, 'CjkUnifiedIdeograph5c33', [], 'Display', None],
  [ 0x5c34, 'CjkUnifiedIdeograph5c34', [], 'Display', None],
  [ 0x5c35, 'CjkUnifiedIdeograph5c35', [], 'Display', None],
  [ 0x5c36, 'CjkUnifiedIdeograph5c36', [], 'Display', None],
  [ 0x5c37, 'CjkUnifiedIdeograph5c37', [], 'Display', None],
  [ 0x5c38, 'CjkUnifiedIdeograph5c38', [], 'Display', None],
  [ 0x5c39, 'CjkUnifiedIdeograph5c39', [], 'Display', None],
  [ 0x5c3a, 'CjkUnifiedIdeograph5c3a', [], 'Display', None],
  [ 0x5c3b, 'CjkUnifiedIdeograph5c3b', [], 'Display', None],
  [ 0x5c3c, 'CjkUnifiedIdeograph5c3c', [], 'Display', None],
  [ 0x5c3d, 'CjkUnifiedIdeograph5c3d', [], 'Display', None],
  [ 0x5c3e, 'CjkUnifiedIdeograph5c3e', [], 'Display', None],
  [ 0x5c3f, 'CjkUnifiedIdeograph5c3f', [], 'Display', None],
  [ 0x5c40, 'CjkUnifiedIdeograph5c40', [], 'Display', None],
  [ 0x5c41, 'CjkUnifiedIdeograph5c41', [], 'Display', None],
  [ 0x5c42, 'CjkUnifiedIdeograph5c42', [], 'Display', None],
  [ 0x5c43, 'CjkUnifiedIdeograph5c43', [], 'Display', None],
  [ 0x5c44, 'CjkUnifiedIdeograph5c44', [], 'Display', None],
  [ 0x5c45, 'CjkUnifiedIdeograph5c45', [], 'Display', None],
  [ 0x5c46, 'CjkUnifiedIdeograph5c46', [], 'Display', None],
  [ 0x5c47, 'CjkUnifiedIdeograph5c47', [], 'Display', None],
  [ 0x5c48, 'CjkUnifiedIdeograph5c48', [], 'Display', None],
  [ 0x5c49, 'CjkUnifiedIdeograph5c49', [], 'Display', None],
  [ 0x5c4a, 'CjkUnifiedIdeograph5c4a', [], 'Display', None],
  [ 0x5c4b, 'CjkUnifiedIdeograph5c4b', [], 'Display', None],
  [ 0x5c4c, 'CjkUnifiedIdeograph5c4c', [], 'Display', None],
  [ 0x5c4d, 'CjkUnifiedIdeograph5c4d', [], 'Display', None],
  [ 0x5c4e, 'CjkUnifiedIdeograph5c4e', [], 'Display', None],
  [ 0x5c4f, 'CjkUnifiedIdeograph5c4f', [], 'Display', None],
  [ 0x5c50, 'CjkUnifiedIdeograph5c50', [], 'Display', None],
  [ 0x5c51, 'CjkUnifiedIdeograph5c51', [], 'Display', None],
  [ 0x5c52, 'CjkUnifiedIdeograph5c52', [], 'Display', None],
  [ 0x5c53, 'CjkUnifiedIdeograph5c53', [], 'Display', None],
  [ 0x5c54, 'CjkUnifiedIdeograph5c54', [], 'Display', None],
  [ 0x5c55, 'CjkUnifiedIdeograph5c55', [], 'Display', None],
  [ 0x5c56, 'CjkUnifiedIdeograph5c56', [], 'Display', None],
  [ 0x5c57, 'CjkUnifiedIdeograph5c57', [], 'Display', None],
  [ 0x5c58, 'CjkUnifiedIdeograph5c58', [], 'Display', None],
  [ 0x5c59, 'CjkUnifiedIdeograph5c59', [], 'Display', None],
  [ 0x5c5a, 'CjkUnifiedIdeograph5c5a', [], 'Display', None],
  [ 0x5c5b, 'CjkUnifiedIdeograph5c5b', [], 'Display', None],
  [ 0x5c5c, 'CjkUnifiedIdeograph5c5c', [], 'Display', None],
  [ 0x5c5d, 'CjkUnifiedIdeograph5c5d', [], 'Display', None],
  [ 0x5c5e, 'CjkUnifiedIdeograph5c5e', [], 'Display', None],
  [ 0x5c5f, 'CjkUnifiedIdeograph5c5f', [], 'Display', None],
  [ 0x5c60, 'CjkUnifiedIdeograph5c60', [], 'Display', None],
  [ 0x5c61, 'CjkUnifiedIdeograph5c61', [], 'Display', None],
  [ 0x5c62, 'CjkUnifiedIdeograph5c62', [], 'Display', None],
  [ 0x5c63, 'CjkUnifiedIdeograph5c63', [], 'Display', None],
  [ 0x5c64, 'CjkUnifiedIdeograph5c64', [], 'Display', None],
  [ 0x5c65, 'CjkUnifiedIdeograph5c65', [], 'Display', None],
  [ 0x5c66, 'CjkUnifiedIdeograph5c66', [], 'Display', None],
  [ 0x5c67, 'CjkUnifiedIdeograph5c67', [], 'Display', None],
  [ 0x5c68, 'CjkUnifiedIdeograph5c68', [], 'Display', None],
  [ 0x5c69, 'CjkUnifiedIdeograph5c69', [], 'Display', None],
  [ 0x5c6a, 'CjkUnifiedIdeograph5c6a', [], 'Display', None],
  [ 0x5c6b, 'CjkUnifiedIdeograph5c6b', [], 'Display', None],
  [ 0x5c6c, 'CjkUnifiedIdeograph5c6c', [], 'Display', None],
  [ 0x5c6d, 'CjkUnifiedIdeograph5c6d', [], 'Display', None],
  [ 0x5c6e, 'CjkUnifiedIdeograph5c6e', [], 'Display', None],
  [ 0x5c6f, 'CjkUnifiedIdeograph5c6f', [], 'Display', None],
  [ 0x5c70, 'CjkUnifiedIdeograph5c70', [], 'Display', None],
  [ 0x5c71, 'CjkUnifiedIdeograph5c71', [], 'Display', None],
  [ 0x5c72, 'CjkUnifiedIdeograph5c72', [], 'Display', None],
  [ 0x5c73, 'CjkUnifiedIdeograph5c73', [], 'Display', None],
  [ 0x5c74, 'CjkUnifiedIdeograph5c74', [], 'Display', None],
  [ 0x5c75, 'CjkUnifiedIdeograph5c75', [], 'Display', None],
  [ 0x5c76, 'CjkUnifiedIdeograph5c76', [], 'Display', None],
  [ 0x5c77, 'CjkUnifiedIdeograph5c77', [], 'Display', None],
  [ 0x5c78, 'CjkUnifiedIdeograph5c78', [], 'Display', None],
  [ 0x5c79, 'CjkUnifiedIdeograph5c79', [], 'Display', None],
  [ 0x5c7a, 'CjkUnifiedIdeograph5c7a', [], 'Display', None],
  [ 0x5c7b, 'CjkUnifiedIdeograph5c7b', [], 'Display', None],
  [ 0x5c7c, 'CjkUnifiedIdeograph5c7c', [], 'Display', None],
  [ 0x5c7d, 'CjkUnifiedIdeograph5c7d', [], 'Display', None],
  [ 0x5c7e, 'CjkUnifiedIdeograph5c7e', [], 'Display', None],
  [ 0x5c7f, 'CjkUnifiedIdeograph5c7f', [], 'Display', None],
  [ 0x5c80, 'CjkUnifiedIdeograph5c80', [], 'Display', None],
  [ 0x5c81, 'CjkUnifiedIdeograph5c81', [], 'Display', None],
  [ 0x5c82, 'CjkUnifiedIdeograph5c82', [], 'Display', None],
  [ 0x5c83, 'CjkUnifiedIdeograph5c83', [], 'Display', None],
  [ 0x5c84, 'CjkUnifiedIdeograph5c84', [], 'Display', None],
  [ 0x5c85, 'CjkUnifiedIdeograph5c85', [], 'Display', None],
  [ 0x5c86, 'CjkUnifiedIdeograph5c86', [], 'Display', None],
  [ 0x5c87, 'CjkUnifiedIdeograph5c87', [], 'Display', None],
  [ 0x5c88, 'CjkUnifiedIdeograph5c88', [], 'Display', None],
  [ 0x5c89, 'CjkUnifiedIdeograph5c89', [], 'Display', None],
  [ 0x5c8a, 'CjkUnifiedIdeograph5c8a', [], 'Display', None],
  [ 0x5c8b, 'CjkUnifiedIdeograph5c8b', [], 'Display', None],
  [ 0x5c8c, 'CjkUnifiedIdeograph5c8c', [], 'Display', None],
  [ 0x5c8d, 'CjkUnifiedIdeograph5c8d', [], 'Display', None],
  [ 0x5c8e, 'CjkUnifiedIdeograph5c8e', [], 'Display', None],
  [ 0x5c8f, 'CjkUnifiedIdeograph5c8f', [], 'Display', None],
  [ 0x5c90, 'CjkUnifiedIdeograph5c90', [], 'Display', None],
  [ 0x5c91, 'CjkUnifiedIdeograph5c91', [], 'Display', None],
  [ 0x5c92, 'CjkUnifiedIdeograph5c92', [], 'Display', None],
  [ 0x5c93, 'CjkUnifiedIdeograph5c93', [], 'Display', None],
  [ 0x5c94, 'CjkUnifiedIdeograph5c94', [], 'Display', None],
  [ 0x5c95, 'CjkUnifiedIdeograph5c95', [], 'Display', None],
  [ 0x5c96, 'CjkUnifiedIdeograph5c96', [], 'Display', None],
  [ 0x5c97, 'CjkUnifiedIdeograph5c97', [], 'Display', None],
  [ 0x5c98, 'CjkUnifiedIdeograph5c98', [], 'Display', None],
  [ 0x5c99, 'CjkUnifiedIdeograph5c99', [], 'Display', None],
  [ 0x5c9a, 'CjkUnifiedIdeograph5c9a', [], 'Display', None],
  [ 0x5c9b, 'CjkUnifiedIdeograph5c9b', [], 'Display', None],
  [ 0x5c9c, 'CjkUnifiedIdeograph5c9c', [], 'Display', None],
  [ 0x5c9d, 'CjkUnifiedIdeograph5c9d', [], 'Display', None],
  [ 0x5c9e, 'CjkUnifiedIdeograph5c9e', [], 'Display', None],
  [ 0x5c9f, 'CjkUnifiedIdeograph5c9f', [], 'Display', None],
  [ 0x5ca0, 'CjkUnifiedIdeograph5ca0', [], 'Display', None],
  [ 0x5ca1, 'CjkUnifiedIdeograph5ca1', [], 'Display', None],
  [ 0x5ca2, 'CjkUnifiedIdeograph5ca2', [], 'Display', None],
  [ 0x5ca3, 'CjkUnifiedIdeograph5ca3', [], 'Display', None],
  [ 0x5ca4, 'CjkUnifiedIdeograph5ca4', [], 'Display', None],
  [ 0x5ca5, 'CjkUnifiedIdeograph5ca5', [], 'Display', None],
  [ 0x5ca6, 'CjkUnifiedIdeograph5ca6', [], 'Display', None],
  [ 0x5ca7, 'CjkUnifiedIdeograph5ca7', [], 'Display', None],
  [ 0x5ca8, 'CjkUnifiedIdeograph5ca8', [], 'Display', None],
  [ 0x5ca9, 'CjkUnifiedIdeograph5ca9', [], 'Display', None],
  [ 0x5caa, 'CjkUnifiedIdeograph5caa', [], 'Display', None],
  [ 0x5cab, 'CjkUnifiedIdeograph5cab', [], 'Display', None],
  [ 0x5cac, 'CjkUnifiedIdeograph5cac', [], 'Display', None],
  [ 0x5cad, 'CjkUnifiedIdeograph5cad', [], 'Display', None],
  [ 0x5cae, 'CjkUnifiedIdeograph5cae', [], 'Display', None],
  [ 0x5caf, 'CjkUnifiedIdeograph5caf', [], 'Display', None],
  [ 0x5cb0, 'CjkUnifiedIdeograph5cb0', [], 'Display', None],
  [ 0x5cb1, 'CjkUnifiedIdeograph5cb1', [], 'Display', None],
  [ 0x5cb2, 'CjkUnifiedIdeograph5cb2', [], 'Display', None],
  [ 0x5cb3, 'CjkUnifiedIdeograph5cb3', [], 'Display', None],
  [ 0x5cb4, 'CjkUnifiedIdeograph5cb4', [], 'Display', None],
  [ 0x5cb5, 'CjkUnifiedIdeograph5cb5', [], 'Display', None],
  [ 0x5cb6, 'CjkUnifiedIdeograph5cb6', [], 'Display', None],
  [ 0x5cb7, 'CjkUnifiedIdeograph5cb7', [], 'Display', None],
  [ 0x5cb8, 'CjkUnifiedIdeograph5cb8', [], 'Display', None],
  [ 0x5cb9, 'CjkUnifiedIdeograph5cb9', [], 'Display', None],
  [ 0x5cba, 'CjkUnifiedIdeograph5cba', [], 'Display', None],
  [ 0x5cbb, 'CjkUnifiedIdeograph5cbb', [], 'Display', None],
  [ 0x5cbc, 'CjkUnifiedIdeograph5cbc', [], 'Display', None],
  [ 0x5cbd, 'CjkUnifiedIdeograph5cbd', [], 'Display', None],
  [ 0x5cbe, 'CjkUnifiedIdeograph5cbe', [], 'Display', None],
  [ 0x5cbf, 'CjkUnifiedIdeograph5cbf', [], 'Display', None],
  [ 0x5cc0, 'CjkUnifiedIdeograph5cc0', [], 'Display', None],
  [ 0x5cc1, 'CjkUnifiedIdeograph5cc1', [], 'Display', None],
  [ 0x5cc2, 'CjkUnifiedIdeograph5cc2', [], 'Display', None],
  [ 0x5cc3, 'CjkUnifiedIdeograph5cc3', [], 'Display', None],
  [ 0x5cc4, 'CjkUnifiedIdeograph5cc4', [], 'Display', None],
  [ 0x5cc5, 'CjkUnifiedIdeograph5cc5', [], 'Display', None],
  [ 0x5cc6, 'CjkUnifiedIdeograph5cc6', [], 'Display', None],
  [ 0x5cc7, 'CjkUnifiedIdeograph5cc7', [], 'Display', None],
  [ 0x5cc8, 'CjkUnifiedIdeograph5cc8', [], 'Display', None],
  [ 0x5cc9, 'CjkUnifiedIdeograph5cc9', [], 'Display', None],
  [ 0x5cca, 'CjkUnifiedIdeograph5cca', [], 'Display', None],
  [ 0x5ccb, 'CjkUnifiedIdeograph5ccb', [], 'Display', None],
  [ 0x5ccc, 'CjkUnifiedIdeograph5ccc', [], 'Display', None],
  [ 0x5ccd, 'CjkUnifiedIdeograph5ccd', [], 'Display', None],
  [ 0x5cce, 'CjkUnifiedIdeograph5cce', [], 'Display', None],
  [ 0x5ccf, 'CjkUnifiedIdeograph5ccf', [], 'Display', None],
  [ 0x5cd0, 'CjkUnifiedIdeograph5cd0', [], 'Display', None],
  [ 0x5cd1, 'CjkUnifiedIdeograph5cd1', [], 'Display', None],
  [ 0x5cd2, 'CjkUnifiedIdeograph5cd2', [], 'Display', None],
  [ 0x5cd3, 'CjkUnifiedIdeograph5cd3', [], 'Display', None],
  [ 0x5cd4, 'CjkUnifiedIdeograph5cd4', [], 'Display', None],
  [ 0x5cd5, 'CjkUnifiedIdeograph5cd5', [], 'Display', None],
  [ 0x5cd6, 'CjkUnifiedIdeograph5cd6', [], 'Display', None],
  [ 0x5cd7, 'CjkUnifiedIdeograph5cd7', [], 'Display', None],
  [ 0x5cd8, 'CjkUnifiedIdeograph5cd8', [], 'Display', None],
  [ 0x5cd9, 'CjkUnifiedIdeograph5cd9', [], 'Display', None],
  [ 0x5cda, 'CjkUnifiedIdeograph5cda', [], 'Display', None],
  [ 0x5cdb, 'CjkUnifiedIdeograph5cdb', [], 'Display', None],
  [ 0x5cdc, 'CjkUnifiedIdeograph5cdc', [], 'Display', None],
  [ 0x5cdd, 'CjkUnifiedIdeograph5cdd', [], 'Display', None],
  [ 0x5cde, 'CjkUnifiedIdeograph5cde', [], 'Display', None],
  [ 0x5cdf, 'CjkUnifiedIdeograph5cdf', [], 'Display', None],
  [ 0x5ce0, 'CjkUnifiedIdeograph5ce0', [], 'Display', None],
  [ 0x5ce1, 'CjkUnifiedIdeograph5ce1', [], 'Display', None],
  [ 0x5ce2, 'CjkUnifiedIdeograph5ce2', [], 'Display', None],
  [ 0x5ce3, 'CjkUnifiedIdeograph5ce3', [], 'Display', None],
  [ 0x5ce4, 'CjkUnifiedIdeograph5ce4', [], 'Display', None],
  [ 0x5ce5, 'CjkUnifiedIdeograph5ce5', [], 'Display', None],
  [ 0x5ce6, 'CjkUnifiedIdeograph5ce6', [], 'Display', None],
  [ 0x5ce7, 'CjkUnifiedIdeograph5ce7', [], 'Display', None],
  [ 0x5ce8, 'CjkUnifiedIdeograph5ce8', [], 'Display', None],
  [ 0x5ce9, 'CjkUnifiedIdeograph5ce9', [], 'Display', None],
  [ 0x5cea, 'CjkUnifiedIdeograph5cea', [], 'Display', None],
  [ 0x5ceb, 'CjkUnifiedIdeograph5ceb', [], 'Display', None],
  [ 0x5cec, 'CjkUnifiedIdeograph5cec', [], 'Display', None],
  [ 0x5ced, 'CjkUnifiedIdeograph5ced', [], 'Display', None],
  [ 0x5cee, 'CjkUnifiedIdeograph5cee', [], 'Display', None],
  [ 0x5cef, 'CjkUnifiedIdeograph5cef', [], 'Display', None],
  [ 0x5cf0, 'CjkUnifiedIdeograph5cf0', [], 'Display', None],
  [ 0x5cf1, 'CjkUnifiedIdeograph5cf1', [], 'Display', None],
  [ 0x5cf2, 'CjkUnifiedIdeograph5cf2', [], 'Display', None],
  [ 0x5cf3, 'CjkUnifiedIdeograph5cf3', [], 'Display', None],
  [ 0x5cf4, 'CjkUnifiedIdeograph5cf4', [], 'Display', None],
  [ 0x5cf5, 'CjkUnifiedIdeograph5cf5', [], 'Display', None],
  [ 0x5cf6, 'CjkUnifiedIdeograph5cf6', [], 'Display', None],
  [ 0x5cf7, 'CjkUnifiedIdeograph5cf7', [], 'Display', None],
  [ 0x5cf8, 'CjkUnifiedIdeograph5cf8', [], 'Display', None],
  [ 0x5cf9, 'CjkUnifiedIdeograph5cf9', [], 'Display', None],
  [ 0x5cfa, 'CjkUnifiedIdeograph5cfa', [], 'Display', None],
  [ 0x5cfb, 'CjkUnifiedIdeograph5cfb', [], 'Display', None],
  [ 0x5cfc, 'CjkUnifiedIdeograph5cfc', [], 'Display', None],
  [ 0x5cfd, 'CjkUnifiedIdeograph5cfd', [], 'Display', None],
  [ 0x5cfe, 'CjkUnifiedIdeograph5cfe', [], 'Display', None],
  [ 0x5cff, 'CjkUnifiedIdeograph5cff', [], 'Display', None],
  [ 0x5d00, 'CjkUnifiedIdeograph5d00', [], 'Display', None],
  [ 0x5d01, 'CjkUnifiedIdeograph5d01', [], 'Display', None],
  [ 0x5d02, 'CjkUnifiedIdeograph5d02', [], 'Display', None],
  [ 0x5d03, 'CjkUnifiedIdeograph5d03', [], 'Display', None],
  [ 0x5d04, 'CjkUnifiedIdeograph5d04', [], 'Display', None],
  [ 0x5d05, 'CjkUnifiedIdeograph5d05', [], 'Display', None],
  [ 0x5d06, 'CjkUnifiedIdeograph5d06', [], 'Display', None],
  [ 0x5d07, 'CjkUnifiedIdeograph5d07', [], 'Display', None],
  [ 0x5d08, 'CjkUnifiedIdeograph5d08', [], 'Display', None],
  [ 0x5d09, 'CjkUnifiedIdeograph5d09', [], 'Display', None],
  [ 0x5d0a, 'CjkUnifiedIdeograph5d0a', [], 'Display', None],
  [ 0x5d0b, 'CjkUnifiedIdeograph5d0b', [], 'Display', None],
  [ 0x5d0c, 'CjkUnifiedIdeograph5d0c', [], 'Display', None],
  [ 0x5d0d, 'CjkUnifiedIdeograph5d0d', [], 'Display', None],
  [ 0x5d0e, 'CjkUnifiedIdeograph5d0e', [], 'Display', None],
  [ 0x5d0f, 'CjkUnifiedIdeograph5d0f', [], 'Display', None],
  [ 0x5d10, 'CjkUnifiedIdeograph5d10', [], 'Display', None],
  [ 0x5d11, 'CjkUnifiedIdeograph5d11', [], 'Display', None],
  [ 0x5d12, 'CjkUnifiedIdeograph5d12', [], 'Display', None],
  [ 0x5d13, 'CjkUnifiedIdeograph5d13', [], 'Display', None],
  [ 0x5d14, 'CjkUnifiedIdeograph5d14', [], 'Display', None],
  [ 0x5d15, 'CjkUnifiedIdeograph5d15', [], 'Display', None],
  [ 0x5d16, 'CjkUnifiedIdeograph5d16', [], 'Display', None],
  [ 0x5d17, 'CjkUnifiedIdeograph5d17', [], 'Display', None],
  [ 0x5d18, 'CjkUnifiedIdeograph5d18', [], 'Display', None],
  [ 0x5d19, 'CjkUnifiedIdeograph5d19', [], 'Display', None],
  [ 0x5d1a, 'CjkUnifiedIdeograph5d1a', [], 'Display', None],
  [ 0x5d1b, 'CjkUnifiedIdeograph5d1b', [], 'Display', None],
  [ 0x5d1c, 'CjkUnifiedIdeograph5d1c', [], 'Display', None],
  [ 0x5d1d, 'CjkUnifiedIdeograph5d1d', [], 'Display', None],
  [ 0x5d1e, 'CjkUnifiedIdeograph5d1e', [], 'Display', None],
  [ 0x5d1f, 'CjkUnifiedIdeograph5d1f', [], 'Display', None],
  [ 0x5d20, 'CjkUnifiedIdeograph5d20', [], 'Display', None],
  [ 0x5d21, 'CjkUnifiedIdeograph5d21', [], 'Display', None],
  [ 0x5d22, 'CjkUnifiedIdeograph5d22', [], 'Display', None],
  [ 0x5d23, 'CjkUnifiedIdeograph5d23', [], 'Display', None],
  [ 0x5d24, 'CjkUnifiedIdeograph5d24', [], 'Display', None],
  [ 0x5d25, 'CjkUnifiedIdeograph5d25', [], 'Display', None],
  [ 0x5d26, 'CjkUnifiedIdeograph5d26', [], 'Display', None],
  [ 0x5d27, 'CjkUnifiedIdeograph5d27', [], 'Display', None],
  [ 0x5d28, 'CjkUnifiedIdeograph5d28', [], 'Display', None],
  [ 0x5d29, 'CjkUnifiedIdeograph5d29', [], 'Display', None],
  [ 0x5d2a, 'CjkUnifiedIdeograph5d2a', [], 'Display', None],
  [ 0x5d2b, 'CjkUnifiedIdeograph5d2b', [], 'Display', None],
  [ 0x5d2c, 'CjkUnifiedIdeograph5d2c', [], 'Display', None],
  [ 0x5d2d, 'CjkUnifiedIdeograph5d2d', [], 'Display', None],
  [ 0x5d2e, 'CjkUnifiedIdeograph5d2e', [], 'Display', None],
  [ 0x5d2f, 'CjkUnifiedIdeograph5d2f', [], 'Display', None],
  [ 0x5d30, 'CjkUnifiedIdeograph5d30', [], 'Display', None],
  [ 0x5d31, 'CjkUnifiedIdeograph5d31', [], 'Display', None],
  [ 0x5d32, 'CjkUnifiedIdeograph5d32', [], 'Display', None],
  [ 0x5d33, 'CjkUnifiedIdeograph5d33', [], 'Display', None],
  [ 0x5d34, 'CjkUnifiedIdeograph5d34', [], 'Display', None],
  [ 0x5d35, 'CjkUnifiedIdeograph5d35', [], 'Display', None],
  [ 0x5d36, 'CjkUnifiedIdeograph5d36', [], 'Display', None],
  [ 0x5d37, 'CjkUnifiedIdeograph5d37', [], 'Display', None],
  [ 0x5d38, 'CjkUnifiedIdeograph5d38', [], 'Display', None],
  [ 0x5d39, 'CjkUnifiedIdeograph5d39', [], 'Display', None],
  [ 0x5d3a, 'CjkUnifiedIdeograph5d3a', [], 'Display', None],
  [ 0x5d3b, 'CjkUnifiedIdeograph5d3b', [], 'Display', None],
  [ 0x5d3c, 'CjkUnifiedIdeograph5d3c', [], 'Display', None],
  [ 0x5d3d, 'CjkUnifiedIdeograph5d3d', [], 'Display', None],
  [ 0x5d3e, 'CjkUnifiedIdeograph5d3e', [], 'Display', None],
  [ 0x5d3f, 'CjkUnifiedIdeograph5d3f', [], 'Display', None],
  [ 0x5d40, 'CjkUnifiedIdeograph5d40', [], 'Display', None],
  [ 0x5d41, 'CjkUnifiedIdeograph5d41', [], 'Display', None],
  [ 0x5d42, 'CjkUnifiedIdeograph5d42', [], 'Display', None],
  [ 0x5d43, 'CjkUnifiedIdeograph5d43', [], 'Display', None],
  [ 0x5d44, 'CjkUnifiedIdeograph5d44', [], 'Display', None],
  [ 0x5d45, 'CjkUnifiedIdeograph5d45', [], 'Display', None],
  [ 0x5d46, 'CjkUnifiedIdeograph5d46', [], 'Display', None],
  [ 0x5d47, 'CjkUnifiedIdeograph5d47', [], 'Display', None],
  [ 0x5d48, 'CjkUnifiedIdeograph5d48', [], 'Display', None],
  [ 0x5d49, 'CjkUnifiedIdeograph5d49', [], 'Display', None],
  [ 0x5d4a, 'CjkUnifiedIdeograph5d4a', [], 'Display', None],
  [ 0x5d4b, 'CjkUnifiedIdeograph5d4b', [], 'Display', None],
  [ 0x5d4c, 'CjkUnifiedIdeograph5d4c', [], 'Display', None],
  [ 0x5d4d, 'CjkUnifiedIdeograph5d4d', [], 'Display', None],
  [ 0x5d4e, 'CjkUnifiedIdeograph5d4e', [], 'Display', None],
  [ 0x5d4f, 'CjkUnifiedIdeograph5d4f', [], 'Display', None],
  [ 0x5d50, 'CjkUnifiedIdeograph5d50', [], 'Display', None],
  [ 0x5d51, 'CjkUnifiedIdeograph5d51', [], 'Display', None],
  [ 0x5d52, 'CjkUnifiedIdeograph5d52', [], 'Display', None],
  [ 0x5d53, 'CjkUnifiedIdeograph5d53', [], 'Display', None],
  [ 0x5d54, 'CjkUnifiedIdeograph5d54', [], 'Display', None],
  [ 0x5d55, 'CjkUnifiedIdeograph5d55', [], 'Display', None],
  [ 0x5d56, 'CjkUnifiedIdeograph5d56', [], 'Display', None],
  [ 0x5d57, 'CjkUnifiedIdeograph5d57', [], 'Display', None],
  [ 0x5d58, 'CjkUnifiedIdeograph5d58', [], 'Display', None],
  [ 0x5d59, 'CjkUnifiedIdeograph5d59', [], 'Display', None],
  [ 0x5d5a, 'CjkUnifiedIdeograph5d5a', [], 'Display', None],
  [ 0x5d5b, 'CjkUnifiedIdeograph5d5b', [], 'Display', None],
  [ 0x5d5c, 'CjkUnifiedIdeograph5d5c', [], 'Display', None],
  [ 0x5d5d, 'CjkUnifiedIdeograph5d5d', [], 'Display', None],
  [ 0x5d5e, 'CjkUnifiedIdeograph5d5e', [], 'Display', None],
  [ 0x5d5f, 'CjkUnifiedIdeograph5d5f', [], 'Display', None],
  [ 0x5d60, 'CjkUnifiedIdeograph5d60', [], 'Display', None],
  [ 0x5d61, 'CjkUnifiedIdeograph5d61', [], 'Display', None],
  [ 0x5d62, 'CjkUnifiedIdeograph5d62', [], 'Display', None],
  [ 0x5d63, 'CjkUnifiedIdeograph5d63', [], 'Display', None],
  [ 0x5d64, 'CjkUnifiedIdeograph5d64', [], 'Display', None],
  [ 0x5d65, 'CjkUnifiedIdeograph5d65', [], 'Display', None],
  [ 0x5d66, 'CjkUnifiedIdeograph5d66', [], 'Display', None],
  [ 0x5d67, 'CjkUnifiedIdeograph5d67', [], 'Display', None],
  [ 0x5d68, 'CjkUnifiedIdeograph5d68', [], 'Display', None],
  [ 0x5d69, 'CjkUnifiedIdeograph5d69', [], 'Display', None],
  [ 0x5d6a, 'CjkUnifiedIdeograph5d6a', [], 'Display', None],
  [ 0x5d6b, 'CjkUnifiedIdeograph5d6b', [], 'Display', None],
  [ 0x5d6c, 'CjkUnifiedIdeograph5d6c', [], 'Display', None],
  [ 0x5d6d, 'CjkUnifiedIdeograph5d6d', [], 'Display', None],
  [ 0x5d6e, 'CjkUnifiedIdeograph5d6e', [], 'Display', None],
  [ 0x5d6f, 'CjkUnifiedIdeograph5d6f', [], 'Display', None],
  [ 0x5d70, 'CjkUnifiedIdeograph5d70', [], 'Display', None],
  [ 0x5d71, 'CjkUnifiedIdeograph5d71', [], 'Display', None],
  [ 0x5d72, 'CjkUnifiedIdeograph5d72', [], 'Display', None],
  [ 0x5d73, 'CjkUnifiedIdeograph5d73', [], 'Display', None],
  [ 0x5d74, 'CjkUnifiedIdeograph5d74', [], 'Display', None],
  [ 0x5d75, 'CjkUnifiedIdeograph5d75', [], 'Display', None],
  [ 0x5d76, 'CjkUnifiedIdeograph5d76', [], 'Display', None],
  [ 0x5d77, 'CjkUnifiedIdeograph5d77', [], 'Display', None],
  [ 0x5d78, 'CjkUnifiedIdeograph5d78', [], 'Display', None],
  [ 0x5d79, 'CjkUnifiedIdeograph5d79', [], 'Display', None],
  [ 0x5d7a, 'CjkUnifiedIdeograph5d7a', [], 'Display', None],
  [ 0x5d7b, 'CjkUnifiedIdeograph5d7b', [], 'Display', None],
  [ 0x5d7c, 'CjkUnifiedIdeograph5d7c', [], 'Display', None],
  [ 0x5d7d, 'CjkUnifiedIdeograph5d7d', [], 'Display', None],
  [ 0x5d7e, 'CjkUnifiedIdeograph5d7e', [], 'Display', None],
  [ 0x5d7f, 'CjkUnifiedIdeograph5d7f', [], 'Display', None],
  [ 0x5d80, 'CjkUnifiedIdeograph5d80', [], 'Display', None],
  [ 0x5d81, 'CjkUnifiedIdeograph5d81', [], 'Display', None],
  [ 0x5d82, 'CjkUnifiedIdeograph5d82', [], 'Display', None],
  [ 0x5d83, 'CjkUnifiedIdeograph5d83', [], 'Display', None],
  [ 0x5d84, 'CjkUnifiedIdeograph5d84', [], 'Display', None],
  [ 0x5d85, 'CjkUnifiedIdeograph5d85', [], 'Display', None],
  [ 0x5d86, 'CjkUnifiedIdeograph5d86', [], 'Display', None],
  [ 0x5d87, 'CjkUnifiedIdeograph5d87', [], 'Display', None],
  [ 0x5d88, 'CjkUnifiedIdeograph5d88', [], 'Display', None],
  [ 0x5d89, 'CjkUnifiedIdeograph5d89', [], 'Display', None],
  [ 0x5d8a, 'CjkUnifiedIdeograph5d8a', [], 'Display', None],
  [ 0x5d8b, 'CjkUnifiedIdeograph5d8b', [], 'Display', None],
  [ 0x5d8c, 'CjkUnifiedIdeograph5d8c', [], 'Display', None],
  [ 0x5d8d, 'CjkUnifiedIdeograph5d8d', [], 'Display', None],
  [ 0x5d8e, 'CjkUnifiedIdeograph5d8e', [], 'Display', None],
  [ 0x5d8f, 'CjkUnifiedIdeograph5d8f', [], 'Display', None],
  [ 0x5d90, 'CjkUnifiedIdeograph5d90', [], 'Display', None],
  [ 0x5d91, 'CjkUnifiedIdeograph5d91', [], 'Display', None],
  [ 0x5d92, 'CjkUnifiedIdeograph5d92', [], 'Display', None],
  [ 0x5d93, 'CjkUnifiedIdeograph5d93', [], 'Display', None],
  [ 0x5d94, 'CjkUnifiedIdeograph5d94', [], 'Display', None],
  [ 0x5d95, 'CjkUnifiedIdeograph5d95', [], 'Display', None],
  [ 0x5d96, 'CjkUnifiedIdeograph5d96', [], 'Display', None],
  [ 0x5d97, 'CjkUnifiedIdeograph5d97', [], 'Display', None],
  [ 0x5d98, 'CjkUnifiedIdeograph5d98', [], 'Display', None],
  [ 0x5d99, 'CjkUnifiedIdeograph5d99', [], 'Display', None],
  [ 0x5d9a, 'CjkUnifiedIdeograph5d9a', [], 'Display', None],
  [ 0x5d9b, 'CjkUnifiedIdeograph5d9b', [], 'Display', None],
  [ 0x5d9c, 'CjkUnifiedIdeograph5d9c', [], 'Display', None],
  [ 0x5d9d, 'CjkUnifiedIdeograph5d9d', [], 'Display', None],
  [ 0x5d9e, 'CjkUnifiedIdeograph5d9e', [], 'Display', None],
  [ 0x5d9f, 'CjkUnifiedIdeograph5d9f', [], 'Display', None],
  [ 0x5da0, 'CjkUnifiedIdeograph5da0', [], 'Display', None],
  [ 0x5da1, 'CjkUnifiedIdeograph5da1', [], 'Display', None],
  [ 0x5da2, 'CjkUnifiedIdeograph5da2', [], 'Display', None],
  [ 0x5da3, 'CjkUnifiedIdeograph5da3', [], 'Display', None],
  [ 0x5da4, 'CjkUnifiedIdeograph5da4', [], 'Display', None],
  [ 0x5da5, 'CjkUnifiedIdeograph5da5', [], 'Display', None],
  [ 0x5da6, 'CjkUnifiedIdeograph5da6', [], 'Display', None],
  [ 0x5da7, 'CjkUnifiedIdeograph5da7', [], 'Display', None],
  [ 0x5da8, 'CjkUnifiedIdeograph5da8', [], 'Display', None],
  [ 0x5da9, 'CjkUnifiedIdeograph5da9', [], 'Display', None],
  [ 0x5daa, 'CjkUnifiedIdeograph5daa', [], 'Display', None],
  [ 0x5dab, 'CjkUnifiedIdeograph5dab', [], 'Display', None],
  [ 0x5dac, 'CjkUnifiedIdeograph5dac', [], 'Display', None],
  [ 0x5dad, 'CjkUnifiedIdeograph5dad', [], 'Display', None],
  [ 0x5dae, 'CjkUnifiedIdeograph5dae', [], 'Display', None],
  [ 0x5daf, 'CjkUnifiedIdeograph5daf', [], 'Display', None],
  [ 0x5db0, 'CjkUnifiedIdeograph5db0', [], 'Display', None],
  [ 0x5db1, 'CjkUnifiedIdeograph5db1', [], 'Display', None],
  [ 0x5db2, 'CjkUnifiedIdeograph5db2', [], 'Display', None],
  [ 0x5db3, 'CjkUnifiedIdeograph5db3', [], 'Display', None],
  [ 0x5db4, 'CjkUnifiedIdeograph5db4', [], 'Display', None],
  [ 0x5db5, 'CjkUnifiedIdeograph5db5', [], 'Display', None],
  [ 0x5db6, 'CjkUnifiedIdeograph5db6', [], 'Display', None],
  [ 0x5db7, 'CjkUnifiedIdeograph5db7', [], 'Display', None],
  [ 0x5db8, 'CjkUnifiedIdeograph5db8', [], 'Display', None],
  [ 0x5db9, 'CjkUnifiedIdeograph5db9', [], 'Display', None],
  [ 0x5dba, 'CjkUnifiedIdeograph5dba', [], 'Display', None],
  [ 0x5dbb, 'CjkUnifiedIdeograph5dbb', [], 'Display', None],
  [ 0x5dbc, 'CjkUnifiedIdeograph5dbc', [], 'Display', None],
  [ 0x5dbd, 'CjkUnifiedIdeograph5dbd', [], 'Display', None],
  [ 0x5dbe, 'CjkUnifiedIdeograph5dbe', [], 'Display', None],
  [ 0x5dbf, 'CjkUnifiedIdeograph5dbf', [], 'Display', None],
  [ 0x5dc0, 'CjkUnifiedIdeograph5dc0', [], 'Display', None],
  [ 0x5dc1, 'CjkUnifiedIdeograph5dc1', [], 'Display', None],
  [ 0x5dc2, 'CjkUnifiedIdeograph5dc2', [], 'Display', None],
  [ 0x5dc3, 'CjkUnifiedIdeograph5dc3', [], 'Display', None],
  [ 0x5dc4, 'CjkUnifiedIdeograph5dc4', [], 'Display', None],
  [ 0x5dc5, 'CjkUnifiedIdeograph5dc5', [], 'Display', None],
  [ 0x5dc6, 'CjkUnifiedIdeograph5dc6', [], 'Display', None],
  [ 0x5dc7, 'CjkUnifiedIdeograph5dc7', [], 'Display', None],
  [ 0x5dc8, 'CjkUnifiedIdeograph5dc8', [], 'Display', None],
  [ 0x5dc9, 'CjkUnifiedIdeograph5dc9', [], 'Display', None],
  [ 0x5dca, 'CjkUnifiedIdeograph5dca', [], 'Display', None],
  [ 0x5dcb, 'CjkUnifiedIdeograph5dcb', [], 'Display', None],
  [ 0x5dcc, 'CjkUnifiedIdeograph5dcc', [], 'Display', None],
  [ 0x5dcd, 'CjkUnifiedIdeograph5dcd', [], 'Display', None],
  [ 0x5dce, 'CjkUnifiedIdeograph5dce', [], 'Display', None],
  [ 0x5dcf, 'CjkUnifiedIdeograph5dcf', [], 'Display', None],
  [ 0x5dd0, 'CjkUnifiedIdeograph5dd0', [], 'Display', None],
  [ 0x5dd1, 'CjkUnifiedIdeograph5dd1', [], 'Display', None],
  [ 0x5dd2, 'CjkUnifiedIdeograph5dd2', [], 'Display', None],
  [ 0x5dd3, 'CjkUnifiedIdeograph5dd3', [], 'Display', None],
  [ 0x5dd4, 'CjkUnifiedIdeograph5dd4', [], 'Display', None],
  [ 0x5dd5, 'CjkUnifiedIdeograph5dd5', [], 'Display', None],
  [ 0x5dd6, 'CjkUnifiedIdeograph5dd6', [], 'Display', None],
  [ 0x5dd7, 'CjkUnifiedIdeograph5dd7', [], 'Display', None],
  [ 0x5dd8, 'CjkUnifiedIdeograph5dd8', [], 'Display', None],
  [ 0x5dd9, 'CjkUnifiedIdeograph5dd9', [], 'Display', None],
  [ 0x5dda, 'CjkUnifiedIdeograph5dda', [], 'Display', None],
  [ 0x5ddb, 'CjkUnifiedIdeograph5ddb', [], 'Display', None],
  [ 0x5ddc, 'CjkUnifiedIdeograph5ddc', [], 'Display', None],
  [ 0x5ddd, 'CjkUnifiedIdeograph5ddd', [], 'Display', None],
  [ 0x5dde, 'CjkUnifiedIdeograph5dde', [], 'Display', None],
  [ 0x5ddf, 'CjkUnifiedIdeograph5ddf', [], 'Display', None],
  [ 0x5de0, 'CjkUnifiedIdeograph5de0', [], 'Display', None],
  [ 0x5de1, 'CjkUnifiedIdeograph5de1', [], 'Display', None],
  [ 0x5de2, 'CjkUnifiedIdeograph5de2', [], 'Display', None],
  [ 0x5de3, 'CjkUnifiedIdeograph5de3', [], 'Display', None],
  [ 0x5de4, 'CjkUnifiedIdeograph5de4', [], 'Display', None],
  [ 0x5de5, 'CjkUnifiedIdeograph5de5', [], 'Display', None],
  [ 0x5de6, 'CjkUnifiedIdeograph5de6', [], 'Display', None],
  [ 0x5de7, 'CjkUnifiedIdeograph5de7', [], 'Display', None],
  [ 0x5de8, 'CjkUnifiedIdeograph5de8', [], 'Display', None],
  [ 0x5de9, 'CjkUnifiedIdeograph5de9', [], 'Display', None],
  [ 0x5dea, 'CjkUnifiedIdeograph5dea', [], 'Display', None],
  [ 0x5deb, 'CjkUnifiedIdeograph5deb', [], 'Display', None],
  [ 0x5dec, 'CjkUnifiedIdeograph5dec', [], 'Display', None],
  [ 0x5ded, 'CjkUnifiedIdeograph5ded', [], 'Display', None],
  [ 0x5dee, 'CjkUnifiedIdeograph5dee', [], 'Display', None],
  [ 0x5def, 'CjkUnifiedIdeograph5def', [], 'Display', None],
  [ 0x5df0, 'CjkUnifiedIdeograph5df0', [], 'Display', None],
  [ 0x5df1, 'CjkUnifiedIdeograph5df1', [], 'Display', None],
  [ 0x5df2, 'CjkUnifiedIdeograph5df2', [], 'Display', None],
  [ 0x5df3, 'CjkUnifiedIdeograph5df3', [], 'Display', None],
  [ 0x5df4, 'CjkUnifiedIdeograph5df4', [], 'Display', None],
  [ 0x5df5, 'CjkUnifiedIdeograph5df5', [], 'Display', None],
  [ 0x5df6, 'CjkUnifiedIdeograph5df6', [], 'Display', None],
  [ 0x5df7, 'CjkUnifiedIdeograph5df7', [], 'Display', None],
  [ 0x5df8, 'CjkUnifiedIdeograph5df8', [], 'Display', None],
  [ 0x5df9, 'CjkUnifiedIdeograph5df9', [], 'Display', None],
  [ 0x5dfa, 'CjkUnifiedIdeograph5dfa', [], 'Display', None],
  [ 0x5dfb, 'CjkUnifiedIdeograph5dfb', [], 'Display', None],
  [ 0x5dfc, 'CjkUnifiedIdeograph5dfc', [], 'Display', None],
  [ 0x5dfd, 'CjkUnifiedIdeograph5dfd', [], 'Display', None],
  [ 0x5dfe, 'CjkUnifiedIdeograph5dfe', [], 'Display', None],
  [ 0x5dff, 'CjkUnifiedIdeograph5dff', [], 'Display', None],
  [ 0x5e00, 'CjkUnifiedIdeograph5e00', [], 'Display', None],
  [ 0x5e01, 'CjkUnifiedIdeograph5e01', [], 'Display', None],
  [ 0x5e02, 'CjkUnifiedIdeograph5e02', [], 'Display', None],
  [ 0x5e03, 'CjkUnifiedIdeograph5e03', [], 'Display', None],
  [ 0x5e04, 'CjkUnifiedIdeograph5e04', [], 'Display', None],
  [ 0x5e05, 'CjkUnifiedIdeograph5e05', [], 'Display', None],
  [ 0x5e06, 'CjkUnifiedIdeograph5e06', [], 'Display', None],
  [ 0x5e07, 'CjkUnifiedIdeograph5e07', [], 'Display', None],
  [ 0x5e08, 'CjkUnifiedIdeograph5e08', [], 'Display', None],
  [ 0x5e09, 'CjkUnifiedIdeograph5e09', [], 'Display', None],
  [ 0x5e0a, 'CjkUnifiedIdeograph5e0a', [], 'Display', None],
  [ 0x5e0b, 'CjkUnifiedIdeograph5e0b', [], 'Display', None],
  [ 0x5e0c, 'CjkUnifiedIdeograph5e0c', [], 'Display', None],
  [ 0x5e0d, 'CjkUnifiedIdeograph5e0d', [], 'Display', None],
  [ 0x5e0e, 'CjkUnifiedIdeograph5e0e', [], 'Display', None],
  [ 0x5e0f, 'CjkUnifiedIdeograph5e0f', [], 'Display', None],
  [ 0x5e10, 'CjkUnifiedIdeograph5e10', [], 'Display', None],
  [ 0x5e11, 'CjkUnifiedIdeograph5e11', [], 'Display', None],
  [ 0x5e12, 'CjkUnifiedIdeograph5e12', [], 'Display', None],
  [ 0x5e13, 'CjkUnifiedIdeograph5e13', [], 'Display', None],
  [ 0x5e14, 'CjkUnifiedIdeograph5e14', [], 'Display', None],
  [ 0x5e15, 'CjkUnifiedIdeograph5e15', [], 'Display', None],
  [ 0x5e16, 'CjkUnifiedIdeograph5e16', [], 'Display', None],
  [ 0x5e17, 'CjkUnifiedIdeograph5e17', [], 'Display', None],
  [ 0x5e18, 'CjkUnifiedIdeograph5e18', [], 'Display', None],
  [ 0x5e19, 'CjkUnifiedIdeograph5e19', [], 'Display', None],
  [ 0x5e1a, 'CjkUnifiedIdeograph5e1a', [], 'Display', None],
  [ 0x5e1b, 'CjkUnifiedIdeograph5e1b', [], 'Display', None],
  [ 0x5e1c, 'CjkUnifiedIdeograph5e1c', [], 'Display', None],
  [ 0x5e1d, 'CjkUnifiedIdeograph5e1d', [], 'Display', None],
  [ 0x5e1e, 'CjkUnifiedIdeograph5e1e', [], 'Display', None],
  [ 0x5e1f, 'CjkUnifiedIdeograph5e1f', [], 'Display', None],
  [ 0x5e20, 'CjkUnifiedIdeograph5e20', [], 'Display', None],
  [ 0x5e21, 'CjkUnifiedIdeograph5e21', [], 'Display', None],
  [ 0x5e22, 'CjkUnifiedIdeograph5e22', [], 'Display', None],
  [ 0x5e23, 'CjkUnifiedIdeograph5e23', [], 'Display', None],
  [ 0x5e24, 'CjkUnifiedIdeograph5e24', [], 'Display', None],
  [ 0x5e25, 'CjkUnifiedIdeograph5e25', [], 'Display', None],
  [ 0x5e26, 'CjkUnifiedIdeograph5e26', [], 'Display', None],
  [ 0x5e27, 'CjkUnifiedIdeograph5e27', [], 'Display', None],
  [ 0x5e28, 'CjkUnifiedIdeograph5e28', [], 'Display', None],
  [ 0x5e29, 'CjkUnifiedIdeograph5e29', [], 'Display', None],
  [ 0x5e2a, 'CjkUnifiedIdeograph5e2a', [], 'Display', None],
  [ 0x5e2b, 'CjkUnifiedIdeograph5e2b', [], 'Display', None],
  [ 0x5e2c, 'CjkUnifiedIdeograph5e2c', [], 'Display', None],
  [ 0x5e2d, 'CjkUnifiedIdeograph5e2d', [], 'Display', None],
  [ 0x5e2e, 'CjkUnifiedIdeograph5e2e', [], 'Display', None],
  [ 0x5e2f, 'CjkUnifiedIdeograph5e2f', [], 'Display', None],
  [ 0x5e30, 'CjkUnifiedIdeograph5e30', [], 'Display', None],
  [ 0x5e31, 'CjkUnifiedIdeograph5e31', [], 'Display', None],
  [ 0x5e32, 'CjkUnifiedIdeograph5e32', [], 'Display', None],
  [ 0x5e33, 'CjkUnifiedIdeograph5e33', [], 'Display', None],
  [ 0x5e34, 'CjkUnifiedIdeograph5e34', [], 'Display', None],
  [ 0x5e35, 'CjkUnifiedIdeograph5e35', [], 'Display', None],
  [ 0x5e36, 'CjkUnifiedIdeograph5e36', [], 'Display', None],
  [ 0x5e37, 'CjkUnifiedIdeograph5e37', [], 'Display', None],
  [ 0x5e38, 'CjkUnifiedIdeograph5e38', [], 'Display', None],
  [ 0x5e39, 'CjkUnifiedIdeograph5e39', [], 'Display', None],
  [ 0x5e3a, 'CjkUnifiedIdeograph5e3a', [], 'Display', None],
  [ 0x5e3b, 'CjkUnifiedIdeograph5e3b', [], 'Display', None],
  [ 0x5e3c, 'CjkUnifiedIdeograph5e3c', [], 'Display', None],
  [ 0x5e3d, 'CjkUnifiedIdeograph5e3d', [], 'Display', None],
  [ 0x5e3e, 'CjkUnifiedIdeograph5e3e', [], 'Display', None],
  [ 0x5e3f, 'CjkUnifiedIdeograph5e3f', [], 'Display', None],
  [ 0x5e40, 'CjkUnifiedIdeograph5e40', [], 'Display', None],
  [ 0x5e41, 'CjkUnifiedIdeograph5e41', [], 'Display', None],
  [ 0x5e42, 'CjkUnifiedIdeograph5e42', [], 'Display', None],
  [ 0x5e43, 'CjkUnifiedIdeograph5e43', [], 'Display', None],
  [ 0x5e44, 'CjkUnifiedIdeograph5e44', [], 'Display', None],
  [ 0x5e45, 'CjkUnifiedIdeograph5e45', [], 'Display', None],
  [ 0x5e46, 'CjkUnifiedIdeograph5e46', [], 'Display', None],
  [ 0x5e47, 'CjkUnifiedIdeograph5e47', [], 'Display', None],
  [ 0x5e48, 'CjkUnifiedIdeograph5e48', [], 'Display', None],
  [ 0x5e49, 'CjkUnifiedIdeograph5e49', [], 'Display', None],
  [ 0x5e4a, 'CjkUnifiedIdeograph5e4a', [], 'Display', None],
  [ 0x5e4b, 'CjkUnifiedIdeograph5e4b', [], 'Display', None],
  [ 0x5e4c, 'CjkUnifiedIdeograph5e4c', [], 'Display', None],
  [ 0x5e4d, 'CjkUnifiedIdeograph5e4d', [], 'Display', None],
  [ 0x5e4e, 'CjkUnifiedIdeograph5e4e', [], 'Display', None],
  [ 0x5e4f, 'CjkUnifiedIdeograph5e4f', [], 'Display', None],
  [ 0x5e50, 'CjkUnifiedIdeograph5e50', [], 'Display', None],
  [ 0x5e51, 'CjkUnifiedIdeograph5e51', [], 'Display', None],
  [ 0x5e52, 'CjkUnifiedIdeograph5e52', [], 'Display', None],
  [ 0x5e53, 'CjkUnifiedIdeograph5e53', [], 'Display', None],
  [ 0x5e54, 'CjkUnifiedIdeograph5e54', [], 'Display', None],
  [ 0x5e55, 'CjkUnifiedIdeograph5e55', [], 'Display', None],
  [ 0x5e56, 'CjkUnifiedIdeograph5e56', [], 'Display', None],
  [ 0x5e57, 'CjkUnifiedIdeograph5e57', [], 'Display', None],
  [ 0x5e58, 'CjkUnifiedIdeograph5e58', [], 'Display', None],
  [ 0x5e59, 'CjkUnifiedIdeograph5e59', [], 'Display', None],
  [ 0x5e5a, 'CjkUnifiedIdeograph5e5a', [], 'Display', None],
  [ 0x5e5b, 'CjkUnifiedIdeograph5e5b', [], 'Display', None],
  [ 0x5e5c, 'CjkUnifiedIdeograph5e5c', [], 'Display', None],
  [ 0x5e5d, 'CjkUnifiedIdeograph5e5d', [], 'Display', None],
  [ 0x5e5e, 'CjkUnifiedIdeograph5e5e', [], 'Display', None],
  [ 0x5e5f, 'CjkUnifiedIdeograph5e5f', [], 'Display', None],
  [ 0x5e60, 'CjkUnifiedIdeograph5e60', [], 'Display', None],
  [ 0x5e61, 'CjkUnifiedIdeograph5e61', [], 'Display', None],
  [ 0x5e62, 'CjkUnifiedIdeograph5e62', [], 'Display', None],
  [ 0x5e63, 'CjkUnifiedIdeograph5e63', [], 'Display', None],
  [ 0x5e64, 'CjkUnifiedIdeograph5e64', [], 'Display', None],
  [ 0x5e65, 'CjkUnifiedIdeograph5e65', [], 'Display', None],
  [ 0x5e66, 'CjkUnifiedIdeograph5e66', [], 'Display', None],
  [ 0x5e67, 'CjkUnifiedIdeograph5e67', [], 'Display', None],
  [ 0x5e68, 'CjkUnifiedIdeograph5e68', [], 'Display', None],
  [ 0x5e69, 'CjkUnifiedIdeograph5e69', [], 'Display', None],
  [ 0x5e6a, 'CjkUnifiedIdeograph5e6a', [], 'Display', None],
  [ 0x5e6b, 'CjkUnifiedIdeograph5e6b', [], 'Display', None],
  [ 0x5e6c, 'CjkUnifiedIdeograph5e6c', [], 'Display', None],
  [ 0x5e6d, 'CjkUnifiedIdeograph5e6d', [], 'Display', None],
  [ 0x5e6e, 'CjkUnifiedIdeograph5e6e', [], 'Display', None],
  [ 0x5e6f, 'CjkUnifiedIdeograph5e6f', [], 'Display', None],
  [ 0x5e70, 'CjkUnifiedIdeograph5e70', [], 'Display', None],
  [ 0x5e71, 'CjkUnifiedIdeograph5e71', [], 'Display', None],
  [ 0x5e72, 'CjkUnifiedIdeograph5e72', [], 'Display', None],
  [ 0x5e73, 'CjkUnifiedIdeograph5e73', [], 'Display', None],
  [ 0x5e74, 'CjkUnifiedIdeograph5e74', [], 'Display', None],
  [ 0x5e75, 'CjkUnifiedIdeograph5e75', [], 'Display', None],
  [ 0x5e76, 'CjkUnifiedIdeograph5e76', [], 'Display', None],
  [ 0x5e77, 'CjkUnifiedIdeograph5e77', [], 'Display', None],
  [ 0x5e78, 'CjkUnifiedIdeograph5e78', [], 'Display', None],
  [ 0x5e79, 'CjkUnifiedIdeograph5e79', [], 'Display', None],
  [ 0x5e7a, 'CjkUnifiedIdeograph5e7a', [], 'Display', None],
  [ 0x5e7b, 'CjkUnifiedIdeograph5e7b', [], 'Display', None],
  [ 0x5e7c, 'CjkUnifiedIdeograph5e7c', [], 'Display', None],
  [ 0x5e7d, 'CjkUnifiedIdeograph5e7d', [], 'Display', None],
  [ 0x5e7e, 'CjkUnifiedIdeograph5e7e', [], 'Display', None],
  [ 0x5e7f, 'CjkUnifiedIdeograph5e7f', [], 'Display', None],
  [ 0x5e80, 'CjkUnifiedIdeograph5e80', [], 'Display', None],
  [ 0x5e81, 'CjkUnifiedIdeograph5e81', [], 'Display', None],
  [ 0x5e82, 'CjkUnifiedIdeograph5e82', [], 'Display', None],
  [ 0x5e83, 'CjkUnifiedIdeograph5e83', [], 'Display', None],
  [ 0x5e84, 'CjkUnifiedIdeograph5e84', [], 'Display', None],
  [ 0x5e85, 'CjkUnifiedIdeograph5e85', [], 'Display', None],
  [ 0x5e86, 'CjkUnifiedIdeograph5e86', [], 'Display', None],
  [ 0x5e87, 'CjkUnifiedIdeograph5e87', [], 'Display', None],
  [ 0x5e88, 'CjkUnifiedIdeograph5e88', [], 'Display', None],
  [ 0x5e89, 'CjkUnifiedIdeograph5e89', [], 'Display', None],
  [ 0x5e8a, 'CjkUnifiedIdeograph5e8a', [], 'Display', None],
  [ 0x5e8b, 'CjkUnifiedIdeograph5e8b', [], 'Display', None],
  [ 0x5e8c, 'CjkUnifiedIdeograph5e8c', [], 'Display', None],
  [ 0x5e8d, 'CjkUnifiedIdeograph5e8d', [], 'Display', None],
  [ 0x5e8e, 'CjkUnifiedIdeograph5e8e', [], 'Display', None],
  [ 0x5e8f, 'CjkUnifiedIdeograph5e8f', [], 'Display', None],
  [ 0x5e90, 'CjkUnifiedIdeograph5e90', [], 'Display', None],
  [ 0x5e91, 'CjkUnifiedIdeograph5e91', [], 'Display', None],
  [ 0x5e92, 'CjkUnifiedIdeograph5e92', [], 'Display', None],
  [ 0x5e93, 'CjkUnifiedIdeograph5e93', [], 'Display', None],
  [ 0x5e94, 'CjkUnifiedIdeograph5e94', [], 'Display', None],
  [ 0x5e95, 'CjkUnifiedIdeograph5e95', [], 'Display', None],
  [ 0x5e96, 'CjkUnifiedIdeograph5e96', [], 'Display', None],
  [ 0x5e97, 'CjkUnifiedIdeograph5e97', [], 'Display', None],
  [ 0x5e98, 'CjkUnifiedIdeograph5e98', [], 'Display', None],
  [ 0x5e99, 'CjkUnifiedIdeograph5e99', [], 'Display', None],
  [ 0x5e9a, 'CjkUnifiedIdeograph5e9a', [], 'Display', None],
  [ 0x5e9b, 'CjkUnifiedIdeograph5e9b', [], 'Display', None],
  [ 0x5e9c, 'CjkUnifiedIdeograph5e9c', [], 'Display', None],
  [ 0x5e9d, 'CjkUnifiedIdeograph5e9d', [], 'Display', None],
  [ 0x5e9e, 'CjkUnifiedIdeograph5e9e', [], 'Display', None],
  [ 0x5e9f, 'CjkUnifiedIdeograph5e9f', [], 'Display', None],
  [ 0x5ea0, 'CjkUnifiedIdeograph5ea0', [], 'Display', None],
  [ 0x5ea1, 'CjkUnifiedIdeograph5ea1', [], 'Display', None],
  [ 0x5ea2, 'CjkUnifiedIdeograph5ea2', [], 'Display', None],
  [ 0x5ea3, 'CjkUnifiedIdeograph5ea3', [], 'Display', None],
  [ 0x5ea4, 'CjkUnifiedIdeograph5ea4', [], 'Display', None],
  [ 0x5ea5, 'CjkUnifiedIdeograph5ea5', [], 'Display', None],
  [ 0x5ea6, 'CjkUnifiedIdeograph5ea6', [], 'Display', None],
  [ 0x5ea7, 'CjkUnifiedIdeograph5ea7', [], 'Display', None],
  [ 0x5ea8, 'CjkUnifiedIdeograph5ea8', [], 'Display', None],
  [ 0x5ea9, 'CjkUnifiedIdeograph5ea9', [], 'Display', None],
  [ 0x5eaa, 'CjkUnifiedIdeograph5eaa', [], 'Display', None],
  [ 0x5eab, 'CjkUnifiedIdeograph5eab', [], 'Display', None],
  [ 0x5eac, 'CjkUnifiedIdeograph5eac', [], 'Display', None],
  [ 0x5ead, 'CjkUnifiedIdeograph5ead', [], 'Display', None],
  [ 0x5eae, 'CjkUnifiedIdeograph5eae', [], 'Display', None],
  [ 0x5eaf, 'CjkUnifiedIdeograph5eaf', [], 'Display', None],
  [ 0x5eb0, 'CjkUnifiedIdeograph5eb0', [], 'Display', None],
  [ 0x5eb1, 'CjkUnifiedIdeograph5eb1', [], 'Display', None],
  [ 0x5eb2, 'CjkUnifiedIdeograph5eb2', [], 'Display', None],
  [ 0x5eb3, 'CjkUnifiedIdeograph5eb3', [], 'Display', None],
  [ 0x5eb4, 'CjkUnifiedIdeograph5eb4', [], 'Display', None],
  [ 0x5eb5, 'CjkUnifiedIdeograph5eb5', [], 'Display', None],
  [ 0x5eb6, 'CjkUnifiedIdeograph5eb6', [], 'Display', None],
  [ 0x5eb7, 'CjkUnifiedIdeograph5eb7', [], 'Display', None],
  [ 0x5eb8, 'CjkUnifiedIdeograph5eb8', [], 'Display', None],
  [ 0x5eb9, 'CjkUnifiedIdeograph5eb9', [], 'Display', None],
  [ 0x5eba, 'CjkUnifiedIdeograph5eba', [], 'Display', None],
  [ 0x5ebb, 'CjkUnifiedIdeograph5ebb', [], 'Display', None],
  [ 0x5ebc, 'CjkUnifiedIdeograph5ebc', [], 'Display', None],
  [ 0x5ebd, 'CjkUnifiedIdeograph5ebd', [], 'Display', None],
  [ 0x5ebe, 'CjkUnifiedIdeograph5ebe', [], 'Display', None],
  [ 0x5ebf, 'CjkUnifiedIdeograph5ebf', [], 'Display', None],
  [ 0x5ec0, 'CjkUnifiedIdeograph5ec0', [], 'Display', None],
  [ 0x5ec1, 'CjkUnifiedIdeograph5ec1', [], 'Display', None],
  [ 0x5ec2, 'CjkUnifiedIdeograph5ec2', [], 'Display', None],
  [ 0x5ec3, 'CjkUnifiedIdeograph5ec3', [], 'Display', None],
  [ 0x5ec4, 'CjkUnifiedIdeograph5ec4', [], 'Display', None],
  [ 0x5ec5, 'CjkUnifiedIdeograph5ec5', [], 'Display', None],
  [ 0x5ec6, 'CjkUnifiedIdeograph5ec6', [], 'Display', None],
  [ 0x5ec7, 'CjkUnifiedIdeograph5ec7', [], 'Display', None],
  [ 0x5ec8, 'CjkUnifiedIdeograph5ec8', [], 'Display', None],
  [ 0x5ec9, 'CjkUnifiedIdeograph5ec9', [], 'Display', None],
  [ 0x5eca, 'CjkUnifiedIdeograph5eca', [], 'Display', None],
  [ 0x5ecb, 'CjkUnifiedIdeograph5ecb', [], 'Display', None],
  [ 0x5ecc, 'CjkUnifiedIdeograph5ecc', [], 'Display', None],
  [ 0x5ecd, 'CjkUnifiedIdeograph5ecd', [], 'Display', None],
  [ 0x5ece, 'CjkUnifiedIdeograph5ece', [], 'Display', None],
  [ 0x5ecf, 'CjkUnifiedIdeograph5ecf', [], 'Display', None],
  [ 0x5ed0, 'CjkUnifiedIdeograph5ed0', [], 'Display', None],
  [ 0x5ed1, 'CjkUnifiedIdeograph5ed1', [], 'Display', None],
  [ 0x5ed2, 'CjkUnifiedIdeograph5ed2', [], 'Display', None],
  [ 0x5ed3, 'CjkUnifiedIdeograph5ed3', [], 'Display', None],
  [ 0x5ed4, 'CjkUnifiedIdeograph5ed4', [], 'Display', None],
  [ 0x5ed5, 'CjkUnifiedIdeograph5ed5', [], 'Display', None],
  [ 0x5ed6, 'CjkUnifiedIdeograph5ed6', [], 'Display', None],
  [ 0x5ed7, 'CjkUnifiedIdeograph5ed7', [], 'Display', None],
  [ 0x5ed8, 'CjkUnifiedIdeograph5ed8', [], 'Display', None],
  [ 0x5ed9, 'CjkUnifiedIdeograph5ed9', [], 'Display', None],
  [ 0x5eda, 'CjkUnifiedIdeograph5eda', [], 'Display', None],
  [ 0x5edb, 'CjkUnifiedIdeograph5edb', [], 'Display', None],
  [ 0x5edc, 'CjkUnifiedIdeograph5edc', [], 'Display', None],
  [ 0x5edd, 'CjkUnifiedIdeograph5edd', [], 'Display', None],
  [ 0x5ede, 'CjkUnifiedIdeograph5ede', [], 'Display', None],
  [ 0x5edf, 'CjkUnifiedIdeograph5edf', [], 'Display', None],
  [ 0x5ee0, 'CjkUnifiedIdeograph5ee0', [], 'Display', None],
  [ 0x5ee1, 'CjkUnifiedIdeograph5ee1', [], 'Display', None],
  [ 0x5ee2, 'CjkUnifiedIdeograph5ee2', [], 'Display', None],
  [ 0x5ee3, 'CjkUnifiedIdeograph5ee3', [], 'Display', None],
  [ 0x5ee4, 'CjkUnifiedIdeograph5ee4', [], 'Display', None],
  [ 0x5ee5, 'CjkUnifiedIdeograph5ee5', [], 'Display', None],
  [ 0x5ee6, 'CjkUnifiedIdeograph5ee6', [], 'Display', None],
  [ 0x5ee7, 'CjkUnifiedIdeograph5ee7', [], 'Display', None],
  [ 0x5ee8, 'CjkUnifiedIdeograph5ee8', [], 'Display', None],
  [ 0x5ee9, 'CjkUnifiedIdeograph5ee9', [], 'Display', None],
  [ 0x5eea, 'CjkUnifiedIdeograph5eea', [], 'Display', None],
  [ 0x5eeb, 'CjkUnifiedIdeograph5eeb', [], 'Display', None],
  [ 0x5eec, 'CjkUnifiedIdeograph5eec', [], 'Display', None],
  [ 0x5eed, 'CjkUnifiedIdeograph5eed', [], 'Display', None],
  [ 0x5eee, 'CjkUnifiedIdeograph5eee', [], 'Display', None],
  [ 0x5eef, 'CjkUnifiedIdeograph5eef', [], 'Display', None],
  [ 0x5ef0, 'CjkUnifiedIdeograph5ef0', [], 'Display', None],
  [ 0x5ef1, 'CjkUnifiedIdeograph5ef1', [], 'Display', None],
  [ 0x5ef2, 'CjkUnifiedIdeograph5ef2', [], 'Display', None],
  [ 0x5ef3, 'CjkUnifiedIdeograph5ef3', [], 'Display', None],
  [ 0x5ef4, 'CjkUnifiedIdeograph5ef4', [], 'Display', None],
  [ 0x5ef5, 'CjkUnifiedIdeograph5ef5', [], 'Display', None],
  [ 0x5ef6, 'CjkUnifiedIdeograph5ef6', [], 'Display', None],
  [ 0x5ef7, 'CjkUnifiedIdeograph5ef7', [], 'Display', None],
  [ 0x5ef8, 'CjkUnifiedIdeograph5ef8', [], 'Display', None],
  [ 0x5ef9, 'CjkUnifiedIdeograph5ef9', [], 'Display', None],
  [ 0x5efa, 'CjkUnifiedIdeograph5efa', [], 'Display', None],
  [ 0x5efb, 'CjkUnifiedIdeograph5efb', [], 'Display', None],
  [ 0x5efc, 'CjkUnifiedIdeograph5efc', [], 'Display', None],
  [ 0x5efd, 'CjkUnifiedIdeograph5efd', [], 'Display', None],
  [ 0x5efe, 'CjkUnifiedIdeograph5efe', [], 'Display', None],
  [ 0x5eff, 'CjkUnifiedIdeograph5eff', [], 'Display', None],
  [ 0x5f00, 'CjkUnifiedIdeograph5f00', [], 'Display', None],
  [ 0x5f01, 'CjkUnifiedIdeograph5f01', [], 'Display', None],
  [ 0x5f02, 'CjkUnifiedIdeograph5f02', [], 'Display', None],
  [ 0x5f03, 'CjkUnifiedIdeograph5f03', [], 'Display', None],
  [ 0x5f04, 'CjkUnifiedIdeograph5f04', [], 'Display', None],
  [ 0x5f05, 'CjkUnifiedIdeograph5f05', [], 'Display', None],
  [ 0x5f06, 'CjkUnifiedIdeograph5f06', [], 'Display', None],
  [ 0x5f07, 'CjkUnifiedIdeograph5f07', [], 'Display', None],
  [ 0x5f08, 'CjkUnifiedIdeograph5f08', [], 'Display', None],
  [ 0x5f09, 'CjkUnifiedIdeograph5f09', [], 'Display', None],
  [ 0x5f0a, 'CjkUnifiedIdeograph5f0a', [], 'Display', None],
  [ 0x5f0b, 'CjkUnifiedIdeograph5f0b', [], 'Display', None],
  [ 0x5f0c, 'CjkUnifiedIdeograph5f0c', [], 'Display', None],
  [ 0x5f0d, 'CjkUnifiedIdeograph5f0d', [], 'Display', None],
  [ 0x5f0e, 'CjkUnifiedIdeograph5f0e', [], 'Display', None],
  [ 0x5f0f, 'CjkUnifiedIdeograph5f0f', [], 'Display', None],
  [ 0x5f10, 'CjkUnifiedIdeograph5f10', [], 'Display', None],
  [ 0x5f11, 'CjkUnifiedIdeograph5f11', [], 'Display', None],
  [ 0x5f12, 'CjkUnifiedIdeograph5f12', [], 'Display', None],
  [ 0x5f13, 'CjkUnifiedIdeograph5f13', [], 'Display', None],
  [ 0x5f14, 'CjkUnifiedIdeograph5f14', [], 'Display', None],
  [ 0x5f15, 'CjkUnifiedIdeograph5f15', [], 'Display', None],
  [ 0x5f16, 'CjkUnifiedIdeograph5f16', [], 'Display', None],
  [ 0x5f17, 'CjkUnifiedIdeograph5f17', [], 'Display', None],
  [ 0x5f18, 'CjkUnifiedIdeograph5f18', [], 'Display', None],
  [ 0x5f19, 'CjkUnifiedIdeograph5f19', [], 'Display', None],
  [ 0x5f1a, 'CjkUnifiedIdeograph5f1a', [], 'Display', None],
  [ 0x5f1b, 'CjkUnifiedIdeograph5f1b', [], 'Display', None],
  [ 0x5f1c, 'CjkUnifiedIdeograph5f1c', [], 'Display', None],
  [ 0x5f1d, 'CjkUnifiedIdeograph5f1d', [], 'Display', None],
  [ 0x5f1e, 'CjkUnifiedIdeograph5f1e', [], 'Display', None],
  [ 0x5f1f, 'CjkUnifiedIdeograph5f1f', [], 'Display', None],
  [ 0x5f20, 'CjkUnifiedIdeograph5f20', [], 'Display', None],
  [ 0x5f21, 'CjkUnifiedIdeograph5f21', [], 'Display', None],
  [ 0x5f22, 'CjkUnifiedIdeograph5f22', [], 'Display', None],
  [ 0x5f23, 'CjkUnifiedIdeograph5f23', [], 'Display', None],
  [ 0x5f24, 'CjkUnifiedIdeograph5f24', [], 'Display', None],
  [ 0x5f25, 'CjkUnifiedIdeograph5f25', [], 'Display', None],
  [ 0x5f26, 'CjkUnifiedIdeograph5f26', [], 'Display', None],
  [ 0x5f27, 'CjkUnifiedIdeograph5f27', [], 'Display', None],
  [ 0x5f28, 'CjkUnifiedIdeograph5f28', [], 'Display', None],
  [ 0x5f29, 'CjkUnifiedIdeograph5f29', [], 'Display', None],
  [ 0x5f2a, 'CjkUnifiedIdeograph5f2a', [], 'Display', None],
  [ 0x5f2b, 'CjkUnifiedIdeograph5f2b', [], 'Display', None],
  [ 0x5f2c, 'CjkUnifiedIdeograph5f2c', [], 'Display', None],
  [ 0x5f2d, 'CjkUnifiedIdeograph5f2d', [], 'Display', None],
  [ 0x5f2e, 'CjkUnifiedIdeograph5f2e', [], 'Display', None],
  [ 0x5f2f, 'CjkUnifiedIdeograph5f2f', [], 'Display', None],
  [ 0x5f30, 'CjkUnifiedIdeograph5f30', [], 'Display', None],
  [ 0x5f31, 'CjkUnifiedIdeograph5f31', [], 'Display', None],
  [ 0x5f32, 'CjkUnifiedIdeograph5f32', [], 'Display', None],
  [ 0x5f33, 'CjkUnifiedIdeograph5f33', [], 'Display', None],
  [ 0x5f34, 'CjkUnifiedIdeograph5f34', [], 'Display', None],
  [ 0x5f35, 'CjkUnifiedIdeograph5f35', [], 'Display', None],
  [ 0x5f36, 'CjkUnifiedIdeograph5f36', [], 'Display', None],
  [ 0x5f37, 'CjkUnifiedIdeograph5f37', [], 'Display', None],
  [ 0x5f38, 'CjkUnifiedIdeograph5f38', [], 'Display', None],
  [ 0x5f39, 'CjkUnifiedIdeograph5f39', [], 'Display', None],
  [ 0x5f3a, 'CjkUnifiedIdeograph5f3a', [], 'Display', None],
  [ 0x5f3b, 'CjkUnifiedIdeograph5f3b', [], 'Display', None],
  [ 0x5f3c, 'CjkUnifiedIdeograph5f3c', [], 'Display', None],
  [ 0x5f3d, 'CjkUnifiedIdeograph5f3d', [], 'Display', None],
  [ 0x5f3e, 'CjkUnifiedIdeograph5f3e', [], 'Display', None],
  [ 0x5f3f, 'CjkUnifiedIdeograph5f3f', [], 'Display', None],
  [ 0x5f40, 'CjkUnifiedIdeograph5f40', [], 'Display', None],
  [ 0x5f41, 'CjkUnifiedIdeograph5f41', [], 'Display', None],
  [ 0x5f42, 'CjkUnifiedIdeograph5f42', [], 'Display', None],
  [ 0x5f43, 'CjkUnifiedIdeograph5f43', [], 'Display', None],
  [ 0x5f44, 'CjkUnifiedIdeograph5f44', [], 'Display', None],
  [ 0x5f45, 'CjkUnifiedIdeograph5f45', [], 'Display', None],
  [ 0x5f46, 'CjkUnifiedIdeograph5f46', [], 'Display', None],
  [ 0x5f47, 'CjkUnifiedIdeograph5f47', [], 'Display', None],
  [ 0x5f48, 'CjkUnifiedIdeograph5f48', [], 'Display', None],
  [ 0x5f49, 'CjkUnifiedIdeograph5f49', [], 'Display', None],
  [ 0x5f4a, 'CjkUnifiedIdeograph5f4a', [], 'Display', None],
  [ 0x5f4b, 'CjkUnifiedIdeograph5f4b', [], 'Display', None],
  [ 0x5f4c, 'CjkUnifiedIdeograph5f4c', [], 'Display', None],
  [ 0x5f4d, 'CjkUnifiedIdeograph5f4d', [], 'Display', None],
  [ 0x5f4e, 'CjkUnifiedIdeograph5f4e', [], 'Display', None],
  [ 0x5f4f, 'CjkUnifiedIdeograph5f4f', [], 'Display', None],
  [ 0x5f50, 'CjkUnifiedIdeograph5f50', [], 'Display', None],
  [ 0x5f51, 'CjkUnifiedIdeograph5f51', [], 'Display', None],
  [ 0x5f52, 'CjkUnifiedIdeograph5f52', [], 'Display', None],
  [ 0x5f53, 'CjkUnifiedIdeograph5f53', [], 'Display', None],
  [ 0x5f54, 'CjkUnifiedIdeograph5f54', [], 'Display', None],
  [ 0x5f55, 'CjkUnifiedIdeograph5f55', [], 'Display', None],
  [ 0x5f56, 'CjkUnifiedIdeograph5f56', [], 'Display', None],
  [ 0x5f57, 'CjkUnifiedIdeograph5f57', [], 'Display', None],
  [ 0x5f58, 'CjkUnifiedIdeograph5f58', [], 'Display', None],
  [ 0x5f59, 'CjkUnifiedIdeograph5f59', [], 'Display', None],
  [ 0x5f5a, 'CjkUnifiedIdeograph5f5a', [], 'Display', None],
  [ 0x5f5b, 'CjkUnifiedIdeograph5f5b', [], 'Display', None],
  [ 0x5f5c, 'CjkUnifiedIdeograph5f5c', [], 'Display', None],
  [ 0x5f5d, 'CjkUnifiedIdeograph5f5d', [], 'Display', None],
  [ 0x5f5e, 'CjkUnifiedIdeograph5f5e', [], 'Display', None],
  [ 0x5f5f, 'CjkUnifiedIdeograph5f5f', [], 'Display', None],
  [ 0x5f60, 'CjkUnifiedIdeograph5f60', [], 'Display', None],
  [ 0x5f61, 'CjkUnifiedIdeograph5f61', [], 'Display', None],
  [ 0x5f62, 'CjkUnifiedIdeograph5f62', [], 'Display', None],
  [ 0x5f63, 'CjkUnifiedIdeograph5f63', [], 'Display', None],
  [ 0x5f64, 'CjkUnifiedIdeograph5f64', [], 'Display', None],
  [ 0x5f65, 'CjkUnifiedIdeograph5f65', [], 'Display', None],
  [ 0x5f66, 'CjkUnifiedIdeograph5f66', [], 'Display', None],
  [ 0x5f67, 'CjkUnifiedIdeograph5f67', [], 'Display', None],
  [ 0x5f68, 'CjkUnifiedIdeograph5f68', [], 'Display', None],
  [ 0x5f69, 'CjkUnifiedIdeograph5f69', [], 'Display', None],
  [ 0x5f6a, 'CjkUnifiedIdeograph5f6a', [], 'Display', None],
  [ 0x5f6b, 'CjkUnifiedIdeograph5f6b', [], 'Display', None],
  [ 0x5f6c, 'CjkUnifiedIdeograph5f6c', [], 'Display', None],
  [ 0x5f6d, 'CjkUnifiedIdeograph5f6d', [], 'Display', None],
  [ 0x5f6e, 'CjkUnifiedIdeograph5f6e', [], 'Display', None],
  [ 0x5f6f, 'CjkUnifiedIdeograph5f6f', [], 'Display', None],
  [ 0x5f70, 'CjkUnifiedIdeograph5f70', [], 'Display', None],
  [ 0x5f71, 'CjkUnifiedIdeograph5f71', [], 'Display', None],
  [ 0x5f72, 'CjkUnifiedIdeograph5f72', [], 'Display', None],
  [ 0x5f73, 'CjkUnifiedIdeograph5f73', [], 'Display', None],
  [ 0x5f74, 'CjkUnifiedIdeograph5f74', [], 'Display', None],
  [ 0x5f75, 'CjkUnifiedIdeograph5f75', [], 'Display', None],
  [ 0x5f76, 'CjkUnifiedIdeograph5f76', [], 'Display', None],
  [ 0x5f77, 'CjkUnifiedIdeograph5f77', [], 'Display', None],
  [ 0x5f78, 'CjkUnifiedIdeograph5f78', [], 'Display', None],
  [ 0x5f79, 'CjkUnifiedIdeograph5f79', [], 'Display', None],
  [ 0x5f7a, 'CjkUnifiedIdeograph5f7a', [], 'Display', None],
  [ 0x5f7b, 'CjkUnifiedIdeograph5f7b', [], 'Display', None],
  [ 0x5f7c, 'CjkUnifiedIdeograph5f7c', [], 'Display', None],
  [ 0x5f7d, 'CjkUnifiedIdeograph5f7d', [], 'Display', None],
  [ 0x5f7e, 'CjkUnifiedIdeograph5f7e', [], 'Display', None],
  [ 0x5f7f, 'CjkUnifiedIdeograph5f7f', [], 'Display', None],
  [ 0x5f80, 'CjkUnifiedIdeograph5f80', [], 'Display', None],
  [ 0x5f81, 'CjkUnifiedIdeograph5f81', [], 'Display', None],
  [ 0x5f82, 'CjkUnifiedIdeograph5f82', [], 'Display', None],
  [ 0x5f83, 'CjkUnifiedIdeograph5f83', [], 'Display', None],
  [ 0x5f84, 'CjkUnifiedIdeograph5f84', [], 'Display', None],
  [ 0x5f85, 'CjkUnifiedIdeograph5f85', [], 'Display', None],
  [ 0x5f86, 'CjkUnifiedIdeograph5f86', [], 'Display', None],
  [ 0x5f87, 'CjkUnifiedIdeograph5f87', [], 'Display', None],
  [ 0x5f88, 'CjkUnifiedIdeograph5f88', [], 'Display', None],
  [ 0x5f89, 'CjkUnifiedIdeograph5f89', [], 'Display', None],
  [ 0x5f8a, 'CjkUnifiedIdeograph5f8a', [], 'Display', None],
  [ 0x5f8b, 'CjkUnifiedIdeograph5f8b', [], 'Display', None],
  [ 0x5f8c, 'CjkUnifiedIdeograph5f8c', [], 'Display', None],
  [ 0x5f8d, 'CjkUnifiedIdeograph5f8d', [], 'Display', None],
  [ 0x5f8e, 'CjkUnifiedIdeograph5f8e', [], 'Display', None],
  [ 0x5f8f, 'CjkUnifiedIdeograph5f8f', [], 'Display', None],
  [ 0x5f90, 'CjkUnifiedIdeograph5f90', [], 'Display', None],
  [ 0x5f91, 'CjkUnifiedIdeograph5f91', [], 'Display', None],
  [ 0x5f92, 'CjkUnifiedIdeograph5f92', [], 'Display', None],
  [ 0x5f93, 'CjkUnifiedIdeograph5f93', [], 'Display', None],
  [ 0x5f94, 'CjkUnifiedIdeograph5f94', [], 'Display', None],
  [ 0x5f95, 'CjkUnifiedIdeograph5f95', [], 'Display', None],
  [ 0x5f96, 'CjkUnifiedIdeograph5f96', [], 'Display', None],
  [ 0x5f97, 'CjkUnifiedIdeograph5f97', [], 'Display', None],
  [ 0x5f98, 'CjkUnifiedIdeograph5f98', [], 'Display', None],
  [ 0x5f99, 'CjkUnifiedIdeograph5f99', [], 'Display', None],
  [ 0x5f9a, 'CjkUnifiedIdeograph5f9a', [], 'Display', None],
  [ 0x5f9b, 'CjkUnifiedIdeograph5f9b', [], 'Display', None],
  [ 0x5f9c, 'CjkUnifiedIdeograph5f9c', [], 'Display', None],
  [ 0x5f9d, 'CjkUnifiedIdeograph5f9d', [], 'Display', None],
  [ 0x5f9e, 'CjkUnifiedIdeograph5f9e', [], 'Display', None],
  [ 0x5f9f, 'CjkUnifiedIdeograph5f9f', [], 'Display', None],
  [ 0x5fa0, 'CjkUnifiedIdeograph5fa0', [], 'Display', None],
  [ 0x5fa1, 'CjkUnifiedIdeograph5fa1', [], 'Display', None],
  [ 0x5fa2, 'CjkUnifiedIdeograph5fa2', [], 'Display', None],
  [ 0x5fa3, 'CjkUnifiedIdeograph5fa3', [], 'Display', None],
  [ 0x5fa4, 'CjkUnifiedIdeograph5fa4', [], 'Display', None],
  [ 0x5fa5, 'CjkUnifiedIdeograph5fa5', [], 'Display', None],
  [ 0x5fa6, 'CjkUnifiedIdeograph5fa6', [], 'Display', None],
  [ 0x5fa7, 'CjkUnifiedIdeograph5fa7', [], 'Display', None],
  [ 0x5fa8, 'CjkUnifiedIdeograph5fa8', [], 'Display', None],
  [ 0x5fa9, 'CjkUnifiedIdeograph5fa9', [], 'Display', None],
  [ 0x5faa, 'CjkUnifiedIdeograph5faa', [], 'Display', None],
  [ 0x5fab, 'CjkUnifiedIdeograph5fab', [], 'Display', None],
  [ 0x5fac, 'CjkUnifiedIdeograph5fac', [], 'Display', None],
  [ 0x5fad, 'CjkUnifiedIdeograph5fad', [], 'Display', None],
  [ 0x5fae, 'CjkUnifiedIdeograph5fae', [], 'Display', None],
  [ 0x5faf, 'CjkUnifiedIdeograph5faf', [], 'Display', None],
  [ 0x5fb0, 'CjkUnifiedIdeograph5fb0', [], 'Display', None],
  [ 0x5fb1, 'CjkUnifiedIdeograph5fb1', [], 'Display', None],
  [ 0x5fb2, 'CjkUnifiedIdeograph5fb2', [], 'Display', None],
  [ 0x5fb3, 'CjkUnifiedIdeograph5fb3', [], 'Display', None],
  [ 0x5fb4, 'CjkUnifiedIdeograph5fb4', [], 'Display', None],
  [ 0x5fb5, 'CjkUnifiedIdeograph5fb5', [], 'Display', None],
  [ 0x5fb6, 'CjkUnifiedIdeograph5fb6', [], 'Display', None],
  [ 0x5fb7, 'CjkUnifiedIdeograph5fb7', [], 'Display', None],
  [ 0x5fb8, 'CjkUnifiedIdeograph5fb8', [], 'Display', None],
  [ 0x5fb9, 'CjkUnifiedIdeograph5fb9', [], 'Display', None],
  [ 0x5fba, 'CjkUnifiedIdeograph5fba', [], 'Display', None],
  [ 0x5fbb, 'CjkUnifiedIdeograph5fbb', [], 'Display', None],
  [ 0x5fbc, 'CjkUnifiedIdeograph5fbc', [], 'Display', None],
  [ 0x5fbd, 'CjkUnifiedIdeograph5fbd', [], 'Display', None],
  [ 0x5fbe, 'CjkUnifiedIdeograph5fbe', [], 'Display', None],
  [ 0x5fbf, 'CjkUnifiedIdeograph5fbf', [], 'Display', None],
  [ 0x5fc0, 'CjkUnifiedIdeograph5fc0', [], 'Display', None],
  [ 0x5fc1, 'CjkUnifiedIdeograph5fc1', [], 'Display', None],
  [ 0x5fc2, 'CjkUnifiedIdeograph5fc2', [], 'Display', None],
  [ 0x5fc3, 'CjkUnifiedIdeograph5fc3', [], 'Display', None],
  [ 0x5fc4, 'CjkUnifiedIdeograph5fc4', [], 'Display', None],
  [ 0x5fc5, 'CjkUnifiedIdeograph5fc5', [], 'Display', None],
  [ 0x5fc6, 'CjkUnifiedIdeograph5fc6', [], 'Display', None],
  [ 0x5fc7, 'CjkUnifiedIdeograph5fc7', [], 'Display', None],
  [ 0x5fc8, 'CjkUnifiedIdeograph5fc8', [], 'Display', None],
  [ 0x5fc9, 'CjkUnifiedIdeograph5fc9', [], 'Display', None],
  [ 0x5fca, 'CjkUnifiedIdeograph5fca', [], 'Display', None],
  [ 0x5fcb, 'CjkUnifiedIdeograph5fcb', [], 'Display', None],
  [ 0x5fcc, 'CjkUnifiedIdeograph5fcc', [], 'Display', None],
  [ 0x5fcd, 'CjkUnifiedIdeograph5fcd', [], 'Display', None],
  [ 0x5fce, 'CjkUnifiedIdeograph5fce', [], 'Display', None],
  [ 0x5fcf, 'CjkUnifiedIdeograph5fcf', [], 'Display', None],
  [ 0x5fd0, 'CjkUnifiedIdeograph5fd0', [], 'Display', None],
  [ 0x5fd1, 'CjkUnifiedIdeograph5fd1', [], 'Display', None],
  [ 0x5fd2, 'CjkUnifiedIdeograph5fd2', [], 'Display', None],
  [ 0x5fd3, 'CjkUnifiedIdeograph5fd3', [], 'Display', None],
  [ 0x5fd4, 'CjkUnifiedIdeograph5fd4', [], 'Display', None],
  [ 0x5fd5, 'CjkUnifiedIdeograph5fd5', [], 'Display', None],
  [ 0x5fd6, 'CjkUnifiedIdeograph5fd6', [], 'Display', None],
  [ 0x5fd7, 'CjkUnifiedIdeograph5fd7', [], 'Display', None],
  [ 0x5fd8, 'CjkUnifiedIdeograph5fd8', [], 'Display', None],
  [ 0x5fd9, 'CjkUnifiedIdeograph5fd9', [], 'Display', None],
  [ 0x5fda, 'CjkUnifiedIdeograph5fda', [], 'Display', None],
  [ 0x5fdb, 'CjkUnifiedIdeograph5fdb', [], 'Display', None],
  [ 0x5fdc, 'CjkUnifiedIdeograph5fdc', [], 'Display', None],
  [ 0x5fdd, 'CjkUnifiedIdeograph5fdd', [], 'Display', None],
  [ 0x5fde, 'CjkUnifiedIdeograph5fde', [], 'Display', None],
  [ 0x5fdf, 'CjkUnifiedIdeograph5fdf', [], 'Display', None],
  [ 0x5fe0, 'CjkUnifiedIdeograph5fe0', [], 'Display', None],
  [ 0x5fe1, 'CjkUnifiedIdeograph5fe1', [], 'Display', None],
  [ 0x5fe2, 'CjkUnifiedIdeograph5fe2', [], 'Display', None],
  [ 0x5fe3, 'CjkUnifiedIdeograph5fe3', [], 'Display', None],
  [ 0x5fe4, 'CjkUnifiedIdeograph5fe4', [], 'Display', None],
  [ 0x5fe5, 'CjkUnifiedIdeograph5fe5', [], 'Display', None],
  [ 0x5fe6, 'CjkUnifiedIdeograph5fe6', [], 'Display', None],
  [ 0x5fe7, 'CjkUnifiedIdeograph5fe7', [], 'Display', None],
  [ 0x5fe8, 'CjkUnifiedIdeograph5fe8', [], 'Display', None],
  [ 0x5fe9, 'CjkUnifiedIdeograph5fe9', [], 'Display', None],
  [ 0x5fea, 'CjkUnifiedIdeograph5fea', [], 'Display', None],
  [ 0x5feb, 'CjkUnifiedIdeograph5feb', [], 'Display', None],
  [ 0x5fec, 'CjkUnifiedIdeograph5fec', [], 'Display', None],
  [ 0x5fed, 'CjkUnifiedIdeograph5fed', [], 'Display', None],
  [ 0x5fee, 'CjkUnifiedIdeograph5fee', [], 'Display', None],
  [ 0x5fef, 'CjkUnifiedIdeograph5fef', [], 'Display', None],
  [ 0x5ff0, 'CjkUnifiedIdeograph5ff0', [], 'Display', None],
  [ 0x5ff1, 'CjkUnifiedIdeograph5ff1', [], 'Display', None],
  [ 0x5ff2, 'CjkUnifiedIdeograph5ff2', [], 'Display', None],
  [ 0x5ff3, 'CjkUnifiedIdeograph5ff3', [], 'Display', None],
  [ 0x5ff4, 'CjkUnifiedIdeograph5ff4', [], 'Display', None],
  [ 0x5ff5, 'CjkUnifiedIdeograph5ff5', [], 'Display', None],
  [ 0x5ff6, 'CjkUnifiedIdeograph5ff6', [], 'Display', None],
  [ 0x5ff7, 'CjkUnifiedIdeograph5ff7', [], 'Display', None],
  [ 0x5ff8, 'CjkUnifiedIdeograph5ff8', [], 'Display', None],
  [ 0x5ff9, 'CjkUnifiedIdeograph5ff9', [], 'Display', None],
  [ 0x5ffa, 'CjkUnifiedIdeograph5ffa', [], 'Display', None],
  [ 0x5ffb, 'CjkUnifiedIdeograph5ffb', [], 'Display', None],
  [ 0x5ffc, 'CjkUnifiedIdeograph5ffc', [], 'Display', None],
  [ 0x5ffd, 'CjkUnifiedIdeograph5ffd', [], 'Display', None],
  [ 0x5ffe, 'CjkUnifiedIdeograph5ffe', [], 'Display', None],
  [ 0x5fff, 'CjkUnifiedIdeograph5fff', [], 'Display', None],
  [ 0x6000, 'CjkUnifiedIdeograph6000', [], 'Display', None],
  [ 0x6001, 'CjkUnifiedIdeograph6001', [], 'Display', None],
  [ 0x6002, 'CjkUnifiedIdeograph6002', [], 'Display', None],
  [ 0x6003, 'CjkUnifiedIdeograph6003', [], 'Display', None],
  [ 0x6004, 'CjkUnifiedIdeograph6004', [], 'Display', None],
  [ 0x6005, 'CjkUnifiedIdeograph6005', [], 'Display', None],
  [ 0x6006, 'CjkUnifiedIdeograph6006', [], 'Display', None],
  [ 0x6007, 'CjkUnifiedIdeograph6007', [], 'Display', None],
  [ 0x6008, 'CjkUnifiedIdeograph6008', [], 'Display', None],
  [ 0x6009, 'CjkUnifiedIdeograph6009', [], 'Display', None],
  [ 0x600a, 'CjkUnifiedIdeograph600a', [], 'Display', None],
  [ 0x600b, 'CjkUnifiedIdeograph600b', [], 'Display', None],
  [ 0x600c, 'CjkUnifiedIdeograph600c', [], 'Display', None],
  [ 0x600d, 'CjkUnifiedIdeograph600d', [], 'Display', None],
  [ 0x600e, 'CjkUnifiedIdeograph600e', [], 'Display', None],
  [ 0x600f, 'CjkUnifiedIdeograph600f', [], 'Display', None],
  [ 0x6010, 'CjkUnifiedIdeograph6010', [], 'Display', None],
  [ 0x6011, 'CjkUnifiedIdeograph6011', [], 'Display', None],
  [ 0x6012, 'CjkUnifiedIdeograph6012', [], 'Display', None],
  [ 0x6013, 'CjkUnifiedIdeograph6013', [], 'Display', None],
  [ 0x6014, 'CjkUnifiedIdeograph6014', [], 'Display', None],
  [ 0x6015, 'CjkUnifiedIdeograph6015', [], 'Display', None],
  [ 0x6016, 'CjkUnifiedIdeograph6016', [], 'Display', None],
  [ 0x6017, 'CjkUnifiedIdeograph6017', [], 'Display', None],
  [ 0x6018, 'CjkUnifiedIdeograph6018', [], 'Display', None],
  [ 0x6019, 'CjkUnifiedIdeograph6019', [], 'Display', None],
  [ 0x601a, 'CjkUnifiedIdeograph601a', [], 'Display', None],
  [ 0x601b, 'CjkUnifiedIdeograph601b', [], 'Display', None],
  [ 0x601c, 'CjkUnifiedIdeograph601c', [], 'Display', None],
  [ 0x601d, 'CjkUnifiedIdeograph601d', [], 'Display', None],
  [ 0x601e, 'CjkUnifiedIdeograph601e', [], 'Display', None],
  [ 0x601f, 'CjkUnifiedIdeograph601f', [], 'Display', None],
  [ 0x6020, 'CjkUnifiedIdeograph6020', [], 'Display', None],
  [ 0x6021, 'CjkUnifiedIdeograph6021', [], 'Display', None],
  [ 0x6022, 'CjkUnifiedIdeograph6022', [], 'Display', None],
  [ 0x6023, 'CjkUnifiedIdeograph6023', [], 'Display', None],
  [ 0x6024, 'CjkUnifiedIdeograph6024', [], 'Display', None],
  [ 0x6025, 'CjkUnifiedIdeograph6025', [], 'Display', None],
  [ 0x6026, 'CjkUnifiedIdeograph6026', [], 'Display', None],
  [ 0x6027, 'CjkUnifiedIdeograph6027', [], 'Display', None],
  [ 0x6028, 'CjkUnifiedIdeograph6028', [], 'Display', None],
  [ 0x6029, 'CjkUnifiedIdeograph6029', [], 'Display', None],
  [ 0x602a, 'CjkUnifiedIdeograph602a', [], 'Display', None],
  [ 0x602b, 'CjkUnifiedIdeograph602b', [], 'Display', None],
  [ 0x602c, 'CjkUnifiedIdeograph602c', [], 'Display', None],
  [ 0x602d, 'CjkUnifiedIdeograph602d', [], 'Display', None],
  [ 0x602e, 'CjkUnifiedIdeograph602e', [], 'Display', None],
  [ 0x602f, 'CjkUnifiedIdeograph602f', [], 'Display', None],
  [ 0x6030, 'CjkUnifiedIdeograph6030', [], 'Display', None],
  [ 0x6031, 'CjkUnifiedIdeograph6031', [], 'Display', None],
  [ 0x6032, 'CjkUnifiedIdeograph6032', [], 'Display', None],
  [ 0x6033, 'CjkUnifiedIdeograph6033', [], 'Display', None],
  [ 0x6034, 'CjkUnifiedIdeograph6034', [], 'Display', None],
  [ 0x6035, 'CjkUnifiedIdeograph6035', [], 'Display', None],
  [ 0x6036, 'CjkUnifiedIdeograph6036', [], 'Display', None],
  [ 0x6037, 'CjkUnifiedIdeograph6037', [], 'Display', None],
  [ 0x6038, 'CjkUnifiedIdeograph6038', [], 'Display', None],
  [ 0x6039, 'CjkUnifiedIdeograph6039', [], 'Display', None],
  [ 0x603a, 'CjkUnifiedIdeograph603a', [], 'Display', None],
  [ 0x603b, 'CjkUnifiedIdeograph603b', [], 'Display', None],
  [ 0x603c, 'CjkUnifiedIdeograph603c', [], 'Display', None],
  [ 0x603d, 'CjkUnifiedIdeograph603d', [], 'Display', None],
  [ 0x603e, 'CjkUnifiedIdeograph603e', [], 'Display', None],
  [ 0x603f, 'CjkUnifiedIdeograph603f', [], 'Display', None],
  [ 0x6040, 'CjkUnifiedIdeograph6040', [], 'Display', None],
  [ 0x6041, 'CjkUnifiedIdeograph6041', [], 'Display', None],
  [ 0x6042, 'CjkUnifiedIdeograph6042', [], 'Display', None],
  [ 0x6043, 'CjkUnifiedIdeograph6043', [], 'Display', None],
  [ 0x6044, 'CjkUnifiedIdeograph6044', [], 'Display', None],
  [ 0x6045, 'CjkUnifiedIdeograph6045', [], 'Display', None],
  [ 0x6046, 'CjkUnifiedIdeograph6046', [], 'Display', None],
  [ 0x6047, 'CjkUnifiedIdeograph6047', [], 'Display', None],
  [ 0x6048, 'CjkUnifiedIdeograph6048', [], 'Display', None],
  [ 0x6049, 'CjkUnifiedIdeograph6049', [], 'Display', None],
  [ 0x604a, 'CjkUnifiedIdeograph604a', [], 'Display', None],
  [ 0x604b, 'CjkUnifiedIdeograph604b', [], 'Display', None],
  [ 0x604c, 'CjkUnifiedIdeograph604c', [], 'Display', None],
  [ 0x604d, 'CjkUnifiedIdeograph604d', [], 'Display', None],
  [ 0x604e, 'CjkUnifiedIdeograph604e', [], 'Display', None],
  [ 0x604f, 'CjkUnifiedIdeograph604f', [], 'Display', None],
  [ 0x6050, 'CjkUnifiedIdeograph6050', [], 'Display', None],
  [ 0x6051, 'CjkUnifiedIdeograph6051', [], 'Display', None],
  [ 0x6052, 'CjkUnifiedIdeograph6052', [], 'Display', None],
  [ 0x6053, 'CjkUnifiedIdeograph6053', [], 'Display', None],
  [ 0x6054, 'CjkUnifiedIdeograph6054', [], 'Display', None],
  [ 0x6055, 'CjkUnifiedIdeograph6055', [], 'Display', None],
  [ 0x6056, 'CjkUnifiedIdeograph6056', [], 'Display', None],
  [ 0x6057, 'CjkUnifiedIdeograph6057', [], 'Display', None],
  [ 0x6058, 'CjkUnifiedIdeograph6058', [], 'Display', None],
  [ 0x6059, 'CjkUnifiedIdeograph6059', [], 'Display', None],
  [ 0x605a, 'CjkUnifiedIdeograph605a', [], 'Display', None],
  [ 0x605b, 'CjkUnifiedIdeograph605b', [], 'Display', None],
  [ 0x605c, 'CjkUnifiedIdeograph605c', [], 'Display', None],
  [ 0x605d, 'CjkUnifiedIdeograph605d', [], 'Display', None],
  [ 0x605e, 'CjkUnifiedIdeograph605e', [], 'Display', None],
  [ 0x605f, 'CjkUnifiedIdeograph605f', [], 'Display', None],
  [ 0x6060, 'CjkUnifiedIdeograph6060', [], 'Display', None],
  [ 0x6061, 'CjkUnifiedIdeograph6061', [], 'Display', None],
  [ 0x6062, 'CjkUnifiedIdeograph6062', [], 'Display', None],
  [ 0x6063, 'CjkUnifiedIdeograph6063', [], 'Display', None],
  [ 0x6064, 'CjkUnifiedIdeograph6064', [], 'Display', None],
  [ 0x6065, 'CjkUnifiedIdeograph6065', [], 'Display', None],
  [ 0x6066, 'CjkUnifiedIdeograph6066', [], 'Display', None],
  [ 0x6067, 'CjkUnifiedIdeograph6067', [], 'Display', None],
  [ 0x6068, 'CjkUnifiedIdeograph6068', [], 'Display', None],
  [ 0x6069, 'CjkUnifiedIdeograph6069', [], 'Display', None],
  [ 0x606a, 'CjkUnifiedIdeograph606a', [], 'Display', None],
  [ 0x606b, 'CjkUnifiedIdeograph606b', [], 'Display', None],
  [ 0x606c, 'CjkUnifiedIdeograph606c', [], 'Display', None],
  [ 0x606d, 'CjkUnifiedIdeograph606d', [], 'Display', None],
  [ 0x606e, 'CjkUnifiedIdeograph606e', [], 'Display', None],
  [ 0x606f, 'CjkUnifiedIdeograph606f', [], 'Display', None],
  [ 0x6070, 'CjkUnifiedIdeograph6070', [], 'Display', None],
  [ 0x6071, 'CjkUnifiedIdeograph6071', [], 'Display', None],
  [ 0x6072, 'CjkUnifiedIdeograph6072', [], 'Display', None],
  [ 0x6073, 'CjkUnifiedIdeograph6073', [], 'Display', None],
  [ 0x6074, 'CjkUnifiedIdeograph6074', [], 'Display', None],
  [ 0x6075, 'CjkUnifiedIdeograph6075', [], 'Display', None],
  [ 0x6076, 'CjkUnifiedIdeograph6076', [], 'Display', None],
  [ 0x6077, 'CjkUnifiedIdeograph6077', [], 'Display', None],
  [ 0x6078, 'CjkUnifiedIdeograph6078', [], 'Display', None],
  [ 0x6079, 'CjkUnifiedIdeograph6079', [], 'Display', None],
  [ 0x607a, 'CjkUnifiedIdeograph607a', [], 'Display', None],
  [ 0x607b, 'CjkUnifiedIdeograph607b', [], 'Display', None],
  [ 0x607c, 'CjkUnifiedIdeograph607c', [], 'Display', None],
  [ 0x607d, 'CjkUnifiedIdeograph607d', [], 'Display', None],
  [ 0x607e, 'CjkUnifiedIdeograph607e', [], 'Display', None],
  [ 0x607f, 'CjkUnifiedIdeograph607f', [], 'Display', None],
  [ 0x6080, 'CjkUnifiedIdeograph6080', [], 'Display', None],
  [ 0x6081, 'CjkUnifiedIdeograph6081', [], 'Display', None],
  [ 0x6082, 'CjkUnifiedIdeograph6082', [], 'Display', None],
  [ 0x6083, 'CjkUnifiedIdeograph6083', [], 'Display', None],
  [ 0x6084, 'CjkUnifiedIdeograph6084', [], 'Display', None],
  [ 0x6085, 'CjkUnifiedIdeograph6085', [], 'Display', None],
  [ 0x6086, 'CjkUnifiedIdeograph6086', [], 'Display', None],
  [ 0x6087, 'CjkUnifiedIdeograph6087', [], 'Display', None],
  [ 0x6088, 'CjkUnifiedIdeograph6088', [], 'Display', None],
  [ 0x6089, 'CjkUnifiedIdeograph6089', [], 'Display', None],
  [ 0x608a, 'CjkUnifiedIdeograph608a', [], 'Display', None],
  [ 0x608b, 'CjkUnifiedIdeograph608b', [], 'Display', None],
  [ 0x608c, 'CjkUnifiedIdeograph608c', [], 'Display', None],
  [ 0x608d, 'CjkUnifiedIdeograph608d', [], 'Display', None],
  [ 0x608e, 'CjkUnifiedIdeograph608e', [], 'Display', None],
  [ 0x608f, 'CjkUnifiedIdeograph608f', [], 'Display', None],
  [ 0x6090, 'CjkUnifiedIdeograph6090', [], 'Display', None],
  [ 0x6091, 'CjkUnifiedIdeograph6091', [], 'Display', None],
  [ 0x6092, 'CjkUnifiedIdeograph6092', [], 'Display', None],
  [ 0x6093, 'CjkUnifiedIdeograph6093', [], 'Display', None],
  [ 0x6094, 'CjkUnifiedIdeograph6094', [], 'Display', None],
  [ 0x6095, 'CjkUnifiedIdeograph6095', [], 'Display', None],
  [ 0x6096, 'CjkUnifiedIdeograph6096', [], 'Display', None],
  [ 0x6097, 'CjkUnifiedIdeograph6097', [], 'Display', None],
  [ 0x6098, 'CjkUnifiedIdeograph6098', [], 'Display', None],
  [ 0x6099, 'CjkUnifiedIdeograph6099', [], 'Display', None],
  [ 0x609a, 'CjkUnifiedIdeograph609a', [], 'Display', None],
  [ 0x609b, 'CjkUnifiedIdeograph609b', [], 'Display', None],
  [ 0x609c, 'CjkUnifiedIdeograph609c', [], 'Display', None],
  [ 0x609d, 'CjkUnifiedIdeograph609d', [], 'Display', None],
  [ 0x609e, 'CjkUnifiedIdeograph609e', [], 'Display', None],
  [ 0x609f, 'CjkUnifiedIdeograph609f', [], 'Display', None],
  [ 0x60a0, 'CjkUnifiedIdeograph60a0', [], 'Display', None],
  [ 0x60a1, 'CjkUnifiedIdeograph60a1', [], 'Display', None],
  [ 0x60a2, 'CjkUnifiedIdeograph60a2', [], 'Display', None],
  [ 0x60a3, 'CjkUnifiedIdeograph60a3', [], 'Display', None],
  [ 0x60a4, 'CjkUnifiedIdeograph60a4', [], 'Display', None],
  [ 0x60a5, 'CjkUnifiedIdeograph60a5', [], 'Display', None],
  [ 0x60a6, 'CjkUnifiedIdeograph60a6', [], 'Display', None],
  [ 0x60a7, 'CjkUnifiedIdeograph60a7', [], 'Display', None],
  [ 0x60a8, 'CjkUnifiedIdeograph60a8', [], 'Display', None],
  [ 0x60a9, 'CjkUnifiedIdeograph60a9', [], 'Display', None],
  [ 0x60aa, 'CjkUnifiedIdeograph60aa', [], 'Display', None],
  [ 0x60ab, 'CjkUnifiedIdeograph60ab', [], 'Display', None],
  [ 0x60ac, 'CjkUnifiedIdeograph60ac', [], 'Display', None],
  [ 0x60ad, 'CjkUnifiedIdeograph60ad', [], 'Display', None],
  [ 0x60ae, 'CjkUnifiedIdeograph60ae', [], 'Display', None],
  [ 0x60af, 'CjkUnifiedIdeograph60af', [], 'Display', None],
  [ 0x60b0, 'CjkUnifiedIdeograph60b0', [], 'Display', None],
  [ 0x60b1, 'CjkUnifiedIdeograph60b1', [], 'Display', None],
  [ 0x60b2, 'CjkUnifiedIdeograph60b2', [], 'Display', None],
  [ 0x60b3, 'CjkUnifiedIdeograph60b3', [], 'Display', None],
  [ 0x60b4, 'CjkUnifiedIdeograph60b4', [], 'Display', None],
  [ 0x60b5, 'CjkUnifiedIdeograph60b5', [], 'Display', None],
  [ 0x60b6, 'CjkUnifiedIdeograph60b6', [], 'Display', None],
  [ 0x60b7, 'CjkUnifiedIdeograph60b7', [], 'Display', None],
  [ 0x60b8, 'CjkUnifiedIdeograph60b8', [], 'Display', None],
  [ 0x60b9, 'CjkUnifiedIdeograph60b9', [], 'Display', None],
  [ 0x60ba, 'CjkUnifiedIdeograph60ba', [], 'Display', None],
  [ 0x60bb, 'CjkUnifiedIdeograph60bb', [], 'Display', None],
  [ 0x60bc, 'CjkUnifiedIdeograph60bc', [], 'Display', None],
  [ 0x60bd, 'CjkUnifiedIdeograph60bd', [], 'Display', None],
  [ 0x60be, 'CjkUnifiedIdeograph60be', [], 'Display', None],
  [ 0x60bf, 'CjkUnifiedIdeograph60bf', [], 'Display', None],
  [ 0x60c0, 'CjkUnifiedIdeograph60c0', [], 'Display', None],
  [ 0x60c1, 'CjkUnifiedIdeograph60c1', [], 'Display', None],
  [ 0x60c2, 'CjkUnifiedIdeograph60c2', [], 'Display', None],
  [ 0x60c3, 'CjkUnifiedIdeograph60c3', [], 'Display', None],
  [ 0x60c4, 'CjkUnifiedIdeograph60c4', [], 'Display', None],
  [ 0x60c5, 'CjkUnifiedIdeograph60c5', [], 'Display', None],
  [ 0x60c6, 'CjkUnifiedIdeograph60c6', [], 'Display', None],
  [ 0x60c7, 'CjkUnifiedIdeograph60c7', [], 'Display', None],
  [ 0x60c8, 'CjkUnifiedIdeograph60c8', [], 'Display', None],
  [ 0x60c9, 'CjkUnifiedIdeograph60c9', [], 'Display', None],
  [ 0x60ca, 'CjkUnifiedIdeograph60ca', [], 'Display', None],
  [ 0x60cb, 'CjkUnifiedIdeograph60cb', [], 'Display', None],
  [ 0x60cc, 'CjkUnifiedIdeograph60cc', [], 'Display', None],
  [ 0x60cd, 'CjkUnifiedIdeograph60cd', [], 'Display', None],
  [ 0x60ce, 'CjkUnifiedIdeograph60ce', [], 'Display', None],
  [ 0x60cf, 'CjkUnifiedIdeograph60cf', [], 'Display', None],
  [ 0x60d0, 'CjkUnifiedIdeograph60d0', [], 'Display', None],
  [ 0x60d1, 'CjkUnifiedIdeograph60d1', [], 'Display', None],
  [ 0x60d2, 'CjkUnifiedIdeograph60d2', [], 'Display', None],
  [ 0x60d3, 'CjkUnifiedIdeograph60d3', [], 'Display', None],
  [ 0x60d4, 'CjkUnifiedIdeograph60d4', [], 'Display', None],
  [ 0x60d5, 'CjkUnifiedIdeograph60d5', [], 'Display', None],
  [ 0x60d6, 'CjkUnifiedIdeograph60d6', [], 'Display', None],
  [ 0x60d7, 'CjkUnifiedIdeograph60d7', [], 'Display', None],
  [ 0x60d8, 'CjkUnifiedIdeograph60d8', [], 'Display', None],
  [ 0x60d9, 'CjkUnifiedIdeograph60d9', [], 'Display', None],
  [ 0x60da, 'CjkUnifiedIdeograph60da', [], 'Display', None],
  [ 0x60db, 'CjkUnifiedIdeograph60db', [], 'Display', None],
  [ 0x60dc, 'CjkUnifiedIdeograph60dc', [], 'Display', None],
  [ 0x60dd, 'CjkUnifiedIdeograph60dd', [], 'Display', None],
  [ 0x60de, 'CjkUnifiedIdeograph60de', [], 'Display', None],
  [ 0x60df, 'CjkUnifiedIdeograph60df', [], 'Display', None],
  [ 0x60e0, 'CjkUnifiedIdeograph60e0', [], 'Display', None],
  [ 0x60e1, 'CjkUnifiedIdeograph60e1', [], 'Display', None],
  [ 0x60e2, 'CjkUnifiedIdeograph60e2', [], 'Display', None],
  [ 0x60e3, 'CjkUnifiedIdeograph60e3', [], 'Display', None],
  [ 0x60e4, 'CjkUnifiedIdeograph60e4', [], 'Display', None],
  [ 0x60e5, 'CjkUnifiedIdeograph60e5', [], 'Display', None],
  [ 0x60e6, 'CjkUnifiedIdeograph60e6', [], 'Display', None],
  [ 0x60e7, 'CjkUnifiedIdeograph60e7', [], 'Display', None],
  [ 0x60e8, 'CjkUnifiedIdeograph60e8', [], 'Display', None],
  [ 0x60e9, 'CjkUnifiedIdeograph60e9', [], 'Display', None],
  [ 0x60ea, 'CjkUnifiedIdeograph60ea', [], 'Display', None],
  [ 0x60eb, 'CjkUnifiedIdeograph60eb', [], 'Display', None],
  [ 0x60ec, 'CjkUnifiedIdeograph60ec', [], 'Display', None],
  [ 0x60ed, 'CjkUnifiedIdeograph60ed', [], 'Display', None],
  [ 0x60ee, 'CjkUnifiedIdeograph60ee', [], 'Display', None],
  [ 0x60ef, 'CjkUnifiedIdeograph60ef', [], 'Display', None],
  [ 0x60f0, 'CjkUnifiedIdeograph60f0', [], 'Display', None],
  [ 0x60f1, 'CjkUnifiedIdeograph60f1', [], 'Display', None],
  [ 0x60f2, 'CjkUnifiedIdeograph60f2', [], 'Display', None],
  [ 0x60f3, 'CjkUnifiedIdeograph60f3', [], 'Display', None],
  [ 0x60f4, 'CjkUnifiedIdeograph60f4', [], 'Display', None],
  [ 0x60f5, 'CjkUnifiedIdeograph60f5', [], 'Display', None],
  [ 0x60f6, 'CjkUnifiedIdeograph60f6', [], 'Display', None],
  [ 0x60f7, 'CjkUnifiedIdeograph60f7', [], 'Display', None],
  [ 0x60f8, 'CjkUnifiedIdeograph60f8', [], 'Display', None],
  [ 0x60f9, 'CjkUnifiedIdeograph60f9', [], 'Display', None],
  [ 0x60fa, 'CjkUnifiedIdeograph60fa', [], 'Display', None],
  [ 0x60fb, 'CjkUnifiedIdeograph60fb', [], 'Display', None],
  [ 0x60fc, 'CjkUnifiedIdeograph60fc', [], 'Display', None],
  [ 0x60fd, 'CjkUnifiedIdeograph60fd', [], 'Display', None],
  [ 0x60fe, 'CjkUnifiedIdeograph60fe', [], 'Display', None],
  [ 0x60ff, 'CjkUnifiedIdeograph60ff', [], 'Display', None],
  [ 0x6100, 'CjkUnifiedIdeograph6100', [], 'Display', None],
  [ 0x6101, 'CjkUnifiedIdeograph6101', [], 'Display', None],
  [ 0x6102, 'CjkUnifiedIdeograph6102', [], 'Display', None],
  [ 0x6103, 'CjkUnifiedIdeograph6103', [], 'Display', None],
  [ 0x6104, 'CjkUnifiedIdeograph6104', [], 'Display', None],
  [ 0x6105, 'CjkUnifiedIdeograph6105', [], 'Display', None],
  [ 0x6106, 'CjkUnifiedIdeograph6106', [], 'Display', None],
  [ 0x6107, 'CjkUnifiedIdeograph6107', [], 'Display', None],
  [ 0x6108, 'CjkUnifiedIdeograph6108', [], 'Display', None],
  [ 0x6109, 'CjkUnifiedIdeograph6109', [], 'Display', None],
  [ 0x610a, 'CjkUnifiedIdeograph610a', [], 'Display', None],
  [ 0x610b, 'CjkUnifiedIdeograph610b', [], 'Display', None],
  [ 0x610c, 'CjkUnifiedIdeograph610c', [], 'Display', None],
  [ 0x610d, 'CjkUnifiedIdeograph610d', [], 'Display', None],
  [ 0x610e, 'CjkUnifiedIdeograph610e', [], 'Display', None],
  [ 0x610f, 'CjkUnifiedIdeograph610f', [], 'Display', None],
  [ 0x6110, 'CjkUnifiedIdeograph6110', [], 'Display', None],
  [ 0x6111, 'CjkUnifiedIdeograph6111', [], 'Display', None],
  [ 0x6112, 'CjkUnifiedIdeograph6112', [], 'Display', None],
  [ 0x6113, 'CjkUnifiedIdeograph6113', [], 'Display', None],
  [ 0x6114, 'CjkUnifiedIdeograph6114', [], 'Display', None],
  [ 0x6115, 'CjkUnifiedIdeograph6115', [], 'Display', None],
  [ 0x6116, 'CjkUnifiedIdeograph6116', [], 'Display', None],
  [ 0x6117, 'CjkUnifiedIdeograph6117', [], 'Display', None],
  [ 0x6118, 'CjkUnifiedIdeograph6118', [], 'Display', None],
  [ 0x6119, 'CjkUnifiedIdeograph6119', [], 'Display', None],
  [ 0x611a, 'CjkUnifiedIdeograph611a', [], 'Display', None],
  [ 0x611b, 'CjkUnifiedIdeograph611b', [], 'Display', None],
  [ 0x611c, 'CjkUnifiedIdeograph611c', [], 'Display', None],
  [ 0x611d, 'CjkUnifiedIdeograph611d', [], 'Display', None],
  [ 0x611e, 'CjkUnifiedIdeograph611e', [], 'Display', None],
  [ 0x611f, 'CjkUnifiedIdeograph611f', [], 'Display', None],
  [ 0x6120, 'CjkUnifiedIdeograph6120', [], 'Display', None],
  [ 0x6121, 'CjkUnifiedIdeograph6121', [], 'Display', None],
  [ 0x6122, 'CjkUnifiedIdeograph6122', [], 'Display', None],
  [ 0x6123, 'CjkUnifiedIdeograph6123', [], 'Display', None],
  [ 0x6124, 'CjkUnifiedIdeograph6124', [], 'Display', None],
  [ 0x6125, 'CjkUnifiedIdeograph6125', [], 'Display', None],
  [ 0x6126, 'CjkUnifiedIdeograph6126', [], 'Display', None],
  [ 0x6127, 'CjkUnifiedIdeograph6127', [], 'Display', None],
  [ 0x6128, 'CjkUnifiedIdeograph6128', [], 'Display', None],
  [ 0x6129, 'CjkUnifiedIdeograph6129', [], 'Display', None],
  [ 0x612a, 'CjkUnifiedIdeograph612a', [], 'Display', None],
  [ 0x612b, 'CjkUnifiedIdeograph612b', [], 'Display', None],
  [ 0x612c, 'CjkUnifiedIdeograph612c', [], 'Display', None],
  [ 0x612d, 'CjkUnifiedIdeograph612d', [], 'Display', None],
  [ 0x612e, 'CjkUnifiedIdeograph612e', [], 'Display', None],
  [ 0x612f, 'CjkUnifiedIdeograph612f', [], 'Display', None],
  [ 0x6130, 'CjkUnifiedIdeograph6130', [], 'Display', None],
  [ 0x6131, 'CjkUnifiedIdeograph6131', [], 'Display', None],
  [ 0x6132, 'CjkUnifiedIdeograph6132', [], 'Display', None],
  [ 0x6133, 'CjkUnifiedIdeograph6133', [], 'Display', None],
  [ 0x6134, 'CjkUnifiedIdeograph6134', [], 'Display', None],
  [ 0x6135, 'CjkUnifiedIdeograph6135', [], 'Display', None],
  [ 0x6136, 'CjkUnifiedIdeograph6136', [], 'Display', None],
  [ 0x6137, 'CjkUnifiedIdeograph6137', [], 'Display', None],
  [ 0x6138, 'CjkUnifiedIdeograph6138', [], 'Display', None],
  [ 0x6139, 'CjkUnifiedIdeograph6139', [], 'Display', None],
  [ 0x613a, 'CjkUnifiedIdeograph613a', [], 'Display', None],
  [ 0x613b, 'CjkUnifiedIdeograph613b', [], 'Display', None],
  [ 0x613c, 'CjkUnifiedIdeograph613c', [], 'Display', None],
  [ 0x613d, 'CjkUnifiedIdeograph613d', [], 'Display', None],
  [ 0x613e, 'CjkUnifiedIdeograph613e', [], 'Display', None],
  [ 0x613f, 'CjkUnifiedIdeograph613f', [], 'Display', None],
  [ 0x6140, 'CjkUnifiedIdeograph6140', [], 'Display', None],
  [ 0x6141, 'CjkUnifiedIdeograph6141', [], 'Display', None],
  [ 0x6142, 'CjkUnifiedIdeograph6142', [], 'Display', None],
  [ 0x6143, 'CjkUnifiedIdeograph6143', [], 'Display', None],
  [ 0x6144, 'CjkUnifiedIdeograph6144', [], 'Display', None],
  [ 0x6145, 'CjkUnifiedIdeograph6145', [], 'Display', None],
  [ 0x6146, 'CjkUnifiedIdeograph6146', [], 'Display', None],
  [ 0x6147, 'CjkUnifiedIdeograph6147', [], 'Display', None],
  [ 0x6148, 'CjkUnifiedIdeograph6148', [], 'Display', None],
  [ 0x6149, 'CjkUnifiedIdeograph6149', [], 'Display', None],
  [ 0x614a, 'CjkUnifiedIdeograph614a', [], 'Display', None],
  [ 0x614b, 'CjkUnifiedIdeograph614b', [], 'Display', None],
  [ 0x614c, 'CjkUnifiedIdeograph614c', [], 'Display', None],
  [ 0x614d, 'CjkUnifiedIdeograph614d', [], 'Display', None],
  [ 0x614e, 'CjkUnifiedIdeograph614e', [], 'Display', None],
  [ 0x614f, 'CjkUnifiedIdeograph614f', [], 'Display', None],
  [ 0x6150, 'CjkUnifiedIdeograph6150', [], 'Display', None],
  [ 0x6151, 'CjkUnifiedIdeograph6151', [], 'Display', None],
  [ 0x6152, 'CjkUnifiedIdeograph6152', [], 'Display', None],
  [ 0x6153, 'CjkUnifiedIdeograph6153', [], 'Display', None],
  [ 0x6154, 'CjkUnifiedIdeograph6154', [], 'Display', None],
  [ 0x6155, 'CjkUnifiedIdeograph6155', [], 'Display', None],
  [ 0x6156, 'CjkUnifiedIdeograph6156', [], 'Display', None],
  [ 0x6157, 'CjkUnifiedIdeograph6157', [], 'Display', None],
  [ 0x6158, 'CjkUnifiedIdeograph6158', [], 'Display', None],
  [ 0x6159, 'CjkUnifiedIdeograph6159', [], 'Display', None],
  [ 0x615a, 'CjkUnifiedIdeograph615a', [], 'Display', None],
  [ 0x615b, 'CjkUnifiedIdeograph615b', [], 'Display', None],
  [ 0x615c, 'CjkUnifiedIdeograph615c', [], 'Display', None],
  [ 0x615d, 'CjkUnifiedIdeograph615d', [], 'Display', None],
  [ 0x615e, 'CjkUnifiedIdeograph615e', [], 'Display', None],
  [ 0x615f, 'CjkUnifiedIdeograph615f', [], 'Display', None],
  [ 0x6160, 'CjkUnifiedIdeograph6160', [], 'Display', None],
  [ 0x6161, 'CjkUnifiedIdeograph6161', [], 'Display', None],
  [ 0x6162, 'CjkUnifiedIdeograph6162', [], 'Display', None],
  [ 0x6163, 'CjkUnifiedIdeograph6163', [], 'Display', None],
  [ 0x6164, 'CjkUnifiedIdeograph6164', [], 'Display', None],
  [ 0x6165, 'CjkUnifiedIdeograph6165', [], 'Display', None],
  [ 0x6166, 'CjkUnifiedIdeograph6166', [], 'Display', None],
  [ 0x6167, 'CjkUnifiedIdeograph6167', [], 'Display', None],
  [ 0x6168, 'CjkUnifiedIdeograph6168', [], 'Display', None],
  [ 0x6169, 'CjkUnifiedIdeograph6169', [], 'Display', None],
  [ 0x616a, 'CjkUnifiedIdeograph616a', [], 'Display', None],
  [ 0x616b, 'CjkUnifiedIdeograph616b', [], 'Display', None],
  [ 0x616c, 'CjkUnifiedIdeograph616c', [], 'Display', None],
  [ 0x616d, 'CjkUnifiedIdeograph616d', [], 'Display', None],
  [ 0x616e, 'CjkUnifiedIdeograph616e', [], 'Display', None],
  [ 0x616f, 'CjkUnifiedIdeograph616f', [], 'Display', None],
  [ 0x6170, 'CjkUnifiedIdeograph6170', [], 'Display', None],
  [ 0x6171, 'CjkUnifiedIdeograph6171', [], 'Display', None],
  [ 0x6172, 'CjkUnifiedIdeograph6172', [], 'Display', None],
  [ 0x6173, 'CjkUnifiedIdeograph6173', [], 'Display', None],
  [ 0x6174, 'CjkUnifiedIdeograph6174', [], 'Display', None],
  [ 0x6175, 'CjkUnifiedIdeograph6175', [], 'Display', None],
  [ 0x6176, 'CjkUnifiedIdeograph6176', [], 'Display', None],
  [ 0x6177, 'CjkUnifiedIdeograph6177', [], 'Display', None],
  [ 0x6178, 'CjkUnifiedIdeograph6178', [], 'Display', None],
  [ 0x6179, 'CjkUnifiedIdeograph6179', [], 'Display', None],
  [ 0x617a, 'CjkUnifiedIdeograph617a', [], 'Display', None],
  [ 0x617b, 'CjkUnifiedIdeograph617b', [], 'Display', None],
  [ 0x617c, 'CjkUnifiedIdeograph617c', [], 'Display', None],
  [ 0x617d, 'CjkUnifiedIdeograph617d', [], 'Display', None],
  [ 0x617e, 'CjkUnifiedIdeograph617e', [], 'Display', None],
  [ 0x617f, 'CjkUnifiedIdeograph617f', [], 'Display', None],
  [ 0x6180, 'CjkUnifiedIdeograph6180', [], 'Display', None],
  [ 0x6181, 'CjkUnifiedIdeograph6181', [], 'Display', None],
  [ 0x6182, 'CjkUnifiedIdeograph6182', [], 'Display', None],
  [ 0x6183, 'CjkUnifiedIdeograph6183', [], 'Display', None],
  [ 0x6184, 'CjkUnifiedIdeograph6184', [], 'Display', None],
  [ 0x6185, 'CjkUnifiedIdeograph6185', [], 'Display', None],
  [ 0x6186, 'CjkUnifiedIdeograph6186', [], 'Display', None],
  [ 0x6187, 'CjkUnifiedIdeograph6187', [], 'Display', None],
  [ 0x6188, 'CjkUnifiedIdeograph6188', [], 'Display', None],
  [ 0x6189, 'CjkUnifiedIdeograph6189', [], 'Display', None],
  [ 0x618a, 'CjkUnifiedIdeograph618a', [], 'Display', None],
  [ 0x618b, 'CjkUnifiedIdeograph618b', [], 'Display', None],
  [ 0x618c, 'CjkUnifiedIdeograph618c', [], 'Display', None],
  [ 0x618d, 'CjkUnifiedIdeograph618d', [], 'Display', None],
  [ 0x618e, 'CjkUnifiedIdeograph618e', [], 'Display', None],
  [ 0x618f, 'CjkUnifiedIdeograph618f', [], 'Display', None],
  [ 0x6190, 'CjkUnifiedIdeograph6190', [], 'Display', None],
  [ 0x6191, 'CjkUnifiedIdeograph6191', [], 'Display', None],
  [ 0x6192, 'CjkUnifiedIdeograph6192', [], 'Display', None],
  [ 0x6193, 'CjkUnifiedIdeograph6193', [], 'Display', None],
  [ 0x6194, 'CjkUnifiedIdeograph6194', [], 'Display', None],
  [ 0x6195, 'CjkUnifiedIdeograph6195', [], 'Display', None],
  [ 0x6196, 'CjkUnifiedIdeograph6196', [], 'Display', None],
  [ 0x6197, 'CjkUnifiedIdeograph6197', [], 'Display', None],
  [ 0x6198, 'CjkUnifiedIdeograph6198', [], 'Display', None],
  [ 0x6199, 'CjkUnifiedIdeograph6199', [], 'Display', None],
  [ 0x619a, 'CjkUnifiedIdeograph619a', [], 'Display', None],
  [ 0x619b, 'CjkUnifiedIdeograph619b', [], 'Display', None],
  [ 0x619c, 'CjkUnifiedIdeograph619c', [], 'Display', None],
  [ 0x619d, 'CjkUnifiedIdeograph619d', [], 'Display', None],
  [ 0x619e, 'CjkUnifiedIdeograph619e', [], 'Display', None],
  [ 0x619f, 'CjkUnifiedIdeograph619f', [], 'Display', None],
  [ 0x61a0, 'CjkUnifiedIdeograph61a0', [], 'Display', None],
  [ 0x61a1, 'CjkUnifiedIdeograph61a1', [], 'Display', None],
  [ 0x61a2, 'CjkUnifiedIdeograph61a2', [], 'Display', None],
  [ 0x61a3, 'CjkUnifiedIdeograph61a3', [], 'Display', None],
  [ 0x61a4, 'CjkUnifiedIdeograph61a4', [], 'Display', None],
  [ 0x61a5, 'CjkUnifiedIdeograph61a5', [], 'Display', None],
  [ 0x61a6, 'CjkUnifiedIdeograph61a6', [], 'Display', None],
  [ 0x61a7, 'CjkUnifiedIdeograph61a7', [], 'Display', None],
  [ 0x61a8, 'CjkUnifiedIdeograph61a8', [], 'Display', None],
  [ 0x61a9, 'CjkUnifiedIdeograph61a9', [], 'Display', None],
  [ 0x61aa, 'CjkUnifiedIdeograph61aa', [], 'Display', None],
  [ 0x61ab, 'CjkUnifiedIdeograph61ab', [], 'Display', None],
  [ 0x61ac, 'CjkUnifiedIdeograph61ac', [], 'Display', None],
  [ 0x61ad, 'CjkUnifiedIdeograph61ad', [], 'Display', None],
  [ 0x61ae, 'CjkUnifiedIdeograph61ae', [], 'Display', None],
  [ 0x61af, 'CjkUnifiedIdeograph61af', [], 'Display', None],
  [ 0x61b0, 'CjkUnifiedIdeograph61b0', [], 'Display', None],
  [ 0x61b1, 'CjkUnifiedIdeograph61b1', [], 'Display', None],
  [ 0x61b2, 'CjkUnifiedIdeograph61b2', [], 'Display', None],
  [ 0x61b3, 'CjkUnifiedIdeograph61b3', [], 'Display', None],
  [ 0x61b4, 'CjkUnifiedIdeograph61b4', [], 'Display', None],
  [ 0x61b5, 'CjkUnifiedIdeograph61b5', [], 'Display', None],
  [ 0x61b6, 'CjkUnifiedIdeograph61b6', [], 'Display', None],
  [ 0x61b7, 'CjkUnifiedIdeograph61b7', [], 'Display', None],
  [ 0x61b8, 'CjkUnifiedIdeograph61b8', [], 'Display', None],
  [ 0x61b9, 'CjkUnifiedIdeograph61b9', [], 'Display', None],
  [ 0x61ba, 'CjkUnifiedIdeograph61ba', [], 'Display', None],
  [ 0x61bb, 'CjkUnifiedIdeograph61bb', [], 'Display', None],
  [ 0x61bc, 'CjkUnifiedIdeograph61bc', [], 'Display', None],
  [ 0x61bd, 'CjkUnifiedIdeograph61bd', [], 'Display', None],
  [ 0x61be, 'CjkUnifiedIdeograph61be', [], 'Display', None],
  [ 0x61bf, 'CjkUnifiedIdeograph61bf', [], 'Display', None],
  [ 0x61c0, 'CjkUnifiedIdeograph61c0', [], 'Display', None],
  [ 0x61c1, 'CjkUnifiedIdeograph61c1', [], 'Display', None],
  [ 0x61c2, 'CjkUnifiedIdeograph61c2', [], 'Display', None],
  [ 0x61c3, 'CjkUnifiedIdeograph61c3', [], 'Display', None],
  [ 0x61c4, 'CjkUnifiedIdeograph61c4', [], 'Display', None],
  [ 0x61c5, 'CjkUnifiedIdeograph61c5', [], 'Display', None],
  [ 0x61c6, 'CjkUnifiedIdeograph61c6', [], 'Display', None],
  [ 0x61c7, 'CjkUnifiedIdeograph61c7', [], 'Display', None],
  [ 0x61c8, 'CjkUnifiedIdeograph61c8', [], 'Display', None],
  [ 0x61c9, 'CjkUnifiedIdeograph61c9', [], 'Display', None],
  [ 0x61ca, 'CjkUnifiedIdeograph61ca', [], 'Display', None],
  [ 0x61cb, 'CjkUnifiedIdeograph61cb', [], 'Display', None],
  [ 0x61cc, 'CjkUnifiedIdeograph61cc', [], 'Display', None],
  [ 0x61cd, 'CjkUnifiedIdeograph61cd', [], 'Display', None],
  [ 0x61ce, 'CjkUnifiedIdeograph61ce', [], 'Display', None],
  [ 0x61cf, 'CjkUnifiedIdeograph61cf', [], 'Display', None],
  [ 0x61d0, 'CjkUnifiedIdeograph61d0', [], 'Display', None],
  [ 0x61d1, 'CjkUnifiedIdeograph61d1', [], 'Display', None],
  [ 0x61d2, 'CjkUnifiedIdeograph61d2', [], 'Display', None],
  [ 0x61d3, 'CjkUnifiedIdeograph61d3', [], 'Display', None],
  [ 0x61d4, 'CjkUnifiedIdeograph61d4', [], 'Display', None],
  [ 0x61d5, 'CjkUnifiedIdeograph61d5', [], 'Display', None],
  [ 0x61d6, 'CjkUnifiedIdeograph61d6', [], 'Display', None],
  [ 0x61d7, 'CjkUnifiedIdeograph61d7', [], 'Display', None],
  [ 0x61d8, 'CjkUnifiedIdeograph61d8', [], 'Display', None],
  [ 0x61d9, 'CjkUnifiedIdeograph61d9', [], 'Display', None],
  [ 0x61da, 'CjkUnifiedIdeograph61da', [], 'Display', None],
  [ 0x61db, 'CjkUnifiedIdeograph61db', [], 'Display', None],
  [ 0x61dc, 'CjkUnifiedIdeograph61dc', [], 'Display', None],
  [ 0x61dd, 'CjkUnifiedIdeograph61dd', [], 'Display', None],
  [ 0x61de, 'CjkUnifiedIdeograph61de', [], 'Display', None],
  [ 0x61df, 'CjkUnifiedIdeograph61df', [], 'Display', None],
  [ 0x61e0, 'CjkUnifiedIdeograph61e0', [], 'Display', None],
  [ 0x61e1, 'CjkUnifiedIdeograph61e1', [], 'Display', None],
  [ 0x61e2, 'CjkUnifiedIdeograph61e2', [], 'Display', None],
  [ 0x61e3, 'CjkUnifiedIdeograph61e3', [], 'Display', None],
  [ 0x61e4, 'CjkUnifiedIdeograph61e4', [], 'Display', None],
  [ 0x61e5, 'CjkUnifiedIdeograph61e5', [], 'Display', None],
  [ 0x61e6, 'CjkUnifiedIdeograph61e6', [], 'Display', None],
  [ 0x61e7, 'CjkUnifiedIdeograph61e7', [], 'Display', None],
  [ 0x61e8, 'CjkUnifiedIdeograph61e8', [], 'Display', None],
  [ 0x61e9, 'CjkUnifiedIdeograph61e9', [], 'Display', None],
  [ 0x61ea, 'CjkUnifiedIdeograph61ea', [], 'Display', None],
  [ 0x61eb, 'CjkUnifiedIdeograph61eb', [], 'Display', None],
  [ 0x61ec, 'CjkUnifiedIdeograph61ec', [], 'Display', None],
  [ 0x61ed, 'CjkUnifiedIdeograph61ed', [], 'Display', None],
  [ 0x61ee, 'CjkUnifiedIdeograph61ee', [], 'Display', None],
  [ 0x61ef, 'CjkUnifiedIdeograph61ef', [], 'Display', None],
  [ 0x61f0, 'CjkUnifiedIdeograph61f0', [], 'Display', None],
  [ 0x61f1, 'CjkUnifiedIdeograph61f1', [], 'Display', None],
  [ 0x61f2, 'CjkUnifiedIdeograph61f2', [], 'Display', None],
  [ 0x61f3, 'CjkUnifiedIdeograph61f3', [], 'Display', None],
  [ 0x61f4, 'CjkUnifiedIdeograph61f4', [], 'Display', None],
  [ 0x61f5, 'CjkUnifiedIdeograph61f5', [], 'Display', None],
  [ 0x61f6, 'CjkUnifiedIdeograph61f6', [], 'Display', None],
  [ 0x61f7, 'CjkUnifiedIdeograph61f7', [], 'Display', None],
  [ 0x61f8, 'CjkUnifiedIdeograph61f8', [], 'Display', None],
  [ 0x61f9, 'CjkUnifiedIdeograph61f9', [], 'Display', None],
  [ 0x61fa, 'CjkUnifiedIdeograph61fa', [], 'Display', None],
  [ 0x61fb, 'CjkUnifiedIdeograph61fb', [], 'Display', None],
  [ 0x61fc, 'CjkUnifiedIdeograph61fc', [], 'Display', None],
  [ 0x61fd, 'CjkUnifiedIdeograph61fd', [], 'Display', None],
  [ 0x61fe, 'CjkUnifiedIdeograph61fe', [], 'Display', None],
  [ 0x61ff, 'CjkUnifiedIdeograph61ff', [], 'Display', None],
  [ 0x6200, 'CjkUnifiedIdeograph6200', [], 'Display', None],
  [ 0x6201, 'CjkUnifiedIdeograph6201', [], 'Display', None],
  [ 0x6202, 'CjkUnifiedIdeograph6202', [], 'Display', None],
  [ 0x6203, 'CjkUnifiedIdeograph6203', [], 'Display', None],
  [ 0x6204, 'CjkUnifiedIdeograph6204', [], 'Display', None],
  [ 0x6205, 'CjkUnifiedIdeograph6205', [], 'Display', None],
  [ 0x6206, 'CjkUnifiedIdeograph6206', [], 'Display', None],
  [ 0x6207, 'CjkUnifiedIdeograph6207', [], 'Display', None],
  [ 0x6208, 'CjkUnifiedIdeograph6208', [], 'Display', None],
  [ 0x6209, 'CjkUnifiedIdeograph6209', [], 'Display', None],
  [ 0x620a, 'CjkUnifiedIdeograph620a', [], 'Display', None],
  [ 0x620b, 'CjkUnifiedIdeograph620b', [], 'Display', None],
  [ 0x620c, 'CjkUnifiedIdeograph620c', [], 'Display', None],
  [ 0x620d, 'CjkUnifiedIdeograph620d', [], 'Display', None],
  [ 0x620e, 'CjkUnifiedIdeograph620e', [], 'Display', None],
  [ 0x620f, 'CjkUnifiedIdeograph620f', [], 'Display', None],
  [ 0x6210, 'CjkUnifiedIdeograph6210', [], 'Display', None],
  [ 0x6211, 'CjkUnifiedIdeograph6211', [], 'Display', None],
  [ 0x6212, 'CjkUnifiedIdeograph6212', [], 'Display', None],
  [ 0x6213, 'CjkUnifiedIdeograph6213', [], 'Display', None],
  [ 0x6214, 'CjkUnifiedIdeograph6214', [], 'Display', None],
  [ 0x6215, 'CjkUnifiedIdeograph6215', [], 'Display', None],
  [ 0x6216, 'CjkUnifiedIdeograph6216', [], 'Display', None],
  [ 0x6217, 'CjkUnifiedIdeograph6217', [], 'Display', None],
  [ 0x6218, 'CjkUnifiedIdeograph6218', [], 'Display', None],
  [ 0x6219, 'CjkUnifiedIdeograph6219', [], 'Display', None],
  [ 0x621a, 'CjkUnifiedIdeograph621a', [], 'Display', None],
  [ 0x621b, 'CjkUnifiedIdeograph621b', [], 'Display', None],
  [ 0x621c, 'CjkUnifiedIdeograph621c', [], 'Display', None],
  [ 0x621d, 'CjkUnifiedIdeograph621d', [], 'Display', None],
  [ 0x621e, 'CjkUnifiedIdeograph621e', [], 'Display', None],
  [ 0x621f, 'CjkUnifiedIdeograph621f', [], 'Display', None],
  [ 0x6220, 'CjkUnifiedIdeograph6220', [], 'Display', None],
  [ 0x6221, 'CjkUnifiedIdeograph6221', [], 'Display', None],
  [ 0x6222, 'CjkUnifiedIdeograph6222', [], 'Display', None],
  [ 0x6223, 'CjkUnifiedIdeograph6223', [], 'Display', None],
  [ 0x6224, 'CjkUnifiedIdeograph6224', [], 'Display', None],
  [ 0x6225, 'CjkUnifiedIdeograph6225', [], 'Display', None],
  [ 0x6226, 'CjkUnifiedIdeograph6226', [], 'Display', None],
  [ 0x6227, 'CjkUnifiedIdeograph6227', [], 'Display', None],
  [ 0x6228, 'CjkUnifiedIdeograph6228', [], 'Display', None],
  [ 0x6229, 'CjkUnifiedIdeograph6229', [], 'Display', None],
  [ 0x622a, 'CjkUnifiedIdeograph622a', [], 'Display', None],
  [ 0x622b, 'CjkUnifiedIdeograph622b', [], 'Display', None],
  [ 0x622c, 'CjkUnifiedIdeograph622c', [], 'Display', None],
  [ 0x622d, 'CjkUnifiedIdeograph622d', [], 'Display', None],
  [ 0x622e, 'CjkUnifiedIdeograph622e', [], 'Display', None],
  [ 0x622f, 'CjkUnifiedIdeograph622f', [], 'Display', None],
  [ 0x6230, 'CjkUnifiedIdeograph6230', [], 'Display', None],
  [ 0x6231, 'CjkUnifiedIdeograph6231', [], 'Display', None],
  [ 0x6232, 'CjkUnifiedIdeograph6232', [], 'Display', None],
  [ 0x6233, 'CjkUnifiedIdeograph6233', [], 'Display', None],
  [ 0x6234, 'CjkUnifiedIdeograph6234', [], 'Display', None],
  [ 0x6235, 'CjkUnifiedIdeograph6235', [], 'Display', None],
  [ 0x6236, 'CjkUnifiedIdeograph6236', [], 'Display', None],
  [ 0x6237, 'CjkUnifiedIdeograph6237', [], 'Display', None],
  [ 0x6238, 'CjkUnifiedIdeograph6238', [], 'Display', None],
  [ 0x6239, 'CjkUnifiedIdeograph6239', [], 'Display', None],
  [ 0x623a, 'CjkUnifiedIdeograph623a', [], 'Display', None],
  [ 0x623b, 'CjkUnifiedIdeograph623b', [], 'Display', None],
  [ 0x623c, 'CjkUnifiedIdeograph623c', [], 'Display', None],
  [ 0x623d, 'CjkUnifiedIdeograph623d', [], 'Display', None],
  [ 0x623e, 'CjkUnifiedIdeograph623e', [], 'Display', None],
  [ 0x623f, 'CjkUnifiedIdeograph623f', [], 'Display', None],
  [ 0x6240, 'CjkUnifiedIdeograph6240', [], 'Display', None],
  [ 0x6241, 'CjkUnifiedIdeograph6241', [], 'Display', None],
  [ 0x6242, 'CjkUnifiedIdeograph6242', [], 'Display', None],
  [ 0x6243, 'CjkUnifiedIdeograph6243', [], 'Display', None],
  [ 0x6244, 'CjkUnifiedIdeograph6244', [], 'Display', None],
  [ 0x6245, 'CjkUnifiedIdeograph6245', [], 'Display', None],
  [ 0x6246, 'CjkUnifiedIdeograph6246', [], 'Display', None],
  [ 0x6247, 'CjkUnifiedIdeograph6247', [], 'Display', None],
  [ 0x6248, 'CjkUnifiedIdeograph6248', [], 'Display', None],
  [ 0x6249, 'CjkUnifiedIdeograph6249', [], 'Display', None],
  [ 0x624a, 'CjkUnifiedIdeograph624a', [], 'Display', None],
  [ 0x624b, 'CjkUnifiedIdeograph624b', [], 'Display', None],
  [ 0x624c, 'CjkUnifiedIdeograph624c', [], 'Display', None],
  [ 0x624d, 'CjkUnifiedIdeograph624d', [], 'Display', None],
  [ 0x624e, 'CjkUnifiedIdeograph624e', [], 'Display', None],
  [ 0x624f, 'CjkUnifiedIdeograph624f', [], 'Display', None],
  [ 0x6250, 'CjkUnifiedIdeograph6250', [], 'Display', None],
  [ 0x6251, 'CjkUnifiedIdeograph6251', [], 'Display', None],
  [ 0x6252, 'CjkUnifiedIdeograph6252', [], 'Display', None],
  [ 0x6253, 'CjkUnifiedIdeograph6253', [], 'Display', None],
  [ 0x6254, 'CjkUnifiedIdeograph6254', [], 'Display', None],
  [ 0x6255, 'CjkUnifiedIdeograph6255', [], 'Display', None],
  [ 0x6256, 'CjkUnifiedIdeograph6256', [], 'Display', None],
  [ 0x6257, 'CjkUnifiedIdeograph6257', [], 'Display', None],
  [ 0x6258, 'CjkUnifiedIdeograph6258', [], 'Display', None],
  [ 0x6259, 'CjkUnifiedIdeograph6259', [], 'Display', None],
  [ 0x625a, 'CjkUnifiedIdeograph625a', [], 'Display', None],
  [ 0x625b, 'CjkUnifiedIdeograph625b', [], 'Display', None],
  [ 0x625c, 'CjkUnifiedIdeograph625c', [], 'Display', None],
  [ 0x625d, 'CjkUnifiedIdeograph625d', [], 'Display', None],
  [ 0x625e, 'CjkUnifiedIdeograph625e', [], 'Display', None],
  [ 0x625f, 'CjkUnifiedIdeograph625f', [], 'Display', None],
  [ 0x6260, 'CjkUnifiedIdeograph6260', [], 'Display', None],
  [ 0x6261, 'CjkUnifiedIdeograph6261', [], 'Display', None],
  [ 0x6262, 'CjkUnifiedIdeograph6262', [], 'Display', None],
  [ 0x6263, 'CjkUnifiedIdeograph6263', [], 'Display', None],
  [ 0x6264, 'CjkUnifiedIdeograph6264', [], 'Display', None],
  [ 0x6265, 'CjkUnifiedIdeograph6265', [], 'Display', None],
  [ 0x6266, 'CjkUnifiedIdeograph6266', [], 'Display', None],
  [ 0x6267, 'CjkUnifiedIdeograph6267', [], 'Display', None],
  [ 0x6268, 'CjkUnifiedIdeograph6268', [], 'Display', None],
  [ 0x6269, 'CjkUnifiedIdeograph6269', [], 'Display', None],
  [ 0x626a, 'CjkUnifiedIdeograph626a', [], 'Display', None],
  [ 0x626b, 'CjkUnifiedIdeograph626b', [], 'Display', None],
  [ 0x626c, 'CjkUnifiedIdeograph626c', [], 'Display', None],
  [ 0x626d, 'CjkUnifiedIdeograph626d', [], 'Display', None],
  [ 0x626e, 'CjkUnifiedIdeograph626e', [], 'Display', None],
  [ 0x626f, 'CjkUnifiedIdeograph626f', [], 'Display', None],
  [ 0x6270, 'CjkUnifiedIdeograph6270', [], 'Display', None],
  [ 0x6271, 'CjkUnifiedIdeograph6271', [], 'Display', None],
  [ 0x6272, 'CjkUnifiedIdeograph6272', [], 'Display', None],
  [ 0x6273, 'CjkUnifiedIdeograph6273', [], 'Display', None],
  [ 0x6274, 'CjkUnifiedIdeograph6274', [], 'Display', None],
  [ 0x6275, 'CjkUnifiedIdeograph6275', [], 'Display', None],
  [ 0x6276, 'CjkUnifiedIdeograph6276', [], 'Display', None],
  [ 0x6277, 'CjkUnifiedIdeograph6277', [], 'Display', None],
  [ 0x6278, 'CjkUnifiedIdeograph6278', [], 'Display', None],
  [ 0x6279, 'CjkUnifiedIdeograph6279', [], 'Display', None],
  [ 0x627a, 'CjkUnifiedIdeograph627a', [], 'Display', None],
  [ 0x627b, 'CjkUnifiedIdeograph627b', [], 'Display', None],
  [ 0x627c, 'CjkUnifiedIdeograph627c', [], 'Display', None],
  [ 0x627d, 'CjkUnifiedIdeograph627d', [], 'Display', None],
  [ 0x627e, 'CjkUnifiedIdeograph627e', [], 'Display', None],
  [ 0x627f, 'CjkUnifiedIdeograph627f', [], 'Display', None],
  [ 0x6280, 'CjkUnifiedIdeograph6280', [], 'Display', None],
  [ 0x6281, 'CjkUnifiedIdeograph6281', [], 'Display', None],
  [ 0x6282, 'CjkUnifiedIdeograph6282', [], 'Display', None],
  [ 0x6283, 'CjkUnifiedIdeograph6283', [], 'Display', None],
  [ 0x6284, 'CjkUnifiedIdeograph6284', [], 'Display', None],
  [ 0x6285, 'CjkUnifiedIdeograph6285', [], 'Display', None],
  [ 0x6286, 'CjkUnifiedIdeograph6286', [], 'Display', None],
  [ 0x6287, 'CjkUnifiedIdeograph6287', [], 'Display', None],
  [ 0x6288, 'CjkUnifiedIdeograph6288', [], 'Display', None],
  [ 0x6289, 'CjkUnifiedIdeograph6289', [], 'Display', None],
  [ 0x628a, 'CjkUnifiedIdeograph628a', [], 'Display', None],
  [ 0x628b, 'CjkUnifiedIdeograph628b', [], 'Display', None],
  [ 0x628c, 'CjkUnifiedIdeograph628c', [], 'Display', None],
  [ 0x628d, 'CjkUnifiedIdeograph628d', [], 'Display', None],
  [ 0x628e, 'CjkUnifiedIdeograph628e', [], 'Display', None],
  [ 0x628f, 'CjkUnifiedIdeograph628f', [], 'Display', None],
  [ 0x6290, 'CjkUnifiedIdeograph6290', [], 'Display', None],
  [ 0x6291, 'CjkUnifiedIdeograph6291', [], 'Display', None],
  [ 0x6292, 'CjkUnifiedIdeograph6292', [], 'Display', None],
  [ 0x6293, 'CjkUnifiedIdeograph6293', [], 'Display', None],
  [ 0x6294, 'CjkUnifiedIdeograph6294', [], 'Display', None],
  [ 0x6295, 'CjkUnifiedIdeograph6295', [], 'Display', None],
  [ 0x6296, 'CjkUnifiedIdeograph6296', [], 'Display', None],
  [ 0x6297, 'CjkUnifiedIdeograph6297', [], 'Display', None],
  [ 0x6298, 'CjkUnifiedIdeograph6298', [], 'Display', None],
  [ 0x6299, 'CjkUnifiedIdeograph6299', [], 'Display', None],
  [ 0x629a, 'CjkUnifiedIdeograph629a', [], 'Display', None],
  [ 0x629b, 'CjkUnifiedIdeograph629b', [], 'Display', None],
  [ 0x629c, 'CjkUnifiedIdeograph629c', [], 'Display', None],
  [ 0x629d, 'CjkUnifiedIdeograph629d', [], 'Display', None],
  [ 0x629e, 'CjkUnifiedIdeograph629e', [], 'Display', None],
  [ 0x629f, 'CjkUnifiedIdeograph629f', [], 'Display', None],
  [ 0x62a0, 'CjkUnifiedIdeograph62a0', [], 'Display', None],
  [ 0x62a1, 'CjkUnifiedIdeograph62a1', [], 'Display', None],
  [ 0x62a2, 'CjkUnifiedIdeograph62a2', [], 'Display', None],
  [ 0x62a3, 'CjkUnifiedIdeograph62a3', [], 'Display', None],
  [ 0x62a4, 'CjkUnifiedIdeograph62a4', [], 'Display', None],
  [ 0x62a5, 'CjkUnifiedIdeograph62a5', [], 'Display', None],
  [ 0x62a6, 'CjkUnifiedIdeograph62a6', [], 'Display', None],
  [ 0x62a7, 'CjkUnifiedIdeograph62a7', [], 'Display', None],
  [ 0x62a8, 'CjkUnifiedIdeograph62a8', [], 'Display', None],
  [ 0x62a9, 'CjkUnifiedIdeograph62a9', [], 'Display', None],
  [ 0x62aa, 'CjkUnifiedIdeograph62aa', [], 'Display', None],
  [ 0x62ab, 'CjkUnifiedIdeograph62ab', [], 'Display', None],
  [ 0x62ac, 'CjkUnifiedIdeograph62ac', [], 'Display', None],
  [ 0x62ad, 'CjkUnifiedIdeograph62ad', [], 'Display', None],
  [ 0x62ae, 'CjkUnifiedIdeograph62ae', [], 'Display', None],
  [ 0x62af, 'CjkUnifiedIdeograph62af', [], 'Display', None],
  [ 0x62b0, 'CjkUnifiedIdeograph62b0', [], 'Display', None],
  [ 0x62b1, 'CjkUnifiedIdeograph62b1', [], 'Display', None],
  [ 0x62b2, 'CjkUnifiedIdeograph62b2', [], 'Display', None],
  [ 0x62b3, 'CjkUnifiedIdeograph62b3', [], 'Display', None],
  [ 0x62b4, 'CjkUnifiedIdeograph62b4', [], 'Display', None],
  [ 0x62b5, 'CjkUnifiedIdeograph62b5', [], 'Display', None],
  [ 0x62b6, 'CjkUnifiedIdeograph62b6', [], 'Display', None],
  [ 0x62b7, 'CjkUnifiedIdeograph62b7', [], 'Display', None],
  [ 0x62b8, 'CjkUnifiedIdeograph62b8', [], 'Display', None],
  [ 0x62b9, 'CjkUnifiedIdeograph62b9', [], 'Display', None],
  [ 0x62ba, 'CjkUnifiedIdeograph62ba', [], 'Display', None],
  [ 0x62bb, 'CjkUnifiedIdeograph62bb', [], 'Display', None],
  [ 0x62bc, 'CjkUnifiedIdeograph62bc', [], 'Display', None],
  [ 0x62bd, 'CjkUnifiedIdeograph62bd', [], 'Display', None],
  [ 0x62be, 'CjkUnifiedIdeograph62be', [], 'Display', None],
  [ 0x62bf, 'CjkUnifiedIdeograph62bf', [], 'Display', None],
  [ 0x62c0, 'CjkUnifiedIdeograph62c0', [], 'Display', None],
  [ 0x62c1, 'CjkUnifiedIdeograph62c1', [], 'Display', None],
  [ 0x62c2, 'CjkUnifiedIdeograph62c2', [], 'Display', None],
  [ 0x62c3, 'CjkUnifiedIdeograph62c3', [], 'Display', None],
  [ 0x62c4, 'CjkUnifiedIdeograph62c4', [], 'Display', None],
  [ 0x62c5, 'CjkUnifiedIdeograph62c5', [], 'Display', None],
  [ 0x62c6, 'CjkUnifiedIdeograph62c6', [], 'Display', None],
  [ 0x62c7, 'CjkUnifiedIdeograph62c7', [], 'Display', None],
  [ 0x62c8, 'CjkUnifiedIdeograph62c8', [], 'Display', None],
  [ 0x62c9, 'CjkUnifiedIdeograph62c9', [], 'Display', None],
  [ 0x62ca, 'CjkUnifiedIdeograph62ca', [], 'Display', None],
  [ 0x62cb, 'CjkUnifiedIdeograph62cb', [], 'Display', None],
  [ 0x62cc, 'CjkUnifiedIdeograph62cc', [], 'Display', None],
  [ 0x62cd, 'CjkUnifiedIdeograph62cd', [], 'Display', None],
  [ 0x62ce, 'CjkUnifiedIdeograph62ce', [], 'Display', None],
  [ 0x62cf, 'CjkUnifiedIdeograph62cf', [], 'Display', None],
  [ 0x62d0, 'CjkUnifiedIdeograph62d0', [], 'Display', None],
  [ 0x62d1, 'CjkUnifiedIdeograph62d1', [], 'Display', None],
  [ 0x62d2, 'CjkUnifiedIdeograph62d2', [], 'Display', None],
  [ 0x62d3, 'CjkUnifiedIdeograph62d3', [], 'Display', None],
  [ 0x62d4, 'CjkUnifiedIdeograph62d4', [], 'Display', None],
  [ 0x62d5, 'CjkUnifiedIdeograph62d5', [], 'Display', None],
  [ 0x62d6, 'CjkUnifiedIdeograph62d6', [], 'Display', None],
  [ 0x62d7, 'CjkUnifiedIdeograph62d7', [], 'Display', None],
  [ 0x62d8, 'CjkUnifiedIdeograph62d8', [], 'Display', None],
  [ 0x62d9, 'CjkUnifiedIdeograph62d9', [], 'Display', None],
  [ 0x62da, 'CjkUnifiedIdeograph62da', [], 'Display', None],
  [ 0x62db, 'CjkUnifiedIdeograph62db', [], 'Display', None],
  [ 0x62dc, 'CjkUnifiedIdeograph62dc', [], 'Display', None],
  [ 0x62dd, 'CjkUnifiedIdeograph62dd', [], 'Display', None],
  [ 0x62de, 'CjkUnifiedIdeograph62de', [], 'Display', None],
  [ 0x62df, 'CjkUnifiedIdeograph62df', [], 'Display', None],
  [ 0x62e0, 'CjkUnifiedIdeograph62e0', [], 'Display', None],
  [ 0x62e1, 'CjkUnifiedIdeograph62e1', [], 'Display', None],
  [ 0x62e2, 'CjkUnifiedIdeograph62e2', [], 'Display', None],
  [ 0x62e3, 'CjkUnifiedIdeograph62e3', [], 'Display', None],
  [ 0x62e4, 'CjkUnifiedIdeograph62e4', [], 'Display', None],
  [ 0x62e5, 'CjkUnifiedIdeograph62e5', [], 'Display', None],
  [ 0x62e6, 'CjkUnifiedIdeograph62e6', [], 'Display', None],
  [ 0x62e7, 'CjkUnifiedIdeograph62e7', [], 'Display', None],
  [ 0x62e8, 'CjkUnifiedIdeograph62e8', [], 'Display', None],
  [ 0x62e9, 'CjkUnifiedIdeograph62e9', [], 'Display', None],
  [ 0x62ea, 'CjkUnifiedIdeograph62ea', [], 'Display', None],
  [ 0x62eb, 'CjkUnifiedIdeograph62eb', [], 'Display', None],
  [ 0x62ec, 'CjkUnifiedIdeograph62ec', [], 'Display', None],
  [ 0x62ed, 'CjkUnifiedIdeograph62ed', [], 'Display', None],
  [ 0x62ee, 'CjkUnifiedIdeograph62ee', [], 'Display', None],
  [ 0x62ef, 'CjkUnifiedIdeograph62ef', [], 'Display', None],
  [ 0x62f0, 'CjkUnifiedIdeograph62f0', [], 'Display', None],
  [ 0x62f1, 'CjkUnifiedIdeograph62f1', [], 'Display', None],
  [ 0x62f2, 'CjkUnifiedIdeograph62f2', [], 'Display', None],
  [ 0x62f3, 'CjkUnifiedIdeograph62f3', [], 'Display', None],
  [ 0x62f4, 'CjkUnifiedIdeograph62f4', [], 'Display', None],
  [ 0x62f5, 'CjkUnifiedIdeograph62f5', [], 'Display', None],
  [ 0x62f6, 'CjkUnifiedIdeograph62f6', [], 'Display', None],
  [ 0x62f7, 'CjkUnifiedIdeograph62f7', [], 'Display', None],
  [ 0x62f8, 'CjkUnifiedIdeograph62f8', [], 'Display', None],
  [ 0x62f9, 'CjkUnifiedIdeograph62f9', [], 'Display', None],
  [ 0x62fa, 'CjkUnifiedIdeograph62fa', [], 'Display', None],
  [ 0x62fb, 'CjkUnifiedIdeograph62fb', [], 'Display', None],
  [ 0x62fc, 'CjkUnifiedIdeograph62fc', [], 'Display', None],
  [ 0x62fd, 'CjkUnifiedIdeograph62fd', [], 'Display', None],
  [ 0x62fe, 'CjkUnifiedIdeograph62fe', [], 'Display', None],
  [ 0x62ff, 'CjkUnifiedIdeograph62ff', [], 'Display', None],
  [ 0x6300, 'CjkUnifiedIdeograph6300', [], 'Display', None],
  [ 0x6301, 'CjkUnifiedIdeograph6301', [], 'Display', None],
  [ 0x6302, 'CjkUnifiedIdeograph6302', [], 'Display', None],
  [ 0x6303, 'CjkUnifiedIdeograph6303', [], 'Display', None],
  [ 0x6304, 'CjkUnifiedIdeograph6304', [], 'Display', None],
  [ 0x6305, 'CjkUnifiedIdeograph6305', [], 'Display', None],
  [ 0x6306, 'CjkUnifiedIdeograph6306', [], 'Display', None],
  [ 0x6307, 'CjkUnifiedIdeograph6307', [], 'Display', None],
  [ 0x6308, 'CjkUnifiedIdeograph6308', [], 'Display', None],
  [ 0x6309, 'CjkUnifiedIdeograph6309', [], 'Display', None],
  [ 0x630a, 'CjkUnifiedIdeograph630a', [], 'Display', None],
  [ 0x630b, 'CjkUnifiedIdeograph630b', [], 'Display', None],
  [ 0x630c, 'CjkUnifiedIdeograph630c', [], 'Display', None],
  [ 0x630d, 'CjkUnifiedIdeograph630d', [], 'Display', None],
  [ 0x630e, 'CjkUnifiedIdeograph630e', [], 'Display', None],
  [ 0x630f, 'CjkUnifiedIdeograph630f', [], 'Display', None],
  [ 0x6310, 'CjkUnifiedIdeograph6310', [], 'Display', None],
  [ 0x6311, 'CjkUnifiedIdeograph6311', [], 'Display', None],
  [ 0x6312, 'CjkUnifiedIdeograph6312', [], 'Display', None],
  [ 0x6313, 'CjkUnifiedIdeograph6313', [], 'Display', None],
  [ 0x6314, 'CjkUnifiedIdeograph6314', [], 'Display', None],
  [ 0x6315, 'CjkUnifiedIdeograph6315', [], 'Display', None],
  [ 0x6316, 'CjkUnifiedIdeograph6316', [], 'Display', None],
  [ 0x6317, 'CjkUnifiedIdeograph6317', [], 'Display', None],
  [ 0x6318, 'CjkUnifiedIdeograph6318', [], 'Display', None],
  [ 0x6319, 'CjkUnifiedIdeograph6319', [], 'Display', None],
  [ 0x631a, 'CjkUnifiedIdeograph631a', [], 'Display', None],
  [ 0x631b, 'CjkUnifiedIdeograph631b', [], 'Display', None],
  [ 0x631c, 'CjkUnifiedIdeograph631c', [], 'Display', None],
  [ 0x631d, 'CjkUnifiedIdeograph631d', [], 'Display', None],
  [ 0x631e, 'CjkUnifiedIdeograph631e', [], 'Display', None],
  [ 0x631f, 'CjkUnifiedIdeograph631f', [], 'Display', None],
  [ 0x6320, 'CjkUnifiedIdeograph6320', [], 'Display', None],
  [ 0x6321, 'CjkUnifiedIdeograph6321', [], 'Display', None],
  [ 0x6322, 'CjkUnifiedIdeograph6322', [], 'Display', None],
  [ 0x6323, 'CjkUnifiedIdeograph6323', [], 'Display', None],
  [ 0x6324, 'CjkUnifiedIdeograph6324', [], 'Display', None],
  [ 0x6325, 'CjkUnifiedIdeograph6325', [], 'Display', None],
  [ 0x6326, 'CjkUnifiedIdeograph6326', [], 'Display', None],
  [ 0x6327, 'CjkUnifiedIdeograph6327', [], 'Display', None],
  [ 0x6328, 'CjkUnifiedIdeograph6328', [], 'Display', None],
  [ 0x6329, 'CjkUnifiedIdeograph6329', [], 'Display', None],
  [ 0x632a, 'CjkUnifiedIdeograph632a', [], 'Display', None],
  [ 0x632b, 'CjkUnifiedIdeograph632b', [], 'Display', None],
  [ 0x632c, 'CjkUnifiedIdeograph632c', [], 'Display', None],
  [ 0x632d, 'CjkUnifiedIdeograph632d', [], 'Display', None],
  [ 0x632e, 'CjkUnifiedIdeograph632e', [], 'Display', None],
  [ 0x632f, 'CjkUnifiedIdeograph632f', [], 'Display', None],
  [ 0x6330, 'CjkUnifiedIdeograph6330', [], 'Display', None],
  [ 0x6331, 'CjkUnifiedIdeograph6331', [], 'Display', None],
  [ 0x6332, 'CjkUnifiedIdeograph6332', [], 'Display', None],
  [ 0x6333, 'CjkUnifiedIdeograph6333', [], 'Display', None],
  [ 0x6334, 'CjkUnifiedIdeograph6334', [], 'Display', None],
  [ 0x6335, 'CjkUnifiedIdeograph6335', [], 'Display', None],
  [ 0x6336, 'CjkUnifiedIdeograph6336', [], 'Display', None],
  [ 0x6337, 'CjkUnifiedIdeograph6337', [], 'Display', None],
  [ 0x6338, 'CjkUnifiedIdeograph6338', [], 'Display', None],
  [ 0x6339, 'CjkUnifiedIdeograph6339', [], 'Display', None],
  [ 0x633a, 'CjkUnifiedIdeograph633a', [], 'Display', None],
  [ 0x633b, 'CjkUnifiedIdeograph633b', [], 'Display', None],
  [ 0x633c, 'CjkUnifiedIdeograph633c', [], 'Display', None],
  [ 0x633d, 'CjkUnifiedIdeograph633d', [], 'Display', None],
  [ 0x633e, 'CjkUnifiedIdeograph633e', [], 'Display', None],
  [ 0x633f, 'CjkUnifiedIdeograph633f', [], 'Display', None],
  [ 0x6340, 'CjkUnifiedIdeograph6340', [], 'Display', None],
  [ 0x6341, 'CjkUnifiedIdeograph6341', [], 'Display', None],
  [ 0x6342, 'CjkUnifiedIdeograph6342', [], 'Display', None],
  [ 0x6343, 'CjkUnifiedIdeograph6343', [], 'Display', None],
  [ 0x6344, 'CjkUnifiedIdeograph6344', [], 'Display', None],
  [ 0x6345, 'CjkUnifiedIdeograph6345', [], 'Display', None],
  [ 0x6346, 'CjkUnifiedIdeograph6346', [], 'Display', None],
  [ 0x6347, 'CjkUnifiedIdeograph6347', [], 'Display', None],
  [ 0x6348, 'CjkUnifiedIdeograph6348', [], 'Display', None],
  [ 0x6349, 'CjkUnifiedIdeograph6349', [], 'Display', None],
  [ 0x634a, 'CjkUnifiedIdeograph634a', [], 'Display', None],
  [ 0x634b, 'CjkUnifiedIdeograph634b', [], 'Display', None],
  [ 0x634c, 'CjkUnifiedIdeograph634c', [], 'Display', None],
  [ 0x634d, 'CjkUnifiedIdeograph634d', [], 'Display', None],
  [ 0x634e, 'CjkUnifiedIdeograph634e', [], 'Display', None],
  [ 0x634f, 'CjkUnifiedIdeograph634f', [], 'Display', None],
  [ 0x6350, 'CjkUnifiedIdeograph6350', [], 'Display', None],
  [ 0x6351, 'CjkUnifiedIdeograph6351', [], 'Display', None],
  [ 0x6352, 'CjkUnifiedIdeograph6352', [], 'Display', None],
  [ 0x6353, 'CjkUnifiedIdeograph6353', [], 'Display', None],
  [ 0x6354, 'CjkUnifiedIdeograph6354', [], 'Display', None],
  [ 0x6355, 'CjkUnifiedIdeograph6355', [], 'Display', None],
  [ 0x6356, 'CjkUnifiedIdeograph6356', [], 'Display', None],
  [ 0x6357, 'CjkUnifiedIdeograph6357', [], 'Display', None],
  [ 0x6358, 'CjkUnifiedIdeograph6358', [], 'Display', None],
  [ 0x6359, 'CjkUnifiedIdeograph6359', [], 'Display', None],
  [ 0x635a, 'CjkUnifiedIdeograph635a', [], 'Display', None],
  [ 0x635b, 'CjkUnifiedIdeograph635b', [], 'Display', None],
  [ 0x635c, 'CjkUnifiedIdeograph635c', [], 'Display', None],
  [ 0x635d, 'CjkUnifiedIdeograph635d', [], 'Display', None],
  [ 0x635e, 'CjkUnifiedIdeograph635e', [], 'Display', None],
  [ 0x635f, 'CjkUnifiedIdeograph635f', [], 'Display', None],
  [ 0x6360, 'CjkUnifiedIdeograph6360', [], 'Display', None],
  [ 0x6361, 'CjkUnifiedIdeograph6361', [], 'Display', None],
  [ 0x6362, 'CjkUnifiedIdeograph6362', [], 'Display', None],
  [ 0x6363, 'CjkUnifiedIdeograph6363', [], 'Display', None],
  [ 0x6364, 'CjkUnifiedIdeograph6364', [], 'Display', None],
  [ 0x6365, 'CjkUnifiedIdeograph6365', [], 'Display', None],
  [ 0x6366, 'CjkUnifiedIdeograph6366', [], 'Display', None],
  [ 0x6367, 'CjkUnifiedIdeograph6367', [], 'Display', None],
  [ 0x6368, 'CjkUnifiedIdeograph6368', [], 'Display', None],
  [ 0x6369, 'CjkUnifiedIdeograph6369', [], 'Display', None],
  [ 0x636a, 'CjkUnifiedIdeograph636a', [], 'Display', None],
  [ 0x636b, 'CjkUnifiedIdeograph636b', [], 'Display', None],
  [ 0x636c, 'CjkUnifiedIdeograph636c', [], 'Display', None],
  [ 0x636d, 'CjkUnifiedIdeograph636d', [], 'Display', None],
  [ 0x636e, 'CjkUnifiedIdeograph636e', [], 'Display', None],
  [ 0x636f, 'CjkUnifiedIdeograph636f', [], 'Display', None],
  [ 0x6370, 'CjkUnifiedIdeograph6370', [], 'Display', None],
  [ 0x6371, 'CjkUnifiedIdeograph6371', [], 'Display', None],
  [ 0x6372, 'CjkUnifiedIdeograph6372', [], 'Display', None],
  [ 0x6373, 'CjkUnifiedIdeograph6373', [], 'Display', None],
  [ 0x6374, 'CjkUnifiedIdeograph6374', [], 'Display', None],
  [ 0x6375, 'CjkUnifiedIdeograph6375', [], 'Display', None],
  [ 0x6376, 'CjkUnifiedIdeograph6376', [], 'Display', None],
  [ 0x6377, 'CjkUnifiedIdeograph6377', [], 'Display', None],
  [ 0x6378, 'CjkUnifiedIdeograph6378', [], 'Display', None],
  [ 0x6379, 'CjkUnifiedIdeograph6379', [], 'Display', None],
  [ 0x637a, 'CjkUnifiedIdeograph637a', [], 'Display', None],
  [ 0x637b, 'CjkUnifiedIdeograph637b', [], 'Display', None],
  [ 0x637c, 'CjkUnifiedIdeograph637c', [], 'Display', None],
  [ 0x637d, 'CjkUnifiedIdeograph637d', [], 'Display', None],
  [ 0x637e, 'CjkUnifiedIdeograph637e', [], 'Display', None],
  [ 0x637f, 'CjkUnifiedIdeograph637f', [], 'Display', None],
  [ 0x6380, 'CjkUnifiedIdeograph6380', [], 'Display', None],
  [ 0x6381, 'CjkUnifiedIdeograph6381', [], 'Display', None],
  [ 0x6382, 'CjkUnifiedIdeograph6382', [], 'Display', None],
  [ 0x6383, 'CjkUnifiedIdeograph6383', [], 'Display', None],
  [ 0x6384, 'CjkUnifiedIdeograph6384', [], 'Display', None],
  [ 0x6385, 'CjkUnifiedIdeograph6385', [], 'Display', None],
  [ 0x6386, 'CjkUnifiedIdeograph6386', [], 'Display', None],
  [ 0x6387, 'CjkUnifiedIdeograph6387', [], 'Display', None],
  [ 0x6388, 'CjkUnifiedIdeograph6388', [], 'Display', None],
  [ 0x6389, 'CjkUnifiedIdeograph6389', [], 'Display', None],
  [ 0x638a, 'CjkUnifiedIdeograph638a', [], 'Display', None],
  [ 0x638b, 'CjkUnifiedIdeograph638b', [], 'Display', None],
  [ 0x638c, 'CjkUnifiedIdeograph638c', [], 'Display', None],
  [ 0x638d, 'CjkUnifiedIdeograph638d', [], 'Display', None],
  [ 0x638e, 'CjkUnifiedIdeograph638e', [], 'Display', None],
  [ 0x638f, 'CjkUnifiedIdeograph638f', [], 'Display', None],
  [ 0x6390, 'CjkUnifiedIdeograph6390', [], 'Display', None],
  [ 0x6391, 'CjkUnifiedIdeograph6391', [], 'Display', None],
  [ 0x6392, 'CjkUnifiedIdeograph6392', [], 'Display', None],
  [ 0x6393, 'CjkUnifiedIdeograph6393', [], 'Display', None],
  [ 0x6394, 'CjkUnifiedIdeograph6394', [], 'Display', None],
  [ 0x6395, 'CjkUnifiedIdeograph6395', [], 'Display', None],
  [ 0x6396, 'CjkUnifiedIdeograph6396', [], 'Display', None],
  [ 0x6397, 'CjkUnifiedIdeograph6397', [], 'Display', None],
  [ 0x6398, 'CjkUnifiedIdeograph6398', [], 'Display', None],
  [ 0x6399, 'CjkUnifiedIdeograph6399', [], 'Display', None],
  [ 0x639a, 'CjkUnifiedIdeograph639a', [], 'Display', None],
  [ 0x639b, 'CjkUnifiedIdeograph639b', [], 'Display', None],
  [ 0x639c, 'CjkUnifiedIdeograph639c', [], 'Display', None],
  [ 0x639d, 'CjkUnifiedIdeograph639d', [], 'Display', None],
  [ 0x639e, 'CjkUnifiedIdeograph639e', [], 'Display', None],
  [ 0x639f, 'CjkUnifiedIdeograph639f', [], 'Display', None],
  [ 0x63a0, 'CjkUnifiedIdeograph63a0', [], 'Display', None],
  [ 0x63a1, 'CjkUnifiedIdeograph63a1', [], 'Display', None],
  [ 0x63a2, 'CjkUnifiedIdeograph63a2', [], 'Display', None],
  [ 0x63a3, 'CjkUnifiedIdeograph63a3', [], 'Display', None],
  [ 0x63a4, 'CjkUnifiedIdeograph63a4', [], 'Display', None],
  [ 0x63a5, 'CjkUnifiedIdeograph63a5', [], 'Display', None],
  [ 0x63a6, 'CjkUnifiedIdeograph63a6', [], 'Display', None],
  [ 0x63a7, 'CjkUnifiedIdeograph63a7', [], 'Display', None],
  [ 0x63a8, 'CjkUnifiedIdeograph63a8', [], 'Display', None],
  [ 0x63a9, 'CjkUnifiedIdeograph63a9', [], 'Display', None],
  [ 0x63aa, 'CjkUnifiedIdeograph63aa', [], 'Display', None],
  [ 0x63ab, 'CjkUnifiedIdeograph63ab', [], 'Display', None],
  [ 0x63ac, 'CjkUnifiedIdeograph63ac', [], 'Display', None],
  [ 0x63ad, 'CjkUnifiedIdeograph63ad', [], 'Display', None],
  [ 0x63ae, 'CjkUnifiedIdeograph63ae', [], 'Display', None],
  [ 0x63af, 'CjkUnifiedIdeograph63af', [], 'Display', None],
  [ 0x63b0, 'CjkUnifiedIdeograph63b0', [], 'Display', None],
  [ 0x63b1, 'CjkUnifiedIdeograph63b1', [], 'Display', None],
  [ 0x63b2, 'CjkUnifiedIdeograph63b2', [], 'Display', None],
  [ 0x63b3, 'CjkUnifiedIdeograph63b3', [], 'Display', None],
  [ 0x63b4, 'CjkUnifiedIdeograph63b4', [], 'Display', None],
  [ 0x63b5, 'CjkUnifiedIdeograph63b5', [], 'Display', None],
  [ 0x63b6, 'CjkUnifiedIdeograph63b6', [], 'Display', None],
  [ 0x63b7, 'CjkUnifiedIdeograph63b7', [], 'Display', None],
  [ 0x63b8, 'CjkUnifiedIdeograph63b8', [], 'Display', None],
  [ 0x63b9, 'CjkUnifiedIdeograph63b9', [], 'Display', None],
  [ 0x63ba, 'CjkUnifiedIdeograph63ba', [], 'Display', None],
  [ 0x63bb, 'CjkUnifiedIdeograph63bb', [], 'Display', None],
  [ 0x63bc, 'CjkUnifiedIdeograph63bc', [], 'Display', None],
  [ 0x63bd, 'CjkUnifiedIdeograph63bd', [], 'Display', None],
  [ 0x63be, 'CjkUnifiedIdeograph63be', [], 'Display', None],
  [ 0x63bf, 'CjkUnifiedIdeograph63bf', [], 'Display', None],
  [ 0x63c0, 'CjkUnifiedIdeograph63c0', [], 'Display', None],
  [ 0x63c1, 'CjkUnifiedIdeograph63c1', [], 'Display', None],
  [ 0x63c2, 'CjkUnifiedIdeograph63c2', [], 'Display', None],
  [ 0x63c3, 'CjkUnifiedIdeograph63c3', [], 'Display', None],
  [ 0x63c4, 'CjkUnifiedIdeograph63c4', [], 'Display', None],
  [ 0x63c5, 'CjkUnifiedIdeograph63c5', [], 'Display', None],
  [ 0x63c6, 'CjkUnifiedIdeograph63c6', [], 'Display', None],
  [ 0x63c7, 'CjkUnifiedIdeograph63c7', [], 'Display', None],
  [ 0x63c8, 'CjkUnifiedIdeograph63c8', [], 'Display', None],
  [ 0x63c9, 'CjkUnifiedIdeograph63c9', [], 'Display', None],
  [ 0x63ca, 'CjkUnifiedIdeograph63ca', [], 'Display', None],
  [ 0x63cb, 'CjkUnifiedIdeograph63cb', [], 'Display', None],
  [ 0x63cc, 'CjkUnifiedIdeograph63cc', [], 'Display', None],
  [ 0x63cd, 'CjkUnifiedIdeograph63cd', [], 'Display', None],
  [ 0x63ce, 'CjkUnifiedIdeograph63ce', [], 'Display', None],
  [ 0x63cf, 'CjkUnifiedIdeograph63cf', [], 'Display', None],
  [ 0x63d0, 'CjkUnifiedIdeograph63d0', [], 'Display', None],
  [ 0x63d1, 'CjkUnifiedIdeograph63d1', [], 'Display', None],
  [ 0x63d2, 'CjkUnifiedIdeograph63d2', [], 'Display', None],
  [ 0x63d3, 'CjkUnifiedIdeograph63d3', [], 'Display', None],
  [ 0x63d4, 'CjkUnifiedIdeograph63d4', [], 'Display', None],
  [ 0x63d5, 'CjkUnifiedIdeograph63d5', [], 'Display', None],
  [ 0x63d6, 'CjkUnifiedIdeograph63d6', [], 'Display', None],
  [ 0x63d7, 'CjkUnifiedIdeograph63d7', [], 'Display', None],
  [ 0x63d8, 'CjkUnifiedIdeograph63d8', [], 'Display', None],
  [ 0x63d9, 'CjkUnifiedIdeograph63d9', [], 'Display', None],
  [ 0x63da, 'CjkUnifiedIdeograph63da', [], 'Display', None],
  [ 0x63db, 'CjkUnifiedIdeograph63db', [], 'Display', None],
  [ 0x63dc, 'CjkUnifiedIdeograph63dc', [], 'Display', None],
  [ 0x63dd, 'CjkUnifiedIdeograph63dd', [], 'Display', None],
  [ 0x63de, 'CjkUnifiedIdeograph63de', [], 'Display', None],
  [ 0x63df, 'CjkUnifiedIdeograph63df', [], 'Display', None],
  [ 0x63e0, 'CjkUnifiedIdeograph63e0', [], 'Display', None],
  [ 0x63e1, 'CjkUnifiedIdeograph63e1', [], 'Display', None],
  [ 0x63e2, 'CjkUnifiedIdeograph63e2', [], 'Display', None],
  [ 0x63e3, 'CjkUnifiedIdeograph63e3', [], 'Display', None],
  [ 0x63e4, 'CjkUnifiedIdeograph63e4', [], 'Display', None],
  [ 0x63e5, 'CjkUnifiedIdeograph63e5', [], 'Display', None],
  [ 0x63e6, 'CjkUnifiedIdeograph63e6', [], 'Display', None],
  [ 0x63e7, 'CjkUnifiedIdeograph63e7', [], 'Display', None],
  [ 0x63e8, 'CjkUnifiedIdeograph63e8', [], 'Display', None],
  [ 0x63e9, 'CjkUnifiedIdeograph63e9', [], 'Display', None],
  [ 0x63ea, 'CjkUnifiedIdeograph63ea', [], 'Display', None],
  [ 0x63eb, 'CjkUnifiedIdeograph63eb', [], 'Display', None],
  [ 0x63ec, 'CjkUnifiedIdeograph63ec', [], 'Display', None],
  [ 0x63ed, 'CjkUnifiedIdeograph63ed', [], 'Display', None],
  [ 0x63ee, 'CjkUnifiedIdeograph63ee', [], 'Display', None],
  [ 0x63ef, 'CjkUnifiedIdeograph63ef', [], 'Display', None],
  [ 0x63f0, 'CjkUnifiedIdeograph63f0', [], 'Display', None],
  [ 0x63f1, 'CjkUnifiedIdeograph63f1', [], 'Display', None],
  [ 0x63f2, 'CjkUnifiedIdeograph63f2', [], 'Display', None],
  [ 0x63f3, 'CjkUnifiedIdeograph63f3', [], 'Display', None],
  [ 0x63f4, 'CjkUnifiedIdeograph63f4', [], 'Display', None],
  [ 0x63f5, 'CjkUnifiedIdeograph63f5', [], 'Display', None],
  [ 0x63f6, 'CjkUnifiedIdeograph63f6', [], 'Display', None],
  [ 0x63f7, 'CjkUnifiedIdeograph63f7', [], 'Display', None],
  [ 0x63f8, 'CjkUnifiedIdeograph63f8', [], 'Display', None],
  [ 0x63f9, 'CjkUnifiedIdeograph63f9', [], 'Display', None],
  [ 0x63fa, 'CjkUnifiedIdeograph63fa', [], 'Display', None],
  [ 0x63fb, 'CjkUnifiedIdeograph63fb', [], 'Display', None],
  [ 0x63fc, 'CjkUnifiedIdeograph63fc', [], 'Display', None],
  [ 0x63fd, 'CjkUnifiedIdeograph63fd', [], 'Display', None],
  [ 0x63fe, 'CjkUnifiedIdeograph63fe', [], 'Display', None],
  [ 0x63ff, 'CjkUnifiedIdeograph63ff', [], 'Display', None],
  [ 0x6400, 'CjkUnifiedIdeograph6400', [], 'Display', None],
  [ 0x6401, 'CjkUnifiedIdeograph6401', [], 'Display', None],
  [ 0x6402, 'CjkUnifiedIdeograph6402', [], 'Display', None],
  [ 0x6403, 'CjkUnifiedIdeograph6403', [], 'Display', None],
  [ 0x6404, 'CjkUnifiedIdeograph6404', [], 'Display', None],
  [ 0x6405, 'CjkUnifiedIdeograph6405', [], 'Display', None],
  [ 0x6406, 'CjkUnifiedIdeograph6406', [], 'Display', None],
  [ 0x6407, 'CjkUnifiedIdeograph6407', [], 'Display', None],
  [ 0x6408, 'CjkUnifiedIdeograph6408', [], 'Display', None],
  [ 0x6409, 'CjkUnifiedIdeograph6409', [], 'Display', None],
  [ 0x640a, 'CjkUnifiedIdeograph640a', [], 'Display', None],
  [ 0x640b, 'CjkUnifiedIdeograph640b', [], 'Display', None],
  [ 0x640c, 'CjkUnifiedIdeograph640c', [], 'Display', None],
  [ 0x640d, 'CjkUnifiedIdeograph640d', [], 'Display', None],
  [ 0x640e, 'CjkUnifiedIdeograph640e', [], 'Display', None],
  [ 0x640f, 'CjkUnifiedIdeograph640f', [], 'Display', None],
  [ 0x6410, 'CjkUnifiedIdeograph6410', [], 'Display', None],
  [ 0x6411, 'CjkUnifiedIdeograph6411', [], 'Display', None],
  [ 0x6412, 'CjkUnifiedIdeograph6412', [], 'Display', None],
  [ 0x6413, 'CjkUnifiedIdeograph6413', [], 'Display', None],
  [ 0x6414, 'CjkUnifiedIdeograph6414', [], 'Display', None],
  [ 0x6415, 'CjkUnifiedIdeograph6415', [], 'Display', None],
  [ 0x6416, 'CjkUnifiedIdeograph6416', [], 'Display', None],
  [ 0x6417, 'CjkUnifiedIdeograph6417', [], 'Display', None],
  [ 0x6418, 'CjkUnifiedIdeograph6418', [], 'Display', None],
  [ 0x6419, 'CjkUnifiedIdeograph6419', [], 'Display', None],
  [ 0x641a, 'CjkUnifiedIdeograph641a', [], 'Display', None],
  [ 0x641b, 'CjkUnifiedIdeograph641b', [], 'Display', None],
  [ 0x641c, 'CjkUnifiedIdeograph641c', [], 'Display', None],
  [ 0x641d, 'CjkUnifiedIdeograph641d', [], 'Display', None],
  [ 0x641e, 'CjkUnifiedIdeograph641e', [], 'Display', None],
  [ 0x641f, 'CjkUnifiedIdeograph641f', [], 'Display', None],
  [ 0x6420, 'CjkUnifiedIdeograph6420', [], 'Display', None],
  [ 0x6421, 'CjkUnifiedIdeograph6421', [], 'Display', None],
  [ 0x6422, 'CjkUnifiedIdeograph6422', [], 'Display', None],
  [ 0x6423, 'CjkUnifiedIdeograph6423', [], 'Display', None],
  [ 0x6424, 'CjkUnifiedIdeograph6424', [], 'Display', None],
  [ 0x6425, 'CjkUnifiedIdeograph6425', [], 'Display', None],
  [ 0x6426, 'CjkUnifiedIdeograph6426', [], 'Display', None],
  [ 0x6427, 'CjkUnifiedIdeograph6427', [], 'Display', None],
  [ 0x6428, 'CjkUnifiedIdeograph6428', [], 'Display', None],
  [ 0x6429, 'CjkUnifiedIdeograph6429', [], 'Display', None],
  [ 0x642a, 'CjkUnifiedIdeograph642a', [], 'Display', None],
  [ 0x642b, 'CjkUnifiedIdeograph642b', [], 'Display', None],
  [ 0x642c, 'CjkUnifiedIdeograph642c', [], 'Display', None],
  [ 0x642d, 'CjkUnifiedIdeograph642d', [], 'Display', None],
  [ 0x642e, 'CjkUnifiedIdeograph642e', [], 'Display', None],
  [ 0x642f, 'CjkUnifiedIdeograph642f', [], 'Display', None],
  [ 0x6430, 'CjkUnifiedIdeograph6430', [], 'Display', None],
  [ 0x6431, 'CjkUnifiedIdeograph6431', [], 'Display', None],
  [ 0x6432, 'CjkUnifiedIdeograph6432', [], 'Display', None],
  [ 0x6433, 'CjkUnifiedIdeograph6433', [], 'Display', None],
  [ 0x6434, 'CjkUnifiedIdeograph6434', [], 'Display', None],
  [ 0x6435, 'CjkUnifiedIdeograph6435', [], 'Display', None],
  [ 0x6436, 'CjkUnifiedIdeograph6436', [], 'Display', None],
  [ 0x6437, 'CjkUnifiedIdeograph6437', [], 'Display', None],
  [ 0x6438, 'CjkUnifiedIdeograph6438', [], 'Display', None],
  [ 0x6439, 'CjkUnifiedIdeograph6439', [], 'Display', None],
  [ 0x643a, 'CjkUnifiedIdeograph643a', [], 'Display', None],
  [ 0x643b, 'CjkUnifiedIdeograph643b', [], 'Display', None],
  [ 0x643c, 'CjkUnifiedIdeograph643c', [], 'Display', None],
  [ 0x643d, 'CjkUnifiedIdeograph643d', [], 'Display', None],
  [ 0x643e, 'CjkUnifiedIdeograph643e', [], 'Display', None],
  [ 0x643f, 'CjkUnifiedIdeograph643f', [], 'Display', None],
  [ 0x6440, 'CjkUnifiedIdeograph6440', [], 'Display', None],
  [ 0x6441, 'CjkUnifiedIdeograph6441', [], 'Display', None],
  [ 0x6442, 'CjkUnifiedIdeograph6442', [], 'Display', None],
  [ 0x6443, 'CjkUnifiedIdeograph6443', [], 'Display', None],
  [ 0x6444, 'CjkUnifiedIdeograph6444', [], 'Display', None],
  [ 0x6445, 'CjkUnifiedIdeograph6445', [], 'Display', None],
  [ 0x6446, 'CjkUnifiedIdeograph6446', [], 'Display', None],
  [ 0x6447, 'CjkUnifiedIdeograph6447', [], 'Display', None],
  [ 0x6448, 'CjkUnifiedIdeograph6448', [], 'Display', None],
  [ 0x6449, 'CjkUnifiedIdeograph6449', [], 'Display', None],
  [ 0x644a, 'CjkUnifiedIdeograph644a', [], 'Display', None],
  [ 0x644b, 'CjkUnifiedIdeograph644b', [], 'Display', None],
  [ 0x644c, 'CjkUnifiedIdeograph644c', [], 'Display', None],
  [ 0x644d, 'CjkUnifiedIdeograph644d', [], 'Display', None],
  [ 0x644e, 'CjkUnifiedIdeograph644e', [], 'Display', None],
  [ 0x644f, 'CjkUnifiedIdeograph644f', [], 'Display', None],
  [ 0x6450, 'CjkUnifiedIdeograph6450', [], 'Display', None],
  [ 0x6451, 'CjkUnifiedIdeograph6451', [], 'Display', None],
  [ 0x6452, 'CjkUnifiedIdeograph6452', [], 'Display', None],
  [ 0x6453, 'CjkUnifiedIdeograph6453', [], 'Display', None],
  [ 0x6454, 'CjkUnifiedIdeograph6454', [], 'Display', None],
  [ 0x6455, 'CjkUnifiedIdeograph6455', [], 'Display', None],
  [ 0x6456, 'CjkUnifiedIdeograph6456', [], 'Display', None],
  [ 0x6457, 'CjkUnifiedIdeograph6457', [], 'Display', None],
  [ 0x6458, 'CjkUnifiedIdeograph6458', [], 'Display', None],
  [ 0x6459, 'CjkUnifiedIdeograph6459', [], 'Display', None],
  [ 0x645a, 'CjkUnifiedIdeograph645a', [], 'Display', None],
  [ 0x645b, 'CjkUnifiedIdeograph645b', [], 'Display', None],
  [ 0x645c, 'CjkUnifiedIdeograph645c', [], 'Display', None],
  [ 0x645d, 'CjkUnifiedIdeograph645d', [], 'Display', None],
  [ 0x645e, 'CjkUnifiedIdeograph645e', [], 'Display', None],
  [ 0x645f, 'CjkUnifiedIdeograph645f', [], 'Display', None],
  [ 0x6460, 'CjkUnifiedIdeograph6460', [], 'Display', None],
  [ 0x6461, 'CjkUnifiedIdeograph6461', [], 'Display', None],
  [ 0x6462, 'CjkUnifiedIdeograph6462', [], 'Display', None],
  [ 0x6463, 'CjkUnifiedIdeograph6463', [], 'Display', None],
  [ 0x6464, 'CjkUnifiedIdeograph6464', [], 'Display', None],
  [ 0x6465, 'CjkUnifiedIdeograph6465', [], 'Display', None],
  [ 0x6466, 'CjkUnifiedIdeograph6466', [], 'Display', None],
  [ 0x6467, 'CjkUnifiedIdeograph6467', [], 'Display', None],
  [ 0x6468, 'CjkUnifiedIdeograph6468', [], 'Display', None],
  [ 0x6469, 'CjkUnifiedIdeograph6469', [], 'Display', None],
  [ 0x646a, 'CjkUnifiedIdeograph646a', [], 'Display', None],
  [ 0x646b, 'CjkUnifiedIdeograph646b', [], 'Display', None],
  [ 0x646c, 'CjkUnifiedIdeograph646c', [], 'Display', None],
  [ 0x646d, 'CjkUnifiedIdeograph646d', [], 'Display', None],
  [ 0x646e, 'CjkUnifiedIdeograph646e', [], 'Display', None],
  [ 0x646f, 'CjkUnifiedIdeograph646f', [], 'Display', None],
  [ 0x6470, 'CjkUnifiedIdeograph6470', [], 'Display', None],
  [ 0x6471, 'CjkUnifiedIdeograph6471', [], 'Display', None],
  [ 0x6472, 'CjkUnifiedIdeograph6472', [], 'Display', None],
  [ 0x6473, 'CjkUnifiedIdeograph6473', [], 'Display', None],
  [ 0x6474, 'CjkUnifiedIdeograph6474', [], 'Display', None],
  [ 0x6475, 'CjkUnifiedIdeograph6475', [], 'Display', None],
  [ 0x6476, 'CjkUnifiedIdeograph6476', [], 'Display', None],
  [ 0x6477, 'CjkUnifiedIdeograph6477', [], 'Display', None],
  [ 0x6478, 'CjkUnifiedIdeograph6478', [], 'Display', None],
  [ 0x6479, 'CjkUnifiedIdeograph6479', [], 'Display', None],
  [ 0x647a, 'CjkUnifiedIdeograph647a', [], 'Display', None],
  [ 0x647b, 'CjkUnifiedIdeograph647b', [], 'Display', None],
  [ 0x647c, 'CjkUnifiedIdeograph647c', [], 'Display', None],
  [ 0x647d, 'CjkUnifiedIdeograph647d', [], 'Display', None],
  [ 0x647e, 'CjkUnifiedIdeograph647e', [], 'Display', None],
  [ 0x647f, 'CjkUnifiedIdeograph647f', [], 'Display', None],
  [ 0x6480, 'CjkUnifiedIdeograph6480', [], 'Display', None],
  [ 0x6481, 'CjkUnifiedIdeograph6481', [], 'Display', None],
  [ 0x6482, 'CjkUnifiedIdeograph6482', [], 'Display', None],
  [ 0x6483, 'CjkUnifiedIdeograph6483', [], 'Display', None],
  [ 0x6484, 'CjkUnifiedIdeograph6484', [], 'Display', None],
  [ 0x6485, 'CjkUnifiedIdeograph6485', [], 'Display', None],
  [ 0x6486, 'CjkUnifiedIdeograph6486', [], 'Display', None],
  [ 0x6487, 'CjkUnifiedIdeograph6487', [], 'Display', None],
  [ 0x6488, 'CjkUnifiedIdeograph6488', [], 'Display', None],
  [ 0x6489, 'CjkUnifiedIdeograph6489', [], 'Display', None],
  [ 0x648a, 'CjkUnifiedIdeograph648a', [], 'Display', None],
  [ 0x648b, 'CjkUnifiedIdeograph648b', [], 'Display', None],
  [ 0x648c, 'CjkUnifiedIdeograph648c', [], 'Display', None],
  [ 0x648d, 'CjkUnifiedIdeograph648d', [], 'Display', None],
  [ 0x648e, 'CjkUnifiedIdeograph648e', [], 'Display', None],
  [ 0x648f, 'CjkUnifiedIdeograph648f', [], 'Display', None],
  [ 0x6490, 'CjkUnifiedIdeograph6490', [], 'Display', None],
  [ 0x6491, 'CjkUnifiedIdeograph6491', [], 'Display', None],
  [ 0x6492, 'CjkUnifiedIdeograph6492', [], 'Display', None],
  [ 0x6493, 'CjkUnifiedIdeograph6493', [], 'Display', None],
  [ 0x6494, 'CjkUnifiedIdeograph6494', [], 'Display', None],
  [ 0x6495, 'CjkUnifiedIdeograph6495', [], 'Display', None],
  [ 0x6496, 'CjkUnifiedIdeograph6496', [], 'Display', None],
  [ 0x6497, 'CjkUnifiedIdeograph6497', [], 'Display', None],
  [ 0x6498, 'CjkUnifiedIdeograph6498', [], 'Display', None],
  [ 0x6499, 'CjkUnifiedIdeograph6499', [], 'Display', None],
  [ 0x649a, 'CjkUnifiedIdeograph649a', [], 'Display', None],
  [ 0x649b, 'CjkUnifiedIdeograph649b', [], 'Display', None],
  [ 0x649c, 'CjkUnifiedIdeograph649c', [], 'Display', None],
  [ 0x649d, 'CjkUnifiedIdeograph649d', [], 'Display', None],
  [ 0x649e, 'CjkUnifiedIdeograph649e', [], 'Display', None],
  [ 0x649f, 'CjkUnifiedIdeograph649f', [], 'Display', None],
  [ 0x64a0, 'CjkUnifiedIdeograph64a0', [], 'Display', None],
  [ 0x64a1, 'CjkUnifiedIdeograph64a1', [], 'Display', None],
  [ 0x64a2, 'CjkUnifiedIdeograph64a2', [], 'Display', None],
  [ 0x64a3, 'CjkUnifiedIdeograph64a3', [], 'Display', None],
  [ 0x64a4, 'CjkUnifiedIdeograph64a4', [], 'Display', None],
  [ 0x64a5, 'CjkUnifiedIdeograph64a5', [], 'Display', None],
  [ 0x64a6, 'CjkUnifiedIdeograph64a6', [], 'Display', None],
  [ 0x64a7, 'CjkUnifiedIdeograph64a7', [], 'Display', None],
  [ 0x64a8, 'CjkUnifiedIdeograph64a8', [], 'Display', None],
  [ 0x64a9, 'CjkUnifiedIdeograph64a9', [], 'Display', None],
  [ 0x64aa, 'CjkUnifiedIdeograph64aa', [], 'Display', None],
  [ 0x64ab, 'CjkUnifiedIdeograph64ab', [], 'Display', None],
  [ 0x64ac, 'CjkUnifiedIdeograph64ac', [], 'Display', None],
  [ 0x64ad, 'CjkUnifiedIdeograph64ad', [], 'Display', None],
  [ 0x64ae, 'CjkUnifiedIdeograph64ae', [], 'Display', None],
  [ 0x64af, 'CjkUnifiedIdeograph64af', [], 'Display', None],
  [ 0x64b0, 'CjkUnifiedIdeograph64b0', [], 'Display', None],
  [ 0x64b1, 'CjkUnifiedIdeograph64b1', [], 'Display', None],
  [ 0x64b2, 'CjkUnifiedIdeograph64b2', [], 'Display', None],
  [ 0x64b3, 'CjkUnifiedIdeograph64b3', [], 'Display', None],
  [ 0x64b4, 'CjkUnifiedIdeograph64b4', [], 'Display', None],
  [ 0x64b5, 'CjkUnifiedIdeograph64b5', [], 'Display', None],
  [ 0x64b6, 'CjkUnifiedIdeograph64b6', [], 'Display', None],
  [ 0x64b7, 'CjkUnifiedIdeograph64b7', [], 'Display', None],
  [ 0x64b8, 'CjkUnifiedIdeograph64b8', [], 'Display', None],
  [ 0x64b9, 'CjkUnifiedIdeograph64b9', [], 'Display', None],
  [ 0x64ba, 'CjkUnifiedIdeograph64ba', [], 'Display', None],
  [ 0x64bb, 'CjkUnifiedIdeograph64bb', [], 'Display', None],
  [ 0x64bc, 'CjkUnifiedIdeograph64bc', [], 'Display', None],
  [ 0x64bd, 'CjkUnifiedIdeograph64bd', [], 'Display', None],
  [ 0x64be, 'CjkUnifiedIdeograph64be', [], 'Display', None],
  [ 0x64bf, 'CjkUnifiedIdeograph64bf', [], 'Display', None],
  [ 0x64c0, 'CjkUnifiedIdeograph64c0', [], 'Display', None],
  [ 0x64c1, 'CjkUnifiedIdeograph64c1', [], 'Display', None],
  [ 0x64c2, 'CjkUnifiedIdeograph64c2', [], 'Display', None],
  [ 0x64c3, 'CjkUnifiedIdeograph64c3', [], 'Display', None],
  [ 0x64c4, 'CjkUnifiedIdeograph64c4', [], 'Display', None],
  [ 0x64c5, 'CjkUnifiedIdeograph64c5', [], 'Display', None],
  [ 0x64c6, 'CjkUnifiedIdeograph64c6', [], 'Display', None],
  [ 0x64c7, 'CjkUnifiedIdeograph64c7', [], 'Display', None],
  [ 0x64c8, 'CjkUnifiedIdeograph64c8', [], 'Display', None],
  [ 0x64c9, 'CjkUnifiedIdeograph64c9', [], 'Display', None],
  [ 0x64ca, 'CjkUnifiedIdeograph64ca', [], 'Display', None],
  [ 0x64cb, 'CjkUnifiedIdeograph64cb', [], 'Display', None],
  [ 0x64cc, 'CjkUnifiedIdeograph64cc', [], 'Display', None],
  [ 0x64cd, 'CjkUnifiedIdeograph64cd', [], 'Display', None],
  [ 0x64ce, 'CjkUnifiedIdeograph64ce', [], 'Display', None],
  [ 0x64cf, 'CjkUnifiedIdeograph64cf', [], 'Display', None],
  [ 0x64d0, 'CjkUnifiedIdeograph64d0', [], 'Display', None],
  [ 0x64d1, 'CjkUnifiedIdeograph64d1', [], 'Display', None],
  [ 0x64d2, 'CjkUnifiedIdeograph64d2', [], 'Display', None],
  [ 0x64d3, 'CjkUnifiedIdeograph64d3', [], 'Display', None],
  [ 0x64d4, 'CjkUnifiedIdeograph64d4', [], 'Display', None],
  [ 0x64d5, 'CjkUnifiedIdeograph64d5', [], 'Display', None],
  [ 0x64d6, 'CjkUnifiedIdeograph64d6', [], 'Display', None],
  [ 0x64d7, 'CjkUnifiedIdeograph64d7', [], 'Display', None],
  [ 0x64d8, 'CjkUnifiedIdeograph64d8', [], 'Display', None],
  [ 0x64d9, 'CjkUnifiedIdeograph64d9', [], 'Display', None],
  [ 0x64da, 'CjkUnifiedIdeograph64da', [], 'Display', None],
  [ 0x64db, 'CjkUnifiedIdeograph64db', [], 'Display', None],
  [ 0x64dc, 'CjkUnifiedIdeograph64dc', [], 'Display', None],
  [ 0x64dd, 'CjkUnifiedIdeograph64dd', [], 'Display', None],
  [ 0x64de, 'CjkUnifiedIdeograph64de', [], 'Display', None],
  [ 0x64df, 'CjkUnifiedIdeograph64df', [], 'Display', None],
  [ 0x64e0, 'CjkUnifiedIdeograph64e0', [], 'Display', None],
  [ 0x64e1, 'CjkUnifiedIdeograph64e1', [], 'Display', None],
  [ 0x64e2, 'CjkUnifiedIdeograph64e2', [], 'Display', None],
  [ 0x64e3, 'CjkUnifiedIdeograph64e3', [], 'Display', None],
  [ 0x64e4, 'CjkUnifiedIdeograph64e4', [], 'Display', None],
  [ 0x64e5, 'CjkUnifiedIdeograph64e5', [], 'Display', None],
  [ 0x64e6, 'CjkUnifiedIdeograph64e6', [], 'Display', None],
  [ 0x64e7, 'CjkUnifiedIdeograph64e7', [], 'Display', None],
  [ 0x64e8, 'CjkUnifiedIdeograph64e8', [], 'Display', None],
  [ 0x64e9, 'CjkUnifiedIdeograph64e9', [], 'Display', None],
  [ 0x64ea, 'CjkUnifiedIdeograph64ea', [], 'Display', None],
  [ 0x64eb, 'CjkUnifiedIdeograph64eb', [], 'Display', None],
  [ 0x64ec, 'CjkUnifiedIdeograph64ec', [], 'Display', None],
  [ 0x64ed, 'CjkUnifiedIdeograph64ed', [], 'Display', None],
  [ 0x64ee, 'CjkUnifiedIdeograph64ee', [], 'Display', None],
  [ 0x64ef, 'CjkUnifiedIdeograph64ef', [], 'Display', None],
  [ 0x64f0, 'CjkUnifiedIdeograph64f0', [], 'Display', None],
  [ 0x64f1, 'CjkUnifiedIdeograph64f1', [], 'Display', None],
  [ 0x64f2, 'CjkUnifiedIdeograph64f2', [], 'Display', None],
  [ 0x64f3, 'CjkUnifiedIdeograph64f3', [], 'Display', None],
  [ 0x64f4, 'CjkUnifiedIdeograph64f4', [], 'Display', None],
  [ 0x64f5, 'CjkUnifiedIdeograph64f5', [], 'Display', None],
  [ 0x64f6, 'CjkUnifiedIdeograph64f6', [], 'Display', None],
  [ 0x64f7, 'CjkUnifiedIdeograph64f7', [], 'Display', None],
  [ 0x64f8, 'CjkUnifiedIdeograph64f8', [], 'Display', None],
  [ 0x64f9, 'CjkUnifiedIdeograph64f9', [], 'Display', None],
  [ 0x64fa, 'CjkUnifiedIdeograph64fa', [], 'Display', None],
  [ 0x64fb, 'CjkUnifiedIdeograph64fb', [], 'Display', None],
  [ 0x64fc, 'CjkUnifiedIdeograph64fc', [], 'Display', None],
  [ 0x64fd, 'CjkUnifiedIdeograph64fd', [], 'Display', None],
  [ 0x64fe, 'CjkUnifiedIdeograph64fe', [], 'Display', None],
  [ 0x64ff, 'CjkUnifiedIdeograph64ff', [], 'Display', None],
  [ 0x6500, 'CjkUnifiedIdeograph6500', [], 'Display', None],
  [ 0x6501, 'CjkUnifiedIdeograph6501', [], 'Display', None],
  [ 0x6502, 'CjkUnifiedIdeograph6502', [], 'Display', None],
  [ 0x6503, 'CjkUnifiedIdeograph6503', [], 'Display', None],
  [ 0x6504, 'CjkUnifiedIdeograph6504', [], 'Display', None],
  [ 0x6505, 'CjkUnifiedIdeograph6505', [], 'Display', None],
  [ 0x6506, 'CjkUnifiedIdeograph6506', [], 'Display', None],
  [ 0x6507, 'CjkUnifiedIdeograph6507', [], 'Display', None],
  [ 0x6508, 'CjkUnifiedIdeograph6508', [], 'Display', None],
  [ 0x6509, 'CjkUnifiedIdeograph6509', [], 'Display', None],
  [ 0x650a, 'CjkUnifiedIdeograph650a', [], 'Display', None],
  [ 0x650b, 'CjkUnifiedIdeograph650b', [], 'Display', None],
  [ 0x650c, 'CjkUnifiedIdeograph650c', [], 'Display', None],
  [ 0x650d, 'CjkUnifiedIdeograph650d', [], 'Display', None],
  [ 0x650e, 'CjkUnifiedIdeograph650e', [], 'Display', None],
  [ 0x650f, 'CjkUnifiedIdeograph650f', [], 'Display', None],
  [ 0x6510, 'CjkUnifiedIdeograph6510', [], 'Display', None],
  [ 0x6511, 'CjkUnifiedIdeograph6511', [], 'Display', None],
  [ 0x6512, 'CjkUnifiedIdeograph6512', [], 'Display', None],
  [ 0x6513, 'CjkUnifiedIdeograph6513', [], 'Display', None],
  [ 0x6514, 'CjkUnifiedIdeograph6514', [], 'Display', None],
  [ 0x6515, 'CjkUnifiedIdeograph6515', [], 'Display', None],
  [ 0x6516, 'CjkUnifiedIdeograph6516', [], 'Display', None],
  [ 0x6517, 'CjkUnifiedIdeograph6517', [], 'Display', None],
  [ 0x6518, 'CjkUnifiedIdeograph6518', [], 'Display', None],
  [ 0x6519, 'CjkUnifiedIdeograph6519', [], 'Display', None],
  [ 0x651a, 'CjkUnifiedIdeograph651a', [], 'Display', None],
  [ 0x651b, 'CjkUnifiedIdeograph651b', [], 'Display', None],
  [ 0x651c, 'CjkUnifiedIdeograph651c', [], 'Display', None],
  [ 0x651d, 'CjkUnifiedIdeograph651d', [], 'Display', None],
  [ 0x651e, 'CjkUnifiedIdeograph651e', [], 'Display', None],
  [ 0x651f, 'CjkUnifiedIdeograph651f', [], 'Display', None],
  [ 0x6520, 'CjkUnifiedIdeograph6520', [], 'Display', None],
  [ 0x6521, 'CjkUnifiedIdeograph6521', [], 'Display', None],
  [ 0x6522, 'CjkUnifiedIdeograph6522', [], 'Display', None],
  [ 0x6523, 'CjkUnifiedIdeograph6523', [], 'Display', None],
  [ 0x6524, 'CjkUnifiedIdeograph6524', [], 'Display', None],
  [ 0x6525, 'CjkUnifiedIdeograph6525', [], 'Display', None],
  [ 0x6526, 'CjkUnifiedIdeograph6526', [], 'Display', None],
  [ 0x6527, 'CjkUnifiedIdeograph6527', [], 'Display', None],
  [ 0x6528, 'CjkUnifiedIdeograph6528', [], 'Display', None],
  [ 0x6529, 'CjkUnifiedIdeograph6529', [], 'Display', None],
  [ 0x652a, 'CjkUnifiedIdeograph652a', [], 'Display', None],
  [ 0x652b, 'CjkUnifiedIdeograph652b', [], 'Display', None],
  [ 0x652c, 'CjkUnifiedIdeograph652c', [], 'Display', None],
  [ 0x652d, 'CjkUnifiedIdeograph652d', [], 'Display', None],
  [ 0x652e, 'CjkUnifiedIdeograph652e', [], 'Display', None],
  [ 0x652f, 'CjkUnifiedIdeograph652f', [], 'Display', None],
  [ 0x6530, 'CjkUnifiedIdeograph6530', [], 'Display', None],
  [ 0x6531, 'CjkUnifiedIdeograph6531', [], 'Display', None],
  [ 0x6532, 'CjkUnifiedIdeograph6532', [], 'Display', None],
  [ 0x6533, 'CjkUnifiedIdeograph6533', [], 'Display', None],
  [ 0x6534, 'CjkUnifiedIdeograph6534', [], 'Display', None],
  [ 0x6535, 'CjkUnifiedIdeograph6535', [], 'Display', None],
  [ 0x6536, 'CjkUnifiedIdeograph6536', [], 'Display', None],
  [ 0x6537, 'CjkUnifiedIdeograph6537', [], 'Display', None],
  [ 0x6538, 'CjkUnifiedIdeograph6538', [], 'Display', None],
  [ 0x6539, 'CjkUnifiedIdeograph6539', [], 'Display', None],
  [ 0x653a, 'CjkUnifiedIdeograph653a', [], 'Display', None],
  [ 0x653b, 'CjkUnifiedIdeograph653b', [], 'Display', None],
  [ 0x653c, 'CjkUnifiedIdeograph653c', [], 'Display', None],
  [ 0x653d, 'CjkUnifiedIdeograph653d', [], 'Display', None],
  [ 0x653e, 'CjkUnifiedIdeograph653e', [], 'Display', None],
  [ 0x653f, 'CjkUnifiedIdeograph653f', [], 'Display', None],
  [ 0x6540, 'CjkUnifiedIdeograph6540', [], 'Display', None],
  [ 0x6541, 'CjkUnifiedIdeograph6541', [], 'Display', None],
  [ 0x6542, 'CjkUnifiedIdeograph6542', [], 'Display', None],
  [ 0x6543, 'CjkUnifiedIdeograph6543', [], 'Display', None],
  [ 0x6544, 'CjkUnifiedIdeograph6544', [], 'Display', None],
  [ 0x6545, 'CjkUnifiedIdeograph6545', [], 'Display', None],
  [ 0x6546, 'CjkUnifiedIdeograph6546', [], 'Display', None],
  [ 0x6547, 'CjkUnifiedIdeograph6547', [], 'Display', None],
  [ 0x6548, 'CjkUnifiedIdeograph6548', [], 'Display', None],
  [ 0x6549, 'CjkUnifiedIdeograph6549', [], 'Display', None],
  [ 0x654a, 'CjkUnifiedIdeograph654a', [], 'Display', None],
  [ 0x654b, 'CjkUnifiedIdeograph654b', [], 'Display', None],
  [ 0x654c, 'CjkUnifiedIdeograph654c', [], 'Display', None],
  [ 0x654d, 'CjkUnifiedIdeograph654d', [], 'Display', None],
  [ 0x654e, 'CjkUnifiedIdeograph654e', [], 'Display', None],
  [ 0x654f, 'CjkUnifiedIdeograph654f', [], 'Display', None],
  [ 0x6550, 'CjkUnifiedIdeograph6550', [], 'Display', None],
  [ 0x6551, 'CjkUnifiedIdeograph6551', [], 'Display', None],
  [ 0x6552, 'CjkUnifiedIdeograph6552', [], 'Display', None],
  [ 0x6553, 'CjkUnifiedIdeograph6553', [], 'Display', None],
  [ 0x6554, 'CjkUnifiedIdeograph6554', [], 'Display', None],
  [ 0x6555, 'CjkUnifiedIdeograph6555', [], 'Display', None],
  [ 0x6556, 'CjkUnifiedIdeograph6556', [], 'Display', None],
  [ 0x6557, 'CjkUnifiedIdeograph6557', [], 'Display', None],
  [ 0x6558, 'CjkUnifiedIdeograph6558', [], 'Display', None],
  [ 0x6559, 'CjkUnifiedIdeograph6559', [], 'Display', None],
  [ 0x655a, 'CjkUnifiedIdeograph655a', [], 'Display', None],
  [ 0x655b, 'CjkUnifiedIdeograph655b', [], 'Display', None],
  [ 0x655c, 'CjkUnifiedIdeograph655c', [], 'Display', None],
  [ 0x655d, 'CjkUnifiedIdeograph655d', [], 'Display', None],
  [ 0x655e, 'CjkUnifiedIdeograph655e', [], 'Display', None],
  [ 0x655f, 'CjkUnifiedIdeograph655f', [], 'Display', None],
  [ 0x6560, 'CjkUnifiedIdeograph6560', [], 'Display', None],
  [ 0x6561, 'CjkUnifiedIdeograph6561', [], 'Display', None],
  [ 0x6562, 'CjkUnifiedIdeograph6562', [], 'Display', None],
  [ 0x6563, 'CjkUnifiedIdeograph6563', [], 'Display', None],
  [ 0x6564, 'CjkUnifiedIdeograph6564', [], 'Display', None],
  [ 0x6565, 'CjkUnifiedIdeograph6565', [], 'Display', None],
  [ 0x6566, 'CjkUnifiedIdeograph6566', [], 'Display', None],
  [ 0x6567, 'CjkUnifiedIdeograph6567', [], 'Display', None],
  [ 0x6568, 'CjkUnifiedIdeograph6568', [], 'Display', None],
  [ 0x6569, 'CjkUnifiedIdeograph6569', [], 'Display', None],
  [ 0x656a, 'CjkUnifiedIdeograph656a', [], 'Display', None],
  [ 0x656b, 'CjkUnifiedIdeograph656b', [], 'Display', None],
  [ 0x656c, 'CjkUnifiedIdeograph656c', [], 'Display', None],
  [ 0x656d, 'CjkUnifiedIdeograph656d', [], 'Display', None],
  [ 0x656e, 'CjkUnifiedIdeograph656e', [], 'Display', None],
  [ 0x656f, 'CjkUnifiedIdeograph656f', [], 'Display', None],
  [ 0x6570, 'CjkUnifiedIdeograph6570', [], 'Display', None],
  [ 0x6571, 'CjkUnifiedIdeograph6571', [], 'Display', None],
  [ 0x6572, 'CjkUnifiedIdeograph6572', [], 'Display', None],
  [ 0x6573, 'CjkUnifiedIdeograph6573', [], 'Display', None],
  [ 0x6574, 'CjkUnifiedIdeograph6574', [], 'Display', None],
  [ 0x6575, 'CjkUnifiedIdeograph6575', [], 'Display', None],
  [ 0x6576, 'CjkUnifiedIdeograph6576', [], 'Display', None],
  [ 0x6577, 'CjkUnifiedIdeograph6577', [], 'Display', None],
  [ 0x6578, 'CjkUnifiedIdeograph6578', [], 'Display', None],
  [ 0x6579, 'CjkUnifiedIdeograph6579', [], 'Display', None],
  [ 0x657a, 'CjkUnifiedIdeograph657a', [], 'Display', None],
  [ 0x657b, 'CjkUnifiedIdeograph657b', [], 'Display', None],
  [ 0x657c, 'CjkUnifiedIdeograph657c', [], 'Display', None],
  [ 0x657d, 'CjkUnifiedIdeograph657d', [], 'Display', None],
  [ 0x657e, 'CjkUnifiedIdeograph657e', [], 'Display', None],
  [ 0x657f, 'CjkUnifiedIdeograph657f', [], 'Display', None],
  [ 0x6580, 'CjkUnifiedIdeograph6580', [], 'Display', None],
  [ 0x6581, 'CjkUnifiedIdeograph6581', [], 'Display', None],
  [ 0x6582, 'CjkUnifiedIdeograph6582', [], 'Display', None],
  [ 0x6583, 'CjkUnifiedIdeograph6583', [], 'Display', None],
  [ 0x6584, 'CjkUnifiedIdeograph6584', [], 'Display', None],
  [ 0x6585, 'CjkUnifiedIdeograph6585', [], 'Display', None],
  [ 0x6586, 'CjkUnifiedIdeograph6586', [], 'Display', None],
  [ 0x6587, 'CjkUnifiedIdeograph6587', [], 'Display', None],
  [ 0x6588, 'CjkUnifiedIdeograph6588', [], 'Display', None],
  [ 0x6589, 'CjkUnifiedIdeograph6589', [], 'Display', None],
  [ 0x658a, 'CjkUnifiedIdeograph658a', [], 'Display', None],
  [ 0x658b, 'CjkUnifiedIdeograph658b', [], 'Display', None],
  [ 0x658c, 'CjkUnifiedIdeograph658c', [], 'Display', None],
  [ 0x658d, 'CjkUnifiedIdeograph658d', [], 'Display', None],
  [ 0x658e, 'CjkUnifiedIdeograph658e', [], 'Display', None],
  [ 0x658f, 'CjkUnifiedIdeograph658f', [], 'Display', None],
  [ 0x6590, 'CjkUnifiedIdeograph6590', [], 'Display', None],
  [ 0x6591, 'CjkUnifiedIdeograph6591', [], 'Display', None],
  [ 0x6592, 'CjkUnifiedIdeograph6592', [], 'Display', None],
  [ 0x6593, 'CjkUnifiedIdeograph6593', [], 'Display', None],
  [ 0x6594, 'CjkUnifiedIdeograph6594', [], 'Display', None],
  [ 0x6595, 'CjkUnifiedIdeograph6595', [], 'Display', None],
  [ 0x6596, 'CjkUnifiedIdeograph6596', [], 'Display', None],
  [ 0x6597, 'CjkUnifiedIdeograph6597', [], 'Display', None],
  [ 0x6598, 'CjkUnifiedIdeograph6598', [], 'Display', None],
  [ 0x6599, 'CjkUnifiedIdeograph6599', [], 'Display', None],
  [ 0x659a, 'CjkUnifiedIdeograph659a', [], 'Display', None],
  [ 0x659b, 'CjkUnifiedIdeograph659b', [], 'Display', None],
  [ 0x659c, 'CjkUnifiedIdeograph659c', [], 'Display', None],
  [ 0x659d, 'CjkUnifiedIdeograph659d', [], 'Display', None],
  [ 0x659e, 'CjkUnifiedIdeograph659e', [], 'Display', None],
  [ 0x659f, 'CjkUnifiedIdeograph659f', [], 'Display', None],
  [ 0x65a0, 'CjkUnifiedIdeograph65a0', [], 'Display', None],
  [ 0x65a1, 'CjkUnifiedIdeograph65a1', [], 'Display', None],
  [ 0x65a2, 'CjkUnifiedIdeograph65a2', [], 'Display', None],
  [ 0x65a3, 'CjkUnifiedIdeograph65a3', [], 'Display', None],
  [ 0x65a4, 'CjkUnifiedIdeograph65a4', [], 'Display', None],
  [ 0x65a5, 'CjkUnifiedIdeograph65a5', [], 'Display', None],
  [ 0x65a6, 'CjkUnifiedIdeograph65a6', [], 'Display', None],
  [ 0x65a7, 'CjkUnifiedIdeograph65a7', [], 'Display', None],
  [ 0x65a8, 'CjkUnifiedIdeograph65a8', [], 'Display', None],
  [ 0x65a9, 'CjkUnifiedIdeograph65a9', [], 'Display', None],
  [ 0x65aa, 'CjkUnifiedIdeograph65aa', [], 'Display', None],
  [ 0x65ab, 'CjkUnifiedIdeograph65ab', [], 'Display', None],
  [ 0x65ac, 'CjkUnifiedIdeograph65ac', [], 'Display', None],
  [ 0x65ad, 'CjkUnifiedIdeograph65ad', [], 'Display', None],
  [ 0x65ae, 'CjkUnifiedIdeograph65ae', [], 'Display', None],
  [ 0x65af, 'CjkUnifiedIdeograph65af', [], 'Display', None],
  [ 0x65b0, 'CjkUnifiedIdeograph65b0', [], 'Display', None],
  [ 0x65b1, 'CjkUnifiedIdeograph65b1', [], 'Display', None],
  [ 0x65b2, 'CjkUnifiedIdeograph65b2', [], 'Display', None],
  [ 0x65b3, 'CjkUnifiedIdeograph65b3', [], 'Display', None],
  [ 0x65b4, 'CjkUnifiedIdeograph65b4', [], 'Display', None],
  [ 0x65b5, 'CjkUnifiedIdeograph65b5', [], 'Display', None],
  [ 0x65b6, 'CjkUnifiedIdeograph65b6', [], 'Display', None],
  [ 0x65b7, 'CjkUnifiedIdeograph65b7', [], 'Display', None],
  [ 0x65b8, 'CjkUnifiedIdeograph65b8', [], 'Display', None],
  [ 0x65b9, 'CjkUnifiedIdeograph65b9', [], 'Display', None],
  [ 0x65ba, 'CjkUnifiedIdeograph65ba', [], 'Display', None],
  [ 0x65bb, 'CjkUnifiedIdeograph65bb', [], 'Display', None],
  [ 0x65bc, 'CjkUnifiedIdeograph65bc', [], 'Display', None],
  [ 0x65bd, 'CjkUnifiedIdeograph65bd', [], 'Display', None],
  [ 0x65be, 'CjkUnifiedIdeograph65be', [], 'Display', None],
  [ 0x65bf, 'CjkUnifiedIdeograph65bf', [], 'Display', None],
  [ 0x65c0, 'CjkUnifiedIdeograph65c0', [], 'Display', None],
  [ 0x65c1, 'CjkUnifiedIdeograph65c1', [], 'Display', None],
  [ 0x65c2, 'CjkUnifiedIdeograph65c2', [], 'Display', None],
  [ 0x65c3, 'CjkUnifiedIdeograph65c3', [], 'Display', None],
  [ 0x65c4, 'CjkUnifiedIdeograph65c4', [], 'Display', None],
  [ 0x65c5, 'CjkUnifiedIdeograph65c5', [], 'Display', None],
  [ 0x65c6, 'CjkUnifiedIdeograph65c6', [], 'Display', None],
  [ 0x65c7, 'CjkUnifiedIdeograph65c7', [], 'Display', None],
  [ 0x65c8, 'CjkUnifiedIdeograph65c8', [], 'Display', None],
  [ 0x65c9, 'CjkUnifiedIdeograph65c9', [], 'Display', None],
  [ 0x65ca, 'CjkUnifiedIdeograph65ca', [], 'Display', None],
  [ 0x65cb, 'CjkUnifiedIdeograph65cb', [], 'Display', None],
  [ 0x65cc, 'CjkUnifiedIdeograph65cc', [], 'Display', None],
  [ 0x65cd, 'CjkUnifiedIdeograph65cd', [], 'Display', None],
  [ 0x65ce, 'CjkUnifiedIdeograph65ce', [], 'Display', None],
  [ 0x65cf, 'CjkUnifiedIdeograph65cf', [], 'Display', None],
  [ 0x65d0, 'CjkUnifiedIdeograph65d0', [], 'Display', None],
  [ 0x65d1, 'CjkUnifiedIdeograph65d1', [], 'Display', None],
  [ 0x65d2, 'CjkUnifiedIdeograph65d2', [], 'Display', None],
  [ 0x65d3, 'CjkUnifiedIdeograph65d3', [], 'Display', None],
  [ 0x65d4, 'CjkUnifiedIdeograph65d4', [], 'Display', None],
  [ 0x65d5, 'CjkUnifiedIdeograph65d5', [], 'Display', None],
  [ 0x65d6, 'CjkUnifiedIdeograph65d6', [], 'Display', None],
  [ 0x65d7, 'CjkUnifiedIdeograph65d7', [], 'Display', None],
  [ 0x65d8, 'CjkUnifiedIdeograph65d8', [], 'Display', None],
  [ 0x65d9, 'CjkUnifiedIdeograph65d9', [], 'Display', None],
  [ 0x65da, 'CjkUnifiedIdeograph65da', [], 'Display', None],
  [ 0x65db, 'CjkUnifiedIdeograph65db', [], 'Display', None],
  [ 0x65dc, 'CjkUnifiedIdeograph65dc', [], 'Display', None],
  [ 0x65dd, 'CjkUnifiedIdeograph65dd', [], 'Display', None],
  [ 0x65de, 'CjkUnifiedIdeograph65de', [], 'Display', None],
  [ 0x65df, 'CjkUnifiedIdeograph65df', [], 'Display', None],
  [ 0x65e0, 'CjkUnifiedIdeograph65e0', [], 'Display', None],
  [ 0x65e1, 'CjkUnifiedIdeograph65e1', [], 'Display', None],
  [ 0x65e2, 'CjkUnifiedIdeograph65e2', [], 'Display', None],
  [ 0x65e3, 'CjkUnifiedIdeograph65e3', [], 'Display', None],
  [ 0x65e4, 'CjkUnifiedIdeograph65e4', [], 'Display', None],
  [ 0x65e5, 'CjkUnifiedIdeograph65e5', [], 'Display', None],
  [ 0x65e6, 'CjkUnifiedIdeograph65e6', [], 'Display', None],
  [ 0x65e7, 'CjkUnifiedIdeograph65e7', [], 'Display', None],
  [ 0x65e8, 'CjkUnifiedIdeograph65e8', [], 'Display', None],
  [ 0x65e9, 'CjkUnifiedIdeograph65e9', [], 'Display', None],
  [ 0x65ea, 'CjkUnifiedIdeograph65ea', [], 'Display', None],
  [ 0x65eb, 'CjkUnifiedIdeograph65eb', [], 'Display', None],
  [ 0x65ec, 'CjkUnifiedIdeograph65ec', [], 'Display', None],
  [ 0x65ed, 'CjkUnifiedIdeograph65ed', [], 'Display', None],
  [ 0x65ee, 'CjkUnifiedIdeograph65ee', [], 'Display', None],
  [ 0x65ef, 'CjkUnifiedIdeograph65ef', [], 'Display', None],
  [ 0x65f0, 'CjkUnifiedIdeograph65f0', [], 'Display', None],
  [ 0x65f1, 'CjkUnifiedIdeograph65f1', [], 'Display', None],
  [ 0x65f2, 'CjkUnifiedIdeograph65f2', [], 'Display', None],
  [ 0x65f3, 'CjkUnifiedIdeograph65f3', [], 'Display', None],
  [ 0x65f4, 'CjkUnifiedIdeograph65f4', [], 'Display', None],
  [ 0x65f5, 'CjkUnifiedIdeograph65f5', [], 'Display', None],
  [ 0x65f6, 'CjkUnifiedIdeograph65f6', [], 'Display', None],
  [ 0x65f7, 'CjkUnifiedIdeograph65f7', [], 'Display', None],
  [ 0x65f8, 'CjkUnifiedIdeograph65f8', [], 'Display', None],
  [ 0x65f9, 'CjkUnifiedIdeograph65f9', [], 'Display', None],
  [ 0x65fa, 'CjkUnifiedIdeograph65fa', [], 'Display', None],
  [ 0x65fb, 'CjkUnifiedIdeograph65fb', [], 'Display', None],
  [ 0x65fc, 'CjkUnifiedIdeograph65fc', [], 'Display', None],
  [ 0x65fd, 'CjkUnifiedIdeograph65fd', [], 'Display', None],
  [ 0x65fe, 'CjkUnifiedIdeograph65fe', [], 'Display', None],
  [ 0x65ff, 'CjkUnifiedIdeograph65ff', [], 'Display', None],
  [ 0x6600, 'CjkUnifiedIdeograph6600', [], 'Display', None],
  [ 0x6601, 'CjkUnifiedIdeograph6601', [], 'Display', None],
  [ 0x6602, 'CjkUnifiedIdeograph6602', [], 'Display', None],
  [ 0x6603, 'CjkUnifiedIdeograph6603', [], 'Display', None],
  [ 0x6604, 'CjkUnifiedIdeograph6604', [], 'Display', None],
  [ 0x6605, 'CjkUnifiedIdeograph6605', [], 'Display', None],
  [ 0x6606, 'CjkUnifiedIdeograph6606', [], 'Display', None],
  [ 0x6607, 'CjkUnifiedIdeograph6607', [], 'Display', None],
  [ 0x6608, 'CjkUnifiedIdeograph6608', [], 'Display', None],
  [ 0x6609, 'CjkUnifiedIdeograph6609', [], 'Display', None],
  [ 0x660a, 'CjkUnifiedIdeograph660a', [], 'Display', None],
  [ 0x660b, 'CjkUnifiedIdeograph660b', [], 'Display', None],
  [ 0x660c, 'CjkUnifiedIdeograph660c', [], 'Display', None],
  [ 0x660d, 'CjkUnifiedIdeograph660d', [], 'Display', None],
  [ 0x660e, 'CjkUnifiedIdeograph660e', [], 'Display', None],
  [ 0x660f, 'CjkUnifiedIdeograph660f', [], 'Display', None],
  [ 0x6610, 'CjkUnifiedIdeograph6610', [], 'Display', None],
  [ 0x6611, 'CjkUnifiedIdeograph6611', [], 'Display', None],
  [ 0x6612, 'CjkUnifiedIdeograph6612', [], 'Display', None],
  [ 0x6613, 'CjkUnifiedIdeograph6613', [], 'Display', None],
  [ 0x6614, 'CjkUnifiedIdeograph6614', [], 'Display', None],
  [ 0x6615, 'CjkUnifiedIdeograph6615', [], 'Display', None],
  [ 0x6616, 'CjkUnifiedIdeograph6616', [], 'Display', None],
  [ 0x6617, 'CjkUnifiedIdeograph6617', [], 'Display', None],
  [ 0x6618, 'CjkUnifiedIdeograph6618', [], 'Display', None],
  [ 0x6619, 'CjkUnifiedIdeograph6619', [], 'Display', None],
  [ 0x661a, 'CjkUnifiedIdeograph661a', [], 'Display', None],
  [ 0x661b, 'CjkUnifiedIdeograph661b', [], 'Display', None],
  [ 0x661c, 'CjkUnifiedIdeograph661c', [], 'Display', None],
  [ 0x661d, 'CjkUnifiedIdeograph661d', [], 'Display', None],
  [ 0x661e, 'CjkUnifiedIdeograph661e', [], 'Display', None],
  [ 0x661f, 'CjkUnifiedIdeograph661f', [], 'Display', None],
  [ 0x6620, 'CjkUnifiedIdeograph6620', [], 'Display', None],
  [ 0x6621, 'CjkUnifiedIdeograph6621', [], 'Display', None],
  [ 0x6622, 'CjkUnifiedIdeograph6622', [], 'Display', None],
  [ 0x6623, 'CjkUnifiedIdeograph6623', [], 'Display', None],
  [ 0x6624, 'CjkUnifiedIdeograph6624', [], 'Display', None],
  [ 0x6625, 'CjkUnifiedIdeograph6625', [], 'Display', None],
  [ 0x6626, 'CjkUnifiedIdeograph6626', [], 'Display', None],
  [ 0x6627, 'CjkUnifiedIdeograph6627', [], 'Display', None],
  [ 0x6628, 'CjkUnifiedIdeograph6628', [], 'Display', None],
  [ 0x6629, 'CjkUnifiedIdeograph6629', [], 'Display', None],
  [ 0x662a, 'CjkUnifiedIdeograph662a', [], 'Display', None],
  [ 0x662b, 'CjkUnifiedIdeograph662b', [], 'Display', None],
  [ 0x662c, 'CjkUnifiedIdeograph662c', [], 'Display', None],
  [ 0x662d, 'CjkUnifiedIdeograph662d', [], 'Display', None],
  [ 0x662e, 'CjkUnifiedIdeograph662e', [], 'Display', None],
  [ 0x662f, 'CjkUnifiedIdeograph662f', [], 'Display', None],
  [ 0x6630, 'CjkUnifiedIdeograph6630', [], 'Display', None],
  [ 0x6631, 'CjkUnifiedIdeograph6631', [], 'Display', None],
  [ 0x6632, 'CjkUnifiedIdeograph6632', [], 'Display', None],
  [ 0x6633, 'CjkUnifiedIdeograph6633', [], 'Display', None],
  [ 0x6634, 'CjkUnifiedIdeograph6634', [], 'Display', None],
  [ 0x6635, 'CjkUnifiedIdeograph6635', [], 'Display', None],
  [ 0x6636, 'CjkUnifiedIdeograph6636', [], 'Display', None],
  [ 0x6637, 'CjkUnifiedIdeograph6637', [], 'Display', None],
  [ 0x6638, 'CjkUnifiedIdeograph6638', [], 'Display', None],
  [ 0x6639, 'CjkUnifiedIdeograph6639', [], 'Display', None],
  [ 0x663a, 'CjkUnifiedIdeograph663a', [], 'Display', None],
  [ 0x663b, 'CjkUnifiedIdeograph663b', [], 'Display', None],
  [ 0x663c, 'CjkUnifiedIdeograph663c', [], 'Display', None],
  [ 0x663d, 'CjkUnifiedIdeograph663d', [], 'Display', None],
  [ 0x663e, 'CjkUnifiedIdeograph663e', [], 'Display', None],
  [ 0x663f, 'CjkUnifiedIdeograph663f', [], 'Display', None],
  [ 0x6640, 'CjkUnifiedIdeograph6640', [], 'Display', None],
  [ 0x6641, 'CjkUnifiedIdeograph6641', [], 'Display', None],
  [ 0x6642, 'CjkUnifiedIdeograph6642', [], 'Display', None],
  [ 0x6643, 'CjkUnifiedIdeograph6643', [], 'Display', None],
  [ 0x6644, 'CjkUnifiedIdeograph6644', [], 'Display', None],
  [ 0x6645, 'CjkUnifiedIdeograph6645', [], 'Display', None],
  [ 0x6646, 'CjkUnifiedIdeograph6646', [], 'Display', None],
  [ 0x6647, 'CjkUnifiedIdeograph6647', [], 'Display', None],
  [ 0x6648, 'CjkUnifiedIdeograph6648', [], 'Display', None],
  [ 0x6649, 'CjkUnifiedIdeograph6649', [], 'Display', None],
  [ 0x664a, 'CjkUnifiedIdeograph664a', [], 'Display', None],
  [ 0x664b, 'CjkUnifiedIdeograph664b', [], 'Display', None],
  [ 0x664c, 'CjkUnifiedIdeograph664c', [], 'Display', None],
  [ 0x664d, 'CjkUnifiedIdeograph664d', [], 'Display', None],
  [ 0x664e, 'CjkUnifiedIdeograph664e', [], 'Display', None],
  [ 0x664f, 'CjkUnifiedIdeograph664f', [], 'Display', None],
  [ 0x6650, 'CjkUnifiedIdeograph6650', [], 'Display', None],
  [ 0x6651, 'CjkUnifiedIdeograph6651', [], 'Display', None],
  [ 0x6652, 'CjkUnifiedIdeograph6652', [], 'Display', None],
  [ 0x6653, 'CjkUnifiedIdeograph6653', [], 'Display', None],
  [ 0x6654, 'CjkUnifiedIdeograph6654', [], 'Display', None],
  [ 0x6655, 'CjkUnifiedIdeograph6655', [], 'Display', None],
  [ 0x6656, 'CjkUnifiedIdeograph6656', [], 'Display', None],
  [ 0x6657, 'CjkUnifiedIdeograph6657', [], 'Display', None],
  [ 0x6658, 'CjkUnifiedIdeograph6658', [], 'Display', None],
  [ 0x6659, 'CjkUnifiedIdeograph6659', [], 'Display', None],
  [ 0x665a, 'CjkUnifiedIdeograph665a', [], 'Display', None],
  [ 0x665b, 'CjkUnifiedIdeograph665b', [], 'Display', None],
  [ 0x665c, 'CjkUnifiedIdeograph665c', [], 'Display', None],
  [ 0x665d, 'CjkUnifiedIdeograph665d', [], 'Display', None],
  [ 0x665e, 'CjkUnifiedIdeograph665e', [], 'Display', None],
  [ 0x665f, 'CjkUnifiedIdeograph665f', [], 'Display', None],
  [ 0x6660, 'CjkUnifiedIdeograph6660', [], 'Display', None],
  [ 0x6661, 'CjkUnifiedIdeograph6661', [], 'Display', None],
  [ 0x6662, 'CjkUnifiedIdeograph6662', [], 'Display', None],
  [ 0x6663, 'CjkUnifiedIdeograph6663', [], 'Display', None],
  [ 0x6664, 'CjkUnifiedIdeograph6664', [], 'Display', None],
  [ 0x6665, 'CjkUnifiedIdeograph6665', [], 'Display', None],
  [ 0x6666, 'CjkUnifiedIdeograph6666', [], 'Display', None],
  [ 0x6667, 'CjkUnifiedIdeograph6667', [], 'Display', None],
  [ 0x6668, 'CjkUnifiedIdeograph6668', [], 'Display', None],
  [ 0x6669, 'CjkUnifiedIdeograph6669', [], 'Display', None],
  [ 0x666a, 'CjkUnifiedIdeograph666a', [], 'Display', None],
  [ 0x666b, 'CjkUnifiedIdeograph666b', [], 'Display', None],
  [ 0x666c, 'CjkUnifiedIdeograph666c', [], 'Display', None],
  [ 0x666d, 'CjkUnifiedIdeograph666d', [], 'Display', None],
  [ 0x666e, 'CjkUnifiedIdeograph666e', [], 'Display', None],
  [ 0x666f, 'CjkUnifiedIdeograph666f', [], 'Display', None],
  [ 0x6670, 'CjkUnifiedIdeograph6670', [], 'Display', None],
  [ 0x6671, 'CjkUnifiedIdeograph6671', [], 'Display', None],
  [ 0x6672, 'CjkUnifiedIdeograph6672', [], 'Display', None],
  [ 0x6673, 'CjkUnifiedIdeograph6673', [], 'Display', None],
  [ 0x6674, 'CjkUnifiedIdeograph6674', [], 'Display', None],
  [ 0x6675, 'CjkUnifiedIdeograph6675', [], 'Display', None],
  [ 0x6676, 'CjkUnifiedIdeograph6676', [], 'Display', None],
  [ 0x6677, 'CjkUnifiedIdeograph6677', [], 'Display', None],
  [ 0x6678, 'CjkUnifiedIdeograph6678', [], 'Display', None],
  [ 0x6679, 'CjkUnifiedIdeograph6679', [], 'Display', None],
  [ 0x667a, 'CjkUnifiedIdeograph667a', [], 'Display', None],
  [ 0x667b, 'CjkUnifiedIdeograph667b', [], 'Display', None],
  [ 0x667c, 'CjkUnifiedIdeograph667c', [], 'Display', None],
  [ 0x667d, 'CjkUnifiedIdeograph667d', [], 'Display', None],
  [ 0x667e, 'CjkUnifiedIdeograph667e', [], 'Display', None],
  [ 0x667f, 'CjkUnifiedIdeograph667f', [], 'Display', None],
  [ 0x6680, 'CjkUnifiedIdeograph6680', [], 'Display', None],
  [ 0x6681, 'CjkUnifiedIdeograph6681', [], 'Display', None],
  [ 0x6682, 'CjkUnifiedIdeograph6682', [], 'Display', None],
  [ 0x6683, 'CjkUnifiedIdeograph6683', [], 'Display', None],
  [ 0x6684, 'CjkUnifiedIdeograph6684', [], 'Display', None],
  [ 0x6685, 'CjkUnifiedIdeograph6685', [], 'Display', None],
  [ 0x6686, 'CjkUnifiedIdeograph6686', [], 'Display', None],
  [ 0x6687, 'CjkUnifiedIdeograph6687', [], 'Display', None],
  [ 0x6688, 'CjkUnifiedIdeograph6688', [], 'Display', None],
  [ 0x6689, 'CjkUnifiedIdeograph6689', [], 'Display', None],
  [ 0x668a, 'CjkUnifiedIdeograph668a', [], 'Display', None],
  [ 0x668b, 'CjkUnifiedIdeograph668b', [], 'Display', None],
  [ 0x668c, 'CjkUnifiedIdeograph668c', [], 'Display', None],
  [ 0x668d, 'CjkUnifiedIdeograph668d', [], 'Display', None],
  [ 0x668e, 'CjkUnifiedIdeograph668e', [], 'Display', None],
  [ 0x668f, 'CjkUnifiedIdeograph668f', [], 'Display', None],
  [ 0x6690, 'CjkUnifiedIdeograph6690', [], 'Display', None],
  [ 0x6691, 'CjkUnifiedIdeograph6691', [], 'Display', None],
  [ 0x6692, 'CjkUnifiedIdeograph6692', [], 'Display', None],
  [ 0x6693, 'CjkUnifiedIdeograph6693', [], 'Display', None],
  [ 0x6694, 'CjkUnifiedIdeograph6694', [], 'Display', None],
  [ 0x6695, 'CjkUnifiedIdeograph6695', [], 'Display', None],
  [ 0x6696, 'CjkUnifiedIdeograph6696', [], 'Display', None],
  [ 0x6697, 'CjkUnifiedIdeograph6697', [], 'Display', None],
  [ 0x6698, 'CjkUnifiedIdeograph6698', [], 'Display', None],
  [ 0x6699, 'CjkUnifiedIdeograph6699', [], 'Display', None],
  [ 0x669a, 'CjkUnifiedIdeograph669a', [], 'Display', None],
  [ 0x669b, 'CjkUnifiedIdeograph669b', [], 'Display', None],
  [ 0x669c, 'CjkUnifiedIdeograph669c', [], 'Display', None],
  [ 0x669d, 'CjkUnifiedIdeograph669d', [], 'Display', None],
  [ 0x669e, 'CjkUnifiedIdeograph669e', [], 'Display', None],
  [ 0x669f, 'CjkUnifiedIdeograph669f', [], 'Display', None],
  [ 0x66a0, 'CjkUnifiedIdeograph66a0', [], 'Display', None],
  [ 0x66a1, 'CjkUnifiedIdeograph66a1', [], 'Display', None],
  [ 0x66a2, 'CjkUnifiedIdeograph66a2', [], 'Display', None],
  [ 0x66a3, 'CjkUnifiedIdeograph66a3', [], 'Display', None],
  [ 0x66a4, 'CjkUnifiedIdeograph66a4', [], 'Display', None],
  [ 0x66a5, 'CjkUnifiedIdeograph66a5', [], 'Display', None],
  [ 0x66a6, 'CjkUnifiedIdeograph66a6', [], 'Display', None],
  [ 0x66a7, 'CjkUnifiedIdeograph66a7', [], 'Display', None],
  [ 0x66a8, 'CjkUnifiedIdeograph66a8', [], 'Display', None],
  [ 0x66a9, 'CjkUnifiedIdeograph66a9', [], 'Display', None],
  [ 0x66aa, 'CjkUnifiedIdeograph66aa', [], 'Display', None],
  [ 0x66ab, 'CjkUnifiedIdeograph66ab', [], 'Display', None],
  [ 0x66ac, 'CjkUnifiedIdeograph66ac', [], 'Display', None],
  [ 0x66ad, 'CjkUnifiedIdeograph66ad', [], 'Display', None],
  [ 0x66ae, 'CjkUnifiedIdeograph66ae', [], 'Display', None],
  [ 0x66af, 'CjkUnifiedIdeograph66af', [], 'Display', None],
  [ 0x66b0, 'CjkUnifiedIdeograph66b0', [], 'Display', None],
  [ 0x66b1, 'CjkUnifiedIdeograph66b1', [], 'Display', None],
  [ 0x66b2, 'CjkUnifiedIdeograph66b2', [], 'Display', None],
  [ 0x66b3, 'CjkUnifiedIdeograph66b3', [], 'Display', None],
  [ 0x66b4, 'CjkUnifiedIdeograph66b4', [], 'Display', None],
  [ 0x66b5, 'CjkUnifiedIdeograph66b5', [], 'Display', None],
  [ 0x66b6, 'CjkUnifiedIdeograph66b6', [], 'Display', None],
  [ 0x66b7, 'CjkUnifiedIdeograph66b7', [], 'Display', None],
  [ 0x66b8, 'CjkUnifiedIdeograph66b8', [], 'Display', None],
  [ 0x66b9, 'CjkUnifiedIdeograph66b9', [], 'Display', None],
  [ 0x66ba, 'CjkUnifiedIdeograph66ba', [], 'Display', None],
  [ 0x66bb, 'CjkUnifiedIdeograph66bb', [], 'Display', None],
  [ 0x66bc, 'CjkUnifiedIdeograph66bc', [], 'Display', None],
  [ 0x66bd, 'CjkUnifiedIdeograph66bd', [], 'Display', None],
  [ 0x66be, 'CjkUnifiedIdeograph66be', [], 'Display', None],
  [ 0x66bf, 'CjkUnifiedIdeograph66bf', [], 'Display', None],
  [ 0x66c0, 'CjkUnifiedIdeograph66c0', [], 'Display', None],
  [ 0x66c1, 'CjkUnifiedIdeograph66c1', [], 'Display', None],
  [ 0x66c2, 'CjkUnifiedIdeograph66c2', [], 'Display', None],
  [ 0x66c3, 'CjkUnifiedIdeograph66c3', [], 'Display', None],
  [ 0x66c4, 'CjkUnifiedIdeograph66c4', [], 'Display', None],
  [ 0x66c5, 'CjkUnifiedIdeograph66c5', [], 'Display', None],
  [ 0x66c6, 'CjkUnifiedIdeograph66c6', [], 'Display', None],
  [ 0x66c7, 'CjkUnifiedIdeograph66c7', [], 'Display', None],
  [ 0x66c8, 'CjkUnifiedIdeograph66c8', [], 'Display', None],
  [ 0x66c9, 'CjkUnifiedIdeograph66c9', [], 'Display', None],
  [ 0x66ca, 'CjkUnifiedIdeograph66ca', [], 'Display', None],
  [ 0x66cb, 'CjkUnifiedIdeograph66cb', [], 'Display', None],
  [ 0x66cc, 'CjkUnifiedIdeograph66cc', [], 'Display', None],
  [ 0x66cd, 'CjkUnifiedIdeograph66cd', [], 'Display', None],
  [ 0x66ce, 'CjkUnifiedIdeograph66ce', [], 'Display', None],
  [ 0x66cf, 'CjkUnifiedIdeograph66cf', [], 'Display', None],
  [ 0x66d0, 'CjkUnifiedIdeograph66d0', [], 'Display', None],
  [ 0x66d1, 'CjkUnifiedIdeograph66d1', [], 'Display', None],
  [ 0x66d2, 'CjkUnifiedIdeograph66d2', [], 'Display', None],
  [ 0x66d3, 'CjkUnifiedIdeograph66d3', [], 'Display', None],
  [ 0x66d4, 'CjkUnifiedIdeograph66d4', [], 'Display', None],
  [ 0x66d5, 'CjkUnifiedIdeograph66d5', [], 'Display', None],
  [ 0x66d6, 'CjkUnifiedIdeograph66d6', [], 'Display', None],
  [ 0x66d7, 'CjkUnifiedIdeograph66d7', [], 'Display', None],
  [ 0x66d8, 'CjkUnifiedIdeograph66d8', [], 'Display', None],
  [ 0x66d9, 'CjkUnifiedIdeograph66d9', [], 'Display', None],
  [ 0x66da, 'CjkUnifiedIdeograph66da', [], 'Display', None],
  [ 0x66db, 'CjkUnifiedIdeograph66db', [], 'Display', None],
  [ 0x66dc, 'CjkUnifiedIdeograph66dc', [], 'Display', None],
  [ 0x66dd, 'CjkUnifiedIdeograph66dd', [], 'Display', None],
  [ 0x66de, 'CjkUnifiedIdeograph66de', [], 'Display', None],
  [ 0x66df, 'CjkUnifiedIdeograph66df', [], 'Display', None],
  [ 0x66e0, 'CjkUnifiedIdeograph66e0', [], 'Display', None],
  [ 0x66e1, 'CjkUnifiedIdeograph66e1', [], 'Display', None],
  [ 0x66e2, 'CjkUnifiedIdeograph66e2', [], 'Display', None],
  [ 0x66e3, 'CjkUnifiedIdeograph66e3', [], 'Display', None],
  [ 0x66e4, 'CjkUnifiedIdeograph66e4', [], 'Display', None],
  [ 0x66e5, 'CjkUnifiedIdeograph66e5', [], 'Display', None],
  [ 0x66e6, 'CjkUnifiedIdeograph66e6', [], 'Display', None],
  [ 0x66e7, 'CjkUnifiedIdeograph66e7', [], 'Display', None],
  [ 0x66e8, 'CjkUnifiedIdeograph66e8', [], 'Display', None],
  [ 0x66e9, 'CjkUnifiedIdeograph66e9', [], 'Display', None],
  [ 0x66ea, 'CjkUnifiedIdeograph66ea', [], 'Display', None],
  [ 0x66eb, 'CjkUnifiedIdeograph66eb', [], 'Display', None],
  [ 0x66ec, 'CjkUnifiedIdeograph66ec', [], 'Display', None],
  [ 0x66ed, 'CjkUnifiedIdeograph66ed', [], 'Display', None],
  [ 0x66ee, 'CjkUnifiedIdeograph66ee', [], 'Display', None],
  [ 0x66ef, 'CjkUnifiedIdeograph66ef', [], 'Display', None],
  [ 0x66f0, 'CjkUnifiedIdeograph66f0', [], 'Display', None],
  [ 0x66f1, 'CjkUnifiedIdeograph66f1', [], 'Display', None],
  [ 0x66f2, 'CjkUnifiedIdeograph66f2', [], 'Display', None],
  [ 0x66f3, 'CjkUnifiedIdeograph66f3', [], 'Display', None],
  [ 0x66f4, 'CjkUnifiedIdeograph66f4', [], 'Display', None],
  [ 0x66f5, 'CjkUnifiedIdeograph66f5', [], 'Display', None],
  [ 0x66f6, 'CjkUnifiedIdeograph66f6', [], 'Display', None],
  [ 0x66f7, 'CjkUnifiedIdeograph66f7', [], 'Display', None],
  [ 0x66f8, 'CjkUnifiedIdeograph66f8', [], 'Display', None],
  [ 0x66f9, 'CjkUnifiedIdeograph66f9', [], 'Display', None],
  [ 0x66fa, 'CjkUnifiedIdeograph66fa', [], 'Display', None],
  [ 0x66fb, 'CjkUnifiedIdeograph66fb', [], 'Display', None],
  [ 0x66fc, 'CjkUnifiedIdeograph66fc', [], 'Display', None],
  [ 0x66fd, 'CjkUnifiedIdeograph66fd', [], 'Display', None],
  [ 0x66fe, 'CjkUnifiedIdeograph66fe', [], 'Display', None],
  [ 0x66ff, 'CjkUnifiedIdeograph66ff', [], 'Display', None],
  [ 0x6700, 'CjkUnifiedIdeograph6700', [], 'Display', None],
  [ 0x6701, 'CjkUnifiedIdeograph6701', [], 'Display', None],
  [ 0x6702, 'CjkUnifiedIdeograph6702', [], 'Display', None],
  [ 0x6703, 'CjkUnifiedIdeograph6703', [], 'Display', None],
  [ 0x6704, 'CjkUnifiedIdeograph6704', [], 'Display', None],
  [ 0x6705, 'CjkUnifiedIdeograph6705', [], 'Display', None],
  [ 0x6706, 'CjkUnifiedIdeograph6706', [], 'Display', None],
  [ 0x6707, 'CjkUnifiedIdeograph6707', [], 'Display', None],
  [ 0x6708, 'CjkUnifiedIdeograph6708', [], 'Display', None],
  [ 0x6709, 'CjkUnifiedIdeograph6709', [], 'Display', None],
  [ 0x670a, 'CjkUnifiedIdeograph670a', [], 'Display', None],
  [ 0x670b, 'CjkUnifiedIdeograph670b', [], 'Display', None],
  [ 0x670c, 'CjkUnifiedIdeograph670c', [], 'Display', None],
  [ 0x670d, 'CjkUnifiedIdeograph670d', [], 'Display', None],
  [ 0x670e, 'CjkUnifiedIdeograph670e', [], 'Display', None],
  [ 0x670f, 'CjkUnifiedIdeograph670f', [], 'Display', None],
  [ 0x6710, 'CjkUnifiedIdeograph6710', [], 'Display', None],
  [ 0x6711, 'CjkUnifiedIdeograph6711', [], 'Display', None],
  [ 0x6712, 'CjkUnifiedIdeograph6712', [], 'Display', None],
  [ 0x6713, 'CjkUnifiedIdeograph6713', [], 'Display', None],
  [ 0x6714, 'CjkUnifiedIdeograph6714', [], 'Display', None],
  [ 0x6715, 'CjkUnifiedIdeograph6715', [], 'Display', None],
  [ 0x6716, 'CjkUnifiedIdeograph6716', [], 'Display', None],
  [ 0x6717, 'CjkUnifiedIdeograph6717', [], 'Display', None],
  [ 0x6718, 'CjkUnifiedIdeograph6718', [], 'Display', None],
  [ 0x6719, 'CjkUnifiedIdeograph6719', [], 'Display', None],
  [ 0x671a, 'CjkUnifiedIdeograph671a', [], 'Display', None],
  [ 0x671b, 'CjkUnifiedIdeograph671b', [], 'Display', None],
  [ 0x671c, 'CjkUnifiedIdeograph671c', [], 'Display', None],
  [ 0x671d, 'CjkUnifiedIdeograph671d', [], 'Display', None],
  [ 0x671e, 'CjkUnifiedIdeograph671e', [], 'Display', None],
  [ 0x671f, 'CjkUnifiedIdeograph671f', [], 'Display', None],
  [ 0x6720, 'CjkUnifiedIdeograph6720', [], 'Display', None],
  [ 0x6721, 'CjkUnifiedIdeograph6721', [], 'Display', None],
  [ 0x6722, 'CjkUnifiedIdeograph6722', [], 'Display', None],
  [ 0x6723, 'CjkUnifiedIdeograph6723', [], 'Display', None],
  [ 0x6724, 'CjkUnifiedIdeograph6724', [], 'Display', None],
  [ 0x6725, 'CjkUnifiedIdeograph6725', [], 'Display', None],
  [ 0x6726, 'CjkUnifiedIdeograph6726', [], 'Display', None],
  [ 0x6727, 'CjkUnifiedIdeograph6727', [], 'Display', None],
  [ 0x6728, 'CjkUnifiedIdeograph6728', [], 'Display', None],
  [ 0x6729, 'CjkUnifiedIdeograph6729', [], 'Display', None],
  [ 0x672a, 'CjkUnifiedIdeograph672a', [], 'Display', None],
  [ 0x672b, 'CjkUnifiedIdeograph672b', [], 'Display', None],
  [ 0x672c, 'CjkUnifiedIdeograph672c', [], 'Display', None],
  [ 0x672d, 'CjkUnifiedIdeograph672d', [], 'Display', None],
  [ 0x672e, 'CjkUnifiedIdeograph672e', [], 'Display', None],
  [ 0x672f, 'CjkUnifiedIdeograph672f', [], 'Display', None],
  [ 0x6730, 'CjkUnifiedIdeograph6730', [], 'Display', None],
  [ 0x6731, 'CjkUnifiedIdeograph6731', [], 'Display', None],
  [ 0x6732, 'CjkUnifiedIdeograph6732', [], 'Display', None],
  [ 0x6733, 'CjkUnifiedIdeograph6733', [], 'Display', None],
  [ 0x6734, 'CjkUnifiedIdeograph6734', [], 'Display', None],
  [ 0x6735, 'CjkUnifiedIdeograph6735', [], 'Display', None],
  [ 0x6736, 'CjkUnifiedIdeograph6736', [], 'Display', None],
  [ 0x6737, 'CjkUnifiedIdeograph6737', [], 'Display', None],
  [ 0x6738, 'CjkUnifiedIdeograph6738', [], 'Display', None],
  [ 0x6739, 'CjkUnifiedIdeograph6739', [], 'Display', None],
  [ 0x673a, 'CjkUnifiedIdeograph673a', [], 'Display', None],
  [ 0x673b, 'CjkUnifiedIdeograph673b', [], 'Display', None],
  [ 0x673c, 'CjkUnifiedIdeograph673c', [], 'Display', None],
  [ 0x673d, 'CjkUnifiedIdeograph673d', [], 'Display', None],
  [ 0x673e, 'CjkUnifiedIdeograph673e', [], 'Display', None],
  [ 0x673f, 'CjkUnifiedIdeograph673f', [], 'Display', None],
  [ 0x6740, 'CjkUnifiedIdeograph6740', [], 'Display', None],
  [ 0x6741, 'CjkUnifiedIdeograph6741', [], 'Display', None],
  [ 0x6742, 'CjkUnifiedIdeograph6742', [], 'Display', None],
  [ 0x6743, 'CjkUnifiedIdeograph6743', [], 'Display', None],
  [ 0x6744, 'CjkUnifiedIdeograph6744', [], 'Display', None],
  [ 0x6745, 'CjkUnifiedIdeograph6745', [], 'Display', None],
  [ 0x6746, 'CjkUnifiedIdeograph6746', [], 'Display', None],
  [ 0x6747, 'CjkUnifiedIdeograph6747', [], 'Display', None],
  [ 0x6748, 'CjkUnifiedIdeograph6748', [], 'Display', None],
  [ 0x6749, 'CjkUnifiedIdeograph6749', [], 'Display', None],
  [ 0x674a, 'CjkUnifiedIdeograph674a', [], 'Display', None],
  [ 0x674b, 'CjkUnifiedIdeograph674b', [], 'Display', None],
  [ 0x674c, 'CjkUnifiedIdeograph674c', [], 'Display', None],
  [ 0x674d, 'CjkUnifiedIdeograph674d', [], 'Display', None],
  [ 0x674e, 'CjkUnifiedIdeograph674e', [], 'Display', None],
  [ 0x674f, 'CjkUnifiedIdeograph674f', [], 'Display', None],
  [ 0x6750, 'CjkUnifiedIdeograph6750', [], 'Display', None],
  [ 0x6751, 'CjkUnifiedIdeograph6751', [], 'Display', None],
  [ 0x6752, 'CjkUnifiedIdeograph6752', [], 'Display', None],
  [ 0x6753, 'CjkUnifiedIdeograph6753', [], 'Display', None],
  [ 0x6754, 'CjkUnifiedIdeograph6754', [], 'Display', None],
  [ 0x6755, 'CjkUnifiedIdeograph6755', [], 'Display', None],
  [ 0x6756, 'CjkUnifiedIdeograph6756', [], 'Display', None],
  [ 0x6757, 'CjkUnifiedIdeograph6757', [], 'Display', None],
  [ 0x6758, 'CjkUnifiedIdeograph6758', [], 'Display', None],
  [ 0x6759, 'CjkUnifiedIdeograph6759', [], 'Display', None],
  [ 0x675a, 'CjkUnifiedIdeograph675a', [], 'Display', None],
  [ 0x675b, 'CjkUnifiedIdeograph675b', [], 'Display', None],
  [ 0x675c, 'CjkUnifiedIdeograph675c', [], 'Display', None],
  [ 0x675d, 'CjkUnifiedIdeograph675d', [], 'Display', None],
  [ 0x675e, 'CjkUnifiedIdeograph675e', [], 'Display', None],
  [ 0x675f, 'CjkUnifiedIdeograph675f', [], 'Display', None],
  [ 0x6760, 'CjkUnifiedIdeograph6760', [], 'Display', None],
  [ 0x6761, 'CjkUnifiedIdeograph6761', [], 'Display', None],
  [ 0x6762, 'CjkUnifiedIdeograph6762', [], 'Display', None],
  [ 0x6763, 'CjkUnifiedIdeograph6763', [], 'Display', None],
  [ 0x6764, 'CjkUnifiedIdeograph6764', [], 'Display', None],
  [ 0x6765, 'CjkUnifiedIdeograph6765', [], 'Display', None],
  [ 0x6766, 'CjkUnifiedIdeograph6766', [], 'Display', None],
  [ 0x6767, 'CjkUnifiedIdeograph6767', [], 'Display', None],
  [ 0x6768, 'CjkUnifiedIdeograph6768', [], 'Display', None],
  [ 0x6769, 'CjkUnifiedIdeograph6769', [], 'Display', None],
  [ 0x676a, 'CjkUnifiedIdeograph676a', [], 'Display', None],
  [ 0x676b, 'CjkUnifiedIdeograph676b', [], 'Display', None],
  [ 0x676c, 'CjkUnifiedIdeograph676c', [], 'Display', None],
  [ 0x676d, 'CjkUnifiedIdeograph676d', [], 'Display', None],
  [ 0x676e, 'CjkUnifiedIdeograph676e', [], 'Display', None],
  [ 0x676f, 'CjkUnifiedIdeograph676f', [], 'Display', None],
  [ 0x6770, 'CjkUnifiedIdeograph6770', [], 'Display', None],
  [ 0x6771, 'CjkUnifiedIdeograph6771', [], 'Display', None],
  [ 0x6772, 'CjkUnifiedIdeograph6772', [], 'Display', None],
  [ 0x6773, 'CjkUnifiedIdeograph6773', [], 'Display', None],
  [ 0x6774, 'CjkUnifiedIdeograph6774', [], 'Display', None],
  [ 0x6775, 'CjkUnifiedIdeograph6775', [], 'Display', None],
  [ 0x6776, 'CjkUnifiedIdeograph6776', [], 'Display', None],
  [ 0x6777, 'CjkUnifiedIdeograph6777', [], 'Display', None],
  [ 0x6778, 'CjkUnifiedIdeograph6778', [], 'Display', None],
  [ 0x6779, 'CjkUnifiedIdeograph6779', [], 'Display', None],
  [ 0x677a, 'CjkUnifiedIdeograph677a', [], 'Display', None],
  [ 0x677b, 'CjkUnifiedIdeograph677b', [], 'Display', None],
  [ 0x677c, 'CjkUnifiedIdeograph677c', [], 'Display', None],
  [ 0x677d, 'CjkUnifiedIdeograph677d', [], 'Display', None],
  [ 0x677e, 'CjkUnifiedIdeograph677e', [], 'Display', None],
  [ 0x677f, 'CjkUnifiedIdeograph677f', [], 'Display', None],
  [ 0x6780, 'CjkUnifiedIdeograph6780', [], 'Display', None],
  [ 0x6781, 'CjkUnifiedIdeograph6781', [], 'Display', None],
  [ 0x6782, 'CjkUnifiedIdeograph6782', [], 'Display', None],
  [ 0x6783, 'CjkUnifiedIdeograph6783', [], 'Display', None],
  [ 0x6784, 'CjkUnifiedIdeograph6784', [], 'Display', None],
  [ 0x6785, 'CjkUnifiedIdeograph6785', [], 'Display', None],
  [ 0x6786, 'CjkUnifiedIdeograph6786', [], 'Display', None],
  [ 0x6787, 'CjkUnifiedIdeograph6787', [], 'Display', None],
  [ 0x6788, 'CjkUnifiedIdeograph6788', [], 'Display', None],
  [ 0x6789, 'CjkUnifiedIdeograph6789', [], 'Display', None],
  [ 0x678a, 'CjkUnifiedIdeograph678a', [], 'Display', None],
  [ 0x678b, 'CjkUnifiedIdeograph678b', [], 'Display', None],
  [ 0x678c, 'CjkUnifiedIdeograph678c', [], 'Display', None],
  [ 0x678d, 'CjkUnifiedIdeograph678d', [], 'Display', None],
  [ 0x678e, 'CjkUnifiedIdeograph678e', [], 'Display', None],
  [ 0x678f, 'CjkUnifiedIdeograph678f', [], 'Display', None],
  [ 0x6790, 'CjkUnifiedIdeograph6790', [], 'Display', None],
  [ 0x6791, 'CjkUnifiedIdeograph6791', [], 'Display', None],
  [ 0x6792, 'CjkUnifiedIdeograph6792', [], 'Display', None],
  [ 0x6793, 'CjkUnifiedIdeograph6793', [], 'Display', None],
  [ 0x6794, 'CjkUnifiedIdeograph6794', [], 'Display', None],
  [ 0x6795, 'CjkUnifiedIdeograph6795', [], 'Display', None],
  [ 0x6796, 'CjkUnifiedIdeograph6796', [], 'Display', None],
  [ 0x6797, 'CjkUnifiedIdeograph6797', [], 'Display', None],
  [ 0x6798, 'CjkUnifiedIdeograph6798', [], 'Display', None],
  [ 0x6799, 'CjkUnifiedIdeograph6799', [], 'Display', None],
  [ 0x679a, 'CjkUnifiedIdeograph679a', [], 'Display', None],
  [ 0x679b, 'CjkUnifiedIdeograph679b', [], 'Display', None],
  [ 0x679c, 'CjkUnifiedIdeograph679c', [], 'Display', None],
  [ 0x679d, 'CjkUnifiedIdeograph679d', [], 'Display', None],
  [ 0x679e, 'CjkUnifiedIdeograph679e', [], 'Display', None],
  [ 0x679f, 'CjkUnifiedIdeograph679f', [], 'Display', None],
  [ 0x67a0, 'CjkUnifiedIdeograph67a0', [], 'Display', None],
  [ 0x67a1, 'CjkUnifiedIdeograph67a1', [], 'Display', None],
  [ 0x67a2, 'CjkUnifiedIdeograph67a2', [], 'Display', None],
  [ 0x67a3, 'CjkUnifiedIdeograph67a3', [], 'Display', None],
  [ 0x67a4, 'CjkUnifiedIdeograph67a4', [], 'Display', None],
  [ 0x67a5, 'CjkUnifiedIdeograph67a5', [], 'Display', None],
  [ 0x67a6, 'CjkUnifiedIdeograph67a6', [], 'Display', None],
  [ 0x67a7, 'CjkUnifiedIdeograph67a7', [], 'Display', None],
  [ 0x67a8, 'CjkUnifiedIdeograph67a8', [], 'Display', None],
  [ 0x67a9, 'CjkUnifiedIdeograph67a9', [], 'Display', None],
  [ 0x67aa, 'CjkUnifiedIdeograph67aa', [], 'Display', None],
  [ 0x67ab, 'CjkUnifiedIdeograph67ab', [], 'Display', None],
  [ 0x67ac, 'CjkUnifiedIdeograph67ac', [], 'Display', None],
  [ 0x67ad, 'CjkUnifiedIdeograph67ad', [], 'Display', None],
  [ 0x67ae, 'CjkUnifiedIdeograph67ae', [], 'Display', None],
  [ 0x67af, 'CjkUnifiedIdeograph67af', [], 'Display', None],
  [ 0x67b0, 'CjkUnifiedIdeograph67b0', [], 'Display', None],
  [ 0x67b1, 'CjkUnifiedIdeograph67b1', [], 'Display', None],
  [ 0x67b2, 'CjkUnifiedIdeograph67b2', [], 'Display', None],
  [ 0x67b3, 'CjkUnifiedIdeograph67b3', [], 'Display', None],
  [ 0x67b4, 'CjkUnifiedIdeograph67b4', [], 'Display', None],
  [ 0x67b5, 'CjkUnifiedIdeograph67b5', [], 'Display', None],
  [ 0x67b6, 'CjkUnifiedIdeograph67b6', [], 'Display', None],
  [ 0x67b7, 'CjkUnifiedIdeograph67b7', [], 'Display', None],
  [ 0x67b8, 'CjkUnifiedIdeograph67b8', [], 'Display', None],
  [ 0x67b9, 'CjkUnifiedIdeograph67b9', [], 'Display', None],
  [ 0x67ba, 'CjkUnifiedIdeograph67ba', [], 'Display', None],
  [ 0x67bb, 'CjkUnifiedIdeograph67bb', [], 'Display', None],
  [ 0x67bc, 'CjkUnifiedIdeograph67bc', [], 'Display', None],
  [ 0x67bd, 'CjkUnifiedIdeograph67bd', [], 'Display', None],
  [ 0x67be, 'CjkUnifiedIdeograph67be', [], 'Display', None],
  [ 0x67bf, 'CjkUnifiedIdeograph67bf', [], 'Display', None],
  [ 0x67c0, 'CjkUnifiedIdeograph67c0', [], 'Display', None],
  [ 0x67c1, 'CjkUnifiedIdeograph67c1', [], 'Display', None],
  [ 0x67c2, 'CjkUnifiedIdeograph67c2', [], 'Display', None],
  [ 0x67c3, 'CjkUnifiedIdeograph67c3', [], 'Display', None],
  [ 0x67c4, 'CjkUnifiedIdeograph67c4', [], 'Display', None],
  [ 0x67c5, 'CjkUnifiedIdeograph67c5', [], 'Display', None],
  [ 0x67c6, 'CjkUnifiedIdeograph67c6', [], 'Display', None],
  [ 0x67c7, 'CjkUnifiedIdeograph67c7', [], 'Display', None],
  [ 0x67c8, 'CjkUnifiedIdeograph67c8', [], 'Display', None],
  [ 0x67c9, 'CjkUnifiedIdeograph67c9', [], 'Display', None],
  [ 0x67ca, 'CjkUnifiedIdeograph67ca', [], 'Display', None],
  [ 0x67cb, 'CjkUnifiedIdeograph67cb', [], 'Display', None],
  [ 0x67cc, 'CjkUnifiedIdeograph67cc', [], 'Display', None],
  [ 0x67cd, 'CjkUnifiedIdeograph67cd', [], 'Display', None],
  [ 0x67ce, 'CjkUnifiedIdeograph67ce', [], 'Display', None],
  [ 0x67cf, 'CjkUnifiedIdeograph67cf', [], 'Display', None],
  [ 0x67d0, 'CjkUnifiedIdeograph67d0', [], 'Display', None],
  [ 0x67d1, 'CjkUnifiedIdeograph67d1', [], 'Display', None],
  [ 0x67d2, 'CjkUnifiedIdeograph67d2', [], 'Display', None],
  [ 0x67d3, 'CjkUnifiedIdeograph67d3', [], 'Display', None],
  [ 0x67d4, 'CjkUnifiedIdeograph67d4', [], 'Display', None],
  [ 0x67d5, 'CjkUnifiedIdeograph67d5', [], 'Display', None],
  [ 0x67d6, 'CjkUnifiedIdeograph67d6', [], 'Display', None],
  [ 0x67d7, 'CjkUnifiedIdeograph67d7', [], 'Display', None],
  [ 0x67d8, 'CjkUnifiedIdeograph67d8', [], 'Display', None],
  [ 0x67d9, 'CjkUnifiedIdeograph67d9', [], 'Display', None],
  [ 0x67da, 'CjkUnifiedIdeograph67da', [], 'Display', None],
  [ 0x67db, 'CjkUnifiedIdeograph67db', [], 'Display', None],
  [ 0x67dc, 'CjkUnifiedIdeograph67dc', [], 'Display', None],
  [ 0x67dd, 'CjkUnifiedIdeograph67dd', [], 'Display', None],
  [ 0x67de, 'CjkUnifiedIdeograph67de', [], 'Display', None],
  [ 0x67df, 'CjkUnifiedIdeograph67df', [], 'Display', None],
  [ 0x67e0, 'CjkUnifiedIdeograph67e0', [], 'Display', None],
  [ 0x67e1, 'CjkUnifiedIdeograph67e1', [], 'Display', None],
  [ 0x67e2, 'CjkUnifiedIdeograph67e2', [], 'Display', None],
  [ 0x67e3, 'CjkUnifiedIdeograph67e3', [], 'Display', None],
  [ 0x67e4, 'CjkUnifiedIdeograph67e4', [], 'Display', None],
  [ 0x67e5, 'CjkUnifiedIdeograph67e5', [], 'Display', None],
  [ 0x67e6, 'CjkUnifiedIdeograph67e6', [], 'Display', None],
  [ 0x67e7, 'CjkUnifiedIdeograph67e7', [], 'Display', None],
  [ 0x67e8, 'CjkUnifiedIdeograph67e8', [], 'Display', None],
  [ 0x67e9, 'CjkUnifiedIdeograph67e9', [], 'Display', None],
  [ 0x67ea, 'CjkUnifiedIdeograph67ea', [], 'Display', None],
  [ 0x67eb, 'CjkUnifiedIdeograph67eb', [], 'Display', None],
  [ 0x67ec, 'CjkUnifiedIdeograph67ec', [], 'Display', None],
  [ 0x67ed, 'CjkUnifiedIdeograph67ed', [], 'Display', None],
  [ 0x67ee, 'CjkUnifiedIdeograph67ee', [], 'Display', None],
  [ 0x67ef, 'CjkUnifiedIdeograph67ef', [], 'Display', None],
  [ 0x67f0, 'CjkUnifiedIdeograph67f0', [], 'Display', None],
  [ 0x67f1, 'CjkUnifiedIdeograph67f1', [], 'Display', None],
  [ 0x67f2, 'CjkUnifiedIdeograph67f2', [], 'Display', None],
  [ 0x67f3, 'CjkUnifiedIdeograph67f3', [], 'Display', None],
  [ 0x67f4, 'CjkUnifiedIdeograph67f4', [], 'Display', None],
  [ 0x67f5, 'CjkUnifiedIdeograph67f5', [], 'Display', None],
  [ 0x67f6, 'CjkUnifiedIdeograph67f6', [], 'Display', None],
  [ 0x67f7, 'CjkUnifiedIdeograph67f7', [], 'Display', None],
  [ 0x67f8, 'CjkUnifiedIdeograph67f8', [], 'Display', None],
  [ 0x67f9, 'CjkUnifiedIdeograph67f9', [], 'Display', None],
  [ 0x67fa, 'CjkUnifiedIdeograph67fa', [], 'Display', None],
  [ 0x67fb, 'CjkUnifiedIdeograph67fb', [], 'Display', None],
  [ 0x67fc, 'CjkUnifiedIdeograph67fc', [], 'Display', None],
  [ 0x67fd, 'CjkUnifiedIdeograph67fd', [], 'Display', None],
  [ 0x67fe, 'CjkUnifiedIdeograph67fe', [], 'Display', None],
  [ 0x67ff, 'CjkUnifiedIdeograph67ff', [], 'Display', None],
  [ 0x6800, 'CjkUnifiedIdeograph6800', [], 'Display', None],
  [ 0x6801, 'CjkUnifiedIdeograph6801', [], 'Display', None],
  [ 0x6802, 'CjkUnifiedIdeograph6802', [], 'Display', None],
  [ 0x6803, 'CjkUnifiedIdeograph6803', [], 'Display', None],
  [ 0x6804, 'CjkUnifiedIdeograph6804', [], 'Display', None],
  [ 0x6805, 'CjkUnifiedIdeograph6805', [], 'Display', None],
  [ 0x6806, 'CjkUnifiedIdeograph6806', [], 'Display', None],
  [ 0x6807, 'CjkUnifiedIdeograph6807', [], 'Display', None],
  [ 0x6808, 'CjkUnifiedIdeograph6808', [], 'Display', None],
  [ 0x6809, 'CjkUnifiedIdeograph6809', [], 'Display', None],
  [ 0x680a, 'CjkUnifiedIdeograph680a', [], 'Display', None],
  [ 0x680b, 'CjkUnifiedIdeograph680b', [], 'Display', None],
  [ 0x680c, 'CjkUnifiedIdeograph680c', [], 'Display', None],
  [ 0x680d, 'CjkUnifiedIdeograph680d', [], 'Display', None],
  [ 0x680e, 'CjkUnifiedIdeograph680e', [], 'Display', None],
  [ 0x680f, 'CjkUnifiedIdeograph680f', [], 'Display', None],
  [ 0x6810, 'CjkUnifiedIdeograph6810', [], 'Display', None],
  [ 0x6811, 'CjkUnifiedIdeograph6811', [], 'Display', None],
  [ 0x6812, 'CjkUnifiedIdeograph6812', [], 'Display', None],
  [ 0x6813, 'CjkUnifiedIdeograph6813', [], 'Display', None],
  [ 0x6814, 'CjkUnifiedIdeograph6814', [], 'Display', None],
  [ 0x6815, 'CjkUnifiedIdeograph6815', [], 'Display', None],
  [ 0x6816, 'CjkUnifiedIdeograph6816', [], 'Display', None],
  [ 0x6817, 'CjkUnifiedIdeograph6817', [], 'Display', None],
  [ 0x6818, 'CjkUnifiedIdeograph6818', [], 'Display', None],
  [ 0x6819, 'CjkUnifiedIdeograph6819', [], 'Display', None],
  [ 0x681a, 'CjkUnifiedIdeograph681a', [], 'Display', None],
  [ 0x681b, 'CjkUnifiedIdeograph681b', [], 'Display', None],
  [ 0x681c, 'CjkUnifiedIdeograph681c', [], 'Display', None],
  [ 0x681d, 'CjkUnifiedIdeograph681d', [], 'Display', None],
  [ 0x681e, 'CjkUnifiedIdeograph681e', [], 'Display', None],
  [ 0x681f, 'CjkUnifiedIdeograph681f', [], 'Display', None],
  [ 0x6820, 'CjkUnifiedIdeograph6820', [], 'Display', None],
  [ 0x6821, 'CjkUnifiedIdeograph6821', [], 'Display', None],
  [ 0x6822, 'CjkUnifiedIdeograph6822', [], 'Display', None],
  [ 0x6823, 'CjkUnifiedIdeograph6823', [], 'Display', None],
  [ 0x6824, 'CjkUnifiedIdeograph6824', [], 'Display', None],
  [ 0x6825, 'CjkUnifiedIdeograph6825', [], 'Display', None],
  [ 0x6826, 'CjkUnifiedIdeograph6826', [], 'Display', None],
  [ 0x6827, 'CjkUnifiedIdeograph6827', [], 'Display', None],
  [ 0x6828, 'CjkUnifiedIdeograph6828', [], 'Display', None],
  [ 0x6829, 'CjkUnifiedIdeograph6829', [], 'Display', None],
  [ 0x682a, 'CjkUnifiedIdeograph682a', [], 'Display', None],
  [ 0x682b, 'CjkUnifiedIdeograph682b', [], 'Display', None],
  [ 0x682c, 'CjkUnifiedIdeograph682c', [], 'Display', None],
  [ 0x682d, 'CjkUnifiedIdeograph682d', [], 'Display', None],
  [ 0x682e, 'CjkUnifiedIdeograph682e', [], 'Display', None],
  [ 0x682f, 'CjkUnifiedIdeograph682f', [], 'Display', None],
  [ 0x6830, 'CjkUnifiedIdeograph6830', [], 'Display', None],
  [ 0x6831, 'CjkUnifiedIdeograph6831', [], 'Display', None],
  [ 0x6832, 'CjkUnifiedIdeograph6832', [], 'Display', None],
  [ 0x6833, 'CjkUnifiedIdeograph6833', [], 'Display', None],
  [ 0x6834, 'CjkUnifiedIdeograph6834', [], 'Display', None],
  [ 0x6835, 'CjkUnifiedIdeograph6835', [], 'Display', None],
  [ 0x6836, 'CjkUnifiedIdeograph6836', [], 'Display', None],
  [ 0x6837, 'CjkUnifiedIdeograph6837', [], 'Display', None],
  [ 0x6838, 'CjkUnifiedIdeograph6838', [], 'Display', None],
  [ 0x6839, 'CjkUnifiedIdeograph6839', [], 'Display', None],
  [ 0x683a, 'CjkUnifiedIdeograph683a', [], 'Display', None],
  [ 0x683b, 'CjkUnifiedIdeograph683b', [], 'Display', None],
  [ 0x683c, 'CjkUnifiedIdeograph683c', [], 'Display', None],
  [ 0x683d, 'CjkUnifiedIdeograph683d', [], 'Display', None],
  [ 0x683e, 'CjkUnifiedIdeograph683e', [], 'Display', None],
  [ 0x683f, 'CjkUnifiedIdeograph683f', [], 'Display', None],
  [ 0x6840, 'CjkUnifiedIdeograph6840', [], 'Display', None],
  [ 0x6841, 'CjkUnifiedIdeograph6841', [], 'Display', None],
  [ 0x6842, 'CjkUnifiedIdeograph6842', [], 'Display', None],
  [ 0x6843, 'CjkUnifiedIdeograph6843', [], 'Display', None],
  [ 0x6844, 'CjkUnifiedIdeograph6844', [], 'Display', None],
  [ 0x6845, 'CjkUnifiedIdeograph6845', [], 'Display', None],
  [ 0x6846, 'CjkUnifiedIdeograph6846', [], 'Display', None],
  [ 0x6847, 'CjkUnifiedIdeograph6847', [], 'Display', None],
  [ 0x6848, 'CjkUnifiedIdeograph6848', [], 'Display', None],
  [ 0x6849, 'CjkUnifiedIdeograph6849', [], 'Display', None],
  [ 0x684a, 'CjkUnifiedIdeograph684a', [], 'Display', None],
  [ 0x684b, 'CjkUnifiedIdeograph684b', [], 'Display', None],
  [ 0x684c, 'CjkUnifiedIdeograph684c', [], 'Display', None],
  [ 0x684d, 'CjkUnifiedIdeograph684d', [], 'Display', None],
  [ 0x684e, 'CjkUnifiedIdeograph684e', [], 'Display', None],
  [ 0x684f, 'CjkUnifiedIdeograph684f', [], 'Display', None],
  [ 0x6850, 'CjkUnifiedIdeograph6850', [], 'Display', None],
  [ 0x6851, 'CjkUnifiedIdeograph6851', [], 'Display', None],
  [ 0x6852, 'CjkUnifiedIdeograph6852', [], 'Display', None],
  [ 0x6853, 'CjkUnifiedIdeograph6853', [], 'Display', None],
  [ 0x6854, 'CjkUnifiedIdeograph6854', [], 'Display', None],
  [ 0x6855, 'CjkUnifiedIdeograph6855', [], 'Display', None],
  [ 0x6856, 'CjkUnifiedIdeograph6856', [], 'Display', None],
  [ 0x6857, 'CjkUnifiedIdeograph6857', [], 'Display', None],
  [ 0x6858, 'CjkUnifiedIdeograph6858', [], 'Display', None],
  [ 0x6859, 'CjkUnifiedIdeograph6859', [], 'Display', None],
  [ 0x685a, 'CjkUnifiedIdeograph685a', [], 'Display', None],
  [ 0x685b, 'CjkUnifiedIdeograph685b', [], 'Display', None],
  [ 0x685c, 'CjkUnifiedIdeograph685c', [], 'Display', None],
  [ 0x685d, 'CjkUnifiedIdeograph685d', [], 'Display', None],
  [ 0x685e, 'CjkUnifiedIdeograph685e', [], 'Display', None],
  [ 0x685f, 'CjkUnifiedIdeograph685f', [], 'Display', None],
  [ 0x6860, 'CjkUnifiedIdeograph6860', [], 'Display', None],
  [ 0x6861, 'CjkUnifiedIdeograph6861', [], 'Display', None],
  [ 0x6862, 'CjkUnifiedIdeograph6862', [], 'Display', None],
  [ 0x6863, 'CjkUnifiedIdeograph6863', [], 'Display', None],
  [ 0x6864, 'CjkUnifiedIdeograph6864', [], 'Display', None],
  [ 0x6865, 'CjkUnifiedIdeograph6865', [], 'Display', None],
  [ 0x6866, 'CjkUnifiedIdeograph6866', [], 'Display', None],
  [ 0x6867, 'CjkUnifiedIdeograph6867', [], 'Display', None],
  [ 0x6868, 'CjkUnifiedIdeograph6868', [], 'Display', None],
  [ 0x6869, 'CjkUnifiedIdeograph6869', [], 'Display', None],
  [ 0x686a, 'CjkUnifiedIdeograph686a', [], 'Display', None],
  [ 0x686b, 'CjkUnifiedIdeograph686b', [], 'Display', None],
  [ 0x686c, 'CjkUnifiedIdeograph686c', [], 'Display', None],
  [ 0x686d, 'CjkUnifiedIdeograph686d', [], 'Display', None],
  [ 0x686e, 'CjkUnifiedIdeograph686e', [], 'Display', None],
  [ 0x686f, 'CjkUnifiedIdeograph686f', [], 'Display', None],
  [ 0x6870, 'CjkUnifiedIdeograph6870', [], 'Display', None],
  [ 0x6871, 'CjkUnifiedIdeograph6871', [], 'Display', None],
  [ 0x6872, 'CjkUnifiedIdeograph6872', [], 'Display', None],
  [ 0x6873, 'CjkUnifiedIdeograph6873', [], 'Display', None],
  [ 0x6874, 'CjkUnifiedIdeograph6874', [], 'Display', None],
  [ 0x6875, 'CjkUnifiedIdeograph6875', [], 'Display', None],
  [ 0x6876, 'CjkUnifiedIdeograph6876', [], 'Display', None],
  [ 0x6877, 'CjkUnifiedIdeograph6877', [], 'Display', None],
  [ 0x6878, 'CjkUnifiedIdeograph6878', [], 'Display', None],
  [ 0x6879, 'CjkUnifiedIdeograph6879', [], 'Display', None],
  [ 0x687a, 'CjkUnifiedIdeograph687a', [], 'Display', None],
  [ 0x687b, 'CjkUnifiedIdeograph687b', [], 'Display', None],
  [ 0x687c, 'CjkUnifiedIdeograph687c', [], 'Display', None],
  [ 0x687d, 'CjkUnifiedIdeograph687d', [], 'Display', None],
  [ 0x687e, 'CjkUnifiedIdeograph687e', [], 'Display', None],
  [ 0x687f, 'CjkUnifiedIdeograph687f', [], 'Display', None],
  [ 0x6880, 'CjkUnifiedIdeograph6880', [], 'Display', None],
  [ 0x6881, 'CjkUnifiedIdeograph6881', [], 'Display', None],
  [ 0x6882, 'CjkUnifiedIdeograph6882', [], 'Display', None],
  [ 0x6883, 'CjkUnifiedIdeograph6883', [], 'Display', None],
  [ 0x6884, 'CjkUnifiedIdeograph6884', [], 'Display', None],
  [ 0x6885, 'CjkUnifiedIdeograph6885', [], 'Display', None],
  [ 0x6886, 'CjkUnifiedIdeograph6886', [], 'Display', None],
  [ 0x6887, 'CjkUnifiedIdeograph6887', [], 'Display', None],
  [ 0x6888, 'CjkUnifiedIdeograph6888', [], 'Display', None],
  [ 0x6889, 'CjkUnifiedIdeograph6889', [], 'Display', None],
  [ 0x688a, 'CjkUnifiedIdeograph688a', [], 'Display', None],
  [ 0x688b, 'CjkUnifiedIdeograph688b', [], 'Display', None],
  [ 0x688c, 'CjkUnifiedIdeograph688c', [], 'Display', None],
  [ 0x688d, 'CjkUnifiedIdeograph688d', [], 'Display', None],
  [ 0x688e, 'CjkUnifiedIdeograph688e', [], 'Display', None],
  [ 0x688f, 'CjkUnifiedIdeograph688f', [], 'Display', None],
  [ 0x6890, 'CjkUnifiedIdeograph6890', [], 'Display', None],
  [ 0x6891, 'CjkUnifiedIdeograph6891', [], 'Display', None],
  [ 0x6892, 'CjkUnifiedIdeograph6892', [], 'Display', None],
  [ 0x6893, 'CjkUnifiedIdeograph6893', [], 'Display', None],
  [ 0x6894, 'CjkUnifiedIdeograph6894', [], 'Display', None],
  [ 0x6895, 'CjkUnifiedIdeograph6895', [], 'Display', None],
  [ 0x6896, 'CjkUnifiedIdeograph6896', [], 'Display', None],
  [ 0x6897, 'CjkUnifiedIdeograph6897', [], 'Display', None],
  [ 0x6898, 'CjkUnifiedIdeograph6898', [], 'Display', None],
  [ 0x6899, 'CjkUnifiedIdeograph6899', [], 'Display', None],
  [ 0x689a, 'CjkUnifiedIdeograph689a', [], 'Display', None],
  [ 0x689b, 'CjkUnifiedIdeograph689b', [], 'Display', None],
  [ 0x689c, 'CjkUnifiedIdeograph689c', [], 'Display', None],
  [ 0x689d, 'CjkUnifiedIdeograph689d', [], 'Display', None],
  [ 0x689e, 'CjkUnifiedIdeograph689e', [], 'Display', None],
  [ 0x689f, 'CjkUnifiedIdeograph689f', [], 'Display', None],
  [ 0x68a0, 'CjkUnifiedIdeograph68a0', [], 'Display', None],
  [ 0x68a1, 'CjkUnifiedIdeograph68a1', [], 'Display', None],
  [ 0x68a2, 'CjkUnifiedIdeograph68a2', [], 'Display', None],
  [ 0x68a3, 'CjkUnifiedIdeograph68a3', [], 'Display', None],
  [ 0x68a4, 'CjkUnifiedIdeograph68a4', [], 'Display', None],
  [ 0x68a5, 'CjkUnifiedIdeograph68a5', [], 'Display', None],
  [ 0x68a6, 'CjkUnifiedIdeograph68a6', [], 'Display', None],
  [ 0x68a7, 'CjkUnifiedIdeograph68a7', [], 'Display', None],
  [ 0x68a8, 'CjkUnifiedIdeograph68a8', [], 'Display', None],
  [ 0x68a9, 'CjkUnifiedIdeograph68a9', [], 'Display', None],
  [ 0x68aa, 'CjkUnifiedIdeograph68aa', [], 'Display', None],
  [ 0x68ab, 'CjkUnifiedIdeograph68ab', [], 'Display', None],
  [ 0x68ac, 'CjkUnifiedIdeograph68ac', [], 'Display', None],
  [ 0x68ad, 'CjkUnifiedIdeograph68ad', [], 'Display', None],
  [ 0x68ae, 'CjkUnifiedIdeograph68ae', [], 'Display', None],
  [ 0x68af, 'CjkUnifiedIdeograph68af', [], 'Display', None],
  [ 0x68b0, 'CjkUnifiedIdeograph68b0', [], 'Display', None],
  [ 0x68b1, 'CjkUnifiedIdeograph68b1', [], 'Display', None],
  [ 0x68b2, 'CjkUnifiedIdeograph68b2', [], 'Display', None],
  [ 0x68b3, 'CjkUnifiedIdeograph68b3', [], 'Display', None],
  [ 0x68b4, 'CjkUnifiedIdeograph68b4', [], 'Display', None],
  [ 0x68b5, 'CjkUnifiedIdeograph68b5', [], 'Display', None],
  [ 0x68b6, 'CjkUnifiedIdeograph68b6', [], 'Display', None],
  [ 0x68b7, 'CjkUnifiedIdeograph68b7', [], 'Display', None],
  [ 0x68b8, 'CjkUnifiedIdeograph68b8', [], 'Display', None],
  [ 0x68b9, 'CjkUnifiedIdeograph68b9', [], 'Display', None],
  [ 0x68ba, 'CjkUnifiedIdeograph68ba', [], 'Display', None],
  [ 0x68bb, 'CjkUnifiedIdeograph68bb', [], 'Display', None],
  [ 0x68bc, 'CjkUnifiedIdeograph68bc', [], 'Display', None],
  [ 0x68bd, 'CjkUnifiedIdeograph68bd', [], 'Display', None],
  [ 0x68be, 'CjkUnifiedIdeograph68be', [], 'Display', None],
  [ 0x68bf, 'CjkUnifiedIdeograph68bf', [], 'Display', None],
  [ 0x68c0, 'CjkUnifiedIdeograph68c0', [], 'Display', None],
  [ 0x68c1, 'CjkUnifiedIdeograph68c1', [], 'Display', None],
  [ 0x68c2, 'CjkUnifiedIdeograph68c2', [], 'Display', None],
  [ 0x68c3, 'CjkUnifiedIdeograph68c3', [], 'Display', None],
  [ 0x68c4, 'CjkUnifiedIdeograph68c4', [], 'Display', None],
  [ 0x68c5, 'CjkUnifiedIdeograph68c5', [], 'Display', None],
  [ 0x68c6, 'CjkUnifiedIdeograph68c6', [], 'Display', None],
  [ 0x68c7, 'CjkUnifiedIdeograph68c7', [], 'Display', None],
  [ 0x68c8, 'CjkUnifiedIdeograph68c8', [], 'Display', None],
  [ 0x68c9, 'CjkUnifiedIdeograph68c9', [], 'Display', None],
  [ 0x68ca, 'CjkUnifiedIdeograph68ca', [], 'Display', None],
  [ 0x68cb, 'CjkUnifiedIdeograph68cb', [], 'Display', None],
  [ 0x68cc, 'CjkUnifiedIdeograph68cc', [], 'Display', None],
  [ 0x68cd, 'CjkUnifiedIdeograph68cd', [], 'Display', None],
  [ 0x68ce, 'CjkUnifiedIdeograph68ce', [], 'Display', None],
  [ 0x68cf, 'CjkUnifiedIdeograph68cf', [], 'Display', None],
  [ 0x68d0, 'CjkUnifiedIdeograph68d0', [], 'Display', None],
  [ 0x68d1, 'CjkUnifiedIdeograph68d1', [], 'Display', None],
  [ 0x68d2, 'CjkUnifiedIdeograph68d2', [], 'Display', None],
  [ 0x68d3, 'CjkUnifiedIdeograph68d3', [], 'Display', None],
  [ 0x68d4, 'CjkUnifiedIdeograph68d4', [], 'Display', None],
  [ 0x68d5, 'CjkUnifiedIdeograph68d5', [], 'Display', None],
  [ 0x68d6, 'CjkUnifiedIdeograph68d6', [], 'Display', None],
  [ 0x68d7, 'CjkUnifiedIdeograph68d7', [], 'Display', None],
  [ 0x68d8, 'CjkUnifiedIdeograph68d8', [], 'Display', None],
  [ 0x68d9, 'CjkUnifiedIdeograph68d9', [], 'Display', None],
  [ 0x68da, 'CjkUnifiedIdeograph68da', [], 'Display', None],
  [ 0x68db, 'CjkUnifiedIdeograph68db', [], 'Display', None],
  [ 0x68dc, 'CjkUnifiedIdeograph68dc', [], 'Display', None],
  [ 0x68dd, 'CjkUnifiedIdeograph68dd', [], 'Display', None],
  [ 0x68de, 'CjkUnifiedIdeograph68de', [], 'Display', None],
  [ 0x68df, 'CjkUnifiedIdeograph68df', [], 'Display', None],
  [ 0x68e0, 'CjkUnifiedIdeograph68e0', [], 'Display', None],
  [ 0x68e1, 'CjkUnifiedIdeograph68e1', [], 'Display', None],
  [ 0x68e2, 'CjkUnifiedIdeograph68e2', [], 'Display', None],
  [ 0x68e3, 'CjkUnifiedIdeograph68e3', [], 'Display', None],
  [ 0x68e4, 'CjkUnifiedIdeograph68e4', [], 'Display', None],
  [ 0x68e5, 'CjkUnifiedIdeograph68e5', [], 'Display', None],
  [ 0x68e6, 'CjkUnifiedIdeograph68e6', [], 'Display', None],
  [ 0x68e7, 'CjkUnifiedIdeograph68e7', [], 'Display', None],
  [ 0x68e8, 'CjkUnifiedIdeograph68e8', [], 'Display', None],
  [ 0x68e9, 'CjkUnifiedIdeograph68e9', [], 'Display', None],
  [ 0x68ea, 'CjkUnifiedIdeograph68ea', [], 'Display', None],
  [ 0x68eb, 'CjkUnifiedIdeograph68eb', [], 'Display', None],
  [ 0x68ec, 'CjkUnifiedIdeograph68ec', [], 'Display', None],
  [ 0x68ed, 'CjkUnifiedIdeograph68ed', [], 'Display', None],
  [ 0x68ee, 'CjkUnifiedIdeograph68ee', [], 'Display', None],
  [ 0x68ef, 'CjkUnifiedIdeograph68ef', [], 'Display', None],
  [ 0x68f0, 'CjkUnifiedIdeograph68f0', [], 'Display', None],
  [ 0x68f1, 'CjkUnifiedIdeograph68f1', [], 'Display', None],
  [ 0x68f2, 'CjkUnifiedIdeograph68f2', [], 'Display', None],
  [ 0x68f3, 'CjkUnifiedIdeograph68f3', [], 'Display', None],
  [ 0x68f4, 'CjkUnifiedIdeograph68f4', [], 'Display', None],
  [ 0x68f5, 'CjkUnifiedIdeograph68f5', [], 'Display', None],
  [ 0x68f6, 'CjkUnifiedIdeograph68f6', [], 'Display', None],
  [ 0x68f7, 'CjkUnifiedIdeograph68f7', [], 'Display', None],
  [ 0x68f8, 'CjkUnifiedIdeograph68f8', [], 'Display', None],
  [ 0x68f9, 'CjkUnifiedIdeograph68f9', [], 'Display', None],
  [ 0x68fa, 'CjkUnifiedIdeograph68fa', [], 'Display', None],
  [ 0x68fb, 'CjkUnifiedIdeograph68fb', [], 'Display', None],
  [ 0x68fc, 'CjkUnifiedIdeograph68fc', [], 'Display', None],
  [ 0x68fd, 'CjkUnifiedIdeograph68fd', [], 'Display', None],
  [ 0x68fe, 'CjkUnifiedIdeograph68fe', [], 'Display', None],
  [ 0x68ff, 'CjkUnifiedIdeograph68ff', [], 'Display', None],
  [ 0x6900, 'CjkUnifiedIdeograph6900', [], 'Display', None],
  [ 0x6901, 'CjkUnifiedIdeograph6901', [], 'Display', None],
  [ 0x6902, 'CjkUnifiedIdeograph6902', [], 'Display', None],
  [ 0x6903, 'CjkUnifiedIdeograph6903', [], 'Display', None],
  [ 0x6904, 'CjkUnifiedIdeograph6904', [], 'Display', None],
  [ 0x6905, 'CjkUnifiedIdeograph6905', [], 'Display', None],
  [ 0x6906, 'CjkUnifiedIdeograph6906', [], 'Display', None],
  [ 0x6907, 'CjkUnifiedIdeograph6907', [], 'Display', None],
  [ 0x6908, 'CjkUnifiedIdeograph6908', [], 'Display', None],
  [ 0x6909, 'CjkUnifiedIdeograph6909', [], 'Display', None],
  [ 0x690a, 'CjkUnifiedIdeograph690a', [], 'Display', None],
  [ 0x690b, 'CjkUnifiedIdeograph690b', [], 'Display', None],
  [ 0x690c, 'CjkUnifiedIdeograph690c', [], 'Display', None],
  [ 0x690d, 'CjkUnifiedIdeograph690d', [], 'Display', None],
  [ 0x690e, 'CjkUnifiedIdeograph690e', [], 'Display', None],
  [ 0x690f, 'CjkUnifiedIdeograph690f', [], 'Display', None],
  [ 0x6910, 'CjkUnifiedIdeograph6910', [], 'Display', None],
  [ 0x6911, 'CjkUnifiedIdeograph6911', [], 'Display', None],
  [ 0x6912, 'CjkUnifiedIdeograph6912', [], 'Display', None],
  [ 0x6913, 'CjkUnifiedIdeograph6913', [], 'Display', None],
  [ 0x6914, 'CjkUnifiedIdeograph6914', [], 'Display', None],
  [ 0x6915, 'CjkUnifiedIdeograph6915', [], 'Display', None],
  [ 0x6916, 'CjkUnifiedIdeograph6916', [], 'Display', None],
  [ 0x6917, 'CjkUnifiedIdeograph6917', [], 'Display', None],
  [ 0x6918, 'CjkUnifiedIdeograph6918', [], 'Display', None],
  [ 0x6919, 'CjkUnifiedIdeograph6919', [], 'Display', None],
  [ 0x691a, 'CjkUnifiedIdeograph691a', [], 'Display', None],
  [ 0x691b, 'CjkUnifiedIdeograph691b', [], 'Display', None],
  [ 0x691c, 'CjkUnifiedIdeograph691c', [], 'Display', None],
  [ 0x691d, 'CjkUnifiedIdeograph691d', [], 'Display', None],
  [ 0x691e, 'CjkUnifiedIdeograph691e', [], 'Display', None],
  [ 0x691f, 'CjkUnifiedIdeograph691f', [], 'Display', None],
  [ 0x6920, 'CjkUnifiedIdeograph6920', [], 'Display', None],
  [ 0x6921, 'CjkUnifiedIdeograph6921', [], 'Display', None],
  [ 0x6922, 'CjkUnifiedIdeograph6922', [], 'Display', None],
  [ 0x6923, 'CjkUnifiedIdeograph6923', [], 'Display', None],
  [ 0x6924, 'CjkUnifiedIdeograph6924', [], 'Display', None],
  [ 0x6925, 'CjkUnifiedIdeograph6925', [], 'Display', None],
  [ 0x6926, 'CjkUnifiedIdeograph6926', [], 'Display', None],
  [ 0x6927, 'CjkUnifiedIdeograph6927', [], 'Display', None],
  [ 0x6928, 'CjkUnifiedIdeograph6928', [], 'Display', None],
  [ 0x6929, 'CjkUnifiedIdeograph6929', [], 'Display', None],
  [ 0x692a, 'CjkUnifiedIdeograph692a', [], 'Display', None],
  [ 0x692b, 'CjkUnifiedIdeograph692b', [], 'Display', None],
  [ 0x692c, 'CjkUnifiedIdeograph692c', [], 'Display', None],
  [ 0x692d, 'CjkUnifiedIdeograph692d', [], 'Display', None],
  [ 0x692e, 'CjkUnifiedIdeograph692e', [], 'Display', None],
  [ 0x692f, 'CjkUnifiedIdeograph692f', [], 'Display', None],
  [ 0x6930, 'CjkUnifiedIdeograph6930', [], 'Display', None],
  [ 0x6931, 'CjkUnifiedIdeograph6931', [], 'Display', None],
  [ 0x6932, 'CjkUnifiedIdeograph6932', [], 'Display', None],
  [ 0x6933, 'CjkUnifiedIdeograph6933', [], 'Display', None],
  [ 0x6934, 'CjkUnifiedIdeograph6934', [], 'Display', None],
  [ 0x6935, 'CjkUnifiedIdeograph6935', [], 'Display', None],
  [ 0x6936, 'CjkUnifiedIdeograph6936', [], 'Display', None],
  [ 0x6937, 'CjkUnifiedIdeograph6937', [], 'Display', None],
  [ 0x6938, 'CjkUnifiedIdeograph6938', [], 'Display', None],
  [ 0x6939, 'CjkUnifiedIdeograph6939', [], 'Display', None],
  [ 0x693a, 'CjkUnifiedIdeograph693a', [], 'Display', None],
  [ 0x693b, 'CjkUnifiedIdeograph693b', [], 'Display', None],
  [ 0x693c, 'CjkUnifiedIdeograph693c', [], 'Display', None],
  [ 0x693d, 'CjkUnifiedIdeograph693d', [], 'Display', None],
  [ 0x693e, 'CjkUnifiedIdeograph693e', [], 'Display', None],
  [ 0x693f, 'CjkUnifiedIdeograph693f', [], 'Display', None],
  [ 0x6940, 'CjkUnifiedIdeograph6940', [], 'Display', None],
  [ 0x6941, 'CjkUnifiedIdeograph6941', [], 'Display', None],
  [ 0x6942, 'CjkUnifiedIdeograph6942', [], 'Display', None],
  [ 0x6943, 'CjkUnifiedIdeograph6943', [], 'Display', None],
  [ 0x6944, 'CjkUnifiedIdeograph6944', [], 'Display', None],
  [ 0x6945, 'CjkUnifiedIdeograph6945', [], 'Display', None],
  [ 0x6946, 'CjkUnifiedIdeograph6946', [], 'Display', None],
  [ 0x6947, 'CjkUnifiedIdeograph6947', [], 'Display', None],
  [ 0x6948, 'CjkUnifiedIdeograph6948', [], 'Display', None],
  [ 0x6949, 'CjkUnifiedIdeograph6949', [], 'Display', None],
  [ 0x694a, 'CjkUnifiedIdeograph694a', [], 'Display', None],
  [ 0x694b, 'CjkUnifiedIdeograph694b', [], 'Display', None],
  [ 0x694c, 'CjkUnifiedIdeograph694c', [], 'Display', None],
  [ 0x694d, 'CjkUnifiedIdeograph694d', [], 'Display', None],
  [ 0x694e, 'CjkUnifiedIdeograph694e', [], 'Display', None],
  [ 0x694f, 'CjkUnifiedIdeograph694f', [], 'Display', None],
  [ 0x6950, 'CjkUnifiedIdeograph6950', [], 'Display', None],
  [ 0x6951, 'CjkUnifiedIdeograph6951', [], 'Display', None],
  [ 0x6952, 'CjkUnifiedIdeograph6952', [], 'Display', None],
  [ 0x6953, 'CjkUnifiedIdeograph6953', [], 'Display', None],
  [ 0x6954, 'CjkUnifiedIdeograph6954', [], 'Display', None],
  [ 0x6955, 'CjkUnifiedIdeograph6955', [], 'Display', None],
  [ 0x6956, 'CjkUnifiedIdeograph6956', [], 'Display', None],
  [ 0x6957, 'CjkUnifiedIdeograph6957', [], 'Display', None],
  [ 0x6958, 'CjkUnifiedIdeograph6958', [], 'Display', None],
  [ 0x6959, 'CjkUnifiedIdeograph6959', [], 'Display', None],
  [ 0x695a, 'CjkUnifiedIdeograph695a', [], 'Display', None],
  [ 0x695b, 'CjkUnifiedIdeograph695b', [], 'Display', None],
  [ 0x695c, 'CjkUnifiedIdeograph695c', [], 'Display', None],
  [ 0x695d, 'CjkUnifiedIdeograph695d', [], 'Display', None],
  [ 0x695e, 'CjkUnifiedIdeograph695e', [], 'Display', None],
  [ 0x695f, 'CjkUnifiedIdeograph695f', [], 'Display', None],
  [ 0x6960, 'CjkUnifiedIdeograph6960', [], 'Display', None],
  [ 0x6961, 'CjkUnifiedIdeograph6961', [], 'Display', None],
  [ 0x6962, 'CjkUnifiedIdeograph6962', [], 'Display', None],
  [ 0x6963, 'CjkUnifiedIdeograph6963', [], 'Display', None],
  [ 0x6964, 'CjkUnifiedIdeograph6964', [], 'Display', None],
  [ 0x6965, 'CjkUnifiedIdeograph6965', [], 'Display', None],
  [ 0x6966, 'CjkUnifiedIdeograph6966', [], 'Display', None],
  [ 0x6967, 'CjkUnifiedIdeograph6967', [], 'Display', None],
  [ 0x6968, 'CjkUnifiedIdeograph6968', [], 'Display', None],
  [ 0x6969, 'CjkUnifiedIdeograph6969', [], 'Display', None],
  [ 0x696a, 'CjkUnifiedIdeograph696a', [], 'Display', None],
  [ 0x696b, 'CjkUnifiedIdeograph696b', [], 'Display', None],
  [ 0x696c, 'CjkUnifiedIdeograph696c', [], 'Display', None],
  [ 0x696d, 'CjkUnifiedIdeograph696d', [], 'Display', None],
  [ 0x696e, 'CjkUnifiedIdeograph696e', [], 'Display', None],
  [ 0x696f, 'CjkUnifiedIdeograph696f', [], 'Display', None],
  [ 0x6970, 'CjkUnifiedIdeograph6970', [], 'Display', None],
  [ 0x6971, 'CjkUnifiedIdeograph6971', [], 'Display', None],
  [ 0x6972, 'CjkUnifiedIdeograph6972', [], 'Display', None],
  [ 0x6973, 'CjkUnifiedIdeograph6973', [], 'Display', None],
  [ 0x6974, 'CjkUnifiedIdeograph6974', [], 'Display', None],
  [ 0x6975, 'CjkUnifiedIdeograph6975', [], 'Display', None],
  [ 0x6976, 'CjkUnifiedIdeograph6976', [], 'Display', None],
  [ 0x6977, 'CjkUnifiedIdeograph6977', [], 'Display', None],
  [ 0x6978, 'CjkUnifiedIdeograph6978', [], 'Display', None],
  [ 0x6979, 'CjkUnifiedIdeograph6979', [], 'Display', None],
  [ 0x697a, 'CjkUnifiedIdeograph697a', [], 'Display', None],
  [ 0x697b, 'CjkUnifiedIdeograph697b', [], 'Display', None],
  [ 0x697c, 'CjkUnifiedIdeograph697c', [], 'Display', None],
  [ 0x697d, 'CjkUnifiedIdeograph697d', [], 'Display', None],
  [ 0x697e, 'CjkUnifiedIdeograph697e', [], 'Display', None],
  [ 0x697f, 'CjkUnifiedIdeograph697f', [], 'Display', None],
  [ 0x6980, 'CjkUnifiedIdeograph6980', [], 'Display', None],
  [ 0x6981, 'CjkUnifiedIdeograph6981', [], 'Display', None],
  [ 0x6982, 'CjkUnifiedIdeograph6982', [], 'Display', None],
  [ 0x6983, 'CjkUnifiedIdeograph6983', [], 'Display', None],
  [ 0x6984, 'CjkUnifiedIdeograph6984', [], 'Display', None],
  [ 0x6985, 'CjkUnifiedIdeograph6985', [], 'Display', None],
  [ 0x6986, 'CjkUnifiedIdeograph6986', [], 'Display', None],
  [ 0x6987, 'CjkUnifiedIdeograph6987', [], 'Display', None],
  [ 0x6988, 'CjkUnifiedIdeograph6988', [], 'Display', None],
  [ 0x6989, 'CjkUnifiedIdeograph6989', [], 'Display', None],
  [ 0x698a, 'CjkUnifiedIdeograph698a', [], 'Display', None],
  [ 0x698b, 'CjkUnifiedIdeograph698b', [], 'Display', None],
  [ 0x698c, 'CjkUnifiedIdeograph698c', [], 'Display', None],
  [ 0x698d, 'CjkUnifiedIdeograph698d', [], 'Display', None],
  [ 0x698e, 'CjkUnifiedIdeograph698e', [], 'Display', None],
  [ 0x698f, 'CjkUnifiedIdeograph698f', [], 'Display', None],
  [ 0x6990, 'CjkUnifiedIdeograph6990', [], 'Display', None],
  [ 0x6991, 'CjkUnifiedIdeograph6991', [], 'Display', None],
  [ 0x6992, 'CjkUnifiedIdeograph6992', [], 'Display', None],
  [ 0x6993, 'CjkUnifiedIdeograph6993', [], 'Display', None],
  [ 0x6994, 'CjkUnifiedIdeograph6994', [], 'Display', None],
  [ 0x6995, 'CjkUnifiedIdeograph6995', [], 'Display', None],
  [ 0x6996, 'CjkUnifiedIdeograph6996', [], 'Display', None],
  [ 0x6997, 'CjkUnifiedIdeograph6997', [], 'Display', None],
  [ 0x6998, 'CjkUnifiedIdeograph6998', [], 'Display', None],
  [ 0x6999, 'CjkUnifiedIdeograph6999', [], 'Display', None],
  [ 0x699a, 'CjkUnifiedIdeograph699a', [], 'Display', None],
  [ 0x699b, 'CjkUnifiedIdeograph699b', [], 'Display', None],
  [ 0x699c, 'CjkUnifiedIdeograph699c', [], 'Display', None],
  [ 0x699d, 'CjkUnifiedIdeograph699d', [], 'Display', None],
  [ 0x699e, 'CjkUnifiedIdeograph699e', [], 'Display', None],
  [ 0x699f, 'CjkUnifiedIdeograph699f', [], 'Display', None],
  [ 0x69a0, 'CjkUnifiedIdeograph69a0', [], 'Display', None],
  [ 0x69a1, 'CjkUnifiedIdeograph69a1', [], 'Display', None],
  [ 0x69a2, 'CjkUnifiedIdeograph69a2', [], 'Display', None],
  [ 0x69a3, 'CjkUnifiedIdeograph69a3', [], 'Display', None],
  [ 0x69a4, 'CjkUnifiedIdeograph69a4', [], 'Display', None],
  [ 0x69a5, 'CjkUnifiedIdeograph69a5', [], 'Display', None],
  [ 0x69a6, 'CjkUnifiedIdeograph69a6', [], 'Display', None],
  [ 0x69a7, 'CjkUnifiedIdeograph69a7', [], 'Display', None],
  [ 0x69a8, 'CjkUnifiedIdeograph69a8', [], 'Display', None],
  [ 0x69a9, 'CjkUnifiedIdeograph69a9', [], 'Display', None],
  [ 0x69aa, 'CjkUnifiedIdeograph69aa', [], 'Display', None],
  [ 0x69ab, 'CjkUnifiedIdeograph69ab', [], 'Display', None],
  [ 0x69ac, 'CjkUnifiedIdeograph69ac', [], 'Display', None],
  [ 0x69ad, 'CjkUnifiedIdeograph69ad', [], 'Display', None],
  [ 0x69ae, 'CjkUnifiedIdeograph69ae', [], 'Display', None],
  [ 0x69af, 'CjkUnifiedIdeograph69af', [], 'Display', None],
  [ 0x69b0, 'CjkUnifiedIdeograph69b0', [], 'Display', None],
  [ 0x69b1, 'CjkUnifiedIdeograph69b1', [], 'Display', None],
  [ 0x69b2, 'CjkUnifiedIdeograph69b2', [], 'Display', None],
  [ 0x69b3, 'CjkUnifiedIdeograph69b3', [], 'Display', None],
  [ 0x69b4, 'CjkUnifiedIdeograph69b4', [], 'Display', None],
  [ 0x69b5, 'CjkUnifiedIdeograph69b5', [], 'Display', None],
  [ 0x69b6, 'CjkUnifiedIdeograph69b6', [], 'Display', None],
  [ 0x69b7, 'CjkUnifiedIdeograph69b7', [], 'Display', None],
  [ 0x69b8, 'CjkUnifiedIdeograph69b8', [], 'Display', None],
  [ 0x69b9, 'CjkUnifiedIdeograph69b9', [], 'Display', None],
  [ 0x69ba, 'CjkUnifiedIdeograph69ba', [], 'Display', None],
  [ 0x69bb, 'CjkUnifiedIdeograph69bb', [], 'Display', None],
  [ 0x69bc, 'CjkUnifiedIdeograph69bc', [], 'Display', None],
  [ 0x69bd, 'CjkUnifiedIdeograph69bd', [], 'Display', None],
  [ 0x69be, 'CjkUnifiedIdeograph69be', [], 'Display', None],
  [ 0x69bf, 'CjkUnifiedIdeograph69bf', [], 'Display', None],
  [ 0x69c0, 'CjkUnifiedIdeograph69c0', [], 'Display', None],
  [ 0x69c1, 'CjkUnifiedIdeograph69c1', [], 'Display', None],
  [ 0x69c2, 'CjkUnifiedIdeograph69c2', [], 'Display', None],
  [ 0x69c3, 'CjkUnifiedIdeograph69c3', [], 'Display', None],
  [ 0x69c4, 'CjkUnifiedIdeograph69c4', [], 'Display', None],
  [ 0x69c5, 'CjkUnifiedIdeograph69c5', [], 'Display', None],
  [ 0x69c6, 'CjkUnifiedIdeograph69c6', [], 'Display', None],
  [ 0x69c7, 'CjkUnifiedIdeograph69c7', [], 'Display', None],
  [ 0x69c8, 'CjkUnifiedIdeograph69c8', [], 'Display', None],
  [ 0x69c9, 'CjkUnifiedIdeograph69c9', [], 'Display', None],
  [ 0x69ca, 'CjkUnifiedIdeograph69ca', [], 'Display', None],
  [ 0x69cb, 'CjkUnifiedIdeograph69cb', [], 'Display', None],
  [ 0x69cc, 'CjkUnifiedIdeograph69cc', [], 'Display', None],
  [ 0x69cd, 'CjkUnifiedIdeograph69cd', [], 'Display', None],
  [ 0x69ce, 'CjkUnifiedIdeograph69ce', [], 'Display', None],
  [ 0x69cf, 'CjkUnifiedIdeograph69cf', [], 'Display', None],
  [ 0x69d0, 'CjkUnifiedIdeograph69d0', [], 'Display', None],
  [ 0x69d1, 'CjkUnifiedIdeograph69d1', [], 'Display', None],
  [ 0x69d2, 'CjkUnifiedIdeograph69d2', [], 'Display', None],
  [ 0x69d3, 'CjkUnifiedIdeograph69d3', [], 'Display', None],
  [ 0x69d4, 'CjkUnifiedIdeograph69d4', [], 'Display', None],
  [ 0x69d5, 'CjkUnifiedIdeograph69d5', [], 'Display', None],
  [ 0x69d6, 'CjkUnifiedIdeograph69d6', [], 'Display', None],
  [ 0x69d7, 'CjkUnifiedIdeograph69d7', [], 'Display', None],
  [ 0x69d8, 'CjkUnifiedIdeograph69d8', [], 'Display', None],
  [ 0x69d9, 'CjkUnifiedIdeograph69d9', [], 'Display', None],
  [ 0x69da, 'CjkUnifiedIdeograph69da', [], 'Display', None],
  [ 0x69db, 'CjkUnifiedIdeograph69db', [], 'Display', None],
  [ 0x69dc, 'CjkUnifiedIdeograph69dc', [], 'Display', None],
  [ 0x69dd, 'CjkUnifiedIdeograph69dd', [], 'Display', None],
  [ 0x69de, 'CjkUnifiedIdeograph69de', [], 'Display', None],
  [ 0x69df, 'CjkUnifiedIdeograph69df', [], 'Display', None],
  [ 0x69e0, 'CjkUnifiedIdeograph69e0', [], 'Display', None],
  [ 0x69e1, 'CjkUnifiedIdeograph69e1', [], 'Display', None],
  [ 0x69e2, 'CjkUnifiedIdeograph69e2', [], 'Display', None],
  [ 0x69e3, 'CjkUnifiedIdeograph69e3', [], 'Display', None],
  [ 0x69e4, 'CjkUnifiedIdeograph69e4', [], 'Display', None],
  [ 0x69e5, 'CjkUnifiedIdeograph69e5', [], 'Display', None],
  [ 0x69e6, 'CjkUnifiedIdeograph69e6', [], 'Display', None],
  [ 0x69e7, 'CjkUnifiedIdeograph69e7', [], 'Display', None],
  [ 0x69e8, 'CjkUnifiedIdeograph69e8', [], 'Display', None],
  [ 0x69e9, 'CjkUnifiedIdeograph69e9', [], 'Display', None],
  [ 0x69ea, 'CjkUnifiedIdeograph69ea', [], 'Display', None],
  [ 0x69eb, 'CjkUnifiedIdeograph69eb', [], 'Display', None],
  [ 0x69ec, 'CjkUnifiedIdeograph69ec', [], 'Display', None],
  [ 0x69ed, 'CjkUnifiedIdeograph69ed', [], 'Display', None],
  [ 0x69ee, 'CjkUnifiedIdeograph69ee', [], 'Display', None],
  [ 0x69ef, 'CjkUnifiedIdeograph69ef', [], 'Display', None],
  [ 0x69f0, 'CjkUnifiedIdeograph69f0', [], 'Display', None],
  [ 0x69f1, 'CjkUnifiedIdeograph69f1', [], 'Display', None],
  [ 0x69f2, 'CjkUnifiedIdeograph69f2', [], 'Display', None],
  [ 0x69f3, 'CjkUnifiedIdeograph69f3', [], 'Display', None],
  [ 0x69f4, 'CjkUnifiedIdeograph69f4', [], 'Display', None],
  [ 0x69f5, 'CjkUnifiedIdeograph69f5', [], 'Display', None],
  [ 0x69f6, 'CjkUnifiedIdeograph69f6', [], 'Display', None],
  [ 0x69f7, 'CjkUnifiedIdeograph69f7', [], 'Display', None],
  [ 0x69f8, 'CjkUnifiedIdeograph69f8', [], 'Display', None],
  [ 0x69f9, 'CjkUnifiedIdeograph69f9', [], 'Display', None],
  [ 0x69fa, 'CjkUnifiedIdeograph69fa', [], 'Display', None],
  [ 0x69fb, 'CjkUnifiedIdeograph69fb', [], 'Display', None],
  [ 0x69fc, 'CjkUnifiedIdeograph69fc', [], 'Display', None],
  [ 0x69fd, 'CjkUnifiedIdeograph69fd', [], 'Display', None],
  [ 0x69fe, 'CjkUnifiedIdeograph69fe', [], 'Display', None],
  [ 0x69ff, 'CjkUnifiedIdeograph69ff', [], 'Display', None],
  [ 0x6a00, 'CjkUnifiedIdeograph6a00', [], 'Display', None],
  [ 0x6a01, 'CjkUnifiedIdeograph6a01', [], 'Display', None],
  [ 0x6a02, 'CjkUnifiedIdeograph6a02', [], 'Display', None],
  [ 0x6a03, 'CjkUnifiedIdeograph6a03', [], 'Display', None],
  [ 0x6a04, 'CjkUnifiedIdeograph6a04', [], 'Display', None],
  [ 0x6a05, 'CjkUnifiedIdeograph6a05', [], 'Display', None],
  [ 0x6a06, 'CjkUnifiedIdeograph6a06', [], 'Display', None],
  [ 0x6a07, 'CjkUnifiedIdeograph6a07', [], 'Display', None],
  [ 0x6a08, 'CjkUnifiedIdeograph6a08', [], 'Display', None],
  [ 0x6a09, 'CjkUnifiedIdeograph6a09', [], 'Display', None],
  [ 0x6a0a, 'CjkUnifiedIdeograph6a0a', [], 'Display', None],
  [ 0x6a0b, 'CjkUnifiedIdeograph6a0b', [], 'Display', None],
  [ 0x6a0c, 'CjkUnifiedIdeograph6a0c', [], 'Display', None],
  [ 0x6a0d, 'CjkUnifiedIdeograph6a0d', [], 'Display', None],
  [ 0x6a0e, 'CjkUnifiedIdeograph6a0e', [], 'Display', None],
  [ 0x6a0f, 'CjkUnifiedIdeograph6a0f', [], 'Display', None],
  [ 0x6a10, 'CjkUnifiedIdeograph6a10', [], 'Display', None],
  [ 0x6a11, 'CjkUnifiedIdeograph6a11', [], 'Display', None],
  [ 0x6a12, 'CjkUnifiedIdeograph6a12', [], 'Display', None],
  [ 0x6a13, 'CjkUnifiedIdeograph6a13', [], 'Display', None],
  [ 0x6a14, 'CjkUnifiedIdeograph6a14', [], 'Display', None],
  [ 0x6a15, 'CjkUnifiedIdeograph6a15', [], 'Display', None],
  [ 0x6a16, 'CjkUnifiedIdeograph6a16', [], 'Display', None],
  [ 0x6a17, 'CjkUnifiedIdeograph6a17', [], 'Display', None],
  [ 0x6a18, 'CjkUnifiedIdeograph6a18', [], 'Display', None],
  [ 0x6a19, 'CjkUnifiedIdeograph6a19', [], 'Display', None],
  [ 0x6a1a, 'CjkUnifiedIdeograph6a1a', [], 'Display', None],
  [ 0x6a1b, 'CjkUnifiedIdeograph6a1b', [], 'Display', None],
  [ 0x6a1c, 'CjkUnifiedIdeograph6a1c', [], 'Display', None],
  [ 0x6a1d, 'CjkUnifiedIdeograph6a1d', [], 'Display', None],
  [ 0x6a1e, 'CjkUnifiedIdeograph6a1e', [], 'Display', None],
  [ 0x6a1f, 'CjkUnifiedIdeograph6a1f', [], 'Display', None],
  [ 0x6a20, 'CjkUnifiedIdeograph6a20', [], 'Display', None],
  [ 0x6a21, 'CjkUnifiedIdeograph6a21', [], 'Display', None],
  [ 0x6a22, 'CjkUnifiedIdeograph6a22', [], 'Display', None],
  [ 0x6a23, 'CjkUnifiedIdeograph6a23', [], 'Display', None],
  [ 0x6a24, 'CjkUnifiedIdeograph6a24', [], 'Display', None],
  [ 0x6a25, 'CjkUnifiedIdeograph6a25', [], 'Display', None],
  [ 0x6a26, 'CjkUnifiedIdeograph6a26', [], 'Display', None],
  [ 0x6a27, 'CjkUnifiedIdeograph6a27', [], 'Display', None],
  [ 0x6a28, 'CjkUnifiedIdeograph6a28', [], 'Display', None],
  [ 0x6a29, 'CjkUnifiedIdeograph6a29', [], 'Display', None],
  [ 0x6a2a, 'CjkUnifiedIdeograph6a2a', [], 'Display', None],
  [ 0x6a2b, 'CjkUnifiedIdeograph6a2b', [], 'Display', None],
  [ 0x6a2c, 'CjkUnifiedIdeograph6a2c', [], 'Display', None],
  [ 0x6a2d, 'CjkUnifiedIdeograph6a2d', [], 'Display', None],
  [ 0x6a2e, 'CjkUnifiedIdeograph6a2e', [], 'Display', None],
  [ 0x6a2f, 'CjkUnifiedIdeograph6a2f', [], 'Display', None],
  [ 0x6a30, 'CjkUnifiedIdeograph6a30', [], 'Display', None],
  [ 0x6a31, 'CjkUnifiedIdeograph6a31', [], 'Display', None],
  [ 0x6a32, 'CjkUnifiedIdeograph6a32', [], 'Display', None],
  [ 0x6a33, 'CjkUnifiedIdeograph6a33', [], 'Display', None],
  [ 0x6a34, 'CjkUnifiedIdeograph6a34', [], 'Display', None],
  [ 0x6a35, 'CjkUnifiedIdeograph6a35', [], 'Display', None],
  [ 0x6a36, 'CjkUnifiedIdeograph6a36', [], 'Display', None],
  [ 0x6a37, 'CjkUnifiedIdeograph6a37', [], 'Display', None],
  [ 0x6a38, 'CjkUnifiedIdeograph6a38', [], 'Display', None],
  [ 0x6a39, 'CjkUnifiedIdeograph6a39', [], 'Display', None],
  [ 0x6a3a, 'CjkUnifiedIdeograph6a3a', [], 'Display', None],
  [ 0x6a3b, 'CjkUnifiedIdeograph6a3b', [], 'Display', None],
  [ 0x6a3c, 'CjkUnifiedIdeograph6a3c', [], 'Display', None],
  [ 0x6a3d, 'CjkUnifiedIdeograph6a3d', [], 'Display', None],
  [ 0x6a3e, 'CjkUnifiedIdeograph6a3e', [], 'Display', None],
  [ 0x6a3f, 'CjkUnifiedIdeograph6a3f', [], 'Display', None],
  [ 0x6a40, 'CjkUnifiedIdeograph6a40', [], 'Display', None],
  [ 0x6a41, 'CjkUnifiedIdeograph6a41', [], 'Display', None],
  [ 0x6a42, 'CjkUnifiedIdeograph6a42', [], 'Display', None],
  [ 0x6a43, 'CjkUnifiedIdeograph6a43', [], 'Display', None],
  [ 0x6a44, 'CjkUnifiedIdeograph6a44', [], 'Display', None],
  [ 0x6a45, 'CjkUnifiedIdeograph6a45', [], 'Display', None],
  [ 0x6a46, 'CjkUnifiedIdeograph6a46', [], 'Display', None],
  [ 0x6a47, 'CjkUnifiedIdeograph6a47', [], 'Display', None],
  [ 0x6a48, 'CjkUnifiedIdeograph6a48', [], 'Display', None],
  [ 0x6a49, 'CjkUnifiedIdeograph6a49', [], 'Display', None],
  [ 0x6a4a, 'CjkUnifiedIdeograph6a4a', [], 'Display', None],
  [ 0x6a4b, 'CjkUnifiedIdeograph6a4b', [], 'Display', None],
  [ 0x6a4c, 'CjkUnifiedIdeograph6a4c', [], 'Display', None],
  [ 0x6a4d, 'CjkUnifiedIdeograph6a4d', [], 'Display', None],
  [ 0x6a4e, 'CjkUnifiedIdeograph6a4e', [], 'Display', None],
  [ 0x6a4f, 'CjkUnifiedIdeograph6a4f', [], 'Display', None],
  [ 0x6a50, 'CjkUnifiedIdeograph6a50', [], 'Display', None],
  [ 0x6a51, 'CjkUnifiedIdeograph6a51', [], 'Display', None],
  [ 0x6a52, 'CjkUnifiedIdeograph6a52', [], 'Display', None],
  [ 0x6a53, 'CjkUnifiedIdeograph6a53', [], 'Display', None],
  [ 0x6a54, 'CjkUnifiedIdeograph6a54', [], 'Display', None],
  [ 0x6a55, 'CjkUnifiedIdeograph6a55', [], 'Display', None],
  [ 0x6a56, 'CjkUnifiedIdeograph6a56', [], 'Display', None],
  [ 0x6a57, 'CjkUnifiedIdeograph6a57', [], 'Display', None],
  [ 0x6a58, 'CjkUnifiedIdeograph6a58', [], 'Display', None],
  [ 0x6a59, 'CjkUnifiedIdeograph6a59', [], 'Display', None],
  [ 0x6a5a, 'CjkUnifiedIdeograph6a5a', [], 'Display', None],
  [ 0x6a5b, 'CjkUnifiedIdeograph6a5b', [], 'Display', None],
  [ 0x6a5c, 'CjkUnifiedIdeograph6a5c', [], 'Display', None],
  [ 0x6a5d, 'CjkUnifiedIdeograph6a5d', [], 'Display', None],
  [ 0x6a5e, 'CjkUnifiedIdeograph6a5e', [], 'Display', None],
  [ 0x6a5f, 'CjkUnifiedIdeograph6a5f', [], 'Display', None],
  [ 0x6a60, 'CjkUnifiedIdeograph6a60', [], 'Display', None],
  [ 0x6a61, 'CjkUnifiedIdeograph6a61', [], 'Display', None],
  [ 0x6a62, 'CjkUnifiedIdeograph6a62', [], 'Display', None],
  [ 0x6a63, 'CjkUnifiedIdeograph6a63', [], 'Display', None],
  [ 0x6a64, 'CjkUnifiedIdeograph6a64', [], 'Display', None],
  [ 0x6a65, 'CjkUnifiedIdeograph6a65', [], 'Display', None],
  [ 0x6a66, 'CjkUnifiedIdeograph6a66', [], 'Display', None],
  [ 0x6a67, 'CjkUnifiedIdeograph6a67', [], 'Display', None],
  [ 0x6a68, 'CjkUnifiedIdeograph6a68', [], 'Display', None],
  [ 0x6a69, 'CjkUnifiedIdeograph6a69', [], 'Display', None],
  [ 0x6a6a, 'CjkUnifiedIdeograph6a6a', [], 'Display', None],
  [ 0x6a6b, 'CjkUnifiedIdeograph6a6b', [], 'Display', None],
  [ 0x6a6c, 'CjkUnifiedIdeograph6a6c', [], 'Display', None],
  [ 0x6a6d, 'CjkUnifiedIdeograph6a6d', [], 'Display', None],
  [ 0x6a6e, 'CjkUnifiedIdeograph6a6e', [], 'Display', None],
  [ 0x6a6f, 'CjkUnifiedIdeograph6a6f', [], 'Display', None],
  [ 0x6a70, 'CjkUnifiedIdeograph6a70', [], 'Display', None],
  [ 0x6a71, 'CjkUnifiedIdeograph6a71', [], 'Display', None],
  [ 0x6a72, 'CjkUnifiedIdeograph6a72', [], 'Display', None],
  [ 0x6a73, 'CjkUnifiedIdeograph6a73', [], 'Display', None],
  [ 0x6a74, 'CjkUnifiedIdeograph6a74', [], 'Display', None],
  [ 0x6a75, 'CjkUnifiedIdeograph6a75', [], 'Display', None],
  [ 0x6a76, 'CjkUnifiedIdeograph6a76', [], 'Display', None],
  [ 0x6a77, 'CjkUnifiedIdeograph6a77', [], 'Display', None],
  [ 0x6a78, 'CjkUnifiedIdeograph6a78', [], 'Display', None],
  [ 0x6a79, 'CjkUnifiedIdeograph6a79', [], 'Display', None],
  [ 0x6a7a, 'CjkUnifiedIdeograph6a7a', [], 'Display', None],
  [ 0x6a7b, 'CjkUnifiedIdeograph6a7b', [], 'Display', None],
  [ 0x6a7c, 'CjkUnifiedIdeograph6a7c', [], 'Display', None],
  [ 0x6a7d, 'CjkUnifiedIdeograph6a7d', [], 'Display', None],
  [ 0x6a7e, 'CjkUnifiedIdeograph6a7e', [], 'Display', None],
  [ 0x6a7f, 'CjkUnifiedIdeograph6a7f', [], 'Display', None],
  [ 0x6a80, 'CjkUnifiedIdeograph6a80', [], 'Display', None],
  [ 0x6a81, 'CjkUnifiedIdeograph6a81', [], 'Display', None],
  [ 0x6a82, 'CjkUnifiedIdeograph6a82', [], 'Display', None],
  [ 0x6a83, 'CjkUnifiedIdeograph6a83', [], 'Display', None],
  [ 0x6a84, 'CjkUnifiedIdeograph6a84', [], 'Display', None],
  [ 0x6a85, 'CjkUnifiedIdeograph6a85', [], 'Display', None],
  [ 0x6a86, 'CjkUnifiedIdeograph6a86', [], 'Display', None],
  [ 0x6a87, 'CjkUnifiedIdeograph6a87', [], 'Display', None],
  [ 0x6a88, 'CjkUnifiedIdeograph6a88', [], 'Display', None],
  [ 0x6a89, 'CjkUnifiedIdeograph6a89', [], 'Display', None],
  [ 0x6a8a, 'CjkUnifiedIdeograph6a8a', [], 'Display', None],
  [ 0x6a8b, 'CjkUnifiedIdeograph6a8b', [], 'Display', None],
  [ 0x6a8c, 'CjkUnifiedIdeograph6a8c', [], 'Display', None],
  [ 0x6a8d, 'CjkUnifiedIdeograph6a8d', [], 'Display', None],
  [ 0x6a8e, 'CjkUnifiedIdeograph6a8e', [], 'Display', None],
  [ 0x6a8f, 'CjkUnifiedIdeograph6a8f', [], 'Display', None],
  [ 0x6a90, 'CjkUnifiedIdeograph6a90', [], 'Display', None],
  [ 0x6a91, 'CjkUnifiedIdeograph6a91', [], 'Display', None],
  [ 0x6a92, 'CjkUnifiedIdeograph6a92', [], 'Display', None],
  [ 0x6a93, 'CjkUnifiedIdeograph6a93', [], 'Display', None],
  [ 0x6a94, 'CjkUnifiedIdeograph6a94', [], 'Display', None],
  [ 0x6a95, 'CjkUnifiedIdeograph6a95', [], 'Display', None],
  [ 0x6a96, 'CjkUnifiedIdeograph6a96', [], 'Display', None],
  [ 0x6a97, 'CjkUnifiedIdeograph6a97', [], 'Display', None],
  [ 0x6a98, 'CjkUnifiedIdeograph6a98', [], 'Display', None],
  [ 0x6a99, 'CjkUnifiedIdeograph6a99', [], 'Display', None],
  [ 0x6a9a, 'CjkUnifiedIdeograph6a9a', [], 'Display', None],
  [ 0x6a9b, 'CjkUnifiedIdeograph6a9b', [], 'Display', None],
  [ 0x6a9c, 'CjkUnifiedIdeograph6a9c', [], 'Display', None],
  [ 0x6a9d, 'CjkUnifiedIdeograph6a9d', [], 'Display', None],
  [ 0x6a9e, 'CjkUnifiedIdeograph6a9e', [], 'Display', None],
  [ 0x6a9f, 'CjkUnifiedIdeograph6a9f', [], 'Display', None],
  [ 0x6aa0, 'CjkUnifiedIdeograph6aa0', [], 'Display', None],
  [ 0x6aa1, 'CjkUnifiedIdeograph6aa1', [], 'Display', None],
  [ 0x6aa2, 'CjkUnifiedIdeograph6aa2', [], 'Display', None],
  [ 0x6aa3, 'CjkUnifiedIdeograph6aa3', [], 'Display', None],
  [ 0x6aa4, 'CjkUnifiedIdeograph6aa4', [], 'Display', None],
  [ 0x6aa5, 'CjkUnifiedIdeograph6aa5', [], 'Display', None],
  [ 0x6aa6, 'CjkUnifiedIdeograph6aa6', [], 'Display', None],
  [ 0x6aa7, 'CjkUnifiedIdeograph6aa7', [], 'Display', None],
  [ 0x6aa8, 'CjkUnifiedIdeograph6aa8', [], 'Display', None],
  [ 0x6aa9, 'CjkUnifiedIdeograph6aa9', [], 'Display', None],
  [ 0x6aaa, 'CjkUnifiedIdeograph6aaa', [], 'Display', None],
  [ 0x6aab, 'CjkUnifiedIdeograph6aab', [], 'Display', None],
  [ 0x6aac, 'CjkUnifiedIdeograph6aac', [], 'Display', None],
  [ 0x6aad, 'CjkUnifiedIdeograph6aad', [], 'Display', None],
  [ 0x6aae, 'CjkUnifiedIdeograph6aae', [], 'Display', None],
  [ 0x6aaf, 'CjkUnifiedIdeograph6aaf', [], 'Display', None],
  [ 0x6ab0, 'CjkUnifiedIdeograph6ab0', [], 'Display', None],
  [ 0x6ab1, 'CjkUnifiedIdeograph6ab1', [], 'Display', None],
  [ 0x6ab2, 'CjkUnifiedIdeograph6ab2', [], 'Display', None],
  [ 0x6ab3, 'CjkUnifiedIdeograph6ab3', [], 'Display', None],
  [ 0x6ab4, 'CjkUnifiedIdeograph6ab4', [], 'Display', None],
  [ 0x6ab5, 'CjkUnifiedIdeograph6ab5', [], 'Display', None],
  [ 0x6ab6, 'CjkUnifiedIdeograph6ab6', [], 'Display', None],
  [ 0x6ab7, 'CjkUnifiedIdeograph6ab7', [], 'Display', None],
  [ 0x6ab8, 'CjkUnifiedIdeograph6ab8', [], 'Display', None],
  [ 0x6ab9, 'CjkUnifiedIdeograph6ab9', [], 'Display', None],
  [ 0x6aba, 'CjkUnifiedIdeograph6aba', [], 'Display', None],
  [ 0x6abb, 'CjkUnifiedIdeograph6abb', [], 'Display', None],
  [ 0x6abc, 'CjkUnifiedIdeograph6abc', [], 'Display', None],
  [ 0x6abd, 'CjkUnifiedIdeograph6abd', [], 'Display', None],
  [ 0x6abe, 'CjkUnifiedIdeograph6abe', [], 'Display', None],
  [ 0x6abf, 'CjkUnifiedIdeograph6abf', [], 'Display', None],
  [ 0x6ac0, 'CjkUnifiedIdeograph6ac0', [], 'Display', None],
  [ 0x6ac1, 'CjkUnifiedIdeograph6ac1', [], 'Display', None],
  [ 0x6ac2, 'CjkUnifiedIdeograph6ac2', [], 'Display', None],
  [ 0x6ac3, 'CjkUnifiedIdeograph6ac3', [], 'Display', None],
  [ 0x6ac4, 'CjkUnifiedIdeograph6ac4', [], 'Display', None],
  [ 0x6ac5, 'CjkUnifiedIdeograph6ac5', [], 'Display', None],
  [ 0x6ac6, 'CjkUnifiedIdeograph6ac6', [], 'Display', None],
  [ 0x6ac7, 'CjkUnifiedIdeograph6ac7', [], 'Display', None],
  [ 0x6ac8, 'CjkUnifiedIdeograph6ac8', [], 'Display', None],
  [ 0x6ac9, 'CjkUnifiedIdeograph6ac9', [], 'Display', None],
  [ 0x6aca, 'CjkUnifiedIdeograph6aca', [], 'Display', None],
  [ 0x6acb, 'CjkUnifiedIdeograph6acb', [], 'Display', None],
  [ 0x6acc, 'CjkUnifiedIdeograph6acc', [], 'Display', None],
  [ 0x6acd, 'CjkUnifiedIdeograph6acd', [], 'Display', None],
  [ 0x6ace, 'CjkUnifiedIdeograph6ace', [], 'Display', None],
  [ 0x6acf, 'CjkUnifiedIdeograph6acf', [], 'Display', None],
  [ 0x6ad0, 'CjkUnifiedIdeograph6ad0', [], 'Display', None],
  [ 0x6ad1, 'CjkUnifiedIdeograph6ad1', [], 'Display', None],
  [ 0x6ad2, 'CjkUnifiedIdeograph6ad2', [], 'Display', None],
  [ 0x6ad3, 'CjkUnifiedIdeograph6ad3', [], 'Display', None],
  [ 0x6ad4, 'CjkUnifiedIdeograph6ad4', [], 'Display', None],
  [ 0x6ad5, 'CjkUnifiedIdeograph6ad5', [], 'Display', None],
  [ 0x6ad6, 'CjkUnifiedIdeograph6ad6', [], 'Display', None],
  [ 0x6ad7, 'CjkUnifiedIdeograph6ad7', [], 'Display', None],
  [ 0x6ad8, 'CjkUnifiedIdeograph6ad8', [], 'Display', None],
  [ 0x6ad9, 'CjkUnifiedIdeograph6ad9', [], 'Display', None],
  [ 0x6ada, 'CjkUnifiedIdeograph6ada', [], 'Display', None],
  [ 0x6adb, 'CjkUnifiedIdeograph6adb', [], 'Display', None],
  [ 0x6adc, 'CjkUnifiedIdeograph6adc', [], 'Display', None],
  [ 0x6add, 'CjkUnifiedIdeograph6add', [], 'Display', None],
  [ 0x6ade, 'CjkUnifiedIdeograph6ade', [], 'Display', None],
  [ 0x6adf, 'CjkUnifiedIdeograph6adf', [], 'Display', None],
  [ 0x6ae0, 'CjkUnifiedIdeograph6ae0', [], 'Display', None],
  [ 0x6ae1, 'CjkUnifiedIdeograph6ae1', [], 'Display', None],
  [ 0x6ae2, 'CjkUnifiedIdeograph6ae2', [], 'Display', None],
  [ 0x6ae3, 'CjkUnifiedIdeograph6ae3', [], 'Display', None],
  [ 0x6ae4, 'CjkUnifiedIdeograph6ae4', [], 'Display', None],
  [ 0x6ae5, 'CjkUnifiedIdeograph6ae5', [], 'Display', None],
  [ 0x6ae6, 'CjkUnifiedIdeograph6ae6', [], 'Display', None],
  [ 0x6ae7, 'CjkUnifiedIdeograph6ae7', [], 'Display', None],
  [ 0x6ae8, 'CjkUnifiedIdeograph6ae8', [], 'Display', None],
  [ 0x6ae9, 'CjkUnifiedIdeograph6ae9', [], 'Display', None],
  [ 0x6aea, 'CjkUnifiedIdeograph6aea', [], 'Display', None],
  [ 0x6aeb, 'CjkUnifiedIdeograph6aeb', [], 'Display', None],
  [ 0x6aec, 'CjkUnifiedIdeograph6aec', [], 'Display', None],
  [ 0x6aed, 'CjkUnifiedIdeograph6aed', [], 'Display', None],
  [ 0x6aee, 'CjkUnifiedIdeograph6aee', [], 'Display', None],
  [ 0x6aef, 'CjkUnifiedIdeograph6aef', [], 'Display', None],
  [ 0x6af0, 'CjkUnifiedIdeograph6af0', [], 'Display', None],
  [ 0x6af1, 'CjkUnifiedIdeograph6af1', [], 'Display', None],
  [ 0x6af2, 'CjkUnifiedIdeograph6af2', [], 'Display', None],
  [ 0x6af3, 'CjkUnifiedIdeograph6af3', [], 'Display', None],
  [ 0x6af4, 'CjkUnifiedIdeograph6af4', [], 'Display', None],
  [ 0x6af5, 'CjkUnifiedIdeograph6af5', [], 'Display', None],
  [ 0x6af6, 'CjkUnifiedIdeograph6af6', [], 'Display', None],
  [ 0x6af7, 'CjkUnifiedIdeograph6af7', [], 'Display', None],
  [ 0x6af8, 'CjkUnifiedIdeograph6af8', [], 'Display', None],
  [ 0x6af9, 'CjkUnifiedIdeograph6af9', [], 'Display', None],
  [ 0x6afa, 'CjkUnifiedIdeograph6afa', [], 'Display', None],
  [ 0x6afb, 'CjkUnifiedIdeograph6afb', [], 'Display', None],
  [ 0x6afc, 'CjkUnifiedIdeograph6afc', [], 'Display', None],
  [ 0x6afd, 'CjkUnifiedIdeograph6afd', [], 'Display', None],
  [ 0x6afe, 'CjkUnifiedIdeograph6afe', [], 'Display', None],
  [ 0x6aff, 'CjkUnifiedIdeograph6aff', [], 'Display', None],
  [ 0x6b00, 'CjkUnifiedIdeograph6b00', [], 'Display', None],
  [ 0x6b01, 'CjkUnifiedIdeograph6b01', [], 'Display', None],
  [ 0x6b02, 'CjkUnifiedIdeograph6b02', [], 'Display', None],
  [ 0x6b03, 'CjkUnifiedIdeograph6b03', [], 'Display', None],
  [ 0x6b04, 'CjkUnifiedIdeograph6b04', [], 'Display', None],
  [ 0x6b05, 'CjkUnifiedIdeograph6b05', [], 'Display', None],
  [ 0x6b06, 'CjkUnifiedIdeograph6b06', [], 'Display', None],
  [ 0x6b07, 'CjkUnifiedIdeograph6b07', [], 'Display', None],
  [ 0x6b08, 'CjkUnifiedIdeograph6b08', [], 'Display', None],
  [ 0x6b09, 'CjkUnifiedIdeograph6b09', [], 'Display', None],
  [ 0x6b0a, 'CjkUnifiedIdeograph6b0a', [], 'Display', None],
  [ 0x6b0b, 'CjkUnifiedIdeograph6b0b', [], 'Display', None],
  [ 0x6b0c, 'CjkUnifiedIdeograph6b0c', [], 'Display', None],
  [ 0x6b0d, 'CjkUnifiedIdeograph6b0d', [], 'Display', None],
  [ 0x6b0e, 'CjkUnifiedIdeograph6b0e', [], 'Display', None],
  [ 0x6b0f, 'CjkUnifiedIdeograph6b0f', [], 'Display', None],
  [ 0x6b10, 'CjkUnifiedIdeograph6b10', [], 'Display', None],
  [ 0x6b11, 'CjkUnifiedIdeograph6b11', [], 'Display', None],
  [ 0x6b12, 'CjkUnifiedIdeograph6b12', [], 'Display', None],
  [ 0x6b13, 'CjkUnifiedIdeograph6b13', [], 'Display', None],
  [ 0x6b14, 'CjkUnifiedIdeograph6b14', [], 'Display', None],
  [ 0x6b15, 'CjkUnifiedIdeograph6b15', [], 'Display', None],
  [ 0x6b16, 'CjkUnifiedIdeograph6b16', [], 'Display', None],
  [ 0x6b17, 'CjkUnifiedIdeograph6b17', [], 'Display', None],
  [ 0x6b18, 'CjkUnifiedIdeograph6b18', [], 'Display', None],
  [ 0x6b19, 'CjkUnifiedIdeograph6b19', [], 'Display', None],
  [ 0x6b1a, 'CjkUnifiedIdeograph6b1a', [], 'Display', None],
  [ 0x6b1b, 'CjkUnifiedIdeograph6b1b', [], 'Display', None],
  [ 0x6b1c, 'CjkUnifiedIdeograph6b1c', [], 'Display', None],
  [ 0x6b1d, 'CjkUnifiedIdeograph6b1d', [], 'Display', None],
  [ 0x6b1e, 'CjkUnifiedIdeograph6b1e', [], 'Display', None],
  [ 0x6b1f, 'CjkUnifiedIdeograph6b1f', [], 'Display', None],
  [ 0x6b20, 'CjkUnifiedIdeograph6b20', [], 'Display', None],
  [ 0x6b21, 'CjkUnifiedIdeograph6b21', [], 'Display', None],
  [ 0x6b22, 'CjkUnifiedIdeograph6b22', [], 'Display', None],
  [ 0x6b23, 'CjkUnifiedIdeograph6b23', [], 'Display', None],
  [ 0x6b24, 'CjkUnifiedIdeograph6b24', [], 'Display', None],
  [ 0x6b25, 'CjkUnifiedIdeograph6b25', [], 'Display', None],
  [ 0x6b26, 'CjkUnifiedIdeograph6b26', [], 'Display', None],
  [ 0x6b27, 'CjkUnifiedIdeograph6b27', [], 'Display', None],
  [ 0x6b28, 'CjkUnifiedIdeograph6b28', [], 'Display', None],
  [ 0x6b29, 'CjkUnifiedIdeograph6b29', [], 'Display', None],
  [ 0x6b2a, 'CjkUnifiedIdeograph6b2a', [], 'Display', None],
  [ 0x6b2b, 'CjkUnifiedIdeograph6b2b', [], 'Display', None],
  [ 0x6b2c, 'CjkUnifiedIdeograph6b2c', [], 'Display', None],
  [ 0x6b2d, 'CjkUnifiedIdeograph6b2d', [], 'Display', None],
  [ 0x6b2e, 'CjkUnifiedIdeograph6b2e', [], 'Display', None],
  [ 0x6b2f, 'CjkUnifiedIdeograph6b2f', [], 'Display', None],
  [ 0x6b30, 'CjkUnifiedIdeograph6b30', [], 'Display', None],
  [ 0x6b31, 'CjkUnifiedIdeograph6b31', [], 'Display', None],
  [ 0x6b32, 'CjkUnifiedIdeograph6b32', [], 'Display', None],
  [ 0x6b33, 'CjkUnifiedIdeograph6b33', [], 'Display', None],
  [ 0x6b34, 'CjkUnifiedIdeograph6b34', [], 'Display', None],
  [ 0x6b35, 'CjkUnifiedIdeograph6b35', [], 'Display', None],
  [ 0x6b36, 'CjkUnifiedIdeograph6b36', [], 'Display', None],
  [ 0x6b37, 'CjkUnifiedIdeograph6b37', [], 'Display', None],
  [ 0x6b38, 'CjkUnifiedIdeograph6b38', [], 'Display', None],
  [ 0x6b39, 'CjkUnifiedIdeograph6b39', [], 'Display', None],
  [ 0x6b3a, 'CjkUnifiedIdeograph6b3a', [], 'Display', None],
  [ 0x6b3b, 'CjkUnifiedIdeograph6b3b', [], 'Display', None],
  [ 0x6b3c, 'CjkUnifiedIdeograph6b3c', [], 'Display', None],
  [ 0x6b3d, 'CjkUnifiedIdeograph6b3d', [], 'Display', None],
  [ 0x6b3e, 'CjkUnifiedIdeograph6b3e', [], 'Display', None],
  [ 0x6b3f, 'CjkUnifiedIdeograph6b3f', [], 'Display', None],
  [ 0x6b40, 'CjkUnifiedIdeograph6b40', [], 'Display', None],
  [ 0x6b41, 'CjkUnifiedIdeograph6b41', [], 'Display', None],
  [ 0x6b42, 'CjkUnifiedIdeograph6b42', [], 'Display', None],
  [ 0x6b43, 'CjkUnifiedIdeograph6b43', [], 'Display', None],
  [ 0x6b44, 'CjkUnifiedIdeograph6b44', [], 'Display', None],
  [ 0x6b45, 'CjkUnifiedIdeograph6b45', [], 'Display', None],
  [ 0x6b46, 'CjkUnifiedIdeograph6b46', [], 'Display', None],
  [ 0x6b47, 'CjkUnifiedIdeograph6b47', [], 'Display', None],
  [ 0x6b48, 'CjkUnifiedIdeograph6b48', [], 'Display', None],
  [ 0x6b49, 'CjkUnifiedIdeograph6b49', [], 'Display', None],
  [ 0x6b4a, 'CjkUnifiedIdeograph6b4a', [], 'Display', None],
  [ 0x6b4b, 'CjkUnifiedIdeograph6b4b', [], 'Display', None],
  [ 0x6b4c, 'CjkUnifiedIdeograph6b4c', [], 'Display', None],
  [ 0x6b4d, 'CjkUnifiedIdeograph6b4d', [], 'Display', None],
  [ 0x6b4e, 'CjkUnifiedIdeograph6b4e', [], 'Display', None],
  [ 0x6b4f, 'CjkUnifiedIdeograph6b4f', [], 'Display', None],
  [ 0x6b50, 'CjkUnifiedIdeograph6b50', [], 'Display', None],
  [ 0x6b51, 'CjkUnifiedIdeograph6b51', [], 'Display', None],
  [ 0x6b52, 'CjkUnifiedIdeograph6b52', [], 'Display', None],
  [ 0x6b53, 'CjkUnifiedIdeograph6b53', [], 'Display', None],
  [ 0x6b54, 'CjkUnifiedIdeograph6b54', [], 'Display', None],
  [ 0x6b55, 'CjkUnifiedIdeograph6b55', [], 'Display', None],
  [ 0x6b56, 'CjkUnifiedIdeograph6b56', [], 'Display', None],
  [ 0x6b57, 'CjkUnifiedIdeograph6b57', [], 'Display', None],
  [ 0x6b58, 'CjkUnifiedIdeograph6b58', [], 'Display', None],
  [ 0x6b59, 'CjkUnifiedIdeograph6b59', [], 'Display', None],
  [ 0x6b5a, 'CjkUnifiedIdeograph6b5a', [], 'Display', None],
  [ 0x6b5b, 'CjkUnifiedIdeograph6b5b', [], 'Display', None],
  [ 0x6b5c, 'CjkUnifiedIdeograph6b5c', [], 'Display', None],
  [ 0x6b5d, 'CjkUnifiedIdeograph6b5d', [], 'Display', None],
  [ 0x6b5e, 'CjkUnifiedIdeograph6b5e', [], 'Display', None],
  [ 0x6b5f, 'CjkUnifiedIdeograph6b5f', [], 'Display', None],
  [ 0x6b60, 'CjkUnifiedIdeograph6b60', [], 'Display', None],
  [ 0x6b61, 'CjkUnifiedIdeograph6b61', [], 'Display', None],
  [ 0x6b62, 'CjkUnifiedIdeograph6b62', [], 'Display', None],
  [ 0x6b63, 'CjkUnifiedIdeograph6b63', [], 'Display', None],
  [ 0x6b64, 'CjkUnifiedIdeograph6b64', [], 'Display', None],
  [ 0x6b65, 'CjkUnifiedIdeograph6b65', [], 'Display', None],
  [ 0x6b66, 'CjkUnifiedIdeograph6b66', [], 'Display', None],
  [ 0x6b67, 'CjkUnifiedIdeograph6b67', [], 'Display', None],
  [ 0x6b68, 'CjkUnifiedIdeograph6b68', [], 'Display', None],
  [ 0x6b69, 'CjkUnifiedIdeograph6b69', [], 'Display', None],
  [ 0x6b6a, 'CjkUnifiedIdeograph6b6a', [], 'Display', None],
  [ 0x6b6b, 'CjkUnifiedIdeograph6b6b', [], 'Display', None],
  [ 0x6b6c, 'CjkUnifiedIdeograph6b6c', [], 'Display', None],
  [ 0x6b6d, 'CjkUnifiedIdeograph6b6d', [], 'Display', None],
  [ 0x6b6e, 'CjkUnifiedIdeograph6b6e', [], 'Display', None],
  [ 0x6b6f, 'CjkUnifiedIdeograph6b6f', [], 'Display', None],
  [ 0x6b70, 'CjkUnifiedIdeograph6b70', [], 'Display', None],
  [ 0x6b71, 'CjkUnifiedIdeograph6b71', [], 'Display', None],
  [ 0x6b72, 'CjkUnifiedIdeograph6b72', [], 'Display', None],
  [ 0x6b73, 'CjkUnifiedIdeograph6b73', [], 'Display', None],
  [ 0x6b74, 'CjkUnifiedIdeograph6b74', [], 'Display', None],
  [ 0x6b75, 'CjkUnifiedIdeograph6b75', [], 'Display', None],
  [ 0x6b76, 'CjkUnifiedIdeograph6b76', [], 'Display', None],
  [ 0x6b77, 'CjkUnifiedIdeograph6b77', [], 'Display', None],
  [ 0x6b78, 'CjkUnifiedIdeograph6b78', [], 'Display', None],
  [ 0x6b79, 'CjkUnifiedIdeograph6b79', [], 'Display', None],
  [ 0x6b7a, 'CjkUnifiedIdeograph6b7a', [], 'Display', None],
  [ 0x6b7b, 'CjkUnifiedIdeograph6b7b', [], 'Display', None],
  [ 0x6b7c, 'CjkUnifiedIdeograph6b7c', [], 'Display', None],
  [ 0x6b7d, 'CjkUnifiedIdeograph6b7d', [], 'Display', None],
  [ 0x6b7e, 'CjkUnifiedIdeograph6b7e', [], 'Display', None],
  [ 0x6b7f, 'CjkUnifiedIdeograph6b7f', [], 'Display', None],
  [ 0x6b80, 'CjkUnifiedIdeograph6b80', [], 'Display', None],
  [ 0x6b81, 'CjkUnifiedIdeograph6b81', [], 'Display', None],
  [ 0x6b82, 'CjkUnifiedIdeograph6b82', [], 'Display', None],
  [ 0x6b83, 'CjkUnifiedIdeograph6b83', [], 'Display', None],
  [ 0x6b84, 'CjkUnifiedIdeograph6b84', [], 'Display', None],
  [ 0x6b85, 'CjkUnifiedIdeograph6b85', [], 'Display', None],
  [ 0x6b86, 'CjkUnifiedIdeograph6b86', [], 'Display', None],
  [ 0x6b87, 'CjkUnifiedIdeograph6b87', [], 'Display', None],
  [ 0x6b88, 'CjkUnifiedIdeograph6b88', [], 'Display', None],
  [ 0x6b89, 'CjkUnifiedIdeograph6b89', [], 'Display', None],
  [ 0x6b8a, 'CjkUnifiedIdeograph6b8a', [], 'Display', None],
  [ 0x6b8b, 'CjkUnifiedIdeograph6b8b', [], 'Display', None],
  [ 0x6b8c, 'CjkUnifiedIdeograph6b8c', [], 'Display', None],
  [ 0x6b8d, 'CjkUnifiedIdeograph6b8d', [], 'Display', None],
  [ 0x6b8e, 'CjkUnifiedIdeograph6b8e', [], 'Display', None],
  [ 0x6b8f, 'CjkUnifiedIdeograph6b8f', [], 'Display', None],
  [ 0x6b90, 'CjkUnifiedIdeograph6b90', [], 'Display', None],
  [ 0x6b91, 'CjkUnifiedIdeograph6b91', [], 'Display', None],
  [ 0x6b92, 'CjkUnifiedIdeograph6b92', [], 'Display', None],
  [ 0x6b93, 'CjkUnifiedIdeograph6b93', [], 'Display', None],
  [ 0x6b94, 'CjkUnifiedIdeograph6b94', [], 'Display', None],
  [ 0x6b95, 'CjkUnifiedIdeograph6b95', [], 'Display', None],
  [ 0x6b96, 'CjkUnifiedIdeograph6b96', [], 'Display', None],
  [ 0x6b97, 'CjkUnifiedIdeograph6b97', [], 'Display', None],
  [ 0x6b98, 'CjkUnifiedIdeograph6b98', [], 'Display', None],
  [ 0x6b99, 'CjkUnifiedIdeograph6b99', [], 'Display', None],
  [ 0x6b9a, 'CjkUnifiedIdeograph6b9a', [], 'Display', None],
  [ 0x6b9b, 'CjkUnifiedIdeograph6b9b', [], 'Display', None],
  [ 0x6b9c, 'CjkUnifiedIdeograph6b9c', [], 'Display', None],
  [ 0x6b9d, 'CjkUnifiedIdeograph6b9d', [], 'Display', None],
  [ 0x6b9e, 'CjkUnifiedIdeograph6b9e', [], 'Display', None],
  [ 0x6b9f, 'CjkUnifiedIdeograph6b9f', [], 'Display', None],
  [ 0x6ba0, 'CjkUnifiedIdeograph6ba0', [], 'Display', None],
  [ 0x6ba1, 'CjkUnifiedIdeograph6ba1', [], 'Display', None],
  [ 0x6ba2, 'CjkUnifiedIdeograph6ba2', [], 'Display', None],
  [ 0x6ba3, 'CjkUnifiedIdeograph6ba3', [], 'Display', None],
  [ 0x6ba4, 'CjkUnifiedIdeograph6ba4', [], 'Display', None],
  [ 0x6ba5, 'CjkUnifiedIdeograph6ba5', [], 'Display', None],
  [ 0x6ba6, 'CjkUnifiedIdeograph6ba6', [], 'Display', None],
  [ 0x6ba7, 'CjkUnifiedIdeograph6ba7', [], 'Display', None],
  [ 0x6ba8, 'CjkUnifiedIdeograph6ba8', [], 'Display', None],
  [ 0x6ba9, 'CjkUnifiedIdeograph6ba9', [], 'Display', None],
  [ 0x6baa, 'CjkUnifiedIdeograph6baa', [], 'Display', None],
  [ 0x6bab, 'CjkUnifiedIdeograph6bab', [], 'Display', None],
  [ 0x6bac, 'CjkUnifiedIdeograph6bac', [], 'Display', None],
  [ 0x6bad, 'CjkUnifiedIdeograph6bad', [], 'Display', None],
  [ 0x6bae, 'CjkUnifiedIdeograph6bae', [], 'Display', None],
  [ 0x6baf, 'CjkUnifiedIdeograph6baf', [], 'Display', None],
  [ 0x6bb0, 'CjkUnifiedIdeograph6bb0', [], 'Display', None],
  [ 0x6bb1, 'CjkUnifiedIdeograph6bb1', [], 'Display', None],
  [ 0x6bb2, 'CjkUnifiedIdeograph6bb2', [], 'Display', None],
  [ 0x6bb3, 'CjkUnifiedIdeograph6bb3', [], 'Display', None],
  [ 0x6bb4, 'CjkUnifiedIdeograph6bb4', [], 'Display', None],
  [ 0x6bb5, 'CjkUnifiedIdeograph6bb5', [], 'Display', None],
  [ 0x6bb6, 'CjkUnifiedIdeograph6bb6', [], 'Display', None],
  [ 0x6bb7, 'CjkUnifiedIdeograph6bb7', [], 'Display', None],
  [ 0x6bb8, 'CjkUnifiedIdeograph6bb8', [], 'Display', None],
  [ 0x6bb9, 'CjkUnifiedIdeograph6bb9', [], 'Display', None],
  [ 0x6bba, 'CjkUnifiedIdeograph6bba', [], 'Display', None],
  [ 0x6bbb, 'CjkUnifiedIdeograph6bbb', [], 'Display', None],
  [ 0x6bbc, 'CjkUnifiedIdeograph6bbc', [], 'Display', None],
  [ 0x6bbd, 'CjkUnifiedIdeograph6bbd', [], 'Display', None],
  [ 0x6bbe, 'CjkUnifiedIdeograph6bbe', [], 'Display', None],
  [ 0x6bbf, 'CjkUnifiedIdeograph6bbf', [], 'Display', None],
  [ 0x6bc0, 'CjkUnifiedIdeograph6bc0', [], 'Display', None],
  [ 0x6bc1, 'CjkUnifiedIdeograph6bc1', [], 'Display', None],
  [ 0x6bc2, 'CjkUnifiedIdeograph6bc2', [], 'Display', None],
  [ 0x6bc3, 'CjkUnifiedIdeograph6bc3', [], 'Display', None],
  [ 0x6bc4, 'CjkUnifiedIdeograph6bc4', [], 'Display', None],
  [ 0x6bc5, 'CjkUnifiedIdeograph6bc5', [], 'Display', None],
  [ 0x6bc6, 'CjkUnifiedIdeograph6bc6', [], 'Display', None],
  [ 0x6bc7, 'CjkUnifiedIdeograph6bc7', [], 'Display', None],
  [ 0x6bc8, 'CjkUnifiedIdeograph6bc8', [], 'Display', None],
  [ 0x6bc9, 'CjkUnifiedIdeograph6bc9', [], 'Display', None],
  [ 0x6bca, 'CjkUnifiedIdeograph6bca', [], 'Display', None],
  [ 0x6bcb, 'CjkUnifiedIdeograph6bcb', [], 'Display', None],
  [ 0x6bcc, 'CjkUnifiedIdeograph6bcc', [], 'Display', None],
  [ 0x6bcd, 'CjkUnifiedIdeograph6bcd', [], 'Display', None],
  [ 0x6bce, 'CjkUnifiedIdeograph6bce', [], 'Display', None],
  [ 0x6bcf, 'CjkUnifiedIdeograph6bcf', [], 'Display', None],
  [ 0x6bd0, 'CjkUnifiedIdeograph6bd0', [], 'Display', None],
  [ 0x6bd1, 'CjkUnifiedIdeograph6bd1', [], 'Display', None],
  [ 0x6bd2, 'CjkUnifiedIdeograph6bd2', [], 'Display', None],
  [ 0x6bd3, 'CjkUnifiedIdeograph6bd3', [], 'Display', None],
  [ 0x6bd4, 'CjkUnifiedIdeograph6bd4', [], 'Display', None],
  [ 0x6bd5, 'CjkUnifiedIdeograph6bd5', [], 'Display', None],
  [ 0x6bd6, 'CjkUnifiedIdeograph6bd6', [], 'Display', None],
  [ 0x6bd7, 'CjkUnifiedIdeograph6bd7', [], 'Display', None],
  [ 0x6bd8, 'CjkUnifiedIdeograph6bd8', [], 'Display', None],
  [ 0x6bd9, 'CjkUnifiedIdeograph6bd9', [], 'Display', None],
  [ 0x6bda, 'CjkUnifiedIdeograph6bda', [], 'Display', None],
  [ 0x6bdb, 'CjkUnifiedIdeograph6bdb', [], 'Display', None],
  [ 0x6bdc, 'CjkUnifiedIdeograph6bdc', [], 'Display', None],
  [ 0x6bdd, 'CjkUnifiedIdeograph6bdd', [], 'Display', None],
  [ 0x6bde, 'CjkUnifiedIdeograph6bde', [], 'Display', None],
  [ 0x6bdf, 'CjkUnifiedIdeograph6bdf', [], 'Display', None],
  [ 0x6be0, 'CjkUnifiedIdeograph6be0', [], 'Display', None],
  [ 0x6be1, 'CjkUnifiedIdeograph6be1', [], 'Display', None],
  [ 0x6be2, 'CjkUnifiedIdeograph6be2', [], 'Display', None],
  [ 0x6be3, 'CjkUnifiedIdeograph6be3', [], 'Display', None],
  [ 0x6be4, 'CjkUnifiedIdeograph6be4', [], 'Display', None],
  [ 0x6be5, 'CjkUnifiedIdeograph6be5', [], 'Display', None],
  [ 0x6be6, 'CjkUnifiedIdeograph6be6', [], 'Display', None],
  [ 0x6be7, 'CjkUnifiedIdeograph6be7', [], 'Display', None],
  [ 0x6be8, 'CjkUnifiedIdeograph6be8', [], 'Display', None],
  [ 0x6be9, 'CjkUnifiedIdeograph6be9', [], 'Display', None],
  [ 0x6bea, 'CjkUnifiedIdeograph6bea', [], 'Display', None],
  [ 0x6beb, 'CjkUnifiedIdeograph6beb', [], 'Display', None],
  [ 0x6bec, 'CjkUnifiedIdeograph6bec', [], 'Display', None],
  [ 0x6bed, 'CjkUnifiedIdeograph6bed', [], 'Display', None],
  [ 0x6bee, 'CjkUnifiedIdeograph6bee', [], 'Display', None],
  [ 0x6bef, 'CjkUnifiedIdeograph6bef', [], 'Display', None],
  [ 0x6bf0, 'CjkUnifiedIdeograph6bf0', [], 'Display', None],
  [ 0x6bf1, 'CjkUnifiedIdeograph6bf1', [], 'Display', None],
  [ 0x6bf2, 'CjkUnifiedIdeograph6bf2', [], 'Display', None],
  [ 0x6bf3, 'CjkUnifiedIdeograph6bf3', [], 'Display', None],
  [ 0x6bf4, 'CjkUnifiedIdeograph6bf4', [], 'Display', None],
  [ 0x6bf5, 'CjkUnifiedIdeograph6bf5', [], 'Display', None],
  [ 0x6bf6, 'CjkUnifiedIdeograph6bf6', [], 'Display', None],
  [ 0x6bf7, 'CjkUnifiedIdeograph6bf7', [], 'Display', None],
  [ 0x6bf8, 'CjkUnifiedIdeograph6bf8', [], 'Display', None],
  [ 0x6bf9, 'CjkUnifiedIdeograph6bf9', [], 'Display', None],
  [ 0x6bfa, 'CjkUnifiedIdeograph6bfa', [], 'Display', None],
  [ 0x6bfb, 'CjkUnifiedIdeograph6bfb', [], 'Display', None],
  [ 0x6bfc, 'CjkUnifiedIdeograph6bfc', [], 'Display', None],
  [ 0x6bfd, 'CjkUnifiedIdeograph6bfd', [], 'Display', None],
  [ 0x6bfe, 'CjkUnifiedIdeograph6bfe', [], 'Display', None],
  [ 0x6bff, 'CjkUnifiedIdeograph6bff', [], 'Display', None],
  [ 0x6c00, 'CjkUnifiedIdeograph6c00', [], 'Display', None],
  [ 0x6c01, 'CjkUnifiedIdeograph6c01', [], 'Display', None],
  [ 0x6c02, 'CjkUnifiedIdeograph6c02', [], 'Display', None],
  [ 0x6c03, 'CjkUnifiedIdeograph6c03', [], 'Display', None],
  [ 0x6c04, 'CjkUnifiedIdeograph6c04', [], 'Display', None],
  [ 0x6c05, 'CjkUnifiedIdeograph6c05', [], 'Display', None],
  [ 0x6c06, 'CjkUnifiedIdeograph6c06', [], 'Display', None],
  [ 0x6c07, 'CjkUnifiedIdeograph6c07', [], 'Display', None],
  [ 0x6c08, 'CjkUnifiedIdeograph6c08', [], 'Display', None],
  [ 0x6c09, 'CjkUnifiedIdeograph6c09', [], 'Display', None],
  [ 0x6c0a, 'CjkUnifiedIdeograph6c0a', [], 'Display', None],
  [ 0x6c0b, 'CjkUnifiedIdeograph6c0b', [], 'Display', None],
  [ 0x6c0c, 'CjkUnifiedIdeograph6c0c', [], 'Display', None],
  [ 0x6c0d, 'CjkUnifiedIdeograph6c0d', [], 'Display', None],
  [ 0x6c0e, 'CjkUnifiedIdeograph6c0e', [], 'Display', None],
  [ 0x6c0f, 'CjkUnifiedIdeograph6c0f', [], 'Display', None],
  [ 0x6c10, 'CjkUnifiedIdeograph6c10', [], 'Display', None],
  [ 0x6c11, 'CjkUnifiedIdeograph6c11', [], 'Display', None],
  [ 0x6c12, 'CjkUnifiedIdeograph6c12', [], 'Display', None],
  [ 0x6c13, 'CjkUnifiedIdeograph6c13', [], 'Display', None],
  [ 0x6c14, 'CjkUnifiedIdeograph6c14', [], 'Display', None],
  [ 0x6c15, 'CjkUnifiedIdeograph6c15', [], 'Display', None],
  [ 0x6c16, 'CjkUnifiedIdeograph6c16', [], 'Display', None],
  [ 0x6c17, 'CjkUnifiedIdeograph6c17', [], 'Display', None],
  [ 0x6c18, 'CjkUnifiedIdeograph6c18', [], 'Display', None],
  [ 0x6c19, 'CjkUnifiedIdeograph6c19', [], 'Display', None],
  [ 0x6c1a, 'CjkUnifiedIdeograph6c1a', [], 'Display', None],
  [ 0x6c1b, 'CjkUnifiedIdeograph6c1b', [], 'Display', None],
  [ 0x6c1c, 'CjkUnifiedIdeograph6c1c', [], 'Display', None],
  [ 0x6c1d, 'CjkUnifiedIdeograph6c1d', [], 'Display', None],
  [ 0x6c1e, 'CjkUnifiedIdeograph6c1e', [], 'Display', None],
  [ 0x6c1f, 'CjkUnifiedIdeograph6c1f', [], 'Display', None],
  [ 0x6c20, 'CjkUnifiedIdeograph6c20', [], 'Display', None],
  [ 0x6c21, 'CjkUnifiedIdeograph6c21', [], 'Display', None],
  [ 0x6c22, 'CjkUnifiedIdeograph6c22', [], 'Display', None],
  [ 0x6c23, 'CjkUnifiedIdeograph6c23', [], 'Display', None],
  [ 0x6c24, 'CjkUnifiedIdeograph6c24', [], 'Display', None],
  [ 0x6c25, 'CjkUnifiedIdeograph6c25', [], 'Display', None],
  [ 0x6c26, 'CjkUnifiedIdeograph6c26', [], 'Display', None],
  [ 0x6c27, 'CjkUnifiedIdeograph6c27', [], 'Display', None],
  [ 0x6c28, 'CjkUnifiedIdeograph6c28', [], 'Display', None],
  [ 0x6c29, 'CjkUnifiedIdeograph6c29', [], 'Display', None],
  [ 0x6c2a, 'CjkUnifiedIdeograph6c2a', [], 'Display', None],
  [ 0x6c2b, 'CjkUnifiedIdeograph6c2b', [], 'Display', None],
  [ 0x6c2c, 'CjkUnifiedIdeograph6c2c', [], 'Display', None],
  [ 0x6c2d, 'CjkUnifiedIdeograph6c2d', [], 'Display', None],
  [ 0x6c2e, 'CjkUnifiedIdeograph6c2e', [], 'Display', None],
  [ 0x6c2f, 'CjkUnifiedIdeograph6c2f', [], 'Display', None],
  [ 0x6c30, 'CjkUnifiedIdeograph6c30', [], 'Display', None],
  [ 0x6c31, 'CjkUnifiedIdeograph6c31', [], 'Display', None],
  [ 0x6c32, 'CjkUnifiedIdeograph6c32', [], 'Display', None],
  [ 0x6c33, 'CjkUnifiedIdeograph6c33', [], 'Display', None],
  [ 0x6c34, 'CjkUnifiedIdeograph6c34', [], 'Display', None],
  [ 0x6c35, 'CjkUnifiedIdeograph6c35', [], 'Display', None],
  [ 0x6c36, 'CjkUnifiedIdeograph6c36', [], 'Display', None],
  [ 0x6c37, 'CjkUnifiedIdeograph6c37', [], 'Display', None],
  [ 0x6c38, 'CjkUnifiedIdeograph6c38', [], 'Display', None],
  [ 0x6c39, 'CjkUnifiedIdeograph6c39', [], 'Display', None],
  [ 0x6c3a, 'CjkUnifiedIdeograph6c3a', [], 'Display', None],
  [ 0x6c3b, 'CjkUnifiedIdeograph6c3b', [], 'Display', None],
  [ 0x6c3c, 'CjkUnifiedIdeograph6c3c', [], 'Display', None],
  [ 0x6c3d, 'CjkUnifiedIdeograph6c3d', [], 'Display', None],
  [ 0x6c3e, 'CjkUnifiedIdeograph6c3e', [], 'Display', None],
  [ 0x6c3f, 'CjkUnifiedIdeograph6c3f', [], 'Display', None],
  [ 0x6c40, 'CjkUnifiedIdeograph6c40', [], 'Display', None],
  [ 0x6c41, 'CjkUnifiedIdeograph6c41', [], 'Display', None],
  [ 0x6c42, 'CjkUnifiedIdeograph6c42', [], 'Display', None],
  [ 0x6c43, 'CjkUnifiedIdeograph6c43', [], 'Display', None],
  [ 0x6c44, 'CjkUnifiedIdeograph6c44', [], 'Display', None],
  [ 0x6c45, 'CjkUnifiedIdeograph6c45', [], 'Display', None],
  [ 0x6c46, 'CjkUnifiedIdeograph6c46', [], 'Display', None],
  [ 0x6c47, 'CjkUnifiedIdeograph6c47', [], 'Display', None],
  [ 0x6c48, 'CjkUnifiedIdeograph6c48', [], 'Display', None],
  [ 0x6c49, 'CjkUnifiedIdeograph6c49', [], 'Display', None],
  [ 0x6c4a, 'CjkUnifiedIdeograph6c4a', [], 'Display', None],
  [ 0x6c4b, 'CjkUnifiedIdeograph6c4b', [], 'Display', None],
  [ 0x6c4c, 'CjkUnifiedIdeograph6c4c', [], 'Display', None],
  [ 0x6c4d, 'CjkUnifiedIdeograph6c4d', [], 'Display', None],
  [ 0x6c4e, 'CjkUnifiedIdeograph6c4e', [], 'Display', None],
  [ 0x6c4f, 'CjkUnifiedIdeograph6c4f', [], 'Display', None],
  [ 0x6c50, 'CjkUnifiedIdeograph6c50', [], 'Display', None],
  [ 0x6c51, 'CjkUnifiedIdeograph6c51', [], 'Display', None],
  [ 0x6c52, 'CjkUnifiedIdeograph6c52', [], 'Display', None],
  [ 0x6c53, 'CjkUnifiedIdeograph6c53', [], 'Display', None],
  [ 0x6c54, 'CjkUnifiedIdeograph6c54', [], 'Display', None],
  [ 0x6c55, 'CjkUnifiedIdeograph6c55', [], 'Display', None],
  [ 0x6c56, 'CjkUnifiedIdeograph6c56', [], 'Display', None],
  [ 0x6c57, 'CjkUnifiedIdeograph6c57', [], 'Display', None],
  [ 0x6c58, 'CjkUnifiedIdeograph6c58', [], 'Display', None],
  [ 0x6c59, 'CjkUnifiedIdeograph6c59', [], 'Display', None],
  [ 0x6c5a, 'CjkUnifiedIdeograph6c5a', [], 'Display', None],
  [ 0x6c5b, 'CjkUnifiedIdeograph6c5b', [], 'Display', None],
  [ 0x6c5c, 'CjkUnifiedIdeograph6c5c', [], 'Display', None],
  [ 0x6c5d, 'CjkUnifiedIdeograph6c5d', [], 'Display', None],
  [ 0x6c5e, 'CjkUnifiedIdeograph6c5e', [], 'Display', None],
  [ 0x6c5f, 'CjkUnifiedIdeograph6c5f', [], 'Display', None],
  [ 0x6c60, 'CjkUnifiedIdeograph6c60', [], 'Display', None],
  [ 0x6c61, 'CjkUnifiedIdeograph6c61', [], 'Display', None],
  [ 0x6c62, 'CjkUnifiedIdeograph6c62', [], 'Display', None],
  [ 0x6c63, 'CjkUnifiedIdeograph6c63', [], 'Display', None],
  [ 0x6c64, 'CjkUnifiedIdeograph6c64', [], 'Display', None],
  [ 0x6c65, 'CjkUnifiedIdeograph6c65', [], 'Display', None],
  [ 0x6c66, 'CjkUnifiedIdeograph6c66', [], 'Display', None],
  [ 0x6c67, 'CjkUnifiedIdeograph6c67', [], 'Display', None],
  [ 0x6c68, 'CjkUnifiedIdeograph6c68', [], 'Display', None],
  [ 0x6c69, 'CjkUnifiedIdeograph6c69', [], 'Display', None],
  [ 0x6c6a, 'CjkUnifiedIdeograph6c6a', [], 'Display', None],
  [ 0x6c6b, 'CjkUnifiedIdeograph6c6b', [], 'Display', None],
  [ 0x6c6c, 'CjkUnifiedIdeograph6c6c', [], 'Display', None],
  [ 0x6c6d, 'CjkUnifiedIdeograph6c6d', [], 'Display', None],
  [ 0x6c6e, 'CjkUnifiedIdeograph6c6e', [], 'Display', None],
  [ 0x6c6f, 'CjkUnifiedIdeograph6c6f', [], 'Display', None],
  [ 0x6c70, 'CjkUnifiedIdeograph6c70', [], 'Display', None],
  [ 0x6c71, 'CjkUnifiedIdeograph6c71', [], 'Display', None],
  [ 0x6c72, 'CjkUnifiedIdeograph6c72', [], 'Display', None],
  [ 0x6c73, 'CjkUnifiedIdeograph6c73', [], 'Display', None],
  [ 0x6c74, 'CjkUnifiedIdeograph6c74', [], 'Display', None],
  [ 0x6c75, 'CjkUnifiedIdeograph6c75', [], 'Display', None],
  [ 0x6c76, 'CjkUnifiedIdeograph6c76', [], 'Display', None],
  [ 0x6c77, 'CjkUnifiedIdeograph6c77', [], 'Display', None],
  [ 0x6c78, 'CjkUnifiedIdeograph6c78', [], 'Display', None],
  [ 0x6c79, 'CjkUnifiedIdeograph6c79', [], 'Display', None],
  [ 0x6c7a, 'CjkUnifiedIdeograph6c7a', [], 'Display', None],
  [ 0x6c7b, 'CjkUnifiedIdeograph6c7b', [], 'Display', None],
  [ 0x6c7c, 'CjkUnifiedIdeograph6c7c', [], 'Display', None],
  [ 0x6c7d, 'CjkUnifiedIdeograph6c7d', [], 'Display', None],
  [ 0x6c7e, 'CjkUnifiedIdeograph6c7e', [], 'Display', None],
  [ 0x6c7f, 'CjkUnifiedIdeograph6c7f', [], 'Display', None],
  [ 0x6c80, 'CjkUnifiedIdeograph6c80', [], 'Display', None],
  [ 0x6c81, 'CjkUnifiedIdeograph6c81', [], 'Display', None],
  [ 0x6c82, 'CjkUnifiedIdeograph6c82', [], 'Display', None],
  [ 0x6c83, 'CjkUnifiedIdeograph6c83', [], 'Display', None],
  [ 0x6c84, 'CjkUnifiedIdeograph6c84', [], 'Display', None],
  [ 0x6c85, 'CjkUnifiedIdeograph6c85', [], 'Display', None],
  [ 0x6c86, 'CjkUnifiedIdeograph6c86', [], 'Display', None],
  [ 0x6c87, 'CjkUnifiedIdeograph6c87', [], 'Display', None],
  [ 0x6c88, 'CjkUnifiedIdeograph6c88', [], 'Display', None],
  [ 0x6c89, 'CjkUnifiedIdeograph6c89', [], 'Display', None],
  [ 0x6c8a, 'CjkUnifiedIdeograph6c8a', [], 'Display', None],
  [ 0x6c8b, 'CjkUnifiedIdeograph6c8b', [], 'Display', None],
  [ 0x6c8c, 'CjkUnifiedIdeograph6c8c', [], 'Display', None],
  [ 0x6c8d, 'CjkUnifiedIdeograph6c8d', [], 'Display', None],
  [ 0x6c8e, 'CjkUnifiedIdeograph6c8e', [], 'Display', None],
  [ 0x6c8f, 'CjkUnifiedIdeograph6c8f', [], 'Display', None],
  [ 0x6c90, 'CjkUnifiedIdeograph6c90', [], 'Display', None],
  [ 0x6c91, 'CjkUnifiedIdeograph6c91', [], 'Display', None],
  [ 0x6c92, 'CjkUnifiedIdeograph6c92', [], 'Display', None],
  [ 0x6c93, 'CjkUnifiedIdeograph6c93', [], 'Display', None],
  [ 0x6c94, 'CjkUnifiedIdeograph6c94', [], 'Display', None],
  [ 0x6c95, 'CjkUnifiedIdeograph6c95', [], 'Display', None],
  [ 0x6c96, 'CjkUnifiedIdeograph6c96', [], 'Display', None],
  [ 0x6c97, 'CjkUnifiedIdeograph6c97', [], 'Display', None],
  [ 0x6c98, 'CjkUnifiedIdeograph6c98', [], 'Display', None],
  [ 0x6c99, 'CjkUnifiedIdeograph6c99', [], 'Display', None],
  [ 0x6c9a, 'CjkUnifiedIdeograph6c9a', [], 'Display', None],
  [ 0x6c9b, 'CjkUnifiedIdeograph6c9b', [], 'Display', None],
  [ 0x6c9c, 'CjkUnifiedIdeograph6c9c', [], 'Display', None],
  [ 0x6c9d, 'CjkUnifiedIdeograph6c9d', [], 'Display', None],
  [ 0x6c9e, 'CjkUnifiedIdeograph6c9e', [], 'Display', None],
  [ 0x6c9f, 'CjkUnifiedIdeograph6c9f', [], 'Display', None],
  [ 0x6ca0, 'CjkUnifiedIdeograph6ca0', [], 'Display', None],
  [ 0x6ca1, 'CjkUnifiedIdeograph6ca1', [], 'Display', None],
  [ 0x6ca2, 'CjkUnifiedIdeograph6ca2', [], 'Display', None],
  [ 0x6ca3, 'CjkUnifiedIdeograph6ca3', [], 'Display', None],
  [ 0x6ca4, 'CjkUnifiedIdeograph6ca4', [], 'Display', None],
  [ 0x6ca5, 'CjkUnifiedIdeograph6ca5', [], 'Display', None],
  [ 0x6ca6, 'CjkUnifiedIdeograph6ca6', [], 'Display', None],
  [ 0x6ca7, 'CjkUnifiedIdeograph6ca7', [], 'Display', None],
  [ 0x6ca8, 'CjkUnifiedIdeograph6ca8', [], 'Display', None],
  [ 0x6ca9, 'CjkUnifiedIdeograph6ca9', [], 'Display', None],
  [ 0x6caa, 'CjkUnifiedIdeograph6caa', [], 'Display', None],
  [ 0x6cab, 'CjkUnifiedIdeograph6cab', [], 'Display', None],
  [ 0x6cac, 'CjkUnifiedIdeograph6cac', [], 'Display', None],
  [ 0x6cad, 'CjkUnifiedIdeograph6cad', [], 'Display', None],
  [ 0x6cae, 'CjkUnifiedIdeograph6cae', [], 'Display', None],
  [ 0x6caf, 'CjkUnifiedIdeograph6caf', [], 'Display', None],
  [ 0x6cb0, 'CjkUnifiedIdeograph6cb0', [], 'Display', None],
  [ 0x6cb1, 'CjkUnifiedIdeograph6cb1', [], 'Display', None],
  [ 0x6cb2, 'CjkUnifiedIdeograph6cb2', [], 'Display', None],
  [ 0x6cb3, 'CjkUnifiedIdeograph6cb3', [], 'Display', None],
  [ 0x6cb4, 'CjkUnifiedIdeograph6cb4', [], 'Display', None],
  [ 0x6cb5, 'CjkUnifiedIdeograph6cb5', [], 'Display', None],
  [ 0x6cb6, 'CjkUnifiedIdeograph6cb6', [], 'Display', None],
  [ 0x6cb7, 'CjkUnifiedIdeograph6cb7', [], 'Display', None],
  [ 0x6cb8, 'CjkUnifiedIdeograph6cb8', [], 'Display', None],
  [ 0x6cb9, 'CjkUnifiedIdeograph6cb9', [], 'Display', None],
  [ 0x6cba, 'CjkUnifiedIdeograph6cba', [], 'Display', None],
  [ 0x6cbb, 'CjkUnifiedIdeograph6cbb', [], 'Display', None],
  [ 0x6cbc, 'CjkUnifiedIdeograph6cbc', [], 'Display', None],
  [ 0x6cbd, 'CjkUnifiedIdeograph6cbd', [], 'Display', None],
  [ 0x6cbe, 'CjkUnifiedIdeograph6cbe', [], 'Display', None],
  [ 0x6cbf, 'CjkUnifiedIdeograph6cbf', [], 'Display', None],
  [ 0x6cc0, 'CjkUnifiedIdeograph6cc0', [], 'Display', None],
  [ 0x6cc1, 'CjkUnifiedIdeograph6cc1', [], 'Display', None],
  [ 0x6cc2, 'CjkUnifiedIdeograph6cc2', [], 'Display', None],
  [ 0x6cc3, 'CjkUnifiedIdeograph6cc3', [], 'Display', None],
  [ 0x6cc4, 'CjkUnifiedIdeograph6cc4', [], 'Display', None],
  [ 0x6cc5, 'CjkUnifiedIdeograph6cc5', [], 'Display', None],
  [ 0x6cc6, 'CjkUnifiedIdeograph6cc6', [], 'Display', None],
  [ 0x6cc7, 'CjkUnifiedIdeograph6cc7', [], 'Display', None],
  [ 0x6cc8, 'CjkUnifiedIdeograph6cc8', [], 'Display', None],
  [ 0x6cc9, 'CjkUnifiedIdeograph6cc9', [], 'Display', None],
  [ 0x6cca, 'CjkUnifiedIdeograph6cca', [], 'Display', None],
  [ 0x6ccb, 'CjkUnifiedIdeograph6ccb', [], 'Display', None],
  [ 0x6ccc, 'CjkUnifiedIdeograph6ccc', [], 'Display', None],
  [ 0x6ccd, 'CjkUnifiedIdeograph6ccd', [], 'Display', None],
  [ 0x6cce, 'CjkUnifiedIdeograph6cce', [], 'Display', None],
  [ 0x6ccf, 'CjkUnifiedIdeograph6ccf', [], 'Display', None],
  [ 0x6cd0, 'CjkUnifiedIdeograph6cd0', [], 'Display', None],
  [ 0x6cd1, 'CjkUnifiedIdeograph6cd1', [], 'Display', None],
  [ 0x6cd2, 'CjkUnifiedIdeograph6cd2', [], 'Display', None],
  [ 0x6cd3, 'CjkUnifiedIdeograph6cd3', [], 'Display', None],
  [ 0x6cd4, 'CjkUnifiedIdeograph6cd4', [], 'Display', None],
  [ 0x6cd5, 'CjkUnifiedIdeograph6cd5', [], 'Display', None],
  [ 0x6cd6, 'CjkUnifiedIdeograph6cd6', [], 'Display', None],
  [ 0x6cd7, 'CjkUnifiedIdeograph6cd7', [], 'Display', None],
  [ 0x6cd8, 'CjkUnifiedIdeograph6cd8', [], 'Display', None],
  [ 0x6cd9, 'CjkUnifiedIdeograph6cd9', [], 'Display', None],
  [ 0x6cda, 'CjkUnifiedIdeograph6cda', [], 'Display', None],
  [ 0x6cdb, 'CjkUnifiedIdeograph6cdb', [], 'Display', None],
  [ 0x6cdc, 'CjkUnifiedIdeograph6cdc', [], 'Display', None],
  [ 0x6cdd, 'CjkUnifiedIdeograph6cdd', [], 'Display', None],
  [ 0x6cde, 'CjkUnifiedIdeograph6cde', [], 'Display', None],
  [ 0x6cdf, 'CjkUnifiedIdeograph6cdf', [], 'Display', None],
  [ 0x6ce0, 'CjkUnifiedIdeograph6ce0', [], 'Display', None],
  [ 0x6ce1, 'CjkUnifiedIdeograph6ce1', [], 'Display', None],
  [ 0x6ce2, 'CjkUnifiedIdeograph6ce2', [], 'Display', None],
  [ 0x6ce3, 'CjkUnifiedIdeograph6ce3', [], 'Display', None],
  [ 0x6ce4, 'CjkUnifiedIdeograph6ce4', [], 'Display', None],
  [ 0x6ce5, 'CjkUnifiedIdeograph6ce5', [], 'Display', None],
  [ 0x6ce6, 'CjkUnifiedIdeograph6ce6', [], 'Display', None],
  [ 0x6ce7, 'CjkUnifiedIdeograph6ce7', [], 'Display', None],
  [ 0x6ce8, 'CjkUnifiedIdeograph6ce8', [], 'Display', None],
  [ 0x6ce9, 'CjkUnifiedIdeograph6ce9', [], 'Display', None],
  [ 0x6cea, 'CjkUnifiedIdeograph6cea', [], 'Display', None],
  [ 0x6ceb, 'CjkUnifiedIdeograph6ceb', [], 'Display', None],
  [ 0x6cec, 'CjkUnifiedIdeograph6cec', [], 'Display', None],
  [ 0x6ced, 'CjkUnifiedIdeograph6ced', [], 'Display', None],
  [ 0x6cee, 'CjkUnifiedIdeograph6cee', [], 'Display', None],
  [ 0x6cef, 'CjkUnifiedIdeograph6cef', [], 'Display', None],
  [ 0x6cf0, 'CjkUnifiedIdeograph6cf0', [], 'Display', None],
  [ 0x6cf1, 'CjkUnifiedIdeograph6cf1', [], 'Display', None],
  [ 0x6cf2, 'CjkUnifiedIdeograph6cf2', [], 'Display', None],
  [ 0x6cf3, 'CjkUnifiedIdeograph6cf3', [], 'Display', None],
  [ 0x6cf4, 'CjkUnifiedIdeograph6cf4', [], 'Display', None],
  [ 0x6cf5, 'CjkUnifiedIdeograph6cf5', [], 'Display', None],
  [ 0x6cf6, 'CjkUnifiedIdeograph6cf6', [], 'Display', None],
  [ 0x6cf7, 'CjkUnifiedIdeograph6cf7', [], 'Display', None],
  [ 0x6cf8, 'CjkUnifiedIdeograph6cf8', [], 'Display', None],
  [ 0x6cf9, 'CjkUnifiedIdeograph6cf9', [], 'Display', None],
  [ 0x6cfa, 'CjkUnifiedIdeograph6cfa', [], 'Display', None],
  [ 0x6cfb, 'CjkUnifiedIdeograph6cfb', [], 'Display', None],
  [ 0x6cfc, 'CjkUnifiedIdeograph6cfc', [], 'Display', None],
  [ 0x6cfd, 'CjkUnifiedIdeograph6cfd', [], 'Display', None],
  [ 0x6cfe, 'CjkUnifiedIdeograph6cfe', [], 'Display', None],
  [ 0x6cff, 'CjkUnifiedIdeograph6cff', [], 'Display', None],
  [ 0x6d00, 'CjkUnifiedIdeograph6d00', [], 'Display', None],
  [ 0x6d01, 'CjkUnifiedIdeograph6d01', [], 'Display', None],
  [ 0x6d02, 'CjkUnifiedIdeograph6d02', [], 'Display', None],
  [ 0x6d03, 'CjkUnifiedIdeograph6d03', [], 'Display', None],
  [ 0x6d04, 'CjkUnifiedIdeograph6d04', [], 'Display', None],
  [ 0x6d05, 'CjkUnifiedIdeograph6d05', [], 'Display', None],
  [ 0x6d06, 'CjkUnifiedIdeograph6d06', [], 'Display', None],
  [ 0x6d07, 'CjkUnifiedIdeograph6d07', [], 'Display', None],
  [ 0x6d08, 'CjkUnifiedIdeograph6d08', [], 'Display', None],
  [ 0x6d09, 'CjkUnifiedIdeograph6d09', [], 'Display', None],
  [ 0x6d0a, 'CjkUnifiedIdeograph6d0a', [], 'Display', None],
  [ 0x6d0b, 'CjkUnifiedIdeograph6d0b', [], 'Display', None],
  [ 0x6d0c, 'CjkUnifiedIdeograph6d0c', [], 'Display', None],
  [ 0x6d0d, 'CjkUnifiedIdeograph6d0d', [], 'Display', None],
  [ 0x6d0e, 'CjkUnifiedIdeograph6d0e', [], 'Display', None],
  [ 0x6d0f, 'CjkUnifiedIdeograph6d0f', [], 'Display', None],
  [ 0x6d10, 'CjkUnifiedIdeograph6d10', [], 'Display', None],
  [ 0x6d11, 'CjkUnifiedIdeograph6d11', [], 'Display', None],
  [ 0x6d12, 'CjkUnifiedIdeograph6d12', [], 'Display', None],
  [ 0x6d13, 'CjkUnifiedIdeograph6d13', [], 'Display', None],
  [ 0x6d14, 'CjkUnifiedIdeograph6d14', [], 'Display', None],
  [ 0x6d15, 'CjkUnifiedIdeograph6d15', [], 'Display', None],
  [ 0x6d16, 'CjkUnifiedIdeograph6d16', [], 'Display', None],
  [ 0x6d17, 'CjkUnifiedIdeograph6d17', [], 'Display', None],
  [ 0x6d18, 'CjkUnifiedIdeograph6d18', [], 'Display', None],
  [ 0x6d19, 'CjkUnifiedIdeograph6d19', [], 'Display', None],
  [ 0x6d1a, 'CjkUnifiedIdeograph6d1a', [], 'Display', None],
  [ 0x6d1b, 'CjkUnifiedIdeograph6d1b', [], 'Display', None],
  [ 0x6d1c, 'CjkUnifiedIdeograph6d1c', [], 'Display', None],
  [ 0x6d1d, 'CjkUnifiedIdeograph6d1d', [], 'Display', None],
  [ 0x6d1e, 'CjkUnifiedIdeograph6d1e', [], 'Display', None],
  [ 0x6d1f, 'CjkUnifiedIdeograph6d1f', [], 'Display', None],
  [ 0x6d20, 'CjkUnifiedIdeograph6d20', [], 'Display', None],
  [ 0x6d21, 'CjkUnifiedIdeograph6d21', [], 'Display', None],
  [ 0x6d22, 'CjkUnifiedIdeograph6d22', [], 'Display', None],
  [ 0x6d23, 'CjkUnifiedIdeograph6d23', [], 'Display', None],
  [ 0x6d24, 'CjkUnifiedIdeograph6d24', [], 'Display', None],
  [ 0x6d25, 'CjkUnifiedIdeograph6d25', [], 'Display', None],
  [ 0x6d26, 'CjkUnifiedIdeograph6d26', [], 'Display', None],
  [ 0x6d27, 'CjkUnifiedIdeograph6d27', [], 'Display', None],
  [ 0x6d28, 'CjkUnifiedIdeograph6d28', [], 'Display', None],
  [ 0x6d29, 'CjkUnifiedIdeograph6d29', [], 'Display', None],
  [ 0x6d2a, 'CjkUnifiedIdeograph6d2a', [], 'Display', None],
  [ 0x6d2b, 'CjkUnifiedIdeograph6d2b', [], 'Display', None],
  [ 0x6d2c, 'CjkUnifiedIdeograph6d2c', [], 'Display', None],
  [ 0x6d2d, 'CjkUnifiedIdeograph6d2d', [], 'Display', None],
  [ 0x6d2e, 'CjkUnifiedIdeograph6d2e', [], 'Display', None],
  [ 0x6d2f, 'CjkUnifiedIdeograph6d2f', [], 'Display', None],
  [ 0x6d30, 'CjkUnifiedIdeograph6d30', [], 'Display', None],
  [ 0x6d31, 'CjkUnifiedIdeograph6d31', [], 'Display', None],
  [ 0x6d32, 'CjkUnifiedIdeograph6d32', [], 'Display', None],
  [ 0x6d33, 'CjkUnifiedIdeograph6d33', [], 'Display', None],
  [ 0x6d34, 'CjkUnifiedIdeograph6d34', [], 'Display', None],
  [ 0x6d35, 'CjkUnifiedIdeograph6d35', [], 'Display', None],
  [ 0x6d36, 'CjkUnifiedIdeograph6d36', [], 'Display', None],
  [ 0x6d37, 'CjkUnifiedIdeograph6d37', [], 'Display', None],
  [ 0x6d38, 'CjkUnifiedIdeograph6d38', [], 'Display', None],
  [ 0x6d39, 'CjkUnifiedIdeograph6d39', [], 'Display', None],
  [ 0x6d3a, 'CjkUnifiedIdeograph6d3a', [], 'Display', None],
  [ 0x6d3b, 'CjkUnifiedIdeograph6d3b', [], 'Display', None],
  [ 0x6d3c, 'CjkUnifiedIdeograph6d3c', [], 'Display', None],
  [ 0x6d3d, 'CjkUnifiedIdeograph6d3d', [], 'Display', None],
  [ 0x6d3e, 'CjkUnifiedIdeograph6d3e', [], 'Display', None],
  [ 0x6d3f, 'CjkUnifiedIdeograph6d3f', [], 'Display', None],
  [ 0x6d40, 'CjkUnifiedIdeograph6d40', [], 'Display', None],
  [ 0x6d41, 'CjkUnifiedIdeograph6d41', [], 'Display', None],
  [ 0x6d42, 'CjkUnifiedIdeograph6d42', [], 'Display', None],
  [ 0x6d43, 'CjkUnifiedIdeograph6d43', [], 'Display', None],
  [ 0x6d44, 'CjkUnifiedIdeograph6d44', [], 'Display', None],
  [ 0x6d45, 'CjkUnifiedIdeograph6d45', [], 'Display', None],
  [ 0x6d46, 'CjkUnifiedIdeograph6d46', [], 'Display', None],
  [ 0x6d47, 'CjkUnifiedIdeograph6d47', [], 'Display', None],
  [ 0x6d48, 'CjkUnifiedIdeograph6d48', [], 'Display', None],
  [ 0x6d49, 'CjkUnifiedIdeograph6d49', [], 'Display', None],
  [ 0x6d4a, 'CjkUnifiedIdeograph6d4a', [], 'Display', None],
  [ 0x6d4b, 'CjkUnifiedIdeograph6d4b', [], 'Display', None],
  [ 0x6d4c, 'CjkUnifiedIdeograph6d4c', [], 'Display', None],
  [ 0x6d4d, 'CjkUnifiedIdeograph6d4d', [], 'Display', None],
  [ 0x6d4e, 'CjkUnifiedIdeograph6d4e', [], 'Display', None],
  [ 0x6d4f, 'CjkUnifiedIdeograph6d4f', [], 'Display', None],
  [ 0x6d50, 'CjkUnifiedIdeograph6d50', [], 'Display', None],
  [ 0x6d51, 'CjkUnifiedIdeograph6d51', [], 'Display', None],
  [ 0x6d52, 'CjkUnifiedIdeograph6d52', [], 'Display', None],
  [ 0x6d53, 'CjkUnifiedIdeograph6d53', [], 'Display', None],
  [ 0x6d54, 'CjkUnifiedIdeograph6d54', [], 'Display', None],
  [ 0x6d55, 'CjkUnifiedIdeograph6d55', [], 'Display', None],
  [ 0x6d56, 'CjkUnifiedIdeograph6d56', [], 'Display', None],
  [ 0x6d57, 'CjkUnifiedIdeograph6d57', [], 'Display', None],
  [ 0x6d58, 'CjkUnifiedIdeograph6d58', [], 'Display', None],
  [ 0x6d59, 'CjkUnifiedIdeograph6d59', [], 'Display', None],
  [ 0x6d5a, 'CjkUnifiedIdeograph6d5a', [], 'Display', None],
  [ 0x6d5b, 'CjkUnifiedIdeograph6d5b', [], 'Display', None],
  [ 0x6d5c, 'CjkUnifiedIdeograph6d5c', [], 'Display', None],
  [ 0x6d5d, 'CjkUnifiedIdeograph6d5d', [], 'Display', None],
  [ 0x6d5e, 'CjkUnifiedIdeograph6d5e', [], 'Display', None],
  [ 0x6d5f, 'CjkUnifiedIdeograph6d5f', [], 'Display', None],
  [ 0x6d60, 'CjkUnifiedIdeograph6d60', [], 'Display', None],
  [ 0x6d61, 'CjkUnifiedIdeograph6d61', [], 'Display', None],
  [ 0x6d62, 'CjkUnifiedIdeograph6d62', [], 'Display', None],
  [ 0x6d63, 'CjkUnifiedIdeograph6d63', [], 'Display', None],
  [ 0x6d64, 'CjkUnifiedIdeograph6d64', [], 'Display', None],
  [ 0x6d65, 'CjkUnifiedIdeograph6d65', [], 'Display', None],
  [ 0x6d66, 'CjkUnifiedIdeograph6d66', [], 'Display', None],
  [ 0x6d67, 'CjkUnifiedIdeograph6d67', [], 'Display', None],
  [ 0x6d68, 'CjkUnifiedIdeograph6d68', [], 'Display', None],
  [ 0x6d69, 'CjkUnifiedIdeograph6d69', [], 'Display', None],
  [ 0x6d6a, 'CjkUnifiedIdeograph6d6a', [], 'Display', None],
  [ 0x6d6b, 'CjkUnifiedIdeograph6d6b', [], 'Display', None],
  [ 0x6d6c, 'CjkUnifiedIdeograph6d6c', [], 'Display', None],
  [ 0x6d6d, 'CjkUnifiedIdeograph6d6d', [], 'Display', None],
  [ 0x6d6e, 'CjkUnifiedIdeograph6d6e', [], 'Display', None],
  [ 0x6d6f, 'CjkUnifiedIdeograph6d6f', [], 'Display', None],
  [ 0x6d70, 'CjkUnifiedIdeograph6d70', [], 'Display', None],
  [ 0x6d71, 'CjkUnifiedIdeograph6d71', [], 'Display', None],
  [ 0x6d72, 'CjkUnifiedIdeograph6d72', [], 'Display', None],
  [ 0x6d73, 'CjkUnifiedIdeograph6d73', [], 'Display', None],
  [ 0x6d74, 'CjkUnifiedIdeograph6d74', [], 'Display', None],
  [ 0x6d75, 'CjkUnifiedIdeograph6d75', [], 'Display', None],
  [ 0x6d76, 'CjkUnifiedIdeograph6d76', [], 'Display', None],
  [ 0x6d77, 'CjkUnifiedIdeograph6d77', [], 'Display', None],
  [ 0x6d78, 'CjkUnifiedIdeograph6d78', [], 'Display', None],
  [ 0x6d79, 'CjkUnifiedIdeograph6d79', [], 'Display', None],
  [ 0x6d7a, 'CjkUnifiedIdeograph6d7a', [], 'Display', None],
  [ 0x6d7b, 'CjkUnifiedIdeograph6d7b', [], 'Display', None],
  [ 0x6d7c, 'CjkUnifiedIdeograph6d7c', [], 'Display', None],
  [ 0x6d7d, 'CjkUnifiedIdeograph6d7d', [], 'Display', None],
  [ 0x6d7e, 'CjkUnifiedIdeograph6d7e', [], 'Display', None],
  [ 0x6d7f, 'CjkUnifiedIdeograph6d7f', [], 'Display', None],
  [ 0x6d80, 'CjkUnifiedIdeograph6d80', [], 'Display', None],
  [ 0x6d81, 'CjkUnifiedIdeograph6d81', [], 'Display', None],
  [ 0x6d82, 'CjkUnifiedIdeograph6d82', [], 'Display', None],
  [ 0x6d83, 'CjkUnifiedIdeograph6d83', [], 'Display', None],
  [ 0x6d84, 'CjkUnifiedIdeograph6d84', [], 'Display', None],
  [ 0x6d85, 'CjkUnifiedIdeograph6d85', [], 'Display', None],
  [ 0x6d86, 'CjkUnifiedIdeograph6d86', [], 'Display', None],
  [ 0x6d87, 'CjkUnifiedIdeograph6d87', [], 'Display', None],
  [ 0x6d88, 'CjkUnifiedIdeograph6d88', [], 'Display', None],
  [ 0x6d89, 'CjkUnifiedIdeograph6d89', [], 'Display', None],
  [ 0x6d8a, 'CjkUnifiedIdeograph6d8a', [], 'Display', None],
  [ 0x6d8b, 'CjkUnifiedIdeograph6d8b', [], 'Display', None],
  [ 0x6d8c, 'CjkUnifiedIdeograph6d8c', [], 'Display', None],
  [ 0x6d8d, 'CjkUnifiedIdeograph6d8d', [], 'Display', None],
  [ 0x6d8e, 'CjkUnifiedIdeograph6d8e', [], 'Display', None],
  [ 0x6d8f, 'CjkUnifiedIdeograph6d8f', [], 'Display', None],
  [ 0x6d90, 'CjkUnifiedIdeograph6d90', [], 'Display', None],
  [ 0x6d91, 'CjkUnifiedIdeograph6d91', [], 'Display', None],
  [ 0x6d92, 'CjkUnifiedIdeograph6d92', [], 'Display', None],
  [ 0x6d93, 'CjkUnifiedIdeograph6d93', [], 'Display', None],
  [ 0x6d94, 'CjkUnifiedIdeograph6d94', [], 'Display', None],
  [ 0x6d95, 'CjkUnifiedIdeograph6d95', [], 'Display', None],
  [ 0x6d96, 'CjkUnifiedIdeograph6d96', [], 'Display', None],
  [ 0x6d97, 'CjkUnifiedIdeograph6d97', [], 'Display', None],
  [ 0x6d98, 'CjkUnifiedIdeograph6d98', [], 'Display', None],
  [ 0x6d99, 'CjkUnifiedIdeograph6d99', [], 'Display', None],
  [ 0x6d9a, 'CjkUnifiedIdeograph6d9a', [], 'Display', None],
  [ 0x6d9b, 'CjkUnifiedIdeograph6d9b', [], 'Display', None],
  [ 0x6d9c, 'CjkUnifiedIdeograph6d9c', [], 'Display', None],
  [ 0x6d9d, 'CjkUnifiedIdeograph6d9d', [], 'Display', None],
  [ 0x6d9e, 'CjkUnifiedIdeograph6d9e', [], 'Display', None],
  [ 0x6d9f, 'CjkUnifiedIdeograph6d9f', [], 'Display', None],
  [ 0x6da0, 'CjkUnifiedIdeograph6da0', [], 'Display', None],
  [ 0x6da1, 'CjkUnifiedIdeograph6da1', [], 'Display', None],
  [ 0x6da2, 'CjkUnifiedIdeograph6da2', [], 'Display', None],
  [ 0x6da3, 'CjkUnifiedIdeograph6da3', [], 'Display', None],
  [ 0x6da4, 'CjkUnifiedIdeograph6da4', [], 'Display', None],
  [ 0x6da5, 'CjkUnifiedIdeograph6da5', [], 'Display', None],
  [ 0x6da6, 'CjkUnifiedIdeograph6da6', [], 'Display', None],
  [ 0x6da7, 'CjkUnifiedIdeograph6da7', [], 'Display', None],
  [ 0x6da8, 'CjkUnifiedIdeograph6da8', [], 'Display', None],
  [ 0x6da9, 'CjkUnifiedIdeograph6da9', [], 'Display', None],
  [ 0x6daa, 'CjkUnifiedIdeograph6daa', [], 'Display', None],
  [ 0x6dab, 'CjkUnifiedIdeograph6dab', [], 'Display', None],
  [ 0x6dac, 'CjkUnifiedIdeograph6dac', [], 'Display', None],
  [ 0x6dad, 'CjkUnifiedIdeograph6dad', [], 'Display', None],
  [ 0x6dae, 'CjkUnifiedIdeograph6dae', [], 'Display', None],
  [ 0x6daf, 'CjkUnifiedIdeograph6daf', [], 'Display', None],
  [ 0x6db0, 'CjkUnifiedIdeograph6db0', [], 'Display', None],
  [ 0x6db1, 'CjkUnifiedIdeograph6db1', [], 'Display', None],
  [ 0x6db2, 'CjkUnifiedIdeograph6db2', [], 'Display', None],
  [ 0x6db3, 'CjkUnifiedIdeograph6db3', [], 'Display', None],
  [ 0x6db4, 'CjkUnifiedIdeograph6db4', [], 'Display', None],
  [ 0x6db5, 'CjkUnifiedIdeograph6db5', [], 'Display', None],
  [ 0x6db6, 'CjkUnifiedIdeograph6db6', [], 'Display', None],
  [ 0x6db7, 'CjkUnifiedIdeograph6db7', [], 'Display', None],
  [ 0x6db8, 'CjkUnifiedIdeograph6db8', [], 'Display', None],
  [ 0x6db9, 'CjkUnifiedIdeograph6db9', [], 'Display', None],
  [ 0x6dba, 'CjkUnifiedIdeograph6dba', [], 'Display', None],
  [ 0x6dbb, 'CjkUnifiedIdeograph6dbb', [], 'Display', None],
  [ 0x6dbc, 'CjkUnifiedIdeograph6dbc', [], 'Display', None],
  [ 0x6dbd, 'CjkUnifiedIdeograph6dbd', [], 'Display', None],
  [ 0x6dbe, 'CjkUnifiedIdeograph6dbe', [], 'Display', None],
  [ 0x6dbf, 'CjkUnifiedIdeograph6dbf', [], 'Display', None],
  [ 0x6dc0, 'CjkUnifiedIdeograph6dc0', [], 'Display', None],
  [ 0x6dc1, 'CjkUnifiedIdeograph6dc1', [], 'Display', None],
  [ 0x6dc2, 'CjkUnifiedIdeograph6dc2', [], 'Display', None],
  [ 0x6dc3, 'CjkUnifiedIdeograph6dc3', [], 'Display', None],
  [ 0x6dc4, 'CjkUnifiedIdeograph6dc4', [], 'Display', None],
  [ 0x6dc5, 'CjkUnifiedIdeograph6dc5', [], 'Display', None],
  [ 0x6dc6, 'CjkUnifiedIdeograph6dc6', [], 'Display', None],
  [ 0x6dc7, 'CjkUnifiedIdeograph6dc7', [], 'Display', None],
  [ 0x6dc8, 'CjkUnifiedIdeograph6dc8', [], 'Display', None],
  [ 0x6dc9, 'CjkUnifiedIdeograph6dc9', [], 'Display', None],
  [ 0x6dca, 'CjkUnifiedIdeograph6dca', [], 'Display', None],
  [ 0x6dcb, 'CjkUnifiedIdeograph6dcb', [], 'Display', None],
  [ 0x6dcc, 'CjkUnifiedIdeograph6dcc', [], 'Display', None],
  [ 0x6dcd, 'CjkUnifiedIdeograph6dcd', [], 'Display', None],
  [ 0x6dce, 'CjkUnifiedIdeograph6dce', [], 'Display', None],
  [ 0x6dcf, 'CjkUnifiedIdeograph6dcf', [], 'Display', None],
  [ 0x6dd0, 'CjkUnifiedIdeograph6dd0', [], 'Display', None],
  [ 0x6dd1, 'CjkUnifiedIdeograph6dd1', [], 'Display', None],
  [ 0x6dd2, 'CjkUnifiedIdeograph6dd2', [], 'Display', None],
  [ 0x6dd3, 'CjkUnifiedIdeograph6dd3', [], 'Display', None],
  [ 0x6dd4, 'CjkUnifiedIdeograph6dd4', [], 'Display', None],
  [ 0x6dd5, 'CjkUnifiedIdeograph6dd5', [], 'Display', None],
  [ 0x6dd6, 'CjkUnifiedIdeograph6dd6', [], 'Display', None],
  [ 0x6dd7, 'CjkUnifiedIdeograph6dd7', [], 'Display', None],
  [ 0x6dd8, 'CjkUnifiedIdeograph6dd8', [], 'Display', None],
  [ 0x6dd9, 'CjkUnifiedIdeograph6dd9', [], 'Display', None],
  [ 0x6dda, 'CjkUnifiedIdeograph6dda', [], 'Display', None],
  [ 0x6ddb, 'CjkUnifiedIdeograph6ddb', [], 'Display', None],
  [ 0x6ddc, 'CjkUnifiedIdeograph6ddc', [], 'Display', None],
  [ 0x6ddd, 'CjkUnifiedIdeograph6ddd', [], 'Display', None],
  [ 0x6dde, 'CjkUnifiedIdeograph6dde', [], 'Display', None],
  [ 0x6ddf, 'CjkUnifiedIdeograph6ddf', [], 'Display', None],
  [ 0x6de0, 'CjkUnifiedIdeograph6de0', [], 'Display', None],
  [ 0x6de1, 'CjkUnifiedIdeograph6de1', [], 'Display', None],
  [ 0x6de2, 'CjkUnifiedIdeograph6de2', [], 'Display', None],
  [ 0x6de3, 'CjkUnifiedIdeograph6de3', [], 'Display', None],
  [ 0x6de4, 'CjkUnifiedIdeograph6de4', [], 'Display', None],
  [ 0x6de5, 'CjkUnifiedIdeograph6de5', [], 'Display', None],
  [ 0x6de6, 'CjkUnifiedIdeograph6de6', [], 'Display', None],
  [ 0x6de7, 'CjkUnifiedIdeograph6de7', [], 'Display', None],
  [ 0x6de8, 'CjkUnifiedIdeograph6de8', [], 'Display', None],
  [ 0x6de9, 'CjkUnifiedIdeograph6de9', [], 'Display', None],
  [ 0x6dea, 'CjkUnifiedIdeograph6dea', [], 'Display', None],
  [ 0x6deb, 'CjkUnifiedIdeograph6deb', [], 'Display', None],
  [ 0x6dec, 'CjkUnifiedIdeograph6dec', [], 'Display', None],
  [ 0x6ded, 'CjkUnifiedIdeograph6ded', [], 'Display', None],
  [ 0x6dee, 'CjkUnifiedIdeograph6dee', [], 'Display', None],
  [ 0x6def, 'CjkUnifiedIdeograph6def', [], 'Display', None],
  [ 0x6df0, 'CjkUnifiedIdeograph6df0', [], 'Display', None],
  [ 0x6df1, 'CjkUnifiedIdeograph6df1', [], 'Display', None],
  [ 0x6df2, 'CjkUnifiedIdeograph6df2', [], 'Display', None],
  [ 0x6df3, 'CjkUnifiedIdeograph6df3', [], 'Display', None],
  [ 0x6df4, 'CjkUnifiedIdeograph6df4', [], 'Display', None],
  [ 0x6df5, 'CjkUnifiedIdeograph6df5', [], 'Display', None],
  [ 0x6df6, 'CjkUnifiedIdeograph6df6', [], 'Display', None],
  [ 0x6df7, 'CjkUnifiedIdeograph6df7', [], 'Display', None],
  [ 0x6df8, 'CjkUnifiedIdeograph6df8', [], 'Display', None],
  [ 0x6df9, 'CjkUnifiedIdeograph6df9', [], 'Display', None],
  [ 0x6dfa, 'CjkUnifiedIdeograph6dfa', [], 'Display', None],
  [ 0x6dfb, 'CjkUnifiedIdeograph6dfb', [], 'Display', None],
  [ 0x6dfc, 'CjkUnifiedIdeograph6dfc', [], 'Display', None],
  [ 0x6dfd, 'CjkUnifiedIdeograph6dfd', [], 'Display', None],
  [ 0x6dfe, 'CjkUnifiedIdeograph6dfe', [], 'Display', None],
  [ 0x6dff, 'CjkUnifiedIdeograph6dff', [], 'Display', None],
  [ 0x6e00, 'CjkUnifiedIdeograph6e00', [], 'Display', None],
  [ 0x6e01, 'CjkUnifiedIdeograph6e01', [], 'Display', None],
  [ 0x6e02, 'CjkUnifiedIdeograph6e02', [], 'Display', None],
  [ 0x6e03, 'CjkUnifiedIdeograph6e03', [], 'Display', None],
  [ 0x6e04, 'CjkUnifiedIdeograph6e04', [], 'Display', None],
  [ 0x6e05, 'CjkUnifiedIdeograph6e05', [], 'Display', None],
  [ 0x6e06, 'CjkUnifiedIdeograph6e06', [], 'Display', None],
  [ 0x6e07, 'CjkUnifiedIdeograph6e07', [], 'Display', None],
  [ 0x6e08, 'CjkUnifiedIdeograph6e08', [], 'Display', None],
  [ 0x6e09, 'CjkUnifiedIdeograph6e09', [], 'Display', None],
  [ 0x6e0a, 'CjkUnifiedIdeograph6e0a', [], 'Display', None],
  [ 0x6e0b, 'CjkUnifiedIdeograph6e0b', [], 'Display', None],
  [ 0x6e0c, 'CjkUnifiedIdeograph6e0c', [], 'Display', None],
  [ 0x6e0d, 'CjkUnifiedIdeograph6e0d', [], 'Display', None],
  [ 0x6e0e, 'CjkUnifiedIdeograph6e0e', [], 'Display', None],
  [ 0x6e0f, 'CjkUnifiedIdeograph6e0f', [], 'Display', None],
  [ 0x6e10, 'CjkUnifiedIdeograph6e10', [], 'Display', None],
  [ 0x6e11, 'CjkUnifiedIdeograph6e11', [], 'Display', None],
  [ 0x6e12, 'CjkUnifiedIdeograph6e12', [], 'Display', None],
  [ 0x6e13, 'CjkUnifiedIdeograph6e13', [], 'Display', None],
  [ 0x6e14, 'CjkUnifiedIdeograph6e14', [], 'Display', None],
  [ 0x6e15, 'CjkUnifiedIdeograph6e15', [], 'Display', None],
  [ 0x6e16, 'CjkUnifiedIdeograph6e16', [], 'Display', None],
  [ 0x6e17, 'CjkUnifiedIdeograph6e17', [], 'Display', None],
  [ 0x6e18, 'CjkUnifiedIdeograph6e18', [], 'Display', None],
  [ 0x6e19, 'CjkUnifiedIdeograph6e19', [], 'Display', None],
  [ 0x6e1a, 'CjkUnifiedIdeograph6e1a', [], 'Display', None],
  [ 0x6e1b, 'CjkUnifiedIdeograph6e1b', [], 'Display', None],
  [ 0x6e1c, 'CjkUnifiedIdeograph6e1c', [], 'Display', None],
  [ 0x6e1d, 'CjkUnifiedIdeograph6e1d', [], 'Display', None],
  [ 0x6e1e, 'CjkUnifiedIdeograph6e1e', [], 'Display', None],
  [ 0x6e1f, 'CjkUnifiedIdeograph6e1f', [], 'Display', None],
  [ 0x6e20, 'CjkUnifiedIdeograph6e20', [], 'Display', None],
  [ 0x6e21, 'CjkUnifiedIdeograph6e21', [], 'Display', None],
  [ 0x6e22, 'CjkUnifiedIdeograph6e22', [], 'Display', None],
  [ 0x6e23, 'CjkUnifiedIdeograph6e23', [], 'Display', None],
  [ 0x6e24, 'CjkUnifiedIdeograph6e24', [], 'Display', None],
  [ 0x6e25, 'CjkUnifiedIdeograph6e25', [], 'Display', None],
  [ 0x6e26, 'CjkUnifiedIdeograph6e26', [], 'Display', None],
  [ 0x6e27, 'CjkUnifiedIdeograph6e27', [], 'Display', None],
  [ 0x6e28, 'CjkUnifiedIdeograph6e28', [], 'Display', None],
  [ 0x6e29, 'CjkUnifiedIdeograph6e29', [], 'Display', None],
  [ 0x6e2a, 'CjkUnifiedIdeograph6e2a', [], 'Display', None],
  [ 0x6e2b, 'CjkUnifiedIdeograph6e2b', [], 'Display', None],
  [ 0x6e2c, 'CjkUnifiedIdeograph6e2c', [], 'Display', None],
  [ 0x6e2d, 'CjkUnifiedIdeograph6e2d', [], 'Display', None],
  [ 0x6e2e, 'CjkUnifiedIdeograph6e2e', [], 'Display', None],
  [ 0x6e2f, 'CjkUnifiedIdeograph6e2f', [], 'Display', None],
  [ 0x6e30, 'CjkUnifiedIdeograph6e30', [], 'Display', None],
  [ 0x6e31, 'CjkUnifiedIdeograph6e31', [], 'Display', None],
  [ 0x6e32, 'CjkUnifiedIdeograph6e32', [], 'Display', None],
  [ 0x6e33, 'CjkUnifiedIdeograph6e33', [], 'Display', None],
  [ 0x6e34, 'CjkUnifiedIdeograph6e34', [], 'Display', None],
  [ 0x6e35, 'CjkUnifiedIdeograph6e35', [], 'Display', None],
  [ 0x6e36, 'CjkUnifiedIdeograph6e36', [], 'Display', None],
  [ 0x6e37, 'CjkUnifiedIdeograph6e37', [], 'Display', None],
  [ 0x6e38, 'CjkUnifiedIdeograph6e38', [], 'Display', None],
  [ 0x6e39, 'CjkUnifiedIdeograph6e39', [], 'Display', None],
  [ 0x6e3a, 'CjkUnifiedIdeograph6e3a', [], 'Display', None],
  [ 0x6e3b, 'CjkUnifiedIdeograph6e3b', [], 'Display', None],
  [ 0x6e3c, 'CjkUnifiedIdeograph6e3c', [], 'Display', None],
  [ 0x6e3d, 'CjkUnifiedIdeograph6e3d', [], 'Display', None],
  [ 0x6e3e, 'CjkUnifiedIdeograph6e3e', [], 'Display', None],
  [ 0x6e3f, 'CjkUnifiedIdeograph6e3f', [], 'Display', None],
  [ 0x6e40, 'CjkUnifiedIdeograph6e40', [], 'Display', None],
  [ 0x6e41, 'CjkUnifiedIdeograph6e41', [], 'Display', None],
  [ 0x6e42, 'CjkUnifiedIdeograph6e42', [], 'Display', None],
  [ 0x6e43, 'CjkUnifiedIdeograph6e43', [], 'Display', None],
  [ 0x6e44, 'CjkUnifiedIdeograph6e44', [], 'Display', None],
  [ 0x6e45, 'CjkUnifiedIdeograph6e45', [], 'Display', None],
  [ 0x6e46, 'CjkUnifiedIdeograph6e46', [], 'Display', None],
  [ 0x6e47, 'CjkUnifiedIdeograph6e47', [], 'Display', None],
  [ 0x6e48, 'CjkUnifiedIdeograph6e48', [], 'Display', None],
  [ 0x6e49, 'CjkUnifiedIdeograph6e49', [], 'Display', None],
  [ 0x6e4a, 'CjkUnifiedIdeograph6e4a', [], 'Display', None],
  [ 0x6e4b, 'CjkUnifiedIdeograph6e4b', [], 'Display', None],
  [ 0x6e4c, 'CjkUnifiedIdeograph6e4c', [], 'Display', None],
  [ 0x6e4d, 'CjkUnifiedIdeograph6e4d', [], 'Display', None],
  [ 0x6e4e, 'CjkUnifiedIdeograph6e4e', [], 'Display', None],
  [ 0x6e4f, 'CjkUnifiedIdeograph6e4f', [], 'Display', None],
  [ 0x6e50, 'CjkUnifiedIdeograph6e50', [], 'Display', None],
  [ 0x6e51, 'CjkUnifiedIdeograph6e51', [], 'Display', None],
  [ 0x6e52, 'CjkUnifiedIdeograph6e52', [], 'Display', None],
  [ 0x6e53, 'CjkUnifiedIdeograph6e53', [], 'Display', None],
  [ 0x6e54, 'CjkUnifiedIdeograph6e54', [], 'Display', None],
  [ 0x6e55, 'CjkUnifiedIdeograph6e55', [], 'Display', None],
  [ 0x6e56, 'CjkUnifiedIdeograph6e56', [], 'Display', None],
  [ 0x6e57, 'CjkUnifiedIdeograph6e57', [], 'Display', None],
  [ 0x6e58, 'CjkUnifiedIdeograph6e58', [], 'Display', None],
  [ 0x6e59, 'CjkUnifiedIdeograph6e59', [], 'Display', None],
  [ 0x6e5a, 'CjkUnifiedIdeograph6e5a', [], 'Display', None],
  [ 0x6e5b, 'CjkUnifiedIdeograph6e5b', [], 'Display', None],
  [ 0x6e5c, 'CjkUnifiedIdeograph6e5c', [], 'Display', None],
  [ 0x6e5d, 'CjkUnifiedIdeograph6e5d', [], 'Display', None],
  [ 0x6e5e, 'CjkUnifiedIdeograph6e5e', [], 'Display', None],
  [ 0x6e5f, 'CjkUnifiedIdeograph6e5f', [], 'Display', None],
  [ 0x6e60, 'CjkUnifiedIdeograph6e60', [], 'Display', None],
  [ 0x6e61, 'CjkUnifiedIdeograph6e61', [], 'Display', None],
  [ 0x6e62, 'CjkUnifiedIdeograph6e62', [], 'Display', None],
  [ 0x6e63, 'CjkUnifiedIdeograph6e63', [], 'Display', None],
  [ 0x6e64, 'CjkUnifiedIdeograph6e64', [], 'Display', None],
  [ 0x6e65, 'CjkUnifiedIdeograph6e65', [], 'Display', None],
  [ 0x6e66, 'CjkUnifiedIdeograph6e66', [], 'Display', None],
  [ 0x6e67, 'CjkUnifiedIdeograph6e67', [], 'Display', None],
  [ 0x6e68, 'CjkUnifiedIdeograph6e68', [], 'Display', None],
  [ 0x6e69, 'CjkUnifiedIdeograph6e69', [], 'Display', None],
  [ 0x6e6a, 'CjkUnifiedIdeograph6e6a', [], 'Display', None],
  [ 0x6e6b, 'CjkUnifiedIdeograph6e6b', [], 'Display', None],
  [ 0x6e6c, 'CjkUnifiedIdeograph6e6c', [], 'Display', None],
  [ 0x6e6d, 'CjkUnifiedIdeograph6e6d', [], 'Display', None],
  [ 0x6e6e, 'CjkUnifiedIdeograph6e6e', [], 'Display', None],
  [ 0x6e6f, 'CjkUnifiedIdeograph6e6f', [], 'Display', None],
  [ 0x6e70, 'CjkUnifiedIdeograph6e70', [], 'Display', None],
  [ 0x6e71, 'CjkUnifiedIdeograph6e71', [], 'Display', None],
  [ 0x6e72, 'CjkUnifiedIdeograph6e72', [], 'Display', None],
  [ 0x6e73, 'CjkUnifiedIdeograph6e73', [], 'Display', None],
  [ 0x6e74, 'CjkUnifiedIdeograph6e74', [], 'Display', None],
  [ 0x6e75, 'CjkUnifiedIdeograph6e75', [], 'Display', None],
  [ 0x6e76, 'CjkUnifiedIdeograph6e76', [], 'Display', None],
  [ 0x6e77, 'CjkUnifiedIdeograph6e77', [], 'Display', None],
  [ 0x6e78, 'CjkUnifiedIdeograph6e78', [], 'Display', None],
  [ 0x6e79, 'CjkUnifiedIdeograph6e79', [], 'Display', None],
  [ 0x6e7a, 'CjkUnifiedIdeograph6e7a', [], 'Display', None],
  [ 0x6e7b, 'CjkUnifiedIdeograph6e7b', [], 'Display', None],
  [ 0x6e7c, 'CjkUnifiedIdeograph6e7c', [], 'Display', None],
  [ 0x6e7d, 'CjkUnifiedIdeograph6e7d', [], 'Display', None],
  [ 0x6e7e, 'CjkUnifiedIdeograph6e7e', [], 'Display', None],
  [ 0x6e7f, 'CjkUnifiedIdeograph6e7f', [], 'Display', None],
  [ 0x6e80, 'CjkUnifiedIdeograph6e80', [], 'Display', None],
  [ 0x6e81, 'CjkUnifiedIdeograph6e81', [], 'Display', None],
  [ 0x6e82, 'CjkUnifiedIdeograph6e82', [], 'Display', None],
  [ 0x6e83, 'CjkUnifiedIdeograph6e83', [], 'Display', None],
  [ 0x6e84, 'CjkUnifiedIdeograph6e84', [], 'Display', None],
  [ 0x6e85, 'CjkUnifiedIdeograph6e85', [], 'Display', None],
  [ 0x6e86, 'CjkUnifiedIdeograph6e86', [], 'Display', None],
  [ 0x6e87, 'CjkUnifiedIdeograph6e87', [], 'Display', None],
  [ 0x6e88, 'CjkUnifiedIdeograph6e88', [], 'Display', None],
  [ 0x6e89, 'CjkUnifiedIdeograph6e89', [], 'Display', None],
  [ 0x6e8a, 'CjkUnifiedIdeograph6e8a', [], 'Display', None],
  [ 0x6e8b, 'CjkUnifiedIdeograph6e8b', [], 'Display', None],
  [ 0x6e8c, 'CjkUnifiedIdeograph6e8c', [], 'Display', None],
  [ 0x6e8d, 'CjkUnifiedIdeograph6e8d', [], 'Display', None],
  [ 0x6e8e, 'CjkUnifiedIdeograph6e8e', [], 'Display', None],
  [ 0x6e8f, 'CjkUnifiedIdeograph6e8f', [], 'Display', None],
  [ 0x6e90, 'CjkUnifiedIdeograph6e90', [], 'Display', None],
  [ 0x6e91, 'CjkUnifiedIdeograph6e91', [], 'Display', None],
  [ 0x6e92, 'CjkUnifiedIdeograph6e92', [], 'Display', None],
  [ 0x6e93, 'CjkUnifiedIdeograph6e93', [], 'Display', None],
  [ 0x6e94, 'CjkUnifiedIdeograph6e94', [], 'Display', None],
  [ 0x6e95, 'CjkUnifiedIdeograph6e95', [], 'Display', None],
  [ 0x6e96, 'CjkUnifiedIdeograph6e96', [], 'Display', None],
  [ 0x6e97, 'CjkUnifiedIdeograph6e97', [], 'Display', None],
  [ 0x6e98, 'CjkUnifiedIdeograph6e98', [], 'Display', None],
  [ 0x6e99, 'CjkUnifiedIdeograph6e99', [], 'Display', None],
  [ 0x6e9a, 'CjkUnifiedIdeograph6e9a', [], 'Display', None],
  [ 0x6e9b, 'CjkUnifiedIdeograph6e9b', [], 'Display', None],
  [ 0x6e9c, 'CjkUnifiedIdeograph6e9c', [], 'Display', None],
  [ 0x6e9d, 'CjkUnifiedIdeograph6e9d', [], 'Display', None],
  [ 0x6e9e, 'CjkUnifiedIdeograph6e9e', [], 'Display', None],
  [ 0x6e9f, 'CjkUnifiedIdeograph6e9f', [], 'Display', None],
  [ 0x6ea0, 'CjkUnifiedIdeograph6ea0', [], 'Display', None],
  [ 0x6ea1, 'CjkUnifiedIdeograph6ea1', [], 'Display', None],
  [ 0x6ea2, 'CjkUnifiedIdeograph6ea2', [], 'Display', None],
  [ 0x6ea3, 'CjkUnifiedIdeograph6ea3', [], 'Display', None],
  [ 0x6ea4, 'CjkUnifiedIdeograph6ea4', [], 'Display', None],
  [ 0x6ea5, 'CjkUnifiedIdeograph6ea5', [], 'Display', None],
  [ 0x6ea6, 'CjkUnifiedIdeograph6ea6', [], 'Display', None],
  [ 0x6ea7, 'CjkUnifiedIdeograph6ea7', [], 'Display', None],
  [ 0x6ea8, 'CjkUnifiedIdeograph6ea8', [], 'Display', None],
  [ 0x6ea9, 'CjkUnifiedIdeograph6ea9', [], 'Display', None],
  [ 0x6eaa, 'CjkUnifiedIdeograph6eaa', [], 'Display', None],
  [ 0x6eab, 'CjkUnifiedIdeograph6eab', [], 'Display', None],
  [ 0x6eac, 'CjkUnifiedIdeograph6eac', [], 'Display', None],
  [ 0x6ead, 'CjkUnifiedIdeograph6ead', [], 'Display', None],
  [ 0x6eae, 'CjkUnifiedIdeograph6eae', [], 'Display', None],
  [ 0x6eaf, 'CjkUnifiedIdeograph6eaf', [], 'Display', None],
  [ 0x6eb0, 'CjkUnifiedIdeograph6eb0', [], 'Display', None],
  [ 0x6eb1, 'CjkUnifiedIdeograph6eb1', [], 'Display', None],
  [ 0x6eb2, 'CjkUnifiedIdeograph6eb2', [], 'Display', None],
  [ 0x6eb3, 'CjkUnifiedIdeograph6eb3', [], 'Display', None],
  [ 0x6eb4, 'CjkUnifiedIdeograph6eb4', [], 'Display', None],
  [ 0x6eb5, 'CjkUnifiedIdeograph6eb5', [], 'Display', None],
  [ 0x6eb6, 'CjkUnifiedIdeograph6eb6', [], 'Display', None],
  [ 0x6eb7, 'CjkUnifiedIdeograph6eb7', [], 'Display', None],
  [ 0x6eb8, 'CjkUnifiedIdeograph6eb8', [], 'Display', None],
  [ 0x6eb9, 'CjkUnifiedIdeograph6eb9', [], 'Display', None],
  [ 0x6eba, 'CjkUnifiedIdeograph6eba', [], 'Display', None],
  [ 0x6ebb, 'CjkUnifiedIdeograph6ebb', [], 'Display', None],
  [ 0x6ebc, 'CjkUnifiedIdeograph6ebc', [], 'Display', None],
  [ 0x6ebd, 'CjkUnifiedIdeograph6ebd', [], 'Display', None],
  [ 0x6ebe, 'CjkUnifiedIdeograph6ebe', [], 'Display', None],
  [ 0x6ebf, 'CjkUnifiedIdeograph6ebf', [], 'Display', None],
  [ 0x6ec0, 'CjkUnifiedIdeograph6ec0', [], 'Display', None],
  [ 0x6ec1, 'CjkUnifiedIdeograph6ec1', [], 'Display', None],
  [ 0x6ec2, 'CjkUnifiedIdeograph6ec2', [], 'Display', None],
  [ 0x6ec3, 'CjkUnifiedIdeograph6ec3', [], 'Display', None],
  [ 0x6ec4, 'CjkUnifiedIdeograph6ec4', [], 'Display', None],
  [ 0x6ec5, 'CjkUnifiedIdeograph6ec5', [], 'Display', None],
  [ 0x6ec6, 'CjkUnifiedIdeograph6ec6', [], 'Display', None],
  [ 0x6ec7, 'CjkUnifiedIdeograph6ec7', [], 'Display', None],
  [ 0x6ec8, 'CjkUnifiedIdeograph6ec8', [], 'Display', None],
  [ 0x6ec9, 'CjkUnifiedIdeograph6ec9', [], 'Display', None],
  [ 0x6eca, 'CjkUnifiedIdeograph6eca', [], 'Display', None],
  [ 0x6ecb, 'CjkUnifiedIdeograph6ecb', [], 'Display', None],
  [ 0x6ecc, 'CjkUnifiedIdeograph6ecc', [], 'Display', None],
  [ 0x6ecd, 'CjkUnifiedIdeograph6ecd', [], 'Display', None],
  [ 0x6ece, 'CjkUnifiedIdeograph6ece', [], 'Display', None],
  [ 0x6ecf, 'CjkUnifiedIdeograph6ecf', [], 'Display', None],
  [ 0x6ed0, 'CjkUnifiedIdeograph6ed0', [], 'Display', None],
  [ 0x6ed1, 'CjkUnifiedIdeograph6ed1', [], 'Display', None],
  [ 0x6ed2, 'CjkUnifiedIdeograph6ed2', [], 'Display', None],
  [ 0x6ed3, 'CjkUnifiedIdeograph6ed3', [], 'Display', None],
  [ 0x6ed4, 'CjkUnifiedIdeograph6ed4', [], 'Display', None],
  [ 0x6ed5, 'CjkUnifiedIdeograph6ed5', [], 'Display', None],
  [ 0x6ed6, 'CjkUnifiedIdeograph6ed6', [], 'Display', None],
  [ 0x6ed7, 'CjkUnifiedIdeograph6ed7', [], 'Display', None],
  [ 0x6ed8, 'CjkUnifiedIdeograph6ed8', [], 'Display', None],
  [ 0x6ed9, 'CjkUnifiedIdeograph6ed9', [], 'Display', None],
  [ 0x6eda, 'CjkUnifiedIdeograph6eda', [], 'Display', None],
  [ 0x6edb, 'CjkUnifiedIdeograph6edb', [], 'Display', None],
  [ 0x6edc, 'CjkUnifiedIdeograph6edc', [], 'Display', None],
  [ 0x6edd, 'CjkUnifiedIdeograph6edd', [], 'Display', None],
  [ 0x6ede, 'CjkUnifiedIdeograph6ede', [], 'Display', None],
  [ 0x6edf, 'CjkUnifiedIdeograph6edf', [], 'Display', None],
  [ 0x6ee0, 'CjkUnifiedIdeograph6ee0', [], 'Display', None],
  [ 0x6ee1, 'CjkUnifiedIdeograph6ee1', [], 'Display', None],
  [ 0x6ee2, 'CjkUnifiedIdeograph6ee2', [], 'Display', None],
  [ 0x6ee3, 'CjkUnifiedIdeograph6ee3', [], 'Display', None],
  [ 0x6ee4, 'CjkUnifiedIdeograph6ee4', [], 'Display', None],
  [ 0x6ee5, 'CjkUnifiedIdeograph6ee5', [], 'Display', None],
  [ 0x6ee6, 'CjkUnifiedIdeograph6ee6', [], 'Display', None],
  [ 0x6ee7, 'CjkUnifiedIdeograph6ee7', [], 'Display', None],
  [ 0x6ee8, 'CjkUnifiedIdeograph6ee8', [], 'Display', None],
  [ 0x6ee9, 'CjkUnifiedIdeograph6ee9', [], 'Display', None],
  [ 0x6eea, 'CjkUnifiedIdeograph6eea', [], 'Display', None],
  [ 0x6eeb, 'CjkUnifiedIdeograph6eeb', [], 'Display', None],
  [ 0x6eec, 'CjkUnifiedIdeograph6eec', [], 'Display', None],
  [ 0x6eed, 'CjkUnifiedIdeograph6eed', [], 'Display', None],
  [ 0x6eee, 'CjkUnifiedIdeograph6eee', [], 'Display', None],
  [ 0x6eef, 'CjkUnifiedIdeograph6eef', [], 'Display', None],
  [ 0x6ef0, 'CjkUnifiedIdeograph6ef0', [], 'Display', None],
  [ 0x6ef1, 'CjkUnifiedIdeograph6ef1', [], 'Display', None],
  [ 0x6ef2, 'CjkUnifiedIdeograph6ef2', [], 'Display', None],
  [ 0x6ef3, 'CjkUnifiedIdeograph6ef3', [], 'Display', None],
  [ 0x6ef4, 'CjkUnifiedIdeograph6ef4', [], 'Display', None],
  [ 0x6ef5, 'CjkUnifiedIdeograph6ef5', [], 'Display', None],
  [ 0x6ef6, 'CjkUnifiedIdeograph6ef6', [], 'Display', None],
  [ 0x6ef7, 'CjkUnifiedIdeograph6ef7', [], 'Display', None],
  [ 0x6ef8, 'CjkUnifiedIdeograph6ef8', [], 'Display', None],
  [ 0x6ef9, 'CjkUnifiedIdeograph6ef9', [], 'Display', None],
  [ 0x6efa, 'CjkUnifiedIdeograph6efa', [], 'Display', None],
  [ 0x6efb, 'CjkUnifiedIdeograph6efb', [], 'Display', None],
  [ 0x6efc, 'CjkUnifiedIdeograph6efc', [], 'Display', None],
  [ 0x6efd, 'CjkUnifiedIdeograph6efd', [], 'Display', None],
  [ 0x6efe, 'CjkUnifiedIdeograph6efe', [], 'Display', None],
  [ 0x6eff, 'CjkUnifiedIdeograph6eff', [], 'Display', None],
  [ 0x6f00, 'CjkUnifiedIdeograph6f00', [], 'Display', None],
  [ 0x6f01, 'CjkUnifiedIdeograph6f01', [], 'Display', None],
  [ 0x6f02, 'CjkUnifiedIdeograph6f02', [], 'Display', None],
  [ 0x6f03, 'CjkUnifiedIdeograph6f03', [], 'Display', None],
  [ 0x6f04, 'CjkUnifiedIdeograph6f04', [], 'Display', None],
  [ 0x6f05, 'CjkUnifiedIdeograph6f05', [], 'Display', None],
  [ 0x6f06, 'CjkUnifiedIdeograph6f06', [], 'Display', None],
  [ 0x6f07, 'CjkUnifiedIdeograph6f07', [], 'Display', None],
  [ 0x6f08, 'CjkUnifiedIdeograph6f08', [], 'Display', None],
  [ 0x6f09, 'CjkUnifiedIdeograph6f09', [], 'Display', None],
  [ 0x6f0a, 'CjkUnifiedIdeograph6f0a', [], 'Display', None],
  [ 0x6f0b, 'CjkUnifiedIdeograph6f0b', [], 'Display', None],
  [ 0x6f0c, 'CjkUnifiedIdeograph6f0c', [], 'Display', None],
  [ 0x6f0d, 'CjkUnifiedIdeograph6f0d', [], 'Display', None],
  [ 0x6f0e, 'CjkUnifiedIdeograph6f0e', [], 'Display', None],
  [ 0x6f0f, 'CjkUnifiedIdeograph6f0f', [], 'Display', None],
  [ 0x6f10, 'CjkUnifiedIdeograph6f10', [], 'Display', None],
  [ 0x6f11, 'CjkUnifiedIdeograph6f11', [], 'Display', None],
  [ 0x6f12, 'CjkUnifiedIdeograph6f12', [], 'Display', None],
  [ 0x6f13, 'CjkUnifiedIdeograph6f13', [], 'Display', None],
  [ 0x6f14, 'CjkUnifiedIdeograph6f14', [], 'Display', None],
  [ 0x6f15, 'CjkUnifiedIdeograph6f15', [], 'Display', None],
  [ 0x6f16, 'CjkUnifiedIdeograph6f16', [], 'Display', None],
  [ 0x6f17, 'CjkUnifiedIdeograph6f17', [], 'Display', None],
  [ 0x6f18, 'CjkUnifiedIdeograph6f18', [], 'Display', None],
  [ 0x6f19, 'CjkUnifiedIdeograph6f19', [], 'Display', None],
  [ 0x6f1a, 'CjkUnifiedIdeograph6f1a', [], 'Display', None],
  [ 0x6f1b, 'CjkUnifiedIdeograph6f1b', [], 'Display', None],
  [ 0x6f1c, 'CjkUnifiedIdeograph6f1c', [], 'Display', None],
  [ 0x6f1d, 'CjkUnifiedIdeograph6f1d', [], 'Display', None],
  [ 0x6f1e, 'CjkUnifiedIdeograph6f1e', [], 'Display', None],
  [ 0x6f1f, 'CjkUnifiedIdeograph6f1f', [], 'Display', None],
  [ 0x6f20, 'CjkUnifiedIdeograph6f20', [], 'Display', None],
  [ 0x6f21, 'CjkUnifiedIdeograph6f21', [], 'Display', None],
  [ 0x6f22, 'CjkUnifiedIdeograph6f22', [], 'Display', None],
  [ 0x6f23, 'CjkUnifiedIdeograph6f23', [], 'Display', None],
  [ 0x6f24, 'CjkUnifiedIdeograph6f24', [], 'Display', None],
  [ 0x6f25, 'CjkUnifiedIdeograph6f25', [], 'Display', None],
  [ 0x6f26, 'CjkUnifiedIdeograph6f26', [], 'Display', None],
  [ 0x6f27, 'CjkUnifiedIdeograph6f27', [], 'Display', None],
  [ 0x6f28, 'CjkUnifiedIdeograph6f28', [], 'Display', None],
  [ 0x6f29, 'CjkUnifiedIdeograph6f29', [], 'Display', None],
  [ 0x6f2a, 'CjkUnifiedIdeograph6f2a', [], 'Display', None],
  [ 0x6f2b, 'CjkUnifiedIdeograph6f2b', [], 'Display', None],
  [ 0x6f2c, 'CjkUnifiedIdeograph6f2c', [], 'Display', None],
  [ 0x6f2d, 'CjkUnifiedIdeograph6f2d', [], 'Display', None],
  [ 0x6f2e, 'CjkUnifiedIdeograph6f2e', [], 'Display', None],
  [ 0x6f2f, 'CjkUnifiedIdeograph6f2f', [], 'Display', None],
  [ 0x6f30, 'CjkUnifiedIdeograph6f30', [], 'Display', None],
  [ 0x6f31, 'CjkUnifiedIdeograph6f31', [], 'Display', None],
  [ 0x6f32, 'CjkUnifiedIdeograph6f32', [], 'Display', None],
  [ 0x6f33, 'CjkUnifiedIdeograph6f33', [], 'Display', None],
  [ 0x6f34, 'CjkUnifiedIdeograph6f34', [], 'Display', None],
  [ 0x6f35, 'CjkUnifiedIdeograph6f35', [], 'Display', None],
  [ 0x6f36, 'CjkUnifiedIdeograph6f36', [], 'Display', None],
  [ 0x6f37, 'CjkUnifiedIdeograph6f37', [], 'Display', None],
  [ 0x6f38, 'CjkUnifiedIdeograph6f38', [], 'Display', None],
  [ 0x6f39, 'CjkUnifiedIdeograph6f39', [], 'Display', None],
  [ 0x6f3a, 'CjkUnifiedIdeograph6f3a', [], 'Display', None],
  [ 0x6f3b, 'CjkUnifiedIdeograph6f3b', [], 'Display', None],
  [ 0x6f3c, 'CjkUnifiedIdeograph6f3c', [], 'Display', None],
  [ 0x6f3d, 'CjkUnifiedIdeograph6f3d', [], 'Display', None],
  [ 0x6f3e, 'CjkUnifiedIdeograph6f3e', [], 'Display', None],
  [ 0x6f3f, 'CjkUnifiedIdeograph6f3f', [], 'Display', None],
  [ 0x6f40, 'CjkUnifiedIdeograph6f40', [], 'Display', None],
  [ 0x6f41, 'CjkUnifiedIdeograph6f41', [], 'Display', None],
  [ 0x6f42, 'CjkUnifiedIdeograph6f42', [], 'Display', None],
  [ 0x6f43, 'CjkUnifiedIdeograph6f43', [], 'Display', None],
  [ 0x6f44, 'CjkUnifiedIdeograph6f44', [], 'Display', None],
  [ 0x6f45, 'CjkUnifiedIdeograph6f45', [], 'Display', None],
  [ 0x6f46, 'CjkUnifiedIdeograph6f46', [], 'Display', None],
  [ 0x6f47, 'CjkUnifiedIdeograph6f47', [], 'Display', None],
  [ 0x6f48, 'CjkUnifiedIdeograph6f48', [], 'Display', None],
  [ 0x6f49, 'CjkUnifiedIdeograph6f49', [], 'Display', None],
  [ 0x6f4a, 'CjkUnifiedIdeograph6f4a', [], 'Display', None],
  [ 0x6f4b, 'CjkUnifiedIdeograph6f4b', [], 'Display', None],
  [ 0x6f4c, 'CjkUnifiedIdeograph6f4c', [], 'Display', None],
  [ 0x6f4d, 'CjkUnifiedIdeograph6f4d', [], 'Display', None],
  [ 0x6f4e, 'CjkUnifiedIdeograph6f4e', [], 'Display', None],
  [ 0x6f4f, 'CjkUnifiedIdeograph6f4f', [], 'Display', None],
  [ 0x6f50, 'CjkUnifiedIdeograph6f50', [], 'Display', None],
  [ 0x6f51, 'CjkUnifiedIdeograph6f51', [], 'Display', None],
  [ 0x6f52, 'CjkUnifiedIdeograph6f52', [], 'Display', None],
  [ 0x6f53, 'CjkUnifiedIdeograph6f53', [], 'Display', None],
  [ 0x6f54, 'CjkUnifiedIdeograph6f54', [], 'Display', None],
  [ 0x6f55, 'CjkUnifiedIdeograph6f55', [], 'Display', None],
  [ 0x6f56, 'CjkUnifiedIdeograph6f56', [], 'Display', None],
  [ 0x6f57, 'CjkUnifiedIdeograph6f57', [], 'Display', None],
  [ 0x6f58, 'CjkUnifiedIdeograph6f58', [], 'Display', None],
  [ 0x6f59, 'CjkUnifiedIdeograph6f59', [], 'Display', None],
  [ 0x6f5a, 'CjkUnifiedIdeograph6f5a', [], 'Display', None],
  [ 0x6f5b, 'CjkUnifiedIdeograph6f5b', [], 'Display', None],
  [ 0x6f5c, 'CjkUnifiedIdeograph6f5c', [], 'Display', None],
  [ 0x6f5d, 'CjkUnifiedIdeograph6f5d', [], 'Display', None],
  [ 0x6f5e, 'CjkUnifiedIdeograph6f5e', [], 'Display', None],
  [ 0x6f5f, 'CjkUnifiedIdeograph6f5f', [], 'Display', None],
  [ 0x6f60, 'CjkUnifiedIdeograph6f60', [], 'Display', None],
  [ 0x6f61, 'CjkUnifiedIdeograph6f61', [], 'Display', None],
  [ 0x6f62, 'CjkUnifiedIdeograph6f62', [], 'Display', None],
  [ 0x6f63, 'CjkUnifiedIdeograph6f63', [], 'Display', None],
  [ 0x6f64, 'CjkUnifiedIdeograph6f64', [], 'Display', None],
  [ 0x6f65, 'CjkUnifiedIdeograph6f65', [], 'Display', None],
  [ 0x6f66, 'CjkUnifiedIdeograph6f66', [], 'Display', None],
  [ 0x6f67, 'CjkUnifiedIdeograph6f67', [], 'Display', None],
  [ 0x6f68, 'CjkUnifiedIdeograph6f68', [], 'Display', None],
  [ 0x6f69, 'CjkUnifiedIdeograph6f69', [], 'Display', None],
  [ 0x6f6a, 'CjkUnifiedIdeograph6f6a', [], 'Display', None],
  [ 0x6f6b, 'CjkUnifiedIdeograph6f6b', [], 'Display', None],
  [ 0x6f6c, 'CjkUnifiedIdeograph6f6c', [], 'Display', None],
  [ 0x6f6d, 'CjkUnifiedIdeograph6f6d', [], 'Display', None],
  [ 0x6f6e, 'CjkUnifiedIdeograph6f6e', [], 'Display', None],
  [ 0x6f6f, 'CjkUnifiedIdeograph6f6f', [], 'Display', None],
  [ 0x6f70, 'CjkUnifiedIdeograph6f70', [], 'Display', None],
  [ 0x6f71, 'CjkUnifiedIdeograph6f71', [], 'Display', None],
  [ 0x6f72, 'CjkUnifiedIdeograph6f72', [], 'Display', None],
  [ 0x6f73, 'CjkUnifiedIdeograph6f73', [], 'Display', None],
  [ 0x6f74, 'CjkUnifiedIdeograph6f74', [], 'Display', None],
  [ 0x6f75, 'CjkUnifiedIdeograph6f75', [], 'Display', None],
  [ 0x6f76, 'CjkUnifiedIdeograph6f76', [], 'Display', None],
  [ 0x6f77, 'CjkUnifiedIdeograph6f77', [], 'Display', None],
  [ 0x6f78, 'CjkUnifiedIdeograph6f78', [], 'Display', None],
  [ 0x6f79, 'CjkUnifiedIdeograph6f79', [], 'Display', None],
  [ 0x6f7a, 'CjkUnifiedIdeograph6f7a', [], 'Display', None],
  [ 0x6f7b, 'CjkUnifiedIdeograph6f7b', [], 'Display', None],
  [ 0x6f7c, 'CjkUnifiedIdeograph6f7c', [], 'Display', None],
  [ 0x6f7d, 'CjkUnifiedIdeograph6f7d', [], 'Display', None],
  [ 0x6f7e, 'CjkUnifiedIdeograph6f7e', [], 'Display', None],
  [ 0x6f7f, 'CjkUnifiedIdeograph6f7f', [], 'Display', None],
  [ 0x6f80, 'CjkUnifiedIdeograph6f80', [], 'Display', None],
  [ 0x6f81, 'CjkUnifiedIdeograph6f81', [], 'Display', None],
  [ 0x6f82, 'CjkUnifiedIdeograph6f82', [], 'Display', None],
  [ 0x6f83, 'CjkUnifiedIdeograph6f83', [], 'Display', None],
  [ 0x6f84, 'CjkUnifiedIdeograph6f84', [], 'Display', None],
  [ 0x6f85, 'CjkUnifiedIdeograph6f85', [], 'Display', None],
  [ 0x6f86, 'CjkUnifiedIdeograph6f86', [], 'Display', None],
  [ 0x6f87, 'CjkUnifiedIdeograph6f87', [], 'Display', None],
  [ 0x6f88, 'CjkUnifiedIdeograph6f88', [], 'Display', None],
  [ 0x6f89, 'CjkUnifiedIdeograph6f89', [], 'Display', None],
  [ 0x6f8a, 'CjkUnifiedIdeograph6f8a', [], 'Display', None],
  [ 0x6f8b, 'CjkUnifiedIdeograph6f8b', [], 'Display', None],
  [ 0x6f8c, 'CjkUnifiedIdeograph6f8c', [], 'Display', None],
  [ 0x6f8d, 'CjkUnifiedIdeograph6f8d', [], 'Display', None],
  [ 0x6f8e, 'CjkUnifiedIdeograph6f8e', [], 'Display', None],
  [ 0x6f8f, 'CjkUnifiedIdeograph6f8f', [], 'Display', None],
  [ 0x6f90, 'CjkUnifiedIdeograph6f90', [], 'Display', None],
  [ 0x6f91, 'CjkUnifiedIdeograph6f91', [], 'Display', None],
  [ 0x6f92, 'CjkUnifiedIdeograph6f92', [], 'Display', None],
  [ 0x6f93, 'CjkUnifiedIdeograph6f93', [], 'Display', None],
  [ 0x6f94, 'CjkUnifiedIdeograph6f94', [], 'Display', None],
  [ 0x6f95, 'CjkUnifiedIdeograph6f95', [], 'Display', None],
  [ 0x6f96, 'CjkUnifiedIdeograph6f96', [], 'Display', None],
  [ 0x6f97, 'CjkUnifiedIdeograph6f97', [], 'Display', None],
  [ 0x6f98, 'CjkUnifiedIdeograph6f98', [], 'Display', None],
  [ 0x6f99, 'CjkUnifiedIdeograph6f99', [], 'Display', None],
  [ 0x6f9a, 'CjkUnifiedIdeograph6f9a', [], 'Display', None],
  [ 0x6f9b, 'CjkUnifiedIdeograph6f9b', [], 'Display', None],
  [ 0x6f9c, 'CjkUnifiedIdeograph6f9c', [], 'Display', None],
  [ 0x6f9d, 'CjkUnifiedIdeograph6f9d', [], 'Display', None],
  [ 0x6f9e, 'CjkUnifiedIdeograph6f9e', [], 'Display', None],
  [ 0x6f9f, 'CjkUnifiedIdeograph6f9f', [], 'Display', None],
  [ 0x6fa0, 'CjkUnifiedIdeograph6fa0', [], 'Display', None],
  [ 0x6fa1, 'CjkUnifiedIdeograph6fa1', [], 'Display', None],
  [ 0x6fa2, 'CjkUnifiedIdeograph6fa2', [], 'Display', None],
  [ 0x6fa3, 'CjkUnifiedIdeograph6fa3', [], 'Display', None],
  [ 0x6fa4, 'CjkUnifiedIdeograph6fa4', [], 'Display', None],
  [ 0x6fa5, 'CjkUnifiedIdeograph6fa5', [], 'Display', None],
  [ 0x6fa6, 'CjkUnifiedIdeograph6fa6', [], 'Display', None],
  [ 0x6fa7, 'CjkUnifiedIdeograph6fa7', [], 'Display', None],
  [ 0x6fa8, 'CjkUnifiedIdeograph6fa8', [], 'Display', None],
  [ 0x6fa9, 'CjkUnifiedIdeograph6fa9', [], 'Display', None],
  [ 0x6faa, 'CjkUnifiedIdeograph6faa', [], 'Display', None],
  [ 0x6fab, 'CjkUnifiedIdeograph6fab', [], 'Display', None],
  [ 0x6fac, 'CjkUnifiedIdeograph6fac', [], 'Display', None],
  [ 0x6fad, 'CjkUnifiedIdeograph6fad', [], 'Display', None],
  [ 0x6fae, 'CjkUnifiedIdeograph6fae', [], 'Display', None],
  [ 0x6faf, 'CjkUnifiedIdeograph6faf', [], 'Display', None],
  [ 0x6fb0, 'CjkUnifiedIdeograph6fb0', [], 'Display', None],
  [ 0x6fb1, 'CjkUnifiedIdeograph6fb1', [], 'Display', None],
  [ 0x6fb2, 'CjkUnifiedIdeograph6fb2', [], 'Display', None],
  [ 0x6fb3, 'CjkUnifiedIdeograph6fb3', [], 'Display', None],
  [ 0x6fb4, 'CjkUnifiedIdeograph6fb4', [], 'Display', None],
  [ 0x6fb5, 'CjkUnifiedIdeograph6fb5', [], 'Display', None],
  [ 0x6fb6, 'CjkUnifiedIdeograph6fb6', [], 'Display', None],
  [ 0x6fb7, 'CjkUnifiedIdeograph6fb7', [], 'Display', None],
  [ 0x6fb8, 'CjkUnifiedIdeograph6fb8', [], 'Display', None],
  [ 0x6fb9, 'CjkUnifiedIdeograph6fb9', [], 'Display', None],
  [ 0x6fba, 'CjkUnifiedIdeograph6fba', [], 'Display', None],
  [ 0x6fbb, 'CjkUnifiedIdeograph6fbb', [], 'Display', None],
  [ 0x6fbc, 'CjkUnifiedIdeograph6fbc', [], 'Display', None],
  [ 0x6fbd, 'CjkUnifiedIdeograph6fbd', [], 'Display', None],
  [ 0x6fbe, 'CjkUnifiedIdeograph6fbe', [], 'Display', None],
  [ 0x6fbf, 'CjkUnifiedIdeograph6fbf', [], 'Display', None],
  [ 0x6fc0, 'CjkUnifiedIdeograph6fc0', [], 'Display', None],
  [ 0x6fc1, 'CjkUnifiedIdeograph6fc1', [], 'Display', None],
  [ 0x6fc2, 'CjkUnifiedIdeograph6fc2', [], 'Display', None],
  [ 0x6fc3, 'CjkUnifiedIdeograph6fc3', [], 'Display', None],
  [ 0x6fc4, 'CjkUnifiedIdeograph6fc4', [], 'Display', None],
  [ 0x6fc5, 'CjkUnifiedIdeograph6fc5', [], 'Display', None],
  [ 0x6fc6, 'CjkUnifiedIdeograph6fc6', [], 'Display', None],
  [ 0x6fc7, 'CjkUnifiedIdeograph6fc7', [], 'Display', None],
  [ 0x6fc8, 'CjkUnifiedIdeograph6fc8', [], 'Display', None],
  [ 0x6fc9, 'CjkUnifiedIdeograph6fc9', [], 'Display', None],
  [ 0x6fca, 'CjkUnifiedIdeograph6fca', [], 'Display', None],
  [ 0x6fcb, 'CjkUnifiedIdeograph6fcb', [], 'Display', None],
  [ 0x6fcc, 'CjkUnifiedIdeograph6fcc', [], 'Display', None],
  [ 0x6fcd, 'CjkUnifiedIdeograph6fcd', [], 'Display', None],
  [ 0x6fce, 'CjkUnifiedIdeograph6fce', [], 'Display', None],
  [ 0x6fcf, 'CjkUnifiedIdeograph6fcf', [], 'Display', None],
  [ 0x6fd0, 'CjkUnifiedIdeograph6fd0', [], 'Display', None],
  [ 0x6fd1, 'CjkUnifiedIdeograph6fd1', [], 'Display', None],
  [ 0x6fd2, 'CjkUnifiedIdeograph6fd2', [], 'Display', None],
  [ 0x6fd3, 'CjkUnifiedIdeograph6fd3', [], 'Display', None],
  [ 0x6fd4, 'CjkUnifiedIdeograph6fd4', [], 'Display', None],
  [ 0x6fd5, 'CjkUnifiedIdeograph6fd5', [], 'Display', None],
  [ 0x6fd6, 'CjkUnifiedIdeograph6fd6', [], 'Display', None],
  [ 0x6fd7, 'CjkUnifiedIdeograph6fd7', [], 'Display', None],
  [ 0x6fd8, 'CjkUnifiedIdeograph6fd8', [], 'Display', None],
  [ 0x6fd9, 'CjkUnifiedIdeograph6fd9', [], 'Display', None],
  [ 0x6fda, 'CjkUnifiedIdeograph6fda', [], 'Display', None],
  [ 0x6fdb, 'CjkUnifiedIdeograph6fdb', [], 'Display', None],
  [ 0x6fdc, 'CjkUnifiedIdeograph6fdc', [], 'Display', None],
  [ 0x6fdd, 'CjkUnifiedIdeograph6fdd', [], 'Display', None],
  [ 0x6fde, 'CjkUnifiedIdeograph6fde', [], 'Display', None],
  [ 0x6fdf, 'CjkUnifiedIdeograph6fdf', [], 'Display', None],
  [ 0x6fe0, 'CjkUnifiedIdeograph6fe0', [], 'Display', None],
  [ 0x6fe1, 'CjkUnifiedIdeograph6fe1', [], 'Display', None],
  [ 0x6fe2, 'CjkUnifiedIdeograph6fe2', [], 'Display', None],
  [ 0x6fe3, 'CjkUnifiedIdeograph6fe3', [], 'Display', None],
  [ 0x6fe4, 'CjkUnifiedIdeograph6fe4', [], 'Display', None],
  [ 0x6fe5, 'CjkUnifiedIdeograph6fe5', [], 'Display', None],
  [ 0x6fe6, 'CjkUnifiedIdeograph6fe6', [], 'Display', None],
  [ 0x6fe7, 'CjkUnifiedIdeograph6fe7', [], 'Display', None],
  [ 0x6fe8, 'CjkUnifiedIdeograph6fe8', [], 'Display', None],
  [ 0x6fe9, 'CjkUnifiedIdeograph6fe9', [], 'Display', None],
  [ 0x6fea, 'CjkUnifiedIdeograph6fea', [], 'Display', None],
  [ 0x6feb, 'CjkUnifiedIdeograph6feb', [], 'Display', None],
  [ 0x6fec, 'CjkUnifiedIdeograph6fec', [], 'Display', None],
  [ 0x6fed, 'CjkUnifiedIdeograph6fed', [], 'Display', None],
  [ 0x6fee, 'CjkUnifiedIdeograph6fee', [], 'Display', None],
  [ 0x6fef, 'CjkUnifiedIdeograph6fef', [], 'Display', None],
  [ 0x6ff0, 'CjkUnifiedIdeograph6ff0', [], 'Display', None],
  [ 0x6ff1, 'CjkUnifiedIdeograph6ff1', [], 'Display', None],
  [ 0x6ff2, 'CjkUnifiedIdeograph6ff2', [], 'Display', None],
  [ 0x6ff3, 'CjkUnifiedIdeograph6ff3', [], 'Display', None],
  [ 0x6ff4, 'CjkUnifiedIdeograph6ff4', [], 'Display', None],
  [ 0x6ff5, 'CjkUnifiedIdeograph6ff5', [], 'Display', None],
  [ 0x6ff6, 'CjkUnifiedIdeograph6ff6', [], 'Display', None],
  [ 0x6ff7, 'CjkUnifiedIdeograph6ff7', [], 'Display', None],
  [ 0x6ff8, 'CjkUnifiedIdeograph6ff8', [], 'Display', None],
  [ 0x6ff9, 'CjkUnifiedIdeograph6ff9', [], 'Display', None],
  [ 0x6ffa, 'CjkUnifiedIdeograph6ffa', [], 'Display', None],
  [ 0x6ffb, 'CjkUnifiedIdeograph6ffb', [], 'Display', None],
  [ 0x6ffc, 'CjkUnifiedIdeograph6ffc', [], 'Display', None],
  [ 0x6ffd, 'CjkUnifiedIdeograph6ffd', [], 'Display', None],
  [ 0x6ffe, 'CjkUnifiedIdeograph6ffe', [], 'Display', None],
  [ 0x6fff, 'CjkUnifiedIdeograph6fff', [], 'Display', None],
  [ 0x7000, 'CjkUnifiedIdeograph7000', [], 'Display', None],
  [ 0x7001, 'CjkUnifiedIdeograph7001', [], 'Display', None],
  [ 0x7002, 'CjkUnifiedIdeograph7002', [], 'Display', None],
  [ 0x7003, 'CjkUnifiedIdeograph7003', [], 'Display', None],
  [ 0x7004, 'CjkUnifiedIdeograph7004', [], 'Display', None],
  [ 0x7005, 'CjkUnifiedIdeograph7005', [], 'Display', None],
  [ 0x7006, 'CjkUnifiedIdeograph7006', [], 'Display', None],
  [ 0x7007, 'CjkUnifiedIdeograph7007', [], 'Display', None],
  [ 0x7008, 'CjkUnifiedIdeograph7008', [], 'Display', None],
  [ 0x7009, 'CjkUnifiedIdeograph7009', [], 'Display', None],
  [ 0x700a, 'CjkUnifiedIdeograph700a', [], 'Display', None],
  [ 0x700b, 'CjkUnifiedIdeograph700b', [], 'Display', None],
  [ 0x700c, 'CjkUnifiedIdeograph700c', [], 'Display', None],
  [ 0x700d, 'CjkUnifiedIdeograph700d', [], 'Display', None],
  [ 0x700e, 'CjkUnifiedIdeograph700e', [], 'Display', None],
  [ 0x700f, 'CjkUnifiedIdeograph700f', [], 'Display', None],
  [ 0x7010, 'CjkUnifiedIdeograph7010', [], 'Display', None],
  [ 0x7011, 'CjkUnifiedIdeograph7011', [], 'Display', None],
  [ 0x7012, 'CjkUnifiedIdeograph7012', [], 'Display', None],
  [ 0x7013, 'CjkUnifiedIdeograph7013', [], 'Display', None],
  [ 0x7014, 'CjkUnifiedIdeograph7014', [], 'Display', None],
  [ 0x7015, 'CjkUnifiedIdeograph7015', [], 'Display', None],
  [ 0x7016, 'CjkUnifiedIdeograph7016', [], 'Display', None],
  [ 0x7017, 'CjkUnifiedIdeograph7017', [], 'Display', None],
  [ 0x7018, 'CjkUnifiedIdeograph7018', [], 'Display', None],
  [ 0x7019, 'CjkUnifiedIdeograph7019', [], 'Display', None],
  [ 0x701a, 'CjkUnifiedIdeograph701a', [], 'Display', None],
  [ 0x701b, 'CjkUnifiedIdeograph701b', [], 'Display', None],
  [ 0x701c, 'CjkUnifiedIdeograph701c', [], 'Display', None],
  [ 0x701d, 'CjkUnifiedIdeograph701d', [], 'Display', None],
  [ 0x701e, 'CjkUnifiedIdeograph701e', [], 'Display', None],
  [ 0x701f, 'CjkUnifiedIdeograph701f', [], 'Display', None],
  [ 0x7020, 'CjkUnifiedIdeograph7020', [], 'Display', None],
  [ 0x7021, 'CjkUnifiedIdeograph7021', [], 'Display', None],
  [ 0x7022, 'CjkUnifiedIdeograph7022', [], 'Display', None],
  [ 0x7023, 'CjkUnifiedIdeograph7023', [], 'Display', None],
  [ 0x7024, 'CjkUnifiedIdeograph7024', [], 'Display', None],
  [ 0x7025, 'CjkUnifiedIdeograph7025', [], 'Display', None],
  [ 0x7026, 'CjkUnifiedIdeograph7026', [], 'Display', None],
  [ 0x7027, 'CjkUnifiedIdeograph7027', [], 'Display', None],
  [ 0x7028, 'CjkUnifiedIdeograph7028', [], 'Display', None],
  [ 0x7029, 'CjkUnifiedIdeograph7029', [], 'Display', None],
  [ 0x702a, 'CjkUnifiedIdeograph702a', [], 'Display', None],
  [ 0x702b, 'CjkUnifiedIdeograph702b', [], 'Display', None],
  [ 0x702c, 'CjkUnifiedIdeograph702c', [], 'Display', None],
  [ 0x702d, 'CjkUnifiedIdeograph702d', [], 'Display', None],
  [ 0x702e, 'CjkUnifiedIdeograph702e', [], 'Display', None],
  [ 0x702f, 'CjkUnifiedIdeograph702f', [], 'Display', None],
  [ 0x7030, 'CjkUnifiedIdeograph7030', [], 'Display', None],
  [ 0x7031, 'CjkUnifiedIdeograph7031', [], 'Display', None],
  [ 0x7032, 'CjkUnifiedIdeograph7032', [], 'Display', None],
  [ 0x7033, 'CjkUnifiedIdeograph7033', [], 'Display', None],
  [ 0x7034, 'CjkUnifiedIdeograph7034', [], 'Display', None],
  [ 0x7035, 'CjkUnifiedIdeograph7035', [], 'Display', None],
  [ 0x7036, 'CjkUnifiedIdeograph7036', [], 'Display', None],
  [ 0x7037, 'CjkUnifiedIdeograph7037', [], 'Display', None],
  [ 0x7038, 'CjkUnifiedIdeograph7038', [], 'Display', None],
  [ 0x7039, 'CjkUnifiedIdeograph7039', [], 'Display', None],
  [ 0x703a, 'CjkUnifiedIdeograph703a', [], 'Display', None],
  [ 0x703b, 'CjkUnifiedIdeograph703b', [], 'Display', None],
  [ 0x703c, 'CjkUnifiedIdeograph703c', [], 'Display', None],
  [ 0x703d, 'CjkUnifiedIdeograph703d', [], 'Display', None],
  [ 0x703e, 'CjkUnifiedIdeograph703e', [], 'Display', None],
  [ 0x703f, 'CjkUnifiedIdeograph703f', [], 'Display', None],
  [ 0x7040, 'CjkUnifiedIdeograph7040', [], 'Display', None],
  [ 0x7041, 'CjkUnifiedIdeograph7041', [], 'Display', None],
  [ 0x7042, 'CjkUnifiedIdeograph7042', [], 'Display', None],
  [ 0x7043, 'CjkUnifiedIdeograph7043', [], 'Display', None],
  [ 0x7044, 'CjkUnifiedIdeograph7044', [], 'Display', None],
  [ 0x7045, 'CjkUnifiedIdeograph7045', [], 'Display', None],
  [ 0x7046, 'CjkUnifiedIdeograph7046', [], 'Display', None],
  [ 0x7047, 'CjkUnifiedIdeograph7047', [], 'Display', None],
  [ 0x7048, 'CjkUnifiedIdeograph7048', [], 'Display', None],
  [ 0x7049, 'CjkUnifiedIdeograph7049', [], 'Display', None],
  [ 0x704a, 'CjkUnifiedIdeograph704a', [], 'Display', None],
  [ 0x704b, 'CjkUnifiedIdeograph704b', [], 'Display', None],
  [ 0x704c, 'CjkUnifiedIdeograph704c', [], 'Display', None],
  [ 0x704d, 'CjkUnifiedIdeograph704d', [], 'Display', None],
  [ 0x704e, 'CjkUnifiedIdeograph704e', [], 'Display', None],
  [ 0x704f, 'CjkUnifiedIdeograph704f', [], 'Display', None],
  [ 0x7050, 'CjkUnifiedIdeograph7050', [], 'Display', None],
  [ 0x7051, 'CjkUnifiedIdeograph7051', [], 'Display', None],
  [ 0x7052, 'CjkUnifiedIdeograph7052', [], 'Display', None],
  [ 0x7053, 'CjkUnifiedIdeograph7053', [], 'Display', None],
  [ 0x7054, 'CjkUnifiedIdeograph7054', [], 'Display', None],
  [ 0x7055, 'CjkUnifiedIdeograph7055', [], 'Display', None],
  [ 0x7056, 'CjkUnifiedIdeograph7056', [], 'Display', None],
  [ 0x7057, 'CjkUnifiedIdeograph7057', [], 'Display', None],
  [ 0x7058, 'CjkUnifiedIdeograph7058', [], 'Display', None],
  [ 0x7059, 'CjkUnifiedIdeograph7059', [], 'Display', None],
  [ 0x705a, 'CjkUnifiedIdeograph705a', [], 'Display', None],
  [ 0x705b, 'CjkUnifiedIdeograph705b', [], 'Display', None],
  [ 0x705c, 'CjkUnifiedIdeograph705c', [], 'Display', None],
  [ 0x705d, 'CjkUnifiedIdeograph705d', [], 'Display', None],
  [ 0x705e, 'CjkUnifiedIdeograph705e', [], 'Display', None],
  [ 0x705f, 'CjkUnifiedIdeograph705f', [], 'Display', None],
  [ 0x7060, 'CjkUnifiedIdeograph7060', [], 'Display', None],
  [ 0x7061, 'CjkUnifiedIdeograph7061', [], 'Display', None],
  [ 0x7062, 'CjkUnifiedIdeograph7062', [], 'Display', None],
  [ 0x7063, 'CjkUnifiedIdeograph7063', [], 'Display', None],
  [ 0x7064, 'CjkUnifiedIdeograph7064', [], 'Display', None],
  [ 0x7065, 'CjkUnifiedIdeograph7065', [], 'Display', None],
  [ 0x7066, 'CjkUnifiedIdeograph7066', [], 'Display', None],
  [ 0x7067, 'CjkUnifiedIdeograph7067', [], 'Display', None],
  [ 0x7068, 'CjkUnifiedIdeograph7068', [], 'Display', None],
  [ 0x7069, 'CjkUnifiedIdeograph7069', [], 'Display', None],
  [ 0x706a, 'CjkUnifiedIdeograph706a', [], 'Display', None],
  [ 0x706b, 'CjkUnifiedIdeograph706b', [], 'Display', None],
  [ 0x706c, 'CjkUnifiedIdeograph706c', [], 'Display', None],
  [ 0x706d, 'CjkUnifiedIdeograph706d', [], 'Display', None],
  [ 0x706e, 'CjkUnifiedIdeograph706e', [], 'Display', None],
  [ 0x706f, 'CjkUnifiedIdeograph706f', [], 'Display', None],
  [ 0x7070, 'CjkUnifiedIdeograph7070', [], 'Display', None],
  [ 0x7071, 'CjkUnifiedIdeograph7071', [], 'Display', None],
  [ 0x7072, 'CjkUnifiedIdeograph7072', [], 'Display', None],
  [ 0x7073, 'CjkUnifiedIdeograph7073', [], 'Display', None],
  [ 0x7074, 'CjkUnifiedIdeograph7074', [], 'Display', None],
  [ 0x7075, 'CjkUnifiedIdeograph7075', [], 'Display', None],
  [ 0x7076, 'CjkUnifiedIdeograph7076', [], 'Display', None],
  [ 0x7077, 'CjkUnifiedIdeograph7077', [], 'Display', None],
  [ 0x7078, 'CjkUnifiedIdeograph7078', [], 'Display', None],
  [ 0x7079, 'CjkUnifiedIdeograph7079', [], 'Display', None],
  [ 0x707a, 'CjkUnifiedIdeograph707a', [], 'Display', None],
  [ 0x707b, 'CjkUnifiedIdeograph707b', [], 'Display', None],
  [ 0x707c, 'CjkUnifiedIdeograph707c', [], 'Display', None],
  [ 0x707d, 'CjkUnifiedIdeograph707d', [], 'Display', None],
  [ 0x707e, 'CjkUnifiedIdeograph707e', [], 'Display', None],
  [ 0x707f, 'CjkUnifiedIdeograph707f', [], 'Display', None],
  [ 0x7080, 'CjkUnifiedIdeograph7080', [], 'Display', None],
  [ 0x7081, 'CjkUnifiedIdeograph7081', [], 'Display', None],
  [ 0x7082, 'CjkUnifiedIdeograph7082', [], 'Display', None],
  [ 0x7083, 'CjkUnifiedIdeograph7083', [], 'Display', None],
  [ 0x7084, 'CjkUnifiedIdeograph7084', [], 'Display', None],
  [ 0x7085, 'CjkUnifiedIdeograph7085', [], 'Display', None],
  [ 0x7086, 'CjkUnifiedIdeograph7086', [], 'Display', None],
  [ 0x7087, 'CjkUnifiedIdeograph7087', [], 'Display', None],
  [ 0x7088, 'CjkUnifiedIdeograph7088', [], 'Display', None],
  [ 0x7089, 'CjkUnifiedIdeograph7089', [], 'Display', None],
  [ 0x708a, 'CjkUnifiedIdeograph708a', [], 'Display', None],
  [ 0x708b, 'CjkUnifiedIdeograph708b', [], 'Display', None],
  [ 0x708c, 'CjkUnifiedIdeograph708c', [], 'Display', None],
  [ 0x708d, 'CjkUnifiedIdeograph708d', [], 'Display', None],
  [ 0x708e, 'CjkUnifiedIdeograph708e', [], 'Display', None],
  [ 0x708f, 'CjkUnifiedIdeograph708f', [], 'Display', None],
  [ 0x7090, 'CjkUnifiedIdeograph7090', [], 'Display', None],
  [ 0x7091, 'CjkUnifiedIdeograph7091', [], 'Display', None],
  [ 0x7092, 'CjkUnifiedIdeograph7092', [], 'Display', None],
  [ 0x7093, 'CjkUnifiedIdeograph7093', [], 'Display', None],
  [ 0x7094, 'CjkUnifiedIdeograph7094', [], 'Display', None],
  [ 0x7095, 'CjkUnifiedIdeograph7095', [], 'Display', None],
  [ 0x7096, 'CjkUnifiedIdeograph7096', [], 'Display', None],
  [ 0x7097, 'CjkUnifiedIdeograph7097', [], 'Display', None],
  [ 0x7098, 'CjkUnifiedIdeograph7098', [], 'Display', None],
  [ 0x7099, 'CjkUnifiedIdeograph7099', [], 'Display', None],
  [ 0x709a, 'CjkUnifiedIdeograph709a', [], 'Display', None],
  [ 0x709b, 'CjkUnifiedIdeograph709b', [], 'Display', None],
  [ 0x709c, 'CjkUnifiedIdeograph709c', [], 'Display', None],
  [ 0x709d, 'CjkUnifiedIdeograph709d', [], 'Display', None],
  [ 0x709e, 'CjkUnifiedIdeograph709e', [], 'Display', None],
  [ 0x709f, 'CjkUnifiedIdeograph709f', [], 'Display', None],
  [ 0x70a0, 'CjkUnifiedIdeograph70a0', [], 'Display', None],
  [ 0x70a1, 'CjkUnifiedIdeograph70a1', [], 'Display', None],
  [ 0x70a2, 'CjkUnifiedIdeograph70a2', [], 'Display', None],
  [ 0x70a3, 'CjkUnifiedIdeograph70a3', [], 'Display', None],
  [ 0x70a4, 'CjkUnifiedIdeograph70a4', [], 'Display', None],
  [ 0x70a5, 'CjkUnifiedIdeograph70a5', [], 'Display', None],
  [ 0x70a6, 'CjkUnifiedIdeograph70a6', [], 'Display', None],
  [ 0x70a7, 'CjkUnifiedIdeograph70a7', [], 'Display', None],
  [ 0x70a8, 'CjkUnifiedIdeograph70a8', [], 'Display', None],
  [ 0x70a9, 'CjkUnifiedIdeograph70a9', [], 'Display', None],
  [ 0x70aa, 'CjkUnifiedIdeograph70aa', [], 'Display', None],
  [ 0x70ab, 'CjkUnifiedIdeograph70ab', [], 'Display', None],
  [ 0x70ac, 'CjkUnifiedIdeograph70ac', [], 'Display', None],
  [ 0x70ad, 'CjkUnifiedIdeograph70ad', [], 'Display', None],
  [ 0x70ae, 'CjkUnifiedIdeograph70ae', [], 'Display', None],
  [ 0x70af, 'CjkUnifiedIdeograph70af', [], 'Display', None],
  [ 0x70b0, 'CjkUnifiedIdeograph70b0', [], 'Display', None],
  [ 0x70b1, 'CjkUnifiedIdeograph70b1', [], 'Display', None],
  [ 0x70b2, 'CjkUnifiedIdeograph70b2', [], 'Display', None],
  [ 0x70b3, 'CjkUnifiedIdeograph70b3', [], 'Display', None],
  [ 0x70b4, 'CjkUnifiedIdeograph70b4', [], 'Display', None],
  [ 0x70b5, 'CjkUnifiedIdeograph70b5', [], 'Display', None],
  [ 0x70b6, 'CjkUnifiedIdeograph70b6', [], 'Display', None],
  [ 0x70b7, 'CjkUnifiedIdeograph70b7', [], 'Display', None],
  [ 0x70b8, 'CjkUnifiedIdeograph70b8', [], 'Display', None],
  [ 0x70b9, 'CjkUnifiedIdeograph70b9', [], 'Display', None],
  [ 0x70ba, 'CjkUnifiedIdeograph70ba', [], 'Display', None],
  [ 0x70bb, 'CjkUnifiedIdeograph70bb', [], 'Display', None],
  [ 0x70bc, 'CjkUnifiedIdeograph70bc', [], 'Display', None],
  [ 0x70bd, 'CjkUnifiedIdeograph70bd', [], 'Display', None],
  [ 0x70be, 'CjkUnifiedIdeograph70be', [], 'Display', None],
  [ 0x70bf, 'CjkUnifiedIdeograph70bf', [], 'Display', None],
  [ 0x70c0, 'CjkUnifiedIdeograph70c0', [], 'Display', None],
  [ 0x70c1, 'CjkUnifiedIdeograph70c1', [], 'Display', None],
  [ 0x70c2, 'CjkUnifiedIdeograph70c2', [], 'Display', None],
  [ 0x70c3, 'CjkUnifiedIdeograph70c3', [], 'Display', None],
  [ 0x70c4, 'CjkUnifiedIdeograph70c4', [], 'Display', None],
  [ 0x70c5, 'CjkUnifiedIdeograph70c5', [], 'Display', None],
  [ 0x70c6, 'CjkUnifiedIdeograph70c6', [], 'Display', None],
  [ 0x70c7, 'CjkUnifiedIdeograph70c7', [], 'Display', None],
  [ 0x70c8, 'CjkUnifiedIdeograph70c8', [], 'Display', None],
  [ 0x70c9, 'CjkUnifiedIdeograph70c9', [], 'Display', None],
  [ 0x70ca, 'CjkUnifiedIdeograph70ca', [], 'Display', None],
  [ 0x70cb, 'CjkUnifiedIdeograph70cb', [], 'Display', None],
  [ 0x70cc, 'CjkUnifiedIdeograph70cc', [], 'Display', None],
  [ 0x70cd, 'CjkUnifiedIdeograph70cd', [], 'Display', None],
  [ 0x70ce, 'CjkUnifiedIdeograph70ce', [], 'Display', None],
  [ 0x70cf, 'CjkUnifiedIdeograph70cf', [], 'Display', None],
  [ 0x70d0, 'CjkUnifiedIdeograph70d0', [], 'Display', None],
  [ 0x70d1, 'CjkUnifiedIdeograph70d1', [], 'Display', None],
  [ 0x70d2, 'CjkUnifiedIdeograph70d2', [], 'Display', None],
  [ 0x70d3, 'CjkUnifiedIdeograph70d3', [], 'Display', None],
  [ 0x70d4, 'CjkUnifiedIdeograph70d4', [], 'Display', None],
  [ 0x70d5, 'CjkUnifiedIdeograph70d5', [], 'Display', None],
  [ 0x70d6, 'CjkUnifiedIdeograph70d6', [], 'Display', None],
  [ 0x70d7, 'CjkUnifiedIdeograph70d7', [], 'Display', None],
  [ 0x70d8, 'CjkUnifiedIdeograph70d8', [], 'Display', None],
  [ 0x70d9, 'CjkUnifiedIdeograph70d9', [], 'Display', None],
  [ 0x70da, 'CjkUnifiedIdeograph70da', [], 'Display', None],
  [ 0x70db, 'CjkUnifiedIdeograph70db', [], 'Display', None],
  [ 0x70dc, 'CjkUnifiedIdeograph70dc', [], 'Display', None],
  [ 0x70dd, 'CjkUnifiedIdeograph70dd', [], 'Display', None],
  [ 0x70de, 'CjkUnifiedIdeograph70de', [], 'Display', None],
  [ 0x70df, 'CjkUnifiedIdeograph70df', [], 'Display', None],
  [ 0x70e0, 'CjkUnifiedIdeograph70e0', [], 'Display', None],
  [ 0x70e1, 'CjkUnifiedIdeograph70e1', [], 'Display', None],
  [ 0x70e2, 'CjkUnifiedIdeograph70e2', [], 'Display', None],
  [ 0x70e3, 'CjkUnifiedIdeograph70e3', [], 'Display', None],
  [ 0x70e4, 'CjkUnifiedIdeograph70e4', [], 'Display', None],
  [ 0x70e5, 'CjkUnifiedIdeograph70e5', [], 'Display', None],
  [ 0x70e6, 'CjkUnifiedIdeograph70e6', [], 'Display', None],
  [ 0x70e7, 'CjkUnifiedIdeograph70e7', [], 'Display', None],
  [ 0x70e8, 'CjkUnifiedIdeograph70e8', [], 'Display', None],
  [ 0x70e9, 'CjkUnifiedIdeograph70e9', [], 'Display', None],
  [ 0x70ea, 'CjkUnifiedIdeograph70ea', [], 'Display', None],
  [ 0x70eb, 'CjkUnifiedIdeograph70eb', [], 'Display', None],
  [ 0x70ec, 'CjkUnifiedIdeograph70ec', [], 'Display', None],
  [ 0x70ed, 'CjkUnifiedIdeograph70ed', [], 'Display', None],
  [ 0x70ee, 'CjkUnifiedIdeograph70ee', [], 'Display', None],
  [ 0x70ef, 'CjkUnifiedIdeograph70ef', [], 'Display', None],
  [ 0x70f0, 'CjkUnifiedIdeograph70f0', [], 'Display', None],
  [ 0x70f1, 'CjkUnifiedIdeograph70f1', [], 'Display', None],
  [ 0x70f2, 'CjkUnifiedIdeograph70f2', [], 'Display', None],
  [ 0x70f3, 'CjkUnifiedIdeograph70f3', [], 'Display', None],
  [ 0x70f4, 'CjkUnifiedIdeograph70f4', [], 'Display', None],
  [ 0x70f5, 'CjkUnifiedIdeograph70f5', [], 'Display', None],
  [ 0x70f6, 'CjkUnifiedIdeograph70f6', [], 'Display', None],
  [ 0x70f7, 'CjkUnifiedIdeograph70f7', [], 'Display', None],
  [ 0x70f8, 'CjkUnifiedIdeograph70f8', [], 'Display', None],
  [ 0x70f9, 'CjkUnifiedIdeograph70f9', [], 'Display', None],
  [ 0x70fa, 'CjkUnifiedIdeograph70fa', [], 'Display', None],
  [ 0x70fb, 'CjkUnifiedIdeograph70fb', [], 'Display', None],
  [ 0x70fc, 'CjkUnifiedIdeograph70fc', [], 'Display', None],
  [ 0x70fd, 'CjkUnifiedIdeograph70fd', [], 'Display', None],
  [ 0x70fe, 'CjkUnifiedIdeograph70fe', [], 'Display', None],
  [ 0x70ff, 'CjkUnifiedIdeograph70ff', [], 'Display', None],
  [ 0x7100, 'CjkUnifiedIdeograph7100', [], 'Display', None],
  [ 0x7101, 'CjkUnifiedIdeograph7101', [], 'Display', None],
  [ 0x7102, 'CjkUnifiedIdeograph7102', [], 'Display', None],
  [ 0x7103, 'CjkUnifiedIdeograph7103', [], 'Display', None],
  [ 0x7104, 'CjkUnifiedIdeograph7104', [], 'Display', None],
  [ 0x7105, 'CjkUnifiedIdeograph7105', [], 'Display', None],
  [ 0x7106, 'CjkUnifiedIdeograph7106', [], 'Display', None],
  [ 0x7107, 'CjkUnifiedIdeograph7107', [], 'Display', None],
  [ 0x7108, 'CjkUnifiedIdeograph7108', [], 'Display', None],
  [ 0x7109, 'CjkUnifiedIdeograph7109', [], 'Display', None],
  [ 0x710a, 'CjkUnifiedIdeograph710a', [], 'Display', None],
  [ 0x710b, 'CjkUnifiedIdeograph710b', [], 'Display', None],
  [ 0x710c, 'CjkUnifiedIdeograph710c', [], 'Display', None],
  [ 0x710d, 'CjkUnifiedIdeograph710d', [], 'Display', None],
  [ 0x710e, 'CjkUnifiedIdeograph710e', [], 'Display', None],
  [ 0x710f, 'CjkUnifiedIdeograph710f', [], 'Display', None],
  [ 0x7110, 'CjkUnifiedIdeograph7110', [], 'Display', None],
  [ 0x7111, 'CjkUnifiedIdeograph7111', [], 'Display', None],
  [ 0x7112, 'CjkUnifiedIdeograph7112', [], 'Display', None],
  [ 0x7113, 'CjkUnifiedIdeograph7113', [], 'Display', None],
  [ 0x7114, 'CjkUnifiedIdeograph7114', [], 'Display', None],
  [ 0x7115, 'CjkUnifiedIdeograph7115', [], 'Display', None],
  [ 0x7116, 'CjkUnifiedIdeograph7116', [], 'Display', None],
  [ 0x7117, 'CjkUnifiedIdeograph7117', [], 'Display', None],
  [ 0x7118, 'CjkUnifiedIdeograph7118', [], 'Display', None],
  [ 0x7119, 'CjkUnifiedIdeograph7119', [], 'Display', None],
  [ 0x711a, 'CjkUnifiedIdeograph711a', [], 'Display', None],
  [ 0x711b, 'CjkUnifiedIdeograph711b', [], 'Display', None],
  [ 0x711c, 'CjkUnifiedIdeograph711c', [], 'Display', None],
  [ 0x711d, 'CjkUnifiedIdeograph711d', [], 'Display', None],
  [ 0x711e, 'CjkUnifiedIdeograph711e', [], 'Display', None],
  [ 0x711f, 'CjkUnifiedIdeograph711f', [], 'Display', None],
  [ 0x7120, 'CjkUnifiedIdeograph7120', [], 'Display', None],
  [ 0x7121, 'CjkUnifiedIdeograph7121', [], 'Display', None],
  [ 0x7122, 'CjkUnifiedIdeograph7122', [], 'Display', None],
  [ 0x7123, 'CjkUnifiedIdeograph7123', [], 'Display', None],
  [ 0x7124, 'CjkUnifiedIdeograph7124', [], 'Display', None],
  [ 0x7125, 'CjkUnifiedIdeograph7125', [], 'Display', None],
  [ 0x7126, 'CjkUnifiedIdeograph7126', [], 'Display', None],
  [ 0x7127, 'CjkUnifiedIdeograph7127', [], 'Display', None],
  [ 0x7128, 'CjkUnifiedIdeograph7128', [], 'Display', None],
  [ 0x7129, 'CjkUnifiedIdeograph7129', [], 'Display', None],
  [ 0x712a, 'CjkUnifiedIdeograph712a', [], 'Display', None],
  [ 0x712b, 'CjkUnifiedIdeograph712b', [], 'Display', None],
  [ 0x712c, 'CjkUnifiedIdeograph712c', [], 'Display', None],
  [ 0x712d, 'CjkUnifiedIdeograph712d', [], 'Display', None],
  [ 0x712e, 'CjkUnifiedIdeograph712e', [], 'Display', None],
  [ 0x712f, 'CjkUnifiedIdeograph712f', [], 'Display', None],
  [ 0x7130, 'CjkUnifiedIdeograph7130', [], 'Display', None],
  [ 0x7131, 'CjkUnifiedIdeograph7131', [], 'Display', None],
  [ 0x7132, 'CjkUnifiedIdeograph7132', [], 'Display', None],
  [ 0x7133, 'CjkUnifiedIdeograph7133', [], 'Display', None],
  [ 0x7134, 'CjkUnifiedIdeograph7134', [], 'Display', None],
  [ 0x7135, 'CjkUnifiedIdeograph7135', [], 'Display', None],
  [ 0x7136, 'CjkUnifiedIdeograph7136', [], 'Display', None],
  [ 0x7137, 'CjkUnifiedIdeograph7137', [], 'Display', None],
  [ 0x7138, 'CjkUnifiedIdeograph7138', [], 'Display', None],
  [ 0x7139, 'CjkUnifiedIdeograph7139', [], 'Display', None],
  [ 0x713a, 'CjkUnifiedIdeograph713a', [], 'Display', None],
  [ 0x713b, 'CjkUnifiedIdeograph713b', [], 'Display', None],
  [ 0x713c, 'CjkUnifiedIdeograph713c', [], 'Display', None],
  [ 0x713d, 'CjkUnifiedIdeograph713d', [], 'Display', None],
  [ 0x713e, 'CjkUnifiedIdeograph713e', [], 'Display', None],
  [ 0x713f, 'CjkUnifiedIdeograph713f', [], 'Display', None],
  [ 0x7140, 'CjkUnifiedIdeograph7140', [], 'Display', None],
  [ 0x7141, 'CjkUnifiedIdeograph7141', [], 'Display', None],
  [ 0x7142, 'CjkUnifiedIdeograph7142', [], 'Display', None],
  [ 0x7143, 'CjkUnifiedIdeograph7143', [], 'Display', None],
  [ 0x7144, 'CjkUnifiedIdeograph7144', [], 'Display', None],
  [ 0x7145, 'CjkUnifiedIdeograph7145', [], 'Display', None],
  [ 0x7146, 'CjkUnifiedIdeograph7146', [], 'Display', None],
  [ 0x7147, 'CjkUnifiedIdeograph7147', [], 'Display', None],
  [ 0x7148, 'CjkUnifiedIdeograph7148', [], 'Display', None],
  [ 0x7149, 'CjkUnifiedIdeograph7149', [], 'Display', None],
  [ 0x714a, 'CjkUnifiedIdeograph714a', [], 'Display', None],
  [ 0x714b, 'CjkUnifiedIdeograph714b', [], 'Display', None],
  [ 0x714c, 'CjkUnifiedIdeograph714c', [], 'Display', None],
  [ 0x714d, 'CjkUnifiedIdeograph714d', [], 'Display', None],
  [ 0x714e, 'CjkUnifiedIdeograph714e', [], 'Display', None],
  [ 0x714f, 'CjkUnifiedIdeograph714f', [], 'Display', None],
  [ 0x7150, 'CjkUnifiedIdeograph7150', [], 'Display', None],
  [ 0x7151, 'CjkUnifiedIdeograph7151', [], 'Display', None],
  [ 0x7152, 'CjkUnifiedIdeograph7152', [], 'Display', None],
  [ 0x7153, 'CjkUnifiedIdeograph7153', [], 'Display', None],
  [ 0x7154, 'CjkUnifiedIdeograph7154', [], 'Display', None],
  [ 0x7155, 'CjkUnifiedIdeograph7155', [], 'Display', None],
  [ 0x7156, 'CjkUnifiedIdeograph7156', [], 'Display', None],
  [ 0x7157, 'CjkUnifiedIdeograph7157', [], 'Display', None],
  [ 0x7158, 'CjkUnifiedIdeograph7158', [], 'Display', None],
  [ 0x7159, 'CjkUnifiedIdeograph7159', [], 'Display', None],
  [ 0x715a, 'CjkUnifiedIdeograph715a', [], 'Display', None],
  [ 0x715b, 'CjkUnifiedIdeograph715b', [], 'Display', None],
  [ 0x715c, 'CjkUnifiedIdeograph715c', [], 'Display', None],
  [ 0x715d, 'CjkUnifiedIdeograph715d', [], 'Display', None],
  [ 0x715e, 'CjkUnifiedIdeograph715e', [], 'Display', None],
  [ 0x715f, 'CjkUnifiedIdeograph715f', [], 'Display', None],
  [ 0x7160, 'CjkUnifiedIdeograph7160', [], 'Display', None],
  [ 0x7161, 'CjkUnifiedIdeograph7161', [], 'Display', None],
  [ 0x7162, 'CjkUnifiedIdeograph7162', [], 'Display', None],
  [ 0x7163, 'CjkUnifiedIdeograph7163', [], 'Display', None],
  [ 0x7164, 'CjkUnifiedIdeograph7164', [], 'Display', None],
  [ 0x7165, 'CjkUnifiedIdeograph7165', [], 'Display', None],
  [ 0x7166, 'CjkUnifiedIdeograph7166', [], 'Display', None],
  [ 0x7167, 'CjkUnifiedIdeograph7167', [], 'Display', None],
  [ 0x7168, 'CjkUnifiedIdeograph7168', [], 'Display', None],
  [ 0x7169, 'CjkUnifiedIdeograph7169', [], 'Display', None],
  [ 0x716a, 'CjkUnifiedIdeograph716a', [], 'Display', None],
  [ 0x716b, 'CjkUnifiedIdeograph716b', [], 'Display', None],
  [ 0x716c, 'CjkUnifiedIdeograph716c', [], 'Display', None],
  [ 0x716d, 'CjkUnifiedIdeograph716d', [], 'Display', None],
  [ 0x716e, 'CjkUnifiedIdeograph716e', [], 'Display', None],
  [ 0x716f, 'CjkUnifiedIdeograph716f', [], 'Display', None],
  [ 0x7170, 'CjkUnifiedIdeograph7170', [], 'Display', None],
  [ 0x7171, 'CjkUnifiedIdeograph7171', [], 'Display', None],
  [ 0x7172, 'CjkUnifiedIdeograph7172', [], 'Display', None],
  [ 0x7173, 'CjkUnifiedIdeograph7173', [], 'Display', None],
  [ 0x7174, 'CjkUnifiedIdeograph7174', [], 'Display', None],
  [ 0x7175, 'CjkUnifiedIdeograph7175', [], 'Display', None],
  [ 0x7176, 'CjkUnifiedIdeograph7176', [], 'Display', None],
  [ 0x7177, 'CjkUnifiedIdeograph7177', [], 'Display', None],
  [ 0x7178, 'CjkUnifiedIdeograph7178', [], 'Display', None],
  [ 0x7179, 'CjkUnifiedIdeograph7179', [], 'Display', None],
  [ 0x717a, 'CjkUnifiedIdeograph717a', [], 'Display', None],
  [ 0x717b, 'CjkUnifiedIdeograph717b', [], 'Display', None],
  [ 0x717c, 'CjkUnifiedIdeograph717c', [], 'Display', None],
  [ 0x717d, 'CjkUnifiedIdeograph717d', [], 'Display', None],
  [ 0x717e, 'CjkUnifiedIdeograph717e', [], 'Display', None],
  [ 0x717f, 'CjkUnifiedIdeograph717f', [], 'Display', None],
  [ 0x7180, 'CjkUnifiedIdeograph7180', [], 'Display', None],
  [ 0x7181, 'CjkUnifiedIdeograph7181', [], 'Display', None],
  [ 0x7182, 'CjkUnifiedIdeograph7182', [], 'Display', None],
  [ 0x7183, 'CjkUnifiedIdeograph7183', [], 'Display', None],
  [ 0x7184, 'CjkUnifiedIdeograph7184', [], 'Display', None],
  [ 0x7185, 'CjkUnifiedIdeograph7185', [], 'Display', None],
  [ 0x7186, 'CjkUnifiedIdeograph7186', [], 'Display', None],
  [ 0x7187, 'CjkUnifiedIdeograph7187', [], 'Display', None],
  [ 0x7188, 'CjkUnifiedIdeograph7188', [], 'Display', None],
  [ 0x7189, 'CjkUnifiedIdeograph7189', [], 'Display', None],
  [ 0x718a, 'CjkUnifiedIdeograph718a', [], 'Display', None],
  [ 0x718b, 'CjkUnifiedIdeograph718b', [], 'Display', None],
  [ 0x718c, 'CjkUnifiedIdeograph718c', [], 'Display', None],
  [ 0x718d, 'CjkUnifiedIdeograph718d', [], 'Display', None],
  [ 0x718e, 'CjkUnifiedIdeograph718e', [], 'Display', None],
  [ 0x718f, 'CjkUnifiedIdeograph718f', [], 'Display', None],
  [ 0x7190, 'CjkUnifiedIdeograph7190', [], 'Display', None],
  [ 0x7191, 'CjkUnifiedIdeograph7191', [], 'Display', None],
  [ 0x7192, 'CjkUnifiedIdeograph7192', [], 'Display', None],
  [ 0x7193, 'CjkUnifiedIdeograph7193', [], 'Display', None],
  [ 0x7194, 'CjkUnifiedIdeograph7194', [], 'Display', None],
  [ 0x7195, 'CjkUnifiedIdeograph7195', [], 'Display', None],
  [ 0x7196, 'CjkUnifiedIdeograph7196', [], 'Display', None],
  [ 0x7197, 'CjkUnifiedIdeograph7197', [], 'Display', None],
  [ 0x7198, 'CjkUnifiedIdeograph7198', [], 'Display', None],
  [ 0x7199, 'CjkUnifiedIdeograph7199', [], 'Display', None],
  [ 0x719a, 'CjkUnifiedIdeograph719a', [], 'Display', None],
  [ 0x719b, 'CjkUnifiedIdeograph719b', [], 'Display', None],
  [ 0x719c, 'CjkUnifiedIdeograph719c', [], 'Display', None],
  [ 0x719d, 'CjkUnifiedIdeograph719d', [], 'Display', None],
  [ 0x719e, 'CjkUnifiedIdeograph719e', [], 'Display', None],
  [ 0x719f, 'CjkUnifiedIdeograph719f', [], 'Display', None],
  [ 0x71a0, 'CjkUnifiedIdeograph71a0', [], 'Display', None],
  [ 0x71a1, 'CjkUnifiedIdeograph71a1', [], 'Display', None],
  [ 0x71a2, 'CjkUnifiedIdeograph71a2', [], 'Display', None],
  [ 0x71a3, 'CjkUnifiedIdeograph71a3', [], 'Display', None],
  [ 0x71a4, 'CjkUnifiedIdeograph71a4', [], 'Display', None],
  [ 0x71a5, 'CjkUnifiedIdeograph71a5', [], 'Display', None],
  [ 0x71a6, 'CjkUnifiedIdeograph71a6', [], 'Display', None],
  [ 0x71a7, 'CjkUnifiedIdeograph71a7', [], 'Display', None],
  [ 0x71a8, 'CjkUnifiedIdeograph71a8', [], 'Display', None],
  [ 0x71a9, 'CjkUnifiedIdeograph71a9', [], 'Display', None],
  [ 0x71aa, 'CjkUnifiedIdeograph71aa', [], 'Display', None],
  [ 0x71ab, 'CjkUnifiedIdeograph71ab', [], 'Display', None],
  [ 0x71ac, 'CjkUnifiedIdeograph71ac', [], 'Display', None],
  [ 0x71ad, 'CjkUnifiedIdeograph71ad', [], 'Display', None],
  [ 0x71ae, 'CjkUnifiedIdeograph71ae', [], 'Display', None],
  [ 0x71af, 'CjkUnifiedIdeograph71af', [], 'Display', None],
  [ 0x71b0, 'CjkUnifiedIdeograph71b0', [], 'Display', None],
  [ 0x71b1, 'CjkUnifiedIdeograph71b1', [], 'Display', None],
  [ 0x71b2, 'CjkUnifiedIdeograph71b2', [], 'Display', None],
  [ 0x71b3, 'CjkUnifiedIdeograph71b3', [], 'Display', None],
  [ 0x71b4, 'CjkUnifiedIdeograph71b4', [], 'Display', None],
  [ 0x71b5, 'CjkUnifiedIdeograph71b5', [], 'Display', None],
  [ 0x71b6, 'CjkUnifiedIdeograph71b6', [], 'Display', None],
  [ 0x71b7, 'CjkUnifiedIdeograph71b7', [], 'Display', None],
  [ 0x71b8, 'CjkUnifiedIdeograph71b8', [], 'Display', None],
  [ 0x71b9, 'CjkUnifiedIdeograph71b9', [], 'Display', None],
  [ 0x71ba, 'CjkUnifiedIdeograph71ba', [], 'Display', None],
  [ 0x71bb, 'CjkUnifiedIdeograph71bb', [], 'Display', None],
  [ 0x71bc, 'CjkUnifiedIdeograph71bc', [], 'Display', None],
  [ 0x71bd, 'CjkUnifiedIdeograph71bd', [], 'Display', None],
  [ 0x71be, 'CjkUnifiedIdeograph71be', [], 'Display', None],
  [ 0x71bf, 'CjkUnifiedIdeograph71bf', [], 'Display', None],
  [ 0x71c0, 'CjkUnifiedIdeograph71c0', [], 'Display', None],
  [ 0x71c1, 'CjkUnifiedIdeograph71c1', [], 'Display', None],
  [ 0x71c2, 'CjkUnifiedIdeograph71c2', [], 'Display', None],
  [ 0x71c3, 'CjkUnifiedIdeograph71c3', [], 'Display', None],
  [ 0x71c4, 'CjkUnifiedIdeograph71c4', [], 'Display', None],
  [ 0x71c5, 'CjkUnifiedIdeograph71c5', [], 'Display', None],
  [ 0x71c6, 'CjkUnifiedIdeograph71c6', [], 'Display', None],
  [ 0x71c7, 'CjkUnifiedIdeograph71c7', [], 'Display', None],
  [ 0x71c8, 'CjkUnifiedIdeograph71c8', [], 'Display', None],
  [ 0x71c9, 'CjkUnifiedIdeograph71c9', [], 'Display', None],
  [ 0x71ca, 'CjkUnifiedIdeograph71ca', [], 'Display', None],
  [ 0x71cb, 'CjkUnifiedIdeograph71cb', [], 'Display', None],
  [ 0x71cc, 'CjkUnifiedIdeograph71cc', [], 'Display', None],
  [ 0x71cd, 'CjkUnifiedIdeograph71cd', [], 'Display', None],
  [ 0x71ce, 'CjkUnifiedIdeograph71ce', [], 'Display', None],
  [ 0x71cf, 'CjkUnifiedIdeograph71cf', [], 'Display', None],
  [ 0x71d0, 'CjkUnifiedIdeograph71d0', [], 'Display', None],
  [ 0x71d1, 'CjkUnifiedIdeograph71d1', [], 'Display', None],
  [ 0x71d2, 'CjkUnifiedIdeograph71d2', [], 'Display', None],
  [ 0x71d3, 'CjkUnifiedIdeograph71d3', [], 'Display', None],
  [ 0x71d4, 'CjkUnifiedIdeograph71d4', [], 'Display', None],
  [ 0x71d5, 'CjkUnifiedIdeograph71d5', [], 'Display', None],
  [ 0x71d6, 'CjkUnifiedIdeograph71d6', [], 'Display', None],
  [ 0x71d7, 'CjkUnifiedIdeograph71d7', [], 'Display', None],
  [ 0x71d8, 'CjkUnifiedIdeograph71d8', [], 'Display', None],
  [ 0x71d9, 'CjkUnifiedIdeograph71d9', [], 'Display', None],
  [ 0x71da, 'CjkUnifiedIdeograph71da', [], 'Display', None],
  [ 0x71db, 'CjkUnifiedIdeograph71db', [], 'Display', None],
  [ 0x71dc, 'CjkUnifiedIdeograph71dc', [], 'Display', None],
  [ 0x71dd, 'CjkUnifiedIdeograph71dd', [], 'Display', None],
  [ 0x71de, 'CjkUnifiedIdeograph71de', [], 'Display', None],
  [ 0x71df, 'CjkUnifiedIdeograph71df', [], 'Display', None],
  [ 0x71e0, 'CjkUnifiedIdeograph71e0', [], 'Display', None],
  [ 0x71e1, 'CjkUnifiedIdeograph71e1', [], 'Display', None],
  [ 0x71e2, 'CjkUnifiedIdeograph71e2', [], 'Display', None],
  [ 0x71e3, 'CjkUnifiedIdeograph71e3', [], 'Display', None],
  [ 0x71e4, 'CjkUnifiedIdeograph71e4', [], 'Display', None],
  [ 0x71e5, 'CjkUnifiedIdeograph71e5', [], 'Display', None],
  [ 0x71e6, 'CjkUnifiedIdeograph71e6', [], 'Display', None],
  [ 0x71e7, 'CjkUnifiedIdeograph71e7', [], 'Display', None],
  [ 0x71e8, 'CjkUnifiedIdeograph71e8', [], 'Display', None],
  [ 0x71e9, 'CjkUnifiedIdeograph71e9', [], 'Display', None],
  [ 0x71ea, 'CjkUnifiedIdeograph71ea', [], 'Display', None],
  [ 0x71eb, 'CjkUnifiedIdeograph71eb', [], 'Display', None],
  [ 0x71ec, 'CjkUnifiedIdeograph71ec', [], 'Display', None],
  [ 0x71ed, 'CjkUnifiedIdeograph71ed', [], 'Display', None],
  [ 0x71ee, 'CjkUnifiedIdeograph71ee', [], 'Display', None],
  [ 0x71ef, 'CjkUnifiedIdeograph71ef', [], 'Display', None],
  [ 0x71f0, 'CjkUnifiedIdeograph71f0', [], 'Display', None],
  [ 0x71f1, 'CjkUnifiedIdeograph71f1', [], 'Display', None],
  [ 0x71f2, 'CjkUnifiedIdeograph71f2', [], 'Display', None],
  [ 0x71f3, 'CjkUnifiedIdeograph71f3', [], 'Display', None],
  [ 0x71f4, 'CjkUnifiedIdeograph71f4', [], 'Display', None],
  [ 0x71f5, 'CjkUnifiedIdeograph71f5', [], 'Display', None],
  [ 0x71f6, 'CjkUnifiedIdeograph71f6', [], 'Display', None],
  [ 0x71f7, 'CjkUnifiedIdeograph71f7', [], 'Display', None],
  [ 0x71f8, 'CjkUnifiedIdeograph71f8', [], 'Display', None],
  [ 0x71f9, 'CjkUnifiedIdeograph71f9', [], 'Display', None],
  [ 0x71fa, 'CjkUnifiedIdeograph71fa', [], 'Display', None],
  [ 0x71fb, 'CjkUnifiedIdeograph71fb', [], 'Display', None],
  [ 0x71fc, 'CjkUnifiedIdeograph71fc', [], 'Display', None],
  [ 0x71fd, 'CjkUnifiedIdeograph71fd', [], 'Display', None],
  [ 0x71fe, 'CjkUnifiedIdeograph71fe', [], 'Display', None],
  [ 0x71ff, 'CjkUnifiedIdeograph71ff', [], 'Display', None],
  [ 0x7200, 'CjkUnifiedIdeograph7200', [], 'Display', None],
  [ 0x7201, 'CjkUnifiedIdeograph7201', [], 'Display', None],
  [ 0x7202, 'CjkUnifiedIdeograph7202', [], 'Display', None],
  [ 0x7203, 'CjkUnifiedIdeograph7203', [], 'Display', None],
  [ 0x7204, 'CjkUnifiedIdeograph7204', [], 'Display', None],
  [ 0x7205, 'CjkUnifiedIdeograph7205', [], 'Display', None],
  [ 0x7206, 'CjkUnifiedIdeograph7206', [], 'Display', None],
  [ 0x7207, 'CjkUnifiedIdeograph7207', [], 'Display', None],
  [ 0x7208, 'CjkUnifiedIdeograph7208', [], 'Display', None],
  [ 0x7209, 'CjkUnifiedIdeograph7209', [], 'Display', None],
  [ 0x720a, 'CjkUnifiedIdeograph720a', [], 'Display', None],
  [ 0x720b, 'CjkUnifiedIdeograph720b', [], 'Display', None],
  [ 0x720c, 'CjkUnifiedIdeograph720c', [], 'Display', None],
  [ 0x720d, 'CjkUnifiedIdeograph720d', [], 'Display', None],
  [ 0x720e, 'CjkUnifiedIdeograph720e', [], 'Display', None],
  [ 0x720f, 'CjkUnifiedIdeograph720f', [], 'Display', None],
  [ 0x7210, 'CjkUnifiedIdeograph7210', [], 'Display', None],
  [ 0x7211, 'CjkUnifiedIdeograph7211', [], 'Display', None],
  [ 0x7212, 'CjkUnifiedIdeograph7212', [], 'Display', None],
  [ 0x7213, 'CjkUnifiedIdeograph7213', [], 'Display', None],
  [ 0x7214, 'CjkUnifiedIdeograph7214', [], 'Display', None],
  [ 0x7215, 'CjkUnifiedIdeograph7215', [], 'Display', None],
  [ 0x7216, 'CjkUnifiedIdeograph7216', [], 'Display', None],
  [ 0x7217, 'CjkUnifiedIdeograph7217', [], 'Display', None],
  [ 0x7218, 'CjkUnifiedIdeograph7218', [], 'Display', None],
  [ 0x7219, 'CjkUnifiedIdeograph7219', [], 'Display', None],
  [ 0x721a, 'CjkUnifiedIdeograph721a', [], 'Display', None],
  [ 0x721b, 'CjkUnifiedIdeograph721b', [], 'Display', None],
  [ 0x721c, 'CjkUnifiedIdeograph721c', [], 'Display', None],
  [ 0x721d, 'CjkUnifiedIdeograph721d', [], 'Display', None],
  [ 0x721e, 'CjkUnifiedIdeograph721e', [], 'Display', None],
  [ 0x721f, 'CjkUnifiedIdeograph721f', [], 'Display', None],
  [ 0x7220, 'CjkUnifiedIdeograph7220', [], 'Display', None],
  [ 0x7221, 'CjkUnifiedIdeograph7221', [], 'Display', None],
  [ 0x7222, 'CjkUnifiedIdeograph7222', [], 'Display', None],
  [ 0x7223, 'CjkUnifiedIdeograph7223', [], 'Display', None],
  [ 0x7224, 'CjkUnifiedIdeograph7224', [], 'Display', None],
  [ 0x7225, 'CjkUnifiedIdeograph7225', [], 'Display', None],
  [ 0x7226, 'CjkUnifiedIdeograph7226', [], 'Display', None],
  [ 0x7227, 'CjkUnifiedIdeograph7227', [], 'Display', None],
  [ 0x7228, 'CjkUnifiedIdeograph7228', [], 'Display', None],
  [ 0x7229, 'CjkUnifiedIdeograph7229', [], 'Display', None],
  [ 0x722a, 'CjkUnifiedIdeograph722a', [], 'Display', None],
  [ 0x722b, 'CjkUnifiedIdeograph722b', [], 'Display', None],
  [ 0x722c, 'CjkUnifiedIdeograph722c', [], 'Display', None],
  [ 0x722d, 'CjkUnifiedIdeograph722d', [], 'Display', None],
  [ 0x722e, 'CjkUnifiedIdeograph722e', [], 'Display', None],
  [ 0x722f, 'CjkUnifiedIdeograph722f', [], 'Display', None],
  [ 0x7230, 'CjkUnifiedIdeograph7230', [], 'Display', None],
  [ 0x7231, 'CjkUnifiedIdeograph7231', [], 'Display', None],
  [ 0x7232, 'CjkUnifiedIdeograph7232', [], 'Display', None],
  [ 0x7233, 'CjkUnifiedIdeograph7233', [], 'Display', None],
  [ 0x7234, 'CjkUnifiedIdeograph7234', [], 'Display', None],
  [ 0x7235, 'CjkUnifiedIdeograph7235', [], 'Display', None],
  [ 0x7236, 'CjkUnifiedIdeograph7236', [], 'Display', None],
  [ 0x7237, 'CjkUnifiedIdeograph7237', [], 'Display', None],
  [ 0x7238, 'CjkUnifiedIdeograph7238', [], 'Display', None],
  [ 0x7239, 'CjkUnifiedIdeograph7239', [], 'Display', None],
  [ 0x723a, 'CjkUnifiedIdeograph723a', [], 'Display', None],
  [ 0x723b, 'CjkUnifiedIdeograph723b', [], 'Display', None],
  [ 0x723c, 'CjkUnifiedIdeograph723c', [], 'Display', None],
  [ 0x723d, 'CjkUnifiedIdeograph723d', [], 'Display', None],
  [ 0x723e, 'CjkUnifiedIdeograph723e', [], 'Display', None],
  [ 0x723f, 'CjkUnifiedIdeograph723f', [], 'Display', None],
  [ 0x7240, 'CjkUnifiedIdeograph7240', [], 'Display', None],
  [ 0x7241, 'CjkUnifiedIdeograph7241', [], 'Display', None],
  [ 0x7242, 'CjkUnifiedIdeograph7242', [], 'Display', None],
  [ 0x7243, 'CjkUnifiedIdeograph7243', [], 'Display', None],
  [ 0x7244, 'CjkUnifiedIdeograph7244', [], 'Display', None],
  [ 0x7245, 'CjkUnifiedIdeograph7245', [], 'Display', None],
  [ 0x7246, 'CjkUnifiedIdeograph7246', [], 'Display', None],
  [ 0x7247, 'CjkUnifiedIdeograph7247', [], 'Display', None],
  [ 0x7248, 'CjkUnifiedIdeograph7248', [], 'Display', None],
  [ 0x7249, 'CjkUnifiedIdeograph7249', [], 'Display', None],
  [ 0x724a, 'CjkUnifiedIdeograph724a', [], 'Display', None],
  [ 0x724b, 'CjkUnifiedIdeograph724b', [], 'Display', None],
  [ 0x724c, 'CjkUnifiedIdeograph724c', [], 'Display', None],
  [ 0x724d, 'CjkUnifiedIdeograph724d', [], 'Display', None],
  [ 0x724e, 'CjkUnifiedIdeograph724e', [], 'Display', None],
  [ 0x724f, 'CjkUnifiedIdeograph724f', [], 'Display', None],
  [ 0x7250, 'CjkUnifiedIdeograph7250', [], 'Display', None],
  [ 0x7251, 'CjkUnifiedIdeograph7251', [], 'Display', None],
  [ 0x7252, 'CjkUnifiedIdeograph7252', [], 'Display', None],
  [ 0x7253, 'CjkUnifiedIdeograph7253', [], 'Display', None],
  [ 0x7254, 'CjkUnifiedIdeograph7254', [], 'Display', None],
  [ 0x7255, 'CjkUnifiedIdeograph7255', [], 'Display', None],
  [ 0x7256, 'CjkUnifiedIdeograph7256', [], 'Display', None],
  [ 0x7257, 'CjkUnifiedIdeograph7257', [], 'Display', None],
  [ 0x7258, 'CjkUnifiedIdeograph7258', [], 'Display', None],
  [ 0x7259, 'CjkUnifiedIdeograph7259', [], 'Display', None],
  [ 0x725a, 'CjkUnifiedIdeograph725a', [], 'Display', None],
  [ 0x725b, 'CjkUnifiedIdeograph725b', [], 'Display', None],
  [ 0x725c, 'CjkUnifiedIdeograph725c', [], 'Display', None],
  [ 0x725d, 'CjkUnifiedIdeograph725d', [], 'Display', None],
  [ 0x725e, 'CjkUnifiedIdeograph725e', [], 'Display', None],
  [ 0x725f, 'CjkUnifiedIdeograph725f', [], 'Display', None],
  [ 0x7260, 'CjkUnifiedIdeograph7260', [], 'Display', None],
  [ 0x7261, 'CjkUnifiedIdeograph7261', [], 'Display', None],
  [ 0x7262, 'CjkUnifiedIdeograph7262', [], 'Display', None],
  [ 0x7263, 'CjkUnifiedIdeograph7263', [], 'Display', None],
  [ 0x7264, 'CjkUnifiedIdeograph7264', [], 'Display', None],
  [ 0x7265, 'CjkUnifiedIdeograph7265', [], 'Display', None],
  [ 0x7266, 'CjkUnifiedIdeograph7266', [], 'Display', None],
  [ 0x7267, 'CjkUnifiedIdeograph7267', [], 'Display', None],
  [ 0x7268, 'CjkUnifiedIdeograph7268', [], 'Display', None],
  [ 0x7269, 'CjkUnifiedIdeograph7269', [], 'Display', None],
  [ 0x726a, 'CjkUnifiedIdeograph726a', [], 'Display', None],
  [ 0x726b, 'CjkUnifiedIdeograph726b', [], 'Display', None],
  [ 0x726c, 'CjkUnifiedIdeograph726c', [], 'Display', None],
  [ 0x726d, 'CjkUnifiedIdeograph726d', [], 'Display', None],
  [ 0x726e, 'CjkUnifiedIdeograph726e', [], 'Display', None],
  [ 0x726f, 'CjkUnifiedIdeograph726f', [], 'Display', None],
  [ 0x7270, 'CjkUnifiedIdeograph7270', [], 'Display', None],
  [ 0x7271, 'CjkUnifiedIdeograph7271', [], 'Display', None],
  [ 0x7272, 'CjkUnifiedIdeograph7272', [], 'Display', None],
  [ 0x7273, 'CjkUnifiedIdeograph7273', [], 'Display', None],
  [ 0x7274, 'CjkUnifiedIdeograph7274', [], 'Display', None],
  [ 0x7275, 'CjkUnifiedIdeograph7275', [], 'Display', None],
  [ 0x7276, 'CjkUnifiedIdeograph7276', [], 'Display', None],
  [ 0x7277, 'CjkUnifiedIdeograph7277', [], 'Display', None],
  [ 0x7278, 'CjkUnifiedIdeograph7278', [], 'Display', None],
  [ 0x7279, 'CjkUnifiedIdeograph7279', [], 'Display', None],
  [ 0x727a, 'CjkUnifiedIdeograph727a', [], 'Display', None],
  [ 0x727b, 'CjkUnifiedIdeograph727b', [], 'Display', None],
  [ 0x727c, 'CjkUnifiedIdeograph727c', [], 'Display', None],
  [ 0x727d, 'CjkUnifiedIdeograph727d', [], 'Display', None],
  [ 0x727e, 'CjkUnifiedIdeograph727e', [], 'Display', None],
  [ 0x727f, 'CjkUnifiedIdeograph727f', [], 'Display', None],
  [ 0x7280, 'CjkUnifiedIdeograph7280', [], 'Display', None],
  [ 0x7281, 'CjkUnifiedIdeograph7281', [], 'Display', None],
  [ 0x7282, 'CjkUnifiedIdeograph7282', [], 'Display', None],
  [ 0x7283, 'CjkUnifiedIdeograph7283', [], 'Display', None],
  [ 0x7284, 'CjkUnifiedIdeograph7284', [], 'Display', None],
  [ 0x7285, 'CjkUnifiedIdeograph7285', [], 'Display', None],
  [ 0x7286, 'CjkUnifiedIdeograph7286', [], 'Display', None],
  [ 0x7287, 'CjkUnifiedIdeograph7287', [], 'Display', None],
  [ 0x7288, 'CjkUnifiedIdeograph7288', [], 'Display', None],
  [ 0x7289, 'CjkUnifiedIdeograph7289', [], 'Display', None],
  [ 0x728a, 'CjkUnifiedIdeograph728a', [], 'Display', None],
  [ 0x728b, 'CjkUnifiedIdeograph728b', [], 'Display', None],
  [ 0x728c, 'CjkUnifiedIdeograph728c', [], 'Display', None],
  [ 0x728d, 'CjkUnifiedIdeograph728d', [], 'Display', None],
  [ 0x728e, 'CjkUnifiedIdeograph728e', [], 'Display', None],
  [ 0x728f, 'CjkUnifiedIdeograph728f', [], 'Display', None],
  [ 0x7290, 'CjkUnifiedIdeograph7290', [], 'Display', None],
  [ 0x7291, 'CjkUnifiedIdeograph7291', [], 'Display', None],
  [ 0x7292, 'CjkUnifiedIdeograph7292', [], 'Display', None],
  [ 0x7293, 'CjkUnifiedIdeograph7293', [], 'Display', None],
  [ 0x7294, 'CjkUnifiedIdeograph7294', [], 'Display', None],
  [ 0x7295, 'CjkUnifiedIdeograph7295', [], 'Display', None],
  [ 0x7296, 'CjkUnifiedIdeograph7296', [], 'Display', None],
  [ 0x7297, 'CjkUnifiedIdeograph7297', [], 'Display', None],
  [ 0x7298, 'CjkUnifiedIdeograph7298', [], 'Display', None],
  [ 0x7299, 'CjkUnifiedIdeograph7299', [], 'Display', None],
  [ 0x729a, 'CjkUnifiedIdeograph729a', [], 'Display', None],
  [ 0x729b, 'CjkUnifiedIdeograph729b', [], 'Display', None],
  [ 0x729c, 'CjkUnifiedIdeograph729c', [], 'Display', None],
  [ 0x729d, 'CjkUnifiedIdeograph729d', [], 'Display', None],
  [ 0x729e, 'CjkUnifiedIdeograph729e', [], 'Display', None],
  [ 0x729f, 'CjkUnifiedIdeograph729f', [], 'Display', None],
  [ 0x72a0, 'CjkUnifiedIdeograph72a0', [], 'Display', None],
  [ 0x72a1, 'CjkUnifiedIdeograph72a1', [], 'Display', None],
  [ 0x72a2, 'CjkUnifiedIdeograph72a2', [], 'Display', None],
  [ 0x72a3, 'CjkUnifiedIdeograph72a3', [], 'Display', None],
  [ 0x72a4, 'CjkUnifiedIdeograph72a4', [], 'Display', None],
  [ 0x72a5, 'CjkUnifiedIdeograph72a5', [], 'Display', None],
  [ 0x72a6, 'CjkUnifiedIdeograph72a6', [], 'Display', None],
  [ 0x72a7, 'CjkUnifiedIdeograph72a7', [], 'Display', None],
  [ 0x72a8, 'CjkUnifiedIdeograph72a8', [], 'Display', None],
  [ 0x72a9, 'CjkUnifiedIdeograph72a9', [], 'Display', None],
  [ 0x72aa, 'CjkUnifiedIdeograph72aa', [], 'Display', None],
  [ 0x72ab, 'CjkUnifiedIdeograph72ab', [], 'Display', None],
  [ 0x72ac, 'CjkUnifiedIdeograph72ac', [], 'Display', None],
  [ 0x72ad, 'CjkUnifiedIdeograph72ad', [], 'Display', None],
  [ 0x72ae, 'CjkUnifiedIdeograph72ae', [], 'Display', None],
  [ 0x72af, 'CjkUnifiedIdeograph72af', [], 'Display', None],
  [ 0x72b0, 'CjkUnifiedIdeograph72b0', [], 'Display', None],
  [ 0x72b1, 'CjkUnifiedIdeograph72b1', [], 'Display', None],
  [ 0x72b2, 'CjkUnifiedIdeograph72b2', [], 'Display', None],
  [ 0x72b3, 'CjkUnifiedIdeograph72b3', [], 'Display', None],
  [ 0x72b4, 'CjkUnifiedIdeograph72b4', [], 'Display', None],
  [ 0x72b5, 'CjkUnifiedIdeograph72b5', [], 'Display', None],
  [ 0x72b6, 'CjkUnifiedIdeograph72b6', [], 'Display', None],
  [ 0x72b7, 'CjkUnifiedIdeograph72b7', [], 'Display', None],
  [ 0x72b8, 'CjkUnifiedIdeograph72b8', [], 'Display', None],
  [ 0x72b9, 'CjkUnifiedIdeograph72b9', [], 'Display', None],
  [ 0x72ba, 'CjkUnifiedIdeograph72ba', [], 'Display', None],
  [ 0x72bb, 'CjkUnifiedIdeograph72bb', [], 'Display', None],
  [ 0x72bc, 'CjkUnifiedIdeograph72bc', [], 'Display', None],
  [ 0x72bd, 'CjkUnifiedIdeograph72bd', [], 'Display', None],
  [ 0x72be, 'CjkUnifiedIdeograph72be', [], 'Display', None],
  [ 0x72bf, 'CjkUnifiedIdeograph72bf', [], 'Display', None],
  [ 0x72c0, 'CjkUnifiedIdeograph72c0', [], 'Display', None],
  [ 0x72c1, 'CjkUnifiedIdeograph72c1', [], 'Display', None],
  [ 0x72c2, 'CjkUnifiedIdeograph72c2', [], 'Display', None],
  [ 0x72c3, 'CjkUnifiedIdeograph72c3', [], 'Display', None],
  [ 0x72c4, 'CjkUnifiedIdeograph72c4', [], 'Display', None],
  [ 0x72c5, 'CjkUnifiedIdeograph72c5', [], 'Display', None],
  [ 0x72c6, 'CjkUnifiedIdeograph72c6', [], 'Display', None],
  [ 0x72c7, 'CjkUnifiedIdeograph72c7', [], 'Display', None],
  [ 0x72c8, 'CjkUnifiedIdeograph72c8', [], 'Display', None],
  [ 0x72c9, 'CjkUnifiedIdeograph72c9', [], 'Display', None],
  [ 0x72ca, 'CjkUnifiedIdeograph72ca', [], 'Display', None],
  [ 0x72cb, 'CjkUnifiedIdeograph72cb', [], 'Display', None],
  [ 0x72cc, 'CjkUnifiedIdeograph72cc', [], 'Display', None],
  [ 0x72cd, 'CjkUnifiedIdeograph72cd', [], 'Display', None],
  [ 0x72ce, 'CjkUnifiedIdeograph72ce', [], 'Display', None],
  [ 0x72cf, 'CjkUnifiedIdeograph72cf', [], 'Display', None],
  [ 0x72d0, 'CjkUnifiedIdeograph72d0', [], 'Display', None],
  [ 0x72d1, 'CjkUnifiedIdeograph72d1', [], 'Display', None],
  [ 0x72d2, 'CjkUnifiedIdeograph72d2', [], 'Display', None],
  [ 0x72d3, 'CjkUnifiedIdeograph72d3', [], 'Display', None],
  [ 0x72d4, 'CjkUnifiedIdeograph72d4', [], 'Display', None],
  [ 0x72d5, 'CjkUnifiedIdeograph72d5', [], 'Display', None],
  [ 0x72d6, 'CjkUnifiedIdeograph72d6', [], 'Display', None],
  [ 0x72d7, 'CjkUnifiedIdeograph72d7', [], 'Display', None],
  [ 0x72d8, 'CjkUnifiedIdeograph72d8', [], 'Display', None],
  [ 0x72d9, 'CjkUnifiedIdeograph72d9', [], 'Display', None],
  [ 0x72da, 'CjkUnifiedIdeograph72da', [], 'Display', None],
  [ 0x72db, 'CjkUnifiedIdeograph72db', [], 'Display', None],
  [ 0x72dc, 'CjkUnifiedIdeograph72dc', [], 'Display', None],
  [ 0x72dd, 'CjkUnifiedIdeograph72dd', [], 'Display', None],
  [ 0x72de, 'CjkUnifiedIdeograph72de', [], 'Display', None],
  [ 0x72df, 'CjkUnifiedIdeograph72df', [], 'Display', None],
  [ 0x72e0, 'CjkUnifiedIdeograph72e0', [], 'Display', None],
  [ 0x72e1, 'CjkUnifiedIdeograph72e1', [], 'Display', None],
  [ 0x72e2, 'CjkUnifiedIdeograph72e2', [], 'Display', None],
  [ 0x72e3, 'CjkUnifiedIdeograph72e3', [], 'Display', None],
  [ 0x72e4, 'CjkUnifiedIdeograph72e4', [], 'Display', None],
  [ 0x72e5, 'CjkUnifiedIdeograph72e5', [], 'Display', None],
  [ 0x72e6, 'CjkUnifiedIdeograph72e6', [], 'Display', None],
  [ 0x72e7, 'CjkUnifiedIdeograph72e7', [], 'Display', None],
  [ 0x72e8, 'CjkUnifiedIdeograph72e8', [], 'Display', None],
  [ 0x72e9, 'CjkUnifiedIdeograph72e9', [], 'Display', None],
  [ 0x72ea, 'CjkUnifiedIdeograph72ea', [], 'Display', None],
  [ 0x72eb, 'CjkUnifiedIdeograph72eb', [], 'Display', None],
  [ 0x72ec, 'CjkUnifiedIdeograph72ec', [], 'Display', None],
  [ 0x72ed, 'CjkUnifiedIdeograph72ed', [], 'Display', None],
  [ 0x72ee, 'CjkUnifiedIdeograph72ee', [], 'Display', None],
  [ 0x72ef, 'CjkUnifiedIdeograph72ef', [], 'Display', None],
  [ 0x72f0, 'CjkUnifiedIdeograph72f0', [], 'Display', None],
  [ 0x72f1, 'CjkUnifiedIdeograph72f1', [], 'Display', None],
  [ 0x72f2, 'CjkUnifiedIdeograph72f2', [], 'Display', None],
  [ 0x72f3, 'CjkUnifiedIdeograph72f3', [], 'Display', None],
  [ 0x72f4, 'CjkUnifiedIdeograph72f4', [], 'Display', None],
  [ 0x72f5, 'CjkUnifiedIdeograph72f5', [], 'Display', None],
  [ 0x72f6, 'CjkUnifiedIdeograph72f6', [], 'Display', None],
  [ 0x72f7, 'CjkUnifiedIdeograph72f7', [], 'Display', None],
  [ 0x72f8, 'CjkUnifiedIdeograph72f8', [], 'Display', None],
  [ 0x72f9, 'CjkUnifiedIdeograph72f9', [], 'Display', None],
  [ 0x72fa, 'CjkUnifiedIdeograph72fa', [], 'Display', None],
  [ 0x72fb, 'CjkUnifiedIdeograph72fb', [], 'Display', None],
  [ 0x72fc, 'CjkUnifiedIdeograph72fc', [], 'Display', None],
  [ 0x72fd, 'CjkUnifiedIdeograph72fd', [], 'Display', None],
  [ 0x72fe, 'CjkUnifiedIdeograph72fe', [], 'Display', None],
  [ 0x72ff, 'CjkUnifiedIdeograph72ff', [], 'Display', None],
  [ 0x7300, 'CjkUnifiedIdeograph7300', [], 'Display', None],
  [ 0x7301, 'CjkUnifiedIdeograph7301', [], 'Display', None],
  [ 0x7302, 'CjkUnifiedIdeograph7302', [], 'Display', None],
  [ 0x7303, 'CjkUnifiedIdeograph7303', [], 'Display', None],
  [ 0x7304, 'CjkUnifiedIdeograph7304', [], 'Display', None],
  [ 0x7305, 'CjkUnifiedIdeograph7305', [], 'Display', None],
  [ 0x7306, 'CjkUnifiedIdeograph7306', [], 'Display', None],
  [ 0x7307, 'CjkUnifiedIdeograph7307', [], 'Display', None],
  [ 0x7308, 'CjkUnifiedIdeograph7308', [], 'Display', None],
  [ 0x7309, 'CjkUnifiedIdeograph7309', [], 'Display', None],
  [ 0x730a, 'CjkUnifiedIdeograph730a', [], 'Display', None],
  [ 0x730b, 'CjkUnifiedIdeograph730b', [], 'Display', None],
  [ 0x730c, 'CjkUnifiedIdeograph730c', [], 'Display', None],
  [ 0x730d, 'CjkUnifiedIdeograph730d', [], 'Display', None],
  [ 0x730e, 'CjkUnifiedIdeograph730e', [], 'Display', None],
  [ 0x730f, 'CjkUnifiedIdeograph730f', [], 'Display', None],
  [ 0x7310, 'CjkUnifiedIdeograph7310', [], 'Display', None],
  [ 0x7311, 'CjkUnifiedIdeograph7311', [], 'Display', None],
  [ 0x7312, 'CjkUnifiedIdeograph7312', [], 'Display', None],
  [ 0x7313, 'CjkUnifiedIdeograph7313', [], 'Display', None],
  [ 0x7314, 'CjkUnifiedIdeograph7314', [], 'Display', None],
  [ 0x7315, 'CjkUnifiedIdeograph7315', [], 'Display', None],
  [ 0x7316, 'CjkUnifiedIdeograph7316', [], 'Display', None],
  [ 0x7317, 'CjkUnifiedIdeograph7317', [], 'Display', None],
  [ 0x7318, 'CjkUnifiedIdeograph7318', [], 'Display', None],
  [ 0x7319, 'CjkUnifiedIdeograph7319', [], 'Display', None],
  [ 0x731a, 'CjkUnifiedIdeograph731a', [], 'Display', None],
  [ 0x731b, 'CjkUnifiedIdeograph731b', [], 'Display', None],
  [ 0x731c, 'CjkUnifiedIdeograph731c', [], 'Display', None],
  [ 0x731d, 'CjkUnifiedIdeograph731d', [], 'Display', None],
  [ 0x731e, 'CjkUnifiedIdeograph731e', [], 'Display', None],
  [ 0x731f, 'CjkUnifiedIdeograph731f', [], 'Display', None],
  [ 0x7320, 'CjkUnifiedIdeograph7320', [], 'Display', None],
  [ 0x7321, 'CjkUnifiedIdeograph7321', [], 'Display', None],
  [ 0x7322, 'CjkUnifiedIdeograph7322', [], 'Display', None],
  [ 0x7323, 'CjkUnifiedIdeograph7323', [], 'Display', None],
  [ 0x7324, 'CjkUnifiedIdeograph7324', [], 'Display', None],
  [ 0x7325, 'CjkUnifiedIdeograph7325', [], 'Display', None],
  [ 0x7326, 'CjkUnifiedIdeograph7326', [], 'Display', None],
  [ 0x7327, 'CjkUnifiedIdeograph7327', [], 'Display', None],
  [ 0x7328, 'CjkUnifiedIdeograph7328', [], 'Display', None],
  [ 0x7329, 'CjkUnifiedIdeograph7329', [], 'Display', None],
  [ 0x732a, 'CjkUnifiedIdeograph732a', [], 'Display', None],
  [ 0x732b, 'CjkUnifiedIdeograph732b', [], 'Display', None],
  [ 0x732c, 'CjkUnifiedIdeograph732c', [], 'Display', None],
  [ 0x732d, 'CjkUnifiedIdeograph732d', [], 'Display', None],
  [ 0x732e, 'CjkUnifiedIdeograph732e', [], 'Display', None],
  [ 0x732f, 'CjkUnifiedIdeograph732f', [], 'Display', None],
  [ 0x7330, 'CjkUnifiedIdeograph7330', [], 'Display', None],
  [ 0x7331, 'CjkUnifiedIdeograph7331', [], 'Display', None],
  [ 0x7332, 'CjkUnifiedIdeograph7332', [], 'Display', None],
  [ 0x7333, 'CjkUnifiedIdeograph7333', [], 'Display', None],
  [ 0x7334, 'CjkUnifiedIdeograph7334', [], 'Display', None],
  [ 0x7335, 'CjkUnifiedIdeograph7335', [], 'Display', None],
  [ 0x7336, 'CjkUnifiedIdeograph7336', [], 'Display', None],
  [ 0x7337, 'CjkUnifiedIdeograph7337', [], 'Display', None],
  [ 0x7338, 'CjkUnifiedIdeograph7338', [], 'Display', None],
  [ 0x7339, 'CjkUnifiedIdeograph7339', [], 'Display', None],
  [ 0x733a, 'CjkUnifiedIdeograph733a', [], 'Display', None],
  [ 0x733b, 'CjkUnifiedIdeograph733b', [], 'Display', None],
  [ 0x733c, 'CjkUnifiedIdeograph733c', [], 'Display', None],
  [ 0x733d, 'CjkUnifiedIdeograph733d', [], 'Display', None],
  [ 0x733e, 'CjkUnifiedIdeograph733e', [], 'Display', None],
  [ 0x733f, 'CjkUnifiedIdeograph733f', [], 'Display', None],
  [ 0x7340, 'CjkUnifiedIdeograph7340', [], 'Display', None],
  [ 0x7341, 'CjkUnifiedIdeograph7341', [], 'Display', None],
  [ 0x7342, 'CjkUnifiedIdeograph7342', [], 'Display', None],
  [ 0x7343, 'CjkUnifiedIdeograph7343', [], 'Display', None],
  [ 0x7344, 'CjkUnifiedIdeograph7344', [], 'Display', None],
  [ 0x7345, 'CjkUnifiedIdeograph7345', [], 'Display', None],
  [ 0x7346, 'CjkUnifiedIdeograph7346', [], 'Display', None],
  [ 0x7347, 'CjkUnifiedIdeograph7347', [], 'Display', None],
  [ 0x7348, 'CjkUnifiedIdeograph7348', [], 'Display', None],
  [ 0x7349, 'CjkUnifiedIdeograph7349', [], 'Display', None],
  [ 0x734a, 'CjkUnifiedIdeograph734a', [], 'Display', None],
  [ 0x734b, 'CjkUnifiedIdeograph734b', [], 'Display', None],
  [ 0x734c, 'CjkUnifiedIdeograph734c', [], 'Display', None],
  [ 0x734d, 'CjkUnifiedIdeograph734d', [], 'Display', None],
  [ 0x734e, 'CjkUnifiedIdeograph734e', [], 'Display', None],
  [ 0x734f, 'CjkUnifiedIdeograph734f', [], 'Display', None],
  [ 0x7350, 'CjkUnifiedIdeograph7350', [], 'Display', None],
  [ 0x7351, 'CjkUnifiedIdeograph7351', [], 'Display', None],
  [ 0x7352, 'CjkUnifiedIdeograph7352', [], 'Display', None],
  [ 0x7353, 'CjkUnifiedIdeograph7353', [], 'Display', None],
  [ 0x7354, 'CjkUnifiedIdeograph7354', [], 'Display', None],
  [ 0x7355, 'CjkUnifiedIdeograph7355', [], 'Display', None],
  [ 0x7356, 'CjkUnifiedIdeograph7356', [], 'Display', None],
  [ 0x7357, 'CjkUnifiedIdeograph7357', [], 'Display', None],
  [ 0x7358, 'CjkUnifiedIdeograph7358', [], 'Display', None],
  [ 0x7359, 'CjkUnifiedIdeograph7359', [], 'Display', None],
  [ 0x735a, 'CjkUnifiedIdeograph735a', [], 'Display', None],
  [ 0x735b, 'CjkUnifiedIdeograph735b', [], 'Display', None],
  [ 0x735c, 'CjkUnifiedIdeograph735c', [], 'Display', None],
  [ 0x735d, 'CjkUnifiedIdeograph735d', [], 'Display', None],
  [ 0x735e, 'CjkUnifiedIdeograph735e', [], 'Display', None],
  [ 0x735f, 'CjkUnifiedIdeograph735f', [], 'Display', None],
  [ 0x7360, 'CjkUnifiedIdeograph7360', [], 'Display', None],
  [ 0x7361, 'CjkUnifiedIdeograph7361', [], 'Display', None],
  [ 0x7362, 'CjkUnifiedIdeograph7362', [], 'Display', None],
  [ 0x7363, 'CjkUnifiedIdeograph7363', [], 'Display', None],
  [ 0x7364, 'CjkUnifiedIdeograph7364', [], 'Display', None],
  [ 0x7365, 'CjkUnifiedIdeograph7365', [], 'Display', None],
  [ 0x7366, 'CjkUnifiedIdeograph7366', [], 'Display', None],
  [ 0x7367, 'CjkUnifiedIdeograph7367', [], 'Display', None],
  [ 0x7368, 'CjkUnifiedIdeograph7368', [], 'Display', None],
  [ 0x7369, 'CjkUnifiedIdeograph7369', [], 'Display', None],
  [ 0x736a, 'CjkUnifiedIdeograph736a', [], 'Display', None],
  [ 0x736b, 'CjkUnifiedIdeograph736b', [], 'Display', None],
  [ 0x736c, 'CjkUnifiedIdeograph736c', [], 'Display', None],
  [ 0x736d, 'CjkUnifiedIdeograph736d', [], 'Display', None],
  [ 0x736e, 'CjkUnifiedIdeograph736e', [], 'Display', None],
  [ 0x736f, 'CjkUnifiedIdeograph736f', [], 'Display', None],
  [ 0x7370, 'CjkUnifiedIdeograph7370', [], 'Display', None],
  [ 0x7371, 'CjkUnifiedIdeograph7371', [], 'Display', None],
  [ 0x7372, 'CjkUnifiedIdeograph7372', [], 'Display', None],
  [ 0x7373, 'CjkUnifiedIdeograph7373', [], 'Display', None],
  [ 0x7374, 'CjkUnifiedIdeograph7374', [], 'Display', None],
  [ 0x7375, 'CjkUnifiedIdeograph7375', [], 'Display', None],
  [ 0x7376, 'CjkUnifiedIdeograph7376', [], 'Display', None],
  [ 0x7377, 'CjkUnifiedIdeograph7377', [], 'Display', None],
  [ 0x7378, 'CjkUnifiedIdeograph7378', [], 'Display', None],
  [ 0x7379, 'CjkUnifiedIdeograph7379', [], 'Display', None],
  [ 0x737a, 'CjkUnifiedIdeograph737a', [], 'Display', None],
  [ 0x737b, 'CjkUnifiedIdeograph737b', [], 'Display', None],
  [ 0x737c, 'CjkUnifiedIdeograph737c', [], 'Display', None],
  [ 0x737d, 'CjkUnifiedIdeograph737d', [], 'Display', None],
  [ 0x737e, 'CjkUnifiedIdeograph737e', [], 'Display', None],
  [ 0x737f, 'CjkUnifiedIdeograph737f', [], 'Display', None],
  [ 0x7380, 'CjkUnifiedIdeograph7380', [], 'Display', None],
  [ 0x7381, 'CjkUnifiedIdeograph7381', [], 'Display', None],
  [ 0x7382, 'CjkUnifiedIdeograph7382', [], 'Display', None],
  [ 0x7383, 'CjkUnifiedIdeograph7383', [], 'Display', None],
  [ 0x7384, 'CjkUnifiedIdeograph7384', [], 'Display', None],
  [ 0x7385, 'CjkUnifiedIdeograph7385', [], 'Display', None],
  [ 0x7386, 'CjkUnifiedIdeograph7386', [], 'Display', None],
  [ 0x7387, 'CjkUnifiedIdeograph7387', [], 'Display', None],
  [ 0x7388, 'CjkUnifiedIdeograph7388', [], 'Display', None],
  [ 0x7389, 'CjkUnifiedIdeograph7389', [], 'Display', None],
  [ 0x738a, 'CjkUnifiedIdeograph738a', [], 'Display', None],
  [ 0x738b, 'CjkUnifiedIdeograph738b', [], 'Display', None],
  [ 0x738c, 'CjkUnifiedIdeograph738c', [], 'Display', None],
  [ 0x738d, 'CjkUnifiedIdeograph738d', [], 'Display', None],
  [ 0x738e, 'CjkUnifiedIdeograph738e', [], 'Display', None],
  [ 0x738f, 'CjkUnifiedIdeograph738f', [], 'Display', None],
  [ 0x7390, 'CjkUnifiedIdeograph7390', [], 'Display', None],
  [ 0x7391, 'CjkUnifiedIdeograph7391', [], 'Display', None],
  [ 0x7392, 'CjkUnifiedIdeograph7392', [], 'Display', None],
  [ 0x7393, 'CjkUnifiedIdeograph7393', [], 'Display', None],
  [ 0x7394, 'CjkUnifiedIdeograph7394', [], 'Display', None],
  [ 0x7395, 'CjkUnifiedIdeograph7395', [], 'Display', None],
  [ 0x7396, 'CjkUnifiedIdeograph7396', [], 'Display', None],
  [ 0x7397, 'CjkUnifiedIdeograph7397', [], 'Display', None],
  [ 0x7398, 'CjkUnifiedIdeograph7398', [], 'Display', None],
  [ 0x7399, 'CjkUnifiedIdeograph7399', [], 'Display', None],
  [ 0x739a, 'CjkUnifiedIdeograph739a', [], 'Display', None],
  [ 0x739b, 'CjkUnifiedIdeograph739b', [], 'Display', None],
  [ 0x739c, 'CjkUnifiedIdeograph739c', [], 'Display', None],
  [ 0x739d, 'CjkUnifiedIdeograph739d', [], 'Display', None],
  [ 0x739e, 'CjkUnifiedIdeograph739e', [], 'Display', None],
  [ 0x739f, 'CjkUnifiedIdeograph739f', [], 'Display', None],
  [ 0x73a0, 'CjkUnifiedIdeograph73a0', [], 'Display', None],
  [ 0x73a1, 'CjkUnifiedIdeograph73a1', [], 'Display', None],
  [ 0x73a2, 'CjkUnifiedIdeograph73a2', [], 'Display', None],
  [ 0x73a3, 'CjkUnifiedIdeograph73a3', [], 'Display', None],
  [ 0x73a4, 'CjkUnifiedIdeograph73a4', [], 'Display', None],
  [ 0x73a5, 'CjkUnifiedIdeograph73a5', [], 'Display', None],
  [ 0x73a6, 'CjkUnifiedIdeograph73a6', [], 'Display', None],
  [ 0x73a7, 'CjkUnifiedIdeograph73a7', [], 'Display', None],
  [ 0x73a8, 'CjkUnifiedIdeograph73a8', [], 'Display', None],
  [ 0x73a9, 'CjkUnifiedIdeograph73a9', [], 'Display', None],
  [ 0x73aa, 'CjkUnifiedIdeograph73aa', [], 'Display', None],
  [ 0x73ab, 'CjkUnifiedIdeograph73ab', [], 'Display', None],
  [ 0x73ac, 'CjkUnifiedIdeograph73ac', [], 'Display', None],
  [ 0x73ad, 'CjkUnifiedIdeograph73ad', [], 'Display', None],
  [ 0x73ae, 'CjkUnifiedIdeograph73ae', [], 'Display', None],
  [ 0x73af, 'CjkUnifiedIdeograph73af', [], 'Display', None],
  [ 0x73b0, 'CjkUnifiedIdeograph73b0', [], 'Display', None],
  [ 0x73b1, 'CjkUnifiedIdeograph73b1', [], 'Display', None],
  [ 0x73b2, 'CjkUnifiedIdeograph73b2', [], 'Display', None],
  [ 0x73b3, 'CjkUnifiedIdeograph73b3', [], 'Display', None],
  [ 0x73b4, 'CjkUnifiedIdeograph73b4', [], 'Display', None],
  [ 0x73b5, 'CjkUnifiedIdeograph73b5', [], 'Display', None],
  [ 0x73b6, 'CjkUnifiedIdeograph73b6', [], 'Display', None],
  [ 0x73b7, 'CjkUnifiedIdeograph73b7', [], 'Display', None],
  [ 0x73b8, 'CjkUnifiedIdeograph73b8', [], 'Display', None],
  [ 0x73b9, 'CjkUnifiedIdeograph73b9', [], 'Display', None],
  [ 0x73ba, 'CjkUnifiedIdeograph73ba', [], 'Display', None],
  [ 0x73bb, 'CjkUnifiedIdeograph73bb', [], 'Display', None],
  [ 0x73bc, 'CjkUnifiedIdeograph73bc', [], 'Display', None],
  [ 0x73bd, 'CjkUnifiedIdeograph73bd', [], 'Display', None],
  [ 0x73be, 'CjkUnifiedIdeograph73be', [], 'Display', None],
  [ 0x73bf, 'CjkUnifiedIdeograph73bf', [], 'Display', None],
  [ 0x73c0, 'CjkUnifiedIdeograph73c0', [], 'Display', None],
  [ 0x73c1, 'CjkUnifiedIdeograph73c1', [], 'Display', None],
  [ 0x73c2, 'CjkUnifiedIdeograph73c2', [], 'Display', None],
  [ 0x73c3, 'CjkUnifiedIdeograph73c3', [], 'Display', None],
  [ 0x73c4, 'CjkUnifiedIdeograph73c4', [], 'Display', None],
  [ 0x73c5, 'CjkUnifiedIdeograph73c5', [], 'Display', None],
  [ 0x73c6, 'CjkUnifiedIdeograph73c6', [], 'Display', None],
  [ 0x73c7, 'CjkUnifiedIdeograph73c7', [], 'Display', None],
  [ 0x73c8, 'CjkUnifiedIdeograph73c8', [], 'Display', None],
  [ 0x73c9, 'CjkUnifiedIdeograph73c9', [], 'Display', None],
  [ 0x73ca, 'CjkUnifiedIdeograph73ca', [], 'Display', None],
  [ 0x73cb, 'CjkUnifiedIdeograph73cb', [], 'Display', None],
  [ 0x73cc, 'CjkUnifiedIdeograph73cc', [], 'Display', None],
  [ 0x73cd, 'CjkUnifiedIdeograph73cd', [], 'Display', None],
  [ 0x73ce, 'CjkUnifiedIdeograph73ce', [], 'Display', None],
  [ 0x73cf, 'CjkUnifiedIdeograph73cf', [], 'Display', None],
  [ 0x73d0, 'CjkUnifiedIdeograph73d0', [], 'Display', None],
  [ 0x73d1, 'CjkUnifiedIdeograph73d1', [], 'Display', None],
  [ 0x73d2, 'CjkUnifiedIdeograph73d2', [], 'Display', None],
  [ 0x73d3, 'CjkUnifiedIdeograph73d3', [], 'Display', None],
  [ 0x73d4, 'CjkUnifiedIdeograph73d4', [], 'Display', None],
  [ 0x73d5, 'CjkUnifiedIdeograph73d5', [], 'Display', None],
  [ 0x73d6, 'CjkUnifiedIdeograph73d6', [], 'Display', None],
  [ 0x73d7, 'CjkUnifiedIdeograph73d7', [], 'Display', None],
  [ 0x73d8, 'CjkUnifiedIdeograph73d8', [], 'Display', None],
  [ 0x73d9, 'CjkUnifiedIdeograph73d9', [], 'Display', None],
  [ 0x73da, 'CjkUnifiedIdeograph73da', [], 'Display', None],
  [ 0x73db, 'CjkUnifiedIdeograph73db', [], 'Display', None],
  [ 0x73dc, 'CjkUnifiedIdeograph73dc', [], 'Display', None],
  [ 0x73dd, 'CjkUnifiedIdeograph73dd', [], 'Display', None],
  [ 0x73de, 'CjkUnifiedIdeograph73de', [], 'Display', None],
  [ 0x73df, 'CjkUnifiedIdeograph73df', [], 'Display', None],
  [ 0x73e0, 'CjkUnifiedIdeograph73e0', [], 'Display', None],
  [ 0x73e1, 'CjkUnifiedIdeograph73e1', [], 'Display', None],
  [ 0x73e2, 'CjkUnifiedIdeograph73e2', [], 'Display', None],
  [ 0x73e3, 'CjkUnifiedIdeograph73e3', [], 'Display', None],
  [ 0x73e4, 'CjkUnifiedIdeograph73e4', [], 'Display', None],
  [ 0x73e5, 'CjkUnifiedIdeograph73e5', [], 'Display', None],
  [ 0x73e6, 'CjkUnifiedIdeograph73e6', [], 'Display', None],
  [ 0x73e7, 'CjkUnifiedIdeograph73e7', [], 'Display', None],
  [ 0x73e8, 'CjkUnifiedIdeograph73e8', [], 'Display', None],
  [ 0x73e9, 'CjkUnifiedIdeograph73e9', [], 'Display', None],
  [ 0x73ea, 'CjkUnifiedIdeograph73ea', [], 'Display', None],
  [ 0x73eb, 'CjkUnifiedIdeograph73eb', [], 'Display', None],
  [ 0x73ec, 'CjkUnifiedIdeograph73ec', [], 'Display', None],
  [ 0x73ed, 'CjkUnifiedIdeograph73ed', [], 'Display', None],
  [ 0x73ee, 'CjkUnifiedIdeograph73ee', [], 'Display', None],
  [ 0x73ef, 'CjkUnifiedIdeograph73ef', [], 'Display', None],
  [ 0x73f0, 'CjkUnifiedIdeograph73f0', [], 'Display', None],
  [ 0x73f1, 'CjkUnifiedIdeograph73f1', [], 'Display', None],
  [ 0x73f2, 'CjkUnifiedIdeograph73f2', [], 'Display', None],
  [ 0x73f3, 'CjkUnifiedIdeograph73f3', [], 'Display', None],
  [ 0x73f4, 'CjkUnifiedIdeograph73f4', [], 'Display', None],
  [ 0x73f5, 'CjkUnifiedIdeograph73f5', [], 'Display', None],
  [ 0x73f6, 'CjkUnifiedIdeograph73f6', [], 'Display', None],
  [ 0x73f7, 'CjkUnifiedIdeograph73f7', [], 'Display', None],
  [ 0x73f8, 'CjkUnifiedIdeograph73f8', [], 'Display', None],
  [ 0x73f9, 'CjkUnifiedIdeograph73f9', [], 'Display', None],
  [ 0x73fa, 'CjkUnifiedIdeograph73fa', [], 'Display', None],
  [ 0x73fb, 'CjkUnifiedIdeograph73fb', [], 'Display', None],
  [ 0x73fc, 'CjkUnifiedIdeograph73fc', [], 'Display', None],
  [ 0x73fd, 'CjkUnifiedIdeograph73fd', [], 'Display', None],
  [ 0x73fe, 'CjkUnifiedIdeograph73fe', [], 'Display', None],
  [ 0x73ff, 'CjkUnifiedIdeograph73ff', [], 'Display', None],
  [ 0x7400, 'CjkUnifiedIdeograph7400', [], 'Display', None],
  [ 0x7401, 'CjkUnifiedIdeograph7401', [], 'Display', None],
  [ 0x7402, 'CjkUnifiedIdeograph7402', [], 'Display', None],
  [ 0x7403, 'CjkUnifiedIdeograph7403', [], 'Display', None],
  [ 0x7404, 'CjkUnifiedIdeograph7404', [], 'Display', None],
  [ 0x7405, 'CjkUnifiedIdeograph7405', [], 'Display', None],
  [ 0x7406, 'CjkUnifiedIdeograph7406', [], 'Display', None],
  [ 0x7407, 'CjkUnifiedIdeograph7407', [], 'Display', None],
  [ 0x7408, 'CjkUnifiedIdeograph7408', [], 'Display', None],
  [ 0x7409, 'CjkUnifiedIdeograph7409', [], 'Display', None],
  [ 0x740a, 'CjkUnifiedIdeograph740a', [], 'Display', None],
  [ 0x740b, 'CjkUnifiedIdeograph740b', [], 'Display', None],
  [ 0x740c, 'CjkUnifiedIdeograph740c', [], 'Display', None],
  [ 0x740d, 'CjkUnifiedIdeograph740d', [], 'Display', None],
  [ 0x740e, 'CjkUnifiedIdeograph740e', [], 'Display', None],
  [ 0x740f, 'CjkUnifiedIdeograph740f', [], 'Display', None],
  [ 0x7410, 'CjkUnifiedIdeograph7410', [], 'Display', None],
  [ 0x7411, 'CjkUnifiedIdeograph7411', [], 'Display', None],
  [ 0x7412, 'CjkUnifiedIdeograph7412', [], 'Display', None],
  [ 0x7413, 'CjkUnifiedIdeograph7413', [], 'Display', None],
  [ 0x7414, 'CjkUnifiedIdeograph7414', [], 'Display', None],
  [ 0x7415, 'CjkUnifiedIdeograph7415', [], 'Display', None],
  [ 0x7416, 'CjkUnifiedIdeograph7416', [], 'Display', None],
  [ 0x7417, 'CjkUnifiedIdeograph7417', [], 'Display', None],
  [ 0x7418, 'CjkUnifiedIdeograph7418', [], 'Display', None],
  [ 0x7419, 'CjkUnifiedIdeograph7419', [], 'Display', None],
  [ 0x741a, 'CjkUnifiedIdeograph741a', [], 'Display', None],
  [ 0x741b, 'CjkUnifiedIdeograph741b', [], 'Display', None],
  [ 0x741c, 'CjkUnifiedIdeograph741c', [], 'Display', None],
  [ 0x741d, 'CjkUnifiedIdeograph741d', [], 'Display', None],
  [ 0x741e, 'CjkUnifiedIdeograph741e', [], 'Display', None],
  [ 0x741f, 'CjkUnifiedIdeograph741f', [], 'Display', None],
  [ 0x7420, 'CjkUnifiedIdeograph7420', [], 'Display', None],
  [ 0x7421, 'CjkUnifiedIdeograph7421', [], 'Display', None],
  [ 0x7422, 'CjkUnifiedIdeograph7422', [], 'Display', None],
  [ 0x7423, 'CjkUnifiedIdeograph7423', [], 'Display', None],
  [ 0x7424, 'CjkUnifiedIdeograph7424', [], 'Display', None],
  [ 0x7425, 'CjkUnifiedIdeograph7425', [], 'Display', None],
  [ 0x7426, 'CjkUnifiedIdeograph7426', [], 'Display', None],
  [ 0x7427, 'CjkUnifiedIdeograph7427', [], 'Display', None],
  [ 0x7428, 'CjkUnifiedIdeograph7428', [], 'Display', None],
  [ 0x7429, 'CjkUnifiedIdeograph7429', [], 'Display', None],
  [ 0x742a, 'CjkUnifiedIdeograph742a', [], 'Display', None],
  [ 0x742b, 'CjkUnifiedIdeograph742b', [], 'Display', None],
  [ 0x742c, 'CjkUnifiedIdeograph742c', [], 'Display', None],
  [ 0x742d, 'CjkUnifiedIdeograph742d', [], 'Display', None],
  [ 0x742e, 'CjkUnifiedIdeograph742e', [], 'Display', None],
  [ 0x742f, 'CjkUnifiedIdeograph742f', [], 'Display', None],
  [ 0x7430, 'CjkUnifiedIdeograph7430', [], 'Display', None],
  [ 0x7431, 'CjkUnifiedIdeograph7431', [], 'Display', None],
  [ 0x7432, 'CjkUnifiedIdeograph7432', [], 'Display', None],
  [ 0x7433, 'CjkUnifiedIdeograph7433', [], 'Display', None],
  [ 0x7434, 'CjkUnifiedIdeograph7434', [], 'Display', None],
  [ 0x7435, 'CjkUnifiedIdeograph7435', [], 'Display', None],
  [ 0x7436, 'CjkUnifiedIdeograph7436', [], 'Display', None],
  [ 0x7437, 'CjkUnifiedIdeograph7437', [], 'Display', None],
  [ 0x7438, 'CjkUnifiedIdeograph7438', [], 'Display', None],
  [ 0x7439, 'CjkUnifiedIdeograph7439', [], 'Display', None],
  [ 0x743a, 'CjkUnifiedIdeograph743a', [], 'Display', None],
  [ 0x743b, 'CjkUnifiedIdeograph743b', [], 'Display', None],
  [ 0x743c, 'CjkUnifiedIdeograph743c', [], 'Display', None],
  [ 0x743d, 'CjkUnifiedIdeograph743d', [], 'Display', None],
  [ 0x743e, 'CjkUnifiedIdeograph743e', [], 'Display', None],
  [ 0x743f, 'CjkUnifiedIdeograph743f', [], 'Display', None],
  [ 0x7440, 'CjkUnifiedIdeograph7440', [], 'Display', None],
  [ 0x7441, 'CjkUnifiedIdeograph7441', [], 'Display', None],
  [ 0x7442, 'CjkUnifiedIdeograph7442', [], 'Display', None],
  [ 0x7443, 'CjkUnifiedIdeograph7443', [], 'Display', None],
  [ 0x7444, 'CjkUnifiedIdeograph7444', [], 'Display', None],
  [ 0x7445, 'CjkUnifiedIdeograph7445', [], 'Display', None],
  [ 0x7446, 'CjkUnifiedIdeograph7446', [], 'Display', None],
  [ 0x7447, 'CjkUnifiedIdeograph7447', [], 'Display', None],
  [ 0x7448, 'CjkUnifiedIdeograph7448', [], 'Display', None],
  [ 0x7449, 'CjkUnifiedIdeograph7449', [], 'Display', None],
  [ 0x744a, 'CjkUnifiedIdeograph744a', [], 'Display', None],
  [ 0x744b, 'CjkUnifiedIdeograph744b', [], 'Display', None],
  [ 0x744c, 'CjkUnifiedIdeograph744c', [], 'Display', None],
  [ 0x744d, 'CjkUnifiedIdeograph744d', [], 'Display', None],
  [ 0x744e, 'CjkUnifiedIdeograph744e', [], 'Display', None],
  [ 0x744f, 'CjkUnifiedIdeograph744f', [], 'Display', None],
  [ 0x7450, 'CjkUnifiedIdeograph7450', [], 'Display', None],
  [ 0x7451, 'CjkUnifiedIdeograph7451', [], 'Display', None],
  [ 0x7452, 'CjkUnifiedIdeograph7452', [], 'Display', None],
  [ 0x7453, 'CjkUnifiedIdeograph7453', [], 'Display', None],
  [ 0x7454, 'CjkUnifiedIdeograph7454', [], 'Display', None],
  [ 0x7455, 'CjkUnifiedIdeograph7455', [], 'Display', None],
  [ 0x7456, 'CjkUnifiedIdeograph7456', [], 'Display', None],
  [ 0x7457, 'CjkUnifiedIdeograph7457', [], 'Display', None],
  [ 0x7458, 'CjkUnifiedIdeograph7458', [], 'Display', None],
  [ 0x7459, 'CjkUnifiedIdeograph7459', [], 'Display', None],
  [ 0x745a, 'CjkUnifiedIdeograph745a', [], 'Display', None],
  [ 0x745b, 'CjkUnifiedIdeograph745b', [], 'Display', None],
  [ 0x745c, 'CjkUnifiedIdeograph745c', [], 'Display', None],
  [ 0x745d, 'CjkUnifiedIdeograph745d', [], 'Display', None],
  [ 0x745e, 'CjkUnifiedIdeograph745e', [], 'Display', None],
  [ 0x745f, 'CjkUnifiedIdeograph745f', [], 'Display', None],
  [ 0x7460, 'CjkUnifiedIdeograph7460', [], 'Display', None],
  [ 0x7461, 'CjkUnifiedIdeograph7461', [], 'Display', None],
  [ 0x7462, 'CjkUnifiedIdeograph7462', [], 'Display', None],
  [ 0x7463, 'CjkUnifiedIdeograph7463', [], 'Display', None],
  [ 0x7464, 'CjkUnifiedIdeograph7464', [], 'Display', None],
  [ 0x7465, 'CjkUnifiedIdeograph7465', [], 'Display', None],
  [ 0x7466, 'CjkUnifiedIdeograph7466', [], 'Display', None],
  [ 0x7467, 'CjkUnifiedIdeograph7467', [], 'Display', None],
  [ 0x7468, 'CjkUnifiedIdeograph7468', [], 'Display', None],
  [ 0x7469, 'CjkUnifiedIdeograph7469', [], 'Display', None],
  [ 0x746a, 'CjkUnifiedIdeograph746a', [], 'Display', None],
  [ 0x746b, 'CjkUnifiedIdeograph746b', [], 'Display', None],
  [ 0x746c, 'CjkUnifiedIdeograph746c', [], 'Display', None],
  [ 0x746d, 'CjkUnifiedIdeograph746d', [], 'Display', None],
  [ 0x746e, 'CjkUnifiedIdeograph746e', [], 'Display', None],
  [ 0x746f, 'CjkUnifiedIdeograph746f', [], 'Display', None],
  [ 0x7470, 'CjkUnifiedIdeograph7470', [], 'Display', None],
  [ 0x7471, 'CjkUnifiedIdeograph7471', [], 'Display', None],
  [ 0x7472, 'CjkUnifiedIdeograph7472', [], 'Display', None],
  [ 0x7473, 'CjkUnifiedIdeograph7473', [], 'Display', None],
  [ 0x7474, 'CjkUnifiedIdeograph7474', [], 'Display', None],
  [ 0x7475, 'CjkUnifiedIdeograph7475', [], 'Display', None],
  [ 0x7476, 'CjkUnifiedIdeograph7476', [], 'Display', None],
  [ 0x7477, 'CjkUnifiedIdeograph7477', [], 'Display', None],
  [ 0x7478, 'CjkUnifiedIdeograph7478', [], 'Display', None],
  [ 0x7479, 'CjkUnifiedIdeograph7479', [], 'Display', None],
  [ 0x747a, 'CjkUnifiedIdeograph747a', [], 'Display', None],
  [ 0x747b, 'CjkUnifiedIdeograph747b', [], 'Display', None],
  [ 0x747c, 'CjkUnifiedIdeograph747c', [], 'Display', None],
  [ 0x747d, 'CjkUnifiedIdeograph747d', [], 'Display', None],
  [ 0x747e, 'CjkUnifiedIdeograph747e', [], 'Display', None],
  [ 0x747f, 'CjkUnifiedIdeograph747f', [], 'Display', None],
  [ 0x7480, 'CjkUnifiedIdeograph7480', [], 'Display', None],
  [ 0x7481, 'CjkUnifiedIdeograph7481', [], 'Display', None],
  [ 0x7482, 'CjkUnifiedIdeograph7482', [], 'Display', None],
  [ 0x7483, 'CjkUnifiedIdeograph7483', [], 'Display', None],
  [ 0x7484, 'CjkUnifiedIdeograph7484', [], 'Display', None],
  [ 0x7485, 'CjkUnifiedIdeograph7485', [], 'Display', None],
  [ 0x7486, 'CjkUnifiedIdeograph7486', [], 'Display', None],
  [ 0x7487, 'CjkUnifiedIdeograph7487', [], 'Display', None],
  [ 0x7488, 'CjkUnifiedIdeograph7488', [], 'Display', None],
  [ 0x7489, 'CjkUnifiedIdeograph7489', [], 'Display', None],
  [ 0x748a, 'CjkUnifiedIdeograph748a', [], 'Display', None],
  [ 0x748b, 'CjkUnifiedIdeograph748b', [], 'Display', None],
  [ 0x748c, 'CjkUnifiedIdeograph748c', [], 'Display', None],
  [ 0x748d, 'CjkUnifiedIdeograph748d', [], 'Display', None],
  [ 0x748e, 'CjkUnifiedIdeograph748e', [], 'Display', None],
  [ 0x748f, 'CjkUnifiedIdeograph748f', [], 'Display', None],
  [ 0x7490, 'CjkUnifiedIdeograph7490', [], 'Display', None],
  [ 0x7491, 'CjkUnifiedIdeograph7491', [], 'Display', None],
  [ 0x7492, 'CjkUnifiedIdeograph7492', [], 'Display', None],
  [ 0x7493, 'CjkUnifiedIdeograph7493', [], 'Display', None],
  [ 0x7494, 'CjkUnifiedIdeograph7494', [], 'Display', None],
  [ 0x7495, 'CjkUnifiedIdeograph7495', [], 'Display', None],
  [ 0x7496, 'CjkUnifiedIdeograph7496', [], 'Display', None],
  [ 0x7497, 'CjkUnifiedIdeograph7497', [], 'Display', None],
  [ 0x7498, 'CjkUnifiedIdeograph7498', [], 'Display', None],
  [ 0x7499, 'CjkUnifiedIdeograph7499', [], 'Display', None],
  [ 0x749a, 'CjkUnifiedIdeograph749a', [], 'Display', None],
  [ 0x749b, 'CjkUnifiedIdeograph749b', [], 'Display', None],
  [ 0x749c, 'CjkUnifiedIdeograph749c', [], 'Display', None],
  [ 0x749d, 'CjkUnifiedIdeograph749d', [], 'Display', None],
  [ 0x749e, 'CjkUnifiedIdeograph749e', [], 'Display', None],
  [ 0x749f, 'CjkUnifiedIdeograph749f', [], 'Display', None],
  [ 0x74a0, 'CjkUnifiedIdeograph74a0', [], 'Display', None],
  [ 0x74a1, 'CjkUnifiedIdeograph74a1', [], 'Display', None],
  [ 0x74a2, 'CjkUnifiedIdeograph74a2', [], 'Display', None],
  [ 0x74a3, 'CjkUnifiedIdeograph74a3', [], 'Display', None],
  [ 0x74a4, 'CjkUnifiedIdeograph74a4', [], 'Display', None],
  [ 0x74a5, 'CjkUnifiedIdeograph74a5', [], 'Display', None],
  [ 0x74a6, 'CjkUnifiedIdeograph74a6', [], 'Display', None],
  [ 0x74a7, 'CjkUnifiedIdeograph74a7', [], 'Display', None],
  [ 0x74a8, 'CjkUnifiedIdeograph74a8', [], 'Display', None],
  [ 0x74a9, 'CjkUnifiedIdeograph74a9', [], 'Display', None],
  [ 0x74aa, 'CjkUnifiedIdeograph74aa', [], 'Display', None],
  [ 0x74ab, 'CjkUnifiedIdeograph74ab', [], 'Display', None],
  [ 0x74ac, 'CjkUnifiedIdeograph74ac', [], 'Display', None],
  [ 0x74ad, 'CjkUnifiedIdeograph74ad', [], 'Display', None],
  [ 0x74ae, 'CjkUnifiedIdeograph74ae', [], 'Display', None],
  [ 0x74af, 'CjkUnifiedIdeograph74af', [], 'Display', None],
  [ 0x74b0, 'CjkUnifiedIdeograph74b0', [], 'Display', None],
  [ 0x74b1, 'CjkUnifiedIdeograph74b1', [], 'Display', None],
  [ 0x74b2, 'CjkUnifiedIdeograph74b2', [], 'Display', None],
  [ 0x74b3, 'CjkUnifiedIdeograph74b3', [], 'Display', None],
  [ 0x74b4, 'CjkUnifiedIdeograph74b4', [], 'Display', None],
  [ 0x74b5, 'CjkUnifiedIdeograph74b5', [], 'Display', None],
  [ 0x74b6, 'CjkUnifiedIdeograph74b6', [], 'Display', None],
  [ 0x74b7, 'CjkUnifiedIdeograph74b7', [], 'Display', None],
  [ 0x74b8, 'CjkUnifiedIdeograph74b8', [], 'Display', None],
  [ 0x74b9, 'CjkUnifiedIdeograph74b9', [], 'Display', None],
  [ 0x74ba, 'CjkUnifiedIdeograph74ba', [], 'Display', None],
  [ 0x74bb, 'CjkUnifiedIdeograph74bb', [], 'Display', None],
  [ 0x74bc, 'CjkUnifiedIdeograph74bc', [], 'Display', None],
  [ 0x74bd, 'CjkUnifiedIdeograph74bd', [], 'Display', None],
  [ 0x74be, 'CjkUnifiedIdeograph74be', [], 'Display', None],
  [ 0x74bf, 'CjkUnifiedIdeograph74bf', [], 'Display', None],
  [ 0x74c0, 'CjkUnifiedIdeograph74c0', [], 'Display', None],
  [ 0x74c1, 'CjkUnifiedIdeograph74c1', [], 'Display', None],
  [ 0x74c2, 'CjkUnifiedIdeograph74c2', [], 'Display', None],
  [ 0x74c3, 'CjkUnifiedIdeograph74c3', [], 'Display', None],
  [ 0x74c4, 'CjkUnifiedIdeograph74c4', [], 'Display', None],
  [ 0x74c5, 'CjkUnifiedIdeograph74c5', [], 'Display', None],
  [ 0x74c6, 'CjkUnifiedIdeograph74c6', [], 'Display', None],
  [ 0x74c7, 'CjkUnifiedIdeograph74c7', [], 'Display', None],
  [ 0x74c8, 'CjkUnifiedIdeograph74c8', [], 'Display', None],
  [ 0x74c9, 'CjkUnifiedIdeograph74c9', [], 'Display', None],
  [ 0x74ca, 'CjkUnifiedIdeograph74ca', [], 'Display', None],
  [ 0x74cb, 'CjkUnifiedIdeograph74cb', [], 'Display', None],
  [ 0x74cc, 'CjkUnifiedIdeograph74cc', [], 'Display', None],
  [ 0x74cd, 'CjkUnifiedIdeograph74cd', [], 'Display', None],
  [ 0x74ce, 'CjkUnifiedIdeograph74ce', [], 'Display', None],
  [ 0x74cf, 'CjkUnifiedIdeograph74cf', [], 'Display', None],
  [ 0x74d0, 'CjkUnifiedIdeograph74d0', [], 'Display', None],
  [ 0x74d1, 'CjkUnifiedIdeograph74d1', [], 'Display', None],
  [ 0x74d2, 'CjkUnifiedIdeograph74d2', [], 'Display', None],
  [ 0x74d3, 'CjkUnifiedIdeograph74d3', [], 'Display', None],
  [ 0x74d4, 'CjkUnifiedIdeograph74d4', [], 'Display', None],
  [ 0x74d5, 'CjkUnifiedIdeograph74d5', [], 'Display', None],
  [ 0x74d6, 'CjkUnifiedIdeograph74d6', [], 'Display', None],
  [ 0x74d7, 'CjkUnifiedIdeograph74d7', [], 'Display', None],
  [ 0x74d8, 'CjkUnifiedIdeograph74d8', [], 'Display', None],
  [ 0x74d9, 'CjkUnifiedIdeograph74d9', [], 'Display', None],
  [ 0x74da, 'CjkUnifiedIdeograph74da', [], 'Display', None],
  [ 0x74db, 'CjkUnifiedIdeograph74db', [], 'Display', None],
  [ 0x74dc, 'CjkUnifiedIdeograph74dc', [], 'Display', None],
  [ 0x74dd, 'CjkUnifiedIdeograph74dd', [], 'Display', None],
  [ 0x74de, 'CjkUnifiedIdeograph74de', [], 'Display', None],
  [ 0x74df, 'CjkUnifiedIdeograph74df', [], 'Display', None],
  [ 0x74e0, 'CjkUnifiedIdeograph74e0', [], 'Display', None],
  [ 0x74e1, 'CjkUnifiedIdeograph74e1', [], 'Display', None],
  [ 0x74e2, 'CjkUnifiedIdeograph74e2', [], 'Display', None],
  [ 0x74e3, 'CjkUnifiedIdeograph74e3', [], 'Display', None],
  [ 0x74e4, 'CjkUnifiedIdeograph74e4', [], 'Display', None],
  [ 0x74e5, 'CjkUnifiedIdeograph74e5', [], 'Display', None],
  [ 0x74e6, 'CjkUnifiedIdeograph74e6', [], 'Display', None],
  [ 0x74e7, 'CjkUnifiedIdeograph74e7', [], 'Display', None],
  [ 0x74e8, 'CjkUnifiedIdeograph74e8', [], 'Display', None],
  [ 0x74e9, 'CjkUnifiedIdeograph74e9', [], 'Display', None],
  [ 0x74ea, 'CjkUnifiedIdeograph74ea', [], 'Display', None],
  [ 0x74eb, 'CjkUnifiedIdeograph74eb', [], 'Display', None],
  [ 0x74ec, 'CjkUnifiedIdeograph74ec', [], 'Display', None],
  [ 0x74ed, 'CjkUnifiedIdeograph74ed', [], 'Display', None],
  [ 0x74ee, 'CjkUnifiedIdeograph74ee', [], 'Display', None],
  [ 0x74ef, 'CjkUnifiedIdeograph74ef', [], 'Display', None],
  [ 0x74f0, 'CjkUnifiedIdeograph74f0', [], 'Display', None],
  [ 0x74f1, 'CjkUnifiedIdeograph74f1', [], 'Display', None],
  [ 0x74f2, 'CjkUnifiedIdeograph74f2', [], 'Display', None],
  [ 0x74f3, 'CjkUnifiedIdeograph74f3', [], 'Display', None],
  [ 0x74f4, 'CjkUnifiedIdeograph74f4', [], 'Display', None],
  [ 0x74f5, 'CjkUnifiedIdeograph74f5', [], 'Display', None],
  [ 0x74f6, 'CjkUnifiedIdeograph74f6', [], 'Display', None],
  [ 0x74f7, 'CjkUnifiedIdeograph74f7', [], 'Display', None],
  [ 0x74f8, 'CjkUnifiedIdeograph74f8', [], 'Display', None],
  [ 0x74f9, 'CjkUnifiedIdeograph74f9', [], 'Display', None],
  [ 0x74fa, 'CjkUnifiedIdeograph74fa', [], 'Display', None],
  [ 0x74fb, 'CjkUnifiedIdeograph74fb', [], 'Display', None],
  [ 0x74fc, 'CjkUnifiedIdeograph74fc', [], 'Display', None],
  [ 0x74fd, 'CjkUnifiedIdeograph74fd', [], 'Display', None],
  [ 0x74fe, 'CjkUnifiedIdeograph74fe', [], 'Display', None],
  [ 0x74ff, 'CjkUnifiedIdeograph74ff', [], 'Display', None],
  [ 0x7500, 'CjkUnifiedIdeograph7500', [], 'Display', None],
  [ 0x7501, 'CjkUnifiedIdeograph7501', [], 'Display', None],
  [ 0x7502, 'CjkUnifiedIdeograph7502', [], 'Display', None],
  [ 0x7503, 'CjkUnifiedIdeograph7503', [], 'Display', None],
  [ 0x7504, 'CjkUnifiedIdeograph7504', [], 'Display', None],
  [ 0x7505, 'CjkUnifiedIdeograph7505', [], 'Display', None],
  [ 0x7506, 'CjkUnifiedIdeograph7506', [], 'Display', None],
  [ 0x7507, 'CjkUnifiedIdeograph7507', [], 'Display', None],
  [ 0x7508, 'CjkUnifiedIdeograph7508', [], 'Display', None],
  [ 0x7509, 'CjkUnifiedIdeograph7509', [], 'Display', None],
  [ 0x750a, 'CjkUnifiedIdeograph750a', [], 'Display', None],
  [ 0x750b, 'CjkUnifiedIdeograph750b', [], 'Display', None],
  [ 0x750c, 'CjkUnifiedIdeograph750c', [], 'Display', None],
  [ 0x750d, 'CjkUnifiedIdeograph750d', [], 'Display', None],
  [ 0x750e, 'CjkUnifiedIdeograph750e', [], 'Display', None],
  [ 0x750f, 'CjkUnifiedIdeograph750f', [], 'Display', None],
  [ 0x7510, 'CjkUnifiedIdeograph7510', [], 'Display', None],
  [ 0x7511, 'CjkUnifiedIdeograph7511', [], 'Display', None],
  [ 0x7512, 'CjkUnifiedIdeograph7512', [], 'Display', None],
  [ 0x7513, 'CjkUnifiedIdeograph7513', [], 'Display', None],
  [ 0x7514, 'CjkUnifiedIdeograph7514', [], 'Display', None],
  [ 0x7515, 'CjkUnifiedIdeograph7515', [], 'Display', None],
  [ 0x7516, 'CjkUnifiedIdeograph7516', [], 'Display', None],
  [ 0x7517, 'CjkUnifiedIdeograph7517', [], 'Display', None],
  [ 0x7518, 'CjkUnifiedIdeograph7518', [], 'Display', None],
  [ 0x7519, 'CjkUnifiedIdeograph7519', [], 'Display', None],
  [ 0x751a, 'CjkUnifiedIdeograph751a', [], 'Display', None],
  [ 0x751b, 'CjkUnifiedIdeograph751b', [], 'Display', None],
  [ 0x751c, 'CjkUnifiedIdeograph751c', [], 'Display', None],
  [ 0x751d, 'CjkUnifiedIdeograph751d', [], 'Display', None],
  [ 0x751e, 'CjkUnifiedIdeograph751e', [], 'Display', None],
  [ 0x751f, 'CjkUnifiedIdeograph751f', [], 'Display', None],
  [ 0x7520, 'CjkUnifiedIdeograph7520', [], 'Display', None],
  [ 0x7521, 'CjkUnifiedIdeograph7521', [], 'Display', None],
  [ 0x7522, 'CjkUnifiedIdeograph7522', [], 'Display', None],
  [ 0x7523, 'CjkUnifiedIdeograph7523', [], 'Display', None],
  [ 0x7524, 'CjkUnifiedIdeograph7524', [], 'Display', None],
  [ 0x7525, 'CjkUnifiedIdeograph7525', [], 'Display', None],
  [ 0x7526, 'CjkUnifiedIdeograph7526', [], 'Display', None],
  [ 0x7527, 'CjkUnifiedIdeograph7527', [], 'Display', None],
  [ 0x7528, 'CjkUnifiedIdeograph7528', [], 'Display', None],
  [ 0x7529, 'CjkUnifiedIdeograph7529', [], 'Display', None],
  [ 0x752a, 'CjkUnifiedIdeograph752a', [], 'Display', None],
  [ 0x752b, 'CjkUnifiedIdeograph752b', [], 'Display', None],
  [ 0x752c, 'CjkUnifiedIdeograph752c', [], 'Display', None],
  [ 0x752d, 'CjkUnifiedIdeograph752d', [], 'Display', None],
  [ 0x752e, 'CjkUnifiedIdeograph752e', [], 'Display', None],
  [ 0x752f, 'CjkUnifiedIdeograph752f', [], 'Display', None],
  [ 0x7530, 'CjkUnifiedIdeograph7530', [], 'Display', None],
  [ 0x7531, 'CjkUnifiedIdeograph7531', [], 'Display', None],
  [ 0x7532, 'CjkUnifiedIdeograph7532', [], 'Display', None],
  [ 0x7533, 'CjkUnifiedIdeograph7533', [], 'Display', None],
  [ 0x7534, 'CjkUnifiedIdeograph7534', [], 'Display', None],
  [ 0x7535, 'CjkUnifiedIdeograph7535', [], 'Display', None],
  [ 0x7536, 'CjkUnifiedIdeograph7536', [], 'Display', None],
  [ 0x7537, 'CjkUnifiedIdeograph7537', [], 'Display', None],
  [ 0x7538, 'CjkUnifiedIdeograph7538', [], 'Display', None],
  [ 0x7539, 'CjkUnifiedIdeograph7539', [], 'Display', None],
  [ 0x753a, 'CjkUnifiedIdeograph753a', [], 'Display', None],
  [ 0x753b, 'CjkUnifiedIdeograph753b', [], 'Display', None],
  [ 0x753c, 'CjkUnifiedIdeograph753c', [], 'Display', None],
  [ 0x753d, 'CjkUnifiedIdeograph753d', [], 'Display', None],
  [ 0x753e, 'CjkUnifiedIdeograph753e', [], 'Display', None],
  [ 0x753f, 'CjkUnifiedIdeograph753f', [], 'Display', None],
  [ 0x7540, 'CjkUnifiedIdeograph7540', [], 'Display', None],
  [ 0x7541, 'CjkUnifiedIdeograph7541', [], 'Display', None],
  [ 0x7542, 'CjkUnifiedIdeograph7542', [], 'Display', None],
  [ 0x7543, 'CjkUnifiedIdeograph7543', [], 'Display', None],
  [ 0x7544, 'CjkUnifiedIdeograph7544', [], 'Display', None],
  [ 0x7545, 'CjkUnifiedIdeograph7545', [], 'Display', None],
  [ 0x7546, 'CjkUnifiedIdeograph7546', [], 'Display', None],
  [ 0x7547, 'CjkUnifiedIdeograph7547', [], 'Display', None],
  [ 0x7548, 'CjkUnifiedIdeograph7548', [], 'Display', None],
  [ 0x7549, 'CjkUnifiedIdeograph7549', [], 'Display', None],
  [ 0x754a, 'CjkUnifiedIdeograph754a', [], 'Display', None],
  [ 0x754b, 'CjkUnifiedIdeograph754b', [], 'Display', None],
  [ 0x754c, 'CjkUnifiedIdeograph754c', [], 'Display', None],
  [ 0x754d, 'CjkUnifiedIdeograph754d', [], 'Display', None],
  [ 0x754e, 'CjkUnifiedIdeograph754e', [], 'Display', None],
  [ 0x754f, 'CjkUnifiedIdeograph754f', [], 'Display', None],
  [ 0x7550, 'CjkUnifiedIdeograph7550', [], 'Display', None],
  [ 0x7551, 'CjkUnifiedIdeograph7551', [], 'Display', None],
  [ 0x7552, 'CjkUnifiedIdeograph7552', [], 'Display', None],
  [ 0x7553, 'CjkUnifiedIdeograph7553', [], 'Display', None],
  [ 0x7554, 'CjkUnifiedIdeograph7554', [], 'Display', None],
  [ 0x7555, 'CjkUnifiedIdeograph7555', [], 'Display', None],
  [ 0x7556, 'CjkUnifiedIdeograph7556', [], 'Display', None],
  [ 0x7557, 'CjkUnifiedIdeograph7557', [], 'Display', None],
  [ 0x7558, 'CjkUnifiedIdeograph7558', [], 'Display', None],
  [ 0x7559, 'CjkUnifiedIdeograph7559', [], 'Display', None],
  [ 0x755a, 'CjkUnifiedIdeograph755a', [], 'Display', None],
  [ 0x755b, 'CjkUnifiedIdeograph755b', [], 'Display', None],
  [ 0x755c, 'CjkUnifiedIdeograph755c', [], 'Display', None],
  [ 0x755d, 'CjkUnifiedIdeograph755d', [], 'Display', None],
  [ 0x755e, 'CjkUnifiedIdeograph755e', [], 'Display', None],
  [ 0x755f, 'CjkUnifiedIdeograph755f', [], 'Display', None],
  [ 0x7560, 'CjkUnifiedIdeograph7560', [], 'Display', None],
  [ 0x7561, 'CjkUnifiedIdeograph7561', [], 'Display', None],
  [ 0x7562, 'CjkUnifiedIdeograph7562', [], 'Display', None],
  [ 0x7563, 'CjkUnifiedIdeograph7563', [], 'Display', None],
  [ 0x7564, 'CjkUnifiedIdeograph7564', [], 'Display', None],
  [ 0x7565, 'CjkUnifiedIdeograph7565', [], 'Display', None],
  [ 0x7566, 'CjkUnifiedIdeograph7566', [], 'Display', None],
  [ 0x7567, 'CjkUnifiedIdeograph7567', [], 'Display', None],
  [ 0x7568, 'CjkUnifiedIdeograph7568', [], 'Display', None],
  [ 0x7569, 'CjkUnifiedIdeograph7569', [], 'Display', None],
  [ 0x756a, 'CjkUnifiedIdeograph756a', [], 'Display', None],
  [ 0x756b, 'CjkUnifiedIdeograph756b', [], 'Display', None],
  [ 0x756c, 'CjkUnifiedIdeograph756c', [], 'Display', None],
  [ 0x756d, 'CjkUnifiedIdeograph756d', [], 'Display', None],
  [ 0x756e, 'CjkUnifiedIdeograph756e', [], 'Display', None],
  [ 0x756f, 'CjkUnifiedIdeograph756f', [], 'Display', None],
  [ 0x7570, 'CjkUnifiedIdeograph7570', [], 'Display', None],
  [ 0x7571, 'CjkUnifiedIdeograph7571', [], 'Display', None],
  [ 0x7572, 'CjkUnifiedIdeograph7572', [], 'Display', None],
  [ 0x7573, 'CjkUnifiedIdeograph7573', [], 'Display', None],
  [ 0x7574, 'CjkUnifiedIdeograph7574', [], 'Display', None],
  [ 0x7575, 'CjkUnifiedIdeograph7575', [], 'Display', None],
  [ 0x7576, 'CjkUnifiedIdeograph7576', [], 'Display', None],
  [ 0x7577, 'CjkUnifiedIdeograph7577', [], 'Display', None],
  [ 0x7578, 'CjkUnifiedIdeograph7578', [], 'Display', None],
  [ 0x7579, 'CjkUnifiedIdeograph7579', [], 'Display', None],
  [ 0x757a, 'CjkUnifiedIdeograph757a', [], 'Display', None],
  [ 0x757b, 'CjkUnifiedIdeograph757b', [], 'Display', None],
  [ 0x757c, 'CjkUnifiedIdeograph757c', [], 'Display', None],
  [ 0x757d, 'CjkUnifiedIdeograph757d', [], 'Display', None],
  [ 0x757e, 'CjkUnifiedIdeograph757e', [], 'Display', None],
  [ 0x757f, 'CjkUnifiedIdeograph757f', [], 'Display', None],
  [ 0x7580, 'CjkUnifiedIdeograph7580', [], 'Display', None],
  [ 0x7581, 'CjkUnifiedIdeograph7581', [], 'Display', None],
  [ 0x7582, 'CjkUnifiedIdeograph7582', [], 'Display', None],
  [ 0x7583, 'CjkUnifiedIdeograph7583', [], 'Display', None],
  [ 0x7584, 'CjkUnifiedIdeograph7584', [], 'Display', None],
  [ 0x7585, 'CjkUnifiedIdeograph7585', [], 'Display', None],
  [ 0x7586, 'CjkUnifiedIdeograph7586', [], 'Display', None],
  [ 0x7587, 'CjkUnifiedIdeograph7587', [], 'Display', None],
  [ 0x7588, 'CjkUnifiedIdeograph7588', [], 'Display', None],
  [ 0x7589, 'CjkUnifiedIdeograph7589', [], 'Display', None],
  [ 0x758a, 'CjkUnifiedIdeograph758a', [], 'Display', None],
  [ 0x758b, 'CjkUnifiedIdeograph758b', [], 'Display', None],
  [ 0x758c, 'CjkUnifiedIdeograph758c', [], 'Display', None],
  [ 0x758d, 'CjkUnifiedIdeograph758d', [], 'Display', None],
  [ 0x758e, 'CjkUnifiedIdeograph758e', [], 'Display', None],
  [ 0x758f, 'CjkUnifiedIdeograph758f', [], 'Display', None],
  [ 0x7590, 'CjkUnifiedIdeograph7590', [], 'Display', None],
  [ 0x7591, 'CjkUnifiedIdeograph7591', [], 'Display', None],
  [ 0x7592, 'CjkUnifiedIdeograph7592', [], 'Display', None],
  [ 0x7593, 'CjkUnifiedIdeograph7593', [], 'Display', None],
  [ 0x7594, 'CjkUnifiedIdeograph7594', [], 'Display', None],
  [ 0x7595, 'CjkUnifiedIdeograph7595', [], 'Display', None],
  [ 0x7596, 'CjkUnifiedIdeograph7596', [], 'Display', None],
  [ 0x7597, 'CjkUnifiedIdeograph7597', [], 'Display', None],
  [ 0x7598, 'CjkUnifiedIdeograph7598', [], 'Display', None],
  [ 0x7599, 'CjkUnifiedIdeograph7599', [], 'Display', None],
  [ 0x759a, 'CjkUnifiedIdeograph759a', [], 'Display', None],
  [ 0x759b, 'CjkUnifiedIdeograph759b', [], 'Display', None],
  [ 0x759c, 'CjkUnifiedIdeograph759c', [], 'Display', None],
  [ 0x759d, 'CjkUnifiedIdeograph759d', [], 'Display', None],
  [ 0x759e, 'CjkUnifiedIdeograph759e', [], 'Display', None],
  [ 0x759f, 'CjkUnifiedIdeograph759f', [], 'Display', None],
  [ 0x75a0, 'CjkUnifiedIdeograph75a0', [], 'Display', None],
  [ 0x75a1, 'CjkUnifiedIdeograph75a1', [], 'Display', None],
  [ 0x75a2, 'CjkUnifiedIdeograph75a2', [], 'Display', None],
  [ 0x75a3, 'CjkUnifiedIdeograph75a3', [], 'Display', None],
  [ 0x75a4, 'CjkUnifiedIdeograph75a4', [], 'Display', None],
  [ 0x75a5, 'CjkUnifiedIdeograph75a5', [], 'Display', None],
  [ 0x75a6, 'CjkUnifiedIdeograph75a6', [], 'Display', None],
  [ 0x75a7, 'CjkUnifiedIdeograph75a7', [], 'Display', None],
  [ 0x75a8, 'CjkUnifiedIdeograph75a8', [], 'Display', None],
  [ 0x75a9, 'CjkUnifiedIdeograph75a9', [], 'Display', None],
  [ 0x75aa, 'CjkUnifiedIdeograph75aa', [], 'Display', None],
  [ 0x75ab, 'CjkUnifiedIdeograph75ab', [], 'Display', None],
  [ 0x75ac, 'CjkUnifiedIdeograph75ac', [], 'Display', None],
  [ 0x75ad, 'CjkUnifiedIdeograph75ad', [], 'Display', None],
  [ 0x75ae, 'CjkUnifiedIdeograph75ae', [], 'Display', None],
  [ 0x75af, 'CjkUnifiedIdeograph75af', [], 'Display', None],
  [ 0x75b0, 'CjkUnifiedIdeograph75b0', [], 'Display', None],
  [ 0x75b1, 'CjkUnifiedIdeograph75b1', [], 'Display', None],
  [ 0x75b2, 'CjkUnifiedIdeograph75b2', [], 'Display', None],
  [ 0x75b3, 'CjkUnifiedIdeograph75b3', [], 'Display', None],
  [ 0x75b4, 'CjkUnifiedIdeograph75b4', [], 'Display', None],
  [ 0x75b5, 'CjkUnifiedIdeograph75b5', [], 'Display', None],
  [ 0x75b6, 'CjkUnifiedIdeograph75b6', [], 'Display', None],
  [ 0x75b7, 'CjkUnifiedIdeograph75b7', [], 'Display', None],
  [ 0x75b8, 'CjkUnifiedIdeograph75b8', [], 'Display', None],
  [ 0x75b9, 'CjkUnifiedIdeograph75b9', [], 'Display', None],
  [ 0x75ba, 'CjkUnifiedIdeograph75ba', [], 'Display', None],
  [ 0x75bb, 'CjkUnifiedIdeograph75bb', [], 'Display', None],
  [ 0x75bc, 'CjkUnifiedIdeograph75bc', [], 'Display', None],
  [ 0x75bd, 'CjkUnifiedIdeograph75bd', [], 'Display', None],
  [ 0x75be, 'CjkUnifiedIdeograph75be', [], 'Display', None],
  [ 0x75bf, 'CjkUnifiedIdeograph75bf', [], 'Display', None],
  [ 0x75c0, 'CjkUnifiedIdeograph75c0', [], 'Display', None],
  [ 0x75c1, 'CjkUnifiedIdeograph75c1', [], 'Display', None],
  [ 0x75c2, 'CjkUnifiedIdeograph75c2', [], 'Display', None],
  [ 0x75c3, 'CjkUnifiedIdeograph75c3', [], 'Display', None],
  [ 0x75c4, 'CjkUnifiedIdeograph75c4', [], 'Display', None],
  [ 0x75c5, 'CjkUnifiedIdeograph75c5', [], 'Display', None],
  [ 0x75c6, 'CjkUnifiedIdeograph75c6', [], 'Display', None],
  [ 0x75c7, 'CjkUnifiedIdeograph75c7', [], 'Display', None],
  [ 0x75c8, 'CjkUnifiedIdeograph75c8', [], 'Display', None],
  [ 0x75c9, 'CjkUnifiedIdeograph75c9', [], 'Display', None],
  [ 0x75ca, 'CjkUnifiedIdeograph75ca', [], 'Display', None],
  [ 0x75cb, 'CjkUnifiedIdeograph75cb', [], 'Display', None],
  [ 0x75cc, 'CjkUnifiedIdeograph75cc', [], 'Display', None],
  [ 0x75cd, 'CjkUnifiedIdeograph75cd', [], 'Display', None],
  [ 0x75ce, 'CjkUnifiedIdeograph75ce', [], 'Display', None],
  [ 0x75cf, 'CjkUnifiedIdeograph75cf', [], 'Display', None],
  [ 0x75d0, 'CjkUnifiedIdeograph75d0', [], 'Display', None],
  [ 0x75d1, 'CjkUnifiedIdeograph75d1', [], 'Display', None],
  [ 0x75d2, 'CjkUnifiedIdeograph75d2', [], 'Display', None],
  [ 0x75d3, 'CjkUnifiedIdeograph75d3', [], 'Display', None],
  [ 0x75d4, 'CjkUnifiedIdeograph75d4', [], 'Display', None],
  [ 0x75d5, 'CjkUnifiedIdeograph75d5', [], 'Display', None],
  [ 0x75d6, 'CjkUnifiedIdeograph75d6', [], 'Display', None],
  [ 0x75d7, 'CjkUnifiedIdeograph75d7', [], 'Display', None],
  [ 0x75d8, 'CjkUnifiedIdeograph75d8', [], 'Display', None],
  [ 0x75d9, 'CjkUnifiedIdeograph75d9', [], 'Display', None],
  [ 0x75da, 'CjkUnifiedIdeograph75da', [], 'Display', None],
  [ 0x75db, 'CjkUnifiedIdeograph75db', [], 'Display', None],
  [ 0x75dc, 'CjkUnifiedIdeograph75dc', [], 'Display', None],
  [ 0x75dd, 'CjkUnifiedIdeograph75dd', [], 'Display', None],
  [ 0x75de, 'CjkUnifiedIdeograph75de', [], 'Display', None],
  [ 0x75df, 'CjkUnifiedIdeograph75df', [], 'Display', None],
  [ 0x75e0, 'CjkUnifiedIdeograph75e0', [], 'Display', None],
  [ 0x75e1, 'CjkUnifiedIdeograph75e1', [], 'Display', None],
  [ 0x75e2, 'CjkUnifiedIdeograph75e2', [], 'Display', None],
  [ 0x75e3, 'CjkUnifiedIdeograph75e3', [], 'Display', None],
  [ 0x75e4, 'CjkUnifiedIdeograph75e4', [], 'Display', None],
  [ 0x75e5, 'CjkUnifiedIdeograph75e5', [], 'Display', None],
  [ 0x75e6, 'CjkUnifiedIdeograph75e6', [], 'Display', None],
  [ 0x75e7, 'CjkUnifiedIdeograph75e7', [], 'Display', None],
  [ 0x75e8, 'CjkUnifiedIdeograph75e8', [], 'Display', None],
  [ 0x75e9, 'CjkUnifiedIdeograph75e9', [], 'Display', None],
  [ 0x75ea, 'CjkUnifiedIdeograph75ea', [], 'Display', None],
  [ 0x75eb, 'CjkUnifiedIdeograph75eb', [], 'Display', None],
  [ 0x75ec, 'CjkUnifiedIdeograph75ec', [], 'Display', None],
  [ 0x75ed, 'CjkUnifiedIdeograph75ed', [], 'Display', None],
  [ 0x75ee, 'CjkUnifiedIdeograph75ee', [], 'Display', None],
  [ 0x75ef, 'CjkUnifiedIdeograph75ef', [], 'Display', None],
  [ 0x75f0, 'CjkUnifiedIdeograph75f0', [], 'Display', None],
  [ 0x75f1, 'CjkUnifiedIdeograph75f1', [], 'Display', None],
  [ 0x75f2, 'CjkUnifiedIdeograph75f2', [], 'Display', None],
  [ 0x75f3, 'CjkUnifiedIdeograph75f3', [], 'Display', None],
  [ 0x75f4, 'CjkUnifiedIdeograph75f4', [], 'Display', None],
  [ 0x75f5, 'CjkUnifiedIdeograph75f5', [], 'Display', None],
  [ 0x75f6, 'CjkUnifiedIdeograph75f6', [], 'Display', None],
  [ 0x75f7, 'CjkUnifiedIdeograph75f7', [], 'Display', None],
  [ 0x75f8, 'CjkUnifiedIdeograph75f8', [], 'Display', None],
  [ 0x75f9, 'CjkUnifiedIdeograph75f9', [], 'Display', None],
  [ 0x75fa, 'CjkUnifiedIdeograph75fa', [], 'Display', None],
  [ 0x75fb, 'CjkUnifiedIdeograph75fb', [], 'Display', None],
  [ 0x75fc, 'CjkUnifiedIdeograph75fc', [], 'Display', None],
  [ 0x75fd, 'CjkUnifiedIdeograph75fd', [], 'Display', None],
  [ 0x75fe, 'CjkUnifiedIdeograph75fe', [], 'Display', None],
  [ 0x75ff, 'CjkUnifiedIdeograph75ff', [], 'Display', None],
  [ 0x7600, 'CjkUnifiedIdeograph7600', [], 'Display', None],
  [ 0x7601, 'CjkUnifiedIdeograph7601', [], 'Display', None],
  [ 0x7602, 'CjkUnifiedIdeograph7602', [], 'Display', None],
  [ 0x7603, 'CjkUnifiedIdeograph7603', [], 'Display', None],
  [ 0x7604, 'CjkUnifiedIdeograph7604', [], 'Display', None],
  [ 0x7605, 'CjkUnifiedIdeograph7605', [], 'Display', None],
  [ 0x7606, 'CjkUnifiedIdeograph7606', [], 'Display', None],
  [ 0x7607, 'CjkUnifiedIdeograph7607', [], 'Display', None],
  [ 0x7608, 'CjkUnifiedIdeograph7608', [], 'Display', None],
  [ 0x7609, 'CjkUnifiedIdeograph7609', [], 'Display', None],
  [ 0x760a, 'CjkUnifiedIdeograph760a', [], 'Display', None],
  [ 0x760b, 'CjkUnifiedIdeograph760b', [], 'Display', None],
  [ 0x760c, 'CjkUnifiedIdeograph760c', [], 'Display', None],
  [ 0x760d, 'CjkUnifiedIdeograph760d', [], 'Display', None],
  [ 0x760e, 'CjkUnifiedIdeograph760e', [], 'Display', None],
  [ 0x760f, 'CjkUnifiedIdeograph760f', [], 'Display', None],
  [ 0x7610, 'CjkUnifiedIdeograph7610', [], 'Display', None],
  [ 0x7611, 'CjkUnifiedIdeograph7611', [], 'Display', None],
  [ 0x7612, 'CjkUnifiedIdeograph7612', [], 'Display', None],
  [ 0x7613, 'CjkUnifiedIdeograph7613', [], 'Display', None],
  [ 0x7614, 'CjkUnifiedIdeograph7614', [], 'Display', None],
  [ 0x7615, 'CjkUnifiedIdeograph7615', [], 'Display', None],
  [ 0x7616, 'CjkUnifiedIdeograph7616', [], 'Display', None],
  [ 0x7617, 'CjkUnifiedIdeograph7617', [], 'Display', None],
  [ 0x7618, 'CjkUnifiedIdeograph7618', [], 'Display', None],
  [ 0x7619, 'CjkUnifiedIdeograph7619', [], 'Display', None],
  [ 0x761a, 'CjkUnifiedIdeograph761a', [], 'Display', None],
  [ 0x761b, 'CjkUnifiedIdeograph761b', [], 'Display', None],
  [ 0x761c, 'CjkUnifiedIdeograph761c', [], 'Display', None],
  [ 0x761d, 'CjkUnifiedIdeograph761d', [], 'Display', None],
  [ 0x761e, 'CjkUnifiedIdeograph761e', [], 'Display', None],
  [ 0x761f, 'CjkUnifiedIdeograph761f', [], 'Display', None],
  [ 0x7620, 'CjkUnifiedIdeograph7620', [], 'Display', None],
  [ 0x7621, 'CjkUnifiedIdeograph7621', [], 'Display', None],
  [ 0x7622, 'CjkUnifiedIdeograph7622', [], 'Display', None],
  [ 0x7623, 'CjkUnifiedIdeograph7623', [], 'Display', None],
  [ 0x7624, 'CjkUnifiedIdeograph7624', [], 'Display', None],
  [ 0x7625, 'CjkUnifiedIdeograph7625', [], 'Display', None],
  [ 0x7626, 'CjkUnifiedIdeograph7626', [], 'Display', None],
  [ 0x7627, 'CjkUnifiedIdeograph7627', [], 'Display', None],
  [ 0x7628, 'CjkUnifiedIdeograph7628', [], 'Display', None],
  [ 0x7629, 'CjkUnifiedIdeograph7629', [], 'Display', None],
  [ 0x762a, 'CjkUnifiedIdeograph762a', [], 'Display', None],
  [ 0x762b, 'CjkUnifiedIdeograph762b', [], 'Display', None],
  [ 0x762c, 'CjkUnifiedIdeograph762c', [], 'Display', None],
  [ 0x762d, 'CjkUnifiedIdeograph762d', [], 'Display', None],
  [ 0x762e, 'CjkUnifiedIdeograph762e', [], 'Display', None],
  [ 0x762f, 'CjkUnifiedIdeograph762f', [], 'Display', None],
  [ 0x7630, 'CjkUnifiedIdeograph7630', [], 'Display', None],
  [ 0x7631, 'CjkUnifiedIdeograph7631', [], 'Display', None],
  [ 0x7632, 'CjkUnifiedIdeograph7632', [], 'Display', None],
  [ 0x7633, 'CjkUnifiedIdeograph7633', [], 'Display', None],
  [ 0x7634, 'CjkUnifiedIdeograph7634', [], 'Display', None],
  [ 0x7635, 'CjkUnifiedIdeograph7635', [], 'Display', None],
  [ 0x7636, 'CjkUnifiedIdeograph7636', [], 'Display', None],
  [ 0x7637, 'CjkUnifiedIdeograph7637', [], 'Display', None],
  [ 0x7638, 'CjkUnifiedIdeograph7638', [], 'Display', None],
  [ 0x7639, 'CjkUnifiedIdeograph7639', [], 'Display', None],
  [ 0x763a, 'CjkUnifiedIdeograph763a', [], 'Display', None],
  [ 0x763b, 'CjkUnifiedIdeograph763b', [], 'Display', None],
  [ 0x763c, 'CjkUnifiedIdeograph763c', [], 'Display', None],
  [ 0x763d, 'CjkUnifiedIdeograph763d', [], 'Display', None],
  [ 0x763e, 'CjkUnifiedIdeograph763e', [], 'Display', None],
  [ 0x763f, 'CjkUnifiedIdeograph763f', [], 'Display', None],
  [ 0x7640, 'CjkUnifiedIdeograph7640', [], 'Display', None],
  [ 0x7641, 'CjkUnifiedIdeograph7641', [], 'Display', None],
  [ 0x7642, 'CjkUnifiedIdeograph7642', [], 'Display', None],
  [ 0x7643, 'CjkUnifiedIdeograph7643', [], 'Display', None],
  [ 0x7644, 'CjkUnifiedIdeograph7644', [], 'Display', None],
  [ 0x7645, 'CjkUnifiedIdeograph7645', [], 'Display', None],
  [ 0x7646, 'CjkUnifiedIdeograph7646', [], 'Display', None],
  [ 0x7647, 'CjkUnifiedIdeograph7647', [], 'Display', None],
  [ 0x7648, 'CjkUnifiedIdeograph7648', [], 'Display', None],
  [ 0x7649, 'CjkUnifiedIdeograph7649', [], 'Display', None],
  [ 0x764a, 'CjkUnifiedIdeograph764a', [], 'Display', None],
  [ 0x764b, 'CjkUnifiedIdeograph764b', [], 'Display', None],
  [ 0x764c, 'CjkUnifiedIdeograph764c', [], 'Display', None],
  [ 0x764d, 'CjkUnifiedIdeograph764d', [], 'Display', None],
  [ 0x764e, 'CjkUnifiedIdeograph764e', [], 'Display', None],
  [ 0x764f, 'CjkUnifiedIdeograph764f', [], 'Display', None],
  [ 0x7650, 'CjkUnifiedIdeograph7650', [], 'Display', None],
  [ 0x7651, 'CjkUnifiedIdeograph7651', [], 'Display', None],
  [ 0x7652, 'CjkUnifiedIdeograph7652', [], 'Display', None],
  [ 0x7653, 'CjkUnifiedIdeograph7653', [], 'Display', None],
  [ 0x7654, 'CjkUnifiedIdeograph7654', [], 'Display', None],
  [ 0x7655, 'CjkUnifiedIdeograph7655', [], 'Display', None],
  [ 0x7656, 'CjkUnifiedIdeograph7656', [], 'Display', None],
  [ 0x7657, 'CjkUnifiedIdeograph7657', [], 'Display', None],
  [ 0x7658, 'CjkUnifiedIdeograph7658', [], 'Display', None],
  [ 0x7659, 'CjkUnifiedIdeograph7659', [], 'Display', None],
  [ 0x765a, 'CjkUnifiedIdeograph765a', [], 'Display', None],
  [ 0x765b, 'CjkUnifiedIdeograph765b', [], 'Display', None],
  [ 0x765c, 'CjkUnifiedIdeograph765c', [], 'Display', None],
  [ 0x765d, 'CjkUnifiedIdeograph765d', [], 'Display', None],
  [ 0x765e, 'CjkUnifiedIdeograph765e', [], 'Display', None],
  [ 0x765f, 'CjkUnifiedIdeograph765f', [], 'Display', None],
  [ 0x7660, 'CjkUnifiedIdeograph7660', [], 'Display', None],
  [ 0x7661, 'CjkUnifiedIdeograph7661', [], 'Display', None],
  [ 0x7662, 'CjkUnifiedIdeograph7662', [], 'Display', None],
  [ 0x7663, 'CjkUnifiedIdeograph7663', [], 'Display', None],
  [ 0x7664, 'CjkUnifiedIdeograph7664', [], 'Display', None],
  [ 0x7665, 'CjkUnifiedIdeograph7665', [], 'Display', None],
  [ 0x7666, 'CjkUnifiedIdeograph7666', [], 'Display', None],
  [ 0x7667, 'CjkUnifiedIdeograph7667', [], 'Display', None],
  [ 0x7668, 'CjkUnifiedIdeograph7668', [], 'Display', None],
  [ 0x7669, 'CjkUnifiedIdeograph7669', [], 'Display', None],
  [ 0x766a, 'CjkUnifiedIdeograph766a', [], 'Display', None],
  [ 0x766b, 'CjkUnifiedIdeograph766b', [], 'Display', None],
  [ 0x766c, 'CjkUnifiedIdeograph766c', [], 'Display', None],
  [ 0x766d, 'CjkUnifiedIdeograph766d', [], 'Display', None],
  [ 0x766e, 'CjkUnifiedIdeograph766e', [], 'Display', None],
  [ 0x766f, 'CjkUnifiedIdeograph766f', [], 'Display', None],
  [ 0x7670, 'CjkUnifiedIdeograph7670', [], 'Display', None],
  [ 0x7671, 'CjkUnifiedIdeograph7671', [], 'Display', None],
  [ 0x7672, 'CjkUnifiedIdeograph7672', [], 'Display', None],
  [ 0x7673, 'CjkUnifiedIdeograph7673', [], 'Display', None],
  [ 0x7674, 'CjkUnifiedIdeograph7674', [], 'Display', None],
  [ 0x7675, 'CjkUnifiedIdeograph7675', [], 'Display', None],
  [ 0x7676, 'CjkUnifiedIdeograph7676', [], 'Display', None],
  [ 0x7677, 'CjkUnifiedIdeograph7677', [], 'Display', None],
  [ 0x7678, 'CjkUnifiedIdeograph7678', [], 'Display', None],
  [ 0x7679, 'CjkUnifiedIdeograph7679', [], 'Display', None],
  [ 0x767a, 'CjkUnifiedIdeograph767a', [], 'Display', None],
  [ 0x767b, 'CjkUnifiedIdeograph767b', [], 'Display', None],
  [ 0x767c, 'CjkUnifiedIdeograph767c', [], 'Display', None],
  [ 0x767d, 'CjkUnifiedIdeograph767d', [], 'Display', None],
  [ 0x767e, 'CjkUnifiedIdeograph767e', [], 'Display', None],
  [ 0x767f, 'CjkUnifiedIdeograph767f', [], 'Display', None],
  [ 0x7680, 'CjkUnifiedIdeograph7680', [], 'Display', None],
  [ 0x7681, 'CjkUnifiedIdeograph7681', [], 'Display', None],
  [ 0x7682, 'CjkUnifiedIdeograph7682', [], 'Display', None],
  [ 0x7683, 'CjkUnifiedIdeograph7683', [], 'Display', None],
  [ 0x7684, 'CjkUnifiedIdeograph7684', [], 'Display', None],
  [ 0x7685, 'CjkUnifiedIdeograph7685', [], 'Display', None],
  [ 0x7686, 'CjkUnifiedIdeograph7686', [], 'Display', None],
  [ 0x7687, 'CjkUnifiedIdeograph7687', [], 'Display', None],
  [ 0x7688, 'CjkUnifiedIdeograph7688', [], 'Display', None],
  [ 0x7689, 'CjkUnifiedIdeograph7689', [], 'Display', None],
  [ 0x768a, 'CjkUnifiedIdeograph768a', [], 'Display', None],
  [ 0x768b, 'CjkUnifiedIdeograph768b', [], 'Display', None],
  [ 0x768c, 'CjkUnifiedIdeograph768c', [], 'Display', None],
  [ 0x768d, 'CjkUnifiedIdeograph768d', [], 'Display', None],
  [ 0x768e, 'CjkUnifiedIdeograph768e', [], 'Display', None],
  [ 0x768f, 'CjkUnifiedIdeograph768f', [], 'Display', None],
  [ 0x7690, 'CjkUnifiedIdeograph7690', [], 'Display', None],
  [ 0x7691, 'CjkUnifiedIdeograph7691', [], 'Display', None],
  [ 0x7692, 'CjkUnifiedIdeograph7692', [], 'Display', None],
  [ 0x7693, 'CjkUnifiedIdeograph7693', [], 'Display', None],
  [ 0x7694, 'CjkUnifiedIdeograph7694', [], 'Display', None],
  [ 0x7695, 'CjkUnifiedIdeograph7695', [], 'Display', None],
  [ 0x7696, 'CjkUnifiedIdeograph7696', [], 'Display', None],
  [ 0x7697, 'CjkUnifiedIdeograph7697', [], 'Display', None],
  [ 0x7698, 'CjkUnifiedIdeograph7698', [], 'Display', None],
  [ 0x7699, 'CjkUnifiedIdeograph7699', [], 'Display', None],
  [ 0x769a, 'CjkUnifiedIdeograph769a', [], 'Display', None],
  [ 0x769b, 'CjkUnifiedIdeograph769b', [], 'Display', None],
  [ 0x769c, 'CjkUnifiedIdeograph769c', [], 'Display', None],
  [ 0x769d, 'CjkUnifiedIdeograph769d', [], 'Display', None],
  [ 0x769e, 'CjkUnifiedIdeograph769e', [], 'Display', None],
  [ 0x769f, 'CjkUnifiedIdeograph769f', [], 'Display', None],
  [ 0x76a0, 'CjkUnifiedIdeograph76a0', [], 'Display', None],
  [ 0x76a1, 'CjkUnifiedIdeograph76a1', [], 'Display', None],
  [ 0x76a2, 'CjkUnifiedIdeograph76a2', [], 'Display', None],
  [ 0x76a3, 'CjkUnifiedIdeograph76a3', [], 'Display', None],
  [ 0x76a4, 'CjkUnifiedIdeograph76a4', [], 'Display', None],
  [ 0x76a5, 'CjkUnifiedIdeograph76a5', [], 'Display', None],
  [ 0x76a6, 'CjkUnifiedIdeograph76a6', [], 'Display', None],
  [ 0x76a7, 'CjkUnifiedIdeograph76a7', [], 'Display', None],
  [ 0x76a8, 'CjkUnifiedIdeograph76a8', [], 'Display', None],
  [ 0x76a9, 'CjkUnifiedIdeograph76a9', [], 'Display', None],
  [ 0x76aa, 'CjkUnifiedIdeograph76aa', [], 'Display', None],
  [ 0x76ab, 'CjkUnifiedIdeograph76ab', [], 'Display', None],
  [ 0x76ac, 'CjkUnifiedIdeograph76ac', [], 'Display', None],
  [ 0x76ad, 'CjkUnifiedIdeograph76ad', [], 'Display', None],
  [ 0x76ae, 'CjkUnifiedIdeograph76ae', [], 'Display', None],
  [ 0x76af, 'CjkUnifiedIdeograph76af', [], 'Display', None],
  [ 0x76b0, 'CjkUnifiedIdeograph76b0', [], 'Display', None],
  [ 0x76b1, 'CjkUnifiedIdeograph76b1', [], 'Display', None],
  [ 0x76b2, 'CjkUnifiedIdeograph76b2', [], 'Display', None],
  [ 0x76b3, 'CjkUnifiedIdeograph76b3', [], 'Display', None],
  [ 0x76b4, 'CjkUnifiedIdeograph76b4', [], 'Display', None],
  [ 0x76b5, 'CjkUnifiedIdeograph76b5', [], 'Display', None],
  [ 0x76b6, 'CjkUnifiedIdeograph76b6', [], 'Display', None],
  [ 0x76b7, 'CjkUnifiedIdeograph76b7', [], 'Display', None],
  [ 0x76b8, 'CjkUnifiedIdeograph76b8', [], 'Display', None],
  [ 0x76b9, 'CjkUnifiedIdeograph76b9', [], 'Display', None],
  [ 0x76ba, 'CjkUnifiedIdeograph76ba', [], 'Display', None],
  [ 0x76bb, 'CjkUnifiedIdeograph76bb', [], 'Display', None],
  [ 0x76bc, 'CjkUnifiedIdeograph76bc', [], 'Display', None],
  [ 0x76bd, 'CjkUnifiedIdeograph76bd', [], 'Display', None],
  [ 0x76be, 'CjkUnifiedIdeograph76be', [], 'Display', None],
  [ 0x76bf, 'CjkUnifiedIdeograph76bf', [], 'Display', None],
  [ 0x76c0, 'CjkUnifiedIdeograph76c0', [], 'Display', None],
  [ 0x76c1, 'CjkUnifiedIdeograph76c1', [], 'Display', None],
  [ 0x76c2, 'CjkUnifiedIdeograph76c2', [], 'Display', None],
  [ 0x76c3, 'CjkUnifiedIdeograph76c3', [], 'Display', None],
  [ 0x76c4, 'CjkUnifiedIdeograph76c4', [], 'Display', None],
  [ 0x76c5, 'CjkUnifiedIdeograph76c5', [], 'Display', None],
  [ 0x76c6, 'CjkUnifiedIdeograph76c6', [], 'Display', None],
  [ 0x76c7, 'CjkUnifiedIdeograph76c7', [], 'Display', None],
  [ 0x76c8, 'CjkUnifiedIdeograph76c8', [], 'Display', None],
  [ 0x76c9, 'CjkUnifiedIdeograph76c9', [], 'Display', None],
  [ 0x76ca, 'CjkUnifiedIdeograph76ca', [], 'Display', None],
  [ 0x76cb, 'CjkUnifiedIdeograph76cb', [], 'Display', None],
  [ 0x76cc, 'CjkUnifiedIdeograph76cc', [], 'Display', None],
  [ 0x76cd, 'CjkUnifiedIdeograph76cd', [], 'Display', None],
  [ 0x76ce, 'CjkUnifiedIdeograph76ce', [], 'Display', None],
  [ 0x76cf, 'CjkUnifiedIdeograph76cf', [], 'Display', None],
  [ 0x76d0, 'CjkUnifiedIdeograph76d0', [], 'Display', None],
  [ 0x76d1, 'CjkUnifiedIdeograph76d1', [], 'Display', None],
  [ 0x76d2, 'CjkUnifiedIdeograph76d2', [], 'Display', None],
  [ 0x76d3, 'CjkUnifiedIdeograph76d3', [], 'Display', None],
  [ 0x76d4, 'CjkUnifiedIdeograph76d4', [], 'Display', None],
  [ 0x76d5, 'CjkUnifiedIdeograph76d5', [], 'Display', None],
  [ 0x76d6, 'CjkUnifiedIdeograph76d6', [], 'Display', None],
  [ 0x76d7, 'CjkUnifiedIdeograph76d7', [], 'Display', None],
  [ 0x76d8, 'CjkUnifiedIdeograph76d8', [], 'Display', None],
  [ 0x76d9, 'CjkUnifiedIdeograph76d9', [], 'Display', None],
  [ 0x76da, 'CjkUnifiedIdeograph76da', [], 'Display', None],
  [ 0x76db, 'CjkUnifiedIdeograph76db', [], 'Display', None],
  [ 0x76dc, 'CjkUnifiedIdeograph76dc', [], 'Display', None],
  [ 0x76dd, 'CjkUnifiedIdeograph76dd', [], 'Display', None],
  [ 0x76de, 'CjkUnifiedIdeograph76de', [], 'Display', None],
  [ 0x76df, 'CjkUnifiedIdeograph76df', [], 'Display', None],
  [ 0x76e0, 'CjkUnifiedIdeograph76e0', [], 'Display', None],
  [ 0x76e1, 'CjkUnifiedIdeograph76e1', [], 'Display', None],
  [ 0x76e2, 'CjkUnifiedIdeograph76e2', [], 'Display', None],
  [ 0x76e3, 'CjkUnifiedIdeograph76e3', [], 'Display', None],
  [ 0x76e4, 'CjkUnifiedIdeograph76e4', [], 'Display', None],
  [ 0x76e5, 'CjkUnifiedIdeograph76e5', [], 'Display', None],
  [ 0x76e6, 'CjkUnifiedIdeograph76e6', [], 'Display', None],
  [ 0x76e7, 'CjkUnifiedIdeograph76e7', [], 'Display', None],
  [ 0x76e8, 'CjkUnifiedIdeograph76e8', [], 'Display', None],
  [ 0x76e9, 'CjkUnifiedIdeograph76e9', [], 'Display', None],
  [ 0x76ea, 'CjkUnifiedIdeograph76ea', [], 'Display', None],
  [ 0x76eb, 'CjkUnifiedIdeograph76eb', [], 'Display', None],
  [ 0x76ec, 'CjkUnifiedIdeograph76ec', [], 'Display', None],
  [ 0x76ed, 'CjkUnifiedIdeograph76ed', [], 'Display', None],
  [ 0x76ee, 'CjkUnifiedIdeograph76ee', [], 'Display', None],
  [ 0x76ef, 'CjkUnifiedIdeograph76ef', [], 'Display', None],
  [ 0x76f0, 'CjkUnifiedIdeograph76f0', [], 'Display', None],
  [ 0x76f1, 'CjkUnifiedIdeograph76f1', [], 'Display', None],
  [ 0x76f2, 'CjkUnifiedIdeograph76f2', [], 'Display', None],
  [ 0x76f3, 'CjkUnifiedIdeograph76f3', [], 'Display', None],
  [ 0x76f4, 'CjkUnifiedIdeograph76f4', [], 'Display', None],
  [ 0x76f5, 'CjkUnifiedIdeograph76f5', [], 'Display', None],
  [ 0x76f6, 'CjkUnifiedIdeograph76f6', [], 'Display', None],
  [ 0x76f7, 'CjkUnifiedIdeograph76f7', [], 'Display', None],
  [ 0x76f8, 'CjkUnifiedIdeograph76f8', [], 'Display', None],
  [ 0x76f9, 'CjkUnifiedIdeograph76f9', [], 'Display', None],
  [ 0x76fa, 'CjkUnifiedIdeograph76fa', [], 'Display', None],
  [ 0x76fb, 'CjkUnifiedIdeograph76fb', [], 'Display', None],
  [ 0x76fc, 'CjkUnifiedIdeograph76fc', [], 'Display', None],
  [ 0x76fd, 'CjkUnifiedIdeograph76fd', [], 'Display', None],
  [ 0x76fe, 'CjkUnifiedIdeograph76fe', [], 'Display', None],
  [ 0x76ff, 'CjkUnifiedIdeograph76ff', [], 'Display', None],
  [ 0x7700, 'CjkUnifiedIdeograph7700', [], 'Display', None],
  [ 0x7701, 'CjkUnifiedIdeograph7701', [], 'Display', None],
  [ 0x7702, 'CjkUnifiedIdeograph7702', [], 'Display', None],
  [ 0x7703, 'CjkUnifiedIdeograph7703', [], 'Display', None],
  [ 0x7704, 'CjkUnifiedIdeograph7704', [], 'Display', None],
  [ 0x7705, 'CjkUnifiedIdeograph7705', [], 'Display', None],
  [ 0x7706, 'CjkUnifiedIdeograph7706', [], 'Display', None],
  [ 0x7707, 'CjkUnifiedIdeograph7707', [], 'Display', None],
  [ 0x7708, 'CjkUnifiedIdeograph7708', [], 'Display', None],
  [ 0x7709, 'CjkUnifiedIdeograph7709', [], 'Display', None],
  [ 0x770a, 'CjkUnifiedIdeograph770a', [], 'Display', None],
  [ 0x770b, 'CjkUnifiedIdeograph770b', [], 'Display', None],
  [ 0x770c, 'CjkUnifiedIdeograph770c', [], 'Display', None],
  [ 0x770d, 'CjkUnifiedIdeograph770d', [], 'Display', None],
  [ 0x770e, 'CjkUnifiedIdeograph770e', [], 'Display', None],
  [ 0x770f, 'CjkUnifiedIdeograph770f', [], 'Display', None],
  [ 0x7710, 'CjkUnifiedIdeograph7710', [], 'Display', None],
  [ 0x7711, 'CjkUnifiedIdeograph7711', [], 'Display', None],
  [ 0x7712, 'CjkUnifiedIdeograph7712', [], 'Display', None],
  [ 0x7713, 'CjkUnifiedIdeograph7713', [], 'Display', None],
  [ 0x7714, 'CjkUnifiedIdeograph7714', [], 'Display', None],
  [ 0x7715, 'CjkUnifiedIdeograph7715', [], 'Display', None],
  [ 0x7716, 'CjkUnifiedIdeograph7716', [], 'Display', None],
  [ 0x7717, 'CjkUnifiedIdeograph7717', [], 'Display', None],
  [ 0x7718, 'CjkUnifiedIdeograph7718', [], 'Display', None],
  [ 0x7719, 'CjkUnifiedIdeograph7719', [], 'Display', None],
  [ 0x771a, 'CjkUnifiedIdeograph771a', [], 'Display', None],
  [ 0x771b, 'CjkUnifiedIdeograph771b', [], 'Display', None],
  [ 0x771c, 'CjkUnifiedIdeograph771c', [], 'Display', None],
  [ 0x771d, 'CjkUnifiedIdeograph771d', [], 'Display', None],
  [ 0x771e, 'CjkUnifiedIdeograph771e', [], 'Display', None],
  [ 0x771f, 'CjkUnifiedIdeograph771f', [], 'Display', None],
  [ 0x7720, 'CjkUnifiedIdeograph7720', [], 'Display', None],
  [ 0x7721, 'CjkUnifiedIdeograph7721', [], 'Display', None],
  [ 0x7722, 'CjkUnifiedIdeograph7722', [], 'Display', None],
  [ 0x7723, 'CjkUnifiedIdeograph7723', [], 'Display', None],
  [ 0x7724, 'CjkUnifiedIdeograph7724', [], 'Display', None],
  [ 0x7725, 'CjkUnifiedIdeograph7725', [], 'Display', None],
  [ 0x7726, 'CjkUnifiedIdeograph7726', [], 'Display', None],
  [ 0x7727, 'CjkUnifiedIdeograph7727', [], 'Display', None],
  [ 0x7728, 'CjkUnifiedIdeograph7728', [], 'Display', None],
  [ 0x7729, 'CjkUnifiedIdeograph7729', [], 'Display', None],
  [ 0x772a, 'CjkUnifiedIdeograph772a', [], 'Display', None],
  [ 0x772b, 'CjkUnifiedIdeograph772b', [], 'Display', None],
  [ 0x772c, 'CjkUnifiedIdeograph772c', [], 'Display', None],
  [ 0x772d, 'CjkUnifiedIdeograph772d', [], 'Display', None],
  [ 0x772e, 'CjkUnifiedIdeograph772e', [], 'Display', None],
  [ 0x772f, 'CjkUnifiedIdeograph772f', [], 'Display', None],
  [ 0x7730, 'CjkUnifiedIdeograph7730', [], 'Display', None],
  [ 0x7731, 'CjkUnifiedIdeograph7731', [], 'Display', None],
  [ 0x7732, 'CjkUnifiedIdeograph7732', [], 'Display', None],
  [ 0x7733, 'CjkUnifiedIdeograph7733', [], 'Display', None],
  [ 0x7734, 'CjkUnifiedIdeograph7734', [], 'Display', None],
  [ 0x7735, 'CjkUnifiedIdeograph7735', [], 'Display', None],
  [ 0x7736, 'CjkUnifiedIdeograph7736', [], 'Display', None],
  [ 0x7737, 'CjkUnifiedIdeograph7737', [], 'Display', None],
  [ 0x7738, 'CjkUnifiedIdeograph7738', [], 'Display', None],
  [ 0x7739, 'CjkUnifiedIdeograph7739', [], 'Display', None],
  [ 0x773a, 'CjkUnifiedIdeograph773a', [], 'Display', None],
  [ 0x773b, 'CjkUnifiedIdeograph773b', [], 'Display', None],
  [ 0x773c, 'CjkUnifiedIdeograph773c', [], 'Display', None],
  [ 0x773d, 'CjkUnifiedIdeograph773d', [], 'Display', None],
  [ 0x773e, 'CjkUnifiedIdeograph773e', [], 'Display', None],
  [ 0x773f, 'CjkUnifiedIdeograph773f', [], 'Display', None],
  [ 0x7740, 'CjkUnifiedIdeograph7740', [], 'Display', None],
  [ 0x7741, 'CjkUnifiedIdeograph7741', [], 'Display', None],
  [ 0x7742, 'CjkUnifiedIdeograph7742', [], 'Display', None],
  [ 0x7743, 'CjkUnifiedIdeograph7743', [], 'Display', None],
  [ 0x7744, 'CjkUnifiedIdeograph7744', [], 'Display', None],
  [ 0x7745, 'CjkUnifiedIdeograph7745', [], 'Display', None],
  [ 0x7746, 'CjkUnifiedIdeograph7746', [], 'Display', None],
  [ 0x7747, 'CjkUnifiedIdeograph7747', [], 'Display', None],
  [ 0x7748, 'CjkUnifiedIdeograph7748', [], 'Display', None],
  [ 0x7749, 'CjkUnifiedIdeograph7749', [], 'Display', None],
  [ 0x774a, 'CjkUnifiedIdeograph774a', [], 'Display', None],
  [ 0x774b, 'CjkUnifiedIdeograph774b', [], 'Display', None],
  [ 0x774c, 'CjkUnifiedIdeograph774c', [], 'Display', None],
  [ 0x774d, 'CjkUnifiedIdeograph774d', [], 'Display', None],
  [ 0x774e, 'CjkUnifiedIdeograph774e', [], 'Display', None],
  [ 0x774f, 'CjkUnifiedIdeograph774f', [], 'Display', None],
  [ 0x7750, 'CjkUnifiedIdeograph7750', [], 'Display', None],
  [ 0x7751, 'CjkUnifiedIdeograph7751', [], 'Display', None],
  [ 0x7752, 'CjkUnifiedIdeograph7752', [], 'Display', None],
  [ 0x7753, 'CjkUnifiedIdeograph7753', [], 'Display', None],
  [ 0x7754, 'CjkUnifiedIdeograph7754', [], 'Display', None],
  [ 0x7755, 'CjkUnifiedIdeograph7755', [], 'Display', None],
  [ 0x7756, 'CjkUnifiedIdeograph7756', [], 'Display', None],
  [ 0x7757, 'CjkUnifiedIdeograph7757', [], 'Display', None],
  [ 0x7758, 'CjkUnifiedIdeograph7758', [], 'Display', None],
  [ 0x7759, 'CjkUnifiedIdeograph7759', [], 'Display', None],
  [ 0x775a, 'CjkUnifiedIdeograph775a', [], 'Display', None],
  [ 0x775b, 'CjkUnifiedIdeograph775b', [], 'Display', None],
  [ 0x775c, 'CjkUnifiedIdeograph775c', [], 'Display', None],
  [ 0x775d, 'CjkUnifiedIdeograph775d', [], 'Display', None],
  [ 0x775e, 'CjkUnifiedIdeograph775e', [], 'Display', None],
  [ 0x775f, 'CjkUnifiedIdeograph775f', [], 'Display', None],
  [ 0x7760, 'CjkUnifiedIdeograph7760', [], 'Display', None],
  [ 0x7761, 'CjkUnifiedIdeograph7761', [], 'Display', None],
  [ 0x7762, 'CjkUnifiedIdeograph7762', [], 'Display', None],
  [ 0x7763, 'CjkUnifiedIdeograph7763', [], 'Display', None],
  [ 0x7764, 'CjkUnifiedIdeograph7764', [], 'Display', None],
  [ 0x7765, 'CjkUnifiedIdeograph7765', [], 'Display', None],
  [ 0x7766, 'CjkUnifiedIdeograph7766', [], 'Display', None],
  [ 0x7767, 'CjkUnifiedIdeograph7767', [], 'Display', None],
  [ 0x7768, 'CjkUnifiedIdeograph7768', [], 'Display', None],
  [ 0x7769, 'CjkUnifiedIdeograph7769', [], 'Display', None],
  [ 0x776a, 'CjkUnifiedIdeograph776a', [], 'Display', None],
  [ 0x776b, 'CjkUnifiedIdeograph776b', [], 'Display', None],
  [ 0x776c, 'CjkUnifiedIdeograph776c', [], 'Display', None],
  [ 0x776d, 'CjkUnifiedIdeograph776d', [], 'Display', None],
  [ 0x776e, 'CjkUnifiedIdeograph776e', [], 'Display', None],
  [ 0x776f, 'CjkUnifiedIdeograph776f', [], 'Display', None],
  [ 0x7770, 'CjkUnifiedIdeograph7770', [], 'Display', None],
  [ 0x7771, 'CjkUnifiedIdeograph7771', [], 'Display', None],
  [ 0x7772, 'CjkUnifiedIdeograph7772', [], 'Display', None],
  [ 0x7773, 'CjkUnifiedIdeograph7773', [], 'Display', None],
  [ 0x7774, 'CjkUnifiedIdeograph7774', [], 'Display', None],
  [ 0x7775, 'CjkUnifiedIdeograph7775', [], 'Display', None],
  [ 0x7776, 'CjkUnifiedIdeograph7776', [], 'Display', None],
  [ 0x7777, 'CjkUnifiedIdeograph7777', [], 'Display', None],
  [ 0x7778, 'CjkUnifiedIdeograph7778', [], 'Display', None],
  [ 0x7779, 'CjkUnifiedIdeograph7779', [], 'Display', None],
  [ 0x777a, 'CjkUnifiedIdeograph777a', [], 'Display', None],
  [ 0x777b, 'CjkUnifiedIdeograph777b', [], 'Display', None],
  [ 0x777c, 'CjkUnifiedIdeograph777c', [], 'Display', None],
  [ 0x777d, 'CjkUnifiedIdeograph777d', [], 'Display', None],
  [ 0x777e, 'CjkUnifiedIdeograph777e', [], 'Display', None],
  [ 0x777f, 'CjkUnifiedIdeograph777f', [], 'Display', None],
  [ 0x7780, 'CjkUnifiedIdeograph7780', [], 'Display', None],
  [ 0x7781, 'CjkUnifiedIdeograph7781', [], 'Display', None],
  [ 0x7782, 'CjkUnifiedIdeograph7782', [], 'Display', None],
  [ 0x7783, 'CjkUnifiedIdeograph7783', [], 'Display', None],
  [ 0x7784, 'CjkUnifiedIdeograph7784', [], 'Display', None],
  [ 0x7785, 'CjkUnifiedIdeograph7785', [], 'Display', None],
  [ 0x7786, 'CjkUnifiedIdeograph7786', [], 'Display', None],
  [ 0x7787, 'CjkUnifiedIdeograph7787', [], 'Display', None],
  [ 0x7788, 'CjkUnifiedIdeograph7788', [], 'Display', None],
  [ 0x7789, 'CjkUnifiedIdeograph7789', [], 'Display', None],
  [ 0x778a, 'CjkUnifiedIdeograph778a', [], 'Display', None],
  [ 0x778b, 'CjkUnifiedIdeograph778b', [], 'Display', None],
  [ 0x778c, 'CjkUnifiedIdeograph778c', [], 'Display', None],
  [ 0x778d, 'CjkUnifiedIdeograph778d', [], 'Display', None],
  [ 0x778e, 'CjkUnifiedIdeograph778e', [], 'Display', None],
  [ 0x778f, 'CjkUnifiedIdeograph778f', [], 'Display', None],
  [ 0x7790, 'CjkUnifiedIdeograph7790', [], 'Display', None],
  [ 0x7791, 'CjkUnifiedIdeograph7791', [], 'Display', None],
  [ 0x7792, 'CjkUnifiedIdeograph7792', [], 'Display', None],
  [ 0x7793, 'CjkUnifiedIdeograph7793', [], 'Display', None],
  [ 0x7794, 'CjkUnifiedIdeograph7794', [], 'Display', None],
  [ 0x7795, 'CjkUnifiedIdeograph7795', [], 'Display', None],
  [ 0x7796, 'CjkUnifiedIdeograph7796', [], 'Display', None],
  [ 0x7797, 'CjkUnifiedIdeograph7797', [], 'Display', None],
  [ 0x7798, 'CjkUnifiedIdeograph7798', [], 'Display', None],
  [ 0x7799, 'CjkUnifiedIdeograph7799', [], 'Display', None],
  [ 0x779a, 'CjkUnifiedIdeograph779a', [], 'Display', None],
  [ 0x779b, 'CjkUnifiedIdeograph779b', [], 'Display', None],
  [ 0x779c, 'CjkUnifiedIdeograph779c', [], 'Display', None],
  [ 0x779d, 'CjkUnifiedIdeograph779d', [], 'Display', None],
  [ 0x779e, 'CjkUnifiedIdeograph779e', [], 'Display', None],
  [ 0x779f, 'CjkUnifiedIdeograph779f', [], 'Display', None],
  [ 0x77a0, 'CjkUnifiedIdeograph77a0', [], 'Display', None],
  [ 0x77a1, 'CjkUnifiedIdeograph77a1', [], 'Display', None],
  [ 0x77a2, 'CjkUnifiedIdeograph77a2', [], 'Display', None],
  [ 0x77a3, 'CjkUnifiedIdeograph77a3', [], 'Display', None],
  [ 0x77a4, 'CjkUnifiedIdeograph77a4', [], 'Display', None],
  [ 0x77a5, 'CjkUnifiedIdeograph77a5', [], 'Display', None],
  [ 0x77a6, 'CjkUnifiedIdeograph77a6', [], 'Display', None],
  [ 0x77a7, 'CjkUnifiedIdeograph77a7', [], 'Display', None],
  [ 0x77a8, 'CjkUnifiedIdeograph77a8', [], 'Display', None],
  [ 0x77a9, 'CjkUnifiedIdeograph77a9', [], 'Display', None],
  [ 0x77aa, 'CjkUnifiedIdeograph77aa', [], 'Display', None],
  [ 0x77ab, 'CjkUnifiedIdeograph77ab', [], 'Display', None],
  [ 0x77ac, 'CjkUnifiedIdeograph77ac', [], 'Display', None],
  [ 0x77ad, 'CjkUnifiedIdeograph77ad', [], 'Display', None],
  [ 0x77ae, 'CjkUnifiedIdeograph77ae', [], 'Display', None],
  [ 0x77af, 'CjkUnifiedIdeograph77af', [], 'Display', None],
  [ 0x77b0, 'CjkUnifiedIdeograph77b0', [], 'Display', None],
  [ 0x77b1, 'CjkUnifiedIdeograph77b1', [], 'Display', None],
  [ 0x77b2, 'CjkUnifiedIdeograph77b2', [], 'Display', None],
  [ 0x77b3, 'CjkUnifiedIdeograph77b3', [], 'Display', None],
  [ 0x77b4, 'CjkUnifiedIdeograph77b4', [], 'Display', None],
  [ 0x77b5, 'CjkUnifiedIdeograph77b5', [], 'Display', None],
  [ 0x77b6, 'CjkUnifiedIdeograph77b6', [], 'Display', None],
  [ 0x77b7, 'CjkUnifiedIdeograph77b7', [], 'Display', None],
  [ 0x77b8, 'CjkUnifiedIdeograph77b8', [], 'Display', None],
  [ 0x77b9, 'CjkUnifiedIdeograph77b9', [], 'Display', None],
  [ 0x77ba, 'CjkUnifiedIdeograph77ba', [], 'Display', None],
  [ 0x77bb, 'CjkUnifiedIdeograph77bb', [], 'Display', None],
  [ 0x77bc, 'CjkUnifiedIdeograph77bc', [], 'Display', None],
  [ 0x77bd, 'CjkUnifiedIdeograph77bd', [], 'Display', None],
  [ 0x77be, 'CjkUnifiedIdeograph77be', [], 'Display', None],
  [ 0x77bf, 'CjkUnifiedIdeograph77bf', [], 'Display', None],
  [ 0x77c0, 'CjkUnifiedIdeograph77c0', [], 'Display', None],
  [ 0x77c1, 'CjkUnifiedIdeograph77c1', [], 'Display', None],
  [ 0x77c2, 'CjkUnifiedIdeograph77c2', [], 'Display', None],
  [ 0x77c3, 'CjkUnifiedIdeograph77c3', [], 'Display', None],
  [ 0x77c4, 'CjkUnifiedIdeograph77c4', [], 'Display', None],
  [ 0x77c5, 'CjkUnifiedIdeograph77c5', [], 'Display', None],
  [ 0x77c6, 'CjkUnifiedIdeograph77c6', [], 'Display', None],
  [ 0x77c7, 'CjkUnifiedIdeograph77c7', [], 'Display', None],
  [ 0x77c8, 'CjkUnifiedIdeograph77c8', [], 'Display', None],
  [ 0x77c9, 'CjkUnifiedIdeograph77c9', [], 'Display', None],
  [ 0x77ca, 'CjkUnifiedIdeograph77ca', [], 'Display', None],
  [ 0x77cb, 'CjkUnifiedIdeograph77cb', [], 'Display', None],
  [ 0x77cc, 'CjkUnifiedIdeograph77cc', [], 'Display', None],
  [ 0x77cd, 'CjkUnifiedIdeograph77cd', [], 'Display', None],
  [ 0x77ce, 'CjkUnifiedIdeograph77ce', [], 'Display', None],
  [ 0x77cf, 'CjkUnifiedIdeograph77cf', [], 'Display', None],
  [ 0x77d0, 'CjkUnifiedIdeograph77d0', [], 'Display', None],
  [ 0x77d1, 'CjkUnifiedIdeograph77d1', [], 'Display', None],
  [ 0x77d2, 'CjkUnifiedIdeograph77d2', [], 'Display', None],
  [ 0x77d3, 'CjkUnifiedIdeograph77d3', [], 'Display', None],
  [ 0x77d4, 'CjkUnifiedIdeograph77d4', [], 'Display', None],
  [ 0x77d5, 'CjkUnifiedIdeograph77d5', [], 'Display', None],
  [ 0x77d6, 'CjkUnifiedIdeograph77d6', [], 'Display', None],
  [ 0x77d7, 'CjkUnifiedIdeograph77d7', [], 'Display', None],
  [ 0x77d8, 'CjkUnifiedIdeograph77d8', [], 'Display', None],
  [ 0x77d9, 'CjkUnifiedIdeograph77d9', [], 'Display', None],
  [ 0x77da, 'CjkUnifiedIdeograph77da', [], 'Display', None],
  [ 0x77db, 'CjkUnifiedIdeograph77db', [], 'Display', None],
  [ 0x77dc, 'CjkUnifiedIdeograph77dc', [], 'Display', None],
  [ 0x77dd, 'CjkUnifiedIdeograph77dd', [], 'Display', None],
  [ 0x77de, 'CjkUnifiedIdeograph77de', [], 'Display', None],
  [ 0x77df, 'CjkUnifiedIdeograph77df', [], 'Display', None],
  [ 0x77e0, 'CjkUnifiedIdeograph77e0', [], 'Display', None],
  [ 0x77e1, 'CjkUnifiedIdeograph77e1', [], 'Display', None],
  [ 0x77e2, 'CjkUnifiedIdeograph77e2', [], 'Display', None],
  [ 0x77e3, 'CjkUnifiedIdeograph77e3', [], 'Display', None],
  [ 0x77e4, 'CjkUnifiedIdeograph77e4', [], 'Display', None],
  [ 0x77e5, 'CjkUnifiedIdeograph77e5', [], 'Display', None],
  [ 0x77e6, 'CjkUnifiedIdeograph77e6', [], 'Display', None],
  [ 0x77e7, 'CjkUnifiedIdeograph77e7', [], 'Display', None],
  [ 0x77e8, 'CjkUnifiedIdeograph77e8', [], 'Display', None],
  [ 0x77e9, 'CjkUnifiedIdeograph77e9', [], 'Display', None],
  [ 0x77ea, 'CjkUnifiedIdeograph77ea', [], 'Display', None],
  [ 0x77eb, 'CjkUnifiedIdeograph77eb', [], 'Display', None],
  [ 0x77ec, 'CjkUnifiedIdeograph77ec', [], 'Display', None],
  [ 0x77ed, 'CjkUnifiedIdeograph77ed', [], 'Display', None],
  [ 0x77ee, 'CjkUnifiedIdeograph77ee', [], 'Display', None],
  [ 0x77ef, 'CjkUnifiedIdeograph77ef', [], 'Display', None],
  [ 0x77f0, 'CjkUnifiedIdeograph77f0', [], 'Display', None],
  [ 0x77f1, 'CjkUnifiedIdeograph77f1', [], 'Display', None],
  [ 0x77f2, 'CjkUnifiedIdeograph77f2', [], 'Display', None],
  [ 0x77f3, 'CjkUnifiedIdeograph77f3', [], 'Display', None],
  [ 0x77f4, 'CjkUnifiedIdeograph77f4', [], 'Display', None],
  [ 0x77f5, 'CjkUnifiedIdeograph77f5', [], 'Display', None],
  [ 0x77f6, 'CjkUnifiedIdeograph77f6', [], 'Display', None],
  [ 0x77f7, 'CjkUnifiedIdeograph77f7', [], 'Display', None],
  [ 0x77f8, 'CjkUnifiedIdeograph77f8', [], 'Display', None],
  [ 0x77f9, 'CjkUnifiedIdeograph77f9', [], 'Display', None],
  [ 0x77fa, 'CjkUnifiedIdeograph77fa', [], 'Display', None],
  [ 0x77fb, 'CjkUnifiedIdeograph77fb', [], 'Display', None],
  [ 0x77fc, 'CjkUnifiedIdeograph77fc', [], 'Display', None],
  [ 0x77fd, 'CjkUnifiedIdeograph77fd', [], 'Display', None],
  [ 0x77fe, 'CjkUnifiedIdeograph77fe', [], 'Display', None],
  [ 0x77ff, 'CjkUnifiedIdeograph77ff', [], 'Display', None],
  [ 0x7800, 'CjkUnifiedIdeograph7800', [], 'Display', None],
  [ 0x7801, 'CjkUnifiedIdeograph7801', [], 'Display', None],
  [ 0x7802, 'CjkUnifiedIdeograph7802', [], 'Display', None],
  [ 0x7803, 'CjkUnifiedIdeograph7803', [], 'Display', None],
  [ 0x7804, 'CjkUnifiedIdeograph7804', [], 'Display', None],
  [ 0x7805, 'CjkUnifiedIdeograph7805', [], 'Display', None],
  [ 0x7806, 'CjkUnifiedIdeograph7806', [], 'Display', None],
  [ 0x7807, 'CjkUnifiedIdeograph7807', [], 'Display', None],
  [ 0x7808, 'CjkUnifiedIdeograph7808', [], 'Display', None],
  [ 0x7809, 'CjkUnifiedIdeograph7809', [], 'Display', None],
  [ 0x780a, 'CjkUnifiedIdeograph780a', [], 'Display', None],
  [ 0x780b, 'CjkUnifiedIdeograph780b', [], 'Display', None],
  [ 0x780c, 'CjkUnifiedIdeograph780c', [], 'Display', None],
  [ 0x780d, 'CjkUnifiedIdeograph780d', [], 'Display', None],
  [ 0x780e, 'CjkUnifiedIdeograph780e', [], 'Display', None],
  [ 0x780f, 'CjkUnifiedIdeograph780f', [], 'Display', None],
  [ 0x7810, 'CjkUnifiedIdeograph7810', [], 'Display', None],
  [ 0x7811, 'CjkUnifiedIdeograph7811', [], 'Display', None],
  [ 0x7812, 'CjkUnifiedIdeograph7812', [], 'Display', None],
  [ 0x7813, 'CjkUnifiedIdeograph7813', [], 'Display', None],
  [ 0x7814, 'CjkUnifiedIdeograph7814', [], 'Display', None],
  [ 0x7815, 'CjkUnifiedIdeograph7815', [], 'Display', None],
  [ 0x7816, 'CjkUnifiedIdeograph7816', [], 'Display', None],
  [ 0x7817, 'CjkUnifiedIdeograph7817', [], 'Display', None],
  [ 0x7818, 'CjkUnifiedIdeograph7818', [], 'Display', None],
  [ 0x7819, 'CjkUnifiedIdeograph7819', [], 'Display', None],
  [ 0x781a, 'CjkUnifiedIdeograph781a', [], 'Display', None],
  [ 0x781b, 'CjkUnifiedIdeograph781b', [], 'Display', None],
  [ 0x781c, 'CjkUnifiedIdeograph781c', [], 'Display', None],
  [ 0x781d, 'CjkUnifiedIdeograph781d', [], 'Display', None],
  [ 0x781e, 'CjkUnifiedIdeograph781e', [], 'Display', None],
  [ 0x781f, 'CjkUnifiedIdeograph781f', [], 'Display', None],
  [ 0x7820, 'CjkUnifiedIdeograph7820', [], 'Display', None],
  [ 0x7821, 'CjkUnifiedIdeograph7821', [], 'Display', None],
  [ 0x7822, 'CjkUnifiedIdeograph7822', [], 'Display', None],
  [ 0x7823, 'CjkUnifiedIdeograph7823', [], 'Display', None],
  [ 0x7824, 'CjkUnifiedIdeograph7824', [], 'Display', None],
  [ 0x7825, 'CjkUnifiedIdeograph7825', [], 'Display', None],
  [ 0x7826, 'CjkUnifiedIdeograph7826', [], 'Display', None],
  [ 0x7827, 'CjkUnifiedIdeograph7827', [], 'Display', None],
  [ 0x7828, 'CjkUnifiedIdeograph7828', [], 'Display', None],
  [ 0x7829, 'CjkUnifiedIdeograph7829', [], 'Display', None],
  [ 0x782a, 'CjkUnifiedIdeograph782a', [], 'Display', None],
  [ 0x782b, 'CjkUnifiedIdeograph782b', [], 'Display', None],
  [ 0x782c, 'CjkUnifiedIdeograph782c', [], 'Display', None],
  [ 0x782d, 'CjkUnifiedIdeograph782d', [], 'Display', None],
  [ 0x782e, 'CjkUnifiedIdeograph782e', [], 'Display', None],
  [ 0x782f, 'CjkUnifiedIdeograph782f', [], 'Display', None],
  [ 0x7830, 'CjkUnifiedIdeograph7830', [], 'Display', None],
  [ 0x7831, 'CjkUnifiedIdeograph7831', [], 'Display', None],
  [ 0x7832, 'CjkUnifiedIdeograph7832', [], 'Display', None],
  [ 0x7833, 'CjkUnifiedIdeograph7833', [], 'Display', None],
  [ 0x7834, 'CjkUnifiedIdeograph7834', [], 'Display', None],
  [ 0x7835, 'CjkUnifiedIdeograph7835', [], 'Display', None],
  [ 0x7836, 'CjkUnifiedIdeograph7836', [], 'Display', None],
  [ 0x7837, 'CjkUnifiedIdeograph7837', [], 'Display', None],
  [ 0x7838, 'CjkUnifiedIdeograph7838', [], 'Display', None],
  [ 0x7839, 'CjkUnifiedIdeograph7839', [], 'Display', None],
  [ 0x783a, 'CjkUnifiedIdeograph783a', [], 'Display', None],
  [ 0x783b, 'CjkUnifiedIdeograph783b', [], 'Display', None],
  [ 0x783c, 'CjkUnifiedIdeograph783c', [], 'Display', None],
  [ 0x783d, 'CjkUnifiedIdeograph783d', [], 'Display', None],
  [ 0x783e, 'CjkUnifiedIdeograph783e', [], 'Display', None],
  [ 0x783f, 'CjkUnifiedIdeograph783f', [], 'Display', None],
  [ 0x7840, 'CjkUnifiedIdeograph7840', [], 'Display', None],
  [ 0x7841, 'CjkUnifiedIdeograph7841', [], 'Display', None],
  [ 0x7842, 'CjkUnifiedIdeograph7842', [], 'Display', None],
  [ 0x7843, 'CjkUnifiedIdeograph7843', [], 'Display', None],
  [ 0x7844, 'CjkUnifiedIdeograph7844', [], 'Display', None],
  [ 0x7845, 'CjkUnifiedIdeograph7845', [], 'Display', None],
  [ 0x7846, 'CjkUnifiedIdeograph7846', [], 'Display', None],
  [ 0x7847, 'CjkUnifiedIdeograph7847', [], 'Display', None],
  [ 0x7848, 'CjkUnifiedIdeograph7848', [], 'Display', None],
  [ 0x7849, 'CjkUnifiedIdeograph7849', [], 'Display', None],
  [ 0x784a, 'CjkUnifiedIdeograph784a', [], 'Display', None],
  [ 0x784b, 'CjkUnifiedIdeograph784b', [], 'Display', None],
  [ 0x784c, 'CjkUnifiedIdeograph784c', [], 'Display', None],
  [ 0x784d, 'CjkUnifiedIdeograph784d', [], 'Display', None],
  [ 0x784e, 'CjkUnifiedIdeograph784e', [], 'Display', None],
  [ 0x784f, 'CjkUnifiedIdeograph784f', [], 'Display', None],
  [ 0x7850, 'CjkUnifiedIdeograph7850', [], 'Display', None],
  [ 0x7851, 'CjkUnifiedIdeograph7851', [], 'Display', None],
  [ 0x7852, 'CjkUnifiedIdeograph7852', [], 'Display', None],
  [ 0x7853, 'CjkUnifiedIdeograph7853', [], 'Display', None],
  [ 0x7854, 'CjkUnifiedIdeograph7854', [], 'Display', None],
  [ 0x7855, 'CjkUnifiedIdeograph7855', [], 'Display', None],
  [ 0x7856, 'CjkUnifiedIdeograph7856', [], 'Display', None],
  [ 0x7857, 'CjkUnifiedIdeograph7857', [], 'Display', None],
  [ 0x7858, 'CjkUnifiedIdeograph7858', [], 'Display', None],
  [ 0x7859, 'CjkUnifiedIdeograph7859', [], 'Display', None],
  [ 0x785a, 'CjkUnifiedIdeograph785a', [], 'Display', None],
  [ 0x785b, 'CjkUnifiedIdeograph785b', [], 'Display', None],
  [ 0x785c, 'CjkUnifiedIdeograph785c', [], 'Display', None],
  [ 0x785d, 'CjkUnifiedIdeograph785d', [], 'Display', None],
  [ 0x785e, 'CjkUnifiedIdeograph785e', [], 'Display', None],
  [ 0x785f, 'CjkUnifiedIdeograph785f', [], 'Display', None],
  [ 0x7860, 'CjkUnifiedIdeograph7860', [], 'Display', None],
  [ 0x7861, 'CjkUnifiedIdeograph7861', [], 'Display', None],
  [ 0x7862, 'CjkUnifiedIdeograph7862', [], 'Display', None],
  [ 0x7863, 'CjkUnifiedIdeograph7863', [], 'Display', None],
  [ 0x7864, 'CjkUnifiedIdeograph7864', [], 'Display', None],
  [ 0x7865, 'CjkUnifiedIdeograph7865', [], 'Display', None],
  [ 0x7866, 'CjkUnifiedIdeograph7866', [], 'Display', None],
  [ 0x7867, 'CjkUnifiedIdeograph7867', [], 'Display', None],
  [ 0x7868, 'CjkUnifiedIdeograph7868', [], 'Display', None],
  [ 0x7869, 'CjkUnifiedIdeograph7869', [], 'Display', None],
  [ 0x786a, 'CjkUnifiedIdeograph786a', [], 'Display', None],
  [ 0x786b, 'CjkUnifiedIdeograph786b', [], 'Display', None],
  [ 0x786c, 'CjkUnifiedIdeograph786c', [], 'Display', None],
  [ 0x786d, 'CjkUnifiedIdeograph786d', [], 'Display', None],
  [ 0x786e, 'CjkUnifiedIdeograph786e', [], 'Display', None],
  [ 0x786f, 'CjkUnifiedIdeograph786f', [], 'Display', None],
  [ 0x7870, 'CjkUnifiedIdeograph7870', [], 'Display', None],
  [ 0x7871, 'CjkUnifiedIdeograph7871', [], 'Display', None],
  [ 0x7872, 'CjkUnifiedIdeograph7872', [], 'Display', None],
  [ 0x7873, 'CjkUnifiedIdeograph7873', [], 'Display', None],
  [ 0x7874, 'CjkUnifiedIdeograph7874', [], 'Display', None],
  [ 0x7875, 'CjkUnifiedIdeograph7875', [], 'Display', None],
  [ 0x7876, 'CjkUnifiedIdeograph7876', [], 'Display', None],
  [ 0x7877, 'CjkUnifiedIdeograph7877', [], 'Display', None],
  [ 0x7878, 'CjkUnifiedIdeograph7878', [], 'Display', None],
  [ 0x7879, 'CjkUnifiedIdeograph7879', [], 'Display', None],
  [ 0x787a, 'CjkUnifiedIdeograph787a', [], 'Display', None],
  [ 0x787b, 'CjkUnifiedIdeograph787b', [], 'Display', None],
  [ 0x787c, 'CjkUnifiedIdeograph787c', [], 'Display', None],
  [ 0x787d, 'CjkUnifiedIdeograph787d', [], 'Display', None],
  [ 0x787e, 'CjkUnifiedIdeograph787e', [], 'Display', None],
  [ 0x787f, 'CjkUnifiedIdeograph787f', [], 'Display', None],
  [ 0x7880, 'CjkUnifiedIdeograph7880', [], 'Display', None],
  [ 0x7881, 'CjkUnifiedIdeograph7881', [], 'Display', None],
  [ 0x7882, 'CjkUnifiedIdeograph7882', [], 'Display', None],
  [ 0x7883, 'CjkUnifiedIdeograph7883', [], 'Display', None],
  [ 0x7884, 'CjkUnifiedIdeograph7884', [], 'Display', None],
  [ 0x7885, 'CjkUnifiedIdeograph7885', [], 'Display', None],
  [ 0x7886, 'CjkUnifiedIdeograph7886', [], 'Display', None],
  [ 0x7887, 'CjkUnifiedIdeograph7887', [], 'Display', None],
  [ 0x7888, 'CjkUnifiedIdeograph7888', [], 'Display', None],
  [ 0x7889, 'CjkUnifiedIdeograph7889', [], 'Display', None],
  [ 0x788a, 'CjkUnifiedIdeograph788a', [], 'Display', None],
  [ 0x788b, 'CjkUnifiedIdeograph788b', [], 'Display', None],
  [ 0x788c, 'CjkUnifiedIdeograph788c', [], 'Display', None],
  [ 0x788d, 'CjkUnifiedIdeograph788d', [], 'Display', None],
  [ 0x788e, 'CjkUnifiedIdeograph788e', [], 'Display', None],
  [ 0x788f, 'CjkUnifiedIdeograph788f', [], 'Display', None],
  [ 0x7890, 'CjkUnifiedIdeograph7890', [], 'Display', None],
  [ 0x7891, 'CjkUnifiedIdeograph7891', [], 'Display', None],
  [ 0x7892, 'CjkUnifiedIdeograph7892', [], 'Display', None],
  [ 0x7893, 'CjkUnifiedIdeograph7893', [], 'Display', None],
  [ 0x7894, 'CjkUnifiedIdeograph7894', [], 'Display', None],
  [ 0x7895, 'CjkUnifiedIdeograph7895', [], 'Display', None],
  [ 0x7896, 'CjkUnifiedIdeograph7896', [], 'Display', None],
  [ 0x7897, 'CjkUnifiedIdeograph7897', [], 'Display', None],
  [ 0x7898, 'CjkUnifiedIdeograph7898', [], 'Display', None],
  [ 0x7899, 'CjkUnifiedIdeograph7899', [], 'Display', None],
  [ 0x789a, 'CjkUnifiedIdeograph789a', [], 'Display', None],
  [ 0x789b, 'CjkUnifiedIdeograph789b', [], 'Display', None],
  [ 0x789c, 'CjkUnifiedIdeograph789c', [], 'Display', None],
  [ 0x789d, 'CjkUnifiedIdeograph789d', [], 'Display', None],
  [ 0x789e, 'CjkUnifiedIdeograph789e', [], 'Display', None],
  [ 0x789f, 'CjkUnifiedIdeograph789f', [], 'Display', None],
  [ 0x78a0, 'CjkUnifiedIdeograph78a0', [], 'Display', None],
  [ 0x78a1, 'CjkUnifiedIdeograph78a1', [], 'Display', None],
  [ 0x78a2, 'CjkUnifiedIdeograph78a2', [], 'Display', None],
  [ 0x78a3, 'CjkUnifiedIdeograph78a3', [], 'Display', None],
  [ 0x78a4, 'CjkUnifiedIdeograph78a4', [], 'Display', None],
  [ 0x78a5, 'CjkUnifiedIdeograph78a5', [], 'Display', None],
  [ 0x78a6, 'CjkUnifiedIdeograph78a6', [], 'Display', None],
  [ 0x78a7, 'CjkUnifiedIdeograph78a7', [], 'Display', None],
  [ 0x78a8, 'CjkUnifiedIdeograph78a8', [], 'Display', None],
  [ 0x78a9, 'CjkUnifiedIdeograph78a9', [], 'Display', None],
  [ 0x78aa, 'CjkUnifiedIdeograph78aa', [], 'Display', None],
  [ 0x78ab, 'CjkUnifiedIdeograph78ab', [], 'Display', None],
  [ 0x78ac, 'CjkUnifiedIdeograph78ac', [], 'Display', None],
  [ 0x78ad, 'CjkUnifiedIdeograph78ad', [], 'Display', None],
  [ 0x78ae, 'CjkUnifiedIdeograph78ae', [], 'Display', None],
  [ 0x78af, 'CjkUnifiedIdeograph78af', [], 'Display', None],
  [ 0x78b0, 'CjkUnifiedIdeograph78b0', [], 'Display', None],
  [ 0x78b1, 'CjkUnifiedIdeograph78b1', [], 'Display', None],
  [ 0x78b2, 'CjkUnifiedIdeograph78b2', [], 'Display', None],
  [ 0x78b3, 'CjkUnifiedIdeograph78b3', [], 'Display', None],
  [ 0x78b4, 'CjkUnifiedIdeograph78b4', [], 'Display', None],
  [ 0x78b5, 'CjkUnifiedIdeograph78b5', [], 'Display', None],
  [ 0x78b6, 'CjkUnifiedIdeograph78b6', [], 'Display', None],
  [ 0x78b7, 'CjkUnifiedIdeograph78b7', [], 'Display', None],
  [ 0x78b8, 'CjkUnifiedIdeograph78b8', [], 'Display', None],
  [ 0x78b9, 'CjkUnifiedIdeograph78b9', [], 'Display', None],
  [ 0x78ba, 'CjkUnifiedIdeograph78ba', [], 'Display', None],
  [ 0x78bb, 'CjkUnifiedIdeograph78bb', [], 'Display', None],
  [ 0x78bc, 'CjkUnifiedIdeograph78bc', [], 'Display', None],
  [ 0x78bd, 'CjkUnifiedIdeograph78bd', [], 'Display', None],
  [ 0x78be, 'CjkUnifiedIdeograph78be', [], 'Display', None],
  [ 0x78bf, 'CjkUnifiedIdeograph78bf', [], 'Display', None],
  [ 0x78c0, 'CjkUnifiedIdeograph78c0', [], 'Display', None],
  [ 0x78c1, 'CjkUnifiedIdeograph78c1', [], 'Display', None],
  [ 0x78c2, 'CjkUnifiedIdeograph78c2', [], 'Display', None],
  [ 0x78c3, 'CjkUnifiedIdeograph78c3', [], 'Display', None],
  [ 0x78c4, 'CjkUnifiedIdeograph78c4', [], 'Display', None],
  [ 0x78c5, 'CjkUnifiedIdeograph78c5', [], 'Display', None],
  [ 0x78c6, 'CjkUnifiedIdeograph78c6', [], 'Display', None],
  [ 0x78c7, 'CjkUnifiedIdeograph78c7', [], 'Display', None],
  [ 0x78c8, 'CjkUnifiedIdeograph78c8', [], 'Display', None],
  [ 0x78c9, 'CjkUnifiedIdeograph78c9', [], 'Display', None],
  [ 0x78ca, 'CjkUnifiedIdeograph78ca', [], 'Display', None],
  [ 0x78cb, 'CjkUnifiedIdeograph78cb', [], 'Display', None],
  [ 0x78cc, 'CjkUnifiedIdeograph78cc', [], 'Display', None],
  [ 0x78cd, 'CjkUnifiedIdeograph78cd', [], 'Display', None],
  [ 0x78ce, 'CjkUnifiedIdeograph78ce', [], 'Display', None],
  [ 0x78cf, 'CjkUnifiedIdeograph78cf', [], 'Display', None],
  [ 0x78d0, 'CjkUnifiedIdeograph78d0', [], 'Display', None],
  [ 0x78d1, 'CjkUnifiedIdeograph78d1', [], 'Display', None],
  [ 0x78d2, 'CjkUnifiedIdeograph78d2', [], 'Display', None],
  [ 0x78d3, 'CjkUnifiedIdeograph78d3', [], 'Display', None],
  [ 0x78d4, 'CjkUnifiedIdeograph78d4', [], 'Display', None],
  [ 0x78d5, 'CjkUnifiedIdeograph78d5', [], 'Display', None],
  [ 0x78d6, 'CjkUnifiedIdeograph78d6', [], 'Display', None],
  [ 0x78d7, 'CjkUnifiedIdeograph78d7', [], 'Display', None],
  [ 0x78d8, 'CjkUnifiedIdeograph78d8', [], 'Display', None],
  [ 0x78d9, 'CjkUnifiedIdeograph78d9', [], 'Display', None],
  [ 0x78da, 'CjkUnifiedIdeograph78da', [], 'Display', None],
  [ 0x78db, 'CjkUnifiedIdeograph78db', [], 'Display', None],
  [ 0x78dc, 'CjkUnifiedIdeograph78dc', [], 'Display', None],
  [ 0x78dd, 'CjkUnifiedIdeograph78dd', [], 'Display', None],
  [ 0x78de, 'CjkUnifiedIdeograph78de', [], 'Display', None],
  [ 0x78df, 'CjkUnifiedIdeograph78df', [], 'Display', None],
  [ 0x78e0, 'CjkUnifiedIdeograph78e0', [], 'Display', None],
  [ 0x78e1, 'CjkUnifiedIdeograph78e1', [], 'Display', None],
  [ 0x78e2, 'CjkUnifiedIdeograph78e2', [], 'Display', None],
  [ 0x78e3, 'CjkUnifiedIdeograph78e3', [], 'Display', None],
  [ 0x78e4, 'CjkUnifiedIdeograph78e4', [], 'Display', None],
  [ 0x78e5, 'CjkUnifiedIdeograph78e5', [], 'Display', None],
  [ 0x78e6, 'CjkUnifiedIdeograph78e6', [], 'Display', None],
  [ 0x78e7, 'CjkUnifiedIdeograph78e7', [], 'Display', None],
  [ 0x78e8, 'CjkUnifiedIdeograph78e8', [], 'Display', None],
  [ 0x78e9, 'CjkUnifiedIdeograph78e9', [], 'Display', None],
  [ 0x78ea, 'CjkUnifiedIdeograph78ea', [], 'Display', None],
  [ 0x78eb, 'CjkUnifiedIdeograph78eb', [], 'Display', None],
  [ 0x78ec, 'CjkUnifiedIdeograph78ec', [], 'Display', None],
  [ 0x78ed, 'CjkUnifiedIdeograph78ed', [], 'Display', None],
  [ 0x78ee, 'CjkUnifiedIdeograph78ee', [], 'Display', None],
  [ 0x78ef, 'CjkUnifiedIdeograph78ef', [], 'Display', None],
  [ 0x78f0, 'CjkUnifiedIdeograph78f0', [], 'Display', None],
  [ 0x78f1, 'CjkUnifiedIdeograph78f1', [], 'Display', None],
  [ 0x78f2, 'CjkUnifiedIdeograph78f2', [], 'Display', None],
  [ 0x78f3, 'CjkUnifiedIdeograph78f3', [], 'Display', None],
  [ 0x78f4, 'CjkUnifiedIdeograph78f4', [], 'Display', None],
  [ 0x78f5, 'CjkUnifiedIdeograph78f5', [], 'Display', None],
  [ 0x78f6, 'CjkUnifiedIdeograph78f6', [], 'Display', None],
  [ 0x78f7, 'CjkUnifiedIdeograph78f7', [], 'Display', None],
  [ 0x78f8, 'CjkUnifiedIdeograph78f8', [], 'Display', None],
  [ 0x78f9, 'CjkUnifiedIdeograph78f9', [], 'Display', None],
  [ 0x78fa, 'CjkUnifiedIdeograph78fa', [], 'Display', None],
  [ 0x78fb, 'CjkUnifiedIdeograph78fb', [], 'Display', None],
  [ 0x78fc, 'CjkUnifiedIdeograph78fc', [], 'Display', None],
  [ 0x78fd, 'CjkUnifiedIdeograph78fd', [], 'Display', None],
  [ 0x78fe, 'CjkUnifiedIdeograph78fe', [], 'Display', None],
  [ 0x78ff, 'CjkUnifiedIdeograph78ff', [], 'Display', None],
  [ 0x7900, 'CjkUnifiedIdeograph7900', [], 'Display', None],
  [ 0x7901, 'CjkUnifiedIdeograph7901', [], 'Display', None],
  [ 0x7902, 'CjkUnifiedIdeograph7902', [], 'Display', None],
  [ 0x7903, 'CjkUnifiedIdeograph7903', [], 'Display', None],
  [ 0x7904, 'CjkUnifiedIdeograph7904', [], 'Display', None],
  [ 0x7905, 'CjkUnifiedIdeograph7905', [], 'Display', None],
  [ 0x7906, 'CjkUnifiedIdeograph7906', [], 'Display', None],
  [ 0x7907, 'CjkUnifiedIdeograph7907', [], 'Display', None],
  [ 0x7908, 'CjkUnifiedIdeograph7908', [], 'Display', None],
  [ 0x7909, 'CjkUnifiedIdeograph7909', [], 'Display', None],
  [ 0x790a, 'CjkUnifiedIdeograph790a', [], 'Display', None],
  [ 0x790b, 'CjkUnifiedIdeograph790b', [], 'Display', None],
  [ 0x790c, 'CjkUnifiedIdeograph790c', [], 'Display', None],
  [ 0x790d, 'CjkUnifiedIdeograph790d', [], 'Display', None],
  [ 0x790e, 'CjkUnifiedIdeograph790e', [], 'Display', None],
  [ 0x790f, 'CjkUnifiedIdeograph790f', [], 'Display', None],
  [ 0x7910, 'CjkUnifiedIdeograph7910', [], 'Display', None],
  [ 0x7911, 'CjkUnifiedIdeograph7911', [], 'Display', None],
  [ 0x7912, 'CjkUnifiedIdeograph7912', [], 'Display', None],
  [ 0x7913, 'CjkUnifiedIdeograph7913', [], 'Display', None],
  [ 0x7914, 'CjkUnifiedIdeograph7914', [], 'Display', None],
  [ 0x7915, 'CjkUnifiedIdeograph7915', [], 'Display', None],
  [ 0x7916, 'CjkUnifiedIdeograph7916', [], 'Display', None],
  [ 0x7917, 'CjkUnifiedIdeograph7917', [], 'Display', None],
  [ 0x7918, 'CjkUnifiedIdeograph7918', [], 'Display', None],
  [ 0x7919, 'CjkUnifiedIdeograph7919', [], 'Display', None],
  [ 0x791a, 'CjkUnifiedIdeograph791a', [], 'Display', None],
  [ 0x791b, 'CjkUnifiedIdeograph791b', [], 'Display', None],
  [ 0x791c, 'CjkUnifiedIdeograph791c', [], 'Display', None],
  [ 0x791d, 'CjkUnifiedIdeograph791d', [], 'Display', None],
  [ 0x791e, 'CjkUnifiedIdeograph791e', [], 'Display', None],
  [ 0x791f, 'CjkUnifiedIdeograph791f', [], 'Display', None],
  [ 0x7920, 'CjkUnifiedIdeograph7920', [], 'Display', None],
  [ 0x7921, 'CjkUnifiedIdeograph7921', [], 'Display', None],
  [ 0x7922, 'CjkUnifiedIdeograph7922', [], 'Display', None],
  [ 0x7923, 'CjkUnifiedIdeograph7923', [], 'Display', None],
  [ 0x7924, 'CjkUnifiedIdeograph7924', [], 'Display', None],
  [ 0x7925, 'CjkUnifiedIdeograph7925', [], 'Display', None],
  [ 0x7926, 'CjkUnifiedIdeograph7926', [], 'Display', None],
  [ 0x7927, 'CjkUnifiedIdeograph7927', [], 'Display', None],
  [ 0x7928, 'CjkUnifiedIdeograph7928', [], 'Display', None],
  [ 0x7929, 'CjkUnifiedIdeograph7929', [], 'Display', None],
  [ 0x792a, 'CjkUnifiedIdeograph792a', [], 'Display', None],
  [ 0x792b, 'CjkUnifiedIdeograph792b', [], 'Display', None],
  [ 0x792c, 'CjkUnifiedIdeograph792c', [], 'Display', None],
  [ 0x792d, 'CjkUnifiedIdeograph792d', [], 'Display', None],
  [ 0x792e, 'CjkUnifiedIdeograph792e', [], 'Display', None],
  [ 0x792f, 'CjkUnifiedIdeograph792f', [], 'Display', None],
  [ 0x7930, 'CjkUnifiedIdeograph7930', [], 'Display', None],
  [ 0x7931, 'CjkUnifiedIdeograph7931', [], 'Display', None],
  [ 0x7932, 'CjkUnifiedIdeograph7932', [], 'Display', None],
  [ 0x7933, 'CjkUnifiedIdeograph7933', [], 'Display', None],
  [ 0x7934, 'CjkUnifiedIdeograph7934', [], 'Display', None],
  [ 0x7935, 'CjkUnifiedIdeograph7935', [], 'Display', None],
  [ 0x7936, 'CjkUnifiedIdeograph7936', [], 'Display', None],
  [ 0x7937, 'CjkUnifiedIdeograph7937', [], 'Display', None],
  [ 0x7938, 'CjkUnifiedIdeograph7938', [], 'Display', None],
  [ 0x7939, 'CjkUnifiedIdeograph7939', [], 'Display', None],
  [ 0x793a, 'CjkUnifiedIdeograph793a', [], 'Display', None],
  [ 0x793b, 'CjkUnifiedIdeograph793b', [], 'Display', None],
  [ 0x793c, 'CjkUnifiedIdeograph793c', [], 'Display', None],
  [ 0x793d, 'CjkUnifiedIdeograph793d', [], 'Display', None],
  [ 0x793e, 'CjkUnifiedIdeograph793e', [], 'Display', None],
  [ 0x793f, 'CjkUnifiedIdeograph793f', [], 'Display', None],
  [ 0x7940, 'CjkUnifiedIdeograph7940', [], 'Display', None],
  [ 0x7941, 'CjkUnifiedIdeograph7941', [], 'Display', None],
  [ 0x7942, 'CjkUnifiedIdeograph7942', [], 'Display', None],
  [ 0x7943, 'CjkUnifiedIdeograph7943', [], 'Display', None],
  [ 0x7944, 'CjkUnifiedIdeograph7944', [], 'Display', None],
  [ 0x7945, 'CjkUnifiedIdeograph7945', [], 'Display', None],
  [ 0x7946, 'CjkUnifiedIdeograph7946', [], 'Display', None],
  [ 0x7947, 'CjkUnifiedIdeograph7947', [], 'Display', None],
  [ 0x7948, 'CjkUnifiedIdeograph7948', [], 'Display', None],
  [ 0x7949, 'CjkUnifiedIdeograph7949', [], 'Display', None],
  [ 0x794a, 'CjkUnifiedIdeograph794a', [], 'Display', None],
  [ 0x794b, 'CjkUnifiedIdeograph794b', [], 'Display', None],
  [ 0x794c, 'CjkUnifiedIdeograph794c', [], 'Display', None],
  [ 0x794d, 'CjkUnifiedIdeograph794d', [], 'Display', None],
  [ 0x794e, 'CjkUnifiedIdeograph794e', [], 'Display', None],
  [ 0x794f, 'CjkUnifiedIdeograph794f', [], 'Display', None],
  [ 0x7950, 'CjkUnifiedIdeograph7950', [], 'Display', None],
  [ 0x7951, 'CjkUnifiedIdeograph7951', [], 'Display', None],
  [ 0x7952, 'CjkUnifiedIdeograph7952', [], 'Display', None],
  [ 0x7953, 'CjkUnifiedIdeograph7953', [], 'Display', None],
  [ 0x7954, 'CjkUnifiedIdeograph7954', [], 'Display', None],
  [ 0x7955, 'CjkUnifiedIdeograph7955', [], 'Display', None],
  [ 0x7956, 'CjkUnifiedIdeograph7956', [], 'Display', None],
  [ 0x7957, 'CjkUnifiedIdeograph7957', [], 'Display', None],
  [ 0x7958, 'CjkUnifiedIdeograph7958', [], 'Display', None],
  [ 0x7959, 'CjkUnifiedIdeograph7959', [], 'Display', None],
  [ 0x795a, 'CjkUnifiedIdeograph795a', [], 'Display', None],
  [ 0x795b, 'CjkUnifiedIdeograph795b', [], 'Display', None],
  [ 0x795c, 'CjkUnifiedIdeograph795c', [], 'Display', None],
  [ 0x795d, 'CjkUnifiedIdeograph795d', [], 'Display', None],
  [ 0x795e, 'CjkUnifiedIdeograph795e', [], 'Display', None],
  [ 0x795f, 'CjkUnifiedIdeograph795f', [], 'Display', None],
  [ 0x7960, 'CjkUnifiedIdeograph7960', [], 'Display', None],
  [ 0x7961, 'CjkUnifiedIdeograph7961', [], 'Display', None],
  [ 0x7962, 'CjkUnifiedIdeograph7962', [], 'Display', None],
  [ 0x7963, 'CjkUnifiedIdeograph7963', [], 'Display', None],
  [ 0x7964, 'CjkUnifiedIdeograph7964', [], 'Display', None],
  [ 0x7965, 'CjkUnifiedIdeograph7965', [], 'Display', None],
  [ 0x7966, 'CjkUnifiedIdeograph7966', [], 'Display', None],
  [ 0x7967, 'CjkUnifiedIdeograph7967', [], 'Display', None],
  [ 0x7968, 'CjkUnifiedIdeograph7968', [], 'Display', None],
  [ 0x7969, 'CjkUnifiedIdeograph7969', [], 'Display', None],
  [ 0x796a, 'CjkUnifiedIdeograph796a', [], 'Display', None],
  [ 0x796b, 'CjkUnifiedIdeograph796b', [], 'Display', None],
  [ 0x796c, 'CjkUnifiedIdeograph796c', [], 'Display', None],
  [ 0x796d, 'CjkUnifiedIdeograph796d', [], 'Display', None],
  [ 0x796e, 'CjkUnifiedIdeograph796e', [], 'Display', None],
  [ 0x796f, 'CjkUnifiedIdeograph796f', [], 'Display', None],
  [ 0x7970, 'CjkUnifiedIdeograph7970', [], 'Display', None],
  [ 0x7971, 'CjkUnifiedIdeograph7971', [], 'Display', None],
  [ 0x7972, 'CjkUnifiedIdeograph7972', [], 'Display', None],
  [ 0x7973, 'CjkUnifiedIdeograph7973', [], 'Display', None],
  [ 0x7974, 'CjkUnifiedIdeograph7974', [], 'Display', None],
  [ 0x7975, 'CjkUnifiedIdeograph7975', [], 'Display', None],
  [ 0x7976, 'CjkUnifiedIdeograph7976', [], 'Display', None],
  [ 0x7977, 'CjkUnifiedIdeograph7977', [], 'Display', None],
  [ 0x7978, 'CjkUnifiedIdeograph7978', [], 'Display', None],
  [ 0x7979, 'CjkUnifiedIdeograph7979', [], 'Display', None],
  [ 0x797a, 'CjkUnifiedIdeograph797a', [], 'Display', None],
  [ 0x797b, 'CjkUnifiedIdeograph797b', [], 'Display', None],
  [ 0x797c, 'CjkUnifiedIdeograph797c', [], 'Display', None],
  [ 0x797d, 'CjkUnifiedIdeograph797d', [], 'Display', None],
  [ 0x797e, 'CjkUnifiedIdeograph797e', [], 'Display', None],
  [ 0x797f, 'CjkUnifiedIdeograph797f', [], 'Display', None],
  [ 0x7980, 'CjkUnifiedIdeograph7980', [], 'Display', None],
  [ 0x7981, 'CjkUnifiedIdeograph7981', [], 'Display', None],
  [ 0x7982, 'CjkUnifiedIdeograph7982', [], 'Display', None],
  [ 0x7983, 'CjkUnifiedIdeograph7983', [], 'Display', None],
  [ 0x7984, 'CjkUnifiedIdeograph7984', [], 'Display', None],
  [ 0x7985, 'CjkUnifiedIdeograph7985', [], 'Display', None],
  [ 0x7986, 'CjkUnifiedIdeograph7986', [], 'Display', None],
  [ 0x7987, 'CjkUnifiedIdeograph7987', [], 'Display', None],
  [ 0x7988, 'CjkUnifiedIdeograph7988', [], 'Display', None],
  [ 0x7989, 'CjkUnifiedIdeograph7989', [], 'Display', None],
  [ 0x798a, 'CjkUnifiedIdeograph798a', [], 'Display', None],
  [ 0x798b, 'CjkUnifiedIdeograph798b', [], 'Display', None],
  [ 0x798c, 'CjkUnifiedIdeograph798c', [], 'Display', None],
  [ 0x798d, 'CjkUnifiedIdeograph798d', [], 'Display', None],
  [ 0x798e, 'CjkUnifiedIdeograph798e', [], 'Display', None],
  [ 0x798f, 'CjkUnifiedIdeograph798f', [], 'Display', None],
  [ 0x7990, 'CjkUnifiedIdeograph7990', [], 'Display', None],
  [ 0x7991, 'CjkUnifiedIdeograph7991', [], 'Display', None],
  [ 0x7992, 'CjkUnifiedIdeograph7992', [], 'Display', None],
  [ 0x7993, 'CjkUnifiedIdeograph7993', [], 'Display', None],
  [ 0x7994, 'CjkUnifiedIdeograph7994', [], 'Display', None],
  [ 0x7995, 'CjkUnifiedIdeograph7995', [], 'Display', None],
  [ 0x7996, 'CjkUnifiedIdeograph7996', [], 'Display', None],
  [ 0x7997, 'CjkUnifiedIdeograph7997', [], 'Display', None],
  [ 0x7998, 'CjkUnifiedIdeograph7998', [], 'Display', None],
  [ 0x7999, 'CjkUnifiedIdeograph7999', [], 'Display', None],
  [ 0x799a, 'CjkUnifiedIdeograph799a', [], 'Display', None],
  [ 0x799b, 'CjkUnifiedIdeograph799b', [], 'Display', None],
  [ 0x799c, 'CjkUnifiedIdeograph799c', [], 'Display', None],
  [ 0x799d, 'CjkUnifiedIdeograph799d', [], 'Display', None],
  [ 0x799e, 'CjkUnifiedIdeograph799e', [], 'Display', None],
  [ 0x799f, 'CjkUnifiedIdeograph799f', [], 'Display', None],
  [ 0x79a0, 'CjkUnifiedIdeograph79a0', [], 'Display', None],
  [ 0x79a1, 'CjkUnifiedIdeograph79a1', [], 'Display', None],
  [ 0x79a2, 'CjkUnifiedIdeograph79a2', [], 'Display', None],
  [ 0x79a3, 'CjkUnifiedIdeograph79a3', [], 'Display', None],
  [ 0x79a4, 'CjkUnifiedIdeograph79a4', [], 'Display', None],
  [ 0x79a5, 'CjkUnifiedIdeograph79a5', [], 'Display', None],
  [ 0x79a6, 'CjkUnifiedIdeograph79a6', [], 'Display', None],
  [ 0x79a7, 'CjkUnifiedIdeograph79a7', [], 'Display', None],
  [ 0x79a8, 'CjkUnifiedIdeograph79a8', [], 'Display', None],
  [ 0x79a9, 'CjkUnifiedIdeograph79a9', [], 'Display', None],
  [ 0x79aa, 'CjkUnifiedIdeograph79aa', [], 'Display', None],
  [ 0x79ab, 'CjkUnifiedIdeograph79ab', [], 'Display', None],
  [ 0x79ac, 'CjkUnifiedIdeograph79ac', [], 'Display', None],
  [ 0x79ad, 'CjkUnifiedIdeograph79ad', [], 'Display', None],
  [ 0x79ae, 'CjkUnifiedIdeograph79ae', [], 'Display', None],
  [ 0x79af, 'CjkUnifiedIdeograph79af', [], 'Display', None],
  [ 0x79b0, 'CjkUnifiedIdeograph79b0', [], 'Display', None],
  [ 0x79b1, 'CjkUnifiedIdeograph79b1', [], 'Display', None],
  [ 0x79b2, 'CjkUnifiedIdeograph79b2', [], 'Display', None],
  [ 0x79b3, 'CjkUnifiedIdeograph79b3', [], 'Display', None],
  [ 0x79b4, 'CjkUnifiedIdeograph79b4', [], 'Display', None],
  [ 0x79b5, 'CjkUnifiedIdeograph79b5', [], 'Display', None],
  [ 0x79b6, 'CjkUnifiedIdeograph79b6', [], 'Display', None],
  [ 0x79b7, 'CjkUnifiedIdeograph79b7', [], 'Display', None],
  [ 0x79b8, 'CjkUnifiedIdeograph79b8', [], 'Display', None],
  [ 0x79b9, 'CjkUnifiedIdeograph79b9', [], 'Display', None],
  [ 0x79ba, 'CjkUnifiedIdeograph79ba', [], 'Display', None],
  [ 0x79bb, 'CjkUnifiedIdeograph79bb', [], 'Display', None],
  [ 0x79bc, 'CjkUnifiedIdeograph79bc', [], 'Display', None],
  [ 0x79bd, 'CjkUnifiedIdeograph79bd', [], 'Display', None],
  [ 0x79be, 'CjkUnifiedIdeograph79be', [], 'Display', None],
  [ 0x79bf, 'CjkUnifiedIdeograph79bf', [], 'Display', None],
  [ 0x79c0, 'CjkUnifiedIdeograph79c0', [], 'Display', None],
  [ 0x79c1, 'CjkUnifiedIdeograph79c1', [], 'Display', None],
  [ 0x79c2, 'CjkUnifiedIdeograph79c2', [], 'Display', None],
  [ 0x79c3, 'CjkUnifiedIdeograph79c3', [], 'Display', None],
  [ 0x79c4, 'CjkUnifiedIdeograph79c4', [], 'Display', None],
  [ 0x79c5, 'CjkUnifiedIdeograph79c5', [], 'Display', None],
  [ 0x79c6, 'CjkUnifiedIdeograph79c6', [], 'Display', None],
  [ 0x79c7, 'CjkUnifiedIdeograph79c7', [], 'Display', None],
  [ 0x79c8, 'CjkUnifiedIdeograph79c8', [], 'Display', None],
  [ 0x79c9, 'CjkUnifiedIdeograph79c9', [], 'Display', None],
  [ 0x79ca, 'CjkUnifiedIdeograph79ca', [], 'Display', None],
  [ 0x79cb, 'CjkUnifiedIdeograph79cb', [], 'Display', None],
  [ 0x79cc, 'CjkUnifiedIdeograph79cc', [], 'Display', None],
  [ 0x79cd, 'CjkUnifiedIdeograph79cd', [], 'Display', None],
  [ 0x79ce, 'CjkUnifiedIdeograph79ce', [], 'Display', None],
  [ 0x79cf, 'CjkUnifiedIdeograph79cf', [], 'Display', None],
  [ 0x79d0, 'CjkUnifiedIdeograph79d0', [], 'Display', None],
  [ 0x79d1, 'CjkUnifiedIdeograph79d1', [], 'Display', None],
  [ 0x79d2, 'CjkUnifiedIdeograph79d2', [], 'Display', None],
  [ 0x79d3, 'CjkUnifiedIdeograph79d3', [], 'Display', None],
  [ 0x79d4, 'CjkUnifiedIdeograph79d4', [], 'Display', None],
  [ 0x79d5, 'CjkUnifiedIdeograph79d5', [], 'Display', None],
  [ 0x79d6, 'CjkUnifiedIdeograph79d6', [], 'Display', None],
  [ 0x79d7, 'CjkUnifiedIdeograph79d7', [], 'Display', None],
  [ 0x79d8, 'CjkUnifiedIdeograph79d8', [], 'Display', None],
  [ 0x79d9, 'CjkUnifiedIdeograph79d9', [], 'Display', None],
  [ 0x79da, 'CjkUnifiedIdeograph79da', [], 'Display', None],
  [ 0x79db, 'CjkUnifiedIdeograph79db', [], 'Display', None],
  [ 0x79dc, 'CjkUnifiedIdeograph79dc', [], 'Display', None],
  [ 0x79dd, 'CjkUnifiedIdeograph79dd', [], 'Display', None],
  [ 0x79de, 'CjkUnifiedIdeograph79de', [], 'Display', None],
  [ 0x79df, 'CjkUnifiedIdeograph79df', [], 'Display', None],
  [ 0x79e0, 'CjkUnifiedIdeograph79e0', [], 'Display', None],
  [ 0x79e1, 'CjkUnifiedIdeograph79e1', [], 'Display', None],
  [ 0x79e2, 'CjkUnifiedIdeograph79e2', [], 'Display', None],
  [ 0x79e3, 'CjkUnifiedIdeograph79e3', [], 'Display', None],
  [ 0x79e4, 'CjkUnifiedIdeograph79e4', [], 'Display', None],
  [ 0x79e5, 'CjkUnifiedIdeograph79e5', [], 'Display', None],
  [ 0x79e6, 'CjkUnifiedIdeograph79e6', [], 'Display', None],
  [ 0x79e7, 'CjkUnifiedIdeograph79e7', [], 'Display', None],
  [ 0x79e8, 'CjkUnifiedIdeograph79e8', [], 'Display', None],
  [ 0x79e9, 'CjkUnifiedIdeograph79e9', [], 'Display', None],
  [ 0x79ea, 'CjkUnifiedIdeograph79ea', [], 'Display', None],
  [ 0x79eb, 'CjkUnifiedIdeograph79eb', [], 'Display', None],
  [ 0x79ec, 'CjkUnifiedIdeograph79ec', [], 'Display', None],
  [ 0x79ed, 'CjkUnifiedIdeograph79ed', [], 'Display', None],
  [ 0x79ee, 'CjkUnifiedIdeograph79ee', [], 'Display', None],
  [ 0x79ef, 'CjkUnifiedIdeograph79ef', [], 'Display', None],
  [ 0x79f0, 'CjkUnifiedIdeograph79f0', [], 'Display', None],
  [ 0x79f1, 'CjkUnifiedIdeograph79f1', [], 'Display', None],
  [ 0x79f2, 'CjkUnifiedIdeograph79f2', [], 'Display', None],
  [ 0x79f3, 'CjkUnifiedIdeograph79f3', [], 'Display', None],
  [ 0x79f4, 'CjkUnifiedIdeograph79f4', [], 'Display', None],
  [ 0x79f5, 'CjkUnifiedIdeograph79f5', [], 'Display', None],
  [ 0x79f6, 'CjkUnifiedIdeograph79f6', [], 'Display', None],
  [ 0x79f7, 'CjkUnifiedIdeograph79f7', [], 'Display', None],
  [ 0x79f8, 'CjkUnifiedIdeograph79f8', [], 'Display', None],
  [ 0x79f9, 'CjkUnifiedIdeograph79f9', [], 'Display', None],
  [ 0x79fa, 'CjkUnifiedIdeograph79fa', [], 'Display', None],
  [ 0x79fb, 'CjkUnifiedIdeograph79fb', [], 'Display', None],
  [ 0x79fc, 'CjkUnifiedIdeograph79fc', [], 'Display', None],
  [ 0x79fd, 'CjkUnifiedIdeograph79fd', [], 'Display', None],
  [ 0x79fe, 'CjkUnifiedIdeograph79fe', [], 'Display', None],
  [ 0x79ff, 'CjkUnifiedIdeograph79ff', [], 'Display', None],
  [ 0x7a00, 'CjkUnifiedIdeograph7a00', [], 'Display', None],
  [ 0x7a01, 'CjkUnifiedIdeograph7a01', [], 'Display', None],
  [ 0x7a02, 'CjkUnifiedIdeograph7a02', [], 'Display', None],
  [ 0x7a03, 'CjkUnifiedIdeograph7a03', [], 'Display', None],
  [ 0x7a04, 'CjkUnifiedIdeograph7a04', [], 'Display', None],
  [ 0x7a05, 'CjkUnifiedIdeograph7a05', [], 'Display', None],
  [ 0x7a06, 'CjkUnifiedIdeograph7a06', [], 'Display', None],
  [ 0x7a07, 'CjkUnifiedIdeograph7a07', [], 'Display', None],
  [ 0x7a08, 'CjkUnifiedIdeograph7a08', [], 'Display', None],
  [ 0x7a09, 'CjkUnifiedIdeograph7a09', [], 'Display', None],
  [ 0x7a0a, 'CjkUnifiedIdeograph7a0a', [], 'Display', None],
  [ 0x7a0b, 'CjkUnifiedIdeograph7a0b', [], 'Display', None],
  [ 0x7a0c, 'CjkUnifiedIdeograph7a0c', [], 'Display', None],
  [ 0x7a0d, 'CjkUnifiedIdeograph7a0d', [], 'Display', None],
  [ 0x7a0e, 'CjkUnifiedIdeograph7a0e', [], 'Display', None],
  [ 0x7a0f, 'CjkUnifiedIdeograph7a0f', [], 'Display', None],
  [ 0x7a10, 'CjkUnifiedIdeograph7a10', [], 'Display', None],
  [ 0x7a11, 'CjkUnifiedIdeograph7a11', [], 'Display', None],
  [ 0x7a12, 'CjkUnifiedIdeograph7a12', [], 'Display', None],
  [ 0x7a13, 'CjkUnifiedIdeograph7a13', [], 'Display', None],
  [ 0x7a14, 'CjkUnifiedIdeograph7a14', [], 'Display', None],
  [ 0x7a15, 'CjkUnifiedIdeograph7a15', [], 'Display', None],
  [ 0x7a16, 'CjkUnifiedIdeograph7a16', [], 'Display', None],
  [ 0x7a17, 'CjkUnifiedIdeograph7a17', [], 'Display', None],
  [ 0x7a18, 'CjkUnifiedIdeograph7a18', [], 'Display', None],
  [ 0x7a19, 'CjkUnifiedIdeograph7a19', [], 'Display', None],
  [ 0x7a1a, 'CjkUnifiedIdeograph7a1a', [], 'Display', None],
  [ 0x7a1b, 'CjkUnifiedIdeograph7a1b', [], 'Display', None],
  [ 0x7a1c, 'CjkUnifiedIdeograph7a1c', [], 'Display', None],
  [ 0x7a1d, 'CjkUnifiedIdeograph7a1d', [], 'Display', None],
  [ 0x7a1e, 'CjkUnifiedIdeograph7a1e', [], 'Display', None],
  [ 0x7a1f, 'CjkUnifiedIdeograph7a1f', [], 'Display', None],
  [ 0x7a20, 'CjkUnifiedIdeograph7a20', [], 'Display', None],
  [ 0x7a21, 'CjkUnifiedIdeograph7a21', [], 'Display', None],
  [ 0x7a22, 'CjkUnifiedIdeograph7a22', [], 'Display', None],
  [ 0x7a23, 'CjkUnifiedIdeograph7a23', [], 'Display', None],
  [ 0x7a24, 'CjkUnifiedIdeograph7a24', [], 'Display', None],
  [ 0x7a25, 'CjkUnifiedIdeograph7a25', [], 'Display', None],
  [ 0x7a26, 'CjkUnifiedIdeograph7a26', [], 'Display', None],
  [ 0x7a27, 'CjkUnifiedIdeograph7a27', [], 'Display', None],
  [ 0x7a28, 'CjkUnifiedIdeograph7a28', [], 'Display', None],
  [ 0x7a29, 'CjkUnifiedIdeograph7a29', [], 'Display', None],
  [ 0x7a2a, 'CjkUnifiedIdeograph7a2a', [], 'Display', None],
  [ 0x7a2b, 'CjkUnifiedIdeograph7a2b', [], 'Display', None],
  [ 0x7a2c, 'CjkUnifiedIdeograph7a2c', [], 'Display', None],
  [ 0x7a2d, 'CjkUnifiedIdeograph7a2d', [], 'Display', None],
  [ 0x7a2e, 'CjkUnifiedIdeograph7a2e', [], 'Display', None],
  [ 0x7a2f, 'CjkUnifiedIdeograph7a2f', [], 'Display', None],
  [ 0x7a30, 'CjkUnifiedIdeograph7a30', [], 'Display', None],
  [ 0x7a31, 'CjkUnifiedIdeograph7a31', [], 'Display', None],
  [ 0x7a32, 'CjkUnifiedIdeograph7a32', [], 'Display', None],
  [ 0x7a33, 'CjkUnifiedIdeograph7a33', [], 'Display', None],
  [ 0x7a34, 'CjkUnifiedIdeograph7a34', [], 'Display', None],
  [ 0x7a35, 'CjkUnifiedIdeograph7a35', [], 'Display', None],
  [ 0x7a36, 'CjkUnifiedIdeograph7a36', [], 'Display', None],
  [ 0x7a37, 'CjkUnifiedIdeograph7a37', [], 'Display', None],
  [ 0x7a38, 'CjkUnifiedIdeograph7a38', [], 'Display', None],
  [ 0x7a39, 'CjkUnifiedIdeograph7a39', [], 'Display', None],
  [ 0x7a3a, 'CjkUnifiedIdeograph7a3a', [], 'Display', None],
  [ 0x7a3b, 'CjkUnifiedIdeograph7a3b', [], 'Display', None],
  [ 0x7a3c, 'CjkUnifiedIdeograph7a3c', [], 'Display', None],
  [ 0x7a3d, 'CjkUnifiedIdeograph7a3d', [], 'Display', None],
  [ 0x7a3e, 'CjkUnifiedIdeograph7a3e', [], 'Display', None],
  [ 0x7a3f, 'CjkUnifiedIdeograph7a3f', [], 'Display', None],
  [ 0x7a40, 'CjkUnifiedIdeograph7a40', [], 'Display', None],
  [ 0x7a41, 'CjkUnifiedIdeograph7a41', [], 'Display', None],
  [ 0x7a42, 'CjkUnifiedIdeograph7a42', [], 'Display', None],
  [ 0x7a43, 'CjkUnifiedIdeograph7a43', [], 'Display', None],
  [ 0x7a44, 'CjkUnifiedIdeograph7a44', [], 'Display', None],
  [ 0x7a45, 'CjkUnifiedIdeograph7a45', [], 'Display', None],
  [ 0x7a46, 'CjkUnifiedIdeograph7a46', [], 'Display', None],
  [ 0x7a47, 'CjkUnifiedIdeograph7a47', [], 'Display', None],
  [ 0x7a48, 'CjkUnifiedIdeograph7a48', [], 'Display', None],
  [ 0x7a49, 'CjkUnifiedIdeograph7a49', [], 'Display', None],
  [ 0x7a4a, 'CjkUnifiedIdeograph7a4a', [], 'Display', None],
  [ 0x7a4b, 'CjkUnifiedIdeograph7a4b', [], 'Display', None],
  [ 0x7a4c, 'CjkUnifiedIdeograph7a4c', [], 'Display', None],
  [ 0x7a4d, 'CjkUnifiedIdeograph7a4d', [], 'Display', None],
  [ 0x7a4e, 'CjkUnifiedIdeograph7a4e', [], 'Display', None],
  [ 0x7a4f, 'CjkUnifiedIdeograph7a4f', [], 'Display', None],
  [ 0x7a50, 'CjkUnifiedIdeograph7a50', [], 'Display', None],
  [ 0x7a51, 'CjkUnifiedIdeograph7a51', [], 'Display', None],
  [ 0x7a52, 'CjkUnifiedIdeograph7a52', [], 'Display', None],
  [ 0x7a53, 'CjkUnifiedIdeograph7a53', [], 'Display', None],
  [ 0x7a54, 'CjkUnifiedIdeograph7a54', [], 'Display', None],
  [ 0x7a55, 'CjkUnifiedIdeograph7a55', [], 'Display', None],
  [ 0x7a56, 'CjkUnifiedIdeograph7a56', [], 'Display', None],
  [ 0x7a57, 'CjkUnifiedIdeograph7a57', [], 'Display', None],
  [ 0x7a58, 'CjkUnifiedIdeograph7a58', [], 'Display', None],
  [ 0x7a59, 'CjkUnifiedIdeograph7a59', [], 'Display', None],
  [ 0x7a5a, 'CjkUnifiedIdeograph7a5a', [], 'Display', None],
  [ 0x7a5b, 'CjkUnifiedIdeograph7a5b', [], 'Display', None],
  [ 0x7a5c, 'CjkUnifiedIdeograph7a5c', [], 'Display', None],
  [ 0x7a5d, 'CjkUnifiedIdeograph7a5d', [], 'Display', None],
  [ 0x7a5e, 'CjkUnifiedIdeograph7a5e', [], 'Display', None],
  [ 0x7a5f, 'CjkUnifiedIdeograph7a5f', [], 'Display', None],
  [ 0x7a60, 'CjkUnifiedIdeograph7a60', [], 'Display', None],
  [ 0x7a61, 'CjkUnifiedIdeograph7a61', [], 'Display', None],
  [ 0x7a62, 'CjkUnifiedIdeograph7a62', [], 'Display', None],
  [ 0x7a63, 'CjkUnifiedIdeograph7a63', [], 'Display', None],
  [ 0x7a64, 'CjkUnifiedIdeograph7a64', [], 'Display', None],
  [ 0x7a65, 'CjkUnifiedIdeograph7a65', [], 'Display', None],
  [ 0x7a66, 'CjkUnifiedIdeograph7a66', [], 'Display', None],
  [ 0x7a67, 'CjkUnifiedIdeograph7a67', [], 'Display', None],
  [ 0x7a68, 'CjkUnifiedIdeograph7a68', [], 'Display', None],
  [ 0x7a69, 'CjkUnifiedIdeograph7a69', [], 'Display', None],
  [ 0x7a6a, 'CjkUnifiedIdeograph7a6a', [], 'Display', None],
  [ 0x7a6b, 'CjkUnifiedIdeograph7a6b', [], 'Display', None],
  [ 0x7a6c, 'CjkUnifiedIdeograph7a6c', [], 'Display', None],
  [ 0x7a6d, 'CjkUnifiedIdeograph7a6d', [], 'Display', None],
  [ 0x7a6e, 'CjkUnifiedIdeograph7a6e', [], 'Display', None],
  [ 0x7a6f, 'CjkUnifiedIdeograph7a6f', [], 'Display', None],
  [ 0x7a70, 'CjkUnifiedIdeograph7a70', [], 'Display', None],
  [ 0x7a71, 'CjkUnifiedIdeograph7a71', [], 'Display', None],
  [ 0x7a72, 'CjkUnifiedIdeograph7a72', [], 'Display', None],
  [ 0x7a73, 'CjkUnifiedIdeograph7a73', [], 'Display', None],
  [ 0x7a74, 'CjkUnifiedIdeograph7a74', [], 'Display', None],
  [ 0x7a75, 'CjkUnifiedIdeograph7a75', [], 'Display', None],
  [ 0x7a76, 'CjkUnifiedIdeograph7a76', [], 'Display', None],
  [ 0x7a77, 'CjkUnifiedIdeograph7a77', [], 'Display', None],
  [ 0x7a78, 'CjkUnifiedIdeograph7a78', [], 'Display', None],
  [ 0x7a79, 'CjkUnifiedIdeograph7a79', [], 'Display', None],
  [ 0x7a7a, 'CjkUnifiedIdeograph7a7a', [], 'Display', None],
  [ 0x7a7b, 'CjkUnifiedIdeograph7a7b', [], 'Display', None],
  [ 0x7a7c, 'CjkUnifiedIdeograph7a7c', [], 'Display', None],
  [ 0x7a7d, 'CjkUnifiedIdeograph7a7d', [], 'Display', None],
  [ 0x7a7e, 'CjkUnifiedIdeograph7a7e', [], 'Display', None],
  [ 0x7a7f, 'CjkUnifiedIdeograph7a7f', [], 'Display', None],
  [ 0x7a80, 'CjkUnifiedIdeograph7a80', [], 'Display', None],
  [ 0x7a81, 'CjkUnifiedIdeograph7a81', [], 'Display', None],
  [ 0x7a82, 'CjkUnifiedIdeograph7a82', [], 'Display', None],
  [ 0x7a83, 'CjkUnifiedIdeograph7a83', [], 'Display', None],
  [ 0x7a84, 'CjkUnifiedIdeograph7a84', [], 'Display', None],
  [ 0x7a85, 'CjkUnifiedIdeograph7a85', [], 'Display', None],
  [ 0x7a86, 'CjkUnifiedIdeograph7a86', [], 'Display', None],
  [ 0x7a87, 'CjkUnifiedIdeograph7a87', [], 'Display', None],
  [ 0x7a88, 'CjkUnifiedIdeograph7a88', [], 'Display', None],
  [ 0x7a89, 'CjkUnifiedIdeograph7a89', [], 'Display', None],
  [ 0x7a8a, 'CjkUnifiedIdeograph7a8a', [], 'Display', None],
  [ 0x7a8b, 'CjkUnifiedIdeograph7a8b', [], 'Display', None],
  [ 0x7a8c, 'CjkUnifiedIdeograph7a8c', [], 'Display', None],
  [ 0x7a8d, 'CjkUnifiedIdeograph7a8d', [], 'Display', None],
  [ 0x7a8e, 'CjkUnifiedIdeograph7a8e', [], 'Display', None],
  [ 0x7a8f, 'CjkUnifiedIdeograph7a8f', [], 'Display', None],
  [ 0x7a90, 'CjkUnifiedIdeograph7a90', [], 'Display', None],
  [ 0x7a91, 'CjkUnifiedIdeograph7a91', [], 'Display', None],
  [ 0x7a92, 'CjkUnifiedIdeograph7a92', [], 'Display', None],
  [ 0x7a93, 'CjkUnifiedIdeograph7a93', [], 'Display', None],
  [ 0x7a94, 'CjkUnifiedIdeograph7a94', [], 'Display', None],
  [ 0x7a95, 'CjkUnifiedIdeograph7a95', [], 'Display', None],
  [ 0x7a96, 'CjkUnifiedIdeograph7a96', [], 'Display', None],
  [ 0x7a97, 'CjkUnifiedIdeograph7a97', [], 'Display', None],
  [ 0x7a98, 'CjkUnifiedIdeograph7a98', [], 'Display', None],
  [ 0x7a99, 'CjkUnifiedIdeograph7a99', [], 'Display', None],
  [ 0x7a9a, 'CjkUnifiedIdeograph7a9a', [], 'Display', None],
  [ 0x7a9b, 'CjkUnifiedIdeograph7a9b', [], 'Display', None],
  [ 0x7a9c, 'CjkUnifiedIdeograph7a9c', [], 'Display', None],
  [ 0x7a9d, 'CjkUnifiedIdeograph7a9d', [], 'Display', None],
  [ 0x7a9e, 'CjkUnifiedIdeograph7a9e', [], 'Display', None],
  [ 0x7a9f, 'CjkUnifiedIdeograph7a9f', [], 'Display', None],
  [ 0x7aa0, 'CjkUnifiedIdeograph7aa0', [], 'Display', None],
  [ 0x7aa1, 'CjkUnifiedIdeograph7aa1', [], 'Display', None],
  [ 0x7aa2, 'CjkUnifiedIdeograph7aa2', [], 'Display', None],
  [ 0x7aa3, 'CjkUnifiedIdeograph7aa3', [], 'Display', None],
  [ 0x7aa4, 'CjkUnifiedIdeograph7aa4', [], 'Display', None],
  [ 0x7aa5, 'CjkUnifiedIdeograph7aa5', [], 'Display', None],
  [ 0x7aa6, 'CjkUnifiedIdeograph7aa6', [], 'Display', None],
  [ 0x7aa7, 'CjkUnifiedIdeograph7aa7', [], 'Display', None],
  [ 0x7aa8, 'CjkUnifiedIdeograph7aa8', [], 'Display', None],
  [ 0x7aa9, 'CjkUnifiedIdeograph7aa9', [], 'Display', None],
  [ 0x7aaa, 'CjkUnifiedIdeograph7aaa', [], 'Display', None],
  [ 0x7aab, 'CjkUnifiedIdeograph7aab', [], 'Display', None],
  [ 0x7aac, 'CjkUnifiedIdeograph7aac', [], 'Display', None],
  [ 0x7aad, 'CjkUnifiedIdeograph7aad', [], 'Display', None],
  [ 0x7aae, 'CjkUnifiedIdeograph7aae', [], 'Display', None],
  [ 0x7aaf, 'CjkUnifiedIdeograph7aaf', [], 'Display', None],
  [ 0x7ab0, 'CjkUnifiedIdeograph7ab0', [], 'Display', None],
  [ 0x7ab1, 'CjkUnifiedIdeograph7ab1', [], 'Display', None],
  [ 0x7ab2, 'CjkUnifiedIdeograph7ab2', [], 'Display', None],
  [ 0x7ab3, 'CjkUnifiedIdeograph7ab3', [], 'Display', None],
  [ 0x7ab4, 'CjkUnifiedIdeograph7ab4', [], 'Display', None],
  [ 0x7ab5, 'CjkUnifiedIdeograph7ab5', [], 'Display', None],
  [ 0x7ab6, 'CjkUnifiedIdeograph7ab6', [], 'Display', None],
  [ 0x7ab7, 'CjkUnifiedIdeograph7ab7', [], 'Display', None],
  [ 0x7ab8, 'CjkUnifiedIdeograph7ab8', [], 'Display', None],
  [ 0x7ab9, 'CjkUnifiedIdeograph7ab9', [], 'Display', None],
  [ 0x7aba, 'CjkUnifiedIdeograph7aba', [], 'Display', None],
  [ 0x7abb, 'CjkUnifiedIdeograph7abb', [], 'Display', None],
  [ 0x7abc, 'CjkUnifiedIdeograph7abc', [], 'Display', None],
  [ 0x7abd, 'CjkUnifiedIdeograph7abd', [], 'Display', None],
  [ 0x7abe, 'CjkUnifiedIdeograph7abe', [], 'Display', None],
  [ 0x7abf, 'CjkUnifiedIdeograph7abf', [], 'Display', None],
  [ 0x7ac0, 'CjkUnifiedIdeograph7ac0', [], 'Display', None],
  [ 0x7ac1, 'CjkUnifiedIdeograph7ac1', [], 'Display', None],
  [ 0x7ac2, 'CjkUnifiedIdeograph7ac2', [], 'Display', None],
  [ 0x7ac3, 'CjkUnifiedIdeograph7ac3', [], 'Display', None],
  [ 0x7ac4, 'CjkUnifiedIdeograph7ac4', [], 'Display', None],
  [ 0x7ac5, 'CjkUnifiedIdeograph7ac5', [], 'Display', None],
  [ 0x7ac6, 'CjkUnifiedIdeograph7ac6', [], 'Display', None],
  [ 0x7ac7, 'CjkUnifiedIdeograph7ac7', [], 'Display', None],
  [ 0x7ac8, 'CjkUnifiedIdeograph7ac8', [], 'Display', None],
  [ 0x7ac9, 'CjkUnifiedIdeograph7ac9', [], 'Display', None],
  [ 0x7aca, 'CjkUnifiedIdeograph7aca', [], 'Display', None],
  [ 0x7acb, 'CjkUnifiedIdeograph7acb', [], 'Display', None],
  [ 0x7acc, 'CjkUnifiedIdeograph7acc', [], 'Display', None],
  [ 0x7acd, 'CjkUnifiedIdeograph7acd', [], 'Display', None],
  [ 0x7ace, 'CjkUnifiedIdeograph7ace', [], 'Display', None],
  [ 0x7acf, 'CjkUnifiedIdeograph7acf', [], 'Display', None],
  [ 0x7ad0, 'CjkUnifiedIdeograph7ad0', [], 'Display', None],
  [ 0x7ad1, 'CjkUnifiedIdeograph7ad1', [], 'Display', None],
  [ 0x7ad2, 'CjkUnifiedIdeograph7ad2', [], 'Display', None],
  [ 0x7ad3, 'CjkUnifiedIdeograph7ad3', [], 'Display', None],
  [ 0x7ad4, 'CjkUnifiedIdeograph7ad4', [], 'Display', None],
  [ 0x7ad5, 'CjkUnifiedIdeograph7ad5', [], 'Display', None],
  [ 0x7ad6, 'CjkUnifiedIdeograph7ad6', [], 'Display', None],
  [ 0x7ad7, 'CjkUnifiedIdeograph7ad7', [], 'Display', None],
  [ 0x7ad8, 'CjkUnifiedIdeograph7ad8', [], 'Display', None],
  [ 0x7ad9, 'CjkUnifiedIdeograph7ad9', [], 'Display', None],
  [ 0x7ada, 'CjkUnifiedIdeograph7ada', [], 'Display', None],
  [ 0x7adb, 'CjkUnifiedIdeograph7adb', [], 'Display', None],
  [ 0x7adc, 'CjkUnifiedIdeograph7adc', [], 'Display', None],
  [ 0x7add, 'CjkUnifiedIdeograph7add', [], 'Display', None],
  [ 0x7ade, 'CjkUnifiedIdeograph7ade', [], 'Display', None],
  [ 0x7adf, 'CjkUnifiedIdeograph7adf', [], 'Display', None],
  [ 0x7ae0, 'CjkUnifiedIdeograph7ae0', [], 'Display', None],
  [ 0x7ae1, 'CjkUnifiedIdeograph7ae1', [], 'Display', None],
  [ 0x7ae2, 'CjkUnifiedIdeograph7ae2', [], 'Display', None],
  [ 0x7ae3, 'CjkUnifiedIdeograph7ae3', [], 'Display', None],
  [ 0x7ae4, 'CjkUnifiedIdeograph7ae4', [], 'Display', None],
  [ 0x7ae5, 'CjkUnifiedIdeograph7ae5', [], 'Display', None],
  [ 0x7ae6, 'CjkUnifiedIdeograph7ae6', [], 'Display', None],
  [ 0x7ae7, 'CjkUnifiedIdeograph7ae7', [], 'Display', None],
  [ 0x7ae8, 'CjkUnifiedIdeograph7ae8', [], 'Display', None],
  [ 0x7ae9, 'CjkUnifiedIdeograph7ae9', [], 'Display', None],
  [ 0x7aea, 'CjkUnifiedIdeograph7aea', [], 'Display', None],
  [ 0x7aeb, 'CjkUnifiedIdeograph7aeb', [], 'Display', None],
  [ 0x7aec, 'CjkUnifiedIdeograph7aec', [], 'Display', None],
  [ 0x7aed, 'CjkUnifiedIdeograph7aed', [], 'Display', None],
  [ 0x7aee, 'CjkUnifiedIdeograph7aee', [], 'Display', None],
  [ 0x7aef, 'CjkUnifiedIdeograph7aef', [], 'Display', None],
  [ 0x7af0, 'CjkUnifiedIdeograph7af0', [], 'Display', None],
  [ 0x7af1, 'CjkUnifiedIdeograph7af1', [], 'Display', None],
  [ 0x7af2, 'CjkUnifiedIdeograph7af2', [], 'Display', None],
  [ 0x7af3, 'CjkUnifiedIdeograph7af3', [], 'Display', None],
  [ 0x7af4, 'CjkUnifiedIdeograph7af4', [], 'Display', None],
  [ 0x7af5, 'CjkUnifiedIdeograph7af5', [], 'Display', None],
  [ 0x7af6, 'CjkUnifiedIdeograph7af6', [], 'Display', None],
  [ 0x7af7, 'CjkUnifiedIdeograph7af7', [], 'Display', None],
  [ 0x7af8, 'CjkUnifiedIdeograph7af8', [], 'Display', None],
  [ 0x7af9, 'CjkUnifiedIdeograph7af9', [], 'Display', None],
  [ 0x7afa, 'CjkUnifiedIdeograph7afa', [], 'Display', None],
  [ 0x7afb, 'CjkUnifiedIdeograph7afb', [], 'Display', None],
  [ 0x7afc, 'CjkUnifiedIdeograph7afc', [], 'Display', None],
  [ 0x7afd, 'CjkUnifiedIdeograph7afd', [], 'Display', None],
  [ 0x7afe, 'CjkUnifiedIdeograph7afe', [], 'Display', None],
  [ 0x7aff, 'CjkUnifiedIdeograph7aff', [], 'Display', None],
  [ 0x7b00, 'CjkUnifiedIdeograph7b00', [], 'Display', None],
  [ 0x7b01, 'CjkUnifiedIdeograph7b01', [], 'Display', None],
  [ 0x7b02, 'CjkUnifiedIdeograph7b02', [], 'Display', None],
  [ 0x7b03, 'CjkUnifiedIdeograph7b03', [], 'Display', None],
  [ 0x7b04, 'CjkUnifiedIdeograph7b04', [], 'Display', None],
  [ 0x7b05, 'CjkUnifiedIdeograph7b05', [], 'Display', None],
  [ 0x7b06, 'CjkUnifiedIdeograph7b06', [], 'Display', None],
  [ 0x7b07, 'CjkUnifiedIdeograph7b07', [], 'Display', None],
  [ 0x7b08, 'CjkUnifiedIdeograph7b08', [], 'Display', None],
  [ 0x7b09, 'CjkUnifiedIdeograph7b09', [], 'Display', None],
  [ 0x7b0a, 'CjkUnifiedIdeograph7b0a', [], 'Display', None],
  [ 0x7b0b, 'CjkUnifiedIdeograph7b0b', [], 'Display', None],
  [ 0x7b0c, 'CjkUnifiedIdeograph7b0c', [], 'Display', None],
  [ 0x7b0d, 'CjkUnifiedIdeograph7b0d', [], 'Display', None],
  [ 0x7b0e, 'CjkUnifiedIdeograph7b0e', [], 'Display', None],
  [ 0x7b0f, 'CjkUnifiedIdeograph7b0f', [], 'Display', None],
  [ 0x7b10, 'CjkUnifiedIdeograph7b10', [], 'Display', None],
  [ 0x7b11, 'CjkUnifiedIdeograph7b11', [], 'Display', None],
  [ 0x7b12, 'CjkUnifiedIdeograph7b12', [], 'Display', None],
  [ 0x7b13, 'CjkUnifiedIdeograph7b13', [], 'Display', None],
  [ 0x7b14, 'CjkUnifiedIdeograph7b14', [], 'Display', None],
  [ 0x7b15, 'CjkUnifiedIdeograph7b15', [], 'Display', None],
  [ 0x7b16, 'CjkUnifiedIdeograph7b16', [], 'Display', None],
  [ 0x7b17, 'CjkUnifiedIdeograph7b17', [], 'Display', None],
  [ 0x7b18, 'CjkUnifiedIdeograph7b18', [], 'Display', None],
  [ 0x7b19, 'CjkUnifiedIdeograph7b19', [], 'Display', None],
  [ 0x7b1a, 'CjkUnifiedIdeograph7b1a', [], 'Display', None],
  [ 0x7b1b, 'CjkUnifiedIdeograph7b1b', [], 'Display', None],
  [ 0x7b1c, 'CjkUnifiedIdeograph7b1c', [], 'Display', None],
  [ 0x7b1d, 'CjkUnifiedIdeograph7b1d', [], 'Display', None],
  [ 0x7b1e, 'CjkUnifiedIdeograph7b1e', [], 'Display', None],
  [ 0x7b1f, 'CjkUnifiedIdeograph7b1f', [], 'Display', None],
  [ 0x7b20, 'CjkUnifiedIdeograph7b20', [], 'Display', None],
  [ 0x7b21, 'CjkUnifiedIdeograph7b21', [], 'Display', None],
  [ 0x7b22, 'CjkUnifiedIdeograph7b22', [], 'Display', None],
  [ 0x7b23, 'CjkUnifiedIdeograph7b23', [], 'Display', None],
  [ 0x7b24, 'CjkUnifiedIdeograph7b24', [], 'Display', None],
  [ 0x7b25, 'CjkUnifiedIdeograph7b25', [], 'Display', None],
  [ 0x7b26, 'CjkUnifiedIdeograph7b26', [], 'Display', None],
  [ 0x7b27, 'CjkUnifiedIdeograph7b27', [], 'Display', None],
  [ 0x7b28, 'CjkUnifiedIdeograph7b28', [], 'Display', None],
  [ 0x7b29, 'CjkUnifiedIdeograph7b29', [], 'Display', None],
  [ 0x7b2a, 'CjkUnifiedIdeograph7b2a', [], 'Display', None],
  [ 0x7b2b, 'CjkUnifiedIdeograph7b2b', [], 'Display', None],
  [ 0x7b2c, 'CjkUnifiedIdeograph7b2c', [], 'Display', None],
  [ 0x7b2d, 'CjkUnifiedIdeograph7b2d', [], 'Display', None],
  [ 0x7b2e, 'CjkUnifiedIdeograph7b2e', [], 'Display', None],
  [ 0x7b2f, 'CjkUnifiedIdeograph7b2f', [], 'Display', None],
  [ 0x7b30, 'CjkUnifiedIdeograph7b30', [], 'Display', None],
  [ 0x7b31, 'CjkUnifiedIdeograph7b31', [], 'Display', None],
  [ 0x7b32, 'CjkUnifiedIdeograph7b32', [], 'Display', None],
  [ 0x7b33, 'CjkUnifiedIdeograph7b33', [], 'Display', None],
  [ 0x7b34, 'CjkUnifiedIdeograph7b34', [], 'Display', None],
  [ 0x7b35, 'CjkUnifiedIdeograph7b35', [], 'Display', None],
  [ 0x7b36, 'CjkUnifiedIdeograph7b36', [], 'Display', None],
  [ 0x7b37, 'CjkUnifiedIdeograph7b37', [], 'Display', None],
  [ 0x7b38, 'CjkUnifiedIdeograph7b38', [], 'Display', None],
  [ 0x7b39, 'CjkUnifiedIdeograph7b39', [], 'Display', None],
  [ 0x7b3a, 'CjkUnifiedIdeograph7b3a', [], 'Display', None],
  [ 0x7b3b, 'CjkUnifiedIdeograph7b3b', [], 'Display', None],
  [ 0x7b3c, 'CjkUnifiedIdeograph7b3c', [], 'Display', None],
  [ 0x7b3d, 'CjkUnifiedIdeograph7b3d', [], 'Display', None],
  [ 0x7b3e, 'CjkUnifiedIdeograph7b3e', [], 'Display', None],
  [ 0x7b3f, 'CjkUnifiedIdeograph7b3f', [], 'Display', None],
  [ 0x7b40, 'CjkUnifiedIdeograph7b40', [], 'Display', None],
  [ 0x7b41, 'CjkUnifiedIdeograph7b41', [], 'Display', None],
  [ 0x7b42, 'CjkUnifiedIdeograph7b42', [], 'Display', None],
  [ 0x7b43, 'CjkUnifiedIdeograph7b43', [], 'Display', None],
  [ 0x7b44, 'CjkUnifiedIdeograph7b44', [], 'Display', None],
  [ 0x7b45, 'CjkUnifiedIdeograph7b45', [], 'Display', None],
  [ 0x7b46, 'CjkUnifiedIdeograph7b46', [], 'Display', None],
  [ 0x7b47, 'CjkUnifiedIdeograph7b47', [], 'Display', None],
  [ 0x7b48, 'CjkUnifiedIdeograph7b48', [], 'Display', None],
  [ 0x7b49, 'CjkUnifiedIdeograph7b49', [], 'Display', None],
  [ 0x7b4a, 'CjkUnifiedIdeograph7b4a', [], 'Display', None],
  [ 0x7b4b, 'CjkUnifiedIdeograph7b4b', [], 'Display', None],
  [ 0x7b4c, 'CjkUnifiedIdeograph7b4c', [], 'Display', None],
  [ 0x7b4d, 'CjkUnifiedIdeograph7b4d', [], 'Display', None],
  [ 0x7b4e, 'CjkUnifiedIdeograph7b4e', [], 'Display', None],
  [ 0x7b4f, 'CjkUnifiedIdeograph7b4f', [], 'Display', None],
  [ 0x7b50, 'CjkUnifiedIdeograph7b50', [], 'Display', None],
  [ 0x7b51, 'CjkUnifiedIdeograph7b51', [], 'Display', None],
  [ 0x7b52, 'CjkUnifiedIdeograph7b52', [], 'Display', None],
  [ 0x7b53, 'CjkUnifiedIdeograph7b53', [], 'Display', None],
  [ 0x7b54, 'CjkUnifiedIdeograph7b54', [], 'Display', None],
  [ 0x7b55, 'CjkUnifiedIdeograph7b55', [], 'Display', None],
  [ 0x7b56, 'CjkUnifiedIdeograph7b56', [], 'Display', None],
  [ 0x7b57, 'CjkUnifiedIdeograph7b57', [], 'Display', None],
  [ 0x7b58, 'CjkUnifiedIdeograph7b58', [], 'Display', None],
  [ 0x7b59, 'CjkUnifiedIdeograph7b59', [], 'Display', None],
  [ 0x7b5a, 'CjkUnifiedIdeograph7b5a', [], 'Display', None],
  [ 0x7b5b, 'CjkUnifiedIdeograph7b5b', [], 'Display', None],
  [ 0x7b5c, 'CjkUnifiedIdeograph7b5c', [], 'Display', None],
  [ 0x7b5d, 'CjkUnifiedIdeograph7b5d', [], 'Display', None],
  [ 0x7b5e, 'CjkUnifiedIdeograph7b5e', [], 'Display', None],
  [ 0x7b5f, 'CjkUnifiedIdeograph7b5f', [], 'Display', None],
  [ 0x7b60, 'CjkUnifiedIdeograph7b60', [], 'Display', None],
  [ 0x7b61, 'CjkUnifiedIdeograph7b61', [], 'Display', None],
  [ 0x7b62, 'CjkUnifiedIdeograph7b62', [], 'Display', None],
  [ 0x7b63, 'CjkUnifiedIdeograph7b63', [], 'Display', None],
  [ 0x7b64, 'CjkUnifiedIdeograph7b64', [], 'Display', None],
  [ 0x7b65, 'CjkUnifiedIdeograph7b65', [], 'Display', None],
  [ 0x7b66, 'CjkUnifiedIdeograph7b66', [], 'Display', None],
  [ 0x7b67, 'CjkUnifiedIdeograph7b67', [], 'Display', None],
  [ 0x7b68, 'CjkUnifiedIdeograph7b68', [], 'Display', None],
  [ 0x7b69, 'CjkUnifiedIdeograph7b69', [], 'Display', None],
  [ 0x7b6a, 'CjkUnifiedIdeograph7b6a', [], 'Display', None],
  [ 0x7b6b, 'CjkUnifiedIdeograph7b6b', [], 'Display', None],
  [ 0x7b6c, 'CjkUnifiedIdeograph7b6c', [], 'Display', None],
  [ 0x7b6d, 'CjkUnifiedIdeograph7b6d', [], 'Display', None],
  [ 0x7b6e, 'CjkUnifiedIdeograph7b6e', [], 'Display', None],
  [ 0x7b6f, 'CjkUnifiedIdeograph7b6f', [], 'Display', None],
  [ 0x7b70, 'CjkUnifiedIdeograph7b70', [], 'Display', None],
  [ 0x7b71, 'CjkUnifiedIdeograph7b71', [], 'Display', None],
  [ 0x7b72, 'CjkUnifiedIdeograph7b72', [], 'Display', None],
  [ 0x7b73, 'CjkUnifiedIdeograph7b73', [], 'Display', None],
  [ 0x7b74, 'CjkUnifiedIdeograph7b74', [], 'Display', None],
  [ 0x7b75, 'CjkUnifiedIdeograph7b75', [], 'Display', None],
  [ 0x7b76, 'CjkUnifiedIdeograph7b76', [], 'Display', None],
  [ 0x7b77, 'CjkUnifiedIdeograph7b77', [], 'Display', None],
  [ 0x7b78, 'CjkUnifiedIdeograph7b78', [], 'Display', None],
  [ 0x7b79, 'CjkUnifiedIdeograph7b79', [], 'Display', None],
  [ 0x7b7a, 'CjkUnifiedIdeograph7b7a', [], 'Display', None],
  [ 0x7b7b, 'CjkUnifiedIdeograph7b7b', [], 'Display', None],
  [ 0x7b7c, 'CjkUnifiedIdeograph7b7c', [], 'Display', None],
  [ 0x7b7d, 'CjkUnifiedIdeograph7b7d', [], 'Display', None],
  [ 0x7b7e, 'CjkUnifiedIdeograph7b7e', [], 'Display', None],
  [ 0x7b7f, 'CjkUnifiedIdeograph7b7f', [], 'Display', None],
  [ 0x7b80, 'CjkUnifiedIdeograph7b80', [], 'Display', None],
  [ 0x7b81, 'CjkUnifiedIdeograph7b81', [], 'Display', None],
  [ 0x7b82, 'CjkUnifiedIdeograph7b82', [], 'Display', None],
  [ 0x7b83, 'CjkUnifiedIdeograph7b83', [], 'Display', None],
  [ 0x7b84, 'CjkUnifiedIdeograph7b84', [], 'Display', None],
  [ 0x7b85, 'CjkUnifiedIdeograph7b85', [], 'Display', None],
  [ 0x7b86, 'CjkUnifiedIdeograph7b86', [], 'Display', None],
  [ 0x7b87, 'CjkUnifiedIdeograph7b87', [], 'Display', None],
  [ 0x7b88, 'CjkUnifiedIdeograph7b88', [], 'Display', None],
  [ 0x7b89, 'CjkUnifiedIdeograph7b89', [], 'Display', None],
  [ 0x7b8a, 'CjkUnifiedIdeograph7b8a', [], 'Display', None],
  [ 0x7b8b, 'CjkUnifiedIdeograph7b8b', [], 'Display', None],
  [ 0x7b8c, 'CjkUnifiedIdeograph7b8c', [], 'Display', None],
  [ 0x7b8d, 'CjkUnifiedIdeograph7b8d', [], 'Display', None],
  [ 0x7b8e, 'CjkUnifiedIdeograph7b8e', [], 'Display', None],
  [ 0x7b8f, 'CjkUnifiedIdeograph7b8f', [], 'Display', None],
  [ 0x7b90, 'CjkUnifiedIdeograph7b90', [], 'Display', None],
  [ 0x7b91, 'CjkUnifiedIdeograph7b91', [], 'Display', None],
  [ 0x7b92, 'CjkUnifiedIdeograph7b92', [], 'Display', None],
  [ 0x7b93, 'CjkUnifiedIdeograph7b93', [], 'Display', None],
  [ 0x7b94, 'CjkUnifiedIdeograph7b94', [], 'Display', None],
  [ 0x7b95, 'CjkUnifiedIdeograph7b95', [], 'Display', None],
  [ 0x7b96, 'CjkUnifiedIdeograph7b96', [], 'Display', None],
  [ 0x7b97, 'CjkUnifiedIdeograph7b97', [], 'Display', None],
  [ 0x7b98, 'CjkUnifiedIdeograph7b98', [], 'Display', None],
  [ 0x7b99, 'CjkUnifiedIdeograph7b99', [], 'Display', None],
  [ 0x7b9a, 'CjkUnifiedIdeograph7b9a', [], 'Display', None],
  [ 0x7b9b, 'CjkUnifiedIdeograph7b9b', [], 'Display', None],
  [ 0x7b9c, 'CjkUnifiedIdeograph7b9c', [], 'Display', None],
  [ 0x7b9d, 'CjkUnifiedIdeograph7b9d', [], 'Display', None],
  [ 0x7b9e, 'CjkUnifiedIdeograph7b9e', [], 'Display', None],
  [ 0x7b9f, 'CjkUnifiedIdeograph7b9f', [], 'Display', None],
  [ 0x7ba0, 'CjkUnifiedIdeograph7ba0', [], 'Display', None],
  [ 0x7ba1, 'CjkUnifiedIdeograph7ba1', [], 'Display', None],
  [ 0x7ba2, 'CjkUnifiedIdeograph7ba2', [], 'Display', None],
  [ 0x7ba3, 'CjkUnifiedIdeograph7ba3', [], 'Display', None],
  [ 0x7ba4, 'CjkUnifiedIdeograph7ba4', [], 'Display', None],
  [ 0x7ba5, 'CjkUnifiedIdeograph7ba5', [], 'Display', None],
  [ 0x7ba6, 'CjkUnifiedIdeograph7ba6', [], 'Display', None],
  [ 0x7ba7, 'CjkUnifiedIdeograph7ba7', [], 'Display', None],
  [ 0x7ba8, 'CjkUnifiedIdeograph7ba8', [], 'Display', None],
  [ 0x7ba9, 'CjkUnifiedIdeograph7ba9', [], 'Display', None],
  [ 0x7baa, 'CjkUnifiedIdeograph7baa', [], 'Display', None],
  [ 0x7bab, 'CjkUnifiedIdeograph7bab', [], 'Display', None],
  [ 0x7bac, 'CjkUnifiedIdeograph7bac', [], 'Display', None],
  [ 0x7bad, 'CjkUnifiedIdeograph7bad', [], 'Display', None],
  [ 0x7bae, 'CjkUnifiedIdeograph7bae', [], 'Display', None],
  [ 0x7baf, 'CjkUnifiedIdeograph7baf', [], 'Display', None],
  [ 0x7bb0, 'CjkUnifiedIdeograph7bb0', [], 'Display', None],
  [ 0x7bb1, 'CjkUnifiedIdeograph7bb1', [], 'Display', None],
  [ 0x7bb2, 'CjkUnifiedIdeograph7bb2', [], 'Display', None],
  [ 0x7bb3, 'CjkUnifiedIdeograph7bb3', [], 'Display', None],
  [ 0x7bb4, 'CjkUnifiedIdeograph7bb4', [], 'Display', None],
  [ 0x7bb5, 'CjkUnifiedIdeograph7bb5', [], 'Display', None],
  [ 0x7bb6, 'CjkUnifiedIdeograph7bb6', [], 'Display', None],
  [ 0x7bb7, 'CjkUnifiedIdeograph7bb7', [], 'Display', None],
  [ 0x7bb8, 'CjkUnifiedIdeograph7bb8', [], 'Display', None],
  [ 0x7bb9, 'CjkUnifiedIdeograph7bb9', [], 'Display', None],
  [ 0x7bba, 'CjkUnifiedIdeograph7bba', [], 'Display', None],
  [ 0x7bbb, 'CjkUnifiedIdeograph7bbb', [], 'Display', None],
  [ 0x7bbc, 'CjkUnifiedIdeograph7bbc', [], 'Display', None],
  [ 0x7bbd, 'CjkUnifiedIdeograph7bbd', [], 'Display', None],
  [ 0x7bbe, 'CjkUnifiedIdeograph7bbe', [], 'Display', None],
  [ 0x7bbf, 'CjkUnifiedIdeograph7bbf', [], 'Display', None],
  [ 0x7bc0, 'CjkUnifiedIdeograph7bc0', [], 'Display', None],
  [ 0x7bc1, 'CjkUnifiedIdeograph7bc1', [], 'Display', None],
  [ 0x7bc2, 'CjkUnifiedIdeograph7bc2', [], 'Display', None],
  [ 0x7bc3, 'CjkUnifiedIdeograph7bc3', [], 'Display', None],
  [ 0x7bc4, 'CjkUnifiedIdeograph7bc4', [], 'Display', None],
  [ 0x7bc5, 'CjkUnifiedIdeograph7bc5', [], 'Display', None],
  [ 0x7bc6, 'CjkUnifiedIdeograph7bc6', [], 'Display', None],
  [ 0x7bc7, 'CjkUnifiedIdeograph7bc7', [], 'Display', None],
  [ 0x7bc8, 'CjkUnifiedIdeograph7bc8', [], 'Display', None],
  [ 0x7bc9, 'CjkUnifiedIdeograph7bc9', [], 'Display', None],
  [ 0x7bca, 'CjkUnifiedIdeograph7bca', [], 'Display', None],
  [ 0x7bcb, 'CjkUnifiedIdeograph7bcb', [], 'Display', None],
  [ 0x7bcc, 'CjkUnifiedIdeograph7bcc', [], 'Display', None],
  [ 0x7bcd, 'CjkUnifiedIdeograph7bcd', [], 'Display', None],
  [ 0x7bce, 'CjkUnifiedIdeograph7bce', [], 'Display', None],
  [ 0x7bcf, 'CjkUnifiedIdeograph7bcf', [], 'Display', None],
  [ 0x7bd0, 'CjkUnifiedIdeograph7bd0', [], 'Display', None],
  [ 0x7bd1, 'CjkUnifiedIdeograph7bd1', [], 'Display', None],
  [ 0x7bd2, 'CjkUnifiedIdeograph7bd2', [], 'Display', None],
  [ 0x7bd3, 'CjkUnifiedIdeograph7bd3', [], 'Display', None],
  [ 0x7bd4, 'CjkUnifiedIdeograph7bd4', [], 'Display', None],
  [ 0x7bd5, 'CjkUnifiedIdeograph7bd5', [], 'Display', None],
  [ 0x7bd6, 'CjkUnifiedIdeograph7bd6', [], 'Display', None],
  [ 0x7bd7, 'CjkUnifiedIdeograph7bd7', [], 'Display', None],
  [ 0x7bd8, 'CjkUnifiedIdeograph7bd8', [], 'Display', None],
  [ 0x7bd9, 'CjkUnifiedIdeograph7bd9', [], 'Display', None],
  [ 0x7bda, 'CjkUnifiedIdeograph7bda', [], 'Display', None],
  [ 0x7bdb, 'CjkUnifiedIdeograph7bdb', [], 'Display', None],
  [ 0x7bdc, 'CjkUnifiedIdeograph7bdc', [], 'Display', None],
  [ 0x7bdd, 'CjkUnifiedIdeograph7bdd', [], 'Display', None],
  [ 0x7bde, 'CjkUnifiedIdeograph7bde', [], 'Display', None],
  [ 0x7bdf, 'CjkUnifiedIdeograph7bdf', [], 'Display', None],
  [ 0x7be0, 'CjkUnifiedIdeograph7be0', [], 'Display', None],
  [ 0x7be1, 'CjkUnifiedIdeograph7be1', [], 'Display', None],
  [ 0x7be2, 'CjkUnifiedIdeograph7be2', [], 'Display', None],
  [ 0x7be3, 'CjkUnifiedIdeograph7be3', [], 'Display', None],
  [ 0x7be4, 'CjkUnifiedIdeograph7be4', [], 'Display', None],
  [ 0x7be5, 'CjkUnifiedIdeograph7be5', [], 'Display', None],
  [ 0x7be6, 'CjkUnifiedIdeograph7be6', [], 'Display', None],
  [ 0x7be7, 'CjkUnifiedIdeograph7be7', [], 'Display', None],
  [ 0x7be8, 'CjkUnifiedIdeograph7be8', [], 'Display', None],
  [ 0x7be9, 'CjkUnifiedIdeograph7be9', [], 'Display', None],
  [ 0x7bea, 'CjkUnifiedIdeograph7bea', [], 'Display', None],
  [ 0x7beb, 'CjkUnifiedIdeograph7beb', [], 'Display', None],
  [ 0x7bec, 'CjkUnifiedIdeograph7bec', [], 'Display', None],
  [ 0x7bed, 'CjkUnifiedIdeograph7bed', [], 'Display', None],
  [ 0x7bee, 'CjkUnifiedIdeograph7bee', [], 'Display', None],
  [ 0x7bef, 'CjkUnifiedIdeograph7bef', [], 'Display', None],
  [ 0x7bf0, 'CjkUnifiedIdeograph7bf0', [], 'Display', None],
  [ 0x7bf1, 'CjkUnifiedIdeograph7bf1', [], 'Display', None],
  [ 0x7bf2, 'CjkUnifiedIdeograph7bf2', [], 'Display', None],
  [ 0x7bf3, 'CjkUnifiedIdeograph7bf3', [], 'Display', None],
  [ 0x7bf4, 'CjkUnifiedIdeograph7bf4', [], 'Display', None],
  [ 0x7bf5, 'CjkUnifiedIdeograph7bf5', [], 'Display', None],
  [ 0x7bf6, 'CjkUnifiedIdeograph7bf6', [], 'Display', None],
  [ 0x7bf7, 'CjkUnifiedIdeograph7bf7', [], 'Display', None],
  [ 0x7bf8, 'CjkUnifiedIdeograph7bf8', [], 'Display', None],
  [ 0x7bf9, 'CjkUnifiedIdeograph7bf9', [], 'Display', None],
  [ 0x7bfa, 'CjkUnifiedIdeograph7bfa', [], 'Display', None],
  [ 0x7bfb, 'CjkUnifiedIdeograph7bfb', [], 'Display', None],
  [ 0x7bfc, 'CjkUnifiedIdeograph7bfc', [], 'Display', None],
  [ 0x7bfd, 'CjkUnifiedIdeograph7bfd', [], 'Display', None],
  [ 0x7bfe, 'CjkUnifiedIdeograph7bfe', [], 'Display', None],
  [ 0x7bff, 'CjkUnifiedIdeograph7bff', [], 'Display', None],
  [ 0x7c00, 'CjkUnifiedIdeograph7c00', [], 'Display', None],
  [ 0x7c01, 'CjkUnifiedIdeograph7c01', [], 'Display', None],
  [ 0x7c02, 'CjkUnifiedIdeograph7c02', [], 'Display', None],
  [ 0x7c03, 'CjkUnifiedIdeograph7c03', [], 'Display', None],
  [ 0x7c04, 'CjkUnifiedIdeograph7c04', [], 'Display', None],
  [ 0x7c05, 'CjkUnifiedIdeograph7c05', [], 'Display', None],
  [ 0x7c06, 'CjkUnifiedIdeograph7c06', [], 'Display', None],
  [ 0x7c07, 'CjkUnifiedIdeograph7c07', [], 'Display', None],
  [ 0x7c08, 'CjkUnifiedIdeograph7c08', [], 'Display', None],
  [ 0x7c09, 'CjkUnifiedIdeograph7c09', [], 'Display', None],
  [ 0x7c0a, 'CjkUnifiedIdeograph7c0a', [], 'Display', None],
  [ 0x7c0b, 'CjkUnifiedIdeograph7c0b', [], 'Display', None],
  [ 0x7c0c, 'CjkUnifiedIdeograph7c0c', [], 'Display', None],
  [ 0x7c0d, 'CjkUnifiedIdeograph7c0d', [], 'Display', None],
  [ 0x7c0e, 'CjkUnifiedIdeograph7c0e', [], 'Display', None],
  [ 0x7c0f, 'CjkUnifiedIdeograph7c0f', [], 'Display', None],
  [ 0x7c10, 'CjkUnifiedIdeograph7c10', [], 'Display', None],
  [ 0x7c11, 'CjkUnifiedIdeograph7c11', [], 'Display', None],
  [ 0x7c12, 'CjkUnifiedIdeograph7c12', [], 'Display', None],
  [ 0x7c13, 'CjkUnifiedIdeograph7c13', [], 'Display', None],
  [ 0x7c14, 'CjkUnifiedIdeograph7c14', [], 'Display', None],
  [ 0x7c15, 'CjkUnifiedIdeograph7c15', [], 'Display', None],
  [ 0x7c16, 'CjkUnifiedIdeograph7c16', [], 'Display', None],
  [ 0x7c17, 'CjkUnifiedIdeograph7c17', [], 'Display', None],
  [ 0x7c18, 'CjkUnifiedIdeograph7c18', [], 'Display', None],
  [ 0x7c19, 'CjkUnifiedIdeograph7c19', [], 'Display', None],
  [ 0x7c1a, 'CjkUnifiedIdeograph7c1a', [], 'Display', None],
  [ 0x7c1b, 'CjkUnifiedIdeograph7c1b', [], 'Display', None],
  [ 0x7c1c, 'CjkUnifiedIdeograph7c1c', [], 'Display', None],
  [ 0x7c1d, 'CjkUnifiedIdeograph7c1d', [], 'Display', None],
  [ 0x7c1e, 'CjkUnifiedIdeograph7c1e', [], 'Display', None],
  [ 0x7c1f, 'CjkUnifiedIdeograph7c1f', [], 'Display', None],
  [ 0x7c20, 'CjkUnifiedIdeograph7c20', [], 'Display', None],
  [ 0x7c21, 'CjkUnifiedIdeograph7c21', [], 'Display', None],
  [ 0x7c22, 'CjkUnifiedIdeograph7c22', [], 'Display', None],
  [ 0x7c23, 'CjkUnifiedIdeograph7c23', [], 'Display', None],
  [ 0x7c24, 'CjkUnifiedIdeograph7c24', [], 'Display', None],
  [ 0x7c25, 'CjkUnifiedIdeograph7c25', [], 'Display', None],
  [ 0x7c26, 'CjkUnifiedIdeograph7c26', [], 'Display', None],
  [ 0x7c27, 'CjkUnifiedIdeograph7c27', [], 'Display', None],
  [ 0x7c28, 'CjkUnifiedIdeograph7c28', [], 'Display', None],
  [ 0x7c29, 'CjkUnifiedIdeograph7c29', [], 'Display', None],
  [ 0x7c2a, 'CjkUnifiedIdeograph7c2a', [], 'Display', None],
  [ 0x7c2b, 'CjkUnifiedIdeograph7c2b', [], 'Display', None],
  [ 0x7c2c, 'CjkUnifiedIdeograph7c2c', [], 'Display', None],
  [ 0x7c2d, 'CjkUnifiedIdeograph7c2d', [], 'Display', None],
  [ 0x7c2e, 'CjkUnifiedIdeograph7c2e', [], 'Display', None],
  [ 0x7c2f, 'CjkUnifiedIdeograph7c2f', [], 'Display', None],
  [ 0x7c30, 'CjkUnifiedIdeograph7c30', [], 'Display', None],
  [ 0x7c31, 'CjkUnifiedIdeograph7c31', [], 'Display', None],
  [ 0x7c32, 'CjkUnifiedIdeograph7c32', [], 'Display', None],
  [ 0x7c33, 'CjkUnifiedIdeograph7c33', [], 'Display', None],
  [ 0x7c34, 'CjkUnifiedIdeograph7c34', [], 'Display', None],
  [ 0x7c35, 'CjkUnifiedIdeograph7c35', [], 'Display', None],
  [ 0x7c36, 'CjkUnifiedIdeograph7c36', [], 'Display', None],
  [ 0x7c37, 'CjkUnifiedIdeograph7c37', [], 'Display', None],
  [ 0x7c38, 'CjkUnifiedIdeograph7c38', [], 'Display', None],
  [ 0x7c39, 'CjkUnifiedIdeograph7c39', [], 'Display', None],
  [ 0x7c3a, 'CjkUnifiedIdeograph7c3a', [], 'Display', None],
  [ 0x7c3b, 'CjkUnifiedIdeograph7c3b', [], 'Display', None],
  [ 0x7c3c, 'CjkUnifiedIdeograph7c3c', [], 'Display', None],
  [ 0x7c3d, 'CjkUnifiedIdeograph7c3d', [], 'Display', None],
  [ 0x7c3e, 'CjkUnifiedIdeograph7c3e', [], 'Display', None],
  [ 0x7c3f, 'CjkUnifiedIdeograph7c3f', [], 'Display', None],
  [ 0x7c40, 'CjkUnifiedIdeograph7c40', [], 'Display', None],
  [ 0x7c41, 'CjkUnifiedIdeograph7c41', [], 'Display', None],
  [ 0x7c42, 'CjkUnifiedIdeograph7c42', [], 'Display', None],
  [ 0x7c43, 'CjkUnifiedIdeograph7c43', [], 'Display', None],
  [ 0x7c44, 'CjkUnifiedIdeograph7c44', [], 'Display', None],
  [ 0x7c45, 'CjkUnifiedIdeograph7c45', [], 'Display', None],
  [ 0x7c46, 'CjkUnifiedIdeograph7c46', [], 'Display', None],
  [ 0x7c47, 'CjkUnifiedIdeograph7c47', [], 'Display', None],
  [ 0x7c48, 'CjkUnifiedIdeograph7c48', [], 'Display', None],
  [ 0x7c49, 'CjkUnifiedIdeograph7c49', [], 'Display', None],
  [ 0x7c4a, 'CjkUnifiedIdeograph7c4a', [], 'Display', None],
  [ 0x7c4b, 'CjkUnifiedIdeograph7c4b', [], 'Display', None],
  [ 0x7c4c, 'CjkUnifiedIdeograph7c4c', [], 'Display', None],
  [ 0x7c4d, 'CjkUnifiedIdeograph7c4d', [], 'Display', None],
  [ 0x7c4e, 'CjkUnifiedIdeograph7c4e', [], 'Display', None],
  [ 0x7c4f, 'CjkUnifiedIdeograph7c4f', [], 'Display', None],
  [ 0x7c50, 'CjkUnifiedIdeograph7c50', [], 'Display', None],
  [ 0x7c51, 'CjkUnifiedIdeograph7c51', [], 'Display', None],
  [ 0x7c52, 'CjkUnifiedIdeograph7c52', [], 'Display', None],
  [ 0x7c53, 'CjkUnifiedIdeograph7c53', [], 'Display', None],
  [ 0x7c54, 'CjkUnifiedIdeograph7c54', [], 'Display', None],
  [ 0x7c55, 'CjkUnifiedIdeograph7c55', [], 'Display', None],
  [ 0x7c56, 'CjkUnifiedIdeograph7c56', [], 'Display', None],
  [ 0x7c57, 'CjkUnifiedIdeograph7c57', [], 'Display', None],
  [ 0x7c58, 'CjkUnifiedIdeograph7c58', [], 'Display', None],
  [ 0x7c59, 'CjkUnifiedIdeograph7c59', [], 'Display', None],
  [ 0x7c5a, 'CjkUnifiedIdeograph7c5a', [], 'Display', None],
  [ 0x7c5b, 'CjkUnifiedIdeograph7c5b', [], 'Display', None],
  [ 0x7c5c, 'CjkUnifiedIdeograph7c5c', [], 'Display', None],
  [ 0x7c5d, 'CjkUnifiedIdeograph7c5d', [], 'Display', None],
  [ 0x7c5e, 'CjkUnifiedIdeograph7c5e', [], 'Display', None],
  [ 0x7c5f, 'CjkUnifiedIdeograph7c5f', [], 'Display', None],
  [ 0x7c60, 'CjkUnifiedIdeograph7c60', [], 'Display', None],
  [ 0x7c61, 'CjkUnifiedIdeograph7c61', [], 'Display', None],
  [ 0x7c62, 'CjkUnifiedIdeograph7c62', [], 'Display', None],
  [ 0x7c63, 'CjkUnifiedIdeograph7c63', [], 'Display', None],
  [ 0x7c64, 'CjkUnifiedIdeograph7c64', [], 'Display', None],
  [ 0x7c65, 'CjkUnifiedIdeograph7c65', [], 'Display', None],
  [ 0x7c66, 'CjkUnifiedIdeograph7c66', [], 'Display', None],
  [ 0x7c67, 'CjkUnifiedIdeograph7c67', [], 'Display', None],
  [ 0x7c68, 'CjkUnifiedIdeograph7c68', [], 'Display', None],
  [ 0x7c69, 'CjkUnifiedIdeograph7c69', [], 'Display', None],
  [ 0x7c6a, 'CjkUnifiedIdeograph7c6a', [], 'Display', None],
  [ 0x7c6b, 'CjkUnifiedIdeograph7c6b', [], 'Display', None],
  [ 0x7c6c, 'CjkUnifiedIdeograph7c6c', [], 'Display', None],
  [ 0x7c6d, 'CjkUnifiedIdeograph7c6d', [], 'Display', None],
  [ 0x7c6e, 'CjkUnifiedIdeograph7c6e', [], 'Display', None],
  [ 0x7c6f, 'CjkUnifiedIdeograph7c6f', [], 'Display', None],
  [ 0x7c70, 'CjkUnifiedIdeograph7c70', [], 'Display', None],
  [ 0x7c71, 'CjkUnifiedIdeograph7c71', [], 'Display', None],
  [ 0x7c72, 'CjkUnifiedIdeograph7c72', [], 'Display', None],
  [ 0x7c73, 'CjkUnifiedIdeograph7c73', [], 'Display', None],
  [ 0x7c74, 'CjkUnifiedIdeograph7c74', [], 'Display', None],
  [ 0x7c75, 'CjkUnifiedIdeograph7c75', [], 'Display', None],
  [ 0x7c76, 'CjkUnifiedIdeograph7c76', [], 'Display', None],
  [ 0x7c77, 'CjkUnifiedIdeograph7c77', [], 'Display', None],
  [ 0x7c78, 'CjkUnifiedIdeograph7c78', [], 'Display', None],
  [ 0x7c79, 'CjkUnifiedIdeograph7c79', [], 'Display', None],
  [ 0x7c7a, 'CjkUnifiedIdeograph7c7a', [], 'Display', None],
  [ 0x7c7b, 'CjkUnifiedIdeograph7c7b', [], 'Display', None],
  [ 0x7c7c, 'CjkUnifiedIdeograph7c7c', [], 'Display', None],
  [ 0x7c7d, 'CjkUnifiedIdeograph7c7d', [], 'Display', None],
  [ 0x7c7e, 'CjkUnifiedIdeograph7c7e', [], 'Display', None],
  [ 0x7c7f, 'CjkUnifiedIdeograph7c7f', [], 'Display', None],
  [ 0x7c80, 'CjkUnifiedIdeograph7c80', [], 'Display', None],
  [ 0x7c81, 'CjkUnifiedIdeograph7c81', [], 'Display', None],
  [ 0x7c82, 'CjkUnifiedIdeograph7c82', [], 'Display', None],
  [ 0x7c83, 'CjkUnifiedIdeograph7c83', [], 'Display', None],
  [ 0x7c84, 'CjkUnifiedIdeograph7c84', [], 'Display', None],
  [ 0x7c85, 'CjkUnifiedIdeograph7c85', [], 'Display', None],
  [ 0x7c86, 'CjkUnifiedIdeograph7c86', [], 'Display', None],
  [ 0x7c87, 'CjkUnifiedIdeograph7c87', [], 'Display', None],
  [ 0x7c88, 'CjkUnifiedIdeograph7c88', [], 'Display', None],
  [ 0x7c89, 'CjkUnifiedIdeograph7c89', [], 'Display', None],
  [ 0x7c8a, 'CjkUnifiedIdeograph7c8a', [], 'Display', None],
  [ 0x7c8b, 'CjkUnifiedIdeograph7c8b', [], 'Display', None],
  [ 0x7c8c, 'CjkUnifiedIdeograph7c8c', [], 'Display', None],
  [ 0x7c8d, 'CjkUnifiedIdeograph7c8d', [], 'Display', None],
  [ 0x7c8e, 'CjkUnifiedIdeograph7c8e', [], 'Display', None],
  [ 0x7c8f, 'CjkUnifiedIdeograph7c8f', [], 'Display', None],
  [ 0x7c90, 'CjkUnifiedIdeograph7c90', [], 'Display', None],
  [ 0x7c91, 'CjkUnifiedIdeograph7c91', [], 'Display', None],
  [ 0x7c92, 'CjkUnifiedIdeograph7c92', [], 'Display', None],
  [ 0x7c93, 'CjkUnifiedIdeograph7c93', [], 'Display', None],
  [ 0x7c94, 'CjkUnifiedIdeograph7c94', [], 'Display', None],
  [ 0x7c95, 'CjkUnifiedIdeograph7c95', [], 'Display', None],
  [ 0x7c96, 'CjkUnifiedIdeograph7c96', [], 'Display', None],
  [ 0x7c97, 'CjkUnifiedIdeograph7c97', [], 'Display', None],
  [ 0x7c98, 'CjkUnifiedIdeograph7c98', [], 'Display', None],
  [ 0x7c99, 'CjkUnifiedIdeograph7c99', [], 'Display', None],
  [ 0x7c9a, 'CjkUnifiedIdeograph7c9a', [], 'Display', None],
  [ 0x7c9b, 'CjkUnifiedIdeograph7c9b', [], 'Display', None],
  [ 0x7c9c, 'CjkUnifiedIdeograph7c9c', [], 'Display', None],
  [ 0x7c9d, 'CjkUnifiedIdeograph7c9d', [], 'Display', None],
  [ 0x7c9e, 'CjkUnifiedIdeograph7c9e', [], 'Display', None],
  [ 0x7c9f, 'CjkUnifiedIdeograph7c9f', [], 'Display', None],
  [ 0x7ca0, 'CjkUnifiedIdeograph7ca0', [], 'Display', None],
  [ 0x7ca1, 'CjkUnifiedIdeograph7ca1', [], 'Display', None],
  [ 0x7ca2, 'CjkUnifiedIdeograph7ca2', [], 'Display', None],
  [ 0x7ca3, 'CjkUnifiedIdeograph7ca3', [], 'Display', None],
  [ 0x7ca4, 'CjkUnifiedIdeograph7ca4', [], 'Display', None],
  [ 0x7ca5, 'CjkUnifiedIdeograph7ca5', [], 'Display', None],
  [ 0x7ca6, 'CjkUnifiedIdeograph7ca6', [], 'Display', None],
  [ 0x7ca7, 'CjkUnifiedIdeograph7ca7', [], 'Display', None],
  [ 0x7ca8, 'CjkUnifiedIdeograph7ca8', [], 'Display', None],
  [ 0x7ca9, 'CjkUnifiedIdeograph7ca9', [], 'Display', None],
  [ 0x7caa, 'CjkUnifiedIdeograph7caa', [], 'Display', None],
  [ 0x7cab, 'CjkUnifiedIdeograph7cab', [], 'Display', None],
  [ 0x7cac, 'CjkUnifiedIdeograph7cac', [], 'Display', None],
  [ 0x7cad, 'CjkUnifiedIdeograph7cad', [], 'Display', None],
  [ 0x7cae, 'CjkUnifiedIdeograph7cae', [], 'Display', None],
  [ 0x7caf, 'CjkUnifiedIdeograph7caf', [], 'Display', None],
  [ 0x7cb0, 'CjkUnifiedIdeograph7cb0', [], 'Display', None],
  [ 0x7cb1, 'CjkUnifiedIdeograph7cb1', [], 'Display', None],
  [ 0x7cb2, 'CjkUnifiedIdeograph7cb2', [], 'Display', None],
  [ 0x7cb3, 'CjkUnifiedIdeograph7cb3', [], 'Display', None],
  [ 0x7cb4, 'CjkUnifiedIdeograph7cb4', [], 'Display', None],
  [ 0x7cb5, 'CjkUnifiedIdeograph7cb5', [], 'Display', None],
  [ 0x7cb6, 'CjkUnifiedIdeograph7cb6', [], 'Display', None],
  [ 0x7cb7, 'CjkUnifiedIdeograph7cb7', [], 'Display', None],
  [ 0x7cb8, 'CjkUnifiedIdeograph7cb8', [], 'Display', None],
  [ 0x7cb9, 'CjkUnifiedIdeograph7cb9', [], 'Display', None],
  [ 0x7cba, 'CjkUnifiedIdeograph7cba', [], 'Display', None],
  [ 0x7cbb, 'CjkUnifiedIdeograph7cbb', [], 'Display', None],
  [ 0x7cbc, 'CjkUnifiedIdeograph7cbc', [], 'Display', None],
  [ 0x7cbd, 'CjkUnifiedIdeograph7cbd', [], 'Display', None],
  [ 0x7cbe, 'CjkUnifiedIdeograph7cbe', [], 'Display', None],
  [ 0x7cbf, 'CjkUnifiedIdeograph7cbf', [], 'Display', None],
  [ 0x7cc0, 'CjkUnifiedIdeograph7cc0', [], 'Display', None],
  [ 0x7cc1, 'CjkUnifiedIdeograph7cc1', [], 'Display', None],
  [ 0x7cc2, 'CjkUnifiedIdeograph7cc2', [], 'Display', None],
  [ 0x7cc3, 'CjkUnifiedIdeograph7cc3', [], 'Display', None],
  [ 0x7cc4, 'CjkUnifiedIdeograph7cc4', [], 'Display', None],
  [ 0x7cc5, 'CjkUnifiedIdeograph7cc5', [], 'Display', None],
  [ 0x7cc6, 'CjkUnifiedIdeograph7cc6', [], 'Display', None],
  [ 0x7cc7, 'CjkUnifiedIdeograph7cc7', [], 'Display', None],
  [ 0x7cc8, 'CjkUnifiedIdeograph7cc8', [], 'Display', None],
  [ 0x7cc9, 'CjkUnifiedIdeograph7cc9', [], 'Display', None],
  [ 0x7cca, 'CjkUnifiedIdeograph7cca', [], 'Display', None],
  [ 0x7ccb, 'CjkUnifiedIdeograph7ccb', [], 'Display', None],
  [ 0x7ccc, 'CjkUnifiedIdeograph7ccc', [], 'Display', None],
  [ 0x7ccd, 'CjkUnifiedIdeograph7ccd', [], 'Display', None],
  [ 0x7cce, 'CjkUnifiedIdeograph7cce', [], 'Display', None],
  [ 0x7ccf, 'CjkUnifiedIdeograph7ccf', [], 'Display', None],
  [ 0x7cd0, 'CjkUnifiedIdeograph7cd0', [], 'Display', None],
  [ 0x7cd1, 'CjkUnifiedIdeograph7cd1', [], 'Display', None],
  [ 0x7cd2, 'CjkUnifiedIdeograph7cd2', [], 'Display', None],
  [ 0x7cd3, 'CjkUnifiedIdeograph7cd3', [], 'Display', None],
  [ 0x7cd4, 'CjkUnifiedIdeograph7cd4', [], 'Display', None],
  [ 0x7cd5, 'CjkUnifiedIdeograph7cd5', [], 'Display', None],
  [ 0x7cd6, 'CjkUnifiedIdeograph7cd6', [], 'Display', None],
  [ 0x7cd7, 'CjkUnifiedIdeograph7cd7', [], 'Display', None],
  [ 0x7cd8, 'CjkUnifiedIdeograph7cd8', [], 'Display', None],
  [ 0x7cd9, 'CjkUnifiedIdeograph7cd9', [], 'Display', None],
  [ 0x7cda, 'CjkUnifiedIdeograph7cda', [], 'Display', None],
  [ 0x7cdb, 'CjkUnifiedIdeograph7cdb', [], 'Display', None],
  [ 0x7cdc, 'CjkUnifiedIdeograph7cdc', [], 'Display', None],
  [ 0x7cdd, 'CjkUnifiedIdeograph7cdd', [], 'Display', None],
  [ 0x7cde, 'CjkUnifiedIdeograph7cde', [], 'Display', None],
  [ 0x7cdf, 'CjkUnifiedIdeograph7cdf', [], 'Display', None],
  [ 0x7ce0, 'CjkUnifiedIdeograph7ce0', [], 'Display', None],
  [ 0x7ce1, 'CjkUnifiedIdeograph7ce1', [], 'Display', None],
  [ 0x7ce2, 'CjkUnifiedIdeograph7ce2', [], 'Display', None],
  [ 0x7ce3, 'CjkUnifiedIdeograph7ce3', [], 'Display', None],
  [ 0x7ce4, 'CjkUnifiedIdeograph7ce4', [], 'Display', None],
  [ 0x7ce5, 'CjkUnifiedIdeograph7ce5', [], 'Display', None],
  [ 0x7ce6, 'CjkUnifiedIdeograph7ce6', [], 'Display', None],
  [ 0x7ce7, 'CjkUnifiedIdeograph7ce7', [], 'Display', None],
  [ 0x7ce8, 'CjkUnifiedIdeograph7ce8', [], 'Display', None],
  [ 0x7ce9, 'CjkUnifiedIdeograph7ce9', [], 'Display', None],
  [ 0x7cea, 'CjkUnifiedIdeograph7cea', [], 'Display', None],
  [ 0x7ceb, 'CjkUnifiedIdeograph7ceb', [], 'Display', None],
  [ 0x7cec, 'CjkUnifiedIdeograph7cec', [], 'Display', None],
  [ 0x7ced, 'CjkUnifiedIdeograph7ced', [], 'Display', None],
  [ 0x7cee, 'CjkUnifiedIdeograph7cee', [], 'Display', None],
  [ 0x7cef, 'CjkUnifiedIdeograph7cef', [], 'Display', None],
  [ 0x7cf0, 'CjkUnifiedIdeograph7cf0', [], 'Display', None],
  [ 0x7cf1, 'CjkUnifiedIdeograph7cf1', [], 'Display', None],
  [ 0x7cf2, 'CjkUnifiedIdeograph7cf2', [], 'Display', None],
  [ 0x7cf3, 'CjkUnifiedIdeograph7cf3', [], 'Display', None],
  [ 0x7cf4, 'CjkUnifiedIdeograph7cf4', [], 'Display', None],
  [ 0x7cf5, 'CjkUnifiedIdeograph7cf5', [], 'Display', None],
  [ 0x7cf6, 'CjkUnifiedIdeograph7cf6', [], 'Display', None],
  [ 0x7cf7, 'CjkUnifiedIdeograph7cf7', [], 'Display', None],
  [ 0x7cf8, 'CjkUnifiedIdeograph7cf8', [], 'Display', None],
  [ 0x7cf9, 'CjkUnifiedIdeograph7cf9', [], 'Display', None],
  [ 0x7cfa, 'CjkUnifiedIdeograph7cfa', [], 'Display', None],
  [ 0x7cfb, 'CjkUnifiedIdeograph7cfb', [], 'Display', None],
  [ 0x7cfc, 'CjkUnifiedIdeograph7cfc', [], 'Display', None],
  [ 0x7cfd, 'CjkUnifiedIdeograph7cfd', [], 'Display', None],
  [ 0x7cfe, 'CjkUnifiedIdeograph7cfe', [], 'Display', None],
  [ 0x7cff, 'CjkUnifiedIdeograph7cff', [], 'Display', None],
  [ 0x7d00, 'CjkUnifiedIdeograph7d00', [], 'Display', None],
  [ 0x7d01, 'CjkUnifiedIdeograph7d01', [], 'Display', None],
  [ 0x7d02, 'CjkUnifiedIdeograph7d02', [], 'Display', None],
  [ 0x7d03, 'CjkUnifiedIdeograph7d03', [], 'Display', None],
  [ 0x7d04, 'CjkUnifiedIdeograph7d04', [], 'Display', None],
  [ 0x7d05, 'CjkUnifiedIdeograph7d05', [], 'Display', None],
  [ 0x7d06, 'CjkUnifiedIdeograph7d06', [], 'Display', None],
  [ 0x7d07, 'CjkUnifiedIdeograph7d07', [], 'Display', None],
  [ 0x7d08, 'CjkUnifiedIdeograph7d08', [], 'Display', None],
  [ 0x7d09, 'CjkUnifiedIdeograph7d09', [], 'Display', None],
  [ 0x7d0a, 'CjkUnifiedIdeograph7d0a', [], 'Display', None],
  [ 0x7d0b, 'CjkUnifiedIdeograph7d0b', [], 'Display', None],
  [ 0x7d0c, 'CjkUnifiedIdeograph7d0c', [], 'Display', None],
  [ 0x7d0d, 'CjkUnifiedIdeograph7d0d', [], 'Display', None],
  [ 0x7d0e, 'CjkUnifiedIdeograph7d0e', [], 'Display', None],
  [ 0x7d0f, 'CjkUnifiedIdeograph7d0f', [], 'Display', None],
  [ 0x7d10, 'CjkUnifiedIdeograph7d10', [], 'Display', None],
  [ 0x7d11, 'CjkUnifiedIdeograph7d11', [], 'Display', None],
  [ 0x7d12, 'CjkUnifiedIdeograph7d12', [], 'Display', None],
  [ 0x7d13, 'CjkUnifiedIdeograph7d13', [], 'Display', None],
  [ 0x7d14, 'CjkUnifiedIdeograph7d14', [], 'Display', None],
  [ 0x7d15, 'CjkUnifiedIdeograph7d15', [], 'Display', None],
  [ 0x7d16, 'CjkUnifiedIdeograph7d16', [], 'Display', None],
  [ 0x7d17, 'CjkUnifiedIdeograph7d17', [], 'Display', None],
  [ 0x7d18, 'CjkUnifiedIdeograph7d18', [], 'Display', None],
  [ 0x7d19, 'CjkUnifiedIdeograph7d19', [], 'Display', None],
  [ 0x7d1a, 'CjkUnifiedIdeograph7d1a', [], 'Display', None],
  [ 0x7d1b, 'CjkUnifiedIdeograph7d1b', [], 'Display', None],
  [ 0x7d1c, 'CjkUnifiedIdeograph7d1c', [], 'Display', None],
  [ 0x7d1d, 'CjkUnifiedIdeograph7d1d', [], 'Display', None],
  [ 0x7d1e, 'CjkUnifiedIdeograph7d1e', [], 'Display', None],
  [ 0x7d1f, 'CjkUnifiedIdeograph7d1f', [], 'Display', None],
  [ 0x7d20, 'CjkUnifiedIdeograph7d20', [], 'Display', None],
  [ 0x7d21, 'CjkUnifiedIdeograph7d21', [], 'Display', None],
  [ 0x7d22, 'CjkUnifiedIdeograph7d22', [], 'Display', None],
  [ 0x7d23, 'CjkUnifiedIdeograph7d23', [], 'Display', None],
  [ 0x7d24, 'CjkUnifiedIdeograph7d24', [], 'Display', None],
  [ 0x7d25, 'CjkUnifiedIdeograph7d25', [], 'Display', None],
  [ 0x7d26, 'CjkUnifiedIdeograph7d26', [], 'Display', None],
  [ 0x7d27, 'CjkUnifiedIdeograph7d27', [], 'Display', None],
  [ 0x7d28, 'CjkUnifiedIdeograph7d28', [], 'Display', None],
  [ 0x7d29, 'CjkUnifiedIdeograph7d29', [], 'Display', None],
  [ 0x7d2a, 'CjkUnifiedIdeograph7d2a', [], 'Display', None],
  [ 0x7d2b, 'CjkUnifiedIdeograph7d2b', [], 'Display', None],
  [ 0x7d2c, 'CjkUnifiedIdeograph7d2c', [], 'Display', None],
  [ 0x7d2d, 'CjkUnifiedIdeograph7d2d', [], 'Display', None],
  [ 0x7d2e, 'CjkUnifiedIdeograph7d2e', [], 'Display', None],
  [ 0x7d2f, 'CjkUnifiedIdeograph7d2f', [], 'Display', None],
  [ 0x7d30, 'CjkUnifiedIdeograph7d30', [], 'Display', None],
  [ 0x7d31, 'CjkUnifiedIdeograph7d31', [], 'Display', None],
  [ 0x7d32, 'CjkUnifiedIdeograph7d32', [], 'Display', None],
  [ 0x7d33, 'CjkUnifiedIdeograph7d33', [], 'Display', None],
  [ 0x7d34, 'CjkUnifiedIdeograph7d34', [], 'Display', None],
  [ 0x7d35, 'CjkUnifiedIdeograph7d35', [], 'Display', None],
  [ 0x7d36, 'CjkUnifiedIdeograph7d36', [], 'Display', None],
  [ 0x7d37, 'CjkUnifiedIdeograph7d37', [], 'Display', None],
  [ 0x7d38, 'CjkUnifiedIdeograph7d38', [], 'Display', None],
  [ 0x7d39, 'CjkUnifiedIdeograph7d39', [], 'Display', None],
  [ 0x7d3a, 'CjkUnifiedIdeograph7d3a', [], 'Display', None],
  [ 0x7d3b, 'CjkUnifiedIdeograph7d3b', [], 'Display', None],
  [ 0x7d3c, 'CjkUnifiedIdeograph7d3c', [], 'Display', None],
  [ 0x7d3d, 'CjkUnifiedIdeograph7d3d', [], 'Display', None],
  [ 0x7d3e, 'CjkUnifiedIdeograph7d3e', [], 'Display', None],
  [ 0x7d3f, 'CjkUnifiedIdeograph7d3f', [], 'Display', None],
  [ 0x7d40, 'CjkUnifiedIdeograph7d40', [], 'Display', None],
  [ 0x7d41, 'CjkUnifiedIdeograph7d41', [], 'Display', None],
  [ 0x7d42, 'CjkUnifiedIdeograph7d42', [], 'Display', None],
  [ 0x7d43, 'CjkUnifiedIdeograph7d43', [], 'Display', None],
  [ 0x7d44, 'CjkUnifiedIdeograph7d44', [], 'Display', None],
  [ 0x7d45, 'CjkUnifiedIdeograph7d45', [], 'Display', None],
  [ 0x7d46, 'CjkUnifiedIdeograph7d46', [], 'Display', None],
  [ 0x7d47, 'CjkUnifiedIdeograph7d47', [], 'Display', None],
  [ 0x7d48, 'CjkUnifiedIdeograph7d48', [], 'Display', None],
  [ 0x7d49, 'CjkUnifiedIdeograph7d49', [], 'Display', None],
  [ 0x7d4a, 'CjkUnifiedIdeograph7d4a', [], 'Display', None],
  [ 0x7d4b, 'CjkUnifiedIdeograph7d4b', [], 'Display', None],
  [ 0x7d4c, 'CjkUnifiedIdeograph7d4c', [], 'Display', None],
  [ 0x7d4d, 'CjkUnifiedIdeograph7d4d', [], 'Display', None],
  [ 0x7d4e, 'CjkUnifiedIdeograph7d4e', [], 'Display', None],
  [ 0x7d4f, 'CjkUnifiedIdeograph7d4f', [], 'Display', None],
  [ 0x7d50, 'CjkUnifiedIdeograph7d50', [], 'Display', None],
  [ 0x7d51, 'CjkUnifiedIdeograph7d51', [], 'Display', None],
  [ 0x7d52, 'CjkUnifiedIdeograph7d52', [], 'Display', None],
  [ 0x7d53, 'CjkUnifiedIdeograph7d53', [], 'Display', None],
  [ 0x7d54, 'CjkUnifiedIdeograph7d54', [], 'Display', None],
  [ 0x7d55, 'CjkUnifiedIdeograph7d55', [], 'Display', None],
  [ 0x7d56, 'CjkUnifiedIdeograph7d56', [], 'Display', None],
  [ 0x7d57, 'CjkUnifiedIdeograph7d57', [], 'Display', None],
  [ 0x7d58, 'CjkUnifiedIdeograph7d58', [], 'Display', None],
  [ 0x7d59, 'CjkUnifiedIdeograph7d59', [], 'Display', None],
  [ 0x7d5a, 'CjkUnifiedIdeograph7d5a', [], 'Display', None],
  [ 0x7d5b, 'CjkUnifiedIdeograph7d5b', [], 'Display', None],
  [ 0x7d5c, 'CjkUnifiedIdeograph7d5c', [], 'Display', None],
  [ 0x7d5d, 'CjkUnifiedIdeograph7d5d', [], 'Display', None],
  [ 0x7d5e, 'CjkUnifiedIdeograph7d5e', [], 'Display', None],
  [ 0x7d5f, 'CjkUnifiedIdeograph7d5f', [], 'Display', None],
  [ 0x7d60, 'CjkUnifiedIdeograph7d60', [], 'Display', None],
  [ 0x7d61, 'CjkUnifiedIdeograph7d61', [], 'Display', None],
  [ 0x7d62, 'CjkUnifiedIdeograph7d62', [], 'Display', None],
  [ 0x7d63, 'CjkUnifiedIdeograph7d63', [], 'Display', None],
  [ 0x7d64, 'CjkUnifiedIdeograph7d64', [], 'Display', None],
  [ 0x7d65, 'CjkUnifiedIdeograph7d65', [], 'Display', None],
  [ 0x7d66, 'CjkUnifiedIdeograph7d66', [], 'Display', None],
  [ 0x7d67, 'CjkUnifiedIdeograph7d67', [], 'Display', None],
  [ 0x7d68, 'CjkUnifiedIdeograph7d68', [], 'Display', None],
  [ 0x7d69, 'CjkUnifiedIdeograph7d69', [], 'Display', None],
  [ 0x7d6a, 'CjkUnifiedIdeograph7d6a', [], 'Display', None],
  [ 0x7d6b, 'CjkUnifiedIdeograph7d6b', [], 'Display', None],
  [ 0x7d6c, 'CjkUnifiedIdeograph7d6c', [], 'Display', None],
  [ 0x7d6d, 'CjkUnifiedIdeograph7d6d', [], 'Display', None],
  [ 0x7d6e, 'CjkUnifiedIdeograph7d6e', [], 'Display', None],
  [ 0x7d6f, 'CjkUnifiedIdeograph7d6f', [], 'Display', None],
  [ 0x7d70, 'CjkUnifiedIdeograph7d70', [], 'Display', None],
  [ 0x7d71, 'CjkUnifiedIdeograph7d71', [], 'Display', None],
  [ 0x7d72, 'CjkUnifiedIdeograph7d72', [], 'Display', None],
  [ 0x7d73, 'CjkUnifiedIdeograph7d73', [], 'Display', None],
  [ 0x7d74, 'CjkUnifiedIdeograph7d74', [], 'Display', None],
  [ 0x7d75, 'CjkUnifiedIdeograph7d75', [], 'Display', None],
  [ 0x7d76, 'CjkUnifiedIdeograph7d76', [], 'Display', None],
  [ 0x7d77, 'CjkUnifiedIdeograph7d77', [], 'Display', None],
  [ 0x7d78, 'CjkUnifiedIdeograph7d78', [], 'Display', None],
  [ 0x7d79, 'CjkUnifiedIdeograph7d79', [], 'Display', None],
  [ 0x7d7a, 'CjkUnifiedIdeograph7d7a', [], 'Display', None],
  [ 0x7d7b, 'CjkUnifiedIdeograph7d7b', [], 'Display', None],
  [ 0x7d7c, 'CjkUnifiedIdeograph7d7c', [], 'Display', None],
  [ 0x7d7d, 'CjkUnifiedIdeograph7d7d', [], 'Display', None],
  [ 0x7d7e, 'CjkUnifiedIdeograph7d7e', [], 'Display', None],
  [ 0x7d7f, 'CjkUnifiedIdeograph7d7f', [], 'Display', None],
  [ 0x7d80, 'CjkUnifiedIdeograph7d80', [], 'Display', None],
  [ 0x7d81, 'CjkUnifiedIdeograph7d81', [], 'Display', None],
  [ 0x7d82, 'CjkUnifiedIdeograph7d82', [], 'Display', None],
  [ 0x7d83, 'CjkUnifiedIdeograph7d83', [], 'Display', None],
  [ 0x7d84, 'CjkUnifiedIdeograph7d84', [], 'Display', None],
  [ 0x7d85, 'CjkUnifiedIdeograph7d85', [], 'Display', None],
  [ 0x7d86, 'CjkUnifiedIdeograph7d86', [], 'Display', None],
  [ 0x7d87, 'CjkUnifiedIdeograph7d87', [], 'Display', None],
  [ 0x7d88, 'CjkUnifiedIdeograph7d88', [], 'Display', None],
  [ 0x7d89, 'CjkUnifiedIdeograph7d89', [], 'Display', None],
  [ 0x7d8a, 'CjkUnifiedIdeograph7d8a', [], 'Display', None],
  [ 0x7d8b, 'CjkUnifiedIdeograph7d8b', [], 'Display', None],
  [ 0x7d8c, 'CjkUnifiedIdeograph7d8c', [], 'Display', None],
  [ 0x7d8d, 'CjkUnifiedIdeograph7d8d', [], 'Display', None],
  [ 0x7d8e, 'CjkUnifiedIdeograph7d8e', [], 'Display', None],
  [ 0x7d8f, 'CjkUnifiedIdeograph7d8f', [], 'Display', None],
  [ 0x7d90, 'CjkUnifiedIdeograph7d90', [], 'Display', None],
  [ 0x7d91, 'CjkUnifiedIdeograph7d91', [], 'Display', None],
  [ 0x7d92, 'CjkUnifiedIdeograph7d92', [], 'Display', None],
  [ 0x7d93, 'CjkUnifiedIdeograph7d93', [], 'Display', None],
  [ 0x7d94, 'CjkUnifiedIdeograph7d94', [], 'Display', None],
  [ 0x7d95, 'CjkUnifiedIdeograph7d95', [], 'Display', None],
  [ 0x7d96, 'CjkUnifiedIdeograph7d96', [], 'Display', None],
  [ 0x7d97, 'CjkUnifiedIdeograph7d97', [], 'Display', None],
  [ 0x7d98, 'CjkUnifiedIdeograph7d98', [], 'Display', None],
  [ 0x7d99, 'CjkUnifiedIdeograph7d99', [], 'Display', None],
  [ 0x7d9a, 'CjkUnifiedIdeograph7d9a', [], 'Display', None],
  [ 0x7d9b, 'CjkUnifiedIdeograph7d9b', [], 'Display', None],
  [ 0x7d9c, 'CjkUnifiedIdeograph7d9c', [], 'Display', None],
  [ 0x7d9d, 'CjkUnifiedIdeograph7d9d', [], 'Display', None],
  [ 0x7d9e, 'CjkUnifiedIdeograph7d9e', [], 'Display', None],
  [ 0x7d9f, 'CjkUnifiedIdeograph7d9f', [], 'Display', None],
  [ 0x7da0, 'CjkUnifiedIdeograph7da0', [], 'Display', None],
  [ 0x7da1, 'CjkUnifiedIdeograph7da1', [], 'Display', None],
  [ 0x7da2, 'CjkUnifiedIdeograph7da2', [], 'Display', None],
  [ 0x7da3, 'CjkUnifiedIdeograph7da3', [], 'Display', None],
  [ 0x7da4, 'CjkUnifiedIdeograph7da4', [], 'Display', None],
  [ 0x7da5, 'CjkUnifiedIdeograph7da5', [], 'Display', None],
  [ 0x7da6, 'CjkUnifiedIdeograph7da6', [], 'Display', None],
  [ 0x7da7, 'CjkUnifiedIdeograph7da7', [], 'Display', None],
  [ 0x7da8, 'CjkUnifiedIdeograph7da8', [], 'Display', None],
  [ 0x7da9, 'CjkUnifiedIdeograph7da9', [], 'Display', None],
  [ 0x7daa, 'CjkUnifiedIdeograph7daa', [], 'Display', None],
  [ 0x7dab, 'CjkUnifiedIdeograph7dab', [], 'Display', None],
  [ 0x7dac, 'CjkUnifiedIdeograph7dac', [], 'Display', None],
  [ 0x7dad, 'CjkUnifiedIdeograph7dad', [], 'Display', None],
  [ 0x7dae, 'CjkUnifiedIdeograph7dae', [], 'Display', None],
  [ 0x7daf, 'CjkUnifiedIdeograph7daf', [], 'Display', None],
  [ 0x7db0, 'CjkUnifiedIdeograph7db0', [], 'Display', None],
  [ 0x7db1, 'CjkUnifiedIdeograph7db1', [], 'Display', None],
  [ 0x7db2, 'CjkUnifiedIdeograph7db2', [], 'Display', None],
  [ 0x7db3, 'CjkUnifiedIdeograph7db3', [], 'Display', None],
  [ 0x7db4, 'CjkUnifiedIdeograph7db4', [], 'Display', None],
  [ 0x7db5, 'CjkUnifiedIdeograph7db5', [], 'Display', None],
  [ 0x7db6, 'CjkUnifiedIdeograph7db6', [], 'Display', None],
  [ 0x7db7, 'CjkUnifiedIdeograph7db7', [], 'Display', None],
  [ 0x7db8, 'CjkUnifiedIdeograph7db8', [], 'Display', None],
  [ 0x7db9, 'CjkUnifiedIdeograph7db9', [], 'Display', None],
  [ 0x7dba, 'CjkUnifiedIdeograph7dba', [], 'Display', None],
  [ 0x7dbb, 'CjkUnifiedIdeograph7dbb', [], 'Display', None],
  [ 0x7dbc, 'CjkUnifiedIdeograph7dbc', [], 'Display', None],
  [ 0x7dbd, 'CjkUnifiedIdeograph7dbd', [], 'Display', None],
  [ 0x7dbe, 'CjkUnifiedIdeograph7dbe', [], 'Display', None],
  [ 0x7dbf, 'CjkUnifiedIdeograph7dbf', [], 'Display', None],
  [ 0x7dc0, 'CjkUnifiedIdeograph7dc0', [], 'Display', None],
  [ 0x7dc1, 'CjkUnifiedIdeograph7dc1', [], 'Display', None],
  [ 0x7dc2, 'CjkUnifiedIdeograph7dc2', [], 'Display', None],
  [ 0x7dc3, 'CjkUnifiedIdeograph7dc3', [], 'Display', None],
  [ 0x7dc4, 'CjkUnifiedIdeograph7dc4', [], 'Display', None],
  [ 0x7dc5, 'CjkUnifiedIdeograph7dc5', [], 'Display', None],
  [ 0x7dc6, 'CjkUnifiedIdeograph7dc6', [], 'Display', None],
  [ 0x7dc7, 'CjkUnifiedIdeograph7dc7', [], 'Display', None],
  [ 0x7dc8, 'CjkUnifiedIdeograph7dc8', [], 'Display', None],
  [ 0x7dc9, 'CjkUnifiedIdeograph7dc9', [], 'Display', None],
  [ 0x7dca, 'CjkUnifiedIdeograph7dca', [], 'Display', None],
  [ 0x7dcb, 'CjkUnifiedIdeograph7dcb', [], 'Display', None],
  [ 0x7dcc, 'CjkUnifiedIdeograph7dcc', [], 'Display', None],
  [ 0x7dcd, 'CjkUnifiedIdeograph7dcd', [], 'Display', None],
  [ 0x7dce, 'CjkUnifiedIdeograph7dce', [], 'Display', None],
  [ 0x7dcf, 'CjkUnifiedIdeograph7dcf', [], 'Display', None],
  [ 0x7dd0, 'CjkUnifiedIdeograph7dd0', [], 'Display', None],
  [ 0x7dd1, 'CjkUnifiedIdeograph7dd1', [], 'Display', None],
  [ 0x7dd2, 'CjkUnifiedIdeograph7dd2', [], 'Display', None],
  [ 0x7dd3, 'CjkUnifiedIdeograph7dd3', [], 'Display', None],
  [ 0x7dd4, 'CjkUnifiedIdeograph7dd4', [], 'Display', None],
  [ 0x7dd5, 'CjkUnifiedIdeograph7dd5', [], 'Display', None],
  [ 0x7dd6, 'CjkUnifiedIdeograph7dd6', [], 'Display', None],
  [ 0x7dd7, 'CjkUnifiedIdeograph7dd7', [], 'Display', None],
  [ 0x7dd8, 'CjkUnifiedIdeograph7dd8', [], 'Display', None],
  [ 0x7dd9, 'CjkUnifiedIdeograph7dd9', [], 'Display', None],
  [ 0x7dda, 'CjkUnifiedIdeograph7dda', [], 'Display', None],
  [ 0x7ddb, 'CjkUnifiedIdeograph7ddb', [], 'Display', None],
  [ 0x7ddc, 'CjkUnifiedIdeograph7ddc', [], 'Display', None],
  [ 0x7ddd, 'CjkUnifiedIdeograph7ddd', [], 'Display', None],
  [ 0x7dde, 'CjkUnifiedIdeograph7dde', [], 'Display', None],
  [ 0x7ddf, 'CjkUnifiedIdeograph7ddf', [], 'Display', None],
  [ 0x7de0, 'CjkUnifiedIdeograph7de0', [], 'Display', None],
  [ 0x7de1, 'CjkUnifiedIdeograph7de1', [], 'Display', None],
  [ 0x7de2, 'CjkUnifiedIdeograph7de2', [], 'Display', None],
  [ 0x7de3, 'CjkUnifiedIdeograph7de3', [], 'Display', None],
  [ 0x7de4, 'CjkUnifiedIdeograph7de4', [], 'Display', None],
  [ 0x7de5, 'CjkUnifiedIdeograph7de5', [], 'Display', None],
  [ 0x7de6, 'CjkUnifiedIdeograph7de6', [], 'Display', None],
  [ 0x7de7, 'CjkUnifiedIdeograph7de7', [], 'Display', None],
  [ 0x7de8, 'CjkUnifiedIdeograph7de8', [], 'Display', None],
  [ 0x7de9, 'CjkUnifiedIdeograph7de9', [], 'Display', None],
  [ 0x7dea, 'CjkUnifiedIdeograph7dea', [], 'Display', None],
  [ 0x7deb, 'CjkUnifiedIdeograph7deb', [], 'Display', None],
  [ 0x7dec, 'CjkUnifiedIdeograph7dec', [], 'Display', None],
  [ 0x7ded, 'CjkUnifiedIdeograph7ded', [], 'Display', None],
  [ 0x7dee, 'CjkUnifiedIdeograph7dee', [], 'Display', None],
  [ 0x7def, 'CjkUnifiedIdeograph7def', [], 'Display', None],
  [ 0x7df0, 'CjkUnifiedIdeograph7df0', [], 'Display', None],
  [ 0x7df1, 'CjkUnifiedIdeograph7df1', [], 'Display', None],
  [ 0x7df2, 'CjkUnifiedIdeograph7df2', [], 'Display', None],
  [ 0x7df3, 'CjkUnifiedIdeograph7df3', [], 'Display', None],
  [ 0x7df4, 'CjkUnifiedIdeograph7df4', [], 'Display', None],
  [ 0x7df5, 'CjkUnifiedIdeograph7df5', [], 'Display', None],
  [ 0x7df6, 'CjkUnifiedIdeograph7df6', [], 'Display', None],
  [ 0x7df7, 'CjkUnifiedIdeograph7df7', [], 'Display', None],
  [ 0x7df8, 'CjkUnifiedIdeograph7df8', [], 'Display', None],
  [ 0x7df9, 'CjkUnifiedIdeograph7df9', [], 'Display', None],
  [ 0x7dfa, 'CjkUnifiedIdeograph7dfa', [], 'Display', None],
  [ 0x7dfb, 'CjkUnifiedIdeograph7dfb', [], 'Display', None],
  [ 0x7dfc, 'CjkUnifiedIdeograph7dfc', [], 'Display', None],
  [ 0x7dfd, 'CjkUnifiedIdeograph7dfd', [], 'Display', None],
  [ 0x7dfe, 'CjkUnifiedIdeograph7dfe', [], 'Display', None],
  [ 0x7dff, 'CjkUnifiedIdeograph7dff', [], 'Display', None],
  [ 0x7e00, 'CjkUnifiedIdeograph7e00', [], 'Display', None],
  [ 0x7e01, 'CjkUnifiedIdeograph7e01', [], 'Display', None],
  [ 0x7e02, 'CjkUnifiedIdeograph7e02', [], 'Display', None],
  [ 0x7e03, 'CjkUnifiedIdeograph7e03', [], 'Display', None],
  [ 0x7e04, 'CjkUnifiedIdeograph7e04', [], 'Display', None],
  [ 0x7e05, 'CjkUnifiedIdeograph7e05', [], 'Display', None],
  [ 0x7e06, 'CjkUnifiedIdeograph7e06', [], 'Display', None],
  [ 0x7e07, 'CjkUnifiedIdeograph7e07', [], 'Display', None],
  [ 0x7e08, 'CjkUnifiedIdeograph7e08', [], 'Display', None],
  [ 0x7e09, 'CjkUnifiedIdeograph7e09', [], 'Display', None],
  [ 0x7e0a, 'CjkUnifiedIdeograph7e0a', [], 'Display', None],
  [ 0x7e0b, 'CjkUnifiedIdeograph7e0b', [], 'Display', None],
  [ 0x7e0c, 'CjkUnifiedIdeograph7e0c', [], 'Display', None],
  [ 0x7e0d, 'CjkUnifiedIdeograph7e0d', [], 'Display', None],
  [ 0x7e0e, 'CjkUnifiedIdeograph7e0e', [], 'Display', None],
  [ 0x7e0f, 'CjkUnifiedIdeograph7e0f', [], 'Display', None],
  [ 0x7e10, 'CjkUnifiedIdeograph7e10', [], 'Display', None],
  [ 0x7e11, 'CjkUnifiedIdeograph7e11', [], 'Display', None],
  [ 0x7e12, 'CjkUnifiedIdeograph7e12', [], 'Display', None],
  [ 0x7e13, 'CjkUnifiedIdeograph7e13', [], 'Display', None],
  [ 0x7e14, 'CjkUnifiedIdeograph7e14', [], 'Display', None],
  [ 0x7e15, 'CjkUnifiedIdeograph7e15', [], 'Display', None],
  [ 0x7e16, 'CjkUnifiedIdeograph7e16', [], 'Display', None],
  [ 0x7e17, 'CjkUnifiedIdeograph7e17', [], 'Display', None],
  [ 0x7e18, 'CjkUnifiedIdeograph7e18', [], 'Display', None],
  [ 0x7e19, 'CjkUnifiedIdeograph7e19', [], 'Display', None],
  [ 0x7e1a, 'CjkUnifiedIdeograph7e1a', [], 'Display', None],
  [ 0x7e1b, 'CjkUnifiedIdeograph7e1b', [], 'Display', None],
  [ 0x7e1c, 'CjkUnifiedIdeograph7e1c', [], 'Display', None],
  [ 0x7e1d, 'CjkUnifiedIdeograph7e1d', [], 'Display', None],
  [ 0x7e1e, 'CjkUnifiedIdeograph7e1e', [], 'Display', None],
  [ 0x7e1f, 'CjkUnifiedIdeograph7e1f', [], 'Display', None],
  [ 0x7e20, 'CjkUnifiedIdeograph7e20', [], 'Display', None],
  [ 0x7e21, 'CjkUnifiedIdeograph7e21', [], 'Display', None],
  [ 0x7e22, 'CjkUnifiedIdeograph7e22', [], 'Display', None],
  [ 0x7e23, 'CjkUnifiedIdeograph7e23', [], 'Display', None],
  [ 0x7e24, 'CjkUnifiedIdeograph7e24', [], 'Display', None],
  [ 0x7e25, 'CjkUnifiedIdeograph7e25', [], 'Display', None],
  [ 0x7e26, 'CjkUnifiedIdeograph7e26', [], 'Display', None],
  [ 0x7e27, 'CjkUnifiedIdeograph7e27', [], 'Display', None],
  [ 0x7e28, 'CjkUnifiedIdeograph7e28', [], 'Display', None],
  [ 0x7e29, 'CjkUnifiedIdeograph7e29', [], 'Display', None],
  [ 0x7e2a, 'CjkUnifiedIdeograph7e2a', [], 'Display', None],
  [ 0x7e2b, 'CjkUnifiedIdeograph7e2b', [], 'Display', None],
  [ 0x7e2c, 'CjkUnifiedIdeograph7e2c', [], 'Display', None],
  [ 0x7e2d, 'CjkUnifiedIdeograph7e2d', [], 'Display', None],
  [ 0x7e2e, 'CjkUnifiedIdeograph7e2e', [], 'Display', None],
  [ 0x7e2f, 'CjkUnifiedIdeograph7e2f', [], 'Display', None],
  [ 0x7e30, 'CjkUnifiedIdeograph7e30', [], 'Display', None],
  [ 0x7e31, 'CjkUnifiedIdeograph7e31', [], 'Display', None],
  [ 0x7e32, 'CjkUnifiedIdeograph7e32', [], 'Display', None],
  [ 0x7e33, 'CjkUnifiedIdeograph7e33', [], 'Display', None],
  [ 0x7e34, 'CjkUnifiedIdeograph7e34', [], 'Display', None],
  [ 0x7e35, 'CjkUnifiedIdeograph7e35', [], 'Display', None],
  [ 0x7e36, 'CjkUnifiedIdeograph7e36', [], 'Display', None],
  [ 0x7e37, 'CjkUnifiedIdeograph7e37', [], 'Display', None],
  [ 0x7e38, 'CjkUnifiedIdeograph7e38', [], 'Display', None],
  [ 0x7e39, 'CjkUnifiedIdeograph7e39', [], 'Display', None],
  [ 0x7e3a, 'CjkUnifiedIdeograph7e3a', [], 'Display', None],
  [ 0x7e3b, 'CjkUnifiedIdeograph7e3b', [], 'Display', None],
  [ 0x7e3c, 'CjkUnifiedIdeograph7e3c', [], 'Display', None],
  [ 0x7e3d, 'CjkUnifiedIdeograph7e3d', [], 'Display', None],
  [ 0x7e3e, 'CjkUnifiedIdeograph7e3e', [], 'Display', None],
  [ 0x7e3f, 'CjkUnifiedIdeograph7e3f', [], 'Display', None],
  [ 0x7e40, 'CjkUnifiedIdeograph7e40', [], 'Display', None],
  [ 0x7e41, 'CjkUnifiedIdeograph7e41', [], 'Display', None],
  [ 0x7e42, 'CjkUnifiedIdeograph7e42', [], 'Display', None],
  [ 0x7e43, 'CjkUnifiedIdeograph7e43', [], 'Display', None],
  [ 0x7e44, 'CjkUnifiedIdeograph7e44', [], 'Display', None],
  [ 0x7e45, 'CjkUnifiedIdeograph7e45', [], 'Display', None],
  [ 0x7e46, 'CjkUnifiedIdeograph7e46', [], 'Display', None],
  [ 0x7e47, 'CjkUnifiedIdeograph7e47', [], 'Display', None],
  [ 0x7e48, 'CjkUnifiedIdeograph7e48', [], 'Display', None],
  [ 0x7e49, 'CjkUnifiedIdeograph7e49', [], 'Display', None],
  [ 0x7e4a, 'CjkUnifiedIdeograph7e4a', [], 'Display', None],
  [ 0x7e4b, 'CjkUnifiedIdeograph7e4b', [], 'Display', None],
  [ 0x7e4c, 'CjkUnifiedIdeograph7e4c', [], 'Display', None],
  [ 0x7e4d, 'CjkUnifiedIdeograph7e4d', [], 'Display', None],
  [ 0x7e4e, 'CjkUnifiedIdeograph7e4e', [], 'Display', None],
  [ 0x7e4f, 'CjkUnifiedIdeograph7e4f', [], 'Display', None],
  [ 0x7e50, 'CjkUnifiedIdeograph7e50', [], 'Display', None],
  [ 0x7e51, 'CjkUnifiedIdeograph7e51', [], 'Display', None],
  [ 0x7e52, 'CjkUnifiedIdeograph7e52', [], 'Display', None],
  [ 0x7e53, 'CjkUnifiedIdeograph7e53', [], 'Display', None],
  [ 0x7e54, 'CjkUnifiedIdeograph7e54', [], 'Display', None],
  [ 0x7e55, 'CjkUnifiedIdeograph7e55', [], 'Display', None],
  [ 0x7e56, 'CjkUnifiedIdeograph7e56', [], 'Display', None],
  [ 0x7e57, 'CjkUnifiedIdeograph7e57', [], 'Display', None],
  [ 0x7e58, 'CjkUnifiedIdeograph7e58', [], 'Display', None],
  [ 0x7e59, 'CjkUnifiedIdeograph7e59', [], 'Display', None],
  [ 0x7e5a, 'CjkUnifiedIdeograph7e5a', [], 'Display', None],
  [ 0x7e5b, 'CjkUnifiedIdeograph7e5b', [], 'Display', None],
  [ 0x7e5c, 'CjkUnifiedIdeograph7e5c', [], 'Display', None],
  [ 0x7e5d, 'CjkUnifiedIdeograph7e5d', [], 'Display', None],
  [ 0x7e5e, 'CjkUnifiedIdeograph7e5e', [], 'Display', None],
  [ 0x7e5f, 'CjkUnifiedIdeograph7e5f', [], 'Display', None],
  [ 0x7e60, 'CjkUnifiedIdeograph7e60', [], 'Display', None],
  [ 0x7e61, 'CjkUnifiedIdeograph7e61', [], 'Display', None],
  [ 0x7e62, 'CjkUnifiedIdeograph7e62', [], 'Display', None],
  [ 0x7e63, 'CjkUnifiedIdeograph7e63', [], 'Display', None],
  [ 0x7e64, 'CjkUnifiedIdeograph7e64', [], 'Display', None],
  [ 0x7e65, 'CjkUnifiedIdeograph7e65', [], 'Display', None],
  [ 0x7e66, 'CjkUnifiedIdeograph7e66', [], 'Display', None],
  [ 0x7e67, 'CjkUnifiedIdeograph7e67', [], 'Display', None],
  [ 0x7e68, 'CjkUnifiedIdeograph7e68', [], 'Display', None],
  [ 0x7e69, 'CjkUnifiedIdeograph7e69', [], 'Display', None],
  [ 0x7e6a, 'CjkUnifiedIdeograph7e6a', [], 'Display', None],
  [ 0x7e6b, 'CjkUnifiedIdeograph7e6b', [], 'Display', None],
  [ 0x7e6c, 'CjkUnifiedIdeograph7e6c', [], 'Display', None],
  [ 0x7e6d, 'CjkUnifiedIdeograph7e6d', [], 'Display', None],
  [ 0x7e6e, 'CjkUnifiedIdeograph7e6e', [], 'Display', None],
  [ 0x7e6f, 'CjkUnifiedIdeograph7e6f', [], 'Display', None],
  [ 0x7e70, 'CjkUnifiedIdeograph7e70', [], 'Display', None],
  [ 0x7e71, 'CjkUnifiedIdeograph7e71', [], 'Display', None],
  [ 0x7e72, 'CjkUnifiedIdeograph7e72', [], 'Display', None],
  [ 0x7e73, 'CjkUnifiedIdeograph7e73', [], 'Display', None],
  [ 0x7e74, 'CjkUnifiedIdeograph7e74', [], 'Display', None],
  [ 0x7e75, 'CjkUnifiedIdeograph7e75', [], 'Display', None],
  [ 0x7e76, 'CjkUnifiedIdeograph7e76', [], 'Display', None],
  [ 0x7e77, 'CjkUnifiedIdeograph7e77', [], 'Display', None],
  [ 0x7e78, 'CjkUnifiedIdeograph7e78', [], 'Display', None],
  [ 0x7e79, 'CjkUnifiedIdeograph7e79', [], 'Display', None],
  [ 0x7e7a, 'CjkUnifiedIdeograph7e7a', [], 'Display', None],
  [ 0x7e7b, 'CjkUnifiedIdeograph7e7b', [], 'Display', None],
  [ 0x7e7c, 'CjkUnifiedIdeograph7e7c', [], 'Display', None],
  [ 0x7e7d, 'CjkUnifiedIdeograph7e7d', [], 'Display', None],
  [ 0x7e7e, 'CjkUnifiedIdeograph7e7e', [], 'Display', None],
  [ 0x7e7f, 'CjkUnifiedIdeograph7e7f', [], 'Display', None],
  [ 0x7e80, 'CjkUnifiedIdeograph7e80', [], 'Display', None],
  [ 0x7e81, 'CjkUnifiedIdeograph7e81', [], 'Display', None],
  [ 0x7e82, 'CjkUnifiedIdeograph7e82', [], 'Display', None],
  [ 0x7e83, 'CjkUnifiedIdeograph7e83', [], 'Display', None],
  [ 0x7e84, 'CjkUnifiedIdeograph7e84', [], 'Display', None],
  [ 0x7e85, 'CjkUnifiedIdeograph7e85', [], 'Display', None],
  [ 0x7e86, 'CjkUnifiedIdeograph7e86', [], 'Display', None],
  [ 0x7e87, 'CjkUnifiedIdeograph7e87', [], 'Display', None],
  [ 0x7e88, 'CjkUnifiedIdeograph7e88', [], 'Display', None],
  [ 0x7e89, 'CjkUnifiedIdeograph7e89', [], 'Display', None],
  [ 0x7e8a, 'CjkUnifiedIdeograph7e8a', [], 'Display', None],
  [ 0x7e8b, 'CjkUnifiedIdeograph7e8b', [], 'Display', None],
  [ 0x7e8c, 'CjkUnifiedIdeograph7e8c', [], 'Display', None],
  [ 0x7e8d, 'CjkUnifiedIdeograph7e8d', [], 'Display', None],
  [ 0x7e8e, 'CjkUnifiedIdeograph7e8e', [], 'Display', None],
  [ 0x7e8f, 'CjkUnifiedIdeograph7e8f', [], 'Display', None],
  [ 0x7e90, 'CjkUnifiedIdeograph7e90', [], 'Display', None],
  [ 0x7e91, 'CjkUnifiedIdeograph7e91', [], 'Display', None],
  [ 0x7e92, 'CjkUnifiedIdeograph7e92', [], 'Display', None],
  [ 0x7e93, 'CjkUnifiedIdeograph7e93', [], 'Display', None],
  [ 0x7e94, 'CjkUnifiedIdeograph7e94', [], 'Display', None],
  [ 0x7e95, 'CjkUnifiedIdeograph7e95', [], 'Display', None],
  [ 0x7e96, 'CjkUnifiedIdeograph7e96', [], 'Display', None],
  [ 0x7e97, 'CjkUnifiedIdeograph7e97', [], 'Display', None],
  [ 0x7e98, 'CjkUnifiedIdeograph7e98', [], 'Display', None],
  [ 0x7e99, 'CjkUnifiedIdeograph7e99', [], 'Display', None],
  [ 0x7e9a, 'CjkUnifiedIdeograph7e9a', [], 'Display', None],
  [ 0x7e9b, 'CjkUnifiedIdeograph7e9b', [], 'Display', None],
  [ 0x7e9c, 'CjkUnifiedIdeograph7e9c', [], 'Display', None],
  [ 0x7e9d, 'CjkUnifiedIdeograph7e9d', [], 'Display', None],
  [ 0x7e9e, 'CjkUnifiedIdeograph7e9e', [], 'Display', None],
  [ 0x7e9f, 'CjkUnifiedIdeograph7e9f', [], 'Display', None],
  [ 0x7ea0, 'CjkUnifiedIdeograph7ea0', [], 'Display', None],
  [ 0x7ea1, 'CjkUnifiedIdeograph7ea1', [], 'Display', None],
  [ 0x7ea2, 'CjkUnifiedIdeograph7ea2', [], 'Display', None],
  [ 0x7ea3, 'CjkUnifiedIdeograph7ea3', [], 'Display', None],
  [ 0x7ea4, 'CjkUnifiedIdeograph7ea4', [], 'Display', None],
  [ 0x7ea5, 'CjkUnifiedIdeograph7ea5', [], 'Display', None],
  [ 0x7ea6, 'CjkUnifiedIdeograph7ea6', [], 'Display', None],
  [ 0x7ea7, 'CjkUnifiedIdeograph7ea7', [], 'Display', None],
  [ 0x7ea8, 'CjkUnifiedIdeograph7ea8', [], 'Display', None],
  [ 0x7ea9, 'CjkUnifiedIdeograph7ea9', [], 'Display', None],
  [ 0x7eaa, 'CjkUnifiedIdeograph7eaa', [], 'Display', None],
  [ 0x7eab, 'CjkUnifiedIdeograph7eab', [], 'Display', None],
  [ 0x7eac, 'CjkUnifiedIdeograph7eac', [], 'Display', None],
  [ 0x7ead, 'CjkUnifiedIdeograph7ead', [], 'Display', None],
  [ 0x7eae, 'CjkUnifiedIdeograph7eae', [], 'Display', None],
  [ 0x7eaf, 'CjkUnifiedIdeograph7eaf', [], 'Display', None],
  [ 0x7eb0, 'CjkUnifiedIdeograph7eb0', [], 'Display', None],
  [ 0x7eb1, 'CjkUnifiedIdeograph7eb1', [], 'Display', None],
  [ 0x7eb2, 'CjkUnifiedIdeograph7eb2', [], 'Display', None],
  [ 0x7eb3, 'CjkUnifiedIdeograph7eb3', [], 'Display', None],
  [ 0x7eb4, 'CjkUnifiedIdeograph7eb4', [], 'Display', None],
  [ 0x7eb5, 'CjkUnifiedIdeograph7eb5', [], 'Display', None],
  [ 0x7eb6, 'CjkUnifiedIdeograph7eb6', [], 'Display', None],
  [ 0x7eb7, 'CjkUnifiedIdeograph7eb7', [], 'Display', None],
  [ 0x7eb8, 'CjkUnifiedIdeograph7eb8', [], 'Display', None],
  [ 0x7eb9, 'CjkUnifiedIdeograph7eb9', [], 'Display', None],
  [ 0x7eba, 'CjkUnifiedIdeograph7eba', [], 'Display', None],
  [ 0x7ebb, 'CjkUnifiedIdeograph7ebb', [], 'Display', None],
  [ 0x7ebc, 'CjkUnifiedIdeograph7ebc', [], 'Display', None],
  [ 0x7ebd, 'CjkUnifiedIdeograph7ebd', [], 'Display', None],
  [ 0x7ebe, 'CjkUnifiedIdeograph7ebe', [], 'Display', None],
  [ 0x7ebf, 'CjkUnifiedIdeograph7ebf', [], 'Display', None],
  [ 0x7ec0, 'CjkUnifiedIdeograph7ec0', [], 'Display', None],
  [ 0x7ec1, 'CjkUnifiedIdeograph7ec1', [], 'Display', None],
  [ 0x7ec2, 'CjkUnifiedIdeograph7ec2', [], 'Display', None],
  [ 0x7ec3, 'CjkUnifiedIdeograph7ec3', [], 'Display', None],
  [ 0x7ec4, 'CjkUnifiedIdeograph7ec4', [], 'Display', None],
  [ 0x7ec5, 'CjkUnifiedIdeograph7ec5', [], 'Display', None],
  [ 0x7ec6, 'CjkUnifiedIdeograph7ec6', [], 'Display', None],
  [ 0x7ec7, 'CjkUnifiedIdeograph7ec7', [], 'Display', None],
  [ 0x7ec8, 'CjkUnifiedIdeograph7ec8', [], 'Display', None],
  [ 0x7ec9, 'CjkUnifiedIdeograph7ec9', [], 'Display', None],
  [ 0x7eca, 'CjkUnifiedIdeograph7eca', [], 'Display', None],
  [ 0x7ecb, 'CjkUnifiedIdeograph7ecb', [], 'Display', None],
  [ 0x7ecc, 'CjkUnifiedIdeograph7ecc', [], 'Display', None],
  [ 0x7ecd, 'CjkUnifiedIdeograph7ecd', [], 'Display', None],
  [ 0x7ece, 'CjkUnifiedIdeograph7ece', [], 'Display', None],
  [ 0x7ecf, 'CjkUnifiedIdeograph7ecf', [], 'Display', None],
  [ 0x7ed0, 'CjkUnifiedIdeograph7ed0', [], 'Display', None],
  [ 0x7ed1, 'CjkUnifiedIdeograph7ed1', [], 'Display', None],
  [ 0x7ed2, 'CjkUnifiedIdeograph7ed2', [], 'Display', None],
  [ 0x7ed3, 'CjkUnifiedIdeograph7ed3', [], 'Display', None],
  [ 0x7ed4, 'CjkUnifiedIdeograph7ed4', [], 'Display', None],
  [ 0x7ed5, 'CjkUnifiedIdeograph7ed5', [], 'Display', None],
  [ 0x7ed6, 'CjkUnifiedIdeograph7ed6', [], 'Display', None],
  [ 0x7ed7, 'CjkUnifiedIdeograph7ed7', [], 'Display', None],
  [ 0x7ed8, 'CjkUnifiedIdeograph7ed8', [], 'Display', None],
  [ 0x7ed9, 'CjkUnifiedIdeograph7ed9', [], 'Display', None],
  [ 0x7eda, 'CjkUnifiedIdeograph7eda', [], 'Display', None],
  [ 0x7edb, 'CjkUnifiedIdeograph7edb', [], 'Display', None],
  [ 0x7edc, 'CjkUnifiedIdeograph7edc', [], 'Display', None],
  [ 0x7edd, 'CjkUnifiedIdeograph7edd', [], 'Display', None],
  [ 0x7ede, 'CjkUnifiedIdeograph7ede', [], 'Display', None],
  [ 0x7edf, 'CjkUnifiedIdeograph7edf', [], 'Display', None],
  [ 0x7ee0, 'CjkUnifiedIdeograph7ee0', [], 'Display', None],
  [ 0x7ee1, 'CjkUnifiedIdeograph7ee1', [], 'Display', None],
  [ 0x7ee2, 'CjkUnifiedIdeograph7ee2', [], 'Display', None],
  [ 0x7ee3, 'CjkUnifiedIdeograph7ee3', [], 'Display', None],
  [ 0x7ee4, 'CjkUnifiedIdeograph7ee4', [], 'Display', None],
  [ 0x7ee5, 'CjkUnifiedIdeograph7ee5', [], 'Display', None],
  [ 0x7ee6, 'CjkUnifiedIdeograph7ee6', [], 'Display', None],
  [ 0x7ee7, 'CjkUnifiedIdeograph7ee7', [], 'Display', None],
  [ 0x7ee8, 'CjkUnifiedIdeograph7ee8', [], 'Display', None],
  [ 0x7ee9, 'CjkUnifiedIdeograph7ee9', [], 'Display', None],
  [ 0x7eea, 'CjkUnifiedIdeograph7eea', [], 'Display', None],
  [ 0x7eeb, 'CjkUnifiedIdeograph7eeb', [], 'Display', None],
  [ 0x7eec, 'CjkUnifiedIdeograph7eec', [], 'Display', None],
  [ 0x7eed, 'CjkUnifiedIdeograph7eed', [], 'Display', None],
  [ 0x7eee, 'CjkUnifiedIdeograph7eee', [], 'Display', None],
  [ 0x7eef, 'CjkUnifiedIdeograph7eef', [], 'Display', None],
  [ 0x7ef0, 'CjkUnifiedIdeograph7ef0', [], 'Display', None],
  [ 0x7ef1, 'CjkUnifiedIdeograph7ef1', [], 'Display', None],
  [ 0x7ef2, 'CjkUnifiedIdeograph7ef2', [], 'Display', None],
  [ 0x7ef3, 'CjkUnifiedIdeograph7ef3', [], 'Display', None],
  [ 0x7ef4, 'CjkUnifiedIdeograph7ef4', [], 'Display', None],
  [ 0x7ef5, 'CjkUnifiedIdeograph7ef5', [], 'Display', None],
  [ 0x7ef6, 'CjkUnifiedIdeograph7ef6', [], 'Display', None],
  [ 0x7ef7, 'CjkUnifiedIdeograph7ef7', [], 'Display', None],
  [ 0x7ef8, 'CjkUnifiedIdeograph7ef8', [], 'Display', None],
  [ 0x7ef9, 'CjkUnifiedIdeograph7ef9', [], 'Display', None],
  [ 0x7efa, 'CjkUnifiedIdeograph7efa', [], 'Display', None],
  [ 0x7efb, 'CjkUnifiedIdeograph7efb', [], 'Display', None],
  [ 0x7efc, 'CjkUnifiedIdeograph7efc', [], 'Display', None],
  [ 0x7efd, 'CjkUnifiedIdeograph7efd', [], 'Display', None],
  [ 0x7efe, 'CjkUnifiedIdeograph7efe', [], 'Display', None],
  [ 0x7eff, 'CjkUnifiedIdeograph7eff', [], 'Display', None],
  [ 0x7f00, 'CjkUnifiedIdeograph7f00', [], 'Display', None],
  [ 0x7f01, 'CjkUnifiedIdeograph7f01', [], 'Display', None],
  [ 0x7f02, 'CjkUnifiedIdeograph7f02', [], 'Display', None],
  [ 0x7f03, 'CjkUnifiedIdeograph7f03', [], 'Display', None],
  [ 0x7f04, 'CjkUnifiedIdeograph7f04', [], 'Display', None],
  [ 0x7f05, 'CjkUnifiedIdeograph7f05', [], 'Display', None],
  [ 0x7f06, 'CjkUnifiedIdeograph7f06', [], 'Display', None],
  [ 0x7f07, 'CjkUnifiedIdeograph7f07', [], 'Display', None],
  [ 0x7f08, 'CjkUnifiedIdeograph7f08', [], 'Display', None],
  [ 0x7f09, 'CjkUnifiedIdeograph7f09', [], 'Display', None],
  [ 0x7f0a, 'CjkUnifiedIdeograph7f0a', [], 'Display', None],
  [ 0x7f0b, 'CjkUnifiedIdeograph7f0b', [], 'Display', None],
  [ 0x7f0c, 'CjkUnifiedIdeograph7f0c', [], 'Display', None],
  [ 0x7f0d, 'CjkUnifiedIdeograph7f0d', [], 'Display', None],
  [ 0x7f0e, 'CjkUnifiedIdeograph7f0e', [], 'Display', None],
  [ 0x7f0f, 'CjkUnifiedIdeograph7f0f', [], 'Display', None],
  [ 0x7f10, 'CjkUnifiedIdeograph7f10', [], 'Display', None],
  [ 0x7f11, 'CjkUnifiedIdeograph7f11', [], 'Display', None],
  [ 0x7f12, 'CjkUnifiedIdeograph7f12', [], 'Display', None],
  [ 0x7f13, 'CjkUnifiedIdeograph7f13', [], 'Display', None],
  [ 0x7f14, 'CjkUnifiedIdeograph7f14', [], 'Display', None],
  [ 0x7f15, 'CjkUnifiedIdeograph7f15', [], 'Display', None],
  [ 0x7f16, 'CjkUnifiedIdeograph7f16', [], 'Display', None],
  [ 0x7f17, 'CjkUnifiedIdeograph7f17', [], 'Display', None],
  [ 0x7f18, 'CjkUnifiedIdeograph7f18', [], 'Display', None],
  [ 0x7f19, 'CjkUnifiedIdeograph7f19', [], 'Display', None],
  [ 0x7f1a, 'CjkUnifiedIdeograph7f1a', [], 'Display', None],
  [ 0x7f1b, 'CjkUnifiedIdeograph7f1b', [], 'Display', None],
  [ 0x7f1c, 'CjkUnifiedIdeograph7f1c', [], 'Display', None],
  [ 0x7f1d, 'CjkUnifiedIdeograph7f1d', [], 'Display', None],
  [ 0x7f1e, 'CjkUnifiedIdeograph7f1e', [], 'Display', None],
  [ 0x7f1f, 'CjkUnifiedIdeograph7f1f', [], 'Display', None],
  [ 0x7f20, 'CjkUnifiedIdeograph7f20', [], 'Display', None],
  [ 0x7f21, 'CjkUnifiedIdeograph7f21', [], 'Display', None],
  [ 0x7f22, 'CjkUnifiedIdeograph7f22', [], 'Display', None],
  [ 0x7f23, 'CjkUnifiedIdeograph7f23', [], 'Display', None],
  [ 0x7f24, 'CjkUnifiedIdeograph7f24', [], 'Display', None],
  [ 0x7f25, 'CjkUnifiedIdeograph7f25', [], 'Display', None],
  [ 0x7f26, 'CjkUnifiedIdeograph7f26', [], 'Display', None],
  [ 0x7f27, 'CjkUnifiedIdeograph7f27', [], 'Display', None],
  [ 0x7f28, 'CjkUnifiedIdeograph7f28', [], 'Display', None],
  [ 0x7f29, 'CjkUnifiedIdeograph7f29', [], 'Display', None],
  [ 0x7f2a, 'CjkUnifiedIdeograph7f2a', [], 'Display', None],
  [ 0x7f2b, 'CjkUnifiedIdeograph7f2b', [], 'Display', None],
  [ 0x7f2c, 'CjkUnifiedIdeograph7f2c', [], 'Display', None],
  [ 0x7f2d, 'CjkUnifiedIdeograph7f2d', [], 'Display', None],
  [ 0x7f2e, 'CjkUnifiedIdeograph7f2e', [], 'Display', None],
  [ 0x7f2f, 'CjkUnifiedIdeograph7f2f', [], 'Display', None],
  [ 0x7f30, 'CjkUnifiedIdeograph7f30', [], 'Display', None],
  [ 0x7f31, 'CjkUnifiedIdeograph7f31', [], 'Display', None],
  [ 0x7f32, 'CjkUnifiedIdeograph7f32', [], 'Display', None],
  [ 0x7f33, 'CjkUnifiedIdeograph7f33', [], 'Display', None],
  [ 0x7f34, 'CjkUnifiedIdeograph7f34', [], 'Display', None],
  [ 0x7f35, 'CjkUnifiedIdeograph7f35', [], 'Display', None],
  [ 0x7f36, 'CjkUnifiedIdeograph7f36', [], 'Display', None],
  [ 0x7f37, 'CjkUnifiedIdeograph7f37', [], 'Display', None],
  [ 0x7f38, 'CjkUnifiedIdeograph7f38', [], 'Display', None],
  [ 0x7f39, 'CjkUnifiedIdeograph7f39', [], 'Display', None],
  [ 0x7f3a, 'CjkUnifiedIdeograph7f3a', [], 'Display', None],
  [ 0x7f3b, 'CjkUnifiedIdeograph7f3b', [], 'Display', None],
  [ 0x7f3c, 'CjkUnifiedIdeograph7f3c', [], 'Display', None],
  [ 0x7f3d, 'CjkUnifiedIdeograph7f3d', [], 'Display', None],
  [ 0x7f3e, 'CjkUnifiedIdeograph7f3e', [], 'Display', None],
  [ 0x7f3f, 'CjkUnifiedIdeograph7f3f', [], 'Display', None],
  [ 0x7f40, 'CjkUnifiedIdeograph7f40', [], 'Display', None],
  [ 0x7f41, 'CjkUnifiedIdeograph7f41', [], 'Display', None],
  [ 0x7f42, 'CjkUnifiedIdeograph7f42', [], 'Display', None],
  [ 0x7f43, 'CjkUnifiedIdeograph7f43', [], 'Display', None],
  [ 0x7f44, 'CjkUnifiedIdeograph7f44', [], 'Display', None],
  [ 0x7f45, 'CjkUnifiedIdeograph7f45', [], 'Display', None],
  [ 0x7f46, 'CjkUnifiedIdeograph7f46', [], 'Display', None],
  [ 0x7f47, 'CjkUnifiedIdeograph7f47', [], 'Display', None],
  [ 0x7f48, 'CjkUnifiedIdeograph7f48', [], 'Display', None],
  [ 0x7f49, 'CjkUnifiedIdeograph7f49', [], 'Display', None],
  [ 0x7f4a, 'CjkUnifiedIdeograph7f4a', [], 'Display', None],
  [ 0x7f4b, 'CjkUnifiedIdeograph7f4b', [], 'Display', None],
  [ 0x7f4c, 'CjkUnifiedIdeograph7f4c', [], 'Display', None],
  [ 0x7f4d, 'CjkUnifiedIdeograph7f4d', [], 'Display', None],
  [ 0x7f4e, 'CjkUnifiedIdeograph7f4e', [], 'Display', None],
  [ 0x7f4f, 'CjkUnifiedIdeograph7f4f', [], 'Display', None],
  [ 0x7f50, 'CjkUnifiedIdeograph7f50', [], 'Display', None],
  [ 0x7f51, 'CjkUnifiedIdeograph7f51', [], 'Display', None],
  [ 0x7f52, 'CjkUnifiedIdeograph7f52', [], 'Display', None],
  [ 0x7f53, 'CjkUnifiedIdeograph7f53', [], 'Display', None],
  [ 0x7f54, 'CjkUnifiedIdeograph7f54', [], 'Display', None],
  [ 0x7f55, 'CjkUnifiedIdeograph7f55', [], 'Display', None],
  [ 0x7f56, 'CjkUnifiedIdeograph7f56', [], 'Display', None],
  [ 0x7f57, 'CjkUnifiedIdeograph7f57', [], 'Display', None],
  [ 0x7f58, 'CjkUnifiedIdeograph7f58', [], 'Display', None],
  [ 0x7f59, 'CjkUnifiedIdeograph7f59', [], 'Display', None],
  [ 0x7f5a, 'CjkUnifiedIdeograph7f5a', [], 'Display', None],
  [ 0x7f5b, 'CjkUnifiedIdeograph7f5b', [], 'Display', None],
  [ 0x7f5c, 'CjkUnifiedIdeograph7f5c', [], 'Display', None],
  [ 0x7f5d, 'CjkUnifiedIdeograph7f5d', [], 'Display', None],
  [ 0x7f5e, 'CjkUnifiedIdeograph7f5e', [], 'Display', None],
  [ 0x7f5f, 'CjkUnifiedIdeograph7f5f', [], 'Display', None],
  [ 0x7f60, 'CjkUnifiedIdeograph7f60', [], 'Display', None],
  [ 0x7f61, 'CjkUnifiedIdeograph7f61', [], 'Display', None],
  [ 0x7f62, 'CjkUnifiedIdeograph7f62', [], 'Display', None],
  [ 0x7f63, 'CjkUnifiedIdeograph7f63', [], 'Display', None],
  [ 0x7f64, 'CjkUnifiedIdeograph7f64', [], 'Display', None],
  [ 0x7f65, 'CjkUnifiedIdeograph7f65', [], 'Display', None],
  [ 0x7f66, 'CjkUnifiedIdeograph7f66', [], 'Display', None],
  [ 0x7f67, 'CjkUnifiedIdeograph7f67', [], 'Display', None],
  [ 0x7f68, 'CjkUnifiedIdeograph7f68', [], 'Display', None],
  [ 0x7f69, 'CjkUnifiedIdeograph7f69', [], 'Display', None],
  [ 0x7f6a, 'CjkUnifiedIdeograph7f6a', [], 'Display', None],
  [ 0x7f6b, 'CjkUnifiedIdeograph7f6b', [], 'Display', None],
  [ 0x7f6c, 'CjkUnifiedIdeograph7f6c', [], 'Display', None],
  [ 0x7f6d, 'CjkUnifiedIdeograph7f6d', [], 'Display', None],
  [ 0x7f6e, 'CjkUnifiedIdeograph7f6e', [], 'Display', None],
  [ 0x7f6f, 'CjkUnifiedIdeograph7f6f', [], 'Display', None],
  [ 0x7f70, 'CjkUnifiedIdeograph7f70', [], 'Display', None],
  [ 0x7f71, 'CjkUnifiedIdeograph7f71', [], 'Display', None],
  [ 0x7f72, 'CjkUnifiedIdeograph7f72', [], 'Display', None],
  [ 0x7f73, 'CjkUnifiedIdeograph7f73', [], 'Display', None],
  [ 0x7f74, 'CjkUnifiedIdeograph7f74', [], 'Display', None],
  [ 0x7f75, 'CjkUnifiedIdeograph7f75', [], 'Display', None],
  [ 0x7f76, 'CjkUnifiedIdeograph7f76', [], 'Display', None],
  [ 0x7f77, 'CjkUnifiedIdeograph7f77', [], 'Display', None],
  [ 0x7f78, 'CjkUnifiedIdeograph7f78', [], 'Display', None],
  [ 0x7f79, 'CjkUnifiedIdeograph7f79', [], 'Display', None],
  [ 0x7f7a, 'CjkUnifiedIdeograph7f7a', [], 'Display', None],
  [ 0x7f7b, 'CjkUnifiedIdeograph7f7b', [], 'Display', None],
  [ 0x7f7c, 'CjkUnifiedIdeograph7f7c', [], 'Display', None],
  [ 0x7f7d, 'CjkUnifiedIdeograph7f7d', [], 'Display', None],
  [ 0x7f7e, 'CjkUnifiedIdeograph7f7e', [], 'Display', None],
  [ 0x7f7f, 'CjkUnifiedIdeograph7f7f', [], 'Display', None],
  [ 0x7f80, 'CjkUnifiedIdeograph7f80', [], 'Display', None],
  [ 0x7f81, 'CjkUnifiedIdeograph7f81', [], 'Display', None],
  [ 0x7f82, 'CjkUnifiedIdeograph7f82', [], 'Display', None],
  [ 0x7f83, 'CjkUnifiedIdeograph7f83', [], 'Display', None],
  [ 0x7f84, 'CjkUnifiedIdeograph7f84', [], 'Display', None],
  [ 0x7f85, 'CjkUnifiedIdeograph7f85', [], 'Display', None],
  [ 0x7f86, 'CjkUnifiedIdeograph7f86', [], 'Display', None],
  [ 0x7f87, 'CjkUnifiedIdeograph7f87', [], 'Display', None],
  [ 0x7f88, 'CjkUnifiedIdeograph7f88', [], 'Display', None],
  [ 0x7f89, 'CjkUnifiedIdeograph7f89', [], 'Display', None],
  [ 0x7f8a, 'CjkUnifiedIdeograph7f8a', [], 'Display', None],
  [ 0x7f8b, 'CjkUnifiedIdeograph7f8b', [], 'Display', None],
  [ 0x7f8c, 'CjkUnifiedIdeograph7f8c', [], 'Display', None],
  [ 0x7f8d, 'CjkUnifiedIdeograph7f8d', [], 'Display', None],
  [ 0x7f8e, 'CjkUnifiedIdeograph7f8e', [], 'Display', None],
  [ 0x7f8f, 'CjkUnifiedIdeograph7f8f', [], 'Display', None],
  [ 0x7f90, 'CjkUnifiedIdeograph7f90', [], 'Display', None],
  [ 0x7f91, 'CjkUnifiedIdeograph7f91', [], 'Display', None],
  [ 0x7f92, 'CjkUnifiedIdeograph7f92', [], 'Display', None],
  [ 0x7f93, 'CjkUnifiedIdeograph7f93', [], 'Display', None],
  [ 0x7f94, 'CjkUnifiedIdeograph7f94', [], 'Display', None],
  [ 0x7f95, 'CjkUnifiedIdeograph7f95', [], 'Display', None],
  [ 0x7f96, 'CjkUnifiedIdeograph7f96', [], 'Display', None],
  [ 0x7f97, 'CjkUnifiedIdeograph7f97', [], 'Display', None],
  [ 0x7f98, 'CjkUnifiedIdeograph7f98', [], 'Display', None],
  [ 0x7f99, 'CjkUnifiedIdeograph7f99', [], 'Display', None],
  [ 0x7f9a, 'CjkUnifiedIdeograph7f9a', [], 'Display', None],
  [ 0x7f9b, 'CjkUnifiedIdeograph7f9b', [], 'Display', None],
  [ 0x7f9c, 'CjkUnifiedIdeograph7f9c', [], 'Display', None],
  [ 0x7f9d, 'CjkUnifiedIdeograph7f9d', [], 'Display', None],
  [ 0x7f9e, 'CjkUnifiedIdeograph7f9e', [], 'Display', None],
  [ 0x7f9f, 'CjkUnifiedIdeograph7f9f', [], 'Display', None],
  [ 0x7fa0, 'CjkUnifiedIdeograph7fa0', [], 'Display', None],
  [ 0x7fa1, 'CjkUnifiedIdeograph7fa1', [], 'Display', None],
  [ 0x7fa2, 'CjkUnifiedIdeograph7fa2', [], 'Display', None],
  [ 0x7fa3, 'CjkUnifiedIdeograph7fa3', [], 'Display', None],
  [ 0x7fa4, 'CjkUnifiedIdeograph7fa4', [], 'Display', None],
  [ 0x7fa5, 'CjkUnifiedIdeograph7fa5', [], 'Display', None],
  [ 0x7fa6, 'CjkUnifiedIdeograph7fa6', [], 'Display', None],
  [ 0x7fa7, 'CjkUnifiedIdeograph7fa7', [], 'Display', None],
  [ 0x7fa8, 'CjkUnifiedIdeograph7fa8', [], 'Display', None],
  [ 0x7fa9, 'CjkUnifiedIdeograph7fa9', [], 'Display', None],
  [ 0x7faa, 'CjkUnifiedIdeograph7faa', [], 'Display', None],
  [ 0x7fab, 'CjkUnifiedIdeograph7fab', [], 'Display', None],
  [ 0x7fac, 'CjkUnifiedIdeograph7fac', [], 'Display', None],
  [ 0x7fad, 'CjkUnifiedIdeograph7fad', [], 'Display', None],
  [ 0x7fae, 'CjkUnifiedIdeograph7fae', [], 'Display', None],
  [ 0x7faf, 'CjkUnifiedIdeograph7faf', [], 'Display', None],
  [ 0x7fb0, 'CjkUnifiedIdeograph7fb0', [], 'Display', None],
  [ 0x7fb1, 'CjkUnifiedIdeograph7fb1', [], 'Display', None],
  [ 0x7fb2, 'CjkUnifiedIdeograph7fb2', [], 'Display', None],
  [ 0x7fb3, 'CjkUnifiedIdeograph7fb3', [], 'Display', None],
  [ 0x7fb4, 'CjkUnifiedIdeograph7fb4', [], 'Display', None],
  [ 0x7fb5, 'CjkUnifiedIdeograph7fb5', [], 'Display', None],
  [ 0x7fb6, 'CjkUnifiedIdeograph7fb6', [], 'Display', None],
  [ 0x7fb7, 'CjkUnifiedIdeograph7fb7', [], 'Display', None],
  [ 0x7fb8, 'CjkUnifiedIdeograph7fb8', [], 'Display', None],
  [ 0x7fb9, 'CjkUnifiedIdeograph7fb9', [], 'Display', None],
  [ 0x7fba, 'CjkUnifiedIdeograph7fba', [], 'Display', None],
  [ 0x7fbb, 'CjkUnifiedIdeograph7fbb', [], 'Display', None],
  [ 0x7fbc, 'CjkUnifiedIdeograph7fbc', [], 'Display', None],
  [ 0x7fbd, 'CjkUnifiedIdeograph7fbd', [], 'Display', None],
  [ 0x7fbe, 'CjkUnifiedIdeograph7fbe', [], 'Display', None],
  [ 0x7fbf, 'CjkUnifiedIdeograph7fbf', [], 'Display', None],
  [ 0x7fc0, 'CjkUnifiedIdeograph7fc0', [], 'Display', None],
  [ 0x7fc1, 'CjkUnifiedIdeograph7fc1', [], 'Display', None],
  [ 0x7fc2, 'CjkUnifiedIdeograph7fc2', [], 'Display', None],
  [ 0x7fc3, 'CjkUnifiedIdeograph7fc3', [], 'Display', None],
  [ 0x7fc4, 'CjkUnifiedIdeograph7fc4', [], 'Display', None],
  [ 0x7fc5, 'CjkUnifiedIdeograph7fc5', [], 'Display', None],
  [ 0x7fc6, 'CjkUnifiedIdeograph7fc6', [], 'Display', None],
  [ 0x7fc7, 'CjkUnifiedIdeograph7fc7', [], 'Display', None],
  [ 0x7fc8, 'CjkUnifiedIdeograph7fc8', [], 'Display', None],
  [ 0x7fc9, 'CjkUnifiedIdeograph7fc9', [], 'Display', None],
  [ 0x7fca, 'CjkUnifiedIdeograph7fca', [], 'Display', None],
  [ 0x7fcb, 'CjkUnifiedIdeograph7fcb', [], 'Display', None],
  [ 0x7fcc, 'CjkUnifiedIdeograph7fcc', [], 'Display', None],
  [ 0x7fcd, 'CjkUnifiedIdeograph7fcd', [], 'Display', None],
  [ 0x7fce, 'CjkUnifiedIdeograph7fce', [], 'Display', None],
  [ 0x7fcf, 'CjkUnifiedIdeograph7fcf', [], 'Display', None],
  [ 0x7fd0, 'CjkUnifiedIdeograph7fd0', [], 'Display', None],
  [ 0x7fd1, 'CjkUnifiedIdeograph7fd1', [], 'Display', None],
  [ 0x7fd2, 'CjkUnifiedIdeograph7fd2', [], 'Display', None],
  [ 0x7fd3, 'CjkUnifiedIdeograph7fd3', [], 'Display', None],
  [ 0x7fd4, 'CjkUnifiedIdeograph7fd4', [], 'Display', None],
  [ 0x7fd5, 'CjkUnifiedIdeograph7fd5', [], 'Display', None],
  [ 0x7fd6, 'CjkUnifiedIdeograph7fd6', [], 'Display', None],
  [ 0x7fd7, 'CjkUnifiedIdeograph7fd7', [], 'Display', None],
  [ 0x7fd8, 'CjkUnifiedIdeograph7fd8', [], 'Display', None],
  [ 0x7fd9, 'CjkUnifiedIdeograph7fd9', [], 'Display', None],
  [ 0x7fda, 'CjkUnifiedIdeograph7fda', [], 'Display', None],
  [ 0x7fdb, 'CjkUnifiedIdeograph7fdb', [], 'Display', None],
  [ 0x7fdc, 'CjkUnifiedIdeograph7fdc', [], 'Display', None],
  [ 0x7fdd, 'CjkUnifiedIdeograph7fdd', [], 'Display', None],
  [ 0x7fde, 'CjkUnifiedIdeograph7fde', [], 'Display', None],
  [ 0x7fdf, 'CjkUnifiedIdeograph7fdf', [], 'Display', None],
  [ 0x7fe0, 'CjkUnifiedIdeograph7fe0', [], 'Display', None],
  [ 0x7fe1, 'CjkUnifiedIdeograph7fe1', [], 'Display', None],
  [ 0x7fe2, 'CjkUnifiedIdeograph7fe2', [], 'Display', None],
  [ 0x7fe3, 'CjkUnifiedIdeograph7fe3', [], 'Display', None],
  [ 0x7fe4, 'CjkUnifiedIdeograph7fe4', [], 'Display', None],
  [ 0x7fe5, 'CjkUnifiedIdeograph7fe5', [], 'Display', None],
  [ 0x7fe6, 'CjkUnifiedIdeograph7fe6', [], 'Display', None],
  [ 0x7fe7, 'CjkUnifiedIdeograph7fe7', [], 'Display', None],
  [ 0x7fe8, 'CjkUnifiedIdeograph7fe8', [], 'Display', None],
  [ 0x7fe9, 'CjkUnifiedIdeograph7fe9', [], 'Display', None],
  [ 0x7fea, 'CjkUnifiedIdeograph7fea', [], 'Display', None],
  [ 0x7feb, 'CjkUnifiedIdeograph7feb', [], 'Display', None],
  [ 0x7fec, 'CjkUnifiedIdeograph7fec', [], 'Display', None],
  [ 0x7fed, 'CjkUnifiedIdeograph7fed', [], 'Display', None],
  [ 0x7fee, 'CjkUnifiedIdeograph7fee', [], 'Display', None],
  [ 0x7fef, 'CjkUnifiedIdeograph7fef', [], 'Display', None],
  [ 0x7ff0, 'CjkUnifiedIdeograph7ff0', [], 'Display', None],
  [ 0x7ff1, 'CjkUnifiedIdeograph7ff1', [], 'Display', None],
  [ 0x7ff2, 'CjkUnifiedIdeograph7ff2', [], 'Display', None],
  [ 0x7ff3, 'CjkUnifiedIdeograph7ff3', [], 'Display', None],
  [ 0x7ff4, 'CjkUnifiedIdeograph7ff4', [], 'Display', None],
  [ 0x7ff5, 'CjkUnifiedIdeograph7ff5', [], 'Display', None],
  [ 0x7ff6, 'CjkUnifiedIdeograph7ff6', [], 'Display', None],
  [ 0x7ff7, 'CjkUnifiedIdeograph7ff7', [], 'Display', None],
  [ 0x7ff8, 'CjkUnifiedIdeograph7ff8', [], 'Display', None],
  [ 0x7ff9, 'CjkUnifiedIdeograph7ff9', [], 'Display', None],
  [ 0x7ffa, 'CjkUnifiedIdeograph7ffa', [], 'Display', None],
  [ 0x7ffb, 'CjkUnifiedIdeograph7ffb', [], 'Display', None],
  [ 0x7ffc, 'CjkUnifiedIdeograph7ffc', [], 'Display', None],
  [ 0x7ffd, 'CjkUnifiedIdeograph7ffd', [], 'Display', None],
  [ 0x7ffe, 'CjkUnifiedIdeograph7ffe', [], 'Display', None],
  [ 0x7fff, 'CjkUnifiedIdeograph7fff', [], 'Display', None],
  [ 0x8000, 'CjkUnifiedIdeograph8000', [], 'Display', None],
  [ 0x8001, 'CjkUnifiedIdeograph8001', [], 'Display', None],
  [ 0x8002, 'CjkUnifiedIdeograph8002', [], 'Display', None],
  [ 0x8003, 'CjkUnifiedIdeograph8003', [], 'Display', None],
  [ 0x8004, 'CjkUnifiedIdeograph8004', [], 'Display', None],
  [ 0x8005, 'CjkUnifiedIdeograph8005', [], 'Display', None],
  [ 0x8006, 'CjkUnifiedIdeograph8006', [], 'Display', None],
  [ 0x8007, 'CjkUnifiedIdeograph8007', [], 'Display', None],
  [ 0x8008, 'CjkUnifiedIdeograph8008', [], 'Display', None],
  [ 0x8009, 'CjkUnifiedIdeograph8009', [], 'Display', None],
  [ 0x800a, 'CjkUnifiedIdeograph800a', [], 'Display', None],
  [ 0x800b, 'CjkUnifiedIdeograph800b', [], 'Display', None],
  [ 0x800c, 'CjkUnifiedIdeograph800c', [], 'Display', None],
  [ 0x800d, 'CjkUnifiedIdeograph800d', [], 'Display', None],
  [ 0x800e, 'CjkUnifiedIdeograph800e', [], 'Display', None],
  [ 0x800f, 'CjkUnifiedIdeograph800f', [], 'Display', None],
  [ 0x8010, 'CjkUnifiedIdeograph8010', [], 'Display', None],
  [ 0x8011, 'CjkUnifiedIdeograph8011', [], 'Display', None],
  [ 0x8012, 'CjkUnifiedIdeograph8012', [], 'Display', None],
  [ 0x8013, 'CjkUnifiedIdeograph8013', [], 'Display', None],
  [ 0x8014, 'CjkUnifiedIdeograph8014', [], 'Display', None],
  [ 0x8015, 'CjkUnifiedIdeograph8015', [], 'Display', None],
  [ 0x8016, 'CjkUnifiedIdeograph8016', [], 'Display', None],
  [ 0x8017, 'CjkUnifiedIdeograph8017', [], 'Display', None],
  [ 0x8018, 'CjkUnifiedIdeograph8018', [], 'Display', None],
  [ 0x8019, 'CjkUnifiedIdeograph8019', [], 'Display', None],
  [ 0x801a, 'CjkUnifiedIdeograph801a', [], 'Display', None],
  [ 0x801b, 'CjkUnifiedIdeograph801b', [], 'Display', None],
  [ 0x801c, 'CjkUnifiedIdeograph801c', [], 'Display', None],
  [ 0x801d, 'CjkUnifiedIdeograph801d', [], 'Display', None],
  [ 0x801e, 'CjkUnifiedIdeograph801e', [], 'Display', None],
  [ 0x801f, 'CjkUnifiedIdeograph801f', [], 'Display', None],
  [ 0x8020, 'CjkUnifiedIdeograph8020', [], 'Display', None],
  [ 0x8021, 'CjkUnifiedIdeograph8021', [], 'Display', None],
  [ 0x8022, 'CjkUnifiedIdeograph8022', [], 'Display', None],
  [ 0x8023, 'CjkUnifiedIdeograph8023', [], 'Display', None],
  [ 0x8024, 'CjkUnifiedIdeograph8024', [], 'Display', None],
  [ 0x8025, 'CjkUnifiedIdeograph8025', [], 'Display', None],
  [ 0x8026, 'CjkUnifiedIdeograph8026', [], 'Display', None],
  [ 0x8027, 'CjkUnifiedIdeograph8027', [], 'Display', None],
  [ 0x8028, 'CjkUnifiedIdeograph8028', [], 'Display', None],
  [ 0x8029, 'CjkUnifiedIdeograph8029', [], 'Display', None],
  [ 0x802a, 'CjkUnifiedIdeograph802a', [], 'Display', None],
  [ 0x802b, 'CjkUnifiedIdeograph802b', [], 'Display', None],
  [ 0x802c, 'CjkUnifiedIdeograph802c', [], 'Display', None],
  [ 0x802d, 'CjkUnifiedIdeograph802d', [], 'Display', None],
  [ 0x802e, 'CjkUnifiedIdeograph802e', [], 'Display', None],
  [ 0x802f, 'CjkUnifiedIdeograph802f', [], 'Display', None],
  [ 0x8030, 'CjkUnifiedIdeograph8030', [], 'Display', None],
  [ 0x8031, 'CjkUnifiedIdeograph8031', [], 'Display', None],
  [ 0x8032, 'CjkUnifiedIdeograph8032', [], 'Display', None],
  [ 0x8033, 'CjkUnifiedIdeograph8033', [], 'Display', None],
  [ 0x8034, 'CjkUnifiedIdeograph8034', [], 'Display', None],
  [ 0x8035, 'CjkUnifiedIdeograph8035', [], 'Display', None],
  [ 0x8036, 'CjkUnifiedIdeograph8036', [], 'Display', None],
  [ 0x8037, 'CjkUnifiedIdeograph8037', [], 'Display', None],
  [ 0x8038, 'CjkUnifiedIdeograph8038', [], 'Display', None],
  [ 0x8039, 'CjkUnifiedIdeograph8039', [], 'Display', None],
  [ 0x803a, 'CjkUnifiedIdeograph803a', [], 'Display', None],
  [ 0x803b, 'CjkUnifiedIdeograph803b', [], 'Display', None],
  [ 0x803c, 'CjkUnifiedIdeograph803c', [], 'Display', None],
  [ 0x803d, 'CjkUnifiedIdeograph803d', [], 'Display', None],
  [ 0x803e, 'CjkUnifiedIdeograph803e', [], 'Display', None],
  [ 0x803f, 'CjkUnifiedIdeograph803f', [], 'Display', None],
  [ 0x8040, 'CjkUnifiedIdeograph8040', [], 'Display', None],
  [ 0x8041, 'CjkUnifiedIdeograph8041', [], 'Display', None],
  [ 0x8042, 'CjkUnifiedIdeograph8042', [], 'Display', None],
  [ 0x8043, 'CjkUnifiedIdeograph8043', [], 'Display', None],
  [ 0x8044, 'CjkUnifiedIdeograph8044', [], 'Display', None],
  [ 0x8045, 'CjkUnifiedIdeograph8045', [], 'Display', None],
  [ 0x8046, 'CjkUnifiedIdeograph8046', [], 'Display', None],
  [ 0x8047, 'CjkUnifiedIdeograph8047', [], 'Display', None],
  [ 0x8048, 'CjkUnifiedIdeograph8048', [], 'Display', None],
  [ 0x8049, 'CjkUnifiedIdeograph8049', [], 'Display', None],
  [ 0x804a, 'CjkUnifiedIdeograph804a', [], 'Display', None],
  [ 0x804b, 'CjkUnifiedIdeograph804b', [], 'Display', None],
  [ 0x804c, 'CjkUnifiedIdeograph804c', [], 'Display', None],
  [ 0x804d, 'CjkUnifiedIdeograph804d', [], 'Display', None],
  [ 0x804e, 'CjkUnifiedIdeograph804e', [], 'Display', None],
  [ 0x804f, 'CjkUnifiedIdeograph804f', [], 'Display', None],
  [ 0x8050, 'CjkUnifiedIdeograph8050', [], 'Display', None],
  [ 0x8051, 'CjkUnifiedIdeograph8051', [], 'Display', None],
  [ 0x8052, 'CjkUnifiedIdeograph8052', [], 'Display', None],
  [ 0x8053, 'CjkUnifiedIdeograph8053', [], 'Display', None],
  [ 0x8054, 'CjkUnifiedIdeograph8054', [], 'Display', None],
  [ 0x8055, 'CjkUnifiedIdeograph8055', [], 'Display', None],
  [ 0x8056, 'CjkUnifiedIdeograph8056', [], 'Display', None],
  [ 0x8057, 'CjkUnifiedIdeograph8057', [], 'Display', None],
  [ 0x8058, 'CjkUnifiedIdeograph8058', [], 'Display', None],
  [ 0x8059, 'CjkUnifiedIdeograph8059', [], 'Display', None],
  [ 0x805a, 'CjkUnifiedIdeograph805a', [], 'Display', None],
  [ 0x805b, 'CjkUnifiedIdeograph805b', [], 'Display', None],
  [ 0x805c, 'CjkUnifiedIdeograph805c', [], 'Display', None],
  [ 0x805d, 'CjkUnifiedIdeograph805d', [], 'Display', None],
  [ 0x805e, 'CjkUnifiedIdeograph805e', [], 'Display', None],
  [ 0x805f, 'CjkUnifiedIdeograph805f', [], 'Display', None],
  [ 0x8060, 'CjkUnifiedIdeograph8060', [], 'Display', None],
  [ 0x8061, 'CjkUnifiedIdeograph8061', [], 'Display', None],
  [ 0x8062, 'CjkUnifiedIdeograph8062', [], 'Display', None],
  [ 0x8063, 'CjkUnifiedIdeograph8063', [], 'Display', None],
  [ 0x8064, 'CjkUnifiedIdeograph8064', [], 'Display', None],
  [ 0x8065, 'CjkUnifiedIdeograph8065', [], 'Display', None],
  [ 0x8066, 'CjkUnifiedIdeograph8066', [], 'Display', None],
  [ 0x8067, 'CjkUnifiedIdeograph8067', [], 'Display', None],
  [ 0x8068, 'CjkUnifiedIdeograph8068', [], 'Display', None],
  [ 0x8069, 'CjkUnifiedIdeograph8069', [], 'Display', None],
  [ 0x806a, 'CjkUnifiedIdeograph806a', [], 'Display', None],
  [ 0x806b, 'CjkUnifiedIdeograph806b', [], 'Display', None],
  [ 0x806c, 'CjkUnifiedIdeograph806c', [], 'Display', None],
  [ 0x806d, 'CjkUnifiedIdeograph806d', [], 'Display', None],
  [ 0x806e, 'CjkUnifiedIdeograph806e', [], 'Display', None],
  [ 0x806f, 'CjkUnifiedIdeograph806f', [], 'Display', None],
  [ 0x8070, 'CjkUnifiedIdeograph8070', [], 'Display', None],
  [ 0x8071, 'CjkUnifiedIdeograph8071', [], 'Display', None],
  [ 0x8072, 'CjkUnifiedIdeograph8072', [], 'Display', None],
  [ 0x8073, 'CjkUnifiedIdeograph8073', [], 'Display', None],
  [ 0x8074, 'CjkUnifiedIdeograph8074', [], 'Display', None],
  [ 0x8075, 'CjkUnifiedIdeograph8075', [], 'Display', None],
  [ 0x8076, 'CjkUnifiedIdeograph8076', [], 'Display', None],
  [ 0x8077, 'CjkUnifiedIdeograph8077', [], 'Display', None],
  [ 0x8078, 'CjkUnifiedIdeograph8078', [], 'Display', None],
  [ 0x8079, 'CjkUnifiedIdeograph8079', [], 'Display', None],
  [ 0x807a, 'CjkUnifiedIdeograph807a', [], 'Display', None],
  [ 0x807b, 'CjkUnifiedIdeograph807b', [], 'Display', None],
  [ 0x807c, 'CjkUnifiedIdeograph807c', [], 'Display', None],
  [ 0x807d, 'CjkUnifiedIdeograph807d', [], 'Display', None],
  [ 0x807e, 'CjkUnifiedIdeograph807e', [], 'Display', None],
  [ 0x807f, 'CjkUnifiedIdeograph807f', [], 'Display', None],
  [ 0x8080, 'CjkUnifiedIdeograph8080', [], 'Display', None],
  [ 0x8081, 'CjkUnifiedIdeograph8081', [], 'Display', None],
  [ 0x8082, 'CjkUnifiedIdeograph8082', [], 'Display', None],
  [ 0x8083, 'CjkUnifiedIdeograph8083', [], 'Display', None],
  [ 0x8084, 'CjkUnifiedIdeograph8084', [], 'Display', None],
  [ 0x8085, 'CjkUnifiedIdeograph8085', [], 'Display', None],
  [ 0x8086, 'CjkUnifiedIdeograph8086', [], 'Display', None],
  [ 0x8087, 'CjkUnifiedIdeograph8087', [], 'Display', None],
  [ 0x8088, 'CjkUnifiedIdeograph8088', [], 'Display', None],
  [ 0x8089, 'CjkUnifiedIdeograph8089', [], 'Display', None],
  [ 0x808a, 'CjkUnifiedIdeograph808a', [], 'Display', None],
  [ 0x808b, 'CjkUnifiedIdeograph808b', [], 'Display', None],
  [ 0x808c, 'CjkUnifiedIdeograph808c', [], 'Display', None],
  [ 0x808d, 'CjkUnifiedIdeograph808d', [], 'Display', None],
  [ 0x808e, 'CjkUnifiedIdeograph808e', [], 'Display', None],
  [ 0x808f, 'CjkUnifiedIdeograph808f', [], 'Display', None],
  [ 0x8090, 'CjkUnifiedIdeograph8090', [], 'Display', None],
  [ 0x8091, 'CjkUnifiedIdeograph8091', [], 'Display', None],
  [ 0x8092, 'CjkUnifiedIdeograph8092', [], 'Display', None],
  [ 0x8093, 'CjkUnifiedIdeograph8093', [], 'Display', None],
  [ 0x8094, 'CjkUnifiedIdeograph8094', [], 'Display', None],
  [ 0x8095, 'CjkUnifiedIdeograph8095', [], 'Display', None],
  [ 0x8096, 'CjkUnifiedIdeograph8096', [], 'Display', None],
  [ 0x8097, 'CjkUnifiedIdeograph8097', [], 'Display', None],
  [ 0x8098, 'CjkUnifiedIdeograph8098', [], 'Display', None],
  [ 0x8099, 'CjkUnifiedIdeograph8099', [], 'Display', None],
  [ 0x809a, 'CjkUnifiedIdeograph809a', [], 'Display', None],
  [ 0x809b, 'CjkUnifiedIdeograph809b', [], 'Display', None],
  [ 0x809c, 'CjkUnifiedIdeograph809c', [], 'Display', None],
  [ 0x809d, 'CjkUnifiedIdeograph809d', [], 'Display', None],
  [ 0x809e, 'CjkUnifiedIdeograph809e', [], 'Display', None],
  [ 0x809f, 'CjkUnifiedIdeograph809f', [], 'Display', None],
  [ 0x80a0, 'CjkUnifiedIdeograph80a0', [], 'Display', None],
  [ 0x80a1, 'CjkUnifiedIdeograph80a1', [], 'Display', None],
  [ 0x80a2, 'CjkUnifiedIdeograph80a2', [], 'Display', None],
  [ 0x80a3, 'CjkUnifiedIdeograph80a3', [], 'Display', None],
  [ 0x80a4, 'CjkUnifiedIdeograph80a4', [], 'Display', None],
  [ 0x80a5, 'CjkUnifiedIdeograph80a5', [], 'Display', None],
  [ 0x80a6, 'CjkUnifiedIdeograph80a6', [], 'Display', None],
  [ 0x80a7, 'CjkUnifiedIdeograph80a7', [], 'Display', None],
  [ 0x80a8, 'CjkUnifiedIdeograph80a8', [], 'Display', None],
  [ 0x80a9, 'CjkUnifiedIdeograph80a9', [], 'Display', None],
  [ 0x80aa, 'CjkUnifiedIdeograph80aa', [], 'Display', None],
  [ 0x80ab, 'CjkUnifiedIdeograph80ab', [], 'Display', None],
  [ 0x80ac, 'CjkUnifiedIdeograph80ac', [], 'Display', None],
  [ 0x80ad, 'CjkUnifiedIdeograph80ad', [], 'Display', None],
  [ 0x80ae, 'CjkUnifiedIdeograph80ae', [], 'Display', None],
  [ 0x80af, 'CjkUnifiedIdeograph80af', [], 'Display', None],
  [ 0x80b0, 'CjkUnifiedIdeograph80b0', [], 'Display', None],
  [ 0x80b1, 'CjkUnifiedIdeograph80b1', [], 'Display', None],
  [ 0x80b2, 'CjkUnifiedIdeograph80b2', [], 'Display', None],
  [ 0x80b3, 'CjkUnifiedIdeograph80b3', [], 'Display', None],
  [ 0x80b4, 'CjkUnifiedIdeograph80b4', [], 'Display', None],
  [ 0x80b5, 'CjkUnifiedIdeograph80b5', [], 'Display', None],
  [ 0x80b6, 'CjkUnifiedIdeograph80b6', [], 'Display', None],
  [ 0x80b7, 'CjkUnifiedIdeograph80b7', [], 'Display', None],
  [ 0x80b8, 'CjkUnifiedIdeograph80b8', [], 'Display', None],
  [ 0x80b9, 'CjkUnifiedIdeograph80b9', [], 'Display', None],
  [ 0x80ba, 'CjkUnifiedIdeograph80ba', [], 'Display', None],
  [ 0x80bb, 'CjkUnifiedIdeograph80bb', [], 'Display', None],
  [ 0x80bc, 'CjkUnifiedIdeograph80bc', [], 'Display', None],
  [ 0x80bd, 'CjkUnifiedIdeograph80bd', [], 'Display', None],
  [ 0x80be, 'CjkUnifiedIdeograph80be', [], 'Display', None],
  [ 0x80bf, 'CjkUnifiedIdeograph80bf', [], 'Display', None],
  [ 0x80c0, 'CjkUnifiedIdeograph80c0', [], 'Display', None],
  [ 0x80c1, 'CjkUnifiedIdeograph80c1', [], 'Display', None],
  [ 0x80c2, 'CjkUnifiedIdeograph80c2', [], 'Display', None],
  [ 0x80c3, 'CjkUnifiedIdeograph80c3', [], 'Display', None],
  [ 0x80c4, 'CjkUnifiedIdeograph80c4', [], 'Display', None],
  [ 0x80c5, 'CjkUnifiedIdeograph80c5', [], 'Display', None],
  [ 0x80c6, 'CjkUnifiedIdeograph80c6', [], 'Display', None],
  [ 0x80c7, 'CjkUnifiedIdeograph80c7', [], 'Display', None],
  [ 0x80c8, 'CjkUnifiedIdeograph80c8', [], 'Display', None],
  [ 0x80c9, 'CjkUnifiedIdeograph80c9', [], 'Display', None],
  [ 0x80ca, 'CjkUnifiedIdeograph80ca', [], 'Display', None],
  [ 0x80cb, 'CjkUnifiedIdeograph80cb', [], 'Display', None],
  [ 0x80cc, 'CjkUnifiedIdeograph80cc', [], 'Display', None],
  [ 0x80cd, 'CjkUnifiedIdeograph80cd', [], 'Display', None],
  [ 0x80ce, 'CjkUnifiedIdeograph80ce', [], 'Display', None],
  [ 0x80cf, 'CjkUnifiedIdeograph80cf', [], 'Display', None],
  [ 0x80d0, 'CjkUnifiedIdeograph80d0', [], 'Display', None],
  [ 0x80d1, 'CjkUnifiedIdeograph80d1', [], 'Display', None],
  [ 0x80d2, 'CjkUnifiedIdeograph80d2', [], 'Display', None],
  [ 0x80d3, 'CjkUnifiedIdeograph80d3', [], 'Display', None],
  [ 0x80d4, 'CjkUnifiedIdeograph80d4', [], 'Display', None],
  [ 0x80d5, 'CjkUnifiedIdeograph80d5', [], 'Display', None],
  [ 0x80d6, 'CjkUnifiedIdeograph80d6', [], 'Display', None],
  [ 0x80d7, 'CjkUnifiedIdeograph80d7', [], 'Display', None],
  [ 0x80d8, 'CjkUnifiedIdeograph80d8', [], 'Display', None],
  [ 0x80d9, 'CjkUnifiedIdeograph80d9', [], 'Display', None],
  [ 0x80da, 'CjkUnifiedIdeograph80da', [], 'Display', None],
  [ 0x80db, 'CjkUnifiedIdeograph80db', [], 'Display', None],
  [ 0x80dc, 'CjkUnifiedIdeograph80dc', [], 'Display', None],
  [ 0x80dd, 'CjkUnifiedIdeograph80dd', [], 'Display', None],
  [ 0x80de, 'CjkUnifiedIdeograph80de', [], 'Display', None],
  [ 0x80df, 'CjkUnifiedIdeograph80df', [], 'Display', None],
  [ 0x80e0, 'CjkUnifiedIdeograph80e0', [], 'Display', None],
  [ 0x80e1, 'CjkUnifiedIdeograph80e1', [], 'Display', None],
  [ 0x80e2, 'CjkUnifiedIdeograph80e2', [], 'Display', None],
  [ 0x80e3, 'CjkUnifiedIdeograph80e3', [], 'Display', None],
  [ 0x80e4, 'CjkUnifiedIdeograph80e4', [], 'Display', None],
  [ 0x80e5, 'CjkUnifiedIdeograph80e5', [], 'Display', None],
  [ 0x80e6, 'CjkUnifiedIdeograph80e6', [], 'Display', None],
  [ 0x80e7, 'CjkUnifiedIdeograph80e7', [], 'Display', None],
  [ 0x80e8, 'CjkUnifiedIdeograph80e8', [], 'Display', None],
  [ 0x80e9, 'CjkUnifiedIdeograph80e9', [], 'Display', None],
  [ 0x80ea, 'CjkUnifiedIdeograph80ea', [], 'Display', None],
  [ 0x80eb, 'CjkUnifiedIdeograph80eb', [], 'Display', None],
  [ 0x80ec, 'CjkUnifiedIdeograph80ec', [], 'Display', None],
  [ 0x80ed, 'CjkUnifiedIdeograph80ed', [], 'Display', None],
  [ 0x80ee, 'CjkUnifiedIdeograph80ee', [], 'Display', None],
  [ 0x80ef, 'CjkUnifiedIdeograph80ef', [], 'Display', None],
  [ 0x80f0, 'CjkUnifiedIdeograph80f0', [], 'Display', None],
  [ 0x80f1, 'CjkUnifiedIdeograph80f1', [], 'Display', None],
  [ 0x80f2, 'CjkUnifiedIdeograph80f2', [], 'Display', None],
  [ 0x80f3, 'CjkUnifiedIdeograph80f3', [], 'Display', None],
  [ 0x80f4, 'CjkUnifiedIdeograph80f4', [], 'Display', None],
  [ 0x80f5, 'CjkUnifiedIdeograph80f5', [], 'Display', None],
  [ 0x80f6, 'CjkUnifiedIdeograph80f6', [], 'Display', None],
  [ 0x80f7, 'CjkUnifiedIdeograph80f7', [], 'Display', None],
  [ 0x80f8, 'CjkUnifiedIdeograph80f8', [], 'Display', None],
  [ 0x80f9, 'CjkUnifiedIdeograph80f9', [], 'Display', None],
  [ 0x80fa, 'CjkUnifiedIdeograph80fa', [], 'Display', None],
  [ 0x80fb, 'CjkUnifiedIdeograph80fb', [], 'Display', None],
  [ 0x80fc, 'CjkUnifiedIdeograph80fc', [], 'Display', None],
  [ 0x80fd, 'CjkUnifiedIdeograph80fd', [], 'Display', None],
  [ 0x80fe, 'CjkUnifiedIdeograph80fe', [], 'Display', None],
  [ 0x80ff, 'CjkUnifiedIdeograph80ff', [], 'Display', None],
  [ 0x8100, 'CjkUnifiedIdeograph8100', [], 'Display', None],
  [ 0x8101, 'CjkUnifiedIdeograph8101', [], 'Display', None],
  [ 0x8102, 'CjkUnifiedIdeograph8102', [], 'Display', None],
  [ 0x8103, 'CjkUnifiedIdeograph8103', [], 'Display', None],
  [ 0x8104, 'CjkUnifiedIdeograph8104', [], 'Display', None],
  [ 0x8105, 'CjkUnifiedIdeograph8105', [], 'Display', None],
  [ 0x8106, 'CjkUnifiedIdeograph8106', [], 'Display', None],
  [ 0x8107, 'CjkUnifiedIdeograph8107', [], 'Display', None],
  [ 0x8108, 'CjkUnifiedIdeograph8108', [], 'Display', None],
  [ 0x8109, 'CjkUnifiedIdeograph8109', [], 'Display', None],
  [ 0x810a, 'CjkUnifiedIdeograph810a', [], 'Display', None],
  [ 0x810b, 'CjkUnifiedIdeograph810b', [], 'Display', None],
  [ 0x810c, 'CjkUnifiedIdeograph810c', [], 'Display', None],
  [ 0x810d, 'CjkUnifiedIdeograph810d', [], 'Display', None],
  [ 0x810e, 'CjkUnifiedIdeograph810e', [], 'Display', None],
  [ 0x810f, 'CjkUnifiedIdeograph810f', [], 'Display', None],
  [ 0x8110, 'CjkUnifiedIdeograph8110', [], 'Display', None],
  [ 0x8111, 'CjkUnifiedIdeograph8111', [], 'Display', None],
  [ 0x8112, 'CjkUnifiedIdeograph8112', [], 'Display', None],
  [ 0x8113, 'CjkUnifiedIdeograph8113', [], 'Display', None],
  [ 0x8114, 'CjkUnifiedIdeograph8114', [], 'Display', None],
  [ 0x8115, 'CjkUnifiedIdeograph8115', [], 'Display', None],
  [ 0x8116, 'CjkUnifiedIdeograph8116', [], 'Display', None],
  [ 0x8117, 'CjkUnifiedIdeograph8117', [], 'Display', None],
  [ 0x8118, 'CjkUnifiedIdeograph8118', [], 'Display', None],
  [ 0x8119, 'CjkUnifiedIdeograph8119', [], 'Display', None],
  [ 0x811a, 'CjkUnifiedIdeograph811a', [], 'Display', None],
  [ 0x811b, 'CjkUnifiedIdeograph811b', [], 'Display', None],
  [ 0x811c, 'CjkUnifiedIdeograph811c', [], 'Display', None],
  [ 0x811d, 'CjkUnifiedIdeograph811d', [], 'Display', None],
  [ 0x811e, 'CjkUnifiedIdeograph811e', [], 'Display', None],
  [ 0x811f, 'CjkUnifiedIdeograph811f', [], 'Display', None],
  [ 0x8120, 'CjkUnifiedIdeograph8120', [], 'Display', None],
  [ 0x8121, 'CjkUnifiedIdeograph8121', [], 'Display', None],
  [ 0x8122, 'CjkUnifiedIdeograph8122', [], 'Display', None],
  [ 0x8123, 'CjkUnifiedIdeograph8123', [], 'Display', None],
  [ 0x8124, 'CjkUnifiedIdeograph8124', [], 'Display', None],
  [ 0x8125, 'CjkUnifiedIdeograph8125', [], 'Display', None],
  [ 0x8126, 'CjkUnifiedIdeograph8126', [], 'Display', None],
  [ 0x8127, 'CjkUnifiedIdeograph8127', [], 'Display', None],
  [ 0x8128, 'CjkUnifiedIdeograph8128', [], 'Display', None],
  [ 0x8129, 'CjkUnifiedIdeograph8129', [], 'Display', None],
  [ 0x812a, 'CjkUnifiedIdeograph812a', [], 'Display', None],
  [ 0x812b, 'CjkUnifiedIdeograph812b', [], 'Display', None],
  [ 0x812c, 'CjkUnifiedIdeograph812c', [], 'Display', None],
  [ 0x812d, 'CjkUnifiedIdeograph812d', [], 'Display', None],
  [ 0x812e, 'CjkUnifiedIdeograph812e', [], 'Display', None],
  [ 0x812f, 'CjkUnifiedIdeograph812f', [], 'Display', None],
  [ 0x8130, 'CjkUnifiedIdeograph8130', [], 'Display', None],
  [ 0x8131, 'CjkUnifiedIdeograph8131', [], 'Display', None],
  [ 0x8132, 'CjkUnifiedIdeograph8132', [], 'Display', None],
  [ 0x8133, 'CjkUnifiedIdeograph8133', [], 'Display', None],
  [ 0x8134, 'CjkUnifiedIdeograph8134', [], 'Display', None],
  [ 0x8135, 'CjkUnifiedIdeograph8135', [], 'Display', None],
  [ 0x8136, 'CjkUnifiedIdeograph8136', [], 'Display', None],
  [ 0x8137, 'CjkUnifiedIdeograph8137', [], 'Display', None],
  [ 0x8138, 'CjkUnifiedIdeograph8138', [], 'Display', None],
  [ 0x8139, 'CjkUnifiedIdeograph8139', [], 'Display', None],
  [ 0x813a, 'CjkUnifiedIdeograph813a', [], 'Display', None],
  [ 0x813b, 'CjkUnifiedIdeograph813b', [], 'Display', None],
  [ 0x813c, 'CjkUnifiedIdeograph813c', [], 'Display', None],
  [ 0x813d, 'CjkUnifiedIdeograph813d', [], 'Display', None],
  [ 0x813e, 'CjkUnifiedIdeograph813e', [], 'Display', None],
  [ 0x813f, 'CjkUnifiedIdeograph813f', [], 'Display', None],
  [ 0x8140, 'CjkUnifiedIdeograph8140', [], 'Display', None],
  [ 0x8141, 'CjkUnifiedIdeograph8141', [], 'Display', None],
  [ 0x8142, 'CjkUnifiedIdeograph8142', [], 'Display', None],
  [ 0x8143, 'CjkUnifiedIdeograph8143', [], 'Display', None],
  [ 0x8144, 'CjkUnifiedIdeograph8144', [], 'Display', None],
  [ 0x8145, 'CjkUnifiedIdeograph8145', [], 'Display', None],
  [ 0x8146, 'CjkUnifiedIdeograph8146', [], 'Display', None],
  [ 0x8147, 'CjkUnifiedIdeograph8147', [], 'Display', None],
  [ 0x8148, 'CjkUnifiedIdeograph8148', [], 'Display', None],
  [ 0x8149, 'CjkUnifiedIdeograph8149', [], 'Display', None],
  [ 0x814a, 'CjkUnifiedIdeograph814a', [], 'Display', None],
  [ 0x814b, 'CjkUnifiedIdeograph814b', [], 'Display', None],
  [ 0x814c, 'CjkUnifiedIdeograph814c', [], 'Display', None],
  [ 0x814d, 'CjkUnifiedIdeograph814d', [], 'Display', None],
  [ 0x814e, 'CjkUnifiedIdeograph814e', [], 'Display', None],
  [ 0x814f, 'CjkUnifiedIdeograph814f', [], 'Display', None],
  [ 0x8150, 'CjkUnifiedIdeograph8150', [], 'Display', None],
  [ 0x8151, 'CjkUnifiedIdeograph8151', [], 'Display', None],
  [ 0x8152, 'CjkUnifiedIdeograph8152', [], 'Display', None],
  [ 0x8153, 'CjkUnifiedIdeograph8153', [], 'Display', None],
  [ 0x8154, 'CjkUnifiedIdeograph8154', [], 'Display', None],
  [ 0x8155, 'CjkUnifiedIdeograph8155', [], 'Display', None],
  [ 0x8156, 'CjkUnifiedIdeograph8156', [], 'Display', None],
  [ 0x8157, 'CjkUnifiedIdeograph8157', [], 'Display', None],
  [ 0x8158, 'CjkUnifiedIdeograph8158', [], 'Display', None],
  [ 0x8159, 'CjkUnifiedIdeograph8159', [], 'Display', None],
  [ 0x815a, 'CjkUnifiedIdeograph815a', [], 'Display', None],
  [ 0x815b, 'CjkUnifiedIdeograph815b', [], 'Display', None],
  [ 0x815c, 'CjkUnifiedIdeograph815c', [], 'Display', None],
  [ 0x815d, 'CjkUnifiedIdeograph815d', [], 'Display', None],
  [ 0x815e, 'CjkUnifiedIdeograph815e', [], 'Display', None],
  [ 0x815f, 'CjkUnifiedIdeograph815f', [], 'Display', None],
  [ 0x8160, 'CjkUnifiedIdeograph8160', [], 'Display', None],
  [ 0x8161, 'CjkUnifiedIdeograph8161', [], 'Display', None],
  [ 0x8162, 'CjkUnifiedIdeograph8162', [], 'Display', None],
  [ 0x8163, 'CjkUnifiedIdeograph8163', [], 'Display', None],
  [ 0x8164, 'CjkUnifiedIdeograph8164', [], 'Display', None],
  [ 0x8165, 'CjkUnifiedIdeograph8165', [], 'Display', None],
  [ 0x8166, 'CjkUnifiedIdeograph8166', [], 'Display', None],
  [ 0x8167, 'CjkUnifiedIdeograph8167', [], 'Display', None],
  [ 0x8168, 'CjkUnifiedIdeograph8168', [], 'Display', None],
  [ 0x8169, 'CjkUnifiedIdeograph8169', [], 'Display', None],
  [ 0x816a, 'CjkUnifiedIdeograph816a', [], 'Display', None],
  [ 0x816b, 'CjkUnifiedIdeograph816b', [], 'Display', None],
  [ 0x816c, 'CjkUnifiedIdeograph816c', [], 'Display', None],
  [ 0x816d, 'CjkUnifiedIdeograph816d', [], 'Display', None],
  [ 0x816e, 'CjkUnifiedIdeograph816e', [], 'Display', None],
  [ 0x816f, 'CjkUnifiedIdeograph816f', [], 'Display', None],
  [ 0x8170, 'CjkUnifiedIdeograph8170', [], 'Display', None],
  [ 0x8171, 'CjkUnifiedIdeograph8171', [], 'Display', None],
  [ 0x8172, 'CjkUnifiedIdeograph8172', [], 'Display', None],
  [ 0x8173, 'CjkUnifiedIdeograph8173', [], 'Display', None],
  [ 0x8174, 'CjkUnifiedIdeograph8174', [], 'Display', None],
  [ 0x8175, 'CjkUnifiedIdeograph8175', [], 'Display', None],
  [ 0x8176, 'CjkUnifiedIdeograph8176', [], 'Display', None],
  [ 0x8177, 'CjkUnifiedIdeograph8177', [], 'Display', None],
  [ 0x8178, 'CjkUnifiedIdeograph8178', [], 'Display', None],
  [ 0x8179, 'CjkUnifiedIdeograph8179', [], 'Display', None],
  [ 0x817a, 'CjkUnifiedIdeograph817a', [], 'Display', None],
  [ 0x817b, 'CjkUnifiedIdeograph817b', [], 'Display', None],
  [ 0x817c, 'CjkUnifiedIdeograph817c', [], 'Display', None],
  [ 0x817d, 'CjkUnifiedIdeograph817d', [], 'Display', None],
  [ 0x817e, 'CjkUnifiedIdeograph817e', [], 'Display', None],
  [ 0x817f, 'CjkUnifiedIdeograph817f', [], 'Display', None],
  [ 0x8180, 'CjkUnifiedIdeograph8180', [], 'Display', None],
  [ 0x8181, 'CjkUnifiedIdeograph8181', [], 'Display', None],
  [ 0x8182, 'CjkUnifiedIdeograph8182', [], 'Display', None],
  [ 0x8183, 'CjkUnifiedIdeograph8183', [], 'Display', None],
  [ 0x8184, 'CjkUnifiedIdeograph8184', [], 'Display', None],
  [ 0x8185, 'CjkUnifiedIdeograph8185', [], 'Display', None],
  [ 0x8186, 'CjkUnifiedIdeograph8186', [], 'Display', None],
  [ 0x8187, 'CjkUnifiedIdeograph8187', [], 'Display', None],
  [ 0x8188, 'CjkUnifiedIdeograph8188', [], 'Display', None],
  [ 0x8189, 'CjkUnifiedIdeograph8189', [], 'Display', None],
  [ 0x818a, 'CjkUnifiedIdeograph818a', [], 'Display', None],
  [ 0x818b, 'CjkUnifiedIdeograph818b', [], 'Display', None],
  [ 0x818c, 'CjkUnifiedIdeograph818c', [], 'Display', None],
  [ 0x818d, 'CjkUnifiedIdeograph818d', [], 'Display', None],
  [ 0x818e, 'CjkUnifiedIdeograph818e', [], 'Display', None],
  [ 0x818f, 'CjkUnifiedIdeograph818f', [], 'Display', None],
  [ 0x8190, 'CjkUnifiedIdeograph8190', [], 'Display', None],
  [ 0x8191, 'CjkUnifiedIdeograph8191', [], 'Display', None],
  [ 0x8192, 'CjkUnifiedIdeograph8192', [], 'Display', None],
  [ 0x8193, 'CjkUnifiedIdeograph8193', [], 'Display', None],
  [ 0x8194, 'CjkUnifiedIdeograph8194', [], 'Display', None],
  [ 0x8195, 'CjkUnifiedIdeograph8195', [], 'Display', None],
  [ 0x8196, 'CjkUnifiedIdeograph8196', [], 'Display', None],
  [ 0x8197, 'CjkUnifiedIdeograph8197', [], 'Display', None],
  [ 0x8198, 'CjkUnifiedIdeograph8198', [], 'Display', None],
  [ 0x8199, 'CjkUnifiedIdeograph8199', [], 'Display', None],
  [ 0x819a, 'CjkUnifiedIdeograph819a', [], 'Display', None],
  [ 0x819b, 'CjkUnifiedIdeograph819b', [], 'Display', None],
  [ 0x819c, 'CjkUnifiedIdeograph819c', [], 'Display', None],
  [ 0x819d, 'CjkUnifiedIdeograph819d', [], 'Display', None],
  [ 0x819e, 'CjkUnifiedIdeograph819e', [], 'Display', None],
  [ 0x819f, 'CjkUnifiedIdeograph819f', [], 'Display', None],
  [ 0x81a0, 'CjkUnifiedIdeograph81a0', [], 'Display', None],
  [ 0x81a1, 'CjkUnifiedIdeograph81a1', [], 'Display', None],
  [ 0x81a2, 'CjkUnifiedIdeograph81a2', [], 'Display', None],
  [ 0x81a3, 'CjkUnifiedIdeograph81a3', [], 'Display', None],
  [ 0x81a4, 'CjkUnifiedIdeograph81a4', [], 'Display', None],
  [ 0x81a5, 'CjkUnifiedIdeograph81a5', [], 'Display', None],
  [ 0x81a6, 'CjkUnifiedIdeograph81a6', [], 'Display', None],
  [ 0x81a7, 'CjkUnifiedIdeograph81a7', [], 'Display', None],
  [ 0x81a8, 'CjkUnifiedIdeograph81a8', [], 'Display', None],
  [ 0x81a9, 'CjkUnifiedIdeograph81a9', [], 'Display', None],
  [ 0x81aa, 'CjkUnifiedIdeograph81aa', [], 'Display', None],
  [ 0x81ab, 'CjkUnifiedIdeograph81ab', [], 'Display', None],
  [ 0x81ac, 'CjkUnifiedIdeograph81ac', [], 'Display', None],
  [ 0x81ad, 'CjkUnifiedIdeograph81ad', [], 'Display', None],
  [ 0x81ae, 'CjkUnifiedIdeograph81ae', [], 'Display', None],
  [ 0x81af, 'CjkUnifiedIdeograph81af', [], 'Display', None],
  [ 0x81b0, 'CjkUnifiedIdeograph81b0', [], 'Display', None],
  [ 0x81b1, 'CjkUnifiedIdeograph81b1', [], 'Display', None],
  [ 0x81b2, 'CjkUnifiedIdeograph81b2', [], 'Display', None],
  [ 0x81b3, 'CjkUnifiedIdeograph81b3', [], 'Display', None],
  [ 0x81b4, 'CjkUnifiedIdeograph81b4', [], 'Display', None],
  [ 0x81b5, 'CjkUnifiedIdeograph81b5', [], 'Display', None],
  [ 0x81b6, 'CjkUnifiedIdeograph81b6', [], 'Display', None],
  [ 0x81b7, 'CjkUnifiedIdeograph81b7', [], 'Display', None],
  [ 0x81b8, 'CjkUnifiedIdeograph81b8', [], 'Display', None],
  [ 0x81b9, 'CjkUnifiedIdeograph81b9', [], 'Display', None],
  [ 0x81ba, 'CjkUnifiedIdeograph81ba', [], 'Display', None],
  [ 0x81bb, 'CjkUnifiedIdeograph81bb', [], 'Display', None],
  [ 0x81bc, 'CjkUnifiedIdeograph81bc', [], 'Display', None],
  [ 0x81bd, 'CjkUnifiedIdeograph81bd', [], 'Display', None],
  [ 0x81be, 'CjkUnifiedIdeograph81be', [], 'Display', None],
  [ 0x81bf, 'CjkUnifiedIdeograph81bf', [], 'Display', None],
  [ 0x81c0, 'CjkUnifiedIdeograph81c0', [], 'Display', None],
  [ 0x81c1, 'CjkUnifiedIdeograph81c1', [], 'Display', None],
  [ 0x81c2, 'CjkUnifiedIdeograph81c2', [], 'Display', None],
  [ 0x81c3, 'CjkUnifiedIdeograph81c3', [], 'Display', None],
  [ 0x81c4, 'CjkUnifiedIdeograph81c4', [], 'Display', None],
  [ 0x81c5, 'CjkUnifiedIdeograph81c5', [], 'Display', None],
  [ 0x81c6, 'CjkUnifiedIdeograph81c6', [], 'Display', None],
  [ 0x81c7, 'CjkUnifiedIdeograph81c7', [], 'Display', None],
  [ 0x81c8, 'CjkUnifiedIdeograph81c8', [], 'Display', None],
  [ 0x81c9, 'CjkUnifiedIdeograph81c9', [], 'Display', None],
  [ 0x81ca, 'CjkUnifiedIdeograph81ca', [], 'Display', None],
  [ 0x81cb, 'CjkUnifiedIdeograph81cb', [], 'Display', None],
  [ 0x81cc, 'CjkUnifiedIdeograph81cc', [], 'Display', None],
  [ 0x81cd, 'CjkUnifiedIdeograph81cd', [], 'Display', None],
  [ 0x81ce, 'CjkUnifiedIdeograph81ce', [], 'Display', None],
  [ 0x81cf, 'CjkUnifiedIdeograph81cf', [], 'Display', None],
  [ 0x81d0, 'CjkUnifiedIdeograph81d0', [], 'Display', None],
  [ 0x81d1, 'CjkUnifiedIdeograph81d1', [], 'Display', None],
  [ 0x81d2, 'CjkUnifiedIdeograph81d2', [], 'Display', None],
  [ 0x81d3, 'CjkUnifiedIdeograph81d3', [], 'Display', None],
  [ 0x81d4, 'CjkUnifiedIdeograph81d4', [], 'Display', None],
  [ 0x81d5, 'CjkUnifiedIdeograph81d5', [], 'Display', None],
  [ 0x81d6, 'CjkUnifiedIdeograph81d6', [], 'Display', None],
  [ 0x81d7, 'CjkUnifiedIdeograph81d7', [], 'Display', None],
  [ 0x81d8, 'CjkUnifiedIdeograph81d8', [], 'Display', None],
  [ 0x81d9, 'CjkUnifiedIdeograph81d9', [], 'Display', None],
  [ 0x81da, 'CjkUnifiedIdeograph81da', [], 'Display', None],
  [ 0x81db, 'CjkUnifiedIdeograph81db', [], 'Display', None],
  [ 0x81dc, 'CjkUnifiedIdeograph81dc', [], 'Display', None],
  [ 0x81dd, 'CjkUnifiedIdeograph81dd', [], 'Display', None],
  [ 0x81de, 'CjkUnifiedIdeograph81de', [], 'Display', None],
  [ 0x81df, 'CjkUnifiedIdeograph81df', [], 'Display', None],
  [ 0x81e0, 'CjkUnifiedIdeograph81e0', [], 'Display', None],
  [ 0x81e1, 'CjkUnifiedIdeograph81e1', [], 'Display', None],
  [ 0x81e2, 'CjkUnifiedIdeograph81e2', [], 'Display', None],
  [ 0x81e3, 'CjkUnifiedIdeograph81e3', [], 'Display', None],
  [ 0x81e4, 'CjkUnifiedIdeograph81e4', [], 'Display', None],
  [ 0x81e5, 'CjkUnifiedIdeograph81e5', [], 'Display', None],
  [ 0x81e6, 'CjkUnifiedIdeograph81e6', [], 'Display', None],
  [ 0x81e7, 'CjkUnifiedIdeograph81e7', [], 'Display', None],
  [ 0x81e8, 'CjkUnifiedIdeograph81e8', [], 'Display', None],
  [ 0x81e9, 'CjkUnifiedIdeograph81e9', [], 'Display', None],
  [ 0x81ea, 'CjkUnifiedIdeograph81ea', [], 'Display', None],
  [ 0x81eb, 'CjkUnifiedIdeograph81eb', [], 'Display', None],
  [ 0x81ec, 'CjkUnifiedIdeograph81ec', [], 'Display', None],
  [ 0x81ed, 'CjkUnifiedIdeograph81ed', [], 'Display', None],
  [ 0x81ee, 'CjkUnifiedIdeograph81ee', [], 'Display', None],
  [ 0x81ef, 'CjkUnifiedIdeograph81ef', [], 'Display', None],
  [ 0x81f0, 'CjkUnifiedIdeograph81f0', [], 'Display', None],
  [ 0x81f1, 'CjkUnifiedIdeograph81f1', [], 'Display', None],
  [ 0x81f2, 'CjkUnifiedIdeograph81f2', [], 'Display', None],
  [ 0x81f3, 'CjkUnifiedIdeograph81f3', [], 'Display', None],
  [ 0x81f4, 'CjkUnifiedIdeograph81f4', [], 'Display', None],
  [ 0x81f5, 'CjkUnifiedIdeograph81f5', [], 'Display', None],
  [ 0x81f6, 'CjkUnifiedIdeograph81f6', [], 'Display', None],
  [ 0x81f7, 'CjkUnifiedIdeograph81f7', [], 'Display', None],
  [ 0x81f8, 'CjkUnifiedIdeograph81f8', [], 'Display', None],
  [ 0x81f9, 'CjkUnifiedIdeograph81f9', [], 'Display', None],
  [ 0x81fa, 'CjkUnifiedIdeograph81fa', [], 'Display', None],
  [ 0x81fb, 'CjkUnifiedIdeograph81fb', [], 'Display', None],
  [ 0x81fc, 'CjkUnifiedIdeograph81fc', [], 'Display', None],
  [ 0x81fd, 'CjkUnifiedIdeograph81fd', [], 'Display', None],
  [ 0x81fe, 'CjkUnifiedIdeograph81fe', [], 'Display', None],
  [ 0x81ff, 'CjkUnifiedIdeograph81ff', [], 'Display', None],
  [ 0x8200, 'CjkUnifiedIdeograph8200', [], 'Display', None],
  [ 0x8201, 'CjkUnifiedIdeograph8201', [], 'Display', None],
  [ 0x8202, 'CjkUnifiedIdeograph8202', [], 'Display', None],
  [ 0x8203, 'CjkUnifiedIdeograph8203', [], 'Display', None],
  [ 0x8204, 'CjkUnifiedIdeograph8204', [], 'Display', None],
  [ 0x8205, 'CjkUnifiedIdeograph8205', [], 'Display', None],
  [ 0x8206, 'CjkUnifiedIdeograph8206', [], 'Display', None],
  [ 0x8207, 'CjkUnifiedIdeograph8207', [], 'Display', None],
  [ 0x8208, 'CjkUnifiedIdeograph8208', [], 'Display', None],
  [ 0x8209, 'CjkUnifiedIdeograph8209', [], 'Display', None],
  [ 0x820a, 'CjkUnifiedIdeograph820a', [], 'Display', None],
  [ 0x820b, 'CjkUnifiedIdeograph820b', [], 'Display', None],
  [ 0x820c, 'CjkUnifiedIdeograph820c', [], 'Display', None],
  [ 0x820d, 'CjkUnifiedIdeograph820d', [], 'Display', None],
  [ 0x820e, 'CjkUnifiedIdeograph820e', [], 'Display', None],
  [ 0x820f, 'CjkUnifiedIdeograph820f', [], 'Display', None],
  [ 0x8210, 'CjkUnifiedIdeograph8210', [], 'Display', None],
  [ 0x8211, 'CjkUnifiedIdeograph8211', [], 'Display', None],
  [ 0x8212, 'CjkUnifiedIdeograph8212', [], 'Display', None],
  [ 0x8213, 'CjkUnifiedIdeograph8213', [], 'Display', None],
  [ 0x8214, 'CjkUnifiedIdeograph8214', [], 'Display', None],
  [ 0x8215, 'CjkUnifiedIdeograph8215', [], 'Display', None],
  [ 0x8216, 'CjkUnifiedIdeograph8216', [], 'Display', None],
  [ 0x8217, 'CjkUnifiedIdeograph8217', [], 'Display', None],
  [ 0x8218, 'CjkUnifiedIdeograph8218', [], 'Display', None],
  [ 0x8219, 'CjkUnifiedIdeograph8219', [], 'Display', None],
  [ 0x821a, 'CjkUnifiedIdeograph821a', [], 'Display', None],
  [ 0x821b, 'CjkUnifiedIdeograph821b', [], 'Display', None],
  [ 0x821c, 'CjkUnifiedIdeograph821c', [], 'Display', None],
  [ 0x821d, 'CjkUnifiedIdeograph821d', [], 'Display', None],
  [ 0x821e, 'CjkUnifiedIdeograph821e', [], 'Display', None],
  [ 0x821f, 'CjkUnifiedIdeograph821f', [], 'Display', None],
  [ 0x8220, 'CjkUnifiedIdeograph8220', [], 'Display', None],
  [ 0x8221, 'CjkUnifiedIdeograph8221', [], 'Display', None],
  [ 0x8222, 'CjkUnifiedIdeograph8222', [], 'Display', None],
  [ 0x8223, 'CjkUnifiedIdeograph8223', [], 'Display', None],
  [ 0x8224, 'CjkUnifiedIdeograph8224', [], 'Display', None],
  [ 0x8225, 'CjkUnifiedIdeograph8225', [], 'Display', None],
  [ 0x8226, 'CjkUnifiedIdeograph8226', [], 'Display', None],
  [ 0x8227, 'CjkUnifiedIdeograph8227', [], 'Display', None],
  [ 0x8228, 'CjkUnifiedIdeograph8228', [], 'Display', None],
  [ 0x8229, 'CjkUnifiedIdeograph8229', [], 'Display', None],
  [ 0x822a, 'CjkUnifiedIdeograph822a', [], 'Display', None],
  [ 0x822b, 'CjkUnifiedIdeograph822b', [], 'Display', None],
  [ 0x822c, 'CjkUnifiedIdeograph822c', [], 'Display', None],
  [ 0x822d, 'CjkUnifiedIdeograph822d', [], 'Display', None],
  [ 0x822e, 'CjkUnifiedIdeograph822e', [], 'Display', None],
  [ 0x822f, 'CjkUnifiedIdeograph822f', [], 'Display', None],
  [ 0x8230, 'CjkUnifiedIdeograph8230', [], 'Display', None],
  [ 0x8231, 'CjkUnifiedIdeograph8231', [], 'Display', None],
  [ 0x8232, 'CjkUnifiedIdeograph8232', [], 'Display', None],
  [ 0x8233, 'CjkUnifiedIdeograph8233', [], 'Display', None],
  [ 0x8234, 'CjkUnifiedIdeograph8234', [], 'Display', None],
  [ 0x8235, 'CjkUnifiedIdeograph8235', [], 'Display', None],
  [ 0x8236, 'CjkUnifiedIdeograph8236', [], 'Display', None],
  [ 0x8237, 'CjkUnifiedIdeograph8237', [], 'Display', None],
  [ 0x8238, 'CjkUnifiedIdeograph8238', [], 'Display', None],
  [ 0x8239, 'CjkUnifiedIdeograph8239', [], 'Display', None],
  [ 0x823a, 'CjkUnifiedIdeograph823a', [], 'Display', None],
  [ 0x823b, 'CjkUnifiedIdeograph823b', [], 'Display', None],
  [ 0x823c, 'CjkUnifiedIdeograph823c', [], 'Display', None],
  [ 0x823d, 'CjkUnifiedIdeograph823d', [], 'Display', None],
  [ 0x823e, 'CjkUnifiedIdeograph823e', [], 'Display', None],
  [ 0x823f, 'CjkUnifiedIdeograph823f', [], 'Display', None],
  [ 0x8240, 'CjkUnifiedIdeograph8240', [], 'Display', None],
  [ 0x8241, 'CjkUnifiedIdeograph8241', [], 'Display', None],
  [ 0x8242, 'CjkUnifiedIdeograph8242', [], 'Display', None],
  [ 0x8243, 'CjkUnifiedIdeograph8243', [], 'Display', None],
  [ 0x8244, 'CjkUnifiedIdeograph8244', [], 'Display', None],
  [ 0x8245, 'CjkUnifiedIdeograph8245', [], 'Display', None],
  [ 0x8246, 'CjkUnifiedIdeograph8246', [], 'Display', None],
  [ 0x8247, 'CjkUnifiedIdeograph8247', [], 'Display', None],
  [ 0x8248, 'CjkUnifiedIdeograph8248', [], 'Display', None],
  [ 0x8249, 'CjkUnifiedIdeograph8249', [], 'Display', None],
  [ 0x824a, 'CjkUnifiedIdeograph824a', [], 'Display', None],
  [ 0x824b, 'CjkUnifiedIdeograph824b', [], 'Display', None],
  [ 0x824c, 'CjkUnifiedIdeograph824c', [], 'Display', None],
  [ 0x824d, 'CjkUnifiedIdeograph824d', [], 'Display', None],
  [ 0x824e, 'CjkUnifiedIdeograph824e', [], 'Display', None],
  [ 0x824f, 'CjkUnifiedIdeograph824f', [], 'Display', None],
  [ 0x8250, 'CjkUnifiedIdeograph8250', [], 'Display', None],
  [ 0x8251, 'CjkUnifiedIdeograph8251', [], 'Display', None],
  [ 0x8252, 'CjkUnifiedIdeograph8252', [], 'Display', None],
  [ 0x8253, 'CjkUnifiedIdeograph8253', [], 'Display', None],
  [ 0x8254, 'CjkUnifiedIdeograph8254', [], 'Display', None],
  [ 0x8255, 'CjkUnifiedIdeograph8255', [], 'Display', None],
  [ 0x8256, 'CjkUnifiedIdeograph8256', [], 'Display', None],
  [ 0x8257, 'CjkUnifiedIdeograph8257', [], 'Display', None],
  [ 0x8258, 'CjkUnifiedIdeograph8258', [], 'Display', None],
  [ 0x8259, 'CjkUnifiedIdeograph8259', [], 'Display', None],
  [ 0x825a, 'CjkUnifiedIdeograph825a', [], 'Display', None],
  [ 0x825b, 'CjkUnifiedIdeograph825b', [], 'Display', None],
  [ 0x825c, 'CjkUnifiedIdeograph825c', [], 'Display', None],
  [ 0x825d, 'CjkUnifiedIdeograph825d', [], 'Display', None],
  [ 0x825e, 'CjkUnifiedIdeograph825e', [], 'Display', None],
  [ 0x825f, 'CjkUnifiedIdeograph825f', [], 'Display', None],
  [ 0x8260, 'CjkUnifiedIdeograph8260', [], 'Display', None],
  [ 0x8261, 'CjkUnifiedIdeograph8261', [], 'Display', None],
  [ 0x8262, 'CjkUnifiedIdeograph8262', [], 'Display', None],
  [ 0x8263, 'CjkUnifiedIdeograph8263', [], 'Display', None],
  [ 0x8264, 'CjkUnifiedIdeograph8264', [], 'Display', None],
  [ 0x8265, 'CjkUnifiedIdeograph8265', [], 'Display', None],
  [ 0x8266, 'CjkUnifiedIdeograph8266', [], 'Display', None],
  [ 0x8267, 'CjkUnifiedIdeograph8267', [], 'Display', None],
  [ 0x8268, 'CjkUnifiedIdeograph8268', [], 'Display', None],
  [ 0x8269, 'CjkUnifiedIdeograph8269', [], 'Display', None],
  [ 0x826a, 'CjkUnifiedIdeograph826a', [], 'Display', None],
  [ 0x826b, 'CjkUnifiedIdeograph826b', [], 'Display', None],
  [ 0x826c, 'CjkUnifiedIdeograph826c', [], 'Display', None],
  [ 0x826d, 'CjkUnifiedIdeograph826d', [], 'Display', None],
  [ 0x826e, 'CjkUnifiedIdeograph826e', [], 'Display', None],
  [ 0x826f, 'CjkUnifiedIdeograph826f', [], 'Display', None],
  [ 0x8270, 'CjkUnifiedIdeograph8270', [], 'Display', None],
  [ 0x8271, 'CjkUnifiedIdeograph8271', [], 'Display', None],
  [ 0x8272, 'CjkUnifiedIdeograph8272', [], 'Display', None],
  [ 0x8273, 'CjkUnifiedIdeograph8273', [], 'Display', None],
  [ 0x8274, 'CjkUnifiedIdeograph8274', [], 'Display', None],
  [ 0x8275, 'CjkUnifiedIdeograph8275', [], 'Display', None],
  [ 0x8276, 'CjkUnifiedIdeograph8276', [], 'Display', None],
  [ 0x8277, 'CjkUnifiedIdeograph8277', [], 'Display', None],
  [ 0x8278, 'CjkUnifiedIdeograph8278', [], 'Display', None],
  [ 0x8279, 'CjkUnifiedIdeograph8279', [], 'Display', None],
  [ 0x827a, 'CjkUnifiedIdeograph827a', [], 'Display', None],
  [ 0x827b, 'CjkUnifiedIdeograph827b', [], 'Display', None],
  [ 0x827c, 'CjkUnifiedIdeograph827c', [], 'Display', None],
  [ 0x827d, 'CjkUnifiedIdeograph827d', [], 'Display', None],
  [ 0x827e, 'CjkUnifiedIdeograph827e', [], 'Display', None],
  [ 0x827f, 'CjkUnifiedIdeograph827f', [], 'Display', None],
  [ 0x8280, 'CjkUnifiedIdeograph8280', [], 'Display', None],
  [ 0x8281, 'CjkUnifiedIdeograph8281', [], 'Display', None],
  [ 0x8282, 'CjkUnifiedIdeograph8282', [], 'Display', None],
  [ 0x8283, 'CjkUnifiedIdeograph8283', [], 'Display', None],
  [ 0x8284, 'CjkUnifiedIdeograph8284', [], 'Display', None],
  [ 0x8285, 'CjkUnifiedIdeograph8285', [], 'Display', None],
  [ 0x8286, 'CjkUnifiedIdeograph8286', [], 'Display', None],
  [ 0x8287, 'CjkUnifiedIdeograph8287', [], 'Display', None],
  [ 0x8288, 'CjkUnifiedIdeograph8288', [], 'Display', None],
  [ 0x8289, 'CjkUnifiedIdeograph8289', [], 'Display', None],
  [ 0x828a, 'CjkUnifiedIdeograph828a', [], 'Display', None],
  [ 0x828b, 'CjkUnifiedIdeograph828b', [], 'Display', None],
  [ 0x828c, 'CjkUnifiedIdeograph828c', [], 'Display', None],
  [ 0x828d, 'CjkUnifiedIdeograph828d', [], 'Display', None],
  [ 0x828e, 'CjkUnifiedIdeograph828e', [], 'Display', None],
  [ 0x828f, 'CjkUnifiedIdeograph828f', [], 'Display', None],
  [ 0x8290, 'CjkUnifiedIdeograph8290', [], 'Display', None],
  [ 0x8291, 'CjkUnifiedIdeograph8291', [], 'Display', None],
  [ 0x8292, 'CjkUnifiedIdeograph8292', [], 'Display', None],
  [ 0x8293, 'CjkUnifiedIdeograph8293', [], 'Display', None],
  [ 0x8294, 'CjkUnifiedIdeograph8294', [], 'Display', None],
  [ 0x8295, 'CjkUnifiedIdeograph8295', [], 'Display', None],
  [ 0x8296, 'CjkUnifiedIdeograph8296', [], 'Display', None],
  [ 0x8297, 'CjkUnifiedIdeograph8297', [], 'Display', None],
  [ 0x8298, 'CjkUnifiedIdeograph8298', [], 'Display', None],
  [ 0x8299, 'CjkUnifiedIdeograph8299', [], 'Display', None],
  [ 0x829a, 'CjkUnifiedIdeograph829a', [], 'Display', None],
  [ 0x829b, 'CjkUnifiedIdeograph829b', [], 'Display', None],
  [ 0x829c, 'CjkUnifiedIdeograph829c', [], 'Display', None],
  [ 0x829d, 'CjkUnifiedIdeograph829d', [], 'Display', None],
  [ 0x829e, 'CjkUnifiedIdeograph829e', [], 'Display', None],
  [ 0x829f, 'CjkUnifiedIdeograph829f', [], 'Display', None],
  [ 0x82a0, 'CjkUnifiedIdeograph82a0', [], 'Display', None],
  [ 0x82a1, 'CjkUnifiedIdeograph82a1', [], 'Display', None],
  [ 0x82a2, 'CjkUnifiedIdeograph82a2', [], 'Display', None],
  [ 0x82a3, 'CjkUnifiedIdeograph82a3', [], 'Display', None],
  [ 0x82a4, 'CjkUnifiedIdeograph82a4', [], 'Display', None],
  [ 0x82a5, 'CjkUnifiedIdeograph82a5', [], 'Display', None],
  [ 0x82a6, 'CjkUnifiedIdeograph82a6', [], 'Display', None],
  [ 0x82a7, 'CjkUnifiedIdeograph82a7', [], 'Display', None],
  [ 0x82a8, 'CjkUnifiedIdeograph82a8', [], 'Display', None],
  [ 0x82a9, 'CjkUnifiedIdeograph82a9', [], 'Display', None],
  [ 0x82aa, 'CjkUnifiedIdeograph82aa', [], 'Display', None],
  [ 0x82ab, 'CjkUnifiedIdeograph82ab', [], 'Display', None],
  [ 0x82ac, 'CjkUnifiedIdeograph82ac', [], 'Display', None],
  [ 0x82ad, 'CjkUnifiedIdeograph82ad', [], 'Display', None],
  [ 0x82ae, 'CjkUnifiedIdeograph82ae', [], 'Display', None],
  [ 0x82af, 'CjkUnifiedIdeograph82af', [], 'Display', None],
  [ 0x82b0, 'CjkUnifiedIdeograph82b0', [], 'Display', None],
  [ 0x82b1, 'CjkUnifiedIdeograph82b1', [], 'Display', None],
  [ 0x82b2, 'CjkUnifiedIdeograph82b2', [], 'Display', None],
  [ 0x82b3, 'CjkUnifiedIdeograph82b3', [], 'Display', None],
  [ 0x82b4, 'CjkUnifiedIdeograph82b4', [], 'Display', None],
  [ 0x82b5, 'CjkUnifiedIdeograph82b5', [], 'Display', None],
  [ 0x82b6, 'CjkUnifiedIdeograph82b6', [], 'Display', None],
  [ 0x82b7, 'CjkUnifiedIdeograph82b7', [], 'Display', None],
  [ 0x82b8, 'CjkUnifiedIdeograph82b8', [], 'Display', None],
  [ 0x82b9, 'CjkUnifiedIdeograph82b9', [], 'Display', None],
  [ 0x82ba, 'CjkUnifiedIdeograph82ba', [], 'Display', None],
  [ 0x82bb, 'CjkUnifiedIdeograph82bb', [], 'Display', None],
  [ 0x82bc, 'CjkUnifiedIdeograph82bc', [], 'Display', None],
  [ 0x82bd, 'CjkUnifiedIdeograph82bd', [], 'Display', None],
  [ 0x82be, 'CjkUnifiedIdeograph82be', [], 'Display', None],
  [ 0x82bf, 'CjkUnifiedIdeograph82bf', [], 'Display', None],
  [ 0x82c0, 'CjkUnifiedIdeograph82c0', [], 'Display', None],
  [ 0x82c1, 'CjkUnifiedIdeograph82c1', [], 'Display', None],
  [ 0x82c2, 'CjkUnifiedIdeograph82c2', [], 'Display', None],
  [ 0x82c3, 'CjkUnifiedIdeograph82c3', [], 'Display', None],
  [ 0x82c4, 'CjkUnifiedIdeograph82c4', [], 'Display', None],
  [ 0x82c5, 'CjkUnifiedIdeograph82c5', [], 'Display', None],
  [ 0x82c6, 'CjkUnifiedIdeograph82c6', [], 'Display', None],
  [ 0x82c7, 'CjkUnifiedIdeograph82c7', [], 'Display', None],
  [ 0x82c8, 'CjkUnifiedIdeograph82c8', [], 'Display', None],
  [ 0x82c9, 'CjkUnifiedIdeograph82c9', [], 'Display', None],
  [ 0x82ca, 'CjkUnifiedIdeograph82ca', [], 'Display', None],
  [ 0x82cb, 'CjkUnifiedIdeograph82cb', [], 'Display', None],
  [ 0x82cc, 'CjkUnifiedIdeograph82cc', [], 'Display', None],
  [ 0x82cd, 'CjkUnifiedIdeograph82cd', [], 'Display', None],
  [ 0x82ce, 'CjkUnifiedIdeograph82ce', [], 'Display', None],
  [ 0x82cf, 'CjkUnifiedIdeograph82cf', [], 'Display', None],
  [ 0x82d0, 'CjkUnifiedIdeograph82d0', [], 'Display', None],
  [ 0x82d1, 'CjkUnifiedIdeograph82d1', [], 'Display', None],
  [ 0x82d2, 'CjkUnifiedIdeograph82d2', [], 'Display', None],
  [ 0x82d3, 'CjkUnifiedIdeograph82d3', [], 'Display', None],
  [ 0x82d4, 'CjkUnifiedIdeograph82d4', [], 'Display', None],
  [ 0x82d5, 'CjkUnifiedIdeograph82d5', [], 'Display', None],
  [ 0x82d6, 'CjkUnifiedIdeograph82d6', [], 'Display', None],
  [ 0x82d7, 'CjkUnifiedIdeograph82d7', [], 'Display', None],
  [ 0x82d8, 'CjkUnifiedIdeograph82d8', [], 'Display', None],
  [ 0x82d9, 'CjkUnifiedIdeograph82d9', [], 'Display', None],
  [ 0x82da, 'CjkUnifiedIdeograph82da', [], 'Display', None],
  [ 0x82db, 'CjkUnifiedIdeograph82db', [], 'Display', None],
  [ 0x82dc, 'CjkUnifiedIdeograph82dc', [], 'Display', None],
  [ 0x82dd, 'CjkUnifiedIdeograph82dd', [], 'Display', None],
  [ 0x82de, 'CjkUnifiedIdeograph82de', [], 'Display', None],
  [ 0x82df, 'CjkUnifiedIdeograph82df', [], 'Display', None],
  [ 0x82e0, 'CjkUnifiedIdeograph82e0', [], 'Display', None],
  [ 0x82e1, 'CjkUnifiedIdeograph82e1', [], 'Display', None],
  [ 0x82e2, 'CjkUnifiedIdeograph82e2', [], 'Display', None],
  [ 0x82e3, 'CjkUnifiedIdeograph82e3', [], 'Display', None],
  [ 0x82e4, 'CjkUnifiedIdeograph82e4', [], 'Display', None],
  [ 0x82e5, 'CjkUnifiedIdeograph82e5', [], 'Display', None],
  [ 0x82e6, 'CjkUnifiedIdeograph82e6', [], 'Display', None],
  [ 0x82e7, 'CjkUnifiedIdeograph82e7', [], 'Display', None],
  [ 0x82e8, 'CjkUnifiedIdeograph82e8', [], 'Display', None],
  [ 0x82e9, 'CjkUnifiedIdeograph82e9', [], 'Display', None],
  [ 0x82ea, 'CjkUnifiedIdeograph82ea', [], 'Display', None],
  [ 0x82eb, 'CjkUnifiedIdeograph82eb', [], 'Display', None],
  [ 0x82ec, 'CjkUnifiedIdeograph82ec', [], 'Display', None],
  [ 0x82ed, 'CjkUnifiedIdeograph82ed', [], 'Display', None],
  [ 0x82ee, 'CjkUnifiedIdeograph82ee', [], 'Display', None],
  [ 0x82ef, 'CjkUnifiedIdeograph82ef', [], 'Display', None],
  [ 0x82f0, 'CjkUnifiedIdeograph82f0', [], 'Display', None],
  [ 0x82f1, 'CjkUnifiedIdeograph82f1', [], 'Display', None],
  [ 0x82f2, 'CjkUnifiedIdeograph82f2', [], 'Display', None],
  [ 0x82f3, 'CjkUnifiedIdeograph82f3', [], 'Display', None],
  [ 0x82f4, 'CjkUnifiedIdeograph82f4', [], 'Display', None],
  [ 0x82f5, 'CjkUnifiedIdeograph82f5', [], 'Display', None],
  [ 0x82f6, 'CjkUnifiedIdeograph82f6', [], 'Display', None],
  [ 0x82f7, 'CjkUnifiedIdeograph82f7', [], 'Display', None],
  [ 0x82f8, 'CjkUnifiedIdeograph82f8', [], 'Display', None],
  [ 0x82f9, 'CjkUnifiedIdeograph82f9', [], 'Display', None],
  [ 0x82fa, 'CjkUnifiedIdeograph82fa', [], 'Display', None],
  [ 0x82fb, 'CjkUnifiedIdeograph82fb', [], 'Display', None],
  [ 0x82fc, 'CjkUnifiedIdeograph82fc', [], 'Display', None],
  [ 0x82fd, 'CjkUnifiedIdeograph82fd', [], 'Display', None],
  [ 0x82fe, 'CjkUnifiedIdeograph82fe', [], 'Display', None],
  [ 0x82ff, 'CjkUnifiedIdeograph82ff', [], 'Display', None],
  [ 0x8300, 'CjkUnifiedIdeograph8300', [], 'Display', None],
  [ 0x8301, 'CjkUnifiedIdeograph8301', [], 'Display', None],
  [ 0x8302, 'CjkUnifiedIdeograph8302', [], 'Display', None],
  [ 0x8303, 'CjkUnifiedIdeograph8303', [], 'Display', None],
  [ 0x8304, 'CjkUnifiedIdeograph8304', [], 'Display', None],
  [ 0x8305, 'CjkUnifiedIdeograph8305', [], 'Display', None],
  [ 0x8306, 'CjkUnifiedIdeograph8306', [], 'Display', None],
  [ 0x8307, 'CjkUnifiedIdeograph8307', [], 'Display', None],
  [ 0x8308, 'CjkUnifiedIdeograph8308', [], 'Display', None],
  [ 0x8309, 'CjkUnifiedIdeograph8309', [], 'Display', None],
  [ 0x830a, 'CjkUnifiedIdeograph830a', [], 'Display', None],
  [ 0x830b, 'CjkUnifiedIdeograph830b', [], 'Display', None],
  [ 0x830c, 'CjkUnifiedIdeograph830c', [], 'Display', None],
  [ 0x830d, 'CjkUnifiedIdeograph830d', [], 'Display', None],
  [ 0x830e, 'CjkUnifiedIdeograph830e', [], 'Display', None],
  [ 0x830f, 'CjkUnifiedIdeograph830f', [], 'Display', None],
  [ 0x8310, 'CjkUnifiedIdeograph8310', [], 'Display', None],
  [ 0x8311, 'CjkUnifiedIdeograph8311', [], 'Display', None],
  [ 0x8312, 'CjkUnifiedIdeograph8312', [], 'Display', None],
  [ 0x8313, 'CjkUnifiedIdeograph8313', [], 'Display', None],
  [ 0x8314, 'CjkUnifiedIdeograph8314', [], 'Display', None],
  [ 0x8315, 'CjkUnifiedIdeograph8315', [], 'Display', None],
  [ 0x8316, 'CjkUnifiedIdeograph8316', [], 'Display', None],
  [ 0x8317, 'CjkUnifiedIdeograph8317', [], 'Display', None],
  [ 0x8318, 'CjkUnifiedIdeograph8318', [], 'Display', None],
  [ 0x8319, 'CjkUnifiedIdeograph8319', [], 'Display', None],
  [ 0x831a, 'CjkUnifiedIdeograph831a', [], 'Display', None],
  [ 0x831b, 'CjkUnifiedIdeograph831b', [], 'Display', None],
  [ 0x831c, 'CjkUnifiedIdeograph831c', [], 'Display', None],
  [ 0x831d, 'CjkUnifiedIdeograph831d', [], 'Display', None],
  [ 0x831e, 'CjkUnifiedIdeograph831e', [], 'Display', None],
  [ 0x831f, 'CjkUnifiedIdeograph831f', [], 'Display', None],
  [ 0x8320, 'CjkUnifiedIdeograph8320', [], 'Display', None],
  [ 0x8321, 'CjkUnifiedIdeograph8321', [], 'Display', None],
  [ 0x8322, 'CjkUnifiedIdeograph8322', [], 'Display', None],
  [ 0x8323, 'CjkUnifiedIdeograph8323', [], 'Display', None],
  [ 0x8324, 'CjkUnifiedIdeograph8324', [], 'Display', None],
  [ 0x8325, 'CjkUnifiedIdeograph8325', [], 'Display', None],
  [ 0x8326, 'CjkUnifiedIdeograph8326', [], 'Display', None],
  [ 0x8327, 'CjkUnifiedIdeograph8327', [], 'Display', None],
  [ 0x8328, 'CjkUnifiedIdeograph8328', [], 'Display', None],
  [ 0x8329, 'CjkUnifiedIdeograph8329', [], 'Display', None],
  [ 0x832a, 'CjkUnifiedIdeograph832a', [], 'Display', None],
  [ 0x832b, 'CjkUnifiedIdeograph832b', [], 'Display', None],
  [ 0x832c, 'CjkUnifiedIdeograph832c', [], 'Display', None],
  [ 0x832d, 'CjkUnifiedIdeograph832d', [], 'Display', None],
  [ 0x832e, 'CjkUnifiedIdeograph832e', [], 'Display', None],
  [ 0x832f, 'CjkUnifiedIdeograph832f', [], 'Display', None],
  [ 0x8330, 'CjkUnifiedIdeograph8330', [], 'Display', None],
  [ 0x8331, 'CjkUnifiedIdeograph8331', [], 'Display', None],
  [ 0x8332, 'CjkUnifiedIdeograph8332', [], 'Display', None],
  [ 0x8333, 'CjkUnifiedIdeograph8333', [], 'Display', None],
  [ 0x8334, 'CjkUnifiedIdeograph8334', [], 'Display', None],
  [ 0x8335, 'CjkUnifiedIdeograph8335', [], 'Display', None],
  [ 0x8336, 'CjkUnifiedIdeograph8336', [], 'Display', None],
  [ 0x8337, 'CjkUnifiedIdeograph8337', [], 'Display', None],
  [ 0x8338, 'CjkUnifiedIdeograph8338', [], 'Display', None],
  [ 0x8339, 'CjkUnifiedIdeograph8339', [], 'Display', None],
  [ 0x833a, 'CjkUnifiedIdeograph833a', [], 'Display', None],
  [ 0x833b, 'CjkUnifiedIdeograph833b', [], 'Display', None],
  [ 0x833c, 'CjkUnifiedIdeograph833c', [], 'Display', None],
  [ 0x833d, 'CjkUnifiedIdeograph833d', [], 'Display', None],
  [ 0x833e, 'CjkUnifiedIdeograph833e', [], 'Display', None],
  [ 0x833f, 'CjkUnifiedIdeograph833f', [], 'Display', None],
  [ 0x8340, 'CjkUnifiedIdeograph8340', [], 'Display', None],
  [ 0x8341, 'CjkUnifiedIdeograph8341', [], 'Display', None],
  [ 0x8342, 'CjkUnifiedIdeograph8342', [], 'Display', None],
  [ 0x8343, 'CjkUnifiedIdeograph8343', [], 'Display', None],
  [ 0x8344, 'CjkUnifiedIdeograph8344', [], 'Display', None],
  [ 0x8345, 'CjkUnifiedIdeograph8345', [], 'Display', None],
  [ 0x8346, 'CjkUnifiedIdeograph8346', [], 'Display', None],
  [ 0x8347, 'CjkUnifiedIdeograph8347', [], 'Display', None],
  [ 0x8348, 'CjkUnifiedIdeograph8348', [], 'Display', None],
  [ 0x8349, 'CjkUnifiedIdeograph8349', [], 'Display', None],
  [ 0x834a, 'CjkUnifiedIdeograph834a', [], 'Display', None],
  [ 0x834b, 'CjkUnifiedIdeograph834b', [], 'Display', None],
  [ 0x834c, 'CjkUnifiedIdeograph834c', [], 'Display', None],
  [ 0x834d, 'CjkUnifiedIdeograph834d', [], 'Display', None],
  [ 0x834e, 'CjkUnifiedIdeograph834e', [], 'Display', None],
  [ 0x834f, 'CjkUnifiedIdeograph834f', [], 'Display', None],
  [ 0x8350, 'CjkUnifiedIdeograph8350', [], 'Display', None],
  [ 0x8351, 'CjkUnifiedIdeograph8351', [], 'Display', None],
  [ 0x8352, 'CjkUnifiedIdeograph8352', [], 'Display', None],
  [ 0x8353, 'CjkUnifiedIdeograph8353', [], 'Display', None],
  [ 0x8354, 'CjkUnifiedIdeograph8354', [], 'Display', None],
  [ 0x8355, 'CjkUnifiedIdeograph8355', [], 'Display', None],
  [ 0x8356, 'CjkUnifiedIdeograph8356', [], 'Display', None],
  [ 0x8357, 'CjkUnifiedIdeograph8357', [], 'Display', None],
  [ 0x8358, 'CjkUnifiedIdeograph8358', [], 'Display', None],
  [ 0x8359, 'CjkUnifiedIdeograph8359', [], 'Display', None],
  [ 0x835a, 'CjkUnifiedIdeograph835a', [], 'Display', None],
  [ 0x835b, 'CjkUnifiedIdeograph835b', [], 'Display', None],
  [ 0x835c, 'CjkUnifiedIdeograph835c', [], 'Display', None],
  [ 0x835d, 'CjkUnifiedIdeograph835d', [], 'Display', None],
  [ 0x835e, 'CjkUnifiedIdeograph835e', [], 'Display', None],
  [ 0x835f, 'CjkUnifiedIdeograph835f', [], 'Display', None],
  [ 0x8360, 'CjkUnifiedIdeograph8360', [], 'Display', None],
  [ 0x8361, 'CjkUnifiedIdeograph8361', [], 'Display', None],
  [ 0x8362, 'CjkUnifiedIdeograph8362', [], 'Display', None],
  [ 0x8363, 'CjkUnifiedIdeograph8363', [], 'Display', None],
  [ 0x8364, 'CjkUnifiedIdeograph8364', [], 'Display', None],
  [ 0x8365, 'CjkUnifiedIdeograph8365', [], 'Display', None],
  [ 0x8366, 'CjkUnifiedIdeograph8366', [], 'Display', None],
  [ 0x8367, 'CjkUnifiedIdeograph8367', [], 'Display', None],
  [ 0x8368, 'CjkUnifiedIdeograph8368', [], 'Display', None],
  [ 0x8369, 'CjkUnifiedIdeograph8369', [], 'Display', None],
  [ 0x836a, 'CjkUnifiedIdeograph836a', [], 'Display', None],
  [ 0x836b, 'CjkUnifiedIdeograph836b', [], 'Display', None],
  [ 0x836c, 'CjkUnifiedIdeograph836c', [], 'Display', None],
  [ 0x836d, 'CjkUnifiedIdeograph836d', [], 'Display', None],
  [ 0x836e, 'CjkUnifiedIdeograph836e', [], 'Display', None],
  [ 0x836f, 'CjkUnifiedIdeograph836f', [], 'Display', None],
  [ 0x8370, 'CjkUnifiedIdeograph8370', [], 'Display', None],
  [ 0x8371, 'CjkUnifiedIdeograph8371', [], 'Display', None],
  [ 0x8372, 'CjkUnifiedIdeograph8372', [], 'Display', None],
  [ 0x8373, 'CjkUnifiedIdeograph8373', [], 'Display', None],
  [ 0x8374, 'CjkUnifiedIdeograph8374', [], 'Display', None],
  [ 0x8375, 'CjkUnifiedIdeograph8375', [], 'Display', None],
  [ 0x8376, 'CjkUnifiedIdeograph8376', [], 'Display', None],
  [ 0x8377, 'CjkUnifiedIdeograph8377', [], 'Display', None],
  [ 0x8378, 'CjkUnifiedIdeograph8378', [], 'Display', None],
  [ 0x8379, 'CjkUnifiedIdeograph8379', [], 'Display', None],
  [ 0x837a, 'CjkUnifiedIdeograph837a', [], 'Display', None],
  [ 0x837b, 'CjkUnifiedIdeograph837b', [], 'Display', None],
  [ 0x837c, 'CjkUnifiedIdeograph837c', [], 'Display', None],
  [ 0x837d, 'CjkUnifiedIdeograph837d', [], 'Display', None],
  [ 0x837e, 'CjkUnifiedIdeograph837e', [], 'Display', None],
  [ 0x837f, 'CjkUnifiedIdeograph837f', [], 'Display', None],
  [ 0x8380, 'CjkUnifiedIdeograph8380', [], 'Display', None],
  [ 0x8381, 'CjkUnifiedIdeograph8381', [], 'Display', None],
  [ 0x8382, 'CjkUnifiedIdeograph8382', [], 'Display', None],
  [ 0x8383, 'CjkUnifiedIdeograph8383', [], 'Display', None],
  [ 0x8384, 'CjkUnifiedIdeograph8384', [], 'Display', None],
  [ 0x8385, 'CjkUnifiedIdeograph8385', [], 'Display', None],
  [ 0x8386, 'CjkUnifiedIdeograph8386', [], 'Display', None],
  [ 0x8387, 'CjkUnifiedIdeograph8387', [], 'Display', None],
  [ 0x8388, 'CjkUnifiedIdeograph8388', [], 'Display', None],
  [ 0x8389, 'CjkUnifiedIdeograph8389', [], 'Display', None],
  [ 0x838a, 'CjkUnifiedIdeograph838a', [], 'Display', None],
  [ 0x838b, 'CjkUnifiedIdeograph838b', [], 'Display', None],
  [ 0x838c, 'CjkUnifiedIdeograph838c', [], 'Display', None],
  [ 0x838d, 'CjkUnifiedIdeograph838d', [], 'Display', None],
  [ 0x838e, 'CjkUnifiedIdeograph838e', [], 'Display', None],
  [ 0x838f, 'CjkUnifiedIdeograph838f', [], 'Display', None],
  [ 0x8390, 'CjkUnifiedIdeograph8390', [], 'Display', None],
  [ 0x8391, 'CjkUnifiedIdeograph8391', [], 'Display', None],
  [ 0x8392, 'CjkUnifiedIdeograph8392', [], 'Display', None],
  [ 0x8393, 'CjkUnifiedIdeograph8393', [], 'Display', None],
  [ 0x8394, 'CjkUnifiedIdeograph8394', [], 'Display', None],
  [ 0x8395, 'CjkUnifiedIdeograph8395', [], 'Display', None],
  [ 0x8396, 'CjkUnifiedIdeograph8396', [], 'Display', None],
  [ 0x8397, 'CjkUnifiedIdeograph8397', [], 'Display', None],
  [ 0x8398, 'CjkUnifiedIdeograph8398', [], 'Display', None],
  [ 0x8399, 'CjkUnifiedIdeograph8399', [], 'Display', None],
  [ 0x839a, 'CjkUnifiedIdeograph839a', [], 'Display', None],
  [ 0x839b, 'CjkUnifiedIdeograph839b', [], 'Display', None],
  [ 0x839c, 'CjkUnifiedIdeograph839c', [], 'Display', None],
  [ 0x839d, 'CjkUnifiedIdeograph839d', [], 'Display', None],
  [ 0x839e, 'CjkUnifiedIdeograph839e', [], 'Display', None],
  [ 0x839f, 'CjkUnifiedIdeograph839f', [], 'Display', None],
  [ 0x83a0, 'CjkUnifiedIdeograph83a0', [], 'Display', None],
  [ 0x83a1, 'CjkUnifiedIdeograph83a1', [], 'Display', None],
  [ 0x83a2, 'CjkUnifiedIdeograph83a2', [], 'Display', None],
  [ 0x83a3, 'CjkUnifiedIdeograph83a3', [], 'Display', None],
  [ 0x83a4, 'CjkUnifiedIdeograph83a4', [], 'Display', None],
  [ 0x83a5, 'CjkUnifiedIdeograph83a5', [], 'Display', None],
  [ 0x83a6, 'CjkUnifiedIdeograph83a6', [], 'Display', None],
  [ 0x83a7, 'CjkUnifiedIdeograph83a7', [], 'Display', None],
  [ 0x83a8, 'CjkUnifiedIdeograph83a8', [], 'Display', None],
  [ 0x83a9, 'CjkUnifiedIdeograph83a9', [], 'Display', None],
  [ 0x83aa, 'CjkUnifiedIdeograph83aa', [], 'Display', None],
  [ 0x83ab, 'CjkUnifiedIdeograph83ab', [], 'Display', None],
  [ 0x83ac, 'CjkUnifiedIdeograph83ac', [], 'Display', None],
  [ 0x83ad, 'CjkUnifiedIdeograph83ad', [], 'Display', None],
  [ 0x83ae, 'CjkUnifiedIdeograph83ae', [], 'Display', None],
  [ 0x83af, 'CjkUnifiedIdeograph83af', [], 'Display', None],
  [ 0x83b0, 'CjkUnifiedIdeograph83b0', [], 'Display', None],
  [ 0x83b1, 'CjkUnifiedIdeograph83b1', [], 'Display', None],
  [ 0x83b2, 'CjkUnifiedIdeograph83b2', [], 'Display', None],
  [ 0x83b3, 'CjkUnifiedIdeograph83b3', [], 'Display', None],
  [ 0x83b4, 'CjkUnifiedIdeograph83b4', [], 'Display', None],
  [ 0x83b5, 'CjkUnifiedIdeograph83b5', [], 'Display', None],
  [ 0x83b6, 'CjkUnifiedIdeograph83b6', [], 'Display', None],
  [ 0x83b7, 'CjkUnifiedIdeograph83b7', [], 'Display', None],
  [ 0x83b8, 'CjkUnifiedIdeograph83b8', [], 'Display', None],
  [ 0x83b9, 'CjkUnifiedIdeograph83b9', [], 'Display', None],
  [ 0x83ba, 'CjkUnifiedIdeograph83ba', [], 'Display', None],
  [ 0x83bb, 'CjkUnifiedIdeograph83bb', [], 'Display', None],
  [ 0x83bc, 'CjkUnifiedIdeograph83bc', [], 'Display', None],
  [ 0x83bd, 'CjkUnifiedIdeograph83bd', [], 'Display', None],
  [ 0x83be, 'CjkUnifiedIdeograph83be', [], 'Display', None],
  [ 0x83bf, 'CjkUnifiedIdeograph83bf', [], 'Display', None],
  [ 0x83c0, 'CjkUnifiedIdeograph83c0', [], 'Display', None],
  [ 0x83c1, 'CjkUnifiedIdeograph83c1', [], 'Display', None],
  [ 0x83c2, 'CjkUnifiedIdeograph83c2', [], 'Display', None],
  [ 0x83c3, 'CjkUnifiedIdeograph83c3', [], 'Display', None],
  [ 0x83c4, 'CjkUnifiedIdeograph83c4', [], 'Display', None],
  [ 0x83c5, 'CjkUnifiedIdeograph83c5', [], 'Display', None],
  [ 0x83c6, 'CjkUnifiedIdeograph83c6', [], 'Display', None],
  [ 0x83c7, 'CjkUnifiedIdeograph83c7', [], 'Display', None],
  [ 0x83c8, 'CjkUnifiedIdeograph83c8', [], 'Display', None],
  [ 0x83c9, 'CjkUnifiedIdeograph83c9', [], 'Display', None],
  [ 0x83ca, 'CjkUnifiedIdeograph83ca', [], 'Display', None],
  [ 0x83cb, 'CjkUnifiedIdeograph83cb', [], 'Display', None],
  [ 0x83cc, 'CjkUnifiedIdeograph83cc', [], 'Display', None],
  [ 0x83cd, 'CjkUnifiedIdeograph83cd', [], 'Display', None],
  [ 0x83ce, 'CjkUnifiedIdeograph83ce', [], 'Display', None],
  [ 0x83cf, 'CjkUnifiedIdeograph83cf', [], 'Display', None],
  [ 0x83d0, 'CjkUnifiedIdeograph83d0', [], 'Display', None],
  [ 0x83d1, 'CjkUnifiedIdeograph83d1', [], 'Display', None],
  [ 0x83d2, 'CjkUnifiedIdeograph83d2', [], 'Display', None],
  [ 0x83d3, 'CjkUnifiedIdeograph83d3', [], 'Display', None],
  [ 0x83d4, 'CjkUnifiedIdeograph83d4', [], 'Display', None],
  [ 0x83d5, 'CjkUnifiedIdeograph83d5', [], 'Display', None],
  [ 0x83d6, 'CjkUnifiedIdeograph83d6', [], 'Display', None],
  [ 0x83d7, 'CjkUnifiedIdeograph83d7', [], 'Display', None],
  [ 0x83d8, 'CjkUnifiedIdeograph83d8', [], 'Display', None],
  [ 0x83d9, 'CjkUnifiedIdeograph83d9', [], 'Display', None],
  [ 0x83da, 'CjkUnifiedIdeograph83da', [], 'Display', None],
  [ 0x83db, 'CjkUnifiedIdeograph83db', [], 'Display', None],
  [ 0x83dc, 'CjkUnifiedIdeograph83dc', [], 'Display', None],
  [ 0x83dd, 'CjkUnifiedIdeograph83dd', [], 'Display', None],
  [ 0x83de, 'CjkUnifiedIdeograph83de', [], 'Display', None],
  [ 0x83df, 'CjkUnifiedIdeograph83df', [], 'Display', None],
  [ 0x83e0, 'CjkUnifiedIdeograph83e0', [], 'Display', None],
  [ 0x83e1, 'CjkUnifiedIdeograph83e1', [], 'Display', None],
  [ 0x83e2, 'CjkUnifiedIdeograph83e2', [], 'Display', None],
  [ 0x83e3, 'CjkUnifiedIdeograph83e3', [], 'Display', None],
  [ 0x83e4, 'CjkUnifiedIdeograph83e4', [], 'Display', None],
  [ 0x83e5, 'CjkUnifiedIdeograph83e5', [], 'Display', None],
  [ 0x83e6, 'CjkUnifiedIdeograph83e6', [], 'Display', None],
  [ 0x83e7, 'CjkUnifiedIdeograph83e7', [], 'Display', None],
  [ 0x83e8, 'CjkUnifiedIdeograph83e8', [], 'Display', None],
  [ 0x83e9, 'CjkUnifiedIdeograph83e9', [], 'Display', None],
  [ 0x83ea, 'CjkUnifiedIdeograph83ea', [], 'Display', None],
  [ 0x83eb, 'CjkUnifiedIdeograph83eb', [], 'Display', None],
  [ 0x83ec, 'CjkUnifiedIdeograph83ec', [], 'Display', None],
  [ 0x83ed, 'CjkUnifiedIdeograph83ed', [], 'Display', None],
  [ 0x83ee, 'CjkUnifiedIdeograph83ee', [], 'Display', None],
  [ 0x83ef, 'CjkUnifiedIdeograph83ef', [], 'Display', None],
  [ 0x83f0, 'CjkUnifiedIdeograph83f0', [], 'Display', None],
  [ 0x83f1, 'CjkUnifiedIdeograph83f1', [], 'Display', None],
  [ 0x83f2, 'CjkUnifiedIdeograph83f2', [], 'Display', None],
  [ 0x83f3, 'CjkUnifiedIdeograph83f3', [], 'Display', None],
  [ 0x83f4, 'CjkUnifiedIdeograph83f4', [], 'Display', None],
  [ 0x83f5, 'CjkUnifiedIdeograph83f5', [], 'Display', None],
  [ 0x83f6, 'CjkUnifiedIdeograph83f6', [], 'Display', None],
  [ 0x83f7, 'CjkUnifiedIdeograph83f7', [], 'Display', None],
  [ 0x83f8, 'CjkUnifiedIdeograph83f8', [], 'Display', None],
  [ 0x83f9, 'CjkUnifiedIdeograph83f9', [], 'Display', None],
  [ 0x83fa, 'CjkUnifiedIdeograph83fa', [], 'Display', None],
  [ 0x83fb, 'CjkUnifiedIdeograph83fb', [], 'Display', None],
  [ 0x83fc, 'CjkUnifiedIdeograph83fc', [], 'Display', None],
  [ 0x83fd, 'CjkUnifiedIdeograph83fd', [], 'Display', None],
  [ 0x83fe, 'CjkUnifiedIdeograph83fe', [], 'Display', None],
  [ 0x83ff, 'CjkUnifiedIdeograph83ff', [], 'Display', None],
  [ 0x8400, 'CjkUnifiedIdeograph8400', [], 'Display', None],
  [ 0x8401, 'CjkUnifiedIdeograph8401', [], 'Display', None],
  [ 0x8402, 'CjkUnifiedIdeograph8402', [], 'Display', None],
  [ 0x8403, 'CjkUnifiedIdeograph8403', [], 'Display', None],
  [ 0x8404, 'CjkUnifiedIdeograph8404', [], 'Display', None],
  [ 0x8405, 'CjkUnifiedIdeograph8405', [], 'Display', None],
  [ 0x8406, 'CjkUnifiedIdeograph8406', [], 'Display', None],
  [ 0x8407, 'CjkUnifiedIdeograph8407', [], 'Display', None],
  [ 0x8408, 'CjkUnifiedIdeograph8408', [], 'Display', None],
  [ 0x8409, 'CjkUnifiedIdeograph8409', [], 'Display', None],
  [ 0x840a, 'CjkUnifiedIdeograph840a', [], 'Display', None],
  [ 0x840b, 'CjkUnifiedIdeograph840b', [], 'Display', None],
  [ 0x840c, 'CjkUnifiedIdeograph840c', [], 'Display', None],
  [ 0x840d, 'CjkUnifiedIdeograph840d', [], 'Display', None],
  [ 0x840e, 'CjkUnifiedIdeograph840e', [], 'Display', None],
  [ 0x840f, 'CjkUnifiedIdeograph840f', [], 'Display', None],
  [ 0x8410, 'CjkUnifiedIdeograph8410', [], 'Display', None],
  [ 0x8411, 'CjkUnifiedIdeograph8411', [], 'Display', None],
  [ 0x8412, 'CjkUnifiedIdeograph8412', [], 'Display', None],
  [ 0x8413, 'CjkUnifiedIdeograph8413', [], 'Display', None],
  [ 0x8414, 'CjkUnifiedIdeograph8414', [], 'Display', None],
  [ 0x8415, 'CjkUnifiedIdeograph8415', [], 'Display', None],
  [ 0x8416, 'CjkUnifiedIdeograph8416', [], 'Display', None],
  [ 0x8417, 'CjkUnifiedIdeograph8417', [], 'Display', None],
  [ 0x8418, 'CjkUnifiedIdeograph8418', [], 'Display', None],
  [ 0x8419, 'CjkUnifiedIdeograph8419', [], 'Display', None],
  [ 0x841a, 'CjkUnifiedIdeograph841a', [], 'Display', None],
  [ 0x841b, 'CjkUnifiedIdeograph841b', [], 'Display', None],
  [ 0x841c, 'CjkUnifiedIdeograph841c', [], 'Display', None],
  [ 0x841d, 'CjkUnifiedIdeograph841d', [], 'Display', None],
  [ 0x841e, 'CjkUnifiedIdeograph841e', [], 'Display', None],
  [ 0x841f, 'CjkUnifiedIdeograph841f', [], 'Display', None],
  [ 0x8420, 'CjkUnifiedIdeograph8420', [], 'Display', None],
  [ 0x8421, 'CjkUnifiedIdeograph8421', [], 'Display', None],
  [ 0x8422, 'CjkUnifiedIdeograph8422', [], 'Display', None],
  [ 0x8423, 'CjkUnifiedIdeograph8423', [], 'Display', None],
  [ 0x8424, 'CjkUnifiedIdeograph8424', [], 'Display', None],
  [ 0x8425, 'CjkUnifiedIdeograph8425', [], 'Display', None],
  [ 0x8426, 'CjkUnifiedIdeograph8426', [], 'Display', None],
  [ 0x8427, 'CjkUnifiedIdeograph8427', [], 'Display', None],
  [ 0x8428, 'CjkUnifiedIdeograph8428', [], 'Display', None],
  [ 0x8429, 'CjkUnifiedIdeograph8429', [], 'Display', None],
  [ 0x842a, 'CjkUnifiedIdeograph842a', [], 'Display', None],
  [ 0x842b, 'CjkUnifiedIdeograph842b', [], 'Display', None],
  [ 0x842c, 'CjkUnifiedIdeograph842c', [], 'Display', None],
  [ 0x842d, 'CjkUnifiedIdeograph842d', [], 'Display', None],
  [ 0x842e, 'CjkUnifiedIdeograph842e', [], 'Display', None],
  [ 0x842f, 'CjkUnifiedIdeograph842f', [], 'Display', None],
  [ 0x8430, 'CjkUnifiedIdeograph8430', [], 'Display', None],
  [ 0x8431, 'CjkUnifiedIdeograph8431', [], 'Display', None],
  [ 0x8432, 'CjkUnifiedIdeograph8432', [], 'Display', None],
  [ 0x8433, 'CjkUnifiedIdeograph8433', [], 'Display', None],
  [ 0x8434, 'CjkUnifiedIdeograph8434', [], 'Display', None],
  [ 0x8435, 'CjkUnifiedIdeograph8435', [], 'Display', None],
  [ 0x8436, 'CjkUnifiedIdeograph8436', [], 'Display', None],
  [ 0x8437, 'CjkUnifiedIdeograph8437', [], 'Display', None],
  [ 0x8438, 'CjkUnifiedIdeograph8438', [], 'Display', None],
  [ 0x8439, 'CjkUnifiedIdeograph8439', [], 'Display', None],
  [ 0x843a, 'CjkUnifiedIdeograph843a', [], 'Display', None],
  [ 0x843b, 'CjkUnifiedIdeograph843b', [], 'Display', None],
  [ 0x843c, 'CjkUnifiedIdeograph843c', [], 'Display', None],
  [ 0x843d, 'CjkUnifiedIdeograph843d', [], 'Display', None],
  [ 0x843e, 'CjkUnifiedIdeograph843e', [], 'Display', None],
  [ 0x843f, 'CjkUnifiedIdeograph843f', [], 'Display', None],
  [ 0x8440, 'CjkUnifiedIdeograph8440', [], 'Display', None],
  [ 0x8441, 'CjkUnifiedIdeograph8441', [], 'Display', None],
  [ 0x8442, 'CjkUnifiedIdeograph8442', [], 'Display', None],
  [ 0x8443, 'CjkUnifiedIdeograph8443', [], 'Display', None],
  [ 0x8444, 'CjkUnifiedIdeograph8444', [], 'Display', None],
  [ 0x8445, 'CjkUnifiedIdeograph8445', [], 'Display', None],
  [ 0x8446, 'CjkUnifiedIdeograph8446', [], 'Display', None],
  [ 0x8447, 'CjkUnifiedIdeograph8447', [], 'Display', None],
  [ 0x8448, 'CjkUnifiedIdeograph8448', [], 'Display', None],
  [ 0x8449, 'CjkUnifiedIdeograph8449', [], 'Display', None],
  [ 0x844a, 'CjkUnifiedIdeograph844a', [], 'Display', None],
  [ 0x844b, 'CjkUnifiedIdeograph844b', [], 'Display', None],
  [ 0x844c, 'CjkUnifiedIdeograph844c', [], 'Display', None],
  [ 0x844d, 'CjkUnifiedIdeograph844d', [], 'Display', None],
  [ 0x844e, 'CjkUnifiedIdeograph844e', [], 'Display', None],
  [ 0x844f, 'CjkUnifiedIdeograph844f', [], 'Display', None],
  [ 0x8450, 'CjkUnifiedIdeograph8450', [], 'Display', None],
  [ 0x8451, 'CjkUnifiedIdeograph8451', [], 'Display', None],
  [ 0x8452, 'CjkUnifiedIdeograph8452', [], 'Display', None],
  [ 0x8453, 'CjkUnifiedIdeograph8453', [], 'Display', None],
  [ 0x8454, 'CjkUnifiedIdeograph8454', [], 'Display', None],
  [ 0x8455, 'CjkUnifiedIdeograph8455', [], 'Display', None],
  [ 0x8456, 'CjkUnifiedIdeograph8456', [], 'Display', None],
  [ 0x8457, 'CjkUnifiedIdeograph8457', [], 'Display', None],
  [ 0x8458, 'CjkUnifiedIdeograph8458', [], 'Display', None],
  [ 0x8459, 'CjkUnifiedIdeograph8459', [], 'Display', None],
  [ 0x845a, 'CjkUnifiedIdeograph845a', [], 'Display', None],
  [ 0x845b, 'CjkUnifiedIdeograph845b', [], 'Display', None],
  [ 0x845c, 'CjkUnifiedIdeograph845c', [], 'Display', None],
  [ 0x845d, 'CjkUnifiedIdeograph845d', [], 'Display', None],
  [ 0x845e, 'CjkUnifiedIdeograph845e', [], 'Display', None],
  [ 0x845f, 'CjkUnifiedIdeograph845f', [], 'Display', None],
  [ 0x8460, 'CjkUnifiedIdeograph8460', [], 'Display', None],
  [ 0x8461, 'CjkUnifiedIdeograph8461', [], 'Display', None],
  [ 0x8462, 'CjkUnifiedIdeograph8462', [], 'Display', None],
  [ 0x8463, 'CjkUnifiedIdeograph8463', [], 'Display', None],
  [ 0x8464, 'CjkUnifiedIdeograph8464', [], 'Display', None],
  [ 0x8465, 'CjkUnifiedIdeograph8465', [], 'Display', None],
  [ 0x8466, 'CjkUnifiedIdeograph8466', [], 'Display', None],
  [ 0x8467, 'CjkUnifiedIdeograph8467', [], 'Display', None],
  [ 0x8468, 'CjkUnifiedIdeograph8468', [], 'Display', None],
  [ 0x8469, 'CjkUnifiedIdeograph8469', [], 'Display', None],
  [ 0x846a, 'CjkUnifiedIdeograph846a', [], 'Display', None],
  [ 0x846b, 'CjkUnifiedIdeograph846b', [], 'Display', None],
  [ 0x846c, 'CjkUnifiedIdeograph846c', [], 'Display', None],
  [ 0x846d, 'CjkUnifiedIdeograph846d', [], 'Display', None],
  [ 0x846e, 'CjkUnifiedIdeograph846e', [], 'Display', None],
  [ 0x846f, 'CjkUnifiedIdeograph846f', [], 'Display', None],
  [ 0x8470, 'CjkUnifiedIdeograph8470', [], 'Display', None],
  [ 0x8471, 'CjkUnifiedIdeograph8471', [], 'Display', None],
  [ 0x8472, 'CjkUnifiedIdeograph8472', [], 'Display', None],
  [ 0x8473, 'CjkUnifiedIdeograph8473', [], 'Display', None],
  [ 0x8474, 'CjkUnifiedIdeograph8474', [], 'Display', None],
  [ 0x8475, 'CjkUnifiedIdeograph8475', [], 'Display', None],
  [ 0x8476, 'CjkUnifiedIdeograph8476', [], 'Display', None],
  [ 0x8477, 'CjkUnifiedIdeograph8477', [], 'Display', None],
  [ 0x8478, 'CjkUnifiedIdeograph8478', [], 'Display', None],
  [ 0x8479, 'CjkUnifiedIdeograph8479', [], 'Display', None],
  [ 0x847a, 'CjkUnifiedIdeograph847a', [], 'Display', None],
  [ 0x847b, 'CjkUnifiedIdeograph847b', [], 'Display', None],
  [ 0x847c, 'CjkUnifiedIdeograph847c', [], 'Display', None],
  [ 0x847d, 'CjkUnifiedIdeograph847d', [], 'Display', None],
  [ 0x847e, 'CjkUnifiedIdeograph847e', [], 'Display', None],
  [ 0x847f, 'CjkUnifiedIdeograph847f', [], 'Display', None],
  [ 0x8480, 'CjkUnifiedIdeograph8480', [], 'Display', None],
  [ 0x8481, 'CjkUnifiedIdeograph8481', [], 'Display', None],
  [ 0x8482, 'CjkUnifiedIdeograph8482', [], 'Display', None],
  [ 0x8483, 'CjkUnifiedIdeograph8483', [], 'Display', None],
  [ 0x8484, 'CjkUnifiedIdeograph8484', [], 'Display', None],
  [ 0x8485, 'CjkUnifiedIdeograph8485', [], 'Display', None],
  [ 0x8486, 'CjkUnifiedIdeograph8486', [], 'Display', None],
  [ 0x8487, 'CjkUnifiedIdeograph8487', [], 'Display', None],
  [ 0x8488, 'CjkUnifiedIdeograph8488', [], 'Display', None],
  [ 0x8489, 'CjkUnifiedIdeograph8489', [], 'Display', None],
  [ 0x848a, 'CjkUnifiedIdeograph848a', [], 'Display', None],
  [ 0x848b, 'CjkUnifiedIdeograph848b', [], 'Display', None],
  [ 0x848c, 'CjkUnifiedIdeograph848c', [], 'Display', None],
  [ 0x848d, 'CjkUnifiedIdeograph848d', [], 'Display', None],
  [ 0x848e, 'CjkUnifiedIdeograph848e', [], 'Display', None],
  [ 0x848f, 'CjkUnifiedIdeograph848f', [], 'Display', None],
  [ 0x8490, 'CjkUnifiedIdeograph8490', [], 'Display', None],
  [ 0x8491, 'CjkUnifiedIdeograph8491', [], 'Display', None],
  [ 0x8492, 'CjkUnifiedIdeograph8492', [], 'Display', None],
  [ 0x8493, 'CjkUnifiedIdeograph8493', [], 'Display', None],
  [ 0x8494, 'CjkUnifiedIdeograph8494', [], 'Display', None],
  [ 0x8495, 'CjkUnifiedIdeograph8495', [], 'Display', None],
  [ 0x8496, 'CjkUnifiedIdeograph8496', [], 'Display', None],
  [ 0x8497, 'CjkUnifiedIdeograph8497', [], 'Display', None],
  [ 0x8498, 'CjkUnifiedIdeograph8498', [], 'Display', None],
  [ 0x8499, 'CjkUnifiedIdeograph8499', [], 'Display', None],
  [ 0x849a, 'CjkUnifiedIdeograph849a', [], 'Display', None],
  [ 0x849b, 'CjkUnifiedIdeograph849b', [], 'Display', None],
  [ 0x849c, 'CjkUnifiedIdeograph849c', [], 'Display', None],
  [ 0x849d, 'CjkUnifiedIdeograph849d', [], 'Display', None],
  [ 0x849e, 'CjkUnifiedIdeograph849e', [], 'Display', None],
  [ 0x849f, 'CjkUnifiedIdeograph849f', [], 'Display', None],
  [ 0x84a0, 'CjkUnifiedIdeograph84a0', [], 'Display', None],
  [ 0x84a1, 'CjkUnifiedIdeograph84a1', [], 'Display', None],
  [ 0x84a2, 'CjkUnifiedIdeograph84a2', [], 'Display', None],
  [ 0x84a3, 'CjkUnifiedIdeograph84a3', [], 'Display', None],
  [ 0x84a4, 'CjkUnifiedIdeograph84a4', [], 'Display', None],
  [ 0x84a5, 'CjkUnifiedIdeograph84a5', [], 'Display', None],
  [ 0x84a6, 'CjkUnifiedIdeograph84a6', [], 'Display', None],
  [ 0x84a7, 'CjkUnifiedIdeograph84a7', [], 'Display', None],
  [ 0x84a8, 'CjkUnifiedIdeograph84a8', [], 'Display', None],
  [ 0x84a9, 'CjkUnifiedIdeograph84a9', [], 'Display', None],
  [ 0x84aa, 'CjkUnifiedIdeograph84aa', [], 'Display', None],
  [ 0x84ab, 'CjkUnifiedIdeograph84ab', [], 'Display', None],
  [ 0x84ac, 'CjkUnifiedIdeograph84ac', [], 'Display', None],
  [ 0x84ad, 'CjkUnifiedIdeograph84ad', [], 'Display', None],
  [ 0x84ae, 'CjkUnifiedIdeograph84ae', [], 'Display', None],
  [ 0x84af, 'CjkUnifiedIdeograph84af', [], 'Display', None],
  [ 0x84b0, 'CjkUnifiedIdeograph84b0', [], 'Display', None],
  [ 0x84b1, 'CjkUnifiedIdeograph84b1', [], 'Display', None],
  [ 0x84b2, 'CjkUnifiedIdeograph84b2', [], 'Display', None],
  [ 0x84b3, 'CjkUnifiedIdeograph84b3', [], 'Display', None],
  [ 0x84b4, 'CjkUnifiedIdeograph84b4', [], 'Display', None],
  [ 0x84b5, 'CjkUnifiedIdeograph84b5', [], 'Display', None],
  [ 0x84b6, 'CjkUnifiedIdeograph84b6', [], 'Display', None],
  [ 0x84b7, 'CjkUnifiedIdeograph84b7', [], 'Display', None],
  [ 0x84b8, 'CjkUnifiedIdeograph84b8', [], 'Display', None],
  [ 0x84b9, 'CjkUnifiedIdeograph84b9', [], 'Display', None],
  [ 0x84ba, 'CjkUnifiedIdeograph84ba', [], 'Display', None],
  [ 0x84bb, 'CjkUnifiedIdeograph84bb', [], 'Display', None],
  [ 0x84bc, 'CjkUnifiedIdeograph84bc', [], 'Display', None],
  [ 0x84bd, 'CjkUnifiedIdeograph84bd', [], 'Display', None],
  [ 0x84be, 'CjkUnifiedIdeograph84be', [], 'Display', None],
  [ 0x84bf, 'CjkUnifiedIdeograph84bf', [], 'Display', None],
  [ 0x84c0, 'CjkUnifiedIdeograph84c0', [], 'Display', None],
  [ 0x84c1, 'CjkUnifiedIdeograph84c1', [], 'Display', None],
  [ 0x84c2, 'CjkUnifiedIdeograph84c2', [], 'Display', None],
  [ 0x84c3, 'CjkUnifiedIdeograph84c3', [], 'Display', None],
  [ 0x84c4, 'CjkUnifiedIdeograph84c4', [], 'Display', None],
  [ 0x84c5, 'CjkUnifiedIdeograph84c5', [], 'Display', None],
  [ 0x84c6, 'CjkUnifiedIdeograph84c6', [], 'Display', None],
  [ 0x84c7, 'CjkUnifiedIdeograph84c7', [], 'Display', None],
  [ 0x84c8, 'CjkUnifiedIdeograph84c8', [], 'Display', None],
  [ 0x84c9, 'CjkUnifiedIdeograph84c9', [], 'Display', None],
  [ 0x84ca, 'CjkUnifiedIdeograph84ca', [], 'Display', None],
  [ 0x84cb, 'CjkUnifiedIdeograph84cb', [], 'Display', None],
  [ 0x84cc, 'CjkUnifiedIdeograph84cc', [], 'Display', None],
  [ 0x84cd, 'CjkUnifiedIdeograph84cd', [], 'Display', None],
  [ 0x84ce, 'CjkUnifiedIdeograph84ce', [], 'Display', None],
  [ 0x84cf, 'CjkUnifiedIdeograph84cf', [], 'Display', None],
  [ 0x84d0, 'CjkUnifiedIdeograph84d0', [], 'Display', None],
  [ 0x84d1, 'CjkUnifiedIdeograph84d1', [], 'Display', None],
  [ 0x84d2, 'CjkUnifiedIdeograph84d2', [], 'Display', None],
  [ 0x84d3, 'CjkUnifiedIdeograph84d3', [], 'Display', None],
  [ 0x84d4, 'CjkUnifiedIdeograph84d4', [], 'Display', None],
  [ 0x84d5, 'CjkUnifiedIdeograph84d5', [], 'Display', None],
  [ 0x84d6, 'CjkUnifiedIdeograph84d6', [], 'Display', None],
  [ 0x84d7, 'CjkUnifiedIdeograph84d7', [], 'Display', None],
  [ 0x84d8, 'CjkUnifiedIdeograph84d8', [], 'Display', None],
  [ 0x84d9, 'CjkUnifiedIdeograph84d9', [], 'Display', None],
  [ 0x84da, 'CjkUnifiedIdeograph84da', [], 'Display', None],
  [ 0x84db, 'CjkUnifiedIdeograph84db', [], 'Display', None],
  [ 0x84dc, 'CjkUnifiedIdeograph84dc', [], 'Display', None],
  [ 0x84dd, 'CjkUnifiedIdeograph84dd', [], 'Display', None],
  [ 0x84de, 'CjkUnifiedIdeograph84de', [], 'Display', None],
  [ 0x84df, 'CjkUnifiedIdeograph84df', [], 'Display', None],
  [ 0x84e0, 'CjkUnifiedIdeograph84e0', [], 'Display', None],
  [ 0x84e1, 'CjkUnifiedIdeograph84e1', [], 'Display', None],
  [ 0x84e2, 'CjkUnifiedIdeograph84e2', [], 'Display', None],
  [ 0x84e3, 'CjkUnifiedIdeograph84e3', [], 'Display', None],
  [ 0x84e4, 'CjkUnifiedIdeograph84e4', [], 'Display', None],
  [ 0x84e5, 'CjkUnifiedIdeograph84e5', [], 'Display', None],
  [ 0x84e6, 'CjkUnifiedIdeograph84e6', [], 'Display', None],
  [ 0x84e7, 'CjkUnifiedIdeograph84e7', [], 'Display', None],
  [ 0x84e8, 'CjkUnifiedIdeograph84e8', [], 'Display', None],
  [ 0x84e9, 'CjkUnifiedIdeograph84e9', [], 'Display', None],
  [ 0x84ea, 'CjkUnifiedIdeograph84ea', [], 'Display', None],
  [ 0x84eb, 'CjkUnifiedIdeograph84eb', [], 'Display', None],
  [ 0x84ec, 'CjkUnifiedIdeograph84ec', [], 'Display', None],
  [ 0x84ed, 'CjkUnifiedIdeograph84ed', [], 'Display', None],
  [ 0x84ee, 'CjkUnifiedIdeograph84ee', [], 'Display', None],
  [ 0x84ef, 'CjkUnifiedIdeograph84ef', [], 'Display', None],
  [ 0x84f0, 'CjkUnifiedIdeograph84f0', [], 'Display', None],
  [ 0x84f1, 'CjkUnifiedIdeograph84f1', [], 'Display', None],
  [ 0x84f2, 'CjkUnifiedIdeograph84f2', [], 'Display', None],
  [ 0x84f3, 'CjkUnifiedIdeograph84f3', [], 'Display', None],
  [ 0x84f4, 'CjkUnifiedIdeograph84f4', [], 'Display', None],
  [ 0x84f5, 'CjkUnifiedIdeograph84f5', [], 'Display', None],
  [ 0x84f6, 'CjkUnifiedIdeograph84f6', [], 'Display', None],
  [ 0x84f7, 'CjkUnifiedIdeograph84f7', [], 'Display', None],
  [ 0x84f8, 'CjkUnifiedIdeograph84f8', [], 'Display', None],
  [ 0x84f9, 'CjkUnifiedIdeograph84f9', [], 'Display', None],
  [ 0x84fa, 'CjkUnifiedIdeograph84fa', [], 'Display', None],
  [ 0x84fb, 'CjkUnifiedIdeograph84fb', [], 'Display', None],
  [ 0x84fc, 'CjkUnifiedIdeograph84fc', [], 'Display', None],
  [ 0x84fd, 'CjkUnifiedIdeograph84fd', [], 'Display', None],
  [ 0x84fe, 'CjkUnifiedIdeograph84fe', [], 'Display', None],
  [ 0x84ff, 'CjkUnifiedIdeograph84ff', [], 'Display', None],
  [ 0x8500, 'CjkUnifiedIdeograph8500', [], 'Display', None],
  [ 0x8501, 'CjkUnifiedIdeograph8501', [], 'Display', None],
  [ 0x8502, 'CjkUnifiedIdeograph8502', [], 'Display', None],
  [ 0x8503, 'CjkUnifiedIdeograph8503', [], 'Display', None],
  [ 0x8504, 'CjkUnifiedIdeograph8504', [], 'Display', None],
  [ 0x8505, 'CjkUnifiedIdeograph8505', [], 'Display', None],
  [ 0x8506, 'CjkUnifiedIdeograph8506', [], 'Display', None],
  [ 0x8507, 'CjkUnifiedIdeograph8507', [], 'Display', None],
  [ 0x8508, 'CjkUnifiedIdeograph8508', [], 'Display', None],
  [ 0x8509, 'CjkUnifiedIdeograph8509', [], 'Display', None],
  [ 0x850a, 'CjkUnifiedIdeograph850a', [], 'Display', None],
  [ 0x850b, 'CjkUnifiedIdeograph850b', [], 'Display', None],
  [ 0x850c, 'CjkUnifiedIdeograph850c', [], 'Display', None],
  [ 0x850d, 'CjkUnifiedIdeograph850d', [], 'Display', None],
  [ 0x850e, 'CjkUnifiedIdeograph850e', [], 'Display', None],
  [ 0x850f, 'CjkUnifiedIdeograph850f', [], 'Display', None],
  [ 0x8510, 'CjkUnifiedIdeograph8510', [], 'Display', None],
  [ 0x8511, 'CjkUnifiedIdeograph8511', [], 'Display', None],
  [ 0x8512, 'CjkUnifiedIdeograph8512', [], 'Display', None],
  [ 0x8513, 'CjkUnifiedIdeograph8513', [], 'Display', None],
  [ 0x8514, 'CjkUnifiedIdeograph8514', [], 'Display', None],
  [ 0x8515, 'CjkUnifiedIdeograph8515', [], 'Display', None],
  [ 0x8516, 'CjkUnifiedIdeograph8516', [], 'Display', None],
  [ 0x8517, 'CjkUnifiedIdeograph8517', [], 'Display', None],
  [ 0x8518, 'CjkUnifiedIdeograph8518', [], 'Display', None],
  [ 0x8519, 'CjkUnifiedIdeograph8519', [], 'Display', None],
  [ 0x851a, 'CjkUnifiedIdeograph851a', [], 'Display', None],
  [ 0x851b, 'CjkUnifiedIdeograph851b', [], 'Display', None],
  [ 0x851c, 'CjkUnifiedIdeograph851c', [], 'Display', None],
  [ 0x851d, 'CjkUnifiedIdeograph851d', [], 'Display', None],
  [ 0x851e, 'CjkUnifiedIdeograph851e', [], 'Display', None],
  [ 0x851f, 'CjkUnifiedIdeograph851f', [], 'Display', None],
  [ 0x8520, 'CjkUnifiedIdeograph8520', [], 'Display', None],
  [ 0x8521, 'CjkUnifiedIdeograph8521', [], 'Display', None],
  [ 0x8522, 'CjkUnifiedIdeograph8522', [], 'Display', None],
  [ 0x8523, 'CjkUnifiedIdeograph8523', [], 'Display', None],
  [ 0x8524, 'CjkUnifiedIdeograph8524', [], 'Display', None],
  [ 0x8525, 'CjkUnifiedIdeograph8525', [], 'Display', None],
  [ 0x8526, 'CjkUnifiedIdeograph8526', [], 'Display', None],
  [ 0x8527, 'CjkUnifiedIdeograph8527', [], 'Display', None],
  [ 0x8528, 'CjkUnifiedIdeograph8528', [], 'Display', None],
  [ 0x8529, 'CjkUnifiedIdeograph8529', [], 'Display', None],
  [ 0x852a, 'CjkUnifiedIdeograph852a', [], 'Display', None],
  [ 0x852b, 'CjkUnifiedIdeograph852b', [], 'Display', None],
  [ 0x852c, 'CjkUnifiedIdeograph852c', [], 'Display', None],
  [ 0x852d, 'CjkUnifiedIdeograph852d', [], 'Display', None],
  [ 0x852e, 'CjkUnifiedIdeograph852e', [], 'Display', None],
  [ 0x852f, 'CjkUnifiedIdeograph852f', [], 'Display', None],
  [ 0x8530, 'CjkUnifiedIdeograph8530', [], 'Display', None],
  [ 0x8531, 'CjkUnifiedIdeograph8531', [], 'Display', None],
  [ 0x8532, 'CjkUnifiedIdeograph8532', [], 'Display', None],
  [ 0x8533, 'CjkUnifiedIdeograph8533', [], 'Display', None],
  [ 0x8534, 'CjkUnifiedIdeograph8534', [], 'Display', None],
  [ 0x8535, 'CjkUnifiedIdeograph8535', [], 'Display', None],
  [ 0x8536, 'CjkUnifiedIdeograph8536', [], 'Display', None],
  [ 0x8537, 'CjkUnifiedIdeograph8537', [], 'Display', None],
  [ 0x8538, 'CjkUnifiedIdeograph8538', [], 'Display', None],
  [ 0x8539, 'CjkUnifiedIdeograph8539', [], 'Display', None],
  [ 0x853a, 'CjkUnifiedIdeograph853a', [], 'Display', None],
  [ 0x853b, 'CjkUnifiedIdeograph853b', [], 'Display', None],
  [ 0x853c, 'CjkUnifiedIdeograph853c', [], 'Display', None],
  [ 0x853d, 'CjkUnifiedIdeograph853d', [], 'Display', None],
  [ 0x853e, 'CjkUnifiedIdeograph853e', [], 'Display', None],
  [ 0x853f, 'CjkUnifiedIdeograph853f', [], 'Display', None],
  [ 0x8540, 'CjkUnifiedIdeograph8540', [], 'Display', None],
  [ 0x8541, 'CjkUnifiedIdeograph8541', [], 'Display', None],
  [ 0x8542, 'CjkUnifiedIdeograph8542', [], 'Display', None],
  [ 0x8543, 'CjkUnifiedIdeograph8543', [], 'Display', None],
  [ 0x8544, 'CjkUnifiedIdeograph8544', [], 'Display', None],
  [ 0x8545, 'CjkUnifiedIdeograph8545', [], 'Display', None],
  [ 0x8546, 'CjkUnifiedIdeograph8546', [], 'Display', None],
  [ 0x8547, 'CjkUnifiedIdeograph8547', [], 'Display', None],
  [ 0x8548, 'CjkUnifiedIdeograph8548', [], 'Display', None],
  [ 0x8549, 'CjkUnifiedIdeograph8549', [], 'Display', None],
  [ 0x854a, 'CjkUnifiedIdeograph854a', [], 'Display', None],
  [ 0x854b, 'CjkUnifiedIdeograph854b', [], 'Display', None],
  [ 0x854c, 'CjkUnifiedIdeograph854c', [], 'Display', None],
  [ 0x854d, 'CjkUnifiedIdeograph854d', [], 'Display', None],
  [ 0x854e, 'CjkUnifiedIdeograph854e', [], 'Display', None],
  [ 0x854f, 'CjkUnifiedIdeograph854f', [], 'Display', None],
  [ 0x8550, 'CjkUnifiedIdeograph8550', [], 'Display', None],
  [ 0x8551, 'CjkUnifiedIdeograph8551', [], 'Display', None],
  [ 0x8552, 'CjkUnifiedIdeograph8552', [], 'Display', None],
  [ 0x8553, 'CjkUnifiedIdeograph8553', [], 'Display', None],
  [ 0x8554, 'CjkUnifiedIdeograph8554', [], 'Display', None],
  [ 0x8555, 'CjkUnifiedIdeograph8555', [], 'Display', None],
  [ 0x8556, 'CjkUnifiedIdeograph8556', [], 'Display', None],
  [ 0x8557, 'CjkUnifiedIdeograph8557', [], 'Display', None],
  [ 0x8558, 'CjkUnifiedIdeograph8558', [], 'Display', None],
  [ 0x8559, 'CjkUnifiedIdeograph8559', [], 'Display', None],
  [ 0x855a, 'CjkUnifiedIdeograph855a', [], 'Display', None],
  [ 0x855b, 'CjkUnifiedIdeograph855b', [], 'Display', None],
  [ 0x855c, 'CjkUnifiedIdeograph855c', [], 'Display', None],
  [ 0x855d, 'CjkUnifiedIdeograph855d', [], 'Display', None],
  [ 0x855e, 'CjkUnifiedIdeograph855e', [], 'Display', None],
  [ 0x855f, 'CjkUnifiedIdeograph855f', [], 'Display', None],
  [ 0x8560, 'CjkUnifiedIdeograph8560', [], 'Display', None],
  [ 0x8561, 'CjkUnifiedIdeograph8561', [], 'Display', None],
  [ 0x8562, 'CjkUnifiedIdeograph8562', [], 'Display', None],
  [ 0x8563, 'CjkUnifiedIdeograph8563', [], 'Display', None],
  [ 0x8564, 'CjkUnifiedIdeograph8564', [], 'Display', None],
  [ 0x8565, 'CjkUnifiedIdeograph8565', [], 'Display', None],
  [ 0x8566, 'CjkUnifiedIdeograph8566', [], 'Display', None],
  [ 0x8567, 'CjkUnifiedIdeograph8567', [], 'Display', None],
  [ 0x8568, 'CjkUnifiedIdeograph8568', [], 'Display', None],
  [ 0x8569, 'CjkUnifiedIdeograph8569', [], 'Display', None],
  [ 0x856a, 'CjkUnifiedIdeograph856a', [], 'Display', None],
  [ 0x856b, 'CjkUnifiedIdeograph856b', [], 'Display', None],
  [ 0x856c, 'CjkUnifiedIdeograph856c', [], 'Display', None],
  [ 0x856d, 'CjkUnifiedIdeograph856d', [], 'Display', None],
  [ 0x856e, 'CjkUnifiedIdeograph856e', [], 'Display', None],
  [ 0x856f, 'CjkUnifiedIdeograph856f', [], 'Display', None],
  [ 0x8570, 'CjkUnifiedIdeograph8570', [], 'Display', None],
  [ 0x8571, 'CjkUnifiedIdeograph8571', [], 'Display', None],
  [ 0x8572, 'CjkUnifiedIdeograph8572', [], 'Display', None],
  [ 0x8573, 'CjkUnifiedIdeograph8573', [], 'Display', None],
  [ 0x8574, 'CjkUnifiedIdeograph8574', [], 'Display', None],
  [ 0x8575, 'CjkUnifiedIdeograph8575', [], 'Display', None],
  [ 0x8576, 'CjkUnifiedIdeograph8576', [], 'Display', None],
  [ 0x8577, 'CjkUnifiedIdeograph8577', [], 'Display', None],
  [ 0x8578, 'CjkUnifiedIdeograph8578', [], 'Display', None],
  [ 0x8579, 'CjkUnifiedIdeograph8579', [], 'Display', None],
  [ 0x857a, 'CjkUnifiedIdeograph857a', [], 'Display', None],
  [ 0x857b, 'CjkUnifiedIdeograph857b', [], 'Display', None],
  [ 0x857c, 'CjkUnifiedIdeograph857c', [], 'Display', None],
  [ 0x857d, 'CjkUnifiedIdeograph857d', [], 'Display', None],
  [ 0x857e, 'CjkUnifiedIdeograph857e', [], 'Display', None],
  [ 0x857f, 'CjkUnifiedIdeograph857f', [], 'Display', None],
  [ 0x8580, 'CjkUnifiedIdeograph8580', [], 'Display', None],
  [ 0x8581, 'CjkUnifiedIdeograph8581', [], 'Display', None],
  [ 0x8582, 'CjkUnifiedIdeograph8582', [], 'Display', None],
  [ 0x8583, 'CjkUnifiedIdeograph8583', [], 'Display', None],
  [ 0x8584, 'CjkUnifiedIdeograph8584', [], 'Display', None],
  [ 0x8585, 'CjkUnifiedIdeograph8585', [], 'Display', None],
  [ 0x8586, 'CjkUnifiedIdeograph8586', [], 'Display', None],
  [ 0x8587, 'CjkUnifiedIdeograph8587', [], 'Display', None],
  [ 0x8588, 'CjkUnifiedIdeograph8588', [], 'Display', None],
  [ 0x8589, 'CjkUnifiedIdeograph8589', [], 'Display', None],
  [ 0x858a, 'CjkUnifiedIdeograph858a', [], 'Display', None],
  [ 0x858b, 'CjkUnifiedIdeograph858b', [], 'Display', None],
  [ 0x858c, 'CjkUnifiedIdeograph858c', [], 'Display', None],
  [ 0x858d, 'CjkUnifiedIdeograph858d', [], 'Display', None],
  [ 0x858e, 'CjkUnifiedIdeograph858e', [], 'Display', None],
  [ 0x858f, 'CjkUnifiedIdeograph858f', [], 'Display', None],
  [ 0x8590, 'CjkUnifiedIdeograph8590', [], 'Display', None],
  [ 0x8591, 'CjkUnifiedIdeograph8591', [], 'Display', None],
  [ 0x8592, 'CjkUnifiedIdeograph8592', [], 'Display', None],
  [ 0x8593, 'CjkUnifiedIdeograph8593', [], 'Display', None],
  [ 0x8594, 'CjkUnifiedIdeograph8594', [], 'Display', None],
  [ 0x8595, 'CjkUnifiedIdeograph8595', [], 'Display', None],
  [ 0x8596, 'CjkUnifiedIdeograph8596', [], 'Display', None],
  [ 0x8597, 'CjkUnifiedIdeograph8597', [], 'Display', None],
  [ 0x8598, 'CjkUnifiedIdeograph8598', [], 'Display', None],
  [ 0x8599, 'CjkUnifiedIdeograph8599', [], 'Display', None],
  [ 0x859a, 'CjkUnifiedIdeograph859a', [], 'Display', None],
  [ 0x859b, 'CjkUnifiedIdeograph859b', [], 'Display', None],
  [ 0x859c, 'CjkUnifiedIdeograph859c', [], 'Display', None],
  [ 0x859d, 'CjkUnifiedIdeograph859d', [], 'Display', None],
  [ 0x859e, 'CjkUnifiedIdeograph859e', [], 'Display', None],
  [ 0x859f, 'CjkUnifiedIdeograph859f', [], 'Display', None],
  [ 0x85a0, 'CjkUnifiedIdeograph85a0', [], 'Display', None],
  [ 0x85a1, 'CjkUnifiedIdeograph85a1', [], 'Display', None],
  [ 0x85a2, 'CjkUnifiedIdeograph85a2', [], 'Display', None],
  [ 0x85a3, 'CjkUnifiedIdeograph85a3', [], 'Display', None],
  [ 0x85a4, 'CjkUnifiedIdeograph85a4', [], 'Display', None],
  [ 0x85a5, 'CjkUnifiedIdeograph85a5', [], 'Display', None],
  [ 0x85a6, 'CjkUnifiedIdeograph85a6', [], 'Display', None],
  [ 0x85a7, 'CjkUnifiedIdeograph85a7', [], 'Display', None],
  [ 0x85a8, 'CjkUnifiedIdeograph85a8', [], 'Display', None],
  [ 0x85a9, 'CjkUnifiedIdeograph85a9', [], 'Display', None],
  [ 0x85aa, 'CjkUnifiedIdeograph85aa', [], 'Display', None],
  [ 0x85ab, 'CjkUnifiedIdeograph85ab', [], 'Display', None],
  [ 0x85ac, 'CjkUnifiedIdeograph85ac', [], 'Display', None],
  [ 0x85ad, 'CjkUnifiedIdeograph85ad', [], 'Display', None],
  [ 0x85ae, 'CjkUnifiedIdeograph85ae', [], 'Display', None],
  [ 0x85af, 'CjkUnifiedIdeograph85af', [], 'Display', None],
  [ 0x85b0, 'CjkUnifiedIdeograph85b0', [], 'Display', None],
  [ 0x85b1, 'CjkUnifiedIdeograph85b1', [], 'Display', None],
  [ 0x85b2, 'CjkUnifiedIdeograph85b2', [], 'Display', None],
  [ 0x85b3, 'CjkUnifiedIdeograph85b3', [], 'Display', None],
  [ 0x85b4, 'CjkUnifiedIdeograph85b4', [], 'Display', None],
  [ 0x85b5, 'CjkUnifiedIdeograph85b5', [], 'Display', None],
  [ 0x85b6, 'CjkUnifiedIdeograph85b6', [], 'Display', None],
  [ 0x85b7, 'CjkUnifiedIdeograph85b7', [], 'Display', None],
  [ 0x85b8, 'CjkUnifiedIdeograph85b8', [], 'Display', None],
  [ 0x85b9, 'CjkUnifiedIdeograph85b9', [], 'Display', None],
  [ 0x85ba, 'CjkUnifiedIdeograph85ba', [], 'Display', None],
  [ 0x85bb, 'CjkUnifiedIdeograph85bb', [], 'Display', None],
  [ 0x85bc, 'CjkUnifiedIdeograph85bc', [], 'Display', None],
  [ 0x85bd, 'CjkUnifiedIdeograph85bd', [], 'Display', None],
  [ 0x85be, 'CjkUnifiedIdeograph85be', [], 'Display', None],
  [ 0x85bf, 'CjkUnifiedIdeograph85bf', [], 'Display', None],
  [ 0x85c0, 'CjkUnifiedIdeograph85c0', [], 'Display', None],
  [ 0x85c1, 'CjkUnifiedIdeograph85c1', [], 'Display', None],
  [ 0x85c2, 'CjkUnifiedIdeograph85c2', [], 'Display', None],
  [ 0x85c3, 'CjkUnifiedIdeograph85c3', [], 'Display', None],
  [ 0x85c4, 'CjkUnifiedIdeograph85c4', [], 'Display', None],
  [ 0x85c5, 'CjkUnifiedIdeograph85c5', [], 'Display', None],
  [ 0x85c6, 'CjkUnifiedIdeograph85c6', [], 'Display', None],
  [ 0x85c7, 'CjkUnifiedIdeograph85c7', [], 'Display', None],
  [ 0x85c8, 'CjkUnifiedIdeograph85c8', [], 'Display', None],
  [ 0x85c9, 'CjkUnifiedIdeograph85c9', [], 'Display', None],
  [ 0x85ca, 'CjkUnifiedIdeograph85ca', [], 'Display', None],
  [ 0x85cb, 'CjkUnifiedIdeograph85cb', [], 'Display', None],
  [ 0x85cc, 'CjkUnifiedIdeograph85cc', [], 'Display', None],
  [ 0x85cd, 'CjkUnifiedIdeograph85cd', [], 'Display', None],
  [ 0x85ce, 'CjkUnifiedIdeograph85ce', [], 'Display', None],
  [ 0x85cf, 'CjkUnifiedIdeograph85cf', [], 'Display', None],
  [ 0x85d0, 'CjkUnifiedIdeograph85d0', [], 'Display', None],
  [ 0x85d1, 'CjkUnifiedIdeograph85d1', [], 'Display', None],
  [ 0x85d2, 'CjkUnifiedIdeograph85d2', [], 'Display', None],
  [ 0x85d3, 'CjkUnifiedIdeograph85d3', [], 'Display', None],
  [ 0x85d4, 'CjkUnifiedIdeograph85d4', [], 'Display', None],
  [ 0x85d5, 'CjkUnifiedIdeograph85d5', [], 'Display', None],
  [ 0x85d6, 'CjkUnifiedIdeograph85d6', [], 'Display', None],
  [ 0x85d7, 'CjkUnifiedIdeograph85d7', [], 'Display', None],
  [ 0x85d8, 'CjkUnifiedIdeograph85d8', [], 'Display', None],
  [ 0x85d9, 'CjkUnifiedIdeograph85d9', [], 'Display', None],
  [ 0x85da, 'CjkUnifiedIdeograph85da', [], 'Display', None],
  [ 0x85db, 'CjkUnifiedIdeograph85db', [], 'Display', None],
  [ 0x85dc, 'CjkUnifiedIdeograph85dc', [], 'Display', None],
  [ 0x85dd, 'CjkUnifiedIdeograph85dd', [], 'Display', None],
  [ 0x85de, 'CjkUnifiedIdeograph85de', [], 'Display', None],
  [ 0x85df, 'CjkUnifiedIdeograph85df', [], 'Display', None],
  [ 0x85e0, 'CjkUnifiedIdeograph85e0', [], 'Display', None],
  [ 0x85e1, 'CjkUnifiedIdeograph85e1', [], 'Display', None],
  [ 0x85e2, 'CjkUnifiedIdeograph85e2', [], 'Display', None],
  [ 0x85e3, 'CjkUnifiedIdeograph85e3', [], 'Display', None],
  [ 0x85e4, 'CjkUnifiedIdeograph85e4', [], 'Display', None],
  [ 0x85e5, 'CjkUnifiedIdeograph85e5', [], 'Display', None],
  [ 0x85e6, 'CjkUnifiedIdeograph85e6', [], 'Display', None],
  [ 0x85e7, 'CjkUnifiedIdeograph85e7', [], 'Display', None],
  [ 0x85e8, 'CjkUnifiedIdeograph85e8', [], 'Display', None],
  [ 0x85e9, 'CjkUnifiedIdeograph85e9', [], 'Display', None],
  [ 0x85ea, 'CjkUnifiedIdeograph85ea', [], 'Display', None],
  [ 0x85eb, 'CjkUnifiedIdeograph85eb', [], 'Display', None],
  [ 0x85ec, 'CjkUnifiedIdeograph85ec', [], 'Display', None],
  [ 0x85ed, 'CjkUnifiedIdeograph85ed', [], 'Display', None],
  [ 0x85ee, 'CjkUnifiedIdeograph85ee', [], 'Display', None],
  [ 0x85ef, 'CjkUnifiedIdeograph85ef', [], 'Display', None],
  [ 0x85f0, 'CjkUnifiedIdeograph85f0', [], 'Display', None],
  [ 0x85f1, 'CjkUnifiedIdeograph85f1', [], 'Display', None],
  [ 0x85f2, 'CjkUnifiedIdeograph85f2', [], 'Display', None],
  [ 0x85f3, 'CjkUnifiedIdeograph85f3', [], 'Display', None],
  [ 0x85f4, 'CjkUnifiedIdeograph85f4', [], 'Display', None],
  [ 0x85f5, 'CjkUnifiedIdeograph85f5', [], 'Display', None],
  [ 0x85f6, 'CjkUnifiedIdeograph85f6', [], 'Display', None],
  [ 0x85f7, 'CjkUnifiedIdeograph85f7', [], 'Display', None],
  [ 0x85f8, 'CjkUnifiedIdeograph85f8', [], 'Display', None],
  [ 0x85f9, 'CjkUnifiedIdeograph85f9', [], 'Display', None],
  [ 0x85fa, 'CjkUnifiedIdeograph85fa', [], 'Display', None],
  [ 0x85fb, 'CjkUnifiedIdeograph85fb', [], 'Display', None],
  [ 0x85fc, 'CjkUnifiedIdeograph85fc', [], 'Display', None],
  [ 0x85fd, 'CjkUnifiedIdeograph85fd', [], 'Display', None],
  [ 0x85fe, 'CjkUnifiedIdeograph85fe', [], 'Display', None],
  [ 0x85ff, 'CjkUnifiedIdeograph85ff', [], 'Display', None],
  [ 0x8600, 'CjkUnifiedIdeograph8600', [], 'Display', None],
  [ 0x8601, 'CjkUnifiedIdeograph8601', [], 'Display', None],
  [ 0x8602, 'CjkUnifiedIdeograph8602', [], 'Display', None],
  [ 0x8603, 'CjkUnifiedIdeograph8603', [], 'Display', None],
  [ 0x8604, 'CjkUnifiedIdeograph8604', [], 'Display', None],
  [ 0x8605, 'CjkUnifiedIdeograph8605', [], 'Display', None],
  [ 0x8606, 'CjkUnifiedIdeograph8606', [], 'Display', None],
  [ 0x8607, 'CjkUnifiedIdeograph8607', [], 'Display', None],
  [ 0x8608, 'CjkUnifiedIdeograph8608', [], 'Display', None],
  [ 0x8609, 'CjkUnifiedIdeograph8609', [], 'Display', None],
  [ 0x860a, 'CjkUnifiedIdeograph860a', [], 'Display', None],
  [ 0x860b, 'CjkUnifiedIdeograph860b', [], 'Display', None],
  [ 0x860c, 'CjkUnifiedIdeograph860c', [], 'Display', None],
  [ 0x860d, 'CjkUnifiedIdeograph860d', [], 'Display', None],
  [ 0x860e, 'CjkUnifiedIdeograph860e', [], 'Display', None],
  [ 0x860f, 'CjkUnifiedIdeograph860f', [], 'Display', None],
  [ 0x8610, 'CjkUnifiedIdeograph8610', [], 'Display', None],
  [ 0x8611, 'CjkUnifiedIdeograph8611', [], 'Display', None],
  [ 0x8612, 'CjkUnifiedIdeograph8612', [], 'Display', None],
  [ 0x8613, 'CjkUnifiedIdeograph8613', [], 'Display', None],
  [ 0x8614, 'CjkUnifiedIdeograph8614', [], 'Display', None],
  [ 0x8615, 'CjkUnifiedIdeograph8615', [], 'Display', None],
  [ 0x8616, 'CjkUnifiedIdeograph8616', [], 'Display', None],
  [ 0x8617, 'CjkUnifiedIdeograph8617', [], 'Display', None],
  [ 0x8618, 'CjkUnifiedIdeograph8618', [], 'Display', None],
  [ 0x8619, 'CjkUnifiedIdeograph8619', [], 'Display', None],
  [ 0x861a, 'CjkUnifiedIdeograph861a', [], 'Display', None],
  [ 0x861b, 'CjkUnifiedIdeograph861b', [], 'Display', None],
  [ 0x861c, 'CjkUnifiedIdeograph861c', [], 'Display', None],
  [ 0x861d, 'CjkUnifiedIdeograph861d', [], 'Display', None],
  [ 0x861e, 'CjkUnifiedIdeograph861e', [], 'Display', None],
  [ 0x861f, 'CjkUnifiedIdeograph861f', [], 'Display', None],
  [ 0x8620, 'CjkUnifiedIdeograph8620', [], 'Display', None],
  [ 0x8621, 'CjkUnifiedIdeograph8621', [], 'Display', None],
  [ 0x8622, 'CjkUnifiedIdeograph8622', [], 'Display', None],
  [ 0x8623, 'CjkUnifiedIdeograph8623', [], 'Display', None],
  [ 0x8624, 'CjkUnifiedIdeograph8624', [], 'Display', None],
  [ 0x8625, 'CjkUnifiedIdeograph8625', [], 'Display', None],
  [ 0x8626, 'CjkUnifiedIdeograph8626', [], 'Display', None],
  [ 0x8627, 'CjkUnifiedIdeograph8627', [], 'Display', None],
  [ 0x8628, 'CjkUnifiedIdeograph8628', [], 'Display', None],
  [ 0x8629, 'CjkUnifiedIdeograph8629', [], 'Display', None],
  [ 0x862a, 'CjkUnifiedIdeograph862a', [], 'Display', None],
  [ 0x862b, 'CjkUnifiedIdeograph862b', [], 'Display', None],
  [ 0x862c, 'CjkUnifiedIdeograph862c', [], 'Display', None],
  [ 0x862d, 'CjkUnifiedIdeograph862d', [], 'Display', None],
  [ 0x862e, 'CjkUnifiedIdeograph862e', [], 'Display', None],
  [ 0x862f, 'CjkUnifiedIdeograph862f', [], 'Display', None],
  [ 0x8630, 'CjkUnifiedIdeograph8630', [], 'Display', None],
  [ 0x8631, 'CjkUnifiedIdeograph8631', [], 'Display', None],
  [ 0x8632, 'CjkUnifiedIdeograph8632', [], 'Display', None],
  [ 0x8633, 'CjkUnifiedIdeograph8633', [], 'Display', None],
  [ 0x8634, 'CjkUnifiedIdeograph8634', [], 'Display', None],
  [ 0x8635, 'CjkUnifiedIdeograph8635', [], 'Display', None],
  [ 0x8636, 'CjkUnifiedIdeograph8636', [], 'Display', None],
  [ 0x8637, 'CjkUnifiedIdeograph8637', [], 'Display', None],
  [ 0x8638, 'CjkUnifiedIdeograph8638', [], 'Display', None],
  [ 0x8639, 'CjkUnifiedIdeograph8639', [], 'Display', None],
  [ 0x863a, 'CjkUnifiedIdeograph863a', [], 'Display', None],
  [ 0x863b, 'CjkUnifiedIdeograph863b', [], 'Display', None],
  [ 0x863c, 'CjkUnifiedIdeograph863c', [], 'Display', None],
  [ 0x863d, 'CjkUnifiedIdeograph863d', [], 'Display', None],
  [ 0x863e, 'CjkUnifiedIdeograph863e', [], 'Display', None],
  [ 0x863f, 'CjkUnifiedIdeograph863f', [], 'Display', None],
  [ 0x8640, 'CjkUnifiedIdeograph8640', [], 'Display', None],
  [ 0x8641, 'CjkUnifiedIdeograph8641', [], 'Display', None],
  [ 0x8642, 'CjkUnifiedIdeograph8642', [], 'Display', None],
  [ 0x8643, 'CjkUnifiedIdeograph8643', [], 'Display', None],
  [ 0x8644, 'CjkUnifiedIdeograph8644', [], 'Display', None],
  [ 0x8645, 'CjkUnifiedIdeograph8645', [], 'Display', None],
  [ 0x8646, 'CjkUnifiedIdeograph8646', [], 'Display', None],
  [ 0x8647, 'CjkUnifiedIdeograph8647', [], 'Display', None],
  [ 0x8648, 'CjkUnifiedIdeograph8648', [], 'Display', None],
  [ 0x8649, 'CjkUnifiedIdeograph8649', [], 'Display', None],
  [ 0x864a, 'CjkUnifiedIdeograph864a', [], 'Display', None],
  [ 0x864b, 'CjkUnifiedIdeograph864b', [], 'Display', None],
  [ 0x864c, 'CjkUnifiedIdeograph864c', [], 'Display', None],
  [ 0x864d, 'CjkUnifiedIdeograph864d', [], 'Display', None],
  [ 0x864e, 'CjkUnifiedIdeograph864e', [], 'Display', None],
  [ 0x864f, 'CjkUnifiedIdeograph864f', [], 'Display', None],
  [ 0x8650, 'CjkUnifiedIdeograph8650', [], 'Display', None],
  [ 0x8651, 'CjkUnifiedIdeograph8651', [], 'Display', None],
  [ 0x8652, 'CjkUnifiedIdeograph8652', [], 'Display', None],
  [ 0x8653, 'CjkUnifiedIdeograph8653', [], 'Display', None],
  [ 0x8654, 'CjkUnifiedIdeograph8654', [], 'Display', None],
  [ 0x8655, 'CjkUnifiedIdeograph8655', [], 'Display', None],
  [ 0x8656, 'CjkUnifiedIdeograph8656', [], 'Display', None],
  [ 0x8657, 'CjkUnifiedIdeograph8657', [], 'Display', None],
  [ 0x8658, 'CjkUnifiedIdeograph8658', [], 'Display', None],
  [ 0x8659, 'CjkUnifiedIdeograph8659', [], 'Display', None],
  [ 0x865a, 'CjkUnifiedIdeograph865a', [], 'Display', None],
  [ 0x865b, 'CjkUnifiedIdeograph865b', [], 'Display', None],
  [ 0x865c, 'CjkUnifiedIdeograph865c', [], 'Display', None],
  [ 0x865d, 'CjkUnifiedIdeograph865d', [], 'Display', None],
  [ 0x865e, 'CjkUnifiedIdeograph865e', [], 'Display', None],
  [ 0x865f, 'CjkUnifiedIdeograph865f', [], 'Display', None],
  [ 0x8660, 'CjkUnifiedIdeograph8660', [], 'Display', None],
  [ 0x8661, 'CjkUnifiedIdeograph8661', [], 'Display', None],
  [ 0x8662, 'CjkUnifiedIdeograph8662', [], 'Display', None],
  [ 0x8663, 'CjkUnifiedIdeograph8663', [], 'Display', None],
  [ 0x8664, 'CjkUnifiedIdeograph8664', [], 'Display', None],
  [ 0x8665, 'CjkUnifiedIdeograph8665', [], 'Display', None],
  [ 0x8666, 'CjkUnifiedIdeograph8666', [], 'Display', None],
  [ 0x8667, 'CjkUnifiedIdeograph8667', [], 'Display', None],
  [ 0x8668, 'CjkUnifiedIdeograph8668', [], 'Display', None],
  [ 0x8669, 'CjkUnifiedIdeograph8669', [], 'Display', None],
  [ 0x866a, 'CjkUnifiedIdeograph866a', [], 'Display', None],
  [ 0x866b, 'CjkUnifiedIdeograph866b', [], 'Display', None],
  [ 0x866c, 'CjkUnifiedIdeograph866c', [], 'Display', None],
  [ 0x866d, 'CjkUnifiedIdeograph866d', [], 'Display', None],
  [ 0x866e, 'CjkUnifiedIdeograph866e', [], 'Display', None],
  [ 0x866f, 'CjkUnifiedIdeograph866f', [], 'Display', None],
  [ 0x8670, 'CjkUnifiedIdeograph8670', [], 'Display', None],
  [ 0x8671, 'CjkUnifiedIdeograph8671', [], 'Display', None],
  [ 0x8672, 'CjkUnifiedIdeograph8672', [], 'Display', None],
  [ 0x8673, 'CjkUnifiedIdeograph8673', [], 'Display', None],
  [ 0x8674, 'CjkUnifiedIdeograph8674', [], 'Display', None],
  [ 0x8675, 'CjkUnifiedIdeograph8675', [], 'Display', None],
  [ 0x8676, 'CjkUnifiedIdeograph8676', [], 'Display', None],
  [ 0x8677, 'CjkUnifiedIdeograph8677', [], 'Display', None],
  [ 0x8678, 'CjkUnifiedIdeograph8678', [], 'Display', None],
  [ 0x8679, 'CjkUnifiedIdeograph8679', [], 'Display', None],
  [ 0x867a, 'CjkUnifiedIdeograph867a', [], 'Display', None],
  [ 0x867b, 'CjkUnifiedIdeograph867b', [], 'Display', None],
  [ 0x867c, 'CjkUnifiedIdeograph867c', [], 'Display', None],
  [ 0x867d, 'CjkUnifiedIdeograph867d', [], 'Display', None],
  [ 0x867e, 'CjkUnifiedIdeograph867e', [], 'Display', None],
  [ 0x867f, 'CjkUnifiedIdeograph867f', [], 'Display', None],
  [ 0x8680, 'CjkUnifiedIdeograph8680', [], 'Display', None],
  [ 0x8681, 'CjkUnifiedIdeograph8681', [], 'Display', None],
  [ 0x8682, 'CjkUnifiedIdeograph8682', [], 'Display', None],
  [ 0x8683, 'CjkUnifiedIdeograph8683', [], 'Display', None],
  [ 0x8684, 'CjkUnifiedIdeograph8684', [], 'Display', None],
  [ 0x8685, 'CjkUnifiedIdeograph8685', [], 'Display', None],
  [ 0x8686, 'CjkUnifiedIdeograph8686', [], 'Display', None],
  [ 0x8687, 'CjkUnifiedIdeograph8687', [], 'Display', None],
  [ 0x8688, 'CjkUnifiedIdeograph8688', [], 'Display', None],
  [ 0x8689, 'CjkUnifiedIdeograph8689', [], 'Display', None],
  [ 0x868a, 'CjkUnifiedIdeograph868a', [], 'Display', None],
  [ 0x868b, 'CjkUnifiedIdeograph868b', [], 'Display', None],
  [ 0x868c, 'CjkUnifiedIdeograph868c', [], 'Display', None],
  [ 0x868d, 'CjkUnifiedIdeograph868d', [], 'Display', None],
  [ 0x868e, 'CjkUnifiedIdeograph868e', [], 'Display', None],
  [ 0x868f, 'CjkUnifiedIdeograph868f', [], 'Display', None],
  [ 0x8690, 'CjkUnifiedIdeograph8690', [], 'Display', None],
  [ 0x8691, 'CjkUnifiedIdeograph8691', [], 'Display', None],
  [ 0x8692, 'CjkUnifiedIdeograph8692', [], 'Display', None],
  [ 0x8693, 'CjkUnifiedIdeograph8693', [], 'Display', None],
  [ 0x8694, 'CjkUnifiedIdeograph8694', [], 'Display', None],
  [ 0x8695, 'CjkUnifiedIdeograph8695', [], 'Display', None],
  [ 0x8696, 'CjkUnifiedIdeograph8696', [], 'Display', None],
  [ 0x8697, 'CjkUnifiedIdeograph8697', [], 'Display', None],
  [ 0x8698, 'CjkUnifiedIdeograph8698', [], 'Display', None],
  [ 0x8699, 'CjkUnifiedIdeograph8699', [], 'Display', None],
  [ 0x869a, 'CjkUnifiedIdeograph869a', [], 'Display', None],
  [ 0x869b, 'CjkUnifiedIdeograph869b', [], 'Display', None],
  [ 0x869c, 'CjkUnifiedIdeograph869c', [], 'Display', None],
  [ 0x869d, 'CjkUnifiedIdeograph869d', [], 'Display', None],
  [ 0x869e, 'CjkUnifiedIdeograph869e', [], 'Display', None],
  [ 0x869f, 'CjkUnifiedIdeograph869f', [], 'Display', None],
  [ 0x86a0, 'CjkUnifiedIdeograph86a0', [], 'Display', None],
  [ 0x86a1, 'CjkUnifiedIdeograph86a1', [], 'Display', None],
  [ 0x86a2, 'CjkUnifiedIdeograph86a2', [], 'Display', None],
  [ 0x86a3, 'CjkUnifiedIdeograph86a3', [], 'Display', None],
  [ 0x86a4, 'CjkUnifiedIdeograph86a4', [], 'Display', None],
  [ 0x86a5, 'CjkUnifiedIdeograph86a5', [], 'Display', None],
  [ 0x86a6, 'CjkUnifiedIdeograph86a6', [], 'Display', None],
  [ 0x86a7, 'CjkUnifiedIdeograph86a7', [], 'Display', None],
  [ 0x86a8, 'CjkUnifiedIdeograph86a8', [], 'Display', None],
  [ 0x86a9, 'CjkUnifiedIdeograph86a9', [], 'Display', None],
  [ 0x86aa, 'CjkUnifiedIdeograph86aa', [], 'Display', None],
  [ 0x86ab, 'CjkUnifiedIdeograph86ab', [], 'Display', None],
  [ 0x86ac, 'CjkUnifiedIdeograph86ac', [], 'Display', None],
  [ 0x86ad, 'CjkUnifiedIdeograph86ad', [], 'Display', None],
  [ 0x86ae, 'CjkUnifiedIdeograph86ae', [], 'Display', None],
  [ 0x86af, 'CjkUnifiedIdeograph86af', [], 'Display', None],
  [ 0x86b0, 'CjkUnifiedIdeograph86b0', [], 'Display', None],
  [ 0x86b1, 'CjkUnifiedIdeograph86b1', [], 'Display', None],
  [ 0x86b2, 'CjkUnifiedIdeograph86b2', [], 'Display', None],
  [ 0x86b3, 'CjkUnifiedIdeograph86b3', [], 'Display', None],
  [ 0x86b4, 'CjkUnifiedIdeograph86b4', [], 'Display', None],
  [ 0x86b5, 'CjkUnifiedIdeograph86b5', [], 'Display', None],
  [ 0x86b6, 'CjkUnifiedIdeograph86b6', [], 'Display', None],
  [ 0x86b7, 'CjkUnifiedIdeograph86b7', [], 'Display', None],
  [ 0x86b8, 'CjkUnifiedIdeograph86b8', [], 'Display', None],
  [ 0x86b9, 'CjkUnifiedIdeograph86b9', [], 'Display', None],
  [ 0x86ba, 'CjkUnifiedIdeograph86ba', [], 'Display', None],
  [ 0x86bb, 'CjkUnifiedIdeograph86bb', [], 'Display', None],
  [ 0x86bc, 'CjkUnifiedIdeograph86bc', [], 'Display', None],
  [ 0x86bd, 'CjkUnifiedIdeograph86bd', [], 'Display', None],
  [ 0x86be, 'CjkUnifiedIdeograph86be', [], 'Display', None],
  [ 0x86bf, 'CjkUnifiedIdeograph86bf', [], 'Display', None],
  [ 0x86c0, 'CjkUnifiedIdeograph86c0', [], 'Display', None],
  [ 0x86c1, 'CjkUnifiedIdeograph86c1', [], 'Display', None],
  [ 0x86c2, 'CjkUnifiedIdeograph86c2', [], 'Display', None],
  [ 0x86c3, 'CjkUnifiedIdeograph86c3', [], 'Display', None],
  [ 0x86c4, 'CjkUnifiedIdeograph86c4', [], 'Display', None],
  [ 0x86c5, 'CjkUnifiedIdeograph86c5', [], 'Display', None],
  [ 0x86c6, 'CjkUnifiedIdeograph86c6', [], 'Display', None],
  [ 0x86c7, 'CjkUnifiedIdeograph86c7', [], 'Display', None],
  [ 0x86c8, 'CjkUnifiedIdeograph86c8', [], 'Display', None],
  [ 0x86c9, 'CjkUnifiedIdeograph86c9', [], 'Display', None],
  [ 0x86ca, 'CjkUnifiedIdeograph86ca', [], 'Display', None],
  [ 0x86cb, 'CjkUnifiedIdeograph86cb', [], 'Display', None],
  [ 0x86cc, 'CjkUnifiedIdeograph86cc', [], 'Display', None],
  [ 0x86cd, 'CjkUnifiedIdeograph86cd', [], 'Display', None],
  [ 0x86ce, 'CjkUnifiedIdeograph86ce', [], 'Display', None],
  [ 0x86cf, 'CjkUnifiedIdeograph86cf', [], 'Display', None],
  [ 0x86d0, 'CjkUnifiedIdeograph86d0', [], 'Display', None],
  [ 0x86d1, 'CjkUnifiedIdeograph86d1', [], 'Display', None],
  [ 0x86d2, 'CjkUnifiedIdeograph86d2', [], 'Display', None],
  [ 0x86d3, 'CjkUnifiedIdeograph86d3', [], 'Display', None],
  [ 0x86d4, 'CjkUnifiedIdeograph86d4', [], 'Display', None],
  [ 0x86d5, 'CjkUnifiedIdeograph86d5', [], 'Display', None],
  [ 0x86d6, 'CjkUnifiedIdeograph86d6', [], 'Display', None],
  [ 0x86d7, 'CjkUnifiedIdeograph86d7', [], 'Display', None],
  [ 0x86d8, 'CjkUnifiedIdeograph86d8', [], 'Display', None],
  [ 0x86d9, 'CjkUnifiedIdeograph86d9', [], 'Display', None],
  [ 0x86da, 'CjkUnifiedIdeograph86da', [], 'Display', None],
  [ 0x86db, 'CjkUnifiedIdeograph86db', [], 'Display', None],
  [ 0x86dc, 'CjkUnifiedIdeograph86dc', [], 'Display', None],
  [ 0x86dd, 'CjkUnifiedIdeograph86dd', [], 'Display', None],
  [ 0x86de, 'CjkUnifiedIdeograph86de', [], 'Display', None],
  [ 0x86df, 'CjkUnifiedIdeograph86df', [], 'Display', None],
  [ 0x86e0, 'CjkUnifiedIdeograph86e0', [], 'Display', None],
  [ 0x86e1, 'CjkUnifiedIdeograph86e1', [], 'Display', None],
  [ 0x86e2, 'CjkUnifiedIdeograph86e2', [], 'Display', None],
  [ 0x86e3, 'CjkUnifiedIdeograph86e3', [], 'Display', None],
  [ 0x86e4, 'CjkUnifiedIdeograph86e4', [], 'Display', None],
  [ 0x86e5, 'CjkUnifiedIdeograph86e5', [], 'Display', None],
  [ 0x86e6, 'CjkUnifiedIdeograph86e6', [], 'Display', None],
  [ 0x86e7, 'CjkUnifiedIdeograph86e7', [], 'Display', None],
  [ 0x86e8, 'CjkUnifiedIdeograph86e8', [], 'Display', None],
  [ 0x86e9, 'CjkUnifiedIdeograph86e9', [], 'Display', None],
  [ 0x86ea, 'CjkUnifiedIdeograph86ea', [], 'Display', None],
  [ 0x86eb, 'CjkUnifiedIdeograph86eb', [], 'Display', None],
  [ 0x86ec, 'CjkUnifiedIdeograph86ec', [], 'Display', None],
  [ 0x86ed, 'CjkUnifiedIdeograph86ed', [], 'Display', None],
  [ 0x86ee, 'CjkUnifiedIdeograph86ee', [], 'Display', None],
  [ 0x86ef, 'CjkUnifiedIdeograph86ef', [], 'Display', None],
  [ 0x86f0, 'CjkUnifiedIdeograph86f0', [], 'Display', None],
  [ 0x86f1, 'CjkUnifiedIdeograph86f1', [], 'Display', None],
  [ 0x86f2, 'CjkUnifiedIdeograph86f2', [], 'Display', None],
  [ 0x86f3, 'CjkUnifiedIdeograph86f3', [], 'Display', None],
  [ 0x86f4, 'CjkUnifiedIdeograph86f4', [], 'Display', None],
  [ 0x86f5, 'CjkUnifiedIdeograph86f5', [], 'Display', None],
  [ 0x86f6, 'CjkUnifiedIdeograph86f6', [], 'Display', None],
  [ 0x86f7, 'CjkUnifiedIdeograph86f7', [], 'Display', None],
  [ 0x86f8, 'CjkUnifiedIdeograph86f8', [], 'Display', None],
  [ 0x86f9, 'CjkUnifiedIdeograph86f9', [], 'Display', None],
  [ 0x86fa, 'CjkUnifiedIdeograph86fa', [], 'Display', None],
  [ 0x86fb, 'CjkUnifiedIdeograph86fb', [], 'Display', None],
  [ 0x86fc, 'CjkUnifiedIdeograph86fc', [], 'Display', None],
  [ 0x86fd, 'CjkUnifiedIdeograph86fd', [], 'Display', None],
  [ 0x86fe, 'CjkUnifiedIdeograph86fe', [], 'Display', None],
  [ 0x86ff, 'CjkUnifiedIdeograph86ff', [], 'Display', None],
  [ 0x8700, 'CjkUnifiedIdeograph8700', [], 'Display', None],
  [ 0x8701, 'CjkUnifiedIdeograph8701', [], 'Display', None],
  [ 0x8702, 'CjkUnifiedIdeograph8702', [], 'Display', None],
  [ 0x8703, 'CjkUnifiedIdeograph8703', [], 'Display', None],
  [ 0x8704, 'CjkUnifiedIdeograph8704', [], 'Display', None],
  [ 0x8705, 'CjkUnifiedIdeograph8705', [], 'Display', None],
  [ 0x8706, 'CjkUnifiedIdeograph8706', [], 'Display', None],
  [ 0x8707, 'CjkUnifiedIdeograph8707', [], 'Display', None],
  [ 0x8708, 'CjkUnifiedIdeograph8708', [], 'Display', None],
  [ 0x8709, 'CjkUnifiedIdeograph8709', [], 'Display', None],
  [ 0x870a, 'CjkUnifiedIdeograph870a', [], 'Display', None],
  [ 0x870b, 'CjkUnifiedIdeograph870b', [], 'Display', None],
  [ 0x870c, 'CjkUnifiedIdeograph870c', [], 'Display', None],
  [ 0x870d, 'CjkUnifiedIdeograph870d', [], 'Display', None],
  [ 0x870e, 'CjkUnifiedIdeograph870e', [], 'Display', None],
  [ 0x870f, 'CjkUnifiedIdeograph870f', [], 'Display', None],
  [ 0x8710, 'CjkUnifiedIdeograph8710', [], 'Display', None],
  [ 0x8711, 'CjkUnifiedIdeograph8711', [], 'Display', None],
  [ 0x8712, 'CjkUnifiedIdeograph8712', [], 'Display', None],
  [ 0x8713, 'CjkUnifiedIdeograph8713', [], 'Display', None],
  [ 0x8714, 'CjkUnifiedIdeograph8714', [], 'Display', None],
  [ 0x8715, 'CjkUnifiedIdeograph8715', [], 'Display', None],
  [ 0x8716, 'CjkUnifiedIdeograph8716', [], 'Display', None],
  [ 0x8717, 'CjkUnifiedIdeograph8717', [], 'Display', None],
  [ 0x8718, 'CjkUnifiedIdeograph8718', [], 'Display', None],
  [ 0x8719, 'CjkUnifiedIdeograph8719', [], 'Display', None],
  [ 0x871a, 'CjkUnifiedIdeograph871a', [], 'Display', None],
  [ 0x871b, 'CjkUnifiedIdeograph871b', [], 'Display', None],
  [ 0x871c, 'CjkUnifiedIdeograph871c', [], 'Display', None],
  [ 0x871d, 'CjkUnifiedIdeograph871d', [], 'Display', None],
  [ 0x871e, 'CjkUnifiedIdeograph871e', [], 'Display', None],
  [ 0x871f, 'CjkUnifiedIdeograph871f', [], 'Display', None],
  [ 0x8720, 'CjkUnifiedIdeograph8720', [], 'Display', None],
  [ 0x8721, 'CjkUnifiedIdeograph8721', [], 'Display', None],
  [ 0x8722, 'CjkUnifiedIdeograph8722', [], 'Display', None],
  [ 0x8723, 'CjkUnifiedIdeograph8723', [], 'Display', None],
  [ 0x8724, 'CjkUnifiedIdeograph8724', [], 'Display', None],
  [ 0x8725, 'CjkUnifiedIdeograph8725', [], 'Display', None],
  [ 0x8726, 'CjkUnifiedIdeograph8726', [], 'Display', None],
  [ 0x8727, 'CjkUnifiedIdeograph8727', [], 'Display', None],
  [ 0x8728, 'CjkUnifiedIdeograph8728', [], 'Display', None],
  [ 0x8729, 'CjkUnifiedIdeograph8729', [], 'Display', None],
  [ 0x872a, 'CjkUnifiedIdeograph872a', [], 'Display', None],
  [ 0x872b, 'CjkUnifiedIdeograph872b', [], 'Display', None],
  [ 0x872c, 'CjkUnifiedIdeograph872c', [], 'Display', None],
  [ 0x872d, 'CjkUnifiedIdeograph872d', [], 'Display', None],
  [ 0x872e, 'CjkUnifiedIdeograph872e', [], 'Display', None],
  [ 0x872f, 'CjkUnifiedIdeograph872f', [], 'Display', None],
  [ 0x8730, 'CjkUnifiedIdeograph8730', [], 'Display', None],
  [ 0x8731, 'CjkUnifiedIdeograph8731', [], 'Display', None],
  [ 0x8732, 'CjkUnifiedIdeograph8732', [], 'Display', None],
  [ 0x8733, 'CjkUnifiedIdeograph8733', [], 'Display', None],
  [ 0x8734, 'CjkUnifiedIdeograph8734', [], 'Display', None],
  [ 0x8735, 'CjkUnifiedIdeograph8735', [], 'Display', None],
  [ 0x8736, 'CjkUnifiedIdeograph8736', [], 'Display', None],
  [ 0x8737, 'CjkUnifiedIdeograph8737', [], 'Display', None],
  [ 0x8738, 'CjkUnifiedIdeograph8738', [], 'Display', None],
  [ 0x8739, 'CjkUnifiedIdeograph8739', [], 'Display', None],
  [ 0x873a, 'CjkUnifiedIdeograph873a', [], 'Display', None],
  [ 0x873b, 'CjkUnifiedIdeograph873b', [], 'Display', None],
  [ 0x873c, 'CjkUnifiedIdeograph873c', [], 'Display', None],
  [ 0x873d, 'CjkUnifiedIdeograph873d', [], 'Display', None],
  [ 0x873e, 'CjkUnifiedIdeograph873e', [], 'Display', None],
  [ 0x873f, 'CjkUnifiedIdeograph873f', [], 'Display', None],
  [ 0x8740, 'CjkUnifiedIdeograph8740', [], 'Display', None],
  [ 0x8741, 'CjkUnifiedIdeograph8741', [], 'Display', None],
  [ 0x8742, 'CjkUnifiedIdeograph8742', [], 'Display', None],
  [ 0x8743, 'CjkUnifiedIdeograph8743', [], 'Display', None],
  [ 0x8744, 'CjkUnifiedIdeograph8744', [], 'Display', None],
  [ 0x8745, 'CjkUnifiedIdeograph8745', [], 'Display', None],
  [ 0x8746, 'CjkUnifiedIdeograph8746', [], 'Display', None],
  [ 0x8747, 'CjkUnifiedIdeograph8747', [], 'Display', None],
  [ 0x8748, 'CjkUnifiedIdeograph8748', [], 'Display', None],
  [ 0x8749, 'CjkUnifiedIdeograph8749', [], 'Display', None],
  [ 0x874a, 'CjkUnifiedIdeograph874a', [], 'Display', None],
  [ 0x874b, 'CjkUnifiedIdeograph874b', [], 'Display', None],
  [ 0x874c, 'CjkUnifiedIdeograph874c', [], 'Display', None],
  [ 0x874d, 'CjkUnifiedIdeograph874d', [], 'Display', None],
  [ 0x874e, 'CjkUnifiedIdeograph874e', [], 'Display', None],
  [ 0x874f, 'CjkUnifiedIdeograph874f', [], 'Display', None],
  [ 0x8750, 'CjkUnifiedIdeograph8750', [], 'Display', None],
  [ 0x8751, 'CjkUnifiedIdeograph8751', [], 'Display', None],
  [ 0x8752, 'CjkUnifiedIdeograph8752', [], 'Display', None],
  [ 0x8753, 'CjkUnifiedIdeograph8753', [], 'Display', None],
  [ 0x8754, 'CjkUnifiedIdeograph8754', [], 'Display', None],
  [ 0x8755, 'CjkUnifiedIdeograph8755', [], 'Display', None],
  [ 0x8756, 'CjkUnifiedIdeograph8756', [], 'Display', None],
  [ 0x8757, 'CjkUnifiedIdeograph8757', [], 'Display', None],
  [ 0x8758, 'CjkUnifiedIdeograph8758', [], 'Display', None],
  [ 0x8759, 'CjkUnifiedIdeograph8759', [], 'Display', None],
  [ 0x875a, 'CjkUnifiedIdeograph875a', [], 'Display', None],
  [ 0x875b, 'CjkUnifiedIdeograph875b', [], 'Display', None],
  [ 0x875c, 'CjkUnifiedIdeograph875c', [], 'Display', None],
  [ 0x875d, 'CjkUnifiedIdeograph875d', [], 'Display', None],
  [ 0x875e, 'CjkUnifiedIdeograph875e', [], 'Display', None],
  [ 0x875f, 'CjkUnifiedIdeograph875f', [], 'Display', None],
  [ 0x8760, 'CjkUnifiedIdeograph8760', [], 'Display', None],
  [ 0x8761, 'CjkUnifiedIdeograph8761', [], 'Display', None],
  [ 0x8762, 'CjkUnifiedIdeograph8762', [], 'Display', None],
  [ 0x8763, 'CjkUnifiedIdeograph8763', [], 'Display', None],
  [ 0x8764, 'CjkUnifiedIdeograph8764', [], 'Display', None],
  [ 0x8765, 'CjkUnifiedIdeograph8765', [], 'Display', None],
  [ 0x8766, 'CjkUnifiedIdeograph8766', [], 'Display', None],
  [ 0x8767, 'CjkUnifiedIdeograph8767', [], 'Display', None],
  [ 0x8768, 'CjkUnifiedIdeograph8768', [], 'Display', None],
  [ 0x8769, 'CjkUnifiedIdeograph8769', [], 'Display', None],
  [ 0x876a, 'CjkUnifiedIdeograph876a', [], 'Display', None],
  [ 0x876b, 'CjkUnifiedIdeograph876b', [], 'Display', None],
  [ 0x876c, 'CjkUnifiedIdeograph876c', [], 'Display', None],
  [ 0x876d, 'CjkUnifiedIdeograph876d', [], 'Display', None],
  [ 0x876e, 'CjkUnifiedIdeograph876e', [], 'Display', None],
  [ 0x876f, 'CjkUnifiedIdeograph876f', [], 'Display', None],
  [ 0x8770, 'CjkUnifiedIdeograph8770', [], 'Display', None],
  [ 0x8771, 'CjkUnifiedIdeograph8771', [], 'Display', None],
  [ 0x8772, 'CjkUnifiedIdeograph8772', [], 'Display', None],
  [ 0x8773, 'CjkUnifiedIdeograph8773', [], 'Display', None],
  [ 0x8774, 'CjkUnifiedIdeograph8774', [], 'Display', None],
  [ 0x8775, 'CjkUnifiedIdeograph8775', [], 'Display', None],
  [ 0x8776, 'CjkUnifiedIdeograph8776', [], 'Display', None],
  [ 0x8777, 'CjkUnifiedIdeograph8777', [], 'Display', None],
  [ 0x8778, 'CjkUnifiedIdeograph8778', [], 'Display', None],
  [ 0x8779, 'CjkUnifiedIdeograph8779', [], 'Display', None],
  [ 0x877a, 'CjkUnifiedIdeograph877a', [], 'Display', None],
  [ 0x877b, 'CjkUnifiedIdeograph877b', [], 'Display', None],
  [ 0x877c, 'CjkUnifiedIdeograph877c', [], 'Display', None],
  [ 0x877d, 'CjkUnifiedIdeograph877d', [], 'Display', None],
  [ 0x877e, 'CjkUnifiedIdeograph877e', [], 'Display', None],
  [ 0x877f, 'CjkUnifiedIdeograph877f', [], 'Display', None],
  [ 0x8780, 'CjkUnifiedIdeograph8780', [], 'Display', None],
  [ 0x8781, 'CjkUnifiedIdeograph8781', [], 'Display', None],
  [ 0x8782, 'CjkUnifiedIdeograph8782', [], 'Display', None],
  [ 0x8783, 'CjkUnifiedIdeograph8783', [], 'Display', None],
  [ 0x8784, 'CjkUnifiedIdeograph8784', [], 'Display', None],
  [ 0x8785, 'CjkUnifiedIdeograph8785', [], 'Display', None],
  [ 0x8786, 'CjkUnifiedIdeograph8786', [], 'Display', None],
  [ 0x8787, 'CjkUnifiedIdeograph8787', [], 'Display', None],
  [ 0x8788, 'CjkUnifiedIdeograph8788', [], 'Display', None],
  [ 0x8789, 'CjkUnifiedIdeograph8789', [], 'Display', None],
  [ 0x878a, 'CjkUnifiedIdeograph878a', [], 'Display', None],
  [ 0x878b, 'CjkUnifiedIdeograph878b', [], 'Display', None],
  [ 0x878c, 'CjkUnifiedIdeograph878c', [], 'Display', None],
  [ 0x878d, 'CjkUnifiedIdeograph878d', [], 'Display', None],
  [ 0x878e, 'CjkUnifiedIdeograph878e', [], 'Display', None],
  [ 0x878f, 'CjkUnifiedIdeograph878f', [], 'Display', None],
  [ 0x8790, 'CjkUnifiedIdeograph8790', [], 'Display', None],
  [ 0x8791, 'CjkUnifiedIdeograph8791', [], 'Display', None],
  [ 0x8792, 'CjkUnifiedIdeograph8792', [], 'Display', None],
  [ 0x8793, 'CjkUnifiedIdeograph8793', [], 'Display', None],
  [ 0x8794, 'CjkUnifiedIdeograph8794', [], 'Display', None],
  [ 0x8795, 'CjkUnifiedIdeograph8795', [], 'Display', None],
  [ 0x8796, 'CjkUnifiedIdeograph8796', [], 'Display', None],
  [ 0x8797, 'CjkUnifiedIdeograph8797', [], 'Display', None],
  [ 0x8798, 'CjkUnifiedIdeograph8798', [], 'Display', None],
  [ 0x8799, 'CjkUnifiedIdeograph8799', [], 'Display', None],
  [ 0x879a, 'CjkUnifiedIdeograph879a', [], 'Display', None],
  [ 0x879b, 'CjkUnifiedIdeograph879b', [], 'Display', None],
  [ 0x879c, 'CjkUnifiedIdeograph879c', [], 'Display', None],
  [ 0x879d, 'CjkUnifiedIdeograph879d', [], 'Display', None],
  [ 0x879e, 'CjkUnifiedIdeograph879e', [], 'Display', None],
  [ 0x879f, 'CjkUnifiedIdeograph879f', [], 'Display', None],
  [ 0x87a0, 'CjkUnifiedIdeograph87a0', [], 'Display', None],
  [ 0x87a1, 'CjkUnifiedIdeograph87a1', [], 'Display', None],
  [ 0x87a2, 'CjkUnifiedIdeograph87a2', [], 'Display', None],
  [ 0x87a3, 'CjkUnifiedIdeograph87a3', [], 'Display', None],
  [ 0x87a4, 'CjkUnifiedIdeograph87a4', [], 'Display', None],
  [ 0x87a5, 'CjkUnifiedIdeograph87a5', [], 'Display', None],
  [ 0x87a6, 'CjkUnifiedIdeograph87a6', [], 'Display', None],
  [ 0x87a7, 'CjkUnifiedIdeograph87a7', [], 'Display', None],
  [ 0x87a8, 'CjkUnifiedIdeograph87a8', [], 'Display', None],
  [ 0x87a9, 'CjkUnifiedIdeograph87a9', [], 'Display', None],
  [ 0x87aa, 'CjkUnifiedIdeograph87aa', [], 'Display', None],
  [ 0x87ab, 'CjkUnifiedIdeograph87ab', [], 'Display', None],
  [ 0x87ac, 'CjkUnifiedIdeograph87ac', [], 'Display', None],
  [ 0x87ad, 'CjkUnifiedIdeograph87ad', [], 'Display', None],
  [ 0x87ae, 'CjkUnifiedIdeograph87ae', [], 'Display', None],
  [ 0x87af, 'CjkUnifiedIdeograph87af', [], 'Display', None],
  [ 0x87b0, 'CjkUnifiedIdeograph87b0', [], 'Display', None],
  [ 0x87b1, 'CjkUnifiedIdeograph87b1', [], 'Display', None],
  [ 0x87b2, 'CjkUnifiedIdeograph87b2', [], 'Display', None],
  [ 0x87b3, 'CjkUnifiedIdeograph87b3', [], 'Display', None],
  [ 0x87b4, 'CjkUnifiedIdeograph87b4', [], 'Display', None],
  [ 0x87b5, 'CjkUnifiedIdeograph87b5', [], 'Display', None],
  [ 0x87b6, 'CjkUnifiedIdeograph87b6', [], 'Display', None],
  [ 0x87b7, 'CjkUnifiedIdeograph87b7', [], 'Display', None],
  [ 0x87b8, 'CjkUnifiedIdeograph87b8', [], 'Display', None],
  [ 0x87b9, 'CjkUnifiedIdeograph87b9', [], 'Display', None],
  [ 0x87ba, 'CjkUnifiedIdeograph87ba', [], 'Display', None],
  [ 0x87bb, 'CjkUnifiedIdeograph87bb', [], 'Display', None],
  [ 0x87bc, 'CjkUnifiedIdeograph87bc', [], 'Display', None],
  [ 0x87bd, 'CjkUnifiedIdeograph87bd', [], 'Display', None],
  [ 0x87be, 'CjkUnifiedIdeograph87be', [], 'Display', None],
  [ 0x87bf, 'CjkUnifiedIdeograph87bf', [], 'Display', None],
  [ 0x87c0, 'CjkUnifiedIdeograph87c0', [], 'Display', None],
  [ 0x87c1, 'CjkUnifiedIdeograph87c1', [], 'Display', None],
  [ 0x87c2, 'CjkUnifiedIdeograph87c2', [], 'Display', None],
  [ 0x87c3, 'CjkUnifiedIdeograph87c3', [], 'Display', None],
  [ 0x87c4, 'CjkUnifiedIdeograph87c4', [], 'Display', None],
  [ 0x87c5, 'CjkUnifiedIdeograph87c5', [], 'Display', None],
  [ 0x87c6, 'CjkUnifiedIdeograph87c6', [], 'Display', None],
  [ 0x87c7, 'CjkUnifiedIdeograph87c7', [], 'Display', None],
  [ 0x87c8, 'CjkUnifiedIdeograph87c8', [], 'Display', None],
  [ 0x87c9, 'CjkUnifiedIdeograph87c9', [], 'Display', None],
  [ 0x87ca, 'CjkUnifiedIdeograph87ca', [], 'Display', None],
  [ 0x87cb, 'CjkUnifiedIdeograph87cb', [], 'Display', None],
  [ 0x87cc, 'CjkUnifiedIdeograph87cc', [], 'Display', None],
  [ 0x87cd, 'CjkUnifiedIdeograph87cd', [], 'Display', None],
  [ 0x87ce, 'CjkUnifiedIdeograph87ce', [], 'Display', None],
  [ 0x87cf, 'CjkUnifiedIdeograph87cf', [], 'Display', None],
  [ 0x87d0, 'CjkUnifiedIdeograph87d0', [], 'Display', None],
  [ 0x87d1, 'CjkUnifiedIdeograph87d1', [], 'Display', None],
  [ 0x87d2, 'CjkUnifiedIdeograph87d2', [], 'Display', None],
  [ 0x87d3, 'CjkUnifiedIdeograph87d3', [], 'Display', None],
  [ 0x87d4, 'CjkUnifiedIdeograph87d4', [], 'Display', None],
  [ 0x87d5, 'CjkUnifiedIdeograph87d5', [], 'Display', None],
  [ 0x87d6, 'CjkUnifiedIdeograph87d6', [], 'Display', None],
  [ 0x87d7, 'CjkUnifiedIdeograph87d7', [], 'Display', None],
  [ 0x87d8, 'CjkUnifiedIdeograph87d8', [], 'Display', None],
  [ 0x87d9, 'CjkUnifiedIdeograph87d9', [], 'Display', None],
  [ 0x87da, 'CjkUnifiedIdeograph87da', [], 'Display', None],
  [ 0x87db, 'CjkUnifiedIdeograph87db', [], 'Display', None],
  [ 0x87dc, 'CjkUnifiedIdeograph87dc', [], 'Display', None],
  [ 0x87dd, 'CjkUnifiedIdeograph87dd', [], 'Display', None],
  [ 0x87de, 'CjkUnifiedIdeograph87de', [], 'Display', None],
  [ 0x87df, 'CjkUnifiedIdeograph87df', [], 'Display', None],
  [ 0x87e0, 'CjkUnifiedIdeograph87e0', [], 'Display', None],
  [ 0x87e1, 'CjkUnifiedIdeograph87e1', [], 'Display', None],
  [ 0x87e2, 'CjkUnifiedIdeograph87e2', [], 'Display', None],
  [ 0x87e3, 'CjkUnifiedIdeograph87e3', [], 'Display', None],
  [ 0x87e4, 'CjkUnifiedIdeograph87e4', [], 'Display', None],
  [ 0x87e5, 'CjkUnifiedIdeograph87e5', [], 'Display', None],
  [ 0x87e6, 'CjkUnifiedIdeograph87e6', [], 'Display', None],
  [ 0x87e7, 'CjkUnifiedIdeograph87e7', [], 'Display', None],
  [ 0x87e8, 'CjkUnifiedIdeograph87e8', [], 'Display', None],
  [ 0x87e9, 'CjkUnifiedIdeograph87e9', [], 'Display', None],
  [ 0x87ea, 'CjkUnifiedIdeograph87ea', [], 'Display', None],
  [ 0x87eb, 'CjkUnifiedIdeograph87eb', [], 'Display', None],
  [ 0x87ec, 'CjkUnifiedIdeograph87ec', [], 'Display', None],
  [ 0x87ed, 'CjkUnifiedIdeograph87ed', [], 'Display', None],
  [ 0x87ee, 'CjkUnifiedIdeograph87ee', [], 'Display', None],
  [ 0x87ef, 'CjkUnifiedIdeograph87ef', [], 'Display', None],
  [ 0x87f0, 'CjkUnifiedIdeograph87f0', [], 'Display', None],
  [ 0x87f1, 'CjkUnifiedIdeograph87f1', [], 'Display', None],
  [ 0x87f2, 'CjkUnifiedIdeograph87f2', [], 'Display', None],
  [ 0x87f3, 'CjkUnifiedIdeograph87f3', [], 'Display', None],
  [ 0x87f4, 'CjkUnifiedIdeograph87f4', [], 'Display', None],
  [ 0x87f5, 'CjkUnifiedIdeograph87f5', [], 'Display', None],
  [ 0x87f6, 'CjkUnifiedIdeograph87f6', [], 'Display', None],
  [ 0x87f7, 'CjkUnifiedIdeograph87f7', [], 'Display', None],
  [ 0x87f8, 'CjkUnifiedIdeograph87f8', [], 'Display', None],
  [ 0x87f9, 'CjkUnifiedIdeograph87f9', [], 'Display', None],
  [ 0x87fa, 'CjkUnifiedIdeograph87fa', [], 'Display', None],
  [ 0x87fb, 'CjkUnifiedIdeograph87fb', [], 'Display', None],
  [ 0x87fc, 'CjkUnifiedIdeograph87fc', [], 'Display', None],
  [ 0x87fd, 'CjkUnifiedIdeograph87fd', [], 'Display', None],
  [ 0x87fe, 'CjkUnifiedIdeograph87fe', [], 'Display', None],
  [ 0x87ff, 'CjkUnifiedIdeograph87ff', [], 'Display', None],
  [ 0x8800, 'CjkUnifiedIdeograph8800', [], 'Display', None],
  [ 0x8801, 'CjkUnifiedIdeograph8801', [], 'Display', None],
  [ 0x8802, 'CjkUnifiedIdeograph8802', [], 'Display', None],
  [ 0x8803, 'CjkUnifiedIdeograph8803', [], 'Display', None],
  [ 0x8804, 'CjkUnifiedIdeograph8804', [], 'Display', None],
  [ 0x8805, 'CjkUnifiedIdeograph8805', [], 'Display', None],
  [ 0x8806, 'CjkUnifiedIdeograph8806', [], 'Display', None],
  [ 0x8807, 'CjkUnifiedIdeograph8807', [], 'Display', None],
  [ 0x8808, 'CjkUnifiedIdeograph8808', [], 'Display', None],
  [ 0x8809, 'CjkUnifiedIdeograph8809', [], 'Display', None],
  [ 0x880a, 'CjkUnifiedIdeograph880a', [], 'Display', None],
  [ 0x880b, 'CjkUnifiedIdeograph880b', [], 'Display', None],
  [ 0x880c, 'CjkUnifiedIdeograph880c', [], 'Display', None],
  [ 0x880d, 'CjkUnifiedIdeograph880d', [], 'Display', None],
  [ 0x880e, 'CjkUnifiedIdeograph880e', [], 'Display', None],
  [ 0x880f, 'CjkUnifiedIdeograph880f', [], 'Display', None],
  [ 0x8810, 'CjkUnifiedIdeograph8810', [], 'Display', None],
  [ 0x8811, 'CjkUnifiedIdeograph8811', [], 'Display', None],
  [ 0x8812, 'CjkUnifiedIdeograph8812', [], 'Display', None],
  [ 0x8813, 'CjkUnifiedIdeograph8813', [], 'Display', None],
  [ 0x8814, 'CjkUnifiedIdeograph8814', [], 'Display', None],
  [ 0x8815, 'CjkUnifiedIdeograph8815', [], 'Display', None],
  [ 0x8816, 'CjkUnifiedIdeograph8816', [], 'Display', None],
  [ 0x8817, 'CjkUnifiedIdeograph8817', [], 'Display', None],
  [ 0x8818, 'CjkUnifiedIdeograph8818', [], 'Display', None],
  [ 0x8819, 'CjkUnifiedIdeograph8819', [], 'Display', None],
  [ 0x881a, 'CjkUnifiedIdeograph881a', [], 'Display', None],
  [ 0x881b, 'CjkUnifiedIdeograph881b', [], 'Display', None],
  [ 0x881c, 'CjkUnifiedIdeograph881c', [], 'Display', None],
  [ 0x881d, 'CjkUnifiedIdeograph881d', [], 'Display', None],
  [ 0x881e, 'CjkUnifiedIdeograph881e', [], 'Display', None],
  [ 0x881f, 'CjkUnifiedIdeograph881f', [], 'Display', None],
  [ 0x8820, 'CjkUnifiedIdeograph8820', [], 'Display', None],
  [ 0x8821, 'CjkUnifiedIdeograph8821', [], 'Display', None],
  [ 0x8822, 'CjkUnifiedIdeograph8822', [], 'Display', None],
  [ 0x8823, 'CjkUnifiedIdeograph8823', [], 'Display', None],
  [ 0x8824, 'CjkUnifiedIdeograph8824', [], 'Display', None],
  [ 0x8825, 'CjkUnifiedIdeograph8825', [], 'Display', None],
  [ 0x8826, 'CjkUnifiedIdeograph8826', [], 'Display', None],
  [ 0x8827, 'CjkUnifiedIdeograph8827', [], 'Display', None],
  [ 0x8828, 'CjkUnifiedIdeograph8828', [], 'Display', None],
  [ 0x8829, 'CjkUnifiedIdeograph8829', [], 'Display', None],
  [ 0x882a, 'CjkUnifiedIdeograph882a', [], 'Display', None],
  [ 0x882b, 'CjkUnifiedIdeograph882b', [], 'Display', None],
  [ 0x882c, 'CjkUnifiedIdeograph882c', [], 'Display', None],
  [ 0x882d, 'CjkUnifiedIdeograph882d', [], 'Display', None],
  [ 0x882e, 'CjkUnifiedIdeograph882e', [], 'Display', None],
  [ 0x882f, 'CjkUnifiedIdeograph882f', [], 'Display', None],
  [ 0x8830, 'CjkUnifiedIdeograph8830', [], 'Display', None],
  [ 0x8831, 'CjkUnifiedIdeograph8831', [], 'Display', None],
  [ 0x8832, 'CjkUnifiedIdeograph8832', [], 'Display', None],
  [ 0x8833, 'CjkUnifiedIdeograph8833', [], 'Display', None],
  [ 0x8834, 'CjkUnifiedIdeograph8834', [], 'Display', None],
  [ 0x8835, 'CjkUnifiedIdeograph8835', [], 'Display', None],
  [ 0x8836, 'CjkUnifiedIdeograph8836', [], 'Display', None],
  [ 0x8837, 'CjkUnifiedIdeograph8837', [], 'Display', None],
  [ 0x8838, 'CjkUnifiedIdeograph8838', [], 'Display', None],
  [ 0x8839, 'CjkUnifiedIdeograph8839', [], 'Display', None],
  [ 0x883a, 'CjkUnifiedIdeograph883a', [], 'Display', None],
  [ 0x883b, 'CjkUnifiedIdeograph883b', [], 'Display', None],
  [ 0x883c, 'CjkUnifiedIdeograph883c', [], 'Display', None],
  [ 0x883d, 'CjkUnifiedIdeograph883d', [], 'Display', None],
  [ 0x883e, 'CjkUnifiedIdeograph883e', [], 'Display', None],
  [ 0x883f, 'CjkUnifiedIdeograph883f', [], 'Display', None],
  [ 0x8840, 'CjkUnifiedIdeograph8840', [], 'Display', None],
  [ 0x8841, 'CjkUnifiedIdeograph8841', [], 'Display', None],
  [ 0x8842, 'CjkUnifiedIdeograph8842', [], 'Display', None],
  [ 0x8843, 'CjkUnifiedIdeograph8843', [], 'Display', None],
  [ 0x8844, 'CjkUnifiedIdeograph8844', [], 'Display', None],
  [ 0x8845, 'CjkUnifiedIdeograph8845', [], 'Display', None],
  [ 0x8846, 'CjkUnifiedIdeograph8846', [], 'Display', None],
  [ 0x8847, 'CjkUnifiedIdeograph8847', [], 'Display', None],
  [ 0x8848, 'CjkUnifiedIdeograph8848', [], 'Display', None],
  [ 0x8849, 'CjkUnifiedIdeograph8849', [], 'Display', None],
  [ 0x884a, 'CjkUnifiedIdeograph884a', [], 'Display', None],
  [ 0x884b, 'CjkUnifiedIdeograph884b', [], 'Display', None],
  [ 0x884c, 'CjkUnifiedIdeograph884c', [], 'Display', None],
  [ 0x884d, 'CjkUnifiedIdeograph884d', [], 'Display', None],
  [ 0x884e, 'CjkUnifiedIdeograph884e', [], 'Display', None],
  [ 0x884f, 'CjkUnifiedIdeograph884f', [], 'Display', None],
  [ 0x8850, 'CjkUnifiedIdeograph8850', [], 'Display', None],
  [ 0x8851, 'CjkUnifiedIdeograph8851', [], 'Display', None],
  [ 0x8852, 'CjkUnifiedIdeograph8852', [], 'Display', None],
  [ 0x8853, 'CjkUnifiedIdeograph8853', [], 'Display', None],
  [ 0x8854, 'CjkUnifiedIdeograph8854', [], 'Display', None],
  [ 0x8855, 'CjkUnifiedIdeograph8855', [], 'Display', None],
  [ 0x8856, 'CjkUnifiedIdeograph8856', [], 'Display', None],
  [ 0x8857, 'CjkUnifiedIdeograph8857', [], 'Display', None],
  [ 0x8858, 'CjkUnifiedIdeograph8858', [], 'Display', None],
  [ 0x8859, 'CjkUnifiedIdeograph8859', [], 'Display', None],
  [ 0x885a, 'CjkUnifiedIdeograph885a', [], 'Display', None],
  [ 0x885b, 'CjkUnifiedIdeograph885b', [], 'Display', None],
  [ 0x885c, 'CjkUnifiedIdeograph885c', [], 'Display', None],
  [ 0x885d, 'CjkUnifiedIdeograph885d', [], 'Display', None],
  [ 0x885e, 'CjkUnifiedIdeograph885e', [], 'Display', None],
  [ 0x885f, 'CjkUnifiedIdeograph885f', [], 'Display', None],
  [ 0x8860, 'CjkUnifiedIdeograph8860', [], 'Display', None],
  [ 0x8861, 'CjkUnifiedIdeograph8861', [], 'Display', None],
  [ 0x8862, 'CjkUnifiedIdeograph8862', [], 'Display', None],
  [ 0x8863, 'CjkUnifiedIdeograph8863', [], 'Display', None],
  [ 0x8864, 'CjkUnifiedIdeograph8864', [], 'Display', None],
  [ 0x8865, 'CjkUnifiedIdeograph8865', [], 'Display', None],
  [ 0x8866, 'CjkUnifiedIdeograph8866', [], 'Display', None],
  [ 0x8867, 'CjkUnifiedIdeograph8867', [], 'Display', None],
  [ 0x8868, 'CjkUnifiedIdeograph8868', [], 'Display', None],
  [ 0x8869, 'CjkUnifiedIdeograph8869', [], 'Display', None],
  [ 0x886a, 'CjkUnifiedIdeograph886a', [], 'Display', None],
  [ 0x886b, 'CjkUnifiedIdeograph886b', [], 'Display', None],
  [ 0x886c, 'CjkUnifiedIdeograph886c', [], 'Display', None],
  [ 0x886d, 'CjkUnifiedIdeograph886d', [], 'Display', None],
  [ 0x886e, 'CjkUnifiedIdeograph886e', [], 'Display', None],
  [ 0x886f, 'CjkUnifiedIdeograph886f', [], 'Display', None],
  [ 0x8870, 'CjkUnifiedIdeograph8870', [], 'Display', None],
  [ 0x8871, 'CjkUnifiedIdeograph8871', [], 'Display', None],
  [ 0x8872, 'CjkUnifiedIdeograph8872', [], 'Display', None],
  [ 0x8873, 'CjkUnifiedIdeograph8873', [], 'Display', None],
  [ 0x8874, 'CjkUnifiedIdeograph8874', [], 'Display', None],
  [ 0x8875, 'CjkUnifiedIdeograph8875', [], 'Display', None],
  [ 0x8876, 'CjkUnifiedIdeograph8876', [], 'Display', None],
  [ 0x8877, 'CjkUnifiedIdeograph8877', [], 'Display', None],
  [ 0x8878, 'CjkUnifiedIdeograph8878', [], 'Display', None],
  [ 0x8879, 'CjkUnifiedIdeograph8879', [], 'Display', None],
  [ 0x887a, 'CjkUnifiedIdeograph887a', [], 'Display', None],
  [ 0x887b, 'CjkUnifiedIdeograph887b', [], 'Display', None],
  [ 0x887c, 'CjkUnifiedIdeograph887c', [], 'Display', None],
  [ 0x887d, 'CjkUnifiedIdeograph887d', [], 'Display', None],
  [ 0x887e, 'CjkUnifiedIdeograph887e', [], 'Display', None],
  [ 0x887f, 'CjkUnifiedIdeograph887f', [], 'Display', None],
  [ 0x8880, 'CjkUnifiedIdeograph8880', [], 'Display', None],
  [ 0x8881, 'CjkUnifiedIdeograph8881', [], 'Display', None],
  [ 0x8882, 'CjkUnifiedIdeograph8882', [], 'Display', None],
  [ 0x8883, 'CjkUnifiedIdeograph8883', [], 'Display', None],
  [ 0x8884, 'CjkUnifiedIdeograph8884', [], 'Display', None],
  [ 0x8885, 'CjkUnifiedIdeograph8885', [], 'Display', None],
  [ 0x8886, 'CjkUnifiedIdeograph8886', [], 'Display', None],
  [ 0x8887, 'CjkUnifiedIdeograph8887', [], 'Display', None],
  [ 0x8888, 'CjkUnifiedIdeograph8888', [], 'Display', None],
  [ 0x8889, 'CjkUnifiedIdeograph8889', [], 'Display', None],
  [ 0x888a, 'CjkUnifiedIdeograph888a', [], 'Display', None],
  [ 0x888b, 'CjkUnifiedIdeograph888b', [], 'Display', None],
  [ 0x888c, 'CjkUnifiedIdeograph888c', [], 'Display', None],
  [ 0x888d, 'CjkUnifiedIdeograph888d', [], 'Display', None],
  [ 0x888e, 'CjkUnifiedIdeograph888e', [], 'Display', None],
  [ 0x888f, 'CjkUnifiedIdeograph888f', [], 'Display', None],
  [ 0x8890, 'CjkUnifiedIdeograph8890', [], 'Display', None],
  [ 0x8891, 'CjkUnifiedIdeograph8891', [], 'Display', None],
  [ 0x8892, 'CjkUnifiedIdeograph8892', [], 'Display', None],
  [ 0x8893, 'CjkUnifiedIdeograph8893', [], 'Display', None],
  [ 0x8894, 'CjkUnifiedIdeograph8894', [], 'Display', None],
  [ 0x8895, 'CjkUnifiedIdeograph8895', [], 'Display', None],
  [ 0x8896, 'CjkUnifiedIdeograph8896', [], 'Display', None],
  [ 0x8897, 'CjkUnifiedIdeograph8897', [], 'Display', None],
  [ 0x8898, 'CjkUnifiedIdeograph8898', [], 'Display', None],
  [ 0x8899, 'CjkUnifiedIdeograph8899', [], 'Display', None],
  [ 0x889a, 'CjkUnifiedIdeograph889a', [], 'Display', None],
  [ 0x889b, 'CjkUnifiedIdeograph889b', [], 'Display', None],
  [ 0x889c, 'CjkUnifiedIdeograph889c', [], 'Display', None],
  [ 0x889d, 'CjkUnifiedIdeograph889d', [], 'Display', None],
  [ 0x889e, 'CjkUnifiedIdeograph889e', [], 'Display', None],
  [ 0x889f, 'CjkUnifiedIdeograph889f', [], 'Display', None],
  [ 0x88a0, 'CjkUnifiedIdeograph88a0', [], 'Display', None],
  [ 0x88a1, 'CjkUnifiedIdeograph88a1', [], 'Display', None],
  [ 0x88a2, 'CjkUnifiedIdeograph88a2', [], 'Display', None],
  [ 0x88a3, 'CjkUnifiedIdeograph88a3', [], 'Display', None],
  [ 0x88a4, 'CjkUnifiedIdeograph88a4', [], 'Display', None],
  [ 0x88a5, 'CjkUnifiedIdeograph88a5', [], 'Display', None],
  [ 0x88a6, 'CjkUnifiedIdeograph88a6', [], 'Display', None],
  [ 0x88a7, 'CjkUnifiedIdeograph88a7', [], 'Display', None],
  [ 0x88a8, 'CjkUnifiedIdeograph88a8', [], 'Display', None],
  [ 0x88a9, 'CjkUnifiedIdeograph88a9', [], 'Display', None],
  [ 0x88aa, 'CjkUnifiedIdeograph88aa', [], 'Display', None],
  [ 0x88ab, 'CjkUnifiedIdeograph88ab', [], 'Display', None],
  [ 0x88ac, 'CjkUnifiedIdeograph88ac', [], 'Display', None],
  [ 0x88ad, 'CjkUnifiedIdeograph88ad', [], 'Display', None],
  [ 0x88ae, 'CjkUnifiedIdeograph88ae', [], 'Display', None],
  [ 0x88af, 'CjkUnifiedIdeograph88af', [], 'Display', None],
  [ 0x88b0, 'CjkUnifiedIdeograph88b0', [], 'Display', None],
  [ 0x88b1, 'CjkUnifiedIdeograph88b1', [], 'Display', None],
  [ 0x88b2, 'CjkUnifiedIdeograph88b2', [], 'Display', None],
  [ 0x88b3, 'CjkUnifiedIdeograph88b3', [], 'Display', None],
  [ 0x88b4, 'CjkUnifiedIdeograph88b4', [], 'Display', None],
  [ 0x88b5, 'CjkUnifiedIdeograph88b5', [], 'Display', None],
  [ 0x88b6, 'CjkUnifiedIdeograph88b6', [], 'Display', None],
  [ 0x88b7, 'CjkUnifiedIdeograph88b7', [], 'Display', None],
  [ 0x88b8, 'CjkUnifiedIdeograph88b8', [], 'Display', None],
  [ 0x88b9, 'CjkUnifiedIdeograph88b9', [], 'Display', None],
  [ 0x88ba, 'CjkUnifiedIdeograph88ba', [], 'Display', None],
  [ 0x88bb, 'CjkUnifiedIdeograph88bb', [], 'Display', None],
  [ 0x88bc, 'CjkUnifiedIdeograph88bc', [], 'Display', None],
  [ 0x88bd, 'CjkUnifiedIdeograph88bd', [], 'Display', None],
  [ 0x88be, 'CjkUnifiedIdeograph88be', [], 'Display', None],
  [ 0x88bf, 'CjkUnifiedIdeograph88bf', [], 'Display', None],
  [ 0x88c0, 'CjkUnifiedIdeograph88c0', [], 'Display', None],
  [ 0x88c1, 'CjkUnifiedIdeograph88c1', [], 'Display', None],
  [ 0x88c2, 'CjkUnifiedIdeograph88c2', [], 'Display', None],
  [ 0x88c3, 'CjkUnifiedIdeograph88c3', [], 'Display', None],
  [ 0x88c4, 'CjkUnifiedIdeograph88c4', [], 'Display', None],
  [ 0x88c5, 'CjkUnifiedIdeograph88c5', [], 'Display', None],
  [ 0x88c6, 'CjkUnifiedIdeograph88c6', [], 'Display', None],
  [ 0x88c7, 'CjkUnifiedIdeograph88c7', [], 'Display', None],
  [ 0x88c8, 'CjkUnifiedIdeograph88c8', [], 'Display', None],
  [ 0x88c9, 'CjkUnifiedIdeograph88c9', [], 'Display', None],
  [ 0x88ca, 'CjkUnifiedIdeograph88ca', [], 'Display', None],
  [ 0x88cb, 'CjkUnifiedIdeograph88cb', [], 'Display', None],
  [ 0x88cc, 'CjkUnifiedIdeograph88cc', [], 'Display', None],
  [ 0x88cd, 'CjkUnifiedIdeograph88cd', [], 'Display', None],
  [ 0x88ce, 'CjkUnifiedIdeograph88ce', [], 'Display', None],
  [ 0x88cf, 'CjkUnifiedIdeograph88cf', [], 'Display', None],
  [ 0x88d0, 'CjkUnifiedIdeograph88d0', [], 'Display', None],
  [ 0x88d1, 'CjkUnifiedIdeograph88d1', [], 'Display', None],
  [ 0x88d2, 'CjkUnifiedIdeograph88d2', [], 'Display', None],
  [ 0x88d3, 'CjkUnifiedIdeograph88d3', [], 'Display', None],
  [ 0x88d4, 'CjkUnifiedIdeograph88d4', [], 'Display', None],
  [ 0x88d5, 'CjkUnifiedIdeograph88d5', [], 'Display', None],
  [ 0x88d6, 'CjkUnifiedIdeograph88d6', [], 'Display', None],
  [ 0x88d7, 'CjkUnifiedIdeograph88d7', [], 'Display', None],
  [ 0x88d8, 'CjkUnifiedIdeograph88d8', [], 'Display', None],
  [ 0x88d9, 'CjkUnifiedIdeograph88d9', [], 'Display', None],
  [ 0x88da, 'CjkUnifiedIdeograph88da', [], 'Display', None],
  [ 0x88db, 'CjkUnifiedIdeograph88db', [], 'Display', None],
  [ 0x88dc, 'CjkUnifiedIdeograph88dc', [], 'Display', None],
  [ 0x88dd, 'CjkUnifiedIdeograph88dd', [], 'Display', None],
  [ 0x88de, 'CjkUnifiedIdeograph88de', [], 'Display', None],
  [ 0x88df, 'CjkUnifiedIdeograph88df', [], 'Display', None],
  [ 0x88e0, 'CjkUnifiedIdeograph88e0', [], 'Display', None],
  [ 0x88e1, 'CjkUnifiedIdeograph88e1', [], 'Display', None],
  [ 0x88e2, 'CjkUnifiedIdeograph88e2', [], 'Display', None],
  [ 0x88e3, 'CjkUnifiedIdeograph88e3', [], 'Display', None],
  [ 0x88e4, 'CjkUnifiedIdeograph88e4', [], 'Display', None],
  [ 0x88e5, 'CjkUnifiedIdeograph88e5', [], 'Display', None],
  [ 0x88e6, 'CjkUnifiedIdeograph88e6', [], 'Display', None],
  [ 0x88e7, 'CjkUnifiedIdeograph88e7', [], 'Display', None],
  [ 0x88e8, 'CjkUnifiedIdeograph88e8', [], 'Display', None],
  [ 0x88e9, 'CjkUnifiedIdeograph88e9', [], 'Display', None],
  [ 0x88ea, 'CjkUnifiedIdeograph88ea', [], 'Display', None],
  [ 0x88eb, 'CjkUnifiedIdeograph88eb', [], 'Display', None],
  [ 0x88ec, 'CjkUnifiedIdeograph88ec', [], 'Display', None],
  [ 0x88ed, 'CjkUnifiedIdeograph88ed', [], 'Display', None],
  [ 0x88ee, 'CjkUnifiedIdeograph88ee', [], 'Display', None],
  [ 0x88ef, 'CjkUnifiedIdeograph88ef', [], 'Display', None],
  [ 0x88f0, 'CjkUnifiedIdeograph88f0', [], 'Display', None],
  [ 0x88f1, 'CjkUnifiedIdeograph88f1', [], 'Display', None],
  [ 0x88f2, 'CjkUnifiedIdeograph88f2', [], 'Display', None],
  [ 0x88f3, 'CjkUnifiedIdeograph88f3', [], 'Display', None],
  [ 0x88f4, 'CjkUnifiedIdeograph88f4', [], 'Display', None],
  [ 0x88f5, 'CjkUnifiedIdeograph88f5', [], 'Display', None],
  [ 0x88f6, 'CjkUnifiedIdeograph88f6', [], 'Display', None],
  [ 0x88f7, 'CjkUnifiedIdeograph88f7', [], 'Display', None],
  [ 0x88f8, 'CjkUnifiedIdeograph88f8', [], 'Display', None],
  [ 0x88f9, 'CjkUnifiedIdeograph88f9', [], 'Display', None],
  [ 0x88fa, 'CjkUnifiedIdeograph88fa', [], 'Display', None],
  [ 0x88fb, 'CjkUnifiedIdeograph88fb', [], 'Display', None],
  [ 0x88fc, 'CjkUnifiedIdeograph88fc', [], 'Display', None],
  [ 0x88fd, 'CjkUnifiedIdeograph88fd', [], 'Display', None],
  [ 0x88fe, 'CjkUnifiedIdeograph88fe', [], 'Display', None],
  [ 0x88ff, 'CjkUnifiedIdeograph88ff', [], 'Display', None],
  [ 0x8900, 'CjkUnifiedIdeograph8900', [], 'Display', None],
  [ 0x8901, 'CjkUnifiedIdeograph8901', [], 'Display', None],
  [ 0x8902, 'CjkUnifiedIdeograph8902', [], 'Display', None],
  [ 0x8903, 'CjkUnifiedIdeograph8903', [], 'Display', None],
  [ 0x8904, 'CjkUnifiedIdeograph8904', [], 'Display', None],
  [ 0x8905, 'CjkUnifiedIdeograph8905', [], 'Display', None],
  [ 0x8906, 'CjkUnifiedIdeograph8906', [], 'Display', None],
  [ 0x8907, 'CjkUnifiedIdeograph8907', [], 'Display', None],
  [ 0x8908, 'CjkUnifiedIdeograph8908', [], 'Display', None],
  [ 0x8909, 'CjkUnifiedIdeograph8909', [], 'Display', None],
  [ 0x890a, 'CjkUnifiedIdeograph890a', [], 'Display', None],
  [ 0x890b, 'CjkUnifiedIdeograph890b', [], 'Display', None],
  [ 0x890c, 'CjkUnifiedIdeograph890c', [], 'Display', None],
  [ 0x890d, 'CjkUnifiedIdeograph890d', [], 'Display', None],
  [ 0x890e, 'CjkUnifiedIdeograph890e', [], 'Display', None],
  [ 0x890f, 'CjkUnifiedIdeograph890f', [], 'Display', None],
  [ 0x8910, 'CjkUnifiedIdeograph8910', [], 'Display', None],
  [ 0x8911, 'CjkUnifiedIdeograph8911', [], 'Display', None],
  [ 0x8912, 'CjkUnifiedIdeograph8912', [], 'Display', None],
  [ 0x8913, 'CjkUnifiedIdeograph8913', [], 'Display', None],
  [ 0x8914, 'CjkUnifiedIdeograph8914', [], 'Display', None],
  [ 0x8915, 'CjkUnifiedIdeograph8915', [], 'Display', None],
  [ 0x8916, 'CjkUnifiedIdeograph8916', [], 'Display', None],
  [ 0x8917, 'CjkUnifiedIdeograph8917', [], 'Display', None],
  [ 0x8918, 'CjkUnifiedIdeograph8918', [], 'Display', None],
  [ 0x8919, 'CjkUnifiedIdeograph8919', [], 'Display', None],
  [ 0x891a, 'CjkUnifiedIdeograph891a', [], 'Display', None],
  [ 0x891b, 'CjkUnifiedIdeograph891b', [], 'Display', None],
  [ 0x891c, 'CjkUnifiedIdeograph891c', [], 'Display', None],
  [ 0x891d, 'CjkUnifiedIdeograph891d', [], 'Display', None],
  [ 0x891e, 'CjkUnifiedIdeograph891e', [], 'Display', None],
  [ 0x891f, 'CjkUnifiedIdeograph891f', [], 'Display', None],
  [ 0x8920, 'CjkUnifiedIdeograph8920', [], 'Display', None],
  [ 0x8921, 'CjkUnifiedIdeograph8921', [], 'Display', None],
  [ 0x8922, 'CjkUnifiedIdeograph8922', [], 'Display', None],
  [ 0x8923, 'CjkUnifiedIdeograph8923', [], 'Display', None],
  [ 0x8924, 'CjkUnifiedIdeograph8924', [], 'Display', None],
  [ 0x8925, 'CjkUnifiedIdeograph8925', [], 'Display', None],
  [ 0x8926, 'CjkUnifiedIdeograph8926', [], 'Display', None],
  [ 0x8927, 'CjkUnifiedIdeograph8927', [], 'Display', None],
  [ 0x8928, 'CjkUnifiedIdeograph8928', [], 'Display', None],
  [ 0x8929, 'CjkUnifiedIdeograph8929', [], 'Display', None],
  [ 0x892a, 'CjkUnifiedIdeograph892a', [], 'Display', None],
  [ 0x892b, 'CjkUnifiedIdeograph892b', [], 'Display', None],
  [ 0x892c, 'CjkUnifiedIdeograph892c', [], 'Display', None],
  [ 0x892d, 'CjkUnifiedIdeograph892d', [], 'Display', None],
  [ 0x892e, 'CjkUnifiedIdeograph892e', [], 'Display', None],
  [ 0x892f, 'CjkUnifiedIdeograph892f', [], 'Display', None],
  [ 0x8930, 'CjkUnifiedIdeograph8930', [], 'Display', None],
  [ 0x8931, 'CjkUnifiedIdeograph8931', [], 'Display', None],
  [ 0x8932, 'CjkUnifiedIdeograph8932', [], 'Display', None],
  [ 0x8933, 'CjkUnifiedIdeograph8933', [], 'Display', None],
  [ 0x8934, 'CjkUnifiedIdeograph8934', [], 'Display', None],
  [ 0x8935, 'CjkUnifiedIdeograph8935', [], 'Display', None],
  [ 0x8936, 'CjkUnifiedIdeograph8936', [], 'Display', None],
  [ 0x8937, 'CjkUnifiedIdeograph8937', [], 'Display', None],
  [ 0x8938, 'CjkUnifiedIdeograph8938', [], 'Display', None],
  [ 0x8939, 'CjkUnifiedIdeograph8939', [], 'Display', None],
  [ 0x893a, 'CjkUnifiedIdeograph893a', [], 'Display', None],
  [ 0x893b, 'CjkUnifiedIdeograph893b', [], 'Display', None],
  [ 0x893c, 'CjkUnifiedIdeograph893c', [], 'Display', None],
  [ 0x893d, 'CjkUnifiedIdeograph893d', [], 'Display', None],
  [ 0x893e, 'CjkUnifiedIdeograph893e', [], 'Display', None],
  [ 0x893f, 'CjkUnifiedIdeograph893f', [], 'Display', None],
  [ 0x8940, 'CjkUnifiedIdeograph8940', [], 'Display', None],
  [ 0x8941, 'CjkUnifiedIdeograph8941', [], 'Display', None],
  [ 0x8942, 'CjkUnifiedIdeograph8942', [], 'Display', None],
  [ 0x8943, 'CjkUnifiedIdeograph8943', [], 'Display', None],
  [ 0x8944, 'CjkUnifiedIdeograph8944', [], 'Display', None],
  [ 0x8945, 'CjkUnifiedIdeograph8945', [], 'Display', None],
  [ 0x8946, 'CjkUnifiedIdeograph8946', [], 'Display', None],
  [ 0x8947, 'CjkUnifiedIdeograph8947', [], 'Display', None],
  [ 0x8948, 'CjkUnifiedIdeograph8948', [], 'Display', None],
  [ 0x8949, 'CjkUnifiedIdeograph8949', [], 'Display', None],
  [ 0x894a, 'CjkUnifiedIdeograph894a', [], 'Display', None],
  [ 0x894b, 'CjkUnifiedIdeograph894b', [], 'Display', None],
  [ 0x894c, 'CjkUnifiedIdeograph894c', [], 'Display', None],
  [ 0x894d, 'CjkUnifiedIdeograph894d', [], 'Display', None],
  [ 0x894e, 'CjkUnifiedIdeograph894e', [], 'Display', None],
  [ 0x894f, 'CjkUnifiedIdeograph894f', [], 'Display', None],
  [ 0x8950, 'CjkUnifiedIdeograph8950', [], 'Display', None],
  [ 0x8951, 'CjkUnifiedIdeograph8951', [], 'Display', None],
  [ 0x8952, 'CjkUnifiedIdeograph8952', [], 'Display', None],
  [ 0x8953, 'CjkUnifiedIdeograph8953', [], 'Display', None],
  [ 0x8954, 'CjkUnifiedIdeograph8954', [], 'Display', None],
  [ 0x8955, 'CjkUnifiedIdeograph8955', [], 'Display', None],
  [ 0x8956, 'CjkUnifiedIdeograph8956', [], 'Display', None],
  [ 0x8957, 'CjkUnifiedIdeograph8957', [], 'Display', None],
  [ 0x8958, 'CjkUnifiedIdeograph8958', [], 'Display', None],
  [ 0x8959, 'CjkUnifiedIdeograph8959', [], 'Display', None],
  [ 0x895a, 'CjkUnifiedIdeograph895a', [], 'Display', None],
  [ 0x895b, 'CjkUnifiedIdeograph895b', [], 'Display', None],
  [ 0x895c, 'CjkUnifiedIdeograph895c', [], 'Display', None],
  [ 0x895d, 'CjkUnifiedIdeograph895d', [], 'Display', None],
  [ 0x895e, 'CjkUnifiedIdeograph895e', [], 'Display', None],
  [ 0x895f, 'CjkUnifiedIdeograph895f', [], 'Display', None],
  [ 0x8960, 'CjkUnifiedIdeograph8960', [], 'Display', None],
  [ 0x8961, 'CjkUnifiedIdeograph8961', [], 'Display', None],
  [ 0x8962, 'CjkUnifiedIdeograph8962', [], 'Display', None],
  [ 0x8963, 'CjkUnifiedIdeograph8963', [], 'Display', None],
  [ 0x8964, 'CjkUnifiedIdeograph8964', [], 'Display', None],
  [ 0x8965, 'CjkUnifiedIdeograph8965', [], 'Display', None],
  [ 0x8966, 'CjkUnifiedIdeograph8966', [], 'Display', None],
  [ 0x8967, 'CjkUnifiedIdeograph8967', [], 'Display', None],
  [ 0x8968, 'CjkUnifiedIdeograph8968', [], 'Display', None],
  [ 0x8969, 'CjkUnifiedIdeograph8969', [], 'Display', None],
  [ 0x896a, 'CjkUnifiedIdeograph896a', [], 'Display', None],
  [ 0x896b, 'CjkUnifiedIdeograph896b', [], 'Display', None],
  [ 0x896c, 'CjkUnifiedIdeograph896c', [], 'Display', None],
  [ 0x896d, 'CjkUnifiedIdeograph896d', [], 'Display', None],
  [ 0x896e, 'CjkUnifiedIdeograph896e', [], 'Display', None],
  [ 0x896f, 'CjkUnifiedIdeograph896f', [], 'Display', None],
  [ 0x8970, 'CjkUnifiedIdeograph8970', [], 'Display', None],
  [ 0x8971, 'CjkUnifiedIdeograph8971', [], 'Display', None],
  [ 0x8972, 'CjkUnifiedIdeograph8972', [], 'Display', None],
  [ 0x8973, 'CjkUnifiedIdeograph8973', [], 'Display', None],
  [ 0x8974, 'CjkUnifiedIdeograph8974', [], 'Display', None],
  [ 0x8975, 'CjkUnifiedIdeograph8975', [], 'Display', None],
  [ 0x8976, 'CjkUnifiedIdeograph8976', [], 'Display', None],
  [ 0x8977, 'CjkUnifiedIdeograph8977', [], 'Display', None],
  [ 0x8978, 'CjkUnifiedIdeograph8978', [], 'Display', None],
  [ 0x8979, 'CjkUnifiedIdeograph8979', [], 'Display', None],
  [ 0x897a, 'CjkUnifiedIdeograph897a', [], 'Display', None],
  [ 0x897b, 'CjkUnifiedIdeograph897b', [], 'Display', None],
  [ 0x897c, 'CjkUnifiedIdeograph897c', [], 'Display', None],
  [ 0x897d, 'CjkUnifiedIdeograph897d', [], 'Display', None],
  [ 0x897e, 'CjkUnifiedIdeograph897e', [], 'Display', None],
  [ 0x897f, 'CjkUnifiedIdeograph897f', [], 'Display', None],
  [ 0x8980, 'CjkUnifiedIdeograph8980', [], 'Display', None],
  [ 0x8981, 'CjkUnifiedIdeograph8981', [], 'Display', None],
  [ 0x8982, 'CjkUnifiedIdeograph8982', [], 'Display', None],
  [ 0x8983, 'CjkUnifiedIdeograph8983', [], 'Display', None],
  [ 0x8984, 'CjkUnifiedIdeograph8984', [], 'Display', None],
  [ 0x8985, 'CjkUnifiedIdeograph8985', [], 'Display', None],
  [ 0x8986, 'CjkUnifiedIdeograph8986', [], 'Display', None],
  [ 0x8987, 'CjkUnifiedIdeograph8987', [], 'Display', None],
  [ 0x8988, 'CjkUnifiedIdeograph8988', [], 'Display', None],
  [ 0x8989, 'CjkUnifiedIdeograph8989', [], 'Display', None],
  [ 0x898a, 'CjkUnifiedIdeograph898a', [], 'Display', None],
  [ 0x898b, 'CjkUnifiedIdeograph898b', [], 'Display', None],
  [ 0x898c, 'CjkUnifiedIdeograph898c', [], 'Display', None],
  [ 0x898d, 'CjkUnifiedIdeograph898d', [], 'Display', None],
  [ 0x898e, 'CjkUnifiedIdeograph898e', [], 'Display', None],
  [ 0x898f, 'CjkUnifiedIdeograph898f', [], 'Display', None],
  [ 0x8990, 'CjkUnifiedIdeograph8990', [], 'Display', None],
  [ 0x8991, 'CjkUnifiedIdeograph8991', [], 'Display', None],
  [ 0x8992, 'CjkUnifiedIdeograph8992', [], 'Display', None],
  [ 0x8993, 'CjkUnifiedIdeograph8993', [], 'Display', None],
  [ 0x8994, 'CjkUnifiedIdeograph8994', [], 'Display', None],
  [ 0x8995, 'CjkUnifiedIdeograph8995', [], 'Display', None],
  [ 0x8996, 'CjkUnifiedIdeograph8996', [], 'Display', None],
  [ 0x8997, 'CjkUnifiedIdeograph8997', [], 'Display', None],
  [ 0x8998, 'CjkUnifiedIdeograph8998', [], 'Display', None],
  [ 0x8999, 'CjkUnifiedIdeograph8999', [], 'Display', None],
  [ 0x899a, 'CjkUnifiedIdeograph899a', [], 'Display', None],
  [ 0x899b, 'CjkUnifiedIdeograph899b', [], 'Display', None],
  [ 0x899c, 'CjkUnifiedIdeograph899c', [], 'Display', None],
  [ 0x899d, 'CjkUnifiedIdeograph899d', [], 'Display', None],
  [ 0x899e, 'CjkUnifiedIdeograph899e', [], 'Display', None],
  [ 0x899f, 'CjkUnifiedIdeograph899f', [], 'Display', None],
  [ 0x89a0, 'CjkUnifiedIdeograph89a0', [], 'Display', None],
  [ 0x89a1, 'CjkUnifiedIdeograph89a1', [], 'Display', None],
  [ 0x89a2, 'CjkUnifiedIdeograph89a2', [], 'Display', None],
  [ 0x89a3, 'CjkUnifiedIdeograph89a3', [], 'Display', None],
  [ 0x89a4, 'CjkUnifiedIdeograph89a4', [], 'Display', None],
  [ 0x89a5, 'CjkUnifiedIdeograph89a5', [], 'Display', None],
  [ 0x89a6, 'CjkUnifiedIdeograph89a6', [], 'Display', None],
  [ 0x89a7, 'CjkUnifiedIdeograph89a7', [], 'Display', None],
  [ 0x89a8, 'CjkUnifiedIdeograph89a8', [], 'Display', None],
  [ 0x89a9, 'CjkUnifiedIdeograph89a9', [], 'Display', None],
  [ 0x89aa, 'CjkUnifiedIdeograph89aa', [], 'Display', None],
  [ 0x89ab, 'CjkUnifiedIdeograph89ab', [], 'Display', None],
  [ 0x89ac, 'CjkUnifiedIdeograph89ac', [], 'Display', None],
  [ 0x89ad, 'CjkUnifiedIdeograph89ad', [], 'Display', None],
  [ 0x89ae, 'CjkUnifiedIdeograph89ae', [], 'Display', None],
  [ 0x89af, 'CjkUnifiedIdeograph89af', [], 'Display', None],
  [ 0x89b0, 'CjkUnifiedIdeograph89b0', [], 'Display', None],
  [ 0x89b1, 'CjkUnifiedIdeograph89b1', [], 'Display', None],
  [ 0x89b2, 'CjkUnifiedIdeograph89b2', [], 'Display', None],
  [ 0x89b3, 'CjkUnifiedIdeograph89b3', [], 'Display', None],
  [ 0x89b4, 'CjkUnifiedIdeograph89b4', [], 'Display', None],
  [ 0x89b5, 'CjkUnifiedIdeograph89b5', [], 'Display', None],
  [ 0x89b6, 'CjkUnifiedIdeograph89b6', [], 'Display', None],
  [ 0x89b7, 'CjkUnifiedIdeograph89b7', [], 'Display', None],
  [ 0x89b8, 'CjkUnifiedIdeograph89b8', [], 'Display', None],
  [ 0x89b9, 'CjkUnifiedIdeograph89b9', [], 'Display', None],
  [ 0x89ba, 'CjkUnifiedIdeograph89ba', [], 'Display', None],
  [ 0x89bb, 'CjkUnifiedIdeograph89bb', [], 'Display', None],
  [ 0x89bc, 'CjkUnifiedIdeograph89bc', [], 'Display', None],
  [ 0x89bd, 'CjkUnifiedIdeograph89bd', [], 'Display', None],
  [ 0x89be, 'CjkUnifiedIdeograph89be', [], 'Display', None],
  [ 0x89bf, 'CjkUnifiedIdeograph89bf', [], 'Display', None],
  [ 0x89c0, 'CjkUnifiedIdeograph89c0', [], 'Display', None],
  [ 0x89c1, 'CjkUnifiedIdeograph89c1', [], 'Display', None],
  [ 0x89c2, 'CjkUnifiedIdeograph89c2', [], 'Display', None],
  [ 0x89c3, 'CjkUnifiedIdeograph89c3', [], 'Display', None],
  [ 0x89c4, 'CjkUnifiedIdeograph89c4', [], 'Display', None],
  [ 0x89c5, 'CjkUnifiedIdeograph89c5', [], 'Display', None],
  [ 0x89c6, 'CjkUnifiedIdeograph89c6', [], 'Display', None],
  [ 0x89c7, 'CjkUnifiedIdeograph89c7', [], 'Display', None],
  [ 0x89c8, 'CjkUnifiedIdeograph89c8', [], 'Display', None],
  [ 0x89c9, 'CjkUnifiedIdeograph89c9', [], 'Display', None],
  [ 0x89ca, 'CjkUnifiedIdeograph89ca', [], 'Display', None],
  [ 0x89cb, 'CjkUnifiedIdeograph89cb', [], 'Display', None],
  [ 0x89cc, 'CjkUnifiedIdeograph89cc', [], 'Display', None],
  [ 0x89cd, 'CjkUnifiedIdeograph89cd', [], 'Display', None],
  [ 0x89ce, 'CjkUnifiedIdeograph89ce', [], 'Display', None],
  [ 0x89cf, 'CjkUnifiedIdeograph89cf', [], 'Display', None],
  [ 0x89d0, 'CjkUnifiedIdeograph89d0', [], 'Display', None],
  [ 0x89d1, 'CjkUnifiedIdeograph89d1', [], 'Display', None],
  [ 0x89d2, 'CjkUnifiedIdeograph89d2', [], 'Display', None],
  [ 0x89d3, 'CjkUnifiedIdeograph89d3', [], 'Display', None],
  [ 0x89d4, 'CjkUnifiedIdeograph89d4', [], 'Display', None],
  [ 0x89d5, 'CjkUnifiedIdeograph89d5', [], 'Display', None],
  [ 0x89d6, 'CjkUnifiedIdeograph89d6', [], 'Display', None],
  [ 0x89d7, 'CjkUnifiedIdeograph89d7', [], 'Display', None],
  [ 0x89d8, 'CjkUnifiedIdeograph89d8', [], 'Display', None],
  [ 0x89d9, 'CjkUnifiedIdeograph89d9', [], 'Display', None],
  [ 0x89da, 'CjkUnifiedIdeograph89da', [], 'Display', None],
  [ 0x89db, 'CjkUnifiedIdeograph89db', [], 'Display', None],
  [ 0x89dc, 'CjkUnifiedIdeograph89dc', [], 'Display', None],
  [ 0x89dd, 'CjkUnifiedIdeograph89dd', [], 'Display', None],
  [ 0x89de, 'CjkUnifiedIdeograph89de', [], 'Display', None],
  [ 0x89df, 'CjkUnifiedIdeograph89df', [], 'Display', None],
  [ 0x89e0, 'CjkUnifiedIdeograph89e0', [], 'Display', None],
  [ 0x89e1, 'CjkUnifiedIdeograph89e1', [], 'Display', None],
  [ 0x89e2, 'CjkUnifiedIdeograph89e2', [], 'Display', None],
  [ 0x89e3, 'CjkUnifiedIdeograph89e3', [], 'Display', None],
  [ 0x89e4, 'CjkUnifiedIdeograph89e4', [], 'Display', None],
  [ 0x89e5, 'CjkUnifiedIdeograph89e5', [], 'Display', None],
  [ 0x89e6, 'CjkUnifiedIdeograph89e6', [], 'Display', None],
  [ 0x89e7, 'CjkUnifiedIdeograph89e7', [], 'Display', None],
  [ 0x89e8, 'CjkUnifiedIdeograph89e8', [], 'Display', None],
  [ 0x89e9, 'CjkUnifiedIdeograph89e9', [], 'Display', None],
  [ 0x89ea, 'CjkUnifiedIdeograph89ea', [], 'Display', None],
  [ 0x89eb, 'CjkUnifiedIdeograph89eb', [], 'Display', None],
  [ 0x89ec, 'CjkUnifiedIdeograph89ec', [], 'Display', None],
  [ 0x89ed, 'CjkUnifiedIdeograph89ed', [], 'Display', None],
  [ 0x89ee, 'CjkUnifiedIdeograph89ee', [], 'Display', None],
  [ 0x89ef, 'CjkUnifiedIdeograph89ef', [], 'Display', None],
  [ 0x89f0, 'CjkUnifiedIdeograph89f0', [], 'Display', None],
  [ 0x89f1, 'CjkUnifiedIdeograph89f1', [], 'Display', None],
  [ 0x89f2, 'CjkUnifiedIdeograph89f2', [], 'Display', None],
  [ 0x89f3, 'CjkUnifiedIdeograph89f3', [], 'Display', None],
  [ 0x89f4, 'CjkUnifiedIdeograph89f4', [], 'Display', None],
  [ 0x89f5, 'CjkUnifiedIdeograph89f5', [], 'Display', None],
  [ 0x89f6, 'CjkUnifiedIdeograph89f6', [], 'Display', None],
  [ 0x89f7, 'CjkUnifiedIdeograph89f7', [], 'Display', None],
  [ 0x89f8, 'CjkUnifiedIdeograph89f8', [], 'Display', None],
  [ 0x89f9, 'CjkUnifiedIdeograph89f9', [], 'Display', None],
  [ 0x89fa, 'CjkUnifiedIdeograph89fa', [], 'Display', None],
  [ 0x89fb, 'CjkUnifiedIdeograph89fb', [], 'Display', None],
  [ 0x89fc, 'CjkUnifiedIdeograph89fc', [], 'Display', None],
  [ 0x89fd, 'CjkUnifiedIdeograph89fd', [], 'Display', None],
  [ 0x89fe, 'CjkUnifiedIdeograph89fe', [], 'Display', None],
  [ 0x89ff, 'CjkUnifiedIdeograph89ff', [], 'Display', None],
  [ 0x8a00, 'CjkUnifiedIdeograph8a00', [], 'Display', None],
  [ 0x8a01, 'CjkUnifiedIdeograph8a01', [], 'Display', None],
  [ 0x8a02, 'CjkUnifiedIdeograph8a02', [], 'Display', None],
  [ 0x8a03, 'CjkUnifiedIdeograph8a03', [], 'Display', None],
  [ 0x8a04, 'CjkUnifiedIdeograph8a04', [], 'Display', None],
  [ 0x8a05, 'CjkUnifiedIdeograph8a05', [], 'Display', None],
  [ 0x8a06, 'CjkUnifiedIdeograph8a06', [], 'Display', None],
  [ 0x8a07, 'CjkUnifiedIdeograph8a07', [], 'Display', None],
  [ 0x8a08, 'CjkUnifiedIdeograph8a08', [], 'Display', None],
  [ 0x8a09, 'CjkUnifiedIdeograph8a09', [], 'Display', None],
  [ 0x8a0a, 'CjkUnifiedIdeograph8a0a', [], 'Display', None],
  [ 0x8a0b, 'CjkUnifiedIdeograph8a0b', [], 'Display', None],
  [ 0x8a0c, 'CjkUnifiedIdeograph8a0c', [], 'Display', None],
  [ 0x8a0d, 'CjkUnifiedIdeograph8a0d', [], 'Display', None],
  [ 0x8a0e, 'CjkUnifiedIdeograph8a0e', [], 'Display', None],
  [ 0x8a0f, 'CjkUnifiedIdeograph8a0f', [], 'Display', None],
  [ 0x8a10, 'CjkUnifiedIdeograph8a10', [], 'Display', None],
  [ 0x8a11, 'CjkUnifiedIdeograph8a11', [], 'Display', None],
  [ 0x8a12, 'CjkUnifiedIdeograph8a12', [], 'Display', None],
  [ 0x8a13, 'CjkUnifiedIdeograph8a13', [], 'Display', None],
  [ 0x8a14, 'CjkUnifiedIdeograph8a14', [], 'Display', None],
  [ 0x8a15, 'CjkUnifiedIdeograph8a15', [], 'Display', None],
  [ 0x8a16, 'CjkUnifiedIdeograph8a16', [], 'Display', None],
  [ 0x8a17, 'CjkUnifiedIdeograph8a17', [], 'Display', None],
  [ 0x8a18, 'CjkUnifiedIdeograph8a18', [], 'Display', None],
  [ 0x8a19, 'CjkUnifiedIdeograph8a19', [], 'Display', None],
  [ 0x8a1a, 'CjkUnifiedIdeograph8a1a', [], 'Display', None],
  [ 0x8a1b, 'CjkUnifiedIdeograph8a1b', [], 'Display', None],
  [ 0x8a1c, 'CjkUnifiedIdeograph8a1c', [], 'Display', None],
  [ 0x8a1d, 'CjkUnifiedIdeograph8a1d', [], 'Display', None],
  [ 0x8a1e, 'CjkUnifiedIdeograph8a1e', [], 'Display', None],
  [ 0x8a1f, 'CjkUnifiedIdeograph8a1f', [], 'Display', None],
  [ 0x8a20, 'CjkUnifiedIdeograph8a20', [], 'Display', None],
  [ 0x8a21, 'CjkUnifiedIdeograph8a21', [], 'Display', None],
  [ 0x8a22, 'CjkUnifiedIdeograph8a22', [], 'Display', None],
  [ 0x8a23, 'CjkUnifiedIdeograph8a23', [], 'Display', None],
  [ 0x8a24, 'CjkUnifiedIdeograph8a24', [], 'Display', None],
  [ 0x8a25, 'CjkUnifiedIdeograph8a25', [], 'Display', None],
  [ 0x8a26, 'CjkUnifiedIdeograph8a26', [], 'Display', None],
  [ 0x8a27, 'CjkUnifiedIdeograph8a27', [], 'Display', None],
  [ 0x8a28, 'CjkUnifiedIdeograph8a28', [], 'Display', None],
  [ 0x8a29, 'CjkUnifiedIdeograph8a29', [], 'Display', None],
  [ 0x8a2a, 'CjkUnifiedIdeograph8a2a', [], 'Display', None],
  [ 0x8a2b, 'CjkUnifiedIdeograph8a2b', [], 'Display', None],
  [ 0x8a2c, 'CjkUnifiedIdeograph8a2c', [], 'Display', None],
  [ 0x8a2d, 'CjkUnifiedIdeograph8a2d', [], 'Display', None],
  [ 0x8a2e, 'CjkUnifiedIdeograph8a2e', [], 'Display', None],
  [ 0x8a2f, 'CjkUnifiedIdeograph8a2f', [], 'Display', None],
  [ 0x8a30, 'CjkUnifiedIdeograph8a30', [], 'Display', None],
  [ 0x8a31, 'CjkUnifiedIdeograph8a31', [], 'Display', None],
  [ 0x8a32, 'CjkUnifiedIdeograph8a32', [], 'Display', None],
  [ 0x8a33, 'CjkUnifiedIdeograph8a33', [], 'Display', None],
  [ 0x8a34, 'CjkUnifiedIdeograph8a34', [], 'Display', None],
  [ 0x8a35, 'CjkUnifiedIdeograph8a35', [], 'Display', None],
  [ 0x8a36, 'CjkUnifiedIdeograph8a36', [], 'Display', None],
  [ 0x8a37, 'CjkUnifiedIdeograph8a37', [], 'Display', None],
  [ 0x8a38, 'CjkUnifiedIdeograph8a38', [], 'Display', None],
  [ 0x8a39, 'CjkUnifiedIdeograph8a39', [], 'Display', None],
  [ 0x8a3a, 'CjkUnifiedIdeograph8a3a', [], 'Display', None],
  [ 0x8a3b, 'CjkUnifiedIdeograph8a3b', [], 'Display', None],
  [ 0x8a3c, 'CjkUnifiedIdeograph8a3c', [], 'Display', None],
  [ 0x8a3d, 'CjkUnifiedIdeograph8a3d', [], 'Display', None],
  [ 0x8a3e, 'CjkUnifiedIdeograph8a3e', [], 'Display', None],
  [ 0x8a3f, 'CjkUnifiedIdeograph8a3f', [], 'Display', None],
  [ 0x8a40, 'CjkUnifiedIdeograph8a40', [], 'Display', None],
  [ 0x8a41, 'CjkUnifiedIdeograph8a41', [], 'Display', None],
  [ 0x8a42, 'CjkUnifiedIdeograph8a42', [], 'Display', None],
  [ 0x8a43, 'CjkUnifiedIdeograph8a43', [], 'Display', None],
  [ 0x8a44, 'CjkUnifiedIdeograph8a44', [], 'Display', None],
  [ 0x8a45, 'CjkUnifiedIdeograph8a45', [], 'Display', None],
  [ 0x8a46, 'CjkUnifiedIdeograph8a46', [], 'Display', None],
  [ 0x8a47, 'CjkUnifiedIdeograph8a47', [], 'Display', None],
  [ 0x8a48, 'CjkUnifiedIdeograph8a48', [], 'Display', None],
  [ 0x8a49, 'CjkUnifiedIdeograph8a49', [], 'Display', None],
  [ 0x8a4a, 'CjkUnifiedIdeograph8a4a', [], 'Display', None],
  [ 0x8a4b, 'CjkUnifiedIdeograph8a4b', [], 'Display', None],
  [ 0x8a4c, 'CjkUnifiedIdeograph8a4c', [], 'Display', None],
  [ 0x8a4d, 'CjkUnifiedIdeograph8a4d', [], 'Display', None],
  [ 0x8a4e, 'CjkUnifiedIdeograph8a4e', [], 'Display', None],
  [ 0x8a4f, 'CjkUnifiedIdeograph8a4f', [], 'Display', None],
  [ 0x8a50, 'CjkUnifiedIdeograph8a50', [], 'Display', None],
  [ 0x8a51, 'CjkUnifiedIdeograph8a51', [], 'Display', None],
  [ 0x8a52, 'CjkUnifiedIdeograph8a52', [], 'Display', None],
  [ 0x8a53, 'CjkUnifiedIdeograph8a53', [], 'Display', None],
  [ 0x8a54, 'CjkUnifiedIdeograph8a54', [], 'Display', None],
  [ 0x8a55, 'CjkUnifiedIdeograph8a55', [], 'Display', None],
  [ 0x8a56, 'CjkUnifiedIdeograph8a56', [], 'Display', None],
  [ 0x8a57, 'CjkUnifiedIdeograph8a57', [], 'Display', None],
  [ 0x8a58, 'CjkUnifiedIdeograph8a58', [], 'Display', None],
  [ 0x8a59, 'CjkUnifiedIdeograph8a59', [], 'Display', None],
  [ 0x8a5a, 'CjkUnifiedIdeograph8a5a', [], 'Display', None],
  [ 0x8a5b, 'CjkUnifiedIdeograph8a5b', [], 'Display', None],
  [ 0x8a5c, 'CjkUnifiedIdeograph8a5c', [], 'Display', None],
  [ 0x8a5d, 'CjkUnifiedIdeograph8a5d', [], 'Display', None],
  [ 0x8a5e, 'CjkUnifiedIdeograph8a5e', [], 'Display', None],
  [ 0x8a5f, 'CjkUnifiedIdeograph8a5f', [], 'Display', None],
  [ 0x8a60, 'CjkUnifiedIdeograph8a60', [], 'Display', None],
  [ 0x8a61, 'CjkUnifiedIdeograph8a61', [], 'Display', None],
  [ 0x8a62, 'CjkUnifiedIdeograph8a62', [], 'Display', None],
  [ 0x8a63, 'CjkUnifiedIdeograph8a63', [], 'Display', None],
  [ 0x8a64, 'CjkUnifiedIdeograph8a64', [], 'Display', None],
  [ 0x8a65, 'CjkUnifiedIdeograph8a65', [], 'Display', None],
  [ 0x8a66, 'CjkUnifiedIdeograph8a66', [], 'Display', None],
  [ 0x8a67, 'CjkUnifiedIdeograph8a67', [], 'Display', None],
  [ 0x8a68, 'CjkUnifiedIdeograph8a68', [], 'Display', None],
  [ 0x8a69, 'CjkUnifiedIdeograph8a69', [], 'Display', None],
  [ 0x8a6a, 'CjkUnifiedIdeograph8a6a', [], 'Display', None],
  [ 0x8a6b, 'CjkUnifiedIdeograph8a6b', [], 'Display', None],
  [ 0x8a6c, 'CjkUnifiedIdeograph8a6c', [], 'Display', None],
  [ 0x8a6d, 'CjkUnifiedIdeograph8a6d', [], 'Display', None],
  [ 0x8a6e, 'CjkUnifiedIdeograph8a6e', [], 'Display', None],
  [ 0x8a6f, 'CjkUnifiedIdeograph8a6f', [], 'Display', None],
  [ 0x8a70, 'CjkUnifiedIdeograph8a70', [], 'Display', None],
  [ 0x8a71, 'CjkUnifiedIdeograph8a71', [], 'Display', None],
  [ 0x8a72, 'CjkUnifiedIdeograph8a72', [], 'Display', None],
  [ 0x8a73, 'CjkUnifiedIdeograph8a73', [], 'Display', None],
  [ 0x8a74, 'CjkUnifiedIdeograph8a74', [], 'Display', None],
  [ 0x8a75, 'CjkUnifiedIdeograph8a75', [], 'Display', None],
  [ 0x8a76, 'CjkUnifiedIdeograph8a76', [], 'Display', None],
  [ 0x8a77, 'CjkUnifiedIdeograph8a77', [], 'Display', None],
  [ 0x8a78, 'CjkUnifiedIdeograph8a78', [], 'Display', None],
  [ 0x8a79, 'CjkUnifiedIdeograph8a79', [], 'Display', None],
  [ 0x8a7a, 'CjkUnifiedIdeograph8a7a', [], 'Display', None],
  [ 0x8a7b, 'CjkUnifiedIdeograph8a7b', [], 'Display', None],
  [ 0x8a7c, 'CjkUnifiedIdeograph8a7c', [], 'Display', None],
  [ 0x8a7d, 'CjkUnifiedIdeograph8a7d', [], 'Display', None],
  [ 0x8a7e, 'CjkUnifiedIdeograph8a7e', [], 'Display', None],
  [ 0x8a7f, 'CjkUnifiedIdeograph8a7f', [], 'Display', None],
  [ 0x8a80, 'CjkUnifiedIdeograph8a80', [], 'Display', None],
  [ 0x8a81, 'CjkUnifiedIdeograph8a81', [], 'Display', None],
  [ 0x8a82, 'CjkUnifiedIdeograph8a82', [], 'Display', None],
  [ 0x8a83, 'CjkUnifiedIdeograph8a83', [], 'Display', None],
  [ 0x8a84, 'CjkUnifiedIdeograph8a84', [], 'Display', None],
  [ 0x8a85, 'CjkUnifiedIdeograph8a85', [], 'Display', None],
  [ 0x8a86, 'CjkUnifiedIdeograph8a86', [], 'Display', None],
  [ 0x8a87, 'CjkUnifiedIdeograph8a87', [], 'Display', None],
  [ 0x8a88, 'CjkUnifiedIdeograph8a88', [], 'Display', None],
  [ 0x8a89, 'CjkUnifiedIdeograph8a89', [], 'Display', None],
  [ 0x8a8a, 'CjkUnifiedIdeograph8a8a', [], 'Display', None],
  [ 0x8a8b, 'CjkUnifiedIdeograph8a8b', [], 'Display', None],
  [ 0x8a8c, 'CjkUnifiedIdeograph8a8c', [], 'Display', None],
  [ 0x8a8d, 'CjkUnifiedIdeograph8a8d', [], 'Display', None],
  [ 0x8a8e, 'CjkUnifiedIdeograph8a8e', [], 'Display', None],
  [ 0x8a8f, 'CjkUnifiedIdeograph8a8f', [], 'Display', None],
  [ 0x8a90, 'CjkUnifiedIdeograph8a90', [], 'Display', None],
  [ 0x8a91, 'CjkUnifiedIdeograph8a91', [], 'Display', None],
  [ 0x8a92, 'CjkUnifiedIdeograph8a92', [], 'Display', None],
  [ 0x8a93, 'CjkUnifiedIdeograph8a93', [], 'Display', None],
  [ 0x8a94, 'CjkUnifiedIdeograph8a94', [], 'Display', None],
  [ 0x8a95, 'CjkUnifiedIdeograph8a95', [], 'Display', None],
  [ 0x8a96, 'CjkUnifiedIdeograph8a96', [], 'Display', None],
  [ 0x8a97, 'CjkUnifiedIdeograph8a97', [], 'Display', None],
  [ 0x8a98, 'CjkUnifiedIdeograph8a98', [], 'Display', None],
  [ 0x8a99, 'CjkUnifiedIdeograph8a99', [], 'Display', None],
  [ 0x8a9a, 'CjkUnifiedIdeograph8a9a', [], 'Display', None],
  [ 0x8a9b, 'CjkUnifiedIdeograph8a9b', [], 'Display', None],
  [ 0x8a9c, 'CjkUnifiedIdeograph8a9c', [], 'Display', None],
  [ 0x8a9d, 'CjkUnifiedIdeograph8a9d', [], 'Display', None],
  [ 0x8a9e, 'CjkUnifiedIdeograph8a9e', [], 'Display', None],
  [ 0x8a9f, 'CjkUnifiedIdeograph8a9f', [], 'Display', None],
  [ 0x8aa0, 'CjkUnifiedIdeograph8aa0', [], 'Display', None],
  [ 0x8aa1, 'CjkUnifiedIdeograph8aa1', [], 'Display', None],
  [ 0x8aa2, 'CjkUnifiedIdeograph8aa2', [], 'Display', None],
  [ 0x8aa3, 'CjkUnifiedIdeograph8aa3', [], 'Display', None],
  [ 0x8aa4, 'CjkUnifiedIdeograph8aa4', [], 'Display', None],
  [ 0x8aa5, 'CjkUnifiedIdeograph8aa5', [], 'Display', None],
  [ 0x8aa6, 'CjkUnifiedIdeograph8aa6', [], 'Display', None],
  [ 0x8aa7, 'CjkUnifiedIdeograph8aa7', [], 'Display', None],
  [ 0x8aa8, 'CjkUnifiedIdeograph8aa8', [], 'Display', None],
  [ 0x8aa9, 'CjkUnifiedIdeograph8aa9', [], 'Display', None],
  [ 0x8aaa, 'CjkUnifiedIdeograph8aaa', [], 'Display', None],
  [ 0x8aab, 'CjkUnifiedIdeograph8aab', [], 'Display', None],
  [ 0x8aac, 'CjkUnifiedIdeograph8aac', [], 'Display', None],
  [ 0x8aad, 'CjkUnifiedIdeograph8aad', [], 'Display', None],
  [ 0x8aae, 'CjkUnifiedIdeograph8aae', [], 'Display', None],
  [ 0x8aaf, 'CjkUnifiedIdeograph8aaf', [], 'Display', None],
  [ 0x8ab0, 'CjkUnifiedIdeograph8ab0', [], 'Display', None],
  [ 0x8ab1, 'CjkUnifiedIdeograph8ab1', [], 'Display', None],
  [ 0x8ab2, 'CjkUnifiedIdeograph8ab2', [], 'Display', None],
  [ 0x8ab3, 'CjkUnifiedIdeograph8ab3', [], 'Display', None],
  [ 0x8ab4, 'CjkUnifiedIdeograph8ab4', [], 'Display', None],
  [ 0x8ab5, 'CjkUnifiedIdeograph8ab5', [], 'Display', None],
  [ 0x8ab6, 'CjkUnifiedIdeograph8ab6', [], 'Display', None],
  [ 0x8ab7, 'CjkUnifiedIdeograph8ab7', [], 'Display', None],
  [ 0x8ab8, 'CjkUnifiedIdeograph8ab8', [], 'Display', None],
  [ 0x8ab9, 'CjkUnifiedIdeograph8ab9', [], 'Display', None],
  [ 0x8aba, 'CjkUnifiedIdeograph8aba', [], 'Display', None],
  [ 0x8abb, 'CjkUnifiedIdeograph8abb', [], 'Display', None],
  [ 0x8abc, 'CjkUnifiedIdeograph8abc', [], 'Display', None],
  [ 0x8abd, 'CjkUnifiedIdeograph8abd', [], 'Display', None],
  [ 0x8abe, 'CjkUnifiedIdeograph8abe', [], 'Display', None],
  [ 0x8abf, 'CjkUnifiedIdeograph8abf', [], 'Display', None],
  [ 0x8ac0, 'CjkUnifiedIdeograph8ac0', [], 'Display', None],
  [ 0x8ac1, 'CjkUnifiedIdeograph8ac1', [], 'Display', None],
  [ 0x8ac2, 'CjkUnifiedIdeograph8ac2', [], 'Display', None],
  [ 0x8ac3, 'CjkUnifiedIdeograph8ac3', [], 'Display', None],
  [ 0x8ac4, 'CjkUnifiedIdeograph8ac4', [], 'Display', None],
  [ 0x8ac5, 'CjkUnifiedIdeograph8ac5', [], 'Display', None],
  [ 0x8ac6, 'CjkUnifiedIdeograph8ac6', [], 'Display', None],
  [ 0x8ac7, 'CjkUnifiedIdeograph8ac7', [], 'Display', None],
  [ 0x8ac8, 'CjkUnifiedIdeograph8ac8', [], 'Display', None],
  [ 0x8ac9, 'CjkUnifiedIdeograph8ac9', [], 'Display', None],
  [ 0x8aca, 'CjkUnifiedIdeograph8aca', [], 'Display', None],
  [ 0x8acb, 'CjkUnifiedIdeograph8acb', [], 'Display', None],
  [ 0x8acc, 'CjkUnifiedIdeograph8acc', [], 'Display', None],
  [ 0x8acd, 'CjkUnifiedIdeograph8acd', [], 'Display', None],
  [ 0x8ace, 'CjkUnifiedIdeograph8ace', [], 'Display', None],
  [ 0x8acf, 'CjkUnifiedIdeograph8acf', [], 'Display', None],
  [ 0x8ad0, 'CjkUnifiedIdeograph8ad0', [], 'Display', None],
  [ 0x8ad1, 'CjkUnifiedIdeograph8ad1', [], 'Display', None],
  [ 0x8ad2, 'CjkUnifiedIdeograph8ad2', [], 'Display', None],
  [ 0x8ad3, 'CjkUnifiedIdeograph8ad3', [], 'Display', None],
  [ 0x8ad4, 'CjkUnifiedIdeograph8ad4', [], 'Display', None],
  [ 0x8ad5, 'CjkUnifiedIdeograph8ad5', [], 'Display', None],
  [ 0x8ad6, 'CjkUnifiedIdeograph8ad6', [], 'Display', None],
  [ 0x8ad7, 'CjkUnifiedIdeograph8ad7', [], 'Display', None],
  [ 0x8ad8, 'CjkUnifiedIdeograph8ad8', [], 'Display', None],
  [ 0x8ad9, 'CjkUnifiedIdeograph8ad9', [], 'Display', None],
  [ 0x8ada, 'CjkUnifiedIdeograph8ada', [], 'Display', None],
  [ 0x8adb, 'CjkUnifiedIdeograph8adb', [], 'Display', None],
  [ 0x8adc, 'CjkUnifiedIdeograph8adc', [], 'Display', None],
  [ 0x8add, 'CjkUnifiedIdeograph8add', [], 'Display', None],
  [ 0x8ade, 'CjkUnifiedIdeograph8ade', [], 'Display', None],
  [ 0x8adf, 'CjkUnifiedIdeograph8adf', [], 'Display', None],
  [ 0x8ae0, 'CjkUnifiedIdeograph8ae0', [], 'Display', None],
  [ 0x8ae1, 'CjkUnifiedIdeograph8ae1', [], 'Display', None],
  [ 0x8ae2, 'CjkUnifiedIdeograph8ae2', [], 'Display', None],
  [ 0x8ae3, 'CjkUnifiedIdeograph8ae3', [], 'Display', None],
  [ 0x8ae4, 'CjkUnifiedIdeograph8ae4', [], 'Display', None],
  [ 0x8ae5, 'CjkUnifiedIdeograph8ae5', [], 'Display', None],
  [ 0x8ae6, 'CjkUnifiedIdeograph8ae6', [], 'Display', None],
  [ 0x8ae7, 'CjkUnifiedIdeograph8ae7', [], 'Display', None],
  [ 0x8ae8, 'CjkUnifiedIdeograph8ae8', [], 'Display', None],
  [ 0x8ae9, 'CjkUnifiedIdeograph8ae9', [], 'Display', None],
  [ 0x8aea, 'CjkUnifiedIdeograph8aea', [], 'Display', None],
  [ 0x8aeb, 'CjkUnifiedIdeograph8aeb', [], 'Display', None],
  [ 0x8aec, 'CjkUnifiedIdeograph8aec', [], 'Display', None],
  [ 0x8aed, 'CjkUnifiedIdeograph8aed', [], 'Display', None],
  [ 0x8aee, 'CjkUnifiedIdeograph8aee', [], 'Display', None],
  [ 0x8aef, 'CjkUnifiedIdeograph8aef', [], 'Display', None],
  [ 0x8af0, 'CjkUnifiedIdeograph8af0', [], 'Display', None],
  [ 0x8af1, 'CjkUnifiedIdeograph8af1', [], 'Display', None],
  [ 0x8af2, 'CjkUnifiedIdeograph8af2', [], 'Display', None],
  [ 0x8af3, 'CjkUnifiedIdeograph8af3', [], 'Display', None],
  [ 0x8af4, 'CjkUnifiedIdeograph8af4', [], 'Display', None],
  [ 0x8af5, 'CjkUnifiedIdeograph8af5', [], 'Display', None],
  [ 0x8af6, 'CjkUnifiedIdeograph8af6', [], 'Display', None],
  [ 0x8af7, 'CjkUnifiedIdeograph8af7', [], 'Display', None],
  [ 0x8af8, 'CjkUnifiedIdeograph8af8', [], 'Display', None],
  [ 0x8af9, 'CjkUnifiedIdeograph8af9', [], 'Display', None],
  [ 0x8afa, 'CjkUnifiedIdeograph8afa', [], 'Display', None],
  [ 0x8afb, 'CjkUnifiedIdeograph8afb', [], 'Display', None],
  [ 0x8afc, 'CjkUnifiedIdeograph8afc', [], 'Display', None],
  [ 0x8afd, 'CjkUnifiedIdeograph8afd', [], 'Display', None],
  [ 0x8afe, 'CjkUnifiedIdeograph8afe', [], 'Display', None],
  [ 0x8aff, 'CjkUnifiedIdeograph8aff', [], 'Display', None],
  [ 0x8b00, 'CjkUnifiedIdeograph8b00', [], 'Display', None],
  [ 0x8b01, 'CjkUnifiedIdeograph8b01', [], 'Display', None],
  [ 0x8b02, 'CjkUnifiedIdeograph8b02', [], 'Display', None],
  [ 0x8b03, 'CjkUnifiedIdeograph8b03', [], 'Display', None],
  [ 0x8b04, 'CjkUnifiedIdeograph8b04', [], 'Display', None],
  [ 0x8b05, 'CjkUnifiedIdeograph8b05', [], 'Display', None],
  [ 0x8b06, 'CjkUnifiedIdeograph8b06', [], 'Display', None],
  [ 0x8b07, 'CjkUnifiedIdeograph8b07', [], 'Display', None],
  [ 0x8b08, 'CjkUnifiedIdeograph8b08', [], 'Display', None],
  [ 0x8b09, 'CjkUnifiedIdeograph8b09', [], 'Display', None],
  [ 0x8b0a, 'CjkUnifiedIdeograph8b0a', [], 'Display', None],
  [ 0x8b0b, 'CjkUnifiedIdeograph8b0b', [], 'Display', None],
  [ 0x8b0c, 'CjkUnifiedIdeograph8b0c', [], 'Display', None],
  [ 0x8b0d, 'CjkUnifiedIdeograph8b0d', [], 'Display', None],
  [ 0x8b0e, 'CjkUnifiedIdeograph8b0e', [], 'Display', None],
  [ 0x8b0f, 'CjkUnifiedIdeograph8b0f', [], 'Display', None],
  [ 0x8b10, 'CjkUnifiedIdeograph8b10', [], 'Display', None],
  [ 0x8b11, 'CjkUnifiedIdeograph8b11', [], 'Display', None],
  [ 0x8b12, 'CjkUnifiedIdeograph8b12', [], 'Display', None],
  [ 0x8b13, 'CjkUnifiedIdeograph8b13', [], 'Display', None],
  [ 0x8b14, 'CjkUnifiedIdeograph8b14', [], 'Display', None],
  [ 0x8b15, 'CjkUnifiedIdeograph8b15', [], 'Display', None],
  [ 0x8b16, 'CjkUnifiedIdeograph8b16', [], 'Display', None],
  [ 0x8b17, 'CjkUnifiedIdeograph8b17', [], 'Display', None],
  [ 0x8b18, 'CjkUnifiedIdeograph8b18', [], 'Display', None],
  [ 0x8b19, 'CjkUnifiedIdeograph8b19', [], 'Display', None],
  [ 0x8b1a, 'CjkUnifiedIdeograph8b1a', [], 'Display', None],
  [ 0x8b1b, 'CjkUnifiedIdeograph8b1b', [], 'Display', None],
  [ 0x8b1c, 'CjkUnifiedIdeograph8b1c', [], 'Display', None],
  [ 0x8b1d, 'CjkUnifiedIdeograph8b1d', [], 'Display', None],
  [ 0x8b1e, 'CjkUnifiedIdeograph8b1e', [], 'Display', None],
  [ 0x8b1f, 'CjkUnifiedIdeograph8b1f', [], 'Display', None],
  [ 0x8b20, 'CjkUnifiedIdeograph8b20', [], 'Display', None],
  [ 0x8b21, 'CjkUnifiedIdeograph8b21', [], 'Display', None],
  [ 0x8b22, 'CjkUnifiedIdeograph8b22', [], 'Display', None],
  [ 0x8b23, 'CjkUnifiedIdeograph8b23', [], 'Display', None],
  [ 0x8b24, 'CjkUnifiedIdeograph8b24', [], 'Display', None],
  [ 0x8b25, 'CjkUnifiedIdeograph8b25', [], 'Display', None],
  [ 0x8b26, 'CjkUnifiedIdeograph8b26', [], 'Display', None],
  [ 0x8b27, 'CjkUnifiedIdeograph8b27', [], 'Display', None],
  [ 0x8b28, 'CjkUnifiedIdeograph8b28', [], 'Display', None],
  [ 0x8b29, 'CjkUnifiedIdeograph8b29', [], 'Display', None],
  [ 0x8b2a, 'CjkUnifiedIdeograph8b2a', [], 'Display', None],
  [ 0x8b2b, 'CjkUnifiedIdeograph8b2b', [], 'Display', None],
  [ 0x8b2c, 'CjkUnifiedIdeograph8b2c', [], 'Display', None],
  [ 0x8b2d, 'CjkUnifiedIdeograph8b2d', [], 'Display', None],
  [ 0x8b2e, 'CjkUnifiedIdeograph8b2e', [], 'Display', None],
  [ 0x8b2f, 'CjkUnifiedIdeograph8b2f', [], 'Display', None],
  [ 0x8b30, 'CjkUnifiedIdeograph8b30', [], 'Display', None],
  [ 0x8b31, 'CjkUnifiedIdeograph8b31', [], 'Display', None],
  [ 0x8b32, 'CjkUnifiedIdeograph8b32', [], 'Display', None],
  [ 0x8b33, 'CjkUnifiedIdeograph8b33', [], 'Display', None],
  [ 0x8b34, 'CjkUnifiedIdeograph8b34', [], 'Display', None],
  [ 0x8b35, 'CjkUnifiedIdeograph8b35', [], 'Display', None],
  [ 0x8b36, 'CjkUnifiedIdeograph8b36', [], 'Display', None],
  [ 0x8b37, 'CjkUnifiedIdeograph8b37', [], 'Display', None],
  [ 0x8b38, 'CjkUnifiedIdeograph8b38', [], 'Display', None],
  [ 0x8b39, 'CjkUnifiedIdeograph8b39', [], 'Display', None],
  [ 0x8b3a, 'CjkUnifiedIdeograph8b3a', [], 'Display', None],
  [ 0x8b3b, 'CjkUnifiedIdeograph8b3b', [], 'Display', None],
  [ 0x8b3c, 'CjkUnifiedIdeograph8b3c', [], 'Display', None],
  [ 0x8b3d, 'CjkUnifiedIdeograph8b3d', [], 'Display', None],
  [ 0x8b3e, 'CjkUnifiedIdeograph8b3e', [], 'Display', None],
  [ 0x8b3f, 'CjkUnifiedIdeograph8b3f', [], 'Display', None],
  [ 0x8b40, 'CjkUnifiedIdeograph8b40', [], 'Display', None],
  [ 0x8b41, 'CjkUnifiedIdeograph8b41', [], 'Display', None],
  [ 0x8b42, 'CjkUnifiedIdeograph8b42', [], 'Display', None],
  [ 0x8b43, 'CjkUnifiedIdeograph8b43', [], 'Display', None],
  [ 0x8b44, 'CjkUnifiedIdeograph8b44', [], 'Display', None],
  [ 0x8b45, 'CjkUnifiedIdeograph8b45', [], 'Display', None],
  [ 0x8b46, 'CjkUnifiedIdeograph8b46', [], 'Display', None],
  [ 0x8b47, 'CjkUnifiedIdeograph8b47', [], 'Display', None],
  [ 0x8b48, 'CjkUnifiedIdeograph8b48', [], 'Display', None],
  [ 0x8b49, 'CjkUnifiedIdeograph8b49', [], 'Display', None],
  [ 0x8b4a, 'CjkUnifiedIdeograph8b4a', [], 'Display', None],
  [ 0x8b4b, 'CjkUnifiedIdeograph8b4b', [], 'Display', None],
  [ 0x8b4c, 'CjkUnifiedIdeograph8b4c', [], 'Display', None],
  [ 0x8b4d, 'CjkUnifiedIdeograph8b4d', [], 'Display', None],
  [ 0x8b4e, 'CjkUnifiedIdeograph8b4e', [], 'Display', None],
  [ 0x8b4f, 'CjkUnifiedIdeograph8b4f', [], 'Display', None],
  [ 0x8b50, 'CjkUnifiedIdeograph8b50', [], 'Display', None],
  [ 0x8b51, 'CjkUnifiedIdeograph8b51', [], 'Display', None],
  [ 0x8b52, 'CjkUnifiedIdeograph8b52', [], 'Display', None],
  [ 0x8b53, 'CjkUnifiedIdeograph8b53', [], 'Display', None],
  [ 0x8b54, 'CjkUnifiedIdeograph8b54', [], 'Display', None],
  [ 0x8b55, 'CjkUnifiedIdeograph8b55', [], 'Display', None],
  [ 0x8b56, 'CjkUnifiedIdeograph8b56', [], 'Display', None],
  [ 0x8b57, 'CjkUnifiedIdeograph8b57', [], 'Display', None],
  [ 0x8b58, 'CjkUnifiedIdeograph8b58', [], 'Display', None],
  [ 0x8b59, 'CjkUnifiedIdeograph8b59', [], 'Display', None],
  [ 0x8b5a, 'CjkUnifiedIdeograph8b5a', [], 'Display', None],
  [ 0x8b5b, 'CjkUnifiedIdeograph8b5b', [], 'Display', None],
  [ 0x8b5c, 'CjkUnifiedIdeograph8b5c', [], 'Display', None],
  [ 0x8b5d, 'CjkUnifiedIdeograph8b5d', [], 'Display', None],
  [ 0x8b5e, 'CjkUnifiedIdeograph8b5e', [], 'Display', None],
  [ 0x8b5f, 'CjkUnifiedIdeograph8b5f', [], 'Display', None],
  [ 0x8b60, 'CjkUnifiedIdeograph8b60', [], 'Display', None],
  [ 0x8b61, 'CjkUnifiedIdeograph8b61', [], 'Display', None],
  [ 0x8b62, 'CjkUnifiedIdeograph8b62', [], 'Display', None],
  [ 0x8b63, 'CjkUnifiedIdeograph8b63', [], 'Display', None],
  [ 0x8b64, 'CjkUnifiedIdeograph8b64', [], 'Display', None],
  [ 0x8b65, 'CjkUnifiedIdeograph8b65', [], 'Display', None],
  [ 0x8b66, 'CjkUnifiedIdeograph8b66', [], 'Display', None],
  [ 0x8b67, 'CjkUnifiedIdeograph8b67', [], 'Display', None],
  [ 0x8b68, 'CjkUnifiedIdeograph8b68', [], 'Display', None],
  [ 0x8b69, 'CjkUnifiedIdeograph8b69', [], 'Display', None],
  [ 0x8b6a, 'CjkUnifiedIdeograph8b6a', [], 'Display', None],
  [ 0x8b6b, 'CjkUnifiedIdeograph8b6b', [], 'Display', None],
  [ 0x8b6c, 'CjkUnifiedIdeograph8b6c', [], 'Display', None],
  [ 0x8b6d, 'CjkUnifiedIdeograph8b6d', [], 'Display', None],
  [ 0x8b6e, 'CjkUnifiedIdeograph8b6e', [], 'Display', None],
  [ 0x8b6f, 'CjkUnifiedIdeograph8b6f', [], 'Display', None],
  [ 0x8b70, 'CjkUnifiedIdeograph8b70', [], 'Display', None],
  [ 0x8b71, 'CjkUnifiedIdeograph8b71', [], 'Display', None],
  [ 0x8b72, 'CjkUnifiedIdeograph8b72', [], 'Display', None],
  [ 0x8b73, 'CjkUnifiedIdeograph8b73', [], 'Display', None],
  [ 0x8b74, 'CjkUnifiedIdeograph8b74', [], 'Display', None],
  [ 0x8b75, 'CjkUnifiedIdeograph8b75', [], 'Display', None],
  [ 0x8b76, 'CjkUnifiedIdeograph8b76', [], 'Display', None],
  [ 0x8b77, 'CjkUnifiedIdeograph8b77', [], 'Display', None],
  [ 0x8b78, 'CjkUnifiedIdeograph8b78', [], 'Display', None],
  [ 0x8b79, 'CjkUnifiedIdeograph8b79', [], 'Display', None],
  [ 0x8b7a, 'CjkUnifiedIdeograph8b7a', [], 'Display', None],
  [ 0x8b7b, 'CjkUnifiedIdeograph8b7b', [], 'Display', None],
  [ 0x8b7c, 'CjkUnifiedIdeograph8b7c', [], 'Display', None],
  [ 0x8b7d, 'CjkUnifiedIdeograph8b7d', [], 'Display', None],
  [ 0x8b7e, 'CjkUnifiedIdeograph8b7e', [], 'Display', None],
  [ 0x8b7f, 'CjkUnifiedIdeograph8b7f', [], 'Display', None],
  [ 0x8b80, 'CjkUnifiedIdeograph8b80', [], 'Display', None],
  [ 0x8b81, 'CjkUnifiedIdeograph8b81', [], 'Display', None],
  [ 0x8b82, 'CjkUnifiedIdeograph8b82', [], 'Display', None],
  [ 0x8b83, 'CjkUnifiedIdeograph8b83', [], 'Display', None],
  [ 0x8b84, 'CjkUnifiedIdeograph8b84', [], 'Display', None],
  [ 0x8b85, 'CjkUnifiedIdeograph8b85', [], 'Display', None],
  [ 0x8b86, 'CjkUnifiedIdeograph8b86', [], 'Display', None],
  [ 0x8b87, 'CjkUnifiedIdeograph8b87', [], 'Display', None],
  [ 0x8b88, 'CjkUnifiedIdeograph8b88', [], 'Display', None],
  [ 0x8b89, 'CjkUnifiedIdeograph8b89', [], 'Display', None],
  [ 0x8b8a, 'CjkUnifiedIdeograph8b8a', [], 'Display', None],
  [ 0x8b8b, 'CjkUnifiedIdeograph8b8b', [], 'Display', None],
  [ 0x8b8c, 'CjkUnifiedIdeograph8b8c', [], 'Display', None],
  [ 0x8b8d, 'CjkUnifiedIdeograph8b8d', [], 'Display', None],
  [ 0x8b8e, 'CjkUnifiedIdeograph8b8e', [], 'Display', None],
  [ 0x8b8f, 'CjkUnifiedIdeograph8b8f', [], 'Display', None],
  [ 0x8b90, 'CjkUnifiedIdeograph8b90', [], 'Display', None],
  [ 0x8b91, 'CjkUnifiedIdeograph8b91', [], 'Display', None],
  [ 0x8b92, 'CjkUnifiedIdeograph8b92', [], 'Display', None],
  [ 0x8b93, 'CjkUnifiedIdeograph8b93', [], 'Display', None],
  [ 0x8b94, 'CjkUnifiedIdeograph8b94', [], 'Display', None],
  [ 0x8b95, 'CjkUnifiedIdeograph8b95', [], 'Display', None],
  [ 0x8b96, 'CjkUnifiedIdeograph8b96', [], 'Display', None],
  [ 0x8b97, 'CjkUnifiedIdeograph8b97', [], 'Display', None],
  [ 0x8b98, 'CjkUnifiedIdeograph8b98', [], 'Display', None],
  [ 0x8b99, 'CjkUnifiedIdeograph8b99', [], 'Display', None],
  [ 0x8b9a, 'CjkUnifiedIdeograph8b9a', [], 'Display', None],
  [ 0x8b9b, 'CjkUnifiedIdeograph8b9b', [], 'Display', None],
  [ 0x8b9c, 'CjkUnifiedIdeograph8b9c', [], 'Display', None],
  [ 0x8b9d, 'CjkUnifiedIdeograph8b9d', [], 'Display', None],
  [ 0x8b9e, 'CjkUnifiedIdeograph8b9e', [], 'Display', None],
  [ 0x8b9f, 'CjkUnifiedIdeograph8b9f', [], 'Display', None],
  [ 0x8ba0, 'CjkUnifiedIdeograph8ba0', [], 'Display', None],
  [ 0x8ba1, 'CjkUnifiedIdeograph8ba1', [], 'Display', None],
  [ 0x8ba2, 'CjkUnifiedIdeograph8ba2', [], 'Display', None],
  [ 0x8ba3, 'CjkUnifiedIdeograph8ba3', [], 'Display', None],
  [ 0x8ba4, 'CjkUnifiedIdeograph8ba4', [], 'Display', None],
  [ 0x8ba5, 'CjkUnifiedIdeograph8ba5', [], 'Display', None],
  [ 0x8ba6, 'CjkUnifiedIdeograph8ba6', [], 'Display', None],
  [ 0x8ba7, 'CjkUnifiedIdeograph8ba7', [], 'Display', None],
  [ 0x8ba8, 'CjkUnifiedIdeograph8ba8', [], 'Display', None],
  [ 0x8ba9, 'CjkUnifiedIdeograph8ba9', [], 'Display', None],
  [ 0x8baa, 'CjkUnifiedIdeograph8baa', [], 'Display', None],
  [ 0x8bab, 'CjkUnifiedIdeograph8bab', [], 'Display', None],
  [ 0x8bac, 'CjkUnifiedIdeograph8bac', [], 'Display', None],
  [ 0x8bad, 'CjkUnifiedIdeograph8bad', [], 'Display', None],
  [ 0x8bae, 'CjkUnifiedIdeograph8bae', [], 'Display', None],
  [ 0x8baf, 'CjkUnifiedIdeograph8baf', [], 'Display', None],
  [ 0x8bb0, 'CjkUnifiedIdeograph8bb0', [], 'Display', None],
  [ 0x8bb1, 'CjkUnifiedIdeograph8bb1', [], 'Display', None],
  [ 0x8bb2, 'CjkUnifiedIdeograph8bb2', [], 'Display', None],
  [ 0x8bb3, 'CjkUnifiedIdeograph8bb3', [], 'Display', None],
  [ 0x8bb4, 'CjkUnifiedIdeograph8bb4', [], 'Display', None],
  [ 0x8bb5, 'CjkUnifiedIdeograph8bb5', [], 'Display', None],
  [ 0x8bb6, 'CjkUnifiedIdeograph8bb6', [], 'Display', None],
  [ 0x8bb7, 'CjkUnifiedIdeograph8bb7', [], 'Display', None],
  [ 0x8bb8, 'CjkUnifiedIdeograph8bb8', [], 'Display', None],
  [ 0x8bb9, 'CjkUnifiedIdeograph8bb9', [], 'Display', None],
  [ 0x8bba, 'CjkUnifiedIdeograph8bba', [], 'Display', None],
  [ 0x8bbb, 'CjkUnifiedIdeograph8bbb', [], 'Display', None],
  [ 0x8bbc, 'CjkUnifiedIdeograph8bbc', [], 'Display', None],
  [ 0x8bbd, 'CjkUnifiedIdeograph8bbd', [], 'Display', None],
  [ 0x8bbe, 'CjkUnifiedIdeograph8bbe', [], 'Display', None],
  [ 0x8bbf, 'CjkUnifiedIdeograph8bbf', [], 'Display', None],
  [ 0x8bc0, 'CjkUnifiedIdeograph8bc0', [], 'Display', None],
  [ 0x8bc1, 'CjkUnifiedIdeograph8bc1', [], 'Display', None],
  [ 0x8bc2, 'CjkUnifiedIdeograph8bc2', [], 'Display', None],
  [ 0x8bc3, 'CjkUnifiedIdeograph8bc3', [], 'Display', None],
  [ 0x8bc4, 'CjkUnifiedIdeograph8bc4', [], 'Display', None],
  [ 0x8bc5, 'CjkUnifiedIdeograph8bc5', [], 'Display', None],
  [ 0x8bc6, 'CjkUnifiedIdeograph8bc6', [], 'Display', None],
  [ 0x8bc7, 'CjkUnifiedIdeograph8bc7', [], 'Display', None],
  [ 0x8bc8, 'CjkUnifiedIdeograph8bc8', [], 'Display', None],
  [ 0x8bc9, 'CjkUnifiedIdeograph8bc9', [], 'Display', None],
  [ 0x8bca, 'CjkUnifiedIdeograph8bca', [], 'Display', None],
  [ 0x8bcb, 'CjkUnifiedIdeograph8bcb', [], 'Display', None],
  [ 0x8bcc, 'CjkUnifiedIdeograph8bcc', [], 'Display', None],
  [ 0x8bcd, 'CjkUnifiedIdeograph8bcd', [], 'Display', None],
  [ 0x8bce, 'CjkUnifiedIdeograph8bce', [], 'Display', None],
  [ 0x8bcf, 'CjkUnifiedIdeograph8bcf', [], 'Display', None],
  [ 0x8bd0, 'CjkUnifiedIdeograph8bd0', [], 'Display', None],
  [ 0x8bd1, 'CjkUnifiedIdeograph8bd1', [], 'Display', None],
  [ 0x8bd2, 'CjkUnifiedIdeograph8bd2', [], 'Display', None],
  [ 0x8bd3, 'CjkUnifiedIdeograph8bd3', [], 'Display', None],
  [ 0x8bd4, 'CjkUnifiedIdeograph8bd4', [], 'Display', None],
  [ 0x8bd5, 'CjkUnifiedIdeograph8bd5', [], 'Display', None],
  [ 0x8bd6, 'CjkUnifiedIdeograph8bd6', [], 'Display', None],
  [ 0x8bd7, 'CjkUnifiedIdeograph8bd7', [], 'Display', None],
  [ 0x8bd8, 'CjkUnifiedIdeograph8bd8', [], 'Display', None],
  [ 0x8bd9, 'CjkUnifiedIdeograph8bd9', [], 'Display', None],
  [ 0x8bda, 'CjkUnifiedIdeograph8bda', [], 'Display', None],
  [ 0x8bdb, 'CjkUnifiedIdeograph8bdb', [], 'Display', None],
  [ 0x8bdc, 'CjkUnifiedIdeograph8bdc', [], 'Display', None],
  [ 0x8bdd, 'CjkUnifiedIdeograph8bdd', [], 'Display', None],
  [ 0x8bde, 'CjkUnifiedIdeograph8bde', [], 'Display', None],
  [ 0x8bdf, 'CjkUnifiedIdeograph8bdf', [], 'Display', None],
  [ 0x8be0, 'CjkUnifiedIdeograph8be0', [], 'Display', None],
  [ 0x8be1, 'CjkUnifiedIdeograph8be1', [], 'Display', None],
  [ 0x8be2, 'CjkUnifiedIdeograph8be2', [], 'Display', None],
  [ 0x8be3, 'CjkUnifiedIdeograph8be3', [], 'Display', None],
  [ 0x8be4, 'CjkUnifiedIdeograph8be4', [], 'Display', None],
  [ 0x8be5, 'CjkUnifiedIdeograph8be5', [], 'Display', None],
  [ 0x8be6, 'CjkUnifiedIdeograph8be6', [], 'Display', None],
  [ 0x8be7, 'CjkUnifiedIdeograph8be7', [], 'Display', None],
  [ 0x8be8, 'CjkUnifiedIdeograph8be8', [], 'Display', None],
  [ 0x8be9, 'CjkUnifiedIdeograph8be9', [], 'Display', None],
  [ 0x8bea, 'CjkUnifiedIdeograph8bea', [], 'Display', None],
  [ 0x8beb, 'CjkUnifiedIdeograph8beb', [], 'Display', None],
  [ 0x8bec, 'CjkUnifiedIdeograph8bec', [], 'Display', None],
  [ 0x8bed, 'CjkUnifiedIdeograph8bed', [], 'Display', None],
  [ 0x8bee, 'CjkUnifiedIdeograph8bee', [], 'Display', None],
  [ 0x8bef, 'CjkUnifiedIdeograph8bef', [], 'Display', None],
  [ 0x8bf0, 'CjkUnifiedIdeograph8bf0', [], 'Display', None],
  [ 0x8bf1, 'CjkUnifiedIdeograph8bf1', [], 'Display', None],
  [ 0x8bf2, 'CjkUnifiedIdeograph8bf2', [], 'Display', None],
  [ 0x8bf3, 'CjkUnifiedIdeograph8bf3', [], 'Display', None],
  [ 0x8bf4, 'CjkUnifiedIdeograph8bf4', [], 'Display', None],
  [ 0x8bf5, 'CjkUnifiedIdeograph8bf5', [], 'Display', None],
  [ 0x8bf6, 'CjkUnifiedIdeograph8bf6', [], 'Display', None],
  [ 0x8bf7, 'CjkUnifiedIdeograph8bf7', [], 'Display', None],
  [ 0x8bf8, 'CjkUnifiedIdeograph8bf8', [], 'Display', None],
  [ 0x8bf9, 'CjkUnifiedIdeograph8bf9', [], 'Display', None],
  [ 0x8bfa, 'CjkUnifiedIdeograph8bfa', [], 'Display', None],
  [ 0x8bfb, 'CjkUnifiedIdeograph8bfb', [], 'Display', None],
  [ 0x8bfc, 'CjkUnifiedIdeograph8bfc', [], 'Display', None],
  [ 0x8bfd, 'CjkUnifiedIdeograph8bfd', [], 'Display', None],
  [ 0x8bfe, 'CjkUnifiedIdeograph8bfe', [], 'Display', None],
  [ 0x8bff, 'CjkUnifiedIdeograph8bff', [], 'Display', None],
  [ 0x8c00, 'CjkUnifiedIdeograph8c00', [], 'Display', None],
  [ 0x8c01, 'CjkUnifiedIdeograph8c01', [], 'Display', None],
  [ 0x8c02, 'CjkUnifiedIdeograph8c02', [], 'Display', None],
  [ 0x8c03, 'CjkUnifiedIdeograph8c03', [], 'Display', None],
  [ 0x8c04, 'CjkUnifiedIdeograph8c04', [], 'Display', None],
  [ 0x8c05, 'CjkUnifiedIdeograph8c05', [], 'Display', None],
  [ 0x8c06, 'CjkUnifiedIdeograph8c06', [], 'Display', None],
  [ 0x8c07, 'CjkUnifiedIdeograph8c07', [], 'Display', None],
  [ 0x8c08, 'CjkUnifiedIdeograph8c08', [], 'Display', None],
  [ 0x8c09, 'CjkUnifiedIdeograph8c09', [], 'Display', None],
  [ 0x8c0a, 'CjkUnifiedIdeograph8c0a', [], 'Display', None],
  [ 0x8c0b, 'CjkUnifiedIdeograph8c0b', [], 'Display', None],
  [ 0x8c0c, 'CjkUnifiedIdeograph8c0c', [], 'Display', None],
  [ 0x8c0d, 'CjkUnifiedIdeograph8c0d', [], 'Display', None],
  [ 0x8c0e, 'CjkUnifiedIdeograph8c0e', [], 'Display', None],
  [ 0x8c0f, 'CjkUnifiedIdeograph8c0f', [], 'Display', None],
  [ 0x8c10, 'CjkUnifiedIdeograph8c10', [], 'Display', None],
  [ 0x8c11, 'CjkUnifiedIdeograph8c11', [], 'Display', None],
  [ 0x8c12, 'CjkUnifiedIdeograph8c12', [], 'Display', None],
  [ 0x8c13, 'CjkUnifiedIdeograph8c13', [], 'Display', None],
  [ 0x8c14, 'CjkUnifiedIdeograph8c14', [], 'Display', None],
  [ 0x8c15, 'CjkUnifiedIdeograph8c15', [], 'Display', None],
  [ 0x8c16, 'CjkUnifiedIdeograph8c16', [], 'Display', None],
  [ 0x8c17, 'CjkUnifiedIdeograph8c17', [], 'Display', None],
  [ 0x8c18, 'CjkUnifiedIdeograph8c18', [], 'Display', None],
  [ 0x8c19, 'CjkUnifiedIdeograph8c19', [], 'Display', None],
  [ 0x8c1a, 'CjkUnifiedIdeograph8c1a', [], 'Display', None],
  [ 0x8c1b, 'CjkUnifiedIdeograph8c1b', [], 'Display', None],
  [ 0x8c1c, 'CjkUnifiedIdeograph8c1c', [], 'Display', None],
  [ 0x8c1d, 'CjkUnifiedIdeograph8c1d', [], 'Display', None],
  [ 0x8c1e, 'CjkUnifiedIdeograph8c1e', [], 'Display', None],
  [ 0x8c1f, 'CjkUnifiedIdeograph8c1f', [], 'Display', None],
  [ 0x8c20, 'CjkUnifiedIdeograph8c20', [], 'Display', None],
  [ 0x8c21, 'CjkUnifiedIdeograph8c21', [], 'Display', None],
  [ 0x8c22, 'CjkUnifiedIdeograph8c22', [], 'Display', None],
  [ 0x8c23, 'CjkUnifiedIdeograph8c23', [], 'Display', None],
  [ 0x8c24, 'CjkUnifiedIdeograph8c24', [], 'Display', None],
  [ 0x8c25, 'CjkUnifiedIdeograph8c25', [], 'Display', None],
  [ 0x8c26, 'CjkUnifiedIdeograph8c26', [], 'Display', None],
  [ 0x8c27, 'CjkUnifiedIdeograph8c27', [], 'Display', None],
  [ 0x8c28, 'CjkUnifiedIdeograph8c28', [], 'Display', None],
  [ 0x8c29, 'CjkUnifiedIdeograph8c29', [], 'Display', None],
  [ 0x8c2a, 'CjkUnifiedIdeograph8c2a', [], 'Display', None],
  [ 0x8c2b, 'CjkUnifiedIdeograph8c2b', [], 'Display', None],
  [ 0x8c2c, 'CjkUnifiedIdeograph8c2c', [], 'Display', None],
  [ 0x8c2d, 'CjkUnifiedIdeograph8c2d', [], 'Display', None],
  [ 0x8c2e, 'CjkUnifiedIdeograph8c2e', [], 'Display', None],
  [ 0x8c2f, 'CjkUnifiedIdeograph8c2f', [], 'Display', None],
  [ 0x8c30, 'CjkUnifiedIdeograph8c30', [], 'Display', None],
  [ 0x8c31, 'CjkUnifiedIdeograph8c31', [], 'Display', None],
  [ 0x8c32, 'CjkUnifiedIdeograph8c32', [], 'Display', None],
  [ 0x8c33, 'CjkUnifiedIdeograph8c33', [], 'Display', None],
  [ 0x8c34, 'CjkUnifiedIdeograph8c34', [], 'Display', None],
  [ 0x8c35, 'CjkUnifiedIdeograph8c35', [], 'Display', None],
  [ 0x8c36, 'CjkUnifiedIdeograph8c36', [], 'Display', None],
  [ 0x8c37, 'CjkUnifiedIdeograph8c37', [], 'Display', None],
  [ 0x8c38, 'CjkUnifiedIdeograph8c38', [], 'Display', None],
  [ 0x8c39, 'CjkUnifiedIdeograph8c39', [], 'Display', None],
  [ 0x8c3a, 'CjkUnifiedIdeograph8c3a', [], 'Display', None],
  [ 0x8c3b, 'CjkUnifiedIdeograph8c3b', [], 'Display', None],
  [ 0x8c3c, 'CjkUnifiedIdeograph8c3c', [], 'Display', None],
  [ 0x8c3d, 'CjkUnifiedIdeograph8c3d', [], 'Display', None],
  [ 0x8c3e, 'CjkUnifiedIdeograph8c3e', [], 'Display', None],
  [ 0x8c3f, 'CjkUnifiedIdeograph8c3f', [], 'Display', None],
  [ 0x8c40, 'CjkUnifiedIdeograph8c40', [], 'Display', None],
  [ 0x8c41, 'CjkUnifiedIdeograph8c41', [], 'Display', None],
  [ 0x8c42, 'CjkUnifiedIdeograph8c42', [], 'Display', None],
  [ 0x8c43, 'CjkUnifiedIdeograph8c43', [], 'Display', None],
  [ 0x8c44, 'CjkUnifiedIdeograph8c44', [], 'Display', None],
  [ 0x8c45, 'CjkUnifiedIdeograph8c45', [], 'Display', None],
  [ 0x8c46, 'CjkUnifiedIdeograph8c46', [], 'Display', None],
  [ 0x8c47, 'CjkUnifiedIdeograph8c47', [], 'Display', None],
  [ 0x8c48, 'CjkUnifiedIdeograph8c48', [], 'Display', None],
  [ 0x8c49, 'CjkUnifiedIdeograph8c49', [], 'Display', None],
  [ 0x8c4a, 'CjkUnifiedIdeograph8c4a', [], 'Display', None],
  [ 0x8c4b, 'CjkUnifiedIdeograph8c4b', [], 'Display', None],
  [ 0x8c4c, 'CjkUnifiedIdeograph8c4c', [], 'Display', None],
  [ 0x8c4d, 'CjkUnifiedIdeograph8c4d', [], 'Display', None],
  [ 0x8c4e, 'CjkUnifiedIdeograph8c4e', [], 'Display', None],
  [ 0x8c4f, 'CjkUnifiedIdeograph8c4f', [], 'Display', None],
  [ 0x8c50, 'CjkUnifiedIdeograph8c50', [], 'Display', None],
  [ 0x8c51, 'CjkUnifiedIdeograph8c51', [], 'Display', None],
  [ 0x8c52, 'CjkUnifiedIdeograph8c52', [], 'Display', None],
  [ 0x8c53, 'CjkUnifiedIdeograph8c53', [], 'Display', None],
  [ 0x8c54, 'CjkUnifiedIdeograph8c54', [], 'Display', None],
  [ 0x8c55, 'CjkUnifiedIdeograph8c55', [], 'Display', None],
  [ 0x8c56, 'CjkUnifiedIdeograph8c56', [], 'Display', None],
  [ 0x8c57, 'CjkUnifiedIdeograph8c57', [], 'Display', None],
  [ 0x8c58, 'CjkUnifiedIdeograph8c58', [], 'Display', None],
  [ 0x8c59, 'CjkUnifiedIdeograph8c59', [], 'Display', None],
  [ 0x8c5a, 'CjkUnifiedIdeograph8c5a', [], 'Display', None],
  [ 0x8c5b, 'CjkUnifiedIdeograph8c5b', [], 'Display', None],
  [ 0x8c5c, 'CjkUnifiedIdeograph8c5c', [], 'Display', None],
  [ 0x8c5d, 'CjkUnifiedIdeograph8c5d', [], 'Display', None],
  [ 0x8c5e, 'CjkUnifiedIdeograph8c5e', [], 'Display', None],
  [ 0x8c5f, 'CjkUnifiedIdeograph8c5f', [], 'Display', None],
  [ 0x8c60, 'CjkUnifiedIdeograph8c60', [], 'Display', None],
  [ 0x8c61, 'CjkUnifiedIdeograph8c61', [], 'Display', None],
  [ 0x8c62, 'CjkUnifiedIdeograph8c62', [], 'Display', None],
  [ 0x8c63, 'CjkUnifiedIdeograph8c63', [], 'Display', None],
  [ 0x8c64, 'CjkUnifiedIdeograph8c64', [], 'Display', None],
  [ 0x8c65, 'CjkUnifiedIdeograph8c65', [], 'Display', None],
  [ 0x8c66, 'CjkUnifiedIdeograph8c66', [], 'Display', None],
  [ 0x8c67, 'CjkUnifiedIdeograph8c67', [], 'Display', None],
  [ 0x8c68, 'CjkUnifiedIdeograph8c68', [], 'Display', None],
  [ 0x8c69, 'CjkUnifiedIdeograph8c69', [], 'Display', None],
  [ 0x8c6a, 'CjkUnifiedIdeograph8c6a', [], 'Display', None],
  [ 0x8c6b, 'CjkUnifiedIdeograph8c6b', [], 'Display', None],
  [ 0x8c6c, 'CjkUnifiedIdeograph8c6c', [], 'Display', None],
  [ 0x8c6d, 'CjkUnifiedIdeograph8c6d', [], 'Display', None],
  [ 0x8c6e, 'CjkUnifiedIdeograph8c6e', [], 'Display', None],
  [ 0x8c6f, 'CjkUnifiedIdeograph8c6f', [], 'Display', None],
  [ 0x8c70, 'CjkUnifiedIdeograph8c70', [], 'Display', None],
  [ 0x8c71, 'CjkUnifiedIdeograph8c71', [], 'Display', None],
  [ 0x8c72, 'CjkUnifiedIdeograph8c72', [], 'Display', None],
  [ 0x8c73, 'CjkUnifiedIdeograph8c73', [], 'Display', None],
  [ 0x8c74, 'CjkUnifiedIdeograph8c74', [], 'Display', None],
  [ 0x8c75, 'CjkUnifiedIdeograph8c75', [], 'Display', None],
  [ 0x8c76, 'CjkUnifiedIdeograph8c76', [], 'Display', None],
  [ 0x8c77, 'CjkUnifiedIdeograph8c77', [], 'Display', None],
  [ 0x8c78, 'CjkUnifiedIdeograph8c78', [], 'Display', None],
  [ 0x8c79, 'CjkUnifiedIdeograph8c79', [], 'Display', None],
  [ 0x8c7a, 'CjkUnifiedIdeograph8c7a', [], 'Display', None],
  [ 0x8c7b, 'CjkUnifiedIdeograph8c7b', [], 'Display', None],
  [ 0x8c7c, 'CjkUnifiedIdeograph8c7c', [], 'Display', None],
  [ 0x8c7d, 'CjkUnifiedIdeograph8c7d', [], 'Display', None],
  [ 0x8c7e, 'CjkUnifiedIdeograph8c7e', [], 'Display', None],
  [ 0x8c7f, 'CjkUnifiedIdeograph8c7f', [], 'Display', None],
  [ 0x8c80, 'CjkUnifiedIdeograph8c80', [], 'Display', None],
  [ 0x8c81, 'CjkUnifiedIdeograph8c81', [], 'Display', None],
  [ 0x8c82, 'CjkUnifiedIdeograph8c82', [], 'Display', None],
  [ 0x8c83, 'CjkUnifiedIdeograph8c83', [], 'Display', None],
  [ 0x8c84, 'CjkUnifiedIdeograph8c84', [], 'Display', None],
  [ 0x8c85, 'CjkUnifiedIdeograph8c85', [], 'Display', None],
  [ 0x8c86, 'CjkUnifiedIdeograph8c86', [], 'Display', None],
  [ 0x8c87, 'CjkUnifiedIdeograph8c87', [], 'Display', None],
  [ 0x8c88, 'CjkUnifiedIdeograph8c88', [], 'Display', None],
  [ 0x8c89, 'CjkUnifiedIdeograph8c89', [], 'Display', None],
  [ 0x8c8a, 'CjkUnifiedIdeograph8c8a', [], 'Display', None],
  [ 0x8c8b, 'CjkUnifiedIdeograph8c8b', [], 'Display', None],
  [ 0x8c8c, 'CjkUnifiedIdeograph8c8c', [], 'Display', None],
  [ 0x8c8d, 'CjkUnifiedIdeograph8c8d', [], 'Display', None],
  [ 0x8c8e, 'CjkUnifiedIdeograph8c8e', [], 'Display', None],
  [ 0x8c8f, 'CjkUnifiedIdeograph8c8f', [], 'Display', None],
  [ 0x8c90, 'CjkUnifiedIdeograph8c90', [], 'Display', None],
  [ 0x8c91, 'CjkUnifiedIdeograph8c91', [], 'Display', None],
  [ 0x8c92, 'CjkUnifiedIdeograph8c92', [], 'Display', None],
  [ 0x8c93, 'CjkUnifiedIdeograph8c93', [], 'Display', None],
  [ 0x8c94, 'CjkUnifiedIdeograph8c94', [], 'Display', None],
  [ 0x8c95, 'CjkUnifiedIdeograph8c95', [], 'Display', None],
  [ 0x8c96, 'CjkUnifiedIdeograph8c96', [], 'Display', None],
  [ 0x8c97, 'CjkUnifiedIdeograph8c97', [], 'Display', None],
  [ 0x8c98, 'CjkUnifiedIdeograph8c98', [], 'Display', None],
  [ 0x8c99, 'CjkUnifiedIdeograph8c99', [], 'Display', None],
  [ 0x8c9a, 'CjkUnifiedIdeograph8c9a', [], 'Display', None],
  [ 0x8c9b, 'CjkUnifiedIdeograph8c9b', [], 'Display', None],
  [ 0x8c9c, 'CjkUnifiedIdeograph8c9c', [], 'Display', None],
  [ 0x8c9d, 'CjkUnifiedIdeograph8c9d', [], 'Display', None],
  [ 0x8c9e, 'CjkUnifiedIdeograph8c9e', [], 'Display', None],
  [ 0x8c9f, 'CjkUnifiedIdeograph8c9f', [], 'Display', None],
  [ 0x8ca0, 'CjkUnifiedIdeograph8ca0', [], 'Display', None],
  [ 0x8ca1, 'CjkUnifiedIdeograph8ca1', [], 'Display', None],
  [ 0x8ca2, 'CjkUnifiedIdeograph8ca2', [], 'Display', None],
  [ 0x8ca3, 'CjkUnifiedIdeograph8ca3', [], 'Display', None],
  [ 0x8ca4, 'CjkUnifiedIdeograph8ca4', [], 'Display', None],
  [ 0x8ca5, 'CjkUnifiedIdeograph8ca5', [], 'Display', None],
  [ 0x8ca6, 'CjkUnifiedIdeograph8ca6', [], 'Display', None],
  [ 0x8ca7, 'CjkUnifiedIdeograph8ca7', [], 'Display', None],
  [ 0x8ca8, 'CjkUnifiedIdeograph8ca8', [], 'Display', None],
  [ 0x8ca9, 'CjkUnifiedIdeograph8ca9', [], 'Display', None],
  [ 0x8caa, 'CjkUnifiedIdeograph8caa', [], 'Display', None],
  [ 0x8cab, 'CjkUnifiedIdeograph8cab', [], 'Display', None],
  [ 0x8cac, 'CjkUnifiedIdeograph8cac', [], 'Display', None],
  [ 0x8cad, 'CjkUnifiedIdeograph8cad', [], 'Display', None],
  [ 0x8cae, 'CjkUnifiedIdeograph8cae', [], 'Display', None],
  [ 0x8caf, 'CjkUnifiedIdeograph8caf', [], 'Display', None],
  [ 0x8cb0, 'CjkUnifiedIdeograph8cb0', [], 'Display', None],
  [ 0x8cb1, 'CjkUnifiedIdeograph8cb1', [], 'Display', None],
  [ 0x8cb2, 'CjkUnifiedIdeograph8cb2', [], 'Display', None],
  [ 0x8cb3, 'CjkUnifiedIdeograph8cb3', [], 'Display', None],
  [ 0x8cb4, 'CjkUnifiedIdeograph8cb4', [], 'Display', None],
  [ 0x8cb5, 'CjkUnifiedIdeograph8cb5', [], 'Display', None],
  [ 0x8cb6, 'CjkUnifiedIdeograph8cb6', [], 'Display', None],
  [ 0x8cb7, 'CjkUnifiedIdeograph8cb7', [], 'Display', None],
  [ 0x8cb8, 'CjkUnifiedIdeograph8cb8', [], 'Display', None],
  [ 0x8cb9, 'CjkUnifiedIdeograph8cb9', [], 'Display', None],
  [ 0x8cba, 'CjkUnifiedIdeograph8cba', [], 'Display', None],
  [ 0x8cbb, 'CjkUnifiedIdeograph8cbb', [], 'Display', None],
  [ 0x8cbc, 'CjkUnifiedIdeograph8cbc', [], 'Display', None],
  [ 0x8cbd, 'CjkUnifiedIdeograph8cbd', [], 'Display', None],
  [ 0x8cbe, 'CjkUnifiedIdeograph8cbe', [], 'Display', None],
  [ 0x8cbf, 'CjkUnifiedIdeograph8cbf', [], 'Display', None],
  [ 0x8cc0, 'CjkUnifiedIdeograph8cc0', [], 'Display', None],
  [ 0x8cc1, 'CjkUnifiedIdeograph8cc1', [], 'Display', None],
  [ 0x8cc2, 'CjkUnifiedIdeograph8cc2', [], 'Display', None],
  [ 0x8cc3, 'CjkUnifiedIdeograph8cc3', [], 'Display', None],
  [ 0x8cc4, 'CjkUnifiedIdeograph8cc4', [], 'Display', None],
  [ 0x8cc5, 'CjkUnifiedIdeograph8cc5', [], 'Display', None],
  [ 0x8cc6, 'CjkUnifiedIdeograph8cc6', [], 'Display', None],
  [ 0x8cc7, 'CjkUnifiedIdeograph8cc7', [], 'Display', None],
  [ 0x8cc8, 'CjkUnifiedIdeograph8cc8', [], 'Display', None],
  [ 0x8cc9, 'CjkUnifiedIdeograph8cc9', [], 'Display', None],
  [ 0x8cca, 'CjkUnifiedIdeograph8cca', [], 'Display', None],
  [ 0x8ccb, 'CjkUnifiedIdeograph8ccb', [], 'Display', None],
  [ 0x8ccc, 'CjkUnifiedIdeograph8ccc', [], 'Display', None],
  [ 0x8ccd, 'CjkUnifiedIdeograph8ccd', [], 'Display', None],
  [ 0x8cce, 'CjkUnifiedIdeograph8cce', [], 'Display', None],
  [ 0x8ccf, 'CjkUnifiedIdeograph8ccf', [], 'Display', None],
  [ 0x8cd0, 'CjkUnifiedIdeograph8cd0', [], 'Display', None],
  [ 0x8cd1, 'CjkUnifiedIdeograph8cd1', [], 'Display', None],
  [ 0x8cd2, 'CjkUnifiedIdeograph8cd2', [], 'Display', None],
  [ 0x8cd3, 'CjkUnifiedIdeograph8cd3', [], 'Display', None],
  [ 0x8cd4, 'CjkUnifiedIdeograph8cd4', [], 'Display', None],
  [ 0x8cd5, 'CjkUnifiedIdeograph8cd5', [], 'Display', None],
  [ 0x8cd6, 'CjkUnifiedIdeograph8cd6', [], 'Display', None],
  [ 0x8cd7, 'CjkUnifiedIdeograph8cd7', [], 'Display', None],
  [ 0x8cd8, 'CjkUnifiedIdeograph8cd8', [], 'Display', None],
  [ 0x8cd9, 'CjkUnifiedIdeograph8cd9', [], 'Display', None],
  [ 0x8cda, 'CjkUnifiedIdeograph8cda', [], 'Display', None],
  [ 0x8cdb, 'CjkUnifiedIdeograph8cdb', [], 'Display', None],
  [ 0x8cdc, 'CjkUnifiedIdeograph8cdc', [], 'Display', None],
  [ 0x8cdd, 'CjkUnifiedIdeograph8cdd', [], 'Display', None],
  [ 0x8cde, 'CjkUnifiedIdeograph8cde', [], 'Display', None],
  [ 0x8cdf, 'CjkUnifiedIdeograph8cdf', [], 'Display', None],
  [ 0x8ce0, 'CjkUnifiedIdeograph8ce0', [], 'Display', None],
  [ 0x8ce1, 'CjkUnifiedIdeograph8ce1', [], 'Display', None],
  [ 0x8ce2, 'CjkUnifiedIdeograph8ce2', [], 'Display', None],
  [ 0x8ce3, 'CjkUnifiedIdeograph8ce3', [], 'Display', None],
  [ 0x8ce4, 'CjkUnifiedIdeograph8ce4', [], 'Display', None],
  [ 0x8ce5, 'CjkUnifiedIdeograph8ce5', [], 'Display', None],
  [ 0x8ce6, 'CjkUnifiedIdeograph8ce6', [], 'Display', None],
  [ 0x8ce7, 'CjkUnifiedIdeograph8ce7', [], 'Display', None],
  [ 0x8ce8, 'CjkUnifiedIdeograph8ce8', [], 'Display', None],
  [ 0x8ce9, 'CjkUnifiedIdeograph8ce9', [], 'Display', None],
  [ 0x8cea, 'CjkUnifiedIdeograph8cea', [], 'Display', None],
  [ 0x8ceb, 'CjkUnifiedIdeograph8ceb', [], 'Display', None],
  [ 0x8cec, 'CjkUnifiedIdeograph8cec', [], 'Display', None],
  [ 0x8ced, 'CjkUnifiedIdeograph8ced', [], 'Display', None],
  [ 0x8cee, 'CjkUnifiedIdeograph8cee', [], 'Display', None],
  [ 0x8cef, 'CjkUnifiedIdeograph8cef', [], 'Display', None],
  [ 0x8cf0, 'CjkUnifiedIdeograph8cf0', [], 'Display', None],
  [ 0x8cf1, 'CjkUnifiedIdeograph8cf1', [], 'Display', None],
  [ 0x8cf2, 'CjkUnifiedIdeograph8cf2', [], 'Display', None],
  [ 0x8cf3, 'CjkUnifiedIdeograph8cf3', [], 'Display', None],
  [ 0x8cf4, 'CjkUnifiedIdeograph8cf4', [], 'Display', None],
  [ 0x8cf5, 'CjkUnifiedIdeograph8cf5', [], 'Display', None],
  [ 0x8cf6, 'CjkUnifiedIdeograph8cf6', [], 'Display', None],
  [ 0x8cf7, 'CjkUnifiedIdeograph8cf7', [], 'Display', None],
  [ 0x8cf8, 'CjkUnifiedIdeograph8cf8', [], 'Display', None],
  [ 0x8cf9, 'CjkUnifiedIdeograph8cf9', [], 'Display', None],
  [ 0x8cfa, 'CjkUnifiedIdeograph8cfa', [], 'Display', None],
  [ 0x8cfb, 'CjkUnifiedIdeograph8cfb', [], 'Display', None],
  [ 0x8cfc, 'CjkUnifiedIdeograph8cfc', [], 'Display', None],
  [ 0x8cfd, 'CjkUnifiedIdeograph8cfd', [], 'Display', None],
  [ 0x8cfe, 'CjkUnifiedIdeograph8cfe', [], 'Display', None],
  [ 0x8cff, 'CjkUnifiedIdeograph8cff', [], 'Display', None],
  [ 0x8d00, 'CjkUnifiedIdeograph8d00', [], 'Display', None],
  [ 0x8d01, 'CjkUnifiedIdeograph8d01', [], 'Display', None],
  [ 0x8d02, 'CjkUnifiedIdeograph8d02', [], 'Display', None],
  [ 0x8d03, 'CjkUnifiedIdeograph8d03', [], 'Display', None],
  [ 0x8d04, 'CjkUnifiedIdeograph8d04', [], 'Display', None],
  [ 0x8d05, 'CjkUnifiedIdeograph8d05', [], 'Display', None],
  [ 0x8d06, 'CjkUnifiedIdeograph8d06', [], 'Display', None],
  [ 0x8d07, 'CjkUnifiedIdeograph8d07', [], 'Display', None],
  [ 0x8d08, 'CjkUnifiedIdeograph8d08', [], 'Display', None],
  [ 0x8d09, 'CjkUnifiedIdeograph8d09', [], 'Display', None],
  [ 0x8d0a, 'CjkUnifiedIdeograph8d0a', [], 'Display', None],
  [ 0x8d0b, 'CjkUnifiedIdeograph8d0b', [], 'Display', None],
  [ 0x8d0c, 'CjkUnifiedIdeograph8d0c', [], 'Display', None],
  [ 0x8d0d, 'CjkUnifiedIdeograph8d0d', [], 'Display', None],
  [ 0x8d0e, 'CjkUnifiedIdeograph8d0e', [], 'Display', None],
  [ 0x8d0f, 'CjkUnifiedIdeograph8d0f', [], 'Display', None],
  [ 0x8d10, 'CjkUnifiedIdeograph8d10', [], 'Display', None],
  [ 0x8d11, 'CjkUnifiedIdeograph8d11', [], 'Display', None],
  [ 0x8d12, 'CjkUnifiedIdeograph8d12', [], 'Display', None],
  [ 0x8d13, 'CjkUnifiedIdeograph8d13', [], 'Display', None],
  [ 0x8d14, 'CjkUnifiedIdeograph8d14', [], 'Display', None],
  [ 0x8d15, 'CjkUnifiedIdeograph8d15', [], 'Display', None],
  [ 0x8d16, 'CjkUnifiedIdeograph8d16', [], 'Display', None],
  [ 0x8d17, 'CjkUnifiedIdeograph8d17', [], 'Display', None],
  [ 0x8d18, 'CjkUnifiedIdeograph8d18', [], 'Display', None],
  [ 0x8d19, 'CjkUnifiedIdeograph8d19', [], 'Display', None],
  [ 0x8d1a, 'CjkUnifiedIdeograph8d1a', [], 'Display', None],
  [ 0x8d1b, 'CjkUnifiedIdeograph8d1b', [], 'Display', None],
  [ 0x8d1c, 'CjkUnifiedIdeograph8d1c', [], 'Display', None],
  [ 0x8d1d, 'CjkUnifiedIdeograph8d1d', [], 'Display', None],
  [ 0x8d1e, 'CjkUnifiedIdeograph8d1e', [], 'Display', None],
  [ 0x8d1f, 'CjkUnifiedIdeograph8d1f', [], 'Display', None],
  [ 0x8d20, 'CjkUnifiedIdeograph8d20', [], 'Display', None],
  [ 0x8d21, 'CjkUnifiedIdeograph8d21', [], 'Display', None],
  [ 0x8d22, 'CjkUnifiedIdeograph8d22', [], 'Display', None],
  [ 0x8d23, 'CjkUnifiedIdeograph8d23', [], 'Display', None],
  [ 0x8d24, 'CjkUnifiedIdeograph8d24', [], 'Display', None],
  [ 0x8d25, 'CjkUnifiedIdeograph8d25', [], 'Display', None],
  [ 0x8d26, 'CjkUnifiedIdeograph8d26', [], 'Display', None],
  [ 0x8d27, 'CjkUnifiedIdeograph8d27', [], 'Display', None],
  [ 0x8d28, 'CjkUnifiedIdeograph8d28', [], 'Display', None],
  [ 0x8d29, 'CjkUnifiedIdeograph8d29', [], 'Display', None],
  [ 0x8d2a, 'CjkUnifiedIdeograph8d2a', [], 'Display', None],
  [ 0x8d2b, 'CjkUnifiedIdeograph8d2b', [], 'Display', None],
  [ 0x8d2c, 'CjkUnifiedIdeograph8d2c', [], 'Display', None],
  [ 0x8d2d, 'CjkUnifiedIdeograph8d2d', [], 'Display', None],
  [ 0x8d2e, 'CjkUnifiedIdeograph8d2e', [], 'Display', None],
  [ 0x8d2f, 'CjkUnifiedIdeograph8d2f', [], 'Display', None],
  [ 0x8d30, 'CjkUnifiedIdeograph8d30', [], 'Display', None],
  [ 0x8d31, 'CjkUnifiedIdeograph8d31', [], 'Display', None],
  [ 0x8d32, 'CjkUnifiedIdeograph8d32', [], 'Display', None],
  [ 0x8d33, 'CjkUnifiedIdeograph8d33', [], 'Display', None],
  [ 0x8d34, 'CjkUnifiedIdeograph8d34', [], 'Display', None],
  [ 0x8d35, 'CjkUnifiedIdeograph8d35', [], 'Display', None],
  [ 0x8d36, 'CjkUnifiedIdeograph8d36', [], 'Display', None],
  [ 0x8d37, 'CjkUnifiedIdeograph8d37', [], 'Display', None],
  [ 0x8d38, 'CjkUnifiedIdeograph8d38', [], 'Display', None],
  [ 0x8d39, 'CjkUnifiedIdeograph8d39', [], 'Display', None],
  [ 0x8d3a, 'CjkUnifiedIdeograph8d3a', [], 'Display', None],
  [ 0x8d3b, 'CjkUnifiedIdeograph8d3b', [], 'Display', None],
  [ 0x8d3c, 'CjkUnifiedIdeograph8d3c', [], 'Display', None],
  [ 0x8d3d, 'CjkUnifiedIdeograph8d3d', [], 'Display', None],
  [ 0x8d3e, 'CjkUnifiedIdeograph8d3e', [], 'Display', None],
  [ 0x8d3f, 'CjkUnifiedIdeograph8d3f', [], 'Display', None],
  [ 0x8d40, 'CjkUnifiedIdeograph8d40', [], 'Display', None],
  [ 0x8d41, 'CjkUnifiedIdeograph8d41', [], 'Display', None],
  [ 0x8d42, 'CjkUnifiedIdeograph8d42', [], 'Display', None],
  [ 0x8d43, 'CjkUnifiedIdeograph8d43', [], 'Display', None],
  [ 0x8d44, 'CjkUnifiedIdeograph8d44', [], 'Display', None],
  [ 0x8d45, 'CjkUnifiedIdeograph8d45', [], 'Display', None],
  [ 0x8d46, 'CjkUnifiedIdeograph8d46', [], 'Display', None],
  [ 0x8d47, 'CjkUnifiedIdeograph8d47', [], 'Display', None],
  [ 0x8d48, 'CjkUnifiedIdeograph8d48', [], 'Display', None],
  [ 0x8d49, 'CjkUnifiedIdeograph8d49', [], 'Display', None],
  [ 0x8d4a, 'CjkUnifiedIdeograph8d4a', [], 'Display', None],
  [ 0x8d4b, 'CjkUnifiedIdeograph8d4b', [], 'Display', None],
  [ 0x8d4c, 'CjkUnifiedIdeograph8d4c', [], 'Display', None],
  [ 0x8d4d, 'CjkUnifiedIdeograph8d4d', [], 'Display', None],
  [ 0x8d4e, 'CjkUnifiedIdeograph8d4e', [], 'Display', None],
  [ 0x8d4f, 'CjkUnifiedIdeograph8d4f', [], 'Display', None],
  [ 0x8d50, 'CjkUnifiedIdeograph8d50', [], 'Display', None],
  [ 0x8d51, 'CjkUnifiedIdeograph8d51', [], 'Display', None],
  [ 0x8d52, 'CjkUnifiedIdeograph8d52', [], 'Display', None],
  [ 0x8d53, 'CjkUnifiedIdeograph8d53', [], 'Display', None],
  [ 0x8d54, 'CjkUnifiedIdeograph8d54', [], 'Display', None],
  [ 0x8d55, 'CjkUnifiedIdeograph8d55', [], 'Display', None],
  [ 0x8d56, 'CjkUnifiedIdeograph8d56', [], 'Display', None],
  [ 0x8d57, 'CjkUnifiedIdeograph8d57', [], 'Display', None],
  [ 0x8d58, 'CjkUnifiedIdeograph8d58', [], 'Display', None],
  [ 0x8d59, 'CjkUnifiedIdeograph8d59', [], 'Display', None],
  [ 0x8d5a, 'CjkUnifiedIdeograph8d5a', [], 'Display', None],
  [ 0x8d5b, 'CjkUnifiedIdeograph8d5b', [], 'Display', None],
  [ 0x8d5c, 'CjkUnifiedIdeograph8d5c', [], 'Display', None],
  [ 0x8d5d, 'CjkUnifiedIdeograph8d5d', [], 'Display', None],
  [ 0x8d5e, 'CjkUnifiedIdeograph8d5e', [], 'Display', None],
  [ 0x8d5f, 'CjkUnifiedIdeograph8d5f', [], 'Display', None],
  [ 0x8d60, 'CjkUnifiedIdeograph8d60', [], 'Display', None],
  [ 0x8d61, 'CjkUnifiedIdeograph8d61', [], 'Display', None],
  [ 0x8d62, 'CjkUnifiedIdeograph8d62', [], 'Display', None],
  [ 0x8d63, 'CjkUnifiedIdeograph8d63', [], 'Display', None],
  [ 0x8d64, 'CjkUnifiedIdeograph8d64', [], 'Display', None],
  [ 0x8d65, 'CjkUnifiedIdeograph8d65', [], 'Display', None],
  [ 0x8d66, 'CjkUnifiedIdeograph8d66', [], 'Display', None],
  [ 0x8d67, 'CjkUnifiedIdeograph8d67', [], 'Display', None],
  [ 0x8d68, 'CjkUnifiedIdeograph8d68', [], 'Display', None],
  [ 0x8d69, 'CjkUnifiedIdeograph8d69', [], 'Display', None],
  [ 0x8d6a, 'CjkUnifiedIdeograph8d6a', [], 'Display', None],
  [ 0x8d6b, 'CjkUnifiedIdeograph8d6b', [], 'Display', None],
  [ 0x8d6c, 'CjkUnifiedIdeograph8d6c', [], 'Display', None],
  [ 0x8d6d, 'CjkUnifiedIdeograph8d6d', [], 'Display', None],
  [ 0x8d6e, 'CjkUnifiedIdeograph8d6e', [], 'Display', None],
  [ 0x8d6f, 'CjkUnifiedIdeograph8d6f', [], 'Display', None],
  [ 0x8d70, 'CjkUnifiedIdeograph8d70', [], 'Display', None],
  [ 0x8d71, 'CjkUnifiedIdeograph8d71', [], 'Display', None],
  [ 0x8d72, 'CjkUnifiedIdeograph8d72', [], 'Display', None],
  [ 0x8d73, 'CjkUnifiedIdeograph8d73', [], 'Display', None],
  [ 0x8d74, 'CjkUnifiedIdeograph8d74', [], 'Display', None],
  [ 0x8d75, 'CjkUnifiedIdeograph8d75', [], 'Display', None],
  [ 0x8d76, 'CjkUnifiedIdeograph8d76', [], 'Display', None],
  [ 0x8d77, 'CjkUnifiedIdeograph8d77', [], 'Display', None],
  [ 0x8d78, 'CjkUnifiedIdeograph8d78', [], 'Display', None],
  [ 0x8d79, 'CjkUnifiedIdeograph8d79', [], 'Display', None],
  [ 0x8d7a, 'CjkUnifiedIdeograph8d7a', [], 'Display', None],
  [ 0x8d7b, 'CjkUnifiedIdeograph8d7b', [], 'Display', None],
  [ 0x8d7c, 'CjkUnifiedIdeograph8d7c', [], 'Display', None],
  [ 0x8d7d, 'CjkUnifiedIdeograph8d7d', [], 'Display', None],
  [ 0x8d7e, 'CjkUnifiedIdeograph8d7e', [], 'Display', None],
  [ 0x8d7f, 'CjkUnifiedIdeograph8d7f', [], 'Display', None],
  [ 0x8d80, 'CjkUnifiedIdeograph8d80', [], 'Display', None],
  [ 0x8d81, 'CjkUnifiedIdeograph8d81', [], 'Display', None],
  [ 0x8d82, 'CjkUnifiedIdeograph8d82', [], 'Display', None],
  [ 0x8d83, 'CjkUnifiedIdeograph8d83', [], 'Display', None],
  [ 0x8d84, 'CjkUnifiedIdeograph8d84', [], 'Display', None],
  [ 0x8d85, 'CjkUnifiedIdeograph8d85', [], 'Display', None],
  [ 0x8d86, 'CjkUnifiedIdeograph8d86', [], 'Display', None],
  [ 0x8d87, 'CjkUnifiedIdeograph8d87', [], 'Display', None],
  [ 0x8d88, 'CjkUnifiedIdeograph8d88', [], 'Display', None],
  [ 0x8d89, 'CjkUnifiedIdeograph8d89', [], 'Display', None],
  [ 0x8d8a, 'CjkUnifiedIdeograph8d8a', [], 'Display', None],
  [ 0x8d8b, 'CjkUnifiedIdeograph8d8b', [], 'Display', None],
  [ 0x8d8c, 'CjkUnifiedIdeograph8d8c', [], 'Display', None],
  [ 0x8d8d, 'CjkUnifiedIdeograph8d8d', [], 'Display', None],
  [ 0x8d8e, 'CjkUnifiedIdeograph8d8e', [], 'Display', None],
  [ 0x8d8f, 'CjkUnifiedIdeograph8d8f', [], 'Display', None],
  [ 0x8d90, 'CjkUnifiedIdeograph8d90', [], 'Display', None],
  [ 0x8d91, 'CjkUnifiedIdeograph8d91', [], 'Display', None],
  [ 0x8d92, 'CjkUnifiedIdeograph8d92', [], 'Display', None],
  [ 0x8d93, 'CjkUnifiedIdeograph8d93', [], 'Display', None],
  [ 0x8d94, 'CjkUnifiedIdeograph8d94', [], 'Display', None],
  [ 0x8d95, 'CjkUnifiedIdeograph8d95', [], 'Display', None],
  [ 0x8d96, 'CjkUnifiedIdeograph8d96', [], 'Display', None],
  [ 0x8d97, 'CjkUnifiedIdeograph8d97', [], 'Display', None],
  [ 0x8d98, 'CjkUnifiedIdeograph8d98', [], 'Display', None],
  [ 0x8d99, 'CjkUnifiedIdeograph8d99', [], 'Display', None],
  [ 0x8d9a, 'CjkUnifiedIdeograph8d9a', [], 'Display', None],
  [ 0x8d9b, 'CjkUnifiedIdeograph8d9b', [], 'Display', None],
  [ 0x8d9c, 'CjkUnifiedIdeograph8d9c', [], 'Display', None],
  [ 0x8d9d, 'CjkUnifiedIdeograph8d9d', [], 'Display', None],
  [ 0x8d9e, 'CjkUnifiedIdeograph8d9e', [], 'Display', None],
  [ 0x8d9f, 'CjkUnifiedIdeograph8d9f', [], 'Display', None],
  [ 0x8da0, 'CjkUnifiedIdeograph8da0', [], 'Display', None],
  [ 0x8da1, 'CjkUnifiedIdeograph8da1', [], 'Display', None],
  [ 0x8da2, 'CjkUnifiedIdeograph8da2', [], 'Display', None],
  [ 0x8da3, 'CjkUnifiedIdeograph8da3', [], 'Display', None],
  [ 0x8da4, 'CjkUnifiedIdeograph8da4', [], 'Display', None],
  [ 0x8da5, 'CjkUnifiedIdeograph8da5', [], 'Display', None],
  [ 0x8da6, 'CjkUnifiedIdeograph8da6', [], 'Display', None],
  [ 0x8da7, 'CjkUnifiedIdeograph8da7', [], 'Display', None],
  [ 0x8da8, 'CjkUnifiedIdeograph8da8', [], 'Display', None],
  [ 0x8da9, 'CjkUnifiedIdeograph8da9', [], 'Display', None],
  [ 0x8daa, 'CjkUnifiedIdeograph8daa', [], 'Display', None],
  [ 0x8dab, 'CjkUnifiedIdeograph8dab', [], 'Display', None],
  [ 0x8dac, 'CjkUnifiedIdeograph8dac', [], 'Display', None],
  [ 0x8dad, 'CjkUnifiedIdeograph8dad', [], 'Display', None],
  [ 0x8dae, 'CjkUnifiedIdeograph8dae', [], 'Display', None],
  [ 0x8daf, 'CjkUnifiedIdeograph8daf', [], 'Display', None],
  [ 0x8db0, 'CjkUnifiedIdeograph8db0', [], 'Display', None],
  [ 0x8db1, 'CjkUnifiedIdeograph8db1', [], 'Display', None],
  [ 0x8db2, 'CjkUnifiedIdeograph8db2', [], 'Display', None],
  [ 0x8db3, 'CjkUnifiedIdeograph8db3', [], 'Display', None],
  [ 0x8db4, 'CjkUnifiedIdeograph8db4', [], 'Display', None],
  [ 0x8db5, 'CjkUnifiedIdeograph8db5', [], 'Display', None],
  [ 0x8db6, 'CjkUnifiedIdeograph8db6', [], 'Display', None],
  [ 0x8db7, 'CjkUnifiedIdeograph8db7', [], 'Display', None],
  [ 0x8db8, 'CjkUnifiedIdeograph8db8', [], 'Display', None],
  [ 0x8db9, 'CjkUnifiedIdeograph8db9', [], 'Display', None],
  [ 0x8dba, 'CjkUnifiedIdeograph8dba', [], 'Display', None],
  [ 0x8dbb, 'CjkUnifiedIdeograph8dbb', [], 'Display', None],
  [ 0x8dbc, 'CjkUnifiedIdeograph8dbc', [], 'Display', None],
  [ 0x8dbd, 'CjkUnifiedIdeograph8dbd', [], 'Display', None],
  [ 0x8dbe, 'CjkUnifiedIdeograph8dbe', [], 'Display', None],
  [ 0x8dbf, 'CjkUnifiedIdeograph8dbf', [], 'Display', None],
  [ 0x8dc0, 'CjkUnifiedIdeograph8dc0', [], 'Display', None],
  [ 0x8dc1, 'CjkUnifiedIdeograph8dc1', [], 'Display', None],
  [ 0x8dc2, 'CjkUnifiedIdeograph8dc2', [], 'Display', None],
  [ 0x8dc3, 'CjkUnifiedIdeograph8dc3', [], 'Display', None],
  [ 0x8dc4, 'CjkUnifiedIdeograph8dc4', [], 'Display', None],
  [ 0x8dc5, 'CjkUnifiedIdeograph8dc5', [], 'Display', None],
  [ 0x8dc6, 'CjkUnifiedIdeograph8dc6', [], 'Display', None],
  [ 0x8dc7, 'CjkUnifiedIdeograph8dc7', [], 'Display', None],
  [ 0x8dc8, 'CjkUnifiedIdeograph8dc8', [], 'Display', None],
  [ 0x8dc9, 'CjkUnifiedIdeograph8dc9', [], 'Display', None],
  [ 0x8dca, 'CjkUnifiedIdeograph8dca', [], 'Display', None],
  [ 0x8dcb, 'CjkUnifiedIdeograph8dcb', [], 'Display', None],
  [ 0x8dcc, 'CjkUnifiedIdeograph8dcc', [], 'Display', None],
  [ 0x8dcd, 'CjkUnifiedIdeograph8dcd', [], 'Display', None],
  [ 0x8dce, 'CjkUnifiedIdeograph8dce', [], 'Display', None],
  [ 0x8dcf, 'CjkUnifiedIdeograph8dcf', [], 'Display', None],
  [ 0x8dd0, 'CjkUnifiedIdeograph8dd0', [], 'Display', None],
  [ 0x8dd1, 'CjkUnifiedIdeograph8dd1', [], 'Display', None],
  [ 0x8dd2, 'CjkUnifiedIdeograph8dd2', [], 'Display', None],
  [ 0x8dd3, 'CjkUnifiedIdeograph8dd3', [], 'Display', None],
  [ 0x8dd4, 'CjkUnifiedIdeograph8dd4', [], 'Display', None],
  [ 0x8dd5, 'CjkUnifiedIdeograph8dd5', [], 'Display', None],
  [ 0x8dd6, 'CjkUnifiedIdeograph8dd6', [], 'Display', None],
  [ 0x8dd7, 'CjkUnifiedIdeograph8dd7', [], 'Display', None],
  [ 0x8dd8, 'CjkUnifiedIdeograph8dd8', [], 'Display', None],
  [ 0x8dd9, 'CjkUnifiedIdeograph8dd9', [], 'Display', None],
  [ 0x8dda, 'CjkUnifiedIdeograph8dda', [], 'Display', None],
  [ 0x8ddb, 'CjkUnifiedIdeograph8ddb', [], 'Display', None],
  [ 0x8ddc, 'CjkUnifiedIdeograph8ddc', [], 'Display', None],
  [ 0x8ddd, 'CjkUnifiedIdeograph8ddd', [], 'Display', None],
  [ 0x8dde, 'CjkUnifiedIdeograph8dde', [], 'Display', None],
  [ 0x8ddf, 'CjkUnifiedIdeograph8ddf', [], 'Display', None],
  [ 0x8de0, 'CjkUnifiedIdeograph8de0', [], 'Display', None],
  [ 0x8de1, 'CjkUnifiedIdeograph8de1', [], 'Display', None],
  [ 0x8de2, 'CjkUnifiedIdeograph8de2', [], 'Display', None],
  [ 0x8de3, 'CjkUnifiedIdeograph8de3', [], 'Display', None],
  [ 0x8de4, 'CjkUnifiedIdeograph8de4', [], 'Display', None],
  [ 0x8de5, 'CjkUnifiedIdeograph8de5', [], 'Display', None],
  [ 0x8de6, 'CjkUnifiedIdeograph8de6', [], 'Display', None],
  [ 0x8de7, 'CjkUnifiedIdeograph8de7', [], 'Display', None],
  [ 0x8de8, 'CjkUnifiedIdeograph8de8', [], 'Display', None],
  [ 0x8de9, 'CjkUnifiedIdeograph8de9', [], 'Display', None],
  [ 0x8dea, 'CjkUnifiedIdeograph8dea', [], 'Display', None],
  [ 0x8deb, 'CjkUnifiedIdeograph8deb', [], 'Display', None],
  [ 0x8dec, 'CjkUnifiedIdeograph8dec', [], 'Display', None],
  [ 0x8ded, 'CjkUnifiedIdeograph8ded', [], 'Display', None],
  [ 0x8dee, 'CjkUnifiedIdeograph8dee', [], 'Display', None],
  [ 0x8def, 'CjkUnifiedIdeograph8def', [], 'Display', None],
  [ 0x8df0, 'CjkUnifiedIdeograph8df0', [], 'Display', None],
  [ 0x8df1, 'CjkUnifiedIdeograph8df1', [], 'Display', None],
  [ 0x8df2, 'CjkUnifiedIdeograph8df2', [], 'Display', None],
  [ 0x8df3, 'CjkUnifiedIdeograph8df3', [], 'Display', None],
  [ 0x8df4, 'CjkUnifiedIdeograph8df4', [], 'Display', None],
  [ 0x8df5, 'CjkUnifiedIdeograph8df5', [], 'Display', None],
  [ 0x8df6, 'CjkUnifiedIdeograph8df6', [], 'Display', None],
  [ 0x8df7, 'CjkUnifiedIdeograph8df7', [], 'Display', None],
  [ 0x8df8, 'CjkUnifiedIdeograph8df8', [], 'Display', None],
  [ 0x8df9, 'CjkUnifiedIdeograph8df9', [], 'Display', None],
  [ 0x8dfa, 'CjkUnifiedIdeograph8dfa', [], 'Display', None],
  [ 0x8dfb, 'CjkUnifiedIdeograph8dfb', [], 'Display', None],
  [ 0x8dfc, 'CjkUnifiedIdeograph8dfc', [], 'Display', None],
  [ 0x8dfd, 'CjkUnifiedIdeograph8dfd', [], 'Display', None],
  [ 0x8dfe, 'CjkUnifiedIdeograph8dfe', [], 'Display', None],
  [ 0x8dff, 'CjkUnifiedIdeograph8dff', [], 'Display', None],
  [ 0x8e00, 'CjkUnifiedIdeograph8e00', [], 'Display', None],
  [ 0x8e01, 'CjkUnifiedIdeograph8e01', [], 'Display', None],
  [ 0x8e02, 'CjkUnifiedIdeograph8e02', [], 'Display', None],
  [ 0x8e03, 'CjkUnifiedIdeograph8e03', [], 'Display', None],
  [ 0x8e04, 'CjkUnifiedIdeograph8e04', [], 'Display', None],
  [ 0x8e05, 'CjkUnifiedIdeograph8e05', [], 'Display', None],
  [ 0x8e06, 'CjkUnifiedIdeograph8e06', [], 'Display', None],
  [ 0x8e07, 'CjkUnifiedIdeograph8e07', [], 'Display', None],
  [ 0x8e08, 'CjkUnifiedIdeograph8e08', [], 'Display', None],
  [ 0x8e09, 'CjkUnifiedIdeograph8e09', [], 'Display', None],
  [ 0x8e0a, 'CjkUnifiedIdeograph8e0a', [], 'Display', None],
  [ 0x8e0b, 'CjkUnifiedIdeograph8e0b', [], 'Display', None],
  [ 0x8e0c, 'CjkUnifiedIdeograph8e0c', [], 'Display', None],
  [ 0x8e0d, 'CjkUnifiedIdeograph8e0d', [], 'Display', None],
  [ 0x8e0e, 'CjkUnifiedIdeograph8e0e', [], 'Display', None],
  [ 0x8e0f, 'CjkUnifiedIdeograph8e0f', [], 'Display', None],
  [ 0x8e10, 'CjkUnifiedIdeograph8e10', [], 'Display', None],
  [ 0x8e11, 'CjkUnifiedIdeograph8e11', [], 'Display', None],
  [ 0x8e12, 'CjkUnifiedIdeograph8e12', [], 'Display', None],
  [ 0x8e13, 'CjkUnifiedIdeograph8e13', [], 'Display', None],
  [ 0x8e14, 'CjkUnifiedIdeograph8e14', [], 'Display', None],
  [ 0x8e15, 'CjkUnifiedIdeograph8e15', [], 'Display', None],
  [ 0x8e16, 'CjkUnifiedIdeograph8e16', [], 'Display', None],
  [ 0x8e17, 'CjkUnifiedIdeograph8e17', [], 'Display', None],
  [ 0x8e18, 'CjkUnifiedIdeograph8e18', [], 'Display', None],
  [ 0x8e19, 'CjkUnifiedIdeograph8e19', [], 'Display', None],
  [ 0x8e1a, 'CjkUnifiedIdeograph8e1a', [], 'Display', None],
  [ 0x8e1b, 'CjkUnifiedIdeograph8e1b', [], 'Display', None],
  [ 0x8e1c, 'CjkUnifiedIdeograph8e1c', [], 'Display', None],
  [ 0x8e1d, 'CjkUnifiedIdeograph8e1d', [], 'Display', None],
  [ 0x8e1e, 'CjkUnifiedIdeograph8e1e', [], 'Display', None],
  [ 0x8e1f, 'CjkUnifiedIdeograph8e1f', [], 'Display', None],
  [ 0x8e20, 'CjkUnifiedIdeograph8e20', [], 'Display', None],
  [ 0x8e21, 'CjkUnifiedIdeograph8e21', [], 'Display', None],
  [ 0x8e22, 'CjkUnifiedIdeograph8e22', [], 'Display', None],
  [ 0x8e23, 'CjkUnifiedIdeograph8e23', [], 'Display', None],
  [ 0x8e24, 'CjkUnifiedIdeograph8e24', [], 'Display', None],
  [ 0x8e25, 'CjkUnifiedIdeograph8e25', [], 'Display', None],
  [ 0x8e26, 'CjkUnifiedIdeograph8e26', [], 'Display', None],
  [ 0x8e27, 'CjkUnifiedIdeograph8e27', [], 'Display', None],
  [ 0x8e28, 'CjkUnifiedIdeograph8e28', [], 'Display', None],
  [ 0x8e29, 'CjkUnifiedIdeograph8e29', [], 'Display', None],
  [ 0x8e2a, 'CjkUnifiedIdeograph8e2a', [], 'Display', None],
  [ 0x8e2b, 'CjkUnifiedIdeograph8e2b', [], 'Display', None],
  [ 0x8e2c, 'CjkUnifiedIdeograph8e2c', [], 'Display', None],
  [ 0x8e2d, 'CjkUnifiedIdeograph8e2d', [], 'Display', None],
  [ 0x8e2e, 'CjkUnifiedIdeograph8e2e', [], 'Display', None],
  [ 0x8e2f, 'CjkUnifiedIdeograph8e2f', [], 'Display', None],
  [ 0x8e30, 'CjkUnifiedIdeograph8e30', [], 'Display', None],
  [ 0x8e31, 'CjkUnifiedIdeograph8e31', [], 'Display', None],
  [ 0x8e32, 'CjkUnifiedIdeograph8e32', [], 'Display', None],
  [ 0x8e33, 'CjkUnifiedIdeograph8e33', [], 'Display', None],
  [ 0x8e34, 'CjkUnifiedIdeograph8e34', [], 'Display', None],
  [ 0x8e35, 'CjkUnifiedIdeograph8e35', [], 'Display', None],
  [ 0x8e36, 'CjkUnifiedIdeograph8e36', [], 'Display', None],
  [ 0x8e37, 'CjkUnifiedIdeograph8e37', [], 'Display', None],
  [ 0x8e38, 'CjkUnifiedIdeograph8e38', [], 'Display', None],
  [ 0x8e39, 'CjkUnifiedIdeograph8e39', [], 'Display', None],
  [ 0x8e3a, 'CjkUnifiedIdeograph8e3a', [], 'Display', None],
  [ 0x8e3b, 'CjkUnifiedIdeograph8e3b', [], 'Display', None],
  [ 0x8e3c, 'CjkUnifiedIdeograph8e3c', [], 'Display', None],
  [ 0x8e3d, 'CjkUnifiedIdeograph8e3d', [], 'Display', None],
  [ 0x8e3e, 'CjkUnifiedIdeograph8e3e', [], 'Display', None],
  [ 0x8e3f, 'CjkUnifiedIdeograph8e3f', [], 'Display', None],
  [ 0x8e40, 'CjkUnifiedIdeograph8e40', [], 'Display', None],
  [ 0x8e41, 'CjkUnifiedIdeograph8e41', [], 'Display', None],
  [ 0x8e42, 'CjkUnifiedIdeograph8e42', [], 'Display', None],
  [ 0x8e43, 'CjkUnifiedIdeograph8e43', [], 'Display', None],
  [ 0x8e44, 'CjkUnifiedIdeograph8e44', [], 'Display', None],
  [ 0x8e45, 'CjkUnifiedIdeograph8e45', [], 'Display', None],
  [ 0x8e46, 'CjkUnifiedIdeograph8e46', [], 'Display', None],
  [ 0x8e47, 'CjkUnifiedIdeograph8e47', [], 'Display', None],
  [ 0x8e48, 'CjkUnifiedIdeograph8e48', [], 'Display', None],
  [ 0x8e49, 'CjkUnifiedIdeograph8e49', [], 'Display', None],
  [ 0x8e4a, 'CjkUnifiedIdeograph8e4a', [], 'Display', None],
  [ 0x8e4b, 'CjkUnifiedIdeograph8e4b', [], 'Display', None],
  [ 0x8e4c, 'CjkUnifiedIdeograph8e4c', [], 'Display', None],
  [ 0x8e4d, 'CjkUnifiedIdeograph8e4d', [], 'Display', None],
  [ 0x8e4e, 'CjkUnifiedIdeograph8e4e', [], 'Display', None],
  [ 0x8e4f, 'CjkUnifiedIdeograph8e4f', [], 'Display', None],
  [ 0x8e50, 'CjkUnifiedIdeograph8e50', [], 'Display', None],
  [ 0x8e51, 'CjkUnifiedIdeograph8e51', [], 'Display', None],
  [ 0x8e52, 'CjkUnifiedIdeograph8e52', [], 'Display', None],
  [ 0x8e53, 'CjkUnifiedIdeograph8e53', [], 'Display', None],
  [ 0x8e54, 'CjkUnifiedIdeograph8e54', [], 'Display', None],
  [ 0x8e55, 'CjkUnifiedIdeograph8e55', [], 'Display', None],
  [ 0x8e56, 'CjkUnifiedIdeograph8e56', [], 'Display', None],
  [ 0x8e57, 'CjkUnifiedIdeograph8e57', [], 'Display', None],
  [ 0x8e58, 'CjkUnifiedIdeograph8e58', [], 'Display', None],
  [ 0x8e59, 'CjkUnifiedIdeograph8e59', [], 'Display', None],
  [ 0x8e5a, 'CjkUnifiedIdeograph8e5a', [], 'Display', None],
  [ 0x8e5b, 'CjkUnifiedIdeograph8e5b', [], 'Display', None],
  [ 0x8e5c, 'CjkUnifiedIdeograph8e5c', [], 'Display', None],
  [ 0x8e5d, 'CjkUnifiedIdeograph8e5d', [], 'Display', None],
  [ 0x8e5e, 'CjkUnifiedIdeograph8e5e', [], 'Display', None],
  [ 0x8e5f, 'CjkUnifiedIdeograph8e5f', [], 'Display', None],
  [ 0x8e60, 'CjkUnifiedIdeograph8e60', [], 'Display', None],
  [ 0x8e61, 'CjkUnifiedIdeograph8e61', [], 'Display', None],
  [ 0x8e62, 'CjkUnifiedIdeograph8e62', [], 'Display', None],
  [ 0x8e63, 'CjkUnifiedIdeograph8e63', [], 'Display', None],
  [ 0x8e64, 'CjkUnifiedIdeograph8e64', [], 'Display', None],
  [ 0x8e65, 'CjkUnifiedIdeograph8e65', [], 'Display', None],
  [ 0x8e66, 'CjkUnifiedIdeograph8e66', [], 'Display', None],
  [ 0x8e67, 'CjkUnifiedIdeograph8e67', [], 'Display', None],
  [ 0x8e68, 'CjkUnifiedIdeograph8e68', [], 'Display', None],
  [ 0x8e69, 'CjkUnifiedIdeograph8e69', [], 'Display', None],
  [ 0x8e6a, 'CjkUnifiedIdeograph8e6a', [], 'Display', None],
  [ 0x8e6b, 'CjkUnifiedIdeograph8e6b', [], 'Display', None],
  [ 0x8e6c, 'CjkUnifiedIdeograph8e6c', [], 'Display', None],
  [ 0x8e6d, 'CjkUnifiedIdeograph8e6d', [], 'Display', None],
  [ 0x8e6e, 'CjkUnifiedIdeograph8e6e', [], 'Display', None],
  [ 0x8e6f, 'CjkUnifiedIdeograph8e6f', [], 'Display', None],
  [ 0x8e70, 'CjkUnifiedIdeograph8e70', [], 'Display', None],
  [ 0x8e71, 'CjkUnifiedIdeograph8e71', [], 'Display', None],
  [ 0x8e72, 'CjkUnifiedIdeograph8e72', [], 'Display', None],
  [ 0x8e73, 'CjkUnifiedIdeograph8e73', [], 'Display', None],
  [ 0x8e74, 'CjkUnifiedIdeograph8e74', [], 'Display', None],
  [ 0x8e75, 'CjkUnifiedIdeograph8e75', [], 'Display', None],
  [ 0x8e76, 'CjkUnifiedIdeograph8e76', [], 'Display', None],
  [ 0x8e77, 'CjkUnifiedIdeograph8e77', [], 'Display', None],
  [ 0x8e78, 'CjkUnifiedIdeograph8e78', [], 'Display', None],
  [ 0x8e79, 'CjkUnifiedIdeograph8e79', [], 'Display', None],
  [ 0x8e7a, 'CjkUnifiedIdeograph8e7a', [], 'Display', None],
  [ 0x8e7b, 'CjkUnifiedIdeograph8e7b', [], 'Display', None],
  [ 0x8e7c, 'CjkUnifiedIdeograph8e7c', [], 'Display', None],
  [ 0x8e7d, 'CjkUnifiedIdeograph8e7d', [], 'Display', None],
  [ 0x8e7e, 'CjkUnifiedIdeograph8e7e', [], 'Display', None],
  [ 0x8e7f, 'CjkUnifiedIdeograph8e7f', [], 'Display', None],
  [ 0x8e80, 'CjkUnifiedIdeograph8e80', [], 'Display', None],
  [ 0x8e81, 'CjkUnifiedIdeograph8e81', [], 'Display', None],
  [ 0x8e82, 'CjkUnifiedIdeograph8e82', [], 'Display', None],
  [ 0x8e83, 'CjkUnifiedIdeograph8e83', [], 'Display', None],
  [ 0x8e84, 'CjkUnifiedIdeograph8e84', [], 'Display', None],
  [ 0x8e85, 'CjkUnifiedIdeograph8e85', [], 'Display', None],
  [ 0x8e86, 'CjkUnifiedIdeograph8e86', [], 'Display', None],
  [ 0x8e87, 'CjkUnifiedIdeograph8e87', [], 'Display', None],
  [ 0x8e88, 'CjkUnifiedIdeograph8e88', [], 'Display', None],
  [ 0x8e89, 'CjkUnifiedIdeograph8e89', [], 'Display', None],
  [ 0x8e8a, 'CjkUnifiedIdeograph8e8a', [], 'Display', None],
  [ 0x8e8b, 'CjkUnifiedIdeograph8e8b', [], 'Display', None],
  [ 0x8e8c, 'CjkUnifiedIdeograph8e8c', [], 'Display', None],
  [ 0x8e8d, 'CjkUnifiedIdeograph8e8d', [], 'Display', None],
  [ 0x8e8e, 'CjkUnifiedIdeograph8e8e', [], 'Display', None],
  [ 0x8e8f, 'CjkUnifiedIdeograph8e8f', [], 'Display', None],
  [ 0x8e90, 'CjkUnifiedIdeograph8e90', [], 'Display', None],
  [ 0x8e91, 'CjkUnifiedIdeograph8e91', [], 'Display', None],
  [ 0x8e92, 'CjkUnifiedIdeograph8e92', [], 'Display', None],
  [ 0x8e93, 'CjkUnifiedIdeograph8e93', [], 'Display', None],
  [ 0x8e94, 'CjkUnifiedIdeograph8e94', [], 'Display', None],
  [ 0x8e95, 'CjkUnifiedIdeograph8e95', [], 'Display', None],
  [ 0x8e96, 'CjkUnifiedIdeograph8e96', [], 'Display', None],
  [ 0x8e97, 'CjkUnifiedIdeograph8e97', [], 'Display', None],
  [ 0x8e98, 'CjkUnifiedIdeograph8e98', [], 'Display', None],
  [ 0x8e99, 'CjkUnifiedIdeograph8e99', [], 'Display', None],
  [ 0x8e9a, 'CjkUnifiedIdeograph8e9a', [], 'Display', None],
  [ 0x8e9b, 'CjkUnifiedIdeograph8e9b', [], 'Display', None],
  [ 0x8e9c, 'CjkUnifiedIdeograph8e9c', [], 'Display', None],
  [ 0x8e9d, 'CjkUnifiedIdeograph8e9d', [], 'Display', None],
  [ 0x8e9e, 'CjkUnifiedIdeograph8e9e', [], 'Display', None],
  [ 0x8e9f, 'CjkUnifiedIdeograph8e9f', [], 'Display', None],
  [ 0x8ea0, 'CjkUnifiedIdeograph8ea0', [], 'Display', None],
  [ 0x8ea1, 'CjkUnifiedIdeograph8ea1', [], 'Display', None],
  [ 0x8ea2, 'CjkUnifiedIdeograph8ea2', [], 'Display', None],
  [ 0x8ea3, 'CjkUnifiedIdeograph8ea3', [], 'Display', None],
  [ 0x8ea4, 'CjkUnifiedIdeograph8ea4', [], 'Display', None],
  [ 0x8ea5, 'CjkUnifiedIdeograph8ea5', [], 'Display', None],
  [ 0x8ea6, 'CjkUnifiedIdeograph8ea6', [], 'Display', None],
  [ 0x8ea7, 'CjkUnifiedIdeograph8ea7', [], 'Display', None],
  [ 0x8ea8, 'CjkUnifiedIdeograph8ea8', [], 'Display', None],
  [ 0x8ea9, 'CjkUnifiedIdeograph8ea9', [], 'Display', None],
  [ 0x8eaa, 'CjkUnifiedIdeograph8eaa', [], 'Display', None],
  [ 0x8eab, 'CjkUnifiedIdeograph8eab', [], 'Display', None],
  [ 0x8eac, 'CjkUnifiedIdeograph8eac', [], 'Display', None],
  [ 0x8ead, 'CjkUnifiedIdeograph8ead', [], 'Display', None],
  [ 0x8eae, 'CjkUnifiedIdeograph8eae', [], 'Display', None],
  [ 0x8eaf, 'CjkUnifiedIdeograph8eaf', [], 'Display', None],
  [ 0x8eb0, 'CjkUnifiedIdeograph8eb0', [], 'Display', None],
  [ 0x8eb1, 'CjkUnifiedIdeograph8eb1', [], 'Display', None],
  [ 0x8eb2, 'CjkUnifiedIdeograph8eb2', [], 'Display', None],
  [ 0x8eb3, 'CjkUnifiedIdeograph8eb3', [], 'Display', None],
  [ 0x8eb4, 'CjkUnifiedIdeograph8eb4', [], 'Display', None],
  [ 0x8eb5, 'CjkUnifiedIdeograph8eb5', [], 'Display', None],
  [ 0x8eb6, 'CjkUnifiedIdeograph8eb6', [], 'Display', None],
  [ 0x8eb7, 'CjkUnifiedIdeograph8eb7', [], 'Display', None],
  [ 0x8eb8, 'CjkUnifiedIdeograph8eb8', [], 'Display', None],
  [ 0x8eb9, 'CjkUnifiedIdeograph8eb9', [], 'Display', None],
  [ 0x8eba, 'CjkUnifiedIdeograph8eba', [], 'Display', None],
  [ 0x8ebb, 'CjkUnifiedIdeograph8ebb', [], 'Display', None],
  [ 0x8ebc, 'CjkUnifiedIdeograph8ebc', [], 'Display', None],
  [ 0x8ebd, 'CjkUnifiedIdeograph8ebd', [], 'Display', None],
  [ 0x8ebe, 'CjkUnifiedIdeograph8ebe', [], 'Display', None],
  [ 0x8ebf, 'CjkUnifiedIdeograph8ebf', [], 'Display', None],
  [ 0x8ec0, 'CjkUnifiedIdeograph8ec0', [], 'Display', None],
  [ 0x8ec1, 'CjkUnifiedIdeograph8ec1', [], 'Display', None],
  [ 0x8ec2, 'CjkUnifiedIdeograph8ec2', [], 'Display', None],
  [ 0x8ec3, 'CjkUnifiedIdeograph8ec3', [], 'Display', None],
  [ 0x8ec4, 'CjkUnifiedIdeograph8ec4', [], 'Display', None],
  [ 0x8ec5, 'CjkUnifiedIdeograph8ec5', [], 'Display', None],
  [ 0x8ec6, 'CjkUnifiedIdeograph8ec6', [], 'Display', None],
  [ 0x8ec7, 'CjkUnifiedIdeograph8ec7', [], 'Display', None],
  [ 0x8ec8, 'CjkUnifiedIdeograph8ec8', [], 'Display', None],
  [ 0x8ec9, 'CjkUnifiedIdeograph8ec9', [], 'Display', None],
  [ 0x8eca, 'CjkUnifiedIdeograph8eca', [], 'Display', None],
  [ 0x8ecb, 'CjkUnifiedIdeograph8ecb', [], 'Display', None],
  [ 0x8ecc, 'CjkUnifiedIdeograph8ecc', [], 'Display', None],
  [ 0x8ecd, 'CjkUnifiedIdeograph8ecd', [], 'Display', None],
  [ 0x8ece, 'CjkUnifiedIdeograph8ece', [], 'Display', None],
  [ 0x8ecf, 'CjkUnifiedIdeograph8ecf', [], 'Display', None],
  [ 0x8ed0, 'CjkUnifiedIdeograph8ed0', [], 'Display', None],
  [ 0x8ed1, 'CjkUnifiedIdeograph8ed1', [], 'Display', None],
  [ 0x8ed2, 'CjkUnifiedIdeograph8ed2', [], 'Display', None],
  [ 0x8ed3, 'CjkUnifiedIdeograph8ed3', [], 'Display', None],
  [ 0x8ed4, 'CjkUnifiedIdeograph8ed4', [], 'Display', None],
  [ 0x8ed5, 'CjkUnifiedIdeograph8ed5', [], 'Display', None],
  [ 0x8ed6, 'CjkUnifiedIdeograph8ed6', [], 'Display', None],
  [ 0x8ed7, 'CjkUnifiedIdeograph8ed7', [], 'Display', None],
  [ 0x8ed8, 'CjkUnifiedIdeograph8ed8', [], 'Display', None],
  [ 0x8ed9, 'CjkUnifiedIdeograph8ed9', [], 'Display', None],
  [ 0x8eda, 'CjkUnifiedIdeograph8eda', [], 'Display', None],
  [ 0x8edb, 'CjkUnifiedIdeograph8edb', [], 'Display', None],
  [ 0x8edc, 'CjkUnifiedIdeograph8edc', [], 'Display', None],
  [ 0x8edd, 'CjkUnifiedIdeograph8edd', [], 'Display', None],
  [ 0x8ede, 'CjkUnifiedIdeograph8ede', [], 'Display', None],
  [ 0x8edf, 'CjkUnifiedIdeograph8edf', [], 'Display', None],
  [ 0x8ee0, 'CjkUnifiedIdeograph8ee0', [], 'Display', None],
  [ 0x8ee1, 'CjkUnifiedIdeograph8ee1', [], 'Display', None],
  [ 0x8ee2, 'CjkUnifiedIdeograph8ee2', [], 'Display', None],
  [ 0x8ee3, 'CjkUnifiedIdeograph8ee3', [], 'Display', None],
  [ 0x8ee4, 'CjkUnifiedIdeograph8ee4', [], 'Display', None],
  [ 0x8ee5, 'CjkUnifiedIdeograph8ee5', [], 'Display', None],
  [ 0x8ee6, 'CjkUnifiedIdeograph8ee6', [], 'Display', None],
  [ 0x8ee7, 'CjkUnifiedIdeograph8ee7', [], 'Display', None],
  [ 0x8ee8, 'CjkUnifiedIdeograph8ee8', [], 'Display', None],
  [ 0x8ee9, 'CjkUnifiedIdeograph8ee9', [], 'Display', None],
  [ 0x8eea, 'CjkUnifiedIdeograph8eea', [], 'Display', None],
  [ 0x8eeb, 'CjkUnifiedIdeograph8eeb', [], 'Display', None],
  [ 0x8eec, 'CjkUnifiedIdeograph8eec', [], 'Display', None],
  [ 0x8eed, 'CjkUnifiedIdeograph8eed', [], 'Display', None],
  [ 0x8eee, 'CjkUnifiedIdeograph8eee', [], 'Display', None],
  [ 0x8eef, 'CjkUnifiedIdeograph8eef', [], 'Display', None],
  [ 0x8ef0, 'CjkUnifiedIdeograph8ef0', [], 'Display', None],
  [ 0x8ef1, 'CjkUnifiedIdeograph8ef1', [], 'Display', None],
  [ 0x8ef2, 'CjkUnifiedIdeograph8ef2', [], 'Display', None],
  [ 0x8ef3, 'CjkUnifiedIdeograph8ef3', [], 'Display', None],
  [ 0x8ef4, 'CjkUnifiedIdeograph8ef4', [], 'Display', None],
  [ 0x8ef5, 'CjkUnifiedIdeograph8ef5', [], 'Display', None],
  [ 0x8ef6, 'CjkUnifiedIdeograph8ef6', [], 'Display', None],
  [ 0x8ef7, 'CjkUnifiedIdeograph8ef7', [], 'Display', None],
  [ 0x8ef8, 'CjkUnifiedIdeograph8ef8', [], 'Display', None],
  [ 0x8ef9, 'CjkUnifiedIdeograph8ef9', [], 'Display', None],
  [ 0x8efa, 'CjkUnifiedIdeograph8efa', [], 'Display', None],
  [ 0x8efb, 'CjkUnifiedIdeograph8efb', [], 'Display', None],
  [ 0x8efc, 'CjkUnifiedIdeograph8efc', [], 'Display', None],
  [ 0x8efd, 'CjkUnifiedIdeograph8efd', [], 'Display', None],
  [ 0x8efe, 'CjkUnifiedIdeograph8efe', [], 'Display', None],
  [ 0x8eff, 'CjkUnifiedIdeograph8eff', [], 'Display', None],
  [ 0x8f00, 'CjkUnifiedIdeograph8f00', [], 'Display', None],
  [ 0x8f01, 'CjkUnifiedIdeograph8f01', [], 'Display', None],
  [ 0x8f02, 'CjkUnifiedIdeograph8f02', [], 'Display', None],
  [ 0x8f03, 'CjkUnifiedIdeograph8f03', [], 'Display', None],
  [ 0x8f04, 'CjkUnifiedIdeograph8f04', [], 'Display', None],
  [ 0x8f05, 'CjkUnifiedIdeograph8f05', [], 'Display', None],
  [ 0x8f06, 'CjkUnifiedIdeograph8f06', [], 'Display', None],
  [ 0x8f07, 'CjkUnifiedIdeograph8f07', [], 'Display', None],
  [ 0x8f08, 'CjkUnifiedIdeograph8f08', [], 'Display', None],
  [ 0x8f09, 'CjkUnifiedIdeograph8f09', [], 'Display', None],
  [ 0x8f0a, 'CjkUnifiedIdeograph8f0a', [], 'Display', None],
  [ 0x8f0b, 'CjkUnifiedIdeograph8f0b', [], 'Display', None],
  [ 0x8f0c, 'CjkUnifiedIdeograph8f0c', [], 'Display', None],
  [ 0x8f0d, 'CjkUnifiedIdeograph8f0d', [], 'Display', None],
  [ 0x8f0e, 'CjkUnifiedIdeograph8f0e', [], 'Display', None],
  [ 0x8f0f, 'CjkUnifiedIdeograph8f0f', [], 'Display', None],
  [ 0x8f10, 'CjkUnifiedIdeograph8f10', [], 'Display', None],
  [ 0x8f11, 'CjkUnifiedIdeograph8f11', [], 'Display', None],
  [ 0x8f12, 'CjkUnifiedIdeograph8f12', [], 'Display', None],
  [ 0x8f13, 'CjkUnifiedIdeograph8f13', [], 'Display', None],
  [ 0x8f14, 'CjkUnifiedIdeograph8f14', [], 'Display', None],
  [ 0x8f15, 'CjkUnifiedIdeograph8f15', [], 'Display', None],
  [ 0x8f16, 'CjkUnifiedIdeograph8f16', [], 'Display', None],
  [ 0x8f17, 'CjkUnifiedIdeograph8f17', [], 'Display', None],
  [ 0x8f18, 'CjkUnifiedIdeograph8f18', [], 'Display', None],
  [ 0x8f19, 'CjkUnifiedIdeograph8f19', [], 'Display', None],
  [ 0x8f1a, 'CjkUnifiedIdeograph8f1a', [], 'Display', None],
  [ 0x8f1b, 'CjkUnifiedIdeograph8f1b', [], 'Display', None],
  [ 0x8f1c, 'CjkUnifiedIdeograph8f1c', [], 'Display', None],
  [ 0x8f1d, 'CjkUnifiedIdeograph8f1d', [], 'Display', None],
  [ 0x8f1e, 'CjkUnifiedIdeograph8f1e', [], 'Display', None],
  [ 0x8f1f, 'CjkUnifiedIdeograph8f1f', [], 'Display', None],
  [ 0x8f20, 'CjkUnifiedIdeograph8f20', [], 'Display', None],
  [ 0x8f21, 'CjkUnifiedIdeograph8f21', [], 'Display', None],
  [ 0x8f22, 'CjkUnifiedIdeograph8f22', [], 'Display', None],
  [ 0x8f23, 'CjkUnifiedIdeograph8f23', [], 'Display', None],
  [ 0x8f24, 'CjkUnifiedIdeograph8f24', [], 'Display', None],
  [ 0x8f25, 'CjkUnifiedIdeograph8f25', [], 'Display', None],
  [ 0x8f26, 'CjkUnifiedIdeograph8f26', [], 'Display', None],
  [ 0x8f27, 'CjkUnifiedIdeograph8f27', [], 'Display', None],
  [ 0x8f28, 'CjkUnifiedIdeograph8f28', [], 'Display', None],
  [ 0x8f29, 'CjkUnifiedIdeograph8f29', [], 'Display', None],
  [ 0x8f2a, 'CjkUnifiedIdeograph8f2a', [], 'Display', None],
  [ 0x8f2b, 'CjkUnifiedIdeograph8f2b', [], 'Display', None],
  [ 0x8f2c, 'CjkUnifiedIdeograph8f2c', [], 'Display', None],
  [ 0x8f2d, 'CjkUnifiedIdeograph8f2d', [], 'Display', None],
  [ 0x8f2e, 'CjkUnifiedIdeograph8f2e', [], 'Display', None],
  [ 0x8f2f, 'CjkUnifiedIdeograph8f2f', [], 'Display', None],
  [ 0x8f30, 'CjkUnifiedIdeograph8f30', [], 'Display', None],
  [ 0x8f31, 'CjkUnifiedIdeograph8f31', [], 'Display', None],
  [ 0x8f32, 'CjkUnifiedIdeograph8f32', [], 'Display', None],
  [ 0x8f33, 'CjkUnifiedIdeograph8f33', [], 'Display', None],
  [ 0x8f34, 'CjkUnifiedIdeograph8f34', [], 'Display', None],
  [ 0x8f35, 'CjkUnifiedIdeograph8f35', [], 'Display', None],
  [ 0x8f36, 'CjkUnifiedIdeograph8f36', [], 'Display', None],
  [ 0x8f37, 'CjkUnifiedIdeograph8f37', [], 'Display', None],
  [ 0x8f38, 'CjkUnifiedIdeograph8f38', [], 'Display', None],
  [ 0x8f39, 'CjkUnifiedIdeograph8f39', [], 'Display', None],
  [ 0x8f3a, 'CjkUnifiedIdeograph8f3a', [], 'Display', None],
  [ 0x8f3b, 'CjkUnifiedIdeograph8f3b', [], 'Display', None],
  [ 0x8f3c, 'CjkUnifiedIdeograph8f3c', [], 'Display', None],
  [ 0x8f3d, 'CjkUnifiedIdeograph8f3d', [], 'Display', None],
  [ 0x8f3e, 'CjkUnifiedIdeograph8f3e', [], 'Display', None],
  [ 0x8f3f, 'CjkUnifiedIdeograph8f3f', [], 'Display', None],
  [ 0x8f40, 'CjkUnifiedIdeograph8f40', [], 'Display', None],
  [ 0x8f41, 'CjkUnifiedIdeograph8f41', [], 'Display', None],
  [ 0x8f42, 'CjkUnifiedIdeograph8f42', [], 'Display', None],
  [ 0x8f43, 'CjkUnifiedIdeograph8f43', [], 'Display', None],
  [ 0x8f44, 'CjkUnifiedIdeograph8f44', [], 'Display', None],
  [ 0x8f45, 'CjkUnifiedIdeograph8f45', [], 'Display', None],
  [ 0x8f46, 'CjkUnifiedIdeograph8f46', [], 'Display', None],
  [ 0x8f47, 'CjkUnifiedIdeograph8f47', [], 'Display', None],
  [ 0x8f48, 'CjkUnifiedIdeograph8f48', [], 'Display', None],
  [ 0x8f49, 'CjkUnifiedIdeograph8f49', [], 'Display', None],
  [ 0x8f4a, 'CjkUnifiedIdeograph8f4a', [], 'Display', None],
  [ 0x8f4b, 'CjkUnifiedIdeograph8f4b', [], 'Display', None],
  [ 0x8f4c, 'CjkUnifiedIdeograph8f4c', [], 'Display', None],
  [ 0x8f4d, 'CjkUnifiedIdeograph8f4d', [], 'Display', None],
  [ 0x8f4e, 'CjkUnifiedIdeograph8f4e', [], 'Display', None],
  [ 0x8f4f, 'CjkUnifiedIdeograph8f4f', [], 'Display', None],
  [ 0x8f50, 'CjkUnifiedIdeograph8f50', [], 'Display', None],
  [ 0x8f51, 'CjkUnifiedIdeograph8f51', [], 'Display', None],
  [ 0x8f52, 'CjkUnifiedIdeograph8f52', [], 'Display', None],
  [ 0x8f53, 'CjkUnifiedIdeograph8f53', [], 'Display', None],
  [ 0x8f54, 'CjkUnifiedIdeograph8f54', [], 'Display', None],
  [ 0x8f55, 'CjkUnifiedIdeograph8f55', [], 'Display', None],
  [ 0x8f56, 'CjkUnifiedIdeograph8f56', [], 'Display', None],
  [ 0x8f57, 'CjkUnifiedIdeograph8f57', [], 'Display', None],
  [ 0x8f58, 'CjkUnifiedIdeograph8f58', [], 'Display', None],
  [ 0x8f59, 'CjkUnifiedIdeograph8f59', [], 'Display', None],
  [ 0x8f5a, 'CjkUnifiedIdeograph8f5a', [], 'Display', None],
  [ 0x8f5b, 'CjkUnifiedIdeograph8f5b', [], 'Display', None],
  [ 0x8f5c, 'CjkUnifiedIdeograph8f5c', [], 'Display', None],
  [ 0x8f5d, 'CjkUnifiedIdeograph8f5d', [], 'Display', None],
  [ 0x8f5e, 'CjkUnifiedIdeograph8f5e', [], 'Display', None],
  [ 0x8f5f, 'CjkUnifiedIdeograph8f5f', [], 'Display', None],
  [ 0x8f60, 'CjkUnifiedIdeograph8f60', [], 'Display', None],
  [ 0x8f61, 'CjkUnifiedIdeograph8f61', [], 'Display', None],
  [ 0x8f62, 'CjkUnifiedIdeograph8f62', [], 'Display', None],
  [ 0x8f63, 'CjkUnifiedIdeograph8f63', [], 'Display', None],
  [ 0x8f64, 'CjkUnifiedIdeograph8f64', [], 'Display', None],
  [ 0x8f65, 'CjkUnifiedIdeograph8f65', [], 'Display', None],
  [ 0x8f66, 'CjkUnifiedIdeograph8f66', [], 'Display', None],
  [ 0x8f67, 'CjkUnifiedIdeograph8f67', [], 'Display', None],
  [ 0x8f68, 'CjkUnifiedIdeograph8f68', [], 'Display', None],
  [ 0x8f69, 'CjkUnifiedIdeograph8f69', [], 'Display', None],
  [ 0x8f6a, 'CjkUnifiedIdeograph8f6a', [], 'Display', None],
  [ 0x8f6b, 'CjkUnifiedIdeograph8f6b', [], 'Display', None],
  [ 0x8f6c, 'CjkUnifiedIdeograph8f6c', [], 'Display', None],
  [ 0x8f6d, 'CjkUnifiedIdeograph8f6d', [], 'Display', None],
  [ 0x8f6e, 'CjkUnifiedIdeograph8f6e', [], 'Display', None],
  [ 0x8f6f, 'CjkUnifiedIdeograph8f6f', [], 'Display', None],
  [ 0x8f70, 'CjkUnifiedIdeograph8f70', [], 'Display', None],
  [ 0x8f71, 'CjkUnifiedIdeograph8f71', [], 'Display', None],
  [ 0x8f72, 'CjkUnifiedIdeograph8f72', [], 'Display', None],
  [ 0x8f73, 'CjkUnifiedIdeograph8f73', [], 'Display', None],
  [ 0x8f74, 'CjkUnifiedIdeograph8f74', [], 'Display', None],
  [ 0x8f75, 'CjkUnifiedIdeograph8f75', [], 'Display', None],
  [ 0x8f76, 'CjkUnifiedIdeograph8f76', [], 'Display', None],
  [ 0x8f77, 'CjkUnifiedIdeograph8f77', [], 'Display', None],
  [ 0x8f78, 'CjkUnifiedIdeograph8f78', [], 'Display', None],
  [ 0x8f79, 'CjkUnifiedIdeograph8f79', [], 'Display', None],
  [ 0x8f7a, 'CjkUnifiedIdeograph8f7a', [], 'Display', None],
  [ 0x8f7b, 'CjkUnifiedIdeograph8f7b', [], 'Display', None],
  [ 0x8f7c, 'CjkUnifiedIdeograph8f7c', [], 'Display', None],
  [ 0x8f7d, 'CjkUnifiedIdeograph8f7d', [], 'Display', None],
  [ 0x8f7e, 'CjkUnifiedIdeograph8f7e', [], 'Display', None],
  [ 0x8f7f, 'CjkUnifiedIdeograph8f7f', [], 'Display', None],
  [ 0x8f80, 'CjkUnifiedIdeograph8f80', [], 'Display', None],
  [ 0x8f81, 'CjkUnifiedIdeograph8f81', [], 'Display', None],
  [ 0x8f82, 'CjkUnifiedIdeograph8f82', [], 'Display', None],
  [ 0x8f83, 'CjkUnifiedIdeograph8f83', [], 'Display', None],
  [ 0x8f84, 'CjkUnifiedIdeograph8f84', [], 'Display', None],
  [ 0x8f85, 'CjkUnifiedIdeograph8f85', [], 'Display', None],
  [ 0x8f86, 'CjkUnifiedIdeograph8f86', [], 'Display', None],
  [ 0x8f87, 'CjkUnifiedIdeograph8f87', [], 'Display', None],
  [ 0x8f88, 'CjkUnifiedIdeograph8f88', [], 'Display', None],
  [ 0x8f89, 'CjkUnifiedIdeograph8f89', [], 'Display', None],
  [ 0x8f8a, 'CjkUnifiedIdeograph8f8a', [], 'Display', None],
  [ 0x8f8b, 'CjkUnifiedIdeograph8f8b', [], 'Display', None],
  [ 0x8f8c, 'CjkUnifiedIdeograph8f8c', [], 'Display', None],
  [ 0x8f8d, 'CjkUnifiedIdeograph8f8d', [], 'Display', None],
  [ 0x8f8e, 'CjkUnifiedIdeograph8f8e', [], 'Display', None],
  [ 0x8f8f, 'CjkUnifiedIdeograph8f8f', [], 'Display', None],
  [ 0x8f90, 'CjkUnifiedIdeograph8f90', [], 'Display', None],
  [ 0x8f91, 'CjkUnifiedIdeograph8f91', [], 'Display', None],
  [ 0x8f92, 'CjkUnifiedIdeograph8f92', [], 'Display', None],
  [ 0x8f93, 'CjkUnifiedIdeograph8f93', [], 'Display', None],
  [ 0x8f94, 'CjkUnifiedIdeograph8f94', [], 'Display', None],
  [ 0x8f95, 'CjkUnifiedIdeograph8f95', [], 'Display', None],
  [ 0x8f96, 'CjkUnifiedIdeograph8f96', [], 'Display', None],
  [ 0x8f97, 'CjkUnifiedIdeograph8f97', [], 'Display', None],
  [ 0x8f98, 'CjkUnifiedIdeograph8f98', [], 'Display', None],
  [ 0x8f99, 'CjkUnifiedIdeograph8f99', [], 'Display', None],
  [ 0x8f9a, 'CjkUnifiedIdeograph8f9a', [], 'Display', None],
  [ 0x8f9b, 'CjkUnifiedIdeograph8f9b', [], 'Display', None],
  [ 0x8f9c, 'CjkUnifiedIdeograph8f9c', [], 'Display', None],
  [ 0x8f9d, 'CjkUnifiedIdeograph8f9d', [], 'Display', None],
  [ 0x8f9e, 'CjkUnifiedIdeograph8f9e', [], 'Display', None],
  [ 0x8f9f, 'CjkUnifiedIdeograph8f9f', [], 'Display', None],
  [ 0x8fa0, 'CjkUnifiedIdeograph8fa0', [], 'Display', None],
  [ 0x8fa1, 'CjkUnifiedIdeograph8fa1', [], 'Display', None],
  [ 0x8fa2, 'CjkUnifiedIdeograph8fa2', [], 'Display', None],
  [ 0x8fa3, 'CjkUnifiedIdeograph8fa3', [], 'Display', None],
  [ 0x8fa4, 'CjkUnifiedIdeograph8fa4', [], 'Display', None],
  [ 0x8fa5, 'CjkUnifiedIdeograph8fa5', [], 'Display', None],
  [ 0x8fa6, 'CjkUnifiedIdeograph8fa6', [], 'Display', None],
  [ 0x8fa7, 'CjkUnifiedIdeograph8fa7', [], 'Display', None],
  [ 0x8fa8, 'CjkUnifiedIdeograph8fa8', [], 'Display', None],
  [ 0x8fa9, 'CjkUnifiedIdeograph8fa9', [], 'Display', None],
  [ 0x8faa, 'CjkUnifiedIdeograph8faa', [], 'Display', None],
  [ 0x8fab, 'CjkUnifiedIdeograph8fab', [], 'Display', None],
  [ 0x8fac, 'CjkUnifiedIdeograph8fac', [], 'Display', None],
  [ 0x8fad, 'CjkUnifiedIdeograph8fad', [], 'Display', None],
  [ 0x8fae, 'CjkUnifiedIdeograph8fae', [], 'Display', None],
  [ 0x8faf, 'CjkUnifiedIdeograph8faf', [], 'Display', None],
  [ 0x8fb0, 'CjkUnifiedIdeograph8fb0', [], 'Display', None],
  [ 0x8fb1, 'CjkUnifiedIdeograph8fb1', [], 'Display', None],
  [ 0x8fb2, 'CjkUnifiedIdeograph8fb2', [], 'Display', None],
  [ 0x8fb3, 'CjkUnifiedIdeograph8fb3', [], 'Display', None],
  [ 0x8fb4, 'CjkUnifiedIdeograph8fb4', [], 'Display', None],
  [ 0x8fb5, 'CjkUnifiedIdeograph8fb5', [], 'Display', None],
  [ 0x8fb6, 'CjkUnifiedIdeograph8fb6', [], 'Display', None],
  [ 0x8fb7, 'CjkUnifiedIdeograph8fb7', [], 'Display', None],
  [ 0x8fb8, 'CjkUnifiedIdeograph8fb8', [], 'Display', None],
  [ 0x8fb9, 'CjkUnifiedIdeograph8fb9', [], 'Display', None],
  [ 0x8fba, 'CjkUnifiedIdeograph8fba', [], 'Display', None],
  [ 0x8fbb, 'CjkUnifiedIdeograph8fbb', [], 'Display', None],
  [ 0x8fbc, 'CjkUnifiedIdeograph8fbc', [], 'Display', None],
  [ 0x8fbd, 'CjkUnifiedIdeograph8fbd', [], 'Display', None],
  [ 0x8fbe, 'CjkUnifiedIdeograph8fbe', [], 'Display', None],
  [ 0x8fbf, 'CjkUnifiedIdeograph8fbf', [], 'Display', None],
  [ 0x8fc0, 'CjkUnifiedIdeograph8fc0', [], 'Display', None],
  [ 0x8fc1, 'CjkUnifiedIdeograph8fc1', [], 'Display', None],
  [ 0x8fc2, 'CjkUnifiedIdeograph8fc2', [], 'Display', None],
  [ 0x8fc3, 'CjkUnifiedIdeograph8fc3', [], 'Display', None],
  [ 0x8fc4, 'CjkUnifiedIdeograph8fc4', [], 'Display', None],
  [ 0x8fc5, 'CjkUnifiedIdeograph8fc5', [], 'Display', None],
  [ 0x8fc6, 'CjkUnifiedIdeograph8fc6', [], 'Display', None],
  [ 0x8fc7, 'CjkUnifiedIdeograph8fc7', [], 'Display', None],
  [ 0x8fc8, 'CjkUnifiedIdeograph8fc8', [], 'Display', None],
  [ 0x8fc9, 'CjkUnifiedIdeograph8fc9', [], 'Display', None],
  [ 0x8fca, 'CjkUnifiedIdeograph8fca', [], 'Display', None],
  [ 0x8fcb, 'CjkUnifiedIdeograph8fcb', [], 'Display', None],
  [ 0x8fcc, 'CjkUnifiedIdeograph8fcc', [], 'Display', None],
  [ 0x8fcd, 'CjkUnifiedIdeograph8fcd', [], 'Display', None],
  [ 0x8fce, 'CjkUnifiedIdeograph8fce', [], 'Display', None],
  [ 0x8fcf, 'CjkUnifiedIdeograph8fcf', [], 'Display', None],
  [ 0x8fd0, 'CjkUnifiedIdeograph8fd0', [], 'Display', None],
  [ 0x8fd1, 'CjkUnifiedIdeograph8fd1', [], 'Display', None],
  [ 0x8fd2, 'CjkUnifiedIdeograph8fd2', [], 'Display', None],
  [ 0x8fd3, 'CjkUnifiedIdeograph8fd3', [], 'Display', None],
  [ 0x8fd4, 'CjkUnifiedIdeograph8fd4', [], 'Display', None],
  [ 0x8fd5, 'CjkUnifiedIdeograph8fd5', [], 'Display', None],
  [ 0x8fd6, 'CjkUnifiedIdeograph8fd6', [], 'Display', None],
  [ 0x8fd7, 'CjkUnifiedIdeograph8fd7', [], 'Display', None],
  [ 0x8fd8, 'CjkUnifiedIdeograph8fd8', [], 'Display', None],
  [ 0x8fd9, 'CjkUnifiedIdeograph8fd9', [], 'Display', None],
  [ 0x8fda, 'CjkUnifiedIdeograph8fda', [], 'Display', None],
  [ 0x8fdb, 'CjkUnifiedIdeograph8fdb', [], 'Display', None],
  [ 0x8fdc, 'CjkUnifiedIdeograph8fdc', [], 'Display', None],
  [ 0x8fdd, 'CjkUnifiedIdeograph8fdd', [], 'Display', None],
  [ 0x8fde, 'CjkUnifiedIdeograph8fde', [], 'Display', None],
  [ 0x8fdf, 'CjkUnifiedIdeograph8fdf', [], 'Display', None],
  [ 0x8fe0, 'CjkUnifiedIdeograph8fe0', [], 'Display', None],
  [ 0x8fe1, 'CjkUnifiedIdeograph8fe1', [], 'Display', None],
  [ 0x8fe2, 'CjkUnifiedIdeograph8fe2', [], 'Display', None],
  [ 0x8fe3, 'CjkUnifiedIdeograph8fe3', [], 'Display', None],
  [ 0x8fe4, 'CjkUnifiedIdeograph8fe4', [], 'Display', None],
  [ 0x8fe5, 'CjkUnifiedIdeograph8fe5', [], 'Display', None],
  [ 0x8fe6, 'CjkUnifiedIdeograph8fe6', [], 'Display', None],
  [ 0x8fe7, 'CjkUnifiedIdeograph8fe7', [], 'Display', None],
  [ 0x8fe8, 'CjkUnifiedIdeograph8fe8', [], 'Display', None],
  [ 0x8fe9, 'CjkUnifiedIdeograph8fe9', [], 'Display', None],
  [ 0x8fea, 'CjkUnifiedIdeograph8fea', [], 'Display', None],
  [ 0x8feb, 'CjkUnifiedIdeograph8feb', [], 'Display', None],
  [ 0x8fec, 'CjkUnifiedIdeograph8fec', [], 'Display', None],
  [ 0x8fed, 'CjkUnifiedIdeograph8fed', [], 'Display', None],
  [ 0x8fee, 'CjkUnifiedIdeograph8fee', [], 'Display', None],
  [ 0x8fef, 'CjkUnifiedIdeograph8fef', [], 'Display', None],
  [ 0x8ff0, 'CjkUnifiedIdeograph8ff0', [], 'Display', None],
  [ 0x8ff1, 'CjkUnifiedIdeograph8ff1', [], 'Display', None],
  [ 0x8ff2, 'CjkUnifiedIdeograph8ff2', [], 'Display', None],
  [ 0x8ff3, 'CjkUnifiedIdeograph8ff3', [], 'Display', None],
  [ 0x8ff4, 'CjkUnifiedIdeograph8ff4', [], 'Display', None],
  [ 0x8ff5, 'CjkUnifiedIdeograph8ff5', [], 'Display', None],
  [ 0x8ff6, 'CjkUnifiedIdeograph8ff6', [], 'Display', None],
  [ 0x8ff7, 'CjkUnifiedIdeograph8ff7', [], 'Display', None],
  [ 0x8ff8, 'CjkUnifiedIdeograph8ff8', [], 'Display', None],
  [ 0x8ff9, 'CjkUnifiedIdeograph8ff9', [], 'Display', None],
  [ 0x8ffa, 'CjkUnifiedIdeograph8ffa', [], 'Display', None],
  [ 0x8ffb, 'CjkUnifiedIdeograph8ffb', [], 'Display', None],
  [ 0x8ffc, 'CjkUnifiedIdeograph8ffc', [], 'Display', None],
  [ 0x8ffd, 'CjkUnifiedIdeograph8ffd', [], 'Display', None],
  [ 0x8ffe, 'CjkUnifiedIdeograph8ffe', [], 'Display', None],
  [ 0x8fff, 'CjkUnifiedIdeograph8fff', [], 'Display', None],
  [ 0x9000, 'CjkUnifiedIdeograph9000', [], 'Display', None],
  [ 0x9001, 'CjkUnifiedIdeograph9001', [], 'Display', None],
  [ 0x9002, 'CjkUnifiedIdeograph9002', [], 'Display', None],
  [ 0x9003, 'CjkUnifiedIdeograph9003', [], 'Display', None],
  [ 0x9004, 'CjkUnifiedIdeograph9004', [], 'Display', None],
  [ 0x9005, 'CjkUnifiedIdeograph9005', [], 'Display', None],
  [ 0x9006, 'CjkUnifiedIdeograph9006', [], 'Display', None],
  [ 0x9007, 'CjkUnifiedIdeograph9007', [], 'Display', None],
  [ 0x9008, 'CjkUnifiedIdeograph9008', [], 'Display', None],
  [ 0x9009, 'CjkUnifiedIdeograph9009', [], 'Display', None],
  [ 0x900a, 'CjkUnifiedIdeograph900a', [], 'Display', None],
  [ 0x900b, 'CjkUnifiedIdeograph900b', [], 'Display', None],
  [ 0x900c, 'CjkUnifiedIdeograph900c', [], 'Display', None],
  [ 0x900d, 'CjkUnifiedIdeograph900d', [], 'Display', None],
  [ 0x900e, 'CjkUnifiedIdeograph900e', [], 'Display', None],
  [ 0x900f, 'CjkUnifiedIdeograph900f', [], 'Display', None],
  [ 0x9010, 'CjkUnifiedIdeograph9010', [], 'Display', None],
  [ 0x9011, 'CjkUnifiedIdeograph9011', [], 'Display', None],
  [ 0x9012, 'CjkUnifiedIdeograph9012', [], 'Display', None],
  [ 0x9013, 'CjkUnifiedIdeograph9013', [], 'Display', None],
  [ 0x9014, 'CjkUnifiedIdeograph9014', [], 'Display', None],
  [ 0x9015, 'CjkUnifiedIdeograph9015', [], 'Display', None],
  [ 0x9016, 'CjkUnifiedIdeograph9016', [], 'Display', None],
  [ 0x9017, 'CjkUnifiedIdeograph9017', [], 'Display', None],
  [ 0x9018, 'CjkUnifiedIdeograph9018', [], 'Display', None],
  [ 0x9019, 'CjkUnifiedIdeograph9019', [], 'Display', None],
  [ 0x901a, 'CjkUnifiedIdeograph901a', [], 'Display', None],
  [ 0x901b, 'CjkUnifiedIdeograph901b', [], 'Display', None],
  [ 0x901c, 'CjkUnifiedIdeograph901c', [], 'Display', None],
  [ 0x901d, 'CjkUnifiedIdeograph901d', [], 'Display', None],
  [ 0x901e, 'CjkUnifiedIdeograph901e', [], 'Display', None],
  [ 0x901f, 'CjkUnifiedIdeograph901f', [], 'Display', None],
  [ 0x9020, 'CjkUnifiedIdeograph9020', [], 'Display', None],
  [ 0x9021, 'CjkUnifiedIdeograph9021', [], 'Display', None],
  [ 0x9022, 'CjkUnifiedIdeograph9022', [], 'Display', None],
  [ 0x9023, 'CjkUnifiedIdeograph9023', [], 'Display', None],
  [ 0x9024, 'CjkUnifiedIdeograph9024', [], 'Display', None],
  [ 0x9025, 'CjkUnifiedIdeograph9025', [], 'Display', None],
  [ 0x9026, 'CjkUnifiedIdeograph9026', [], 'Display', None],
  [ 0x9027, 'CjkUnifiedIdeograph9027', [], 'Display', None],
  [ 0x9028, 'CjkUnifiedIdeograph9028', [], 'Display', None],
  [ 0x9029, 'CjkUnifiedIdeograph9029', [], 'Display', None],
  [ 0x902a, 'CjkUnifiedIdeograph902a', [], 'Display', None],
  [ 0x902b, 'CjkUnifiedIdeograph902b', [], 'Display', None],
  [ 0x902c, 'CjkUnifiedIdeograph902c', [], 'Display', None],
  [ 0x902d, 'CjkUnifiedIdeograph902d', [], 'Display', None],
  [ 0x902e, 'CjkUnifiedIdeograph902e', [], 'Display', None],
  [ 0x902f, 'CjkUnifiedIdeograph902f', [], 'Display', None],
  [ 0x9030, 'CjkUnifiedIdeograph9030', [], 'Display', None],
  [ 0x9031, 'CjkUnifiedIdeograph9031', [], 'Display', None],
  [ 0x9032, 'CjkUnifiedIdeograph9032', [], 'Display', None],
  [ 0x9033, 'CjkUnifiedIdeograph9033', [], 'Display', None],
  [ 0x9034, 'CjkUnifiedIdeograph9034', [], 'Display', None],
  [ 0x9035, 'CjkUnifiedIdeograph9035', [], 'Display', None],
  [ 0x9036, 'CjkUnifiedIdeograph9036', [], 'Display', None],
  [ 0x9037, 'CjkUnifiedIdeograph9037', [], 'Display', None],
  [ 0x9038, 'CjkUnifiedIdeograph9038', [], 'Display', None],
  [ 0x9039, 'CjkUnifiedIdeograph9039', [], 'Display', None],
  [ 0x903a, 'CjkUnifiedIdeograph903a', [], 'Display', None],
  [ 0x903b, 'CjkUnifiedIdeograph903b', [], 'Display', None],
  [ 0x903c, 'CjkUnifiedIdeograph903c', [], 'Display', None],
  [ 0x903d, 'CjkUnifiedIdeograph903d', [], 'Display', None],
  [ 0x903e, 'CjkUnifiedIdeograph903e', [], 'Display', None],
  [ 0x903f, 'CjkUnifiedIdeograph903f', [], 'Display', None],
  [ 0x9040, 'CjkUnifiedIdeograph9040', [], 'Display', None],
  [ 0x9041, 'CjkUnifiedIdeograph9041', [], 'Display', None],
  [ 0x9042, 'CjkUnifiedIdeograph9042', [], 'Display', None],
  [ 0x9043, 'CjkUnifiedIdeograph9043', [], 'Display', None],
  [ 0x9044, 'CjkUnifiedIdeograph9044', [], 'Display', None],
  [ 0x9045, 'CjkUnifiedIdeograph9045', [], 'Display', None],
  [ 0x9046, 'CjkUnifiedIdeograph9046', [], 'Display', None],
  [ 0x9047, 'CjkUnifiedIdeograph9047', [], 'Display', None],
  [ 0x9048, 'CjkUnifiedIdeograph9048', [], 'Display', None],
  [ 0x9049, 'CjkUnifiedIdeograph9049', [], 'Display', None],
  [ 0x904a, 'CjkUnifiedIdeograph904a', [], 'Display', None],
  [ 0x904b, 'CjkUnifiedIdeograph904b', [], 'Display', None],
  [ 0x904c, 'CjkUnifiedIdeograph904c', [], 'Display', None],
  [ 0x904d, 'CjkUnifiedIdeograph904d', [], 'Display', None],
  [ 0x904e, 'CjkUnifiedIdeograph904e', [], 'Display', None],
  [ 0x904f, 'CjkUnifiedIdeograph904f', [], 'Display', None],
  [ 0x9050, 'CjkUnifiedIdeograph9050', [], 'Display', None],
  [ 0x9051, 'CjkUnifiedIdeograph9051', [], 'Display', None],
  [ 0x9052, 'CjkUnifiedIdeograph9052', [], 'Display', None],
  [ 0x9053, 'CjkUnifiedIdeograph9053', [], 'Display', None],
  [ 0x9054, 'CjkUnifiedIdeograph9054', [], 'Display', None],
  [ 0x9055, 'CjkUnifiedIdeograph9055', [], 'Display', None],
  [ 0x9056, 'CjkUnifiedIdeograph9056', [], 'Display', None],
  [ 0x9057, 'CjkUnifiedIdeograph9057', [], 'Display', None],
  [ 0x9058, 'CjkUnifiedIdeograph9058', [], 'Display', None],
  [ 0x9059, 'CjkUnifiedIdeograph9059', [], 'Display', None],
  [ 0x905a, 'CjkUnifiedIdeograph905a', [], 'Display', None],
  [ 0x905b, 'CjkUnifiedIdeograph905b', [], 'Display', None],
  [ 0x905c, 'CjkUnifiedIdeograph905c', [], 'Display', None],
  [ 0x905d, 'CjkUnifiedIdeograph905d', [], 'Display', None],
  [ 0x905e, 'CjkUnifiedIdeograph905e', [], 'Display', None],
  [ 0x905f, 'CjkUnifiedIdeograph905f', [], 'Display', None],
  [ 0x9060, 'CjkUnifiedIdeograph9060', [], 'Display', None],
  [ 0x9061, 'CjkUnifiedIdeograph9061', [], 'Display', None],
  [ 0x9062, 'CjkUnifiedIdeograph9062', [], 'Display', None],
  [ 0x9063, 'CjkUnifiedIdeograph9063', [], 'Display', None],
  [ 0x9064, 'CjkUnifiedIdeograph9064', [], 'Display', None],
  [ 0x9065, 'CjkUnifiedIdeograph9065', [], 'Display', None],
  [ 0x9066, 'CjkUnifiedIdeograph9066', [], 'Display', None],
  [ 0x9067, 'CjkUnifiedIdeograph9067', [], 'Display', None],
  [ 0x9068, 'CjkUnifiedIdeograph9068', [], 'Display', None],
  [ 0x9069, 'CjkUnifiedIdeograph9069', [], 'Display', None],
  [ 0x906a, 'CjkUnifiedIdeograph906a', [], 'Display', None],
  [ 0x906b, 'CjkUnifiedIdeograph906b', [], 'Display', None],
  [ 0x906c, 'CjkUnifiedIdeograph906c', [], 'Display', None],
  [ 0x906d, 'CjkUnifiedIdeograph906d', [], 'Display', None],
  [ 0x906e, 'CjkUnifiedIdeograph906e', [], 'Display', None],
  [ 0x906f, 'CjkUnifiedIdeograph906f', [], 'Display', None],
  [ 0x9070, 'CjkUnifiedIdeograph9070', [], 'Display', None],
  [ 0x9071, 'CjkUnifiedIdeograph9071', [], 'Display', None],
  [ 0x9072, 'CjkUnifiedIdeograph9072', [], 'Display', None],
  [ 0x9073, 'CjkUnifiedIdeograph9073', [], 'Display', None],
  [ 0x9074, 'CjkUnifiedIdeograph9074', [], 'Display', None],
  [ 0x9075, 'CjkUnifiedIdeograph9075', [], 'Display', None],
  [ 0x9076, 'CjkUnifiedIdeograph9076', [], 'Display', None],
  [ 0x9077, 'CjkUnifiedIdeograph9077', [], 'Display', None],
  [ 0x9078, 'CjkUnifiedIdeograph9078', [], 'Display', None],
  [ 0x9079, 'CjkUnifiedIdeograph9079', [], 'Display', None],
  [ 0x907a, 'CjkUnifiedIdeograph907a', [], 'Display', None],
  [ 0x907b, 'CjkUnifiedIdeograph907b', [], 'Display', None],
  [ 0x907c, 'CjkUnifiedIdeograph907c', [], 'Display', None],
  [ 0x907d, 'CjkUnifiedIdeograph907d', [], 'Display', None],
  [ 0x907e, 'CjkUnifiedIdeograph907e', [], 'Display', None],
  [ 0x907f, 'CjkUnifiedIdeograph907f', [], 'Display', None],
  [ 0x9080, 'CjkUnifiedIdeograph9080', [], 'Display', None],
  [ 0x9081, 'CjkUnifiedIdeograph9081', [], 'Display', None],
  [ 0x9082, 'CjkUnifiedIdeograph9082', [], 'Display', None],
  [ 0x9083, 'CjkUnifiedIdeograph9083', [], 'Display', None],
  [ 0x9084, 'CjkUnifiedIdeograph9084', [], 'Display', None],
  [ 0x9085, 'CjkUnifiedIdeograph9085', [], 'Display', None],
  [ 0x9086, 'CjkUnifiedIdeograph9086', [], 'Display', None],
  [ 0x9087, 'CjkUnifiedIdeograph9087', [], 'Display', None],
  [ 0x9088, 'CjkUnifiedIdeograph9088', [], 'Display', None],
  [ 0x9089, 'CjkUnifiedIdeograph9089', [], 'Display', None],
  [ 0x908a, 'CjkUnifiedIdeograph908a', [], 'Display', None],
  [ 0x908b, 'CjkUnifiedIdeograph908b', [], 'Display', None],
  [ 0x908c, 'CjkUnifiedIdeograph908c', [], 'Display', None],
  [ 0x908d, 'CjkUnifiedIdeograph908d', [], 'Display', None],
  [ 0x908e, 'CjkUnifiedIdeograph908e', [], 'Display', None],
  [ 0x908f, 'CjkUnifiedIdeograph908f', [], 'Display', None],
  [ 0x9090, 'CjkUnifiedIdeograph9090', [], 'Display', None],
  [ 0x9091, 'CjkUnifiedIdeograph9091', [], 'Display', None],
  [ 0x9092, 'CjkUnifiedIdeograph9092', [], 'Display', None],
  [ 0x9093, 'CjkUnifiedIdeograph9093', [], 'Display', None],
  [ 0x9094, 'CjkUnifiedIdeograph9094', [], 'Display', None],
  [ 0x9095, 'CjkUnifiedIdeograph9095', [], 'Display', None],
  [ 0x9096, 'CjkUnifiedIdeograph9096', [], 'Display', None],
  [ 0x9097, 'CjkUnifiedIdeograph9097', [], 'Display', None],
  [ 0x9098, 'CjkUnifiedIdeograph9098', [], 'Display', None],
  [ 0x9099, 'CjkUnifiedIdeograph9099', [], 'Display', None],
  [ 0x909a, 'CjkUnifiedIdeograph909a', [], 'Display', None],
  [ 0x909b, 'CjkUnifiedIdeograph909b', [], 'Display', None],
  [ 0x909c, 'CjkUnifiedIdeograph909c', [], 'Display', None],
  [ 0x909d, 'CjkUnifiedIdeograph909d', [], 'Display', None],
  [ 0x909e, 'CjkUnifiedIdeograph909e', [], 'Display', None],
  [ 0x909f, 'CjkUnifiedIdeograph909f', [], 'Display', None],
  [ 0x90a0, 'CjkUnifiedIdeograph90a0', [], 'Display', None],
  [ 0x90a1, 'CjkUnifiedIdeograph90a1', [], 'Display', None],
  [ 0x90a2, 'CjkUnifiedIdeograph90a2', [], 'Display', None],
  [ 0x90a3, 'CjkUnifiedIdeograph90a3', [], 'Display', None],
  [ 0x90a4, 'CjkUnifiedIdeograph90a4', [], 'Display', None],
  [ 0x90a5, 'CjkUnifiedIdeograph90a5', [], 'Display', None],
  [ 0x90a6, 'CjkUnifiedIdeograph90a6', [], 'Display', None],
  [ 0x90a7, 'CjkUnifiedIdeograph90a7', [], 'Display', None],
  [ 0x90a8, 'CjkUnifiedIdeograph90a8', [], 'Display', None],
  [ 0x90a9, 'CjkUnifiedIdeograph90a9', [], 'Display', None],
  [ 0x90aa, 'CjkUnifiedIdeograph90aa', [], 'Display', None],
  [ 0x90ab, 'CjkUnifiedIdeograph90ab', [], 'Display', None],
  [ 0x90ac, 'CjkUnifiedIdeograph90ac', [], 'Display', None],
  [ 0x90ad, 'CjkUnifiedIdeograph90ad', [], 'Display', None],
  [ 0x90ae, 'CjkUnifiedIdeograph90ae', [], 'Display', None],
  [ 0x90af, 'CjkUnifiedIdeograph90af', [], 'Display', None],
  [ 0x90b0, 'CjkUnifiedIdeograph90b0', [], 'Display', None],
  [ 0x90b1, 'CjkUnifiedIdeograph90b1', [], 'Display', None],
  [ 0x90b2, 'CjkUnifiedIdeograph90b2', [], 'Display', None],
  [ 0x90b3, 'CjkUnifiedIdeograph90b3', [], 'Display', None],
  [ 0x90b4, 'CjkUnifiedIdeograph90b4', [], 'Display', None],
  [ 0x90b5, 'CjkUnifiedIdeograph90b5', [], 'Display', None],
  [ 0x90b6, 'CjkUnifiedIdeograph90b6', [], 'Display', None],
  [ 0x90b7, 'CjkUnifiedIdeograph90b7', [], 'Display', None],
  [ 0x90b8, 'CjkUnifiedIdeograph90b8', [], 'Display', None],
  [ 0x90b9, 'CjkUnifiedIdeograph90b9', [], 'Display', None],
  [ 0x90ba, 'CjkUnifiedIdeograph90ba', [], 'Display', None],
  [ 0x90bb, 'CjkUnifiedIdeograph90bb', [], 'Display', None],
  [ 0x90bc, 'CjkUnifiedIdeograph90bc', [], 'Display', None],
  [ 0x90bd, 'CjkUnifiedIdeograph90bd', [], 'Display', None],
  [ 0x90be, 'CjkUnifiedIdeograph90be', [], 'Display', None],
  [ 0x90bf, 'CjkUnifiedIdeograph90bf', [], 'Display', None],
  [ 0x90c0, 'CjkUnifiedIdeograph90c0', [], 'Display', None],
  [ 0x90c1, 'CjkUnifiedIdeograph90c1', [], 'Display', None],
  [ 0x90c2, 'CjkUnifiedIdeograph90c2', [], 'Display', None],
  [ 0x90c3, 'CjkUnifiedIdeograph90c3', [], 'Display', None],
  [ 0x90c4, 'CjkUnifiedIdeograph90c4', [], 'Display', None],
  [ 0x90c5, 'CjkUnifiedIdeograph90c5', [], 'Display', None],
  [ 0x90c6, 'CjkUnifiedIdeograph90c6', [], 'Display', None],
  [ 0x90c7, 'CjkUnifiedIdeograph90c7', [], 'Display', None],
  [ 0x90c8, 'CjkUnifiedIdeograph90c8', [], 'Display', None],
  [ 0x90c9, 'CjkUnifiedIdeograph90c9', [], 'Display', None],
  [ 0x90ca, 'CjkUnifiedIdeograph90ca', [], 'Display', None],
  [ 0x90cb, 'CjkUnifiedIdeograph90cb', [], 'Display', None],
  [ 0x90cc, 'CjkUnifiedIdeograph90cc', [], 'Display', None],
  [ 0x90cd, 'CjkUnifiedIdeograph90cd', [], 'Display', None],
  [ 0x90ce, 'CjkUnifiedIdeograph90ce', [], 'Display', None],
  [ 0x90cf, 'CjkUnifiedIdeograph90cf', [], 'Display', None],
  [ 0x90d0, 'CjkUnifiedIdeograph90d0', [], 'Display', None],
  [ 0x90d1, 'CjkUnifiedIdeograph90d1', [], 'Display', None],
  [ 0x90d2, 'CjkUnifiedIdeograph90d2', [], 'Display', None],
  [ 0x90d3, 'CjkUnifiedIdeograph90d3', [], 'Display', None],
  [ 0x90d4, 'CjkUnifiedIdeograph90d4', [], 'Display', None],
  [ 0x90d5, 'CjkUnifiedIdeograph90d5', [], 'Display', None],
  [ 0x90d6, 'CjkUnifiedIdeograph90d6', [], 'Display', None],
  [ 0x90d7, 'CjkUnifiedIdeograph90d7', [], 'Display', None],
  [ 0x90d8, 'CjkUnifiedIdeograph90d8', [], 'Display', None],
  [ 0x90d9, 'CjkUnifiedIdeograph90d9', [], 'Display', None],
  [ 0x90da, 'CjkUnifiedIdeograph90da', [], 'Display', None],
  [ 0x90db, 'CjkUnifiedIdeograph90db', [], 'Display', None],
  [ 0x90dc, 'CjkUnifiedIdeograph90dc', [], 'Display', None],
  [ 0x90dd, 'CjkUnifiedIdeograph90dd', [], 'Display', None],
  [ 0x90de, 'CjkUnifiedIdeograph90de', [], 'Display', None],
  [ 0x90df, 'CjkUnifiedIdeograph90df', [], 'Display', None],
  [ 0x90e0, 'CjkUnifiedIdeograph90e0', [], 'Display', None],
  [ 0x90e1, 'CjkUnifiedIdeograph90e1', [], 'Display', None],
  [ 0x90e2, 'CjkUnifiedIdeograph90e2', [], 'Display', None],
  [ 0x90e3, 'CjkUnifiedIdeograph90e3', [], 'Display', None],
  [ 0x90e4, 'CjkUnifiedIdeograph90e4', [], 'Display', None],
  [ 0x90e5, 'CjkUnifiedIdeograph90e5', [], 'Display', None],
  [ 0x90e6, 'CjkUnifiedIdeograph90e6', [], 'Display', None],
  [ 0x90e7, 'CjkUnifiedIdeograph90e7', [], 'Display', None],
  [ 0x90e8, 'CjkUnifiedIdeograph90e8', [], 'Display', None],
  [ 0x90e9, 'CjkUnifiedIdeograph90e9', [], 'Display', None],
  [ 0x90ea, 'CjkUnifiedIdeograph90ea', [], 'Display', None],
  [ 0x90eb, 'CjkUnifiedIdeograph90eb', [], 'Display', None],
  [ 0x90ec, 'CjkUnifiedIdeograph90ec', [], 'Display', None],
  [ 0x90ed, 'CjkUnifiedIdeograph90ed', [], 'Display', None],
  [ 0x90ee, 'CjkUnifiedIdeograph90ee', [], 'Display', None],
  [ 0x90ef, 'CjkUnifiedIdeograph90ef', [], 'Display', None],
  [ 0x90f0, 'CjkUnifiedIdeograph90f0', [], 'Display', None],
  [ 0x90f1, 'CjkUnifiedIdeograph90f1', [], 'Display', None],
  [ 0x90f2, 'CjkUnifiedIdeograph90f2', [], 'Display', None],
  [ 0x90f3, 'CjkUnifiedIdeograph90f3', [], 'Display', None],
  [ 0x90f4, 'CjkUnifiedIdeograph90f4', [], 'Display', None],
  [ 0x90f5, 'CjkUnifiedIdeograph90f5', [], 'Display', None],
  [ 0x90f6, 'CjkUnifiedIdeograph90f6', [], 'Display', None],
  [ 0x90f7, 'CjkUnifiedIdeograph90f7', [], 'Display', None],
  [ 0x90f8, 'CjkUnifiedIdeograph90f8', [], 'Display', None],
  [ 0x90f9, 'CjkUnifiedIdeograph90f9', [], 'Display', None],
  [ 0x90fa, 'CjkUnifiedIdeograph90fa', [], 'Display', None],
  [ 0x90fb, 'CjkUnifiedIdeograph90fb', [], 'Display', None],
  [ 0x90fc, 'CjkUnifiedIdeograph90fc', [], 'Display', None],
  [ 0x90fd, 'CjkUnifiedIdeograph90fd', [], 'Display', None],
  [ 0x90fe, 'CjkUnifiedIdeograph90fe', [], 'Display', None],
  [ 0x90ff, 'CjkUnifiedIdeograph90ff', [], 'Display', None],
  [ 0x9100, 'CjkUnifiedIdeograph9100', [], 'Display', None],
  [ 0x9101, 'CjkUnifiedIdeograph9101', [], 'Display', None],
  [ 0x9102, 'CjkUnifiedIdeograph9102', [], 'Display', None],
  [ 0x9103, 'CjkUnifiedIdeograph9103', [], 'Display', None],
  [ 0x9104, 'CjkUnifiedIdeograph9104', [], 'Display', None],
  [ 0x9105, 'CjkUnifiedIdeograph9105', [], 'Display', None],
  [ 0x9106, 'CjkUnifiedIdeograph9106', [], 'Display', None],
  [ 0x9107, 'CjkUnifiedIdeograph9107', [], 'Display', None],
  [ 0x9108, 'CjkUnifiedIdeograph9108', [], 'Display', None],
  [ 0x9109, 'CjkUnifiedIdeograph9109', [], 'Display', None],
  [ 0x910a, 'CjkUnifiedIdeograph910a', [], 'Display', None],
  [ 0x910b, 'CjkUnifiedIdeograph910b', [], 'Display', None],
  [ 0x910c, 'CjkUnifiedIdeograph910c', [], 'Display', None],
  [ 0x910d, 'CjkUnifiedIdeograph910d', [], 'Display', None],
  [ 0x910e, 'CjkUnifiedIdeograph910e', [], 'Display', None],
  [ 0x910f, 'CjkUnifiedIdeograph910f', [], 'Display', None],
  [ 0x9110, 'CjkUnifiedIdeograph9110', [], 'Display', None],
  [ 0x9111, 'CjkUnifiedIdeograph9111', [], 'Display', None],
  [ 0x9112, 'CjkUnifiedIdeograph9112', [], 'Display', None],
  [ 0x9113, 'CjkUnifiedIdeograph9113', [], 'Display', None],
  [ 0x9114, 'CjkUnifiedIdeograph9114', [], 'Display', None],
  [ 0x9115, 'CjkUnifiedIdeograph9115', [], 'Display', None],
  [ 0x9116, 'CjkUnifiedIdeograph9116', [], 'Display', None],
  [ 0x9117, 'CjkUnifiedIdeograph9117', [], 'Display', None],
  [ 0x9118, 'CjkUnifiedIdeograph9118', [], 'Display', None],
  [ 0x9119, 'CjkUnifiedIdeograph9119', [], 'Display', None],
  [ 0x911a, 'CjkUnifiedIdeograph911a', [], 'Display', None],
  [ 0x911b, 'CjkUnifiedIdeograph911b', [], 'Display', None],
  [ 0x911c, 'CjkUnifiedIdeograph911c', [], 'Display', None],
  [ 0x911d, 'CjkUnifiedIdeograph911d', [], 'Display', None],
  [ 0x911e, 'CjkUnifiedIdeograph911e', [], 'Display', None],
  [ 0x911f, 'CjkUnifiedIdeograph911f', [], 'Display', None],
  [ 0x9120, 'CjkUnifiedIdeograph9120', [], 'Display', None],
  [ 0x9121, 'CjkUnifiedIdeograph9121', [], 'Display', None],
  [ 0x9122, 'CjkUnifiedIdeograph9122', [], 'Display', None],
  [ 0x9123, 'CjkUnifiedIdeograph9123', [], 'Display', None],
  [ 0x9124, 'CjkUnifiedIdeograph9124', [], 'Display', None],
  [ 0x9125, 'CjkUnifiedIdeograph9125', [], 'Display', None],
  [ 0x9126, 'CjkUnifiedIdeograph9126', [], 'Display', None],
  [ 0x9127, 'CjkUnifiedIdeograph9127', [], 'Display', None],
  [ 0x9128, 'CjkUnifiedIdeograph9128', [], 'Display', None],
  [ 0x9129, 'CjkUnifiedIdeograph9129', [], 'Display', None],
  [ 0x912a, 'CjkUnifiedIdeograph912a', [], 'Display', None],
  [ 0x912b, 'CjkUnifiedIdeograph912b', [], 'Display', None],
  [ 0x912c, 'CjkUnifiedIdeograph912c', [], 'Display', None],
  [ 0x912d, 'CjkUnifiedIdeograph912d', [], 'Display', None],
  [ 0x912e, 'CjkUnifiedIdeograph912e', [], 'Display', None],
  [ 0x912f, 'CjkUnifiedIdeograph912f', [], 'Display', None],
  [ 0x9130, 'CjkUnifiedIdeograph9130', [], 'Display', None],
  [ 0x9131, 'CjkUnifiedIdeograph9131', [], 'Display', None],
  [ 0x9132, 'CjkUnifiedIdeograph9132', [], 'Display', None],
  [ 0x9133, 'CjkUnifiedIdeograph9133', [], 'Display', None],
  [ 0x9134, 'CjkUnifiedIdeograph9134', [], 'Display', None],
  [ 0x9135, 'CjkUnifiedIdeograph9135', [], 'Display', None],
  [ 0x9136, 'CjkUnifiedIdeograph9136', [], 'Display', None],
  [ 0x9137, 'CjkUnifiedIdeograph9137', [], 'Display', None],
  [ 0x9138, 'CjkUnifiedIdeograph9138', [], 'Display', None],
  [ 0x9139, 'CjkUnifiedIdeograph9139', [], 'Display', None],
  [ 0x913a, 'CjkUnifiedIdeograph913a', [], 'Display', None],
  [ 0x913b, 'CjkUnifiedIdeograph913b', [], 'Display', None],
  [ 0x913c, 'CjkUnifiedIdeograph913c', [], 'Display', None],
  [ 0x913d, 'CjkUnifiedIdeograph913d', [], 'Display', None],
  [ 0x913e, 'CjkUnifiedIdeograph913e', [], 'Display', None],
  [ 0x913f, 'CjkUnifiedIdeograph913f', [], 'Display', None],
  [ 0x9140, 'CjkUnifiedIdeograph9140', [], 'Display', None],
  [ 0x9141, 'CjkUnifiedIdeograph9141', [], 'Display', None],
  [ 0x9142, 'CjkUnifiedIdeograph9142', [], 'Display', None],
  [ 0x9143, 'CjkUnifiedIdeograph9143', [], 'Display', None],
  [ 0x9144, 'CjkUnifiedIdeograph9144', [], 'Display', None],
  [ 0x9145, 'CjkUnifiedIdeograph9145', [], 'Display', None],
  [ 0x9146, 'CjkUnifiedIdeograph9146', [], 'Display', None],
  [ 0x9147, 'CjkUnifiedIdeograph9147', [], 'Display', None],
  [ 0x9148, 'CjkUnifiedIdeograph9148', [], 'Display', None],
  [ 0x9149, 'CjkUnifiedIdeograph9149', [], 'Display', None],
  [ 0x914a, 'CjkUnifiedIdeograph914a', [], 'Display', None],
  [ 0x914b, 'CjkUnifiedIdeograph914b', [], 'Display', None],
  [ 0x914c, 'CjkUnifiedIdeograph914c', [], 'Display', None],
  [ 0x914d, 'CjkUnifiedIdeograph914d', [], 'Display', None],
  [ 0x914e, 'CjkUnifiedIdeograph914e', [], 'Display', None],
  [ 0x914f, 'CjkUnifiedIdeograph914f', [], 'Display', None],
  [ 0x9150, 'CjkUnifiedIdeograph9150', [], 'Display', None],
  [ 0x9151, 'CjkUnifiedIdeograph9151', [], 'Display', None],
  [ 0x9152, 'CjkUnifiedIdeograph9152', [], 'Display', None],
  [ 0x9153, 'CjkUnifiedIdeograph9153', [], 'Display', None],
  [ 0x9154, 'CjkUnifiedIdeograph9154', [], 'Display', None],
  [ 0x9155, 'CjkUnifiedIdeograph9155', [], 'Display', None],
  [ 0x9156, 'CjkUnifiedIdeograph9156', [], 'Display', None],
  [ 0x9157, 'CjkUnifiedIdeograph9157', [], 'Display', None],
  [ 0x9158, 'CjkUnifiedIdeograph9158', [], 'Display', None],
  [ 0x9159, 'CjkUnifiedIdeograph9159', [], 'Display', None],
  [ 0x915a, 'CjkUnifiedIdeograph915a', [], 'Display', None],
  [ 0x915b, 'CjkUnifiedIdeograph915b', [], 'Display', None],
  [ 0x915c, 'CjkUnifiedIdeograph915c', [], 'Display', None],
  [ 0x915d, 'CjkUnifiedIdeograph915d', [], 'Display', None],
  [ 0x915e, 'CjkUnifiedIdeograph915e', [], 'Display', None],
  [ 0x915f, 'CjkUnifiedIdeograph915f', [], 'Display', None],
  [ 0x9160, 'CjkUnifiedIdeograph9160', [], 'Display', None],
  [ 0x9161, 'CjkUnifiedIdeograph9161', [], 'Display', None],
  [ 0x9162, 'CjkUnifiedIdeograph9162', [], 'Display', None],
  [ 0x9163, 'CjkUnifiedIdeograph9163', [], 'Display', None],
  [ 0x9164, 'CjkUnifiedIdeograph9164', [], 'Display', None],
  [ 0x9165, 'CjkUnifiedIdeograph9165', [], 'Display', None],
  [ 0x9166, 'CjkUnifiedIdeograph9166', [], 'Display', None],
  [ 0x9167, 'CjkUnifiedIdeograph9167', [], 'Display', None],
  [ 0x9168, 'CjkUnifiedIdeograph9168', [], 'Display', None],
  [ 0x9169, 'CjkUnifiedIdeograph9169', [], 'Display', None],
  [ 0x916a, 'CjkUnifiedIdeograph916a', [], 'Display', None],
  [ 0x916b, 'CjkUnifiedIdeograph916b', [], 'Display', None],
  [ 0x916c, 'CjkUnifiedIdeograph916c', [], 'Display', None],
  [ 0x916d, 'CjkUnifiedIdeograph916d', [], 'Display', None],
  [ 0x916e, 'CjkUnifiedIdeograph916e', [], 'Display', None],
  [ 0x916f, 'CjkUnifiedIdeograph916f', [], 'Display', None],
  [ 0x9170, 'CjkUnifiedIdeograph9170', [], 'Display', None],
  [ 0x9171, 'CjkUnifiedIdeograph9171', [], 'Display', None],
  [ 0x9172, 'CjkUnifiedIdeograph9172', [], 'Display', None],
  [ 0x9173, 'CjkUnifiedIdeograph9173', [], 'Display', None],
  [ 0x9174, 'CjkUnifiedIdeograph9174', [], 'Display', None],
  [ 0x9175, 'CjkUnifiedIdeograph9175', [], 'Display', None],
  [ 0x9176, 'CjkUnifiedIdeograph9176', [], 'Display', None],
  [ 0x9177, 'CjkUnifiedIdeograph9177', [], 'Display', None],
  [ 0x9178, 'CjkUnifiedIdeograph9178', [], 'Display', None],
  [ 0x9179, 'CjkUnifiedIdeograph9179', [], 'Display', None],
  [ 0x917a, 'CjkUnifiedIdeograph917a', [], 'Display', None],
  [ 0x917b, 'CjkUnifiedIdeograph917b', [], 'Display', None],
  [ 0x917c, 'CjkUnifiedIdeograph917c', [], 'Display', None],
  [ 0x917d, 'CjkUnifiedIdeograph917d', [], 'Display', None],
  [ 0x917e, 'CjkUnifiedIdeograph917e', [], 'Display', None],
  [ 0x917f, 'CjkUnifiedIdeograph917f', [], 'Display', None],
  [ 0x9180, 'CjkUnifiedIdeograph9180', [], 'Display', None],
  [ 0x9181, 'CjkUnifiedIdeograph9181', [], 'Display', None],
  [ 0x9182, 'CjkUnifiedIdeograph9182', [], 'Display', None],
  [ 0x9183, 'CjkUnifiedIdeograph9183', [], 'Display', None],
  [ 0x9184, 'CjkUnifiedIdeograph9184', [], 'Display', None],
  [ 0x9185, 'CjkUnifiedIdeograph9185', [], 'Display', None],
  [ 0x9186, 'CjkUnifiedIdeograph9186', [], 'Display', None],
  [ 0x9187, 'CjkUnifiedIdeograph9187', [], 'Display', None],
  [ 0x9188, 'CjkUnifiedIdeograph9188', [], 'Display', None],
  [ 0x9189, 'CjkUnifiedIdeograph9189', [], 'Display', None],
  [ 0x918a, 'CjkUnifiedIdeograph918a', [], 'Display', None],
  [ 0x918b, 'CjkUnifiedIdeograph918b', [], 'Display', None],
  [ 0x918c, 'CjkUnifiedIdeograph918c', [], 'Display', None],
  [ 0x918d, 'CjkUnifiedIdeograph918d', [], 'Display', None],
  [ 0x918e, 'CjkUnifiedIdeograph918e', [], 'Display', None],
  [ 0x918f, 'CjkUnifiedIdeograph918f', [], 'Display', None],
  [ 0x9190, 'CjkUnifiedIdeograph9190', [], 'Display', None],
  [ 0x9191, 'CjkUnifiedIdeograph9191', [], 'Display', None],
  [ 0x9192, 'CjkUnifiedIdeograph9192', [], 'Display', None],
  [ 0x9193, 'CjkUnifiedIdeograph9193', [], 'Display', None],
  [ 0x9194, 'CjkUnifiedIdeograph9194', [], 'Display', None],
  [ 0x9195, 'CjkUnifiedIdeograph9195', [], 'Display', None],
  [ 0x9196, 'CjkUnifiedIdeograph9196', [], 'Display', None],
  [ 0x9197, 'CjkUnifiedIdeograph9197', [], 'Display', None],
  [ 0x9198, 'CjkUnifiedIdeograph9198', [], 'Display', None],
  [ 0x9199, 'CjkUnifiedIdeograph9199', [], 'Display', None],
  [ 0x919a, 'CjkUnifiedIdeograph919a', [], 'Display', None],
  [ 0x919b, 'CjkUnifiedIdeograph919b', [], 'Display', None],
  [ 0x919c, 'CjkUnifiedIdeograph919c', [], 'Display', None],
  [ 0x919d, 'CjkUnifiedIdeograph919d', [], 'Display', None],
  [ 0x919e, 'CjkUnifiedIdeograph919e', [], 'Display', None],
  [ 0x919f, 'CjkUnifiedIdeograph919f', [], 'Display', None],
  [ 0x91a0, 'CjkUnifiedIdeograph91a0', [], 'Display', None],
  [ 0x91a1, 'CjkUnifiedIdeograph91a1', [], 'Display', None],
  [ 0x91a2, 'CjkUnifiedIdeograph91a2', [], 'Display', None],
  [ 0x91a3, 'CjkUnifiedIdeograph91a3', [], 'Display', None],
  [ 0x91a4, 'CjkUnifiedIdeograph91a4', [], 'Display', None],
  [ 0x91a5, 'CjkUnifiedIdeograph91a5', [], 'Display', None],
  [ 0x91a6, 'CjkUnifiedIdeograph91a6', [], 'Display', None],
  [ 0x91a7, 'CjkUnifiedIdeograph91a7', [], 'Display', None],
  [ 0x91a8, 'CjkUnifiedIdeograph91a8', [], 'Display', None],
  [ 0x91a9, 'CjkUnifiedIdeograph91a9', [], 'Display', None],
  [ 0x91aa, 'CjkUnifiedIdeograph91aa', [], 'Display', None],
  [ 0x91ab, 'CjkUnifiedIdeograph91ab', [], 'Display', None],
  [ 0x91ac, 'CjkUnifiedIdeograph91ac', [], 'Display', None],
  [ 0x91ad, 'CjkUnifiedIdeograph91ad', [], 'Display', None],
  [ 0x91ae, 'CjkUnifiedIdeograph91ae', [], 'Display', None],
  [ 0x91af, 'CjkUnifiedIdeograph91af', [], 'Display', None],
  [ 0x91b0, 'CjkUnifiedIdeograph91b0', [], 'Display', None],
  [ 0x91b1, 'CjkUnifiedIdeograph91b1', [], 'Display', None],
  [ 0x91b2, 'CjkUnifiedIdeograph91b2', [], 'Display', None],
  [ 0x91b3, 'CjkUnifiedIdeograph91b3', [], 'Display', None],
  [ 0x91b4, 'CjkUnifiedIdeograph91b4', [], 'Display', None],
  [ 0x91b5, 'CjkUnifiedIdeograph91b5', [], 'Display', None],
  [ 0x91b6, 'CjkUnifiedIdeograph91b6', [], 'Display', None],
  [ 0x91b7, 'CjkUnifiedIdeograph91b7', [], 'Display', None],
  [ 0x91b8, 'CjkUnifiedIdeograph91b8', [], 'Display', None],
  [ 0x91b9, 'CjkUnifiedIdeograph91b9', [], 'Display', None],
  [ 0x91ba, 'CjkUnifiedIdeograph91ba', [], 'Display', None],
  [ 0x91bb, 'CjkUnifiedIdeograph91bb', [], 'Display', None],
  [ 0x91bc, 'CjkUnifiedIdeograph91bc', [], 'Display', None],
  [ 0x91bd, 'CjkUnifiedIdeograph91bd', [], 'Display', None],
  [ 0x91be, 'CjkUnifiedIdeograph91be', [], 'Display', None],
  [ 0x91bf, 'CjkUnifiedIdeograph91bf', [], 'Display', None],
  [ 0x91c0, 'CjkUnifiedIdeograph91c0', [], 'Display', None],
  [ 0x91c1, 'CjkUnifiedIdeograph91c1', [], 'Display', None],
  [ 0x91c2, 'CjkUnifiedIdeograph91c2', [], 'Display', None],
  [ 0x91c3, 'CjkUnifiedIdeograph91c3', [], 'Display', None],
  [ 0x91c4, 'CjkUnifiedIdeograph91c4', [], 'Display', None],
  [ 0x91c5, 'CjkUnifiedIdeograph91c5', [], 'Display', None],
  [ 0x91c6, 'CjkUnifiedIdeograph91c6', [], 'Display', None],
  [ 0x91c7, 'CjkUnifiedIdeograph91c7', [], 'Display', None],
  [ 0x91c8, 'CjkUnifiedIdeograph91c8', [], 'Display', None],
  [ 0x91c9, 'CjkUnifiedIdeograph91c9', [], 'Display', None],
  [ 0x91ca, 'CjkUnifiedIdeograph91ca', [], 'Display', None],
  [ 0x91cb, 'CjkUnifiedIdeograph91cb', [], 'Display', None],
  [ 0x91cc, 'CjkUnifiedIdeograph91cc', [], 'Display', None],
  [ 0x91cd, 'CjkUnifiedIdeograph91cd', [], 'Display', None],
  [ 0x91ce, 'CjkUnifiedIdeograph91ce', [], 'Display', None],
  [ 0x91cf, 'CjkUnifiedIdeograph91cf', [], 'Display', None],
  [ 0x91d0, 'CjkUnifiedIdeograph91d0', [], 'Display', None],
  [ 0x91d1, 'CjkUnifiedIdeograph91d1', [], 'Display', None],
  [ 0x91d2, 'CjkUnifiedIdeograph91d2', [], 'Display', None],
  [ 0x91d3, 'CjkUnifiedIdeograph91d3', [], 'Display', None],
  [ 0x91d4, 'CjkUnifiedIdeograph91d4', [], 'Display', None],
  [ 0x91d5, 'CjkUnifiedIdeograph91d5', [], 'Display', None],
  [ 0x91d6, 'CjkUnifiedIdeograph91d6', [], 'Display', None],
  [ 0x91d7, 'CjkUnifiedIdeograph91d7', [], 'Display', None],
  [ 0x91d8, 'CjkUnifiedIdeograph91d8', [], 'Display', None],
  [ 0x91d9, 'CjkUnifiedIdeograph91d9', [], 'Display', None],
  [ 0x91da, 'CjkUnifiedIdeograph91da', [], 'Display', None],
  [ 0x91db, 'CjkUnifiedIdeograph91db', [], 'Display', None],
  [ 0x91dc, 'CjkUnifiedIdeograph91dc', [], 'Display', None],
  [ 0x91dd, 'CjkUnifiedIdeograph91dd', [], 'Display', None],
  [ 0x91de, 'CjkUnifiedIdeograph91de', [], 'Display', None],
  [ 0x91df, 'CjkUnifiedIdeograph91df', [], 'Display', None],
  [ 0x91e0, 'CjkUnifiedIdeograph91e0', [], 'Display', None],
  [ 0x91e1, 'CjkUnifiedIdeograph91e1', [], 'Display', None],
  [ 0x91e2, 'CjkUnifiedIdeograph91e2', [], 'Display', None],
  [ 0x91e3, 'CjkUnifiedIdeograph91e3', [], 'Display', None],
  [ 0x91e4, 'CjkUnifiedIdeograph91e4', [], 'Display', None],
  [ 0x91e5, 'CjkUnifiedIdeograph91e5', [], 'Display', None],
  [ 0x91e6, 'CjkUnifiedIdeograph91e6', [], 'Display', None],
  [ 0x91e7, 'CjkUnifiedIdeograph91e7', [], 'Display', None],
  [ 0x91e8, 'CjkUnifiedIdeograph91e8', [], 'Display', None],
  [ 0x91e9, 'CjkUnifiedIdeograph91e9', [], 'Display', None],
  [ 0x91ea, 'CjkUnifiedIdeograph91ea', [], 'Display', None],
  [ 0x91eb, 'CjkUnifiedIdeograph91eb', [], 'Display', None],
  [ 0x91ec, 'CjkUnifiedIdeograph91ec', [], 'Display', None],
  [ 0x91ed, 'CjkUnifiedIdeograph91ed', [], 'Display', None],
  [ 0x91ee, 'CjkUnifiedIdeograph91ee', [], 'Display', None],
  [ 0x91ef, 'CjkUnifiedIdeograph91ef', [], 'Display', None],
  [ 0x91f0, 'CjkUnifiedIdeograph91f0', [], 'Display', None],
  [ 0x91f1, 'CjkUnifiedIdeograph91f1', [], 'Display', None],
  [ 0x91f2, 'CjkUnifiedIdeograph91f2', [], 'Display', None],
  [ 0x91f3, 'CjkUnifiedIdeograph91f3', [], 'Display', None],
  [ 0x91f4, 'CjkUnifiedIdeograph91f4', [], 'Display', None],
  [ 0x91f5, 'CjkUnifiedIdeograph91f5', [], 'Display', None],
  [ 0x91f6, 'CjkUnifiedIdeograph91f6', [], 'Display', None],
  [ 0x91f7, 'CjkUnifiedIdeograph91f7', [], 'Display', None],
  [ 0x91f8, 'CjkUnifiedIdeograph91f8', [], 'Display', None],
  [ 0x91f9, 'CjkUnifiedIdeograph91f9', [], 'Display', None],
  [ 0x91fa, 'CjkUnifiedIdeograph91fa', [], 'Display', None],
  [ 0x91fb, 'CjkUnifiedIdeograph91fb', [], 'Display', None],
  [ 0x91fc, 'CjkUnifiedIdeograph91fc', [], 'Display', None],
  [ 0x91fd, 'CjkUnifiedIdeograph91fd', [], 'Display', None],
  [ 0x91fe, 'CjkUnifiedIdeograph91fe', [], 'Display', None],
  [ 0x91ff, 'CjkUnifiedIdeograph91ff', [], 'Display', None],
  [ 0x9200, 'CjkUnifiedIdeograph9200', [], 'Display', None],
  [ 0x9201, 'CjkUnifiedIdeograph9201', [], 'Display', None],
  [ 0x9202, 'CjkUnifiedIdeograph9202', [], 'Display', None],
  [ 0x9203, 'CjkUnifiedIdeograph9203', [], 'Display', None],
  [ 0x9204, 'CjkUnifiedIdeograph9204', [], 'Display', None],
  [ 0x9205, 'CjkUnifiedIdeograph9205', [], 'Display', None],
  [ 0x9206, 'CjkUnifiedIdeograph9206', [], 'Display', None],
  [ 0x9207, 'CjkUnifiedIdeograph9207', [], 'Display', None],
  [ 0x9208, 'CjkUnifiedIdeograph9208', [], 'Display', None],
  [ 0x9209, 'CjkUnifiedIdeograph9209', [], 'Display', None],
  [ 0x920a, 'CjkUnifiedIdeograph920a', [], 'Display', None],
  [ 0x920b, 'CjkUnifiedIdeograph920b', [], 'Display', None],
  [ 0x920c, 'CjkUnifiedIdeograph920c', [], 'Display', None],
  [ 0x920d, 'CjkUnifiedIdeograph920d', [], 'Display', None],
  [ 0x920e, 'CjkUnifiedIdeograph920e', [], 'Display', None],
  [ 0x920f, 'CjkUnifiedIdeograph920f', [], 'Display', None],
  [ 0x9210, 'CjkUnifiedIdeograph9210', [], 'Display', None],
  [ 0x9211, 'CjkUnifiedIdeograph9211', [], 'Display', None],
  [ 0x9212, 'CjkUnifiedIdeograph9212', [], 'Display', None],
  [ 0x9213, 'CjkUnifiedIdeograph9213', [], 'Display', None],
  [ 0x9214, 'CjkUnifiedIdeograph9214', [], 'Display', None],
  [ 0x9215, 'CjkUnifiedIdeograph9215', [], 'Display', None],
  [ 0x9216, 'CjkUnifiedIdeograph9216', [], 'Display', None],
  [ 0x9217, 'CjkUnifiedIdeograph9217', [], 'Display', None],
  [ 0x9218, 'CjkUnifiedIdeograph9218', [], 'Display', None],
  [ 0x9219, 'CjkUnifiedIdeograph9219', [], 'Display', None],
  [ 0x921a, 'CjkUnifiedIdeograph921a', [], 'Display', None],
  [ 0x921b, 'CjkUnifiedIdeograph921b', [], 'Display', None],
  [ 0x921c, 'CjkUnifiedIdeograph921c', [], 'Display', None],
  [ 0x921d, 'CjkUnifiedIdeograph921d', [], 'Display', None],
  [ 0x921e, 'CjkUnifiedIdeograph921e', [], 'Display', None],
  [ 0x921f, 'CjkUnifiedIdeograph921f', [], 'Display', None],
  [ 0x9220, 'CjkUnifiedIdeograph9220', [], 'Display', None],
  [ 0x9221, 'CjkUnifiedIdeograph9221', [], 'Display', None],
  [ 0x9222, 'CjkUnifiedIdeograph9222', [], 'Display', None],
  [ 0x9223, 'CjkUnifiedIdeograph9223', [], 'Display', None],
  [ 0x9224, 'CjkUnifiedIdeograph9224', [], 'Display', None],
  [ 0x9225, 'CjkUnifiedIdeograph9225', [], 'Display', None],
  [ 0x9226, 'CjkUnifiedIdeograph9226', [], 'Display', None],
  [ 0x9227, 'CjkUnifiedIdeograph9227', [], 'Display', None],
  [ 0x9228, 'CjkUnifiedIdeograph9228', [], 'Display', None],
  [ 0x9229, 'CjkUnifiedIdeograph9229', [], 'Display', None],
  [ 0x922a, 'CjkUnifiedIdeograph922a', [], 'Display', None],
  [ 0x922b, 'CjkUnifiedIdeograph922b', [], 'Display', None],
  [ 0x922c, 'CjkUnifiedIdeograph922c', [], 'Display', None],
  [ 0x922d, 'CjkUnifiedIdeograph922d', [], 'Display', None],
  [ 0x922e, 'CjkUnifiedIdeograph922e', [], 'Display', None],
  [ 0x922f, 'CjkUnifiedIdeograph922f', [], 'Display', None],
  [ 0x9230, 'CjkUnifiedIdeograph9230', [], 'Display', None],
  [ 0x9231, 'CjkUnifiedIdeograph9231', [], 'Display', None],
  [ 0x9232, 'CjkUnifiedIdeograph9232', [], 'Display', None],
  [ 0x9233, 'CjkUnifiedIdeograph9233', [], 'Display', None],
  [ 0x9234, 'CjkUnifiedIdeograph9234', [], 'Display', None],
  [ 0x9235, 'CjkUnifiedIdeograph9235', [], 'Display', None],
  [ 0x9236, 'CjkUnifiedIdeograph9236', [], 'Display', None],
  [ 0x9237, 'CjkUnifiedIdeograph9237', [], 'Display', None],
  [ 0x9238, 'CjkUnifiedIdeograph9238', [], 'Display', None],
  [ 0x9239, 'CjkUnifiedIdeograph9239', [], 'Display', None],
  [ 0x923a, 'CjkUnifiedIdeograph923a', [], 'Display', None],
  [ 0x923b, 'CjkUnifiedIdeograph923b', [], 'Display', None],
  [ 0x923c, 'CjkUnifiedIdeograph923c', [], 'Display', None],
  [ 0x923d, 'CjkUnifiedIdeograph923d', [], 'Display', None],
  [ 0x923e, 'CjkUnifiedIdeograph923e', [], 'Display', None],
  [ 0x923f, 'CjkUnifiedIdeograph923f', [], 'Display', None],
  [ 0x9240, 'CjkUnifiedIdeograph9240', [], 'Display', None],
  [ 0x9241, 'CjkUnifiedIdeograph9241', [], 'Display', None],
  [ 0x9242, 'CjkUnifiedIdeograph9242', [], 'Display', None],
  [ 0x9243, 'CjkUnifiedIdeograph9243', [], 'Display', None],
  [ 0x9244, 'CjkUnifiedIdeograph9244', [], 'Display', None],
  [ 0x9245, 'CjkUnifiedIdeograph9245', [], 'Display', None],
  [ 0x9246, 'CjkUnifiedIdeograph9246', [], 'Display', None],
  [ 0x9247, 'CjkUnifiedIdeograph9247', [], 'Display', None],
  [ 0x9248, 'CjkUnifiedIdeograph9248', [], 'Display', None],
  [ 0x9249, 'CjkUnifiedIdeograph9249', [], 'Display', None],
  [ 0x924a, 'CjkUnifiedIdeograph924a', [], 'Display', None],
  [ 0x924b, 'CjkUnifiedIdeograph924b', [], 'Display', None],
  [ 0x924c, 'CjkUnifiedIdeograph924c', [], 'Display', None],
  [ 0x924d, 'CjkUnifiedIdeograph924d', [], 'Display', None],
  [ 0x924e, 'CjkUnifiedIdeograph924e', [], 'Display', None],
  [ 0x924f, 'CjkUnifiedIdeograph924f', [], 'Display', None],
  [ 0x9250, 'CjkUnifiedIdeograph9250', [], 'Display', None],
  [ 0x9251, 'CjkUnifiedIdeograph9251', [], 'Display', None],
  [ 0x9252, 'CjkUnifiedIdeograph9252', [], 'Display', None],
  [ 0x9253, 'CjkUnifiedIdeograph9253', [], 'Display', None],
  [ 0x9254, 'CjkUnifiedIdeograph9254', [], 'Display', None],
  [ 0x9255, 'CjkUnifiedIdeograph9255', [], 'Display', None],
  [ 0x9256, 'CjkUnifiedIdeograph9256', [], 'Display', None],
  [ 0x9257, 'CjkUnifiedIdeograph9257', [], 'Display', None],
  [ 0x9258, 'CjkUnifiedIdeograph9258', [], 'Display', None],
  [ 0x9259, 'CjkUnifiedIdeograph9259', [], 'Display', None],
  [ 0x925a, 'CjkUnifiedIdeograph925a', [], 'Display', None],
  [ 0x925b, 'CjkUnifiedIdeograph925b', [], 'Display', None],
  [ 0x925c, 'CjkUnifiedIdeograph925c', [], 'Display', None],
  [ 0x925d, 'CjkUnifiedIdeograph925d', [], 'Display', None],
  [ 0x925e, 'CjkUnifiedIdeograph925e', [], 'Display', None],
  [ 0x925f, 'CjkUnifiedIdeograph925f', [], 'Display', None],
  [ 0x9260, 'CjkUnifiedIdeograph9260', [], 'Display', None],
  [ 0x9261, 'CjkUnifiedIdeograph9261', [], 'Display', None],
  [ 0x9262, 'CjkUnifiedIdeograph9262', [], 'Display', None],
  [ 0x9263, 'CjkUnifiedIdeograph9263', [], 'Display', None],
  [ 0x9264, 'CjkUnifiedIdeograph9264', [], 'Display', None],
  [ 0x9265, 'CjkUnifiedIdeograph9265', [], 'Display', None],
  [ 0x9266, 'CjkUnifiedIdeograph9266', [], 'Display', None],
  [ 0x9267, 'CjkUnifiedIdeograph9267', [], 'Display', None],
  [ 0x9268, 'CjkUnifiedIdeograph9268', [], 'Display', None],
  [ 0x9269, 'CjkUnifiedIdeograph9269', [], 'Display', None],
  [ 0x926a, 'CjkUnifiedIdeograph926a', [], 'Display', None],
  [ 0x926b, 'CjkUnifiedIdeograph926b', [], 'Display', None],
  [ 0x926c, 'CjkUnifiedIdeograph926c', [], 'Display', None],
  [ 0x926d, 'CjkUnifiedIdeograph926d', [], 'Display', None],
  [ 0x926e, 'CjkUnifiedIdeograph926e', [], 'Display', None],
  [ 0x926f, 'CjkUnifiedIdeograph926f', [], 'Display', None],
  [ 0x9270, 'CjkUnifiedIdeograph9270', [], 'Display', None],
  [ 0x9271, 'CjkUnifiedIdeograph9271', [], 'Display', None],
  [ 0x9272, 'CjkUnifiedIdeograph9272', [], 'Display', None],
  [ 0x9273, 'CjkUnifiedIdeograph9273', [], 'Display', None],
  [ 0x9274, 'CjkUnifiedIdeograph9274', [], 'Display', None],
  [ 0x9275, 'CjkUnifiedIdeograph9275', [], 'Display', None],
  [ 0x9276, 'CjkUnifiedIdeograph9276', [], 'Display', None],
  [ 0x9277, 'CjkUnifiedIdeograph9277', [], 'Display', None],
  [ 0x9278, 'CjkUnifiedIdeograph9278', [], 'Display', None],
  [ 0x9279, 'CjkUnifiedIdeograph9279', [], 'Display', None],
  [ 0x927a, 'CjkUnifiedIdeograph927a', [], 'Display', None],
  [ 0x927b, 'CjkUnifiedIdeograph927b', [], 'Display', None],
  [ 0x927c, 'CjkUnifiedIdeograph927c', [], 'Display', None],
  [ 0x927d, 'CjkUnifiedIdeograph927d', [], 'Display', None],
  [ 0x927e, 'CjkUnifiedIdeograph927e', [], 'Display', None],
  [ 0x927f, 'CjkUnifiedIdeograph927f', [], 'Display', None],
  [ 0x9280, 'CjkUnifiedIdeograph9280', [], 'Display', None],
  [ 0x9281, 'CjkUnifiedIdeograph9281', [], 'Display', None],
  [ 0x9282, 'CjkUnifiedIdeograph9282', [], 'Display', None],
  [ 0x9283, 'CjkUnifiedIdeograph9283', [], 'Display', None],
  [ 0x9284, 'CjkUnifiedIdeograph9284', [], 'Display', None],
  [ 0x9285, 'CjkUnifiedIdeograph9285', [], 'Display', None],
  [ 0x9286, 'CjkUnifiedIdeograph9286', [], 'Display', None],
  [ 0x9287, 'CjkUnifiedIdeograph9287', [], 'Display', None],
  [ 0x9288, 'CjkUnifiedIdeograph9288', [], 'Display', None],
  [ 0x9289, 'CjkUnifiedIdeograph9289', [], 'Display', None],
  [ 0x928a, 'CjkUnifiedIdeograph928a', [], 'Display', None],
  [ 0x928b, 'CjkUnifiedIdeograph928b', [], 'Display', None],
  [ 0x928c, 'CjkUnifiedIdeograph928c', [], 'Display', None],
  [ 0x928d, 'CjkUnifiedIdeograph928d', [], 'Display', None],
  [ 0x928e, 'CjkUnifiedIdeograph928e', [], 'Display', None],
  [ 0x928f, 'CjkUnifiedIdeograph928f', [], 'Display', None],
  [ 0x9290, 'CjkUnifiedIdeograph9290', [], 'Display', None],
  [ 0x9291, 'CjkUnifiedIdeograph9291', [], 'Display', None],
  [ 0x9292, 'CjkUnifiedIdeograph9292', [], 'Display', None],
  [ 0x9293, 'CjkUnifiedIdeograph9293', [], 'Display', None],
  [ 0x9294, 'CjkUnifiedIdeograph9294', [], 'Display', None],
  [ 0x9295, 'CjkUnifiedIdeograph9295', [], 'Display', None],
  [ 0x9296, 'CjkUnifiedIdeograph9296', [], 'Display', None],
  [ 0x9297, 'CjkUnifiedIdeograph9297', [], 'Display', None],
  [ 0x9298, 'CjkUnifiedIdeograph9298', [], 'Display', None],
  [ 0x9299, 'CjkUnifiedIdeograph9299', [], 'Display', None],
  [ 0x929a, 'CjkUnifiedIdeograph929a', [], 'Display', None],
  [ 0x929b, 'CjkUnifiedIdeograph929b', [], 'Display', None],
  [ 0x929c, 'CjkUnifiedIdeograph929c', [], 'Display', None],
  [ 0x929d, 'CjkUnifiedIdeograph929d', [], 'Display', None],
  [ 0x929e, 'CjkUnifiedIdeograph929e', [], 'Display', None],
  [ 0x929f, 'CjkUnifiedIdeograph929f', [], 'Display', None],
  [ 0x92a0, 'CjkUnifiedIdeograph92a0', [], 'Display', None],
  [ 0x92a1, 'CjkUnifiedIdeograph92a1', [], 'Display', None],
  [ 0x92a2, 'CjkUnifiedIdeograph92a2', [], 'Display', None],
  [ 0x92a3, 'CjkUnifiedIdeograph92a3', [], 'Display', None],
  [ 0x92a4, 'CjkUnifiedIdeograph92a4', [], 'Display', None],
  [ 0x92a5, 'CjkUnifiedIdeograph92a5', [], 'Display', None],
  [ 0x92a6, 'CjkUnifiedIdeograph92a6', [], 'Display', None],
  [ 0x92a7, 'CjkUnifiedIdeograph92a7', [], 'Display', None],
  [ 0x92a8, 'CjkUnifiedIdeograph92a8', [], 'Display', None],
  [ 0x92a9, 'CjkUnifiedIdeograph92a9', [], 'Display', None],
  [ 0x92aa, 'CjkUnifiedIdeograph92aa', [], 'Display', None],
  [ 0x92ab, 'CjkUnifiedIdeograph92ab', [], 'Display', None],
  [ 0x92ac, 'CjkUnifiedIdeograph92ac', [], 'Display', None],
  [ 0x92ad, 'CjkUnifiedIdeograph92ad', [], 'Display', None],
  [ 0x92ae, 'CjkUnifiedIdeograph92ae', [], 'Display', None],
  [ 0x92af, 'CjkUnifiedIdeograph92af', [], 'Display', None],
  [ 0x92b0, 'CjkUnifiedIdeograph92b0', [], 'Display', None],
  [ 0x92b1, 'CjkUnifiedIdeograph92b1', [], 'Display', None],
  [ 0x92b2, 'CjkUnifiedIdeograph92b2', [], 'Display', None],
  [ 0x92b3, 'CjkUnifiedIdeograph92b3', [], 'Display', None],
  [ 0x92b4, 'CjkUnifiedIdeograph92b4', [], 'Display', None],
  [ 0x92b5, 'CjkUnifiedIdeograph92b5', [], 'Display', None],
  [ 0x92b6, 'CjkUnifiedIdeograph92b6', [], 'Display', None],
  [ 0x92b7, 'CjkUnifiedIdeograph92b7', [], 'Display', None],
  [ 0x92b8, 'CjkUnifiedIdeograph92b8', [], 'Display', None],
  [ 0x92b9, 'CjkUnifiedIdeograph92b9', [], 'Display', None],
  [ 0x92ba, 'CjkUnifiedIdeograph92ba', [], 'Display', None],
  [ 0x92bb, 'CjkUnifiedIdeograph92bb', [], 'Display', None],
  [ 0x92bc, 'CjkUnifiedIdeograph92bc', [], 'Display', None],
  [ 0x92bd, 'CjkUnifiedIdeograph92bd', [], 'Display', None],
  [ 0x92be, 'CjkUnifiedIdeograph92be', [], 'Display', None],
  [ 0x92bf, 'CjkUnifiedIdeograph92bf', [], 'Display', None],
  [ 0x92c0, 'CjkUnifiedIdeograph92c0', [], 'Display', None],
  [ 0x92c1, 'CjkUnifiedIdeograph92c1', [], 'Display', None],
  [ 0x92c2, 'CjkUnifiedIdeograph92c2', [], 'Display', None],
  [ 0x92c3, 'CjkUnifiedIdeograph92c3', [], 'Display', None],
  [ 0x92c4, 'CjkUnifiedIdeograph92c4', [], 'Display', None],
  [ 0x92c5, 'CjkUnifiedIdeograph92c5', [], 'Display', None],
  [ 0x92c6, 'CjkUnifiedIdeograph92c6', [], 'Display', None],
  [ 0x92c7, 'CjkUnifiedIdeograph92c7', [], 'Display', None],
  [ 0x92c8, 'CjkUnifiedIdeograph92c8', [], 'Display', None],
  [ 0x92c9, 'CjkUnifiedIdeograph92c9', [], 'Display', None],
  [ 0x92ca, 'CjkUnifiedIdeograph92ca', [], 'Display', None],
  [ 0x92cb, 'CjkUnifiedIdeograph92cb', [], 'Display', None],
  [ 0x92cc, 'CjkUnifiedIdeograph92cc', [], 'Display', None],
  [ 0x92cd, 'CjkUnifiedIdeograph92cd', [], 'Display', None],
  [ 0x92ce, 'CjkUnifiedIdeograph92ce', [], 'Display', None],
  [ 0x92cf, 'CjkUnifiedIdeograph92cf', [], 'Display', None],
  [ 0x92d0, 'CjkUnifiedIdeograph92d0', [], 'Display', None],
  [ 0x92d1, 'CjkUnifiedIdeograph92d1', [], 'Display', None],
  [ 0x92d2, 'CjkUnifiedIdeograph92d2', [], 'Display', None],
  [ 0x92d3, 'CjkUnifiedIdeograph92d3', [], 'Display', None],
  [ 0x92d4, 'CjkUnifiedIdeograph92d4', [], 'Display', None],
  [ 0x92d5, 'CjkUnifiedIdeograph92d5', [], 'Display', None],
  [ 0x92d6, 'CjkUnifiedIdeograph92d6', [], 'Display', None],
  [ 0x92d7, 'CjkUnifiedIdeograph92d7', [], 'Display', None],
  [ 0x92d8, 'CjkUnifiedIdeograph92d8', [], 'Display', None],
  [ 0x92d9, 'CjkUnifiedIdeograph92d9', [], 'Display', None],
  [ 0x92da, 'CjkUnifiedIdeograph92da', [], 'Display', None],
  [ 0x92db, 'CjkUnifiedIdeograph92db', [], 'Display', None],
  [ 0x92dc, 'CjkUnifiedIdeograph92dc', [], 'Display', None],
  [ 0x92dd, 'CjkUnifiedIdeograph92dd', [], 'Display', None],
  [ 0x92de, 'CjkUnifiedIdeograph92de', [], 'Display', None],
  [ 0x92df, 'CjkUnifiedIdeograph92df', [], 'Display', None],
  [ 0x92e0, 'CjkUnifiedIdeograph92e0', [], 'Display', None],
  [ 0x92e1, 'CjkUnifiedIdeograph92e1', [], 'Display', None],
  [ 0x92e2, 'CjkUnifiedIdeograph92e2', [], 'Display', None],
  [ 0x92e3, 'CjkUnifiedIdeograph92e3', [], 'Display', None],
  [ 0x92e4, 'CjkUnifiedIdeograph92e4', [], 'Display', None],
  [ 0x92e5, 'CjkUnifiedIdeograph92e5', [], 'Display', None],
  [ 0x92e6, 'CjkUnifiedIdeograph92e6', [], 'Display', None],
  [ 0x92e7, 'CjkUnifiedIdeograph92e7', [], 'Display', None],
  [ 0x92e8, 'CjkUnifiedIdeograph92e8', [], 'Display', None],
  [ 0x92e9, 'CjkUnifiedIdeograph92e9', [], 'Display', None],
  [ 0x92ea, 'CjkUnifiedIdeograph92ea', [], 'Display', None],
  [ 0x92eb, 'CjkUnifiedIdeograph92eb', [], 'Display', None],
  [ 0x92ec, 'CjkUnifiedIdeograph92ec', [], 'Display', None],
  [ 0x92ed, 'CjkUnifiedIdeograph92ed', [], 'Display', None],
  [ 0x92ee, 'CjkUnifiedIdeograph92ee', [], 'Display', None],
  [ 0x92ef, 'CjkUnifiedIdeograph92ef', [], 'Display', None],
  [ 0x92f0, 'CjkUnifiedIdeograph92f0', [], 'Display', None],
  [ 0x92f1, 'CjkUnifiedIdeograph92f1', [], 'Display', None],
  [ 0x92f2, 'CjkUnifiedIdeograph92f2', [], 'Display', None],
  [ 0x92f3, 'CjkUnifiedIdeograph92f3', [], 'Display', None],
  [ 0x92f4, 'CjkUnifiedIdeograph92f4', [], 'Display', None],
  [ 0x92f5, 'CjkUnifiedIdeograph92f5', [], 'Display', None],
  [ 0x92f6, 'CjkUnifiedIdeograph92f6', [], 'Display', None],
  [ 0x92f7, 'CjkUnifiedIdeograph92f7', [], 'Display', None],
  [ 0x92f8, 'CjkUnifiedIdeograph92f8', [], 'Display', None],
  [ 0x92f9, 'CjkUnifiedIdeograph92f9', [], 'Display', None],
  [ 0x92fa, 'CjkUnifiedIdeograph92fa', [], 'Display', None],
  [ 0x92fb, 'CjkUnifiedIdeograph92fb', [], 'Display', None],
  [ 0x92fc, 'CjkUnifiedIdeograph92fc', [], 'Display', None],
  [ 0x92fd, 'CjkUnifiedIdeograph92fd', [], 'Display', None],
  [ 0x92fe, 'CjkUnifiedIdeograph92fe', [], 'Display', None],
  [ 0x92ff, 'CjkUnifiedIdeograph92ff', [], 'Display', None],
  [ 0x9300, 'CjkUnifiedIdeograph9300', [], 'Display', None],
  [ 0x9301, 'CjkUnifiedIdeograph9301', [], 'Display', None],
  [ 0x9302, 'CjkUnifiedIdeograph9302', [], 'Display', None],
  [ 0x9303, 'CjkUnifiedIdeograph9303', [], 'Display', None],
  [ 0x9304, 'CjkUnifiedIdeograph9304', [], 'Display', None],
  [ 0x9305, 'CjkUnifiedIdeograph9305', [], 'Display', None],
  [ 0x9306, 'CjkUnifiedIdeograph9306', [], 'Display', None],
  [ 0x9307, 'CjkUnifiedIdeograph9307', [], 'Display', None],
  [ 0x9308, 'CjkUnifiedIdeograph9308', [], 'Display', None],
  [ 0x9309, 'CjkUnifiedIdeograph9309', [], 'Display', None],
  [ 0x930a, 'CjkUnifiedIdeograph930a', [], 'Display', None],
  [ 0x930b, 'CjkUnifiedIdeograph930b', [], 'Display', None],
  [ 0x930c, 'CjkUnifiedIdeograph930c', [], 'Display', None],
  [ 0x930d, 'CjkUnifiedIdeograph930d', [], 'Display', None],
  [ 0x930e, 'CjkUnifiedIdeograph930e', [], 'Display', None],
  [ 0x930f, 'CjkUnifiedIdeograph930f', [], 'Display', None],
  [ 0x9310, 'CjkUnifiedIdeograph9310', [], 'Display', None],
  [ 0x9311, 'CjkUnifiedIdeograph9311', [], 'Display', None],
  [ 0x9312, 'CjkUnifiedIdeograph9312', [], 'Display', None],
  [ 0x9313, 'CjkUnifiedIdeograph9313', [], 'Display', None],
  [ 0x9314, 'CjkUnifiedIdeograph9314', [], 'Display', None],
  [ 0x9315, 'CjkUnifiedIdeograph9315', [], 'Display', None],
  [ 0x9316, 'CjkUnifiedIdeograph9316', [], 'Display', None],
  [ 0x9317, 'CjkUnifiedIdeograph9317', [], 'Display', None],
  [ 0x9318, 'CjkUnifiedIdeograph9318', [], 'Display', None],
  [ 0x9319, 'CjkUnifiedIdeograph9319', [], 'Display', None],
  [ 0x931a, 'CjkUnifiedIdeograph931a', [], 'Display', None],
  [ 0x931b, 'CjkUnifiedIdeograph931b', [], 'Display', None],
  [ 0x931c, 'CjkUnifiedIdeograph931c', [], 'Display', None],
  [ 0x931d, 'CjkUnifiedIdeograph931d', [], 'Display', None],
  [ 0x931e, 'CjkUnifiedIdeograph931e', [], 'Display', None],
  [ 0x931f, 'CjkUnifiedIdeograph931f', [], 'Display', None],
  [ 0x9320, 'CjkUnifiedIdeograph9320', [], 'Display', None],
  [ 0x9321, 'CjkUnifiedIdeograph9321', [], 'Display', None],
  [ 0x9322, 'CjkUnifiedIdeograph9322', [], 'Display', None],
  [ 0x9323, 'CjkUnifiedIdeograph9323', [], 'Display', None],
  [ 0x9324, 'CjkUnifiedIdeograph9324', [], 'Display', None],
  [ 0x9325, 'CjkUnifiedIdeograph9325', [], 'Display', None],
  [ 0x9326, 'CjkUnifiedIdeograph9326', [], 'Display', None],
  [ 0x9327, 'CjkUnifiedIdeograph9327', [], 'Display', None],
  [ 0x9328, 'CjkUnifiedIdeograph9328', [], 'Display', None],
  [ 0x9329, 'CjkUnifiedIdeograph9329', [], 'Display', None],
  [ 0x932a, 'CjkUnifiedIdeograph932a', [], 'Display', None],
  [ 0x932b, 'CjkUnifiedIdeograph932b', [], 'Display', None],
  [ 0x932c, 'CjkUnifiedIdeograph932c', [], 'Display', None],
  [ 0x932d, 'CjkUnifiedIdeograph932d', [], 'Display', None],
  [ 0x932e, 'CjkUnifiedIdeograph932e', [], 'Display', None],
  [ 0x932f, 'CjkUnifiedIdeograph932f', [], 'Display', None],
  [ 0x9330, 'CjkUnifiedIdeograph9330', [], 'Display', None],
  [ 0x9331, 'CjkUnifiedIdeograph9331', [], 'Display', None],
  [ 0x9332, 'CjkUnifiedIdeograph9332', [], 'Display', None],
  [ 0x9333, 'CjkUnifiedIdeograph9333', [], 'Display', None],
  [ 0x9334, 'CjkUnifiedIdeograph9334', [], 'Display', None],
  [ 0x9335, 'CjkUnifiedIdeograph9335', [], 'Display', None],
  [ 0x9336, 'CjkUnifiedIdeograph9336', [], 'Display', None],
  [ 0x9337, 'CjkUnifiedIdeograph9337', [], 'Display', None],
  [ 0x9338, 'CjkUnifiedIdeograph9338', [], 'Display', None],
  [ 0x9339, 'CjkUnifiedIdeograph9339', [], 'Display', None],
  [ 0x933a, 'CjkUnifiedIdeograph933a', [], 'Display', None],
  [ 0x933b, 'CjkUnifiedIdeograph933b', [], 'Display', None],
  [ 0x933c, 'CjkUnifiedIdeograph933c', [], 'Display', None],
  [ 0x933d, 'CjkUnifiedIdeograph933d', [], 'Display', None],
  [ 0x933e, 'CjkUnifiedIdeograph933e', [], 'Display', None],
  [ 0x933f, 'CjkUnifiedIdeograph933f', [], 'Display', None],
  [ 0x9340, 'CjkUnifiedIdeograph9340', [], 'Display', None],
  [ 0x9341, 'CjkUnifiedIdeograph9341', [], 'Display', None],
  [ 0x9342, 'CjkUnifiedIdeograph9342', [], 'Display', None],
  [ 0x9343, 'CjkUnifiedIdeograph9343', [], 'Display', None],
  [ 0x9344, 'CjkUnifiedIdeograph9344', [], 'Display', None],
  [ 0x9345, 'CjkUnifiedIdeograph9345', [], 'Display', None],
  [ 0x9346, 'CjkUnifiedIdeograph9346', [], 'Display', None],
  [ 0x9347, 'CjkUnifiedIdeograph9347', [], 'Display', None],
  [ 0x9348, 'CjkUnifiedIdeograph9348', [], 'Display', None],
  [ 0x9349, 'CjkUnifiedIdeograph9349', [], 'Display', None],
  [ 0x934a, 'CjkUnifiedIdeograph934a', [], 'Display', None],
  [ 0x934b, 'CjkUnifiedIdeograph934b', [], 'Display', None],
  [ 0x934c, 'CjkUnifiedIdeograph934c', [], 'Display', None],
  [ 0x934d, 'CjkUnifiedIdeograph934d', [], 'Display', None],
  [ 0x934e, 'CjkUnifiedIdeograph934e', [], 'Display', None],
  [ 0x934f, 'CjkUnifiedIdeograph934f', [], 'Display', None],
  [ 0x9350, 'CjkUnifiedIdeograph9350', [], 'Display', None],
  [ 0x9351, 'CjkUnifiedIdeograph9351', [], 'Display', None],
  [ 0x9352, 'CjkUnifiedIdeograph9352', [], 'Display', None],
  [ 0x9353, 'CjkUnifiedIdeograph9353', [], 'Display', None],
  [ 0x9354, 'CjkUnifiedIdeograph9354', [], 'Display', None],
  [ 0x9355, 'CjkUnifiedIdeograph9355', [], 'Display', None],
  [ 0x9356, 'CjkUnifiedIdeograph9356', [], 'Display', None],
  [ 0x9357, 'CjkUnifiedIdeograph9357', [], 'Display', None],
  [ 0x9358, 'CjkUnifiedIdeograph9358', [], 'Display', None],
  [ 0x9359, 'CjkUnifiedIdeograph9359', [], 'Display', None],
  [ 0x935a, 'CjkUnifiedIdeograph935a', [], 'Display', None],
  [ 0x935b, 'CjkUnifiedIdeograph935b', [], 'Display', None],
  [ 0x935c, 'CjkUnifiedIdeograph935c', [], 'Display', None],
  [ 0x935d, 'CjkUnifiedIdeograph935d', [], 'Display', None],
  [ 0x935e, 'CjkUnifiedIdeograph935e', [], 'Display', None],
  [ 0x935f, 'CjkUnifiedIdeograph935f', [], 'Display', None],
  [ 0x9360, 'CjkUnifiedIdeograph9360', [], 'Display', None],
  [ 0x9361, 'CjkUnifiedIdeograph9361', [], 'Display', None],
  [ 0x9362, 'CjkUnifiedIdeograph9362', [], 'Display', None],
  [ 0x9363, 'CjkUnifiedIdeograph9363', [], 'Display', None],
  [ 0x9364, 'CjkUnifiedIdeograph9364', [], 'Display', None],
  [ 0x9365, 'CjkUnifiedIdeograph9365', [], 'Display', None],
  [ 0x9366, 'CjkUnifiedIdeograph9366', [], 'Display', None],
  [ 0x9367, 'CjkUnifiedIdeograph9367', [], 'Display', None],
  [ 0x9368, 'CjkUnifiedIdeograph9368', [], 'Display', None],
  [ 0x9369, 'CjkUnifiedIdeograph9369', [], 'Display', None],
  [ 0x936a, 'CjkUnifiedIdeograph936a', [], 'Display', None],
  [ 0x936b, 'CjkUnifiedIdeograph936b', [], 'Display', None],
  [ 0x936c, 'CjkUnifiedIdeograph936c', [], 'Display', None],
  [ 0x936d, 'CjkUnifiedIdeograph936d', [], 'Display', None],
  [ 0x936e, 'CjkUnifiedIdeograph936e', [], 'Display', None],
  [ 0x936f, 'CjkUnifiedIdeograph936f', [], 'Display', None],
  [ 0x9370, 'CjkUnifiedIdeograph9370', [], 'Display', None],
  [ 0x9371, 'CjkUnifiedIdeograph9371', [], 'Display', None],
  [ 0x9372, 'CjkUnifiedIdeograph9372', [], 'Display', None],
  [ 0x9373, 'CjkUnifiedIdeograph9373', [], 'Display', None],
  [ 0x9374, 'CjkUnifiedIdeograph9374', [], 'Display', None],
  [ 0x9375, 'CjkUnifiedIdeograph9375', [], 'Display', None],
  [ 0x9376, 'CjkUnifiedIdeograph9376', [], 'Display', None],
  [ 0x9377, 'CjkUnifiedIdeograph9377', [], 'Display', None],
  [ 0x9378, 'CjkUnifiedIdeograph9378', [], 'Display', None],
  [ 0x9379, 'CjkUnifiedIdeograph9379', [], 'Display', None],
  [ 0x937a, 'CjkUnifiedIdeograph937a', [], 'Display', None],
  [ 0x937b, 'CjkUnifiedIdeograph937b', [], 'Display', None],
  [ 0x937c, 'CjkUnifiedIdeograph937c', [], 'Display', None],
  [ 0x937d, 'CjkUnifiedIdeograph937d', [], 'Display', None],
  [ 0x937e, 'CjkUnifiedIdeograph937e', [], 'Display', None],
  [ 0x937f, 'CjkUnifiedIdeograph937f', [], 'Display', None],
  [ 0x9380, 'CjkUnifiedIdeograph9380', [], 'Display', None],
  [ 0x9381, 'CjkUnifiedIdeograph9381', [], 'Display', None],
  [ 0x9382, 'CjkUnifiedIdeograph9382', [], 'Display', None],
  [ 0x9383, 'CjkUnifiedIdeograph9383', [], 'Display', None],
  [ 0x9384, 'CjkUnifiedIdeograph9384', [], 'Display', None],
  [ 0x9385, 'CjkUnifiedIdeograph9385', [], 'Display', None],
  [ 0x9386, 'CjkUnifiedIdeograph9386', [], 'Display', None],
  [ 0x9387, 'CjkUnifiedIdeograph9387', [], 'Display', None],
  [ 0x9388, 'CjkUnifiedIdeograph9388', [], 'Display', None],
  [ 0x9389, 'CjkUnifiedIdeograph9389', [], 'Display', None],
  [ 0x938a, 'CjkUnifiedIdeograph938a', [], 'Display', None],
  [ 0x938b, 'CjkUnifiedIdeograph938b', [], 'Display', None],
  [ 0x938c, 'CjkUnifiedIdeograph938c', [], 'Display', None],
  [ 0x938d, 'CjkUnifiedIdeograph938d', [], 'Display', None],
  [ 0x938e, 'CjkUnifiedIdeograph938e', [], 'Display', None],
  [ 0x938f, 'CjkUnifiedIdeograph938f', [], 'Display', None],
  [ 0x9390, 'CjkUnifiedIdeograph9390', [], 'Display', None],
  [ 0x9391, 'CjkUnifiedIdeograph9391', [], 'Display', None],
  [ 0x9392, 'CjkUnifiedIdeograph9392', [], 'Display', None],
  [ 0x9393, 'CjkUnifiedIdeograph9393', [], 'Display', None],
  [ 0x9394, 'CjkUnifiedIdeograph9394', [], 'Display', None],
  [ 0x9395, 'CjkUnifiedIdeograph9395', [], 'Display', None],
  [ 0x9396, 'CjkUnifiedIdeograph9396', [], 'Display', None],
  [ 0x9397, 'CjkUnifiedIdeograph9397', [], 'Display', None],
  [ 0x9398, 'CjkUnifiedIdeograph9398', [], 'Display', None],
  [ 0x9399, 'CjkUnifiedIdeograph9399', [], 'Display', None],
  [ 0x939a, 'CjkUnifiedIdeograph939a', [], 'Display', None],
  [ 0x939b, 'CjkUnifiedIdeograph939b', [], 'Display', None],
  [ 0x939c, 'CjkUnifiedIdeograph939c', [], 'Display', None],
  [ 0x939d, 'CjkUnifiedIdeograph939d', [], 'Display', None],
  [ 0x939e, 'CjkUnifiedIdeograph939e', [], 'Display', None],
  [ 0x939f, 'CjkUnifiedIdeograph939f', [], 'Display', None],
  [ 0x93a0, 'CjkUnifiedIdeograph93a0', [], 'Display', None],
  [ 0x93a1, 'CjkUnifiedIdeograph93a1', [], 'Display', None],
  [ 0x93a2, 'CjkUnifiedIdeograph93a2', [], 'Display', None],
  [ 0x93a3, 'CjkUnifiedIdeograph93a3', [], 'Display', None],
  [ 0x93a4, 'CjkUnifiedIdeograph93a4', [], 'Display', None],
  [ 0x93a5, 'CjkUnifiedIdeograph93a5', [], 'Display', None],
  [ 0x93a6, 'CjkUnifiedIdeograph93a6', [], 'Display', None],
  [ 0x93a7, 'CjkUnifiedIdeograph93a7', [], 'Display', None],
  [ 0x93a8, 'CjkUnifiedIdeograph93a8', [], 'Display', None],
  [ 0x93a9, 'CjkUnifiedIdeograph93a9', [], 'Display', None],
  [ 0x93aa, 'CjkUnifiedIdeograph93aa', [], 'Display', None],
  [ 0x93ab, 'CjkUnifiedIdeograph93ab', [], 'Display', None],
  [ 0x93ac, 'CjkUnifiedIdeograph93ac', [], 'Display', None],
  [ 0x93ad, 'CjkUnifiedIdeograph93ad', [], 'Display', None],
  [ 0x93ae, 'CjkUnifiedIdeograph93ae', [], 'Display', None],
  [ 0x93af, 'CjkUnifiedIdeograph93af', [], 'Display', None],
  [ 0x93b0, 'CjkUnifiedIdeograph93b0', [], 'Display', None],
  [ 0x93b1, 'CjkUnifiedIdeograph93b1', [], 'Display', None],
  [ 0x93b2, 'CjkUnifiedIdeograph93b2', [], 'Display', None],
  [ 0x93b3, 'CjkUnifiedIdeograph93b3', [], 'Display', None],
  [ 0x93b4, 'CjkUnifiedIdeograph93b4', [], 'Display', None],
  [ 0x93b5, 'CjkUnifiedIdeograph93b5', [], 'Display', None],
  [ 0x93b6, 'CjkUnifiedIdeograph93b6', [], 'Display', None],
  [ 0x93b7, 'CjkUnifiedIdeograph93b7', [], 'Display', None],
  [ 0x93b8, 'CjkUnifiedIdeograph93b8', [], 'Display', None],
  [ 0x93b9, 'CjkUnifiedIdeograph93b9', [], 'Display', None],
  [ 0x93ba, 'CjkUnifiedIdeograph93ba', [], 'Display', None],
  [ 0x93bb, 'CjkUnifiedIdeograph93bb', [], 'Display', None],
  [ 0x93bc, 'CjkUnifiedIdeograph93bc', [], 'Display', None],
  [ 0x93bd, 'CjkUnifiedIdeograph93bd', [], 'Display', None],
  [ 0x93be, 'CjkUnifiedIdeograph93be', [], 'Display', None],
  [ 0x93bf, 'CjkUnifiedIdeograph93bf', [], 'Display', None],
  [ 0x93c0, 'CjkUnifiedIdeograph93c0', [], 'Display', None],
  [ 0x93c1, 'CjkUnifiedIdeograph93c1', [], 'Display', None],
  [ 0x93c2, 'CjkUnifiedIdeograph93c2', [], 'Display', None],
  [ 0x93c3, 'CjkUnifiedIdeograph93c3', [], 'Display', None],
  [ 0x93c4, 'CjkUnifiedIdeograph93c4', [], 'Display', None],
  [ 0x93c5, 'CjkUnifiedIdeograph93c5', [], 'Display', None],
  [ 0x93c6, 'CjkUnifiedIdeograph93c6', [], 'Display', None],
  [ 0x93c7, 'CjkUnifiedIdeograph93c7', [], 'Display', None],
  [ 0x93c8, 'CjkUnifiedIdeograph93c8', [], 'Display', None],
  [ 0x93c9, 'CjkUnifiedIdeograph93c9', [], 'Display', None],
  [ 0x93ca, 'CjkUnifiedIdeograph93ca', [], 'Display', None],
  [ 0x93cb, 'CjkUnifiedIdeograph93cb', [], 'Display', None],
  [ 0x93cc, 'CjkUnifiedIdeograph93cc', [], 'Display', None],
  [ 0x93cd, 'CjkUnifiedIdeograph93cd', [], 'Display', None],
  [ 0x93ce, 'CjkUnifiedIdeograph93ce', [], 'Display', None],
  [ 0x93cf, 'CjkUnifiedIdeograph93cf', [], 'Display', None],
  [ 0x93d0, 'CjkUnifiedIdeograph93d0', [], 'Display', None],
  [ 0x93d1, 'CjkUnifiedIdeograph93d1', [], 'Display', None],
  [ 0x93d2, 'CjkUnifiedIdeograph93d2', [], 'Display', None],
  [ 0x93d3, 'CjkUnifiedIdeograph93d3', [], 'Display', None],
  [ 0x93d4, 'CjkUnifiedIdeograph93d4', [], 'Display', None],
  [ 0x93d5, 'CjkUnifiedIdeograph93d5', [], 'Display', None],
  [ 0x93d6, 'CjkUnifiedIdeograph93d6', [], 'Display', None],
  [ 0x93d7, 'CjkUnifiedIdeograph93d7', [], 'Display', None],
  [ 0x93d8, 'CjkUnifiedIdeograph93d8', [], 'Display', None],
  [ 0x93d9, 'CjkUnifiedIdeograph93d9', [], 'Display', None],
  [ 0x93da, 'CjkUnifiedIdeograph93da', [], 'Display', None],
  [ 0x93db, 'CjkUnifiedIdeograph93db', [], 'Display', None],
  [ 0x93dc, 'CjkUnifiedIdeograph93dc', [], 'Display', None],
  [ 0x93dd, 'CjkUnifiedIdeograph93dd', [], 'Display', None],
  [ 0x93de, 'CjkUnifiedIdeograph93de', [], 'Display', None],
  [ 0x93df, 'CjkUnifiedIdeograph93df', [], 'Display', None],
  [ 0x93e0, 'CjkUnifiedIdeograph93e0', [], 'Display', None],
  [ 0x93e1, 'CjkUnifiedIdeograph93e1', [], 'Display', None],
  [ 0x93e2, 'CjkUnifiedIdeograph93e2', [], 'Display', None],
  [ 0x93e3, 'CjkUnifiedIdeograph93e3', [], 'Display', None],
  [ 0x93e4, 'CjkUnifiedIdeograph93e4', [], 'Display', None],
  [ 0x93e5, 'CjkUnifiedIdeograph93e5', [], 'Display', None],
  [ 0x93e6, 'CjkUnifiedIdeograph93e6', [], 'Display', None],
  [ 0x93e7, 'CjkUnifiedIdeograph93e7', [], 'Display', None],
  [ 0x93e8, 'CjkUnifiedIdeograph93e8', [], 'Display', None],
  [ 0x93e9, 'CjkUnifiedIdeograph93e9', [], 'Display', None],
  [ 0x93ea, 'CjkUnifiedIdeograph93ea', [], 'Display', None],
  [ 0x93eb, 'CjkUnifiedIdeograph93eb', [], 'Display', None],
  [ 0x93ec, 'CjkUnifiedIdeograph93ec', [], 'Display', None],
  [ 0x93ed, 'CjkUnifiedIdeograph93ed', [], 'Display', None],
  [ 0x93ee, 'CjkUnifiedIdeograph93ee', [], 'Display', None],
  [ 0x93ef, 'CjkUnifiedIdeograph93ef', [], 'Display', None],
  [ 0x93f0, 'CjkUnifiedIdeograph93f0', [], 'Display', None],
  [ 0x93f1, 'CjkUnifiedIdeograph93f1', [], 'Display', None],
  [ 0x93f2, 'CjkUnifiedIdeograph93f2', [], 'Display', None],
  [ 0x93f3, 'CjkUnifiedIdeograph93f3', [], 'Display', None],
  [ 0x93f4, 'CjkUnifiedIdeograph93f4', [], 'Display', None],
  [ 0x93f5, 'CjkUnifiedIdeograph93f5', [], 'Display', None],
  [ 0x93f6, 'CjkUnifiedIdeograph93f6', [], 'Display', None],
  [ 0x93f7, 'CjkUnifiedIdeograph93f7', [], 'Display', None],
  [ 0x93f8, 'CjkUnifiedIdeograph93f8', [], 'Display', None],
  [ 0x93f9, 'CjkUnifiedIdeograph93f9', [], 'Display', None],
  [ 0x93fa, 'CjkUnifiedIdeograph93fa', [], 'Display', None],
  [ 0x93fb, 'CjkUnifiedIdeograph93fb', [], 'Display', None],
  [ 0x93fc, 'CjkUnifiedIdeograph93fc', [], 'Display', None],
  [ 0x93fd, 'CjkUnifiedIdeograph93fd', [], 'Display', None],
  [ 0x93fe, 'CjkUnifiedIdeograph93fe', [], 'Display', None],
  [ 0x93ff, 'CjkUnifiedIdeograph93ff', [], 'Display', None],
  [ 0x9400, 'CjkUnifiedIdeograph9400', [], 'Display', None],
  [ 0x9401, 'CjkUnifiedIdeograph9401', [], 'Display', None],
  [ 0x9402, 'CjkUnifiedIdeograph9402', [], 'Display', None],
  [ 0x9403, 'CjkUnifiedIdeograph9403', [], 'Display', None],
  [ 0x9404, 'CjkUnifiedIdeograph9404', [], 'Display', None],
  [ 0x9405, 'CjkUnifiedIdeograph9405', [], 'Display', None],
  [ 0x9406, 'CjkUnifiedIdeograph9406', [], 'Display', None],
  [ 0x9407, 'CjkUnifiedIdeograph9407', [], 'Display', None],
  [ 0x9408, 'CjkUnifiedIdeograph9408', [], 'Display', None],
  [ 0x9409, 'CjkUnifiedIdeograph9409', [], 'Display', None],
  [ 0x940a, 'CjkUnifiedIdeograph940a', [], 'Display', None],
  [ 0x940b, 'CjkUnifiedIdeograph940b', [], 'Display', None],
  [ 0x940c, 'CjkUnifiedIdeograph940c', [], 'Display', None],
  [ 0x940d, 'CjkUnifiedIdeograph940d', [], 'Display', None],
  [ 0x940e, 'CjkUnifiedIdeograph940e', [], 'Display', None],
  [ 0x940f, 'CjkUnifiedIdeograph940f', [], 'Display', None],
  [ 0x9410, 'CjkUnifiedIdeograph9410', [], 'Display', None],
  [ 0x9411, 'CjkUnifiedIdeograph9411', [], 'Display', None],
  [ 0x9412, 'CjkUnifiedIdeograph9412', [], 'Display', None],
  [ 0x9413, 'CjkUnifiedIdeograph9413', [], 'Display', None],
  [ 0x9414, 'CjkUnifiedIdeograph9414', [], 'Display', None],
  [ 0x9415, 'CjkUnifiedIdeograph9415', [], 'Display', None],
  [ 0x9416, 'CjkUnifiedIdeograph9416', [], 'Display', None],
  [ 0x9417, 'CjkUnifiedIdeograph9417', [], 'Display', None],
  [ 0x9418, 'CjkUnifiedIdeograph9418', [], 'Display', None],
  [ 0x9419, 'CjkUnifiedIdeograph9419', [], 'Display', None],
  [ 0x941a, 'CjkUnifiedIdeograph941a', [], 'Display', None],
  [ 0x941b, 'CjkUnifiedIdeograph941b', [], 'Display', None],
  [ 0x941c, 'CjkUnifiedIdeograph941c', [], 'Display', None],
  [ 0x941d, 'CjkUnifiedIdeograph941d', [], 'Display', None],
  [ 0x941e, 'CjkUnifiedIdeograph941e', [], 'Display', None],
  [ 0x941f, 'CjkUnifiedIdeograph941f', [], 'Display', None],
  [ 0x9420, 'CjkUnifiedIdeograph9420', [], 'Display', None],
  [ 0x9421, 'CjkUnifiedIdeograph9421', [], 'Display', None],
  [ 0x9422, 'CjkUnifiedIdeograph9422', [], 'Display', None],
  [ 0x9423, 'CjkUnifiedIdeograph9423', [], 'Display', None],
  [ 0x9424, 'CjkUnifiedIdeograph9424', [], 'Display', None],
  [ 0x9425, 'CjkUnifiedIdeograph9425', [], 'Display', None],
  [ 0x9426, 'CjkUnifiedIdeograph9426', [], 'Display', None],
  [ 0x9427, 'CjkUnifiedIdeograph9427', [], 'Display', None],
  [ 0x9428, 'CjkUnifiedIdeograph9428', [], 'Display', None],
  [ 0x9429, 'CjkUnifiedIdeograph9429', [], 'Display', None],
  [ 0x942a, 'CjkUnifiedIdeograph942a', [], 'Display', None],
  [ 0x942b, 'CjkUnifiedIdeograph942b', [], 'Display', None],
  [ 0x942c, 'CjkUnifiedIdeograph942c', [], 'Display', None],
  [ 0x942d, 'CjkUnifiedIdeograph942d', [], 'Display', None],
  [ 0x942e, 'CjkUnifiedIdeograph942e', [], 'Display', None],
  [ 0x942f, 'CjkUnifiedIdeograph942f', [], 'Display', None],
  [ 0x9430, 'CjkUnifiedIdeograph9430', [], 'Display', None],
  [ 0x9431, 'CjkUnifiedIdeograph9431', [], 'Display', None],
  [ 0x9432, 'CjkUnifiedIdeograph9432', [], 'Display', None],
  [ 0x9433, 'CjkUnifiedIdeograph9433', [], 'Display', None],
  [ 0x9434, 'CjkUnifiedIdeograph9434', [], 'Display', None],
  [ 0x9435, 'CjkUnifiedIdeograph9435', [], 'Display', None],
  [ 0x9436, 'CjkUnifiedIdeograph9436', [], 'Display', None],
  [ 0x9437, 'CjkUnifiedIdeograph9437', [], 'Display', None],
  [ 0x9438, 'CjkUnifiedIdeograph9438', [], 'Display', None],
  [ 0x9439, 'CjkUnifiedIdeograph9439', [], 'Display', None],
  [ 0x943a, 'CjkUnifiedIdeograph943a', [], 'Display', None],
  [ 0x943b, 'CjkUnifiedIdeograph943b', [], 'Display', None],
  [ 0x943c, 'CjkUnifiedIdeograph943c', [], 'Display', None],
  [ 0x943d, 'CjkUnifiedIdeograph943d', [], 'Display', None],
  [ 0x943e, 'CjkUnifiedIdeograph943e', [], 'Display', None],
  [ 0x943f, 'CjkUnifiedIdeograph943f', [], 'Display', None],
  [ 0x9440, 'CjkUnifiedIdeograph9440', [], 'Display', None],
  [ 0x9441, 'CjkUnifiedIdeograph9441', [], 'Display', None],
  [ 0x9442, 'CjkUnifiedIdeograph9442', [], 'Display', None],
  [ 0x9443, 'CjkUnifiedIdeograph9443', [], 'Display', None],
  [ 0x9444, 'CjkUnifiedIdeograph9444', [], 'Display', None],
  [ 0x9445, 'CjkUnifiedIdeograph9445', [], 'Display', None],
  [ 0x9446, 'CjkUnifiedIdeograph9446', [], 'Display', None],
  [ 0x9447, 'CjkUnifiedIdeograph9447', [], 'Display', None],
  [ 0x9448, 'CjkUnifiedIdeograph9448', [], 'Display', None],
  [ 0x9449, 'CjkUnifiedIdeograph9449', [], 'Display', None],
  [ 0x944a, 'CjkUnifiedIdeograph944a', [], 'Display', None],
  [ 0x944b, 'CjkUnifiedIdeograph944b', [], 'Display', None],
  [ 0x944c, 'CjkUnifiedIdeograph944c', [], 'Display', None],
  [ 0x944d, 'CjkUnifiedIdeograph944d', [], 'Display', None],
  [ 0x944e, 'CjkUnifiedIdeograph944e', [], 'Display', None],
  [ 0x944f, 'CjkUnifiedIdeograph944f', [], 'Display', None],
  [ 0x9450, 'CjkUnifiedIdeograph9450', [], 'Display', None],
  [ 0x9451, 'CjkUnifiedIdeograph9451', [], 'Display', None],
  [ 0x9452, 'CjkUnifiedIdeograph9452', [], 'Display', None],
  [ 0x9453, 'CjkUnifiedIdeograph9453', [], 'Display', None],
  [ 0x9454, 'CjkUnifiedIdeograph9454', [], 'Display', None],
  [ 0x9455, 'CjkUnifiedIdeograph9455', [], 'Display', None],
  [ 0x9456, 'CjkUnifiedIdeograph9456', [], 'Display', None],
  [ 0x9457, 'CjkUnifiedIdeograph9457', [], 'Display', None],
  [ 0x9458, 'CjkUnifiedIdeograph9458', [], 'Display', None],
  [ 0x9459, 'CjkUnifiedIdeograph9459', [], 'Display', None],
  [ 0x945a, 'CjkUnifiedIdeograph945a', [], 'Display', None],
  [ 0x945b, 'CjkUnifiedIdeograph945b', [], 'Display', None],
  [ 0x945c, 'CjkUnifiedIdeograph945c', [], 'Display', None],
  [ 0x945d, 'CjkUnifiedIdeograph945d', [], 'Display', None],
  [ 0x945e, 'CjkUnifiedIdeograph945e', [], 'Display', None],
  [ 0x945f, 'CjkUnifiedIdeograph945f', [], 'Display', None],
  [ 0x9460, 'CjkUnifiedIdeograph9460', [], 'Display', None],
  [ 0x9461, 'CjkUnifiedIdeograph9461', [], 'Display', None],
  [ 0x9462, 'CjkUnifiedIdeograph9462', [], 'Display', None],
  [ 0x9463, 'CjkUnifiedIdeograph9463', [], 'Display', None],
  [ 0x9464, 'CjkUnifiedIdeograph9464', [], 'Display', None],
  [ 0x9465, 'CjkUnifiedIdeograph9465', [], 'Display', None],
  [ 0x9466, 'CjkUnifiedIdeograph9466', [], 'Display', None],
  [ 0x9467, 'CjkUnifiedIdeograph9467', [], 'Display', None],
  [ 0x9468, 'CjkUnifiedIdeograph9468', [], 'Display', None],
  [ 0x9469, 'CjkUnifiedIdeograph9469', [], 'Display', None],
  [ 0x946a, 'CjkUnifiedIdeograph946a', [], 'Display', None],
  [ 0x946b, 'CjkUnifiedIdeograph946b', [], 'Display', None],
  [ 0x946c, 'CjkUnifiedIdeograph946c', [], 'Display', None],
  [ 0x946d, 'CjkUnifiedIdeograph946d', [], 'Display', None],
  [ 0x946e, 'CjkUnifiedIdeograph946e', [], 'Display', None],
  [ 0x946f, 'CjkUnifiedIdeograph946f', [], 'Display', None],
  [ 0x9470, 'CjkUnifiedIdeograph9470', [], 'Display', None],
  [ 0x9471, 'CjkUnifiedIdeograph9471', [], 'Display', None],
  [ 0x9472, 'CjkUnifiedIdeograph9472', [], 'Display', None],
  [ 0x9473, 'CjkUnifiedIdeograph9473', [], 'Display', None],
  [ 0x9474, 'CjkUnifiedIdeograph9474', [], 'Display', None],
  [ 0x9475, 'CjkUnifiedIdeograph9475', [], 'Display', None],
  [ 0x9476, 'CjkUnifiedIdeograph9476', [], 'Display', None],
  [ 0x9477, 'CjkUnifiedIdeograph9477', [], 'Display', None],
  [ 0x9478, 'CjkUnifiedIdeograph9478', [], 'Display', None],
  [ 0x9479, 'CjkUnifiedIdeograph9479', [], 'Display', None],
  [ 0x947a, 'CjkUnifiedIdeograph947a', [], 'Display', None],
  [ 0x947b, 'CjkUnifiedIdeograph947b', [], 'Display', None],
  [ 0x947c, 'CjkUnifiedIdeograph947c', [], 'Display', None],
  [ 0x947d, 'CjkUnifiedIdeograph947d', [], 'Display', None],
  [ 0x947e, 'CjkUnifiedIdeograph947e', [], 'Display', None],
  [ 0x947f, 'CjkUnifiedIdeograph947f', [], 'Display', None],
  [ 0x9480, 'CjkUnifiedIdeograph9480', [], 'Display', None],
  [ 0x9481, 'CjkUnifiedIdeograph9481', [], 'Display', None],
  [ 0x9482, 'CjkUnifiedIdeograph9482', [], 'Display', None],
  [ 0x9483, 'CjkUnifiedIdeograph9483', [], 'Display', None],
  [ 0x9484, 'CjkUnifiedIdeograph9484', [], 'Display', None],
  [ 0x9485, 'CjkUnifiedIdeograph9485', [], 'Display', None],
  [ 0x9486, 'CjkUnifiedIdeograph9486', [], 'Display', None],
  [ 0x9487, 'CjkUnifiedIdeograph9487', [], 'Display', None],
  [ 0x9488, 'CjkUnifiedIdeograph9488', [], 'Display', None],
  [ 0x9489, 'CjkUnifiedIdeograph9489', [], 'Display', None],
  [ 0x948a, 'CjkUnifiedIdeograph948a', [], 'Display', None],
  [ 0x948b, 'CjkUnifiedIdeograph948b', [], 'Display', None],
  [ 0x948c, 'CjkUnifiedIdeograph948c', [], 'Display', None],
  [ 0x948d, 'CjkUnifiedIdeograph948d', [], 'Display', None],
  [ 0x948e, 'CjkUnifiedIdeograph948e', [], 'Display', None],
  [ 0x948f, 'CjkUnifiedIdeograph948f', [], 'Display', None],
  [ 0x9490, 'CjkUnifiedIdeograph9490', [], 'Display', None],
  [ 0x9491, 'CjkUnifiedIdeograph9491', [], 'Display', None],
  [ 0x9492, 'CjkUnifiedIdeograph9492', [], 'Display', None],
  [ 0x9493, 'CjkUnifiedIdeograph9493', [], 'Display', None],
  [ 0x9494, 'CjkUnifiedIdeograph9494', [], 'Display', None],
  [ 0x9495, 'CjkUnifiedIdeograph9495', [], 'Display', None],
  [ 0x9496, 'CjkUnifiedIdeograph9496', [], 'Display', None],
  [ 0x9497, 'CjkUnifiedIdeograph9497', [], 'Display', None],
  [ 0x9498, 'CjkUnifiedIdeograph9498', [], 'Display', None],
  [ 0x9499, 'CjkUnifiedIdeograph9499', [], 'Display', None],
  [ 0x949a, 'CjkUnifiedIdeograph949a', [], 'Display', None],
  [ 0x949b, 'CjkUnifiedIdeograph949b', [], 'Display', None],
  [ 0x949c, 'CjkUnifiedIdeograph949c', [], 'Display', None],
  [ 0x949d, 'CjkUnifiedIdeograph949d', [], 'Display', None],
  [ 0x949e, 'CjkUnifiedIdeograph949e', [], 'Display', None],
  [ 0x949f, 'CjkUnifiedIdeograph949f', [], 'Display', None],
  [ 0x94a0, 'CjkUnifiedIdeograph94a0', [], 'Display', None],
  [ 0x94a1, 'CjkUnifiedIdeograph94a1', [], 'Display', None],
  [ 0x94a2, 'CjkUnifiedIdeograph94a2', [], 'Display', None],
  [ 0x94a3, 'CjkUnifiedIdeograph94a3', [], 'Display', None],
  [ 0x94a4, 'CjkUnifiedIdeograph94a4', [], 'Display', None],
  [ 0x94a5, 'CjkUnifiedIdeograph94a5', [], 'Display', None],
  [ 0x94a6, 'CjkUnifiedIdeograph94a6', [], 'Display', None],
  [ 0x94a7, 'CjkUnifiedIdeograph94a7', [], 'Display', None],
  [ 0x94a8, 'CjkUnifiedIdeograph94a8', [], 'Display', None],
  [ 0x94a9, 'CjkUnifiedIdeograph94a9', [], 'Display', None],
  [ 0x94aa, 'CjkUnifiedIdeograph94aa', [], 'Display', None],
  [ 0x94ab, 'CjkUnifiedIdeograph94ab', [], 'Display', None],
  [ 0x94ac, 'CjkUnifiedIdeograph94ac', [], 'Display', None],
  [ 0x94ad, 'CjkUnifiedIdeograph94ad', [], 'Display', None],
  [ 0x94ae, 'CjkUnifiedIdeograph94ae', [], 'Display', None],
  [ 0x94af, 'CjkUnifiedIdeograph94af', [], 'Display', None],
  [ 0x94b0, 'CjkUnifiedIdeograph94b0', [], 'Display', None],
  [ 0x94b1, 'CjkUnifiedIdeograph94b1', [], 'Display', None],
  [ 0x94b2, 'CjkUnifiedIdeograph94b2', [], 'Display', None],
  [ 0x94b3, 'CjkUnifiedIdeograph94b3', [], 'Display', None],
  [ 0x94b4, 'CjkUnifiedIdeograph94b4', [], 'Display', None],
  [ 0x94b5, 'CjkUnifiedIdeograph94b5', [], 'Display', None],
  [ 0x94b6, 'CjkUnifiedIdeograph94b6', [], 'Display', None],
  [ 0x94b7, 'CjkUnifiedIdeograph94b7', [], 'Display', None],
  [ 0x94b8, 'CjkUnifiedIdeograph94b8', [], 'Display', None],
  [ 0x94b9, 'CjkUnifiedIdeograph94b9', [], 'Display', None],
  [ 0x94ba, 'CjkUnifiedIdeograph94ba', [], 'Display', None],
  [ 0x94bb, 'CjkUnifiedIdeograph94bb', [], 'Display', None],
  [ 0x94bc, 'CjkUnifiedIdeograph94bc', [], 'Display', None],
  [ 0x94bd, 'CjkUnifiedIdeograph94bd', [], 'Display', None],
  [ 0x94be, 'CjkUnifiedIdeograph94be', [], 'Display', None],
  [ 0x94bf, 'CjkUnifiedIdeograph94bf', [], 'Display', None],
  [ 0x94c0, 'CjkUnifiedIdeograph94c0', [], 'Display', None],
  [ 0x94c1, 'CjkUnifiedIdeograph94c1', [], 'Display', None],
  [ 0x94c2, 'CjkUnifiedIdeograph94c2', [], 'Display', None],
  [ 0x94c3, 'CjkUnifiedIdeograph94c3', [], 'Display', None],
  [ 0x94c4, 'CjkUnifiedIdeograph94c4', [], 'Display', None],
  [ 0x94c5, 'CjkUnifiedIdeograph94c5', [], 'Display', None],
  [ 0x94c6, 'CjkUnifiedIdeograph94c6', [], 'Display', None],
  [ 0x94c7, 'CjkUnifiedIdeograph94c7', [], 'Display', None],
  [ 0x94c8, 'CjkUnifiedIdeograph94c8', [], 'Display', None],
  [ 0x94c9, 'CjkUnifiedIdeograph94c9', [], 'Display', None],
  [ 0x94ca, 'CjkUnifiedIdeograph94ca', [], 'Display', None],
  [ 0x94cb, 'CjkUnifiedIdeograph94cb', [], 'Display', None],
  [ 0x94cc, 'CjkUnifiedIdeograph94cc', [], 'Display', None],
  [ 0x94cd, 'CjkUnifiedIdeograph94cd', [], 'Display', None],
  [ 0x94ce, 'CjkUnifiedIdeograph94ce', [], 'Display', None],
  [ 0x94cf, 'CjkUnifiedIdeograph94cf', [], 'Display', None],
  [ 0x94d0, 'CjkUnifiedIdeograph94d0', [], 'Display', None],
  [ 0x94d1, 'CjkUnifiedIdeograph94d1', [], 'Display', None],
  [ 0x94d2, 'CjkUnifiedIdeograph94d2', [], 'Display', None],
  [ 0x94d3, 'CjkUnifiedIdeograph94d3', [], 'Display', None],
  [ 0x94d4, 'CjkUnifiedIdeograph94d4', [], 'Display', None],
  [ 0x94d5, 'CjkUnifiedIdeograph94d5', [], 'Display', None],
  [ 0x94d6, 'CjkUnifiedIdeograph94d6', [], 'Display', None],
  [ 0x94d7, 'CjkUnifiedIdeograph94d7', [], 'Display', None],
  [ 0x94d8, 'CjkUnifiedIdeograph94d8', [], 'Display', None],
  [ 0x94d9, 'CjkUnifiedIdeograph94d9', [], 'Display', None],
  [ 0x94da, 'CjkUnifiedIdeograph94da', [], 'Display', None],
  [ 0x94db, 'CjkUnifiedIdeograph94db', [], 'Display', None],
  [ 0x94dc, 'CjkUnifiedIdeograph94dc', [], 'Display', None],
  [ 0x94dd, 'CjkUnifiedIdeograph94dd', [], 'Display', None],
  [ 0x94de, 'CjkUnifiedIdeograph94de', [], 'Display', None],
  [ 0x94df, 'CjkUnifiedIdeograph94df', [], 'Display', None],
  [ 0x94e0, 'CjkUnifiedIdeograph94e0', [], 'Display', None],
  [ 0x94e1, 'CjkUnifiedIdeograph94e1', [], 'Display', None],
  [ 0x94e2, 'CjkUnifiedIdeograph94e2', [], 'Display', None],
  [ 0x94e3, 'CjkUnifiedIdeograph94e3', [], 'Display', None],
  [ 0x94e4, 'CjkUnifiedIdeograph94e4', [], 'Display', None],
  [ 0x94e5, 'CjkUnifiedIdeograph94e5', [], 'Display', None],
  [ 0x94e6, 'CjkUnifiedIdeograph94e6', [], 'Display', None],
  [ 0x94e7, 'CjkUnifiedIdeograph94e7', [], 'Display', None],
  [ 0x94e8, 'CjkUnifiedIdeograph94e8', [], 'Display', None],
  [ 0x94e9, 'CjkUnifiedIdeograph94e9', [], 'Display', None],
  [ 0x94ea, 'CjkUnifiedIdeograph94ea', [], 'Display', None],
  [ 0x94eb, 'CjkUnifiedIdeograph94eb', [], 'Display', None],
  [ 0x94ec, 'CjkUnifiedIdeograph94ec', [], 'Display', None],
  [ 0x94ed, 'CjkUnifiedIdeograph94ed', [], 'Display', None],
  [ 0x94ee, 'CjkUnifiedIdeograph94ee', [], 'Display', None],
  [ 0x94ef, 'CjkUnifiedIdeograph94ef', [], 'Display', None],
  [ 0x94f0, 'CjkUnifiedIdeograph94f0', [], 'Display', None],
  [ 0x94f1, 'CjkUnifiedIdeograph94f1', [], 'Display', None],
  [ 0x94f2, 'CjkUnifiedIdeograph94f2', [], 'Display', None],
  [ 0x94f3, 'CjkUnifiedIdeograph94f3', [], 'Display', None],
  [ 0x94f4, 'CjkUnifiedIdeograph94f4', [], 'Display', None],
  [ 0x94f5, 'CjkUnifiedIdeograph94f5', [], 'Display', None],
  [ 0x94f6, 'CjkUnifiedIdeograph94f6', [], 'Display', None],
  [ 0x94f7, 'CjkUnifiedIdeograph94f7', [], 'Display', None],
  [ 0x94f8, 'CjkUnifiedIdeograph94f8', [], 'Display', None],
  [ 0x94f9, 'CjkUnifiedIdeograph94f9', [], 'Display', None],
  [ 0x94fa, 'CjkUnifiedIdeograph94fa', [], 'Display', None],
  [ 0x94fb, 'CjkUnifiedIdeograph94fb', [], 'Display', None],
  [ 0x94fc, 'CjkUnifiedIdeograph94fc', [], 'Display', None],
  [ 0x94fd, 'CjkUnifiedIdeograph94fd', [], 'Display', None],
  [ 0x94fe, 'CjkUnifiedIdeograph94fe', [], 'Display', None],
  [ 0x94ff, 'CjkUnifiedIdeograph94ff', [], 'Display', None],
  [ 0x9500, 'CjkUnifiedIdeograph9500', [], 'Display', None],
  [ 0x9501, 'CjkUnifiedIdeograph9501', [], 'Display', None],
  [ 0x9502, 'CjkUnifiedIdeograph9502', [], 'Display', None],
  [ 0x9503, 'CjkUnifiedIdeograph9503', [], 'Display', None],
  [ 0x9504, 'CjkUnifiedIdeograph9504', [], 'Display', None],
  [ 0x9505, 'CjkUnifiedIdeograph9505', [], 'Display', None],
  [ 0x9506, 'CjkUnifiedIdeograph9506', [], 'Display', None],
  [ 0x9507, 'CjkUnifiedIdeograph9507', [], 'Display', None],
  [ 0x9508, 'CjkUnifiedIdeograph9508', [], 'Display', None],
  [ 0x9509, 'CjkUnifiedIdeograph9509', [], 'Display', None],
  [ 0x950a, 'CjkUnifiedIdeograph950a', [], 'Display', None],
  [ 0x950b, 'CjkUnifiedIdeograph950b', [], 'Display', None],
  [ 0x950c, 'CjkUnifiedIdeograph950c', [], 'Display', None],
  [ 0x950d, 'CjkUnifiedIdeograph950d', [], 'Display', None],
  [ 0x950e, 'CjkUnifiedIdeograph950e', [], 'Display', None],
  [ 0x950f, 'CjkUnifiedIdeograph950f', [], 'Display', None],
  [ 0x9510, 'CjkUnifiedIdeograph9510', [], 'Display', None],
  [ 0x9511, 'CjkUnifiedIdeograph9511', [], 'Display', None],
  [ 0x9512, 'CjkUnifiedIdeograph9512', [], 'Display', None],
  [ 0x9513, 'CjkUnifiedIdeograph9513', [], 'Display', None],
  [ 0x9514, 'CjkUnifiedIdeograph9514', [], 'Display', None],
  [ 0x9515, 'CjkUnifiedIdeograph9515', [], 'Display', None],
  [ 0x9516, 'CjkUnifiedIdeograph9516', [], 'Display', None],
  [ 0x9517, 'CjkUnifiedIdeograph9517', [], 'Display', None],
  [ 0x9518, 'CjkUnifiedIdeograph9518', [], 'Display', None],
  [ 0x9519, 'CjkUnifiedIdeograph9519', [], 'Display', None],
  [ 0x951a, 'CjkUnifiedIdeograph951a', [], 'Display', None],
  [ 0x951b, 'CjkUnifiedIdeograph951b', [], 'Display', None],
  [ 0x951c, 'CjkUnifiedIdeograph951c', [], 'Display', None],
  [ 0x951d, 'CjkUnifiedIdeograph951d', [], 'Display', None],
  [ 0x951e, 'CjkUnifiedIdeograph951e', [], 'Display', None],
  [ 0x951f, 'CjkUnifiedIdeograph951f', [], 'Display', None],
  [ 0x9520, 'CjkUnifiedIdeograph9520', [], 'Display', None],
  [ 0x9521, 'CjkUnifiedIdeograph9521', [], 'Display', None],
  [ 0x9522, 'CjkUnifiedIdeograph9522', [], 'Display', None],
  [ 0x9523, 'CjkUnifiedIdeograph9523', [], 'Display', None],
  [ 0x9524, 'CjkUnifiedIdeograph9524', [], 'Display', None],
  [ 0x9525, 'CjkUnifiedIdeograph9525', [], 'Display', None],
  [ 0x9526, 'CjkUnifiedIdeograph9526', [], 'Display', None],
  [ 0x9527, 'CjkUnifiedIdeograph9527', [], 'Display', None],
  [ 0x9528, 'CjkUnifiedIdeograph9528', [], 'Display', None],
  [ 0x9529, 'CjkUnifiedIdeograph9529', [], 'Display', None],
  [ 0x952a, 'CjkUnifiedIdeograph952a', [], 'Display', None],
  [ 0x952b, 'CjkUnifiedIdeograph952b', [], 'Display', None],
  [ 0x952c, 'CjkUnifiedIdeograph952c', [], 'Display', None],
  [ 0x952d, 'CjkUnifiedIdeograph952d', [], 'Display', None],
  [ 0x952e, 'CjkUnifiedIdeograph952e', [], 'Display', None],
  [ 0x952f, 'CjkUnifiedIdeograph952f', [], 'Display', None],
  [ 0x9530, 'CjkUnifiedIdeograph9530', [], 'Display', None],
  [ 0x9531, 'CjkUnifiedIdeograph9531', [], 'Display', None],
  [ 0x9532, 'CjkUnifiedIdeograph9532', [], 'Display', None],
  [ 0x9533, 'CjkUnifiedIdeograph9533', [], 'Display', None],
  [ 0x9534, 'CjkUnifiedIdeograph9534', [], 'Display', None],
  [ 0x9535, 'CjkUnifiedIdeograph9535', [], 'Display', None],
  [ 0x9536, 'CjkUnifiedIdeograph9536', [], 'Display', None],
  [ 0x9537, 'CjkUnifiedIdeograph9537', [], 'Display', None],
  [ 0x9538, 'CjkUnifiedIdeograph9538', [], 'Display', None],
  [ 0x9539, 'CjkUnifiedIdeograph9539', [], 'Display', None],
  [ 0x953a, 'CjkUnifiedIdeograph953a', [], 'Display', None],
  [ 0x953b, 'CjkUnifiedIdeograph953b', [], 'Display', None],
  [ 0x953c, 'CjkUnifiedIdeograph953c', [], 'Display', None],
  [ 0x953d, 'CjkUnifiedIdeograph953d', [], 'Display', None],
  [ 0x953e, 'CjkUnifiedIdeograph953e', [], 'Display', None],
  [ 0x953f, 'CjkUnifiedIdeograph953f', [], 'Display', None],
  [ 0x9540, 'CjkUnifiedIdeograph9540', [], 'Display', None],
  [ 0x9541, 'CjkUnifiedIdeograph9541', [], 'Display', None],
  [ 0x9542, 'CjkUnifiedIdeograph9542', [], 'Display', None],
  [ 0x9543, 'CjkUnifiedIdeograph9543', [], 'Display', None],
  [ 0x9544, 'CjkUnifiedIdeograph9544', [], 'Display', None],
  [ 0x9545, 'CjkUnifiedIdeograph9545', [], 'Display', None],
  [ 0x9546, 'CjkUnifiedIdeograph9546', [], 'Display', None],
  [ 0x9547, 'CjkUnifiedIdeograph9547', [], 'Display', None],
  [ 0x9548, 'CjkUnifiedIdeograph9548', [], 'Display', None],
  [ 0x9549, 'CjkUnifiedIdeograph9549', [], 'Display', None],
  [ 0x954a, 'CjkUnifiedIdeograph954a', [], 'Display', None],
  [ 0x954b, 'CjkUnifiedIdeograph954b', [], 'Display', None],
  [ 0x954c, 'CjkUnifiedIdeograph954c', [], 'Display', None],
  [ 0x954d, 'CjkUnifiedIdeograph954d', [], 'Display', None],
  [ 0x954e, 'CjkUnifiedIdeograph954e', [], 'Display', None],
  [ 0x954f, 'CjkUnifiedIdeograph954f', [], 'Display', None],
  [ 0x9550, 'CjkUnifiedIdeograph9550', [], 'Display', None],
  [ 0x9551, 'CjkUnifiedIdeograph9551', [], 'Display', None],
  [ 0x9552, 'CjkUnifiedIdeograph9552', [], 'Display', None],
  [ 0x9553, 'CjkUnifiedIdeograph9553', [], 'Display', None],
  [ 0x9554, 'CjkUnifiedIdeograph9554', [], 'Display', None],
  [ 0x9555, 'CjkUnifiedIdeograph9555', [], 'Display', None],
  [ 0x9556, 'CjkUnifiedIdeograph9556', [], 'Display', None],
  [ 0x9557, 'CjkUnifiedIdeograph9557', [], 'Display', None],
  [ 0x9558, 'CjkUnifiedIdeograph9558', [], 'Display', None],
  [ 0x9559, 'CjkUnifiedIdeograph9559', [], 'Display', None],
  [ 0x955a, 'CjkUnifiedIdeograph955a', [], 'Display', None],
  [ 0x955b, 'CjkUnifiedIdeograph955b', [], 'Display', None],
  [ 0x955c, 'CjkUnifiedIdeograph955c', [], 'Display', None],
  [ 0x955d, 'CjkUnifiedIdeograph955d', [], 'Display', None],
  [ 0x955e, 'CjkUnifiedIdeograph955e', [], 'Display', None],
  [ 0x955f, 'CjkUnifiedIdeograph955f', [], 'Display', None],
  [ 0x9560, 'CjkUnifiedIdeograph9560', [], 'Display', None],
  [ 0x9561, 'CjkUnifiedIdeograph9561', [], 'Display', None],
  [ 0x9562, 'CjkUnifiedIdeograph9562', [], 'Display', None],
  [ 0x9563, 'CjkUnifiedIdeograph9563', [], 'Display', None],
  [ 0x9564, 'CjkUnifiedIdeograph9564', [], 'Display', None],
  [ 0x9565, 'CjkUnifiedIdeograph9565', [], 'Display', None],
  [ 0x9566, 'CjkUnifiedIdeograph9566', [], 'Display', None],
  [ 0x9567, 'CjkUnifiedIdeograph9567', [], 'Display', None],
  [ 0x9568, 'CjkUnifiedIdeograph9568', [], 'Display', None],
  [ 0x9569, 'CjkUnifiedIdeograph9569', [], 'Display', None],
  [ 0x956a, 'CjkUnifiedIdeograph956a', [], 'Display', None],
  [ 0x956b, 'CjkUnifiedIdeograph956b', [], 'Display', None],
  [ 0x956c, 'CjkUnifiedIdeograph956c', [], 'Display', None],
  [ 0x956d, 'CjkUnifiedIdeograph956d', [], 'Display', None],
  [ 0x956e, 'CjkUnifiedIdeograph956e', [], 'Display', None],
  [ 0x956f, 'CjkUnifiedIdeograph956f', [], 'Display', None],
  [ 0x9570, 'CjkUnifiedIdeograph9570', [], 'Display', None],
  [ 0x9571, 'CjkUnifiedIdeograph9571', [], 'Display', None],
  [ 0x9572, 'CjkUnifiedIdeograph9572', [], 'Display', None],
  [ 0x9573, 'CjkUnifiedIdeograph9573', [], 'Display', None],
  [ 0x9574, 'CjkUnifiedIdeograph9574', [], 'Display', None],
  [ 0x9575, 'CjkUnifiedIdeograph9575', [], 'Display', None],
  [ 0x9576, 'CjkUnifiedIdeograph9576', [], 'Display', None],
  [ 0x9577, 'CjkUnifiedIdeograph9577', [], 'Display', None],
  [ 0x9578, 'CjkUnifiedIdeograph9578', [], 'Display', None],
  [ 0x9579, 'CjkUnifiedIdeograph9579', [], 'Display', None],
  [ 0x957a, 'CjkUnifiedIdeograph957a', [], 'Display', None],
  [ 0x957b, 'CjkUnifiedIdeograph957b', [], 'Display', None],
  [ 0x957c, 'CjkUnifiedIdeograph957c', [], 'Display', None],
  [ 0x957d, 'CjkUnifiedIdeograph957d', [], 'Display', None],
  [ 0x957e, 'CjkUnifiedIdeograph957e', [], 'Display', None],
  [ 0x957f, 'CjkUnifiedIdeograph957f', [], 'Display', None],
  [ 0x9580, 'CjkUnifiedIdeograph9580', [], 'Display', None],
  [ 0x9581, 'CjkUnifiedIdeograph9581', [], 'Display', None],
  [ 0x9582, 'CjkUnifiedIdeograph9582', [], 'Display', None],
  [ 0x9583, 'CjkUnifiedIdeograph9583', [], 'Display', None],
  [ 0x9584, 'CjkUnifiedIdeograph9584', [], 'Display', None],
  [ 0x9585, 'CjkUnifiedIdeograph9585', [], 'Display', None],
  [ 0x9586, 'CjkUnifiedIdeograph9586', [], 'Display', None],
  [ 0x9587, 'CjkUnifiedIdeograph9587', [], 'Display', None],
  [ 0x9588, 'CjkUnifiedIdeograph9588', [], 'Display', None],
  [ 0x9589, 'CjkUnifiedIdeograph9589', [], 'Display', None],
  [ 0x958a, 'CjkUnifiedIdeograph958a', [], 'Display', None],
  [ 0x958b, 'CjkUnifiedIdeograph958b', [], 'Display', None],
  [ 0x958c, 'CjkUnifiedIdeograph958c', [], 'Display', None],
  [ 0x958d, 'CjkUnifiedIdeograph958d', [], 'Display', None],
  [ 0x958e, 'CjkUnifiedIdeograph958e', [], 'Display', None],
  [ 0x958f, 'CjkUnifiedIdeograph958f', [], 'Display', None],
  [ 0x9590, 'CjkUnifiedIdeograph9590', [], 'Display', None],
  [ 0x9591, 'CjkUnifiedIdeograph9591', [], 'Display', None],
  [ 0x9592, 'CjkUnifiedIdeograph9592', [], 'Display', None],
  [ 0x9593, 'CjkUnifiedIdeograph9593', [], 'Display', None],
  [ 0x9594, 'CjkUnifiedIdeograph9594', [], 'Display', None],
  [ 0x9595, 'CjkUnifiedIdeograph9595', [], 'Display', None],
  [ 0x9596, 'CjkUnifiedIdeograph9596', [], 'Display', None],
  [ 0x9597, 'CjkUnifiedIdeograph9597', [], 'Display', None],
  [ 0x9598, 'CjkUnifiedIdeograph9598', [], 'Display', None],
  [ 0x9599, 'CjkUnifiedIdeograph9599', [], 'Display', None],
  [ 0x959a, 'CjkUnifiedIdeograph959a', [], 'Display', None],
  [ 0x959b, 'CjkUnifiedIdeograph959b', [], 'Display', None],
  [ 0x959c, 'CjkUnifiedIdeograph959c', [], 'Display', None],
  [ 0x959d, 'CjkUnifiedIdeograph959d', [], 'Display', None],
  [ 0x959e, 'CjkUnifiedIdeograph959e', [], 'Display', None],
  [ 0x959f, 'CjkUnifiedIdeograph959f', [], 'Display', None],
  [ 0x95a0, 'CjkUnifiedIdeograph95a0', [], 'Display', None],
  [ 0x95a1, 'CjkUnifiedIdeograph95a1', [], 'Display', None],
  [ 0x95a2, 'CjkUnifiedIdeograph95a2', [], 'Display', None],
  [ 0x95a3, 'CjkUnifiedIdeograph95a3', [], 'Display', None],
  [ 0x95a4, 'CjkUnifiedIdeograph95a4', [], 'Display', None],
  [ 0x95a5, 'CjkUnifiedIdeograph95a5', [], 'Display', None],
  [ 0x95a6, 'CjkUnifiedIdeograph95a6', [], 'Display', None],
  [ 0x95a7, 'CjkUnifiedIdeograph95a7', [], 'Display', None],
  [ 0x95a8, 'CjkUnifiedIdeograph95a8', [], 'Display', None],
  [ 0x95a9, 'CjkUnifiedIdeograph95a9', [], 'Display', None],
  [ 0x95aa, 'CjkUnifiedIdeograph95aa', [], 'Display', None],
  [ 0x95ab, 'CjkUnifiedIdeograph95ab', [], 'Display', None],
  [ 0x95ac, 'CjkUnifiedIdeograph95ac', [], 'Display', None],
  [ 0x95ad, 'CjkUnifiedIdeograph95ad', [], 'Display', None],
  [ 0x95ae, 'CjkUnifiedIdeograph95ae', [], 'Display', None],
  [ 0x95af, 'CjkUnifiedIdeograph95af', [], 'Display', None],
  [ 0x95b0, 'CjkUnifiedIdeograph95b0', [], 'Display', None],
  [ 0x95b1, 'CjkUnifiedIdeograph95b1', [], 'Display', None],
  [ 0x95b2, 'CjkUnifiedIdeograph95b2', [], 'Display', None],
  [ 0x95b3, 'CjkUnifiedIdeograph95b3', [], 'Display', None],
  [ 0x95b4, 'CjkUnifiedIdeograph95b4', [], 'Display', None],
  [ 0x95b5, 'CjkUnifiedIdeograph95b5', [], 'Display', None],
  [ 0x95b6, 'CjkUnifiedIdeograph95b6', [], 'Display', None],
  [ 0x95b7, 'CjkUnifiedIdeograph95b7', [], 'Display', None],
  [ 0x95b8, 'CjkUnifiedIdeograph95b8', [], 'Display', None],
  [ 0x95b9, 'CjkUnifiedIdeograph95b9', [], 'Display', None],
  [ 0x95ba, 'CjkUnifiedIdeograph95ba', [], 'Display', None],
  [ 0x95bb, 'CjkUnifiedIdeograph95bb', [], 'Display', None],
  [ 0x95bc, 'CjkUnifiedIdeograph95bc', [], 'Display', None],
  [ 0x95bd, 'CjkUnifiedIdeograph95bd', [], 'Display', None],
  [ 0x95be, 'CjkUnifiedIdeograph95be', [], 'Display', None],
  [ 0x95bf, 'CjkUnifiedIdeograph95bf', [], 'Display', None],
  [ 0x95c0, 'CjkUnifiedIdeograph95c0', [], 'Display', None],
  [ 0x95c1, 'CjkUnifiedIdeograph95c1', [], 'Display', None],
  [ 0x95c2, 'CjkUnifiedIdeograph95c2', [], 'Display', None],
  [ 0x95c3, 'CjkUnifiedIdeograph95c3', [], 'Display', None],
  [ 0x95c4, 'CjkUnifiedIdeograph95c4', [], 'Display', None],
  [ 0x95c5, 'CjkUnifiedIdeograph95c5', [], 'Display', None],
  [ 0x95c6, 'CjkUnifiedIdeograph95c6', [], 'Display', None],
  [ 0x95c7, 'CjkUnifiedIdeograph95c7', [], 'Display', None],
  [ 0x95c8, 'CjkUnifiedIdeograph95c8', [], 'Display', None],
  [ 0x95c9, 'CjkUnifiedIdeograph95c9', [], 'Display', None],
  [ 0x95ca, 'CjkUnifiedIdeograph95ca', [], 'Display', None],
  [ 0x95cb, 'CjkUnifiedIdeograph95cb', [], 'Display', None],
  [ 0x95cc, 'CjkUnifiedIdeograph95cc', [], 'Display', None],
  [ 0x95cd, 'CjkUnifiedIdeograph95cd', [], 'Display', None],
  [ 0x95ce, 'CjkUnifiedIdeograph95ce', [], 'Display', None],
  [ 0x95cf, 'CjkUnifiedIdeograph95cf', [], 'Display', None],
  [ 0x95d0, 'CjkUnifiedIdeograph95d0', [], 'Display', None],
  [ 0x95d1, 'CjkUnifiedIdeograph95d1', [], 'Display', None],
  [ 0x95d2, 'CjkUnifiedIdeograph95d2', [], 'Display', None],
  [ 0x95d3, 'CjkUnifiedIdeograph95d3', [], 'Display', None],
  [ 0x95d4, 'CjkUnifiedIdeograph95d4', [], 'Display', None],
  [ 0x95d5, 'CjkUnifiedIdeograph95d5', [], 'Display', None],
  [ 0x95d6, 'CjkUnifiedIdeograph95d6', [], 'Display', None],
  [ 0x95d7, 'CjkUnifiedIdeograph95d7', [], 'Display', None],
  [ 0x95d8, 'CjkUnifiedIdeograph95d8', [], 'Display', None],
  [ 0x95d9, 'CjkUnifiedIdeograph95d9', [], 'Display', None],
  [ 0x95da, 'CjkUnifiedIdeograph95da', [], 'Display', None],
  [ 0x95db, 'CjkUnifiedIdeograph95db', [], 'Display', None],
  [ 0x95dc, 'CjkUnifiedIdeograph95dc', [], 'Display', None],
  [ 0x95dd, 'CjkUnifiedIdeograph95dd', [], 'Display', None],
  [ 0x95de, 'CjkUnifiedIdeograph95de', [], 'Display', None],
  [ 0x95df, 'CjkUnifiedIdeograph95df', [], 'Display', None],
  [ 0x95e0, 'CjkUnifiedIdeograph95e0', [], 'Display', None],
  [ 0x95e1, 'CjkUnifiedIdeograph95e1', [], 'Display', None],
  [ 0x95e2, 'CjkUnifiedIdeograph95e2', [], 'Display', None],
  [ 0x95e3, 'CjkUnifiedIdeograph95e3', [], 'Display', None],
  [ 0x95e4, 'CjkUnifiedIdeograph95e4', [], 'Display', None],
  [ 0x95e5, 'CjkUnifiedIdeograph95e5', [], 'Display', None],
  [ 0x95e6, 'CjkUnifiedIdeograph95e6', [], 'Display', None],
  [ 0x95e7, 'CjkUnifiedIdeograph95e7', [], 'Display', None],
  [ 0x95e8, 'CjkUnifiedIdeograph95e8', [], 'Display', None],
  [ 0x95e9, 'CjkUnifiedIdeograph95e9', [], 'Display', None],
  [ 0x95ea, 'CjkUnifiedIdeograph95ea', [], 'Display', None],
  [ 0x95eb, 'CjkUnifiedIdeograph95eb', [], 'Display', None],
  [ 0x95ec, 'CjkUnifiedIdeograph95ec', [], 'Display', None],
  [ 0x95ed, 'CjkUnifiedIdeograph95ed', [], 'Display', None],
  [ 0x95ee, 'CjkUnifiedIdeograph95ee', [], 'Display', None],
  [ 0x95ef, 'CjkUnifiedIdeograph95ef', [], 'Display', None],
  [ 0x95f0, 'CjkUnifiedIdeograph95f0', [], 'Display', None],
  [ 0x95f1, 'CjkUnifiedIdeograph95f1', [], 'Display', None],
  [ 0x95f2, 'CjkUnifiedIdeograph95f2', [], 'Display', None],
  [ 0x95f3, 'CjkUnifiedIdeograph95f3', [], 'Display', None],
  [ 0x95f4, 'CjkUnifiedIdeograph95f4', [], 'Display', None],
  [ 0x95f5, 'CjkUnifiedIdeograph95f5', [], 'Display', None],
  [ 0x95f6, 'CjkUnifiedIdeograph95f6', [], 'Display', None],
  [ 0x95f7, 'CjkUnifiedIdeograph95f7', [], 'Display', None],
  [ 0x95f8, 'CjkUnifiedIdeograph95f8', [], 'Display', None],
  [ 0x95f9, 'CjkUnifiedIdeograph95f9', [], 'Display', None],
  [ 0x95fa, 'CjkUnifiedIdeograph95fa', [], 'Display', None],
  [ 0x95fb, 'CjkUnifiedIdeograph95fb', [], 'Display', None],
  [ 0x95fc, 'CjkUnifiedIdeograph95fc', [], 'Display', None],
  [ 0x95fd, 'CjkUnifiedIdeograph95fd', [], 'Display', None],
  [ 0x95fe, 'CjkUnifiedIdeograph95fe', [], 'Display', None],
  [ 0x95ff, 'CjkUnifiedIdeograph95ff', [], 'Display', None],
  [ 0x9600, 'CjkUnifiedIdeograph9600', [], 'Display', None],
  [ 0x9601, 'CjkUnifiedIdeograph9601', [], 'Display', None],
  [ 0x9602, 'CjkUnifiedIdeograph9602', [], 'Display', None],
  [ 0x9603, 'CjkUnifiedIdeograph9603', [], 'Display', None],
  [ 0x9604, 'CjkUnifiedIdeograph9604', [], 'Display', None],
  [ 0x9605, 'CjkUnifiedIdeograph9605', [], 'Display', None],
  [ 0x9606, 'CjkUnifiedIdeograph9606', [], 'Display', None],
  [ 0x9607, 'CjkUnifiedIdeograph9607', [], 'Display', None],
  [ 0x9608, 'CjkUnifiedIdeograph9608', [], 'Display', None],
  [ 0x9609, 'CjkUnifiedIdeograph9609', [], 'Display', None],
  [ 0x960a, 'CjkUnifiedIdeograph960a', [], 'Display', None],
  [ 0x960b, 'CjkUnifiedIdeograph960b', [], 'Display', None],
  [ 0x960c, 'CjkUnifiedIdeograph960c', [], 'Display', None],
  [ 0x960d, 'CjkUnifiedIdeograph960d', [], 'Display', None],
  [ 0x960e, 'CjkUnifiedIdeograph960e', [], 'Display', None],
  [ 0x960f, 'CjkUnifiedIdeograph960f', [], 'Display', None],
  [ 0x9610, 'CjkUnifiedIdeograph9610', [], 'Display', None],
  [ 0x9611, 'CjkUnifiedIdeograph9611', [], 'Display', None],
  [ 0x9612, 'CjkUnifiedIdeograph9612', [], 'Display', None],
  [ 0x9613, 'CjkUnifiedIdeograph9613', [], 'Display', None],
  [ 0x9614, 'CjkUnifiedIdeograph9614', [], 'Display', None],
  [ 0x9615, 'CjkUnifiedIdeograph9615', [], 'Display', None],
  [ 0x9616, 'CjkUnifiedIdeograph9616', [], 'Display', None],
  [ 0x9617, 'CjkUnifiedIdeograph9617', [], 'Display', None],
  [ 0x9618, 'CjkUnifiedIdeograph9618', [], 'Display', None],
  [ 0x9619, 'CjkUnifiedIdeograph9619', [], 'Display', None],
  [ 0x961a, 'CjkUnifiedIdeograph961a', [], 'Display', None],
  [ 0x961b, 'CjkUnifiedIdeograph961b', [], 'Display', None],
  [ 0x961c, 'CjkUnifiedIdeograph961c', [], 'Display', None],
  [ 0x961d, 'CjkUnifiedIdeograph961d', [], 'Display', None],
  [ 0x961e, 'CjkUnifiedIdeograph961e', [], 'Display', None],
  [ 0x961f, 'CjkUnifiedIdeograph961f', [], 'Display', None],
  [ 0x9620, 'CjkUnifiedIdeograph9620', [], 'Display', None],
  [ 0x9621, 'CjkUnifiedIdeograph9621', [], 'Display', None],
  [ 0x9622, 'CjkUnifiedIdeograph9622', [], 'Display', None],
  [ 0x9623, 'CjkUnifiedIdeograph9623', [], 'Display', None],
  [ 0x9624, 'CjkUnifiedIdeograph9624', [], 'Display', None],
  [ 0x9625, 'CjkUnifiedIdeograph9625', [], 'Display', None],
  [ 0x9626, 'CjkUnifiedIdeograph9626', [], 'Display', None],
  [ 0x9627, 'CjkUnifiedIdeograph9627', [], 'Display', None],
  [ 0x9628, 'CjkUnifiedIdeograph9628', [], 'Display', None],
  [ 0x9629, 'CjkUnifiedIdeograph9629', [], 'Display', None],
  [ 0x962a, 'CjkUnifiedIdeograph962a', [], 'Display', None],
  [ 0x962b, 'CjkUnifiedIdeograph962b', [], 'Display', None],
  [ 0x962c, 'CjkUnifiedIdeograph962c', [], 'Display', None],
  [ 0x962d, 'CjkUnifiedIdeograph962d', [], 'Display', None],
  [ 0x962e, 'CjkUnifiedIdeograph962e', [], 'Display', None],
  [ 0x962f, 'CjkUnifiedIdeograph962f', [], 'Display', None],
  [ 0x9630, 'CjkUnifiedIdeograph9630', [], 'Display', None],
  [ 0x9631, 'CjkUnifiedIdeograph9631', [], 'Display', None],
  [ 0x9632, 'CjkUnifiedIdeograph9632', [], 'Display', None],
  [ 0x9633, 'CjkUnifiedIdeograph9633', [], 'Display', None],
  [ 0x9634, 'CjkUnifiedIdeograph9634', [], 'Display', None],
  [ 0x9635, 'CjkUnifiedIdeograph9635', [], 'Display', None],
  [ 0x9636, 'CjkUnifiedIdeograph9636', [], 'Display', None],
  [ 0x9637, 'CjkUnifiedIdeograph9637', [], 'Display', None],
  [ 0x9638, 'CjkUnifiedIdeograph9638', [], 'Display', None],
  [ 0x9639, 'CjkUnifiedIdeograph9639', [], 'Display', None],
  [ 0x963a, 'CjkUnifiedIdeograph963a', [], 'Display', None],
  [ 0x963b, 'CjkUnifiedIdeograph963b', [], 'Display', None],
  [ 0x963c, 'CjkUnifiedIdeograph963c', [], 'Display', None],
  [ 0x963d, 'CjkUnifiedIdeograph963d', [], 'Display', None],
  [ 0x963e, 'CjkUnifiedIdeograph963e', [], 'Display', None],
  [ 0x963f, 'CjkUnifiedIdeograph963f', [], 'Display', None],
  [ 0x9640, 'CjkUnifiedIdeograph9640', [], 'Display', None],
  [ 0x9641, 'CjkUnifiedIdeograph9641', [], 'Display', None],
  [ 0x9642, 'CjkUnifiedIdeograph9642', [], 'Display', None],
  [ 0x9643, 'CjkUnifiedIdeograph9643', [], 'Display', None],
  [ 0x9644, 'CjkUnifiedIdeograph9644', [], 'Display', None],
  [ 0x9645, 'CjkUnifiedIdeograph9645', [], 'Display', None],
  [ 0x9646, 'CjkUnifiedIdeograph9646', [], 'Display', None],
  [ 0x9647, 'CjkUnifiedIdeograph9647', [], 'Display', None],
  [ 0x9648, 'CjkUnifiedIdeograph9648', [], 'Display', None],
  [ 0x9649, 'CjkUnifiedIdeograph9649', [], 'Display', None],
  [ 0x964a, 'CjkUnifiedIdeograph964a', [], 'Display', None],
  [ 0x964b, 'CjkUnifiedIdeograph964b', [], 'Display', None],
  [ 0x964c, 'CjkUnifiedIdeograph964c', [], 'Display', None],
  [ 0x964d, 'CjkUnifiedIdeograph964d', [], 'Display', None],
  [ 0x964e, 'CjkUnifiedIdeograph964e', [], 'Display', None],
  [ 0x964f, 'CjkUnifiedIdeograph964f', [], 'Display', None],
  [ 0x9650, 'CjkUnifiedIdeograph9650', [], 'Display', None],
  [ 0x9651, 'CjkUnifiedIdeograph9651', [], 'Display', None],
  [ 0x9652, 'CjkUnifiedIdeograph9652', [], 'Display', None],
  [ 0x9653, 'CjkUnifiedIdeograph9653', [], 'Display', None],
  [ 0x9654, 'CjkUnifiedIdeograph9654', [], 'Display', None],
  [ 0x9655, 'CjkUnifiedIdeograph9655', [], 'Display', None],
  [ 0x9656, 'CjkUnifiedIdeograph9656', [], 'Display', None],
  [ 0x9657, 'CjkUnifiedIdeograph9657', [], 'Display', None],
  [ 0x9658, 'CjkUnifiedIdeograph9658', [], 'Display', None],
  [ 0x9659, 'CjkUnifiedIdeograph9659', [], 'Display', None],
  [ 0x965a, 'CjkUnifiedIdeograph965a', [], 'Display', None],
  [ 0x965b, 'CjkUnifiedIdeograph965b', [], 'Display', None],
  [ 0x965c, 'CjkUnifiedIdeograph965c', [], 'Display', None],
  [ 0x965d, 'CjkUnifiedIdeograph965d', [], 'Display', None],
  [ 0x965e, 'CjkUnifiedIdeograph965e', [], 'Display', None],
  [ 0x965f, 'CjkUnifiedIdeograph965f', [], 'Display', None],
  [ 0x9660, 'CjkUnifiedIdeograph9660', [], 'Display', None],
  [ 0x9661, 'CjkUnifiedIdeograph9661', [], 'Display', None],
  [ 0x9662, 'CjkUnifiedIdeograph9662', [], 'Display', None],
  [ 0x9663, 'CjkUnifiedIdeograph9663', [], 'Display', None],
  [ 0x9664, 'CjkUnifiedIdeograph9664', [], 'Display', None],
  [ 0x9665, 'CjkUnifiedIdeograph9665', [], 'Display', None],
  [ 0x9666, 'CjkUnifiedIdeograph9666', [], 'Display', None],
  [ 0x9667, 'CjkUnifiedIdeograph9667', [], 'Display', None],
  [ 0x9668, 'CjkUnifiedIdeograph9668', [], 'Display', None],
  [ 0x9669, 'CjkUnifiedIdeograph9669', [], 'Display', None],
  [ 0x966a, 'CjkUnifiedIdeograph966a', [], 'Display', None],
  [ 0x966b, 'CjkUnifiedIdeograph966b', [], 'Display', None],
  [ 0x966c, 'CjkUnifiedIdeograph966c', [], 'Display', None],
  [ 0x966d, 'CjkUnifiedIdeograph966d', [], 'Display', None],
  [ 0x966e, 'CjkUnifiedIdeograph966e', [], 'Display', None],
  [ 0x966f, 'CjkUnifiedIdeograph966f', [], 'Display', None],
  [ 0x9670, 'CjkUnifiedIdeograph9670', [], 'Display', None],
  [ 0x9671, 'CjkUnifiedIdeograph9671', [], 'Display', None],
  [ 0x9672, 'CjkUnifiedIdeograph9672', [], 'Display', None],
  [ 0x9673, 'CjkUnifiedIdeograph9673', [], 'Display', None],
  [ 0x9674, 'CjkUnifiedIdeograph9674', [], 'Display', None],
  [ 0x9675, 'CjkUnifiedIdeograph9675', [], 'Display', None],
  [ 0x9676, 'CjkUnifiedIdeograph9676', [], 'Display', None],
  [ 0x9677, 'CjkUnifiedIdeograph9677', [], 'Display', None],
  [ 0x9678, 'CjkUnifiedIdeograph9678', [], 'Display', None],
  [ 0x9679, 'CjkUnifiedIdeograph9679', [], 'Display', None],
  [ 0x967a, 'CjkUnifiedIdeograph967a', [], 'Display', None],
  [ 0x967b, 'CjkUnifiedIdeograph967b', [], 'Display', None],
  [ 0x967c, 'CjkUnifiedIdeograph967c', [], 'Display', None],
  [ 0x967d, 'CjkUnifiedIdeograph967d', [], 'Display', None],
  [ 0x967e, 'CjkUnifiedIdeograph967e', [], 'Display', None],
  [ 0x967f, 'CjkUnifiedIdeograph967f', [], 'Display', None],
  [ 0x9680, 'CjkUnifiedIdeograph9680', [], 'Display', None],
  [ 0x9681, 'CjkUnifiedIdeograph9681', [], 'Display', None],
  [ 0x9682, 'CjkUnifiedIdeograph9682', [], 'Display', None],
  [ 0x9683, 'CjkUnifiedIdeograph9683', [], 'Display', None],
  [ 0x9684, 'CjkUnifiedIdeograph9684', [], 'Display', None],
  [ 0x9685, 'CjkUnifiedIdeograph9685', [], 'Display', None],
  [ 0x9686, 'CjkUnifiedIdeograph9686', [], 'Display', None],
  [ 0x9687, 'CjkUnifiedIdeograph9687', [], 'Display', None],
  [ 0x9688, 'CjkUnifiedIdeograph9688', [], 'Display', None],
  [ 0x9689, 'CjkUnifiedIdeograph9689', [], 'Display', None],
  [ 0x968a, 'CjkUnifiedIdeograph968a', [], 'Display', None],
  [ 0x968b, 'CjkUnifiedIdeograph968b', [], 'Display', None],
  [ 0x968c, 'CjkUnifiedIdeograph968c', [], 'Display', None],
  [ 0x968d, 'CjkUnifiedIdeograph968d', [], 'Display', None],
  [ 0x968e, 'CjkUnifiedIdeograph968e', [], 'Display', None],
  [ 0x968f, 'CjkUnifiedIdeograph968f', [], 'Display', None],
  [ 0x9690, 'CjkUnifiedIdeograph9690', [], 'Display', None],
  [ 0x9691, 'CjkUnifiedIdeograph9691', [], 'Display', None],
  [ 0x9692, 'CjkUnifiedIdeograph9692', [], 'Display', None],
  [ 0x9693, 'CjkUnifiedIdeograph9693', [], 'Display', None],
  [ 0x9694, 'CjkUnifiedIdeograph9694', [], 'Display', None],
  [ 0x9695, 'CjkUnifiedIdeograph9695', [], 'Display', None],
  [ 0x9696, 'CjkUnifiedIdeograph9696', [], 'Display', None],
  [ 0x9697, 'CjkUnifiedIdeograph9697', [], 'Display', None],
  [ 0x9698, 'CjkUnifiedIdeograph9698', [], 'Display', None],
  [ 0x9699, 'CjkUnifiedIdeograph9699', [], 'Display', None],
  [ 0x969a, 'CjkUnifiedIdeograph969a', [], 'Display', None],
  [ 0x969b, 'CjkUnifiedIdeograph969b', [], 'Display', None],
  [ 0x969c, 'CjkUnifiedIdeograph969c', [], 'Display', None],
  [ 0x969d, 'CjkUnifiedIdeograph969d', [], 'Display', None],
  [ 0x969e, 'CjkUnifiedIdeograph969e', [], 'Display', None],
  [ 0x969f, 'CjkUnifiedIdeograph969f', [], 'Display', None],
  [ 0x96a0, 'CjkUnifiedIdeograph96a0', [], 'Display', None],
  [ 0x96a1, 'CjkUnifiedIdeograph96a1', [], 'Display', None],
  [ 0x96a2, 'CjkUnifiedIdeograph96a2', [], 'Display', None],
  [ 0x96a3, 'CjkUnifiedIdeograph96a3', [], 'Display', None],
  [ 0x96a4, 'CjkUnifiedIdeograph96a4', [], 'Display', None],
  [ 0x96a5, 'CjkUnifiedIdeograph96a5', [], 'Display', None],
  [ 0x96a6, 'CjkUnifiedIdeograph96a6', [], 'Display', None],
  [ 0x96a7, 'CjkUnifiedIdeograph96a7', [], 'Display', None],
  [ 0x96a8, 'CjkUnifiedIdeograph96a8', [], 'Display', None],
  [ 0x96a9, 'CjkUnifiedIdeograph96a9', [], 'Display', None],
  [ 0x96aa, 'CjkUnifiedIdeograph96aa', [], 'Display', None],
  [ 0x96ab, 'CjkUnifiedIdeograph96ab', [], 'Display', None],
  [ 0x96ac, 'CjkUnifiedIdeograph96ac', [], 'Display', None],
  [ 0x96ad, 'CjkUnifiedIdeograph96ad', [], 'Display', None],
  [ 0x96ae, 'CjkUnifiedIdeograph96ae', [], 'Display', None],
  [ 0x96af, 'CjkUnifiedIdeograph96af', [], 'Display', None],
  [ 0x96b0, 'CjkUnifiedIdeograph96b0', [], 'Display', None],
  [ 0x96b1, 'CjkUnifiedIdeograph96b1', [], 'Display', None],
  [ 0x96b2, 'CjkUnifiedIdeograph96b2', [], 'Display', None],
  [ 0x96b3, 'CjkUnifiedIdeograph96b3', [], 'Display', None],
  [ 0x96b4, 'CjkUnifiedIdeograph96b4', [], 'Display', None],
  [ 0x96b5, 'CjkUnifiedIdeograph96b5', [], 'Display', None],
  [ 0x96b6, 'CjkUnifiedIdeograph96b6', [], 'Display', None],
  [ 0x96b7, 'CjkUnifiedIdeograph96b7', [], 'Display', None],
  [ 0x96b8, 'CjkUnifiedIdeograph96b8', [], 'Display', None],
  [ 0x96b9, 'CjkUnifiedIdeograph96b9', [], 'Display', None],
  [ 0x96ba, 'CjkUnifiedIdeograph96ba', [], 'Display', None],
  [ 0x96bb, 'CjkUnifiedIdeograph96bb', [], 'Display', None],
  [ 0x96bc, 'CjkUnifiedIdeograph96bc', [], 'Display', None],
  [ 0x96bd, 'CjkUnifiedIdeograph96bd', [], 'Display', None],
  [ 0x96be, 'CjkUnifiedIdeograph96be', [], 'Display', None],
  [ 0x96bf, 'CjkUnifiedIdeograph96bf', [], 'Display', None],
  [ 0x96c0, 'CjkUnifiedIdeograph96c0', [], 'Display', None],
  [ 0x96c1, 'CjkUnifiedIdeograph96c1', [], 'Display', None],
  [ 0x96c2, 'CjkUnifiedIdeograph96c2', [], 'Display', None],
  [ 0x96c3, 'CjkUnifiedIdeograph96c3', [], 'Display', None],
  [ 0x96c4, 'CjkUnifiedIdeograph96c4', [], 'Display', None],
  [ 0x96c5, 'CjkUnifiedIdeograph96c5', [], 'Display', None],
  [ 0x96c6, 'CjkUnifiedIdeograph96c6', [], 'Display', None],
  [ 0x96c7, 'CjkUnifiedIdeograph96c7', [], 'Display', None],
  [ 0x96c8, 'CjkUnifiedIdeograph96c8', [], 'Display', None],
  [ 0x96c9, 'CjkUnifiedIdeograph96c9', [], 'Display', None],
  [ 0x96ca, 'CjkUnifiedIdeograph96ca', [], 'Display', None],
  [ 0x96cb, 'CjkUnifiedIdeograph96cb', [], 'Display', None],
  [ 0x96cc, 'CjkUnifiedIdeograph96cc', [], 'Display', None],
  [ 0x96cd, 'CjkUnifiedIdeograph96cd', [], 'Display', None],
  [ 0x96ce, 'CjkUnifiedIdeograph96ce', [], 'Display', None],
  [ 0x96cf, 'CjkUnifiedIdeograph96cf', [], 'Display', None],
  [ 0x96d0, 'CjkUnifiedIdeograph96d0', [], 'Display', None],
  [ 0x96d1, 'CjkUnifiedIdeograph96d1', [], 'Display', None],
  [ 0x96d2, 'CjkUnifiedIdeograph96d2', [], 'Display', None],
  [ 0x96d3, 'CjkUnifiedIdeograph96d3', [], 'Display', None],
  [ 0x96d4, 'CjkUnifiedIdeograph96d4', [], 'Display', None],
  [ 0x96d5, 'CjkUnifiedIdeograph96d5', [], 'Display', None],
  [ 0x96d6, 'CjkUnifiedIdeograph96d6', [], 'Display', None],
  [ 0x96d7, 'CjkUnifiedIdeograph96d7', [], 'Display', None],
  [ 0x96d8, 'CjkUnifiedIdeograph96d8', [], 'Display', None],
  [ 0x96d9, 'CjkUnifiedIdeograph96d9', [], 'Display', None],
  [ 0x96da, 'CjkUnifiedIdeograph96da', [], 'Display', None],
  [ 0x96db, 'CjkUnifiedIdeograph96db', [], 'Display', None],
  [ 0x96dc, 'CjkUnifiedIdeograph96dc', [], 'Display', None],
  [ 0x96dd, 'CjkUnifiedIdeograph96dd', [], 'Display', None],
  [ 0x96de, 'CjkUnifiedIdeograph96de', [], 'Display', None],
  [ 0x96df, 'CjkUnifiedIdeograph96df', [], 'Display', None],
  [ 0x96e0, 'CjkUnifiedIdeograph96e0', [], 'Display', None],
  [ 0x96e1, 'CjkUnifiedIdeograph96e1', [], 'Display', None],
  [ 0x96e2, 'CjkUnifiedIdeograph96e2', [], 'Display', None],
  [ 0x96e3, 'CjkUnifiedIdeograph96e3', [], 'Display', None],
  [ 0x96e4, 'CjkUnifiedIdeograph96e4', [], 'Display', None],
  [ 0x96e5, 'CjkUnifiedIdeograph96e5', [], 'Display', None],
  [ 0x96e6, 'CjkUnifiedIdeograph96e6', [], 'Display', None],
  [ 0x96e7, 'CjkUnifiedIdeograph96e7', [], 'Display', None],
  [ 0x96e8, 'CjkUnifiedIdeograph96e8', [], 'Display', None],
  [ 0x96e9, 'CjkUnifiedIdeograph96e9', [], 'Display', None],
  [ 0x96ea, 'CjkUnifiedIdeograph96ea', [], 'Display', None],
  [ 0x96eb, 'CjkUnifiedIdeograph96eb', [], 'Display', None],
  [ 0x96ec, 'CjkUnifiedIdeograph96ec', [], 'Display', None],
  [ 0x96ed, 'CjkUnifiedIdeograph96ed', [], 'Display', None],
  [ 0x96ee, 'CjkUnifiedIdeograph96ee', [], 'Display', None],
  [ 0x96ef, 'CjkUnifiedIdeograph96ef', [], 'Display', None],
  [ 0x96f0, 'CjkUnifiedIdeograph96f0', [], 'Display', None],
  [ 0x96f1, 'CjkUnifiedIdeograph96f1', [], 'Display', None],
  [ 0x96f2, 'CjkUnifiedIdeograph96f2', [], 'Display', None],
  [ 0x96f3, 'CjkUnifiedIdeograph96f3', [], 'Display', None],
  [ 0x96f4, 'CjkUnifiedIdeograph96f4', [], 'Display', None],
  [ 0x96f5, 'CjkUnifiedIdeograph96f5', [], 'Display', None],
  [ 0x96f6, 'CjkUnifiedIdeograph96f6', [], 'Display', None],
  [ 0x96f7, 'CjkUnifiedIdeograph96f7', [], 'Display', None],
  [ 0x96f8, 'CjkUnifiedIdeograph96f8', [], 'Display', None],
  [ 0x96f9, 'CjkUnifiedIdeograph96f9', [], 'Display', None],
  [ 0x96fa, 'CjkUnifiedIdeograph96fa', [], 'Display', None],
  [ 0x96fb, 'CjkUnifiedIdeograph96fb', [], 'Display', None],
  [ 0x96fc, 'CjkUnifiedIdeograph96fc', [], 'Display', None],
  [ 0x96fd, 'CjkUnifiedIdeograph96fd', [], 'Display', None],
  [ 0x96fe, 'CjkUnifiedIdeograph96fe', [], 'Display', None],
  [ 0x96ff, 'CjkUnifiedIdeograph96ff', [], 'Display', None],
  [ 0x9700, 'CjkUnifiedIdeograph9700', [], 'Display', None],
  [ 0x9701, 'CjkUnifiedIdeograph9701', [], 'Display', None],
  [ 0x9702, 'CjkUnifiedIdeograph9702', [], 'Display', None],
  [ 0x9703, 'CjkUnifiedIdeograph9703', [], 'Display', None],
  [ 0x9704, 'CjkUnifiedIdeograph9704', [], 'Display', None],
  [ 0x9705, 'CjkUnifiedIdeograph9705', [], 'Display', None],
  [ 0x9706, 'CjkUnifiedIdeograph9706', [], 'Display', None],
  [ 0x9707, 'CjkUnifiedIdeograph9707', [], 'Display', None],
  [ 0x9708, 'CjkUnifiedIdeograph9708', [], 'Display', None],
  [ 0x9709, 'CjkUnifiedIdeograph9709', [], 'Display', None],
  [ 0x970a, 'CjkUnifiedIdeograph970a', [], 'Display', None],
  [ 0x970b, 'CjkUnifiedIdeograph970b', [], 'Display', None],
  [ 0x970c, 'CjkUnifiedIdeograph970c', [], 'Display', None],
  [ 0x970d, 'CjkUnifiedIdeograph970d', [], 'Display', None],
  [ 0x970e, 'CjkUnifiedIdeograph970e', [], 'Display', None],
  [ 0x970f, 'CjkUnifiedIdeograph970f', [], 'Display', None],
  [ 0x9710, 'CjkUnifiedIdeograph9710', [], 'Display', None],
  [ 0x9711, 'CjkUnifiedIdeograph9711', [], 'Display', None],
  [ 0x9712, 'CjkUnifiedIdeograph9712', [], 'Display', None],
  [ 0x9713, 'CjkUnifiedIdeograph9713', [], 'Display', None],
  [ 0x9714, 'CjkUnifiedIdeograph9714', [], 'Display', None],
  [ 0x9715, 'CjkUnifiedIdeograph9715', [], 'Display', None],
  [ 0x9716, 'CjkUnifiedIdeograph9716', [], 'Display', None],
  [ 0x9717, 'CjkUnifiedIdeograph9717', [], 'Display', None],
  [ 0x9718, 'CjkUnifiedIdeograph9718', [], 'Display', None],
  [ 0x9719, 'CjkUnifiedIdeograph9719', [], 'Display', None],
  [ 0x971a, 'CjkUnifiedIdeograph971a', [], 'Display', None],
  [ 0x971b, 'CjkUnifiedIdeograph971b', [], 'Display', None],
  [ 0x971c, 'CjkUnifiedIdeograph971c', [], 'Display', None],
  [ 0x971d, 'CjkUnifiedIdeograph971d', [], 'Display', None],
  [ 0x971e, 'CjkUnifiedIdeograph971e', [], 'Display', None],
  [ 0x971f, 'CjkUnifiedIdeograph971f', [], 'Display', None],
  [ 0x9720, 'CjkUnifiedIdeograph9720', [], 'Display', None],
  [ 0x9721, 'CjkUnifiedIdeograph9721', [], 'Display', None],
  [ 0x9722, 'CjkUnifiedIdeograph9722', [], 'Display', None],
  [ 0x9723, 'CjkUnifiedIdeograph9723', [], 'Display', None],
  [ 0x9724, 'CjkUnifiedIdeograph9724', [], 'Display', None],
  [ 0x9725, 'CjkUnifiedIdeograph9725', [], 'Display', None],
  [ 0x9726, 'CjkUnifiedIdeograph9726', [], 'Display', None],
  [ 0x9727, 'CjkUnifiedIdeograph9727', [], 'Display', None],
  [ 0x9728, 'CjkUnifiedIdeograph9728', [], 'Display', None],
  [ 0x9729, 'CjkUnifiedIdeograph9729', [], 'Display', None],
  [ 0x972a, 'CjkUnifiedIdeograph972a', [], 'Display', None],
  [ 0x972b, 'CjkUnifiedIdeograph972b', [], 'Display', None],
  [ 0x972c, 'CjkUnifiedIdeograph972c', [], 'Display', None],
  [ 0x972d, 'CjkUnifiedIdeograph972d', [], 'Display', None],
  [ 0x972e, 'CjkUnifiedIdeograph972e', [], 'Display', None],
  [ 0x972f, 'CjkUnifiedIdeograph972f', [], 'Display', None],
  [ 0x9730, 'CjkUnifiedIdeograph9730', [], 'Display', None],
  [ 0x9731, 'CjkUnifiedIdeograph9731', [], 'Display', None],
  [ 0x9732, 'CjkUnifiedIdeograph9732', [], 'Display', None],
  [ 0x9733, 'CjkUnifiedIdeograph9733', [], 'Display', None],
  [ 0x9734, 'CjkUnifiedIdeograph9734', [], 'Display', None],
  [ 0x9735, 'CjkUnifiedIdeograph9735', [], 'Display', None],
  [ 0x9736, 'CjkUnifiedIdeograph9736', [], 'Display', None],
  [ 0x9737, 'CjkUnifiedIdeograph9737', [], 'Display', None],
  [ 0x9738, 'CjkUnifiedIdeograph9738', [], 'Display', None],
  [ 0x9739, 'CjkUnifiedIdeograph9739', [], 'Display', None],
  [ 0x973a, 'CjkUnifiedIdeograph973a', [], 'Display', None],
  [ 0x973b, 'CjkUnifiedIdeograph973b', [], 'Display', None],
  [ 0x973c, 'CjkUnifiedIdeograph973c', [], 'Display', None],
  [ 0x973d, 'CjkUnifiedIdeograph973d', [], 'Display', None],
  [ 0x973e, 'CjkUnifiedIdeograph973e', [], 'Display', None],
  [ 0x973f, 'CjkUnifiedIdeograph973f', [], 'Display', None],
  [ 0x9740, 'CjkUnifiedIdeograph9740', [], 'Display', None],
  [ 0x9741, 'CjkUnifiedIdeograph9741', [], 'Display', None],
  [ 0x9742, 'CjkUnifiedIdeograph9742', [], 'Display', None],
  [ 0x9743, 'CjkUnifiedIdeograph9743', [], 'Display', None],
  [ 0x9744, 'CjkUnifiedIdeograph9744', [], 'Display', None],
  [ 0x9745, 'CjkUnifiedIdeograph9745', [], 'Display', None],
  [ 0x9746, 'CjkUnifiedIdeograph9746', [], 'Display', None],
  [ 0x9747, 'CjkUnifiedIdeograph9747', [], 'Display', None],
  [ 0x9748, 'CjkUnifiedIdeograph9748', [], 'Display', None],
  [ 0x9749, 'CjkUnifiedIdeograph9749', [], 'Display', None],
  [ 0x974a, 'CjkUnifiedIdeograph974a', [], 'Display', None],
  [ 0x974b, 'CjkUnifiedIdeograph974b', [], 'Display', None],
  [ 0x974c, 'CjkUnifiedIdeograph974c', [], 'Display', None],
  [ 0x974d, 'CjkUnifiedIdeograph974d', [], 'Display', None],
  [ 0x974e, 'CjkUnifiedIdeograph974e', [], 'Display', None],
  [ 0x974f, 'CjkUnifiedIdeograph974f', [], 'Display', None],
  [ 0x9750, 'CjkUnifiedIdeograph9750', [], 'Display', None],
  [ 0x9751, 'CjkUnifiedIdeograph9751', [], 'Display', None],
  [ 0x9752, 'CjkUnifiedIdeograph9752', [], 'Display', None],
  [ 0x9753, 'CjkUnifiedIdeograph9753', [], 'Display', None],
  [ 0x9754, 'CjkUnifiedIdeograph9754', [], 'Display', None],
  [ 0x9755, 'CjkUnifiedIdeograph9755', [], 'Display', None],
  [ 0x9756, 'CjkUnifiedIdeograph9756', [], 'Display', None],
  [ 0x9757, 'CjkUnifiedIdeograph9757', [], 'Display', None],
  [ 0x9758, 'CjkUnifiedIdeograph9758', [], 'Display', None],
  [ 0x9759, 'CjkUnifiedIdeograph9759', [], 'Display', None],
  [ 0x975a, 'CjkUnifiedIdeograph975a', [], 'Display', None],
  [ 0x975b, 'CjkUnifiedIdeograph975b', [], 'Display', None],
  [ 0x975c, 'CjkUnifiedIdeograph975c', [], 'Display', None],
  [ 0x975d, 'CjkUnifiedIdeograph975d', [], 'Display', None],
  [ 0x975e, 'CjkUnifiedIdeograph975e', [], 'Display', None],
  [ 0x975f, 'CjkUnifiedIdeograph975f', [], 'Display', None],
  [ 0x9760, 'CjkUnifiedIdeograph9760', [], 'Display', None],
  [ 0x9761, 'CjkUnifiedIdeograph9761', [], 'Display', None],
  [ 0x9762, 'CjkUnifiedIdeograph9762', [], 'Display', None],
  [ 0x9763, 'CjkUnifiedIdeograph9763', [], 'Display', None],
  [ 0x9764, 'CjkUnifiedIdeograph9764', [], 'Display', None],
  [ 0x9765, 'CjkUnifiedIdeograph9765', [], 'Display', None],
  [ 0x9766, 'CjkUnifiedIdeograph9766', [], 'Display', None],
  [ 0x9767, 'CjkUnifiedIdeograph9767', [], 'Display', None],
  [ 0x9768, 'CjkUnifiedIdeograph9768', [], 'Display', None],
  [ 0x9769, 'CjkUnifiedIdeograph9769', [], 'Display', None],
  [ 0x976a, 'CjkUnifiedIdeograph976a', [], 'Display', None],
  [ 0x976b, 'CjkUnifiedIdeograph976b', [], 'Display', None],
  [ 0x976c, 'CjkUnifiedIdeograph976c', [], 'Display', None],
  [ 0x976d, 'CjkUnifiedIdeograph976d', [], 'Display', None],
  [ 0x976e, 'CjkUnifiedIdeograph976e', [], 'Display', None],
  [ 0x976f, 'CjkUnifiedIdeograph976f', [], 'Display', None],
  [ 0x9770, 'CjkUnifiedIdeograph9770', [], 'Display', None],
  [ 0x9771, 'CjkUnifiedIdeograph9771', [], 'Display', None],
  [ 0x9772, 'CjkUnifiedIdeograph9772', [], 'Display', None],
  [ 0x9773, 'CjkUnifiedIdeograph9773', [], 'Display', None],
  [ 0x9774, 'CjkUnifiedIdeograph9774', [], 'Display', None],
  [ 0x9775, 'CjkUnifiedIdeograph9775', [], 'Display', None],
  [ 0x9776, 'CjkUnifiedIdeograph9776', [], 'Display', None],
  [ 0x9777, 'CjkUnifiedIdeograph9777', [], 'Display', None],
  [ 0x9778, 'CjkUnifiedIdeograph9778', [], 'Display', None],
  [ 0x9779, 'CjkUnifiedIdeograph9779', [], 'Display', None],
  [ 0x977a, 'CjkUnifiedIdeograph977a', [], 'Display', None],
  [ 0x977b, 'CjkUnifiedIdeograph977b', [], 'Display', None],
  [ 0x977c, 'CjkUnifiedIdeograph977c', [], 'Display', None],
  [ 0x977d, 'CjkUnifiedIdeograph977d', [], 'Display', None],
  [ 0x977e, 'CjkUnifiedIdeograph977e', [], 'Display', None],
  [ 0x977f, 'CjkUnifiedIdeograph977f', [], 'Display', None],
  [ 0x9780, 'CjkUnifiedIdeograph9780', [], 'Display', None],
  [ 0x9781, 'CjkUnifiedIdeograph9781', [], 'Display', None],
  [ 0x9782, 'CjkUnifiedIdeograph9782', [], 'Display', None],
  [ 0x9783, 'CjkUnifiedIdeograph9783', [], 'Display', None],
  [ 0x9784, 'CjkUnifiedIdeograph9784', [], 'Display', None],
  [ 0x9785, 'CjkUnifiedIdeograph9785', [], 'Display', None],
  [ 0x9786, 'CjkUnifiedIdeograph9786', [], 'Display', None],
  [ 0x9787, 'CjkUnifiedIdeograph9787', [], 'Display', None],
  [ 0x9788, 'CjkUnifiedIdeograph9788', [], 'Display', None],
  [ 0x9789, 'CjkUnifiedIdeograph9789', [], 'Display', None],
  [ 0x978a, 'CjkUnifiedIdeograph978a', [], 'Display', None],
  [ 0x978b, 'CjkUnifiedIdeograph978b', [], 'Display', None],
  [ 0x978c, 'CjkUnifiedIdeograph978c', [], 'Display', None],
  [ 0x978d, 'CjkUnifiedIdeograph978d', [], 'Display', None],
  [ 0x978e, 'CjkUnifiedIdeograph978e', [], 'Display', None],
  [ 0x978f, 'CjkUnifiedIdeograph978f', [], 'Display', None],
  [ 0x9790, 'CjkUnifiedIdeograph9790', [], 'Display', None],
  [ 0x9791, 'CjkUnifiedIdeograph9791', [], 'Display', None],
  [ 0x9792, 'CjkUnifiedIdeograph9792', [], 'Display', None],
  [ 0x9793, 'CjkUnifiedIdeograph9793', [], 'Display', None],
  [ 0x9794, 'CjkUnifiedIdeograph9794', [], 'Display', None],
  [ 0x9795, 'CjkUnifiedIdeograph9795', [], 'Display', None],
  [ 0x9796, 'CjkUnifiedIdeograph9796', [], 'Display', None],
  [ 0x9797, 'CjkUnifiedIdeograph9797', [], 'Display', None],
  [ 0x9798, 'CjkUnifiedIdeograph9798', [], 'Display', None],
  [ 0x9799, 'CjkUnifiedIdeograph9799', [], 'Display', None],
  [ 0x979a, 'CjkUnifiedIdeograph979a', [], 'Display', None],
  [ 0x979b, 'CjkUnifiedIdeograph979b', [], 'Display', None],
  [ 0x979c, 'CjkUnifiedIdeograph979c', [], 'Display', None],
  [ 0x979d, 'CjkUnifiedIdeograph979d', [], 'Display', None],
  [ 0x979e, 'CjkUnifiedIdeograph979e', [], 'Display', None],
  [ 0x979f, 'CjkUnifiedIdeograph979f', [], 'Display', None],
  [ 0x97a0, 'CjkUnifiedIdeograph97a0', [], 'Display', None],
  [ 0x97a1, 'CjkUnifiedIdeograph97a1', [], 'Display', None],
  [ 0x97a2, 'CjkUnifiedIdeograph97a2', [], 'Display', None],
  [ 0x97a3, 'CjkUnifiedIdeograph97a3', [], 'Display', None],
  [ 0x97a4, 'CjkUnifiedIdeograph97a4', [], 'Display', None],
  [ 0x97a5, 'CjkUnifiedIdeograph97a5', [], 'Display', None],
  [ 0x97a6, 'CjkUnifiedIdeograph97a6', [], 'Display', None],
  [ 0x97a7, 'CjkUnifiedIdeograph97a7', [], 'Display', None],
  [ 0x97a8, 'CjkUnifiedIdeograph97a8', [], 'Display', None],
  [ 0x97a9, 'CjkUnifiedIdeograph97a9', [], 'Display', None],
  [ 0x97aa, 'CjkUnifiedIdeograph97aa', [], 'Display', None],
  [ 0x97ab, 'CjkUnifiedIdeograph97ab', [], 'Display', None],
  [ 0x97ac, 'CjkUnifiedIdeograph97ac', [], 'Display', None],
  [ 0x97ad, 'CjkUnifiedIdeograph97ad', [], 'Display', None],
  [ 0x97ae, 'CjkUnifiedIdeograph97ae', [], 'Display', None],
  [ 0x97af, 'CjkUnifiedIdeograph97af', [], 'Display', None],
  [ 0x97b0, 'CjkUnifiedIdeograph97b0', [], 'Display', None],
  [ 0x97b1, 'CjkUnifiedIdeograph97b1', [], 'Display', None],
  [ 0x97b2, 'CjkUnifiedIdeograph97b2', [], 'Display', None],
  [ 0x97b3, 'CjkUnifiedIdeograph97b3', [], 'Display', None],
  [ 0x97b4, 'CjkUnifiedIdeograph97b4', [], 'Display', None],
  [ 0x97b5, 'CjkUnifiedIdeograph97b5', [], 'Display', None],
  [ 0x97b6, 'CjkUnifiedIdeograph97b6', [], 'Display', None],
  [ 0x97b7, 'CjkUnifiedIdeograph97b7', [], 'Display', None],
  [ 0x97b8, 'CjkUnifiedIdeograph97b8', [], 'Display', None],
  [ 0x97b9, 'CjkUnifiedIdeograph97b9', [], 'Display', None],
  [ 0x97ba, 'CjkUnifiedIdeograph97ba', [], 'Display', None],
  [ 0x97bb, 'CjkUnifiedIdeograph97bb', [], 'Display', None],
  [ 0x97bc, 'CjkUnifiedIdeograph97bc', [], 'Display', None],
  [ 0x97bd, 'CjkUnifiedIdeograph97bd', [], 'Display', None],
  [ 0x97be, 'CjkUnifiedIdeograph97be', [], 'Display', None],
  [ 0x97bf, 'CjkUnifiedIdeograph97bf', [], 'Display', None],
  [ 0x97c0, 'CjkUnifiedIdeograph97c0', [], 'Display', None],
  [ 0x97c1, 'CjkUnifiedIdeograph97c1', [], 'Display', None],
  [ 0x97c2, 'CjkUnifiedIdeograph97c2', [], 'Display', None],
  [ 0x97c3, 'CjkUnifiedIdeograph97c3', [], 'Display', None],
  [ 0x97c4, 'CjkUnifiedIdeograph97c4', [], 'Display', None],
  [ 0x97c5, 'CjkUnifiedIdeograph97c5', [], 'Display', None],
  [ 0x97c6, 'CjkUnifiedIdeograph97c6', [], 'Display', None],
  [ 0x97c7, 'CjkUnifiedIdeograph97c7', [], 'Display', None],
  [ 0x97c8, 'CjkUnifiedIdeograph97c8', [], 'Display', None],
  [ 0x97c9, 'CjkUnifiedIdeograph97c9', [], 'Display', None],
  [ 0x97ca, 'CjkUnifiedIdeograph97ca', [], 'Display', None],
  [ 0x97cb, 'CjkUnifiedIdeograph97cb', [], 'Display', None],
  [ 0x97cc, 'CjkUnifiedIdeograph97cc', [], 'Display', None],
  [ 0x97cd, 'CjkUnifiedIdeograph97cd', [], 'Display', None],
  [ 0x97ce, 'CjkUnifiedIdeograph97ce', [], 'Display', None],
  [ 0x97cf, 'CjkUnifiedIdeograph97cf', [], 'Display', None],
  [ 0x97d0, 'CjkUnifiedIdeograph97d0', [], 'Display', None],
  [ 0x97d1, 'CjkUnifiedIdeograph97d1', [], 'Display', None],
  [ 0x97d2, 'CjkUnifiedIdeograph97d2', [], 'Display', None],
  [ 0x97d3, 'CjkUnifiedIdeograph97d3', [], 'Display', None],
  [ 0x97d4, 'CjkUnifiedIdeograph97d4', [], 'Display', None],
  [ 0x97d5, 'CjkUnifiedIdeograph97d5', [], 'Display', None],
  [ 0x97d6, 'CjkUnifiedIdeograph97d6', [], 'Display', None],
  [ 0x97d7, 'CjkUnifiedIdeograph97d7', [], 'Display', None],
  [ 0x97d8, 'CjkUnifiedIdeograph97d8', [], 'Display', None],
  [ 0x97d9, 'CjkUnifiedIdeograph97d9', [], 'Display', None],
  [ 0x97da, 'CjkUnifiedIdeograph97da', [], 'Display', None],
  [ 0x97db, 'CjkUnifiedIdeograph97db', [], 'Display', None],
  [ 0x97dc, 'CjkUnifiedIdeograph97dc', [], 'Display', None],
  [ 0x97dd, 'CjkUnifiedIdeograph97dd', [], 'Display', None],
  [ 0x97de, 'CjkUnifiedIdeograph97de', [], 'Display', None],
  [ 0x97df, 'CjkUnifiedIdeograph97df', [], 'Display', None],
  [ 0x97e0, 'CjkUnifiedIdeograph97e0', [], 'Display', None],
  [ 0x97e1, 'CjkUnifiedIdeograph97e1', [], 'Display', None],
  [ 0x97e2, 'CjkUnifiedIdeograph97e2', [], 'Display', None],
  [ 0x97e3, 'CjkUnifiedIdeograph97e3', [], 'Display', None],
  [ 0x97e4, 'CjkUnifiedIdeograph97e4', [], 'Display', None],
  [ 0x97e5, 'CjkUnifiedIdeograph97e5', [], 'Display', None],
  [ 0x97e6, 'CjkUnifiedIdeograph97e6', [], 'Display', None],
  [ 0x97e7, 'CjkUnifiedIdeograph97e7', [], 'Display', None],
  [ 0x97e8, 'CjkUnifiedIdeograph97e8', [], 'Display', None],
  [ 0x97e9, 'CjkUnifiedIdeograph97e9', [], 'Display', None],
  [ 0x97ea, 'CjkUnifiedIdeograph97ea', [], 'Display', None],
  [ 0x97eb, 'CjkUnifiedIdeograph97eb', [], 'Display', None],
  [ 0x97ec, 'CjkUnifiedIdeograph97ec', [], 'Display', None],
  [ 0x97ed, 'CjkUnifiedIdeograph97ed', [], 'Display', None],
  [ 0x97ee, 'CjkUnifiedIdeograph97ee', [], 'Display', None],
  [ 0x97ef, 'CjkUnifiedIdeograph97ef', [], 'Display', None],
  [ 0x97f0, 'CjkUnifiedIdeograph97f0', [], 'Display', None],
  [ 0x97f1, 'CjkUnifiedIdeograph97f1', [], 'Display', None],
  [ 0x97f2, 'CjkUnifiedIdeograph97f2', [], 'Display', None],
  [ 0x97f3, 'CjkUnifiedIdeograph97f3', [], 'Display', None],
  [ 0x97f4, 'CjkUnifiedIdeograph97f4', [], 'Display', None],
  [ 0x97f5, 'CjkUnifiedIdeograph97f5', [], 'Display', None],
  [ 0x97f6, 'CjkUnifiedIdeograph97f6', [], 'Display', None],
  [ 0x97f7, 'CjkUnifiedIdeograph97f7', [], 'Display', None],
  [ 0x97f8, 'CjkUnifiedIdeograph97f8', [], 'Display', None],
  [ 0x97f9, 'CjkUnifiedIdeograph97f9', [], 'Display', None],
  [ 0x97fa, 'CjkUnifiedIdeograph97fa', [], 'Display', None],
  [ 0x97fb, 'CjkUnifiedIdeograph97fb', [], 'Display', None],
  [ 0x97fc, 'CjkUnifiedIdeograph97fc', [], 'Display', None],
  [ 0x97fd, 'CjkUnifiedIdeograph97fd', [], 'Display', None],
  [ 0x97fe, 'CjkUnifiedIdeograph97fe', [], 'Display', None],
  [ 0x97ff, 'CjkUnifiedIdeograph97ff', [], 'Display', None],
  [ 0x9800, 'CjkUnifiedIdeograph9800', [], 'Display', None],
  [ 0x9801, 'CjkUnifiedIdeograph9801', [], 'Display', None],
  [ 0x9802, 'CjkUnifiedIdeograph9802', [], 'Display', None],
  [ 0x9803, 'CjkUnifiedIdeograph9803', [], 'Display', None],
  [ 0x9804, 'CjkUnifiedIdeograph9804', [], 'Display', None],
  [ 0x9805, 'CjkUnifiedIdeograph9805', [], 'Display', None],
  [ 0x9806, 'CjkUnifiedIdeograph9806', [], 'Display', None],
  [ 0x9807, 'CjkUnifiedIdeograph9807', [], 'Display', None],
  [ 0x9808, 'CjkUnifiedIdeograph9808', [], 'Display', None],
  [ 0x9809, 'CjkUnifiedIdeograph9809', [], 'Display', None],
  [ 0x980a, 'CjkUnifiedIdeograph980a', [], 'Display', None],
  [ 0x980b, 'CjkUnifiedIdeograph980b', [], 'Display', None],
  [ 0x980c, 'CjkUnifiedIdeograph980c', [], 'Display', None],
  [ 0x980d, 'CjkUnifiedIdeograph980d', [], 'Display', None],
  [ 0x980e, 'CjkUnifiedIdeograph980e', [], 'Display', None],
  [ 0x980f, 'CjkUnifiedIdeograph980f', [], 'Display', None],
  [ 0x9810, 'CjkUnifiedIdeograph9810', [], 'Display', None],
  [ 0x9811, 'CjkUnifiedIdeograph9811', [], 'Display', None],
  [ 0x9812, 'CjkUnifiedIdeograph9812', [], 'Display', None],
  [ 0x9813, 'CjkUnifiedIdeograph9813', [], 'Display', None],
  [ 0x9814, 'CjkUnifiedIdeograph9814', [], 'Display', None],
  [ 0x9815, 'CjkUnifiedIdeograph9815', [], 'Display', None],
  [ 0x9816, 'CjkUnifiedIdeograph9816', [], 'Display', None],
  [ 0x9817, 'CjkUnifiedIdeograph9817', [], 'Display', None],
  [ 0x9818, 'CjkUnifiedIdeograph9818', [], 'Display', None],
  [ 0x9819, 'CjkUnifiedIdeograph9819', [], 'Display', None],
  [ 0x981a, 'CjkUnifiedIdeograph981a', [], 'Display', None],
  [ 0x981b, 'CjkUnifiedIdeograph981b', [], 'Display', None],
  [ 0x981c, 'CjkUnifiedIdeograph981c', [], 'Display', None],
  [ 0x981d, 'CjkUnifiedIdeograph981d', [], 'Display', None],
  [ 0x981e, 'CjkUnifiedIdeograph981e', [], 'Display', None],
  [ 0x981f, 'CjkUnifiedIdeograph981f', [], 'Display', None],
  [ 0x9820, 'CjkUnifiedIdeograph9820', [], 'Display', None],
  [ 0x9821, 'CjkUnifiedIdeograph9821', [], 'Display', None],
  [ 0x9822, 'CjkUnifiedIdeograph9822', [], 'Display', None],
  [ 0x9823, 'CjkUnifiedIdeograph9823', [], 'Display', None],
  [ 0x9824, 'CjkUnifiedIdeograph9824', [], 'Display', None],
  [ 0x9825, 'CjkUnifiedIdeograph9825', [], 'Display', None],
  [ 0x9826, 'CjkUnifiedIdeograph9826', [], 'Display', None],
  [ 0x9827, 'CjkUnifiedIdeograph9827', [], 'Display', None],
  [ 0x9828, 'CjkUnifiedIdeograph9828', [], 'Display', None],
  [ 0x9829, 'CjkUnifiedIdeograph9829', [], 'Display', None],
  [ 0x982a, 'CjkUnifiedIdeograph982a', [], 'Display', None],
  [ 0x982b, 'CjkUnifiedIdeograph982b', [], 'Display', None],
  [ 0x982c, 'CjkUnifiedIdeograph982c', [], 'Display', None],
  [ 0x982d, 'CjkUnifiedIdeograph982d', [], 'Display', None],
  [ 0x982e, 'CjkUnifiedIdeograph982e', [], 'Display', None],
  [ 0x982f, 'CjkUnifiedIdeograph982f', [], 'Display', None],
  [ 0x9830, 'CjkUnifiedIdeograph9830', [], 'Display', None],
  [ 0x9831, 'CjkUnifiedIdeograph9831', [], 'Display', None],
  [ 0x9832, 'CjkUnifiedIdeograph9832', [], 'Display', None],
  [ 0x9833, 'CjkUnifiedIdeograph9833', [], 'Display', None],
  [ 0x9834, 'CjkUnifiedIdeograph9834', [], 'Display', None],
  [ 0x9835, 'CjkUnifiedIdeograph9835', [], 'Display', None],
  [ 0x9836, 'CjkUnifiedIdeograph9836', [], 'Display', None],
  [ 0x9837, 'CjkUnifiedIdeograph9837', [], 'Display', None],
  [ 0x9838, 'CjkUnifiedIdeograph9838', [], 'Display', None],
  [ 0x9839, 'CjkUnifiedIdeograph9839', [], 'Display', None],
  [ 0x983a, 'CjkUnifiedIdeograph983a', [], 'Display', None],
  [ 0x983b, 'CjkUnifiedIdeograph983b', [], 'Display', None],
  [ 0x983c, 'CjkUnifiedIdeograph983c', [], 'Display', None],
  [ 0x983d, 'CjkUnifiedIdeograph983d', [], 'Display', None],
  [ 0x983e, 'CjkUnifiedIdeograph983e', [], 'Display', None],
  [ 0x983f, 'CjkUnifiedIdeograph983f', [], 'Display', None],
  [ 0x9840, 'CjkUnifiedIdeograph9840', [], 'Display', None],
  [ 0x9841, 'CjkUnifiedIdeograph9841', [], 'Display', None],
  [ 0x9842, 'CjkUnifiedIdeograph9842', [], 'Display', None],
  [ 0x9843, 'CjkUnifiedIdeograph9843', [], 'Display', None],
  [ 0x9844, 'CjkUnifiedIdeograph9844', [], 'Display', None],
  [ 0x9845, 'CjkUnifiedIdeograph9845', [], 'Display', None],
  [ 0x9846, 'CjkUnifiedIdeograph9846', [], 'Display', None],
  [ 0x9847, 'CjkUnifiedIdeograph9847', [], 'Display', None],
  [ 0x9848, 'CjkUnifiedIdeograph9848', [], 'Display', None],
  [ 0x9849, 'CjkUnifiedIdeograph9849', [], 'Display', None],
  [ 0x984a, 'CjkUnifiedIdeograph984a', [], 'Display', None],
  [ 0x984b, 'CjkUnifiedIdeograph984b', [], 'Display', None],
  [ 0x984c, 'CjkUnifiedIdeograph984c', [], 'Display', None],
  [ 0x984d, 'CjkUnifiedIdeograph984d', [], 'Display', None],
  [ 0x984e, 'CjkUnifiedIdeograph984e', [], 'Display', None],
  [ 0x984f, 'CjkUnifiedIdeograph984f', [], 'Display', None],
  [ 0x9850, 'CjkUnifiedIdeograph9850', [], 'Display', None],
  [ 0x9851, 'CjkUnifiedIdeograph9851', [], 'Display', None],
  [ 0x9852, 'CjkUnifiedIdeograph9852', [], 'Display', None],
  [ 0x9853, 'CjkUnifiedIdeograph9853', [], 'Display', None],
  [ 0x9854, 'CjkUnifiedIdeograph9854', [], 'Display', None],
  [ 0x9855, 'CjkUnifiedIdeograph9855', [], 'Display', None],
  [ 0x9856, 'CjkUnifiedIdeograph9856', [], 'Display', None],
  [ 0x9857, 'CjkUnifiedIdeograph9857', [], 'Display', None],
  [ 0x9858, 'CjkUnifiedIdeograph9858', [], 'Display', None],
  [ 0x9859, 'CjkUnifiedIdeograph9859', [], 'Display', None],
  [ 0x985a, 'CjkUnifiedIdeograph985a', [], 'Display', None],
  [ 0x985b, 'CjkUnifiedIdeograph985b', [], 'Display', None],
  [ 0x985c, 'CjkUnifiedIdeograph985c', [], 'Display', None],
  [ 0x985d, 'CjkUnifiedIdeograph985d', [], 'Display', None],
  [ 0x985e, 'CjkUnifiedIdeograph985e', [], 'Display', None],
  [ 0x985f, 'CjkUnifiedIdeograph985f', [], 'Display', None],
  [ 0x9860, 'CjkUnifiedIdeograph9860', [], 'Display', None],
  [ 0x9861, 'CjkUnifiedIdeograph9861', [], 'Display', None],
  [ 0x9862, 'CjkUnifiedIdeograph9862', [], 'Display', None],
  [ 0x9863, 'CjkUnifiedIdeograph9863', [], 'Display', None],
  [ 0x9864, 'CjkUnifiedIdeograph9864', [], 'Display', None],
  [ 0x9865, 'CjkUnifiedIdeograph9865', [], 'Display', None],
  [ 0x9866, 'CjkUnifiedIdeograph9866', [], 'Display', None],
  [ 0x9867, 'CjkUnifiedIdeograph9867', [], 'Display', None],
  [ 0x9868, 'CjkUnifiedIdeograph9868', [], 'Display', None],
  [ 0x9869, 'CjkUnifiedIdeograph9869', [], 'Display', None],
  [ 0x986a, 'CjkUnifiedIdeograph986a', [], 'Display', None],
  [ 0x986b, 'CjkUnifiedIdeograph986b', [], 'Display', None],
  [ 0x986c, 'CjkUnifiedIdeograph986c', [], 'Display', None],
  [ 0x986d, 'CjkUnifiedIdeograph986d', [], 'Display', None],
  [ 0x986e, 'CjkUnifiedIdeograph986e', [], 'Display', None],
  [ 0x986f, 'CjkUnifiedIdeograph986f', [], 'Display', None],
  [ 0x9870, 'CjkUnifiedIdeograph9870', [], 'Display', None],
  [ 0x9871, 'CjkUnifiedIdeograph9871', [], 'Display', None],
  [ 0x9872, 'CjkUnifiedIdeograph9872', [], 'Display', None],
  [ 0x9873, 'CjkUnifiedIdeograph9873', [], 'Display', None],
  [ 0x9874, 'CjkUnifiedIdeograph9874', [], 'Display', None],
  [ 0x9875, 'CjkUnifiedIdeograph9875', [], 'Display', None],
  [ 0x9876, 'CjkUnifiedIdeograph9876', [], 'Display', None],
  [ 0x9877, 'CjkUnifiedIdeograph9877', [], 'Display', None],
  [ 0x9878, 'CjkUnifiedIdeograph9878', [], 'Display', None],
  [ 0x9879, 'CjkUnifiedIdeograph9879', [], 'Display', None],
  [ 0x987a, 'CjkUnifiedIdeograph987a', [], 'Display', None],
  [ 0x987b, 'CjkUnifiedIdeograph987b', [], 'Display', None],
  [ 0x987c, 'CjkUnifiedIdeograph987c', [], 'Display', None],
  [ 0x987d, 'CjkUnifiedIdeograph987d', [], 'Display', None],
  [ 0x987e, 'CjkUnifiedIdeograph987e', [], 'Display', None],
  [ 0x987f, 'CjkUnifiedIdeograph987f', [], 'Display', None],
  [ 0x9880, 'CjkUnifiedIdeograph9880', [], 'Display', None],
  [ 0x9881, 'CjkUnifiedIdeograph9881', [], 'Display', None],
  [ 0x9882, 'CjkUnifiedIdeograph9882', [], 'Display', None],
  [ 0x9883, 'CjkUnifiedIdeograph9883', [], 'Display', None],
  [ 0x9884, 'CjkUnifiedIdeograph9884', [], 'Display', None],
  [ 0x9885, 'CjkUnifiedIdeograph9885', [], 'Display', None],
  [ 0x9886, 'CjkUnifiedIdeograph9886', [], 'Display', None],
  [ 0x9887, 'CjkUnifiedIdeograph9887', [], 'Display', None],
  [ 0x9888, 'CjkUnifiedIdeograph9888', [], 'Display', None],
  [ 0x9889, 'CjkUnifiedIdeograph9889', [], 'Display', None],
  [ 0x988a, 'CjkUnifiedIdeograph988a', [], 'Display', None],
  [ 0x988b, 'CjkUnifiedIdeograph988b', [], 'Display', None],
  [ 0x988c, 'CjkUnifiedIdeograph988c', [], 'Display', None],
  [ 0x988d, 'CjkUnifiedIdeograph988d', [], 'Display', None],
  [ 0x988e, 'CjkUnifiedIdeograph988e', [], 'Display', None],
  [ 0x988f, 'CjkUnifiedIdeograph988f', [], 'Display', None],
  [ 0x9890, 'CjkUnifiedIdeograph9890', [], 'Display', None],
  [ 0x9891, 'CjkUnifiedIdeograph9891', [], 'Display', None],
  [ 0x9892, 'CjkUnifiedIdeograph9892', [], 'Display', None],
  [ 0x9893, 'CjkUnifiedIdeograph9893', [], 'Display', None],
  [ 0x9894, 'CjkUnifiedIdeograph9894', [], 'Display', None],
  [ 0x9895, 'CjkUnifiedIdeograph9895', [], 'Display', None],
  [ 0x9896, 'CjkUnifiedIdeograph9896', [], 'Display', None],
  [ 0x9897, 'CjkUnifiedIdeograph9897', [], 'Display', None],
  [ 0x9898, 'CjkUnifiedIdeograph9898', [], 'Display', None],
  [ 0x9899, 'CjkUnifiedIdeograph9899', [], 'Display', None],
  [ 0x989a, 'CjkUnifiedIdeograph989a', [], 'Display', None],
  [ 0x989b, 'CjkUnifiedIdeograph989b', [], 'Display', None],
  [ 0x989c, 'CjkUnifiedIdeograph989c', [], 'Display', None],
  [ 0x989d, 'CjkUnifiedIdeograph989d', [], 'Display', None],
  [ 0x989e, 'CjkUnifiedIdeograph989e', [], 'Display', None],
  [ 0x989f, 'CjkUnifiedIdeograph989f', [], 'Display', None],
  [ 0x98a0, 'CjkUnifiedIdeograph98a0', [], 'Display', None],
  [ 0x98a1, 'CjkUnifiedIdeograph98a1', [], 'Display', None],
  [ 0x98a2, 'CjkUnifiedIdeograph98a2', [], 'Display', None],
  [ 0x98a3, 'CjkUnifiedIdeograph98a3', [], 'Display', None],
  [ 0x98a4, 'CjkUnifiedIdeograph98a4', [], 'Display', None],
  [ 0x98a5, 'CjkUnifiedIdeograph98a5', [], 'Display', None],
  [ 0x98a6, 'CjkUnifiedIdeograph98a6', [], 'Display', None],
  [ 0x98a7, 'CjkUnifiedIdeograph98a7', [], 'Display', None],
  [ 0x98a8, 'CjkUnifiedIdeograph98a8', [], 'Display', None],
  [ 0x98a9, 'CjkUnifiedIdeograph98a9', [], 'Display', None],
  [ 0x98aa, 'CjkUnifiedIdeograph98aa', [], 'Display', None],
  [ 0x98ab, 'CjkUnifiedIdeograph98ab', [], 'Display', None],
  [ 0x98ac, 'CjkUnifiedIdeograph98ac', [], 'Display', None],
  [ 0x98ad, 'CjkUnifiedIdeograph98ad', [], 'Display', None],
  [ 0x98ae, 'CjkUnifiedIdeograph98ae', [], 'Display', None],
  [ 0x98af, 'CjkUnifiedIdeograph98af', [], 'Display', None],
  [ 0x98b0, 'CjkUnifiedIdeograph98b0', [], 'Display', None],
  [ 0x98b1, 'CjkUnifiedIdeograph98b1', [], 'Display', None],
  [ 0x98b2, 'CjkUnifiedIdeograph98b2', [], 'Display', None],
  [ 0x98b3, 'CjkUnifiedIdeograph98b3', [], 'Display', None],
  [ 0x98b4, 'CjkUnifiedIdeograph98b4', [], 'Display', None],
  [ 0x98b5, 'CjkUnifiedIdeograph98b5', [], 'Display', None],
  [ 0x98b6, 'CjkUnifiedIdeograph98b6', [], 'Display', None],
  [ 0x98b7, 'CjkUnifiedIdeograph98b7', [], 'Display', None],
  [ 0x98b8, 'CjkUnifiedIdeograph98b8', [], 'Display', None],
  [ 0x98b9, 'CjkUnifiedIdeograph98b9', [], 'Display', None],
  [ 0x98ba, 'CjkUnifiedIdeograph98ba', [], 'Display', None],
  [ 0x98bb, 'CjkUnifiedIdeograph98bb', [], 'Display', None],
  [ 0x98bc, 'CjkUnifiedIdeograph98bc', [], 'Display', None],
  [ 0x98bd, 'CjkUnifiedIdeograph98bd', [], 'Display', None],
  [ 0x98be, 'CjkUnifiedIdeograph98be', [], 'Display', None],
  [ 0x98bf, 'CjkUnifiedIdeograph98bf', [], 'Display', None],
  [ 0x98c0, 'CjkUnifiedIdeograph98c0', [], 'Display', None],
  [ 0x98c1, 'CjkUnifiedIdeograph98c1', [], 'Display', None],
  [ 0x98c2, 'CjkUnifiedIdeograph98c2', [], 'Display', None],
  [ 0x98c3, 'CjkUnifiedIdeograph98c3', [], 'Display', None],
  [ 0x98c4, 'CjkUnifiedIdeograph98c4', [], 'Display', None],
  [ 0x98c5, 'CjkUnifiedIdeograph98c5', [], 'Display', None],
  [ 0x98c6, 'CjkUnifiedIdeograph98c6', [], 'Display', None],
  [ 0x98c7, 'CjkUnifiedIdeograph98c7', [], 'Display', None],
  [ 0x98c8, 'CjkUnifiedIdeograph98c8', [], 'Display', None],
  [ 0x98c9, 'CjkUnifiedIdeograph98c9', [], 'Display', None],
  [ 0x98ca, 'CjkUnifiedIdeograph98ca', [], 'Display', None],
  [ 0x98cb, 'CjkUnifiedIdeograph98cb', [], 'Display', None],
  [ 0x98cc, 'CjkUnifiedIdeograph98cc', [], 'Display', None],
  [ 0x98cd, 'CjkUnifiedIdeograph98cd', [], 'Display', None],
  [ 0x98ce, 'CjkUnifiedIdeograph98ce', [], 'Display', None],
  [ 0x98cf, 'CjkUnifiedIdeograph98cf', [], 'Display', None],
  [ 0x98d0, 'CjkUnifiedIdeograph98d0', [], 'Display', None],
  [ 0x98d1, 'CjkUnifiedIdeograph98d1', [], 'Display', None],
  [ 0x98d2, 'CjkUnifiedIdeograph98d2', [], 'Display', None],
  [ 0x98d3, 'CjkUnifiedIdeograph98d3', [], 'Display', None],
  [ 0x98d4, 'CjkUnifiedIdeograph98d4', [], 'Display', None],
  [ 0x98d5, 'CjkUnifiedIdeograph98d5', [], 'Display', None],
  [ 0x98d6, 'CjkUnifiedIdeograph98d6', [], 'Display', None],
  [ 0x98d7, 'CjkUnifiedIdeograph98d7', [], 'Display', None],
  [ 0x98d8, 'CjkUnifiedIdeograph98d8', [], 'Display', None],
  [ 0x98d9, 'CjkUnifiedIdeograph98d9', [], 'Display', None],
  [ 0x98da, 'CjkUnifiedIdeograph98da', [], 'Display', None],
  [ 0x98db, 'CjkUnifiedIdeograph98db', [], 'Display', None],
  [ 0x98dc, 'CjkUnifiedIdeograph98dc', [], 'Display', None],
  [ 0x98dd, 'CjkUnifiedIdeograph98dd', [], 'Display', None],
  [ 0x98de, 'CjkUnifiedIdeograph98de', [], 'Display', None],
  [ 0x98df, 'CjkUnifiedIdeograph98df', [], 'Display', None],
  [ 0x98e0, 'CjkUnifiedIdeograph98e0', [], 'Display', None],
  [ 0x98e1, 'CjkUnifiedIdeograph98e1', [], 'Display', None],
  [ 0x98e2, 'CjkUnifiedIdeograph98e2', [], 'Display', None],
  [ 0x98e3, 'CjkUnifiedIdeograph98e3', [], 'Display', None],
  [ 0x98e4, 'CjkUnifiedIdeograph98e4', [], 'Display', None],
  [ 0x98e5, 'CjkUnifiedIdeograph98e5', [], 'Display', None],
  [ 0x98e6, 'CjkUnifiedIdeograph98e6', [], 'Display', None],
  [ 0x98e7, 'CjkUnifiedIdeograph98e7', [], 'Display', None],
  [ 0x98e8, 'CjkUnifiedIdeograph98e8', [], 'Display', None],
  [ 0x98e9, 'CjkUnifiedIdeograph98e9', [], 'Display', None],
  [ 0x98ea, 'CjkUnifiedIdeograph98ea', [], 'Display', None],
  [ 0x98eb, 'CjkUnifiedIdeograph98eb', [], 'Display', None],
  [ 0x98ec, 'CjkUnifiedIdeograph98ec', [], 'Display', None],
  [ 0x98ed, 'CjkUnifiedIdeograph98ed', [], 'Display', None],
  [ 0x98ee, 'CjkUnifiedIdeograph98ee', [], 'Display', None],
  [ 0x98ef, 'CjkUnifiedIdeograph98ef', [], 'Display', None],
  [ 0x98f0, 'CjkUnifiedIdeograph98f0', [], 'Display', None],
  [ 0x98f1, 'CjkUnifiedIdeograph98f1', [], 'Display', None],
  [ 0x98f2, 'CjkUnifiedIdeograph98f2', [], 'Display', None],
  [ 0x98f3, 'CjkUnifiedIdeograph98f3', [], 'Display', None],
  [ 0x98f4, 'CjkUnifiedIdeograph98f4', [], 'Display', None],
  [ 0x98f5, 'CjkUnifiedIdeograph98f5', [], 'Display', None],
  [ 0x98f6, 'CjkUnifiedIdeograph98f6', [], 'Display', None],
  [ 0x98f7, 'CjkUnifiedIdeograph98f7', [], 'Display', None],
  [ 0x98f8, 'CjkUnifiedIdeograph98f8', [], 'Display', None],
  [ 0x98f9, 'CjkUnifiedIdeograph98f9', [], 'Display', None],
  [ 0x98fa, 'CjkUnifiedIdeograph98fa', [], 'Display', None],
  [ 0x98fb, 'CjkUnifiedIdeograph98fb', [], 'Display', None],
  [ 0x98fc, 'CjkUnifiedIdeograph98fc', [], 'Display', None],
  [ 0x98fd, 'CjkUnifiedIdeograph98fd', [], 'Display', None],
  [ 0x98fe, 'CjkUnifiedIdeograph98fe', [], 'Display', None],
  [ 0x98ff, 'CjkUnifiedIdeograph98ff', [], 'Display', None],
  [ 0x9900, 'CjkUnifiedIdeograph9900', [], 'Display', None],
  [ 0x9901, 'CjkUnifiedIdeograph9901', [], 'Display', None],
  [ 0x9902, 'CjkUnifiedIdeograph9902', [], 'Display', None],
  [ 0x9903, 'CjkUnifiedIdeograph9903', [], 'Display', None],
  [ 0x9904, 'CjkUnifiedIdeograph9904', [], 'Display', None],
  [ 0x9905, 'CjkUnifiedIdeograph9905', [], 'Display', None],
  [ 0x9906, 'CjkUnifiedIdeograph9906', [], 'Display', None],
  [ 0x9907, 'CjkUnifiedIdeograph9907', [], 'Display', None],
  [ 0x9908, 'CjkUnifiedIdeograph9908', [], 'Display', None],
  [ 0x9909, 'CjkUnifiedIdeograph9909', [], 'Display', None],
  [ 0x990a, 'CjkUnifiedIdeograph990a', [], 'Display', None],
  [ 0x990b, 'CjkUnifiedIdeograph990b', [], 'Display', None],
  [ 0x990c, 'CjkUnifiedIdeograph990c', [], 'Display', None],
  [ 0x990d, 'CjkUnifiedIdeograph990d', [], 'Display', None],
  [ 0x990e, 'CjkUnifiedIdeograph990e', [], 'Display', None],
  [ 0x990f, 'CjkUnifiedIdeograph990f', [], 'Display', None],
  [ 0x9910, 'CjkUnifiedIdeograph9910', [], 'Display', None],
  [ 0x9911, 'CjkUnifiedIdeograph9911', [], 'Display', None],
  [ 0x9912, 'CjkUnifiedIdeograph9912', [], 'Display', None],
  [ 0x9913, 'CjkUnifiedIdeograph9913', [], 'Display', None],
  [ 0x9914, 'CjkUnifiedIdeograph9914', [], 'Display', None],
  [ 0x9915, 'CjkUnifiedIdeograph9915', [], 'Display', None],
  [ 0x9916, 'CjkUnifiedIdeograph9916', [], 'Display', None],
  [ 0x9917, 'CjkUnifiedIdeograph9917', [], 'Display', None],
  [ 0x9918, 'CjkUnifiedIdeograph9918', [], 'Display', None],
  [ 0x9919, 'CjkUnifiedIdeograph9919', [], 'Display', None],
  [ 0x991a, 'CjkUnifiedIdeograph991a', [], 'Display', None],
  [ 0x991b, 'CjkUnifiedIdeograph991b', [], 'Display', None],
  [ 0x991c, 'CjkUnifiedIdeograph991c', [], 'Display', None],
  [ 0x991d, 'CjkUnifiedIdeograph991d', [], 'Display', None],
  [ 0x991e, 'CjkUnifiedIdeograph991e', [], 'Display', None],
  [ 0x991f, 'CjkUnifiedIdeograph991f', [], 'Display', None],
  [ 0x9920, 'CjkUnifiedIdeograph9920', [], 'Display', None],
  [ 0x9921, 'CjkUnifiedIdeograph9921', [], 'Display', None],
  [ 0x9922, 'CjkUnifiedIdeograph9922', [], 'Display', None],
  [ 0x9923, 'CjkUnifiedIdeograph9923', [], 'Display', None],
  [ 0x9924, 'CjkUnifiedIdeograph9924', [], 'Display', None],
  [ 0x9925, 'CjkUnifiedIdeograph9925', [], 'Display', None],
  [ 0x9926, 'CjkUnifiedIdeograph9926', [], 'Display', None],
  [ 0x9927, 'CjkUnifiedIdeograph9927', [], 'Display', None],
  [ 0x9928, 'CjkUnifiedIdeograph9928', [], 'Display', None],
  [ 0x9929, 'CjkUnifiedIdeograph9929', [], 'Display', None],
  [ 0x992a, 'CjkUnifiedIdeograph992a', [], 'Display', None],
  [ 0x992b, 'CjkUnifiedIdeograph992b', [], 'Display', None],
  [ 0x992c, 'CjkUnifiedIdeograph992c', [], 'Display', None],
  [ 0x992d, 'CjkUnifiedIdeograph992d', [], 'Display', None],
  [ 0x992e, 'CjkUnifiedIdeograph992e', [], 'Display', None],
  [ 0x992f, 'CjkUnifiedIdeograph992f', [], 'Display', None],
  [ 0x9930, 'CjkUnifiedIdeograph9930', [], 'Display', None],
  [ 0x9931, 'CjkUnifiedIdeograph9931', [], 'Display', None],
  [ 0x9932, 'CjkUnifiedIdeograph9932', [], 'Display', None],
  [ 0x9933, 'CjkUnifiedIdeograph9933', [], 'Display', None],
  [ 0x9934, 'CjkUnifiedIdeograph9934', [], 'Display', None],
  [ 0x9935, 'CjkUnifiedIdeograph9935', [], 'Display', None],
  [ 0x9936, 'CjkUnifiedIdeograph9936', [], 'Display', None],
  [ 0x9937, 'CjkUnifiedIdeograph9937', [], 'Display', None],
  [ 0x9938, 'CjkUnifiedIdeograph9938', [], 'Display', None],
  [ 0x9939, 'CjkUnifiedIdeograph9939', [], 'Display', None],
  [ 0x993a, 'CjkUnifiedIdeograph993a', [], 'Display', None],
  [ 0x993b, 'CjkUnifiedIdeograph993b', [], 'Display', None],
  [ 0x993c, 'CjkUnifiedIdeograph993c', [], 'Display', None],
  [ 0x993d, 'CjkUnifiedIdeograph993d', [], 'Display', None],
  [ 0x993e, 'CjkUnifiedIdeograph993e', [], 'Display', None],
  [ 0x993f, 'CjkUnifiedIdeograph993f', [], 'Display', None],
  [ 0x9940, 'CjkUnifiedIdeograph9940', [], 'Display', None],
  [ 0x9941, 'CjkUnifiedIdeograph9941', [], 'Display', None],
  [ 0x9942, 'CjkUnifiedIdeograph9942', [], 'Display', None],
  [ 0x9943, 'CjkUnifiedIdeograph9943', [], 'Display', None],
  [ 0x9944, 'CjkUnifiedIdeograph9944', [], 'Display', None],
  [ 0x9945, 'CjkUnifiedIdeograph9945', [], 'Display', None],
  [ 0x9946, 'CjkUnifiedIdeograph9946', [], 'Display', None],
  [ 0x9947, 'CjkUnifiedIdeograph9947', [], 'Display', None],
  [ 0x9948, 'CjkUnifiedIdeograph9948', [], 'Display', None],
  [ 0x9949, 'CjkUnifiedIdeograph9949', [], 'Display', None],
  [ 0x994a, 'CjkUnifiedIdeograph994a', [], 'Display', None],
  [ 0x994b, 'CjkUnifiedIdeograph994b', [], 'Display', None],
  [ 0x994c, 'CjkUnifiedIdeograph994c', [], 'Display', None],
  [ 0x994d, 'CjkUnifiedIdeograph994d', [], 'Display', None],
  [ 0x994e, 'CjkUnifiedIdeograph994e', [], 'Display', None],
  [ 0x994f, 'CjkUnifiedIdeograph994f', [], 'Display', None],
  [ 0x9950, 'CjkUnifiedIdeograph9950', [], 'Display', None],
  [ 0x9951, 'CjkUnifiedIdeograph9951', [], 'Display', None],
  [ 0x9952, 'CjkUnifiedIdeograph9952', [], 'Display', None],
  [ 0x9953, 'CjkUnifiedIdeograph9953', [], 'Display', None],
  [ 0x9954, 'CjkUnifiedIdeograph9954', [], 'Display', None],
  [ 0x9955, 'CjkUnifiedIdeograph9955', [], 'Display', None],
  [ 0x9956, 'CjkUnifiedIdeograph9956', [], 'Display', None],
  [ 0x9957, 'CjkUnifiedIdeograph9957', [], 'Display', None],
  [ 0x9958, 'CjkUnifiedIdeograph9958', [], 'Display', None],
  [ 0x9959, 'CjkUnifiedIdeograph9959', [], 'Display', None],
  [ 0x995a, 'CjkUnifiedIdeograph995a', [], 'Display', None],
  [ 0x995b, 'CjkUnifiedIdeograph995b', [], 'Display', None],
  [ 0x995c, 'CjkUnifiedIdeograph995c', [], 'Display', None],
  [ 0x995d, 'CjkUnifiedIdeograph995d', [], 'Display', None],
  [ 0x995e, 'CjkUnifiedIdeograph995e', [], 'Display', None],
  [ 0x995f, 'CjkUnifiedIdeograph995f', [], 'Display', None],
  [ 0x9960, 'CjkUnifiedIdeograph9960', [], 'Display', None],
  [ 0x9961, 'CjkUnifiedIdeograph9961', [], 'Display', None],
  [ 0x9962, 'CjkUnifiedIdeograph9962', [], 'Display', None],
  [ 0x9963, 'CjkUnifiedIdeograph9963', [], 'Display', None],
  [ 0x9964, 'CjkUnifiedIdeograph9964', [], 'Display', None],
  [ 0x9965, 'CjkUnifiedIdeograph9965', [], 'Display', None],
  [ 0x9966, 'CjkUnifiedIdeograph9966', [], 'Display', None],
  [ 0x9967, 'CjkUnifiedIdeograph9967', [], 'Display', None],
  [ 0x9968, 'CjkUnifiedIdeograph9968', [], 'Display', None],
  [ 0x9969, 'CjkUnifiedIdeograph9969', [], 'Display', None],
  [ 0x996a, 'CjkUnifiedIdeograph996a', [], 'Display', None],
  [ 0x996b, 'CjkUnifiedIdeograph996b', [], 'Display', None],
  [ 0x996c, 'CjkUnifiedIdeograph996c', [], 'Display', None],
  [ 0x996d, 'CjkUnifiedIdeograph996d', [], 'Display', None],
  [ 0x996e, 'CjkUnifiedIdeograph996e', [], 'Display', None],
  [ 0x996f, 'CjkUnifiedIdeograph996f', [], 'Display', None],
  [ 0x9970, 'CjkUnifiedIdeograph9970', [], 'Display', None],
  [ 0x9971, 'CjkUnifiedIdeograph9971', [], 'Display', None],
  [ 0x9972, 'CjkUnifiedIdeograph9972', [], 'Display', None],
  [ 0x9973, 'CjkUnifiedIdeograph9973', [], 'Display', None],
  [ 0x9974, 'CjkUnifiedIdeograph9974', [], 'Display', None],
  [ 0x9975, 'CjkUnifiedIdeograph9975', [], 'Display', None],
  [ 0x9976, 'CjkUnifiedIdeograph9976', [], 'Display', None],
  [ 0x9977, 'CjkUnifiedIdeograph9977', [], 'Display', None],
  [ 0x9978, 'CjkUnifiedIdeograph9978', [], 'Display', None],
  [ 0x9979, 'CjkUnifiedIdeograph9979', [], 'Display', None],
  [ 0x997a, 'CjkUnifiedIdeograph997a', [], 'Display', None],
  [ 0x997b, 'CjkUnifiedIdeograph997b', [], 'Display', None],
  [ 0x997c, 'CjkUnifiedIdeograph997c', [], 'Display', None],
  [ 0x997d, 'CjkUnifiedIdeograph997d', [], 'Display', None],
  [ 0x997e, 'CjkUnifiedIdeograph997e', [], 'Display', None],
  [ 0x997f, 'CjkUnifiedIdeograph997f', [], 'Display', None],
  [ 0x9980, 'CjkUnifiedIdeograph9980', [], 'Display', None],
  [ 0x9981, 'CjkUnifiedIdeograph9981', [], 'Display', None],
  [ 0x9982, 'CjkUnifiedIdeograph9982', [], 'Display', None],
  [ 0x9983, 'CjkUnifiedIdeograph9983', [], 'Display', None],
  [ 0x9984, 'CjkUnifiedIdeograph9984', [], 'Display', None],
  [ 0x9985, 'CjkUnifiedIdeograph9985', [], 'Display', None],
  [ 0x9986, 'CjkUnifiedIdeograph9986', [], 'Display', None],
  [ 0x9987, 'CjkUnifiedIdeograph9987', [], 'Display', None],
  [ 0x9988, 'CjkUnifiedIdeograph9988', [], 'Display', None],
  [ 0x9989, 'CjkUnifiedIdeograph9989', [], 'Display', None],
  [ 0x998a, 'CjkUnifiedIdeograph998a', [], 'Display', None],
  [ 0x998b, 'CjkUnifiedIdeograph998b', [], 'Display', None],
  [ 0x998c, 'CjkUnifiedIdeograph998c', [], 'Display', None],
  [ 0x998d, 'CjkUnifiedIdeograph998d', [], 'Display', None],
  [ 0x998e, 'CjkUnifiedIdeograph998e', [], 'Display', None],
  [ 0x998f, 'CjkUnifiedIdeograph998f', [], 'Display', None],
  [ 0x9990, 'CjkUnifiedIdeograph9990', [], 'Display', None],
  [ 0x9991, 'CjkUnifiedIdeograph9991', [], 'Display', None],
  [ 0x9992, 'CjkUnifiedIdeograph9992', [], 'Display', None],
  [ 0x9993, 'CjkUnifiedIdeograph9993', [], 'Display', None],
  [ 0x9994, 'CjkUnifiedIdeograph9994', [], 'Display', None],
  [ 0x9995, 'CjkUnifiedIdeograph9995', [], 'Display', None],
  [ 0x9996, 'CjkUnifiedIdeograph9996', [], 'Display', None],
  [ 0x9997, 'CjkUnifiedIdeograph9997', [], 'Display', None],
  [ 0x9998, 'CjkUnifiedIdeograph9998', [], 'Display', None],
  [ 0x9999, 'CjkUnifiedIdeograph9999', [], 'Display', None],
  [ 0x999a, 'CjkUnifiedIdeograph999a', [], 'Display', None],
  [ 0x999b, 'CjkUnifiedIdeograph999b', [], 'Display', None],
  [ 0x999c, 'CjkUnifiedIdeograph999c', [], 'Display', None],
  [ 0x999d, 'CjkUnifiedIdeograph999d', [], 'Display', None],
  [ 0x999e, 'CjkUnifiedIdeograph999e', [], 'Display', None],
  [ 0x999f, 'CjkUnifiedIdeograph999f', [], 'Display', None],
  [ 0x99a0, 'CjkUnifiedIdeograph99a0', [], 'Display', None],
  [ 0x99a1, 'CjkUnifiedIdeograph99a1', [], 'Display', None],
  [ 0x99a2, 'CjkUnifiedIdeograph99a2', [], 'Display', None],
  [ 0x99a3, 'CjkUnifiedIdeograph99a3', [], 'Display', None],
  [ 0x99a4, 'CjkUnifiedIdeograph99a4', [], 'Display', None],
  [ 0x99a5, 'CjkUnifiedIdeograph99a5', [], 'Display', None],
  [ 0x99a6, 'CjkUnifiedIdeograph99a6', [], 'Display', None],
  [ 0x99a7, 'CjkUnifiedIdeograph99a7', [], 'Display', None],
  [ 0x99a8, 'CjkUnifiedIdeograph99a8', [], 'Display', None],
  [ 0x99a9, 'CjkUnifiedIdeograph99a9', [], 'Display', None],
  [ 0x99aa, 'CjkUnifiedIdeograph99aa', [], 'Display', None],
  [ 0x99ab, 'CjkUnifiedIdeograph99ab', [], 'Display', None],
  [ 0x99ac, 'CjkUnifiedIdeograph99ac', [], 'Display', None],
  [ 0x99ad, 'CjkUnifiedIdeograph99ad', [], 'Display', None],
  [ 0x99ae, 'CjkUnifiedIdeograph99ae', [], 'Display', None],
  [ 0x99af, 'CjkUnifiedIdeograph99af', [], 'Display', None],
  [ 0x99b0, 'CjkUnifiedIdeograph99b0', [], 'Display', None],
  [ 0x99b1, 'CjkUnifiedIdeograph99b1', [], 'Display', None],
  [ 0x99b2, 'CjkUnifiedIdeograph99b2', [], 'Display', None],
  [ 0x99b3, 'CjkUnifiedIdeograph99b3', [], 'Display', None],
  [ 0x99b4, 'CjkUnifiedIdeograph99b4', [], 'Display', None],
  [ 0x99b5, 'CjkUnifiedIdeograph99b5', [], 'Display', None],
  [ 0x99b6, 'CjkUnifiedIdeograph99b6', [], 'Display', None],
  [ 0x99b7, 'CjkUnifiedIdeograph99b7', [], 'Display', None],
  [ 0x99b8, 'CjkUnifiedIdeograph99b8', [], 'Display', None],
  [ 0x99b9, 'CjkUnifiedIdeograph99b9', [], 'Display', None],
  [ 0x99ba, 'CjkUnifiedIdeograph99ba', [], 'Display', None],
  [ 0x99bb, 'CjkUnifiedIdeograph99bb', [], 'Display', None],
  [ 0x99bc, 'CjkUnifiedIdeograph99bc', [], 'Display', None],
  [ 0x99bd, 'CjkUnifiedIdeograph99bd', [], 'Display', None],
  [ 0x99be, 'CjkUnifiedIdeograph99be', [], 'Display', None],
  [ 0x99bf, 'CjkUnifiedIdeograph99bf', [], 'Display', None],
  [ 0x99c0, 'CjkUnifiedIdeograph99c0', [], 'Display', None],
  [ 0x99c1, 'CjkUnifiedIdeograph99c1', [], 'Display', None],
  [ 0x99c2, 'CjkUnifiedIdeograph99c2', [], 'Display', None],
  [ 0x99c3, 'CjkUnifiedIdeograph99c3', [], 'Display', None],
  [ 0x99c4, 'CjkUnifiedIdeograph99c4', [], 'Display', None],
  [ 0x99c5, 'CjkUnifiedIdeograph99c5', [], 'Display', None],
  [ 0x99c6, 'CjkUnifiedIdeograph99c6', [], 'Display', None],
  [ 0x99c7, 'CjkUnifiedIdeograph99c7', [], 'Display', None],
  [ 0x99c8, 'CjkUnifiedIdeograph99c8', [], 'Display', None],
  [ 0x99c9, 'CjkUnifiedIdeograph99c9', [], 'Display', None],
  [ 0x99ca, 'CjkUnifiedIdeograph99ca', [], 'Display', None],
  [ 0x99cb, 'CjkUnifiedIdeograph99cb', [], 'Display', None],
  [ 0x99cc, 'CjkUnifiedIdeograph99cc', [], 'Display', None],
  [ 0x99cd, 'CjkUnifiedIdeograph99cd', [], 'Display', None],
  [ 0x99ce, 'CjkUnifiedIdeograph99ce', [], 'Display', None],
  [ 0x99cf, 'CjkUnifiedIdeograph99cf', [], 'Display', None],
  [ 0x99d0, 'CjkUnifiedIdeograph99d0', [], 'Display', None],
  [ 0x99d1, 'CjkUnifiedIdeograph99d1', [], 'Display', None],
  [ 0x99d2, 'CjkUnifiedIdeograph99d2', [], 'Display', None],
  [ 0x99d3, 'CjkUnifiedIdeograph99d3', [], 'Display', None],
  [ 0x99d4, 'CjkUnifiedIdeograph99d4', [], 'Display', None],
  [ 0x99d5, 'CjkUnifiedIdeograph99d5', [], 'Display', None],
  [ 0x99d6, 'CjkUnifiedIdeograph99d6', [], 'Display', None],
  [ 0x99d7, 'CjkUnifiedIdeograph99d7', [], 'Display', None],
  [ 0x99d8, 'CjkUnifiedIdeograph99d8', [], 'Display', None],
  [ 0x99d9, 'CjkUnifiedIdeograph99d9', [], 'Display', None],
  [ 0x99da, 'CjkUnifiedIdeograph99da', [], 'Display', None],
  [ 0x99db, 'CjkUnifiedIdeograph99db', [], 'Display', None],
  [ 0x99dc, 'CjkUnifiedIdeograph99dc', [], 'Display', None],
  [ 0x99dd, 'CjkUnifiedIdeograph99dd', [], 'Display', None],
  [ 0x99de, 'CjkUnifiedIdeograph99de', [], 'Display', None],
  [ 0x99df, 'CjkUnifiedIdeograph99df', [], 'Display', None],
  [ 0x99e0, 'CjkUnifiedIdeograph99e0', [], 'Display', None],
  [ 0x99e1, 'CjkUnifiedIdeograph99e1', [], 'Display', None],
  [ 0x99e2, 'CjkUnifiedIdeograph99e2', [], 'Display', None],
  [ 0x99e3, 'CjkUnifiedIdeograph99e3', [], 'Display', None],
  [ 0x99e4, 'CjkUnifiedIdeograph99e4', [], 'Display', None],
  [ 0x99e5, 'CjkUnifiedIdeograph99e5', [], 'Display', None],
  [ 0x99e6, 'CjkUnifiedIdeograph99e6', [], 'Display', None],
  [ 0x99e7, 'CjkUnifiedIdeograph99e7', [], 'Display', None],
  [ 0x99e8, 'CjkUnifiedIdeograph99e8', [], 'Display', None],
  [ 0x99e9, 'CjkUnifiedIdeograph99e9', [], 'Display', None],
  [ 0x99ea, 'CjkUnifiedIdeograph99ea', [], 'Display', None],
  [ 0x99eb, 'CjkUnifiedIdeograph99eb', [], 'Display', None],
  [ 0x99ec, 'CjkUnifiedIdeograph99ec', [], 'Display', None],
  [ 0x99ed, 'CjkUnifiedIdeograph99ed', [], 'Display', None],
  [ 0x99ee, 'CjkUnifiedIdeograph99ee', [], 'Display', None],
  [ 0x99ef, 'CjkUnifiedIdeograph99ef', [], 'Display', None],
  [ 0x99f0, 'CjkUnifiedIdeograph99f0', [], 'Display', None],
  [ 0x99f1, 'CjkUnifiedIdeograph99f1', [], 'Display', None],
  [ 0x99f2, 'CjkUnifiedIdeograph99f2', [], 'Display', None],
  [ 0x99f3, 'CjkUnifiedIdeograph99f3', [], 'Display', None],
  [ 0x99f4, 'CjkUnifiedIdeograph99f4', [], 'Display', None],
  [ 0x99f5, 'CjkUnifiedIdeograph99f5', [], 'Display', None],
  [ 0x99f6, 'CjkUnifiedIdeograph99f6', [], 'Display', None],
  [ 0x99f7, 'CjkUnifiedIdeograph99f7', [], 'Display', None],
  [ 0x99f8, 'CjkUnifiedIdeograph99f8', [], 'Display', None],
  [ 0x99f9, 'CjkUnifiedIdeograph99f9', [], 'Display', None],
  [ 0x99fa, 'CjkUnifiedIdeograph99fa', [], 'Display', None],
  [ 0x99fb, 'CjkUnifiedIdeograph99fb', [], 'Display', None],
  [ 0x99fc, 'CjkUnifiedIdeograph99fc', [], 'Display', None],
  [ 0x99fd, 'CjkUnifiedIdeograph99fd', [], 'Display', None],
  [ 0x99fe, 'CjkUnifiedIdeograph99fe', [], 'Display', None],
  [ 0x99ff, 'CjkUnifiedIdeograph99ff', [], 'Display', None],
  [ 0x9a00, 'CjkUnifiedIdeograph9a00', [], 'Display', None],
  [ 0x9a01, 'CjkUnifiedIdeograph9a01', [], 'Display', None],
  [ 0x9a02, 'CjkUnifiedIdeograph9a02', [], 'Display', None],
  [ 0x9a03, 'CjkUnifiedIdeograph9a03', [], 'Display', None],
  [ 0x9a04, 'CjkUnifiedIdeograph9a04', [], 'Display', None],
  [ 0x9a05, 'CjkUnifiedIdeograph9a05', [], 'Display', None],
  [ 0x9a06, 'CjkUnifiedIdeograph9a06', [], 'Display', None],
  [ 0x9a07, 'CjkUnifiedIdeograph9a07', [], 'Display', None],
  [ 0x9a08, 'CjkUnifiedIdeograph9a08', [], 'Display', None],
  [ 0x9a09, 'CjkUnifiedIdeograph9a09', [], 'Display', None],
  [ 0x9a0a, 'CjkUnifiedIdeograph9a0a', [], 'Display', None],
  [ 0x9a0b, 'CjkUnifiedIdeograph9a0b', [], 'Display', None],
  [ 0x9a0c, 'CjkUnifiedIdeograph9a0c', [], 'Display', None],
  [ 0x9a0d, 'CjkUnifiedIdeograph9a0d', [], 'Display', None],
  [ 0x9a0e, 'CjkUnifiedIdeograph9a0e', [], 'Display', None],
  [ 0x9a0f, 'CjkUnifiedIdeograph9a0f', [], 'Display', None],
  [ 0x9a10, 'CjkUnifiedIdeograph9a10', [], 'Display', None],
  [ 0x9a11, 'CjkUnifiedIdeograph9a11', [], 'Display', None],
  [ 0x9a12, 'CjkUnifiedIdeograph9a12', [], 'Display', None],
  [ 0x9a13, 'CjkUnifiedIdeograph9a13', [], 'Display', None],
  [ 0x9a14, 'CjkUnifiedIdeograph9a14', [], 'Display', None],
  [ 0x9a15, 'CjkUnifiedIdeograph9a15', [], 'Display', None],
  [ 0x9a16, 'CjkUnifiedIdeograph9a16', [], 'Display', None],
  [ 0x9a17, 'CjkUnifiedIdeograph9a17', [], 'Display', None],
  [ 0x9a18, 'CjkUnifiedIdeograph9a18', [], 'Display', None],
  [ 0x9a19, 'CjkUnifiedIdeograph9a19', [], 'Display', None],
  [ 0x9a1a, 'CjkUnifiedIdeograph9a1a', [], 'Display', None],
  [ 0x9a1b, 'CjkUnifiedIdeograph9a1b', [], 'Display', None],
  [ 0x9a1c, 'CjkUnifiedIdeograph9a1c', [], 'Display', None],
  [ 0x9a1d, 'CjkUnifiedIdeograph9a1d', [], 'Display', None],
  [ 0x9a1e, 'CjkUnifiedIdeograph9a1e', [], 'Display', None],
  [ 0x9a1f, 'CjkUnifiedIdeograph9a1f', [], 'Display', None],
  [ 0x9a20, 'CjkUnifiedIdeograph9a20', [], 'Display', None],
  [ 0x9a21, 'CjkUnifiedIdeograph9a21', [], 'Display', None],
  [ 0x9a22, 'CjkUnifiedIdeograph9a22', [], 'Display', None],
  [ 0x9a23, 'CjkUnifiedIdeograph9a23', [], 'Display', None],
  [ 0x9a24, 'CjkUnifiedIdeograph9a24', [], 'Display', None],
  [ 0x9a25, 'CjkUnifiedIdeograph9a25', [], 'Display', None],
  [ 0x9a26, 'CjkUnifiedIdeograph9a26', [], 'Display', None],
  [ 0x9a27, 'CjkUnifiedIdeograph9a27', [], 'Display', None],
  [ 0x9a28, 'CjkUnifiedIdeograph9a28', [], 'Display', None],
  [ 0x9a29, 'CjkUnifiedIdeograph9a29', [], 'Display', None],
  [ 0x9a2a, 'CjkUnifiedIdeograph9a2a', [], 'Display', None],
  [ 0x9a2b, 'CjkUnifiedIdeograph9a2b', [], 'Display', None],
  [ 0x9a2c, 'CjkUnifiedIdeograph9a2c', [], 'Display', None],
  [ 0x9a2d, 'CjkUnifiedIdeograph9a2d', [], 'Display', None],
  [ 0x9a2e, 'CjkUnifiedIdeograph9a2e', [], 'Display', None],
  [ 0x9a2f, 'CjkUnifiedIdeograph9a2f', [], 'Display', None],
  [ 0x9a30, 'CjkUnifiedIdeograph9a30', [], 'Display', None],
  [ 0x9a31, 'CjkUnifiedIdeograph9a31', [], 'Display', None],
  [ 0x9a32, 'CjkUnifiedIdeograph9a32', [], 'Display', None],
  [ 0x9a33, 'CjkUnifiedIdeograph9a33', [], 'Display', None],
  [ 0x9a34, 'CjkUnifiedIdeograph9a34', [], 'Display', None],
  [ 0x9a35, 'CjkUnifiedIdeograph9a35', [], 'Display', None],
  [ 0x9a36, 'CjkUnifiedIdeograph9a36', [], 'Display', None],
  [ 0x9a37, 'CjkUnifiedIdeograph9a37', [], 'Display', None],
  [ 0x9a38, 'CjkUnifiedIdeograph9a38', [], 'Display', None],
  [ 0x9a39, 'CjkUnifiedIdeograph9a39', [], 'Display', None],
  [ 0x9a3a, 'CjkUnifiedIdeograph9a3a', [], 'Display', None],
  [ 0x9a3b, 'CjkUnifiedIdeograph9a3b', [], 'Display', None],
  [ 0x9a3c, 'CjkUnifiedIdeograph9a3c', [], 'Display', None],
  [ 0x9a3d, 'CjkUnifiedIdeograph9a3d', [], 'Display', None],
  [ 0x9a3e, 'CjkUnifiedIdeograph9a3e', [], 'Display', None],
  [ 0x9a3f, 'CjkUnifiedIdeograph9a3f', [], 'Display', None],
  [ 0x9a40, 'CjkUnifiedIdeograph9a40', [], 'Display', None],
  [ 0x9a41, 'CjkUnifiedIdeograph9a41', [], 'Display', None],
  [ 0x9a42, 'CjkUnifiedIdeograph9a42', [], 'Display', None],
  [ 0x9a43, 'CjkUnifiedIdeograph9a43', [], 'Display', None],
  [ 0x9a44, 'CjkUnifiedIdeograph9a44', [], 'Display', None],
  [ 0x9a45, 'CjkUnifiedIdeograph9a45', [], 'Display', None],
  [ 0x9a46, 'CjkUnifiedIdeograph9a46', [], 'Display', None],
  [ 0x9a47, 'CjkUnifiedIdeograph9a47', [], 'Display', None],
  [ 0x9a48, 'CjkUnifiedIdeograph9a48', [], 'Display', None],
  [ 0x9a49, 'CjkUnifiedIdeograph9a49', [], 'Display', None],
  [ 0x9a4a, 'CjkUnifiedIdeograph9a4a', [], 'Display', None],
  [ 0x9a4b, 'CjkUnifiedIdeograph9a4b', [], 'Display', None],
  [ 0x9a4c, 'CjkUnifiedIdeograph9a4c', [], 'Display', None],
  [ 0x9a4d, 'CjkUnifiedIdeograph9a4d', [], 'Display', None],
  [ 0x9a4e, 'CjkUnifiedIdeograph9a4e', [], 'Display', None],
  [ 0x9a4f, 'CjkUnifiedIdeograph9a4f', [], 'Display', None],
  [ 0x9a50, 'CjkUnifiedIdeograph9a50', [], 'Display', None],
  [ 0x9a51, 'CjkUnifiedIdeograph9a51', [], 'Display', None],
  [ 0x9a52, 'CjkUnifiedIdeograph9a52', [], 'Display', None],
  [ 0x9a53, 'CjkUnifiedIdeograph9a53', [], 'Display', None],
  [ 0x9a54, 'CjkUnifiedIdeograph9a54', [], 'Display', None],
  [ 0x9a55, 'CjkUnifiedIdeograph9a55', [], 'Display', None],
  [ 0x9a56, 'CjkUnifiedIdeograph9a56', [], 'Display', None],
  [ 0x9a57, 'CjkUnifiedIdeograph9a57', [], 'Display', None],
  [ 0x9a58, 'CjkUnifiedIdeograph9a58', [], 'Display', None],
  [ 0x9a59, 'CjkUnifiedIdeograph9a59', [], 'Display', None],
  [ 0x9a5a, 'CjkUnifiedIdeograph9a5a', [], 'Display', None],
  [ 0x9a5b, 'CjkUnifiedIdeograph9a5b', [], 'Display', None],
  [ 0x9a5c, 'CjkUnifiedIdeograph9a5c', [], 'Display', None],
  [ 0x9a5d, 'CjkUnifiedIdeograph9a5d', [], 'Display', None],
  [ 0x9a5e, 'CjkUnifiedIdeograph9a5e', [], 'Display', None],
  [ 0x9a5f, 'CjkUnifiedIdeograph9a5f', [], 'Display', None],
  [ 0x9a60, 'CjkUnifiedIdeograph9a60', [], 'Display', None],
  [ 0x9a61, 'CjkUnifiedIdeograph9a61', [], 'Display', None],
  [ 0x9a62, 'CjkUnifiedIdeograph9a62', [], 'Display', None],
  [ 0x9a63, 'CjkUnifiedIdeograph9a63', [], 'Display', None],
  [ 0x9a64, 'CjkUnifiedIdeograph9a64', [], 'Display', None],
  [ 0x9a65, 'CjkUnifiedIdeograph9a65', [], 'Display', None],
  [ 0x9a66, 'CjkUnifiedIdeograph9a66', [], 'Display', None],
  [ 0x9a67, 'CjkUnifiedIdeograph9a67', [], 'Display', None],
  [ 0x9a68, 'CjkUnifiedIdeograph9a68', [], 'Display', None],
  [ 0x9a69, 'CjkUnifiedIdeograph9a69', [], 'Display', None],
  [ 0x9a6a, 'CjkUnifiedIdeograph9a6a', [], 'Display', None],
  [ 0x9a6b, 'CjkUnifiedIdeograph9a6b', [], 'Display', None],
  [ 0x9a6c, 'CjkUnifiedIdeograph9a6c', [], 'Display', None],
  [ 0x9a6d, 'CjkUnifiedIdeograph9a6d', [], 'Display', None],
  [ 0x9a6e, 'CjkUnifiedIdeograph9a6e', [], 'Display', None],
  [ 0x9a6f, 'CjkUnifiedIdeograph9a6f', [], 'Display', None],
  [ 0x9a70, 'CjkUnifiedIdeograph9a70', [], 'Display', None],
  [ 0x9a71, 'CjkUnifiedIdeograph9a71', [], 'Display', None],
  [ 0x9a72, 'CjkUnifiedIdeograph9a72', [], 'Display', None],
  [ 0x9a73, 'CjkUnifiedIdeograph9a73', [], 'Display', None],
  [ 0x9a74, 'CjkUnifiedIdeograph9a74', [], 'Display', None],
  [ 0x9a75, 'CjkUnifiedIdeograph9a75', [], 'Display', None],
  [ 0x9a76, 'CjkUnifiedIdeograph9a76', [], 'Display', None],
  [ 0x9a77, 'CjkUnifiedIdeograph9a77', [], 'Display', None],
  [ 0x9a78, 'CjkUnifiedIdeograph9a78', [], 'Display', None],
  [ 0x9a79, 'CjkUnifiedIdeograph9a79', [], 'Display', None],
  [ 0x9a7a, 'CjkUnifiedIdeograph9a7a', [], 'Display', None],
  [ 0x9a7b, 'CjkUnifiedIdeograph9a7b', [], 'Display', None],
  [ 0x9a7c, 'CjkUnifiedIdeograph9a7c', [], 'Display', None],
  [ 0x9a7d, 'CjkUnifiedIdeograph9a7d', [], 'Display', None],
  [ 0x9a7e, 'CjkUnifiedIdeograph9a7e', [], 'Display', None],
  [ 0x9a7f, 'CjkUnifiedIdeograph9a7f', [], 'Display', None],
  [ 0x9a80, 'CjkUnifiedIdeograph9a80', [], 'Display', None],
  [ 0x9a81, 'CjkUnifiedIdeograph9a81', [], 'Display', None],
  [ 0x9a82, 'CjkUnifiedIdeograph9a82', [], 'Display', None],
  [ 0x9a83, 'CjkUnifiedIdeograph9a83', [], 'Display', None],
  [ 0x9a84, 'CjkUnifiedIdeograph9a84', [], 'Display', None],
  [ 0x9a85, 'CjkUnifiedIdeograph9a85', [], 'Display', None],
  [ 0x9a86, 'CjkUnifiedIdeograph9a86', [], 'Display', None],
  [ 0x9a87, 'CjkUnifiedIdeograph9a87', [], 'Display', None],
  [ 0x9a88, 'CjkUnifiedIdeograph9a88', [], 'Display', None],
  [ 0x9a89, 'CjkUnifiedIdeograph9a89', [], 'Display', None],
  [ 0x9a8a, 'CjkUnifiedIdeograph9a8a', [], 'Display', None],
  [ 0x9a8b, 'CjkUnifiedIdeograph9a8b', [], 'Display', None],
  [ 0x9a8c, 'CjkUnifiedIdeograph9a8c', [], 'Display', None],
  [ 0x9a8d, 'CjkUnifiedIdeograph9a8d', [], 'Display', None],
  [ 0x9a8e, 'CjkUnifiedIdeograph9a8e', [], 'Display', None],
  [ 0x9a8f, 'CjkUnifiedIdeograph9a8f', [], 'Display', None],
  [ 0x9a90, 'CjkUnifiedIdeograph9a90', [], 'Display', None],
  [ 0x9a91, 'CjkUnifiedIdeograph9a91', [], 'Display', None],
  [ 0x9a92, 'CjkUnifiedIdeograph9a92', [], 'Display', None],
  [ 0x9a93, 'CjkUnifiedIdeograph9a93', [], 'Display', None],
  [ 0x9a94, 'CjkUnifiedIdeograph9a94', [], 'Display', None],
  [ 0x9a95, 'CjkUnifiedIdeograph9a95', [], 'Display', None],
  [ 0x9a96, 'CjkUnifiedIdeograph9a96', [], 'Display', None],
  [ 0x9a97, 'CjkUnifiedIdeograph9a97', [], 'Display', None],
  [ 0x9a98, 'CjkUnifiedIdeograph9a98', [], 'Display', None],
  [ 0x9a99, 'CjkUnifiedIdeograph9a99', [], 'Display', None],
  [ 0x9a9a, 'CjkUnifiedIdeograph9a9a', [], 'Display', None],
  [ 0x9a9b, 'CjkUnifiedIdeograph9a9b', [], 'Display', None],
  [ 0x9a9c, 'CjkUnifiedIdeograph9a9c', [], 'Display', None],
  [ 0x9a9d, 'CjkUnifiedIdeograph9a9d', [], 'Display', None],
  [ 0x9a9e, 'CjkUnifiedIdeograph9a9e', [], 'Display', None],
  [ 0x9a9f, 'CjkUnifiedIdeograph9a9f', [], 'Display', None],
  [ 0x9aa0, 'CjkUnifiedIdeograph9aa0', [], 'Display', None],
  [ 0x9aa1, 'CjkUnifiedIdeograph9aa1', [], 'Display', None],
  [ 0x9aa2, 'CjkUnifiedIdeograph9aa2', [], 'Display', None],
  [ 0x9aa3, 'CjkUnifiedIdeograph9aa3', [], 'Display', None],
  [ 0x9aa4, 'CjkUnifiedIdeograph9aa4', [], 'Display', None],
  [ 0x9aa5, 'CjkUnifiedIdeograph9aa5', [], 'Display', None],
  [ 0x9aa6, 'CjkUnifiedIdeograph9aa6', [], 'Display', None],
  [ 0x9aa7, 'CjkUnifiedIdeograph9aa7', [], 'Display', None],
  [ 0x9aa8, 'CjkUnifiedIdeograph9aa8', [], 'Display', None],
  [ 0x9aa9, 'CjkUnifiedIdeograph9aa9', [], 'Display', None],
  [ 0x9aaa, 'CjkUnifiedIdeograph9aaa', [], 'Display', None],
  [ 0x9aab, 'CjkUnifiedIdeograph9aab', [], 'Display', None],
  [ 0x9aac, 'CjkUnifiedIdeograph9aac', [], 'Display', None],
  [ 0x9aad, 'CjkUnifiedIdeograph9aad', [], 'Display', None],
  [ 0x9aae, 'CjkUnifiedIdeograph9aae', [], 'Display', None],
  [ 0x9aaf, 'CjkUnifiedIdeograph9aaf', [], 'Display', None],
  [ 0x9ab0, 'CjkUnifiedIdeograph9ab0', [], 'Display', None],
  [ 0x9ab1, 'CjkUnifiedIdeograph9ab1', [], 'Display', None],
  [ 0x9ab2, 'CjkUnifiedIdeograph9ab2', [], 'Display', None],
  [ 0x9ab3, 'CjkUnifiedIdeograph9ab3', [], 'Display', None],
  [ 0x9ab4, 'CjkUnifiedIdeograph9ab4', [], 'Display', None],
  [ 0x9ab5, 'CjkUnifiedIdeograph9ab5', [], 'Display', None],
  [ 0x9ab6, 'CjkUnifiedIdeograph9ab6', [], 'Display', None],
  [ 0x9ab7, 'CjkUnifiedIdeograph9ab7', [], 'Display', None],
  [ 0x9ab8, 'CjkUnifiedIdeograph9ab8', [], 'Display', None],
  [ 0x9ab9, 'CjkUnifiedIdeograph9ab9', [], 'Display', None],
  [ 0x9aba, 'CjkUnifiedIdeograph9aba', [], 'Display', None],
  [ 0x9abb, 'CjkUnifiedIdeograph9abb', [], 'Display', None],
  [ 0x9abc, 'CjkUnifiedIdeograph9abc', [], 'Display', None],
  [ 0x9abd, 'CjkUnifiedIdeograph9abd', [], 'Display', None],
  [ 0x9abe, 'CjkUnifiedIdeograph9abe', [], 'Display', None],
  [ 0x9abf, 'CjkUnifiedIdeograph9abf', [], 'Display', None],
  [ 0x9ac0, 'CjkUnifiedIdeograph9ac0', [], 'Display', None],
  [ 0x9ac1, 'CjkUnifiedIdeograph9ac1', [], 'Display', None],
  [ 0x9ac2, 'CjkUnifiedIdeograph9ac2', [], 'Display', None],
  [ 0x9ac3, 'CjkUnifiedIdeograph9ac3', [], 'Display', None],
  [ 0x9ac4, 'CjkUnifiedIdeograph9ac4', [], 'Display', None],
  [ 0x9ac5, 'CjkUnifiedIdeograph9ac5', [], 'Display', None],
  [ 0x9ac6, 'CjkUnifiedIdeograph9ac6', [], 'Display', None],
  [ 0x9ac7, 'CjkUnifiedIdeograph9ac7', [], 'Display', None],
  [ 0x9ac8, 'CjkUnifiedIdeograph9ac8', [], 'Display', None],
  [ 0x9ac9, 'CjkUnifiedIdeograph9ac9', [], 'Display', None],
  [ 0x9aca, 'CjkUnifiedIdeograph9aca', [], 'Display', None],
  [ 0x9acb, 'CjkUnifiedIdeograph9acb', [], 'Display', None],
  [ 0x9acc, 'CjkUnifiedIdeograph9acc', [], 'Display', None],
  [ 0x9acd, 'CjkUnifiedIdeograph9acd', [], 'Display', None],
  [ 0x9ace, 'CjkUnifiedIdeograph9ace', [], 'Display', None],
  [ 0x9acf, 'CjkUnifiedIdeograph9acf', [], 'Display', None],
  [ 0x9ad0, 'CjkUnifiedIdeograph9ad0', [], 'Display', None],
  [ 0x9ad1, 'CjkUnifiedIdeograph9ad1', [], 'Display', None],
  [ 0x9ad2, 'CjkUnifiedIdeograph9ad2', [], 'Display', None],
  [ 0x9ad3, 'CjkUnifiedIdeograph9ad3', [], 'Display', None],
  [ 0x9ad4, 'CjkUnifiedIdeograph9ad4', [], 'Display', None],
  [ 0x9ad5, 'CjkUnifiedIdeograph9ad5', [], 'Display', None],
  [ 0x9ad6, 'CjkUnifiedIdeograph9ad6', [], 'Display', None],
  [ 0x9ad7, 'CjkUnifiedIdeograph9ad7', [], 'Display', None],
  [ 0x9ad8, 'CjkUnifiedIdeograph9ad8', [], 'Display', None],
  [ 0x9ad9, 'CjkUnifiedIdeograph9ad9', [], 'Display', None],
  [ 0x9ada, 'CjkUnifiedIdeograph9ada', [], 'Display', None],
  [ 0x9adb, 'CjkUnifiedIdeograph9adb', [], 'Display', None],
  [ 0x9adc, 'CjkUnifiedIdeograph9adc', [], 'Display', None],
  [ 0x9add, 'CjkUnifiedIdeograph9add', [], 'Display', None],
  [ 0x9ade, 'CjkUnifiedIdeograph9ade', [], 'Display', None],
  [ 0x9adf, 'CjkUnifiedIdeograph9adf', [], 'Display', None],
  [ 0x9ae0, 'CjkUnifiedIdeograph9ae0', [], 'Display', None],
  [ 0x9ae1, 'CjkUnifiedIdeograph9ae1', [], 'Display', None],
  [ 0x9ae2, 'CjkUnifiedIdeograph9ae2', [], 'Display', None],
  [ 0x9ae3, 'CjkUnifiedIdeograph9ae3', [], 'Display', None],
  [ 0x9ae4, 'CjkUnifiedIdeograph9ae4', [], 'Display', None],
  [ 0x9ae5, 'CjkUnifiedIdeograph9ae5', [], 'Display', None],
  [ 0x9ae6, 'CjkUnifiedIdeograph9ae6', [], 'Display', None],
  [ 0x9ae7, 'CjkUnifiedIdeograph9ae7', [], 'Display', None],
  [ 0x9ae8, 'CjkUnifiedIdeograph9ae8', [], 'Display', None],
  [ 0x9ae9, 'CjkUnifiedIdeograph9ae9', [], 'Display', None],
  [ 0x9aea, 'CjkUnifiedIdeograph9aea', [], 'Display', None],
  [ 0x9aeb, 'CjkUnifiedIdeograph9aeb', [], 'Display', None],
  [ 0x9aec, 'CjkUnifiedIdeograph9aec', [], 'Display', None],
  [ 0x9aed, 'CjkUnifiedIdeograph9aed', [], 'Display', None],
  [ 0x9aee, 'CjkUnifiedIdeograph9aee', [], 'Display', None],
  [ 0x9aef, 'CjkUnifiedIdeograph9aef', [], 'Display', None],
  [ 0x9af0, 'CjkUnifiedIdeograph9af0', [], 'Display', None],
  [ 0x9af1, 'CjkUnifiedIdeograph9af1', [], 'Display', None],
  [ 0x9af2, 'CjkUnifiedIdeograph9af2', [], 'Display', None],
  [ 0x9af3, 'CjkUnifiedIdeograph9af3', [], 'Display', None],
  [ 0x9af4, 'CjkUnifiedIdeograph9af4', [], 'Display', None],
  [ 0x9af5, 'CjkUnifiedIdeograph9af5', [], 'Display', None],
  [ 0x9af6, 'CjkUnifiedIdeograph9af6', [], 'Display', None],
  [ 0x9af7, 'CjkUnifiedIdeograph9af7', [], 'Display', None],
  [ 0x9af8, 'CjkUnifiedIdeograph9af8', [], 'Display', None],
  [ 0x9af9, 'CjkUnifiedIdeograph9af9', [], 'Display', None],
  [ 0x9afa, 'CjkUnifiedIdeograph9afa', [], 'Display', None],
  [ 0x9afb, 'CjkUnifiedIdeograph9afb', [], 'Display', None],
  [ 0x9afc, 'CjkUnifiedIdeograph9afc', [], 'Display', None],
  [ 0x9afd, 'CjkUnifiedIdeograph9afd', [], 'Display', None],
  [ 0x9afe, 'CjkUnifiedIdeograph9afe', [], 'Display', None],
  [ 0x9aff, 'CjkUnifiedIdeograph9aff', [], 'Display', None],
  [ 0x9b00, 'CjkUnifiedIdeograph9b00', [], 'Display', None],
  [ 0x9b01, 'CjkUnifiedIdeograph9b01', [], 'Display', None],
  [ 0x9b02, 'CjkUnifiedIdeograph9b02', [], 'Display', None],
  [ 0x9b03, 'CjkUnifiedIdeograph9b03', [], 'Display', None],
  [ 0x9b04, 'CjkUnifiedIdeograph9b04', [], 'Display', None],
  [ 0x9b05, 'CjkUnifiedIdeograph9b05', [], 'Display', None],
  [ 0x9b06, 'CjkUnifiedIdeograph9b06', [], 'Display', None],
  [ 0x9b07, 'CjkUnifiedIdeograph9b07', [], 'Display', None],
  [ 0x9b08, 'CjkUnifiedIdeograph9b08', [], 'Display', None],
  [ 0x9b09, 'CjkUnifiedIdeograph9b09', [], 'Display', None],
  [ 0x9b0a, 'CjkUnifiedIdeograph9b0a', [], 'Display', None],
  [ 0x9b0b, 'CjkUnifiedIdeograph9b0b', [], 'Display', None],
  [ 0x9b0c, 'CjkUnifiedIdeograph9b0c', [], 'Display', None],
  [ 0x9b0d, 'CjkUnifiedIdeograph9b0d', [], 'Display', None],
  [ 0x9b0e, 'CjkUnifiedIdeograph9b0e', [], 'Display', None],
  [ 0x9b0f, 'CjkUnifiedIdeograph9b0f', [], 'Display', None],
  [ 0x9b10, 'CjkUnifiedIdeograph9b10', [], 'Display', None],
  [ 0x9b11, 'CjkUnifiedIdeograph9b11', [], 'Display', None],
  [ 0x9b12, 'CjkUnifiedIdeograph9b12', [], 'Display', None],
  [ 0x9b13, 'CjkUnifiedIdeograph9b13', [], 'Display', None],
  [ 0x9b14, 'CjkUnifiedIdeograph9b14', [], 'Display', None],
  [ 0x9b15, 'CjkUnifiedIdeograph9b15', [], 'Display', None],
  [ 0x9b16, 'CjkUnifiedIdeograph9b16', [], 'Display', None],
  [ 0x9b17, 'CjkUnifiedIdeograph9b17', [], 'Display', None],
  [ 0x9b18, 'CjkUnifiedIdeograph9b18', [], 'Display', None],
  [ 0x9b19, 'CjkUnifiedIdeograph9b19', [], 'Display', None],
  [ 0x9b1a, 'CjkUnifiedIdeograph9b1a', [], 'Display', None],
  [ 0x9b1b, 'CjkUnifiedIdeograph9b1b', [], 'Display', None],
  [ 0x9b1c, 'CjkUnifiedIdeograph9b1c', [], 'Display', None],
  [ 0x9b1d, 'CjkUnifiedIdeograph9b1d', [], 'Display', None],
  [ 0x9b1e, 'CjkUnifiedIdeograph9b1e', [], 'Display', None],
  [ 0x9b1f, 'CjkUnifiedIdeograph9b1f', [], 'Display', None],
  [ 0x9b20, 'CjkUnifiedIdeograph9b20', [], 'Display', None],
  [ 0x9b21, 'CjkUnifiedIdeograph9b21', [], 'Display', None],
  [ 0x9b22, 'CjkUnifiedIdeograph9b22', [], 'Display', None],
  [ 0x9b23, 'CjkUnifiedIdeograph9b23', [], 'Display', None],
  [ 0x9b24, 'CjkUnifiedIdeograph9b24', [], 'Display', None],
  [ 0x9b25, 'CjkUnifiedIdeograph9b25', [], 'Display', None],
  [ 0x9b26, 'CjkUnifiedIdeograph9b26', [], 'Display', None],
  [ 0x9b27, 'CjkUnifiedIdeograph9b27', [], 'Display', None],
  [ 0x9b28, 'CjkUnifiedIdeograph9b28', [], 'Display', None],
  [ 0x9b29, 'CjkUnifiedIdeograph9b29', [], 'Display', None],
  [ 0x9b2a, 'CjkUnifiedIdeograph9b2a', [], 'Display', None],
  [ 0x9b2b, 'CjkUnifiedIdeograph9b2b', [], 'Display', None],
  [ 0x9b2c, 'CjkUnifiedIdeograph9b2c', [], 'Display', None],
  [ 0x9b2d, 'CjkUnifiedIdeograph9b2d', [], 'Display', None],
  [ 0x9b2e, 'CjkUnifiedIdeograph9b2e', [], 'Display', None],
  [ 0x9b2f, 'CjkUnifiedIdeograph9b2f', [], 'Display', None],
  [ 0x9b30, 'CjkUnifiedIdeograph9b30', [], 'Display', None],
  [ 0x9b31, 'CjkUnifiedIdeograph9b31', [], 'Display', None],
  [ 0x9b32, 'CjkUnifiedIdeograph9b32', [], 'Display', None],
  [ 0x9b33, 'CjkUnifiedIdeograph9b33', [], 'Display', None],
  [ 0x9b34, 'CjkUnifiedIdeograph9b34', [], 'Display', None],
  [ 0x9b35, 'CjkUnifiedIdeograph9b35', [], 'Display', None],
  [ 0x9b36, 'CjkUnifiedIdeograph9b36', [], 'Display', None],
  [ 0x9b37, 'CjkUnifiedIdeograph9b37', [], 'Display', None],
  [ 0x9b38, 'CjkUnifiedIdeograph9b38', [], 'Display', None],
  [ 0x9b39, 'CjkUnifiedIdeograph9b39', [], 'Display', None],
  [ 0x9b3a, 'CjkUnifiedIdeograph9b3a', [], 'Display', None],
  [ 0x9b3b, 'CjkUnifiedIdeograph9b3b', [], 'Display', None],
  [ 0x9b3c, 'CjkUnifiedIdeograph9b3c', [], 'Display', None],
  [ 0x9b3d, 'CjkUnifiedIdeograph9b3d', [], 'Display', None],
  [ 0x9b3e, 'CjkUnifiedIdeograph9b3e', [], 'Display', None],
  [ 0x9b3f, 'CjkUnifiedIdeograph9b3f', [], 'Display', None],
  [ 0x9b40, 'CjkUnifiedIdeograph9b40', [], 'Display', None],
  [ 0x9b41, 'CjkUnifiedIdeograph9b41', [], 'Display', None],
  [ 0x9b42, 'CjkUnifiedIdeograph9b42', [], 'Display', None],
  [ 0x9b43, 'CjkUnifiedIdeograph9b43', [], 'Display', None],
  [ 0x9b44, 'CjkUnifiedIdeograph9b44', [], 'Display', None],
  [ 0x9b45, 'CjkUnifiedIdeograph9b45', [], 'Display', None],
  [ 0x9b46, 'CjkUnifiedIdeograph9b46', [], 'Display', None],
  [ 0x9b47, 'CjkUnifiedIdeograph9b47', [], 'Display', None],
  [ 0x9b48, 'CjkUnifiedIdeograph9b48', [], 'Display', None],
  [ 0x9b49, 'CjkUnifiedIdeograph9b49', [], 'Display', None],
  [ 0x9b4a, 'CjkUnifiedIdeograph9b4a', [], 'Display', None],
  [ 0x9b4b, 'CjkUnifiedIdeograph9b4b', [], 'Display', None],
  [ 0x9b4c, 'CjkUnifiedIdeograph9b4c', [], 'Display', None],
  [ 0x9b4d, 'CjkUnifiedIdeograph9b4d', [], 'Display', None],
  [ 0x9b4e, 'CjkUnifiedIdeograph9b4e', [], 'Display', None],
  [ 0x9b4f, 'CjkUnifiedIdeograph9b4f', [], 'Display', None],
  [ 0x9b50, 'CjkUnifiedIdeograph9b50', [], 'Display', None],
  [ 0x9b51, 'CjkUnifiedIdeograph9b51', [], 'Display', None],
  [ 0x9b52, 'CjkUnifiedIdeograph9b52', [], 'Display', None],
  [ 0x9b53, 'CjkUnifiedIdeograph9b53', [], 'Display', None],
  [ 0x9b54, 'CjkUnifiedIdeograph9b54', [], 'Display', None],
  [ 0x9b55, 'CjkUnifiedIdeograph9b55', [], 'Display', None],
  [ 0x9b56, 'CjkUnifiedIdeograph9b56', [], 'Display', None],
  [ 0x9b57, 'CjkUnifiedIdeograph9b57', [], 'Display', None],
  [ 0x9b58, 'CjkUnifiedIdeograph9b58', [], 'Display', None],
  [ 0x9b59, 'CjkUnifiedIdeograph9b59', [], 'Display', None],
  [ 0x9b5a, 'CjkUnifiedIdeograph9b5a', [], 'Display', None],
  [ 0x9b5b, 'CjkUnifiedIdeograph9b5b', [], 'Display', None],
  [ 0x9b5c, 'CjkUnifiedIdeograph9b5c', [], 'Display', None],
  [ 0x9b5d, 'CjkUnifiedIdeograph9b5d', [], 'Display', None],
  [ 0x9b5e, 'CjkUnifiedIdeograph9b5e', [], 'Display', None],
  [ 0x9b5f, 'CjkUnifiedIdeograph9b5f', [], 'Display', None],
  [ 0x9b60, 'CjkUnifiedIdeograph9b60', [], 'Display', None],
  [ 0x9b61, 'CjkUnifiedIdeograph9b61', [], 'Display', None],
  [ 0x9b62, 'CjkUnifiedIdeograph9b62', [], 'Display', None],
  [ 0x9b63, 'CjkUnifiedIdeograph9b63', [], 'Display', None],
  [ 0x9b64, 'CjkUnifiedIdeograph9b64', [], 'Display', None],
  [ 0x9b65, 'CjkUnifiedIdeograph9b65', [], 'Display', None],
  [ 0x9b66, 'CjkUnifiedIdeograph9b66', [], 'Display', None],
  [ 0x9b67, 'CjkUnifiedIdeograph9b67', [], 'Display', None],
  [ 0x9b68, 'CjkUnifiedIdeograph9b68', [], 'Display', None],
  [ 0x9b69, 'CjkUnifiedIdeograph9b69', [], 'Display', None],
  [ 0x9b6a, 'CjkUnifiedIdeograph9b6a', [], 'Display', None],
  [ 0x9b6b, 'CjkUnifiedIdeograph9b6b', [], 'Display', None],
  [ 0x9b6c, 'CjkUnifiedIdeograph9b6c', [], 'Display', None],
  [ 0x9b6d, 'CjkUnifiedIdeograph9b6d', [], 'Display', None],
  [ 0x9b6e, 'CjkUnifiedIdeograph9b6e', [], 'Display', None],
  [ 0x9b6f, 'CjkUnifiedIdeograph9b6f', [], 'Display', None],
  [ 0x9b70, 'CjkUnifiedIdeograph9b70', [], 'Display', None],
  [ 0x9b71, 'CjkUnifiedIdeograph9b71', [], 'Display', None],
  [ 0x9b72, 'CjkUnifiedIdeograph9b72', [], 'Display', None],
  [ 0x9b73, 'CjkUnifiedIdeograph9b73', [], 'Display', None],
  [ 0x9b74, 'CjkUnifiedIdeograph9b74', [], 'Display', None],
  [ 0x9b75, 'CjkUnifiedIdeograph9b75', [], 'Display', None],
  [ 0x9b76, 'CjkUnifiedIdeograph9b76', [], 'Display', None],
  [ 0x9b77, 'CjkUnifiedIdeograph9b77', [], 'Display', None],
  [ 0x9b78, 'CjkUnifiedIdeograph9b78', [], 'Display', None],
  [ 0x9b79, 'CjkUnifiedIdeograph9b79', [], 'Display', None],
  [ 0x9b7a, 'CjkUnifiedIdeograph9b7a', [], 'Display', None],
  [ 0x9b7b, 'CjkUnifiedIdeograph9b7b', [], 'Display', None],
  [ 0x9b7c, 'CjkUnifiedIdeograph9b7c', [], 'Display', None],
  [ 0x9b7d, 'CjkUnifiedIdeograph9b7d', [], 'Display', None],
  [ 0x9b7e, 'CjkUnifiedIdeograph9b7e', [], 'Display', None],
  [ 0x9b7f, 'CjkUnifiedIdeograph9b7f', [], 'Display', None],
  [ 0x9b80, 'CjkUnifiedIdeograph9b80', [], 'Display', None],
  [ 0x9b81, 'CjkUnifiedIdeograph9b81', [], 'Display', None],
  [ 0x9b82, 'CjkUnifiedIdeograph9b82', [], 'Display', None],
  [ 0x9b83, 'CjkUnifiedIdeograph9b83', [], 'Display', None],
  [ 0x9b84, 'CjkUnifiedIdeograph9b84', [], 'Display', None],
  [ 0x9b85, 'CjkUnifiedIdeograph9b85', [], 'Display', None],
  [ 0x9b86, 'CjkUnifiedIdeograph9b86', [], 'Display', None],
  [ 0x9b87, 'CjkUnifiedIdeograph9b87', [], 'Display', None],
  [ 0x9b88, 'CjkUnifiedIdeograph9b88', [], 'Display', None],
  [ 0x9b89, 'CjkUnifiedIdeograph9b89', [], 'Display', None],
  [ 0x9b8a, 'CjkUnifiedIdeograph9b8a', [], 'Display', None],
  [ 0x9b8b, 'CjkUnifiedIdeograph9b8b', [], 'Display', None],
  [ 0x9b8c, 'CjkUnifiedIdeograph9b8c', [], 'Display', None],
  [ 0x9b8d, 'CjkUnifiedIdeograph9b8d', [], 'Display', None],
  [ 0x9b8e, 'CjkUnifiedIdeograph9b8e', [], 'Display', None],
  [ 0x9b8f, 'CjkUnifiedIdeograph9b8f', [], 'Display', None],
  [ 0x9b90, 'CjkUnifiedIdeograph9b90', [], 'Display', None],
  [ 0x9b91, 'CjkUnifiedIdeograph9b91', [], 'Display', None],
  [ 0x9b92, 'CjkUnifiedIdeograph9b92', [], 'Display', None],
  [ 0x9b93, 'CjkUnifiedIdeograph9b93', [], 'Display', None],
  [ 0x9b94, 'CjkUnifiedIdeograph9b94', [], 'Display', None],
  [ 0x9b95, 'CjkUnifiedIdeograph9b95', [], 'Display', None],
  [ 0x9b96, 'CjkUnifiedIdeograph9b96', [], 'Display', None],
  [ 0x9b97, 'CjkUnifiedIdeograph9b97', [], 'Display', None],
  [ 0x9b98, 'CjkUnifiedIdeograph9b98', [], 'Display', None],
  [ 0x9b99, 'CjkUnifiedIdeograph9b99', [], 'Display', None],
  [ 0x9b9a, 'CjkUnifiedIdeograph9b9a', [], 'Display', None],
  [ 0x9b9b, 'CjkUnifiedIdeograph9b9b', [], 'Display', None],
  [ 0x9b9c, 'CjkUnifiedIdeograph9b9c', [], 'Display', None],
  [ 0x9b9d, 'CjkUnifiedIdeograph9b9d', [], 'Display', None],
  [ 0x9b9e, 'CjkUnifiedIdeograph9b9e', [], 'Display', None],
  [ 0x9b9f, 'CjkUnifiedIdeograph9b9f', [], 'Display', None],
  [ 0x9ba0, 'CjkUnifiedIdeograph9ba0', [], 'Display', None],
  [ 0x9ba1, 'CjkUnifiedIdeograph9ba1', [], 'Display', None],
  [ 0x9ba2, 'CjkUnifiedIdeograph9ba2', [], 'Display', None],
  [ 0x9ba3, 'CjkUnifiedIdeograph9ba3', [], 'Display', None],
  [ 0x9ba4, 'CjkUnifiedIdeograph9ba4', [], 'Display', None],
  [ 0x9ba5, 'CjkUnifiedIdeograph9ba5', [], 'Display', None],
  [ 0x9ba6, 'CjkUnifiedIdeograph9ba6', [], 'Display', None],
  [ 0x9ba7, 'CjkUnifiedIdeograph9ba7', [], 'Display', None],
  [ 0x9ba8, 'CjkUnifiedIdeograph9ba8', [], 'Display', None],
  [ 0x9ba9, 'CjkUnifiedIdeograph9ba9', [], 'Display', None],
  [ 0x9baa, 'CjkUnifiedIdeograph9baa', [], 'Display', None],
  [ 0x9bab, 'CjkUnifiedIdeograph9bab', [], 'Display', None],
  [ 0x9bac, 'CjkUnifiedIdeograph9bac', [], 'Display', None],
  [ 0x9bad, 'CjkUnifiedIdeograph9bad', [], 'Display', None],
  [ 0x9bae, 'CjkUnifiedIdeograph9bae', [], 'Display', None],
  [ 0x9baf, 'CjkUnifiedIdeograph9baf', [], 'Display', None],
  [ 0x9bb0, 'CjkUnifiedIdeograph9bb0', [], 'Display', None],
  [ 0x9bb1, 'CjkUnifiedIdeograph9bb1', [], 'Display', None],
  [ 0x9bb2, 'CjkUnifiedIdeograph9bb2', [], 'Display', None],
  [ 0x9bb3, 'CjkUnifiedIdeograph9bb3', [], 'Display', None],
  [ 0x9bb4, 'CjkUnifiedIdeograph9bb4', [], 'Display', None],
  [ 0x9bb5, 'CjkUnifiedIdeograph9bb5', [], 'Display', None],
  [ 0x9bb6, 'CjkUnifiedIdeograph9bb6', [], 'Display', None],
  [ 0x9bb7, 'CjkUnifiedIdeograph9bb7', [], 'Display', None],
  [ 0x9bb8, 'CjkUnifiedIdeograph9bb8', [], 'Display', None],
  [ 0x9bb9, 'CjkUnifiedIdeograph9bb9', [], 'Display', None],
  [ 0x9bba, 'CjkUnifiedIdeograph9bba', [], 'Display', None],
  [ 0x9bbb, 'CjkUnifiedIdeograph9bbb', [], 'Display', None],
  [ 0x9bbc, 'CjkUnifiedIdeograph9bbc', [], 'Display', None],
  [ 0x9bbd, 'CjkUnifiedIdeograph9bbd', [], 'Display', None],
  [ 0x9bbe, 'CjkUnifiedIdeograph9bbe', [], 'Display', None],
  [ 0x9bbf, 'CjkUnifiedIdeograph9bbf', [], 'Display', None],
  [ 0x9bc0, 'CjkUnifiedIdeograph9bc0', [], 'Display', None],
  [ 0x9bc1, 'CjkUnifiedIdeograph9bc1', [], 'Display', None],
  [ 0x9bc2, 'CjkUnifiedIdeograph9bc2', [], 'Display', None],
  [ 0x9bc3, 'CjkUnifiedIdeograph9bc3', [], 'Display', None],
  [ 0x9bc4, 'CjkUnifiedIdeograph9bc4', [], 'Display', None],
  [ 0x9bc5, 'CjkUnifiedIdeograph9bc5', [], 'Display', None],
  [ 0x9bc6, 'CjkUnifiedIdeograph9bc6', [], 'Display', None],
  [ 0x9bc7, 'CjkUnifiedIdeograph9bc7', [], 'Display', None],
  [ 0x9bc8, 'CjkUnifiedIdeograph9bc8', [], 'Display', None],
  [ 0x9bc9, 'CjkUnifiedIdeograph9bc9', [], 'Display', None],
  [ 0x9bca, 'CjkUnifiedIdeograph9bca', [], 'Display', None],
  [ 0x9bcb, 'CjkUnifiedIdeograph9bcb', [], 'Display', None],
  [ 0x9bcc, 'CjkUnifiedIdeograph9bcc', [], 'Display', None],
  [ 0x9bcd, 'CjkUnifiedIdeograph9bcd', [], 'Display', None],
  [ 0x9bce, 'CjkUnifiedIdeograph9bce', [], 'Display', None],
  [ 0x9bcf, 'CjkUnifiedIdeograph9bcf', [], 'Display', None],
  [ 0x9bd0, 'CjkUnifiedIdeograph9bd0', [], 'Display', None],
  [ 0x9bd1, 'CjkUnifiedIdeograph9bd1', [], 'Display', None],
  [ 0x9bd2, 'CjkUnifiedIdeograph9bd2', [], 'Display', None],
  [ 0x9bd3, 'CjkUnifiedIdeograph9bd3', [], 'Display', None],
  [ 0x9bd4, 'CjkUnifiedIdeograph9bd4', [], 'Display', None],
  [ 0x9bd5, 'CjkUnifiedIdeograph9bd5', [], 'Display', None],
  [ 0x9bd6, 'CjkUnifiedIdeograph9bd6', [], 'Display', None],
  [ 0x9bd7, 'CjkUnifiedIdeograph9bd7', [], 'Display', None],
  [ 0x9bd8, 'CjkUnifiedIdeograph9bd8', [], 'Display', None],
  [ 0x9bd9, 'CjkUnifiedIdeograph9bd9', [], 'Display', None],
  [ 0x9bda, 'CjkUnifiedIdeograph9bda', [], 'Display', None],
  [ 0x9bdb, 'CjkUnifiedIdeograph9bdb', [], 'Display', None],
  [ 0x9bdc, 'CjkUnifiedIdeograph9bdc', [], 'Display', None],
  [ 0x9bdd, 'CjkUnifiedIdeograph9bdd', [], 'Display', None],
  [ 0x9bde, 'CjkUnifiedIdeograph9bde', [], 'Display', None],
  [ 0x9bdf, 'CjkUnifiedIdeograph9bdf', [], 'Display', None],
  [ 0x9be0, 'CjkUnifiedIdeograph9be0', [], 'Display', None],
  [ 0x9be1, 'CjkUnifiedIdeograph9be1', [], 'Display', None],
  [ 0x9be2, 'CjkUnifiedIdeograph9be2', [], 'Display', None],
  [ 0x9be3, 'CjkUnifiedIdeograph9be3', [], 'Display', None],
  [ 0x9be4, 'CjkUnifiedIdeograph9be4', [], 'Display', None],
  [ 0x9be5, 'CjkUnifiedIdeograph9be5', [], 'Display', None],
  [ 0x9be6, 'CjkUnifiedIdeograph9be6', [], 'Display', None],
  [ 0x9be7, 'CjkUnifiedIdeograph9be7', [], 'Display', None],
  [ 0x9be8, 'CjkUnifiedIdeograph9be8', [], 'Display', None],
  [ 0x9be9, 'CjkUnifiedIdeograph9be9', [], 'Display', None],
  [ 0x9bea, 'CjkUnifiedIdeograph9bea', [], 'Display', None],
  [ 0x9beb, 'CjkUnifiedIdeograph9beb', [], 'Display', None],
  [ 0x9bec, 'CjkUnifiedIdeograph9bec', [], 'Display', None],
  [ 0x9bed, 'CjkUnifiedIdeograph9bed', [], 'Display', None],
  [ 0x9bee, 'CjkUnifiedIdeograph9bee', [], 'Display', None],
  [ 0x9bef, 'CjkUnifiedIdeograph9bef', [], 'Display', None],
  [ 0x9bf0, 'CjkUnifiedIdeograph9bf0', [], 'Display', None],
  [ 0x9bf1, 'CjkUnifiedIdeograph9bf1', [], 'Display', None],
  [ 0x9bf2, 'CjkUnifiedIdeograph9bf2', [], 'Display', None],
  [ 0x9bf3, 'CjkUnifiedIdeograph9bf3', [], 'Display', None],
  [ 0x9bf4, 'CjkUnifiedIdeograph9bf4', [], 'Display', None],
  [ 0x9bf5, 'CjkUnifiedIdeograph9bf5', [], 'Display', None],
  [ 0x9bf6, 'CjkUnifiedIdeograph9bf6', [], 'Display', None],
  [ 0x9bf7, 'CjkUnifiedIdeograph9bf7', [], 'Display', None],
  [ 0x9bf8, 'CjkUnifiedIdeograph9bf8', [], 'Display', None],
  [ 0x9bf9, 'CjkUnifiedIdeograph9bf9', [], 'Display', None],
  [ 0x9bfa, 'CjkUnifiedIdeograph9bfa', [], 'Display', None],
  [ 0x9bfb, 'CjkUnifiedIdeograph9bfb', [], 'Display', None],
  [ 0x9bfc, 'CjkUnifiedIdeograph9bfc', [], 'Display', None],
  [ 0x9bfd, 'CjkUnifiedIdeograph9bfd', [], 'Display', None],
  [ 0x9bfe, 'CjkUnifiedIdeograph9bfe', [], 'Display', None],
  [ 0x9bff, 'CjkUnifiedIdeograph9bff', [], 'Display', None],
  [ 0x9c00, 'CjkUnifiedIdeograph9c00', [], 'Display', None],
  [ 0x9c01, 'CjkUnifiedIdeograph9c01', [], 'Display', None],
  [ 0x9c02, 'CjkUnifiedIdeograph9c02', [], 'Display', None],
  [ 0x9c03, 'CjkUnifiedIdeograph9c03', [], 'Display', None],
  [ 0x9c04, 'CjkUnifiedIdeograph9c04', [], 'Display', None],
  [ 0x9c05, 'CjkUnifiedIdeograph9c05', [], 'Display', None],
  [ 0x9c06, 'CjkUnifiedIdeograph9c06', [], 'Display', None],
  [ 0x9c07, 'CjkUnifiedIdeograph9c07', [], 'Display', None],
  [ 0x9c08, 'CjkUnifiedIdeograph9c08', [], 'Display', None],
  [ 0x9c09, 'CjkUnifiedIdeograph9c09', [], 'Display', None],
  [ 0x9c0a, 'CjkUnifiedIdeograph9c0a', [], 'Display', None],
  [ 0x9c0b, 'CjkUnifiedIdeograph9c0b', [], 'Display', None],
  [ 0x9c0c, 'CjkUnifiedIdeograph9c0c', [], 'Display', None],
  [ 0x9c0d, 'CjkUnifiedIdeograph9c0d', [], 'Display', None],
  [ 0x9c0e, 'CjkUnifiedIdeograph9c0e', [], 'Display', None],
  [ 0x9c0f, 'CjkUnifiedIdeograph9c0f', [], 'Display', None],
  [ 0x9c10, 'CjkUnifiedIdeograph9c10', [], 'Display', None],
  [ 0x9c11, 'CjkUnifiedIdeograph9c11', [], 'Display', None],
  [ 0x9c12, 'CjkUnifiedIdeograph9c12', [], 'Display', None],
  [ 0x9c13, 'CjkUnifiedIdeograph9c13', [], 'Display', None],
  [ 0x9c14, 'CjkUnifiedIdeograph9c14', [], 'Display', None],
  [ 0x9c15, 'CjkUnifiedIdeograph9c15', [], 'Display', None],
  [ 0x9c16, 'CjkUnifiedIdeograph9c16', [], 'Display', None],
  [ 0x9c17, 'CjkUnifiedIdeograph9c17', [], 'Display', None],
  [ 0x9c18, 'CjkUnifiedIdeograph9c18', [], 'Display', None],
  [ 0x9c19, 'CjkUnifiedIdeograph9c19', [], 'Display', None],
  [ 0x9c1a, 'CjkUnifiedIdeograph9c1a', [], 'Display', None],
  [ 0x9c1b, 'CjkUnifiedIdeograph9c1b', [], 'Display', None],
  [ 0x9c1c, 'CjkUnifiedIdeograph9c1c', [], 'Display', None],
  [ 0x9c1d, 'CjkUnifiedIdeograph9c1d', [], 'Display', None],
  [ 0x9c1e, 'CjkUnifiedIdeograph9c1e', [], 'Display', None],
  [ 0x9c1f, 'CjkUnifiedIdeograph9c1f', [], 'Display', None],
  [ 0x9c20, 'CjkUnifiedIdeograph9c20', [], 'Display', None],
  [ 0x9c21, 'CjkUnifiedIdeograph9c21', [], 'Display', None],
  [ 0x9c22, 'CjkUnifiedIdeograph9c22', [], 'Display', None],
  [ 0x9c23, 'CjkUnifiedIdeograph9c23', [], 'Display', None],
  [ 0x9c24, 'CjkUnifiedIdeograph9c24', [], 'Display', None],
  [ 0x9c25, 'CjkUnifiedIdeograph9c25', [], 'Display', None],
  [ 0x9c26, 'CjkUnifiedIdeograph9c26', [], 'Display', None],
  [ 0x9c27, 'CjkUnifiedIdeograph9c27', [], 'Display', None],
  [ 0x9c28, 'CjkUnifiedIdeograph9c28', [], 'Display', None],
  [ 0x9c29, 'CjkUnifiedIdeograph9c29', [], 'Display', None],
  [ 0x9c2a, 'CjkUnifiedIdeograph9c2a', [], 'Display', None],
  [ 0x9c2b, 'CjkUnifiedIdeograph9c2b', [], 'Display', None],
  [ 0x9c2c, 'CjkUnifiedIdeograph9c2c', [], 'Display', None],
  [ 0x9c2d, 'CjkUnifiedIdeograph9c2d', [], 'Display', None],
  [ 0x9c2e, 'CjkUnifiedIdeograph9c2e', [], 'Display', None],
  [ 0x9c2f, 'CjkUnifiedIdeograph9c2f', [], 'Display', None],
  [ 0x9c30, 'CjkUnifiedIdeograph9c30', [], 'Display', None],
  [ 0x9c31, 'CjkUnifiedIdeograph9c31', [], 'Display', None],
  [ 0x9c32, 'CjkUnifiedIdeograph9c32', [], 'Display', None],
  [ 0x9c33, 'CjkUnifiedIdeograph9c33', [], 'Display', None],
  [ 0x9c34, 'CjkUnifiedIdeograph9c34', [], 'Display', None],
  [ 0x9c35, 'CjkUnifiedIdeograph9c35', [], 'Display', None],
  [ 0x9c36, 'CjkUnifiedIdeograph9c36', [], 'Display', None],
  [ 0x9c37, 'CjkUnifiedIdeograph9c37', [], 'Display', None],
  [ 0x9c38, 'CjkUnifiedIdeograph9c38', [], 'Display', None],
  [ 0x9c39, 'CjkUnifiedIdeograph9c39', [], 'Display', None],
  [ 0x9c3a, 'CjkUnifiedIdeograph9c3a', [], 'Display', None],
  [ 0x9c3b, 'CjkUnifiedIdeograph9c3b', [], 'Display', None],
  [ 0x9c3c, 'CjkUnifiedIdeograph9c3c', [], 'Display', None],
  [ 0x9c3d, 'CjkUnifiedIdeograph9c3d', [], 'Display', None],
  [ 0x9c3e, 'CjkUnifiedIdeograph9c3e', [], 'Display', None],
  [ 0x9c3f, 'CjkUnifiedIdeograph9c3f', [], 'Display', None],
  [ 0x9c40, 'CjkUnifiedIdeograph9c40', [], 'Display', None],
  [ 0x9c41, 'CjkUnifiedIdeograph9c41', [], 'Display', None],
  [ 0x9c42, 'CjkUnifiedIdeograph9c42', [], 'Display', None],
  [ 0x9c43, 'CjkUnifiedIdeograph9c43', [], 'Display', None],
  [ 0x9c44, 'CjkUnifiedIdeograph9c44', [], 'Display', None],
  [ 0x9c45, 'CjkUnifiedIdeograph9c45', [], 'Display', None],
  [ 0x9c46, 'CjkUnifiedIdeograph9c46', [], 'Display', None],
  [ 0x9c47, 'CjkUnifiedIdeograph9c47', [], 'Display', None],
  [ 0x9c48, 'CjkUnifiedIdeograph9c48', [], 'Display', None],
  [ 0x9c49, 'CjkUnifiedIdeograph9c49', [], 'Display', None],
  [ 0x9c4a, 'CjkUnifiedIdeograph9c4a', [], 'Display', None],
  [ 0x9c4b, 'CjkUnifiedIdeograph9c4b', [], 'Display', None],
  [ 0x9c4c, 'CjkUnifiedIdeograph9c4c', [], 'Display', None],
  [ 0x9c4d, 'CjkUnifiedIdeograph9c4d', [], 'Display', None],
  [ 0x9c4e, 'CjkUnifiedIdeograph9c4e', [], 'Display', None],
  [ 0x9c4f, 'CjkUnifiedIdeograph9c4f', [], 'Display', None],
  [ 0x9c50, 'CjkUnifiedIdeograph9c50', [], 'Display', None],
  [ 0x9c51, 'CjkUnifiedIdeograph9c51', [], 'Display', None],
  [ 0x9c52, 'CjkUnifiedIdeograph9c52', [], 'Display', None],
  [ 0x9c53, 'CjkUnifiedIdeograph9c53', [], 'Display', None],
  [ 0x9c54, 'CjkUnifiedIdeograph9c54', [], 'Display', None],
  [ 0x9c55, 'CjkUnifiedIdeograph9c55', [], 'Display', None],
  [ 0x9c56, 'CjkUnifiedIdeograph9c56', [], 'Display', None],
  [ 0x9c57, 'CjkUnifiedIdeograph9c57', [], 'Display', None],
  [ 0x9c58, 'CjkUnifiedIdeograph9c58', [], 'Display', None],
  [ 0x9c59, 'CjkUnifiedIdeograph9c59', [], 'Display', None],
  [ 0x9c5a, 'CjkUnifiedIdeograph9c5a', [], 'Display', None],
  [ 0x9c5b, 'CjkUnifiedIdeograph9c5b', [], 'Display', None],
  [ 0x9c5c, 'CjkUnifiedIdeograph9c5c', [], 'Display', None],
  [ 0x9c5d, 'CjkUnifiedIdeograph9c5d', [], 'Display', None],
  [ 0x9c5e, 'CjkUnifiedIdeograph9c5e', [], 'Display', None],
  [ 0x9c5f, 'CjkUnifiedIdeograph9c5f', [], 'Display', None],
  [ 0x9c60, 'CjkUnifiedIdeograph9c60', [], 'Display', None],
  [ 0x9c61, 'CjkUnifiedIdeograph9c61', [], 'Display', None],
  [ 0x9c62, 'CjkUnifiedIdeograph9c62', [], 'Display', None],
  [ 0x9c63, 'CjkUnifiedIdeograph9c63', [], 'Display', None],
  [ 0x9c64, 'CjkUnifiedIdeograph9c64', [], 'Display', None],
  [ 0x9c65, 'CjkUnifiedIdeograph9c65', [], 'Display', None],
  [ 0x9c66, 'CjkUnifiedIdeograph9c66', [], 'Display', None],
  [ 0x9c67, 'CjkUnifiedIdeograph9c67', [], 'Display', None],
  [ 0x9c68, 'CjkUnifiedIdeograph9c68', [], 'Display', None],
  [ 0x9c69, 'CjkUnifiedIdeograph9c69', [], 'Display', None],
  [ 0x9c6a, 'CjkUnifiedIdeograph9c6a', [], 'Display', None],
  [ 0x9c6b, 'CjkUnifiedIdeograph9c6b', [], 'Display', None],
  [ 0x9c6c, 'CjkUnifiedIdeograph9c6c', [], 'Display', None],
  [ 0x9c6d, 'CjkUnifiedIdeograph9c6d', [], 'Display', None],
  [ 0x9c6e, 'CjkUnifiedIdeograph9c6e', [], 'Display', None],
  [ 0x9c6f, 'CjkUnifiedIdeograph9c6f', [], 'Display', None],
  [ 0x9c70, 'CjkUnifiedIdeograph9c70', [], 'Display', None],
  [ 0x9c71, 'CjkUnifiedIdeograph9c71', [], 'Display', None],
  [ 0x9c72, 'CjkUnifiedIdeograph9c72', [], 'Display', None],
  [ 0x9c73, 'CjkUnifiedIdeograph9c73', [], 'Display', None],
  [ 0x9c74, 'CjkUnifiedIdeograph9c74', [], 'Display', None],
  [ 0x9c75, 'CjkUnifiedIdeograph9c75', [], 'Display', None],
  [ 0x9c76, 'CjkUnifiedIdeograph9c76', [], 'Display', None],
  [ 0x9c77, 'CjkUnifiedIdeograph9c77', [], 'Display', None],
  [ 0x9c78, 'CjkUnifiedIdeograph9c78', [], 'Display', None],
  [ 0x9c79, 'CjkUnifiedIdeograph9c79', [], 'Display', None],
  [ 0x9c7a, 'CjkUnifiedIdeograph9c7a', [], 'Display', None],
  [ 0x9c7b, 'CjkUnifiedIdeograph9c7b', [], 'Display', None],
  [ 0x9c7c, 'CjkUnifiedIdeograph9c7c', [], 'Display', None],
  [ 0x9c7d, 'CjkUnifiedIdeograph9c7d', [], 'Display', None],
  [ 0x9c7e, 'CjkUnifiedIdeograph9c7e', [], 'Display', None],
  [ 0x9c7f, 'CjkUnifiedIdeograph9c7f', [], 'Display', None],
  [ 0x9c80, 'CjkUnifiedIdeograph9c80', [], 'Display', None],
  [ 0x9c81, 'CjkUnifiedIdeograph9c81', [], 'Display', None],
  [ 0x9c82, 'CjkUnifiedIdeograph9c82', [], 'Display', None],
  [ 0x9c83, 'CjkUnifiedIdeograph9c83', [], 'Display', None],
  [ 0x9c84, 'CjkUnifiedIdeograph9c84', [], 'Display', None],
  [ 0x9c85, 'CjkUnifiedIdeograph9c85', [], 'Display', None],
  [ 0x9c86, 'CjkUnifiedIdeograph9c86', [], 'Display', None],
  [ 0x9c87, 'CjkUnifiedIdeograph9c87', [], 'Display', None],
  [ 0x9c88, 'CjkUnifiedIdeograph9c88', [], 'Display', None],
  [ 0x9c89, 'CjkUnifiedIdeograph9c89', [], 'Display', None],
  [ 0x9c8a, 'CjkUnifiedIdeograph9c8a', [], 'Display', None],
  [ 0x9c8b, 'CjkUnifiedIdeograph9c8b', [], 'Display', None],
  [ 0x9c8c, 'CjkUnifiedIdeograph9c8c', [], 'Display', None],
  [ 0x9c8d, 'CjkUnifiedIdeograph9c8d', [], 'Display', None],
  [ 0x9c8e, 'CjkUnifiedIdeograph9c8e', [], 'Display', None],
  [ 0x9c8f, 'CjkUnifiedIdeograph9c8f', [], 'Display', None],
  [ 0x9c90, 'CjkUnifiedIdeograph9c90', [], 'Display', None],
  [ 0x9c91, 'CjkUnifiedIdeograph9c91', [], 'Display', None],
  [ 0x9c92, 'CjkUnifiedIdeograph9c92', [], 'Display', None],
  [ 0x9c93, 'CjkUnifiedIdeograph9c93', [], 'Display', None],
  [ 0x9c94, 'CjkUnifiedIdeograph9c94', [], 'Display', None],
  [ 0x9c95, 'CjkUnifiedIdeograph9c95', [], 'Display', None],
  [ 0x9c96, 'CjkUnifiedIdeograph9c96', [], 'Display', None],
  [ 0x9c97, 'CjkUnifiedIdeograph9c97', [], 'Display', None],
  [ 0x9c98, 'CjkUnifiedIdeograph9c98', [], 'Display', None],
  [ 0x9c99, 'CjkUnifiedIdeograph9c99', [], 'Display', None],
  [ 0x9c9a, 'CjkUnifiedIdeograph9c9a', [], 'Display', None],
  [ 0x9c9b, 'CjkUnifiedIdeograph9c9b', [], 'Display', None],
  [ 0x9c9c, 'CjkUnifiedIdeograph9c9c', [], 'Display', None],
  [ 0x9c9d, 'CjkUnifiedIdeograph9c9d', [], 'Display', None],
  [ 0x9c9e, 'CjkUnifiedIdeograph9c9e', [], 'Display', None],
  [ 0x9c9f, 'CjkUnifiedIdeograph9c9f', [], 'Display', None],
  [ 0x9ca0, 'CjkUnifiedIdeograph9ca0', [], 'Display', None],
  [ 0x9ca1, 'CjkUnifiedIdeograph9ca1', [], 'Display', None],
  [ 0x9ca2, 'CjkUnifiedIdeograph9ca2', [], 'Display', None],
  [ 0x9ca3, 'CjkUnifiedIdeograph9ca3', [], 'Display', None],
  [ 0x9ca4, 'CjkUnifiedIdeograph9ca4', [], 'Display', None],
  [ 0x9ca5, 'CjkUnifiedIdeograph9ca5', [], 'Display', None],
  [ 0x9ca6, 'CjkUnifiedIdeograph9ca6', [], 'Display', None],
  [ 0x9ca7, 'CjkUnifiedIdeograph9ca7', [], 'Display', None],
  [ 0x9ca8, 'CjkUnifiedIdeograph9ca8', [], 'Display', None],
  [ 0x9ca9, 'CjkUnifiedIdeograph9ca9', [], 'Display', None],
  [ 0x9caa, 'CjkUnifiedIdeograph9caa', [], 'Display', None],
  [ 0x9cab, 'CjkUnifiedIdeograph9cab', [], 'Display', None],
  [ 0x9cac, 'CjkUnifiedIdeograph9cac', [], 'Display', None],
  [ 0x9cad, 'CjkUnifiedIdeograph9cad', [], 'Display', None],
  [ 0x9cae, 'CjkUnifiedIdeograph9cae', [], 'Display', None],
  [ 0x9caf, 'CjkUnifiedIdeograph9caf', [], 'Display', None],
  [ 0x9cb0, 'CjkUnifiedIdeograph9cb0', [], 'Display', None],
  [ 0x9cb1, 'CjkUnifiedIdeograph9cb1', [], 'Display', None],
  [ 0x9cb2, 'CjkUnifiedIdeograph9cb2', [], 'Display', None],
  [ 0x9cb3, 'CjkUnifiedIdeograph9cb3', [], 'Display', None],
  [ 0x9cb4, 'CjkUnifiedIdeograph9cb4', [], 'Display', None],
  [ 0x9cb5, 'CjkUnifiedIdeograph9cb5', [], 'Display', None],
  [ 0x9cb6, 'CjkUnifiedIdeograph9cb6', [], 'Display', None],
  [ 0x9cb7, 'CjkUnifiedIdeograph9cb7', [], 'Display', None],
  [ 0x9cb8, 'CjkUnifiedIdeograph9cb8', [], 'Display', None],
  [ 0x9cb9, 'CjkUnifiedIdeograph9cb9', [], 'Display', None],
  [ 0x9cba, 'CjkUnifiedIdeograph9cba', [], 'Display', None],
  [ 0x9cbb, 'CjkUnifiedIdeograph9cbb', [], 'Display', None],
  [ 0x9cbc, 'CjkUnifiedIdeograph9cbc', [], 'Display', None],
  [ 0x9cbd, 'CjkUnifiedIdeograph9cbd', [], 'Display', None],
  [ 0x9cbe, 'CjkUnifiedIdeograph9cbe', [], 'Display', None],
  [ 0x9cbf, 'CjkUnifiedIdeograph9cbf', [], 'Display', None],
  [ 0x9cc0, 'CjkUnifiedIdeograph9cc0', [], 'Display', None],
  [ 0x9cc1, 'CjkUnifiedIdeograph9cc1', [], 'Display', None],
  [ 0x9cc2, 'CjkUnifiedIdeograph9cc2', [], 'Display', None],
  [ 0x9cc3, 'CjkUnifiedIdeograph9cc3', [], 'Display', None],
  [ 0x9cc4, 'CjkUnifiedIdeograph9cc4', [], 'Display', None],
  [ 0x9cc5, 'CjkUnifiedIdeograph9cc5', [], 'Display', None],
  [ 0x9cc6, 'CjkUnifiedIdeograph9cc6', [], 'Display', None],
  [ 0x9cc7, 'CjkUnifiedIdeograph9cc7', [], 'Display', None],
  [ 0x9cc8, 'CjkUnifiedIdeograph9cc8', [], 'Display', None],
  [ 0x9cc9, 'CjkUnifiedIdeograph9cc9', [], 'Display', None],
  [ 0x9cca, 'CjkUnifiedIdeograph9cca', [], 'Display', None],
  [ 0x9ccb, 'CjkUnifiedIdeograph9ccb', [], 'Display', None],
  [ 0x9ccc, 'CjkUnifiedIdeograph9ccc', [], 'Display', None],
  [ 0x9ccd, 'CjkUnifiedIdeograph9ccd', [], 'Display', None],
  [ 0x9cce, 'CjkUnifiedIdeograph9cce', [], 'Display', None],
  [ 0x9ccf, 'CjkUnifiedIdeograph9ccf', [], 'Display', None],
  [ 0x9cd0, 'CjkUnifiedIdeograph9cd0', [], 'Display', None],
  [ 0x9cd1, 'CjkUnifiedIdeograph9cd1', [], 'Display', None],
  [ 0x9cd2, 'CjkUnifiedIdeograph9cd2', [], 'Display', None],
  [ 0x9cd3, 'CjkUnifiedIdeograph9cd3', [], 'Display', None],
  [ 0x9cd4, 'CjkUnifiedIdeograph9cd4', [], 'Display', None],
  [ 0x9cd5, 'CjkUnifiedIdeograph9cd5', [], 'Display', None],
  [ 0x9cd6, 'CjkUnifiedIdeograph9cd6', [], 'Display', None],
  [ 0x9cd7, 'CjkUnifiedIdeograph9cd7', [], 'Display', None],
  [ 0x9cd8, 'CjkUnifiedIdeograph9cd8', [], 'Display', None],
  [ 0x9cd9, 'CjkUnifiedIdeograph9cd9', [], 'Display', None],
  [ 0x9cda, 'CjkUnifiedIdeograph9cda', [], 'Display', None],
  [ 0x9cdb, 'CjkUnifiedIdeograph9cdb', [], 'Display', None],
  [ 0x9cdc, 'CjkUnifiedIdeograph9cdc', [], 'Display', None],
  [ 0x9cdd, 'CjkUnifiedIdeograph9cdd', [], 'Display', None],
  [ 0x9cde, 'CjkUnifiedIdeograph9cde', [], 'Display', None],
  [ 0x9cdf, 'CjkUnifiedIdeograph9cdf', [], 'Display', None],
  [ 0x9ce0, 'CjkUnifiedIdeograph9ce0', [], 'Display', None],
  [ 0x9ce1, 'CjkUnifiedIdeograph9ce1', [], 'Display', None],
  [ 0x9ce2, 'CjkUnifiedIdeograph9ce2', [], 'Display', None],
  [ 0x9ce3, 'CjkUnifiedIdeograph9ce3', [], 'Display', None],
  [ 0x9ce4, 'CjkUnifiedIdeograph9ce4', [], 'Display', None],
  [ 0x9ce5, 'CjkUnifiedIdeograph9ce5', [], 'Display', None],
  [ 0x9ce6, 'CjkUnifiedIdeograph9ce6', [], 'Display', None],
  [ 0x9ce7, 'CjkUnifiedIdeograph9ce7', [], 'Display', None],
  [ 0x9ce8, 'CjkUnifiedIdeograph9ce8', [], 'Display', None],
  [ 0x9ce9, 'CjkUnifiedIdeograph9ce9', [], 'Display', None],
  [ 0x9cea, 'CjkUnifiedIdeograph9cea', [], 'Display', None],
  [ 0x9ceb, 'CjkUnifiedIdeograph9ceb', [], 'Display', None],
  [ 0x9cec, 'CjkUnifiedIdeograph9cec', [], 'Display', None],
  [ 0x9ced, 'CjkUnifiedIdeograph9ced', [], 'Display', None],
  [ 0x9cee, 'CjkUnifiedIdeograph9cee', [], 'Display', None],
  [ 0x9cef, 'CjkUnifiedIdeograph9cef', [], 'Display', None],
  [ 0x9cf0, 'CjkUnifiedIdeograph9cf0', [], 'Display', None],
  [ 0x9cf1, 'CjkUnifiedIdeograph9cf1', [], 'Display', None],
  [ 0x9cf2, 'CjkUnifiedIdeograph9cf2', [], 'Display', None],
  [ 0x9cf3, 'CjkUnifiedIdeograph9cf3', [], 'Display', None],
  [ 0x9cf4, 'CjkUnifiedIdeograph9cf4', [], 'Display', None],
  [ 0x9cf5, 'CjkUnifiedIdeograph9cf5', [], 'Display', None],
  [ 0x9cf6, 'CjkUnifiedIdeograph9cf6', [], 'Display', None],
  [ 0x9cf7, 'CjkUnifiedIdeograph9cf7', [], 'Display', None],
  [ 0x9cf8, 'CjkUnifiedIdeograph9cf8', [], 'Display', None],
  [ 0x9cf9, 'CjkUnifiedIdeograph9cf9', [], 'Display', None],
  [ 0x9cfa, 'CjkUnifiedIdeograph9cfa', [], 'Display', None],
  [ 0x9cfb, 'CjkUnifiedIdeograph9cfb', [], 'Display', None],
  [ 0x9cfc, 'CjkUnifiedIdeograph9cfc', [], 'Display', None],
  [ 0x9cfd, 'CjkUnifiedIdeograph9cfd', [], 'Display', None],
  [ 0x9cfe, 'CjkUnifiedIdeograph9cfe', [], 'Display', None],
  [ 0x9cff, 'CjkUnifiedIdeograph9cff', [], 'Display', None],
  [ 0x9d00, 'CjkUnifiedIdeograph9d00', [], 'Display', None],
  [ 0x9d01, 'CjkUnifiedIdeograph9d01', [], 'Display', None],
  [ 0x9d02, 'CjkUnifiedIdeograph9d02', [], 'Display', None],
  [ 0x9d03, 'CjkUnifiedIdeograph9d03', [], 'Display', None],
  [ 0x9d04, 'CjkUnifiedIdeograph9d04', [], 'Display', None],
  [ 0x9d05, 'CjkUnifiedIdeograph9d05', [], 'Display', None],
  [ 0x9d06, 'CjkUnifiedIdeograph9d06', [], 'Display', None],
  [ 0x9d07, 'CjkUnifiedIdeograph9d07', [], 'Display', None],
  [ 0x9d08, 'CjkUnifiedIdeograph9d08', [], 'Display', None],
  [ 0x9d09, 'CjkUnifiedIdeograph9d09', [], 'Display', None],
  [ 0x9d0a, 'CjkUnifiedIdeograph9d0a', [], 'Display', None],
  [ 0x9d0b, 'CjkUnifiedIdeograph9d0b', [], 'Display', None],
  [ 0x9d0c, 'CjkUnifiedIdeograph9d0c', [], 'Display', None],
  [ 0x9d0d, 'CjkUnifiedIdeograph9d0d', [], 'Display', None],
  [ 0x9d0e, 'CjkUnifiedIdeograph9d0e', [], 'Display', None],
  [ 0x9d0f, 'CjkUnifiedIdeograph9d0f', [], 'Display', None],
  [ 0x9d10, 'CjkUnifiedIdeograph9d10', [], 'Display', None],
  [ 0x9d11, 'CjkUnifiedIdeograph9d11', [], 'Display', None],
  [ 0x9d12, 'CjkUnifiedIdeograph9d12', [], 'Display', None],
  [ 0x9d13, 'CjkUnifiedIdeograph9d13', [], 'Display', None],
  [ 0x9d14, 'CjkUnifiedIdeograph9d14', [], 'Display', None],
  [ 0x9d15, 'CjkUnifiedIdeograph9d15', [], 'Display', None],
  [ 0x9d16, 'CjkUnifiedIdeograph9d16', [], 'Display', None],
  [ 0x9d17, 'CjkUnifiedIdeograph9d17', [], 'Display', None],
  [ 0x9d18, 'CjkUnifiedIdeograph9d18', [], 'Display', None],
  [ 0x9d19, 'CjkUnifiedIdeograph9d19', [], 'Display', None],
  [ 0x9d1a, 'CjkUnifiedIdeograph9d1a', [], 'Display', None],
  [ 0x9d1b, 'CjkUnifiedIdeograph9d1b', [], 'Display', None],
  [ 0x9d1c, 'CjkUnifiedIdeograph9d1c', [], 'Display', None],
  [ 0x9d1d, 'CjkUnifiedIdeograph9d1d', [], 'Display', None],
  [ 0x9d1e, 'CjkUnifiedIdeograph9d1e', [], 'Display', None],
  [ 0x9d1f, 'CjkUnifiedIdeograph9d1f', [], 'Display', None],
  [ 0x9d20, 'CjkUnifiedIdeograph9d20', [], 'Display', None],
  [ 0x9d21, 'CjkUnifiedIdeograph9d21', [], 'Display', None],
  [ 0x9d22, 'CjkUnifiedIdeograph9d22', [], 'Display', None],
  [ 0x9d23, 'CjkUnifiedIdeograph9d23', [], 'Display', None],
  [ 0x9d24, 'CjkUnifiedIdeograph9d24', [], 'Display', None],
  [ 0x9d25, 'CjkUnifiedIdeograph9d25', [], 'Display', None],
  [ 0x9d26, 'CjkUnifiedIdeograph9d26', [], 'Display', None],
  [ 0x9d27, 'CjkUnifiedIdeograph9d27', [], 'Display', None],
  [ 0x9d28, 'CjkUnifiedIdeograph9d28', [], 'Display', None],
  [ 0x9d29, 'CjkUnifiedIdeograph9d29', [], 'Display', None],
  [ 0x9d2a, 'CjkUnifiedIdeograph9d2a', [], 'Display', None],
  [ 0x9d2b, 'CjkUnifiedIdeograph9d2b', [], 'Display', None],
  [ 0x9d2c, 'CjkUnifiedIdeograph9d2c', [], 'Display', None],
  [ 0x9d2d, 'CjkUnifiedIdeograph9d2d', [], 'Display', None],
  [ 0x9d2e, 'CjkUnifiedIdeograph9d2e', [], 'Display', None],
  [ 0x9d2f, 'CjkUnifiedIdeograph9d2f', [], 'Display', None],
  [ 0x9d30, 'CjkUnifiedIdeograph9d30', [], 'Display', None],
  [ 0x9d31, 'CjkUnifiedIdeograph9d31', [], 'Display', None],
  [ 0x9d32, 'CjkUnifiedIdeograph9d32', [], 'Display', None],
  [ 0x9d33, 'CjkUnifiedIdeograph9d33', [], 'Display', None],
  [ 0x9d34, 'CjkUnifiedIdeograph9d34', [], 'Display', None],
  [ 0x9d35, 'CjkUnifiedIdeograph9d35', [], 'Display', None],
  [ 0x9d36, 'CjkUnifiedIdeograph9d36', [], 'Display', None],
  [ 0x9d37, 'CjkUnifiedIdeograph9d37', [], 'Display', None],
  [ 0x9d38, 'CjkUnifiedIdeograph9d38', [], 'Display', None],
  [ 0x9d39, 'CjkUnifiedIdeograph9d39', [], 'Display', None],
  [ 0x9d3a, 'CjkUnifiedIdeograph9d3a', [], 'Display', None],
  [ 0x9d3b, 'CjkUnifiedIdeograph9d3b', [], 'Display', None],
  [ 0x9d3c, 'CjkUnifiedIdeograph9d3c', [], 'Display', None],
  [ 0x9d3d, 'CjkUnifiedIdeograph9d3d', [], 'Display', None],
  [ 0x9d3e, 'CjkUnifiedIdeograph9d3e', [], 'Display', None],
  [ 0x9d3f, 'CjkUnifiedIdeograph9d3f', [], 'Display', None],
  [ 0x9d40, 'CjkUnifiedIdeograph9d40', [], 'Display', None],
  [ 0x9d41, 'CjkUnifiedIdeograph9d41', [], 'Display', None],
  [ 0x9d42, 'CjkUnifiedIdeograph9d42', [], 'Display', None],
  [ 0x9d43, 'CjkUnifiedIdeograph9d43', [], 'Display', None],
  [ 0x9d44, 'CjkUnifiedIdeograph9d44', [], 'Display', None],
  [ 0x9d45, 'CjkUnifiedIdeograph9d45', [], 'Display', None],
  [ 0x9d46, 'CjkUnifiedIdeograph9d46', [], 'Display', None],
  [ 0x9d47, 'CjkUnifiedIdeograph9d47', [], 'Display', None],
  [ 0x9d48, 'CjkUnifiedIdeograph9d48', [], 'Display', None],
  [ 0x9d49, 'CjkUnifiedIdeograph9d49', [], 'Display', None],
  [ 0x9d4a, 'CjkUnifiedIdeograph9d4a', [], 'Display', None],
  [ 0x9d4b, 'CjkUnifiedIdeograph9d4b', [], 'Display', None],
  [ 0x9d4c, 'CjkUnifiedIdeograph9d4c', [], 'Display', None],
  [ 0x9d4d, 'CjkUnifiedIdeograph9d4d', [], 'Display', None],
  [ 0x9d4e, 'CjkUnifiedIdeograph9d4e', [], 'Display', None],
  [ 0x9d4f, 'CjkUnifiedIdeograph9d4f', [], 'Display', None],
  [ 0x9d50, 'CjkUnifiedIdeograph9d50', [], 'Display', None],
  [ 0x9d51, 'CjkUnifiedIdeograph9d51', [], 'Display', None],
  [ 0x9d52, 'CjkUnifiedIdeograph9d52', [], 'Display', None],
  [ 0x9d53, 'CjkUnifiedIdeograph9d53', [], 'Display', None],
  [ 0x9d54, 'CjkUnifiedIdeograph9d54', [], 'Display', None],
  [ 0x9d55, 'CjkUnifiedIdeograph9d55', [], 'Display', None],
  [ 0x9d56, 'CjkUnifiedIdeograph9d56', [], 'Display', None],
  [ 0x9d57, 'CjkUnifiedIdeograph9d57', [], 'Display', None],
  [ 0x9d58, 'CjkUnifiedIdeograph9d58', [], 'Display', None],
  [ 0x9d59, 'CjkUnifiedIdeograph9d59', [], 'Display', None],
  [ 0x9d5a, 'CjkUnifiedIdeograph9d5a', [], 'Display', None],
  [ 0x9d5b, 'CjkUnifiedIdeograph9d5b', [], 'Display', None],
  [ 0x9d5c, 'CjkUnifiedIdeograph9d5c', [], 'Display', None],
  [ 0x9d5d, 'CjkUnifiedIdeograph9d5d', [], 'Display', None],
  [ 0x9d5e, 'CjkUnifiedIdeograph9d5e', [], 'Display', None],
  [ 0x9d5f, 'CjkUnifiedIdeograph9d5f', [], 'Display', None],
  [ 0x9d60, 'CjkUnifiedIdeograph9d60', [], 'Display', None],
  [ 0x9d61, 'CjkUnifiedIdeograph9d61', [], 'Display', None],
  [ 0x9d62, 'CjkUnifiedIdeograph9d62', [], 'Display', None],
  [ 0x9d63, 'CjkUnifiedIdeograph9d63', [], 'Display', None],
  [ 0x9d64, 'CjkUnifiedIdeograph9d64', [], 'Display', None],
  [ 0x9d65, 'CjkUnifiedIdeograph9d65', [], 'Display', None],
  [ 0x9d66, 'CjkUnifiedIdeograph9d66', [], 'Display', None],
  [ 0x9d67, 'CjkUnifiedIdeograph9d67', [], 'Display', None],
  [ 0x9d68, 'CjkUnifiedIdeograph9d68', [], 'Display', None],
  [ 0x9d69, 'CjkUnifiedIdeograph9d69', [], 'Display', None],
  [ 0x9d6a, 'CjkUnifiedIdeograph9d6a', [], 'Display', None],
  [ 0x9d6b, 'CjkUnifiedIdeograph9d6b', [], 'Display', None],
  [ 0x9d6c, 'CjkUnifiedIdeograph9d6c', [], 'Display', None],
  [ 0x9d6d, 'CjkUnifiedIdeograph9d6d', [], 'Display', None],
  [ 0x9d6e, 'CjkUnifiedIdeograph9d6e', [], 'Display', None],
  [ 0x9d6f, 'CjkUnifiedIdeograph9d6f', [], 'Display', None],
  [ 0x9d70, 'CjkUnifiedIdeograph9d70', [], 'Display', None],
  [ 0x9d71, 'CjkUnifiedIdeograph9d71', [], 'Display', None],
  [ 0x9d72, 'CjkUnifiedIdeograph9d72', [], 'Display', None],
  [ 0x9d73, 'CjkUnifiedIdeograph9d73', [], 'Display', None],
  [ 0x9d74, 'CjkUnifiedIdeograph9d74', [], 'Display', None],
  [ 0x9d75, 'CjkUnifiedIdeograph9d75', [], 'Display', None],
  [ 0x9d76, 'CjkUnifiedIdeograph9d76', [], 'Display', None],
  [ 0x9d77, 'CjkUnifiedIdeograph9d77', [], 'Display', None],
  [ 0x9d78, 'CjkUnifiedIdeograph9d78', [], 'Display', None],
  [ 0x9d79, 'CjkUnifiedIdeograph9d79', [], 'Display', None],
  [ 0x9d7a, 'CjkUnifiedIdeograph9d7a', [], 'Display', None],
  [ 0x9d7b, 'CjkUnifiedIdeograph9d7b', [], 'Display', None],
  [ 0x9d7c, 'CjkUnifiedIdeograph9d7c', [], 'Display', None],
  [ 0x9d7d, 'CjkUnifiedIdeograph9d7d', [], 'Display', None],
  [ 0x9d7e, 'CjkUnifiedIdeograph9d7e', [], 'Display', None],
  [ 0x9d7f, 'CjkUnifiedIdeograph9d7f', [], 'Display', None],
  [ 0x9d80, 'CjkUnifiedIdeograph9d80', [], 'Display', None],
  [ 0x9d81, 'CjkUnifiedIdeograph9d81', [], 'Display', None],
  [ 0x9d82, 'CjkUnifiedIdeograph9d82', [], 'Display', None],
  [ 0x9d83, 'CjkUnifiedIdeograph9d83', [], 'Display', None],
  [ 0x9d84, 'CjkUnifiedIdeograph9d84', [], 'Display', None],
  [ 0x9d85, 'CjkUnifiedIdeograph9d85', [], 'Display', None],
  [ 0x9d86, 'CjkUnifiedIdeograph9d86', [], 'Display', None],
  [ 0x9d87, 'CjkUnifiedIdeograph9d87', [], 'Display', None],
  [ 0x9d88, 'CjkUnifiedIdeograph9d88', [], 'Display', None],
  [ 0x9d89, 'CjkUnifiedIdeograph9d89', [], 'Display', None],
  [ 0x9d8a, 'CjkUnifiedIdeograph9d8a', [], 'Display', None],
  [ 0x9d8b, 'CjkUnifiedIdeograph9d8b', [], 'Display', None],
  [ 0x9d8c, 'CjkUnifiedIdeograph9d8c', [], 'Display', None],
  [ 0x9d8d, 'CjkUnifiedIdeograph9d8d', [], 'Display', None],
  [ 0x9d8e, 'CjkUnifiedIdeograph9d8e', [], 'Display', None],
  [ 0x9d8f, 'CjkUnifiedIdeograph9d8f', [], 'Display', None],
  [ 0x9d90, 'CjkUnifiedIdeograph9d90', [], 'Display', None],
  [ 0x9d91, 'CjkUnifiedIdeograph9d91', [], 'Display', None],
  [ 0x9d92, 'CjkUnifiedIdeograph9d92', [], 'Display', None],
  [ 0x9d93, 'CjkUnifiedIdeograph9d93', [], 'Display', None],
  [ 0x9d94, 'CjkUnifiedIdeograph9d94', [], 'Display', None],
  [ 0x9d95, 'CjkUnifiedIdeograph9d95', [], 'Display', None],
  [ 0x9d96, 'CjkUnifiedIdeograph9d96', [], 'Display', None],
  [ 0x9d97, 'CjkUnifiedIdeograph9d97', [], 'Display', None],
  [ 0x9d98, 'CjkUnifiedIdeograph9d98', [], 'Display', None],
  [ 0x9d99, 'CjkUnifiedIdeograph9d99', [], 'Display', None],
  [ 0x9d9a, 'CjkUnifiedIdeograph9d9a', [], 'Display', None],
  [ 0x9d9b, 'CjkUnifiedIdeograph9d9b', [], 'Display', None],
  [ 0x9d9c, 'CjkUnifiedIdeograph9d9c', [], 'Display', None],
  [ 0x9d9d, 'CjkUnifiedIdeograph9d9d', [], 'Display', None],
  [ 0x9d9e, 'CjkUnifiedIdeograph9d9e', [], 'Display', None],
  [ 0x9d9f, 'CjkUnifiedIdeograph9d9f', [], 'Display', None],
  [ 0x9da0, 'CjkUnifiedIdeograph9da0', [], 'Display', None],
  [ 0x9da1, 'CjkUnifiedIdeograph9da1', [], 'Display', None],
  [ 0x9da2, 'CjkUnifiedIdeograph9da2', [], 'Display', None],
  [ 0x9da3, 'CjkUnifiedIdeograph9da3', [], 'Display', None],
  [ 0x9da4, 'CjkUnifiedIdeograph9da4', [], 'Display', None],
  [ 0x9da5, 'CjkUnifiedIdeograph9da5', [], 'Display', None],
  [ 0x9da6, 'CjkUnifiedIdeograph9da6', [], 'Display', None],
  [ 0x9da7, 'CjkUnifiedIdeograph9da7', [], 'Display', None],
  [ 0x9da8, 'CjkUnifiedIdeograph9da8', [], 'Display', None],
  [ 0x9da9, 'CjkUnifiedIdeograph9da9', [], 'Display', None],
  [ 0x9daa, 'CjkUnifiedIdeograph9daa', [], 'Display', None],
  [ 0x9dab, 'CjkUnifiedIdeograph9dab', [], 'Display', None],
  [ 0x9dac, 'CjkUnifiedIdeograph9dac', [], 'Display', None],
  [ 0x9dad, 'CjkUnifiedIdeograph9dad', [], 'Display', None],
  [ 0x9dae, 'CjkUnifiedIdeograph9dae', [], 'Display', None],
  [ 0x9daf, 'CjkUnifiedIdeograph9daf', [], 'Display', None],
  [ 0x9db0, 'CjkUnifiedIdeograph9db0', [], 'Display', None],
  [ 0x9db1, 'CjkUnifiedIdeograph9db1', [], 'Display', None],
  [ 0x9db2, 'CjkUnifiedIdeograph9db2', [], 'Display', None],
  [ 0x9db3, 'CjkUnifiedIdeograph9db3', [], 'Display', None],
  [ 0x9db4, 'CjkUnifiedIdeograph9db4', [], 'Display', None],
  [ 0x9db5, 'CjkUnifiedIdeograph9db5', [], 'Display', None],
  [ 0x9db6, 'CjkUnifiedIdeograph9db6', [], 'Display', None],
  [ 0x9db7, 'CjkUnifiedIdeograph9db7', [], 'Display', None],
  [ 0x9db8, 'CjkUnifiedIdeograph9db8', [], 'Display', None],
  [ 0x9db9, 'CjkUnifiedIdeograph9db9', [], 'Display', None],
  [ 0x9dba, 'CjkUnifiedIdeograph9dba', [], 'Display', None],
  [ 0x9dbb, 'CjkUnifiedIdeograph9dbb', [], 'Display', None],
  [ 0x9dbc, 'CjkUnifiedIdeograph9dbc', [], 'Display', None],
  [ 0x9dbd, 'CjkUnifiedIdeograph9dbd', [], 'Display', None],
  [ 0x9dbe, 'CjkUnifiedIdeograph9dbe', [], 'Display', None],
  [ 0x9dbf, 'CjkUnifiedIdeograph9dbf', [], 'Display', None],
  [ 0x9dc0, 'CjkUnifiedIdeograph9dc0', [], 'Display', None],
  [ 0x9dc1, 'CjkUnifiedIdeograph9dc1', [], 'Display', None],
  [ 0x9dc2, 'CjkUnifiedIdeograph9dc2', [], 'Display', None],
  [ 0x9dc3, 'CjkUnifiedIdeograph9dc3', [], 'Display', None],
  [ 0x9dc4, 'CjkUnifiedIdeograph9dc4', [], 'Display', None],
  [ 0x9dc5, 'CjkUnifiedIdeograph9dc5', [], 'Display', None],
  [ 0x9dc6, 'CjkUnifiedIdeograph9dc6', [], 'Display', None],
  [ 0x9dc7, 'CjkUnifiedIdeograph9dc7', [], 'Display', None],
  [ 0x9dc8, 'CjkUnifiedIdeograph9dc8', [], 'Display', None],
  [ 0x9dc9, 'CjkUnifiedIdeograph9dc9', [], 'Display', None],
  [ 0x9dca, 'CjkUnifiedIdeograph9dca', [], 'Display', None],
  [ 0x9dcb, 'CjkUnifiedIdeograph9dcb', [], 'Display', None],
  [ 0x9dcc, 'CjkUnifiedIdeograph9dcc', [], 'Display', None],
  [ 0x9dcd, 'CjkUnifiedIdeograph9dcd', [], 'Display', None],
  [ 0x9dce, 'CjkUnifiedIdeograph9dce', [], 'Display', None],
  [ 0x9dcf, 'CjkUnifiedIdeograph9dcf', [], 'Display', None],
  [ 0x9dd0, 'CjkUnifiedIdeograph9dd0', [], 'Display', None],
  [ 0x9dd1, 'CjkUnifiedIdeograph9dd1', [], 'Display', None],
  [ 0x9dd2, 'CjkUnifiedIdeograph9dd2', [], 'Display', None],
  [ 0x9dd3, 'CjkUnifiedIdeograph9dd3', [], 'Display', None],
  [ 0x9dd4, 'CjkUnifiedIdeograph9dd4', [], 'Display', None],
  [ 0x9dd5, 'CjkUnifiedIdeograph9dd5', [], 'Display', None],
  [ 0x9dd6, 'CjkUnifiedIdeograph9dd6', [], 'Display', None],
  [ 0x9dd7, 'CjkUnifiedIdeograph9dd7', [], 'Display', None],
  [ 0x9dd8, 'CjkUnifiedIdeograph9dd8', [], 'Display', None],
  [ 0x9dd9, 'CjkUnifiedIdeograph9dd9', [], 'Display', None],
  [ 0x9dda, 'CjkUnifiedIdeograph9dda', [], 'Display', None],
  [ 0x9ddb, 'CjkUnifiedIdeograph9ddb', [], 'Display', None],
  [ 0x9ddc, 'CjkUnifiedIdeograph9ddc', [], 'Display', None],
  [ 0x9ddd, 'CjkUnifiedIdeograph9ddd', [], 'Display', None],
  [ 0x9dde, 'CjkUnifiedIdeograph9dde', [], 'Display', None],
  [ 0x9ddf, 'CjkUnifiedIdeograph9ddf', [], 'Display', None],
  [ 0x9de0, 'CjkUnifiedIdeograph9de0', [], 'Display', None],
  [ 0x9de1, 'CjkUnifiedIdeograph9de1', [], 'Display', None],
  [ 0x9de2, 'CjkUnifiedIdeograph9de2', [], 'Display', None],
  [ 0x9de3, 'CjkUnifiedIdeograph9de3', [], 'Display', None],
  [ 0x9de4, 'CjkUnifiedIdeograph9de4', [], 'Display', None],
  [ 0x9de5, 'CjkUnifiedIdeograph9de5', [], 'Display', None],
  [ 0x9de6, 'CjkUnifiedIdeograph9de6', [], 'Display', None],
  [ 0x9de7, 'CjkUnifiedIdeograph9de7', [], 'Display', None],
  [ 0x9de8, 'CjkUnifiedIdeograph9de8', [], 'Display', None],
  [ 0x9de9, 'CjkUnifiedIdeograph9de9', [], 'Display', None],
  [ 0x9dea, 'CjkUnifiedIdeograph9dea', [], 'Display', None],
  [ 0x9deb, 'CjkUnifiedIdeograph9deb', [], 'Display', None],
  [ 0x9dec, 'CjkUnifiedIdeograph9dec', [], 'Display', None],
  [ 0x9ded, 'CjkUnifiedIdeograph9ded', [], 'Display', None],
  [ 0x9dee, 'CjkUnifiedIdeograph9dee', [], 'Display', None],
  [ 0x9def, 'CjkUnifiedIdeograph9def', [], 'Display', None],
  [ 0x9df0, 'CjkUnifiedIdeograph9df0', [], 'Display', None],
  [ 0x9df1, 'CjkUnifiedIdeograph9df1', [], 'Display', None],
  [ 0x9df2, 'CjkUnifiedIdeograph9df2', [], 'Display', None],
  [ 0x9df3, 'CjkUnifiedIdeograph9df3', [], 'Display', None],
  [ 0x9df4, 'CjkUnifiedIdeograph9df4', [], 'Display', None],
  [ 0x9df5, 'CjkUnifiedIdeograph9df5', [], 'Display', None],
  [ 0x9df6, 'CjkUnifiedIdeograph9df6', [], 'Display', None],
  [ 0x9df7, 'CjkUnifiedIdeograph9df7', [], 'Display', None],
  [ 0x9df8, 'CjkUnifiedIdeograph9df8', [], 'Display', None],
  [ 0x9df9, 'CjkUnifiedIdeograph9df9', [], 'Display', None],
  [ 0x9dfa, 'CjkUnifiedIdeograph9dfa', [], 'Display', None],
  [ 0x9dfb, 'CjkUnifiedIdeograph9dfb', [], 'Display', None],
  [ 0x9dfc, 'CjkUnifiedIdeograph9dfc', [], 'Display', None],
  [ 0x9dfd, 'CjkUnifiedIdeograph9dfd', [], 'Display', None],
  [ 0x9dfe, 'CjkUnifiedIdeograph9dfe', [], 'Display', None],
  [ 0x9dff, 'CjkUnifiedIdeograph9dff', [], 'Display', None],
  [ 0x9e00, 'CjkUnifiedIdeograph9e00', [], 'Display', None],
  [ 0x9e01, 'CjkUnifiedIdeograph9e01', [], 'Display', None],
  [ 0x9e02, 'CjkUnifiedIdeograph9e02', [], 'Display', None],
  [ 0x9e03, 'CjkUnifiedIdeograph9e03', [], 'Display', None],
  [ 0x9e04, 'CjkUnifiedIdeograph9e04', [], 'Display', None],
  [ 0x9e05, 'CjkUnifiedIdeograph9e05', [], 'Display', None],
  [ 0x9e06, 'CjkUnifiedIdeograph9e06', [], 'Display', None],
  [ 0x9e07, 'CjkUnifiedIdeograph9e07', [], 'Display', None],
  [ 0x9e08, 'CjkUnifiedIdeograph9e08', [], 'Display', None],
  [ 0x9e09, 'CjkUnifiedIdeograph9e09', [], 'Display', None],
  [ 0x9e0a, 'CjkUnifiedIdeograph9e0a', [], 'Display', None],
  [ 0x9e0b, 'CjkUnifiedIdeograph9e0b', [], 'Display', None],
  [ 0x9e0c, 'CjkUnifiedIdeograph9e0c', [], 'Display', None],
  [ 0x9e0d, 'CjkUnifiedIdeograph9e0d', [], 'Display', None],
  [ 0x9e0e, 'CjkUnifiedIdeograph9e0e', [], 'Display', None],
  [ 0x9e0f, 'CjkUnifiedIdeograph9e0f', [], 'Display', None],
  [ 0x9e10, 'CjkUnifiedIdeograph9e10', [], 'Display', None],
  [ 0x9e11, 'CjkUnifiedIdeograph9e11', [], 'Display', None],
  [ 0x9e12, 'CjkUnifiedIdeograph9e12', [], 'Display', None],
  [ 0x9e13, 'CjkUnifiedIdeograph9e13', [], 'Display', None],
  [ 0x9e14, 'CjkUnifiedIdeograph9e14', [], 'Display', None],
  [ 0x9e15, 'CjkUnifiedIdeograph9e15', [], 'Display', None],
  [ 0x9e16, 'CjkUnifiedIdeograph9e16', [], 'Display', None],
  [ 0x9e17, 'CjkUnifiedIdeograph9e17', [], 'Display', None],
  [ 0x9e18, 'CjkUnifiedIdeograph9e18', [], 'Display', None],
  [ 0x9e19, 'CjkUnifiedIdeograph9e19', [], 'Display', None],
  [ 0x9e1a, 'CjkUnifiedIdeograph9e1a', [], 'Display', None],
  [ 0x9e1b, 'CjkUnifiedIdeograph9e1b', [], 'Display', None],
  [ 0x9e1c, 'CjkUnifiedIdeograph9e1c', [], 'Display', None],
  [ 0x9e1d, 'CjkUnifiedIdeograph9e1d', [], 'Display', None],
  [ 0x9e1e, 'CjkUnifiedIdeograph9e1e', [], 'Display', None],
  [ 0x9e1f, 'CjkUnifiedIdeograph9e1f', [], 'Display', None],
  [ 0x9e20, 'CjkUnifiedIdeograph9e20', [], 'Display', None],
  [ 0x9e21, 'CjkUnifiedIdeograph9e21', [], 'Display', None],
  [ 0x9e22, 'CjkUnifiedIdeograph9e22', [], 'Display', None],
  [ 0x9e23, 'CjkUnifiedIdeograph9e23', [], 'Display', None],
  [ 0x9e24, 'CjkUnifiedIdeograph9e24', [], 'Display', None],
  [ 0x9e25, 'CjkUnifiedIdeograph9e25', [], 'Display', None],
  [ 0x9e26, 'CjkUnifiedIdeograph9e26', [], 'Display', None],
  [ 0x9e27, 'CjkUnifiedIdeograph9e27', [], 'Display', None],
  [ 0x9e28, 'CjkUnifiedIdeograph9e28', [], 'Display', None],
  [ 0x9e29, 'CjkUnifiedIdeograph9e29', [], 'Display', None],
  [ 0x9e2a, 'CjkUnifiedIdeograph9e2a', [], 'Display', None],
  [ 0x9e2b, 'CjkUnifiedIdeograph9e2b', [], 'Display', None],
  [ 0x9e2c, 'CjkUnifiedIdeograph9e2c', [], 'Display', None],
  [ 0x9e2d, 'CjkUnifiedIdeograph9e2d', [], 'Display', None],
  [ 0x9e2e, 'CjkUnifiedIdeograph9e2e', [], 'Display', None],
  [ 0x9e2f, 'CjkUnifiedIdeograph9e2f', [], 'Display', None],
  [ 0x9e30, 'CjkUnifiedIdeograph9e30', [], 'Display', None],
  [ 0x9e31, 'CjkUnifiedIdeograph9e31', [], 'Display', None],
  [ 0x9e32, 'CjkUnifiedIdeograph9e32', [], 'Display', None],
  [ 0x9e33, 'CjkUnifiedIdeograph9e33', [], 'Display', None],
  [ 0x9e34, 'CjkUnifiedIdeograph9e34', [], 'Display', None],
  [ 0x9e35, 'CjkUnifiedIdeograph9e35', [], 'Display', None],
  [ 0x9e36, 'CjkUnifiedIdeograph9e36', [], 'Display', None],
  [ 0x9e37, 'CjkUnifiedIdeograph9e37', [], 'Display', None],
  [ 0x9e38, 'CjkUnifiedIdeograph9e38', [], 'Display', None],
  [ 0x9e39, 'CjkUnifiedIdeograph9e39', [], 'Display', None],
  [ 0x9e3a, 'CjkUnifiedIdeograph9e3a', [], 'Display', None],
  [ 0x9e3b, 'CjkUnifiedIdeograph9e3b', [], 'Display', None],
  [ 0x9e3c, 'CjkUnifiedIdeograph9e3c', [], 'Display', None],
  [ 0x9e3d, 'CjkUnifiedIdeograph9e3d', [], 'Display', None],
  [ 0x9e3e, 'CjkUnifiedIdeograph9e3e', [], 'Display', None],
  [ 0x9e3f, 'CjkUnifiedIdeograph9e3f', [], 'Display', None],
  [ 0x9e40, 'CjkUnifiedIdeograph9e40', [], 'Display', None],
  [ 0x9e41, 'CjkUnifiedIdeograph9e41', [], 'Display', None],
  [ 0x9e42, 'CjkUnifiedIdeograph9e42', [], 'Display', None],
  [ 0x9e43, 'CjkUnifiedIdeograph9e43', [], 'Display', None],
  [ 0x9e44, 'CjkUnifiedIdeograph9e44', [], 'Display', None],
  [ 0x9e45, 'CjkUnifiedIdeograph9e45', [], 'Display', None],
  [ 0x9e46, 'CjkUnifiedIdeograph9e46', [], 'Display', None],
  [ 0x9e47, 'CjkUnifiedIdeograph9e47', [], 'Display', None],
  [ 0x9e48, 'CjkUnifiedIdeograph9e48', [], 'Display', None],
  [ 0x9e49, 'CjkUnifiedIdeograph9e49', [], 'Display', None],
  [ 0x9e4a, 'CjkUnifiedIdeograph9e4a', [], 'Display', None],
  [ 0x9e4b, 'CjkUnifiedIdeograph9e4b', [], 'Display', None],
  [ 0x9e4c, 'CjkUnifiedIdeograph9e4c', [], 'Display', None],
  [ 0x9e4d, 'CjkUnifiedIdeograph9e4d', [], 'Display', None],
  [ 0x9e4e, 'CjkUnifiedIdeograph9e4e', [], 'Display', None],
  [ 0x9e4f, 'CjkUnifiedIdeograph9e4f', [], 'Display', None],
  [ 0x9e50, 'CjkUnifiedIdeograph9e50', [], 'Display', None],
  [ 0x9e51, 'CjkUnifiedIdeograph9e51', [], 'Display', None],
  [ 0x9e52, 'CjkUnifiedIdeograph9e52', [], 'Display', None],
  [ 0x9e53, 'CjkUnifiedIdeograph9e53', [], 'Display', None],
  [ 0x9e54, 'CjkUnifiedIdeograph9e54', [], 'Display', None],
  [ 0x9e55, 'CjkUnifiedIdeograph9e55', [], 'Display', None],
  [ 0x9e56, 'CjkUnifiedIdeograph9e56', [], 'Display', None],
  [ 0x9e57, 'CjkUnifiedIdeograph9e57', [], 'Display', None],
  [ 0x9e58, 'CjkUnifiedIdeograph9e58', [], 'Display', None],
  [ 0x9e59, 'CjkUnifiedIdeograph9e59', [], 'Display', None],
  [ 0x9e5a, 'CjkUnifiedIdeograph9e5a', [], 'Display', None],
  [ 0x9e5b, 'CjkUnifiedIdeograph9e5b', [], 'Display', None],
  [ 0x9e5c, 'CjkUnifiedIdeograph9e5c', [], 'Display', None],
  [ 0x9e5d, 'CjkUnifiedIdeograph9e5d', [], 'Display', None],
  [ 0x9e5e, 'CjkUnifiedIdeograph9e5e', [], 'Display', None],
  [ 0x9e5f, 'CjkUnifiedIdeograph9e5f', [], 'Display', None],
  [ 0x9e60, 'CjkUnifiedIdeograph9e60', [], 'Display', None],
  [ 0x9e61, 'CjkUnifiedIdeograph9e61', [], 'Display', None],
  [ 0x9e62, 'CjkUnifiedIdeograph9e62', [], 'Display', None],
  [ 0x9e63, 'CjkUnifiedIdeograph9e63', [], 'Display', None],
  [ 0x9e64, 'CjkUnifiedIdeograph9e64', [], 'Display', None],
  [ 0x9e65, 'CjkUnifiedIdeograph9e65', [], 'Display', None],
  [ 0x9e66, 'CjkUnifiedIdeograph9e66', [], 'Display', None],
  [ 0x9e67, 'CjkUnifiedIdeograph9e67', [], 'Display', None],
  [ 0x9e68, 'CjkUnifiedIdeograph9e68', [], 'Display', None],
  [ 0x9e69, 'CjkUnifiedIdeograph9e69', [], 'Display', None],
  [ 0x9e6a, 'CjkUnifiedIdeograph9e6a', [], 'Display', None],
  [ 0x9e6b, 'CjkUnifiedIdeograph9e6b', [], 'Display', None],
  [ 0x9e6c, 'CjkUnifiedIdeograph9e6c', [], 'Display', None],
  [ 0x9e6d, 'CjkUnifiedIdeograph9e6d', [], 'Display', None],
  [ 0x9e6e, 'CjkUnifiedIdeograph9e6e', [], 'Display', None],
  [ 0x9e6f, 'CjkUnifiedIdeograph9e6f', [], 'Display', None],
  [ 0x9e70, 'CjkUnifiedIdeograph9e70', [], 'Display', None],
  [ 0x9e71, 'CjkUnifiedIdeograph9e71', [], 'Display', None],
  [ 0x9e72, 'CjkUnifiedIdeograph9e72', [], 'Display', None],
  [ 0x9e73, 'CjkUnifiedIdeograph9e73', [], 'Display', None],
  [ 0x9e74, 'CjkUnifiedIdeograph9e74', [], 'Display', None],
  [ 0x9e75, 'CjkUnifiedIdeograph9e75', [], 'Display', None],
  [ 0x9e76, 'CjkUnifiedIdeograph9e76', [], 'Display', None],
  [ 0x9e77, 'CjkUnifiedIdeograph9e77', [], 'Display', None],
  [ 0x9e78, 'CjkUnifiedIdeograph9e78', [], 'Display', None],
  [ 0x9e79, 'CjkUnifiedIdeograph9e79', [], 'Display', None],
  [ 0x9e7a, 'CjkUnifiedIdeograph9e7a', [], 'Display', None],
  [ 0x9e7b, 'CjkUnifiedIdeograph9e7b', [], 'Display', None],
  [ 0x9e7c, 'CjkUnifiedIdeograph9e7c', [], 'Display', None],
  [ 0x9e7d, 'CjkUnifiedIdeograph9e7d', [], 'Display', None],
  [ 0x9e7e, 'CjkUnifiedIdeograph9e7e', [], 'Display', None],
  [ 0x9e7f, 'CjkUnifiedIdeograph9e7f', [], 'Display', None],
  [ 0x9e80, 'CjkUnifiedIdeograph9e80', [], 'Display', None],
  [ 0x9e81, 'CjkUnifiedIdeograph9e81', [], 'Display', None],
  [ 0x9e82, 'CjkUnifiedIdeograph9e82', [], 'Display', None],
  [ 0x9e83, 'CjkUnifiedIdeograph9e83', [], 'Display', None],
  [ 0x9e84, 'CjkUnifiedIdeograph9e84', [], 'Display', None],
  [ 0x9e85, 'CjkUnifiedIdeograph9e85', [], 'Display', None],
  [ 0x9e86, 'CjkUnifiedIdeograph9e86', [], 'Display', None],
  [ 0x9e87, 'CjkUnifiedIdeograph9e87', [], 'Display', None],
  [ 0x9e88, 'CjkUnifiedIdeograph9e88', [], 'Display', None],
  [ 0x9e89, 'CjkUnifiedIdeograph9e89', [], 'Display', None],
  [ 0x9e8a, 'CjkUnifiedIdeograph9e8a', [], 'Display', None],
  [ 0x9e8b, 'CjkUnifiedIdeograph9e8b', [], 'Display', None],
  [ 0x9e8c, 'CjkUnifiedIdeograph9e8c', [], 'Display', None],
  [ 0x9e8d, 'CjkUnifiedIdeograph9e8d', [], 'Display', None],
  [ 0x9e8e, 'CjkUnifiedIdeograph9e8e', [], 'Display', None],
  [ 0x9e8f, 'CjkUnifiedIdeograph9e8f', [], 'Display', None],
  [ 0x9e90, 'CjkUnifiedIdeograph9e90', [], 'Display', None],
  [ 0x9e91, 'CjkUnifiedIdeograph9e91', [], 'Display', None],
  [ 0x9e92, 'CjkUnifiedIdeograph9e92', [], 'Display', None],
  [ 0x9e93, 'CjkUnifiedIdeograph9e93', [], 'Display', None],
  [ 0x9e94, 'CjkUnifiedIdeograph9e94', [], 'Display', None],
  [ 0x9e95, 'CjkUnifiedIdeograph9e95', [], 'Display', None],
  [ 0x9e96, 'CjkUnifiedIdeograph9e96', [], 'Display', None],
  [ 0x9e97, 'CjkUnifiedIdeograph9e97', [], 'Display', None],
  [ 0x9e98, 'CjkUnifiedIdeograph9e98', [], 'Display', None],
  [ 0x9e99, 'CjkUnifiedIdeograph9e99', [], 'Display', None],
  [ 0x9e9a, 'CjkUnifiedIdeograph9e9a', [], 'Display', None],
  [ 0x9e9b, 'CjkUnifiedIdeograph9e9b', [], 'Display', None],
  [ 0x9e9c, 'CjkUnifiedIdeograph9e9c', [], 'Display', None],
  [ 0x9e9d, 'CjkUnifiedIdeograph9e9d', [], 'Display', None],
  [ 0x9e9e, 'CjkUnifiedIdeograph9e9e', [], 'Display', None],
  [ 0x9e9f, 'CjkUnifiedIdeograph9e9f', [], 'Display', None],
  [ 0x9ea0, 'CjkUnifiedIdeograph9ea0', [], 'Display', None],
  [ 0x9ea1, 'CjkUnifiedIdeograph9ea1', [], 'Display', None],
  [ 0x9ea2, 'CjkUnifiedIdeograph9ea2', [], 'Display', None],
  [ 0x9ea3, 'CjkUnifiedIdeograph9ea3', [], 'Display', None],
  [ 0x9ea4, 'CjkUnifiedIdeograph9ea4', [], 'Display', None],
  [ 0x9ea5, 'CjkUnifiedIdeograph9ea5', [], 'Display', None],
  [ 0x9ea6, 'CjkUnifiedIdeograph9ea6', [], 'Display', None],
  [ 0x9ea7, 'CjkUnifiedIdeograph9ea7', [], 'Display', None],
  [ 0x9ea8, 'CjkUnifiedIdeograph9ea8', [], 'Display', None],
  [ 0x9ea9, 'CjkUnifiedIdeograph9ea9', [], 'Display', None],
  [ 0x9eaa, 'CjkUnifiedIdeograph9eaa', [], 'Display', None],
  [ 0x9eab, 'CjkUnifiedIdeograph9eab', [], 'Display', None],
  [ 0x9eac, 'CjkUnifiedIdeograph9eac', [], 'Display', None],
  [ 0x9ead, 'CjkUnifiedIdeograph9ead', [], 'Display', None],
  [ 0x9eae, 'CjkUnifiedIdeograph9eae', [], 'Display', None],
  [ 0x9eaf, 'CjkUnifiedIdeograph9eaf', [], 'Display', None],
  [ 0x9eb0, 'CjkUnifiedIdeograph9eb0', [], 'Display', None],
  [ 0x9eb1, 'CjkUnifiedIdeograph9eb1', [], 'Display', None],
  [ 0x9eb2, 'CjkUnifiedIdeograph9eb2', [], 'Display', None],
  [ 0x9eb3, 'CjkUnifiedIdeograph9eb3', [], 'Display', None],
  [ 0x9eb4, 'CjkUnifiedIdeograph9eb4', [], 'Display', None],
  [ 0x9eb5, 'CjkUnifiedIdeograph9eb5', [], 'Display', None],
  [ 0x9eb6, 'CjkUnifiedIdeograph9eb6', [], 'Display', None],
  [ 0x9eb7, 'CjkUnifiedIdeograph9eb7', [], 'Display', None],
  [ 0x9eb8, 'CjkUnifiedIdeograph9eb8', [], 'Display', None],
  [ 0x9eb9, 'CjkUnifiedIdeograph9eb9', [], 'Display', None],
  [ 0x9eba, 'CjkUnifiedIdeograph9eba', [], 'Display', None],
  [ 0x9ebb, 'CjkUnifiedIdeograph9ebb', [], 'Display', None],
  [ 0x9ebc, 'CjkUnifiedIdeograph9ebc', [], 'Display', None],
  [ 0x9ebd, 'CjkUnifiedIdeograph9ebd', [], 'Display', None],
  [ 0x9ebe, 'CjkUnifiedIdeograph9ebe', [], 'Display', None],
  [ 0x9ebf, 'CjkUnifiedIdeograph9ebf', [], 'Display', None],
  [ 0x9ec0, 'CjkUnifiedIdeograph9ec0', [], 'Display', None],
  [ 0x9ec1, 'CjkUnifiedIdeograph9ec1', [], 'Display', None],
  [ 0x9ec2, 'CjkUnifiedIdeograph9ec2', [], 'Display', None],
  [ 0x9ec3, 'CjkUnifiedIdeograph9ec3', [], 'Display', None],
  [ 0x9ec4, 'CjkUnifiedIdeograph9ec4', [], 'Display', None],
  [ 0x9ec5, 'CjkUnifiedIdeograph9ec5', [], 'Display', None],
  [ 0x9ec6, 'CjkUnifiedIdeograph9ec6', [], 'Display', None],
  [ 0x9ec7, 'CjkUnifiedIdeograph9ec7', [], 'Display', None],
  [ 0x9ec8, 'CjkUnifiedIdeograph9ec8', [], 'Display', None],
  [ 0x9ec9, 'CjkUnifiedIdeograph9ec9', [], 'Display', None],
  [ 0x9eca, 'CjkUnifiedIdeograph9eca', [], 'Display', None],
  [ 0x9ecb, 'CjkUnifiedIdeograph9ecb', [], 'Display', None],
  [ 0x9ecc, 'CjkUnifiedIdeograph9ecc', [], 'Display', None],
  [ 0x9ecd, 'CjkUnifiedIdeograph9ecd', [], 'Display', None],
  [ 0x9ece, 'CjkUnifiedIdeograph9ece', [], 'Display', None],
  [ 0x9ecf, 'CjkUnifiedIdeograph9ecf', [], 'Display', None],
  [ 0x9ed0, 'CjkUnifiedIdeograph9ed0', [], 'Display', None],
  [ 0x9ed1, 'CjkUnifiedIdeograph9ed1', [], 'Display', None],
  [ 0x9ed2, 'CjkUnifiedIdeograph9ed2', [], 'Display', None],
  [ 0x9ed3, 'CjkUnifiedIdeograph9ed3', [], 'Display', None],
  [ 0x9ed4, 'CjkUnifiedIdeograph9ed4', [], 'Display', None],
  [ 0x9ed5, 'CjkUnifiedIdeograph9ed5', [], 'Display', None],
  [ 0x9ed6, 'CjkUnifiedIdeograph9ed6', [], 'Display', None],
  [ 0x9ed7, 'CjkUnifiedIdeograph9ed7', [], 'Display', None],
  [ 0x9ed8, 'CjkUnifiedIdeograph9ed8', [], 'Display', None],
  [ 0x9ed9, 'CjkUnifiedIdeograph9ed9', [], 'Display', None],
  [ 0x9eda, 'CjkUnifiedIdeograph9eda', [], 'Display', None],
  [ 0x9edb, 'CjkUnifiedIdeograph9edb', [], 'Display', None],
  [ 0x9edc, 'CjkUnifiedIdeograph9edc', [], 'Display', None],
  [ 0x9edd, 'CjkUnifiedIdeograph9edd', [], 'Display', None],
  [ 0x9ede, 'CjkUnifiedIdeograph9ede', [], 'Display', None],
  [ 0x9edf, 'CjkUnifiedIdeograph9edf', [], 'Display', None],
  [ 0x9ee0, 'CjkUnifiedIdeograph9ee0', [], 'Display', None],
  [ 0x9ee1, 'CjkUnifiedIdeograph9ee1', [], 'Display', None],
  [ 0x9ee2, 'CjkUnifiedIdeograph9ee2', [], 'Display', None],
  [ 0x9ee3, 'CjkUnifiedIdeograph9ee3', [], 'Display', None],
  [ 0x9ee4, 'CjkUnifiedIdeograph9ee4', [], 'Display', None],
  [ 0x9ee5, 'CjkUnifiedIdeograph9ee5', [], 'Display', None],
  [ 0x9ee6, 'CjkUnifiedIdeograph9ee6', [], 'Display', None],
  [ 0x9ee7, 'CjkUnifiedIdeograph9ee7', [], 'Display', None],
  [ 0x9ee8, 'CjkUnifiedIdeograph9ee8', [], 'Display', None],
  [ 0x9ee9, 'CjkUnifiedIdeograph9ee9', [], 'Display', None],
  [ 0x9eea, 'CjkUnifiedIdeograph9eea', [], 'Display', None],
  [ 0x9eeb, 'CjkUnifiedIdeograph9eeb', [], 'Display', None],
  [ 0x9eec, 'CjkUnifiedIdeograph9eec', [], 'Display', None],
  [ 0x9eed, 'CjkUnifiedIdeograph9eed', [], 'Display', None],
  [ 0x9eee, 'CjkUnifiedIdeograph9eee', [], 'Display', None],
  [ 0x9eef, 'CjkUnifiedIdeograph9eef', [], 'Display', None],
  [ 0x9ef0, 'CjkUnifiedIdeograph9ef0', [], 'Display', None],
  [ 0x9ef1, 'CjkUnifiedIdeograph9ef1', [], 'Display', None],
  [ 0x9ef2, 'CjkUnifiedIdeograph9ef2', [], 'Display', None],
  [ 0x9ef3, 'CjkUnifiedIdeograph9ef3', [], 'Display', None],
  [ 0x9ef4, 'CjkUnifiedIdeograph9ef4', [], 'Display', None],
  [ 0x9ef5, 'CjkUnifiedIdeograph9ef5', [], 'Display', None],
  [ 0x9ef6, 'CjkUnifiedIdeograph9ef6', [], 'Display', None],
  [ 0x9ef7, 'CjkUnifiedIdeograph9ef7', [], 'Display', None],
  [ 0x9ef8, 'CjkUnifiedIdeograph9ef8', [], 'Display', None],
  [ 0x9ef9, 'CjkUnifiedIdeograph9ef9', [], 'Display', None],
  [ 0x9efa, 'CjkUnifiedIdeograph9efa', [], 'Display', None],
  [ 0x9efb, 'CjkUnifiedIdeograph9efb', [], 'Display', None],
  [ 0x9efc, 'CjkUnifiedIdeograph9efc', [], 'Display', None],
  [ 0x9efd, 'CjkUnifiedIdeograph9efd', [], 'Display', None],
  [ 0x9efe, 'CjkUnifiedIdeograph9efe', [], 'Display', None],
  [ 0x9eff, 'CjkUnifiedIdeograph9eff', [], 'Display', None],
  [ 0x9f00, 'CjkUnifiedIdeograph9f00', [], 'Display', None],
  [ 0x9f01, 'CjkUnifiedIdeograph9f01', [], 'Display', None],
  [ 0x9f02, 'CjkUnifiedIdeograph9f02', [], 'Display', None],
  [ 0x9f03, 'CjkUnifiedIdeograph9f03', [], 'Display', None],
  [ 0x9f04, 'CjkUnifiedIdeograph9f04', [], 'Display', None],
  [ 0x9f05, 'CjkUnifiedIdeograph9f05', [], 'Display', None],
  [ 0x9f06, 'CjkUnifiedIdeograph9f06', [], 'Display', None],
  [ 0x9f07, 'CjkUnifiedIdeograph9f07', [], 'Display', None],
  [ 0x9f08, 'CjkUnifiedIdeograph9f08', [], 'Display', None],
  [ 0x9f09, 'CjkUnifiedIdeograph9f09', [], 'Display', None],
  [ 0x9f0a, 'CjkUnifiedIdeograph9f0a', [], 'Display', None],
  [ 0x9f0b, 'CjkUnifiedIdeograph9f0b', [], 'Display', None],
  [ 0x9f0c, 'CjkUnifiedIdeograph9f0c', [], 'Display', None],
  [ 0x9f0d, 'CjkUnifiedIdeograph9f0d', [], 'Display', None],
  [ 0x9f0e, 'CjkUnifiedIdeograph9f0e', [], 'Display', None],
  [ 0x9f0f, 'CjkUnifiedIdeograph9f0f', [], 'Display', None],
  [ 0x9f10, 'CjkUnifiedIdeograph9f10', [], 'Display', None],
  [ 0x9f11, 'CjkUnifiedIdeograph9f11', [], 'Display', None],
  [ 0x9f12, 'CjkUnifiedIdeograph9f12', [], 'Display', None],
  [ 0x9f13, 'CjkUnifiedIdeograph9f13', [], 'Display', None],
  [ 0x9f14, 'CjkUnifiedIdeograph9f14', [], 'Display', None],
  [ 0x9f15, 'CjkUnifiedIdeograph9f15', [], 'Display', None],
  [ 0x9f16, 'CjkUnifiedIdeograph9f16', [], 'Display', None],
  [ 0x9f17, 'CjkUnifiedIdeograph9f17', [], 'Display', None],
  [ 0x9f18, 'CjkUnifiedIdeograph9f18', [], 'Display', None],
  [ 0x9f19, 'CjkUnifiedIdeograph9f19', [], 'Display', None],
  [ 0x9f1a, 'CjkUnifiedIdeograph9f1a', [], 'Display', None],
  [ 0x9f1b, 'CjkUnifiedIdeograph9f1b', [], 'Display', None],
  [ 0x9f1c, 'CjkUnifiedIdeograph9f1c', [], 'Display', None],
  [ 0x9f1d, 'CjkUnifiedIdeograph9f1d', [], 'Display', None],
  [ 0x9f1e, 'CjkUnifiedIdeograph9f1e', [], 'Display', None],
  [ 0x9f1f, 'CjkUnifiedIdeograph9f1f', [], 'Display', None],
  [ 0x9f20, 'CjkUnifiedIdeograph9f20', [], 'Display', None],
  [ 0x9f21, 'CjkUnifiedIdeograph9f21', [], 'Display', None],
  [ 0x9f22, 'CjkUnifiedIdeograph9f22', [], 'Display', None],
  [ 0x9f23, 'CjkUnifiedIdeograph9f23', [], 'Display', None],
  [ 0x9f24, 'CjkUnifiedIdeograph9f24', [], 'Display', None],
  [ 0x9f25, 'CjkUnifiedIdeograph9f25', [], 'Display', None],
  [ 0x9f26, 'CjkUnifiedIdeograph9f26', [], 'Display', None],
  [ 0x9f27, 'CjkUnifiedIdeograph9f27', [], 'Display', None],
  [ 0x9f28, 'CjkUnifiedIdeograph9f28', [], 'Display', None],
  [ 0x9f29, 'CjkUnifiedIdeograph9f29', [], 'Display', None],
  [ 0x9f2a, 'CjkUnifiedIdeograph9f2a', [], 'Display', None],
  [ 0x9f2b, 'CjkUnifiedIdeograph9f2b', [], 'Display', None],
  [ 0x9f2c, 'CjkUnifiedIdeograph9f2c', [], 'Display', None],
  [ 0x9f2d, 'CjkUnifiedIdeograph9f2d', [], 'Display', None],
  [ 0x9f2e, 'CjkUnifiedIdeograph9f2e', [], 'Display', None],
  [ 0x9f2f, 'CjkUnifiedIdeograph9f2f', [], 'Display', None],
  [ 0x9f30, 'CjkUnifiedIdeograph9f30', [], 'Display', None],
  [ 0x9f31, 'CjkUnifiedIdeograph9f31', [], 'Display', None],
  [ 0x9f32, 'CjkUnifiedIdeograph9f32', [], 'Display', None],
  [ 0x9f33, 'CjkUnifiedIdeograph9f33', [], 'Display', None],
  [ 0x9f34, 'CjkUnifiedIdeograph9f34', [], 'Display', None],
  [ 0x9f35, 'CjkUnifiedIdeograph9f35', [], 'Display', None],
  [ 0x9f36, 'CjkUnifiedIdeograph9f36', [], 'Display', None],
  [ 0x9f37, 'CjkUnifiedIdeograph9f37', [], 'Display', None],
  [ 0x9f38, 'CjkUnifiedIdeograph9f38', [], 'Display', None],
  [ 0x9f39, 'CjkUnifiedIdeograph9f39', [], 'Display', None],
  [ 0x9f3a, 'CjkUnifiedIdeograph9f3a', [], 'Display', None],
  [ 0x9f3b, 'CjkUnifiedIdeograph9f3b', [], 'Display', None],
  [ 0x9f3c, 'CjkUnifiedIdeograph9f3c', [], 'Display', None],
  [ 0x9f3d, 'CjkUnifiedIdeograph9f3d', [], 'Display', None],
  [ 0x9f3e, 'CjkUnifiedIdeograph9f3e', [], 'Display', None],
  [ 0x9f3f, 'CjkUnifiedIdeograph9f3f', [], 'Display', None],
  [ 0x9f40, 'CjkUnifiedIdeograph9f40', [], 'Display', None],
  [ 0x9f41, 'CjkUnifiedIdeograph9f41', [], 'Display', None],
  [ 0x9f42, 'CjkUnifiedIdeograph9f42', [], 'Display', None],
  [ 0x9f43, 'CjkUnifiedIdeograph9f43', [], 'Display', None],
  [ 0x9f44, 'CjkUnifiedIdeograph9f44', [], 'Display', None],
  [ 0x9f45, 'CjkUnifiedIdeograph9f45', [], 'Display', None],
  [ 0x9f46, 'CjkUnifiedIdeograph9f46', [], 'Display', None],
  [ 0x9f47, 'CjkUnifiedIdeograph9f47', [], 'Display', None],
  [ 0x9f48, 'CjkUnifiedIdeograph9f48', [], 'Display', None],
  [ 0x9f49, 'CjkUnifiedIdeograph9f49', [], 'Display', None],
  [ 0x9f4a, 'CjkUnifiedIdeograph9f4a', [], 'Display', None],
  [ 0x9f4b, 'CjkUnifiedIdeograph9f4b', [], 'Display', None],
  [ 0x9f4c, 'CjkUnifiedIdeograph9f4c', [], 'Display', None],
  [ 0x9f4d, 'CjkUnifiedIdeograph9f4d', [], 'Display', None],
  [ 0x9f4e, 'CjkUnifiedIdeograph9f4e', [], 'Display', None],
  [ 0x9f4f, 'CjkUnifiedIdeograph9f4f', [], 'Display', None],
  [ 0x9f50, 'CjkUnifiedIdeograph9f50', [], 'Display', None],
  [ 0x9f51, 'CjkUnifiedIdeograph9f51', [], 'Display', None],
  [ 0x9f52, 'CjkUnifiedIdeograph9f52', [], 'Display', None],
  [ 0x9f53, 'CjkUnifiedIdeograph9f53', [], 'Display', None],
  [ 0x9f54, 'CjkUnifiedIdeograph9f54', [], 'Display', None],
  [ 0x9f55, 'CjkUnifiedIdeograph9f55', [], 'Display', None],
  [ 0x9f56, 'CjkUnifiedIdeograph9f56', [], 'Display', None],
  [ 0x9f57, 'CjkUnifiedIdeograph9f57', [], 'Display', None],
  [ 0x9f58, 'CjkUnifiedIdeograph9f58', [], 'Display', None],
  [ 0x9f59, 'CjkUnifiedIdeograph9f59', [], 'Display', None],
  [ 0x9f5a, 'CjkUnifiedIdeograph9f5a', [], 'Display', None],
  [ 0x9f5b, 'CjkUnifiedIdeograph9f5b', [], 'Display', None],
  [ 0x9f5c, 'CjkUnifiedIdeograph9f5c', [], 'Display', None],
  [ 0x9f5d, 'CjkUnifiedIdeograph9f5d', [], 'Display', None],
  [ 0x9f5e, 'CjkUnifiedIdeograph9f5e', [], 'Display', None],
  [ 0x9f5f, 'CjkUnifiedIdeograph9f5f', [], 'Display', None],
  [ 0x9f60, 'CjkUnifiedIdeograph9f60', [], 'Display', None],
  [ 0x9f61, 'CjkUnifiedIdeograph9f61', [], 'Display', None],
  [ 0x9f62, 'CjkUnifiedIdeograph9f62', [], 'Display', None],
  [ 0x9f63, 'CjkUnifiedIdeograph9f63', [], 'Display', None],
  [ 0x9f64, 'CjkUnifiedIdeograph9f64', [], 'Display', None],
  [ 0x9f65, 'CjkUnifiedIdeograph9f65', [], 'Display', None],
  [ 0x9f66, 'CjkUnifiedIdeograph9f66', [], 'Display', None],
  [ 0x9f67, 'CjkUnifiedIdeograph9f67', [], 'Display', None],
  [ 0x9f68, 'CjkUnifiedIdeograph9f68', [], 'Display', None],
  [ 0x9f69, 'CjkUnifiedIdeograph9f69', [], 'Display', None],
  [ 0x9f6a, 'CjkUnifiedIdeograph9f6a', [], 'Display', None],
  [ 0x9f6b, 'CjkUnifiedIdeograph9f6b', [], 'Display', None],
  [ 0x9f6c, 'CjkUnifiedIdeograph9f6c', [], 'Display', None],
  [ 0x9f6d, 'CjkUnifiedIdeograph9f6d', [], 'Display', None],
  [ 0x9f6e, 'CjkUnifiedIdeograph9f6e', [], 'Display', None],
  [ 0x9f6f, 'CjkUnifiedIdeograph9f6f', [], 'Display', None],
  [ 0x9f70, 'CjkUnifiedIdeograph9f70', [], 'Display', None],
  [ 0x9f71, 'CjkUnifiedIdeograph9f71', [], 'Display', None],
  [ 0x9f72, 'CjkUnifiedIdeograph9f72', [], 'Display', None],
  [ 0x9f73, 'CjkUnifiedIdeograph9f73', [], 'Display', None],
  [ 0x9f74, 'CjkUnifiedIdeograph9f74', [], 'Display', None],
  [ 0x9f75, 'CjkUnifiedIdeograph9f75', [], 'Display', None],
  [ 0x9f76, 'CjkUnifiedIdeograph9f76', [], 'Display', None],
  [ 0x9f77, 'CjkUnifiedIdeograph9f77', [], 'Display', None],
  [ 0x9f78, 'CjkUnifiedIdeograph9f78', [], 'Display', None],
  [ 0x9f79, 'CjkUnifiedIdeograph9f79', [], 'Display', None],
  [ 0x9f7a, 'CjkUnifiedIdeograph9f7a', [], 'Display', None],
  [ 0x9f7b, 'CjkUnifiedIdeograph9f7b', [], 'Display', None],
  [ 0x9f7c, 'CjkUnifiedIdeograph9f7c', [], 'Display', None],
  [ 0x9f7d, 'CjkUnifiedIdeograph9f7d', [], 'Display', None],
  [ 0x9f7e, 'CjkUnifiedIdeograph9f7e', [], 'Display', None],
  [ 0x9f7f, 'CjkUnifiedIdeograph9f7f', [], 'Display', None],
  [ 0x9f80, 'CjkUnifiedIdeograph9f80', [], 'Display', None],
  [ 0x9f81, 'CjkUnifiedIdeograph9f81', [], 'Display', None],
  [ 0x9f82, 'CjkUnifiedIdeograph9f82', [], 'Display', None],
  [ 0x9f83, 'CjkUnifiedIdeograph9f83', [], 'Display', None],
  [ 0x9f84, 'CjkUnifiedIdeograph9f84', [], 'Display', None],
  [ 0x9f85, 'CjkUnifiedIdeograph9f85', [], 'Display', None],
  [ 0x9f86, 'CjkUnifiedIdeograph9f86', [], 'Display', None],
  [ 0x9f87, 'CjkUnifiedIdeograph9f87', [], 'Display', None],
  [ 0x9f88, 'CjkUnifiedIdeograph9f88', [], 'Display', None],
  [ 0x9f89, 'CjkUnifiedIdeograph9f89', [], 'Display', None],
  [ 0x9f8a, 'CjkUnifiedIdeograph9f8a', [], 'Display', None],
  [ 0x9f8b, 'CjkUnifiedIdeograph9f8b', [], 'Display', None],
  [ 0x9f8c, 'CjkUnifiedIdeograph9f8c', [], 'Display', None],
  [ 0x9f8d, 'CjkUnifiedIdeograph9f8d', [], 'Display', None],
  [ 0x9f8e, 'CjkUnifiedIdeograph9f8e', [], 'Display', None],
  [ 0x9f8f, 'CjkUnifiedIdeograph9f8f', [], 'Display', None],
  [ 0x9f90, 'CjkUnifiedIdeograph9f90', [], 'Display', None],
  [ 0x9f91, 'CjkUnifiedIdeograph9f91', [], 'Display', None],
  [ 0x9f92, 'CjkUnifiedIdeograph9f92', [], 'Display', None],
  [ 0x9f93, 'CjkUnifiedIdeograph9f93', [], 'Display', None],
  [ 0x9f94, 'CjkUnifiedIdeograph9f94', [], 'Display', None],
  [ 0x9f95, 'CjkUnifiedIdeograph9f95', [], 'Display', None],
  [ 0x9f96, 'CjkUnifiedIdeograph9f96', [], 'Display', None],
  [ 0x9f97, 'CjkUnifiedIdeograph9f97', [], 'Display', None],
  [ 0x9f98, 'CjkUnifiedIdeograph9f98', [], 'Display', None],
  [ 0x9f99, 'CjkUnifiedIdeograph9f99', [], 'Display', None],
  [ 0x9f9a, 'CjkUnifiedIdeograph9f9a', [], 'Display', None],
  [ 0x9f9b, 'CjkUnifiedIdeograph9f9b', [], 'Display', None],
  [ 0x9f9c, 'CjkUnifiedIdeograph9f9c', [], 'Display', None],
  [ 0x9f9d, 'CjkUnifiedIdeograph9f9d', [], 'Display', None],
  [ 0x9f9e, 'CjkUnifiedIdeograph9f9e', [], 'Display', None],
  [ 0x9f9f, 'CjkUnifiedIdeograph9f9f', [], 'Display', None],
  [ 0x9fa0, 'CjkUnifiedIdeograph9fa0', [], 'Display', None],
  [ 0x9fa1, 'CjkUnifiedIdeograph9fa1', [], 'Display', None],
  [ 0x9fa2, 'CjkUnifiedIdeograph9fa2', [], 'Display', None],
  [ 0x9fa3, 'CjkUnifiedIdeograph9fa3', [], 'Display', None],
  [ 0x9fa4, 'CjkUnifiedIdeograph9fa4', [], 'Display', None],
  [ 0x9fa5, 'CjkUnifiedIdeograph9fa5', [], 'Display', None],
  [ 0x9fa6, 'CjkUnifiedIdeograph9fa6', [], 'Display', None],
  [ 0x9fa7, 'CjkUnifiedIdeograph9fa7', [], 'Display', None],
  [ 0x9fa8, 'CjkUnifiedIdeograph9fa8', [], 'Display', None],
  [ 0x9fa9, 'CjkUnifiedIdeograph9fa9', [], 'Display', None],
  [ 0x9faa, 'CjkUnifiedIdeograph9faa', [], 'Display', None],
  [ 0x9fab, 'CjkUnifiedIdeograph9fab', [], 'Display', None],
  [ 0x9fac, 'CjkUnifiedIdeograph9fac', [], 'Display', None],
  [ 0x9fad, 'CjkUnifiedIdeograph9fad', [], 'Display', None],
  [ 0x9fae, 'CjkUnifiedIdeograph9fae', [], 'Display', None],
  [ 0x9faf, 'CjkUnifiedIdeograph9faf', [], 'Display', None],
  [ 0x9fb0, 'CjkUnifiedIdeograph9fb0', [], 'Display', None],
  [ 0x9fb1, 'CjkUnifiedIdeograph9fb1', [], 'Display', None],
  [ 0x9fb2, 'CjkUnifiedIdeograph9fb2', [], 'Display', None],
  [ 0x9fb3, 'CjkUnifiedIdeograph9fb3', [], 'Display', None],
  [ 0x9fb4, 'CjkUnifiedIdeograph9fb4', [], 'Display', None],
  [ 0x9fb5, 'CjkUnifiedIdeograph9fb5', [], 'Display', None],
  [ 0x9fb6, 'CjkUnifiedIdeograph9fb6', [], 'Display', None],
  [ 0x9fb7, 'CjkUnifiedIdeograph9fb7', [], 'Display', None],
  [ 0x9fb8, 'CjkUnifiedIdeograph9fb8', [], 'Display', None],
  [ 0x9fb9, 'CjkUnifiedIdeograph9fb9', [], 'Display', None],
  [ 0x9fba, 'CjkUnifiedIdeograph9fba', [], 'Display', None],
  [ 0x9fbb, 'CjkUnifiedIdeograph9fbb', [], 'Display', None],
  [ 0x9fbc, 'UnicodeCharacter0x9fbc', [], 'Display', None],
  [ 0x9fbd, 'UnicodeCharacter0x9fbd', [], 'Display', None],
  [ 0x9fbe, 'UnicodeCharacter0x9fbe', [], 'Display', None],
  [ 0x9fbf, 'UnicodeCharacter0x9fbf', [], 'Display', None],
  [ 0x9fc0, 'UnicodeCharacter0x9fc0', [], 'Display', None],
  [ 0x9fc1, 'UnicodeCharacter0x9fc1', [], 'Display', None],
  [ 0x9fc2, 'UnicodeCharacter0x9fc2', [], 'Display', None],
  [ 0x9fc3, 'UnicodeCharacter0x9fc3', [], 'Display', None],
  [ 0x9fc4, 'UnicodeCharacter0x9fc4', [], 'Display', None],
  [ 0x9fc5, 'UnicodeCharacter0x9fc5', [], 'Display', None],
  [ 0x9fc6, 'UnicodeCharacter0x9fc6', [], 'Display', None],
  [ 0x9fc7, 'UnicodeCharacter0x9fc7', [], 'Display', None],
  [ 0x9fc8, 'UnicodeCharacter0x9fc8', [], 'Display', None],
  [ 0x9fc9, 'UnicodeCharacter0x9fc9', [], 'Display', None],
  [ 0x9fca, 'UnicodeCharacter0x9fca', [], 'Display', None],
  [ 0x9fcb, 'UnicodeCharacter0x9fcb', [], 'Display', None],
  [ 0x9fcc, 'UnicodeCharacter0x9fcc', [], 'Display', None],
  [ 0x9fcd, 'UnicodeCharacter0x9fcd', [], 'Display', None],
  [ 0x9fce, 'UnicodeCharacter0x9fce', [], 'Display', None],
  [ 0x9fcf, 'UnicodeCharacter0x9fcf', [], 'Display', None],
  [ 0x9fd0, 'UnicodeCharacter0x9fd0', [], 'Display', None],
  [ 0x9fd1, 'UnicodeCharacter0x9fd1', [], 'Display', None],
  [ 0x9fd2, 'UnicodeCharacter0x9fd2', [], 'Display', None],
  [ 0x9fd3, 'UnicodeCharacter0x9fd3', [], 'Display', None],
  [ 0x9fd4, 'UnicodeCharacter0x9fd4', [], 'Display', None],
  [ 0x9fd5, 'UnicodeCharacter0x9fd5', [], 'Display', None],
  [ 0x9fd6, 'UnicodeCharacter0x9fd6', [], 'Display', None],
  [ 0x9fd7, 'UnicodeCharacter0x9fd7', [], 'Display', None],
  [ 0x9fd8, 'UnicodeCharacter0x9fd8', [], 'Display', None],
  [ 0x9fd9, 'UnicodeCharacter0x9fd9', [], 'Display', None],
  [ 0x9fda, 'UnicodeCharacter0x9fda', [], 'Display', None],
  [ 0x9fdb, 'UnicodeCharacter0x9fdb', [], 'Display', None],
  [ 0x9fdc, 'UnicodeCharacter0x9fdc', [], 'Display', None],
  [ 0x9fdd, 'UnicodeCharacter0x9fdd', [], 'Display', None],
  [ 0x9fde, 'UnicodeCharacter0x9fde', [], 'Display', None],
  [ 0x9fdf, 'UnicodeCharacter0x9fdf', [], 'Display', None],
  [ 0x9fe0, 'UnicodeCharacter0x9fe0', [], 'Display', None],
  [ 0x9fe1, 'UnicodeCharacter0x9fe1', [], 'Display', None],
  [ 0x9fe2, 'UnicodeCharacter0x9fe2', [], 'Display', None],
  [ 0x9fe3, 'UnicodeCharacter0x9fe3', [], 'Display', None],
  [ 0x9fe4, 'UnicodeCharacter0x9fe4', [], 'Display', None],
  [ 0x9fe5, 'UnicodeCharacter0x9fe5', [], 'Display', None],
  [ 0x9fe6, 'UnicodeCharacter0x9fe6', [], 'Display', None],
  [ 0x9fe7, 'UnicodeCharacter0x9fe7', [], 'Display', None],
  [ 0x9fe8, 'UnicodeCharacter0x9fe8', [], 'Display', None],
  [ 0x9fe9, 'UnicodeCharacter0x9fe9', [], 'Display', None],
  [ 0x9fea, 'UnicodeCharacter0x9fea', [], 'Display', None],
  [ 0x9feb, 'UnicodeCharacter0x9feb', [], 'Display', None],
  [ 0x9fec, 'UnicodeCharacter0x9fec', [], 'Display', None],
  [ 0x9fed, 'UnicodeCharacter0x9fed', [], 'Display', None],
  [ 0x9fee, 'UnicodeCharacter0x9fee', [], 'Display', None],
  [ 0x9fef, 'UnicodeCharacter0x9fef', [], 'Display', None],
  [ 0x9ff0, 'UnicodeCharacter0x9ff0', [], 'Display', None],
  [ 0x9ff1, 'UnicodeCharacter0x9ff1', [], 'Display', None],
  [ 0x9ff2, 'UnicodeCharacter0x9ff2', [], 'Display', None],
  [ 0x9ff3, 'UnicodeCharacter0x9ff3', [], 'Display', None],
  [ 0x9ff4, 'UnicodeCharacter0x9ff4', [], 'Display', None],
  [ 0x9ff5, 'UnicodeCharacter0x9ff5', [], 'Display', None],
  [ 0x9ff6, 'UnicodeCharacter0x9ff6', [], 'Display', None],
  [ 0x9ff7, 'UnicodeCharacter0x9ff7', [], 'Display', None],
  [ 0x9ff8, 'UnicodeCharacter0x9ff8', [], 'Display', None],
  [ 0x9ff9, 'UnicodeCharacter0x9ff9', [], 'Display', None],
  [ 0x9ffa, 'UnicodeCharacter0x9ffa', [], 'Display', None],
  [ 0x9ffb, 'UnicodeCharacter0x9ffb', [], 'Display', None],
  [ 0x9ffc, 'UnicodeCharacter0x9ffc', [], 'Display', None],
  [ 0x9ffd, 'UnicodeCharacter0x9ffd', [], 'Display', None],
  [ 0x9ffe, 'UnicodeCharacter0x9ffe', [], 'Display', None],
  [ 0x9fff, 'UnicodeCharacter0x9fff', [], 'Display', None],
  [ 0xa000, 'YiSyllableIt', [], 'Display', None],
  [ 0xa001, 'YiSyllableIx', [], 'Display', None],
  [ 0xa002, 'YiSyllableI', [], 'Display', None],
  [ 0xa003, 'YiSyllableIp', [], 'Display', None],
  [ 0xa004, 'YiSyllableIet', [], 'Display', None],
  [ 0xa005, 'YiSyllableIex', [], 'Display', None],
  [ 0xa006, 'YiSyllableIe', [], 'Display', None],
  [ 0xa007, 'YiSyllableIep', [], 'Display', None],
  [ 0xa008, 'YiSyllableAt', [], 'Display', None],
  [ 0xa009, 'YiSyllableAx', [], 'Display', None],
  [ 0xa00a, 'YiSyllableA', [], 'Display', None],
  [ 0xa00b, 'YiSyllableAp', [], 'Display', None],
  [ 0xa00c, 'YiSyllableUox', [], 'Display', None],
  [ 0xa00d, 'YiSyllableUo', [], 'Display', None],
  [ 0xa00e, 'YiSyllableUop', [], 'Display', None],
  [ 0xa00f, 'YiSyllableOt', [], 'Display', None],
  [ 0xa010, 'YiSyllableOx', [], 'Display', None],
  [ 0xa011, 'YiSyllableO', [], 'Display', None],
  [ 0xa012, 'YiSyllableOp', [], 'Display', None],
  [ 0xa013, 'YiSyllableEx', [], 'Display', None],
  [ 0xa014, 'YiSyllableE', [], 'Display', None],
  [ 0xa015, 'YiSyllableWu', [], 'Display', None],
  [ 0xa016, 'YiSyllableBit', [], 'Display', None],
  [ 0xa017, 'YiSyllableBix', [], 'Display', None],
  [ 0xa018, 'YiSyllableBi', [], 'Display', None],
  [ 0xa019, 'YiSyllableBip', [], 'Display', None],
  [ 0xa01a, 'YiSyllableBiet', [], 'Display', None],
  [ 0xa01b, 'YiSyllableBiex', [], 'Display', None],
  [ 0xa01c, 'YiSyllableBie', [], 'Display', None],
  [ 0xa01d, 'YiSyllableBiep', [], 'Display', None],
  [ 0xa01e, 'YiSyllableBat', [], 'Display', None],
  [ 0xa01f, 'YiSyllableBax', [], 'Display', None],
  [ 0xa020, 'YiSyllableBa', [], 'Display', None],
  [ 0xa021, 'YiSyllableBap', [], 'Display', None],
  [ 0xa022, 'YiSyllableBuox', [], 'Display', None],
  [ 0xa023, 'YiSyllableBuo', [], 'Display', None],
  [ 0xa024, 'YiSyllableBuop', [], 'Display', None],
  [ 0xa025, 'YiSyllableBot', [], 'Display', None],
  [ 0xa026, 'YiSyllableBox', [], 'Display', None],
  [ 0xa027, 'YiSyllableBo', [], 'Display', None],
  [ 0xa028, 'YiSyllableBop', [], 'Display', None],
  [ 0xa029, 'YiSyllableBex', [], 'Display', None],
  [ 0xa02a, 'YiSyllableBe', [], 'Display', None],
  [ 0xa02b, 'YiSyllableBep', [], 'Display', None],
  [ 0xa02c, 'YiSyllableBut', [], 'Display', None],
  [ 0xa02d, 'YiSyllableBux', [], 'Display', None],
  [ 0xa02e, 'YiSyllableBu', [], 'Display', None],
  [ 0xa02f, 'YiSyllableBup', [], 'Display', None],
  [ 0xa030, 'YiSyllableBurx', [], 'Display', None],
  [ 0xa031, 'YiSyllableBur', [], 'Display', None],
  [ 0xa032, 'YiSyllableByt', [], 'Display', None],
  [ 0xa033, 'YiSyllableByx', [], 'Display', None],
  [ 0xa034, 'YiSyllableBy', [], 'Display', None],
  [ 0xa035, 'YiSyllableByp', [], 'Display', None],
  [ 0xa036, 'YiSyllableByrx', [], 'Display', None],
  [ 0xa037, 'YiSyllableByr', [], 'Display', None],
  [ 0xa038, 'YiSyllablePit', [], 'Display', None],
  [ 0xa039, 'YiSyllablePix', [], 'Display', None],
  [ 0xa03a, 'YiSyllablePi', [], 'Display', None],
  [ 0xa03b, 'YiSyllablePip', [], 'Display', None],
  [ 0xa03c, 'YiSyllablePiex', [], 'Display', None],
  [ 0xa03d, 'YiSyllablePie', [], 'Display', None],
  [ 0xa03e, 'YiSyllablePiep', [], 'Display', None],
  [ 0xa03f, 'YiSyllablePat', [], 'Display', None],
  [ 0xa040, 'YiSyllablePax', [], 'Display', None],
  [ 0xa041, 'YiSyllablePa', [], 'Display', None],
  [ 0xa042, 'YiSyllablePap', [], 'Display', None],
  [ 0xa043, 'YiSyllablePuox', [], 'Display', None],
  [ 0xa044, 'YiSyllablePuo', [], 'Display', None],
  [ 0xa045, 'YiSyllablePuop', [], 'Display', None],
  [ 0xa046, 'YiSyllablePot', [], 'Display', None],
  [ 0xa047, 'YiSyllablePox', [], 'Display', None],
  [ 0xa048, 'YiSyllablePo', [], 'Display', None],
  [ 0xa049, 'YiSyllablePop', [], 'Display', None],
  [ 0xa04a, 'YiSyllablePut', [], 'Display', None],
  [ 0xa04b, 'YiSyllablePux', [], 'Display', None],
  [ 0xa04c, 'YiSyllablePu', [], 'Display', None],
  [ 0xa04d, 'YiSyllablePup', [], 'Display', None],
  [ 0xa04e, 'YiSyllablePurx', [], 'Display', None],
  [ 0xa04f, 'YiSyllablePur', [], 'Display', None],
  [ 0xa050, 'YiSyllablePyt', [], 'Display', None],
  [ 0xa051, 'YiSyllablePyx', [], 'Display', None],
  [ 0xa052, 'YiSyllablePy', [], 'Display', None],
  [ 0xa053, 'YiSyllablePyp', [], 'Display', None],
  [ 0xa054, 'YiSyllablePyrx', [], 'Display', None],
  [ 0xa055, 'YiSyllablePyr', [], 'Display', None],
  [ 0xa056, 'YiSyllableBbit', [], 'Display', None],
  [ 0xa057, 'YiSyllableBbix', [], 'Display', None],
  [ 0xa058, 'YiSyllableBbi', [], 'Display', None],
  [ 0xa059, 'YiSyllableBbip', [], 'Display', None],
  [ 0xa05a, 'YiSyllableBbiet', [], 'Display', None],
  [ 0xa05b, 'YiSyllableBbiex', [], 'Display', None],
  [ 0xa05c, 'YiSyllableBbie', [], 'Display', None],
  [ 0xa05d, 'YiSyllableBbiep', [], 'Display', None],
  [ 0xa05e, 'YiSyllableBbat', [], 'Display', None],
  [ 0xa05f, 'YiSyllableBbax', [], 'Display', None],
  [ 0xa060, 'YiSyllableBba', [], 'Display', None],
  [ 0xa061, 'YiSyllableBbap', [], 'Display', None],
  [ 0xa062, 'YiSyllableBbuox', [], 'Display', None],
  [ 0xa063, 'YiSyllableBbuo', [], 'Display', None],
  [ 0xa064, 'YiSyllableBbuop', [], 'Display', None],
  [ 0xa065, 'YiSyllableBbot', [], 'Display', None],
  [ 0xa066, 'YiSyllableBbox', [], 'Display', None],
  [ 0xa067, 'YiSyllableBbo', [], 'Display', None],
  [ 0xa068, 'YiSyllableBbop', [], 'Display', None],
  [ 0xa069, 'YiSyllableBbex', [], 'Display', None],
  [ 0xa06a, 'YiSyllableBbe', [], 'Display', None],
  [ 0xa06b, 'YiSyllableBbep', [], 'Display', None],
  [ 0xa06c, 'YiSyllableBbut', [], 'Display', None],
  [ 0xa06d, 'YiSyllableBbux', [], 'Display', None],
  [ 0xa06e, 'YiSyllableBbu', [], 'Display', None],
  [ 0xa06f, 'YiSyllableBbup', [], 'Display', None],
  [ 0xa070, 'YiSyllableBburx', [], 'Display', None],
  [ 0xa071, 'YiSyllableBbur', [], 'Display', None],
  [ 0xa072, 'YiSyllableBbyt', [], 'Display', None],
  [ 0xa073, 'YiSyllableBbyx', [], 'Display', None],
  [ 0xa074, 'YiSyllableBby', [], 'Display', None],
  [ 0xa075, 'YiSyllableBbyp', [], 'Display', None],
  [ 0xa076, 'YiSyllableNbit', [], 'Display', None],
  [ 0xa077, 'YiSyllableNbix', [], 'Display', None],
  [ 0xa078, 'YiSyllableNbi', [], 'Display', None],
  [ 0xa079, 'YiSyllableNbip', [], 'Display', None],
  [ 0xa07a, 'YiSyllableNbiex', [], 'Display', None],
  [ 0xa07b, 'YiSyllableNbie', [], 'Display', None],
  [ 0xa07c, 'YiSyllableNbiep', [], 'Display', None],
  [ 0xa07d, 'YiSyllableNbat', [], 'Display', None],
  [ 0xa07e, 'YiSyllableNbax', [], 'Display', None],
  [ 0xa07f, 'YiSyllableNba', [], 'Display', None],
  [ 0xa080, 'YiSyllableNbap', [], 'Display', None],
  [ 0xa081, 'YiSyllableNbot', [], 'Display', None],
  [ 0xa082, 'YiSyllableNbox', [], 'Display', None],
  [ 0xa083, 'YiSyllableNbo', [], 'Display', None],
  [ 0xa084, 'YiSyllableNbop', [], 'Display', None],
  [ 0xa085, 'YiSyllableNbut', [], 'Display', None],
  [ 0xa086, 'YiSyllableNbux', [], 'Display', None],
  [ 0xa087, 'YiSyllableNbu', [], 'Display', None],
  [ 0xa088, 'YiSyllableNbup', [], 'Display', None],
  [ 0xa089, 'YiSyllableNburx', [], 'Display', None],
  [ 0xa08a, 'YiSyllableNbur', [], 'Display', None],
  [ 0xa08b, 'YiSyllableNbyt', [], 'Display', None],
  [ 0xa08c, 'YiSyllableNbyx', [], 'Display', None],
  [ 0xa08d, 'YiSyllableNby', [], 'Display', None],
  [ 0xa08e, 'YiSyllableNbyp', [], 'Display', None],
  [ 0xa08f, 'YiSyllableNbyrx', [], 'Display', None],
  [ 0xa090, 'YiSyllableNbyr', [], 'Display', None],
  [ 0xa091, 'YiSyllableHmit', [], 'Display', None],
  [ 0xa092, 'YiSyllableHmix', [], 'Display', None],
  [ 0xa093, 'YiSyllableHmi', [], 'Display', None],
  [ 0xa094, 'YiSyllableHmip', [], 'Display', None],
  [ 0xa095, 'YiSyllableHmiex', [], 'Display', None],
  [ 0xa096, 'YiSyllableHmie', [], 'Display', None],
  [ 0xa097, 'YiSyllableHmiep', [], 'Display', None],
  [ 0xa098, 'YiSyllableHmat', [], 'Display', None],
  [ 0xa099, 'YiSyllableHmax', [], 'Display', None],
  [ 0xa09a, 'YiSyllableHma', [], 'Display', None],
  [ 0xa09b, 'YiSyllableHmap', [], 'Display', None],
  [ 0xa09c, 'YiSyllableHmuox', [], 'Display', None],
  [ 0xa09d, 'YiSyllableHmuo', [], 'Display', None],
  [ 0xa09e, 'YiSyllableHmuop', [], 'Display', None],
  [ 0xa09f, 'YiSyllableHmot', [], 'Display', None],
  [ 0xa0a0, 'YiSyllableHmox', [], 'Display', None],
  [ 0xa0a1, 'YiSyllableHmo', [], 'Display', None],
  [ 0xa0a2, 'YiSyllableHmop', [], 'Display', None],
  [ 0xa0a3, 'YiSyllableHmut', [], 'Display', None],
  [ 0xa0a4, 'YiSyllableHmux', [], 'Display', None],
  [ 0xa0a5, 'YiSyllableHmu', [], 'Display', None],
  [ 0xa0a6, 'YiSyllableHmup', [], 'Display', None],
  [ 0xa0a7, 'YiSyllableHmurx', [], 'Display', None],
  [ 0xa0a8, 'YiSyllableHmur', [], 'Display', None],
  [ 0xa0a9, 'YiSyllableHmyx', [], 'Display', None],
  [ 0xa0aa, 'YiSyllableHmy', [], 'Display', None],
  [ 0xa0ab, 'YiSyllableHmyp', [], 'Display', None],
  [ 0xa0ac, 'YiSyllableHmyrx', [], 'Display', None],
  [ 0xa0ad, 'YiSyllableHmyr', [], 'Display', None],
  [ 0xa0ae, 'YiSyllableMit', [], 'Display', None],
  [ 0xa0af, 'YiSyllableMix', [], 'Display', None],
  [ 0xa0b0, 'YiSyllableMi', [], 'Display', None],
  [ 0xa0b1, 'YiSyllableMip', [], 'Display', None],
  [ 0xa0b2, 'YiSyllableMiex', [], 'Display', None],
  [ 0xa0b3, 'YiSyllableMie', [], 'Display', None],
  [ 0xa0b4, 'YiSyllableMiep', [], 'Display', None],
  [ 0xa0b5, 'YiSyllableMat', [], 'Display', None],
  [ 0xa0b6, 'YiSyllableMax', [], 'Display', None],
  [ 0xa0b7, 'YiSyllableMa', [], 'Display', None],
  [ 0xa0b8, 'YiSyllableMap', [], 'Display', None],
  [ 0xa0b9, 'YiSyllableMuot', [], 'Display', None],
  [ 0xa0ba, 'YiSyllableMuox', [], 'Display', None],
  [ 0xa0bb, 'YiSyllableMuo', [], 'Display', None],
  [ 0xa0bc, 'YiSyllableMuop', [], 'Display', None],
  [ 0xa0bd, 'YiSyllableMot', [], 'Display', None],
  [ 0xa0be, 'YiSyllableMox', [], 'Display', None],
  [ 0xa0bf, 'YiSyllableMo', [], 'Display', None],
  [ 0xa0c0, 'YiSyllableMop', [], 'Display', None],
  [ 0xa0c1, 'YiSyllableMex', [], 'Display', None],
  [ 0xa0c2, 'YiSyllableMe', [], 'Display', None],
  [ 0xa0c3, 'YiSyllableMut', [], 'Display', None],
  [ 0xa0c4, 'YiSyllableMux', [], 'Display', None],
  [ 0xa0c5, 'YiSyllableMu', [], 'Display', None],
  [ 0xa0c6, 'YiSyllableMup', [], 'Display', None],
  [ 0xa0c7, 'YiSyllableMurx', [], 'Display', None],
  [ 0xa0c8, 'YiSyllableMur', [], 'Display', None],
  [ 0xa0c9, 'YiSyllableMyt', [], 'Display', None],
  [ 0xa0ca, 'YiSyllableMyx', [], 'Display', None],
  [ 0xa0cb, 'YiSyllableMy', [], 'Display', None],
  [ 0xa0cc, 'YiSyllableMyp', [], 'Display', None],
  [ 0xa0cd, 'YiSyllableFit', [], 'Display', None],
  [ 0xa0ce, 'YiSyllableFix', [], 'Display', None],
  [ 0xa0cf, 'YiSyllableFi', [], 'Display', None],
  [ 0xa0d0, 'YiSyllableFip', [], 'Display', None],
  [ 0xa0d1, 'YiSyllableFat', [], 'Display', None],
  [ 0xa0d2, 'YiSyllableFax', [], 'Display', None],
  [ 0xa0d3, 'YiSyllableFa', [], 'Display', None],
  [ 0xa0d4, 'YiSyllableFap', [], 'Display', None],
  [ 0xa0d5, 'YiSyllableFox', [], 'Display', None],
  [ 0xa0d6, 'YiSyllableFo', [], 'Display', None],
  [ 0xa0d7, 'YiSyllableFop', [], 'Display', None],
  [ 0xa0d8, 'YiSyllableFut', [], 'Display', None],
  [ 0xa0d9, 'YiSyllableFux', [], 'Display', None],
  [ 0xa0da, 'YiSyllableFu', [], 'Display', None],
  [ 0xa0db, 'YiSyllableFup', [], 'Display', None],
  [ 0xa0dc, 'YiSyllableFurx', [], 'Display', None],
  [ 0xa0dd, 'YiSyllableFur', [], 'Display', None],
  [ 0xa0de, 'YiSyllableFyt', [], 'Display', None],
  [ 0xa0df, 'YiSyllableFyx', [], 'Display', None],
  [ 0xa0e0, 'YiSyllableFy', [], 'Display', None],
  [ 0xa0e1, 'YiSyllableFyp', [], 'Display', None],
  [ 0xa0e2, 'YiSyllableVit', [], 'Display', None],
  [ 0xa0e3, 'YiSyllableVix', [], 'Display', None],
  [ 0xa0e4, 'YiSyllableVi', [], 'Display', None],
  [ 0xa0e5, 'YiSyllableVip', [], 'Display', None],
  [ 0xa0e6, 'YiSyllableViet', [], 'Display', None],
  [ 0xa0e7, 'YiSyllableViex', [], 'Display', None],
  [ 0xa0e8, 'YiSyllableVie', [], 'Display', None],
  [ 0xa0e9, 'YiSyllableViep', [], 'Display', None],
  [ 0xa0ea, 'YiSyllableVat', [], 'Display', None],
  [ 0xa0eb, 'YiSyllableVax', [], 'Display', None],
  [ 0xa0ec, 'YiSyllableVa', [], 'Display', None],
  [ 0xa0ed, 'YiSyllableVap', [], 'Display', None],
  [ 0xa0ee, 'YiSyllableVot', [], 'Display', None],
  [ 0xa0ef, 'YiSyllableVox', [], 'Display', None],
  [ 0xa0f0, 'YiSyllableVo', [], 'Display', None],
  [ 0xa0f1, 'YiSyllableVop', [], 'Display', None],
  [ 0xa0f2, 'YiSyllableVex', [], 'Display', None],
  [ 0xa0f3, 'YiSyllableVep', [], 'Display', None],
  [ 0xa0f4, 'YiSyllableVut', [], 'Display', None],
  [ 0xa0f5, 'YiSyllableVux', [], 'Display', None],
  [ 0xa0f6, 'YiSyllableVu', [], 'Display', None],
  [ 0xa0f7, 'YiSyllableVup', [], 'Display', None],
  [ 0xa0f8, 'YiSyllableVurx', [], 'Display', None],
  [ 0xa0f9, 'YiSyllableVur', [], 'Display', None],
  [ 0xa0fa, 'YiSyllableVyt', [], 'Display', None],
  [ 0xa0fb, 'YiSyllableVyx', [], 'Display', None],
  [ 0xa0fc, 'YiSyllableVy', [], 'Display', None],
  [ 0xa0fd, 'YiSyllableVyp', [], 'Display', None],
  [ 0xa0fe, 'YiSyllableVyrx', [], 'Display', None],
  [ 0xa0ff, 'YiSyllableVyr', [], 'Display', None],
  [ 0xa100, 'YiSyllableDit', [], 'Display', None],
  [ 0xa101, 'YiSyllableDix', [], 'Display', None],
  [ 0xa102, 'YiSyllableDi', [], 'Display', None],
  [ 0xa103, 'YiSyllableDip', [], 'Display', None],
  [ 0xa104, 'YiSyllableDiex', [], 'Display', None],
  [ 0xa105, 'YiSyllableDie', [], 'Display', None],
  [ 0xa106, 'YiSyllableDiep', [], 'Display', None],
  [ 0xa107, 'YiSyllableDat', [], 'Display', None],
  [ 0xa108, 'YiSyllableDax', [], 'Display', None],
  [ 0xa109, 'YiSyllableDa', [], 'Display', None],
  [ 0xa10a, 'YiSyllableDap', [], 'Display', None],
  [ 0xa10b, 'YiSyllableDuox', [], 'Display', None],
  [ 0xa10c, 'YiSyllableDuo', [], 'Display', None],
  [ 0xa10d, 'YiSyllableDot', [], 'Display', None],
  [ 0xa10e, 'YiSyllableDox', [], 'Display', None],
  [ 0xa10f, 'YiSyllableDo', [], 'Display', None],
  [ 0xa110, 'YiSyllableDop', [], 'Display', None],
  [ 0xa111, 'YiSyllableDex', [], 'Display', None],
  [ 0xa112, 'YiSyllableDe', [], 'Display', None],
  [ 0xa113, 'YiSyllableDep', [], 'Display', None],
  [ 0xa114, 'YiSyllableDut', [], 'Display', None],
  [ 0xa115, 'YiSyllableDux', [], 'Display', None],
  [ 0xa116, 'YiSyllableDu', [], 'Display', None],
  [ 0xa117, 'YiSyllableDup', [], 'Display', None],
  [ 0xa118, 'YiSyllableDurx', [], 'Display', None],
  [ 0xa119, 'YiSyllableDur', [], 'Display', None],
  [ 0xa11a, 'YiSyllableTit', [], 'Display', None],
  [ 0xa11b, 'YiSyllableTix', [], 'Display', None],
  [ 0xa11c, 'YiSyllableTi', [], 'Display', None],
  [ 0xa11d, 'YiSyllableTip', [], 'Display', None],
  [ 0xa11e, 'YiSyllableTiex', [], 'Display', None],
  [ 0xa11f, 'YiSyllableTie', [], 'Display', None],
  [ 0xa120, 'YiSyllableTiep', [], 'Display', None],
  [ 0xa121, 'YiSyllableTat', [], 'Display', None],
  [ 0xa122, 'YiSyllableTax', [], 'Display', None],
  [ 0xa123, 'YiSyllableTa', [], 'Display', None],
  [ 0xa124, 'YiSyllableTap', [], 'Display', None],
  [ 0xa125, 'YiSyllableTuot', [], 'Display', None],
  [ 0xa126, 'YiSyllableTuox', [], 'Display', None],
  [ 0xa127, 'YiSyllableTuo', [], 'Display', None],
  [ 0xa128, 'YiSyllableTuop', [], 'Display', None],
  [ 0xa129, 'YiSyllableTot', [], 'Display', None],
  [ 0xa12a, 'YiSyllableTox', [], 'Display', None],
  [ 0xa12b, 'YiSyllableTo', [], 'Display', None],
  [ 0xa12c, 'YiSyllableTop', [], 'Display', None],
  [ 0xa12d, 'YiSyllableTex', [], 'Display', None],
  [ 0xa12e, 'YiSyllableTe', [], 'Display', None],
  [ 0xa12f, 'YiSyllableTep', [], 'Display', None],
  [ 0xa130, 'YiSyllableTut', [], 'Display', None],
  [ 0xa131, 'YiSyllableTux', [], 'Display', None],
  [ 0xa132, 'YiSyllableTu', [], 'Display', None],
  [ 0xa133, 'YiSyllableTup', [], 'Display', None],
  [ 0xa134, 'YiSyllableTurx', [], 'Display', None],
  [ 0xa135, 'YiSyllableTur', [], 'Display', None],
  [ 0xa136, 'YiSyllableDdit', [], 'Display', None],
  [ 0xa137, 'YiSyllableDdix', [], 'Display', None],
  [ 0xa138, 'YiSyllableDdi', [], 'Display', None],
  [ 0xa139, 'YiSyllableDdip', [], 'Display', None],
  [ 0xa13a, 'YiSyllableDdiex', [], 'Display', None],
  [ 0xa13b, 'YiSyllableDdie', [], 'Display', None],
  [ 0xa13c, 'YiSyllableDdiep', [], 'Display', None],
  [ 0xa13d, 'YiSyllableDdat', [], 'Display', None],
  [ 0xa13e, 'YiSyllableDdax', [], 'Display', None],
  [ 0xa13f, 'YiSyllableDda', [], 'Display', None],
  [ 0xa140, 'YiSyllableDdap', [], 'Display', None],
  [ 0xa141, 'YiSyllableDduox', [], 'Display', None],
  [ 0xa142, 'YiSyllableDduo', [], 'Display', None],
  [ 0xa143, 'YiSyllableDduop', [], 'Display', None],
  [ 0xa144, 'YiSyllableDdot', [], 'Display', None],
  [ 0xa145, 'YiSyllableDdox', [], 'Display', None],
  [ 0xa146, 'YiSyllableDdo', [], 'Display', None],
  [ 0xa147, 'YiSyllableDdop', [], 'Display', None],
  [ 0xa148, 'YiSyllableDdex', [], 'Display', None],
  [ 0xa149, 'YiSyllableDde', [], 'Display', None],
  [ 0xa14a, 'YiSyllableDdep', [], 'Display', None],
  [ 0xa14b, 'YiSyllableDdut', [], 'Display', None],
  [ 0xa14c, 'YiSyllableDdux', [], 'Display', None],
  [ 0xa14d, 'YiSyllableDdu', [], 'Display', None],
  [ 0xa14e, 'YiSyllableDdup', [], 'Display', None],
  [ 0xa14f, 'YiSyllableDdurx', [], 'Display', None],
  [ 0xa150, 'YiSyllableDdur', [], 'Display', None],
  [ 0xa151, 'YiSyllableNdit', [], 'Display', None],
  [ 0xa152, 'YiSyllableNdix', [], 'Display', None],
  [ 0xa153, 'YiSyllableNdi', [], 'Display', None],
  [ 0xa154, 'YiSyllableNdip', [], 'Display', None],
  [ 0xa155, 'YiSyllableNdiex', [], 'Display', None],
  [ 0xa156, 'YiSyllableNdie', [], 'Display', None],
  [ 0xa157, 'YiSyllableNdat', [], 'Display', None],
  [ 0xa158, 'YiSyllableNdax', [], 'Display', None],
  [ 0xa159, 'YiSyllableNda', [], 'Display', None],
  [ 0xa15a, 'YiSyllableNdap', [], 'Display', None],
  [ 0xa15b, 'YiSyllableNdot', [], 'Display', None],
  [ 0xa15c, 'YiSyllableNdox', [], 'Display', None],
  [ 0xa15d, 'YiSyllableNdo', [], 'Display', None],
  [ 0xa15e, 'YiSyllableNdop', [], 'Display', None],
  [ 0xa15f, 'YiSyllableNdex', [], 'Display', None],
  [ 0xa160, 'YiSyllableNde', [], 'Display', None],
  [ 0xa161, 'YiSyllableNdep', [], 'Display', None],
  [ 0xa162, 'YiSyllableNdut', [], 'Display', None],
  [ 0xa163, 'YiSyllableNdux', [], 'Display', None],
  [ 0xa164, 'YiSyllableNdu', [], 'Display', None],
  [ 0xa165, 'YiSyllableNdup', [], 'Display', None],
  [ 0xa166, 'YiSyllableNdurx', [], 'Display', None],
  [ 0xa167, 'YiSyllableNdur', [], 'Display', None],
  [ 0xa168, 'YiSyllableHnit', [], 'Display', None],
  [ 0xa169, 'YiSyllableHnix', [], 'Display', None],
  [ 0xa16a, 'YiSyllableHni', [], 'Display', None],
  [ 0xa16b, 'YiSyllableHnip', [], 'Display', None],
  [ 0xa16c, 'YiSyllableHniet', [], 'Display', None],
  [ 0xa16d, 'YiSyllableHniex', [], 'Display', None],
  [ 0xa16e, 'YiSyllableHnie', [], 'Display', None],
  [ 0xa16f, 'YiSyllableHniep', [], 'Display', None],
  [ 0xa170, 'YiSyllableHnat', [], 'Display', None],
  [ 0xa171, 'YiSyllableHnax', [], 'Display', None],
  [ 0xa172, 'YiSyllableHna', [], 'Display', None],
  [ 0xa173, 'YiSyllableHnap', [], 'Display', None],
  [ 0xa174, 'YiSyllableHnuox', [], 'Display', None],
  [ 0xa175, 'YiSyllableHnuo', [], 'Display', None],
  [ 0xa176, 'YiSyllableHnot', [], 'Display', None],
  [ 0xa177, 'YiSyllableHnox', [], 'Display', None],
  [ 0xa178, 'YiSyllableHnop', [], 'Display', None],
  [ 0xa179, 'YiSyllableHnex', [], 'Display', None],
  [ 0xa17a, 'YiSyllableHne', [], 'Display', None],
  [ 0xa17b, 'YiSyllableHnep', [], 'Display', None],
  [ 0xa17c, 'YiSyllableHnut', [], 'Display', None],
  [ 0xa17d, 'YiSyllableNit', [], 'Display', None],
  [ 0xa17e, 'YiSyllableNix', [], 'Display', None],
  [ 0xa17f, 'YiSyllableNi', [], 'Display', None],
  [ 0xa180, 'YiSyllableNip', [], 'Display', None],
  [ 0xa181, 'YiSyllableNiex', [], 'Display', None],
  [ 0xa182, 'YiSyllableNie', [], 'Display', None],
  [ 0xa183, 'YiSyllableNiep', [], 'Display', None],
  [ 0xa184, 'YiSyllableNax', [], 'Display', None],
  [ 0xa185, 'YiSyllableNa', [], 'Display', None],
  [ 0xa186, 'YiSyllableNap', [], 'Display', None],
  [ 0xa187, 'YiSyllableNuox', [], 'Display', None],
  [ 0xa188, 'YiSyllableNuo', [], 'Display', None],
  [ 0xa189, 'YiSyllableNuop', [], 'Display', None],
  [ 0xa18a, 'YiSyllableNot', [], 'Display', None],
  [ 0xa18b, 'YiSyllableNox', [], 'Display', None],
  [ 0xa18c, 'YiSyllableNo', [], 'Display', None],
  [ 0xa18d, 'YiSyllableNop', [], 'Display', None],
  [ 0xa18e, 'YiSyllableNex', [], 'Display', None],
  [ 0xa18f, 'YiSyllableNe', [], 'Display', None],
  [ 0xa190, 'YiSyllableNep', [], 'Display', None],
  [ 0xa191, 'YiSyllableNut', [], 'Display', None],
  [ 0xa192, 'YiSyllableNux', [], 'Display', None],
  [ 0xa193, 'YiSyllableNu', [], 'Display', None],
  [ 0xa194, 'YiSyllableNup', [], 'Display', None],
  [ 0xa195, 'YiSyllableNurx', [], 'Display', None],
  [ 0xa196, 'YiSyllableNur', [], 'Display', None],
  [ 0xa197, 'YiSyllableHlit', [], 'Display', None],
  [ 0xa198, 'YiSyllableHlix', [], 'Display', None],
  [ 0xa199, 'YiSyllableHli', [], 'Display', None],
  [ 0xa19a, 'YiSyllableHlip', [], 'Display', None],
  [ 0xa19b, 'YiSyllableHliex', [], 'Display', None],
  [ 0xa19c, 'YiSyllableHlie', [], 'Display', None],
  [ 0xa19d, 'YiSyllableHliep', [], 'Display', None],
  [ 0xa19e, 'YiSyllableHlat', [], 'Display', None],
  [ 0xa19f, 'YiSyllableHlax', [], 'Display', None],
  [ 0xa1a0, 'YiSyllableHla', [], 'Display', None],
  [ 0xa1a1, 'YiSyllableHlap', [], 'Display', None],
  [ 0xa1a2, 'YiSyllableHluox', [], 'Display', None],
  [ 0xa1a3, 'YiSyllableHluo', [], 'Display', None],
  [ 0xa1a4, 'YiSyllableHluop', [], 'Display', None],
  [ 0xa1a5, 'YiSyllableHlox', [], 'Display', None],
  [ 0xa1a6, 'YiSyllableHlo', [], 'Display', None],
  [ 0xa1a7, 'YiSyllableHlop', [], 'Display', None],
  [ 0xa1a8, 'YiSyllableHlex', [], 'Display', None],
  [ 0xa1a9, 'YiSyllableHle', [], 'Display', None],
  [ 0xa1aa, 'YiSyllableHlep', [], 'Display', None],
  [ 0xa1ab, 'YiSyllableHlut', [], 'Display', None],
  [ 0xa1ac, 'YiSyllableHlux', [], 'Display', None],
  [ 0xa1ad, 'YiSyllableHlu', [], 'Display', None],
  [ 0xa1ae, 'YiSyllableHlup', [], 'Display', None],
  [ 0xa1af, 'YiSyllableHlurx', [], 'Display', None],
  [ 0xa1b0, 'YiSyllableHlur', [], 'Display', None],
  [ 0xa1b1, 'YiSyllableHlyt', [], 'Display', None],
  [ 0xa1b2, 'YiSyllableHlyx', [], 'Display', None],
  [ 0xa1b3, 'YiSyllableHly', [], 'Display', None],
  [ 0xa1b4, 'YiSyllableHlyp', [], 'Display', None],
  [ 0xa1b5, 'YiSyllableHlyrx', [], 'Display', None],
  [ 0xa1b6, 'YiSyllableHlyr', [], 'Display', None],
  [ 0xa1b7, 'YiSyllableLit', [], 'Display', None],
  [ 0xa1b8, 'YiSyllableLix', [], 'Display', None],
  [ 0xa1b9, 'YiSyllableLi', [], 'Display', None],
  [ 0xa1ba, 'YiSyllableLip', [], 'Display', None],
  [ 0xa1bb, 'YiSyllableLiet', [], 'Display', None],
  [ 0xa1bc, 'YiSyllableLiex', [], 'Display', None],
  [ 0xa1bd, 'YiSyllableLie', [], 'Display', None],
  [ 0xa1be, 'YiSyllableLiep', [], 'Display', None],
  [ 0xa1bf, 'YiSyllableLat', [], 'Display', None],
  [ 0xa1c0, 'YiSyllableLax', [], 'Display', None],
  [ 0xa1c1, 'YiSyllableLa', [], 'Display', None],
  [ 0xa1c2, 'YiSyllableLap', [], 'Display', None],
  [ 0xa1c3, 'YiSyllableLuot', [], 'Display', None],
  [ 0xa1c4, 'YiSyllableLuox', [], 'Display', None],
  [ 0xa1c5, 'YiSyllableLuo', [], 'Display', None],
  [ 0xa1c6, 'YiSyllableLuop', [], 'Display', None],
  [ 0xa1c7, 'YiSyllableLot', [], 'Display', None],
  [ 0xa1c8, 'YiSyllableLox', [], 'Display', None],
  [ 0xa1c9, 'YiSyllableLo', [], 'Display', None],
  [ 0xa1ca, 'YiSyllableLop', [], 'Display', None],
  [ 0xa1cb, 'YiSyllableLex', [], 'Display', None],
  [ 0xa1cc, 'YiSyllableLe', [], 'Display', None],
  [ 0xa1cd, 'YiSyllableLep', [], 'Display', None],
  [ 0xa1ce, 'YiSyllableLut', [], 'Display', None],
  [ 0xa1cf, 'YiSyllableLux', [], 'Display', None],
  [ 0xa1d0, 'YiSyllableLu', [], 'Display', None],
  [ 0xa1d1, 'YiSyllableLup', [], 'Display', None],
  [ 0xa1d2, 'YiSyllableLurx', [], 'Display', None],
  [ 0xa1d3, 'YiSyllableLur', [], 'Display', None],
  [ 0xa1d4, 'YiSyllableLyt', [], 'Display', None],
  [ 0xa1d5, 'YiSyllableLyx', [], 'Display', None],
  [ 0xa1d6, 'YiSyllableLy', [], 'Display', None],
  [ 0xa1d7, 'YiSyllableLyp', [], 'Display', None],
  [ 0xa1d8, 'YiSyllableLyrx', [], 'Display', None],
  [ 0xa1d9, 'YiSyllableLyr', [], 'Display', None],
  [ 0xa1da, 'YiSyllableGit', [], 'Display', None],
  [ 0xa1db, 'YiSyllableGix', [], 'Display', None],
  [ 0xa1dc, 'YiSyllableGi', [], 'Display', None],
  [ 0xa1dd, 'YiSyllableGip', [], 'Display', None],
  [ 0xa1de, 'YiSyllableGiet', [], 'Display', None],
  [ 0xa1df, 'YiSyllableGiex', [], 'Display', None],
  [ 0xa1e0, 'YiSyllableGie', [], 'Display', None],
  [ 0xa1e1, 'YiSyllableGiep', [], 'Display', None],
  [ 0xa1e2, 'YiSyllableGat', [], 'Display', None],
  [ 0xa1e3, 'YiSyllableGax', [], 'Display', None],
  [ 0xa1e4, 'YiSyllableGa', [], 'Display', None],
  [ 0xa1e5, 'YiSyllableGap', [], 'Display', None],
  [ 0xa1e6, 'YiSyllableGuot', [], 'Display', None],
  [ 0xa1e7, 'YiSyllableGuox', [], 'Display', None],
  [ 0xa1e8, 'YiSyllableGuo', [], 'Display', None],
  [ 0xa1e9, 'YiSyllableGuop', [], 'Display', None],
  [ 0xa1ea, 'YiSyllableGot', [], 'Display', None],
  [ 0xa1eb, 'YiSyllableGox', [], 'Display', None],
  [ 0xa1ec, 'YiSyllableGo', [], 'Display', None],
  [ 0xa1ed, 'YiSyllableGop', [], 'Display', None],
  [ 0xa1ee, 'YiSyllableGet', [], 'Display', None],
  [ 0xa1ef, 'YiSyllableGex', [], 'Display', None],
  [ 0xa1f0, 'YiSyllableGe', [], 'Display', None],
  [ 0xa1f1, 'YiSyllableGep', [], 'Display', None],
  [ 0xa1f2, 'YiSyllableGut', [], 'Display', None],
  [ 0xa1f3, 'YiSyllableGux', [], 'Display', None],
  [ 0xa1f4, 'YiSyllableGu', [], 'Display', None],
  [ 0xa1f5, 'YiSyllableGup', [], 'Display', None],
  [ 0xa1f6, 'YiSyllableGurx', [], 'Display', None],
  [ 0xa1f7, 'YiSyllableGur', [], 'Display', None],
  [ 0xa1f8, 'YiSyllableKit', [], 'Display', None],
  [ 0xa1f9, 'YiSyllableKix', [], 'Display', None],
  [ 0xa1fa, 'YiSyllableKi', [], 'Display', None],
  [ 0xa1fb, 'YiSyllableKip', [], 'Display', None],
  [ 0xa1fc, 'YiSyllableKiex', [], 'Display', None],
  [ 0xa1fd, 'YiSyllableKie', [], 'Display', None],
  [ 0xa1fe, 'YiSyllableKiep', [], 'Display', None],
  [ 0xa1ff, 'YiSyllableKat', [], 'Display', None],
  [ 0xa200, 'YiSyllableKax', [], 'Display', None],
  [ 0xa201, 'YiSyllableKa', [], 'Display', None],
  [ 0xa202, 'YiSyllableKap', [], 'Display', None],
  [ 0xa203, 'YiSyllableKuox', [], 'Display', None],
  [ 0xa204, 'YiSyllableKuo', [], 'Display', None],
  [ 0xa205, 'YiSyllableKuop', [], 'Display', None],
  [ 0xa206, 'YiSyllableKot', [], 'Display', None],
  [ 0xa207, 'YiSyllableKox', [], 'Display', None],
  [ 0xa208, 'YiSyllableKo', [], 'Display', None],
  [ 0xa209, 'YiSyllableKop', [], 'Display', None],
  [ 0xa20a, 'YiSyllableKet', [], 'Display', None],
  [ 0xa20b, 'YiSyllableKex', [], 'Display', None],
  [ 0xa20c, 'YiSyllableKe', [], 'Display', None],
  [ 0xa20d, 'YiSyllableKep', [], 'Display', None],
  [ 0xa20e, 'YiSyllableKut', [], 'Display', None],
  [ 0xa20f, 'YiSyllableKux', [], 'Display', None],
  [ 0xa210, 'YiSyllableKu', [], 'Display', None],
  [ 0xa211, 'YiSyllableKup', [], 'Display', None],
  [ 0xa212, 'YiSyllableKurx', [], 'Display', None],
  [ 0xa213, 'YiSyllableKur', [], 'Display', None],
  [ 0xa214, 'YiSyllableGgit', [], 'Display', None],
  [ 0xa215, 'YiSyllableGgix', [], 'Display', None],
  [ 0xa216, 'YiSyllableGgi', [], 'Display', None],
  [ 0xa217, 'YiSyllableGgiex', [], 'Display', None],
  [ 0xa218, 'YiSyllableGgie', [], 'Display', None],
  [ 0xa219, 'YiSyllableGgiep', [], 'Display', None],
  [ 0xa21a, 'YiSyllableGgat', [], 'Display', None],
  [ 0xa21b, 'YiSyllableGgax', [], 'Display', None],
  [ 0xa21c, 'YiSyllableGga', [], 'Display', None],
  [ 0xa21d, 'YiSyllableGgap', [], 'Display', None],
  [ 0xa21e, 'YiSyllableGguot', [], 'Display', None],
  [ 0xa21f, 'YiSyllableGguox', [], 'Display', None],
  [ 0xa220, 'YiSyllableGguo', [], 'Display', None],
  [ 0xa221, 'YiSyllableGguop', [], 'Display', None],
  [ 0xa222, 'YiSyllableGgot', [], 'Display', None],
  [ 0xa223, 'YiSyllableGgox', [], 'Display', None],
  [ 0xa224, 'YiSyllableGgo', [], 'Display', None],
  [ 0xa225, 'YiSyllableGgop', [], 'Display', None],
  [ 0xa226, 'YiSyllableGget', [], 'Display', None],
  [ 0xa227, 'YiSyllableGgex', [], 'Display', None],
  [ 0xa228, 'YiSyllableGge', [], 'Display', None],
  [ 0xa229, 'YiSyllableGgep', [], 'Display', None],
  [ 0xa22a, 'YiSyllableGgut', [], 'Display', None],
  [ 0xa22b, 'YiSyllableGgux', [], 'Display', None],
  [ 0xa22c, 'YiSyllableGgu', [], 'Display', None],
  [ 0xa22d, 'YiSyllableGgup', [], 'Display', None],
  [ 0xa22e, 'YiSyllableGgurx', [], 'Display', None],
  [ 0xa22f, 'YiSyllableGgur', [], 'Display', None],
  [ 0xa230, 'YiSyllableMgiex', [], 'Display', None],
  [ 0xa231, 'YiSyllableMgie', [], 'Display', None],
  [ 0xa232, 'YiSyllableMgat', [], 'Display', None],
  [ 0xa233, 'YiSyllableMgax', [], 'Display', None],
  [ 0xa234, 'YiSyllableMga', [], 'Display', None],
  [ 0xa235, 'YiSyllableMgap', [], 'Display', None],
  [ 0xa236, 'YiSyllableMguox', [], 'Display', None],
  [ 0xa237, 'YiSyllableMguo', [], 'Display', None],
  [ 0xa238, 'YiSyllableMguop', [], 'Display', None],
  [ 0xa239, 'YiSyllableMgot', [], 'Display', None],
  [ 0xa23a, 'YiSyllableMgox', [], 'Display', None],
  [ 0xa23b, 'YiSyllableMgo', [], 'Display', None],
  [ 0xa23c, 'YiSyllableMgop', [], 'Display', None],
  [ 0xa23d, 'YiSyllableMgex', [], 'Display', None],
  [ 0xa23e, 'YiSyllableMge', [], 'Display', None],
  [ 0xa23f, 'YiSyllableMgep', [], 'Display', None],
  [ 0xa240, 'YiSyllableMgut', [], 'Display', None],
  [ 0xa241, 'YiSyllableMgux', [], 'Display', None],
  [ 0xa242, 'YiSyllableMgu', [], 'Display', None],
  [ 0xa243, 'YiSyllableMgup', [], 'Display', None],
  [ 0xa244, 'YiSyllableMgurx', [], 'Display', None],
  [ 0xa245, 'YiSyllableMgur', [], 'Display', None],
  [ 0xa246, 'YiSyllableHxit', [], 'Display', None],
  [ 0xa247, 'YiSyllableHxix', [], 'Display', None],
  [ 0xa248, 'YiSyllableHxi', [], 'Display', None],
  [ 0xa249, 'YiSyllableHxip', [], 'Display', None],
  [ 0xa24a, 'YiSyllableHxiet', [], 'Display', None],
  [ 0xa24b, 'YiSyllableHxiex', [], 'Display', None],
  [ 0xa24c, 'YiSyllableHxie', [], 'Display', None],
  [ 0xa24d, 'YiSyllableHxiep', [], 'Display', None],
  [ 0xa24e, 'YiSyllableHxat', [], 'Display', None],
  [ 0xa24f, 'YiSyllableHxax', [], 'Display', None],
  [ 0xa250, 'YiSyllableHxa', [], 'Display', None],
  [ 0xa251, 'YiSyllableHxap', [], 'Display', None],
  [ 0xa252, 'YiSyllableHxuot', [], 'Display', None],
  [ 0xa253, 'YiSyllableHxuox', [], 'Display', None],
  [ 0xa254, 'YiSyllableHxuo', [], 'Display', None],
  [ 0xa255, 'YiSyllableHxuop', [], 'Display', None],
  [ 0xa256, 'YiSyllableHxot', [], 'Display', None],
  [ 0xa257, 'YiSyllableHxox', [], 'Display', None],
  [ 0xa258, 'YiSyllableHxo', [], 'Display', None],
  [ 0xa259, 'YiSyllableHxop', [], 'Display', None],
  [ 0xa25a, 'YiSyllableHxex', [], 'Display', None],
  [ 0xa25b, 'YiSyllableHxe', [], 'Display', None],
  [ 0xa25c, 'YiSyllableHxep', [], 'Display', None],
  [ 0xa25d, 'YiSyllableNgiex', [], 'Display', None],
  [ 0xa25e, 'YiSyllableNgie', [], 'Display', None],
  [ 0xa25f, 'YiSyllableNgiep', [], 'Display', None],
  [ 0xa260, 'YiSyllableNgat', [], 'Display', None],
  [ 0xa261, 'YiSyllableNgax', [], 'Display', None],
  [ 0xa262, 'YiSyllableNga', [], 'Display', None],
  [ 0xa263, 'YiSyllableNgap', [], 'Display', None],
  [ 0xa264, 'YiSyllableNguot', [], 'Display', None],
  [ 0xa265, 'YiSyllableNguox', [], 'Display', None],
  [ 0xa266, 'YiSyllableNguo', [], 'Display', None],
  [ 0xa267, 'YiSyllableNgot', [], 'Display', None],
  [ 0xa268, 'YiSyllableNgox', [], 'Display', None],
  [ 0xa269, 'YiSyllableNgo', [], 'Display', None],
  [ 0xa26a, 'YiSyllableNgop', [], 'Display', None],
  [ 0xa26b, 'YiSyllableNgex', [], 'Display', None],
  [ 0xa26c, 'YiSyllableNge', [], 'Display', None],
  [ 0xa26d, 'YiSyllableNgep', [], 'Display', None],
  [ 0xa26e, 'YiSyllableHit', [], 'Display', None],
  [ 0xa26f, 'YiSyllableHiex', [], 'Display', None],
  [ 0xa270, 'YiSyllableHie', [], 'Display', None],
  [ 0xa271, 'YiSyllableHat', [], 'Display', None],
  [ 0xa272, 'YiSyllableHax', [], 'Display', None],
  [ 0xa273, 'YiSyllableHa', [], 'Display', None],
  [ 0xa274, 'YiSyllableHap', [], 'Display', None],
  [ 0xa275, 'YiSyllableHuot', [], 'Display', None],
  [ 0xa276, 'YiSyllableHuox', [], 'Display', None],
  [ 0xa277, 'YiSyllableHuo', [], 'Display', None],
  [ 0xa278, 'YiSyllableHuop', [], 'Display', None],
  [ 0xa279, 'YiSyllableHot', [], 'Display', None],
  [ 0xa27a, 'YiSyllableHox', [], 'Display', None],
  [ 0xa27b, 'YiSyllableHo', [], 'Display', None],
  [ 0xa27c, 'YiSyllableHop', [], 'Display', None],
  [ 0xa27d, 'YiSyllableHex', [], 'Display', None],
  [ 0xa27e, 'YiSyllableHe', [], 'Display', None],
  [ 0xa27f, 'YiSyllableHep', [], 'Display', None],
  [ 0xa280, 'YiSyllableWat', [], 'Display', None],
  [ 0xa281, 'YiSyllableWax', [], 'Display', None],
  [ 0xa282, 'YiSyllableWa', [], 'Display', None],
  [ 0xa283, 'YiSyllableWap', [], 'Display', None],
  [ 0xa284, 'YiSyllableWuox', [], 'Display', None],
  [ 0xa285, 'YiSyllableWuo', [], 'Display', None],
  [ 0xa286, 'YiSyllableWuop', [], 'Display', None],
  [ 0xa287, 'YiSyllableWox', [], 'Display', None],
  [ 0xa288, 'YiSyllableWo', [], 'Display', None],
  [ 0xa289, 'YiSyllableWop', [], 'Display', None],
  [ 0xa28a, 'YiSyllableWex', [], 'Display', None],
  [ 0xa28b, 'YiSyllableWe', [], 'Display', None],
  [ 0xa28c, 'YiSyllableWep', [], 'Display', None],
  [ 0xa28d, 'YiSyllableZit', [], 'Display', None],
  [ 0xa28e, 'YiSyllableZix', [], 'Display', None],
  [ 0xa28f, 'YiSyllableZi', [], 'Display', None],
  [ 0xa290, 'YiSyllableZip', [], 'Display', None],
  [ 0xa291, 'YiSyllableZiex', [], 'Display', None],
  [ 0xa292, 'YiSyllableZie', [], 'Display', None],
  [ 0xa293, 'YiSyllableZiep', [], 'Display', None],
  [ 0xa294, 'YiSyllableZat', [], 'Display', None],
  [ 0xa295, 'YiSyllableZax', [], 'Display', None],
  [ 0xa296, 'YiSyllableZa', [], 'Display', None],
  [ 0xa297, 'YiSyllableZap', [], 'Display', None],
  [ 0xa298, 'YiSyllableZuox', [], 'Display', None],
  [ 0xa299, 'YiSyllableZuo', [], 'Display', None],
  [ 0xa29a, 'YiSyllableZuop', [], 'Display', None],
  [ 0xa29b, 'YiSyllableZot', [], 'Display', None],
  [ 0xa29c, 'YiSyllableZox', [], 'Display', None],
  [ 0xa29d, 'YiSyllableZo', [], 'Display', None],
  [ 0xa29e, 'YiSyllableZop', [], 'Display', None],
  [ 0xa29f, 'YiSyllableZex', [], 'Display', None],
  [ 0xa2a0, 'YiSyllableZe', [], 'Display', None],
  [ 0xa2a1, 'YiSyllableZep', [], 'Display', None],
  [ 0xa2a2, 'YiSyllableZut', [], 'Display', None],
  [ 0xa2a3, 'YiSyllableZux', [], 'Display', None],
  [ 0xa2a4, 'YiSyllableZu', [], 'Display', None],
  [ 0xa2a5, 'YiSyllableZup', [], 'Display', None],
  [ 0xa2a6, 'YiSyllableZurx', [], 'Display', None],
  [ 0xa2a7, 'YiSyllableZur', [], 'Display', None],
  [ 0xa2a8, 'YiSyllableZyt', [], 'Display', None],
  [ 0xa2a9, 'YiSyllableZyx', [], 'Display', None],
  [ 0xa2aa, 'YiSyllableZy', [], 'Display', None],
  [ 0xa2ab, 'YiSyllableZyp', [], 'Display', None],
  [ 0xa2ac, 'YiSyllableZyrx', [], 'Display', None],
  [ 0xa2ad, 'YiSyllableZyr', [], 'Display', None],
  [ 0xa2ae, 'YiSyllableCit', [], 'Display', None],
  [ 0xa2af, 'YiSyllableCix', [], 'Display', None],
  [ 0xa2b0, 'YiSyllableCi', [], 'Display', None],
  [ 0xa2b1, 'YiSyllableCip', [], 'Display', None],
  [ 0xa2b2, 'YiSyllableCiet', [], 'Display', None],
  [ 0xa2b3, 'YiSyllableCiex', [], 'Display', None],
  [ 0xa2b4, 'YiSyllableCie', [], 'Display', None],
  [ 0xa2b5, 'YiSyllableCiep', [], 'Display', None],
  [ 0xa2b6, 'YiSyllableCat', [], 'Display', None],
  [ 0xa2b7, 'YiSyllableCax', [], 'Display', None],
  [ 0xa2b8, 'YiSyllableCa', [], 'Display', None],
  [ 0xa2b9, 'YiSyllableCap', [], 'Display', None],
  [ 0xa2ba, 'YiSyllableCuox', [], 'Display', None],
  [ 0xa2bb, 'YiSyllableCuo', [], 'Display', None],
  [ 0xa2bc, 'YiSyllableCuop', [], 'Display', None],
  [ 0xa2bd, 'YiSyllableCot', [], 'Display', None],
  [ 0xa2be, 'YiSyllableCox', [], 'Display', None],
  [ 0xa2bf, 'YiSyllableCo', [], 'Display', None],
  [ 0xa2c0, 'YiSyllableCop', [], 'Display', None],
  [ 0xa2c1, 'YiSyllableCex', [], 'Display', None],
  [ 0xa2c2, 'YiSyllableCe', [], 'Display', None],
  [ 0xa2c3, 'YiSyllableCep', [], 'Display', None],
  [ 0xa2c4, 'YiSyllableCut', [], 'Display', None],
  [ 0xa2c5, 'YiSyllableCux', [], 'Display', None],
  [ 0xa2c6, 'YiSyllableCu', [], 'Display', None],
  [ 0xa2c7, 'YiSyllableCup', [], 'Display', None],
  [ 0xa2c8, 'YiSyllableCurx', [], 'Display', None],
  [ 0xa2c9, 'YiSyllableCur', [], 'Display', None],
  [ 0xa2ca, 'YiSyllableCyt', [], 'Display', None],
  [ 0xa2cb, 'YiSyllableCyx', [], 'Display', None],
  [ 0xa2cc, 'YiSyllableCy', [], 'Display', None],
  [ 0xa2cd, 'YiSyllableCyp', [], 'Display', None],
  [ 0xa2ce, 'YiSyllableCyrx', [], 'Display', None],
  [ 0xa2cf, 'YiSyllableCyr', [], 'Display', None],
  [ 0xa2d0, 'YiSyllableZzit', [], 'Display', None],
  [ 0xa2d1, 'YiSyllableZzix', [], 'Display', None],
  [ 0xa2d2, 'YiSyllableZzi', [], 'Display', None],
  [ 0xa2d3, 'YiSyllableZzip', [], 'Display', None],
  [ 0xa2d4, 'YiSyllableZziet', [], 'Display', None],
  [ 0xa2d5, 'YiSyllableZziex', [], 'Display', None],
  [ 0xa2d6, 'YiSyllableZzie', [], 'Display', None],
  [ 0xa2d7, 'YiSyllableZziep', [], 'Display', None],
  [ 0xa2d8, 'YiSyllableZzat', [], 'Display', None],
  [ 0xa2d9, 'YiSyllableZzax', [], 'Display', None],
  [ 0xa2da, 'YiSyllableZza', [], 'Display', None],
  [ 0xa2db, 'YiSyllableZzap', [], 'Display', None],
  [ 0xa2dc, 'YiSyllableZzox', [], 'Display', None],
  [ 0xa2dd, 'YiSyllableZzo', [], 'Display', None],
  [ 0xa2de, 'YiSyllableZzop', [], 'Display', None],
  [ 0xa2df, 'YiSyllableZzex', [], 'Display', None],
  [ 0xa2e0, 'YiSyllableZze', [], 'Display', None],
  [ 0xa2e1, 'YiSyllableZzep', [], 'Display', None],
  [ 0xa2e2, 'YiSyllableZzux', [], 'Display', None],
  [ 0xa2e3, 'YiSyllableZzu', [], 'Display', None],
  [ 0xa2e4, 'YiSyllableZzup', [], 'Display', None],
  [ 0xa2e5, 'YiSyllableZzurx', [], 'Display', None],
  [ 0xa2e6, 'YiSyllableZzur', [], 'Display', None],
  [ 0xa2e7, 'YiSyllableZzyt', [], 'Display', None],
  [ 0xa2e8, 'YiSyllableZzyx', [], 'Display', None],
  [ 0xa2e9, 'YiSyllableZzy', [], 'Display', None],
  [ 0xa2ea, 'YiSyllableZzyp', [], 'Display', None],
  [ 0xa2eb, 'YiSyllableZzyrx', [], 'Display', None],
  [ 0xa2ec, 'YiSyllableZzyr', [], 'Display', None],
  [ 0xa2ed, 'YiSyllableNzit', [], 'Display', None],
  [ 0xa2ee, 'YiSyllableNzix', [], 'Display', None],
  [ 0xa2ef, 'YiSyllableNzi', [], 'Display', None],
  [ 0xa2f0, 'YiSyllableNzip', [], 'Display', None],
  [ 0xa2f1, 'YiSyllableNziex', [], 'Display', None],
  [ 0xa2f2, 'YiSyllableNzie', [], 'Display', None],
  [ 0xa2f3, 'YiSyllableNziep', [], 'Display', None],
  [ 0xa2f4, 'YiSyllableNzat', [], 'Display', None],
  [ 0xa2f5, 'YiSyllableNzax', [], 'Display', None],
  [ 0xa2f6, 'YiSyllableNza', [], 'Display', None],
  [ 0xa2f7, 'YiSyllableNzap', [], 'Display', None],
  [ 0xa2f8, 'YiSyllableNzuox', [], 'Display', None],
  [ 0xa2f9, 'YiSyllableNzuo', [], 'Display', None],
  [ 0xa2fa, 'YiSyllableNzox', [], 'Display', None],
  [ 0xa2fb, 'YiSyllableNzop', [], 'Display', None],
  [ 0xa2fc, 'YiSyllableNzex', [], 'Display', None],
  [ 0xa2fd, 'YiSyllableNze', [], 'Display', None],
  [ 0xa2fe, 'YiSyllableNzux', [], 'Display', None],
  [ 0xa2ff, 'YiSyllableNzu', [], 'Display', None],
  [ 0xa300, 'YiSyllableNzup', [], 'Display', None],
  [ 0xa301, 'YiSyllableNzurx', [], 'Display', None],
  [ 0xa302, 'YiSyllableNzur', [], 'Display', None],
  [ 0xa303, 'YiSyllableNzyt', [], 'Display', None],
  [ 0xa304, 'YiSyllableNzyx', [], 'Display', None],
  [ 0xa305, 'YiSyllableNzy', [], 'Display', None],
  [ 0xa306, 'YiSyllableNzyp', [], 'Display', None],
  [ 0xa307, 'YiSyllableNzyrx', [], 'Display', None],
  [ 0xa308, 'YiSyllableNzyr', [], 'Display', None],
  [ 0xa309, 'YiSyllableSit', [], 'Display', None],
  [ 0xa30a, 'YiSyllableSix', [], 'Display', None],
  [ 0xa30b, 'YiSyllableSi', [], 'Display', None],
  [ 0xa30c, 'YiSyllableSip', [], 'Display', None],
  [ 0xa30d, 'YiSyllableSiex', [], 'Display', None],
  [ 0xa30e, 'YiSyllableSie', [], 'Display', None],
  [ 0xa30f, 'YiSyllableSiep', [], 'Display', None],
  [ 0xa310, 'YiSyllableSat', [], 'Display', None],
  [ 0xa311, 'YiSyllableSax', [], 'Display', None],
  [ 0xa312, 'YiSyllableSa', [], 'Display', None],
  [ 0xa313, 'YiSyllableSap', [], 'Display', None],
  [ 0xa314, 'YiSyllableSuox', [], 'Display', None],
  [ 0xa315, 'YiSyllableSuo', [], 'Display', None],
  [ 0xa316, 'YiSyllableSuop', [], 'Display', None],
  [ 0xa317, 'YiSyllableSot', [], 'Display', None],
  [ 0xa318, 'YiSyllableSox', [], 'Display', None],
  [ 0xa319, 'YiSyllableSo', [], 'Display', None],
  [ 0xa31a, 'YiSyllableSop', [], 'Display', None],
  [ 0xa31b, 'YiSyllableSex', [], 'Display', None],
  [ 0xa31c, 'YiSyllableSe', [], 'Display', None],
  [ 0xa31d, 'YiSyllableSep', [], 'Display', None],
  [ 0xa31e, 'YiSyllableSut', [], 'Display', None],
  [ 0xa31f, 'YiSyllableSux', [], 'Display', None],
  [ 0xa320, 'YiSyllableSu', [], 'Display', None],
  [ 0xa321, 'YiSyllableSup', [], 'Display', None],
  [ 0xa322, 'YiSyllableSurx', [], 'Display', None],
  [ 0xa323, 'YiSyllableSur', [], 'Display', None],
  [ 0xa324, 'YiSyllableSyt', [], 'Display', None],
  [ 0xa325, 'YiSyllableSyx', [], 'Display', None],
  [ 0xa326, 'YiSyllableSy', [], 'Display', None],
  [ 0xa327, 'YiSyllableSyp', [], 'Display', None],
  [ 0xa328, 'YiSyllableSyrx', [], 'Display', None],
  [ 0xa329, 'YiSyllableSyr', [], 'Display', None],
  [ 0xa32a, 'YiSyllableSsit', [], 'Display', None],
  [ 0xa32b, 'YiSyllableSsix', [], 'Display', None],
  [ 0xa32c, 'YiSyllableSsi', [], 'Display', None],
  [ 0xa32d, 'YiSyllableSsip', [], 'Display', None],
  [ 0xa32e, 'YiSyllableSsiex', [], 'Display', None],
  [ 0xa32f, 'YiSyllableSsie', [], 'Display', None],
  [ 0xa330, 'YiSyllableSsiep', [], 'Display', None],
  [ 0xa331, 'YiSyllableSsat', [], 'Display', None],
  [ 0xa332, 'YiSyllableSsax', [], 'Display', None],
  [ 0xa333, 'YiSyllableSsa', [], 'Display', None],
  [ 0xa334, 'YiSyllableSsap', [], 'Display', None],
  [ 0xa335, 'YiSyllableSsot', [], 'Display', None],
  [ 0xa336, 'YiSyllableSsox', [], 'Display', None],
  [ 0xa337, 'YiSyllableSso', [], 'Display', None],
  [ 0xa338, 'YiSyllableSsop', [], 'Display', None],
  [ 0xa339, 'YiSyllableSsex', [], 'Display', None],
  [ 0xa33a, 'YiSyllableSse', [], 'Display', None],
  [ 0xa33b, 'YiSyllableSsep', [], 'Display', None],
  [ 0xa33c, 'YiSyllableSsut', [], 'Display', None],
  [ 0xa33d, 'YiSyllableSsux', [], 'Display', None],
  [ 0xa33e, 'YiSyllableSsu', [], 'Display', None],
  [ 0xa33f, 'YiSyllableSsup', [], 'Display', None],
  [ 0xa340, 'YiSyllableSsyt', [], 'Display', None],
  [ 0xa341, 'YiSyllableSsyx', [], 'Display', None],
  [ 0xa342, 'YiSyllableSsy', [], 'Display', None],
  [ 0xa343, 'YiSyllableSsyp', [], 'Display', None],
  [ 0xa344, 'YiSyllableSsyrx', [], 'Display', None],
  [ 0xa345, 'YiSyllableSsyr', [], 'Display', None],
  [ 0xa346, 'YiSyllableZhat', [], 'Display', None],
  [ 0xa347, 'YiSyllableZhax', [], 'Display', None],
  [ 0xa348, 'YiSyllableZha', [], 'Display', None],
  [ 0xa349, 'YiSyllableZhap', [], 'Display', None],
  [ 0xa34a, 'YiSyllableZhuox', [], 'Display', None],
  [ 0xa34b, 'YiSyllableZhuo', [], 'Display', None],
  [ 0xa34c, 'YiSyllableZhuop', [], 'Display', None],
  [ 0xa34d, 'YiSyllableZhot', [], 'Display', None],
  [ 0xa34e, 'YiSyllableZhox', [], 'Display', None],
  [ 0xa34f, 'YiSyllableZho', [], 'Display', None],
  [ 0xa350, 'YiSyllableZhop', [], 'Display', None],
  [ 0xa351, 'YiSyllableZhet', [], 'Display', None],
  [ 0xa352, 'YiSyllableZhex', [], 'Display', None],
  [ 0xa353, 'YiSyllableZhe', [], 'Display', None],
  [ 0xa354, 'YiSyllableZhep', [], 'Display', None],
  [ 0xa355, 'YiSyllableZhut', [], 'Display', None],
  [ 0xa356, 'YiSyllableZhux', [], 'Display', None],
  [ 0xa357, 'YiSyllableZhu', [], 'Display', None],
  [ 0xa358, 'YiSyllableZhup', [], 'Display', None],
  [ 0xa359, 'YiSyllableZhurx', [], 'Display', None],
  [ 0xa35a, 'YiSyllableZhur', [], 'Display', None],
  [ 0xa35b, 'YiSyllableZhyt', [], 'Display', None],
  [ 0xa35c, 'YiSyllableZhyx', [], 'Display', None],
  [ 0xa35d, 'YiSyllableZhy', [], 'Display', None],
  [ 0xa35e, 'YiSyllableZhyp', [], 'Display', None],
  [ 0xa35f, 'YiSyllableZhyrx', [], 'Display', None],
  [ 0xa360, 'YiSyllableZhyr', [], 'Display', None],
  [ 0xa361, 'YiSyllableChat', [], 'Display', None],
  [ 0xa362, 'YiSyllableChax', [], 'Display', None],
  [ 0xa363, 'YiSyllableCha', [], 'Display', None],
  [ 0xa364, 'YiSyllableChap', [], 'Display', None],
  [ 0xa365, 'YiSyllableChuot', [], 'Display', None],
  [ 0xa366, 'YiSyllableChuox', [], 'Display', None],
  [ 0xa367, 'YiSyllableChuo', [], 'Display', None],
  [ 0xa368, 'YiSyllableChuop', [], 'Display', None],
  [ 0xa369, 'YiSyllableChot', [], 'Display', None],
  [ 0xa36a, 'YiSyllableChox', [], 'Display', None],
  [ 0xa36b, 'YiSyllableCho', [], 'Display', None],
  [ 0xa36c, 'YiSyllableChop', [], 'Display', None],
  [ 0xa36d, 'YiSyllableChet', [], 'Display', None],
  [ 0xa36e, 'YiSyllableChex', [], 'Display', None],
  [ 0xa36f, 'YiSyllableChe', [], 'Display', None],
  [ 0xa370, 'YiSyllableChep', [], 'Display', None],
  [ 0xa371, 'YiSyllableChux', [], 'Display', None],
  [ 0xa372, 'YiSyllableChu', [], 'Display', None],
  [ 0xa373, 'YiSyllableChup', [], 'Display', None],
  [ 0xa374, 'YiSyllableChurx', [], 'Display', None],
  [ 0xa375, 'YiSyllableChur', [], 'Display', None],
  [ 0xa376, 'YiSyllableChyt', [], 'Display', None],
  [ 0xa377, 'YiSyllableChyx', [], 'Display', None],
  [ 0xa378, 'YiSyllableChy', [], 'Display', None],
  [ 0xa379, 'YiSyllableChyp', [], 'Display', None],
  [ 0xa37a, 'YiSyllableChyrx', [], 'Display', None],
  [ 0xa37b, 'YiSyllableChyr', [], 'Display', None],
  [ 0xa37c, 'YiSyllableRrax', [], 'Display', None],
  [ 0xa37d, 'YiSyllableRra', [], 'Display', None],
  [ 0xa37e, 'YiSyllableRruox', [], 'Display', None],
  [ 0xa37f, 'YiSyllableRruo', [], 'Display', None],
  [ 0xa380, 'YiSyllableRrot', [], 'Display', None],
  [ 0xa381, 'YiSyllableRrox', [], 'Display', None],
  [ 0xa382, 'YiSyllableRro', [], 'Display', None],
  [ 0xa383, 'YiSyllableRrop', [], 'Display', None],
  [ 0xa384, 'YiSyllableRret', [], 'Display', None],
  [ 0xa385, 'YiSyllableRrex', [], 'Display', None],
  [ 0xa386, 'YiSyllableRre', [], 'Display', None],
  [ 0xa387, 'YiSyllableRrep', [], 'Display', None],
  [ 0xa388, 'YiSyllableRrut', [], 'Display', None],
  [ 0xa389, 'YiSyllableRrux', [], 'Display', None],
  [ 0xa38a, 'YiSyllableRru', [], 'Display', None],
  [ 0xa38b, 'YiSyllableRrup', [], 'Display', None],
  [ 0xa38c, 'YiSyllableRrurx', [], 'Display', None],
  [ 0xa38d, 'YiSyllableRrur', [], 'Display', None],
  [ 0xa38e, 'YiSyllableRryt', [], 'Display', None],
  [ 0xa38f, 'YiSyllableRryx', [], 'Display', None],
  [ 0xa390, 'YiSyllableRry', [], 'Display', None],
  [ 0xa391, 'YiSyllableRryp', [], 'Display', None],
  [ 0xa392, 'YiSyllableRryrx', [], 'Display', None],
  [ 0xa393, 'YiSyllableRryr', [], 'Display', None],
  [ 0xa394, 'YiSyllableNrat', [], 'Display', None],
  [ 0xa395, 'YiSyllableNrax', [], 'Display', None],
  [ 0xa396, 'YiSyllableNra', [], 'Display', None],
  [ 0xa397, 'YiSyllableNrap', [], 'Display', None],
  [ 0xa398, 'YiSyllableNrox', [], 'Display', None],
  [ 0xa399, 'YiSyllableNro', [], 'Display', None],
  [ 0xa39a, 'YiSyllableNrop', [], 'Display', None],
  [ 0xa39b, 'YiSyllableNret', [], 'Display', None],
  [ 0xa39c, 'YiSyllableNrex', [], 'Display', None],
  [ 0xa39d, 'YiSyllableNre', [], 'Display', None],
  [ 0xa39e, 'YiSyllableNrep', [], 'Display', None],
  [ 0xa39f, 'YiSyllableNrut', [], 'Display', None],
  [ 0xa3a0, 'YiSyllableNrux', [], 'Display', None],
  [ 0xa3a1, 'YiSyllableNru', [], 'Display', None],
  [ 0xa3a2, 'YiSyllableNrup', [], 'Display', None],
  [ 0xa3a3, 'YiSyllableNrurx', [], 'Display', None],
  [ 0xa3a4, 'YiSyllableNrur', [], 'Display', None],
  [ 0xa3a5, 'YiSyllableNryt', [], 'Display', None],
  [ 0xa3a6, 'YiSyllableNryx', [], 'Display', None],
  [ 0xa3a7, 'YiSyllableNry', [], 'Display', None],
  [ 0xa3a8, 'YiSyllableNryp', [], 'Display', None],
  [ 0xa3a9, 'YiSyllableNryrx', [], 'Display', None],
  [ 0xa3aa, 'YiSyllableNryr', [], 'Display', None],
  [ 0xa3ab, 'YiSyllableShat', [], 'Display', None],
  [ 0xa3ac, 'YiSyllableShax', [], 'Display', None],
  [ 0xa3ad, 'YiSyllableSha', [], 'Display', None],
  [ 0xa3ae, 'YiSyllableShap', [], 'Display', None],
  [ 0xa3af, 'YiSyllableShuox', [], 'Display', None],
  [ 0xa3b0, 'YiSyllableShuo', [], 'Display', None],
  [ 0xa3b1, 'YiSyllableShuop', [], 'Display', None],
  [ 0xa3b2, 'YiSyllableShot', [], 'Display', None],
  [ 0xa3b3, 'YiSyllableShox', [], 'Display', None],
  [ 0xa3b4, 'YiSyllableSho', [], 'Display', None],
  [ 0xa3b5, 'YiSyllableShop', [], 'Display', None],
  [ 0xa3b6, 'YiSyllableShet', [], 'Display', None],
  [ 0xa3b7, 'YiSyllableShex', [], 'Display', None],
  [ 0xa3b8, 'YiSyllableShe', [], 'Display', None],
  [ 0xa3b9, 'YiSyllableShep', [], 'Display', None],
  [ 0xa3ba, 'YiSyllableShut', [], 'Display', None],
  [ 0xa3bb, 'YiSyllableShux', [], 'Display', None],
  [ 0xa3bc, 'YiSyllableShu', [], 'Display', None],
  [ 0xa3bd, 'YiSyllableShup', [], 'Display', None],
  [ 0xa3be, 'YiSyllableShurx', [], 'Display', None],
  [ 0xa3bf, 'YiSyllableShur', [], 'Display', None],
  [ 0xa3c0, 'YiSyllableShyt', [], 'Display', None],
  [ 0xa3c1, 'YiSyllableShyx', [], 'Display', None],
  [ 0xa3c2, 'YiSyllableShy', [], 'Display', None],
  [ 0xa3c3, 'YiSyllableShyp', [], 'Display', None],
  [ 0xa3c4, 'YiSyllableShyrx', [], 'Display', None],
  [ 0xa3c5, 'YiSyllableShyr', [], 'Display', None],
  [ 0xa3c6, 'YiSyllableRat', [], 'Display', None],
  [ 0xa3c7, 'YiSyllableRax', [], 'Display', None],
  [ 0xa3c8, 'YiSyllableRa', [], 'Display', None],
  [ 0xa3c9, 'YiSyllableRap', [], 'Display', None],
  [ 0xa3ca, 'YiSyllableRuox', [], 'Display', None],
  [ 0xa3cb, 'YiSyllableRuo', [], 'Display', None],
  [ 0xa3cc, 'YiSyllableRuop', [], 'Display', None],
  [ 0xa3cd, 'YiSyllableRot', [], 'Display', None],
  [ 0xa3ce, 'YiSyllableRox', [], 'Display', None],
  [ 0xa3cf, 'YiSyllableRo', [], 'Display', None],
  [ 0xa3d0, 'YiSyllableRop', [], 'Display', None],
  [ 0xa3d1, 'YiSyllableRex', [], 'Display', None],
  [ 0xa3d2, 'YiSyllableRe', [], 'Display', None],
  [ 0xa3d3, 'YiSyllableRep', [], 'Display', None],
  [ 0xa3d4, 'YiSyllableRut', [], 'Display', None],
  [ 0xa3d5, 'YiSyllableRux', [], 'Display', None],
  [ 0xa3d6, 'YiSyllableRu', [], 'Display', None],
  [ 0xa3d7, 'YiSyllableRup', [], 'Display', None],
  [ 0xa3d8, 'YiSyllableRurx', [], 'Display', None],
  [ 0xa3d9, 'YiSyllableRur', [], 'Display', None],
  [ 0xa3da, 'YiSyllableRyt', [], 'Display', None],
  [ 0xa3db, 'YiSyllableRyx', [], 'Display', None],
  [ 0xa3dc, 'YiSyllableRy', [], 'Display', None],
  [ 0xa3dd, 'YiSyllableRyp', [], 'Display', None],
  [ 0xa3de, 'YiSyllableRyrx', [], 'Display', None],
  [ 0xa3df, 'YiSyllableRyr', [], 'Display', None],
  [ 0xa3e0, 'YiSyllableJit', [], 'Display', None],
  [ 0xa3e1, 'YiSyllableJix', [], 'Display', None],
  [ 0xa3e2, 'YiSyllableJi', [], 'Display', None],
  [ 0xa3e3, 'YiSyllableJip', [], 'Display', None],
  [ 0xa3e4, 'YiSyllableJiet', [], 'Display', None],
  [ 0xa3e5, 'YiSyllableJiex', [], 'Display', None],
  [ 0xa3e6, 'YiSyllableJie', [], 'Display', None],
  [ 0xa3e7, 'YiSyllableJiep', [], 'Display', None],
  [ 0xa3e8, 'YiSyllableJuot', [], 'Display', None],
  [ 0xa3e9, 'YiSyllableJuox', [], 'Display', None],
  [ 0xa3ea, 'YiSyllableJuo', [], 'Display', None],
  [ 0xa3eb, 'YiSyllableJuop', [], 'Display', None],
  [ 0xa3ec, 'YiSyllableJot', [], 'Display', None],
  [ 0xa3ed, 'YiSyllableJox', [], 'Display', None],
  [ 0xa3ee, 'YiSyllableJo', [], 'Display', None],
  [ 0xa3ef, 'YiSyllableJop', [], 'Display', None],
  [ 0xa3f0, 'YiSyllableJut', [], 'Display', None],
  [ 0xa3f1, 'YiSyllableJux', [], 'Display', None],
  [ 0xa3f2, 'YiSyllableJu', [], 'Display', None],
  [ 0xa3f3, 'YiSyllableJup', [], 'Display', None],
  [ 0xa3f4, 'YiSyllableJurx', [], 'Display', None],
  [ 0xa3f5, 'YiSyllableJur', [], 'Display', None],
  [ 0xa3f6, 'YiSyllableJyt', [], 'Display', None],
  [ 0xa3f7, 'YiSyllableJyx', [], 'Display', None],
  [ 0xa3f8, 'YiSyllableJy', [], 'Display', None],
  [ 0xa3f9, 'YiSyllableJyp', [], 'Display', None],
  [ 0xa3fa, 'YiSyllableJyrx', [], 'Display', None],
  [ 0xa3fb, 'YiSyllableJyr', [], 'Display', None],
  [ 0xa3fc, 'YiSyllableQit', [], 'Display', None],
  [ 0xa3fd, 'YiSyllableQix', [], 'Display', None],
  [ 0xa3fe, 'YiSyllableQi', [], 'Display', None],
  [ 0xa3ff, 'YiSyllableQip', [], 'Display', None],
  [ 0xa400, 'YiSyllableQiet', [], 'Display', None],
  [ 0xa401, 'YiSyllableQiex', [], 'Display', None],
  [ 0xa402, 'YiSyllableQie', [], 'Display', None],
  [ 0xa403, 'YiSyllableQiep', [], 'Display', None],
  [ 0xa404, 'YiSyllableQuot', [], 'Display', None],
  [ 0xa405, 'YiSyllableQuox', [], 'Display', None],
  [ 0xa406, 'YiSyllableQuo', [], 'Display', None],
  [ 0xa407, 'YiSyllableQuop', [], 'Display', None],
  [ 0xa408, 'YiSyllableQot', [], 'Display', None],
  [ 0xa409, 'YiSyllableQox', [], 'Display', None],
  [ 0xa40a, 'YiSyllableQo', [], 'Display', None],
  [ 0xa40b, 'YiSyllableQop', [], 'Display', None],
  [ 0xa40c, 'YiSyllableQut', [], 'Display', None],
  [ 0xa40d, 'YiSyllableQux', [], 'Display', None],
  [ 0xa40e, 'YiSyllableQu', [], 'Display', None],
  [ 0xa40f, 'YiSyllableQup', [], 'Display', None],
  [ 0xa410, 'YiSyllableQurx', [], 'Display', None],
  [ 0xa411, 'YiSyllableQur', [], 'Display', None],
  [ 0xa412, 'YiSyllableQyt', [], 'Display', None],
  [ 0xa413, 'YiSyllableQyx', [], 'Display', None],
  [ 0xa414, 'YiSyllableQy', [], 'Display', None],
  [ 0xa415, 'YiSyllableQyp', [], 'Display', None],
  [ 0xa416, 'YiSyllableQyrx', [], 'Display', None],
  [ 0xa417, 'YiSyllableQyr', [], 'Display', None],
  [ 0xa418, 'YiSyllableJjit', [], 'Display', None],
  [ 0xa419, 'YiSyllableJjix', [], 'Display', None],
  [ 0xa41a, 'YiSyllableJji', [], 'Display', None],
  [ 0xa41b, 'YiSyllableJjip', [], 'Display', None],
  [ 0xa41c, 'YiSyllableJjiet', [], 'Display', None],
  [ 0xa41d, 'YiSyllableJjiex', [], 'Display', None],
  [ 0xa41e, 'YiSyllableJjie', [], 'Display', None],
  [ 0xa41f, 'YiSyllableJjiep', [], 'Display', None],
  [ 0xa420, 'YiSyllableJjuox', [], 'Display', None],
  [ 0xa421, 'YiSyllableJjuo', [], 'Display', None],
  [ 0xa422, 'YiSyllableJjuop', [], 'Display', None],
  [ 0xa423, 'YiSyllableJjot', [], 'Display', None],
  [ 0xa424, 'YiSyllableJjox', [], 'Display', None],
  [ 0xa425, 'YiSyllableJjo', [], 'Display', None],
  [ 0xa426, 'YiSyllableJjop', [], 'Display', None],
  [ 0xa427, 'YiSyllableJjut', [], 'Display', None],
  [ 0xa428, 'YiSyllableJjux', [], 'Display', None],
  [ 0xa429, 'YiSyllableJju', [], 'Display', None],
  [ 0xa42a, 'YiSyllableJjup', [], 'Display', None],
  [ 0xa42b, 'YiSyllableJjurx', [], 'Display', None],
  [ 0xa42c, 'YiSyllableJjur', [], 'Display', None],
  [ 0xa42d, 'YiSyllableJjyt', [], 'Display', None],
  [ 0xa42e, 'YiSyllableJjyx', [], 'Display', None],
  [ 0xa42f, 'YiSyllableJjy', [], 'Display', None],
  [ 0xa430, 'YiSyllableJjyp', [], 'Display', None],
  [ 0xa431, 'YiSyllableNjit', [], 'Display', None],
  [ 0xa432, 'YiSyllableNjix', [], 'Display', None],
  [ 0xa433, 'YiSyllableNji', [], 'Display', None],
  [ 0xa434, 'YiSyllableNjip', [], 'Display', None],
  [ 0xa435, 'YiSyllableNjiet', [], 'Display', None],
  [ 0xa436, 'YiSyllableNjiex', [], 'Display', None],
  [ 0xa437, 'YiSyllableNjie', [], 'Display', None],
  [ 0xa438, 'YiSyllableNjiep', [], 'Display', None],
  [ 0xa439, 'YiSyllableNjuox', [], 'Display', None],
  [ 0xa43a, 'YiSyllableNjuo', [], 'Display', None],
  [ 0xa43b, 'YiSyllableNjot', [], 'Display', None],
  [ 0xa43c, 'YiSyllableNjox', [], 'Display', None],
  [ 0xa43d, 'YiSyllableNjo', [], 'Display', None],
  [ 0xa43e, 'YiSyllableNjop', [], 'Display', None],
  [ 0xa43f, 'YiSyllableNjux', [], 'Display', None],
  [ 0xa440, 'YiSyllableNju', [], 'Display', None],
  [ 0xa441, 'YiSyllableNjup', [], 'Display', None],
  [ 0xa442, 'YiSyllableNjurx', [], 'Display', None],
  [ 0xa443, 'YiSyllableNjur', [], 'Display', None],
  [ 0xa444, 'YiSyllableNjyt', [], 'Display', None],
  [ 0xa445, 'YiSyllableNjyx', [], 'Display', None],
  [ 0xa446, 'YiSyllableNjy', [], 'Display', None],
  [ 0xa447, 'YiSyllableNjyp', [], 'Display', None],
  [ 0xa448, 'YiSyllableNjyrx', [], 'Display', None],
  [ 0xa449, 'YiSyllableNjyr', [], 'Display', None],
  [ 0xa44a, 'YiSyllableNyit', [], 'Display', None],
  [ 0xa44b, 'YiSyllableNyix', [], 'Display', None],
  [ 0xa44c, 'YiSyllableNyi', [], 'Display', None],
  [ 0xa44d, 'YiSyllableNyip', [], 'Display', None],
  [ 0xa44e, 'YiSyllableNyiet', [], 'Display', None],
  [ 0xa44f, 'YiSyllableNyiex', [], 'Display', None],
  [ 0xa450, 'YiSyllableNyie', [], 'Display', None],
  [ 0xa451, 'YiSyllableNyiep', [], 'Display', None],
  [ 0xa452, 'YiSyllableNyuox', [], 'Display', None],
  [ 0xa453, 'YiSyllableNyuo', [], 'Display', None],
  [ 0xa454, 'YiSyllableNyuop', [], 'Display', None],
  [ 0xa455, 'YiSyllableNyot', [], 'Display', None],
  [ 0xa456, 'YiSyllableNyox', [], 'Display', None],
  [ 0xa457, 'YiSyllableNyo', [], 'Display', None],
  [ 0xa458, 'YiSyllableNyop', [], 'Display', None],
  [ 0xa459, 'YiSyllableNyut', [], 'Display', None],
  [ 0xa45a, 'YiSyllableNyux', [], 'Display', None],
  [ 0xa45b, 'YiSyllableNyu', [], 'Display', None],
  [ 0xa45c, 'YiSyllableNyup', [], 'Display', None],
  [ 0xa45d, 'YiSyllableXit', [], 'Display', None],
  [ 0xa45e, 'YiSyllableXix', [], 'Display', None],
  [ 0xa45f, 'YiSyllableXi', [], 'Display', None],
  [ 0xa460, 'YiSyllableXip', [], 'Display', None],
  [ 0xa461, 'YiSyllableXiet', [], 'Display', None],
  [ 0xa462, 'YiSyllableXiex', [], 'Display', None],
  [ 0xa463, 'YiSyllableXie', [], 'Display', None],
  [ 0xa464, 'YiSyllableXiep', [], 'Display', None],
  [ 0xa465, 'YiSyllableXuox', [], 'Display', None],
  [ 0xa466, 'YiSyllableXuo', [], 'Display', None],
  [ 0xa467, 'YiSyllableXot', [], 'Display', None],
  [ 0xa468, 'YiSyllableXox', [], 'Display', None],
  [ 0xa469, 'YiSyllableXo', [], 'Display', None],
  [ 0xa46a, 'YiSyllableXop', [], 'Display', None],
  [ 0xa46b, 'YiSyllableXyt', [], 'Display', None],
  [ 0xa46c, 'YiSyllableXyx', [], 'Display', None],
  [ 0xa46d, 'YiSyllableXy', [], 'Display', None],
  [ 0xa46e, 'YiSyllableXyp', [], 'Display', None],
  [ 0xa46f, 'YiSyllableXyrx', [], 'Display', None],
  [ 0xa470, 'YiSyllableXyr', [], 'Display', None],
  [ 0xa471, 'YiSyllableYit', [], 'Display', None],
  [ 0xa472, 'YiSyllableYix', [], 'Display', None],
  [ 0xa473, 'YiSyllableYi', [], 'Display', None],
  [ 0xa474, 'YiSyllableYip', [], 'Display', None],
  [ 0xa475, 'YiSyllableYiet', [], 'Display', None],
  [ 0xa476, 'YiSyllableYiex', [], 'Display', None],
  [ 0xa477, 'YiSyllableYie', [], 'Display', None],
  [ 0xa478, 'YiSyllableYiep', [], 'Display', None],
  [ 0xa479, 'YiSyllableYuot', [], 'Display', None],
  [ 0xa47a, 'YiSyllableYuox', [], 'Display', None],
  [ 0xa47b, 'YiSyllableYuo', [], 'Display', None],
  [ 0xa47c, 'YiSyllableYuop', [], 'Display', None],
  [ 0xa47d, 'YiSyllableYot', [], 'Display', None],
  [ 0xa47e, 'YiSyllableYox', [], 'Display', None],
  [ 0xa47f, 'YiSyllableYo', [], 'Display', None],
  [ 0xa480, 'YiSyllableYop', [], 'Display', None],
  [ 0xa481, 'YiSyllableYut', [], 'Display', None],
  [ 0xa482, 'YiSyllableYux', [], 'Display', None],
  [ 0xa483, 'YiSyllableYu', [], 'Display', None],
  [ 0xa484, 'YiSyllableYup', [], 'Display', None],
  [ 0xa485, 'YiSyllableYurx', [], 'Display', None],
  [ 0xa486, 'YiSyllableYur', [], 'Display', None],
  [ 0xa487, 'YiSyllableYyt', [], 'Display', None],
  [ 0xa488, 'YiSyllableYyx', [], 'Display', None],
  [ 0xa489, 'YiSyllableYy', [], 'Display', None],
  [ 0xa48a, 'YiSyllableYyp', [], 'Display', None],
  [ 0xa48b, 'YiSyllableYyrx', [], 'Display', None],
  [ 0xa48c, 'YiSyllableYyr', [], 'Display', None],
  [ 0xa48d, 'UnicodeCharacter0xa48d', [], 'Display', None],
  [ 0xa48e, 'UnicodeCharacter0xa48e', [], 'Display', None],
  [ 0xa48f, 'UnicodeCharacter0xa48f', [], 'Display', None],
  [ 0xa490, 'YiRadicalQot', [], 'Display', None],
  [ 0xa491, 'YiRadicalLi', [], 'Display', None],
  [ 0xa492, 'YiRadicalKit', [], 'Display', None],
  [ 0xa493, 'YiRadicalNyip', [], 'Display', None],
  [ 0xa494, 'YiRadicalCyp', [], 'Display', None],
  [ 0xa495, 'YiRadicalSsi', [], 'Display', None],
  [ 0xa496, 'YiRadicalGgop', [], 'Display', None],
  [ 0xa497, 'YiRadicalGep', [], 'Display', None],
  [ 0xa498, 'YiRadicalMi', [], 'Display', None],
  [ 0xa499, 'YiRadicalHxit', [], 'Display', None],
  [ 0xa49a, 'YiRadicalLyr', [], 'Display', None],
  [ 0xa49b, 'YiRadicalBbut', [], 'Display', None],
  [ 0xa49c, 'YiRadicalMop', [], 'Display', None],
  [ 0xa49d, 'YiRadicalYo', [], 'Display', None],
  [ 0xa49e, 'YiRadicalPut', [], 'Display', None],
  [ 0xa49f, 'YiRadicalHxuo', [], 'Display', None],
  [ 0xa4a0, 'YiRadicalTat', [], 'Display', None],
  [ 0xa4a1, 'YiRadicalGa', [], 'Display', None],
  [ 0xa4a2, 'YiRadicalZup', [], 'Display', None],
  [ 0xa4a3, 'YiRadicalCyt', [], 'Display', None],
  [ 0xa4a4, 'YiRadicalDdur', [], 'Display', None],
  [ 0xa4a5, 'YiRadicalBur', [], 'Display', None],
  [ 0xa4a6, 'YiRadicalGguo', [], 'Display', None],
  [ 0xa4a7, 'YiRadicalNyop', [], 'Display', None],
  [ 0xa4a8, 'YiRadicalTu', [], 'Display', None],
  [ 0xa4a9, 'YiRadicalOp', [], 'Display', None],
  [ 0xa4aa, 'YiRadicalJjut', [], 'Display', None],
  [ 0xa4ab, 'YiRadicalZot', [], 'Display', None],
  [ 0xa4ac, 'YiRadicalPyt', [], 'Display', None],
  [ 0xa4ad, 'YiRadicalHmo', [], 'Display', None],
  [ 0xa4ae, 'YiRadicalYit', [], 'Display', None],
  [ 0xa4af, 'YiRadicalVur', [], 'Display', None],
  [ 0xa4b0, 'YiRadicalShy', [], 'Display', None],
  [ 0xa4b1, 'YiRadicalVep', [], 'Display', None],
  [ 0xa4b2, 'YiRadicalZa', [], 'Display', None],
  [ 0xa4b3, 'YiRadicalJo', [], 'Display', None],
  [ 0xa4b4, 'YiRadicalNzup', [], 'Display', None],
  [ 0xa4b5, 'YiRadicalJjy', [], 'Display', None],
  [ 0xa4b6, 'YiRadicalGot', [], 'Display', None],
  [ 0xa4b7, 'YiRadicalJjie', [], 'Display', None],
  [ 0xa4b8, 'YiRadicalWo', [], 'Display', None],
  [ 0xa4b9, 'YiRadicalDu', [], 'Display', None],
  [ 0xa4ba, 'YiRadicalShur', [], 'Display', None],
  [ 0xa4bb, 'YiRadicalLie', [], 'Display', None],
  [ 0xa4bc, 'YiRadicalCy', [], 'Display', None],
  [ 0xa4bd, 'YiRadicalCuop', [], 'Display', None],
  [ 0xa4be, 'YiRadicalCip', [], 'Display', None],
  [ 0xa4bf, 'YiRadicalHxop', [], 'Display', None],
  [ 0xa4c0, 'YiRadicalShat', [], 'Display', None],
  [ 0xa4c1, 'YiRadicalZur', [], 'Display', None],
  [ 0xa4c2, 'YiRadicalShop', [], 'Display', None],
  [ 0xa4c3, 'YiRadicalChe', [], 'Display', None],
  [ 0xa4c4, 'YiRadicalZziet', [], 'Display', None],
  [ 0xa4c5, 'YiRadicalNbie', [], 'Display', None],
  [ 0xa4c6, 'YiRadicalKe', [], 'Display', None],
  [ 0xa4c7, 'UnicodeCharacter0xa4c7', [], 'Display', None],
  [ 0xa4c8, 'UnicodeCharacter0xa4c8', [], 'Display', None],
  [ 0xa4c9, 'UnicodeCharacter0xa4c9', [], 'Display', None],
  [ 0xa4ca, 'UnicodeCharacter0xa4ca', [], 'Display', None],
  [ 0xa4cb, 'UnicodeCharacter0xa4cb', [], 'Display', None],
  [ 0xa4cc, 'UnicodeCharacter0xa4cc', [], 'Display', None],
  [ 0xa4cd, 'UnicodeCharacter0xa4cd', [], 'Display', None],
  [ 0xa4ce, 'UnicodeCharacter0xa4ce', [], 'Display', None],
  [ 0xa4cf, 'UnicodeCharacter0xa4cf', [], 'Display', None],
  [ 0xa4d0, 'UnicodeCharacter0xa4d0', [], 'Display', None],
  [ 0xa4d1, 'UnicodeCharacter0xa4d1', [], 'Display', None],
  [ 0xa4d2, 'UnicodeCharacter0xa4d2', [], 'Display', None],
  [ 0xa4d3, 'UnicodeCharacter0xa4d3', [], 'Display', None],
  [ 0xa4d4, 'UnicodeCharacter0xa4d4', [], 'Display', None],
  [ 0xa4d5, 'UnicodeCharacter0xa4d5', [], 'Display', None],
  [ 0xa4d6, 'UnicodeCharacter0xa4d6', [], 'Display', None],
  [ 0xa4d7, 'UnicodeCharacter0xa4d7', [], 'Display', None],
  [ 0xa4d8, 'UnicodeCharacter0xa4d8', [], 'Display', None],
  [ 0xa4d9, 'UnicodeCharacter0xa4d9', [], 'Display', None],
  [ 0xa4da, 'UnicodeCharacter0xa4da', [], 'Display', None],
  [ 0xa4db, 'UnicodeCharacter0xa4db', [], 'Display', None],
  [ 0xa4dc, 'UnicodeCharacter0xa4dc', [], 'Display', None],
  [ 0xa4dd, 'UnicodeCharacter0xa4dd', [], 'Display', None],
  [ 0xa4de, 'UnicodeCharacter0xa4de', [], 'Display', None],
  [ 0xa4df, 'UnicodeCharacter0xa4df', [], 'Display', None],
  [ 0xa4e0, 'UnicodeCharacter0xa4e0', [], 'Display', None],
  [ 0xa4e1, 'UnicodeCharacter0xa4e1', [], 'Display', None],
  [ 0xa4e2, 'UnicodeCharacter0xa4e2', [], 'Display', None],
  [ 0xa4e3, 'UnicodeCharacter0xa4e3', [], 'Display', None],
  [ 0xa4e4, 'UnicodeCharacter0xa4e4', [], 'Display', None],
  [ 0xa4e5, 'UnicodeCharacter0xa4e5', [], 'Display', None],
  [ 0xa4e6, 'UnicodeCharacter0xa4e6', [], 'Display', None],
  [ 0xa4e7, 'UnicodeCharacter0xa4e7', [], 'Display', None],
  [ 0xa4e8, 'UnicodeCharacter0xa4e8', [], 'Display', None],
  [ 0xa4e9, 'UnicodeCharacter0xa4e9', [], 'Display', None],
  [ 0xa4ea, 'UnicodeCharacter0xa4ea', [], 'Display', None],
  [ 0xa4eb, 'UnicodeCharacter0xa4eb', [], 'Display', None],
  [ 0xa4ec, 'UnicodeCharacter0xa4ec', [], 'Display', None],
  [ 0xa4ed, 'UnicodeCharacter0xa4ed', [], 'Display', None],
  [ 0xa4ee, 'UnicodeCharacter0xa4ee', [], 'Display', None],
  [ 0xa4ef, 'UnicodeCharacter0xa4ef', [], 'Display', None],
  [ 0xa4f0, 'UnicodeCharacter0xa4f0', [], 'Display', None],
  [ 0xa4f1, 'UnicodeCharacter0xa4f1', [], 'Display', None],
  [ 0xa4f2, 'UnicodeCharacter0xa4f2', [], 'Display', None],
  [ 0xa4f3, 'UnicodeCharacter0xa4f3', [], 'Display', None],
  [ 0xa4f4, 'UnicodeCharacter0xa4f4', [], 'Display', None],
  [ 0xa4f5, 'UnicodeCharacter0xa4f5', [], 'Display', None],
  [ 0xa4f6, 'UnicodeCharacter0xa4f6', [], 'Display', None],
  [ 0xa4f7, 'UnicodeCharacter0xa4f7', [], 'Display', None],
  [ 0xa4f8, 'UnicodeCharacter0xa4f8', [], 'Display', None],
  [ 0xa4f9, 'UnicodeCharacter0xa4f9', [], 'Display', None],
  [ 0xa4fa, 'UnicodeCharacter0xa4fa', [], 'Display', None],
  [ 0xa4fb, 'UnicodeCharacter0xa4fb', [], 'Display', None],
  [ 0xa4fc, 'UnicodeCharacter0xa4fc', [], 'Display', None],
  [ 0xa4fd, 'UnicodeCharacter0xa4fd', [], 'Display', None],
  [ 0xa4fe, 'UnicodeCharacter0xa4fe', [], 'Display', None],
  [ 0xa4ff, 'UnicodeCharacter0xa4ff', [], 'Display', None],
  [ 0xa500, 'VaiSyllableEe', [], 'Display', None],
  [ 0xa501, 'VaiSyllableEen', [], 'Display', None],
  [ 0xa502, 'VaiSyllableHee', [], 'Display', None],
  [ 0xa503, 'VaiSyllableWee', [], 'Display', None],
  [ 0xa504, 'VaiSyllableWeen', [], 'Display', None],
  [ 0xa505, 'VaiSyllablePee', [], 'Display', None],
  [ 0xa506, 'VaiSyllableBhee', [], 'Display', None],
  [ 0xa507, 'VaiSyllableBee', [], 'Display', None],
  [ 0xa508, 'VaiSyllableMbee', [], 'Display', None],
  [ 0xa509, 'VaiSyllableKpee', [], 'Display', None],
  [ 0xa50a, 'VaiSyllableMgbee', [], 'Display', None],
  [ 0xa50b, 'VaiSyllableGbee', [], 'Display', None],
  [ 0xa50c, 'VaiSyllableFee', [], 'Display', None],
  [ 0xa50d, 'VaiSyllableVee', [], 'Display', None],
  [ 0xa50e, 'VaiSyllableTee', [], 'Display', None],
  [ 0xa50f, 'VaiSyllableThee', [], 'Display', None],
  [ 0xa510, 'VaiSyllableDhee', [], 'Display', None],
  [ 0xa511, 'VaiSyllableDhhee', [], 'Display', None],
  [ 0xa512, 'VaiSyllableLee', [], 'Display', None],
  [ 0xa513, 'VaiSyllableRee', [], 'Display', None],
  [ 0xa514, 'VaiSyllableDee', [], 'Display', None],
  [ 0xa515, 'VaiSyllableNdee', [], 'Display', None],
  [ 0xa516, 'VaiSyllableSee', [], 'Display', None],
  [ 0xa517, 'VaiSyllableShee', [], 'Display', None],
  [ 0xa518, 'VaiSyllableZee', [], 'Display', None],
  [ 0xa519, 'VaiSyllableZhee', [], 'Display', None],
  [ 0xa51a, 'VaiSyllableCee', [], 'Display', None],
  [ 0xa51b, 'VaiSyllableJee', [], 'Display', None],
  [ 0xa51c, 'VaiSyllableNjee', [], 'Display', None],
  [ 0xa51d, 'VaiSyllableYee', [], 'Display', None],
  [ 0xa51e, 'VaiSyllableKee', [], 'Display', None],
  [ 0xa51f, 'VaiSyllableNggee', [], 'Display', None],
  [ 0xa520, 'VaiSyllableGee', [], 'Display', None],
  [ 0xa521, 'VaiSyllableMee', [], 'Display', None],
  [ 0xa522, 'VaiSyllableNee', [], 'Display', None],
  [ 0xa523, 'VaiSyllableNyee', [], 'Display', None],
  [ 0xa524, 'VaiSyllableI', [], 'Display', None],
  [ 0xa525, 'VaiSyllableIn', [], 'Display', None],
  [ 0xa526, 'VaiSyllableHi', [], 'Display', None],
  [ 0xa527, 'VaiSyllableHin', [], 'Display', None],
  [ 0xa528, 'VaiSyllableWi', [], 'Display', None],
  [ 0xa529, 'VaiSyllableWin', [], 'Display', None],
  [ 0xa52a, 'VaiSyllablePi', [], 'Display', None],
  [ 0xa52b, 'VaiSyllableBhi', [], 'Display', None],
  [ 0xa52c, 'VaiSyllableBi', [], 'Display', None],
  [ 0xa52d, 'VaiSyllableMbi', [], 'Display', None],
  [ 0xa52e, 'VaiSyllableKpi', [], 'Display', None],
  [ 0xa52f, 'VaiSyllableMgbi', [], 'Display', None],
  [ 0xa530, 'VaiSyllableGbi', [], 'Display', None],
  [ 0xa531, 'VaiSyllableFi', [], 'Display', None],
  [ 0xa532, 'VaiSyllableVi', [], 'Display', None],
  [ 0xa533, 'VaiSyllableTi', [], 'Display', None],
  [ 0xa534, 'VaiSyllableThi', [], 'Display', None],
  [ 0xa535, 'VaiSyllableDhi', [], 'Display', None],
  [ 0xa536, 'VaiSyllableDhhi', [], 'Display', None],
  [ 0xa537, 'VaiSyllableLi', [], 'Display', None],
  [ 0xa538, 'VaiSyllableRi', [], 'Display', None],
  [ 0xa539, 'VaiSyllableDi', [], 'Display', None],
  [ 0xa53a, 'VaiSyllableNdi', [], 'Display', None],
  [ 0xa53b, 'VaiSyllableSi', [], 'Display', None],
  [ 0xa53c, 'VaiSyllableShi', [], 'Display', None],
  [ 0xa53d, 'VaiSyllableZi', [], 'Display', None],
  [ 0xa53e, 'VaiSyllableZhi', [], 'Display', None],
  [ 0xa53f, 'VaiSyllableCi', [], 'Display', None],
  [ 0xa540, 'VaiSyllableJi', [], 'Display', None],
  [ 0xa541, 'VaiSyllableNji', [], 'Display', None],
  [ 0xa542, 'VaiSyllableYi', [], 'Display', None],
  [ 0xa543, 'VaiSyllableKi', [], 'Display', None],
  [ 0xa544, 'VaiSyllableNggi', [], 'Display', None],
  [ 0xa545, 'VaiSyllableGi', [], 'Display', None],
  [ 0xa546, 'VaiSyllableMi', [], 'Display', None],
  [ 0xa547, 'VaiSyllableNi', [], 'Display', None],
  [ 0xa548, 'VaiSyllableNyi', [], 'Display', None],
  [ 0xa549, 'VaiSyllableA', [], 'Display', None],
  [ 0xa54a, 'VaiSyllableAn', [], 'Display', None],
  [ 0xa54b, 'VaiSyllableNgan', [], 'Display', None],
  [ 0xa54c, 'VaiSyllableHa', [], 'Display', None],
  [ 0xa54d, 'VaiSyllableHan', [], 'Display', None],
  [ 0xa54e, 'VaiSyllableWa', [], 'Display', None],
  [ 0xa54f, 'VaiSyllableWan', [], 'Display', None],
  [ 0xa550, 'VaiSyllablePa', [], 'Display', None],
  [ 0xa551, 'VaiSyllableBha', [], 'Display', None],
  [ 0xa552, 'VaiSyllableBa', [], 'Display', None],
  [ 0xa553, 'VaiSyllableMba', [], 'Display', None],
  [ 0xa554, 'VaiSyllableKpa', [], 'Display', None],
  [ 0xa555, 'VaiSyllableKpan', [], 'Display', None],
  [ 0xa556, 'VaiSyllableMgba', [], 'Display', None],
  [ 0xa557, 'VaiSyllableGba', [], 'Display', None],
  [ 0xa558, 'VaiSyllableFa', [], 'Display', None],
  [ 0xa559, 'VaiSyllableVa', [], 'Display', None],
  [ 0xa55a, 'VaiSyllableTa', [], 'Display', None],
  [ 0xa55b, 'VaiSyllableTha', [], 'Display', None],
  [ 0xa55c, 'VaiSyllableDha', [], 'Display', None],
  [ 0xa55d, 'VaiSyllableDhha', [], 'Display', None],
  [ 0xa55e, 'VaiSyllableLa', [], 'Display', None],
  [ 0xa55f, 'VaiSyllableRa', [], 'Display', None],
  [ 0xa560, 'VaiSyllableDa', [], 'Display', None],
  [ 0xa561, 'VaiSyllableNda', [], 'Display', None],
  [ 0xa562, 'VaiSyllableSa', [], 'Display', None],
  [ 0xa563, 'VaiSyllableSha', [], 'Display', None],
  [ 0xa564, 'VaiSyllableZa', [], 'Display', None],
  [ 0xa565, 'VaiSyllableZha', [], 'Display', None],
  [ 0xa566, 'VaiSyllableCa', [], 'Display', None],
  [ 0xa567, 'VaiSyllableJa', [], 'Display', None],
  [ 0xa568, 'VaiSyllableNja', [], 'Display', None],
  [ 0xa569, 'VaiSyllableYa', [], 'Display', None],
  [ 0xa56a, 'VaiSyllableKa', [], 'Display', None],
  [ 0xa56b, 'VaiSyllableKan', [], 'Display', None],
  [ 0xa56c, 'VaiSyllableNgga', [], 'Display', None],
  [ 0xa56d, 'VaiSyllableGa', [], 'Display', None],
  [ 0xa56e, 'VaiSyllableMa', [], 'Display', None],
  [ 0xa56f, 'VaiSyllableNa', [], 'Display', None],
  [ 0xa570, 'VaiSyllableNya', [], 'Display', None],
  [ 0xa571, 'VaiSyllableOo', [], 'Display', None],
  [ 0xa572, 'VaiSyllableOon', [], 'Display', None],
  [ 0xa573, 'VaiSyllableHoo', [], 'Display', None],
  [ 0xa574, 'VaiSyllableWoo', [], 'Display', None],
  [ 0xa575, 'VaiSyllableWoon', [], 'Display', None],
  [ 0xa576, 'VaiSyllablePoo', [], 'Display', None],
  [ 0xa577, 'VaiSyllableBhoo', [], 'Display', None],
  [ 0xa578, 'VaiSyllableBoo', [], 'Display', None],
  [ 0xa579, 'VaiSyllableMboo', [], 'Display', None],
  [ 0xa57a, 'VaiSyllableKpoo', [], 'Display', None],
  [ 0xa57b, 'VaiSyllableMgboo', [], 'Display', None],
  [ 0xa57c, 'VaiSyllableGboo', [], 'Display', None],
  [ 0xa57d, 'VaiSyllableFoo', [], 'Display', None],
  [ 0xa57e, 'VaiSyllableVoo', [], 'Display', None],
  [ 0xa57f, 'VaiSyllableToo', [], 'Display', None],
  [ 0xa580, 'VaiSyllableThoo', [], 'Display', None],
  [ 0xa581, 'VaiSyllableDhoo', [], 'Display', None],
  [ 0xa582, 'VaiSyllableDhhoo', [], 'Display', None],
  [ 0xa583, 'VaiSyllableLoo', [], 'Display', None],
  [ 0xa584, 'VaiSyllableRoo', [], 'Display', None],
  [ 0xa585, 'VaiSyllableDoo', [], 'Display', None],
  [ 0xa586, 'VaiSyllableNdoo', [], 'Display', None],
  [ 0xa587, 'VaiSyllableSoo', [], 'Display', None],
  [ 0xa588, 'VaiSyllableShoo', [], 'Display', None],
  [ 0xa589, 'VaiSyllableZoo', [], 'Display', None],
  [ 0xa58a, 'VaiSyllableZhoo', [], 'Display', None],
  [ 0xa58b, 'VaiSyllableCoo', [], 'Display', None],
  [ 0xa58c, 'VaiSyllableJoo', [], 'Display', None],
  [ 0xa58d, 'VaiSyllableNjoo', [], 'Display', None],
  [ 0xa58e, 'VaiSyllableYoo', [], 'Display', None],
  [ 0xa58f, 'VaiSyllableKoo', [], 'Display', None],
  [ 0xa590, 'VaiSyllableNggoo', [], 'Display', None],
  [ 0xa591, 'VaiSyllableGoo', [], 'Display', None],
  [ 0xa592, 'VaiSyllableMoo', [], 'Display', None],
  [ 0xa593, 'VaiSyllableNoo', [], 'Display', None],
  [ 0xa594, 'VaiSyllableNyoo', [], 'Display', None],
  [ 0xa595, 'VaiSyllableU', [], 'Display', None],
  [ 0xa596, 'VaiSyllableUn', [], 'Display', None],
  [ 0xa597, 'VaiSyllableHu', [], 'Display', None],
  [ 0xa598, 'VaiSyllableHun', [], 'Display', None],
  [ 0xa599, 'VaiSyllableWu', [], 'Display', None],
  [ 0xa59a, 'VaiSyllableWun', [], 'Display', None],
  [ 0xa59b, 'VaiSyllablePu', [], 'Display', None],
  [ 0xa59c, 'VaiSyllableBhu', [], 'Display', None],
  [ 0xa59d, 'VaiSyllableBu', [], 'Display', None],
  [ 0xa59e, 'VaiSyllableMbu', [], 'Display', None],
  [ 0xa59f, 'VaiSyllableKpu', [], 'Display', None],
  [ 0xa5a0, 'VaiSyllableMgbu', [], 'Display', None],
  [ 0xa5a1, 'VaiSyllableGbu', [], 'Display', None],
  [ 0xa5a2, 'VaiSyllableFu', [], 'Display', None],
  [ 0xa5a3, 'VaiSyllableVu', [], 'Display', None],
  [ 0xa5a4, 'VaiSyllableTu', [], 'Display', None],
  [ 0xa5a5, 'VaiSyllableThu', [], 'Display', None],
  [ 0xa5a6, 'VaiSyllableDhu', [], 'Display', None],
  [ 0xa5a7, 'VaiSyllableDhhu', [], 'Display', None],
  [ 0xa5a8, 'VaiSyllableLu', [], 'Display', None],
  [ 0xa5a9, 'VaiSyllableRu', [], 'Display', None],
  [ 0xa5aa, 'VaiSyllableDu', [], 'Display', None],
  [ 0xa5ab, 'VaiSyllableNdu', [], 'Display', None],
  [ 0xa5ac, 'VaiSyllableSu', [], 'Display', None],
  [ 0xa5ad, 'VaiSyllableShu', [], 'Display', None],
  [ 0xa5ae, 'VaiSyllableZu', [], 'Display', None],
  [ 0xa5af, 'VaiSyllableZhu', [], 'Display', None],
  [ 0xa5b0, 'VaiSyllableCu', [], 'Display', None],
  [ 0xa5b1, 'VaiSyllableJu', [], 'Display', None],
  [ 0xa5b2, 'VaiSyllableNju', [], 'Display', None],
  [ 0xa5b3, 'VaiSyllableYu', [], 'Display', None],
  [ 0xa5b4, 'VaiSyllableKu', [], 'Display', None],
  [ 0xa5b5, 'VaiSyllableNggu', [], 'Display', None],
  [ 0xa5b6, 'VaiSyllableGu', [], 'Display', None],
  [ 0xa5b7, 'VaiSyllableMu', [], 'Display', None],
  [ 0xa5b8, 'VaiSyllableNu', [], 'Display', None],
  [ 0xa5b9, 'VaiSyllableNyu', [], 'Display', None],
  [ 0xa5ba, 'VaiSyllableO', [], 'Display', None],
  [ 0xa5bb, 'VaiSyllableOn', [], 'Display', None],
  [ 0xa5bc, 'VaiSyllableNgon', [], 'Display', None],
  [ 0xa5bd, 'VaiSyllableHo', [], 'Display', None],
  [ 0xa5be, 'VaiSyllableHon', [], 'Display', None],
  [ 0xa5bf, 'VaiSyllableWo', [], 'Display', None],
  [ 0xa5c0, 'VaiSyllableWon', [], 'Display', None],
  [ 0xa5c1, 'VaiSyllablePo', [], 'Display', None],
  [ 0xa5c2, 'VaiSyllableBho', [], 'Display', None],
  [ 0xa5c3, 'VaiSyllableBo', [], 'Display', None],
  [ 0xa5c4, 'VaiSyllableMbo', [], 'Display', None],
  [ 0xa5c5, 'VaiSyllableKpo', [], 'Display', None],
  [ 0xa5c6, 'VaiSyllableMgbo', [], 'Display', None],
  [ 0xa5c7, 'VaiSyllableGbo', [], 'Display', None],
  [ 0xa5c8, 'VaiSyllableGbon', [], 'Display', None],
  [ 0xa5c9, 'VaiSyllableFo', [], 'Display', None],
  [ 0xa5ca, 'VaiSyllableVo', [], 'Display', None],
  [ 0xa5cb, 'VaiSyllableTo', [], 'Display', None],
  [ 0xa5cc, 'VaiSyllableTho', [], 'Display', None],
  [ 0xa5cd, 'VaiSyllableDho', [], 'Display', None],
  [ 0xa5ce, 'VaiSyllableDhho', [], 'Display', None],
  [ 0xa5cf, 'VaiSyllableLo', [], 'Display', None],
  [ 0xa5d0, 'VaiSyllableRo', [], 'Display', None],
  [ 0xa5d1, 'VaiSyllableDo', [], 'Display', None],
  [ 0xa5d2, 'VaiSyllableNdo', [], 'Display', None],
  [ 0xa5d3, 'VaiSyllableSo', [], 'Display', None],
  [ 0xa5d4, 'VaiSyllableSho', [], 'Display', None],
  [ 0xa5d5, 'VaiSyllableZo', [], 'Display', None],
  [ 0xa5d6, 'VaiSyllableZho', [], 'Display', None],
  [ 0xa5d7, 'VaiSyllableCo', [], 'Display', None],
  [ 0xa5d8, 'VaiSyllableJo', [], 'Display', None],
  [ 0xa5d9, 'VaiSyllableNjo', [], 'Display', None],
  [ 0xa5da, 'VaiSyllableYo', [], 'Display', None],
  [ 0xa5db, 'VaiSyllableKo', [], 'Display', None],
  [ 0xa5dc, 'VaiSyllableNggo', [], 'Display', None],
  [ 0xa5dd, 'VaiSyllableGo', [], 'Display', None],
  [ 0xa5de, 'VaiSyllableMo', [], 'Display', None],
  [ 0xa5df, 'VaiSyllableNo', [], 'Display', None],
  [ 0xa5e0, 'VaiSyllableNyo', [], 'Display', None],
  [ 0xa5e1, 'VaiSyllableE', [], 'Display', None],
  [ 0xa5e2, 'VaiSyllableEn', [], 'Display', None],
  [ 0xa5e3, 'VaiSyllableNgen', [], 'Display', None],
  [ 0xa5e4, 'VaiSyllableHe', [], 'Display', None],
  [ 0xa5e5, 'VaiSyllableHen', [], 'Display', None],
  [ 0xa5e6, 'VaiSyllableWe', [], 'Display', None],
  [ 0xa5e7, 'VaiSyllableWen', [], 'Display', None],
  [ 0xa5e8, 'VaiSyllablePe', [], 'Display', None],
  [ 0xa5e9, 'VaiSyllableBhe', [], 'Display', None],
  [ 0xa5ea, 'VaiSyllableBe', [], 'Display', None],
  [ 0xa5eb, 'VaiSyllableMbe', [], 'Display', None],
  [ 0xa5ec, 'VaiSyllableKpe', [], 'Display', None],
  [ 0xa5ed, 'VaiSyllableKpen', [], 'Display', None],
  [ 0xa5ee, 'VaiSyllableMgbe', [], 'Display', None],
  [ 0xa5ef, 'VaiSyllableGbe', [], 'Display', None],
  [ 0xa5f0, 'VaiSyllableGben', [], 'Display', None],
  [ 0xa5f1, 'VaiSyllableFe', [], 'Display', None],
  [ 0xa5f2, 'VaiSyllableVe', [], 'Display', None],
  [ 0xa5f3, 'VaiSyllableTe', [], 'Display', None],
  [ 0xa5f4, 'VaiSyllableThe', [], 'Display', None],
  [ 0xa5f5, 'VaiSyllableDhe', [], 'Display', None],
  [ 0xa5f6, 'VaiSyllableDhhe', [], 'Display', None],
  [ 0xa5f7, 'VaiSyllableLe', [], 'Display', None],
  [ 0xa5f8, 'VaiSyllableRe', [], 'Display', None],
  [ 0xa5f9, 'VaiSyllableDe', [], 'Display', None],
  [ 0xa5fa, 'VaiSyllableNde', [], 'Display', None],
  [ 0xa5fb, 'VaiSyllableSe', [], 'Display', None],
  [ 0xa5fc, 'VaiSyllableShe', [], 'Display', None],
  [ 0xa5fd, 'VaiSyllableZe', [], 'Display', None],
  [ 0xa5fe, 'VaiSyllableZhe', [], 'Display', None],
  [ 0xa5ff, 'VaiSyllableCe', [], 'Display', None],
  [ 0xa600, 'VaiSyllableJe', [], 'Display', None],
  [ 0xa601, 'VaiSyllableNje', [], 'Display', None],
  [ 0xa602, 'VaiSyllableYe', [], 'Display', None],
  [ 0xa603, 'VaiSyllableKe', [], 'Display', None],
  [ 0xa604, 'VaiSyllableNgge', [], 'Display', None],
  [ 0xa605, 'VaiSyllableNggen', [], 'Display', None],
  [ 0xa606, 'VaiSyllableGe', [], 'Display', None],
  [ 0xa607, 'VaiSyllableGen', [], 'Display', None],
  [ 0xa608, 'VaiSyllableMe', [], 'Display', None],
  [ 0xa609, 'VaiSyllableNe', [], 'Display', None],
  [ 0xa60a, 'VaiSyllableNye', [], 'Display', None],
  [ 0xa60b, 'VaiSyllableNg', [], 'Display', None],
  [ 0xa60c, 'VaiSyllableLengthener', [], 'Display', None],
  [ 0xa60d, 'VaiComma', [], 'Display', None],
  [ 0xa60e, 'VaiFullStop', [], 'Display', None],
  [ 0xa60f, 'VaiQuestionMark', [], 'Display', None],
  [ 0xa610, 'VaiSyllableNdoleFa', [], 'Display', None],
  [ 0xa611, 'VaiSyllableNdoleKa', [], 'Display', None],
  [ 0xa612, 'VaiSyllableNdoleSoo', [], 'Display', None],
  [ 0xa613, 'VaiSymbolFeeng', [], 'Display', None],
  [ 0xa614, 'VaiSymbolKeeng', [], 'Display', None],
  [ 0xa615, 'VaiSymbolTing', [], 'Display', None],
  [ 0xa616, 'VaiSymbolNii', [], 'Display', None],
  [ 0xa617, 'VaiSymbolBang', [], 'Display', None],
  [ 0xa618, 'VaiSymbolFaa', [], 'Display', None],
  [ 0xa619, 'VaiSymbolTaa', [], 'Display', None],
  [ 0xa61a, 'VaiSymbolDang', [], 'Display', None],
  [ 0xa61b, 'VaiSymbolDoong', [], 'Display', None],
  [ 0xa61c, 'VaiSymbolKung', [], 'Display', None],
  [ 0xa61d, 'VaiSymbolTong', [], 'Display', None],
  [ 0xa61e, 'VaiSymbolDoo', [], 'Display', None],
  [ 0xa61f, 'VaiSymbolJong', [], 'Display', None],
  [ 0xa620, 'VaiDigitZero', [], 'Display', None],
  [ 0xa621, 'VaiDigitOne', [], 'Display', None],
  [ 0xa622, 'VaiDigitTwo', [], 'Display', None],
  [ 0xa623, 'VaiDigitThree', [], 'Display', None],
  [ 0xa624, 'VaiDigitFour', [], 'Display', None],
  [ 0xa625, 'VaiDigitFive', [], 'Display', None],
  [ 0xa626, 'VaiDigitSix', [], 'Display', None],
  [ 0xa627, 'VaiDigitSeven', [], 'Display', None],
  [ 0xa628, 'VaiDigitEight', [], 'Display', None],
  [ 0xa629, 'VaiDigitNine', [], 'Display', None],
  [ 0xa62a, 'VaiSyllableNdoleMa', [], 'Display', None],
  [ 0xa62b, 'VaiSyllableNdoleDo', [], 'Display', None],
  [ 0xa62c, 'UnicodeCharacter0xa62c', [], 'Display', None],
  [ 0xa62d, 'UnicodeCharacter0xa62d', [], 'Display', None],
  [ 0xa62e, 'UnicodeCharacter0xa62e', [], 'Display', None],
  [ 0xa62f, 'UnicodeCharacter0xa62f', [], 'Display', None],
  [ 0xa630, 'UnicodeCharacter0xa630', [], 'Display', None],
  [ 0xa631, 'UnicodeCharacter0xa631', [], 'Display', None],
  [ 0xa632, 'UnicodeCharacter0xa632', [], 'Display', None],
  [ 0xa633, 'UnicodeCharacter0xa633', [], 'Display', None],
  [ 0xa634, 'UnicodeCharacter0xa634', [], 'Display', None],
  [ 0xa635, 'UnicodeCharacter0xa635', [], 'Display', None],
  [ 0xa636, 'UnicodeCharacter0xa636', [], 'Display', None],
  [ 0xa637, 'UnicodeCharacter0xa637', [], 'Display', None],
  [ 0xa638, 'UnicodeCharacter0xa638', [], 'Display', None],
  [ 0xa639, 'UnicodeCharacter0xa639', [], 'Display', None],
  [ 0xa63a, 'UnicodeCharacter0xa63a', [], 'Display', None],
  [ 0xa63b, 'UnicodeCharacter0xa63b', [], 'Display', None],
  [ 0xa63c, 'UnicodeCharacter0xa63c', [], 'Display', None],
  [ 0xa63d, 'UnicodeCharacter0xa63d', [], 'Display', None],
  [ 0xa63e, 'UnicodeCharacter0xa63e', [], 'Display', None],
  [ 0xa63f, 'UnicodeCharacter0xa63f', [], 'Display', None],
  [ 0xa640, 'CyrillicCapitalLetterZemlya', [], 'Display', None],
  [ 0xa641, 'CyrillicSmallLetterZemlya', [], 'Display', None],
  [ 0xa642, 'CyrillicCapitalLetterDzelo', [], 'Display', None],
  [ 0xa643, 'CyrillicSmallLetterDzelo', [], 'Display', None],
  [ 0xa644, 'CyrillicCapitalLetterReversedDze', [], 'Display', None],
  [ 0xa645, 'CyrillicSmallLetterReversedDze', [], 'Display', None],
  [ 0xa646, 'CyrillicCapitalLetterIota', [], 'Display', None],
  [ 0xa647, 'CyrillicSmallLetterIota', [], 'Display', None],
  [ 0xa648, 'CyrillicCapitalLetterDjerv', [], 'Display', None],
  [ 0xa649, 'CyrillicSmallLetterDjerv', [], 'Display', None],
  [ 0xa64a, 'CyrillicCapitalLetterMonographUk', [], 'Display', None],
  [ 0xa64b, 'CyrillicSmallLetterMonographUk', [], 'Display', None],
  [ 0xa64c, 'CyrillicCapitalLetterBroadOmega', [], 'Display', None],
  [ 0xa64d, 'CyrillicSmallLetterBroadOmega', [], 'Display', None],
  [ 0xa64e, 'CyrillicCapitalLetterNeutralYer', [], 'Display', None],
  [ 0xa64f, 'CyrillicSmallLetterNeutralYer', [], 'Display', None],
  [ 0xa650, 'CyrillicCapitalLetterYeruWithBackYer', [], 'Display', None],
  [ 0xa651, 'CyrillicSmallLetterYeruWithBackYer', [], 'Display', None],
  [ 0xa652, 'CyrillicCapitalLetterIotifiedYat', [], 'Display', None],
  [ 0xa653, 'CyrillicSmallLetterIotifiedYat', [], 'Display', None],
  [ 0xa654, 'CyrillicCapitalLetterReversedYu', [], 'Display', None],
  [ 0xa655, 'CyrillicSmallLetterReversedYu', [], 'Display', None],
  [ 0xa656, 'CyrillicCapitalLetterIotifiedA', [], 'Display', None],
  [ 0xa657, 'CyrillicSmallLetterIotifiedA', [], 'Display', None],
  [ 0xa658, 'CyrillicCapitalLetterClosedLittleYus', [], 'Display', None],
  [ 0xa659, 'CyrillicSmallLetterClosedLittleYus', [], 'Display', None],
  [ 0xa65a, 'CyrillicCapitalLetterBlendedYus', [], 'Display', None],
  [ 0xa65b, 'CyrillicSmallLetterBlendedYus', [], 'Display', None],
  [ 0xa65c, 'CyrillicCapitalLetterIotifiedClosedLittleYus', [], 'Display', None],
  [ 0xa65d, 'CyrillicSmallLetterIotifiedClosedLittleYus', [], 'Display', None],
  [ 0xa65e, 'CyrillicCapitalLetterYn', [], 'Display', None],
  [ 0xa65f, 'CyrillicSmallLetterYn', [], 'Display', None],
  [ 0xa660, 'UnicodeCharacter0xa660', [], 'Display', None],
  [ 0xa661, 'UnicodeCharacter0xa661', [], 'Display', None],
  [ 0xa662, 'CyrillicCapitalLetterSoftDe', [], 'Display', None],
  [ 0xa663, 'CyrillicSmallLetterSoftDe', [], 'Display', None],
  [ 0xa664, 'CyrillicCapitalLetterSoftEl', [], 'Display', None],
  [ 0xa665, 'CyrillicSmallLetterSoftEl', [], 'Display', None],
  [ 0xa666, 'CyrillicCapitalLetterSoftEm', [], 'Display', None],
  [ 0xa667, 'CyrillicSmallLetterSoftEm', [], 'Display', None],
  [ 0xa668, 'CyrillicCapitalLetterMonocularO', [], 'Display', None],
  [ 0xa669, 'CyrillicSmallLetterMonocularO', [], 'Display', None],
  [ 0xa66a, 'CyrillicCapitalLetterBinocularO', [], 'Display', None],
  [ 0xa66b, 'CyrillicSmallLetterBinocularO', [], 'Display', None],
  [ 0xa66c, 'CyrillicCapitalLetterDoubleMonocularO', [], 'Display', None],
  [ 0xa66d, 'CyrillicSmallLetterDoubleMonocularO', [], 'Display', None],
  [ 0xa66e, 'CyrillicLetterMultiocularO', [], 'Display', None],
  [ 0xa66f, 'CombiningCyrillicVzmet', [], 'Display', None],
  [ 0xa670, 'CombiningCyrillicTenMillionsSign', [], 'Display', None],
  [ 0xa671, 'CombiningCyrillicHundredMillionsSign', [], 'Display', None],
  [ 0xa672, 'CombiningCyrillicThousandMillionsSign', [], 'Display', None],
  [ 0xa673, 'SlavonicAsterisk', [], 'Display', None],
  [ 0xa674, 'UnicodeCharacter0xa674', [], 'Display', None],
  [ 0xa675, 'UnicodeCharacter0xa675', [], 'Display', None],
  [ 0xa676, 'UnicodeCharacter0xa676', [], 'Display', None],
  [ 0xa677, 'UnicodeCharacter0xa677', [], 'Display', None],
  [ 0xa678, 'UnicodeCharacter0xa678', [], 'Display', None],
  [ 0xa679, 'UnicodeCharacter0xa679', [], 'Display', None],
  [ 0xa67a, 'UnicodeCharacter0xa67a', [], 'Display', None],
  [ 0xa67b, 'UnicodeCharacter0xa67b', [], 'Display', None],
  [ 0xa67c, 'CombiningCyrillicKavyka', [], 'Display', None],
  [ 0xa67d, 'CombiningCyrillicPayerok', [], 'Display', None],
  [ 0xa67e, 'CyrillicKavyka', [], 'Display', None],
  [ 0xa67f, 'CyrillicPayerok', [], 'Display', None],
  [ 0xa680, 'CyrillicCapitalLetterDwe', [], 'Display', None],
  [ 0xa681, 'CyrillicSmallLetterDwe', [], 'Display', None],
  [ 0xa682, 'CyrillicCapitalLetterDzwe', [], 'Display', None],
  [ 0xa683, 'CyrillicSmallLetterDzwe', [], 'Display', None],
  [ 0xa684, 'CyrillicCapitalLetterZhwe', [], 'Display', None],
  [ 0xa685, 'CyrillicSmallLetterZhwe', [], 'Display', None],
  [ 0xa686, 'CyrillicCapitalLetterCche', [], 'Display', None],
  [ 0xa687, 'CyrillicSmallLetterCche', [], 'Display', None],
  [ 0xa688, 'CyrillicCapitalLetterDzze', [], 'Display', None],
  [ 0xa689, 'CyrillicSmallLetterDzze', [], 'Display', None],
  [ 0xa68a, 'CyrillicCapitalLetterTeWithMiddleHook', [], 'Display', None],
  [ 0xa68b, 'CyrillicSmallLetterTeWithMiddleHook', [], 'Display', None],
  [ 0xa68c, 'CyrillicCapitalLetterTwe', [], 'Display', None],
  [ 0xa68d, 'CyrillicSmallLetterTwe', [], 'Display', None],
  [ 0xa68e, 'CyrillicCapitalLetterTswe', [], 'Display', None],
  [ 0xa68f, 'CyrillicSmallLetterTswe', [], 'Display', None],
  [ 0xa690, 'CyrillicCapitalLetterTsse', [], 'Display', None],
  [ 0xa691, 'CyrillicSmallLetterTsse', [], 'Display', None],
  [ 0xa692, 'CyrillicCapitalLetterTche', [], 'Display', None],
  [ 0xa693, 'CyrillicSmallLetterTche', [], 'Display', None],
  [ 0xa694, 'CyrillicCapitalLetterHwe', [], 'Display', None],
  [ 0xa695, 'CyrillicSmallLetterHwe', [], 'Display', None],
  [ 0xa696, 'CyrillicCapitalLetterShwe', [], 'Display', None],
  [ 0xa697, 'CyrillicSmallLetterShwe', [], 'Display', None],
  [ 0xa698, 'UnicodeCharacter0xa698', [], 'Display', None],
  [ 0xa699, 'UnicodeCharacter0xa699', [], 'Display', None],
  [ 0xa69a, 'UnicodeCharacter0xa69a', [], 'Display', None],
  [ 0xa69b, 'UnicodeCharacter0xa69b', [], 'Display', None],
  [ 0xa69c, 'UnicodeCharacter0xa69c', [], 'Display', None],
  [ 0xa69d, 'UnicodeCharacter0xa69d', [], 'Display', None],
  [ 0xa69e, 'UnicodeCharacter0xa69e', [], 'Display', None],
  [ 0xa69f, 'UnicodeCharacter0xa69f', [], 'Display', None],
  [ 0xa6a0, 'UnicodeCharacter0xa6a0', [], 'Display', None],
  [ 0xa6a1, 'UnicodeCharacter0xa6a1', [], 'Display', None],
  [ 0xa6a2, 'UnicodeCharacter0xa6a2', [], 'Display', None],
  [ 0xa6a3, 'UnicodeCharacter0xa6a3', [], 'Display', None],
  [ 0xa6a4, 'UnicodeCharacter0xa6a4', [], 'Display', None],
  [ 0xa6a5, 'UnicodeCharacter0xa6a5', [], 'Display', None],
  [ 0xa6a6, 'UnicodeCharacter0xa6a6', [], 'Display', None],
  [ 0xa6a7, 'UnicodeCharacter0xa6a7', [], 'Display', None],
  [ 0xa6a8, 'UnicodeCharacter0xa6a8', [], 'Display', None],
  [ 0xa6a9, 'UnicodeCharacter0xa6a9', [], 'Display', None],
  [ 0xa6aa, 'UnicodeCharacter0xa6aa', [], 'Display', None],
  [ 0xa6ab, 'UnicodeCharacter0xa6ab', [], 'Display', None],
  [ 0xa6ac, 'UnicodeCharacter0xa6ac', [], 'Display', None],
  [ 0xa6ad, 'UnicodeCharacter0xa6ad', [], 'Display', None],
  [ 0xa6ae, 'UnicodeCharacter0xa6ae', [], 'Display', None],
  [ 0xa6af, 'UnicodeCharacter0xa6af', [], 'Display', None],
  [ 0xa6b0, 'UnicodeCharacter0xa6b0', [], 'Display', None],
  [ 0xa6b1, 'UnicodeCharacter0xa6b1', [], 'Display', None],
  [ 0xa6b2, 'UnicodeCharacter0xa6b2', [], 'Display', None],
  [ 0xa6b3, 'UnicodeCharacter0xa6b3', [], 'Display', None],
  [ 0xa6b4, 'UnicodeCharacter0xa6b4', [], 'Display', None],
  [ 0xa6b5, 'UnicodeCharacter0xa6b5', [], 'Display', None],
  [ 0xa6b6, 'UnicodeCharacter0xa6b6', [], 'Display', None],
  [ 0xa6b7, 'UnicodeCharacter0xa6b7', [], 'Display', None],
  [ 0xa6b8, 'UnicodeCharacter0xa6b8', [], 'Display', None],
  [ 0xa6b9, 'UnicodeCharacter0xa6b9', [], 'Display', None],
  [ 0xa6ba, 'UnicodeCharacter0xa6ba', [], 'Display', None],
  [ 0xa6bb, 'UnicodeCharacter0xa6bb', [], 'Display', None],
  [ 0xa6bc, 'UnicodeCharacter0xa6bc', [], 'Display', None],
  [ 0xa6bd, 'UnicodeCharacter0xa6bd', [], 'Display', None],
  [ 0xa6be, 'UnicodeCharacter0xa6be', [], 'Display', None],
  [ 0xa6bf, 'UnicodeCharacter0xa6bf', [], 'Display', None],
  [ 0xa6c0, 'UnicodeCharacter0xa6c0', [], 'Display', None],
  [ 0xa6c1, 'UnicodeCharacter0xa6c1', [], 'Display', None],
  [ 0xa6c2, 'UnicodeCharacter0xa6c2', [], 'Display', None],
  [ 0xa6c3, 'UnicodeCharacter0xa6c3', [], 'Display', None],
  [ 0xa6c4, 'UnicodeCharacter0xa6c4', [], 'Display', None],
  [ 0xa6c5, 'UnicodeCharacter0xa6c5', [], 'Display', None],
  [ 0xa6c6, 'UnicodeCharacter0xa6c6', [], 'Display', None],
  [ 0xa6c7, 'UnicodeCharacter0xa6c7', [], 'Display', None],
  [ 0xa6c8, 'UnicodeCharacter0xa6c8', [], 'Display', None],
  [ 0xa6c9, 'UnicodeCharacter0xa6c9', [], 'Display', None],
  [ 0xa6ca, 'UnicodeCharacter0xa6ca', [], 'Display', None],
  [ 0xa6cb, 'UnicodeCharacter0xa6cb', [], 'Display', None],
  [ 0xa6cc, 'UnicodeCharacter0xa6cc', [], 'Display', None],
  [ 0xa6cd, 'UnicodeCharacter0xa6cd', [], 'Display', None],
  [ 0xa6ce, 'UnicodeCharacter0xa6ce', [], 'Display', None],
  [ 0xa6cf, 'UnicodeCharacter0xa6cf', [], 'Display', None],
  [ 0xa6d0, 'UnicodeCharacter0xa6d0', [], 'Display', None],
  [ 0xa6d1, 'UnicodeCharacter0xa6d1', [], 'Display', None],
  [ 0xa6d2, 'UnicodeCharacter0xa6d2', [], 'Display', None],
  [ 0xa6d3, 'UnicodeCharacter0xa6d3', [], 'Display', None],
  [ 0xa6d4, 'UnicodeCharacter0xa6d4', [], 'Display', None],
  [ 0xa6d5, 'UnicodeCharacter0xa6d5', [], 'Display', None],
  [ 0xa6d6, 'UnicodeCharacter0xa6d6', [], 'Display', None],
  [ 0xa6d7, 'UnicodeCharacter0xa6d7', [], 'Display', None],
  [ 0xa6d8, 'UnicodeCharacter0xa6d8', [], 'Display', None],
  [ 0xa6d9, 'UnicodeCharacter0xa6d9', [], 'Display', None],
  [ 0xa6da, 'UnicodeCharacter0xa6da', [], 'Display', None],
  [ 0xa6db, 'UnicodeCharacter0xa6db', [], 'Display', None],
  [ 0xa6dc, 'UnicodeCharacter0xa6dc', [], 'Display', None],
  [ 0xa6dd, 'UnicodeCharacter0xa6dd', [], 'Display', None],
  [ 0xa6de, 'UnicodeCharacter0xa6de', [], 'Display', None],
  [ 0xa6df, 'UnicodeCharacter0xa6df', [], 'Display', None],
  [ 0xa6e0, 'UnicodeCharacter0xa6e0', [], 'Display', None],
  [ 0xa6e1, 'UnicodeCharacter0xa6e1', [], 'Display', None],
  [ 0xa6e2, 'UnicodeCharacter0xa6e2', [], 'Display', None],
  [ 0xa6e3, 'UnicodeCharacter0xa6e3', [], 'Display', None],
  [ 0xa6e4, 'UnicodeCharacter0xa6e4', [], 'Display', None],
  [ 0xa6e5, 'UnicodeCharacter0xa6e5', [], 'Display', None],
  [ 0xa6e6, 'UnicodeCharacter0xa6e6', [], 'Display', None],
  [ 0xa6e7, 'UnicodeCharacter0xa6e7', [], 'Display', None],
  [ 0xa6e8, 'UnicodeCharacter0xa6e8', [], 'Display', None],
  [ 0xa6e9, 'UnicodeCharacter0xa6e9', [], 'Display', None],
  [ 0xa6ea, 'UnicodeCharacter0xa6ea', [], 'Display', None],
  [ 0xa6eb, 'UnicodeCharacter0xa6eb', [], 'Display', None],
  [ 0xa6ec, 'UnicodeCharacter0xa6ec', [], 'Display', None],
  [ 0xa6ed, 'UnicodeCharacter0xa6ed', [], 'Display', None],
  [ 0xa6ee, 'UnicodeCharacter0xa6ee', [], 'Display', None],
  [ 0xa6ef, 'UnicodeCharacter0xa6ef', [], 'Display', None],
  [ 0xa6f0, 'UnicodeCharacter0xa6f0', [], 'Display', None],
  [ 0xa6f1, 'UnicodeCharacter0xa6f1', [], 'Display', None],
  [ 0xa6f2, 'UnicodeCharacter0xa6f2', [], 'Display', None],
  [ 0xa6f3, 'UnicodeCharacter0xa6f3', [], 'Display', None],
  [ 0xa6f4, 'UnicodeCharacter0xa6f4', [], 'Display', None],
  [ 0xa6f5, 'UnicodeCharacter0xa6f5', [], 'Display', None],
  [ 0xa6f6, 'UnicodeCharacter0xa6f6', [], 'Display', None],
  [ 0xa6f7, 'UnicodeCharacter0xa6f7', [], 'Display', None],
  [ 0xa6f8, 'UnicodeCharacter0xa6f8', [], 'Display', None],
  [ 0xa6f9, 'UnicodeCharacter0xa6f9', [], 'Display', None],
  [ 0xa6fa, 'UnicodeCharacter0xa6fa', [], 'Display', None],
  [ 0xa6fb, 'UnicodeCharacter0xa6fb', [], 'Display', None],
  [ 0xa6fc, 'UnicodeCharacter0xa6fc', [], 'Display', None],
  [ 0xa6fd, 'UnicodeCharacter0xa6fd', [], 'Display', None],
  [ 0xa6fe, 'UnicodeCharacter0xa6fe', [], 'Display', None],
  [ 0xa6ff, 'UnicodeCharacter0xa6ff', [], 'Display', None],
  [ 0xa700, 'ModifierLetterChineseToneYinPing', [], 'Display', None],
  [ 0xa701, 'ModifierLetterChineseToneYangPing', [], 'Display', None],
  [ 0xa702, 'ModifierLetterChineseToneYinShang', [], 'Display', None],
  [ 0xa703, 'ModifierLetterChineseToneYangShang', [], 'Display', None],
  [ 0xa704, 'ModifierLetterChineseToneYinQu', [], 'Display', None],
  [ 0xa705, 'ModifierLetterChineseToneYangQu', [], 'Display', None],
  [ 0xa706, 'ModifierLetterChineseToneYinRu', [], 'Display', None],
  [ 0xa707, 'ModifierLetterChineseToneYangRu', [], 'Display', None],
  [ 0xa708, 'ModifierLetterExtrahighDottedToneBar', [], 'Display', None],
  [ 0xa709, 'ModifierLetterHighDottedToneBar', [], 'Display', None],
  [ 0xa70a, 'ModifierLetterMidDottedToneBar', [], 'Display', None],
  [ 0xa70b, 'ModifierLetterLowDottedToneBar', [], 'Display', None],
  [ 0xa70c, 'ModifierLetterExtralowDottedToneBar', [], 'Display', None],
  [ 0xa70d, 'ModifierLetterExtrahighDottedLeftstemToneBar', [], 'Display', None],
  [ 0xa70e, 'ModifierLetterHighDottedLeftstemToneBar', [], 'Display', None],
  [ 0xa70f, 'ModifierLetterMidDottedLeftstemToneBar', [], 'Display', None],
  [ 0xa710, 'ModifierLetterLowDottedLeftstemToneBar', [], 'Display', None],
  [ 0xa711, 'ModifierLetterExtralowDottedLeftstemToneBar', [], 'Display', None],
  [ 0xa712, 'ModifierLetterExtrahighLeftstemToneBar', [], 'Display', None],
  [ 0xa713, 'ModifierLetterHighLeftstemToneBar', [], 'Display', None],
  [ 0xa714, 'ModifierLetterMidLeftstemToneBar', [], 'Display', None],
  [ 0xa715, 'ModifierLetterLowLeftstemToneBar', [], 'Display', None],
  [ 0xa716, 'ModifierLetterExtralowLeftstemToneBar', [], 'Display', None],
  [ 0xa717, 'ModifierLetterDotVerticalBar', [], 'Display', None],
  [ 0xa718, 'ModifierLetterDotSlash', [], 'Display', None],
  [ 0xa719, 'ModifierLetterDotHorizontalBar', [], 'Display', None],
  [ 0xa71a, 'ModifierLetterLowerRightCornerAngle', [], 'Display', None],
  [ 0xa71b, 'ModifierLetterRaisedUpArrow', [], 'Display', None],
  [ 0xa71c, 'ModifierLetterRaisedDownArrow', [], 'Display', None],
  [ 0xa71d, 'ModifierLetterRaisedExclamationMark', [], 'Display', None],
  [ 0xa71e, 'ModifierLetterRaisedInvertedExclamationMark', [], 'Display', None],
  [ 0xa71f, 'ModifierLetterLowInvertedExclamationMark', [], 'Display', None],
  [ 0xa720, 'ModifierLetterStressAndHighTone', [], 'Display', None],
  [ 0xa721, 'ModifierLetterStressAndLowTone', [], 'Display', None],
  [ 0xa722, 'LatinCapitalLetterEgyptologicalAlef', [], 'Display', None],
  [ 0xa723, 'LatinSmallLetterEgyptologicalAlef', [], 'Display', None],
  [ 0xa724, 'LatinCapitalLetterEgyptologicalAin', [], 'Display', None],
  [ 0xa725, 'LatinSmallLetterEgyptologicalAin', [], 'Display', None],
  [ 0xa726, 'LatinCapitalLetterHeng', [], 'Display', None],
  [ 0xa727, 'LatinSmallLetterHeng', [], 'Display', None],
  [ 0xa728, 'LatinCapitalLetterTz', [], 'Display', None],
  [ 0xa729, 'LatinSmallLetterTz', [], 'Display', None],
  [ 0xa72a, 'LatinCapitalLetterTresillo', [], 'Display', None],
  [ 0xa72b, 'LatinSmallLetterTresillo', [], 'Display', None],
  [ 0xa72c, 'LatinCapitalLetterCuatrillo', [], 'Display', None],
  [ 0xa72d, 'LatinSmallLetterCuatrillo', [], 'Display', None],
  [ 0xa72e, 'LatinCapitalLetterCuatrilloWithComma', [], 'Display', None],
  [ 0xa72f, 'LatinSmallLetterCuatrilloWithComma', [], 'Display', None],
  [ 0xa730, 'LatinLetterSmallCapitalF', [], 'Display', None],
  [ 0xa731, 'LatinLetterSmallCapitalS', [], 'Display', None],
  [ 0xa732, 'LatinCapitalLetterAa', [], 'Display', None],
  [ 0xa733, 'LatinSmallLetterAa', [], 'Display', None],
  [ 0xa734, 'LatinCapitalLetterAo', [], 'Display', None],
  [ 0xa735, 'LatinSmallLetterAo', [], 'Display', None],
  [ 0xa736, 'LatinCapitalLetterAu', [], 'Display', None],
  [ 0xa737, 'LatinSmallLetterAu', [], 'Display', None],
  [ 0xa738, 'LatinCapitalLetterAv', [], 'Display', None],
  [ 0xa739, 'LatinSmallLetterAv', [], 'Display', None],
  [ 0xa73a, 'LatinCapitalLetterAvWithHorizontalBar', [], 'Display', None],
  [ 0xa73b, 'LatinSmallLetterAvWithHorizontalBar', [], 'Display', None],
  [ 0xa73c, 'LatinCapitalLetterAy', [], 'Display', None],
  [ 0xa73d, 'LatinSmallLetterAy', [], 'Display', None],
  [ 0xa73e, 'LatinCapitalLetterReversedCWithDot', [], 'Display', None],
  [ 0xa73f, 'LatinSmallLetterReversedCWithDot', [], 'Display', None],
  [ 0xa740, 'LatinCapitalLetterKWithStroke', [], 'Display', None],
  [ 0xa741, 'LatinSmallLetterKWithStroke', [], 'Display', None],
  [ 0xa742, 'LatinCapitalLetterKWithDiagonalStroke', [], 'Display', None],
  [ 0xa743, 'LatinSmallLetterKWithDiagonalStroke', [], 'Display', None],
  [ 0xa744, 'LatinCapitalLetterKWithStrokeAndDiagonalStroke', [], 'Display', None],
  [ 0xa745, 'LatinSmallLetterKWithStrokeAndDiagonalStroke', [], 'Display', None],
  [ 0xa746, 'LatinCapitalLetterBrokenL', [], 'Display', None],
  [ 0xa747, 'LatinSmallLetterBrokenL', [], 'Display', None],
  [ 0xa748, 'LatinCapitalLetterLWithHighStroke', [], 'Display', None],
  [ 0xa749, 'LatinSmallLetterLWithHighStroke', [], 'Display', None],
  [ 0xa74a, 'LatinCapitalLetterOWithLongStrokeOverlay', [], 'Display', None],
  [ 0xa74b, 'LatinSmallLetterOWithLongStrokeOverlay', [], 'Display', None],
  [ 0xa74c, 'LatinCapitalLetterOWithLoop', [], 'Display', None],
  [ 0xa74d, 'LatinSmallLetterOWithLoop', [], 'Display', None],
  [ 0xa74e, 'LatinCapitalLetterOo', [], 'Display', None],
  [ 0xa74f, 'LatinSmallLetterOo', [], 'Display', None],
  [ 0xa750, 'LatinCapitalLetterPWithStrokeThroughDescender', [], 'Display', None],
  [ 0xa751, 'LatinSmallLetterPWithStrokeThroughDescender', [], 'Display', None],
  [ 0xa752, 'LatinCapitalLetterPWithFlourish', [], 'Display', None],
  [ 0xa753, 'LatinSmallLetterPWithFlourish', [], 'Display', None],
  [ 0xa754, 'LatinCapitalLetterPWithSquirrelTail', [], 'Display', None],
  [ 0xa755, 'LatinSmallLetterPWithSquirrelTail', [], 'Display', None],
  [ 0xa756, 'LatinCapitalLetterQWithStrokeThroughDescender', [], 'Display', None],
  [ 0xa757, 'LatinSmallLetterQWithStrokeThroughDescender', [], 'Display', None],
  [ 0xa758, 'LatinCapitalLetterQWithDiagonalStroke', [], 'Display', None],
  [ 0xa759, 'LatinSmallLetterQWithDiagonalStroke', [], 'Display', None],
  [ 0xa75a, 'LatinCapitalLetterRRotunda', [], 'Display', None],
  [ 0xa75b, 'LatinSmallLetterRRotunda', [], 'Display', None],
  [ 0xa75c, 'LatinCapitalLetterRumRotunda', [], 'Display', None],
  [ 0xa75d, 'LatinSmallLetterRumRotunda', [], 'Display', None],
  [ 0xa75e, 'LatinCapitalLetterVWithDiagonalStroke', [], 'Display', None],
  [ 0xa75f, 'LatinSmallLetterVWithDiagonalStroke', [], 'Display', None],
  [ 0xa760, 'LatinCapitalLetterVy', [], 'Display', None],
  [ 0xa761, 'LatinSmallLetterVy', [], 'Display', None],
  [ 0xa762, 'LatinCapitalLetterVisigothicZ', [], 'Display', None],
  [ 0xa763, 'LatinSmallLetterVisigothicZ', [], 'Display', None],
  [ 0xa764, 'LatinCapitalLetterThornWithStroke', [], 'Display', None],
  [ 0xa765, 'LatinSmallLetterThornWithStroke', [], 'Display', None],
  [ 0xa766, 'LatinCapitalLetterThornWithStrokeThroughDescender', [], 'Display', None],
  [ 0xa767, 'LatinSmallLetterThornWithStrokeThroughDescender', [], 'Display', None],
  [ 0xa768, 'LatinCapitalLetterVend', [], 'Display', None],
  [ 0xa769, 'LatinSmallLetterVend', [], 'Display', None],
  [ 0xa76a, 'LatinCapitalLetterEt', [], 'Display', None],
  [ 0xa76b, 'LatinSmallLetterEt', [], 'Display', None],
  [ 0xa76c, 'LatinCapitalLetterIs', [], 'Display', None],
  [ 0xa76d, 'LatinSmallLetterIs', [], 'Display', None],
  [ 0xa76e, 'LatinCapitalLetterCon', [], 'Display', None],
  [ 0xa76f, 'LatinSmallLetterCon', [], 'Display', None],
  [ 0xa770, 'ModifierLetterUs', [], 'Display', None],
  [ 0xa771, 'LatinSmallLetterDum', [], 'Display', None],
  [ 0xa772, 'LatinSmallLetterLum', [], 'Display', None],
  [ 0xa773, 'LatinSmallLetterMum', [], 'Display', None],
  [ 0xa774, 'LatinSmallLetterNum', [], 'Display', None],
  [ 0xa775, 'LatinSmallLetterRum', [], 'Display', None],
  [ 0xa776, 'LatinLetterSmallCapitalRum', [], 'Display', None],
  [ 0xa777, 'LatinSmallLetterTum', [], 'Display', None],
  [ 0xa778, 'LatinSmallLetterUm', [], 'Display', None],
  [ 0xa779, 'LatinCapitalLetterInsularD', [], 'Display', None],
  [ 0xa77a, 'LatinSmallLetterInsularD', [], 'Display', None],
  [ 0xa77b, 'LatinCapitalLetterInsularF', [], 'Display', None],
  [ 0xa77c, 'LatinSmallLetterInsularF', [], 'Display', None],
  [ 0xa77d, 'LatinCapitalLetterInsularG', [], 'Display', None],
  [ 0xa77e, 'LatinCapitalLetterTurnedInsularG', [], 'Display', None],
  [ 0xa77f, 'LatinSmallLetterTurnedInsularG', [], 'Display', None],
  [ 0xa780, 'LatinCapitalLetterTurnedL', [], 'Display', None],
  [ 0xa781, 'LatinSmallLetterTurnedL', [], 'Display', None],
  [ 0xa782, 'LatinCapitalLetterInsularR', [], 'Display', None],
  [ 0xa783, 'LatinSmallLetterInsularR', [], 'Display', None],
  [ 0xa784, 'LatinCapitalLetterInsularS', [], 'Display', None],
  [ 0xa785, 'LatinSmallLetterInsularS', [], 'Display', None],
  [ 0xa786, 'LatinCapitalLetterInsularT', [], 'Display', None],
  [ 0xa787, 'LatinSmallLetterInsularT', [], 'Display', None],
  [ 0xa788, 'ModifierLetterLowCircumflexAccent', [], 'Display', None],
  [ 0xa789, 'ModifierLetterColon', [], 'Display', None],
  [ 0xa78a, 'ModifierLetterShortEqualsSign', [], 'Display', None],
  [ 0xa78b, 'LatinCapitalLetterSaltillo', [], 'Display', None],
  [ 0xa78c, 'LatinSmallLetterSaltillo', [], 'Display', None],
  [ 0xa78d, 'UnicodeCharacter0xa78d', [], 'Display', None],
  [ 0xa78e, 'UnicodeCharacter0xa78e', [], 'Display', None],
  [ 0xa78f, 'UnicodeCharacter0xa78f', [], 'Display', None],
  [ 0xa790, 'UnicodeCharacter0xa790', [], 'Display', None],
  [ 0xa791, 'UnicodeCharacter0xa791', [], 'Display', None],
  [ 0xa792, 'UnicodeCharacter0xa792', [], 'Display', None],
  [ 0xa793, 'UnicodeCharacter0xa793', [], 'Display', None],
  [ 0xa794, 'UnicodeCharacter0xa794', [], 'Display', None],
  [ 0xa795, 'UnicodeCharacter0xa795', [], 'Display', None],
  [ 0xa796, 'UnicodeCharacter0xa796', [], 'Display', None],
  [ 0xa797, 'UnicodeCharacter0xa797', [], 'Display', None],
  [ 0xa798, 'UnicodeCharacter0xa798', [], 'Display', None],
  [ 0xa799, 'UnicodeCharacter0xa799', [], 'Display', None],
  [ 0xa79a, 'UnicodeCharacter0xa79a', [], 'Display', None],
  [ 0xa79b, 'UnicodeCharacter0xa79b', [], 'Display', None],
  [ 0xa79c, 'UnicodeCharacter0xa79c', [], 'Display', None],
  [ 0xa79d, 'UnicodeCharacter0xa79d', [], 'Display', None],
  [ 0xa79e, 'UnicodeCharacter0xa79e', [], 'Display', None],
  [ 0xa79f, 'UnicodeCharacter0xa79f', [], 'Display', None],
  [ 0xa7a0, 'UnicodeCharacter0xa7a0', [], 'Display', None],
  [ 0xa7a1, 'UnicodeCharacter0xa7a1', [], 'Display', None],
  [ 0xa7a2, 'UnicodeCharacter0xa7a2', [], 'Display', None],
  [ 0xa7a3, 'UnicodeCharacter0xa7a3', [], 'Display', None],
  [ 0xa7a4, 'UnicodeCharacter0xa7a4', [], 'Display', None],
  [ 0xa7a5, 'UnicodeCharacter0xa7a5', [], 'Display', None],
  [ 0xa7a6, 'UnicodeCharacter0xa7a6', [], 'Display', None],
  [ 0xa7a7, 'UnicodeCharacter0xa7a7', [], 'Display', None],
  [ 0xa7a8, 'UnicodeCharacter0xa7a8', [], 'Display', None],
  [ 0xa7a9, 'UnicodeCharacter0xa7a9', [], 'Display', None],
  [ 0xa7aa, 'UnicodeCharacter0xa7aa', [], 'Display', None],
  [ 0xa7ab, 'UnicodeCharacter0xa7ab', [], 'Display', None],
  [ 0xa7ac, 'UnicodeCharacter0xa7ac', [], 'Display', None],
  [ 0xa7ad, 'UnicodeCharacter0xa7ad', [], 'Display', None],
  [ 0xa7ae, 'UnicodeCharacter0xa7ae', [], 'Display', None],
  [ 0xa7af, 'UnicodeCharacter0xa7af', [], 'Display', None],
  [ 0xa7b0, 'UnicodeCharacter0xa7b0', [], 'Display', None],
  [ 0xa7b1, 'UnicodeCharacter0xa7b1', [], 'Display', None],
  [ 0xa7b2, 'UnicodeCharacter0xa7b2', [], 'Display', None],
  [ 0xa7b3, 'UnicodeCharacter0xa7b3', [], 'Display', None],
  [ 0xa7b4, 'UnicodeCharacter0xa7b4', [], 'Display', None],
  [ 0xa7b5, 'UnicodeCharacter0xa7b5', [], 'Display', None],
  [ 0xa7b6, 'UnicodeCharacter0xa7b6', [], 'Display', None],
  [ 0xa7b7, 'UnicodeCharacter0xa7b7', [], 'Display', None],
  [ 0xa7b8, 'UnicodeCharacter0xa7b8', [], 'Display', None],
  [ 0xa7b9, 'UnicodeCharacter0xa7b9', [], 'Display', None],
  [ 0xa7ba, 'UnicodeCharacter0xa7ba', [], 'Display', None],
  [ 0xa7bb, 'UnicodeCharacter0xa7bb', [], 'Display', None],
  [ 0xa7bc, 'UnicodeCharacter0xa7bc', [], 'Display', None],
  [ 0xa7bd, 'UnicodeCharacter0xa7bd', [], 'Display', None],
  [ 0xa7be, 'UnicodeCharacter0xa7be', [], 'Display', None],
  [ 0xa7bf, 'UnicodeCharacter0xa7bf', [], 'Display', None],
  [ 0xa7c0, 'UnicodeCharacter0xa7c0', [], 'Display', None],
  [ 0xa7c1, 'UnicodeCharacter0xa7c1', [], 'Display', None],
  [ 0xa7c2, 'UnicodeCharacter0xa7c2', [], 'Display', None],
  [ 0xa7c3, 'UnicodeCharacter0xa7c3', [], 'Display', None],
  [ 0xa7c4, 'UnicodeCharacter0xa7c4', [], 'Display', None],
  [ 0xa7c5, 'UnicodeCharacter0xa7c5', [], 'Display', None],
  [ 0xa7c6, 'UnicodeCharacter0xa7c6', [], 'Display', None],
  [ 0xa7c7, 'UnicodeCharacter0xa7c7', [], 'Display', None],
  [ 0xa7c8, 'UnicodeCharacter0xa7c8', [], 'Display', None],
  [ 0xa7c9, 'UnicodeCharacter0xa7c9', [], 'Display', None],
  [ 0xa7ca, 'UnicodeCharacter0xa7ca', [], 'Display', None],
  [ 0xa7cb, 'UnicodeCharacter0xa7cb', [], 'Display', None],
  [ 0xa7cc, 'UnicodeCharacter0xa7cc', [], 'Display', None],
  [ 0xa7cd, 'UnicodeCharacter0xa7cd', [], 'Display', None],
  [ 0xa7ce, 'UnicodeCharacter0xa7ce', [], 'Display', None],
  [ 0xa7cf, 'UnicodeCharacter0xa7cf', [], 'Display', None],
  [ 0xa7d0, 'UnicodeCharacter0xa7d0', [], 'Display', None],
  [ 0xa7d1, 'UnicodeCharacter0xa7d1', [], 'Display', None],
  [ 0xa7d2, 'UnicodeCharacter0xa7d2', [], 'Display', None],
  [ 0xa7d3, 'UnicodeCharacter0xa7d3', [], 'Display', None],
  [ 0xa7d4, 'UnicodeCharacter0xa7d4', [], 'Display', None],
  [ 0xa7d5, 'UnicodeCharacter0xa7d5', [], 'Display', None],
  [ 0xa7d6, 'UnicodeCharacter0xa7d6', [], 'Display', None],
  [ 0xa7d7, 'UnicodeCharacter0xa7d7', [], 'Display', None],
  [ 0xa7d8, 'UnicodeCharacter0xa7d8', [], 'Display', None],
  [ 0xa7d9, 'UnicodeCharacter0xa7d9', [], 'Display', None],
  [ 0xa7da, 'UnicodeCharacter0xa7da', [], 'Display', None],
  [ 0xa7db, 'UnicodeCharacter0xa7db', [], 'Display', None],
  [ 0xa7dc, 'UnicodeCharacter0xa7dc', [], 'Display', None],
  [ 0xa7dd, 'UnicodeCharacter0xa7dd', [], 'Display', None],
  [ 0xa7de, 'UnicodeCharacter0xa7de', [], 'Display', None],
  [ 0xa7df, 'UnicodeCharacter0xa7df', [], 'Display', None],
  [ 0xa7e0, 'UnicodeCharacter0xa7e0', [], 'Display', None],
  [ 0xa7e1, 'UnicodeCharacter0xa7e1', [], 'Display', None],
  [ 0xa7e2, 'UnicodeCharacter0xa7e2', [], 'Display', None],
  [ 0xa7e3, 'UnicodeCharacter0xa7e3', [], 'Display', None],
  [ 0xa7e4, 'UnicodeCharacter0xa7e4', [], 'Display', None],
  [ 0xa7e5, 'UnicodeCharacter0xa7e5', [], 'Display', None],
  [ 0xa7e6, 'UnicodeCharacter0xa7e6', [], 'Display', None],
  [ 0xa7e7, 'UnicodeCharacter0xa7e7', [], 'Display', None],
  [ 0xa7e8, 'UnicodeCharacter0xa7e8', [], 'Display', None],
  [ 0xa7e9, 'UnicodeCharacter0xa7e9', [], 'Display', None],
  [ 0xa7ea, 'UnicodeCharacter0xa7ea', [], 'Display', None],
  [ 0xa7eb, 'UnicodeCharacter0xa7eb', [], 'Display', None],
  [ 0xa7ec, 'UnicodeCharacter0xa7ec', [], 'Display', None],
  [ 0xa7ed, 'UnicodeCharacter0xa7ed', [], 'Display', None],
  [ 0xa7ee, 'UnicodeCharacter0xa7ee', [], 'Display', None],
  [ 0xa7ef, 'UnicodeCharacter0xa7ef', [], 'Display', None],
  [ 0xa7f0, 'UnicodeCharacter0xa7f0', [], 'Display', None],
  [ 0xa7f1, 'UnicodeCharacter0xa7f1', [], 'Display', None],
  [ 0xa7f2, 'UnicodeCharacter0xa7f2', [], 'Display', None],
  [ 0xa7f3, 'UnicodeCharacter0xa7f3', [], 'Display', None],
  [ 0xa7f4, 'UnicodeCharacter0xa7f4', [], 'Display', None],
  [ 0xa7f5, 'UnicodeCharacter0xa7f5', [], 'Display', None],
  [ 0xa7f6, 'UnicodeCharacter0xa7f6', [], 'Display', None],
  [ 0xa7f7, 'UnicodeCharacter0xa7f7', [], 'Display', None],
  [ 0xa7f8, 'UnicodeCharacter0xa7f8', [], 'Display', None],
  [ 0xa7f9, 'UnicodeCharacter0xa7f9', [], 'Display', None],
  [ 0xa7fa, 'UnicodeCharacter0xa7fa', [], 'Display', None],
  [ 0xa7fb, 'LatinEpigraphicLetterReversedF', [], 'Display', None],
  [ 0xa7fc, 'LatinEpigraphicLetterReversedP', [], 'Display', None],
  [ 0xa7fd, 'LatinEpigraphicLetterInvertedM', [], 'Display', None],
  [ 0xa7fe, 'LatinEpigraphicLetterILonga', [], 'Display', None],
  [ 0xa7ff, 'LatinEpigraphicLetterArchaicM', [], 'Display', None],
  [ 0xa800, 'SylotiNagriLetterA', [], 'Display', None],
  [ 0xa801, 'SylotiNagriLetterI', [], 'Display', None],
  [ 0xa802, 'SylotiNagriSignDvisvara', [], 'Display', None],
  [ 0xa803, 'SylotiNagriLetterU', [], 'Display', None],
  [ 0xa804, 'SylotiNagriLetterE', [], 'Display', None],
  [ 0xa805, 'SylotiNagriLetterO', [], 'Display', None],
  [ 0xa806, 'SylotiNagriSignHasanta', [], 'Display', None],
  [ 0xa807, 'SylotiNagriLetterKo', [], 'Display', None],
  [ 0xa808, 'SylotiNagriLetterKho', [], 'Display', None],
  [ 0xa809, 'SylotiNagriLetterGo', [], 'Display', None],
  [ 0xa80a, 'SylotiNagriLetterGho', [], 'Display', None],
  [ 0xa80b, 'SylotiNagriSignAnusvara', [], 'Display', None],
  [ 0xa80c, 'SylotiNagriLetterCo', [], 'Display', None],
  [ 0xa80d, 'SylotiNagriLetterCho', [], 'Display', None],
  [ 0xa80e, 'SylotiNagriLetterJo', [], 'Display', None],
  [ 0xa80f, 'SylotiNagriLetterJho', [], 'Display', None],
  [ 0xa810, 'SylotiNagriLetterTto', [], 'Display', None],
  [ 0xa811, 'SylotiNagriLetterTtho', [], 'Display', None],
  [ 0xa812, 'SylotiNagriLetterDdo', [], 'Display', None],
  [ 0xa813, 'SylotiNagriLetterDdho', [], 'Display', None],
  [ 0xa814, 'SylotiNagriLetterTo', [], 'Display', None],
  [ 0xa815, 'SylotiNagriLetterTho', [], 'Display', None],
  [ 0xa816, 'SylotiNagriLetterDo', [], 'Display', None],
  [ 0xa817, 'SylotiNagriLetterDho', [], 'Display', None],
  [ 0xa818, 'SylotiNagriLetterNo', [], 'Display', None],
  [ 0xa819, 'SylotiNagriLetterPo', [], 'Display', None],
  [ 0xa81a, 'SylotiNagriLetterPho', [], 'Display', None],
  [ 0xa81b, 'SylotiNagriLetterBo', [], 'Display', None],
  [ 0xa81c, 'SylotiNagriLetterBho', [], 'Display', None],
  [ 0xa81d, 'SylotiNagriLetterMo', [], 'Display', None],
  [ 0xa81e, 'SylotiNagriLetterRo', [], 'Display', None],
  [ 0xa81f, 'SylotiNagriLetterLo', [], 'Display', None],
  [ 0xa820, 'SylotiNagriLetterRro', [], 'Display', None],
  [ 0xa821, 'SylotiNagriLetterSo', [], 'Display', None],
  [ 0xa822, 'SylotiNagriLetterHo', [], 'Display', None],
  [ 0xa823, 'SylotiNagriVowelSignA', [], 'Display', None],
  [ 0xa824, 'SylotiNagriVowelSignI', [], 'Display', None],
  [ 0xa825, 'SylotiNagriVowelSignU', [], 'Display', None],
  [ 0xa826, 'SylotiNagriVowelSignE', [], 'Display', None],
  [ 0xa827, 'SylotiNagriVowelSignOo', [], 'Display', None],
  [ 0xa828, 'SylotiNagriPoetryMark1', [], 'Display', None],
  [ 0xa829, 'SylotiNagriPoetryMark2', [], 'Display', None],
  [ 0xa82a, 'SylotiNagriPoetryMark3', [], 'Display', None],
  [ 0xa82b, 'SylotiNagriPoetryMark4', [], 'Display', None],
  [ 0xa82c, 'UnicodeCharacter0xa82c', [], 'Display', None],
  [ 0xa82d, 'UnicodeCharacter0xa82d', [], 'Display', None],
  [ 0xa82e, 'UnicodeCharacter0xa82e', [], 'Display', None],
  [ 0xa82f, 'UnicodeCharacter0xa82f', [], 'Display', None],
  [ 0xa830, 'UnicodeCharacter0xa830', [], 'Display', None],
  [ 0xa831, 'UnicodeCharacter0xa831', [], 'Display', None],
  [ 0xa832, 'UnicodeCharacter0xa832', [], 'Display', None],
  [ 0xa833, 'UnicodeCharacter0xa833', [], 'Display', None],
  [ 0xa834, 'UnicodeCharacter0xa834', [], 'Display', None],
  [ 0xa835, 'UnicodeCharacter0xa835', [], 'Display', None],
  [ 0xa836, 'UnicodeCharacter0xa836', [], 'Display', None],
  [ 0xa837, 'UnicodeCharacter0xa837', [], 'Display', None],
  [ 0xa838, 'UnicodeCharacter0xa838', [], 'Display', None],
  [ 0xa839, 'UnicodeCharacter0xa839', [], 'Display', None],
  [ 0xa83a, 'UnicodeCharacter0xa83a', [], 'Display', None],
  [ 0xa83b, 'UnicodeCharacter0xa83b', [], 'Display', None],
  [ 0xa83c, 'UnicodeCharacter0xa83c', [], 'Display', None],
  [ 0xa83d, 'UnicodeCharacter0xa83d', [], 'Display', None],
  [ 0xa83e, 'UnicodeCharacter0xa83e', [], 'Display', None],
  [ 0xa83f, 'UnicodeCharacter0xa83f', [], 'Display', None],
  [ 0xa840, 'PhagspaLetterKa', [], 'Display', None],
  [ 0xa841, 'PhagspaLetterKha', [], 'Display', None],
  [ 0xa842, 'PhagspaLetterGa', [], 'Display', None],
  [ 0xa843, 'PhagspaLetterNga', [], 'Display', None],
  [ 0xa844, 'PhagspaLetterCa', [], 'Display', None],
  [ 0xa845, 'PhagspaLetterCha', [], 'Display', None],
  [ 0xa846, 'PhagspaLetterJa', [], 'Display', None],
  [ 0xa847, 'PhagspaLetterNya', [], 'Display', None],
  [ 0xa848, 'PhagspaLetterTa', [], 'Display', None],
  [ 0xa849, 'PhagspaLetterTha', [], 'Display', None],
  [ 0xa84a, 'PhagspaLetterDa', [], 'Display', None],
  [ 0xa84b, 'PhagspaLetterNa', [], 'Display', None],
  [ 0xa84c, 'PhagspaLetterPa', [], 'Display', None],
  [ 0xa84d, 'PhagspaLetterPha', [], 'Display', None],
  [ 0xa84e, 'PhagspaLetterBa', [], 'Display', None],
  [ 0xa84f, 'PhagspaLetterMa', [], 'Display', None],
  [ 0xa850, 'PhagspaLetterTsa', [], 'Display', None],
  [ 0xa851, 'PhagspaLetterTsha', [], 'Display', None],
  [ 0xa852, 'PhagspaLetterDza', [], 'Display', None],
  [ 0xa853, 'PhagspaLetterWa', [], 'Display', None],
  [ 0xa854, 'PhagspaLetterZha', [], 'Display', None],
  [ 0xa855, 'PhagspaLetterZa', [], 'Display', None],
  [ 0xa856, 'PhagspaLetterSmallA', [], 'Display', None],
  [ 0xa857, 'PhagspaLetterYa', [], 'Display', None],
  [ 0xa858, 'PhagspaLetterRa', [], 'Display', None],
  [ 0xa859, 'PhagspaLetterLa', [], 'Display', None],
  [ 0xa85a, 'PhagspaLetterSha', [], 'Display', None],
  [ 0xa85b, 'PhagspaLetterSa', [], 'Display', None],
  [ 0xa85c, 'PhagspaLetterHa', [], 'Display', None],
  [ 0xa85d, 'PhagspaLetterA', [], 'Display', None],
  [ 0xa85e, 'PhagspaLetterI', [], 'Display', None],
  [ 0xa85f, 'PhagspaLetterU', [], 'Display', None],
  [ 0xa860, 'PhagspaLetterE', [], 'Display', None],
  [ 0xa861, 'PhagspaLetterO', [], 'Display', None],
  [ 0xa862, 'PhagspaLetterQa', [], 'Display', None],
  [ 0xa863, 'PhagspaLetterXa', [], 'Display', None],
  [ 0xa864, 'PhagspaLetterFa', [], 'Display', None],
  [ 0xa865, 'PhagspaLetterGga', [], 'Display', None],
  [ 0xa866, 'PhagspaLetterEe', [], 'Display', None],
  [ 0xa867, 'PhagspaSubjoinedLetterWa', [], 'Display', None],
  [ 0xa868, 'PhagspaSubjoinedLetterYa', [], 'Display', None],
  [ 0xa869, 'PhagspaLetterTta', [], 'Display', None],
  [ 0xa86a, 'PhagspaLetterTtha', [], 'Display', None],
  [ 0xa86b, 'PhagspaLetterDda', [], 'Display', None],
  [ 0xa86c, 'PhagspaLetterNna', [], 'Display', None],
  [ 0xa86d, 'PhagspaLetterAlternateYa', [], 'Display', None],
  [ 0xa86e, 'PhagspaLetterVoicelessSha', [], 'Display', None],
  [ 0xa86f, 'PhagspaLetterVoicedHa', [], 'Display', None],
  [ 0xa870, 'PhagspaLetterAspiratedFa', [], 'Display', None],
  [ 0xa871, 'PhagspaSubjoinedLetterRa', [], 'Display', None],
  [ 0xa872, 'PhagspaSuperfixedLetterRa', [], 'Display', None],
  [ 0xa873, 'PhagspaLetterCandrabindu', [], 'Display', None],
  [ 0xa874, 'PhagspaSingleHeadMark', [], 'Display', None],
  [ 0xa875, 'PhagspaDoubleHeadMark', [], 'Display', None],
  [ 0xa876, 'PhagspaMarkShad', [], 'Display', None],
  [ 0xa877, 'PhagspaMarkDoubleShad', [], 'Display', None],
  [ 0xa878, 'UnicodeCharacter0xa878', [], 'Display', None],
  [ 0xa879, 'UnicodeCharacter0xa879', [], 'Display', None],
  [ 0xa87a, 'UnicodeCharacter0xa87a', [], 'Display', None],
  [ 0xa87b, 'UnicodeCharacter0xa87b', [], 'Display', None],
  [ 0xa87c, 'UnicodeCharacter0xa87c', [], 'Display', None],
  [ 0xa87d, 'UnicodeCharacter0xa87d', [], 'Display', None],
  [ 0xa87e, 'UnicodeCharacter0xa87e', [], 'Display', None],
  [ 0xa87f, 'UnicodeCharacter0xa87f', [], 'Display', None],
  [ 0xa880, 'SaurashtraSignAnusvara', [], 'Display', None],
  [ 0xa881, 'SaurashtraSignVisarga', [], 'Display', None],
  [ 0xa882, 'SaurashtraLetterA', [], 'Display', None],
  [ 0xa883, 'SaurashtraLetterAa', [], 'Display', None],
  [ 0xa884, 'SaurashtraLetterI', [], 'Display', None],
  [ 0xa885, 'SaurashtraLetterIi', [], 'Display', None],
  [ 0xa886, 'SaurashtraLetterU', [], 'Display', None],
  [ 0xa887, 'SaurashtraLetterUu', [], 'Display', None],
  [ 0xa888, 'SaurashtraLetterVocalicR', [], 'Display', None],
  [ 0xa889, 'SaurashtraLetterVocalicRr', [], 'Display', None],
  [ 0xa88a, 'SaurashtraLetterVocalicL', [], 'Display', None],
  [ 0xa88b, 'SaurashtraLetterVocalicLl', [], 'Display', None],
  [ 0xa88c, 'SaurashtraLetterE', [], 'Display', None],
  [ 0xa88d, 'SaurashtraLetterEe', [], 'Display', None],
  [ 0xa88e, 'SaurashtraLetterAi', [], 'Display', None],
  [ 0xa88f, 'SaurashtraLetterO', [], 'Display', None],
  [ 0xa890, 'SaurashtraLetterOo', [], 'Display', None],
  [ 0xa891, 'SaurashtraLetterAu', [], 'Display', None],
  [ 0xa892, 'SaurashtraLetterKa', [], 'Display', None],
  [ 0xa893, 'SaurashtraLetterKha', [], 'Display', None],
  [ 0xa894, 'SaurashtraLetterGa', [], 'Display', None],
  [ 0xa895, 'SaurashtraLetterGha', [], 'Display', None],
  [ 0xa896, 'SaurashtraLetterNga', [], 'Display', None],
  [ 0xa897, 'SaurashtraLetterCa', [], 'Display', None],
  [ 0xa898, 'SaurashtraLetterCha', [], 'Display', None],
  [ 0xa899, 'SaurashtraLetterJa', [], 'Display', None],
  [ 0xa89a, 'SaurashtraLetterJha', [], 'Display', None],
  [ 0xa89b, 'SaurashtraLetterNya', [], 'Display', None],
  [ 0xa89c, 'SaurashtraLetterTta', [], 'Display', None],
  [ 0xa89d, 'SaurashtraLetterTtha', [], 'Display', None],
  [ 0xa89e, 'SaurashtraLetterDda', [], 'Display', None],
  [ 0xa89f, 'SaurashtraLetterDdha', [], 'Display', None],
  [ 0xa8a0, 'SaurashtraLetterNna', [], 'Display', None],
  [ 0xa8a1, 'SaurashtraLetterTa', [], 'Display', None],
  [ 0xa8a2, 'SaurashtraLetterTha', [], 'Display', None],
  [ 0xa8a3, 'SaurashtraLetterDa', [], 'Display', None],
  [ 0xa8a4, 'SaurashtraLetterDha', [], 'Display', None],
  [ 0xa8a5, 'SaurashtraLetterNa', [], 'Display', None],
  [ 0xa8a6, 'SaurashtraLetterPa', [], 'Display', None],
  [ 0xa8a7, 'SaurashtraLetterPha', [], 'Display', None],
  [ 0xa8a8, 'SaurashtraLetterBa', [], 'Display', None],
  [ 0xa8a9, 'SaurashtraLetterBha', [], 'Display', None],
  [ 0xa8aa, 'SaurashtraLetterMa', [], 'Display', None],
  [ 0xa8ab, 'SaurashtraLetterYa', [], 'Display', None],
  [ 0xa8ac, 'SaurashtraLetterRa', [], 'Display', None],
  [ 0xa8ad, 'SaurashtraLetterLa', [], 'Display', None],
  [ 0xa8ae, 'SaurashtraLetterVa', [], 'Display', None],
  [ 0xa8af, 'SaurashtraLetterSha', [], 'Display', None],
  [ 0xa8b0, 'SaurashtraLetterSsa', [], 'Display', None],
  [ 0xa8b1, 'SaurashtraLetterSa', [], 'Display', None],
  [ 0xa8b2, 'SaurashtraLetterHa', [], 'Display', None],
  [ 0xa8b3, 'SaurashtraLetterLla', [], 'Display', None],
  [ 0xa8b4, 'SaurashtraConsonantSignHaaru', [], 'Display', None],
  [ 0xa8b5, 'SaurashtraVowelSignAa', [], 'Display', None],
  [ 0xa8b6, 'SaurashtraVowelSignI', [], 'Display', None],
  [ 0xa8b7, 'SaurashtraVowelSignIi', [], 'Display', None],
  [ 0xa8b8, 'SaurashtraVowelSignU', [], 'Display', None],
  [ 0xa8b9, 'SaurashtraVowelSignUu', [], 'Display', None],
  [ 0xa8ba, 'SaurashtraVowelSignVocalicR', [], 'Display', None],
  [ 0xa8bb, 'SaurashtraVowelSignVocalicRr', [], 'Display', None],
  [ 0xa8bc, 'SaurashtraVowelSignVocalicL', [], 'Display', None],
  [ 0xa8bd, 'SaurashtraVowelSignVocalicLl', [], 'Display', None],
  [ 0xa8be, 'SaurashtraVowelSignE', [], 'Display', None],
  [ 0xa8bf, 'SaurashtraVowelSignEe', [], 'Display', None],
  [ 0xa8c0, 'SaurashtraVowelSignAi', [], 'Display', None],
  [ 0xa8c1, 'SaurashtraVowelSignO', [], 'Display', None],
  [ 0xa8c2, 'SaurashtraVowelSignOo', [], 'Display', None],
  [ 0xa8c3, 'SaurashtraVowelSignAu', [], 'Display', None],
  [ 0xa8c4, 'SaurashtraSignVirama', [], 'Display', None],
  [ 0xa8c5, 'UnicodeCharacter0xa8c5', [], 'Display', None],
  [ 0xa8c6, 'UnicodeCharacter0xa8c6', [], 'Display', None],
  [ 0xa8c7, 'UnicodeCharacter0xa8c7', [], 'Display', None],
  [ 0xa8c8, 'UnicodeCharacter0xa8c8', [], 'Display', None],
  [ 0xa8c9, 'UnicodeCharacter0xa8c9', [], 'Display', None],
  [ 0xa8ca, 'UnicodeCharacter0xa8ca', [], 'Display', None],
  [ 0xa8cb, 'UnicodeCharacter0xa8cb', [], 'Display', None],
  [ 0xa8cc, 'UnicodeCharacter0xa8cc', [], 'Display', None],
  [ 0xa8cd, 'UnicodeCharacter0xa8cd', [], 'Display', None],
  [ 0xa8ce, 'SaurashtraDanda', [], 'Display', None],
  [ 0xa8cf, 'SaurashtraDoubleDanda', [], 'Display', None],
  [ 0xa8d0, 'SaurashtraDigitZero', [], 'Display', None],
  [ 0xa8d1, 'SaurashtraDigitOne', [], 'Display', None],
  [ 0xa8d2, 'SaurashtraDigitTwo', [], 'Display', None],
  [ 0xa8d3, 'SaurashtraDigitThree', [], 'Display', None],
  [ 0xa8d4, 'SaurashtraDigitFour', [], 'Display', None],
  [ 0xa8d5, 'SaurashtraDigitFive', [], 'Display', None],
  [ 0xa8d6, 'SaurashtraDigitSix', [], 'Display', None],
  [ 0xa8d7, 'SaurashtraDigitSeven', [], 'Display', None],
  [ 0xa8d8, 'SaurashtraDigitEight', [], 'Display', None],
  [ 0xa8d9, 'SaurashtraDigitNine', [], 'Display', None],
  [ 0xa8da, 'UnicodeCharacter0xa8da', [], 'Display', None],
  [ 0xa8db, 'UnicodeCharacter0xa8db', [], 'Display', None],
  [ 0xa8dc, 'UnicodeCharacter0xa8dc', [], 'Display', None],
  [ 0xa8dd, 'UnicodeCharacter0xa8dd', [], 'Display', None],
  [ 0xa8de, 'UnicodeCharacter0xa8de', [], 'Display', None],
  [ 0xa8df, 'UnicodeCharacter0xa8df', [], 'Display', None],
  [ 0xa8e0, 'UnicodeCharacter0xa8e0', [], 'Display', None],
  [ 0xa8e1, 'UnicodeCharacter0xa8e1', [], 'Display', None],
  [ 0xa8e2, 'UnicodeCharacter0xa8e2', [], 'Display', None],
  [ 0xa8e3, 'UnicodeCharacter0xa8e3', [], 'Display', None],
  [ 0xa8e4, 'UnicodeCharacter0xa8e4', [], 'Display', None],
  [ 0xa8e5, 'UnicodeCharacter0xa8e5', [], 'Display', None],
  [ 0xa8e6, 'UnicodeCharacter0xa8e6', [], 'Display', None],
  [ 0xa8e7, 'UnicodeCharacter0xa8e7', [], 'Display', None],
  [ 0xa8e8, 'UnicodeCharacter0xa8e8', [], 'Display', None],
  [ 0xa8e9, 'UnicodeCharacter0xa8e9', [], 'Display', None],
  [ 0xa8ea, 'UnicodeCharacter0xa8ea', [], 'Display', None],
  [ 0xa8eb, 'UnicodeCharacter0xa8eb', [], 'Display', None],
  [ 0xa8ec, 'UnicodeCharacter0xa8ec', [], 'Display', None],
  [ 0xa8ed, 'UnicodeCharacter0xa8ed', [], 'Display', None],
  [ 0xa8ee, 'UnicodeCharacter0xa8ee', [], 'Display', None],
  [ 0xa8ef, 'UnicodeCharacter0xa8ef', [], 'Display', None],
  [ 0xa8f0, 'UnicodeCharacter0xa8f0', [], 'Display', None],
  [ 0xa8f1, 'UnicodeCharacter0xa8f1', [], 'Display', None],
  [ 0xa8f2, 'UnicodeCharacter0xa8f2', [], 'Display', None],
  [ 0xa8f3, 'UnicodeCharacter0xa8f3', [], 'Display', None],
  [ 0xa8f4, 'UnicodeCharacter0xa8f4', [], 'Display', None],
  [ 0xa8f5, 'UnicodeCharacter0xa8f5', [], 'Display', None],
  [ 0xa8f6, 'UnicodeCharacter0xa8f6', [], 'Display', None],
  [ 0xa8f7, 'UnicodeCharacter0xa8f7', [], 'Display', None],
  [ 0xa8f8, 'UnicodeCharacter0xa8f8', [], 'Display', None],
  [ 0xa8f9, 'UnicodeCharacter0xa8f9', [], 'Display', None],
  [ 0xa8fa, 'UnicodeCharacter0xa8fa', [], 'Display', None],
  [ 0xa8fb, 'UnicodeCharacter0xa8fb', [], 'Display', None],
  [ 0xa8fc, 'UnicodeCharacter0xa8fc', [], 'Display', None],
  [ 0xa8fd, 'UnicodeCharacter0xa8fd', [], 'Display', None],
  [ 0xa8fe, 'UnicodeCharacter0xa8fe', [], 'Display', None],
  [ 0xa8ff, 'UnicodeCharacter0xa8ff', [], 'Display', None],
  [ 0xa900, 'KayahLiDigitZero', [], 'Display', None],
  [ 0xa901, 'KayahLiDigitOne', [], 'Display', None],
  [ 0xa902, 'KayahLiDigitTwo', [], 'Display', None],
  [ 0xa903, 'KayahLiDigitThree', [], 'Display', None],
  [ 0xa904, 'KayahLiDigitFour', [], 'Display', None],
  [ 0xa905, 'KayahLiDigitFive', [], 'Display', None],
  [ 0xa906, 'KayahLiDigitSix', [], 'Display', None],
  [ 0xa907, 'KayahLiDigitSeven', [], 'Display', None],
  [ 0xa908, 'KayahLiDigitEight', [], 'Display', None],
  [ 0xa909, 'KayahLiDigitNine', [], 'Display', None],
  [ 0xa90a, 'KayahLiLetterKa', [], 'Display', None],
  [ 0xa90b, 'KayahLiLetterKha', [], 'Display', None],
  [ 0xa90c, 'KayahLiLetterGa', [], 'Display', None],
  [ 0xa90d, 'KayahLiLetterNga', [], 'Display', None],
  [ 0xa90e, 'KayahLiLetterSa', [], 'Display', None],
  [ 0xa90f, 'KayahLiLetterSha', [], 'Display', None],
  [ 0xa910, 'KayahLiLetterZa', [], 'Display', None],
  [ 0xa911, 'KayahLiLetterNya', [], 'Display', None],
  [ 0xa912, 'KayahLiLetterTa', [], 'Display', None],
  [ 0xa913, 'KayahLiLetterHta', [], 'Display', None],
  [ 0xa914, 'KayahLiLetterNa', [], 'Display', None],
  [ 0xa915, 'KayahLiLetterPa', [], 'Display', None],
  [ 0xa916, 'KayahLiLetterPha', [], 'Display', None],
  [ 0xa917, 'KayahLiLetterMa', [], 'Display', None],
  [ 0xa918, 'KayahLiLetterDa', [], 'Display', None],
  [ 0xa919, 'KayahLiLetterBa', [], 'Display', None],
  [ 0xa91a, 'KayahLiLetterRa', [], 'Display', None],
  [ 0xa91b, 'KayahLiLetterYa', [], 'Display', None],
  [ 0xa91c, 'KayahLiLetterLa', [], 'Display', None],
  [ 0xa91d, 'KayahLiLetterWa', [], 'Display', None],
  [ 0xa91e, 'KayahLiLetterTha', [], 'Display', None],
  [ 0xa91f, 'KayahLiLetterHa', [], 'Display', None],
  [ 0xa920, 'KayahLiLetterVa', [], 'Display', None],
  [ 0xa921, 'KayahLiLetterCa', [], 'Display', None],
  [ 0xa922, 'KayahLiLetterA', [], 'Display', None],
  [ 0xa923, 'KayahLiLetterOe', [], 'Display', None],
  [ 0xa924, 'KayahLiLetterI', [], 'Display', None],
  [ 0xa925, 'KayahLiLetterOo', [], 'Display', None],
  [ 0xa926, 'KayahLiVowelUe', [], 'Display', None],
  [ 0xa927, 'KayahLiVowelE', [], 'Display', None],
  [ 0xa928, 'KayahLiVowelU', [], 'Display', None],
  [ 0xa929, 'KayahLiVowelEe', [], 'Display', None],
  [ 0xa92a, 'KayahLiVowelO', [], 'Display', None],
  [ 0xa92b, 'KayahLiTonePlophu', [], 'Display', None],
  [ 0xa92c, 'KayahLiToneCalya', [], 'Display', None],
  [ 0xa92d, 'KayahLiToneCalyaPlophu', [], 'Display', None],
  [ 0xa92e, 'KayahLiSignCwi', [], 'Display', None],
  [ 0xa92f, 'KayahLiSignShya', [], 'Display', None],
  [ 0xa930, 'RejangLetterKa', [], 'Display', None],
  [ 0xa931, 'RejangLetterGa', [], 'Display', None],
  [ 0xa932, 'RejangLetterNga', [], 'Display', None],
  [ 0xa933, 'RejangLetterTa', [], 'Display', None],
  [ 0xa934, 'RejangLetterDa', [], 'Display', None],
  [ 0xa935, 'RejangLetterNa', [], 'Display', None],
  [ 0xa936, 'RejangLetterPa', [], 'Display', None],
  [ 0xa937, 'RejangLetterBa', [], 'Display', None],
  [ 0xa938, 'RejangLetterMa', [], 'Display', None],
  [ 0xa939, 'RejangLetterCa', [], 'Display', None],
  [ 0xa93a, 'RejangLetterJa', [], 'Display', None],
  [ 0xa93b, 'RejangLetterNya', [], 'Display', None],
  [ 0xa93c, 'RejangLetterSa', [], 'Display', None],
  [ 0xa93d, 'RejangLetterRa', [], 'Display', None],
  [ 0xa93e, 'RejangLetterLa', [], 'Display', None],
  [ 0xa93f, 'RejangLetterYa', [], 'Display', None],
  [ 0xa940, 'RejangLetterWa', [], 'Display', None],
  [ 0xa941, 'RejangLetterHa', [], 'Display', None],
  [ 0xa942, 'RejangLetterMba', [], 'Display', None],
  [ 0xa943, 'RejangLetterNgga', [], 'Display', None],
  [ 0xa944, 'RejangLetterNda', [], 'Display', None],
  [ 0xa945, 'RejangLetterNyja', [], 'Display', None],
  [ 0xa946, 'RejangLetterA', [], 'Display', None],
  [ 0xa947, 'RejangVowelSignI', [], 'Display', None],
  [ 0xa948, 'RejangVowelSignU', [], 'Display', None],
  [ 0xa949, 'RejangVowelSignE', [], 'Display', None],
  [ 0xa94a, 'RejangVowelSignAi', [], 'Display', None],
  [ 0xa94b, 'RejangVowelSignO', [], 'Display', None],
  [ 0xa94c, 'RejangVowelSignAu', [], 'Display', None],
  [ 0xa94d, 'RejangVowelSignEu', [], 'Display', None],
  [ 0xa94e, 'RejangVowelSignEa', [], 'Display', None],
  [ 0xa94f, 'RejangConsonantSignNg', [], 'Display', None],
  [ 0xa950, 'RejangConsonantSignN', [], 'Display', None],
  [ 0xa951, 'RejangConsonantSignR', [], 'Display', None],
  [ 0xa952, 'RejangConsonantSignH', [], 'Display', None],
  [ 0xa953, 'RejangVirama', [], 'Display', None],
  [ 0xa954, 'UnicodeCharacter0xa954', [], 'Display', None],
  [ 0xa955, 'UnicodeCharacter0xa955', [], 'Display', None],
  [ 0xa956, 'UnicodeCharacter0xa956', [], 'Display', None],
  [ 0xa957, 'UnicodeCharacter0xa957', [], 'Display', None],
  [ 0xa958, 'UnicodeCharacter0xa958', [], 'Display', None],
  [ 0xa959, 'UnicodeCharacter0xa959', [], 'Display', None],
  [ 0xa95a, 'UnicodeCharacter0xa95a', [], 'Display', None],
  [ 0xa95b, 'UnicodeCharacter0xa95b', [], 'Display', None],
  [ 0xa95c, 'UnicodeCharacter0xa95c', [], 'Display', None],
  [ 0xa95d, 'UnicodeCharacter0xa95d', [], 'Display', None],
  [ 0xa95e, 'UnicodeCharacter0xa95e', [], 'Display', None],
  [ 0xa95f, 'RejangSectionMark', [], 'Display', None],
  [ 0xa960, 'UnicodeCharacter0xa960', [], 'Display', None],
  [ 0xa961, 'UnicodeCharacter0xa961', [], 'Display', None],
  [ 0xa962, 'UnicodeCharacter0xa962', [], 'Display', None],
  [ 0xa963, 'UnicodeCharacter0xa963', [], 'Display', None],
  [ 0xa964, 'UnicodeCharacter0xa964', [], 'Display', None],
  [ 0xa965, 'UnicodeCharacter0xa965', [], 'Display', None],
  [ 0xa966, 'UnicodeCharacter0xa966', [], 'Display', None],
  [ 0xa967, 'UnicodeCharacter0xa967', [], 'Display', None],
  [ 0xa968, 'UnicodeCharacter0xa968', [], 'Display', None],
  [ 0xa969, 'UnicodeCharacter0xa969', [], 'Display', None],
  [ 0xa96a, 'UnicodeCharacter0xa96a', [], 'Display', None],
  [ 0xa96b, 'UnicodeCharacter0xa96b', [], 'Display', None],
  [ 0xa96c, 'UnicodeCharacter0xa96c', [], 'Display', None],
  [ 0xa96d, 'UnicodeCharacter0xa96d', [], 'Display', None],
  [ 0xa96e, 'UnicodeCharacter0xa96e', [], 'Display', None],
  [ 0xa96f, 'UnicodeCharacter0xa96f', [], 'Display', None],
  [ 0xa970, 'UnicodeCharacter0xa970', [], 'Display', None],
  [ 0xa971, 'UnicodeCharacter0xa971', [], 'Display', None],
  [ 0xa972, 'UnicodeCharacter0xa972', [], 'Display', None],
  [ 0xa973, 'UnicodeCharacter0xa973', [], 'Display', None],
  [ 0xa974, 'UnicodeCharacter0xa974', [], 'Display', None],
  [ 0xa975, 'UnicodeCharacter0xa975', [], 'Display', None],
  [ 0xa976, 'UnicodeCharacter0xa976', [], 'Display', None],
  [ 0xa977, 'UnicodeCharacter0xa977', [], 'Display', None],
  [ 0xa978, 'UnicodeCharacter0xa978', [], 'Display', None],
  [ 0xa979, 'UnicodeCharacter0xa979', [], 'Display', None],
  [ 0xa97a, 'UnicodeCharacter0xa97a', [], 'Display', None],
  [ 0xa97b, 'UnicodeCharacter0xa97b', [], 'Display', None],
  [ 0xa97c, 'UnicodeCharacter0xa97c', [], 'Display', None],
  [ 0xa97d, 'UnicodeCharacter0xa97d', [], 'Display', None],
  [ 0xa97e, 'UnicodeCharacter0xa97e', [], 'Display', None],
  [ 0xa97f, 'UnicodeCharacter0xa97f', [], 'Display', None],
  [ 0xa980, 'UnicodeCharacter0xa980', [], 'Display', None],
  [ 0xa981, 'UnicodeCharacter0xa981', [], 'Display', None],
  [ 0xa982, 'UnicodeCharacter0xa982', [], 'Display', None],
  [ 0xa983, 'UnicodeCharacter0xa983', [], 'Display', None],
  [ 0xa984, 'UnicodeCharacter0xa984', [], 'Display', None],
  [ 0xa985, 'UnicodeCharacter0xa985', [], 'Display', None],
  [ 0xa986, 'UnicodeCharacter0xa986', [], 'Display', None],
  [ 0xa987, 'UnicodeCharacter0xa987', [], 'Display', None],
  [ 0xa988, 'UnicodeCharacter0xa988', [], 'Display', None],
  [ 0xa989, 'UnicodeCharacter0xa989', [], 'Display', None],
  [ 0xa98a, 'UnicodeCharacter0xa98a', [], 'Display', None],
  [ 0xa98b, 'UnicodeCharacter0xa98b', [], 'Display', None],
  [ 0xa98c, 'UnicodeCharacter0xa98c', [], 'Display', None],
  [ 0xa98d, 'UnicodeCharacter0xa98d', [], 'Display', None],
  [ 0xa98e, 'UnicodeCharacter0xa98e', [], 'Display', None],
  [ 0xa98f, 'UnicodeCharacter0xa98f', [], 'Display', None],
  [ 0xa990, 'UnicodeCharacter0xa990', [], 'Display', None],
  [ 0xa991, 'UnicodeCharacter0xa991', [], 'Display', None],
  [ 0xa992, 'UnicodeCharacter0xa992', [], 'Display', None],
  [ 0xa993, 'UnicodeCharacter0xa993', [], 'Display', None],
  [ 0xa994, 'UnicodeCharacter0xa994', [], 'Display', None],
  [ 0xa995, 'UnicodeCharacter0xa995', [], 'Display', None],
  [ 0xa996, 'UnicodeCharacter0xa996', [], 'Display', None],
  [ 0xa997, 'UnicodeCharacter0xa997', [], 'Display', None],
  [ 0xa998, 'UnicodeCharacter0xa998', [], 'Display', None],
  [ 0xa999, 'UnicodeCharacter0xa999', [], 'Display', None],
  [ 0xa99a, 'UnicodeCharacter0xa99a', [], 'Display', None],
  [ 0xa99b, 'UnicodeCharacter0xa99b', [], 'Display', None],
  [ 0xa99c, 'UnicodeCharacter0xa99c', [], 'Display', None],
  [ 0xa99d, 'UnicodeCharacter0xa99d', [], 'Display', None],
  [ 0xa99e, 'UnicodeCharacter0xa99e', [], 'Display', None],
  [ 0xa99f, 'UnicodeCharacter0xa99f', [], 'Display', None],
  [ 0xa9a0, 'UnicodeCharacter0xa9a0', [], 'Display', None],
  [ 0xa9a1, 'UnicodeCharacter0xa9a1', [], 'Display', None],
  [ 0xa9a2, 'UnicodeCharacter0xa9a2', [], 'Display', None],
  [ 0xa9a3, 'UnicodeCharacter0xa9a3', [], 'Display', None],
  [ 0xa9a4, 'UnicodeCharacter0xa9a4', [], 'Display', None],
  [ 0xa9a5, 'UnicodeCharacter0xa9a5', [], 'Display', None],
  [ 0xa9a6, 'UnicodeCharacter0xa9a6', [], 'Display', None],
  [ 0xa9a7, 'UnicodeCharacter0xa9a7', [], 'Display', None],
  [ 0xa9a8, 'UnicodeCharacter0xa9a8', [], 'Display', None],
  [ 0xa9a9, 'UnicodeCharacter0xa9a9', [], 'Display', None],
  [ 0xa9aa, 'UnicodeCharacter0xa9aa', [], 'Display', None],
  [ 0xa9ab, 'UnicodeCharacter0xa9ab', [], 'Display', None],
  [ 0xa9ac, 'UnicodeCharacter0xa9ac', [], 'Display', None],
  [ 0xa9ad, 'UnicodeCharacter0xa9ad', [], 'Display', None],
  [ 0xa9ae, 'UnicodeCharacter0xa9ae', [], 'Display', None],
  [ 0xa9af, 'UnicodeCharacter0xa9af', [], 'Display', None],
  [ 0xa9b0, 'UnicodeCharacter0xa9b0', [], 'Display', None],
  [ 0xa9b1, 'UnicodeCharacter0xa9b1', [], 'Display', None],
  [ 0xa9b2, 'UnicodeCharacter0xa9b2', [], 'Display', None],
  [ 0xa9b3, 'UnicodeCharacter0xa9b3', [], 'Display', None],
  [ 0xa9b4, 'UnicodeCharacter0xa9b4', [], 'Display', None],
  [ 0xa9b5, 'UnicodeCharacter0xa9b5', [], 'Display', None],
  [ 0xa9b6, 'UnicodeCharacter0xa9b6', [], 'Display', None],
  [ 0xa9b7, 'UnicodeCharacter0xa9b7', [], 'Display', None],
  [ 0xa9b8, 'UnicodeCharacter0xa9b8', [], 'Display', None],
  [ 0xa9b9, 'UnicodeCharacter0xa9b9', [], 'Display', None],
  [ 0xa9ba, 'UnicodeCharacter0xa9ba', [], 'Display', None],
  [ 0xa9bb, 'UnicodeCharacter0xa9bb', [], 'Display', None],
  [ 0xa9bc, 'UnicodeCharacter0xa9bc', [], 'Display', None],
  [ 0xa9bd, 'UnicodeCharacter0xa9bd', [], 'Display', None],
  [ 0xa9be, 'UnicodeCharacter0xa9be', [], 'Display', None],
  [ 0xa9bf, 'UnicodeCharacter0xa9bf', [], 'Display', None],
  [ 0xa9c0, 'UnicodeCharacter0xa9c0', [], 'Display', None],
  [ 0xa9c1, 'UnicodeCharacter0xa9c1', [], 'Display', None],
  [ 0xa9c2, 'UnicodeCharacter0xa9c2', [], 'Display', None],
  [ 0xa9c3, 'UnicodeCharacter0xa9c3', [], 'Display', None],
  [ 0xa9c4, 'UnicodeCharacter0xa9c4', [], 'Display', None],
  [ 0xa9c5, 'UnicodeCharacter0xa9c5', [], 'Display', None],
  [ 0xa9c6, 'UnicodeCharacter0xa9c6', [], 'Display', None],
  [ 0xa9c7, 'UnicodeCharacter0xa9c7', [], 'Display', None],
  [ 0xa9c8, 'UnicodeCharacter0xa9c8', [], 'Display', None],
  [ 0xa9c9, 'UnicodeCharacter0xa9c9', [], 'Display', None],
  [ 0xa9ca, 'UnicodeCharacter0xa9ca', [], 'Display', None],
  [ 0xa9cb, 'UnicodeCharacter0xa9cb', [], 'Display', None],
  [ 0xa9cc, 'UnicodeCharacter0xa9cc', [], 'Display', None],
  [ 0xa9cd, 'UnicodeCharacter0xa9cd', [], 'Display', None],
  [ 0xa9ce, 'UnicodeCharacter0xa9ce', [], 'Display', None],
  [ 0xa9cf, 'UnicodeCharacter0xa9cf', [], 'Display', None],
  [ 0xa9d0, 'UnicodeCharacter0xa9d0', [], 'Display', None],
  [ 0xa9d1, 'UnicodeCharacter0xa9d1', [], 'Display', None],
  [ 0xa9d2, 'UnicodeCharacter0xa9d2', [], 'Display', None],
  [ 0xa9d3, 'UnicodeCharacter0xa9d3', [], 'Display', None],
  [ 0xa9d4, 'UnicodeCharacter0xa9d4', [], 'Display', None],
  [ 0xa9d5, 'UnicodeCharacter0xa9d5', [], 'Display', None],
  [ 0xa9d6, 'UnicodeCharacter0xa9d6', [], 'Display', None],
  [ 0xa9d7, 'UnicodeCharacter0xa9d7', [], 'Display', None],
  [ 0xa9d8, 'UnicodeCharacter0xa9d8', [], 'Display', None],
  [ 0xa9d9, 'UnicodeCharacter0xa9d9', [], 'Display', None],
  [ 0xa9da, 'UnicodeCharacter0xa9da', [], 'Display', None],
  [ 0xa9db, 'UnicodeCharacter0xa9db', [], 'Display', None],
  [ 0xa9dc, 'UnicodeCharacter0xa9dc', [], 'Display', None],
  [ 0xa9dd, 'UnicodeCharacter0xa9dd', [], 'Display', None],
  [ 0xa9de, 'UnicodeCharacter0xa9de', [], 'Display', None],
  [ 0xa9df, 'UnicodeCharacter0xa9df', [], 'Display', None],
  [ 0xa9e0, 'UnicodeCharacter0xa9e0', [], 'Display', None],
  [ 0xa9e1, 'UnicodeCharacter0xa9e1', [], 'Display', None],
  [ 0xa9e2, 'UnicodeCharacter0xa9e2', [], 'Display', None],
  [ 0xa9e3, 'UnicodeCharacter0xa9e3', [], 'Display', None],
  [ 0xa9e4, 'UnicodeCharacter0xa9e4', [], 'Display', None],
  [ 0xa9e5, 'UnicodeCharacter0xa9e5', [], 'Display', None],
  [ 0xa9e6, 'UnicodeCharacter0xa9e6', [], 'Display', None],
  [ 0xa9e7, 'UnicodeCharacter0xa9e7', [], 'Display', None],
  [ 0xa9e8, 'UnicodeCharacter0xa9e8', [], 'Display', None],
  [ 0xa9e9, 'UnicodeCharacter0xa9e9', [], 'Display', None],
  [ 0xa9ea, 'UnicodeCharacter0xa9ea', [], 'Display', None],
  [ 0xa9eb, 'UnicodeCharacter0xa9eb', [], 'Display', None],
  [ 0xa9ec, 'UnicodeCharacter0xa9ec', [], 'Display', None],
  [ 0xa9ed, 'UnicodeCharacter0xa9ed', [], 'Display', None],
  [ 0xa9ee, 'UnicodeCharacter0xa9ee', [], 'Display', None],
  [ 0xa9ef, 'UnicodeCharacter0xa9ef', [], 'Display', None],
  [ 0xa9f0, 'UnicodeCharacter0xa9f0', [], 'Display', None],
  [ 0xa9f1, 'UnicodeCharacter0xa9f1', [], 'Display', None],
  [ 0xa9f2, 'UnicodeCharacter0xa9f2', [], 'Display', None],
  [ 0xa9f3, 'UnicodeCharacter0xa9f3', [], 'Display', None],
  [ 0xa9f4, 'UnicodeCharacter0xa9f4', [], 'Display', None],
  [ 0xa9f5, 'UnicodeCharacter0xa9f5', [], 'Display', None],
  [ 0xa9f6, 'UnicodeCharacter0xa9f6', [], 'Display', None],
  [ 0xa9f7, 'UnicodeCharacter0xa9f7', [], 'Display', None],
  [ 0xa9f8, 'UnicodeCharacter0xa9f8', [], 'Display', None],
  [ 0xa9f9, 'UnicodeCharacter0xa9f9', [], 'Display', None],
  [ 0xa9fa, 'UnicodeCharacter0xa9fa', [], 'Display', None],
  [ 0xa9fb, 'UnicodeCharacter0xa9fb', [], 'Display', None],
  [ 0xa9fc, 'UnicodeCharacter0xa9fc', [], 'Display', None],
  [ 0xa9fd, 'UnicodeCharacter0xa9fd', [], 'Display', None],
  [ 0xa9fe, 'UnicodeCharacter0xa9fe', [], 'Display', None],
  [ 0xa9ff, 'UnicodeCharacter0xa9ff', [], 'Display', None],
  [ 0xaa00, 'ChamLetterA', [], 'Display', None],
  [ 0xaa01, 'ChamLetterI', [], 'Display', None],
  [ 0xaa02, 'ChamLetterU', [], 'Display', None],
  [ 0xaa03, 'ChamLetterE', [], 'Display', None],
  [ 0xaa04, 'ChamLetterAi', [], 'Display', None],
  [ 0xaa05, 'ChamLetterO', [], 'Display', None],
  [ 0xaa06, 'ChamLetterKa', [], 'Display', None],
  [ 0xaa07, 'ChamLetterKha', [], 'Display', None],
  [ 0xaa08, 'ChamLetterGa', [], 'Display', None],
  [ 0xaa09, 'ChamLetterGha', [], 'Display', None],
  [ 0xaa0a, 'ChamLetterNgue', [], 'Display', None],
  [ 0xaa0b, 'ChamLetterNga', [], 'Display', None],
  [ 0xaa0c, 'ChamLetterCha', [], 'Display', None],
  [ 0xaa0d, 'ChamLetterChha', [], 'Display', None],
  [ 0xaa0e, 'ChamLetterJa', [], 'Display', None],
  [ 0xaa0f, 'ChamLetterJha', [], 'Display', None],
  [ 0xaa10, 'ChamLetterNhue', [], 'Display', None],
  [ 0xaa11, 'ChamLetterNha', [], 'Display', None],
  [ 0xaa12, 'ChamLetterNhja', [], 'Display', None],
  [ 0xaa13, 'ChamLetterTa', [], 'Display', None],
  [ 0xaa14, 'ChamLetterTha', [], 'Display', None],
  [ 0xaa15, 'ChamLetterDa', [], 'Display', None],
  [ 0xaa16, 'ChamLetterDha', [], 'Display', None],
  [ 0xaa17, 'ChamLetterNue', [], 'Display', None],
  [ 0xaa18, 'ChamLetterNa', [], 'Display', None],
  [ 0xaa19, 'ChamLetterDda', [], 'Display', None],
  [ 0xaa1a, 'ChamLetterPa', [], 'Display', None],
  [ 0xaa1b, 'ChamLetterPpa', [], 'Display', None],
  [ 0xaa1c, 'ChamLetterPha', [], 'Display', None],
  [ 0xaa1d, 'ChamLetterBa', [], 'Display', None],
  [ 0xaa1e, 'ChamLetterBha', [], 'Display', None],
  [ 0xaa1f, 'ChamLetterMue', [], 'Display', None],
  [ 0xaa20, 'ChamLetterMa', [], 'Display', None],
  [ 0xaa21, 'ChamLetterBba', [], 'Display', None],
  [ 0xaa22, 'ChamLetterYa', [], 'Display', None],
  [ 0xaa23, 'ChamLetterRa', [], 'Display', None],
  [ 0xaa24, 'ChamLetterLa', [], 'Display', None],
  [ 0xaa25, 'ChamLetterVa', [], 'Display', None],
  [ 0xaa26, 'ChamLetterSsa', [], 'Display', None],
  [ 0xaa27, 'ChamLetterSa', [], 'Display', None],
  [ 0xaa28, 'ChamLetterHa', [], 'Display', None],
  [ 0xaa29, 'ChamVowelSignAa', [], 'Display', None],
  [ 0xaa2a, 'ChamVowelSignI', [], 'Display', None],
  [ 0xaa2b, 'ChamVowelSignIi', [], 'Display', None],
  [ 0xaa2c, 'ChamVowelSignEi', [], 'Display', None],
  [ 0xaa2d, 'ChamVowelSignU', [], 'Display', None],
  [ 0xaa2e, 'ChamVowelSignOe', [], 'Display', None],
  [ 0xaa2f, 'ChamVowelSignO', [], 'Display', None],
  [ 0xaa30, 'ChamVowelSignAi', [], 'Display', None],
  [ 0xaa31, 'ChamVowelSignAu', [], 'Display', None],
  [ 0xaa32, 'ChamVowelSignUe', [], 'Display', None],
  [ 0xaa33, 'ChamConsonantSignYa', [], 'Display', None],
  [ 0xaa34, 'ChamConsonantSignRa', [], 'Display', None],
  [ 0xaa35, 'ChamConsonantSignLa', [], 'Display', None],
  [ 0xaa36, 'ChamConsonantSignWa', [], 'Display', None],
  [ 0xaa37, 'UnicodeCharacter0xaa37', [], 'Display', None],
  [ 0xaa38, 'UnicodeCharacter0xaa38', [], 'Display', None],
  [ 0xaa39, 'UnicodeCharacter0xaa39', [], 'Display', None],
  [ 0xaa3a, 'UnicodeCharacter0xaa3a', [], 'Display', None],
  [ 0xaa3b, 'UnicodeCharacter0xaa3b', [], 'Display', None],
  [ 0xaa3c, 'UnicodeCharacter0xaa3c', [], 'Display', None],
  [ 0xaa3d, 'UnicodeCharacter0xaa3d', [], 'Display', None],
  [ 0xaa3e, 'UnicodeCharacter0xaa3e', [], 'Display', None],
  [ 0xaa3f, 'UnicodeCharacter0xaa3f', [], 'Display', None],
  [ 0xaa40, 'ChamLetterFinalK', [], 'Display', None],
  [ 0xaa41, 'ChamLetterFinalG', [], 'Display', None],
  [ 0xaa42, 'ChamLetterFinalNg', [], 'Display', None],
  [ 0xaa43, 'ChamConsonantSignFinalNg', [], 'Display', None],
  [ 0xaa44, 'ChamLetterFinalCh', [], 'Display', None],
  [ 0xaa45, 'ChamLetterFinalT', [], 'Display', None],
  [ 0xaa46, 'ChamLetterFinalN', [], 'Display', None],
  [ 0xaa47, 'ChamLetterFinalP', [], 'Display', None],
  [ 0xaa48, 'ChamLetterFinalY', [], 'Display', None],
  [ 0xaa49, 'ChamLetterFinalR', [], 'Display', None],
  [ 0xaa4a, 'ChamLetterFinalL', [], 'Display', None],
  [ 0xaa4b, 'ChamLetterFinalSs', [], 'Display', None],
  [ 0xaa4c, 'ChamConsonantSignFinalM', [], 'Display', None],
  [ 0xaa4d, 'ChamConsonantSignFinalH', [], 'Display', None],
  [ 0xaa4e, 'UnicodeCharacter0xaa4e', [], 'Display', None],
  [ 0xaa4f, 'UnicodeCharacter0xaa4f', [], 'Display', None],
  [ 0xaa50, 'ChamDigitZero', [], 'Display', None],
  [ 0xaa51, 'ChamDigitOne', [], 'Display', None],
  [ 0xaa52, 'ChamDigitTwo', [], 'Display', None],
  [ 0xaa53, 'ChamDigitThree', [], 'Display', None],
  [ 0xaa54, 'ChamDigitFour', [], 'Display', None],
  [ 0xaa55, 'ChamDigitFive', [], 'Display', None],
  [ 0xaa56, 'ChamDigitSix', [], 'Display', None],
  [ 0xaa57, 'ChamDigitSeven', [], 'Display', None],
  [ 0xaa58, 'ChamDigitEight', [], 'Display', None],
  [ 0xaa59, 'ChamDigitNine', [], 'Display', None],
  [ 0xaa5a, 'UnicodeCharacter0xaa5a', [], 'Display', None],
  [ 0xaa5b, 'UnicodeCharacter0xaa5b', [], 'Display', None],
  [ 0xaa5c, 'ChamPunctuationSpiral', [], 'Display', None],
  [ 0xaa5d, 'ChamPunctuationDanda', [], 'Display', None],
  [ 0xaa5e, 'ChamPunctuationDoubleDanda', [], 'Display', None],
  [ 0xaa5f, 'ChamPunctuationTripleDanda', [], 'Display', None],
  [ 0xaa60, 'UnicodeCharacter0xaa60', [], 'Display', None],
  [ 0xaa61, 'UnicodeCharacter0xaa61', [], 'Display', None],
  [ 0xaa62, 'UnicodeCharacter0xaa62', [], 'Display', None],
  [ 0xaa63, 'UnicodeCharacter0xaa63', [], 'Display', None],
  [ 0xaa64, 'UnicodeCharacter0xaa64', [], 'Display', None],
  [ 0xaa65, 'UnicodeCharacter0xaa65', [], 'Display', None],
  [ 0xaa66, 'UnicodeCharacter0xaa66', [], 'Display', None],
  [ 0xaa67, 'UnicodeCharacter0xaa67', [], 'Display', None],
  [ 0xaa68, 'UnicodeCharacter0xaa68', [], 'Display', None],
  [ 0xaa69, 'UnicodeCharacter0xaa69', [], 'Display', None],
  [ 0xaa6a, 'UnicodeCharacter0xaa6a', [], 'Display', None],
  [ 0xaa6b, 'UnicodeCharacter0xaa6b', [], 'Display', None],
  [ 0xaa6c, 'UnicodeCharacter0xaa6c', [], 'Display', None],
  [ 0xaa6d, 'UnicodeCharacter0xaa6d', [], 'Display', None],
  [ 0xaa6e, 'UnicodeCharacter0xaa6e', [], 'Display', None],
  [ 0xaa6f, 'UnicodeCharacter0xaa6f', [], 'Display', None],
  [ 0xaa70, 'UnicodeCharacter0xaa70', [], 'Display', None],
  [ 0xaa71, 'UnicodeCharacter0xaa71', [], 'Display', None],
  [ 0xaa72, 'UnicodeCharacter0xaa72', [], 'Display', None],
  [ 0xaa73, 'UnicodeCharacter0xaa73', [], 'Display', None],
  [ 0xaa74, 'UnicodeCharacter0xaa74', [], 'Display', None],
  [ 0xaa75, 'UnicodeCharacter0xaa75', [], 'Display', None],
  [ 0xaa76, 'UnicodeCharacter0xaa76', [], 'Display', None],
  [ 0xaa77, 'UnicodeCharacter0xaa77', [], 'Display', None],
  [ 0xaa78, 'UnicodeCharacter0xaa78', [], 'Display', None],
  [ 0xaa79, 'UnicodeCharacter0xaa79', [], 'Display', None],
  [ 0xaa7a, 'UnicodeCharacter0xaa7a', [], 'Display', None],
  [ 0xaa7b, 'UnicodeCharacter0xaa7b', [], 'Display', None],
  [ 0xaa7c, 'UnicodeCharacter0xaa7c', [], 'Display', None],
  [ 0xaa7d, 'UnicodeCharacter0xaa7d', [], 'Display', None],
  [ 0xaa7e, 'UnicodeCharacter0xaa7e', [], 'Display', None],
  [ 0xaa7f, 'UnicodeCharacter0xaa7f', [], 'Display', None],
  [ 0xaa80, 'UnicodeCharacter0xaa80', [], 'Display', None],
  [ 0xaa81, 'UnicodeCharacter0xaa81', [], 'Display', None],
  [ 0xaa82, 'UnicodeCharacter0xaa82', [], 'Display', None],
  [ 0xaa83, 'UnicodeCharacter0xaa83', [], 'Display', None],
  [ 0xaa84, 'UnicodeCharacter0xaa84', [], 'Display', None],
  [ 0xaa85, 'UnicodeCharacter0xaa85', [], 'Display', None],
  [ 0xaa86, 'UnicodeCharacter0xaa86', [], 'Display', None],
  [ 0xaa87, 'UnicodeCharacter0xaa87', [], 'Display', None],
  [ 0xaa88, 'UnicodeCharacter0xaa88', [], 'Display', None],
  [ 0xaa89, 'UnicodeCharacter0xaa89', [], 'Display', None],
  [ 0xaa8a, 'UnicodeCharacter0xaa8a', [], 'Display', None],
  [ 0xaa8b, 'UnicodeCharacter0xaa8b', [], 'Display', None],
  [ 0xaa8c, 'UnicodeCharacter0xaa8c', [], 'Display', None],
  [ 0xaa8d, 'UnicodeCharacter0xaa8d', [], 'Display', None],
  [ 0xaa8e, 'UnicodeCharacter0xaa8e', [], 'Display', None],
  [ 0xaa8f, 'UnicodeCharacter0xaa8f', [], 'Display', None],
  [ 0xaa90, 'UnicodeCharacter0xaa90', [], 'Display', None],
  [ 0xaa91, 'UnicodeCharacter0xaa91', [], 'Display', None],
  [ 0xaa92, 'UnicodeCharacter0xaa92', [], 'Display', None],
  [ 0xaa93, 'UnicodeCharacter0xaa93', [], 'Display', None],
  [ 0xaa94, 'UnicodeCharacter0xaa94', [], 'Display', None],
  [ 0xaa95, 'UnicodeCharacter0xaa95', [], 'Display', None],
  [ 0xaa96, 'UnicodeCharacter0xaa96', [], 'Display', None],
  [ 0xaa97, 'UnicodeCharacter0xaa97', [], 'Display', None],
  [ 0xaa98, 'UnicodeCharacter0xaa98', [], 'Display', None],
  [ 0xaa99, 'UnicodeCharacter0xaa99', [], 'Display', None],
  [ 0xaa9a, 'UnicodeCharacter0xaa9a', [], 'Display', None],
  [ 0xaa9b, 'UnicodeCharacter0xaa9b', [], 'Display', None],
  [ 0xaa9c, 'UnicodeCharacter0xaa9c', [], 'Display', None],
  [ 0xaa9d, 'UnicodeCharacter0xaa9d', [], 'Display', None],
  [ 0xaa9e, 'UnicodeCharacter0xaa9e', [], 'Display', None],
  [ 0xaa9f, 'UnicodeCharacter0xaa9f', [], 'Display', None],
  [ 0xaaa0, 'UnicodeCharacter0xaaa0', [], 'Display', None],
  [ 0xaaa1, 'UnicodeCharacter0xaaa1', [], 'Display', None],
  [ 0xaaa2, 'UnicodeCharacter0xaaa2', [], 'Display', None],
  [ 0xaaa3, 'UnicodeCharacter0xaaa3', [], 'Display', None],
  [ 0xaaa4, 'UnicodeCharacter0xaaa4', [], 'Display', None],
  [ 0xaaa5, 'UnicodeCharacter0xaaa5', [], 'Display', None],
  [ 0xaaa6, 'UnicodeCharacter0xaaa6', [], 'Display', None],
  [ 0xaaa7, 'UnicodeCharacter0xaaa7', [], 'Display', None],
  [ 0xaaa8, 'UnicodeCharacter0xaaa8', [], 'Display', None],
  [ 0xaaa9, 'UnicodeCharacter0xaaa9', [], 'Display', None],
  [ 0xaaaa, 'UnicodeCharacter0xaaaa', [], 'Display', None],
  [ 0xaaab, 'UnicodeCharacter0xaaab', [], 'Display', None],
  [ 0xaaac, 'UnicodeCharacter0xaaac', [], 'Display', None],
  [ 0xaaad, 'UnicodeCharacter0xaaad', [], 'Display', None],
  [ 0xaaae, 'UnicodeCharacter0xaaae', [], 'Display', None],
  [ 0xaaaf, 'UnicodeCharacter0xaaaf', [], 'Display', None],
  [ 0xaab0, 'UnicodeCharacter0xaab0', [], 'Display', None],
  [ 0xaab1, 'UnicodeCharacter0xaab1', [], 'Display', None],
  [ 0xaab2, 'UnicodeCharacter0xaab2', [], 'Display', None],
  [ 0xaab3, 'UnicodeCharacter0xaab3', [], 'Display', None],
  [ 0xaab4, 'UnicodeCharacter0xaab4', [], 'Display', None],
  [ 0xaab5, 'UnicodeCharacter0xaab5', [], 'Display', None],
  [ 0xaab6, 'UnicodeCharacter0xaab6', [], 'Display', None],
  [ 0xaab7, 'UnicodeCharacter0xaab7', [], 'Display', None],
  [ 0xaab8, 'UnicodeCharacter0xaab8', [], 'Display', None],
  [ 0xaab9, 'UnicodeCharacter0xaab9', [], 'Display', None],
  [ 0xaaba, 'UnicodeCharacter0xaaba', [], 'Display', None],
  [ 0xaabb, 'UnicodeCharacter0xaabb', [], 'Display', None],
  [ 0xaabc, 'UnicodeCharacter0xaabc', [], 'Display', None],
  [ 0xaabd, 'UnicodeCharacter0xaabd', [], 'Display', None],
  [ 0xaabe, 'UnicodeCharacter0xaabe', [], 'Display', None],
  [ 0xaabf, 'UnicodeCharacter0xaabf', [], 'Display', None],
  [ 0xaac0, 'UnicodeCharacter0xaac0', [], 'Display', None],
  [ 0xaac1, 'UnicodeCharacter0xaac1', [], 'Display', None],
  [ 0xaac2, 'UnicodeCharacter0xaac2', [], 'Display', None],
  [ 0xaac3, 'UnicodeCharacter0xaac3', [], 'Display', None],
  [ 0xaac4, 'UnicodeCharacter0xaac4', [], 'Display', None],
  [ 0xaac5, 'UnicodeCharacter0xaac5', [], 'Display', None],
  [ 0xaac6, 'UnicodeCharacter0xaac6', [], 'Display', None],
  [ 0xaac7, 'UnicodeCharacter0xaac7', [], 'Display', None],
  [ 0xaac8, 'UnicodeCharacter0xaac8', [], 'Display', None],
  [ 0xaac9, 'UnicodeCharacter0xaac9', [], 'Display', None],
  [ 0xaaca, 'UnicodeCharacter0xaaca', [], 'Display', None],
  [ 0xaacb, 'UnicodeCharacter0xaacb', [], 'Display', None],
  [ 0xaacc, 'UnicodeCharacter0xaacc', [], 'Display', None],
  [ 0xaacd, 'UnicodeCharacter0xaacd', [], 'Display', None],
  [ 0xaace, 'UnicodeCharacter0xaace', [], 'Display', None],
  [ 0xaacf, 'UnicodeCharacter0xaacf', [], 'Display', None],
  [ 0xaad0, 'UnicodeCharacter0xaad0', [], 'Display', None],
  [ 0xaad1, 'UnicodeCharacter0xaad1', [], 'Display', None],
  [ 0xaad2, 'UnicodeCharacter0xaad2', [], 'Display', None],
  [ 0xaad3, 'UnicodeCharacter0xaad3', [], 'Display', None],
  [ 0xaad4, 'UnicodeCharacter0xaad4', [], 'Display', None],
  [ 0xaad5, 'UnicodeCharacter0xaad5', [], 'Display', None],
  [ 0xaad6, 'UnicodeCharacter0xaad6', [], 'Display', None],
  [ 0xaad7, 'UnicodeCharacter0xaad7', [], 'Display', None],
  [ 0xaad8, 'UnicodeCharacter0xaad8', [], 'Display', None],
  [ 0xaad9, 'UnicodeCharacter0xaad9', [], 'Display', None],
  [ 0xaada, 'UnicodeCharacter0xaada', [], 'Display', None],
  [ 0xaadb, 'UnicodeCharacter0xaadb', [], 'Display', None],
  [ 0xaadc, 'UnicodeCharacter0xaadc', [], 'Display', None],
  [ 0xaadd, 'UnicodeCharacter0xaadd', [], 'Display', None],
  [ 0xaade, 'UnicodeCharacter0xaade', [], 'Display', None],
  [ 0xaadf, 'UnicodeCharacter0xaadf', [], 'Display', None],
  [ 0xaae0, 'UnicodeCharacter0xaae0', [], 'Display', None],
  [ 0xaae1, 'UnicodeCharacter0xaae1', [], 'Display', None],
  [ 0xaae2, 'UnicodeCharacter0xaae2', [], 'Display', None],
  [ 0xaae3, 'UnicodeCharacter0xaae3', [], 'Display', None],
  [ 0xaae4, 'UnicodeCharacter0xaae4', [], 'Display', None],
  [ 0xaae5, 'UnicodeCharacter0xaae5', [], 'Display', None],
  [ 0xaae6, 'UnicodeCharacter0xaae6', [], 'Display', None],
  [ 0xaae7, 'UnicodeCharacter0xaae7', [], 'Display', None],
  [ 0xaae8, 'UnicodeCharacter0xaae8', [], 'Display', None],
  [ 0xaae9, 'UnicodeCharacter0xaae9', [], 'Display', None],
  [ 0xaaea, 'UnicodeCharacter0xaaea', [], 'Display', None],
  [ 0xaaeb, 'UnicodeCharacter0xaaeb', [], 'Display', None],
  [ 0xaaec, 'UnicodeCharacter0xaaec', [], 'Display', None],
  [ 0xaaed, 'UnicodeCharacter0xaaed', [], 'Display', None],
  [ 0xaaee, 'UnicodeCharacter0xaaee', [], 'Display', None],
  [ 0xaaef, 'UnicodeCharacter0xaaef', [], 'Display', None],
  [ 0xaaf0, 'UnicodeCharacter0xaaf0', [], 'Display', None],
  [ 0xaaf1, 'UnicodeCharacter0xaaf1', [], 'Display', None],
  [ 0xaaf2, 'UnicodeCharacter0xaaf2', [], 'Display', None],
  [ 0xaaf3, 'UnicodeCharacter0xaaf3', [], 'Display', None],
  [ 0xaaf4, 'UnicodeCharacter0xaaf4', [], 'Display', None],
  [ 0xaaf5, 'UnicodeCharacter0xaaf5', [], 'Display', None],
  [ 0xaaf6, 'UnicodeCharacter0xaaf6', [], 'Display', None],
  [ 0xaaf7, 'UnicodeCharacter0xaaf7', [], 'Display', None],
  [ 0xaaf8, 'UnicodeCharacter0xaaf8', [], 'Display', None],
  [ 0xaaf9, 'UnicodeCharacter0xaaf9', [], 'Display', None],
  [ 0xaafa, 'UnicodeCharacter0xaafa', [], 'Display', None],
  [ 0xaafb, 'UnicodeCharacter0xaafb', [], 'Display', None],
  [ 0xaafc, 'UnicodeCharacter0xaafc', [], 'Display', None],
  [ 0xaafd, 'UnicodeCharacter0xaafd', [], 'Display', None],
  [ 0xaafe, 'UnicodeCharacter0xaafe', [], 'Display', None],
  [ 0xaaff, 'UnicodeCharacter0xaaff', [], 'Display', None],
  [ 0xab00, 'UnicodeCharacter0xab00', [], 'Display', None],
  [ 0xab01, 'UnicodeCharacter0xab01', [], 'Display', None],
  [ 0xab02, 'UnicodeCharacter0xab02', [], 'Display', None],
  [ 0xab03, 'UnicodeCharacter0xab03', [], 'Display', None],
  [ 0xab04, 'UnicodeCharacter0xab04', [], 'Display', None],
  [ 0xab05, 'UnicodeCharacter0xab05', [], 'Display', None],
  [ 0xab06, 'UnicodeCharacter0xab06', [], 'Display', None],
  [ 0xab07, 'UnicodeCharacter0xab07', [], 'Display', None],
  [ 0xab08, 'UnicodeCharacter0xab08', [], 'Display', None],
  [ 0xab09, 'UnicodeCharacter0xab09', [], 'Display', None],
  [ 0xab0a, 'UnicodeCharacter0xab0a', [], 'Display', None],
  [ 0xab0b, 'UnicodeCharacter0xab0b', [], 'Display', None],
  [ 0xab0c, 'UnicodeCharacter0xab0c', [], 'Display', None],
  [ 0xab0d, 'UnicodeCharacter0xab0d', [], 'Display', None],
  [ 0xab0e, 'UnicodeCharacter0xab0e', [], 'Display', None],
  [ 0xab0f, 'UnicodeCharacter0xab0f', [], 'Display', None],
  [ 0xab10, 'UnicodeCharacter0xab10', [], 'Display', None],
  [ 0xab11, 'UnicodeCharacter0xab11', [], 'Display', None],
  [ 0xab12, 'UnicodeCharacter0xab12', [], 'Display', None],
  [ 0xab13, 'UnicodeCharacter0xab13', [], 'Display', None],
  [ 0xab14, 'UnicodeCharacter0xab14', [], 'Display', None],
  [ 0xab15, 'UnicodeCharacter0xab15', [], 'Display', None],
  [ 0xab16, 'UnicodeCharacter0xab16', [], 'Display', None],
  [ 0xab17, 'UnicodeCharacter0xab17', [], 'Display', None],
  [ 0xab18, 'UnicodeCharacter0xab18', [], 'Display', None],
  [ 0xab19, 'UnicodeCharacter0xab19', [], 'Display', None],
  [ 0xab1a, 'UnicodeCharacter0xab1a', [], 'Display', None],
  [ 0xab1b, 'UnicodeCharacter0xab1b', [], 'Display', None],
  [ 0xab1c, 'UnicodeCharacter0xab1c', [], 'Display', None],
  [ 0xab1d, 'UnicodeCharacter0xab1d', [], 'Display', None],
  [ 0xab1e, 'UnicodeCharacter0xab1e', [], 'Display', None],
  [ 0xab1f, 'UnicodeCharacter0xab1f', [], 'Display', None],
  [ 0xab20, 'UnicodeCharacter0xab20', [], 'Display', None],
  [ 0xab21, 'UnicodeCharacter0xab21', [], 'Display', None],
  [ 0xab22, 'UnicodeCharacter0xab22', [], 'Display', None],
  [ 0xab23, 'UnicodeCharacter0xab23', [], 'Display', None],
  [ 0xab24, 'UnicodeCharacter0xab24', [], 'Display', None],
  [ 0xab25, 'UnicodeCharacter0xab25', [], 'Display', None],
  [ 0xab26, 'UnicodeCharacter0xab26', [], 'Display', None],
  [ 0xab27, 'UnicodeCharacter0xab27', [], 'Display', None],
  [ 0xab28, 'UnicodeCharacter0xab28', [], 'Display', None],
  [ 0xab29, 'UnicodeCharacter0xab29', [], 'Display', None],
  [ 0xab2a, 'UnicodeCharacter0xab2a', [], 'Display', None],
  [ 0xab2b, 'UnicodeCharacter0xab2b', [], 'Display', None],
  [ 0xab2c, 'UnicodeCharacter0xab2c', [], 'Display', None],
  [ 0xab2d, 'UnicodeCharacter0xab2d', [], 'Display', None],
  [ 0xab2e, 'UnicodeCharacter0xab2e', [], 'Display', None],
  [ 0xab2f, 'UnicodeCharacter0xab2f', [], 'Display', None],
  [ 0xab30, 'UnicodeCharacter0xab30', [], 'Display', None],
  [ 0xab31, 'UnicodeCharacter0xab31', [], 'Display', None],
  [ 0xab32, 'UnicodeCharacter0xab32', [], 'Display', None],
  [ 0xab33, 'UnicodeCharacter0xab33', [], 'Display', None],
  [ 0xab34, 'UnicodeCharacter0xab34', [], 'Display', None],
  [ 0xab35, 'UnicodeCharacter0xab35', [], 'Display', None],
  [ 0xab36, 'UnicodeCharacter0xab36', [], 'Display', None],
  [ 0xab37, 'UnicodeCharacter0xab37', [], 'Display', None],
  [ 0xab38, 'UnicodeCharacter0xab38', [], 'Display', None],
  [ 0xab39, 'UnicodeCharacter0xab39', [], 'Display', None],
  [ 0xab3a, 'UnicodeCharacter0xab3a', [], 'Display', None],
  [ 0xab3b, 'UnicodeCharacter0xab3b', [], 'Display', None],
  [ 0xab3c, 'UnicodeCharacter0xab3c', [], 'Display', None],
  [ 0xab3d, 'UnicodeCharacter0xab3d', [], 'Display', None],
  [ 0xab3e, 'UnicodeCharacter0xab3e', [], 'Display', None],
  [ 0xab3f, 'UnicodeCharacter0xab3f', [], 'Display', None],
  [ 0xab40, 'UnicodeCharacter0xab40', [], 'Display', None],
  [ 0xab41, 'UnicodeCharacter0xab41', [], 'Display', None],
  [ 0xab42, 'UnicodeCharacter0xab42', [], 'Display', None],
  [ 0xab43, 'UnicodeCharacter0xab43', [], 'Display', None],
  [ 0xab44, 'UnicodeCharacter0xab44', [], 'Display', None],
  [ 0xab45, 'UnicodeCharacter0xab45', [], 'Display', None],
  [ 0xab46, 'UnicodeCharacter0xab46', [], 'Display', None],
  [ 0xab47, 'UnicodeCharacter0xab47', [], 'Display', None],
  [ 0xab48, 'UnicodeCharacter0xab48', [], 'Display', None],
  [ 0xab49, 'UnicodeCharacter0xab49', [], 'Display', None],
  [ 0xab4a, 'UnicodeCharacter0xab4a', [], 'Display', None],
  [ 0xab4b, 'UnicodeCharacter0xab4b', [], 'Display', None],
  [ 0xab4c, 'UnicodeCharacter0xab4c', [], 'Display', None],
  [ 0xab4d, 'UnicodeCharacter0xab4d', [], 'Display', None],
  [ 0xab4e, 'UnicodeCharacter0xab4e', [], 'Display', None],
  [ 0xab4f, 'UnicodeCharacter0xab4f', [], 'Display', None],
  [ 0xab50, 'UnicodeCharacter0xab50', [], 'Display', None],
  [ 0xab51, 'UnicodeCharacter0xab51', [], 'Display', None],
  [ 0xab52, 'UnicodeCharacter0xab52', [], 'Display', None],
  [ 0xab53, 'UnicodeCharacter0xab53', [], 'Display', None],
  [ 0xab54, 'UnicodeCharacter0xab54', [], 'Display', None],
  [ 0xab55, 'UnicodeCharacter0xab55', [], 'Display', None],
  [ 0xab56, 'UnicodeCharacter0xab56', [], 'Display', None],
  [ 0xab57, 'UnicodeCharacter0xab57', [], 'Display', None],
  [ 0xab58, 'UnicodeCharacter0xab58', [], 'Display', None],
  [ 0xab59, 'UnicodeCharacter0xab59', [], 'Display', None],
  [ 0xab5a, 'UnicodeCharacter0xab5a', [], 'Display', None],
  [ 0xab5b, 'UnicodeCharacter0xab5b', [], 'Display', None],
  [ 0xab5c, 'UnicodeCharacter0xab5c', [], 'Display', None],
  [ 0xab5d, 'UnicodeCharacter0xab5d', [], 'Display', None],
  [ 0xab5e, 'UnicodeCharacter0xab5e', [], 'Display', None],
  [ 0xab5f, 'UnicodeCharacter0xab5f', [], 'Display', None],
  [ 0xab60, 'UnicodeCharacter0xab60', [], 'Display', None],
  [ 0xab61, 'UnicodeCharacter0xab61', [], 'Display', None],
  [ 0xab62, 'UnicodeCharacter0xab62', [], 'Display', None],
  [ 0xab63, 'UnicodeCharacter0xab63', [], 'Display', None],
  [ 0xab64, 'UnicodeCharacter0xab64', [], 'Display', None],
  [ 0xab65, 'UnicodeCharacter0xab65', [], 'Display', None],
  [ 0xab66, 'UnicodeCharacter0xab66', [], 'Display', None],
  [ 0xab67, 'UnicodeCharacter0xab67', [], 'Display', None],
  [ 0xab68, 'UnicodeCharacter0xab68', [], 'Display', None],
  [ 0xab69, 'UnicodeCharacter0xab69', [], 'Display', None],
  [ 0xab6a, 'UnicodeCharacter0xab6a', [], 'Display', None],
  [ 0xab6b, 'UnicodeCharacter0xab6b', [], 'Display', None],
  [ 0xab6c, 'UnicodeCharacter0xab6c', [], 'Display', None],
  [ 0xab6d, 'UnicodeCharacter0xab6d', [], 'Display', None],
  [ 0xab6e, 'UnicodeCharacter0xab6e', [], 'Display', None],
  [ 0xab6f, 'UnicodeCharacter0xab6f', [], 'Display', None],
  [ 0xab70, 'UnicodeCharacter0xab70', [], 'Display', None],
  [ 0xab71, 'UnicodeCharacter0xab71', [], 'Display', None],
  [ 0xab72, 'UnicodeCharacter0xab72', [], 'Display', None],
  [ 0xab73, 'UnicodeCharacter0xab73', [], 'Display', None],
  [ 0xab74, 'UnicodeCharacter0xab74', [], 'Display', None],
  [ 0xab75, 'UnicodeCharacter0xab75', [], 'Display', None],
  [ 0xab76, 'UnicodeCharacter0xab76', [], 'Display', None],
  [ 0xab77, 'UnicodeCharacter0xab77', [], 'Display', None],
  [ 0xab78, 'UnicodeCharacter0xab78', [], 'Display', None],
  [ 0xab79, 'UnicodeCharacter0xab79', [], 'Display', None],
  [ 0xab7a, 'UnicodeCharacter0xab7a', [], 'Display', None],
  [ 0xab7b, 'UnicodeCharacter0xab7b', [], 'Display', None],
  [ 0xab7c, 'UnicodeCharacter0xab7c', [], 'Display', None],
  [ 0xab7d, 'UnicodeCharacter0xab7d', [], 'Display', None],
  [ 0xab7e, 'UnicodeCharacter0xab7e', [], 'Display', None],
  [ 0xab7f, 'UnicodeCharacter0xab7f', [], 'Display', None],
  [ 0xab80, 'UnicodeCharacter0xab80', [], 'Display', None],
  [ 0xab81, 'UnicodeCharacter0xab81', [], 'Display', None],
  [ 0xab82, 'UnicodeCharacter0xab82', [], 'Display', None],
  [ 0xab83, 'UnicodeCharacter0xab83', [], 'Display', None],
  [ 0xab84, 'UnicodeCharacter0xab84', [], 'Display', None],
  [ 0xab85, 'UnicodeCharacter0xab85', [], 'Display', None],
  [ 0xab86, 'UnicodeCharacter0xab86', [], 'Display', None],
  [ 0xab87, 'UnicodeCharacter0xab87', [], 'Display', None],
  [ 0xab88, 'UnicodeCharacter0xab88', [], 'Display', None],
  [ 0xab89, 'UnicodeCharacter0xab89', [], 'Display', None],
  [ 0xab8a, 'UnicodeCharacter0xab8a', [], 'Display', None],
  [ 0xab8b, 'UnicodeCharacter0xab8b', [], 'Display', None],
  [ 0xab8c, 'UnicodeCharacter0xab8c', [], 'Display', None],
  [ 0xab8d, 'UnicodeCharacter0xab8d', [], 'Display', None],
  [ 0xab8e, 'UnicodeCharacter0xab8e', [], 'Display', None],
  [ 0xab8f, 'UnicodeCharacter0xab8f', [], 'Display', None],
  [ 0xab90, 'UnicodeCharacter0xab90', [], 'Display', None],
  [ 0xab91, 'UnicodeCharacter0xab91', [], 'Display', None],
  [ 0xab92, 'UnicodeCharacter0xab92', [], 'Display', None],
  [ 0xab93, 'UnicodeCharacter0xab93', [], 'Display', None],
  [ 0xab94, 'UnicodeCharacter0xab94', [], 'Display', None],
  [ 0xab95, 'UnicodeCharacter0xab95', [], 'Display', None],
  [ 0xab96, 'UnicodeCharacter0xab96', [], 'Display', None],
  [ 0xab97, 'UnicodeCharacter0xab97', [], 'Display', None],
  [ 0xab98, 'UnicodeCharacter0xab98', [], 'Display', None],
  [ 0xab99, 'UnicodeCharacter0xab99', [], 'Display', None],
  [ 0xab9a, 'UnicodeCharacter0xab9a', [], 'Display', None],
  [ 0xab9b, 'UnicodeCharacter0xab9b', [], 'Display', None],
  [ 0xab9c, 'UnicodeCharacter0xab9c', [], 'Display', None],
  [ 0xab9d, 'UnicodeCharacter0xab9d', [], 'Display', None],
  [ 0xab9e, 'UnicodeCharacter0xab9e', [], 'Display', None],
  [ 0xab9f, 'UnicodeCharacter0xab9f', [], 'Display', None],
  [ 0xaba0, 'UnicodeCharacter0xaba0', [], 'Display', None],
  [ 0xaba1, 'UnicodeCharacter0xaba1', [], 'Display', None],
  [ 0xaba2, 'UnicodeCharacter0xaba2', [], 'Display', None],
  [ 0xaba3, 'UnicodeCharacter0xaba3', [], 'Display', None],
  [ 0xaba4, 'UnicodeCharacter0xaba4', [], 'Display', None],
  [ 0xaba5, 'UnicodeCharacter0xaba5', [], 'Display', None],
  [ 0xaba6, 'UnicodeCharacter0xaba6', [], 'Display', None],
  [ 0xaba7, 'UnicodeCharacter0xaba7', [], 'Display', None],
  [ 0xaba8, 'UnicodeCharacter0xaba8', [], 'Display', None],
  [ 0xaba9, 'UnicodeCharacter0xaba9', [], 'Display', None],
  [ 0xabaa, 'UnicodeCharacter0xabaa', [], 'Display', None],
  [ 0xabab, 'UnicodeCharacter0xabab', [], 'Display', None],
  [ 0xabac, 'UnicodeCharacter0xabac', [], 'Display', None],
  [ 0xabad, 'UnicodeCharacter0xabad', [], 'Display', None],
  [ 0xabae, 'UnicodeCharacter0xabae', [], 'Display', None],
  [ 0xabaf, 'UnicodeCharacter0xabaf', [], 'Display', None],
  [ 0xabb0, 'UnicodeCharacter0xabb0', [], 'Display', None],
  [ 0xabb1, 'UnicodeCharacter0xabb1', [], 'Display', None],
  [ 0xabb2, 'UnicodeCharacter0xabb2', [], 'Display', None],
  [ 0xabb3, 'UnicodeCharacter0xabb3', [], 'Display', None],
  [ 0xabb4, 'UnicodeCharacter0xabb4', [], 'Display', None],
  [ 0xabb5, 'UnicodeCharacter0xabb5', [], 'Display', None],
  [ 0xabb6, 'UnicodeCharacter0xabb6', [], 'Display', None],
  [ 0xabb7, 'UnicodeCharacter0xabb7', [], 'Display', None],
  [ 0xabb8, 'UnicodeCharacter0xabb8', [], 'Display', None],
  [ 0xabb9, 'UnicodeCharacter0xabb9', [], 'Display', None],
  [ 0xabba, 'UnicodeCharacter0xabba', [], 'Display', None],
  [ 0xabbb, 'UnicodeCharacter0xabbb', [], 'Display', None],
  [ 0xabbc, 'UnicodeCharacter0xabbc', [], 'Display', None],
  [ 0xabbd, 'UnicodeCharacter0xabbd', [], 'Display', None],
  [ 0xabbe, 'UnicodeCharacter0xabbe', [], 'Display', None],
  [ 0xabbf, 'UnicodeCharacter0xabbf', [], 'Display', None],
  [ 0xabc0, 'UnicodeCharacter0xabc0', [], 'Display', None],
  [ 0xabc1, 'UnicodeCharacter0xabc1', [], 'Display', None],
  [ 0xabc2, 'UnicodeCharacter0xabc2', [], 'Display', None],
  [ 0xabc3, 'UnicodeCharacter0xabc3', [], 'Display', None],
  [ 0xabc4, 'UnicodeCharacter0xabc4', [], 'Display', None],
  [ 0xabc5, 'UnicodeCharacter0xabc5', [], 'Display', None],
  [ 0xabc6, 'UnicodeCharacter0xabc6', [], 'Display', None],
  [ 0xabc7, 'UnicodeCharacter0xabc7', [], 'Display', None],
  [ 0xabc8, 'UnicodeCharacter0xabc8', [], 'Display', None],
  [ 0xabc9, 'UnicodeCharacter0xabc9', [], 'Display', None],
  [ 0xabca, 'UnicodeCharacter0xabca', [], 'Display', None],
  [ 0xabcb, 'UnicodeCharacter0xabcb', [], 'Display', None],
  [ 0xabcc, 'UnicodeCharacter0xabcc', [], 'Display', None],
  [ 0xabcd, 'UnicodeCharacter0xabcd', [], 'Display', None],
  [ 0xabce, 'UnicodeCharacter0xabce', [], 'Display', None],
  [ 0xabcf, 'UnicodeCharacter0xabcf', [], 'Display', None],
  [ 0xabd0, 'UnicodeCharacter0xabd0', [], 'Display', None],
  [ 0xabd1, 'UnicodeCharacter0xabd1', [], 'Display', None],
  [ 0xabd2, 'UnicodeCharacter0xabd2', [], 'Display', None],
  [ 0xabd3, 'UnicodeCharacter0xabd3', [], 'Display', None],
  [ 0xabd4, 'UnicodeCharacter0xabd4', [], 'Display', None],
  [ 0xabd5, 'UnicodeCharacter0xabd5', [], 'Display', None],
  [ 0xabd6, 'UnicodeCharacter0xabd6', [], 'Display', None],
  [ 0xabd7, 'UnicodeCharacter0xabd7', [], 'Display', None],
  [ 0xabd8, 'UnicodeCharacter0xabd8', [], 'Display', None],
  [ 0xabd9, 'UnicodeCharacter0xabd9', [], 'Display', None],
  [ 0xabda, 'UnicodeCharacter0xabda', [], 'Display', None],
  [ 0xabdb, 'UnicodeCharacter0xabdb', [], 'Display', None],
  [ 0xabdc, 'UnicodeCharacter0xabdc', [], 'Display', None],
  [ 0xabdd, 'UnicodeCharacter0xabdd', [], 'Display', None],
  [ 0xabde, 'UnicodeCharacter0xabde', [], 'Display', None],
  [ 0xabdf, 'UnicodeCharacter0xabdf', [], 'Display', None],
  [ 0xabe0, 'UnicodeCharacter0xabe0', [], 'Display', None],
  [ 0xabe1, 'UnicodeCharacter0xabe1', [], 'Display', None],
  [ 0xabe2, 'UnicodeCharacter0xabe2', [], 'Display', None],
  [ 0xabe3, 'UnicodeCharacter0xabe3', [], 'Display', None],
  [ 0xabe4, 'UnicodeCharacter0xabe4', [], 'Display', None],
  [ 0xabe5, 'UnicodeCharacter0xabe5', [], 'Display', None],
  [ 0xabe6, 'UnicodeCharacter0xabe6', [], 'Display', None],
  [ 0xabe7, 'UnicodeCharacter0xabe7', [], 'Display', None],
  [ 0xabe8, 'UnicodeCharacter0xabe8', [], 'Display', None],
  [ 0xabe9, 'UnicodeCharacter0xabe9', [], 'Display', None],
  [ 0xabea, 'UnicodeCharacter0xabea', [], 'Display', None],
  [ 0xabeb, 'UnicodeCharacter0xabeb', [], 'Display', None],
  [ 0xabec, 'UnicodeCharacter0xabec', [], 'Display', None],
  [ 0xabed, 'UnicodeCharacter0xabed', [], 'Display', None],
  [ 0xabee, 'UnicodeCharacter0xabee', [], 'Display', None],
  [ 0xabef, 'UnicodeCharacter0xabef', [], 'Display', None],
  [ 0xabf0, 'UnicodeCharacter0xabf0', [], 'Display', None],
  [ 0xabf1, 'UnicodeCharacter0xabf1', [], 'Display', None],
  [ 0xabf2, 'UnicodeCharacter0xabf2', [], 'Display', None],
  [ 0xabf3, 'UnicodeCharacter0xabf3', [], 'Display', None],
  [ 0xabf4, 'UnicodeCharacter0xabf4', [], 'Display', None],
  [ 0xabf5, 'UnicodeCharacter0xabf5', [], 'Display', None],
  [ 0xabf6, 'UnicodeCharacter0xabf6', [], 'Display', None],
  [ 0xabf7, 'UnicodeCharacter0xabf7', [], 'Display', None],
  [ 0xabf8, 'UnicodeCharacter0xabf8', [], 'Display', None],
  [ 0xabf9, 'UnicodeCharacter0xabf9', [], 'Display', None],
  [ 0xabfa, 'UnicodeCharacter0xabfa', [], 'Display', None],
  [ 0xabfb, 'UnicodeCharacter0xabfb', [], 'Display', None],
  [ 0xabfc, 'UnicodeCharacter0xabfc', [], 'Display', None],
  [ 0xabfd, 'UnicodeCharacter0xabfd', [], 'Display', None],
  [ 0xabfe, 'UnicodeCharacter0xabfe', [], 'Display', None],
  [ 0xabff, 'UnicodeCharacter0xabff', [], 'Display', None],
  [ 0xac00, 'HangulSyllableGa', [], 'Display', None],
  [ 0xac01, 'HangulSyllableGag', [], 'Display', None],
  [ 0xac02, 'HangulSyllableGagg', [], 'Display', None],
  [ 0xac03, 'HangulSyllableGags', [], 'Display', None],
  [ 0xac04, 'HangulSyllableGan', [], 'Display', None],
  [ 0xac05, 'HangulSyllableGanj', [], 'Display', None],
  [ 0xac06, 'HangulSyllableGanh', [], 'Display', None],
  [ 0xac07, 'HangulSyllableGad', [], 'Display', None],
  [ 0xac08, 'HangulSyllableGal', [], 'Display', None],
  [ 0xac09, 'HangulSyllableGalg', [], 'Display', None],
  [ 0xac0a, 'HangulSyllableGalm', [], 'Display', None],
  [ 0xac0b, 'HangulSyllableGalb', [], 'Display', None],
  [ 0xac0c, 'HangulSyllableGals', [], 'Display', None],
  [ 0xac0d, 'HangulSyllableGalt', [], 'Display', None],
  [ 0xac0e, 'HangulSyllableGalp', [], 'Display', None],
  [ 0xac0f, 'HangulSyllableGalh', [], 'Display', None],
  [ 0xac10, 'HangulSyllableGam', [], 'Display', None],
  [ 0xac11, 'HangulSyllableGab', [], 'Display', None],
  [ 0xac12, 'HangulSyllableGabs', [], 'Display', None],
  [ 0xac13, 'HangulSyllableGas', [], 'Display', None],
  [ 0xac14, 'HangulSyllableGass', [], 'Display', None],
  [ 0xac15, 'HangulSyllableGang', [], 'Display', None],
  [ 0xac16, 'HangulSyllableGaj', [], 'Display', None],
  [ 0xac17, 'HangulSyllableGac', [], 'Display', None],
  [ 0xac18, 'HangulSyllableGak', [], 'Display', None],
  [ 0xac19, 'HangulSyllableGat', [], 'Display', None],
  [ 0xac1a, 'HangulSyllableGap', [], 'Display', None],
  [ 0xac1b, 'HangulSyllableGah', [], 'Display', None],
  [ 0xac1c, 'HangulSyllableGae', [], 'Display', None],
  [ 0xac1d, 'HangulSyllableGaeg', [], 'Display', None],
  [ 0xac1e, 'HangulSyllableGaegg', [], 'Display', None],
  [ 0xac1f, 'HangulSyllableGaegs', [], 'Display', None],
  [ 0xac20, 'HangulSyllableGaen', [], 'Display', None],
  [ 0xac21, 'HangulSyllableGaenj', [], 'Display', None],
  [ 0xac22, 'HangulSyllableGaenh', [], 'Display', None],
  [ 0xac23, 'HangulSyllableGaed', [], 'Display', None],
  [ 0xac24, 'HangulSyllableGael', [], 'Display', None],
  [ 0xac25, 'HangulSyllableGaelg', [], 'Display', None],
  [ 0xac26, 'HangulSyllableGaelm', [], 'Display', None],
  [ 0xac27, 'HangulSyllableGaelb', [], 'Display', None],
  [ 0xac28, 'HangulSyllableGaels', [], 'Display', None],
  [ 0xac29, 'HangulSyllableGaelt', [], 'Display', None],
  [ 0xac2a, 'HangulSyllableGaelp', [], 'Display', None],
  [ 0xac2b, 'HangulSyllableGaelh', [], 'Display', None],
  [ 0xac2c, 'HangulSyllableGaem', [], 'Display', None],
  [ 0xac2d, 'HangulSyllableGaeb', [], 'Display', None],
  [ 0xac2e, 'HangulSyllableGaebs', [], 'Display', None],
  [ 0xac2f, 'HangulSyllableGaes', [], 'Display', None],
  [ 0xac30, 'HangulSyllableGaess', [], 'Display', None],
  [ 0xac31, 'HangulSyllableGaeng', [], 'Display', None],
  [ 0xac32, 'HangulSyllableGaej', [], 'Display', None],
  [ 0xac33, 'HangulSyllableGaec', [], 'Display', None],
  [ 0xac34, 'HangulSyllableGaek', [], 'Display', None],
  [ 0xac35, 'HangulSyllableGaet', [], 'Display', None],
  [ 0xac36, 'HangulSyllableGaep', [], 'Display', None],
  [ 0xac37, 'HangulSyllableGaeh', [], 'Display', None],
  [ 0xac38, 'HangulSyllableGya', [], 'Display', None],
  [ 0xac39, 'HangulSyllableGyag', [], 'Display', None],
  [ 0xac3a, 'HangulSyllableGyagg', [], 'Display', None],
  [ 0xac3b, 'HangulSyllableGyags', [], 'Display', None],
  [ 0xac3c, 'HangulSyllableGyan', [], 'Display', None],
  [ 0xac3d, 'HangulSyllableGyanj', [], 'Display', None],
  [ 0xac3e, 'HangulSyllableGyanh', [], 'Display', None],
  [ 0xac3f, 'HangulSyllableGyad', [], 'Display', None],
  [ 0xac40, 'HangulSyllableGyal', [], 'Display', None],
  [ 0xac41, 'HangulSyllableGyalg', [], 'Display', None],
  [ 0xac42, 'HangulSyllableGyalm', [], 'Display', None],
  [ 0xac43, 'HangulSyllableGyalb', [], 'Display', None],
  [ 0xac44, 'HangulSyllableGyals', [], 'Display', None],
  [ 0xac45, 'HangulSyllableGyalt', [], 'Display', None],
  [ 0xac46, 'HangulSyllableGyalp', [], 'Display', None],
  [ 0xac47, 'HangulSyllableGyalh', [], 'Display', None],
  [ 0xac48, 'HangulSyllableGyam', [], 'Display', None],
  [ 0xac49, 'HangulSyllableGyab', [], 'Display', None],
  [ 0xac4a, 'HangulSyllableGyabs', [], 'Display', None],
  [ 0xac4b, 'HangulSyllableGyas', [], 'Display', None],
  [ 0xac4c, 'HangulSyllableGyass', [], 'Display', None],
  [ 0xac4d, 'HangulSyllableGyang', [], 'Display', None],
  [ 0xac4e, 'HangulSyllableGyaj', [], 'Display', None],
  [ 0xac4f, 'HangulSyllableGyac', [], 'Display', None],
  [ 0xac50, 'HangulSyllableGyak', [], 'Display', None],
  [ 0xac51, 'HangulSyllableGyat', [], 'Display', None],
  [ 0xac52, 'HangulSyllableGyap', [], 'Display', None],
  [ 0xac53, 'HangulSyllableGyah', [], 'Display', None],
  [ 0xac54, 'HangulSyllableGyae', [], 'Display', None],
  [ 0xac55, 'HangulSyllableGyaeg', [], 'Display', None],
  [ 0xac56, 'HangulSyllableGyaegg', [], 'Display', None],
  [ 0xac57, 'HangulSyllableGyaegs', [], 'Display', None],
  [ 0xac58, 'HangulSyllableGyaen', [], 'Display', None],
  [ 0xac59, 'HangulSyllableGyaenj', [], 'Display', None],
  [ 0xac5a, 'HangulSyllableGyaenh', [], 'Display', None],
  [ 0xac5b, 'HangulSyllableGyaed', [], 'Display', None],
  [ 0xac5c, 'HangulSyllableGyael', [], 'Display', None],
  [ 0xac5d, 'HangulSyllableGyaelg', [], 'Display', None],
  [ 0xac5e, 'HangulSyllableGyaelm', [], 'Display', None],
  [ 0xac5f, 'HangulSyllableGyaelb', [], 'Display', None],
  [ 0xac60, 'HangulSyllableGyaels', [], 'Display', None],
  [ 0xac61, 'HangulSyllableGyaelt', [], 'Display', None],
  [ 0xac62, 'HangulSyllableGyaelp', [], 'Display', None],
  [ 0xac63, 'HangulSyllableGyaelh', [], 'Display', None],
  [ 0xac64, 'HangulSyllableGyaem', [], 'Display', None],
  [ 0xac65, 'HangulSyllableGyaeb', [], 'Display', None],
  [ 0xac66, 'HangulSyllableGyaebs', [], 'Display', None],
  [ 0xac67, 'HangulSyllableGyaes', [], 'Display', None],
  [ 0xac68, 'HangulSyllableGyaess', [], 'Display', None],
  [ 0xac69, 'HangulSyllableGyaeng', [], 'Display', None],
  [ 0xac6a, 'HangulSyllableGyaej', [], 'Display', None],
  [ 0xac6b, 'HangulSyllableGyaec', [], 'Display', None],
  [ 0xac6c, 'HangulSyllableGyaek', [], 'Display', None],
  [ 0xac6d, 'HangulSyllableGyaet', [], 'Display', None],
  [ 0xac6e, 'HangulSyllableGyaep', [], 'Display', None],
  [ 0xac6f, 'HangulSyllableGyaeh', [], 'Display', None],
  [ 0xac70, 'HangulSyllableGeo', [], 'Display', None],
  [ 0xac71, 'HangulSyllableGeog', [], 'Display', None],
  [ 0xac72, 'HangulSyllableGeogg', [], 'Display', None],
  [ 0xac73, 'HangulSyllableGeogs', [], 'Display', None],
  [ 0xac74, 'HangulSyllableGeon', [], 'Display', None],
  [ 0xac75, 'HangulSyllableGeonj', [], 'Display', None],
  [ 0xac76, 'HangulSyllableGeonh', [], 'Display', None],
  [ 0xac77, 'HangulSyllableGeod', [], 'Display', None],
  [ 0xac78, 'HangulSyllableGeol', [], 'Display', None],
  [ 0xac79, 'HangulSyllableGeolg', [], 'Display', None],
  [ 0xac7a, 'HangulSyllableGeolm', [], 'Display', None],
  [ 0xac7b, 'HangulSyllableGeolb', [], 'Display', None],
  [ 0xac7c, 'HangulSyllableGeols', [], 'Display', None],
  [ 0xac7d, 'HangulSyllableGeolt', [], 'Display', None],
  [ 0xac7e, 'HangulSyllableGeolp', [], 'Display', None],
  [ 0xac7f, 'HangulSyllableGeolh', [], 'Display', None],
  [ 0xac80, 'HangulSyllableGeom', [], 'Display', None],
  [ 0xac81, 'HangulSyllableGeob', [], 'Display', None],
  [ 0xac82, 'HangulSyllableGeobs', [], 'Display', None],
  [ 0xac83, 'HangulSyllableGeos', [], 'Display', None],
  [ 0xac84, 'HangulSyllableGeoss', [], 'Display', None],
  [ 0xac85, 'HangulSyllableGeong', [], 'Display', None],
  [ 0xac86, 'HangulSyllableGeoj', [], 'Display', None],
  [ 0xac87, 'HangulSyllableGeoc', [], 'Display', None],
  [ 0xac88, 'HangulSyllableGeok', [], 'Display', None],
  [ 0xac89, 'HangulSyllableGeot', [], 'Display', None],
  [ 0xac8a, 'HangulSyllableGeop', [], 'Display', None],
  [ 0xac8b, 'HangulSyllableGeoh', [], 'Display', None],
  [ 0xac8c, 'HangulSyllableGe', [], 'Display', None],
  [ 0xac8d, 'HangulSyllableGeg', [], 'Display', None],
  [ 0xac8e, 'HangulSyllableGegg', [], 'Display', None],
  [ 0xac8f, 'HangulSyllableGegs', [], 'Display', None],
  [ 0xac90, 'HangulSyllableGen', [], 'Display', None],
  [ 0xac91, 'HangulSyllableGenj', [], 'Display', None],
  [ 0xac92, 'HangulSyllableGenh', [], 'Display', None],
  [ 0xac93, 'HangulSyllableGed', [], 'Display', None],
  [ 0xac94, 'HangulSyllableGel', [], 'Display', None],
  [ 0xac95, 'HangulSyllableGelg', [], 'Display', None],
  [ 0xac96, 'HangulSyllableGelm', [], 'Display', None],
  [ 0xac97, 'HangulSyllableGelb', [], 'Display', None],
  [ 0xac98, 'HangulSyllableGels', [], 'Display', None],
  [ 0xac99, 'HangulSyllableGelt', [], 'Display', None],
  [ 0xac9a, 'HangulSyllableGelp', [], 'Display', None],
  [ 0xac9b, 'HangulSyllableGelh', [], 'Display', None],
  [ 0xac9c, 'HangulSyllableGem', [], 'Display', None],
  [ 0xac9d, 'HangulSyllableGeb', [], 'Display', None],
  [ 0xac9e, 'HangulSyllableGebs', [], 'Display', None],
  [ 0xac9f, 'HangulSyllableGes', [], 'Display', None],
  [ 0xaca0, 'HangulSyllableGess', [], 'Display', None],
  [ 0xaca1, 'HangulSyllableGeng', [], 'Display', None],
  [ 0xaca2, 'HangulSyllableGej', [], 'Display', None],
  [ 0xaca3, 'HangulSyllableGec', [], 'Display', None],
  [ 0xaca4, 'HangulSyllableGek', [], 'Display', None],
  [ 0xaca5, 'HangulSyllableGet', [], 'Display', None],
  [ 0xaca6, 'HangulSyllableGep', [], 'Display', None],
  [ 0xaca7, 'HangulSyllableGeh', [], 'Display', None],
  [ 0xaca8, 'HangulSyllableGyeo', [], 'Display', None],
  [ 0xaca9, 'HangulSyllableGyeog', [], 'Display', None],
  [ 0xacaa, 'HangulSyllableGyeogg', [], 'Display', None],
  [ 0xacab, 'HangulSyllableGyeogs', [], 'Display', None],
  [ 0xacac, 'HangulSyllableGyeon', [], 'Display', None],
  [ 0xacad, 'HangulSyllableGyeonj', [], 'Display', None],
  [ 0xacae, 'HangulSyllableGyeonh', [], 'Display', None],
  [ 0xacaf, 'HangulSyllableGyeod', [], 'Display', None],
  [ 0xacb0, 'HangulSyllableGyeol', [], 'Display', None],
  [ 0xacb1, 'HangulSyllableGyeolg', [], 'Display', None],
  [ 0xacb2, 'HangulSyllableGyeolm', [], 'Display', None],
  [ 0xacb3, 'HangulSyllableGyeolb', [], 'Display', None],
  [ 0xacb4, 'HangulSyllableGyeols', [], 'Display', None],
  [ 0xacb5, 'HangulSyllableGyeolt', [], 'Display', None],
  [ 0xacb6, 'HangulSyllableGyeolp', [], 'Display', None],
  [ 0xacb7, 'HangulSyllableGyeolh', [], 'Display', None],
  [ 0xacb8, 'HangulSyllableGyeom', [], 'Display', None],
  [ 0xacb9, 'HangulSyllableGyeob', [], 'Display', None],
  [ 0xacba, 'HangulSyllableGyeobs', [], 'Display', None],
  [ 0xacbb, 'HangulSyllableGyeos', [], 'Display', None],
  [ 0xacbc, 'HangulSyllableGyeoss', [], 'Display', None],
  [ 0xacbd, 'HangulSyllableGyeong', [], 'Display', None],
  [ 0xacbe, 'HangulSyllableGyeoj', [], 'Display', None],
  [ 0xacbf, 'HangulSyllableGyeoc', [], 'Display', None],
  [ 0xacc0, 'HangulSyllableGyeok', [], 'Display', None],
  [ 0xacc1, 'HangulSyllableGyeot', [], 'Display', None],
  [ 0xacc2, 'HangulSyllableGyeop', [], 'Display', None],
  [ 0xacc3, 'HangulSyllableGyeoh', [], 'Display', None],
  [ 0xacc4, 'HangulSyllableGye', [], 'Display', None],
  [ 0xacc5, 'HangulSyllableGyeg', [], 'Display', None],
  [ 0xacc6, 'HangulSyllableGyegg', [], 'Display', None],
  [ 0xacc7, 'HangulSyllableGyegs', [], 'Display', None],
  [ 0xacc8, 'HangulSyllableGyen', [], 'Display', None],
  [ 0xacc9, 'HangulSyllableGyenj', [], 'Display', None],
  [ 0xacca, 'HangulSyllableGyenh', [], 'Display', None],
  [ 0xaccb, 'HangulSyllableGyed', [], 'Display', None],
  [ 0xaccc, 'HangulSyllableGyel', [], 'Display', None],
  [ 0xaccd, 'HangulSyllableGyelg', [], 'Display', None],
  [ 0xacce, 'HangulSyllableGyelm', [], 'Display', None],
  [ 0xaccf, 'HangulSyllableGyelb', [], 'Display', None],
  [ 0xacd0, 'HangulSyllableGyels', [], 'Display', None],
  [ 0xacd1, 'HangulSyllableGyelt', [], 'Display', None],
  [ 0xacd2, 'HangulSyllableGyelp', [], 'Display', None],
  [ 0xacd3, 'HangulSyllableGyelh', [], 'Display', None],
  [ 0xacd4, 'HangulSyllableGyem', [], 'Display', None],
  [ 0xacd5, 'HangulSyllableGyeb', [], 'Display', None],
  [ 0xacd6, 'HangulSyllableGyebs', [], 'Display', None],
  [ 0xacd7, 'HangulSyllableGyes', [], 'Display', None],
  [ 0xacd8, 'HangulSyllableGyess', [], 'Display', None],
  [ 0xacd9, 'HangulSyllableGyeng', [], 'Display', None],
  [ 0xacda, 'HangulSyllableGyej', [], 'Display', None],
  [ 0xacdb, 'HangulSyllableGyec', [], 'Display', None],
  [ 0xacdc, 'HangulSyllableGyek', [], 'Display', None],
  [ 0xacdd, 'HangulSyllableGyet', [], 'Display', None],
  [ 0xacde, 'HangulSyllableGyep', [], 'Display', None],
  [ 0xacdf, 'HangulSyllableGyeh', [], 'Display', None],
  [ 0xace0, 'HangulSyllableGo', [], 'Display', None],
  [ 0xace1, 'HangulSyllableGog', [], 'Display', None],
  [ 0xace2, 'HangulSyllableGogg', [], 'Display', None],
  [ 0xace3, 'HangulSyllableGogs', [], 'Display', None],
  [ 0xace4, 'HangulSyllableGon', [], 'Display', None],
  [ 0xace5, 'HangulSyllableGonj', [], 'Display', None],
  [ 0xace6, 'HangulSyllableGonh', [], 'Display', None],
  [ 0xace7, 'HangulSyllableGod', [], 'Display', None],
  [ 0xace8, 'HangulSyllableGol', [], 'Display', None],
  [ 0xace9, 'HangulSyllableGolg', [], 'Display', None],
  [ 0xacea, 'HangulSyllableGolm', [], 'Display', None],
  [ 0xaceb, 'HangulSyllableGolb', [], 'Display', None],
  [ 0xacec, 'HangulSyllableGols', [], 'Display', None],
  [ 0xaced, 'HangulSyllableGolt', [], 'Display', None],
  [ 0xacee, 'HangulSyllableGolp', [], 'Display', None],
  [ 0xacef, 'HangulSyllableGolh', [], 'Display', None],
  [ 0xacf0, 'HangulSyllableGom', [], 'Display', None],
  [ 0xacf1, 'HangulSyllableGob', [], 'Display', None],
  [ 0xacf2, 'HangulSyllableGobs', [], 'Display', None],
  [ 0xacf3, 'HangulSyllableGos', [], 'Display', None],
  [ 0xacf4, 'HangulSyllableGoss', [], 'Display', None],
  [ 0xacf5, 'HangulSyllableGong', [], 'Display', None],
  [ 0xacf6, 'HangulSyllableGoj', [], 'Display', None],
  [ 0xacf7, 'HangulSyllableGoc', [], 'Display', None],
  [ 0xacf8, 'HangulSyllableGok', [], 'Display', None],
  [ 0xacf9, 'HangulSyllableGot', [], 'Display', None],
  [ 0xacfa, 'HangulSyllableGop', [], 'Display', None],
  [ 0xacfb, 'HangulSyllableGoh', [], 'Display', None],
  [ 0xacfc, 'HangulSyllableGwa', [], 'Display', None],
  [ 0xacfd, 'HangulSyllableGwag', [], 'Display', None],
  [ 0xacfe, 'HangulSyllableGwagg', [], 'Display', None],
  [ 0xacff, 'HangulSyllableGwags', [], 'Display', None],
  [ 0xad00, 'HangulSyllableGwan', [], 'Display', None],
  [ 0xad01, 'HangulSyllableGwanj', [], 'Display', None],
  [ 0xad02, 'HangulSyllableGwanh', [], 'Display', None],
  [ 0xad03, 'HangulSyllableGwad', [], 'Display', None],
  [ 0xad04, 'HangulSyllableGwal', [], 'Display', None],
  [ 0xad05, 'HangulSyllableGwalg', [], 'Display', None],
  [ 0xad06, 'HangulSyllableGwalm', [], 'Display', None],
  [ 0xad07, 'HangulSyllableGwalb', [], 'Display', None],
  [ 0xad08, 'HangulSyllableGwals', [], 'Display', None],
  [ 0xad09, 'HangulSyllableGwalt', [], 'Display', None],
  [ 0xad0a, 'HangulSyllableGwalp', [], 'Display', None],
  [ 0xad0b, 'HangulSyllableGwalh', [], 'Display', None],
  [ 0xad0c, 'HangulSyllableGwam', [], 'Display', None],
  [ 0xad0d, 'HangulSyllableGwab', [], 'Display', None],
  [ 0xad0e, 'HangulSyllableGwabs', [], 'Display', None],
  [ 0xad0f, 'HangulSyllableGwas', [], 'Display', None],
  [ 0xad10, 'HangulSyllableGwass', [], 'Display', None],
  [ 0xad11, 'HangulSyllableGwang', [], 'Display', None],
  [ 0xad12, 'HangulSyllableGwaj', [], 'Display', None],
  [ 0xad13, 'HangulSyllableGwac', [], 'Display', None],
  [ 0xad14, 'HangulSyllableGwak', [], 'Display', None],
  [ 0xad15, 'HangulSyllableGwat', [], 'Display', None],
  [ 0xad16, 'HangulSyllableGwap', [], 'Display', None],
  [ 0xad17, 'HangulSyllableGwah', [], 'Display', None],
  [ 0xad18, 'HangulSyllableGwae', [], 'Display', None],
  [ 0xad19, 'HangulSyllableGwaeg', [], 'Display', None],
  [ 0xad1a, 'HangulSyllableGwaegg', [], 'Display', None],
  [ 0xad1b, 'HangulSyllableGwaegs', [], 'Display', None],
  [ 0xad1c, 'HangulSyllableGwaen', [], 'Display', None],
  [ 0xad1d, 'HangulSyllableGwaenj', [], 'Display', None],
  [ 0xad1e, 'HangulSyllableGwaenh', [], 'Display', None],
  [ 0xad1f, 'HangulSyllableGwaed', [], 'Display', None],
  [ 0xad20, 'HangulSyllableGwael', [], 'Display', None],
  [ 0xad21, 'HangulSyllableGwaelg', [], 'Display', None],
  [ 0xad22, 'HangulSyllableGwaelm', [], 'Display', None],
  [ 0xad23, 'HangulSyllableGwaelb', [], 'Display', None],
  [ 0xad24, 'HangulSyllableGwaels', [], 'Display', None],
  [ 0xad25, 'HangulSyllableGwaelt', [], 'Display', None],
  [ 0xad26, 'HangulSyllableGwaelp', [], 'Display', None],
  [ 0xad27, 'HangulSyllableGwaelh', [], 'Display', None],
  [ 0xad28, 'HangulSyllableGwaem', [], 'Display', None],
  [ 0xad29, 'HangulSyllableGwaeb', [], 'Display', None],
  [ 0xad2a, 'HangulSyllableGwaebs', [], 'Display', None],
  [ 0xad2b, 'HangulSyllableGwaes', [], 'Display', None],
  [ 0xad2c, 'HangulSyllableGwaess', [], 'Display', None],
  [ 0xad2d, 'HangulSyllableGwaeng', [], 'Display', None],
  [ 0xad2e, 'HangulSyllableGwaej', [], 'Display', None],
  [ 0xad2f, 'HangulSyllableGwaec', [], 'Display', None],
  [ 0xad30, 'HangulSyllableGwaek', [], 'Display', None],
  [ 0xad31, 'HangulSyllableGwaet', [], 'Display', None],
  [ 0xad32, 'HangulSyllableGwaep', [], 'Display', None],
  [ 0xad33, 'HangulSyllableGwaeh', [], 'Display', None],
  [ 0xad34, 'HangulSyllableGoe', [], 'Display', None],
  [ 0xad35, 'HangulSyllableGoeg', [], 'Display', None],
  [ 0xad36, 'HangulSyllableGoegg', [], 'Display', None],
  [ 0xad37, 'HangulSyllableGoegs', [], 'Display', None],
  [ 0xad38, 'HangulSyllableGoen', [], 'Display', None],
  [ 0xad39, 'HangulSyllableGoenj', [], 'Display', None],
  [ 0xad3a, 'HangulSyllableGoenh', [], 'Display', None],
  [ 0xad3b, 'HangulSyllableGoed', [], 'Display', None],
  [ 0xad3c, 'HangulSyllableGoel', [], 'Display', None],
  [ 0xad3d, 'HangulSyllableGoelg', [], 'Display', None],
  [ 0xad3e, 'HangulSyllableGoelm', [], 'Display', None],
  [ 0xad3f, 'HangulSyllableGoelb', [], 'Display', None],
  [ 0xad40, 'HangulSyllableGoels', [], 'Display', None],
  [ 0xad41, 'HangulSyllableGoelt', [], 'Display', None],
  [ 0xad42, 'HangulSyllableGoelp', [], 'Display', None],
  [ 0xad43, 'HangulSyllableGoelh', [], 'Display', None],
  [ 0xad44, 'HangulSyllableGoem', [], 'Display', None],
  [ 0xad45, 'HangulSyllableGoeb', [], 'Display', None],
  [ 0xad46, 'HangulSyllableGoebs', [], 'Display', None],
  [ 0xad47, 'HangulSyllableGoes', [], 'Display', None],
  [ 0xad48, 'HangulSyllableGoess', [], 'Display', None],
  [ 0xad49, 'HangulSyllableGoeng', [], 'Display', None],
  [ 0xad4a, 'HangulSyllableGoej', [], 'Display', None],
  [ 0xad4b, 'HangulSyllableGoec', [], 'Display', None],
  [ 0xad4c, 'HangulSyllableGoek', [], 'Display', None],
  [ 0xad4d, 'HangulSyllableGoet', [], 'Display', None],
  [ 0xad4e, 'HangulSyllableGoep', [], 'Display', None],
  [ 0xad4f, 'HangulSyllableGoeh', [], 'Display', None],
  [ 0xad50, 'HangulSyllableGyo', [], 'Display', None],
  [ 0xad51, 'HangulSyllableGyog', [], 'Display', None],
  [ 0xad52, 'HangulSyllableGyogg', [], 'Display', None],
  [ 0xad53, 'HangulSyllableGyogs', [], 'Display', None],
  [ 0xad54, 'HangulSyllableGyon', [], 'Display', None],
  [ 0xad55, 'HangulSyllableGyonj', [], 'Display', None],
  [ 0xad56, 'HangulSyllableGyonh', [], 'Display', None],
  [ 0xad57, 'HangulSyllableGyod', [], 'Display', None],
  [ 0xad58, 'HangulSyllableGyol', [], 'Display', None],
  [ 0xad59, 'HangulSyllableGyolg', [], 'Display', None],
  [ 0xad5a, 'HangulSyllableGyolm', [], 'Display', None],
  [ 0xad5b, 'HangulSyllableGyolb', [], 'Display', None],
  [ 0xad5c, 'HangulSyllableGyols', [], 'Display', None],
  [ 0xad5d, 'HangulSyllableGyolt', [], 'Display', None],
  [ 0xad5e, 'HangulSyllableGyolp', [], 'Display', None],
  [ 0xad5f, 'HangulSyllableGyolh', [], 'Display', None],
  [ 0xad60, 'HangulSyllableGyom', [], 'Display', None],
  [ 0xad61, 'HangulSyllableGyob', [], 'Display', None],
  [ 0xad62, 'HangulSyllableGyobs', [], 'Display', None],
  [ 0xad63, 'HangulSyllableGyos', [], 'Display', None],
  [ 0xad64, 'HangulSyllableGyoss', [], 'Display', None],
  [ 0xad65, 'HangulSyllableGyong', [], 'Display', None],
  [ 0xad66, 'HangulSyllableGyoj', [], 'Display', None],
  [ 0xad67, 'HangulSyllableGyoc', [], 'Display', None],
  [ 0xad68, 'HangulSyllableGyok', [], 'Display', None],
  [ 0xad69, 'HangulSyllableGyot', [], 'Display', None],
  [ 0xad6a, 'HangulSyllableGyop', [], 'Display', None],
  [ 0xad6b, 'HangulSyllableGyoh', [], 'Display', None],
  [ 0xad6c, 'HangulSyllableGu', [], 'Display', None],
  [ 0xad6d, 'HangulSyllableGug', [], 'Display', None],
  [ 0xad6e, 'HangulSyllableGugg', [], 'Display', None],
  [ 0xad6f, 'HangulSyllableGugs', [], 'Display', None],
  [ 0xad70, 'HangulSyllableGun', [], 'Display', None],
  [ 0xad71, 'HangulSyllableGunj', [], 'Display', None],
  [ 0xad72, 'HangulSyllableGunh', [], 'Display', None],
  [ 0xad73, 'HangulSyllableGud', [], 'Display', None],
  [ 0xad74, 'HangulSyllableGul', [], 'Display', None],
  [ 0xad75, 'HangulSyllableGulg', [], 'Display', None],
  [ 0xad76, 'HangulSyllableGulm', [], 'Display', None],
  [ 0xad77, 'HangulSyllableGulb', [], 'Display', None],
  [ 0xad78, 'HangulSyllableGuls', [], 'Display', None],
  [ 0xad79, 'HangulSyllableGult', [], 'Display', None],
  [ 0xad7a, 'HangulSyllableGulp', [], 'Display', None],
  [ 0xad7b, 'HangulSyllableGulh', [], 'Display', None],
  [ 0xad7c, 'HangulSyllableGum', [], 'Display', None],
  [ 0xad7d, 'HangulSyllableGub', [], 'Display', None],
  [ 0xad7e, 'HangulSyllableGubs', [], 'Display', None],
  [ 0xad7f, 'HangulSyllableGus', [], 'Display', None],
  [ 0xad80, 'HangulSyllableGuss', [], 'Display', None],
  [ 0xad81, 'HangulSyllableGung', [], 'Display', None],
  [ 0xad82, 'HangulSyllableGuj', [], 'Display', None],
  [ 0xad83, 'HangulSyllableGuc', [], 'Display', None],
  [ 0xad84, 'HangulSyllableGuk', [], 'Display', None],
  [ 0xad85, 'HangulSyllableGut', [], 'Display', None],
  [ 0xad86, 'HangulSyllableGup', [], 'Display', None],
  [ 0xad87, 'HangulSyllableGuh', [], 'Display', None],
  [ 0xad88, 'HangulSyllableGweo', [], 'Display', None],
  [ 0xad89, 'HangulSyllableGweog', [], 'Display', None],
  [ 0xad8a, 'HangulSyllableGweogg', [], 'Display', None],
  [ 0xad8b, 'HangulSyllableGweogs', [], 'Display', None],
  [ 0xad8c, 'HangulSyllableGweon', [], 'Display', None],
  [ 0xad8d, 'HangulSyllableGweonj', [], 'Display', None],
  [ 0xad8e, 'HangulSyllableGweonh', [], 'Display', None],
  [ 0xad8f, 'HangulSyllableGweod', [], 'Display', None],
  [ 0xad90, 'HangulSyllableGweol', [], 'Display', None],
  [ 0xad91, 'HangulSyllableGweolg', [], 'Display', None],
  [ 0xad92, 'HangulSyllableGweolm', [], 'Display', None],
  [ 0xad93, 'HangulSyllableGweolb', [], 'Display', None],
  [ 0xad94, 'HangulSyllableGweols', [], 'Display', None],
  [ 0xad95, 'HangulSyllableGweolt', [], 'Display', None],
  [ 0xad96, 'HangulSyllableGweolp', [], 'Display', None],
  [ 0xad97, 'HangulSyllableGweolh', [], 'Display', None],
  [ 0xad98, 'HangulSyllableGweom', [], 'Display', None],
  [ 0xad99, 'HangulSyllableGweob', [], 'Display', None],
  [ 0xad9a, 'HangulSyllableGweobs', [], 'Display', None],
  [ 0xad9b, 'HangulSyllableGweos', [], 'Display', None],
  [ 0xad9c, 'HangulSyllableGweoss', [], 'Display', None],
  [ 0xad9d, 'HangulSyllableGweong', [], 'Display', None],
  [ 0xad9e, 'HangulSyllableGweoj', [], 'Display', None],
  [ 0xad9f, 'HangulSyllableGweoc', [], 'Display', None],
  [ 0xada0, 'HangulSyllableGweok', [], 'Display', None],
  [ 0xada1, 'HangulSyllableGweot', [], 'Display', None],
  [ 0xada2, 'HangulSyllableGweop', [], 'Display', None],
  [ 0xada3, 'HangulSyllableGweoh', [], 'Display', None],
  [ 0xada4, 'HangulSyllableGwe', [], 'Display', None],
  [ 0xada5, 'HangulSyllableGweg', [], 'Display', None],
  [ 0xada6, 'HangulSyllableGwegg', [], 'Display', None],
  [ 0xada7, 'HangulSyllableGwegs', [], 'Display', None],
  [ 0xada8, 'HangulSyllableGwen', [], 'Display', None],
  [ 0xada9, 'HangulSyllableGwenj', [], 'Display', None],
  [ 0xadaa, 'HangulSyllableGwenh', [], 'Display', None],
  [ 0xadab, 'HangulSyllableGwed', [], 'Display', None],
  [ 0xadac, 'HangulSyllableGwel', [], 'Display', None],
  [ 0xadad, 'HangulSyllableGwelg', [], 'Display', None],
  [ 0xadae, 'HangulSyllableGwelm', [], 'Display', None],
  [ 0xadaf, 'HangulSyllableGwelb', [], 'Display', None],
  [ 0xadb0, 'HangulSyllableGwels', [], 'Display', None],
  [ 0xadb1, 'HangulSyllableGwelt', [], 'Display', None],
  [ 0xadb2, 'HangulSyllableGwelp', [], 'Display', None],
  [ 0xadb3, 'HangulSyllableGwelh', [], 'Display', None],
  [ 0xadb4, 'HangulSyllableGwem', [], 'Display', None],
  [ 0xadb5, 'HangulSyllableGweb', [], 'Display', None],
  [ 0xadb6, 'HangulSyllableGwebs', [], 'Display', None],
  [ 0xadb7, 'HangulSyllableGwes', [], 'Display', None],
  [ 0xadb8, 'HangulSyllableGwess', [], 'Display', None],
  [ 0xadb9, 'HangulSyllableGweng', [], 'Display', None],
  [ 0xadba, 'HangulSyllableGwej', [], 'Display', None],
  [ 0xadbb, 'HangulSyllableGwec', [], 'Display', None],
  [ 0xadbc, 'HangulSyllableGwek', [], 'Display', None],
  [ 0xadbd, 'HangulSyllableGwet', [], 'Display', None],
  [ 0xadbe, 'HangulSyllableGwep', [], 'Display', None],
  [ 0xadbf, 'HangulSyllableGweh', [], 'Display', None],
  [ 0xadc0, 'HangulSyllableGwi', [], 'Display', None],
  [ 0xadc1, 'HangulSyllableGwig', [], 'Display', None],
  [ 0xadc2, 'HangulSyllableGwigg', [], 'Display', None],
  [ 0xadc3, 'HangulSyllableGwigs', [], 'Display', None],
  [ 0xadc4, 'HangulSyllableGwin', [], 'Display', None],
  [ 0xadc5, 'HangulSyllableGwinj', [], 'Display', None],
  [ 0xadc6, 'HangulSyllableGwinh', [], 'Display', None],
  [ 0xadc7, 'HangulSyllableGwid', [], 'Display', None],
  [ 0xadc8, 'HangulSyllableGwil', [], 'Display', None],
  [ 0xadc9, 'HangulSyllableGwilg', [], 'Display', None],
  [ 0xadca, 'HangulSyllableGwilm', [], 'Display', None],
  [ 0xadcb, 'HangulSyllableGwilb', [], 'Display', None],
  [ 0xadcc, 'HangulSyllableGwils', [], 'Display', None],
  [ 0xadcd, 'HangulSyllableGwilt', [], 'Display', None],
  [ 0xadce, 'HangulSyllableGwilp', [], 'Display', None],
  [ 0xadcf, 'HangulSyllableGwilh', [], 'Display', None],
  [ 0xadd0, 'HangulSyllableGwim', [], 'Display', None],
  [ 0xadd1, 'HangulSyllableGwib', [], 'Display', None],
  [ 0xadd2, 'HangulSyllableGwibs', [], 'Display', None],
  [ 0xadd3, 'HangulSyllableGwis', [], 'Display', None],
  [ 0xadd4, 'HangulSyllableGwiss', [], 'Display', None],
  [ 0xadd5, 'HangulSyllableGwing', [], 'Display', None],
  [ 0xadd6, 'HangulSyllableGwij', [], 'Display', None],
  [ 0xadd7, 'HangulSyllableGwic', [], 'Display', None],
  [ 0xadd8, 'HangulSyllableGwik', [], 'Display', None],
  [ 0xadd9, 'HangulSyllableGwit', [], 'Display', None],
  [ 0xadda, 'HangulSyllableGwip', [], 'Display', None],
  [ 0xaddb, 'HangulSyllableGwih', [], 'Display', None],
  [ 0xaddc, 'HangulSyllableGyu', [], 'Display', None],
  [ 0xaddd, 'HangulSyllableGyug', [], 'Display', None],
  [ 0xadde, 'HangulSyllableGyugg', [], 'Display', None],
  [ 0xaddf, 'HangulSyllableGyugs', [], 'Display', None],
  [ 0xade0, 'HangulSyllableGyun', [], 'Display', None],
  [ 0xade1, 'HangulSyllableGyunj', [], 'Display', None],
  [ 0xade2, 'HangulSyllableGyunh', [], 'Display', None],
  [ 0xade3, 'HangulSyllableGyud', [], 'Display', None],
  [ 0xade4, 'HangulSyllableGyul', [], 'Display', None],
  [ 0xade5, 'HangulSyllableGyulg', [], 'Display', None],
  [ 0xade6, 'HangulSyllableGyulm', [], 'Display', None],
  [ 0xade7, 'HangulSyllableGyulb', [], 'Display', None],
  [ 0xade8, 'HangulSyllableGyuls', [], 'Display', None],
  [ 0xade9, 'HangulSyllableGyult', [], 'Display', None],
  [ 0xadea, 'HangulSyllableGyulp', [], 'Display', None],
  [ 0xadeb, 'HangulSyllableGyulh', [], 'Display', None],
  [ 0xadec, 'HangulSyllableGyum', [], 'Display', None],
  [ 0xaded, 'HangulSyllableGyub', [], 'Display', None],
  [ 0xadee, 'HangulSyllableGyubs', [], 'Display', None],
  [ 0xadef, 'HangulSyllableGyus', [], 'Display', None],
  [ 0xadf0, 'HangulSyllableGyuss', [], 'Display', None],
  [ 0xadf1, 'HangulSyllableGyung', [], 'Display', None],
  [ 0xadf2, 'HangulSyllableGyuj', [], 'Display', None],
  [ 0xadf3, 'HangulSyllableGyuc', [], 'Display', None],
  [ 0xadf4, 'HangulSyllableGyuk', [], 'Display', None],
  [ 0xadf5, 'HangulSyllableGyut', [], 'Display', None],
  [ 0xadf6, 'HangulSyllableGyup', [], 'Display', None],
  [ 0xadf7, 'HangulSyllableGyuh', [], 'Display', None],
  [ 0xadf8, 'HangulSyllableGeu', [], 'Display', None],
  [ 0xadf9, 'HangulSyllableGeug', [], 'Display', None],
  [ 0xadfa, 'HangulSyllableGeugg', [], 'Display', None],
  [ 0xadfb, 'HangulSyllableGeugs', [], 'Display', None],
  [ 0xadfc, 'HangulSyllableGeun', [], 'Display', None],
  [ 0xadfd, 'HangulSyllableGeunj', [], 'Display', None],
  [ 0xadfe, 'HangulSyllableGeunh', [], 'Display', None],
  [ 0xadff, 'HangulSyllableGeud', [], 'Display', None],
  [ 0xae00, 'HangulSyllableGeul', [], 'Display', None],
  [ 0xae01, 'HangulSyllableGeulg', [], 'Display', None],
  [ 0xae02, 'HangulSyllableGeulm', [], 'Display', None],
  [ 0xae03, 'HangulSyllableGeulb', [], 'Display', None],
  [ 0xae04, 'HangulSyllableGeuls', [], 'Display', None],
  [ 0xae05, 'HangulSyllableGeult', [], 'Display', None],
  [ 0xae06, 'HangulSyllableGeulp', [], 'Display', None],
  [ 0xae07, 'HangulSyllableGeulh', [], 'Display', None],
  [ 0xae08, 'HangulSyllableGeum', [], 'Display', None],
  [ 0xae09, 'HangulSyllableGeub', [], 'Display', None],
  [ 0xae0a, 'HangulSyllableGeubs', [], 'Display', None],
  [ 0xae0b, 'HangulSyllableGeus', [], 'Display', None],
  [ 0xae0c, 'HangulSyllableGeuss', [], 'Display', None],
  [ 0xae0d, 'HangulSyllableGeung', [], 'Display', None],
  [ 0xae0e, 'HangulSyllableGeuj', [], 'Display', None],
  [ 0xae0f, 'HangulSyllableGeuc', [], 'Display', None],
  [ 0xae10, 'HangulSyllableGeuk', [], 'Display', None],
  [ 0xae11, 'HangulSyllableGeut', [], 'Display', None],
  [ 0xae12, 'HangulSyllableGeup', [], 'Display', None],
  [ 0xae13, 'HangulSyllableGeuh', [], 'Display', None],
  [ 0xae14, 'HangulSyllableGyi', [], 'Display', None],
  [ 0xae15, 'HangulSyllableGyig', [], 'Display', None],
  [ 0xae16, 'HangulSyllableGyigg', [], 'Display', None],
  [ 0xae17, 'HangulSyllableGyigs', [], 'Display', None],
  [ 0xae18, 'HangulSyllableGyin', [], 'Display', None],
  [ 0xae19, 'HangulSyllableGyinj', [], 'Display', None],
  [ 0xae1a, 'HangulSyllableGyinh', [], 'Display', None],
  [ 0xae1b, 'HangulSyllableGyid', [], 'Display', None],
  [ 0xae1c, 'HangulSyllableGyil', [], 'Display', None],
  [ 0xae1d, 'HangulSyllableGyilg', [], 'Display', None],
  [ 0xae1e, 'HangulSyllableGyilm', [], 'Display', None],
  [ 0xae1f, 'HangulSyllableGyilb', [], 'Display', None],
  [ 0xae20, 'HangulSyllableGyils', [], 'Display', None],
  [ 0xae21, 'HangulSyllableGyilt', [], 'Display', None],
  [ 0xae22, 'HangulSyllableGyilp', [], 'Display', None],
  [ 0xae23, 'HangulSyllableGyilh', [], 'Display', None],
  [ 0xae24, 'HangulSyllableGyim', [], 'Display', None],
  [ 0xae25, 'HangulSyllableGyib', [], 'Display', None],
  [ 0xae26, 'HangulSyllableGyibs', [], 'Display', None],
  [ 0xae27, 'HangulSyllableGyis', [], 'Display', None],
  [ 0xae28, 'HangulSyllableGyiss', [], 'Display', None],
  [ 0xae29, 'HangulSyllableGying', [], 'Display', None],
  [ 0xae2a, 'HangulSyllableGyij', [], 'Display', None],
  [ 0xae2b, 'HangulSyllableGyic', [], 'Display', None],
  [ 0xae2c, 'HangulSyllableGyik', [], 'Display', None],
  [ 0xae2d, 'HangulSyllableGyit', [], 'Display', None],
  [ 0xae2e, 'HangulSyllableGyip', [], 'Display', None],
  [ 0xae2f, 'HangulSyllableGyih', [], 'Display', None],
  [ 0xae30, 'HangulSyllableGi', [], 'Display', None],
  [ 0xae31, 'HangulSyllableGig', [], 'Display', None],
  [ 0xae32, 'HangulSyllableGigg', [], 'Display', None],
  [ 0xae33, 'HangulSyllableGigs', [], 'Display', None],
  [ 0xae34, 'HangulSyllableGin', [], 'Display', None],
  [ 0xae35, 'HangulSyllableGinj', [], 'Display', None],
  [ 0xae36, 'HangulSyllableGinh', [], 'Display', None],
  [ 0xae37, 'HangulSyllableGid', [], 'Display', None],
  [ 0xae38, 'HangulSyllableGil', [], 'Display', None],
  [ 0xae39, 'HangulSyllableGilg', [], 'Display', None],
  [ 0xae3a, 'HangulSyllableGilm', [], 'Display', None],
  [ 0xae3b, 'HangulSyllableGilb', [], 'Display', None],
  [ 0xae3c, 'HangulSyllableGils', [], 'Display', None],
  [ 0xae3d, 'HangulSyllableGilt', [], 'Display', None],
  [ 0xae3e, 'HangulSyllableGilp', [], 'Display', None],
  [ 0xae3f, 'HangulSyllableGilh', [], 'Display', None],
  [ 0xae40, 'HangulSyllableGim', [], 'Display', None],
  [ 0xae41, 'HangulSyllableGib', [], 'Display', None],
  [ 0xae42, 'HangulSyllableGibs', [], 'Display', None],
  [ 0xae43, 'HangulSyllableGis', [], 'Display', None],
  [ 0xae44, 'HangulSyllableGiss', [], 'Display', None],
  [ 0xae45, 'HangulSyllableGing', [], 'Display', None],
  [ 0xae46, 'HangulSyllableGij', [], 'Display', None],
  [ 0xae47, 'HangulSyllableGic', [], 'Display', None],
  [ 0xae48, 'HangulSyllableGik', [], 'Display', None],
  [ 0xae49, 'HangulSyllableGit', [], 'Display', None],
  [ 0xae4a, 'HangulSyllableGip', [], 'Display', None],
  [ 0xae4b, 'HangulSyllableGih', [], 'Display', None],
  [ 0xae4c, 'HangulSyllableGga', [], 'Display', None],
  [ 0xae4d, 'HangulSyllableGgag', [], 'Display', None],
  [ 0xae4e, 'HangulSyllableGgagg', [], 'Display', None],
  [ 0xae4f, 'HangulSyllableGgags', [], 'Display', None],
  [ 0xae50, 'HangulSyllableGgan', [], 'Display', None],
  [ 0xae51, 'HangulSyllableGganj', [], 'Display', None],
  [ 0xae52, 'HangulSyllableGganh', [], 'Display', None],
  [ 0xae53, 'HangulSyllableGgad', [], 'Display', None],
  [ 0xae54, 'HangulSyllableGgal', [], 'Display', None],
  [ 0xae55, 'HangulSyllableGgalg', [], 'Display', None],
  [ 0xae56, 'HangulSyllableGgalm', [], 'Display', None],
  [ 0xae57, 'HangulSyllableGgalb', [], 'Display', None],
  [ 0xae58, 'HangulSyllableGgals', [], 'Display', None],
  [ 0xae59, 'HangulSyllableGgalt', [], 'Display', None],
  [ 0xae5a, 'HangulSyllableGgalp', [], 'Display', None],
  [ 0xae5b, 'HangulSyllableGgalh', [], 'Display', None],
  [ 0xae5c, 'HangulSyllableGgam', [], 'Display', None],
  [ 0xae5d, 'HangulSyllableGgab', [], 'Display', None],
  [ 0xae5e, 'HangulSyllableGgabs', [], 'Display', None],
  [ 0xae5f, 'HangulSyllableGgas', [], 'Display', None],
  [ 0xae60, 'HangulSyllableGgass', [], 'Display', None],
  [ 0xae61, 'HangulSyllableGgang', [], 'Display', None],
  [ 0xae62, 'HangulSyllableGgaj', [], 'Display', None],
  [ 0xae63, 'HangulSyllableGgac', [], 'Display', None],
  [ 0xae64, 'HangulSyllableGgak', [], 'Display', None],
  [ 0xae65, 'HangulSyllableGgat', [], 'Display', None],
  [ 0xae66, 'HangulSyllableGgap', [], 'Display', None],
  [ 0xae67, 'HangulSyllableGgah', [], 'Display', None],
  [ 0xae68, 'HangulSyllableGgae', [], 'Display', None],
  [ 0xae69, 'HangulSyllableGgaeg', [], 'Display', None],
  [ 0xae6a, 'HangulSyllableGgaegg', [], 'Display', None],
  [ 0xae6b, 'HangulSyllableGgaegs', [], 'Display', None],
  [ 0xae6c, 'HangulSyllableGgaen', [], 'Display', None],
  [ 0xae6d, 'HangulSyllableGgaenj', [], 'Display', None],
  [ 0xae6e, 'HangulSyllableGgaenh', [], 'Display', None],
  [ 0xae6f, 'HangulSyllableGgaed', [], 'Display', None],
  [ 0xae70, 'HangulSyllableGgael', [], 'Display', None],
  [ 0xae71, 'HangulSyllableGgaelg', [], 'Display', None],
  [ 0xae72, 'HangulSyllableGgaelm', [], 'Display', None],
  [ 0xae73, 'HangulSyllableGgaelb', [], 'Display', None],
  [ 0xae74, 'HangulSyllableGgaels', [], 'Display', None],
  [ 0xae75, 'HangulSyllableGgaelt', [], 'Display', None],
  [ 0xae76, 'HangulSyllableGgaelp', [], 'Display', None],
  [ 0xae77, 'HangulSyllableGgaelh', [], 'Display', None],
  [ 0xae78, 'HangulSyllableGgaem', [], 'Display', None],
  [ 0xae79, 'HangulSyllableGgaeb', [], 'Display', None],
  [ 0xae7a, 'HangulSyllableGgaebs', [], 'Display', None],
  [ 0xae7b, 'HangulSyllableGgaes', [], 'Display', None],
  [ 0xae7c, 'HangulSyllableGgaess', [], 'Display', None],
  [ 0xae7d, 'HangulSyllableGgaeng', [], 'Display', None],
  [ 0xae7e, 'HangulSyllableGgaej', [], 'Display', None],
  [ 0xae7f, 'HangulSyllableGgaec', [], 'Display', None],
  [ 0xae80, 'HangulSyllableGgaek', [], 'Display', None],
  [ 0xae81, 'HangulSyllableGgaet', [], 'Display', None],
  [ 0xae82, 'HangulSyllableGgaep', [], 'Display', None],
  [ 0xae83, 'HangulSyllableGgaeh', [], 'Display', None],
  [ 0xae84, 'HangulSyllableGgya', [], 'Display', None],
  [ 0xae85, 'HangulSyllableGgyag', [], 'Display', None],
  [ 0xae86, 'HangulSyllableGgyagg', [], 'Display', None],
  [ 0xae87, 'HangulSyllableGgyags', [], 'Display', None],
  [ 0xae88, 'HangulSyllableGgyan', [], 'Display', None],
  [ 0xae89, 'HangulSyllableGgyanj', [], 'Display', None],
  [ 0xae8a, 'HangulSyllableGgyanh', [], 'Display', None],
  [ 0xae8b, 'HangulSyllableGgyad', [], 'Display', None],
  [ 0xae8c, 'HangulSyllableGgyal', [], 'Display', None],
  [ 0xae8d, 'HangulSyllableGgyalg', [], 'Display', None],
  [ 0xae8e, 'HangulSyllableGgyalm', [], 'Display', None],
  [ 0xae8f, 'HangulSyllableGgyalb', [], 'Display', None],
  [ 0xae90, 'HangulSyllableGgyals', [], 'Display', None],
  [ 0xae91, 'HangulSyllableGgyalt', [], 'Display', None],
  [ 0xae92, 'HangulSyllableGgyalp', [], 'Display', None],
  [ 0xae93, 'HangulSyllableGgyalh', [], 'Display', None],
  [ 0xae94, 'HangulSyllableGgyam', [], 'Display', None],
  [ 0xae95, 'HangulSyllableGgyab', [], 'Display', None],
  [ 0xae96, 'HangulSyllableGgyabs', [], 'Display', None],
  [ 0xae97, 'HangulSyllableGgyas', [], 'Display', None],
  [ 0xae98, 'HangulSyllableGgyass', [], 'Display', None],
  [ 0xae99, 'HangulSyllableGgyang', [], 'Display', None],
  [ 0xae9a, 'HangulSyllableGgyaj', [], 'Display', None],
  [ 0xae9b, 'HangulSyllableGgyac', [], 'Display', None],
  [ 0xae9c, 'HangulSyllableGgyak', [], 'Display', None],
  [ 0xae9d, 'HangulSyllableGgyat', [], 'Display', None],
  [ 0xae9e, 'HangulSyllableGgyap', [], 'Display', None],
  [ 0xae9f, 'HangulSyllableGgyah', [], 'Display', None],
  [ 0xaea0, 'HangulSyllableGgyae', [], 'Display', None],
  [ 0xaea1, 'HangulSyllableGgyaeg', [], 'Display', None],
  [ 0xaea2, 'HangulSyllableGgyaegg', [], 'Display', None],
  [ 0xaea3, 'HangulSyllableGgyaegs', [], 'Display', None],
  [ 0xaea4, 'HangulSyllableGgyaen', [], 'Display', None],
  [ 0xaea5, 'HangulSyllableGgyaenj', [], 'Display', None],
  [ 0xaea6, 'HangulSyllableGgyaenh', [], 'Display', None],
  [ 0xaea7, 'HangulSyllableGgyaed', [], 'Display', None],
  [ 0xaea8, 'HangulSyllableGgyael', [], 'Display', None],
  [ 0xaea9, 'HangulSyllableGgyaelg', [], 'Display', None],
  [ 0xaeaa, 'HangulSyllableGgyaelm', [], 'Display', None],
  [ 0xaeab, 'HangulSyllableGgyaelb', [], 'Display', None],
  [ 0xaeac, 'HangulSyllableGgyaels', [], 'Display', None],
  [ 0xaead, 'HangulSyllableGgyaelt', [], 'Display', None],
  [ 0xaeae, 'HangulSyllableGgyaelp', [], 'Display', None],
  [ 0xaeaf, 'HangulSyllableGgyaelh', [], 'Display', None],
  [ 0xaeb0, 'HangulSyllableGgyaem', [], 'Display', None],
  [ 0xaeb1, 'HangulSyllableGgyaeb', [], 'Display', None],
  [ 0xaeb2, 'HangulSyllableGgyaebs', [], 'Display', None],
  [ 0xaeb3, 'HangulSyllableGgyaes', [], 'Display', None],
  [ 0xaeb4, 'HangulSyllableGgyaess', [], 'Display', None],
  [ 0xaeb5, 'HangulSyllableGgyaeng', [], 'Display', None],
  [ 0xaeb6, 'HangulSyllableGgyaej', [], 'Display', None],
  [ 0xaeb7, 'HangulSyllableGgyaec', [], 'Display', None],
  [ 0xaeb8, 'HangulSyllableGgyaek', [], 'Display', None],
  [ 0xaeb9, 'HangulSyllableGgyaet', [], 'Display', None],
  [ 0xaeba, 'HangulSyllableGgyaep', [], 'Display', None],
  [ 0xaebb, 'HangulSyllableGgyaeh', [], 'Display', None],
  [ 0xaebc, 'HangulSyllableGgeo', [], 'Display', None],
  [ 0xaebd, 'HangulSyllableGgeog', [], 'Display', None],
  [ 0xaebe, 'HangulSyllableGgeogg', [], 'Display', None],
  [ 0xaebf, 'HangulSyllableGgeogs', [], 'Display', None],
  [ 0xaec0, 'HangulSyllableGgeon', [], 'Display', None],
  [ 0xaec1, 'HangulSyllableGgeonj', [], 'Display', None],
  [ 0xaec2, 'HangulSyllableGgeonh', [], 'Display', None],
  [ 0xaec3, 'HangulSyllableGgeod', [], 'Display', None],
  [ 0xaec4, 'HangulSyllableGgeol', [], 'Display', None],
  [ 0xaec5, 'HangulSyllableGgeolg', [], 'Display', None],
  [ 0xaec6, 'HangulSyllableGgeolm', [], 'Display', None],
  [ 0xaec7, 'HangulSyllableGgeolb', [], 'Display', None],
  [ 0xaec8, 'HangulSyllableGgeols', [], 'Display', None],
  [ 0xaec9, 'HangulSyllableGgeolt', [], 'Display', None],
  [ 0xaeca, 'HangulSyllableGgeolp', [], 'Display', None],
  [ 0xaecb, 'HangulSyllableGgeolh', [], 'Display', None],
  [ 0xaecc, 'HangulSyllableGgeom', [], 'Display', None],
  [ 0xaecd, 'HangulSyllableGgeob', [], 'Display', None],
  [ 0xaece, 'HangulSyllableGgeobs', [], 'Display', None],
  [ 0xaecf, 'HangulSyllableGgeos', [], 'Display', None],
  [ 0xaed0, 'HangulSyllableGgeoss', [], 'Display', None],
  [ 0xaed1, 'HangulSyllableGgeong', [], 'Display', None],
  [ 0xaed2, 'HangulSyllableGgeoj', [], 'Display', None],
  [ 0xaed3, 'HangulSyllableGgeoc', [], 'Display', None],
  [ 0xaed4, 'HangulSyllableGgeok', [], 'Display', None],
  [ 0xaed5, 'HangulSyllableGgeot', [], 'Display', None],
  [ 0xaed6, 'HangulSyllableGgeop', [], 'Display', None],
  [ 0xaed7, 'HangulSyllableGgeoh', [], 'Display', None],
  [ 0xaed8, 'HangulSyllableGge', [], 'Display', None],
  [ 0xaed9, 'HangulSyllableGgeg', [], 'Display', None],
  [ 0xaeda, 'HangulSyllableGgegg', [], 'Display', None],
  [ 0xaedb, 'HangulSyllableGgegs', [], 'Display', None],
  [ 0xaedc, 'HangulSyllableGgen', [], 'Display', None],
  [ 0xaedd, 'HangulSyllableGgenj', [], 'Display', None],
  [ 0xaede, 'HangulSyllableGgenh', [], 'Display', None],
  [ 0xaedf, 'HangulSyllableGged', [], 'Display', None],
  [ 0xaee0, 'HangulSyllableGgel', [], 'Display', None],
  [ 0xaee1, 'HangulSyllableGgelg', [], 'Display', None],
  [ 0xaee2, 'HangulSyllableGgelm', [], 'Display', None],
  [ 0xaee3, 'HangulSyllableGgelb', [], 'Display', None],
  [ 0xaee4, 'HangulSyllableGgels', [], 'Display', None],
  [ 0xaee5, 'HangulSyllableGgelt', [], 'Display', None],
  [ 0xaee6, 'HangulSyllableGgelp', [], 'Display', None],
  [ 0xaee7, 'HangulSyllableGgelh', [], 'Display', None],
  [ 0xaee8, 'HangulSyllableGgem', [], 'Display', None],
  [ 0xaee9, 'HangulSyllableGgeb', [], 'Display', None],
  [ 0xaeea, 'HangulSyllableGgebs', [], 'Display', None],
  [ 0xaeeb, 'HangulSyllableGges', [], 'Display', None],
  [ 0xaeec, 'HangulSyllableGgess', [], 'Display', None],
  [ 0xaeed, 'HangulSyllableGgeng', [], 'Display', None],
  [ 0xaeee, 'HangulSyllableGgej', [], 'Display', None],
  [ 0xaeef, 'HangulSyllableGgec', [], 'Display', None],
  [ 0xaef0, 'HangulSyllableGgek', [], 'Display', None],
  [ 0xaef1, 'HangulSyllableGget', [], 'Display', None],
  [ 0xaef2, 'HangulSyllableGgep', [], 'Display', None],
  [ 0xaef3, 'HangulSyllableGgeh', [], 'Display', None],
  [ 0xaef4, 'HangulSyllableGgyeo', [], 'Display', None],
  [ 0xaef5, 'HangulSyllableGgyeog', [], 'Display', None],
  [ 0xaef6, 'HangulSyllableGgyeogg', [], 'Display', None],
  [ 0xaef7, 'HangulSyllableGgyeogs', [], 'Display', None],
  [ 0xaef8, 'HangulSyllableGgyeon', [], 'Display', None],
  [ 0xaef9, 'HangulSyllableGgyeonj', [], 'Display', None],
  [ 0xaefa, 'HangulSyllableGgyeonh', [], 'Display', None],
  [ 0xaefb, 'HangulSyllableGgyeod', [], 'Display', None],
  [ 0xaefc, 'HangulSyllableGgyeol', [], 'Display', None],
  [ 0xaefd, 'HangulSyllableGgyeolg', [], 'Display', None],
  [ 0xaefe, 'HangulSyllableGgyeolm', [], 'Display', None],
  [ 0xaeff, 'HangulSyllableGgyeolb', [], 'Display', None],
  [ 0xaf00, 'HangulSyllableGgyeols', [], 'Display', None],
  [ 0xaf01, 'HangulSyllableGgyeolt', [], 'Display', None],
  [ 0xaf02, 'HangulSyllableGgyeolp', [], 'Display', None],
  [ 0xaf03, 'HangulSyllableGgyeolh', [], 'Display', None],
  [ 0xaf04, 'HangulSyllableGgyeom', [], 'Display', None],
  [ 0xaf05, 'HangulSyllableGgyeob', [], 'Display', None],
  [ 0xaf06, 'HangulSyllableGgyeobs', [], 'Display', None],
  [ 0xaf07, 'HangulSyllableGgyeos', [], 'Display', None],
  [ 0xaf08, 'HangulSyllableGgyeoss', [], 'Display', None],
  [ 0xaf09, 'HangulSyllableGgyeong', [], 'Display', None],
  [ 0xaf0a, 'HangulSyllableGgyeoj', [], 'Display', None],
  [ 0xaf0b, 'HangulSyllableGgyeoc', [], 'Display', None],
  [ 0xaf0c, 'HangulSyllableGgyeok', [], 'Display', None],
  [ 0xaf0d, 'HangulSyllableGgyeot', [], 'Display', None],
  [ 0xaf0e, 'HangulSyllableGgyeop', [], 'Display', None],
  [ 0xaf0f, 'HangulSyllableGgyeoh', [], 'Display', None],
  [ 0xaf10, 'HangulSyllableGgye', [], 'Display', None],
  [ 0xaf11, 'HangulSyllableGgyeg', [], 'Display', None],
  [ 0xaf12, 'HangulSyllableGgyegg', [], 'Display', None],
  [ 0xaf13, 'HangulSyllableGgyegs', [], 'Display', None],
  [ 0xaf14, 'HangulSyllableGgyen', [], 'Display', None],
  [ 0xaf15, 'HangulSyllableGgyenj', [], 'Display', None],
  [ 0xaf16, 'HangulSyllableGgyenh', [], 'Display', None],
  [ 0xaf17, 'HangulSyllableGgyed', [], 'Display', None],
  [ 0xaf18, 'HangulSyllableGgyel', [], 'Display', None],
  [ 0xaf19, 'HangulSyllableGgyelg', [], 'Display', None],
  [ 0xaf1a, 'HangulSyllableGgyelm', [], 'Display', None],
  [ 0xaf1b, 'HangulSyllableGgyelb', [], 'Display', None],
  [ 0xaf1c, 'HangulSyllableGgyels', [], 'Display', None],
  [ 0xaf1d, 'HangulSyllableGgyelt', [], 'Display', None],
  [ 0xaf1e, 'HangulSyllableGgyelp', [], 'Display', None],
  [ 0xaf1f, 'HangulSyllableGgyelh', [], 'Display', None],
  [ 0xaf20, 'HangulSyllableGgyem', [], 'Display', None],
  [ 0xaf21, 'HangulSyllableGgyeb', [], 'Display', None],
  [ 0xaf22, 'HangulSyllableGgyebs', [], 'Display', None],
  [ 0xaf23, 'HangulSyllableGgyes', [], 'Display', None],
  [ 0xaf24, 'HangulSyllableGgyess', [], 'Display', None],
  [ 0xaf25, 'HangulSyllableGgyeng', [], 'Display', None],
  [ 0xaf26, 'HangulSyllableGgyej', [], 'Display', None],
  [ 0xaf27, 'HangulSyllableGgyec', [], 'Display', None],
  [ 0xaf28, 'HangulSyllableGgyek', [], 'Display', None],
  [ 0xaf29, 'HangulSyllableGgyet', [], 'Display', None],
  [ 0xaf2a, 'HangulSyllableGgyep', [], 'Display', None],
  [ 0xaf2b, 'HangulSyllableGgyeh', [], 'Display', None],
  [ 0xaf2c, 'HangulSyllableGgo', [], 'Display', None],
  [ 0xaf2d, 'HangulSyllableGgog', [], 'Display', None],
  [ 0xaf2e, 'HangulSyllableGgogg', [], 'Display', None],
  [ 0xaf2f, 'HangulSyllableGgogs', [], 'Display', None],
  [ 0xaf30, 'HangulSyllableGgon', [], 'Display', None],
  [ 0xaf31, 'HangulSyllableGgonj', [], 'Display', None],
  [ 0xaf32, 'HangulSyllableGgonh', [], 'Display', None],
  [ 0xaf33, 'HangulSyllableGgod', [], 'Display', None],
  [ 0xaf34, 'HangulSyllableGgol', [], 'Display', None],
  [ 0xaf35, 'HangulSyllableGgolg', [], 'Display', None],
  [ 0xaf36, 'HangulSyllableGgolm', [], 'Display', None],
  [ 0xaf37, 'HangulSyllableGgolb', [], 'Display', None],
  [ 0xaf38, 'HangulSyllableGgols', [], 'Display', None],
  [ 0xaf39, 'HangulSyllableGgolt', [], 'Display', None],
  [ 0xaf3a, 'HangulSyllableGgolp', [], 'Display', None],
  [ 0xaf3b, 'HangulSyllableGgolh', [], 'Display', None],
  [ 0xaf3c, 'HangulSyllableGgom', [], 'Display', None],
  [ 0xaf3d, 'HangulSyllableGgob', [], 'Display', None],
  [ 0xaf3e, 'HangulSyllableGgobs', [], 'Display', None],
  [ 0xaf3f, 'HangulSyllableGgos', [], 'Display', None],
  [ 0xaf40, 'HangulSyllableGgoss', [], 'Display', None],
  [ 0xaf41, 'HangulSyllableGgong', [], 'Display', None],
  [ 0xaf42, 'HangulSyllableGgoj', [], 'Display', None],
  [ 0xaf43, 'HangulSyllableGgoc', [], 'Display', None],
  [ 0xaf44, 'HangulSyllableGgok', [], 'Display', None],
  [ 0xaf45, 'HangulSyllableGgot', [], 'Display', None],
  [ 0xaf46, 'HangulSyllableGgop', [], 'Display', None],
  [ 0xaf47, 'HangulSyllableGgoh', [], 'Display', None],
  [ 0xaf48, 'HangulSyllableGgwa', [], 'Display', None],
  [ 0xaf49, 'HangulSyllableGgwag', [], 'Display', None],
  [ 0xaf4a, 'HangulSyllableGgwagg', [], 'Display', None],
  [ 0xaf4b, 'HangulSyllableGgwags', [], 'Display', None],
  [ 0xaf4c, 'HangulSyllableGgwan', [], 'Display', None],
  [ 0xaf4d, 'HangulSyllableGgwanj', [], 'Display', None],
  [ 0xaf4e, 'HangulSyllableGgwanh', [], 'Display', None],
  [ 0xaf4f, 'HangulSyllableGgwad', [], 'Display', None],
  [ 0xaf50, 'HangulSyllableGgwal', [], 'Display', None],
  [ 0xaf51, 'HangulSyllableGgwalg', [], 'Display', None],
  [ 0xaf52, 'HangulSyllableGgwalm', [], 'Display', None],
  [ 0xaf53, 'HangulSyllableGgwalb', [], 'Display', None],
  [ 0xaf54, 'HangulSyllableGgwals', [], 'Display', None],
  [ 0xaf55, 'HangulSyllableGgwalt', [], 'Display', None],
  [ 0xaf56, 'HangulSyllableGgwalp', [], 'Display', None],
  [ 0xaf57, 'HangulSyllableGgwalh', [], 'Display', None],
  [ 0xaf58, 'HangulSyllableGgwam', [], 'Display', None],
  [ 0xaf59, 'HangulSyllableGgwab', [], 'Display', None],
  [ 0xaf5a, 'HangulSyllableGgwabs', [], 'Display', None],
  [ 0xaf5b, 'HangulSyllableGgwas', [], 'Display', None],
  [ 0xaf5c, 'HangulSyllableGgwass', [], 'Display', None],
  [ 0xaf5d, 'HangulSyllableGgwang', [], 'Display', None],
  [ 0xaf5e, 'HangulSyllableGgwaj', [], 'Display', None],
  [ 0xaf5f, 'HangulSyllableGgwac', [], 'Display', None],
  [ 0xaf60, 'HangulSyllableGgwak', [], 'Display', None],
  [ 0xaf61, 'HangulSyllableGgwat', [], 'Display', None],
  [ 0xaf62, 'HangulSyllableGgwap', [], 'Display', None],
  [ 0xaf63, 'HangulSyllableGgwah', [], 'Display', None],
  [ 0xaf64, 'HangulSyllableGgwae', [], 'Display', None],
  [ 0xaf65, 'HangulSyllableGgwaeg', [], 'Display', None],
  [ 0xaf66, 'HangulSyllableGgwaegg', [], 'Display', None],
  [ 0xaf67, 'HangulSyllableGgwaegs', [], 'Display', None],
  [ 0xaf68, 'HangulSyllableGgwaen', [], 'Display', None],
  [ 0xaf69, 'HangulSyllableGgwaenj', [], 'Display', None],
  [ 0xaf6a, 'HangulSyllableGgwaenh', [], 'Display', None],
  [ 0xaf6b, 'HangulSyllableGgwaed', [], 'Display', None],
  [ 0xaf6c, 'HangulSyllableGgwael', [], 'Display', None],
  [ 0xaf6d, 'HangulSyllableGgwaelg', [], 'Display', None],
  [ 0xaf6e, 'HangulSyllableGgwaelm', [], 'Display', None],
  [ 0xaf6f, 'HangulSyllableGgwaelb', [], 'Display', None],
  [ 0xaf70, 'HangulSyllableGgwaels', [], 'Display', None],
  [ 0xaf71, 'HangulSyllableGgwaelt', [], 'Display', None],
  [ 0xaf72, 'HangulSyllableGgwaelp', [], 'Display', None],
  [ 0xaf73, 'HangulSyllableGgwaelh', [], 'Display', None],
  [ 0xaf74, 'HangulSyllableGgwaem', [], 'Display', None],
  [ 0xaf75, 'HangulSyllableGgwaeb', [], 'Display', None],
  [ 0xaf76, 'HangulSyllableGgwaebs', [], 'Display', None],
  [ 0xaf77, 'HangulSyllableGgwaes', [], 'Display', None],
  [ 0xaf78, 'HangulSyllableGgwaess', [], 'Display', None],
  [ 0xaf79, 'HangulSyllableGgwaeng', [], 'Display', None],
  [ 0xaf7a, 'HangulSyllableGgwaej', [], 'Display', None],
  [ 0xaf7b, 'HangulSyllableGgwaec', [], 'Display', None],
  [ 0xaf7c, 'HangulSyllableGgwaek', [], 'Display', None],
  [ 0xaf7d, 'HangulSyllableGgwaet', [], 'Display', None],
  [ 0xaf7e, 'HangulSyllableGgwaep', [], 'Display', None],
  [ 0xaf7f, 'HangulSyllableGgwaeh', [], 'Display', None],
  [ 0xaf80, 'HangulSyllableGgoe', [], 'Display', None],
  [ 0xaf81, 'HangulSyllableGgoeg', [], 'Display', None],
  [ 0xaf82, 'HangulSyllableGgoegg', [], 'Display', None],
  [ 0xaf83, 'HangulSyllableGgoegs', [], 'Display', None],
  [ 0xaf84, 'HangulSyllableGgoen', [], 'Display', None],
  [ 0xaf85, 'HangulSyllableGgoenj', [], 'Display', None],
  [ 0xaf86, 'HangulSyllableGgoenh', [], 'Display', None],
  [ 0xaf87, 'HangulSyllableGgoed', [], 'Display', None],
  [ 0xaf88, 'HangulSyllableGgoel', [], 'Display', None],
  [ 0xaf89, 'HangulSyllableGgoelg', [], 'Display', None],
  [ 0xaf8a, 'HangulSyllableGgoelm', [], 'Display', None],
  [ 0xaf8b, 'HangulSyllableGgoelb', [], 'Display', None],
  [ 0xaf8c, 'HangulSyllableGgoels', [], 'Display', None],
  [ 0xaf8d, 'HangulSyllableGgoelt', [], 'Display', None],
  [ 0xaf8e, 'HangulSyllableGgoelp', [], 'Display', None],
  [ 0xaf8f, 'HangulSyllableGgoelh', [], 'Display', None],
  [ 0xaf90, 'HangulSyllableGgoem', [], 'Display', None],
  [ 0xaf91, 'HangulSyllableGgoeb', [], 'Display', None],
  [ 0xaf92, 'HangulSyllableGgoebs', [], 'Display', None],
  [ 0xaf93, 'HangulSyllableGgoes', [], 'Display', None],
  [ 0xaf94, 'HangulSyllableGgoess', [], 'Display', None],
  [ 0xaf95, 'HangulSyllableGgoeng', [], 'Display', None],
  [ 0xaf96, 'HangulSyllableGgoej', [], 'Display', None],
  [ 0xaf97, 'HangulSyllableGgoec', [], 'Display', None],
  [ 0xaf98, 'HangulSyllableGgoek', [], 'Display', None],
  [ 0xaf99, 'HangulSyllableGgoet', [], 'Display', None],
  [ 0xaf9a, 'HangulSyllableGgoep', [], 'Display', None],
  [ 0xaf9b, 'HangulSyllableGgoeh', [], 'Display', None],
  [ 0xaf9c, 'HangulSyllableGgyo', [], 'Display', None],
  [ 0xaf9d, 'HangulSyllableGgyog', [], 'Display', None],
  [ 0xaf9e, 'HangulSyllableGgyogg', [], 'Display', None],
  [ 0xaf9f, 'HangulSyllableGgyogs', [], 'Display', None],
  [ 0xafa0, 'HangulSyllableGgyon', [], 'Display', None],
  [ 0xafa1, 'HangulSyllableGgyonj', [], 'Display', None],
  [ 0xafa2, 'HangulSyllableGgyonh', [], 'Display', None],
  [ 0xafa3, 'HangulSyllableGgyod', [], 'Display', None],
  [ 0xafa4, 'HangulSyllableGgyol', [], 'Display', None],
  [ 0xafa5, 'HangulSyllableGgyolg', [], 'Display', None],
  [ 0xafa6, 'HangulSyllableGgyolm', [], 'Display', None],
  [ 0xafa7, 'HangulSyllableGgyolb', [], 'Display', None],
  [ 0xafa8, 'HangulSyllableGgyols', [], 'Display', None],
  [ 0xafa9, 'HangulSyllableGgyolt', [], 'Display', None],
  [ 0xafaa, 'HangulSyllableGgyolp', [], 'Display', None],
  [ 0xafab, 'HangulSyllableGgyolh', [], 'Display', None],
  [ 0xafac, 'HangulSyllableGgyom', [], 'Display', None],
  [ 0xafad, 'HangulSyllableGgyob', [], 'Display', None],
  [ 0xafae, 'HangulSyllableGgyobs', [], 'Display', None],
  [ 0xafaf, 'HangulSyllableGgyos', [], 'Display', None],
  [ 0xafb0, 'HangulSyllableGgyoss', [], 'Display', None],
  [ 0xafb1, 'HangulSyllableGgyong', [], 'Display', None],
  [ 0xafb2, 'HangulSyllableGgyoj', [], 'Display', None],
  [ 0xafb3, 'HangulSyllableGgyoc', [], 'Display', None],
  [ 0xafb4, 'HangulSyllableGgyok', [], 'Display', None],
  [ 0xafb5, 'HangulSyllableGgyot', [], 'Display', None],
  [ 0xafb6, 'HangulSyllableGgyop', [], 'Display', None],
  [ 0xafb7, 'HangulSyllableGgyoh', [], 'Display', None],
  [ 0xafb8, 'HangulSyllableGgu', [], 'Display', None],
  [ 0xafb9, 'HangulSyllableGgug', [], 'Display', None],
  [ 0xafba, 'HangulSyllableGgugg', [], 'Display', None],
  [ 0xafbb, 'HangulSyllableGgugs', [], 'Display', None],
  [ 0xafbc, 'HangulSyllableGgun', [], 'Display', None],
  [ 0xafbd, 'HangulSyllableGgunj', [], 'Display', None],
  [ 0xafbe, 'HangulSyllableGgunh', [], 'Display', None],
  [ 0xafbf, 'HangulSyllableGgud', [], 'Display', None],
  [ 0xafc0, 'HangulSyllableGgul', [], 'Display', None],
  [ 0xafc1, 'HangulSyllableGgulg', [], 'Display', None],
  [ 0xafc2, 'HangulSyllableGgulm', [], 'Display', None],
  [ 0xafc3, 'HangulSyllableGgulb', [], 'Display', None],
  [ 0xafc4, 'HangulSyllableGguls', [], 'Display', None],
  [ 0xafc5, 'HangulSyllableGgult', [], 'Display', None],
  [ 0xafc6, 'HangulSyllableGgulp', [], 'Display', None],
  [ 0xafc7, 'HangulSyllableGgulh', [], 'Display', None],
  [ 0xafc8, 'HangulSyllableGgum', [], 'Display', None],
  [ 0xafc9, 'HangulSyllableGgub', [], 'Display', None],
  [ 0xafca, 'HangulSyllableGgubs', [], 'Display', None],
  [ 0xafcb, 'HangulSyllableGgus', [], 'Display', None],
  [ 0xafcc, 'HangulSyllableGguss', [], 'Display', None],
  [ 0xafcd, 'HangulSyllableGgung', [], 'Display', None],
  [ 0xafce, 'HangulSyllableGguj', [], 'Display', None],
  [ 0xafcf, 'HangulSyllableGguc', [], 'Display', None],
  [ 0xafd0, 'HangulSyllableGguk', [], 'Display', None],
  [ 0xafd1, 'HangulSyllableGgut', [], 'Display', None],
  [ 0xafd2, 'HangulSyllableGgup', [], 'Display', None],
  [ 0xafd3, 'HangulSyllableGguh', [], 'Display', None],
  [ 0xafd4, 'HangulSyllableGgweo', [], 'Display', None],
  [ 0xafd5, 'HangulSyllableGgweog', [], 'Display', None],
  [ 0xafd6, 'HangulSyllableGgweogg', [], 'Display', None],
  [ 0xafd7, 'HangulSyllableGgweogs', [], 'Display', None],
  [ 0xafd8, 'HangulSyllableGgweon', [], 'Display', None],
  [ 0xafd9, 'HangulSyllableGgweonj', [], 'Display', None],
  [ 0xafda, 'HangulSyllableGgweonh', [], 'Display', None],
  [ 0xafdb, 'HangulSyllableGgweod', [], 'Display', None],
  [ 0xafdc, 'HangulSyllableGgweol', [], 'Display', None],
  [ 0xafdd, 'HangulSyllableGgweolg', [], 'Display', None],
  [ 0xafde, 'HangulSyllableGgweolm', [], 'Display', None],
  [ 0xafdf, 'HangulSyllableGgweolb', [], 'Display', None],
  [ 0xafe0, 'HangulSyllableGgweols', [], 'Display', None],
  [ 0xafe1, 'HangulSyllableGgweolt', [], 'Display', None],
  [ 0xafe2, 'HangulSyllableGgweolp', [], 'Display', None],
  [ 0xafe3, 'HangulSyllableGgweolh', [], 'Display', None],
  [ 0xafe4, 'HangulSyllableGgweom', [], 'Display', None],
  [ 0xafe5, 'HangulSyllableGgweob', [], 'Display', None],
  [ 0xafe6, 'HangulSyllableGgweobs', [], 'Display', None],
  [ 0xafe7, 'HangulSyllableGgweos', [], 'Display', None],
  [ 0xafe8, 'HangulSyllableGgweoss', [], 'Display', None],
  [ 0xafe9, 'HangulSyllableGgweong', [], 'Display', None],
  [ 0xafea, 'HangulSyllableGgweoj', [], 'Display', None],
  [ 0xafeb, 'HangulSyllableGgweoc', [], 'Display', None],
  [ 0xafec, 'HangulSyllableGgweok', [], 'Display', None],
  [ 0xafed, 'HangulSyllableGgweot', [], 'Display', None],
  [ 0xafee, 'HangulSyllableGgweop', [], 'Display', None],
  [ 0xafef, 'HangulSyllableGgweoh', [], 'Display', None],
  [ 0xaff0, 'HangulSyllableGgwe', [], 'Display', None],
  [ 0xaff1, 'HangulSyllableGgweg', [], 'Display', None],
  [ 0xaff2, 'HangulSyllableGgwegg', [], 'Display', None],
  [ 0xaff3, 'HangulSyllableGgwegs', [], 'Display', None],
  [ 0xaff4, 'HangulSyllableGgwen', [], 'Display', None],
  [ 0xaff5, 'HangulSyllableGgwenj', [], 'Display', None],
  [ 0xaff6, 'HangulSyllableGgwenh', [], 'Display', None],
  [ 0xaff7, 'HangulSyllableGgwed', [], 'Display', None],
  [ 0xaff8, 'HangulSyllableGgwel', [], 'Display', None],
  [ 0xaff9, 'HangulSyllableGgwelg', [], 'Display', None],
  [ 0xaffa, 'HangulSyllableGgwelm', [], 'Display', None],
  [ 0xaffb, 'HangulSyllableGgwelb', [], 'Display', None],
  [ 0xaffc, 'HangulSyllableGgwels', [], 'Display', None],
  [ 0xaffd, 'HangulSyllableGgwelt', [], 'Display', None],
  [ 0xaffe, 'HangulSyllableGgwelp', [], 'Display', None],
  [ 0xafff, 'HangulSyllableGgwelh', [], 'Display', None],
  [ 0xb000, 'HangulSyllableGgwem', [], 'Display', None],
  [ 0xb001, 'HangulSyllableGgweb', [], 'Display', None],
  [ 0xb002, 'HangulSyllableGgwebs', [], 'Display', None],
  [ 0xb003, 'HangulSyllableGgwes', [], 'Display', None],
  [ 0xb004, 'HangulSyllableGgwess', [], 'Display', None],
  [ 0xb005, 'HangulSyllableGgweng', [], 'Display', None],
  [ 0xb006, 'HangulSyllableGgwej', [], 'Display', None],
  [ 0xb007, 'HangulSyllableGgwec', [], 'Display', None],
  [ 0xb008, 'HangulSyllableGgwek', [], 'Display', None],
  [ 0xb009, 'HangulSyllableGgwet', [], 'Display', None],
  [ 0xb00a, 'HangulSyllableGgwep', [], 'Display', None],
  [ 0xb00b, 'HangulSyllableGgweh', [], 'Display', None],
  [ 0xb00c, 'HangulSyllableGgwi', [], 'Display', None],
  [ 0xb00d, 'HangulSyllableGgwig', [], 'Display', None],
  [ 0xb00e, 'HangulSyllableGgwigg', [], 'Display', None],
  [ 0xb00f, 'HangulSyllableGgwigs', [], 'Display', None],
  [ 0xb010, 'HangulSyllableGgwin', [], 'Display', None],
  [ 0xb011, 'HangulSyllableGgwinj', [], 'Display', None],
  [ 0xb012, 'HangulSyllableGgwinh', [], 'Display', None],
  [ 0xb013, 'HangulSyllableGgwid', [], 'Display', None],
  [ 0xb014, 'HangulSyllableGgwil', [], 'Display', None],
  [ 0xb015, 'HangulSyllableGgwilg', [], 'Display', None],
  [ 0xb016, 'HangulSyllableGgwilm', [], 'Display', None],
  [ 0xb017, 'HangulSyllableGgwilb', [], 'Display', None],
  [ 0xb018, 'HangulSyllableGgwils', [], 'Display', None],
  [ 0xb019, 'HangulSyllableGgwilt', [], 'Display', None],
  [ 0xb01a, 'HangulSyllableGgwilp', [], 'Display', None],
  [ 0xb01b, 'HangulSyllableGgwilh', [], 'Display', None],
  [ 0xb01c, 'HangulSyllableGgwim', [], 'Display', None],
  [ 0xb01d, 'HangulSyllableGgwib', [], 'Display', None],
  [ 0xb01e, 'HangulSyllableGgwibs', [], 'Display', None],
  [ 0xb01f, 'HangulSyllableGgwis', [], 'Display', None],
  [ 0xb020, 'HangulSyllableGgwiss', [], 'Display', None],
  [ 0xb021, 'HangulSyllableGgwing', [], 'Display', None],
  [ 0xb022, 'HangulSyllableGgwij', [], 'Display', None],
  [ 0xb023, 'HangulSyllableGgwic', [], 'Display', None],
  [ 0xb024, 'HangulSyllableGgwik', [], 'Display', None],
  [ 0xb025, 'HangulSyllableGgwit', [], 'Display', None],
  [ 0xb026, 'HangulSyllableGgwip', [], 'Display', None],
  [ 0xb027, 'HangulSyllableGgwih', [], 'Display', None],
  [ 0xb028, 'HangulSyllableGgyu', [], 'Display', None],
  [ 0xb029, 'HangulSyllableGgyug', [], 'Display', None],
  [ 0xb02a, 'HangulSyllableGgyugg', [], 'Display', None],
  [ 0xb02b, 'HangulSyllableGgyugs', [], 'Display', None],
  [ 0xb02c, 'HangulSyllableGgyun', [], 'Display', None],
  [ 0xb02d, 'HangulSyllableGgyunj', [], 'Display', None],
  [ 0xb02e, 'HangulSyllableGgyunh', [], 'Display', None],
  [ 0xb02f, 'HangulSyllableGgyud', [], 'Display', None],
  [ 0xb030, 'HangulSyllableGgyul', [], 'Display', None],
  [ 0xb031, 'HangulSyllableGgyulg', [], 'Display', None],
  [ 0xb032, 'HangulSyllableGgyulm', [], 'Display', None],
  [ 0xb033, 'HangulSyllableGgyulb', [], 'Display', None],
  [ 0xb034, 'HangulSyllableGgyuls', [], 'Display', None],
  [ 0xb035, 'HangulSyllableGgyult', [], 'Display', None],
  [ 0xb036, 'HangulSyllableGgyulp', [], 'Display', None],
  [ 0xb037, 'HangulSyllableGgyulh', [], 'Display', None],
  [ 0xb038, 'HangulSyllableGgyum', [], 'Display', None],
  [ 0xb039, 'HangulSyllableGgyub', [], 'Display', None],
  [ 0xb03a, 'HangulSyllableGgyubs', [], 'Display', None],
  [ 0xb03b, 'HangulSyllableGgyus', [], 'Display', None],
  [ 0xb03c, 'HangulSyllableGgyuss', [], 'Display', None],
  [ 0xb03d, 'HangulSyllableGgyung', [], 'Display', None],
  [ 0xb03e, 'HangulSyllableGgyuj', [], 'Display', None],
  [ 0xb03f, 'HangulSyllableGgyuc', [], 'Display', None],
  [ 0xb040, 'HangulSyllableGgyuk', [], 'Display', None],
  [ 0xb041, 'HangulSyllableGgyut', [], 'Display', None],
  [ 0xb042, 'HangulSyllableGgyup', [], 'Display', None],
  [ 0xb043, 'HangulSyllableGgyuh', [], 'Display', None],
  [ 0xb044, 'HangulSyllableGgeu', [], 'Display', None],
  [ 0xb045, 'HangulSyllableGgeug', [], 'Display', None],
  [ 0xb046, 'HangulSyllableGgeugg', [], 'Display', None],
  [ 0xb047, 'HangulSyllableGgeugs', [], 'Display', None],
  [ 0xb048, 'HangulSyllableGgeun', [], 'Display', None],
  [ 0xb049, 'HangulSyllableGgeunj', [], 'Display', None],
  [ 0xb04a, 'HangulSyllableGgeunh', [], 'Display', None],
  [ 0xb04b, 'HangulSyllableGgeud', [], 'Display', None],
  [ 0xb04c, 'HangulSyllableGgeul', [], 'Display', None],
  [ 0xb04d, 'HangulSyllableGgeulg', [], 'Display', None],
  [ 0xb04e, 'HangulSyllableGgeulm', [], 'Display', None],
  [ 0xb04f, 'HangulSyllableGgeulb', [], 'Display', None],
  [ 0xb050, 'HangulSyllableGgeuls', [], 'Display', None],
  [ 0xb051, 'HangulSyllableGgeult', [], 'Display', None],
  [ 0xb052, 'HangulSyllableGgeulp', [], 'Display', None],
  [ 0xb053, 'HangulSyllableGgeulh', [], 'Display', None],
  [ 0xb054, 'HangulSyllableGgeum', [], 'Display', None],
  [ 0xb055, 'HangulSyllableGgeub', [], 'Display', None],
  [ 0xb056, 'HangulSyllableGgeubs', [], 'Display', None],
  [ 0xb057, 'HangulSyllableGgeus', [], 'Display', None],
  [ 0xb058, 'HangulSyllableGgeuss', [], 'Display', None],
  [ 0xb059, 'HangulSyllableGgeung', [], 'Display', None],
  [ 0xb05a, 'HangulSyllableGgeuj', [], 'Display', None],
  [ 0xb05b, 'HangulSyllableGgeuc', [], 'Display', None],
  [ 0xb05c, 'HangulSyllableGgeuk', [], 'Display', None],
  [ 0xb05d, 'HangulSyllableGgeut', [], 'Display', None],
  [ 0xb05e, 'HangulSyllableGgeup', [], 'Display', None],
  [ 0xb05f, 'HangulSyllableGgeuh', [], 'Display', None],
  [ 0xb060, 'HangulSyllableGgyi', [], 'Display', None],
  [ 0xb061, 'HangulSyllableGgyig', [], 'Display', None],
  [ 0xb062, 'HangulSyllableGgyigg', [], 'Display', None],
  [ 0xb063, 'HangulSyllableGgyigs', [], 'Display', None],
  [ 0xb064, 'HangulSyllableGgyin', [], 'Display', None],
  [ 0xb065, 'HangulSyllableGgyinj', [], 'Display', None],
  [ 0xb066, 'HangulSyllableGgyinh', [], 'Display', None],
  [ 0xb067, 'HangulSyllableGgyid', [], 'Display', None],
  [ 0xb068, 'HangulSyllableGgyil', [], 'Display', None],
  [ 0xb069, 'HangulSyllableGgyilg', [], 'Display', None],
  [ 0xb06a, 'HangulSyllableGgyilm', [], 'Display', None],
  [ 0xb06b, 'HangulSyllableGgyilb', [], 'Display', None],
  [ 0xb06c, 'HangulSyllableGgyils', [], 'Display', None],
  [ 0xb06d, 'HangulSyllableGgyilt', [], 'Display', None],
  [ 0xb06e, 'HangulSyllableGgyilp', [], 'Display', None],
  [ 0xb06f, 'HangulSyllableGgyilh', [], 'Display', None],
  [ 0xb070, 'HangulSyllableGgyim', [], 'Display', None],
  [ 0xb071, 'HangulSyllableGgyib', [], 'Display', None],
  [ 0xb072, 'HangulSyllableGgyibs', [], 'Display', None],
  [ 0xb073, 'HangulSyllableGgyis', [], 'Display', None],
  [ 0xb074, 'HangulSyllableGgyiss', [], 'Display', None],
  [ 0xb075, 'HangulSyllableGgying', [], 'Display', None],
  [ 0xb076, 'HangulSyllableGgyij', [], 'Display', None],
  [ 0xb077, 'HangulSyllableGgyic', [], 'Display', None],
  [ 0xb078, 'HangulSyllableGgyik', [], 'Display', None],
  [ 0xb079, 'HangulSyllableGgyit', [], 'Display', None],
  [ 0xb07a, 'HangulSyllableGgyip', [], 'Display', None],
  [ 0xb07b, 'HangulSyllableGgyih', [], 'Display', None],
  [ 0xb07c, 'HangulSyllableGgi', [], 'Display', None],
  [ 0xb07d, 'HangulSyllableGgig', [], 'Display', None],
  [ 0xb07e, 'HangulSyllableGgigg', [], 'Display', None],
  [ 0xb07f, 'HangulSyllableGgigs', [], 'Display', None],
  [ 0xb080, 'HangulSyllableGgin', [], 'Display', None],
  [ 0xb081, 'HangulSyllableGginj', [], 'Display', None],
  [ 0xb082, 'HangulSyllableGginh', [], 'Display', None],
  [ 0xb083, 'HangulSyllableGgid', [], 'Display', None],
  [ 0xb084, 'HangulSyllableGgil', [], 'Display', None],
  [ 0xb085, 'HangulSyllableGgilg', [], 'Display', None],
  [ 0xb086, 'HangulSyllableGgilm', [], 'Display', None],
  [ 0xb087, 'HangulSyllableGgilb', [], 'Display', None],
  [ 0xb088, 'HangulSyllableGgils', [], 'Display', None],
  [ 0xb089, 'HangulSyllableGgilt', [], 'Display', None],
  [ 0xb08a, 'HangulSyllableGgilp', [], 'Display', None],
  [ 0xb08b, 'HangulSyllableGgilh', [], 'Display', None],
  [ 0xb08c, 'HangulSyllableGgim', [], 'Display', None],
  [ 0xb08d, 'HangulSyllableGgib', [], 'Display', None],
  [ 0xb08e, 'HangulSyllableGgibs', [], 'Display', None],
  [ 0xb08f, 'HangulSyllableGgis', [], 'Display', None],
  [ 0xb090, 'HangulSyllableGgiss', [], 'Display', None],
  [ 0xb091, 'HangulSyllableGging', [], 'Display', None],
  [ 0xb092, 'HangulSyllableGgij', [], 'Display', None],
  [ 0xb093, 'HangulSyllableGgic', [], 'Display', None],
  [ 0xb094, 'HangulSyllableGgik', [], 'Display', None],
  [ 0xb095, 'HangulSyllableGgit', [], 'Display', None],
  [ 0xb096, 'HangulSyllableGgip', [], 'Display', None],
  [ 0xb097, 'HangulSyllableGgih', [], 'Display', None],
  [ 0xb098, 'HangulSyllableNa', [], 'Display', None],
  [ 0xb099, 'HangulSyllableNag', [], 'Display', None],
  [ 0xb09a, 'HangulSyllableNagg', [], 'Display', None],
  [ 0xb09b, 'HangulSyllableNags', [], 'Display', None],
  [ 0xb09c, 'HangulSyllableNan', [], 'Display', None],
  [ 0xb09d, 'HangulSyllableNanj', [], 'Display', None],
  [ 0xb09e, 'HangulSyllableNanh', [], 'Display', None],
  [ 0xb09f, 'HangulSyllableNad', [], 'Display', None],
  [ 0xb0a0, 'HangulSyllableNal', [], 'Display', None],
  [ 0xb0a1, 'HangulSyllableNalg', [], 'Display', None],
  [ 0xb0a2, 'HangulSyllableNalm', [], 'Display', None],
  [ 0xb0a3, 'HangulSyllableNalb', [], 'Display', None],
  [ 0xb0a4, 'HangulSyllableNals', [], 'Display', None],
  [ 0xb0a5, 'HangulSyllableNalt', [], 'Display', None],
  [ 0xb0a6, 'HangulSyllableNalp', [], 'Display', None],
  [ 0xb0a7, 'HangulSyllableNalh', [], 'Display', None],
  [ 0xb0a8, 'HangulSyllableNam', [], 'Display', None],
  [ 0xb0a9, 'HangulSyllableNab', [], 'Display', None],
  [ 0xb0aa, 'HangulSyllableNabs', [], 'Display', None],
  [ 0xb0ab, 'HangulSyllableNas', [], 'Display', None],
  [ 0xb0ac, 'HangulSyllableNass', [], 'Display', None],
  [ 0xb0ad, 'HangulSyllableNang', [], 'Display', None],
  [ 0xb0ae, 'HangulSyllableNaj', [], 'Display', None],
  [ 0xb0af, 'HangulSyllableNac', [], 'Display', None],
  [ 0xb0b0, 'HangulSyllableNak', [], 'Display', None],
  [ 0xb0b1, 'HangulSyllableNat', [], 'Display', None],
  [ 0xb0b2, 'HangulSyllableNap', [], 'Display', None],
  [ 0xb0b3, 'HangulSyllableNah', [], 'Display', None],
  [ 0xb0b4, 'HangulSyllableNae', [], 'Display', None],
  [ 0xb0b5, 'HangulSyllableNaeg', [], 'Display', None],
  [ 0xb0b6, 'HangulSyllableNaegg', [], 'Display', None],
  [ 0xb0b7, 'HangulSyllableNaegs', [], 'Display', None],
  [ 0xb0b8, 'HangulSyllableNaen', [], 'Display', None],
  [ 0xb0b9, 'HangulSyllableNaenj', [], 'Display', None],
  [ 0xb0ba, 'HangulSyllableNaenh', [], 'Display', None],
  [ 0xb0bb, 'HangulSyllableNaed', [], 'Display', None],
  [ 0xb0bc, 'HangulSyllableNael', [], 'Display', None],
  [ 0xb0bd, 'HangulSyllableNaelg', [], 'Display', None],
  [ 0xb0be, 'HangulSyllableNaelm', [], 'Display', None],
  [ 0xb0bf, 'HangulSyllableNaelb', [], 'Display', None],
  [ 0xb0c0, 'HangulSyllableNaels', [], 'Display', None],
  [ 0xb0c1, 'HangulSyllableNaelt', [], 'Display', None],
  [ 0xb0c2, 'HangulSyllableNaelp', [], 'Display', None],
  [ 0xb0c3, 'HangulSyllableNaelh', [], 'Display', None],
  [ 0xb0c4, 'HangulSyllableNaem', [], 'Display', None],
  [ 0xb0c5, 'HangulSyllableNaeb', [], 'Display', None],
  [ 0xb0c6, 'HangulSyllableNaebs', [], 'Display', None],
  [ 0xb0c7, 'HangulSyllableNaes', [], 'Display', None],
  [ 0xb0c8, 'HangulSyllableNaess', [], 'Display', None],
  [ 0xb0c9, 'HangulSyllableNaeng', [], 'Display', None],
  [ 0xb0ca, 'HangulSyllableNaej', [], 'Display', None],
  [ 0xb0cb, 'HangulSyllableNaec', [], 'Display', None],
  [ 0xb0cc, 'HangulSyllableNaek', [], 'Display', None],
  [ 0xb0cd, 'HangulSyllableNaet', [], 'Display', None],
  [ 0xb0ce, 'HangulSyllableNaep', [], 'Display', None],
  [ 0xb0cf, 'HangulSyllableNaeh', [], 'Display', None],
  [ 0xb0d0, 'HangulSyllableNya', [], 'Display', None],
  [ 0xb0d1, 'HangulSyllableNyag', [], 'Display', None],
  [ 0xb0d2, 'HangulSyllableNyagg', [], 'Display', None],
  [ 0xb0d3, 'HangulSyllableNyags', [], 'Display', None],
  [ 0xb0d4, 'HangulSyllableNyan', [], 'Display', None],
  [ 0xb0d5, 'HangulSyllableNyanj', [], 'Display', None],
  [ 0xb0d6, 'HangulSyllableNyanh', [], 'Display', None],
  [ 0xb0d7, 'HangulSyllableNyad', [], 'Display', None],
  [ 0xb0d8, 'HangulSyllableNyal', [], 'Display', None],
  [ 0xb0d9, 'HangulSyllableNyalg', [], 'Display', None],
  [ 0xb0da, 'HangulSyllableNyalm', [], 'Display', None],
  [ 0xb0db, 'HangulSyllableNyalb', [], 'Display', None],
  [ 0xb0dc, 'HangulSyllableNyals', [], 'Display', None],
  [ 0xb0dd, 'HangulSyllableNyalt', [], 'Display', None],
  [ 0xb0de, 'HangulSyllableNyalp', [], 'Display', None],
  [ 0xb0df, 'HangulSyllableNyalh', [], 'Display', None],
  [ 0xb0e0, 'HangulSyllableNyam', [], 'Display', None],
  [ 0xb0e1, 'HangulSyllableNyab', [], 'Display', None],
  [ 0xb0e2, 'HangulSyllableNyabs', [], 'Display', None],
  [ 0xb0e3, 'HangulSyllableNyas', [], 'Display', None],
  [ 0xb0e4, 'HangulSyllableNyass', [], 'Display', None],
  [ 0xb0e5, 'HangulSyllableNyang', [], 'Display', None],
  [ 0xb0e6, 'HangulSyllableNyaj', [], 'Display', None],
  [ 0xb0e7, 'HangulSyllableNyac', [], 'Display', None],
  [ 0xb0e8, 'HangulSyllableNyak', [], 'Display', None],
  [ 0xb0e9, 'HangulSyllableNyat', [], 'Display', None],
  [ 0xb0ea, 'HangulSyllableNyap', [], 'Display', None],
  [ 0xb0eb, 'HangulSyllableNyah', [], 'Display', None],
  [ 0xb0ec, 'HangulSyllableNyae', [], 'Display', None],
  [ 0xb0ed, 'HangulSyllableNyaeg', [], 'Display', None],
  [ 0xb0ee, 'HangulSyllableNyaegg', [], 'Display', None],
  [ 0xb0ef, 'HangulSyllableNyaegs', [], 'Display', None],
  [ 0xb0f0, 'HangulSyllableNyaen', [], 'Display', None],
  [ 0xb0f1, 'HangulSyllableNyaenj', [], 'Display', None],
  [ 0xb0f2, 'HangulSyllableNyaenh', [], 'Display', None],
  [ 0xb0f3, 'HangulSyllableNyaed', [], 'Display', None],
  [ 0xb0f4, 'HangulSyllableNyael', [], 'Display', None],
  [ 0xb0f5, 'HangulSyllableNyaelg', [], 'Display', None],
  [ 0xb0f6, 'HangulSyllableNyaelm', [], 'Display', None],
  [ 0xb0f7, 'HangulSyllableNyaelb', [], 'Display', None],
  [ 0xb0f8, 'HangulSyllableNyaels', [], 'Display', None],
  [ 0xb0f9, 'HangulSyllableNyaelt', [], 'Display', None],
  [ 0xb0fa, 'HangulSyllableNyaelp', [], 'Display', None],
  [ 0xb0fb, 'HangulSyllableNyaelh', [], 'Display', None],
  [ 0xb0fc, 'HangulSyllableNyaem', [], 'Display', None],
  [ 0xb0fd, 'HangulSyllableNyaeb', [], 'Display', None],
  [ 0xb0fe, 'HangulSyllableNyaebs', [], 'Display', None],
  [ 0xb0ff, 'HangulSyllableNyaes', [], 'Display', None],
  [ 0xb100, 'HangulSyllableNyaess', [], 'Display', None],
  [ 0xb101, 'HangulSyllableNyaeng', [], 'Display', None],
  [ 0xb102, 'HangulSyllableNyaej', [], 'Display', None],
  [ 0xb103, 'HangulSyllableNyaec', [], 'Display', None],
  [ 0xb104, 'HangulSyllableNyaek', [], 'Display', None],
  [ 0xb105, 'HangulSyllableNyaet', [], 'Display', None],
  [ 0xb106, 'HangulSyllableNyaep', [], 'Display', None],
  [ 0xb107, 'HangulSyllableNyaeh', [], 'Display', None],
  [ 0xb108, 'HangulSyllableNeo', [], 'Display', None],
  [ 0xb109, 'HangulSyllableNeog', [], 'Display', None],
  [ 0xb10a, 'HangulSyllableNeogg', [], 'Display', None],
  [ 0xb10b, 'HangulSyllableNeogs', [], 'Display', None],
  [ 0xb10c, 'HangulSyllableNeon', [], 'Display', None],
  [ 0xb10d, 'HangulSyllableNeonj', [], 'Display', None],
  [ 0xb10e, 'HangulSyllableNeonh', [], 'Display', None],
  [ 0xb10f, 'HangulSyllableNeod', [], 'Display', None],
  [ 0xb110, 'HangulSyllableNeol', [], 'Display', None],
  [ 0xb111, 'HangulSyllableNeolg', [], 'Display', None],
  [ 0xb112, 'HangulSyllableNeolm', [], 'Display', None],
  [ 0xb113, 'HangulSyllableNeolb', [], 'Display', None],
  [ 0xb114, 'HangulSyllableNeols', [], 'Display', None],
  [ 0xb115, 'HangulSyllableNeolt', [], 'Display', None],
  [ 0xb116, 'HangulSyllableNeolp', [], 'Display', None],
  [ 0xb117, 'HangulSyllableNeolh', [], 'Display', None],
  [ 0xb118, 'HangulSyllableNeom', [], 'Display', None],
  [ 0xb119, 'HangulSyllableNeob', [], 'Display', None],
  [ 0xb11a, 'HangulSyllableNeobs', [], 'Display', None],
  [ 0xb11b, 'HangulSyllableNeos', [], 'Display', None],
  [ 0xb11c, 'HangulSyllableNeoss', [], 'Display', None],
  [ 0xb11d, 'HangulSyllableNeong', [], 'Display', None],
  [ 0xb11e, 'HangulSyllableNeoj', [], 'Display', None],
  [ 0xb11f, 'HangulSyllableNeoc', [], 'Display', None],
  [ 0xb120, 'HangulSyllableNeok', [], 'Display', None],
  [ 0xb121, 'HangulSyllableNeot', [], 'Display', None],
  [ 0xb122, 'HangulSyllableNeop', [], 'Display', None],
  [ 0xb123, 'HangulSyllableNeoh', [], 'Display', None],
  [ 0xb124, 'HangulSyllableNe', [], 'Display', None],
  [ 0xb125, 'HangulSyllableNeg', [], 'Display', None],
  [ 0xb126, 'HangulSyllableNegg', [], 'Display', None],
  [ 0xb127, 'HangulSyllableNegs', [], 'Display', None],
  [ 0xb128, 'HangulSyllableNen', [], 'Display', None],
  [ 0xb129, 'HangulSyllableNenj', [], 'Display', None],
  [ 0xb12a, 'HangulSyllableNenh', [], 'Display', None],
  [ 0xb12b, 'HangulSyllableNed', [], 'Display', None],
  [ 0xb12c, 'HangulSyllableNel', [], 'Display', None],
  [ 0xb12d, 'HangulSyllableNelg', [], 'Display', None],
  [ 0xb12e, 'HangulSyllableNelm', [], 'Display', None],
  [ 0xb12f, 'HangulSyllableNelb', [], 'Display', None],
  [ 0xb130, 'HangulSyllableNels', [], 'Display', None],
  [ 0xb131, 'HangulSyllableNelt', [], 'Display', None],
  [ 0xb132, 'HangulSyllableNelp', [], 'Display', None],
  [ 0xb133, 'HangulSyllableNelh', [], 'Display', None],
  [ 0xb134, 'HangulSyllableNem', [], 'Display', None],
  [ 0xb135, 'HangulSyllableNeb', [], 'Display', None],
  [ 0xb136, 'HangulSyllableNebs', [], 'Display', None],
  [ 0xb137, 'HangulSyllableNes', [], 'Display', None],
  [ 0xb138, 'HangulSyllableNess', [], 'Display', None],
  [ 0xb139, 'HangulSyllableNeng', [], 'Display', None],
  [ 0xb13a, 'HangulSyllableNej', [], 'Display', None],
  [ 0xb13b, 'HangulSyllableNec', [], 'Display', None],
  [ 0xb13c, 'HangulSyllableNek', [], 'Display', None],
  [ 0xb13d, 'HangulSyllableNet', [], 'Display', None],
  [ 0xb13e, 'HangulSyllableNep', [], 'Display', None],
  [ 0xb13f, 'HangulSyllableNeh', [], 'Display', None],
  [ 0xb140, 'HangulSyllableNyeo', [], 'Display', None],
  [ 0xb141, 'HangulSyllableNyeog', [], 'Display', None],
  [ 0xb142, 'HangulSyllableNyeogg', [], 'Display', None],
  [ 0xb143, 'HangulSyllableNyeogs', [], 'Display', None],
  [ 0xb144, 'HangulSyllableNyeon', [], 'Display', None],
  [ 0xb145, 'HangulSyllableNyeonj', [], 'Display', None],
  [ 0xb146, 'HangulSyllableNyeonh', [], 'Display', None],
  [ 0xb147, 'HangulSyllableNyeod', [], 'Display', None],
  [ 0xb148, 'HangulSyllableNyeol', [], 'Display', None],
  [ 0xb149, 'HangulSyllableNyeolg', [], 'Display', None],
  [ 0xb14a, 'HangulSyllableNyeolm', [], 'Display', None],
  [ 0xb14b, 'HangulSyllableNyeolb', [], 'Display', None],
  [ 0xb14c, 'HangulSyllableNyeols', [], 'Display', None],
  [ 0xb14d, 'HangulSyllableNyeolt', [], 'Display', None],
  [ 0xb14e, 'HangulSyllableNyeolp', [], 'Display', None],
  [ 0xb14f, 'HangulSyllableNyeolh', [], 'Display', None],
  [ 0xb150, 'HangulSyllableNyeom', [], 'Display', None],
  [ 0xb151, 'HangulSyllableNyeob', [], 'Display', None],
  [ 0xb152, 'HangulSyllableNyeobs', [], 'Display', None],
  [ 0xb153, 'HangulSyllableNyeos', [], 'Display', None],
  [ 0xb154, 'HangulSyllableNyeoss', [], 'Display', None],
  [ 0xb155, 'HangulSyllableNyeong', [], 'Display', None],
  [ 0xb156, 'HangulSyllableNyeoj', [], 'Display', None],
  [ 0xb157, 'HangulSyllableNyeoc', [], 'Display', None],
  [ 0xb158, 'HangulSyllableNyeok', [], 'Display', None],
  [ 0xb159, 'HangulSyllableNyeot', [], 'Display', None],
  [ 0xb15a, 'HangulSyllableNyeop', [], 'Display', None],
  [ 0xb15b, 'HangulSyllableNyeoh', [], 'Display', None],
  [ 0xb15c, 'HangulSyllableNye', [], 'Display', None],
  [ 0xb15d, 'HangulSyllableNyeg', [], 'Display', None],
  [ 0xb15e, 'HangulSyllableNyegg', [], 'Display', None],
  [ 0xb15f, 'HangulSyllableNyegs', [], 'Display', None],
  [ 0xb160, 'HangulSyllableNyen', [], 'Display', None],
  [ 0xb161, 'HangulSyllableNyenj', [], 'Display', None],
  [ 0xb162, 'HangulSyllableNyenh', [], 'Display', None],
  [ 0xb163, 'HangulSyllableNyed', [], 'Display', None],
  [ 0xb164, 'HangulSyllableNyel', [], 'Display', None],
  [ 0xb165, 'HangulSyllableNyelg', [], 'Display', None],
  [ 0xb166, 'HangulSyllableNyelm', [], 'Display', None],
  [ 0xb167, 'HangulSyllableNyelb', [], 'Display', None],
  [ 0xb168, 'HangulSyllableNyels', [], 'Display', None],
  [ 0xb169, 'HangulSyllableNyelt', [], 'Display', None],
  [ 0xb16a, 'HangulSyllableNyelp', [], 'Display', None],
  [ 0xb16b, 'HangulSyllableNyelh', [], 'Display', None],
  [ 0xb16c, 'HangulSyllableNyem', [], 'Display', None],
  [ 0xb16d, 'HangulSyllableNyeb', [], 'Display', None],
  [ 0xb16e, 'HangulSyllableNyebs', [], 'Display', None],
  [ 0xb16f, 'HangulSyllableNyes', [], 'Display', None],
  [ 0xb170, 'HangulSyllableNyess', [], 'Display', None],
  [ 0xb171, 'HangulSyllableNyeng', [], 'Display', None],
  [ 0xb172, 'HangulSyllableNyej', [], 'Display', None],
  [ 0xb173, 'HangulSyllableNyec', [], 'Display', None],
  [ 0xb174, 'HangulSyllableNyek', [], 'Display', None],
  [ 0xb175, 'HangulSyllableNyet', [], 'Display', None],
  [ 0xb176, 'HangulSyllableNyep', [], 'Display', None],
  [ 0xb177, 'HangulSyllableNyeh', [], 'Display', None],
  [ 0xb178, 'HangulSyllableNo', [], 'Display', None],
  [ 0xb179, 'HangulSyllableNog', [], 'Display', None],
  [ 0xb17a, 'HangulSyllableNogg', [], 'Display', None],
  [ 0xb17b, 'HangulSyllableNogs', [], 'Display', None],
  [ 0xb17c, 'HangulSyllableNon', [], 'Display', None],
  [ 0xb17d, 'HangulSyllableNonj', [], 'Display', None],
  [ 0xb17e, 'HangulSyllableNonh', [], 'Display', None],
  [ 0xb17f, 'HangulSyllableNod', [], 'Display', None],
  [ 0xb180, 'HangulSyllableNol', [], 'Display', None],
  [ 0xb181, 'HangulSyllableNolg', [], 'Display', None],
  [ 0xb182, 'HangulSyllableNolm', [], 'Display', None],
  [ 0xb183, 'HangulSyllableNolb', [], 'Display', None],
  [ 0xb184, 'HangulSyllableNols', [], 'Display', None],
  [ 0xb185, 'HangulSyllableNolt', [], 'Display', None],
  [ 0xb186, 'HangulSyllableNolp', [], 'Display', None],
  [ 0xb187, 'HangulSyllableNolh', [], 'Display', None],
  [ 0xb188, 'HangulSyllableNom', [], 'Display', None],
  [ 0xb189, 'HangulSyllableNob', [], 'Display', None],
  [ 0xb18a, 'HangulSyllableNobs', [], 'Display', None],
  [ 0xb18b, 'HangulSyllableNos', [], 'Display', None],
  [ 0xb18c, 'HangulSyllableNoss', [], 'Display', None],
  [ 0xb18d, 'HangulSyllableNong', [], 'Display', None],
  [ 0xb18e, 'HangulSyllableNoj', [], 'Display', None],
  [ 0xb18f, 'HangulSyllableNoc', [], 'Display', None],
  [ 0xb190, 'HangulSyllableNok', [], 'Display', None],
  [ 0xb191, 'HangulSyllableNot', [], 'Display', None],
  [ 0xb192, 'HangulSyllableNop', [], 'Display', None],
  [ 0xb193, 'HangulSyllableNoh', [], 'Display', None],
  [ 0xb194, 'HangulSyllableNwa', [], 'Display', None],
  [ 0xb195, 'HangulSyllableNwag', [], 'Display', None],
  [ 0xb196, 'HangulSyllableNwagg', [], 'Display', None],
  [ 0xb197, 'HangulSyllableNwags', [], 'Display', None],
  [ 0xb198, 'HangulSyllableNwan', [], 'Display', None],
  [ 0xb199, 'HangulSyllableNwanj', [], 'Display', None],
  [ 0xb19a, 'HangulSyllableNwanh', [], 'Display', None],
  [ 0xb19b, 'HangulSyllableNwad', [], 'Display', None],
  [ 0xb19c, 'HangulSyllableNwal', [], 'Display', None],
  [ 0xb19d, 'HangulSyllableNwalg', [], 'Display', None],
  [ 0xb19e, 'HangulSyllableNwalm', [], 'Display', None],
  [ 0xb19f, 'HangulSyllableNwalb', [], 'Display', None],
  [ 0xb1a0, 'HangulSyllableNwals', [], 'Display', None],
  [ 0xb1a1, 'HangulSyllableNwalt', [], 'Display', None],
  [ 0xb1a2, 'HangulSyllableNwalp', [], 'Display', None],
  [ 0xb1a3, 'HangulSyllableNwalh', [], 'Display', None],
  [ 0xb1a4, 'HangulSyllableNwam', [], 'Display', None],
  [ 0xb1a5, 'HangulSyllableNwab', [], 'Display', None],
  [ 0xb1a6, 'HangulSyllableNwabs', [], 'Display', None],
  [ 0xb1a7, 'HangulSyllableNwas', [], 'Display', None],
  [ 0xb1a8, 'HangulSyllableNwass', [], 'Display', None],
  [ 0xb1a9, 'HangulSyllableNwang', [], 'Display', None],
  [ 0xb1aa, 'HangulSyllableNwaj', [], 'Display', None],
  [ 0xb1ab, 'HangulSyllableNwac', [], 'Display', None],
  [ 0xb1ac, 'HangulSyllableNwak', [], 'Display', None],
  [ 0xb1ad, 'HangulSyllableNwat', [], 'Display', None],
  [ 0xb1ae, 'HangulSyllableNwap', [], 'Display', None],
  [ 0xb1af, 'HangulSyllableNwah', [], 'Display', None],
  [ 0xb1b0, 'HangulSyllableNwae', [], 'Display', None],
  [ 0xb1b1, 'HangulSyllableNwaeg', [], 'Display', None],
  [ 0xb1b2, 'HangulSyllableNwaegg', [], 'Display', None],
  [ 0xb1b3, 'HangulSyllableNwaegs', [], 'Display', None],
  [ 0xb1b4, 'HangulSyllableNwaen', [], 'Display', None],
  [ 0xb1b5, 'HangulSyllableNwaenj', [], 'Display', None],
  [ 0xb1b6, 'HangulSyllableNwaenh', [], 'Display', None],
  [ 0xb1b7, 'HangulSyllableNwaed', [], 'Display', None],
  [ 0xb1b8, 'HangulSyllableNwael', [], 'Display', None],
  [ 0xb1b9, 'HangulSyllableNwaelg', [], 'Display', None],
  [ 0xb1ba, 'HangulSyllableNwaelm', [], 'Display', None],
  [ 0xb1bb, 'HangulSyllableNwaelb', [], 'Display', None],
  [ 0xb1bc, 'HangulSyllableNwaels', [], 'Display', None],
  [ 0xb1bd, 'HangulSyllableNwaelt', [], 'Display', None],
  [ 0xb1be, 'HangulSyllableNwaelp', [], 'Display', None],
  [ 0xb1bf, 'HangulSyllableNwaelh', [], 'Display', None],
  [ 0xb1c0, 'HangulSyllableNwaem', [], 'Display', None],
  [ 0xb1c1, 'HangulSyllableNwaeb', [], 'Display', None],
  [ 0xb1c2, 'HangulSyllableNwaebs', [], 'Display', None],
  [ 0xb1c3, 'HangulSyllableNwaes', [], 'Display', None],
  [ 0xb1c4, 'HangulSyllableNwaess', [], 'Display', None],
  [ 0xb1c5, 'HangulSyllableNwaeng', [], 'Display', None],
  [ 0xb1c6, 'HangulSyllableNwaej', [], 'Display', None],
  [ 0xb1c7, 'HangulSyllableNwaec', [], 'Display', None],
  [ 0xb1c8, 'HangulSyllableNwaek', [], 'Display', None],
  [ 0xb1c9, 'HangulSyllableNwaet', [], 'Display', None],
  [ 0xb1ca, 'HangulSyllableNwaep', [], 'Display', None],
  [ 0xb1cb, 'HangulSyllableNwaeh', [], 'Display', None],
  [ 0xb1cc, 'HangulSyllableNoe', [], 'Display', None],
  [ 0xb1cd, 'HangulSyllableNoeg', [], 'Display', None],
  [ 0xb1ce, 'HangulSyllableNoegg', [], 'Display', None],
  [ 0xb1cf, 'HangulSyllableNoegs', [], 'Display', None],
  [ 0xb1d0, 'HangulSyllableNoen', [], 'Display', None],
  [ 0xb1d1, 'HangulSyllableNoenj', [], 'Display', None],
  [ 0xb1d2, 'HangulSyllableNoenh', [], 'Display', None],
  [ 0xb1d3, 'HangulSyllableNoed', [], 'Display', None],
  [ 0xb1d4, 'HangulSyllableNoel', [], 'Display', None],
  [ 0xb1d5, 'HangulSyllableNoelg', [], 'Display', None],
  [ 0xb1d6, 'HangulSyllableNoelm', [], 'Display', None],
  [ 0xb1d7, 'HangulSyllableNoelb', [], 'Display', None],
  [ 0xb1d8, 'HangulSyllableNoels', [], 'Display', None],
  [ 0xb1d9, 'HangulSyllableNoelt', [], 'Display', None],
  [ 0xb1da, 'HangulSyllableNoelp', [], 'Display', None],
  [ 0xb1db, 'HangulSyllableNoelh', [], 'Display', None],
  [ 0xb1dc, 'HangulSyllableNoem', [], 'Display', None],
  [ 0xb1dd, 'HangulSyllableNoeb', [], 'Display', None],
  [ 0xb1de, 'HangulSyllableNoebs', [], 'Display', None],
  [ 0xb1df, 'HangulSyllableNoes', [], 'Display', None],
  [ 0xb1e0, 'HangulSyllableNoess', [], 'Display', None],
  [ 0xb1e1, 'HangulSyllableNoeng', [], 'Display', None],
  [ 0xb1e2, 'HangulSyllableNoej', [], 'Display', None],
  [ 0xb1e3, 'HangulSyllableNoec', [], 'Display', None],
  [ 0xb1e4, 'HangulSyllableNoek', [], 'Display', None],
  [ 0xb1e5, 'HangulSyllableNoet', [], 'Display', None],
  [ 0xb1e6, 'HangulSyllableNoep', [], 'Display', None],
  [ 0xb1e7, 'HangulSyllableNoeh', [], 'Display', None],
  [ 0xb1e8, 'HangulSyllableNyo', [], 'Display', None],
  [ 0xb1e9, 'HangulSyllableNyog', [], 'Display', None],
  [ 0xb1ea, 'HangulSyllableNyogg', [], 'Display', None],
  [ 0xb1eb, 'HangulSyllableNyogs', [], 'Display', None],
  [ 0xb1ec, 'HangulSyllableNyon', [], 'Display', None],
  [ 0xb1ed, 'HangulSyllableNyonj', [], 'Display', None],
  [ 0xb1ee, 'HangulSyllableNyonh', [], 'Display', None],
  [ 0xb1ef, 'HangulSyllableNyod', [], 'Display', None],
  [ 0xb1f0, 'HangulSyllableNyol', [], 'Display', None],
  [ 0xb1f1, 'HangulSyllableNyolg', [], 'Display', None],
  [ 0xb1f2, 'HangulSyllableNyolm', [], 'Display', None],
  [ 0xb1f3, 'HangulSyllableNyolb', [], 'Display', None],
  [ 0xb1f4, 'HangulSyllableNyols', [], 'Display', None],
  [ 0xb1f5, 'HangulSyllableNyolt', [], 'Display', None],
  [ 0xb1f6, 'HangulSyllableNyolp', [], 'Display', None],
  [ 0xb1f7, 'HangulSyllableNyolh', [], 'Display', None],
  [ 0xb1f8, 'HangulSyllableNyom', [], 'Display', None],
  [ 0xb1f9, 'HangulSyllableNyob', [], 'Display', None],
  [ 0xb1fa, 'HangulSyllableNyobs', [], 'Display', None],
  [ 0xb1fb, 'HangulSyllableNyos', [], 'Display', None],
  [ 0xb1fc, 'HangulSyllableNyoss', [], 'Display', None],
  [ 0xb1fd, 'HangulSyllableNyong', [], 'Display', None],
  [ 0xb1fe, 'HangulSyllableNyoj', [], 'Display', None],
  [ 0xb1ff, 'HangulSyllableNyoc', [], 'Display', None],
  [ 0xb200, 'HangulSyllableNyok', [], 'Display', None],
  [ 0xb201, 'HangulSyllableNyot', [], 'Display', None],
  [ 0xb202, 'HangulSyllableNyop', [], 'Display', None],
  [ 0xb203, 'HangulSyllableNyoh', [], 'Display', None],
  [ 0xb204, 'HangulSyllableNu', [], 'Display', None],
  [ 0xb205, 'HangulSyllableNug', [], 'Display', None],
  [ 0xb206, 'HangulSyllableNugg', [], 'Display', None],
  [ 0xb207, 'HangulSyllableNugs', [], 'Display', None],
  [ 0xb208, 'HangulSyllableNun', [], 'Display', None],
  [ 0xb209, 'HangulSyllableNunj', [], 'Display', None],
  [ 0xb20a, 'HangulSyllableNunh', [], 'Display', None],
  [ 0xb20b, 'HangulSyllableNud', [], 'Display', None],
  [ 0xb20c, 'HangulSyllableNul', [], 'Display', None],
  [ 0xb20d, 'HangulSyllableNulg', [], 'Display', None],
  [ 0xb20e, 'HangulSyllableNulm', [], 'Display', None],
  [ 0xb20f, 'HangulSyllableNulb', [], 'Display', None],
  [ 0xb210, 'HangulSyllableNuls', [], 'Display', None],
  [ 0xb211, 'HangulSyllableNult', [], 'Display', None],
  [ 0xb212, 'HangulSyllableNulp', [], 'Display', None],
  [ 0xb213, 'HangulSyllableNulh', [], 'Display', None],
  [ 0xb214, 'HangulSyllableNum', [], 'Display', None],
  [ 0xb215, 'HangulSyllableNub', [], 'Display', None],
  [ 0xb216, 'HangulSyllableNubs', [], 'Display', None],
  [ 0xb217, 'HangulSyllableNus', [], 'Display', None],
  [ 0xb218, 'HangulSyllableNuss', [], 'Display', None],
  [ 0xb219, 'HangulSyllableNung', [], 'Display', None],
  [ 0xb21a, 'HangulSyllableNuj', [], 'Display', None],
  [ 0xb21b, 'HangulSyllableNuc', [], 'Display', None],
  [ 0xb21c, 'HangulSyllableNuk', [], 'Display', None],
  [ 0xb21d, 'HangulSyllableNut', [], 'Display', None],
  [ 0xb21e, 'HangulSyllableNup', [], 'Display', None],
  [ 0xb21f, 'HangulSyllableNuh', [], 'Display', None],
  [ 0xb220, 'HangulSyllableNweo', [], 'Display', None],
  [ 0xb221, 'HangulSyllableNweog', [], 'Display', None],
  [ 0xb222, 'HangulSyllableNweogg', [], 'Display', None],
  [ 0xb223, 'HangulSyllableNweogs', [], 'Display', None],
  [ 0xb224, 'HangulSyllableNweon', [], 'Display', None],
  [ 0xb225, 'HangulSyllableNweonj', [], 'Display', None],
  [ 0xb226, 'HangulSyllableNweonh', [], 'Display', None],
  [ 0xb227, 'HangulSyllableNweod', [], 'Display', None],
  [ 0xb228, 'HangulSyllableNweol', [], 'Display', None],
  [ 0xb229, 'HangulSyllableNweolg', [], 'Display', None],
  [ 0xb22a, 'HangulSyllableNweolm', [], 'Display', None],
  [ 0xb22b, 'HangulSyllableNweolb', [], 'Display', None],
  [ 0xb22c, 'HangulSyllableNweols', [], 'Display', None],
  [ 0xb22d, 'HangulSyllableNweolt', [], 'Display', None],
  [ 0xb22e, 'HangulSyllableNweolp', [], 'Display', None],
  [ 0xb22f, 'HangulSyllableNweolh', [], 'Display', None],
  [ 0xb230, 'HangulSyllableNweom', [], 'Display', None],
  [ 0xb231, 'HangulSyllableNweob', [], 'Display', None],
  [ 0xb232, 'HangulSyllableNweobs', [], 'Display', None],
  [ 0xb233, 'HangulSyllableNweos', [], 'Display', None],
  [ 0xb234, 'HangulSyllableNweoss', [], 'Display', None],
  [ 0xb235, 'HangulSyllableNweong', [], 'Display', None],
  [ 0xb236, 'HangulSyllableNweoj', [], 'Display', None],
  [ 0xb237, 'HangulSyllableNweoc', [], 'Display', None],
  [ 0xb238, 'HangulSyllableNweok', [], 'Display', None],
  [ 0xb239, 'HangulSyllableNweot', [], 'Display', None],
  [ 0xb23a, 'HangulSyllableNweop', [], 'Display', None],
  [ 0xb23b, 'HangulSyllableNweoh', [], 'Display', None],
  [ 0xb23c, 'HangulSyllableNwe', [], 'Display', None],
  [ 0xb23d, 'HangulSyllableNweg', [], 'Display', None],
  [ 0xb23e, 'HangulSyllableNwegg', [], 'Display', None],
  [ 0xb23f, 'HangulSyllableNwegs', [], 'Display', None],
  [ 0xb240, 'HangulSyllableNwen', [], 'Display', None],
  [ 0xb241, 'HangulSyllableNwenj', [], 'Display', None],
  [ 0xb242, 'HangulSyllableNwenh', [], 'Display', None],
  [ 0xb243, 'HangulSyllableNwed', [], 'Display', None],
  [ 0xb244, 'HangulSyllableNwel', [], 'Display', None],
  [ 0xb245, 'HangulSyllableNwelg', [], 'Display', None],
  [ 0xb246, 'HangulSyllableNwelm', [], 'Display', None],
  [ 0xb247, 'HangulSyllableNwelb', [], 'Display', None],
  [ 0xb248, 'HangulSyllableNwels', [], 'Display', None],
  [ 0xb249, 'HangulSyllableNwelt', [], 'Display', None],
  [ 0xb24a, 'HangulSyllableNwelp', [], 'Display', None],
  [ 0xb24b, 'HangulSyllableNwelh', [], 'Display', None],
  [ 0xb24c, 'HangulSyllableNwem', [], 'Display', None],
  [ 0xb24d, 'HangulSyllableNweb', [], 'Display', None],
  [ 0xb24e, 'HangulSyllableNwebs', [], 'Display', None],
  [ 0xb24f, 'HangulSyllableNwes', [], 'Display', None],
  [ 0xb250, 'HangulSyllableNwess', [], 'Display', None],
  [ 0xb251, 'HangulSyllableNweng', [], 'Display', None],
  [ 0xb252, 'HangulSyllableNwej', [], 'Display', None],
  [ 0xb253, 'HangulSyllableNwec', [], 'Display', None],
  [ 0xb254, 'HangulSyllableNwek', [], 'Display', None],
  [ 0xb255, 'HangulSyllableNwet', [], 'Display', None],
  [ 0xb256, 'HangulSyllableNwep', [], 'Display', None],
  [ 0xb257, 'HangulSyllableNweh', [], 'Display', None],
  [ 0xb258, 'HangulSyllableNwi', [], 'Display', None],
  [ 0xb259, 'HangulSyllableNwig', [], 'Display', None],
  [ 0xb25a, 'HangulSyllableNwigg', [], 'Display', None],
  [ 0xb25b, 'HangulSyllableNwigs', [], 'Display', None],
  [ 0xb25c, 'HangulSyllableNwin', [], 'Display', None],
  [ 0xb25d, 'HangulSyllableNwinj', [], 'Display', None],
  [ 0xb25e, 'HangulSyllableNwinh', [], 'Display', None],
  [ 0xb25f, 'HangulSyllableNwid', [], 'Display', None],
  [ 0xb260, 'HangulSyllableNwil', [], 'Display', None],
  [ 0xb261, 'HangulSyllableNwilg', [], 'Display', None],
  [ 0xb262, 'HangulSyllableNwilm', [], 'Display', None],
  [ 0xb263, 'HangulSyllableNwilb', [], 'Display', None],
  [ 0xb264, 'HangulSyllableNwils', [], 'Display', None],
  [ 0xb265, 'HangulSyllableNwilt', [], 'Display', None],
  [ 0xb266, 'HangulSyllableNwilp', [], 'Display', None],
  [ 0xb267, 'HangulSyllableNwilh', [], 'Display', None],
  [ 0xb268, 'HangulSyllableNwim', [], 'Display', None],
  [ 0xb269, 'HangulSyllableNwib', [], 'Display', None],
  [ 0xb26a, 'HangulSyllableNwibs', [], 'Display', None],
  [ 0xb26b, 'HangulSyllableNwis', [], 'Display', None],
  [ 0xb26c, 'HangulSyllableNwiss', [], 'Display', None],
  [ 0xb26d, 'HangulSyllableNwing', [], 'Display', None],
  [ 0xb26e, 'HangulSyllableNwij', [], 'Display', None],
  [ 0xb26f, 'HangulSyllableNwic', [], 'Display', None],
  [ 0xb270, 'HangulSyllableNwik', [], 'Display', None],
  [ 0xb271, 'HangulSyllableNwit', [], 'Display', None],
  [ 0xb272, 'HangulSyllableNwip', [], 'Display', None],
  [ 0xb273, 'HangulSyllableNwih', [], 'Display', None],
  [ 0xb274, 'HangulSyllableNyu', [], 'Display', None],
  [ 0xb275, 'HangulSyllableNyug', [], 'Display', None],
  [ 0xb276, 'HangulSyllableNyugg', [], 'Display', None],
  [ 0xb277, 'HangulSyllableNyugs', [], 'Display', None],
  [ 0xb278, 'HangulSyllableNyun', [], 'Display', None],
  [ 0xb279, 'HangulSyllableNyunj', [], 'Display', None],
  [ 0xb27a, 'HangulSyllableNyunh', [], 'Display', None],
  [ 0xb27b, 'HangulSyllableNyud', [], 'Display', None],
  [ 0xb27c, 'HangulSyllableNyul', [], 'Display', None],
  [ 0xb27d, 'HangulSyllableNyulg', [], 'Display', None],
  [ 0xb27e, 'HangulSyllableNyulm', [], 'Display', None],
  [ 0xb27f, 'HangulSyllableNyulb', [], 'Display', None],
  [ 0xb280, 'HangulSyllableNyuls', [], 'Display', None],
  [ 0xb281, 'HangulSyllableNyult', [], 'Display', None],
  [ 0xb282, 'HangulSyllableNyulp', [], 'Display', None],
  [ 0xb283, 'HangulSyllableNyulh', [], 'Display', None],
  [ 0xb284, 'HangulSyllableNyum', [], 'Display', None],
  [ 0xb285, 'HangulSyllableNyub', [], 'Display', None],
  [ 0xb286, 'HangulSyllableNyubs', [], 'Display', None],
  [ 0xb287, 'HangulSyllableNyus', [], 'Display', None],
  [ 0xb288, 'HangulSyllableNyuss', [], 'Display', None],
  [ 0xb289, 'HangulSyllableNyung', [], 'Display', None],
  [ 0xb28a, 'HangulSyllableNyuj', [], 'Display', None],
  [ 0xb28b, 'HangulSyllableNyuc', [], 'Display', None],
  [ 0xb28c, 'HangulSyllableNyuk', [], 'Display', None],
  [ 0xb28d, 'HangulSyllableNyut', [], 'Display', None],
  [ 0xb28e, 'HangulSyllableNyup', [], 'Display', None],
  [ 0xb28f, 'HangulSyllableNyuh', [], 'Display', None],
  [ 0xb290, 'HangulSyllableNeu', [], 'Display', None],
  [ 0xb291, 'HangulSyllableNeug', [], 'Display', None],
  [ 0xb292, 'HangulSyllableNeugg', [], 'Display', None],
  [ 0xb293, 'HangulSyllableNeugs', [], 'Display', None],
  [ 0xb294, 'HangulSyllableNeun', [], 'Display', None],
  [ 0xb295, 'HangulSyllableNeunj', [], 'Display', None],
  [ 0xb296, 'HangulSyllableNeunh', [], 'Display', None],
  [ 0xb297, 'HangulSyllableNeud', [], 'Display', None],
  [ 0xb298, 'HangulSyllableNeul', [], 'Display', None],
  [ 0xb299, 'HangulSyllableNeulg', [], 'Display', None],
  [ 0xb29a, 'HangulSyllableNeulm', [], 'Display', None],
  [ 0xb29b, 'HangulSyllableNeulb', [], 'Display', None],
  [ 0xb29c, 'HangulSyllableNeuls', [], 'Display', None],
  [ 0xb29d, 'HangulSyllableNeult', [], 'Display', None],
  [ 0xb29e, 'HangulSyllableNeulp', [], 'Display', None],
  [ 0xb29f, 'HangulSyllableNeulh', [], 'Display', None],
  [ 0xb2a0, 'HangulSyllableNeum', [], 'Display', None],
  [ 0xb2a1, 'HangulSyllableNeub', [], 'Display', None],
  [ 0xb2a2, 'HangulSyllableNeubs', [], 'Display', None],
  [ 0xb2a3, 'HangulSyllableNeus', [], 'Display', None],
  [ 0xb2a4, 'HangulSyllableNeuss', [], 'Display', None],
  [ 0xb2a5, 'HangulSyllableNeung', [], 'Display', None],
  [ 0xb2a6, 'HangulSyllableNeuj', [], 'Display', None],
  [ 0xb2a7, 'HangulSyllableNeuc', [], 'Display', None],
  [ 0xb2a8, 'HangulSyllableNeuk', [], 'Display', None],
  [ 0xb2a9, 'HangulSyllableNeut', [], 'Display', None],
  [ 0xb2aa, 'HangulSyllableNeup', [], 'Display', None],
  [ 0xb2ab, 'HangulSyllableNeuh', [], 'Display', None],
  [ 0xb2ac, 'HangulSyllableNyi', [], 'Display', None],
  [ 0xb2ad, 'HangulSyllableNyig', [], 'Display', None],
  [ 0xb2ae, 'HangulSyllableNyigg', [], 'Display', None],
  [ 0xb2af, 'HangulSyllableNyigs', [], 'Display', None],
  [ 0xb2b0, 'HangulSyllableNyin', [], 'Display', None],
  [ 0xb2b1, 'HangulSyllableNyinj', [], 'Display', None],
  [ 0xb2b2, 'HangulSyllableNyinh', [], 'Display', None],
  [ 0xb2b3, 'HangulSyllableNyid', [], 'Display', None],
  [ 0xb2b4, 'HangulSyllableNyil', [], 'Display', None],
  [ 0xb2b5, 'HangulSyllableNyilg', [], 'Display', None],
  [ 0xb2b6, 'HangulSyllableNyilm', [], 'Display', None],
  [ 0xb2b7, 'HangulSyllableNyilb', [], 'Display', None],
  [ 0xb2b8, 'HangulSyllableNyils', [], 'Display', None],
  [ 0xb2b9, 'HangulSyllableNyilt', [], 'Display', None],
  [ 0xb2ba, 'HangulSyllableNyilp', [], 'Display', None],
  [ 0xb2bb, 'HangulSyllableNyilh', [], 'Display', None],
  [ 0xb2bc, 'HangulSyllableNyim', [], 'Display', None],
  [ 0xb2bd, 'HangulSyllableNyib', [], 'Display', None],
  [ 0xb2be, 'HangulSyllableNyibs', [], 'Display', None],
  [ 0xb2bf, 'HangulSyllableNyis', [], 'Display', None],
  [ 0xb2c0, 'HangulSyllableNyiss', [], 'Display', None],
  [ 0xb2c1, 'HangulSyllableNying', [], 'Display', None],
  [ 0xb2c2, 'HangulSyllableNyij', [], 'Display', None],
  [ 0xb2c3, 'HangulSyllableNyic', [], 'Display', None],
  [ 0xb2c4, 'HangulSyllableNyik', [], 'Display', None],
  [ 0xb2c5, 'HangulSyllableNyit', [], 'Display', None],
  [ 0xb2c6, 'HangulSyllableNyip', [], 'Display', None],
  [ 0xb2c7, 'HangulSyllableNyih', [], 'Display', None],
  [ 0xb2c8, 'HangulSyllableNi', [], 'Display', None],
  [ 0xb2c9, 'HangulSyllableNig', [], 'Display', None],
  [ 0xb2ca, 'HangulSyllableNigg', [], 'Display', None],
  [ 0xb2cb, 'HangulSyllableNigs', [], 'Display', None],
  [ 0xb2cc, 'HangulSyllableNin', [], 'Display', None],
  [ 0xb2cd, 'HangulSyllableNinj', [], 'Display', None],
  [ 0xb2ce, 'HangulSyllableNinh', [], 'Display', None],
  [ 0xb2cf, 'HangulSyllableNid', [], 'Display', None],
  [ 0xb2d0, 'HangulSyllableNil', [], 'Display', None],
  [ 0xb2d1, 'HangulSyllableNilg', [], 'Display', None],
  [ 0xb2d2, 'HangulSyllableNilm', [], 'Display', None],
  [ 0xb2d3, 'HangulSyllableNilb', [], 'Display', None],
  [ 0xb2d4, 'HangulSyllableNils', [], 'Display', None],
  [ 0xb2d5, 'HangulSyllableNilt', [], 'Display', None],
  [ 0xb2d6, 'HangulSyllableNilp', [], 'Display', None],
  [ 0xb2d7, 'HangulSyllableNilh', [], 'Display', None],
  [ 0xb2d8, 'HangulSyllableNim', [], 'Display', None],
  [ 0xb2d9, 'HangulSyllableNib', [], 'Display', None],
  [ 0xb2da, 'HangulSyllableNibs', [], 'Display', None],
  [ 0xb2db, 'HangulSyllableNis', [], 'Display', None],
  [ 0xb2dc, 'HangulSyllableNiss', [], 'Display', None],
  [ 0xb2dd, 'HangulSyllableNing', [], 'Display', None],
  [ 0xb2de, 'HangulSyllableNij', [], 'Display', None],
  [ 0xb2df, 'HangulSyllableNic', [], 'Display', None],
  [ 0xb2e0, 'HangulSyllableNik', [], 'Display', None],
  [ 0xb2e1, 'HangulSyllableNit', [], 'Display', None],
  [ 0xb2e2, 'HangulSyllableNip', [], 'Display', None],
  [ 0xb2e3, 'HangulSyllableNih', [], 'Display', None],
  [ 0xb2e4, 'HangulSyllableDa', [], 'Display', None],
  [ 0xb2e5, 'HangulSyllableDag', [], 'Display', None],
  [ 0xb2e6, 'HangulSyllableDagg', [], 'Display', None],
  [ 0xb2e7, 'HangulSyllableDags', [], 'Display', None],
  [ 0xb2e8, 'HangulSyllableDan', [], 'Display', None],
  [ 0xb2e9, 'HangulSyllableDanj', [], 'Display', None],
  [ 0xb2ea, 'HangulSyllableDanh', [], 'Display', None],
  [ 0xb2eb, 'HangulSyllableDad', [], 'Display', None],
  [ 0xb2ec, 'HangulSyllableDal', [], 'Display', None],
  [ 0xb2ed, 'HangulSyllableDalg', [], 'Display', None],
  [ 0xb2ee, 'HangulSyllableDalm', [], 'Display', None],
  [ 0xb2ef, 'HangulSyllableDalb', [], 'Display', None],
  [ 0xb2f0, 'HangulSyllableDals', [], 'Display', None],
  [ 0xb2f1, 'HangulSyllableDalt', [], 'Display', None],
  [ 0xb2f2, 'HangulSyllableDalp', [], 'Display', None],
  [ 0xb2f3, 'HangulSyllableDalh', [], 'Display', None],
  [ 0xb2f4, 'HangulSyllableDam', [], 'Display', None],
  [ 0xb2f5, 'HangulSyllableDab', [], 'Display', None],
  [ 0xb2f6, 'HangulSyllableDabs', [], 'Display', None],
  [ 0xb2f7, 'HangulSyllableDas', [], 'Display', None],
  [ 0xb2f8, 'HangulSyllableDass', [], 'Display', None],
  [ 0xb2f9, 'HangulSyllableDang', [], 'Display', None],
  [ 0xb2fa, 'HangulSyllableDaj', [], 'Display', None],
  [ 0xb2fb, 'HangulSyllableDac', [], 'Display', None],
  [ 0xb2fc, 'HangulSyllableDak', [], 'Display', None],
  [ 0xb2fd, 'HangulSyllableDat', [], 'Display', None],
  [ 0xb2fe, 'HangulSyllableDap', [], 'Display', None],
  [ 0xb2ff, 'HangulSyllableDah', [], 'Display', None],
  [ 0xb300, 'HangulSyllableDae', [], 'Display', None],
  [ 0xb301, 'HangulSyllableDaeg', [], 'Display', None],
  [ 0xb302, 'HangulSyllableDaegg', [], 'Display', None],
  [ 0xb303, 'HangulSyllableDaegs', [], 'Display', None],
  [ 0xb304, 'HangulSyllableDaen', [], 'Display', None],
  [ 0xb305, 'HangulSyllableDaenj', [], 'Display', None],
  [ 0xb306, 'HangulSyllableDaenh', [], 'Display', None],
  [ 0xb307, 'HangulSyllableDaed', [], 'Display', None],
  [ 0xb308, 'HangulSyllableDael', [], 'Display', None],
  [ 0xb309, 'HangulSyllableDaelg', [], 'Display', None],
  [ 0xb30a, 'HangulSyllableDaelm', [], 'Display', None],
  [ 0xb30b, 'HangulSyllableDaelb', [], 'Display', None],
  [ 0xb30c, 'HangulSyllableDaels', [], 'Display', None],
  [ 0xb30d, 'HangulSyllableDaelt', [], 'Display', None],
  [ 0xb30e, 'HangulSyllableDaelp', [], 'Display', None],
  [ 0xb30f, 'HangulSyllableDaelh', [], 'Display', None],
  [ 0xb310, 'HangulSyllableDaem', [], 'Display', None],
  [ 0xb311, 'HangulSyllableDaeb', [], 'Display', None],
  [ 0xb312, 'HangulSyllableDaebs', [], 'Display', None],
  [ 0xb313, 'HangulSyllableDaes', [], 'Display', None],
  [ 0xb314, 'HangulSyllableDaess', [], 'Display', None],
  [ 0xb315, 'HangulSyllableDaeng', [], 'Display', None],
  [ 0xb316, 'HangulSyllableDaej', [], 'Display', None],
  [ 0xb317, 'HangulSyllableDaec', [], 'Display', None],
  [ 0xb318, 'HangulSyllableDaek', [], 'Display', None],
  [ 0xb319, 'HangulSyllableDaet', [], 'Display', None],
  [ 0xb31a, 'HangulSyllableDaep', [], 'Display', None],
  [ 0xb31b, 'HangulSyllableDaeh', [], 'Display', None],
  [ 0xb31c, 'HangulSyllableDya', [], 'Display', None],
  [ 0xb31d, 'HangulSyllableDyag', [], 'Display', None],
  [ 0xb31e, 'HangulSyllableDyagg', [], 'Display', None],
  [ 0xb31f, 'HangulSyllableDyags', [], 'Display', None],
  [ 0xb320, 'HangulSyllableDyan', [], 'Display', None],
  [ 0xb321, 'HangulSyllableDyanj', [], 'Display', None],
  [ 0xb322, 'HangulSyllableDyanh', [], 'Display', None],
  [ 0xb323, 'HangulSyllableDyad', [], 'Display', None],
  [ 0xb324, 'HangulSyllableDyal', [], 'Display', None],
  [ 0xb325, 'HangulSyllableDyalg', [], 'Display', None],
  [ 0xb326, 'HangulSyllableDyalm', [], 'Display', None],
  [ 0xb327, 'HangulSyllableDyalb', [], 'Display', None],
  [ 0xb328, 'HangulSyllableDyals', [], 'Display', None],
  [ 0xb329, 'HangulSyllableDyalt', [], 'Display', None],
  [ 0xb32a, 'HangulSyllableDyalp', [], 'Display', None],
  [ 0xb32b, 'HangulSyllableDyalh', [], 'Display', None],
  [ 0xb32c, 'HangulSyllableDyam', [], 'Display', None],
  [ 0xb32d, 'HangulSyllableDyab', [], 'Display', None],
  [ 0xb32e, 'HangulSyllableDyabs', [], 'Display', None],
  [ 0xb32f, 'HangulSyllableDyas', [], 'Display', None],
  [ 0xb330, 'HangulSyllableDyass', [], 'Display', None],
  [ 0xb331, 'HangulSyllableDyang', [], 'Display', None],
  [ 0xb332, 'HangulSyllableDyaj', [], 'Display', None],
  [ 0xb333, 'HangulSyllableDyac', [], 'Display', None],
  [ 0xb334, 'HangulSyllableDyak', [], 'Display', None],
  [ 0xb335, 'HangulSyllableDyat', [], 'Display', None],
  [ 0xb336, 'HangulSyllableDyap', [], 'Display', None],
  [ 0xb337, 'HangulSyllableDyah', [], 'Display', None],
  [ 0xb338, 'HangulSyllableDyae', [], 'Display', None],
  [ 0xb339, 'HangulSyllableDyaeg', [], 'Display', None],
  [ 0xb33a, 'HangulSyllableDyaegg', [], 'Display', None],
  [ 0xb33b, 'HangulSyllableDyaegs', [], 'Display', None],
  [ 0xb33c, 'HangulSyllableDyaen', [], 'Display', None],
  [ 0xb33d, 'HangulSyllableDyaenj', [], 'Display', None],
  [ 0xb33e, 'HangulSyllableDyaenh', [], 'Display', None],
  [ 0xb33f, 'HangulSyllableDyaed', [], 'Display', None],
  [ 0xb340, 'HangulSyllableDyael', [], 'Display', None],
  [ 0xb341, 'HangulSyllableDyaelg', [], 'Display', None],
  [ 0xb342, 'HangulSyllableDyaelm', [], 'Display', None],
  [ 0xb343, 'HangulSyllableDyaelb', [], 'Display', None],
  [ 0xb344, 'HangulSyllableDyaels', [], 'Display', None],
  [ 0xb345, 'HangulSyllableDyaelt', [], 'Display', None],
  [ 0xb346, 'HangulSyllableDyaelp', [], 'Display', None],
  [ 0xb347, 'HangulSyllableDyaelh', [], 'Display', None],
  [ 0xb348, 'HangulSyllableDyaem', [], 'Display', None],
  [ 0xb349, 'HangulSyllableDyaeb', [], 'Display', None],
  [ 0xb34a, 'HangulSyllableDyaebs', [], 'Display', None],
  [ 0xb34b, 'HangulSyllableDyaes', [], 'Display', None],
  [ 0xb34c, 'HangulSyllableDyaess', [], 'Display', None],
  [ 0xb34d, 'HangulSyllableDyaeng', [], 'Display', None],
  [ 0xb34e, 'HangulSyllableDyaej', [], 'Display', None],
  [ 0xb34f, 'HangulSyllableDyaec', [], 'Display', None],
  [ 0xb350, 'HangulSyllableDyaek', [], 'Display', None],
  [ 0xb351, 'HangulSyllableDyaet', [], 'Display', None],
  [ 0xb352, 'HangulSyllableDyaep', [], 'Display', None],
  [ 0xb353, 'HangulSyllableDyaeh', [], 'Display', None],
  [ 0xb354, 'HangulSyllableDeo', [], 'Display', None],
  [ 0xb355, 'HangulSyllableDeog', [], 'Display', None],
  [ 0xb356, 'HangulSyllableDeogg', [], 'Display', None],
  [ 0xb357, 'HangulSyllableDeogs', [], 'Display', None],
  [ 0xb358, 'HangulSyllableDeon', [], 'Display', None],
  [ 0xb359, 'HangulSyllableDeonj', [], 'Display', None],
  [ 0xb35a, 'HangulSyllableDeonh', [], 'Display', None],
  [ 0xb35b, 'HangulSyllableDeod', [], 'Display', None],
  [ 0xb35c, 'HangulSyllableDeol', [], 'Display', None],
  [ 0xb35d, 'HangulSyllableDeolg', [], 'Display', None],
  [ 0xb35e, 'HangulSyllableDeolm', [], 'Display', None],
  [ 0xb35f, 'HangulSyllableDeolb', [], 'Display', None],
  [ 0xb360, 'HangulSyllableDeols', [], 'Display', None],
  [ 0xb361, 'HangulSyllableDeolt', [], 'Display', None],
  [ 0xb362, 'HangulSyllableDeolp', [], 'Display', None],
  [ 0xb363, 'HangulSyllableDeolh', [], 'Display', None],
  [ 0xb364, 'HangulSyllableDeom', [], 'Display', None],
  [ 0xb365, 'HangulSyllableDeob', [], 'Display', None],
  [ 0xb366, 'HangulSyllableDeobs', [], 'Display', None],
  [ 0xb367, 'HangulSyllableDeos', [], 'Display', None],
  [ 0xb368, 'HangulSyllableDeoss', [], 'Display', None],
  [ 0xb369, 'HangulSyllableDeong', [], 'Display', None],
  [ 0xb36a, 'HangulSyllableDeoj', [], 'Display', None],
  [ 0xb36b, 'HangulSyllableDeoc', [], 'Display', None],
  [ 0xb36c, 'HangulSyllableDeok', [], 'Display', None],
  [ 0xb36d, 'HangulSyllableDeot', [], 'Display', None],
  [ 0xb36e, 'HangulSyllableDeop', [], 'Display', None],
  [ 0xb36f, 'HangulSyllableDeoh', [], 'Display', None],
  [ 0xb370, 'HangulSyllableDe', [], 'Display', None],
  [ 0xb371, 'HangulSyllableDeg', [], 'Display', None],
  [ 0xb372, 'HangulSyllableDegg', [], 'Display', None],
  [ 0xb373, 'HangulSyllableDegs', [], 'Display', None],
  [ 0xb374, 'HangulSyllableDen', [], 'Display', None],
  [ 0xb375, 'HangulSyllableDenj', [], 'Display', None],
  [ 0xb376, 'HangulSyllableDenh', [], 'Display', None],
  [ 0xb377, 'HangulSyllableDed', [], 'Display', None],
  [ 0xb378, 'HangulSyllableDel', [], 'Display', None],
  [ 0xb379, 'HangulSyllableDelg', [], 'Display', None],
  [ 0xb37a, 'HangulSyllableDelm', [], 'Display', None],
  [ 0xb37b, 'HangulSyllableDelb', [], 'Display', None],
  [ 0xb37c, 'HangulSyllableDels', [], 'Display', None],
  [ 0xb37d, 'HangulSyllableDelt', [], 'Display', None],
  [ 0xb37e, 'HangulSyllableDelp', [], 'Display', None],
  [ 0xb37f, 'HangulSyllableDelh', [], 'Display', None],
  [ 0xb380, 'HangulSyllableDem', [], 'Display', None],
  [ 0xb381, 'HangulSyllableDeb', [], 'Display', None],
  [ 0xb382, 'HangulSyllableDebs', [], 'Display', None],
  [ 0xb383, 'HangulSyllableDes', [], 'Display', None],
  [ 0xb384, 'HangulSyllableDess', [], 'Display', None],
  [ 0xb385, 'HangulSyllableDeng', [], 'Display', None],
  [ 0xb386, 'HangulSyllableDej', [], 'Display', None],
  [ 0xb387, 'HangulSyllableDec', [], 'Display', None],
  [ 0xb388, 'HangulSyllableDek', [], 'Display', None],
  [ 0xb389, 'HangulSyllableDet', [], 'Display', None],
  [ 0xb38a, 'HangulSyllableDep', [], 'Display', None],
  [ 0xb38b, 'HangulSyllableDeh', [], 'Display', None],
  [ 0xb38c, 'HangulSyllableDyeo', [], 'Display', None],
  [ 0xb38d, 'HangulSyllableDyeog', [], 'Display', None],
  [ 0xb38e, 'HangulSyllableDyeogg', [], 'Display', None],
  [ 0xb38f, 'HangulSyllableDyeogs', [], 'Display', None],
  [ 0xb390, 'HangulSyllableDyeon', [], 'Display', None],
  [ 0xb391, 'HangulSyllableDyeonj', [], 'Display', None],
  [ 0xb392, 'HangulSyllableDyeonh', [], 'Display', None],
  [ 0xb393, 'HangulSyllableDyeod', [], 'Display', None],
  [ 0xb394, 'HangulSyllableDyeol', [], 'Display', None],
  [ 0xb395, 'HangulSyllableDyeolg', [], 'Display', None],
  [ 0xb396, 'HangulSyllableDyeolm', [], 'Display', None],
  [ 0xb397, 'HangulSyllableDyeolb', [], 'Display', None],
  [ 0xb398, 'HangulSyllableDyeols', [], 'Display', None],
  [ 0xb399, 'HangulSyllableDyeolt', [], 'Display', None],
  [ 0xb39a, 'HangulSyllableDyeolp', [], 'Display', None],
  [ 0xb39b, 'HangulSyllableDyeolh', [], 'Display', None],
  [ 0xb39c, 'HangulSyllableDyeom', [], 'Display', None],
  [ 0xb39d, 'HangulSyllableDyeob', [], 'Display', None],
  [ 0xb39e, 'HangulSyllableDyeobs', [], 'Display', None],
  [ 0xb39f, 'HangulSyllableDyeos', [], 'Display', None],
  [ 0xb3a0, 'HangulSyllableDyeoss', [], 'Display', None],
  [ 0xb3a1, 'HangulSyllableDyeong', [], 'Display', None],
  [ 0xb3a2, 'HangulSyllableDyeoj', [], 'Display', None],
  [ 0xb3a3, 'HangulSyllableDyeoc', [], 'Display', None],
  [ 0xb3a4, 'HangulSyllableDyeok', [], 'Display', None],
  [ 0xb3a5, 'HangulSyllableDyeot', [], 'Display', None],
  [ 0xb3a6, 'HangulSyllableDyeop', [], 'Display', None],
  [ 0xb3a7, 'HangulSyllableDyeoh', [], 'Display', None],
  [ 0xb3a8, 'HangulSyllableDye', [], 'Display', None],
  [ 0xb3a9, 'HangulSyllableDyeg', [], 'Display', None],
  [ 0xb3aa, 'HangulSyllableDyegg', [], 'Display', None],
  [ 0xb3ab, 'HangulSyllableDyegs', [], 'Display', None],
  [ 0xb3ac, 'HangulSyllableDyen', [], 'Display', None],
  [ 0xb3ad, 'HangulSyllableDyenj', [], 'Display', None],
  [ 0xb3ae, 'HangulSyllableDyenh', [], 'Display', None],
  [ 0xb3af, 'HangulSyllableDyed', [], 'Display', None],
  [ 0xb3b0, 'HangulSyllableDyel', [], 'Display', None],
  [ 0xb3b1, 'HangulSyllableDyelg', [], 'Display', None],
  [ 0xb3b2, 'HangulSyllableDyelm', [], 'Display', None],
  [ 0xb3b3, 'HangulSyllableDyelb', [], 'Display', None],
  [ 0xb3b4, 'HangulSyllableDyels', [], 'Display', None],
  [ 0xb3b5, 'HangulSyllableDyelt', [], 'Display', None],
  [ 0xb3b6, 'HangulSyllableDyelp', [], 'Display', None],
  [ 0xb3b7, 'HangulSyllableDyelh', [], 'Display', None],
  [ 0xb3b8, 'HangulSyllableDyem', [], 'Display', None],
  [ 0xb3b9, 'HangulSyllableDyeb', [], 'Display', None],
  [ 0xb3ba, 'HangulSyllableDyebs', [], 'Display', None],
  [ 0xb3bb, 'HangulSyllableDyes', [], 'Display', None],
  [ 0xb3bc, 'HangulSyllableDyess', [], 'Display', None],
  [ 0xb3bd, 'HangulSyllableDyeng', [], 'Display', None],
  [ 0xb3be, 'HangulSyllableDyej', [], 'Display', None],
  [ 0xb3bf, 'HangulSyllableDyec', [], 'Display', None],
  [ 0xb3c0, 'HangulSyllableDyek', [], 'Display', None],
  [ 0xb3c1, 'HangulSyllableDyet', [], 'Display', None],
  [ 0xb3c2, 'HangulSyllableDyep', [], 'Display', None],
  [ 0xb3c3, 'HangulSyllableDyeh', [], 'Display', None],
  [ 0xb3c4, 'HangulSyllableDo', [], 'Display', None],
  [ 0xb3c5, 'HangulSyllableDog', [], 'Display', None],
  [ 0xb3c6, 'HangulSyllableDogg', [], 'Display', None],
  [ 0xb3c7, 'HangulSyllableDogs', [], 'Display', None],
  [ 0xb3c8, 'HangulSyllableDon', [], 'Display', None],
  [ 0xb3c9, 'HangulSyllableDonj', [], 'Display', None],
  [ 0xb3ca, 'HangulSyllableDonh', [], 'Display', None],
  [ 0xb3cb, 'HangulSyllableDod', [], 'Display', None],
  [ 0xb3cc, 'HangulSyllableDol', [], 'Display', None],
  [ 0xb3cd, 'HangulSyllableDolg', [], 'Display', None],
  [ 0xb3ce, 'HangulSyllableDolm', [], 'Display', None],
  [ 0xb3cf, 'HangulSyllableDolb', [], 'Display', None],
  [ 0xb3d0, 'HangulSyllableDols', [], 'Display', None],
  [ 0xb3d1, 'HangulSyllableDolt', [], 'Display', None],
  [ 0xb3d2, 'HangulSyllableDolp', [], 'Display', None],
  [ 0xb3d3, 'HangulSyllableDolh', [], 'Display', None],
  [ 0xb3d4, 'HangulSyllableDom', [], 'Display', None],
  [ 0xb3d5, 'HangulSyllableDob', [], 'Display', None],
  [ 0xb3d6, 'HangulSyllableDobs', [], 'Display', None],
  [ 0xb3d7, 'HangulSyllableDos', [], 'Display', None],
  [ 0xb3d8, 'HangulSyllableDoss', [], 'Display', None],
  [ 0xb3d9, 'HangulSyllableDong', [], 'Display', None],
  [ 0xb3da, 'HangulSyllableDoj', [], 'Display', None],
  [ 0xb3db, 'HangulSyllableDoc', [], 'Display', None],
  [ 0xb3dc, 'HangulSyllableDok', [], 'Display', None],
  [ 0xb3dd, 'HangulSyllableDot', [], 'Display', None],
  [ 0xb3de, 'HangulSyllableDop', [], 'Display', None],
  [ 0xb3df, 'HangulSyllableDoh', [], 'Display', None],
  [ 0xb3e0, 'HangulSyllableDwa', [], 'Display', None],
  [ 0xb3e1, 'HangulSyllableDwag', [], 'Display', None],
  [ 0xb3e2, 'HangulSyllableDwagg', [], 'Display', None],
  [ 0xb3e3, 'HangulSyllableDwags', [], 'Display', None],
  [ 0xb3e4, 'HangulSyllableDwan', [], 'Display', None],
  [ 0xb3e5, 'HangulSyllableDwanj', [], 'Display', None],
  [ 0xb3e6, 'HangulSyllableDwanh', [], 'Display', None],
  [ 0xb3e7, 'HangulSyllableDwad', [], 'Display', None],
  [ 0xb3e8, 'HangulSyllableDwal', [], 'Display', None],
  [ 0xb3e9, 'HangulSyllableDwalg', [], 'Display', None],
  [ 0xb3ea, 'HangulSyllableDwalm', [], 'Display', None],
  [ 0xb3eb, 'HangulSyllableDwalb', [], 'Display', None],
  [ 0xb3ec, 'HangulSyllableDwals', [], 'Display', None],
  [ 0xb3ed, 'HangulSyllableDwalt', [], 'Display', None],
  [ 0xb3ee, 'HangulSyllableDwalp', [], 'Display', None],
  [ 0xb3ef, 'HangulSyllableDwalh', [], 'Display', None],
  [ 0xb3f0, 'HangulSyllableDwam', [], 'Display', None],
  [ 0xb3f1, 'HangulSyllableDwab', [], 'Display', None],
  [ 0xb3f2, 'HangulSyllableDwabs', [], 'Display', None],
  [ 0xb3f3, 'HangulSyllableDwas', [], 'Display', None],
  [ 0xb3f4, 'HangulSyllableDwass', [], 'Display', None],
  [ 0xb3f5, 'HangulSyllableDwang', [], 'Display', None],
  [ 0xb3f6, 'HangulSyllableDwaj', [], 'Display', None],
  [ 0xb3f7, 'HangulSyllableDwac', [], 'Display', None],
  [ 0xb3f8, 'HangulSyllableDwak', [], 'Display', None],
  [ 0xb3f9, 'HangulSyllableDwat', [], 'Display', None],
  [ 0xb3fa, 'HangulSyllableDwap', [], 'Display', None],
  [ 0xb3fb, 'HangulSyllableDwah', [], 'Display', None],
  [ 0xb3fc, 'HangulSyllableDwae', [], 'Display', None],
  [ 0xb3fd, 'HangulSyllableDwaeg', [], 'Display', None],
  [ 0xb3fe, 'HangulSyllableDwaegg', [], 'Display', None],
  [ 0xb3ff, 'HangulSyllableDwaegs', [], 'Display', None],
  [ 0xb400, 'HangulSyllableDwaen', [], 'Display', None],
  [ 0xb401, 'HangulSyllableDwaenj', [], 'Display', None],
  [ 0xb402, 'HangulSyllableDwaenh', [], 'Display', None],
  [ 0xb403, 'HangulSyllableDwaed', [], 'Display', None],
  [ 0xb404, 'HangulSyllableDwael', [], 'Display', None],
  [ 0xb405, 'HangulSyllableDwaelg', [], 'Display', None],
  [ 0xb406, 'HangulSyllableDwaelm', [], 'Display', None],
  [ 0xb407, 'HangulSyllableDwaelb', [], 'Display', None],
  [ 0xb408, 'HangulSyllableDwaels', [], 'Display', None],
  [ 0xb409, 'HangulSyllableDwaelt', [], 'Display', None],
  [ 0xb40a, 'HangulSyllableDwaelp', [], 'Display', None],
  [ 0xb40b, 'HangulSyllableDwaelh', [], 'Display', None],
  [ 0xb40c, 'HangulSyllableDwaem', [], 'Display', None],
  [ 0xb40d, 'HangulSyllableDwaeb', [], 'Display', None],
  [ 0xb40e, 'HangulSyllableDwaebs', [], 'Display', None],
  [ 0xb40f, 'HangulSyllableDwaes', [], 'Display', None],
  [ 0xb410, 'HangulSyllableDwaess', [], 'Display', None],
  [ 0xb411, 'HangulSyllableDwaeng', [], 'Display', None],
  [ 0xb412, 'HangulSyllableDwaej', [], 'Display', None],
  [ 0xb413, 'HangulSyllableDwaec', [], 'Display', None],
  [ 0xb414, 'HangulSyllableDwaek', [], 'Display', None],
  [ 0xb415, 'HangulSyllableDwaet', [], 'Display', None],
  [ 0xb416, 'HangulSyllableDwaep', [], 'Display', None],
  [ 0xb417, 'HangulSyllableDwaeh', [], 'Display', None],
  [ 0xb418, 'HangulSyllableDoe', [], 'Display', None],
  [ 0xb419, 'HangulSyllableDoeg', [], 'Display', None],
  [ 0xb41a, 'HangulSyllableDoegg', [], 'Display', None],
  [ 0xb41b, 'HangulSyllableDoegs', [], 'Display', None],
  [ 0xb41c, 'HangulSyllableDoen', [], 'Display', None],
  [ 0xb41d, 'HangulSyllableDoenj', [], 'Display', None],
  [ 0xb41e, 'HangulSyllableDoenh', [], 'Display', None],
  [ 0xb41f, 'HangulSyllableDoed', [], 'Display', None],
  [ 0xb420, 'HangulSyllableDoel', [], 'Display', None],
  [ 0xb421, 'HangulSyllableDoelg', [], 'Display', None],
  [ 0xb422, 'HangulSyllableDoelm', [], 'Display', None],
  [ 0xb423, 'HangulSyllableDoelb', [], 'Display', None],
  [ 0xb424, 'HangulSyllableDoels', [], 'Display', None],
  [ 0xb425, 'HangulSyllableDoelt', [], 'Display', None],
  [ 0xb426, 'HangulSyllableDoelp', [], 'Display', None],
  [ 0xb427, 'HangulSyllableDoelh', [], 'Display', None],
  [ 0xb428, 'HangulSyllableDoem', [], 'Display', None],
  [ 0xb429, 'HangulSyllableDoeb', [], 'Display', None],
  [ 0xb42a, 'HangulSyllableDoebs', [], 'Display', None],
  [ 0xb42b, 'HangulSyllableDoes', [], 'Display', None],
  [ 0xb42c, 'HangulSyllableDoess', [], 'Display', None],
  [ 0xb42d, 'HangulSyllableDoeng', [], 'Display', None],
  [ 0xb42e, 'HangulSyllableDoej', [], 'Display', None],
  [ 0xb42f, 'HangulSyllableDoec', [], 'Display', None],
  [ 0xb430, 'HangulSyllableDoek', [], 'Display', None],
  [ 0xb431, 'HangulSyllableDoet', [], 'Display', None],
  [ 0xb432, 'HangulSyllableDoep', [], 'Display', None],
  [ 0xb433, 'HangulSyllableDoeh', [], 'Display', None],
  [ 0xb434, 'HangulSyllableDyo', [], 'Display', None],
  [ 0xb435, 'HangulSyllableDyog', [], 'Display', None],
  [ 0xb436, 'HangulSyllableDyogg', [], 'Display', None],
  [ 0xb437, 'HangulSyllableDyogs', [], 'Display', None],
  [ 0xb438, 'HangulSyllableDyon', [], 'Display', None],
  [ 0xb439, 'HangulSyllableDyonj', [], 'Display', None],
  [ 0xb43a, 'HangulSyllableDyonh', [], 'Display', None],
  [ 0xb43b, 'HangulSyllableDyod', [], 'Display', None],
  [ 0xb43c, 'HangulSyllableDyol', [], 'Display', None],
  [ 0xb43d, 'HangulSyllableDyolg', [], 'Display', None],
  [ 0xb43e, 'HangulSyllableDyolm', [], 'Display', None],
  [ 0xb43f, 'HangulSyllableDyolb', [], 'Display', None],
  [ 0xb440, 'HangulSyllableDyols', [], 'Display', None],
  [ 0xb441, 'HangulSyllableDyolt', [], 'Display', None],
  [ 0xb442, 'HangulSyllableDyolp', [], 'Display', None],
  [ 0xb443, 'HangulSyllableDyolh', [], 'Display', None],
  [ 0xb444, 'HangulSyllableDyom', [], 'Display', None],
  [ 0xb445, 'HangulSyllableDyob', [], 'Display', None],
  [ 0xb446, 'HangulSyllableDyobs', [], 'Display', None],
  [ 0xb447, 'HangulSyllableDyos', [], 'Display', None],
  [ 0xb448, 'HangulSyllableDyoss', [], 'Display', None],
  [ 0xb449, 'HangulSyllableDyong', [], 'Display', None],
  [ 0xb44a, 'HangulSyllableDyoj', [], 'Display', None],
  [ 0xb44b, 'HangulSyllableDyoc', [], 'Display', None],
  [ 0xb44c, 'HangulSyllableDyok', [], 'Display', None],
  [ 0xb44d, 'HangulSyllableDyot', [], 'Display', None],
  [ 0xb44e, 'HangulSyllableDyop', [], 'Display', None],
  [ 0xb44f, 'HangulSyllableDyoh', [], 'Display', None],
  [ 0xb450, 'HangulSyllableDu', [], 'Display', None],
  [ 0xb451, 'HangulSyllableDug', [], 'Display', None],
  [ 0xb452, 'HangulSyllableDugg', [], 'Display', None],
  [ 0xb453, 'HangulSyllableDugs', [], 'Display', None],
  [ 0xb454, 'HangulSyllableDun', [], 'Display', None],
  [ 0xb455, 'HangulSyllableDunj', [], 'Display', None],
  [ 0xb456, 'HangulSyllableDunh', [], 'Display', None],
  [ 0xb457, 'HangulSyllableDud', [], 'Display', None],
  [ 0xb458, 'HangulSyllableDul', [], 'Display', None],
  [ 0xb459, 'HangulSyllableDulg', [], 'Display', None],
  [ 0xb45a, 'HangulSyllableDulm', [], 'Display', None],
  [ 0xb45b, 'HangulSyllableDulb', [], 'Display', None],
  [ 0xb45c, 'HangulSyllableDuls', [], 'Display', None],
  [ 0xb45d, 'HangulSyllableDult', [], 'Display', None],
  [ 0xb45e, 'HangulSyllableDulp', [], 'Display', None],
  [ 0xb45f, 'HangulSyllableDulh', [], 'Display', None],
  [ 0xb460, 'HangulSyllableDum', [], 'Display', None],
  [ 0xb461, 'HangulSyllableDub', [], 'Display', None],
  [ 0xb462, 'HangulSyllableDubs', [], 'Display', None],
  [ 0xb463, 'HangulSyllableDus', [], 'Display', None],
  [ 0xb464, 'HangulSyllableDuss', [], 'Display', None],
  [ 0xb465, 'HangulSyllableDung', [], 'Display', None],
  [ 0xb466, 'HangulSyllableDuj', [], 'Display', None],
  [ 0xb467, 'HangulSyllableDuc', [], 'Display', None],
  [ 0xb468, 'HangulSyllableDuk', [], 'Display', None],
  [ 0xb469, 'HangulSyllableDut', [], 'Display', None],
  [ 0xb46a, 'HangulSyllableDup', [], 'Display', None],
  [ 0xb46b, 'HangulSyllableDuh', [], 'Display', None],
  [ 0xb46c, 'HangulSyllableDweo', [], 'Display', None],
  [ 0xb46d, 'HangulSyllableDweog', [], 'Display', None],
  [ 0xb46e, 'HangulSyllableDweogg', [], 'Display', None],
  [ 0xb46f, 'HangulSyllableDweogs', [], 'Display', None],
  [ 0xb470, 'HangulSyllableDweon', [], 'Display', None],
  [ 0xb471, 'HangulSyllableDweonj', [], 'Display', None],
  [ 0xb472, 'HangulSyllableDweonh', [], 'Display', None],
  [ 0xb473, 'HangulSyllableDweod', [], 'Display', None],
  [ 0xb474, 'HangulSyllableDweol', [], 'Display', None],
  [ 0xb475, 'HangulSyllableDweolg', [], 'Display', None],
  [ 0xb476, 'HangulSyllableDweolm', [], 'Display', None],
  [ 0xb477, 'HangulSyllableDweolb', [], 'Display', None],
  [ 0xb478, 'HangulSyllableDweols', [], 'Display', None],
  [ 0xb479, 'HangulSyllableDweolt', [], 'Display', None],
  [ 0xb47a, 'HangulSyllableDweolp', [], 'Display', None],
  [ 0xb47b, 'HangulSyllableDweolh', [], 'Display', None],
  [ 0xb47c, 'HangulSyllableDweom', [], 'Display', None],
  [ 0xb47d, 'HangulSyllableDweob', [], 'Display', None],
  [ 0xb47e, 'HangulSyllableDweobs', [], 'Display', None],
  [ 0xb47f, 'HangulSyllableDweos', [], 'Display', None],
  [ 0xb480, 'HangulSyllableDweoss', [], 'Display', None],
  [ 0xb481, 'HangulSyllableDweong', [], 'Display', None],
  [ 0xb482, 'HangulSyllableDweoj', [], 'Display', None],
  [ 0xb483, 'HangulSyllableDweoc', [], 'Display', None],
  [ 0xb484, 'HangulSyllableDweok', [], 'Display', None],
  [ 0xb485, 'HangulSyllableDweot', [], 'Display', None],
  [ 0xb486, 'HangulSyllableDweop', [], 'Display', None],
  [ 0xb487, 'HangulSyllableDweoh', [], 'Display', None],
  [ 0xb488, 'HangulSyllableDwe', [], 'Display', None],
  [ 0xb489, 'HangulSyllableDweg', [], 'Display', None],
  [ 0xb48a, 'HangulSyllableDwegg', [], 'Display', None],
  [ 0xb48b, 'HangulSyllableDwegs', [], 'Display', None],
  [ 0xb48c, 'HangulSyllableDwen', [], 'Display', None],
  [ 0xb48d, 'HangulSyllableDwenj', [], 'Display', None],
  [ 0xb48e, 'HangulSyllableDwenh', [], 'Display', None],
  [ 0xb48f, 'HangulSyllableDwed', [], 'Display', None],
  [ 0xb490, 'HangulSyllableDwel', [], 'Display', None],
  [ 0xb491, 'HangulSyllableDwelg', [], 'Display', None],
  [ 0xb492, 'HangulSyllableDwelm', [], 'Display', None],
  [ 0xb493, 'HangulSyllableDwelb', [], 'Display', None],
  [ 0xb494, 'HangulSyllableDwels', [], 'Display', None],
  [ 0xb495, 'HangulSyllableDwelt', [], 'Display', None],
  [ 0xb496, 'HangulSyllableDwelp', [], 'Display', None],
  [ 0xb497, 'HangulSyllableDwelh', [], 'Display', None],
  [ 0xb498, 'HangulSyllableDwem', [], 'Display', None],
  [ 0xb499, 'HangulSyllableDweb', [], 'Display', None],
  [ 0xb49a, 'HangulSyllableDwebs', [], 'Display', None],
  [ 0xb49b, 'HangulSyllableDwes', [], 'Display', None],
  [ 0xb49c, 'HangulSyllableDwess', [], 'Display', None],
  [ 0xb49d, 'HangulSyllableDweng', [], 'Display', None],
  [ 0xb49e, 'HangulSyllableDwej', [], 'Display', None],
  [ 0xb49f, 'HangulSyllableDwec', [], 'Display', None],
  [ 0xb4a0, 'HangulSyllableDwek', [], 'Display', None],
  [ 0xb4a1, 'HangulSyllableDwet', [], 'Display', None],
  [ 0xb4a2, 'HangulSyllableDwep', [], 'Display', None],
  [ 0xb4a3, 'HangulSyllableDweh', [], 'Display', None],
  [ 0xb4a4, 'HangulSyllableDwi', [], 'Display', None],
  [ 0xb4a5, 'HangulSyllableDwig', [], 'Display', None],
  [ 0xb4a6, 'HangulSyllableDwigg', [], 'Display', None],
  [ 0xb4a7, 'HangulSyllableDwigs', [], 'Display', None],
  [ 0xb4a8, 'HangulSyllableDwin', [], 'Display', None],
  [ 0xb4a9, 'HangulSyllableDwinj', [], 'Display', None],
  [ 0xb4aa, 'HangulSyllableDwinh', [], 'Display', None],
  [ 0xb4ab, 'HangulSyllableDwid', [], 'Display', None],
  [ 0xb4ac, 'HangulSyllableDwil', [], 'Display', None],
  [ 0xb4ad, 'HangulSyllableDwilg', [], 'Display', None],
  [ 0xb4ae, 'HangulSyllableDwilm', [], 'Display', None],
  [ 0xb4af, 'HangulSyllableDwilb', [], 'Display', None],
  [ 0xb4b0, 'HangulSyllableDwils', [], 'Display', None],
  [ 0xb4b1, 'HangulSyllableDwilt', [], 'Display', None],
  [ 0xb4b2, 'HangulSyllableDwilp', [], 'Display', None],
  [ 0xb4b3, 'HangulSyllableDwilh', [], 'Display', None],
  [ 0xb4b4, 'HangulSyllableDwim', [], 'Display', None],
  [ 0xb4b5, 'HangulSyllableDwib', [], 'Display', None],
  [ 0xb4b6, 'HangulSyllableDwibs', [], 'Display', None],
  [ 0xb4b7, 'HangulSyllableDwis', [], 'Display', None],
  [ 0xb4b8, 'HangulSyllableDwiss', [], 'Display', None],
  [ 0xb4b9, 'HangulSyllableDwing', [], 'Display', None],
  [ 0xb4ba, 'HangulSyllableDwij', [], 'Display', None],
  [ 0xb4bb, 'HangulSyllableDwic', [], 'Display', None],
  [ 0xb4bc, 'HangulSyllableDwik', [], 'Display', None],
  [ 0xb4bd, 'HangulSyllableDwit', [], 'Display', None],
  [ 0xb4be, 'HangulSyllableDwip', [], 'Display', None],
  [ 0xb4bf, 'HangulSyllableDwih', [], 'Display', None],
  [ 0xb4c0, 'HangulSyllableDyu', [], 'Display', None],
  [ 0xb4c1, 'HangulSyllableDyug', [], 'Display', None],
  [ 0xb4c2, 'HangulSyllableDyugg', [], 'Display', None],
  [ 0xb4c3, 'HangulSyllableDyugs', [], 'Display', None],
  [ 0xb4c4, 'HangulSyllableDyun', [], 'Display', None],
  [ 0xb4c5, 'HangulSyllableDyunj', [], 'Display', None],
  [ 0xb4c6, 'HangulSyllableDyunh', [], 'Display', None],
  [ 0xb4c7, 'HangulSyllableDyud', [], 'Display', None],
  [ 0xb4c8, 'HangulSyllableDyul', [], 'Display', None],
  [ 0xb4c9, 'HangulSyllableDyulg', [], 'Display', None],
  [ 0xb4ca, 'HangulSyllableDyulm', [], 'Display', None],
  [ 0xb4cb, 'HangulSyllableDyulb', [], 'Display', None],
  [ 0xb4cc, 'HangulSyllableDyuls', [], 'Display', None],
  [ 0xb4cd, 'HangulSyllableDyult', [], 'Display', None],
  [ 0xb4ce, 'HangulSyllableDyulp', [], 'Display', None],
  [ 0xb4cf, 'HangulSyllableDyulh', [], 'Display', None],
  [ 0xb4d0, 'HangulSyllableDyum', [], 'Display', None],
  [ 0xb4d1, 'HangulSyllableDyub', [], 'Display', None],
  [ 0xb4d2, 'HangulSyllableDyubs', [], 'Display', None],
  [ 0xb4d3, 'HangulSyllableDyus', [], 'Display', None],
  [ 0xb4d4, 'HangulSyllableDyuss', [], 'Display', None],
  [ 0xb4d5, 'HangulSyllableDyung', [], 'Display', None],
  [ 0xb4d6, 'HangulSyllableDyuj', [], 'Display', None],
  [ 0xb4d7, 'HangulSyllableDyuc', [], 'Display', None],
  [ 0xb4d8, 'HangulSyllableDyuk', [], 'Display', None],
  [ 0xb4d9, 'HangulSyllableDyut', [], 'Display', None],
  [ 0xb4da, 'HangulSyllableDyup', [], 'Display', None],
  [ 0xb4db, 'HangulSyllableDyuh', [], 'Display', None],
  [ 0xb4dc, 'HangulSyllableDeu', [], 'Display', None],
  [ 0xb4dd, 'HangulSyllableDeug', [], 'Display', None],
  [ 0xb4de, 'HangulSyllableDeugg', [], 'Display', None],
  [ 0xb4df, 'HangulSyllableDeugs', [], 'Display', None],
  [ 0xb4e0, 'HangulSyllableDeun', [], 'Display', None],
  [ 0xb4e1, 'HangulSyllableDeunj', [], 'Display', None],
  [ 0xb4e2, 'HangulSyllableDeunh', [], 'Display', None],
  [ 0xb4e3, 'HangulSyllableDeud', [], 'Display', None],
  [ 0xb4e4, 'HangulSyllableDeul', [], 'Display', None],
  [ 0xb4e5, 'HangulSyllableDeulg', [], 'Display', None],
  [ 0xb4e6, 'HangulSyllableDeulm', [], 'Display', None],
  [ 0xb4e7, 'HangulSyllableDeulb', [], 'Display', None],
  [ 0xb4e8, 'HangulSyllableDeuls', [], 'Display', None],
  [ 0xb4e9, 'HangulSyllableDeult', [], 'Display', None],
  [ 0xb4ea, 'HangulSyllableDeulp', [], 'Display', None],
  [ 0xb4eb, 'HangulSyllableDeulh', [], 'Display', None],
  [ 0xb4ec, 'HangulSyllableDeum', [], 'Display', None],
  [ 0xb4ed, 'HangulSyllableDeub', [], 'Display', None],
  [ 0xb4ee, 'HangulSyllableDeubs', [], 'Display', None],
  [ 0xb4ef, 'HangulSyllableDeus', [], 'Display', None],
  [ 0xb4f0, 'HangulSyllableDeuss', [], 'Display', None],
  [ 0xb4f1, 'HangulSyllableDeung', [], 'Display', None],
  [ 0xb4f2, 'HangulSyllableDeuj', [], 'Display', None],
  [ 0xb4f3, 'HangulSyllableDeuc', [], 'Display', None],
  [ 0xb4f4, 'HangulSyllableDeuk', [], 'Display', None],
  [ 0xb4f5, 'HangulSyllableDeut', [], 'Display', None],
  [ 0xb4f6, 'HangulSyllableDeup', [], 'Display', None],
  [ 0xb4f7, 'HangulSyllableDeuh', [], 'Display', None],
  [ 0xb4f8, 'HangulSyllableDyi', [], 'Display', None],
  [ 0xb4f9, 'HangulSyllableDyig', [], 'Display', None],
  [ 0xb4fa, 'HangulSyllableDyigg', [], 'Display', None],
  [ 0xb4fb, 'HangulSyllableDyigs', [], 'Display', None],
  [ 0xb4fc, 'HangulSyllableDyin', [], 'Display', None],
  [ 0xb4fd, 'HangulSyllableDyinj', [], 'Display', None],
  [ 0xb4fe, 'HangulSyllableDyinh', [], 'Display', None],
  [ 0xb4ff, 'HangulSyllableDyid', [], 'Display', None],
  [ 0xb500, 'HangulSyllableDyil', [], 'Display', None],
  [ 0xb501, 'HangulSyllableDyilg', [], 'Display', None],
  [ 0xb502, 'HangulSyllableDyilm', [], 'Display', None],
  [ 0xb503, 'HangulSyllableDyilb', [], 'Display', None],
  [ 0xb504, 'HangulSyllableDyils', [], 'Display', None],
  [ 0xb505, 'HangulSyllableDyilt', [], 'Display', None],
  [ 0xb506, 'HangulSyllableDyilp', [], 'Display', None],
  [ 0xb507, 'HangulSyllableDyilh', [], 'Display', None],
  [ 0xb508, 'HangulSyllableDyim', [], 'Display', None],
  [ 0xb509, 'HangulSyllableDyib', [], 'Display', None],
  [ 0xb50a, 'HangulSyllableDyibs', [], 'Display', None],
  [ 0xb50b, 'HangulSyllableDyis', [], 'Display', None],
  [ 0xb50c, 'HangulSyllableDyiss', [], 'Display', None],
  [ 0xb50d, 'HangulSyllableDying', [], 'Display', None],
  [ 0xb50e, 'HangulSyllableDyij', [], 'Display', None],
  [ 0xb50f, 'HangulSyllableDyic', [], 'Display', None],
  [ 0xb510, 'HangulSyllableDyik', [], 'Display', None],
  [ 0xb511, 'HangulSyllableDyit', [], 'Display', None],
  [ 0xb512, 'HangulSyllableDyip', [], 'Display', None],
  [ 0xb513, 'HangulSyllableDyih', [], 'Display', None],
  [ 0xb514, 'HangulSyllableDi', [], 'Display', None],
  [ 0xb515, 'HangulSyllableDig', [], 'Display', None],
  [ 0xb516, 'HangulSyllableDigg', [], 'Display', None],
  [ 0xb517, 'HangulSyllableDigs', [], 'Display', None],
  [ 0xb518, 'HangulSyllableDin', [], 'Display', None],
  [ 0xb519, 'HangulSyllableDinj', [], 'Display', None],
  [ 0xb51a, 'HangulSyllableDinh', [], 'Display', None],
  [ 0xb51b, 'HangulSyllableDid', [], 'Display', None],
  [ 0xb51c, 'HangulSyllableDil', [], 'Display', None],
  [ 0xb51d, 'HangulSyllableDilg', [], 'Display', None],
  [ 0xb51e, 'HangulSyllableDilm', [], 'Display', None],
  [ 0xb51f, 'HangulSyllableDilb', [], 'Display', None],
  [ 0xb520, 'HangulSyllableDils', [], 'Display', None],
  [ 0xb521, 'HangulSyllableDilt', [], 'Display', None],
  [ 0xb522, 'HangulSyllableDilp', [], 'Display', None],
  [ 0xb523, 'HangulSyllableDilh', [], 'Display', None],
  [ 0xb524, 'HangulSyllableDim', [], 'Display', None],
  [ 0xb525, 'HangulSyllableDib', [], 'Display', None],
  [ 0xb526, 'HangulSyllableDibs', [], 'Display', None],
  [ 0xb527, 'HangulSyllableDis', [], 'Display', None],
  [ 0xb528, 'HangulSyllableDiss', [], 'Display', None],
  [ 0xb529, 'HangulSyllableDing', [], 'Display', None],
  [ 0xb52a, 'HangulSyllableDij', [], 'Display', None],
  [ 0xb52b, 'HangulSyllableDic', [], 'Display', None],
  [ 0xb52c, 'HangulSyllableDik', [], 'Display', None],
  [ 0xb52d, 'HangulSyllableDit', [], 'Display', None],
  [ 0xb52e, 'HangulSyllableDip', [], 'Display', None],
  [ 0xb52f, 'HangulSyllableDih', [], 'Display', None],
  [ 0xb530, 'HangulSyllableDda', [], 'Display', None],
  [ 0xb531, 'HangulSyllableDdag', [], 'Display', None],
  [ 0xb532, 'HangulSyllableDdagg', [], 'Display', None],
  [ 0xb533, 'HangulSyllableDdags', [], 'Display', None],
  [ 0xb534, 'HangulSyllableDdan', [], 'Display', None],
  [ 0xb535, 'HangulSyllableDdanj', [], 'Display', None],
  [ 0xb536, 'HangulSyllableDdanh', [], 'Display', None],
  [ 0xb537, 'HangulSyllableDdad', [], 'Display', None],
  [ 0xb538, 'HangulSyllableDdal', [], 'Display', None],
  [ 0xb539, 'HangulSyllableDdalg', [], 'Display', None],
  [ 0xb53a, 'HangulSyllableDdalm', [], 'Display', None],
  [ 0xb53b, 'HangulSyllableDdalb', [], 'Display', None],
  [ 0xb53c, 'HangulSyllableDdals', [], 'Display', None],
  [ 0xb53d, 'HangulSyllableDdalt', [], 'Display', None],
  [ 0xb53e, 'HangulSyllableDdalp', [], 'Display', None],
  [ 0xb53f, 'HangulSyllableDdalh', [], 'Display', None],
  [ 0xb540, 'HangulSyllableDdam', [], 'Display', None],
  [ 0xb541, 'HangulSyllableDdab', [], 'Display', None],
  [ 0xb542, 'HangulSyllableDdabs', [], 'Display', None],
  [ 0xb543, 'HangulSyllableDdas', [], 'Display', None],
  [ 0xb544, 'HangulSyllableDdass', [], 'Display', None],
  [ 0xb545, 'HangulSyllableDdang', [], 'Display', None],
  [ 0xb546, 'HangulSyllableDdaj', [], 'Display', None],
  [ 0xb547, 'HangulSyllableDdac', [], 'Display', None],
  [ 0xb548, 'HangulSyllableDdak', [], 'Display', None],
  [ 0xb549, 'HangulSyllableDdat', [], 'Display', None],
  [ 0xb54a, 'HangulSyllableDdap', [], 'Display', None],
  [ 0xb54b, 'HangulSyllableDdah', [], 'Display', None],
  [ 0xb54c, 'HangulSyllableDdae', [], 'Display', None],
  [ 0xb54d, 'HangulSyllableDdaeg', [], 'Display', None],
  [ 0xb54e, 'HangulSyllableDdaegg', [], 'Display', None],
  [ 0xb54f, 'HangulSyllableDdaegs', [], 'Display', None],
  [ 0xb550, 'HangulSyllableDdaen', [], 'Display', None],
  [ 0xb551, 'HangulSyllableDdaenj', [], 'Display', None],
  [ 0xb552, 'HangulSyllableDdaenh', [], 'Display', None],
  [ 0xb553, 'HangulSyllableDdaed', [], 'Display', None],
  [ 0xb554, 'HangulSyllableDdael', [], 'Display', None],
  [ 0xb555, 'HangulSyllableDdaelg', [], 'Display', None],
  [ 0xb556, 'HangulSyllableDdaelm', [], 'Display', None],
  [ 0xb557, 'HangulSyllableDdaelb', [], 'Display', None],
  [ 0xb558, 'HangulSyllableDdaels', [], 'Display', None],
  [ 0xb559, 'HangulSyllableDdaelt', [], 'Display', None],
  [ 0xb55a, 'HangulSyllableDdaelp', [], 'Display', None],
  [ 0xb55b, 'HangulSyllableDdaelh', [], 'Display', None],
  [ 0xb55c, 'HangulSyllableDdaem', [], 'Display', None],
  [ 0xb55d, 'HangulSyllableDdaeb', [], 'Display', None],
  [ 0xb55e, 'HangulSyllableDdaebs', [], 'Display', None],
  [ 0xb55f, 'HangulSyllableDdaes', [], 'Display', None],
  [ 0xb560, 'HangulSyllableDdaess', [], 'Display', None],
  [ 0xb561, 'HangulSyllableDdaeng', [], 'Display', None],
  [ 0xb562, 'HangulSyllableDdaej', [], 'Display', None],
  [ 0xb563, 'HangulSyllableDdaec', [], 'Display', None],
  [ 0xb564, 'HangulSyllableDdaek', [], 'Display', None],
  [ 0xb565, 'HangulSyllableDdaet', [], 'Display', None],
  [ 0xb566, 'HangulSyllableDdaep', [], 'Display', None],
  [ 0xb567, 'HangulSyllableDdaeh', [], 'Display', None],
  [ 0xb568, 'HangulSyllableDdya', [], 'Display', None],
  [ 0xb569, 'HangulSyllableDdyag', [], 'Display', None],
  [ 0xb56a, 'HangulSyllableDdyagg', [], 'Display', None],
  [ 0xb56b, 'HangulSyllableDdyags', [], 'Display', None],
  [ 0xb56c, 'HangulSyllableDdyan', [], 'Display', None],
  [ 0xb56d, 'HangulSyllableDdyanj', [], 'Display', None],
  [ 0xb56e, 'HangulSyllableDdyanh', [], 'Display', None],
  [ 0xb56f, 'HangulSyllableDdyad', [], 'Display', None],
  [ 0xb570, 'HangulSyllableDdyal', [], 'Display', None],
  [ 0xb571, 'HangulSyllableDdyalg', [], 'Display', None],
  [ 0xb572, 'HangulSyllableDdyalm', [], 'Display', None],
  [ 0xb573, 'HangulSyllableDdyalb', [], 'Display', None],
  [ 0xb574, 'HangulSyllableDdyals', [], 'Display', None],
  [ 0xb575, 'HangulSyllableDdyalt', [], 'Display', None],
  [ 0xb576, 'HangulSyllableDdyalp', [], 'Display', None],
  [ 0xb577, 'HangulSyllableDdyalh', [], 'Display', None],
  [ 0xb578, 'HangulSyllableDdyam', [], 'Display', None],
  [ 0xb579, 'HangulSyllableDdyab', [], 'Display', None],
  [ 0xb57a, 'HangulSyllableDdyabs', [], 'Display', None],
  [ 0xb57b, 'HangulSyllableDdyas', [], 'Display', None],
  [ 0xb57c, 'HangulSyllableDdyass', [], 'Display', None],
  [ 0xb57d, 'HangulSyllableDdyang', [], 'Display', None],
  [ 0xb57e, 'HangulSyllableDdyaj', [], 'Display', None],
  [ 0xb57f, 'HangulSyllableDdyac', [], 'Display', None],
  [ 0xb580, 'HangulSyllableDdyak', [], 'Display', None],
  [ 0xb581, 'HangulSyllableDdyat', [], 'Display', None],
  [ 0xb582, 'HangulSyllableDdyap', [], 'Display', None],
  [ 0xb583, 'HangulSyllableDdyah', [], 'Display', None],
  [ 0xb584, 'HangulSyllableDdyae', [], 'Display', None],
  [ 0xb585, 'HangulSyllableDdyaeg', [], 'Display', None],
  [ 0xb586, 'HangulSyllableDdyaegg', [], 'Display', None],
  [ 0xb587, 'HangulSyllableDdyaegs', [], 'Display', None],
  [ 0xb588, 'HangulSyllableDdyaen', [], 'Display', None],
  [ 0xb589, 'HangulSyllableDdyaenj', [], 'Display', None],
  [ 0xb58a, 'HangulSyllableDdyaenh', [], 'Display', None],
  [ 0xb58b, 'HangulSyllableDdyaed', [], 'Display', None],
  [ 0xb58c, 'HangulSyllableDdyael', [], 'Display', None],
  [ 0xb58d, 'HangulSyllableDdyaelg', [], 'Display', None],
  [ 0xb58e, 'HangulSyllableDdyaelm', [], 'Display', None],
  [ 0xb58f, 'HangulSyllableDdyaelb', [], 'Display', None],
  [ 0xb590, 'HangulSyllableDdyaels', [], 'Display', None],
  [ 0xb591, 'HangulSyllableDdyaelt', [], 'Display', None],
  [ 0xb592, 'HangulSyllableDdyaelp', [], 'Display', None],
  [ 0xb593, 'HangulSyllableDdyaelh', [], 'Display', None],
  [ 0xb594, 'HangulSyllableDdyaem', [], 'Display', None],
  [ 0xb595, 'HangulSyllableDdyaeb', [], 'Display', None],
  [ 0xb596, 'HangulSyllableDdyaebs', [], 'Display', None],
  [ 0xb597, 'HangulSyllableDdyaes', [], 'Display', None],
  [ 0xb598, 'HangulSyllableDdyaess', [], 'Display', None],
  [ 0xb599, 'HangulSyllableDdyaeng', [], 'Display', None],
  [ 0xb59a, 'HangulSyllableDdyaej', [], 'Display', None],
  [ 0xb59b, 'HangulSyllableDdyaec', [], 'Display', None],
  [ 0xb59c, 'HangulSyllableDdyaek', [], 'Display', None],
  [ 0xb59d, 'HangulSyllableDdyaet', [], 'Display', None],
  [ 0xb59e, 'HangulSyllableDdyaep', [], 'Display', None],
  [ 0xb59f, 'HangulSyllableDdyaeh', [], 'Display', None],
  [ 0xb5a0, 'HangulSyllableDdeo', [], 'Display', None],
  [ 0xb5a1, 'HangulSyllableDdeog', [], 'Display', None],
  [ 0xb5a2, 'HangulSyllableDdeogg', [], 'Display', None],
  [ 0xb5a3, 'HangulSyllableDdeogs', [], 'Display', None],
  [ 0xb5a4, 'HangulSyllableDdeon', [], 'Display', None],
  [ 0xb5a5, 'HangulSyllableDdeonj', [], 'Display', None],
  [ 0xb5a6, 'HangulSyllableDdeonh', [], 'Display', None],
  [ 0xb5a7, 'HangulSyllableDdeod', [], 'Display', None],
  [ 0xb5a8, 'HangulSyllableDdeol', [], 'Display', None],
  [ 0xb5a9, 'HangulSyllableDdeolg', [], 'Display', None],
  [ 0xb5aa, 'HangulSyllableDdeolm', [], 'Display', None],
  [ 0xb5ab, 'HangulSyllableDdeolb', [], 'Display', None],
  [ 0xb5ac, 'HangulSyllableDdeols', [], 'Display', None],
  [ 0xb5ad, 'HangulSyllableDdeolt', [], 'Display', None],
  [ 0xb5ae, 'HangulSyllableDdeolp', [], 'Display', None],
  [ 0xb5af, 'HangulSyllableDdeolh', [], 'Display', None],
  [ 0xb5b0, 'HangulSyllableDdeom', [], 'Display', None],
  [ 0xb5b1, 'HangulSyllableDdeob', [], 'Display', None],
  [ 0xb5b2, 'HangulSyllableDdeobs', [], 'Display', None],
  [ 0xb5b3, 'HangulSyllableDdeos', [], 'Display', None],
  [ 0xb5b4, 'HangulSyllableDdeoss', [], 'Display', None],
  [ 0xb5b5, 'HangulSyllableDdeong', [], 'Display', None],
  [ 0xb5b6, 'HangulSyllableDdeoj', [], 'Display', None],
  [ 0xb5b7, 'HangulSyllableDdeoc', [], 'Display', None],
  [ 0xb5b8, 'HangulSyllableDdeok', [], 'Display', None],
  [ 0xb5b9, 'HangulSyllableDdeot', [], 'Display', None],
  [ 0xb5ba, 'HangulSyllableDdeop', [], 'Display', None],
  [ 0xb5bb, 'HangulSyllableDdeoh', [], 'Display', None],
  [ 0xb5bc, 'HangulSyllableDde', [], 'Display', None],
  [ 0xb5bd, 'HangulSyllableDdeg', [], 'Display', None],
  [ 0xb5be, 'HangulSyllableDdegg', [], 'Display', None],
  [ 0xb5bf, 'HangulSyllableDdegs', [], 'Display', None],
  [ 0xb5c0, 'HangulSyllableDden', [], 'Display', None],
  [ 0xb5c1, 'HangulSyllableDdenj', [], 'Display', None],
  [ 0xb5c2, 'HangulSyllableDdenh', [], 'Display', None],
  [ 0xb5c3, 'HangulSyllableDded', [], 'Display', None],
  [ 0xb5c4, 'HangulSyllableDdel', [], 'Display', None],
  [ 0xb5c5, 'HangulSyllableDdelg', [], 'Display', None],
  [ 0xb5c6, 'HangulSyllableDdelm', [], 'Display', None],
  [ 0xb5c7, 'HangulSyllableDdelb', [], 'Display', None],
  [ 0xb5c8, 'HangulSyllableDdels', [], 'Display', None],
  [ 0xb5c9, 'HangulSyllableDdelt', [], 'Display', None],
  [ 0xb5ca, 'HangulSyllableDdelp', [], 'Display', None],
  [ 0xb5cb, 'HangulSyllableDdelh', [], 'Display', None],
  [ 0xb5cc, 'HangulSyllableDdem', [], 'Display', None],
  [ 0xb5cd, 'HangulSyllableDdeb', [], 'Display', None],
  [ 0xb5ce, 'HangulSyllableDdebs', [], 'Display', None],
  [ 0xb5cf, 'HangulSyllableDdes', [], 'Display', None],
  [ 0xb5d0, 'HangulSyllableDdess', [], 'Display', None],
  [ 0xb5d1, 'HangulSyllableDdeng', [], 'Display', None],
  [ 0xb5d2, 'HangulSyllableDdej', [], 'Display', None],
  [ 0xb5d3, 'HangulSyllableDdec', [], 'Display', None],
  [ 0xb5d4, 'HangulSyllableDdek', [], 'Display', None],
  [ 0xb5d5, 'HangulSyllableDdet', [], 'Display', None],
  [ 0xb5d6, 'HangulSyllableDdep', [], 'Display', None],
  [ 0xb5d7, 'HangulSyllableDdeh', [], 'Display', None],
  [ 0xb5d8, 'HangulSyllableDdyeo', [], 'Display', None],
  [ 0xb5d9, 'HangulSyllableDdyeog', [], 'Display', None],
  [ 0xb5da, 'HangulSyllableDdyeogg', [], 'Display', None],
  [ 0xb5db, 'HangulSyllableDdyeogs', [], 'Display', None],
  [ 0xb5dc, 'HangulSyllableDdyeon', [], 'Display', None],
  [ 0xb5dd, 'HangulSyllableDdyeonj', [], 'Display', None],
  [ 0xb5de, 'HangulSyllableDdyeonh', [], 'Display', None],
  [ 0xb5df, 'HangulSyllableDdyeod', [], 'Display', None],
  [ 0xb5e0, 'HangulSyllableDdyeol', [], 'Display', None],
  [ 0xb5e1, 'HangulSyllableDdyeolg', [], 'Display', None],
  [ 0xb5e2, 'HangulSyllableDdyeolm', [], 'Display', None],
  [ 0xb5e3, 'HangulSyllableDdyeolb', [], 'Display', None],
  [ 0xb5e4, 'HangulSyllableDdyeols', [], 'Display', None],
  [ 0xb5e5, 'HangulSyllableDdyeolt', [], 'Display', None],
  [ 0xb5e6, 'HangulSyllableDdyeolp', [], 'Display', None],
  [ 0xb5e7, 'HangulSyllableDdyeolh', [], 'Display', None],
  [ 0xb5e8, 'HangulSyllableDdyeom', [], 'Display', None],
  [ 0xb5e9, 'HangulSyllableDdyeob', [], 'Display', None],
  [ 0xb5ea, 'HangulSyllableDdyeobs', [], 'Display', None],
  [ 0xb5eb, 'HangulSyllableDdyeos', [], 'Display', None],
  [ 0xb5ec, 'HangulSyllableDdyeoss', [], 'Display', None],
  [ 0xb5ed, 'HangulSyllableDdyeong', [], 'Display', None],
  [ 0xb5ee, 'HangulSyllableDdyeoj', [], 'Display', None],
  [ 0xb5ef, 'HangulSyllableDdyeoc', [], 'Display', None],
  [ 0xb5f0, 'HangulSyllableDdyeok', [], 'Display', None],
  [ 0xb5f1, 'HangulSyllableDdyeot', [], 'Display', None],
  [ 0xb5f2, 'HangulSyllableDdyeop', [], 'Display', None],
  [ 0xb5f3, 'HangulSyllableDdyeoh', [], 'Display', None],
  [ 0xb5f4, 'HangulSyllableDdye', [], 'Display', None],
  [ 0xb5f5, 'HangulSyllableDdyeg', [], 'Display', None],
  [ 0xb5f6, 'HangulSyllableDdyegg', [], 'Display', None],
  [ 0xb5f7, 'HangulSyllableDdyegs', [], 'Display', None],
  [ 0xb5f8, 'HangulSyllableDdyen', [], 'Display', None],
  [ 0xb5f9, 'HangulSyllableDdyenj', [], 'Display', None],
  [ 0xb5fa, 'HangulSyllableDdyenh', [], 'Display', None],
  [ 0xb5fb, 'HangulSyllableDdyed', [], 'Display', None],
  [ 0xb5fc, 'HangulSyllableDdyel', [], 'Display', None],
  [ 0xb5fd, 'HangulSyllableDdyelg', [], 'Display', None],
  [ 0xb5fe, 'HangulSyllableDdyelm', [], 'Display', None],
  [ 0xb5ff, 'HangulSyllableDdyelb', [], 'Display', None],
  [ 0xb600, 'HangulSyllableDdyels', [], 'Display', None],
  [ 0xb601, 'HangulSyllableDdyelt', [], 'Display', None],
  [ 0xb602, 'HangulSyllableDdyelp', [], 'Display', None],
  [ 0xb603, 'HangulSyllableDdyelh', [], 'Display', None],
  [ 0xb604, 'HangulSyllableDdyem', [], 'Display', None],
  [ 0xb605, 'HangulSyllableDdyeb', [], 'Display', None],
  [ 0xb606, 'HangulSyllableDdyebs', [], 'Display', None],
  [ 0xb607, 'HangulSyllableDdyes', [], 'Display', None],
  [ 0xb608, 'HangulSyllableDdyess', [], 'Display', None],
  [ 0xb609, 'HangulSyllableDdyeng', [], 'Display', None],
  [ 0xb60a, 'HangulSyllableDdyej', [], 'Display', None],
  [ 0xb60b, 'HangulSyllableDdyec', [], 'Display', None],
  [ 0xb60c, 'HangulSyllableDdyek', [], 'Display', None],
  [ 0xb60d, 'HangulSyllableDdyet', [], 'Display', None],
  [ 0xb60e, 'HangulSyllableDdyep', [], 'Display', None],
  [ 0xb60f, 'HangulSyllableDdyeh', [], 'Display', None],
  [ 0xb610, 'HangulSyllableDdo', [], 'Display', None],
  [ 0xb611, 'HangulSyllableDdog', [], 'Display', None],
  [ 0xb612, 'HangulSyllableDdogg', [], 'Display', None],
  [ 0xb613, 'HangulSyllableDdogs', [], 'Display', None],
  [ 0xb614, 'HangulSyllableDdon', [], 'Display', None],
  [ 0xb615, 'HangulSyllableDdonj', [], 'Display', None],
  [ 0xb616, 'HangulSyllableDdonh', [], 'Display', None],
  [ 0xb617, 'HangulSyllableDdod', [], 'Display', None],
  [ 0xb618, 'HangulSyllableDdol', [], 'Display', None],
  [ 0xb619, 'HangulSyllableDdolg', [], 'Display', None],
  [ 0xb61a, 'HangulSyllableDdolm', [], 'Display', None],
  [ 0xb61b, 'HangulSyllableDdolb', [], 'Display', None],
  [ 0xb61c, 'HangulSyllableDdols', [], 'Display', None],
  [ 0xb61d, 'HangulSyllableDdolt', [], 'Display', None],
  [ 0xb61e, 'HangulSyllableDdolp', [], 'Display', None],
  [ 0xb61f, 'HangulSyllableDdolh', [], 'Display', None],
  [ 0xb620, 'HangulSyllableDdom', [], 'Display', None],
  [ 0xb621, 'HangulSyllableDdob', [], 'Display', None],
  [ 0xb622, 'HangulSyllableDdobs', [], 'Display', None],
  [ 0xb623, 'HangulSyllableDdos', [], 'Display', None],
  [ 0xb624, 'HangulSyllableDdoss', [], 'Display', None],
  [ 0xb625, 'HangulSyllableDdong', [], 'Display', None],
  [ 0xb626, 'HangulSyllableDdoj', [], 'Display', None],
  [ 0xb627, 'HangulSyllableDdoc', [], 'Display', None],
  [ 0xb628, 'HangulSyllableDdok', [], 'Display', None],
  [ 0xb629, 'HangulSyllableDdot', [], 'Display', None],
  [ 0xb62a, 'HangulSyllableDdop', [], 'Display', None],
  [ 0xb62b, 'HangulSyllableDdoh', [], 'Display', None],
  [ 0xb62c, 'HangulSyllableDdwa', [], 'Display', None],
  [ 0xb62d, 'HangulSyllableDdwag', [], 'Display', None],
  [ 0xb62e, 'HangulSyllableDdwagg', [], 'Display', None],
  [ 0xb62f, 'HangulSyllableDdwags', [], 'Display', None],
  [ 0xb630, 'HangulSyllableDdwan', [], 'Display', None],
  [ 0xb631, 'HangulSyllableDdwanj', [], 'Display', None],
  [ 0xb632, 'HangulSyllableDdwanh', [], 'Display', None],
  [ 0xb633, 'HangulSyllableDdwad', [], 'Display', None],
  [ 0xb634, 'HangulSyllableDdwal', [], 'Display', None],
  [ 0xb635, 'HangulSyllableDdwalg', [], 'Display', None],
  [ 0xb636, 'HangulSyllableDdwalm', [], 'Display', None],
  [ 0xb637, 'HangulSyllableDdwalb', [], 'Display', None],
  [ 0xb638, 'HangulSyllableDdwals', [], 'Display', None],
  [ 0xb639, 'HangulSyllableDdwalt', [], 'Display', None],
  [ 0xb63a, 'HangulSyllableDdwalp', [], 'Display', None],
  [ 0xb63b, 'HangulSyllableDdwalh', [], 'Display', None],
  [ 0xb63c, 'HangulSyllableDdwam', [], 'Display', None],
  [ 0xb63d, 'HangulSyllableDdwab', [], 'Display', None],
  [ 0xb63e, 'HangulSyllableDdwabs', [], 'Display', None],
  [ 0xb63f, 'HangulSyllableDdwas', [], 'Display', None],
  [ 0xb640, 'HangulSyllableDdwass', [], 'Display', None],
  [ 0xb641, 'HangulSyllableDdwang', [], 'Display', None],
  [ 0xb642, 'HangulSyllableDdwaj', [], 'Display', None],
  [ 0xb643, 'HangulSyllableDdwac', [], 'Display', None],
  [ 0xb644, 'HangulSyllableDdwak', [], 'Display', None],
  [ 0xb645, 'HangulSyllableDdwat', [], 'Display', None],
  [ 0xb646, 'HangulSyllableDdwap', [], 'Display', None],
  [ 0xb647, 'HangulSyllableDdwah', [], 'Display', None],
  [ 0xb648, 'HangulSyllableDdwae', [], 'Display', None],
  [ 0xb649, 'HangulSyllableDdwaeg', [], 'Display', None],
  [ 0xb64a, 'HangulSyllableDdwaegg', [], 'Display', None],
  [ 0xb64b, 'HangulSyllableDdwaegs', [], 'Display', None],
  [ 0xb64c, 'HangulSyllableDdwaen', [], 'Display', None],
  [ 0xb64d, 'HangulSyllableDdwaenj', [], 'Display', None],
  [ 0xb64e, 'HangulSyllableDdwaenh', [], 'Display', None],
  [ 0xb64f, 'HangulSyllableDdwaed', [], 'Display', None],
  [ 0xb650, 'HangulSyllableDdwael', [], 'Display', None],
  [ 0xb651, 'HangulSyllableDdwaelg', [], 'Display', None],
  [ 0xb652, 'HangulSyllableDdwaelm', [], 'Display', None],
  [ 0xb653, 'HangulSyllableDdwaelb', [], 'Display', None],
  [ 0xb654, 'HangulSyllableDdwaels', [], 'Display', None],
  [ 0xb655, 'HangulSyllableDdwaelt', [], 'Display', None],
  [ 0xb656, 'HangulSyllableDdwaelp', [], 'Display', None],
  [ 0xb657, 'HangulSyllableDdwaelh', [], 'Display', None],
  [ 0xb658, 'HangulSyllableDdwaem', [], 'Display', None],
  [ 0xb659, 'HangulSyllableDdwaeb', [], 'Display', None],
  [ 0xb65a, 'HangulSyllableDdwaebs', [], 'Display', None],
  [ 0xb65b, 'HangulSyllableDdwaes', [], 'Display', None],
  [ 0xb65c, 'HangulSyllableDdwaess', [], 'Display', None],
  [ 0xb65d, 'HangulSyllableDdwaeng', [], 'Display', None],
  [ 0xb65e, 'HangulSyllableDdwaej', [], 'Display', None],
  [ 0xb65f, 'HangulSyllableDdwaec', [], 'Display', None],
  [ 0xb660, 'HangulSyllableDdwaek', [], 'Display', None],
  [ 0xb661, 'HangulSyllableDdwaet', [], 'Display', None],
  [ 0xb662, 'HangulSyllableDdwaep', [], 'Display', None],
  [ 0xb663, 'HangulSyllableDdwaeh', [], 'Display', None],
  [ 0xb664, 'HangulSyllableDdoe', [], 'Display', None],
  [ 0xb665, 'HangulSyllableDdoeg', [], 'Display', None],
  [ 0xb666, 'HangulSyllableDdoegg', [], 'Display', None],
  [ 0xb667, 'HangulSyllableDdoegs', [], 'Display', None],
  [ 0xb668, 'HangulSyllableDdoen', [], 'Display', None],
  [ 0xb669, 'HangulSyllableDdoenj', [], 'Display', None],
  [ 0xb66a, 'HangulSyllableDdoenh', [], 'Display', None],
  [ 0xb66b, 'HangulSyllableDdoed', [], 'Display', None],
  [ 0xb66c, 'HangulSyllableDdoel', [], 'Display', None],
  [ 0xb66d, 'HangulSyllableDdoelg', [], 'Display', None],
  [ 0xb66e, 'HangulSyllableDdoelm', [], 'Display', None],
  [ 0xb66f, 'HangulSyllableDdoelb', [], 'Display', None],
  [ 0xb670, 'HangulSyllableDdoels', [], 'Display', None],
  [ 0xb671, 'HangulSyllableDdoelt', [], 'Display', None],
  [ 0xb672, 'HangulSyllableDdoelp', [], 'Display', None],
  [ 0xb673, 'HangulSyllableDdoelh', [], 'Display', None],
  [ 0xb674, 'HangulSyllableDdoem', [], 'Display', None],
  [ 0xb675, 'HangulSyllableDdoeb', [], 'Display', None],
  [ 0xb676, 'HangulSyllableDdoebs', [], 'Display', None],
  [ 0xb677, 'HangulSyllableDdoes', [], 'Display', None],
  [ 0xb678, 'HangulSyllableDdoess', [], 'Display', None],
  [ 0xb679, 'HangulSyllableDdoeng', [], 'Display', None],
  [ 0xb67a, 'HangulSyllableDdoej', [], 'Display', None],
  [ 0xb67b, 'HangulSyllableDdoec', [], 'Display', None],
  [ 0xb67c, 'HangulSyllableDdoek', [], 'Display', None],
  [ 0xb67d, 'HangulSyllableDdoet', [], 'Display', None],
  [ 0xb67e, 'HangulSyllableDdoep', [], 'Display', None],
  [ 0xb67f, 'HangulSyllableDdoeh', [], 'Display', None],
  [ 0xb680, 'HangulSyllableDdyo', [], 'Display', None],
  [ 0xb681, 'HangulSyllableDdyog', [], 'Display', None],
  [ 0xb682, 'HangulSyllableDdyogg', [], 'Display', None],
  [ 0xb683, 'HangulSyllableDdyogs', [], 'Display', None],
  [ 0xb684, 'HangulSyllableDdyon', [], 'Display', None],
  [ 0xb685, 'HangulSyllableDdyonj', [], 'Display', None],
  [ 0xb686, 'HangulSyllableDdyonh', [], 'Display', None],
  [ 0xb687, 'HangulSyllableDdyod', [], 'Display', None],
  [ 0xb688, 'HangulSyllableDdyol', [], 'Display', None],
  [ 0xb689, 'HangulSyllableDdyolg', [], 'Display', None],
  [ 0xb68a, 'HangulSyllableDdyolm', [], 'Display', None],
  [ 0xb68b, 'HangulSyllableDdyolb', [], 'Display', None],
  [ 0xb68c, 'HangulSyllableDdyols', [], 'Display', None],
  [ 0xb68d, 'HangulSyllableDdyolt', [], 'Display', None],
  [ 0xb68e, 'HangulSyllableDdyolp', [], 'Display', None],
  [ 0xb68f, 'HangulSyllableDdyolh', [], 'Display', None],
  [ 0xb690, 'HangulSyllableDdyom', [], 'Display', None],
  [ 0xb691, 'HangulSyllableDdyob', [], 'Display', None],
  [ 0xb692, 'HangulSyllableDdyobs', [], 'Display', None],
  [ 0xb693, 'HangulSyllableDdyos', [], 'Display', None],
  [ 0xb694, 'HangulSyllableDdyoss', [], 'Display', None],
  [ 0xb695, 'HangulSyllableDdyong', [], 'Display', None],
  [ 0xb696, 'HangulSyllableDdyoj', [], 'Display', None],
  [ 0xb697, 'HangulSyllableDdyoc', [], 'Display', None],
  [ 0xb698, 'HangulSyllableDdyok', [], 'Display', None],
  [ 0xb699, 'HangulSyllableDdyot', [], 'Display', None],
  [ 0xb69a, 'HangulSyllableDdyop', [], 'Display', None],
  [ 0xb69b, 'HangulSyllableDdyoh', [], 'Display', None],
  [ 0xb69c, 'HangulSyllableDdu', [], 'Display', None],
  [ 0xb69d, 'HangulSyllableDdug', [], 'Display', None],
  [ 0xb69e, 'HangulSyllableDdugg', [], 'Display', None],
  [ 0xb69f, 'HangulSyllableDdugs', [], 'Display', None],
  [ 0xb6a0, 'HangulSyllableDdun', [], 'Display', None],
  [ 0xb6a1, 'HangulSyllableDdunj', [], 'Display', None],
  [ 0xb6a2, 'HangulSyllableDdunh', [], 'Display', None],
  [ 0xb6a3, 'HangulSyllableDdud', [], 'Display', None],
  [ 0xb6a4, 'HangulSyllableDdul', [], 'Display', None],
  [ 0xb6a5, 'HangulSyllableDdulg', [], 'Display', None],
  [ 0xb6a6, 'HangulSyllableDdulm', [], 'Display', None],
  [ 0xb6a7, 'HangulSyllableDdulb', [], 'Display', None],
  [ 0xb6a8, 'HangulSyllableDduls', [], 'Display', None],
  [ 0xb6a9, 'HangulSyllableDdult', [], 'Display', None],
  [ 0xb6aa, 'HangulSyllableDdulp', [], 'Display', None],
  [ 0xb6ab, 'HangulSyllableDdulh', [], 'Display', None],
  [ 0xb6ac, 'HangulSyllableDdum', [], 'Display', None],
  [ 0xb6ad, 'HangulSyllableDdub', [], 'Display', None],
  [ 0xb6ae, 'HangulSyllableDdubs', [], 'Display', None],
  [ 0xb6af, 'HangulSyllableDdus', [], 'Display', None],
  [ 0xb6b0, 'HangulSyllableDduss', [], 'Display', None],
  [ 0xb6b1, 'HangulSyllableDdung', [], 'Display', None],
  [ 0xb6b2, 'HangulSyllableDduj', [], 'Display', None],
  [ 0xb6b3, 'HangulSyllableDduc', [], 'Display', None],
  [ 0xb6b4, 'HangulSyllableDduk', [], 'Display', None],
  [ 0xb6b5, 'HangulSyllableDdut', [], 'Display', None],
  [ 0xb6b6, 'HangulSyllableDdup', [], 'Display', None],
  [ 0xb6b7, 'HangulSyllableDduh', [], 'Display', None],
  [ 0xb6b8, 'HangulSyllableDdweo', [], 'Display', None],
  [ 0xb6b9, 'HangulSyllableDdweog', [], 'Display', None],
  [ 0xb6ba, 'HangulSyllableDdweogg', [], 'Display', None],
  [ 0xb6bb, 'HangulSyllableDdweogs', [], 'Display', None],
  [ 0xb6bc, 'HangulSyllableDdweon', [], 'Display', None],
  [ 0xb6bd, 'HangulSyllableDdweonj', [], 'Display', None],
  [ 0xb6be, 'HangulSyllableDdweonh', [], 'Display', None],
  [ 0xb6bf, 'HangulSyllableDdweod', [], 'Display', None],
  [ 0xb6c0, 'HangulSyllableDdweol', [], 'Display', None],
  [ 0xb6c1, 'HangulSyllableDdweolg', [], 'Display', None],
  [ 0xb6c2, 'HangulSyllableDdweolm', [], 'Display', None],
  [ 0xb6c3, 'HangulSyllableDdweolb', [], 'Display', None],
  [ 0xb6c4, 'HangulSyllableDdweols', [], 'Display', None],
  [ 0xb6c5, 'HangulSyllableDdweolt', [], 'Display', None],
  [ 0xb6c6, 'HangulSyllableDdweolp', [], 'Display', None],
  [ 0xb6c7, 'HangulSyllableDdweolh', [], 'Display', None],
  [ 0xb6c8, 'HangulSyllableDdweom', [], 'Display', None],
  [ 0xb6c9, 'HangulSyllableDdweob', [], 'Display', None],
  [ 0xb6ca, 'HangulSyllableDdweobs', [], 'Display', None],
  [ 0xb6cb, 'HangulSyllableDdweos', [], 'Display', None],
  [ 0xb6cc, 'HangulSyllableDdweoss', [], 'Display', None],
  [ 0xb6cd, 'HangulSyllableDdweong', [], 'Display', None],
  [ 0xb6ce, 'HangulSyllableDdweoj', [], 'Display', None],
  [ 0xb6cf, 'HangulSyllableDdweoc', [], 'Display', None],
  [ 0xb6d0, 'HangulSyllableDdweok', [], 'Display', None],
  [ 0xb6d1, 'HangulSyllableDdweot', [], 'Display', None],
  [ 0xb6d2, 'HangulSyllableDdweop', [], 'Display', None],
  [ 0xb6d3, 'HangulSyllableDdweoh', [], 'Display', None],
  [ 0xb6d4, 'HangulSyllableDdwe', [], 'Display', None],
  [ 0xb6d5, 'HangulSyllableDdweg', [], 'Display', None],
  [ 0xb6d6, 'HangulSyllableDdwegg', [], 'Display', None],
  [ 0xb6d7, 'HangulSyllableDdwegs', [], 'Display', None],
  [ 0xb6d8, 'HangulSyllableDdwen', [], 'Display', None],
  [ 0xb6d9, 'HangulSyllableDdwenj', [], 'Display', None],
  [ 0xb6da, 'HangulSyllableDdwenh', [], 'Display', None],
  [ 0xb6db, 'HangulSyllableDdwed', [], 'Display', None],
  [ 0xb6dc, 'HangulSyllableDdwel', [], 'Display', None],
  [ 0xb6dd, 'HangulSyllableDdwelg', [], 'Display', None],
  [ 0xb6de, 'HangulSyllableDdwelm', [], 'Display', None],
  [ 0xb6df, 'HangulSyllableDdwelb', [], 'Display', None],
  [ 0xb6e0, 'HangulSyllableDdwels', [], 'Display', None],
  [ 0xb6e1, 'HangulSyllableDdwelt', [], 'Display', None],
  [ 0xb6e2, 'HangulSyllableDdwelp', [], 'Display', None],
  [ 0xb6e3, 'HangulSyllableDdwelh', [], 'Display', None],
  [ 0xb6e4, 'HangulSyllableDdwem', [], 'Display', None],
  [ 0xb6e5, 'HangulSyllableDdweb', [], 'Display', None],
  [ 0xb6e6, 'HangulSyllableDdwebs', [], 'Display', None],
  [ 0xb6e7, 'HangulSyllableDdwes', [], 'Display', None],
  [ 0xb6e8, 'HangulSyllableDdwess', [], 'Display', None],
  [ 0xb6e9, 'HangulSyllableDdweng', [], 'Display', None],
  [ 0xb6ea, 'HangulSyllableDdwej', [], 'Display', None],
  [ 0xb6eb, 'HangulSyllableDdwec', [], 'Display', None],
  [ 0xb6ec, 'HangulSyllableDdwek', [], 'Display', None],
  [ 0xb6ed, 'HangulSyllableDdwet', [], 'Display', None],
  [ 0xb6ee, 'HangulSyllableDdwep', [], 'Display', None],
  [ 0xb6ef, 'HangulSyllableDdweh', [], 'Display', None],
  [ 0xb6f0, 'HangulSyllableDdwi', [], 'Display', None],
  [ 0xb6f1, 'HangulSyllableDdwig', [], 'Display', None],
  [ 0xb6f2, 'HangulSyllableDdwigg', [], 'Display', None],
  [ 0xb6f3, 'HangulSyllableDdwigs', [], 'Display', None],
  [ 0xb6f4, 'HangulSyllableDdwin', [], 'Display', None],
  [ 0xb6f5, 'HangulSyllableDdwinj', [], 'Display', None],
  [ 0xb6f6, 'HangulSyllableDdwinh', [], 'Display', None],
  [ 0xb6f7, 'HangulSyllableDdwid', [], 'Display', None],
  [ 0xb6f8, 'HangulSyllableDdwil', [], 'Display', None],
  [ 0xb6f9, 'HangulSyllableDdwilg', [], 'Display', None],
  [ 0xb6fa, 'HangulSyllableDdwilm', [], 'Display', None],
  [ 0xb6fb, 'HangulSyllableDdwilb', [], 'Display', None],
  [ 0xb6fc, 'HangulSyllableDdwils', [], 'Display', None],
  [ 0xb6fd, 'HangulSyllableDdwilt', [], 'Display', None],
  [ 0xb6fe, 'HangulSyllableDdwilp', [], 'Display', None],
  [ 0xb6ff, 'HangulSyllableDdwilh', [], 'Display', None],
  [ 0xb700, 'HangulSyllableDdwim', [], 'Display', None],
  [ 0xb701, 'HangulSyllableDdwib', [], 'Display', None],
  [ 0xb702, 'HangulSyllableDdwibs', [], 'Display', None],
  [ 0xb703, 'HangulSyllableDdwis', [], 'Display', None],
  [ 0xb704, 'HangulSyllableDdwiss', [], 'Display', None],
  [ 0xb705, 'HangulSyllableDdwing', [], 'Display', None],
  [ 0xb706, 'HangulSyllableDdwij', [], 'Display', None],
  [ 0xb707, 'HangulSyllableDdwic', [], 'Display', None],
  [ 0xb708, 'HangulSyllableDdwik', [], 'Display', None],
  [ 0xb709, 'HangulSyllableDdwit', [], 'Display', None],
  [ 0xb70a, 'HangulSyllableDdwip', [], 'Display', None],
  [ 0xb70b, 'HangulSyllableDdwih', [], 'Display', None],
  [ 0xb70c, 'HangulSyllableDdyu', [], 'Display', None],
  [ 0xb70d, 'HangulSyllableDdyug', [], 'Display', None],
  [ 0xb70e, 'HangulSyllableDdyugg', [], 'Display', None],
  [ 0xb70f, 'HangulSyllableDdyugs', [], 'Display', None],
  [ 0xb710, 'HangulSyllableDdyun', [], 'Display', None],
  [ 0xb711, 'HangulSyllableDdyunj', [], 'Display', None],
  [ 0xb712, 'HangulSyllableDdyunh', [], 'Display', None],
  [ 0xb713, 'HangulSyllableDdyud', [], 'Display', None],
  [ 0xb714, 'HangulSyllableDdyul', [], 'Display', None],
  [ 0xb715, 'HangulSyllableDdyulg', [], 'Display', None],
  [ 0xb716, 'HangulSyllableDdyulm', [], 'Display', None],
  [ 0xb717, 'HangulSyllableDdyulb', [], 'Display', None],
  [ 0xb718, 'HangulSyllableDdyuls', [], 'Display', None],
  [ 0xb719, 'HangulSyllableDdyult', [], 'Display', None],
  [ 0xb71a, 'HangulSyllableDdyulp', [], 'Display', None],
  [ 0xb71b, 'HangulSyllableDdyulh', [], 'Display', None],
  [ 0xb71c, 'HangulSyllableDdyum', [], 'Display', None],
  [ 0xb71d, 'HangulSyllableDdyub', [], 'Display', None],
  [ 0xb71e, 'HangulSyllableDdyubs', [], 'Display', None],
  [ 0xb71f, 'HangulSyllableDdyus', [], 'Display', None],
  [ 0xb720, 'HangulSyllableDdyuss', [], 'Display', None],
  [ 0xb721, 'HangulSyllableDdyung', [], 'Display', None],
  [ 0xb722, 'HangulSyllableDdyuj', [], 'Display', None],
  [ 0xb723, 'HangulSyllableDdyuc', [], 'Display', None],
  [ 0xb724, 'HangulSyllableDdyuk', [], 'Display', None],
  [ 0xb725, 'HangulSyllableDdyut', [], 'Display', None],
  [ 0xb726, 'HangulSyllableDdyup', [], 'Display', None],
  [ 0xb727, 'HangulSyllableDdyuh', [], 'Display', None],
  [ 0xb728, 'HangulSyllableDdeu', [], 'Display', None],
  [ 0xb729, 'HangulSyllableDdeug', [], 'Display', None],
  [ 0xb72a, 'HangulSyllableDdeugg', [], 'Display', None],
  [ 0xb72b, 'HangulSyllableDdeugs', [], 'Display', None],
  [ 0xb72c, 'HangulSyllableDdeun', [], 'Display', None],
  [ 0xb72d, 'HangulSyllableDdeunj', [], 'Display', None],
  [ 0xb72e, 'HangulSyllableDdeunh', [], 'Display', None],
  [ 0xb72f, 'HangulSyllableDdeud', [], 'Display', None],
  [ 0xb730, 'HangulSyllableDdeul', [], 'Display', None],
  [ 0xb731, 'HangulSyllableDdeulg', [], 'Display', None],
  [ 0xb732, 'HangulSyllableDdeulm', [], 'Display', None],
  [ 0xb733, 'HangulSyllableDdeulb', [], 'Display', None],
  [ 0xb734, 'HangulSyllableDdeuls', [], 'Display', None],
  [ 0xb735, 'HangulSyllableDdeult', [], 'Display', None],
  [ 0xb736, 'HangulSyllableDdeulp', [], 'Display', None],
  [ 0xb737, 'HangulSyllableDdeulh', [], 'Display', None],
  [ 0xb738, 'HangulSyllableDdeum', [], 'Display', None],
  [ 0xb739, 'HangulSyllableDdeub', [], 'Display', None],
  [ 0xb73a, 'HangulSyllableDdeubs', [], 'Display', None],
  [ 0xb73b, 'HangulSyllableDdeus', [], 'Display', None],
  [ 0xb73c, 'HangulSyllableDdeuss', [], 'Display', None],
  [ 0xb73d, 'HangulSyllableDdeung', [], 'Display', None],
  [ 0xb73e, 'HangulSyllableDdeuj', [], 'Display', None],
  [ 0xb73f, 'HangulSyllableDdeuc', [], 'Display', None],
  [ 0xb740, 'HangulSyllableDdeuk', [], 'Display', None],
  [ 0xb741, 'HangulSyllableDdeut', [], 'Display', None],
  [ 0xb742, 'HangulSyllableDdeup', [], 'Display', None],
  [ 0xb743, 'HangulSyllableDdeuh', [], 'Display', None],
  [ 0xb744, 'HangulSyllableDdyi', [], 'Display', None],
  [ 0xb745, 'HangulSyllableDdyig', [], 'Display', None],
  [ 0xb746, 'HangulSyllableDdyigg', [], 'Display', None],
  [ 0xb747, 'HangulSyllableDdyigs', [], 'Display', None],
  [ 0xb748, 'HangulSyllableDdyin', [], 'Display', None],
  [ 0xb749, 'HangulSyllableDdyinj', [], 'Display', None],
  [ 0xb74a, 'HangulSyllableDdyinh', [], 'Display', None],
  [ 0xb74b, 'HangulSyllableDdyid', [], 'Display', None],
  [ 0xb74c, 'HangulSyllableDdyil', [], 'Display', None],
  [ 0xb74d, 'HangulSyllableDdyilg', [], 'Display', None],
  [ 0xb74e, 'HangulSyllableDdyilm', [], 'Display', None],
  [ 0xb74f, 'HangulSyllableDdyilb', [], 'Display', None],
  [ 0xb750, 'HangulSyllableDdyils', [], 'Display', None],
  [ 0xb751, 'HangulSyllableDdyilt', [], 'Display', None],
  [ 0xb752, 'HangulSyllableDdyilp', [], 'Display', None],
  [ 0xb753, 'HangulSyllableDdyilh', [], 'Display', None],
  [ 0xb754, 'HangulSyllableDdyim', [], 'Display', None],
  [ 0xb755, 'HangulSyllableDdyib', [], 'Display', None],
  [ 0xb756, 'HangulSyllableDdyibs', [], 'Display', None],
  [ 0xb757, 'HangulSyllableDdyis', [], 'Display', None],
  [ 0xb758, 'HangulSyllableDdyiss', [], 'Display', None],
  [ 0xb759, 'HangulSyllableDdying', [], 'Display', None],
  [ 0xb75a, 'HangulSyllableDdyij', [], 'Display', None],
  [ 0xb75b, 'HangulSyllableDdyic', [], 'Display', None],
  [ 0xb75c, 'HangulSyllableDdyik', [], 'Display', None],
  [ 0xb75d, 'HangulSyllableDdyit', [], 'Display', None],
  [ 0xb75e, 'HangulSyllableDdyip', [], 'Display', None],
  [ 0xb75f, 'HangulSyllableDdyih', [], 'Display', None],
  [ 0xb760, 'HangulSyllableDdi', [], 'Display', None],
  [ 0xb761, 'HangulSyllableDdig', [], 'Display', None],
  [ 0xb762, 'HangulSyllableDdigg', [], 'Display', None],
  [ 0xb763, 'HangulSyllableDdigs', [], 'Display', None],
  [ 0xb764, 'HangulSyllableDdin', [], 'Display', None],
  [ 0xb765, 'HangulSyllableDdinj', [], 'Display', None],
  [ 0xb766, 'HangulSyllableDdinh', [], 'Display', None],
  [ 0xb767, 'HangulSyllableDdid', [], 'Display', None],
  [ 0xb768, 'HangulSyllableDdil', [], 'Display', None],
  [ 0xb769, 'HangulSyllableDdilg', [], 'Display', None],
  [ 0xb76a, 'HangulSyllableDdilm', [], 'Display', None],
  [ 0xb76b, 'HangulSyllableDdilb', [], 'Display', None],
  [ 0xb76c, 'HangulSyllableDdils', [], 'Display', None],
  [ 0xb76d, 'HangulSyllableDdilt', [], 'Display', None],
  [ 0xb76e, 'HangulSyllableDdilp', [], 'Display', None],
  [ 0xb76f, 'HangulSyllableDdilh', [], 'Display', None],
  [ 0xb770, 'HangulSyllableDdim', [], 'Display', None],
  [ 0xb771, 'HangulSyllableDdib', [], 'Display', None],
  [ 0xb772, 'HangulSyllableDdibs', [], 'Display', None],
  [ 0xb773, 'HangulSyllableDdis', [], 'Display', None],
  [ 0xb774, 'HangulSyllableDdiss', [], 'Display', None],
  [ 0xb775, 'HangulSyllableDding', [], 'Display', None],
  [ 0xb776, 'HangulSyllableDdij', [], 'Display', None],
  [ 0xb777, 'HangulSyllableDdic', [], 'Display', None],
  [ 0xb778, 'HangulSyllableDdik', [], 'Display', None],
  [ 0xb779, 'HangulSyllableDdit', [], 'Display', None],
  [ 0xb77a, 'HangulSyllableDdip', [], 'Display', None],
  [ 0xb77b, 'HangulSyllableDdih', [], 'Display', None],
  [ 0xb77c, 'HangulSyllableRa', [], 'Display', None],
  [ 0xb77d, 'HangulSyllableRag', [], 'Display', None],
  [ 0xb77e, 'HangulSyllableRagg', [], 'Display', None],
  [ 0xb77f, 'HangulSyllableRags', [], 'Display', None],
  [ 0xb780, 'HangulSyllableRan', [], 'Display', None],
  [ 0xb781, 'HangulSyllableRanj', [], 'Display', None],
  [ 0xb782, 'HangulSyllableRanh', [], 'Display', None],
  [ 0xb783, 'HangulSyllableRad', [], 'Display', None],
  [ 0xb784, 'HangulSyllableRal', [], 'Display', None],
  [ 0xb785, 'HangulSyllableRalg', [], 'Display', None],
  [ 0xb786, 'HangulSyllableRalm', [], 'Display', None],
  [ 0xb787, 'HangulSyllableRalb', [], 'Display', None],
  [ 0xb788, 'HangulSyllableRals', [], 'Display', None],
  [ 0xb789, 'HangulSyllableRalt', [], 'Display', None],
  [ 0xb78a, 'HangulSyllableRalp', [], 'Display', None],
  [ 0xb78b, 'HangulSyllableRalh', [], 'Display', None],
  [ 0xb78c, 'HangulSyllableRam', [], 'Display', None],
  [ 0xb78d, 'HangulSyllableRab', [], 'Display', None],
  [ 0xb78e, 'HangulSyllableRabs', [], 'Display', None],
  [ 0xb78f, 'HangulSyllableRas', [], 'Display', None],
  [ 0xb790, 'HangulSyllableRass', [], 'Display', None],
  [ 0xb791, 'HangulSyllableRang', [], 'Display', None],
  [ 0xb792, 'HangulSyllableRaj', [], 'Display', None],
  [ 0xb793, 'HangulSyllableRac', [], 'Display', None],
  [ 0xb794, 'HangulSyllableRak', [], 'Display', None],
  [ 0xb795, 'HangulSyllableRat', [], 'Display', None],
  [ 0xb796, 'HangulSyllableRap', [], 'Display', None],
  [ 0xb797, 'HangulSyllableRah', [], 'Display', None],
  [ 0xb798, 'HangulSyllableRae', [], 'Display', None],
  [ 0xb799, 'HangulSyllableRaeg', [], 'Display', None],
  [ 0xb79a, 'HangulSyllableRaegg', [], 'Display', None],
  [ 0xb79b, 'HangulSyllableRaegs', [], 'Display', None],
  [ 0xb79c, 'HangulSyllableRaen', [], 'Display', None],
  [ 0xb79d, 'HangulSyllableRaenj', [], 'Display', None],
  [ 0xb79e, 'HangulSyllableRaenh', [], 'Display', None],
  [ 0xb79f, 'HangulSyllableRaed', [], 'Display', None],
  [ 0xb7a0, 'HangulSyllableRael', [], 'Display', None],
  [ 0xb7a1, 'HangulSyllableRaelg', [], 'Display', None],
  [ 0xb7a2, 'HangulSyllableRaelm', [], 'Display', None],
  [ 0xb7a3, 'HangulSyllableRaelb', [], 'Display', None],
  [ 0xb7a4, 'HangulSyllableRaels', [], 'Display', None],
  [ 0xb7a5, 'HangulSyllableRaelt', [], 'Display', None],
  [ 0xb7a6, 'HangulSyllableRaelp', [], 'Display', None],
  [ 0xb7a7, 'HangulSyllableRaelh', [], 'Display', None],
  [ 0xb7a8, 'HangulSyllableRaem', [], 'Display', None],
  [ 0xb7a9, 'HangulSyllableRaeb', [], 'Display', None],
  [ 0xb7aa, 'HangulSyllableRaebs', [], 'Display', None],
  [ 0xb7ab, 'HangulSyllableRaes', [], 'Display', None],
  [ 0xb7ac, 'HangulSyllableRaess', [], 'Display', None],
  [ 0xb7ad, 'HangulSyllableRaeng', [], 'Display', None],
  [ 0xb7ae, 'HangulSyllableRaej', [], 'Display', None],
  [ 0xb7af, 'HangulSyllableRaec', [], 'Display', None],
  [ 0xb7b0, 'HangulSyllableRaek', [], 'Display', None],
  [ 0xb7b1, 'HangulSyllableRaet', [], 'Display', None],
  [ 0xb7b2, 'HangulSyllableRaep', [], 'Display', None],
  [ 0xb7b3, 'HangulSyllableRaeh', [], 'Display', None],
  [ 0xb7b4, 'HangulSyllableRya', [], 'Display', None],
  [ 0xb7b5, 'HangulSyllableRyag', [], 'Display', None],
  [ 0xb7b6, 'HangulSyllableRyagg', [], 'Display', None],
  [ 0xb7b7, 'HangulSyllableRyags', [], 'Display', None],
  [ 0xb7b8, 'HangulSyllableRyan', [], 'Display', None],
  [ 0xb7b9, 'HangulSyllableRyanj', [], 'Display', None],
  [ 0xb7ba, 'HangulSyllableRyanh', [], 'Display', None],
  [ 0xb7bb, 'HangulSyllableRyad', [], 'Display', None],
  [ 0xb7bc, 'HangulSyllableRyal', [], 'Display', None],
  [ 0xb7bd, 'HangulSyllableRyalg', [], 'Display', None],
  [ 0xb7be, 'HangulSyllableRyalm', [], 'Display', None],
  [ 0xb7bf, 'HangulSyllableRyalb', [], 'Display', None],
  [ 0xb7c0, 'HangulSyllableRyals', [], 'Display', None],
  [ 0xb7c1, 'HangulSyllableRyalt', [], 'Display', None],
  [ 0xb7c2, 'HangulSyllableRyalp', [], 'Display', None],
  [ 0xb7c3, 'HangulSyllableRyalh', [], 'Display', None],
  [ 0xb7c4, 'HangulSyllableRyam', [], 'Display', None],
  [ 0xb7c5, 'HangulSyllableRyab', [], 'Display', None],
  [ 0xb7c6, 'HangulSyllableRyabs', [], 'Display', None],
  [ 0xb7c7, 'HangulSyllableRyas', [], 'Display', None],
  [ 0xb7c8, 'HangulSyllableRyass', [], 'Display', None],
  [ 0xb7c9, 'HangulSyllableRyang', [], 'Display', None],
  [ 0xb7ca, 'HangulSyllableRyaj', [], 'Display', None],
  [ 0xb7cb, 'HangulSyllableRyac', [], 'Display', None],
  [ 0xb7cc, 'HangulSyllableRyak', [], 'Display', None],
  [ 0xb7cd, 'HangulSyllableRyat', [], 'Display', None],
  [ 0xb7ce, 'HangulSyllableRyap', [], 'Display', None],
  [ 0xb7cf, 'HangulSyllableRyah', [], 'Display', None],
  [ 0xb7d0, 'HangulSyllableRyae', [], 'Display', None],
  [ 0xb7d1, 'HangulSyllableRyaeg', [], 'Display', None],
  [ 0xb7d2, 'HangulSyllableRyaegg', [], 'Display', None],
  [ 0xb7d3, 'HangulSyllableRyaegs', [], 'Display', None],
  [ 0xb7d4, 'HangulSyllableRyaen', [], 'Display', None],
  [ 0xb7d5, 'HangulSyllableRyaenj', [], 'Display', None],
  [ 0xb7d6, 'HangulSyllableRyaenh', [], 'Display', None],
  [ 0xb7d7, 'HangulSyllableRyaed', [], 'Display', None],
  [ 0xb7d8, 'HangulSyllableRyael', [], 'Display', None],
  [ 0xb7d9, 'HangulSyllableRyaelg', [], 'Display', None],
  [ 0xb7da, 'HangulSyllableRyaelm', [], 'Display', None],
  [ 0xb7db, 'HangulSyllableRyaelb', [], 'Display', None],
  [ 0xb7dc, 'HangulSyllableRyaels', [], 'Display', None],
  [ 0xb7dd, 'HangulSyllableRyaelt', [], 'Display', None],
  [ 0xb7de, 'HangulSyllableRyaelp', [], 'Display', None],
  [ 0xb7df, 'HangulSyllableRyaelh', [], 'Display', None],
  [ 0xb7e0, 'HangulSyllableRyaem', [], 'Display', None],
  [ 0xb7e1, 'HangulSyllableRyaeb', [], 'Display', None],
  [ 0xb7e2, 'HangulSyllableRyaebs', [], 'Display', None],
  [ 0xb7e3, 'HangulSyllableRyaes', [], 'Display', None],
  [ 0xb7e4, 'HangulSyllableRyaess', [], 'Display', None],
  [ 0xb7e5, 'HangulSyllableRyaeng', [], 'Display', None],
  [ 0xb7e6, 'HangulSyllableRyaej', [], 'Display', None],
  [ 0xb7e7, 'HangulSyllableRyaec', [], 'Display', None],
  [ 0xb7e8, 'HangulSyllableRyaek', [], 'Display', None],
  [ 0xb7e9, 'HangulSyllableRyaet', [], 'Display', None],
  [ 0xb7ea, 'HangulSyllableRyaep', [], 'Display', None],
  [ 0xb7eb, 'HangulSyllableRyaeh', [], 'Display', None],
  [ 0xb7ec, 'HangulSyllableReo', [], 'Display', None],
  [ 0xb7ed, 'HangulSyllableReog', [], 'Display', None],
  [ 0xb7ee, 'HangulSyllableReogg', [], 'Display', None],
  [ 0xb7ef, 'HangulSyllableReogs', [], 'Display', None],
  [ 0xb7f0, 'HangulSyllableReon', [], 'Display', None],
  [ 0xb7f1, 'HangulSyllableReonj', [], 'Display', None],
  [ 0xb7f2, 'HangulSyllableReonh', [], 'Display', None],
  [ 0xb7f3, 'HangulSyllableReod', [], 'Display', None],
  [ 0xb7f4, 'HangulSyllableReol', [], 'Display', None],
  [ 0xb7f5, 'HangulSyllableReolg', [], 'Display', None],
  [ 0xb7f6, 'HangulSyllableReolm', [], 'Display', None],
  [ 0xb7f7, 'HangulSyllableReolb', [], 'Display', None],
  [ 0xb7f8, 'HangulSyllableReols', [], 'Display', None],
  [ 0xb7f9, 'HangulSyllableReolt', [], 'Display', None],
  [ 0xb7fa, 'HangulSyllableReolp', [], 'Display', None],
  [ 0xb7fb, 'HangulSyllableReolh', [], 'Display', None],
  [ 0xb7fc, 'HangulSyllableReom', [], 'Display', None],
  [ 0xb7fd, 'HangulSyllableReob', [], 'Display', None],
  [ 0xb7fe, 'HangulSyllableReobs', [], 'Display', None],
  [ 0xb7ff, 'HangulSyllableReos', [], 'Display', None],
  [ 0xb800, 'HangulSyllableReoss', [], 'Display', None],
  [ 0xb801, 'HangulSyllableReong', [], 'Display', None],
  [ 0xb802, 'HangulSyllableReoj', [], 'Display', None],
  [ 0xb803, 'HangulSyllableReoc', [], 'Display', None],
  [ 0xb804, 'HangulSyllableReok', [], 'Display', None],
  [ 0xb805, 'HangulSyllableReot', [], 'Display', None],
  [ 0xb806, 'HangulSyllableReop', [], 'Display', None],
  [ 0xb807, 'HangulSyllableReoh', [], 'Display', None],
  [ 0xb808, 'HangulSyllableRe', [], 'Display', None],
  [ 0xb809, 'HangulSyllableReg', [], 'Display', None],
  [ 0xb80a, 'HangulSyllableRegg', [], 'Display', None],
  [ 0xb80b, 'HangulSyllableRegs', [], 'Display', None],
  [ 0xb80c, 'HangulSyllableRen', [], 'Display', None],
  [ 0xb80d, 'HangulSyllableRenj', [], 'Display', None],
  [ 0xb80e, 'HangulSyllableRenh', [], 'Display', None],
  [ 0xb80f, 'HangulSyllableRed', [], 'Display', None],
  [ 0xb810, 'HangulSyllableRel', [], 'Display', None],
  [ 0xb811, 'HangulSyllableRelg', [], 'Display', None],
  [ 0xb812, 'HangulSyllableRelm', [], 'Display', None],
  [ 0xb813, 'HangulSyllableRelb', [], 'Display', None],
  [ 0xb814, 'HangulSyllableRels', [], 'Display', None],
  [ 0xb815, 'HangulSyllableRelt', [], 'Display', None],
  [ 0xb816, 'HangulSyllableRelp', [], 'Display', None],
  [ 0xb817, 'HangulSyllableRelh', [], 'Display', None],
  [ 0xb818, 'HangulSyllableRem', [], 'Display', None],
  [ 0xb819, 'HangulSyllableReb', [], 'Display', None],
  [ 0xb81a, 'HangulSyllableRebs', [], 'Display', None],
  [ 0xb81b, 'HangulSyllableRes', [], 'Display', None],
  [ 0xb81c, 'HangulSyllableRess', [], 'Display', None],
  [ 0xb81d, 'HangulSyllableReng', [], 'Display', None],
  [ 0xb81e, 'HangulSyllableRej', [], 'Display', None],
  [ 0xb81f, 'HangulSyllableRec', [], 'Display', None],
  [ 0xb820, 'HangulSyllableRek', [], 'Display', None],
  [ 0xb821, 'HangulSyllableRet', [], 'Display', None],
  [ 0xb822, 'HangulSyllableRep', [], 'Display', None],
  [ 0xb823, 'HangulSyllableReh', [], 'Display', None],
  [ 0xb824, 'HangulSyllableRyeo', [], 'Display', None],
  [ 0xb825, 'HangulSyllableRyeog', [], 'Display', None],
  [ 0xb826, 'HangulSyllableRyeogg', [], 'Display', None],
  [ 0xb827, 'HangulSyllableRyeogs', [], 'Display', None],
  [ 0xb828, 'HangulSyllableRyeon', [], 'Display', None],
  [ 0xb829, 'HangulSyllableRyeonj', [], 'Display', None],
  [ 0xb82a, 'HangulSyllableRyeonh', [], 'Display', None],
  [ 0xb82b, 'HangulSyllableRyeod', [], 'Display', None],
  [ 0xb82c, 'HangulSyllableRyeol', [], 'Display', None],
  [ 0xb82d, 'HangulSyllableRyeolg', [], 'Display', None],
  [ 0xb82e, 'HangulSyllableRyeolm', [], 'Display', None],
  [ 0xb82f, 'HangulSyllableRyeolb', [], 'Display', None],
  [ 0xb830, 'HangulSyllableRyeols', [], 'Display', None],
  [ 0xb831, 'HangulSyllableRyeolt', [], 'Display', None],
  [ 0xb832, 'HangulSyllableRyeolp', [], 'Display', None],
  [ 0xb833, 'HangulSyllableRyeolh', [], 'Display', None],
  [ 0xb834, 'HangulSyllableRyeom', [], 'Display', None],
  [ 0xb835, 'HangulSyllableRyeob', [], 'Display', None],
  [ 0xb836, 'HangulSyllableRyeobs', [], 'Display', None],
  [ 0xb837, 'HangulSyllableRyeos', [], 'Display', None],
  [ 0xb838, 'HangulSyllableRyeoss', [], 'Display', None],
  [ 0xb839, 'HangulSyllableRyeong', [], 'Display', None],
  [ 0xb83a, 'HangulSyllableRyeoj', [], 'Display', None],
  [ 0xb83b, 'HangulSyllableRyeoc', [], 'Display', None],
  [ 0xb83c, 'HangulSyllableRyeok', [], 'Display', None],
  [ 0xb83d, 'HangulSyllableRyeot', [], 'Display', None],
  [ 0xb83e, 'HangulSyllableRyeop', [], 'Display', None],
  [ 0xb83f, 'HangulSyllableRyeoh', [], 'Display', None],
  [ 0xb840, 'HangulSyllableRye', [], 'Display', None],
  [ 0xb841, 'HangulSyllableRyeg', [], 'Display', None],
  [ 0xb842, 'HangulSyllableRyegg', [], 'Display', None],
  [ 0xb843, 'HangulSyllableRyegs', [], 'Display', None],
  [ 0xb844, 'HangulSyllableRyen', [], 'Display', None],
  [ 0xb845, 'HangulSyllableRyenj', [], 'Display', None],
  [ 0xb846, 'HangulSyllableRyenh', [], 'Display', None],
  [ 0xb847, 'HangulSyllableRyed', [], 'Display', None],
  [ 0xb848, 'HangulSyllableRyel', [], 'Display', None],
  [ 0xb849, 'HangulSyllableRyelg', [], 'Display', None],
  [ 0xb84a, 'HangulSyllableRyelm', [], 'Display', None],
  [ 0xb84b, 'HangulSyllableRyelb', [], 'Display', None],
  [ 0xb84c, 'HangulSyllableRyels', [], 'Display', None],
  [ 0xb84d, 'HangulSyllableRyelt', [], 'Display', None],
  [ 0xb84e, 'HangulSyllableRyelp', [], 'Display', None],
  [ 0xb84f, 'HangulSyllableRyelh', [], 'Display', None],
  [ 0xb850, 'HangulSyllableRyem', [], 'Display', None],
  [ 0xb851, 'HangulSyllableRyeb', [], 'Display', None],
  [ 0xb852, 'HangulSyllableRyebs', [], 'Display', None],
  [ 0xb853, 'HangulSyllableRyes', [], 'Display', None],
  [ 0xb854, 'HangulSyllableRyess', [], 'Display', None],
  [ 0xb855, 'HangulSyllableRyeng', [], 'Display', None],
  [ 0xb856, 'HangulSyllableRyej', [], 'Display', None],
  [ 0xb857, 'HangulSyllableRyec', [], 'Display', None],
  [ 0xb858, 'HangulSyllableRyek', [], 'Display', None],
  [ 0xb859, 'HangulSyllableRyet', [], 'Display', None],
  [ 0xb85a, 'HangulSyllableRyep', [], 'Display', None],
  [ 0xb85b, 'HangulSyllableRyeh', [], 'Display', None],
  [ 0xb85c, 'HangulSyllableRo', [], 'Display', None],
  [ 0xb85d, 'HangulSyllableRog', [], 'Display', None],
  [ 0xb85e, 'HangulSyllableRogg', [], 'Display', None],
  [ 0xb85f, 'HangulSyllableRogs', [], 'Display', None],
  [ 0xb860, 'HangulSyllableRon', [], 'Display', None],
  [ 0xb861, 'HangulSyllableRonj', [], 'Display', None],
  [ 0xb862, 'HangulSyllableRonh', [], 'Display', None],
  [ 0xb863, 'HangulSyllableRod', [], 'Display', None],
  [ 0xb864, 'HangulSyllableRol', [], 'Display', None],
  [ 0xb865, 'HangulSyllableRolg', [], 'Display', None],
  [ 0xb866, 'HangulSyllableRolm', [], 'Display', None],
  [ 0xb867, 'HangulSyllableRolb', [], 'Display', None],
  [ 0xb868, 'HangulSyllableRols', [], 'Display', None],
  [ 0xb869, 'HangulSyllableRolt', [], 'Display', None],
  [ 0xb86a, 'HangulSyllableRolp', [], 'Display', None],
  [ 0xb86b, 'HangulSyllableRolh', [], 'Display', None],
  [ 0xb86c, 'HangulSyllableRom', [], 'Display', None],
  [ 0xb86d, 'HangulSyllableRob', [], 'Display', None],
  [ 0xb86e, 'HangulSyllableRobs', [], 'Display', None],
  [ 0xb86f, 'HangulSyllableRos', [], 'Display', None],
  [ 0xb870, 'HangulSyllableRoss', [], 'Display', None],
  [ 0xb871, 'HangulSyllableRong', [], 'Display', None],
  [ 0xb872, 'HangulSyllableRoj', [], 'Display', None],
  [ 0xb873, 'HangulSyllableRoc', [], 'Display', None],
  [ 0xb874, 'HangulSyllableRok', [], 'Display', None],
  [ 0xb875, 'HangulSyllableRot', [], 'Display', None],
  [ 0xb876, 'HangulSyllableRop', [], 'Display', None],
  [ 0xb877, 'HangulSyllableRoh', [], 'Display', None],
  [ 0xb878, 'HangulSyllableRwa', [], 'Display', None],
  [ 0xb879, 'HangulSyllableRwag', [], 'Display', None],
  [ 0xb87a, 'HangulSyllableRwagg', [], 'Display', None],
  [ 0xb87b, 'HangulSyllableRwags', [], 'Display', None],
  [ 0xb87c, 'HangulSyllableRwan', [], 'Display', None],
  [ 0xb87d, 'HangulSyllableRwanj', [], 'Display', None],
  [ 0xb87e, 'HangulSyllableRwanh', [], 'Display', None],
  [ 0xb87f, 'HangulSyllableRwad', [], 'Display', None],
  [ 0xb880, 'HangulSyllableRwal', [], 'Display', None],
  [ 0xb881, 'HangulSyllableRwalg', [], 'Display', None],
  [ 0xb882, 'HangulSyllableRwalm', [], 'Display', None],
  [ 0xb883, 'HangulSyllableRwalb', [], 'Display', None],
  [ 0xb884, 'HangulSyllableRwals', [], 'Display', None],
  [ 0xb885, 'HangulSyllableRwalt', [], 'Display', None],
  [ 0xb886, 'HangulSyllableRwalp', [], 'Display', None],
  [ 0xb887, 'HangulSyllableRwalh', [], 'Display', None],
  [ 0xb888, 'HangulSyllableRwam', [], 'Display', None],
  [ 0xb889, 'HangulSyllableRwab', [], 'Display', None],
  [ 0xb88a, 'HangulSyllableRwabs', [], 'Display', None],
  [ 0xb88b, 'HangulSyllableRwas', [], 'Display', None],
  [ 0xb88c, 'HangulSyllableRwass', [], 'Display', None],
  [ 0xb88d, 'HangulSyllableRwang', [], 'Display', None],
  [ 0xb88e, 'HangulSyllableRwaj', [], 'Display', None],
  [ 0xb88f, 'HangulSyllableRwac', [], 'Display', None],
  [ 0xb890, 'HangulSyllableRwak', [], 'Display', None],
  [ 0xb891, 'HangulSyllableRwat', [], 'Display', None],
  [ 0xb892, 'HangulSyllableRwap', [], 'Display', None],
  [ 0xb893, 'HangulSyllableRwah', [], 'Display', None],
  [ 0xb894, 'HangulSyllableRwae', [], 'Display', None],
  [ 0xb895, 'HangulSyllableRwaeg', [], 'Display', None],
  [ 0xb896, 'HangulSyllableRwaegg', [], 'Display', None],
  [ 0xb897, 'HangulSyllableRwaegs', [], 'Display', None],
  [ 0xb898, 'HangulSyllableRwaen', [], 'Display', None],
  [ 0xb899, 'HangulSyllableRwaenj', [], 'Display', None],
  [ 0xb89a, 'HangulSyllableRwaenh', [], 'Display', None],
  [ 0xb89b, 'HangulSyllableRwaed', [], 'Display', None],
  [ 0xb89c, 'HangulSyllableRwael', [], 'Display', None],
  [ 0xb89d, 'HangulSyllableRwaelg', [], 'Display', None],
  [ 0xb89e, 'HangulSyllableRwaelm', [], 'Display', None],
  [ 0xb89f, 'HangulSyllableRwaelb', [], 'Display', None],
  [ 0xb8a0, 'HangulSyllableRwaels', [], 'Display', None],
  [ 0xb8a1, 'HangulSyllableRwaelt', [], 'Display', None],
  [ 0xb8a2, 'HangulSyllableRwaelp', [], 'Display', None],
  [ 0xb8a3, 'HangulSyllableRwaelh', [], 'Display', None],
  [ 0xb8a4, 'HangulSyllableRwaem', [], 'Display', None],
  [ 0xb8a5, 'HangulSyllableRwaeb', [], 'Display', None],
  [ 0xb8a6, 'HangulSyllableRwaebs', [], 'Display', None],
  [ 0xb8a7, 'HangulSyllableRwaes', [], 'Display', None],
  [ 0xb8a8, 'HangulSyllableRwaess', [], 'Display', None],
  [ 0xb8a9, 'HangulSyllableRwaeng', [], 'Display', None],
  [ 0xb8aa, 'HangulSyllableRwaej', [], 'Display', None],
  [ 0xb8ab, 'HangulSyllableRwaec', [], 'Display', None],
  [ 0xb8ac, 'HangulSyllableRwaek', [], 'Display', None],
  [ 0xb8ad, 'HangulSyllableRwaet', [], 'Display', None],
  [ 0xb8ae, 'HangulSyllableRwaep', [], 'Display', None],
  [ 0xb8af, 'HangulSyllableRwaeh', [], 'Display', None],
  [ 0xb8b0, 'HangulSyllableRoe', [], 'Display', None],
  [ 0xb8b1, 'HangulSyllableRoeg', [], 'Display', None],
  [ 0xb8b2, 'HangulSyllableRoegg', [], 'Display', None],
  [ 0xb8b3, 'HangulSyllableRoegs', [], 'Display', None],
  [ 0xb8b4, 'HangulSyllableRoen', [], 'Display', None],
  [ 0xb8b5, 'HangulSyllableRoenj', [], 'Display', None],
  [ 0xb8b6, 'HangulSyllableRoenh', [], 'Display', None],
  [ 0xb8b7, 'HangulSyllableRoed', [], 'Display', None],
  [ 0xb8b8, 'HangulSyllableRoel', [], 'Display', None],
  [ 0xb8b9, 'HangulSyllableRoelg', [], 'Display', None],
  [ 0xb8ba, 'HangulSyllableRoelm', [], 'Display', None],
  [ 0xb8bb, 'HangulSyllableRoelb', [], 'Display', None],
  [ 0xb8bc, 'HangulSyllableRoels', [], 'Display', None],
  [ 0xb8bd, 'HangulSyllableRoelt', [], 'Display', None],
  [ 0xb8be, 'HangulSyllableRoelp', [], 'Display', None],
  [ 0xb8bf, 'HangulSyllableRoelh', [], 'Display', None],
  [ 0xb8c0, 'HangulSyllableRoem', [], 'Display', None],
  [ 0xb8c1, 'HangulSyllableRoeb', [], 'Display', None],
  [ 0xb8c2, 'HangulSyllableRoebs', [], 'Display', None],
  [ 0xb8c3, 'HangulSyllableRoes', [], 'Display', None],
  [ 0xb8c4, 'HangulSyllableRoess', [], 'Display', None],
  [ 0xb8c5, 'HangulSyllableRoeng', [], 'Display', None],
  [ 0xb8c6, 'HangulSyllableRoej', [], 'Display', None],
  [ 0xb8c7, 'HangulSyllableRoec', [], 'Display', None],
  [ 0xb8c8, 'HangulSyllableRoek', [], 'Display', None],
  [ 0xb8c9, 'HangulSyllableRoet', [], 'Display', None],
  [ 0xb8ca, 'HangulSyllableRoep', [], 'Display', None],
  [ 0xb8cb, 'HangulSyllableRoeh', [], 'Display', None],
  [ 0xb8cc, 'HangulSyllableRyo', [], 'Display', None],
  [ 0xb8cd, 'HangulSyllableRyog', [], 'Display', None],
  [ 0xb8ce, 'HangulSyllableRyogg', [], 'Display', None],
  [ 0xb8cf, 'HangulSyllableRyogs', [], 'Display', None],
  [ 0xb8d0, 'HangulSyllableRyon', [], 'Display', None],
  [ 0xb8d1, 'HangulSyllableRyonj', [], 'Display', None],
  [ 0xb8d2, 'HangulSyllableRyonh', [], 'Display', None],
  [ 0xb8d3, 'HangulSyllableRyod', [], 'Display', None],
  [ 0xb8d4, 'HangulSyllableRyol', [], 'Display', None],
  [ 0xb8d5, 'HangulSyllableRyolg', [], 'Display', None],
  [ 0xb8d6, 'HangulSyllableRyolm', [], 'Display', None],
  [ 0xb8d7, 'HangulSyllableRyolb', [], 'Display', None],
  [ 0xb8d8, 'HangulSyllableRyols', [], 'Display', None],
  [ 0xb8d9, 'HangulSyllableRyolt', [], 'Display', None],
  [ 0xb8da, 'HangulSyllableRyolp', [], 'Display', None],
  [ 0xb8db, 'HangulSyllableRyolh', [], 'Display', None],
  [ 0xb8dc, 'HangulSyllableRyom', [], 'Display', None],
  [ 0xb8dd, 'HangulSyllableRyob', [], 'Display', None],
  [ 0xb8de, 'HangulSyllableRyobs', [], 'Display', None],
  [ 0xb8df, 'HangulSyllableRyos', [], 'Display', None],
  [ 0xb8e0, 'HangulSyllableRyoss', [], 'Display', None],
  [ 0xb8e1, 'HangulSyllableRyong', [], 'Display', None],
  [ 0xb8e2, 'HangulSyllableRyoj', [], 'Display', None],
  [ 0xb8e3, 'HangulSyllableRyoc', [], 'Display', None],
  [ 0xb8e4, 'HangulSyllableRyok', [], 'Display', None],
  [ 0xb8e5, 'HangulSyllableRyot', [], 'Display', None],
  [ 0xb8e6, 'HangulSyllableRyop', [], 'Display', None],
  [ 0xb8e7, 'HangulSyllableRyoh', [], 'Display', None],
  [ 0xb8e8, 'HangulSyllableRu', [], 'Display', None],
  [ 0xb8e9, 'HangulSyllableRug', [], 'Display', None],
  [ 0xb8ea, 'HangulSyllableRugg', [], 'Display', None],
  [ 0xb8eb, 'HangulSyllableRugs', [], 'Display', None],
  [ 0xb8ec, 'HangulSyllableRun', [], 'Display', None],
  [ 0xb8ed, 'HangulSyllableRunj', [], 'Display', None],
  [ 0xb8ee, 'HangulSyllableRunh', [], 'Display', None],
  [ 0xb8ef, 'HangulSyllableRud', [], 'Display', None],
  [ 0xb8f0, 'HangulSyllableRul', [], 'Display', None],
  [ 0xb8f1, 'HangulSyllableRulg', [], 'Display', None],
  [ 0xb8f2, 'HangulSyllableRulm', [], 'Display', None],
  [ 0xb8f3, 'HangulSyllableRulb', [], 'Display', None],
  [ 0xb8f4, 'HangulSyllableRuls', [], 'Display', None],
  [ 0xb8f5, 'HangulSyllableRult', [], 'Display', None],
  [ 0xb8f6, 'HangulSyllableRulp', [], 'Display', None],
  [ 0xb8f7, 'HangulSyllableRulh', [], 'Display', None],
  [ 0xb8f8, 'HangulSyllableRum', [], 'Display', None],
  [ 0xb8f9, 'HangulSyllableRub', [], 'Display', None],
  [ 0xb8fa, 'HangulSyllableRubs', [], 'Display', None],
  [ 0xb8fb, 'HangulSyllableRus', [], 'Display', None],
  [ 0xb8fc, 'HangulSyllableRuss', [], 'Display', None],
  [ 0xb8fd, 'HangulSyllableRung', [], 'Display', None],
  [ 0xb8fe, 'HangulSyllableRuj', [], 'Display', None],
  [ 0xb8ff, 'HangulSyllableRuc', [], 'Display', None],
  [ 0xb900, 'HangulSyllableRuk', [], 'Display', None],
  [ 0xb901, 'HangulSyllableRut', [], 'Display', None],
  [ 0xb902, 'HangulSyllableRup', [], 'Display', None],
  [ 0xb903, 'HangulSyllableRuh', [], 'Display', None],
  [ 0xb904, 'HangulSyllableRweo', [], 'Display', None],
  [ 0xb905, 'HangulSyllableRweog', [], 'Display', None],
  [ 0xb906, 'HangulSyllableRweogg', [], 'Display', None],
  [ 0xb907, 'HangulSyllableRweogs', [], 'Display', None],
  [ 0xb908, 'HangulSyllableRweon', [], 'Display', None],
  [ 0xb909, 'HangulSyllableRweonj', [], 'Display', None],
  [ 0xb90a, 'HangulSyllableRweonh', [], 'Display', None],
  [ 0xb90b, 'HangulSyllableRweod', [], 'Display', None],
  [ 0xb90c, 'HangulSyllableRweol', [], 'Display', None],
  [ 0xb90d, 'HangulSyllableRweolg', [], 'Display', None],
  [ 0xb90e, 'HangulSyllableRweolm', [], 'Display', None],
  [ 0xb90f, 'HangulSyllableRweolb', [], 'Display', None],
  [ 0xb910, 'HangulSyllableRweols', [], 'Display', None],
  [ 0xb911, 'HangulSyllableRweolt', [], 'Display', None],
  [ 0xb912, 'HangulSyllableRweolp', [], 'Display', None],
  [ 0xb913, 'HangulSyllableRweolh', [], 'Display', None],
  [ 0xb914, 'HangulSyllableRweom', [], 'Display', None],
  [ 0xb915, 'HangulSyllableRweob', [], 'Display', None],
  [ 0xb916, 'HangulSyllableRweobs', [], 'Display', None],
  [ 0xb917, 'HangulSyllableRweos', [], 'Display', None],
  [ 0xb918, 'HangulSyllableRweoss', [], 'Display', None],
  [ 0xb919, 'HangulSyllableRweong', [], 'Display', None],
  [ 0xb91a, 'HangulSyllableRweoj', [], 'Display', None],
  [ 0xb91b, 'HangulSyllableRweoc', [], 'Display', None],
  [ 0xb91c, 'HangulSyllableRweok', [], 'Display', None],
  [ 0xb91d, 'HangulSyllableRweot', [], 'Display', None],
  [ 0xb91e, 'HangulSyllableRweop', [], 'Display', None],
  [ 0xb91f, 'HangulSyllableRweoh', [], 'Display', None],
  [ 0xb920, 'HangulSyllableRwe', [], 'Display', None],
  [ 0xb921, 'HangulSyllableRweg', [], 'Display', None],
  [ 0xb922, 'HangulSyllableRwegg', [], 'Display', None],
  [ 0xb923, 'HangulSyllableRwegs', [], 'Display', None],
  [ 0xb924, 'HangulSyllableRwen', [], 'Display', None],
  [ 0xb925, 'HangulSyllableRwenj', [], 'Display', None],
  [ 0xb926, 'HangulSyllableRwenh', [], 'Display', None],
  [ 0xb927, 'HangulSyllableRwed', [], 'Display', None],
  [ 0xb928, 'HangulSyllableRwel', [], 'Display', None],
  [ 0xb929, 'HangulSyllableRwelg', [], 'Display', None],
  [ 0xb92a, 'HangulSyllableRwelm', [], 'Display', None],
  [ 0xb92b, 'HangulSyllableRwelb', [], 'Display', None],
  [ 0xb92c, 'HangulSyllableRwels', [], 'Display', None],
  [ 0xb92d, 'HangulSyllableRwelt', [], 'Display', None],
  [ 0xb92e, 'HangulSyllableRwelp', [], 'Display', None],
  [ 0xb92f, 'HangulSyllableRwelh', [], 'Display', None],
  [ 0xb930, 'HangulSyllableRwem', [], 'Display', None],
  [ 0xb931, 'HangulSyllableRweb', [], 'Display', None],
  [ 0xb932, 'HangulSyllableRwebs', [], 'Display', None],
  [ 0xb933, 'HangulSyllableRwes', [], 'Display', None],
  [ 0xb934, 'HangulSyllableRwess', [], 'Display', None],
  [ 0xb935, 'HangulSyllableRweng', [], 'Display', None],
  [ 0xb936, 'HangulSyllableRwej', [], 'Display', None],
  [ 0xb937, 'HangulSyllableRwec', [], 'Display', None],
  [ 0xb938, 'HangulSyllableRwek', [], 'Display', None],
  [ 0xb939, 'HangulSyllableRwet', [], 'Display', None],
  [ 0xb93a, 'HangulSyllableRwep', [], 'Display', None],
  [ 0xb93b, 'HangulSyllableRweh', [], 'Display', None],
  [ 0xb93c, 'HangulSyllableRwi', [], 'Display', None],
  [ 0xb93d, 'HangulSyllableRwig', [], 'Display', None],
  [ 0xb93e, 'HangulSyllableRwigg', [], 'Display', None],
  [ 0xb93f, 'HangulSyllableRwigs', [], 'Display', None],
  [ 0xb940, 'HangulSyllableRwin', [], 'Display', None],
  [ 0xb941, 'HangulSyllableRwinj', [], 'Display', None],
  [ 0xb942, 'HangulSyllableRwinh', [], 'Display', None],
  [ 0xb943, 'HangulSyllableRwid', [], 'Display', None],
  [ 0xb944, 'HangulSyllableRwil', [], 'Display', None],
  [ 0xb945, 'HangulSyllableRwilg', [], 'Display', None],
  [ 0xb946, 'HangulSyllableRwilm', [], 'Display', None],
  [ 0xb947, 'HangulSyllableRwilb', [], 'Display', None],
  [ 0xb948, 'HangulSyllableRwils', [], 'Display', None],
  [ 0xb949, 'HangulSyllableRwilt', [], 'Display', None],
  [ 0xb94a, 'HangulSyllableRwilp', [], 'Display', None],
  [ 0xb94b, 'HangulSyllableRwilh', [], 'Display', None],
  [ 0xb94c, 'HangulSyllableRwim', [], 'Display', None],
  [ 0xb94d, 'HangulSyllableRwib', [], 'Display', None],
  [ 0xb94e, 'HangulSyllableRwibs', [], 'Display', None],
  [ 0xb94f, 'HangulSyllableRwis', [], 'Display', None],
  [ 0xb950, 'HangulSyllableRwiss', [], 'Display', None],
  [ 0xb951, 'HangulSyllableRwing', [], 'Display', None],
  [ 0xb952, 'HangulSyllableRwij', [], 'Display', None],
  [ 0xb953, 'HangulSyllableRwic', [], 'Display', None],
  [ 0xb954, 'HangulSyllableRwik', [], 'Display', None],
  [ 0xb955, 'HangulSyllableRwit', [], 'Display', None],
  [ 0xb956, 'HangulSyllableRwip', [], 'Display', None],
  [ 0xb957, 'HangulSyllableRwih', [], 'Display', None],
  [ 0xb958, 'HangulSyllableRyu', [], 'Display', None],
  [ 0xb959, 'HangulSyllableRyug', [], 'Display', None],
  [ 0xb95a, 'HangulSyllableRyugg', [], 'Display', None],
  [ 0xb95b, 'HangulSyllableRyugs', [], 'Display', None],
  [ 0xb95c, 'HangulSyllableRyun', [], 'Display', None],
  [ 0xb95d, 'HangulSyllableRyunj', [], 'Display', None],
  [ 0xb95e, 'HangulSyllableRyunh', [], 'Display', None],
  [ 0xb95f, 'HangulSyllableRyud', [], 'Display', None],
  [ 0xb960, 'HangulSyllableRyul', [], 'Display', None],
  [ 0xb961, 'HangulSyllableRyulg', [], 'Display', None],
  [ 0xb962, 'HangulSyllableRyulm', [], 'Display', None],
  [ 0xb963, 'HangulSyllableRyulb', [], 'Display', None],
  [ 0xb964, 'HangulSyllableRyuls', [], 'Display', None],
  [ 0xb965, 'HangulSyllableRyult', [], 'Display', None],
  [ 0xb966, 'HangulSyllableRyulp', [], 'Display', None],
  [ 0xb967, 'HangulSyllableRyulh', [], 'Display', None],
  [ 0xb968, 'HangulSyllableRyum', [], 'Display', None],
  [ 0xb969, 'HangulSyllableRyub', [], 'Display', None],
  [ 0xb96a, 'HangulSyllableRyubs', [], 'Display', None],
  [ 0xb96b, 'HangulSyllableRyus', [], 'Display', None],
  [ 0xb96c, 'HangulSyllableRyuss', [], 'Display', None],
  [ 0xb96d, 'HangulSyllableRyung', [], 'Display', None],
  [ 0xb96e, 'HangulSyllableRyuj', [], 'Display', None],
  [ 0xb96f, 'HangulSyllableRyuc', [], 'Display', None],
  [ 0xb970, 'HangulSyllableRyuk', [], 'Display', None],
  [ 0xb971, 'HangulSyllableRyut', [], 'Display', None],
  [ 0xb972, 'HangulSyllableRyup', [], 'Display', None],
  [ 0xb973, 'HangulSyllableRyuh', [], 'Display', None],
  [ 0xb974, 'HangulSyllableReu', [], 'Display', None],
  [ 0xb975, 'HangulSyllableReug', [], 'Display', None],
  [ 0xb976, 'HangulSyllableReugg', [], 'Display', None],
  [ 0xb977, 'HangulSyllableReugs', [], 'Display', None],
  [ 0xb978, 'HangulSyllableReun', [], 'Display', None],
  [ 0xb979, 'HangulSyllableReunj', [], 'Display', None],
  [ 0xb97a, 'HangulSyllableReunh', [], 'Display', None],
  [ 0xb97b, 'HangulSyllableReud', [], 'Display', None],
  [ 0xb97c, 'HangulSyllableReul', [], 'Display', None],
  [ 0xb97d, 'HangulSyllableReulg', [], 'Display', None],
  [ 0xb97e, 'HangulSyllableReulm', [], 'Display', None],
  [ 0xb97f, 'HangulSyllableReulb', [], 'Display', None],
  [ 0xb980, 'HangulSyllableReuls', [], 'Display', None],
  [ 0xb981, 'HangulSyllableReult', [], 'Display', None],
  [ 0xb982, 'HangulSyllableReulp', [], 'Display', None],
  [ 0xb983, 'HangulSyllableReulh', [], 'Display', None],
  [ 0xb984, 'HangulSyllableReum', [], 'Display', None],
  [ 0xb985, 'HangulSyllableReub', [], 'Display', None],
  [ 0xb986, 'HangulSyllableReubs', [], 'Display', None],
  [ 0xb987, 'HangulSyllableReus', [], 'Display', None],
  [ 0xb988, 'HangulSyllableReuss', [], 'Display', None],
  [ 0xb989, 'HangulSyllableReung', [], 'Display', None],
  [ 0xb98a, 'HangulSyllableReuj', [], 'Display', None],
  [ 0xb98b, 'HangulSyllableReuc', [], 'Display', None],
  [ 0xb98c, 'HangulSyllableReuk', [], 'Display', None],
  [ 0xb98d, 'HangulSyllableReut', [], 'Display', None],
  [ 0xb98e, 'HangulSyllableReup', [], 'Display', None],
  [ 0xb98f, 'HangulSyllableReuh', [], 'Display', None],
  [ 0xb990, 'HangulSyllableRyi', [], 'Display', None],
  [ 0xb991, 'HangulSyllableRyig', [], 'Display', None],
  [ 0xb992, 'HangulSyllableRyigg', [], 'Display', None],
  [ 0xb993, 'HangulSyllableRyigs', [], 'Display', None],
  [ 0xb994, 'HangulSyllableRyin', [], 'Display', None],
  [ 0xb995, 'HangulSyllableRyinj', [], 'Display', None],
  [ 0xb996, 'HangulSyllableRyinh', [], 'Display', None],
  [ 0xb997, 'HangulSyllableRyid', [], 'Display', None],
  [ 0xb998, 'HangulSyllableRyil', [], 'Display', None],
  [ 0xb999, 'HangulSyllableRyilg', [], 'Display', None],
  [ 0xb99a, 'HangulSyllableRyilm', [], 'Display', None],
  [ 0xb99b, 'HangulSyllableRyilb', [], 'Display', None],
  [ 0xb99c, 'HangulSyllableRyils', [], 'Display', None],
  [ 0xb99d, 'HangulSyllableRyilt', [], 'Display', None],
  [ 0xb99e, 'HangulSyllableRyilp', [], 'Display', None],
  [ 0xb99f, 'HangulSyllableRyilh', [], 'Display', None],
  [ 0xb9a0, 'HangulSyllableRyim', [], 'Display', None],
  [ 0xb9a1, 'HangulSyllableRyib', [], 'Display', None],
  [ 0xb9a2, 'HangulSyllableRyibs', [], 'Display', None],
  [ 0xb9a3, 'HangulSyllableRyis', [], 'Display', None],
  [ 0xb9a4, 'HangulSyllableRyiss', [], 'Display', None],
  [ 0xb9a5, 'HangulSyllableRying', [], 'Display', None],
  [ 0xb9a6, 'HangulSyllableRyij', [], 'Display', None],
  [ 0xb9a7, 'HangulSyllableRyic', [], 'Display', None],
  [ 0xb9a8, 'HangulSyllableRyik', [], 'Display', None],
  [ 0xb9a9, 'HangulSyllableRyit', [], 'Display', None],
  [ 0xb9aa, 'HangulSyllableRyip', [], 'Display', None],
  [ 0xb9ab, 'HangulSyllableRyih', [], 'Display', None],
  [ 0xb9ac, 'HangulSyllableRi', [], 'Display', None],
  [ 0xb9ad, 'HangulSyllableRig', [], 'Display', None],
  [ 0xb9ae, 'HangulSyllableRigg', [], 'Display', None],
  [ 0xb9af, 'HangulSyllableRigs', [], 'Display', None],
  [ 0xb9b0, 'HangulSyllableRin', [], 'Display', None],
  [ 0xb9b1, 'HangulSyllableRinj', [], 'Display', None],
  [ 0xb9b2, 'HangulSyllableRinh', [], 'Display', None],
  [ 0xb9b3, 'HangulSyllableRid', [], 'Display', None],
  [ 0xb9b4, 'HangulSyllableRil', [], 'Display', None],
  [ 0xb9b5, 'HangulSyllableRilg', [], 'Display', None],
  [ 0xb9b6, 'HangulSyllableRilm', [], 'Display', None],
  [ 0xb9b7, 'HangulSyllableRilb', [], 'Display', None],
  [ 0xb9b8, 'HangulSyllableRils', [], 'Display', None],
  [ 0xb9b9, 'HangulSyllableRilt', [], 'Display', None],
  [ 0xb9ba, 'HangulSyllableRilp', [], 'Display', None],
  [ 0xb9bb, 'HangulSyllableRilh', [], 'Display', None],
  [ 0xb9bc, 'HangulSyllableRim', [], 'Display', None],
  [ 0xb9bd, 'HangulSyllableRib', [], 'Display', None],
  [ 0xb9be, 'HangulSyllableRibs', [], 'Display', None],
  [ 0xb9bf, 'HangulSyllableRis', [], 'Display', None],
  [ 0xb9c0, 'HangulSyllableRiss', [], 'Display', None],
  [ 0xb9c1, 'HangulSyllableRing', [], 'Display', None],
  [ 0xb9c2, 'HangulSyllableRij', [], 'Display', None],
  [ 0xb9c3, 'HangulSyllableRic', [], 'Display', None],
  [ 0xb9c4, 'HangulSyllableRik', [], 'Display', None],
  [ 0xb9c5, 'HangulSyllableRit', [], 'Display', None],
  [ 0xb9c6, 'HangulSyllableRip', [], 'Display', None],
  [ 0xb9c7, 'HangulSyllableRih', [], 'Display', None],
  [ 0xb9c8, 'HangulSyllableMa', [], 'Display', None],
  [ 0xb9c9, 'HangulSyllableMag', [], 'Display', None],
  [ 0xb9ca, 'HangulSyllableMagg', [], 'Display', None],
  [ 0xb9cb, 'HangulSyllableMags', [], 'Display', None],
  [ 0xb9cc, 'HangulSyllableMan', [], 'Display', None],
  [ 0xb9cd, 'HangulSyllableManj', [], 'Display', None],
  [ 0xb9ce, 'HangulSyllableManh', [], 'Display', None],
  [ 0xb9cf, 'HangulSyllableMad', [], 'Display', None],
  [ 0xb9d0, 'HangulSyllableMal', [], 'Display', None],
  [ 0xb9d1, 'HangulSyllableMalg', [], 'Display', None],
  [ 0xb9d2, 'HangulSyllableMalm', [], 'Display', None],
  [ 0xb9d3, 'HangulSyllableMalb', [], 'Display', None],
  [ 0xb9d4, 'HangulSyllableMals', [], 'Display', None],
  [ 0xb9d5, 'HangulSyllableMalt', [], 'Display', None],
  [ 0xb9d6, 'HangulSyllableMalp', [], 'Display', None],
  [ 0xb9d7, 'HangulSyllableMalh', [], 'Display', None],
  [ 0xb9d8, 'HangulSyllableMam', [], 'Display', None],
  [ 0xb9d9, 'HangulSyllableMab', [], 'Display', None],
  [ 0xb9da, 'HangulSyllableMabs', [], 'Display', None],
  [ 0xb9db, 'HangulSyllableMas', [], 'Display', None],
  [ 0xb9dc, 'HangulSyllableMass', [], 'Display', None],
  [ 0xb9dd, 'HangulSyllableMang', [], 'Display', None],
  [ 0xb9de, 'HangulSyllableMaj', [], 'Display', None],
  [ 0xb9df, 'HangulSyllableMac', [], 'Display', None],
  [ 0xb9e0, 'HangulSyllableMak', [], 'Display', None],
  [ 0xb9e1, 'HangulSyllableMat', [], 'Display', None],
  [ 0xb9e2, 'HangulSyllableMap', [], 'Display', None],
  [ 0xb9e3, 'HangulSyllableMah', [], 'Display', None],
  [ 0xb9e4, 'HangulSyllableMae', [], 'Display', None],
  [ 0xb9e5, 'HangulSyllableMaeg', [], 'Display', None],
  [ 0xb9e6, 'HangulSyllableMaegg', [], 'Display', None],
  [ 0xb9e7, 'HangulSyllableMaegs', [], 'Display', None],
  [ 0xb9e8, 'HangulSyllableMaen', [], 'Display', None],
  [ 0xb9e9, 'HangulSyllableMaenj', [], 'Display', None],
  [ 0xb9ea, 'HangulSyllableMaenh', [], 'Display', None],
  [ 0xb9eb, 'HangulSyllableMaed', [], 'Display', None],
  [ 0xb9ec, 'HangulSyllableMael', [], 'Display', None],
  [ 0xb9ed, 'HangulSyllableMaelg', [], 'Display', None],
  [ 0xb9ee, 'HangulSyllableMaelm', [], 'Display', None],
  [ 0xb9ef, 'HangulSyllableMaelb', [], 'Display', None],
  [ 0xb9f0, 'HangulSyllableMaels', [], 'Display', None],
  [ 0xb9f1, 'HangulSyllableMaelt', [], 'Display', None],
  [ 0xb9f2, 'HangulSyllableMaelp', [], 'Display', None],
  [ 0xb9f3, 'HangulSyllableMaelh', [], 'Display', None],
  [ 0xb9f4, 'HangulSyllableMaem', [], 'Display', None],
  [ 0xb9f5, 'HangulSyllableMaeb', [], 'Display', None],
  [ 0xb9f6, 'HangulSyllableMaebs', [], 'Display', None],
  [ 0xb9f7, 'HangulSyllableMaes', [], 'Display', None],
  [ 0xb9f8, 'HangulSyllableMaess', [], 'Display', None],
  [ 0xb9f9, 'HangulSyllableMaeng', [], 'Display', None],
  [ 0xb9fa, 'HangulSyllableMaej', [], 'Display', None],
  [ 0xb9fb, 'HangulSyllableMaec', [], 'Display', None],
  [ 0xb9fc, 'HangulSyllableMaek', [], 'Display', None],
  [ 0xb9fd, 'HangulSyllableMaet', [], 'Display', None],
  [ 0xb9fe, 'HangulSyllableMaep', [], 'Display', None],
  [ 0xb9ff, 'HangulSyllableMaeh', [], 'Display', None],
  [ 0xba00, 'HangulSyllableMya', [], 'Display', None],
  [ 0xba01, 'HangulSyllableMyag', [], 'Display', None],
  [ 0xba02, 'HangulSyllableMyagg', [], 'Display', None],
  [ 0xba03, 'HangulSyllableMyags', [], 'Display', None],
  [ 0xba04, 'HangulSyllableMyan', [], 'Display', None],
  [ 0xba05, 'HangulSyllableMyanj', [], 'Display', None],
  [ 0xba06, 'HangulSyllableMyanh', [], 'Display', None],
  [ 0xba07, 'HangulSyllableMyad', [], 'Display', None],
  [ 0xba08, 'HangulSyllableMyal', [], 'Display', None],
  [ 0xba09, 'HangulSyllableMyalg', [], 'Display', None],
  [ 0xba0a, 'HangulSyllableMyalm', [], 'Display', None],
  [ 0xba0b, 'HangulSyllableMyalb', [], 'Display', None],
  [ 0xba0c, 'HangulSyllableMyals', [], 'Display', None],
  [ 0xba0d, 'HangulSyllableMyalt', [], 'Display', None],
  [ 0xba0e, 'HangulSyllableMyalp', [], 'Display', None],
  [ 0xba0f, 'HangulSyllableMyalh', [], 'Display', None],
  [ 0xba10, 'HangulSyllableMyam', [], 'Display', None],
  [ 0xba11, 'HangulSyllableMyab', [], 'Display', None],
  [ 0xba12, 'HangulSyllableMyabs', [], 'Display', None],
  [ 0xba13, 'HangulSyllableMyas', [], 'Display', None],
  [ 0xba14, 'HangulSyllableMyass', [], 'Display', None],
  [ 0xba15, 'HangulSyllableMyang', [], 'Display', None],
  [ 0xba16, 'HangulSyllableMyaj', [], 'Display', None],
  [ 0xba17, 'HangulSyllableMyac', [], 'Display', None],
  [ 0xba18, 'HangulSyllableMyak', [], 'Display', None],
  [ 0xba19, 'HangulSyllableMyat', [], 'Display', None],
  [ 0xba1a, 'HangulSyllableMyap', [], 'Display', None],
  [ 0xba1b, 'HangulSyllableMyah', [], 'Display', None],
  [ 0xba1c, 'HangulSyllableMyae', [], 'Display', None],
  [ 0xba1d, 'HangulSyllableMyaeg', [], 'Display', None],
  [ 0xba1e, 'HangulSyllableMyaegg', [], 'Display', None],
  [ 0xba1f, 'HangulSyllableMyaegs', [], 'Display', None],
  [ 0xba20, 'HangulSyllableMyaen', [], 'Display', None],
  [ 0xba21, 'HangulSyllableMyaenj', [], 'Display', None],
  [ 0xba22, 'HangulSyllableMyaenh', [], 'Display', None],
  [ 0xba23, 'HangulSyllableMyaed', [], 'Display', None],
  [ 0xba24, 'HangulSyllableMyael', [], 'Display', None],
  [ 0xba25, 'HangulSyllableMyaelg', [], 'Display', None],
  [ 0xba26, 'HangulSyllableMyaelm', [], 'Display', None],
  [ 0xba27, 'HangulSyllableMyaelb', [], 'Display', None],
  [ 0xba28, 'HangulSyllableMyaels', [], 'Display', None],
  [ 0xba29, 'HangulSyllableMyaelt', [], 'Display', None],
  [ 0xba2a, 'HangulSyllableMyaelp', [], 'Display', None],
  [ 0xba2b, 'HangulSyllableMyaelh', [], 'Display', None],
  [ 0xba2c, 'HangulSyllableMyaem', [], 'Display', None],
  [ 0xba2d, 'HangulSyllableMyaeb', [], 'Display', None],
  [ 0xba2e, 'HangulSyllableMyaebs', [], 'Display', None],
  [ 0xba2f, 'HangulSyllableMyaes', [], 'Display', None],
  [ 0xba30, 'HangulSyllableMyaess', [], 'Display', None],
  [ 0xba31, 'HangulSyllableMyaeng', [], 'Display', None],
  [ 0xba32, 'HangulSyllableMyaej', [], 'Display', None],
  [ 0xba33, 'HangulSyllableMyaec', [], 'Display', None],
  [ 0xba34, 'HangulSyllableMyaek', [], 'Display', None],
  [ 0xba35, 'HangulSyllableMyaet', [], 'Display', None],
  [ 0xba36, 'HangulSyllableMyaep', [], 'Display', None],
  [ 0xba37, 'HangulSyllableMyaeh', [], 'Display', None],
  [ 0xba38, 'HangulSyllableMeo', [], 'Display', None],
  [ 0xba39, 'HangulSyllableMeog', [], 'Display', None],
  [ 0xba3a, 'HangulSyllableMeogg', [], 'Display', None],
  [ 0xba3b, 'HangulSyllableMeogs', [], 'Display', None],
  [ 0xba3c, 'HangulSyllableMeon', [], 'Display', None],
  [ 0xba3d, 'HangulSyllableMeonj', [], 'Display', None],
  [ 0xba3e, 'HangulSyllableMeonh', [], 'Display', None],
  [ 0xba3f, 'HangulSyllableMeod', [], 'Display', None],
  [ 0xba40, 'HangulSyllableMeol', [], 'Display', None],
  [ 0xba41, 'HangulSyllableMeolg', [], 'Display', None],
  [ 0xba42, 'HangulSyllableMeolm', [], 'Display', None],
  [ 0xba43, 'HangulSyllableMeolb', [], 'Display', None],
  [ 0xba44, 'HangulSyllableMeols', [], 'Display', None],
  [ 0xba45, 'HangulSyllableMeolt', [], 'Display', None],
  [ 0xba46, 'HangulSyllableMeolp', [], 'Display', None],
  [ 0xba47, 'HangulSyllableMeolh', [], 'Display', None],
  [ 0xba48, 'HangulSyllableMeom', [], 'Display', None],
  [ 0xba49, 'HangulSyllableMeob', [], 'Display', None],
  [ 0xba4a, 'HangulSyllableMeobs', [], 'Display', None],
  [ 0xba4b, 'HangulSyllableMeos', [], 'Display', None],
  [ 0xba4c, 'HangulSyllableMeoss', [], 'Display', None],
  [ 0xba4d, 'HangulSyllableMeong', [], 'Display', None],
  [ 0xba4e, 'HangulSyllableMeoj', [], 'Display', None],
  [ 0xba4f, 'HangulSyllableMeoc', [], 'Display', None],
  [ 0xba50, 'HangulSyllableMeok', [], 'Display', None],
  [ 0xba51, 'HangulSyllableMeot', [], 'Display', None],
  [ 0xba52, 'HangulSyllableMeop', [], 'Display', None],
  [ 0xba53, 'HangulSyllableMeoh', [], 'Display', None],
  [ 0xba54, 'HangulSyllableMe', [], 'Display', None],
  [ 0xba55, 'HangulSyllableMeg', [], 'Display', None],
  [ 0xba56, 'HangulSyllableMegg', [], 'Display', None],
  [ 0xba57, 'HangulSyllableMegs', [], 'Display', None],
  [ 0xba58, 'HangulSyllableMen', [], 'Display', None],
  [ 0xba59, 'HangulSyllableMenj', [], 'Display', None],
  [ 0xba5a, 'HangulSyllableMenh', [], 'Display', None],
  [ 0xba5b, 'HangulSyllableMed', [], 'Display', None],
  [ 0xba5c, 'HangulSyllableMel', [], 'Display', None],
  [ 0xba5d, 'HangulSyllableMelg', [], 'Display', None],
  [ 0xba5e, 'HangulSyllableMelm', [], 'Display', None],
  [ 0xba5f, 'HangulSyllableMelb', [], 'Display', None],
  [ 0xba60, 'HangulSyllableMels', [], 'Display', None],
  [ 0xba61, 'HangulSyllableMelt', [], 'Display', None],
  [ 0xba62, 'HangulSyllableMelp', [], 'Display', None],
  [ 0xba63, 'HangulSyllableMelh', [], 'Display', None],
  [ 0xba64, 'HangulSyllableMem', [], 'Display', None],
  [ 0xba65, 'HangulSyllableMeb', [], 'Display', None],
  [ 0xba66, 'HangulSyllableMebs', [], 'Display', None],
  [ 0xba67, 'HangulSyllableMes', [], 'Display', None],
  [ 0xba68, 'HangulSyllableMess', [], 'Display', None],
  [ 0xba69, 'HangulSyllableMeng', [], 'Display', None],
  [ 0xba6a, 'HangulSyllableMej', [], 'Display', None],
  [ 0xba6b, 'HangulSyllableMec', [], 'Display', None],
  [ 0xba6c, 'HangulSyllableMek', [], 'Display', None],
  [ 0xba6d, 'HangulSyllableMet', [], 'Display', None],
  [ 0xba6e, 'HangulSyllableMep', [], 'Display', None],
  [ 0xba6f, 'HangulSyllableMeh', [], 'Display', None],
  [ 0xba70, 'HangulSyllableMyeo', [], 'Display', None],
  [ 0xba71, 'HangulSyllableMyeog', [], 'Display', None],
  [ 0xba72, 'HangulSyllableMyeogg', [], 'Display', None],
  [ 0xba73, 'HangulSyllableMyeogs', [], 'Display', None],
  [ 0xba74, 'HangulSyllableMyeon', [], 'Display', None],
  [ 0xba75, 'HangulSyllableMyeonj', [], 'Display', None],
  [ 0xba76, 'HangulSyllableMyeonh', [], 'Display', None],
  [ 0xba77, 'HangulSyllableMyeod', [], 'Display', None],
  [ 0xba78, 'HangulSyllableMyeol', [], 'Display', None],
  [ 0xba79, 'HangulSyllableMyeolg', [], 'Display', None],
  [ 0xba7a, 'HangulSyllableMyeolm', [], 'Display', None],
  [ 0xba7b, 'HangulSyllableMyeolb', [], 'Display', None],
  [ 0xba7c, 'HangulSyllableMyeols', [], 'Display', None],
  [ 0xba7d, 'HangulSyllableMyeolt', [], 'Display', None],
  [ 0xba7e, 'HangulSyllableMyeolp', [], 'Display', None],
  [ 0xba7f, 'HangulSyllableMyeolh', [], 'Display', None],
  [ 0xba80, 'HangulSyllableMyeom', [], 'Display', None],
  [ 0xba81, 'HangulSyllableMyeob', [], 'Display', None],
  [ 0xba82, 'HangulSyllableMyeobs', [], 'Display', None],
  [ 0xba83, 'HangulSyllableMyeos', [], 'Display', None],
  [ 0xba84, 'HangulSyllableMyeoss', [], 'Display', None],
  [ 0xba85, 'HangulSyllableMyeong', [], 'Display', None],
  [ 0xba86, 'HangulSyllableMyeoj', [], 'Display', None],
  [ 0xba87, 'HangulSyllableMyeoc', [], 'Display', None],
  [ 0xba88, 'HangulSyllableMyeok', [], 'Display', None],
  [ 0xba89, 'HangulSyllableMyeot', [], 'Display', None],
  [ 0xba8a, 'HangulSyllableMyeop', [], 'Display', None],
  [ 0xba8b, 'HangulSyllableMyeoh', [], 'Display', None],
  [ 0xba8c, 'HangulSyllableMye', [], 'Display', None],
  [ 0xba8d, 'HangulSyllableMyeg', [], 'Display', None],
  [ 0xba8e, 'HangulSyllableMyegg', [], 'Display', None],
  [ 0xba8f, 'HangulSyllableMyegs', [], 'Display', None],
  [ 0xba90, 'HangulSyllableMyen', [], 'Display', None],
  [ 0xba91, 'HangulSyllableMyenj', [], 'Display', None],
  [ 0xba92, 'HangulSyllableMyenh', [], 'Display', None],
  [ 0xba93, 'HangulSyllableMyed', [], 'Display', None],
  [ 0xba94, 'HangulSyllableMyel', [], 'Display', None],
  [ 0xba95, 'HangulSyllableMyelg', [], 'Display', None],
  [ 0xba96, 'HangulSyllableMyelm', [], 'Display', None],
  [ 0xba97, 'HangulSyllableMyelb', [], 'Display', None],
  [ 0xba98, 'HangulSyllableMyels', [], 'Display', None],
  [ 0xba99, 'HangulSyllableMyelt', [], 'Display', None],
  [ 0xba9a, 'HangulSyllableMyelp', [], 'Display', None],
  [ 0xba9b, 'HangulSyllableMyelh', [], 'Display', None],
  [ 0xba9c, 'HangulSyllableMyem', [], 'Display', None],
  [ 0xba9d, 'HangulSyllableMyeb', [], 'Display', None],
  [ 0xba9e, 'HangulSyllableMyebs', [], 'Display', None],
  [ 0xba9f, 'HangulSyllableMyes', [], 'Display', None],
  [ 0xbaa0, 'HangulSyllableMyess', [], 'Display', None],
  [ 0xbaa1, 'HangulSyllableMyeng', [], 'Display', None],
  [ 0xbaa2, 'HangulSyllableMyej', [], 'Display', None],
  [ 0xbaa3, 'HangulSyllableMyec', [], 'Display', None],
  [ 0xbaa4, 'HangulSyllableMyek', [], 'Display', None],
  [ 0xbaa5, 'HangulSyllableMyet', [], 'Display', None],
  [ 0xbaa6, 'HangulSyllableMyep', [], 'Display', None],
  [ 0xbaa7, 'HangulSyllableMyeh', [], 'Display', None],
  [ 0xbaa8, 'HangulSyllableMo', [], 'Display', None],
  [ 0xbaa9, 'HangulSyllableMog', [], 'Display', None],
  [ 0xbaaa, 'HangulSyllableMogg', [], 'Display', None],
  [ 0xbaab, 'HangulSyllableMogs', [], 'Display', None],
  [ 0xbaac, 'HangulSyllableMon', [], 'Display', None],
  [ 0xbaad, 'HangulSyllableMonj', [], 'Display', None],
  [ 0xbaae, 'HangulSyllableMonh', [], 'Display', None],
  [ 0xbaaf, 'HangulSyllableMod', [], 'Display', None],
  [ 0xbab0, 'HangulSyllableMol', [], 'Display', None],
  [ 0xbab1, 'HangulSyllableMolg', [], 'Display', None],
  [ 0xbab2, 'HangulSyllableMolm', [], 'Display', None],
  [ 0xbab3, 'HangulSyllableMolb', [], 'Display', None],
  [ 0xbab4, 'HangulSyllableMols', [], 'Display', None],
  [ 0xbab5, 'HangulSyllableMolt', [], 'Display', None],
  [ 0xbab6, 'HangulSyllableMolp', [], 'Display', None],
  [ 0xbab7, 'HangulSyllableMolh', [], 'Display', None],
  [ 0xbab8, 'HangulSyllableMom', [], 'Display', None],
  [ 0xbab9, 'HangulSyllableMob', [], 'Display', None],
  [ 0xbaba, 'HangulSyllableMobs', [], 'Display', None],
  [ 0xbabb, 'HangulSyllableMos', [], 'Display', None],
  [ 0xbabc, 'HangulSyllableMoss', [], 'Display', None],
  [ 0xbabd, 'HangulSyllableMong', [], 'Display', None],
  [ 0xbabe, 'HangulSyllableMoj', [], 'Display', None],
  [ 0xbabf, 'HangulSyllableMoc', [], 'Display', None],
  [ 0xbac0, 'HangulSyllableMok', [], 'Display', None],
  [ 0xbac1, 'HangulSyllableMot', [], 'Display', None],
  [ 0xbac2, 'HangulSyllableMop', [], 'Display', None],
  [ 0xbac3, 'HangulSyllableMoh', [], 'Display', None],
  [ 0xbac4, 'HangulSyllableMwa', [], 'Display', None],
  [ 0xbac5, 'HangulSyllableMwag', [], 'Display', None],
  [ 0xbac6, 'HangulSyllableMwagg', [], 'Display', None],
  [ 0xbac7, 'HangulSyllableMwags', [], 'Display', None],
  [ 0xbac8, 'HangulSyllableMwan', [], 'Display', None],
  [ 0xbac9, 'HangulSyllableMwanj', [], 'Display', None],
  [ 0xbaca, 'HangulSyllableMwanh', [], 'Display', None],
  [ 0xbacb, 'HangulSyllableMwad', [], 'Display', None],
  [ 0xbacc, 'HangulSyllableMwal', [], 'Display', None],
  [ 0xbacd, 'HangulSyllableMwalg', [], 'Display', None],
  [ 0xbace, 'HangulSyllableMwalm', [], 'Display', None],
  [ 0xbacf, 'HangulSyllableMwalb', [], 'Display', None],
  [ 0xbad0, 'HangulSyllableMwals', [], 'Display', None],
  [ 0xbad1, 'HangulSyllableMwalt', [], 'Display', None],
  [ 0xbad2, 'HangulSyllableMwalp', [], 'Display', None],
  [ 0xbad3, 'HangulSyllableMwalh', [], 'Display', None],
  [ 0xbad4, 'HangulSyllableMwam', [], 'Display', None],
  [ 0xbad5, 'HangulSyllableMwab', [], 'Display', None],
  [ 0xbad6, 'HangulSyllableMwabs', [], 'Display', None],
  [ 0xbad7, 'HangulSyllableMwas', [], 'Display', None],
  [ 0xbad8, 'HangulSyllableMwass', [], 'Display', None],
  [ 0xbad9, 'HangulSyllableMwang', [], 'Display', None],
  [ 0xbada, 'HangulSyllableMwaj', [], 'Display', None],
  [ 0xbadb, 'HangulSyllableMwac', [], 'Display', None],
  [ 0xbadc, 'HangulSyllableMwak', [], 'Display', None],
  [ 0xbadd, 'HangulSyllableMwat', [], 'Display', None],
  [ 0xbade, 'HangulSyllableMwap', [], 'Display', None],
  [ 0xbadf, 'HangulSyllableMwah', [], 'Display', None],
  [ 0xbae0, 'HangulSyllableMwae', [], 'Display', None],
  [ 0xbae1, 'HangulSyllableMwaeg', [], 'Display', None],
  [ 0xbae2, 'HangulSyllableMwaegg', [], 'Display', None],
  [ 0xbae3, 'HangulSyllableMwaegs', [], 'Display', None],
  [ 0xbae4, 'HangulSyllableMwaen', [], 'Display', None],
  [ 0xbae5, 'HangulSyllableMwaenj', [], 'Display', None],
  [ 0xbae6, 'HangulSyllableMwaenh', [], 'Display', None],
  [ 0xbae7, 'HangulSyllableMwaed', [], 'Display', None],
  [ 0xbae8, 'HangulSyllableMwael', [], 'Display', None],
  [ 0xbae9, 'HangulSyllableMwaelg', [], 'Display', None],
  [ 0xbaea, 'HangulSyllableMwaelm', [], 'Display', None],
  [ 0xbaeb, 'HangulSyllableMwaelb', [], 'Display', None],
  [ 0xbaec, 'HangulSyllableMwaels', [], 'Display', None],
  [ 0xbaed, 'HangulSyllableMwaelt', [], 'Display', None],
  [ 0xbaee, 'HangulSyllableMwaelp', [], 'Display', None],
  [ 0xbaef, 'HangulSyllableMwaelh', [], 'Display', None],
  [ 0xbaf0, 'HangulSyllableMwaem', [], 'Display', None],
  [ 0xbaf1, 'HangulSyllableMwaeb', [], 'Display', None],
  [ 0xbaf2, 'HangulSyllableMwaebs', [], 'Display', None],
  [ 0xbaf3, 'HangulSyllableMwaes', [], 'Display', None],
  [ 0xbaf4, 'HangulSyllableMwaess', [], 'Display', None],
  [ 0xbaf5, 'HangulSyllableMwaeng', [], 'Display', None],
  [ 0xbaf6, 'HangulSyllableMwaej', [], 'Display', None],
  [ 0xbaf7, 'HangulSyllableMwaec', [], 'Display', None],
  [ 0xbaf8, 'HangulSyllableMwaek', [], 'Display', None],
  [ 0xbaf9, 'HangulSyllableMwaet', [], 'Display', None],
  [ 0xbafa, 'HangulSyllableMwaep', [], 'Display', None],
  [ 0xbafb, 'HangulSyllableMwaeh', [], 'Display', None],
  [ 0xbafc, 'HangulSyllableMoe', [], 'Display', None],
  [ 0xbafd, 'HangulSyllableMoeg', [], 'Display', None],
  [ 0xbafe, 'HangulSyllableMoegg', [], 'Display', None],
  [ 0xbaff, 'HangulSyllableMoegs', [], 'Display', None],
  [ 0xbb00, 'HangulSyllableMoen', [], 'Display', None],
  [ 0xbb01, 'HangulSyllableMoenj', [], 'Display', None],
  [ 0xbb02, 'HangulSyllableMoenh', [], 'Display', None],
  [ 0xbb03, 'HangulSyllableMoed', [], 'Display', None],
  [ 0xbb04, 'HangulSyllableMoel', [], 'Display', None],
  [ 0xbb05, 'HangulSyllableMoelg', [], 'Display', None],
  [ 0xbb06, 'HangulSyllableMoelm', [], 'Display', None],
  [ 0xbb07, 'HangulSyllableMoelb', [], 'Display', None],
  [ 0xbb08, 'HangulSyllableMoels', [], 'Display', None],
  [ 0xbb09, 'HangulSyllableMoelt', [], 'Display', None],
  [ 0xbb0a, 'HangulSyllableMoelp', [], 'Display', None],
  [ 0xbb0b, 'HangulSyllableMoelh', [], 'Display', None],
  [ 0xbb0c, 'HangulSyllableMoem', [], 'Display', None],
  [ 0xbb0d, 'HangulSyllableMoeb', [], 'Display', None],
  [ 0xbb0e, 'HangulSyllableMoebs', [], 'Display', None],
  [ 0xbb0f, 'HangulSyllableMoes', [], 'Display', None],
  [ 0xbb10, 'HangulSyllableMoess', [], 'Display', None],
  [ 0xbb11, 'HangulSyllableMoeng', [], 'Display', None],
  [ 0xbb12, 'HangulSyllableMoej', [], 'Display', None],
  [ 0xbb13, 'HangulSyllableMoec', [], 'Display', None],
  [ 0xbb14, 'HangulSyllableMoek', [], 'Display', None],
  [ 0xbb15, 'HangulSyllableMoet', [], 'Display', None],
  [ 0xbb16, 'HangulSyllableMoep', [], 'Display', None],
  [ 0xbb17, 'HangulSyllableMoeh', [], 'Display', None],
  [ 0xbb18, 'HangulSyllableMyo', [], 'Display', None],
  [ 0xbb19, 'HangulSyllableMyog', [], 'Display', None],
  [ 0xbb1a, 'HangulSyllableMyogg', [], 'Display', None],
  [ 0xbb1b, 'HangulSyllableMyogs', [], 'Display', None],
  [ 0xbb1c, 'HangulSyllableMyon', [], 'Display', None],
  [ 0xbb1d, 'HangulSyllableMyonj', [], 'Display', None],
  [ 0xbb1e, 'HangulSyllableMyonh', [], 'Display', None],
  [ 0xbb1f, 'HangulSyllableMyod', [], 'Display', None],
  [ 0xbb20, 'HangulSyllableMyol', [], 'Display', None],
  [ 0xbb21, 'HangulSyllableMyolg', [], 'Display', None],
  [ 0xbb22, 'HangulSyllableMyolm', [], 'Display', None],
  [ 0xbb23, 'HangulSyllableMyolb', [], 'Display', None],
  [ 0xbb24, 'HangulSyllableMyols', [], 'Display', None],
  [ 0xbb25, 'HangulSyllableMyolt', [], 'Display', None],
  [ 0xbb26, 'HangulSyllableMyolp', [], 'Display', None],
  [ 0xbb27, 'HangulSyllableMyolh', [], 'Display', None],
  [ 0xbb28, 'HangulSyllableMyom', [], 'Display', None],
  [ 0xbb29, 'HangulSyllableMyob', [], 'Display', None],
  [ 0xbb2a, 'HangulSyllableMyobs', [], 'Display', None],
  [ 0xbb2b, 'HangulSyllableMyos', [], 'Display', None],
  [ 0xbb2c, 'HangulSyllableMyoss', [], 'Display', None],
  [ 0xbb2d, 'HangulSyllableMyong', [], 'Display', None],
  [ 0xbb2e, 'HangulSyllableMyoj', [], 'Display', None],
  [ 0xbb2f, 'HangulSyllableMyoc', [], 'Display', None],
  [ 0xbb30, 'HangulSyllableMyok', [], 'Display', None],
  [ 0xbb31, 'HangulSyllableMyot', [], 'Display', None],
  [ 0xbb32, 'HangulSyllableMyop', [], 'Display', None],
  [ 0xbb33, 'HangulSyllableMyoh', [], 'Display', None],
  [ 0xbb34, 'HangulSyllableMu', [], 'Display', None],
  [ 0xbb35, 'HangulSyllableMug', [], 'Display', None],
  [ 0xbb36, 'HangulSyllableMugg', [], 'Display', None],
  [ 0xbb37, 'HangulSyllableMugs', [], 'Display', None],
  [ 0xbb38, 'HangulSyllableMun', [], 'Display', None],
  [ 0xbb39, 'HangulSyllableMunj', [], 'Display', None],
  [ 0xbb3a, 'HangulSyllableMunh', [], 'Display', None],
  [ 0xbb3b, 'HangulSyllableMud', [], 'Display', None],
  [ 0xbb3c, 'HangulSyllableMul', [], 'Display', None],
  [ 0xbb3d, 'HangulSyllableMulg', [], 'Display', None],
  [ 0xbb3e, 'HangulSyllableMulm', [], 'Display', None],
  [ 0xbb3f, 'HangulSyllableMulb', [], 'Display', None],
  [ 0xbb40, 'HangulSyllableMuls', [], 'Display', None],
  [ 0xbb41, 'HangulSyllableMult', [], 'Display', None],
  [ 0xbb42, 'HangulSyllableMulp', [], 'Display', None],
  [ 0xbb43, 'HangulSyllableMulh', [], 'Display', None],
  [ 0xbb44, 'HangulSyllableMum', [], 'Display', None],
  [ 0xbb45, 'HangulSyllableMub', [], 'Display', None],
  [ 0xbb46, 'HangulSyllableMubs', [], 'Display', None],
  [ 0xbb47, 'HangulSyllableMus', [], 'Display', None],
  [ 0xbb48, 'HangulSyllableMuss', [], 'Display', None],
  [ 0xbb49, 'HangulSyllableMung', [], 'Display', None],
  [ 0xbb4a, 'HangulSyllableMuj', [], 'Display', None],
  [ 0xbb4b, 'HangulSyllableMuc', [], 'Display', None],
  [ 0xbb4c, 'HangulSyllableMuk', [], 'Display', None],
  [ 0xbb4d, 'HangulSyllableMut', [], 'Display', None],
  [ 0xbb4e, 'HangulSyllableMup', [], 'Display', None],
  [ 0xbb4f, 'HangulSyllableMuh', [], 'Display', None],
  [ 0xbb50, 'HangulSyllableMweo', [], 'Display', None],
  [ 0xbb51, 'HangulSyllableMweog', [], 'Display', None],
  [ 0xbb52, 'HangulSyllableMweogg', [], 'Display', None],
  [ 0xbb53, 'HangulSyllableMweogs', [], 'Display', None],
  [ 0xbb54, 'HangulSyllableMweon', [], 'Display', None],
  [ 0xbb55, 'HangulSyllableMweonj', [], 'Display', None],
  [ 0xbb56, 'HangulSyllableMweonh', [], 'Display', None],
  [ 0xbb57, 'HangulSyllableMweod', [], 'Display', None],
  [ 0xbb58, 'HangulSyllableMweol', [], 'Display', None],
  [ 0xbb59, 'HangulSyllableMweolg', [], 'Display', None],
  [ 0xbb5a, 'HangulSyllableMweolm', [], 'Display', None],
  [ 0xbb5b, 'HangulSyllableMweolb', [], 'Display', None],
  [ 0xbb5c, 'HangulSyllableMweols', [], 'Display', None],
  [ 0xbb5d, 'HangulSyllableMweolt', [], 'Display', None],
  [ 0xbb5e, 'HangulSyllableMweolp', [], 'Display', None],
  [ 0xbb5f, 'HangulSyllableMweolh', [], 'Display', None],
  [ 0xbb60, 'HangulSyllableMweom', [], 'Display', None],
  [ 0xbb61, 'HangulSyllableMweob', [], 'Display', None],
  [ 0xbb62, 'HangulSyllableMweobs', [], 'Display', None],
  [ 0xbb63, 'HangulSyllableMweos', [], 'Display', None],
  [ 0xbb64, 'HangulSyllableMweoss', [], 'Display', None],
  [ 0xbb65, 'HangulSyllableMweong', [], 'Display', None],
  [ 0xbb66, 'HangulSyllableMweoj', [], 'Display', None],
  [ 0xbb67, 'HangulSyllableMweoc', [], 'Display', None],
  [ 0xbb68, 'HangulSyllableMweok', [], 'Display', None],
  [ 0xbb69, 'HangulSyllableMweot', [], 'Display', None],
  [ 0xbb6a, 'HangulSyllableMweop', [], 'Display', None],
  [ 0xbb6b, 'HangulSyllableMweoh', [], 'Display', None],
  [ 0xbb6c, 'HangulSyllableMwe', [], 'Display', None],
  [ 0xbb6d, 'HangulSyllableMweg', [], 'Display', None],
  [ 0xbb6e, 'HangulSyllableMwegg', [], 'Display', None],
  [ 0xbb6f, 'HangulSyllableMwegs', [], 'Display', None],
  [ 0xbb70, 'HangulSyllableMwen', [], 'Display', None],
  [ 0xbb71, 'HangulSyllableMwenj', [], 'Display', None],
  [ 0xbb72, 'HangulSyllableMwenh', [], 'Display', None],
  [ 0xbb73, 'HangulSyllableMwed', [], 'Display', None],
  [ 0xbb74, 'HangulSyllableMwel', [], 'Display', None],
  [ 0xbb75, 'HangulSyllableMwelg', [], 'Display', None],
  [ 0xbb76, 'HangulSyllableMwelm', [], 'Display', None],
  [ 0xbb77, 'HangulSyllableMwelb', [], 'Display', None],
  [ 0xbb78, 'HangulSyllableMwels', [], 'Display', None],
  [ 0xbb79, 'HangulSyllableMwelt', [], 'Display', None],
  [ 0xbb7a, 'HangulSyllableMwelp', [], 'Display', None],
  [ 0xbb7b, 'HangulSyllableMwelh', [], 'Display', None],
  [ 0xbb7c, 'HangulSyllableMwem', [], 'Display', None],
  [ 0xbb7d, 'HangulSyllableMweb', [], 'Display', None],
  [ 0xbb7e, 'HangulSyllableMwebs', [], 'Display', None],
  [ 0xbb7f, 'HangulSyllableMwes', [], 'Display', None],
  [ 0xbb80, 'HangulSyllableMwess', [], 'Display', None],
  [ 0xbb81, 'HangulSyllableMweng', [], 'Display', None],
  [ 0xbb82, 'HangulSyllableMwej', [], 'Display', None],
  [ 0xbb83, 'HangulSyllableMwec', [], 'Display', None],
  [ 0xbb84, 'HangulSyllableMwek', [], 'Display', None],
  [ 0xbb85, 'HangulSyllableMwet', [], 'Display', None],
  [ 0xbb86, 'HangulSyllableMwep', [], 'Display', None],
  [ 0xbb87, 'HangulSyllableMweh', [], 'Display', None],
  [ 0xbb88, 'HangulSyllableMwi', [], 'Display', None],
  [ 0xbb89, 'HangulSyllableMwig', [], 'Display', None],
  [ 0xbb8a, 'HangulSyllableMwigg', [], 'Display', None],
  [ 0xbb8b, 'HangulSyllableMwigs', [], 'Display', None],
  [ 0xbb8c, 'HangulSyllableMwin', [], 'Display', None],
  [ 0xbb8d, 'HangulSyllableMwinj', [], 'Display', None],
  [ 0xbb8e, 'HangulSyllableMwinh', [], 'Display', None],
  [ 0xbb8f, 'HangulSyllableMwid', [], 'Display', None],
  [ 0xbb90, 'HangulSyllableMwil', [], 'Display', None],
  [ 0xbb91, 'HangulSyllableMwilg', [], 'Display', None],
  [ 0xbb92, 'HangulSyllableMwilm', [], 'Display', None],
  [ 0xbb93, 'HangulSyllableMwilb', [], 'Display', None],
  [ 0xbb94, 'HangulSyllableMwils', [], 'Display', None],
  [ 0xbb95, 'HangulSyllableMwilt', [], 'Display', None],
  [ 0xbb96, 'HangulSyllableMwilp', [], 'Display', None],
  [ 0xbb97, 'HangulSyllableMwilh', [], 'Display', None],
  [ 0xbb98, 'HangulSyllableMwim', [], 'Display', None],
  [ 0xbb99, 'HangulSyllableMwib', [], 'Display', None],
  [ 0xbb9a, 'HangulSyllableMwibs', [], 'Display', None],
  [ 0xbb9b, 'HangulSyllableMwis', [], 'Display', None],
  [ 0xbb9c, 'HangulSyllableMwiss', [], 'Display', None],
  [ 0xbb9d, 'HangulSyllableMwing', [], 'Display', None],
  [ 0xbb9e, 'HangulSyllableMwij', [], 'Display', None],
  [ 0xbb9f, 'HangulSyllableMwic', [], 'Display', None],
  [ 0xbba0, 'HangulSyllableMwik', [], 'Display', None],
  [ 0xbba1, 'HangulSyllableMwit', [], 'Display', None],
  [ 0xbba2, 'HangulSyllableMwip', [], 'Display', None],
  [ 0xbba3, 'HangulSyllableMwih', [], 'Display', None],
  [ 0xbba4, 'HangulSyllableMyu', [], 'Display', None],
  [ 0xbba5, 'HangulSyllableMyug', [], 'Display', None],
  [ 0xbba6, 'HangulSyllableMyugg', [], 'Display', None],
  [ 0xbba7, 'HangulSyllableMyugs', [], 'Display', None],
  [ 0xbba8, 'HangulSyllableMyun', [], 'Display', None],
  [ 0xbba9, 'HangulSyllableMyunj', [], 'Display', None],
  [ 0xbbaa, 'HangulSyllableMyunh', [], 'Display', None],
  [ 0xbbab, 'HangulSyllableMyud', [], 'Display', None],
  [ 0xbbac, 'HangulSyllableMyul', [], 'Display', None],
  [ 0xbbad, 'HangulSyllableMyulg', [], 'Display', None],
  [ 0xbbae, 'HangulSyllableMyulm', [], 'Display', None],
  [ 0xbbaf, 'HangulSyllableMyulb', [], 'Display', None],
  [ 0xbbb0, 'HangulSyllableMyuls', [], 'Display', None],
  [ 0xbbb1, 'HangulSyllableMyult', [], 'Display', None],
  [ 0xbbb2, 'HangulSyllableMyulp', [], 'Display', None],
  [ 0xbbb3, 'HangulSyllableMyulh', [], 'Display', None],
  [ 0xbbb4, 'HangulSyllableMyum', [], 'Display', None],
  [ 0xbbb5, 'HangulSyllableMyub', [], 'Display', None],
  [ 0xbbb6, 'HangulSyllableMyubs', [], 'Display', None],
  [ 0xbbb7, 'HangulSyllableMyus', [], 'Display', None],
  [ 0xbbb8, 'HangulSyllableMyuss', [], 'Display', None],
  [ 0xbbb9, 'HangulSyllableMyung', [], 'Display', None],
  [ 0xbbba, 'HangulSyllableMyuj', [], 'Display', None],
  [ 0xbbbb, 'HangulSyllableMyuc', [], 'Display', None],
  [ 0xbbbc, 'HangulSyllableMyuk', [], 'Display', None],
  [ 0xbbbd, 'HangulSyllableMyut', [], 'Display', None],
  [ 0xbbbe, 'HangulSyllableMyup', [], 'Display', None],
  [ 0xbbbf, 'HangulSyllableMyuh', [], 'Display', None],
  [ 0xbbc0, 'HangulSyllableMeu', [], 'Display', None],
  [ 0xbbc1, 'HangulSyllableMeug', [], 'Display', None],
  [ 0xbbc2, 'HangulSyllableMeugg', [], 'Display', None],
  [ 0xbbc3, 'HangulSyllableMeugs', [], 'Display', None],
  [ 0xbbc4, 'HangulSyllableMeun', [], 'Display', None],
  [ 0xbbc5, 'HangulSyllableMeunj', [], 'Display', None],
  [ 0xbbc6, 'HangulSyllableMeunh', [], 'Display', None],
  [ 0xbbc7, 'HangulSyllableMeud', [], 'Display', None],
  [ 0xbbc8, 'HangulSyllableMeul', [], 'Display', None],
  [ 0xbbc9, 'HangulSyllableMeulg', [], 'Display', None],
  [ 0xbbca, 'HangulSyllableMeulm', [], 'Display', None],
  [ 0xbbcb, 'HangulSyllableMeulb', [], 'Display', None],
  [ 0xbbcc, 'HangulSyllableMeuls', [], 'Display', None],
  [ 0xbbcd, 'HangulSyllableMeult', [], 'Display', None],
  [ 0xbbce, 'HangulSyllableMeulp', [], 'Display', None],
  [ 0xbbcf, 'HangulSyllableMeulh', [], 'Display', None],
  [ 0xbbd0, 'HangulSyllableMeum', [], 'Display', None],
  [ 0xbbd1, 'HangulSyllableMeub', [], 'Display', None],
  [ 0xbbd2, 'HangulSyllableMeubs', [], 'Display', None],
  [ 0xbbd3, 'HangulSyllableMeus', [], 'Display', None],
  [ 0xbbd4, 'HangulSyllableMeuss', [], 'Display', None],
  [ 0xbbd5, 'HangulSyllableMeung', [], 'Display', None],
  [ 0xbbd6, 'HangulSyllableMeuj', [], 'Display', None],
  [ 0xbbd7, 'HangulSyllableMeuc', [], 'Display', None],
  [ 0xbbd8, 'HangulSyllableMeuk', [], 'Display', None],
  [ 0xbbd9, 'HangulSyllableMeut', [], 'Display', None],
  [ 0xbbda, 'HangulSyllableMeup', [], 'Display', None],
  [ 0xbbdb, 'HangulSyllableMeuh', [], 'Display', None],
  [ 0xbbdc, 'HangulSyllableMyi', [], 'Display', None],
  [ 0xbbdd, 'HangulSyllableMyig', [], 'Display', None],
  [ 0xbbde, 'HangulSyllableMyigg', [], 'Display', None],
  [ 0xbbdf, 'HangulSyllableMyigs', [], 'Display', None],
  [ 0xbbe0, 'HangulSyllableMyin', [], 'Display', None],
  [ 0xbbe1, 'HangulSyllableMyinj', [], 'Display', None],
  [ 0xbbe2, 'HangulSyllableMyinh', [], 'Display', None],
  [ 0xbbe3, 'HangulSyllableMyid', [], 'Display', None],
  [ 0xbbe4, 'HangulSyllableMyil', [], 'Display', None],
  [ 0xbbe5, 'HangulSyllableMyilg', [], 'Display', None],
  [ 0xbbe6, 'HangulSyllableMyilm', [], 'Display', None],
  [ 0xbbe7, 'HangulSyllableMyilb', [], 'Display', None],
  [ 0xbbe8, 'HangulSyllableMyils', [], 'Display', None],
  [ 0xbbe9, 'HangulSyllableMyilt', [], 'Display', None],
  [ 0xbbea, 'HangulSyllableMyilp', [], 'Display', None],
  [ 0xbbeb, 'HangulSyllableMyilh', [], 'Display', None],
  [ 0xbbec, 'HangulSyllableMyim', [], 'Display', None],
  [ 0xbbed, 'HangulSyllableMyib', [], 'Display', None],
  [ 0xbbee, 'HangulSyllableMyibs', [], 'Display', None],
  [ 0xbbef, 'HangulSyllableMyis', [], 'Display', None],
  [ 0xbbf0, 'HangulSyllableMyiss', [], 'Display', None],
  [ 0xbbf1, 'HangulSyllableMying', [], 'Display', None],
  [ 0xbbf2, 'HangulSyllableMyij', [], 'Display', None],
  [ 0xbbf3, 'HangulSyllableMyic', [], 'Display', None],
  [ 0xbbf4, 'HangulSyllableMyik', [], 'Display', None],
  [ 0xbbf5, 'HangulSyllableMyit', [], 'Display', None],
  [ 0xbbf6, 'HangulSyllableMyip', [], 'Display', None],
  [ 0xbbf7, 'HangulSyllableMyih', [], 'Display', None],
  [ 0xbbf8, 'HangulSyllableMi', [], 'Display', None],
  [ 0xbbf9, 'HangulSyllableMig', [], 'Display', None],
  [ 0xbbfa, 'HangulSyllableMigg', [], 'Display', None],
  [ 0xbbfb, 'HangulSyllableMigs', [], 'Display', None],
  [ 0xbbfc, 'HangulSyllableMin', [], 'Display', None],
  [ 0xbbfd, 'HangulSyllableMinj', [], 'Display', None],
  [ 0xbbfe, 'HangulSyllableMinh', [], 'Display', None],
  [ 0xbbff, 'HangulSyllableMid', [], 'Display', None],
  [ 0xbc00, 'HangulSyllableMil', [], 'Display', None],
  [ 0xbc01, 'HangulSyllableMilg', [], 'Display', None],
  [ 0xbc02, 'HangulSyllableMilm', [], 'Display', None],
  [ 0xbc03, 'HangulSyllableMilb', [], 'Display', None],
  [ 0xbc04, 'HangulSyllableMils', [], 'Display', None],
  [ 0xbc05, 'HangulSyllableMilt', [], 'Display', None],
  [ 0xbc06, 'HangulSyllableMilp', [], 'Display', None],
  [ 0xbc07, 'HangulSyllableMilh', [], 'Display', None],
  [ 0xbc08, 'HangulSyllableMim', [], 'Display', None],
  [ 0xbc09, 'HangulSyllableMib', [], 'Display', None],
  [ 0xbc0a, 'HangulSyllableMibs', [], 'Display', None],
  [ 0xbc0b, 'HangulSyllableMis', [], 'Display', None],
  [ 0xbc0c, 'HangulSyllableMiss', [], 'Display', None],
  [ 0xbc0d, 'HangulSyllableMing', [], 'Display', None],
  [ 0xbc0e, 'HangulSyllableMij', [], 'Display', None],
  [ 0xbc0f, 'HangulSyllableMic', [], 'Display', None],
  [ 0xbc10, 'HangulSyllableMik', [], 'Display', None],
  [ 0xbc11, 'HangulSyllableMit', [], 'Display', None],
  [ 0xbc12, 'HangulSyllableMip', [], 'Display', None],
  [ 0xbc13, 'HangulSyllableMih', [], 'Display', None],
  [ 0xbc14, 'HangulSyllableBa', [], 'Display', None],
  [ 0xbc15, 'HangulSyllableBag', [], 'Display', None],
  [ 0xbc16, 'HangulSyllableBagg', [], 'Display', None],
  [ 0xbc17, 'HangulSyllableBags', [], 'Display', None],
  [ 0xbc18, 'HangulSyllableBan', [], 'Display', None],
  [ 0xbc19, 'HangulSyllableBanj', [], 'Display', None],
  [ 0xbc1a, 'HangulSyllableBanh', [], 'Display', None],
  [ 0xbc1b, 'HangulSyllableBad', [], 'Display', None],
  [ 0xbc1c, 'HangulSyllableBal', [], 'Display', None],
  [ 0xbc1d, 'HangulSyllableBalg', [], 'Display', None],
  [ 0xbc1e, 'HangulSyllableBalm', [], 'Display', None],
  [ 0xbc1f, 'HangulSyllableBalb', [], 'Display', None],
  [ 0xbc20, 'HangulSyllableBals', [], 'Display', None],
  [ 0xbc21, 'HangulSyllableBalt', [], 'Display', None],
  [ 0xbc22, 'HangulSyllableBalp', [], 'Display', None],
  [ 0xbc23, 'HangulSyllableBalh', [], 'Display', None],
  [ 0xbc24, 'HangulSyllableBam', [], 'Display', None],
  [ 0xbc25, 'HangulSyllableBab', [], 'Display', None],
  [ 0xbc26, 'HangulSyllableBabs', [], 'Display', None],
  [ 0xbc27, 'HangulSyllableBas', [], 'Display', None],
  [ 0xbc28, 'HangulSyllableBass', [], 'Display', None],
  [ 0xbc29, 'HangulSyllableBang', [], 'Display', None],
  [ 0xbc2a, 'HangulSyllableBaj', [], 'Display', None],
  [ 0xbc2b, 'HangulSyllableBac', [], 'Display', None],
  [ 0xbc2c, 'HangulSyllableBak', [], 'Display', None],
  [ 0xbc2d, 'HangulSyllableBat', [], 'Display', None],
  [ 0xbc2e, 'HangulSyllableBap', [], 'Display', None],
  [ 0xbc2f, 'HangulSyllableBah', [], 'Display', None],
  [ 0xbc30, 'HangulSyllableBae', [], 'Display', None],
  [ 0xbc31, 'HangulSyllableBaeg', [], 'Display', None],
  [ 0xbc32, 'HangulSyllableBaegg', [], 'Display', None],
  [ 0xbc33, 'HangulSyllableBaegs', [], 'Display', None],
  [ 0xbc34, 'HangulSyllableBaen', [], 'Display', None],
  [ 0xbc35, 'HangulSyllableBaenj', [], 'Display', None],
  [ 0xbc36, 'HangulSyllableBaenh', [], 'Display', None],
  [ 0xbc37, 'HangulSyllableBaed', [], 'Display', None],
  [ 0xbc38, 'HangulSyllableBael', [], 'Display', None],
  [ 0xbc39, 'HangulSyllableBaelg', [], 'Display', None],
  [ 0xbc3a, 'HangulSyllableBaelm', [], 'Display', None],
  [ 0xbc3b, 'HangulSyllableBaelb', [], 'Display', None],
  [ 0xbc3c, 'HangulSyllableBaels', [], 'Display', None],
  [ 0xbc3d, 'HangulSyllableBaelt', [], 'Display', None],
  [ 0xbc3e, 'HangulSyllableBaelp', [], 'Display', None],
  [ 0xbc3f, 'HangulSyllableBaelh', [], 'Display', None],
  [ 0xbc40, 'HangulSyllableBaem', [], 'Display', None],
  [ 0xbc41, 'HangulSyllableBaeb', [], 'Display', None],
  [ 0xbc42, 'HangulSyllableBaebs', [], 'Display', None],
  [ 0xbc43, 'HangulSyllableBaes', [], 'Display', None],
  [ 0xbc44, 'HangulSyllableBaess', [], 'Display', None],
  [ 0xbc45, 'HangulSyllableBaeng', [], 'Display', None],
  [ 0xbc46, 'HangulSyllableBaej', [], 'Display', None],
  [ 0xbc47, 'HangulSyllableBaec', [], 'Display', None],
  [ 0xbc48, 'HangulSyllableBaek', [], 'Display', None],
  [ 0xbc49, 'HangulSyllableBaet', [], 'Display', None],
  [ 0xbc4a, 'HangulSyllableBaep', [], 'Display', None],
  [ 0xbc4b, 'HangulSyllableBaeh', [], 'Display', None],
  [ 0xbc4c, 'HangulSyllableBya', [], 'Display', None],
  [ 0xbc4d, 'HangulSyllableByag', [], 'Display', None],
  [ 0xbc4e, 'HangulSyllableByagg', [], 'Display', None],
  [ 0xbc4f, 'HangulSyllableByags', [], 'Display', None],
  [ 0xbc50, 'HangulSyllableByan', [], 'Display', None],
  [ 0xbc51, 'HangulSyllableByanj', [], 'Display', None],
  [ 0xbc52, 'HangulSyllableByanh', [], 'Display', None],
  [ 0xbc53, 'HangulSyllableByad', [], 'Display', None],
  [ 0xbc54, 'HangulSyllableByal', [], 'Display', None],
  [ 0xbc55, 'HangulSyllableByalg', [], 'Display', None],
  [ 0xbc56, 'HangulSyllableByalm', [], 'Display', None],
  [ 0xbc57, 'HangulSyllableByalb', [], 'Display', None],
  [ 0xbc58, 'HangulSyllableByals', [], 'Display', None],
  [ 0xbc59, 'HangulSyllableByalt', [], 'Display', None],
  [ 0xbc5a, 'HangulSyllableByalp', [], 'Display', None],
  [ 0xbc5b, 'HangulSyllableByalh', [], 'Display', None],
  [ 0xbc5c, 'HangulSyllableByam', [], 'Display', None],
  [ 0xbc5d, 'HangulSyllableByab', [], 'Display', None],
  [ 0xbc5e, 'HangulSyllableByabs', [], 'Display', None],
  [ 0xbc5f, 'HangulSyllableByas', [], 'Display', None],
  [ 0xbc60, 'HangulSyllableByass', [], 'Display', None],
  [ 0xbc61, 'HangulSyllableByang', [], 'Display', None],
  [ 0xbc62, 'HangulSyllableByaj', [], 'Display', None],
  [ 0xbc63, 'HangulSyllableByac', [], 'Display', None],
  [ 0xbc64, 'HangulSyllableByak', [], 'Display', None],
  [ 0xbc65, 'HangulSyllableByat', [], 'Display', None],
  [ 0xbc66, 'HangulSyllableByap', [], 'Display', None],
  [ 0xbc67, 'HangulSyllableByah', [], 'Display', None],
  [ 0xbc68, 'HangulSyllableByae', [], 'Display', None],
  [ 0xbc69, 'HangulSyllableByaeg', [], 'Display', None],
  [ 0xbc6a, 'HangulSyllableByaegg', [], 'Display', None],
  [ 0xbc6b, 'HangulSyllableByaegs', [], 'Display', None],
  [ 0xbc6c, 'HangulSyllableByaen', [], 'Display', None],
  [ 0xbc6d, 'HangulSyllableByaenj', [], 'Display', None],
  [ 0xbc6e, 'HangulSyllableByaenh', [], 'Display', None],
  [ 0xbc6f, 'HangulSyllableByaed', [], 'Display', None],
  [ 0xbc70, 'HangulSyllableByael', [], 'Display', None],
  [ 0xbc71, 'HangulSyllableByaelg', [], 'Display', None],
  [ 0xbc72, 'HangulSyllableByaelm', [], 'Display', None],
  [ 0xbc73, 'HangulSyllableByaelb', [], 'Display', None],
  [ 0xbc74, 'HangulSyllableByaels', [], 'Display', None],
  [ 0xbc75, 'HangulSyllableByaelt', [], 'Display', None],
  [ 0xbc76, 'HangulSyllableByaelp', [], 'Display', None],
  [ 0xbc77, 'HangulSyllableByaelh', [], 'Display', None],
  [ 0xbc78, 'HangulSyllableByaem', [], 'Display', None],
  [ 0xbc79, 'HangulSyllableByaeb', [], 'Display', None],
  [ 0xbc7a, 'HangulSyllableByaebs', [], 'Display', None],
  [ 0xbc7b, 'HangulSyllableByaes', [], 'Display', None],
  [ 0xbc7c, 'HangulSyllableByaess', [], 'Display', None],
  [ 0xbc7d, 'HangulSyllableByaeng', [], 'Display', None],
  [ 0xbc7e, 'HangulSyllableByaej', [], 'Display', None],
  [ 0xbc7f, 'HangulSyllableByaec', [], 'Display', None],
  [ 0xbc80, 'HangulSyllableByaek', [], 'Display', None],
  [ 0xbc81, 'HangulSyllableByaet', [], 'Display', None],
  [ 0xbc82, 'HangulSyllableByaep', [], 'Display', None],
  [ 0xbc83, 'HangulSyllableByaeh', [], 'Display', None],
  [ 0xbc84, 'HangulSyllableBeo', [], 'Display', None],
  [ 0xbc85, 'HangulSyllableBeog', [], 'Display', None],
  [ 0xbc86, 'HangulSyllableBeogg', [], 'Display', None],
  [ 0xbc87, 'HangulSyllableBeogs', [], 'Display', None],
  [ 0xbc88, 'HangulSyllableBeon', [], 'Display', None],
  [ 0xbc89, 'HangulSyllableBeonj', [], 'Display', None],
  [ 0xbc8a, 'HangulSyllableBeonh', [], 'Display', None],
  [ 0xbc8b, 'HangulSyllableBeod', [], 'Display', None],
  [ 0xbc8c, 'HangulSyllableBeol', [], 'Display', None],
  [ 0xbc8d, 'HangulSyllableBeolg', [], 'Display', None],
  [ 0xbc8e, 'HangulSyllableBeolm', [], 'Display', None],
  [ 0xbc8f, 'HangulSyllableBeolb', [], 'Display', None],
  [ 0xbc90, 'HangulSyllableBeols', [], 'Display', None],
  [ 0xbc91, 'HangulSyllableBeolt', [], 'Display', None],
  [ 0xbc92, 'HangulSyllableBeolp', [], 'Display', None],
  [ 0xbc93, 'HangulSyllableBeolh', [], 'Display', None],
  [ 0xbc94, 'HangulSyllableBeom', [], 'Display', None],
  [ 0xbc95, 'HangulSyllableBeob', [], 'Display', None],
  [ 0xbc96, 'HangulSyllableBeobs', [], 'Display', None],
  [ 0xbc97, 'HangulSyllableBeos', [], 'Display', None],
  [ 0xbc98, 'HangulSyllableBeoss', [], 'Display', None],
  [ 0xbc99, 'HangulSyllableBeong', [], 'Display', None],
  [ 0xbc9a, 'HangulSyllableBeoj', [], 'Display', None],
  [ 0xbc9b, 'HangulSyllableBeoc', [], 'Display', None],
  [ 0xbc9c, 'HangulSyllableBeok', [], 'Display', None],
  [ 0xbc9d, 'HangulSyllableBeot', [], 'Display', None],
  [ 0xbc9e, 'HangulSyllableBeop', [], 'Display', None],
  [ 0xbc9f, 'HangulSyllableBeoh', [], 'Display', None],
  [ 0xbca0, 'HangulSyllableBe', [], 'Display', None],
  [ 0xbca1, 'HangulSyllableBeg', [], 'Display', None],
  [ 0xbca2, 'HangulSyllableBegg', [], 'Display', None],
  [ 0xbca3, 'HangulSyllableBegs', [], 'Display', None],
  [ 0xbca4, 'HangulSyllableBen', [], 'Display', None],
  [ 0xbca5, 'HangulSyllableBenj', [], 'Display', None],
  [ 0xbca6, 'HangulSyllableBenh', [], 'Display', None],
  [ 0xbca7, 'HangulSyllableBed', [], 'Display', None],
  [ 0xbca8, 'HangulSyllableBel', [], 'Display', None],
  [ 0xbca9, 'HangulSyllableBelg', [], 'Display', None],
  [ 0xbcaa, 'HangulSyllableBelm', [], 'Display', None],
  [ 0xbcab, 'HangulSyllableBelb', [], 'Display', None],
  [ 0xbcac, 'HangulSyllableBels', [], 'Display', None],
  [ 0xbcad, 'HangulSyllableBelt', [], 'Display', None],
  [ 0xbcae, 'HangulSyllableBelp', [], 'Display', None],
  [ 0xbcaf, 'HangulSyllableBelh', [], 'Display', None],
  [ 0xbcb0, 'HangulSyllableBem', [], 'Display', None],
  [ 0xbcb1, 'HangulSyllableBeb', [], 'Display', None],
  [ 0xbcb2, 'HangulSyllableBebs', [], 'Display', None],
  [ 0xbcb3, 'HangulSyllableBes', [], 'Display', None],
  [ 0xbcb4, 'HangulSyllableBess', [], 'Display', None],
  [ 0xbcb5, 'HangulSyllableBeng', [], 'Display', None],
  [ 0xbcb6, 'HangulSyllableBej', [], 'Display', None],
  [ 0xbcb7, 'HangulSyllableBec', [], 'Display', None],
  [ 0xbcb8, 'HangulSyllableBek', [], 'Display', None],
  [ 0xbcb9, 'HangulSyllableBet', [], 'Display', None],
  [ 0xbcba, 'HangulSyllableBep', [], 'Display', None],
  [ 0xbcbb, 'HangulSyllableBeh', [], 'Display', None],
  [ 0xbcbc, 'HangulSyllableByeo', [], 'Display', None],
  [ 0xbcbd, 'HangulSyllableByeog', [], 'Display', None],
  [ 0xbcbe, 'HangulSyllableByeogg', [], 'Display', None],
  [ 0xbcbf, 'HangulSyllableByeogs', [], 'Display', None],
  [ 0xbcc0, 'HangulSyllableByeon', [], 'Display', None],
  [ 0xbcc1, 'HangulSyllableByeonj', [], 'Display', None],
  [ 0xbcc2, 'HangulSyllableByeonh', [], 'Display', None],
  [ 0xbcc3, 'HangulSyllableByeod', [], 'Display', None],
  [ 0xbcc4, 'HangulSyllableByeol', [], 'Display', None],
  [ 0xbcc5, 'HangulSyllableByeolg', [], 'Display', None],
  [ 0xbcc6, 'HangulSyllableByeolm', [], 'Display', None],
  [ 0xbcc7, 'HangulSyllableByeolb', [], 'Display', None],
  [ 0xbcc8, 'HangulSyllableByeols', [], 'Display', None],
  [ 0xbcc9, 'HangulSyllableByeolt', [], 'Display', None],
  [ 0xbcca, 'HangulSyllableByeolp', [], 'Display', None],
  [ 0xbccb, 'HangulSyllableByeolh', [], 'Display', None],
  [ 0xbccc, 'HangulSyllableByeom', [], 'Display', None],
  [ 0xbccd, 'HangulSyllableByeob', [], 'Display', None],
  [ 0xbcce, 'HangulSyllableByeobs', [], 'Display', None],
  [ 0xbccf, 'HangulSyllableByeos', [], 'Display', None],
  [ 0xbcd0, 'HangulSyllableByeoss', [], 'Display', None],
  [ 0xbcd1, 'HangulSyllableByeong', [], 'Display', None],
  [ 0xbcd2, 'HangulSyllableByeoj', [], 'Display', None],
  [ 0xbcd3, 'HangulSyllableByeoc', [], 'Display', None],
  [ 0xbcd4, 'HangulSyllableByeok', [], 'Display', None],
  [ 0xbcd5, 'HangulSyllableByeot', [], 'Display', None],
  [ 0xbcd6, 'HangulSyllableByeop', [], 'Display', None],
  [ 0xbcd7, 'HangulSyllableByeoh', [], 'Display', None],
  [ 0xbcd8, 'HangulSyllableBye', [], 'Display', None],
  [ 0xbcd9, 'HangulSyllableByeg', [], 'Display', None],
  [ 0xbcda, 'HangulSyllableByegg', [], 'Display', None],
  [ 0xbcdb, 'HangulSyllableByegs', [], 'Display', None],
  [ 0xbcdc, 'HangulSyllableByen', [], 'Display', None],
  [ 0xbcdd, 'HangulSyllableByenj', [], 'Display', None],
  [ 0xbcde, 'HangulSyllableByenh', [], 'Display', None],
  [ 0xbcdf, 'HangulSyllableByed', [], 'Display', None],
  [ 0xbce0, 'HangulSyllableByel', [], 'Display', None],
  [ 0xbce1, 'HangulSyllableByelg', [], 'Display', None],
  [ 0xbce2, 'HangulSyllableByelm', [], 'Display', None],
  [ 0xbce3, 'HangulSyllableByelb', [], 'Display', None],
  [ 0xbce4, 'HangulSyllableByels', [], 'Display', None],
  [ 0xbce5, 'HangulSyllableByelt', [], 'Display', None],
  [ 0xbce6, 'HangulSyllableByelp', [], 'Display', None],
  [ 0xbce7, 'HangulSyllableByelh', [], 'Display', None],
  [ 0xbce8, 'HangulSyllableByem', [], 'Display', None],
  [ 0xbce9, 'HangulSyllableByeb', [], 'Display', None],
  [ 0xbcea, 'HangulSyllableByebs', [], 'Display', None],
  [ 0xbceb, 'HangulSyllableByes', [], 'Display', None],
  [ 0xbcec, 'HangulSyllableByess', [], 'Display', None],
  [ 0xbced, 'HangulSyllableByeng', [], 'Display', None],
  [ 0xbcee, 'HangulSyllableByej', [], 'Display', None],
  [ 0xbcef, 'HangulSyllableByec', [], 'Display', None],
  [ 0xbcf0, 'HangulSyllableByek', [], 'Display', None],
  [ 0xbcf1, 'HangulSyllableByet', [], 'Display', None],
  [ 0xbcf2, 'HangulSyllableByep', [], 'Display', None],
  [ 0xbcf3, 'HangulSyllableByeh', [], 'Display', None],
  [ 0xbcf4, 'HangulSyllableBo', [], 'Display', None],
  [ 0xbcf5, 'HangulSyllableBog', [], 'Display', None],
  [ 0xbcf6, 'HangulSyllableBogg', [], 'Display', None],
  [ 0xbcf7, 'HangulSyllableBogs', [], 'Display', None],
  [ 0xbcf8, 'HangulSyllableBon', [], 'Display', None],
  [ 0xbcf9, 'HangulSyllableBonj', [], 'Display', None],
  [ 0xbcfa, 'HangulSyllableBonh', [], 'Display', None],
  [ 0xbcfb, 'HangulSyllableBod', [], 'Display', None],
  [ 0xbcfc, 'HangulSyllableBol', [], 'Display', None],
  [ 0xbcfd, 'HangulSyllableBolg', [], 'Display', None],
  [ 0xbcfe, 'HangulSyllableBolm', [], 'Display', None],
  [ 0xbcff, 'HangulSyllableBolb', [], 'Display', None],
  [ 0xbd00, 'HangulSyllableBols', [], 'Display', None],
  [ 0xbd01, 'HangulSyllableBolt', [], 'Display', None],
  [ 0xbd02, 'HangulSyllableBolp', [], 'Display', None],
  [ 0xbd03, 'HangulSyllableBolh', [], 'Display', None],
  [ 0xbd04, 'HangulSyllableBom', [], 'Display', None],
  [ 0xbd05, 'HangulSyllableBob', [], 'Display', None],
  [ 0xbd06, 'HangulSyllableBobs', [], 'Display', None],
  [ 0xbd07, 'HangulSyllableBos', [], 'Display', None],
  [ 0xbd08, 'HangulSyllableBoss', [], 'Display', None],
  [ 0xbd09, 'HangulSyllableBong', [], 'Display', None],
  [ 0xbd0a, 'HangulSyllableBoj', [], 'Display', None],
  [ 0xbd0b, 'HangulSyllableBoc', [], 'Display', None],
  [ 0xbd0c, 'HangulSyllableBok', [], 'Display', None],
  [ 0xbd0d, 'HangulSyllableBot', [], 'Display', None],
  [ 0xbd0e, 'HangulSyllableBop', [], 'Display', None],
  [ 0xbd0f, 'HangulSyllableBoh', [], 'Display', None],
  [ 0xbd10, 'HangulSyllableBwa', [], 'Display', None],
  [ 0xbd11, 'HangulSyllableBwag', [], 'Display', None],
  [ 0xbd12, 'HangulSyllableBwagg', [], 'Display', None],
  [ 0xbd13, 'HangulSyllableBwags', [], 'Display', None],
  [ 0xbd14, 'HangulSyllableBwan', [], 'Display', None],
  [ 0xbd15, 'HangulSyllableBwanj', [], 'Display', None],
  [ 0xbd16, 'HangulSyllableBwanh', [], 'Display', None],
  [ 0xbd17, 'HangulSyllableBwad', [], 'Display', None],
  [ 0xbd18, 'HangulSyllableBwal', [], 'Display', None],
  [ 0xbd19, 'HangulSyllableBwalg', [], 'Display', None],
  [ 0xbd1a, 'HangulSyllableBwalm', [], 'Display', None],
  [ 0xbd1b, 'HangulSyllableBwalb', [], 'Display', None],
  [ 0xbd1c, 'HangulSyllableBwals', [], 'Display', None],
  [ 0xbd1d, 'HangulSyllableBwalt', [], 'Display', None],
  [ 0xbd1e, 'HangulSyllableBwalp', [], 'Display', None],
  [ 0xbd1f, 'HangulSyllableBwalh', [], 'Display', None],
  [ 0xbd20, 'HangulSyllableBwam', [], 'Display', None],
  [ 0xbd21, 'HangulSyllableBwab', [], 'Display', None],
  [ 0xbd22, 'HangulSyllableBwabs', [], 'Display', None],
  [ 0xbd23, 'HangulSyllableBwas', [], 'Display', None],
  [ 0xbd24, 'HangulSyllableBwass', [], 'Display', None],
  [ 0xbd25, 'HangulSyllableBwang', [], 'Display', None],
  [ 0xbd26, 'HangulSyllableBwaj', [], 'Display', None],
  [ 0xbd27, 'HangulSyllableBwac', [], 'Display', None],
  [ 0xbd28, 'HangulSyllableBwak', [], 'Display', None],
  [ 0xbd29, 'HangulSyllableBwat', [], 'Display', None],
  [ 0xbd2a, 'HangulSyllableBwap', [], 'Display', None],
  [ 0xbd2b, 'HangulSyllableBwah', [], 'Display', None],
  [ 0xbd2c, 'HangulSyllableBwae', [], 'Display', None],
  [ 0xbd2d, 'HangulSyllableBwaeg', [], 'Display', None],
  [ 0xbd2e, 'HangulSyllableBwaegg', [], 'Display', None],
  [ 0xbd2f, 'HangulSyllableBwaegs', [], 'Display', None],
  [ 0xbd30, 'HangulSyllableBwaen', [], 'Display', None],
  [ 0xbd31, 'HangulSyllableBwaenj', [], 'Display', None],
  [ 0xbd32, 'HangulSyllableBwaenh', [], 'Display', None],
  [ 0xbd33, 'HangulSyllableBwaed', [], 'Display', None],
  [ 0xbd34, 'HangulSyllableBwael', [], 'Display', None],
  [ 0xbd35, 'HangulSyllableBwaelg', [], 'Display', None],
  [ 0xbd36, 'HangulSyllableBwaelm', [], 'Display', None],
  [ 0xbd37, 'HangulSyllableBwaelb', [], 'Display', None],
  [ 0xbd38, 'HangulSyllableBwaels', [], 'Display', None],
  [ 0xbd39, 'HangulSyllableBwaelt', [], 'Display', None],
  [ 0xbd3a, 'HangulSyllableBwaelp', [], 'Display', None],
  [ 0xbd3b, 'HangulSyllableBwaelh', [], 'Display', None],
  [ 0xbd3c, 'HangulSyllableBwaem', [], 'Display', None],
  [ 0xbd3d, 'HangulSyllableBwaeb', [], 'Display', None],
  [ 0xbd3e, 'HangulSyllableBwaebs', [], 'Display', None],
  [ 0xbd3f, 'HangulSyllableBwaes', [], 'Display', None],
  [ 0xbd40, 'HangulSyllableBwaess', [], 'Display', None],
  [ 0xbd41, 'HangulSyllableBwaeng', [], 'Display', None],
  [ 0xbd42, 'HangulSyllableBwaej', [], 'Display', None],
  [ 0xbd43, 'HangulSyllableBwaec', [], 'Display', None],
  [ 0xbd44, 'HangulSyllableBwaek', [], 'Display', None],
  [ 0xbd45, 'HangulSyllableBwaet', [], 'Display', None],
  [ 0xbd46, 'HangulSyllableBwaep', [], 'Display', None],
  [ 0xbd47, 'HangulSyllableBwaeh', [], 'Display', None],
  [ 0xbd48, 'HangulSyllableBoe', [], 'Display', None],
  [ 0xbd49, 'HangulSyllableBoeg', [], 'Display', None],
  [ 0xbd4a, 'HangulSyllableBoegg', [], 'Display', None],
  [ 0xbd4b, 'HangulSyllableBoegs', [], 'Display', None],
  [ 0xbd4c, 'HangulSyllableBoen', [], 'Display', None],
  [ 0xbd4d, 'HangulSyllableBoenj', [], 'Display', None],
  [ 0xbd4e, 'HangulSyllableBoenh', [], 'Display', None],
  [ 0xbd4f, 'HangulSyllableBoed', [], 'Display', None],
  [ 0xbd50, 'HangulSyllableBoel', [], 'Display', None],
  [ 0xbd51, 'HangulSyllableBoelg', [], 'Display', None],
  [ 0xbd52, 'HangulSyllableBoelm', [], 'Display', None],
  [ 0xbd53, 'HangulSyllableBoelb', [], 'Display', None],
  [ 0xbd54, 'HangulSyllableBoels', [], 'Display', None],
  [ 0xbd55, 'HangulSyllableBoelt', [], 'Display', None],
  [ 0xbd56, 'HangulSyllableBoelp', [], 'Display', None],
  [ 0xbd57, 'HangulSyllableBoelh', [], 'Display', None],
  [ 0xbd58, 'HangulSyllableBoem', [], 'Display', None],
  [ 0xbd59, 'HangulSyllableBoeb', [], 'Display', None],
  [ 0xbd5a, 'HangulSyllableBoebs', [], 'Display', None],
  [ 0xbd5b, 'HangulSyllableBoes', [], 'Display', None],
  [ 0xbd5c, 'HangulSyllableBoess', [], 'Display', None],
  [ 0xbd5d, 'HangulSyllableBoeng', [], 'Display', None],
  [ 0xbd5e, 'HangulSyllableBoej', [], 'Display', None],
  [ 0xbd5f, 'HangulSyllableBoec', [], 'Display', None],
  [ 0xbd60, 'HangulSyllableBoek', [], 'Display', None],
  [ 0xbd61, 'HangulSyllableBoet', [], 'Display', None],
  [ 0xbd62, 'HangulSyllableBoep', [], 'Display', None],
  [ 0xbd63, 'HangulSyllableBoeh', [], 'Display', None],
  [ 0xbd64, 'HangulSyllableByo', [], 'Display', None],
  [ 0xbd65, 'HangulSyllableByog', [], 'Display', None],
  [ 0xbd66, 'HangulSyllableByogg', [], 'Display', None],
  [ 0xbd67, 'HangulSyllableByogs', [], 'Display', None],
  [ 0xbd68, 'HangulSyllableByon', [], 'Display', None],
  [ 0xbd69, 'HangulSyllableByonj', [], 'Display', None],
  [ 0xbd6a, 'HangulSyllableByonh', [], 'Display', None],
  [ 0xbd6b, 'HangulSyllableByod', [], 'Display', None],
  [ 0xbd6c, 'HangulSyllableByol', [], 'Display', None],
  [ 0xbd6d, 'HangulSyllableByolg', [], 'Display', None],
  [ 0xbd6e, 'HangulSyllableByolm', [], 'Display', None],
  [ 0xbd6f, 'HangulSyllableByolb', [], 'Display', None],
  [ 0xbd70, 'HangulSyllableByols', [], 'Display', None],
  [ 0xbd71, 'HangulSyllableByolt', [], 'Display', None],
  [ 0xbd72, 'HangulSyllableByolp', [], 'Display', None],
  [ 0xbd73, 'HangulSyllableByolh', [], 'Display', None],
  [ 0xbd74, 'HangulSyllableByom', [], 'Display', None],
  [ 0xbd75, 'HangulSyllableByob', [], 'Display', None],
  [ 0xbd76, 'HangulSyllableByobs', [], 'Display', None],
  [ 0xbd77, 'HangulSyllableByos', [], 'Display', None],
  [ 0xbd78, 'HangulSyllableByoss', [], 'Display', None],
  [ 0xbd79, 'HangulSyllableByong', [], 'Display', None],
  [ 0xbd7a, 'HangulSyllableByoj', [], 'Display', None],
  [ 0xbd7b, 'HangulSyllableByoc', [], 'Display', None],
  [ 0xbd7c, 'HangulSyllableByok', [], 'Display', None],
  [ 0xbd7d, 'HangulSyllableByot', [], 'Display', None],
  [ 0xbd7e, 'HangulSyllableByop', [], 'Display', None],
  [ 0xbd7f, 'HangulSyllableByoh', [], 'Display', None],
  [ 0xbd80, 'HangulSyllableBu', [], 'Display', None],
  [ 0xbd81, 'HangulSyllableBug', [], 'Display', None],
  [ 0xbd82, 'HangulSyllableBugg', [], 'Display', None],
  [ 0xbd83, 'HangulSyllableBugs', [], 'Display', None],
  [ 0xbd84, 'HangulSyllableBun', [], 'Display', None],
  [ 0xbd85, 'HangulSyllableBunj', [], 'Display', None],
  [ 0xbd86, 'HangulSyllableBunh', [], 'Display', None],
  [ 0xbd87, 'HangulSyllableBud', [], 'Display', None],
  [ 0xbd88, 'HangulSyllableBul', [], 'Display', None],
  [ 0xbd89, 'HangulSyllableBulg', [], 'Display', None],
  [ 0xbd8a, 'HangulSyllableBulm', [], 'Display', None],
  [ 0xbd8b, 'HangulSyllableBulb', [], 'Display', None],
  [ 0xbd8c, 'HangulSyllableBuls', [], 'Display', None],
  [ 0xbd8d, 'HangulSyllableBult', [], 'Display', None],
  [ 0xbd8e, 'HangulSyllableBulp', [], 'Display', None],
  [ 0xbd8f, 'HangulSyllableBulh', [], 'Display', None],
  [ 0xbd90, 'HangulSyllableBum', [], 'Display', None],
  [ 0xbd91, 'HangulSyllableBub', [], 'Display', None],
  [ 0xbd92, 'HangulSyllableBubs', [], 'Display', None],
  [ 0xbd93, 'HangulSyllableBus', [], 'Display', None],
  [ 0xbd94, 'HangulSyllableBuss', [], 'Display', None],
  [ 0xbd95, 'HangulSyllableBung', [], 'Display', None],
  [ 0xbd96, 'HangulSyllableBuj', [], 'Display', None],
  [ 0xbd97, 'HangulSyllableBuc', [], 'Display', None],
  [ 0xbd98, 'HangulSyllableBuk', [], 'Display', None],
  [ 0xbd99, 'HangulSyllableBut', [], 'Display', None],
  [ 0xbd9a, 'HangulSyllableBup', [], 'Display', None],
  [ 0xbd9b, 'HangulSyllableBuh', [], 'Display', None],
  [ 0xbd9c, 'HangulSyllableBweo', [], 'Display', None],
  [ 0xbd9d, 'HangulSyllableBweog', [], 'Display', None],
  [ 0xbd9e, 'HangulSyllableBweogg', [], 'Display', None],
  [ 0xbd9f, 'HangulSyllableBweogs', [], 'Display', None],
  [ 0xbda0, 'HangulSyllableBweon', [], 'Display', None],
  [ 0xbda1, 'HangulSyllableBweonj', [], 'Display', None],
  [ 0xbda2, 'HangulSyllableBweonh', [], 'Display', None],
  [ 0xbda3, 'HangulSyllableBweod', [], 'Display', None],
  [ 0xbda4, 'HangulSyllableBweol', [], 'Display', None],
  [ 0xbda5, 'HangulSyllableBweolg', [], 'Display', None],
  [ 0xbda6, 'HangulSyllableBweolm', [], 'Display', None],
  [ 0xbda7, 'HangulSyllableBweolb', [], 'Display', None],
  [ 0xbda8, 'HangulSyllableBweols', [], 'Display', None],
  [ 0xbda9, 'HangulSyllableBweolt', [], 'Display', None],
  [ 0xbdaa, 'HangulSyllableBweolp', [], 'Display', None],
  [ 0xbdab, 'HangulSyllableBweolh', [], 'Display', None],
  [ 0xbdac, 'HangulSyllableBweom', [], 'Display', None],
  [ 0xbdad, 'HangulSyllableBweob', [], 'Display', None],
  [ 0xbdae, 'HangulSyllableBweobs', [], 'Display', None],
  [ 0xbdaf, 'HangulSyllableBweos', [], 'Display', None],
  [ 0xbdb0, 'HangulSyllableBweoss', [], 'Display', None],
  [ 0xbdb1, 'HangulSyllableBweong', [], 'Display', None],
  [ 0xbdb2, 'HangulSyllableBweoj', [], 'Display', None],
  [ 0xbdb3, 'HangulSyllableBweoc', [], 'Display', None],
  [ 0xbdb4, 'HangulSyllableBweok', [], 'Display', None],
  [ 0xbdb5, 'HangulSyllableBweot', [], 'Display', None],
  [ 0xbdb6, 'HangulSyllableBweop', [], 'Display', None],
  [ 0xbdb7, 'HangulSyllableBweoh', [], 'Display', None],
  [ 0xbdb8, 'HangulSyllableBwe', [], 'Display', None],
  [ 0xbdb9, 'HangulSyllableBweg', [], 'Display', None],
  [ 0xbdba, 'HangulSyllableBwegg', [], 'Display', None],
  [ 0xbdbb, 'HangulSyllableBwegs', [], 'Display', None],
  [ 0xbdbc, 'HangulSyllableBwen', [], 'Display', None],
  [ 0xbdbd, 'HangulSyllableBwenj', [], 'Display', None],
  [ 0xbdbe, 'HangulSyllableBwenh', [], 'Display', None],
  [ 0xbdbf, 'HangulSyllableBwed', [], 'Display', None],
  [ 0xbdc0, 'HangulSyllableBwel', [], 'Display', None],
  [ 0xbdc1, 'HangulSyllableBwelg', [], 'Display', None],
  [ 0xbdc2, 'HangulSyllableBwelm', [], 'Display', None],
  [ 0xbdc3, 'HangulSyllableBwelb', [], 'Display', None],
  [ 0xbdc4, 'HangulSyllableBwels', [], 'Display', None],
  [ 0xbdc5, 'HangulSyllableBwelt', [], 'Display', None],
  [ 0xbdc6, 'HangulSyllableBwelp', [], 'Display', None],
  [ 0xbdc7, 'HangulSyllableBwelh', [], 'Display', None],
  [ 0xbdc8, 'HangulSyllableBwem', [], 'Display', None],
  [ 0xbdc9, 'HangulSyllableBweb', [], 'Display', None],
  [ 0xbdca, 'HangulSyllableBwebs', [], 'Display', None],
  [ 0xbdcb, 'HangulSyllableBwes', [], 'Display', None],
  [ 0xbdcc, 'HangulSyllableBwess', [], 'Display', None],
  [ 0xbdcd, 'HangulSyllableBweng', [], 'Display', None],
  [ 0xbdce, 'HangulSyllableBwej', [], 'Display', None],
  [ 0xbdcf, 'HangulSyllableBwec', [], 'Display', None],
  [ 0xbdd0, 'HangulSyllableBwek', [], 'Display', None],
  [ 0xbdd1, 'HangulSyllableBwet', [], 'Display', None],
  [ 0xbdd2, 'HangulSyllableBwep', [], 'Display', None],
  [ 0xbdd3, 'HangulSyllableBweh', [], 'Display', None],
  [ 0xbdd4, 'HangulSyllableBwi', [], 'Display', None],
  [ 0xbdd5, 'HangulSyllableBwig', [], 'Display', None],
  [ 0xbdd6, 'HangulSyllableBwigg', [], 'Display', None],
  [ 0xbdd7, 'HangulSyllableBwigs', [], 'Display', None],
  [ 0xbdd8, 'HangulSyllableBwin', [], 'Display', None],
  [ 0xbdd9, 'HangulSyllableBwinj', [], 'Display', None],
  [ 0xbdda, 'HangulSyllableBwinh', [], 'Display', None],
  [ 0xbddb, 'HangulSyllableBwid', [], 'Display', None],
  [ 0xbddc, 'HangulSyllableBwil', [], 'Display', None],
  [ 0xbddd, 'HangulSyllableBwilg', [], 'Display', None],
  [ 0xbdde, 'HangulSyllableBwilm', [], 'Display', None],
  [ 0xbddf, 'HangulSyllableBwilb', [], 'Display', None],
  [ 0xbde0, 'HangulSyllableBwils', [], 'Display', None],
  [ 0xbde1, 'HangulSyllableBwilt', [], 'Display', None],
  [ 0xbde2, 'HangulSyllableBwilp', [], 'Display', None],
  [ 0xbde3, 'HangulSyllableBwilh', [], 'Display', None],
  [ 0xbde4, 'HangulSyllableBwim', [], 'Display', None],
  [ 0xbde5, 'HangulSyllableBwib', [], 'Display', None],
  [ 0xbde6, 'HangulSyllableBwibs', [], 'Display', None],
  [ 0xbde7, 'HangulSyllableBwis', [], 'Display', None],
  [ 0xbde8, 'HangulSyllableBwiss', [], 'Display', None],
  [ 0xbde9, 'HangulSyllableBwing', [], 'Display', None],
  [ 0xbdea, 'HangulSyllableBwij', [], 'Display', None],
  [ 0xbdeb, 'HangulSyllableBwic', [], 'Display', None],
  [ 0xbdec, 'HangulSyllableBwik', [], 'Display', None],
  [ 0xbded, 'HangulSyllableBwit', [], 'Display', None],
  [ 0xbdee, 'HangulSyllableBwip', [], 'Display', None],
  [ 0xbdef, 'HangulSyllableBwih', [], 'Display', None],
  [ 0xbdf0, 'HangulSyllableByu', [], 'Display', None],
  [ 0xbdf1, 'HangulSyllableByug', [], 'Display', None],
  [ 0xbdf2, 'HangulSyllableByugg', [], 'Display', None],
  [ 0xbdf3, 'HangulSyllableByugs', [], 'Display', None],
  [ 0xbdf4, 'HangulSyllableByun', [], 'Display', None],
  [ 0xbdf5, 'HangulSyllableByunj', [], 'Display', None],
  [ 0xbdf6, 'HangulSyllableByunh', [], 'Display', None],
  [ 0xbdf7, 'HangulSyllableByud', [], 'Display', None],
  [ 0xbdf8, 'HangulSyllableByul', [], 'Display', None],
  [ 0xbdf9, 'HangulSyllableByulg', [], 'Display', None],
  [ 0xbdfa, 'HangulSyllableByulm', [], 'Display', None],
  [ 0xbdfb, 'HangulSyllableByulb', [], 'Display', None],
  [ 0xbdfc, 'HangulSyllableByuls', [], 'Display', None],
  [ 0xbdfd, 'HangulSyllableByult', [], 'Display', None],
  [ 0xbdfe, 'HangulSyllableByulp', [], 'Display', None],
  [ 0xbdff, 'HangulSyllableByulh', [], 'Display', None],
  [ 0xbe00, 'HangulSyllableByum', [], 'Display', None],
  [ 0xbe01, 'HangulSyllableByub', [], 'Display', None],
  [ 0xbe02, 'HangulSyllableByubs', [], 'Display', None],
  [ 0xbe03, 'HangulSyllableByus', [], 'Display', None],
  [ 0xbe04, 'HangulSyllableByuss', [], 'Display', None],
  [ 0xbe05, 'HangulSyllableByung', [], 'Display', None],
  [ 0xbe06, 'HangulSyllableByuj', [], 'Display', None],
  [ 0xbe07, 'HangulSyllableByuc', [], 'Display', None],
  [ 0xbe08, 'HangulSyllableByuk', [], 'Display', None],
  [ 0xbe09, 'HangulSyllableByut', [], 'Display', None],
  [ 0xbe0a, 'HangulSyllableByup', [], 'Display', None],
  [ 0xbe0b, 'HangulSyllableByuh', [], 'Display', None],
  [ 0xbe0c, 'HangulSyllableBeu', [], 'Display', None],
  [ 0xbe0d, 'HangulSyllableBeug', [], 'Display', None],
  [ 0xbe0e, 'HangulSyllableBeugg', [], 'Display', None],
  [ 0xbe0f, 'HangulSyllableBeugs', [], 'Display', None],
  [ 0xbe10, 'HangulSyllableBeun', [], 'Display', None],
  [ 0xbe11, 'HangulSyllableBeunj', [], 'Display', None],
  [ 0xbe12, 'HangulSyllableBeunh', [], 'Display', None],
  [ 0xbe13, 'HangulSyllableBeud', [], 'Display', None],
  [ 0xbe14, 'HangulSyllableBeul', [], 'Display', None],
  [ 0xbe15, 'HangulSyllableBeulg', [], 'Display', None],
  [ 0xbe16, 'HangulSyllableBeulm', [], 'Display', None],
  [ 0xbe17, 'HangulSyllableBeulb', [], 'Display', None],
  [ 0xbe18, 'HangulSyllableBeuls', [], 'Display', None],
  [ 0xbe19, 'HangulSyllableBeult', [], 'Display', None],
  [ 0xbe1a, 'HangulSyllableBeulp', [], 'Display', None],
  [ 0xbe1b, 'HangulSyllableBeulh', [], 'Display', None],
  [ 0xbe1c, 'HangulSyllableBeum', [], 'Display', None],
  [ 0xbe1d, 'HangulSyllableBeub', [], 'Display', None],
  [ 0xbe1e, 'HangulSyllableBeubs', [], 'Display', None],
  [ 0xbe1f, 'HangulSyllableBeus', [], 'Display', None],
  [ 0xbe20, 'HangulSyllableBeuss', [], 'Display', None],
  [ 0xbe21, 'HangulSyllableBeung', [], 'Display', None],
  [ 0xbe22, 'HangulSyllableBeuj', [], 'Display', None],
  [ 0xbe23, 'HangulSyllableBeuc', [], 'Display', None],
  [ 0xbe24, 'HangulSyllableBeuk', [], 'Display', None],
  [ 0xbe25, 'HangulSyllableBeut', [], 'Display', None],
  [ 0xbe26, 'HangulSyllableBeup', [], 'Display', None],
  [ 0xbe27, 'HangulSyllableBeuh', [], 'Display', None],
  [ 0xbe28, 'HangulSyllableByi', [], 'Display', None],
  [ 0xbe29, 'HangulSyllableByig', [], 'Display', None],
  [ 0xbe2a, 'HangulSyllableByigg', [], 'Display', None],
  [ 0xbe2b, 'HangulSyllableByigs', [], 'Display', None],
  [ 0xbe2c, 'HangulSyllableByin', [], 'Display', None],
  [ 0xbe2d, 'HangulSyllableByinj', [], 'Display', None],
  [ 0xbe2e, 'HangulSyllableByinh', [], 'Display', None],
  [ 0xbe2f, 'HangulSyllableByid', [], 'Display', None],
  [ 0xbe30, 'HangulSyllableByil', [], 'Display', None],
  [ 0xbe31, 'HangulSyllableByilg', [], 'Display', None],
  [ 0xbe32, 'HangulSyllableByilm', [], 'Display', None],
  [ 0xbe33, 'HangulSyllableByilb', [], 'Display', None],
  [ 0xbe34, 'HangulSyllableByils', [], 'Display', None],
  [ 0xbe35, 'HangulSyllableByilt', [], 'Display', None],
  [ 0xbe36, 'HangulSyllableByilp', [], 'Display', None],
  [ 0xbe37, 'HangulSyllableByilh', [], 'Display', None],
  [ 0xbe38, 'HangulSyllableByim', [], 'Display', None],
  [ 0xbe39, 'HangulSyllableByib', [], 'Display', None],
  [ 0xbe3a, 'HangulSyllableByibs', [], 'Display', None],
  [ 0xbe3b, 'HangulSyllableByis', [], 'Display', None],
  [ 0xbe3c, 'HangulSyllableByiss', [], 'Display', None],
  [ 0xbe3d, 'HangulSyllableBying', [], 'Display', None],
  [ 0xbe3e, 'HangulSyllableByij', [], 'Display', None],
  [ 0xbe3f, 'HangulSyllableByic', [], 'Display', None],
  [ 0xbe40, 'HangulSyllableByik', [], 'Display', None],
  [ 0xbe41, 'HangulSyllableByit', [], 'Display', None],
  [ 0xbe42, 'HangulSyllableByip', [], 'Display', None],
  [ 0xbe43, 'HangulSyllableByih', [], 'Display', None],
  [ 0xbe44, 'HangulSyllableBi', [], 'Display', None],
  [ 0xbe45, 'HangulSyllableBig', [], 'Display', None],
  [ 0xbe46, 'HangulSyllableBigg', [], 'Display', None],
  [ 0xbe47, 'HangulSyllableBigs', [], 'Display', None],
  [ 0xbe48, 'HangulSyllableBin', [], 'Display', None],
  [ 0xbe49, 'HangulSyllableBinj', [], 'Display', None],
  [ 0xbe4a, 'HangulSyllableBinh', [], 'Display', None],
  [ 0xbe4b, 'HangulSyllableBid', [], 'Display', None],
  [ 0xbe4c, 'HangulSyllableBil', [], 'Display', None],
  [ 0xbe4d, 'HangulSyllableBilg', [], 'Display', None],
  [ 0xbe4e, 'HangulSyllableBilm', [], 'Display', None],
  [ 0xbe4f, 'HangulSyllableBilb', [], 'Display', None],
  [ 0xbe50, 'HangulSyllableBils', [], 'Display', None],
  [ 0xbe51, 'HangulSyllableBilt', [], 'Display', None],
  [ 0xbe52, 'HangulSyllableBilp', [], 'Display', None],
  [ 0xbe53, 'HangulSyllableBilh', [], 'Display', None],
  [ 0xbe54, 'HangulSyllableBim', [], 'Display', None],
  [ 0xbe55, 'HangulSyllableBib', [], 'Display', None],
  [ 0xbe56, 'HangulSyllableBibs', [], 'Display', None],
  [ 0xbe57, 'HangulSyllableBis', [], 'Display', None],
  [ 0xbe58, 'HangulSyllableBiss', [], 'Display', None],
  [ 0xbe59, 'HangulSyllableBing', [], 'Display', None],
  [ 0xbe5a, 'HangulSyllableBij', [], 'Display', None],
  [ 0xbe5b, 'HangulSyllableBic', [], 'Display', None],
  [ 0xbe5c, 'HangulSyllableBik', [], 'Display', None],
  [ 0xbe5d, 'HangulSyllableBit', [], 'Display', None],
  [ 0xbe5e, 'HangulSyllableBip', [], 'Display', None],
  [ 0xbe5f, 'HangulSyllableBih', [], 'Display', None],
  [ 0xbe60, 'HangulSyllableBba', [], 'Display', None],
  [ 0xbe61, 'HangulSyllableBbag', [], 'Display', None],
  [ 0xbe62, 'HangulSyllableBbagg', [], 'Display', None],
  [ 0xbe63, 'HangulSyllableBbags', [], 'Display', None],
  [ 0xbe64, 'HangulSyllableBban', [], 'Display', None],
  [ 0xbe65, 'HangulSyllableBbanj', [], 'Display', None],
  [ 0xbe66, 'HangulSyllableBbanh', [], 'Display', None],
  [ 0xbe67, 'HangulSyllableBbad', [], 'Display', None],
  [ 0xbe68, 'HangulSyllableBbal', [], 'Display', None],
  [ 0xbe69, 'HangulSyllableBbalg', [], 'Display', None],
  [ 0xbe6a, 'HangulSyllableBbalm', [], 'Display', None],
  [ 0xbe6b, 'HangulSyllableBbalb', [], 'Display', None],
  [ 0xbe6c, 'HangulSyllableBbals', [], 'Display', None],
  [ 0xbe6d, 'HangulSyllableBbalt', [], 'Display', None],
  [ 0xbe6e, 'HangulSyllableBbalp', [], 'Display', None],
  [ 0xbe6f, 'HangulSyllableBbalh', [], 'Display', None],
  [ 0xbe70, 'HangulSyllableBbam', [], 'Display', None],
  [ 0xbe71, 'HangulSyllableBbab', [], 'Display', None],
  [ 0xbe72, 'HangulSyllableBbabs', [], 'Display', None],
  [ 0xbe73, 'HangulSyllableBbas', [], 'Display', None],
  [ 0xbe74, 'HangulSyllableBbass', [], 'Display', None],
  [ 0xbe75, 'HangulSyllableBbang', [], 'Display', None],
  [ 0xbe76, 'HangulSyllableBbaj', [], 'Display', None],
  [ 0xbe77, 'HangulSyllableBbac', [], 'Display', None],
  [ 0xbe78, 'HangulSyllableBbak', [], 'Display', None],
  [ 0xbe79, 'HangulSyllableBbat', [], 'Display', None],
  [ 0xbe7a, 'HangulSyllableBbap', [], 'Display', None],
  [ 0xbe7b, 'HangulSyllableBbah', [], 'Display', None],
  [ 0xbe7c, 'HangulSyllableBbae', [], 'Display', None],
  [ 0xbe7d, 'HangulSyllableBbaeg', [], 'Display', None],
  [ 0xbe7e, 'HangulSyllableBbaegg', [], 'Display', None],
  [ 0xbe7f, 'HangulSyllableBbaegs', [], 'Display', None],
  [ 0xbe80, 'HangulSyllableBbaen', [], 'Display', None],
  [ 0xbe81, 'HangulSyllableBbaenj', [], 'Display', None],
  [ 0xbe82, 'HangulSyllableBbaenh', [], 'Display', None],
  [ 0xbe83, 'HangulSyllableBbaed', [], 'Display', None],
  [ 0xbe84, 'HangulSyllableBbael', [], 'Display', None],
  [ 0xbe85, 'HangulSyllableBbaelg', [], 'Display', None],
  [ 0xbe86, 'HangulSyllableBbaelm', [], 'Display', None],
  [ 0xbe87, 'HangulSyllableBbaelb', [], 'Display', None],
  [ 0xbe88, 'HangulSyllableBbaels', [], 'Display', None],
  [ 0xbe89, 'HangulSyllableBbaelt', [], 'Display', None],
  [ 0xbe8a, 'HangulSyllableBbaelp', [], 'Display', None],
  [ 0xbe8b, 'HangulSyllableBbaelh', [], 'Display', None],
  [ 0xbe8c, 'HangulSyllableBbaem', [], 'Display', None],
  [ 0xbe8d, 'HangulSyllableBbaeb', [], 'Display', None],
  [ 0xbe8e, 'HangulSyllableBbaebs', [], 'Display', None],
  [ 0xbe8f, 'HangulSyllableBbaes', [], 'Display', None],
  [ 0xbe90, 'HangulSyllableBbaess', [], 'Display', None],
  [ 0xbe91, 'HangulSyllableBbaeng', [], 'Display', None],
  [ 0xbe92, 'HangulSyllableBbaej', [], 'Display', None],
  [ 0xbe93, 'HangulSyllableBbaec', [], 'Display', None],
  [ 0xbe94, 'HangulSyllableBbaek', [], 'Display', None],
  [ 0xbe95, 'HangulSyllableBbaet', [], 'Display', None],
  [ 0xbe96, 'HangulSyllableBbaep', [], 'Display', None],
  [ 0xbe97, 'HangulSyllableBbaeh', [], 'Display', None],
  [ 0xbe98, 'HangulSyllableBbya', [], 'Display', None],
  [ 0xbe99, 'HangulSyllableBbyag', [], 'Display', None],
  [ 0xbe9a, 'HangulSyllableBbyagg', [], 'Display', None],
  [ 0xbe9b, 'HangulSyllableBbyags', [], 'Display', None],
  [ 0xbe9c, 'HangulSyllableBbyan', [], 'Display', None],
  [ 0xbe9d, 'HangulSyllableBbyanj', [], 'Display', None],
  [ 0xbe9e, 'HangulSyllableBbyanh', [], 'Display', None],
  [ 0xbe9f, 'HangulSyllableBbyad', [], 'Display', None],
  [ 0xbea0, 'HangulSyllableBbyal', [], 'Display', None],
  [ 0xbea1, 'HangulSyllableBbyalg', [], 'Display', None],
  [ 0xbea2, 'HangulSyllableBbyalm', [], 'Display', None],
  [ 0xbea3, 'HangulSyllableBbyalb', [], 'Display', None],
  [ 0xbea4, 'HangulSyllableBbyals', [], 'Display', None],
  [ 0xbea5, 'HangulSyllableBbyalt', [], 'Display', None],
  [ 0xbea6, 'HangulSyllableBbyalp', [], 'Display', None],
  [ 0xbea7, 'HangulSyllableBbyalh', [], 'Display', None],
  [ 0xbea8, 'HangulSyllableBbyam', [], 'Display', None],
  [ 0xbea9, 'HangulSyllableBbyab', [], 'Display', None],
  [ 0xbeaa, 'HangulSyllableBbyabs', [], 'Display', None],
  [ 0xbeab, 'HangulSyllableBbyas', [], 'Display', None],
  [ 0xbeac, 'HangulSyllableBbyass', [], 'Display', None],
  [ 0xbead, 'HangulSyllableBbyang', [], 'Display', None],
  [ 0xbeae, 'HangulSyllableBbyaj', [], 'Display', None],
  [ 0xbeaf, 'HangulSyllableBbyac', [], 'Display', None],
  [ 0xbeb0, 'HangulSyllableBbyak', [], 'Display', None],
  [ 0xbeb1, 'HangulSyllableBbyat', [], 'Display', None],
  [ 0xbeb2, 'HangulSyllableBbyap', [], 'Display', None],
  [ 0xbeb3, 'HangulSyllableBbyah', [], 'Display', None],
  [ 0xbeb4, 'HangulSyllableBbyae', [], 'Display', None],
  [ 0xbeb5, 'HangulSyllableBbyaeg', [], 'Display', None],
  [ 0xbeb6, 'HangulSyllableBbyaegg', [], 'Display', None],
  [ 0xbeb7, 'HangulSyllableBbyaegs', [], 'Display', None],
  [ 0xbeb8, 'HangulSyllableBbyaen', [], 'Display', None],
  [ 0xbeb9, 'HangulSyllableBbyaenj', [], 'Display', None],
  [ 0xbeba, 'HangulSyllableBbyaenh', [], 'Display', None],
  [ 0xbebb, 'HangulSyllableBbyaed', [], 'Display', None],
  [ 0xbebc, 'HangulSyllableBbyael', [], 'Display', None],
  [ 0xbebd, 'HangulSyllableBbyaelg', [], 'Display', None],
  [ 0xbebe, 'HangulSyllableBbyaelm', [], 'Display', None],
  [ 0xbebf, 'HangulSyllableBbyaelb', [], 'Display', None],
  [ 0xbec0, 'HangulSyllableBbyaels', [], 'Display', None],
  [ 0xbec1, 'HangulSyllableBbyaelt', [], 'Display', None],
  [ 0xbec2, 'HangulSyllableBbyaelp', [], 'Display', None],
  [ 0xbec3, 'HangulSyllableBbyaelh', [], 'Display', None],
  [ 0xbec4, 'HangulSyllableBbyaem', [], 'Display', None],
  [ 0xbec5, 'HangulSyllableBbyaeb', [], 'Display', None],
  [ 0xbec6, 'HangulSyllableBbyaebs', [], 'Display', None],
  [ 0xbec7, 'HangulSyllableBbyaes', [], 'Display', None],
  [ 0xbec8, 'HangulSyllableBbyaess', [], 'Display', None],
  [ 0xbec9, 'HangulSyllableBbyaeng', [], 'Display', None],
  [ 0xbeca, 'HangulSyllableBbyaej', [], 'Display', None],
  [ 0xbecb, 'HangulSyllableBbyaec', [], 'Display', None],
  [ 0xbecc, 'HangulSyllableBbyaek', [], 'Display', None],
  [ 0xbecd, 'HangulSyllableBbyaet', [], 'Display', None],
  [ 0xbece, 'HangulSyllableBbyaep', [], 'Display', None],
  [ 0xbecf, 'HangulSyllableBbyaeh', [], 'Display', None],
  [ 0xbed0, 'HangulSyllableBbeo', [], 'Display', None],
  [ 0xbed1, 'HangulSyllableBbeog', [], 'Display', None],
  [ 0xbed2, 'HangulSyllableBbeogg', [], 'Display', None],
  [ 0xbed3, 'HangulSyllableBbeogs', [], 'Display', None],
  [ 0xbed4, 'HangulSyllableBbeon', [], 'Display', None],
  [ 0xbed5, 'HangulSyllableBbeonj', [], 'Display', None],
  [ 0xbed6, 'HangulSyllableBbeonh', [], 'Display', None],
  [ 0xbed7, 'HangulSyllableBbeod', [], 'Display', None],
  [ 0xbed8, 'HangulSyllableBbeol', [], 'Display', None],
  [ 0xbed9, 'HangulSyllableBbeolg', [], 'Display', None],
  [ 0xbeda, 'HangulSyllableBbeolm', [], 'Display', None],
  [ 0xbedb, 'HangulSyllableBbeolb', [], 'Display', None],
  [ 0xbedc, 'HangulSyllableBbeols', [], 'Display', None],
  [ 0xbedd, 'HangulSyllableBbeolt', [], 'Display', None],
  [ 0xbede, 'HangulSyllableBbeolp', [], 'Display', None],
  [ 0xbedf, 'HangulSyllableBbeolh', [], 'Display', None],
  [ 0xbee0, 'HangulSyllableBbeom', [], 'Display', None],
  [ 0xbee1, 'HangulSyllableBbeob', [], 'Display', None],
  [ 0xbee2, 'HangulSyllableBbeobs', [], 'Display', None],
  [ 0xbee3, 'HangulSyllableBbeos', [], 'Display', None],
  [ 0xbee4, 'HangulSyllableBbeoss', [], 'Display', None],
  [ 0xbee5, 'HangulSyllableBbeong', [], 'Display', None],
  [ 0xbee6, 'HangulSyllableBbeoj', [], 'Display', None],
  [ 0xbee7, 'HangulSyllableBbeoc', [], 'Display', None],
  [ 0xbee8, 'HangulSyllableBbeok', [], 'Display', None],
  [ 0xbee9, 'HangulSyllableBbeot', [], 'Display', None],
  [ 0xbeea, 'HangulSyllableBbeop', [], 'Display', None],
  [ 0xbeeb, 'HangulSyllableBbeoh', [], 'Display', None],
  [ 0xbeec, 'HangulSyllableBbe', [], 'Display', None],
  [ 0xbeed, 'HangulSyllableBbeg', [], 'Display', None],
  [ 0xbeee, 'HangulSyllableBbegg', [], 'Display', None],
  [ 0xbeef, 'HangulSyllableBbegs', [], 'Display', None],
  [ 0xbef0, 'HangulSyllableBben', [], 'Display', None],
  [ 0xbef1, 'HangulSyllableBbenj', [], 'Display', None],
  [ 0xbef2, 'HangulSyllableBbenh', [], 'Display', None],
  [ 0xbef3, 'HangulSyllableBbed', [], 'Display', None],
  [ 0xbef4, 'HangulSyllableBbel', [], 'Display', None],
  [ 0xbef5, 'HangulSyllableBbelg', [], 'Display', None],
  [ 0xbef6, 'HangulSyllableBbelm', [], 'Display', None],
  [ 0xbef7, 'HangulSyllableBbelb', [], 'Display', None],
  [ 0xbef8, 'HangulSyllableBbels', [], 'Display', None],
  [ 0xbef9, 'HangulSyllableBbelt', [], 'Display', None],
  [ 0xbefa, 'HangulSyllableBbelp', [], 'Display', None],
  [ 0xbefb, 'HangulSyllableBbelh', [], 'Display', None],
  [ 0xbefc, 'HangulSyllableBbem', [], 'Display', None],
  [ 0xbefd, 'HangulSyllableBbeb', [], 'Display', None],
  [ 0xbefe, 'HangulSyllableBbebs', [], 'Display', None],
  [ 0xbeff, 'HangulSyllableBbes', [], 'Display', None],
  [ 0xbf00, 'HangulSyllableBbess', [], 'Display', None],
  [ 0xbf01, 'HangulSyllableBbeng', [], 'Display', None],
  [ 0xbf02, 'HangulSyllableBbej', [], 'Display', None],
  [ 0xbf03, 'HangulSyllableBbec', [], 'Display', None],
  [ 0xbf04, 'HangulSyllableBbek', [], 'Display', None],
  [ 0xbf05, 'HangulSyllableBbet', [], 'Display', None],
  [ 0xbf06, 'HangulSyllableBbep', [], 'Display', None],
  [ 0xbf07, 'HangulSyllableBbeh', [], 'Display', None],
  [ 0xbf08, 'HangulSyllableBbyeo', [], 'Display', None],
  [ 0xbf09, 'HangulSyllableBbyeog', [], 'Display', None],
  [ 0xbf0a, 'HangulSyllableBbyeogg', [], 'Display', None],
  [ 0xbf0b, 'HangulSyllableBbyeogs', [], 'Display', None],
  [ 0xbf0c, 'HangulSyllableBbyeon', [], 'Display', None],
  [ 0xbf0d, 'HangulSyllableBbyeonj', [], 'Display', None],
  [ 0xbf0e, 'HangulSyllableBbyeonh', [], 'Display', None],
  [ 0xbf0f, 'HangulSyllableBbyeod', [], 'Display', None],
  [ 0xbf10, 'HangulSyllableBbyeol', [], 'Display', None],
  [ 0xbf11, 'HangulSyllableBbyeolg', [], 'Display', None],
  [ 0xbf12, 'HangulSyllableBbyeolm', [], 'Display', None],
  [ 0xbf13, 'HangulSyllableBbyeolb', [], 'Display', None],
  [ 0xbf14, 'HangulSyllableBbyeols', [], 'Display', None],
  [ 0xbf15, 'HangulSyllableBbyeolt', [], 'Display', None],
  [ 0xbf16, 'HangulSyllableBbyeolp', [], 'Display', None],
  [ 0xbf17, 'HangulSyllableBbyeolh', [], 'Display', None],
  [ 0xbf18, 'HangulSyllableBbyeom', [], 'Display', None],
  [ 0xbf19, 'HangulSyllableBbyeob', [], 'Display', None],
  [ 0xbf1a, 'HangulSyllableBbyeobs', [], 'Display', None],
  [ 0xbf1b, 'HangulSyllableBbyeos', [], 'Display', None],
  [ 0xbf1c, 'HangulSyllableBbyeoss', [], 'Display', None],
  [ 0xbf1d, 'HangulSyllableBbyeong', [], 'Display', None],
  [ 0xbf1e, 'HangulSyllableBbyeoj', [], 'Display', None],
  [ 0xbf1f, 'HangulSyllableBbyeoc', [], 'Display', None],
  [ 0xbf20, 'HangulSyllableBbyeok', [], 'Display', None],
  [ 0xbf21, 'HangulSyllableBbyeot', [], 'Display', None],
  [ 0xbf22, 'HangulSyllableBbyeop', [], 'Display', None],
  [ 0xbf23, 'HangulSyllableBbyeoh', [], 'Display', None],
  [ 0xbf24, 'HangulSyllableBbye', [], 'Display', None],
  [ 0xbf25, 'HangulSyllableBbyeg', [], 'Display', None],
  [ 0xbf26, 'HangulSyllableBbyegg', [], 'Display', None],
  [ 0xbf27, 'HangulSyllableBbyegs', [], 'Display', None],
  [ 0xbf28, 'HangulSyllableBbyen', [], 'Display', None],
  [ 0xbf29, 'HangulSyllableBbyenj', [], 'Display', None],
  [ 0xbf2a, 'HangulSyllableBbyenh', [], 'Display', None],
  [ 0xbf2b, 'HangulSyllableBbyed', [], 'Display', None],
  [ 0xbf2c, 'HangulSyllableBbyel', [], 'Display', None],
  [ 0xbf2d, 'HangulSyllableBbyelg', [], 'Display', None],
  [ 0xbf2e, 'HangulSyllableBbyelm', [], 'Display', None],
  [ 0xbf2f, 'HangulSyllableBbyelb', [], 'Display', None],
  [ 0xbf30, 'HangulSyllableBbyels', [], 'Display', None],
  [ 0xbf31, 'HangulSyllableBbyelt', [], 'Display', None],
  [ 0xbf32, 'HangulSyllableBbyelp', [], 'Display', None],
  [ 0xbf33, 'HangulSyllableBbyelh', [], 'Display', None],
  [ 0xbf34, 'HangulSyllableBbyem', [], 'Display', None],
  [ 0xbf35, 'HangulSyllableBbyeb', [], 'Display', None],
  [ 0xbf36, 'HangulSyllableBbyebs', [], 'Display', None],
  [ 0xbf37, 'HangulSyllableBbyes', [], 'Display', None],
  [ 0xbf38, 'HangulSyllableBbyess', [], 'Display', None],
  [ 0xbf39, 'HangulSyllableBbyeng', [], 'Display', None],
  [ 0xbf3a, 'HangulSyllableBbyej', [], 'Display', None],
  [ 0xbf3b, 'HangulSyllableBbyec', [], 'Display', None],
  [ 0xbf3c, 'HangulSyllableBbyek', [], 'Display', None],
  [ 0xbf3d, 'HangulSyllableBbyet', [], 'Display', None],
  [ 0xbf3e, 'HangulSyllableBbyep', [], 'Display', None],
  [ 0xbf3f, 'HangulSyllableBbyeh', [], 'Display', None],
  [ 0xbf40, 'HangulSyllableBbo', [], 'Display', None],
  [ 0xbf41, 'HangulSyllableBbog', [], 'Display', None],
  [ 0xbf42, 'HangulSyllableBbogg', [], 'Display', None],
  [ 0xbf43, 'HangulSyllableBbogs', [], 'Display', None],
  [ 0xbf44, 'HangulSyllableBbon', [], 'Display', None],
  [ 0xbf45, 'HangulSyllableBbonj', [], 'Display', None],
  [ 0xbf46, 'HangulSyllableBbonh', [], 'Display', None],
  [ 0xbf47, 'HangulSyllableBbod', [], 'Display', None],
  [ 0xbf48, 'HangulSyllableBbol', [], 'Display', None],
  [ 0xbf49, 'HangulSyllableBbolg', [], 'Display', None],
  [ 0xbf4a, 'HangulSyllableBbolm', [], 'Display', None],
  [ 0xbf4b, 'HangulSyllableBbolb', [], 'Display', None],
  [ 0xbf4c, 'HangulSyllableBbols', [], 'Display', None],
  [ 0xbf4d, 'HangulSyllableBbolt', [], 'Display', None],
  [ 0xbf4e, 'HangulSyllableBbolp', [], 'Display', None],
  [ 0xbf4f, 'HangulSyllableBbolh', [], 'Display', None],
  [ 0xbf50, 'HangulSyllableBbom', [], 'Display', None],
  [ 0xbf51, 'HangulSyllableBbob', [], 'Display', None],
  [ 0xbf52, 'HangulSyllableBbobs', [], 'Display', None],
  [ 0xbf53, 'HangulSyllableBbos', [], 'Display', None],
  [ 0xbf54, 'HangulSyllableBboss', [], 'Display', None],
  [ 0xbf55, 'HangulSyllableBbong', [], 'Display', None],
  [ 0xbf56, 'HangulSyllableBboj', [], 'Display', None],
  [ 0xbf57, 'HangulSyllableBboc', [], 'Display', None],
  [ 0xbf58, 'HangulSyllableBbok', [], 'Display', None],
  [ 0xbf59, 'HangulSyllableBbot', [], 'Display', None],
  [ 0xbf5a, 'HangulSyllableBbop', [], 'Display', None],
  [ 0xbf5b, 'HangulSyllableBboh', [], 'Display', None],
  [ 0xbf5c, 'HangulSyllableBbwa', [], 'Display', None],
  [ 0xbf5d, 'HangulSyllableBbwag', [], 'Display', None],
  [ 0xbf5e, 'HangulSyllableBbwagg', [], 'Display', None],
  [ 0xbf5f, 'HangulSyllableBbwags', [], 'Display', None],
  [ 0xbf60, 'HangulSyllableBbwan', [], 'Display', None],
  [ 0xbf61, 'HangulSyllableBbwanj', [], 'Display', None],
  [ 0xbf62, 'HangulSyllableBbwanh', [], 'Display', None],
  [ 0xbf63, 'HangulSyllableBbwad', [], 'Display', None],
  [ 0xbf64, 'HangulSyllableBbwal', [], 'Display', None],
  [ 0xbf65, 'HangulSyllableBbwalg', [], 'Display', None],
  [ 0xbf66, 'HangulSyllableBbwalm', [], 'Display', None],
  [ 0xbf67, 'HangulSyllableBbwalb', [], 'Display', None],
  [ 0xbf68, 'HangulSyllableBbwals', [], 'Display', None],
  [ 0xbf69, 'HangulSyllableBbwalt', [], 'Display', None],
  [ 0xbf6a, 'HangulSyllableBbwalp', [], 'Display', None],
  [ 0xbf6b, 'HangulSyllableBbwalh', [], 'Display', None],
  [ 0xbf6c, 'HangulSyllableBbwam', [], 'Display', None],
  [ 0xbf6d, 'HangulSyllableBbwab', [], 'Display', None],
  [ 0xbf6e, 'HangulSyllableBbwabs', [], 'Display', None],
  [ 0xbf6f, 'HangulSyllableBbwas', [], 'Display', None],
  [ 0xbf70, 'HangulSyllableBbwass', [], 'Display', None],
  [ 0xbf71, 'HangulSyllableBbwang', [], 'Display', None],
  [ 0xbf72, 'HangulSyllableBbwaj', [], 'Display', None],
  [ 0xbf73, 'HangulSyllableBbwac', [], 'Display', None],
  [ 0xbf74, 'HangulSyllableBbwak', [], 'Display', None],
  [ 0xbf75, 'HangulSyllableBbwat', [], 'Display', None],
  [ 0xbf76, 'HangulSyllableBbwap', [], 'Display', None],
  [ 0xbf77, 'HangulSyllableBbwah', [], 'Display', None],
  [ 0xbf78, 'HangulSyllableBbwae', [], 'Display', None],
  [ 0xbf79, 'HangulSyllableBbwaeg', [], 'Display', None],
  [ 0xbf7a, 'HangulSyllableBbwaegg', [], 'Display', None],
  [ 0xbf7b, 'HangulSyllableBbwaegs', [], 'Display', None],
  [ 0xbf7c, 'HangulSyllableBbwaen', [], 'Display', None],
  [ 0xbf7d, 'HangulSyllableBbwaenj', [], 'Display', None],
  [ 0xbf7e, 'HangulSyllableBbwaenh', [], 'Display', None],
  [ 0xbf7f, 'HangulSyllableBbwaed', [], 'Display', None],
  [ 0xbf80, 'HangulSyllableBbwael', [], 'Display', None],
  [ 0xbf81, 'HangulSyllableBbwaelg', [], 'Display', None],
  [ 0xbf82, 'HangulSyllableBbwaelm', [], 'Display', None],
  [ 0xbf83, 'HangulSyllableBbwaelb', [], 'Display', None],
  [ 0xbf84, 'HangulSyllableBbwaels', [], 'Display', None],
  [ 0xbf85, 'HangulSyllableBbwaelt', [], 'Display', None],
  [ 0xbf86, 'HangulSyllableBbwaelp', [], 'Display', None],
  [ 0xbf87, 'HangulSyllableBbwaelh', [], 'Display', None],
  [ 0xbf88, 'HangulSyllableBbwaem', [], 'Display', None],
  [ 0xbf89, 'HangulSyllableBbwaeb', [], 'Display', None],
  [ 0xbf8a, 'HangulSyllableBbwaebs', [], 'Display', None],
  [ 0xbf8b, 'HangulSyllableBbwaes', [], 'Display', None],
  [ 0xbf8c, 'HangulSyllableBbwaess', [], 'Display', None],
  [ 0xbf8d, 'HangulSyllableBbwaeng', [], 'Display', None],
  [ 0xbf8e, 'HangulSyllableBbwaej', [], 'Display', None],
  [ 0xbf8f, 'HangulSyllableBbwaec', [], 'Display', None],
  [ 0xbf90, 'HangulSyllableBbwaek', [], 'Display', None],
  [ 0xbf91, 'HangulSyllableBbwaet', [], 'Display', None],
  [ 0xbf92, 'HangulSyllableBbwaep', [], 'Display', None],
  [ 0xbf93, 'HangulSyllableBbwaeh', [], 'Display', None],
  [ 0xbf94, 'HangulSyllableBboe', [], 'Display', None],
  [ 0xbf95, 'HangulSyllableBboeg', [], 'Display', None],
  [ 0xbf96, 'HangulSyllableBboegg', [], 'Display', None],
  [ 0xbf97, 'HangulSyllableBboegs', [], 'Display', None],
  [ 0xbf98, 'HangulSyllableBboen', [], 'Display', None],
  [ 0xbf99, 'HangulSyllableBboenj', [], 'Display', None],
  [ 0xbf9a, 'HangulSyllableBboenh', [], 'Display', None],
  [ 0xbf9b, 'HangulSyllableBboed', [], 'Display', None],
  [ 0xbf9c, 'HangulSyllableBboel', [], 'Display', None],
  [ 0xbf9d, 'HangulSyllableBboelg', [], 'Display', None],
  [ 0xbf9e, 'HangulSyllableBboelm', [], 'Display', None],
  [ 0xbf9f, 'HangulSyllableBboelb', [], 'Display', None],
  [ 0xbfa0, 'HangulSyllableBboels', [], 'Display', None],
  [ 0xbfa1, 'HangulSyllableBboelt', [], 'Display', None],
  [ 0xbfa2, 'HangulSyllableBboelp', [], 'Display', None],
  [ 0xbfa3, 'HangulSyllableBboelh', [], 'Display', None],
  [ 0xbfa4, 'HangulSyllableBboem', [], 'Display', None],
  [ 0xbfa5, 'HangulSyllableBboeb', [], 'Display', None],
  [ 0xbfa6, 'HangulSyllableBboebs', [], 'Display', None],
  [ 0xbfa7, 'HangulSyllableBboes', [], 'Display', None],
  [ 0xbfa8, 'HangulSyllableBboess', [], 'Display', None],
  [ 0xbfa9, 'HangulSyllableBboeng', [], 'Display', None],
  [ 0xbfaa, 'HangulSyllableBboej', [], 'Display', None],
  [ 0xbfab, 'HangulSyllableBboec', [], 'Display', None],
  [ 0xbfac, 'HangulSyllableBboek', [], 'Display', None],
  [ 0xbfad, 'HangulSyllableBboet', [], 'Display', None],
  [ 0xbfae, 'HangulSyllableBboep', [], 'Display', None],
  [ 0xbfaf, 'HangulSyllableBboeh', [], 'Display', None],
  [ 0xbfb0, 'HangulSyllableBbyo', [], 'Display', None],
  [ 0xbfb1, 'HangulSyllableBbyog', [], 'Display', None],
  [ 0xbfb2, 'HangulSyllableBbyogg', [], 'Display', None],
  [ 0xbfb3, 'HangulSyllableBbyogs', [], 'Display', None],
  [ 0xbfb4, 'HangulSyllableBbyon', [], 'Display', None],
  [ 0xbfb5, 'HangulSyllableBbyonj', [], 'Display', None],
  [ 0xbfb6, 'HangulSyllableBbyonh', [], 'Display', None],
  [ 0xbfb7, 'HangulSyllableBbyod', [], 'Display', None],
  [ 0xbfb8, 'HangulSyllableBbyol', [], 'Display', None],
  [ 0xbfb9, 'HangulSyllableBbyolg', [], 'Display', None],
  [ 0xbfba, 'HangulSyllableBbyolm', [], 'Display', None],
  [ 0xbfbb, 'HangulSyllableBbyolb', [], 'Display', None],
  [ 0xbfbc, 'HangulSyllableBbyols', [], 'Display', None],
  [ 0xbfbd, 'HangulSyllableBbyolt', [], 'Display', None],
  [ 0xbfbe, 'HangulSyllableBbyolp', [], 'Display', None],
  [ 0xbfbf, 'HangulSyllableBbyolh', [], 'Display', None],
  [ 0xbfc0, 'HangulSyllableBbyom', [], 'Display', None],
  [ 0xbfc1, 'HangulSyllableBbyob', [], 'Display', None],
  [ 0xbfc2, 'HangulSyllableBbyobs', [], 'Display', None],
  [ 0xbfc3, 'HangulSyllableBbyos', [], 'Display', None],
  [ 0xbfc4, 'HangulSyllableBbyoss', [], 'Display', None],
  [ 0xbfc5, 'HangulSyllableBbyong', [], 'Display', None],
  [ 0xbfc6, 'HangulSyllableBbyoj', [], 'Display', None],
  [ 0xbfc7, 'HangulSyllableBbyoc', [], 'Display', None],
  [ 0xbfc8, 'HangulSyllableBbyok', [], 'Display', None],
  [ 0xbfc9, 'HangulSyllableBbyot', [], 'Display', None],
  [ 0xbfca, 'HangulSyllableBbyop', [], 'Display', None],
  [ 0xbfcb, 'HangulSyllableBbyoh', [], 'Display', None],
  [ 0xbfcc, 'HangulSyllableBbu', [], 'Display', None],
  [ 0xbfcd, 'HangulSyllableBbug', [], 'Display', None],
  [ 0xbfce, 'HangulSyllableBbugg', [], 'Display', None],
  [ 0xbfcf, 'HangulSyllableBbugs', [], 'Display', None],
  [ 0xbfd0, 'HangulSyllableBbun', [], 'Display', None],
  [ 0xbfd1, 'HangulSyllableBbunj', [], 'Display', None],
  [ 0xbfd2, 'HangulSyllableBbunh', [], 'Display', None],
  [ 0xbfd3, 'HangulSyllableBbud', [], 'Display', None],
  [ 0xbfd4, 'HangulSyllableBbul', [], 'Display', None],
  [ 0xbfd5, 'HangulSyllableBbulg', [], 'Display', None],
  [ 0xbfd6, 'HangulSyllableBbulm', [], 'Display', None],
  [ 0xbfd7, 'HangulSyllableBbulb', [], 'Display', None],
  [ 0xbfd8, 'HangulSyllableBbuls', [], 'Display', None],
  [ 0xbfd9, 'HangulSyllableBbult', [], 'Display', None],
  [ 0xbfda, 'HangulSyllableBbulp', [], 'Display', None],
  [ 0xbfdb, 'HangulSyllableBbulh', [], 'Display', None],
  [ 0xbfdc, 'HangulSyllableBbum', [], 'Display', None],
  [ 0xbfdd, 'HangulSyllableBbub', [], 'Display', None],
  [ 0xbfde, 'HangulSyllableBbubs', [], 'Display', None],
  [ 0xbfdf, 'HangulSyllableBbus', [], 'Display', None],
  [ 0xbfe0, 'HangulSyllableBbuss', [], 'Display', None],
  [ 0xbfe1, 'HangulSyllableBbung', [], 'Display', None],
  [ 0xbfe2, 'HangulSyllableBbuj', [], 'Display', None],
  [ 0xbfe3, 'HangulSyllableBbuc', [], 'Display', None],
  [ 0xbfe4, 'HangulSyllableBbuk', [], 'Display', None],
  [ 0xbfe5, 'HangulSyllableBbut', [], 'Display', None],
  [ 0xbfe6, 'HangulSyllableBbup', [], 'Display', None],
  [ 0xbfe7, 'HangulSyllableBbuh', [], 'Display', None],
  [ 0xbfe8, 'HangulSyllableBbweo', [], 'Display', None],
  [ 0xbfe9, 'HangulSyllableBbweog', [], 'Display', None],
  [ 0xbfea, 'HangulSyllableBbweogg', [], 'Display', None],
  [ 0xbfeb, 'HangulSyllableBbweogs', [], 'Display', None],
  [ 0xbfec, 'HangulSyllableBbweon', [], 'Display', None],
  [ 0xbfed, 'HangulSyllableBbweonj', [], 'Display', None],
  [ 0xbfee, 'HangulSyllableBbweonh', [], 'Display', None],
  [ 0xbfef, 'HangulSyllableBbweod', [], 'Display', None],
  [ 0xbff0, 'HangulSyllableBbweol', [], 'Display', None],
  [ 0xbff1, 'HangulSyllableBbweolg', [], 'Display', None],
  [ 0xbff2, 'HangulSyllableBbweolm', [], 'Display', None],
  [ 0xbff3, 'HangulSyllableBbweolb', [], 'Display', None],
  [ 0xbff4, 'HangulSyllableBbweols', [], 'Display', None],
  [ 0xbff5, 'HangulSyllableBbweolt', [], 'Display', None],
  [ 0xbff6, 'HangulSyllableBbweolp', [], 'Display', None],
  [ 0xbff7, 'HangulSyllableBbweolh', [], 'Display', None],
  [ 0xbff8, 'HangulSyllableBbweom', [], 'Display', None],
  [ 0xbff9, 'HangulSyllableBbweob', [], 'Display', None],
  [ 0xbffa, 'HangulSyllableBbweobs', [], 'Display', None],
  [ 0xbffb, 'HangulSyllableBbweos', [], 'Display', None],
  [ 0xbffc, 'HangulSyllableBbweoss', [], 'Display', None],
  [ 0xbffd, 'HangulSyllableBbweong', [], 'Display', None],
  [ 0xbffe, 'HangulSyllableBbweoj', [], 'Display', None],
  [ 0xbfff, 'HangulSyllableBbweoc', [], 'Display', None],
  [ 0xc000, 'HangulSyllableBbweok', [], 'Display', None],
  [ 0xc001, 'HangulSyllableBbweot', [], 'Display', None],
  [ 0xc002, 'HangulSyllableBbweop', [], 'Display', None],
  [ 0xc003, 'HangulSyllableBbweoh', [], 'Display', None],
  [ 0xc004, 'HangulSyllableBbwe', [], 'Display', None],
  [ 0xc005, 'HangulSyllableBbweg', [], 'Display', None],
  [ 0xc006, 'HangulSyllableBbwegg', [], 'Display', None],
  [ 0xc007, 'HangulSyllableBbwegs', [], 'Display', None],
  [ 0xc008, 'HangulSyllableBbwen', [], 'Display', None],
  [ 0xc009, 'HangulSyllableBbwenj', [], 'Display', None],
  [ 0xc00a, 'HangulSyllableBbwenh', [], 'Display', None],
  [ 0xc00b, 'HangulSyllableBbwed', [], 'Display', None],
  [ 0xc00c, 'HangulSyllableBbwel', [], 'Display', None],
  [ 0xc00d, 'HangulSyllableBbwelg', [], 'Display', None],
  [ 0xc00e, 'HangulSyllableBbwelm', [], 'Display', None],
  [ 0xc00f, 'HangulSyllableBbwelb', [], 'Display', None],
  [ 0xc010, 'HangulSyllableBbwels', [], 'Display', None],
  [ 0xc011, 'HangulSyllableBbwelt', [], 'Display', None],
  [ 0xc012, 'HangulSyllableBbwelp', [], 'Display', None],
  [ 0xc013, 'HangulSyllableBbwelh', [], 'Display', None],
  [ 0xc014, 'HangulSyllableBbwem', [], 'Display', None],
  [ 0xc015, 'HangulSyllableBbweb', [], 'Display', None],
  [ 0xc016, 'HangulSyllableBbwebs', [], 'Display', None],
  [ 0xc017, 'HangulSyllableBbwes', [], 'Display', None],
  [ 0xc018, 'HangulSyllableBbwess', [], 'Display', None],
  [ 0xc019, 'HangulSyllableBbweng', [], 'Display', None],
  [ 0xc01a, 'HangulSyllableBbwej', [], 'Display', None],
  [ 0xc01b, 'HangulSyllableBbwec', [], 'Display', None],
  [ 0xc01c, 'HangulSyllableBbwek', [], 'Display', None],
  [ 0xc01d, 'HangulSyllableBbwet', [], 'Display', None],
  [ 0xc01e, 'HangulSyllableBbwep', [], 'Display', None],
  [ 0xc01f, 'HangulSyllableBbweh', [], 'Display', None],
  [ 0xc020, 'HangulSyllableBbwi', [], 'Display', None],
  [ 0xc021, 'HangulSyllableBbwig', [], 'Display', None],
  [ 0xc022, 'HangulSyllableBbwigg', [], 'Display', None],
  [ 0xc023, 'HangulSyllableBbwigs', [], 'Display', None],
  [ 0xc024, 'HangulSyllableBbwin', [], 'Display', None],
  [ 0xc025, 'HangulSyllableBbwinj', [], 'Display', None],
  [ 0xc026, 'HangulSyllableBbwinh', [], 'Display', None],
  [ 0xc027, 'HangulSyllableBbwid', [], 'Display', None],
  [ 0xc028, 'HangulSyllableBbwil', [], 'Display', None],
  [ 0xc029, 'HangulSyllableBbwilg', [], 'Display', None],
  [ 0xc02a, 'HangulSyllableBbwilm', [], 'Display', None],
  [ 0xc02b, 'HangulSyllableBbwilb', [], 'Display', None],
  [ 0xc02c, 'HangulSyllableBbwils', [], 'Display', None],
  [ 0xc02d, 'HangulSyllableBbwilt', [], 'Display', None],
  [ 0xc02e, 'HangulSyllableBbwilp', [], 'Display', None],
  [ 0xc02f, 'HangulSyllableBbwilh', [], 'Display', None],
  [ 0xc030, 'HangulSyllableBbwim', [], 'Display', None],
  [ 0xc031, 'HangulSyllableBbwib', [], 'Display', None],
  [ 0xc032, 'HangulSyllableBbwibs', [], 'Display', None],
  [ 0xc033, 'HangulSyllableBbwis', [], 'Display', None],
  [ 0xc034, 'HangulSyllableBbwiss', [], 'Display', None],
  [ 0xc035, 'HangulSyllableBbwing', [], 'Display', None],
  [ 0xc036, 'HangulSyllableBbwij', [], 'Display', None],
  [ 0xc037, 'HangulSyllableBbwic', [], 'Display', None],
  [ 0xc038, 'HangulSyllableBbwik', [], 'Display', None],
  [ 0xc039, 'HangulSyllableBbwit', [], 'Display', None],
  [ 0xc03a, 'HangulSyllableBbwip', [], 'Display', None],
  [ 0xc03b, 'HangulSyllableBbwih', [], 'Display', None],
  [ 0xc03c, 'HangulSyllableBbyu', [], 'Display', None],
  [ 0xc03d, 'HangulSyllableBbyug', [], 'Display', None],
  [ 0xc03e, 'HangulSyllableBbyugg', [], 'Display', None],
  [ 0xc03f, 'HangulSyllableBbyugs', [], 'Display', None],
  [ 0xc040, 'HangulSyllableBbyun', [], 'Display', None],
  [ 0xc041, 'HangulSyllableBbyunj', [], 'Display', None],
  [ 0xc042, 'HangulSyllableBbyunh', [], 'Display', None],
  [ 0xc043, 'HangulSyllableBbyud', [], 'Display', None],
  [ 0xc044, 'HangulSyllableBbyul', [], 'Display', None],
  [ 0xc045, 'HangulSyllableBbyulg', [], 'Display', None],
  [ 0xc046, 'HangulSyllableBbyulm', [], 'Display', None],
  [ 0xc047, 'HangulSyllableBbyulb', [], 'Display', None],
  [ 0xc048, 'HangulSyllableBbyuls', [], 'Display', None],
  [ 0xc049, 'HangulSyllableBbyult', [], 'Display', None],
  [ 0xc04a, 'HangulSyllableBbyulp', [], 'Display', None],
  [ 0xc04b, 'HangulSyllableBbyulh', [], 'Display', None],
  [ 0xc04c, 'HangulSyllableBbyum', [], 'Display', None],
  [ 0xc04d, 'HangulSyllableBbyub', [], 'Display', None],
  [ 0xc04e, 'HangulSyllableBbyubs', [], 'Display', None],
  [ 0xc04f, 'HangulSyllableBbyus', [], 'Display', None],
  [ 0xc050, 'HangulSyllableBbyuss', [], 'Display', None],
  [ 0xc051, 'HangulSyllableBbyung', [], 'Display', None],
  [ 0xc052, 'HangulSyllableBbyuj', [], 'Display', None],
  [ 0xc053, 'HangulSyllableBbyuc', [], 'Display', None],
  [ 0xc054, 'HangulSyllableBbyuk', [], 'Display', None],
  [ 0xc055, 'HangulSyllableBbyut', [], 'Display', None],
  [ 0xc056, 'HangulSyllableBbyup', [], 'Display', None],
  [ 0xc057, 'HangulSyllableBbyuh', [], 'Display', None],
  [ 0xc058, 'HangulSyllableBbeu', [], 'Display', None],
  [ 0xc059, 'HangulSyllableBbeug', [], 'Display', None],
  [ 0xc05a, 'HangulSyllableBbeugg', [], 'Display', None],
  [ 0xc05b, 'HangulSyllableBbeugs', [], 'Display', None],
  [ 0xc05c, 'HangulSyllableBbeun', [], 'Display', None],
  [ 0xc05d, 'HangulSyllableBbeunj', [], 'Display', None],
  [ 0xc05e, 'HangulSyllableBbeunh', [], 'Display', None],
  [ 0xc05f, 'HangulSyllableBbeud', [], 'Display', None],
  [ 0xc060, 'HangulSyllableBbeul', [], 'Display', None],
  [ 0xc061, 'HangulSyllableBbeulg', [], 'Display', None],
  [ 0xc062, 'HangulSyllableBbeulm', [], 'Display', None],
  [ 0xc063, 'HangulSyllableBbeulb', [], 'Display', None],
  [ 0xc064, 'HangulSyllableBbeuls', [], 'Display', None],
  [ 0xc065, 'HangulSyllableBbeult', [], 'Display', None],
  [ 0xc066, 'HangulSyllableBbeulp', [], 'Display', None],
  [ 0xc067, 'HangulSyllableBbeulh', [], 'Display', None],
  [ 0xc068, 'HangulSyllableBbeum', [], 'Display', None],
  [ 0xc069, 'HangulSyllableBbeub', [], 'Display', None],
  [ 0xc06a, 'HangulSyllableBbeubs', [], 'Display', None],
  [ 0xc06b, 'HangulSyllableBbeus', [], 'Display', None],
  [ 0xc06c, 'HangulSyllableBbeuss', [], 'Display', None],
  [ 0xc06d, 'HangulSyllableBbeung', [], 'Display', None],
  [ 0xc06e, 'HangulSyllableBbeuj', [], 'Display', None],
  [ 0xc06f, 'HangulSyllableBbeuc', [], 'Display', None],
  [ 0xc070, 'HangulSyllableBbeuk', [], 'Display', None],
  [ 0xc071, 'HangulSyllableBbeut', [], 'Display', None],
  [ 0xc072, 'HangulSyllableBbeup', [], 'Display', None],
  [ 0xc073, 'HangulSyllableBbeuh', [], 'Display', None],
  [ 0xc074, 'HangulSyllableBbyi', [], 'Display', None],
  [ 0xc075, 'HangulSyllableBbyig', [], 'Display', None],
  [ 0xc076, 'HangulSyllableBbyigg', [], 'Display', None],
  [ 0xc077, 'HangulSyllableBbyigs', [], 'Display', None],
  [ 0xc078, 'HangulSyllableBbyin', [], 'Display', None],
  [ 0xc079, 'HangulSyllableBbyinj', [], 'Display', None],
  [ 0xc07a, 'HangulSyllableBbyinh', [], 'Display', None],
  [ 0xc07b, 'HangulSyllableBbyid', [], 'Display', None],
  [ 0xc07c, 'HangulSyllableBbyil', [], 'Display', None],
  [ 0xc07d, 'HangulSyllableBbyilg', [], 'Display', None],
  [ 0xc07e, 'HangulSyllableBbyilm', [], 'Display', None],
  [ 0xc07f, 'HangulSyllableBbyilb', [], 'Display', None],
  [ 0xc080, 'HangulSyllableBbyils', [], 'Display', None],
  [ 0xc081, 'HangulSyllableBbyilt', [], 'Display', None],
  [ 0xc082, 'HangulSyllableBbyilp', [], 'Display', None],
  [ 0xc083, 'HangulSyllableBbyilh', [], 'Display', None],
  [ 0xc084, 'HangulSyllableBbyim', [], 'Display', None],
  [ 0xc085, 'HangulSyllableBbyib', [], 'Display', None],
  [ 0xc086, 'HangulSyllableBbyibs', [], 'Display', None],
  [ 0xc087, 'HangulSyllableBbyis', [], 'Display', None],
  [ 0xc088, 'HangulSyllableBbyiss', [], 'Display', None],
  [ 0xc089, 'HangulSyllableBbying', [], 'Display', None],
  [ 0xc08a, 'HangulSyllableBbyij', [], 'Display', None],
  [ 0xc08b, 'HangulSyllableBbyic', [], 'Display', None],
  [ 0xc08c, 'HangulSyllableBbyik', [], 'Display', None],
  [ 0xc08d, 'HangulSyllableBbyit', [], 'Display', None],
  [ 0xc08e, 'HangulSyllableBbyip', [], 'Display', None],
  [ 0xc08f, 'HangulSyllableBbyih', [], 'Display', None],
  [ 0xc090, 'HangulSyllableBbi', [], 'Display', None],
  [ 0xc091, 'HangulSyllableBbig', [], 'Display', None],
  [ 0xc092, 'HangulSyllableBbigg', [], 'Display', None],
  [ 0xc093, 'HangulSyllableBbigs', [], 'Display', None],
  [ 0xc094, 'HangulSyllableBbin', [], 'Display', None],
  [ 0xc095, 'HangulSyllableBbinj', [], 'Display', None],
  [ 0xc096, 'HangulSyllableBbinh', [], 'Display', None],
  [ 0xc097, 'HangulSyllableBbid', [], 'Display', None],
  [ 0xc098, 'HangulSyllableBbil', [], 'Display', None],
  [ 0xc099, 'HangulSyllableBbilg', [], 'Display', None],
  [ 0xc09a, 'HangulSyllableBbilm', [], 'Display', None],
  [ 0xc09b, 'HangulSyllableBbilb', [], 'Display', None],
  [ 0xc09c, 'HangulSyllableBbils', [], 'Display', None],
  [ 0xc09d, 'HangulSyllableBbilt', [], 'Display', None],
  [ 0xc09e, 'HangulSyllableBbilp', [], 'Display', None],
  [ 0xc09f, 'HangulSyllableBbilh', [], 'Display', None],
  [ 0xc0a0, 'HangulSyllableBbim', [], 'Display', None],
  [ 0xc0a1, 'HangulSyllableBbib', [], 'Display', None],
  [ 0xc0a2, 'HangulSyllableBbibs', [], 'Display', None],
  [ 0xc0a3, 'HangulSyllableBbis', [], 'Display', None],
  [ 0xc0a4, 'HangulSyllableBbiss', [], 'Display', None],
  [ 0xc0a5, 'HangulSyllableBbing', [], 'Display', None],
  [ 0xc0a6, 'HangulSyllableBbij', [], 'Display', None],
  [ 0xc0a7, 'HangulSyllableBbic', [], 'Display', None],
  [ 0xc0a8, 'HangulSyllableBbik', [], 'Display', None],
  [ 0xc0a9, 'HangulSyllableBbit', [], 'Display', None],
  [ 0xc0aa, 'HangulSyllableBbip', [], 'Display', None],
  [ 0xc0ab, 'HangulSyllableBbih', [], 'Display', None],
  [ 0xc0ac, 'HangulSyllableSa', [], 'Display', None],
  [ 0xc0ad, 'HangulSyllableSag', [], 'Display', None],
  [ 0xc0ae, 'HangulSyllableSagg', [], 'Display', None],
  [ 0xc0af, 'HangulSyllableSags', [], 'Display', None],
  [ 0xc0b0, 'HangulSyllableSan', [], 'Display', None],
  [ 0xc0b1, 'HangulSyllableSanj', [], 'Display', None],
  [ 0xc0b2, 'HangulSyllableSanh', [], 'Display', None],
  [ 0xc0b3, 'HangulSyllableSad', [], 'Display', None],
  [ 0xc0b4, 'HangulSyllableSal', [], 'Display', None],
  [ 0xc0b5, 'HangulSyllableSalg', [], 'Display', None],
  [ 0xc0b6, 'HangulSyllableSalm', [], 'Display', None],
  [ 0xc0b7, 'HangulSyllableSalb', [], 'Display', None],
  [ 0xc0b8, 'HangulSyllableSals', [], 'Display', None],
  [ 0xc0b9, 'HangulSyllableSalt', [], 'Display', None],
  [ 0xc0ba, 'HangulSyllableSalp', [], 'Display', None],
  [ 0xc0bb, 'HangulSyllableSalh', [], 'Display', None],
  [ 0xc0bc, 'HangulSyllableSam', [], 'Display', None],
  [ 0xc0bd, 'HangulSyllableSab', [], 'Display', None],
  [ 0xc0be, 'HangulSyllableSabs', [], 'Display', None],
  [ 0xc0bf, 'HangulSyllableSas', [], 'Display', None],
  [ 0xc0c0, 'HangulSyllableSass', [], 'Display', None],
  [ 0xc0c1, 'HangulSyllableSang', [], 'Display', None],
  [ 0xc0c2, 'HangulSyllableSaj', [], 'Display', None],
  [ 0xc0c3, 'HangulSyllableSac', [], 'Display', None],
  [ 0xc0c4, 'HangulSyllableSak', [], 'Display', None],
  [ 0xc0c5, 'HangulSyllableSat', [], 'Display', None],
  [ 0xc0c6, 'HangulSyllableSap', [], 'Display', None],
  [ 0xc0c7, 'HangulSyllableSah', [], 'Display', None],
  [ 0xc0c8, 'HangulSyllableSae', [], 'Display', None],
  [ 0xc0c9, 'HangulSyllableSaeg', [], 'Display', None],
  [ 0xc0ca, 'HangulSyllableSaegg', [], 'Display', None],
  [ 0xc0cb, 'HangulSyllableSaegs', [], 'Display', None],
  [ 0xc0cc, 'HangulSyllableSaen', [], 'Display', None],
  [ 0xc0cd, 'HangulSyllableSaenj', [], 'Display', None],
  [ 0xc0ce, 'HangulSyllableSaenh', [], 'Display', None],
  [ 0xc0cf, 'HangulSyllableSaed', [], 'Display', None],
  [ 0xc0d0, 'HangulSyllableSael', [], 'Display', None],
  [ 0xc0d1, 'HangulSyllableSaelg', [], 'Display', None],
  [ 0xc0d2, 'HangulSyllableSaelm', [], 'Display', None],
  [ 0xc0d3, 'HangulSyllableSaelb', [], 'Display', None],
  [ 0xc0d4, 'HangulSyllableSaels', [], 'Display', None],
  [ 0xc0d5, 'HangulSyllableSaelt', [], 'Display', None],
  [ 0xc0d6, 'HangulSyllableSaelp', [], 'Display', None],
  [ 0xc0d7, 'HangulSyllableSaelh', [], 'Display', None],
  [ 0xc0d8, 'HangulSyllableSaem', [], 'Display', None],
  [ 0xc0d9, 'HangulSyllableSaeb', [], 'Display', None],
  [ 0xc0da, 'HangulSyllableSaebs', [], 'Display', None],
  [ 0xc0db, 'HangulSyllableSaes', [], 'Display', None],
  [ 0xc0dc, 'HangulSyllableSaess', [], 'Display', None],
  [ 0xc0dd, 'HangulSyllableSaeng', [], 'Display', None],
  [ 0xc0de, 'HangulSyllableSaej', [], 'Display', None],
  [ 0xc0df, 'HangulSyllableSaec', [], 'Display', None],
  [ 0xc0e0, 'HangulSyllableSaek', [], 'Display', None],
  [ 0xc0e1, 'HangulSyllableSaet', [], 'Display', None],
  [ 0xc0e2, 'HangulSyllableSaep', [], 'Display', None],
  [ 0xc0e3, 'HangulSyllableSaeh', [], 'Display', None],
  [ 0xc0e4, 'HangulSyllableSya', [], 'Display', None],
  [ 0xc0e5, 'HangulSyllableSyag', [], 'Display', None],
  [ 0xc0e6, 'HangulSyllableSyagg', [], 'Display', None],
  [ 0xc0e7, 'HangulSyllableSyags', [], 'Display', None],
  [ 0xc0e8, 'HangulSyllableSyan', [], 'Display', None],
  [ 0xc0e9, 'HangulSyllableSyanj', [], 'Display', None],
  [ 0xc0ea, 'HangulSyllableSyanh', [], 'Display', None],
  [ 0xc0eb, 'HangulSyllableSyad', [], 'Display', None],
  [ 0xc0ec, 'HangulSyllableSyal', [], 'Display', None],
  [ 0xc0ed, 'HangulSyllableSyalg', [], 'Display', None],
  [ 0xc0ee, 'HangulSyllableSyalm', [], 'Display', None],
  [ 0xc0ef, 'HangulSyllableSyalb', [], 'Display', None],
  [ 0xc0f0, 'HangulSyllableSyals', [], 'Display', None],
  [ 0xc0f1, 'HangulSyllableSyalt', [], 'Display', None],
  [ 0xc0f2, 'HangulSyllableSyalp', [], 'Display', None],
  [ 0xc0f3, 'HangulSyllableSyalh', [], 'Display', None],
  [ 0xc0f4, 'HangulSyllableSyam', [], 'Display', None],
  [ 0xc0f5, 'HangulSyllableSyab', [], 'Display', None],
  [ 0xc0f6, 'HangulSyllableSyabs', [], 'Display', None],
  [ 0xc0f7, 'HangulSyllableSyas', [], 'Display', None],
  [ 0xc0f8, 'HangulSyllableSyass', [], 'Display', None],
  [ 0xc0f9, 'HangulSyllableSyang', [], 'Display', None],
  [ 0xc0fa, 'HangulSyllableSyaj', [], 'Display', None],
  [ 0xc0fb, 'HangulSyllableSyac', [], 'Display', None],
  [ 0xc0fc, 'HangulSyllableSyak', [], 'Display', None],
  [ 0xc0fd, 'HangulSyllableSyat', [], 'Display', None],
  [ 0xc0fe, 'HangulSyllableSyap', [], 'Display', None],
  [ 0xc0ff, 'HangulSyllableSyah', [], 'Display', None],
  [ 0xc100, 'HangulSyllableSyae', [], 'Display', None],
  [ 0xc101, 'HangulSyllableSyaeg', [], 'Display', None],
  [ 0xc102, 'HangulSyllableSyaegg', [], 'Display', None],
  [ 0xc103, 'HangulSyllableSyaegs', [], 'Display', None],
  [ 0xc104, 'HangulSyllableSyaen', [], 'Display', None],
  [ 0xc105, 'HangulSyllableSyaenj', [], 'Display', None],
  [ 0xc106, 'HangulSyllableSyaenh', [], 'Display', None],
  [ 0xc107, 'HangulSyllableSyaed', [], 'Display', None],
  [ 0xc108, 'HangulSyllableSyael', [], 'Display', None],
  [ 0xc109, 'HangulSyllableSyaelg', [], 'Display', None],
  [ 0xc10a, 'HangulSyllableSyaelm', [], 'Display', None],
  [ 0xc10b, 'HangulSyllableSyaelb', [], 'Display', None],
  [ 0xc10c, 'HangulSyllableSyaels', [], 'Display', None],
  [ 0xc10d, 'HangulSyllableSyaelt', [], 'Display', None],
  [ 0xc10e, 'HangulSyllableSyaelp', [], 'Display', None],
  [ 0xc10f, 'HangulSyllableSyaelh', [], 'Display', None],
  [ 0xc110, 'HangulSyllableSyaem', [], 'Display', None],
  [ 0xc111, 'HangulSyllableSyaeb', [], 'Display', None],
  [ 0xc112, 'HangulSyllableSyaebs', [], 'Display', None],
  [ 0xc113, 'HangulSyllableSyaes', [], 'Display', None],
  [ 0xc114, 'HangulSyllableSyaess', [], 'Display', None],
  [ 0xc115, 'HangulSyllableSyaeng', [], 'Display', None],
  [ 0xc116, 'HangulSyllableSyaej', [], 'Display', None],
  [ 0xc117, 'HangulSyllableSyaec', [], 'Display', None],
  [ 0xc118, 'HangulSyllableSyaek', [], 'Display', None],
  [ 0xc119, 'HangulSyllableSyaet', [], 'Display', None],
  [ 0xc11a, 'HangulSyllableSyaep', [], 'Display', None],
  [ 0xc11b, 'HangulSyllableSyaeh', [], 'Display', None],
  [ 0xc11c, 'HangulSyllableSeo', [], 'Display', None],
  [ 0xc11d, 'HangulSyllableSeog', [], 'Display', None],
  [ 0xc11e, 'HangulSyllableSeogg', [], 'Display', None],
  [ 0xc11f, 'HangulSyllableSeogs', [], 'Display', None],
  [ 0xc120, 'HangulSyllableSeon', [], 'Display', None],
  [ 0xc121, 'HangulSyllableSeonj', [], 'Display', None],
  [ 0xc122, 'HangulSyllableSeonh', [], 'Display', None],
  [ 0xc123, 'HangulSyllableSeod', [], 'Display', None],
  [ 0xc124, 'HangulSyllableSeol', [], 'Display', None],
  [ 0xc125, 'HangulSyllableSeolg', [], 'Display', None],
  [ 0xc126, 'HangulSyllableSeolm', [], 'Display', None],
  [ 0xc127, 'HangulSyllableSeolb', [], 'Display', None],
  [ 0xc128, 'HangulSyllableSeols', [], 'Display', None],
  [ 0xc129, 'HangulSyllableSeolt', [], 'Display', None],
  [ 0xc12a, 'HangulSyllableSeolp', [], 'Display', None],
  [ 0xc12b, 'HangulSyllableSeolh', [], 'Display', None],
  [ 0xc12c, 'HangulSyllableSeom', [], 'Display', None],
  [ 0xc12d, 'HangulSyllableSeob', [], 'Display', None],
  [ 0xc12e, 'HangulSyllableSeobs', [], 'Display', None],
  [ 0xc12f, 'HangulSyllableSeos', [], 'Display', None],
  [ 0xc130, 'HangulSyllableSeoss', [], 'Display', None],
  [ 0xc131, 'HangulSyllableSeong', [], 'Display', None],
  [ 0xc132, 'HangulSyllableSeoj', [], 'Display', None],
  [ 0xc133, 'HangulSyllableSeoc', [], 'Display', None],
  [ 0xc134, 'HangulSyllableSeok', [], 'Display', None],
  [ 0xc135, 'HangulSyllableSeot', [], 'Display', None],
  [ 0xc136, 'HangulSyllableSeop', [], 'Display', None],
  [ 0xc137, 'HangulSyllableSeoh', [], 'Display', None],
  [ 0xc138, 'HangulSyllableSe', [], 'Display', None],
  [ 0xc139, 'HangulSyllableSeg', [], 'Display', None],
  [ 0xc13a, 'HangulSyllableSegg', [], 'Display', None],
  [ 0xc13b, 'HangulSyllableSegs', [], 'Display', None],
  [ 0xc13c, 'HangulSyllableSen', [], 'Display', None],
  [ 0xc13d, 'HangulSyllableSenj', [], 'Display', None],
  [ 0xc13e, 'HangulSyllableSenh', [], 'Display', None],
  [ 0xc13f, 'HangulSyllableSed', [], 'Display', None],
  [ 0xc140, 'HangulSyllableSel', [], 'Display', None],
  [ 0xc141, 'HangulSyllableSelg', [], 'Display', None],
  [ 0xc142, 'HangulSyllableSelm', [], 'Display', None],
  [ 0xc143, 'HangulSyllableSelb', [], 'Display', None],
  [ 0xc144, 'HangulSyllableSels', [], 'Display', None],
  [ 0xc145, 'HangulSyllableSelt', [], 'Display', None],
  [ 0xc146, 'HangulSyllableSelp', [], 'Display', None],
  [ 0xc147, 'HangulSyllableSelh', [], 'Display', None],
  [ 0xc148, 'HangulSyllableSem', [], 'Display', None],
  [ 0xc149, 'HangulSyllableSeb', [], 'Display', None],
  [ 0xc14a, 'HangulSyllableSebs', [], 'Display', None],
  [ 0xc14b, 'HangulSyllableSes', [], 'Display', None],
  [ 0xc14c, 'HangulSyllableSess', [], 'Display', None],
  [ 0xc14d, 'HangulSyllableSeng', [], 'Display', None],
  [ 0xc14e, 'HangulSyllableSej', [], 'Display', None],
  [ 0xc14f, 'HangulSyllableSec', [], 'Display', None],
  [ 0xc150, 'HangulSyllableSek', [], 'Display', None],
  [ 0xc151, 'HangulSyllableSet', [], 'Display', None],
  [ 0xc152, 'HangulSyllableSep', [], 'Display', None],
  [ 0xc153, 'HangulSyllableSeh', [], 'Display', None],
  [ 0xc154, 'HangulSyllableSyeo', [], 'Display', None],
  [ 0xc155, 'HangulSyllableSyeog', [], 'Display', None],
  [ 0xc156, 'HangulSyllableSyeogg', [], 'Display', None],
  [ 0xc157, 'HangulSyllableSyeogs', [], 'Display', None],
  [ 0xc158, 'HangulSyllableSyeon', [], 'Display', None],
  [ 0xc159, 'HangulSyllableSyeonj', [], 'Display', None],
  [ 0xc15a, 'HangulSyllableSyeonh', [], 'Display', None],
  [ 0xc15b, 'HangulSyllableSyeod', [], 'Display', None],
  [ 0xc15c, 'HangulSyllableSyeol', [], 'Display', None],
  [ 0xc15d, 'HangulSyllableSyeolg', [], 'Display', None],
  [ 0xc15e, 'HangulSyllableSyeolm', [], 'Display', None],
  [ 0xc15f, 'HangulSyllableSyeolb', [], 'Display', None],
  [ 0xc160, 'HangulSyllableSyeols', [], 'Display', None],
  [ 0xc161, 'HangulSyllableSyeolt', [], 'Display', None],
  [ 0xc162, 'HangulSyllableSyeolp', [], 'Display', None],
  [ 0xc163, 'HangulSyllableSyeolh', [], 'Display', None],
  [ 0xc164, 'HangulSyllableSyeom', [], 'Display', None],
  [ 0xc165, 'HangulSyllableSyeob', [], 'Display', None],
  [ 0xc166, 'HangulSyllableSyeobs', [], 'Display', None],
  [ 0xc167, 'HangulSyllableSyeos', [], 'Display', None],
  [ 0xc168, 'HangulSyllableSyeoss', [], 'Display', None],
  [ 0xc169, 'HangulSyllableSyeong', [], 'Display', None],
  [ 0xc16a, 'HangulSyllableSyeoj', [], 'Display', None],
  [ 0xc16b, 'HangulSyllableSyeoc', [], 'Display', None],
  [ 0xc16c, 'HangulSyllableSyeok', [], 'Display', None],
  [ 0xc16d, 'HangulSyllableSyeot', [], 'Display', None],
  [ 0xc16e, 'HangulSyllableSyeop', [], 'Display', None],
  [ 0xc16f, 'HangulSyllableSyeoh', [], 'Display', None],
  [ 0xc170, 'HangulSyllableSye', [], 'Display', None],
  [ 0xc171, 'HangulSyllableSyeg', [], 'Display', None],
  [ 0xc172, 'HangulSyllableSyegg', [], 'Display', None],
  [ 0xc173, 'HangulSyllableSyegs', [], 'Display', None],
  [ 0xc174, 'HangulSyllableSyen', [], 'Display', None],
  [ 0xc175, 'HangulSyllableSyenj', [], 'Display', None],
  [ 0xc176, 'HangulSyllableSyenh', [], 'Display', None],
  [ 0xc177, 'HangulSyllableSyed', [], 'Display', None],
  [ 0xc178, 'HangulSyllableSyel', [], 'Display', None],
  [ 0xc179, 'HangulSyllableSyelg', [], 'Display', None],
  [ 0xc17a, 'HangulSyllableSyelm', [], 'Display', None],
  [ 0xc17b, 'HangulSyllableSyelb', [], 'Display', None],
  [ 0xc17c, 'HangulSyllableSyels', [], 'Display', None],
  [ 0xc17d, 'HangulSyllableSyelt', [], 'Display', None],
  [ 0xc17e, 'HangulSyllableSyelp', [], 'Display', None],
  [ 0xc17f, 'HangulSyllableSyelh', [], 'Display', None],
  [ 0xc180, 'HangulSyllableSyem', [], 'Display', None],
  [ 0xc181, 'HangulSyllableSyeb', [], 'Display', None],
  [ 0xc182, 'HangulSyllableSyebs', [], 'Display', None],
  [ 0xc183, 'HangulSyllableSyes', [], 'Display', None],
  [ 0xc184, 'HangulSyllableSyess', [], 'Display', None],
  [ 0xc185, 'HangulSyllableSyeng', [], 'Display', None],
  [ 0xc186, 'HangulSyllableSyej', [], 'Display', None],
  [ 0xc187, 'HangulSyllableSyec', [], 'Display', None],
  [ 0xc188, 'HangulSyllableSyek', [], 'Display', None],
  [ 0xc189, 'HangulSyllableSyet', [], 'Display', None],
  [ 0xc18a, 'HangulSyllableSyep', [], 'Display', None],
  [ 0xc18b, 'HangulSyllableSyeh', [], 'Display', None],
  [ 0xc18c, 'HangulSyllableSo', [], 'Display', None],
  [ 0xc18d, 'HangulSyllableSog', [], 'Display', None],
  [ 0xc18e, 'HangulSyllableSogg', [], 'Display', None],
  [ 0xc18f, 'HangulSyllableSogs', [], 'Display', None],
  [ 0xc190, 'HangulSyllableSon', [], 'Display', None],
  [ 0xc191, 'HangulSyllableSonj', [], 'Display', None],
  [ 0xc192, 'HangulSyllableSonh', [], 'Display', None],
  [ 0xc193, 'HangulSyllableSod', [], 'Display', None],
  [ 0xc194, 'HangulSyllableSol', [], 'Display', None],
  [ 0xc195, 'HangulSyllableSolg', [], 'Display', None],
  [ 0xc196, 'HangulSyllableSolm', [], 'Display', None],
  [ 0xc197, 'HangulSyllableSolb', [], 'Display', None],
  [ 0xc198, 'HangulSyllableSols', [], 'Display', None],
  [ 0xc199, 'HangulSyllableSolt', [], 'Display', None],
  [ 0xc19a, 'HangulSyllableSolp', [], 'Display', None],
  [ 0xc19b, 'HangulSyllableSolh', [], 'Display', None],
  [ 0xc19c, 'HangulSyllableSom', [], 'Display', None],
  [ 0xc19d, 'HangulSyllableSob', [], 'Display', None],
  [ 0xc19e, 'HangulSyllableSobs', [], 'Display', None],
  [ 0xc19f, 'HangulSyllableSos', [], 'Display', None],
  [ 0xc1a0, 'HangulSyllableSoss', [], 'Display', None],
  [ 0xc1a1, 'HangulSyllableSong', [], 'Display', None],
  [ 0xc1a2, 'HangulSyllableSoj', [], 'Display', None],
  [ 0xc1a3, 'HangulSyllableSoc', [], 'Display', None],
  [ 0xc1a4, 'HangulSyllableSok', [], 'Display', None],
  [ 0xc1a5, 'HangulSyllableSot', [], 'Display', None],
  [ 0xc1a6, 'HangulSyllableSop', [], 'Display', None],
  [ 0xc1a7, 'HangulSyllableSoh', [], 'Display', None],
  [ 0xc1a8, 'HangulSyllableSwa', [], 'Display', None],
  [ 0xc1a9, 'HangulSyllableSwag', [], 'Display', None],
  [ 0xc1aa, 'HangulSyllableSwagg', [], 'Display', None],
  [ 0xc1ab, 'HangulSyllableSwags', [], 'Display', None],
  [ 0xc1ac, 'HangulSyllableSwan', [], 'Display', None],
  [ 0xc1ad, 'HangulSyllableSwanj', [], 'Display', None],
  [ 0xc1ae, 'HangulSyllableSwanh', [], 'Display', None],
  [ 0xc1af, 'HangulSyllableSwad', [], 'Display', None],
  [ 0xc1b0, 'HangulSyllableSwal', [], 'Display', None],
  [ 0xc1b1, 'HangulSyllableSwalg', [], 'Display', None],
  [ 0xc1b2, 'HangulSyllableSwalm', [], 'Display', None],
  [ 0xc1b3, 'HangulSyllableSwalb', [], 'Display', None],
  [ 0xc1b4, 'HangulSyllableSwals', [], 'Display', None],
  [ 0xc1b5, 'HangulSyllableSwalt', [], 'Display', None],
  [ 0xc1b6, 'HangulSyllableSwalp', [], 'Display', None],
  [ 0xc1b7, 'HangulSyllableSwalh', [], 'Display', None],
  [ 0xc1b8, 'HangulSyllableSwam', [], 'Display', None],
  [ 0xc1b9, 'HangulSyllableSwab', [], 'Display', None],
  [ 0xc1ba, 'HangulSyllableSwabs', [], 'Display', None],
  [ 0xc1bb, 'HangulSyllableSwas', [], 'Display', None],
  [ 0xc1bc, 'HangulSyllableSwass', [], 'Display', None],
  [ 0xc1bd, 'HangulSyllableSwang', [], 'Display', None],
  [ 0xc1be, 'HangulSyllableSwaj', [], 'Display', None],
  [ 0xc1bf, 'HangulSyllableSwac', [], 'Display', None],
  [ 0xc1c0, 'HangulSyllableSwak', [], 'Display', None],
  [ 0xc1c1, 'HangulSyllableSwat', [], 'Display', None],
  [ 0xc1c2, 'HangulSyllableSwap', [], 'Display', None],
  [ 0xc1c3, 'HangulSyllableSwah', [], 'Display', None],
  [ 0xc1c4, 'HangulSyllableSwae', [], 'Display', None],
  [ 0xc1c5, 'HangulSyllableSwaeg', [], 'Display', None],
  [ 0xc1c6, 'HangulSyllableSwaegg', [], 'Display', None],
  [ 0xc1c7, 'HangulSyllableSwaegs', [], 'Display', None],
  [ 0xc1c8, 'HangulSyllableSwaen', [], 'Display', None],
  [ 0xc1c9, 'HangulSyllableSwaenj', [], 'Display', None],
  [ 0xc1ca, 'HangulSyllableSwaenh', [], 'Display', None],
  [ 0xc1cb, 'HangulSyllableSwaed', [], 'Display', None],
  [ 0xc1cc, 'HangulSyllableSwael', [], 'Display', None],
  [ 0xc1cd, 'HangulSyllableSwaelg', [], 'Display', None],
  [ 0xc1ce, 'HangulSyllableSwaelm', [], 'Display', None],
  [ 0xc1cf, 'HangulSyllableSwaelb', [], 'Display', None],
  [ 0xc1d0, 'HangulSyllableSwaels', [], 'Display', None],
  [ 0xc1d1, 'HangulSyllableSwaelt', [], 'Display', None],
  [ 0xc1d2, 'HangulSyllableSwaelp', [], 'Display', None],
  [ 0xc1d3, 'HangulSyllableSwaelh', [], 'Display', None],
  [ 0xc1d4, 'HangulSyllableSwaem', [], 'Display', None],
  [ 0xc1d5, 'HangulSyllableSwaeb', [], 'Display', None],
  [ 0xc1d6, 'HangulSyllableSwaebs', [], 'Display', None],
  [ 0xc1d7, 'HangulSyllableSwaes', [], 'Display', None],
  [ 0xc1d8, 'HangulSyllableSwaess', [], 'Display', None],
  [ 0xc1d9, 'HangulSyllableSwaeng', [], 'Display', None],
  [ 0xc1da, 'HangulSyllableSwaej', [], 'Display', None],
  [ 0xc1db, 'HangulSyllableSwaec', [], 'Display', None],
  [ 0xc1dc, 'HangulSyllableSwaek', [], 'Display', None],
  [ 0xc1dd, 'HangulSyllableSwaet', [], 'Display', None],
  [ 0xc1de, 'HangulSyllableSwaep', [], 'Display', None],
  [ 0xc1df, 'HangulSyllableSwaeh', [], 'Display', None],
  [ 0xc1e0, 'HangulSyllableSoe', [], 'Display', None],
  [ 0xc1e1, 'HangulSyllableSoeg', [], 'Display', None],
  [ 0xc1e2, 'HangulSyllableSoegg', [], 'Display', None],
  [ 0xc1e3, 'HangulSyllableSoegs', [], 'Display', None],
  [ 0xc1e4, 'HangulSyllableSoen', [], 'Display', None],
  [ 0xc1e5, 'HangulSyllableSoenj', [], 'Display', None],
  [ 0xc1e6, 'HangulSyllableSoenh', [], 'Display', None],
  [ 0xc1e7, 'HangulSyllableSoed', [], 'Display', None],
  [ 0xc1e8, 'HangulSyllableSoel', [], 'Display', None],
  [ 0xc1e9, 'HangulSyllableSoelg', [], 'Display', None],
  [ 0xc1ea, 'HangulSyllableSoelm', [], 'Display', None],
  [ 0xc1eb, 'HangulSyllableSoelb', [], 'Display', None],
  [ 0xc1ec, 'HangulSyllableSoels', [], 'Display', None],
  [ 0xc1ed, 'HangulSyllableSoelt', [], 'Display', None],
  [ 0xc1ee, 'HangulSyllableSoelp', [], 'Display', None],
  [ 0xc1ef, 'HangulSyllableSoelh', [], 'Display', None],
  [ 0xc1f0, 'HangulSyllableSoem', [], 'Display', None],
  [ 0xc1f1, 'HangulSyllableSoeb', [], 'Display', None],
  [ 0xc1f2, 'HangulSyllableSoebs', [], 'Display', None],
  [ 0xc1f3, 'HangulSyllableSoes', [], 'Display', None],
  [ 0xc1f4, 'HangulSyllableSoess', [], 'Display', None],
  [ 0xc1f5, 'HangulSyllableSoeng', [], 'Display', None],
  [ 0xc1f6, 'HangulSyllableSoej', [], 'Display', None],
  [ 0xc1f7, 'HangulSyllableSoec', [], 'Display', None],
  [ 0xc1f8, 'HangulSyllableSoek', [], 'Display', None],
  [ 0xc1f9, 'HangulSyllableSoet', [], 'Display', None],
  [ 0xc1fa, 'HangulSyllableSoep', [], 'Display', None],
  [ 0xc1fb, 'HangulSyllableSoeh', [], 'Display', None],
  [ 0xc1fc, 'HangulSyllableSyo', [], 'Display', None],
  [ 0xc1fd, 'HangulSyllableSyog', [], 'Display', None],
  [ 0xc1fe, 'HangulSyllableSyogg', [], 'Display', None],
  [ 0xc1ff, 'HangulSyllableSyogs', [], 'Display', None],
  [ 0xc200, 'HangulSyllableSyon', [], 'Display', None],
  [ 0xc201, 'HangulSyllableSyonj', [], 'Display', None],
  [ 0xc202, 'HangulSyllableSyonh', [], 'Display', None],
  [ 0xc203, 'HangulSyllableSyod', [], 'Display', None],
  [ 0xc204, 'HangulSyllableSyol', [], 'Display', None],
  [ 0xc205, 'HangulSyllableSyolg', [], 'Display', None],
  [ 0xc206, 'HangulSyllableSyolm', [], 'Display', None],
  [ 0xc207, 'HangulSyllableSyolb', [], 'Display', None],
  [ 0xc208, 'HangulSyllableSyols', [], 'Display', None],
  [ 0xc209, 'HangulSyllableSyolt', [], 'Display', None],
  [ 0xc20a, 'HangulSyllableSyolp', [], 'Display', None],
  [ 0xc20b, 'HangulSyllableSyolh', [], 'Display', None],
  [ 0xc20c, 'HangulSyllableSyom', [], 'Display', None],
  [ 0xc20d, 'HangulSyllableSyob', [], 'Display', None],
  [ 0xc20e, 'HangulSyllableSyobs', [], 'Display', None],
  [ 0xc20f, 'HangulSyllableSyos', [], 'Display', None],
  [ 0xc210, 'HangulSyllableSyoss', [], 'Display', None],
  [ 0xc211, 'HangulSyllableSyong', [], 'Display', None],
  [ 0xc212, 'HangulSyllableSyoj', [], 'Display', None],
  [ 0xc213, 'HangulSyllableSyoc', [], 'Display', None],
  [ 0xc214, 'HangulSyllableSyok', [], 'Display', None],
  [ 0xc215, 'HangulSyllableSyot', [], 'Display', None],
  [ 0xc216, 'HangulSyllableSyop', [], 'Display', None],
  [ 0xc217, 'HangulSyllableSyoh', [], 'Display', None],
  [ 0xc218, 'HangulSyllableSu', [], 'Display', None],
  [ 0xc219, 'HangulSyllableSug', [], 'Display', None],
  [ 0xc21a, 'HangulSyllableSugg', [], 'Display', None],
  [ 0xc21b, 'HangulSyllableSugs', [], 'Display', None],
  [ 0xc21c, 'HangulSyllableSun', [], 'Display', None],
  [ 0xc21d, 'HangulSyllableSunj', [], 'Display', None],
  [ 0xc21e, 'HangulSyllableSunh', [], 'Display', None],
  [ 0xc21f, 'HangulSyllableSud', [], 'Display', None],
  [ 0xc220, 'HangulSyllableSul', [], 'Display', None],
  [ 0xc221, 'HangulSyllableSulg', [], 'Display', None],
  [ 0xc222, 'HangulSyllableSulm', [], 'Display', None],
  [ 0xc223, 'HangulSyllableSulb', [], 'Display', None],
  [ 0xc224, 'HangulSyllableSuls', [], 'Display', None],
  [ 0xc225, 'HangulSyllableSult', [], 'Display', None],
  [ 0xc226, 'HangulSyllableSulp', [], 'Display', None],
  [ 0xc227, 'HangulSyllableSulh', [], 'Display', None],
  [ 0xc228, 'HangulSyllableSum', [], 'Display', None],
  [ 0xc229, 'HangulSyllableSub', [], 'Display', None],
  [ 0xc22a, 'HangulSyllableSubs', [], 'Display', None],
  [ 0xc22b, 'HangulSyllableSus', [], 'Display', None],
  [ 0xc22c, 'HangulSyllableSuss', [], 'Display', None],
  [ 0xc22d, 'HangulSyllableSung', [], 'Display', None],
  [ 0xc22e, 'HangulSyllableSuj', [], 'Display', None],
  [ 0xc22f, 'HangulSyllableSuc', [], 'Display', None],
  [ 0xc230, 'HangulSyllableSuk', [], 'Display', None],
  [ 0xc231, 'HangulSyllableSut', [], 'Display', None],
  [ 0xc232, 'HangulSyllableSup', [], 'Display', None],
  [ 0xc233, 'HangulSyllableSuh', [], 'Display', None],
  [ 0xc234, 'HangulSyllableSweo', [], 'Display', None],
  [ 0xc235, 'HangulSyllableSweog', [], 'Display', None],
  [ 0xc236, 'HangulSyllableSweogg', [], 'Display', None],
  [ 0xc237, 'HangulSyllableSweogs', [], 'Display', None],
  [ 0xc238, 'HangulSyllableSweon', [], 'Display', None],
  [ 0xc239, 'HangulSyllableSweonj', [], 'Display', None],
  [ 0xc23a, 'HangulSyllableSweonh', [], 'Display', None],
  [ 0xc23b, 'HangulSyllableSweod', [], 'Display', None],
  [ 0xc23c, 'HangulSyllableSweol', [], 'Display', None],
  [ 0xc23d, 'HangulSyllableSweolg', [], 'Display', None],
  [ 0xc23e, 'HangulSyllableSweolm', [], 'Display', None],
  [ 0xc23f, 'HangulSyllableSweolb', [], 'Display', None],
  [ 0xc240, 'HangulSyllableSweols', [], 'Display', None],
  [ 0xc241, 'HangulSyllableSweolt', [], 'Display', None],
  [ 0xc242, 'HangulSyllableSweolp', [], 'Display', None],
  [ 0xc243, 'HangulSyllableSweolh', [], 'Display', None],
  [ 0xc244, 'HangulSyllableSweom', [], 'Display', None],
  [ 0xc245, 'HangulSyllableSweob', [], 'Display', None],
  [ 0xc246, 'HangulSyllableSweobs', [], 'Display', None],
  [ 0xc247, 'HangulSyllableSweos', [], 'Display', None],
  [ 0xc248, 'HangulSyllableSweoss', [], 'Display', None],
  [ 0xc249, 'HangulSyllableSweong', [], 'Display', None],
  [ 0xc24a, 'HangulSyllableSweoj', [], 'Display', None],
  [ 0xc24b, 'HangulSyllableSweoc', [], 'Display', None],
  [ 0xc24c, 'HangulSyllableSweok', [], 'Display', None],
  [ 0xc24d, 'HangulSyllableSweot', [], 'Display', None],
  [ 0xc24e, 'HangulSyllableSweop', [], 'Display', None],
  [ 0xc24f, 'HangulSyllableSweoh', [], 'Display', None],
  [ 0xc250, 'HangulSyllableSwe', [], 'Display', None],
  [ 0xc251, 'HangulSyllableSweg', [], 'Display', None],
  [ 0xc252, 'HangulSyllableSwegg', [], 'Display', None],
  [ 0xc253, 'HangulSyllableSwegs', [], 'Display', None],
  [ 0xc254, 'HangulSyllableSwen', [], 'Display', None],
  [ 0xc255, 'HangulSyllableSwenj', [], 'Display', None],
  [ 0xc256, 'HangulSyllableSwenh', [], 'Display', None],
  [ 0xc257, 'HangulSyllableSwed', [], 'Display', None],
  [ 0xc258, 'HangulSyllableSwel', [], 'Display', None],
  [ 0xc259, 'HangulSyllableSwelg', [], 'Display', None],
  [ 0xc25a, 'HangulSyllableSwelm', [], 'Display', None],
  [ 0xc25b, 'HangulSyllableSwelb', [], 'Display', None],
  [ 0xc25c, 'HangulSyllableSwels', [], 'Display', None],
  [ 0xc25d, 'HangulSyllableSwelt', [], 'Display', None],
  [ 0xc25e, 'HangulSyllableSwelp', [], 'Display', None],
  [ 0xc25f, 'HangulSyllableSwelh', [], 'Display', None],
  [ 0xc260, 'HangulSyllableSwem', [], 'Display', None],
  [ 0xc261, 'HangulSyllableSweb', [], 'Display', None],
  [ 0xc262, 'HangulSyllableSwebs', [], 'Display', None],
  [ 0xc263, 'HangulSyllableSwes', [], 'Display', None],
  [ 0xc264, 'HangulSyllableSwess', [], 'Display', None],
  [ 0xc265, 'HangulSyllableSweng', [], 'Display', None],
  [ 0xc266, 'HangulSyllableSwej', [], 'Display', None],
  [ 0xc267, 'HangulSyllableSwec', [], 'Display', None],
  [ 0xc268, 'HangulSyllableSwek', [], 'Display', None],
  [ 0xc269, 'HangulSyllableSwet', [], 'Display', None],
  [ 0xc26a, 'HangulSyllableSwep', [], 'Display', None],
  [ 0xc26b, 'HangulSyllableSweh', [], 'Display', None],
  [ 0xc26c, 'HangulSyllableSwi', [], 'Display', None],
  [ 0xc26d, 'HangulSyllableSwig', [], 'Display', None],
  [ 0xc26e, 'HangulSyllableSwigg', [], 'Display', None],
  [ 0xc26f, 'HangulSyllableSwigs', [], 'Display', None],
  [ 0xc270, 'HangulSyllableSwin', [], 'Display', None],
  [ 0xc271, 'HangulSyllableSwinj', [], 'Display', None],
  [ 0xc272, 'HangulSyllableSwinh', [], 'Display', None],
  [ 0xc273, 'HangulSyllableSwid', [], 'Display', None],
  [ 0xc274, 'HangulSyllableSwil', [], 'Display', None],
  [ 0xc275, 'HangulSyllableSwilg', [], 'Display', None],
  [ 0xc276, 'HangulSyllableSwilm', [], 'Display', None],
  [ 0xc277, 'HangulSyllableSwilb', [], 'Display', None],
  [ 0xc278, 'HangulSyllableSwils', [], 'Display', None],
  [ 0xc279, 'HangulSyllableSwilt', [], 'Display', None],
  [ 0xc27a, 'HangulSyllableSwilp', [], 'Display', None],
  [ 0xc27b, 'HangulSyllableSwilh', [], 'Display', None],
  [ 0xc27c, 'HangulSyllableSwim', [], 'Display', None],
  [ 0xc27d, 'HangulSyllableSwib', [], 'Display', None],
  [ 0xc27e, 'HangulSyllableSwibs', [], 'Display', None],
  [ 0xc27f, 'HangulSyllableSwis', [], 'Display', None],
  [ 0xc280, 'HangulSyllableSwiss', [], 'Display', None],
  [ 0xc281, 'HangulSyllableSwing', [], 'Display', None],
  [ 0xc282, 'HangulSyllableSwij', [], 'Display', None],
  [ 0xc283, 'HangulSyllableSwic', [], 'Display', None],
  [ 0xc284, 'HangulSyllableSwik', [], 'Display', None],
  [ 0xc285, 'HangulSyllableSwit', [], 'Display', None],
  [ 0xc286, 'HangulSyllableSwip', [], 'Display', None],
  [ 0xc287, 'HangulSyllableSwih', [], 'Display', None],
  [ 0xc288, 'HangulSyllableSyu', [], 'Display', None],
  [ 0xc289, 'HangulSyllableSyug', [], 'Display', None],
  [ 0xc28a, 'HangulSyllableSyugg', [], 'Display', None],
  [ 0xc28b, 'HangulSyllableSyugs', [], 'Display', None],
  [ 0xc28c, 'HangulSyllableSyun', [], 'Display', None],
  [ 0xc28d, 'HangulSyllableSyunj', [], 'Display', None],
  [ 0xc28e, 'HangulSyllableSyunh', [], 'Display', None],
  [ 0xc28f, 'HangulSyllableSyud', [], 'Display', None],
  [ 0xc290, 'HangulSyllableSyul', [], 'Display', None],
  [ 0xc291, 'HangulSyllableSyulg', [], 'Display', None],
  [ 0xc292, 'HangulSyllableSyulm', [], 'Display', None],
  [ 0xc293, 'HangulSyllableSyulb', [], 'Display', None],
  [ 0xc294, 'HangulSyllableSyuls', [], 'Display', None],
  [ 0xc295, 'HangulSyllableSyult', [], 'Display', None],
  [ 0xc296, 'HangulSyllableSyulp', [], 'Display', None],
  [ 0xc297, 'HangulSyllableSyulh', [], 'Display', None],
  [ 0xc298, 'HangulSyllableSyum', [], 'Display', None],
  [ 0xc299, 'HangulSyllableSyub', [], 'Display', None],
  [ 0xc29a, 'HangulSyllableSyubs', [], 'Display', None],
  [ 0xc29b, 'HangulSyllableSyus', [], 'Display', None],
  [ 0xc29c, 'HangulSyllableSyuss', [], 'Display', None],
  [ 0xc29d, 'HangulSyllableSyung', [], 'Display', None],
  [ 0xc29e, 'HangulSyllableSyuj', [], 'Display', None],
  [ 0xc29f, 'HangulSyllableSyuc', [], 'Display', None],
  [ 0xc2a0, 'HangulSyllableSyuk', [], 'Display', None],
  [ 0xc2a1, 'HangulSyllableSyut', [], 'Display', None],
  [ 0xc2a2, 'HangulSyllableSyup', [], 'Display', None],
  [ 0xc2a3, 'HangulSyllableSyuh', [], 'Display', None],
  [ 0xc2a4, 'HangulSyllableSeu', [], 'Display', None],
  [ 0xc2a5, 'HangulSyllableSeug', [], 'Display', None],
  [ 0xc2a6, 'HangulSyllableSeugg', [], 'Display', None],
  [ 0xc2a7, 'HangulSyllableSeugs', [], 'Display', None],
  [ 0xc2a8, 'HangulSyllableSeun', [], 'Display', None],
  [ 0xc2a9, 'HangulSyllableSeunj', [], 'Display', None],
  [ 0xc2aa, 'HangulSyllableSeunh', [], 'Display', None],
  [ 0xc2ab, 'HangulSyllableSeud', [], 'Display', None],
  [ 0xc2ac, 'HangulSyllableSeul', [], 'Display', None],
  [ 0xc2ad, 'HangulSyllableSeulg', [], 'Display', None],
  [ 0xc2ae, 'HangulSyllableSeulm', [], 'Display', None],
  [ 0xc2af, 'HangulSyllableSeulb', [], 'Display', None],
  [ 0xc2b0, 'HangulSyllableSeuls', [], 'Display', None],
  [ 0xc2b1, 'HangulSyllableSeult', [], 'Display', None],
  [ 0xc2b2, 'HangulSyllableSeulp', [], 'Display', None],
  [ 0xc2b3, 'HangulSyllableSeulh', [], 'Display', None],
  [ 0xc2b4, 'HangulSyllableSeum', [], 'Display', None],
  [ 0xc2b5, 'HangulSyllableSeub', [], 'Display', None],
  [ 0xc2b6, 'HangulSyllableSeubs', [], 'Display', None],
  [ 0xc2b7, 'HangulSyllableSeus', [], 'Display', None],
  [ 0xc2b8, 'HangulSyllableSeuss', [], 'Display', None],
  [ 0xc2b9, 'HangulSyllableSeung', [], 'Display', None],
  [ 0xc2ba, 'HangulSyllableSeuj', [], 'Display', None],
  [ 0xc2bb, 'HangulSyllableSeuc', [], 'Display', None],
  [ 0xc2bc, 'HangulSyllableSeuk', [], 'Display', None],
  [ 0xc2bd, 'HangulSyllableSeut', [], 'Display', None],
  [ 0xc2be, 'HangulSyllableSeup', [], 'Display', None],
  [ 0xc2bf, 'HangulSyllableSeuh', [], 'Display', None],
  [ 0xc2c0, 'HangulSyllableSyi', [], 'Display', None],
  [ 0xc2c1, 'HangulSyllableSyig', [], 'Display', None],
  [ 0xc2c2, 'HangulSyllableSyigg', [], 'Display', None],
  [ 0xc2c3, 'HangulSyllableSyigs', [], 'Display', None],
  [ 0xc2c4, 'HangulSyllableSyin', [], 'Display', None],
  [ 0xc2c5, 'HangulSyllableSyinj', [], 'Display', None],
  [ 0xc2c6, 'HangulSyllableSyinh', [], 'Display', None],
  [ 0xc2c7, 'HangulSyllableSyid', [], 'Display', None],
  [ 0xc2c8, 'HangulSyllableSyil', [], 'Display', None],
  [ 0xc2c9, 'HangulSyllableSyilg', [], 'Display', None],
  [ 0xc2ca, 'HangulSyllableSyilm', [], 'Display', None],
  [ 0xc2cb, 'HangulSyllableSyilb', [], 'Display', None],
  [ 0xc2cc, 'HangulSyllableSyils', [], 'Display', None],
  [ 0xc2cd, 'HangulSyllableSyilt', [], 'Display', None],
  [ 0xc2ce, 'HangulSyllableSyilp', [], 'Display', None],
  [ 0xc2cf, 'HangulSyllableSyilh', [], 'Display', None],
  [ 0xc2d0, 'HangulSyllableSyim', [], 'Display', None],
  [ 0xc2d1, 'HangulSyllableSyib', [], 'Display', None],
  [ 0xc2d2, 'HangulSyllableSyibs', [], 'Display', None],
  [ 0xc2d3, 'HangulSyllableSyis', [], 'Display', None],
  [ 0xc2d4, 'HangulSyllableSyiss', [], 'Display', None],
  [ 0xc2d5, 'HangulSyllableSying', [], 'Display', None],
  [ 0xc2d6, 'HangulSyllableSyij', [], 'Display', None],
  [ 0xc2d7, 'HangulSyllableSyic', [], 'Display', None],
  [ 0xc2d8, 'HangulSyllableSyik', [], 'Display', None],
  [ 0xc2d9, 'HangulSyllableSyit', [], 'Display', None],
  [ 0xc2da, 'HangulSyllableSyip', [], 'Display', None],
  [ 0xc2db, 'HangulSyllableSyih', [], 'Display', None],
  [ 0xc2dc, 'HangulSyllableSi', [], 'Display', None],
  [ 0xc2dd, 'HangulSyllableSig', [], 'Display', None],
  [ 0xc2de, 'HangulSyllableSigg', [], 'Display', None],
  [ 0xc2df, 'HangulSyllableSigs', [], 'Display', None],
  [ 0xc2e0, 'HangulSyllableSin', [], 'Display', None],
  [ 0xc2e1, 'HangulSyllableSinj', [], 'Display', None],
  [ 0xc2e2, 'HangulSyllableSinh', [], 'Display', None],
  [ 0xc2e3, 'HangulSyllableSid', [], 'Display', None],
  [ 0xc2e4, 'HangulSyllableSil', [], 'Display', None],
  [ 0xc2e5, 'HangulSyllableSilg', [], 'Display', None],
  [ 0xc2e6, 'HangulSyllableSilm', [], 'Display', None],
  [ 0xc2e7, 'HangulSyllableSilb', [], 'Display', None],
  [ 0xc2e8, 'HangulSyllableSils', [], 'Display', None],
  [ 0xc2e9, 'HangulSyllableSilt', [], 'Display', None],
  [ 0xc2ea, 'HangulSyllableSilp', [], 'Display', None],
  [ 0xc2eb, 'HangulSyllableSilh', [], 'Display', None],
  [ 0xc2ec, 'HangulSyllableSim', [], 'Display', None],
  [ 0xc2ed, 'HangulSyllableSib', [], 'Display', None],
  [ 0xc2ee, 'HangulSyllableSibs', [], 'Display', None],
  [ 0xc2ef, 'HangulSyllableSis', [], 'Display', None],
  [ 0xc2f0, 'HangulSyllableSiss', [], 'Display', None],
  [ 0xc2f1, 'HangulSyllableSing', [], 'Display', None],
  [ 0xc2f2, 'HangulSyllableSij', [], 'Display', None],
  [ 0xc2f3, 'HangulSyllableSic', [], 'Display', None],
  [ 0xc2f4, 'HangulSyllableSik', [], 'Display', None],
  [ 0xc2f5, 'HangulSyllableSit', [], 'Display', None],
  [ 0xc2f6, 'HangulSyllableSip', [], 'Display', None],
  [ 0xc2f7, 'HangulSyllableSih', [], 'Display', None],
  [ 0xc2f8, 'HangulSyllableSsa', [], 'Display', None],
  [ 0xc2f9, 'HangulSyllableSsag', [], 'Display', None],
  [ 0xc2fa, 'HangulSyllableSsagg', [], 'Display', None],
  [ 0xc2fb, 'HangulSyllableSsags', [], 'Display', None],
  [ 0xc2fc, 'HangulSyllableSsan', [], 'Display', None],
  [ 0xc2fd, 'HangulSyllableSsanj', [], 'Display', None],
  [ 0xc2fe, 'HangulSyllableSsanh', [], 'Display', None],
  [ 0xc2ff, 'HangulSyllableSsad', [], 'Display', None],
  [ 0xc300, 'HangulSyllableSsal', [], 'Display', None],
  [ 0xc301, 'HangulSyllableSsalg', [], 'Display', None],
  [ 0xc302, 'HangulSyllableSsalm', [], 'Display', None],
  [ 0xc303, 'HangulSyllableSsalb', [], 'Display', None],
  [ 0xc304, 'HangulSyllableSsals', [], 'Display', None],
  [ 0xc305, 'HangulSyllableSsalt', [], 'Display', None],
  [ 0xc306, 'HangulSyllableSsalp', [], 'Display', None],
  [ 0xc307, 'HangulSyllableSsalh', [], 'Display', None],
  [ 0xc308, 'HangulSyllableSsam', [], 'Display', None],
  [ 0xc309, 'HangulSyllableSsab', [], 'Display', None],
  [ 0xc30a, 'HangulSyllableSsabs', [], 'Display', None],
  [ 0xc30b, 'HangulSyllableSsas', [], 'Display', None],
  [ 0xc30c, 'HangulSyllableSsass', [], 'Display', None],
  [ 0xc30d, 'HangulSyllableSsang', [], 'Display', None],
  [ 0xc30e, 'HangulSyllableSsaj', [], 'Display', None],
  [ 0xc30f, 'HangulSyllableSsac', [], 'Display', None],
  [ 0xc310, 'HangulSyllableSsak', [], 'Display', None],
  [ 0xc311, 'HangulSyllableSsat', [], 'Display', None],
  [ 0xc312, 'HangulSyllableSsap', [], 'Display', None],
  [ 0xc313, 'HangulSyllableSsah', [], 'Display', None],
  [ 0xc314, 'HangulSyllableSsae', [], 'Display', None],
  [ 0xc315, 'HangulSyllableSsaeg', [], 'Display', None],
  [ 0xc316, 'HangulSyllableSsaegg', [], 'Display', None],
  [ 0xc317, 'HangulSyllableSsaegs', [], 'Display', None],
  [ 0xc318, 'HangulSyllableSsaen', [], 'Display', None],
  [ 0xc319, 'HangulSyllableSsaenj', [], 'Display', None],
  [ 0xc31a, 'HangulSyllableSsaenh', [], 'Display', None],
  [ 0xc31b, 'HangulSyllableSsaed', [], 'Display', None],
  [ 0xc31c, 'HangulSyllableSsael', [], 'Display', None],
  [ 0xc31d, 'HangulSyllableSsaelg', [], 'Display', None],
  [ 0xc31e, 'HangulSyllableSsaelm', [], 'Display', None],
  [ 0xc31f, 'HangulSyllableSsaelb', [], 'Display', None],
  [ 0xc320, 'HangulSyllableSsaels', [], 'Display', None],
  [ 0xc321, 'HangulSyllableSsaelt', [], 'Display', None],
  [ 0xc322, 'HangulSyllableSsaelp', [], 'Display', None],
  [ 0xc323, 'HangulSyllableSsaelh', [], 'Display', None],
  [ 0xc324, 'HangulSyllableSsaem', [], 'Display', None],
  [ 0xc325, 'HangulSyllableSsaeb', [], 'Display', None],
  [ 0xc326, 'HangulSyllableSsaebs', [], 'Display', None],
  [ 0xc327, 'HangulSyllableSsaes', [], 'Display', None],
  [ 0xc328, 'HangulSyllableSsaess', [], 'Display', None],
  [ 0xc329, 'HangulSyllableSsaeng', [], 'Display', None],
  [ 0xc32a, 'HangulSyllableSsaej', [], 'Display', None],
  [ 0xc32b, 'HangulSyllableSsaec', [], 'Display', None],
  [ 0xc32c, 'HangulSyllableSsaek', [], 'Display', None],
  [ 0xc32d, 'HangulSyllableSsaet', [], 'Display', None],
  [ 0xc32e, 'HangulSyllableSsaep', [], 'Display', None],
  [ 0xc32f, 'HangulSyllableSsaeh', [], 'Display', None],
  [ 0xc330, 'HangulSyllableSsya', [], 'Display', None],
  [ 0xc331, 'HangulSyllableSsyag', [], 'Display', None],
  [ 0xc332, 'HangulSyllableSsyagg', [], 'Display', None],
  [ 0xc333, 'HangulSyllableSsyags', [], 'Display', None],
  [ 0xc334, 'HangulSyllableSsyan', [], 'Display', None],
  [ 0xc335, 'HangulSyllableSsyanj', [], 'Display', None],
  [ 0xc336, 'HangulSyllableSsyanh', [], 'Display', None],
  [ 0xc337, 'HangulSyllableSsyad', [], 'Display', None],
  [ 0xc338, 'HangulSyllableSsyal', [], 'Display', None],
  [ 0xc339, 'HangulSyllableSsyalg', [], 'Display', None],
  [ 0xc33a, 'HangulSyllableSsyalm', [], 'Display', None],
  [ 0xc33b, 'HangulSyllableSsyalb', [], 'Display', None],
  [ 0xc33c, 'HangulSyllableSsyals', [], 'Display', None],
  [ 0xc33d, 'HangulSyllableSsyalt', [], 'Display', None],
  [ 0xc33e, 'HangulSyllableSsyalp', [], 'Display', None],
  [ 0xc33f, 'HangulSyllableSsyalh', [], 'Display', None],
  [ 0xc340, 'HangulSyllableSsyam', [], 'Display', None],
  [ 0xc341, 'HangulSyllableSsyab', [], 'Display', None],
  [ 0xc342, 'HangulSyllableSsyabs', [], 'Display', None],
  [ 0xc343, 'HangulSyllableSsyas', [], 'Display', None],
  [ 0xc344, 'HangulSyllableSsyass', [], 'Display', None],
  [ 0xc345, 'HangulSyllableSsyang', [], 'Display', None],
  [ 0xc346, 'HangulSyllableSsyaj', [], 'Display', None],
  [ 0xc347, 'HangulSyllableSsyac', [], 'Display', None],
  [ 0xc348, 'HangulSyllableSsyak', [], 'Display', None],
  [ 0xc349, 'HangulSyllableSsyat', [], 'Display', None],
  [ 0xc34a, 'HangulSyllableSsyap', [], 'Display', None],
  [ 0xc34b, 'HangulSyllableSsyah', [], 'Display', None],
  [ 0xc34c, 'HangulSyllableSsyae', [], 'Display', None],
  [ 0xc34d, 'HangulSyllableSsyaeg', [], 'Display', None],
  [ 0xc34e, 'HangulSyllableSsyaegg', [], 'Display', None],
  [ 0xc34f, 'HangulSyllableSsyaegs', [], 'Display', None],
  [ 0xc350, 'HangulSyllableSsyaen', [], 'Display', None],
  [ 0xc351, 'HangulSyllableSsyaenj', [], 'Display', None],
  [ 0xc352, 'HangulSyllableSsyaenh', [], 'Display', None],
  [ 0xc353, 'HangulSyllableSsyaed', [], 'Display', None],
  [ 0xc354, 'HangulSyllableSsyael', [], 'Display', None],
  [ 0xc355, 'HangulSyllableSsyaelg', [], 'Display', None],
  [ 0xc356, 'HangulSyllableSsyaelm', [], 'Display', None],
  [ 0xc357, 'HangulSyllableSsyaelb', [], 'Display', None],
  [ 0xc358, 'HangulSyllableSsyaels', [], 'Display', None],
  [ 0xc359, 'HangulSyllableSsyaelt', [], 'Display', None],
  [ 0xc35a, 'HangulSyllableSsyaelp', [], 'Display', None],
  [ 0xc35b, 'HangulSyllableSsyaelh', [], 'Display', None],
  [ 0xc35c, 'HangulSyllableSsyaem', [], 'Display', None],
  [ 0xc35d, 'HangulSyllableSsyaeb', [], 'Display', None],
  [ 0xc35e, 'HangulSyllableSsyaebs', [], 'Display', None],
  [ 0xc35f, 'HangulSyllableSsyaes', [], 'Display', None],
  [ 0xc360, 'HangulSyllableSsyaess', [], 'Display', None],
  [ 0xc361, 'HangulSyllableSsyaeng', [], 'Display', None],
  [ 0xc362, 'HangulSyllableSsyaej', [], 'Display', None],
  [ 0xc363, 'HangulSyllableSsyaec', [], 'Display', None],
  [ 0xc364, 'HangulSyllableSsyaek', [], 'Display', None],
  [ 0xc365, 'HangulSyllableSsyaet', [], 'Display', None],
  [ 0xc366, 'HangulSyllableSsyaep', [], 'Display', None],
  [ 0xc367, 'HangulSyllableSsyaeh', [], 'Display', None],
  [ 0xc368, 'HangulSyllableSseo', [], 'Display', None],
  [ 0xc369, 'HangulSyllableSseog', [], 'Display', None],
  [ 0xc36a, 'HangulSyllableSseogg', [], 'Display', None],
  [ 0xc36b, 'HangulSyllableSseogs', [], 'Display', None],
  [ 0xc36c, 'HangulSyllableSseon', [], 'Display', None],
  [ 0xc36d, 'HangulSyllableSseonj', [], 'Display', None],
  [ 0xc36e, 'HangulSyllableSseonh', [], 'Display', None],
  [ 0xc36f, 'HangulSyllableSseod', [], 'Display', None],
  [ 0xc370, 'HangulSyllableSseol', [], 'Display', None],
  [ 0xc371, 'HangulSyllableSseolg', [], 'Display', None],
  [ 0xc372, 'HangulSyllableSseolm', [], 'Display', None],
  [ 0xc373, 'HangulSyllableSseolb', [], 'Display', None],
  [ 0xc374, 'HangulSyllableSseols', [], 'Display', None],
  [ 0xc375, 'HangulSyllableSseolt', [], 'Display', None],
  [ 0xc376, 'HangulSyllableSseolp', [], 'Display', None],
  [ 0xc377, 'HangulSyllableSseolh', [], 'Display', None],
  [ 0xc378, 'HangulSyllableSseom', [], 'Display', None],
  [ 0xc379, 'HangulSyllableSseob', [], 'Display', None],
  [ 0xc37a, 'HangulSyllableSseobs', [], 'Display', None],
  [ 0xc37b, 'HangulSyllableSseos', [], 'Display', None],
  [ 0xc37c, 'HangulSyllableSseoss', [], 'Display', None],
  [ 0xc37d, 'HangulSyllableSseong', [], 'Display', None],
  [ 0xc37e, 'HangulSyllableSseoj', [], 'Display', None],
  [ 0xc37f, 'HangulSyllableSseoc', [], 'Display', None],
  [ 0xc380, 'HangulSyllableSseok', [], 'Display', None],
  [ 0xc381, 'HangulSyllableSseot', [], 'Display', None],
  [ 0xc382, 'HangulSyllableSseop', [], 'Display', None],
  [ 0xc383, 'HangulSyllableSseoh', [], 'Display', None],
  [ 0xc384, 'HangulSyllableSse', [], 'Display', None],
  [ 0xc385, 'HangulSyllableSseg', [], 'Display', None],
  [ 0xc386, 'HangulSyllableSsegg', [], 'Display', None],
  [ 0xc387, 'HangulSyllableSsegs', [], 'Display', None],
  [ 0xc388, 'HangulSyllableSsen', [], 'Display', None],
  [ 0xc389, 'HangulSyllableSsenj', [], 'Display', None],
  [ 0xc38a, 'HangulSyllableSsenh', [], 'Display', None],
  [ 0xc38b, 'HangulSyllableSsed', [], 'Display', None],
  [ 0xc38c, 'HangulSyllableSsel', [], 'Display', None],
  [ 0xc38d, 'HangulSyllableSselg', [], 'Display', None],
  [ 0xc38e, 'HangulSyllableSselm', [], 'Display', None],
  [ 0xc38f, 'HangulSyllableSselb', [], 'Display', None],
  [ 0xc390, 'HangulSyllableSsels', [], 'Display', None],
  [ 0xc391, 'HangulSyllableSselt', [], 'Display', None],
  [ 0xc392, 'HangulSyllableSselp', [], 'Display', None],
  [ 0xc393, 'HangulSyllableSselh', [], 'Display', None],
  [ 0xc394, 'HangulSyllableSsem', [], 'Display', None],
  [ 0xc395, 'HangulSyllableSseb', [], 'Display', None],
  [ 0xc396, 'HangulSyllableSsebs', [], 'Display', None],
  [ 0xc397, 'HangulSyllableSses', [], 'Display', None],
  [ 0xc398, 'HangulSyllableSsess', [], 'Display', None],
  [ 0xc399, 'HangulSyllableSseng', [], 'Display', None],
  [ 0xc39a, 'HangulSyllableSsej', [], 'Display', None],
  [ 0xc39b, 'HangulSyllableSsec', [], 'Display', None],
  [ 0xc39c, 'HangulSyllableSsek', [], 'Display', None],
  [ 0xc39d, 'HangulSyllableSset', [], 'Display', None],
  [ 0xc39e, 'HangulSyllableSsep', [], 'Display', None],
  [ 0xc39f, 'HangulSyllableSseh', [], 'Display', None],
  [ 0xc3a0, 'HangulSyllableSsyeo', [], 'Display', None],
  [ 0xc3a1, 'HangulSyllableSsyeog', [], 'Display', None],
  [ 0xc3a2, 'HangulSyllableSsyeogg', [], 'Display', None],
  [ 0xc3a3, 'HangulSyllableSsyeogs', [], 'Display', None],
  [ 0xc3a4, 'HangulSyllableSsyeon', [], 'Display', None],
  [ 0xc3a5, 'HangulSyllableSsyeonj', [], 'Display', None],
  [ 0xc3a6, 'HangulSyllableSsyeonh', [], 'Display', None],
  [ 0xc3a7, 'HangulSyllableSsyeod', [], 'Display', None],
  [ 0xc3a8, 'HangulSyllableSsyeol', [], 'Display', None],
  [ 0xc3a9, 'HangulSyllableSsyeolg', [], 'Display', None],
  [ 0xc3aa, 'HangulSyllableSsyeolm', [], 'Display', None],
  [ 0xc3ab, 'HangulSyllableSsyeolb', [], 'Display', None],
  [ 0xc3ac, 'HangulSyllableSsyeols', [], 'Display', None],
  [ 0xc3ad, 'HangulSyllableSsyeolt', [], 'Display', None],
  [ 0xc3ae, 'HangulSyllableSsyeolp', [], 'Display', None],
  [ 0xc3af, 'HangulSyllableSsyeolh', [], 'Display', None],
  [ 0xc3b0, 'HangulSyllableSsyeom', [], 'Display', None],
  [ 0xc3b1, 'HangulSyllableSsyeob', [], 'Display', None],
  [ 0xc3b2, 'HangulSyllableSsyeobs', [], 'Display', None],
  [ 0xc3b3, 'HangulSyllableSsyeos', [], 'Display', None],
  [ 0xc3b4, 'HangulSyllableSsyeoss', [], 'Display', None],
  [ 0xc3b5, 'HangulSyllableSsyeong', [], 'Display', None],
  [ 0xc3b6, 'HangulSyllableSsyeoj', [], 'Display', None],
  [ 0xc3b7, 'HangulSyllableSsyeoc', [], 'Display', None],
  [ 0xc3b8, 'HangulSyllableSsyeok', [], 'Display', None],
  [ 0xc3b9, 'HangulSyllableSsyeot', [], 'Display', None],
  [ 0xc3ba, 'HangulSyllableSsyeop', [], 'Display', None],
  [ 0xc3bb, 'HangulSyllableSsyeoh', [], 'Display', None],
  [ 0xc3bc, 'HangulSyllableSsye', [], 'Display', None],
  [ 0xc3bd, 'HangulSyllableSsyeg', [], 'Display', None],
  [ 0xc3be, 'HangulSyllableSsyegg', [], 'Display', None],
  [ 0xc3bf, 'HangulSyllableSsyegs', [], 'Display', None],
  [ 0xc3c0, 'HangulSyllableSsyen', [], 'Display', None],
  [ 0xc3c1, 'HangulSyllableSsyenj', [], 'Display', None],
  [ 0xc3c2, 'HangulSyllableSsyenh', [], 'Display', None],
  [ 0xc3c3, 'HangulSyllableSsyed', [], 'Display', None],
  [ 0xc3c4, 'HangulSyllableSsyel', [], 'Display', None],
  [ 0xc3c5, 'HangulSyllableSsyelg', [], 'Display', None],
  [ 0xc3c6, 'HangulSyllableSsyelm', [], 'Display', None],
  [ 0xc3c7, 'HangulSyllableSsyelb', [], 'Display', None],
  [ 0xc3c8, 'HangulSyllableSsyels', [], 'Display', None],
  [ 0xc3c9, 'HangulSyllableSsyelt', [], 'Display', None],
  [ 0xc3ca, 'HangulSyllableSsyelp', [], 'Display', None],
  [ 0xc3cb, 'HangulSyllableSsyelh', [], 'Display', None],
  [ 0xc3cc, 'HangulSyllableSsyem', [], 'Display', None],
  [ 0xc3cd, 'HangulSyllableSsyeb', [], 'Display', None],
  [ 0xc3ce, 'HangulSyllableSsyebs', [], 'Display', None],
  [ 0xc3cf, 'HangulSyllableSsyes', [], 'Display', None],
  [ 0xc3d0, 'HangulSyllableSsyess', [], 'Display', None],
  [ 0xc3d1, 'HangulSyllableSsyeng', [], 'Display', None],
  [ 0xc3d2, 'HangulSyllableSsyej', [], 'Display', None],
  [ 0xc3d3, 'HangulSyllableSsyec', [], 'Display', None],
  [ 0xc3d4, 'HangulSyllableSsyek', [], 'Display', None],
  [ 0xc3d5, 'HangulSyllableSsyet', [], 'Display', None],
  [ 0xc3d6, 'HangulSyllableSsyep', [], 'Display', None],
  [ 0xc3d7, 'HangulSyllableSsyeh', [], 'Display', None],
  [ 0xc3d8, 'HangulSyllableSso', [], 'Display', None],
  [ 0xc3d9, 'HangulSyllableSsog', [], 'Display', None],
  [ 0xc3da, 'HangulSyllableSsogg', [], 'Display', None],
  [ 0xc3db, 'HangulSyllableSsogs', [], 'Display', None],
  [ 0xc3dc, 'HangulSyllableSson', [], 'Display', None],
  [ 0xc3dd, 'HangulSyllableSsonj', [], 'Display', None],
  [ 0xc3de, 'HangulSyllableSsonh', [], 'Display', None],
  [ 0xc3df, 'HangulSyllableSsod', [], 'Display', None],
  [ 0xc3e0, 'HangulSyllableSsol', [], 'Display', None],
  [ 0xc3e1, 'HangulSyllableSsolg', [], 'Display', None],
  [ 0xc3e2, 'HangulSyllableSsolm', [], 'Display', None],
  [ 0xc3e3, 'HangulSyllableSsolb', [], 'Display', None],
  [ 0xc3e4, 'HangulSyllableSsols', [], 'Display', None],
  [ 0xc3e5, 'HangulSyllableSsolt', [], 'Display', None],
  [ 0xc3e6, 'HangulSyllableSsolp', [], 'Display', None],
  [ 0xc3e7, 'HangulSyllableSsolh', [], 'Display', None],
  [ 0xc3e8, 'HangulSyllableSsom', [], 'Display', None],
  [ 0xc3e9, 'HangulSyllableSsob', [], 'Display', None],
  [ 0xc3ea, 'HangulSyllableSsobs', [], 'Display', None],
  [ 0xc3eb, 'HangulSyllableSsos', [], 'Display', None],
  [ 0xc3ec, 'HangulSyllableSsoss', [], 'Display', None],
  [ 0xc3ed, 'HangulSyllableSsong', [], 'Display', None],
  [ 0xc3ee, 'HangulSyllableSsoj', [], 'Display', None],
  [ 0xc3ef, 'HangulSyllableSsoc', [], 'Display', None],
  [ 0xc3f0, 'HangulSyllableSsok', [], 'Display', None],
  [ 0xc3f1, 'HangulSyllableSsot', [], 'Display', None],
  [ 0xc3f2, 'HangulSyllableSsop', [], 'Display', None],
  [ 0xc3f3, 'HangulSyllableSsoh', [], 'Display', None],
  [ 0xc3f4, 'HangulSyllableSswa', [], 'Display', None],
  [ 0xc3f5, 'HangulSyllableSswag', [], 'Display', None],
  [ 0xc3f6, 'HangulSyllableSswagg', [], 'Display', None],
  [ 0xc3f7, 'HangulSyllableSswags', [], 'Display', None],
  [ 0xc3f8, 'HangulSyllableSswan', [], 'Display', None],
  [ 0xc3f9, 'HangulSyllableSswanj', [], 'Display', None],
  [ 0xc3fa, 'HangulSyllableSswanh', [], 'Display', None],
  [ 0xc3fb, 'HangulSyllableSswad', [], 'Display', None],
  [ 0xc3fc, 'HangulSyllableSswal', [], 'Display', None],
  [ 0xc3fd, 'HangulSyllableSswalg', [], 'Display', None],
  [ 0xc3fe, 'HangulSyllableSswalm', [], 'Display', None],
  [ 0xc3ff, 'HangulSyllableSswalb', [], 'Display', None],
  [ 0xc400, 'HangulSyllableSswals', [], 'Display', None],
  [ 0xc401, 'HangulSyllableSswalt', [], 'Display', None],
  [ 0xc402, 'HangulSyllableSswalp', [], 'Display', None],
  [ 0xc403, 'HangulSyllableSswalh', [], 'Display', None],
  [ 0xc404, 'HangulSyllableSswam', [], 'Display', None],
  [ 0xc405, 'HangulSyllableSswab', [], 'Display', None],
  [ 0xc406, 'HangulSyllableSswabs', [], 'Display', None],
  [ 0xc407, 'HangulSyllableSswas', [], 'Display', None],
  [ 0xc408, 'HangulSyllableSswass', [], 'Display', None],
  [ 0xc409, 'HangulSyllableSswang', [], 'Display', None],
  [ 0xc40a, 'HangulSyllableSswaj', [], 'Display', None],
  [ 0xc40b, 'HangulSyllableSswac', [], 'Display', None],
  [ 0xc40c, 'HangulSyllableSswak', [], 'Display', None],
  [ 0xc40d, 'HangulSyllableSswat', [], 'Display', None],
  [ 0xc40e, 'HangulSyllableSswap', [], 'Display', None],
  [ 0xc40f, 'HangulSyllableSswah', [], 'Display', None],
  [ 0xc410, 'HangulSyllableSswae', [], 'Display', None],
  [ 0xc411, 'HangulSyllableSswaeg', [], 'Display', None],
  [ 0xc412, 'HangulSyllableSswaegg', [], 'Display', None],
  [ 0xc413, 'HangulSyllableSswaegs', [], 'Display', None],
  [ 0xc414, 'HangulSyllableSswaen', [], 'Display', None],
  [ 0xc415, 'HangulSyllableSswaenj', [], 'Display', None],
  [ 0xc416, 'HangulSyllableSswaenh', [], 'Display', None],
  [ 0xc417, 'HangulSyllableSswaed', [], 'Display', None],
  [ 0xc418, 'HangulSyllableSswael', [], 'Display', None],
  [ 0xc419, 'HangulSyllableSswaelg', [], 'Display', None],
  [ 0xc41a, 'HangulSyllableSswaelm', [], 'Display', None],
  [ 0xc41b, 'HangulSyllableSswaelb', [], 'Display', None],
  [ 0xc41c, 'HangulSyllableSswaels', [], 'Display', None],
  [ 0xc41d, 'HangulSyllableSswaelt', [], 'Display', None],
  [ 0xc41e, 'HangulSyllableSswaelp', [], 'Display', None],
  [ 0xc41f, 'HangulSyllableSswaelh', [], 'Display', None],
  [ 0xc420, 'HangulSyllableSswaem', [], 'Display', None],
  [ 0xc421, 'HangulSyllableSswaeb', [], 'Display', None],
  [ 0xc422, 'HangulSyllableSswaebs', [], 'Display', None],
  [ 0xc423, 'HangulSyllableSswaes', [], 'Display', None],
  [ 0xc424, 'HangulSyllableSswaess', [], 'Display', None],
  [ 0xc425, 'HangulSyllableSswaeng', [], 'Display', None],
  [ 0xc426, 'HangulSyllableSswaej', [], 'Display', None],
  [ 0xc427, 'HangulSyllableSswaec', [], 'Display', None],
  [ 0xc428, 'HangulSyllableSswaek', [], 'Display', None],
  [ 0xc429, 'HangulSyllableSswaet', [], 'Display', None],
  [ 0xc42a, 'HangulSyllableSswaep', [], 'Display', None],
  [ 0xc42b, 'HangulSyllableSswaeh', [], 'Display', None],
  [ 0xc42c, 'HangulSyllableSsoe', [], 'Display', None],
  [ 0xc42d, 'HangulSyllableSsoeg', [], 'Display', None],
  [ 0xc42e, 'HangulSyllableSsoegg', [], 'Display', None],
  [ 0xc42f, 'HangulSyllableSsoegs', [], 'Display', None],
  [ 0xc430, 'HangulSyllableSsoen', [], 'Display', None],
  [ 0xc431, 'HangulSyllableSsoenj', [], 'Display', None],
  [ 0xc432, 'HangulSyllableSsoenh', [], 'Display', None],
  [ 0xc433, 'HangulSyllableSsoed', [], 'Display', None],
  [ 0xc434, 'HangulSyllableSsoel', [], 'Display', None],
  [ 0xc435, 'HangulSyllableSsoelg', [], 'Display', None],
  [ 0xc436, 'HangulSyllableSsoelm', [], 'Display', None],
  [ 0xc437, 'HangulSyllableSsoelb', [], 'Display', None],
  [ 0xc438, 'HangulSyllableSsoels', [], 'Display', None],
  [ 0xc439, 'HangulSyllableSsoelt', [], 'Display', None],
  [ 0xc43a, 'HangulSyllableSsoelp', [], 'Display', None],
  [ 0xc43b, 'HangulSyllableSsoelh', [], 'Display', None],
  [ 0xc43c, 'HangulSyllableSsoem', [], 'Display', None],
  [ 0xc43d, 'HangulSyllableSsoeb', [], 'Display', None],
  [ 0xc43e, 'HangulSyllableSsoebs', [], 'Display', None],
  [ 0xc43f, 'HangulSyllableSsoes', [], 'Display', None],
  [ 0xc440, 'HangulSyllableSsoess', [], 'Display', None],
  [ 0xc441, 'HangulSyllableSsoeng', [], 'Display', None],
  [ 0xc442, 'HangulSyllableSsoej', [], 'Display', None],
  [ 0xc443, 'HangulSyllableSsoec', [], 'Display', None],
  [ 0xc444, 'HangulSyllableSsoek', [], 'Display', None],
  [ 0xc445, 'HangulSyllableSsoet', [], 'Display', None],
  [ 0xc446, 'HangulSyllableSsoep', [], 'Display', None],
  [ 0xc447, 'HangulSyllableSsoeh', [], 'Display', None],
  [ 0xc448, 'HangulSyllableSsyo', [], 'Display', None],
  [ 0xc449, 'HangulSyllableSsyog', [], 'Display', None],
  [ 0xc44a, 'HangulSyllableSsyogg', [], 'Display', None],
  [ 0xc44b, 'HangulSyllableSsyogs', [], 'Display', None],
  [ 0xc44c, 'HangulSyllableSsyon', [], 'Display', None],
  [ 0xc44d, 'HangulSyllableSsyonj', [], 'Display', None],
  [ 0xc44e, 'HangulSyllableSsyonh', [], 'Display', None],
  [ 0xc44f, 'HangulSyllableSsyod', [], 'Display', None],
  [ 0xc450, 'HangulSyllableSsyol', [], 'Display', None],
  [ 0xc451, 'HangulSyllableSsyolg', [], 'Display', None],
  [ 0xc452, 'HangulSyllableSsyolm', [], 'Display', None],
  [ 0xc453, 'HangulSyllableSsyolb', [], 'Display', None],
  [ 0xc454, 'HangulSyllableSsyols', [], 'Display', None],
  [ 0xc455, 'HangulSyllableSsyolt', [], 'Display', None],
  [ 0xc456, 'HangulSyllableSsyolp', [], 'Display', None],
  [ 0xc457, 'HangulSyllableSsyolh', [], 'Display', None],
  [ 0xc458, 'HangulSyllableSsyom', [], 'Display', None],
  [ 0xc459, 'HangulSyllableSsyob', [], 'Display', None],
  [ 0xc45a, 'HangulSyllableSsyobs', [], 'Display', None],
  [ 0xc45b, 'HangulSyllableSsyos', [], 'Display', None],
  [ 0xc45c, 'HangulSyllableSsyoss', [], 'Display', None],
  [ 0xc45d, 'HangulSyllableSsyong', [], 'Display', None],
  [ 0xc45e, 'HangulSyllableSsyoj', [], 'Display', None],
  [ 0xc45f, 'HangulSyllableSsyoc', [], 'Display', None],
  [ 0xc460, 'HangulSyllableSsyok', [], 'Display', None],
  [ 0xc461, 'HangulSyllableSsyot', [], 'Display', None],
  [ 0xc462, 'HangulSyllableSsyop', [], 'Display', None],
  [ 0xc463, 'HangulSyllableSsyoh', [], 'Display', None],
  [ 0xc464, 'HangulSyllableSsu', [], 'Display', None],
  [ 0xc465, 'HangulSyllableSsug', [], 'Display', None],
  [ 0xc466, 'HangulSyllableSsugg', [], 'Display', None],
  [ 0xc467, 'HangulSyllableSsugs', [], 'Display', None],
  [ 0xc468, 'HangulSyllableSsun', [], 'Display', None],
  [ 0xc469, 'HangulSyllableSsunj', [], 'Display', None],
  [ 0xc46a, 'HangulSyllableSsunh', [], 'Display', None],
  [ 0xc46b, 'HangulSyllableSsud', [], 'Display', None],
  [ 0xc46c, 'HangulSyllableSsul', [], 'Display', None],
  [ 0xc46d, 'HangulSyllableSsulg', [], 'Display', None],
  [ 0xc46e, 'HangulSyllableSsulm', [], 'Display', None],
  [ 0xc46f, 'HangulSyllableSsulb', [], 'Display', None],
  [ 0xc470, 'HangulSyllableSsuls', [], 'Display', None],
  [ 0xc471, 'HangulSyllableSsult', [], 'Display', None],
  [ 0xc472, 'HangulSyllableSsulp', [], 'Display', None],
  [ 0xc473, 'HangulSyllableSsulh', [], 'Display', None],
  [ 0xc474, 'HangulSyllableSsum', [], 'Display', None],
  [ 0xc475, 'HangulSyllableSsub', [], 'Display', None],
  [ 0xc476, 'HangulSyllableSsubs', [], 'Display', None],
  [ 0xc477, 'HangulSyllableSsus', [], 'Display', None],
  [ 0xc478, 'HangulSyllableSsuss', [], 'Display', None],
  [ 0xc479, 'HangulSyllableSsung', [], 'Display', None],
  [ 0xc47a, 'HangulSyllableSsuj', [], 'Display', None],
  [ 0xc47b, 'HangulSyllableSsuc', [], 'Display', None],
  [ 0xc47c, 'HangulSyllableSsuk', [], 'Display', None],
  [ 0xc47d, 'HangulSyllableSsut', [], 'Display', None],
  [ 0xc47e, 'HangulSyllableSsup', [], 'Display', None],
  [ 0xc47f, 'HangulSyllableSsuh', [], 'Display', None],
  [ 0xc480, 'HangulSyllableSsweo', [], 'Display', None],
  [ 0xc481, 'HangulSyllableSsweog', [], 'Display', None],
  [ 0xc482, 'HangulSyllableSsweogg', [], 'Display', None],
  [ 0xc483, 'HangulSyllableSsweogs', [], 'Display', None],
  [ 0xc484, 'HangulSyllableSsweon', [], 'Display', None],
  [ 0xc485, 'HangulSyllableSsweonj', [], 'Display', None],
  [ 0xc486, 'HangulSyllableSsweonh', [], 'Display', None],
  [ 0xc487, 'HangulSyllableSsweod', [], 'Display', None],
  [ 0xc488, 'HangulSyllableSsweol', [], 'Display', None],
  [ 0xc489, 'HangulSyllableSsweolg', [], 'Display', None],
  [ 0xc48a, 'HangulSyllableSsweolm', [], 'Display', None],
  [ 0xc48b, 'HangulSyllableSsweolb', [], 'Display', None],
  [ 0xc48c, 'HangulSyllableSsweols', [], 'Display', None],
  [ 0xc48d, 'HangulSyllableSsweolt', [], 'Display', None],
  [ 0xc48e, 'HangulSyllableSsweolp', [], 'Display', None],
  [ 0xc48f, 'HangulSyllableSsweolh', [], 'Display', None],
  [ 0xc490, 'HangulSyllableSsweom', [], 'Display', None],
  [ 0xc491, 'HangulSyllableSsweob', [], 'Display', None],
  [ 0xc492, 'HangulSyllableSsweobs', [], 'Display', None],
  [ 0xc493, 'HangulSyllableSsweos', [], 'Display', None],
  [ 0xc494, 'HangulSyllableSsweoss', [], 'Display', None],
  [ 0xc495, 'HangulSyllableSsweong', [], 'Display', None],
  [ 0xc496, 'HangulSyllableSsweoj', [], 'Display', None],
  [ 0xc497, 'HangulSyllableSsweoc', [], 'Display', None],
  [ 0xc498, 'HangulSyllableSsweok', [], 'Display', None],
  [ 0xc499, 'HangulSyllableSsweot', [], 'Display', None],
  [ 0xc49a, 'HangulSyllableSsweop', [], 'Display', None],
  [ 0xc49b, 'HangulSyllableSsweoh', [], 'Display', None],
  [ 0xc49c, 'HangulSyllableSswe', [], 'Display', None],
  [ 0xc49d, 'HangulSyllableSsweg', [], 'Display', None],
  [ 0xc49e, 'HangulSyllableSswegg', [], 'Display', None],
  [ 0xc49f, 'HangulSyllableSswegs', [], 'Display', None],
  [ 0xc4a0, 'HangulSyllableSswen', [], 'Display', None],
  [ 0xc4a1, 'HangulSyllableSswenj', [], 'Display', None],
  [ 0xc4a2, 'HangulSyllableSswenh', [], 'Display', None],
  [ 0xc4a3, 'HangulSyllableSswed', [], 'Display', None],
  [ 0xc4a4, 'HangulSyllableSswel', [], 'Display', None],
  [ 0xc4a5, 'HangulSyllableSswelg', [], 'Display', None],
  [ 0xc4a6, 'HangulSyllableSswelm', [], 'Display', None],
  [ 0xc4a7, 'HangulSyllableSswelb', [], 'Display', None],
  [ 0xc4a8, 'HangulSyllableSswels', [], 'Display', None],
  [ 0xc4a9, 'HangulSyllableSswelt', [], 'Display', None],
  [ 0xc4aa, 'HangulSyllableSswelp', [], 'Display', None],
  [ 0xc4ab, 'HangulSyllableSswelh', [], 'Display', None],
  [ 0xc4ac, 'HangulSyllableSswem', [], 'Display', None],
  [ 0xc4ad, 'HangulSyllableSsweb', [], 'Display', None],
  [ 0xc4ae, 'HangulSyllableSswebs', [], 'Display', None],
  [ 0xc4af, 'HangulSyllableSswes', [], 'Display', None],
  [ 0xc4b0, 'HangulSyllableSswess', [], 'Display', None],
  [ 0xc4b1, 'HangulSyllableSsweng', [], 'Display', None],
  [ 0xc4b2, 'HangulSyllableSswej', [], 'Display', None],
  [ 0xc4b3, 'HangulSyllableSswec', [], 'Display', None],
  [ 0xc4b4, 'HangulSyllableSswek', [], 'Display', None],
  [ 0xc4b5, 'HangulSyllableSswet', [], 'Display', None],
  [ 0xc4b6, 'HangulSyllableSswep', [], 'Display', None],
  [ 0xc4b7, 'HangulSyllableSsweh', [], 'Display', None],
  [ 0xc4b8, 'HangulSyllableSswi', [], 'Display', None],
  [ 0xc4b9, 'HangulSyllableSswig', [], 'Display', None],
  [ 0xc4ba, 'HangulSyllableSswigg', [], 'Display', None],
  [ 0xc4bb, 'HangulSyllableSswigs', [], 'Display', None],
  [ 0xc4bc, 'HangulSyllableSswin', [], 'Display', None],
  [ 0xc4bd, 'HangulSyllableSswinj', [], 'Display', None],
  [ 0xc4be, 'HangulSyllableSswinh', [], 'Display', None],
  [ 0xc4bf, 'HangulSyllableSswid', [], 'Display', None],
  [ 0xc4c0, 'HangulSyllableSswil', [], 'Display', None],
  [ 0xc4c1, 'HangulSyllableSswilg', [], 'Display', None],
  [ 0xc4c2, 'HangulSyllableSswilm', [], 'Display', None],
  [ 0xc4c3, 'HangulSyllableSswilb', [], 'Display', None],
  [ 0xc4c4, 'HangulSyllableSswils', [], 'Display', None],
  [ 0xc4c5, 'HangulSyllableSswilt', [], 'Display', None],
  [ 0xc4c6, 'HangulSyllableSswilp', [], 'Display', None],
  [ 0xc4c7, 'HangulSyllableSswilh', [], 'Display', None],
  [ 0xc4c8, 'HangulSyllableSswim', [], 'Display', None],
  [ 0xc4c9, 'HangulSyllableSswib', [], 'Display', None],
  [ 0xc4ca, 'HangulSyllableSswibs', [], 'Display', None],
  [ 0xc4cb, 'HangulSyllableSswis', [], 'Display', None],
  [ 0xc4cc, 'HangulSyllableSswiss', [], 'Display', None],
  [ 0xc4cd, 'HangulSyllableSswing', [], 'Display', None],
  [ 0xc4ce, 'HangulSyllableSswij', [], 'Display', None],
  [ 0xc4cf, 'HangulSyllableSswic', [], 'Display', None],
  [ 0xc4d0, 'HangulSyllableSswik', [], 'Display', None],
  [ 0xc4d1, 'HangulSyllableSswit', [], 'Display', None],
  [ 0xc4d2, 'HangulSyllableSswip', [], 'Display', None],
  [ 0xc4d3, 'HangulSyllableSswih', [], 'Display', None],
  [ 0xc4d4, 'HangulSyllableSsyu', [], 'Display', None],
  [ 0xc4d5, 'HangulSyllableSsyug', [], 'Display', None],
  [ 0xc4d6, 'HangulSyllableSsyugg', [], 'Display', None],
  [ 0xc4d7, 'HangulSyllableSsyugs', [], 'Display', None],
  [ 0xc4d8, 'HangulSyllableSsyun', [], 'Display', None],
  [ 0xc4d9, 'HangulSyllableSsyunj', [], 'Display', None],
  [ 0xc4da, 'HangulSyllableSsyunh', [], 'Display', None],
  [ 0xc4db, 'HangulSyllableSsyud', [], 'Display', None],
  [ 0xc4dc, 'HangulSyllableSsyul', [], 'Display', None],
  [ 0xc4dd, 'HangulSyllableSsyulg', [], 'Display', None],
  [ 0xc4de, 'HangulSyllableSsyulm', [], 'Display', None],
  [ 0xc4df, 'HangulSyllableSsyulb', [], 'Display', None],
  [ 0xc4e0, 'HangulSyllableSsyuls', [], 'Display', None],
  [ 0xc4e1, 'HangulSyllableSsyult', [], 'Display', None],
  [ 0xc4e2, 'HangulSyllableSsyulp', [], 'Display', None],
  [ 0xc4e3, 'HangulSyllableSsyulh', [], 'Display', None],
  [ 0xc4e4, 'HangulSyllableSsyum', [], 'Display', None],
  [ 0xc4e5, 'HangulSyllableSsyub', [], 'Display', None],
  [ 0xc4e6, 'HangulSyllableSsyubs', [], 'Display', None],
  [ 0xc4e7, 'HangulSyllableSsyus', [], 'Display', None],
  [ 0xc4e8, 'HangulSyllableSsyuss', [], 'Display', None],
  [ 0xc4e9, 'HangulSyllableSsyung', [], 'Display', None],
  [ 0xc4ea, 'HangulSyllableSsyuj', [], 'Display', None],
  [ 0xc4eb, 'HangulSyllableSsyuc', [], 'Display', None],
  [ 0xc4ec, 'HangulSyllableSsyuk', [], 'Display', None],
  [ 0xc4ed, 'HangulSyllableSsyut', [], 'Display', None],
  [ 0xc4ee, 'HangulSyllableSsyup', [], 'Display', None],
  [ 0xc4ef, 'HangulSyllableSsyuh', [], 'Display', None],
  [ 0xc4f0, 'HangulSyllableSseu', [], 'Display', None],
  [ 0xc4f1, 'HangulSyllableSseug', [], 'Display', None],
  [ 0xc4f2, 'HangulSyllableSseugg', [], 'Display', None],
  [ 0xc4f3, 'HangulSyllableSseugs', [], 'Display', None],
  [ 0xc4f4, 'HangulSyllableSseun', [], 'Display', None],
  [ 0xc4f5, 'HangulSyllableSseunj', [], 'Display', None],
  [ 0xc4f6, 'HangulSyllableSseunh', [], 'Display', None],
  [ 0xc4f7, 'HangulSyllableSseud', [], 'Display', None],
  [ 0xc4f8, 'HangulSyllableSseul', [], 'Display', None],
  [ 0xc4f9, 'HangulSyllableSseulg', [], 'Display', None],
  [ 0xc4fa, 'HangulSyllableSseulm', [], 'Display', None],
  [ 0xc4fb, 'HangulSyllableSseulb', [], 'Display', None],
  [ 0xc4fc, 'HangulSyllableSseuls', [], 'Display', None],
  [ 0xc4fd, 'HangulSyllableSseult', [], 'Display', None],
  [ 0xc4fe, 'HangulSyllableSseulp', [], 'Display', None],
  [ 0xc4ff, 'HangulSyllableSseulh', [], 'Display', None],
  [ 0xc500, 'HangulSyllableSseum', [], 'Display', None],
  [ 0xc501, 'HangulSyllableSseub', [], 'Display', None],
  [ 0xc502, 'HangulSyllableSseubs', [], 'Display', None],
  [ 0xc503, 'HangulSyllableSseus', [], 'Display', None],
  [ 0xc504, 'HangulSyllableSseuss', [], 'Display', None],
  [ 0xc505, 'HangulSyllableSseung', [], 'Display', None],
  [ 0xc506, 'HangulSyllableSseuj', [], 'Display', None],
  [ 0xc507, 'HangulSyllableSseuc', [], 'Display', None],
  [ 0xc508, 'HangulSyllableSseuk', [], 'Display', None],
  [ 0xc509, 'HangulSyllableSseut', [], 'Display', None],
  [ 0xc50a, 'HangulSyllableSseup', [], 'Display', None],
  [ 0xc50b, 'HangulSyllableSseuh', [], 'Display', None],
  [ 0xc50c, 'HangulSyllableSsyi', [], 'Display', None],
  [ 0xc50d, 'HangulSyllableSsyig', [], 'Display', None],
  [ 0xc50e, 'HangulSyllableSsyigg', [], 'Display', None],
  [ 0xc50f, 'HangulSyllableSsyigs', [], 'Display', None],
  [ 0xc510, 'HangulSyllableSsyin', [], 'Display', None],
  [ 0xc511, 'HangulSyllableSsyinj', [], 'Display', None],
  [ 0xc512, 'HangulSyllableSsyinh', [], 'Display', None],
  [ 0xc513, 'HangulSyllableSsyid', [], 'Display', None],
  [ 0xc514, 'HangulSyllableSsyil', [], 'Display', None],
  [ 0xc515, 'HangulSyllableSsyilg', [], 'Display', None],
  [ 0xc516, 'HangulSyllableSsyilm', [], 'Display', None],
  [ 0xc517, 'HangulSyllableSsyilb', [], 'Display', None],
  [ 0xc518, 'HangulSyllableSsyils', [], 'Display', None],
  [ 0xc519, 'HangulSyllableSsyilt', [], 'Display', None],
  [ 0xc51a, 'HangulSyllableSsyilp', [], 'Display', None],
  [ 0xc51b, 'HangulSyllableSsyilh', [], 'Display', None],
  [ 0xc51c, 'HangulSyllableSsyim', [], 'Display', None],
  [ 0xc51d, 'HangulSyllableSsyib', [], 'Display', None],
  [ 0xc51e, 'HangulSyllableSsyibs', [], 'Display', None],
  [ 0xc51f, 'HangulSyllableSsyis', [], 'Display', None],
  [ 0xc520, 'HangulSyllableSsyiss', [], 'Display', None],
  [ 0xc521, 'HangulSyllableSsying', [], 'Display', None],
  [ 0xc522, 'HangulSyllableSsyij', [], 'Display', None],
  [ 0xc523, 'HangulSyllableSsyic', [], 'Display', None],
  [ 0xc524, 'HangulSyllableSsyik', [], 'Display', None],
  [ 0xc525, 'HangulSyllableSsyit', [], 'Display', None],
  [ 0xc526, 'HangulSyllableSsyip', [], 'Display', None],
  [ 0xc527, 'HangulSyllableSsyih', [], 'Display', None],
  [ 0xc528, 'HangulSyllableSsi', [], 'Display', None],
  [ 0xc529, 'HangulSyllableSsig', [], 'Display', None],
  [ 0xc52a, 'HangulSyllableSsigg', [], 'Display', None],
  [ 0xc52b, 'HangulSyllableSsigs', [], 'Display', None],
  [ 0xc52c, 'HangulSyllableSsin', [], 'Display', None],
  [ 0xc52d, 'HangulSyllableSsinj', [], 'Display', None],
  [ 0xc52e, 'HangulSyllableSsinh', [], 'Display', None],
  [ 0xc52f, 'HangulSyllableSsid', [], 'Display', None],
  [ 0xc530, 'HangulSyllableSsil', [], 'Display', None],
  [ 0xc531, 'HangulSyllableSsilg', [], 'Display', None],
  [ 0xc532, 'HangulSyllableSsilm', [], 'Display', None],
  [ 0xc533, 'HangulSyllableSsilb', [], 'Display', None],
  [ 0xc534, 'HangulSyllableSsils', [], 'Display', None],
  [ 0xc535, 'HangulSyllableSsilt', [], 'Display', None],
  [ 0xc536, 'HangulSyllableSsilp', [], 'Display', None],
  [ 0xc537, 'HangulSyllableSsilh', [], 'Display', None],
  [ 0xc538, 'HangulSyllableSsim', [], 'Display', None],
  [ 0xc539, 'HangulSyllableSsib', [], 'Display', None],
  [ 0xc53a, 'HangulSyllableSsibs', [], 'Display', None],
  [ 0xc53b, 'HangulSyllableSsis', [], 'Display', None],
  [ 0xc53c, 'HangulSyllableSsiss', [], 'Display', None],
  [ 0xc53d, 'HangulSyllableSsing', [], 'Display', None],
  [ 0xc53e, 'HangulSyllableSsij', [], 'Display', None],
  [ 0xc53f, 'HangulSyllableSsic', [], 'Display', None],
  [ 0xc540, 'HangulSyllableSsik', [], 'Display', None],
  [ 0xc541, 'HangulSyllableSsit', [], 'Display', None],
  [ 0xc542, 'HangulSyllableSsip', [], 'Display', None],
  [ 0xc543, 'HangulSyllableSsih', [], 'Display', None],
  [ 0xc544, 'HangulSyllableA', [], 'Display', None],
  [ 0xc545, 'HangulSyllableAg', [], 'Display', None],
  [ 0xc546, 'HangulSyllableAgg', [], 'Display', None],
  [ 0xc547, 'HangulSyllableAgs', [], 'Display', None],
  [ 0xc548, 'HangulSyllableAn', [], 'Display', None],
  [ 0xc549, 'HangulSyllableAnj', [], 'Display', None],
  [ 0xc54a, 'HangulSyllableAnh', [], 'Display', None],
  [ 0xc54b, 'HangulSyllableAd', [], 'Display', None],
  [ 0xc54c, 'HangulSyllableAl', [], 'Display', None],
  [ 0xc54d, 'HangulSyllableAlg', [], 'Display', None],
  [ 0xc54e, 'HangulSyllableAlm', [], 'Display', None],
  [ 0xc54f, 'HangulSyllableAlb', [], 'Display', None],
  [ 0xc550, 'HangulSyllableAls', [], 'Display', None],
  [ 0xc551, 'HangulSyllableAlt', [], 'Display', None],
  [ 0xc552, 'HangulSyllableAlp', [], 'Display', None],
  [ 0xc553, 'HangulSyllableAlh', [], 'Display', None],
  [ 0xc554, 'HangulSyllableAm', [], 'Display', None],
  [ 0xc555, 'HangulSyllableAb', [], 'Display', None],
  [ 0xc556, 'HangulSyllableAbs', [], 'Display', None],
  [ 0xc557, 'HangulSyllableAs', [], 'Display', None],
  [ 0xc558, 'HangulSyllableAss', [], 'Display', None],
  [ 0xc559, 'HangulSyllableAng', [], 'Display', None],
  [ 0xc55a, 'HangulSyllableAj', [], 'Display', None],
  [ 0xc55b, 'HangulSyllableAc', [], 'Display', None],
  [ 0xc55c, 'HangulSyllableAk', [], 'Display', None],
  [ 0xc55d, 'HangulSyllableAt', [], 'Display', None],
  [ 0xc55e, 'HangulSyllableAp', [], 'Display', None],
  [ 0xc55f, 'HangulSyllableAh', [], 'Display', None],
  [ 0xc560, 'HangulSyllableAe', [], 'Display', None],
  [ 0xc561, 'HangulSyllableAeg', [], 'Display', None],
  [ 0xc562, 'HangulSyllableAegg', [], 'Display', None],
  [ 0xc563, 'HangulSyllableAegs', [], 'Display', None],
  [ 0xc564, 'HangulSyllableAen', [], 'Display', None],
  [ 0xc565, 'HangulSyllableAenj', [], 'Display', None],
  [ 0xc566, 'HangulSyllableAenh', [], 'Display', None],
  [ 0xc567, 'HangulSyllableAed', [], 'Display', None],
  [ 0xc568, 'HangulSyllableAel', [], 'Display', None],
  [ 0xc569, 'HangulSyllableAelg', [], 'Display', None],
  [ 0xc56a, 'HangulSyllableAelm', [], 'Display', None],
  [ 0xc56b, 'HangulSyllableAelb', [], 'Display', None],
  [ 0xc56c, 'HangulSyllableAels', [], 'Display', None],
  [ 0xc56d, 'HangulSyllableAelt', [], 'Display', None],
  [ 0xc56e, 'HangulSyllableAelp', [], 'Display', None],
  [ 0xc56f, 'HangulSyllableAelh', [], 'Display', None],
  [ 0xc570, 'HangulSyllableAem', [], 'Display', None],
  [ 0xc571, 'HangulSyllableAeb', [], 'Display', None],
  [ 0xc572, 'HangulSyllableAebs', [], 'Display', None],
  [ 0xc573, 'HangulSyllableAes', [], 'Display', None],
  [ 0xc574, 'HangulSyllableAess', [], 'Display', None],
  [ 0xc575, 'HangulSyllableAeng', [], 'Display', None],
  [ 0xc576, 'HangulSyllableAej', [], 'Display', None],
  [ 0xc577, 'HangulSyllableAec', [], 'Display', None],
  [ 0xc578, 'HangulSyllableAek', [], 'Display', None],
  [ 0xc579, 'HangulSyllableAet', [], 'Display', None],
  [ 0xc57a, 'HangulSyllableAep', [], 'Display', None],
  [ 0xc57b, 'HangulSyllableAeh', [], 'Display', None],
  [ 0xc57c, 'HangulSyllableYa', [], 'Display', None],
  [ 0xc57d, 'HangulSyllableYag', [], 'Display', None],
  [ 0xc57e, 'HangulSyllableYagg', [], 'Display', None],
  [ 0xc57f, 'HangulSyllableYags', [], 'Display', None],
  [ 0xc580, 'HangulSyllableYan', [], 'Display', None],
  [ 0xc581, 'HangulSyllableYanj', [], 'Display', None],
  [ 0xc582, 'HangulSyllableYanh', [], 'Display', None],
  [ 0xc583, 'HangulSyllableYad', [], 'Display', None],
  [ 0xc584, 'HangulSyllableYal', [], 'Display', None],
  [ 0xc585, 'HangulSyllableYalg', [], 'Display', None],
  [ 0xc586, 'HangulSyllableYalm', [], 'Display', None],
  [ 0xc587, 'HangulSyllableYalb', [], 'Display', None],
  [ 0xc588, 'HangulSyllableYals', [], 'Display', None],
  [ 0xc589, 'HangulSyllableYalt', [], 'Display', None],
  [ 0xc58a, 'HangulSyllableYalp', [], 'Display', None],
  [ 0xc58b, 'HangulSyllableYalh', [], 'Display', None],
  [ 0xc58c, 'HangulSyllableYam', [], 'Display', None],
  [ 0xc58d, 'HangulSyllableYab', [], 'Display', None],
  [ 0xc58e, 'HangulSyllableYabs', [], 'Display', None],
  [ 0xc58f, 'HangulSyllableYas', [], 'Display', None],
  [ 0xc590, 'HangulSyllableYass', [], 'Display', None],
  [ 0xc591, 'HangulSyllableYang', [], 'Display', None],
  [ 0xc592, 'HangulSyllableYaj', [], 'Display', None],
  [ 0xc593, 'HangulSyllableYac', [], 'Display', None],
  [ 0xc594, 'HangulSyllableYak', [], 'Display', None],
  [ 0xc595, 'HangulSyllableYat', [], 'Display', None],
  [ 0xc596, 'HangulSyllableYap', [], 'Display', None],
  [ 0xc597, 'HangulSyllableYah', [], 'Display', None],
  [ 0xc598, 'HangulSyllableYae', [], 'Display', None],
  [ 0xc599, 'HangulSyllableYaeg', [], 'Display', None],
  [ 0xc59a, 'HangulSyllableYaegg', [], 'Display', None],
  [ 0xc59b, 'HangulSyllableYaegs', [], 'Display', None],
  [ 0xc59c, 'HangulSyllableYaen', [], 'Display', None],
  [ 0xc59d, 'HangulSyllableYaenj', [], 'Display', None],
  [ 0xc59e, 'HangulSyllableYaenh', [], 'Display', None],
  [ 0xc59f, 'HangulSyllableYaed', [], 'Display', None],
  [ 0xc5a0, 'HangulSyllableYael', [], 'Display', None],
  [ 0xc5a1, 'HangulSyllableYaelg', [], 'Display', None],
  [ 0xc5a2, 'HangulSyllableYaelm', [], 'Display', None],
  [ 0xc5a3, 'HangulSyllableYaelb', [], 'Display', None],
  [ 0xc5a4, 'HangulSyllableYaels', [], 'Display', None],
  [ 0xc5a5, 'HangulSyllableYaelt', [], 'Display', None],
  [ 0xc5a6, 'HangulSyllableYaelp', [], 'Display', None],
  [ 0xc5a7, 'HangulSyllableYaelh', [], 'Display', None],
  [ 0xc5a8, 'HangulSyllableYaem', [], 'Display', None],
  [ 0xc5a9, 'HangulSyllableYaeb', [], 'Display', None],
  [ 0xc5aa, 'HangulSyllableYaebs', [], 'Display', None],
  [ 0xc5ab, 'HangulSyllableYaes', [], 'Display', None],
  [ 0xc5ac, 'HangulSyllableYaess', [], 'Display', None],
  [ 0xc5ad, 'HangulSyllableYaeng', [], 'Display', None],
  [ 0xc5ae, 'HangulSyllableYaej', [], 'Display', None],
  [ 0xc5af, 'HangulSyllableYaec', [], 'Display', None],
  [ 0xc5b0, 'HangulSyllableYaek', [], 'Display', None],
  [ 0xc5b1, 'HangulSyllableYaet', [], 'Display', None],
  [ 0xc5b2, 'HangulSyllableYaep', [], 'Display', None],
  [ 0xc5b3, 'HangulSyllableYaeh', [], 'Display', None],
  [ 0xc5b4, 'HangulSyllableEo', [], 'Display', None],
  [ 0xc5b5, 'HangulSyllableEog', [], 'Display', None],
  [ 0xc5b6, 'HangulSyllableEogg', [], 'Display', None],
  [ 0xc5b7, 'HangulSyllableEogs', [], 'Display', None],
  [ 0xc5b8, 'HangulSyllableEon', [], 'Display', None],
  [ 0xc5b9, 'HangulSyllableEonj', [], 'Display', None],
  [ 0xc5ba, 'HangulSyllableEonh', [], 'Display', None],
  [ 0xc5bb, 'HangulSyllableEod', [], 'Display', None],
  [ 0xc5bc, 'HangulSyllableEol', [], 'Display', None],
  [ 0xc5bd, 'HangulSyllableEolg', [], 'Display', None],
  [ 0xc5be, 'HangulSyllableEolm', [], 'Display', None],
  [ 0xc5bf, 'HangulSyllableEolb', [], 'Display', None],
  [ 0xc5c0, 'HangulSyllableEols', [], 'Display', None],
  [ 0xc5c1, 'HangulSyllableEolt', [], 'Display', None],
  [ 0xc5c2, 'HangulSyllableEolp', [], 'Display', None],
  [ 0xc5c3, 'HangulSyllableEolh', [], 'Display', None],
  [ 0xc5c4, 'HangulSyllableEom', [], 'Display', None],
  [ 0xc5c5, 'HangulSyllableEob', [], 'Display', None],
  [ 0xc5c6, 'HangulSyllableEobs', [], 'Display', None],
  [ 0xc5c7, 'HangulSyllableEos', [], 'Display', None],
  [ 0xc5c8, 'HangulSyllableEoss', [], 'Display', None],
  [ 0xc5c9, 'HangulSyllableEong', [], 'Display', None],
  [ 0xc5ca, 'HangulSyllableEoj', [], 'Display', None],
  [ 0xc5cb, 'HangulSyllableEoc', [], 'Display', None],
  [ 0xc5cc, 'HangulSyllableEok', [], 'Display', None],
  [ 0xc5cd, 'HangulSyllableEot', [], 'Display', None],
  [ 0xc5ce, 'HangulSyllableEop', [], 'Display', None],
  [ 0xc5cf, 'HangulSyllableEoh', [], 'Display', None],
  [ 0xc5d0, 'HangulSyllableE', [], 'Display', None],
  [ 0xc5d1, 'HangulSyllableEg', [], 'Display', None],
  [ 0xc5d2, 'HangulSyllableEgg', [], 'Display', None],
  [ 0xc5d3, 'HangulSyllableEgs', [], 'Display', None],
  [ 0xc5d4, 'HangulSyllableEn', [], 'Display', None],
  [ 0xc5d5, 'HangulSyllableEnj', [], 'Display', None],
  [ 0xc5d6, 'HangulSyllableEnh', [], 'Display', None],
  [ 0xc5d7, 'HangulSyllableEd', [], 'Display', None],
  [ 0xc5d8, 'HangulSyllableEl', [], 'Display', None],
  [ 0xc5d9, 'HangulSyllableElg', [], 'Display', None],
  [ 0xc5da, 'HangulSyllableElm', [], 'Display', None],
  [ 0xc5db, 'HangulSyllableElb', [], 'Display', None],
  [ 0xc5dc, 'HangulSyllableEls', [], 'Display', None],
  [ 0xc5dd, 'HangulSyllableElt', [], 'Display', None],
  [ 0xc5de, 'HangulSyllableElp', [], 'Display', None],
  [ 0xc5df, 'HangulSyllableElh', [], 'Display', None],
  [ 0xc5e0, 'HangulSyllableEm', [], 'Display', None],
  [ 0xc5e1, 'HangulSyllableEb', [], 'Display', None],
  [ 0xc5e2, 'HangulSyllableEbs', [], 'Display', None],
  [ 0xc5e3, 'HangulSyllableEs', [], 'Display', None],
  [ 0xc5e4, 'HangulSyllableEss', [], 'Display', None],
  [ 0xc5e5, 'HangulSyllableEng', [], 'Display', None],
  [ 0xc5e6, 'HangulSyllableEj', [], 'Display', None],
  [ 0xc5e7, 'HangulSyllableEc', [], 'Display', None],
  [ 0xc5e8, 'HangulSyllableEk', [], 'Display', None],
  [ 0xc5e9, 'HangulSyllableEt', [], 'Display', None],
  [ 0xc5ea, 'HangulSyllableEp', [], 'Display', None],
  [ 0xc5eb, 'HangulSyllableEh', [], 'Display', None],
  [ 0xc5ec, 'HangulSyllableYeo', [], 'Display', None],
  [ 0xc5ed, 'HangulSyllableYeog', [], 'Display', None],
  [ 0xc5ee, 'HangulSyllableYeogg', [], 'Display', None],
  [ 0xc5ef, 'HangulSyllableYeogs', [], 'Display', None],
  [ 0xc5f0, 'HangulSyllableYeon', [], 'Display', None],
  [ 0xc5f1, 'HangulSyllableYeonj', [], 'Display', None],
  [ 0xc5f2, 'HangulSyllableYeonh', [], 'Display', None],
  [ 0xc5f3, 'HangulSyllableYeod', [], 'Display', None],
  [ 0xc5f4, 'HangulSyllableYeol', [], 'Display', None],
  [ 0xc5f5, 'HangulSyllableYeolg', [], 'Display', None],
  [ 0xc5f6, 'HangulSyllableYeolm', [], 'Display', None],
  [ 0xc5f7, 'HangulSyllableYeolb', [], 'Display', None],
  [ 0xc5f8, 'HangulSyllableYeols', [], 'Display', None],
  [ 0xc5f9, 'HangulSyllableYeolt', [], 'Display', None],
  [ 0xc5fa, 'HangulSyllableYeolp', [], 'Display', None],
  [ 0xc5fb, 'HangulSyllableYeolh', [], 'Display', None],
  [ 0xc5fc, 'HangulSyllableYeom', [], 'Display', None],
  [ 0xc5fd, 'HangulSyllableYeob', [], 'Display', None],
  [ 0xc5fe, 'HangulSyllableYeobs', [], 'Display', None],
  [ 0xc5ff, 'HangulSyllableYeos', [], 'Display', None],
  [ 0xc600, 'HangulSyllableYeoss', [], 'Display', None],
  [ 0xc601, 'HangulSyllableYeong', [], 'Display', None],
  [ 0xc602, 'HangulSyllableYeoj', [], 'Display', None],
  [ 0xc603, 'HangulSyllableYeoc', [], 'Display', None],
  [ 0xc604, 'HangulSyllableYeok', [], 'Display', None],
  [ 0xc605, 'HangulSyllableYeot', [], 'Display', None],
  [ 0xc606, 'HangulSyllableYeop', [], 'Display', None],
  [ 0xc607, 'HangulSyllableYeoh', [], 'Display', None],
  [ 0xc608, 'HangulSyllableYe', [], 'Display', None],
  [ 0xc609, 'HangulSyllableYeg', [], 'Display', None],
  [ 0xc60a, 'HangulSyllableYegg', [], 'Display', None],
  [ 0xc60b, 'HangulSyllableYegs', [], 'Display', None],
  [ 0xc60c, 'HangulSyllableYen', [], 'Display', None],
  [ 0xc60d, 'HangulSyllableYenj', [], 'Display', None],
  [ 0xc60e, 'HangulSyllableYenh', [], 'Display', None],
  [ 0xc60f, 'HangulSyllableYed', [], 'Display', None],
  [ 0xc610, 'HangulSyllableYel', [], 'Display', None],
  [ 0xc611, 'HangulSyllableYelg', [], 'Display', None],
  [ 0xc612, 'HangulSyllableYelm', [], 'Display', None],
  [ 0xc613, 'HangulSyllableYelb', [], 'Display', None],
  [ 0xc614, 'HangulSyllableYels', [], 'Display', None],
  [ 0xc615, 'HangulSyllableYelt', [], 'Display', None],
  [ 0xc616, 'HangulSyllableYelp', [], 'Display', None],
  [ 0xc617, 'HangulSyllableYelh', [], 'Display', None],
  [ 0xc618, 'HangulSyllableYem', [], 'Display', None],
  [ 0xc619, 'HangulSyllableYeb', [], 'Display', None],
  [ 0xc61a, 'HangulSyllableYebs', [], 'Display', None],
  [ 0xc61b, 'HangulSyllableYes', [], 'Display', None],
  [ 0xc61c, 'HangulSyllableYess', [], 'Display', None],
  [ 0xc61d, 'HangulSyllableYeng', [], 'Display', None],
  [ 0xc61e, 'HangulSyllableYej', [], 'Display', None],
  [ 0xc61f, 'HangulSyllableYec', [], 'Display', None],
  [ 0xc620, 'HangulSyllableYek', [], 'Display', None],
  [ 0xc621, 'HangulSyllableYet', [], 'Display', None],
  [ 0xc622, 'HangulSyllableYep', [], 'Display', None],
  [ 0xc623, 'HangulSyllableYeh', [], 'Display', None],
  [ 0xc624, 'HangulSyllableO', [], 'Display', None],
  [ 0xc625, 'HangulSyllableOg', [], 'Display', None],
  [ 0xc626, 'HangulSyllableOgg', [], 'Display', None],
  [ 0xc627, 'HangulSyllableOgs', [], 'Display', None],
  [ 0xc628, 'HangulSyllableOn', [], 'Display', None],
  [ 0xc629, 'HangulSyllableOnj', [], 'Display', None],
  [ 0xc62a, 'HangulSyllableOnh', [], 'Display', None],
  [ 0xc62b, 'HangulSyllableOd', [], 'Display', None],
  [ 0xc62c, 'HangulSyllableOl', [], 'Display', None],
  [ 0xc62d, 'HangulSyllableOlg', [], 'Display', None],
  [ 0xc62e, 'HangulSyllableOlm', [], 'Display', None],
  [ 0xc62f, 'HangulSyllableOlb', [], 'Display', None],
  [ 0xc630, 'HangulSyllableOls', [], 'Display', None],
  [ 0xc631, 'HangulSyllableOlt', [], 'Display', None],
  [ 0xc632, 'HangulSyllableOlp', [], 'Display', None],
  [ 0xc633, 'HangulSyllableOlh', [], 'Display', None],
  [ 0xc634, 'HangulSyllableOm', [], 'Display', None],
  [ 0xc635, 'HangulSyllableOb', [], 'Display', None],
  [ 0xc636, 'HangulSyllableObs', [], 'Display', None],
  [ 0xc637, 'HangulSyllableOs', [], 'Display', None],
  [ 0xc638, 'HangulSyllableOss', [], 'Display', None],
  [ 0xc639, 'HangulSyllableOng', [], 'Display', None],
  [ 0xc63a, 'HangulSyllableOj', [], 'Display', None],
  [ 0xc63b, 'HangulSyllableOc', [], 'Display', None],
  [ 0xc63c, 'HangulSyllableOk', [], 'Display', None],
  [ 0xc63d, 'HangulSyllableOt', [], 'Display', None],
  [ 0xc63e, 'HangulSyllableOp', [], 'Display', None],
  [ 0xc63f, 'HangulSyllableOh', [], 'Display', None],
  [ 0xc640, 'HangulSyllableWa', [], 'Display', None],
  [ 0xc641, 'HangulSyllableWag', [], 'Display', None],
  [ 0xc642, 'HangulSyllableWagg', [], 'Display', None],
  [ 0xc643, 'HangulSyllableWags', [], 'Display', None],
  [ 0xc644, 'HangulSyllableWan', [], 'Display', None],
  [ 0xc645, 'HangulSyllableWanj', [], 'Display', None],
  [ 0xc646, 'HangulSyllableWanh', [], 'Display', None],
  [ 0xc647, 'HangulSyllableWad', [], 'Display', None],
  [ 0xc648, 'HangulSyllableWal', [], 'Display', None],
  [ 0xc649, 'HangulSyllableWalg', [], 'Display', None],
  [ 0xc64a, 'HangulSyllableWalm', [], 'Display', None],
  [ 0xc64b, 'HangulSyllableWalb', [], 'Display', None],
  [ 0xc64c, 'HangulSyllableWals', [], 'Display', None],
  [ 0xc64d, 'HangulSyllableWalt', [], 'Display', None],
  [ 0xc64e, 'HangulSyllableWalp', [], 'Display', None],
  [ 0xc64f, 'HangulSyllableWalh', [], 'Display', None],
  [ 0xc650, 'HangulSyllableWam', [], 'Display', None],
  [ 0xc651, 'HangulSyllableWab', [], 'Display', None],
  [ 0xc652, 'HangulSyllableWabs', [], 'Display', None],
  [ 0xc653, 'HangulSyllableWas', [], 'Display', None],
  [ 0xc654, 'HangulSyllableWass', [], 'Display', None],
  [ 0xc655, 'HangulSyllableWang', [], 'Display', None],
  [ 0xc656, 'HangulSyllableWaj', [], 'Display', None],
  [ 0xc657, 'HangulSyllableWac', [], 'Display', None],
  [ 0xc658, 'HangulSyllableWak', [], 'Display', None],
  [ 0xc659, 'HangulSyllableWat', [], 'Display', None],
  [ 0xc65a, 'HangulSyllableWap', [], 'Display', None],
  [ 0xc65b, 'HangulSyllableWah', [], 'Display', None],
  [ 0xc65c, 'HangulSyllableWae', [], 'Display', None],
  [ 0xc65d, 'HangulSyllableWaeg', [], 'Display', None],
  [ 0xc65e, 'HangulSyllableWaegg', [], 'Display', None],
  [ 0xc65f, 'HangulSyllableWaegs', [], 'Display', None],
  [ 0xc660, 'HangulSyllableWaen', [], 'Display', None],
  [ 0xc661, 'HangulSyllableWaenj', [], 'Display', None],
  [ 0xc662, 'HangulSyllableWaenh', [], 'Display', None],
  [ 0xc663, 'HangulSyllableWaed', [], 'Display', None],
  [ 0xc664, 'HangulSyllableWael', [], 'Display', None],
  [ 0xc665, 'HangulSyllableWaelg', [], 'Display', None],
  [ 0xc666, 'HangulSyllableWaelm', [], 'Display', None],
  [ 0xc667, 'HangulSyllableWaelb', [], 'Display', None],
  [ 0xc668, 'HangulSyllableWaels', [], 'Display', None],
  [ 0xc669, 'HangulSyllableWaelt', [], 'Display', None],
  [ 0xc66a, 'HangulSyllableWaelp', [], 'Display', None],
  [ 0xc66b, 'HangulSyllableWaelh', [], 'Display', None],
  [ 0xc66c, 'HangulSyllableWaem', [], 'Display', None],
  [ 0xc66d, 'HangulSyllableWaeb', [], 'Display', None],
  [ 0xc66e, 'HangulSyllableWaebs', [], 'Display', None],
  [ 0xc66f, 'HangulSyllableWaes', [], 'Display', None],
  [ 0xc670, 'HangulSyllableWaess', [], 'Display', None],
  [ 0xc671, 'HangulSyllableWaeng', [], 'Display', None],
  [ 0xc672, 'HangulSyllableWaej', [], 'Display', None],
  [ 0xc673, 'HangulSyllableWaec', [], 'Display', None],
  [ 0xc674, 'HangulSyllableWaek', [], 'Display', None],
  [ 0xc675, 'HangulSyllableWaet', [], 'Display', None],
  [ 0xc676, 'HangulSyllableWaep', [], 'Display', None],
  [ 0xc677, 'HangulSyllableWaeh', [], 'Display', None],
  [ 0xc678, 'HangulSyllableOe', [], 'Display', None],
  [ 0xc679, 'HangulSyllableOeg', [], 'Display', None],
  [ 0xc67a, 'HangulSyllableOegg', [], 'Display', None],
  [ 0xc67b, 'HangulSyllableOegs', [], 'Display', None],
  [ 0xc67c, 'HangulSyllableOen', [], 'Display', None],
  [ 0xc67d, 'HangulSyllableOenj', [], 'Display', None],
  [ 0xc67e, 'HangulSyllableOenh', [], 'Display', None],
  [ 0xc67f, 'HangulSyllableOed', [], 'Display', None],
  [ 0xc680, 'HangulSyllableOel', [], 'Display', None],
  [ 0xc681, 'HangulSyllableOelg', [], 'Display', None],
  [ 0xc682, 'HangulSyllableOelm', [], 'Display', None],
  [ 0xc683, 'HangulSyllableOelb', [], 'Display', None],
  [ 0xc684, 'HangulSyllableOels', [], 'Display', None],
  [ 0xc685, 'HangulSyllableOelt', [], 'Display', None],
  [ 0xc686, 'HangulSyllableOelp', [], 'Display', None],
  [ 0xc687, 'HangulSyllableOelh', [], 'Display', None],
  [ 0xc688, 'HangulSyllableOem', [], 'Display', None],
  [ 0xc689, 'HangulSyllableOeb', [], 'Display', None],
  [ 0xc68a, 'HangulSyllableOebs', [], 'Display', None],
  [ 0xc68b, 'HangulSyllableOes', [], 'Display', None],
  [ 0xc68c, 'HangulSyllableOess', [], 'Display', None],
  [ 0xc68d, 'HangulSyllableOeng', [], 'Display', None],
  [ 0xc68e, 'HangulSyllableOej', [], 'Display', None],
  [ 0xc68f, 'HangulSyllableOec', [], 'Display', None],
  [ 0xc690, 'HangulSyllableOek', [], 'Display', None],
  [ 0xc691, 'HangulSyllableOet', [], 'Display', None],
  [ 0xc692, 'HangulSyllableOep', [], 'Display', None],
  [ 0xc693, 'HangulSyllableOeh', [], 'Display', None],
  [ 0xc694, 'HangulSyllableYo', [], 'Display', None],
  [ 0xc695, 'HangulSyllableYog', [], 'Display', None],
  [ 0xc696, 'HangulSyllableYogg', [], 'Display', None],
  [ 0xc697, 'HangulSyllableYogs', [], 'Display', None],
  [ 0xc698, 'HangulSyllableYon', [], 'Display', None],
  [ 0xc699, 'HangulSyllableYonj', [], 'Display', None],
  [ 0xc69a, 'HangulSyllableYonh', [], 'Display', None],
  [ 0xc69b, 'HangulSyllableYod', [], 'Display', None],
  [ 0xc69c, 'HangulSyllableYol', [], 'Display', None],
  [ 0xc69d, 'HangulSyllableYolg', [], 'Display', None],
  [ 0xc69e, 'HangulSyllableYolm', [], 'Display', None],
  [ 0xc69f, 'HangulSyllableYolb', [], 'Display', None],
  [ 0xc6a0, 'HangulSyllableYols', [], 'Display', None],
  [ 0xc6a1, 'HangulSyllableYolt', [], 'Display', None],
  [ 0xc6a2, 'HangulSyllableYolp', [], 'Display', None],
  [ 0xc6a3, 'HangulSyllableYolh', [], 'Display', None],
  [ 0xc6a4, 'HangulSyllableYom', [], 'Display', None],
  [ 0xc6a5, 'HangulSyllableYob', [], 'Display', None],
  [ 0xc6a6, 'HangulSyllableYobs', [], 'Display', None],
  [ 0xc6a7, 'HangulSyllableYos', [], 'Display', None],
  [ 0xc6a8, 'HangulSyllableYoss', [], 'Display', None],
  [ 0xc6a9, 'HangulSyllableYong', [], 'Display', None],
  [ 0xc6aa, 'HangulSyllableYoj', [], 'Display', None],
  [ 0xc6ab, 'HangulSyllableYoc', [], 'Display', None],
  [ 0xc6ac, 'HangulSyllableYok', [], 'Display', None],
  [ 0xc6ad, 'HangulSyllableYot', [], 'Display', None],
  [ 0xc6ae, 'HangulSyllableYop', [], 'Display', None],
  [ 0xc6af, 'HangulSyllableYoh', [], 'Display', None],
  [ 0xc6b0, 'HangulSyllableU', [], 'Display', None],
  [ 0xc6b1, 'HangulSyllableUg', [], 'Display', None],
  [ 0xc6b2, 'HangulSyllableUgg', [], 'Display', None],
  [ 0xc6b3, 'HangulSyllableUgs', [], 'Display', None],
  [ 0xc6b4, 'HangulSyllableUn', [], 'Display', None],
  [ 0xc6b5, 'HangulSyllableUnj', [], 'Display', None],
  [ 0xc6b6, 'HangulSyllableUnh', [], 'Display', None],
  [ 0xc6b7, 'HangulSyllableUd', [], 'Display', None],
  [ 0xc6b8, 'HangulSyllableUl', [], 'Display', None],
  [ 0xc6b9, 'HangulSyllableUlg', [], 'Display', None],
  [ 0xc6ba, 'HangulSyllableUlm', [], 'Display', None],
  [ 0xc6bb, 'HangulSyllableUlb', [], 'Display', None],
  [ 0xc6bc, 'HangulSyllableUls', [], 'Display', None],
  [ 0xc6bd, 'HangulSyllableUlt', [], 'Display', None],
  [ 0xc6be, 'HangulSyllableUlp', [], 'Display', None],
  [ 0xc6bf, 'HangulSyllableUlh', [], 'Display', None],
  [ 0xc6c0, 'HangulSyllableUm', [], 'Display', None],
  [ 0xc6c1, 'HangulSyllableUb', [], 'Display', None],
  [ 0xc6c2, 'HangulSyllableUbs', [], 'Display', None],
  [ 0xc6c3, 'HangulSyllableUs', [], 'Display', None],
  [ 0xc6c4, 'HangulSyllableUss', [], 'Display', None],
  [ 0xc6c5, 'HangulSyllableUng', [], 'Display', None],
  [ 0xc6c6, 'HangulSyllableUj', [], 'Display', None],
  [ 0xc6c7, 'HangulSyllableUc', [], 'Display', None],
  [ 0xc6c8, 'HangulSyllableUk', [], 'Display', None],
  [ 0xc6c9, 'HangulSyllableUt', [], 'Display', None],
  [ 0xc6ca, 'HangulSyllableUp', [], 'Display', None],
  [ 0xc6cb, 'HangulSyllableUh', [], 'Display', None],
  [ 0xc6cc, 'HangulSyllableWeo', [], 'Display', None],
  [ 0xc6cd, 'HangulSyllableWeog', [], 'Display', None],
  [ 0xc6ce, 'HangulSyllableWeogg', [], 'Display', None],
  [ 0xc6cf, 'HangulSyllableWeogs', [], 'Display', None],
  [ 0xc6d0, 'HangulSyllableWeon', [], 'Display', None],
  [ 0xc6d1, 'HangulSyllableWeonj', [], 'Display', None],
  [ 0xc6d2, 'HangulSyllableWeonh', [], 'Display', None],
  [ 0xc6d3, 'HangulSyllableWeod', [], 'Display', None],
  [ 0xc6d4, 'HangulSyllableWeol', [], 'Display', None],
  [ 0xc6d5, 'HangulSyllableWeolg', [], 'Display', None],
  [ 0xc6d6, 'HangulSyllableWeolm', [], 'Display', None],
  [ 0xc6d7, 'HangulSyllableWeolb', [], 'Display', None],
  [ 0xc6d8, 'HangulSyllableWeols', [], 'Display', None],
  [ 0xc6d9, 'HangulSyllableWeolt', [], 'Display', None],
  [ 0xc6da, 'HangulSyllableWeolp', [], 'Display', None],
  [ 0xc6db, 'HangulSyllableWeolh', [], 'Display', None],
  [ 0xc6dc, 'HangulSyllableWeom', [], 'Display', None],
  [ 0xc6dd, 'HangulSyllableWeob', [], 'Display', None],
  [ 0xc6de, 'HangulSyllableWeobs', [], 'Display', None],
  [ 0xc6df, 'HangulSyllableWeos', [], 'Display', None],
  [ 0xc6e0, 'HangulSyllableWeoss', [], 'Display', None],
  [ 0xc6e1, 'HangulSyllableWeong', [], 'Display', None],
  [ 0xc6e2, 'HangulSyllableWeoj', [], 'Display', None],
  [ 0xc6e3, 'HangulSyllableWeoc', [], 'Display', None],
  [ 0xc6e4, 'HangulSyllableWeok', [], 'Display', None],
  [ 0xc6e5, 'HangulSyllableWeot', [], 'Display', None],
  [ 0xc6e6, 'HangulSyllableWeop', [], 'Display', None],
  [ 0xc6e7, 'HangulSyllableWeoh', [], 'Display', None],
  [ 0xc6e8, 'HangulSyllableWe', [], 'Display', None],
  [ 0xc6e9, 'HangulSyllableWeg', [], 'Display', None],
  [ 0xc6ea, 'HangulSyllableWegg', [], 'Display', None],
  [ 0xc6eb, 'HangulSyllableWegs', [], 'Display', None],
  [ 0xc6ec, 'HangulSyllableWen', [], 'Display', None],
  [ 0xc6ed, 'HangulSyllableWenj', [], 'Display', None],
  [ 0xc6ee, 'HangulSyllableWenh', [], 'Display', None],
  [ 0xc6ef, 'HangulSyllableWed', [], 'Display', None],
  [ 0xc6f0, 'HangulSyllableWel', [], 'Display', None],
  [ 0xc6f1, 'HangulSyllableWelg', [], 'Display', None],
  [ 0xc6f2, 'HangulSyllableWelm', [], 'Display', None],
  [ 0xc6f3, 'HangulSyllableWelb', [], 'Display', None],
  [ 0xc6f4, 'HangulSyllableWels', [], 'Display', None],
  [ 0xc6f5, 'HangulSyllableWelt', [], 'Display', None],
  [ 0xc6f6, 'HangulSyllableWelp', [], 'Display', None],
  [ 0xc6f7, 'HangulSyllableWelh', [], 'Display', None],
  [ 0xc6f8, 'HangulSyllableWem', [], 'Display', None],
  [ 0xc6f9, 'HangulSyllableWeb', [], 'Display', None],
  [ 0xc6fa, 'HangulSyllableWebs', [], 'Display', None],
  [ 0xc6fb, 'HangulSyllableWes', [], 'Display', None],
  [ 0xc6fc, 'HangulSyllableWess', [], 'Display', None],
  [ 0xc6fd, 'HangulSyllableWeng', [], 'Display', None],
  [ 0xc6fe, 'HangulSyllableWej', [], 'Display', None],
  [ 0xc6ff, 'HangulSyllableWec', [], 'Display', None],
  [ 0xc700, 'HangulSyllableWek', [], 'Display', None],
  [ 0xc701, 'HangulSyllableWet', [], 'Display', None],
  [ 0xc702, 'HangulSyllableWep', [], 'Display', None],
  [ 0xc703, 'HangulSyllableWeh', [], 'Display', None],
  [ 0xc704, 'HangulSyllableWi', [], 'Display', None],
  [ 0xc705, 'HangulSyllableWig', [], 'Display', None],
  [ 0xc706, 'HangulSyllableWigg', [], 'Display', None],
  [ 0xc707, 'HangulSyllableWigs', [], 'Display', None],
  [ 0xc708, 'HangulSyllableWin', [], 'Display', None],
  [ 0xc709, 'HangulSyllableWinj', [], 'Display', None],
  [ 0xc70a, 'HangulSyllableWinh', [], 'Display', None],
  [ 0xc70b, 'HangulSyllableWid', [], 'Display', None],
  [ 0xc70c, 'HangulSyllableWil', [], 'Display', None],
  [ 0xc70d, 'HangulSyllableWilg', [], 'Display', None],
  [ 0xc70e, 'HangulSyllableWilm', [], 'Display', None],
  [ 0xc70f, 'HangulSyllableWilb', [], 'Display', None],
  [ 0xc710, 'HangulSyllableWils', [], 'Display', None],
  [ 0xc711, 'HangulSyllableWilt', [], 'Display', None],
  [ 0xc712, 'HangulSyllableWilp', [], 'Display', None],
  [ 0xc713, 'HangulSyllableWilh', [], 'Display', None],
  [ 0xc714, 'HangulSyllableWim', [], 'Display', None],
  [ 0xc715, 'HangulSyllableWib', [], 'Display', None],
  [ 0xc716, 'HangulSyllableWibs', [], 'Display', None],
  [ 0xc717, 'HangulSyllableWis', [], 'Display', None],
  [ 0xc718, 'HangulSyllableWiss', [], 'Display', None],
  [ 0xc719, 'HangulSyllableWing', [], 'Display', None],
  [ 0xc71a, 'HangulSyllableWij', [], 'Display', None],
  [ 0xc71b, 'HangulSyllableWic', [], 'Display', None],
  [ 0xc71c, 'HangulSyllableWik', [], 'Display', None],
  [ 0xc71d, 'HangulSyllableWit', [], 'Display', None],
  [ 0xc71e, 'HangulSyllableWip', [], 'Display', None],
  [ 0xc71f, 'HangulSyllableWih', [], 'Display', None],
  [ 0xc720, 'HangulSyllableYu', [], 'Display', None],
  [ 0xc721, 'HangulSyllableYug', [], 'Display', None],
  [ 0xc722, 'HangulSyllableYugg', [], 'Display', None],
  [ 0xc723, 'HangulSyllableYugs', [], 'Display', None],
  [ 0xc724, 'HangulSyllableYun', [], 'Display', None],
  [ 0xc725, 'HangulSyllableYunj', [], 'Display', None],
  [ 0xc726, 'HangulSyllableYunh', [], 'Display', None],
  [ 0xc727, 'HangulSyllableYud', [], 'Display', None],
  [ 0xc728, 'HangulSyllableYul', [], 'Display', None],
  [ 0xc729, 'HangulSyllableYulg', [], 'Display', None],
  [ 0xc72a, 'HangulSyllableYulm', [], 'Display', None],
  [ 0xc72b, 'HangulSyllableYulb', [], 'Display', None],
  [ 0xc72c, 'HangulSyllableYuls', [], 'Display', None],
  [ 0xc72d, 'HangulSyllableYult', [], 'Display', None],
  [ 0xc72e, 'HangulSyllableYulp', [], 'Display', None],
  [ 0xc72f, 'HangulSyllableYulh', [], 'Display', None],
  [ 0xc730, 'HangulSyllableYum', [], 'Display', None],
  [ 0xc731, 'HangulSyllableYub', [], 'Display', None],
  [ 0xc732, 'HangulSyllableYubs', [], 'Display', None],
  [ 0xc733, 'HangulSyllableYus', [], 'Display', None],
  [ 0xc734, 'HangulSyllableYuss', [], 'Display', None],
  [ 0xc735, 'HangulSyllableYung', [], 'Display', None],
  [ 0xc736, 'HangulSyllableYuj', [], 'Display', None],
  [ 0xc737, 'HangulSyllableYuc', [], 'Display', None],
  [ 0xc738, 'HangulSyllableYuk', [], 'Display', None],
  [ 0xc739, 'HangulSyllableYut', [], 'Display', None],
  [ 0xc73a, 'HangulSyllableYup', [], 'Display', None],
  [ 0xc73b, 'HangulSyllableYuh', [], 'Display', None],
  [ 0xc73c, 'HangulSyllableEu', [], 'Display', None],
  [ 0xc73d, 'HangulSyllableEug', [], 'Display', None],
  [ 0xc73e, 'HangulSyllableEugg', [], 'Display', None],
  [ 0xc73f, 'HangulSyllableEugs', [], 'Display', None],
  [ 0xc740, 'HangulSyllableEun', [], 'Display', None],
  [ 0xc741, 'HangulSyllableEunj', [], 'Display', None],
  [ 0xc742, 'HangulSyllableEunh', [], 'Display', None],
  [ 0xc743, 'HangulSyllableEud', [], 'Display', None],
  [ 0xc744, 'HangulSyllableEul', [], 'Display', None],
  [ 0xc745, 'HangulSyllableEulg', [], 'Display', None],
  [ 0xc746, 'HangulSyllableEulm', [], 'Display', None],
  [ 0xc747, 'HangulSyllableEulb', [], 'Display', None],
  [ 0xc748, 'HangulSyllableEuls', [], 'Display', None],
  [ 0xc749, 'HangulSyllableEult', [], 'Display', None],
  [ 0xc74a, 'HangulSyllableEulp', [], 'Display', None],
  [ 0xc74b, 'HangulSyllableEulh', [], 'Display', None],
  [ 0xc74c, 'HangulSyllableEum', [], 'Display', None],
  [ 0xc74d, 'HangulSyllableEub', [], 'Display', None],
  [ 0xc74e, 'HangulSyllableEubs', [], 'Display', None],
  [ 0xc74f, 'HangulSyllableEus', [], 'Display', None],
  [ 0xc750, 'HangulSyllableEuss', [], 'Display', None],
  [ 0xc751, 'HangulSyllableEung', [], 'Display', None],
  [ 0xc752, 'HangulSyllableEuj', [], 'Display', None],
  [ 0xc753, 'HangulSyllableEuc', [], 'Display', None],
  [ 0xc754, 'HangulSyllableEuk', [], 'Display', None],
  [ 0xc755, 'HangulSyllableEut', [], 'Display', None],
  [ 0xc756, 'HangulSyllableEup', [], 'Display', None],
  [ 0xc757, 'HangulSyllableEuh', [], 'Display', None],
  [ 0xc758, 'HangulSyllableYi', [], 'Display', None],
  [ 0xc759, 'HangulSyllableYig', [], 'Display', None],
  [ 0xc75a, 'HangulSyllableYigg', [], 'Display', None],
  [ 0xc75b, 'HangulSyllableYigs', [], 'Display', None],
  [ 0xc75c, 'HangulSyllableYin', [], 'Display', None],
  [ 0xc75d, 'HangulSyllableYinj', [], 'Display', None],
  [ 0xc75e, 'HangulSyllableYinh', [], 'Display', None],
  [ 0xc75f, 'HangulSyllableYid', [], 'Display', None],
  [ 0xc760, 'HangulSyllableYil', [], 'Display', None],
  [ 0xc761, 'HangulSyllableYilg', [], 'Display', None],
  [ 0xc762, 'HangulSyllableYilm', [], 'Display', None],
  [ 0xc763, 'HangulSyllableYilb', [], 'Display', None],
  [ 0xc764, 'HangulSyllableYils', [], 'Display', None],
  [ 0xc765, 'HangulSyllableYilt', [], 'Display', None],
  [ 0xc766, 'HangulSyllableYilp', [], 'Display', None],
  [ 0xc767, 'HangulSyllableYilh', [], 'Display', None],
  [ 0xc768, 'HangulSyllableYim', [], 'Display', None],
  [ 0xc769, 'HangulSyllableYib', [], 'Display', None],
  [ 0xc76a, 'HangulSyllableYibs', [], 'Display', None],
  [ 0xc76b, 'HangulSyllableYis', [], 'Display', None],
  [ 0xc76c, 'HangulSyllableYiss', [], 'Display', None],
  [ 0xc76d, 'HangulSyllableYing', [], 'Display', None],
  [ 0xc76e, 'HangulSyllableYij', [], 'Display', None],
  [ 0xc76f, 'HangulSyllableYic', [], 'Display', None],
  [ 0xc770, 'HangulSyllableYik', [], 'Display', None],
  [ 0xc771, 'HangulSyllableYit', [], 'Display', None],
  [ 0xc772, 'HangulSyllableYip', [], 'Display', None],
  [ 0xc773, 'HangulSyllableYih', [], 'Display', None],
  [ 0xc774, 'HangulSyllableI', [], 'Display', None],
  [ 0xc775, 'HangulSyllableIg', [], 'Display', None],
  [ 0xc776, 'HangulSyllableIgg', [], 'Display', None],
  [ 0xc777, 'HangulSyllableIgs', [], 'Display', None],
  [ 0xc778, 'HangulSyllableIn', [], 'Display', None],
  [ 0xc779, 'HangulSyllableInj', [], 'Display', None],
  [ 0xc77a, 'HangulSyllableInh', [], 'Display', None],
  [ 0xc77b, 'HangulSyllableId', [], 'Display', None],
  [ 0xc77c, 'HangulSyllableIl', [], 'Display', None],
  [ 0xc77d, 'HangulSyllableIlg', [], 'Display', None],
  [ 0xc77e, 'HangulSyllableIlm', [], 'Display', None],
  [ 0xc77f, 'HangulSyllableIlb', [], 'Display', None],
  [ 0xc780, 'HangulSyllableIls', [], 'Display', None],
  [ 0xc781, 'HangulSyllableIlt', [], 'Display', None],
  [ 0xc782, 'HangulSyllableIlp', [], 'Display', None],
  [ 0xc783, 'HangulSyllableIlh', [], 'Display', None],
  [ 0xc784, 'HangulSyllableIm', [], 'Display', None],
  [ 0xc785, 'HangulSyllableIb', [], 'Display', None],
  [ 0xc786, 'HangulSyllableIbs', [], 'Display', None],
  [ 0xc787, 'HangulSyllableIs', [], 'Display', None],
  [ 0xc788, 'HangulSyllableIss', [], 'Display', None],
  [ 0xc789, 'HangulSyllableIng', [], 'Display', None],
  [ 0xc78a, 'HangulSyllableIj', [], 'Display', None],
  [ 0xc78b, 'HangulSyllableIc', [], 'Display', None],
  [ 0xc78c, 'HangulSyllableIk', [], 'Display', None],
  [ 0xc78d, 'HangulSyllableIt', [], 'Display', None],
  [ 0xc78e, 'HangulSyllableIp', [], 'Display', None],
  [ 0xc78f, 'HangulSyllableIh', [], 'Display', None],
  [ 0xc790, 'HangulSyllableJa', [], 'Display', None],
  [ 0xc791, 'HangulSyllableJag', [], 'Display', None],
  [ 0xc792, 'HangulSyllableJagg', [], 'Display', None],
  [ 0xc793, 'HangulSyllableJags', [], 'Display', None],
  [ 0xc794, 'HangulSyllableJan', [], 'Display', None],
  [ 0xc795, 'HangulSyllableJanj', [], 'Display', None],
  [ 0xc796, 'HangulSyllableJanh', [], 'Display', None],
  [ 0xc797, 'HangulSyllableJad', [], 'Display', None],
  [ 0xc798, 'HangulSyllableJal', [], 'Display', None],
  [ 0xc799, 'HangulSyllableJalg', [], 'Display', None],
  [ 0xc79a, 'HangulSyllableJalm', [], 'Display', None],
  [ 0xc79b, 'HangulSyllableJalb', [], 'Display', None],
  [ 0xc79c, 'HangulSyllableJals', [], 'Display', None],
  [ 0xc79d, 'HangulSyllableJalt', [], 'Display', None],
  [ 0xc79e, 'HangulSyllableJalp', [], 'Display', None],
  [ 0xc79f, 'HangulSyllableJalh', [], 'Display', None],
  [ 0xc7a0, 'HangulSyllableJam', [], 'Display', None],
  [ 0xc7a1, 'HangulSyllableJab', [], 'Display', None],
  [ 0xc7a2, 'HangulSyllableJabs', [], 'Display', None],
  [ 0xc7a3, 'HangulSyllableJas', [], 'Display', None],
  [ 0xc7a4, 'HangulSyllableJass', [], 'Display', None],
  [ 0xc7a5, 'HangulSyllableJang', [], 'Display', None],
  [ 0xc7a6, 'HangulSyllableJaj', [], 'Display', None],
  [ 0xc7a7, 'HangulSyllableJac', [], 'Display', None],
  [ 0xc7a8, 'HangulSyllableJak', [], 'Display', None],
  [ 0xc7a9, 'HangulSyllableJat', [], 'Display', None],
  [ 0xc7aa, 'HangulSyllableJap', [], 'Display', None],
  [ 0xc7ab, 'HangulSyllableJah', [], 'Display', None],
  [ 0xc7ac, 'HangulSyllableJae', [], 'Display', None],
  [ 0xc7ad, 'HangulSyllableJaeg', [], 'Display', None],
  [ 0xc7ae, 'HangulSyllableJaegg', [], 'Display', None],
  [ 0xc7af, 'HangulSyllableJaegs', [], 'Display', None],
  [ 0xc7b0, 'HangulSyllableJaen', [], 'Display', None],
  [ 0xc7b1, 'HangulSyllableJaenj', [], 'Display', None],
  [ 0xc7b2, 'HangulSyllableJaenh', [], 'Display', None],
  [ 0xc7b3, 'HangulSyllableJaed', [], 'Display', None],
  [ 0xc7b4, 'HangulSyllableJael', [], 'Display', None],
  [ 0xc7b5, 'HangulSyllableJaelg', [], 'Display', None],
  [ 0xc7b6, 'HangulSyllableJaelm', [], 'Display', None],
  [ 0xc7b7, 'HangulSyllableJaelb', [], 'Display', None],
  [ 0xc7b8, 'HangulSyllableJaels', [], 'Display', None],
  [ 0xc7b9, 'HangulSyllableJaelt', [], 'Display', None],
  [ 0xc7ba, 'HangulSyllableJaelp', [], 'Display', None],
  [ 0xc7bb, 'HangulSyllableJaelh', [], 'Display', None],
  [ 0xc7bc, 'HangulSyllableJaem', [], 'Display', None],
  [ 0xc7bd, 'HangulSyllableJaeb', [], 'Display', None],
  [ 0xc7be, 'HangulSyllableJaebs', [], 'Display', None],
  [ 0xc7bf, 'HangulSyllableJaes', [], 'Display', None],
  [ 0xc7c0, 'HangulSyllableJaess', [], 'Display', None],
  [ 0xc7c1, 'HangulSyllableJaeng', [], 'Display', None],
  [ 0xc7c2, 'HangulSyllableJaej', [], 'Display', None],
  [ 0xc7c3, 'HangulSyllableJaec', [], 'Display', None],
  [ 0xc7c4, 'HangulSyllableJaek', [], 'Display', None],
  [ 0xc7c5, 'HangulSyllableJaet', [], 'Display', None],
  [ 0xc7c6, 'HangulSyllableJaep', [], 'Display', None],
  [ 0xc7c7, 'HangulSyllableJaeh', [], 'Display', None],
  [ 0xc7c8, 'HangulSyllableJya', [], 'Display', None],
  [ 0xc7c9, 'HangulSyllableJyag', [], 'Display', None],
  [ 0xc7ca, 'HangulSyllableJyagg', [], 'Display', None],
  [ 0xc7cb, 'HangulSyllableJyags', [], 'Display', None],
  [ 0xc7cc, 'HangulSyllableJyan', [], 'Display', None],
  [ 0xc7cd, 'HangulSyllableJyanj', [], 'Display', None],
  [ 0xc7ce, 'HangulSyllableJyanh', [], 'Display', None],
  [ 0xc7cf, 'HangulSyllableJyad', [], 'Display', None],
  [ 0xc7d0, 'HangulSyllableJyal', [], 'Display', None],
  [ 0xc7d1, 'HangulSyllableJyalg', [], 'Display', None],
  [ 0xc7d2, 'HangulSyllableJyalm', [], 'Display', None],
  [ 0xc7d3, 'HangulSyllableJyalb', [], 'Display', None],
  [ 0xc7d4, 'HangulSyllableJyals', [], 'Display', None],
  [ 0xc7d5, 'HangulSyllableJyalt', [], 'Display', None],
  [ 0xc7d6, 'HangulSyllableJyalp', [], 'Display', None],
  [ 0xc7d7, 'HangulSyllableJyalh', [], 'Display', None],
  [ 0xc7d8, 'HangulSyllableJyam', [], 'Display', None],
  [ 0xc7d9, 'HangulSyllableJyab', [], 'Display', None],
  [ 0xc7da, 'HangulSyllableJyabs', [], 'Display', None],
  [ 0xc7db, 'HangulSyllableJyas', [], 'Display', None],
  [ 0xc7dc, 'HangulSyllableJyass', [], 'Display', None],
  [ 0xc7dd, 'HangulSyllableJyang', [], 'Display', None],
  [ 0xc7de, 'HangulSyllableJyaj', [], 'Display', None],
  [ 0xc7df, 'HangulSyllableJyac', [], 'Display', None],
  [ 0xc7e0, 'HangulSyllableJyak', [], 'Display', None],
  [ 0xc7e1, 'HangulSyllableJyat', [], 'Display', None],
  [ 0xc7e2, 'HangulSyllableJyap', [], 'Display', None],
  [ 0xc7e3, 'HangulSyllableJyah', [], 'Display', None],
  [ 0xc7e4, 'HangulSyllableJyae', [], 'Display', None],
  [ 0xc7e5, 'HangulSyllableJyaeg', [], 'Display', None],
  [ 0xc7e6, 'HangulSyllableJyaegg', [], 'Display', None],
  [ 0xc7e7, 'HangulSyllableJyaegs', [], 'Display', None],
  [ 0xc7e8, 'HangulSyllableJyaen', [], 'Display', None],
  [ 0xc7e9, 'HangulSyllableJyaenj', [], 'Display', None],
  [ 0xc7ea, 'HangulSyllableJyaenh', [], 'Display', None],
  [ 0xc7eb, 'HangulSyllableJyaed', [], 'Display', None],
  [ 0xc7ec, 'HangulSyllableJyael', [], 'Display', None],
  [ 0xc7ed, 'HangulSyllableJyaelg', [], 'Display', None],
  [ 0xc7ee, 'HangulSyllableJyaelm', [], 'Display', None],
  [ 0xc7ef, 'HangulSyllableJyaelb', [], 'Display', None],
  [ 0xc7f0, 'HangulSyllableJyaels', [], 'Display', None],
  [ 0xc7f1, 'HangulSyllableJyaelt', [], 'Display', None],
  [ 0xc7f2, 'HangulSyllableJyaelp', [], 'Display', None],
  [ 0xc7f3, 'HangulSyllableJyaelh', [], 'Display', None],
  [ 0xc7f4, 'HangulSyllableJyaem', [], 'Display', None],
  [ 0xc7f5, 'HangulSyllableJyaeb', [], 'Display', None],
  [ 0xc7f6, 'HangulSyllableJyaebs', [], 'Display', None],
  [ 0xc7f7, 'HangulSyllableJyaes', [], 'Display', None],
  [ 0xc7f8, 'HangulSyllableJyaess', [], 'Display', None],
  [ 0xc7f9, 'HangulSyllableJyaeng', [], 'Display', None],
  [ 0xc7fa, 'HangulSyllableJyaej', [], 'Display', None],
  [ 0xc7fb, 'HangulSyllableJyaec', [], 'Display', None],
  [ 0xc7fc, 'HangulSyllableJyaek', [], 'Display', None],
  [ 0xc7fd, 'HangulSyllableJyaet', [], 'Display', None],
  [ 0xc7fe, 'HangulSyllableJyaep', [], 'Display', None],
  [ 0xc7ff, 'HangulSyllableJyaeh', [], 'Display', None],
  [ 0xc800, 'HangulSyllableJeo', [], 'Display', None],
  [ 0xc801, 'HangulSyllableJeog', [], 'Display', None],
  [ 0xc802, 'HangulSyllableJeogg', [], 'Display', None],
  [ 0xc803, 'HangulSyllableJeogs', [], 'Display', None],
  [ 0xc804, 'HangulSyllableJeon', [], 'Display', None],
  [ 0xc805, 'HangulSyllableJeonj', [], 'Display', None],
  [ 0xc806, 'HangulSyllableJeonh', [], 'Display', None],
  [ 0xc807, 'HangulSyllableJeod', [], 'Display', None],
  [ 0xc808, 'HangulSyllableJeol', [], 'Display', None],
  [ 0xc809, 'HangulSyllableJeolg', [], 'Display', None],
  [ 0xc80a, 'HangulSyllableJeolm', [], 'Display', None],
  [ 0xc80b, 'HangulSyllableJeolb', [], 'Display', None],
  [ 0xc80c, 'HangulSyllableJeols', [], 'Display', None],
  [ 0xc80d, 'HangulSyllableJeolt', [], 'Display', None],
  [ 0xc80e, 'HangulSyllableJeolp', [], 'Display', None],
  [ 0xc80f, 'HangulSyllableJeolh', [], 'Display', None],
  [ 0xc810, 'HangulSyllableJeom', [], 'Display', None],
  [ 0xc811, 'HangulSyllableJeob', [], 'Display', None],
  [ 0xc812, 'HangulSyllableJeobs', [], 'Display', None],
  [ 0xc813, 'HangulSyllableJeos', [], 'Display', None],
  [ 0xc814, 'HangulSyllableJeoss', [], 'Display', None],
  [ 0xc815, 'HangulSyllableJeong', [], 'Display', None],
  [ 0xc816, 'HangulSyllableJeoj', [], 'Display', None],
  [ 0xc817, 'HangulSyllableJeoc', [], 'Display', None],
  [ 0xc818, 'HangulSyllableJeok', [], 'Display', None],
  [ 0xc819, 'HangulSyllableJeot', [], 'Display', None],
  [ 0xc81a, 'HangulSyllableJeop', [], 'Display', None],
  [ 0xc81b, 'HangulSyllableJeoh', [], 'Display', None],
  [ 0xc81c, 'HangulSyllableJe', [], 'Display', None],
  [ 0xc81d, 'HangulSyllableJeg', [], 'Display', None],
  [ 0xc81e, 'HangulSyllableJegg', [], 'Display', None],
  [ 0xc81f, 'HangulSyllableJegs', [], 'Display', None],
  [ 0xc820, 'HangulSyllableJen', [], 'Display', None],
  [ 0xc821, 'HangulSyllableJenj', [], 'Display', None],
  [ 0xc822, 'HangulSyllableJenh', [], 'Display', None],
  [ 0xc823, 'HangulSyllableJed', [], 'Display', None],
  [ 0xc824, 'HangulSyllableJel', [], 'Display', None],
  [ 0xc825, 'HangulSyllableJelg', [], 'Display', None],
  [ 0xc826, 'HangulSyllableJelm', [], 'Display', None],
  [ 0xc827, 'HangulSyllableJelb', [], 'Display', None],
  [ 0xc828, 'HangulSyllableJels', [], 'Display', None],
  [ 0xc829, 'HangulSyllableJelt', [], 'Display', None],
  [ 0xc82a, 'HangulSyllableJelp', [], 'Display', None],
  [ 0xc82b, 'HangulSyllableJelh', [], 'Display', None],
  [ 0xc82c, 'HangulSyllableJem', [], 'Display', None],
  [ 0xc82d, 'HangulSyllableJeb', [], 'Display', None],
  [ 0xc82e, 'HangulSyllableJebs', [], 'Display', None],
  [ 0xc82f, 'HangulSyllableJes', [], 'Display', None],
  [ 0xc830, 'HangulSyllableJess', [], 'Display', None],
  [ 0xc831, 'HangulSyllableJeng', [], 'Display', None],
  [ 0xc832, 'HangulSyllableJej', [], 'Display', None],
  [ 0xc833, 'HangulSyllableJec', [], 'Display', None],
  [ 0xc834, 'HangulSyllableJek', [], 'Display', None],
  [ 0xc835, 'HangulSyllableJet', [], 'Display', None],
  [ 0xc836, 'HangulSyllableJep', [], 'Display', None],
  [ 0xc837, 'HangulSyllableJeh', [], 'Display', None],
  [ 0xc838, 'HangulSyllableJyeo', [], 'Display', None],
  [ 0xc839, 'HangulSyllableJyeog', [], 'Display', None],
  [ 0xc83a, 'HangulSyllableJyeogg', [], 'Display', None],
  [ 0xc83b, 'HangulSyllableJyeogs', [], 'Display', None],
  [ 0xc83c, 'HangulSyllableJyeon', [], 'Display', None],
  [ 0xc83d, 'HangulSyllableJyeonj', [], 'Display', None],
  [ 0xc83e, 'HangulSyllableJyeonh', [], 'Display', None],
  [ 0xc83f, 'HangulSyllableJyeod', [], 'Display', None],
  [ 0xc840, 'HangulSyllableJyeol', [], 'Display', None],
  [ 0xc841, 'HangulSyllableJyeolg', [], 'Display', None],
  [ 0xc842, 'HangulSyllableJyeolm', [], 'Display', None],
  [ 0xc843, 'HangulSyllableJyeolb', [], 'Display', None],
  [ 0xc844, 'HangulSyllableJyeols', [], 'Display', None],
  [ 0xc845, 'HangulSyllableJyeolt', [], 'Display', None],
  [ 0xc846, 'HangulSyllableJyeolp', [], 'Display', None],
  [ 0xc847, 'HangulSyllableJyeolh', [], 'Display', None],
  [ 0xc848, 'HangulSyllableJyeom', [], 'Display', None],
  [ 0xc849, 'HangulSyllableJyeob', [], 'Display', None],
  [ 0xc84a, 'HangulSyllableJyeobs', [], 'Display', None],
  [ 0xc84b, 'HangulSyllableJyeos', [], 'Display', None],
  [ 0xc84c, 'HangulSyllableJyeoss', [], 'Display', None],
  [ 0xc84d, 'HangulSyllableJyeong', [], 'Display', None],
  [ 0xc84e, 'HangulSyllableJyeoj', [], 'Display', None],
  [ 0xc84f, 'HangulSyllableJyeoc', [], 'Display', None],
  [ 0xc850, 'HangulSyllableJyeok', [], 'Display', None],
  [ 0xc851, 'HangulSyllableJyeot', [], 'Display', None],
  [ 0xc852, 'HangulSyllableJyeop', [], 'Display', None],
  [ 0xc853, 'HangulSyllableJyeoh', [], 'Display', None],
  [ 0xc854, 'HangulSyllableJye', [], 'Display', None],
  [ 0xc855, 'HangulSyllableJyeg', [], 'Display', None],
  [ 0xc856, 'HangulSyllableJyegg', [], 'Display', None],
  [ 0xc857, 'HangulSyllableJyegs', [], 'Display', None],
  [ 0xc858, 'HangulSyllableJyen', [], 'Display', None],
  [ 0xc859, 'HangulSyllableJyenj', [], 'Display', None],
  [ 0xc85a, 'HangulSyllableJyenh', [], 'Display', None],
  [ 0xc85b, 'HangulSyllableJyed', [], 'Display', None],
  [ 0xc85c, 'HangulSyllableJyel', [], 'Display', None],
  [ 0xc85d, 'HangulSyllableJyelg', [], 'Display', None],
  [ 0xc85e, 'HangulSyllableJyelm', [], 'Display', None],
  [ 0xc85f, 'HangulSyllableJyelb', [], 'Display', None],
  [ 0xc860, 'HangulSyllableJyels', [], 'Display', None],
  [ 0xc861, 'HangulSyllableJyelt', [], 'Display', None],
  [ 0xc862, 'HangulSyllableJyelp', [], 'Display', None],
  [ 0xc863, 'HangulSyllableJyelh', [], 'Display', None],
  [ 0xc864, 'HangulSyllableJyem', [], 'Display', None],
  [ 0xc865, 'HangulSyllableJyeb', [], 'Display', None],
  [ 0xc866, 'HangulSyllableJyebs', [], 'Display', None],
  [ 0xc867, 'HangulSyllableJyes', [], 'Display', None],
  [ 0xc868, 'HangulSyllableJyess', [], 'Display', None],
  [ 0xc869, 'HangulSyllableJyeng', [], 'Display', None],
  [ 0xc86a, 'HangulSyllableJyej', [], 'Display', None],
  [ 0xc86b, 'HangulSyllableJyec', [], 'Display', None],
  [ 0xc86c, 'HangulSyllableJyek', [], 'Display', None],
  [ 0xc86d, 'HangulSyllableJyet', [], 'Display', None],
  [ 0xc86e, 'HangulSyllableJyep', [], 'Display', None],
  [ 0xc86f, 'HangulSyllableJyeh', [], 'Display', None],
  [ 0xc870, 'HangulSyllableJo', [], 'Display', None],
  [ 0xc871, 'HangulSyllableJog', [], 'Display', None],
  [ 0xc872, 'HangulSyllableJogg', [], 'Display', None],
  [ 0xc873, 'HangulSyllableJogs', [], 'Display', None],
  [ 0xc874, 'HangulSyllableJon', [], 'Display', None],
  [ 0xc875, 'HangulSyllableJonj', [], 'Display', None],
  [ 0xc876, 'HangulSyllableJonh', [], 'Display', None],
  [ 0xc877, 'HangulSyllableJod', [], 'Display', None],
  [ 0xc878, 'HangulSyllableJol', [], 'Display', None],
  [ 0xc879, 'HangulSyllableJolg', [], 'Display', None],
  [ 0xc87a, 'HangulSyllableJolm', [], 'Display', None],
  [ 0xc87b, 'HangulSyllableJolb', [], 'Display', None],
  [ 0xc87c, 'HangulSyllableJols', [], 'Display', None],
  [ 0xc87d, 'HangulSyllableJolt', [], 'Display', None],
  [ 0xc87e, 'HangulSyllableJolp', [], 'Display', None],
  [ 0xc87f, 'HangulSyllableJolh', [], 'Display', None],
  [ 0xc880, 'HangulSyllableJom', [], 'Display', None],
  [ 0xc881, 'HangulSyllableJob', [], 'Display', None],
  [ 0xc882, 'HangulSyllableJobs', [], 'Display', None],
  [ 0xc883, 'HangulSyllableJos', [], 'Display', None],
  [ 0xc884, 'HangulSyllableJoss', [], 'Display', None],
  [ 0xc885, 'HangulSyllableJong', [], 'Display', None],
  [ 0xc886, 'HangulSyllableJoj', [], 'Display', None],
  [ 0xc887, 'HangulSyllableJoc', [], 'Display', None],
  [ 0xc888, 'HangulSyllableJok', [], 'Display', None],
  [ 0xc889, 'HangulSyllableJot', [], 'Display', None],
  [ 0xc88a, 'HangulSyllableJop', [], 'Display', None],
  [ 0xc88b, 'HangulSyllableJoh', [], 'Display', None],
  [ 0xc88c, 'HangulSyllableJwa', [], 'Display', None],
  [ 0xc88d, 'HangulSyllableJwag', [], 'Display', None],
  [ 0xc88e, 'HangulSyllableJwagg', [], 'Display', None],
  [ 0xc88f, 'HangulSyllableJwags', [], 'Display', None],
  [ 0xc890, 'HangulSyllableJwan', [], 'Display', None],
  [ 0xc891, 'HangulSyllableJwanj', [], 'Display', None],
  [ 0xc892, 'HangulSyllableJwanh', [], 'Display', None],
  [ 0xc893, 'HangulSyllableJwad', [], 'Display', None],
  [ 0xc894, 'HangulSyllableJwal', [], 'Display', None],
  [ 0xc895, 'HangulSyllableJwalg', [], 'Display', None],
  [ 0xc896, 'HangulSyllableJwalm', [], 'Display', None],
  [ 0xc897, 'HangulSyllableJwalb', [], 'Display', None],
  [ 0xc898, 'HangulSyllableJwals', [], 'Display', None],
  [ 0xc899, 'HangulSyllableJwalt', [], 'Display', None],
  [ 0xc89a, 'HangulSyllableJwalp', [], 'Display', None],
  [ 0xc89b, 'HangulSyllableJwalh', [], 'Display', None],
  [ 0xc89c, 'HangulSyllableJwam', [], 'Display', None],
  [ 0xc89d, 'HangulSyllableJwab', [], 'Display', None],
  [ 0xc89e, 'HangulSyllableJwabs', [], 'Display', None],
  [ 0xc89f, 'HangulSyllableJwas', [], 'Display', None],
  [ 0xc8a0, 'HangulSyllableJwass', [], 'Display', None],
  [ 0xc8a1, 'HangulSyllableJwang', [], 'Display', None],
  [ 0xc8a2, 'HangulSyllableJwaj', [], 'Display', None],
  [ 0xc8a3, 'HangulSyllableJwac', [], 'Display', None],
  [ 0xc8a4, 'HangulSyllableJwak', [], 'Display', None],
  [ 0xc8a5, 'HangulSyllableJwat', [], 'Display', None],
  [ 0xc8a6, 'HangulSyllableJwap', [], 'Display', None],
  [ 0xc8a7, 'HangulSyllableJwah', [], 'Display', None],
  [ 0xc8a8, 'HangulSyllableJwae', [], 'Display', None],
  [ 0xc8a9, 'HangulSyllableJwaeg', [], 'Display', None],
  [ 0xc8aa, 'HangulSyllableJwaegg', [], 'Display', None],
  [ 0xc8ab, 'HangulSyllableJwaegs', [], 'Display', None],
  [ 0xc8ac, 'HangulSyllableJwaen', [], 'Display', None],
  [ 0xc8ad, 'HangulSyllableJwaenj', [], 'Display', None],
  [ 0xc8ae, 'HangulSyllableJwaenh', [], 'Display', None],
  [ 0xc8af, 'HangulSyllableJwaed', [], 'Display', None],
  [ 0xc8b0, 'HangulSyllableJwael', [], 'Display', None],
  [ 0xc8b1, 'HangulSyllableJwaelg', [], 'Display', None],
  [ 0xc8b2, 'HangulSyllableJwaelm', [], 'Display', None],
  [ 0xc8b3, 'HangulSyllableJwaelb', [], 'Display', None],
  [ 0xc8b4, 'HangulSyllableJwaels', [], 'Display', None],
  [ 0xc8b5, 'HangulSyllableJwaelt', [], 'Display', None],
  [ 0xc8b6, 'HangulSyllableJwaelp', [], 'Display', None],
  [ 0xc8b7, 'HangulSyllableJwaelh', [], 'Display', None],
  [ 0xc8b8, 'HangulSyllableJwaem', [], 'Display', None],
  [ 0xc8b9, 'HangulSyllableJwaeb', [], 'Display', None],
  [ 0xc8ba, 'HangulSyllableJwaebs', [], 'Display', None],
  [ 0xc8bb, 'HangulSyllableJwaes', [], 'Display', None],
  [ 0xc8bc, 'HangulSyllableJwaess', [], 'Display', None],
  [ 0xc8bd, 'HangulSyllableJwaeng', [], 'Display', None],
  [ 0xc8be, 'HangulSyllableJwaej', [], 'Display', None],
  [ 0xc8bf, 'HangulSyllableJwaec', [], 'Display', None],
  [ 0xc8c0, 'HangulSyllableJwaek', [], 'Display', None],
  [ 0xc8c1, 'HangulSyllableJwaet', [], 'Display', None],
  [ 0xc8c2, 'HangulSyllableJwaep', [], 'Display', None],
  [ 0xc8c3, 'HangulSyllableJwaeh', [], 'Display', None],
  [ 0xc8c4, 'HangulSyllableJoe', [], 'Display', None],
  [ 0xc8c5, 'HangulSyllableJoeg', [], 'Display', None],
  [ 0xc8c6, 'HangulSyllableJoegg', [], 'Display', None],
  [ 0xc8c7, 'HangulSyllableJoegs', [], 'Display', None],
  [ 0xc8c8, 'HangulSyllableJoen', [], 'Display', None],
  [ 0xc8c9, 'HangulSyllableJoenj', [], 'Display', None],
  [ 0xc8ca, 'HangulSyllableJoenh', [], 'Display', None],
  [ 0xc8cb, 'HangulSyllableJoed', [], 'Display', None],
  [ 0xc8cc, 'HangulSyllableJoel', [], 'Display', None],
  [ 0xc8cd, 'HangulSyllableJoelg', [], 'Display', None],
  [ 0xc8ce, 'HangulSyllableJoelm', [], 'Display', None],
  [ 0xc8cf, 'HangulSyllableJoelb', [], 'Display', None],
  [ 0xc8d0, 'HangulSyllableJoels', [], 'Display', None],
  [ 0xc8d1, 'HangulSyllableJoelt', [], 'Display', None],
  [ 0xc8d2, 'HangulSyllableJoelp', [], 'Display', None],
  [ 0xc8d3, 'HangulSyllableJoelh', [], 'Display', None],
  [ 0xc8d4, 'HangulSyllableJoem', [], 'Display', None],
  [ 0xc8d5, 'HangulSyllableJoeb', [], 'Display', None],
  [ 0xc8d6, 'HangulSyllableJoebs', [], 'Display', None],
  [ 0xc8d7, 'HangulSyllableJoes', [], 'Display', None],
  [ 0xc8d8, 'HangulSyllableJoess', [], 'Display', None],
  [ 0xc8d9, 'HangulSyllableJoeng', [], 'Display', None],
  [ 0xc8da, 'HangulSyllableJoej', [], 'Display', None],
  [ 0xc8db, 'HangulSyllableJoec', [], 'Display', None],
  [ 0xc8dc, 'HangulSyllableJoek', [], 'Display', None],
  [ 0xc8dd, 'HangulSyllableJoet', [], 'Display', None],
  [ 0xc8de, 'HangulSyllableJoep', [], 'Display', None],
  [ 0xc8df, 'HangulSyllableJoeh', [], 'Display', None],
  [ 0xc8e0, 'HangulSyllableJyo', [], 'Display', None],
  [ 0xc8e1, 'HangulSyllableJyog', [], 'Display', None],
  [ 0xc8e2, 'HangulSyllableJyogg', [], 'Display', None],
  [ 0xc8e3, 'HangulSyllableJyogs', [], 'Display', None],
  [ 0xc8e4, 'HangulSyllableJyon', [], 'Display', None],
  [ 0xc8e5, 'HangulSyllableJyonj', [], 'Display', None],
  [ 0xc8e6, 'HangulSyllableJyonh', [], 'Display', None],
  [ 0xc8e7, 'HangulSyllableJyod', [], 'Display', None],
  [ 0xc8e8, 'HangulSyllableJyol', [], 'Display', None],
  [ 0xc8e9, 'HangulSyllableJyolg', [], 'Display', None],
  [ 0xc8ea, 'HangulSyllableJyolm', [], 'Display', None],
  [ 0xc8eb, 'HangulSyllableJyolb', [], 'Display', None],
  [ 0xc8ec, 'HangulSyllableJyols', [], 'Display', None],
  [ 0xc8ed, 'HangulSyllableJyolt', [], 'Display', None],
  [ 0xc8ee, 'HangulSyllableJyolp', [], 'Display', None],
  [ 0xc8ef, 'HangulSyllableJyolh', [], 'Display', None],
  [ 0xc8f0, 'HangulSyllableJyom', [], 'Display', None],
  [ 0xc8f1, 'HangulSyllableJyob', [], 'Display', None],
  [ 0xc8f2, 'HangulSyllableJyobs', [], 'Display', None],
  [ 0xc8f3, 'HangulSyllableJyos', [], 'Display', None],
  [ 0xc8f4, 'HangulSyllableJyoss', [], 'Display', None],
  [ 0xc8f5, 'HangulSyllableJyong', [], 'Display', None],
  [ 0xc8f6, 'HangulSyllableJyoj', [], 'Display', None],
  [ 0xc8f7, 'HangulSyllableJyoc', [], 'Display', None],
  [ 0xc8f8, 'HangulSyllableJyok', [], 'Display', None],
  [ 0xc8f9, 'HangulSyllableJyot', [], 'Display', None],
  [ 0xc8fa, 'HangulSyllableJyop', [], 'Display', None],
  [ 0xc8fb, 'HangulSyllableJyoh', [], 'Display', None],
  [ 0xc8fc, 'HangulSyllableJu', [], 'Display', None],
  [ 0xc8fd, 'HangulSyllableJug', [], 'Display', None],
  [ 0xc8fe, 'HangulSyllableJugg', [], 'Display', None],
  [ 0xc8ff, 'HangulSyllableJugs', [], 'Display', None],
  [ 0xc900, 'HangulSyllableJun', [], 'Display', None],
  [ 0xc901, 'HangulSyllableJunj', [], 'Display', None],
  [ 0xc902, 'HangulSyllableJunh', [], 'Display', None],
  [ 0xc903, 'HangulSyllableJud', [], 'Display', None],
  [ 0xc904, 'HangulSyllableJul', [], 'Display', None],
  [ 0xc905, 'HangulSyllableJulg', [], 'Display', None],
  [ 0xc906, 'HangulSyllableJulm', [], 'Display', None],
  [ 0xc907, 'HangulSyllableJulb', [], 'Display', None],
  [ 0xc908, 'HangulSyllableJuls', [], 'Display', None],
  [ 0xc909, 'HangulSyllableJult', [], 'Display', None],
  [ 0xc90a, 'HangulSyllableJulp', [], 'Display', None],
  [ 0xc90b, 'HangulSyllableJulh', [], 'Display', None],
  [ 0xc90c, 'HangulSyllableJum', [], 'Display', None],
  [ 0xc90d, 'HangulSyllableJub', [], 'Display', None],
  [ 0xc90e, 'HangulSyllableJubs', [], 'Display', None],
  [ 0xc90f, 'HangulSyllableJus', [], 'Display', None],
  [ 0xc910, 'HangulSyllableJuss', [], 'Display', None],
  [ 0xc911, 'HangulSyllableJung', [], 'Display', None],
  [ 0xc912, 'HangulSyllableJuj', [], 'Display', None],
  [ 0xc913, 'HangulSyllableJuc', [], 'Display', None],
  [ 0xc914, 'HangulSyllableJuk', [], 'Display', None],
  [ 0xc915, 'HangulSyllableJut', [], 'Display', None],
  [ 0xc916, 'HangulSyllableJup', [], 'Display', None],
  [ 0xc917, 'HangulSyllableJuh', [], 'Display', None],
  [ 0xc918, 'HangulSyllableJweo', [], 'Display', None],
  [ 0xc919, 'HangulSyllableJweog', [], 'Display', None],
  [ 0xc91a, 'HangulSyllableJweogg', [], 'Display', None],
  [ 0xc91b, 'HangulSyllableJweogs', [], 'Display', None],
  [ 0xc91c, 'HangulSyllableJweon', [], 'Display', None],
  [ 0xc91d, 'HangulSyllableJweonj', [], 'Display', None],
  [ 0xc91e, 'HangulSyllableJweonh', [], 'Display', None],
  [ 0xc91f, 'HangulSyllableJweod', [], 'Display', None],
  [ 0xc920, 'HangulSyllableJweol', [], 'Display', None],
  [ 0xc921, 'HangulSyllableJweolg', [], 'Display', None],
  [ 0xc922, 'HangulSyllableJweolm', [], 'Display', None],
  [ 0xc923, 'HangulSyllableJweolb', [], 'Display', None],
  [ 0xc924, 'HangulSyllableJweols', [], 'Display', None],
  [ 0xc925, 'HangulSyllableJweolt', [], 'Display', None],
  [ 0xc926, 'HangulSyllableJweolp', [], 'Display', None],
  [ 0xc927, 'HangulSyllableJweolh', [], 'Display', None],
  [ 0xc928, 'HangulSyllableJweom', [], 'Display', None],
  [ 0xc929, 'HangulSyllableJweob', [], 'Display', None],
  [ 0xc92a, 'HangulSyllableJweobs', [], 'Display', None],
  [ 0xc92b, 'HangulSyllableJweos', [], 'Display', None],
  [ 0xc92c, 'HangulSyllableJweoss', [], 'Display', None],
  [ 0xc92d, 'HangulSyllableJweong', [], 'Display', None],
  [ 0xc92e, 'HangulSyllableJweoj', [], 'Display', None],
  [ 0xc92f, 'HangulSyllableJweoc', [], 'Display', None],
  [ 0xc930, 'HangulSyllableJweok', [], 'Display', None],
  [ 0xc931, 'HangulSyllableJweot', [], 'Display', None],
  [ 0xc932, 'HangulSyllableJweop', [], 'Display', None],
  [ 0xc933, 'HangulSyllableJweoh', [], 'Display', None],
  [ 0xc934, 'HangulSyllableJwe', [], 'Display', None],
  [ 0xc935, 'HangulSyllableJweg', [], 'Display', None],
  [ 0xc936, 'HangulSyllableJwegg', [], 'Display', None],
  [ 0xc937, 'HangulSyllableJwegs', [], 'Display', None],
  [ 0xc938, 'HangulSyllableJwen', [], 'Display', None],
  [ 0xc939, 'HangulSyllableJwenj', [], 'Display', None],
  [ 0xc93a, 'HangulSyllableJwenh', [], 'Display', None],
  [ 0xc93b, 'HangulSyllableJwed', [], 'Display', None],
  [ 0xc93c, 'HangulSyllableJwel', [], 'Display', None],
  [ 0xc93d, 'HangulSyllableJwelg', [], 'Display', None],
  [ 0xc93e, 'HangulSyllableJwelm', [], 'Display', None],
  [ 0xc93f, 'HangulSyllableJwelb', [], 'Display', None],
  [ 0xc940, 'HangulSyllableJwels', [], 'Display', None],
  [ 0xc941, 'HangulSyllableJwelt', [], 'Display', None],
  [ 0xc942, 'HangulSyllableJwelp', [], 'Display', None],
  [ 0xc943, 'HangulSyllableJwelh', [], 'Display', None],
  [ 0xc944, 'HangulSyllableJwem', [], 'Display', None],
  [ 0xc945, 'HangulSyllableJweb', [], 'Display', None],
  [ 0xc946, 'HangulSyllableJwebs', [], 'Display', None],
  [ 0xc947, 'HangulSyllableJwes', [], 'Display', None],
  [ 0xc948, 'HangulSyllableJwess', [], 'Display', None],
  [ 0xc949, 'HangulSyllableJweng', [], 'Display', None],
  [ 0xc94a, 'HangulSyllableJwej', [], 'Display', None],
  [ 0xc94b, 'HangulSyllableJwec', [], 'Display', None],
  [ 0xc94c, 'HangulSyllableJwek', [], 'Display', None],
  [ 0xc94d, 'HangulSyllableJwet', [], 'Display', None],
  [ 0xc94e, 'HangulSyllableJwep', [], 'Display', None],
  [ 0xc94f, 'HangulSyllableJweh', [], 'Display', None],
  [ 0xc950, 'HangulSyllableJwi', [], 'Display', None],
  [ 0xc951, 'HangulSyllableJwig', [], 'Display', None],
  [ 0xc952, 'HangulSyllableJwigg', [], 'Display', None],
  [ 0xc953, 'HangulSyllableJwigs', [], 'Display', None],
  [ 0xc954, 'HangulSyllableJwin', [], 'Display', None],
  [ 0xc955, 'HangulSyllableJwinj', [], 'Display', None],
  [ 0xc956, 'HangulSyllableJwinh', [], 'Display', None],
  [ 0xc957, 'HangulSyllableJwid', [], 'Display', None],
  [ 0xc958, 'HangulSyllableJwil', [], 'Display', None],
  [ 0xc959, 'HangulSyllableJwilg', [], 'Display', None],
  [ 0xc95a, 'HangulSyllableJwilm', [], 'Display', None],
  [ 0xc95b, 'HangulSyllableJwilb', [], 'Display', None],
  [ 0xc95c, 'HangulSyllableJwils', [], 'Display', None],
  [ 0xc95d, 'HangulSyllableJwilt', [], 'Display', None],
  [ 0xc95e, 'HangulSyllableJwilp', [], 'Display', None],
  [ 0xc95f, 'HangulSyllableJwilh', [], 'Display', None],
  [ 0xc960, 'HangulSyllableJwim', [], 'Display', None],
  [ 0xc961, 'HangulSyllableJwib', [], 'Display', None],
  [ 0xc962, 'HangulSyllableJwibs', [], 'Display', None],
  [ 0xc963, 'HangulSyllableJwis', [], 'Display', None],
  [ 0xc964, 'HangulSyllableJwiss', [], 'Display', None],
  [ 0xc965, 'HangulSyllableJwing', [], 'Display', None],
  [ 0xc966, 'HangulSyllableJwij', [], 'Display', None],
  [ 0xc967, 'HangulSyllableJwic', [], 'Display', None],
  [ 0xc968, 'HangulSyllableJwik', [], 'Display', None],
  [ 0xc969, 'HangulSyllableJwit', [], 'Display', None],
  [ 0xc96a, 'HangulSyllableJwip', [], 'Display', None],
  [ 0xc96b, 'HangulSyllableJwih', [], 'Display', None],
  [ 0xc96c, 'HangulSyllableJyu', [], 'Display', None],
  [ 0xc96d, 'HangulSyllableJyug', [], 'Display', None],
  [ 0xc96e, 'HangulSyllableJyugg', [], 'Display', None],
  [ 0xc96f, 'HangulSyllableJyugs', [], 'Display', None],
  [ 0xc970, 'HangulSyllableJyun', [], 'Display', None],
  [ 0xc971, 'HangulSyllableJyunj', [], 'Display', None],
  [ 0xc972, 'HangulSyllableJyunh', [], 'Display', None],
  [ 0xc973, 'HangulSyllableJyud', [], 'Display', None],
  [ 0xc974, 'HangulSyllableJyul', [], 'Display', None],
  [ 0xc975, 'HangulSyllableJyulg', [], 'Display', None],
  [ 0xc976, 'HangulSyllableJyulm', [], 'Display', None],
  [ 0xc977, 'HangulSyllableJyulb', [], 'Display', None],
  [ 0xc978, 'HangulSyllableJyuls', [], 'Display', None],
  [ 0xc979, 'HangulSyllableJyult', [], 'Display', None],
  [ 0xc97a, 'HangulSyllableJyulp', [], 'Display', None],
  [ 0xc97b, 'HangulSyllableJyulh', [], 'Display', None],
  [ 0xc97c, 'HangulSyllableJyum', [], 'Display', None],
  [ 0xc97d, 'HangulSyllableJyub', [], 'Display', None],
  [ 0xc97e, 'HangulSyllableJyubs', [], 'Display', None],
  [ 0xc97f, 'HangulSyllableJyus', [], 'Display', None],
  [ 0xc980, 'HangulSyllableJyuss', [], 'Display', None],
  [ 0xc981, 'HangulSyllableJyung', [], 'Display', None],
  [ 0xc982, 'HangulSyllableJyuj', [], 'Display', None],
  [ 0xc983, 'HangulSyllableJyuc', [], 'Display', None],
  [ 0xc984, 'HangulSyllableJyuk', [], 'Display', None],
  [ 0xc985, 'HangulSyllableJyut', [], 'Display', None],
  [ 0xc986, 'HangulSyllableJyup', [], 'Display', None],
  [ 0xc987, 'HangulSyllableJyuh', [], 'Display', None],
  [ 0xc988, 'HangulSyllableJeu', [], 'Display', None],
  [ 0xc989, 'HangulSyllableJeug', [], 'Display', None],
  [ 0xc98a, 'HangulSyllableJeugg', [], 'Display', None],
  [ 0xc98b, 'HangulSyllableJeugs', [], 'Display', None],
  [ 0xc98c, 'HangulSyllableJeun', [], 'Display', None],
  [ 0xc98d, 'HangulSyllableJeunj', [], 'Display', None],
  [ 0xc98e, 'HangulSyllableJeunh', [], 'Display', None],
  [ 0xc98f, 'HangulSyllableJeud', [], 'Display', None],
  [ 0xc990, 'HangulSyllableJeul', [], 'Display', None],
  [ 0xc991, 'HangulSyllableJeulg', [], 'Display', None],
  [ 0xc992, 'HangulSyllableJeulm', [], 'Display', None],
  [ 0xc993, 'HangulSyllableJeulb', [], 'Display', None],
  [ 0xc994, 'HangulSyllableJeuls', [], 'Display', None],
  [ 0xc995, 'HangulSyllableJeult', [], 'Display', None],
  [ 0xc996, 'HangulSyllableJeulp', [], 'Display', None],
  [ 0xc997, 'HangulSyllableJeulh', [], 'Display', None],
  [ 0xc998, 'HangulSyllableJeum', [], 'Display', None],
  [ 0xc999, 'HangulSyllableJeub', [], 'Display', None],
  [ 0xc99a, 'HangulSyllableJeubs', [], 'Display', None],
  [ 0xc99b, 'HangulSyllableJeus', [], 'Display', None],
  [ 0xc99c, 'HangulSyllableJeuss', [], 'Display', None],
  [ 0xc99d, 'HangulSyllableJeung', [], 'Display', None],
  [ 0xc99e, 'HangulSyllableJeuj', [], 'Display', None],
  [ 0xc99f, 'HangulSyllableJeuc', [], 'Display', None],
  [ 0xc9a0, 'HangulSyllableJeuk', [], 'Display', None],
  [ 0xc9a1, 'HangulSyllableJeut', [], 'Display', None],
  [ 0xc9a2, 'HangulSyllableJeup', [], 'Display', None],
  [ 0xc9a3, 'HangulSyllableJeuh', [], 'Display', None],
  [ 0xc9a4, 'HangulSyllableJyi', [], 'Display', None],
  [ 0xc9a5, 'HangulSyllableJyig', [], 'Display', None],
  [ 0xc9a6, 'HangulSyllableJyigg', [], 'Display', None],
  [ 0xc9a7, 'HangulSyllableJyigs', [], 'Display', None],
  [ 0xc9a8, 'HangulSyllableJyin', [], 'Display', None],
  [ 0xc9a9, 'HangulSyllableJyinj', [], 'Display', None],
  [ 0xc9aa, 'HangulSyllableJyinh', [], 'Display', None],
  [ 0xc9ab, 'HangulSyllableJyid', [], 'Display', None],
  [ 0xc9ac, 'HangulSyllableJyil', [], 'Display', None],
  [ 0xc9ad, 'HangulSyllableJyilg', [], 'Display', None],
  [ 0xc9ae, 'HangulSyllableJyilm', [], 'Display', None],
  [ 0xc9af, 'HangulSyllableJyilb', [], 'Display', None],
  [ 0xc9b0, 'HangulSyllableJyils', [], 'Display', None],
  [ 0xc9b1, 'HangulSyllableJyilt', [], 'Display', None],
  [ 0xc9b2, 'HangulSyllableJyilp', [], 'Display', None],
  [ 0xc9b3, 'HangulSyllableJyilh', [], 'Display', None],
  [ 0xc9b4, 'HangulSyllableJyim', [], 'Display', None],
  [ 0xc9b5, 'HangulSyllableJyib', [], 'Display', None],
  [ 0xc9b6, 'HangulSyllableJyibs', [], 'Display', None],
  [ 0xc9b7, 'HangulSyllableJyis', [], 'Display', None],
  [ 0xc9b8, 'HangulSyllableJyiss', [], 'Display', None],
  [ 0xc9b9, 'HangulSyllableJying', [], 'Display', None],
  [ 0xc9ba, 'HangulSyllableJyij', [], 'Display', None],
  [ 0xc9bb, 'HangulSyllableJyic', [], 'Display', None],
  [ 0xc9bc, 'HangulSyllableJyik', [], 'Display', None],
  [ 0xc9bd, 'HangulSyllableJyit', [], 'Display', None],
  [ 0xc9be, 'HangulSyllableJyip', [], 'Display', None],
  [ 0xc9bf, 'HangulSyllableJyih', [], 'Display', None],
  [ 0xc9c0, 'HangulSyllableJi', [], 'Display', None],
  [ 0xc9c1, 'HangulSyllableJig', [], 'Display', None],
  [ 0xc9c2, 'HangulSyllableJigg', [], 'Display', None],
  [ 0xc9c3, 'HangulSyllableJigs', [], 'Display', None],
  [ 0xc9c4, 'HangulSyllableJin', [], 'Display', None],
  [ 0xc9c5, 'HangulSyllableJinj', [], 'Display', None],
  [ 0xc9c6, 'HangulSyllableJinh', [], 'Display', None],
  [ 0xc9c7, 'HangulSyllableJid', [], 'Display', None],
  [ 0xc9c8, 'HangulSyllableJil', [], 'Display', None],
  [ 0xc9c9, 'HangulSyllableJilg', [], 'Display', None],
  [ 0xc9ca, 'HangulSyllableJilm', [], 'Display', None],
  [ 0xc9cb, 'HangulSyllableJilb', [], 'Display', None],
  [ 0xc9cc, 'HangulSyllableJils', [], 'Display', None],
  [ 0xc9cd, 'HangulSyllableJilt', [], 'Display', None],
  [ 0xc9ce, 'HangulSyllableJilp', [], 'Display', None],
  [ 0xc9cf, 'HangulSyllableJilh', [], 'Display', None],
  [ 0xc9d0, 'HangulSyllableJim', [], 'Display', None],
  [ 0xc9d1, 'HangulSyllableJib', [], 'Display', None],
  [ 0xc9d2, 'HangulSyllableJibs', [], 'Display', None],
  [ 0xc9d3, 'HangulSyllableJis', [], 'Display', None],
  [ 0xc9d4, 'HangulSyllableJiss', [], 'Display', None],
  [ 0xc9d5, 'HangulSyllableJing', [], 'Display', None],
  [ 0xc9d6, 'HangulSyllableJij', [], 'Display', None],
  [ 0xc9d7, 'HangulSyllableJic', [], 'Display', None],
  [ 0xc9d8, 'HangulSyllableJik', [], 'Display', None],
  [ 0xc9d9, 'HangulSyllableJit', [], 'Display', None],
  [ 0xc9da, 'HangulSyllableJip', [], 'Display', None],
  [ 0xc9db, 'HangulSyllableJih', [], 'Display', None],
  [ 0xc9dc, 'HangulSyllableJja', [], 'Display', None],
  [ 0xc9dd, 'HangulSyllableJjag', [], 'Display', None],
  [ 0xc9de, 'HangulSyllableJjagg', [], 'Display', None],
  [ 0xc9df, 'HangulSyllableJjags', [], 'Display', None],
  [ 0xc9e0, 'HangulSyllableJjan', [], 'Display', None],
  [ 0xc9e1, 'HangulSyllableJjanj', [], 'Display', None],
  [ 0xc9e2, 'HangulSyllableJjanh', [], 'Display', None],
  [ 0xc9e3, 'HangulSyllableJjad', [], 'Display', None],
  [ 0xc9e4, 'HangulSyllableJjal', [], 'Display', None],
  [ 0xc9e5, 'HangulSyllableJjalg', [], 'Display', None],
  [ 0xc9e6, 'HangulSyllableJjalm', [], 'Display', None],
  [ 0xc9e7, 'HangulSyllableJjalb', [], 'Display', None],
  [ 0xc9e8, 'HangulSyllableJjals', [], 'Display', None],
  [ 0xc9e9, 'HangulSyllableJjalt', [], 'Display', None],
  [ 0xc9ea, 'HangulSyllableJjalp', [], 'Display', None],
  [ 0xc9eb, 'HangulSyllableJjalh', [], 'Display', None],
  [ 0xc9ec, 'HangulSyllableJjam', [], 'Display', None],
  [ 0xc9ed, 'HangulSyllableJjab', [], 'Display', None],
  [ 0xc9ee, 'HangulSyllableJjabs', [], 'Display', None],
  [ 0xc9ef, 'HangulSyllableJjas', [], 'Display', None],
  [ 0xc9f0, 'HangulSyllableJjass', [], 'Display', None],
  [ 0xc9f1, 'HangulSyllableJjang', [], 'Display', None],
  [ 0xc9f2, 'HangulSyllableJjaj', [], 'Display', None],
  [ 0xc9f3, 'HangulSyllableJjac', [], 'Display', None],
  [ 0xc9f4, 'HangulSyllableJjak', [], 'Display', None],
  [ 0xc9f5, 'HangulSyllableJjat', [], 'Display', None],
  [ 0xc9f6, 'HangulSyllableJjap', [], 'Display', None],
  [ 0xc9f7, 'HangulSyllableJjah', [], 'Display', None],
  [ 0xc9f8, 'HangulSyllableJjae', [], 'Display', None],
  [ 0xc9f9, 'HangulSyllableJjaeg', [], 'Display', None],
  [ 0xc9fa, 'HangulSyllableJjaegg', [], 'Display', None],
  [ 0xc9fb, 'HangulSyllableJjaegs', [], 'Display', None],
  [ 0xc9fc, 'HangulSyllableJjaen', [], 'Display', None],
  [ 0xc9fd, 'HangulSyllableJjaenj', [], 'Display', None],
  [ 0xc9fe, 'HangulSyllableJjaenh', [], 'Display', None],
  [ 0xc9ff, 'HangulSyllableJjaed', [], 'Display', None],
  [ 0xca00, 'HangulSyllableJjael', [], 'Display', None],
  [ 0xca01, 'HangulSyllableJjaelg', [], 'Display', None],
  [ 0xca02, 'HangulSyllableJjaelm', [], 'Display', None],
  [ 0xca03, 'HangulSyllableJjaelb', [], 'Display', None],
  [ 0xca04, 'HangulSyllableJjaels', [], 'Display', None],
  [ 0xca05, 'HangulSyllableJjaelt', [], 'Display', None],
  [ 0xca06, 'HangulSyllableJjaelp', [], 'Display', None],
  [ 0xca07, 'HangulSyllableJjaelh', [], 'Display', None],
  [ 0xca08, 'HangulSyllableJjaem', [], 'Display', None],
  [ 0xca09, 'HangulSyllableJjaeb', [], 'Display', None],
  [ 0xca0a, 'HangulSyllableJjaebs', [], 'Display', None],
  [ 0xca0b, 'HangulSyllableJjaes', [], 'Display', None],
  [ 0xca0c, 'HangulSyllableJjaess', [], 'Display', None],
  [ 0xca0d, 'HangulSyllableJjaeng', [], 'Display', None],
  [ 0xca0e, 'HangulSyllableJjaej', [], 'Display', None],
  [ 0xca0f, 'HangulSyllableJjaec', [], 'Display', None],
  [ 0xca10, 'HangulSyllableJjaek', [], 'Display', None],
  [ 0xca11, 'HangulSyllableJjaet', [], 'Display', None],
  [ 0xca12, 'HangulSyllableJjaep', [], 'Display', None],
  [ 0xca13, 'HangulSyllableJjaeh', [], 'Display', None],
  [ 0xca14, 'HangulSyllableJjya', [], 'Display', None],
  [ 0xca15, 'HangulSyllableJjyag', [], 'Display', None],
  [ 0xca16, 'HangulSyllableJjyagg', [], 'Display', None],
  [ 0xca17, 'HangulSyllableJjyags', [], 'Display', None],
  [ 0xca18, 'HangulSyllableJjyan', [], 'Display', None],
  [ 0xca19, 'HangulSyllableJjyanj', [], 'Display', None],
  [ 0xca1a, 'HangulSyllableJjyanh', [], 'Display', None],
  [ 0xca1b, 'HangulSyllableJjyad', [], 'Display', None],
  [ 0xca1c, 'HangulSyllableJjyal', [], 'Display', None],
  [ 0xca1d, 'HangulSyllableJjyalg', [], 'Display', None],
  [ 0xca1e, 'HangulSyllableJjyalm', [], 'Display', None],
  [ 0xca1f, 'HangulSyllableJjyalb', [], 'Display', None],
  [ 0xca20, 'HangulSyllableJjyals', [], 'Display', None],
  [ 0xca21, 'HangulSyllableJjyalt', [], 'Display', None],
  [ 0xca22, 'HangulSyllableJjyalp', [], 'Display', None],
  [ 0xca23, 'HangulSyllableJjyalh', [], 'Display', None],
  [ 0xca24, 'HangulSyllableJjyam', [], 'Display', None],
  [ 0xca25, 'HangulSyllableJjyab', [], 'Display', None],
  [ 0xca26, 'HangulSyllableJjyabs', [], 'Display', None],
  [ 0xca27, 'HangulSyllableJjyas', [], 'Display', None],
  [ 0xca28, 'HangulSyllableJjyass', [], 'Display', None],
  [ 0xca29, 'HangulSyllableJjyang', [], 'Display', None],
  [ 0xca2a, 'HangulSyllableJjyaj', [], 'Display', None],
  [ 0xca2b, 'HangulSyllableJjyac', [], 'Display', None],
  [ 0xca2c, 'HangulSyllableJjyak', [], 'Display', None],
  [ 0xca2d, 'HangulSyllableJjyat', [], 'Display', None],
  [ 0xca2e, 'HangulSyllableJjyap', [], 'Display', None],
  [ 0xca2f, 'HangulSyllableJjyah', [], 'Display', None],
  [ 0xca30, 'HangulSyllableJjyae', [], 'Display', None],
  [ 0xca31, 'HangulSyllableJjyaeg', [], 'Display', None],
  [ 0xca32, 'HangulSyllableJjyaegg', [], 'Display', None],
  [ 0xca33, 'HangulSyllableJjyaegs', [], 'Display', None],
  [ 0xca34, 'HangulSyllableJjyaen', [], 'Display', None],
  [ 0xca35, 'HangulSyllableJjyaenj', [], 'Display', None],
  [ 0xca36, 'HangulSyllableJjyaenh', [], 'Display', None],
  [ 0xca37, 'HangulSyllableJjyaed', [], 'Display', None],
  [ 0xca38, 'HangulSyllableJjyael', [], 'Display', None],
  [ 0xca39, 'HangulSyllableJjyaelg', [], 'Display', None],
  [ 0xca3a, 'HangulSyllableJjyaelm', [], 'Display', None],
  [ 0xca3b, 'HangulSyllableJjyaelb', [], 'Display', None],
  [ 0xca3c, 'HangulSyllableJjyaels', [], 'Display', None],
  [ 0xca3d, 'HangulSyllableJjyaelt', [], 'Display', None],
  [ 0xca3e, 'HangulSyllableJjyaelp', [], 'Display', None],
  [ 0xca3f, 'HangulSyllableJjyaelh', [], 'Display', None],
  [ 0xca40, 'HangulSyllableJjyaem', [], 'Display', None],
  [ 0xca41, 'HangulSyllableJjyaeb', [], 'Display', None],
  [ 0xca42, 'HangulSyllableJjyaebs', [], 'Display', None],
  [ 0xca43, 'HangulSyllableJjyaes', [], 'Display', None],
  [ 0xca44, 'HangulSyllableJjyaess', [], 'Display', None],
  [ 0xca45, 'HangulSyllableJjyaeng', [], 'Display', None],
  [ 0xca46, 'HangulSyllableJjyaej', [], 'Display', None],
  [ 0xca47, 'HangulSyllableJjyaec', [], 'Display', None],
  [ 0xca48, 'HangulSyllableJjyaek', [], 'Display', None],
  [ 0xca49, 'HangulSyllableJjyaet', [], 'Display', None],
  [ 0xca4a, 'HangulSyllableJjyaep', [], 'Display', None],
  [ 0xca4b, 'HangulSyllableJjyaeh', [], 'Display', None],
  [ 0xca4c, 'HangulSyllableJjeo', [], 'Display', None],
  [ 0xca4d, 'HangulSyllableJjeog', [], 'Display', None],
  [ 0xca4e, 'HangulSyllableJjeogg', [], 'Display', None],
  [ 0xca4f, 'HangulSyllableJjeogs', [], 'Display', None],
  [ 0xca50, 'HangulSyllableJjeon', [], 'Display', None],
  [ 0xca51, 'HangulSyllableJjeonj', [], 'Display', None],
  [ 0xca52, 'HangulSyllableJjeonh', [], 'Display', None],
  [ 0xca53, 'HangulSyllableJjeod', [], 'Display', None],
  [ 0xca54, 'HangulSyllableJjeol', [], 'Display', None],
  [ 0xca55, 'HangulSyllableJjeolg', [], 'Display', None],
  [ 0xca56, 'HangulSyllableJjeolm', [], 'Display', None],
  [ 0xca57, 'HangulSyllableJjeolb', [], 'Display', None],
  [ 0xca58, 'HangulSyllableJjeols', [], 'Display', None],
  [ 0xca59, 'HangulSyllableJjeolt', [], 'Display', None],
  [ 0xca5a, 'HangulSyllableJjeolp', [], 'Display', None],
  [ 0xca5b, 'HangulSyllableJjeolh', [], 'Display', None],
  [ 0xca5c, 'HangulSyllableJjeom', [], 'Display', None],
  [ 0xca5d, 'HangulSyllableJjeob', [], 'Display', None],
  [ 0xca5e, 'HangulSyllableJjeobs', [], 'Display', None],
  [ 0xca5f, 'HangulSyllableJjeos', [], 'Display', None],
  [ 0xca60, 'HangulSyllableJjeoss', [], 'Display', None],
  [ 0xca61, 'HangulSyllableJjeong', [], 'Display', None],
  [ 0xca62, 'HangulSyllableJjeoj', [], 'Display', None],
  [ 0xca63, 'HangulSyllableJjeoc', [], 'Display', None],
  [ 0xca64, 'HangulSyllableJjeok', [], 'Display', None],
  [ 0xca65, 'HangulSyllableJjeot', [], 'Display', None],
  [ 0xca66, 'HangulSyllableJjeop', [], 'Display', None],
  [ 0xca67, 'HangulSyllableJjeoh', [], 'Display', None],
  [ 0xca68, 'HangulSyllableJje', [], 'Display', None],
  [ 0xca69, 'HangulSyllableJjeg', [], 'Display', None],
  [ 0xca6a, 'HangulSyllableJjegg', [], 'Display', None],
  [ 0xca6b, 'HangulSyllableJjegs', [], 'Display', None],
  [ 0xca6c, 'HangulSyllableJjen', [], 'Display', None],
  [ 0xca6d, 'HangulSyllableJjenj', [], 'Display', None],
  [ 0xca6e, 'HangulSyllableJjenh', [], 'Display', None],
  [ 0xca6f, 'HangulSyllableJjed', [], 'Display', None],
  [ 0xca70, 'HangulSyllableJjel', [], 'Display', None],
  [ 0xca71, 'HangulSyllableJjelg', [], 'Display', None],
  [ 0xca72, 'HangulSyllableJjelm', [], 'Display', None],
  [ 0xca73, 'HangulSyllableJjelb', [], 'Display', None],
  [ 0xca74, 'HangulSyllableJjels', [], 'Display', None],
  [ 0xca75, 'HangulSyllableJjelt', [], 'Display', None],
  [ 0xca76, 'HangulSyllableJjelp', [], 'Display', None],
  [ 0xca77, 'HangulSyllableJjelh', [], 'Display', None],
  [ 0xca78, 'HangulSyllableJjem', [], 'Display', None],
  [ 0xca79, 'HangulSyllableJjeb', [], 'Display', None],
  [ 0xca7a, 'HangulSyllableJjebs', [], 'Display', None],
  [ 0xca7b, 'HangulSyllableJjes', [], 'Display', None],
  [ 0xca7c, 'HangulSyllableJjess', [], 'Display', None],
  [ 0xca7d, 'HangulSyllableJjeng', [], 'Display', None],
  [ 0xca7e, 'HangulSyllableJjej', [], 'Display', None],
  [ 0xca7f, 'HangulSyllableJjec', [], 'Display', None],
  [ 0xca80, 'HangulSyllableJjek', [], 'Display', None],
  [ 0xca81, 'HangulSyllableJjet', [], 'Display', None],
  [ 0xca82, 'HangulSyllableJjep', [], 'Display', None],
  [ 0xca83, 'HangulSyllableJjeh', [], 'Display', None],
  [ 0xca84, 'HangulSyllableJjyeo', [], 'Display', None],
  [ 0xca85, 'HangulSyllableJjyeog', [], 'Display', None],
  [ 0xca86, 'HangulSyllableJjyeogg', [], 'Display', None],
  [ 0xca87, 'HangulSyllableJjyeogs', [], 'Display', None],
  [ 0xca88, 'HangulSyllableJjyeon', [], 'Display', None],
  [ 0xca89, 'HangulSyllableJjyeonj', [], 'Display', None],
  [ 0xca8a, 'HangulSyllableJjyeonh', [], 'Display', None],
  [ 0xca8b, 'HangulSyllableJjyeod', [], 'Display', None],
  [ 0xca8c, 'HangulSyllableJjyeol', [], 'Display', None],
  [ 0xca8d, 'HangulSyllableJjyeolg', [], 'Display', None],
  [ 0xca8e, 'HangulSyllableJjyeolm', [], 'Display', None],
  [ 0xca8f, 'HangulSyllableJjyeolb', [], 'Display', None],
  [ 0xca90, 'HangulSyllableJjyeols', [], 'Display', None],
  [ 0xca91, 'HangulSyllableJjyeolt', [], 'Display', None],
  [ 0xca92, 'HangulSyllableJjyeolp', [], 'Display', None],
  [ 0xca93, 'HangulSyllableJjyeolh', [], 'Display', None],
  [ 0xca94, 'HangulSyllableJjyeom', [], 'Display', None],
  [ 0xca95, 'HangulSyllableJjyeob', [], 'Display', None],
  [ 0xca96, 'HangulSyllableJjyeobs', [], 'Display', None],
  [ 0xca97, 'HangulSyllableJjyeos', [], 'Display', None],
  [ 0xca98, 'HangulSyllableJjyeoss', [], 'Display', None],
  [ 0xca99, 'HangulSyllableJjyeong', [], 'Display', None],
  [ 0xca9a, 'HangulSyllableJjyeoj', [], 'Display', None],
  [ 0xca9b, 'HangulSyllableJjyeoc', [], 'Display', None],
  [ 0xca9c, 'HangulSyllableJjyeok', [], 'Display', None],
  [ 0xca9d, 'HangulSyllableJjyeot', [], 'Display', None],
  [ 0xca9e, 'HangulSyllableJjyeop', [], 'Display', None],
  [ 0xca9f, 'HangulSyllableJjyeoh', [], 'Display', None],
  [ 0xcaa0, 'HangulSyllableJjye', [], 'Display', None],
  [ 0xcaa1, 'HangulSyllableJjyeg', [], 'Display', None],
  [ 0xcaa2, 'HangulSyllableJjyegg', [], 'Display', None],
  [ 0xcaa3, 'HangulSyllableJjyegs', [], 'Display', None],
  [ 0xcaa4, 'HangulSyllableJjyen', [], 'Display', None],
  [ 0xcaa5, 'HangulSyllableJjyenj', [], 'Display', None],
  [ 0xcaa6, 'HangulSyllableJjyenh', [], 'Display', None],
  [ 0xcaa7, 'HangulSyllableJjyed', [], 'Display', None],
  [ 0xcaa8, 'HangulSyllableJjyel', [], 'Display', None],
  [ 0xcaa9, 'HangulSyllableJjyelg', [], 'Display', None],
  [ 0xcaaa, 'HangulSyllableJjyelm', [], 'Display', None],
  [ 0xcaab, 'HangulSyllableJjyelb', [], 'Display', None],
  [ 0xcaac, 'HangulSyllableJjyels', [], 'Display', None],
  [ 0xcaad, 'HangulSyllableJjyelt', [], 'Display', None],
  [ 0xcaae, 'HangulSyllableJjyelp', [], 'Display', None],
  [ 0xcaaf, 'HangulSyllableJjyelh', [], 'Display', None],
  [ 0xcab0, 'HangulSyllableJjyem', [], 'Display', None],
  [ 0xcab1, 'HangulSyllableJjyeb', [], 'Display', None],
  [ 0xcab2, 'HangulSyllableJjyebs', [], 'Display', None],
  [ 0xcab3, 'HangulSyllableJjyes', [], 'Display', None],
  [ 0xcab4, 'HangulSyllableJjyess', [], 'Display', None],
  [ 0xcab5, 'HangulSyllableJjyeng', [], 'Display', None],
  [ 0xcab6, 'HangulSyllableJjyej', [], 'Display', None],
  [ 0xcab7, 'HangulSyllableJjyec', [], 'Display', None],
  [ 0xcab8, 'HangulSyllableJjyek', [], 'Display', None],
  [ 0xcab9, 'HangulSyllableJjyet', [], 'Display', None],
  [ 0xcaba, 'HangulSyllableJjyep', [], 'Display', None],
  [ 0xcabb, 'HangulSyllableJjyeh', [], 'Display', None],
  [ 0xcabc, 'HangulSyllableJjo', [], 'Display', None],
  [ 0xcabd, 'HangulSyllableJjog', [], 'Display', None],
  [ 0xcabe, 'HangulSyllableJjogg', [], 'Display', None],
  [ 0xcabf, 'HangulSyllableJjogs', [], 'Display', None],
  [ 0xcac0, 'HangulSyllableJjon', [], 'Display', None],
  [ 0xcac1, 'HangulSyllableJjonj', [], 'Display', None],
  [ 0xcac2, 'HangulSyllableJjonh', [], 'Display', None],
  [ 0xcac3, 'HangulSyllableJjod', [], 'Display', None],
  [ 0xcac4, 'HangulSyllableJjol', [], 'Display', None],
  [ 0xcac5, 'HangulSyllableJjolg', [], 'Display', None],
  [ 0xcac6, 'HangulSyllableJjolm', [], 'Display', None],
  [ 0xcac7, 'HangulSyllableJjolb', [], 'Display', None],
  [ 0xcac8, 'HangulSyllableJjols', [], 'Display', None],
  [ 0xcac9, 'HangulSyllableJjolt', [], 'Display', None],
  [ 0xcaca, 'HangulSyllableJjolp', [], 'Display', None],
  [ 0xcacb, 'HangulSyllableJjolh', [], 'Display', None],
  [ 0xcacc, 'HangulSyllableJjom', [], 'Display', None],
  [ 0xcacd, 'HangulSyllableJjob', [], 'Display', None],
  [ 0xcace, 'HangulSyllableJjobs', [], 'Display', None],
  [ 0xcacf, 'HangulSyllableJjos', [], 'Display', None],
  [ 0xcad0, 'HangulSyllableJjoss', [], 'Display', None],
  [ 0xcad1, 'HangulSyllableJjong', [], 'Display', None],
  [ 0xcad2, 'HangulSyllableJjoj', [], 'Display', None],
  [ 0xcad3, 'HangulSyllableJjoc', [], 'Display', None],
  [ 0xcad4, 'HangulSyllableJjok', [], 'Display', None],
  [ 0xcad5, 'HangulSyllableJjot', [], 'Display', None],
  [ 0xcad6, 'HangulSyllableJjop', [], 'Display', None],
  [ 0xcad7, 'HangulSyllableJjoh', [], 'Display', None],
  [ 0xcad8, 'HangulSyllableJjwa', [], 'Display', None],
  [ 0xcad9, 'HangulSyllableJjwag', [], 'Display', None],
  [ 0xcada, 'HangulSyllableJjwagg', [], 'Display', None],
  [ 0xcadb, 'HangulSyllableJjwags', [], 'Display', None],
  [ 0xcadc, 'HangulSyllableJjwan', [], 'Display', None],
  [ 0xcadd, 'HangulSyllableJjwanj', [], 'Display', None],
  [ 0xcade, 'HangulSyllableJjwanh', [], 'Display', None],
  [ 0xcadf, 'HangulSyllableJjwad', [], 'Display', None],
  [ 0xcae0, 'HangulSyllableJjwal', [], 'Display', None],
  [ 0xcae1, 'HangulSyllableJjwalg', [], 'Display', None],
  [ 0xcae2, 'HangulSyllableJjwalm', [], 'Display', None],
  [ 0xcae3, 'HangulSyllableJjwalb', [], 'Display', None],
  [ 0xcae4, 'HangulSyllableJjwals', [], 'Display', None],
  [ 0xcae5, 'HangulSyllableJjwalt', [], 'Display', None],
  [ 0xcae6, 'HangulSyllableJjwalp', [], 'Display', None],
  [ 0xcae7, 'HangulSyllableJjwalh', [], 'Display', None],
  [ 0xcae8, 'HangulSyllableJjwam', [], 'Display', None],
  [ 0xcae9, 'HangulSyllableJjwab', [], 'Display', None],
  [ 0xcaea, 'HangulSyllableJjwabs', [], 'Display', None],
  [ 0xcaeb, 'HangulSyllableJjwas', [], 'Display', None],
  [ 0xcaec, 'HangulSyllableJjwass', [], 'Display', None],
  [ 0xcaed, 'HangulSyllableJjwang', [], 'Display', None],
  [ 0xcaee, 'HangulSyllableJjwaj', [], 'Display', None],
  [ 0xcaef, 'HangulSyllableJjwac', [], 'Display', None],
  [ 0xcaf0, 'HangulSyllableJjwak', [], 'Display', None],
  [ 0xcaf1, 'HangulSyllableJjwat', [], 'Display', None],
  [ 0xcaf2, 'HangulSyllableJjwap', [], 'Display', None],
  [ 0xcaf3, 'HangulSyllableJjwah', [], 'Display', None],
  [ 0xcaf4, 'HangulSyllableJjwae', [], 'Display', None],
  [ 0xcaf5, 'HangulSyllableJjwaeg', [], 'Display', None],
  [ 0xcaf6, 'HangulSyllableJjwaegg', [], 'Display', None],
  [ 0xcaf7, 'HangulSyllableJjwaegs', [], 'Display', None],
  [ 0xcaf8, 'HangulSyllableJjwaen', [], 'Display', None],
  [ 0xcaf9, 'HangulSyllableJjwaenj', [], 'Display', None],
  [ 0xcafa, 'HangulSyllableJjwaenh', [], 'Display', None],
  [ 0xcafb, 'HangulSyllableJjwaed', [], 'Display', None],
  [ 0xcafc, 'HangulSyllableJjwael', [], 'Display', None],
  [ 0xcafd, 'HangulSyllableJjwaelg', [], 'Display', None],
  [ 0xcafe, 'HangulSyllableJjwaelm', [], 'Display', None],
  [ 0xcaff, 'HangulSyllableJjwaelb', [], 'Display', None],
  [ 0xcb00, 'HangulSyllableJjwaels', [], 'Display', None],
  [ 0xcb01, 'HangulSyllableJjwaelt', [], 'Display', None],
  [ 0xcb02, 'HangulSyllableJjwaelp', [], 'Display', None],
  [ 0xcb03, 'HangulSyllableJjwaelh', [], 'Display', None],
  [ 0xcb04, 'HangulSyllableJjwaem', [], 'Display', None],
  [ 0xcb05, 'HangulSyllableJjwaeb', [], 'Display', None],
  [ 0xcb06, 'HangulSyllableJjwaebs', [], 'Display', None],
  [ 0xcb07, 'HangulSyllableJjwaes', [], 'Display', None],
  [ 0xcb08, 'HangulSyllableJjwaess', [], 'Display', None],
  [ 0xcb09, 'HangulSyllableJjwaeng', [], 'Display', None],
  [ 0xcb0a, 'HangulSyllableJjwaej', [], 'Display', None],
  [ 0xcb0b, 'HangulSyllableJjwaec', [], 'Display', None],
  [ 0xcb0c, 'HangulSyllableJjwaek', [], 'Display', None],
  [ 0xcb0d, 'HangulSyllableJjwaet', [], 'Display', None],
  [ 0xcb0e, 'HangulSyllableJjwaep', [], 'Display', None],
  [ 0xcb0f, 'HangulSyllableJjwaeh', [], 'Display', None],
  [ 0xcb10, 'HangulSyllableJjoe', [], 'Display', None],
  [ 0xcb11, 'HangulSyllableJjoeg', [], 'Display', None],
  [ 0xcb12, 'HangulSyllableJjoegg', [], 'Display', None],
  [ 0xcb13, 'HangulSyllableJjoegs', [], 'Display', None],
  [ 0xcb14, 'HangulSyllableJjoen', [], 'Display', None],
  [ 0xcb15, 'HangulSyllableJjoenj', [], 'Display', None],
  [ 0xcb16, 'HangulSyllableJjoenh', [], 'Display', None],
  [ 0xcb17, 'HangulSyllableJjoed', [], 'Display', None],
  [ 0xcb18, 'HangulSyllableJjoel', [], 'Display', None],
  [ 0xcb19, 'HangulSyllableJjoelg', [], 'Display', None],
  [ 0xcb1a, 'HangulSyllableJjoelm', [], 'Display', None],
  [ 0xcb1b, 'HangulSyllableJjoelb', [], 'Display', None],
  [ 0xcb1c, 'HangulSyllableJjoels', [], 'Display', None],
  [ 0xcb1d, 'HangulSyllableJjoelt', [], 'Display', None],
  [ 0xcb1e, 'HangulSyllableJjoelp', [], 'Display', None],
  [ 0xcb1f, 'HangulSyllableJjoelh', [], 'Display', None],
  [ 0xcb20, 'HangulSyllableJjoem', [], 'Display', None],
  [ 0xcb21, 'HangulSyllableJjoeb', [], 'Display', None],
  [ 0xcb22, 'HangulSyllableJjoebs', [], 'Display', None],
  [ 0xcb23, 'HangulSyllableJjoes', [], 'Display', None],
  [ 0xcb24, 'HangulSyllableJjoess', [], 'Display', None],
  [ 0xcb25, 'HangulSyllableJjoeng', [], 'Display', None],
  [ 0xcb26, 'HangulSyllableJjoej', [], 'Display', None],
  [ 0xcb27, 'HangulSyllableJjoec', [], 'Display', None],
  [ 0xcb28, 'HangulSyllableJjoek', [], 'Display', None],
  [ 0xcb29, 'HangulSyllableJjoet', [], 'Display', None],
  [ 0xcb2a, 'HangulSyllableJjoep', [], 'Display', None],
  [ 0xcb2b, 'HangulSyllableJjoeh', [], 'Display', None],
  [ 0xcb2c, 'HangulSyllableJjyo', [], 'Display', None],
  [ 0xcb2d, 'HangulSyllableJjyog', [], 'Display', None],
  [ 0xcb2e, 'HangulSyllableJjyogg', [], 'Display', None],
  [ 0xcb2f, 'HangulSyllableJjyogs', [], 'Display', None],
  [ 0xcb30, 'HangulSyllableJjyon', [], 'Display', None],
  [ 0xcb31, 'HangulSyllableJjyonj', [], 'Display', None],
  [ 0xcb32, 'HangulSyllableJjyonh', [], 'Display', None],
  [ 0xcb33, 'HangulSyllableJjyod', [], 'Display', None],
  [ 0xcb34, 'HangulSyllableJjyol', [], 'Display', None],
  [ 0xcb35, 'HangulSyllableJjyolg', [], 'Display', None],
  [ 0xcb36, 'HangulSyllableJjyolm', [], 'Display', None],
  [ 0xcb37, 'HangulSyllableJjyolb', [], 'Display', None],
  [ 0xcb38, 'HangulSyllableJjyols', [], 'Display', None],
  [ 0xcb39, 'HangulSyllableJjyolt', [], 'Display', None],
  [ 0xcb3a, 'HangulSyllableJjyolp', [], 'Display', None],
  [ 0xcb3b, 'HangulSyllableJjyolh', [], 'Display', None],
  [ 0xcb3c, 'HangulSyllableJjyom', [], 'Display', None],
  [ 0xcb3d, 'HangulSyllableJjyob', [], 'Display', None],
  [ 0xcb3e, 'HangulSyllableJjyobs', [], 'Display', None],
  [ 0xcb3f, 'HangulSyllableJjyos', [], 'Display', None],
  [ 0xcb40, 'HangulSyllableJjyoss', [], 'Display', None],
  [ 0xcb41, 'HangulSyllableJjyong', [], 'Display', None],
  [ 0xcb42, 'HangulSyllableJjyoj', [], 'Display', None],
  [ 0xcb43, 'HangulSyllableJjyoc', [], 'Display', None],
  [ 0xcb44, 'HangulSyllableJjyok', [], 'Display', None],
  [ 0xcb45, 'HangulSyllableJjyot', [], 'Display', None],
  [ 0xcb46, 'HangulSyllableJjyop', [], 'Display', None],
  [ 0xcb47, 'HangulSyllableJjyoh', [], 'Display', None],
  [ 0xcb48, 'HangulSyllableJju', [], 'Display', None],
  [ 0xcb49, 'HangulSyllableJjug', [], 'Display', None],
  [ 0xcb4a, 'HangulSyllableJjugg', [], 'Display', None],
  [ 0xcb4b, 'HangulSyllableJjugs', [], 'Display', None],
  [ 0xcb4c, 'HangulSyllableJjun', [], 'Display', None],
  [ 0xcb4d, 'HangulSyllableJjunj', [], 'Display', None],
  [ 0xcb4e, 'HangulSyllableJjunh', [], 'Display', None],
  [ 0xcb4f, 'HangulSyllableJjud', [], 'Display', None],
  [ 0xcb50, 'HangulSyllableJjul', [], 'Display', None],
  [ 0xcb51, 'HangulSyllableJjulg', [], 'Display', None],
  [ 0xcb52, 'HangulSyllableJjulm', [], 'Display', None],
  [ 0xcb53, 'HangulSyllableJjulb', [], 'Display', None],
  [ 0xcb54, 'HangulSyllableJjuls', [], 'Display', None],
  [ 0xcb55, 'HangulSyllableJjult', [], 'Display', None],
  [ 0xcb56, 'HangulSyllableJjulp', [], 'Display', None],
  [ 0xcb57, 'HangulSyllableJjulh', [], 'Display', None],
  [ 0xcb58, 'HangulSyllableJjum', [], 'Display', None],
  [ 0xcb59, 'HangulSyllableJjub', [], 'Display', None],
  [ 0xcb5a, 'HangulSyllableJjubs', [], 'Display', None],
  [ 0xcb5b, 'HangulSyllableJjus', [], 'Display', None],
  [ 0xcb5c, 'HangulSyllableJjuss', [], 'Display', None],
  [ 0xcb5d, 'HangulSyllableJjung', [], 'Display', None],
  [ 0xcb5e, 'HangulSyllableJjuj', [], 'Display', None],
  [ 0xcb5f, 'HangulSyllableJjuc', [], 'Display', None],
  [ 0xcb60, 'HangulSyllableJjuk', [], 'Display', None],
  [ 0xcb61, 'HangulSyllableJjut', [], 'Display', None],
  [ 0xcb62, 'HangulSyllableJjup', [], 'Display', None],
  [ 0xcb63, 'HangulSyllableJjuh', [], 'Display', None],
  [ 0xcb64, 'HangulSyllableJjweo', [], 'Display', None],
  [ 0xcb65, 'HangulSyllableJjweog', [], 'Display', None],
  [ 0xcb66, 'HangulSyllableJjweogg', [], 'Display', None],
  [ 0xcb67, 'HangulSyllableJjweogs', [], 'Display', None],
  [ 0xcb68, 'HangulSyllableJjweon', [], 'Display', None],
  [ 0xcb69, 'HangulSyllableJjweonj', [], 'Display', None],
  [ 0xcb6a, 'HangulSyllableJjweonh', [], 'Display', None],
  [ 0xcb6b, 'HangulSyllableJjweod', [], 'Display', None],
  [ 0xcb6c, 'HangulSyllableJjweol', [], 'Display', None],
  [ 0xcb6d, 'HangulSyllableJjweolg', [], 'Display', None],
  [ 0xcb6e, 'HangulSyllableJjweolm', [], 'Display', None],
  [ 0xcb6f, 'HangulSyllableJjweolb', [], 'Display', None],
  [ 0xcb70, 'HangulSyllableJjweols', [], 'Display', None],
  [ 0xcb71, 'HangulSyllableJjweolt', [], 'Display', None],
  [ 0xcb72, 'HangulSyllableJjweolp', [], 'Display', None],
  [ 0xcb73, 'HangulSyllableJjweolh', [], 'Display', None],
  [ 0xcb74, 'HangulSyllableJjweom', [], 'Display', None],
  [ 0xcb75, 'HangulSyllableJjweob', [], 'Display', None],
  [ 0xcb76, 'HangulSyllableJjweobs', [], 'Display', None],
  [ 0xcb77, 'HangulSyllableJjweos', [], 'Display', None],
  [ 0xcb78, 'HangulSyllableJjweoss', [], 'Display', None],
  [ 0xcb79, 'HangulSyllableJjweong', [], 'Display', None],
  [ 0xcb7a, 'HangulSyllableJjweoj', [], 'Display', None],
  [ 0xcb7b, 'HangulSyllableJjweoc', [], 'Display', None],
  [ 0xcb7c, 'HangulSyllableJjweok', [], 'Display', None],
  [ 0xcb7d, 'HangulSyllableJjweot', [], 'Display', None],
  [ 0xcb7e, 'HangulSyllableJjweop', [], 'Display', None],
  [ 0xcb7f, 'HangulSyllableJjweoh', [], 'Display', None],
  [ 0xcb80, 'HangulSyllableJjwe', [], 'Display', None],
  [ 0xcb81, 'HangulSyllableJjweg', [], 'Display', None],
  [ 0xcb82, 'HangulSyllableJjwegg', [], 'Display', None],
  [ 0xcb83, 'HangulSyllableJjwegs', [], 'Display', None],
  [ 0xcb84, 'HangulSyllableJjwen', [], 'Display', None],
  [ 0xcb85, 'HangulSyllableJjwenj', [], 'Display', None],
  [ 0xcb86, 'HangulSyllableJjwenh', [], 'Display', None],
  [ 0xcb87, 'HangulSyllableJjwed', [], 'Display', None],
  [ 0xcb88, 'HangulSyllableJjwel', [], 'Display', None],
  [ 0xcb89, 'HangulSyllableJjwelg', [], 'Display', None],
  [ 0xcb8a, 'HangulSyllableJjwelm', [], 'Display', None],
  [ 0xcb8b, 'HangulSyllableJjwelb', [], 'Display', None],
  [ 0xcb8c, 'HangulSyllableJjwels', [], 'Display', None],
  [ 0xcb8d, 'HangulSyllableJjwelt', [], 'Display', None],
  [ 0xcb8e, 'HangulSyllableJjwelp', [], 'Display', None],
  [ 0xcb8f, 'HangulSyllableJjwelh', [], 'Display', None],
  [ 0xcb90, 'HangulSyllableJjwem', [], 'Display', None],
  [ 0xcb91, 'HangulSyllableJjweb', [], 'Display', None],
  [ 0xcb92, 'HangulSyllableJjwebs', [], 'Display', None],
  [ 0xcb93, 'HangulSyllableJjwes', [], 'Display', None],
  [ 0xcb94, 'HangulSyllableJjwess', [], 'Display', None],
  [ 0xcb95, 'HangulSyllableJjweng', [], 'Display', None],
  [ 0xcb96, 'HangulSyllableJjwej', [], 'Display', None],
  [ 0xcb97, 'HangulSyllableJjwec', [], 'Display', None],
  [ 0xcb98, 'HangulSyllableJjwek', [], 'Display', None],
  [ 0xcb99, 'HangulSyllableJjwet', [], 'Display', None],
  [ 0xcb9a, 'HangulSyllableJjwep', [], 'Display', None],
  [ 0xcb9b, 'HangulSyllableJjweh', [], 'Display', None],
  [ 0xcb9c, 'HangulSyllableJjwi', [], 'Display', None],
  [ 0xcb9d, 'HangulSyllableJjwig', [], 'Display', None],
  [ 0xcb9e, 'HangulSyllableJjwigg', [], 'Display', None],
  [ 0xcb9f, 'HangulSyllableJjwigs', [], 'Display', None],
  [ 0xcba0, 'HangulSyllableJjwin', [], 'Display', None],
  [ 0xcba1, 'HangulSyllableJjwinj', [], 'Display', None],
  [ 0xcba2, 'HangulSyllableJjwinh', [], 'Display', None],
  [ 0xcba3, 'HangulSyllableJjwid', [], 'Display', None],
  [ 0xcba4, 'HangulSyllableJjwil', [], 'Display', None],
  [ 0xcba5, 'HangulSyllableJjwilg', [], 'Display', None],
  [ 0xcba6, 'HangulSyllableJjwilm', [], 'Display', None],
  [ 0xcba7, 'HangulSyllableJjwilb', [], 'Display', None],
  [ 0xcba8, 'HangulSyllableJjwils', [], 'Display', None],
  [ 0xcba9, 'HangulSyllableJjwilt', [], 'Display', None],
  [ 0xcbaa, 'HangulSyllableJjwilp', [], 'Display', None],
  [ 0xcbab, 'HangulSyllableJjwilh', [], 'Display', None],
  [ 0xcbac, 'HangulSyllableJjwim', [], 'Display', None],
  [ 0xcbad, 'HangulSyllableJjwib', [], 'Display', None],
  [ 0xcbae, 'HangulSyllableJjwibs', [], 'Display', None],
  [ 0xcbaf, 'HangulSyllableJjwis', [], 'Display', None],
  [ 0xcbb0, 'HangulSyllableJjwiss', [], 'Display', None],
  [ 0xcbb1, 'HangulSyllableJjwing', [], 'Display', None],
  [ 0xcbb2, 'HangulSyllableJjwij', [], 'Display', None],
  [ 0xcbb3, 'HangulSyllableJjwic', [], 'Display', None],
  [ 0xcbb4, 'HangulSyllableJjwik', [], 'Display', None],
  [ 0xcbb5, 'HangulSyllableJjwit', [], 'Display', None],
  [ 0xcbb6, 'HangulSyllableJjwip', [], 'Display', None],
  [ 0xcbb7, 'HangulSyllableJjwih', [], 'Display', None],
  [ 0xcbb8, 'HangulSyllableJjyu', [], 'Display', None],
  [ 0xcbb9, 'HangulSyllableJjyug', [], 'Display', None],
  [ 0xcbba, 'HangulSyllableJjyugg', [], 'Display', None],
  [ 0xcbbb, 'HangulSyllableJjyugs', [], 'Display', None],
  [ 0xcbbc, 'HangulSyllableJjyun', [], 'Display', None],
  [ 0xcbbd, 'HangulSyllableJjyunj', [], 'Display', None],
  [ 0xcbbe, 'HangulSyllableJjyunh', [], 'Display', None],
  [ 0xcbbf, 'HangulSyllableJjyud', [], 'Display', None],
  [ 0xcbc0, 'HangulSyllableJjyul', [], 'Display', None],
  [ 0xcbc1, 'HangulSyllableJjyulg', [], 'Display', None],
  [ 0xcbc2, 'HangulSyllableJjyulm', [], 'Display', None],
  [ 0xcbc3, 'HangulSyllableJjyulb', [], 'Display', None],
  [ 0xcbc4, 'HangulSyllableJjyuls', [], 'Display', None],
  [ 0xcbc5, 'HangulSyllableJjyult', [], 'Display', None],
  [ 0xcbc6, 'HangulSyllableJjyulp', [], 'Display', None],
  [ 0xcbc7, 'HangulSyllableJjyulh', [], 'Display', None],
  [ 0xcbc8, 'HangulSyllableJjyum', [], 'Display', None],
  [ 0xcbc9, 'HangulSyllableJjyub', [], 'Display', None],
  [ 0xcbca, 'HangulSyllableJjyubs', [], 'Display', None],
  [ 0xcbcb, 'HangulSyllableJjyus', [], 'Display', None],
  [ 0xcbcc, 'HangulSyllableJjyuss', [], 'Display', None],
  [ 0xcbcd, 'HangulSyllableJjyung', [], 'Display', None],
  [ 0xcbce, 'HangulSyllableJjyuj', [], 'Display', None],
  [ 0xcbcf, 'HangulSyllableJjyuc', [], 'Display', None],
  [ 0xcbd0, 'HangulSyllableJjyuk', [], 'Display', None],
  [ 0xcbd1, 'HangulSyllableJjyut', [], 'Display', None],
  [ 0xcbd2, 'HangulSyllableJjyup', [], 'Display', None],
  [ 0xcbd3, 'HangulSyllableJjyuh', [], 'Display', None],
  [ 0xcbd4, 'HangulSyllableJjeu', [], 'Display', None],
  [ 0xcbd5, 'HangulSyllableJjeug', [], 'Display', None],
  [ 0xcbd6, 'HangulSyllableJjeugg', [], 'Display', None],
  [ 0xcbd7, 'HangulSyllableJjeugs', [], 'Display', None],
  [ 0xcbd8, 'HangulSyllableJjeun', [], 'Display', None],
  [ 0xcbd9, 'HangulSyllableJjeunj', [], 'Display', None],
  [ 0xcbda, 'HangulSyllableJjeunh', [], 'Display', None],
  [ 0xcbdb, 'HangulSyllableJjeud', [], 'Display', None],
  [ 0xcbdc, 'HangulSyllableJjeul', [], 'Display', None],
  [ 0xcbdd, 'HangulSyllableJjeulg', [], 'Display', None],
  [ 0xcbde, 'HangulSyllableJjeulm', [], 'Display', None],
  [ 0xcbdf, 'HangulSyllableJjeulb', [], 'Display', None],
  [ 0xcbe0, 'HangulSyllableJjeuls', [], 'Display', None],
  [ 0xcbe1, 'HangulSyllableJjeult', [], 'Display', None],
  [ 0xcbe2, 'HangulSyllableJjeulp', [], 'Display', None],
  [ 0xcbe3, 'HangulSyllableJjeulh', [], 'Display', None],
  [ 0xcbe4, 'HangulSyllableJjeum', [], 'Display', None],
  [ 0xcbe5, 'HangulSyllableJjeub', [], 'Display', None],
  [ 0xcbe6, 'HangulSyllableJjeubs', [], 'Display', None],
  [ 0xcbe7, 'HangulSyllableJjeus', [], 'Display', None],
  [ 0xcbe8, 'HangulSyllableJjeuss', [], 'Display', None],
  [ 0xcbe9, 'HangulSyllableJjeung', [], 'Display', None],
  [ 0xcbea, 'HangulSyllableJjeuj', [], 'Display', None],
  [ 0xcbeb, 'HangulSyllableJjeuc', [], 'Display', None],
  [ 0xcbec, 'HangulSyllableJjeuk', [], 'Display', None],
  [ 0xcbed, 'HangulSyllableJjeut', [], 'Display', None],
  [ 0xcbee, 'HangulSyllableJjeup', [], 'Display', None],
  [ 0xcbef, 'HangulSyllableJjeuh', [], 'Display', None],
  [ 0xcbf0, 'HangulSyllableJjyi', [], 'Display', None],
  [ 0xcbf1, 'HangulSyllableJjyig', [], 'Display', None],
  [ 0xcbf2, 'HangulSyllableJjyigg', [], 'Display', None],
  [ 0xcbf3, 'HangulSyllableJjyigs', [], 'Display', None],
  [ 0xcbf4, 'HangulSyllableJjyin', [], 'Display', None],
  [ 0xcbf5, 'HangulSyllableJjyinj', [], 'Display', None],
  [ 0xcbf6, 'HangulSyllableJjyinh', [], 'Display', None],
  [ 0xcbf7, 'HangulSyllableJjyid', [], 'Display', None],
  [ 0xcbf8, 'HangulSyllableJjyil', [], 'Display', None],
  [ 0xcbf9, 'HangulSyllableJjyilg', [], 'Display', None],
  [ 0xcbfa, 'HangulSyllableJjyilm', [], 'Display', None],
  [ 0xcbfb, 'HangulSyllableJjyilb', [], 'Display', None],
  [ 0xcbfc, 'HangulSyllableJjyils', [], 'Display', None],
  [ 0xcbfd, 'HangulSyllableJjyilt', [], 'Display', None],
  [ 0xcbfe, 'HangulSyllableJjyilp', [], 'Display', None],
  [ 0xcbff, 'HangulSyllableJjyilh', [], 'Display', None],
  [ 0xcc00, 'HangulSyllableJjyim', [], 'Display', None],
  [ 0xcc01, 'HangulSyllableJjyib', [], 'Display', None],
  [ 0xcc02, 'HangulSyllableJjyibs', [], 'Display', None],
  [ 0xcc03, 'HangulSyllableJjyis', [], 'Display', None],
  [ 0xcc04, 'HangulSyllableJjyiss', [], 'Display', None],
  [ 0xcc05, 'HangulSyllableJjying', [], 'Display', None],
  [ 0xcc06, 'HangulSyllableJjyij', [], 'Display', None],
  [ 0xcc07, 'HangulSyllableJjyic', [], 'Display', None],
  [ 0xcc08, 'HangulSyllableJjyik', [], 'Display', None],
  [ 0xcc09, 'HangulSyllableJjyit', [], 'Display', None],
  [ 0xcc0a, 'HangulSyllableJjyip', [], 'Display', None],
  [ 0xcc0b, 'HangulSyllableJjyih', [], 'Display', None],
  [ 0xcc0c, 'HangulSyllableJji', [], 'Display', None],
  [ 0xcc0d, 'HangulSyllableJjig', [], 'Display', None],
  [ 0xcc0e, 'HangulSyllableJjigg', [], 'Display', None],
  [ 0xcc0f, 'HangulSyllableJjigs', [], 'Display', None],
  [ 0xcc10, 'HangulSyllableJjin', [], 'Display', None],
  [ 0xcc11, 'HangulSyllableJjinj', [], 'Display', None],
  [ 0xcc12, 'HangulSyllableJjinh', [], 'Display', None],
  [ 0xcc13, 'HangulSyllableJjid', [], 'Display', None],
  [ 0xcc14, 'HangulSyllableJjil', [], 'Display', None],
  [ 0xcc15, 'HangulSyllableJjilg', [], 'Display', None],
  [ 0xcc16, 'HangulSyllableJjilm', [], 'Display', None],
  [ 0xcc17, 'HangulSyllableJjilb', [], 'Display', None],
  [ 0xcc18, 'HangulSyllableJjils', [], 'Display', None],
  [ 0xcc19, 'HangulSyllableJjilt', [], 'Display', None],
  [ 0xcc1a, 'HangulSyllableJjilp', [], 'Display', None],
  [ 0xcc1b, 'HangulSyllableJjilh', [], 'Display', None],
  [ 0xcc1c, 'HangulSyllableJjim', [], 'Display', None],
  [ 0xcc1d, 'HangulSyllableJjib', [], 'Display', None],
  [ 0xcc1e, 'HangulSyllableJjibs', [], 'Display', None],
  [ 0xcc1f, 'HangulSyllableJjis', [], 'Display', None],
  [ 0xcc20, 'HangulSyllableJjiss', [], 'Display', None],
  [ 0xcc21, 'HangulSyllableJjing', [], 'Display', None],
  [ 0xcc22, 'HangulSyllableJjij', [], 'Display', None],
  [ 0xcc23, 'HangulSyllableJjic', [], 'Display', None],
  [ 0xcc24, 'HangulSyllableJjik', [], 'Display', None],
  [ 0xcc25, 'HangulSyllableJjit', [], 'Display', None],
  [ 0xcc26, 'HangulSyllableJjip', [], 'Display', None],
  [ 0xcc27, 'HangulSyllableJjih', [], 'Display', None],
  [ 0xcc28, 'HangulSyllableCa', [], 'Display', None],
  [ 0xcc29, 'HangulSyllableCag', [], 'Display', None],
  [ 0xcc2a, 'HangulSyllableCagg', [], 'Display', None],
  [ 0xcc2b, 'HangulSyllableCags', [], 'Display', None],
  [ 0xcc2c, 'HangulSyllableCan', [], 'Display', None],
  [ 0xcc2d, 'HangulSyllableCanj', [], 'Display', None],
  [ 0xcc2e, 'HangulSyllableCanh', [], 'Display', None],
  [ 0xcc2f, 'HangulSyllableCad', [], 'Display', None],
  [ 0xcc30, 'HangulSyllableCal', [], 'Display', None],
  [ 0xcc31, 'HangulSyllableCalg', [], 'Display', None],
  [ 0xcc32, 'HangulSyllableCalm', [], 'Display', None],
  [ 0xcc33, 'HangulSyllableCalb', [], 'Display', None],
  [ 0xcc34, 'HangulSyllableCals', [], 'Display', None],
  [ 0xcc35, 'HangulSyllableCalt', [], 'Display', None],
  [ 0xcc36, 'HangulSyllableCalp', [], 'Display', None],
  [ 0xcc37, 'HangulSyllableCalh', [], 'Display', None],
  [ 0xcc38, 'HangulSyllableCam', [], 'Display', None],
  [ 0xcc39, 'HangulSyllableCab', [], 'Display', None],
  [ 0xcc3a, 'HangulSyllableCabs', [], 'Display', None],
  [ 0xcc3b, 'HangulSyllableCas', [], 'Display', None],
  [ 0xcc3c, 'HangulSyllableCass', [], 'Display', None],
  [ 0xcc3d, 'HangulSyllableCang', [], 'Display', None],
  [ 0xcc3e, 'HangulSyllableCaj', [], 'Display', None],
  [ 0xcc3f, 'HangulSyllableCac', [], 'Display', None],
  [ 0xcc40, 'HangulSyllableCak', [], 'Display', None],
  [ 0xcc41, 'HangulSyllableCat', [], 'Display', None],
  [ 0xcc42, 'HangulSyllableCap', [], 'Display', None],
  [ 0xcc43, 'HangulSyllableCah', [], 'Display', None],
  [ 0xcc44, 'HangulSyllableCae', [], 'Display', None],
  [ 0xcc45, 'HangulSyllableCaeg', [], 'Display', None],
  [ 0xcc46, 'HangulSyllableCaegg', [], 'Display', None],
  [ 0xcc47, 'HangulSyllableCaegs', [], 'Display', None],
  [ 0xcc48, 'HangulSyllableCaen', [], 'Display', None],
  [ 0xcc49, 'HangulSyllableCaenj', [], 'Display', None],
  [ 0xcc4a, 'HangulSyllableCaenh', [], 'Display', None],
  [ 0xcc4b, 'HangulSyllableCaed', [], 'Display', None],
  [ 0xcc4c, 'HangulSyllableCael', [], 'Display', None],
  [ 0xcc4d, 'HangulSyllableCaelg', [], 'Display', None],
  [ 0xcc4e, 'HangulSyllableCaelm', [], 'Display', None],
  [ 0xcc4f, 'HangulSyllableCaelb', [], 'Display', None],
  [ 0xcc50, 'HangulSyllableCaels', [], 'Display', None],
  [ 0xcc51, 'HangulSyllableCaelt', [], 'Display', None],
  [ 0xcc52, 'HangulSyllableCaelp', [], 'Display', None],
  [ 0xcc53, 'HangulSyllableCaelh', [], 'Display', None],
  [ 0xcc54, 'HangulSyllableCaem', [], 'Display', None],
  [ 0xcc55, 'HangulSyllableCaeb', [], 'Display', None],
  [ 0xcc56, 'HangulSyllableCaebs', [], 'Display', None],
  [ 0xcc57, 'HangulSyllableCaes', [], 'Display', None],
  [ 0xcc58, 'HangulSyllableCaess', [], 'Display', None],
  [ 0xcc59, 'HangulSyllableCaeng', [], 'Display', None],
  [ 0xcc5a, 'HangulSyllableCaej', [], 'Display', None],
  [ 0xcc5b, 'HangulSyllableCaec', [], 'Display', None],
  [ 0xcc5c, 'HangulSyllableCaek', [], 'Display', None],
  [ 0xcc5d, 'HangulSyllableCaet', [], 'Display', None],
  [ 0xcc5e, 'HangulSyllableCaep', [], 'Display', None],
  [ 0xcc5f, 'HangulSyllableCaeh', [], 'Display', None],
  [ 0xcc60, 'HangulSyllableCya', [], 'Display', None],
  [ 0xcc61, 'HangulSyllableCyag', [], 'Display', None],
  [ 0xcc62, 'HangulSyllableCyagg', [], 'Display', None],
  [ 0xcc63, 'HangulSyllableCyags', [], 'Display', None],
  [ 0xcc64, 'HangulSyllableCyan', [], 'Display', None],
  [ 0xcc65, 'HangulSyllableCyanj', [], 'Display', None],
  [ 0xcc66, 'HangulSyllableCyanh', [], 'Display', None],
  [ 0xcc67, 'HangulSyllableCyad', [], 'Display', None],
  [ 0xcc68, 'HangulSyllableCyal', [], 'Display', None],
  [ 0xcc69, 'HangulSyllableCyalg', [], 'Display', None],
  [ 0xcc6a, 'HangulSyllableCyalm', [], 'Display', None],
  [ 0xcc6b, 'HangulSyllableCyalb', [], 'Display', None],
  [ 0xcc6c, 'HangulSyllableCyals', [], 'Display', None],
  [ 0xcc6d, 'HangulSyllableCyalt', [], 'Display', None],
  [ 0xcc6e, 'HangulSyllableCyalp', [], 'Display', None],
  [ 0xcc6f, 'HangulSyllableCyalh', [], 'Display', None],
  [ 0xcc70, 'HangulSyllableCyam', [], 'Display', None],
  [ 0xcc71, 'HangulSyllableCyab', [], 'Display', None],
  [ 0xcc72, 'HangulSyllableCyabs', [], 'Display', None],
  [ 0xcc73, 'HangulSyllableCyas', [], 'Display', None],
  [ 0xcc74, 'HangulSyllableCyass', [], 'Display', None],
  [ 0xcc75, 'HangulSyllableCyang', [], 'Display', None],
  [ 0xcc76, 'HangulSyllableCyaj', [], 'Display', None],
  [ 0xcc77, 'HangulSyllableCyac', [], 'Display', None],
  [ 0xcc78, 'HangulSyllableCyak', [], 'Display', None],
  [ 0xcc79, 'HangulSyllableCyat', [], 'Display', None],
  [ 0xcc7a, 'HangulSyllableCyap', [], 'Display', None],
  [ 0xcc7b, 'HangulSyllableCyah', [], 'Display', None],
  [ 0xcc7c, 'HangulSyllableCyae', [], 'Display', None],
  [ 0xcc7d, 'HangulSyllableCyaeg', [], 'Display', None],
  [ 0xcc7e, 'HangulSyllableCyaegg', [], 'Display', None],
  [ 0xcc7f, 'HangulSyllableCyaegs', [], 'Display', None],
  [ 0xcc80, 'HangulSyllableCyaen', [], 'Display', None],
  [ 0xcc81, 'HangulSyllableCyaenj', [], 'Display', None],
  [ 0xcc82, 'HangulSyllableCyaenh', [], 'Display', None],
  [ 0xcc83, 'HangulSyllableCyaed', [], 'Display', None],
  [ 0xcc84, 'HangulSyllableCyael', [], 'Display', None],
  [ 0xcc85, 'HangulSyllableCyaelg', [], 'Display', None],
  [ 0xcc86, 'HangulSyllableCyaelm', [], 'Display', None],
  [ 0xcc87, 'HangulSyllableCyaelb', [], 'Display', None],
  [ 0xcc88, 'HangulSyllableCyaels', [], 'Display', None],
  [ 0xcc89, 'HangulSyllableCyaelt', [], 'Display', None],
  [ 0xcc8a, 'HangulSyllableCyaelp', [], 'Display', None],
  [ 0xcc8b, 'HangulSyllableCyaelh', [], 'Display', None],
  [ 0xcc8c, 'HangulSyllableCyaem', [], 'Display', None],
  [ 0xcc8d, 'HangulSyllableCyaeb', [], 'Display', None],
  [ 0xcc8e, 'HangulSyllableCyaebs', [], 'Display', None],
  [ 0xcc8f, 'HangulSyllableCyaes', [], 'Display', None],
  [ 0xcc90, 'HangulSyllableCyaess', [], 'Display', None],
  [ 0xcc91, 'HangulSyllableCyaeng', [], 'Display', None],
  [ 0xcc92, 'HangulSyllableCyaej', [], 'Display', None],
  [ 0xcc93, 'HangulSyllableCyaec', [], 'Display', None],
  [ 0xcc94, 'HangulSyllableCyaek', [], 'Display', None],
  [ 0xcc95, 'HangulSyllableCyaet', [], 'Display', None],
  [ 0xcc96, 'HangulSyllableCyaep', [], 'Display', None],
  [ 0xcc97, 'HangulSyllableCyaeh', [], 'Display', None],
  [ 0xcc98, 'HangulSyllableCeo', [], 'Display', None],
  [ 0xcc99, 'HangulSyllableCeog', [], 'Display', None],
  [ 0xcc9a, 'HangulSyllableCeogg', [], 'Display', None],
  [ 0xcc9b, 'HangulSyllableCeogs', [], 'Display', None],
  [ 0xcc9c, 'HangulSyllableCeon', [], 'Display', None],
  [ 0xcc9d, 'HangulSyllableCeonj', [], 'Display', None],
  [ 0xcc9e, 'HangulSyllableCeonh', [], 'Display', None],
  [ 0xcc9f, 'HangulSyllableCeod', [], 'Display', None],
  [ 0xcca0, 'HangulSyllableCeol', [], 'Display', None],
  [ 0xcca1, 'HangulSyllableCeolg', [], 'Display', None],
  [ 0xcca2, 'HangulSyllableCeolm', [], 'Display', None],
  [ 0xcca3, 'HangulSyllableCeolb', [], 'Display', None],
  [ 0xcca4, 'HangulSyllableCeols', [], 'Display', None],
  [ 0xcca5, 'HangulSyllableCeolt', [], 'Display', None],
  [ 0xcca6, 'HangulSyllableCeolp', [], 'Display', None],
  [ 0xcca7, 'HangulSyllableCeolh', [], 'Display', None],
  [ 0xcca8, 'HangulSyllableCeom', [], 'Display', None],
  [ 0xcca9, 'HangulSyllableCeob', [], 'Display', None],
  [ 0xccaa, 'HangulSyllableCeobs', [], 'Display', None],
  [ 0xccab, 'HangulSyllableCeos', [], 'Display', None],
  [ 0xccac, 'HangulSyllableCeoss', [], 'Display', None],
  [ 0xccad, 'HangulSyllableCeong', [], 'Display', None],
  [ 0xccae, 'HangulSyllableCeoj', [], 'Display', None],
  [ 0xccaf, 'HangulSyllableCeoc', [], 'Display', None],
  [ 0xccb0, 'HangulSyllableCeok', [], 'Display', None],
  [ 0xccb1, 'HangulSyllableCeot', [], 'Display', None],
  [ 0xccb2, 'HangulSyllableCeop', [], 'Display', None],
  [ 0xccb3, 'HangulSyllableCeoh', [], 'Display', None],
  [ 0xccb4, 'HangulSyllableCe', [], 'Display', None],
  [ 0xccb5, 'HangulSyllableCeg', [], 'Display', None],
  [ 0xccb6, 'HangulSyllableCegg', [], 'Display', None],
  [ 0xccb7, 'HangulSyllableCegs', [], 'Display', None],
  [ 0xccb8, 'HangulSyllableCen', [], 'Display', None],
  [ 0xccb9, 'HangulSyllableCenj', [], 'Display', None],
  [ 0xccba, 'HangulSyllableCenh', [], 'Display', None],
  [ 0xccbb, 'HangulSyllableCed', [], 'Display', None],
  [ 0xccbc, 'HangulSyllableCel', [], 'Display', None],
  [ 0xccbd, 'HangulSyllableCelg', [], 'Display', None],
  [ 0xccbe, 'HangulSyllableCelm', [], 'Display', None],
  [ 0xccbf, 'HangulSyllableCelb', [], 'Display', None],
  [ 0xccc0, 'HangulSyllableCels', [], 'Display', None],
  [ 0xccc1, 'HangulSyllableCelt', [], 'Display', None],
  [ 0xccc2, 'HangulSyllableCelp', [], 'Display', None],
  [ 0xccc3, 'HangulSyllableCelh', [], 'Display', None],
  [ 0xccc4, 'HangulSyllableCem', [], 'Display', None],
  [ 0xccc5, 'HangulSyllableCeb', [], 'Display', None],
  [ 0xccc6, 'HangulSyllableCebs', [], 'Display', None],
  [ 0xccc7, 'HangulSyllableCes', [], 'Display', None],
  [ 0xccc8, 'HangulSyllableCess', [], 'Display', None],
  [ 0xccc9, 'HangulSyllableCeng', [], 'Display', None],
  [ 0xccca, 'HangulSyllableCej', [], 'Display', None],
  [ 0xcccb, 'HangulSyllableCec', [], 'Display', None],
  [ 0xcccc, 'HangulSyllableCek', [], 'Display', None],
  [ 0xcccd, 'HangulSyllableCet', [], 'Display', None],
  [ 0xccce, 'HangulSyllableCep', [], 'Display', None],
  [ 0xcccf, 'HangulSyllableCeh', [], 'Display', None],
  [ 0xccd0, 'HangulSyllableCyeo', [], 'Display', None],
  [ 0xccd1, 'HangulSyllableCyeog', [], 'Display', None],
  [ 0xccd2, 'HangulSyllableCyeogg', [], 'Display', None],
  [ 0xccd3, 'HangulSyllableCyeogs', [], 'Display', None],
  [ 0xccd4, 'HangulSyllableCyeon', [], 'Display', None],
  [ 0xccd5, 'HangulSyllableCyeonj', [], 'Display', None],
  [ 0xccd6, 'HangulSyllableCyeonh', [], 'Display', None],
  [ 0xccd7, 'HangulSyllableCyeod', [], 'Display', None],
  [ 0xccd8, 'HangulSyllableCyeol', [], 'Display', None],
  [ 0xccd9, 'HangulSyllableCyeolg', [], 'Display', None],
  [ 0xccda, 'HangulSyllableCyeolm', [], 'Display', None],
  [ 0xccdb, 'HangulSyllableCyeolb', [], 'Display', None],
  [ 0xccdc, 'HangulSyllableCyeols', [], 'Display', None],
  [ 0xccdd, 'HangulSyllableCyeolt', [], 'Display', None],
  [ 0xccde, 'HangulSyllableCyeolp', [], 'Display', None],
  [ 0xccdf, 'HangulSyllableCyeolh', [], 'Display', None],
  [ 0xcce0, 'HangulSyllableCyeom', [], 'Display', None],
  [ 0xcce1, 'HangulSyllableCyeob', [], 'Display', None],
  [ 0xcce2, 'HangulSyllableCyeobs', [], 'Display', None],
  [ 0xcce3, 'HangulSyllableCyeos', [], 'Display', None],
  [ 0xcce4, 'HangulSyllableCyeoss', [], 'Display', None],
  [ 0xcce5, 'HangulSyllableCyeong', [], 'Display', None],
  [ 0xcce6, 'HangulSyllableCyeoj', [], 'Display', None],
  [ 0xcce7, 'HangulSyllableCyeoc', [], 'Display', None],
  [ 0xcce8, 'HangulSyllableCyeok', [], 'Display', None],
  [ 0xcce9, 'HangulSyllableCyeot', [], 'Display', None],
  [ 0xccea, 'HangulSyllableCyeop', [], 'Display', None],
  [ 0xcceb, 'HangulSyllableCyeoh', [], 'Display', None],
  [ 0xccec, 'HangulSyllableCye', [], 'Display', None],
  [ 0xcced, 'HangulSyllableCyeg', [], 'Display', None],
  [ 0xccee, 'HangulSyllableCyegg', [], 'Display', None],
  [ 0xccef, 'HangulSyllableCyegs', [], 'Display', None],
  [ 0xccf0, 'HangulSyllableCyen', [], 'Display', None],
  [ 0xccf1, 'HangulSyllableCyenj', [], 'Display', None],
  [ 0xccf2, 'HangulSyllableCyenh', [], 'Display', None],
  [ 0xccf3, 'HangulSyllableCyed', [], 'Display', None],
  [ 0xccf4, 'HangulSyllableCyel', [], 'Display', None],
  [ 0xccf5, 'HangulSyllableCyelg', [], 'Display', None],
  [ 0xccf6, 'HangulSyllableCyelm', [], 'Display', None],
  [ 0xccf7, 'HangulSyllableCyelb', [], 'Display', None],
  [ 0xccf8, 'HangulSyllableCyels', [], 'Display', None],
  [ 0xccf9, 'HangulSyllableCyelt', [], 'Display', None],
  [ 0xccfa, 'HangulSyllableCyelp', [], 'Display', None],
  [ 0xccfb, 'HangulSyllableCyelh', [], 'Display', None],
  [ 0xccfc, 'HangulSyllableCyem', [], 'Display', None],
  [ 0xccfd, 'HangulSyllableCyeb', [], 'Display', None],
  [ 0xccfe, 'HangulSyllableCyebs', [], 'Display', None],
  [ 0xccff, 'HangulSyllableCyes', [], 'Display', None],
  [ 0xcd00, 'HangulSyllableCyess', [], 'Display', None],
  [ 0xcd01, 'HangulSyllableCyeng', [], 'Display', None],
  [ 0xcd02, 'HangulSyllableCyej', [], 'Display', None],
  [ 0xcd03, 'HangulSyllableCyec', [], 'Display', None],
  [ 0xcd04, 'HangulSyllableCyek', [], 'Display', None],
  [ 0xcd05, 'HangulSyllableCyet', [], 'Display', None],
  [ 0xcd06, 'HangulSyllableCyep', [], 'Display', None],
  [ 0xcd07, 'HangulSyllableCyeh', [], 'Display', None],
  [ 0xcd08, 'HangulSyllableCo', [], 'Display', None],
  [ 0xcd09, 'HangulSyllableCog', [], 'Display', None],
  [ 0xcd0a, 'HangulSyllableCogg', [], 'Display', None],
  [ 0xcd0b, 'HangulSyllableCogs', [], 'Display', None],
  [ 0xcd0c, 'HangulSyllableCon', [], 'Display', None],
  [ 0xcd0d, 'HangulSyllableConj', [], 'Display', None],
  [ 0xcd0e, 'HangulSyllableConh', [], 'Display', None],
  [ 0xcd0f, 'HangulSyllableCod', [], 'Display', None],
  [ 0xcd10, 'HangulSyllableCol', [], 'Display', None],
  [ 0xcd11, 'HangulSyllableColg', [], 'Display', None],
  [ 0xcd12, 'HangulSyllableColm', [], 'Display', None],
  [ 0xcd13, 'HangulSyllableColb', [], 'Display', None],
  [ 0xcd14, 'HangulSyllableCols', [], 'Display', None],
  [ 0xcd15, 'HangulSyllableColt', [], 'Display', None],
  [ 0xcd16, 'HangulSyllableColp', [], 'Display', None],
  [ 0xcd17, 'HangulSyllableColh', [], 'Display', None],
  [ 0xcd18, 'HangulSyllableCom', [], 'Display', None],
  [ 0xcd19, 'HangulSyllableCob', [], 'Display', None],
  [ 0xcd1a, 'HangulSyllableCobs', [], 'Display', None],
  [ 0xcd1b, 'HangulSyllableCos', [], 'Display', None],
  [ 0xcd1c, 'HangulSyllableCoss', [], 'Display', None],
  [ 0xcd1d, 'HangulSyllableCong', [], 'Display', None],
  [ 0xcd1e, 'HangulSyllableCoj', [], 'Display', None],
  [ 0xcd1f, 'HangulSyllableCoc', [], 'Display', None],
  [ 0xcd20, 'HangulSyllableCok', [], 'Display', None],
  [ 0xcd21, 'HangulSyllableCot', [], 'Display', None],
  [ 0xcd22, 'HangulSyllableCop', [], 'Display', None],
  [ 0xcd23, 'HangulSyllableCoh', [], 'Display', None],
  [ 0xcd24, 'HangulSyllableCwa', [], 'Display', None],
  [ 0xcd25, 'HangulSyllableCwag', [], 'Display', None],
  [ 0xcd26, 'HangulSyllableCwagg', [], 'Display', None],
  [ 0xcd27, 'HangulSyllableCwags', [], 'Display', None],
  [ 0xcd28, 'HangulSyllableCwan', [], 'Display', None],
  [ 0xcd29, 'HangulSyllableCwanj', [], 'Display', None],
  [ 0xcd2a, 'HangulSyllableCwanh', [], 'Display', None],
  [ 0xcd2b, 'HangulSyllableCwad', [], 'Display', None],
  [ 0xcd2c, 'HangulSyllableCwal', [], 'Display', None],
  [ 0xcd2d, 'HangulSyllableCwalg', [], 'Display', None],
  [ 0xcd2e, 'HangulSyllableCwalm', [], 'Display', None],
  [ 0xcd2f, 'HangulSyllableCwalb', [], 'Display', None],
  [ 0xcd30, 'HangulSyllableCwals', [], 'Display', None],
  [ 0xcd31, 'HangulSyllableCwalt', [], 'Display', None],
  [ 0xcd32, 'HangulSyllableCwalp', [], 'Display', None],
  [ 0xcd33, 'HangulSyllableCwalh', [], 'Display', None],
  [ 0xcd34, 'HangulSyllableCwam', [], 'Display', None],
  [ 0xcd35, 'HangulSyllableCwab', [], 'Display', None],
  [ 0xcd36, 'HangulSyllableCwabs', [], 'Display', None],
  [ 0xcd37, 'HangulSyllableCwas', [], 'Display', None],
  [ 0xcd38, 'HangulSyllableCwass', [], 'Display', None],
  [ 0xcd39, 'HangulSyllableCwang', [], 'Display', None],
  [ 0xcd3a, 'HangulSyllableCwaj', [], 'Display', None],
  [ 0xcd3b, 'HangulSyllableCwac', [], 'Display', None],
  [ 0xcd3c, 'HangulSyllableCwak', [], 'Display', None],
  [ 0xcd3d, 'HangulSyllableCwat', [], 'Display', None],
  [ 0xcd3e, 'HangulSyllableCwap', [], 'Display', None],
  [ 0xcd3f, 'HangulSyllableCwah', [], 'Display', None],
  [ 0xcd40, 'HangulSyllableCwae', [], 'Display', None],
  [ 0xcd41, 'HangulSyllableCwaeg', [], 'Display', None],
  [ 0xcd42, 'HangulSyllableCwaegg', [], 'Display', None],
  [ 0xcd43, 'HangulSyllableCwaegs', [], 'Display', None],
  [ 0xcd44, 'HangulSyllableCwaen', [], 'Display', None],
  [ 0xcd45, 'HangulSyllableCwaenj', [], 'Display', None],
  [ 0xcd46, 'HangulSyllableCwaenh', [], 'Display', None],
  [ 0xcd47, 'HangulSyllableCwaed', [], 'Display', None],
  [ 0xcd48, 'HangulSyllableCwael', [], 'Display', None],
  [ 0xcd49, 'HangulSyllableCwaelg', [], 'Display', None],
  [ 0xcd4a, 'HangulSyllableCwaelm', [], 'Display', None],
  [ 0xcd4b, 'HangulSyllableCwaelb', [], 'Display', None],
  [ 0xcd4c, 'HangulSyllableCwaels', [], 'Display', None],
  [ 0xcd4d, 'HangulSyllableCwaelt', [], 'Display', None],
  [ 0xcd4e, 'HangulSyllableCwaelp', [], 'Display', None],
  [ 0xcd4f, 'HangulSyllableCwaelh', [], 'Display', None],
  [ 0xcd50, 'HangulSyllableCwaem', [], 'Display', None],
  [ 0xcd51, 'HangulSyllableCwaeb', [], 'Display', None],
  [ 0xcd52, 'HangulSyllableCwaebs', [], 'Display', None],
  [ 0xcd53, 'HangulSyllableCwaes', [], 'Display', None],
  [ 0xcd54, 'HangulSyllableCwaess', [], 'Display', None],
  [ 0xcd55, 'HangulSyllableCwaeng', [], 'Display', None],
  [ 0xcd56, 'HangulSyllableCwaej', [], 'Display', None],
  [ 0xcd57, 'HangulSyllableCwaec', [], 'Display', None],
  [ 0xcd58, 'HangulSyllableCwaek', [], 'Display', None],
  [ 0xcd59, 'HangulSyllableCwaet', [], 'Display', None],
  [ 0xcd5a, 'HangulSyllableCwaep', [], 'Display', None],
  [ 0xcd5b, 'HangulSyllableCwaeh', [], 'Display', None],
  [ 0xcd5c, 'HangulSyllableCoe', [], 'Display', None],
  [ 0xcd5d, 'HangulSyllableCoeg', [], 'Display', None],
  [ 0xcd5e, 'HangulSyllableCoegg', [], 'Display', None],
  [ 0xcd5f, 'HangulSyllableCoegs', [], 'Display', None],
  [ 0xcd60, 'HangulSyllableCoen', [], 'Display', None],
  [ 0xcd61, 'HangulSyllableCoenj', [], 'Display', None],
  [ 0xcd62, 'HangulSyllableCoenh', [], 'Display', None],
  [ 0xcd63, 'HangulSyllableCoed', [], 'Display', None],
  [ 0xcd64, 'HangulSyllableCoel', [], 'Display', None],
  [ 0xcd65, 'HangulSyllableCoelg', [], 'Display', None],
  [ 0xcd66, 'HangulSyllableCoelm', [], 'Display', None],
  [ 0xcd67, 'HangulSyllableCoelb', [], 'Display', None],
  [ 0xcd68, 'HangulSyllableCoels', [], 'Display', None],
  [ 0xcd69, 'HangulSyllableCoelt', [], 'Display', None],
  [ 0xcd6a, 'HangulSyllableCoelp', [], 'Display', None],
  [ 0xcd6b, 'HangulSyllableCoelh', [], 'Display', None],
  [ 0xcd6c, 'HangulSyllableCoem', [], 'Display', None],
  [ 0xcd6d, 'HangulSyllableCoeb', [], 'Display', None],
  [ 0xcd6e, 'HangulSyllableCoebs', [], 'Display', None],
  [ 0xcd6f, 'HangulSyllableCoes', [], 'Display', None],
  [ 0xcd70, 'HangulSyllableCoess', [], 'Display', None],
  [ 0xcd71, 'HangulSyllableCoeng', [], 'Display', None],
  [ 0xcd72, 'HangulSyllableCoej', [], 'Display', None],
  [ 0xcd73, 'HangulSyllableCoec', [], 'Display', None],
  [ 0xcd74, 'HangulSyllableCoek', [], 'Display', None],
  [ 0xcd75, 'HangulSyllableCoet', [], 'Display', None],
  [ 0xcd76, 'HangulSyllableCoep', [], 'Display', None],
  [ 0xcd77, 'HangulSyllableCoeh', [], 'Display', None],
  [ 0xcd78, 'HangulSyllableCyo', [], 'Display', None],
  [ 0xcd79, 'HangulSyllableCyog', [], 'Display', None],
  [ 0xcd7a, 'HangulSyllableCyogg', [], 'Display', None],
  [ 0xcd7b, 'HangulSyllableCyogs', [], 'Display', None],
  [ 0xcd7c, 'HangulSyllableCyon', [], 'Display', None],
  [ 0xcd7d, 'HangulSyllableCyonj', [], 'Display', None],
  [ 0xcd7e, 'HangulSyllableCyonh', [], 'Display', None],
  [ 0xcd7f, 'HangulSyllableCyod', [], 'Display', None],
  [ 0xcd80, 'HangulSyllableCyol', [], 'Display', None],
  [ 0xcd81, 'HangulSyllableCyolg', [], 'Display', None],
  [ 0xcd82, 'HangulSyllableCyolm', [], 'Display', None],
  [ 0xcd83, 'HangulSyllableCyolb', [], 'Display', None],
  [ 0xcd84, 'HangulSyllableCyols', [], 'Display', None],
  [ 0xcd85, 'HangulSyllableCyolt', [], 'Display', None],
  [ 0xcd86, 'HangulSyllableCyolp', [], 'Display', None],
  [ 0xcd87, 'HangulSyllableCyolh', [], 'Display', None],
  [ 0xcd88, 'HangulSyllableCyom', [], 'Display', None],
  [ 0xcd89, 'HangulSyllableCyob', [], 'Display', None],
  [ 0xcd8a, 'HangulSyllableCyobs', [], 'Display', None],
  [ 0xcd8b, 'HangulSyllableCyos', [], 'Display', None],
  [ 0xcd8c, 'HangulSyllableCyoss', [], 'Display', None],
  [ 0xcd8d, 'HangulSyllableCyong', [], 'Display', None],
  [ 0xcd8e, 'HangulSyllableCyoj', [], 'Display', None],
  [ 0xcd8f, 'HangulSyllableCyoc', [], 'Display', None],
  [ 0xcd90, 'HangulSyllableCyok', [], 'Display', None],
  [ 0xcd91, 'HangulSyllableCyot', [], 'Display', None],
  [ 0xcd92, 'HangulSyllableCyop', [], 'Display', None],
  [ 0xcd93, 'HangulSyllableCyoh', [], 'Display', None],
  [ 0xcd94, 'HangulSyllableCu', [], 'Display', None],
  [ 0xcd95, 'HangulSyllableCug', [], 'Display', None],
  [ 0xcd96, 'HangulSyllableCugg', [], 'Display', None],
  [ 0xcd97, 'HangulSyllableCugs', [], 'Display', None],
  [ 0xcd98, 'HangulSyllableCun', [], 'Display', None],
  [ 0xcd99, 'HangulSyllableCunj', [], 'Display', None],
  [ 0xcd9a, 'HangulSyllableCunh', [], 'Display', None],
  [ 0xcd9b, 'HangulSyllableCud', [], 'Display', None],
  [ 0xcd9c, 'HangulSyllableCul', [], 'Display', None],
  [ 0xcd9d, 'HangulSyllableCulg', [], 'Display', None],
  [ 0xcd9e, 'HangulSyllableCulm', [], 'Display', None],
  [ 0xcd9f, 'HangulSyllableCulb', [], 'Display', None],
  [ 0xcda0, 'HangulSyllableCuls', [], 'Display', None],
  [ 0xcda1, 'HangulSyllableCult', [], 'Display', None],
  [ 0xcda2, 'HangulSyllableCulp', [], 'Display', None],
  [ 0xcda3, 'HangulSyllableCulh', [], 'Display', None],
  [ 0xcda4, 'HangulSyllableCum', [], 'Display', None],
  [ 0xcda5, 'HangulSyllableCub', [], 'Display', None],
  [ 0xcda6, 'HangulSyllableCubs', [], 'Display', None],
  [ 0xcda7, 'HangulSyllableCus', [], 'Display', None],
  [ 0xcda8, 'HangulSyllableCuss', [], 'Display', None],
  [ 0xcda9, 'HangulSyllableCung', [], 'Display', None],
  [ 0xcdaa, 'HangulSyllableCuj', [], 'Display', None],
  [ 0xcdab, 'HangulSyllableCuc', [], 'Display', None],
  [ 0xcdac, 'HangulSyllableCuk', [], 'Display', None],
  [ 0xcdad, 'HangulSyllableCut', [], 'Display', None],
  [ 0xcdae, 'HangulSyllableCup', [], 'Display', None],
  [ 0xcdaf, 'HangulSyllableCuh', [], 'Display', None],
  [ 0xcdb0, 'HangulSyllableCweo', [], 'Display', None],
  [ 0xcdb1, 'HangulSyllableCweog', [], 'Display', None],
  [ 0xcdb2, 'HangulSyllableCweogg', [], 'Display', None],
  [ 0xcdb3, 'HangulSyllableCweogs', [], 'Display', None],
  [ 0xcdb4, 'HangulSyllableCweon', [], 'Display', None],
  [ 0xcdb5, 'HangulSyllableCweonj', [], 'Display', None],
  [ 0xcdb6, 'HangulSyllableCweonh', [], 'Display', None],
  [ 0xcdb7, 'HangulSyllableCweod', [], 'Display', None],
  [ 0xcdb8, 'HangulSyllableCweol', [], 'Display', None],
  [ 0xcdb9, 'HangulSyllableCweolg', [], 'Display', None],
  [ 0xcdba, 'HangulSyllableCweolm', [], 'Display', None],
  [ 0xcdbb, 'HangulSyllableCweolb', [], 'Display', None],
  [ 0xcdbc, 'HangulSyllableCweols', [], 'Display', None],
  [ 0xcdbd, 'HangulSyllableCweolt', [], 'Display', None],
  [ 0xcdbe, 'HangulSyllableCweolp', [], 'Display', None],
  [ 0xcdbf, 'HangulSyllableCweolh', [], 'Display', None],
  [ 0xcdc0, 'HangulSyllableCweom', [], 'Display', None],
  [ 0xcdc1, 'HangulSyllableCweob', [], 'Display', None],
  [ 0xcdc2, 'HangulSyllableCweobs', [], 'Display', None],
  [ 0xcdc3, 'HangulSyllableCweos', [], 'Display', None],
  [ 0xcdc4, 'HangulSyllableCweoss', [], 'Display', None],
  [ 0xcdc5, 'HangulSyllableCweong', [], 'Display', None],
  [ 0xcdc6, 'HangulSyllableCweoj', [], 'Display', None],
  [ 0xcdc7, 'HangulSyllableCweoc', [], 'Display', None],
  [ 0xcdc8, 'HangulSyllableCweok', [], 'Display', None],
  [ 0xcdc9, 'HangulSyllableCweot', [], 'Display', None],
  [ 0xcdca, 'HangulSyllableCweop', [], 'Display', None],
  [ 0xcdcb, 'HangulSyllableCweoh', [], 'Display', None],
  [ 0xcdcc, 'HangulSyllableCwe', [], 'Display', None],
  [ 0xcdcd, 'HangulSyllableCweg', [], 'Display', None],
  [ 0xcdce, 'HangulSyllableCwegg', [], 'Display', None],
  [ 0xcdcf, 'HangulSyllableCwegs', [], 'Display', None],
  [ 0xcdd0, 'HangulSyllableCwen', [], 'Display', None],
  [ 0xcdd1, 'HangulSyllableCwenj', [], 'Display', None],
  [ 0xcdd2, 'HangulSyllableCwenh', [], 'Display', None],
  [ 0xcdd3, 'HangulSyllableCwed', [], 'Display', None],
  [ 0xcdd4, 'HangulSyllableCwel', [], 'Display', None],
  [ 0xcdd5, 'HangulSyllableCwelg', [], 'Display', None],
  [ 0xcdd6, 'HangulSyllableCwelm', [], 'Display', None],
  [ 0xcdd7, 'HangulSyllableCwelb', [], 'Display', None],
  [ 0xcdd8, 'HangulSyllableCwels', [], 'Display', None],
  [ 0xcdd9, 'HangulSyllableCwelt', [], 'Display', None],
  [ 0xcdda, 'HangulSyllableCwelp', [], 'Display', None],
  [ 0xcddb, 'HangulSyllableCwelh', [], 'Display', None],
  [ 0xcddc, 'HangulSyllableCwem', [], 'Display', None],
  [ 0xcddd, 'HangulSyllableCweb', [], 'Display', None],
  [ 0xcdde, 'HangulSyllableCwebs', [], 'Display', None],
  [ 0xcddf, 'HangulSyllableCwes', [], 'Display', None],
  [ 0xcde0, 'HangulSyllableCwess', [], 'Display', None],
  [ 0xcde1, 'HangulSyllableCweng', [], 'Display', None],
  [ 0xcde2, 'HangulSyllableCwej', [], 'Display', None],
  [ 0xcde3, 'HangulSyllableCwec', [], 'Display', None],
  [ 0xcde4, 'HangulSyllableCwek', [], 'Display', None],
  [ 0xcde5, 'HangulSyllableCwet', [], 'Display', None],
  [ 0xcde6, 'HangulSyllableCwep', [], 'Display', None],
  [ 0xcde7, 'HangulSyllableCweh', [], 'Display', None],
  [ 0xcde8, 'HangulSyllableCwi', [], 'Display', None],
  [ 0xcde9, 'HangulSyllableCwig', [], 'Display', None],
  [ 0xcdea, 'HangulSyllableCwigg', [], 'Display', None],
  [ 0xcdeb, 'HangulSyllableCwigs', [], 'Display', None],
  [ 0xcdec, 'HangulSyllableCwin', [], 'Display', None],
  [ 0xcded, 'HangulSyllableCwinj', [], 'Display', None],
  [ 0xcdee, 'HangulSyllableCwinh', [], 'Display', None],
  [ 0xcdef, 'HangulSyllableCwid', [], 'Display', None],
  [ 0xcdf0, 'HangulSyllableCwil', [], 'Display', None],
  [ 0xcdf1, 'HangulSyllableCwilg', [], 'Display', None],
  [ 0xcdf2, 'HangulSyllableCwilm', [], 'Display', None],
  [ 0xcdf3, 'HangulSyllableCwilb', [], 'Display', None],
  [ 0xcdf4, 'HangulSyllableCwils', [], 'Display', None],
  [ 0xcdf5, 'HangulSyllableCwilt', [], 'Display', None],
  [ 0xcdf6, 'HangulSyllableCwilp', [], 'Display', None],
  [ 0xcdf7, 'HangulSyllableCwilh', [], 'Display', None],
  [ 0xcdf8, 'HangulSyllableCwim', [], 'Display', None],
  [ 0xcdf9, 'HangulSyllableCwib', [], 'Display', None],
  [ 0xcdfa, 'HangulSyllableCwibs', [], 'Display', None],
  [ 0xcdfb, 'HangulSyllableCwis', [], 'Display', None],
  [ 0xcdfc, 'HangulSyllableCwiss', [], 'Display', None],
  [ 0xcdfd, 'HangulSyllableCwing', [], 'Display', None],
  [ 0xcdfe, 'HangulSyllableCwij', [], 'Display', None],
  [ 0xcdff, 'HangulSyllableCwic', [], 'Display', None],
  [ 0xce00, 'HangulSyllableCwik', [], 'Display', None],
  [ 0xce01, 'HangulSyllableCwit', [], 'Display', None],
  [ 0xce02, 'HangulSyllableCwip', [], 'Display', None],
  [ 0xce03, 'HangulSyllableCwih', [], 'Display', None],
  [ 0xce04, 'HangulSyllableCyu', [], 'Display', None],
  [ 0xce05, 'HangulSyllableCyug', [], 'Display', None],
  [ 0xce06, 'HangulSyllableCyugg', [], 'Display', None],
  [ 0xce07, 'HangulSyllableCyugs', [], 'Display', None],
  [ 0xce08, 'HangulSyllableCyun', [], 'Display', None],
  [ 0xce09, 'HangulSyllableCyunj', [], 'Display', None],
  [ 0xce0a, 'HangulSyllableCyunh', [], 'Display', None],
  [ 0xce0b, 'HangulSyllableCyud', [], 'Display', None],
  [ 0xce0c, 'HangulSyllableCyul', [], 'Display', None],
  [ 0xce0d, 'HangulSyllableCyulg', [], 'Display', None],
  [ 0xce0e, 'HangulSyllableCyulm', [], 'Display', None],
  [ 0xce0f, 'HangulSyllableCyulb', [], 'Display', None],
  [ 0xce10, 'HangulSyllableCyuls', [], 'Display', None],
  [ 0xce11, 'HangulSyllableCyult', [], 'Display', None],
  [ 0xce12, 'HangulSyllableCyulp', [], 'Display', None],
  [ 0xce13, 'HangulSyllableCyulh', [], 'Display', None],
  [ 0xce14, 'HangulSyllableCyum', [], 'Display', None],
  [ 0xce15, 'HangulSyllableCyub', [], 'Display', None],
  [ 0xce16, 'HangulSyllableCyubs', [], 'Display', None],
  [ 0xce17, 'HangulSyllableCyus', [], 'Display', None],
  [ 0xce18, 'HangulSyllableCyuss', [], 'Display', None],
  [ 0xce19, 'HangulSyllableCyung', [], 'Display', None],
  [ 0xce1a, 'HangulSyllableCyuj', [], 'Display', None],
  [ 0xce1b, 'HangulSyllableCyuc', [], 'Display', None],
  [ 0xce1c, 'HangulSyllableCyuk', [], 'Display', None],
  [ 0xce1d, 'HangulSyllableCyut', [], 'Display', None],
  [ 0xce1e, 'HangulSyllableCyup', [], 'Display', None],
  [ 0xce1f, 'HangulSyllableCyuh', [], 'Display', None],
  [ 0xce20, 'HangulSyllableCeu', [], 'Display', None],
  [ 0xce21, 'HangulSyllableCeug', [], 'Display', None],
  [ 0xce22, 'HangulSyllableCeugg', [], 'Display', None],
  [ 0xce23, 'HangulSyllableCeugs', [], 'Display', None],
  [ 0xce24, 'HangulSyllableCeun', [], 'Display', None],
  [ 0xce25, 'HangulSyllableCeunj', [], 'Display', None],
  [ 0xce26, 'HangulSyllableCeunh', [], 'Display', None],
  [ 0xce27, 'HangulSyllableCeud', [], 'Display', None],
  [ 0xce28, 'HangulSyllableCeul', [], 'Display', None],
  [ 0xce29, 'HangulSyllableCeulg', [], 'Display', None],
  [ 0xce2a, 'HangulSyllableCeulm', [], 'Display', None],
  [ 0xce2b, 'HangulSyllableCeulb', [], 'Display', None],
  [ 0xce2c, 'HangulSyllableCeuls', [], 'Display', None],
  [ 0xce2d, 'HangulSyllableCeult', [], 'Display', None],
  [ 0xce2e, 'HangulSyllableCeulp', [], 'Display', None],
  [ 0xce2f, 'HangulSyllableCeulh', [], 'Display', None],
  [ 0xce30, 'HangulSyllableCeum', [], 'Display', None],
  [ 0xce31, 'HangulSyllableCeub', [], 'Display', None],
  [ 0xce32, 'HangulSyllableCeubs', [], 'Display', None],
  [ 0xce33, 'HangulSyllableCeus', [], 'Display', None],
  [ 0xce34, 'HangulSyllableCeuss', [], 'Display', None],
  [ 0xce35, 'HangulSyllableCeung', [], 'Display', None],
  [ 0xce36, 'HangulSyllableCeuj', [], 'Display', None],
  [ 0xce37, 'HangulSyllableCeuc', [], 'Display', None],
  [ 0xce38, 'HangulSyllableCeuk', [], 'Display', None],
  [ 0xce39, 'HangulSyllableCeut', [], 'Display', None],
  [ 0xce3a, 'HangulSyllableCeup', [], 'Display', None],
  [ 0xce3b, 'HangulSyllableCeuh', [], 'Display', None],
  [ 0xce3c, 'HangulSyllableCyi', [], 'Display', None],
  [ 0xce3d, 'HangulSyllableCyig', [], 'Display', None],
  [ 0xce3e, 'HangulSyllableCyigg', [], 'Display', None],
  [ 0xce3f, 'HangulSyllableCyigs', [], 'Display', None],
  [ 0xce40, 'HangulSyllableCyin', [], 'Display', None],
  [ 0xce41, 'HangulSyllableCyinj', [], 'Display', None],
  [ 0xce42, 'HangulSyllableCyinh', [], 'Display', None],
  [ 0xce43, 'HangulSyllableCyid', [], 'Display', None],
  [ 0xce44, 'HangulSyllableCyil', [], 'Display', None],
  [ 0xce45, 'HangulSyllableCyilg', [], 'Display', None],
  [ 0xce46, 'HangulSyllableCyilm', [], 'Display', None],
  [ 0xce47, 'HangulSyllableCyilb', [], 'Display', None],
  [ 0xce48, 'HangulSyllableCyils', [], 'Display', None],
  [ 0xce49, 'HangulSyllableCyilt', [], 'Display', None],
  [ 0xce4a, 'HangulSyllableCyilp', [], 'Display', None],
  [ 0xce4b, 'HangulSyllableCyilh', [], 'Display', None],
  [ 0xce4c, 'HangulSyllableCyim', [], 'Display', None],
  [ 0xce4d, 'HangulSyllableCyib', [], 'Display', None],
  [ 0xce4e, 'HangulSyllableCyibs', [], 'Display', None],
  [ 0xce4f, 'HangulSyllableCyis', [], 'Display', None],
  [ 0xce50, 'HangulSyllableCyiss', [], 'Display', None],
  [ 0xce51, 'HangulSyllableCying', [], 'Display', None],
  [ 0xce52, 'HangulSyllableCyij', [], 'Display', None],
  [ 0xce53, 'HangulSyllableCyic', [], 'Display', None],
  [ 0xce54, 'HangulSyllableCyik', [], 'Display', None],
  [ 0xce55, 'HangulSyllableCyit', [], 'Display', None],
  [ 0xce56, 'HangulSyllableCyip', [], 'Display', None],
  [ 0xce57, 'HangulSyllableCyih', [], 'Display', None],
  [ 0xce58, 'HangulSyllableCi', [], 'Display', None],
  [ 0xce59, 'HangulSyllableCig', [], 'Display', None],
  [ 0xce5a, 'HangulSyllableCigg', [], 'Display', None],
  [ 0xce5b, 'HangulSyllableCigs', [], 'Display', None],
  [ 0xce5c, 'HangulSyllableCin', [], 'Display', None],
  [ 0xce5d, 'HangulSyllableCinj', [], 'Display', None],
  [ 0xce5e, 'HangulSyllableCinh', [], 'Display', None],
  [ 0xce5f, 'HangulSyllableCid', [], 'Display', None],
  [ 0xce60, 'HangulSyllableCil', [], 'Display', None],
  [ 0xce61, 'HangulSyllableCilg', [], 'Display', None],
  [ 0xce62, 'HangulSyllableCilm', [], 'Display', None],
  [ 0xce63, 'HangulSyllableCilb', [], 'Display', None],
  [ 0xce64, 'HangulSyllableCils', [], 'Display', None],
  [ 0xce65, 'HangulSyllableCilt', [], 'Display', None],
  [ 0xce66, 'HangulSyllableCilp', [], 'Display', None],
  [ 0xce67, 'HangulSyllableCilh', [], 'Display', None],
  [ 0xce68, 'HangulSyllableCim', [], 'Display', None],
  [ 0xce69, 'HangulSyllableCib', [], 'Display', None],
  [ 0xce6a, 'HangulSyllableCibs', [], 'Display', None],
  [ 0xce6b, 'HangulSyllableCis', [], 'Display', None],
  [ 0xce6c, 'HangulSyllableCiss', [], 'Display', None],
  [ 0xce6d, 'HangulSyllableCing', [], 'Display', None],
  [ 0xce6e, 'HangulSyllableCij', [], 'Display', None],
  [ 0xce6f, 'HangulSyllableCic', [], 'Display', None],
  [ 0xce70, 'HangulSyllableCik', [], 'Display', None],
  [ 0xce71, 'HangulSyllableCit', [], 'Display', None],
  [ 0xce72, 'HangulSyllableCip', [], 'Display', None],
  [ 0xce73, 'HangulSyllableCih', [], 'Display', None],
  [ 0xce74, 'HangulSyllableKa', [], 'Display', None],
  [ 0xce75, 'HangulSyllableKag', [], 'Display', None],
  [ 0xce76, 'HangulSyllableKagg', [], 'Display', None],
  [ 0xce77, 'HangulSyllableKags', [], 'Display', None],
  [ 0xce78, 'HangulSyllableKan', [], 'Display', None],
  [ 0xce79, 'HangulSyllableKanj', [], 'Display', None],
  [ 0xce7a, 'HangulSyllableKanh', [], 'Display', None],
  [ 0xce7b, 'HangulSyllableKad', [], 'Display', None],
  [ 0xce7c, 'HangulSyllableKal', [], 'Display', None],
  [ 0xce7d, 'HangulSyllableKalg', [], 'Display', None],
  [ 0xce7e, 'HangulSyllableKalm', [], 'Display', None],
  [ 0xce7f, 'HangulSyllableKalb', [], 'Display', None],
  [ 0xce80, 'HangulSyllableKals', [], 'Display', None],
  [ 0xce81, 'HangulSyllableKalt', [], 'Display', None],
  [ 0xce82, 'HangulSyllableKalp', [], 'Display', None],
  [ 0xce83, 'HangulSyllableKalh', [], 'Display', None],
  [ 0xce84, 'HangulSyllableKam', [], 'Display', None],
  [ 0xce85, 'HangulSyllableKab', [], 'Display', None],
  [ 0xce86, 'HangulSyllableKabs', [], 'Display', None],
  [ 0xce87, 'HangulSyllableKas', [], 'Display', None],
  [ 0xce88, 'HangulSyllableKass', [], 'Display', None],
  [ 0xce89, 'HangulSyllableKang', [], 'Display', None],
  [ 0xce8a, 'HangulSyllableKaj', [], 'Display', None],
  [ 0xce8b, 'HangulSyllableKac', [], 'Display', None],
  [ 0xce8c, 'HangulSyllableKak', [], 'Display', None],
  [ 0xce8d, 'HangulSyllableKat', [], 'Display', None],
  [ 0xce8e, 'HangulSyllableKap', [], 'Display', None],
  [ 0xce8f, 'HangulSyllableKah', [], 'Display', None],
  [ 0xce90, 'HangulSyllableKae', [], 'Display', None],
  [ 0xce91, 'HangulSyllableKaeg', [], 'Display', None],
  [ 0xce92, 'HangulSyllableKaegg', [], 'Display', None],
  [ 0xce93, 'HangulSyllableKaegs', [], 'Display', None],
  [ 0xce94, 'HangulSyllableKaen', [], 'Display', None],
  [ 0xce95, 'HangulSyllableKaenj', [], 'Display', None],
  [ 0xce96, 'HangulSyllableKaenh', [], 'Display', None],
  [ 0xce97, 'HangulSyllableKaed', [], 'Display', None],
  [ 0xce98, 'HangulSyllableKael', [], 'Display', None],
  [ 0xce99, 'HangulSyllableKaelg', [], 'Display', None],
  [ 0xce9a, 'HangulSyllableKaelm', [], 'Display', None],
  [ 0xce9b, 'HangulSyllableKaelb', [], 'Display', None],
  [ 0xce9c, 'HangulSyllableKaels', [], 'Display', None],
  [ 0xce9d, 'HangulSyllableKaelt', [], 'Display', None],
  [ 0xce9e, 'HangulSyllableKaelp', [], 'Display', None],
  [ 0xce9f, 'HangulSyllableKaelh', [], 'Display', None],
  [ 0xcea0, 'HangulSyllableKaem', [], 'Display', None],
  [ 0xcea1, 'HangulSyllableKaeb', [], 'Display', None],
  [ 0xcea2, 'HangulSyllableKaebs', [], 'Display', None],
  [ 0xcea3, 'HangulSyllableKaes', [], 'Display', None],
  [ 0xcea4, 'HangulSyllableKaess', [], 'Display', None],
  [ 0xcea5, 'HangulSyllableKaeng', [], 'Display', None],
  [ 0xcea6, 'HangulSyllableKaej', [], 'Display', None],
  [ 0xcea7, 'HangulSyllableKaec', [], 'Display', None],
  [ 0xcea8, 'HangulSyllableKaek', [], 'Display', None],
  [ 0xcea9, 'HangulSyllableKaet', [], 'Display', None],
  [ 0xceaa, 'HangulSyllableKaep', [], 'Display', None],
  [ 0xceab, 'HangulSyllableKaeh', [], 'Display', None],
  [ 0xceac, 'HangulSyllableKya', [], 'Display', None],
  [ 0xcead, 'HangulSyllableKyag', [], 'Display', None],
  [ 0xceae, 'HangulSyllableKyagg', [], 'Display', None],
  [ 0xceaf, 'HangulSyllableKyags', [], 'Display', None],
  [ 0xceb0, 'HangulSyllableKyan', [], 'Display', None],
  [ 0xceb1, 'HangulSyllableKyanj', [], 'Display', None],
  [ 0xceb2, 'HangulSyllableKyanh', [], 'Display', None],
  [ 0xceb3, 'HangulSyllableKyad', [], 'Display', None],
  [ 0xceb4, 'HangulSyllableKyal', [], 'Display', None],
  [ 0xceb5, 'HangulSyllableKyalg', [], 'Display', None],
  [ 0xceb6, 'HangulSyllableKyalm', [], 'Display', None],
  [ 0xceb7, 'HangulSyllableKyalb', [], 'Display', None],
  [ 0xceb8, 'HangulSyllableKyals', [], 'Display', None],
  [ 0xceb9, 'HangulSyllableKyalt', [], 'Display', None],
  [ 0xceba, 'HangulSyllableKyalp', [], 'Display', None],
  [ 0xcebb, 'HangulSyllableKyalh', [], 'Display', None],
  [ 0xcebc, 'HangulSyllableKyam', [], 'Display', None],
  [ 0xcebd, 'HangulSyllableKyab', [], 'Display', None],
  [ 0xcebe, 'HangulSyllableKyabs', [], 'Display', None],
  [ 0xcebf, 'HangulSyllableKyas', [], 'Display', None],
  [ 0xcec0, 'HangulSyllableKyass', [], 'Display', None],
  [ 0xcec1, 'HangulSyllableKyang', [], 'Display', None],
  [ 0xcec2, 'HangulSyllableKyaj', [], 'Display', None],
  [ 0xcec3, 'HangulSyllableKyac', [], 'Display', None],
  [ 0xcec4, 'HangulSyllableKyak', [], 'Display', None],
  [ 0xcec5, 'HangulSyllableKyat', [], 'Display', None],
  [ 0xcec6, 'HangulSyllableKyap', [], 'Display', None],
  [ 0xcec7, 'HangulSyllableKyah', [], 'Display', None],
  [ 0xcec8, 'HangulSyllableKyae', [], 'Display', None],
  [ 0xcec9, 'HangulSyllableKyaeg', [], 'Display', None],
  [ 0xceca, 'HangulSyllableKyaegg', [], 'Display', None],
  [ 0xcecb, 'HangulSyllableKyaegs', [], 'Display', None],
  [ 0xcecc, 'HangulSyllableKyaen', [], 'Display', None],
  [ 0xcecd, 'HangulSyllableKyaenj', [], 'Display', None],
  [ 0xcece, 'HangulSyllableKyaenh', [], 'Display', None],
  [ 0xcecf, 'HangulSyllableKyaed', [], 'Display', None],
  [ 0xced0, 'HangulSyllableKyael', [], 'Display', None],
  [ 0xced1, 'HangulSyllableKyaelg', [], 'Display', None],
  [ 0xced2, 'HangulSyllableKyaelm', [], 'Display', None],
  [ 0xced3, 'HangulSyllableKyaelb', [], 'Display', None],
  [ 0xced4, 'HangulSyllableKyaels', [], 'Display', None],
  [ 0xced5, 'HangulSyllableKyaelt', [], 'Display', None],
  [ 0xced6, 'HangulSyllableKyaelp', [], 'Display', None],
  [ 0xced7, 'HangulSyllableKyaelh', [], 'Display', None],
  [ 0xced8, 'HangulSyllableKyaem', [], 'Display', None],
  [ 0xced9, 'HangulSyllableKyaeb', [], 'Display', None],
  [ 0xceda, 'HangulSyllableKyaebs', [], 'Display', None],
  [ 0xcedb, 'HangulSyllableKyaes', [], 'Display', None],
  [ 0xcedc, 'HangulSyllableKyaess', [], 'Display', None],
  [ 0xcedd, 'HangulSyllableKyaeng', [], 'Display', None],
  [ 0xcede, 'HangulSyllableKyaej', [], 'Display', None],
  [ 0xcedf, 'HangulSyllableKyaec', [], 'Display', None],
  [ 0xcee0, 'HangulSyllableKyaek', [], 'Display', None],
  [ 0xcee1, 'HangulSyllableKyaet', [], 'Display', None],
  [ 0xcee2, 'HangulSyllableKyaep', [], 'Display', None],
  [ 0xcee3, 'HangulSyllableKyaeh', [], 'Display', None],
  [ 0xcee4, 'HangulSyllableKeo', [], 'Display', None],
  [ 0xcee5, 'HangulSyllableKeog', [], 'Display', None],
  [ 0xcee6, 'HangulSyllableKeogg', [], 'Display', None],
  [ 0xcee7, 'HangulSyllableKeogs', [], 'Display', None],
  [ 0xcee8, 'HangulSyllableKeon', [], 'Display', None],
  [ 0xcee9, 'HangulSyllableKeonj', [], 'Display', None],
  [ 0xceea, 'HangulSyllableKeonh', [], 'Display', None],
  [ 0xceeb, 'HangulSyllableKeod', [], 'Display', None],
  [ 0xceec, 'HangulSyllableKeol', [], 'Display', None],
  [ 0xceed, 'HangulSyllableKeolg', [], 'Display', None],
  [ 0xceee, 'HangulSyllableKeolm', [], 'Display', None],
  [ 0xceef, 'HangulSyllableKeolb', [], 'Display', None],
  [ 0xcef0, 'HangulSyllableKeols', [], 'Display', None],
  [ 0xcef1, 'HangulSyllableKeolt', [], 'Display', None],
  [ 0xcef2, 'HangulSyllableKeolp', [], 'Display', None],
  [ 0xcef3, 'HangulSyllableKeolh', [], 'Display', None],
  [ 0xcef4, 'HangulSyllableKeom', [], 'Display', None],
  [ 0xcef5, 'HangulSyllableKeob', [], 'Display', None],
  [ 0xcef6, 'HangulSyllableKeobs', [], 'Display', None],
  [ 0xcef7, 'HangulSyllableKeos', [], 'Display', None],
  [ 0xcef8, 'HangulSyllableKeoss', [], 'Display', None],
  [ 0xcef9, 'HangulSyllableKeong', [], 'Display', None],
  [ 0xcefa, 'HangulSyllableKeoj', [], 'Display', None],
  [ 0xcefb, 'HangulSyllableKeoc', [], 'Display', None],
  [ 0xcefc, 'HangulSyllableKeok', [], 'Display', None],
  [ 0xcefd, 'HangulSyllableKeot', [], 'Display', None],
  [ 0xcefe, 'HangulSyllableKeop', [], 'Display', None],
  [ 0xceff, 'HangulSyllableKeoh', [], 'Display', None],
  [ 0xcf00, 'HangulSyllableKe', [], 'Display', None],
  [ 0xcf01, 'HangulSyllableKeg', [], 'Display', None],
  [ 0xcf02, 'HangulSyllableKegg', [], 'Display', None],
  [ 0xcf03, 'HangulSyllableKegs', [], 'Display', None],
  [ 0xcf04, 'HangulSyllableKen', [], 'Display', None],
  [ 0xcf05, 'HangulSyllableKenj', [], 'Display', None],
  [ 0xcf06, 'HangulSyllableKenh', [], 'Display', None],
  [ 0xcf07, 'HangulSyllableKed', [], 'Display', None],
  [ 0xcf08, 'HangulSyllableKel', [], 'Display', None],
  [ 0xcf09, 'HangulSyllableKelg', [], 'Display', None],
  [ 0xcf0a, 'HangulSyllableKelm', [], 'Display', None],
  [ 0xcf0b, 'HangulSyllableKelb', [], 'Display', None],
  [ 0xcf0c, 'HangulSyllableKels', [], 'Display', None],
  [ 0xcf0d, 'HangulSyllableKelt', [], 'Display', None],
  [ 0xcf0e, 'HangulSyllableKelp', [], 'Display', None],
  [ 0xcf0f, 'HangulSyllableKelh', [], 'Display', None],
  [ 0xcf10, 'HangulSyllableKem', [], 'Display', None],
  [ 0xcf11, 'HangulSyllableKeb', [], 'Display', None],
  [ 0xcf12, 'HangulSyllableKebs', [], 'Display', None],
  [ 0xcf13, 'HangulSyllableKes', [], 'Display', None],
  [ 0xcf14, 'HangulSyllableKess', [], 'Display', None],
  [ 0xcf15, 'HangulSyllableKeng', [], 'Display', None],
  [ 0xcf16, 'HangulSyllableKej', [], 'Display', None],
  [ 0xcf17, 'HangulSyllableKec', [], 'Display', None],
  [ 0xcf18, 'HangulSyllableKek', [], 'Display', None],
  [ 0xcf19, 'HangulSyllableKet', [], 'Display', None],
  [ 0xcf1a, 'HangulSyllableKep', [], 'Display', None],
  [ 0xcf1b, 'HangulSyllableKeh', [], 'Display', None],
  [ 0xcf1c, 'HangulSyllableKyeo', [], 'Display', None],
  [ 0xcf1d, 'HangulSyllableKyeog', [], 'Display', None],
  [ 0xcf1e, 'HangulSyllableKyeogg', [], 'Display', None],
  [ 0xcf1f, 'HangulSyllableKyeogs', [], 'Display', None],
  [ 0xcf20, 'HangulSyllableKyeon', [], 'Display', None],
  [ 0xcf21, 'HangulSyllableKyeonj', [], 'Display', None],
  [ 0xcf22, 'HangulSyllableKyeonh', [], 'Display', None],
  [ 0xcf23, 'HangulSyllableKyeod', [], 'Display', None],
  [ 0xcf24, 'HangulSyllableKyeol', [], 'Display', None],
  [ 0xcf25, 'HangulSyllableKyeolg', [], 'Display', None],
  [ 0xcf26, 'HangulSyllableKyeolm', [], 'Display', None],
  [ 0xcf27, 'HangulSyllableKyeolb', [], 'Display', None],
  [ 0xcf28, 'HangulSyllableKyeols', [], 'Display', None],
  [ 0xcf29, 'HangulSyllableKyeolt', [], 'Display', None],
  [ 0xcf2a, 'HangulSyllableKyeolp', [], 'Display', None],
  [ 0xcf2b, 'HangulSyllableKyeolh', [], 'Display', None],
  [ 0xcf2c, 'HangulSyllableKyeom', [], 'Display', None],
  [ 0xcf2d, 'HangulSyllableKyeob', [], 'Display', None],
  [ 0xcf2e, 'HangulSyllableKyeobs', [], 'Display', None],
  [ 0xcf2f, 'HangulSyllableKyeos', [], 'Display', None],
  [ 0xcf30, 'HangulSyllableKyeoss', [], 'Display', None],
  [ 0xcf31, 'HangulSyllableKyeong', [], 'Display', None],
  [ 0xcf32, 'HangulSyllableKyeoj', [], 'Display', None],
  [ 0xcf33, 'HangulSyllableKyeoc', [], 'Display', None],
  [ 0xcf34, 'HangulSyllableKyeok', [], 'Display', None],
  [ 0xcf35, 'HangulSyllableKyeot', [], 'Display', None],
  [ 0xcf36, 'HangulSyllableKyeop', [], 'Display', None],
  [ 0xcf37, 'HangulSyllableKyeoh', [], 'Display', None],
  [ 0xcf38, 'HangulSyllableKye', [], 'Display', None],
  [ 0xcf39, 'HangulSyllableKyeg', [], 'Display', None],
  [ 0xcf3a, 'HangulSyllableKyegg', [], 'Display', None],
  [ 0xcf3b, 'HangulSyllableKyegs', [], 'Display', None],
  [ 0xcf3c, 'HangulSyllableKyen', [], 'Display', None],
  [ 0xcf3d, 'HangulSyllableKyenj', [], 'Display', None],
  [ 0xcf3e, 'HangulSyllableKyenh', [], 'Display', None],
  [ 0xcf3f, 'HangulSyllableKyed', [], 'Display', None],
  [ 0xcf40, 'HangulSyllableKyel', [], 'Display', None],
  [ 0xcf41, 'HangulSyllableKyelg', [], 'Display', None],
  [ 0xcf42, 'HangulSyllableKyelm', [], 'Display', None],
  [ 0xcf43, 'HangulSyllableKyelb', [], 'Display', None],
  [ 0xcf44, 'HangulSyllableKyels', [], 'Display', None],
  [ 0xcf45, 'HangulSyllableKyelt', [], 'Display', None],
  [ 0xcf46, 'HangulSyllableKyelp', [], 'Display', None],
  [ 0xcf47, 'HangulSyllableKyelh', [], 'Display', None],
  [ 0xcf48, 'HangulSyllableKyem', [], 'Display', None],
  [ 0xcf49, 'HangulSyllableKyeb', [], 'Display', None],
  [ 0xcf4a, 'HangulSyllableKyebs', [], 'Display', None],
  [ 0xcf4b, 'HangulSyllableKyes', [], 'Display', None],
  [ 0xcf4c, 'HangulSyllableKyess', [], 'Display', None],
  [ 0xcf4d, 'HangulSyllableKyeng', [], 'Display', None],
  [ 0xcf4e, 'HangulSyllableKyej', [], 'Display', None],
  [ 0xcf4f, 'HangulSyllableKyec', [], 'Display', None],
  [ 0xcf50, 'HangulSyllableKyek', [], 'Display', None],
  [ 0xcf51, 'HangulSyllableKyet', [], 'Display', None],
  [ 0xcf52, 'HangulSyllableKyep', [], 'Display', None],
  [ 0xcf53, 'HangulSyllableKyeh', [], 'Display', None],
  [ 0xcf54, 'HangulSyllableKo', [], 'Display', None],
  [ 0xcf55, 'HangulSyllableKog', [], 'Display', None],
  [ 0xcf56, 'HangulSyllableKogg', [], 'Display', None],
  [ 0xcf57, 'HangulSyllableKogs', [], 'Display', None],
  [ 0xcf58, 'HangulSyllableKon', [], 'Display', None],
  [ 0xcf59, 'HangulSyllableKonj', [], 'Display', None],
  [ 0xcf5a, 'HangulSyllableKonh', [], 'Display', None],
  [ 0xcf5b, 'HangulSyllableKod', [], 'Display', None],
  [ 0xcf5c, 'HangulSyllableKol', [], 'Display', None],
  [ 0xcf5d, 'HangulSyllableKolg', [], 'Display', None],
  [ 0xcf5e, 'HangulSyllableKolm', [], 'Display', None],
  [ 0xcf5f, 'HangulSyllableKolb', [], 'Display', None],
  [ 0xcf60, 'HangulSyllableKols', [], 'Display', None],
  [ 0xcf61, 'HangulSyllableKolt', [], 'Display', None],
  [ 0xcf62, 'HangulSyllableKolp', [], 'Display', None],
  [ 0xcf63, 'HangulSyllableKolh', [], 'Display', None],
  [ 0xcf64, 'HangulSyllableKom', [], 'Display', None],
  [ 0xcf65, 'HangulSyllableKob', [], 'Display', None],
  [ 0xcf66, 'HangulSyllableKobs', [], 'Display', None],
  [ 0xcf67, 'HangulSyllableKos', [], 'Display', None],
  [ 0xcf68, 'HangulSyllableKoss', [], 'Display', None],
  [ 0xcf69, 'HangulSyllableKong', [], 'Display', None],
  [ 0xcf6a, 'HangulSyllableKoj', [], 'Display', None],
  [ 0xcf6b, 'HangulSyllableKoc', [], 'Display', None],
  [ 0xcf6c, 'HangulSyllableKok', [], 'Display', None],
  [ 0xcf6d, 'HangulSyllableKot', [], 'Display', None],
  [ 0xcf6e, 'HangulSyllableKop', [], 'Display', None],
  [ 0xcf6f, 'HangulSyllableKoh', [], 'Display', None],
  [ 0xcf70, 'HangulSyllableKwa', [], 'Display', None],
  [ 0xcf71, 'HangulSyllableKwag', [], 'Display', None],
  [ 0xcf72, 'HangulSyllableKwagg', [], 'Display', None],
  [ 0xcf73, 'HangulSyllableKwags', [], 'Display', None],
  [ 0xcf74, 'HangulSyllableKwan', [], 'Display', None],
  [ 0xcf75, 'HangulSyllableKwanj', [], 'Display', None],
  [ 0xcf76, 'HangulSyllableKwanh', [], 'Display', None],
  [ 0xcf77, 'HangulSyllableKwad', [], 'Display', None],
  [ 0xcf78, 'HangulSyllableKwal', [], 'Display', None],
  [ 0xcf79, 'HangulSyllableKwalg', [], 'Display', None],
  [ 0xcf7a, 'HangulSyllableKwalm', [], 'Display', None],
  [ 0xcf7b, 'HangulSyllableKwalb', [], 'Display', None],
  [ 0xcf7c, 'HangulSyllableKwals', [], 'Display', None],
  [ 0xcf7d, 'HangulSyllableKwalt', [], 'Display', None],
  [ 0xcf7e, 'HangulSyllableKwalp', [], 'Display', None],
  [ 0xcf7f, 'HangulSyllableKwalh', [], 'Display', None],
  [ 0xcf80, 'HangulSyllableKwam', [], 'Display', None],
  [ 0xcf81, 'HangulSyllableKwab', [], 'Display', None],
  [ 0xcf82, 'HangulSyllableKwabs', [], 'Display', None],
  [ 0xcf83, 'HangulSyllableKwas', [], 'Display', None],
  [ 0xcf84, 'HangulSyllableKwass', [], 'Display', None],
  [ 0xcf85, 'HangulSyllableKwang', [], 'Display', None],
  [ 0xcf86, 'HangulSyllableKwaj', [], 'Display', None],
  [ 0xcf87, 'HangulSyllableKwac', [], 'Display', None],
  [ 0xcf88, 'HangulSyllableKwak', [], 'Display', None],
  [ 0xcf89, 'HangulSyllableKwat', [], 'Display', None],
  [ 0xcf8a, 'HangulSyllableKwap', [], 'Display', None],
  [ 0xcf8b, 'HangulSyllableKwah', [], 'Display', None],
  [ 0xcf8c, 'HangulSyllableKwae', [], 'Display', None],
  [ 0xcf8d, 'HangulSyllableKwaeg', [], 'Display', None],
  [ 0xcf8e, 'HangulSyllableKwaegg', [], 'Display', None],
  [ 0xcf8f, 'HangulSyllableKwaegs', [], 'Display', None],
  [ 0xcf90, 'HangulSyllableKwaen', [], 'Display', None],
  [ 0xcf91, 'HangulSyllableKwaenj', [], 'Display', None],
  [ 0xcf92, 'HangulSyllableKwaenh', [], 'Display', None],
  [ 0xcf93, 'HangulSyllableKwaed', [], 'Display', None],
  [ 0xcf94, 'HangulSyllableKwael', [], 'Display', None],
  [ 0xcf95, 'HangulSyllableKwaelg', [], 'Display', None],
  [ 0xcf96, 'HangulSyllableKwaelm', [], 'Display', None],
  [ 0xcf97, 'HangulSyllableKwaelb', [], 'Display', None],
  [ 0xcf98, 'HangulSyllableKwaels', [], 'Display', None],
  [ 0xcf99, 'HangulSyllableKwaelt', [], 'Display', None],
  [ 0xcf9a, 'HangulSyllableKwaelp', [], 'Display', None],
  [ 0xcf9b, 'HangulSyllableKwaelh', [], 'Display', None],
  [ 0xcf9c, 'HangulSyllableKwaem', [], 'Display', None],
  [ 0xcf9d, 'HangulSyllableKwaeb', [], 'Display', None],
  [ 0xcf9e, 'HangulSyllableKwaebs', [], 'Display', None],
  [ 0xcf9f, 'HangulSyllableKwaes', [], 'Display', None],
  [ 0xcfa0, 'HangulSyllableKwaess', [], 'Display', None],
  [ 0xcfa1, 'HangulSyllableKwaeng', [], 'Display', None],
  [ 0xcfa2, 'HangulSyllableKwaej', [], 'Display', None],
  [ 0xcfa3, 'HangulSyllableKwaec', [], 'Display', None],
  [ 0xcfa4, 'HangulSyllableKwaek', [], 'Display', None],
  [ 0xcfa5, 'HangulSyllableKwaet', [], 'Display', None],
  [ 0xcfa6, 'HangulSyllableKwaep', [], 'Display', None],
  [ 0xcfa7, 'HangulSyllableKwaeh', [], 'Display', None],
  [ 0xcfa8, 'HangulSyllableKoe', [], 'Display', None],
  [ 0xcfa9, 'HangulSyllableKoeg', [], 'Display', None],
  [ 0xcfaa, 'HangulSyllableKoegg', [], 'Display', None],
  [ 0xcfab, 'HangulSyllableKoegs', [], 'Display', None],
  [ 0xcfac, 'HangulSyllableKoen', [], 'Display', None],
  [ 0xcfad, 'HangulSyllableKoenj', [], 'Display', None],
  [ 0xcfae, 'HangulSyllableKoenh', [], 'Display', None],
  [ 0xcfaf, 'HangulSyllableKoed', [], 'Display', None],
  [ 0xcfb0, 'HangulSyllableKoel', [], 'Display', None],
  [ 0xcfb1, 'HangulSyllableKoelg', [], 'Display', None],
  [ 0xcfb2, 'HangulSyllableKoelm', [], 'Display', None],
  [ 0xcfb3, 'HangulSyllableKoelb', [], 'Display', None],
  [ 0xcfb4, 'HangulSyllableKoels', [], 'Display', None],
  [ 0xcfb5, 'HangulSyllableKoelt', [], 'Display', None],
  [ 0xcfb6, 'HangulSyllableKoelp', [], 'Display', None],
  [ 0xcfb7, 'HangulSyllableKoelh', [], 'Display', None],
  [ 0xcfb8, 'HangulSyllableKoem', [], 'Display', None],
  [ 0xcfb9, 'HangulSyllableKoeb', [], 'Display', None],
  [ 0xcfba, 'HangulSyllableKoebs', [], 'Display', None],
  [ 0xcfbb, 'HangulSyllableKoes', [], 'Display', None],
  [ 0xcfbc, 'HangulSyllableKoess', [], 'Display', None],
  [ 0xcfbd, 'HangulSyllableKoeng', [], 'Display', None],
  [ 0xcfbe, 'HangulSyllableKoej', [], 'Display', None],
  [ 0xcfbf, 'HangulSyllableKoec', [], 'Display', None],
  [ 0xcfc0, 'HangulSyllableKoek', [], 'Display', None],
  [ 0xcfc1, 'HangulSyllableKoet', [], 'Display', None],
  [ 0xcfc2, 'HangulSyllableKoep', [], 'Display', None],
  [ 0xcfc3, 'HangulSyllableKoeh', [], 'Display', None],
  [ 0xcfc4, 'HangulSyllableKyo', [], 'Display', None],
  [ 0xcfc5, 'HangulSyllableKyog', [], 'Display', None],
  [ 0xcfc6, 'HangulSyllableKyogg', [], 'Display', None],
  [ 0xcfc7, 'HangulSyllableKyogs', [], 'Display', None],
  [ 0xcfc8, 'HangulSyllableKyon', [], 'Display', None],
  [ 0xcfc9, 'HangulSyllableKyonj', [], 'Display', None],
  [ 0xcfca, 'HangulSyllableKyonh', [], 'Display', None],
  [ 0xcfcb, 'HangulSyllableKyod', [], 'Display', None],
  [ 0xcfcc, 'HangulSyllableKyol', [], 'Display', None],
  [ 0xcfcd, 'HangulSyllableKyolg', [], 'Display', None],
  [ 0xcfce, 'HangulSyllableKyolm', [], 'Display', None],
  [ 0xcfcf, 'HangulSyllableKyolb', [], 'Display', None],
  [ 0xcfd0, 'HangulSyllableKyols', [], 'Display', None],
  [ 0xcfd1, 'HangulSyllableKyolt', [], 'Display', None],
  [ 0xcfd2, 'HangulSyllableKyolp', [], 'Display', None],
  [ 0xcfd3, 'HangulSyllableKyolh', [], 'Display', None],
  [ 0xcfd4, 'HangulSyllableKyom', [], 'Display', None],
  [ 0xcfd5, 'HangulSyllableKyob', [], 'Display', None],
  [ 0xcfd6, 'HangulSyllableKyobs', [], 'Display', None],
  [ 0xcfd7, 'HangulSyllableKyos', [], 'Display', None],
  [ 0xcfd8, 'HangulSyllableKyoss', [], 'Display', None],
  [ 0xcfd9, 'HangulSyllableKyong', [], 'Display', None],
  [ 0xcfda, 'HangulSyllableKyoj', [], 'Display', None],
  [ 0xcfdb, 'HangulSyllableKyoc', [], 'Display', None],
  [ 0xcfdc, 'HangulSyllableKyok', [], 'Display', None],
  [ 0xcfdd, 'HangulSyllableKyot', [], 'Display', None],
  [ 0xcfde, 'HangulSyllableKyop', [], 'Display', None],
  [ 0xcfdf, 'HangulSyllableKyoh', [], 'Display', None],
  [ 0xcfe0, 'HangulSyllableKu', [], 'Display', None],
  [ 0xcfe1, 'HangulSyllableKug', [], 'Display', None],
  [ 0xcfe2, 'HangulSyllableKugg', [], 'Display', None],
  [ 0xcfe3, 'HangulSyllableKugs', [], 'Display', None],
  [ 0xcfe4, 'HangulSyllableKun', [], 'Display', None],
  [ 0xcfe5, 'HangulSyllableKunj', [], 'Display', None],
  [ 0xcfe6, 'HangulSyllableKunh', [], 'Display', None],
  [ 0xcfe7, 'HangulSyllableKud', [], 'Display', None],
  [ 0xcfe8, 'HangulSyllableKul', [], 'Display', None],
  [ 0xcfe9, 'HangulSyllableKulg', [], 'Display', None],
  [ 0xcfea, 'HangulSyllableKulm', [], 'Display', None],
  [ 0xcfeb, 'HangulSyllableKulb', [], 'Display', None],
  [ 0xcfec, 'HangulSyllableKuls', [], 'Display', None],
  [ 0xcfed, 'HangulSyllableKult', [], 'Display', None],
  [ 0xcfee, 'HangulSyllableKulp', [], 'Display', None],
  [ 0xcfef, 'HangulSyllableKulh', [], 'Display', None],
  [ 0xcff0, 'HangulSyllableKum', [], 'Display', None],
  [ 0xcff1, 'HangulSyllableKub', [], 'Display', None],
  [ 0xcff2, 'HangulSyllableKubs', [], 'Display', None],
  [ 0xcff3, 'HangulSyllableKus', [], 'Display', None],
  [ 0xcff4, 'HangulSyllableKuss', [], 'Display', None],
  [ 0xcff5, 'HangulSyllableKung', [], 'Display', None],
  [ 0xcff6, 'HangulSyllableKuj', [], 'Display', None],
  [ 0xcff7, 'HangulSyllableKuc', [], 'Display', None],
  [ 0xcff8, 'HangulSyllableKuk', [], 'Display', None],
  [ 0xcff9, 'HangulSyllableKut', [], 'Display', None],
  [ 0xcffa, 'HangulSyllableKup', [], 'Display', None],
  [ 0xcffb, 'HangulSyllableKuh', [], 'Display', None],
  [ 0xcffc, 'HangulSyllableKweo', [], 'Display', None],
  [ 0xcffd, 'HangulSyllableKweog', [], 'Display', None],
  [ 0xcffe, 'HangulSyllableKweogg', [], 'Display', None],
  [ 0xcfff, 'HangulSyllableKweogs', [], 'Display', None],
  [ 0xd000, 'HangulSyllableKweon', [], 'Display', None],
  [ 0xd001, 'HangulSyllableKweonj', [], 'Display', None],
  [ 0xd002, 'HangulSyllableKweonh', [], 'Display', None],
  [ 0xd003, 'HangulSyllableKweod', [], 'Display', None],
  [ 0xd004, 'HangulSyllableKweol', [], 'Display', None],
  [ 0xd005, 'HangulSyllableKweolg', [], 'Display', None],
  [ 0xd006, 'HangulSyllableKweolm', [], 'Display', None],
  [ 0xd007, 'HangulSyllableKweolb', [], 'Display', None],
  [ 0xd008, 'HangulSyllableKweols', [], 'Display', None],
  [ 0xd009, 'HangulSyllableKweolt', [], 'Display', None],
  [ 0xd00a, 'HangulSyllableKweolp', [], 'Display', None],
  [ 0xd00b, 'HangulSyllableKweolh', [], 'Display', None],
  [ 0xd00c, 'HangulSyllableKweom', [], 'Display', None],
  [ 0xd00d, 'HangulSyllableKweob', [], 'Display', None],
  [ 0xd00e, 'HangulSyllableKweobs', [], 'Display', None],
  [ 0xd00f, 'HangulSyllableKweos', [], 'Display', None],
  [ 0xd010, 'HangulSyllableKweoss', [], 'Display', None],
  [ 0xd011, 'HangulSyllableKweong', [], 'Display', None],
  [ 0xd012, 'HangulSyllableKweoj', [], 'Display', None],
  [ 0xd013, 'HangulSyllableKweoc', [], 'Display', None],
  [ 0xd014, 'HangulSyllableKweok', [], 'Display', None],
  [ 0xd015, 'HangulSyllableKweot', [], 'Display', None],
  [ 0xd016, 'HangulSyllableKweop', [], 'Display', None],
  [ 0xd017, 'HangulSyllableKweoh', [], 'Display', None],
  [ 0xd018, 'HangulSyllableKwe', [], 'Display', None],
  [ 0xd019, 'HangulSyllableKweg', [], 'Display', None],
  [ 0xd01a, 'HangulSyllableKwegg', [], 'Display', None],
  [ 0xd01b, 'HangulSyllableKwegs', [], 'Display', None],
  [ 0xd01c, 'HangulSyllableKwen', [], 'Display', None],
  [ 0xd01d, 'HangulSyllableKwenj', [], 'Display', None],
  [ 0xd01e, 'HangulSyllableKwenh', [], 'Display', None],
  [ 0xd01f, 'HangulSyllableKwed', [], 'Display', None],
  [ 0xd020, 'HangulSyllableKwel', [], 'Display', None],
  [ 0xd021, 'HangulSyllableKwelg', [], 'Display', None],
  [ 0xd022, 'HangulSyllableKwelm', [], 'Display', None],
  [ 0xd023, 'HangulSyllableKwelb', [], 'Display', None],
  [ 0xd024, 'HangulSyllableKwels', [], 'Display', None],
  [ 0xd025, 'HangulSyllableKwelt', [], 'Display', None],
  [ 0xd026, 'HangulSyllableKwelp', [], 'Display', None],
  [ 0xd027, 'HangulSyllableKwelh', [], 'Display', None],
  [ 0xd028, 'HangulSyllableKwem', [], 'Display', None],
  [ 0xd029, 'HangulSyllableKweb', [], 'Display', None],
  [ 0xd02a, 'HangulSyllableKwebs', [], 'Display', None],
  [ 0xd02b, 'HangulSyllableKwes', [], 'Display', None],
  [ 0xd02c, 'HangulSyllableKwess', [], 'Display', None],
  [ 0xd02d, 'HangulSyllableKweng', [], 'Display', None],
  [ 0xd02e, 'HangulSyllableKwej', [], 'Display', None],
  [ 0xd02f, 'HangulSyllableKwec', [], 'Display', None],
  [ 0xd030, 'HangulSyllableKwek', [], 'Display', None],
  [ 0xd031, 'HangulSyllableKwet', [], 'Display', None],
  [ 0xd032, 'HangulSyllableKwep', [], 'Display', None],
  [ 0xd033, 'HangulSyllableKweh', [], 'Display', None],
  [ 0xd034, 'HangulSyllableKwi', [], 'Display', None],
  [ 0xd035, 'HangulSyllableKwig', [], 'Display', None],
  [ 0xd036, 'HangulSyllableKwigg', [], 'Display', None],
  [ 0xd037, 'HangulSyllableKwigs', [], 'Display', None],
  [ 0xd038, 'HangulSyllableKwin', [], 'Display', None],
  [ 0xd039, 'HangulSyllableKwinj', [], 'Display', None],
  [ 0xd03a, 'HangulSyllableKwinh', [], 'Display', None],
  [ 0xd03b, 'HangulSyllableKwid', [], 'Display', None],
  [ 0xd03c, 'HangulSyllableKwil', [], 'Display', None],
  [ 0xd03d, 'HangulSyllableKwilg', [], 'Display', None],
  [ 0xd03e, 'HangulSyllableKwilm', [], 'Display', None],
  [ 0xd03f, 'HangulSyllableKwilb', [], 'Display', None],
  [ 0xd040, 'HangulSyllableKwils', [], 'Display', None],
  [ 0xd041, 'HangulSyllableKwilt', [], 'Display', None],
  [ 0xd042, 'HangulSyllableKwilp', [], 'Display', None],
  [ 0xd043, 'HangulSyllableKwilh', [], 'Display', None],
  [ 0xd044, 'HangulSyllableKwim', [], 'Display', None],
  [ 0xd045, 'HangulSyllableKwib', [], 'Display', None],
  [ 0xd046, 'HangulSyllableKwibs', [], 'Display', None],
  [ 0xd047, 'HangulSyllableKwis', [], 'Display', None],
  [ 0xd048, 'HangulSyllableKwiss', [], 'Display', None],
  [ 0xd049, 'HangulSyllableKwing', [], 'Display', None],
  [ 0xd04a, 'HangulSyllableKwij', [], 'Display', None],
  [ 0xd04b, 'HangulSyllableKwic', [], 'Display', None],
  [ 0xd04c, 'HangulSyllableKwik', [], 'Display', None],
  [ 0xd04d, 'HangulSyllableKwit', [], 'Display', None],
  [ 0xd04e, 'HangulSyllableKwip', [], 'Display', None],
  [ 0xd04f, 'HangulSyllableKwih', [], 'Display', None],
  [ 0xd050, 'HangulSyllableKyu', [], 'Display', None],
  [ 0xd051, 'HangulSyllableKyug', [], 'Display', None],
  [ 0xd052, 'HangulSyllableKyugg', [], 'Display', None],
  [ 0xd053, 'HangulSyllableKyugs', [], 'Display', None],
  [ 0xd054, 'HangulSyllableKyun', [], 'Display', None],
  [ 0xd055, 'HangulSyllableKyunj', [], 'Display', None],
  [ 0xd056, 'HangulSyllableKyunh', [], 'Display', None],
  [ 0xd057, 'HangulSyllableKyud', [], 'Display', None],
  [ 0xd058, 'HangulSyllableKyul', [], 'Display', None],
  [ 0xd059, 'HangulSyllableKyulg', [], 'Display', None],
  [ 0xd05a, 'HangulSyllableKyulm', [], 'Display', None],
  [ 0xd05b, 'HangulSyllableKyulb', [], 'Display', None],
  [ 0xd05c, 'HangulSyllableKyuls', [], 'Display', None],
  [ 0xd05d, 'HangulSyllableKyult', [], 'Display', None],
  [ 0xd05e, 'HangulSyllableKyulp', [], 'Display', None],
  [ 0xd05f, 'HangulSyllableKyulh', [], 'Display', None],
  [ 0xd060, 'HangulSyllableKyum', [], 'Display', None],
  [ 0xd061, 'HangulSyllableKyub', [], 'Display', None],
  [ 0xd062, 'HangulSyllableKyubs', [], 'Display', None],
  [ 0xd063, 'HangulSyllableKyus', [], 'Display', None],
  [ 0xd064, 'HangulSyllableKyuss', [], 'Display', None],
  [ 0xd065, 'HangulSyllableKyung', [], 'Display', None],
  [ 0xd066, 'HangulSyllableKyuj', [], 'Display', None],
  [ 0xd067, 'HangulSyllableKyuc', [], 'Display', None],
  [ 0xd068, 'HangulSyllableKyuk', [], 'Display', None],
  [ 0xd069, 'HangulSyllableKyut', [], 'Display', None],
  [ 0xd06a, 'HangulSyllableKyup', [], 'Display', None],
  [ 0xd06b, 'HangulSyllableKyuh', [], 'Display', None],
  [ 0xd06c, 'HangulSyllableKeu', [], 'Display', None],
  [ 0xd06d, 'HangulSyllableKeug', [], 'Display', None],
  [ 0xd06e, 'HangulSyllableKeugg', [], 'Display', None],
  [ 0xd06f, 'HangulSyllableKeugs', [], 'Display', None],
  [ 0xd070, 'HangulSyllableKeun', [], 'Display', None],
  [ 0xd071, 'HangulSyllableKeunj', [], 'Display', None],
  [ 0xd072, 'HangulSyllableKeunh', [], 'Display', None],
  [ 0xd073, 'HangulSyllableKeud', [], 'Display', None],
  [ 0xd074, 'HangulSyllableKeul', [], 'Display', None],
  [ 0xd075, 'HangulSyllableKeulg', [], 'Display', None],
  [ 0xd076, 'HangulSyllableKeulm', [], 'Display', None],
  [ 0xd077, 'HangulSyllableKeulb', [], 'Display', None],
  [ 0xd078, 'HangulSyllableKeuls', [], 'Display', None],
  [ 0xd079, 'HangulSyllableKeult', [], 'Display', None],
  [ 0xd07a, 'HangulSyllableKeulp', [], 'Display', None],
  [ 0xd07b, 'HangulSyllableKeulh', [], 'Display', None],
  [ 0xd07c, 'HangulSyllableKeum', [], 'Display', None],
  [ 0xd07d, 'HangulSyllableKeub', [], 'Display', None],
  [ 0xd07e, 'HangulSyllableKeubs', [], 'Display', None],
  [ 0xd07f, 'HangulSyllableKeus', [], 'Display', None],
  [ 0xd080, 'HangulSyllableKeuss', [], 'Display', None],
  [ 0xd081, 'HangulSyllableKeung', [], 'Display', None],
  [ 0xd082, 'HangulSyllableKeuj', [], 'Display', None],
  [ 0xd083, 'HangulSyllableKeuc', [], 'Display', None],
  [ 0xd084, 'HangulSyllableKeuk', [], 'Display', None],
  [ 0xd085, 'HangulSyllableKeut', [], 'Display', None],
  [ 0xd086, 'HangulSyllableKeup', [], 'Display', None],
  [ 0xd087, 'HangulSyllableKeuh', [], 'Display', None],
  [ 0xd088, 'HangulSyllableKyi', [], 'Display', None],
  [ 0xd089, 'HangulSyllableKyig', [], 'Display', None],
  [ 0xd08a, 'HangulSyllableKyigg', [], 'Display', None],
  [ 0xd08b, 'HangulSyllableKyigs', [], 'Display', None],
  [ 0xd08c, 'HangulSyllableKyin', [], 'Display', None],
  [ 0xd08d, 'HangulSyllableKyinj', [], 'Display', None],
  [ 0xd08e, 'HangulSyllableKyinh', [], 'Display', None],
  [ 0xd08f, 'HangulSyllableKyid', [], 'Display', None],
  [ 0xd090, 'HangulSyllableKyil', [], 'Display', None],
  [ 0xd091, 'HangulSyllableKyilg', [], 'Display', None],
  [ 0xd092, 'HangulSyllableKyilm', [], 'Display', None],
  [ 0xd093, 'HangulSyllableKyilb', [], 'Display', None],
  [ 0xd094, 'HangulSyllableKyils', [], 'Display', None],
  [ 0xd095, 'HangulSyllableKyilt', [], 'Display', None],
  [ 0xd096, 'HangulSyllableKyilp', [], 'Display', None],
  [ 0xd097, 'HangulSyllableKyilh', [], 'Display', None],
  [ 0xd098, 'HangulSyllableKyim', [], 'Display', None],
  [ 0xd099, 'HangulSyllableKyib', [], 'Display', None],
  [ 0xd09a, 'HangulSyllableKyibs', [], 'Display', None],
  [ 0xd09b, 'HangulSyllableKyis', [], 'Display', None],
  [ 0xd09c, 'HangulSyllableKyiss', [], 'Display', None],
  [ 0xd09d, 'HangulSyllableKying', [], 'Display', None],
  [ 0xd09e, 'HangulSyllableKyij', [], 'Display', None],
  [ 0xd09f, 'HangulSyllableKyic', [], 'Display', None],
  [ 0xd0a0, 'HangulSyllableKyik', [], 'Display', None],
  [ 0xd0a1, 'HangulSyllableKyit', [], 'Display', None],
  [ 0xd0a2, 'HangulSyllableKyip', [], 'Display', None],
  [ 0xd0a3, 'HangulSyllableKyih', [], 'Display', None],
  [ 0xd0a4, 'HangulSyllableKi', [], 'Display', None],
  [ 0xd0a5, 'HangulSyllableKig', [], 'Display', None],
  [ 0xd0a6, 'HangulSyllableKigg', [], 'Display', None],
  [ 0xd0a7, 'HangulSyllableKigs', [], 'Display', None],
  [ 0xd0a8, 'HangulSyllableKin', [], 'Display', None],
  [ 0xd0a9, 'HangulSyllableKinj', [], 'Display', None],
  [ 0xd0aa, 'HangulSyllableKinh', [], 'Display', None],
  [ 0xd0ab, 'HangulSyllableKid', [], 'Display', None],
  [ 0xd0ac, 'HangulSyllableKil', [], 'Display', None],
  [ 0xd0ad, 'HangulSyllableKilg', [], 'Display', None],
  [ 0xd0ae, 'HangulSyllableKilm', [], 'Display', None],
  [ 0xd0af, 'HangulSyllableKilb', [], 'Display', None],
  [ 0xd0b0, 'HangulSyllableKils', [], 'Display', None],
  [ 0xd0b1, 'HangulSyllableKilt', [], 'Display', None],
  [ 0xd0b2, 'HangulSyllableKilp', [], 'Display', None],
  [ 0xd0b3, 'HangulSyllableKilh', [], 'Display', None],
  [ 0xd0b4, 'HangulSyllableKim', [], 'Display', None],
  [ 0xd0b5, 'HangulSyllableKib', [], 'Display', None],
  [ 0xd0b6, 'HangulSyllableKibs', [], 'Display', None],
  [ 0xd0b7, 'HangulSyllableKis', [], 'Display', None],
  [ 0xd0b8, 'HangulSyllableKiss', [], 'Display', None],
  [ 0xd0b9, 'HangulSyllableKing', [], 'Display', None],
  [ 0xd0ba, 'HangulSyllableKij', [], 'Display', None],
  [ 0xd0bb, 'HangulSyllableKic', [], 'Display', None],
  [ 0xd0bc, 'HangulSyllableKik', [], 'Display', None],
  [ 0xd0bd, 'HangulSyllableKit', [], 'Display', None],
  [ 0xd0be, 'HangulSyllableKip', [], 'Display', None],
  [ 0xd0bf, 'HangulSyllableKih', [], 'Display', None],
  [ 0xd0c0, 'HangulSyllableTa', [], 'Display', None],
  [ 0xd0c1, 'HangulSyllableTag', [], 'Display', None],
  [ 0xd0c2, 'HangulSyllableTagg', [], 'Display', None],
  [ 0xd0c3, 'HangulSyllableTags', [], 'Display', None],
  [ 0xd0c4, 'HangulSyllableTan', [], 'Display', None],
  [ 0xd0c5, 'HangulSyllableTanj', [], 'Display', None],
  [ 0xd0c6, 'HangulSyllableTanh', [], 'Display', None],
  [ 0xd0c7, 'HangulSyllableTad', [], 'Display', None],
  [ 0xd0c8, 'HangulSyllableTal', [], 'Display', None],
  [ 0xd0c9, 'HangulSyllableTalg', [], 'Display', None],
  [ 0xd0ca, 'HangulSyllableTalm', [], 'Display', None],
  [ 0xd0cb, 'HangulSyllableTalb', [], 'Display', None],
  [ 0xd0cc, 'HangulSyllableTals', [], 'Display', None],
  [ 0xd0cd, 'HangulSyllableTalt', [], 'Display', None],
  [ 0xd0ce, 'HangulSyllableTalp', [], 'Display', None],
  [ 0xd0cf, 'HangulSyllableTalh', [], 'Display', None],
  [ 0xd0d0, 'HangulSyllableTam', [], 'Display', None],
  [ 0xd0d1, 'HangulSyllableTab', [], 'Display', None],
  [ 0xd0d2, 'HangulSyllableTabs', [], 'Display', None],
  [ 0xd0d3, 'HangulSyllableTas', [], 'Display', None],
  [ 0xd0d4, 'HangulSyllableTass', [], 'Display', None],
  [ 0xd0d5, 'HangulSyllableTang', [], 'Display', None],
  [ 0xd0d6, 'HangulSyllableTaj', [], 'Display', None],
  [ 0xd0d7, 'HangulSyllableTac', [], 'Display', None],
  [ 0xd0d8, 'HangulSyllableTak', [], 'Display', None],
  [ 0xd0d9, 'HangulSyllableTat', [], 'Display', None],
  [ 0xd0da, 'HangulSyllableTap', [], 'Display', None],
  [ 0xd0db, 'HangulSyllableTah', [], 'Display', None],
  [ 0xd0dc, 'HangulSyllableTae', [], 'Display', None],
  [ 0xd0dd, 'HangulSyllableTaeg', [], 'Display', None],
  [ 0xd0de, 'HangulSyllableTaegg', [], 'Display', None],
  [ 0xd0df, 'HangulSyllableTaegs', [], 'Display', None],
  [ 0xd0e0, 'HangulSyllableTaen', [], 'Display', None],
  [ 0xd0e1, 'HangulSyllableTaenj', [], 'Display', None],
  [ 0xd0e2, 'HangulSyllableTaenh', [], 'Display', None],
  [ 0xd0e3, 'HangulSyllableTaed', [], 'Display', None],
  [ 0xd0e4, 'HangulSyllableTael', [], 'Display', None],
  [ 0xd0e5, 'HangulSyllableTaelg', [], 'Display', None],
  [ 0xd0e6, 'HangulSyllableTaelm', [], 'Display', None],
  [ 0xd0e7, 'HangulSyllableTaelb', [], 'Display', None],
  [ 0xd0e8, 'HangulSyllableTaels', [], 'Display', None],
  [ 0xd0e9, 'HangulSyllableTaelt', [], 'Display', None],
  [ 0xd0ea, 'HangulSyllableTaelp', [], 'Display', None],
  [ 0xd0eb, 'HangulSyllableTaelh', [], 'Display', None],
  [ 0xd0ec, 'HangulSyllableTaem', [], 'Display', None],
  [ 0xd0ed, 'HangulSyllableTaeb', [], 'Display', None],
  [ 0xd0ee, 'HangulSyllableTaebs', [], 'Display', None],
  [ 0xd0ef, 'HangulSyllableTaes', [], 'Display', None],
  [ 0xd0f0, 'HangulSyllableTaess', [], 'Display', None],
  [ 0xd0f1, 'HangulSyllableTaeng', [], 'Display', None],
  [ 0xd0f2, 'HangulSyllableTaej', [], 'Display', None],
  [ 0xd0f3, 'HangulSyllableTaec', [], 'Display', None],
  [ 0xd0f4, 'HangulSyllableTaek', [], 'Display', None],
  [ 0xd0f5, 'HangulSyllableTaet', [], 'Display', None],
  [ 0xd0f6, 'HangulSyllableTaep', [], 'Display', None],
  [ 0xd0f7, 'HangulSyllableTaeh', [], 'Display', None],
  [ 0xd0f8, 'HangulSyllableTya', [], 'Display', None],
  [ 0xd0f9, 'HangulSyllableTyag', [], 'Display', None],
  [ 0xd0fa, 'HangulSyllableTyagg', [], 'Display', None],
  [ 0xd0fb, 'HangulSyllableTyags', [], 'Display', None],
  [ 0xd0fc, 'HangulSyllableTyan', [], 'Display', None],
  [ 0xd0fd, 'HangulSyllableTyanj', [], 'Display', None],
  [ 0xd0fe, 'HangulSyllableTyanh', [], 'Display', None],
  [ 0xd0ff, 'HangulSyllableTyad', [], 'Display', None],
  [ 0xd100, 'HangulSyllableTyal', [], 'Display', None],
  [ 0xd101, 'HangulSyllableTyalg', [], 'Display', None],
  [ 0xd102, 'HangulSyllableTyalm', [], 'Display', None],
  [ 0xd103, 'HangulSyllableTyalb', [], 'Display', None],
  [ 0xd104, 'HangulSyllableTyals', [], 'Display', None],
  [ 0xd105, 'HangulSyllableTyalt', [], 'Display', None],
  [ 0xd106, 'HangulSyllableTyalp', [], 'Display', None],
  [ 0xd107, 'HangulSyllableTyalh', [], 'Display', None],
  [ 0xd108, 'HangulSyllableTyam', [], 'Display', None],
  [ 0xd109, 'HangulSyllableTyab', [], 'Display', None],
  [ 0xd10a, 'HangulSyllableTyabs', [], 'Display', None],
  [ 0xd10b, 'HangulSyllableTyas', [], 'Display', None],
  [ 0xd10c, 'HangulSyllableTyass', [], 'Display', None],
  [ 0xd10d, 'HangulSyllableTyang', [], 'Display', None],
  [ 0xd10e, 'HangulSyllableTyaj', [], 'Display', None],
  [ 0xd10f, 'HangulSyllableTyac', [], 'Display', None],
  [ 0xd110, 'HangulSyllableTyak', [], 'Display', None],
  [ 0xd111, 'HangulSyllableTyat', [], 'Display', None],
  [ 0xd112, 'HangulSyllableTyap', [], 'Display', None],
  [ 0xd113, 'HangulSyllableTyah', [], 'Display', None],
  [ 0xd114, 'HangulSyllableTyae', [], 'Display', None],
  [ 0xd115, 'HangulSyllableTyaeg', [], 'Display', None],
  [ 0xd116, 'HangulSyllableTyaegg', [], 'Display', None],
  [ 0xd117, 'HangulSyllableTyaegs', [], 'Display', None],
  [ 0xd118, 'HangulSyllableTyaen', [], 'Display', None],
  [ 0xd119, 'HangulSyllableTyaenj', [], 'Display', None],
  [ 0xd11a, 'HangulSyllableTyaenh', [], 'Display', None],
  [ 0xd11b, 'HangulSyllableTyaed', [], 'Display', None],
  [ 0xd11c, 'HangulSyllableTyael', [], 'Display', None],
  [ 0xd11d, 'HangulSyllableTyaelg', [], 'Display', None],
  [ 0xd11e, 'HangulSyllableTyaelm', [], 'Display', None],
  [ 0xd11f, 'HangulSyllableTyaelb', [], 'Display', None],
  [ 0xd120, 'HangulSyllableTyaels', [], 'Display', None],
  [ 0xd121, 'HangulSyllableTyaelt', [], 'Display', None],
  [ 0xd122, 'HangulSyllableTyaelp', [], 'Display', None],
  [ 0xd123, 'HangulSyllableTyaelh', [], 'Display', None],
  [ 0xd124, 'HangulSyllableTyaem', [], 'Display', None],
  [ 0xd125, 'HangulSyllableTyaeb', [], 'Display', None],
  [ 0xd126, 'HangulSyllableTyaebs', [], 'Display', None],
  [ 0xd127, 'HangulSyllableTyaes', [], 'Display', None],
  [ 0xd128, 'HangulSyllableTyaess', [], 'Display', None],
  [ 0xd129, 'HangulSyllableTyaeng', [], 'Display', None],
  [ 0xd12a, 'HangulSyllableTyaej', [], 'Display', None],
  [ 0xd12b, 'HangulSyllableTyaec', [], 'Display', None],
  [ 0xd12c, 'HangulSyllableTyaek', [], 'Display', None],
  [ 0xd12d, 'HangulSyllableTyaet', [], 'Display', None],
  [ 0xd12e, 'HangulSyllableTyaep', [], 'Display', None],
  [ 0xd12f, 'HangulSyllableTyaeh', [], 'Display', None],
  [ 0xd130, 'HangulSyllableTeo', [], 'Display', None],
  [ 0xd131, 'HangulSyllableTeog', [], 'Display', None],
  [ 0xd132, 'HangulSyllableTeogg', [], 'Display', None],
  [ 0xd133, 'HangulSyllableTeogs', [], 'Display', None],
  [ 0xd134, 'HangulSyllableTeon', [], 'Display', None],
  [ 0xd135, 'HangulSyllableTeonj', [], 'Display', None],
  [ 0xd136, 'HangulSyllableTeonh', [], 'Display', None],
  [ 0xd137, 'HangulSyllableTeod', [], 'Display', None],
  [ 0xd138, 'HangulSyllableTeol', [], 'Display', None],
  [ 0xd139, 'HangulSyllableTeolg', [], 'Display', None],
  [ 0xd13a, 'HangulSyllableTeolm', [], 'Display', None],
  [ 0xd13b, 'HangulSyllableTeolb', [], 'Display', None],
  [ 0xd13c, 'HangulSyllableTeols', [], 'Display', None],
  [ 0xd13d, 'HangulSyllableTeolt', [], 'Display', None],
  [ 0xd13e, 'HangulSyllableTeolp', [], 'Display', None],
  [ 0xd13f, 'HangulSyllableTeolh', [], 'Display', None],
  [ 0xd140, 'HangulSyllableTeom', [], 'Display', None],
  [ 0xd141, 'HangulSyllableTeob', [], 'Display', None],
  [ 0xd142, 'HangulSyllableTeobs', [], 'Display', None],
  [ 0xd143, 'HangulSyllableTeos', [], 'Display', None],
  [ 0xd144, 'HangulSyllableTeoss', [], 'Display', None],
  [ 0xd145, 'HangulSyllableTeong', [], 'Display', None],
  [ 0xd146, 'HangulSyllableTeoj', [], 'Display', None],
  [ 0xd147, 'HangulSyllableTeoc', [], 'Display', None],
  [ 0xd148, 'HangulSyllableTeok', [], 'Display', None],
  [ 0xd149, 'HangulSyllableTeot', [], 'Display', None],
  [ 0xd14a, 'HangulSyllableTeop', [], 'Display', None],
  [ 0xd14b, 'HangulSyllableTeoh', [], 'Display', None],
  [ 0xd14c, 'HangulSyllableTe', [], 'Display', None],
  [ 0xd14d, 'HangulSyllableTeg', [], 'Display', None],
  [ 0xd14e, 'HangulSyllableTegg', [], 'Display', None],
  [ 0xd14f, 'HangulSyllableTegs', [], 'Display', None],
  [ 0xd150, 'HangulSyllableTen', [], 'Display', None],
  [ 0xd151, 'HangulSyllableTenj', [], 'Display', None],
  [ 0xd152, 'HangulSyllableTenh', [], 'Display', None],
  [ 0xd153, 'HangulSyllableTed', [], 'Display', None],
  [ 0xd154, 'HangulSyllableTel', [], 'Display', None],
  [ 0xd155, 'HangulSyllableTelg', [], 'Display', None],
  [ 0xd156, 'HangulSyllableTelm', [], 'Display', None],
  [ 0xd157, 'HangulSyllableTelb', [], 'Display', None],
  [ 0xd158, 'HangulSyllableTels', [], 'Display', None],
  [ 0xd159, 'HangulSyllableTelt', [], 'Display', None],
  [ 0xd15a, 'HangulSyllableTelp', [], 'Display', None],
  [ 0xd15b, 'HangulSyllableTelh', [], 'Display', None],
  [ 0xd15c, 'HangulSyllableTem', [], 'Display', None],
  [ 0xd15d, 'HangulSyllableTeb', [], 'Display', None],
  [ 0xd15e, 'HangulSyllableTebs', [], 'Display', None],
  [ 0xd15f, 'HangulSyllableTes', [], 'Display', None],
  [ 0xd160, 'HangulSyllableTess', [], 'Display', None],
  [ 0xd161, 'HangulSyllableTeng', [], 'Display', None],
  [ 0xd162, 'HangulSyllableTej', [], 'Display', None],
  [ 0xd163, 'HangulSyllableTec', [], 'Display', None],
  [ 0xd164, 'HangulSyllableTek', [], 'Display', None],
  [ 0xd165, 'HangulSyllableTet', [], 'Display', None],
  [ 0xd166, 'HangulSyllableTep', [], 'Display', None],
  [ 0xd167, 'HangulSyllableTeh', [], 'Display', None],
  [ 0xd168, 'HangulSyllableTyeo', [], 'Display', None],
  [ 0xd169, 'HangulSyllableTyeog', [], 'Display', None],
  [ 0xd16a, 'HangulSyllableTyeogg', [], 'Display', None],
  [ 0xd16b, 'HangulSyllableTyeogs', [], 'Display', None],
  [ 0xd16c, 'HangulSyllableTyeon', [], 'Display', None],
  [ 0xd16d, 'HangulSyllableTyeonj', [], 'Display', None],
  [ 0xd16e, 'HangulSyllableTyeonh', [], 'Display', None],
  [ 0xd16f, 'HangulSyllableTyeod', [], 'Display', None],
  [ 0xd170, 'HangulSyllableTyeol', [], 'Display', None],
  [ 0xd171, 'HangulSyllableTyeolg', [], 'Display', None],
  [ 0xd172, 'HangulSyllableTyeolm', [], 'Display', None],
  [ 0xd173, 'HangulSyllableTyeolb', [], 'Display', None],
  [ 0xd174, 'HangulSyllableTyeols', [], 'Display', None],
  [ 0xd175, 'HangulSyllableTyeolt', [], 'Display', None],
  [ 0xd176, 'HangulSyllableTyeolp', [], 'Display', None],
  [ 0xd177, 'HangulSyllableTyeolh', [], 'Display', None],
  [ 0xd178, 'HangulSyllableTyeom', [], 'Display', None],
  [ 0xd179, 'HangulSyllableTyeob', [], 'Display', None],
  [ 0xd17a, 'HangulSyllableTyeobs', [], 'Display', None],
  [ 0xd17b, 'HangulSyllableTyeos', [], 'Display', None],
  [ 0xd17c, 'HangulSyllableTyeoss', [], 'Display', None],
  [ 0xd17d, 'HangulSyllableTyeong', [], 'Display', None],
  [ 0xd17e, 'HangulSyllableTyeoj', [], 'Display', None],
  [ 0xd17f, 'HangulSyllableTyeoc', [], 'Display', None],
  [ 0xd180, 'HangulSyllableTyeok', [], 'Display', None],
  [ 0xd181, 'HangulSyllableTyeot', [], 'Display', None],
  [ 0xd182, 'HangulSyllableTyeop', [], 'Display', None],
  [ 0xd183, 'HangulSyllableTyeoh', [], 'Display', None],
  [ 0xd184, 'HangulSyllableTye', [], 'Display', None],
  [ 0xd185, 'HangulSyllableTyeg', [], 'Display', None],
  [ 0xd186, 'HangulSyllableTyegg', [], 'Display', None],
  [ 0xd187, 'HangulSyllableTyegs', [], 'Display', None],
  [ 0xd188, 'HangulSyllableTyen', [], 'Display', None],
  [ 0xd189, 'HangulSyllableTyenj', [], 'Display', None],
  [ 0xd18a, 'HangulSyllableTyenh', [], 'Display', None],
  [ 0xd18b, 'HangulSyllableTyed', [], 'Display', None],
  [ 0xd18c, 'HangulSyllableTyel', [], 'Display', None],
  [ 0xd18d, 'HangulSyllableTyelg', [], 'Display', None],
  [ 0xd18e, 'HangulSyllableTyelm', [], 'Display', None],
  [ 0xd18f, 'HangulSyllableTyelb', [], 'Display', None],
  [ 0xd190, 'HangulSyllableTyels', [], 'Display', None],
  [ 0xd191, 'HangulSyllableTyelt', [], 'Display', None],
  [ 0xd192, 'HangulSyllableTyelp', [], 'Display', None],
  [ 0xd193, 'HangulSyllableTyelh', [], 'Display', None],
  [ 0xd194, 'HangulSyllableTyem', [], 'Display', None],
  [ 0xd195, 'HangulSyllableTyeb', [], 'Display', None],
  [ 0xd196, 'HangulSyllableTyebs', [], 'Display', None],
  [ 0xd197, 'HangulSyllableTyes', [], 'Display', None],
  [ 0xd198, 'HangulSyllableTyess', [], 'Display', None],
  [ 0xd199, 'HangulSyllableTyeng', [], 'Display', None],
  [ 0xd19a, 'HangulSyllableTyej', [], 'Display', None],
  [ 0xd19b, 'HangulSyllableTyec', [], 'Display', None],
  [ 0xd19c, 'HangulSyllableTyek', [], 'Display', None],
  [ 0xd19d, 'HangulSyllableTyet', [], 'Display', None],
  [ 0xd19e, 'HangulSyllableTyep', [], 'Display', None],
  [ 0xd19f, 'HangulSyllableTyeh', [], 'Display', None],
  [ 0xd1a0, 'HangulSyllableTo', [], 'Display', None],
  [ 0xd1a1, 'HangulSyllableTog', [], 'Display', None],
  [ 0xd1a2, 'HangulSyllableTogg', [], 'Display', None],
  [ 0xd1a3, 'HangulSyllableTogs', [], 'Display', None],
  [ 0xd1a4, 'HangulSyllableTon', [], 'Display', None],
  [ 0xd1a5, 'HangulSyllableTonj', [], 'Display', None],
  [ 0xd1a6, 'HangulSyllableTonh', [], 'Display', None],
  [ 0xd1a7, 'HangulSyllableTod', [], 'Display', None],
  [ 0xd1a8, 'HangulSyllableTol', [], 'Display', None],
  [ 0xd1a9, 'HangulSyllableTolg', [], 'Display', None],
  [ 0xd1aa, 'HangulSyllableTolm', [], 'Display', None],
  [ 0xd1ab, 'HangulSyllableTolb', [], 'Display', None],
  [ 0xd1ac, 'HangulSyllableTols', [], 'Display', None],
  [ 0xd1ad, 'HangulSyllableTolt', [], 'Display', None],
  [ 0xd1ae, 'HangulSyllableTolp', [], 'Display', None],
  [ 0xd1af, 'HangulSyllableTolh', [], 'Display', None],
  [ 0xd1b0, 'HangulSyllableTom', [], 'Display', None],
  [ 0xd1b1, 'HangulSyllableTob', [], 'Display', None],
  [ 0xd1b2, 'HangulSyllableTobs', [], 'Display', None],
  [ 0xd1b3, 'HangulSyllableTos', [], 'Display', None],
  [ 0xd1b4, 'HangulSyllableToss', [], 'Display', None],
  [ 0xd1b5, 'HangulSyllableTong', [], 'Display', None],
  [ 0xd1b6, 'HangulSyllableToj', [], 'Display', None],
  [ 0xd1b7, 'HangulSyllableToc', [], 'Display', None],
  [ 0xd1b8, 'HangulSyllableTok', [], 'Display', None],
  [ 0xd1b9, 'HangulSyllableTot', [], 'Display', None],
  [ 0xd1ba, 'HangulSyllableTop', [], 'Display', None],
  [ 0xd1bb, 'HangulSyllableToh', [], 'Display', None],
  [ 0xd1bc, 'HangulSyllableTwa', [], 'Display', None],
  [ 0xd1bd, 'HangulSyllableTwag', [], 'Display', None],
  [ 0xd1be, 'HangulSyllableTwagg', [], 'Display', None],
  [ 0xd1bf, 'HangulSyllableTwags', [], 'Display', None],
  [ 0xd1c0, 'HangulSyllableTwan', [], 'Display', None],
  [ 0xd1c1, 'HangulSyllableTwanj', [], 'Display', None],
  [ 0xd1c2, 'HangulSyllableTwanh', [], 'Display', None],
  [ 0xd1c3, 'HangulSyllableTwad', [], 'Display', None],
  [ 0xd1c4, 'HangulSyllableTwal', [], 'Display', None],
  [ 0xd1c5, 'HangulSyllableTwalg', [], 'Display', None],
  [ 0xd1c6, 'HangulSyllableTwalm', [], 'Display', None],
  [ 0xd1c7, 'HangulSyllableTwalb', [], 'Display', None],
  [ 0xd1c8, 'HangulSyllableTwals', [], 'Display', None],
  [ 0xd1c9, 'HangulSyllableTwalt', [], 'Display', None],
  [ 0xd1ca, 'HangulSyllableTwalp', [], 'Display', None],
  [ 0xd1cb, 'HangulSyllableTwalh', [], 'Display', None],
  [ 0xd1cc, 'HangulSyllableTwam', [], 'Display', None],
  [ 0xd1cd, 'HangulSyllableTwab', [], 'Display', None],
  [ 0xd1ce, 'HangulSyllableTwabs', [], 'Display', None],
  [ 0xd1cf, 'HangulSyllableTwas', [], 'Display', None],
  [ 0xd1d0, 'HangulSyllableTwass', [], 'Display', None],
  [ 0xd1d1, 'HangulSyllableTwang', [], 'Display', None],
  [ 0xd1d2, 'HangulSyllableTwaj', [], 'Display', None],
  [ 0xd1d3, 'HangulSyllableTwac', [], 'Display', None],
  [ 0xd1d4, 'HangulSyllableTwak', [], 'Display', None],
  [ 0xd1d5, 'HangulSyllableTwat', [], 'Display', None],
  [ 0xd1d6, 'HangulSyllableTwap', [], 'Display', None],
  [ 0xd1d7, 'HangulSyllableTwah', [], 'Display', None],
  [ 0xd1d8, 'HangulSyllableTwae', [], 'Display', None],
  [ 0xd1d9, 'HangulSyllableTwaeg', [], 'Display', None],
  [ 0xd1da, 'HangulSyllableTwaegg', [], 'Display', None],
  [ 0xd1db, 'HangulSyllableTwaegs', [], 'Display', None],
  [ 0xd1dc, 'HangulSyllableTwaen', [], 'Display', None],
  [ 0xd1dd, 'HangulSyllableTwaenj', [], 'Display', None],
  [ 0xd1de, 'HangulSyllableTwaenh', [], 'Display', None],
  [ 0xd1df, 'HangulSyllableTwaed', [], 'Display', None],
  [ 0xd1e0, 'HangulSyllableTwael', [], 'Display', None],
  [ 0xd1e1, 'HangulSyllableTwaelg', [], 'Display', None],
  [ 0xd1e2, 'HangulSyllableTwaelm', [], 'Display', None],
  [ 0xd1e3, 'HangulSyllableTwaelb', [], 'Display', None],
  [ 0xd1e4, 'HangulSyllableTwaels', [], 'Display', None],
  [ 0xd1e5, 'HangulSyllableTwaelt', [], 'Display', None],
  [ 0xd1e6, 'HangulSyllableTwaelp', [], 'Display', None],
  [ 0xd1e7, 'HangulSyllableTwaelh', [], 'Display', None],
  [ 0xd1e8, 'HangulSyllableTwaem', [], 'Display', None],
  [ 0xd1e9, 'HangulSyllableTwaeb', [], 'Display', None],
  [ 0xd1ea, 'HangulSyllableTwaebs', [], 'Display', None],
  [ 0xd1eb, 'HangulSyllableTwaes', [], 'Display', None],
  [ 0xd1ec, 'HangulSyllableTwaess', [], 'Display', None],
  [ 0xd1ed, 'HangulSyllableTwaeng', [], 'Display', None],
  [ 0xd1ee, 'HangulSyllableTwaej', [], 'Display', None],
  [ 0xd1ef, 'HangulSyllableTwaec', [], 'Display', None],
  [ 0xd1f0, 'HangulSyllableTwaek', [], 'Display', None],
  [ 0xd1f1, 'HangulSyllableTwaet', [], 'Display', None],
  [ 0xd1f2, 'HangulSyllableTwaep', [], 'Display', None],
  [ 0xd1f3, 'HangulSyllableTwaeh', [], 'Display', None],
  [ 0xd1f4, 'HangulSyllableToe', [], 'Display', None],
  [ 0xd1f5, 'HangulSyllableToeg', [], 'Display', None],
  [ 0xd1f6, 'HangulSyllableToegg', [], 'Display', None],
  [ 0xd1f7, 'HangulSyllableToegs', [], 'Display', None],
  [ 0xd1f8, 'HangulSyllableToen', [], 'Display', None],
  [ 0xd1f9, 'HangulSyllableToenj', [], 'Display', None],
  [ 0xd1fa, 'HangulSyllableToenh', [], 'Display', None],
  [ 0xd1fb, 'HangulSyllableToed', [], 'Display', None],
  [ 0xd1fc, 'HangulSyllableToel', [], 'Display', None],
  [ 0xd1fd, 'HangulSyllableToelg', [], 'Display', None],
  [ 0xd1fe, 'HangulSyllableToelm', [], 'Display', None],
  [ 0xd1ff, 'HangulSyllableToelb', [], 'Display', None],
  [ 0xd200, 'HangulSyllableToels', [], 'Display', None],
  [ 0xd201, 'HangulSyllableToelt', [], 'Display', None],
  [ 0xd202, 'HangulSyllableToelp', [], 'Display', None],
  [ 0xd203, 'HangulSyllableToelh', [], 'Display', None],
  [ 0xd204, 'HangulSyllableToem', [], 'Display', None],
  [ 0xd205, 'HangulSyllableToeb', [], 'Display', None],
  [ 0xd206, 'HangulSyllableToebs', [], 'Display', None],
  [ 0xd207, 'HangulSyllableToes', [], 'Display', None],
  [ 0xd208, 'HangulSyllableToess', [], 'Display', None],
  [ 0xd209, 'HangulSyllableToeng', [], 'Display', None],
  [ 0xd20a, 'HangulSyllableToej', [], 'Display', None],
  [ 0xd20b, 'HangulSyllableToec', [], 'Display', None],
  [ 0xd20c, 'HangulSyllableToek', [], 'Display', None],
  [ 0xd20d, 'HangulSyllableToet', [], 'Display', None],
  [ 0xd20e, 'HangulSyllableToep', [], 'Display', None],
  [ 0xd20f, 'HangulSyllableToeh', [], 'Display', None],
  [ 0xd210, 'HangulSyllableTyo', [], 'Display', None],
  [ 0xd211, 'HangulSyllableTyog', [], 'Display', None],
  [ 0xd212, 'HangulSyllableTyogg', [], 'Display', None],
  [ 0xd213, 'HangulSyllableTyogs', [], 'Display', None],
  [ 0xd214, 'HangulSyllableTyon', [], 'Display', None],
  [ 0xd215, 'HangulSyllableTyonj', [], 'Display', None],
  [ 0xd216, 'HangulSyllableTyonh', [], 'Display', None],
  [ 0xd217, 'HangulSyllableTyod', [], 'Display', None],
  [ 0xd218, 'HangulSyllableTyol', [], 'Display', None],
  [ 0xd219, 'HangulSyllableTyolg', [], 'Display', None],
  [ 0xd21a, 'HangulSyllableTyolm', [], 'Display', None],
  [ 0xd21b, 'HangulSyllableTyolb', [], 'Display', None],
  [ 0xd21c, 'HangulSyllableTyols', [], 'Display', None],
  [ 0xd21d, 'HangulSyllableTyolt', [], 'Display', None],
  [ 0xd21e, 'HangulSyllableTyolp', [], 'Display', None],
  [ 0xd21f, 'HangulSyllableTyolh', [], 'Display', None],
  [ 0xd220, 'HangulSyllableTyom', [], 'Display', None],
  [ 0xd221, 'HangulSyllableTyob', [], 'Display', None],
  [ 0xd222, 'HangulSyllableTyobs', [], 'Display', None],
  [ 0xd223, 'HangulSyllableTyos', [], 'Display', None],
  [ 0xd224, 'HangulSyllableTyoss', [], 'Display', None],
  [ 0xd225, 'HangulSyllableTyong', [], 'Display', None],
  [ 0xd226, 'HangulSyllableTyoj', [], 'Display', None],
  [ 0xd227, 'HangulSyllableTyoc', [], 'Display', None],
  [ 0xd228, 'HangulSyllableTyok', [], 'Display', None],
  [ 0xd229, 'HangulSyllableTyot', [], 'Display', None],
  [ 0xd22a, 'HangulSyllableTyop', [], 'Display', None],
  [ 0xd22b, 'HangulSyllableTyoh', [], 'Display', None],
  [ 0xd22c, 'HangulSyllableTu', [], 'Display', None],
  [ 0xd22d, 'HangulSyllableTug', [], 'Display', None],
  [ 0xd22e, 'HangulSyllableTugg', [], 'Display', None],
  [ 0xd22f, 'HangulSyllableTugs', [], 'Display', None],
  [ 0xd230, 'HangulSyllableTun', [], 'Display', None],
  [ 0xd231, 'HangulSyllableTunj', [], 'Display', None],
  [ 0xd232, 'HangulSyllableTunh', [], 'Display', None],
  [ 0xd233, 'HangulSyllableTud', [], 'Display', None],
  [ 0xd234, 'HangulSyllableTul', [], 'Display', None],
  [ 0xd235, 'HangulSyllableTulg', [], 'Display', None],
  [ 0xd236, 'HangulSyllableTulm', [], 'Display', None],
  [ 0xd237, 'HangulSyllableTulb', [], 'Display', None],
  [ 0xd238, 'HangulSyllableTuls', [], 'Display', None],
  [ 0xd239, 'HangulSyllableTult', [], 'Display', None],
  [ 0xd23a, 'HangulSyllableTulp', [], 'Display', None],
  [ 0xd23b, 'HangulSyllableTulh', [], 'Display', None],
  [ 0xd23c, 'HangulSyllableTum', [], 'Display', None],
  [ 0xd23d, 'HangulSyllableTub', [], 'Display', None],
  [ 0xd23e, 'HangulSyllableTubs', [], 'Display', None],
  [ 0xd23f, 'HangulSyllableTus', [], 'Display', None],
  [ 0xd240, 'HangulSyllableTuss', [], 'Display', None],
  [ 0xd241, 'HangulSyllableTung', [], 'Display', None],
  [ 0xd242, 'HangulSyllableTuj', [], 'Display', None],
  [ 0xd243, 'HangulSyllableTuc', [], 'Display', None],
  [ 0xd244, 'HangulSyllableTuk', [], 'Display', None],
  [ 0xd245, 'HangulSyllableTut', [], 'Display', None],
  [ 0xd246, 'HangulSyllableTup', [], 'Display', None],
  [ 0xd247, 'HangulSyllableTuh', [], 'Display', None],
  [ 0xd248, 'HangulSyllableTweo', [], 'Display', None],
  [ 0xd249, 'HangulSyllableTweog', [], 'Display', None],
  [ 0xd24a, 'HangulSyllableTweogg', [], 'Display', None],
  [ 0xd24b, 'HangulSyllableTweogs', [], 'Display', None],
  [ 0xd24c, 'HangulSyllableTweon', [], 'Display', None],
  [ 0xd24d, 'HangulSyllableTweonj', [], 'Display', None],
  [ 0xd24e, 'HangulSyllableTweonh', [], 'Display', None],
  [ 0xd24f, 'HangulSyllableTweod', [], 'Display', None],
  [ 0xd250, 'HangulSyllableTweol', [], 'Display', None],
  [ 0xd251, 'HangulSyllableTweolg', [], 'Display', None],
  [ 0xd252, 'HangulSyllableTweolm', [], 'Display', None],
  [ 0xd253, 'HangulSyllableTweolb', [], 'Display', None],
  [ 0xd254, 'HangulSyllableTweols', [], 'Display', None],
  [ 0xd255, 'HangulSyllableTweolt', [], 'Display', None],
  [ 0xd256, 'HangulSyllableTweolp', [], 'Display', None],
  [ 0xd257, 'HangulSyllableTweolh', [], 'Display', None],
  [ 0xd258, 'HangulSyllableTweom', [], 'Display', None],
  [ 0xd259, 'HangulSyllableTweob', [], 'Display', None],
  [ 0xd25a, 'HangulSyllableTweobs', [], 'Display', None],
  [ 0xd25b, 'HangulSyllableTweos', [], 'Display', None],
  [ 0xd25c, 'HangulSyllableTweoss', [], 'Display', None],
  [ 0xd25d, 'HangulSyllableTweong', [], 'Display', None],
  [ 0xd25e, 'HangulSyllableTweoj', [], 'Display', None],
  [ 0xd25f, 'HangulSyllableTweoc', [], 'Display', None],
  [ 0xd260, 'HangulSyllableTweok', [], 'Display', None],
  [ 0xd261, 'HangulSyllableTweot', [], 'Display', None],
  [ 0xd262, 'HangulSyllableTweop', [], 'Display', None],
  [ 0xd263, 'HangulSyllableTweoh', [], 'Display', None],
  [ 0xd264, 'HangulSyllableTwe', [], 'Display', None],
  [ 0xd265, 'HangulSyllableTweg', [], 'Display', None],
  [ 0xd266, 'HangulSyllableTwegg', [], 'Display', None],
  [ 0xd267, 'HangulSyllableTwegs', [], 'Display', None],
  [ 0xd268, 'HangulSyllableTwen', [], 'Display', None],
  [ 0xd269, 'HangulSyllableTwenj', [], 'Display', None],
  [ 0xd26a, 'HangulSyllableTwenh', [], 'Display', None],
  [ 0xd26b, 'HangulSyllableTwed', [], 'Display', None],
  [ 0xd26c, 'HangulSyllableTwel', [], 'Display', None],
  [ 0xd26d, 'HangulSyllableTwelg', [], 'Display', None],
  [ 0xd26e, 'HangulSyllableTwelm', [], 'Display', None],
  [ 0xd26f, 'HangulSyllableTwelb', [], 'Display', None],
  [ 0xd270, 'HangulSyllableTwels', [], 'Display', None],
  [ 0xd271, 'HangulSyllableTwelt', [], 'Display', None],
  [ 0xd272, 'HangulSyllableTwelp', [], 'Display', None],
  [ 0xd273, 'HangulSyllableTwelh', [], 'Display', None],
  [ 0xd274, 'HangulSyllableTwem', [], 'Display', None],
  [ 0xd275, 'HangulSyllableTweb', [], 'Display', None],
  [ 0xd276, 'HangulSyllableTwebs', [], 'Display', None],
  [ 0xd277, 'HangulSyllableTwes', [], 'Display', None],
  [ 0xd278, 'HangulSyllableTwess', [], 'Display', None],
  [ 0xd279, 'HangulSyllableTweng', [], 'Display', None],
  [ 0xd27a, 'HangulSyllableTwej', [], 'Display', None],
  [ 0xd27b, 'HangulSyllableTwec', [], 'Display', None],
  [ 0xd27c, 'HangulSyllableTwek', [], 'Display', None],
  [ 0xd27d, 'HangulSyllableTwet', [], 'Display', None],
  [ 0xd27e, 'HangulSyllableTwep', [], 'Display', None],
  [ 0xd27f, 'HangulSyllableTweh', [], 'Display', None],
  [ 0xd280, 'HangulSyllableTwi', [], 'Display', None],
  [ 0xd281, 'HangulSyllableTwig', [], 'Display', None],
  [ 0xd282, 'HangulSyllableTwigg', [], 'Display', None],
  [ 0xd283, 'HangulSyllableTwigs', [], 'Display', None],
  [ 0xd284, 'HangulSyllableTwin', [], 'Display', None],
  [ 0xd285, 'HangulSyllableTwinj', [], 'Display', None],
  [ 0xd286, 'HangulSyllableTwinh', [], 'Display', None],
  [ 0xd287, 'HangulSyllableTwid', [], 'Display', None],
  [ 0xd288, 'HangulSyllableTwil', [], 'Display', None],
  [ 0xd289, 'HangulSyllableTwilg', [], 'Display', None],
  [ 0xd28a, 'HangulSyllableTwilm', [], 'Display', None],
  [ 0xd28b, 'HangulSyllableTwilb', [], 'Display', None],
  [ 0xd28c, 'HangulSyllableTwils', [], 'Display', None],
  [ 0xd28d, 'HangulSyllableTwilt', [], 'Display', None],
  [ 0xd28e, 'HangulSyllableTwilp', [], 'Display', None],
  [ 0xd28f, 'HangulSyllableTwilh', [], 'Display', None],
  [ 0xd290, 'HangulSyllableTwim', [], 'Display', None],
  [ 0xd291, 'HangulSyllableTwib', [], 'Display', None],
  [ 0xd292, 'HangulSyllableTwibs', [], 'Display', None],
  [ 0xd293, 'HangulSyllableTwis', [], 'Display', None],
  [ 0xd294, 'HangulSyllableTwiss', [], 'Display', None],
  [ 0xd295, 'HangulSyllableTwing', [], 'Display', None],
  [ 0xd296, 'HangulSyllableTwij', [], 'Display', None],
  [ 0xd297, 'HangulSyllableTwic', [], 'Display', None],
  [ 0xd298, 'HangulSyllableTwik', [], 'Display', None],
  [ 0xd299, 'HangulSyllableTwit', [], 'Display', None],
  [ 0xd29a, 'HangulSyllableTwip', [], 'Display', None],
  [ 0xd29b, 'HangulSyllableTwih', [], 'Display', None],
  [ 0xd29c, 'HangulSyllableTyu', [], 'Display', None],
  [ 0xd29d, 'HangulSyllableTyug', [], 'Display', None],
  [ 0xd29e, 'HangulSyllableTyugg', [], 'Display', None],
  [ 0xd29f, 'HangulSyllableTyugs', [], 'Display', None],
  [ 0xd2a0, 'HangulSyllableTyun', [], 'Display', None],
  [ 0xd2a1, 'HangulSyllableTyunj', [], 'Display', None],
  [ 0xd2a2, 'HangulSyllableTyunh', [], 'Display', None],
  [ 0xd2a3, 'HangulSyllableTyud', [], 'Display', None],
  [ 0xd2a4, 'HangulSyllableTyul', [], 'Display', None],
  [ 0xd2a5, 'HangulSyllableTyulg', [], 'Display', None],
  [ 0xd2a6, 'HangulSyllableTyulm', [], 'Display', None],
  [ 0xd2a7, 'HangulSyllableTyulb', [], 'Display', None],
  [ 0xd2a8, 'HangulSyllableTyuls', [], 'Display', None],
  [ 0xd2a9, 'HangulSyllableTyult', [], 'Display', None],
  [ 0xd2aa, 'HangulSyllableTyulp', [], 'Display', None],
  [ 0xd2ab, 'HangulSyllableTyulh', [], 'Display', None],
  [ 0xd2ac, 'HangulSyllableTyum', [], 'Display', None],
  [ 0xd2ad, 'HangulSyllableTyub', [], 'Display', None],
  [ 0xd2ae, 'HangulSyllableTyubs', [], 'Display', None],
  [ 0xd2af, 'HangulSyllableTyus', [], 'Display', None],
  [ 0xd2b0, 'HangulSyllableTyuss', [], 'Display', None],
  [ 0xd2b1, 'HangulSyllableTyung', [], 'Display', None],
  [ 0xd2b2, 'HangulSyllableTyuj', [], 'Display', None],
  [ 0xd2b3, 'HangulSyllableTyuc', [], 'Display', None],
  [ 0xd2b4, 'HangulSyllableTyuk', [], 'Display', None],
  [ 0xd2b5, 'HangulSyllableTyut', [], 'Display', None],
  [ 0xd2b6, 'HangulSyllableTyup', [], 'Display', None],
  [ 0xd2b7, 'HangulSyllableTyuh', [], 'Display', None],
  [ 0xd2b8, 'HangulSyllableTeu', [], 'Display', None],
  [ 0xd2b9, 'HangulSyllableTeug', [], 'Display', None],
  [ 0xd2ba, 'HangulSyllableTeugg', [], 'Display', None],
  [ 0xd2bb, 'HangulSyllableTeugs', [], 'Display', None],
  [ 0xd2bc, 'HangulSyllableTeun', [], 'Display', None],
  [ 0xd2bd, 'HangulSyllableTeunj', [], 'Display', None],
  [ 0xd2be, 'HangulSyllableTeunh', [], 'Display', None],
  [ 0xd2bf, 'HangulSyllableTeud', [], 'Display', None],
  [ 0xd2c0, 'HangulSyllableTeul', [], 'Display', None],
  [ 0xd2c1, 'HangulSyllableTeulg', [], 'Display', None],
  [ 0xd2c2, 'HangulSyllableTeulm', [], 'Display', None],
  [ 0xd2c3, 'HangulSyllableTeulb', [], 'Display', None],
  [ 0xd2c4, 'HangulSyllableTeuls', [], 'Display', None],
  [ 0xd2c5, 'HangulSyllableTeult', [], 'Display', None],
  [ 0xd2c6, 'HangulSyllableTeulp', [], 'Display', None],
  [ 0xd2c7, 'HangulSyllableTeulh', [], 'Display', None],
  [ 0xd2c8, 'HangulSyllableTeum', [], 'Display', None],
  [ 0xd2c9, 'HangulSyllableTeub', [], 'Display', None],
  [ 0xd2ca, 'HangulSyllableTeubs', [], 'Display', None],
  [ 0xd2cb, 'HangulSyllableTeus', [], 'Display', None],
  [ 0xd2cc, 'HangulSyllableTeuss', [], 'Display', None],
  [ 0xd2cd, 'HangulSyllableTeung', [], 'Display', None],
  [ 0xd2ce, 'HangulSyllableTeuj', [], 'Display', None],
  [ 0xd2cf, 'HangulSyllableTeuc', [], 'Display', None],
  [ 0xd2d0, 'HangulSyllableTeuk', [], 'Display', None],
  [ 0xd2d1, 'HangulSyllableTeut', [], 'Display', None],
  [ 0xd2d2, 'HangulSyllableTeup', [], 'Display', None],
  [ 0xd2d3, 'HangulSyllableTeuh', [], 'Display', None],
  [ 0xd2d4, 'HangulSyllableTyi', [], 'Display', None],
  [ 0xd2d5, 'HangulSyllableTyig', [], 'Display', None],
  [ 0xd2d6, 'HangulSyllableTyigg', [], 'Display', None],
  [ 0xd2d7, 'HangulSyllableTyigs', [], 'Display', None],
  [ 0xd2d8, 'HangulSyllableTyin', [], 'Display', None],
  [ 0xd2d9, 'HangulSyllableTyinj', [], 'Display', None],
  [ 0xd2da, 'HangulSyllableTyinh', [], 'Display', None],
  [ 0xd2db, 'HangulSyllableTyid', [], 'Display', None],
  [ 0xd2dc, 'HangulSyllableTyil', [], 'Display', None],
  [ 0xd2dd, 'HangulSyllableTyilg', [], 'Display', None],
  [ 0xd2de, 'HangulSyllableTyilm', [], 'Display', None],
  [ 0xd2df, 'HangulSyllableTyilb', [], 'Display', None],
  [ 0xd2e0, 'HangulSyllableTyils', [], 'Display', None],
  [ 0xd2e1, 'HangulSyllableTyilt', [], 'Display', None],
  [ 0xd2e2, 'HangulSyllableTyilp', [], 'Display', None],
  [ 0xd2e3, 'HangulSyllableTyilh', [], 'Display', None],
  [ 0xd2e4, 'HangulSyllableTyim', [], 'Display', None],
  [ 0xd2e5, 'HangulSyllableTyib', [], 'Display', None],
  [ 0xd2e6, 'HangulSyllableTyibs', [], 'Display', None],
  [ 0xd2e7, 'HangulSyllableTyis', [], 'Display', None],
  [ 0xd2e8, 'HangulSyllableTyiss', [], 'Display', None],
  [ 0xd2e9, 'HangulSyllableTying', [], 'Display', None],
  [ 0xd2ea, 'HangulSyllableTyij', [], 'Display', None],
  [ 0xd2eb, 'HangulSyllableTyic', [], 'Display', None],
  [ 0xd2ec, 'HangulSyllableTyik', [], 'Display', None],
  [ 0xd2ed, 'HangulSyllableTyit', [], 'Display', None],
  [ 0xd2ee, 'HangulSyllableTyip', [], 'Display', None],
  [ 0xd2ef, 'HangulSyllableTyih', [], 'Display', None],
  [ 0xd2f0, 'HangulSyllableTi', [], 'Display', None],
  [ 0xd2f1, 'HangulSyllableTig', [], 'Display', None],
  [ 0xd2f2, 'HangulSyllableTigg', [], 'Display', None],
  [ 0xd2f3, 'HangulSyllableTigs', [], 'Display', None],
  [ 0xd2f4, 'HangulSyllableTin', [], 'Display', None],
  [ 0xd2f5, 'HangulSyllableTinj', [], 'Display', None],
  [ 0xd2f6, 'HangulSyllableTinh', [], 'Display', None],
  [ 0xd2f7, 'HangulSyllableTid', [], 'Display', None],
  [ 0xd2f8, 'HangulSyllableTil', [], 'Display', None],
  [ 0xd2f9, 'HangulSyllableTilg', [], 'Display', None],
  [ 0xd2fa, 'HangulSyllableTilm', [], 'Display', None],
  [ 0xd2fb, 'HangulSyllableTilb', [], 'Display', None],
  [ 0xd2fc, 'HangulSyllableTils', [], 'Display', None],
  [ 0xd2fd, 'HangulSyllableTilt', [], 'Display', None],
  [ 0xd2fe, 'HangulSyllableTilp', [], 'Display', None],
  [ 0xd2ff, 'HangulSyllableTilh', [], 'Display', None],
  [ 0xd300, 'HangulSyllableTim', [], 'Display', None],
  [ 0xd301, 'HangulSyllableTib', [], 'Display', None],
  [ 0xd302, 'HangulSyllableTibs', [], 'Display', None],
  [ 0xd303, 'HangulSyllableTis', [], 'Display', None],
  [ 0xd304, 'HangulSyllableTiss', [], 'Display', None],
  [ 0xd305, 'HangulSyllableTing', [], 'Display', None],
  [ 0xd306, 'HangulSyllableTij', [], 'Display', None],
  [ 0xd307, 'HangulSyllableTic', [], 'Display', None],
  [ 0xd308, 'HangulSyllableTik', [], 'Display', None],
  [ 0xd309, 'HangulSyllableTit', [], 'Display', None],
  [ 0xd30a, 'HangulSyllableTip', [], 'Display', None],
  [ 0xd30b, 'HangulSyllableTih', [], 'Display', None],
  [ 0xd30c, 'HangulSyllablePa', [], 'Display', None],
  [ 0xd30d, 'HangulSyllablePag', [], 'Display', None],
  [ 0xd30e, 'HangulSyllablePagg', [], 'Display', None],
  [ 0xd30f, 'HangulSyllablePags', [], 'Display', None],
  [ 0xd310, 'HangulSyllablePan', [], 'Display', None],
  [ 0xd311, 'HangulSyllablePanj', [], 'Display', None],
  [ 0xd312, 'HangulSyllablePanh', [], 'Display', None],
  [ 0xd313, 'HangulSyllablePad', [], 'Display', None],
  [ 0xd314, 'HangulSyllablePal', [], 'Display', None],
  [ 0xd315, 'HangulSyllablePalg', [], 'Display', None],
  [ 0xd316, 'HangulSyllablePalm', [], 'Display', None],
  [ 0xd317, 'HangulSyllablePalb', [], 'Display', None],
  [ 0xd318, 'HangulSyllablePals', [], 'Display', None],
  [ 0xd319, 'HangulSyllablePalt', [], 'Display', None],
  [ 0xd31a, 'HangulSyllablePalp', [], 'Display', None],
  [ 0xd31b, 'HangulSyllablePalh', [], 'Display', None],
  [ 0xd31c, 'HangulSyllablePam', [], 'Display', None],
  [ 0xd31d, 'HangulSyllablePab', [], 'Display', None],
  [ 0xd31e, 'HangulSyllablePabs', [], 'Display', None],
  [ 0xd31f, 'HangulSyllablePas', [], 'Display', None],
  [ 0xd320, 'HangulSyllablePass', [], 'Display', None],
  [ 0xd321, 'HangulSyllablePang', [], 'Display', None],
  [ 0xd322, 'HangulSyllablePaj', [], 'Display', None],
  [ 0xd323, 'HangulSyllablePac', [], 'Display', None],
  [ 0xd324, 'HangulSyllablePak', [], 'Display', None],
  [ 0xd325, 'HangulSyllablePat', [], 'Display', None],
  [ 0xd326, 'HangulSyllablePap', [], 'Display', None],
  [ 0xd327, 'HangulSyllablePah', [], 'Display', None],
  [ 0xd328, 'HangulSyllablePae', [], 'Display', None],
  [ 0xd329, 'HangulSyllablePaeg', [], 'Display', None],
  [ 0xd32a, 'HangulSyllablePaegg', [], 'Display', None],
  [ 0xd32b, 'HangulSyllablePaegs', [], 'Display', None],
  [ 0xd32c, 'HangulSyllablePaen', [], 'Display', None],
  [ 0xd32d, 'HangulSyllablePaenj', [], 'Display', None],
  [ 0xd32e, 'HangulSyllablePaenh', [], 'Display', None],
  [ 0xd32f, 'HangulSyllablePaed', [], 'Display', None],
  [ 0xd330, 'HangulSyllablePael', [], 'Display', None],
  [ 0xd331, 'HangulSyllablePaelg', [], 'Display', None],
  [ 0xd332, 'HangulSyllablePaelm', [], 'Display', None],
  [ 0xd333, 'HangulSyllablePaelb', [], 'Display', None],
  [ 0xd334, 'HangulSyllablePaels', [], 'Display', None],
  [ 0xd335, 'HangulSyllablePaelt', [], 'Display', None],
  [ 0xd336, 'HangulSyllablePaelp', [], 'Display', None],
  [ 0xd337, 'HangulSyllablePaelh', [], 'Display', None],
  [ 0xd338, 'HangulSyllablePaem', [], 'Display', None],
  [ 0xd339, 'HangulSyllablePaeb', [], 'Display', None],
  [ 0xd33a, 'HangulSyllablePaebs', [], 'Display', None],
  [ 0xd33b, 'HangulSyllablePaes', [], 'Display', None],
  [ 0xd33c, 'HangulSyllablePaess', [], 'Display', None],
  [ 0xd33d, 'HangulSyllablePaeng', [], 'Display', None],
  [ 0xd33e, 'HangulSyllablePaej', [], 'Display', None],
  [ 0xd33f, 'HangulSyllablePaec', [], 'Display', None],
  [ 0xd340, 'HangulSyllablePaek', [], 'Display', None],
  [ 0xd341, 'HangulSyllablePaet', [], 'Display', None],
  [ 0xd342, 'HangulSyllablePaep', [], 'Display', None],
  [ 0xd343, 'HangulSyllablePaeh', [], 'Display', None],
  [ 0xd344, 'HangulSyllablePya', [], 'Display', None],
  [ 0xd345, 'HangulSyllablePyag', [], 'Display', None],
  [ 0xd346, 'HangulSyllablePyagg', [], 'Display', None],
  [ 0xd347, 'HangulSyllablePyags', [], 'Display', None],
  [ 0xd348, 'HangulSyllablePyan', [], 'Display', None],
  [ 0xd349, 'HangulSyllablePyanj', [], 'Display', None],
  [ 0xd34a, 'HangulSyllablePyanh', [], 'Display', None],
  [ 0xd34b, 'HangulSyllablePyad', [], 'Display', None],
  [ 0xd34c, 'HangulSyllablePyal', [], 'Display', None],
  [ 0xd34d, 'HangulSyllablePyalg', [], 'Display', None],
  [ 0xd34e, 'HangulSyllablePyalm', [], 'Display', None],
  [ 0xd34f, 'HangulSyllablePyalb', [], 'Display', None],
  [ 0xd350, 'HangulSyllablePyals', [], 'Display', None],
  [ 0xd351, 'HangulSyllablePyalt', [], 'Display', None],
  [ 0xd352, 'HangulSyllablePyalp', [], 'Display', None],
  [ 0xd353, 'HangulSyllablePyalh', [], 'Display', None],
  [ 0xd354, 'HangulSyllablePyam', [], 'Display', None],
  [ 0xd355, 'HangulSyllablePyab', [], 'Display', None],
  [ 0xd356, 'HangulSyllablePyabs', [], 'Display', None],
  [ 0xd357, 'HangulSyllablePyas', [], 'Display', None],
  [ 0xd358, 'HangulSyllablePyass', [], 'Display', None],
  [ 0xd359, 'HangulSyllablePyang', [], 'Display', None],
  [ 0xd35a, 'HangulSyllablePyaj', [], 'Display', None],
  [ 0xd35b, 'HangulSyllablePyac', [], 'Display', None],
  [ 0xd35c, 'HangulSyllablePyak', [], 'Display', None],
  [ 0xd35d, 'HangulSyllablePyat', [], 'Display', None],
  [ 0xd35e, 'HangulSyllablePyap', [], 'Display', None],
  [ 0xd35f, 'HangulSyllablePyah', [], 'Display', None],
  [ 0xd360, 'HangulSyllablePyae', [], 'Display', None],
  [ 0xd361, 'HangulSyllablePyaeg', [], 'Display', None],
  [ 0xd362, 'HangulSyllablePyaegg', [], 'Display', None],
  [ 0xd363, 'HangulSyllablePyaegs', [], 'Display', None],
  [ 0xd364, 'HangulSyllablePyaen', [], 'Display', None],
  [ 0xd365, 'HangulSyllablePyaenj', [], 'Display', None],
  [ 0xd366, 'HangulSyllablePyaenh', [], 'Display', None],
  [ 0xd367, 'HangulSyllablePyaed', [], 'Display', None],
  [ 0xd368, 'HangulSyllablePyael', [], 'Display', None],
  [ 0xd369, 'HangulSyllablePyaelg', [], 'Display', None],
  [ 0xd36a, 'HangulSyllablePyaelm', [], 'Display', None],
  [ 0xd36b, 'HangulSyllablePyaelb', [], 'Display', None],
  [ 0xd36c, 'HangulSyllablePyaels', [], 'Display', None],
  [ 0xd36d, 'HangulSyllablePyaelt', [], 'Display', None],
  [ 0xd36e, 'HangulSyllablePyaelp', [], 'Display', None],
  [ 0xd36f, 'HangulSyllablePyaelh', [], 'Display', None],
  [ 0xd370, 'HangulSyllablePyaem', [], 'Display', None],
  [ 0xd371, 'HangulSyllablePyaeb', [], 'Display', None],
  [ 0xd372, 'HangulSyllablePyaebs', [], 'Display', None],
  [ 0xd373, 'HangulSyllablePyaes', [], 'Display', None],
  [ 0xd374, 'HangulSyllablePyaess', [], 'Display', None],
  [ 0xd375, 'HangulSyllablePyaeng', [], 'Display', None],
  [ 0xd376, 'HangulSyllablePyaej', [], 'Display', None],
  [ 0xd377, 'HangulSyllablePyaec', [], 'Display', None],
  [ 0xd378, 'HangulSyllablePyaek', [], 'Display', None],
  [ 0xd379, 'HangulSyllablePyaet', [], 'Display', None],
  [ 0xd37a, 'HangulSyllablePyaep', [], 'Display', None],
  [ 0xd37b, 'HangulSyllablePyaeh', [], 'Display', None],
  [ 0xd37c, 'HangulSyllablePeo', [], 'Display', None],
  [ 0xd37d, 'HangulSyllablePeog', [], 'Display', None],
  [ 0xd37e, 'HangulSyllablePeogg', [], 'Display', None],
  [ 0xd37f, 'HangulSyllablePeogs', [], 'Display', None],
  [ 0xd380, 'HangulSyllablePeon', [], 'Display', None],
  [ 0xd381, 'HangulSyllablePeonj', [], 'Display', None],
  [ 0xd382, 'HangulSyllablePeonh', [], 'Display', None],
  [ 0xd383, 'HangulSyllablePeod', [], 'Display', None],
  [ 0xd384, 'HangulSyllablePeol', [], 'Display', None],
  [ 0xd385, 'HangulSyllablePeolg', [], 'Display', None],
  [ 0xd386, 'HangulSyllablePeolm', [], 'Display', None],
  [ 0xd387, 'HangulSyllablePeolb', [], 'Display', None],
  [ 0xd388, 'HangulSyllablePeols', [], 'Display', None],
  [ 0xd389, 'HangulSyllablePeolt', [], 'Display', None],
  [ 0xd38a, 'HangulSyllablePeolp', [], 'Display', None],
  [ 0xd38b, 'HangulSyllablePeolh', [], 'Display', None],
  [ 0xd38c, 'HangulSyllablePeom', [], 'Display', None],
  [ 0xd38d, 'HangulSyllablePeob', [], 'Display', None],
  [ 0xd38e, 'HangulSyllablePeobs', [], 'Display', None],
  [ 0xd38f, 'HangulSyllablePeos', [], 'Display', None],
  [ 0xd390, 'HangulSyllablePeoss', [], 'Display', None],
  [ 0xd391, 'HangulSyllablePeong', [], 'Display', None],
  [ 0xd392, 'HangulSyllablePeoj', [], 'Display', None],
  [ 0xd393, 'HangulSyllablePeoc', [], 'Display', None],
  [ 0xd394, 'HangulSyllablePeok', [], 'Display', None],
  [ 0xd395, 'HangulSyllablePeot', [], 'Display', None],
  [ 0xd396, 'HangulSyllablePeop', [], 'Display', None],
  [ 0xd397, 'HangulSyllablePeoh', [], 'Display', None],
  [ 0xd398, 'HangulSyllablePe', [], 'Display', None],
  [ 0xd399, 'HangulSyllablePeg', [], 'Display', None],
  [ 0xd39a, 'HangulSyllablePegg', [], 'Display', None],
  [ 0xd39b, 'HangulSyllablePegs', [], 'Display', None],
  [ 0xd39c, 'HangulSyllablePen', [], 'Display', None],
  [ 0xd39d, 'HangulSyllablePenj', [], 'Display', None],
  [ 0xd39e, 'HangulSyllablePenh', [], 'Display', None],
  [ 0xd39f, 'HangulSyllablePed', [], 'Display', None],
  [ 0xd3a0, 'HangulSyllablePel', [], 'Display', None],
  [ 0xd3a1, 'HangulSyllablePelg', [], 'Display', None],
  [ 0xd3a2, 'HangulSyllablePelm', [], 'Display', None],
  [ 0xd3a3, 'HangulSyllablePelb', [], 'Display', None],
  [ 0xd3a4, 'HangulSyllablePels', [], 'Display', None],
  [ 0xd3a5, 'HangulSyllablePelt', [], 'Display', None],
  [ 0xd3a6, 'HangulSyllablePelp', [], 'Display', None],
  [ 0xd3a7, 'HangulSyllablePelh', [], 'Display', None],
  [ 0xd3a8, 'HangulSyllablePem', [], 'Display', None],
  [ 0xd3a9, 'HangulSyllablePeb', [], 'Display', None],
  [ 0xd3aa, 'HangulSyllablePebs', [], 'Display', None],
  [ 0xd3ab, 'HangulSyllablePes', [], 'Display', None],
  [ 0xd3ac, 'HangulSyllablePess', [], 'Display', None],
  [ 0xd3ad, 'HangulSyllablePeng', [], 'Display', None],
  [ 0xd3ae, 'HangulSyllablePej', [], 'Display', None],
  [ 0xd3af, 'HangulSyllablePec', [], 'Display', None],
  [ 0xd3b0, 'HangulSyllablePek', [], 'Display', None],
  [ 0xd3b1, 'HangulSyllablePet', [], 'Display', None],
  [ 0xd3b2, 'HangulSyllablePep', [], 'Display', None],
  [ 0xd3b3, 'HangulSyllablePeh', [], 'Display', None],
  [ 0xd3b4, 'HangulSyllablePyeo', [], 'Display', None],
  [ 0xd3b5, 'HangulSyllablePyeog', [], 'Display', None],
  [ 0xd3b6, 'HangulSyllablePyeogg', [], 'Display', None],
  [ 0xd3b7, 'HangulSyllablePyeogs', [], 'Display', None],
  [ 0xd3b8, 'HangulSyllablePyeon', [], 'Display', None],
  [ 0xd3b9, 'HangulSyllablePyeonj', [], 'Display', None],
  [ 0xd3ba, 'HangulSyllablePyeonh', [], 'Display', None],
  [ 0xd3bb, 'HangulSyllablePyeod', [], 'Display', None],
  [ 0xd3bc, 'HangulSyllablePyeol', [], 'Display', None],
  [ 0xd3bd, 'HangulSyllablePyeolg', [], 'Display', None],
  [ 0xd3be, 'HangulSyllablePyeolm', [], 'Display', None],
  [ 0xd3bf, 'HangulSyllablePyeolb', [], 'Display', None],
  [ 0xd3c0, 'HangulSyllablePyeols', [], 'Display', None],
  [ 0xd3c1, 'HangulSyllablePyeolt', [], 'Display', None],
  [ 0xd3c2, 'HangulSyllablePyeolp', [], 'Display', None],
  [ 0xd3c3, 'HangulSyllablePyeolh', [], 'Display', None],
  [ 0xd3c4, 'HangulSyllablePyeom', [], 'Display', None],
  [ 0xd3c5, 'HangulSyllablePyeob', [], 'Display', None],
  [ 0xd3c6, 'HangulSyllablePyeobs', [], 'Display', None],
  [ 0xd3c7, 'HangulSyllablePyeos', [], 'Display', None],
  [ 0xd3c8, 'HangulSyllablePyeoss', [], 'Display', None],
  [ 0xd3c9, 'HangulSyllablePyeong', [], 'Display', None],
  [ 0xd3ca, 'HangulSyllablePyeoj', [], 'Display', None],
  [ 0xd3cb, 'HangulSyllablePyeoc', [], 'Display', None],
  [ 0xd3cc, 'HangulSyllablePyeok', [], 'Display', None],
  [ 0xd3cd, 'HangulSyllablePyeot', [], 'Display', None],
  [ 0xd3ce, 'HangulSyllablePyeop', [], 'Display', None],
  [ 0xd3cf, 'HangulSyllablePyeoh', [], 'Display', None],
  [ 0xd3d0, 'HangulSyllablePye', [], 'Display', None],
  [ 0xd3d1, 'HangulSyllablePyeg', [], 'Display', None],
  [ 0xd3d2, 'HangulSyllablePyegg', [], 'Display', None],
  [ 0xd3d3, 'HangulSyllablePyegs', [], 'Display', None],
  [ 0xd3d4, 'HangulSyllablePyen', [], 'Display', None],
  [ 0xd3d5, 'HangulSyllablePyenj', [], 'Display', None],
  [ 0xd3d6, 'HangulSyllablePyenh', [], 'Display', None],
  [ 0xd3d7, 'HangulSyllablePyed', [], 'Display', None],
  [ 0xd3d8, 'HangulSyllablePyel', [], 'Display', None],
  [ 0xd3d9, 'HangulSyllablePyelg', [], 'Display', None],
  [ 0xd3da, 'HangulSyllablePyelm', [], 'Display', None],
  [ 0xd3db, 'HangulSyllablePyelb', [], 'Display', None],
  [ 0xd3dc, 'HangulSyllablePyels', [], 'Display', None],
  [ 0xd3dd, 'HangulSyllablePyelt', [], 'Display', None],
  [ 0xd3de, 'HangulSyllablePyelp', [], 'Display', None],
  [ 0xd3df, 'HangulSyllablePyelh', [], 'Display', None],
  [ 0xd3e0, 'HangulSyllablePyem', [], 'Display', None],
  [ 0xd3e1, 'HangulSyllablePyeb', [], 'Display', None],
  [ 0xd3e2, 'HangulSyllablePyebs', [], 'Display', None],
  [ 0xd3e3, 'HangulSyllablePyes', [], 'Display', None],
  [ 0xd3e4, 'HangulSyllablePyess', [], 'Display', None],
  [ 0xd3e5, 'HangulSyllablePyeng', [], 'Display', None],
  [ 0xd3e6, 'HangulSyllablePyej', [], 'Display', None],
  [ 0xd3e7, 'HangulSyllablePyec', [], 'Display', None],
  [ 0xd3e8, 'HangulSyllablePyek', [], 'Display', None],
  [ 0xd3e9, 'HangulSyllablePyet', [], 'Display', None],
  [ 0xd3ea, 'HangulSyllablePyep', [], 'Display', None],
  [ 0xd3eb, 'HangulSyllablePyeh', [], 'Display', None],
  [ 0xd3ec, 'HangulSyllablePo', [], 'Display', None],
  [ 0xd3ed, 'HangulSyllablePog', [], 'Display', None],
  [ 0xd3ee, 'HangulSyllablePogg', [], 'Display', None],
  [ 0xd3ef, 'HangulSyllablePogs', [], 'Display', None],
  [ 0xd3f0, 'HangulSyllablePon', [], 'Display', None],
  [ 0xd3f1, 'HangulSyllablePonj', [], 'Display', None],
  [ 0xd3f2, 'HangulSyllablePonh', [], 'Display', None],
  [ 0xd3f3, 'HangulSyllablePod', [], 'Display', None],
  [ 0xd3f4, 'HangulSyllablePol', [], 'Display', None],
  [ 0xd3f5, 'HangulSyllablePolg', [], 'Display', None],
  [ 0xd3f6, 'HangulSyllablePolm', [], 'Display', None],
  [ 0xd3f7, 'HangulSyllablePolb', [], 'Display', None],
  [ 0xd3f8, 'HangulSyllablePols', [], 'Display', None],
  [ 0xd3f9, 'HangulSyllablePolt', [], 'Display', None],
  [ 0xd3fa, 'HangulSyllablePolp', [], 'Display', None],
  [ 0xd3fb, 'HangulSyllablePolh', [], 'Display', None],
  [ 0xd3fc, 'HangulSyllablePom', [], 'Display', None],
  [ 0xd3fd, 'HangulSyllablePob', [], 'Display', None],
  [ 0xd3fe, 'HangulSyllablePobs', [], 'Display', None],
  [ 0xd3ff, 'HangulSyllablePos', [], 'Display', None],
  [ 0xd400, 'HangulSyllablePoss', [], 'Display', None],
  [ 0xd401, 'HangulSyllablePong', [], 'Display', None],
  [ 0xd402, 'HangulSyllablePoj', [], 'Display', None],
  [ 0xd403, 'HangulSyllablePoc', [], 'Display', None],
  [ 0xd404, 'HangulSyllablePok', [], 'Display', None],
  [ 0xd405, 'HangulSyllablePot', [], 'Display', None],
  [ 0xd406, 'HangulSyllablePop', [], 'Display', None],
  [ 0xd407, 'HangulSyllablePoh', [], 'Display', None],
  [ 0xd408, 'HangulSyllablePwa', [], 'Display', None],
  [ 0xd409, 'HangulSyllablePwag', [], 'Display', None],
  [ 0xd40a, 'HangulSyllablePwagg', [], 'Display', None],
  [ 0xd40b, 'HangulSyllablePwags', [], 'Display', None],
  [ 0xd40c, 'HangulSyllablePwan', [], 'Display', None],
  [ 0xd40d, 'HangulSyllablePwanj', [], 'Display', None],
  [ 0xd40e, 'HangulSyllablePwanh', [], 'Display', None],
  [ 0xd40f, 'HangulSyllablePwad', [], 'Display', None],
  [ 0xd410, 'HangulSyllablePwal', [], 'Display', None],
  [ 0xd411, 'HangulSyllablePwalg', [], 'Display', None],
  [ 0xd412, 'HangulSyllablePwalm', [], 'Display', None],
  [ 0xd413, 'HangulSyllablePwalb', [], 'Display', None],
  [ 0xd414, 'HangulSyllablePwals', [], 'Display', None],
  [ 0xd415, 'HangulSyllablePwalt', [], 'Display', None],
  [ 0xd416, 'HangulSyllablePwalp', [], 'Display', None],
  [ 0xd417, 'HangulSyllablePwalh', [], 'Display', None],
  [ 0xd418, 'HangulSyllablePwam', [], 'Display', None],
  [ 0xd419, 'HangulSyllablePwab', [], 'Display', None],
  [ 0xd41a, 'HangulSyllablePwabs', [], 'Display', None],
  [ 0xd41b, 'HangulSyllablePwas', [], 'Display', None],
  [ 0xd41c, 'HangulSyllablePwass', [], 'Display', None],
  [ 0xd41d, 'HangulSyllablePwang', [], 'Display', None],
  [ 0xd41e, 'HangulSyllablePwaj', [], 'Display', None],
  [ 0xd41f, 'HangulSyllablePwac', [], 'Display', None],
  [ 0xd420, 'HangulSyllablePwak', [], 'Display', None],
  [ 0xd421, 'HangulSyllablePwat', [], 'Display', None],
  [ 0xd422, 'HangulSyllablePwap', [], 'Display', None],
  [ 0xd423, 'HangulSyllablePwah', [], 'Display', None],
  [ 0xd424, 'HangulSyllablePwae', [], 'Display', None],
  [ 0xd425, 'HangulSyllablePwaeg', [], 'Display', None],
  [ 0xd426, 'HangulSyllablePwaegg', [], 'Display', None],
  [ 0xd427, 'HangulSyllablePwaegs', [], 'Display', None],
  [ 0xd428, 'HangulSyllablePwaen', [], 'Display', None],
  [ 0xd429, 'HangulSyllablePwaenj', [], 'Display', None],
  [ 0xd42a, 'HangulSyllablePwaenh', [], 'Display', None],
  [ 0xd42b, 'HangulSyllablePwaed', [], 'Display', None],
  [ 0xd42c, 'HangulSyllablePwael', [], 'Display', None],
  [ 0xd42d, 'HangulSyllablePwaelg', [], 'Display', None],
  [ 0xd42e, 'HangulSyllablePwaelm', [], 'Display', None],
  [ 0xd42f, 'HangulSyllablePwaelb', [], 'Display', None],
  [ 0xd430, 'HangulSyllablePwaels', [], 'Display', None],
  [ 0xd431, 'HangulSyllablePwaelt', [], 'Display', None],
  [ 0xd432, 'HangulSyllablePwaelp', [], 'Display', None],
  [ 0xd433, 'HangulSyllablePwaelh', [], 'Display', None],
  [ 0xd434, 'HangulSyllablePwaem', [], 'Display', None],
  [ 0xd435, 'HangulSyllablePwaeb', [], 'Display', None],
  [ 0xd436, 'HangulSyllablePwaebs', [], 'Display', None],
  [ 0xd437, 'HangulSyllablePwaes', [], 'Display', None],
  [ 0xd438, 'HangulSyllablePwaess', [], 'Display', None],
  [ 0xd439, 'HangulSyllablePwaeng', [], 'Display', None],
  [ 0xd43a, 'HangulSyllablePwaej', [], 'Display', None],
  [ 0xd43b, 'HangulSyllablePwaec', [], 'Display', None],
  [ 0xd43c, 'HangulSyllablePwaek', [], 'Display', None],
  [ 0xd43d, 'HangulSyllablePwaet', [], 'Display', None],
  [ 0xd43e, 'HangulSyllablePwaep', [], 'Display', None],
  [ 0xd43f, 'HangulSyllablePwaeh', [], 'Display', None],
  [ 0xd440, 'HangulSyllablePoe', [], 'Display', None],
  [ 0xd441, 'HangulSyllablePoeg', [], 'Display', None],
  [ 0xd442, 'HangulSyllablePoegg', [], 'Display', None],
  [ 0xd443, 'HangulSyllablePoegs', [], 'Display', None],
  [ 0xd444, 'HangulSyllablePoen', [], 'Display', None],
  [ 0xd445, 'HangulSyllablePoenj', [], 'Display', None],
  [ 0xd446, 'HangulSyllablePoenh', [], 'Display', None],
  [ 0xd447, 'HangulSyllablePoed', [], 'Display', None],
  [ 0xd448, 'HangulSyllablePoel', [], 'Display', None],
  [ 0xd449, 'HangulSyllablePoelg', [], 'Display', None],
  [ 0xd44a, 'HangulSyllablePoelm', [], 'Display', None],
  [ 0xd44b, 'HangulSyllablePoelb', [], 'Display', None],
  [ 0xd44c, 'HangulSyllablePoels', [], 'Display', None],
  [ 0xd44d, 'HangulSyllablePoelt', [], 'Display', None],
  [ 0xd44e, 'HangulSyllablePoelp', [], 'Display', None],
  [ 0xd44f, 'HangulSyllablePoelh', [], 'Display', None],
  [ 0xd450, 'HangulSyllablePoem', [], 'Display', None],
  [ 0xd451, 'HangulSyllablePoeb', [], 'Display', None],
  [ 0xd452, 'HangulSyllablePoebs', [], 'Display', None],
  [ 0xd453, 'HangulSyllablePoes', [], 'Display', None],
  [ 0xd454, 'HangulSyllablePoess', [], 'Display', None],
  [ 0xd455, 'HangulSyllablePoeng', [], 'Display', None],
  [ 0xd456, 'HangulSyllablePoej', [], 'Display', None],
  [ 0xd457, 'HangulSyllablePoec', [], 'Display', None],
  [ 0xd458, 'HangulSyllablePoek', [], 'Display', None],
  [ 0xd459, 'HangulSyllablePoet', [], 'Display', None],
  [ 0xd45a, 'HangulSyllablePoep', [], 'Display', None],
  [ 0xd45b, 'HangulSyllablePoeh', [], 'Display', None],
  [ 0xd45c, 'HangulSyllablePyo', [], 'Display', None],
  [ 0xd45d, 'HangulSyllablePyog', [], 'Display', None],
  [ 0xd45e, 'HangulSyllablePyogg', [], 'Display', None],
  [ 0xd45f, 'HangulSyllablePyogs', [], 'Display', None],
  [ 0xd460, 'HangulSyllablePyon', [], 'Display', None],
  [ 0xd461, 'HangulSyllablePyonj', [], 'Display', None],
  [ 0xd462, 'HangulSyllablePyonh', [], 'Display', None],
  [ 0xd463, 'HangulSyllablePyod', [], 'Display', None],
  [ 0xd464, 'HangulSyllablePyol', [], 'Display', None],
  [ 0xd465, 'HangulSyllablePyolg', [], 'Display', None],
  [ 0xd466, 'HangulSyllablePyolm', [], 'Display', None],
  [ 0xd467, 'HangulSyllablePyolb', [], 'Display', None],
  [ 0xd468, 'HangulSyllablePyols', [], 'Display', None],
  [ 0xd469, 'HangulSyllablePyolt', [], 'Display', None],
  [ 0xd46a, 'HangulSyllablePyolp', [], 'Display', None],
  [ 0xd46b, 'HangulSyllablePyolh', [], 'Display', None],
  [ 0xd46c, 'HangulSyllablePyom', [], 'Display', None],
  [ 0xd46d, 'HangulSyllablePyob', [], 'Display', None],
  [ 0xd46e, 'HangulSyllablePyobs', [], 'Display', None],
  [ 0xd46f, 'HangulSyllablePyos', [], 'Display', None],
  [ 0xd470, 'HangulSyllablePyoss', [], 'Display', None],
  [ 0xd471, 'HangulSyllablePyong', [], 'Display', None],
  [ 0xd472, 'HangulSyllablePyoj', [], 'Display', None],
  [ 0xd473, 'HangulSyllablePyoc', [], 'Display', None],
  [ 0xd474, 'HangulSyllablePyok', [], 'Display', None],
  [ 0xd475, 'HangulSyllablePyot', [], 'Display', None],
  [ 0xd476, 'HangulSyllablePyop', [], 'Display', None],
  [ 0xd477, 'HangulSyllablePyoh', [], 'Display', None],
  [ 0xd478, 'HangulSyllablePu', [], 'Display', None],
  [ 0xd479, 'HangulSyllablePug', [], 'Display', None],
  [ 0xd47a, 'HangulSyllablePugg', [], 'Display', None],
  [ 0xd47b, 'HangulSyllablePugs', [], 'Display', None],
  [ 0xd47c, 'HangulSyllablePun', [], 'Display', None],
  [ 0xd47d, 'HangulSyllablePunj', [], 'Display', None],
  [ 0xd47e, 'HangulSyllablePunh', [], 'Display', None],
  [ 0xd47f, 'HangulSyllablePud', [], 'Display', None],
  [ 0xd480, 'HangulSyllablePul', [], 'Display', None],
  [ 0xd481, 'HangulSyllablePulg', [], 'Display', None],
  [ 0xd482, 'HangulSyllablePulm', [], 'Display', None],
  [ 0xd483, 'HangulSyllablePulb', [], 'Display', None],
  [ 0xd484, 'HangulSyllablePuls', [], 'Display', None],
  [ 0xd485, 'HangulSyllablePult', [], 'Display', None],
  [ 0xd486, 'HangulSyllablePulp', [], 'Display', None],
  [ 0xd487, 'HangulSyllablePulh', [], 'Display', None],
  [ 0xd488, 'HangulSyllablePum', [], 'Display', None],
  [ 0xd489, 'HangulSyllablePub', [], 'Display', None],
  [ 0xd48a, 'HangulSyllablePubs', [], 'Display', None],
  [ 0xd48b, 'HangulSyllablePus', [], 'Display', None],
  [ 0xd48c, 'HangulSyllablePuss', [], 'Display', None],
  [ 0xd48d, 'HangulSyllablePung', [], 'Display', None],
  [ 0xd48e, 'HangulSyllablePuj', [], 'Display', None],
  [ 0xd48f, 'HangulSyllablePuc', [], 'Display', None],
  [ 0xd490, 'HangulSyllablePuk', [], 'Display', None],
  [ 0xd491, 'HangulSyllablePut', [], 'Display', None],
  [ 0xd492, 'HangulSyllablePup', [], 'Display', None],
  [ 0xd493, 'HangulSyllablePuh', [], 'Display', None],
  [ 0xd494, 'HangulSyllablePweo', [], 'Display', None],
  [ 0xd495, 'HangulSyllablePweog', [], 'Display', None],
  [ 0xd496, 'HangulSyllablePweogg', [], 'Display', None],
  [ 0xd497, 'HangulSyllablePweogs', [], 'Display', None],
  [ 0xd498, 'HangulSyllablePweon', [], 'Display', None],
  [ 0xd499, 'HangulSyllablePweonj', [], 'Display', None],
  [ 0xd49a, 'HangulSyllablePweonh', [], 'Display', None],
  [ 0xd49b, 'HangulSyllablePweod', [], 'Display', None],
  [ 0xd49c, 'HangulSyllablePweol', [], 'Display', None],
  [ 0xd49d, 'HangulSyllablePweolg', [], 'Display', None],
  [ 0xd49e, 'HangulSyllablePweolm', [], 'Display', None],
  [ 0xd49f, 'HangulSyllablePweolb', [], 'Display', None],
  [ 0xd4a0, 'HangulSyllablePweols', [], 'Display', None],
  [ 0xd4a1, 'HangulSyllablePweolt', [], 'Display', None],
  [ 0xd4a2, 'HangulSyllablePweolp', [], 'Display', None],
  [ 0xd4a3, 'HangulSyllablePweolh', [], 'Display', None],
  [ 0xd4a4, 'HangulSyllablePweom', [], 'Display', None],
  [ 0xd4a5, 'HangulSyllablePweob', [], 'Display', None],
  [ 0xd4a6, 'HangulSyllablePweobs', [], 'Display', None],
  [ 0xd4a7, 'HangulSyllablePweos', [], 'Display', None],
  [ 0xd4a8, 'HangulSyllablePweoss', [], 'Display', None],
  [ 0xd4a9, 'HangulSyllablePweong', [], 'Display', None],
  [ 0xd4aa, 'HangulSyllablePweoj', [], 'Display', None],
  [ 0xd4ab, 'HangulSyllablePweoc', [], 'Display', None],
  [ 0xd4ac, 'HangulSyllablePweok', [], 'Display', None],
  [ 0xd4ad, 'HangulSyllablePweot', [], 'Display', None],
  [ 0xd4ae, 'HangulSyllablePweop', [], 'Display', None],
  [ 0xd4af, 'HangulSyllablePweoh', [], 'Display', None],
  [ 0xd4b0, 'HangulSyllablePwe', [], 'Display', None],
  [ 0xd4b1, 'HangulSyllablePweg', [], 'Display', None],
  [ 0xd4b2, 'HangulSyllablePwegg', [], 'Display', None],
  [ 0xd4b3, 'HangulSyllablePwegs', [], 'Display', None],
  [ 0xd4b4, 'HangulSyllablePwen', [], 'Display', None],
  [ 0xd4b5, 'HangulSyllablePwenj', [], 'Display', None],
  [ 0xd4b6, 'HangulSyllablePwenh', [], 'Display', None],
  [ 0xd4b7, 'HangulSyllablePwed', [], 'Display', None],
  [ 0xd4b8, 'HangulSyllablePwel', [], 'Display', None],
  [ 0xd4b9, 'HangulSyllablePwelg', [], 'Display', None],
  [ 0xd4ba, 'HangulSyllablePwelm', [], 'Display', None],
  [ 0xd4bb, 'HangulSyllablePwelb', [], 'Display', None],
  [ 0xd4bc, 'HangulSyllablePwels', [], 'Display', None],
  [ 0xd4bd, 'HangulSyllablePwelt', [], 'Display', None],
  [ 0xd4be, 'HangulSyllablePwelp', [], 'Display', None],
  [ 0xd4bf, 'HangulSyllablePwelh', [], 'Display', None],
  [ 0xd4c0, 'HangulSyllablePwem', [], 'Display', None],
  [ 0xd4c1, 'HangulSyllablePweb', [], 'Display', None],
  [ 0xd4c2, 'HangulSyllablePwebs', [], 'Display', None],
  [ 0xd4c3, 'HangulSyllablePwes', [], 'Display', None],
  [ 0xd4c4, 'HangulSyllablePwess', [], 'Display', None],
  [ 0xd4c5, 'HangulSyllablePweng', [], 'Display', None],
  [ 0xd4c6, 'HangulSyllablePwej', [], 'Display', None],
  [ 0xd4c7, 'HangulSyllablePwec', [], 'Display', None],
  [ 0xd4c8, 'HangulSyllablePwek', [], 'Display', None],
  [ 0xd4c9, 'HangulSyllablePwet', [], 'Display', None],
  [ 0xd4ca, 'HangulSyllablePwep', [], 'Display', None],
  [ 0xd4cb, 'HangulSyllablePweh', [], 'Display', None],
  [ 0xd4cc, 'HangulSyllablePwi', [], 'Display', None],
  [ 0xd4cd, 'HangulSyllablePwig', [], 'Display', None],
  [ 0xd4ce, 'HangulSyllablePwigg', [], 'Display', None],
  [ 0xd4cf, 'HangulSyllablePwigs', [], 'Display', None],
  [ 0xd4d0, 'HangulSyllablePwin', [], 'Display', None],
  [ 0xd4d1, 'HangulSyllablePwinj', [], 'Display', None],
  [ 0xd4d2, 'HangulSyllablePwinh', [], 'Display', None],
  [ 0xd4d3, 'HangulSyllablePwid', [], 'Display', None],
  [ 0xd4d4, 'HangulSyllablePwil', [], 'Display', None],
  [ 0xd4d5, 'HangulSyllablePwilg', [], 'Display', None],
  [ 0xd4d6, 'HangulSyllablePwilm', [], 'Display', None],
  [ 0xd4d7, 'HangulSyllablePwilb', [], 'Display', None],
  [ 0xd4d8, 'HangulSyllablePwils', [], 'Display', None],
  [ 0xd4d9, 'HangulSyllablePwilt', [], 'Display', None],
  [ 0xd4da, 'HangulSyllablePwilp', [], 'Display', None],
  [ 0xd4db, 'HangulSyllablePwilh', [], 'Display', None],
  [ 0xd4dc, 'HangulSyllablePwim', [], 'Display', None],
  [ 0xd4dd, 'HangulSyllablePwib', [], 'Display', None],
  [ 0xd4de, 'HangulSyllablePwibs', [], 'Display', None],
  [ 0xd4df, 'HangulSyllablePwis', [], 'Display', None],
  [ 0xd4e0, 'HangulSyllablePwiss', [], 'Display', None],
  [ 0xd4e1, 'HangulSyllablePwing', [], 'Display', None],
  [ 0xd4e2, 'HangulSyllablePwij', [], 'Display', None],
  [ 0xd4e3, 'HangulSyllablePwic', [], 'Display', None],
  [ 0xd4e4, 'HangulSyllablePwik', [], 'Display', None],
  [ 0xd4e5, 'HangulSyllablePwit', [], 'Display', None],
  [ 0xd4e6, 'HangulSyllablePwip', [], 'Display', None],
  [ 0xd4e7, 'HangulSyllablePwih', [], 'Display', None],
  [ 0xd4e8, 'HangulSyllablePyu', [], 'Display', None],
  [ 0xd4e9, 'HangulSyllablePyug', [], 'Display', None],
  [ 0xd4ea, 'HangulSyllablePyugg', [], 'Display', None],
  [ 0xd4eb, 'HangulSyllablePyugs', [], 'Display', None],
  [ 0xd4ec, 'HangulSyllablePyun', [], 'Display', None],
  [ 0xd4ed, 'HangulSyllablePyunj', [], 'Display', None],
  [ 0xd4ee, 'HangulSyllablePyunh', [], 'Display', None],
  [ 0xd4ef, 'HangulSyllablePyud', [], 'Display', None],
  [ 0xd4f0, 'HangulSyllablePyul', [], 'Display', None],
  [ 0xd4f1, 'HangulSyllablePyulg', [], 'Display', None],
  [ 0xd4f2, 'HangulSyllablePyulm', [], 'Display', None],
  [ 0xd4f3, 'HangulSyllablePyulb', [], 'Display', None],
  [ 0xd4f4, 'HangulSyllablePyuls', [], 'Display', None],
  [ 0xd4f5, 'HangulSyllablePyult', [], 'Display', None],
  [ 0xd4f6, 'HangulSyllablePyulp', [], 'Display', None],
  [ 0xd4f7, 'HangulSyllablePyulh', [], 'Display', None],
  [ 0xd4f8, 'HangulSyllablePyum', [], 'Display', None],
  [ 0xd4f9, 'HangulSyllablePyub', [], 'Display', None],
  [ 0xd4fa, 'HangulSyllablePyubs', [], 'Display', None],
  [ 0xd4fb, 'HangulSyllablePyus', [], 'Display', None],
  [ 0xd4fc, 'HangulSyllablePyuss', [], 'Display', None],
  [ 0xd4fd, 'HangulSyllablePyung', [], 'Display', None],
  [ 0xd4fe, 'HangulSyllablePyuj', [], 'Display', None],
  [ 0xd4ff, 'HangulSyllablePyuc', [], 'Display', None],
  [ 0xd500, 'HangulSyllablePyuk', [], 'Display', None],
  [ 0xd501, 'HangulSyllablePyut', [], 'Display', None],
  [ 0xd502, 'HangulSyllablePyup', [], 'Display', None],
  [ 0xd503, 'HangulSyllablePyuh', [], 'Display', None],
  [ 0xd504, 'HangulSyllablePeu', [], 'Display', None],
  [ 0xd505, 'HangulSyllablePeug', [], 'Display', None],
  [ 0xd506, 'HangulSyllablePeugg', [], 'Display', None],
  [ 0xd507, 'HangulSyllablePeugs', [], 'Display', None],
  [ 0xd508, 'HangulSyllablePeun', [], 'Display', None],
  [ 0xd509, 'HangulSyllablePeunj', [], 'Display', None],
  [ 0xd50a, 'HangulSyllablePeunh', [], 'Display', None],
  [ 0xd50b, 'HangulSyllablePeud', [], 'Display', None],
  [ 0xd50c, 'HangulSyllablePeul', [], 'Display', None],
  [ 0xd50d, 'HangulSyllablePeulg', [], 'Display', None],
  [ 0xd50e, 'HangulSyllablePeulm', [], 'Display', None],
  [ 0xd50f, 'HangulSyllablePeulb', [], 'Display', None],
  [ 0xd510, 'HangulSyllablePeuls', [], 'Display', None],
  [ 0xd511, 'HangulSyllablePeult', [], 'Display', None],
  [ 0xd512, 'HangulSyllablePeulp', [], 'Display', None],
  [ 0xd513, 'HangulSyllablePeulh', [], 'Display', None],
  [ 0xd514, 'HangulSyllablePeum', [], 'Display', None],
  [ 0xd515, 'HangulSyllablePeub', [], 'Display', None],
  [ 0xd516, 'HangulSyllablePeubs', [], 'Display', None],
  [ 0xd517, 'HangulSyllablePeus', [], 'Display', None],
  [ 0xd518, 'HangulSyllablePeuss', [], 'Display', None],
  [ 0xd519, 'HangulSyllablePeung', [], 'Display', None],
  [ 0xd51a, 'HangulSyllablePeuj', [], 'Display', None],
  [ 0xd51b, 'HangulSyllablePeuc', [], 'Display', None],
  [ 0xd51c, 'HangulSyllablePeuk', [], 'Display', None],
  [ 0xd51d, 'HangulSyllablePeut', [], 'Display', None],
  [ 0xd51e, 'HangulSyllablePeup', [], 'Display', None],
  [ 0xd51f, 'HangulSyllablePeuh', [], 'Display', None],
  [ 0xd520, 'HangulSyllablePyi', [], 'Display', None],
  [ 0xd521, 'HangulSyllablePyig', [], 'Display', None],
  [ 0xd522, 'HangulSyllablePyigg', [], 'Display', None],
  [ 0xd523, 'HangulSyllablePyigs', [], 'Display', None],
  [ 0xd524, 'HangulSyllablePyin', [], 'Display', None],
  [ 0xd525, 'HangulSyllablePyinj', [], 'Display', None],
  [ 0xd526, 'HangulSyllablePyinh', [], 'Display', None],
  [ 0xd527, 'HangulSyllablePyid', [], 'Display', None],
  [ 0xd528, 'HangulSyllablePyil', [], 'Display', None],
  [ 0xd529, 'HangulSyllablePyilg', [], 'Display', None],
  [ 0xd52a, 'HangulSyllablePyilm', [], 'Display', None],
  [ 0xd52b, 'HangulSyllablePyilb', [], 'Display', None],
  [ 0xd52c, 'HangulSyllablePyils', [], 'Display', None],
  [ 0xd52d, 'HangulSyllablePyilt', [], 'Display', None],
  [ 0xd52e, 'HangulSyllablePyilp', [], 'Display', None],
  [ 0xd52f, 'HangulSyllablePyilh', [], 'Display', None],
  [ 0xd530, 'HangulSyllablePyim', [], 'Display', None],
  [ 0xd531, 'HangulSyllablePyib', [], 'Display', None],
  [ 0xd532, 'HangulSyllablePyibs', [], 'Display', None],
  [ 0xd533, 'HangulSyllablePyis', [], 'Display', None],
  [ 0xd534, 'HangulSyllablePyiss', [], 'Display', None],
  [ 0xd535, 'HangulSyllablePying', [], 'Display', None],
  [ 0xd536, 'HangulSyllablePyij', [], 'Display', None],
  [ 0xd537, 'HangulSyllablePyic', [], 'Display', None],
  [ 0xd538, 'HangulSyllablePyik', [], 'Display', None],
  [ 0xd539, 'HangulSyllablePyit', [], 'Display', None],
  [ 0xd53a, 'HangulSyllablePyip', [], 'Display', None],
  [ 0xd53b, 'HangulSyllablePyih', [], 'Display', None],
  [ 0xd53c, 'HangulSyllablePi', [], 'Display', None],
  [ 0xd53d, 'HangulSyllablePig', [], 'Display', None],
  [ 0xd53e, 'HangulSyllablePigg', [], 'Display', None],
  [ 0xd53f, 'HangulSyllablePigs', [], 'Display', None],
  [ 0xd540, 'HangulSyllablePin', [], 'Display', None],
  [ 0xd541, 'HangulSyllablePinj', [], 'Display', None],
  [ 0xd542, 'HangulSyllablePinh', [], 'Display', None],
  [ 0xd543, 'HangulSyllablePid', [], 'Display', None],
  [ 0xd544, 'HangulSyllablePil', [], 'Display', None],
  [ 0xd545, 'HangulSyllablePilg', [], 'Display', None],
  [ 0xd546, 'HangulSyllablePilm', [], 'Display', None],
  [ 0xd547, 'HangulSyllablePilb', [], 'Display', None],
  [ 0xd548, 'HangulSyllablePils', [], 'Display', None],
  [ 0xd549, 'HangulSyllablePilt', [], 'Display', None],
  [ 0xd54a, 'HangulSyllablePilp', [], 'Display', None],
  [ 0xd54b, 'HangulSyllablePilh', [], 'Display', None],
  [ 0xd54c, 'HangulSyllablePim', [], 'Display', None],
  [ 0xd54d, 'HangulSyllablePib', [], 'Display', None],
  [ 0xd54e, 'HangulSyllablePibs', [], 'Display', None],
  [ 0xd54f, 'HangulSyllablePis', [], 'Display', None],
  [ 0xd550, 'HangulSyllablePiss', [], 'Display', None],
  [ 0xd551, 'HangulSyllablePing', [], 'Display', None],
  [ 0xd552, 'HangulSyllablePij', [], 'Display', None],
  [ 0xd553, 'HangulSyllablePic', [], 'Display', None],
  [ 0xd554, 'HangulSyllablePik', [], 'Display', None],
  [ 0xd555, 'HangulSyllablePit', [], 'Display', None],
  [ 0xd556, 'HangulSyllablePip', [], 'Display', None],
  [ 0xd557, 'HangulSyllablePih', [], 'Display', None],
  [ 0xd558, 'HangulSyllableHa', [], 'Display', None],
  [ 0xd559, 'HangulSyllableHag', [], 'Display', None],
  [ 0xd55a, 'HangulSyllableHagg', [], 'Display', None],
  [ 0xd55b, 'HangulSyllableHags', [], 'Display', None],
  [ 0xd55c, 'HangulSyllableHan', [], 'Display', None],
  [ 0xd55d, 'HangulSyllableHanj', [], 'Display', None],
  [ 0xd55e, 'HangulSyllableHanh', [], 'Display', None],
  [ 0xd55f, 'HangulSyllableHad', [], 'Display', None],
  [ 0xd560, 'HangulSyllableHal', [], 'Display', None],
  [ 0xd561, 'HangulSyllableHalg', [], 'Display', None],
  [ 0xd562, 'HangulSyllableHalm', [], 'Display', None],
  [ 0xd563, 'HangulSyllableHalb', [], 'Display', None],
  [ 0xd564, 'HangulSyllableHals', [], 'Display', None],
  [ 0xd565, 'HangulSyllableHalt', [], 'Display', None],
  [ 0xd566, 'HangulSyllableHalp', [], 'Display', None],
  [ 0xd567, 'HangulSyllableHalh', [], 'Display', None],
  [ 0xd568, 'HangulSyllableHam', [], 'Display', None],
  [ 0xd569, 'HangulSyllableHab', [], 'Display', None],
  [ 0xd56a, 'HangulSyllableHabs', [], 'Display', None],
  [ 0xd56b, 'HangulSyllableHas', [], 'Display', None],
  [ 0xd56c, 'HangulSyllableHass', [], 'Display', None],
  [ 0xd56d, 'HangulSyllableHang', [], 'Display', None],
  [ 0xd56e, 'HangulSyllableHaj', [], 'Display', None],
  [ 0xd56f, 'HangulSyllableHac', [], 'Display', None],
  [ 0xd570, 'HangulSyllableHak', [], 'Display', None],
  [ 0xd571, 'HangulSyllableHat', [], 'Display', None],
  [ 0xd572, 'HangulSyllableHap', [], 'Display', None],
  [ 0xd573, 'HangulSyllableHah', [], 'Display', None],
  [ 0xd574, 'HangulSyllableHae', [], 'Display', None],
  [ 0xd575, 'HangulSyllableHaeg', [], 'Display', None],
  [ 0xd576, 'HangulSyllableHaegg', [], 'Display', None],
  [ 0xd577, 'HangulSyllableHaegs', [], 'Display', None],
  [ 0xd578, 'HangulSyllableHaen', [], 'Display', None],
  [ 0xd579, 'HangulSyllableHaenj', [], 'Display', None],
  [ 0xd57a, 'HangulSyllableHaenh', [], 'Display', None],
  [ 0xd57b, 'HangulSyllableHaed', [], 'Display', None],
  [ 0xd57c, 'HangulSyllableHael', [], 'Display', None],
  [ 0xd57d, 'HangulSyllableHaelg', [], 'Display', None],
  [ 0xd57e, 'HangulSyllableHaelm', [], 'Display', None],
  [ 0xd57f, 'HangulSyllableHaelb', [], 'Display', None],
  [ 0xd580, 'HangulSyllableHaels', [], 'Display', None],
  [ 0xd581, 'HangulSyllableHaelt', [], 'Display', None],
  [ 0xd582, 'HangulSyllableHaelp', [], 'Display', None],
  [ 0xd583, 'HangulSyllableHaelh', [], 'Display', None],
  [ 0xd584, 'HangulSyllableHaem', [], 'Display', None],
  [ 0xd585, 'HangulSyllableHaeb', [], 'Display', None],
  [ 0xd586, 'HangulSyllableHaebs', [], 'Display', None],
  [ 0xd587, 'HangulSyllableHaes', [], 'Display', None],
  [ 0xd588, 'HangulSyllableHaess', [], 'Display', None],
  [ 0xd589, 'HangulSyllableHaeng', [], 'Display', None],
  [ 0xd58a, 'HangulSyllableHaej', [], 'Display', None],
  [ 0xd58b, 'HangulSyllableHaec', [], 'Display', None],
  [ 0xd58c, 'HangulSyllableHaek', [], 'Display', None],
  [ 0xd58d, 'HangulSyllableHaet', [], 'Display', None],
  [ 0xd58e, 'HangulSyllableHaep', [], 'Display', None],
  [ 0xd58f, 'HangulSyllableHaeh', [], 'Display', None],
  [ 0xd590, 'HangulSyllableHya', [], 'Display', None],
  [ 0xd591, 'HangulSyllableHyag', [], 'Display', None],
  [ 0xd592, 'HangulSyllableHyagg', [], 'Display', None],
  [ 0xd593, 'HangulSyllableHyags', [], 'Display', None],
  [ 0xd594, 'HangulSyllableHyan', [], 'Display', None],
  [ 0xd595, 'HangulSyllableHyanj', [], 'Display', None],
  [ 0xd596, 'HangulSyllableHyanh', [], 'Display', None],
  [ 0xd597, 'HangulSyllableHyad', [], 'Display', None],
  [ 0xd598, 'HangulSyllableHyal', [], 'Display', None],
  [ 0xd599, 'HangulSyllableHyalg', [], 'Display', None],
  [ 0xd59a, 'HangulSyllableHyalm', [], 'Display', None],
  [ 0xd59b, 'HangulSyllableHyalb', [], 'Display', None],
  [ 0xd59c, 'HangulSyllableHyals', [], 'Display', None],
  [ 0xd59d, 'HangulSyllableHyalt', [], 'Display', None],
  [ 0xd59e, 'HangulSyllableHyalp', [], 'Display', None],
  [ 0xd59f, 'HangulSyllableHyalh', [], 'Display', None],
  [ 0xd5a0, 'HangulSyllableHyam', [], 'Display', None],
  [ 0xd5a1, 'HangulSyllableHyab', [], 'Display', None],
  [ 0xd5a2, 'HangulSyllableHyabs', [], 'Display', None],
  [ 0xd5a3, 'HangulSyllableHyas', [], 'Display', None],
  [ 0xd5a4, 'HangulSyllableHyass', [], 'Display', None],
  [ 0xd5a5, 'HangulSyllableHyang', [], 'Display', None],
  [ 0xd5a6, 'HangulSyllableHyaj', [], 'Display', None],
  [ 0xd5a7, 'HangulSyllableHyac', [], 'Display', None],
  [ 0xd5a8, 'HangulSyllableHyak', [], 'Display', None],
  [ 0xd5a9, 'HangulSyllableHyat', [], 'Display', None],
  [ 0xd5aa, 'HangulSyllableHyap', [], 'Display', None],
  [ 0xd5ab, 'HangulSyllableHyah', [], 'Display', None],
  [ 0xd5ac, 'HangulSyllableHyae', [], 'Display', None],
  [ 0xd5ad, 'HangulSyllableHyaeg', [], 'Display', None],
  [ 0xd5ae, 'HangulSyllableHyaegg', [], 'Display', None],
  [ 0xd5af, 'HangulSyllableHyaegs', [], 'Display', None],
  [ 0xd5b0, 'HangulSyllableHyaen', [], 'Display', None],
  [ 0xd5b1, 'HangulSyllableHyaenj', [], 'Display', None],
  [ 0xd5b2, 'HangulSyllableHyaenh', [], 'Display', None],
  [ 0xd5b3, 'HangulSyllableHyaed', [], 'Display', None],
  [ 0xd5b4, 'HangulSyllableHyael', [], 'Display', None],
  [ 0xd5b5, 'HangulSyllableHyaelg', [], 'Display', None],
  [ 0xd5b6, 'HangulSyllableHyaelm', [], 'Display', None],
  [ 0xd5b7, 'HangulSyllableHyaelb', [], 'Display', None],
  [ 0xd5b8, 'HangulSyllableHyaels', [], 'Display', None],
  [ 0xd5b9, 'HangulSyllableHyaelt', [], 'Display', None],
  [ 0xd5ba, 'HangulSyllableHyaelp', [], 'Display', None],
  [ 0xd5bb, 'HangulSyllableHyaelh', [], 'Display', None],
  [ 0xd5bc, 'HangulSyllableHyaem', [], 'Display', None],
  [ 0xd5bd, 'HangulSyllableHyaeb', [], 'Display', None],
  [ 0xd5be, 'HangulSyllableHyaebs', [], 'Display', None],
  [ 0xd5bf, 'HangulSyllableHyaes', [], 'Display', None],
  [ 0xd5c0, 'HangulSyllableHyaess', [], 'Display', None],
  [ 0xd5c1, 'HangulSyllableHyaeng', [], 'Display', None],
  [ 0xd5c2, 'HangulSyllableHyaej', [], 'Display', None],
  [ 0xd5c3, 'HangulSyllableHyaec', [], 'Display', None],
  [ 0xd5c4, 'HangulSyllableHyaek', [], 'Display', None],
  [ 0xd5c5, 'HangulSyllableHyaet', [], 'Display', None],
  [ 0xd5c6, 'HangulSyllableHyaep', [], 'Display', None],
  [ 0xd5c7, 'HangulSyllableHyaeh', [], 'Display', None],
  [ 0xd5c8, 'HangulSyllableHeo', [], 'Display', None],
  [ 0xd5c9, 'HangulSyllableHeog', [], 'Display', None],
  [ 0xd5ca, 'HangulSyllableHeogg', [], 'Display', None],
  [ 0xd5cb, 'HangulSyllableHeogs', [], 'Display', None],
  [ 0xd5cc, 'HangulSyllableHeon', [], 'Display', None],
  [ 0xd5cd, 'HangulSyllableHeonj', [], 'Display', None],
  [ 0xd5ce, 'HangulSyllableHeonh', [], 'Display', None],
  [ 0xd5cf, 'HangulSyllableHeod', [], 'Display', None],
  [ 0xd5d0, 'HangulSyllableHeol', [], 'Display', None],
  [ 0xd5d1, 'HangulSyllableHeolg', [], 'Display', None],
  [ 0xd5d2, 'HangulSyllableHeolm', [], 'Display', None],
  [ 0xd5d3, 'HangulSyllableHeolb', [], 'Display', None],
  [ 0xd5d4, 'HangulSyllableHeols', [], 'Display', None],
  [ 0xd5d5, 'HangulSyllableHeolt', [], 'Display', None],
  [ 0xd5d6, 'HangulSyllableHeolp', [], 'Display', None],
  [ 0xd5d7, 'HangulSyllableHeolh', [], 'Display', None],
  [ 0xd5d8, 'HangulSyllableHeom', [], 'Display', None],
  [ 0xd5d9, 'HangulSyllableHeob', [], 'Display', None],
  [ 0xd5da, 'HangulSyllableHeobs', [], 'Display', None],
  [ 0xd5db, 'HangulSyllableHeos', [], 'Display', None],
  [ 0xd5dc, 'HangulSyllableHeoss', [], 'Display', None],
  [ 0xd5dd, 'HangulSyllableHeong', [], 'Display', None],
  [ 0xd5de, 'HangulSyllableHeoj', [], 'Display', None],
  [ 0xd5df, 'HangulSyllableHeoc', [], 'Display', None],
  [ 0xd5e0, 'HangulSyllableHeok', [], 'Display', None],
  [ 0xd5e1, 'HangulSyllableHeot', [], 'Display', None],
  [ 0xd5e2, 'HangulSyllableHeop', [], 'Display', None],
  [ 0xd5e3, 'HangulSyllableHeoh', [], 'Display', None],
  [ 0xd5e4, 'HangulSyllableHe', [], 'Display', None],
  [ 0xd5e5, 'HangulSyllableHeg', [], 'Display', None],
  [ 0xd5e6, 'HangulSyllableHegg', [], 'Display', None],
  [ 0xd5e7, 'HangulSyllableHegs', [], 'Display', None],
  [ 0xd5e8, 'HangulSyllableHen', [], 'Display', None],
  [ 0xd5e9, 'HangulSyllableHenj', [], 'Display', None],
  [ 0xd5ea, 'HangulSyllableHenh', [], 'Display', None],
  [ 0xd5eb, 'HangulSyllableHed', [], 'Display', None],
  [ 0xd5ec, 'HangulSyllableHel', [], 'Display', None],
  [ 0xd5ed, 'HangulSyllableHelg', [], 'Display', None],
  [ 0xd5ee, 'HangulSyllableHelm', [], 'Display', None],
  [ 0xd5ef, 'HangulSyllableHelb', [], 'Display', None],
  [ 0xd5f0, 'HangulSyllableHels', [], 'Display', None],
  [ 0xd5f1, 'HangulSyllableHelt', [], 'Display', None],
  [ 0xd5f2, 'HangulSyllableHelp', [], 'Display', None],
  [ 0xd5f3, 'HangulSyllableHelh', [], 'Display', None],
  [ 0xd5f4, 'HangulSyllableHem', [], 'Display', None],
  [ 0xd5f5, 'HangulSyllableHeb', [], 'Display', None],
  [ 0xd5f6, 'HangulSyllableHebs', [], 'Display', None],
  [ 0xd5f7, 'HangulSyllableHes', [], 'Display', None],
  [ 0xd5f8, 'HangulSyllableHess', [], 'Display', None],
  [ 0xd5f9, 'HangulSyllableHeng', [], 'Display', None],
  [ 0xd5fa, 'HangulSyllableHej', [], 'Display', None],
  [ 0xd5fb, 'HangulSyllableHec', [], 'Display', None],
  [ 0xd5fc, 'HangulSyllableHek', [], 'Display', None],
  [ 0xd5fd, 'HangulSyllableHet', [], 'Display', None],
  [ 0xd5fe, 'HangulSyllableHep', [], 'Display', None],
  [ 0xd5ff, 'HangulSyllableHeh', [], 'Display', None],
  [ 0xd600, 'HangulSyllableHyeo', [], 'Display', None],
  [ 0xd601, 'HangulSyllableHyeog', [], 'Display', None],
  [ 0xd602, 'HangulSyllableHyeogg', [], 'Display', None],
  [ 0xd603, 'HangulSyllableHyeogs', [], 'Display', None],
  [ 0xd604, 'HangulSyllableHyeon', [], 'Display', None],
  [ 0xd605, 'HangulSyllableHyeonj', [], 'Display', None],
  [ 0xd606, 'HangulSyllableHyeonh', [], 'Display', None],
  [ 0xd607, 'HangulSyllableHyeod', [], 'Display', None],
  [ 0xd608, 'HangulSyllableHyeol', [], 'Display', None],
  [ 0xd609, 'HangulSyllableHyeolg', [], 'Display', None],
  [ 0xd60a, 'HangulSyllableHyeolm', [], 'Display', None],
  [ 0xd60b, 'HangulSyllableHyeolb', [], 'Display', None],
  [ 0xd60c, 'HangulSyllableHyeols', [], 'Display', None],
  [ 0xd60d, 'HangulSyllableHyeolt', [], 'Display', None],
  [ 0xd60e, 'HangulSyllableHyeolp', [], 'Display', None],
  [ 0xd60f, 'HangulSyllableHyeolh', [], 'Display', None],
  [ 0xd610, 'HangulSyllableHyeom', [], 'Display', None],
  [ 0xd611, 'HangulSyllableHyeob', [], 'Display', None],
  [ 0xd612, 'HangulSyllableHyeobs', [], 'Display', None],
  [ 0xd613, 'HangulSyllableHyeos', [], 'Display', None],
  [ 0xd614, 'HangulSyllableHyeoss', [], 'Display', None],
  [ 0xd615, 'HangulSyllableHyeong', [], 'Display', None],
  [ 0xd616, 'HangulSyllableHyeoj', [], 'Display', None],
  [ 0xd617, 'HangulSyllableHyeoc', [], 'Display', None],
  [ 0xd618, 'HangulSyllableHyeok', [], 'Display', None],
  [ 0xd619, 'HangulSyllableHyeot', [], 'Display', None],
  [ 0xd61a, 'HangulSyllableHyeop', [], 'Display', None],
  [ 0xd61b, 'HangulSyllableHyeoh', [], 'Display', None],
  [ 0xd61c, 'HangulSyllableHye', [], 'Display', None],
  [ 0xd61d, 'HangulSyllableHyeg', [], 'Display', None],
  [ 0xd61e, 'HangulSyllableHyegg', [], 'Display', None],
  [ 0xd61f, 'HangulSyllableHyegs', [], 'Display', None],
  [ 0xd620, 'HangulSyllableHyen', [], 'Display', None],
  [ 0xd621, 'HangulSyllableHyenj', [], 'Display', None],
  [ 0xd622, 'HangulSyllableHyenh', [], 'Display', None],
  [ 0xd623, 'HangulSyllableHyed', [], 'Display', None],
  [ 0xd624, 'HangulSyllableHyel', [], 'Display', None],
  [ 0xd625, 'HangulSyllableHyelg', [], 'Display', None],
  [ 0xd626, 'HangulSyllableHyelm', [], 'Display', None],
  [ 0xd627, 'HangulSyllableHyelb', [], 'Display', None],
  [ 0xd628, 'HangulSyllableHyels', [], 'Display', None],
  [ 0xd629, 'HangulSyllableHyelt', [], 'Display', None],
  [ 0xd62a, 'HangulSyllableHyelp', [], 'Display', None],
  [ 0xd62b, 'HangulSyllableHyelh', [], 'Display', None],
  [ 0xd62c, 'HangulSyllableHyem', [], 'Display', None],
  [ 0xd62d, 'HangulSyllableHyeb', [], 'Display', None],
  [ 0xd62e, 'HangulSyllableHyebs', [], 'Display', None],
  [ 0xd62f, 'HangulSyllableHyes', [], 'Display', None],
  [ 0xd630, 'HangulSyllableHyess', [], 'Display', None],
  [ 0xd631, 'HangulSyllableHyeng', [], 'Display', None],
  [ 0xd632, 'HangulSyllableHyej', [], 'Display', None],
  [ 0xd633, 'HangulSyllableHyec', [], 'Display', None],
  [ 0xd634, 'HangulSyllableHyek', [], 'Display', None],
  [ 0xd635, 'HangulSyllableHyet', [], 'Display', None],
  [ 0xd636, 'HangulSyllableHyep', [], 'Display', None],
  [ 0xd637, 'HangulSyllableHyeh', [], 'Display', None],
  [ 0xd638, 'HangulSyllableHo', [], 'Display', None],
  [ 0xd639, 'HangulSyllableHog', [], 'Display', None],
  [ 0xd63a, 'HangulSyllableHogg', [], 'Display', None],
  [ 0xd63b, 'HangulSyllableHogs', [], 'Display', None],
  [ 0xd63c, 'HangulSyllableHon', [], 'Display', None],
  [ 0xd63d, 'HangulSyllableHonj', [], 'Display', None],
  [ 0xd63e, 'HangulSyllableHonh', [], 'Display', None],
  [ 0xd63f, 'HangulSyllableHod', [], 'Display', None],
  [ 0xd640, 'HangulSyllableHol', [], 'Display', None],
  [ 0xd641, 'HangulSyllableHolg', [], 'Display', None],
  [ 0xd642, 'HangulSyllableHolm', [], 'Display', None],
  [ 0xd643, 'HangulSyllableHolb', [], 'Display', None],
  [ 0xd644, 'HangulSyllableHols', [], 'Display', None],
  [ 0xd645, 'HangulSyllableHolt', [], 'Display', None],
  [ 0xd646, 'HangulSyllableHolp', [], 'Display', None],
  [ 0xd647, 'HangulSyllableHolh', [], 'Display', None],
  [ 0xd648, 'HangulSyllableHom', [], 'Display', None],
  [ 0xd649, 'HangulSyllableHob', [], 'Display', None],
  [ 0xd64a, 'HangulSyllableHobs', [], 'Display', None],
  [ 0xd64b, 'HangulSyllableHos', [], 'Display', None],
  [ 0xd64c, 'HangulSyllableHoss', [], 'Display', None],
  [ 0xd64d, 'HangulSyllableHong', [], 'Display', None],
  [ 0xd64e, 'HangulSyllableHoj', [], 'Display', None],
  [ 0xd64f, 'HangulSyllableHoc', [], 'Display', None],
  [ 0xd650, 'HangulSyllableHok', [], 'Display', None],
  [ 0xd651, 'HangulSyllableHot', [], 'Display', None],
  [ 0xd652, 'HangulSyllableHop', [], 'Display', None],
  [ 0xd653, 'HangulSyllableHoh', [], 'Display', None],
  [ 0xd654, 'HangulSyllableHwa', [], 'Display', None],
  [ 0xd655, 'HangulSyllableHwag', [], 'Display', None],
  [ 0xd656, 'HangulSyllableHwagg', [], 'Display', None],
  [ 0xd657, 'HangulSyllableHwags', [], 'Display', None],
  [ 0xd658, 'HangulSyllableHwan', [], 'Display', None],
  [ 0xd659, 'HangulSyllableHwanj', [], 'Display', None],
  [ 0xd65a, 'HangulSyllableHwanh', [], 'Display', None],
  [ 0xd65b, 'HangulSyllableHwad', [], 'Display', None],
  [ 0xd65c, 'HangulSyllableHwal', [], 'Display', None],
  [ 0xd65d, 'HangulSyllableHwalg', [], 'Display', None],
  [ 0xd65e, 'HangulSyllableHwalm', [], 'Display', None],
  [ 0xd65f, 'HangulSyllableHwalb', [], 'Display', None],
  [ 0xd660, 'HangulSyllableHwals', [], 'Display', None],
  [ 0xd661, 'HangulSyllableHwalt', [], 'Display', None],
  [ 0xd662, 'HangulSyllableHwalp', [], 'Display', None],
  [ 0xd663, 'HangulSyllableHwalh', [], 'Display', None],
  [ 0xd664, 'HangulSyllableHwam', [], 'Display', None],
  [ 0xd665, 'HangulSyllableHwab', [], 'Display', None],
  [ 0xd666, 'HangulSyllableHwabs', [], 'Display', None],
  [ 0xd667, 'HangulSyllableHwas', [], 'Display', None],
  [ 0xd668, 'HangulSyllableHwass', [], 'Display', None],
  [ 0xd669, 'HangulSyllableHwang', [], 'Display', None],
  [ 0xd66a, 'HangulSyllableHwaj', [], 'Display', None],
  [ 0xd66b, 'HangulSyllableHwac', [], 'Display', None],
  [ 0xd66c, 'HangulSyllableHwak', [], 'Display', None],
  [ 0xd66d, 'HangulSyllableHwat', [], 'Display', None],
  [ 0xd66e, 'HangulSyllableHwap', [], 'Display', None],
  [ 0xd66f, 'HangulSyllableHwah', [], 'Display', None],
  [ 0xd670, 'HangulSyllableHwae', [], 'Display', None],
  [ 0xd671, 'HangulSyllableHwaeg', [], 'Display', None],
  [ 0xd672, 'HangulSyllableHwaegg', [], 'Display', None],
  [ 0xd673, 'HangulSyllableHwaegs', [], 'Display', None],
  [ 0xd674, 'HangulSyllableHwaen', [], 'Display', None],
  [ 0xd675, 'HangulSyllableHwaenj', [], 'Display', None],
  [ 0xd676, 'HangulSyllableHwaenh', [], 'Display', None],
  [ 0xd677, 'HangulSyllableHwaed', [], 'Display', None],
  [ 0xd678, 'HangulSyllableHwael', [], 'Display', None],
  [ 0xd679, 'HangulSyllableHwaelg', [], 'Display', None],
  [ 0xd67a, 'HangulSyllableHwaelm', [], 'Display', None],
  [ 0xd67b, 'HangulSyllableHwaelb', [], 'Display', None],
  [ 0xd67c, 'HangulSyllableHwaels', [], 'Display', None],
  [ 0xd67d, 'HangulSyllableHwaelt', [], 'Display', None],
  [ 0xd67e, 'HangulSyllableHwaelp', [], 'Display', None],
  [ 0xd67f, 'HangulSyllableHwaelh', [], 'Display', None],
  [ 0xd680, 'HangulSyllableHwaem', [], 'Display', None],
  [ 0xd681, 'HangulSyllableHwaeb', [], 'Display', None],
  [ 0xd682, 'HangulSyllableHwaebs', [], 'Display', None],
  [ 0xd683, 'HangulSyllableHwaes', [], 'Display', None],
  [ 0xd684, 'HangulSyllableHwaess', [], 'Display', None],
  [ 0xd685, 'HangulSyllableHwaeng', [], 'Display', None],
  [ 0xd686, 'HangulSyllableHwaej', [], 'Display', None],
  [ 0xd687, 'HangulSyllableHwaec', [], 'Display', None],
  [ 0xd688, 'HangulSyllableHwaek', [], 'Display', None],
  [ 0xd689, 'HangulSyllableHwaet', [], 'Display', None],
  [ 0xd68a, 'HangulSyllableHwaep', [], 'Display', None],
  [ 0xd68b, 'HangulSyllableHwaeh', [], 'Display', None],
  [ 0xd68c, 'HangulSyllableHoe', [], 'Display', None],
  [ 0xd68d, 'HangulSyllableHoeg', [], 'Display', None],
  [ 0xd68e, 'HangulSyllableHoegg', [], 'Display', None],
  [ 0xd68f, 'HangulSyllableHoegs', [], 'Display', None],
  [ 0xd690, 'HangulSyllableHoen', [], 'Display', None],
  [ 0xd691, 'HangulSyllableHoenj', [], 'Display', None],
  [ 0xd692, 'HangulSyllableHoenh', [], 'Display', None],
  [ 0xd693, 'HangulSyllableHoed', [], 'Display', None],
  [ 0xd694, 'HangulSyllableHoel', [], 'Display', None],
  [ 0xd695, 'HangulSyllableHoelg', [], 'Display', None],
  [ 0xd696, 'HangulSyllableHoelm', [], 'Display', None],
  [ 0xd697, 'HangulSyllableHoelb', [], 'Display', None],
  [ 0xd698, 'HangulSyllableHoels', [], 'Display', None],
  [ 0xd699, 'HangulSyllableHoelt', [], 'Display', None],
  [ 0xd69a, 'HangulSyllableHoelp', [], 'Display', None],
  [ 0xd69b, 'HangulSyllableHoelh', [], 'Display', None],
  [ 0xd69c, 'HangulSyllableHoem', [], 'Display', None],
  [ 0xd69d, 'HangulSyllableHoeb', [], 'Display', None],
  [ 0xd69e, 'HangulSyllableHoebs', [], 'Display', None],
  [ 0xd69f, 'HangulSyllableHoes', [], 'Display', None],
  [ 0xd6a0, 'HangulSyllableHoess', [], 'Display', None],
  [ 0xd6a1, 'HangulSyllableHoeng', [], 'Display', None],
  [ 0xd6a2, 'HangulSyllableHoej', [], 'Display', None],
  [ 0xd6a3, 'HangulSyllableHoec', [], 'Display', None],
  [ 0xd6a4, 'HangulSyllableHoek', [], 'Display', None],
  [ 0xd6a5, 'HangulSyllableHoet', [], 'Display', None],
  [ 0xd6a6, 'HangulSyllableHoep', [], 'Display', None],
  [ 0xd6a7, 'HangulSyllableHoeh', [], 'Display', None],
  [ 0xd6a8, 'HangulSyllableHyo', [], 'Display', None],
  [ 0xd6a9, 'HangulSyllableHyog', [], 'Display', None],
  [ 0xd6aa, 'HangulSyllableHyogg', [], 'Display', None],
  [ 0xd6ab, 'HangulSyllableHyogs', [], 'Display', None],
  [ 0xd6ac, 'HangulSyllableHyon', [], 'Display', None],
  [ 0xd6ad, 'HangulSyllableHyonj', [], 'Display', None],
  [ 0xd6ae, 'HangulSyllableHyonh', [], 'Display', None],
  [ 0xd6af, 'HangulSyllableHyod', [], 'Display', None],
  [ 0xd6b0, 'HangulSyllableHyol', [], 'Display', None],
  [ 0xd6b1, 'HangulSyllableHyolg', [], 'Display', None],
  [ 0xd6b2, 'HangulSyllableHyolm', [], 'Display', None],
  [ 0xd6b3, 'HangulSyllableHyolb', [], 'Display', None],
  [ 0xd6b4, 'HangulSyllableHyols', [], 'Display', None],
  [ 0xd6b5, 'HangulSyllableHyolt', [], 'Display', None],
  [ 0xd6b6, 'HangulSyllableHyolp', [], 'Display', None],
  [ 0xd6b7, 'HangulSyllableHyolh', [], 'Display', None],
  [ 0xd6b8, 'HangulSyllableHyom', [], 'Display', None],
  [ 0xd6b9, 'HangulSyllableHyob', [], 'Display', None],
  [ 0xd6ba, 'HangulSyllableHyobs', [], 'Display', None],
  [ 0xd6bb, 'HangulSyllableHyos', [], 'Display', None],
  [ 0xd6bc, 'HangulSyllableHyoss', [], 'Display', None],
  [ 0xd6bd, 'HangulSyllableHyong', [], 'Display', None],
  [ 0xd6be, 'HangulSyllableHyoj', [], 'Display', None],
  [ 0xd6bf, 'HangulSyllableHyoc', [], 'Display', None],
  [ 0xd6c0, 'HangulSyllableHyok', [], 'Display', None],
  [ 0xd6c1, 'HangulSyllableHyot', [], 'Display', None],
  [ 0xd6c2, 'HangulSyllableHyop', [], 'Display', None],
  [ 0xd6c3, 'HangulSyllableHyoh', [], 'Display', None],
  [ 0xd6c4, 'HangulSyllableHu', [], 'Display', None],
  [ 0xd6c5, 'HangulSyllableHug', [], 'Display', None],
  [ 0xd6c6, 'HangulSyllableHugg', [], 'Display', None],
  [ 0xd6c7, 'HangulSyllableHugs', [], 'Display', None],
  [ 0xd6c8, 'HangulSyllableHun', [], 'Display', None],
  [ 0xd6c9, 'HangulSyllableHunj', [], 'Display', None],
  [ 0xd6ca, 'HangulSyllableHunh', [], 'Display', None],
  [ 0xd6cb, 'HangulSyllableHud', [], 'Display', None],
  [ 0xd6cc, 'HangulSyllableHul', [], 'Display', None],
  [ 0xd6cd, 'HangulSyllableHulg', [], 'Display', None],
  [ 0xd6ce, 'HangulSyllableHulm', [], 'Display', None],
  [ 0xd6cf, 'HangulSyllableHulb', [], 'Display', None],
  [ 0xd6d0, 'HangulSyllableHuls', [], 'Display', None],
  [ 0xd6d1, 'HangulSyllableHult', [], 'Display', None],
  [ 0xd6d2, 'HangulSyllableHulp', [], 'Display', None],
  [ 0xd6d3, 'HangulSyllableHulh', [], 'Display', None],
  [ 0xd6d4, 'HangulSyllableHum', [], 'Display', None],
  [ 0xd6d5, 'HangulSyllableHub', [], 'Display', None],
  [ 0xd6d6, 'HangulSyllableHubs', [], 'Display', None],
  [ 0xd6d7, 'HangulSyllableHus', [], 'Display', None],
  [ 0xd6d8, 'HangulSyllableHuss', [], 'Display', None],
  [ 0xd6d9, 'HangulSyllableHung', [], 'Display', None],
  [ 0xd6da, 'HangulSyllableHuj', [], 'Display', None],
  [ 0xd6db, 'HangulSyllableHuc', [], 'Display', None],
  [ 0xd6dc, 'HangulSyllableHuk', [], 'Display', None],
  [ 0xd6dd, 'HangulSyllableHut', [], 'Display', None],
  [ 0xd6de, 'HangulSyllableHup', [], 'Display', None],
  [ 0xd6df, 'HangulSyllableHuh', [], 'Display', None],
  [ 0xd6e0, 'HangulSyllableHweo', [], 'Display', None],
  [ 0xd6e1, 'HangulSyllableHweog', [], 'Display', None],
  [ 0xd6e2, 'HangulSyllableHweogg', [], 'Display', None],
  [ 0xd6e3, 'HangulSyllableHweogs', [], 'Display', None],
  [ 0xd6e4, 'HangulSyllableHweon', [], 'Display', None],
  [ 0xd6e5, 'HangulSyllableHweonj', [], 'Display', None],
  [ 0xd6e6, 'HangulSyllableHweonh', [], 'Display', None],
  [ 0xd6e7, 'HangulSyllableHweod', [], 'Display', None],
  [ 0xd6e8, 'HangulSyllableHweol', [], 'Display', None],
  [ 0xd6e9, 'HangulSyllableHweolg', [], 'Display', None],
  [ 0xd6ea, 'HangulSyllableHweolm', [], 'Display', None],
  [ 0xd6eb, 'HangulSyllableHweolb', [], 'Display', None],
  [ 0xd6ec, 'HangulSyllableHweols', [], 'Display', None],
  [ 0xd6ed, 'HangulSyllableHweolt', [], 'Display', None],
  [ 0xd6ee, 'HangulSyllableHweolp', [], 'Display', None],
  [ 0xd6ef, 'HangulSyllableHweolh', [], 'Display', None],
  [ 0xd6f0, 'HangulSyllableHweom', [], 'Display', None],
  [ 0xd6f1, 'HangulSyllableHweob', [], 'Display', None],
  [ 0xd6f2, 'HangulSyllableHweobs', [], 'Display', None],
  [ 0xd6f3, 'HangulSyllableHweos', [], 'Display', None],
  [ 0xd6f4, 'HangulSyllableHweoss', [], 'Display', None],
  [ 0xd6f5, 'HangulSyllableHweong', [], 'Display', None],
  [ 0xd6f6, 'HangulSyllableHweoj', [], 'Display', None],
  [ 0xd6f7, 'HangulSyllableHweoc', [], 'Display', None],
  [ 0xd6f8, 'HangulSyllableHweok', [], 'Display', None],
  [ 0xd6f9, 'HangulSyllableHweot', [], 'Display', None],
  [ 0xd6fa, 'HangulSyllableHweop', [], 'Display', None],
  [ 0xd6fb, 'HangulSyllableHweoh', [], 'Display', None],
  [ 0xd6fc, 'HangulSyllableHwe', [], 'Display', None],
  [ 0xd6fd, 'HangulSyllableHweg', [], 'Display', None],
  [ 0xd6fe, 'HangulSyllableHwegg', [], 'Display', None],
  [ 0xd6ff, 'HangulSyllableHwegs', [], 'Display', None],
  [ 0xd700, 'HangulSyllableHwen', [], 'Display', None],
  [ 0xd701, 'HangulSyllableHwenj', [], 'Display', None],
  [ 0xd702, 'HangulSyllableHwenh', [], 'Display', None],
  [ 0xd703, 'HangulSyllableHwed', [], 'Display', None],
  [ 0xd704, 'HangulSyllableHwel', [], 'Display', None],
  [ 0xd705, 'HangulSyllableHwelg', [], 'Display', None],
  [ 0xd706, 'HangulSyllableHwelm', [], 'Display', None],
  [ 0xd707, 'HangulSyllableHwelb', [], 'Display', None],
  [ 0xd708, 'HangulSyllableHwels', [], 'Display', None],
  [ 0xd709, 'HangulSyllableHwelt', [], 'Display', None],
  [ 0xd70a, 'HangulSyllableHwelp', [], 'Display', None],
  [ 0xd70b, 'HangulSyllableHwelh', [], 'Display', None],
  [ 0xd70c, 'HangulSyllableHwem', [], 'Display', None],
  [ 0xd70d, 'HangulSyllableHweb', [], 'Display', None],
  [ 0xd70e, 'HangulSyllableHwebs', [], 'Display', None],
  [ 0xd70f, 'HangulSyllableHwes', [], 'Display', None],
  [ 0xd710, 'HangulSyllableHwess', [], 'Display', None],
  [ 0xd711, 'HangulSyllableHweng', [], 'Display', None],
  [ 0xd712, 'HangulSyllableHwej', [], 'Display', None],
  [ 0xd713, 'HangulSyllableHwec', [], 'Display', None],
  [ 0xd714, 'HangulSyllableHwek', [], 'Display', None],
  [ 0xd715, 'HangulSyllableHwet', [], 'Display', None],
  [ 0xd716, 'HangulSyllableHwep', [], 'Display', None],
  [ 0xd717, 'HangulSyllableHweh', [], 'Display', None],
  [ 0xd718, 'HangulSyllableHwi', [], 'Display', None],
  [ 0xd719, 'HangulSyllableHwig', [], 'Display', None],
  [ 0xd71a, 'HangulSyllableHwigg', [], 'Display', None],
  [ 0xd71b, 'HangulSyllableHwigs', [], 'Display', None],
  [ 0xd71c, 'HangulSyllableHwin', [], 'Display', None],
  [ 0xd71d, 'HangulSyllableHwinj', [], 'Display', None],
  [ 0xd71e, 'HangulSyllableHwinh', [], 'Display', None],
  [ 0xd71f, 'HangulSyllableHwid', [], 'Display', None],
  [ 0xd720, 'HangulSyllableHwil', [], 'Display', None],
  [ 0xd721, 'HangulSyllableHwilg', [], 'Display', None],
  [ 0xd722, 'HangulSyllableHwilm', [], 'Display', None],
  [ 0xd723, 'HangulSyllableHwilb', [], 'Display', None],
  [ 0xd724, 'HangulSyllableHwils', [], 'Display', None],
  [ 0xd725, 'HangulSyllableHwilt', [], 'Display', None],
  [ 0xd726, 'HangulSyllableHwilp', [], 'Display', None],
  [ 0xd727, 'HangulSyllableHwilh', [], 'Display', None],
  [ 0xd728, 'HangulSyllableHwim', [], 'Display', None],
  [ 0xd729, 'HangulSyllableHwib', [], 'Display', None],
  [ 0xd72a, 'HangulSyllableHwibs', [], 'Display', None],
  [ 0xd72b, 'HangulSyllableHwis', [], 'Display', None],
  [ 0xd72c, 'HangulSyllableHwiss', [], 'Display', None],
  [ 0xd72d, 'HangulSyllableHwing', [], 'Display', None],
  [ 0xd72e, 'HangulSyllableHwij', [], 'Display', None],
  [ 0xd72f, 'HangulSyllableHwic', [], 'Display', None],
  [ 0xd730, 'HangulSyllableHwik', [], 'Display', None],
  [ 0xd731, 'HangulSyllableHwit', [], 'Display', None],
  [ 0xd732, 'HangulSyllableHwip', [], 'Display', None],
  [ 0xd733, 'HangulSyllableHwih', [], 'Display', None],
  [ 0xd734, 'HangulSyllableHyu', [], 'Display', None],
  [ 0xd735, 'HangulSyllableHyug', [], 'Display', None],
  [ 0xd736, 'HangulSyllableHyugg', [], 'Display', None],
  [ 0xd737, 'HangulSyllableHyugs', [], 'Display', None],
  [ 0xd738, 'HangulSyllableHyun', [], 'Display', None],
  [ 0xd739, 'HangulSyllableHyunj', [], 'Display', None],
  [ 0xd73a, 'HangulSyllableHyunh', [], 'Display', None],
  [ 0xd73b, 'HangulSyllableHyud', [], 'Display', None],
  [ 0xd73c, 'HangulSyllableHyul', [], 'Display', None],
  [ 0xd73d, 'HangulSyllableHyulg', [], 'Display', None],
  [ 0xd73e, 'HangulSyllableHyulm', [], 'Display', None],
  [ 0xd73f, 'HangulSyllableHyulb', [], 'Display', None],
  [ 0xd740, 'HangulSyllableHyuls', [], 'Display', None],
  [ 0xd741, 'HangulSyllableHyult', [], 'Display', None],
  [ 0xd742, 'HangulSyllableHyulp', [], 'Display', None],
  [ 0xd743, 'HangulSyllableHyulh', [], 'Display', None],
  [ 0xd744, 'HangulSyllableHyum', [], 'Display', None],
  [ 0xd745, 'HangulSyllableHyub', [], 'Display', None],
  [ 0xd746, 'HangulSyllableHyubs', [], 'Display', None],
  [ 0xd747, 'HangulSyllableHyus', [], 'Display', None],
  [ 0xd748, 'HangulSyllableHyuss', [], 'Display', None],
  [ 0xd749, 'HangulSyllableHyung', [], 'Display', None],
  [ 0xd74a, 'HangulSyllableHyuj', [], 'Display', None],
  [ 0xd74b, 'HangulSyllableHyuc', [], 'Display', None],
  [ 0xd74c, 'HangulSyllableHyuk', [], 'Display', None],
  [ 0xd74d, 'HangulSyllableHyut', [], 'Display', None],
  [ 0xd74e, 'HangulSyllableHyup', [], 'Display', None],
  [ 0xd74f, 'HangulSyllableHyuh', [], 'Display', None],
  [ 0xd750, 'HangulSyllableHeu', [], 'Display', None],
  [ 0xd751, 'HangulSyllableHeug', [], 'Display', None],
  [ 0xd752, 'HangulSyllableHeugg', [], 'Display', None],
  [ 0xd753, 'HangulSyllableHeugs', [], 'Display', None],
  [ 0xd754, 'HangulSyllableHeun', [], 'Display', None],
  [ 0xd755, 'HangulSyllableHeunj', [], 'Display', None],
  [ 0xd756, 'HangulSyllableHeunh', [], 'Display', None],
  [ 0xd757, 'HangulSyllableHeud', [], 'Display', None],
  [ 0xd758, 'HangulSyllableHeul', [], 'Display', None],
  [ 0xd759, 'HangulSyllableHeulg', [], 'Display', None],
  [ 0xd75a, 'HangulSyllableHeulm', [], 'Display', None],
  [ 0xd75b, 'HangulSyllableHeulb', [], 'Display', None],
  [ 0xd75c, 'HangulSyllableHeuls', [], 'Display', None],
  [ 0xd75d, 'HangulSyllableHeult', [], 'Display', None],
  [ 0xd75e, 'HangulSyllableHeulp', [], 'Display', None],
  [ 0xd75f, 'HangulSyllableHeulh', [], 'Display', None],
  [ 0xd760, 'HangulSyllableHeum', [], 'Display', None],
  [ 0xd761, 'HangulSyllableHeub', [], 'Display', None],
  [ 0xd762, 'HangulSyllableHeubs', [], 'Display', None],
  [ 0xd763, 'HangulSyllableHeus', [], 'Display', None],
  [ 0xd764, 'HangulSyllableHeuss', [], 'Display', None],
  [ 0xd765, 'HangulSyllableHeung', [], 'Display', None],
  [ 0xd766, 'HangulSyllableHeuj', [], 'Display', None],
  [ 0xd767, 'HangulSyllableHeuc', [], 'Display', None],
  [ 0xd768, 'HangulSyllableHeuk', [], 'Display', None],
  [ 0xd769, 'HangulSyllableHeut', [], 'Display', None],
  [ 0xd76a, 'HangulSyllableHeup', [], 'Display', None],
  [ 0xd76b, 'HangulSyllableHeuh', [], 'Display', None],
  [ 0xd76c, 'HangulSyllableHyi', [], 'Display', None],
  [ 0xd76d, 'HangulSyllableHyig', [], 'Display', None],
  [ 0xd76e, 'HangulSyllableHyigg', [], 'Display', None],
  [ 0xd76f, 'HangulSyllableHyigs', [], 'Display', None],
  [ 0xd770, 'HangulSyllableHyin', [], 'Display', None],
  [ 0xd771, 'HangulSyllableHyinj', [], 'Display', None],
  [ 0xd772, 'HangulSyllableHyinh', [], 'Display', None],
  [ 0xd773, 'HangulSyllableHyid', [], 'Display', None],
  [ 0xd774, 'HangulSyllableHyil', [], 'Display', None],
  [ 0xd775, 'HangulSyllableHyilg', [], 'Display', None],
  [ 0xd776, 'HangulSyllableHyilm', [], 'Display', None],
  [ 0xd777, 'HangulSyllableHyilb', [], 'Display', None],
  [ 0xd778, 'HangulSyllableHyils', [], 'Display', None],
  [ 0xd779, 'HangulSyllableHyilt', [], 'Display', None],
  [ 0xd77a, 'HangulSyllableHyilp', [], 'Display', None],
  [ 0xd77b, 'HangulSyllableHyilh', [], 'Display', None],
  [ 0xd77c, 'HangulSyllableHyim', [], 'Display', None],
  [ 0xd77d, 'HangulSyllableHyib', [], 'Display', None],
  [ 0xd77e, 'HangulSyllableHyibs', [], 'Display', None],
  [ 0xd77f, 'HangulSyllableHyis', [], 'Display', None],
  [ 0xd780, 'HangulSyllableHyiss', [], 'Display', None],
  [ 0xd781, 'HangulSyllableHying', [], 'Display', None],
  [ 0xd782, 'HangulSyllableHyij', [], 'Display', None],
  [ 0xd783, 'HangulSyllableHyic', [], 'Display', None],
  [ 0xd784, 'HangulSyllableHyik', [], 'Display', None],
  [ 0xd785, 'HangulSyllableHyit', [], 'Display', None],
  [ 0xd786, 'HangulSyllableHyip', [], 'Display', None],
  [ 0xd787, 'HangulSyllableHyih', [], 'Display', None],
  [ 0xd788, 'HangulSyllableHi', [], 'Display', None],
  [ 0xd789, 'HangulSyllableHig', [], 'Display', None],
  [ 0xd78a, 'HangulSyllableHigg', [], 'Display', None],
  [ 0xd78b, 'HangulSyllableHigs', [], 'Display', None],
  [ 0xd78c, 'HangulSyllableHin', [], 'Display', None],
  [ 0xd78d, 'HangulSyllableHinj', [], 'Display', None],
  [ 0xd78e, 'HangulSyllableHinh', [], 'Display', None],
  [ 0xd78f, 'HangulSyllableHid', [], 'Display', None],
  [ 0xd790, 'HangulSyllableHil', [], 'Display', None],
  [ 0xd791, 'HangulSyllableHilg', [], 'Display', None],
  [ 0xd792, 'HangulSyllableHilm', [], 'Display', None],
  [ 0xd793, 'HangulSyllableHilb', [], 'Display', None],
  [ 0xd794, 'HangulSyllableHils', [], 'Display', None],
  [ 0xd795, 'HangulSyllableHilt', [], 'Display', None],
  [ 0xd796, 'HangulSyllableHilp', [], 'Display', None],
  [ 0xd797, 'HangulSyllableHilh', [], 'Display', None],
  [ 0xd798, 'HangulSyllableHim', [], 'Display', None],
  [ 0xd799, 'HangulSyllableHib', [], 'Display', None],
  [ 0xd79a, 'HangulSyllableHibs', [], 'Display', None],
  [ 0xd79b, 'HangulSyllableHis', [], 'Display', None],
  [ 0xd79c, 'HangulSyllableHiss', [], 'Display', None],
  [ 0xd79d, 'HangulSyllableHing', [], 'Display', None],
  [ 0xd79e, 'HangulSyllableHij', [], 'Display', None],
  [ 0xd79f, 'HangulSyllableHic', [], 'Display', None],
  [ 0xd7a0, 'HangulSyllableHik', [], 'Display', None],
  [ 0xd7a1, 'HangulSyllableHit', [], 'Display', None],
  [ 0xd7a2, 'HangulSyllableHip', [], 'Display', None],
  [ 0xd7a3, 'HangulSyllableHih', [], 'Display', None],
  [ 0xd7a4, 'UnicodeCharacter0xd7a4', [], 'Display', None],
  [ 0xd7a5, 'UnicodeCharacter0xd7a5', [], 'Display', None],
  [ 0xd7a6, 'UnicodeCharacter0xd7a6', [], 'Display', None],
  [ 0xd7a7, 'UnicodeCharacter0xd7a7', [], 'Display', None],
  [ 0xd7a8, 'UnicodeCharacter0xd7a8', [], 'Display', None],
  [ 0xd7a9, 'UnicodeCharacter0xd7a9', [], 'Display', None],
  [ 0xd7aa, 'UnicodeCharacter0xd7aa', [], 'Display', None],
  [ 0xd7ab, 'UnicodeCharacter0xd7ab', [], 'Display', None],
  [ 0xd7ac, 'UnicodeCharacter0xd7ac', [], 'Display', None],
  [ 0xd7ad, 'UnicodeCharacter0xd7ad', [], 'Display', None],
  [ 0xd7ae, 'UnicodeCharacter0xd7ae', [], 'Display', None],
  [ 0xd7af, 'UnicodeCharacter0xd7af', [], 'Display', None],
  [ 0xd7b0, 'UnicodeCharacter0xd7b0', [], 'Display', None],
  [ 0xd7b1, 'UnicodeCharacter0xd7b1', [], 'Display', None],
  [ 0xd7b2, 'UnicodeCharacter0xd7b2', [], 'Display', None],
  [ 0xd7b3, 'UnicodeCharacter0xd7b3', [], 'Display', None],
  [ 0xd7b4, 'UnicodeCharacter0xd7b4', [], 'Display', None],
  [ 0xd7b5, 'UnicodeCharacter0xd7b5', [], 'Display', None],
  [ 0xd7b6, 'UnicodeCharacter0xd7b6', [], 'Display', None],
  [ 0xd7b7, 'UnicodeCharacter0xd7b7', [], 'Display', None],
  [ 0xd7b8, 'UnicodeCharacter0xd7b8', [], 'Display', None],
  [ 0xd7b9, 'UnicodeCharacter0xd7b9', [], 'Display', None],
  [ 0xd7ba, 'UnicodeCharacter0xd7ba', [], 'Display', None],
  [ 0xd7bb, 'UnicodeCharacter0xd7bb', [], 'Display', None],
  [ 0xd7bc, 'UnicodeCharacter0xd7bc', [], 'Display', None],
  [ 0xd7bd, 'UnicodeCharacter0xd7bd', [], 'Display', None],
  [ 0xd7be, 'UnicodeCharacter0xd7be', [], 'Display', None],
  [ 0xd7bf, 'UnicodeCharacter0xd7bf', [], 'Display', None],
  [ 0xd7c0, 'UnicodeCharacter0xd7c0', [], 'Display', None],
  [ 0xd7c1, 'UnicodeCharacter0xd7c1', [], 'Display', None],
  [ 0xd7c2, 'UnicodeCharacter0xd7c2', [], 'Display', None],
  [ 0xd7c3, 'UnicodeCharacter0xd7c3', [], 'Display', None],
  [ 0xd7c4, 'UnicodeCharacter0xd7c4', [], 'Display', None],
  [ 0xd7c5, 'UnicodeCharacter0xd7c5', [], 'Display', None],
  [ 0xd7c6, 'UnicodeCharacter0xd7c6', [], 'Display', None],
  [ 0xd7c7, 'UnicodeCharacter0xd7c7', [], 'Display', None],
  [ 0xd7c8, 'UnicodeCharacter0xd7c8', [], 'Display', None],
  [ 0xd7c9, 'UnicodeCharacter0xd7c9', [], 'Display', None],
  [ 0xd7ca, 'UnicodeCharacter0xd7ca', [], 'Display', None],
  [ 0xd7cb, 'UnicodeCharacter0xd7cb', [], 'Display', None],
  [ 0xd7cc, 'UnicodeCharacter0xd7cc', [], 'Display', None],
  [ 0xd7cd, 'UnicodeCharacter0xd7cd', [], 'Display', None],
  [ 0xd7ce, 'UnicodeCharacter0xd7ce', [], 'Display', None],
  [ 0xd7cf, 'UnicodeCharacter0xd7cf', [], 'Display', None],
  [ 0xd7d0, 'UnicodeCharacter0xd7d0', [], 'Display', None],
  [ 0xd7d1, 'UnicodeCharacter0xd7d1', [], 'Display', None],
  [ 0xd7d2, 'UnicodeCharacter0xd7d2', [], 'Display', None],
  [ 0xd7d3, 'UnicodeCharacter0xd7d3', [], 'Display', None],
  [ 0xd7d4, 'UnicodeCharacter0xd7d4', [], 'Display', None],
  [ 0xd7d5, 'UnicodeCharacter0xd7d5', [], 'Display', None],
  [ 0xd7d6, 'UnicodeCharacter0xd7d6', [], 'Display', None],
  [ 0xd7d7, 'UnicodeCharacter0xd7d7', [], 'Display', None],
  [ 0xd7d8, 'UnicodeCharacter0xd7d8', [], 'Display', None],
  [ 0xd7d9, 'UnicodeCharacter0xd7d9', [], 'Display', None],
  [ 0xd7da, 'UnicodeCharacter0xd7da', [], 'Display', None],
  [ 0xd7db, 'UnicodeCharacter0xd7db', [], 'Display', None],
  [ 0xd7dc, 'UnicodeCharacter0xd7dc', [], 'Display', None],
  [ 0xd7dd, 'UnicodeCharacter0xd7dd', [], 'Display', None],
  [ 0xd7de, 'UnicodeCharacter0xd7de', [], 'Display', None],
  [ 0xd7df, 'UnicodeCharacter0xd7df', [], 'Display', None],
  [ 0xd7e0, 'UnicodeCharacter0xd7e0', [], 'Display', None],
  [ 0xd7e1, 'UnicodeCharacter0xd7e1', [], 'Display', None],
  [ 0xd7e2, 'UnicodeCharacter0xd7e2', [], 'Display', None],
  [ 0xd7e3, 'UnicodeCharacter0xd7e3', [], 'Display', None],
  [ 0xd7e4, 'UnicodeCharacter0xd7e4', [], 'Display', None],
  [ 0xd7e5, 'UnicodeCharacter0xd7e5', [], 'Display', None],
  [ 0xd7e6, 'UnicodeCharacter0xd7e6', [], 'Display', None],
  [ 0xd7e7, 'UnicodeCharacter0xd7e7', [], 'Display', None],
  [ 0xd7e8, 'UnicodeCharacter0xd7e8', [], 'Display', None],
  [ 0xd7e9, 'UnicodeCharacter0xd7e9', [], 'Display', None],
  [ 0xd7ea, 'UnicodeCharacter0xd7ea', [], 'Display', None],
  [ 0xd7eb, 'UnicodeCharacter0xd7eb', [], 'Display', None],
  [ 0xd7ec, 'UnicodeCharacter0xd7ec', [], 'Display', None],
  [ 0xd7ed, 'UnicodeCharacter0xd7ed', [], 'Display', None],
  [ 0xd7ee, 'UnicodeCharacter0xd7ee', [], 'Display', None],
  [ 0xd7ef, 'UnicodeCharacter0xd7ef', [], 'Display', None],
  [ 0xd7f0, 'UnicodeCharacter0xd7f0', [], 'Display', None],
  [ 0xd7f1, 'UnicodeCharacter0xd7f1', [], 'Display', None],
  [ 0xd7f2, 'UnicodeCharacter0xd7f2', [], 'Display', None],
  [ 0xd7f3, 'UnicodeCharacter0xd7f3', [], 'Display', None],
  [ 0xd7f4, 'UnicodeCharacter0xd7f4', [], 'Display', None],
  [ 0xd7f5, 'UnicodeCharacter0xd7f5', [], 'Display', None],
  [ 0xd7f6, 'UnicodeCharacter0xd7f6', [], 'Display', None],
  [ 0xd7f7, 'UnicodeCharacter0xd7f7', [], 'Display', None],
  [ 0xd7f8, 'UnicodeCharacter0xd7f8', [], 'Display', None],
  [ 0xd7f9, 'UnicodeCharacter0xd7f9', [], 'Display', None],
  [ 0xd7fa, 'UnicodeCharacter0xd7fa', [], 'Display', None],
  [ 0xd7fb, 'UnicodeCharacter0xd7fb', [], 'Display', None],
  [ 0xd7fc, 'UnicodeCharacter0xd7fc', [], 'Display', None],
  [ 0xd7fd, 'UnicodeCharacter0xd7fd', [], 'Display', None],
  [ 0xd7fe, 'UnicodeCharacter0xd7fe', [], 'Display', None],
  [ 0xd7ff, 'UnicodeCharacter0xd7ff', [], 'Display', None],
  [ 0xd800, 'UnicodeCharacter0xd800', [], 'Display', None],
  [ 0xd801, 'UnicodeCharacter0xd801', [], 'Display', None],
  [ 0xd802, 'UnicodeCharacter0xd802', [], 'Display', None],
  [ 0xd803, 'UnicodeCharacter0xd803', [], 'Display', None],
  [ 0xd804, 'UnicodeCharacter0xd804', [], 'Display', None],
  [ 0xd805, 'UnicodeCharacter0xd805', [], 'Display', None],
  [ 0xd806, 'UnicodeCharacter0xd806', [], 'Display', None],
  [ 0xd807, 'UnicodeCharacter0xd807', [], 'Display', None],
  [ 0xd808, 'UnicodeCharacter0xd808', [], 'Display', None],
  [ 0xd809, 'UnicodeCharacter0xd809', [], 'Display', None],
  [ 0xd80a, 'UnicodeCharacter0xd80a', [], 'Display', None],
  [ 0xd80b, 'UnicodeCharacter0xd80b', [], 'Display', None],
  [ 0xd80c, 'UnicodeCharacter0xd80c', [], 'Display', None],
  [ 0xd80d, 'UnicodeCharacter0xd80d', [], 'Display', None],
  [ 0xd80e, 'UnicodeCharacter0xd80e', [], 'Display', None],
  [ 0xd80f, 'UnicodeCharacter0xd80f', [], 'Display', None],
  [ 0xd810, 'UnicodeCharacter0xd810', [], 'Display', None],
  [ 0xd811, 'UnicodeCharacter0xd811', [], 'Display', None],
  [ 0xd812, 'UnicodeCharacter0xd812', [], 'Display', None],
  [ 0xd813, 'UnicodeCharacter0xd813', [], 'Display', None],
  [ 0xd814, 'UnicodeCharacter0xd814', [], 'Display', None],
  [ 0xd815, 'UnicodeCharacter0xd815', [], 'Display', None],
  [ 0xd816, 'UnicodeCharacter0xd816', [], 'Display', None],
  [ 0xd817, 'UnicodeCharacter0xd817', [], 'Display', None],
  [ 0xd818, 'UnicodeCharacter0xd818', [], 'Display', None],
  [ 0xd819, 'UnicodeCharacter0xd819', [], 'Display', None],
  [ 0xd81a, 'UnicodeCharacter0xd81a', [], 'Display', None],
  [ 0xd81b, 'UnicodeCharacter0xd81b', [], 'Display', None],
  [ 0xd81c, 'UnicodeCharacter0xd81c', [], 'Display', None],
  [ 0xd81d, 'UnicodeCharacter0xd81d', [], 'Display', None],
  [ 0xd81e, 'UnicodeCharacter0xd81e', [], 'Display', None],
  [ 0xd81f, 'UnicodeCharacter0xd81f', [], 'Display', None],
  [ 0xd820, 'UnicodeCharacter0xd820', [], 'Display', None],
  [ 0xd821, 'UnicodeCharacter0xd821', [], 'Display', None],
  [ 0xd822, 'UnicodeCharacter0xd822', [], 'Display', None],
  [ 0xd823, 'UnicodeCharacter0xd823', [], 'Display', None],
  [ 0xd824, 'UnicodeCharacter0xd824', [], 'Display', None],
  [ 0xd825, 'UnicodeCharacter0xd825', [], 'Display', None],
  [ 0xd826, 'UnicodeCharacter0xd826', [], 'Display', None],
  [ 0xd827, 'UnicodeCharacter0xd827', [], 'Display', None],
  [ 0xd828, 'UnicodeCharacter0xd828', [], 'Display', None],
  [ 0xd829, 'UnicodeCharacter0xd829', [], 'Display', None],
  [ 0xd82a, 'UnicodeCharacter0xd82a', [], 'Display', None],
  [ 0xd82b, 'UnicodeCharacter0xd82b', [], 'Display', None],
  [ 0xd82c, 'UnicodeCharacter0xd82c', [], 'Display', None],
  [ 0xd82d, 'UnicodeCharacter0xd82d', [], 'Display', None],
  [ 0xd82e, 'UnicodeCharacter0xd82e', [], 'Display', None],
  [ 0xd82f, 'UnicodeCharacter0xd82f', [], 'Display', None],
  [ 0xd830, 'UnicodeCharacter0xd830', [], 'Display', None],
  [ 0xd831, 'UnicodeCharacter0xd831', [], 'Display', None],
  [ 0xd832, 'UnicodeCharacter0xd832', [], 'Display', None],
  [ 0xd833, 'UnicodeCharacter0xd833', [], 'Display', None],
  [ 0xd834, 'UnicodeCharacter0xd834', [], 'Display', None],
  [ 0xd835, 'UnicodeCharacter0xd835', [], 'Display', None],
  [ 0xd836, 'UnicodeCharacter0xd836', [], 'Display', None],
  [ 0xd837, 'UnicodeCharacter0xd837', [], 'Display', None],
  [ 0xd838, 'UnicodeCharacter0xd838', [], 'Display', None],
  [ 0xd839, 'UnicodeCharacter0xd839', [], 'Display', None],
  [ 0xd83a, 'UnicodeCharacter0xd83a', [], 'Display', None],
  [ 0xd83b, 'UnicodeCharacter0xd83b', [], 'Display', None],
  [ 0xd83c, 'UnicodeCharacter0xd83c', [], 'Display', None],
  [ 0xd83d, 'UnicodeCharacter0xd83d', [], 'Display', None],
  [ 0xd83e, 'UnicodeCharacter0xd83e', [], 'Display', None],
  [ 0xd83f, 'UnicodeCharacter0xd83f', [], 'Display', None],
  [ 0xd840, 'UnicodeCharacter0xd840', [], 'Display', None],
  [ 0xd841, 'UnicodeCharacter0xd841', [], 'Display', None],
  [ 0xd842, 'UnicodeCharacter0xd842', [], 'Display', None],
  [ 0xd843, 'UnicodeCharacter0xd843', [], 'Display', None],
  [ 0xd844, 'UnicodeCharacter0xd844', [], 'Display', None],
  [ 0xd845, 'UnicodeCharacter0xd845', [], 'Display', None],
  [ 0xd846, 'UnicodeCharacter0xd846', [], 'Display', None],
  [ 0xd847, 'UnicodeCharacter0xd847', [], 'Display', None],
  [ 0xd848, 'UnicodeCharacter0xd848', [], 'Display', None],
  [ 0xd849, 'UnicodeCharacter0xd849', [], 'Display', None],
  [ 0xd84a, 'UnicodeCharacter0xd84a', [], 'Display', None],
  [ 0xd84b, 'UnicodeCharacter0xd84b', [], 'Display', None],
  [ 0xd84c, 'UnicodeCharacter0xd84c', [], 'Display', None],
  [ 0xd84d, 'UnicodeCharacter0xd84d', [], 'Display', None],
  [ 0xd84e, 'UnicodeCharacter0xd84e', [], 'Display', None],
  [ 0xd84f, 'UnicodeCharacter0xd84f', [], 'Display', None],
  [ 0xd850, 'UnicodeCharacter0xd850', [], 'Display', None],
  [ 0xd851, 'UnicodeCharacter0xd851', [], 'Display', None],
  [ 0xd852, 'UnicodeCharacter0xd852', [], 'Display', None],
  [ 0xd853, 'UnicodeCharacter0xd853', [], 'Display', None],
  [ 0xd854, 'UnicodeCharacter0xd854', [], 'Display', None],
  [ 0xd855, 'UnicodeCharacter0xd855', [], 'Display', None],
  [ 0xd856, 'UnicodeCharacter0xd856', [], 'Display', None],
  [ 0xd857, 'UnicodeCharacter0xd857', [], 'Display', None],
  [ 0xd858, 'UnicodeCharacter0xd858', [], 'Display', None],
  [ 0xd859, 'UnicodeCharacter0xd859', [], 'Display', None],
  [ 0xd85a, 'UnicodeCharacter0xd85a', [], 'Display', None],
  [ 0xd85b, 'UnicodeCharacter0xd85b', [], 'Display', None],
  [ 0xd85c, 'UnicodeCharacter0xd85c', [], 'Display', None],
  [ 0xd85d, 'UnicodeCharacter0xd85d', [], 'Display', None],
  [ 0xd85e, 'UnicodeCharacter0xd85e', [], 'Display', None],
  [ 0xd85f, 'UnicodeCharacter0xd85f', [], 'Display', None],
  [ 0xd860, 'UnicodeCharacter0xd860', [], 'Display', None],
  [ 0xd861, 'UnicodeCharacter0xd861', [], 'Display', None],
  [ 0xd862, 'UnicodeCharacter0xd862', [], 'Display', None],
  [ 0xd863, 'UnicodeCharacter0xd863', [], 'Display', None],
  [ 0xd864, 'UnicodeCharacter0xd864', [], 'Display', None],
  [ 0xd865, 'UnicodeCharacter0xd865', [], 'Display', None],
  [ 0xd866, 'UnicodeCharacter0xd866', [], 'Display', None],
  [ 0xd867, 'UnicodeCharacter0xd867', [], 'Display', None],
  [ 0xd868, 'UnicodeCharacter0xd868', [], 'Display', None],
  [ 0xd869, 'UnicodeCharacter0xd869', [], 'Display', None],
  [ 0xd86a, 'UnicodeCharacter0xd86a', [], 'Display', None],
  [ 0xd86b, 'UnicodeCharacter0xd86b', [], 'Display', None],
  [ 0xd86c, 'UnicodeCharacter0xd86c', [], 'Display', None],
  [ 0xd86d, 'UnicodeCharacter0xd86d', [], 'Display', None],
  [ 0xd86e, 'UnicodeCharacter0xd86e', [], 'Display', None],
  [ 0xd86f, 'UnicodeCharacter0xd86f', [], 'Display', None],
  [ 0xd870, 'UnicodeCharacter0xd870', [], 'Display', None],
  [ 0xd871, 'UnicodeCharacter0xd871', [], 'Display', None],
  [ 0xd872, 'UnicodeCharacter0xd872', [], 'Display', None],
  [ 0xd873, 'UnicodeCharacter0xd873', [], 'Display', None],
  [ 0xd874, 'UnicodeCharacter0xd874', [], 'Display', None],
  [ 0xd875, 'UnicodeCharacter0xd875', [], 'Display', None],
  [ 0xd876, 'UnicodeCharacter0xd876', [], 'Display', None],
  [ 0xd877, 'UnicodeCharacter0xd877', [], 'Display', None],
  [ 0xd878, 'UnicodeCharacter0xd878', [], 'Display', None],
  [ 0xd879, 'UnicodeCharacter0xd879', [], 'Display', None],
  [ 0xd87a, 'UnicodeCharacter0xd87a', [], 'Display', None],
  [ 0xd87b, 'UnicodeCharacter0xd87b', [], 'Display', None],
  [ 0xd87c, 'UnicodeCharacter0xd87c', [], 'Display', None],
  [ 0xd87d, 'UnicodeCharacter0xd87d', [], 'Display', None],
  [ 0xd87e, 'UnicodeCharacter0xd87e', [], 'Display', None],
  [ 0xd87f, 'UnicodeCharacter0xd87f', [], 'Display', None],
  [ 0xd880, 'UnicodeCharacter0xd880', [], 'Display', None],
  [ 0xd881, 'UnicodeCharacter0xd881', [], 'Display', None],
  [ 0xd882, 'UnicodeCharacter0xd882', [], 'Display', None],
  [ 0xd883, 'UnicodeCharacter0xd883', [], 'Display', None],
  [ 0xd884, 'UnicodeCharacter0xd884', [], 'Display', None],
  [ 0xd885, 'UnicodeCharacter0xd885', [], 'Display', None],
  [ 0xd886, 'UnicodeCharacter0xd886', [], 'Display', None],
  [ 0xd887, 'UnicodeCharacter0xd887', [], 'Display', None],
  [ 0xd888, 'UnicodeCharacter0xd888', [], 'Display', None],
  [ 0xd889, 'UnicodeCharacter0xd889', [], 'Display', None],
  [ 0xd88a, 'UnicodeCharacter0xd88a', [], 'Display', None],
  [ 0xd88b, 'UnicodeCharacter0xd88b', [], 'Display', None],
  [ 0xd88c, 'UnicodeCharacter0xd88c', [], 'Display', None],
  [ 0xd88d, 'UnicodeCharacter0xd88d', [], 'Display', None],
  [ 0xd88e, 'UnicodeCharacter0xd88e', [], 'Display', None],
  [ 0xd88f, 'UnicodeCharacter0xd88f', [], 'Display', None],
  [ 0xd890, 'UnicodeCharacter0xd890', [], 'Display', None],
  [ 0xd891, 'UnicodeCharacter0xd891', [], 'Display', None],
  [ 0xd892, 'UnicodeCharacter0xd892', [], 'Display', None],
  [ 0xd893, 'UnicodeCharacter0xd893', [], 'Display', None],
  [ 0xd894, 'UnicodeCharacter0xd894', [], 'Display', None],
  [ 0xd895, 'UnicodeCharacter0xd895', [], 'Display', None],
  [ 0xd896, 'UnicodeCharacter0xd896', [], 'Display', None],
  [ 0xd897, 'UnicodeCharacter0xd897', [], 'Display', None],
  [ 0xd898, 'UnicodeCharacter0xd898', [], 'Display', None],
  [ 0xd899, 'UnicodeCharacter0xd899', [], 'Display', None],
  [ 0xd89a, 'UnicodeCharacter0xd89a', [], 'Display', None],
  [ 0xd89b, 'UnicodeCharacter0xd89b', [], 'Display', None],
  [ 0xd89c, 'UnicodeCharacter0xd89c', [], 'Display', None],
  [ 0xd89d, 'UnicodeCharacter0xd89d', [], 'Display', None],
  [ 0xd89e, 'UnicodeCharacter0xd89e', [], 'Display', None],
  [ 0xd89f, 'UnicodeCharacter0xd89f', [], 'Display', None],
  [ 0xd8a0, 'UnicodeCharacter0xd8a0', [], 'Display', None],
  [ 0xd8a1, 'UnicodeCharacter0xd8a1', [], 'Display', None],
  [ 0xd8a2, 'UnicodeCharacter0xd8a2', [], 'Display', None],
  [ 0xd8a3, 'UnicodeCharacter0xd8a3', [], 'Display', None],
  [ 0xd8a4, 'UnicodeCharacter0xd8a4', [], 'Display', None],
  [ 0xd8a5, 'UnicodeCharacter0xd8a5', [], 'Display', None],
  [ 0xd8a6, 'UnicodeCharacter0xd8a6', [], 'Display', None],
  [ 0xd8a7, 'UnicodeCharacter0xd8a7', [], 'Display', None],
  [ 0xd8a8, 'UnicodeCharacter0xd8a8', [], 'Display', None],
  [ 0xd8a9, 'UnicodeCharacter0xd8a9', [], 'Display', None],
  [ 0xd8aa, 'UnicodeCharacter0xd8aa', [], 'Display', None],
  [ 0xd8ab, 'UnicodeCharacter0xd8ab', [], 'Display', None],
  [ 0xd8ac, 'UnicodeCharacter0xd8ac', [], 'Display', None],
  [ 0xd8ad, 'UnicodeCharacter0xd8ad', [], 'Display', None],
  [ 0xd8ae, 'UnicodeCharacter0xd8ae', [], 'Display', None],
  [ 0xd8af, 'UnicodeCharacter0xd8af', [], 'Display', None],
  [ 0xd8b0, 'UnicodeCharacter0xd8b0', [], 'Display', None],
  [ 0xd8b1, 'UnicodeCharacter0xd8b1', [], 'Display', None],
  [ 0xd8b2, 'UnicodeCharacter0xd8b2', [], 'Display', None],
  [ 0xd8b3, 'UnicodeCharacter0xd8b3', [], 'Display', None],
  [ 0xd8b4, 'UnicodeCharacter0xd8b4', [], 'Display', None],
  [ 0xd8b5, 'UnicodeCharacter0xd8b5', [], 'Display', None],
  [ 0xd8b6, 'UnicodeCharacter0xd8b6', [], 'Display', None],
  [ 0xd8b7, 'UnicodeCharacter0xd8b7', [], 'Display', None],
  [ 0xd8b8, 'UnicodeCharacter0xd8b8', [], 'Display', None],
  [ 0xd8b9, 'UnicodeCharacter0xd8b9', [], 'Display', None],
  [ 0xd8ba, 'UnicodeCharacter0xd8ba', [], 'Display', None],
  [ 0xd8bb, 'UnicodeCharacter0xd8bb', [], 'Display', None],
  [ 0xd8bc, 'UnicodeCharacter0xd8bc', [], 'Display', None],
  [ 0xd8bd, 'UnicodeCharacter0xd8bd', [], 'Display', None],
  [ 0xd8be, 'UnicodeCharacter0xd8be', [], 'Display', None],
  [ 0xd8bf, 'UnicodeCharacter0xd8bf', [], 'Display', None],
  [ 0xd8c0, 'UnicodeCharacter0xd8c0', [], 'Display', None],
  [ 0xd8c1, 'UnicodeCharacter0xd8c1', [], 'Display', None],
  [ 0xd8c2, 'UnicodeCharacter0xd8c2', [], 'Display', None],
  [ 0xd8c3, 'UnicodeCharacter0xd8c3', [], 'Display', None],
  [ 0xd8c4, 'UnicodeCharacter0xd8c4', [], 'Display', None],
  [ 0xd8c5, 'UnicodeCharacter0xd8c5', [], 'Display', None],
  [ 0xd8c6, 'UnicodeCharacter0xd8c6', [], 'Display', None],
  [ 0xd8c7, 'UnicodeCharacter0xd8c7', [], 'Display', None],
  [ 0xd8c8, 'UnicodeCharacter0xd8c8', [], 'Display', None],
  [ 0xd8c9, 'UnicodeCharacter0xd8c9', [], 'Display', None],
  [ 0xd8ca, 'UnicodeCharacter0xd8ca', [], 'Display', None],
  [ 0xd8cb, 'UnicodeCharacter0xd8cb', [], 'Display', None],
  [ 0xd8cc, 'UnicodeCharacter0xd8cc', [], 'Display', None],
  [ 0xd8cd, 'UnicodeCharacter0xd8cd', [], 'Display', None],
  [ 0xd8ce, 'UnicodeCharacter0xd8ce', [], 'Display', None],
  [ 0xd8cf, 'UnicodeCharacter0xd8cf', [], 'Display', None],
  [ 0xd8d0, 'UnicodeCharacter0xd8d0', [], 'Display', None],
  [ 0xd8d1, 'UnicodeCharacter0xd8d1', [], 'Display', None],
  [ 0xd8d2, 'UnicodeCharacter0xd8d2', [], 'Display', None],
  [ 0xd8d3, 'UnicodeCharacter0xd8d3', [], 'Display', None],
  [ 0xd8d4, 'UnicodeCharacter0xd8d4', [], 'Display', None],
  [ 0xd8d5, 'UnicodeCharacter0xd8d5', [], 'Display', None],
  [ 0xd8d6, 'UnicodeCharacter0xd8d6', [], 'Display', None],
  [ 0xd8d7, 'UnicodeCharacter0xd8d7', [], 'Display', None],
  [ 0xd8d8, 'UnicodeCharacter0xd8d8', [], 'Display', None],
  [ 0xd8d9, 'UnicodeCharacter0xd8d9', [], 'Display', None],
  [ 0xd8da, 'UnicodeCharacter0xd8da', [], 'Display', None],
  [ 0xd8db, 'UnicodeCharacter0xd8db', [], 'Display', None],
  [ 0xd8dc, 'UnicodeCharacter0xd8dc', [], 'Display', None],
  [ 0xd8dd, 'UnicodeCharacter0xd8dd', [], 'Display', None],
  [ 0xd8de, 'UnicodeCharacter0xd8de', [], 'Display', None],
  [ 0xd8df, 'UnicodeCharacter0xd8df', [], 'Display', None],
  [ 0xd8e0, 'UnicodeCharacter0xd8e0', [], 'Display', None],
  [ 0xd8e1, 'UnicodeCharacter0xd8e1', [], 'Display', None],
  [ 0xd8e2, 'UnicodeCharacter0xd8e2', [], 'Display', None],
  [ 0xd8e3, 'UnicodeCharacter0xd8e3', [], 'Display', None],
  [ 0xd8e4, 'UnicodeCharacter0xd8e4', [], 'Display', None],
  [ 0xd8e5, 'UnicodeCharacter0xd8e5', [], 'Display', None],
  [ 0xd8e6, 'UnicodeCharacter0xd8e6', [], 'Display', None],
  [ 0xd8e7, 'UnicodeCharacter0xd8e7', [], 'Display', None],
  [ 0xd8e8, 'UnicodeCharacter0xd8e8', [], 'Display', None],
  [ 0xd8e9, 'UnicodeCharacter0xd8e9', [], 'Display', None],
  [ 0xd8ea, 'UnicodeCharacter0xd8ea', [], 'Display', None],
  [ 0xd8eb, 'UnicodeCharacter0xd8eb', [], 'Display', None],
  [ 0xd8ec, 'UnicodeCharacter0xd8ec', [], 'Display', None],
  [ 0xd8ed, 'UnicodeCharacter0xd8ed', [], 'Display', None],
  [ 0xd8ee, 'UnicodeCharacter0xd8ee', [], 'Display', None],
  [ 0xd8ef, 'UnicodeCharacter0xd8ef', [], 'Display', None],
  [ 0xd8f0, 'UnicodeCharacter0xd8f0', [], 'Display', None],
  [ 0xd8f1, 'UnicodeCharacter0xd8f1', [], 'Display', None],
  [ 0xd8f2, 'UnicodeCharacter0xd8f2', [], 'Display', None],
  [ 0xd8f3, 'UnicodeCharacter0xd8f3', [], 'Display', None],
  [ 0xd8f4, 'UnicodeCharacter0xd8f4', [], 'Display', None],
  [ 0xd8f5, 'UnicodeCharacter0xd8f5', [], 'Display', None],
  [ 0xd8f6, 'UnicodeCharacter0xd8f6', [], 'Display', None],
  [ 0xd8f7, 'UnicodeCharacter0xd8f7', [], 'Display', None],
  [ 0xd8f8, 'UnicodeCharacter0xd8f8', [], 'Display', None],
  [ 0xd8f9, 'UnicodeCharacter0xd8f9', [], 'Display', None],
  [ 0xd8fa, 'UnicodeCharacter0xd8fa', [], 'Display', None],
  [ 0xd8fb, 'UnicodeCharacter0xd8fb', [], 'Display', None],
  [ 0xd8fc, 'UnicodeCharacter0xd8fc', [], 'Display', None],
  [ 0xd8fd, 'UnicodeCharacter0xd8fd', [], 'Display', None],
  [ 0xd8fe, 'UnicodeCharacter0xd8fe', [], 'Display', None],
  [ 0xd8ff, 'UnicodeCharacter0xd8ff', [], 'Display', None],
  [ 0xd900, 'UnicodeCharacter0xd900', [], 'Display', None],
  [ 0xd901, 'UnicodeCharacter0xd901', [], 'Display', None],
  [ 0xd902, 'UnicodeCharacter0xd902', [], 'Display', None],
  [ 0xd903, 'UnicodeCharacter0xd903', [], 'Display', None],
  [ 0xd904, 'UnicodeCharacter0xd904', [], 'Display', None],
  [ 0xd905, 'UnicodeCharacter0xd905', [], 'Display', None],
  [ 0xd906, 'UnicodeCharacter0xd906', [], 'Display', None],
  [ 0xd907, 'UnicodeCharacter0xd907', [], 'Display', None],
  [ 0xd908, 'UnicodeCharacter0xd908', [], 'Display', None],
  [ 0xd909, 'UnicodeCharacter0xd909', [], 'Display', None],
  [ 0xd90a, 'UnicodeCharacter0xd90a', [], 'Display', None],
  [ 0xd90b, 'UnicodeCharacter0xd90b', [], 'Display', None],
  [ 0xd90c, 'UnicodeCharacter0xd90c', [], 'Display', None],
  [ 0xd90d, 'UnicodeCharacter0xd90d', [], 'Display', None],
  [ 0xd90e, 'UnicodeCharacter0xd90e', [], 'Display', None],
  [ 0xd90f, 'UnicodeCharacter0xd90f', [], 'Display', None],
  [ 0xd910, 'UnicodeCharacter0xd910', [], 'Display', None],
  [ 0xd911, 'UnicodeCharacter0xd911', [], 'Display', None],
  [ 0xd912, 'UnicodeCharacter0xd912', [], 'Display', None],
  [ 0xd913, 'UnicodeCharacter0xd913', [], 'Display', None],
  [ 0xd914, 'UnicodeCharacter0xd914', [], 'Display', None],
  [ 0xd915, 'UnicodeCharacter0xd915', [], 'Display', None],
  [ 0xd916, 'UnicodeCharacter0xd916', [], 'Display', None],
  [ 0xd917, 'UnicodeCharacter0xd917', [], 'Display', None],
  [ 0xd918, 'UnicodeCharacter0xd918', [], 'Display', None],
  [ 0xd919, 'UnicodeCharacter0xd919', [], 'Display', None],
  [ 0xd91a, 'UnicodeCharacter0xd91a', [], 'Display', None],
  [ 0xd91b, 'UnicodeCharacter0xd91b', [], 'Display', None],
  [ 0xd91c, 'UnicodeCharacter0xd91c', [], 'Display', None],
  [ 0xd91d, 'UnicodeCharacter0xd91d', [], 'Display', None],
  [ 0xd91e, 'UnicodeCharacter0xd91e', [], 'Display', None],
  [ 0xd91f, 'UnicodeCharacter0xd91f', [], 'Display', None],
  [ 0xd920, 'UnicodeCharacter0xd920', [], 'Display', None],
  [ 0xd921, 'UnicodeCharacter0xd921', [], 'Display', None],
  [ 0xd922, 'UnicodeCharacter0xd922', [], 'Display', None],
  [ 0xd923, 'UnicodeCharacter0xd923', [], 'Display', None],
  [ 0xd924, 'UnicodeCharacter0xd924', [], 'Display', None],
  [ 0xd925, 'UnicodeCharacter0xd925', [], 'Display', None],
  [ 0xd926, 'UnicodeCharacter0xd926', [], 'Display', None],
  [ 0xd927, 'UnicodeCharacter0xd927', [], 'Display', None],
  [ 0xd928, 'UnicodeCharacter0xd928', [], 'Display', None],
  [ 0xd929, 'UnicodeCharacter0xd929', [], 'Display', None],
  [ 0xd92a, 'UnicodeCharacter0xd92a', [], 'Display', None],
  [ 0xd92b, 'UnicodeCharacter0xd92b', [], 'Display', None],
  [ 0xd92c, 'UnicodeCharacter0xd92c', [], 'Display', None],
  [ 0xd92d, 'UnicodeCharacter0xd92d', [], 'Display', None],
  [ 0xd92e, 'UnicodeCharacter0xd92e', [], 'Display', None],
  [ 0xd92f, 'UnicodeCharacter0xd92f', [], 'Display', None],
  [ 0xd930, 'UnicodeCharacter0xd930', [], 'Display', None],
  [ 0xd931, 'UnicodeCharacter0xd931', [], 'Display', None],
  [ 0xd932, 'UnicodeCharacter0xd932', [], 'Display', None],
  [ 0xd933, 'UnicodeCharacter0xd933', [], 'Display', None],
  [ 0xd934, 'UnicodeCharacter0xd934', [], 'Display', None],
  [ 0xd935, 'UnicodeCharacter0xd935', [], 'Display', None],
  [ 0xd936, 'UnicodeCharacter0xd936', [], 'Display', None],
  [ 0xd937, 'UnicodeCharacter0xd937', [], 'Display', None],
  [ 0xd938, 'UnicodeCharacter0xd938', [], 'Display', None],
  [ 0xd939, 'UnicodeCharacter0xd939', [], 'Display', None],
  [ 0xd93a, 'UnicodeCharacter0xd93a', [], 'Display', None],
  [ 0xd93b, 'UnicodeCharacter0xd93b', [], 'Display', None],
  [ 0xd93c, 'UnicodeCharacter0xd93c', [], 'Display', None],
  [ 0xd93d, 'UnicodeCharacter0xd93d', [], 'Display', None],
  [ 0xd93e, 'UnicodeCharacter0xd93e', [], 'Display', None],
  [ 0xd93f, 'UnicodeCharacter0xd93f', [], 'Display', None],
  [ 0xd940, 'UnicodeCharacter0xd940', [], 'Display', None],
  [ 0xd941, 'UnicodeCharacter0xd941', [], 'Display', None],
  [ 0xd942, 'UnicodeCharacter0xd942', [], 'Display', None],
  [ 0xd943, 'UnicodeCharacter0xd943', [], 'Display', None],
  [ 0xd944, 'UnicodeCharacter0xd944', [], 'Display', None],
  [ 0xd945, 'UnicodeCharacter0xd945', [], 'Display', None],
  [ 0xd946, 'UnicodeCharacter0xd946', [], 'Display', None],
  [ 0xd947, 'UnicodeCharacter0xd947', [], 'Display', None],
  [ 0xd948, 'UnicodeCharacter0xd948', [], 'Display', None],
  [ 0xd949, 'UnicodeCharacter0xd949', [], 'Display', None],
  [ 0xd94a, 'UnicodeCharacter0xd94a', [], 'Display', None],
  [ 0xd94b, 'UnicodeCharacter0xd94b', [], 'Display', None],
  [ 0xd94c, 'UnicodeCharacter0xd94c', [], 'Display', None],
  [ 0xd94d, 'UnicodeCharacter0xd94d', [], 'Display', None],
  [ 0xd94e, 'UnicodeCharacter0xd94e', [], 'Display', None],
  [ 0xd94f, 'UnicodeCharacter0xd94f', [], 'Display', None],
  [ 0xd950, 'UnicodeCharacter0xd950', [], 'Display', None],
  [ 0xd951, 'UnicodeCharacter0xd951', [], 'Display', None],
  [ 0xd952, 'UnicodeCharacter0xd952', [], 'Display', None],
  [ 0xd953, 'UnicodeCharacter0xd953', [], 'Display', None],
  [ 0xd954, 'UnicodeCharacter0xd954', [], 'Display', None],
  [ 0xd955, 'UnicodeCharacter0xd955', [], 'Display', None],
  [ 0xd956, 'UnicodeCharacter0xd956', [], 'Display', None],
  [ 0xd957, 'UnicodeCharacter0xd957', [], 'Display', None],
  [ 0xd958, 'UnicodeCharacter0xd958', [], 'Display', None],
  [ 0xd959, 'UnicodeCharacter0xd959', [], 'Display', None],
  [ 0xd95a, 'UnicodeCharacter0xd95a', [], 'Display', None],
  [ 0xd95b, 'UnicodeCharacter0xd95b', [], 'Display', None],
  [ 0xd95c, 'UnicodeCharacter0xd95c', [], 'Display', None],
  [ 0xd95d, 'UnicodeCharacter0xd95d', [], 'Display', None],
  [ 0xd95e, 'UnicodeCharacter0xd95e', [], 'Display', None],
  [ 0xd95f, 'UnicodeCharacter0xd95f', [], 'Display', None],
  [ 0xd960, 'UnicodeCharacter0xd960', [], 'Display', None],
  [ 0xd961, 'UnicodeCharacter0xd961', [], 'Display', None],
  [ 0xd962, 'UnicodeCharacter0xd962', [], 'Display', None],
  [ 0xd963, 'UnicodeCharacter0xd963', [], 'Display', None],
  [ 0xd964, 'UnicodeCharacter0xd964', [], 'Display', None],
  [ 0xd965, 'UnicodeCharacter0xd965', [], 'Display', None],
  [ 0xd966, 'UnicodeCharacter0xd966', [], 'Display', None],
  [ 0xd967, 'UnicodeCharacter0xd967', [], 'Display', None],
  [ 0xd968, 'UnicodeCharacter0xd968', [], 'Display', None],
  [ 0xd969, 'UnicodeCharacter0xd969', [], 'Display', None],
  [ 0xd96a, 'UnicodeCharacter0xd96a', [], 'Display', None],
  [ 0xd96b, 'UnicodeCharacter0xd96b', [], 'Display', None],
  [ 0xd96c, 'UnicodeCharacter0xd96c', [], 'Display', None],
  [ 0xd96d, 'UnicodeCharacter0xd96d', [], 'Display', None],
  [ 0xd96e, 'UnicodeCharacter0xd96e', [], 'Display', None],
  [ 0xd96f, 'UnicodeCharacter0xd96f', [], 'Display', None],
  [ 0xd970, 'UnicodeCharacter0xd970', [], 'Display', None],
  [ 0xd971, 'UnicodeCharacter0xd971', [], 'Display', None],
  [ 0xd972, 'UnicodeCharacter0xd972', [], 'Display', None],
  [ 0xd973, 'UnicodeCharacter0xd973', [], 'Display', None],
  [ 0xd974, 'UnicodeCharacter0xd974', [], 'Display', None],
  [ 0xd975, 'UnicodeCharacter0xd975', [], 'Display', None],
  [ 0xd976, 'UnicodeCharacter0xd976', [], 'Display', None],
  [ 0xd977, 'UnicodeCharacter0xd977', [], 'Display', None],
  [ 0xd978, 'UnicodeCharacter0xd978', [], 'Display', None],
  [ 0xd979, 'UnicodeCharacter0xd979', [], 'Display', None],
  [ 0xd97a, 'UnicodeCharacter0xd97a', [], 'Display', None],
  [ 0xd97b, 'UnicodeCharacter0xd97b', [], 'Display', None],
  [ 0xd97c, 'UnicodeCharacter0xd97c', [], 'Display', None],
  [ 0xd97d, 'UnicodeCharacter0xd97d', [], 'Display', None],
  [ 0xd97e, 'UnicodeCharacter0xd97e', [], 'Display', None],
  [ 0xd97f, 'UnicodeCharacter0xd97f', [], 'Display', None],
  [ 0xd980, 'UnicodeCharacter0xd980', [], 'Display', None],
  [ 0xd981, 'UnicodeCharacter0xd981', [], 'Display', None],
  [ 0xd982, 'UnicodeCharacter0xd982', [], 'Display', None],
  [ 0xd983, 'UnicodeCharacter0xd983', [], 'Display', None],
  [ 0xd984, 'UnicodeCharacter0xd984', [], 'Display', None],
  [ 0xd985, 'UnicodeCharacter0xd985', [], 'Display', None],
  [ 0xd986, 'UnicodeCharacter0xd986', [], 'Display', None],
  [ 0xd987, 'UnicodeCharacter0xd987', [], 'Display', None],
  [ 0xd988, 'UnicodeCharacter0xd988', [], 'Display', None],
  [ 0xd989, 'UnicodeCharacter0xd989', [], 'Display', None],
  [ 0xd98a, 'UnicodeCharacter0xd98a', [], 'Display', None],
  [ 0xd98b, 'UnicodeCharacter0xd98b', [], 'Display', None],
  [ 0xd98c, 'UnicodeCharacter0xd98c', [], 'Display', None],
  [ 0xd98d, 'UnicodeCharacter0xd98d', [], 'Display', None],
  [ 0xd98e, 'UnicodeCharacter0xd98e', [], 'Display', None],
  [ 0xd98f, 'UnicodeCharacter0xd98f', [], 'Display', None],
  [ 0xd990, 'UnicodeCharacter0xd990', [], 'Display', None],
  [ 0xd991, 'UnicodeCharacter0xd991', [], 'Display', None],
  [ 0xd992, 'UnicodeCharacter0xd992', [], 'Display', None],
  [ 0xd993, 'UnicodeCharacter0xd993', [], 'Display', None],
  [ 0xd994, 'UnicodeCharacter0xd994', [], 'Display', None],
  [ 0xd995, 'UnicodeCharacter0xd995', [], 'Display', None],
  [ 0xd996, 'UnicodeCharacter0xd996', [], 'Display', None],
  [ 0xd997, 'UnicodeCharacter0xd997', [], 'Display', None],
  [ 0xd998, 'UnicodeCharacter0xd998', [], 'Display', None],
  [ 0xd999, 'UnicodeCharacter0xd999', [], 'Display', None],
  [ 0xd99a, 'UnicodeCharacter0xd99a', [], 'Display', None],
  [ 0xd99b, 'UnicodeCharacter0xd99b', [], 'Display', None],
  [ 0xd99c, 'UnicodeCharacter0xd99c', [], 'Display', None],
  [ 0xd99d, 'UnicodeCharacter0xd99d', [], 'Display', None],
  [ 0xd99e, 'UnicodeCharacter0xd99e', [], 'Display', None],
  [ 0xd99f, 'UnicodeCharacter0xd99f', [], 'Display', None],
  [ 0xd9a0, 'UnicodeCharacter0xd9a0', [], 'Display', None],
  [ 0xd9a1, 'UnicodeCharacter0xd9a1', [], 'Display', None],
  [ 0xd9a2, 'UnicodeCharacter0xd9a2', [], 'Display', None],
  [ 0xd9a3, 'UnicodeCharacter0xd9a3', [], 'Display', None],
  [ 0xd9a4, 'UnicodeCharacter0xd9a4', [], 'Display', None],
  [ 0xd9a5, 'UnicodeCharacter0xd9a5', [], 'Display', None],
  [ 0xd9a6, 'UnicodeCharacter0xd9a6', [], 'Display', None],
  [ 0xd9a7, 'UnicodeCharacter0xd9a7', [], 'Display', None],
  [ 0xd9a8, 'UnicodeCharacter0xd9a8', [], 'Display', None],
  [ 0xd9a9, 'UnicodeCharacter0xd9a9', [], 'Display', None],
  [ 0xd9aa, 'UnicodeCharacter0xd9aa', [], 'Display', None],
  [ 0xd9ab, 'UnicodeCharacter0xd9ab', [], 'Display', None],
  [ 0xd9ac, 'UnicodeCharacter0xd9ac', [], 'Display', None],
  [ 0xd9ad, 'UnicodeCharacter0xd9ad', [], 'Display', None],
  [ 0xd9ae, 'UnicodeCharacter0xd9ae', [], 'Display', None],
  [ 0xd9af, 'UnicodeCharacter0xd9af', [], 'Display', None],
  [ 0xd9b0, 'UnicodeCharacter0xd9b0', [], 'Display', None],
  [ 0xd9b1, 'UnicodeCharacter0xd9b1', [], 'Display', None],
  [ 0xd9b2, 'UnicodeCharacter0xd9b2', [], 'Display', None],
  [ 0xd9b3, 'UnicodeCharacter0xd9b3', [], 'Display', None],
  [ 0xd9b4, 'UnicodeCharacter0xd9b4', [], 'Display', None],
  [ 0xd9b5, 'UnicodeCharacter0xd9b5', [], 'Display', None],
  [ 0xd9b6, 'UnicodeCharacter0xd9b6', [], 'Display', None],
  [ 0xd9b7, 'UnicodeCharacter0xd9b7', [], 'Display', None],
  [ 0xd9b8, 'UnicodeCharacter0xd9b8', [], 'Display', None],
  [ 0xd9b9, 'UnicodeCharacter0xd9b9', [], 'Display', None],
  [ 0xd9ba, 'UnicodeCharacter0xd9ba', [], 'Display', None],
  [ 0xd9bb, 'UnicodeCharacter0xd9bb', [], 'Display', None],
  [ 0xd9bc, 'UnicodeCharacter0xd9bc', [], 'Display', None],
  [ 0xd9bd, 'UnicodeCharacter0xd9bd', [], 'Display', None],
  [ 0xd9be, 'UnicodeCharacter0xd9be', [], 'Display', None],
  [ 0xd9bf, 'UnicodeCharacter0xd9bf', [], 'Display', None],
  [ 0xd9c0, 'UnicodeCharacter0xd9c0', [], 'Display', None],
  [ 0xd9c1, 'UnicodeCharacter0xd9c1', [], 'Display', None],
  [ 0xd9c2, 'UnicodeCharacter0xd9c2', [], 'Display', None],
  [ 0xd9c3, 'UnicodeCharacter0xd9c3', [], 'Display', None],
  [ 0xd9c4, 'UnicodeCharacter0xd9c4', [], 'Display', None],
  [ 0xd9c5, 'UnicodeCharacter0xd9c5', [], 'Display', None],
  [ 0xd9c6, 'UnicodeCharacter0xd9c6', [], 'Display', None],
  [ 0xd9c7, 'UnicodeCharacter0xd9c7', [], 'Display', None],
  [ 0xd9c8, 'UnicodeCharacter0xd9c8', [], 'Display', None],
  [ 0xd9c9, 'UnicodeCharacter0xd9c9', [], 'Display', None],
  [ 0xd9ca, 'UnicodeCharacter0xd9ca', [], 'Display', None],
  [ 0xd9cb, 'UnicodeCharacter0xd9cb', [], 'Display', None],
  [ 0xd9cc, 'UnicodeCharacter0xd9cc', [], 'Display', None],
  [ 0xd9cd, 'UnicodeCharacter0xd9cd', [], 'Display', None],
  [ 0xd9ce, 'UnicodeCharacter0xd9ce', [], 'Display', None],
  [ 0xd9cf, 'UnicodeCharacter0xd9cf', [], 'Display', None],
  [ 0xd9d0, 'UnicodeCharacter0xd9d0', [], 'Display', None],
  [ 0xd9d1, 'UnicodeCharacter0xd9d1', [], 'Display', None],
  [ 0xd9d2, 'UnicodeCharacter0xd9d2', [], 'Display', None],
  [ 0xd9d3, 'UnicodeCharacter0xd9d3', [], 'Display', None],
  [ 0xd9d4, 'UnicodeCharacter0xd9d4', [], 'Display', None],
  [ 0xd9d5, 'UnicodeCharacter0xd9d5', [], 'Display', None],
  [ 0xd9d6, 'UnicodeCharacter0xd9d6', [], 'Display', None],
  [ 0xd9d7, 'UnicodeCharacter0xd9d7', [], 'Display', None],
  [ 0xd9d8, 'UnicodeCharacter0xd9d8', [], 'Display', None],
  [ 0xd9d9, 'UnicodeCharacter0xd9d9', [], 'Display', None],
  [ 0xd9da, 'UnicodeCharacter0xd9da', [], 'Display', None],
  [ 0xd9db, 'UnicodeCharacter0xd9db', [], 'Display', None],
  [ 0xd9dc, 'UnicodeCharacter0xd9dc', [], 'Display', None],
  [ 0xd9dd, 'UnicodeCharacter0xd9dd', [], 'Display', None],
  [ 0xd9de, 'UnicodeCharacter0xd9de', [], 'Display', None],
  [ 0xd9df, 'UnicodeCharacter0xd9df', [], 'Display', None],
  [ 0xd9e0, 'UnicodeCharacter0xd9e0', [], 'Display', None],
  [ 0xd9e1, 'UnicodeCharacter0xd9e1', [], 'Display', None],
  [ 0xd9e2, 'UnicodeCharacter0xd9e2', [], 'Display', None],
  [ 0xd9e3, 'UnicodeCharacter0xd9e3', [], 'Display', None],
  [ 0xd9e4, 'UnicodeCharacter0xd9e4', [], 'Display', None],
  [ 0xd9e5, 'UnicodeCharacter0xd9e5', [], 'Display', None],
  [ 0xd9e6, 'UnicodeCharacter0xd9e6', [], 'Display', None],
  [ 0xd9e7, 'UnicodeCharacter0xd9e7', [], 'Display', None],
  [ 0xd9e8, 'UnicodeCharacter0xd9e8', [], 'Display', None],
  [ 0xd9e9, 'UnicodeCharacter0xd9e9', [], 'Display', None],
  [ 0xd9ea, 'UnicodeCharacter0xd9ea', [], 'Display', None],
  [ 0xd9eb, 'UnicodeCharacter0xd9eb', [], 'Display', None],
  [ 0xd9ec, 'UnicodeCharacter0xd9ec', [], 'Display', None],
  [ 0xd9ed, 'UnicodeCharacter0xd9ed', [], 'Display', None],
  [ 0xd9ee, 'UnicodeCharacter0xd9ee', [], 'Display', None],
  [ 0xd9ef, 'UnicodeCharacter0xd9ef', [], 'Display', None],
  [ 0xd9f0, 'UnicodeCharacter0xd9f0', [], 'Display', None],
  [ 0xd9f1, 'UnicodeCharacter0xd9f1', [], 'Display', None],
  [ 0xd9f2, 'UnicodeCharacter0xd9f2', [], 'Display', None],
  [ 0xd9f3, 'UnicodeCharacter0xd9f3', [], 'Display', None],
  [ 0xd9f4, 'UnicodeCharacter0xd9f4', [], 'Display', None],
  [ 0xd9f5, 'UnicodeCharacter0xd9f5', [], 'Display', None],
  [ 0xd9f6, 'UnicodeCharacter0xd9f6', [], 'Display', None],
  [ 0xd9f7, 'UnicodeCharacter0xd9f7', [], 'Display', None],
  [ 0xd9f8, 'UnicodeCharacter0xd9f8', [], 'Display', None],
  [ 0xd9f9, 'UnicodeCharacter0xd9f9', [], 'Display', None],
  [ 0xd9fa, 'UnicodeCharacter0xd9fa', [], 'Display', None],
  [ 0xd9fb, 'UnicodeCharacter0xd9fb', [], 'Display', None],
  [ 0xd9fc, 'UnicodeCharacter0xd9fc', [], 'Display', None],
  [ 0xd9fd, 'UnicodeCharacter0xd9fd', [], 'Display', None],
  [ 0xd9fe, 'UnicodeCharacter0xd9fe', [], 'Display', None],
  [ 0xd9ff, 'UnicodeCharacter0xd9ff', [], 'Display', None],
  [ 0xda00, 'UnicodeCharacter0xda00', [], 'Display', None],
  [ 0xda01, 'UnicodeCharacter0xda01', [], 'Display', None],
  [ 0xda02, 'UnicodeCharacter0xda02', [], 'Display', None],
  [ 0xda03, 'UnicodeCharacter0xda03', [], 'Display', None],
  [ 0xda04, 'UnicodeCharacter0xda04', [], 'Display', None],
  [ 0xda05, 'UnicodeCharacter0xda05', [], 'Display', None],
  [ 0xda06, 'UnicodeCharacter0xda06', [], 'Display', None],
  [ 0xda07, 'UnicodeCharacter0xda07', [], 'Display', None],
  [ 0xda08, 'UnicodeCharacter0xda08', [], 'Display', None],
  [ 0xda09, 'UnicodeCharacter0xda09', [], 'Display', None],
  [ 0xda0a, 'UnicodeCharacter0xda0a', [], 'Display', None],
  [ 0xda0b, 'UnicodeCharacter0xda0b', [], 'Display', None],
  [ 0xda0c, 'UnicodeCharacter0xda0c', [], 'Display', None],
  [ 0xda0d, 'UnicodeCharacter0xda0d', [], 'Display', None],
  [ 0xda0e, 'UnicodeCharacter0xda0e', [], 'Display', None],
  [ 0xda0f, 'UnicodeCharacter0xda0f', [], 'Display', None],
  [ 0xda10, 'UnicodeCharacter0xda10', [], 'Display', None],
  [ 0xda11, 'UnicodeCharacter0xda11', [], 'Display', None],
  [ 0xda12, 'UnicodeCharacter0xda12', [], 'Display', None],
  [ 0xda13, 'UnicodeCharacter0xda13', [], 'Display', None],
  [ 0xda14, 'UnicodeCharacter0xda14', [], 'Display', None],
  [ 0xda15, 'UnicodeCharacter0xda15', [], 'Display', None],
  [ 0xda16, 'UnicodeCharacter0xda16', [], 'Display', None],
  [ 0xda17, 'UnicodeCharacter0xda17', [], 'Display', None],
  [ 0xda18, 'UnicodeCharacter0xda18', [], 'Display', None],
  [ 0xda19, 'UnicodeCharacter0xda19', [], 'Display', None],
  [ 0xda1a, 'UnicodeCharacter0xda1a', [], 'Display', None],
  [ 0xda1b, 'UnicodeCharacter0xda1b', [], 'Display', None],
  [ 0xda1c, 'UnicodeCharacter0xda1c', [], 'Display', None],
  [ 0xda1d, 'UnicodeCharacter0xda1d', [], 'Display', None],
  [ 0xda1e, 'UnicodeCharacter0xda1e', [], 'Display', None],
  [ 0xda1f, 'UnicodeCharacter0xda1f', [], 'Display', None],
  [ 0xda20, 'UnicodeCharacter0xda20', [], 'Display', None],
  [ 0xda21, 'UnicodeCharacter0xda21', [], 'Display', None],
  [ 0xda22, 'UnicodeCharacter0xda22', [], 'Display', None],
  [ 0xda23, 'UnicodeCharacter0xda23', [], 'Display', None],
  [ 0xda24, 'UnicodeCharacter0xda24', [], 'Display', None],
  [ 0xda25, 'UnicodeCharacter0xda25', [], 'Display', None],
  [ 0xda26, 'UnicodeCharacter0xda26', [], 'Display', None],
  [ 0xda27, 'UnicodeCharacter0xda27', [], 'Display', None],
  [ 0xda28, 'UnicodeCharacter0xda28', [], 'Display', None],
  [ 0xda29, 'UnicodeCharacter0xda29', [], 'Display', None],
  [ 0xda2a, 'UnicodeCharacter0xda2a', [], 'Display', None],
  [ 0xda2b, 'UnicodeCharacter0xda2b', [], 'Display', None],
  [ 0xda2c, 'UnicodeCharacter0xda2c', [], 'Display', None],
  [ 0xda2d, 'UnicodeCharacter0xda2d', [], 'Display', None],
  [ 0xda2e, 'UnicodeCharacter0xda2e', [], 'Display', None],
  [ 0xda2f, 'UnicodeCharacter0xda2f', [], 'Display', None],
  [ 0xda30, 'UnicodeCharacter0xda30', [], 'Display', None],
  [ 0xda31, 'UnicodeCharacter0xda31', [], 'Display', None],
  [ 0xda32, 'UnicodeCharacter0xda32', [], 'Display', None],
  [ 0xda33, 'UnicodeCharacter0xda33', [], 'Display', None],
  [ 0xda34, 'UnicodeCharacter0xda34', [], 'Display', None],
  [ 0xda35, 'UnicodeCharacter0xda35', [], 'Display', None],
  [ 0xda36, 'UnicodeCharacter0xda36', [], 'Display', None],
  [ 0xda37, 'UnicodeCharacter0xda37', [], 'Display', None],
  [ 0xda38, 'UnicodeCharacter0xda38', [], 'Display', None],
  [ 0xda39, 'UnicodeCharacter0xda39', [], 'Display', None],
  [ 0xda3a, 'UnicodeCharacter0xda3a', [], 'Display', None],
  [ 0xda3b, 'UnicodeCharacter0xda3b', [], 'Display', None],
  [ 0xda3c, 'UnicodeCharacter0xda3c', [], 'Display', None],
  [ 0xda3d, 'UnicodeCharacter0xda3d', [], 'Display', None],
  [ 0xda3e, 'UnicodeCharacter0xda3e', [], 'Display', None],
  [ 0xda3f, 'UnicodeCharacter0xda3f', [], 'Display', None],
  [ 0xda40, 'UnicodeCharacter0xda40', [], 'Display', None],
  [ 0xda41, 'UnicodeCharacter0xda41', [], 'Display', None],
  [ 0xda42, 'UnicodeCharacter0xda42', [], 'Display', None],
  [ 0xda43, 'UnicodeCharacter0xda43', [], 'Display', None],
  [ 0xda44, 'UnicodeCharacter0xda44', [], 'Display', None],
  [ 0xda45, 'UnicodeCharacter0xda45', [], 'Display', None],
  [ 0xda46, 'UnicodeCharacter0xda46', [], 'Display', None],
  [ 0xda47, 'UnicodeCharacter0xda47', [], 'Display', None],
  [ 0xda48, 'UnicodeCharacter0xda48', [], 'Display', None],
  [ 0xda49, 'UnicodeCharacter0xda49', [], 'Display', None],
  [ 0xda4a, 'UnicodeCharacter0xda4a', [], 'Display', None],
  [ 0xda4b, 'UnicodeCharacter0xda4b', [], 'Display', None],
  [ 0xda4c, 'UnicodeCharacter0xda4c', [], 'Display', None],
  [ 0xda4d, 'UnicodeCharacter0xda4d', [], 'Display', None],
  [ 0xda4e, 'UnicodeCharacter0xda4e', [], 'Display', None],
  [ 0xda4f, 'UnicodeCharacter0xda4f', [], 'Display', None],
  [ 0xda50, 'UnicodeCharacter0xda50', [], 'Display', None],
  [ 0xda51, 'UnicodeCharacter0xda51', [], 'Display', None],
  [ 0xda52, 'UnicodeCharacter0xda52', [], 'Display', None],
  [ 0xda53, 'UnicodeCharacter0xda53', [], 'Display', None],
  [ 0xda54, 'UnicodeCharacter0xda54', [], 'Display', None],
  [ 0xda55, 'UnicodeCharacter0xda55', [], 'Display', None],
  [ 0xda56, 'UnicodeCharacter0xda56', [], 'Display', None],
  [ 0xda57, 'UnicodeCharacter0xda57', [], 'Display', None],
  [ 0xda58, 'UnicodeCharacter0xda58', [], 'Display', None],
  [ 0xda59, 'UnicodeCharacter0xda59', [], 'Display', None],
  [ 0xda5a, 'UnicodeCharacter0xda5a', [], 'Display', None],
  [ 0xda5b, 'UnicodeCharacter0xda5b', [], 'Display', None],
  [ 0xda5c, 'UnicodeCharacter0xda5c', [], 'Display', None],
  [ 0xda5d, 'UnicodeCharacter0xda5d', [], 'Display', None],
  [ 0xda5e, 'UnicodeCharacter0xda5e', [], 'Display', None],
  [ 0xda5f, 'UnicodeCharacter0xda5f', [], 'Display', None],
  [ 0xda60, 'UnicodeCharacter0xda60', [], 'Display', None],
  [ 0xda61, 'UnicodeCharacter0xda61', [], 'Display', None],
  [ 0xda62, 'UnicodeCharacter0xda62', [], 'Display', None],
  [ 0xda63, 'UnicodeCharacter0xda63', [], 'Display', None],
  [ 0xda64, 'UnicodeCharacter0xda64', [], 'Display', None],
  [ 0xda65, 'UnicodeCharacter0xda65', [], 'Display', None],
  [ 0xda66, 'UnicodeCharacter0xda66', [], 'Display', None],
  [ 0xda67, 'UnicodeCharacter0xda67', [], 'Display', None],
  [ 0xda68, 'UnicodeCharacter0xda68', [], 'Display', None],
  [ 0xda69, 'UnicodeCharacter0xda69', [], 'Display', None],
  [ 0xda6a, 'UnicodeCharacter0xda6a', [], 'Display', None],
  [ 0xda6b, 'UnicodeCharacter0xda6b', [], 'Display', None],
  [ 0xda6c, 'UnicodeCharacter0xda6c', [], 'Display', None],
  [ 0xda6d, 'UnicodeCharacter0xda6d', [], 'Display', None],
  [ 0xda6e, 'UnicodeCharacter0xda6e', [], 'Display', None],
  [ 0xda6f, 'UnicodeCharacter0xda6f', [], 'Display', None],
  [ 0xda70, 'UnicodeCharacter0xda70', [], 'Display', None],
  [ 0xda71, 'UnicodeCharacter0xda71', [], 'Display', None],
  [ 0xda72, 'UnicodeCharacter0xda72', [], 'Display', None],
  [ 0xda73, 'UnicodeCharacter0xda73', [], 'Display', None],
  [ 0xda74, 'UnicodeCharacter0xda74', [], 'Display', None],
  [ 0xda75, 'UnicodeCharacter0xda75', [], 'Display', None],
  [ 0xda76, 'UnicodeCharacter0xda76', [], 'Display', None],
  [ 0xda77, 'UnicodeCharacter0xda77', [], 'Display', None],
  [ 0xda78, 'UnicodeCharacter0xda78', [], 'Display', None],
  [ 0xda79, 'UnicodeCharacter0xda79', [], 'Display', None],
  [ 0xda7a, 'UnicodeCharacter0xda7a', [], 'Display', None],
  [ 0xda7b, 'UnicodeCharacter0xda7b', [], 'Display', None],
  [ 0xda7c, 'UnicodeCharacter0xda7c', [], 'Display', None],
  [ 0xda7d, 'UnicodeCharacter0xda7d', [], 'Display', None],
  [ 0xda7e, 'UnicodeCharacter0xda7e', [], 'Display', None],
  [ 0xda7f, 'UnicodeCharacter0xda7f', [], 'Display', None],
  [ 0xda80, 'UnicodeCharacter0xda80', [], 'Display', None],
  [ 0xda81, 'UnicodeCharacter0xda81', [], 'Display', None],
  [ 0xda82, 'UnicodeCharacter0xda82', [], 'Display', None],
  [ 0xda83, 'UnicodeCharacter0xda83', [], 'Display', None],
  [ 0xda84, 'UnicodeCharacter0xda84', [], 'Display', None],
  [ 0xda85, 'UnicodeCharacter0xda85', [], 'Display', None],
  [ 0xda86, 'UnicodeCharacter0xda86', [], 'Display', None],
  [ 0xda87, 'UnicodeCharacter0xda87', [], 'Display', None],
  [ 0xda88, 'UnicodeCharacter0xda88', [], 'Display', None],
  [ 0xda89, 'UnicodeCharacter0xda89', [], 'Display', None],
  [ 0xda8a, 'UnicodeCharacter0xda8a', [], 'Display', None],
  [ 0xda8b, 'UnicodeCharacter0xda8b', [], 'Display', None],
  [ 0xda8c, 'UnicodeCharacter0xda8c', [], 'Display', None],
  [ 0xda8d, 'UnicodeCharacter0xda8d', [], 'Display', None],
  [ 0xda8e, 'UnicodeCharacter0xda8e', [], 'Display', None],
  [ 0xda8f, 'UnicodeCharacter0xda8f', [], 'Display', None],
  [ 0xda90, 'UnicodeCharacter0xda90', [], 'Display', None],
  [ 0xda91, 'UnicodeCharacter0xda91', [], 'Display', None],
  [ 0xda92, 'UnicodeCharacter0xda92', [], 'Display', None],
  [ 0xda93, 'UnicodeCharacter0xda93', [], 'Display', None],
  [ 0xda94, 'UnicodeCharacter0xda94', [], 'Display', None],
  [ 0xda95, 'UnicodeCharacter0xda95', [], 'Display', None],
  [ 0xda96, 'UnicodeCharacter0xda96', [], 'Display', None],
  [ 0xda97, 'UnicodeCharacter0xda97', [], 'Display', None],
  [ 0xda98, 'UnicodeCharacter0xda98', [], 'Display', None],
  [ 0xda99, 'UnicodeCharacter0xda99', [], 'Display', None],
  [ 0xda9a, 'UnicodeCharacter0xda9a', [], 'Display', None],
  [ 0xda9b, 'UnicodeCharacter0xda9b', [], 'Display', None],
  [ 0xda9c, 'UnicodeCharacter0xda9c', [], 'Display', None],
  [ 0xda9d, 'UnicodeCharacter0xda9d', [], 'Display', None],
  [ 0xda9e, 'UnicodeCharacter0xda9e', [], 'Display', None],
  [ 0xda9f, 'UnicodeCharacter0xda9f', [], 'Display', None],
  [ 0xdaa0, 'UnicodeCharacter0xdaa0', [], 'Display', None],
  [ 0xdaa1, 'UnicodeCharacter0xdaa1', [], 'Display', None],
  [ 0xdaa2, 'UnicodeCharacter0xdaa2', [], 'Display', None],
  [ 0xdaa3, 'UnicodeCharacter0xdaa3', [], 'Display', None],
  [ 0xdaa4, 'UnicodeCharacter0xdaa4', [], 'Display', None],
  [ 0xdaa5, 'UnicodeCharacter0xdaa5', [], 'Display', None],
  [ 0xdaa6, 'UnicodeCharacter0xdaa6', [], 'Display', None],
  [ 0xdaa7, 'UnicodeCharacter0xdaa7', [], 'Display', None],
  [ 0xdaa8, 'UnicodeCharacter0xdaa8', [], 'Display', None],
  [ 0xdaa9, 'UnicodeCharacter0xdaa9', [], 'Display', None],
  [ 0xdaaa, 'UnicodeCharacter0xdaaa', [], 'Display', None],
  [ 0xdaab, 'UnicodeCharacter0xdaab', [], 'Display', None],
  [ 0xdaac, 'UnicodeCharacter0xdaac', [], 'Display', None],
  [ 0xdaad, 'UnicodeCharacter0xdaad', [], 'Display', None],
  [ 0xdaae, 'UnicodeCharacter0xdaae', [], 'Display', None],
  [ 0xdaaf, 'UnicodeCharacter0xdaaf', [], 'Display', None],
  [ 0xdab0, 'UnicodeCharacter0xdab0', [], 'Display', None],
  [ 0xdab1, 'UnicodeCharacter0xdab1', [], 'Display', None],
  [ 0xdab2, 'UnicodeCharacter0xdab2', [], 'Display', None],
  [ 0xdab3, 'UnicodeCharacter0xdab3', [], 'Display', None],
  [ 0xdab4, 'UnicodeCharacter0xdab4', [], 'Display', None],
  [ 0xdab5, 'UnicodeCharacter0xdab5', [], 'Display', None],
  [ 0xdab6, 'UnicodeCharacter0xdab6', [], 'Display', None],
  [ 0xdab7, 'UnicodeCharacter0xdab7', [], 'Display', None],
  [ 0xdab8, 'UnicodeCharacter0xdab8', [], 'Display', None],
  [ 0xdab9, 'UnicodeCharacter0xdab9', [], 'Display', None],
  [ 0xdaba, 'UnicodeCharacter0xdaba', [], 'Display', None],
  [ 0xdabb, 'UnicodeCharacter0xdabb', [], 'Display', None],
  [ 0xdabc, 'UnicodeCharacter0xdabc', [], 'Display', None],
  [ 0xdabd, 'UnicodeCharacter0xdabd', [], 'Display', None],
  [ 0xdabe, 'UnicodeCharacter0xdabe', [], 'Display', None],
  [ 0xdabf, 'UnicodeCharacter0xdabf', [], 'Display', None],
  [ 0xdac0, 'UnicodeCharacter0xdac0', [], 'Display', None],
  [ 0xdac1, 'UnicodeCharacter0xdac1', [], 'Display', None],
  [ 0xdac2, 'UnicodeCharacter0xdac2', [], 'Display', None],
  [ 0xdac3, 'UnicodeCharacter0xdac3', [], 'Display', None],
  [ 0xdac4, 'UnicodeCharacter0xdac4', [], 'Display', None],
  [ 0xdac5, 'UnicodeCharacter0xdac5', [], 'Display', None],
  [ 0xdac6, 'UnicodeCharacter0xdac6', [], 'Display', None],
  [ 0xdac7, 'UnicodeCharacter0xdac7', [], 'Display', None],
  [ 0xdac8, 'UnicodeCharacter0xdac8', [], 'Display', None],
  [ 0xdac9, 'UnicodeCharacter0xdac9', [], 'Display', None],
  [ 0xdaca, 'UnicodeCharacter0xdaca', [], 'Display', None],
  [ 0xdacb, 'UnicodeCharacter0xdacb', [], 'Display', None],
  [ 0xdacc, 'UnicodeCharacter0xdacc', [], 'Display', None],
  [ 0xdacd, 'UnicodeCharacter0xdacd', [], 'Display', None],
  [ 0xdace, 'UnicodeCharacter0xdace', [], 'Display', None],
  [ 0xdacf, 'UnicodeCharacter0xdacf', [], 'Display', None],
  [ 0xdad0, 'UnicodeCharacter0xdad0', [], 'Display', None],
  [ 0xdad1, 'UnicodeCharacter0xdad1', [], 'Display', None],
  [ 0xdad2, 'UnicodeCharacter0xdad2', [], 'Display', None],
  [ 0xdad3, 'UnicodeCharacter0xdad3', [], 'Display', None],
  [ 0xdad4, 'UnicodeCharacter0xdad4', [], 'Display', None],
  [ 0xdad5, 'UnicodeCharacter0xdad5', [], 'Display', None],
  [ 0xdad6, 'UnicodeCharacter0xdad6', [], 'Display', None],
  [ 0xdad7, 'UnicodeCharacter0xdad7', [], 'Display', None],
  [ 0xdad8, 'UnicodeCharacter0xdad8', [], 'Display', None],
  [ 0xdad9, 'UnicodeCharacter0xdad9', [], 'Display', None],
  [ 0xdada, 'UnicodeCharacter0xdada', [], 'Display', None],
  [ 0xdadb, 'UnicodeCharacter0xdadb', [], 'Display', None],
  [ 0xdadc, 'UnicodeCharacter0xdadc', [], 'Display', None],
  [ 0xdadd, 'UnicodeCharacter0xdadd', [], 'Display', None],
  [ 0xdade, 'UnicodeCharacter0xdade', [], 'Display', None],
  [ 0xdadf, 'UnicodeCharacter0xdadf', [], 'Display', None],
  [ 0xdae0, 'UnicodeCharacter0xdae0', [], 'Display', None],
  [ 0xdae1, 'UnicodeCharacter0xdae1', [], 'Display', None],
  [ 0xdae2, 'UnicodeCharacter0xdae2', [], 'Display', None],
  [ 0xdae3, 'UnicodeCharacter0xdae3', [], 'Display', None],
  [ 0xdae4, 'UnicodeCharacter0xdae4', [], 'Display', None],
  [ 0xdae5, 'UnicodeCharacter0xdae5', [], 'Display', None],
  [ 0xdae6, 'UnicodeCharacter0xdae6', [], 'Display', None],
  [ 0xdae7, 'UnicodeCharacter0xdae7', [], 'Display', None],
  [ 0xdae8, 'UnicodeCharacter0xdae8', [], 'Display', None],
  [ 0xdae9, 'UnicodeCharacter0xdae9', [], 'Display', None],
  [ 0xdaea, 'UnicodeCharacter0xdaea', [], 'Display', None],
  [ 0xdaeb, 'UnicodeCharacter0xdaeb', [], 'Display', None],
  [ 0xdaec, 'UnicodeCharacter0xdaec', [], 'Display', None],
  [ 0xdaed, 'UnicodeCharacter0xdaed', [], 'Display', None],
  [ 0xdaee, 'UnicodeCharacter0xdaee', [], 'Display', None],
  [ 0xdaef, 'UnicodeCharacter0xdaef', [], 'Display', None],
  [ 0xdaf0, 'UnicodeCharacter0xdaf0', [], 'Display', None],
  [ 0xdaf1, 'UnicodeCharacter0xdaf1', [], 'Display', None],
  [ 0xdaf2, 'UnicodeCharacter0xdaf2', [], 'Display', None],
  [ 0xdaf3, 'UnicodeCharacter0xdaf3', [], 'Display', None],
  [ 0xdaf4, 'UnicodeCharacter0xdaf4', [], 'Display', None],
  [ 0xdaf5, 'UnicodeCharacter0xdaf5', [], 'Display', None],
  [ 0xdaf6, 'UnicodeCharacter0xdaf6', [], 'Display', None],
  [ 0xdaf7, 'UnicodeCharacter0xdaf7', [], 'Display', None],
  [ 0xdaf8, 'UnicodeCharacter0xdaf8', [], 'Display', None],
  [ 0xdaf9, 'UnicodeCharacter0xdaf9', [], 'Display', None],
  [ 0xdafa, 'UnicodeCharacter0xdafa', [], 'Display', None],
  [ 0xdafb, 'UnicodeCharacter0xdafb', [], 'Display', None],
  [ 0xdafc, 'UnicodeCharacter0xdafc', [], 'Display', None],
  [ 0xdafd, 'UnicodeCharacter0xdafd', [], 'Display', None],
  [ 0xdafe, 'UnicodeCharacter0xdafe', [], 'Display', None],
  [ 0xdaff, 'UnicodeCharacter0xdaff', [], 'Display', None],
  [ 0xdb00, 'UnicodeCharacter0xdb00', [], 'Display', None],
  [ 0xdb01, 'UnicodeCharacter0xdb01', [], 'Display', None],
  [ 0xdb02, 'UnicodeCharacter0xdb02', [], 'Display', None],
  [ 0xdb03, 'UnicodeCharacter0xdb03', [], 'Display', None],
  [ 0xdb04, 'UnicodeCharacter0xdb04', [], 'Display', None],
  [ 0xdb05, 'UnicodeCharacter0xdb05', [], 'Display', None],
  [ 0xdb06, 'UnicodeCharacter0xdb06', [], 'Display', None],
  [ 0xdb07, 'UnicodeCharacter0xdb07', [], 'Display', None],
  [ 0xdb08, 'UnicodeCharacter0xdb08', [], 'Display', None],
  [ 0xdb09, 'UnicodeCharacter0xdb09', [], 'Display', None],
  [ 0xdb0a, 'UnicodeCharacter0xdb0a', [], 'Display', None],
  [ 0xdb0b, 'UnicodeCharacter0xdb0b', [], 'Display', None],
  [ 0xdb0c, 'UnicodeCharacter0xdb0c', [], 'Display', None],
  [ 0xdb0d, 'UnicodeCharacter0xdb0d', [], 'Display', None],
  [ 0xdb0e, 'UnicodeCharacter0xdb0e', [], 'Display', None],
  [ 0xdb0f, 'UnicodeCharacter0xdb0f', [], 'Display', None],
  [ 0xdb10, 'UnicodeCharacter0xdb10', [], 'Display', None],
  [ 0xdb11, 'UnicodeCharacter0xdb11', [], 'Display', None],
  [ 0xdb12, 'UnicodeCharacter0xdb12', [], 'Display', None],
  [ 0xdb13, 'UnicodeCharacter0xdb13', [], 'Display', None],
  [ 0xdb14, 'UnicodeCharacter0xdb14', [], 'Display', None],
  [ 0xdb15, 'UnicodeCharacter0xdb15', [], 'Display', None],
  [ 0xdb16, 'UnicodeCharacter0xdb16', [], 'Display', None],
  [ 0xdb17, 'UnicodeCharacter0xdb17', [], 'Display', None],
  [ 0xdb18, 'UnicodeCharacter0xdb18', [], 'Display', None],
  [ 0xdb19, 'UnicodeCharacter0xdb19', [], 'Display', None],
  [ 0xdb1a, 'UnicodeCharacter0xdb1a', [], 'Display', None],
  [ 0xdb1b, 'UnicodeCharacter0xdb1b', [], 'Display', None],
  [ 0xdb1c, 'UnicodeCharacter0xdb1c', [], 'Display', None],
  [ 0xdb1d, 'UnicodeCharacter0xdb1d', [], 'Display', None],
  [ 0xdb1e, 'UnicodeCharacter0xdb1e', [], 'Display', None],
  [ 0xdb1f, 'UnicodeCharacter0xdb1f', [], 'Display', None],
  [ 0xdb20, 'UnicodeCharacter0xdb20', [], 'Display', None],
  [ 0xdb21, 'UnicodeCharacter0xdb21', [], 'Display', None],
  [ 0xdb22, 'UnicodeCharacter0xdb22', [], 'Display', None],
  [ 0xdb23, 'UnicodeCharacter0xdb23', [], 'Display', None],
  [ 0xdb24, 'UnicodeCharacter0xdb24', [], 'Display', None],
  [ 0xdb25, 'UnicodeCharacter0xdb25', [], 'Display', None],
  [ 0xdb26, 'UnicodeCharacter0xdb26', [], 'Display', None],
  [ 0xdb27, 'UnicodeCharacter0xdb27', [], 'Display', None],
  [ 0xdb28, 'UnicodeCharacter0xdb28', [], 'Display', None],
  [ 0xdb29, 'UnicodeCharacter0xdb29', [], 'Display', None],
  [ 0xdb2a, 'UnicodeCharacter0xdb2a', [], 'Display', None],
  [ 0xdb2b, 'UnicodeCharacter0xdb2b', [], 'Display', None],
  [ 0xdb2c, 'UnicodeCharacter0xdb2c', [], 'Display', None],
  [ 0xdb2d, 'UnicodeCharacter0xdb2d', [], 'Display', None],
  [ 0xdb2e, 'UnicodeCharacter0xdb2e', [], 'Display', None],
  [ 0xdb2f, 'UnicodeCharacter0xdb2f', [], 'Display', None],
  [ 0xdb30, 'UnicodeCharacter0xdb30', [], 'Display', None],
  [ 0xdb31, 'UnicodeCharacter0xdb31', [], 'Display', None],
  [ 0xdb32, 'UnicodeCharacter0xdb32', [], 'Display', None],
  [ 0xdb33, 'UnicodeCharacter0xdb33', [], 'Display', None],
  [ 0xdb34, 'UnicodeCharacter0xdb34', [], 'Display', None],
  [ 0xdb35, 'UnicodeCharacter0xdb35', [], 'Display', None],
  [ 0xdb36, 'UnicodeCharacter0xdb36', [], 'Display', None],
  [ 0xdb37, 'UnicodeCharacter0xdb37', [], 'Display', None],
  [ 0xdb38, 'UnicodeCharacter0xdb38', [], 'Display', None],
  [ 0xdb39, 'UnicodeCharacter0xdb39', [], 'Display', None],
  [ 0xdb3a, 'UnicodeCharacter0xdb3a', [], 'Display', None],
  [ 0xdb3b, 'UnicodeCharacter0xdb3b', [], 'Display', None],
  [ 0xdb3c, 'UnicodeCharacter0xdb3c', [], 'Display', None],
  [ 0xdb3d, 'UnicodeCharacter0xdb3d', [], 'Display', None],
  [ 0xdb3e, 'UnicodeCharacter0xdb3e', [], 'Display', None],
  [ 0xdb3f, 'UnicodeCharacter0xdb3f', [], 'Display', None],
  [ 0xdb40, 'UnicodeCharacter0xdb40', [], 'Display', None],
  [ 0xdb41, 'UnicodeCharacter0xdb41', [], 'Display', None],
  [ 0xdb42, 'UnicodeCharacter0xdb42', [], 'Display', None],
  [ 0xdb43, 'UnicodeCharacter0xdb43', [], 'Display', None],
  [ 0xdb44, 'UnicodeCharacter0xdb44', [], 'Display', None],
  [ 0xdb45, 'UnicodeCharacter0xdb45', [], 'Display', None],
  [ 0xdb46, 'UnicodeCharacter0xdb46', [], 'Display', None],
  [ 0xdb47, 'UnicodeCharacter0xdb47', [], 'Display', None],
  [ 0xdb48, 'UnicodeCharacter0xdb48', [], 'Display', None],
  [ 0xdb49, 'UnicodeCharacter0xdb49', [], 'Display', None],
  [ 0xdb4a, 'UnicodeCharacter0xdb4a', [], 'Display', None],
  [ 0xdb4b, 'UnicodeCharacter0xdb4b', [], 'Display', None],
  [ 0xdb4c, 'UnicodeCharacter0xdb4c', [], 'Display', None],
  [ 0xdb4d, 'UnicodeCharacter0xdb4d', [], 'Display', None],
  [ 0xdb4e, 'UnicodeCharacter0xdb4e', [], 'Display', None],
  [ 0xdb4f, 'UnicodeCharacter0xdb4f', [], 'Display', None],
  [ 0xdb50, 'UnicodeCharacter0xdb50', [], 'Display', None],
  [ 0xdb51, 'UnicodeCharacter0xdb51', [], 'Display', None],
  [ 0xdb52, 'UnicodeCharacter0xdb52', [], 'Display', None],
  [ 0xdb53, 'UnicodeCharacter0xdb53', [], 'Display', None],
  [ 0xdb54, 'UnicodeCharacter0xdb54', [], 'Display', None],
  [ 0xdb55, 'UnicodeCharacter0xdb55', [], 'Display', None],
  [ 0xdb56, 'UnicodeCharacter0xdb56', [], 'Display', None],
  [ 0xdb57, 'UnicodeCharacter0xdb57', [], 'Display', None],
  [ 0xdb58, 'UnicodeCharacter0xdb58', [], 'Display', None],
  [ 0xdb59, 'UnicodeCharacter0xdb59', [], 'Display', None],
  [ 0xdb5a, 'UnicodeCharacter0xdb5a', [], 'Display', None],
  [ 0xdb5b, 'UnicodeCharacter0xdb5b', [], 'Display', None],
  [ 0xdb5c, 'UnicodeCharacter0xdb5c', [], 'Display', None],
  [ 0xdb5d, 'UnicodeCharacter0xdb5d', [], 'Display', None],
  [ 0xdb5e, 'UnicodeCharacter0xdb5e', [], 'Display', None],
  [ 0xdb5f, 'UnicodeCharacter0xdb5f', [], 'Display', None],
  [ 0xdb60, 'UnicodeCharacter0xdb60', [], 'Display', None],
  [ 0xdb61, 'UnicodeCharacter0xdb61', [], 'Display', None],
  [ 0xdb62, 'UnicodeCharacter0xdb62', [], 'Display', None],
  [ 0xdb63, 'UnicodeCharacter0xdb63', [], 'Display', None],
  [ 0xdb64, 'UnicodeCharacter0xdb64', [], 'Display', None],
  [ 0xdb65, 'UnicodeCharacter0xdb65', [], 'Display', None],
  [ 0xdb66, 'UnicodeCharacter0xdb66', [], 'Display', None],
  [ 0xdb67, 'UnicodeCharacter0xdb67', [], 'Display', None],
  [ 0xdb68, 'UnicodeCharacter0xdb68', [], 'Display', None],
  [ 0xdb69, 'UnicodeCharacter0xdb69', [], 'Display', None],
  [ 0xdb6a, 'UnicodeCharacter0xdb6a', [], 'Display', None],
  [ 0xdb6b, 'UnicodeCharacter0xdb6b', [], 'Display', None],
  [ 0xdb6c, 'UnicodeCharacter0xdb6c', [], 'Display', None],
  [ 0xdb6d, 'UnicodeCharacter0xdb6d', [], 'Display', None],
  [ 0xdb6e, 'UnicodeCharacter0xdb6e', [], 'Display', None],
  [ 0xdb6f, 'UnicodeCharacter0xdb6f', [], 'Display', None],
  [ 0xdb70, 'UnicodeCharacter0xdb70', [], 'Display', None],
  [ 0xdb71, 'UnicodeCharacter0xdb71', [], 'Display', None],
  [ 0xdb72, 'UnicodeCharacter0xdb72', [], 'Display', None],
  [ 0xdb73, 'UnicodeCharacter0xdb73', [], 'Display', None],
  [ 0xdb74, 'UnicodeCharacter0xdb74', [], 'Display', None],
  [ 0xdb75, 'UnicodeCharacter0xdb75', [], 'Display', None],
  [ 0xdb76, 'UnicodeCharacter0xdb76', [], 'Display', None],
  [ 0xdb77, 'UnicodeCharacter0xdb77', [], 'Display', None],
  [ 0xdb78, 'UnicodeCharacter0xdb78', [], 'Display', None],
  [ 0xdb79, 'UnicodeCharacter0xdb79', [], 'Display', None],
  [ 0xdb7a, 'UnicodeCharacter0xdb7a', [], 'Display', None],
  [ 0xdb7b, 'UnicodeCharacter0xdb7b', [], 'Display', None],
  [ 0xdb7c, 'UnicodeCharacter0xdb7c', [], 'Display', None],
  [ 0xdb7d, 'UnicodeCharacter0xdb7d', [], 'Display', None],
  [ 0xdb7e, 'UnicodeCharacter0xdb7e', [], 'Display', None],
  [ 0xdb7f, 'UnicodeCharacter0xdb7f', [], 'Display', None],
  [ 0xdb80, 'UnicodeCharacter0xdb80', [], 'Display', None],
  [ 0xdb81, 'UnicodeCharacter0xdb81', [], 'Display', None],
  [ 0xdb82, 'UnicodeCharacter0xdb82', [], 'Display', None],
  [ 0xdb83, 'UnicodeCharacter0xdb83', [], 'Display', None],
  [ 0xdb84, 'UnicodeCharacter0xdb84', [], 'Display', None],
  [ 0xdb85, 'UnicodeCharacter0xdb85', [], 'Display', None],
  [ 0xdb86, 'UnicodeCharacter0xdb86', [], 'Display', None],
  [ 0xdb87, 'UnicodeCharacter0xdb87', [], 'Display', None],
  [ 0xdb88, 'UnicodeCharacter0xdb88', [], 'Display', None],
  [ 0xdb89, 'UnicodeCharacter0xdb89', [], 'Display', None],
  [ 0xdb8a, 'UnicodeCharacter0xdb8a', [], 'Display', None],
  [ 0xdb8b, 'UnicodeCharacter0xdb8b', [], 'Display', None],
  [ 0xdb8c, 'UnicodeCharacter0xdb8c', [], 'Display', None],
  [ 0xdb8d, 'UnicodeCharacter0xdb8d', [], 'Display', None],
  [ 0xdb8e, 'UnicodeCharacter0xdb8e', [], 'Display', None],
  [ 0xdb8f, 'UnicodeCharacter0xdb8f', [], 'Display', None],
  [ 0xdb90, 'UnicodeCharacter0xdb90', [], 'Display', None],
  [ 0xdb91, 'UnicodeCharacter0xdb91', [], 'Display', None],
  [ 0xdb92, 'UnicodeCharacter0xdb92', [], 'Display', None],
  [ 0xdb93, 'UnicodeCharacter0xdb93', [], 'Display', None],
  [ 0xdb94, 'UnicodeCharacter0xdb94', [], 'Display', None],
  [ 0xdb95, 'UnicodeCharacter0xdb95', [], 'Display', None],
  [ 0xdb96, 'UnicodeCharacter0xdb96', [], 'Display', None],
  [ 0xdb97, 'UnicodeCharacter0xdb97', [], 'Display', None],
  [ 0xdb98, 'UnicodeCharacter0xdb98', [], 'Display', None],
  [ 0xdb99, 'UnicodeCharacter0xdb99', [], 'Display', None],
  [ 0xdb9a, 'UnicodeCharacter0xdb9a', [], 'Display', None],
  [ 0xdb9b, 'UnicodeCharacter0xdb9b', [], 'Display', None],
  [ 0xdb9c, 'UnicodeCharacter0xdb9c', [], 'Display', None],
  [ 0xdb9d, 'UnicodeCharacter0xdb9d', [], 'Display', None],
  [ 0xdb9e, 'UnicodeCharacter0xdb9e', [], 'Display', None],
  [ 0xdb9f, 'UnicodeCharacter0xdb9f', [], 'Display', None],
  [ 0xdba0, 'UnicodeCharacter0xdba0', [], 'Display', None],
  [ 0xdba1, 'UnicodeCharacter0xdba1', [], 'Display', None],
  [ 0xdba2, 'UnicodeCharacter0xdba2', [], 'Display', None],
  [ 0xdba3, 'UnicodeCharacter0xdba3', [], 'Display', None],
  [ 0xdba4, 'UnicodeCharacter0xdba4', [], 'Display', None],
  [ 0xdba5, 'UnicodeCharacter0xdba5', [], 'Display', None],
  [ 0xdba6, 'UnicodeCharacter0xdba6', [], 'Display', None],
  [ 0xdba7, 'UnicodeCharacter0xdba7', [], 'Display', None],
  [ 0xdba8, 'UnicodeCharacter0xdba8', [], 'Display', None],
  [ 0xdba9, 'UnicodeCharacter0xdba9', [], 'Display', None],
  [ 0xdbaa, 'UnicodeCharacter0xdbaa', [], 'Display', None],
  [ 0xdbab, 'UnicodeCharacter0xdbab', [], 'Display', None],
  [ 0xdbac, 'UnicodeCharacter0xdbac', [], 'Display', None],
  [ 0xdbad, 'UnicodeCharacter0xdbad', [], 'Display', None],
  [ 0xdbae, 'UnicodeCharacter0xdbae', [], 'Display', None],
  [ 0xdbaf, 'UnicodeCharacter0xdbaf', [], 'Display', None],
  [ 0xdbb0, 'UnicodeCharacter0xdbb0', [], 'Display', None],
  [ 0xdbb1, 'UnicodeCharacter0xdbb1', [], 'Display', None],
  [ 0xdbb2, 'UnicodeCharacter0xdbb2', [], 'Display', None],
  [ 0xdbb3, 'UnicodeCharacter0xdbb3', [], 'Display', None],
  [ 0xdbb4, 'UnicodeCharacter0xdbb4', [], 'Display', None],
  [ 0xdbb5, 'UnicodeCharacter0xdbb5', [], 'Display', None],
  [ 0xdbb6, 'UnicodeCharacter0xdbb6', [], 'Display', None],
  [ 0xdbb7, 'UnicodeCharacter0xdbb7', [], 'Display', None],
  [ 0xdbb8, 'UnicodeCharacter0xdbb8', [], 'Display', None],
  [ 0xdbb9, 'UnicodeCharacter0xdbb9', [], 'Display', None],
  [ 0xdbba, 'UnicodeCharacter0xdbba', [], 'Display', None],
  [ 0xdbbb, 'UnicodeCharacter0xdbbb', [], 'Display', None],
  [ 0xdbbc, 'UnicodeCharacter0xdbbc', [], 'Display', None],
  [ 0xdbbd, 'UnicodeCharacter0xdbbd', [], 'Display', None],
  [ 0xdbbe, 'UnicodeCharacter0xdbbe', [], 'Display', None],
  [ 0xdbbf, 'UnicodeCharacter0xdbbf', [], 'Display', None],
  [ 0xdbc0, 'UnicodeCharacter0xdbc0', [], 'Display', None],
  [ 0xdbc1, 'UnicodeCharacter0xdbc1', [], 'Display', None],
  [ 0xdbc2, 'UnicodeCharacter0xdbc2', [], 'Display', None],
  [ 0xdbc3, 'UnicodeCharacter0xdbc3', [], 'Display', None],
  [ 0xdbc4, 'UnicodeCharacter0xdbc4', [], 'Display', None],
  [ 0xdbc5, 'UnicodeCharacter0xdbc5', [], 'Display', None],
  [ 0xdbc6, 'UnicodeCharacter0xdbc6', [], 'Display', None],
  [ 0xdbc7, 'UnicodeCharacter0xdbc7', [], 'Display', None],
  [ 0xdbc8, 'UnicodeCharacter0xdbc8', [], 'Display', None],
  [ 0xdbc9, 'UnicodeCharacter0xdbc9', [], 'Display', None],
  [ 0xdbca, 'UnicodeCharacter0xdbca', [], 'Display', None],
  [ 0xdbcb, 'UnicodeCharacter0xdbcb', [], 'Display', None],
  [ 0xdbcc, 'UnicodeCharacter0xdbcc', [], 'Display', None],
  [ 0xdbcd, 'UnicodeCharacter0xdbcd', [], 'Display', None],
  [ 0xdbce, 'UnicodeCharacter0xdbce', [], 'Display', None],
  [ 0xdbcf, 'UnicodeCharacter0xdbcf', [], 'Display', None],
  [ 0xdbd0, 'UnicodeCharacter0xdbd0', [], 'Display', None],
  [ 0xdbd1, 'UnicodeCharacter0xdbd1', [], 'Display', None],
  [ 0xdbd2, 'UnicodeCharacter0xdbd2', [], 'Display', None],
  [ 0xdbd3, 'UnicodeCharacter0xdbd3', [], 'Display', None],
  [ 0xdbd4, 'UnicodeCharacter0xdbd4', [], 'Display', None],
  [ 0xdbd5, 'UnicodeCharacter0xdbd5', [], 'Display', None],
  [ 0xdbd6, 'UnicodeCharacter0xdbd6', [], 'Display', None],
  [ 0xdbd7, 'UnicodeCharacter0xdbd7', [], 'Display', None],
  [ 0xdbd8, 'UnicodeCharacter0xdbd8', [], 'Display', None],
  [ 0xdbd9, 'UnicodeCharacter0xdbd9', [], 'Display', None],
  [ 0xdbda, 'UnicodeCharacter0xdbda', [], 'Display', None],
  [ 0xdbdb, 'UnicodeCharacter0xdbdb', [], 'Display', None],
  [ 0xdbdc, 'UnicodeCharacter0xdbdc', [], 'Display', None],
  [ 0xdbdd, 'UnicodeCharacter0xdbdd', [], 'Display', None],
  [ 0xdbde, 'UnicodeCharacter0xdbde', [], 'Display', None],
  [ 0xdbdf, 'UnicodeCharacter0xdbdf', [], 'Display', None],
  [ 0xdbe0, 'UnicodeCharacter0xdbe0', [], 'Display', None],
  [ 0xdbe1, 'UnicodeCharacter0xdbe1', [], 'Display', None],
  [ 0xdbe2, 'UnicodeCharacter0xdbe2', [], 'Display', None],
  [ 0xdbe3, 'UnicodeCharacter0xdbe3', [], 'Display', None],
  [ 0xdbe4, 'UnicodeCharacter0xdbe4', [], 'Display', None],
  [ 0xdbe5, 'UnicodeCharacter0xdbe5', [], 'Display', None],
  [ 0xdbe6, 'UnicodeCharacter0xdbe6', [], 'Display', None],
  [ 0xdbe7, 'UnicodeCharacter0xdbe7', [], 'Display', None],
  [ 0xdbe8, 'UnicodeCharacter0xdbe8', [], 'Display', None],
  [ 0xdbe9, 'UnicodeCharacter0xdbe9', [], 'Display', None],
  [ 0xdbea, 'UnicodeCharacter0xdbea', [], 'Display', None],
  [ 0xdbeb, 'UnicodeCharacter0xdbeb', [], 'Display', None],
  [ 0xdbec, 'UnicodeCharacter0xdbec', [], 'Display', None],
  [ 0xdbed, 'UnicodeCharacter0xdbed', [], 'Display', None],
  [ 0xdbee, 'UnicodeCharacter0xdbee', [], 'Display', None],
  [ 0xdbef, 'UnicodeCharacter0xdbef', [], 'Display', None],
  [ 0xdbf0, 'UnicodeCharacter0xdbf0', [], 'Display', None],
  [ 0xdbf1, 'UnicodeCharacter0xdbf1', [], 'Display', None],
  [ 0xdbf2, 'UnicodeCharacter0xdbf2', [], 'Display', None],
  [ 0xdbf3, 'UnicodeCharacter0xdbf3', [], 'Display', None],
  [ 0xdbf4, 'UnicodeCharacter0xdbf4', [], 'Display', None],
  [ 0xdbf5, 'UnicodeCharacter0xdbf5', [], 'Display', None],
  [ 0xdbf6, 'UnicodeCharacter0xdbf6', [], 'Display', None],
  [ 0xdbf7, 'UnicodeCharacter0xdbf7', [], 'Display', None],
  [ 0xdbf8, 'UnicodeCharacter0xdbf8', [], 'Display', None],
  [ 0xdbf9, 'UnicodeCharacter0xdbf9', [], 'Display', None],
  [ 0xdbfa, 'UnicodeCharacter0xdbfa', [], 'Display', None],
  [ 0xdbfb, 'UnicodeCharacter0xdbfb', [], 'Display', None],
  [ 0xdbfc, 'UnicodeCharacter0xdbfc', [], 'Display', None],
  [ 0xdbfd, 'UnicodeCharacter0xdbfd', [], 'Display', None],
  [ 0xdbfe, 'UnicodeCharacter0xdbfe', [], 'Display', None],
  [ 0xdbff, 'UnicodeCharacter0xdbff', [], 'Display', None],
  [ 0xdc00, 'UnicodeCharacter0xdc00', [], 'Display', None],
  [ 0xdc01, 'UnicodeCharacter0xdc01', [], 'Display', None],
  [ 0xdc02, 'UnicodeCharacter0xdc02', [], 'Display', None],
  [ 0xdc03, 'UnicodeCharacter0xdc03', [], 'Display', None],
  [ 0xdc04, 'UnicodeCharacter0xdc04', [], 'Display', None],
  [ 0xdc05, 'UnicodeCharacter0xdc05', [], 'Display', None],
  [ 0xdc06, 'UnicodeCharacter0xdc06', [], 'Display', None],
  [ 0xdc07, 'UnicodeCharacter0xdc07', [], 'Display', None],
  [ 0xdc08, 'UnicodeCharacter0xdc08', [], 'Display', None],
  [ 0xdc09, 'UnicodeCharacter0xdc09', [], 'Display', None],
  [ 0xdc0a, 'UnicodeCharacter0xdc0a', [], 'Display', None],
  [ 0xdc0b, 'UnicodeCharacter0xdc0b', [], 'Display', None],
  [ 0xdc0c, 'UnicodeCharacter0xdc0c', [], 'Display', None],
  [ 0xdc0d, 'UnicodeCharacter0xdc0d', [], 'Display', None],
  [ 0xdc0e, 'UnicodeCharacter0xdc0e', [], 'Display', None],
  [ 0xdc0f, 'UnicodeCharacter0xdc0f', [], 'Display', None],
  [ 0xdc10, 'UnicodeCharacter0xdc10', [], 'Display', None],
  [ 0xdc11, 'UnicodeCharacter0xdc11', [], 'Display', None],
  [ 0xdc12, 'UnicodeCharacter0xdc12', [], 'Display', None],
  [ 0xdc13, 'UnicodeCharacter0xdc13', [], 'Display', None],
  [ 0xdc14, 'UnicodeCharacter0xdc14', [], 'Display', None],
  [ 0xdc15, 'UnicodeCharacter0xdc15', [], 'Display', None],
  [ 0xdc16, 'UnicodeCharacter0xdc16', [], 'Display', None],
  [ 0xdc17, 'UnicodeCharacter0xdc17', [], 'Display', None],
  [ 0xdc18, 'UnicodeCharacter0xdc18', [], 'Display', None],
  [ 0xdc19, 'UnicodeCharacter0xdc19', [], 'Display', None],
  [ 0xdc1a, 'UnicodeCharacter0xdc1a', [], 'Display', None],
  [ 0xdc1b, 'UnicodeCharacter0xdc1b', [], 'Display', None],
  [ 0xdc1c, 'UnicodeCharacter0xdc1c', [], 'Display', None],
  [ 0xdc1d, 'UnicodeCharacter0xdc1d', [], 'Display', None],
  [ 0xdc1e, 'UnicodeCharacter0xdc1e', [], 'Display', None],
  [ 0xdc1f, 'UnicodeCharacter0xdc1f', [], 'Display', None],
  [ 0xdc20, 'UnicodeCharacter0xdc20', [], 'Display', None],
  [ 0xdc21, 'UnicodeCharacter0xdc21', [], 'Display', None],
  [ 0xdc22, 'UnicodeCharacter0xdc22', [], 'Display', None],
  [ 0xdc23, 'UnicodeCharacter0xdc23', [], 'Display', None],
  [ 0xdc24, 'UnicodeCharacter0xdc24', [], 'Display', None],
  [ 0xdc25, 'UnicodeCharacter0xdc25', [], 'Display', None],
  [ 0xdc26, 'UnicodeCharacter0xdc26', [], 'Display', None],
  [ 0xdc27, 'UnicodeCharacter0xdc27', [], 'Display', None],
  [ 0xdc28, 'UnicodeCharacter0xdc28', [], 'Display', None],
  [ 0xdc29, 'UnicodeCharacter0xdc29', [], 'Display', None],
  [ 0xdc2a, 'UnicodeCharacter0xdc2a', [], 'Display', None],
  [ 0xdc2b, 'UnicodeCharacter0xdc2b', [], 'Display', None],
  [ 0xdc2c, 'UnicodeCharacter0xdc2c', [], 'Display', None],
  [ 0xdc2d, 'UnicodeCharacter0xdc2d', [], 'Display', None],
  [ 0xdc2e, 'UnicodeCharacter0xdc2e', [], 'Display', None],
  [ 0xdc2f, 'UnicodeCharacter0xdc2f', [], 'Display', None],
  [ 0xdc30, 'UnicodeCharacter0xdc30', [], 'Display', None],
  [ 0xdc31, 'UnicodeCharacter0xdc31', [], 'Display', None],
  [ 0xdc32, 'UnicodeCharacter0xdc32', [], 'Display', None],
  [ 0xdc33, 'UnicodeCharacter0xdc33', [], 'Display', None],
  [ 0xdc34, 'UnicodeCharacter0xdc34', [], 'Display', None],
  [ 0xdc35, 'UnicodeCharacter0xdc35', [], 'Display', None],
  [ 0xdc36, 'UnicodeCharacter0xdc36', [], 'Display', None],
  [ 0xdc37, 'UnicodeCharacter0xdc37', [], 'Display', None],
  [ 0xdc38, 'UnicodeCharacter0xdc38', [], 'Display', None],
  [ 0xdc39, 'UnicodeCharacter0xdc39', [], 'Display', None],
  [ 0xdc3a, 'UnicodeCharacter0xdc3a', [], 'Display', None],
  [ 0xdc3b, 'UnicodeCharacter0xdc3b', [], 'Display', None],
  [ 0xdc3c, 'UnicodeCharacter0xdc3c', [], 'Display', None],
  [ 0xdc3d, 'UnicodeCharacter0xdc3d', [], 'Display', None],
  [ 0xdc3e, 'UnicodeCharacter0xdc3e', [], 'Display', None],
  [ 0xdc3f, 'UnicodeCharacter0xdc3f', [], 'Display', None],
  [ 0xdc40, 'UnicodeCharacter0xdc40', [], 'Display', None],
  [ 0xdc41, 'UnicodeCharacter0xdc41', [], 'Display', None],
  [ 0xdc42, 'UnicodeCharacter0xdc42', [], 'Display', None],
  [ 0xdc43, 'UnicodeCharacter0xdc43', [], 'Display', None],
  [ 0xdc44, 'UnicodeCharacter0xdc44', [], 'Display', None],
  [ 0xdc45, 'UnicodeCharacter0xdc45', [], 'Display', None],
  [ 0xdc46, 'UnicodeCharacter0xdc46', [], 'Display', None],
  [ 0xdc47, 'UnicodeCharacter0xdc47', [], 'Display', None],
  [ 0xdc48, 'UnicodeCharacter0xdc48', [], 'Display', None],
  [ 0xdc49, 'UnicodeCharacter0xdc49', [], 'Display', None],
  [ 0xdc4a, 'UnicodeCharacter0xdc4a', [], 'Display', None],
  [ 0xdc4b, 'UnicodeCharacter0xdc4b', [], 'Display', None],
  [ 0xdc4c, 'UnicodeCharacter0xdc4c', [], 'Display', None],
  [ 0xdc4d, 'UnicodeCharacter0xdc4d', [], 'Display', None],
  [ 0xdc4e, 'UnicodeCharacter0xdc4e', [], 'Display', None],
  [ 0xdc4f, 'UnicodeCharacter0xdc4f', [], 'Display', None],
  [ 0xdc50, 'UnicodeCharacter0xdc50', [], 'Display', None],
  [ 0xdc51, 'UnicodeCharacter0xdc51', [], 'Display', None],
  [ 0xdc52, 'UnicodeCharacter0xdc52', [], 'Display', None],
  [ 0xdc53, 'UnicodeCharacter0xdc53', [], 'Display', None],
  [ 0xdc54, 'UnicodeCharacter0xdc54', [], 'Display', None],
  [ 0xdc55, 'UnicodeCharacter0xdc55', [], 'Display', None],
  [ 0xdc56, 'UnicodeCharacter0xdc56', [], 'Display', None],
  [ 0xdc57, 'UnicodeCharacter0xdc57', [], 'Display', None],
  [ 0xdc58, 'UnicodeCharacter0xdc58', [], 'Display', None],
  [ 0xdc59, 'UnicodeCharacter0xdc59', [], 'Display', None],
  [ 0xdc5a, 'UnicodeCharacter0xdc5a', [], 'Display', None],
  [ 0xdc5b, 'UnicodeCharacter0xdc5b', [], 'Display', None],
  [ 0xdc5c, 'UnicodeCharacter0xdc5c', [], 'Display', None],
  [ 0xdc5d, 'UnicodeCharacter0xdc5d', [], 'Display', None],
  [ 0xdc5e, 'UnicodeCharacter0xdc5e', [], 'Display', None],
  [ 0xdc5f, 'UnicodeCharacter0xdc5f', [], 'Display', None],
  [ 0xdc60, 'UnicodeCharacter0xdc60', [], 'Display', None],
  [ 0xdc61, 'UnicodeCharacter0xdc61', [], 'Display', None],
  [ 0xdc62, 'UnicodeCharacter0xdc62', [], 'Display', None],
  [ 0xdc63, 'UnicodeCharacter0xdc63', [], 'Display', None],
  [ 0xdc64, 'UnicodeCharacter0xdc64', [], 'Display', None],
  [ 0xdc65, 'UnicodeCharacter0xdc65', [], 'Display', None],
  [ 0xdc66, 'UnicodeCharacter0xdc66', [], 'Display', None],
  [ 0xdc67, 'UnicodeCharacter0xdc67', [], 'Display', None],
  [ 0xdc68, 'UnicodeCharacter0xdc68', [], 'Display', None],
  [ 0xdc69, 'UnicodeCharacter0xdc69', [], 'Display', None],
  [ 0xdc6a, 'UnicodeCharacter0xdc6a', [], 'Display', None],
  [ 0xdc6b, 'UnicodeCharacter0xdc6b', [], 'Display', None],
  [ 0xdc6c, 'UnicodeCharacter0xdc6c', [], 'Display', None],
  [ 0xdc6d, 'UnicodeCharacter0xdc6d', [], 'Display', None],
  [ 0xdc6e, 'UnicodeCharacter0xdc6e', [], 'Display', None],
  [ 0xdc6f, 'UnicodeCharacter0xdc6f', [], 'Display', None],
  [ 0xdc70, 'UnicodeCharacter0xdc70', [], 'Display', None],
  [ 0xdc71, 'UnicodeCharacter0xdc71', [], 'Display', None],
  [ 0xdc72, 'UnicodeCharacter0xdc72', [], 'Display', None],
  [ 0xdc73, 'UnicodeCharacter0xdc73', [], 'Display', None],
  [ 0xdc74, 'UnicodeCharacter0xdc74', [], 'Display', None],
  [ 0xdc75, 'UnicodeCharacter0xdc75', [], 'Display', None],
  [ 0xdc76, 'UnicodeCharacter0xdc76', [], 'Display', None],
  [ 0xdc77, 'UnicodeCharacter0xdc77', [], 'Display', None],
  [ 0xdc78, 'UnicodeCharacter0xdc78', [], 'Display', None],
  [ 0xdc79, 'UnicodeCharacter0xdc79', [], 'Display', None],
  [ 0xdc7a, 'UnicodeCharacter0xdc7a', [], 'Display', None],
  [ 0xdc7b, 'UnicodeCharacter0xdc7b', [], 'Display', None],
  [ 0xdc7c, 'UnicodeCharacter0xdc7c', [], 'Display', None],
  [ 0xdc7d, 'UnicodeCharacter0xdc7d', [], 'Display', None],
  [ 0xdc7e, 'UnicodeCharacter0xdc7e', [], 'Display', None],
  [ 0xdc7f, 'UnicodeCharacter0xdc7f', [], 'Display', None],
  [ 0xdc80, 'UnicodeCharacter0xdc80', [], 'Display', None],
  [ 0xdc81, 'UnicodeCharacter0xdc81', [], 'Display', None],
  [ 0xdc82, 'UnicodeCharacter0xdc82', [], 'Display', None],
  [ 0xdc83, 'UnicodeCharacter0xdc83', [], 'Display', None],
  [ 0xdc84, 'UnicodeCharacter0xdc84', [], 'Display', None],
  [ 0xdc85, 'UnicodeCharacter0xdc85', [], 'Display', None],
  [ 0xdc86, 'UnicodeCharacter0xdc86', [], 'Display', None],
  [ 0xdc87, 'UnicodeCharacter0xdc87', [], 'Display', None],
  [ 0xdc88, 'UnicodeCharacter0xdc88', [], 'Display', None],
  [ 0xdc89, 'UnicodeCharacter0xdc89', [], 'Display', None],
  [ 0xdc8a, 'UnicodeCharacter0xdc8a', [], 'Display', None],
  [ 0xdc8b, 'UnicodeCharacter0xdc8b', [], 'Display', None],
  [ 0xdc8c, 'UnicodeCharacter0xdc8c', [], 'Display', None],
  [ 0xdc8d, 'UnicodeCharacter0xdc8d', [], 'Display', None],
  [ 0xdc8e, 'UnicodeCharacter0xdc8e', [], 'Display', None],
  [ 0xdc8f, 'UnicodeCharacter0xdc8f', [], 'Display', None],
  [ 0xdc90, 'UnicodeCharacter0xdc90', [], 'Display', None],
  [ 0xdc91, 'UnicodeCharacter0xdc91', [], 'Display', None],
  [ 0xdc92, 'UnicodeCharacter0xdc92', [], 'Display', None],
  [ 0xdc93, 'UnicodeCharacter0xdc93', [], 'Display', None],
  [ 0xdc94, 'UnicodeCharacter0xdc94', [], 'Display', None],
  [ 0xdc95, 'UnicodeCharacter0xdc95', [], 'Display', None],
  [ 0xdc96, 'UnicodeCharacter0xdc96', [], 'Display', None],
  [ 0xdc97, 'UnicodeCharacter0xdc97', [], 'Display', None],
  [ 0xdc98, 'UnicodeCharacter0xdc98', [], 'Display', None],
  [ 0xdc99, 'UnicodeCharacter0xdc99', [], 'Display', None],
  [ 0xdc9a, 'UnicodeCharacter0xdc9a', [], 'Display', None],
  [ 0xdc9b, 'UnicodeCharacter0xdc9b', [], 'Display', None],
  [ 0xdc9c, 'UnicodeCharacter0xdc9c', [], 'Display', None],
  [ 0xdc9d, 'UnicodeCharacter0xdc9d', [], 'Display', None],
  [ 0xdc9e, 'UnicodeCharacter0xdc9e', [], 'Display', None],
  [ 0xdc9f, 'UnicodeCharacter0xdc9f', [], 'Display', None],
  [ 0xdca0, 'UnicodeCharacter0xdca0', [], 'Display', None],
  [ 0xdca1, 'UnicodeCharacter0xdca1', [], 'Display', None],
  [ 0xdca2, 'UnicodeCharacter0xdca2', [], 'Display', None],
  [ 0xdca3, 'UnicodeCharacter0xdca3', [], 'Display', None],
  [ 0xdca4, 'UnicodeCharacter0xdca4', [], 'Display', None],
  [ 0xdca5, 'UnicodeCharacter0xdca5', [], 'Display', None],
  [ 0xdca6, 'UnicodeCharacter0xdca6', [], 'Display', None],
  [ 0xdca7, 'UnicodeCharacter0xdca7', [], 'Display', None],
  [ 0xdca8, 'UnicodeCharacter0xdca8', [], 'Display', None],
  [ 0xdca9, 'UnicodeCharacter0xdca9', [], 'Display', None],
  [ 0xdcaa, 'UnicodeCharacter0xdcaa', [], 'Display', None],
  [ 0xdcab, 'UnicodeCharacter0xdcab', [], 'Display', None],
  [ 0xdcac, 'UnicodeCharacter0xdcac', [], 'Display', None],
  [ 0xdcad, 'UnicodeCharacter0xdcad', [], 'Display', None],
  [ 0xdcae, 'UnicodeCharacter0xdcae', [], 'Display', None],
  [ 0xdcaf, 'UnicodeCharacter0xdcaf', [], 'Display', None],
  [ 0xdcb0, 'UnicodeCharacter0xdcb0', [], 'Display', None],
  [ 0xdcb1, 'UnicodeCharacter0xdcb1', [], 'Display', None],
  [ 0xdcb2, 'UnicodeCharacter0xdcb2', [], 'Display', None],
  [ 0xdcb3, 'UnicodeCharacter0xdcb3', [], 'Display', None],
  [ 0xdcb4, 'UnicodeCharacter0xdcb4', [], 'Display', None],
  [ 0xdcb5, 'UnicodeCharacter0xdcb5', [], 'Display', None],
  [ 0xdcb6, 'UnicodeCharacter0xdcb6', [], 'Display', None],
  [ 0xdcb7, 'UnicodeCharacter0xdcb7', [], 'Display', None],
  [ 0xdcb8, 'UnicodeCharacter0xdcb8', [], 'Display', None],
  [ 0xdcb9, 'UnicodeCharacter0xdcb9', [], 'Display', None],
  [ 0xdcba, 'UnicodeCharacter0xdcba', [], 'Display', None],
  [ 0xdcbb, 'UnicodeCharacter0xdcbb', [], 'Display', None],
  [ 0xdcbc, 'UnicodeCharacter0xdcbc', [], 'Display', None],
  [ 0xdcbd, 'UnicodeCharacter0xdcbd', [], 'Display', None],
  [ 0xdcbe, 'UnicodeCharacter0xdcbe', [], 'Display', None],
  [ 0xdcbf, 'UnicodeCharacter0xdcbf', [], 'Display', None],
  [ 0xdcc0, 'UnicodeCharacter0xdcc0', [], 'Display', None],
  [ 0xdcc1, 'UnicodeCharacter0xdcc1', [], 'Display', None],
  [ 0xdcc2, 'UnicodeCharacter0xdcc2', [], 'Display', None],
  [ 0xdcc3, 'UnicodeCharacter0xdcc3', [], 'Display', None],
  [ 0xdcc4, 'UnicodeCharacter0xdcc4', [], 'Display', None],
  [ 0xdcc5, 'UnicodeCharacter0xdcc5', [], 'Display', None],
  [ 0xdcc6, 'UnicodeCharacter0xdcc6', [], 'Display', None],
  [ 0xdcc7, 'UnicodeCharacter0xdcc7', [], 'Display', None],
  [ 0xdcc8, 'UnicodeCharacter0xdcc8', [], 'Display', None],
  [ 0xdcc9, 'UnicodeCharacter0xdcc9', [], 'Display', None],
  [ 0xdcca, 'UnicodeCharacter0xdcca', [], 'Display', None],
  [ 0xdccb, 'UnicodeCharacter0xdccb', [], 'Display', None],
  [ 0xdccc, 'UnicodeCharacter0xdccc', [], 'Display', None],
  [ 0xdccd, 'UnicodeCharacter0xdccd', [], 'Display', None],
  [ 0xdcce, 'UnicodeCharacter0xdcce', [], 'Display', None],
  [ 0xdccf, 'UnicodeCharacter0xdccf', [], 'Display', None],
  [ 0xdcd0, 'UnicodeCharacter0xdcd0', [], 'Display', None],
  [ 0xdcd1, 'UnicodeCharacter0xdcd1', [], 'Display', None],
  [ 0xdcd2, 'UnicodeCharacter0xdcd2', [], 'Display', None],
  [ 0xdcd3, 'UnicodeCharacter0xdcd3', [], 'Display', None],
  [ 0xdcd4, 'UnicodeCharacter0xdcd4', [], 'Display', None],
  [ 0xdcd5, 'UnicodeCharacter0xdcd5', [], 'Display', None],
  [ 0xdcd6, 'UnicodeCharacter0xdcd6', [], 'Display', None],
  [ 0xdcd7, 'UnicodeCharacter0xdcd7', [], 'Display', None],
  [ 0xdcd8, 'UnicodeCharacter0xdcd8', [], 'Display', None],
  [ 0xdcd9, 'UnicodeCharacter0xdcd9', [], 'Display', None],
  [ 0xdcda, 'UnicodeCharacter0xdcda', [], 'Display', None],
  [ 0xdcdb, 'UnicodeCharacter0xdcdb', [], 'Display', None],
  [ 0xdcdc, 'UnicodeCharacter0xdcdc', [], 'Display', None],
  [ 0xdcdd, 'UnicodeCharacter0xdcdd', [], 'Display', None],
  [ 0xdcde, 'UnicodeCharacter0xdcde', [], 'Display', None],
  [ 0xdcdf, 'UnicodeCharacter0xdcdf', [], 'Display', None],
  [ 0xdce0, 'UnicodeCharacter0xdce0', [], 'Display', None],
  [ 0xdce1, 'UnicodeCharacter0xdce1', [], 'Display', None],
  [ 0xdce2, 'UnicodeCharacter0xdce2', [], 'Display', None],
  [ 0xdce3, 'UnicodeCharacter0xdce3', [], 'Display', None],
  [ 0xdce4, 'UnicodeCharacter0xdce4', [], 'Display', None],
  [ 0xdce5, 'UnicodeCharacter0xdce5', [], 'Display', None],
  [ 0xdce6, 'UnicodeCharacter0xdce6', [], 'Display', None],
  [ 0xdce7, 'UnicodeCharacter0xdce7', [], 'Display', None],
  [ 0xdce8, 'UnicodeCharacter0xdce8', [], 'Display', None],
  [ 0xdce9, 'UnicodeCharacter0xdce9', [], 'Display', None],
  [ 0xdcea, 'UnicodeCharacter0xdcea', [], 'Display', None],
  [ 0xdceb, 'UnicodeCharacter0xdceb', [], 'Display', None],
  [ 0xdcec, 'UnicodeCharacter0xdcec', [], 'Display', None],
  [ 0xdced, 'UnicodeCharacter0xdced', [], 'Display', None],
  [ 0xdcee, 'UnicodeCharacter0xdcee', [], 'Display', None],
  [ 0xdcef, 'UnicodeCharacter0xdcef', [], 'Display', None],
  [ 0xdcf0, 'UnicodeCharacter0xdcf0', [], 'Display', None],
  [ 0xdcf1, 'UnicodeCharacter0xdcf1', [], 'Display', None],
  [ 0xdcf2, 'UnicodeCharacter0xdcf2', [], 'Display', None],
  [ 0xdcf3, 'UnicodeCharacter0xdcf3', [], 'Display', None],
  [ 0xdcf4, 'UnicodeCharacter0xdcf4', [], 'Display', None],
  [ 0xdcf5, 'UnicodeCharacter0xdcf5', [], 'Display', None],
  [ 0xdcf6, 'UnicodeCharacter0xdcf6', [], 'Display', None],
  [ 0xdcf7, 'UnicodeCharacter0xdcf7', [], 'Display', None],
  [ 0xdcf8, 'UnicodeCharacter0xdcf8', [], 'Display', None],
  [ 0xdcf9, 'UnicodeCharacter0xdcf9', [], 'Display', None],
  [ 0xdcfa, 'UnicodeCharacter0xdcfa', [], 'Display', None],
  [ 0xdcfb, 'UnicodeCharacter0xdcfb', [], 'Display', None],
  [ 0xdcfc, 'UnicodeCharacter0xdcfc', [], 'Display', None],
  [ 0xdcfd, 'UnicodeCharacter0xdcfd', [], 'Display', None],
  [ 0xdcfe, 'UnicodeCharacter0xdcfe', [], 'Display', None],
  [ 0xdcff, 'UnicodeCharacter0xdcff', [], 'Display', None],
  [ 0xdd00, 'UnicodeCharacter0xdd00', [], 'Display', None],
  [ 0xdd01, 'UnicodeCharacter0xdd01', [], 'Display', None],
  [ 0xdd02, 'UnicodeCharacter0xdd02', [], 'Display', None],
  [ 0xdd03, 'UnicodeCharacter0xdd03', [], 'Display', None],
  [ 0xdd04, 'UnicodeCharacter0xdd04', [], 'Display', None],
  [ 0xdd05, 'UnicodeCharacter0xdd05', [], 'Display', None],
  [ 0xdd06, 'UnicodeCharacter0xdd06', [], 'Display', None],
  [ 0xdd07, 'UnicodeCharacter0xdd07', [], 'Display', None],
  [ 0xdd08, 'UnicodeCharacter0xdd08', [], 'Display', None],
  [ 0xdd09, 'UnicodeCharacter0xdd09', [], 'Display', None],
  [ 0xdd0a, 'UnicodeCharacter0xdd0a', [], 'Display', None],
  [ 0xdd0b, 'UnicodeCharacter0xdd0b', [], 'Display', None],
  [ 0xdd0c, 'UnicodeCharacter0xdd0c', [], 'Display', None],
  [ 0xdd0d, 'UnicodeCharacter0xdd0d', [], 'Display', None],
  [ 0xdd0e, 'UnicodeCharacter0xdd0e', [], 'Display', None],
  [ 0xdd0f, 'UnicodeCharacter0xdd0f', [], 'Display', None],
  [ 0xdd10, 'UnicodeCharacter0xdd10', [], 'Display', None],
  [ 0xdd11, 'UnicodeCharacter0xdd11', [], 'Display', None],
  [ 0xdd12, 'UnicodeCharacter0xdd12', [], 'Display', None],
  [ 0xdd13, 'UnicodeCharacter0xdd13', [], 'Display', None],
  [ 0xdd14, 'UnicodeCharacter0xdd14', [], 'Display', None],
  [ 0xdd15, 'UnicodeCharacter0xdd15', [], 'Display', None],
  [ 0xdd16, 'UnicodeCharacter0xdd16', [], 'Display', None],
  [ 0xdd17, 'UnicodeCharacter0xdd17', [], 'Display', None],
  [ 0xdd18, 'UnicodeCharacter0xdd18', [], 'Display', None],
  [ 0xdd19, 'UnicodeCharacter0xdd19', [], 'Display', None],
  [ 0xdd1a, 'UnicodeCharacter0xdd1a', [], 'Display', None],
  [ 0xdd1b, 'UnicodeCharacter0xdd1b', [], 'Display', None],
  [ 0xdd1c, 'UnicodeCharacter0xdd1c', [], 'Display', None],
  [ 0xdd1d, 'UnicodeCharacter0xdd1d', [], 'Display', None],
  [ 0xdd1e, 'UnicodeCharacter0xdd1e', [], 'Display', None],
  [ 0xdd1f, 'UnicodeCharacter0xdd1f', [], 'Display', None],
  [ 0xdd20, 'UnicodeCharacter0xdd20', [], 'Display', None],
  [ 0xdd21, 'UnicodeCharacter0xdd21', [], 'Display', None],
  [ 0xdd22, 'UnicodeCharacter0xdd22', [], 'Display', None],
  [ 0xdd23, 'UnicodeCharacter0xdd23', [], 'Display', None],
  [ 0xdd24, 'UnicodeCharacter0xdd24', [], 'Display', None],
  [ 0xdd25, 'UnicodeCharacter0xdd25', [], 'Display', None],
  [ 0xdd26, 'UnicodeCharacter0xdd26', [], 'Display', None],
  [ 0xdd27, 'UnicodeCharacter0xdd27', [], 'Display', None],
  [ 0xdd28, 'UnicodeCharacter0xdd28', [], 'Display', None],
  [ 0xdd29, 'UnicodeCharacter0xdd29', [], 'Display', None],
  [ 0xdd2a, 'UnicodeCharacter0xdd2a', [], 'Display', None],
  [ 0xdd2b, 'UnicodeCharacter0xdd2b', [], 'Display', None],
  [ 0xdd2c, 'UnicodeCharacter0xdd2c', [], 'Display', None],
  [ 0xdd2d, 'UnicodeCharacter0xdd2d', [], 'Display', None],
  [ 0xdd2e, 'UnicodeCharacter0xdd2e', [], 'Display', None],
  [ 0xdd2f, 'UnicodeCharacter0xdd2f', [], 'Display', None],
  [ 0xdd30, 'UnicodeCharacter0xdd30', [], 'Display', None],
  [ 0xdd31, 'UnicodeCharacter0xdd31', [], 'Display', None],
  [ 0xdd32, 'UnicodeCharacter0xdd32', [], 'Display', None],
  [ 0xdd33, 'UnicodeCharacter0xdd33', [], 'Display', None],
  [ 0xdd34, 'UnicodeCharacter0xdd34', [], 'Display', None],
  [ 0xdd35, 'UnicodeCharacter0xdd35', [], 'Display', None],
  [ 0xdd36, 'UnicodeCharacter0xdd36', [], 'Display', None],
  [ 0xdd37, 'UnicodeCharacter0xdd37', [], 'Display', None],
  [ 0xdd38, 'UnicodeCharacter0xdd38', [], 'Display', None],
  [ 0xdd39, 'UnicodeCharacter0xdd39', [], 'Display', None],
  [ 0xdd3a, 'UnicodeCharacter0xdd3a', [], 'Display', None],
  [ 0xdd3b, 'UnicodeCharacter0xdd3b', [], 'Display', None],
  [ 0xdd3c, 'UnicodeCharacter0xdd3c', [], 'Display', None],
  [ 0xdd3d, 'UnicodeCharacter0xdd3d', [], 'Display', None],
  [ 0xdd3e, 'UnicodeCharacter0xdd3e', [], 'Display', None],
  [ 0xdd3f, 'UnicodeCharacter0xdd3f', [], 'Display', None],
  [ 0xdd40, 'UnicodeCharacter0xdd40', [], 'Display', None],
  [ 0xdd41, 'UnicodeCharacter0xdd41', [], 'Display', None],
  [ 0xdd42, 'UnicodeCharacter0xdd42', [], 'Display', None],
  [ 0xdd43, 'UnicodeCharacter0xdd43', [], 'Display', None],
  [ 0xdd44, 'UnicodeCharacter0xdd44', [], 'Display', None],
  [ 0xdd45, 'UnicodeCharacter0xdd45', [], 'Display', None],
  [ 0xdd46, 'UnicodeCharacter0xdd46', [], 'Display', None],
  [ 0xdd47, 'UnicodeCharacter0xdd47', [], 'Display', None],
  [ 0xdd48, 'UnicodeCharacter0xdd48', [], 'Display', None],
  [ 0xdd49, 'UnicodeCharacter0xdd49', [], 'Display', None],
  [ 0xdd4a, 'UnicodeCharacter0xdd4a', [], 'Display', None],
  [ 0xdd4b, 'UnicodeCharacter0xdd4b', [], 'Display', None],
  [ 0xdd4c, 'UnicodeCharacter0xdd4c', [], 'Display', None],
  [ 0xdd4d, 'UnicodeCharacter0xdd4d', [], 'Display', None],
  [ 0xdd4e, 'UnicodeCharacter0xdd4e', [], 'Display', None],
  [ 0xdd4f, 'UnicodeCharacter0xdd4f', [], 'Display', None],
  [ 0xdd50, 'UnicodeCharacter0xdd50', [], 'Display', None],
  [ 0xdd51, 'UnicodeCharacter0xdd51', [], 'Display', None],
  [ 0xdd52, 'UnicodeCharacter0xdd52', [], 'Display', None],
  [ 0xdd53, 'UnicodeCharacter0xdd53', [], 'Display', None],
  [ 0xdd54, 'UnicodeCharacter0xdd54', [], 'Display', None],
  [ 0xdd55, 'UnicodeCharacter0xdd55', [], 'Display', None],
  [ 0xdd56, 'UnicodeCharacter0xdd56', [], 'Display', None],
  [ 0xdd57, 'UnicodeCharacter0xdd57', [], 'Display', None],
  [ 0xdd58, 'UnicodeCharacter0xdd58', [], 'Display', None],
  [ 0xdd59, 'UnicodeCharacter0xdd59', [], 'Display', None],
  [ 0xdd5a, 'UnicodeCharacter0xdd5a', [], 'Display', None],
  [ 0xdd5b, 'UnicodeCharacter0xdd5b', [], 'Display', None],
  [ 0xdd5c, 'UnicodeCharacter0xdd5c', [], 'Display', None],
  [ 0xdd5d, 'UnicodeCharacter0xdd5d', [], 'Display', None],
  [ 0xdd5e, 'UnicodeCharacter0xdd5e', [], 'Display', None],
  [ 0xdd5f, 'UnicodeCharacter0xdd5f', [], 'Display', None],
  [ 0xdd60, 'UnicodeCharacter0xdd60', [], 'Display', None],
  [ 0xdd61, 'UnicodeCharacter0xdd61', [], 'Display', None],
  [ 0xdd62, 'UnicodeCharacter0xdd62', [], 'Display', None],
  [ 0xdd63, 'UnicodeCharacter0xdd63', [], 'Display', None],
  [ 0xdd64, 'UnicodeCharacter0xdd64', [], 'Display', None],
  [ 0xdd65, 'UnicodeCharacter0xdd65', [], 'Display', None],
  [ 0xdd66, 'UnicodeCharacter0xdd66', [], 'Display', None],
  [ 0xdd67, 'UnicodeCharacter0xdd67', [], 'Display', None],
  [ 0xdd68, 'UnicodeCharacter0xdd68', [], 'Display', None],
  [ 0xdd69, 'UnicodeCharacter0xdd69', [], 'Display', None],
  [ 0xdd6a, 'UnicodeCharacter0xdd6a', [], 'Display', None],
  [ 0xdd6b, 'UnicodeCharacter0xdd6b', [], 'Display', None],
  [ 0xdd6c, 'UnicodeCharacter0xdd6c', [], 'Display', None],
  [ 0xdd6d, 'UnicodeCharacter0xdd6d', [], 'Display', None],
  [ 0xdd6e, 'UnicodeCharacter0xdd6e', [], 'Display', None],
  [ 0xdd6f, 'UnicodeCharacter0xdd6f', [], 'Display', None],
  [ 0xdd70, 'UnicodeCharacter0xdd70', [], 'Display', None],
  [ 0xdd71, 'UnicodeCharacter0xdd71', [], 'Display', None],
  [ 0xdd72, 'UnicodeCharacter0xdd72', [], 'Display', None],
  [ 0xdd73, 'UnicodeCharacter0xdd73', [], 'Display', None],
  [ 0xdd74, 'UnicodeCharacter0xdd74', [], 'Display', None],
  [ 0xdd75, 'UnicodeCharacter0xdd75', [], 'Display', None],
  [ 0xdd76, 'UnicodeCharacter0xdd76', [], 'Display', None],
  [ 0xdd77, 'UnicodeCharacter0xdd77', [], 'Display', None],
  [ 0xdd78, 'UnicodeCharacter0xdd78', [], 'Display', None],
  [ 0xdd79, 'UnicodeCharacter0xdd79', [], 'Display', None],
  [ 0xdd7a, 'UnicodeCharacter0xdd7a', [], 'Display', None],
  [ 0xdd7b, 'UnicodeCharacter0xdd7b', [], 'Display', None],
  [ 0xdd7c, 'UnicodeCharacter0xdd7c', [], 'Display', None],
  [ 0xdd7d, 'UnicodeCharacter0xdd7d', [], 'Display', None],
  [ 0xdd7e, 'UnicodeCharacter0xdd7e', [], 'Display', None],
  [ 0xdd7f, 'UnicodeCharacter0xdd7f', [], 'Display', None],
  [ 0xdd80, 'UnicodeCharacter0xdd80', [], 'Display', None],
  [ 0xdd81, 'UnicodeCharacter0xdd81', [], 'Display', None],
  [ 0xdd82, 'UnicodeCharacter0xdd82', [], 'Display', None],
  [ 0xdd83, 'UnicodeCharacter0xdd83', [], 'Display', None],
  [ 0xdd84, 'UnicodeCharacter0xdd84', [], 'Display', None],
  [ 0xdd85, 'UnicodeCharacter0xdd85', [], 'Display', None],
  [ 0xdd86, 'UnicodeCharacter0xdd86', [], 'Display', None],
  [ 0xdd87, 'UnicodeCharacter0xdd87', [], 'Display', None],
  [ 0xdd88, 'UnicodeCharacter0xdd88', [], 'Display', None],
  [ 0xdd89, 'UnicodeCharacter0xdd89', [], 'Display', None],
  [ 0xdd8a, 'UnicodeCharacter0xdd8a', [], 'Display', None],
  [ 0xdd8b, 'UnicodeCharacter0xdd8b', [], 'Display', None],
  [ 0xdd8c, 'UnicodeCharacter0xdd8c', [], 'Display', None],
  [ 0xdd8d, 'UnicodeCharacter0xdd8d', [], 'Display', None],
  [ 0xdd8e, 'UnicodeCharacter0xdd8e', [], 'Display', None],
  [ 0xdd8f, 'UnicodeCharacter0xdd8f', [], 'Display', None],
  [ 0xdd90, 'UnicodeCharacter0xdd90', [], 'Display', None],
  [ 0xdd91, 'UnicodeCharacter0xdd91', [], 'Display', None],
  [ 0xdd92, 'UnicodeCharacter0xdd92', [], 'Display', None],
  [ 0xdd93, 'UnicodeCharacter0xdd93', [], 'Display', None],
  [ 0xdd94, 'UnicodeCharacter0xdd94', [], 'Display', None],
  [ 0xdd95, 'UnicodeCharacter0xdd95', [], 'Display', None],
  [ 0xdd96, 'UnicodeCharacter0xdd96', [], 'Display', None],
  [ 0xdd97, 'UnicodeCharacter0xdd97', [], 'Display', None],
  [ 0xdd98, 'UnicodeCharacter0xdd98', [], 'Display', None],
  [ 0xdd99, 'UnicodeCharacter0xdd99', [], 'Display', None],
  [ 0xdd9a, 'UnicodeCharacter0xdd9a', [], 'Display', None],
  [ 0xdd9b, 'UnicodeCharacter0xdd9b', [], 'Display', None],
  [ 0xdd9c, 'UnicodeCharacter0xdd9c', [], 'Display', None],
  [ 0xdd9d, 'UnicodeCharacter0xdd9d', [], 'Display', None],
  [ 0xdd9e, 'UnicodeCharacter0xdd9e', [], 'Display', None],
  [ 0xdd9f, 'UnicodeCharacter0xdd9f', [], 'Display', None],
  [ 0xdda0, 'UnicodeCharacter0xdda0', [], 'Display', None],
  [ 0xdda1, 'UnicodeCharacter0xdda1', [], 'Display', None],
  [ 0xdda2, 'UnicodeCharacter0xdda2', [], 'Display', None],
  [ 0xdda3, 'UnicodeCharacter0xdda3', [], 'Display', None],
  [ 0xdda4, 'UnicodeCharacter0xdda4', [], 'Display', None],
  [ 0xdda5, 'UnicodeCharacter0xdda5', [], 'Display', None],
  [ 0xdda6, 'UnicodeCharacter0xdda6', [], 'Display', None],
  [ 0xdda7, 'UnicodeCharacter0xdda7', [], 'Display', None],
  [ 0xdda8, 'UnicodeCharacter0xdda8', [], 'Display', None],
  [ 0xdda9, 'UnicodeCharacter0xdda9', [], 'Display', None],
  [ 0xddaa, 'UnicodeCharacter0xddaa', [], 'Display', None],
  [ 0xddab, 'UnicodeCharacter0xddab', [], 'Display', None],
  [ 0xddac, 'UnicodeCharacter0xddac', [], 'Display', None],
  [ 0xddad, 'UnicodeCharacter0xddad', [], 'Display', None],
  [ 0xddae, 'UnicodeCharacter0xddae', [], 'Display', None],
  [ 0xddaf, 'UnicodeCharacter0xddaf', [], 'Display', None],
  [ 0xddb0, 'UnicodeCharacter0xddb0', [], 'Display', None],
  [ 0xddb1, 'UnicodeCharacter0xddb1', [], 'Display', None],
  [ 0xddb2, 'UnicodeCharacter0xddb2', [], 'Display', None],
  [ 0xddb3, 'UnicodeCharacter0xddb3', [], 'Display', None],
  [ 0xddb4, 'UnicodeCharacter0xddb4', [], 'Display', None],
  [ 0xddb5, 'UnicodeCharacter0xddb5', [], 'Display', None],
  [ 0xddb6, 'UnicodeCharacter0xddb6', [], 'Display', None],
  [ 0xddb7, 'UnicodeCharacter0xddb7', [], 'Display', None],
  [ 0xddb8, 'UnicodeCharacter0xddb8', [], 'Display', None],
  [ 0xddb9, 'UnicodeCharacter0xddb9', [], 'Display', None],
  [ 0xddba, 'UnicodeCharacter0xddba', [], 'Display', None],
  [ 0xddbb, 'UnicodeCharacter0xddbb', [], 'Display', None],
  [ 0xddbc, 'UnicodeCharacter0xddbc', [], 'Display', None],
  [ 0xddbd, 'UnicodeCharacter0xddbd', [], 'Display', None],
  [ 0xddbe, 'UnicodeCharacter0xddbe', [], 'Display', None],
  [ 0xddbf, 'UnicodeCharacter0xddbf', [], 'Display', None],
  [ 0xddc0, 'UnicodeCharacter0xddc0', [], 'Display', None],
  [ 0xddc1, 'UnicodeCharacter0xddc1', [], 'Display', None],
  [ 0xddc2, 'UnicodeCharacter0xddc2', [], 'Display', None],
  [ 0xddc3, 'UnicodeCharacter0xddc3', [], 'Display', None],
  [ 0xddc4, 'UnicodeCharacter0xddc4', [], 'Display', None],
  [ 0xddc5, 'UnicodeCharacter0xddc5', [], 'Display', None],
  [ 0xddc6, 'UnicodeCharacter0xddc6', [], 'Display', None],
  [ 0xddc7, 'UnicodeCharacter0xddc7', [], 'Display', None],
  [ 0xddc8, 'UnicodeCharacter0xddc8', [], 'Display', None],
  [ 0xddc9, 'UnicodeCharacter0xddc9', [], 'Display', None],
  [ 0xddca, 'UnicodeCharacter0xddca', [], 'Display', None],
  [ 0xddcb, 'UnicodeCharacter0xddcb', [], 'Display', None],
  [ 0xddcc, 'UnicodeCharacter0xddcc', [], 'Display', None],
  [ 0xddcd, 'UnicodeCharacter0xddcd', [], 'Display', None],
  [ 0xddce, 'UnicodeCharacter0xddce', [], 'Display', None],
  [ 0xddcf, 'UnicodeCharacter0xddcf', [], 'Display', None],
  [ 0xddd0, 'UnicodeCharacter0xddd0', [], 'Display', None],
  [ 0xddd1, 'UnicodeCharacter0xddd1', [], 'Display', None],
  [ 0xddd2, 'UnicodeCharacter0xddd2', [], 'Display', None],
  [ 0xddd3, 'UnicodeCharacter0xddd3', [], 'Display', None],
  [ 0xddd4, 'UnicodeCharacter0xddd4', [], 'Display', None],
  [ 0xddd5, 'UnicodeCharacter0xddd5', [], 'Display', None],
  [ 0xddd6, 'UnicodeCharacter0xddd6', [], 'Display', None],
  [ 0xddd7, 'UnicodeCharacter0xddd7', [], 'Display', None],
  [ 0xddd8, 'UnicodeCharacter0xddd8', [], 'Display', None],
  [ 0xddd9, 'UnicodeCharacter0xddd9', [], 'Display', None],
  [ 0xddda, 'UnicodeCharacter0xddda', [], 'Display', None],
  [ 0xdddb, 'UnicodeCharacter0xdddb', [], 'Display', None],
  [ 0xdddc, 'UnicodeCharacter0xdddc', [], 'Display', None],
  [ 0xdddd, 'UnicodeCharacter0xdddd', [], 'Display', None],
  [ 0xddde, 'UnicodeCharacter0xddde', [], 'Display', None],
  [ 0xdddf, 'UnicodeCharacter0xdddf', [], 'Display', None],
  [ 0xdde0, 'UnicodeCharacter0xdde0', [], 'Display', None],
  [ 0xdde1, 'UnicodeCharacter0xdde1', [], 'Display', None],
  [ 0xdde2, 'UnicodeCharacter0xdde2', [], 'Display', None],
  [ 0xdde3, 'UnicodeCharacter0xdde3', [], 'Display', None],
  [ 0xdde4, 'UnicodeCharacter0xdde4', [], 'Display', None],
  [ 0xdde5, 'UnicodeCharacter0xdde5', [], 'Display', None],
  [ 0xdde6, 'UnicodeCharacter0xdde6', [], 'Display', None],
  [ 0xdde7, 'UnicodeCharacter0xdde7', [], 'Display', None],
  [ 0xdde8, 'UnicodeCharacter0xdde8', [], 'Display', None],
  [ 0xdde9, 'UnicodeCharacter0xdde9', [], 'Display', None],
  [ 0xddea, 'UnicodeCharacter0xddea', [], 'Display', None],
  [ 0xddeb, 'UnicodeCharacter0xddeb', [], 'Display', None],
  [ 0xddec, 'UnicodeCharacter0xddec', [], 'Display', None],
  [ 0xdded, 'UnicodeCharacter0xdded', [], 'Display', None],
  [ 0xddee, 'UnicodeCharacter0xddee', [], 'Display', None],
  [ 0xddef, 'UnicodeCharacter0xddef', [], 'Display', None],
  [ 0xddf0, 'UnicodeCharacter0xddf0', [], 'Display', None],
  [ 0xddf1, 'UnicodeCharacter0xddf1', [], 'Display', None],
  [ 0xddf2, 'UnicodeCharacter0xddf2', [], 'Display', None],
  [ 0xddf3, 'UnicodeCharacter0xddf3', [], 'Display', None],
  [ 0xddf4, 'UnicodeCharacter0xddf4', [], 'Display', None],
  [ 0xddf5, 'UnicodeCharacter0xddf5', [], 'Display', None],
  [ 0xddf6, 'UnicodeCharacter0xddf6', [], 'Display', None],
  [ 0xddf7, 'UnicodeCharacter0xddf7', [], 'Display', None],
  [ 0xddf8, 'UnicodeCharacter0xddf8', [], 'Display', None],
  [ 0xddf9, 'UnicodeCharacter0xddf9', [], 'Display', None],
  [ 0xddfa, 'UnicodeCharacter0xddfa', [], 'Display', None],
  [ 0xddfb, 'UnicodeCharacter0xddfb', [], 'Display', None],
  [ 0xddfc, 'UnicodeCharacter0xddfc', [], 'Display', None],
  [ 0xddfd, 'UnicodeCharacter0xddfd', [], 'Display', None],
  [ 0xddfe, 'UnicodeCharacter0xddfe', [], 'Display', None],
  [ 0xddff, 'UnicodeCharacter0xddff', [], 'Display', None],
  [ 0xde00, 'UnicodeCharacter0xde00', [], 'Display', None],
  [ 0xde01, 'UnicodeCharacter0xde01', [], 'Display', None],
  [ 0xde02, 'UnicodeCharacter0xde02', [], 'Display', None],
  [ 0xde03, 'UnicodeCharacter0xde03', [], 'Display', None],
  [ 0xde04, 'UnicodeCharacter0xde04', [], 'Display', None],
  [ 0xde05, 'UnicodeCharacter0xde05', [], 'Display', None],
  [ 0xde06, 'UnicodeCharacter0xde06', [], 'Display', None],
  [ 0xde07, 'UnicodeCharacter0xde07', [], 'Display', None],
  [ 0xde08, 'UnicodeCharacter0xde08', [], 'Display', None],
  [ 0xde09, 'UnicodeCharacter0xde09', [], 'Display', None],
  [ 0xde0a, 'UnicodeCharacter0xde0a', [], 'Display', None],
  [ 0xde0b, 'UnicodeCharacter0xde0b', [], 'Display', None],
  [ 0xde0c, 'UnicodeCharacter0xde0c', [], 'Display', None],
  [ 0xde0d, 'UnicodeCharacter0xde0d', [], 'Display', None],
  [ 0xde0e, 'UnicodeCharacter0xde0e', [], 'Display', None],
  [ 0xde0f, 'UnicodeCharacter0xde0f', [], 'Display', None],
  [ 0xde10, 'UnicodeCharacter0xde10', [], 'Display', None],
  [ 0xde11, 'UnicodeCharacter0xde11', [], 'Display', None],
  [ 0xde12, 'UnicodeCharacter0xde12', [], 'Display', None],
  [ 0xde13, 'UnicodeCharacter0xde13', [], 'Display', None],
  [ 0xde14, 'UnicodeCharacter0xde14', [], 'Display', None],
  [ 0xde15, 'UnicodeCharacter0xde15', [], 'Display', None],
  [ 0xde16, 'UnicodeCharacter0xde16', [], 'Display', None],
  [ 0xde17, 'UnicodeCharacter0xde17', [], 'Display', None],
  [ 0xde18, 'UnicodeCharacter0xde18', [], 'Display', None],
  [ 0xde19, 'UnicodeCharacter0xde19', [], 'Display', None],
  [ 0xde1a, 'UnicodeCharacter0xde1a', [], 'Display', None],
  [ 0xde1b, 'UnicodeCharacter0xde1b', [], 'Display', None],
  [ 0xde1c, 'UnicodeCharacter0xde1c', [], 'Display', None],
  [ 0xde1d, 'UnicodeCharacter0xde1d', [], 'Display', None],
  [ 0xde1e, 'UnicodeCharacter0xde1e', [], 'Display', None],
  [ 0xde1f, 'UnicodeCharacter0xde1f', [], 'Display', None],
  [ 0xde20, 'UnicodeCharacter0xde20', [], 'Display', None],
  [ 0xde21, 'UnicodeCharacter0xde21', [], 'Display', None],
  [ 0xde22, 'UnicodeCharacter0xde22', [], 'Display', None],
  [ 0xde23, 'UnicodeCharacter0xde23', [], 'Display', None],
  [ 0xde24, 'UnicodeCharacter0xde24', [], 'Display', None],
  [ 0xde25, 'UnicodeCharacter0xde25', [], 'Display', None],
  [ 0xde26, 'UnicodeCharacter0xde26', [], 'Display', None],
  [ 0xde27, 'UnicodeCharacter0xde27', [], 'Display', None],
  [ 0xde28, 'UnicodeCharacter0xde28', [], 'Display', None],
  [ 0xde29, 'UnicodeCharacter0xde29', [], 'Display', None],
  [ 0xde2a, 'UnicodeCharacter0xde2a', [], 'Display', None],
  [ 0xde2b, 'UnicodeCharacter0xde2b', [], 'Display', None],
  [ 0xde2c, 'UnicodeCharacter0xde2c', [], 'Display', None],
  [ 0xde2d, 'UnicodeCharacter0xde2d', [], 'Display', None],
  [ 0xde2e, 'UnicodeCharacter0xde2e', [], 'Display', None],
  [ 0xde2f, 'UnicodeCharacter0xde2f', [], 'Display', None],
  [ 0xde30, 'UnicodeCharacter0xde30', [], 'Display', None],
  [ 0xde31, 'UnicodeCharacter0xde31', [], 'Display', None],
  [ 0xde32, 'UnicodeCharacter0xde32', [], 'Display', None],
  [ 0xde33, 'UnicodeCharacter0xde33', [], 'Display', None],
  [ 0xde34, 'UnicodeCharacter0xde34', [], 'Display', None],
  [ 0xde35, 'UnicodeCharacter0xde35', [], 'Display', None],
  [ 0xde36, 'UnicodeCharacter0xde36', [], 'Display', None],
  [ 0xde37, 'UnicodeCharacter0xde37', [], 'Display', None],
  [ 0xde38, 'UnicodeCharacter0xde38', [], 'Display', None],
  [ 0xde39, 'UnicodeCharacter0xde39', [], 'Display', None],
  [ 0xde3a, 'UnicodeCharacter0xde3a', [], 'Display', None],
  [ 0xde3b, 'UnicodeCharacter0xde3b', [], 'Display', None],
  [ 0xde3c, 'UnicodeCharacter0xde3c', [], 'Display', None],
  [ 0xde3d, 'UnicodeCharacter0xde3d', [], 'Display', None],
  [ 0xde3e, 'UnicodeCharacter0xde3e', [], 'Display', None],
  [ 0xde3f, 'UnicodeCharacter0xde3f', [], 'Display', None],
  [ 0xde40, 'UnicodeCharacter0xde40', [], 'Display', None],
  [ 0xde41, 'UnicodeCharacter0xde41', [], 'Display', None],
  [ 0xde42, 'UnicodeCharacter0xde42', [], 'Display', None],
  [ 0xde43, 'UnicodeCharacter0xde43', [], 'Display', None],
  [ 0xde44, 'UnicodeCharacter0xde44', [], 'Display', None],
  [ 0xde45, 'UnicodeCharacter0xde45', [], 'Display', None],
  [ 0xde46, 'UnicodeCharacter0xde46', [], 'Display', None],
  [ 0xde47, 'UnicodeCharacter0xde47', [], 'Display', None],
  [ 0xde48, 'UnicodeCharacter0xde48', [], 'Display', None],
  [ 0xde49, 'UnicodeCharacter0xde49', [], 'Display', None],
  [ 0xde4a, 'UnicodeCharacter0xde4a', [], 'Display', None],
  [ 0xde4b, 'UnicodeCharacter0xde4b', [], 'Display', None],
  [ 0xde4c, 'UnicodeCharacter0xde4c', [], 'Display', None],
  [ 0xde4d, 'UnicodeCharacter0xde4d', [], 'Display', None],
  [ 0xde4e, 'UnicodeCharacter0xde4e', [], 'Display', None],
  [ 0xde4f, 'UnicodeCharacter0xde4f', [], 'Display', None],
  [ 0xde50, 'UnicodeCharacter0xde50', [], 'Display', None],
  [ 0xde51, 'UnicodeCharacter0xde51', [], 'Display', None],
  [ 0xde52, 'UnicodeCharacter0xde52', [], 'Display', None],
  [ 0xde53, 'UnicodeCharacter0xde53', [], 'Display', None],
  [ 0xde54, 'UnicodeCharacter0xde54', [], 'Display', None],
  [ 0xde55, 'UnicodeCharacter0xde55', [], 'Display', None],
  [ 0xde56, 'UnicodeCharacter0xde56', [], 'Display', None],
  [ 0xde57, 'UnicodeCharacter0xde57', [], 'Display', None],
  [ 0xde58, 'UnicodeCharacter0xde58', [], 'Display', None],
  [ 0xde59, 'UnicodeCharacter0xde59', [], 'Display', None],
  [ 0xde5a, 'UnicodeCharacter0xde5a', [], 'Display', None],
  [ 0xde5b, 'UnicodeCharacter0xde5b', [], 'Display', None],
  [ 0xde5c, 'UnicodeCharacter0xde5c', [], 'Display', None],
  [ 0xde5d, 'UnicodeCharacter0xde5d', [], 'Display', None],
  [ 0xde5e, 'UnicodeCharacter0xde5e', [], 'Display', None],
  [ 0xde5f, 'UnicodeCharacter0xde5f', [], 'Display', None],
  [ 0xde60, 'UnicodeCharacter0xde60', [], 'Display', None],
  [ 0xde61, 'UnicodeCharacter0xde61', [], 'Display', None],
  [ 0xde62, 'UnicodeCharacter0xde62', [], 'Display', None],
  [ 0xde63, 'UnicodeCharacter0xde63', [], 'Display', None],
  [ 0xde64, 'UnicodeCharacter0xde64', [], 'Display', None],
  [ 0xde65, 'UnicodeCharacter0xde65', [], 'Display', None],
  [ 0xde66, 'UnicodeCharacter0xde66', [], 'Display', None],
  [ 0xde67, 'UnicodeCharacter0xde67', [], 'Display', None],
  [ 0xde68, 'UnicodeCharacter0xde68', [], 'Display', None],
  [ 0xde69, 'UnicodeCharacter0xde69', [], 'Display', None],
  [ 0xde6a, 'UnicodeCharacter0xde6a', [], 'Display', None],
  [ 0xde6b, 'UnicodeCharacter0xde6b', [], 'Display', None],
  [ 0xde6c, 'UnicodeCharacter0xde6c', [], 'Display', None],
  [ 0xde6d, 'UnicodeCharacter0xde6d', [], 'Display', None],
  [ 0xde6e, 'UnicodeCharacter0xde6e', [], 'Display', None],
  [ 0xde6f, 'UnicodeCharacter0xde6f', [], 'Display', None],
  [ 0xde70, 'UnicodeCharacter0xde70', [], 'Display', None],
  [ 0xde71, 'UnicodeCharacter0xde71', [], 'Display', None],
  [ 0xde72, 'UnicodeCharacter0xde72', [], 'Display', None],
  [ 0xde73, 'UnicodeCharacter0xde73', [], 'Display', None],
  [ 0xde74, 'UnicodeCharacter0xde74', [], 'Display', None],
  [ 0xde75, 'UnicodeCharacter0xde75', [], 'Display', None],
  [ 0xde76, 'UnicodeCharacter0xde76', [], 'Display', None],
  [ 0xde77, 'UnicodeCharacter0xde77', [], 'Display', None],
  [ 0xde78, 'UnicodeCharacter0xde78', [], 'Display', None],
  [ 0xde79, 'UnicodeCharacter0xde79', [], 'Display', None],
  [ 0xde7a, 'UnicodeCharacter0xde7a', [], 'Display', None],
  [ 0xde7b, 'UnicodeCharacter0xde7b', [], 'Display', None],
  [ 0xde7c, 'UnicodeCharacter0xde7c', [], 'Display', None],
  [ 0xde7d, 'UnicodeCharacter0xde7d', [], 'Display', None],
  [ 0xde7e, 'UnicodeCharacter0xde7e', [], 'Display', None],
  [ 0xde7f, 'UnicodeCharacter0xde7f', [], 'Display', None],
  [ 0xde80, 'UnicodeCharacter0xde80', [], 'Display', None],
  [ 0xde81, 'UnicodeCharacter0xde81', [], 'Display', None],
  [ 0xde82, 'UnicodeCharacter0xde82', [], 'Display', None],
  [ 0xde83, 'UnicodeCharacter0xde83', [], 'Display', None],
  [ 0xde84, 'UnicodeCharacter0xde84', [], 'Display', None],
  [ 0xde85, 'UnicodeCharacter0xde85', [], 'Display', None],
  [ 0xde86, 'UnicodeCharacter0xde86', [], 'Display', None],
  [ 0xde87, 'UnicodeCharacter0xde87', [], 'Display', None],
  [ 0xde88, 'UnicodeCharacter0xde88', [], 'Display', None],
  [ 0xde89, 'UnicodeCharacter0xde89', [], 'Display', None],
  [ 0xde8a, 'UnicodeCharacter0xde8a', [], 'Display', None],
  [ 0xde8b, 'UnicodeCharacter0xde8b', [], 'Display', None],
  [ 0xde8c, 'UnicodeCharacter0xde8c', [], 'Display', None],
  [ 0xde8d, 'UnicodeCharacter0xde8d', [], 'Display', None],
  [ 0xde8e, 'UnicodeCharacter0xde8e', [], 'Display', None],
  [ 0xde8f, 'UnicodeCharacter0xde8f', [], 'Display', None],
  [ 0xde90, 'UnicodeCharacter0xde90', [], 'Display', None],
  [ 0xde91, 'UnicodeCharacter0xde91', [], 'Display', None],
  [ 0xde92, 'UnicodeCharacter0xde92', [], 'Display', None],
  [ 0xde93, 'UnicodeCharacter0xde93', [], 'Display', None],
  [ 0xde94, 'UnicodeCharacter0xde94', [], 'Display', None],
  [ 0xde95, 'UnicodeCharacter0xde95', [], 'Display', None],
  [ 0xde96, 'UnicodeCharacter0xde96', [], 'Display', None],
  [ 0xde97, 'UnicodeCharacter0xde97', [], 'Display', None],
  [ 0xde98, 'UnicodeCharacter0xde98', [], 'Display', None],
  [ 0xde99, 'UnicodeCharacter0xde99', [], 'Display', None],
  [ 0xde9a, 'UnicodeCharacter0xde9a', [], 'Display', None],
  [ 0xde9b, 'UnicodeCharacter0xde9b', [], 'Display', None],
  [ 0xde9c, 'UnicodeCharacter0xde9c', [], 'Display', None],
  [ 0xde9d, 'UnicodeCharacter0xde9d', [], 'Display', None],
  [ 0xde9e, 'UnicodeCharacter0xde9e', [], 'Display', None],
  [ 0xde9f, 'UnicodeCharacter0xde9f', [], 'Display', None],
  [ 0xdea0, 'UnicodeCharacter0xdea0', [], 'Display', None],
  [ 0xdea1, 'UnicodeCharacter0xdea1', [], 'Display', None],
  [ 0xdea2, 'UnicodeCharacter0xdea2', [], 'Display', None],
  [ 0xdea3, 'UnicodeCharacter0xdea3', [], 'Display', None],
  [ 0xdea4, 'UnicodeCharacter0xdea4', [], 'Display', None],
  [ 0xdea5, 'UnicodeCharacter0xdea5', [], 'Display', None],
  [ 0xdea6, 'UnicodeCharacter0xdea6', [], 'Display', None],
  [ 0xdea7, 'UnicodeCharacter0xdea7', [], 'Display', None],
  [ 0xdea8, 'UnicodeCharacter0xdea8', [], 'Display', None],
  [ 0xdea9, 'UnicodeCharacter0xdea9', [], 'Display', None],
  [ 0xdeaa, 'UnicodeCharacter0xdeaa', [], 'Display', None],
  [ 0xdeab, 'UnicodeCharacter0xdeab', [], 'Display', None],
  [ 0xdeac, 'UnicodeCharacter0xdeac', [], 'Display', None],
  [ 0xdead, 'UnicodeCharacter0xdead', [], 'Display', None],
  [ 0xdeae, 'UnicodeCharacter0xdeae', [], 'Display', None],
  [ 0xdeaf, 'UnicodeCharacter0xdeaf', [], 'Display', None],
  [ 0xdeb0, 'UnicodeCharacter0xdeb0', [], 'Display', None],
  [ 0xdeb1, 'UnicodeCharacter0xdeb1', [], 'Display', None],
  [ 0xdeb2, 'UnicodeCharacter0xdeb2', [], 'Display', None],
  [ 0xdeb3, 'UnicodeCharacter0xdeb3', [], 'Display', None],
  [ 0xdeb4, 'UnicodeCharacter0xdeb4', [], 'Display', None],
  [ 0xdeb5, 'UnicodeCharacter0xdeb5', [], 'Display', None],
  [ 0xdeb6, 'UnicodeCharacter0xdeb6', [], 'Display', None],
  [ 0xdeb7, 'UnicodeCharacter0xdeb7', [], 'Display', None],
  [ 0xdeb8, 'UnicodeCharacter0xdeb8', [], 'Display', None],
  [ 0xdeb9, 'UnicodeCharacter0xdeb9', [], 'Display', None],
  [ 0xdeba, 'UnicodeCharacter0xdeba', [], 'Display', None],
  [ 0xdebb, 'UnicodeCharacter0xdebb', [], 'Display', None],
  [ 0xdebc, 'UnicodeCharacter0xdebc', [], 'Display', None],
  [ 0xdebd, 'UnicodeCharacter0xdebd', [], 'Display', None],
  [ 0xdebe, 'UnicodeCharacter0xdebe', [], 'Display', None],
  [ 0xdebf, 'UnicodeCharacter0xdebf', [], 'Display', None],
  [ 0xdec0, 'UnicodeCharacter0xdec0', [], 'Display', None],
  [ 0xdec1, 'UnicodeCharacter0xdec1', [], 'Display', None],
  [ 0xdec2, 'UnicodeCharacter0xdec2', [], 'Display', None],
  [ 0xdec3, 'UnicodeCharacter0xdec3', [], 'Display', None],
  [ 0xdec4, 'UnicodeCharacter0xdec4', [], 'Display', None],
  [ 0xdec5, 'UnicodeCharacter0xdec5', [], 'Display', None],
  [ 0xdec6, 'UnicodeCharacter0xdec6', [], 'Display', None],
  [ 0xdec7, 'UnicodeCharacter0xdec7', [], 'Display', None],
  [ 0xdec8, 'UnicodeCharacter0xdec8', [], 'Display', None],
  [ 0xdec9, 'UnicodeCharacter0xdec9', [], 'Display', None],
  [ 0xdeca, 'UnicodeCharacter0xdeca', [], 'Display', None],
  [ 0xdecb, 'UnicodeCharacter0xdecb', [], 'Display', None],
  [ 0xdecc, 'UnicodeCharacter0xdecc', [], 'Display', None],
  [ 0xdecd, 'UnicodeCharacter0xdecd', [], 'Display', None],
  [ 0xdece, 'UnicodeCharacter0xdece', [], 'Display', None],
  [ 0xdecf, 'UnicodeCharacter0xdecf', [], 'Display', None],
  [ 0xded0, 'UnicodeCharacter0xded0', [], 'Display', None],
  [ 0xded1, 'UnicodeCharacter0xded1', [], 'Display', None],
  [ 0xded2, 'UnicodeCharacter0xded2', [], 'Display', None],
  [ 0xded3, 'UnicodeCharacter0xded3', [], 'Display', None],
  [ 0xded4, 'UnicodeCharacter0xded4', [], 'Display', None],
  [ 0xded5, 'UnicodeCharacter0xded5', [], 'Display', None],
  [ 0xded6, 'UnicodeCharacter0xded6', [], 'Display', None],
  [ 0xded7, 'UnicodeCharacter0xded7', [], 'Display', None],
  [ 0xded8, 'UnicodeCharacter0xded8', [], 'Display', None],
  [ 0xded9, 'UnicodeCharacter0xded9', [], 'Display', None],
  [ 0xdeda, 'UnicodeCharacter0xdeda', [], 'Display', None],
  [ 0xdedb, 'UnicodeCharacter0xdedb', [], 'Display', None],
  [ 0xdedc, 'UnicodeCharacter0xdedc', [], 'Display', None],
  [ 0xdedd, 'UnicodeCharacter0xdedd', [], 'Display', None],
  [ 0xdede, 'UnicodeCharacter0xdede', [], 'Display', None],
  [ 0xdedf, 'UnicodeCharacter0xdedf', [], 'Display', None],
  [ 0xdee0, 'UnicodeCharacter0xdee0', [], 'Display', None],
  [ 0xdee1, 'UnicodeCharacter0xdee1', [], 'Display', None],
  [ 0xdee2, 'UnicodeCharacter0xdee2', [], 'Display', None],
  [ 0xdee3, 'UnicodeCharacter0xdee3', [], 'Display', None],
  [ 0xdee4, 'UnicodeCharacter0xdee4', [], 'Display', None],
  [ 0xdee5, 'UnicodeCharacter0xdee5', [], 'Display', None],
  [ 0xdee6, 'UnicodeCharacter0xdee6', [], 'Display', None],
  [ 0xdee7, 'UnicodeCharacter0xdee7', [], 'Display', None],
  [ 0xdee8, 'UnicodeCharacter0xdee8', [], 'Display', None],
  [ 0xdee9, 'UnicodeCharacter0xdee9', [], 'Display', None],
  [ 0xdeea, 'UnicodeCharacter0xdeea', [], 'Display', None],
  [ 0xdeeb, 'UnicodeCharacter0xdeeb', [], 'Display', None],
  [ 0xdeec, 'UnicodeCharacter0xdeec', [], 'Display', None],
  [ 0xdeed, 'UnicodeCharacter0xdeed', [], 'Display', None],
  [ 0xdeee, 'UnicodeCharacter0xdeee', [], 'Display', None],
  [ 0xdeef, 'UnicodeCharacter0xdeef', [], 'Display', None],
  [ 0xdef0, 'UnicodeCharacter0xdef0', [], 'Display', None],
  [ 0xdef1, 'UnicodeCharacter0xdef1', [], 'Display', None],
  [ 0xdef2, 'UnicodeCharacter0xdef2', [], 'Display', None],
  [ 0xdef3, 'UnicodeCharacter0xdef3', [], 'Display', None],
  [ 0xdef4, 'UnicodeCharacter0xdef4', [], 'Display', None],
  [ 0xdef5, 'UnicodeCharacter0xdef5', [], 'Display', None],
  [ 0xdef6, 'UnicodeCharacter0xdef6', [], 'Display', None],
  [ 0xdef7, 'UnicodeCharacter0xdef7', [], 'Display', None],
  [ 0xdef8, 'UnicodeCharacter0xdef8', [], 'Display', None],
  [ 0xdef9, 'UnicodeCharacter0xdef9', [], 'Display', None],
  [ 0xdefa, 'UnicodeCharacter0xdefa', [], 'Display', None],
  [ 0xdefb, 'UnicodeCharacter0xdefb', [], 'Display', None],
  [ 0xdefc, 'UnicodeCharacter0xdefc', [], 'Display', None],
  [ 0xdefd, 'UnicodeCharacter0xdefd', [], 'Display', None],
  [ 0xdefe, 'UnicodeCharacter0xdefe', [], 'Display', None],
  [ 0xdeff, 'UnicodeCharacter0xdeff', [], 'Display', None],
  [ 0xdf00, 'UnicodeCharacter0xdf00', [], 'Display', None],
  [ 0xdf01, 'UnicodeCharacter0xdf01', [], 'Display', None],
  [ 0xdf02, 'UnicodeCharacter0xdf02', [], 'Display', None],
  [ 0xdf03, 'UnicodeCharacter0xdf03', [], 'Display', None],
  [ 0xdf04, 'UnicodeCharacter0xdf04', [], 'Display', None],
  [ 0xdf05, 'UnicodeCharacter0xdf05', [], 'Display', None],
  [ 0xdf06, 'UnicodeCharacter0xdf06', [], 'Display', None],
  [ 0xdf07, 'UnicodeCharacter0xdf07', [], 'Display', None],
  [ 0xdf08, 'UnicodeCharacter0xdf08', [], 'Display', None],
  [ 0xdf09, 'UnicodeCharacter0xdf09', [], 'Display', None],
  [ 0xdf0a, 'UnicodeCharacter0xdf0a', [], 'Display', None],
  [ 0xdf0b, 'UnicodeCharacter0xdf0b', [], 'Display', None],
  [ 0xdf0c, 'UnicodeCharacter0xdf0c', [], 'Display', None],
  [ 0xdf0d, 'UnicodeCharacter0xdf0d', [], 'Display', None],
  [ 0xdf0e, 'UnicodeCharacter0xdf0e', [], 'Display', None],
  [ 0xdf0f, 'UnicodeCharacter0xdf0f', [], 'Display', None],
  [ 0xdf10, 'UnicodeCharacter0xdf10', [], 'Display', None],
  [ 0xdf11, 'UnicodeCharacter0xdf11', [], 'Display', None],
  [ 0xdf12, 'UnicodeCharacter0xdf12', [], 'Display', None],
  [ 0xdf13, 'UnicodeCharacter0xdf13', [], 'Display', None],
  [ 0xdf14, 'UnicodeCharacter0xdf14', [], 'Display', None],
  [ 0xdf15, 'UnicodeCharacter0xdf15', [], 'Display', None],
  [ 0xdf16, 'UnicodeCharacter0xdf16', [], 'Display', None],
  [ 0xdf17, 'UnicodeCharacter0xdf17', [], 'Display', None],
  [ 0xdf18, 'UnicodeCharacter0xdf18', [], 'Display', None],
  [ 0xdf19, 'UnicodeCharacter0xdf19', [], 'Display', None],
  [ 0xdf1a, 'UnicodeCharacter0xdf1a', [], 'Display', None],
  [ 0xdf1b, 'UnicodeCharacter0xdf1b', [], 'Display', None],
  [ 0xdf1c, 'UnicodeCharacter0xdf1c', [], 'Display', None],
  [ 0xdf1d, 'UnicodeCharacter0xdf1d', [], 'Display', None],
  [ 0xdf1e, 'UnicodeCharacter0xdf1e', [], 'Display', None],
  [ 0xdf1f, 'UnicodeCharacter0xdf1f', [], 'Display', None],
  [ 0xdf20, 'UnicodeCharacter0xdf20', [], 'Display', None],
  [ 0xdf21, 'UnicodeCharacter0xdf21', [], 'Display', None],
  [ 0xdf22, 'UnicodeCharacter0xdf22', [], 'Display', None],
  [ 0xdf23, 'UnicodeCharacter0xdf23', [], 'Display', None],
  [ 0xdf24, 'UnicodeCharacter0xdf24', [], 'Display', None],
  [ 0xdf25, 'UnicodeCharacter0xdf25', [], 'Display', None],
  [ 0xdf26, 'UnicodeCharacter0xdf26', [], 'Display', None],
  [ 0xdf27, 'UnicodeCharacter0xdf27', [], 'Display', None],
  [ 0xdf28, 'UnicodeCharacter0xdf28', [], 'Display', None],
  [ 0xdf29, 'UnicodeCharacter0xdf29', [], 'Display', None],
  [ 0xdf2a, 'UnicodeCharacter0xdf2a', [], 'Display', None],
  [ 0xdf2b, 'UnicodeCharacter0xdf2b', [], 'Display', None],
  [ 0xdf2c, 'UnicodeCharacter0xdf2c', [], 'Display', None],
  [ 0xdf2d, 'UnicodeCharacter0xdf2d', [], 'Display', None],
  [ 0xdf2e, 'UnicodeCharacter0xdf2e', [], 'Display', None],
  [ 0xdf2f, 'UnicodeCharacter0xdf2f', [], 'Display', None],
  [ 0xdf30, 'UnicodeCharacter0xdf30', [], 'Display', None],
  [ 0xdf31, 'UnicodeCharacter0xdf31', [], 'Display', None],
  [ 0xdf32, 'UnicodeCharacter0xdf32', [], 'Display', None],
  [ 0xdf33, 'UnicodeCharacter0xdf33', [], 'Display', None],
  [ 0xdf34, 'UnicodeCharacter0xdf34', [], 'Display', None],
  [ 0xdf35, 'UnicodeCharacter0xdf35', [], 'Display', None],
  [ 0xdf36, 'UnicodeCharacter0xdf36', [], 'Display', None],
  [ 0xdf37, 'UnicodeCharacter0xdf37', [], 'Display', None],
  [ 0xdf38, 'UnicodeCharacter0xdf38', [], 'Display', None],
  [ 0xdf39, 'UnicodeCharacter0xdf39', [], 'Display', None],
  [ 0xdf3a, 'UnicodeCharacter0xdf3a', [], 'Display', None],
  [ 0xdf3b, 'UnicodeCharacter0xdf3b', [], 'Display', None],
  [ 0xdf3c, 'UnicodeCharacter0xdf3c', [], 'Display', None],
  [ 0xdf3d, 'UnicodeCharacter0xdf3d', [], 'Display', None],
  [ 0xdf3e, 'UnicodeCharacter0xdf3e', [], 'Display', None],
  [ 0xdf3f, 'UnicodeCharacter0xdf3f', [], 'Display', None],
  [ 0xdf40, 'UnicodeCharacter0xdf40', [], 'Display', None],
  [ 0xdf41, 'UnicodeCharacter0xdf41', [], 'Display', None],
  [ 0xdf42, 'UnicodeCharacter0xdf42', [], 'Display', None],
  [ 0xdf43, 'UnicodeCharacter0xdf43', [], 'Display', None],
  [ 0xdf44, 'UnicodeCharacter0xdf44', [], 'Display', None],
  [ 0xdf45, 'UnicodeCharacter0xdf45', [], 'Display', None],
  [ 0xdf46, 'UnicodeCharacter0xdf46', [], 'Display', None],
  [ 0xdf47, 'UnicodeCharacter0xdf47', [], 'Display', None],
  [ 0xdf48, 'UnicodeCharacter0xdf48', [], 'Display', None],
  [ 0xdf49, 'UnicodeCharacter0xdf49', [], 'Display', None],
  [ 0xdf4a, 'UnicodeCharacter0xdf4a', [], 'Display', None],
  [ 0xdf4b, 'UnicodeCharacter0xdf4b', [], 'Display', None],
  [ 0xdf4c, 'UnicodeCharacter0xdf4c', [], 'Display', None],
  [ 0xdf4d, 'UnicodeCharacter0xdf4d', [], 'Display', None],
  [ 0xdf4e, 'UnicodeCharacter0xdf4e', [], 'Display', None],
  [ 0xdf4f, 'UnicodeCharacter0xdf4f', [], 'Display', None],
  [ 0xdf50, 'UnicodeCharacter0xdf50', [], 'Display', None],
  [ 0xdf51, 'UnicodeCharacter0xdf51', [], 'Display', None],
  [ 0xdf52, 'UnicodeCharacter0xdf52', [], 'Display', None],
  [ 0xdf53, 'UnicodeCharacter0xdf53', [], 'Display', None],
  [ 0xdf54, 'UnicodeCharacter0xdf54', [], 'Display', None],
  [ 0xdf55, 'UnicodeCharacter0xdf55', [], 'Display', None],
  [ 0xdf56, 'UnicodeCharacter0xdf56', [], 'Display', None],
  [ 0xdf57, 'UnicodeCharacter0xdf57', [], 'Display', None],
  [ 0xdf58, 'UnicodeCharacter0xdf58', [], 'Display', None],
  [ 0xdf59, 'UnicodeCharacter0xdf59', [], 'Display', None],
  [ 0xdf5a, 'UnicodeCharacter0xdf5a', [], 'Display', None],
  [ 0xdf5b, 'UnicodeCharacter0xdf5b', [], 'Display', None],
  [ 0xdf5c, 'UnicodeCharacter0xdf5c', [], 'Display', None],
  [ 0xdf5d, 'UnicodeCharacter0xdf5d', [], 'Display', None],
  [ 0xdf5e, 'UnicodeCharacter0xdf5e', [], 'Display', None],
  [ 0xdf5f, 'UnicodeCharacter0xdf5f', [], 'Display', None],
  [ 0xdf60, 'UnicodeCharacter0xdf60', [], 'Display', None],
  [ 0xdf61, 'UnicodeCharacter0xdf61', [], 'Display', None],
  [ 0xdf62, 'UnicodeCharacter0xdf62', [], 'Display', None],
  [ 0xdf63, 'UnicodeCharacter0xdf63', [], 'Display', None],
  [ 0xdf64, 'UnicodeCharacter0xdf64', [], 'Display', None],
  [ 0xdf65, 'UnicodeCharacter0xdf65', [], 'Display', None],
  [ 0xdf66, 'UnicodeCharacter0xdf66', [], 'Display', None],
  [ 0xdf67, 'UnicodeCharacter0xdf67', [], 'Display', None],
  [ 0xdf68, 'UnicodeCharacter0xdf68', [], 'Display', None],
  [ 0xdf69, 'UnicodeCharacter0xdf69', [], 'Display', None],
  [ 0xdf6a, 'UnicodeCharacter0xdf6a', [], 'Display', None],
  [ 0xdf6b, 'UnicodeCharacter0xdf6b', [], 'Display', None],
  [ 0xdf6c, 'UnicodeCharacter0xdf6c', [], 'Display', None],
  [ 0xdf6d, 'UnicodeCharacter0xdf6d', [], 'Display', None],
  [ 0xdf6e, 'UnicodeCharacter0xdf6e', [], 'Display', None],
  [ 0xdf6f, 'UnicodeCharacter0xdf6f', [], 'Display', None],
  [ 0xdf70, 'UnicodeCharacter0xdf70', [], 'Display', None],
  [ 0xdf71, 'UnicodeCharacter0xdf71', [], 'Display', None],
  [ 0xdf72, 'UnicodeCharacter0xdf72', [], 'Display', None],
  [ 0xdf73, 'UnicodeCharacter0xdf73', [], 'Display', None],
  [ 0xdf74, 'UnicodeCharacter0xdf74', [], 'Display', None],
  [ 0xdf75, 'UnicodeCharacter0xdf75', [], 'Display', None],
  [ 0xdf76, 'UnicodeCharacter0xdf76', [], 'Display', None],
  [ 0xdf77, 'UnicodeCharacter0xdf77', [], 'Display', None],
  [ 0xdf78, 'UnicodeCharacter0xdf78', [], 'Display', None],
  [ 0xdf79, 'UnicodeCharacter0xdf79', [], 'Display', None],
  [ 0xdf7a, 'UnicodeCharacter0xdf7a', [], 'Display', None],
  [ 0xdf7b, 'UnicodeCharacter0xdf7b', [], 'Display', None],
  [ 0xdf7c, 'UnicodeCharacter0xdf7c', [], 'Display', None],
  [ 0xdf7d, 'UnicodeCharacter0xdf7d', [], 'Display', None],
  [ 0xdf7e, 'UnicodeCharacter0xdf7e', [], 'Display', None],
  [ 0xdf7f, 'UnicodeCharacter0xdf7f', [], 'Display', None],
  [ 0xdf80, 'UnicodeCharacter0xdf80', [], 'Display', None],
  [ 0xdf81, 'UnicodeCharacter0xdf81', [], 'Display', None],
  [ 0xdf82, 'UnicodeCharacter0xdf82', [], 'Display', None],
  [ 0xdf83, 'UnicodeCharacter0xdf83', [], 'Display', None],
  [ 0xdf84, 'UnicodeCharacter0xdf84', [], 'Display', None],
  [ 0xdf85, 'UnicodeCharacter0xdf85', [], 'Display', None],
  [ 0xdf86, 'UnicodeCharacter0xdf86', [], 'Display', None],
  [ 0xdf87, 'UnicodeCharacter0xdf87', [], 'Display', None],
  [ 0xdf88, 'UnicodeCharacter0xdf88', [], 'Display', None],
  [ 0xdf89, 'UnicodeCharacter0xdf89', [], 'Display', None],
  [ 0xdf8a, 'UnicodeCharacter0xdf8a', [], 'Display', None],
  [ 0xdf8b, 'UnicodeCharacter0xdf8b', [], 'Display', None],
  [ 0xdf8c, 'UnicodeCharacter0xdf8c', [], 'Display', None],
  [ 0xdf8d, 'UnicodeCharacter0xdf8d', [], 'Display', None],
  [ 0xdf8e, 'UnicodeCharacter0xdf8e', [], 'Display', None],
  [ 0xdf8f, 'UnicodeCharacter0xdf8f', [], 'Display', None],
  [ 0xdf90, 'UnicodeCharacter0xdf90', [], 'Display', None],
  [ 0xdf91, 'UnicodeCharacter0xdf91', [], 'Display', None],
  [ 0xdf92, 'UnicodeCharacter0xdf92', [], 'Display', None],
  [ 0xdf93, 'UnicodeCharacter0xdf93', [], 'Display', None],
  [ 0xdf94, 'UnicodeCharacter0xdf94', [], 'Display', None],
  [ 0xdf95, 'UnicodeCharacter0xdf95', [], 'Display', None],
  [ 0xdf96, 'UnicodeCharacter0xdf96', [], 'Display', None],
  [ 0xdf97, 'UnicodeCharacter0xdf97', [], 'Display', None],
  [ 0xdf98, 'UnicodeCharacter0xdf98', [], 'Display', None],
  [ 0xdf99, 'UnicodeCharacter0xdf99', [], 'Display', None],
  [ 0xdf9a, 'UnicodeCharacter0xdf9a', [], 'Display', None],
  [ 0xdf9b, 'UnicodeCharacter0xdf9b', [], 'Display', None],
  [ 0xdf9c, 'UnicodeCharacter0xdf9c', [], 'Display', None],
  [ 0xdf9d, 'UnicodeCharacter0xdf9d', [], 'Display', None],
  [ 0xdf9e, 'UnicodeCharacter0xdf9e', [], 'Display', None],
  [ 0xdf9f, 'UnicodeCharacter0xdf9f', [], 'Display', None],
  [ 0xdfa0, 'UnicodeCharacter0xdfa0', [], 'Display', None],
  [ 0xdfa1, 'UnicodeCharacter0xdfa1', [], 'Display', None],
  [ 0xdfa2, 'UnicodeCharacter0xdfa2', [], 'Display', None],
  [ 0xdfa3, 'UnicodeCharacter0xdfa3', [], 'Display', None],
  [ 0xdfa4, 'UnicodeCharacter0xdfa4', [], 'Display', None],
  [ 0xdfa5, 'UnicodeCharacter0xdfa5', [], 'Display', None],
  [ 0xdfa6, 'UnicodeCharacter0xdfa6', [], 'Display', None],
  [ 0xdfa7, 'UnicodeCharacter0xdfa7', [], 'Display', None],
  [ 0xdfa8, 'UnicodeCharacter0xdfa8', [], 'Display', None],
  [ 0xdfa9, 'UnicodeCharacter0xdfa9', [], 'Display', None],
  [ 0xdfaa, 'UnicodeCharacter0xdfaa', [], 'Display', None],
  [ 0xdfab, 'UnicodeCharacter0xdfab', [], 'Display', None],
  [ 0xdfac, 'UnicodeCharacter0xdfac', [], 'Display', None],
  [ 0xdfad, 'UnicodeCharacter0xdfad', [], 'Display', None],
  [ 0xdfae, 'UnicodeCharacter0xdfae', [], 'Display', None],
  [ 0xdfaf, 'UnicodeCharacter0xdfaf', [], 'Display', None],
  [ 0xdfb0, 'UnicodeCharacter0xdfb0', [], 'Display', None],
  [ 0xdfb1, 'UnicodeCharacter0xdfb1', [], 'Display', None],
  [ 0xdfb2, 'UnicodeCharacter0xdfb2', [], 'Display', None],
  [ 0xdfb3, 'UnicodeCharacter0xdfb3', [], 'Display', None],
  [ 0xdfb4, 'UnicodeCharacter0xdfb4', [], 'Display', None],
  [ 0xdfb5, 'UnicodeCharacter0xdfb5', [], 'Display', None],
  [ 0xdfb6, 'UnicodeCharacter0xdfb6', [], 'Display', None],
  [ 0xdfb7, 'UnicodeCharacter0xdfb7', [], 'Display', None],
  [ 0xdfb8, 'UnicodeCharacter0xdfb8', [], 'Display', None],
  [ 0xdfb9, 'UnicodeCharacter0xdfb9', [], 'Display', None],
  [ 0xdfba, 'UnicodeCharacter0xdfba', [], 'Display', None],
  [ 0xdfbb, 'UnicodeCharacter0xdfbb', [], 'Display', None],
  [ 0xdfbc, 'UnicodeCharacter0xdfbc', [], 'Display', None],
  [ 0xdfbd, 'UnicodeCharacter0xdfbd', [], 'Display', None],
  [ 0xdfbe, 'UnicodeCharacter0xdfbe', [], 'Display', None],
  [ 0xdfbf, 'UnicodeCharacter0xdfbf', [], 'Display', None],
  [ 0xdfc0, 'UnicodeCharacter0xdfc0', [], 'Display', None],
  [ 0xdfc1, 'UnicodeCharacter0xdfc1', [], 'Display', None],
  [ 0xdfc2, 'UnicodeCharacter0xdfc2', [], 'Display', None],
  [ 0xdfc3, 'UnicodeCharacter0xdfc3', [], 'Display', None],
  [ 0xdfc4, 'UnicodeCharacter0xdfc4', [], 'Display', None],
  [ 0xdfc5, 'UnicodeCharacter0xdfc5', [], 'Display', None],
  [ 0xdfc6, 'UnicodeCharacter0xdfc6', [], 'Display', None],
  [ 0xdfc7, 'UnicodeCharacter0xdfc7', [], 'Display', None],
  [ 0xdfc8, 'UnicodeCharacter0xdfc8', [], 'Display', None],
  [ 0xdfc9, 'UnicodeCharacter0xdfc9', [], 'Display', None],
  [ 0xdfca, 'UnicodeCharacter0xdfca', [], 'Display', None],
  [ 0xdfcb, 'UnicodeCharacter0xdfcb', [], 'Display', None],
  [ 0xdfcc, 'UnicodeCharacter0xdfcc', [], 'Display', None],
  [ 0xdfcd, 'UnicodeCharacter0xdfcd', [], 'Display', None],
  [ 0xdfce, 'UnicodeCharacter0xdfce', [], 'Display', None],
  [ 0xdfcf, 'UnicodeCharacter0xdfcf', [], 'Display', None],
  [ 0xdfd0, 'UnicodeCharacter0xdfd0', [], 'Display', None],
  [ 0xdfd1, 'UnicodeCharacter0xdfd1', [], 'Display', None],
  [ 0xdfd2, 'UnicodeCharacter0xdfd2', [], 'Display', None],
  [ 0xdfd3, 'UnicodeCharacter0xdfd3', [], 'Display', None],
  [ 0xdfd4, 'UnicodeCharacter0xdfd4', [], 'Display', None],
  [ 0xdfd5, 'UnicodeCharacter0xdfd5', [], 'Display', None],
  [ 0xdfd6, 'UnicodeCharacter0xdfd6', [], 'Display', None],
  [ 0xdfd7, 'UnicodeCharacter0xdfd7', [], 'Display', None],
  [ 0xdfd8, 'UnicodeCharacter0xdfd8', [], 'Display', None],
  [ 0xdfd9, 'UnicodeCharacter0xdfd9', [], 'Display', None],
  [ 0xdfda, 'UnicodeCharacter0xdfda', [], 'Display', None],
  [ 0xdfdb, 'UnicodeCharacter0xdfdb', [], 'Display', None],
  [ 0xdfdc, 'UnicodeCharacter0xdfdc', [], 'Display', None],
  [ 0xdfdd, 'UnicodeCharacter0xdfdd', [], 'Display', None],
  [ 0xdfde, 'UnicodeCharacter0xdfde', [], 'Display', None],
  [ 0xdfdf, 'UnicodeCharacter0xdfdf', [], 'Display', None],
  [ 0xdfe0, 'UnicodeCharacter0xdfe0', [], 'Display', None],
  [ 0xdfe1, 'UnicodeCharacter0xdfe1', [], 'Display', None],
  [ 0xdfe2, 'UnicodeCharacter0xdfe2', [], 'Display', None],
  [ 0xdfe3, 'UnicodeCharacter0xdfe3', [], 'Display', None],
  [ 0xdfe4, 'UnicodeCharacter0xdfe4', [], 'Display', None],
  [ 0xdfe5, 'UnicodeCharacter0xdfe5', [], 'Display', None],
  [ 0xdfe6, 'UnicodeCharacter0xdfe6', [], 'Display', None],
  [ 0xdfe7, 'UnicodeCharacter0xdfe7', [], 'Display', None],
  [ 0xdfe8, 'UnicodeCharacter0xdfe8', [], 'Display', None],
  [ 0xdfe9, 'UnicodeCharacter0xdfe9', [], 'Display', None],
  [ 0xdfea, 'UnicodeCharacter0xdfea', [], 'Display', None],
  [ 0xdfeb, 'UnicodeCharacter0xdfeb', [], 'Display', None],
  [ 0xdfec, 'UnicodeCharacter0xdfec', [], 'Display', None],
  [ 0xdfed, 'UnicodeCharacter0xdfed', [], 'Display', None],
  [ 0xdfee, 'UnicodeCharacter0xdfee', [], 'Display', None],
  [ 0xdfef, 'UnicodeCharacter0xdfef', [], 'Display', None],
  [ 0xdff0, 'UnicodeCharacter0xdff0', [], 'Display', None],
  [ 0xdff1, 'UnicodeCharacter0xdff1', [], 'Display', None],
  [ 0xdff2, 'UnicodeCharacter0xdff2', [], 'Display', None],
  [ 0xdff3, 'UnicodeCharacter0xdff3', [], 'Display', None],
  [ 0xdff4, 'UnicodeCharacter0xdff4', [], 'Display', None],
  [ 0xdff5, 'UnicodeCharacter0xdff5', [], 'Display', None],
  [ 0xdff6, 'UnicodeCharacter0xdff6', [], 'Display', None],
  [ 0xdff7, 'UnicodeCharacter0xdff7', [], 'Display', None],
  [ 0xdff8, 'UnicodeCharacter0xdff8', [], 'Display', None],
  [ 0xdff9, 'UnicodeCharacter0xdff9', [], 'Display', None],
  [ 0xdffa, 'UnicodeCharacter0xdffa', [], 'Display', None],
  [ 0xdffb, 'UnicodeCharacter0xdffb', [], 'Display', None],
  [ 0xdffc, 'UnicodeCharacter0xdffc', [], 'Display', None],
  [ 0xdffd, 'UnicodeCharacter0xdffd', [], 'Display', None],
  [ 0xdffe, 'UnicodeCharacter0xdffe', [], 'Display', None],
  [ 0xdfff, 'UnicodeCharacter0xdfff', [], 'Display', None],
  [ 0xe000, 'UnicodeCharacter0xe000', [], 'Display', None],
  [ 0xe001, 'UnicodeCharacter0xe001', [], 'Display', None],
  [ 0xe002, 'UnicodeCharacter0xe002', [], 'Display', None],
  [ 0xe003, 'UnicodeCharacter0xe003', [], 'Display', None],
  [ 0xe004, 'UnicodeCharacter0xe004', [], 'Display', None],
  [ 0xe005, 'UnicodeCharacter0xe005', [], 'Display', None],
  [ 0xe006, 'UnicodeCharacter0xe006', [], 'Display', None],
  [ 0xe007, 'UnicodeCharacter0xe007', [], 'Display', None],
  [ 0xe008, 'UnicodeCharacter0xe008', [], 'Display', None],
  [ 0xe009, 'UnicodeCharacter0xe009', [], 'Display', None],
  [ 0xe00a, 'UnicodeCharacter0xe00a', [], 'Display', None],
  [ 0xe00b, 'UnicodeCharacter0xe00b', [], 'Display', None],
  [ 0xe00c, 'UnicodeCharacter0xe00c', [], 'Display', None],
  [ 0xe00d, 'UnicodeCharacter0xe00d', [], 'Display', None],
  [ 0xe00e, 'UnicodeCharacter0xe00e', [], 'Display', None],
  [ 0xe00f, 'UnicodeCharacter0xe00f', [], 'Display', None],
  [ 0xe010, 'UnicodeCharacter0xe010', [], 'Display', None],
  [ 0xe011, 'UnicodeCharacter0xe011', [], 'Display', None],
  [ 0xe012, 'UnicodeCharacter0xe012', [], 'Display', None],
  [ 0xe013, 'UnicodeCharacter0xe013', [], 'Display', None],
  [ 0xe014, 'UnicodeCharacter0xe014', [], 'Display', None],
  [ 0xe015, 'UnicodeCharacter0xe015', [], 'Display', None],
  [ 0xe016, 'UnicodeCharacter0xe016', [], 'Display', None],
  [ 0xe017, 'UnicodeCharacter0xe017', [], 'Display', None],
  [ 0xe018, 'UnicodeCharacter0xe018', [], 'Display', None],
  [ 0xe019, 'UnicodeCharacter0xe019', [], 'Display', None],
  [ 0xe01a, 'UnicodeCharacter0xe01a', [], 'Display', None],
  [ 0xe01b, 'UnicodeCharacter0xe01b', [], 'Display', None],
  [ 0xe01c, 'UnicodeCharacter0xe01c', [], 'Display', None],
  [ 0xe01d, 'UnicodeCharacter0xe01d', [], 'Display', None],
  [ 0xe01e, 'UnicodeCharacter0xe01e', [], 'Display', None],
  [ 0xe01f, 'UnicodeCharacter0xe01f', [], 'Display', None],
  [ 0xe020, 'UnicodeCharacter0xe020', [], 'Display', None],
  [ 0xe021, 'UnicodeCharacter0xe021', [], 'Display', None],
  [ 0xe022, 'UnicodeCharacter0xe022', [], 'Display', None],
  [ 0xe023, 'UnicodeCharacter0xe023', [], 'Display', None],
  [ 0xe024, 'UnicodeCharacter0xe024', [], 'Display', None],
  [ 0xe025, 'UnicodeCharacter0xe025', [], 'Display', None],
  [ 0xe026, 'UnicodeCharacter0xe026', [], 'Display', None],
  [ 0xe027, 'UnicodeCharacter0xe027', [], 'Display', None],
  [ 0xe028, 'UnicodeCharacter0xe028', [], 'Display', None],
  [ 0xe029, 'UnicodeCharacter0xe029', [], 'Display', None],
  [ 0xe02a, 'UnicodeCharacter0xe02a', [], 'Display', None],
  [ 0xe02b, 'UnicodeCharacter0xe02b', [], 'Display', None],
  [ 0xe02c, 'UnicodeCharacter0xe02c', [], 'Display', None],
  [ 0xe02d, 'UnicodeCharacter0xe02d', [], 'Display', None],
  [ 0xe02e, 'UnicodeCharacter0xe02e', [], 'Display', None],
  [ 0xe02f, 'UnicodeCharacter0xe02f', [], 'Display', None],
  [ 0xe030, 'UnicodeCharacter0xe030', [], 'Display', None],
  [ 0xe031, 'UnicodeCharacter0xe031', [], 'Display', None],
  [ 0xe032, 'UnicodeCharacter0xe032', [], 'Display', None],
  [ 0xe033, 'UnicodeCharacter0xe033', [], 'Display', None],
  [ 0xe034, 'UnicodeCharacter0xe034', [], 'Display', None],
  [ 0xe035, 'UnicodeCharacter0xe035', [], 'Display', None],
  [ 0xe036, 'UnicodeCharacter0xe036', [], 'Display', None],
  [ 0xe037, 'UnicodeCharacter0xe037', [], 'Display', None],
  [ 0xe038, 'UnicodeCharacter0xe038', [], 'Display', None],
  [ 0xe039, 'UnicodeCharacter0xe039', [], 'Display', None],
  [ 0xe03a, 'UnicodeCharacter0xe03a', [], 'Display', None],
  [ 0xe03b, 'UnicodeCharacter0xe03b', [], 'Display', None],
  [ 0xe03c, 'UnicodeCharacter0xe03c', [], 'Display', None],
  [ 0xe03d, 'UnicodeCharacter0xe03d', [], 'Display', None],
  [ 0xe03e, 'UnicodeCharacter0xe03e', [], 'Display', None],
  [ 0xe03f, 'UnicodeCharacter0xe03f', [], 'Display', None],
  [ 0xe040, 'UnicodeCharacter0xe040', [], 'Display', None],
  [ 0xe041, 'UnicodeCharacter0xe041', [], 'Display', None],
  [ 0xe042, 'UnicodeCharacter0xe042', [], 'Display', None],
  [ 0xe043, 'UnicodeCharacter0xe043', [], 'Display', None],
  [ 0xe044, 'UnicodeCharacter0xe044', [], 'Display', None],
  [ 0xe045, 'UnicodeCharacter0xe045', [], 'Display', None],
  [ 0xe046, 'UnicodeCharacter0xe046', [], 'Display', None],
  [ 0xe047, 'UnicodeCharacter0xe047', [], 'Display', None],
  [ 0xe048, 'UnicodeCharacter0xe048', [], 'Display', None],
  [ 0xe049, 'UnicodeCharacter0xe049', [], 'Display', None],
  [ 0xe04a, 'UnicodeCharacter0xe04a', [], 'Display', None],
  [ 0xe04b, 'UnicodeCharacter0xe04b', [], 'Display', None],
  [ 0xe04c, 'UnicodeCharacter0xe04c', [], 'Display', None],
  [ 0xe04d, 'UnicodeCharacter0xe04d', [], 'Display', None],
  [ 0xe04e, 'UnicodeCharacter0xe04e', [], 'Display', None],
  [ 0xe04f, 'UnicodeCharacter0xe04f', [], 'Display', None],
  [ 0xe050, 'UnicodeCharacter0xe050', [], 'Display', None],
  [ 0xe051, 'UnicodeCharacter0xe051', [], 'Display', None],
  [ 0xe052, 'UnicodeCharacter0xe052', [], 'Display', None],
  [ 0xe053, 'UnicodeCharacter0xe053', [], 'Display', None],
  [ 0xe054, 'UnicodeCharacter0xe054', [], 'Display', None],
  [ 0xe055, 'UnicodeCharacter0xe055', [], 'Display', None],
  [ 0xe056, 'UnicodeCharacter0xe056', [], 'Display', None],
  [ 0xe057, 'UnicodeCharacter0xe057', [], 'Display', None],
  [ 0xe058, 'UnicodeCharacter0xe058', [], 'Display', None],
  [ 0xe059, 'UnicodeCharacter0xe059', [], 'Display', None],
  [ 0xe05a, 'UnicodeCharacter0xe05a', [], 'Display', None],
  [ 0xe05b, 'UnicodeCharacter0xe05b', [], 'Display', None],
  [ 0xe05c, 'UnicodeCharacter0xe05c', [], 'Display', None],
  [ 0xe05d, 'UnicodeCharacter0xe05d', [], 'Display', None],
  [ 0xe05e, 'UnicodeCharacter0xe05e', [], 'Display', None],
  [ 0xe05f, 'UnicodeCharacter0xe05f', [], 'Display', None],
  [ 0xe060, 'UnicodeCharacter0xe060', [], 'Display', None],
  [ 0xe061, 'UnicodeCharacter0xe061', [], 'Display', None],
  [ 0xe062, 'UnicodeCharacter0xe062', [], 'Display', None],
  [ 0xe063, 'UnicodeCharacter0xe063', [], 'Display', None],
  [ 0xe064, 'UnicodeCharacter0xe064', [], 'Display', None],
  [ 0xe065, 'UnicodeCharacter0xe065', [], 'Display', None],
  [ 0xe066, 'UnicodeCharacter0xe066', [], 'Display', None],
  [ 0xe067, 'UnicodeCharacter0xe067', [], 'Display', None],
  [ 0xe068, 'UnicodeCharacter0xe068', [], 'Display', None],
  [ 0xe069, 'UnicodeCharacter0xe069', [], 'Display', None],
  [ 0xe06a, 'UnicodeCharacter0xe06a', [], 'Display', None],
  [ 0xe06b, 'UnicodeCharacter0xe06b', [], 'Display', None],
  [ 0xe06c, 'UnicodeCharacter0xe06c', [], 'Display', None],
  [ 0xe06d, 'UnicodeCharacter0xe06d', [], 'Display', None],
  [ 0xe06e, 'UnicodeCharacter0xe06e', [], 'Display', None],
  [ 0xe06f, 'UnicodeCharacter0xe06f', [], 'Display', None],
  [ 0xe070, 'UnicodeCharacter0xe070', [], 'Display', None],
  [ 0xe071, 'UnicodeCharacter0xe071', [], 'Display', None],
  [ 0xe072, 'UnicodeCharacter0xe072', [], 'Display', None],
  [ 0xe073, 'UnicodeCharacter0xe073', [], 'Display', None],
  [ 0xe074, 'UnicodeCharacter0xe074', [], 'Display', None],
  [ 0xe075, 'UnicodeCharacter0xe075', [], 'Display', None],
  [ 0xe076, 'UnicodeCharacter0xe076', [], 'Display', None],
  [ 0xe077, 'UnicodeCharacter0xe077', [], 'Display', None],
  [ 0xe078, 'UnicodeCharacter0xe078', [], 'Display', None],
  [ 0xe079, 'UnicodeCharacter0xe079', [], 'Display', None],
  [ 0xe07a, 'UnicodeCharacter0xe07a', [], 'Display', None],
  [ 0xe07b, 'UnicodeCharacter0xe07b', [], 'Display', None],
  [ 0xe07c, 'UnicodeCharacter0xe07c', [], 'Display', None],
  [ 0xe07d, 'UnicodeCharacter0xe07d', [], 'Display', None],
  [ 0xe07e, 'UnicodeCharacter0xe07e', [], 'Display', None],
  [ 0xe07f, 'UnicodeCharacter0xe07f', [], 'Display', None],
  [ 0xe080, 'UnicodeCharacter0xe080', [], 'Display', None],
  [ 0xe081, 'UnicodeCharacter0xe081', [], 'Display', None],
  [ 0xe082, 'UnicodeCharacter0xe082', [], 'Display', None],
  [ 0xe083, 'UnicodeCharacter0xe083', [], 'Display', None],
  [ 0xe084, 'UnicodeCharacter0xe084', [], 'Display', None],
  [ 0xe085, 'UnicodeCharacter0xe085', [], 'Display', None],
  [ 0xe086, 'UnicodeCharacter0xe086', [], 'Display', None],
  [ 0xe087, 'UnicodeCharacter0xe087', [], 'Display', None],
  [ 0xe088, 'UnicodeCharacter0xe088', [], 'Display', None],
  [ 0xe089, 'UnicodeCharacter0xe089', [], 'Display', None],
  [ 0xe08a, 'UnicodeCharacter0xe08a', [], 'Display', None],
  [ 0xe08b, 'UnicodeCharacter0xe08b', [], 'Display', None],
  [ 0xe08c, 'UnicodeCharacter0xe08c', [], 'Display', None],
  [ 0xe08d, 'UnicodeCharacter0xe08d', [], 'Display', None],
  [ 0xe08e, 'UnicodeCharacter0xe08e', [], 'Display', None],
  [ 0xe08f, 'UnicodeCharacter0xe08f', [], 'Display', None],
  [ 0xe090, 'UnicodeCharacter0xe090', [], 'Display', None],
  [ 0xe091, 'UnicodeCharacter0xe091', [], 'Display', None],
  [ 0xe092, 'UnicodeCharacter0xe092', [], 'Display', None],
  [ 0xe093, 'UnicodeCharacter0xe093', [], 'Display', None],
  [ 0xe094, 'UnicodeCharacter0xe094', [], 'Display', None],
  [ 0xe095, 'UnicodeCharacter0xe095', [], 'Display', None],
  [ 0xe096, 'UnicodeCharacter0xe096', [], 'Display', None],
  [ 0xe097, 'UnicodeCharacter0xe097', [], 'Display', None],
  [ 0xe098, 'UnicodeCharacter0xe098', [], 'Display', None],
  [ 0xe099, 'UnicodeCharacter0xe099', [], 'Display', None],
  [ 0xe09a, 'UnicodeCharacter0xe09a', [], 'Display', None],
  [ 0xe09b, 'UnicodeCharacter0xe09b', [], 'Display', None],
  [ 0xe09c, 'UnicodeCharacter0xe09c', [], 'Display', None],
  [ 0xe09d, 'UnicodeCharacter0xe09d', [], 'Display', None],
  [ 0xe09e, 'UnicodeCharacter0xe09e', [], 'Display', None],
  [ 0xe09f, 'UnicodeCharacter0xe09f', [], 'Display', None],
  [ 0xe0a0, 'UnicodeCharacter0xe0a0', [], 'Display', None],
  [ 0xe0a1, 'UnicodeCharacter0xe0a1', [], 'Display', None],
  [ 0xe0a2, 'UnicodeCharacter0xe0a2', [], 'Display', None],
  [ 0xe0a3, 'UnicodeCharacter0xe0a3', [], 'Display', None],
  [ 0xe0a4, 'UnicodeCharacter0xe0a4', [], 'Display', None],
  [ 0xe0a5, 'UnicodeCharacter0xe0a5', [], 'Display', None],
  [ 0xe0a6, 'UnicodeCharacter0xe0a6', [], 'Display', None],
  [ 0xe0a7, 'UnicodeCharacter0xe0a7', [], 'Display', None],
  [ 0xe0a8, 'UnicodeCharacter0xe0a8', [], 'Display', None],
  [ 0xe0a9, 'UnicodeCharacter0xe0a9', [], 'Display', None],
  [ 0xe0aa, 'UnicodeCharacter0xe0aa', [], 'Display', None],
  [ 0xe0ab, 'UnicodeCharacter0xe0ab', [], 'Display', None],
  [ 0xe0ac, 'UnicodeCharacter0xe0ac', [], 'Display', None],
  [ 0xe0ad, 'UnicodeCharacter0xe0ad', [], 'Display', None],
  [ 0xe0ae, 'UnicodeCharacter0xe0ae', [], 'Display', None],
  [ 0xe0af, 'UnicodeCharacter0xe0af', [], 'Display', None],
  [ 0xe0b0, 'UnicodeCharacter0xe0b0', [], 'Display', None],
  [ 0xe0b1, 'UnicodeCharacter0xe0b1', [], 'Display', None],
  [ 0xe0b2, 'UnicodeCharacter0xe0b2', [], 'Display', None],
  [ 0xe0b3, 'UnicodeCharacter0xe0b3', [], 'Display', None],
  [ 0xe0b4, 'UnicodeCharacter0xe0b4', [], 'Display', None],
  [ 0xe0b5, 'UnicodeCharacter0xe0b5', [], 'Display', None],
  [ 0xe0b6, 'UnicodeCharacter0xe0b6', [], 'Display', None],
  [ 0xe0b7, 'UnicodeCharacter0xe0b7', [], 'Display', None],
  [ 0xe0b8, 'UnicodeCharacter0xe0b8', [], 'Display', None],
  [ 0xe0b9, 'UnicodeCharacter0xe0b9', [], 'Display', None],
  [ 0xe0ba, 'UnicodeCharacter0xe0ba', [], 'Display', None],
  [ 0xe0bb, 'UnicodeCharacter0xe0bb', [], 'Display', None],
  [ 0xe0bc, 'UnicodeCharacter0xe0bc', [], 'Display', None],
  [ 0xe0bd, 'UnicodeCharacter0xe0bd', [], 'Display', None],
  [ 0xe0be, 'UnicodeCharacter0xe0be', [], 'Display', None],
  [ 0xe0bf, 'UnicodeCharacter0xe0bf', [], 'Display', None],
  [ 0xe0c0, 'UnicodeCharacter0xe0c0', [], 'Display', None],
  [ 0xe0c1, 'UnicodeCharacter0xe0c1', [], 'Display', None],
  [ 0xe0c2, 'UnicodeCharacter0xe0c2', [], 'Display', None],
  [ 0xe0c3, 'UnicodeCharacter0xe0c3', [], 'Display', None],
  [ 0xe0c4, 'UnicodeCharacter0xe0c4', [], 'Display', None],
  [ 0xe0c5, 'UnicodeCharacter0xe0c5', [], 'Display', None],
  [ 0xe0c6, 'UnicodeCharacter0xe0c6', [], 'Display', None],
  [ 0xe0c7, 'UnicodeCharacter0xe0c7', [], 'Display', None],
  [ 0xe0c8, 'UnicodeCharacter0xe0c8', [], 'Display', None],
  [ 0xe0c9, 'UnicodeCharacter0xe0c9', [], 'Display', None],
  [ 0xe0ca, 'UnicodeCharacter0xe0ca', [], 'Display', None],
  [ 0xe0cb, 'UnicodeCharacter0xe0cb', [], 'Display', None],
  [ 0xe0cc, 'UnicodeCharacter0xe0cc', [], 'Display', None],
  [ 0xe0cd, 'UnicodeCharacter0xe0cd', [], 'Display', None],
  [ 0xe0ce, 'UnicodeCharacter0xe0ce', [], 'Display', None],
  [ 0xe0cf, 'UnicodeCharacter0xe0cf', [], 'Display', None],
  [ 0xe0d0, 'UnicodeCharacter0xe0d0', [], 'Display', None],
  [ 0xe0d1, 'UnicodeCharacter0xe0d1', [], 'Display', None],
  [ 0xe0d2, 'UnicodeCharacter0xe0d2', [], 'Display', None],
  [ 0xe0d3, 'UnicodeCharacter0xe0d3', [], 'Display', None],
  [ 0xe0d4, 'UnicodeCharacter0xe0d4', [], 'Display', None],
  [ 0xe0d5, 'UnicodeCharacter0xe0d5', [], 'Display', None],
  [ 0xe0d6, 'UnicodeCharacter0xe0d6', [], 'Display', None],
  [ 0xe0d7, 'UnicodeCharacter0xe0d7', [], 'Display', None],
  [ 0xe0d8, 'UnicodeCharacter0xe0d8', [], 'Display', None],
  [ 0xe0d9, 'UnicodeCharacter0xe0d9', [], 'Display', None],
  [ 0xe0da, 'UnicodeCharacter0xe0da', [], 'Display', None],
  [ 0xe0db, 'UnicodeCharacter0xe0db', [], 'Display', None],
  [ 0xe0dc, 'UnicodeCharacter0xe0dc', [], 'Display', None],
  [ 0xe0dd, 'UnicodeCharacter0xe0dd', [], 'Display', None],
  [ 0xe0de, 'UnicodeCharacter0xe0de', [], 'Display', None],
  [ 0xe0df, 'UnicodeCharacter0xe0df', [], 'Display', None],
  [ 0xe0e0, 'UnicodeCharacter0xe0e0', [], 'Display', None],
  [ 0xe0e1, 'UnicodeCharacter0xe0e1', [], 'Display', None],
  [ 0xe0e2, 'UnicodeCharacter0xe0e2', [], 'Display', None],
  [ 0xe0e3, 'UnicodeCharacter0xe0e3', [], 'Display', None],
  [ 0xe0e4, 'UnicodeCharacter0xe0e4', [], 'Display', None],
  [ 0xe0e5, 'UnicodeCharacter0xe0e5', [], 'Display', None],
  [ 0xe0e6, 'UnicodeCharacter0xe0e6', [], 'Display', None],
  [ 0xe0e7, 'UnicodeCharacter0xe0e7', [], 'Display', None],
  [ 0xe0e8, 'UnicodeCharacter0xe0e8', [], 'Display', None],
  [ 0xe0e9, 'UnicodeCharacter0xe0e9', [], 'Display', None],
  [ 0xe0ea, 'UnicodeCharacter0xe0ea', [], 'Display', None],
  [ 0xe0eb, 'UnicodeCharacter0xe0eb', [], 'Display', None],
  [ 0xe0ec, 'UnicodeCharacter0xe0ec', [], 'Display', None],
  [ 0xe0ed, 'UnicodeCharacter0xe0ed', [], 'Display', None],
  [ 0xe0ee, 'UnicodeCharacter0xe0ee', [], 'Display', None],
  [ 0xe0ef, 'UnicodeCharacter0xe0ef', [], 'Display', None],
  [ 0xe0f0, 'UnicodeCharacter0xe0f0', [], 'Display', None],
  [ 0xe0f1, 'UnicodeCharacter0xe0f1', [], 'Display', None],
  [ 0xe0f2, 'UnicodeCharacter0xe0f2', [], 'Display', None],
  [ 0xe0f3, 'UnicodeCharacter0xe0f3', [], 'Display', None],
  [ 0xe0f4, 'UnicodeCharacter0xe0f4', [], 'Display', None],
  [ 0xe0f5, 'UnicodeCharacter0xe0f5', [], 'Display', None],
  [ 0xe0f6, 'UnicodeCharacter0xe0f6', [], 'Display', None],
  [ 0xe0f7, 'UnicodeCharacter0xe0f7', [], 'Display', None],
  [ 0xe0f8, 'UnicodeCharacter0xe0f8', [], 'Display', None],
  [ 0xe0f9, 'UnicodeCharacter0xe0f9', [], 'Display', None],
  [ 0xe0fa, 'UnicodeCharacter0xe0fa', [], 'Display', None],
  [ 0xe0fb, 'UnicodeCharacter0xe0fb', [], 'Display', None],
  [ 0xe0fc, 'UnicodeCharacter0xe0fc', [], 'Display', None],
  [ 0xe0fd, 'UnicodeCharacter0xe0fd', [], 'Display', None],
  [ 0xe0fe, 'UnicodeCharacter0xe0fe', [], 'Display', None],
  [ 0xe0ff, 'UnicodeCharacter0xe0ff', [], 'Display', None],
  [ 0xe100, 'UnicodeCharacter0xe100', [], 'Display', None],
  [ 0xe101, 'UnicodeCharacter0xe101', [], 'Display', None],
  [ 0xe102, 'UnicodeCharacter0xe102', [], 'Display', None],
  [ 0xe103, 'UnicodeCharacter0xe103', [], 'Display', None],
  [ 0xe104, 'UnicodeCharacter0xe104', [], 'Display', None],
  [ 0xe105, 'UnicodeCharacter0xe105', [], 'Display', None],
  [ 0xe106, 'UnicodeCharacter0xe106', [], 'Display', None],
  [ 0xe107, 'UnicodeCharacter0xe107', [], 'Display', None],
  [ 0xe108, 'UnicodeCharacter0xe108', [], 'Display', None],
  [ 0xe109, 'UnicodeCharacter0xe109', [], 'Display', None],
  [ 0xe10a, 'UnicodeCharacter0xe10a', [], 'Display', None],
  [ 0xe10b, 'UnicodeCharacter0xe10b', [], 'Display', None],
  [ 0xe10c, 'UnicodeCharacter0xe10c', [], 'Display', None],
  [ 0xe10d, 'UnicodeCharacter0xe10d', [], 'Display', None],
  [ 0xe10e, 'UnicodeCharacter0xe10e', [], 'Display', None],
  [ 0xe10f, 'UnicodeCharacter0xe10f', [], 'Display', None],
  [ 0xe110, 'UnicodeCharacter0xe110', [], 'Display', None],
  [ 0xe111, 'UnicodeCharacter0xe111', [], 'Display', None],
  [ 0xe112, 'UnicodeCharacter0xe112', [], 'Display', None],
  [ 0xe113, 'UnicodeCharacter0xe113', [], 'Display', None],
  [ 0xe114, 'UnicodeCharacter0xe114', [], 'Display', None],
  [ 0xe115, 'UnicodeCharacter0xe115', [], 'Display', None],
  [ 0xe116, 'UnicodeCharacter0xe116', [], 'Display', None],
  [ 0xe117, 'UnicodeCharacter0xe117', [], 'Display', None],
  [ 0xe118, 'UnicodeCharacter0xe118', [], 'Display', None],
  [ 0xe119, 'UnicodeCharacter0xe119', [], 'Display', None],
  [ 0xe11a, 'UnicodeCharacter0xe11a', [], 'Display', None],
  [ 0xe11b, 'UnicodeCharacter0xe11b', [], 'Display', None],
  [ 0xe11c, 'UnicodeCharacter0xe11c', [], 'Display', None],
  [ 0xe11d, 'UnicodeCharacter0xe11d', [], 'Display', None],
  [ 0xe11e, 'UnicodeCharacter0xe11e', [], 'Display', None],
  [ 0xe11f, 'UnicodeCharacter0xe11f', [], 'Display', None],
  [ 0xe120, 'UnicodeCharacter0xe120', [], 'Display', None],
  [ 0xe121, 'UnicodeCharacter0xe121', [], 'Display', None],
  [ 0xe122, 'UnicodeCharacter0xe122', [], 'Display', None],
  [ 0xe123, 'UnicodeCharacter0xe123', [], 'Display', None],
  [ 0xe124, 'UnicodeCharacter0xe124', [], 'Display', None],
  [ 0xe125, 'UnicodeCharacter0xe125', [], 'Display', None],
  [ 0xe126, 'UnicodeCharacter0xe126', [], 'Display', None],
  [ 0xe127, 'UnicodeCharacter0xe127', [], 'Display', None],
  [ 0xe128, 'UnicodeCharacter0xe128', [], 'Display', None],
  [ 0xe129, 'UnicodeCharacter0xe129', [], 'Display', None],
  [ 0xe12a, 'UnicodeCharacter0xe12a', [], 'Display', None],
  [ 0xe12b, 'UnicodeCharacter0xe12b', [], 'Display', None],
  [ 0xe12c, 'UnicodeCharacter0xe12c', [], 'Display', None],
  [ 0xe12d, 'UnicodeCharacter0xe12d', [], 'Display', None],
  [ 0xe12e, 'UnicodeCharacter0xe12e', [], 'Display', None],
  [ 0xe12f, 'UnicodeCharacter0xe12f', [], 'Display', None],
  [ 0xe130, 'UnicodeCharacter0xe130', [], 'Display', None],
  [ 0xe131, 'UnicodeCharacter0xe131', [], 'Display', None],
  [ 0xe132, 'UnicodeCharacter0xe132', [], 'Display', None],
  [ 0xe133, 'UnicodeCharacter0xe133', [], 'Display', None],
  [ 0xe134, 'UnicodeCharacter0xe134', [], 'Display', None],
  [ 0xe135, 'UnicodeCharacter0xe135', [], 'Display', None],
  [ 0xe136, 'UnicodeCharacter0xe136', [], 'Display', None],
  [ 0xe137, 'UnicodeCharacter0xe137', [], 'Display', None],
  [ 0xe138, 'UnicodeCharacter0xe138', [], 'Display', None],
  [ 0xe139, 'UnicodeCharacter0xe139', [], 'Display', None],
  [ 0xe13a, 'UnicodeCharacter0xe13a', [], 'Display', None],
  [ 0xe13b, 'UnicodeCharacter0xe13b', [], 'Display', None],
  [ 0xe13c, 'UnicodeCharacter0xe13c', [], 'Display', None],
  [ 0xe13d, 'UnicodeCharacter0xe13d', [], 'Display', None],
  [ 0xe13e, 'UnicodeCharacter0xe13e', [], 'Display', None],
  [ 0xe13f, 'UnicodeCharacter0xe13f', [], 'Display', None],
  [ 0xe140, 'UnicodeCharacter0xe140', [], 'Display', None],
  [ 0xe141, 'UnicodeCharacter0xe141', [], 'Display', None],
  [ 0xe142, 'UnicodeCharacter0xe142', [], 'Display', None],
  [ 0xe143, 'UnicodeCharacter0xe143', [], 'Display', None],
  [ 0xe144, 'UnicodeCharacter0xe144', [], 'Display', None],
  [ 0xe145, 'UnicodeCharacter0xe145', [], 'Display', None],
  [ 0xe146, 'UnicodeCharacter0xe146', [], 'Display', None],
  [ 0xe147, 'UnicodeCharacter0xe147', [], 'Display', None],
  [ 0xe148, 'UnicodeCharacter0xe148', [], 'Display', None],
  [ 0xe149, 'UnicodeCharacter0xe149', [], 'Display', None],
  [ 0xe14a, 'UnicodeCharacter0xe14a', [], 'Display', None],
  [ 0xe14b, 'UnicodeCharacter0xe14b', [], 'Display', None],
  [ 0xe14c, 'UnicodeCharacter0xe14c', [], 'Display', None],
  [ 0xe14d, 'UnicodeCharacter0xe14d', [], 'Display', None],
  [ 0xe14e, 'UnicodeCharacter0xe14e', [], 'Display', None],
  [ 0xe14f, 'UnicodeCharacter0xe14f', [], 'Display', None],
  [ 0xe150, 'UnicodeCharacter0xe150', [], 'Display', None],
  [ 0xe151, 'UnicodeCharacter0xe151', [], 'Display', None],
  [ 0xe152, 'UnicodeCharacter0xe152', [], 'Display', None],
  [ 0xe153, 'UnicodeCharacter0xe153', [], 'Display', None],
  [ 0xe154, 'UnicodeCharacter0xe154', [], 'Display', None],
  [ 0xe155, 'UnicodeCharacter0xe155', [], 'Display', None],
  [ 0xe156, 'UnicodeCharacter0xe156', [], 'Display', None],
  [ 0xe157, 'UnicodeCharacter0xe157', [], 'Display', None],
  [ 0xe158, 'UnicodeCharacter0xe158', [], 'Display', None],
  [ 0xe159, 'UnicodeCharacter0xe159', [], 'Display', None],
  [ 0xe15a, 'UnicodeCharacter0xe15a', [], 'Display', None],
  [ 0xe15b, 'UnicodeCharacter0xe15b', [], 'Display', None],
  [ 0xe15c, 'UnicodeCharacter0xe15c', [], 'Display', None],
  [ 0xe15d, 'UnicodeCharacter0xe15d', [], 'Display', None],
  [ 0xe15e, 'UnicodeCharacter0xe15e', [], 'Display', None],
  [ 0xe15f, 'UnicodeCharacter0xe15f', [], 'Display', None],
  [ 0xe160, 'UnicodeCharacter0xe160', [], 'Display', None],
  [ 0xe161, 'UnicodeCharacter0xe161', [], 'Display', None],
  [ 0xe162, 'UnicodeCharacter0xe162', [], 'Display', None],
  [ 0xe163, 'UnicodeCharacter0xe163', [], 'Display', None],
  [ 0xe164, 'UnicodeCharacter0xe164', [], 'Display', None],
  [ 0xe165, 'UnicodeCharacter0xe165', [], 'Display', None],
  [ 0xe166, 'UnicodeCharacter0xe166', [], 'Display', None],
  [ 0xe167, 'UnicodeCharacter0xe167', [], 'Display', None],
  [ 0xe168, 'UnicodeCharacter0xe168', [], 'Display', None],
  [ 0xe169, 'UnicodeCharacter0xe169', [], 'Display', None],
  [ 0xe16a, 'UnicodeCharacter0xe16a', [], 'Display', None],
  [ 0xe16b, 'UnicodeCharacter0xe16b', [], 'Display', None],
  [ 0xe16c, 'UnicodeCharacter0xe16c', [], 'Display', None],
  [ 0xe16d, 'UnicodeCharacter0xe16d', [], 'Display', None],
  [ 0xe16e, 'UnicodeCharacter0xe16e', [], 'Display', None],
  [ 0xe16f, 'UnicodeCharacter0xe16f', [], 'Display', None],
  [ 0xe170, 'UnicodeCharacter0xe170', [], 'Display', None],
  [ 0xe171, 'UnicodeCharacter0xe171', [], 'Display', None],
  [ 0xe172, 'UnicodeCharacter0xe172', [], 'Display', None],
  [ 0xe173, 'UnicodeCharacter0xe173', [], 'Display', None],
  [ 0xe174, 'UnicodeCharacter0xe174', [], 'Display', None],
  [ 0xe175, 'UnicodeCharacter0xe175', [], 'Display', None],
  [ 0xe176, 'UnicodeCharacter0xe176', [], 'Display', None],
  [ 0xe177, 'UnicodeCharacter0xe177', [], 'Display', None],
  [ 0xe178, 'UnicodeCharacter0xe178', [], 'Display', None],
  [ 0xe179, 'UnicodeCharacter0xe179', [], 'Display', None],
  [ 0xe17a, 'UnicodeCharacter0xe17a', [], 'Display', None],
  [ 0xe17b, 'UnicodeCharacter0xe17b', [], 'Display', None],
  [ 0xe17c, 'UnicodeCharacter0xe17c', [], 'Display', None],
  [ 0xe17d, 'UnicodeCharacter0xe17d', [], 'Display', None],
  [ 0xe17e, 'UnicodeCharacter0xe17e', [], 'Display', None],
  [ 0xe17f, 'UnicodeCharacter0xe17f', [], 'Display', None],
  [ 0xe180, 'UnicodeCharacter0xe180', [], 'Display', None],
  [ 0xe181, 'UnicodeCharacter0xe181', [], 'Display', None],
  [ 0xe182, 'UnicodeCharacter0xe182', [], 'Display', None],
  [ 0xe183, 'UnicodeCharacter0xe183', [], 'Display', None],
  [ 0xe184, 'UnicodeCharacter0xe184', [], 'Display', None],
  [ 0xe185, 'UnicodeCharacter0xe185', [], 'Display', None],
  [ 0xe186, 'UnicodeCharacter0xe186', [], 'Display', None],
  [ 0xe187, 'UnicodeCharacter0xe187', [], 'Display', None],
  [ 0xe188, 'UnicodeCharacter0xe188', [], 'Display', None],
  [ 0xe189, 'UnicodeCharacter0xe189', [], 'Display', None],
  [ 0xe18a, 'UnicodeCharacter0xe18a', [], 'Display', None],
  [ 0xe18b, 'UnicodeCharacter0xe18b', [], 'Display', None],
  [ 0xe18c, 'UnicodeCharacter0xe18c', [], 'Display', None],
  [ 0xe18d, 'UnicodeCharacter0xe18d', [], 'Display', None],
  [ 0xe18e, 'UnicodeCharacter0xe18e', [], 'Display', None],
  [ 0xe18f, 'UnicodeCharacter0xe18f', [], 'Display', None],
  [ 0xe190, 'UnicodeCharacter0xe190', [], 'Display', None],
  [ 0xe191, 'UnicodeCharacter0xe191', [], 'Display', None],
  [ 0xe192, 'UnicodeCharacter0xe192', [], 'Display', None],
  [ 0xe193, 'UnicodeCharacter0xe193', [], 'Display', None],
  [ 0xe194, 'UnicodeCharacter0xe194', [], 'Display', None],
  [ 0xe195, 'UnicodeCharacter0xe195', [], 'Display', None],
  [ 0xe196, 'UnicodeCharacter0xe196', [], 'Display', None],
  [ 0xe197, 'UnicodeCharacter0xe197', [], 'Display', None],
  [ 0xe198, 'UnicodeCharacter0xe198', [], 'Display', None],
  [ 0xe199, 'UnicodeCharacter0xe199', [], 'Display', None],
  [ 0xe19a, 'UnicodeCharacter0xe19a', [], 'Display', None],
  [ 0xe19b, 'UnicodeCharacter0xe19b', [], 'Display', None],
  [ 0xe19c, 'UnicodeCharacter0xe19c', [], 'Display', None],
  [ 0xe19d, 'UnicodeCharacter0xe19d', [], 'Display', None],
  [ 0xe19e, 'UnicodeCharacter0xe19e', [], 'Display', None],
  [ 0xe19f, 'UnicodeCharacter0xe19f', [], 'Display', None],
  [ 0xe1a0, 'UnicodeCharacter0xe1a0', [], 'Display', None],
  [ 0xe1a1, 'UnicodeCharacter0xe1a1', [], 'Display', None],
  [ 0xe1a2, 'UnicodeCharacter0xe1a2', [], 'Display', None],
  [ 0xe1a3, 'UnicodeCharacter0xe1a3', [], 'Display', None],
  [ 0xe1a4, 'UnicodeCharacter0xe1a4', [], 'Display', None],
  [ 0xe1a5, 'UnicodeCharacter0xe1a5', [], 'Display', None],
  [ 0xe1a6, 'UnicodeCharacter0xe1a6', [], 'Display', None],
  [ 0xe1a7, 'UnicodeCharacter0xe1a7', [], 'Display', None],
  [ 0xe1a8, 'UnicodeCharacter0xe1a8', [], 'Display', None],
  [ 0xe1a9, 'UnicodeCharacter0xe1a9', [], 'Display', None],
  [ 0xe1aa, 'UnicodeCharacter0xe1aa', [], 'Display', None],
  [ 0xe1ab, 'UnicodeCharacter0xe1ab', [], 'Display', None],
  [ 0xe1ac, 'UnicodeCharacter0xe1ac', [], 'Display', None],
  [ 0xe1ad, 'UnicodeCharacter0xe1ad', [], 'Display', None],
  [ 0xe1ae, 'UnicodeCharacter0xe1ae', [], 'Display', None],
  [ 0xe1af, 'UnicodeCharacter0xe1af', [], 'Display', None],
  [ 0xe1b0, 'UnicodeCharacter0xe1b0', [], 'Display', None],
  [ 0xe1b1, 'UnicodeCharacter0xe1b1', [], 'Display', None],
  [ 0xe1b2, 'UnicodeCharacter0xe1b2', [], 'Display', None],
  [ 0xe1b3, 'UnicodeCharacter0xe1b3', [], 'Display', None],
  [ 0xe1b4, 'UnicodeCharacter0xe1b4', [], 'Display', None],
  [ 0xe1b5, 'UnicodeCharacter0xe1b5', [], 'Display', None],
  [ 0xe1b6, 'UnicodeCharacter0xe1b6', [], 'Display', None],
  [ 0xe1b7, 'UnicodeCharacter0xe1b7', [], 'Display', None],
  [ 0xe1b8, 'UnicodeCharacter0xe1b8', [], 'Display', None],
  [ 0xe1b9, 'UnicodeCharacter0xe1b9', [], 'Display', None],
  [ 0xe1ba, 'UnicodeCharacter0xe1ba', [], 'Display', None],
  [ 0xe1bb, 'UnicodeCharacter0xe1bb', [], 'Display', None],
  [ 0xe1bc, 'UnicodeCharacter0xe1bc', [], 'Display', None],
  [ 0xe1bd, 'UnicodeCharacter0xe1bd', [], 'Display', None],
  [ 0xe1be, 'UnicodeCharacter0xe1be', [], 'Display', None],
  [ 0xe1bf, 'UnicodeCharacter0xe1bf', [], 'Display', None],
  [ 0xe1c0, 'UnicodeCharacter0xe1c0', [], 'Display', None],
  [ 0xe1c1, 'UnicodeCharacter0xe1c1', [], 'Display', None],
  [ 0xe1c2, 'UnicodeCharacter0xe1c2', [], 'Display', None],
  [ 0xe1c3, 'UnicodeCharacter0xe1c3', [], 'Display', None],
  [ 0xe1c4, 'UnicodeCharacter0xe1c4', [], 'Display', None],
  [ 0xe1c5, 'UnicodeCharacter0xe1c5', [], 'Display', None],
  [ 0xe1c6, 'UnicodeCharacter0xe1c6', [], 'Display', None],
  [ 0xe1c7, 'UnicodeCharacter0xe1c7', [], 'Display', None],
  [ 0xe1c8, 'UnicodeCharacter0xe1c8', [], 'Display', None],
  [ 0xe1c9, 'UnicodeCharacter0xe1c9', [], 'Display', None],
  [ 0xe1ca, 'UnicodeCharacter0xe1ca', [], 'Display', None],
  [ 0xe1cb, 'UnicodeCharacter0xe1cb', [], 'Display', None],
  [ 0xe1cc, 'UnicodeCharacter0xe1cc', [], 'Display', None],
  [ 0xe1cd, 'UnicodeCharacter0xe1cd', [], 'Display', None],
  [ 0xe1ce, 'UnicodeCharacter0xe1ce', [], 'Display', None],
  [ 0xe1cf, 'UnicodeCharacter0xe1cf', [], 'Display', None],
  [ 0xe1d0, 'UnicodeCharacter0xe1d0', [], 'Display', None],
  [ 0xe1d1, 'UnicodeCharacter0xe1d1', [], 'Display', None],
  [ 0xe1d2, 'UnicodeCharacter0xe1d2', [], 'Display', None],
  [ 0xe1d3, 'UnicodeCharacter0xe1d3', [], 'Display', None],
  [ 0xe1d4, 'UnicodeCharacter0xe1d4', [], 'Display', None],
  [ 0xe1d5, 'UnicodeCharacter0xe1d5', [], 'Display', None],
  [ 0xe1d6, 'UnicodeCharacter0xe1d6', [], 'Display', None],
  [ 0xe1d7, 'UnicodeCharacter0xe1d7', [], 'Display', None],
  [ 0xe1d8, 'UnicodeCharacter0xe1d8', [], 'Display', None],
  [ 0xe1d9, 'UnicodeCharacter0xe1d9', [], 'Display', None],
  [ 0xe1da, 'UnicodeCharacter0xe1da', [], 'Display', None],
  [ 0xe1db, 'UnicodeCharacter0xe1db', [], 'Display', None],
  [ 0xe1dc, 'UnicodeCharacter0xe1dc', [], 'Display', None],
  [ 0xe1dd, 'UnicodeCharacter0xe1dd', [], 'Display', None],
  [ 0xe1de, 'UnicodeCharacter0xe1de', [], 'Display', None],
  [ 0xe1df, 'UnicodeCharacter0xe1df', [], 'Display', None],
  [ 0xe1e0, 'UnicodeCharacter0xe1e0', [], 'Display', None],
  [ 0xe1e1, 'UnicodeCharacter0xe1e1', [], 'Display', None],
  [ 0xe1e2, 'UnicodeCharacter0xe1e2', [], 'Display', None],
  [ 0xe1e3, 'UnicodeCharacter0xe1e3', [], 'Display', None],
  [ 0xe1e4, 'UnicodeCharacter0xe1e4', [], 'Display', None],
  [ 0xe1e5, 'UnicodeCharacter0xe1e5', [], 'Display', None],
  [ 0xe1e6, 'UnicodeCharacter0xe1e6', [], 'Display', None],
  [ 0xe1e7, 'UnicodeCharacter0xe1e7', [], 'Display', None],
  [ 0xe1e8, 'UnicodeCharacter0xe1e8', [], 'Display', None],
  [ 0xe1e9, 'UnicodeCharacter0xe1e9', [], 'Display', None],
  [ 0xe1ea, 'UnicodeCharacter0xe1ea', [], 'Display', None],
  [ 0xe1eb, 'UnicodeCharacter0xe1eb', [], 'Display', None],
  [ 0xe1ec, 'UnicodeCharacter0xe1ec', [], 'Display', None],
  [ 0xe1ed, 'UnicodeCharacter0xe1ed', [], 'Display', None],
  [ 0xe1ee, 'UnicodeCharacter0xe1ee', [], 'Display', None],
  [ 0xe1ef, 'UnicodeCharacter0xe1ef', [], 'Display', None],
  [ 0xe1f0, 'UnicodeCharacter0xe1f0', [], 'Display', None],
  [ 0xe1f1, 'UnicodeCharacter0xe1f1', [], 'Display', None],
  [ 0xe1f2, 'UnicodeCharacter0xe1f2', [], 'Display', None],
  [ 0xe1f3, 'UnicodeCharacter0xe1f3', [], 'Display', None],
  [ 0xe1f4, 'UnicodeCharacter0xe1f4', [], 'Display', None],
  [ 0xe1f5, 'UnicodeCharacter0xe1f5', [], 'Display', None],
  [ 0xe1f6, 'UnicodeCharacter0xe1f6', [], 'Display', None],
  [ 0xe1f7, 'UnicodeCharacter0xe1f7', [], 'Display', None],
  [ 0xe1f8, 'UnicodeCharacter0xe1f8', [], 'Display', None],
  [ 0xe1f9, 'UnicodeCharacter0xe1f9', [], 'Display', None],
  [ 0xe1fa, 'UnicodeCharacter0xe1fa', [], 'Display', None],
  [ 0xe1fb, 'UnicodeCharacter0xe1fb', [], 'Display', None],
  [ 0xe1fc, 'UnicodeCharacter0xe1fc', [], 'Display', None],
  [ 0xe1fd, 'UnicodeCharacter0xe1fd', [], 'Display', None],
  [ 0xe1fe, 'UnicodeCharacter0xe1fe', [], 'Display', None],
  [ 0xe1ff, 'UnicodeCharacter0xe1ff', [], 'Display', None],
  [ 0xe200, 'UnicodeCharacter0xe200', [], 'Display', None],
  [ 0xe201, 'UnicodeCharacter0xe201', [], 'Display', None],
  [ 0xe202, 'UnicodeCharacter0xe202', [], 'Display', None],
  [ 0xe203, 'UnicodeCharacter0xe203', [], 'Display', None],
  [ 0xe204, 'UnicodeCharacter0xe204', [], 'Display', None],
  [ 0xe205, 'UnicodeCharacter0xe205', [], 'Display', None],
  [ 0xe206, 'UnicodeCharacter0xe206', [], 'Display', None],
  [ 0xe207, 'UnicodeCharacter0xe207', [], 'Display', None],
  [ 0xe208, 'UnicodeCharacter0xe208', [], 'Display', None],
  [ 0xe209, 'UnicodeCharacter0xe209', [], 'Display', None],
  [ 0xe20a, 'UnicodeCharacter0xe20a', [], 'Display', None],
  [ 0xe20b, 'UnicodeCharacter0xe20b', [], 'Display', None],
  [ 0xe20c, 'UnicodeCharacter0xe20c', [], 'Display', None],
  [ 0xe20d, 'UnicodeCharacter0xe20d', [], 'Display', None],
  [ 0xe20e, 'UnicodeCharacter0xe20e', [], 'Display', None],
  [ 0xe20f, 'UnicodeCharacter0xe20f', [], 'Display', None],
  [ 0xe210, 'UnicodeCharacter0xe210', [], 'Display', None],
  [ 0xe211, 'UnicodeCharacter0xe211', [], 'Display', None],
  [ 0xe212, 'UnicodeCharacter0xe212', [], 'Display', None],
  [ 0xe213, 'UnicodeCharacter0xe213', [], 'Display', None],
  [ 0xe214, 'UnicodeCharacter0xe214', [], 'Display', None],
  [ 0xe215, 'UnicodeCharacter0xe215', [], 'Display', None],
  [ 0xe216, 'UnicodeCharacter0xe216', [], 'Display', None],
  [ 0xe217, 'UnicodeCharacter0xe217', [], 'Display', None],
  [ 0xe218, 'UnicodeCharacter0xe218', [], 'Display', None],
  [ 0xe219, 'UnicodeCharacter0xe219', [], 'Display', None],
  [ 0xe21a, 'UnicodeCharacter0xe21a', [], 'Display', None],
  [ 0xe21b, 'UnicodeCharacter0xe21b', [], 'Display', None],
  [ 0xe21c, 'UnicodeCharacter0xe21c', [], 'Display', None],
  [ 0xe21d, 'UnicodeCharacter0xe21d', [], 'Display', None],
  [ 0xe21e, 'UnicodeCharacter0xe21e', [], 'Display', None],
  [ 0xe21f, 'UnicodeCharacter0xe21f', [], 'Display', None],
  [ 0xe220, 'UnicodeCharacter0xe220', [], 'Display', None],
  [ 0xe221, 'UnicodeCharacter0xe221', [], 'Display', None],
  [ 0xe222, 'UnicodeCharacter0xe222', [], 'Display', None],
  [ 0xe223, 'UnicodeCharacter0xe223', [], 'Display', None],
  [ 0xe224, 'UnicodeCharacter0xe224', [], 'Display', None],
  [ 0xe225, 'UnicodeCharacter0xe225', [], 'Display', None],
  [ 0xe226, 'UnicodeCharacter0xe226', [], 'Display', None],
  [ 0xe227, 'UnicodeCharacter0xe227', [], 'Display', None],
  [ 0xe228, 'UnicodeCharacter0xe228', [], 'Display', None],
  [ 0xe229, 'UnicodeCharacter0xe229', [], 'Display', None],
  [ 0xe22a, 'UnicodeCharacter0xe22a', [], 'Display', None],
  [ 0xe22b, 'UnicodeCharacter0xe22b', [], 'Display', None],
  [ 0xe22c, 'UnicodeCharacter0xe22c', [], 'Display', None],
  [ 0xe22d, 'UnicodeCharacter0xe22d', [], 'Display', None],
  [ 0xe22e, 'UnicodeCharacter0xe22e', [], 'Display', None],
  [ 0xe22f, 'UnicodeCharacter0xe22f', [], 'Display', None],
  [ 0xe230, 'UnicodeCharacter0xe230', [], 'Display', None],
  [ 0xe231, 'UnicodeCharacter0xe231', [], 'Display', None],
  [ 0xe232, 'UnicodeCharacter0xe232', [], 'Display', None],
  [ 0xe233, 'UnicodeCharacter0xe233', [], 'Display', None],
  [ 0xe234, 'UnicodeCharacter0xe234', [], 'Display', None],
  [ 0xe235, 'UnicodeCharacter0xe235', [], 'Display', None],
  [ 0xe236, 'UnicodeCharacter0xe236', [], 'Display', None],
  [ 0xe237, 'UnicodeCharacter0xe237', [], 'Display', None],
  [ 0xe238, 'UnicodeCharacter0xe238', [], 'Display', None],
  [ 0xe239, 'UnicodeCharacter0xe239', [], 'Display', None],
  [ 0xe23a, 'UnicodeCharacter0xe23a', [], 'Display', None],
  [ 0xe23b, 'UnicodeCharacter0xe23b', [], 'Display', None],
  [ 0xe23c, 'UnicodeCharacter0xe23c', [], 'Display', None],
  [ 0xe23d, 'UnicodeCharacter0xe23d', [], 'Display', None],
  [ 0xe23e, 'UnicodeCharacter0xe23e', [], 'Display', None],
  [ 0xe23f, 'UnicodeCharacter0xe23f', [], 'Display', None],
  [ 0xe240, 'UnicodeCharacter0xe240', [], 'Display', None],
  [ 0xe241, 'UnicodeCharacter0xe241', [], 'Display', None],
  [ 0xe242, 'UnicodeCharacter0xe242', [], 'Display', None],
  [ 0xe243, 'UnicodeCharacter0xe243', [], 'Display', None],
  [ 0xe244, 'UnicodeCharacter0xe244', [], 'Display', None],
  [ 0xe245, 'UnicodeCharacter0xe245', [], 'Display', None],
  [ 0xe246, 'UnicodeCharacter0xe246', [], 'Display', None],
  [ 0xe247, 'UnicodeCharacter0xe247', [], 'Display', None],
  [ 0xe248, 'UnicodeCharacter0xe248', [], 'Display', None],
  [ 0xe249, 'UnicodeCharacter0xe249', [], 'Display', None],
  [ 0xe24a, 'UnicodeCharacter0xe24a', [], 'Display', None],
  [ 0xe24b, 'UnicodeCharacter0xe24b', [], 'Display', None],
  [ 0xe24c, 'UnicodeCharacter0xe24c', [], 'Display', None],
  [ 0xe24d, 'UnicodeCharacter0xe24d', [], 'Display', None],
  [ 0xe24e, 'UnicodeCharacter0xe24e', [], 'Display', None],
  [ 0xe24f, 'UnicodeCharacter0xe24f', [], 'Display', None],
  [ 0xe250, 'UnicodeCharacter0xe250', [], 'Display', None],
  [ 0xe251, 'UnicodeCharacter0xe251', [], 'Display', None],
  [ 0xe252, 'UnicodeCharacter0xe252', [], 'Display', None],
  [ 0xe253, 'UnicodeCharacter0xe253', [], 'Display', None],
  [ 0xe254, 'UnicodeCharacter0xe254', [], 'Display', None],
  [ 0xe255, 'UnicodeCharacter0xe255', [], 'Display', None],
  [ 0xe256, 'UnicodeCharacter0xe256', [], 'Display', None],
  [ 0xe257, 'UnicodeCharacter0xe257', [], 'Display', None],
  [ 0xe258, 'UnicodeCharacter0xe258', [], 'Display', None],
  [ 0xe259, 'UnicodeCharacter0xe259', [], 'Display', None],
  [ 0xe25a, 'UnicodeCharacter0xe25a', [], 'Display', None],
  [ 0xe25b, 'UnicodeCharacter0xe25b', [], 'Display', None],
  [ 0xe25c, 'UnicodeCharacter0xe25c', [], 'Display', None],
  [ 0xe25d, 'UnicodeCharacter0xe25d', [], 'Display', None],
  [ 0xe25e, 'UnicodeCharacter0xe25e', [], 'Display', None],
  [ 0xe25f, 'UnicodeCharacter0xe25f', [], 'Display', None],
  [ 0xe260, 'UnicodeCharacter0xe260', [], 'Display', None],
  [ 0xe261, 'UnicodeCharacter0xe261', [], 'Display', None],
  [ 0xe262, 'UnicodeCharacter0xe262', [], 'Display', None],
  [ 0xe263, 'UnicodeCharacter0xe263', [], 'Display', None],
  [ 0xe264, 'UnicodeCharacter0xe264', [], 'Display', None],
  [ 0xe265, 'UnicodeCharacter0xe265', [], 'Display', None],
  [ 0xe266, 'UnicodeCharacter0xe266', [], 'Display', None],
  [ 0xe267, 'UnicodeCharacter0xe267', [], 'Display', None],
  [ 0xe268, 'UnicodeCharacter0xe268', [], 'Display', None],
  [ 0xe269, 'UnicodeCharacter0xe269', [], 'Display', None],
  [ 0xe26a, 'UnicodeCharacter0xe26a', [], 'Display', None],
  [ 0xe26b, 'UnicodeCharacter0xe26b', [], 'Display', None],
  [ 0xe26c, 'UnicodeCharacter0xe26c', [], 'Display', None],
  [ 0xe26d, 'UnicodeCharacter0xe26d', [], 'Display', None],
  [ 0xe26e, 'UnicodeCharacter0xe26e', [], 'Display', None],
  [ 0xe26f, 'UnicodeCharacter0xe26f', [], 'Display', None],
  [ 0xe270, 'UnicodeCharacter0xe270', [], 'Display', None],
  [ 0xe271, 'UnicodeCharacter0xe271', [], 'Display', None],
  [ 0xe272, 'UnicodeCharacter0xe272', [], 'Display', None],
  [ 0xe273, 'UnicodeCharacter0xe273', [], 'Display', None],
  [ 0xe274, 'UnicodeCharacter0xe274', [], 'Display', None],
  [ 0xe275, 'UnicodeCharacter0xe275', [], 'Display', None],
  [ 0xe276, 'UnicodeCharacter0xe276', [], 'Display', None],
  [ 0xe277, 'UnicodeCharacter0xe277', [], 'Display', None],
  [ 0xe278, 'UnicodeCharacter0xe278', [], 'Display', None],
  [ 0xe279, 'UnicodeCharacter0xe279', [], 'Display', None],
  [ 0xe27a, 'UnicodeCharacter0xe27a', [], 'Display', None],
  [ 0xe27b, 'UnicodeCharacter0xe27b', [], 'Display', None],
  [ 0xe27c, 'UnicodeCharacter0xe27c', [], 'Display', None],
  [ 0xe27d, 'UnicodeCharacter0xe27d', [], 'Display', None],
  [ 0xe27e, 'UnicodeCharacter0xe27e', [], 'Display', None],
  [ 0xe27f, 'UnicodeCharacter0xe27f', [], 'Display', None],
  [ 0xe280, 'UnicodeCharacter0xe280', [], 'Display', None],
  [ 0xe281, 'UnicodeCharacter0xe281', [], 'Display', None],
  [ 0xe282, 'UnicodeCharacter0xe282', [], 'Display', None],
  [ 0xe283, 'UnicodeCharacter0xe283', [], 'Display', None],
  [ 0xe284, 'UnicodeCharacter0xe284', [], 'Display', None],
  [ 0xe285, 'UnicodeCharacter0xe285', [], 'Display', None],
  [ 0xe286, 'UnicodeCharacter0xe286', [], 'Display', None],
  [ 0xe287, 'UnicodeCharacter0xe287', [], 'Display', None],
  [ 0xe288, 'UnicodeCharacter0xe288', [], 'Display', None],
  [ 0xe289, 'UnicodeCharacter0xe289', [], 'Display', None],
  [ 0xe28a, 'UnicodeCharacter0xe28a', [], 'Display', None],
  [ 0xe28b, 'UnicodeCharacter0xe28b', [], 'Display', None],
  [ 0xe28c, 'UnicodeCharacter0xe28c', [], 'Display', None],
  [ 0xe28d, 'UnicodeCharacter0xe28d', [], 'Display', None],
  [ 0xe28e, 'UnicodeCharacter0xe28e', [], 'Display', None],
  [ 0xe28f, 'UnicodeCharacter0xe28f', [], 'Display', None],
  [ 0xe290, 'UnicodeCharacter0xe290', [], 'Display', None],
  [ 0xe291, 'UnicodeCharacter0xe291', [], 'Display', None],
  [ 0xe292, 'UnicodeCharacter0xe292', [], 'Display', None],
  [ 0xe293, 'UnicodeCharacter0xe293', [], 'Display', None],
  [ 0xe294, 'UnicodeCharacter0xe294', [], 'Display', None],
  [ 0xe295, 'UnicodeCharacter0xe295', [], 'Display', None],
  [ 0xe296, 'UnicodeCharacter0xe296', [], 'Display', None],
  [ 0xe297, 'UnicodeCharacter0xe297', [], 'Display', None],
  [ 0xe298, 'UnicodeCharacter0xe298', [], 'Display', None],
  [ 0xe299, 'UnicodeCharacter0xe299', [], 'Display', None],
  [ 0xe29a, 'UnicodeCharacter0xe29a', [], 'Display', None],
  [ 0xe29b, 'UnicodeCharacter0xe29b', [], 'Display', None],
  [ 0xe29c, 'UnicodeCharacter0xe29c', [], 'Display', None],
  [ 0xe29d, 'UnicodeCharacter0xe29d', [], 'Display', None],
  [ 0xe29e, 'UnicodeCharacter0xe29e', [], 'Display', None],
  [ 0xe29f, 'UnicodeCharacter0xe29f', [], 'Display', None],
  [ 0xe2a0, 'UnicodeCharacter0xe2a0', [], 'Display', None],
  [ 0xe2a1, 'UnicodeCharacter0xe2a1', [], 'Display', None],
  [ 0xe2a2, 'UnicodeCharacter0xe2a2', [], 'Display', None],
  [ 0xe2a3, 'UnicodeCharacter0xe2a3', [], 'Display', None],
  [ 0xe2a4, 'UnicodeCharacter0xe2a4', [], 'Display', None],
  [ 0xe2a5, 'UnicodeCharacter0xe2a5', [], 'Display', None],
  [ 0xe2a6, 'UnicodeCharacter0xe2a6', [], 'Display', None],
  [ 0xe2a7, 'UnicodeCharacter0xe2a7', [], 'Display', None],
  [ 0xe2a8, 'UnicodeCharacter0xe2a8', [], 'Display', None],
  [ 0xe2a9, 'UnicodeCharacter0xe2a9', [], 'Display', None],
  [ 0xe2aa, 'UnicodeCharacter0xe2aa', [], 'Display', None],
  [ 0xe2ab, 'UnicodeCharacter0xe2ab', [], 'Display', None],
  [ 0xe2ac, 'UnicodeCharacter0xe2ac', [], 'Display', None],
  [ 0xe2ad, 'UnicodeCharacter0xe2ad', [], 'Display', None],
  [ 0xe2ae, 'UnicodeCharacter0xe2ae', [], 'Display', None],
  [ 0xe2af, 'UnicodeCharacter0xe2af', [], 'Display', None],
  [ 0xe2b0, 'UnicodeCharacter0xe2b0', [], 'Display', None],
  [ 0xe2b1, 'UnicodeCharacter0xe2b1', [], 'Display', None],
  [ 0xe2b2, 'UnicodeCharacter0xe2b2', [], 'Display', None],
  [ 0xe2b3, 'UnicodeCharacter0xe2b3', [], 'Display', None],
  [ 0xe2b4, 'UnicodeCharacter0xe2b4', [], 'Display', None],
  [ 0xe2b5, 'UnicodeCharacter0xe2b5', [], 'Display', None],
  [ 0xe2b6, 'UnicodeCharacter0xe2b6', [], 'Display', None],
  [ 0xe2b7, 'UnicodeCharacter0xe2b7', [], 'Display', None],
  [ 0xe2b8, 'UnicodeCharacter0xe2b8', [], 'Display', None],
  [ 0xe2b9, 'UnicodeCharacter0xe2b9', [], 'Display', None],
  [ 0xe2ba, 'UnicodeCharacter0xe2ba', [], 'Display', None],
  [ 0xe2bb, 'UnicodeCharacter0xe2bb', [], 'Display', None],
  [ 0xe2bc, 'UnicodeCharacter0xe2bc', [], 'Display', None],
  [ 0xe2bd, 'UnicodeCharacter0xe2bd', [], 'Display', None],
  [ 0xe2be, 'UnicodeCharacter0xe2be', [], 'Display', None],
  [ 0xe2bf, 'UnicodeCharacter0xe2bf', [], 'Display', None],
  [ 0xe2c0, 'UnicodeCharacter0xe2c0', [], 'Display', None],
  [ 0xe2c1, 'UnicodeCharacter0xe2c1', [], 'Display', None],
  [ 0xe2c2, 'UnicodeCharacter0xe2c2', [], 'Display', None],
  [ 0xe2c3, 'UnicodeCharacter0xe2c3', [], 'Display', None],
  [ 0xe2c4, 'UnicodeCharacter0xe2c4', [], 'Display', None],
  [ 0xe2c5, 'UnicodeCharacter0xe2c5', [], 'Display', None],
  [ 0xe2c6, 'UnicodeCharacter0xe2c6', [], 'Display', None],
  [ 0xe2c7, 'UnicodeCharacter0xe2c7', [], 'Display', None],
  [ 0xe2c8, 'UnicodeCharacter0xe2c8', [], 'Display', None],
  [ 0xe2c9, 'UnicodeCharacter0xe2c9', [], 'Display', None],
  [ 0xe2ca, 'UnicodeCharacter0xe2ca', [], 'Display', None],
  [ 0xe2cb, 'UnicodeCharacter0xe2cb', [], 'Display', None],
  [ 0xe2cc, 'UnicodeCharacter0xe2cc', [], 'Display', None],
  [ 0xe2cd, 'UnicodeCharacter0xe2cd', [], 'Display', None],
  [ 0xe2ce, 'UnicodeCharacter0xe2ce', [], 'Display', None],
  [ 0xe2cf, 'UnicodeCharacter0xe2cf', [], 'Display', None],
  [ 0xe2d0, 'UnicodeCharacter0xe2d0', [], 'Display', None],
  [ 0xe2d1, 'UnicodeCharacter0xe2d1', [], 'Display', None],
  [ 0xe2d2, 'UnicodeCharacter0xe2d2', [], 'Display', None],
  [ 0xe2d3, 'UnicodeCharacter0xe2d3', [], 'Display', None],
  [ 0xe2d4, 'UnicodeCharacter0xe2d4', [], 'Display', None],
  [ 0xe2d5, 'UnicodeCharacter0xe2d5', [], 'Display', None],
  [ 0xe2d6, 'UnicodeCharacter0xe2d6', [], 'Display', None],
  [ 0xe2d7, 'UnicodeCharacter0xe2d7', [], 'Display', None],
  [ 0xe2d8, 'UnicodeCharacter0xe2d8', [], 'Display', None],
  [ 0xe2d9, 'UnicodeCharacter0xe2d9', [], 'Display', None],
  [ 0xe2da, 'UnicodeCharacter0xe2da', [], 'Display', None],
  [ 0xe2db, 'UnicodeCharacter0xe2db', [], 'Display', None],
  [ 0xe2dc, 'UnicodeCharacter0xe2dc', [], 'Display', None],
  [ 0xe2dd, 'UnicodeCharacter0xe2dd', [], 'Display', None],
  [ 0xe2de, 'UnicodeCharacter0xe2de', [], 'Display', None],
  [ 0xe2df, 'UnicodeCharacter0xe2df', [], 'Display', None],
  [ 0xe2e0, 'UnicodeCharacter0xe2e0', [], 'Display', None],
  [ 0xe2e1, 'UnicodeCharacter0xe2e1', [], 'Display', None],
  [ 0xe2e2, 'UnicodeCharacter0xe2e2', [], 'Display', None],
  [ 0xe2e3, 'UnicodeCharacter0xe2e3', [], 'Display', None],
  [ 0xe2e4, 'UnicodeCharacter0xe2e4', [], 'Display', None],
  [ 0xe2e5, 'UnicodeCharacter0xe2e5', [], 'Display', None],
  [ 0xe2e6, 'UnicodeCharacter0xe2e6', [], 'Display', None],
  [ 0xe2e7, 'UnicodeCharacter0xe2e7', [], 'Display', None],
  [ 0xe2e8, 'UnicodeCharacter0xe2e8', [], 'Display', None],
  [ 0xe2e9, 'UnicodeCharacter0xe2e9', [], 'Display', None],
  [ 0xe2ea, 'UnicodeCharacter0xe2ea', [], 'Display', None],
  [ 0xe2eb, 'UnicodeCharacter0xe2eb', [], 'Display', None],
  [ 0xe2ec, 'UnicodeCharacter0xe2ec', [], 'Display', None],
  [ 0xe2ed, 'UnicodeCharacter0xe2ed', [], 'Display', None],
  [ 0xe2ee, 'UnicodeCharacter0xe2ee', [], 'Display', None],
  [ 0xe2ef, 'UnicodeCharacter0xe2ef', [], 'Display', None],
  [ 0xe2f0, 'UnicodeCharacter0xe2f0', [], 'Display', None],
  [ 0xe2f1, 'UnicodeCharacter0xe2f1', [], 'Display', None],
  [ 0xe2f2, 'UnicodeCharacter0xe2f2', [], 'Display', None],
  [ 0xe2f3, 'UnicodeCharacter0xe2f3', [], 'Display', None],
  [ 0xe2f4, 'UnicodeCharacter0xe2f4', [], 'Display', None],
  [ 0xe2f5, 'UnicodeCharacter0xe2f5', [], 'Display', None],
  [ 0xe2f6, 'UnicodeCharacter0xe2f6', [], 'Display', None],
  [ 0xe2f7, 'UnicodeCharacter0xe2f7', [], 'Display', None],
  [ 0xe2f8, 'UnicodeCharacter0xe2f8', [], 'Display', None],
  [ 0xe2f9, 'UnicodeCharacter0xe2f9', [], 'Display', None],
  [ 0xe2fa, 'UnicodeCharacter0xe2fa', [], 'Display', None],
  [ 0xe2fb, 'UnicodeCharacter0xe2fb', [], 'Display', None],
  [ 0xe2fc, 'UnicodeCharacter0xe2fc', [], 'Display', None],
  [ 0xe2fd, 'UnicodeCharacter0xe2fd', [], 'Display', None],
  [ 0xe2fe, 'UnicodeCharacter0xe2fe', [], 'Display', None],
  [ 0xe2ff, 'UnicodeCharacter0xe2ff', [], 'Display', None],
  [ 0xe300, 'UnicodeCharacter0xe300', [], 'Display', None],
  [ 0xe301, 'UnicodeCharacter0xe301', [], 'Display', None],
  [ 0xe302, 'UnicodeCharacter0xe302', [], 'Display', None],
  [ 0xe303, 'UnicodeCharacter0xe303', [], 'Display', None],
  [ 0xe304, 'UnicodeCharacter0xe304', [], 'Display', None],
  [ 0xe305, 'UnicodeCharacter0xe305', [], 'Display', None],
  [ 0xe306, 'UnicodeCharacter0xe306', [], 'Display', None],
  [ 0xe307, 'UnicodeCharacter0xe307', [], 'Display', None],
  [ 0xe308, 'UnicodeCharacter0xe308', [], 'Display', None],
  [ 0xe309, 'UnicodeCharacter0xe309', [], 'Display', None],
  [ 0xe30a, 'UnicodeCharacter0xe30a', [], 'Display', None],
  [ 0xe30b, 'UnicodeCharacter0xe30b', [], 'Display', None],
  [ 0xe30c, 'UnicodeCharacter0xe30c', [], 'Display', None],
  [ 0xe30d, 'UnicodeCharacter0xe30d', [], 'Display', None],
  [ 0xe30e, 'UnicodeCharacter0xe30e', [], 'Display', None],
  [ 0xe30f, 'UnicodeCharacter0xe30f', [], 'Display', None],
  [ 0xe310, 'UnicodeCharacter0xe310', [], 'Display', None],
  [ 0xe311, 'UnicodeCharacter0xe311', [], 'Display', None],
  [ 0xe312, 'UnicodeCharacter0xe312', [], 'Display', None],
  [ 0xe313, 'UnicodeCharacter0xe313', [], 'Display', None],
  [ 0xe314, 'UnicodeCharacter0xe314', [], 'Display', None],
  [ 0xe315, 'UnicodeCharacter0xe315', [], 'Display', None],
  [ 0xe316, 'UnicodeCharacter0xe316', [], 'Display', None],
  [ 0xe317, 'UnicodeCharacter0xe317', [], 'Display', None],
  [ 0xe318, 'UnicodeCharacter0xe318', [], 'Display', None],
  [ 0xe319, 'UnicodeCharacter0xe319', [], 'Display', None],
  [ 0xe31a, 'UnicodeCharacter0xe31a', [], 'Display', None],
  [ 0xe31b, 'UnicodeCharacter0xe31b', [], 'Display', None],
  [ 0xe31c, 'UnicodeCharacter0xe31c', [], 'Display', None],
  [ 0xe31d, 'UnicodeCharacter0xe31d', [], 'Display', None],
  [ 0xe31e, 'UnicodeCharacter0xe31e', [], 'Display', None],
  [ 0xe31f, 'UnicodeCharacter0xe31f', [], 'Display', None],
  [ 0xe320, 'UnicodeCharacter0xe320', [], 'Display', None],
  [ 0xe321, 'UnicodeCharacter0xe321', [], 'Display', None],
  [ 0xe322, 'UnicodeCharacter0xe322', [], 'Display', None],
  [ 0xe323, 'UnicodeCharacter0xe323', [], 'Display', None],
  [ 0xe324, 'UnicodeCharacter0xe324', [], 'Display', None],
  [ 0xe325, 'UnicodeCharacter0xe325', [], 'Display', None],
  [ 0xe326, 'UnicodeCharacter0xe326', [], 'Display', None],
  [ 0xe327, 'UnicodeCharacter0xe327', [], 'Display', None],
  [ 0xe328, 'UnicodeCharacter0xe328', [], 'Display', None],
  [ 0xe329, 'UnicodeCharacter0xe329', [], 'Display', None],
  [ 0xe32a, 'UnicodeCharacter0xe32a', [], 'Display', None],
  [ 0xe32b, 'UnicodeCharacter0xe32b', [], 'Display', None],
  [ 0xe32c, 'UnicodeCharacter0xe32c', [], 'Display', None],
  [ 0xe32d, 'UnicodeCharacter0xe32d', [], 'Display', None],
  [ 0xe32e, 'UnicodeCharacter0xe32e', [], 'Display', None],
  [ 0xe32f, 'UnicodeCharacter0xe32f', [], 'Display', None],
  [ 0xe330, 'UnicodeCharacter0xe330', [], 'Display', None],
  [ 0xe331, 'UnicodeCharacter0xe331', [], 'Display', None],
  [ 0xe332, 'UnicodeCharacter0xe332', [], 'Display', None],
  [ 0xe333, 'UnicodeCharacter0xe333', [], 'Display', None],
  [ 0xe334, 'UnicodeCharacter0xe334', [], 'Display', None],
  [ 0xe335, 'UnicodeCharacter0xe335', [], 'Display', None],
  [ 0xe336, 'UnicodeCharacter0xe336', [], 'Display', None],
  [ 0xe337, 'UnicodeCharacter0xe337', [], 'Display', None],
  [ 0xe338, 'UnicodeCharacter0xe338', [], 'Display', None],
  [ 0xe339, 'UnicodeCharacter0xe339', [], 'Display', None],
  [ 0xe33a, 'UnicodeCharacter0xe33a', [], 'Display', None],
  [ 0xe33b, 'UnicodeCharacter0xe33b', [], 'Display', None],
  [ 0xe33c, 'UnicodeCharacter0xe33c', [], 'Display', None],
  [ 0xe33d, 'UnicodeCharacter0xe33d', [], 'Display', None],
  [ 0xe33e, 'UnicodeCharacter0xe33e', [], 'Display', None],
  [ 0xe33f, 'UnicodeCharacter0xe33f', [], 'Display', None],
  [ 0xe340, 'UnicodeCharacter0xe340', [], 'Display', None],
  [ 0xe341, 'UnicodeCharacter0xe341', [], 'Display', None],
  [ 0xe342, 'UnicodeCharacter0xe342', [], 'Display', None],
  [ 0xe343, 'UnicodeCharacter0xe343', [], 'Display', None],
  [ 0xe344, 'UnicodeCharacter0xe344', [], 'Display', None],
  [ 0xe345, 'UnicodeCharacter0xe345', [], 'Display', None],
  [ 0xe346, 'UnicodeCharacter0xe346', [], 'Display', None],
  [ 0xe347, 'UnicodeCharacter0xe347', [], 'Display', None],
  [ 0xe348, 'UnicodeCharacter0xe348', [], 'Display', None],
  [ 0xe349, 'UnicodeCharacter0xe349', [], 'Display', None],
  [ 0xe34a, 'UnicodeCharacter0xe34a', [], 'Display', None],
  [ 0xe34b, 'UnicodeCharacter0xe34b', [], 'Display', None],
  [ 0xe34c, 'UnicodeCharacter0xe34c', [], 'Display', None],
  [ 0xe34d, 'UnicodeCharacter0xe34d', [], 'Display', None],
  [ 0xe34e, 'UnicodeCharacter0xe34e', [], 'Display', None],
  [ 0xe34f, 'UnicodeCharacter0xe34f', [], 'Display', None],
  [ 0xe350, 'UnicodeCharacter0xe350', [], 'Display', None],
  [ 0xe351, 'UnicodeCharacter0xe351', [], 'Display', None],
  [ 0xe352, 'UnicodeCharacter0xe352', [], 'Display', None],
  [ 0xe353, 'UnicodeCharacter0xe353', [], 'Display', None],
  [ 0xe354, 'UnicodeCharacter0xe354', [], 'Display', None],
  [ 0xe355, 'UnicodeCharacter0xe355', [], 'Display', None],
  [ 0xe356, 'UnicodeCharacter0xe356', [], 'Display', None],
  [ 0xe357, 'UnicodeCharacter0xe357', [], 'Display', None],
  [ 0xe358, 'UnicodeCharacter0xe358', [], 'Display', None],
  [ 0xe359, 'UnicodeCharacter0xe359', [], 'Display', None],
  [ 0xe35a, 'UnicodeCharacter0xe35a', [], 'Display', None],
  [ 0xe35b, 'UnicodeCharacter0xe35b', [], 'Display', None],
  [ 0xe35c, 'UnicodeCharacter0xe35c', [], 'Display', None],
  [ 0xe35d, 'UnicodeCharacter0xe35d', [], 'Display', None],
  [ 0xe35e, 'UnicodeCharacter0xe35e', [], 'Display', None],
  [ 0xe35f, 'UnicodeCharacter0xe35f', [], 'Display', None],
  [ 0xe360, 'UnicodeCharacter0xe360', [], 'Display', None],
  [ 0xe361, 'UnicodeCharacter0xe361', [], 'Display', None],
  [ 0xe362, 'UnicodeCharacter0xe362', [], 'Display', None],
  [ 0xe363, 'UnicodeCharacter0xe363', [], 'Display', None],
  [ 0xe364, 'UnicodeCharacter0xe364', [], 'Display', None],
  [ 0xe365, 'UnicodeCharacter0xe365', [], 'Display', None],
  [ 0xe366, 'UnicodeCharacter0xe366', [], 'Display', None],
  [ 0xe367, 'UnicodeCharacter0xe367', [], 'Display', None],
  [ 0xe368, 'UnicodeCharacter0xe368', [], 'Display', None],
  [ 0xe369, 'UnicodeCharacter0xe369', [], 'Display', None],
  [ 0xe36a, 'UnicodeCharacter0xe36a', [], 'Display', None],
  [ 0xe36b, 'UnicodeCharacter0xe36b', [], 'Display', None],
  [ 0xe36c, 'UnicodeCharacter0xe36c', [], 'Display', None],
  [ 0xe36d, 'UnicodeCharacter0xe36d', [], 'Display', None],
  [ 0xe36e, 'UnicodeCharacter0xe36e', [], 'Display', None],
  [ 0xe36f, 'UnicodeCharacter0xe36f', [], 'Display', None],
  [ 0xe370, 'UnicodeCharacter0xe370', [], 'Display', None],
  [ 0xe371, 'UnicodeCharacter0xe371', [], 'Display', None],
  [ 0xe372, 'UnicodeCharacter0xe372', [], 'Display', None],
  [ 0xe373, 'UnicodeCharacter0xe373', [], 'Display', None],
  [ 0xe374, 'UnicodeCharacter0xe374', [], 'Display', None],
  [ 0xe375, 'UnicodeCharacter0xe375', [], 'Display', None],
  [ 0xe376, 'UnicodeCharacter0xe376', [], 'Display', None],
  [ 0xe377, 'UnicodeCharacter0xe377', [], 'Display', None],
  [ 0xe378, 'UnicodeCharacter0xe378', [], 'Display', None],
  [ 0xe379, 'UnicodeCharacter0xe379', [], 'Display', None],
  [ 0xe37a, 'UnicodeCharacter0xe37a', [], 'Display', None],
  [ 0xe37b, 'UnicodeCharacter0xe37b', [], 'Display', None],
  [ 0xe37c, 'UnicodeCharacter0xe37c', [], 'Display', None],
  [ 0xe37d, 'UnicodeCharacter0xe37d', [], 'Display', None],
  [ 0xe37e, 'UnicodeCharacter0xe37e', [], 'Display', None],
  [ 0xe37f, 'UnicodeCharacter0xe37f', [], 'Display', None],
  [ 0xe380, 'UnicodeCharacter0xe380', [], 'Display', None],
  [ 0xe381, 'UnicodeCharacter0xe381', [], 'Display', None],
  [ 0xe382, 'UnicodeCharacter0xe382', [], 'Display', None],
  [ 0xe383, 'UnicodeCharacter0xe383', [], 'Display', None],
  [ 0xe384, 'UnicodeCharacter0xe384', [], 'Display', None],
  [ 0xe385, 'UnicodeCharacter0xe385', [], 'Display', None],
  [ 0xe386, 'UnicodeCharacter0xe386', [], 'Display', None],
  [ 0xe387, 'UnicodeCharacter0xe387', [], 'Display', None],
  [ 0xe388, 'UnicodeCharacter0xe388', [], 'Display', None],
  [ 0xe389, 'UnicodeCharacter0xe389', [], 'Display', None],
  [ 0xe38a, 'UnicodeCharacter0xe38a', [], 'Display', None],
  [ 0xe38b, 'UnicodeCharacter0xe38b', [], 'Display', None],
  [ 0xe38c, 'UnicodeCharacter0xe38c', [], 'Display', None],
  [ 0xe38d, 'UnicodeCharacter0xe38d', [], 'Display', None],
  [ 0xe38e, 'UnicodeCharacter0xe38e', [], 'Display', None],
  [ 0xe38f, 'UnicodeCharacter0xe38f', [], 'Display', None],
  [ 0xe390, 'UnicodeCharacter0xe390', [], 'Display', None],
  [ 0xe391, 'UnicodeCharacter0xe391', [], 'Display', None],
  [ 0xe392, 'UnicodeCharacter0xe392', [], 'Display', None],
  [ 0xe393, 'UnicodeCharacter0xe393', [], 'Display', None],
  [ 0xe394, 'UnicodeCharacter0xe394', [], 'Display', None],
  [ 0xe395, 'UnicodeCharacter0xe395', [], 'Display', None],
  [ 0xe396, 'UnicodeCharacter0xe396', [], 'Display', None],
  [ 0xe397, 'UnicodeCharacter0xe397', [], 'Display', None],
  [ 0xe398, 'UnicodeCharacter0xe398', [], 'Display', None],
  [ 0xe399, 'UnicodeCharacter0xe399', [], 'Display', None],
  [ 0xe39a, 'UnicodeCharacter0xe39a', [], 'Display', None],
  [ 0xe39b, 'UnicodeCharacter0xe39b', [], 'Display', None],
  [ 0xe39c, 'UnicodeCharacter0xe39c', [], 'Display', None],
  [ 0xe39d, 'UnicodeCharacter0xe39d', [], 'Display', None],
  [ 0xe39e, 'UnicodeCharacter0xe39e', [], 'Display', None],
  [ 0xe39f, 'UnicodeCharacter0xe39f', [], 'Display', None],
  [ 0xe3a0, 'UnicodeCharacter0xe3a0', [], 'Display', None],
  [ 0xe3a1, 'UnicodeCharacter0xe3a1', [], 'Display', None],
  [ 0xe3a2, 'UnicodeCharacter0xe3a2', [], 'Display', None],
  [ 0xe3a3, 'UnicodeCharacter0xe3a3', [], 'Display', None],
  [ 0xe3a4, 'UnicodeCharacter0xe3a4', [], 'Display', None],
  [ 0xe3a5, 'UnicodeCharacter0xe3a5', [], 'Display', None],
  [ 0xe3a6, 'UnicodeCharacter0xe3a6', [], 'Display', None],
  [ 0xe3a7, 'UnicodeCharacter0xe3a7', [], 'Display', None],
  [ 0xe3a8, 'UnicodeCharacter0xe3a8', [], 'Display', None],
  [ 0xe3a9, 'UnicodeCharacter0xe3a9', [], 'Display', None],
  [ 0xe3aa, 'UnicodeCharacter0xe3aa', [], 'Display', None],
  [ 0xe3ab, 'UnicodeCharacter0xe3ab', [], 'Display', None],
  [ 0xe3ac, 'UnicodeCharacter0xe3ac', [], 'Display', None],
  [ 0xe3ad, 'UnicodeCharacter0xe3ad', [], 'Display', None],
  [ 0xe3ae, 'UnicodeCharacter0xe3ae', [], 'Display', None],
  [ 0xe3af, 'UnicodeCharacter0xe3af', [], 'Display', None],
  [ 0xe3b0, 'UnicodeCharacter0xe3b0', [], 'Display', None],
  [ 0xe3b1, 'UnicodeCharacter0xe3b1', [], 'Display', None],
  [ 0xe3b2, 'UnicodeCharacter0xe3b2', [], 'Display', None],
  [ 0xe3b3, 'UnicodeCharacter0xe3b3', [], 'Display', None],
  [ 0xe3b4, 'UnicodeCharacter0xe3b4', [], 'Display', None],
  [ 0xe3b5, 'UnicodeCharacter0xe3b5', [], 'Display', None],
  [ 0xe3b6, 'UnicodeCharacter0xe3b6', [], 'Display', None],
  [ 0xe3b7, 'UnicodeCharacter0xe3b7', [], 'Display', None],
  [ 0xe3b8, 'UnicodeCharacter0xe3b8', [], 'Display', None],
  [ 0xe3b9, 'UnicodeCharacter0xe3b9', [], 'Display', None],
  [ 0xe3ba, 'UnicodeCharacter0xe3ba', [], 'Display', None],
  [ 0xe3bb, 'UnicodeCharacter0xe3bb', [], 'Display', None],
  [ 0xe3bc, 'UnicodeCharacter0xe3bc', [], 'Display', None],
  [ 0xe3bd, 'UnicodeCharacter0xe3bd', [], 'Display', None],
  [ 0xe3be, 'UnicodeCharacter0xe3be', [], 'Display', None],
  [ 0xe3bf, 'UnicodeCharacter0xe3bf', [], 'Display', None],
  [ 0xe3c0, 'UnicodeCharacter0xe3c0', [], 'Display', None],
  [ 0xe3c1, 'UnicodeCharacter0xe3c1', [], 'Display', None],
  [ 0xe3c2, 'UnicodeCharacter0xe3c2', [], 'Display', None],
  [ 0xe3c3, 'UnicodeCharacter0xe3c3', [], 'Display', None],
  [ 0xe3c4, 'UnicodeCharacter0xe3c4', [], 'Display', None],
  [ 0xe3c5, 'UnicodeCharacter0xe3c5', [], 'Display', None],
  [ 0xe3c6, 'UnicodeCharacter0xe3c6', [], 'Display', None],
  [ 0xe3c7, 'UnicodeCharacter0xe3c7', [], 'Display', None],
  [ 0xe3c8, 'UnicodeCharacter0xe3c8', [], 'Display', None],
  [ 0xe3c9, 'UnicodeCharacter0xe3c9', [], 'Display', None],
  [ 0xe3ca, 'UnicodeCharacter0xe3ca', [], 'Display', None],
  [ 0xe3cb, 'UnicodeCharacter0xe3cb', [], 'Display', None],
  [ 0xe3cc, 'UnicodeCharacter0xe3cc', [], 'Display', None],
  [ 0xe3cd, 'UnicodeCharacter0xe3cd', [], 'Display', None],
  [ 0xe3ce, 'UnicodeCharacter0xe3ce', [], 'Display', None],
  [ 0xe3cf, 'UnicodeCharacter0xe3cf', [], 'Display', None],
  [ 0xe3d0, 'UnicodeCharacter0xe3d0', [], 'Display', None],
  [ 0xe3d1, 'UnicodeCharacter0xe3d1', [], 'Display', None],
  [ 0xe3d2, 'UnicodeCharacter0xe3d2', [], 'Display', None],
  [ 0xe3d3, 'UnicodeCharacter0xe3d3', [], 'Display', None],
  [ 0xe3d4, 'UnicodeCharacter0xe3d4', [], 'Display', None],
  [ 0xe3d5, 'UnicodeCharacter0xe3d5', [], 'Display', None],
  [ 0xe3d6, 'UnicodeCharacter0xe3d6', [], 'Display', None],
  [ 0xe3d7, 'UnicodeCharacter0xe3d7', [], 'Display', None],
  [ 0xe3d8, 'UnicodeCharacter0xe3d8', [], 'Display', None],
  [ 0xe3d9, 'UnicodeCharacter0xe3d9', [], 'Display', None],
  [ 0xe3da, 'UnicodeCharacter0xe3da', [], 'Display', None],
  [ 0xe3db, 'UnicodeCharacter0xe3db', [], 'Display', None],
  [ 0xe3dc, 'UnicodeCharacter0xe3dc', [], 'Display', None],
  [ 0xe3dd, 'UnicodeCharacter0xe3dd', [], 'Display', None],
  [ 0xe3de, 'UnicodeCharacter0xe3de', [], 'Display', None],
  [ 0xe3df, 'UnicodeCharacter0xe3df', [], 'Display', None],
  [ 0xe3e0, 'UnicodeCharacter0xe3e0', [], 'Display', None],
  [ 0xe3e1, 'UnicodeCharacter0xe3e1', [], 'Display', None],
  [ 0xe3e2, 'UnicodeCharacter0xe3e2', [], 'Display', None],
  [ 0xe3e3, 'UnicodeCharacter0xe3e3', [], 'Display', None],
  [ 0xe3e4, 'UnicodeCharacter0xe3e4', [], 'Display', None],
  [ 0xe3e5, 'UnicodeCharacter0xe3e5', [], 'Display', None],
  [ 0xe3e6, 'UnicodeCharacter0xe3e6', [], 'Display', None],
  [ 0xe3e7, 'UnicodeCharacter0xe3e7', [], 'Display', None],
  [ 0xe3e8, 'UnicodeCharacter0xe3e8', [], 'Display', None],
  [ 0xe3e9, 'UnicodeCharacter0xe3e9', [], 'Display', None],
  [ 0xe3ea, 'UnicodeCharacter0xe3ea', [], 'Display', None],
  [ 0xe3eb, 'UnicodeCharacter0xe3eb', [], 'Display', None],
  [ 0xe3ec, 'UnicodeCharacter0xe3ec', [], 'Display', None],
  [ 0xe3ed, 'UnicodeCharacter0xe3ed', [], 'Display', None],
  [ 0xe3ee, 'UnicodeCharacter0xe3ee', [], 'Display', None],
  [ 0xe3ef, 'UnicodeCharacter0xe3ef', [], 'Display', None],
  [ 0xe3f0, 'UnicodeCharacter0xe3f0', [], 'Display', None],
  [ 0xe3f1, 'UnicodeCharacter0xe3f1', [], 'Display', None],
  [ 0xe3f2, 'UnicodeCharacter0xe3f2', [], 'Display', None],
  [ 0xe3f3, 'UnicodeCharacter0xe3f3', [], 'Display', None],
  [ 0xe3f4, 'UnicodeCharacter0xe3f4', [], 'Display', None],
  [ 0xe3f5, 'UnicodeCharacter0xe3f5', [], 'Display', None],
  [ 0xe3f6, 'UnicodeCharacter0xe3f6', [], 'Display', None],
  [ 0xe3f7, 'UnicodeCharacter0xe3f7', [], 'Display', None],
  [ 0xe3f8, 'UnicodeCharacter0xe3f8', [], 'Display', None],
  [ 0xe3f9, 'UnicodeCharacter0xe3f9', [], 'Display', None],
  [ 0xe3fa, 'UnicodeCharacter0xe3fa', [], 'Display', None],
  [ 0xe3fb, 'UnicodeCharacter0xe3fb', [], 'Display', None],
  [ 0xe3fc, 'UnicodeCharacter0xe3fc', [], 'Display', None],
  [ 0xe3fd, 'UnicodeCharacter0xe3fd', [], 'Display', None],
  [ 0xe3fe, 'UnicodeCharacter0xe3fe', [], 'Display', None],
  [ 0xe3ff, 'UnicodeCharacter0xe3ff', [], 'Display', None],
  [ 0xe400, 'UnicodeCharacter0xe400', [], 'Display', None],
  [ 0xe401, 'UnicodeCharacter0xe401', [], 'Display', None],
  [ 0xe402, 'UnicodeCharacter0xe402', [], 'Display', None],
  [ 0xe403, 'UnicodeCharacter0xe403', [], 'Display', None],
  [ 0xe404, 'UnicodeCharacter0xe404', [], 'Display', None],
  [ 0xe405, 'UnicodeCharacter0xe405', [], 'Display', None],
  [ 0xe406, 'UnicodeCharacter0xe406', [], 'Display', None],
  [ 0xe407, 'UnicodeCharacter0xe407', [], 'Display', None],
  [ 0xe408, 'UnicodeCharacter0xe408', [], 'Display', None],
  [ 0xe409, 'UnicodeCharacter0xe409', [], 'Display', None],
  [ 0xe40a, 'UnicodeCharacter0xe40a', [], 'Display', None],
  [ 0xe40b, 'UnicodeCharacter0xe40b', [], 'Display', None],
  [ 0xe40c, 'UnicodeCharacter0xe40c', [], 'Display', None],
  [ 0xe40d, 'UnicodeCharacter0xe40d', [], 'Display', None],
  [ 0xe40e, 'UnicodeCharacter0xe40e', [], 'Display', None],
  [ 0xe40f, 'UnicodeCharacter0xe40f', [], 'Display', None],
  [ 0xe410, 'UnicodeCharacter0xe410', [], 'Display', None],
  [ 0xe411, 'UnicodeCharacter0xe411', [], 'Display', None],
  [ 0xe412, 'UnicodeCharacter0xe412', [], 'Display', None],
  [ 0xe413, 'UnicodeCharacter0xe413', [], 'Display', None],
  [ 0xe414, 'UnicodeCharacter0xe414', [], 'Display', None],
  [ 0xe415, 'UnicodeCharacter0xe415', [], 'Display', None],
  [ 0xe416, 'UnicodeCharacter0xe416', [], 'Display', None],
  [ 0xe417, 'UnicodeCharacter0xe417', [], 'Display', None],
  [ 0xe418, 'UnicodeCharacter0xe418', [], 'Display', None],
  [ 0xe419, 'UnicodeCharacter0xe419', [], 'Display', None],
  [ 0xe41a, 'UnicodeCharacter0xe41a', [], 'Display', None],
  [ 0xe41b, 'UnicodeCharacter0xe41b', [], 'Display', None],
  [ 0xe41c, 'UnicodeCharacter0xe41c', [], 'Display', None],
  [ 0xe41d, 'UnicodeCharacter0xe41d', [], 'Display', None],
  [ 0xe41e, 'UnicodeCharacter0xe41e', [], 'Display', None],
  [ 0xe41f, 'UnicodeCharacter0xe41f', [], 'Display', None],
  [ 0xe420, 'UnicodeCharacter0xe420', [], 'Display', None],
  [ 0xe421, 'UnicodeCharacter0xe421', [], 'Display', None],
  [ 0xe422, 'UnicodeCharacter0xe422', [], 'Display', None],
  [ 0xe423, 'UnicodeCharacter0xe423', [], 'Display', None],
  [ 0xe424, 'UnicodeCharacter0xe424', [], 'Display', None],
  [ 0xe425, 'UnicodeCharacter0xe425', [], 'Display', None],
  [ 0xe426, 'UnicodeCharacter0xe426', [], 'Display', None],
  [ 0xe427, 'UnicodeCharacter0xe427', [], 'Display', None],
  [ 0xe428, 'UnicodeCharacter0xe428', [], 'Display', None],
  [ 0xe429, 'UnicodeCharacter0xe429', [], 'Display', None],
  [ 0xe42a, 'UnicodeCharacter0xe42a', [], 'Display', None],
  [ 0xe42b, 'UnicodeCharacter0xe42b', [], 'Display', None],
  [ 0xe42c, 'UnicodeCharacter0xe42c', [], 'Display', None],
  [ 0xe42d, 'UnicodeCharacter0xe42d', [], 'Display', None],
  [ 0xe42e, 'UnicodeCharacter0xe42e', [], 'Display', None],
  [ 0xe42f, 'UnicodeCharacter0xe42f', [], 'Display', None],
  [ 0xe430, 'UnicodeCharacter0xe430', [], 'Display', None],
  [ 0xe431, 'UnicodeCharacter0xe431', [], 'Display', None],
  [ 0xe432, 'UnicodeCharacter0xe432', [], 'Display', None],
  [ 0xe433, 'UnicodeCharacter0xe433', [], 'Display', None],
  [ 0xe434, 'UnicodeCharacter0xe434', [], 'Display', None],
  [ 0xe435, 'UnicodeCharacter0xe435', [], 'Display', None],
  [ 0xe436, 'UnicodeCharacter0xe436', [], 'Display', None],
  [ 0xe437, 'UnicodeCharacter0xe437', [], 'Display', None],
  [ 0xe438, 'UnicodeCharacter0xe438', [], 'Display', None],
  [ 0xe439, 'UnicodeCharacter0xe439', [], 'Display', None],
  [ 0xe43a, 'UnicodeCharacter0xe43a', [], 'Display', None],
  [ 0xe43b, 'UnicodeCharacter0xe43b', [], 'Display', None],
  [ 0xe43c, 'UnicodeCharacter0xe43c', [], 'Display', None],
  [ 0xe43d, 'UnicodeCharacter0xe43d', [], 'Display', None],
  [ 0xe43e, 'UnicodeCharacter0xe43e', [], 'Display', None],
  [ 0xe43f, 'UnicodeCharacter0xe43f', [], 'Display', None],
  [ 0xe440, 'UnicodeCharacter0xe440', [], 'Display', None],
  [ 0xe441, 'UnicodeCharacter0xe441', [], 'Display', None],
  [ 0xe442, 'UnicodeCharacter0xe442', [], 'Display', None],
  [ 0xe443, 'UnicodeCharacter0xe443', [], 'Display', None],
  [ 0xe444, 'UnicodeCharacter0xe444', [], 'Display', None],
  [ 0xe445, 'UnicodeCharacter0xe445', [], 'Display', None],
  [ 0xe446, 'UnicodeCharacter0xe446', [], 'Display', None],
  [ 0xe447, 'UnicodeCharacter0xe447', [], 'Display', None],
  [ 0xe448, 'UnicodeCharacter0xe448', [], 'Display', None],
  [ 0xe449, 'UnicodeCharacter0xe449', [], 'Display', None],
  [ 0xe44a, 'UnicodeCharacter0xe44a', [], 'Display', None],
  [ 0xe44b, 'UnicodeCharacter0xe44b', [], 'Display', None],
  [ 0xe44c, 'UnicodeCharacter0xe44c', [], 'Display', None],
  [ 0xe44d, 'UnicodeCharacter0xe44d', [], 'Display', None],
  [ 0xe44e, 'UnicodeCharacter0xe44e', [], 'Display', None],
  [ 0xe44f, 'UnicodeCharacter0xe44f', [], 'Display', None],
  [ 0xe450, 'UnicodeCharacter0xe450', [], 'Display', None],
  [ 0xe451, 'UnicodeCharacter0xe451', [], 'Display', None],
  [ 0xe452, 'UnicodeCharacter0xe452', [], 'Display', None],
  [ 0xe453, 'UnicodeCharacter0xe453', [], 'Display', None],
  [ 0xe454, 'UnicodeCharacter0xe454', [], 'Display', None],
  [ 0xe455, 'UnicodeCharacter0xe455', [], 'Display', None],
  [ 0xe456, 'UnicodeCharacter0xe456', [], 'Display', None],
  [ 0xe457, 'UnicodeCharacter0xe457', [], 'Display', None],
  [ 0xe458, 'UnicodeCharacter0xe458', [], 'Display', None],
  [ 0xe459, 'UnicodeCharacter0xe459', [], 'Display', None],
  [ 0xe45a, 'UnicodeCharacter0xe45a', [], 'Display', None],
  [ 0xe45b, 'UnicodeCharacter0xe45b', [], 'Display', None],
  [ 0xe45c, 'UnicodeCharacter0xe45c', [], 'Display', None],
  [ 0xe45d, 'UnicodeCharacter0xe45d', [], 'Display', None],
  [ 0xe45e, 'UnicodeCharacter0xe45e', [], 'Display', None],
  [ 0xe45f, 'UnicodeCharacter0xe45f', [], 'Display', None],
  [ 0xe460, 'UnicodeCharacter0xe460', [], 'Display', None],
  [ 0xe461, 'UnicodeCharacter0xe461', [], 'Display', None],
  [ 0xe462, 'UnicodeCharacter0xe462', [], 'Display', None],
  [ 0xe463, 'UnicodeCharacter0xe463', [], 'Display', None],
  [ 0xe464, 'UnicodeCharacter0xe464', [], 'Display', None],
  [ 0xe465, 'UnicodeCharacter0xe465', [], 'Display', None],
  [ 0xe466, 'UnicodeCharacter0xe466', [], 'Display', None],
  [ 0xe467, 'UnicodeCharacter0xe467', [], 'Display', None],
  [ 0xe468, 'UnicodeCharacter0xe468', [], 'Display', None],
  [ 0xe469, 'UnicodeCharacter0xe469', [], 'Display', None],
  [ 0xe46a, 'UnicodeCharacter0xe46a', [], 'Display', None],
  [ 0xe46b, 'UnicodeCharacter0xe46b', [], 'Display', None],
  [ 0xe46c, 'UnicodeCharacter0xe46c', [], 'Display', None],
  [ 0xe46d, 'UnicodeCharacter0xe46d', [], 'Display', None],
  [ 0xe46e, 'UnicodeCharacter0xe46e', [], 'Display', None],
  [ 0xe46f, 'UnicodeCharacter0xe46f', [], 'Display', None],
  [ 0xe470, 'UnicodeCharacter0xe470', [], 'Display', None],
  [ 0xe471, 'UnicodeCharacter0xe471', [], 'Display', None],
  [ 0xe472, 'UnicodeCharacter0xe472', [], 'Display', None],
  [ 0xe473, 'UnicodeCharacter0xe473', [], 'Display', None],
  [ 0xe474, 'UnicodeCharacter0xe474', [], 'Display', None],
  [ 0xe475, 'UnicodeCharacter0xe475', [], 'Display', None],
  [ 0xe476, 'UnicodeCharacter0xe476', [], 'Display', None],
  [ 0xe477, 'UnicodeCharacter0xe477', [], 'Display', None],
  [ 0xe478, 'UnicodeCharacter0xe478', [], 'Display', None],
  [ 0xe479, 'UnicodeCharacter0xe479', [], 'Display', None],
  [ 0xe47a, 'UnicodeCharacter0xe47a', [], 'Display', None],
  [ 0xe47b, 'UnicodeCharacter0xe47b', [], 'Display', None],
  [ 0xe47c, 'UnicodeCharacter0xe47c', [], 'Display', None],
  [ 0xe47d, 'UnicodeCharacter0xe47d', [], 'Display', None],
  [ 0xe47e, 'UnicodeCharacter0xe47e', [], 'Display', None],
  [ 0xe47f, 'UnicodeCharacter0xe47f', [], 'Display', None],
  [ 0xe480, 'UnicodeCharacter0xe480', [], 'Display', None],
  [ 0xe481, 'UnicodeCharacter0xe481', [], 'Display', None],
  [ 0xe482, 'UnicodeCharacter0xe482', [], 'Display', None],
  [ 0xe483, 'UnicodeCharacter0xe483', [], 'Display', None],
  [ 0xe484, 'UnicodeCharacter0xe484', [], 'Display', None],
  [ 0xe485, 'UnicodeCharacter0xe485', [], 'Display', None],
  [ 0xe486, 'UnicodeCharacter0xe486', [], 'Display', None],
  [ 0xe487, 'UnicodeCharacter0xe487', [], 'Display', None],
  [ 0xe488, 'UnicodeCharacter0xe488', [], 'Display', None],
  [ 0xe489, 'UnicodeCharacter0xe489', [], 'Display', None],
  [ 0xe48a, 'UnicodeCharacter0xe48a', [], 'Display', None],
  [ 0xe48b, 'UnicodeCharacter0xe48b', [], 'Display', None],
  [ 0xe48c, 'UnicodeCharacter0xe48c', [], 'Display', None],
  [ 0xe48d, 'UnicodeCharacter0xe48d', [], 'Display', None],
  [ 0xe48e, 'UnicodeCharacter0xe48e', [], 'Display', None],
  [ 0xe48f, 'UnicodeCharacter0xe48f', [], 'Display', None],
  [ 0xe490, 'UnicodeCharacter0xe490', [], 'Display', None],
  [ 0xe491, 'UnicodeCharacter0xe491', [], 'Display', None],
  [ 0xe492, 'UnicodeCharacter0xe492', [], 'Display', None],
  [ 0xe493, 'UnicodeCharacter0xe493', [], 'Display', None],
  [ 0xe494, 'UnicodeCharacter0xe494', [], 'Display', None],
  [ 0xe495, 'UnicodeCharacter0xe495', [], 'Display', None],
  [ 0xe496, 'UnicodeCharacter0xe496', [], 'Display', None],
  [ 0xe497, 'UnicodeCharacter0xe497', [], 'Display', None],
  [ 0xe498, 'UnicodeCharacter0xe498', [], 'Display', None],
  [ 0xe499, 'UnicodeCharacter0xe499', [], 'Display', None],
  [ 0xe49a, 'UnicodeCharacter0xe49a', [], 'Display', None],
  [ 0xe49b, 'UnicodeCharacter0xe49b', [], 'Display', None],
  [ 0xe49c, 'UnicodeCharacter0xe49c', [], 'Display', None],
  [ 0xe49d, 'UnicodeCharacter0xe49d', [], 'Display', None],
  [ 0xe49e, 'UnicodeCharacter0xe49e', [], 'Display', None],
  [ 0xe49f, 'UnicodeCharacter0xe49f', [], 'Display', None],
  [ 0xe4a0, 'UnicodeCharacter0xe4a0', [], 'Display', None],
  [ 0xe4a1, 'UnicodeCharacter0xe4a1', [], 'Display', None],
  [ 0xe4a2, 'UnicodeCharacter0xe4a2', [], 'Display', None],
  [ 0xe4a3, 'UnicodeCharacter0xe4a3', [], 'Display', None],
  [ 0xe4a4, 'UnicodeCharacter0xe4a4', [], 'Display', None],
  [ 0xe4a5, 'UnicodeCharacter0xe4a5', [], 'Display', None],
  [ 0xe4a6, 'UnicodeCharacter0xe4a6', [], 'Display', None],
  [ 0xe4a7, 'UnicodeCharacter0xe4a7', [], 'Display', None],
  [ 0xe4a8, 'UnicodeCharacter0xe4a8', [], 'Display', None],
  [ 0xe4a9, 'UnicodeCharacter0xe4a9', [], 'Display', None],
  [ 0xe4aa, 'UnicodeCharacter0xe4aa', [], 'Display', None],
  [ 0xe4ab, 'UnicodeCharacter0xe4ab', [], 'Display', None],
  [ 0xe4ac, 'UnicodeCharacter0xe4ac', [], 'Display', None],
  [ 0xe4ad, 'UnicodeCharacter0xe4ad', [], 'Display', None],
  [ 0xe4ae, 'UnicodeCharacter0xe4ae', [], 'Display', None],
  [ 0xe4af, 'UnicodeCharacter0xe4af', [], 'Display', None],
  [ 0xe4b0, 'UnicodeCharacter0xe4b0', [], 'Display', None],
  [ 0xe4b1, 'UnicodeCharacter0xe4b1', [], 'Display', None],
  [ 0xe4b2, 'UnicodeCharacter0xe4b2', [], 'Display', None],
  [ 0xe4b3, 'UnicodeCharacter0xe4b3', [], 'Display', None],
  [ 0xe4b4, 'UnicodeCharacter0xe4b4', [], 'Display', None],
  [ 0xe4b5, 'UnicodeCharacter0xe4b5', [], 'Display', None],
  [ 0xe4b6, 'UnicodeCharacter0xe4b6', [], 'Display', None],
  [ 0xe4b7, 'UnicodeCharacter0xe4b7', [], 'Display', None],
  [ 0xe4b8, 'UnicodeCharacter0xe4b8', [], 'Display', None],
  [ 0xe4b9, 'UnicodeCharacter0xe4b9', [], 'Display', None],
  [ 0xe4ba, 'UnicodeCharacter0xe4ba', [], 'Display', None],
  [ 0xe4bb, 'UnicodeCharacter0xe4bb', [], 'Display', None],
  [ 0xe4bc, 'UnicodeCharacter0xe4bc', [], 'Display', None],
  [ 0xe4bd, 'UnicodeCharacter0xe4bd', [], 'Display', None],
  [ 0xe4be, 'UnicodeCharacter0xe4be', [], 'Display', None],
  [ 0xe4bf, 'UnicodeCharacter0xe4bf', [], 'Display', None],
  [ 0xe4c0, 'UnicodeCharacter0xe4c0', [], 'Display', None],
  [ 0xe4c1, 'UnicodeCharacter0xe4c1', [], 'Display', None],
  [ 0xe4c2, 'UnicodeCharacter0xe4c2', [], 'Display', None],
  [ 0xe4c3, 'UnicodeCharacter0xe4c3', [], 'Display', None],
  [ 0xe4c4, 'UnicodeCharacter0xe4c4', [], 'Display', None],
  [ 0xe4c5, 'UnicodeCharacter0xe4c5', [], 'Display', None],
  [ 0xe4c6, 'UnicodeCharacter0xe4c6', [], 'Display', None],
  [ 0xe4c7, 'UnicodeCharacter0xe4c7', [], 'Display', None],
  [ 0xe4c8, 'UnicodeCharacter0xe4c8', [], 'Display', None],
  [ 0xe4c9, 'UnicodeCharacter0xe4c9', [], 'Display', None],
  [ 0xe4ca, 'UnicodeCharacter0xe4ca', [], 'Display', None],
  [ 0xe4cb, 'UnicodeCharacter0xe4cb', [], 'Display', None],
  [ 0xe4cc, 'UnicodeCharacter0xe4cc', [], 'Display', None],
  [ 0xe4cd, 'UnicodeCharacter0xe4cd', [], 'Display', None],
  [ 0xe4ce, 'UnicodeCharacter0xe4ce', [], 'Display', None],
  [ 0xe4cf, 'UnicodeCharacter0xe4cf', [], 'Display', None],
  [ 0xe4d0, 'UnicodeCharacter0xe4d0', [], 'Display', None],
  [ 0xe4d1, 'UnicodeCharacter0xe4d1', [], 'Display', None],
  [ 0xe4d2, 'UnicodeCharacter0xe4d2', [], 'Display', None],
  [ 0xe4d3, 'UnicodeCharacter0xe4d3', [], 'Display', None],
  [ 0xe4d4, 'UnicodeCharacter0xe4d4', [], 'Display', None],
  [ 0xe4d5, 'UnicodeCharacter0xe4d5', [], 'Display', None],
  [ 0xe4d6, 'UnicodeCharacter0xe4d6', [], 'Display', None],
  [ 0xe4d7, 'UnicodeCharacter0xe4d7', [], 'Display', None],
  [ 0xe4d8, 'UnicodeCharacter0xe4d8', [], 'Display', None],
  [ 0xe4d9, 'UnicodeCharacter0xe4d9', [], 'Display', None],
  [ 0xe4da, 'UnicodeCharacter0xe4da', [], 'Display', None],
  [ 0xe4db, 'UnicodeCharacter0xe4db', [], 'Display', None],
  [ 0xe4dc, 'UnicodeCharacter0xe4dc', [], 'Display', None],
  [ 0xe4dd, 'UnicodeCharacter0xe4dd', [], 'Display', None],
  [ 0xe4de, 'UnicodeCharacter0xe4de', [], 'Display', None],
  [ 0xe4df, 'UnicodeCharacter0xe4df', [], 'Display', None],
  [ 0xe4e0, 'UnicodeCharacter0xe4e0', [], 'Display', None],
  [ 0xe4e1, 'UnicodeCharacter0xe4e1', [], 'Display', None],
  [ 0xe4e2, 'UnicodeCharacter0xe4e2', [], 'Display', None],
  [ 0xe4e3, 'UnicodeCharacter0xe4e3', [], 'Display', None],
  [ 0xe4e4, 'UnicodeCharacter0xe4e4', [], 'Display', None],
  [ 0xe4e5, 'UnicodeCharacter0xe4e5', [], 'Display', None],
  [ 0xe4e6, 'UnicodeCharacter0xe4e6', [], 'Display', None],
  [ 0xe4e7, 'UnicodeCharacter0xe4e7', [], 'Display', None],
  [ 0xe4e8, 'UnicodeCharacter0xe4e8', [], 'Display', None],
  [ 0xe4e9, 'UnicodeCharacter0xe4e9', [], 'Display', None],
  [ 0xe4ea, 'UnicodeCharacter0xe4ea', [], 'Display', None],
  [ 0xe4eb, 'UnicodeCharacter0xe4eb', [], 'Display', None],
  [ 0xe4ec, 'UnicodeCharacter0xe4ec', [], 'Display', None],
  [ 0xe4ed, 'UnicodeCharacter0xe4ed', [], 'Display', None],
  [ 0xe4ee, 'UnicodeCharacter0xe4ee', [], 'Display', None],
  [ 0xe4ef, 'UnicodeCharacter0xe4ef', [], 'Display', None],
  [ 0xe4f0, 'UnicodeCharacter0xe4f0', [], 'Display', None],
  [ 0xe4f1, 'UnicodeCharacter0xe4f1', [], 'Display', None],
  [ 0xe4f2, 'UnicodeCharacter0xe4f2', [], 'Display', None],
  [ 0xe4f3, 'UnicodeCharacter0xe4f3', [], 'Display', None],
  [ 0xe4f4, 'UnicodeCharacter0xe4f4', [], 'Display', None],
  [ 0xe4f5, 'UnicodeCharacter0xe4f5', [], 'Display', None],
  [ 0xe4f6, 'UnicodeCharacter0xe4f6', [], 'Display', None],
  [ 0xe4f7, 'UnicodeCharacter0xe4f7', [], 'Display', None],
  [ 0xe4f8, 'UnicodeCharacter0xe4f8', [], 'Display', None],
  [ 0xe4f9, 'UnicodeCharacter0xe4f9', [], 'Display', None],
  [ 0xe4fa, 'UnicodeCharacter0xe4fa', [], 'Display', None],
  [ 0xe4fb, 'UnicodeCharacter0xe4fb', [], 'Display', None],
  [ 0xe4fc, 'UnicodeCharacter0xe4fc', [], 'Display', None],
  [ 0xe4fd, 'UnicodeCharacter0xe4fd', [], 'Display', None],
  [ 0xe4fe, 'UnicodeCharacter0xe4fe', [], 'Display', None],
  [ 0xe4ff, 'UnicodeCharacter0xe4ff', [], 'Display', None],
  [ 0xe500, 'UnicodeCharacter0xe500', [], 'Display', None],
  [ 0xe501, 'UnicodeCharacter0xe501', [], 'Display', None],
  [ 0xe502, 'UnicodeCharacter0xe502', [], 'Display', None],
  [ 0xe503, 'UnicodeCharacter0xe503', [], 'Display', None],
  [ 0xe504, 'UnicodeCharacter0xe504', [], 'Display', None],
  [ 0xe505, 'UnicodeCharacter0xe505', [], 'Display', None],
  [ 0xe506, 'UnicodeCharacter0xe506', [], 'Display', None],
  [ 0xe507, 'UnicodeCharacter0xe507', [], 'Display', None],
  [ 0xe508, 'UnicodeCharacter0xe508', [], 'Display', None],
  [ 0xe509, 'UnicodeCharacter0xe509', [], 'Display', None],
  [ 0xe50a, 'UnicodeCharacter0xe50a', [], 'Display', None],
  [ 0xe50b, 'UnicodeCharacter0xe50b', [], 'Display', None],
  [ 0xe50c, 'UnicodeCharacter0xe50c', [], 'Display', None],
  [ 0xe50d, 'UnicodeCharacter0xe50d', [], 'Display', None],
  [ 0xe50e, 'UnicodeCharacter0xe50e', [], 'Display', None],
  [ 0xe50f, 'UnicodeCharacter0xe50f', [], 'Display', None],
  [ 0xe510, 'UnicodeCharacter0xe510', [], 'Display', None],
  [ 0xe511, 'UnicodeCharacter0xe511', [], 'Display', None],
  [ 0xe512, 'UnicodeCharacter0xe512', [], 'Display', None],
  [ 0xe513, 'UnicodeCharacter0xe513', [], 'Display', None],
  [ 0xe514, 'UnicodeCharacter0xe514', [], 'Display', None],
  [ 0xe515, 'UnicodeCharacter0xe515', [], 'Display', None],
  [ 0xe516, 'UnicodeCharacter0xe516', [], 'Display', None],
  [ 0xe517, 'UnicodeCharacter0xe517', [], 'Display', None],
  [ 0xe518, 'UnicodeCharacter0xe518', [], 'Display', None],
  [ 0xe519, 'UnicodeCharacter0xe519', [], 'Display', None],
  [ 0xe51a, 'UnicodeCharacter0xe51a', [], 'Display', None],
  [ 0xe51b, 'UnicodeCharacter0xe51b', [], 'Display', None],
  [ 0xe51c, 'UnicodeCharacter0xe51c', [], 'Display', None],
  [ 0xe51d, 'UnicodeCharacter0xe51d', [], 'Display', None],
  [ 0xe51e, 'UnicodeCharacter0xe51e', [], 'Display', None],
  [ 0xe51f, 'UnicodeCharacter0xe51f', [], 'Display', None],
  [ 0xe520, 'UnicodeCharacter0xe520', [], 'Display', None],
  [ 0xe521, 'UnicodeCharacter0xe521', [], 'Display', None],
  [ 0xe522, 'UnicodeCharacter0xe522', [], 'Display', None],
  [ 0xe523, 'UnicodeCharacter0xe523', [], 'Display', None],
  [ 0xe524, 'UnicodeCharacter0xe524', [], 'Display', None],
  [ 0xe525, 'UnicodeCharacter0xe525', [], 'Display', None],
  [ 0xe526, 'UnicodeCharacter0xe526', [], 'Display', None],
  [ 0xe527, 'UnicodeCharacter0xe527', [], 'Display', None],
  [ 0xe528, 'UnicodeCharacter0xe528', [], 'Display', None],
  [ 0xe529, 'UnicodeCharacter0xe529', [], 'Display', None],
  [ 0xe52a, 'UnicodeCharacter0xe52a', [], 'Display', None],
  [ 0xe52b, 'UnicodeCharacter0xe52b', [], 'Display', None],
  [ 0xe52c, 'UnicodeCharacter0xe52c', [], 'Display', None],
  [ 0xe52d, 'UnicodeCharacter0xe52d', [], 'Display', None],
  [ 0xe52e, 'UnicodeCharacter0xe52e', [], 'Display', None],
  [ 0xe52f, 'UnicodeCharacter0xe52f', [], 'Display', None],
  [ 0xe530, 'UnicodeCharacter0xe530', [], 'Display', None],
  [ 0xe531, 'UnicodeCharacter0xe531', [], 'Display', None],
  [ 0xe532, 'UnicodeCharacter0xe532', [], 'Display', None],
  [ 0xe533, 'UnicodeCharacter0xe533', [], 'Display', None],
  [ 0xe534, 'UnicodeCharacter0xe534', [], 'Display', None],
  [ 0xe535, 'UnicodeCharacter0xe535', [], 'Display', None],
  [ 0xe536, 'UnicodeCharacter0xe536', [], 'Display', None],
  [ 0xe537, 'UnicodeCharacter0xe537', [], 'Display', None],
  [ 0xe538, 'UnicodeCharacter0xe538', [], 'Display', None],
  [ 0xe539, 'UnicodeCharacter0xe539', [], 'Display', None],
  [ 0xe53a, 'UnicodeCharacter0xe53a', [], 'Display', None],
  [ 0xe53b, 'UnicodeCharacter0xe53b', [], 'Display', None],
  [ 0xe53c, 'UnicodeCharacter0xe53c', [], 'Display', None],
  [ 0xe53d, 'UnicodeCharacter0xe53d', [], 'Display', None],
  [ 0xe53e, 'UnicodeCharacter0xe53e', [], 'Display', None],
  [ 0xe53f, 'UnicodeCharacter0xe53f', [], 'Display', None],
  [ 0xe540, 'UnicodeCharacter0xe540', [], 'Display', None],
  [ 0xe541, 'UnicodeCharacter0xe541', [], 'Display', None],
  [ 0xe542, 'UnicodeCharacter0xe542', [], 'Display', None],
  [ 0xe543, 'UnicodeCharacter0xe543', [], 'Display', None],
  [ 0xe544, 'UnicodeCharacter0xe544', [], 'Display', None],
  [ 0xe545, 'UnicodeCharacter0xe545', [], 'Display', None],
  [ 0xe546, 'UnicodeCharacter0xe546', [], 'Display', None],
  [ 0xe547, 'UnicodeCharacter0xe547', [], 'Display', None],
  [ 0xe548, 'UnicodeCharacter0xe548', [], 'Display', None],
  [ 0xe549, 'UnicodeCharacter0xe549', [], 'Display', None],
  [ 0xe54a, 'UnicodeCharacter0xe54a', [], 'Display', None],
  [ 0xe54b, 'UnicodeCharacter0xe54b', [], 'Display', None],
  [ 0xe54c, 'UnicodeCharacter0xe54c', [], 'Display', None],
  [ 0xe54d, 'UnicodeCharacter0xe54d', [], 'Display', None],
  [ 0xe54e, 'UnicodeCharacter0xe54e', [], 'Display', None],
  [ 0xe54f, 'UnicodeCharacter0xe54f', [], 'Display', None],
  [ 0xe550, 'UnicodeCharacter0xe550', [], 'Display', None],
  [ 0xe551, 'UnicodeCharacter0xe551', [], 'Display', None],
  [ 0xe552, 'UnicodeCharacter0xe552', [], 'Display', None],
  [ 0xe553, 'UnicodeCharacter0xe553', [], 'Display', None],
  [ 0xe554, 'UnicodeCharacter0xe554', [], 'Display', None],
  [ 0xe555, 'UnicodeCharacter0xe555', [], 'Display', None],
  [ 0xe556, 'UnicodeCharacter0xe556', [], 'Display', None],
  [ 0xe557, 'UnicodeCharacter0xe557', [], 'Display', None],
  [ 0xe558, 'UnicodeCharacter0xe558', [], 'Display', None],
  [ 0xe559, 'UnicodeCharacter0xe559', [], 'Display', None],
  [ 0xe55a, 'UnicodeCharacter0xe55a', [], 'Display', None],
  [ 0xe55b, 'UnicodeCharacter0xe55b', [], 'Display', None],
  [ 0xe55c, 'UnicodeCharacter0xe55c', [], 'Display', None],
  [ 0xe55d, 'UnicodeCharacter0xe55d', [], 'Display', None],
  [ 0xe55e, 'UnicodeCharacter0xe55e', [], 'Display', None],
  [ 0xe55f, 'UnicodeCharacter0xe55f', [], 'Display', None],
  [ 0xe560, 'UnicodeCharacter0xe560', [], 'Display', None],
  [ 0xe561, 'UnicodeCharacter0xe561', [], 'Display', None],
  [ 0xe562, 'UnicodeCharacter0xe562', [], 'Display', None],
  [ 0xe563, 'UnicodeCharacter0xe563', [], 'Display', None],
  [ 0xe564, 'UnicodeCharacter0xe564', [], 'Display', None],
  [ 0xe565, 'UnicodeCharacter0xe565', [], 'Display', None],
  [ 0xe566, 'UnicodeCharacter0xe566', [], 'Display', None],
  [ 0xe567, 'UnicodeCharacter0xe567', [], 'Display', None],
  [ 0xe568, 'UnicodeCharacter0xe568', [], 'Display', None],
  [ 0xe569, 'UnicodeCharacter0xe569', [], 'Display', None],
  [ 0xe56a, 'UnicodeCharacter0xe56a', [], 'Display', None],
  [ 0xe56b, 'UnicodeCharacter0xe56b', [], 'Display', None],
  [ 0xe56c, 'UnicodeCharacter0xe56c', [], 'Display', None],
  [ 0xe56d, 'UnicodeCharacter0xe56d', [], 'Display', None],
  [ 0xe56e, 'UnicodeCharacter0xe56e', [], 'Display', None],
  [ 0xe56f, 'UnicodeCharacter0xe56f', [], 'Display', None],
  [ 0xe570, 'UnicodeCharacter0xe570', [], 'Display', None],
  [ 0xe571, 'UnicodeCharacter0xe571', [], 'Display', None],
  [ 0xe572, 'UnicodeCharacter0xe572', [], 'Display', None],
  [ 0xe573, 'UnicodeCharacter0xe573', [], 'Display', None],
  [ 0xe574, 'UnicodeCharacter0xe574', [], 'Display', None],
  [ 0xe575, 'UnicodeCharacter0xe575', [], 'Display', None],
  [ 0xe576, 'UnicodeCharacter0xe576', [], 'Display', None],
  [ 0xe577, 'UnicodeCharacter0xe577', [], 'Display', None],
  [ 0xe578, 'UnicodeCharacter0xe578', [], 'Display', None],
  [ 0xe579, 'UnicodeCharacter0xe579', [], 'Display', None],
  [ 0xe57a, 'UnicodeCharacter0xe57a', [], 'Display', None],
  [ 0xe57b, 'UnicodeCharacter0xe57b', [], 'Display', None],
  [ 0xe57c, 'UnicodeCharacter0xe57c', [], 'Display', None],
  [ 0xe57d, 'UnicodeCharacter0xe57d', [], 'Display', None],
  [ 0xe57e, 'UnicodeCharacter0xe57e', [], 'Display', None],
  [ 0xe57f, 'UnicodeCharacter0xe57f', [], 'Display', None],
  [ 0xe580, 'UnicodeCharacter0xe580', [], 'Display', None],
  [ 0xe581, 'UnicodeCharacter0xe581', [], 'Display', None],
  [ 0xe582, 'UnicodeCharacter0xe582', [], 'Display', None],
  [ 0xe583, 'UnicodeCharacter0xe583', [], 'Display', None],
  [ 0xe584, 'UnicodeCharacter0xe584', [], 'Display', None],
  [ 0xe585, 'UnicodeCharacter0xe585', [], 'Display', None],
  [ 0xe586, 'UnicodeCharacter0xe586', [], 'Display', None],
  [ 0xe587, 'UnicodeCharacter0xe587', [], 'Display', None],
  [ 0xe588, 'UnicodeCharacter0xe588', [], 'Display', None],
  [ 0xe589, 'UnicodeCharacter0xe589', [], 'Display', None],
  [ 0xe58a, 'UnicodeCharacter0xe58a', [], 'Display', None],
  [ 0xe58b, 'UnicodeCharacter0xe58b', [], 'Display', None],
  [ 0xe58c, 'UnicodeCharacter0xe58c', [], 'Display', None],
  [ 0xe58d, 'UnicodeCharacter0xe58d', [], 'Display', None],
  [ 0xe58e, 'UnicodeCharacter0xe58e', [], 'Display', None],
  [ 0xe58f, 'UnicodeCharacter0xe58f', [], 'Display', None],
  [ 0xe590, 'UnicodeCharacter0xe590', [], 'Display', None],
  [ 0xe591, 'UnicodeCharacter0xe591', [], 'Display', None],
  [ 0xe592, 'UnicodeCharacter0xe592', [], 'Display', None],
  [ 0xe593, 'UnicodeCharacter0xe593', [], 'Display', None],
  [ 0xe594, 'UnicodeCharacter0xe594', [], 'Display', None],
  [ 0xe595, 'UnicodeCharacter0xe595', [], 'Display', None],
  [ 0xe596, 'UnicodeCharacter0xe596', [], 'Display', None],
  [ 0xe597, 'UnicodeCharacter0xe597', [], 'Display', None],
  [ 0xe598, 'UnicodeCharacter0xe598', [], 'Display', None],
  [ 0xe599, 'UnicodeCharacter0xe599', [], 'Display', None],
  [ 0xe59a, 'UnicodeCharacter0xe59a', [], 'Display', None],
  [ 0xe59b, 'UnicodeCharacter0xe59b', [], 'Display', None],
  [ 0xe59c, 'UnicodeCharacter0xe59c', [], 'Display', None],
  [ 0xe59d, 'UnicodeCharacter0xe59d', [], 'Display', None],
  [ 0xe59e, 'UnicodeCharacter0xe59e', [], 'Display', None],
  [ 0xe59f, 'UnicodeCharacter0xe59f', [], 'Display', None],
  [ 0xe5a0, 'UnicodeCharacter0xe5a0', [], 'Display', None],
  [ 0xe5a1, 'UnicodeCharacter0xe5a1', [], 'Display', None],
  [ 0xe5a2, 'UnicodeCharacter0xe5a2', [], 'Display', None],
  [ 0xe5a3, 'UnicodeCharacter0xe5a3', [], 'Display', None],
  [ 0xe5a4, 'UnicodeCharacter0xe5a4', [], 'Display', None],
  [ 0xe5a5, 'UnicodeCharacter0xe5a5', [], 'Display', None],
  [ 0xe5a6, 'UnicodeCharacter0xe5a6', [], 'Display', None],
  [ 0xe5a7, 'UnicodeCharacter0xe5a7', [], 'Display', None],
  [ 0xe5a8, 'UnicodeCharacter0xe5a8', [], 'Display', None],
  [ 0xe5a9, 'UnicodeCharacter0xe5a9', [], 'Display', None],
  [ 0xe5aa, 'UnicodeCharacter0xe5aa', [], 'Display', None],
  [ 0xe5ab, 'UnicodeCharacter0xe5ab', [], 'Display', None],
  [ 0xe5ac, 'UnicodeCharacter0xe5ac', [], 'Display', None],
  [ 0xe5ad, 'UnicodeCharacter0xe5ad', [], 'Display', None],
  [ 0xe5ae, 'UnicodeCharacter0xe5ae', [], 'Display', None],
  [ 0xe5af, 'UnicodeCharacter0xe5af', [], 'Display', None],
  [ 0xe5b0, 'UnicodeCharacter0xe5b0', [], 'Display', None],
  [ 0xe5b1, 'UnicodeCharacter0xe5b1', [], 'Display', None],
  [ 0xe5b2, 'UnicodeCharacter0xe5b2', [], 'Display', None],
  [ 0xe5b3, 'UnicodeCharacter0xe5b3', [], 'Display', None],
  [ 0xe5b4, 'UnicodeCharacter0xe5b4', [], 'Display', None],
  [ 0xe5b5, 'UnicodeCharacter0xe5b5', [], 'Display', None],
  [ 0xe5b6, 'UnicodeCharacter0xe5b6', [], 'Display', None],
  [ 0xe5b7, 'UnicodeCharacter0xe5b7', [], 'Display', None],
  [ 0xe5b8, 'UnicodeCharacter0xe5b8', [], 'Display', None],
  [ 0xe5b9, 'UnicodeCharacter0xe5b9', [], 'Display', None],
  [ 0xe5ba, 'UnicodeCharacter0xe5ba', [], 'Display', None],
  [ 0xe5bb, 'UnicodeCharacter0xe5bb', [], 'Display', None],
  [ 0xe5bc, 'UnicodeCharacter0xe5bc', [], 'Display', None],
  [ 0xe5bd, 'UnicodeCharacter0xe5bd', [], 'Display', None],
  [ 0xe5be, 'UnicodeCharacter0xe5be', [], 'Display', None],
  [ 0xe5bf, 'UnicodeCharacter0xe5bf', [], 'Display', None],
  [ 0xe5c0, 'UnicodeCharacter0xe5c0', [], 'Display', None],
  [ 0xe5c1, 'UnicodeCharacter0xe5c1', [], 'Display', None],
  [ 0xe5c2, 'UnicodeCharacter0xe5c2', [], 'Display', None],
  [ 0xe5c3, 'UnicodeCharacter0xe5c3', [], 'Display', None],
  [ 0xe5c4, 'UnicodeCharacter0xe5c4', [], 'Display', None],
  [ 0xe5c5, 'UnicodeCharacter0xe5c5', [], 'Display', None],
  [ 0xe5c6, 'UnicodeCharacter0xe5c6', [], 'Display', None],
  [ 0xe5c7, 'UnicodeCharacter0xe5c7', [], 'Display', None],
  [ 0xe5c8, 'UnicodeCharacter0xe5c8', [], 'Display', None],
  [ 0xe5c9, 'UnicodeCharacter0xe5c9', [], 'Display', None],
  [ 0xe5ca, 'UnicodeCharacter0xe5ca', [], 'Display', None],
  [ 0xe5cb, 'UnicodeCharacter0xe5cb', [], 'Display', None],
  [ 0xe5cc, 'UnicodeCharacter0xe5cc', [], 'Display', None],
  [ 0xe5cd, 'UnicodeCharacter0xe5cd', [], 'Display', None],
  [ 0xe5ce, 'UnicodeCharacter0xe5ce', [], 'Display', None],
  [ 0xe5cf, 'UnicodeCharacter0xe5cf', [], 'Display', None],
  [ 0xe5d0, 'UnicodeCharacter0xe5d0', [], 'Display', None],
  [ 0xe5d1, 'UnicodeCharacter0xe5d1', [], 'Display', None],
  [ 0xe5d2, 'UnicodeCharacter0xe5d2', [], 'Display', None],
  [ 0xe5d3, 'UnicodeCharacter0xe5d3', [], 'Display', None],
  [ 0xe5d4, 'UnicodeCharacter0xe5d4', [], 'Display', None],
  [ 0xe5d5, 'UnicodeCharacter0xe5d5', [], 'Display', None],
  [ 0xe5d6, 'UnicodeCharacter0xe5d6', [], 'Display', None],
  [ 0xe5d7, 'UnicodeCharacter0xe5d7', [], 'Display', None],
  [ 0xe5d8, 'UnicodeCharacter0xe5d8', [], 'Display', None],
  [ 0xe5d9, 'UnicodeCharacter0xe5d9', [], 'Display', None],
  [ 0xe5da, 'UnicodeCharacter0xe5da', [], 'Display', None],
  [ 0xe5db, 'UnicodeCharacter0xe5db', [], 'Display', None],
  [ 0xe5dc, 'UnicodeCharacter0xe5dc', [], 'Display', None],
  [ 0xe5dd, 'UnicodeCharacter0xe5dd', [], 'Display', None],
  [ 0xe5de, 'UnicodeCharacter0xe5de', [], 'Display', None],
  [ 0xe5df, 'UnicodeCharacter0xe5df', [], 'Display', None],
  [ 0xe5e0, 'UnicodeCharacter0xe5e0', [], 'Display', None],
  [ 0xe5e1, 'UnicodeCharacter0xe5e1', [], 'Display', None],
  [ 0xe5e2, 'UnicodeCharacter0xe5e2', [], 'Display', None],
  [ 0xe5e3, 'UnicodeCharacter0xe5e3', [], 'Display', None],
  [ 0xe5e4, 'UnicodeCharacter0xe5e4', [], 'Display', None],
  [ 0xe5e5, 'UnicodeCharacter0xe5e5', [], 'Display', None],
  [ 0xe5e6, 'UnicodeCharacter0xe5e6', [], 'Display', None],
  [ 0xe5e7, 'UnicodeCharacter0xe5e7', [], 'Display', None],
  [ 0xe5e8, 'UnicodeCharacter0xe5e8', [], 'Display', None],
  [ 0xe5e9, 'UnicodeCharacter0xe5e9', [], 'Display', None],
  [ 0xe5ea, 'UnicodeCharacter0xe5ea', [], 'Display', None],
  [ 0xe5eb, 'UnicodeCharacter0xe5eb', [], 'Display', None],
  [ 0xe5ec, 'UnicodeCharacter0xe5ec', [], 'Display', None],
  [ 0xe5ed, 'UnicodeCharacter0xe5ed', [], 'Display', None],
  [ 0xe5ee, 'UnicodeCharacter0xe5ee', [], 'Display', None],
  [ 0xe5ef, 'UnicodeCharacter0xe5ef', [], 'Display', None],
  [ 0xe5f0, 'UnicodeCharacter0xe5f0', [], 'Display', None],
  [ 0xe5f1, 'UnicodeCharacter0xe5f1', [], 'Display', None],
  [ 0xe5f2, 'UnicodeCharacter0xe5f2', [], 'Display', None],
  [ 0xe5f3, 'UnicodeCharacter0xe5f3', [], 'Display', None],
  [ 0xe5f4, 'UnicodeCharacter0xe5f4', [], 'Display', None],
  [ 0xe5f5, 'UnicodeCharacter0xe5f5', [], 'Display', None],
  [ 0xe5f6, 'UnicodeCharacter0xe5f6', [], 'Display', None],
  [ 0xe5f7, 'UnicodeCharacter0xe5f7', [], 'Display', None],
  [ 0xe5f8, 'UnicodeCharacter0xe5f8', [], 'Display', None],
  [ 0xe5f9, 'UnicodeCharacter0xe5f9', [], 'Display', None],
  [ 0xe5fa, 'UnicodeCharacter0xe5fa', [], 'Display', None],
  [ 0xe5fb, 'UnicodeCharacter0xe5fb', [], 'Display', None],
  [ 0xe5fc, 'UnicodeCharacter0xe5fc', [], 'Display', None],
  [ 0xe5fd, 'UnicodeCharacter0xe5fd', [], 'Display', None],
  [ 0xe5fe, 'UnicodeCharacter0xe5fe', [], 'Display', None],
  [ 0xe5ff, 'UnicodeCharacter0xe5ff', [], 'Display', None],
  [ 0xe600, 'UnicodeCharacter0xe600', [], 'Display', None],
  [ 0xe601, 'UnicodeCharacter0xe601', [], 'Display', None],
  [ 0xe602, 'UnicodeCharacter0xe602', [], 'Display', None],
  [ 0xe603, 'UnicodeCharacter0xe603', [], 'Display', None],
  [ 0xe604, 'UnicodeCharacter0xe604', [], 'Display', None],
  [ 0xe605, 'UnicodeCharacter0xe605', [], 'Display', None],
  [ 0xe606, 'UnicodeCharacter0xe606', [], 'Display', None],
  [ 0xe607, 'UnicodeCharacter0xe607', [], 'Display', None],
  [ 0xe608, 'UnicodeCharacter0xe608', [], 'Display', None],
  [ 0xe609, 'UnicodeCharacter0xe609', [], 'Display', None],
  [ 0xe60a, 'UnicodeCharacter0xe60a', [], 'Display', None],
  [ 0xe60b, 'UnicodeCharacter0xe60b', [], 'Display', None],
  [ 0xe60c, 'UnicodeCharacter0xe60c', [], 'Display', None],
  [ 0xe60d, 'UnicodeCharacter0xe60d', [], 'Display', None],
  [ 0xe60e, 'UnicodeCharacter0xe60e', [], 'Display', None],
  [ 0xe60f, 'UnicodeCharacter0xe60f', [], 'Display', None],
  [ 0xe610, 'UnicodeCharacter0xe610', [], 'Display', None],
  [ 0xe611, 'UnicodeCharacter0xe611', [], 'Display', None],
  [ 0xe612, 'UnicodeCharacter0xe612', [], 'Display', None],
  [ 0xe613, 'UnicodeCharacter0xe613', [], 'Display', None],
  [ 0xe614, 'UnicodeCharacter0xe614', [], 'Display', None],
  [ 0xe615, 'UnicodeCharacter0xe615', [], 'Display', None],
  [ 0xe616, 'UnicodeCharacter0xe616', [], 'Display', None],
  [ 0xe617, 'UnicodeCharacter0xe617', [], 'Display', None],
  [ 0xe618, 'UnicodeCharacter0xe618', [], 'Display', None],
  [ 0xe619, 'UnicodeCharacter0xe619', [], 'Display', None],
  [ 0xe61a, 'UnicodeCharacter0xe61a', [], 'Display', None],
  [ 0xe61b, 'UnicodeCharacter0xe61b', [], 'Display', None],
  [ 0xe61c, 'UnicodeCharacter0xe61c', [], 'Display', None],
  [ 0xe61d, 'UnicodeCharacter0xe61d', [], 'Display', None],
  [ 0xe61e, 'UnicodeCharacter0xe61e', [], 'Display', None],
  [ 0xe61f, 'UnicodeCharacter0xe61f', [], 'Display', None],
  [ 0xe620, 'UnicodeCharacter0xe620', [], 'Display', None],
  [ 0xe621, 'UnicodeCharacter0xe621', [], 'Display', None],
  [ 0xe622, 'UnicodeCharacter0xe622', [], 'Display', None],
  [ 0xe623, 'UnicodeCharacter0xe623', [], 'Display', None],
  [ 0xe624, 'UnicodeCharacter0xe624', [], 'Display', None],
  [ 0xe625, 'UnicodeCharacter0xe625', [], 'Display', None],
  [ 0xe626, 'UnicodeCharacter0xe626', [], 'Display', None],
  [ 0xe627, 'UnicodeCharacter0xe627', [], 'Display', None],
  [ 0xe628, 'UnicodeCharacter0xe628', [], 'Display', None],
  [ 0xe629, 'UnicodeCharacter0xe629', [], 'Display', None],
  [ 0xe62a, 'UnicodeCharacter0xe62a', [], 'Display', None],
  [ 0xe62b, 'UnicodeCharacter0xe62b', [], 'Display', None],
  [ 0xe62c, 'UnicodeCharacter0xe62c', [], 'Display', None],
  [ 0xe62d, 'UnicodeCharacter0xe62d', [], 'Display', None],
  [ 0xe62e, 'UnicodeCharacter0xe62e', [], 'Display', None],
  [ 0xe62f, 'UnicodeCharacter0xe62f', [], 'Display', None],
  [ 0xe630, 'UnicodeCharacter0xe630', [], 'Display', None],
  [ 0xe631, 'UnicodeCharacter0xe631', [], 'Display', None],
  [ 0xe632, 'UnicodeCharacter0xe632', [], 'Display', None],
  [ 0xe633, 'UnicodeCharacter0xe633', [], 'Display', None],
  [ 0xe634, 'UnicodeCharacter0xe634', [], 'Display', None],
  [ 0xe635, 'UnicodeCharacter0xe635', [], 'Display', None],
  [ 0xe636, 'UnicodeCharacter0xe636', [], 'Display', None],
  [ 0xe637, 'UnicodeCharacter0xe637', [], 'Display', None],
  [ 0xe638, 'UnicodeCharacter0xe638', [], 'Display', None],
  [ 0xe639, 'UnicodeCharacter0xe639', [], 'Display', None],
  [ 0xe63a, 'UnicodeCharacter0xe63a', [], 'Display', None],
  [ 0xe63b, 'UnicodeCharacter0xe63b', [], 'Display', None],
  [ 0xe63c, 'UnicodeCharacter0xe63c', [], 'Display', None],
  [ 0xe63d, 'UnicodeCharacter0xe63d', [], 'Display', None],
  [ 0xe63e, 'UnicodeCharacter0xe63e', [], 'Display', None],
  [ 0xe63f, 'UnicodeCharacter0xe63f', [], 'Display', None],
  [ 0xe640, 'UnicodeCharacter0xe640', [], 'Display', None],
  [ 0xe641, 'UnicodeCharacter0xe641', [], 'Display', None],
  [ 0xe642, 'UnicodeCharacter0xe642', [], 'Display', None],
  [ 0xe643, 'UnicodeCharacter0xe643', [], 'Display', None],
  [ 0xe644, 'UnicodeCharacter0xe644', [], 'Display', None],
  [ 0xe645, 'UnicodeCharacter0xe645', [], 'Display', None],
  [ 0xe646, 'UnicodeCharacter0xe646', [], 'Display', None],
  [ 0xe647, 'UnicodeCharacter0xe647', [], 'Display', None],
  [ 0xe648, 'UnicodeCharacter0xe648', [], 'Display', None],
  [ 0xe649, 'UnicodeCharacter0xe649', [], 'Display', None],
  [ 0xe64a, 'UnicodeCharacter0xe64a', [], 'Display', None],
  [ 0xe64b, 'UnicodeCharacter0xe64b', [], 'Display', None],
  [ 0xe64c, 'UnicodeCharacter0xe64c', [], 'Display', None],
  [ 0xe64d, 'UnicodeCharacter0xe64d', [], 'Display', None],
  [ 0xe64e, 'UnicodeCharacter0xe64e', [], 'Display', None],
  [ 0xe64f, 'UnicodeCharacter0xe64f', [], 'Display', None],
  [ 0xe650, 'UnicodeCharacter0xe650', [], 'Display', None],
  [ 0xe651, 'UnicodeCharacter0xe651', [], 'Display', None],
  [ 0xe652, 'UnicodeCharacter0xe652', [], 'Display', None],
  [ 0xe653, 'UnicodeCharacter0xe653', [], 'Display', None],
  [ 0xe654, 'UnicodeCharacter0xe654', [], 'Display', None],
  [ 0xe655, 'UnicodeCharacter0xe655', [], 'Display', None],
  [ 0xe656, 'UnicodeCharacter0xe656', [], 'Display', None],
  [ 0xe657, 'UnicodeCharacter0xe657', [], 'Display', None],
  [ 0xe658, 'UnicodeCharacter0xe658', [], 'Display', None],
  [ 0xe659, 'UnicodeCharacter0xe659', [], 'Display', None],
  [ 0xe65a, 'UnicodeCharacter0xe65a', [], 'Display', None],
  [ 0xe65b, 'UnicodeCharacter0xe65b', [], 'Display', None],
  [ 0xe65c, 'UnicodeCharacter0xe65c', [], 'Display', None],
  [ 0xe65d, 'UnicodeCharacter0xe65d', [], 'Display', None],
  [ 0xe65e, 'UnicodeCharacter0xe65e', [], 'Display', None],
  [ 0xe65f, 'UnicodeCharacter0xe65f', [], 'Display', None],
  [ 0xe660, 'UnicodeCharacter0xe660', [], 'Display', None],
  [ 0xe661, 'UnicodeCharacter0xe661', [], 'Display', None],
  [ 0xe662, 'UnicodeCharacter0xe662', [], 'Display', None],
  [ 0xe663, 'UnicodeCharacter0xe663', [], 'Display', None],
  [ 0xe664, 'UnicodeCharacter0xe664', [], 'Display', None],
  [ 0xe665, 'UnicodeCharacter0xe665', [], 'Display', None],
  [ 0xe666, 'UnicodeCharacter0xe666', [], 'Display', None],
  [ 0xe667, 'UnicodeCharacter0xe667', [], 'Display', None],
  [ 0xe668, 'UnicodeCharacter0xe668', [], 'Display', None],
  [ 0xe669, 'UnicodeCharacter0xe669', [], 'Display', None],
  [ 0xe66a, 'UnicodeCharacter0xe66a', [], 'Display', None],
  [ 0xe66b, 'UnicodeCharacter0xe66b', [], 'Display', None],
  [ 0xe66c, 'UnicodeCharacter0xe66c', [], 'Display', None],
  [ 0xe66d, 'UnicodeCharacter0xe66d', [], 'Display', None],
  [ 0xe66e, 'UnicodeCharacter0xe66e', [], 'Display', None],
  [ 0xe66f, 'UnicodeCharacter0xe66f', [], 'Display', None],
  [ 0xe670, 'UnicodeCharacter0xe670', [], 'Display', None],
  [ 0xe671, 'UnicodeCharacter0xe671', [], 'Display', None],
  [ 0xe672, 'UnicodeCharacter0xe672', [], 'Display', None],
  [ 0xe673, 'UnicodeCharacter0xe673', [], 'Display', None],
  [ 0xe674, 'UnicodeCharacter0xe674', [], 'Display', None],
  [ 0xe675, 'UnicodeCharacter0xe675', [], 'Display', None],
  [ 0xe676, 'UnicodeCharacter0xe676', [], 'Display', None],
  [ 0xe677, 'UnicodeCharacter0xe677', [], 'Display', None],
  [ 0xe678, 'UnicodeCharacter0xe678', [], 'Display', None],
  [ 0xe679, 'UnicodeCharacter0xe679', [], 'Display', None],
  [ 0xe67a, 'UnicodeCharacter0xe67a', [], 'Display', None],
  [ 0xe67b, 'UnicodeCharacter0xe67b', [], 'Display', None],
  [ 0xe67c, 'UnicodeCharacter0xe67c', [], 'Display', None],
  [ 0xe67d, 'UnicodeCharacter0xe67d', [], 'Display', None],
  [ 0xe67e, 'UnicodeCharacter0xe67e', [], 'Display', None],
  [ 0xe67f, 'UnicodeCharacter0xe67f', [], 'Display', None],
  [ 0xe680, 'UnicodeCharacter0xe680', [], 'Display', None],
  [ 0xe681, 'UnicodeCharacter0xe681', [], 'Display', None],
  [ 0xe682, 'UnicodeCharacter0xe682', [], 'Display', None],
  [ 0xe683, 'UnicodeCharacter0xe683', [], 'Display', None],
  [ 0xe684, 'UnicodeCharacter0xe684', [], 'Display', None],
  [ 0xe685, 'UnicodeCharacter0xe685', [], 'Display', None],
  [ 0xe686, 'UnicodeCharacter0xe686', [], 'Display', None],
  [ 0xe687, 'UnicodeCharacter0xe687', [], 'Display', None],
  [ 0xe688, 'UnicodeCharacter0xe688', [], 'Display', None],
  [ 0xe689, 'UnicodeCharacter0xe689', [], 'Display', None],
  [ 0xe68a, 'UnicodeCharacter0xe68a', [], 'Display', None],
  [ 0xe68b, 'UnicodeCharacter0xe68b', [], 'Display', None],
  [ 0xe68c, 'UnicodeCharacter0xe68c', [], 'Display', None],
  [ 0xe68d, 'UnicodeCharacter0xe68d', [], 'Display', None],
  [ 0xe68e, 'UnicodeCharacter0xe68e', [], 'Display', None],
  [ 0xe68f, 'UnicodeCharacter0xe68f', [], 'Display', None],
  [ 0xe690, 'UnicodeCharacter0xe690', [], 'Display', None],
  [ 0xe691, 'UnicodeCharacter0xe691', [], 'Display', None],
  [ 0xe692, 'UnicodeCharacter0xe692', [], 'Display', None],
  [ 0xe693, 'UnicodeCharacter0xe693', [], 'Display', None],
  [ 0xe694, 'UnicodeCharacter0xe694', [], 'Display', None],
  [ 0xe695, 'UnicodeCharacter0xe695', [], 'Display', None],
  [ 0xe696, 'UnicodeCharacter0xe696', [], 'Display', None],
  [ 0xe697, 'UnicodeCharacter0xe697', [], 'Display', None],
  [ 0xe698, 'UnicodeCharacter0xe698', [], 'Display', None],
  [ 0xe699, 'UnicodeCharacter0xe699', [], 'Display', None],
  [ 0xe69a, 'UnicodeCharacter0xe69a', [], 'Display', None],
  [ 0xe69b, 'UnicodeCharacter0xe69b', [], 'Display', None],
  [ 0xe69c, 'UnicodeCharacter0xe69c', [], 'Display', None],
  [ 0xe69d, 'UnicodeCharacter0xe69d', [], 'Display', None],
  [ 0xe69e, 'UnicodeCharacter0xe69e', [], 'Display', None],
  [ 0xe69f, 'UnicodeCharacter0xe69f', [], 'Display', None],
  [ 0xe6a0, 'UnicodeCharacter0xe6a0', [], 'Display', None],
  [ 0xe6a1, 'UnicodeCharacter0xe6a1', [], 'Display', None],
  [ 0xe6a2, 'UnicodeCharacter0xe6a2', [], 'Display', None],
  [ 0xe6a3, 'UnicodeCharacter0xe6a3', [], 'Display', None],
  [ 0xe6a4, 'UnicodeCharacter0xe6a4', [], 'Display', None],
  [ 0xe6a5, 'UnicodeCharacter0xe6a5', [], 'Display', None],
  [ 0xe6a6, 'UnicodeCharacter0xe6a6', [], 'Display', None],
  [ 0xe6a7, 'UnicodeCharacter0xe6a7', [], 'Display', None],
  [ 0xe6a8, 'UnicodeCharacter0xe6a8', [], 'Display', None],
  [ 0xe6a9, 'UnicodeCharacter0xe6a9', [], 'Display', None],
  [ 0xe6aa, 'UnicodeCharacter0xe6aa', [], 'Display', None],
  [ 0xe6ab, 'UnicodeCharacter0xe6ab', [], 'Display', None],
  [ 0xe6ac, 'UnicodeCharacter0xe6ac', [], 'Display', None],
  [ 0xe6ad, 'UnicodeCharacter0xe6ad', [], 'Display', None],
  [ 0xe6ae, 'UnicodeCharacter0xe6ae', [], 'Display', None],
  [ 0xe6af, 'UnicodeCharacter0xe6af', [], 'Display', None],
  [ 0xe6b0, 'UnicodeCharacter0xe6b0', [], 'Display', None],
  [ 0xe6b1, 'UnicodeCharacter0xe6b1', [], 'Display', None],
  [ 0xe6b2, 'UnicodeCharacter0xe6b2', [], 'Display', None],
  [ 0xe6b3, 'UnicodeCharacter0xe6b3', [], 'Display', None],
  [ 0xe6b4, 'UnicodeCharacter0xe6b4', [], 'Display', None],
  [ 0xe6b5, 'UnicodeCharacter0xe6b5', [], 'Display', None],
  [ 0xe6b6, 'UnicodeCharacter0xe6b6', [], 'Display', None],
  [ 0xe6b7, 'UnicodeCharacter0xe6b7', [], 'Display', None],
  [ 0xe6b8, 'UnicodeCharacter0xe6b8', [], 'Display', None],
  [ 0xe6b9, 'UnicodeCharacter0xe6b9', [], 'Display', None],
  [ 0xe6ba, 'UnicodeCharacter0xe6ba', [], 'Display', None],
  [ 0xe6bb, 'UnicodeCharacter0xe6bb', [], 'Display', None],
  [ 0xe6bc, 'UnicodeCharacter0xe6bc', [], 'Display', None],
  [ 0xe6bd, 'UnicodeCharacter0xe6bd', [], 'Display', None],
  [ 0xe6be, 'UnicodeCharacter0xe6be', [], 'Display', None],
  [ 0xe6bf, 'UnicodeCharacter0xe6bf', [], 'Display', None],
  [ 0xe6c0, 'UnicodeCharacter0xe6c0', [], 'Display', None],
  [ 0xe6c1, 'UnicodeCharacter0xe6c1', [], 'Display', None],
  [ 0xe6c2, 'UnicodeCharacter0xe6c2', [], 'Display', None],
  [ 0xe6c3, 'UnicodeCharacter0xe6c3', [], 'Display', None],
  [ 0xe6c4, 'UnicodeCharacter0xe6c4', [], 'Display', None],
  [ 0xe6c5, 'UnicodeCharacter0xe6c5', [], 'Display', None],
  [ 0xe6c6, 'UnicodeCharacter0xe6c6', [], 'Display', None],
  [ 0xe6c7, 'UnicodeCharacter0xe6c7', [], 'Display', None],
  [ 0xe6c8, 'UnicodeCharacter0xe6c8', [], 'Display', None],
  [ 0xe6c9, 'UnicodeCharacter0xe6c9', [], 'Display', None],
  [ 0xe6ca, 'UnicodeCharacter0xe6ca', [], 'Display', None],
  [ 0xe6cb, 'UnicodeCharacter0xe6cb', [], 'Display', None],
  [ 0xe6cc, 'UnicodeCharacter0xe6cc', [], 'Display', None],
  [ 0xe6cd, 'UnicodeCharacter0xe6cd', [], 'Display', None],
  [ 0xe6ce, 'UnicodeCharacter0xe6ce', [], 'Display', None],
  [ 0xe6cf, 'UnicodeCharacter0xe6cf', [], 'Display', None],
  [ 0xe6d0, 'UnicodeCharacter0xe6d0', [], 'Display', None],
  [ 0xe6d1, 'UnicodeCharacter0xe6d1', [], 'Display', None],
  [ 0xe6d2, 'UnicodeCharacter0xe6d2', [], 'Display', None],
  [ 0xe6d3, 'UnicodeCharacter0xe6d3', [], 'Display', None],
  [ 0xe6d4, 'UnicodeCharacter0xe6d4', [], 'Display', None],
  [ 0xe6d5, 'UnicodeCharacter0xe6d5', [], 'Display', None],
  [ 0xe6d6, 'UnicodeCharacter0xe6d6', [], 'Display', None],
  [ 0xe6d7, 'UnicodeCharacter0xe6d7', [], 'Display', None],
  [ 0xe6d8, 'UnicodeCharacter0xe6d8', [], 'Display', None],
  [ 0xe6d9, 'UnicodeCharacter0xe6d9', [], 'Display', None],
  [ 0xe6da, 'UnicodeCharacter0xe6da', [], 'Display', None],
  [ 0xe6db, 'UnicodeCharacter0xe6db', [], 'Display', None],
  [ 0xe6dc, 'UnicodeCharacter0xe6dc', [], 'Display', None],
  [ 0xe6dd, 'UnicodeCharacter0xe6dd', [], 'Display', None],
  [ 0xe6de, 'UnicodeCharacter0xe6de', [], 'Display', None],
  [ 0xe6df, 'UnicodeCharacter0xe6df', [], 'Display', None],
  [ 0xe6e0, 'UnicodeCharacter0xe6e0', [], 'Display', None],
  [ 0xe6e1, 'UnicodeCharacter0xe6e1', [], 'Display', None],
  [ 0xe6e2, 'UnicodeCharacter0xe6e2', [], 'Display', None],
  [ 0xe6e3, 'UnicodeCharacter0xe6e3', [], 'Display', None],
  [ 0xe6e4, 'UnicodeCharacter0xe6e4', [], 'Display', None],
  [ 0xe6e5, 'UnicodeCharacter0xe6e5', [], 'Display', None],
  [ 0xe6e6, 'UnicodeCharacter0xe6e6', [], 'Display', None],
  [ 0xe6e7, 'UnicodeCharacter0xe6e7', [], 'Display', None],
  [ 0xe6e8, 'UnicodeCharacter0xe6e8', [], 'Display', None],
  [ 0xe6e9, 'UnicodeCharacter0xe6e9', [], 'Display', None],
  [ 0xe6ea, 'UnicodeCharacter0xe6ea', [], 'Display', None],
  [ 0xe6eb, 'UnicodeCharacter0xe6eb', [], 'Display', None],
  [ 0xe6ec, 'UnicodeCharacter0xe6ec', [], 'Display', None],
  [ 0xe6ed, 'UnicodeCharacter0xe6ed', [], 'Display', None],
  [ 0xe6ee, 'UnicodeCharacter0xe6ee', [], 'Display', None],
  [ 0xe6ef, 'UnicodeCharacter0xe6ef', [], 'Display', None],
  [ 0xe6f0, 'UnicodeCharacter0xe6f0', [], 'Display', None],
  [ 0xe6f1, 'UnicodeCharacter0xe6f1', [], 'Display', None],
  [ 0xe6f2, 'UnicodeCharacter0xe6f2', [], 'Display', None],
  [ 0xe6f3, 'UnicodeCharacter0xe6f3', [], 'Display', None],
  [ 0xe6f4, 'UnicodeCharacter0xe6f4', [], 'Display', None],
  [ 0xe6f5, 'UnicodeCharacter0xe6f5', [], 'Display', None],
  [ 0xe6f6, 'UnicodeCharacter0xe6f6', [], 'Display', None],
  [ 0xe6f7, 'UnicodeCharacter0xe6f7', [], 'Display', None],
  [ 0xe6f8, 'UnicodeCharacter0xe6f8', [], 'Display', None],
  [ 0xe6f9, 'UnicodeCharacter0xe6f9', [], 'Display', None],
  [ 0xe6fa, 'UnicodeCharacter0xe6fa', [], 'Display', None],
  [ 0xe6fb, 'UnicodeCharacter0xe6fb', [], 'Display', None],
  [ 0xe6fc, 'UnicodeCharacter0xe6fc', [], 'Display', None],
  [ 0xe6fd, 'UnicodeCharacter0xe6fd', [], 'Display', None],
  [ 0xe6fe, 'UnicodeCharacter0xe6fe', [], 'Display', None],
  [ 0xe6ff, 'UnicodeCharacter0xe6ff', [], 'Display', None],
  [ 0xe700, 'UnicodeCharacter0xe700', [], 'Display', None],
  [ 0xe701, 'UnicodeCharacter0xe701', [], 'Display', None],
  [ 0xe702, 'UnicodeCharacter0xe702', [], 'Display', None],
  [ 0xe703, 'UnicodeCharacter0xe703', [], 'Display', None],
  [ 0xe704, 'UnicodeCharacter0xe704', [], 'Display', None],
  [ 0xe705, 'UnicodeCharacter0xe705', [], 'Display', None],
  [ 0xe706, 'UnicodeCharacter0xe706', [], 'Display', None],
  [ 0xe707, 'UnicodeCharacter0xe707', [], 'Display', None],
  [ 0xe708, 'UnicodeCharacter0xe708', [], 'Display', None],
  [ 0xe709, 'UnicodeCharacter0xe709', [], 'Display', None],
  [ 0xe70a, 'UnicodeCharacter0xe70a', [], 'Display', None],
  [ 0xe70b, 'UnicodeCharacter0xe70b', [], 'Display', None],
  [ 0xe70c, 'UnicodeCharacter0xe70c', [], 'Display', None],
  [ 0xe70d, 'UnicodeCharacter0xe70d', [], 'Display', None],
  [ 0xe70e, 'UnicodeCharacter0xe70e', [], 'Display', None],
  [ 0xe70f, 'UnicodeCharacter0xe70f', [], 'Display', None],
  [ 0xe710, 'UnicodeCharacter0xe710', [], 'Display', None],
  [ 0xe711, 'UnicodeCharacter0xe711', [], 'Display', None],
  [ 0xe712, 'UnicodeCharacter0xe712', [], 'Display', None],
  [ 0xe713, 'UnicodeCharacter0xe713', [], 'Display', None],
  [ 0xe714, 'UnicodeCharacter0xe714', [], 'Display', None],
  [ 0xe715, 'UnicodeCharacter0xe715', [], 'Display', None],
  [ 0xe716, 'UnicodeCharacter0xe716', [], 'Display', None],
  [ 0xe717, 'UnicodeCharacter0xe717', [], 'Display', None],
  [ 0xe718, 'UnicodeCharacter0xe718', [], 'Display', None],
  [ 0xe719, 'UnicodeCharacter0xe719', [], 'Display', None],
  [ 0xe71a, 'UnicodeCharacter0xe71a', [], 'Display', None],
  [ 0xe71b, 'UnicodeCharacter0xe71b', [], 'Display', None],
  [ 0xe71c, 'UnicodeCharacter0xe71c', [], 'Display', None],
  [ 0xe71d, 'UnicodeCharacter0xe71d', [], 'Display', None],
  [ 0xe71e, 'UnicodeCharacter0xe71e', [], 'Display', None],
  [ 0xe71f, 'UnicodeCharacter0xe71f', [], 'Display', None],
  [ 0xe720, 'UnicodeCharacter0xe720', [], 'Display', None],
  [ 0xe721, 'UnicodeCharacter0xe721', [], 'Display', None],
  [ 0xe722, 'UnicodeCharacter0xe722', [], 'Display', None],
  [ 0xe723, 'UnicodeCharacter0xe723', [], 'Display', None],
  [ 0xe724, 'UnicodeCharacter0xe724', [], 'Display', None],
  [ 0xe725, 'UnicodeCharacter0xe725', [], 'Display', None],
  [ 0xe726, 'UnicodeCharacter0xe726', [], 'Display', None],
  [ 0xe727, 'UnicodeCharacter0xe727', [], 'Display', None],
  [ 0xe728, 'UnicodeCharacter0xe728', [], 'Display', None],
  [ 0xe729, 'UnicodeCharacter0xe729', [], 'Display', None],
  [ 0xe72a, 'UnicodeCharacter0xe72a', [], 'Display', None],
  [ 0xe72b, 'UnicodeCharacter0xe72b', [], 'Display', None],
  [ 0xe72c, 'UnicodeCharacter0xe72c', [], 'Display', None],
  [ 0xe72d, 'UnicodeCharacter0xe72d', [], 'Display', None],
  [ 0xe72e, 'UnicodeCharacter0xe72e', [], 'Display', None],
  [ 0xe72f, 'UnicodeCharacter0xe72f', [], 'Display', None],
  [ 0xe730, 'UnicodeCharacter0xe730', [], 'Display', None],
  [ 0xe731, 'UnicodeCharacter0xe731', [], 'Display', None],
  [ 0xe732, 'UnicodeCharacter0xe732', [], 'Display', None],
  [ 0xe733, 'UnicodeCharacter0xe733', [], 'Display', None],
  [ 0xe734, 'UnicodeCharacter0xe734', [], 'Display', None],
  [ 0xe735, 'UnicodeCharacter0xe735', [], 'Display', None],
  [ 0xe736, 'UnicodeCharacter0xe736', [], 'Display', None],
  [ 0xe737, 'UnicodeCharacter0xe737', [], 'Display', None],
  [ 0xe738, 'UnicodeCharacter0xe738', [], 'Display', None],
  [ 0xe739, 'UnicodeCharacter0xe739', [], 'Display', None],
  [ 0xe73a, 'UnicodeCharacter0xe73a', [], 'Display', None],
  [ 0xe73b, 'UnicodeCharacter0xe73b', [], 'Display', None],
  [ 0xe73c, 'UnicodeCharacter0xe73c', [], 'Display', None],
  [ 0xe73d, 'UnicodeCharacter0xe73d', [], 'Display', None],
  [ 0xe73e, 'UnicodeCharacter0xe73e', [], 'Display', None],
  [ 0xe73f, 'UnicodeCharacter0xe73f', [], 'Display', None],
  [ 0xe740, 'UnicodeCharacter0xe740', [], 'Display', None],
  [ 0xe741, 'UnicodeCharacter0xe741', [], 'Display', None],
  [ 0xe742, 'UnicodeCharacter0xe742', [], 'Display', None],
  [ 0xe743, 'UnicodeCharacter0xe743', [], 'Display', None],
  [ 0xe744, 'UnicodeCharacter0xe744', [], 'Display', None],
  [ 0xe745, 'UnicodeCharacter0xe745', [], 'Display', None],
  [ 0xe746, 'UnicodeCharacter0xe746', [], 'Display', None],
  [ 0xe747, 'UnicodeCharacter0xe747', [], 'Display', None],
  [ 0xe748, 'UnicodeCharacter0xe748', [], 'Display', None],
  [ 0xe749, 'UnicodeCharacter0xe749', [], 'Display', None],
  [ 0xe74a, 'UnicodeCharacter0xe74a', [], 'Display', None],
  [ 0xe74b, 'UnicodeCharacter0xe74b', [], 'Display', None],
  [ 0xe74c, 'UnicodeCharacter0xe74c', [], 'Display', None],
  [ 0xe74d, 'UnicodeCharacter0xe74d', [], 'Display', None],
  [ 0xe74e, 'UnicodeCharacter0xe74e', [], 'Display', None],
  [ 0xe74f, 'UnicodeCharacter0xe74f', [], 'Display', None],
  [ 0xe750, 'UnicodeCharacter0xe750', [], 'Display', None],
  [ 0xe751, 'UnicodeCharacter0xe751', [], 'Display', None],
  [ 0xe752, 'UnicodeCharacter0xe752', [], 'Display', None],
  [ 0xe753, 'UnicodeCharacter0xe753', [], 'Display', None],
  [ 0xe754, 'UnicodeCharacter0xe754', [], 'Display', None],
  [ 0xe755, 'UnicodeCharacter0xe755', [], 'Display', None],
  [ 0xe756, 'UnicodeCharacter0xe756', [], 'Display', None],
  [ 0xe757, 'UnicodeCharacter0xe757', [], 'Display', None],
  [ 0xe758, 'UnicodeCharacter0xe758', [], 'Display', None],
  [ 0xe759, 'UnicodeCharacter0xe759', [], 'Display', None],
  [ 0xe75a, 'UnicodeCharacter0xe75a', [], 'Display', None],
  [ 0xe75b, 'UnicodeCharacter0xe75b', [], 'Display', None],
  [ 0xe75c, 'UnicodeCharacter0xe75c', [], 'Display', None],
  [ 0xe75d, 'UnicodeCharacter0xe75d', [], 'Display', None],
  [ 0xe75e, 'UnicodeCharacter0xe75e', [], 'Display', None],
  [ 0xe75f, 'UnicodeCharacter0xe75f', [], 'Display', None],
  [ 0xe760, 'UnicodeCharacter0xe760', [], 'Display', None],
  [ 0xe761, 'UnicodeCharacter0xe761', [], 'Display', None],
  [ 0xe762, 'UnicodeCharacter0xe762', [], 'Display', None],
  [ 0xe763, 'UnicodeCharacter0xe763', [], 'Display', None],
  [ 0xe764, 'UnicodeCharacter0xe764', [], 'Display', None],
  [ 0xe765, 'UnicodeCharacter0xe765', [], 'Display', None],
  [ 0xe766, 'UnicodeCharacter0xe766', [], 'Display', None],
  [ 0xe767, 'UnicodeCharacter0xe767', [], 'Display', None],
  [ 0xe768, 'UnicodeCharacter0xe768', [], 'Display', None],
  [ 0xe769, 'UnicodeCharacter0xe769', [], 'Display', None],
  [ 0xe76a, 'UnicodeCharacter0xe76a', [], 'Display', None],
  [ 0xe76b, 'UnicodeCharacter0xe76b', [], 'Display', None],
  [ 0xe76c, 'UnicodeCharacter0xe76c', [], 'Display', None],
  [ 0xe76d, 'UnicodeCharacter0xe76d', [], 'Display', None],
  [ 0xe76e, 'UnicodeCharacter0xe76e', [], 'Display', None],
  [ 0xe76f, 'UnicodeCharacter0xe76f', [], 'Display', None],
  [ 0xe770, 'UnicodeCharacter0xe770', [], 'Display', None],
  [ 0xe771, 'UnicodeCharacter0xe771', [], 'Display', None],
  [ 0xe772, 'UnicodeCharacter0xe772', [], 'Display', None],
  [ 0xe773, 'UnicodeCharacter0xe773', [], 'Display', None],
  [ 0xe774, 'UnicodeCharacter0xe774', [], 'Display', None],
  [ 0xe775, 'UnicodeCharacter0xe775', [], 'Display', None],
  [ 0xe776, 'UnicodeCharacter0xe776', [], 'Display', None],
  [ 0xe777, 'UnicodeCharacter0xe777', [], 'Display', None],
  [ 0xe778, 'UnicodeCharacter0xe778', [], 'Display', None],
  [ 0xe779, 'UnicodeCharacter0xe779', [], 'Display', None],
  [ 0xe77a, 'UnicodeCharacter0xe77a', [], 'Display', None],
  [ 0xe77b, 'UnicodeCharacter0xe77b', [], 'Display', None],
  [ 0xe77c, 'UnicodeCharacter0xe77c', [], 'Display', None],
  [ 0xe77d, 'UnicodeCharacter0xe77d', [], 'Display', None],
  [ 0xe77e, 'UnicodeCharacter0xe77e', [], 'Display', None],
  [ 0xe77f, 'UnicodeCharacter0xe77f', [], 'Display', None],
  [ 0xe780, 'UnicodeCharacter0xe780', [], 'Display', None],
  [ 0xe781, 'UnicodeCharacter0xe781', [], 'Display', None],
  [ 0xe782, 'UnicodeCharacter0xe782', [], 'Display', None],
  [ 0xe783, 'UnicodeCharacter0xe783', [], 'Display', None],
  [ 0xe784, 'UnicodeCharacter0xe784', [], 'Display', None],
  [ 0xe785, 'UnicodeCharacter0xe785', [], 'Display', None],
  [ 0xe786, 'UnicodeCharacter0xe786', [], 'Display', None],
  [ 0xe787, 'UnicodeCharacter0xe787', [], 'Display', None],
  [ 0xe788, 'UnicodeCharacter0xe788', [], 'Display', None],
  [ 0xe789, 'UnicodeCharacter0xe789', [], 'Display', None],
  [ 0xe78a, 'UnicodeCharacter0xe78a', [], 'Display', None],
  [ 0xe78b, 'UnicodeCharacter0xe78b', [], 'Display', None],
  [ 0xe78c, 'UnicodeCharacter0xe78c', [], 'Display', None],
  [ 0xe78d, 'UnicodeCharacter0xe78d', [], 'Display', None],
  [ 0xe78e, 'UnicodeCharacter0xe78e', [], 'Display', None],
  [ 0xe78f, 'UnicodeCharacter0xe78f', [], 'Display', None],
  [ 0xe790, 'UnicodeCharacter0xe790', [], 'Display', None],
  [ 0xe791, 'UnicodeCharacter0xe791', [], 'Display', None],
  [ 0xe792, 'UnicodeCharacter0xe792', [], 'Display', None],
  [ 0xe793, 'UnicodeCharacter0xe793', [], 'Display', None],
  [ 0xe794, 'UnicodeCharacter0xe794', [], 'Display', None],
  [ 0xe795, 'UnicodeCharacter0xe795', [], 'Display', None],
  [ 0xe796, 'UnicodeCharacter0xe796', [], 'Display', None],
  [ 0xe797, 'UnicodeCharacter0xe797', [], 'Display', None],
  [ 0xe798, 'UnicodeCharacter0xe798', [], 'Display', None],
  [ 0xe799, 'UnicodeCharacter0xe799', [], 'Display', None],
  [ 0xe79a, 'UnicodeCharacter0xe79a', [], 'Display', None],
  [ 0xe79b, 'UnicodeCharacter0xe79b', [], 'Display', None],
  [ 0xe79c, 'UnicodeCharacter0xe79c', [], 'Display', None],
  [ 0xe79d, 'UnicodeCharacter0xe79d', [], 'Display', None],
  [ 0xe79e, 'UnicodeCharacter0xe79e', [], 'Display', None],
  [ 0xe79f, 'UnicodeCharacter0xe79f', [], 'Display', None],
  [ 0xe7a0, 'UnicodeCharacter0xe7a0', [], 'Display', None],
  [ 0xe7a1, 'UnicodeCharacter0xe7a1', [], 'Display', None],
  [ 0xe7a2, 'UnicodeCharacter0xe7a2', [], 'Display', None],
  [ 0xe7a3, 'UnicodeCharacter0xe7a3', [], 'Display', None],
  [ 0xe7a4, 'UnicodeCharacter0xe7a4', [], 'Display', None],
  [ 0xe7a5, 'UnicodeCharacter0xe7a5', [], 'Display', None],
  [ 0xe7a6, 'UnicodeCharacter0xe7a6', [], 'Display', None],
  [ 0xe7a7, 'UnicodeCharacter0xe7a7', [], 'Display', None],
  [ 0xe7a8, 'UnicodeCharacter0xe7a8', [], 'Display', None],
  [ 0xe7a9, 'UnicodeCharacter0xe7a9', [], 'Display', None],
  [ 0xe7aa, 'UnicodeCharacter0xe7aa', [], 'Display', None],
  [ 0xe7ab, 'UnicodeCharacter0xe7ab', [], 'Display', None],
  [ 0xe7ac, 'UnicodeCharacter0xe7ac', [], 'Display', None],
  [ 0xe7ad, 'UnicodeCharacter0xe7ad', [], 'Display', None],
  [ 0xe7ae, 'UnicodeCharacter0xe7ae', [], 'Display', None],
  [ 0xe7af, 'UnicodeCharacter0xe7af', [], 'Display', None],
  [ 0xe7b0, 'UnicodeCharacter0xe7b0', [], 'Display', None],
  [ 0xe7b1, 'UnicodeCharacter0xe7b1', [], 'Display', None],
  [ 0xe7b2, 'UnicodeCharacter0xe7b2', [], 'Display', None],
  [ 0xe7b3, 'UnicodeCharacter0xe7b3', [], 'Display', None],
  [ 0xe7b4, 'UnicodeCharacter0xe7b4', [], 'Display', None],
  [ 0xe7b5, 'UnicodeCharacter0xe7b5', [], 'Display', None],
  [ 0xe7b6, 'UnicodeCharacter0xe7b6', [], 'Display', None],
  [ 0xe7b7, 'UnicodeCharacter0xe7b7', [], 'Display', None],
  [ 0xe7b8, 'UnicodeCharacter0xe7b8', [], 'Display', None],
  [ 0xe7b9, 'UnicodeCharacter0xe7b9', [], 'Display', None],
  [ 0xe7ba, 'UnicodeCharacter0xe7ba', [], 'Display', None],
  [ 0xe7bb, 'UnicodeCharacter0xe7bb', [], 'Display', None],
  [ 0xe7bc, 'UnicodeCharacter0xe7bc', [], 'Display', None],
  [ 0xe7bd, 'UnicodeCharacter0xe7bd', [], 'Display', None],
  [ 0xe7be, 'UnicodeCharacter0xe7be', [], 'Display', None],
  [ 0xe7bf, 'UnicodeCharacter0xe7bf', [], 'Display', None],
  [ 0xe7c0, 'UnicodeCharacter0xe7c0', [], 'Display', None],
  [ 0xe7c1, 'UnicodeCharacter0xe7c1', [], 'Display', None],
  [ 0xe7c2, 'UnicodeCharacter0xe7c2', [], 'Display', None],
  [ 0xe7c3, 'UnicodeCharacter0xe7c3', [], 'Display', None],
  [ 0xe7c4, 'UnicodeCharacter0xe7c4', [], 'Display', None],
  [ 0xe7c5, 'UnicodeCharacter0xe7c5', [], 'Display', None],
  [ 0xe7c6, 'UnicodeCharacter0xe7c6', [], 'Display', None],
  [ 0xe7c7, 'UnicodeCharacter0xe7c7', [], 'Display', None],
  [ 0xe7c8, 'UnicodeCharacter0xe7c8', [], 'Display', None],
  [ 0xe7c9, 'UnicodeCharacter0xe7c9', [], 'Display', None],
  [ 0xe7ca, 'UnicodeCharacter0xe7ca', [], 'Display', None],
  [ 0xe7cb, 'UnicodeCharacter0xe7cb', [], 'Display', None],
  [ 0xe7cc, 'UnicodeCharacter0xe7cc', [], 'Display', None],
  [ 0xe7cd, 'UnicodeCharacter0xe7cd', [], 'Display', None],
  [ 0xe7ce, 'UnicodeCharacter0xe7ce', [], 'Display', None],
  [ 0xe7cf, 'UnicodeCharacter0xe7cf', [], 'Display', None],
  [ 0xe7d0, 'UnicodeCharacter0xe7d0', [], 'Display', None],
  [ 0xe7d1, 'UnicodeCharacter0xe7d1', [], 'Display', None],
  [ 0xe7d2, 'UnicodeCharacter0xe7d2', [], 'Display', None],
  [ 0xe7d3, 'UnicodeCharacter0xe7d3', [], 'Display', None],
  [ 0xe7d4, 'UnicodeCharacter0xe7d4', [], 'Display', None],
  [ 0xe7d5, 'UnicodeCharacter0xe7d5', [], 'Display', None],
  [ 0xe7d6, 'UnicodeCharacter0xe7d6', [], 'Display', None],
  [ 0xe7d7, 'UnicodeCharacter0xe7d7', [], 'Display', None],
  [ 0xe7d8, 'UnicodeCharacter0xe7d8', [], 'Display', None],
  [ 0xe7d9, 'UnicodeCharacter0xe7d9', [], 'Display', None],
  [ 0xe7da, 'UnicodeCharacter0xe7da', [], 'Display', None],
  [ 0xe7db, 'UnicodeCharacter0xe7db', [], 'Display', None],
  [ 0xe7dc, 'UnicodeCharacter0xe7dc', [], 'Display', None],
  [ 0xe7dd, 'UnicodeCharacter0xe7dd', [], 'Display', None],
  [ 0xe7de, 'UnicodeCharacter0xe7de', [], 'Display', None],
  [ 0xe7df, 'UnicodeCharacter0xe7df', [], 'Display', None],
  [ 0xe7e0, 'UnicodeCharacter0xe7e0', [], 'Display', None],
  [ 0xe7e1, 'UnicodeCharacter0xe7e1', [], 'Display', None],
  [ 0xe7e2, 'UnicodeCharacter0xe7e2', [], 'Display', None],
  [ 0xe7e3, 'UnicodeCharacter0xe7e3', [], 'Display', None],
  [ 0xe7e4, 'UnicodeCharacter0xe7e4', [], 'Display', None],
  [ 0xe7e5, 'UnicodeCharacter0xe7e5', [], 'Display', None],
  [ 0xe7e6, 'UnicodeCharacter0xe7e6', [], 'Display', None],
  [ 0xe7e7, 'UnicodeCharacter0xe7e7', [], 'Display', None],
  [ 0xe7e8, 'UnicodeCharacter0xe7e8', [], 'Display', None],
  [ 0xe7e9, 'UnicodeCharacter0xe7e9', [], 'Display', None],
  [ 0xe7ea, 'UnicodeCharacter0xe7ea', [], 'Display', None],
  [ 0xe7eb, 'UnicodeCharacter0xe7eb', [], 'Display', None],
  [ 0xe7ec, 'UnicodeCharacter0xe7ec', [], 'Display', None],
  [ 0xe7ed, 'UnicodeCharacter0xe7ed', [], 'Display', None],
  [ 0xe7ee, 'UnicodeCharacter0xe7ee', [], 'Display', None],
  [ 0xe7ef, 'UnicodeCharacter0xe7ef', [], 'Display', None],
  [ 0xe7f0, 'UnicodeCharacter0xe7f0', [], 'Display', None],
  [ 0xe7f1, 'UnicodeCharacter0xe7f1', [], 'Display', None],
  [ 0xe7f2, 'UnicodeCharacter0xe7f2', [], 'Display', None],
  [ 0xe7f3, 'UnicodeCharacter0xe7f3', [], 'Display', None],
  [ 0xe7f4, 'UnicodeCharacter0xe7f4', [], 'Display', None],
  [ 0xe7f5, 'UnicodeCharacter0xe7f5', [], 'Display', None],
  [ 0xe7f6, 'UnicodeCharacter0xe7f6', [], 'Display', None],
  [ 0xe7f7, 'UnicodeCharacter0xe7f7', [], 'Display', None],
  [ 0xe7f8, 'UnicodeCharacter0xe7f8', [], 'Display', None],
  [ 0xe7f9, 'UnicodeCharacter0xe7f9', [], 'Display', None],
  [ 0xe7fa, 'UnicodeCharacter0xe7fa', [], 'Display', None],
  [ 0xe7fb, 'UnicodeCharacter0xe7fb', [], 'Display', None],
  [ 0xe7fc, 'UnicodeCharacter0xe7fc', [], 'Display', None],
  [ 0xe7fd, 'UnicodeCharacter0xe7fd', [], 'Display', None],
  [ 0xe7fe, 'UnicodeCharacter0xe7fe', [], 'Display', None],
  [ 0xe7ff, 'UnicodeCharacter0xe7ff', [], 'Display', None],
  [ 0xe800, 'UnicodeCharacter0xe800', [], 'Display', None],
  [ 0xe801, 'UnicodeCharacter0xe801', [], 'Display', None],
  [ 0xe802, 'UnicodeCharacter0xe802', [], 'Display', None],
  [ 0xe803, 'UnicodeCharacter0xe803', [], 'Display', None],
  [ 0xe804, 'UnicodeCharacter0xe804', [], 'Display', None],
  [ 0xe805, 'UnicodeCharacter0xe805', [], 'Display', None],
  [ 0xe806, 'UnicodeCharacter0xe806', [], 'Display', None],
  [ 0xe807, 'UnicodeCharacter0xe807', [], 'Display', None],
  [ 0xe808, 'UnicodeCharacter0xe808', [], 'Display', None],
  [ 0xe809, 'UnicodeCharacter0xe809', [], 'Display', None],
  [ 0xe80a, 'UnicodeCharacter0xe80a', [], 'Display', None],
  [ 0xe80b, 'UnicodeCharacter0xe80b', [], 'Display', None],
  [ 0xe80c, 'UnicodeCharacter0xe80c', [], 'Display', None],
  [ 0xe80d, 'UnicodeCharacter0xe80d', [], 'Display', None],
  [ 0xe80e, 'UnicodeCharacter0xe80e', [], 'Display', None],
  [ 0xe80f, 'UnicodeCharacter0xe80f', [], 'Display', None],
  [ 0xe810, 'UnicodeCharacter0xe810', [], 'Display', None],
  [ 0xe811, 'UnicodeCharacter0xe811', [], 'Display', None],
  [ 0xe812, 'UnicodeCharacter0xe812', [], 'Display', None],
  [ 0xe813, 'UnicodeCharacter0xe813', [], 'Display', None],
  [ 0xe814, 'UnicodeCharacter0xe814', [], 'Display', None],
  [ 0xe815, 'UnicodeCharacter0xe815', [], 'Display', None],
  [ 0xe816, 'UnicodeCharacter0xe816', [], 'Display', None],
  [ 0xe817, 'UnicodeCharacter0xe817', [], 'Display', None],
  [ 0xe818, 'UnicodeCharacter0xe818', [], 'Display', None],
  [ 0xe819, 'UnicodeCharacter0xe819', [], 'Display', None],
  [ 0xe81a, 'UnicodeCharacter0xe81a', [], 'Display', None],
  [ 0xe81b, 'UnicodeCharacter0xe81b', [], 'Display', None],
  [ 0xe81c, 'UnicodeCharacter0xe81c', [], 'Display', None],
  [ 0xe81d, 'UnicodeCharacter0xe81d', [], 'Display', None],
  [ 0xe81e, 'UnicodeCharacter0xe81e', [], 'Display', None],
  [ 0xe81f, 'UnicodeCharacter0xe81f', [], 'Display', None],
  [ 0xe820, 'UnicodeCharacter0xe820', [], 'Display', None],
  [ 0xe821, 'UnicodeCharacter0xe821', [], 'Display', None],
  [ 0xe822, 'UnicodeCharacter0xe822', [], 'Display', None],
  [ 0xe823, 'UnicodeCharacter0xe823', [], 'Display', None],
  [ 0xe824, 'UnicodeCharacter0xe824', [], 'Display', None],
  [ 0xe825, 'UnicodeCharacter0xe825', [], 'Display', None],
  [ 0xe826, 'UnicodeCharacter0xe826', [], 'Display', None],
  [ 0xe827, 'UnicodeCharacter0xe827', [], 'Display', None],
  [ 0xe828, 'UnicodeCharacter0xe828', [], 'Display', None],
  [ 0xe829, 'UnicodeCharacter0xe829', [], 'Display', None],
  [ 0xe82a, 'UnicodeCharacter0xe82a', [], 'Display', None],
  [ 0xe82b, 'UnicodeCharacter0xe82b', [], 'Display', None],
  [ 0xe82c, 'UnicodeCharacter0xe82c', [], 'Display', None],
  [ 0xe82d, 'UnicodeCharacter0xe82d', [], 'Display', None],
  [ 0xe82e, 'UnicodeCharacter0xe82e', [], 'Display', None],
  [ 0xe82f, 'UnicodeCharacter0xe82f', [], 'Display', None],
  [ 0xe830, 'UnicodeCharacter0xe830', [], 'Display', None],
  [ 0xe831, 'UnicodeCharacter0xe831', [], 'Display', None],
  [ 0xe832, 'UnicodeCharacter0xe832', [], 'Display', None],
  [ 0xe833, 'UnicodeCharacter0xe833', [], 'Display', None],
  [ 0xe834, 'UnicodeCharacter0xe834', [], 'Display', None],
  [ 0xe835, 'UnicodeCharacter0xe835', [], 'Display', None],
  [ 0xe836, 'UnicodeCharacter0xe836', [], 'Display', None],
  [ 0xe837, 'UnicodeCharacter0xe837', [], 'Display', None],
  [ 0xe838, 'UnicodeCharacter0xe838', [], 'Display', None],
  [ 0xe839, 'UnicodeCharacter0xe839', [], 'Display', None],
  [ 0xe83a, 'UnicodeCharacter0xe83a', [], 'Display', None],
  [ 0xe83b, 'UnicodeCharacter0xe83b', [], 'Display', None],
  [ 0xe83c, 'UnicodeCharacter0xe83c', [], 'Display', None],
  [ 0xe83d, 'UnicodeCharacter0xe83d', [], 'Display', None],
  [ 0xe83e, 'UnicodeCharacter0xe83e', [], 'Display', None],
  [ 0xe83f, 'UnicodeCharacter0xe83f', [], 'Display', None],
  [ 0xe840, 'UnicodeCharacter0xe840', [], 'Display', None],
  [ 0xe841, 'UnicodeCharacter0xe841', [], 'Display', None],
  [ 0xe842, 'UnicodeCharacter0xe842', [], 'Display', None],
  [ 0xe843, 'UnicodeCharacter0xe843', [], 'Display', None],
  [ 0xe844, 'UnicodeCharacter0xe844', [], 'Display', None],
  [ 0xe845, 'UnicodeCharacter0xe845', [], 'Display', None],
  [ 0xe846, 'UnicodeCharacter0xe846', [], 'Display', None],
  [ 0xe847, 'UnicodeCharacter0xe847', [], 'Display', None],
  [ 0xe848, 'UnicodeCharacter0xe848', [], 'Display', None],
  [ 0xe849, 'UnicodeCharacter0xe849', [], 'Display', None],
  [ 0xe84a, 'UnicodeCharacter0xe84a', [], 'Display', None],
  [ 0xe84b, 'UnicodeCharacter0xe84b', [], 'Display', None],
  [ 0xe84c, 'UnicodeCharacter0xe84c', [], 'Display', None],
  [ 0xe84d, 'UnicodeCharacter0xe84d', [], 'Display', None],
  [ 0xe84e, 'UnicodeCharacter0xe84e', [], 'Display', None],
  [ 0xe84f, 'UnicodeCharacter0xe84f', [], 'Display', None],
  [ 0xe850, 'UnicodeCharacter0xe850', [], 'Display', None],
  [ 0xe851, 'UnicodeCharacter0xe851', [], 'Display', None],
  [ 0xe852, 'UnicodeCharacter0xe852', [], 'Display', None],
  [ 0xe853, 'UnicodeCharacter0xe853', [], 'Display', None],
  [ 0xe854, 'UnicodeCharacter0xe854', [], 'Display', None],
  [ 0xe855, 'UnicodeCharacter0xe855', [], 'Display', None],
  [ 0xe856, 'UnicodeCharacter0xe856', [], 'Display', None],
  [ 0xe857, 'UnicodeCharacter0xe857', [], 'Display', None],
  [ 0xe858, 'UnicodeCharacter0xe858', [], 'Display', None],
  [ 0xe859, 'UnicodeCharacter0xe859', [], 'Display', None],
  [ 0xe85a, 'UnicodeCharacter0xe85a', [], 'Display', None],
  [ 0xe85b, 'UnicodeCharacter0xe85b', [], 'Display', None],
  [ 0xe85c, 'UnicodeCharacter0xe85c', [], 'Display', None],
  [ 0xe85d, 'UnicodeCharacter0xe85d', [], 'Display', None],
  [ 0xe85e, 'UnicodeCharacter0xe85e', [], 'Display', None],
  [ 0xe85f, 'UnicodeCharacter0xe85f', [], 'Display', None],
  [ 0xe860, 'UnicodeCharacter0xe860', [], 'Display', None],
  [ 0xe861, 'UnicodeCharacter0xe861', [], 'Display', None],
  [ 0xe862, 'UnicodeCharacter0xe862', [], 'Display', None],
  [ 0xe863, 'UnicodeCharacter0xe863', [], 'Display', None],
  [ 0xe864, 'UnicodeCharacter0xe864', [], 'Display', None],
  [ 0xe865, 'UnicodeCharacter0xe865', [], 'Display', None],
  [ 0xe866, 'UnicodeCharacter0xe866', [], 'Display', None],
  [ 0xe867, 'UnicodeCharacter0xe867', [], 'Display', None],
  [ 0xe868, 'UnicodeCharacter0xe868', [], 'Display', None],
  [ 0xe869, 'UnicodeCharacter0xe869', [], 'Display', None],
  [ 0xe86a, 'UnicodeCharacter0xe86a', [], 'Display', None],
  [ 0xe86b, 'UnicodeCharacter0xe86b', [], 'Display', None],
  [ 0xe86c, 'UnicodeCharacter0xe86c', [], 'Display', None],
  [ 0xe86d, 'UnicodeCharacter0xe86d', [], 'Display', None],
  [ 0xe86e, 'UnicodeCharacter0xe86e', [], 'Display', None],
  [ 0xe86f, 'UnicodeCharacter0xe86f', [], 'Display', None],
  [ 0xe870, 'UnicodeCharacter0xe870', [], 'Display', None],
  [ 0xe871, 'UnicodeCharacter0xe871', [], 'Display', None],
  [ 0xe872, 'UnicodeCharacter0xe872', [], 'Display', None],
  [ 0xe873, 'UnicodeCharacter0xe873', [], 'Display', None],
  [ 0xe874, 'UnicodeCharacter0xe874', [], 'Display', None],
  [ 0xe875, 'UnicodeCharacter0xe875', [], 'Display', None],
  [ 0xe876, 'UnicodeCharacter0xe876', [], 'Display', None],
  [ 0xe877, 'UnicodeCharacter0xe877', [], 'Display', None],
  [ 0xe878, 'UnicodeCharacter0xe878', [], 'Display', None],
  [ 0xe879, 'UnicodeCharacter0xe879', [], 'Display', None],
  [ 0xe87a, 'UnicodeCharacter0xe87a', [], 'Display', None],
  [ 0xe87b, 'UnicodeCharacter0xe87b', [], 'Display', None],
  [ 0xe87c, 'UnicodeCharacter0xe87c', [], 'Display', None],
  [ 0xe87d, 'UnicodeCharacter0xe87d', [], 'Display', None],
  [ 0xe87e, 'UnicodeCharacter0xe87e', [], 'Display', None],
  [ 0xe87f, 'UnicodeCharacter0xe87f', [], 'Display', None],
  [ 0xe880, 'UnicodeCharacter0xe880', [], 'Display', None],
  [ 0xe881, 'UnicodeCharacter0xe881', [], 'Display', None],
  [ 0xe882, 'UnicodeCharacter0xe882', [], 'Display', None],
  [ 0xe883, 'UnicodeCharacter0xe883', [], 'Display', None],
  [ 0xe884, 'UnicodeCharacter0xe884', [], 'Display', None],
  [ 0xe885, 'UnicodeCharacter0xe885', [], 'Display', None],
  [ 0xe886, 'UnicodeCharacter0xe886', [], 'Display', None],
  [ 0xe887, 'UnicodeCharacter0xe887', [], 'Display', None],
  [ 0xe888, 'UnicodeCharacter0xe888', [], 'Display', None],
  [ 0xe889, 'UnicodeCharacter0xe889', [], 'Display', None],
  [ 0xe88a, 'UnicodeCharacter0xe88a', [], 'Display', None],
  [ 0xe88b, 'UnicodeCharacter0xe88b', [], 'Display', None],
  [ 0xe88c, 'UnicodeCharacter0xe88c', [], 'Display', None],
  [ 0xe88d, 'UnicodeCharacter0xe88d', [], 'Display', None],
  [ 0xe88e, 'UnicodeCharacter0xe88e', [], 'Display', None],
  [ 0xe88f, 'UnicodeCharacter0xe88f', [], 'Display', None],
  [ 0xe890, 'UnicodeCharacter0xe890', [], 'Display', None],
  [ 0xe891, 'UnicodeCharacter0xe891', [], 'Display', None],
  [ 0xe892, 'UnicodeCharacter0xe892', [], 'Display', None],
  [ 0xe893, 'UnicodeCharacter0xe893', [], 'Display', None],
  [ 0xe894, 'UnicodeCharacter0xe894', [], 'Display', None],
  [ 0xe895, 'UnicodeCharacter0xe895', [], 'Display', None],
  [ 0xe896, 'UnicodeCharacter0xe896', [], 'Display', None],
  [ 0xe897, 'UnicodeCharacter0xe897', [], 'Display', None],
  [ 0xe898, 'UnicodeCharacter0xe898', [], 'Display', None],
  [ 0xe899, 'UnicodeCharacter0xe899', [], 'Display', None],
  [ 0xe89a, 'UnicodeCharacter0xe89a', [], 'Display', None],
  [ 0xe89b, 'UnicodeCharacter0xe89b', [], 'Display', None],
  [ 0xe89c, 'UnicodeCharacter0xe89c', [], 'Display', None],
  [ 0xe89d, 'UnicodeCharacter0xe89d', [], 'Display', None],
  [ 0xe89e, 'UnicodeCharacter0xe89e', [], 'Display', None],
  [ 0xe89f, 'UnicodeCharacter0xe89f', [], 'Display', None],
  [ 0xe8a0, 'UnicodeCharacter0xe8a0', [], 'Display', None],
  [ 0xe8a1, 'UnicodeCharacter0xe8a1', [], 'Display', None],
  [ 0xe8a2, 'UnicodeCharacter0xe8a2', [], 'Display', None],
  [ 0xe8a3, 'UnicodeCharacter0xe8a3', [], 'Display', None],
  [ 0xe8a4, 'UnicodeCharacter0xe8a4', [], 'Display', None],
  [ 0xe8a5, 'UnicodeCharacter0xe8a5', [], 'Display', None],
  [ 0xe8a6, 'UnicodeCharacter0xe8a6', [], 'Display', None],
  [ 0xe8a7, 'UnicodeCharacter0xe8a7', [], 'Display', None],
  [ 0xe8a8, 'UnicodeCharacter0xe8a8', [], 'Display', None],
  [ 0xe8a9, 'UnicodeCharacter0xe8a9', [], 'Display', None],
  [ 0xe8aa, 'UnicodeCharacter0xe8aa', [], 'Display', None],
  [ 0xe8ab, 'UnicodeCharacter0xe8ab', [], 'Display', None],
  [ 0xe8ac, 'UnicodeCharacter0xe8ac', [], 'Display', None],
  [ 0xe8ad, 'UnicodeCharacter0xe8ad', [], 'Display', None],
  [ 0xe8ae, 'UnicodeCharacter0xe8ae', [], 'Display', None],
  [ 0xe8af, 'UnicodeCharacter0xe8af', [], 'Display', None],
  [ 0xe8b0, 'UnicodeCharacter0xe8b0', [], 'Display', None],
  [ 0xe8b1, 'UnicodeCharacter0xe8b1', [], 'Display', None],
  [ 0xe8b2, 'UnicodeCharacter0xe8b2', [], 'Display', None],
  [ 0xe8b3, 'UnicodeCharacter0xe8b3', [], 'Display', None],
  [ 0xe8b4, 'UnicodeCharacter0xe8b4', [], 'Display', None],
  [ 0xe8b5, 'UnicodeCharacter0xe8b5', [], 'Display', None],
  [ 0xe8b6, 'UnicodeCharacter0xe8b6', [], 'Display', None],
  [ 0xe8b7, 'UnicodeCharacter0xe8b7', [], 'Display', None],
  [ 0xe8b8, 'UnicodeCharacter0xe8b8', [], 'Display', None],
  [ 0xe8b9, 'UnicodeCharacter0xe8b9', [], 'Display', None],
  [ 0xe8ba, 'UnicodeCharacter0xe8ba', [], 'Display', None],
  [ 0xe8bb, 'UnicodeCharacter0xe8bb', [], 'Display', None],
  [ 0xe8bc, 'UnicodeCharacter0xe8bc', [], 'Display', None],
  [ 0xe8bd, 'UnicodeCharacter0xe8bd', [], 'Display', None],
  [ 0xe8be, 'UnicodeCharacter0xe8be', [], 'Display', None],
  [ 0xe8bf, 'UnicodeCharacter0xe8bf', [], 'Display', None],
  [ 0xe8c0, 'UnicodeCharacter0xe8c0', [], 'Display', None],
  [ 0xe8c1, 'UnicodeCharacter0xe8c1', [], 'Display', None],
  [ 0xe8c2, 'UnicodeCharacter0xe8c2', [], 'Display', None],
  [ 0xe8c3, 'UnicodeCharacter0xe8c3', [], 'Display', None],
  [ 0xe8c4, 'UnicodeCharacter0xe8c4', [], 'Display', None],
  [ 0xe8c5, 'UnicodeCharacter0xe8c5', [], 'Display', None],
  [ 0xe8c6, 'UnicodeCharacter0xe8c6', [], 'Display', None],
  [ 0xe8c7, 'UnicodeCharacter0xe8c7', [], 'Display', None],
  [ 0xe8c8, 'UnicodeCharacter0xe8c8', [], 'Display', None],
  [ 0xe8c9, 'UnicodeCharacter0xe8c9', [], 'Display', None],
  [ 0xe8ca, 'UnicodeCharacter0xe8ca', [], 'Display', None],
  [ 0xe8cb, 'UnicodeCharacter0xe8cb', [], 'Display', None],
  [ 0xe8cc, 'UnicodeCharacter0xe8cc', [], 'Display', None],
  [ 0xe8cd, 'UnicodeCharacter0xe8cd', [], 'Display', None],
  [ 0xe8ce, 'UnicodeCharacter0xe8ce', [], 'Display', None],
  [ 0xe8cf, 'UnicodeCharacter0xe8cf', [], 'Display', None],
  [ 0xe8d0, 'UnicodeCharacter0xe8d0', [], 'Display', None],
  [ 0xe8d1, 'UnicodeCharacter0xe8d1', [], 'Display', None],
  [ 0xe8d2, 'UnicodeCharacter0xe8d2', [], 'Display', None],
  [ 0xe8d3, 'UnicodeCharacter0xe8d3', [], 'Display', None],
  [ 0xe8d4, 'UnicodeCharacter0xe8d4', [], 'Display', None],
  [ 0xe8d5, 'UnicodeCharacter0xe8d5', [], 'Display', None],
  [ 0xe8d6, 'UnicodeCharacter0xe8d6', [], 'Display', None],
  [ 0xe8d7, 'UnicodeCharacter0xe8d7', [], 'Display', None],
  [ 0xe8d8, 'UnicodeCharacter0xe8d8', [], 'Display', None],
  [ 0xe8d9, 'UnicodeCharacter0xe8d9', [], 'Display', None],
  [ 0xe8da, 'UnicodeCharacter0xe8da', [], 'Display', None],
  [ 0xe8db, 'UnicodeCharacter0xe8db', [], 'Display', None],
  [ 0xe8dc, 'UnicodeCharacter0xe8dc', [], 'Display', None],
  [ 0xe8dd, 'UnicodeCharacter0xe8dd', [], 'Display', None],
  [ 0xe8de, 'UnicodeCharacter0xe8de', [], 'Display', None],
  [ 0xe8df, 'UnicodeCharacter0xe8df', [], 'Display', None],
  [ 0xe8e0, 'UnicodeCharacter0xe8e0', [], 'Display', None],
  [ 0xe8e1, 'UnicodeCharacter0xe8e1', [], 'Display', None],
  [ 0xe8e2, 'UnicodeCharacter0xe8e2', [], 'Display', None],
  [ 0xe8e3, 'UnicodeCharacter0xe8e3', [], 'Display', None],
  [ 0xe8e4, 'UnicodeCharacter0xe8e4', [], 'Display', None],
  [ 0xe8e5, 'UnicodeCharacter0xe8e5', [], 'Display', None],
  [ 0xe8e6, 'UnicodeCharacter0xe8e6', [], 'Display', None],
  [ 0xe8e7, 'UnicodeCharacter0xe8e7', [], 'Display', None],
  [ 0xe8e8, 'UnicodeCharacter0xe8e8', [], 'Display', None],
  [ 0xe8e9, 'UnicodeCharacter0xe8e9', [], 'Display', None],
  [ 0xe8ea, 'UnicodeCharacter0xe8ea', [], 'Display', None],
  [ 0xe8eb, 'UnicodeCharacter0xe8eb', [], 'Display', None],
  [ 0xe8ec, 'UnicodeCharacter0xe8ec', [], 'Display', None],
  [ 0xe8ed, 'UnicodeCharacter0xe8ed', [], 'Display', None],
  [ 0xe8ee, 'UnicodeCharacter0xe8ee', [], 'Display', None],
  [ 0xe8ef, 'UnicodeCharacter0xe8ef', [], 'Display', None],
  [ 0xe8f0, 'UnicodeCharacter0xe8f0', [], 'Display', None],
  [ 0xe8f1, 'UnicodeCharacter0xe8f1', [], 'Display', None],
  [ 0xe8f2, 'UnicodeCharacter0xe8f2', [], 'Display', None],
  [ 0xe8f3, 'UnicodeCharacter0xe8f3', [], 'Display', None],
  [ 0xe8f4, 'UnicodeCharacter0xe8f4', [], 'Display', None],
  [ 0xe8f5, 'UnicodeCharacter0xe8f5', [], 'Display', None],
  [ 0xe8f6, 'UnicodeCharacter0xe8f6', [], 'Display', None],
  [ 0xe8f7, 'UnicodeCharacter0xe8f7', [], 'Display', None],
  [ 0xe8f8, 'UnicodeCharacter0xe8f8', [], 'Display', None],
  [ 0xe8f9, 'UnicodeCharacter0xe8f9', [], 'Display', None],
  [ 0xe8fa, 'UnicodeCharacter0xe8fa', [], 'Display', None],
  [ 0xe8fb, 'UnicodeCharacter0xe8fb', [], 'Display', None],
  [ 0xe8fc, 'UnicodeCharacter0xe8fc', [], 'Display', None],
  [ 0xe8fd, 'UnicodeCharacter0xe8fd', [], 'Display', None],
  [ 0xe8fe, 'UnicodeCharacter0xe8fe', [], 'Display', None],
  [ 0xe8ff, 'UnicodeCharacter0xe8ff', [], 'Display', None],
  [ 0xe900, 'UnicodeCharacter0xe900', [], 'Display', None],
  [ 0xe901, 'UnicodeCharacter0xe901', [], 'Display', None],
  [ 0xe902, 'UnicodeCharacter0xe902', [], 'Display', None],
  [ 0xe903, 'UnicodeCharacter0xe903', [], 'Display', None],
  [ 0xe904, 'UnicodeCharacter0xe904', [], 'Display', None],
  [ 0xe905, 'UnicodeCharacter0xe905', [], 'Display', None],
  [ 0xe906, 'UnicodeCharacter0xe906', [], 'Display', None],
  [ 0xe907, 'UnicodeCharacter0xe907', [], 'Display', None],
  [ 0xe908, 'UnicodeCharacter0xe908', [], 'Display', None],
  [ 0xe909, 'UnicodeCharacter0xe909', [], 'Display', None],
  [ 0xe90a, 'UnicodeCharacter0xe90a', [], 'Display', None],
  [ 0xe90b, 'UnicodeCharacter0xe90b', [], 'Display', None],
  [ 0xe90c, 'UnicodeCharacter0xe90c', [], 'Display', None],
  [ 0xe90d, 'UnicodeCharacter0xe90d', [], 'Display', None],
  [ 0xe90e, 'UnicodeCharacter0xe90e', [], 'Display', None],
  [ 0xe90f, 'UnicodeCharacter0xe90f', [], 'Display', None],
  [ 0xe910, 'UnicodeCharacter0xe910', [], 'Display', None],
  [ 0xe911, 'UnicodeCharacter0xe911', [], 'Display', None],
  [ 0xe912, 'UnicodeCharacter0xe912', [], 'Display', None],
  [ 0xe913, 'UnicodeCharacter0xe913', [], 'Display', None],
  [ 0xe914, 'UnicodeCharacter0xe914', [], 'Display', None],
  [ 0xe915, 'UnicodeCharacter0xe915', [], 'Display', None],
  [ 0xe916, 'UnicodeCharacter0xe916', [], 'Display', None],
  [ 0xe917, 'UnicodeCharacter0xe917', [], 'Display', None],
  [ 0xe918, 'UnicodeCharacter0xe918', [], 'Display', None],
  [ 0xe919, 'UnicodeCharacter0xe919', [], 'Display', None],
  [ 0xe91a, 'UnicodeCharacter0xe91a', [], 'Display', None],
  [ 0xe91b, 'UnicodeCharacter0xe91b', [], 'Display', None],
  [ 0xe91c, 'UnicodeCharacter0xe91c', [], 'Display', None],
  [ 0xe91d, 'UnicodeCharacter0xe91d', [], 'Display', None],
  [ 0xe91e, 'UnicodeCharacter0xe91e', [], 'Display', None],
  [ 0xe91f, 'UnicodeCharacter0xe91f', [], 'Display', None],
  [ 0xe920, 'UnicodeCharacter0xe920', [], 'Display', None],
  [ 0xe921, 'UnicodeCharacter0xe921', [], 'Display', None],
  [ 0xe922, 'UnicodeCharacter0xe922', [], 'Display', None],
  [ 0xe923, 'UnicodeCharacter0xe923', [], 'Display', None],
  [ 0xe924, 'UnicodeCharacter0xe924', [], 'Display', None],
  [ 0xe925, 'UnicodeCharacter0xe925', [], 'Display', None],
  [ 0xe926, 'UnicodeCharacter0xe926', [], 'Display', None],
  [ 0xe927, 'UnicodeCharacter0xe927', [], 'Display', None],
  [ 0xe928, 'UnicodeCharacter0xe928', [], 'Display', None],
  [ 0xe929, 'UnicodeCharacter0xe929', [], 'Display', None],
  [ 0xe92a, 'UnicodeCharacter0xe92a', [], 'Display', None],
  [ 0xe92b, 'UnicodeCharacter0xe92b', [], 'Display', None],
  [ 0xe92c, 'UnicodeCharacter0xe92c', [], 'Display', None],
  [ 0xe92d, 'UnicodeCharacter0xe92d', [], 'Display', None],
  [ 0xe92e, 'UnicodeCharacter0xe92e', [], 'Display', None],
  [ 0xe92f, 'UnicodeCharacter0xe92f', [], 'Display', None],
  [ 0xe930, 'UnicodeCharacter0xe930', [], 'Display', None],
  [ 0xe931, 'UnicodeCharacter0xe931', [], 'Display', None],
  [ 0xe932, 'UnicodeCharacter0xe932', [], 'Display', None],
  [ 0xe933, 'UnicodeCharacter0xe933', [], 'Display', None],
  [ 0xe934, 'UnicodeCharacter0xe934', [], 'Display', None],
  [ 0xe935, 'UnicodeCharacter0xe935', [], 'Display', None],
  [ 0xe936, 'UnicodeCharacter0xe936', [], 'Display', None],
  [ 0xe937, 'UnicodeCharacter0xe937', [], 'Display', None],
  [ 0xe938, 'UnicodeCharacter0xe938', [], 'Display', None],
  [ 0xe939, 'UnicodeCharacter0xe939', [], 'Display', None],
  [ 0xe93a, 'UnicodeCharacter0xe93a', [], 'Display', None],
  [ 0xe93b, 'UnicodeCharacter0xe93b', [], 'Display', None],
  [ 0xe93c, 'UnicodeCharacter0xe93c', [], 'Display', None],
  [ 0xe93d, 'UnicodeCharacter0xe93d', [], 'Display', None],
  [ 0xe93e, 'UnicodeCharacter0xe93e', [], 'Display', None],
  [ 0xe93f, 'UnicodeCharacter0xe93f', [], 'Display', None],
  [ 0xe940, 'UnicodeCharacter0xe940', [], 'Display', None],
  [ 0xe941, 'UnicodeCharacter0xe941', [], 'Display', None],
  [ 0xe942, 'UnicodeCharacter0xe942', [], 'Display', None],
  [ 0xe943, 'UnicodeCharacter0xe943', [], 'Display', None],
  [ 0xe944, 'UnicodeCharacter0xe944', [], 'Display', None],
  [ 0xe945, 'UnicodeCharacter0xe945', [], 'Display', None],
  [ 0xe946, 'UnicodeCharacter0xe946', [], 'Display', None],
  [ 0xe947, 'UnicodeCharacter0xe947', [], 'Display', None],
  [ 0xe948, 'UnicodeCharacter0xe948', [], 'Display', None],
  [ 0xe949, 'UnicodeCharacter0xe949', [], 'Display', None],
  [ 0xe94a, 'UnicodeCharacter0xe94a', [], 'Display', None],
  [ 0xe94b, 'UnicodeCharacter0xe94b', [], 'Display', None],
  [ 0xe94c, 'UnicodeCharacter0xe94c', [], 'Display', None],
  [ 0xe94d, 'UnicodeCharacter0xe94d', [], 'Display', None],
  [ 0xe94e, 'UnicodeCharacter0xe94e', [], 'Display', None],
  [ 0xe94f, 'UnicodeCharacter0xe94f', [], 'Display', None],
  [ 0xe950, 'UnicodeCharacter0xe950', [], 'Display', None],
  [ 0xe951, 'UnicodeCharacter0xe951', [], 'Display', None],
  [ 0xe952, 'UnicodeCharacter0xe952', [], 'Display', None],
  [ 0xe953, 'UnicodeCharacter0xe953', [], 'Display', None],
  [ 0xe954, 'UnicodeCharacter0xe954', [], 'Display', None],
  [ 0xe955, 'UnicodeCharacter0xe955', [], 'Display', None],
  [ 0xe956, 'UnicodeCharacter0xe956', [], 'Display', None],
  [ 0xe957, 'UnicodeCharacter0xe957', [], 'Display', None],
  [ 0xe958, 'UnicodeCharacter0xe958', [], 'Display', None],
  [ 0xe959, 'UnicodeCharacter0xe959', [], 'Display', None],
  [ 0xe95a, 'UnicodeCharacter0xe95a', [], 'Display', None],
  [ 0xe95b, 'UnicodeCharacter0xe95b', [], 'Display', None],
  [ 0xe95c, 'UnicodeCharacter0xe95c', [], 'Display', None],
  [ 0xe95d, 'UnicodeCharacter0xe95d', [], 'Display', None],
  [ 0xe95e, 'UnicodeCharacter0xe95e', [], 'Display', None],
  [ 0xe95f, 'UnicodeCharacter0xe95f', [], 'Display', None],
  [ 0xe960, 'UnicodeCharacter0xe960', [], 'Display', None],
  [ 0xe961, 'UnicodeCharacter0xe961', [], 'Display', None],
  [ 0xe962, 'UnicodeCharacter0xe962', [], 'Display', None],
  [ 0xe963, 'UnicodeCharacter0xe963', [], 'Display', None],
  [ 0xe964, 'UnicodeCharacter0xe964', [], 'Display', None],
  [ 0xe965, 'UnicodeCharacter0xe965', [], 'Display', None],
  [ 0xe966, 'UnicodeCharacter0xe966', [], 'Display', None],
  [ 0xe967, 'UnicodeCharacter0xe967', [], 'Display', None],
  [ 0xe968, 'UnicodeCharacter0xe968', [], 'Display', None],
  [ 0xe969, 'UnicodeCharacter0xe969', [], 'Display', None],
  [ 0xe96a, 'UnicodeCharacter0xe96a', [], 'Display', None],
  [ 0xe96b, 'UnicodeCharacter0xe96b', [], 'Display', None],
  [ 0xe96c, 'UnicodeCharacter0xe96c', [], 'Display', None],
  [ 0xe96d, 'UnicodeCharacter0xe96d', [], 'Display', None],
  [ 0xe96e, 'UnicodeCharacter0xe96e', [], 'Display', None],
  [ 0xe96f, 'UnicodeCharacter0xe96f', [], 'Display', None],
  [ 0xe970, 'UnicodeCharacter0xe970', [], 'Display', None],
  [ 0xe971, 'UnicodeCharacter0xe971', [], 'Display', None],
  [ 0xe972, 'UnicodeCharacter0xe972', [], 'Display', None],
  [ 0xe973, 'UnicodeCharacter0xe973', [], 'Display', None],
  [ 0xe974, 'UnicodeCharacter0xe974', [], 'Display', None],
  [ 0xe975, 'UnicodeCharacter0xe975', [], 'Display', None],
  [ 0xe976, 'UnicodeCharacter0xe976', [], 'Display', None],
  [ 0xe977, 'UnicodeCharacter0xe977', [], 'Display', None],
  [ 0xe978, 'UnicodeCharacter0xe978', [], 'Display', None],
  [ 0xe979, 'UnicodeCharacter0xe979', [], 'Display', None],
  [ 0xe97a, 'UnicodeCharacter0xe97a', [], 'Display', None],
  [ 0xe97b, 'UnicodeCharacter0xe97b', [], 'Display', None],
  [ 0xe97c, 'UnicodeCharacter0xe97c', [], 'Display', None],
  [ 0xe97d, 'UnicodeCharacter0xe97d', [], 'Display', None],
  [ 0xe97e, 'UnicodeCharacter0xe97e', [], 'Display', None],
  [ 0xe97f, 'UnicodeCharacter0xe97f', [], 'Display', None],
  [ 0xe980, 'UnicodeCharacter0xe980', [], 'Display', None],
  [ 0xe981, 'UnicodeCharacter0xe981', [], 'Display', None],
  [ 0xe982, 'UnicodeCharacter0xe982', [], 'Display', None],
  [ 0xe983, 'UnicodeCharacter0xe983', [], 'Display', None],
  [ 0xe984, 'UnicodeCharacter0xe984', [], 'Display', None],
  [ 0xe985, 'UnicodeCharacter0xe985', [], 'Display', None],
  [ 0xe986, 'UnicodeCharacter0xe986', [], 'Display', None],
  [ 0xe987, 'UnicodeCharacter0xe987', [], 'Display', None],
  [ 0xe988, 'UnicodeCharacter0xe988', [], 'Display', None],
  [ 0xe989, 'UnicodeCharacter0xe989', [], 'Display', None],
  [ 0xe98a, 'UnicodeCharacter0xe98a', [], 'Display', None],
  [ 0xe98b, 'UnicodeCharacter0xe98b', [], 'Display', None],
  [ 0xe98c, 'UnicodeCharacter0xe98c', [], 'Display', None],
  [ 0xe98d, 'UnicodeCharacter0xe98d', [], 'Display', None],
  [ 0xe98e, 'UnicodeCharacter0xe98e', [], 'Display', None],
  [ 0xe98f, 'UnicodeCharacter0xe98f', [], 'Display', None],
  [ 0xe990, 'UnicodeCharacter0xe990', [], 'Display', None],
  [ 0xe991, 'UnicodeCharacter0xe991', [], 'Display', None],
  [ 0xe992, 'UnicodeCharacter0xe992', [], 'Display', None],
  [ 0xe993, 'UnicodeCharacter0xe993', [], 'Display', None],
  [ 0xe994, 'UnicodeCharacter0xe994', [], 'Display', None],
  [ 0xe995, 'UnicodeCharacter0xe995', [], 'Display', None],
  [ 0xe996, 'UnicodeCharacter0xe996', [], 'Display', None],
  [ 0xe997, 'UnicodeCharacter0xe997', [], 'Display', None],
  [ 0xe998, 'UnicodeCharacter0xe998', [], 'Display', None],
  [ 0xe999, 'UnicodeCharacter0xe999', [], 'Display', None],
  [ 0xe99a, 'UnicodeCharacter0xe99a', [], 'Display', None],
  [ 0xe99b, 'UnicodeCharacter0xe99b', [], 'Display', None],
  [ 0xe99c, 'UnicodeCharacter0xe99c', [], 'Display', None],
  [ 0xe99d, 'UnicodeCharacter0xe99d', [], 'Display', None],
  [ 0xe99e, 'UnicodeCharacter0xe99e', [], 'Display', None],
  [ 0xe99f, 'UnicodeCharacter0xe99f', [], 'Display', None],
  [ 0xe9a0, 'UnicodeCharacter0xe9a0', [], 'Display', None],
  [ 0xe9a1, 'UnicodeCharacter0xe9a1', [], 'Display', None],
  [ 0xe9a2, 'UnicodeCharacter0xe9a2', [], 'Display', None],
  [ 0xe9a3, 'UnicodeCharacter0xe9a3', [], 'Display', None],
  [ 0xe9a4, 'UnicodeCharacter0xe9a4', [], 'Display', None],
  [ 0xe9a5, 'UnicodeCharacter0xe9a5', [], 'Display', None],
  [ 0xe9a6, 'UnicodeCharacter0xe9a6', [], 'Display', None],
  [ 0xe9a7, 'UnicodeCharacter0xe9a7', [], 'Display', None],
  [ 0xe9a8, 'UnicodeCharacter0xe9a8', [], 'Display', None],
  [ 0xe9a9, 'UnicodeCharacter0xe9a9', [], 'Display', None],
  [ 0xe9aa, 'UnicodeCharacter0xe9aa', [], 'Display', None],
  [ 0xe9ab, 'UnicodeCharacter0xe9ab', [], 'Display', None],
  [ 0xe9ac, 'UnicodeCharacter0xe9ac', [], 'Display', None],
  [ 0xe9ad, 'UnicodeCharacter0xe9ad', [], 'Display', None],
  [ 0xe9ae, 'UnicodeCharacter0xe9ae', [], 'Display', None],
  [ 0xe9af, 'UnicodeCharacter0xe9af', [], 'Display', None],
  [ 0xe9b0, 'UnicodeCharacter0xe9b0', [], 'Display', None],
  [ 0xe9b1, 'UnicodeCharacter0xe9b1', [], 'Display', None],
  [ 0xe9b2, 'UnicodeCharacter0xe9b2', [], 'Display', None],
  [ 0xe9b3, 'UnicodeCharacter0xe9b3', [], 'Display', None],
  [ 0xe9b4, 'UnicodeCharacter0xe9b4', [], 'Display', None],
  [ 0xe9b5, 'UnicodeCharacter0xe9b5', [], 'Display', None],
  [ 0xe9b6, 'UnicodeCharacter0xe9b6', [], 'Display', None],
  [ 0xe9b7, 'UnicodeCharacter0xe9b7', [], 'Display', None],
  [ 0xe9b8, 'UnicodeCharacter0xe9b8', [], 'Display', None],
  [ 0xe9b9, 'UnicodeCharacter0xe9b9', [], 'Display', None],
  [ 0xe9ba, 'UnicodeCharacter0xe9ba', [], 'Display', None],
  [ 0xe9bb, 'UnicodeCharacter0xe9bb', [], 'Display', None],
  [ 0xe9bc, 'UnicodeCharacter0xe9bc', [], 'Display', None],
  [ 0xe9bd, 'UnicodeCharacter0xe9bd', [], 'Display', None],
  [ 0xe9be, 'UnicodeCharacter0xe9be', [], 'Display', None],
  [ 0xe9bf, 'UnicodeCharacter0xe9bf', [], 'Display', None],
  [ 0xe9c0, 'UnicodeCharacter0xe9c0', [], 'Display', None],
  [ 0xe9c1, 'UnicodeCharacter0xe9c1', [], 'Display', None],
  [ 0xe9c2, 'UnicodeCharacter0xe9c2', [], 'Display', None],
  [ 0xe9c3, 'UnicodeCharacter0xe9c3', [], 'Display', None],
  [ 0xe9c4, 'UnicodeCharacter0xe9c4', [], 'Display', None],
  [ 0xe9c5, 'UnicodeCharacter0xe9c5', [], 'Display', None],
  [ 0xe9c6, 'UnicodeCharacter0xe9c6', [], 'Display', None],
  [ 0xe9c7, 'UnicodeCharacter0xe9c7', [], 'Display', None],
  [ 0xe9c8, 'UnicodeCharacter0xe9c8', [], 'Display', None],
  [ 0xe9c9, 'UnicodeCharacter0xe9c9', [], 'Display', None],
  [ 0xe9ca, 'UnicodeCharacter0xe9ca', [], 'Display', None],
  [ 0xe9cb, 'UnicodeCharacter0xe9cb', [], 'Display', None],
  [ 0xe9cc, 'UnicodeCharacter0xe9cc', [], 'Display', None],
  [ 0xe9cd, 'UnicodeCharacter0xe9cd', [], 'Display', None],
  [ 0xe9ce, 'UnicodeCharacter0xe9ce', [], 'Display', None],
  [ 0xe9cf, 'UnicodeCharacter0xe9cf', [], 'Display', None],
  [ 0xe9d0, 'UnicodeCharacter0xe9d0', [], 'Display', None],
  [ 0xe9d1, 'UnicodeCharacter0xe9d1', [], 'Display', None],
  [ 0xe9d2, 'UnicodeCharacter0xe9d2', [], 'Display', None],
  [ 0xe9d3, 'UnicodeCharacter0xe9d3', [], 'Display', None],
  [ 0xe9d4, 'UnicodeCharacter0xe9d4', [], 'Display', None],
  [ 0xe9d5, 'UnicodeCharacter0xe9d5', [], 'Display', None],
  [ 0xe9d6, 'UnicodeCharacter0xe9d6', [], 'Display', None],
  [ 0xe9d7, 'UnicodeCharacter0xe9d7', [], 'Display', None],
  [ 0xe9d8, 'UnicodeCharacter0xe9d8', [], 'Display', None],
  [ 0xe9d9, 'UnicodeCharacter0xe9d9', [], 'Display', None],
  [ 0xe9da, 'UnicodeCharacter0xe9da', [], 'Display', None],
  [ 0xe9db, 'UnicodeCharacter0xe9db', [], 'Display', None],
  [ 0xe9dc, 'UnicodeCharacter0xe9dc', [], 'Display', None],
  [ 0xe9dd, 'UnicodeCharacter0xe9dd', [], 'Display', None],
  [ 0xe9de, 'UnicodeCharacter0xe9de', [], 'Display', None],
  [ 0xe9df, 'UnicodeCharacter0xe9df', [], 'Display', None],
  [ 0xe9e0, 'UnicodeCharacter0xe9e0', [], 'Display', None],
  [ 0xe9e1, 'UnicodeCharacter0xe9e1', [], 'Display', None],
  [ 0xe9e2, 'UnicodeCharacter0xe9e2', [], 'Display', None],
  [ 0xe9e3, 'UnicodeCharacter0xe9e3', [], 'Display', None],
  [ 0xe9e4, 'UnicodeCharacter0xe9e4', [], 'Display', None],
  [ 0xe9e5, 'UnicodeCharacter0xe9e5', [], 'Display', None],
  [ 0xe9e6, 'UnicodeCharacter0xe9e6', [], 'Display', None],
  [ 0xe9e7, 'UnicodeCharacter0xe9e7', [], 'Display', None],
  [ 0xe9e8, 'UnicodeCharacter0xe9e8', [], 'Display', None],
  [ 0xe9e9, 'UnicodeCharacter0xe9e9', [], 'Display', None],
  [ 0xe9ea, 'UnicodeCharacter0xe9ea', [], 'Display', None],
  [ 0xe9eb, 'UnicodeCharacter0xe9eb', [], 'Display', None],
  [ 0xe9ec, 'UnicodeCharacter0xe9ec', [], 'Display', None],
  [ 0xe9ed, 'UnicodeCharacter0xe9ed', [], 'Display', None],
  [ 0xe9ee, 'UnicodeCharacter0xe9ee', [], 'Display', None],
  [ 0xe9ef, 'UnicodeCharacter0xe9ef', [], 'Display', None],
  [ 0xe9f0, 'UnicodeCharacter0xe9f0', [], 'Display', None],
  [ 0xe9f1, 'UnicodeCharacter0xe9f1', [], 'Display', None],
  [ 0xe9f2, 'UnicodeCharacter0xe9f2', [], 'Display', None],
  [ 0xe9f3, 'UnicodeCharacter0xe9f3', [], 'Display', None],
  [ 0xe9f4, 'UnicodeCharacter0xe9f4', [], 'Display', None],
  [ 0xe9f5, 'UnicodeCharacter0xe9f5', [], 'Display', None],
  [ 0xe9f6, 'UnicodeCharacter0xe9f6', [], 'Display', None],
  [ 0xe9f7, 'UnicodeCharacter0xe9f7', [], 'Display', None],
  [ 0xe9f8, 'UnicodeCharacter0xe9f8', [], 'Display', None],
  [ 0xe9f9, 'UnicodeCharacter0xe9f9', [], 'Display', None],
  [ 0xe9fa, 'UnicodeCharacter0xe9fa', [], 'Display', None],
  [ 0xe9fb, 'UnicodeCharacter0xe9fb', [], 'Display', None],
  [ 0xe9fc, 'UnicodeCharacter0xe9fc', [], 'Display', None],
  [ 0xe9fd, 'UnicodeCharacter0xe9fd', [], 'Display', None],
  [ 0xe9fe, 'UnicodeCharacter0xe9fe', [], 'Display', None],
  [ 0xe9ff, 'UnicodeCharacter0xe9ff', [], 'Display', None],
  [ 0xea00, 'UnicodeCharacter0xea00', [], 'Display', None],
  [ 0xea01, 'UnicodeCharacter0xea01', [], 'Display', None],
  [ 0xea02, 'UnicodeCharacter0xea02', [], 'Display', None],
  [ 0xea03, 'UnicodeCharacter0xea03', [], 'Display', None],
  [ 0xea04, 'UnicodeCharacter0xea04', [], 'Display', None],
  [ 0xea05, 'UnicodeCharacter0xea05', [], 'Display', None],
  [ 0xea06, 'UnicodeCharacter0xea06', [], 'Display', None],
  [ 0xea07, 'UnicodeCharacter0xea07', [], 'Display', None],
  [ 0xea08, 'UnicodeCharacter0xea08', [], 'Display', None],
  [ 0xea09, 'UnicodeCharacter0xea09', [], 'Display', None],
  [ 0xea0a, 'UnicodeCharacter0xea0a', [], 'Display', None],
  [ 0xea0b, 'UnicodeCharacter0xea0b', [], 'Display', None],
  [ 0xea0c, 'UnicodeCharacter0xea0c', [], 'Display', None],
  [ 0xea0d, 'UnicodeCharacter0xea0d', [], 'Display', None],
  [ 0xea0e, 'UnicodeCharacter0xea0e', [], 'Display', None],
  [ 0xea0f, 'UnicodeCharacter0xea0f', [], 'Display', None],
  [ 0xea10, 'UnicodeCharacter0xea10', [], 'Display', None],
  [ 0xea11, 'UnicodeCharacter0xea11', [], 'Display', None],
  [ 0xea12, 'UnicodeCharacter0xea12', [], 'Display', None],
  [ 0xea13, 'UnicodeCharacter0xea13', [], 'Display', None],
  [ 0xea14, 'UnicodeCharacter0xea14', [], 'Display', None],
  [ 0xea15, 'UnicodeCharacter0xea15', [], 'Display', None],
  [ 0xea16, 'UnicodeCharacter0xea16', [], 'Display', None],
  [ 0xea17, 'UnicodeCharacter0xea17', [], 'Display', None],
  [ 0xea18, 'UnicodeCharacter0xea18', [], 'Display', None],
  [ 0xea19, 'UnicodeCharacter0xea19', [], 'Display', None],
  [ 0xea1a, 'UnicodeCharacter0xea1a', [], 'Display', None],
  [ 0xea1b, 'UnicodeCharacter0xea1b', [], 'Display', None],
  [ 0xea1c, 'UnicodeCharacter0xea1c', [], 'Display', None],
  [ 0xea1d, 'UnicodeCharacter0xea1d', [], 'Display', None],
  [ 0xea1e, 'UnicodeCharacter0xea1e', [], 'Display', None],
  [ 0xea1f, 'UnicodeCharacter0xea1f', [], 'Display', None],
  [ 0xea20, 'UnicodeCharacter0xea20', [], 'Display', None],
  [ 0xea21, 'UnicodeCharacter0xea21', [], 'Display', None],
  [ 0xea22, 'UnicodeCharacter0xea22', [], 'Display', None],
  [ 0xea23, 'UnicodeCharacter0xea23', [], 'Display', None],
  [ 0xea24, 'UnicodeCharacter0xea24', [], 'Display', None],
  [ 0xea25, 'UnicodeCharacter0xea25', [], 'Display', None],
  [ 0xea26, 'UnicodeCharacter0xea26', [], 'Display', None],
  [ 0xea27, 'UnicodeCharacter0xea27', [], 'Display', None],
  [ 0xea28, 'UnicodeCharacter0xea28', [], 'Display', None],
  [ 0xea29, 'UnicodeCharacter0xea29', [], 'Display', None],
  [ 0xea2a, 'UnicodeCharacter0xea2a', [], 'Display', None],
  [ 0xea2b, 'UnicodeCharacter0xea2b', [], 'Display', None],
  [ 0xea2c, 'UnicodeCharacter0xea2c', [], 'Display', None],
  [ 0xea2d, 'UnicodeCharacter0xea2d', [], 'Display', None],
  [ 0xea2e, 'UnicodeCharacter0xea2e', [], 'Display', None],
  [ 0xea2f, 'UnicodeCharacter0xea2f', [], 'Display', None],
  [ 0xea30, 'UnicodeCharacter0xea30', [], 'Display', None],
  [ 0xea31, 'UnicodeCharacter0xea31', [], 'Display', None],
  [ 0xea32, 'UnicodeCharacter0xea32', [], 'Display', None],
  [ 0xea33, 'UnicodeCharacter0xea33', [], 'Display', None],
  [ 0xea34, 'UnicodeCharacter0xea34', [], 'Display', None],
  [ 0xea35, 'UnicodeCharacter0xea35', [], 'Display', None],
  [ 0xea36, 'UnicodeCharacter0xea36', [], 'Display', None],
  [ 0xea37, 'UnicodeCharacter0xea37', [], 'Display', None],
  [ 0xea38, 'UnicodeCharacter0xea38', [], 'Display', None],
  [ 0xea39, 'UnicodeCharacter0xea39', [], 'Display', None],
  [ 0xea3a, 'UnicodeCharacter0xea3a', [], 'Display', None],
  [ 0xea3b, 'UnicodeCharacter0xea3b', [], 'Display', None],
  [ 0xea3c, 'UnicodeCharacter0xea3c', [], 'Display', None],
  [ 0xea3d, 'UnicodeCharacter0xea3d', [], 'Display', None],
  [ 0xea3e, 'UnicodeCharacter0xea3e', [], 'Display', None],
  [ 0xea3f, 'UnicodeCharacter0xea3f', [], 'Display', None],
  [ 0xea40, 'UnicodeCharacter0xea40', [], 'Display', None],
  [ 0xea41, 'UnicodeCharacter0xea41', [], 'Display', None],
  [ 0xea42, 'UnicodeCharacter0xea42', [], 'Display', None],
  [ 0xea43, 'UnicodeCharacter0xea43', [], 'Display', None],
  [ 0xea44, 'UnicodeCharacter0xea44', [], 'Display', None],
  [ 0xea45, 'UnicodeCharacter0xea45', [], 'Display', None],
  [ 0xea46, 'UnicodeCharacter0xea46', [], 'Display', None],
  [ 0xea47, 'UnicodeCharacter0xea47', [], 'Display', None],
  [ 0xea48, 'UnicodeCharacter0xea48', [], 'Display', None],
  [ 0xea49, 'UnicodeCharacter0xea49', [], 'Display', None],
  [ 0xea4a, 'UnicodeCharacter0xea4a', [], 'Display', None],
  [ 0xea4b, 'UnicodeCharacter0xea4b', [], 'Display', None],
  [ 0xea4c, 'UnicodeCharacter0xea4c', [], 'Display', None],
  [ 0xea4d, 'UnicodeCharacter0xea4d', [], 'Display', None],
  [ 0xea4e, 'UnicodeCharacter0xea4e', [], 'Display', None],
  [ 0xea4f, 'UnicodeCharacter0xea4f', [], 'Display', None],
  [ 0xea50, 'UnicodeCharacter0xea50', [], 'Display', None],
  [ 0xea51, 'UnicodeCharacter0xea51', [], 'Display', None],
  [ 0xea52, 'UnicodeCharacter0xea52', [], 'Display', None],
  [ 0xea53, 'UnicodeCharacter0xea53', [], 'Display', None],
  [ 0xea54, 'UnicodeCharacter0xea54', [], 'Display', None],
  [ 0xea55, 'UnicodeCharacter0xea55', [], 'Display', None],
  [ 0xea56, 'UnicodeCharacter0xea56', [], 'Display', None],
  [ 0xea57, 'UnicodeCharacter0xea57', [], 'Display', None],
  [ 0xea58, 'UnicodeCharacter0xea58', [], 'Display', None],
  [ 0xea59, 'UnicodeCharacter0xea59', [], 'Display', None],
  [ 0xea5a, 'UnicodeCharacter0xea5a', [], 'Display', None],
  [ 0xea5b, 'UnicodeCharacter0xea5b', [], 'Display', None],
  [ 0xea5c, 'UnicodeCharacter0xea5c', [], 'Display', None],
  [ 0xea5d, 'UnicodeCharacter0xea5d', [], 'Display', None],
  [ 0xea5e, 'UnicodeCharacter0xea5e', [], 'Display', None],
  [ 0xea5f, 'UnicodeCharacter0xea5f', [], 'Display', None],
  [ 0xea60, 'UnicodeCharacter0xea60', [], 'Display', None],
  [ 0xea61, 'UnicodeCharacter0xea61', [], 'Display', None],
  [ 0xea62, 'UnicodeCharacter0xea62', [], 'Display', None],
  [ 0xea63, 'UnicodeCharacter0xea63', [], 'Display', None],
  [ 0xea64, 'UnicodeCharacter0xea64', [], 'Display', None],
  [ 0xea65, 'UnicodeCharacter0xea65', [], 'Display', None],
  [ 0xea66, 'UnicodeCharacter0xea66', [], 'Display', None],
  [ 0xea67, 'UnicodeCharacter0xea67', [], 'Display', None],
  [ 0xea68, 'UnicodeCharacter0xea68', [], 'Display', None],
  [ 0xea69, 'UnicodeCharacter0xea69', [], 'Display', None],
  [ 0xea6a, 'UnicodeCharacter0xea6a', [], 'Display', None],
  [ 0xea6b, 'UnicodeCharacter0xea6b', [], 'Display', None],
  [ 0xea6c, 'UnicodeCharacter0xea6c', [], 'Display', None],
  [ 0xea6d, 'UnicodeCharacter0xea6d', [], 'Display', None],
  [ 0xea6e, 'UnicodeCharacter0xea6e', [], 'Display', None],
  [ 0xea6f, 'UnicodeCharacter0xea6f', [], 'Display', None],
  [ 0xea70, 'UnicodeCharacter0xea70', [], 'Display', None],
  [ 0xea71, 'UnicodeCharacter0xea71', [], 'Display', None],
  [ 0xea72, 'UnicodeCharacter0xea72', [], 'Display', None],
  [ 0xea73, 'UnicodeCharacter0xea73', [], 'Display', None],
  [ 0xea74, 'UnicodeCharacter0xea74', [], 'Display', None],
  [ 0xea75, 'UnicodeCharacter0xea75', [], 'Display', None],
  [ 0xea76, 'UnicodeCharacter0xea76', [], 'Display', None],
  [ 0xea77, 'UnicodeCharacter0xea77', [], 'Display', None],
  [ 0xea78, 'UnicodeCharacter0xea78', [], 'Display', None],
  [ 0xea79, 'UnicodeCharacter0xea79', [], 'Display', None],
  [ 0xea7a, 'UnicodeCharacter0xea7a', [], 'Display', None],
  [ 0xea7b, 'UnicodeCharacter0xea7b', [], 'Display', None],
  [ 0xea7c, 'UnicodeCharacter0xea7c', [], 'Display', None],
  [ 0xea7d, 'UnicodeCharacter0xea7d', [], 'Display', None],
  [ 0xea7e, 'UnicodeCharacter0xea7e', [], 'Display', None],
  [ 0xea7f, 'UnicodeCharacter0xea7f', [], 'Display', None],
  [ 0xea80, 'UnicodeCharacter0xea80', [], 'Display', None],
  [ 0xea81, 'UnicodeCharacter0xea81', [], 'Display', None],
  [ 0xea82, 'UnicodeCharacter0xea82', [], 'Display', None],
  [ 0xea83, 'UnicodeCharacter0xea83', [], 'Display', None],
  [ 0xea84, 'UnicodeCharacter0xea84', [], 'Display', None],
  [ 0xea85, 'UnicodeCharacter0xea85', [], 'Display', None],
  [ 0xea86, 'UnicodeCharacter0xea86', [], 'Display', None],
  [ 0xea87, 'UnicodeCharacter0xea87', [], 'Display', None],
  [ 0xea88, 'UnicodeCharacter0xea88', [], 'Display', None],
  [ 0xea89, 'UnicodeCharacter0xea89', [], 'Display', None],
  [ 0xea8a, 'UnicodeCharacter0xea8a', [], 'Display', None],
  [ 0xea8b, 'UnicodeCharacter0xea8b', [], 'Display', None],
  [ 0xea8c, 'UnicodeCharacter0xea8c', [], 'Display', None],
  [ 0xea8d, 'UnicodeCharacter0xea8d', [], 'Display', None],
  [ 0xea8e, 'UnicodeCharacter0xea8e', [], 'Display', None],
  [ 0xea8f, 'UnicodeCharacter0xea8f', [], 'Display', None],
  [ 0xea90, 'UnicodeCharacter0xea90', [], 'Display', None],
  [ 0xea91, 'UnicodeCharacter0xea91', [], 'Display', None],
  [ 0xea92, 'UnicodeCharacter0xea92', [], 'Display', None],
  [ 0xea93, 'UnicodeCharacter0xea93', [], 'Display', None],
  [ 0xea94, 'UnicodeCharacter0xea94', [], 'Display', None],
  [ 0xea95, 'UnicodeCharacter0xea95', [], 'Display', None],
  [ 0xea96, 'UnicodeCharacter0xea96', [], 'Display', None],
  [ 0xea97, 'UnicodeCharacter0xea97', [], 'Display', None],
  [ 0xea98, 'UnicodeCharacter0xea98', [], 'Display', None],
  [ 0xea99, 'UnicodeCharacter0xea99', [], 'Display', None],
  [ 0xea9a, 'UnicodeCharacter0xea9a', [], 'Display', None],
  [ 0xea9b, 'UnicodeCharacter0xea9b', [], 'Display', None],
  [ 0xea9c, 'UnicodeCharacter0xea9c', [], 'Display', None],
  [ 0xea9d, 'UnicodeCharacter0xea9d', [], 'Display', None],
  [ 0xea9e, 'UnicodeCharacter0xea9e', [], 'Display', None],
  [ 0xea9f, 'UnicodeCharacter0xea9f', [], 'Display', None],
  [ 0xeaa0, 'UnicodeCharacter0xeaa0', [], 'Display', None],
  [ 0xeaa1, 'UnicodeCharacter0xeaa1', [], 'Display', None],
  [ 0xeaa2, 'UnicodeCharacter0xeaa2', [], 'Display', None],
  [ 0xeaa3, 'UnicodeCharacter0xeaa3', [], 'Display', None],
  [ 0xeaa4, 'UnicodeCharacter0xeaa4', [], 'Display', None],
  [ 0xeaa5, 'UnicodeCharacter0xeaa5', [], 'Display', None],
  [ 0xeaa6, 'UnicodeCharacter0xeaa6', [], 'Display', None],
  [ 0xeaa7, 'UnicodeCharacter0xeaa7', [], 'Display', None],
  [ 0xeaa8, 'UnicodeCharacter0xeaa8', [], 'Display', None],
  [ 0xeaa9, 'UnicodeCharacter0xeaa9', [], 'Display', None],
  [ 0xeaaa, 'UnicodeCharacter0xeaaa', [], 'Display', None],
  [ 0xeaab, 'UnicodeCharacter0xeaab', [], 'Display', None],
  [ 0xeaac, 'UnicodeCharacter0xeaac', [], 'Display', None],
  [ 0xeaad, 'UnicodeCharacter0xeaad', [], 'Display', None],
  [ 0xeaae, 'UnicodeCharacter0xeaae', [], 'Display', None],
  [ 0xeaaf, 'UnicodeCharacter0xeaaf', [], 'Display', None],
  [ 0xeab0, 'UnicodeCharacter0xeab0', [], 'Display', None],
  [ 0xeab1, 'UnicodeCharacter0xeab1', [], 'Display', None],
  [ 0xeab2, 'UnicodeCharacter0xeab2', [], 'Display', None],
  [ 0xeab3, 'UnicodeCharacter0xeab3', [], 'Display', None],
  [ 0xeab4, 'UnicodeCharacter0xeab4', [], 'Display', None],
  [ 0xeab5, 'UnicodeCharacter0xeab5', [], 'Display', None],
  [ 0xeab6, 'UnicodeCharacter0xeab6', [], 'Display', None],
  [ 0xeab7, 'UnicodeCharacter0xeab7', [], 'Display', None],
  [ 0xeab8, 'UnicodeCharacter0xeab8', [], 'Display', None],
  [ 0xeab9, 'UnicodeCharacter0xeab9', [], 'Display', None],
  [ 0xeaba, 'UnicodeCharacter0xeaba', [], 'Display', None],
  [ 0xeabb, 'UnicodeCharacter0xeabb', [], 'Display', None],
  [ 0xeabc, 'UnicodeCharacter0xeabc', [], 'Display', None],
  [ 0xeabd, 'UnicodeCharacter0xeabd', [], 'Display', None],
  [ 0xeabe, 'UnicodeCharacter0xeabe', [], 'Display', None],
  [ 0xeabf, 'UnicodeCharacter0xeabf', [], 'Display', None],
  [ 0xeac0, 'UnicodeCharacter0xeac0', [], 'Display', None],
  [ 0xeac1, 'UnicodeCharacter0xeac1', [], 'Display', None],
  [ 0xeac2, 'UnicodeCharacter0xeac2', [], 'Display', None],
  [ 0xeac3, 'UnicodeCharacter0xeac3', [], 'Display', None],
  [ 0xeac4, 'UnicodeCharacter0xeac4', [], 'Display', None],
  [ 0xeac5, 'UnicodeCharacter0xeac5', [], 'Display', None],
  [ 0xeac6, 'UnicodeCharacter0xeac6', [], 'Display', None],
  [ 0xeac7, 'UnicodeCharacter0xeac7', [], 'Display', None],
  [ 0xeac8, 'UnicodeCharacter0xeac8', [], 'Display', None],
  [ 0xeac9, 'UnicodeCharacter0xeac9', [], 'Display', None],
  [ 0xeaca, 'UnicodeCharacter0xeaca', [], 'Display', None],
  [ 0xeacb, 'UnicodeCharacter0xeacb', [], 'Display', None],
  [ 0xeacc, 'UnicodeCharacter0xeacc', [], 'Display', None],
  [ 0xeacd, 'UnicodeCharacter0xeacd', [], 'Display', None],
  [ 0xeace, 'UnicodeCharacter0xeace', [], 'Display', None],
  [ 0xeacf, 'UnicodeCharacter0xeacf', [], 'Display', None],
  [ 0xead0, 'UnicodeCharacter0xead0', [], 'Display', None],
  [ 0xead1, 'UnicodeCharacter0xead1', [], 'Display', None],
  [ 0xead2, 'UnicodeCharacter0xead2', [], 'Display', None],
  [ 0xead3, 'UnicodeCharacter0xead3', [], 'Display', None],
  [ 0xead4, 'UnicodeCharacter0xead4', [], 'Display', None],
  [ 0xead5, 'UnicodeCharacter0xead5', [], 'Display', None],
  [ 0xead6, 'UnicodeCharacter0xead6', [], 'Display', None],
  [ 0xead7, 'UnicodeCharacter0xead7', [], 'Display', None],
  [ 0xead8, 'UnicodeCharacter0xead8', [], 'Display', None],
  [ 0xead9, 'UnicodeCharacter0xead9', [], 'Display', None],
  [ 0xeada, 'UnicodeCharacter0xeada', [], 'Display', None],
  [ 0xeadb, 'UnicodeCharacter0xeadb', [], 'Display', None],
  [ 0xeadc, 'UnicodeCharacter0xeadc', [], 'Display', None],
  [ 0xeadd, 'UnicodeCharacter0xeadd', [], 'Display', None],
  [ 0xeade, 'UnicodeCharacter0xeade', [], 'Display', None],
  [ 0xeadf, 'UnicodeCharacter0xeadf', [], 'Display', None],
  [ 0xeae0, 'UnicodeCharacter0xeae0', [], 'Display', None],
  [ 0xeae1, 'UnicodeCharacter0xeae1', [], 'Display', None],
  [ 0xeae2, 'UnicodeCharacter0xeae2', [], 'Display', None],
  [ 0xeae3, 'UnicodeCharacter0xeae3', [], 'Display', None],
  [ 0xeae4, 'UnicodeCharacter0xeae4', [], 'Display', None],
  [ 0xeae5, 'UnicodeCharacter0xeae5', [], 'Display', None],
  [ 0xeae6, 'UnicodeCharacter0xeae6', [], 'Display', None],
  [ 0xeae7, 'UnicodeCharacter0xeae7', [], 'Display', None],
  [ 0xeae8, 'UnicodeCharacter0xeae8', [], 'Display', None],
  [ 0xeae9, 'UnicodeCharacter0xeae9', [], 'Display', None],
  [ 0xeaea, 'UnicodeCharacter0xeaea', [], 'Display', None],
  [ 0xeaeb, 'UnicodeCharacter0xeaeb', [], 'Display', None],
  [ 0xeaec, 'UnicodeCharacter0xeaec', [], 'Display', None],
  [ 0xeaed, 'UnicodeCharacter0xeaed', [], 'Display', None],
  [ 0xeaee, 'UnicodeCharacter0xeaee', [], 'Display', None],
  [ 0xeaef, 'UnicodeCharacter0xeaef', [], 'Display', None],
  [ 0xeaf0, 'UnicodeCharacter0xeaf0', [], 'Display', None],
  [ 0xeaf1, 'UnicodeCharacter0xeaf1', [], 'Display', None],
  [ 0xeaf2, 'UnicodeCharacter0xeaf2', [], 'Display', None],
  [ 0xeaf3, 'UnicodeCharacter0xeaf3', [], 'Display', None],
  [ 0xeaf4, 'UnicodeCharacter0xeaf4', [], 'Display', None],
  [ 0xeaf5, 'UnicodeCharacter0xeaf5', [], 'Display', None],
  [ 0xeaf6, 'UnicodeCharacter0xeaf6', [], 'Display', None],
  [ 0xeaf7, 'UnicodeCharacter0xeaf7', [], 'Display', None],
  [ 0xeaf8, 'UnicodeCharacter0xeaf8', [], 'Display', None],
  [ 0xeaf9, 'UnicodeCharacter0xeaf9', [], 'Display', None],
  [ 0xeafa, 'UnicodeCharacter0xeafa', [], 'Display', None],
  [ 0xeafb, 'UnicodeCharacter0xeafb', [], 'Display', None],
  [ 0xeafc, 'UnicodeCharacter0xeafc', [], 'Display', None],
  [ 0xeafd, 'UnicodeCharacter0xeafd', [], 'Display', None],
  [ 0xeafe, 'UnicodeCharacter0xeafe', [], 'Display', None],
  [ 0xeaff, 'UnicodeCharacter0xeaff', [], 'Display', None],
  [ 0xeb00, 'UnicodeCharacter0xeb00', [], 'Display', None],
  [ 0xeb01, 'UnicodeCharacter0xeb01', [], 'Display', None],
  [ 0xeb02, 'UnicodeCharacter0xeb02', [], 'Display', None],
  [ 0xeb03, 'UnicodeCharacter0xeb03', [], 'Display', None],
  [ 0xeb04, 'UnicodeCharacter0xeb04', [], 'Display', None],
  [ 0xeb05, 'UnicodeCharacter0xeb05', [], 'Display', None],
  [ 0xeb06, 'UnicodeCharacter0xeb06', [], 'Display', None],
  [ 0xeb07, 'UnicodeCharacter0xeb07', [], 'Display', None],
  [ 0xeb08, 'UnicodeCharacter0xeb08', [], 'Display', None],
  [ 0xeb09, 'UnicodeCharacter0xeb09', [], 'Display', None],
  [ 0xeb0a, 'UnicodeCharacter0xeb0a', [], 'Display', None],
  [ 0xeb0b, 'UnicodeCharacter0xeb0b', [], 'Display', None],
  [ 0xeb0c, 'UnicodeCharacter0xeb0c', [], 'Display', None],
  [ 0xeb0d, 'UnicodeCharacter0xeb0d', [], 'Display', None],
  [ 0xeb0e, 'UnicodeCharacter0xeb0e', [], 'Display', None],
  [ 0xeb0f, 'UnicodeCharacter0xeb0f', [], 'Display', None],
  [ 0xeb10, 'UnicodeCharacter0xeb10', [], 'Display', None],
  [ 0xeb11, 'UnicodeCharacter0xeb11', [], 'Display', None],
  [ 0xeb12, 'UnicodeCharacter0xeb12', [], 'Display', None],
  [ 0xeb13, 'UnicodeCharacter0xeb13', [], 'Display', None],
  [ 0xeb14, 'UnicodeCharacter0xeb14', [], 'Display', None],
  [ 0xeb15, 'UnicodeCharacter0xeb15', [], 'Display', None],
  [ 0xeb16, 'UnicodeCharacter0xeb16', [], 'Display', None],
  [ 0xeb17, 'UnicodeCharacter0xeb17', [], 'Display', None],
  [ 0xeb18, 'UnicodeCharacter0xeb18', [], 'Display', None],
  [ 0xeb19, 'UnicodeCharacter0xeb19', [], 'Display', None],
  [ 0xeb1a, 'UnicodeCharacter0xeb1a', [], 'Display', None],
  [ 0xeb1b, 'UnicodeCharacter0xeb1b', [], 'Display', None],
  [ 0xeb1c, 'UnicodeCharacter0xeb1c', [], 'Display', None],
  [ 0xeb1d, 'UnicodeCharacter0xeb1d', [], 'Display', None],
  [ 0xeb1e, 'UnicodeCharacter0xeb1e', [], 'Display', None],
  [ 0xeb1f, 'UnicodeCharacter0xeb1f', [], 'Display', None],
  [ 0xeb20, 'UnicodeCharacter0xeb20', [], 'Display', None],
  [ 0xeb21, 'UnicodeCharacter0xeb21', [], 'Display', None],
  [ 0xeb22, 'UnicodeCharacter0xeb22', [], 'Display', None],
  [ 0xeb23, 'UnicodeCharacter0xeb23', [], 'Display', None],
  [ 0xeb24, 'UnicodeCharacter0xeb24', [], 'Display', None],
  [ 0xeb25, 'UnicodeCharacter0xeb25', [], 'Display', None],
  [ 0xeb26, 'UnicodeCharacter0xeb26', [], 'Display', None],
  [ 0xeb27, 'UnicodeCharacter0xeb27', [], 'Display', None],
  [ 0xeb28, 'UnicodeCharacter0xeb28', [], 'Display', None],
  [ 0xeb29, 'UnicodeCharacter0xeb29', [], 'Display', None],
  [ 0xeb2a, 'UnicodeCharacter0xeb2a', [], 'Display', None],
  [ 0xeb2b, 'UnicodeCharacter0xeb2b', [], 'Display', None],
  [ 0xeb2c, 'UnicodeCharacter0xeb2c', [], 'Display', None],
  [ 0xeb2d, 'UnicodeCharacter0xeb2d', [], 'Display', None],
  [ 0xeb2e, 'UnicodeCharacter0xeb2e', [], 'Display', None],
  [ 0xeb2f, 'UnicodeCharacter0xeb2f', [], 'Display', None],
  [ 0xeb30, 'UnicodeCharacter0xeb30', [], 'Display', None],
  [ 0xeb31, 'UnicodeCharacter0xeb31', [], 'Display', None],
  [ 0xeb32, 'UnicodeCharacter0xeb32', [], 'Display', None],
  [ 0xeb33, 'UnicodeCharacter0xeb33', [], 'Display', None],
  [ 0xeb34, 'UnicodeCharacter0xeb34', [], 'Display', None],
  [ 0xeb35, 'UnicodeCharacter0xeb35', [], 'Display', None],
  [ 0xeb36, 'UnicodeCharacter0xeb36', [], 'Display', None],
  [ 0xeb37, 'UnicodeCharacter0xeb37', [], 'Display', None],
  [ 0xeb38, 'UnicodeCharacter0xeb38', [], 'Display', None],
  [ 0xeb39, 'UnicodeCharacter0xeb39', [], 'Display', None],
  [ 0xeb3a, 'UnicodeCharacter0xeb3a', [], 'Display', None],
  [ 0xeb3b, 'UnicodeCharacter0xeb3b', [], 'Display', None],
  [ 0xeb3c, 'UnicodeCharacter0xeb3c', [], 'Display', None],
  [ 0xeb3d, 'UnicodeCharacter0xeb3d', [], 'Display', None],
  [ 0xeb3e, 'UnicodeCharacter0xeb3e', [], 'Display', None],
  [ 0xeb3f, 'UnicodeCharacter0xeb3f', [], 'Display', None],
  [ 0xeb40, 'UnicodeCharacter0xeb40', [], 'Display', None],
  [ 0xeb41, 'UnicodeCharacter0xeb41', [], 'Display', None],
  [ 0xeb42, 'UnicodeCharacter0xeb42', [], 'Display', None],
  [ 0xeb43, 'UnicodeCharacter0xeb43', [], 'Display', None],
  [ 0xeb44, 'UnicodeCharacter0xeb44', [], 'Display', None],
  [ 0xeb45, 'UnicodeCharacter0xeb45', [], 'Display', None],
  [ 0xeb46, 'UnicodeCharacter0xeb46', [], 'Display', None],
  [ 0xeb47, 'UnicodeCharacter0xeb47', [], 'Display', None],
  [ 0xeb48, 'UnicodeCharacter0xeb48', [], 'Display', None],
  [ 0xeb49, 'UnicodeCharacter0xeb49', [], 'Display', None],
  [ 0xeb4a, 'UnicodeCharacter0xeb4a', [], 'Display', None],
  [ 0xeb4b, 'UnicodeCharacter0xeb4b', [], 'Display', None],
  [ 0xeb4c, 'UnicodeCharacter0xeb4c', [], 'Display', None],
  [ 0xeb4d, 'UnicodeCharacter0xeb4d', [], 'Display', None],
  [ 0xeb4e, 'UnicodeCharacter0xeb4e', [], 'Display', None],
  [ 0xeb4f, 'UnicodeCharacter0xeb4f', [], 'Display', None],
  [ 0xeb50, 'UnicodeCharacter0xeb50', [], 'Display', None],
  [ 0xeb51, 'UnicodeCharacter0xeb51', [], 'Display', None],
  [ 0xeb52, 'UnicodeCharacter0xeb52', [], 'Display', None],
  [ 0xeb53, 'UnicodeCharacter0xeb53', [], 'Display', None],
  [ 0xeb54, 'UnicodeCharacter0xeb54', [], 'Display', None],
  [ 0xeb55, 'UnicodeCharacter0xeb55', [], 'Display', None],
  [ 0xeb56, 'UnicodeCharacter0xeb56', [], 'Display', None],
  [ 0xeb57, 'UnicodeCharacter0xeb57', [], 'Display', None],
  [ 0xeb58, 'UnicodeCharacter0xeb58', [], 'Display', None],
  [ 0xeb59, 'UnicodeCharacter0xeb59', [], 'Display', None],
  [ 0xeb5a, 'UnicodeCharacter0xeb5a', [], 'Display', None],
  [ 0xeb5b, 'UnicodeCharacter0xeb5b', [], 'Display', None],
  [ 0xeb5c, 'UnicodeCharacter0xeb5c', [], 'Display', None],
  [ 0xeb5d, 'UnicodeCharacter0xeb5d', [], 'Display', None],
  [ 0xeb5e, 'UnicodeCharacter0xeb5e', [], 'Display', None],
  [ 0xeb5f, 'UnicodeCharacter0xeb5f', [], 'Display', None],
  [ 0xeb60, 'UnicodeCharacter0xeb60', [], 'Display', None],
  [ 0xeb61, 'UnicodeCharacter0xeb61', [], 'Display', None],
  [ 0xeb62, 'UnicodeCharacter0xeb62', [], 'Display', None],
  [ 0xeb63, 'UnicodeCharacter0xeb63', [], 'Display', None],
  [ 0xeb64, 'UnicodeCharacter0xeb64', [], 'Display', None],
  [ 0xeb65, 'UnicodeCharacter0xeb65', [], 'Display', None],
  [ 0xeb66, 'UnicodeCharacter0xeb66', [], 'Display', None],
  [ 0xeb67, 'UnicodeCharacter0xeb67', [], 'Display', None],
  [ 0xeb68, 'UnicodeCharacter0xeb68', [], 'Display', None],
  [ 0xeb69, 'UnicodeCharacter0xeb69', [], 'Display', None],
  [ 0xeb6a, 'UnicodeCharacter0xeb6a', [], 'Display', None],
  [ 0xeb6b, 'UnicodeCharacter0xeb6b', [], 'Display', None],
  [ 0xeb6c, 'UnicodeCharacter0xeb6c', [], 'Display', None],
  [ 0xeb6d, 'UnicodeCharacter0xeb6d', [], 'Display', None],
  [ 0xeb6e, 'UnicodeCharacter0xeb6e', [], 'Display', None],
  [ 0xeb6f, 'UnicodeCharacter0xeb6f', [], 'Display', None],
  [ 0xeb70, 'UnicodeCharacter0xeb70', [], 'Display', None],
  [ 0xeb71, 'UnicodeCharacter0xeb71', [], 'Display', None],
  [ 0xeb72, 'UnicodeCharacter0xeb72', [], 'Display', None],
  [ 0xeb73, 'UnicodeCharacter0xeb73', [], 'Display', None],
  [ 0xeb74, 'UnicodeCharacter0xeb74', [], 'Display', None],
  [ 0xeb75, 'UnicodeCharacter0xeb75', [], 'Display', None],
  [ 0xeb76, 'UnicodeCharacter0xeb76', [], 'Display', None],
  [ 0xeb77, 'UnicodeCharacter0xeb77', [], 'Display', None],
  [ 0xeb78, 'UnicodeCharacter0xeb78', [], 'Display', None],
  [ 0xeb79, 'UnicodeCharacter0xeb79', [], 'Display', None],
  [ 0xeb7a, 'UnicodeCharacter0xeb7a', [], 'Display', None],
  [ 0xeb7b, 'UnicodeCharacter0xeb7b', [], 'Display', None],
  [ 0xeb7c, 'UnicodeCharacter0xeb7c', [], 'Display', None],
  [ 0xeb7d, 'UnicodeCharacter0xeb7d', [], 'Display', None],
  [ 0xeb7e, 'UnicodeCharacter0xeb7e', [], 'Display', None],
  [ 0xeb7f, 'UnicodeCharacter0xeb7f', [], 'Display', None],
  [ 0xeb80, 'UnicodeCharacter0xeb80', [], 'Display', None],
  [ 0xeb81, 'UnicodeCharacter0xeb81', [], 'Display', None],
  [ 0xeb82, 'UnicodeCharacter0xeb82', [], 'Display', None],
  [ 0xeb83, 'UnicodeCharacter0xeb83', [], 'Display', None],
  [ 0xeb84, 'UnicodeCharacter0xeb84', [], 'Display', None],
  [ 0xeb85, 'UnicodeCharacter0xeb85', [], 'Display', None],
  [ 0xeb86, 'UnicodeCharacter0xeb86', [], 'Display', None],
  [ 0xeb87, 'UnicodeCharacter0xeb87', [], 'Display', None],
  [ 0xeb88, 'UnicodeCharacter0xeb88', [], 'Display', None],
  [ 0xeb89, 'UnicodeCharacter0xeb89', [], 'Display', None],
  [ 0xeb8a, 'UnicodeCharacter0xeb8a', [], 'Display', None],
  [ 0xeb8b, 'UnicodeCharacter0xeb8b', [], 'Display', None],
  [ 0xeb8c, 'UnicodeCharacter0xeb8c', [], 'Display', None],
  [ 0xeb8d, 'UnicodeCharacter0xeb8d', [], 'Display', None],
  [ 0xeb8e, 'UnicodeCharacter0xeb8e', [], 'Display', None],
  [ 0xeb8f, 'UnicodeCharacter0xeb8f', [], 'Display', None],
  [ 0xeb90, 'UnicodeCharacter0xeb90', [], 'Display', None],
  [ 0xeb91, 'UnicodeCharacter0xeb91', [], 'Display', None],
  [ 0xeb92, 'UnicodeCharacter0xeb92', [], 'Display', None],
  [ 0xeb93, 'UnicodeCharacter0xeb93', [], 'Display', None],
  [ 0xeb94, 'UnicodeCharacter0xeb94', [], 'Display', None],
  [ 0xeb95, 'UnicodeCharacter0xeb95', [], 'Display', None],
  [ 0xeb96, 'UnicodeCharacter0xeb96', [], 'Display', None],
  [ 0xeb97, 'UnicodeCharacter0xeb97', [], 'Display', None],
  [ 0xeb98, 'UnicodeCharacter0xeb98', [], 'Display', None],
  [ 0xeb99, 'UnicodeCharacter0xeb99', [], 'Display', None],
  [ 0xeb9a, 'UnicodeCharacter0xeb9a', [], 'Display', None],
  [ 0xeb9b, 'UnicodeCharacter0xeb9b', [], 'Display', None],
  [ 0xeb9c, 'UnicodeCharacter0xeb9c', [], 'Display', None],
  [ 0xeb9d, 'UnicodeCharacter0xeb9d', [], 'Display', None],
  [ 0xeb9e, 'UnicodeCharacter0xeb9e', [], 'Display', None],
  [ 0xeb9f, 'UnicodeCharacter0xeb9f', [], 'Display', None],
  [ 0xeba0, 'UnicodeCharacter0xeba0', [], 'Display', None],
  [ 0xeba1, 'UnicodeCharacter0xeba1', [], 'Display', None],
  [ 0xeba2, 'UnicodeCharacter0xeba2', [], 'Display', None],
  [ 0xeba3, 'UnicodeCharacter0xeba3', [], 'Display', None],
  [ 0xeba4, 'UnicodeCharacter0xeba4', [], 'Display', None],
  [ 0xeba5, 'UnicodeCharacter0xeba5', [], 'Display', None],
  [ 0xeba6, 'UnicodeCharacter0xeba6', [], 'Display', None],
  [ 0xeba7, 'UnicodeCharacter0xeba7', [], 'Display', None],
  [ 0xeba8, 'UnicodeCharacter0xeba8', [], 'Display', None],
  [ 0xeba9, 'UnicodeCharacter0xeba9', [], 'Display', None],
  [ 0xebaa, 'UnicodeCharacter0xebaa', [], 'Display', None],
  [ 0xebab, 'UnicodeCharacter0xebab', [], 'Display', None],
  [ 0xebac, 'UnicodeCharacter0xebac', [], 'Display', None],
  [ 0xebad, 'UnicodeCharacter0xebad', [], 'Display', None],
  [ 0xebae, 'UnicodeCharacter0xebae', [], 'Display', None],
  [ 0xebaf, 'UnicodeCharacter0xebaf', [], 'Display', None],
  [ 0xebb0, 'UnicodeCharacter0xebb0', [], 'Display', None],
  [ 0xebb1, 'UnicodeCharacter0xebb1', [], 'Display', None],
  [ 0xebb2, 'UnicodeCharacter0xebb2', [], 'Display', None],
  [ 0xebb3, 'UnicodeCharacter0xebb3', [], 'Display', None],
  [ 0xebb4, 'UnicodeCharacter0xebb4', [], 'Display', None],
  [ 0xebb5, 'UnicodeCharacter0xebb5', [], 'Display', None],
  [ 0xebb6, 'UnicodeCharacter0xebb6', [], 'Display', None],
  [ 0xebb7, 'UnicodeCharacter0xebb7', [], 'Display', None],
  [ 0xebb8, 'UnicodeCharacter0xebb8', [], 'Display', None],
  [ 0xebb9, 'UnicodeCharacter0xebb9', [], 'Display', None],
  [ 0xebba, 'UnicodeCharacter0xebba', [], 'Display', None],
  [ 0xebbb, 'UnicodeCharacter0xebbb', [], 'Display', None],
  [ 0xebbc, 'UnicodeCharacter0xebbc', [], 'Display', None],
  [ 0xebbd, 'UnicodeCharacter0xebbd', [], 'Display', None],
  [ 0xebbe, 'UnicodeCharacter0xebbe', [], 'Display', None],
  [ 0xebbf, 'UnicodeCharacter0xebbf', [], 'Display', None],
  [ 0xebc0, 'UnicodeCharacter0xebc0', [], 'Display', None],
  [ 0xebc1, 'UnicodeCharacter0xebc1', [], 'Display', None],
  [ 0xebc2, 'UnicodeCharacter0xebc2', [], 'Display', None],
  [ 0xebc3, 'UnicodeCharacter0xebc3', [], 'Display', None],
  [ 0xebc4, 'UnicodeCharacter0xebc4', [], 'Display', None],
  [ 0xebc5, 'UnicodeCharacter0xebc5', [], 'Display', None],
  [ 0xebc6, 'UnicodeCharacter0xebc6', [], 'Display', None],
  [ 0xebc7, 'UnicodeCharacter0xebc7', [], 'Display', None],
  [ 0xebc8, 'UnicodeCharacter0xebc8', [], 'Display', None],
  [ 0xebc9, 'UnicodeCharacter0xebc9', [], 'Display', None],
  [ 0xebca, 'UnicodeCharacter0xebca', [], 'Display', None],
  [ 0xebcb, 'UnicodeCharacter0xebcb', [], 'Display', None],
  [ 0xebcc, 'UnicodeCharacter0xebcc', [], 'Display', None],
  [ 0xebcd, 'UnicodeCharacter0xebcd', [], 'Display', None],
  [ 0xebce, 'UnicodeCharacter0xebce', [], 'Display', None],
  [ 0xebcf, 'UnicodeCharacter0xebcf', [], 'Display', None],
  [ 0xebd0, 'UnicodeCharacter0xebd0', [], 'Display', None],
  [ 0xebd1, 'UnicodeCharacter0xebd1', [], 'Display', None],
  [ 0xebd2, 'UnicodeCharacter0xebd2', [], 'Display', None],
  [ 0xebd3, 'UnicodeCharacter0xebd3', [], 'Display', None],
  [ 0xebd4, 'UnicodeCharacter0xebd4', [], 'Display', None],
  [ 0xebd5, 'UnicodeCharacter0xebd5', [], 'Display', None],
  [ 0xebd6, 'UnicodeCharacter0xebd6', [], 'Display', None],
  [ 0xebd7, 'UnicodeCharacter0xebd7', [], 'Display', None],
  [ 0xebd8, 'UnicodeCharacter0xebd8', [], 'Display', None],
  [ 0xebd9, 'UnicodeCharacter0xebd9', [], 'Display', None],
  [ 0xebda, 'UnicodeCharacter0xebda', [], 'Display', None],
  [ 0xebdb, 'UnicodeCharacter0xebdb', [], 'Display', None],
  [ 0xebdc, 'UnicodeCharacter0xebdc', [], 'Display', None],
  [ 0xebdd, 'UnicodeCharacter0xebdd', [], 'Display', None],
  [ 0xebde, 'UnicodeCharacter0xebde', [], 'Display', None],
  [ 0xebdf, 'UnicodeCharacter0xebdf', [], 'Display', None],
  [ 0xebe0, 'UnicodeCharacter0xebe0', [], 'Display', None],
  [ 0xebe1, 'UnicodeCharacter0xebe1', [], 'Display', None],
  [ 0xebe2, 'UnicodeCharacter0xebe2', [], 'Display', None],
  [ 0xebe3, 'UnicodeCharacter0xebe3', [], 'Display', None],
  [ 0xebe4, 'UnicodeCharacter0xebe4', [], 'Display', None],
  [ 0xebe5, 'UnicodeCharacter0xebe5', [], 'Display', None],
  [ 0xebe6, 'UnicodeCharacter0xebe6', [], 'Display', None],
  [ 0xebe7, 'UnicodeCharacter0xebe7', [], 'Display', None],
  [ 0xebe8, 'UnicodeCharacter0xebe8', [], 'Display', None],
  [ 0xebe9, 'UnicodeCharacter0xebe9', [], 'Display', None],
  [ 0xebea, 'UnicodeCharacter0xebea', [], 'Display', None],
  [ 0xebeb, 'UnicodeCharacter0xebeb', [], 'Display', None],
  [ 0xebec, 'UnicodeCharacter0xebec', [], 'Display', None],
  [ 0xebed, 'UnicodeCharacter0xebed', [], 'Display', None],
  [ 0xebee, 'UnicodeCharacter0xebee', [], 'Display', None],
  [ 0xebef, 'UnicodeCharacter0xebef', [], 'Display', None],
  [ 0xebf0, 'UnicodeCharacter0xebf0', [], 'Display', None],
  [ 0xebf1, 'UnicodeCharacter0xebf1', [], 'Display', None],
  [ 0xebf2, 'UnicodeCharacter0xebf2', [], 'Display', None],
  [ 0xebf3, 'UnicodeCharacter0xebf3', [], 'Display', None],
  [ 0xebf4, 'UnicodeCharacter0xebf4', [], 'Display', None],
  [ 0xebf5, 'UnicodeCharacter0xebf5', [], 'Display', None],
  [ 0xebf6, 'UnicodeCharacter0xebf6', [], 'Display', None],
  [ 0xebf7, 'UnicodeCharacter0xebf7', [], 'Display', None],
  [ 0xebf8, 'UnicodeCharacter0xebf8', [], 'Display', None],
  [ 0xebf9, 'UnicodeCharacter0xebf9', [], 'Display', None],
  [ 0xebfa, 'UnicodeCharacter0xebfa', [], 'Display', None],
  [ 0xebfb, 'UnicodeCharacter0xebfb', [], 'Display', None],
  [ 0xebfc, 'UnicodeCharacter0xebfc', [], 'Display', None],
  [ 0xebfd, 'UnicodeCharacter0xebfd', [], 'Display', None],
  [ 0xebfe, 'UnicodeCharacter0xebfe', [], 'Display', None],
  [ 0xebff, 'UnicodeCharacter0xebff', [], 'Display', None],
  [ 0xec00, 'UnicodeCharacter0xec00', [], 'Display', None],
  [ 0xec01, 'UnicodeCharacter0xec01', [], 'Display', None],
  [ 0xec02, 'UnicodeCharacter0xec02', [], 'Display', None],
  [ 0xec03, 'UnicodeCharacter0xec03', [], 'Display', None],
  [ 0xec04, 'UnicodeCharacter0xec04', [], 'Display', None],
  [ 0xec05, 'UnicodeCharacter0xec05', [], 'Display', None],
  [ 0xec06, 'UnicodeCharacter0xec06', [], 'Display', None],
  [ 0xec07, 'UnicodeCharacter0xec07', [], 'Display', None],
  [ 0xec08, 'UnicodeCharacter0xec08', [], 'Display', None],
  [ 0xec09, 'UnicodeCharacter0xec09', [], 'Display', None],
  [ 0xec0a, 'UnicodeCharacter0xec0a', [], 'Display', None],
  [ 0xec0b, 'UnicodeCharacter0xec0b', [], 'Display', None],
  [ 0xec0c, 'UnicodeCharacter0xec0c', [], 'Display', None],
  [ 0xec0d, 'UnicodeCharacter0xec0d', [], 'Display', None],
  [ 0xec0e, 'UnicodeCharacter0xec0e', [], 'Display', None],
  [ 0xec0f, 'UnicodeCharacter0xec0f', [], 'Display', None],
  [ 0xec10, 'UnicodeCharacter0xec10', [], 'Display', None],
  [ 0xec11, 'UnicodeCharacter0xec11', [], 'Display', None],
  [ 0xec12, 'UnicodeCharacter0xec12', [], 'Display', None],
  [ 0xec13, 'UnicodeCharacter0xec13', [], 'Display', None],
  [ 0xec14, 'UnicodeCharacter0xec14', [], 'Display', None],
  [ 0xec15, 'UnicodeCharacter0xec15', [], 'Display', None],
  [ 0xec16, 'UnicodeCharacter0xec16', [], 'Display', None],
  [ 0xec17, 'UnicodeCharacter0xec17', [], 'Display', None],
  [ 0xec18, 'UnicodeCharacter0xec18', [], 'Display', None],
  [ 0xec19, 'UnicodeCharacter0xec19', [], 'Display', None],
  [ 0xec1a, 'UnicodeCharacter0xec1a', [], 'Display', None],
  [ 0xec1b, 'UnicodeCharacter0xec1b', [], 'Display', None],
  [ 0xec1c, 'UnicodeCharacter0xec1c', [], 'Display', None],
  [ 0xec1d, 'UnicodeCharacter0xec1d', [], 'Display', None],
  [ 0xec1e, 'UnicodeCharacter0xec1e', [], 'Display', None],
  [ 0xec1f, 'UnicodeCharacter0xec1f', [], 'Display', None],
  [ 0xec20, 'UnicodeCharacter0xec20', [], 'Display', None],
  [ 0xec21, 'UnicodeCharacter0xec21', [], 'Display', None],
  [ 0xec22, 'UnicodeCharacter0xec22', [], 'Display', None],
  [ 0xec23, 'UnicodeCharacter0xec23', [], 'Display', None],
  [ 0xec24, 'UnicodeCharacter0xec24', [], 'Display', None],
  [ 0xec25, 'UnicodeCharacter0xec25', [], 'Display', None],
  [ 0xec26, 'UnicodeCharacter0xec26', [], 'Display', None],
  [ 0xec27, 'UnicodeCharacter0xec27', [], 'Display', None],
  [ 0xec28, 'UnicodeCharacter0xec28', [], 'Display', None],
  [ 0xec29, 'UnicodeCharacter0xec29', [], 'Display', None],
  [ 0xec2a, 'UnicodeCharacter0xec2a', [], 'Display', None],
  [ 0xec2b, 'UnicodeCharacter0xec2b', [], 'Display', None],
  [ 0xec2c, 'UnicodeCharacter0xec2c', [], 'Display', None],
  [ 0xec2d, 'UnicodeCharacter0xec2d', [], 'Display', None],
  [ 0xec2e, 'UnicodeCharacter0xec2e', [], 'Display', None],
  [ 0xec2f, 'UnicodeCharacter0xec2f', [], 'Display', None],
  [ 0xec30, 'UnicodeCharacter0xec30', [], 'Display', None],
  [ 0xec31, 'UnicodeCharacter0xec31', [], 'Display', None],
  [ 0xec32, 'UnicodeCharacter0xec32', [], 'Display', None],
  [ 0xec33, 'UnicodeCharacter0xec33', [], 'Display', None],
  [ 0xec34, 'UnicodeCharacter0xec34', [], 'Display', None],
  [ 0xec35, 'UnicodeCharacter0xec35', [], 'Display', None],
  [ 0xec36, 'UnicodeCharacter0xec36', [], 'Display', None],
  [ 0xec37, 'UnicodeCharacter0xec37', [], 'Display', None],
  [ 0xec38, 'UnicodeCharacter0xec38', [], 'Display', None],
  [ 0xec39, 'UnicodeCharacter0xec39', [], 'Display', None],
  [ 0xec3a, 'UnicodeCharacter0xec3a', [], 'Display', None],
  [ 0xec3b, 'UnicodeCharacter0xec3b', [], 'Display', None],
  [ 0xec3c, 'UnicodeCharacter0xec3c', [], 'Display', None],
  [ 0xec3d, 'UnicodeCharacter0xec3d', [], 'Display', None],
  [ 0xec3e, 'UnicodeCharacter0xec3e', [], 'Display', None],
  [ 0xec3f, 'UnicodeCharacter0xec3f', [], 'Display', None],
  [ 0xec40, 'UnicodeCharacter0xec40', [], 'Display', None],
  [ 0xec41, 'UnicodeCharacter0xec41', [], 'Display', None],
  [ 0xec42, 'UnicodeCharacter0xec42', [], 'Display', None],
  [ 0xec43, 'UnicodeCharacter0xec43', [], 'Display', None],
  [ 0xec44, 'UnicodeCharacter0xec44', [], 'Display', None],
  [ 0xec45, 'UnicodeCharacter0xec45', [], 'Display', None],
  [ 0xec46, 'UnicodeCharacter0xec46', [], 'Display', None],
  [ 0xec47, 'UnicodeCharacter0xec47', [], 'Display', None],
  [ 0xec48, 'UnicodeCharacter0xec48', [], 'Display', None],
  [ 0xec49, 'UnicodeCharacter0xec49', [], 'Display', None],
  [ 0xec4a, 'UnicodeCharacter0xec4a', [], 'Display', None],
  [ 0xec4b, 'UnicodeCharacter0xec4b', [], 'Display', None],
  [ 0xec4c, 'UnicodeCharacter0xec4c', [], 'Display', None],
  [ 0xec4d, 'UnicodeCharacter0xec4d', [], 'Display', None],
  [ 0xec4e, 'UnicodeCharacter0xec4e', [], 'Display', None],
  [ 0xec4f, 'UnicodeCharacter0xec4f', [], 'Display', None],
  [ 0xec50, 'UnicodeCharacter0xec50', [], 'Display', None],
  [ 0xec51, 'UnicodeCharacter0xec51', [], 'Display', None],
  [ 0xec52, 'UnicodeCharacter0xec52', [], 'Display', None],
  [ 0xec53, 'UnicodeCharacter0xec53', [], 'Display', None],
  [ 0xec54, 'UnicodeCharacter0xec54', [], 'Display', None],
  [ 0xec55, 'UnicodeCharacter0xec55', [], 'Display', None],
  [ 0xec56, 'UnicodeCharacter0xec56', [], 'Display', None],
  [ 0xec57, 'UnicodeCharacter0xec57', [], 'Display', None],
  [ 0xec58, 'UnicodeCharacter0xec58', [], 'Display', None],
  [ 0xec59, 'UnicodeCharacter0xec59', [], 'Display', None],
  [ 0xec5a, 'UnicodeCharacter0xec5a', [], 'Display', None],
  [ 0xec5b, 'UnicodeCharacter0xec5b', [], 'Display', None],
  [ 0xec5c, 'UnicodeCharacter0xec5c', [], 'Display', None],
  [ 0xec5d, 'UnicodeCharacter0xec5d', [], 'Display', None],
  [ 0xec5e, 'UnicodeCharacter0xec5e', [], 'Display', None],
  [ 0xec5f, 'UnicodeCharacter0xec5f', [], 'Display', None],
  [ 0xec60, 'UnicodeCharacter0xec60', [], 'Display', None],
  [ 0xec61, 'UnicodeCharacter0xec61', [], 'Display', None],
  [ 0xec62, 'UnicodeCharacter0xec62', [], 'Display', None],
  [ 0xec63, 'UnicodeCharacter0xec63', [], 'Display', None],
  [ 0xec64, 'UnicodeCharacter0xec64', [], 'Display', None],
  [ 0xec65, 'UnicodeCharacter0xec65', [], 'Display', None],
  [ 0xec66, 'UnicodeCharacter0xec66', [], 'Display', None],
  [ 0xec67, 'UnicodeCharacter0xec67', [], 'Display', None],
  [ 0xec68, 'UnicodeCharacter0xec68', [], 'Display', None],
  [ 0xec69, 'UnicodeCharacter0xec69', [], 'Display', None],
  [ 0xec6a, 'UnicodeCharacter0xec6a', [], 'Display', None],
  [ 0xec6b, 'UnicodeCharacter0xec6b', [], 'Display', None],
  [ 0xec6c, 'UnicodeCharacter0xec6c', [], 'Display', None],
  [ 0xec6d, 'UnicodeCharacter0xec6d', [], 'Display', None],
  [ 0xec6e, 'UnicodeCharacter0xec6e', [], 'Display', None],
  [ 0xec6f, 'UnicodeCharacter0xec6f', [], 'Display', None],
  [ 0xec70, 'UnicodeCharacter0xec70', [], 'Display', None],
  [ 0xec71, 'UnicodeCharacter0xec71', [], 'Display', None],
  [ 0xec72, 'UnicodeCharacter0xec72', [], 'Display', None],
  [ 0xec73, 'UnicodeCharacter0xec73', [], 'Display', None],
  [ 0xec74, 'UnicodeCharacter0xec74', [], 'Display', None],
  [ 0xec75, 'UnicodeCharacter0xec75', [], 'Display', None],
  [ 0xec76, 'UnicodeCharacter0xec76', [], 'Display', None],
  [ 0xec77, 'UnicodeCharacter0xec77', [], 'Display', None],
  [ 0xec78, 'UnicodeCharacter0xec78', [], 'Display', None],
  [ 0xec79, 'UnicodeCharacter0xec79', [], 'Display', None],
  [ 0xec7a, 'UnicodeCharacter0xec7a', [], 'Display', None],
  [ 0xec7b, 'UnicodeCharacter0xec7b', [], 'Display', None],
  [ 0xec7c, 'UnicodeCharacter0xec7c', [], 'Display', None],
  [ 0xec7d, 'UnicodeCharacter0xec7d', [], 'Display', None],
  [ 0xec7e, 'UnicodeCharacter0xec7e', [], 'Display', None],
  [ 0xec7f, 'UnicodeCharacter0xec7f', [], 'Display', None],
  [ 0xec80, 'UnicodeCharacter0xec80', [], 'Display', None],
  [ 0xec81, 'UnicodeCharacter0xec81', [], 'Display', None],
  [ 0xec82, 'UnicodeCharacter0xec82', [], 'Display', None],
  [ 0xec83, 'UnicodeCharacter0xec83', [], 'Display', None],
  [ 0xec84, 'UnicodeCharacter0xec84', [], 'Display', None],
  [ 0xec85, 'UnicodeCharacter0xec85', [], 'Display', None],
  [ 0xec86, 'UnicodeCharacter0xec86', [], 'Display', None],
  [ 0xec87, 'UnicodeCharacter0xec87', [], 'Display', None],
  [ 0xec88, 'UnicodeCharacter0xec88', [], 'Display', None],
  [ 0xec89, 'UnicodeCharacter0xec89', [], 'Display', None],
  [ 0xec8a, 'UnicodeCharacter0xec8a', [], 'Display', None],
  [ 0xec8b, 'UnicodeCharacter0xec8b', [], 'Display', None],
  [ 0xec8c, 'UnicodeCharacter0xec8c', [], 'Display', None],
  [ 0xec8d, 'UnicodeCharacter0xec8d', [], 'Display', None],
  [ 0xec8e, 'UnicodeCharacter0xec8e', [], 'Display', None],
  [ 0xec8f, 'UnicodeCharacter0xec8f', [], 'Display', None],
  [ 0xec90, 'UnicodeCharacter0xec90', [], 'Display', None],
  [ 0xec91, 'UnicodeCharacter0xec91', [], 'Display', None],
  [ 0xec92, 'UnicodeCharacter0xec92', [], 'Display', None],
  [ 0xec93, 'UnicodeCharacter0xec93', [], 'Display', None],
  [ 0xec94, 'UnicodeCharacter0xec94', [], 'Display', None],
  [ 0xec95, 'UnicodeCharacter0xec95', [], 'Display', None],
  [ 0xec96, 'UnicodeCharacter0xec96', [], 'Display', None],
  [ 0xec97, 'UnicodeCharacter0xec97', [], 'Display', None],
  [ 0xec98, 'UnicodeCharacter0xec98', [], 'Display', None],
  [ 0xec99, 'UnicodeCharacter0xec99', [], 'Display', None],
  [ 0xec9a, 'UnicodeCharacter0xec9a', [], 'Display', None],
  [ 0xec9b, 'UnicodeCharacter0xec9b', [], 'Display', None],
  [ 0xec9c, 'UnicodeCharacter0xec9c', [], 'Display', None],
  [ 0xec9d, 'UnicodeCharacter0xec9d', [], 'Display', None],
  [ 0xec9e, 'UnicodeCharacter0xec9e', [], 'Display', None],
  [ 0xec9f, 'UnicodeCharacter0xec9f', [], 'Display', None],
  [ 0xeca0, 'UnicodeCharacter0xeca0', [], 'Display', None],
  [ 0xeca1, 'UnicodeCharacter0xeca1', [], 'Display', None],
  [ 0xeca2, 'UnicodeCharacter0xeca2', [], 'Display', None],
  [ 0xeca3, 'UnicodeCharacter0xeca3', [], 'Display', None],
  [ 0xeca4, 'UnicodeCharacter0xeca4', [], 'Display', None],
  [ 0xeca5, 'UnicodeCharacter0xeca5', [], 'Display', None],
  [ 0xeca6, 'UnicodeCharacter0xeca6', [], 'Display', None],
  [ 0xeca7, 'UnicodeCharacter0xeca7', [], 'Display', None],
  [ 0xeca8, 'UnicodeCharacter0xeca8', [], 'Display', None],
  [ 0xeca9, 'UnicodeCharacter0xeca9', [], 'Display', None],
  [ 0xecaa, 'UnicodeCharacter0xecaa', [], 'Display', None],
  [ 0xecab, 'UnicodeCharacter0xecab', [], 'Display', None],
  [ 0xecac, 'UnicodeCharacter0xecac', [], 'Display', None],
  [ 0xecad, 'UnicodeCharacter0xecad', [], 'Display', None],
  [ 0xecae, 'UnicodeCharacter0xecae', [], 'Display', None],
  [ 0xecaf, 'UnicodeCharacter0xecaf', [], 'Display', None],
  [ 0xecb0, 'UnicodeCharacter0xecb0', [], 'Display', None],
  [ 0xecb1, 'UnicodeCharacter0xecb1', [], 'Display', None],
  [ 0xecb2, 'UnicodeCharacter0xecb2', [], 'Display', None],
  [ 0xecb3, 'UnicodeCharacter0xecb3', [], 'Display', None],
  [ 0xecb4, 'UnicodeCharacter0xecb4', [], 'Display', None],
  [ 0xecb5, 'UnicodeCharacter0xecb5', [], 'Display', None],
  [ 0xecb6, 'UnicodeCharacter0xecb6', [], 'Display', None],
  [ 0xecb7, 'UnicodeCharacter0xecb7', [], 'Display', None],
  [ 0xecb8, 'UnicodeCharacter0xecb8', [], 'Display', None],
  [ 0xecb9, 'UnicodeCharacter0xecb9', [], 'Display', None],
  [ 0xecba, 'UnicodeCharacter0xecba', [], 'Display', None],
  [ 0xecbb, 'UnicodeCharacter0xecbb', [], 'Display', None],
  [ 0xecbc, 'UnicodeCharacter0xecbc', [], 'Display', None],
  [ 0xecbd, 'UnicodeCharacter0xecbd', [], 'Display', None],
  [ 0xecbe, 'UnicodeCharacter0xecbe', [], 'Display', None],
  [ 0xecbf, 'UnicodeCharacter0xecbf', [], 'Display', None],
  [ 0xecc0, 'UnicodeCharacter0xecc0', [], 'Display', None],
  [ 0xecc1, 'UnicodeCharacter0xecc1', [], 'Display', None],
  [ 0xecc2, 'UnicodeCharacter0xecc2', [], 'Display', None],
  [ 0xecc3, 'UnicodeCharacter0xecc3', [], 'Display', None],
  [ 0xecc4, 'UnicodeCharacter0xecc4', [], 'Display', None],
  [ 0xecc5, 'UnicodeCharacter0xecc5', [], 'Display', None],
  [ 0xecc6, 'UnicodeCharacter0xecc6', [], 'Display', None],
  [ 0xecc7, 'UnicodeCharacter0xecc7', [], 'Display', None],
  [ 0xecc8, 'UnicodeCharacter0xecc8', [], 'Display', None],
  [ 0xecc9, 'UnicodeCharacter0xecc9', [], 'Display', None],
  [ 0xecca, 'UnicodeCharacter0xecca', [], 'Display', None],
  [ 0xeccb, 'UnicodeCharacter0xeccb', [], 'Display', None],
  [ 0xeccc, 'UnicodeCharacter0xeccc', [], 'Display', None],
  [ 0xeccd, 'UnicodeCharacter0xeccd', [], 'Display', None],
  [ 0xecce, 'UnicodeCharacter0xecce', [], 'Display', None],
  [ 0xeccf, 'UnicodeCharacter0xeccf', [], 'Display', None],
  [ 0xecd0, 'UnicodeCharacter0xecd0', [], 'Display', None],
  [ 0xecd1, 'UnicodeCharacter0xecd1', [], 'Display', None],
  [ 0xecd2, 'UnicodeCharacter0xecd2', [], 'Display', None],
  [ 0xecd3, 'UnicodeCharacter0xecd3', [], 'Display', None],
  [ 0xecd4, 'UnicodeCharacter0xecd4', [], 'Display', None],
  [ 0xecd5, 'UnicodeCharacter0xecd5', [], 'Display', None],
  [ 0xecd6, 'UnicodeCharacter0xecd6', [], 'Display', None],
  [ 0xecd7, 'UnicodeCharacter0xecd7', [], 'Display', None],
  [ 0xecd8, 'UnicodeCharacter0xecd8', [], 'Display', None],
  [ 0xecd9, 'UnicodeCharacter0xecd9', [], 'Display', None],
  [ 0xecda, 'UnicodeCharacter0xecda', [], 'Display', None],
  [ 0xecdb, 'UnicodeCharacter0xecdb', [], 'Display', None],
  [ 0xecdc, 'UnicodeCharacter0xecdc', [], 'Display', None],
  [ 0xecdd, 'UnicodeCharacter0xecdd', [], 'Display', None],
  [ 0xecde, 'UnicodeCharacter0xecde', [], 'Display', None],
  [ 0xecdf, 'UnicodeCharacter0xecdf', [], 'Display', None],
  [ 0xece0, 'UnicodeCharacter0xece0', [], 'Display', None],
  [ 0xece1, 'UnicodeCharacter0xece1', [], 'Display', None],
  [ 0xece2, 'UnicodeCharacter0xece2', [], 'Display', None],
  [ 0xece3, 'UnicodeCharacter0xece3', [], 'Display', None],
  [ 0xece4, 'UnicodeCharacter0xece4', [], 'Display', None],
  [ 0xece5, 'UnicodeCharacter0xece5', [], 'Display', None],
  [ 0xece6, 'UnicodeCharacter0xece6', [], 'Display', None],
  [ 0xece7, 'UnicodeCharacter0xece7', [], 'Display', None],
  [ 0xece8, 'UnicodeCharacter0xece8', [], 'Display', None],
  [ 0xece9, 'UnicodeCharacter0xece9', [], 'Display', None],
  [ 0xecea, 'UnicodeCharacter0xecea', [], 'Display', None],
  [ 0xeceb, 'UnicodeCharacter0xeceb', [], 'Display', None],
  [ 0xecec, 'UnicodeCharacter0xecec', [], 'Display', None],
  [ 0xeced, 'UnicodeCharacter0xeced', [], 'Display', None],
  [ 0xecee, 'UnicodeCharacter0xecee', [], 'Display', None],
  [ 0xecef, 'UnicodeCharacter0xecef', [], 'Display', None],
  [ 0xecf0, 'UnicodeCharacter0xecf0', [], 'Display', None],
  [ 0xecf1, 'UnicodeCharacter0xecf1', [], 'Display', None],
  [ 0xecf2, 'UnicodeCharacter0xecf2', [], 'Display', None],
  [ 0xecf3, 'UnicodeCharacter0xecf3', [], 'Display', None],
  [ 0xecf4, 'UnicodeCharacter0xecf4', [], 'Display', None],
  [ 0xecf5, 'UnicodeCharacter0xecf5', [], 'Display', None],
  [ 0xecf6, 'UnicodeCharacter0xecf6', [], 'Display', None],
  [ 0xecf7, 'UnicodeCharacter0xecf7', [], 'Display', None],
  [ 0xecf8, 'UnicodeCharacter0xecf8', [], 'Display', None],
  [ 0xecf9, 'UnicodeCharacter0xecf9', [], 'Display', None],
  [ 0xecfa, 'UnicodeCharacter0xecfa', [], 'Display', None],
  [ 0xecfb, 'UnicodeCharacter0xecfb', [], 'Display', None],
  [ 0xecfc, 'UnicodeCharacter0xecfc', [], 'Display', None],
  [ 0xecfd, 'UnicodeCharacter0xecfd', [], 'Display', None],
  [ 0xecfe, 'UnicodeCharacter0xecfe', [], 'Display', None],
  [ 0xecff, 'UnicodeCharacter0xecff', [], 'Display', None],
  [ 0xed00, 'UnicodeCharacter0xed00', [], 'Display', None],
  [ 0xed01, 'UnicodeCharacter0xed01', [], 'Display', None],
  [ 0xed02, 'UnicodeCharacter0xed02', [], 'Display', None],
  [ 0xed03, 'UnicodeCharacter0xed03', [], 'Display', None],
  [ 0xed04, 'UnicodeCharacter0xed04', [], 'Display', None],
  [ 0xed05, 'UnicodeCharacter0xed05', [], 'Display', None],
  [ 0xed06, 'UnicodeCharacter0xed06', [], 'Display', None],
  [ 0xed07, 'UnicodeCharacter0xed07', [], 'Display', None],
  [ 0xed08, 'UnicodeCharacter0xed08', [], 'Display', None],
  [ 0xed09, 'UnicodeCharacter0xed09', [], 'Display', None],
  [ 0xed0a, 'UnicodeCharacter0xed0a', [], 'Display', None],
  [ 0xed0b, 'UnicodeCharacter0xed0b', [], 'Display', None],
  [ 0xed0c, 'UnicodeCharacter0xed0c', [], 'Display', None],
  [ 0xed0d, 'UnicodeCharacter0xed0d', [], 'Display', None],
  [ 0xed0e, 'UnicodeCharacter0xed0e', [], 'Display', None],
  [ 0xed0f, 'UnicodeCharacter0xed0f', [], 'Display', None],
  [ 0xed10, 'UnicodeCharacter0xed10', [], 'Display', None],
  [ 0xed11, 'UnicodeCharacter0xed11', [], 'Display', None],
  [ 0xed12, 'UnicodeCharacter0xed12', [], 'Display', None],
  [ 0xed13, 'UnicodeCharacter0xed13', [], 'Display', None],
  [ 0xed14, 'UnicodeCharacter0xed14', [], 'Display', None],
  [ 0xed15, 'UnicodeCharacter0xed15', [], 'Display', None],
  [ 0xed16, 'UnicodeCharacter0xed16', [], 'Display', None],
  [ 0xed17, 'UnicodeCharacter0xed17', [], 'Display', None],
  [ 0xed18, 'UnicodeCharacter0xed18', [], 'Display', None],
  [ 0xed19, 'UnicodeCharacter0xed19', [], 'Display', None],
  [ 0xed1a, 'UnicodeCharacter0xed1a', [], 'Display', None],
  [ 0xed1b, 'UnicodeCharacter0xed1b', [], 'Display', None],
  [ 0xed1c, 'UnicodeCharacter0xed1c', [], 'Display', None],
  [ 0xed1d, 'UnicodeCharacter0xed1d', [], 'Display', None],
  [ 0xed1e, 'UnicodeCharacter0xed1e', [], 'Display', None],
  [ 0xed1f, 'UnicodeCharacter0xed1f', [], 'Display', None],
  [ 0xed20, 'UnicodeCharacter0xed20', [], 'Display', None],
  [ 0xed21, 'UnicodeCharacter0xed21', [], 'Display', None],
  [ 0xed22, 'UnicodeCharacter0xed22', [], 'Display', None],
  [ 0xed23, 'UnicodeCharacter0xed23', [], 'Display', None],
  [ 0xed24, 'UnicodeCharacter0xed24', [], 'Display', None],
  [ 0xed25, 'UnicodeCharacter0xed25', [], 'Display', None],
  [ 0xed26, 'UnicodeCharacter0xed26', [], 'Display', None],
  [ 0xed27, 'UnicodeCharacter0xed27', [], 'Display', None],
  [ 0xed28, 'UnicodeCharacter0xed28', [], 'Display', None],
  [ 0xed29, 'UnicodeCharacter0xed29', [], 'Display', None],
  [ 0xed2a, 'UnicodeCharacter0xed2a', [], 'Display', None],
  [ 0xed2b, 'UnicodeCharacter0xed2b', [], 'Display', None],
  [ 0xed2c, 'UnicodeCharacter0xed2c', [], 'Display', None],
  [ 0xed2d, 'UnicodeCharacter0xed2d', [], 'Display', None],
  [ 0xed2e, 'UnicodeCharacter0xed2e', [], 'Display', None],
  [ 0xed2f, 'UnicodeCharacter0xed2f', [], 'Display', None],
  [ 0xed30, 'UnicodeCharacter0xed30', [], 'Display', None],
  [ 0xed31, 'UnicodeCharacter0xed31', [], 'Display', None],
  [ 0xed32, 'UnicodeCharacter0xed32', [], 'Display', None],
  [ 0xed33, 'UnicodeCharacter0xed33', [], 'Display', None],
  [ 0xed34, 'UnicodeCharacter0xed34', [], 'Display', None],
  [ 0xed35, 'UnicodeCharacter0xed35', [], 'Display', None],
  [ 0xed36, 'UnicodeCharacter0xed36', [], 'Display', None],
  [ 0xed37, 'UnicodeCharacter0xed37', [], 'Display', None],
  [ 0xed38, 'UnicodeCharacter0xed38', [], 'Display', None],
  [ 0xed39, 'UnicodeCharacter0xed39', [], 'Display', None],
  [ 0xed3a, 'UnicodeCharacter0xed3a', [], 'Display', None],
  [ 0xed3b, 'UnicodeCharacter0xed3b', [], 'Display', None],
  [ 0xed3c, 'UnicodeCharacter0xed3c', [], 'Display', None],
  [ 0xed3d, 'UnicodeCharacter0xed3d', [], 'Display', None],
  [ 0xed3e, 'UnicodeCharacter0xed3e', [], 'Display', None],
  [ 0xed3f, 'UnicodeCharacter0xed3f', [], 'Display', None],
  [ 0xed40, 'UnicodeCharacter0xed40', [], 'Display', None],
  [ 0xed41, 'UnicodeCharacter0xed41', [], 'Display', None],
  [ 0xed42, 'UnicodeCharacter0xed42', [], 'Display', None],
  [ 0xed43, 'UnicodeCharacter0xed43', [], 'Display', None],
  [ 0xed44, 'UnicodeCharacter0xed44', [], 'Display', None],
  [ 0xed45, 'UnicodeCharacter0xed45', [], 'Display', None],
  [ 0xed46, 'UnicodeCharacter0xed46', [], 'Display', None],
  [ 0xed47, 'UnicodeCharacter0xed47', [], 'Display', None],
  [ 0xed48, 'UnicodeCharacter0xed48', [], 'Display', None],
  [ 0xed49, 'UnicodeCharacter0xed49', [], 'Display', None],
  [ 0xed4a, 'UnicodeCharacter0xed4a', [], 'Display', None],
  [ 0xed4b, 'UnicodeCharacter0xed4b', [], 'Display', None],
  [ 0xed4c, 'UnicodeCharacter0xed4c', [], 'Display', None],
  [ 0xed4d, 'UnicodeCharacter0xed4d', [], 'Display', None],
  [ 0xed4e, 'UnicodeCharacter0xed4e', [], 'Display', None],
  [ 0xed4f, 'UnicodeCharacter0xed4f', [], 'Display', None],
  [ 0xed50, 'UnicodeCharacter0xed50', [], 'Display', None],
  [ 0xed51, 'UnicodeCharacter0xed51', [], 'Display', None],
  [ 0xed52, 'UnicodeCharacter0xed52', [], 'Display', None],
  [ 0xed53, 'UnicodeCharacter0xed53', [], 'Display', None],
  [ 0xed54, 'UnicodeCharacter0xed54', [], 'Display', None],
  [ 0xed55, 'UnicodeCharacter0xed55', [], 'Display', None],
  [ 0xed56, 'UnicodeCharacter0xed56', [], 'Display', None],
  [ 0xed57, 'UnicodeCharacter0xed57', [], 'Display', None],
  [ 0xed58, 'UnicodeCharacter0xed58', [], 'Display', None],
  [ 0xed59, 'UnicodeCharacter0xed59', [], 'Display', None],
  [ 0xed5a, 'UnicodeCharacter0xed5a', [], 'Display', None],
  [ 0xed5b, 'UnicodeCharacter0xed5b', [], 'Display', None],
  [ 0xed5c, 'UnicodeCharacter0xed5c', [], 'Display', None],
  [ 0xed5d, 'UnicodeCharacter0xed5d', [], 'Display', None],
  [ 0xed5e, 'UnicodeCharacter0xed5e', [], 'Display', None],
  [ 0xed5f, 'UnicodeCharacter0xed5f', [], 'Display', None],
  [ 0xed60, 'UnicodeCharacter0xed60', [], 'Display', None],
  [ 0xed61, 'UnicodeCharacter0xed61', [], 'Display', None],
  [ 0xed62, 'UnicodeCharacter0xed62', [], 'Display', None],
  [ 0xed63, 'UnicodeCharacter0xed63', [], 'Display', None],
  [ 0xed64, 'UnicodeCharacter0xed64', [], 'Display', None],
  [ 0xed65, 'UnicodeCharacter0xed65', [], 'Display', None],
  [ 0xed66, 'UnicodeCharacter0xed66', [], 'Display', None],
  [ 0xed67, 'UnicodeCharacter0xed67', [], 'Display', None],
  [ 0xed68, 'UnicodeCharacter0xed68', [], 'Display', None],
  [ 0xed69, 'UnicodeCharacter0xed69', [], 'Display', None],
  [ 0xed6a, 'UnicodeCharacter0xed6a', [], 'Display', None],
  [ 0xed6b, 'UnicodeCharacter0xed6b', [], 'Display', None],
  [ 0xed6c, 'UnicodeCharacter0xed6c', [], 'Display', None],
  [ 0xed6d, 'UnicodeCharacter0xed6d', [], 'Display', None],
  [ 0xed6e, 'UnicodeCharacter0xed6e', [], 'Display', None],
  [ 0xed6f, 'UnicodeCharacter0xed6f', [], 'Display', None],
  [ 0xed70, 'UnicodeCharacter0xed70', [], 'Display', None],
  [ 0xed71, 'UnicodeCharacter0xed71', [], 'Display', None],
  [ 0xed72, 'UnicodeCharacter0xed72', [], 'Display', None],
  [ 0xed73, 'UnicodeCharacter0xed73', [], 'Display', None],
  [ 0xed74, 'UnicodeCharacter0xed74', [], 'Display', None],
  [ 0xed75, 'UnicodeCharacter0xed75', [], 'Display', None],
  [ 0xed76, 'UnicodeCharacter0xed76', [], 'Display', None],
  [ 0xed77, 'UnicodeCharacter0xed77', [], 'Display', None],
  [ 0xed78, 'UnicodeCharacter0xed78', [], 'Display', None],
  [ 0xed79, 'UnicodeCharacter0xed79', [], 'Display', None],
  [ 0xed7a, 'UnicodeCharacter0xed7a', [], 'Display', None],
  [ 0xed7b, 'UnicodeCharacter0xed7b', [], 'Display', None],
  [ 0xed7c, 'UnicodeCharacter0xed7c', [], 'Display', None],
  [ 0xed7d, 'UnicodeCharacter0xed7d', [], 'Display', None],
  [ 0xed7e, 'UnicodeCharacter0xed7e', [], 'Display', None],
  [ 0xed7f, 'UnicodeCharacter0xed7f', [], 'Display', None],
  [ 0xed80, 'UnicodeCharacter0xed80', [], 'Display', None],
  [ 0xed81, 'UnicodeCharacter0xed81', [], 'Display', None],
  [ 0xed82, 'UnicodeCharacter0xed82', [], 'Display', None],
  [ 0xed83, 'UnicodeCharacter0xed83', [], 'Display', None],
  [ 0xed84, 'UnicodeCharacter0xed84', [], 'Display', None],
  [ 0xed85, 'UnicodeCharacter0xed85', [], 'Display', None],
  [ 0xed86, 'UnicodeCharacter0xed86', [], 'Display', None],
  [ 0xed87, 'UnicodeCharacter0xed87', [], 'Display', None],
  [ 0xed88, 'UnicodeCharacter0xed88', [], 'Display', None],
  [ 0xed89, 'UnicodeCharacter0xed89', [], 'Display', None],
  [ 0xed8a, 'UnicodeCharacter0xed8a', [], 'Display', None],
  [ 0xed8b, 'UnicodeCharacter0xed8b', [], 'Display', None],
  [ 0xed8c, 'UnicodeCharacter0xed8c', [], 'Display', None],
  [ 0xed8d, 'UnicodeCharacter0xed8d', [], 'Display', None],
  [ 0xed8e, 'UnicodeCharacter0xed8e', [], 'Display', None],
  [ 0xed8f, 'UnicodeCharacter0xed8f', [], 'Display', None],
  [ 0xed90, 'UnicodeCharacter0xed90', [], 'Display', None],
  [ 0xed91, 'UnicodeCharacter0xed91', [], 'Display', None],
  [ 0xed92, 'UnicodeCharacter0xed92', [], 'Display', None],
  [ 0xed93, 'UnicodeCharacter0xed93', [], 'Display', None],
  [ 0xed94, 'UnicodeCharacter0xed94', [], 'Display', None],
  [ 0xed95, 'UnicodeCharacter0xed95', [], 'Display', None],
  [ 0xed96, 'UnicodeCharacter0xed96', [], 'Display', None],
  [ 0xed97, 'UnicodeCharacter0xed97', [], 'Display', None],
  [ 0xed98, 'UnicodeCharacter0xed98', [], 'Display', None],
  [ 0xed99, 'UnicodeCharacter0xed99', [], 'Display', None],
  [ 0xed9a, 'UnicodeCharacter0xed9a', [], 'Display', None],
  [ 0xed9b, 'UnicodeCharacter0xed9b', [], 'Display', None],
  [ 0xed9c, 'UnicodeCharacter0xed9c', [], 'Display', None],
  [ 0xed9d, 'UnicodeCharacter0xed9d', [], 'Display', None],
  [ 0xed9e, 'UnicodeCharacter0xed9e', [], 'Display', None],
  [ 0xed9f, 'UnicodeCharacter0xed9f', [], 'Display', None],
  [ 0xeda0, 'UnicodeCharacter0xeda0', [], 'Display', None],
  [ 0xeda1, 'UnicodeCharacter0xeda1', [], 'Display', None],
  [ 0xeda2, 'UnicodeCharacter0xeda2', [], 'Display', None],
  [ 0xeda3, 'UnicodeCharacter0xeda3', [], 'Display', None],
  [ 0xeda4, 'UnicodeCharacter0xeda4', [], 'Display', None],
  [ 0xeda5, 'UnicodeCharacter0xeda5', [], 'Display', None],
  [ 0xeda6, 'UnicodeCharacter0xeda6', [], 'Display', None],
  [ 0xeda7, 'UnicodeCharacter0xeda7', [], 'Display', None],
  [ 0xeda8, 'UnicodeCharacter0xeda8', [], 'Display', None],
  [ 0xeda9, 'UnicodeCharacter0xeda9', [], 'Display', None],
  [ 0xedaa, 'UnicodeCharacter0xedaa', [], 'Display', None],
  [ 0xedab, 'UnicodeCharacter0xedab', [], 'Display', None],
  [ 0xedac, 'UnicodeCharacter0xedac', [], 'Display', None],
  [ 0xedad, 'UnicodeCharacter0xedad', [], 'Display', None],
  [ 0xedae, 'UnicodeCharacter0xedae', [], 'Display', None],
  [ 0xedaf, 'UnicodeCharacter0xedaf', [], 'Display', None],
  [ 0xedb0, 'UnicodeCharacter0xedb0', [], 'Display', None],
  [ 0xedb1, 'UnicodeCharacter0xedb1', [], 'Display', None],
  [ 0xedb2, 'UnicodeCharacter0xedb2', [], 'Display', None],
  [ 0xedb3, 'UnicodeCharacter0xedb3', [], 'Display', None],
  [ 0xedb4, 'UnicodeCharacter0xedb4', [], 'Display', None],
  [ 0xedb5, 'UnicodeCharacter0xedb5', [], 'Display', None],
  [ 0xedb6, 'UnicodeCharacter0xedb6', [], 'Display', None],
  [ 0xedb7, 'UnicodeCharacter0xedb7', [], 'Display', None],
  [ 0xedb8, 'UnicodeCharacter0xedb8', [], 'Display', None],
  [ 0xedb9, 'UnicodeCharacter0xedb9', [], 'Display', None],
  [ 0xedba, 'UnicodeCharacter0xedba', [], 'Display', None],
  [ 0xedbb, 'UnicodeCharacter0xedbb', [], 'Display', None],
  [ 0xedbc, 'UnicodeCharacter0xedbc', [], 'Display', None],
  [ 0xedbd, 'UnicodeCharacter0xedbd', [], 'Display', None],
  [ 0xedbe, 'UnicodeCharacter0xedbe', [], 'Display', None],
  [ 0xedbf, 'UnicodeCharacter0xedbf', [], 'Display', None],
  [ 0xedc0, 'UnicodeCharacter0xedc0', [], 'Display', None],
  [ 0xedc1, 'UnicodeCharacter0xedc1', [], 'Display', None],
  [ 0xedc2, 'UnicodeCharacter0xedc2', [], 'Display', None],
  [ 0xedc3, 'UnicodeCharacter0xedc3', [], 'Display', None],
  [ 0xedc4, 'UnicodeCharacter0xedc4', [], 'Display', None],
  [ 0xedc5, 'UnicodeCharacter0xedc5', [], 'Display', None],
  [ 0xedc6, 'UnicodeCharacter0xedc6', [], 'Display', None],
  [ 0xedc7, 'UnicodeCharacter0xedc7', [], 'Display', None],
  [ 0xedc8, 'UnicodeCharacter0xedc8', [], 'Display', None],
  [ 0xedc9, 'UnicodeCharacter0xedc9', [], 'Display', None],
  [ 0xedca, 'UnicodeCharacter0xedca', [], 'Display', None],
  [ 0xedcb, 'UnicodeCharacter0xedcb', [], 'Display', None],
  [ 0xedcc, 'UnicodeCharacter0xedcc', [], 'Display', None],
  [ 0xedcd, 'UnicodeCharacter0xedcd', [], 'Display', None],
  [ 0xedce, 'UnicodeCharacter0xedce', [], 'Display', None],
  [ 0xedcf, 'UnicodeCharacter0xedcf', [], 'Display', None],
  [ 0xedd0, 'UnicodeCharacter0xedd0', [], 'Display', None],
  [ 0xedd1, 'UnicodeCharacter0xedd1', [], 'Display', None],
  [ 0xedd2, 'UnicodeCharacter0xedd2', [], 'Display', None],
  [ 0xedd3, 'UnicodeCharacter0xedd3', [], 'Display', None],
  [ 0xedd4, 'UnicodeCharacter0xedd4', [], 'Display', None],
  [ 0xedd5, 'UnicodeCharacter0xedd5', [], 'Display', None],
  [ 0xedd6, 'UnicodeCharacter0xedd6', [], 'Display', None],
  [ 0xedd7, 'UnicodeCharacter0xedd7', [], 'Display', None],
  [ 0xedd8, 'UnicodeCharacter0xedd8', [], 'Display', None],
  [ 0xedd9, 'UnicodeCharacter0xedd9', [], 'Display', None],
  [ 0xedda, 'UnicodeCharacter0xedda', [], 'Display', None],
  [ 0xeddb, 'UnicodeCharacter0xeddb', [], 'Display', None],
  [ 0xeddc, 'UnicodeCharacter0xeddc', [], 'Display', None],
  [ 0xeddd, 'UnicodeCharacter0xeddd', [], 'Display', None],
  [ 0xedde, 'UnicodeCharacter0xedde', [], 'Display', None],
  [ 0xeddf, 'UnicodeCharacter0xeddf', [], 'Display', None],
  [ 0xede0, 'UnicodeCharacter0xede0', [], 'Display', None],
  [ 0xede1, 'UnicodeCharacter0xede1', [], 'Display', None],
  [ 0xede2, 'UnicodeCharacter0xede2', [], 'Display', None],
  [ 0xede3, 'UnicodeCharacter0xede3', [], 'Display', None],
  [ 0xede4, 'UnicodeCharacter0xede4', [], 'Display', None],
  [ 0xede5, 'UnicodeCharacter0xede5', [], 'Display', None],
  [ 0xede6, 'UnicodeCharacter0xede6', [], 'Display', None],
  [ 0xede7, 'UnicodeCharacter0xede7', [], 'Display', None],
  [ 0xede8, 'UnicodeCharacter0xede8', [], 'Display', None],
  [ 0xede9, 'UnicodeCharacter0xede9', [], 'Display', None],
  [ 0xedea, 'UnicodeCharacter0xedea', [], 'Display', None],
  [ 0xedeb, 'UnicodeCharacter0xedeb', [], 'Display', None],
  [ 0xedec, 'UnicodeCharacter0xedec', [], 'Display', None],
  [ 0xeded, 'UnicodeCharacter0xeded', [], 'Display', None],
  [ 0xedee, 'UnicodeCharacter0xedee', [], 'Display', None],
  [ 0xedef, 'UnicodeCharacter0xedef', [], 'Display', None],
  [ 0xedf0, 'UnicodeCharacter0xedf0', [], 'Display', None],
  [ 0xedf1, 'UnicodeCharacter0xedf1', [], 'Display', None],
  [ 0xedf2, 'UnicodeCharacter0xedf2', [], 'Display', None],
  [ 0xedf3, 'UnicodeCharacter0xedf3', [], 'Display', None],
  [ 0xedf4, 'UnicodeCharacter0xedf4', [], 'Display', None],
  [ 0xedf5, 'UnicodeCharacter0xedf5', [], 'Display', None],
  [ 0xedf6, 'UnicodeCharacter0xedf6', [], 'Display', None],
  [ 0xedf7, 'UnicodeCharacter0xedf7', [], 'Display', None],
  [ 0xedf8, 'UnicodeCharacter0xedf8', [], 'Display', None],
  [ 0xedf9, 'UnicodeCharacter0xedf9', [], 'Display', None],
  [ 0xedfa, 'UnicodeCharacter0xedfa', [], 'Display', None],
  [ 0xedfb, 'UnicodeCharacter0xedfb', [], 'Display', None],
  [ 0xedfc, 'UnicodeCharacter0xedfc', [], 'Display', None],
  [ 0xedfd, 'UnicodeCharacter0xedfd', [], 'Display', None],
  [ 0xedfe, 'UnicodeCharacter0xedfe', [], 'Display', None],
  [ 0xedff, 'UnicodeCharacter0xedff', [], 'Display', None],
  [ 0xee00, 'UnicodeCharacter0xee00', [], 'Display', None],
  [ 0xee01, 'UnicodeCharacter0xee01', [], 'Display', None],
  [ 0xee02, 'UnicodeCharacter0xee02', [], 'Display', None],
  [ 0xee03, 'UnicodeCharacter0xee03', [], 'Display', None],
  [ 0xee04, 'UnicodeCharacter0xee04', [], 'Display', None],
  [ 0xee05, 'UnicodeCharacter0xee05', [], 'Display', None],
  [ 0xee06, 'UnicodeCharacter0xee06', [], 'Display', None],
  [ 0xee07, 'UnicodeCharacter0xee07', [], 'Display', None],
  [ 0xee08, 'UnicodeCharacter0xee08', [], 'Display', None],
  [ 0xee09, 'UnicodeCharacter0xee09', [], 'Display', None],
  [ 0xee0a, 'UnicodeCharacter0xee0a', [], 'Display', None],
  [ 0xee0b, 'UnicodeCharacter0xee0b', [], 'Display', None],
  [ 0xee0c, 'UnicodeCharacter0xee0c', [], 'Display', None],
  [ 0xee0d, 'UnicodeCharacter0xee0d', [], 'Display', None],
  [ 0xee0e, 'UnicodeCharacter0xee0e', [], 'Display', None],
  [ 0xee0f, 'UnicodeCharacter0xee0f', [], 'Display', None],
  [ 0xee10, 'UnicodeCharacter0xee10', [], 'Display', None],
  [ 0xee11, 'UnicodeCharacter0xee11', [], 'Display', None],
  [ 0xee12, 'UnicodeCharacter0xee12', [], 'Display', None],
  [ 0xee13, 'UnicodeCharacter0xee13', [], 'Display', None],
  [ 0xee14, 'UnicodeCharacter0xee14', [], 'Display', None],
  [ 0xee15, 'UnicodeCharacter0xee15', [], 'Display', None],
  [ 0xee16, 'UnicodeCharacter0xee16', [], 'Display', None],
  [ 0xee17, 'UnicodeCharacter0xee17', [], 'Display', None],
  [ 0xee18, 'UnicodeCharacter0xee18', [], 'Display', None],
  [ 0xee19, 'UnicodeCharacter0xee19', [], 'Display', None],
  [ 0xee1a, 'UnicodeCharacter0xee1a', [], 'Display', None],
  [ 0xee1b, 'UnicodeCharacter0xee1b', [], 'Display', None],
  [ 0xee1c, 'UnicodeCharacter0xee1c', [], 'Display', None],
  [ 0xee1d, 'UnicodeCharacter0xee1d', [], 'Display', None],
  [ 0xee1e, 'UnicodeCharacter0xee1e', [], 'Display', None],
  [ 0xee1f, 'UnicodeCharacter0xee1f', [], 'Display', None],
  [ 0xee20, 'UnicodeCharacter0xee20', [], 'Display', None],
  [ 0xee21, 'UnicodeCharacter0xee21', [], 'Display', None],
  [ 0xee22, 'UnicodeCharacter0xee22', [], 'Display', None],
  [ 0xee23, 'UnicodeCharacter0xee23', [], 'Display', None],
  [ 0xee24, 'UnicodeCharacter0xee24', [], 'Display', None],
  [ 0xee25, 'UnicodeCharacter0xee25', [], 'Display', None],
  [ 0xee26, 'UnicodeCharacter0xee26', [], 'Display', None],
  [ 0xee27, 'UnicodeCharacter0xee27', [], 'Display', None],
  [ 0xee28, 'UnicodeCharacter0xee28', [], 'Display', None],
  [ 0xee29, 'UnicodeCharacter0xee29', [], 'Display', None],
  [ 0xee2a, 'UnicodeCharacter0xee2a', [], 'Display', None],
  [ 0xee2b, 'UnicodeCharacter0xee2b', [], 'Display', None],
  [ 0xee2c, 'UnicodeCharacter0xee2c', [], 'Display', None],
  [ 0xee2d, 'UnicodeCharacter0xee2d', [], 'Display', None],
  [ 0xee2e, 'UnicodeCharacter0xee2e', [], 'Display', None],
  [ 0xee2f, 'UnicodeCharacter0xee2f', [], 'Display', None],
  [ 0xee30, 'UnicodeCharacter0xee30', [], 'Display', None],
  [ 0xee31, 'UnicodeCharacter0xee31', [], 'Display', None],
  [ 0xee32, 'UnicodeCharacter0xee32', [], 'Display', None],
  [ 0xee33, 'UnicodeCharacter0xee33', [], 'Display', None],
  [ 0xee34, 'UnicodeCharacter0xee34', [], 'Display', None],
  [ 0xee35, 'UnicodeCharacter0xee35', [], 'Display', None],
  [ 0xee36, 'UnicodeCharacter0xee36', [], 'Display', None],
  [ 0xee37, 'UnicodeCharacter0xee37', [], 'Display', None],
  [ 0xee38, 'UnicodeCharacter0xee38', [], 'Display', None],
  [ 0xee39, 'UnicodeCharacter0xee39', [], 'Display', None],
  [ 0xee3a, 'UnicodeCharacter0xee3a', [], 'Display', None],
  [ 0xee3b, 'UnicodeCharacter0xee3b', [], 'Display', None],
  [ 0xee3c, 'UnicodeCharacter0xee3c', [], 'Display', None],
  [ 0xee3d, 'UnicodeCharacter0xee3d', [], 'Display', None],
  [ 0xee3e, 'UnicodeCharacter0xee3e', [], 'Display', None],
  [ 0xee3f, 'UnicodeCharacter0xee3f', [], 'Display', None],
  [ 0xee40, 'UnicodeCharacter0xee40', [], 'Display', None],
  [ 0xee41, 'UnicodeCharacter0xee41', [], 'Display', None],
  [ 0xee42, 'UnicodeCharacter0xee42', [], 'Display', None],
  [ 0xee43, 'UnicodeCharacter0xee43', [], 'Display', None],
  [ 0xee44, 'UnicodeCharacter0xee44', [], 'Display', None],
  [ 0xee45, 'UnicodeCharacter0xee45', [], 'Display', None],
  [ 0xee46, 'UnicodeCharacter0xee46', [], 'Display', None],
  [ 0xee47, 'UnicodeCharacter0xee47', [], 'Display', None],
  [ 0xee48, 'UnicodeCharacter0xee48', [], 'Display', None],
  [ 0xee49, 'UnicodeCharacter0xee49', [], 'Display', None],
  [ 0xee4a, 'UnicodeCharacter0xee4a', [], 'Display', None],
  [ 0xee4b, 'UnicodeCharacter0xee4b', [], 'Display', None],
  [ 0xee4c, 'UnicodeCharacter0xee4c', [], 'Display', None],
  [ 0xee4d, 'UnicodeCharacter0xee4d', [], 'Display', None],
  [ 0xee4e, 'UnicodeCharacter0xee4e', [], 'Display', None],
  [ 0xee4f, 'UnicodeCharacter0xee4f', [], 'Display', None],
  [ 0xee50, 'UnicodeCharacter0xee50', [], 'Display', None],
  [ 0xee51, 'UnicodeCharacter0xee51', [], 'Display', None],
  [ 0xee52, 'UnicodeCharacter0xee52', [], 'Display', None],
  [ 0xee53, 'UnicodeCharacter0xee53', [], 'Display', None],
  [ 0xee54, 'UnicodeCharacter0xee54', [], 'Display', None],
  [ 0xee55, 'UnicodeCharacter0xee55', [], 'Display', None],
  [ 0xee56, 'UnicodeCharacter0xee56', [], 'Display', None],
  [ 0xee57, 'UnicodeCharacter0xee57', [], 'Display', None],
  [ 0xee58, 'UnicodeCharacter0xee58', [], 'Display', None],
  [ 0xee59, 'UnicodeCharacter0xee59', [], 'Display', None],
  [ 0xee5a, 'UnicodeCharacter0xee5a', [], 'Display', None],
  [ 0xee5b, 'UnicodeCharacter0xee5b', [], 'Display', None],
  [ 0xee5c, 'UnicodeCharacter0xee5c', [], 'Display', None],
  [ 0xee5d, 'UnicodeCharacter0xee5d', [], 'Display', None],
  [ 0xee5e, 'UnicodeCharacter0xee5e', [], 'Display', None],
  [ 0xee5f, 'UnicodeCharacter0xee5f', [], 'Display', None],
  [ 0xee60, 'UnicodeCharacter0xee60', [], 'Display', None],
  [ 0xee61, 'UnicodeCharacter0xee61', [], 'Display', None],
  [ 0xee62, 'UnicodeCharacter0xee62', [], 'Display', None],
  [ 0xee63, 'UnicodeCharacter0xee63', [], 'Display', None],
  [ 0xee64, 'UnicodeCharacter0xee64', [], 'Display', None],
  [ 0xee65, 'UnicodeCharacter0xee65', [], 'Display', None],
  [ 0xee66, 'UnicodeCharacter0xee66', [], 'Display', None],
  [ 0xee67, 'UnicodeCharacter0xee67', [], 'Display', None],
  [ 0xee68, 'UnicodeCharacter0xee68', [], 'Display', None],
  [ 0xee69, 'UnicodeCharacter0xee69', [], 'Display', None],
  [ 0xee6a, 'UnicodeCharacter0xee6a', [], 'Display', None],
  [ 0xee6b, 'UnicodeCharacter0xee6b', [], 'Display', None],
  [ 0xee6c, 'UnicodeCharacter0xee6c', [], 'Display', None],
  [ 0xee6d, 'UnicodeCharacter0xee6d', [], 'Display', None],
  [ 0xee6e, 'UnicodeCharacter0xee6e', [], 'Display', None],
  [ 0xee6f, 'UnicodeCharacter0xee6f', [], 'Display', None],
  [ 0xee70, 'UnicodeCharacter0xee70', [], 'Display', None],
  [ 0xee71, 'UnicodeCharacter0xee71', [], 'Display', None],
  [ 0xee72, 'UnicodeCharacter0xee72', [], 'Display', None],
  [ 0xee73, 'UnicodeCharacter0xee73', [], 'Display', None],
  [ 0xee74, 'UnicodeCharacter0xee74', [], 'Display', None],
  [ 0xee75, 'UnicodeCharacter0xee75', [], 'Display', None],
  [ 0xee76, 'UnicodeCharacter0xee76', [], 'Display', None],
  [ 0xee77, 'UnicodeCharacter0xee77', [], 'Display', None],
  [ 0xee78, 'UnicodeCharacter0xee78', [], 'Display', None],
  [ 0xee79, 'UnicodeCharacter0xee79', [], 'Display', None],
  [ 0xee7a, 'UnicodeCharacter0xee7a', [], 'Display', None],
  [ 0xee7b, 'UnicodeCharacter0xee7b', [], 'Display', None],
  [ 0xee7c, 'UnicodeCharacter0xee7c', [], 'Display', None],
  [ 0xee7d, 'UnicodeCharacter0xee7d', [], 'Display', None],
  [ 0xee7e, 'UnicodeCharacter0xee7e', [], 'Display', None],
  [ 0xee7f, 'UnicodeCharacter0xee7f', [], 'Display', None],
  [ 0xee80, 'UnicodeCharacter0xee80', [], 'Display', None],
  [ 0xee81, 'UnicodeCharacter0xee81', [], 'Display', None],
  [ 0xee82, 'UnicodeCharacter0xee82', [], 'Display', None],
  [ 0xee83, 'UnicodeCharacter0xee83', [], 'Display', None],
  [ 0xee84, 'UnicodeCharacter0xee84', [], 'Display', None],
  [ 0xee85, 'UnicodeCharacter0xee85', [], 'Display', None],
  [ 0xee86, 'UnicodeCharacter0xee86', [], 'Display', None],
  [ 0xee87, 'UnicodeCharacter0xee87', [], 'Display', None],
  [ 0xee88, 'UnicodeCharacter0xee88', [], 'Display', None],
  [ 0xee89, 'UnicodeCharacter0xee89', [], 'Display', None],
  [ 0xee8a, 'UnicodeCharacter0xee8a', [], 'Display', None],
  [ 0xee8b, 'UnicodeCharacter0xee8b', [], 'Display', None],
  [ 0xee8c, 'UnicodeCharacter0xee8c', [], 'Display', None],
  [ 0xee8d, 'UnicodeCharacter0xee8d', [], 'Display', None],
  [ 0xee8e, 'UnicodeCharacter0xee8e', [], 'Display', None],
  [ 0xee8f, 'UnicodeCharacter0xee8f', [], 'Display', None],
  [ 0xee90, 'UnicodeCharacter0xee90', [], 'Display', None],
  [ 0xee91, 'UnicodeCharacter0xee91', [], 'Display', None],
  [ 0xee92, 'UnicodeCharacter0xee92', [], 'Display', None],
  [ 0xee93, 'UnicodeCharacter0xee93', [], 'Display', None],
  [ 0xee94, 'UnicodeCharacter0xee94', [], 'Display', None],
  [ 0xee95, 'UnicodeCharacter0xee95', [], 'Display', None],
  [ 0xee96, 'UnicodeCharacter0xee96', [], 'Display', None],
  [ 0xee97, 'UnicodeCharacter0xee97', [], 'Display', None],
  [ 0xee98, 'UnicodeCharacter0xee98', [], 'Display', None],
  [ 0xee99, 'UnicodeCharacter0xee99', [], 'Display', None],
  [ 0xee9a, 'UnicodeCharacter0xee9a', [], 'Display', None],
  [ 0xee9b, 'UnicodeCharacter0xee9b', [], 'Display', None],
  [ 0xee9c, 'UnicodeCharacter0xee9c', [], 'Display', None],
  [ 0xee9d, 'UnicodeCharacter0xee9d', [], 'Display', None],
  [ 0xee9e, 'UnicodeCharacter0xee9e', [], 'Display', None],
  [ 0xee9f, 'UnicodeCharacter0xee9f', [], 'Display', None],
  [ 0xeea0, 'UnicodeCharacter0xeea0', [], 'Display', None],
  [ 0xeea1, 'UnicodeCharacter0xeea1', [], 'Display', None],
  [ 0xeea2, 'UnicodeCharacter0xeea2', [], 'Display', None],
  [ 0xeea3, 'UnicodeCharacter0xeea3', [], 'Display', None],
  [ 0xeea4, 'UnicodeCharacter0xeea4', [], 'Display', None],
  [ 0xeea5, 'UnicodeCharacter0xeea5', [], 'Display', None],
  [ 0xeea6, 'UnicodeCharacter0xeea6', [], 'Display', None],
  [ 0xeea7, 'UnicodeCharacter0xeea7', [], 'Display', None],
  [ 0xeea8, 'UnicodeCharacter0xeea8', [], 'Display', None],
  [ 0xeea9, 'UnicodeCharacter0xeea9', [], 'Display', None],
  [ 0xeeaa, 'UnicodeCharacter0xeeaa', [], 'Display', None],
  [ 0xeeab, 'UnicodeCharacter0xeeab', [], 'Display', None],
  [ 0xeeac, 'UnicodeCharacter0xeeac', [], 'Display', None],
  [ 0xeead, 'UnicodeCharacter0xeead', [], 'Display', None],
  [ 0xeeae, 'UnicodeCharacter0xeeae', [], 'Display', None],
  [ 0xeeaf, 'UnicodeCharacter0xeeaf', [], 'Display', None],
  [ 0xeeb0, 'UnicodeCharacter0xeeb0', [], 'Display', None],
  [ 0xeeb1, 'UnicodeCharacter0xeeb1', [], 'Display', None],
  [ 0xeeb2, 'UnicodeCharacter0xeeb2', [], 'Display', None],
  [ 0xeeb3, 'UnicodeCharacter0xeeb3', [], 'Display', None],
  [ 0xeeb4, 'UnicodeCharacter0xeeb4', [], 'Display', None],
  [ 0xeeb5, 'UnicodeCharacter0xeeb5', [], 'Display', None],
  [ 0xeeb6, 'UnicodeCharacter0xeeb6', [], 'Display', None],
  [ 0xeeb7, 'UnicodeCharacter0xeeb7', [], 'Display', None],
  [ 0xeeb8, 'UnicodeCharacter0xeeb8', [], 'Display', None],
  [ 0xeeb9, 'UnicodeCharacter0xeeb9', [], 'Display', None],
  [ 0xeeba, 'UnicodeCharacter0xeeba', [], 'Display', None],
  [ 0xeebb, 'UnicodeCharacter0xeebb', [], 'Display', None],
  [ 0xeebc, 'UnicodeCharacter0xeebc', [], 'Display', None],
  [ 0xeebd, 'UnicodeCharacter0xeebd', [], 'Display', None],
  [ 0xeebe, 'UnicodeCharacter0xeebe', [], 'Display', None],
  [ 0xeebf, 'UnicodeCharacter0xeebf', [], 'Display', None],
  [ 0xeec0, 'UnicodeCharacter0xeec0', [], 'Display', None],
  [ 0xeec1, 'UnicodeCharacter0xeec1', [], 'Display', None],
  [ 0xeec2, 'UnicodeCharacter0xeec2', [], 'Display', None],
  [ 0xeec3, 'UnicodeCharacter0xeec3', [], 'Display', None],
  [ 0xeec4, 'UnicodeCharacter0xeec4', [], 'Display', None],
  [ 0xeec5, 'UnicodeCharacter0xeec5', [], 'Display', None],
  [ 0xeec6, 'UnicodeCharacter0xeec6', [], 'Display', None],
  [ 0xeec7, 'UnicodeCharacter0xeec7', [], 'Display', None],
  [ 0xeec8, 'UnicodeCharacter0xeec8', [], 'Display', None],
  [ 0xeec9, 'UnicodeCharacter0xeec9', [], 'Display', None],
  [ 0xeeca, 'UnicodeCharacter0xeeca', [], 'Display', None],
  [ 0xeecb, 'UnicodeCharacter0xeecb', [], 'Display', None],
  [ 0xeecc, 'UnicodeCharacter0xeecc', [], 'Display', None],
  [ 0xeecd, 'UnicodeCharacter0xeecd', [], 'Display', None],
  [ 0xeece, 'UnicodeCharacter0xeece', [], 'Display', None],
  [ 0xeecf, 'UnicodeCharacter0xeecf', [], 'Display', None],
  [ 0xeed0, 'UnicodeCharacter0xeed0', [], 'Display', None],
  [ 0xeed1, 'UnicodeCharacter0xeed1', [], 'Display', None],
  [ 0xeed2, 'UnicodeCharacter0xeed2', [], 'Display', None],
  [ 0xeed3, 'UnicodeCharacter0xeed3', [], 'Display', None],
  [ 0xeed4, 'UnicodeCharacter0xeed4', [], 'Display', None],
  [ 0xeed5, 'UnicodeCharacter0xeed5', [], 'Display', None],
  [ 0xeed6, 'UnicodeCharacter0xeed6', [], 'Display', None],
  [ 0xeed7, 'UnicodeCharacter0xeed7', [], 'Display', None],
  [ 0xeed8, 'UnicodeCharacter0xeed8', [], 'Display', None],
  [ 0xeed9, 'UnicodeCharacter0xeed9', [], 'Display', None],
  [ 0xeeda, 'UnicodeCharacter0xeeda', [], 'Display', None],
  [ 0xeedb, 'UnicodeCharacter0xeedb', [], 'Display', None],
  [ 0xeedc, 'UnicodeCharacter0xeedc', [], 'Display', None],
  [ 0xeedd, 'UnicodeCharacter0xeedd', [], 'Display', None],
  [ 0xeede, 'UnicodeCharacter0xeede', [], 'Display', None],
  [ 0xeedf, 'UnicodeCharacter0xeedf', [], 'Display', None],
  [ 0xeee0, 'UnicodeCharacter0xeee0', [], 'Display', None],
  [ 0xeee1, 'UnicodeCharacter0xeee1', [], 'Display', None],
  [ 0xeee2, 'UnicodeCharacter0xeee2', [], 'Display', None],
  [ 0xeee3, 'UnicodeCharacter0xeee3', [], 'Display', None],
  [ 0xeee4, 'UnicodeCharacter0xeee4', [], 'Display', None],
  [ 0xeee5, 'UnicodeCharacter0xeee5', [], 'Display', None],
  [ 0xeee6, 'UnicodeCharacter0xeee6', [], 'Display', None],
  [ 0xeee7, 'UnicodeCharacter0xeee7', [], 'Display', None],
  [ 0xeee8, 'UnicodeCharacter0xeee8', [], 'Display', None],
  [ 0xeee9, 'UnicodeCharacter0xeee9', [], 'Display', None],
  [ 0xeeea, 'UnicodeCharacter0xeeea', [], 'Display', None],
  [ 0xeeeb, 'UnicodeCharacter0xeeeb', [], 'Display', None],
  [ 0xeeec, 'UnicodeCharacter0xeeec', [], 'Display', None],
  [ 0xeeed, 'UnicodeCharacter0xeeed', [], 'Display', None],
  [ 0xeeee, 'UnicodeCharacter0xeeee', [], 'Display', None],
  [ 0xeeef, 'UnicodeCharacter0xeeef', [], 'Display', None],
  [ 0xeef0, 'UnicodeCharacter0xeef0', [], 'Display', None],
  [ 0xeef1, 'UnicodeCharacter0xeef1', [], 'Display', None],
  [ 0xeef2, 'UnicodeCharacter0xeef2', [], 'Display', None],
  [ 0xeef3, 'UnicodeCharacter0xeef3', [], 'Display', None],
  [ 0xeef4, 'UnicodeCharacter0xeef4', [], 'Display', None],
  [ 0xeef5, 'UnicodeCharacter0xeef5', [], 'Display', None],
  [ 0xeef6, 'UnicodeCharacter0xeef6', [], 'Display', None],
  [ 0xeef7, 'UnicodeCharacter0xeef7', [], 'Display', None],
  [ 0xeef8, 'UnicodeCharacter0xeef8', [], 'Display', None],
  [ 0xeef9, 'UnicodeCharacter0xeef9', [], 'Display', None],
  [ 0xeefa, 'UnicodeCharacter0xeefa', [], 'Display', None],
  [ 0xeefb, 'UnicodeCharacter0xeefb', [], 'Display', None],
  [ 0xeefc, 'UnicodeCharacter0xeefc', [], 'Display', None],
  [ 0xeefd, 'UnicodeCharacter0xeefd', [], 'Display', None],
  [ 0xeefe, 'UnicodeCharacter0xeefe', [], 'Display', None],
  [ 0xeeff, 'UnicodeCharacter0xeeff', [], 'Display', None],
  [ 0xef00, 'UnicodeCharacter0xef00', [], 'Display', None],
  [ 0xef01, 'UnicodeCharacter0xef01', [], 'Display', None],
  [ 0xef02, 'UnicodeCharacter0xef02', [], 'Display', None],
  [ 0xef03, 'UnicodeCharacter0xef03', [], 'Display', None],
  [ 0xef04, 'UnicodeCharacter0xef04', [], 'Display', None],
  [ 0xef05, 'UnicodeCharacter0xef05', [], 'Display', None],
  [ 0xef06, 'UnicodeCharacter0xef06', [], 'Display', None],
  [ 0xef07, 'UnicodeCharacter0xef07', [], 'Display', None],
  [ 0xef08, 'UnicodeCharacter0xef08', [], 'Display', None],
  [ 0xef09, 'UnicodeCharacter0xef09', [], 'Display', None],
  [ 0xef0a, 'UnicodeCharacter0xef0a', [], 'Display', None],
  [ 0xef0b, 'UnicodeCharacter0xef0b', [], 'Display', None],
  [ 0xef0c, 'UnicodeCharacter0xef0c', [], 'Display', None],
  [ 0xef0d, 'UnicodeCharacter0xef0d', [], 'Display', None],
  [ 0xef0e, 'UnicodeCharacter0xef0e', [], 'Display', None],
  [ 0xef0f, 'UnicodeCharacter0xef0f', [], 'Display', None],
  [ 0xef10, 'UnicodeCharacter0xef10', [], 'Display', None],
  [ 0xef11, 'UnicodeCharacter0xef11', [], 'Display', None],
  [ 0xef12, 'UnicodeCharacter0xef12', [], 'Display', None],
  [ 0xef13, 'UnicodeCharacter0xef13', [], 'Display', None],
  [ 0xef14, 'UnicodeCharacter0xef14', [], 'Display', None],
  [ 0xef15, 'UnicodeCharacter0xef15', [], 'Display', None],
  [ 0xef16, 'UnicodeCharacter0xef16', [], 'Display', None],
  [ 0xef17, 'UnicodeCharacter0xef17', [], 'Display', None],
  [ 0xef18, 'UnicodeCharacter0xef18', [], 'Display', None],
  [ 0xef19, 'UnicodeCharacter0xef19', [], 'Display', None],
  [ 0xef1a, 'UnicodeCharacter0xef1a', [], 'Display', None],
  [ 0xef1b, 'UnicodeCharacter0xef1b', [], 'Display', None],
  [ 0xef1c, 'UnicodeCharacter0xef1c', [], 'Display', None],
  [ 0xef1d, 'UnicodeCharacter0xef1d', [], 'Display', None],
  [ 0xef1e, 'UnicodeCharacter0xef1e', [], 'Display', None],
  [ 0xef1f, 'UnicodeCharacter0xef1f', [], 'Display', None],
  [ 0xef20, 'UnicodeCharacter0xef20', [], 'Display', None],
  [ 0xef21, 'UnicodeCharacter0xef21', [], 'Display', None],
  [ 0xef22, 'UnicodeCharacter0xef22', [], 'Display', None],
  [ 0xef23, 'UnicodeCharacter0xef23', [], 'Display', None],
  [ 0xef24, 'UnicodeCharacter0xef24', [], 'Display', None],
  [ 0xef25, 'UnicodeCharacter0xef25', [], 'Display', None],
  [ 0xef26, 'UnicodeCharacter0xef26', [], 'Display', None],
  [ 0xef27, 'UnicodeCharacter0xef27', [], 'Display', None],
  [ 0xef28, 'UnicodeCharacter0xef28', [], 'Display', None],
  [ 0xef29, 'UnicodeCharacter0xef29', [], 'Display', None],
  [ 0xef2a, 'UnicodeCharacter0xef2a', [], 'Display', None],
  [ 0xef2b, 'UnicodeCharacter0xef2b', [], 'Display', None],
  [ 0xef2c, 'UnicodeCharacter0xef2c', [], 'Display', None],
  [ 0xef2d, 'UnicodeCharacter0xef2d', [], 'Display', None],
  [ 0xef2e, 'UnicodeCharacter0xef2e', [], 'Display', None],
  [ 0xef2f, 'UnicodeCharacter0xef2f', [], 'Display', None],
  [ 0xef30, 'UnicodeCharacter0xef30', [], 'Display', None],
  [ 0xef31, 'UnicodeCharacter0xef31', [], 'Display', None],
  [ 0xef32, 'UnicodeCharacter0xef32', [], 'Display', None],
  [ 0xef33, 'UnicodeCharacter0xef33', [], 'Display', None],
  [ 0xef34, 'UnicodeCharacter0xef34', [], 'Display', None],
  [ 0xef35, 'UnicodeCharacter0xef35', [], 'Display', None],
  [ 0xef36, 'UnicodeCharacter0xef36', [], 'Display', None],
  [ 0xef37, 'UnicodeCharacter0xef37', [], 'Display', None],
  [ 0xef38, 'UnicodeCharacter0xef38', [], 'Display', None],
  [ 0xef39, 'UnicodeCharacter0xef39', [], 'Display', None],
  [ 0xef3a, 'UnicodeCharacter0xef3a', [], 'Display', None],
  [ 0xef3b, 'UnicodeCharacter0xef3b', [], 'Display', None],
  [ 0xef3c, 'UnicodeCharacter0xef3c', [], 'Display', None],
  [ 0xef3d, 'UnicodeCharacter0xef3d', [], 'Display', None],
  [ 0xef3e, 'UnicodeCharacter0xef3e', [], 'Display', None],
  [ 0xef3f, 'UnicodeCharacter0xef3f', [], 'Display', None],
  [ 0xef40, 'UnicodeCharacter0xef40', [], 'Display', None],
  [ 0xef41, 'UnicodeCharacter0xef41', [], 'Display', None],
  [ 0xef42, 'UnicodeCharacter0xef42', [], 'Display', None],
  [ 0xef43, 'UnicodeCharacter0xef43', [], 'Display', None],
  [ 0xef44, 'UnicodeCharacter0xef44', [], 'Display', None],
  [ 0xef45, 'UnicodeCharacter0xef45', [], 'Display', None],
  [ 0xef46, 'UnicodeCharacter0xef46', [], 'Display', None],
  [ 0xef47, 'UnicodeCharacter0xef47', [], 'Display', None],
  [ 0xef48, 'UnicodeCharacter0xef48', [], 'Display', None],
  [ 0xef49, 'UnicodeCharacter0xef49', [], 'Display', None],
  [ 0xef4a, 'UnicodeCharacter0xef4a', [], 'Display', None],
  [ 0xef4b, 'UnicodeCharacter0xef4b', [], 'Display', None],
  [ 0xef4c, 'UnicodeCharacter0xef4c', [], 'Display', None],
  [ 0xef4d, 'UnicodeCharacter0xef4d', [], 'Display', None],
  [ 0xef4e, 'UnicodeCharacter0xef4e', [], 'Display', None],
  [ 0xef4f, 'UnicodeCharacter0xef4f', [], 'Display', None],
  [ 0xef50, 'UnicodeCharacter0xef50', [], 'Display', None],
  [ 0xef51, 'UnicodeCharacter0xef51', [], 'Display', None],
  [ 0xef52, 'UnicodeCharacter0xef52', [], 'Display', None],
  [ 0xef53, 'UnicodeCharacter0xef53', [], 'Display', None],
  [ 0xef54, 'UnicodeCharacter0xef54', [], 'Display', None],
  [ 0xef55, 'UnicodeCharacter0xef55', [], 'Display', None],
  [ 0xef56, 'UnicodeCharacter0xef56', [], 'Display', None],
  [ 0xef57, 'UnicodeCharacter0xef57', [], 'Display', None],
  [ 0xef58, 'UnicodeCharacter0xef58', [], 'Display', None],
  [ 0xef59, 'UnicodeCharacter0xef59', [], 'Display', None],
  [ 0xef5a, 'UnicodeCharacter0xef5a', [], 'Display', None],
  [ 0xef5b, 'UnicodeCharacter0xef5b', [], 'Display', None],
  [ 0xef5c, 'UnicodeCharacter0xef5c', [], 'Display', None],
  [ 0xef5d, 'UnicodeCharacter0xef5d', [], 'Display', None],
  [ 0xef5e, 'UnicodeCharacter0xef5e', [], 'Display', None],
  [ 0xef5f, 'UnicodeCharacter0xef5f', [], 'Display', None],
  [ 0xef60, 'UnicodeCharacter0xef60', [], 'Display', None],
  [ 0xef61, 'UnicodeCharacter0xef61', [], 'Display', None],
  [ 0xef62, 'UnicodeCharacter0xef62', [], 'Display', None],
  [ 0xef63, 'UnicodeCharacter0xef63', [], 'Display', None],
  [ 0xef64, 'UnicodeCharacter0xef64', [], 'Display', None],
  [ 0xef65, 'UnicodeCharacter0xef65', [], 'Display', None],
  [ 0xef66, 'UnicodeCharacter0xef66', [], 'Display', None],
  [ 0xef67, 'UnicodeCharacter0xef67', [], 'Display', None],
  [ 0xef68, 'UnicodeCharacter0xef68', [], 'Display', None],
  [ 0xef69, 'UnicodeCharacter0xef69', [], 'Display', None],
  [ 0xef6a, 'UnicodeCharacter0xef6a', [], 'Display', None],
  [ 0xef6b, 'UnicodeCharacter0xef6b', [], 'Display', None],
  [ 0xef6c, 'UnicodeCharacter0xef6c', [], 'Display', None],
  [ 0xef6d, 'UnicodeCharacter0xef6d', [], 'Display', None],
  [ 0xef6e, 'UnicodeCharacter0xef6e', [], 'Display', None],
  [ 0xef6f, 'UnicodeCharacter0xef6f', [], 'Display', None],
  [ 0xef70, 'UnicodeCharacter0xef70', [], 'Display', None],
  [ 0xef71, 'UnicodeCharacter0xef71', [], 'Display', None],
  [ 0xef72, 'UnicodeCharacter0xef72', [], 'Display', None],
  [ 0xef73, 'UnicodeCharacter0xef73', [], 'Display', None],
  [ 0xef74, 'UnicodeCharacter0xef74', [], 'Display', None],
  [ 0xef75, 'UnicodeCharacter0xef75', [], 'Display', None],
  [ 0xef76, 'UnicodeCharacter0xef76', [], 'Display', None],
  [ 0xef77, 'UnicodeCharacter0xef77', [], 'Display', None],
  [ 0xef78, 'UnicodeCharacter0xef78', [], 'Display', None],
  [ 0xef79, 'UnicodeCharacter0xef79', [], 'Display', None],
  [ 0xef7a, 'UnicodeCharacter0xef7a', [], 'Display', None],
  [ 0xef7b, 'UnicodeCharacter0xef7b', [], 'Display', None],
  [ 0xef7c, 'UnicodeCharacter0xef7c', [], 'Display', None],
  [ 0xef7d, 'UnicodeCharacter0xef7d', [], 'Display', None],
  [ 0xef7e, 'UnicodeCharacter0xef7e', [], 'Display', None],
  [ 0xef7f, 'UnicodeCharacter0xef7f', [], 'Display', None],
  [ 0xef80, 'UnicodeCharacter0xef80', [], 'Display', None],
  [ 0xef81, 'UnicodeCharacter0xef81', [], 'Display', None],
  [ 0xef82, 'UnicodeCharacter0xef82', [], 'Display', None],
  [ 0xef83, 'UnicodeCharacter0xef83', [], 'Display', None],
  [ 0xef84, 'UnicodeCharacter0xef84', [], 'Display', None],
  [ 0xef85, 'UnicodeCharacter0xef85', [], 'Display', None],
  [ 0xef86, 'UnicodeCharacter0xef86', [], 'Display', None],
  [ 0xef87, 'UnicodeCharacter0xef87', [], 'Display', None],
  [ 0xef88, 'UnicodeCharacter0xef88', [], 'Display', None],
  [ 0xef89, 'UnicodeCharacter0xef89', [], 'Display', None],
  [ 0xef8a, 'UnicodeCharacter0xef8a', [], 'Display', None],
  [ 0xef8b, 'UnicodeCharacter0xef8b', [], 'Display', None],
  [ 0xef8c, 'UnicodeCharacter0xef8c', [], 'Display', None],
  [ 0xef8d, 'UnicodeCharacter0xef8d', [], 'Display', None],
  [ 0xef8e, 'UnicodeCharacter0xef8e', [], 'Display', None],
  [ 0xef8f, 'UnicodeCharacter0xef8f', [], 'Display', None],
  [ 0xef90, 'UnicodeCharacter0xef90', [], 'Display', None],
  [ 0xef91, 'UnicodeCharacter0xef91', [], 'Display', None],
  [ 0xef92, 'UnicodeCharacter0xef92', [], 'Display', None],
  [ 0xef93, 'UnicodeCharacter0xef93', [], 'Display', None],
  [ 0xef94, 'UnicodeCharacter0xef94', [], 'Display', None],
  [ 0xef95, 'UnicodeCharacter0xef95', [], 'Display', None],
  [ 0xef96, 'UnicodeCharacter0xef96', [], 'Display', None],
  [ 0xef97, 'UnicodeCharacter0xef97', [], 'Display', None],
  [ 0xef98, 'UnicodeCharacter0xef98', [], 'Display', None],
  [ 0xef99, 'UnicodeCharacter0xef99', [], 'Display', None],
  [ 0xef9a, 'UnicodeCharacter0xef9a', [], 'Display', None],
  [ 0xef9b, 'UnicodeCharacter0xef9b', [], 'Display', None],
  [ 0xef9c, 'UnicodeCharacter0xef9c', [], 'Display', None],
  [ 0xef9d, 'UnicodeCharacter0xef9d', [], 'Display', None],
  [ 0xef9e, 'UnicodeCharacter0xef9e', [], 'Display', None],
  [ 0xef9f, 'UnicodeCharacter0xef9f', [], 'Display', None],
  [ 0xefa0, 'UnicodeCharacter0xefa0', [], 'Display', None],
  [ 0xefa1, 'UnicodeCharacter0xefa1', [], 'Display', None],
  [ 0xefa2, 'UnicodeCharacter0xefa2', [], 'Display', None],
  [ 0xefa3, 'UnicodeCharacter0xefa3', [], 'Display', None],
  [ 0xefa4, 'UnicodeCharacter0xefa4', [], 'Display', None],
  [ 0xefa5, 'UnicodeCharacter0xefa5', [], 'Display', None],
  [ 0xefa6, 'UnicodeCharacter0xefa6', [], 'Display', None],
  [ 0xefa7, 'UnicodeCharacter0xefa7', [], 'Display', None],
  [ 0xefa8, 'UnicodeCharacter0xefa8', [], 'Display', None],
  [ 0xefa9, 'UnicodeCharacter0xefa9', [], 'Display', None],
  [ 0xefaa, 'UnicodeCharacter0xefaa', [], 'Display', None],
  [ 0xefab, 'UnicodeCharacter0xefab', [], 'Display', None],
  [ 0xefac, 'UnicodeCharacter0xefac', [], 'Display', None],
  [ 0xefad, 'UnicodeCharacter0xefad', [], 'Display', None],
  [ 0xefae, 'UnicodeCharacter0xefae', [], 'Display', None],
  [ 0xefaf, 'UnicodeCharacter0xefaf', [], 'Display', None],
  [ 0xefb0, 'UnicodeCharacter0xefb0', [], 'Display', None],
  [ 0xefb1, 'UnicodeCharacter0xefb1', [], 'Display', None],
  [ 0xefb2, 'UnicodeCharacter0xefb2', [], 'Display', None],
  [ 0xefb3, 'UnicodeCharacter0xefb3', [], 'Display', None],
  [ 0xefb4, 'UnicodeCharacter0xefb4', [], 'Display', None],
  [ 0xefb5, 'UnicodeCharacter0xefb5', [], 'Display', None],
  [ 0xefb6, 'UnicodeCharacter0xefb6', [], 'Display', None],
  [ 0xefb7, 'UnicodeCharacter0xefb7', [], 'Display', None],
  [ 0xefb8, 'UnicodeCharacter0xefb8', [], 'Display', None],
  [ 0xefb9, 'UnicodeCharacter0xefb9', [], 'Display', None],
  [ 0xefba, 'UnicodeCharacter0xefba', [], 'Display', None],
  [ 0xefbb, 'UnicodeCharacter0xefbb', [], 'Display', None],
  [ 0xefbc, 'UnicodeCharacter0xefbc', [], 'Display', None],
  [ 0xefbd, 'UnicodeCharacter0xefbd', [], 'Display', None],
  [ 0xefbe, 'UnicodeCharacter0xefbe', [], 'Display', None],
  [ 0xefbf, 'UnicodeCharacter0xefbf', [], 'Display', None],
  [ 0xefc0, 'UnicodeCharacter0xefc0', [], 'Display', None],
  [ 0xefc1, 'UnicodeCharacter0xefc1', [], 'Display', None],
  [ 0xefc2, 'UnicodeCharacter0xefc2', [], 'Display', None],
  [ 0xefc3, 'UnicodeCharacter0xefc3', [], 'Display', None],
  [ 0xefc4, 'UnicodeCharacter0xefc4', [], 'Display', None],
  [ 0xefc5, 'UnicodeCharacter0xefc5', [], 'Display', None],
  [ 0xefc6, 'UnicodeCharacter0xefc6', [], 'Display', None],
  [ 0xefc7, 'UnicodeCharacter0xefc7', [], 'Display', None],
  [ 0xefc8, 'UnicodeCharacter0xefc8', [], 'Display', None],
  [ 0xefc9, 'UnicodeCharacter0xefc9', [], 'Display', None],
  [ 0xefca, 'UnicodeCharacter0xefca', [], 'Display', None],
  [ 0xefcb, 'UnicodeCharacter0xefcb', [], 'Display', None],
  [ 0xefcc, 'UnicodeCharacter0xefcc', [], 'Display', None],
  [ 0xefcd, 'UnicodeCharacter0xefcd', [], 'Display', None],
  [ 0xefce, 'UnicodeCharacter0xefce', [], 'Display', None],
  [ 0xefcf, 'UnicodeCharacter0xefcf', [], 'Display', None],
  [ 0xefd0, 'UnicodeCharacter0xefd0', [], 'Display', None],
  [ 0xefd1, 'UnicodeCharacter0xefd1', [], 'Display', None],
  [ 0xefd2, 'UnicodeCharacter0xefd2', [], 'Display', None],
  [ 0xefd3, 'UnicodeCharacter0xefd3', [], 'Display', None],
  [ 0xefd4, 'UnicodeCharacter0xefd4', [], 'Display', None],
  [ 0xefd5, 'UnicodeCharacter0xefd5', [], 'Display', None],
  [ 0xefd6, 'UnicodeCharacter0xefd6', [], 'Display', None],
  [ 0xefd7, 'UnicodeCharacter0xefd7', [], 'Display', None],
  [ 0xefd8, 'UnicodeCharacter0xefd8', [], 'Display', None],
  [ 0xefd9, 'UnicodeCharacter0xefd9', [], 'Display', None],
  [ 0xefda, 'UnicodeCharacter0xefda', [], 'Display', None],
  [ 0xefdb, 'UnicodeCharacter0xefdb', [], 'Display', None],
  [ 0xefdc, 'UnicodeCharacter0xefdc', [], 'Display', None],
  [ 0xefdd, 'UnicodeCharacter0xefdd', [], 'Display', None],
  [ 0xefde, 'UnicodeCharacter0xefde', [], 'Display', None],
  [ 0xefdf, 'UnicodeCharacter0xefdf', [], 'Display', None],
  [ 0xefe0, 'UnicodeCharacter0xefe0', [], 'Display', None],
  [ 0xefe1, 'UnicodeCharacter0xefe1', [], 'Display', None],
  [ 0xefe2, 'UnicodeCharacter0xefe2', [], 'Display', None],
  [ 0xefe3, 'UnicodeCharacter0xefe3', [], 'Display', None],
  [ 0xefe4, 'UnicodeCharacter0xefe4', [], 'Display', None],
  [ 0xefe5, 'UnicodeCharacter0xefe5', [], 'Display', None],
  [ 0xefe6, 'UnicodeCharacter0xefe6', [], 'Display', None],
  [ 0xefe7, 'UnicodeCharacter0xefe7', [], 'Display', None],
  [ 0xefe8, 'UnicodeCharacter0xefe8', [], 'Display', None],
  [ 0xefe9, 'UnicodeCharacter0xefe9', [], 'Display', None],
  [ 0xefea, 'UnicodeCharacter0xefea', [], 'Display', None],
  [ 0xefeb, 'UnicodeCharacter0xefeb', [], 'Display', None],
  [ 0xefec, 'UnicodeCharacter0xefec', [], 'Display', None],
  [ 0xefed, 'UnicodeCharacter0xefed', [], 'Display', None],
  [ 0xefee, 'UnicodeCharacter0xefee', [], 'Display', None],
  [ 0xefef, 'UnicodeCharacter0xefef', [], 'Display', None],
  [ 0xeff0, 'UnicodeCharacter0xeff0', [], 'Display', None],
  [ 0xeff1, 'UnicodeCharacter0xeff1', [], 'Display', None],
  [ 0xeff2, 'UnicodeCharacter0xeff2', [], 'Display', None],
  [ 0xeff3, 'UnicodeCharacter0xeff3', [], 'Display', None],
  [ 0xeff4, 'UnicodeCharacter0xeff4', [], 'Display', None],
  [ 0xeff5, 'UnicodeCharacter0xeff5', [], 'Display', None],
  [ 0xeff6, 'UnicodeCharacter0xeff6', [], 'Display', None],
  [ 0xeff7, 'UnicodeCharacter0xeff7', [], 'Display', None],
  [ 0xeff8, 'UnicodeCharacter0xeff8', [], 'Display', None],
  [ 0xeff9, 'UnicodeCharacter0xeff9', [], 'Display', None],
  [ 0xeffa, 'UnicodeCharacter0xeffa', [], 'Display', None],
  [ 0xeffb, 'UnicodeCharacter0xeffb', [], 'Display', None],
  [ 0xeffc, 'UnicodeCharacter0xeffc', [], 'Display', None],
  [ 0xeffd, 'UnicodeCharacter0xeffd', [], 'Display', None],
  [ 0xeffe, 'UnicodeCharacter0xeffe', [], 'Display', None],
  [ 0xefff, 'UnicodeCharacter0xefff', [], 'Display', None],
  [ 0xf000, 'UnicodeCharacter0xf000', [], 'Display', None],
  [ 0xf001, 'UnicodeCharacter0xf001', [], 'Display', None],
  [ 0xf002, 'UnicodeCharacter0xf002', [], 'Display', None],
  [ 0xf003, 'UnicodeCharacter0xf003', [], 'Display', None],
  [ 0xf004, 'UnicodeCharacter0xf004', [], 'Display', None],
  [ 0xf005, 'UnicodeCharacter0xf005', [], 'Display', None],
  [ 0xf006, 'UnicodeCharacter0xf006', [], 'Display', None],
  [ 0xf007, 'UnicodeCharacter0xf007', [], 'Display', None],
  [ 0xf008, 'UnicodeCharacter0xf008', [], 'Display', None],
  [ 0xf009, 'UnicodeCharacter0xf009', [], 'Display', None],
  [ 0xf00a, 'UnicodeCharacter0xf00a', [], 'Display', None],
  [ 0xf00b, 'UnicodeCharacter0xf00b', [], 'Display', None],
  [ 0xf00c, 'UnicodeCharacter0xf00c', [], 'Display', None],
  [ 0xf00d, 'UnicodeCharacter0xf00d', [], 'Display', None],
  [ 0xf00e, 'UnicodeCharacter0xf00e', [], 'Display', None],
  [ 0xf00f, 'UnicodeCharacter0xf00f', [], 'Display', None],
  [ 0xf010, 'UnicodeCharacter0xf010', [], 'Display', None],
  [ 0xf011, 'UnicodeCharacter0xf011', [], 'Display', None],
  [ 0xf012, 'UnicodeCharacter0xf012', [], 'Display', None],
  [ 0xf013, 'UnicodeCharacter0xf013', [], 'Display', None],
  [ 0xf014, 'UnicodeCharacter0xf014', [], 'Display', None],
  [ 0xf015, 'UnicodeCharacter0xf015', [], 'Display', None],
  [ 0xf016, 'UnicodeCharacter0xf016', [], 'Display', None],
  [ 0xf017, 'UnicodeCharacter0xf017', [], 'Display', None],
  [ 0xf018, 'UnicodeCharacter0xf018', [], 'Display', None],
  [ 0xf019, 'UnicodeCharacter0xf019', [], 'Display', None],
  [ 0xf01a, 'UnicodeCharacter0xf01a', [], 'Display', None],
  [ 0xf01b, 'UnicodeCharacter0xf01b', [], 'Display', None],
  [ 0xf01c, 'UnicodeCharacter0xf01c', [], 'Display', None],
  [ 0xf01d, 'UnicodeCharacter0xf01d', [], 'Display', None],
  [ 0xf01e, 'UnicodeCharacter0xf01e', [], 'Display', None],
  [ 0xf01f, 'UnicodeCharacter0xf01f', [], 'Display', None],
  [ 0xf020, 'UnicodeCharacter0xf020', [], 'Display', None],
  [ 0xf021, 'UnicodeCharacter0xf021', [], 'Display', None],
  [ 0xf022, 'UnicodeCharacter0xf022', [], 'Display', None],
  [ 0xf023, 'UnicodeCharacter0xf023', [], 'Display', None],
  [ 0xf024, 'UnicodeCharacter0xf024', [], 'Display', None],
  [ 0xf025, 'UnicodeCharacter0xf025', [], 'Display', None],
  [ 0xf026, 'UnicodeCharacter0xf026', [], 'Display', None],
  [ 0xf027, 'UnicodeCharacter0xf027', [], 'Display', None],
  [ 0xf028, 'UnicodeCharacter0xf028', [], 'Display', None],
  [ 0xf029, 'UnicodeCharacter0xf029', [], 'Display', None],
  [ 0xf02a, 'UnicodeCharacter0xf02a', [], 'Display', None],
  [ 0xf02b, 'UnicodeCharacter0xf02b', [], 'Display', None],
  [ 0xf02c, 'UnicodeCharacter0xf02c', [], 'Display', None],
  [ 0xf02d, 'UnicodeCharacter0xf02d', [], 'Display', None],
  [ 0xf02e, 'UnicodeCharacter0xf02e', [], 'Display', None],
  [ 0xf02f, 'UnicodeCharacter0xf02f', [], 'Display', None],
  [ 0xf030, 'UnicodeCharacter0xf030', [], 'Display', None],
  [ 0xf031, 'UnicodeCharacter0xf031', [], 'Display', None],
  [ 0xf032, 'UnicodeCharacter0xf032', [], 'Display', None],
  [ 0xf033, 'UnicodeCharacter0xf033', [], 'Display', None],
  [ 0xf034, 'UnicodeCharacter0xf034', [], 'Display', None],
  [ 0xf035, 'UnicodeCharacter0xf035', [], 'Display', None],
  [ 0xf036, 'UnicodeCharacter0xf036', [], 'Display', None],
  [ 0xf037, 'UnicodeCharacter0xf037', [], 'Display', None],
  [ 0xf038, 'UnicodeCharacter0xf038', [], 'Display', None],
  [ 0xf039, 'UnicodeCharacter0xf039', [], 'Display', None],
  [ 0xf03a, 'UnicodeCharacter0xf03a', [], 'Display', None],
  [ 0xf03b, 'UnicodeCharacter0xf03b', [], 'Display', None],
  [ 0xf03c, 'UnicodeCharacter0xf03c', [], 'Display', None],
  [ 0xf03d, 'UnicodeCharacter0xf03d', [], 'Display', None],
  [ 0xf03e, 'UnicodeCharacter0xf03e', [], 'Display', None],
  [ 0xf03f, 'UnicodeCharacter0xf03f', [], 'Display', None],
  [ 0xf040, 'UnicodeCharacter0xf040', [], 'Display', None],
  [ 0xf041, 'UnicodeCharacter0xf041', [], 'Display', None],
  [ 0xf042, 'UnicodeCharacter0xf042', [], 'Display', None],
  [ 0xf043, 'UnicodeCharacter0xf043', [], 'Display', None],
  [ 0xf044, 'UnicodeCharacter0xf044', [], 'Display', None],
  [ 0xf045, 'UnicodeCharacter0xf045', [], 'Display', None],
  [ 0xf046, 'UnicodeCharacter0xf046', [], 'Display', None],
  [ 0xf047, 'UnicodeCharacter0xf047', [], 'Display', None],
  [ 0xf048, 'UnicodeCharacter0xf048', [], 'Display', None],
  [ 0xf049, 'UnicodeCharacter0xf049', [], 'Display', None],
  [ 0xf04a, 'UnicodeCharacter0xf04a', [], 'Display', None],
  [ 0xf04b, 'UnicodeCharacter0xf04b', [], 'Display', None],
  [ 0xf04c, 'UnicodeCharacter0xf04c', [], 'Display', None],
  [ 0xf04d, 'UnicodeCharacter0xf04d', [], 'Display', None],
  [ 0xf04e, 'UnicodeCharacter0xf04e', [], 'Display', None],
  [ 0xf04f, 'UnicodeCharacter0xf04f', [], 'Display', None],
  [ 0xf050, 'UnicodeCharacter0xf050', [], 'Display', None],
  [ 0xf051, 'UnicodeCharacter0xf051', [], 'Display', None],
  [ 0xf052, 'UnicodeCharacter0xf052', [], 'Display', None],
  [ 0xf053, 'UnicodeCharacter0xf053', [], 'Display', None],
  [ 0xf054, 'UnicodeCharacter0xf054', [], 'Display', None],
  [ 0xf055, 'UnicodeCharacter0xf055', [], 'Display', None],
  [ 0xf056, 'UnicodeCharacter0xf056', [], 'Display', None],
  [ 0xf057, 'UnicodeCharacter0xf057', [], 'Display', None],
  [ 0xf058, 'UnicodeCharacter0xf058', [], 'Display', None],
  [ 0xf059, 'UnicodeCharacter0xf059', [], 'Display', None],
  [ 0xf05a, 'UnicodeCharacter0xf05a', [], 'Display', None],
  [ 0xf05b, 'UnicodeCharacter0xf05b', [], 'Display', None],
  [ 0xf05c, 'UnicodeCharacter0xf05c', [], 'Display', None],
  [ 0xf05d, 'UnicodeCharacter0xf05d', [], 'Display', None],
  [ 0xf05e, 'UnicodeCharacter0xf05e', [], 'Display', None],
  [ 0xf05f, 'UnicodeCharacter0xf05f', [], 'Display', None],
  [ 0xf060, 'UnicodeCharacter0xf060', [], 'Display', None],
  [ 0xf061, 'UnicodeCharacter0xf061', [], 'Display', None],
  [ 0xf062, 'UnicodeCharacter0xf062', [], 'Display', None],
  [ 0xf063, 'UnicodeCharacter0xf063', [], 'Display', None],
  [ 0xf064, 'UnicodeCharacter0xf064', [], 'Display', None],
  [ 0xf065, 'UnicodeCharacter0xf065', [], 'Display', None],
  [ 0xf066, 'UnicodeCharacter0xf066', [], 'Display', None],
  [ 0xf067, 'UnicodeCharacter0xf067', [], 'Display', None],
  [ 0xf068, 'UnicodeCharacter0xf068', [], 'Display', None],
  [ 0xf069, 'UnicodeCharacter0xf069', [], 'Display', None],
  [ 0xf06a, 'UnicodeCharacter0xf06a', [], 'Display', None],
  [ 0xf06b, 'UnicodeCharacter0xf06b', [], 'Display', None],
  [ 0xf06c, 'UnicodeCharacter0xf06c', [], 'Display', None],
  [ 0xf06d, 'UnicodeCharacter0xf06d', [], 'Display', None],
  [ 0xf06e, 'UnicodeCharacter0xf06e', [], 'Display', None],
  [ 0xf06f, 'UnicodeCharacter0xf06f', [], 'Display', None],
  [ 0xf070, 'UnicodeCharacter0xf070', [], 'Display', None],
  [ 0xf071, 'UnicodeCharacter0xf071', [], 'Display', None],
  [ 0xf072, 'UnicodeCharacter0xf072', [], 'Display', None],
  [ 0xf073, 'UnicodeCharacter0xf073', [], 'Display', None],
  [ 0xf074, 'UnicodeCharacter0xf074', [], 'Display', None],
  [ 0xf075, 'UnicodeCharacter0xf075', [], 'Display', None],
  [ 0xf076, 'UnicodeCharacter0xf076', [], 'Display', None],
  [ 0xf077, 'UnicodeCharacter0xf077', [], 'Display', None],
  [ 0xf078, 'UnicodeCharacter0xf078', [], 'Display', None],
  [ 0xf079, 'UnicodeCharacter0xf079', [], 'Display', None],
  [ 0xf07a, 'UnicodeCharacter0xf07a', [], 'Display', None],
  [ 0xf07b, 'UnicodeCharacter0xf07b', [], 'Display', None],
  [ 0xf07c, 'UnicodeCharacter0xf07c', [], 'Display', None],
  [ 0xf07d, 'UnicodeCharacter0xf07d', [], 'Display', None],
  [ 0xf07e, 'UnicodeCharacter0xf07e', [], 'Display', None],
  [ 0xf07f, 'UnicodeCharacter0xf07f', [], 'Display', None],
  [ 0xf080, 'UnicodeCharacter0xf080', [], 'Display', None],
  [ 0xf081, 'UnicodeCharacter0xf081', [], 'Display', None],
  [ 0xf082, 'UnicodeCharacter0xf082', [], 'Display', None],
  [ 0xf083, 'UnicodeCharacter0xf083', [], 'Display', None],
  [ 0xf084, 'UnicodeCharacter0xf084', [], 'Display', None],
  [ 0xf085, 'UnicodeCharacter0xf085', [], 'Display', None],
  [ 0xf086, 'UnicodeCharacter0xf086', [], 'Display', None],
  [ 0xf087, 'UnicodeCharacter0xf087', [], 'Display', None],
  [ 0xf088, 'UnicodeCharacter0xf088', [], 'Display', None],
  [ 0xf089, 'UnicodeCharacter0xf089', [], 'Display', None],
  [ 0xf08a, 'UnicodeCharacter0xf08a', [], 'Display', None],
  [ 0xf08b, 'UnicodeCharacter0xf08b', [], 'Display', None],
  [ 0xf08c, 'UnicodeCharacter0xf08c', [], 'Display', None],
  [ 0xf08d, 'UnicodeCharacter0xf08d', [], 'Display', None],
  [ 0xf08e, 'UnicodeCharacter0xf08e', [], 'Display', None],
  [ 0xf08f, 'UnicodeCharacter0xf08f', [], 'Display', None],
  [ 0xf090, 'UnicodeCharacter0xf090', [], 'Display', None],
  [ 0xf091, 'UnicodeCharacter0xf091', [], 'Display', None],
  [ 0xf092, 'UnicodeCharacter0xf092', [], 'Display', None],
  [ 0xf093, 'UnicodeCharacter0xf093', [], 'Display', None],
  [ 0xf094, 'UnicodeCharacter0xf094', [], 'Display', None],
  [ 0xf095, 'UnicodeCharacter0xf095', [], 'Display', None],
  [ 0xf096, 'UnicodeCharacter0xf096', [], 'Display', None],
  [ 0xf097, 'UnicodeCharacter0xf097', [], 'Display', None],
  [ 0xf098, 'UnicodeCharacter0xf098', [], 'Display', None],
  [ 0xf099, 'UnicodeCharacter0xf099', [], 'Display', None],
  [ 0xf09a, 'UnicodeCharacter0xf09a', [], 'Display', None],
  [ 0xf09b, 'UnicodeCharacter0xf09b', [], 'Display', None],
  [ 0xf09c, 'UnicodeCharacter0xf09c', [], 'Display', None],
  [ 0xf09d, 'UnicodeCharacter0xf09d', [], 'Display', None],
  [ 0xf09e, 'UnicodeCharacter0xf09e', [], 'Display', None],
  [ 0xf09f, 'UnicodeCharacter0xf09f', [], 'Display', None],
  [ 0xf0a0, 'UnicodeCharacter0xf0a0', [], 'Display', None],
  [ 0xf0a1, 'UnicodeCharacter0xf0a1', [], 'Display', None],
  [ 0xf0a2, 'UnicodeCharacter0xf0a2', [], 'Display', None],
  [ 0xf0a3, 'UnicodeCharacter0xf0a3', [], 'Display', None],
  [ 0xf0a4, 'UnicodeCharacter0xf0a4', [], 'Display', None],
  [ 0xf0a5, 'UnicodeCharacter0xf0a5', [], 'Display', None],
  [ 0xf0a6, 'UnicodeCharacter0xf0a6', [], 'Display', None],
  [ 0xf0a7, 'UnicodeCharacter0xf0a7', [], 'Display', None],
  [ 0xf0a8, 'UnicodeCharacter0xf0a8', [], 'Display', None],
  [ 0xf0a9, 'UnicodeCharacter0xf0a9', [], 'Display', None],
  [ 0xf0aa, 'UnicodeCharacter0xf0aa', [], 'Display', None],
  [ 0xf0ab, 'UnicodeCharacter0xf0ab', [], 'Display', None],
  [ 0xf0ac, 'UnicodeCharacter0xf0ac', [], 'Display', None],
  [ 0xf0ad, 'UnicodeCharacter0xf0ad', [], 'Display', None],
  [ 0xf0ae, 'UnicodeCharacter0xf0ae', [], 'Display', None],
  [ 0xf0af, 'UnicodeCharacter0xf0af', [], 'Display', None],
  [ 0xf0b0, 'UnicodeCharacter0xf0b0', [], 'Display', None],
  [ 0xf0b1, 'UnicodeCharacter0xf0b1', [], 'Display', None],
  [ 0xf0b2, 'UnicodeCharacter0xf0b2', [], 'Display', None],
  [ 0xf0b3, 'UnicodeCharacter0xf0b3', [], 'Display', None],
  [ 0xf0b4, 'UnicodeCharacter0xf0b4', [], 'Display', None],
  [ 0xf0b5, 'UnicodeCharacter0xf0b5', [], 'Display', None],
  [ 0xf0b6, 'UnicodeCharacter0xf0b6', [], 'Display', None],
  [ 0xf0b7, 'UnicodeCharacter0xf0b7', [], 'Display', None],
  [ 0xf0b8, 'UnicodeCharacter0xf0b8', [], 'Display', None],
  [ 0xf0b9, 'UnicodeCharacter0xf0b9', [], 'Display', None],
  [ 0xf0ba, 'UnicodeCharacter0xf0ba', [], 'Display', None],
  [ 0xf0bb, 'UnicodeCharacter0xf0bb', [], 'Display', None],
  [ 0xf0bc, 'UnicodeCharacter0xf0bc', [], 'Display', None],
  [ 0xf0bd, 'UnicodeCharacter0xf0bd', [], 'Display', None],
  [ 0xf0be, 'UnicodeCharacter0xf0be', [], 'Display', None],
  [ 0xf0bf, 'UnicodeCharacter0xf0bf', [], 'Display', None],
  [ 0xf0c0, 'UnicodeCharacter0xf0c0', [], 'Display', None],
  [ 0xf0c1, 'UnicodeCharacter0xf0c1', [], 'Display', None],
  [ 0xf0c2, 'UnicodeCharacter0xf0c2', [], 'Display', None],
  [ 0xf0c3, 'UnicodeCharacter0xf0c3', [], 'Display', None],
  [ 0xf0c4, 'UnicodeCharacter0xf0c4', [], 'Display', None],
  [ 0xf0c5, 'UnicodeCharacter0xf0c5', [], 'Display', None],
  [ 0xf0c6, 'UnicodeCharacter0xf0c6', [], 'Display', None],
  [ 0xf0c7, 'UnicodeCharacter0xf0c7', [], 'Display', None],
  [ 0xf0c8, 'UnicodeCharacter0xf0c8', [], 'Display', None],
  [ 0xf0c9, 'UnicodeCharacter0xf0c9', [], 'Display', None],
  [ 0xf0ca, 'UnicodeCharacter0xf0ca', [], 'Display', None],
  [ 0xf0cb, 'UnicodeCharacter0xf0cb', [], 'Display', None],
  [ 0xf0cc, 'UnicodeCharacter0xf0cc', [], 'Display', None],
  [ 0xf0cd, 'UnicodeCharacter0xf0cd', [], 'Display', None],
  [ 0xf0ce, 'UnicodeCharacter0xf0ce', [], 'Display', None],
  [ 0xf0cf, 'UnicodeCharacter0xf0cf', [], 'Display', None],
  [ 0xf0d0, 'UnicodeCharacter0xf0d0', [], 'Display', None],
  [ 0xf0d1, 'UnicodeCharacter0xf0d1', [], 'Display', None],
  [ 0xf0d2, 'UnicodeCharacter0xf0d2', [], 'Display', None],
  [ 0xf0d3, 'UnicodeCharacter0xf0d3', [], 'Display', None],
  [ 0xf0d4, 'UnicodeCharacter0xf0d4', [], 'Display', None],
  [ 0xf0d5, 'UnicodeCharacter0xf0d5', [], 'Display', None],
  [ 0xf0d6, 'UnicodeCharacter0xf0d6', [], 'Display', None],
  [ 0xf0d7, 'UnicodeCharacter0xf0d7', [], 'Display', None],
  [ 0xf0d8, 'UnicodeCharacter0xf0d8', [], 'Display', None],
  [ 0xf0d9, 'UnicodeCharacter0xf0d9', [], 'Display', None],
  [ 0xf0da, 'UnicodeCharacter0xf0da', [], 'Display', None],
  [ 0xf0db, 'UnicodeCharacter0xf0db', [], 'Display', None],
  [ 0xf0dc, 'UnicodeCharacter0xf0dc', [], 'Display', None],
  [ 0xf0dd, 'UnicodeCharacter0xf0dd', [], 'Display', None],
  [ 0xf0de, 'UnicodeCharacter0xf0de', [], 'Display', None],
  [ 0xf0df, 'UnicodeCharacter0xf0df', [], 'Display', None],
  [ 0xf0e0, 'UnicodeCharacter0xf0e0', [], 'Display', None],
  [ 0xf0e1, 'UnicodeCharacter0xf0e1', [], 'Display', None],
  [ 0xf0e2, 'UnicodeCharacter0xf0e2', [], 'Display', None],
  [ 0xf0e3, 'UnicodeCharacter0xf0e3', [], 'Display', None],
  [ 0xf0e4, 'UnicodeCharacter0xf0e4', [], 'Display', None],
  [ 0xf0e5, 'UnicodeCharacter0xf0e5', [], 'Display', None],
  [ 0xf0e6, 'UnicodeCharacter0xf0e6', [], 'Display', None],
  [ 0xf0e7, 'UnicodeCharacter0xf0e7', [], 'Display', None],
  [ 0xf0e8, 'UnicodeCharacter0xf0e8', [], 'Display', None],
  [ 0xf0e9, 'UnicodeCharacter0xf0e9', [], 'Display', None],
  [ 0xf0ea, 'UnicodeCharacter0xf0ea', [], 'Display', None],
  [ 0xf0eb, 'UnicodeCharacter0xf0eb', [], 'Display', None],
  [ 0xf0ec, 'UnicodeCharacter0xf0ec', [], 'Display', None],
  [ 0xf0ed, 'UnicodeCharacter0xf0ed', [], 'Display', None],
  [ 0xf0ee, 'UnicodeCharacter0xf0ee', [], 'Display', None],
  [ 0xf0ef, 'UnicodeCharacter0xf0ef', [], 'Display', None],
  [ 0xf0f0, 'UnicodeCharacter0xf0f0', [], 'Display', None],
  [ 0xf0f1, 'UnicodeCharacter0xf0f1', [], 'Display', None],
  [ 0xf0f2, 'UnicodeCharacter0xf0f2', [], 'Display', None],
  [ 0xf0f3, 'UnicodeCharacter0xf0f3', [], 'Display', None],
  [ 0xf0f4, 'UnicodeCharacter0xf0f4', [], 'Display', None],
  [ 0xf0f5, 'UnicodeCharacter0xf0f5', [], 'Display', None],
  [ 0xf0f6, 'UnicodeCharacter0xf0f6', [], 'Display', None],
  [ 0xf0f7, 'UnicodeCharacter0xf0f7', [], 'Display', None],
  [ 0xf0f8, 'UnicodeCharacter0xf0f8', [], 'Display', None],
  [ 0xf0f9, 'UnicodeCharacter0xf0f9', [], 'Display', None],
  [ 0xf0fa, 'UnicodeCharacter0xf0fa', [], 'Display', None],
  [ 0xf0fb, 'UnicodeCharacter0xf0fb', [], 'Display', None],
  [ 0xf0fc, 'UnicodeCharacter0xf0fc', [], 'Display', None],
  [ 0xf0fd, 'UnicodeCharacter0xf0fd', [], 'Display', None],
  [ 0xf0fe, 'UnicodeCharacter0xf0fe', [], 'Display', None],
  [ 0xf0ff, 'UnicodeCharacter0xf0ff', [], 'Display', None],
  [ 0xf100, 'UnicodeCharacter0xf100', [], 'Display', None],
  [ 0xf101, 'UnicodeCharacter0xf101', [], 'Display', None],
  [ 0xf102, 'UnicodeCharacter0xf102', [], 'Display', None],
  [ 0xf103, 'UnicodeCharacter0xf103', [], 'Display', None],
  [ 0xf104, 'UnicodeCharacter0xf104', [], 'Display', None],
  [ 0xf105, 'UnicodeCharacter0xf105', [], 'Display', None],
  [ 0xf106, 'UnicodeCharacter0xf106', [], 'Display', None],
  [ 0xf107, 'UnicodeCharacter0xf107', [], 'Display', None],
  [ 0xf108, 'UnicodeCharacter0xf108', [], 'Display', None],
  [ 0xf109, 'UnicodeCharacter0xf109', [], 'Display', None],
  [ 0xf10a, 'UnicodeCharacter0xf10a', [], 'Display', None],
  [ 0xf10b, 'UnicodeCharacter0xf10b', [], 'Display', None],
  [ 0xf10c, 'UnicodeCharacter0xf10c', [], 'Display', None],
  [ 0xf10d, 'UnicodeCharacter0xf10d', [], 'Display', None],
  [ 0xf10e, 'UnicodeCharacter0xf10e', [], 'Display', None],
  [ 0xf10f, 'UnicodeCharacter0xf10f', [], 'Display', None],
  [ 0xf110, 'UnicodeCharacter0xf110', [], 'Display', None],
  [ 0xf111, 'UnicodeCharacter0xf111', [], 'Display', None],
  [ 0xf112, 'UnicodeCharacter0xf112', [], 'Display', None],
  [ 0xf113, 'UnicodeCharacter0xf113', [], 'Display', None],
  [ 0xf114, 'UnicodeCharacter0xf114', [], 'Display', None],
  [ 0xf115, 'UnicodeCharacter0xf115', [], 'Display', None],
  [ 0xf116, 'UnicodeCharacter0xf116', [], 'Display', None],
  [ 0xf117, 'UnicodeCharacter0xf117', [], 'Display', None],
  [ 0xf118, 'UnicodeCharacter0xf118', [], 'Display', None],
  [ 0xf119, 'UnicodeCharacter0xf119', [], 'Display', None],
  [ 0xf11a, 'UnicodeCharacter0xf11a', [], 'Display', None],
  [ 0xf11b, 'UnicodeCharacter0xf11b', [], 'Display', None],
  [ 0xf11c, 'UnicodeCharacter0xf11c', [], 'Display', None],
  [ 0xf11d, 'UnicodeCharacter0xf11d', [], 'Display', None],
  [ 0xf11e, 'UnicodeCharacter0xf11e', [], 'Display', None],
  [ 0xf11f, 'UnicodeCharacter0xf11f', [], 'Display', None],
  [ 0xf120, 'UnicodeCharacter0xf120', [], 'Display', None],
  [ 0xf121, 'UnicodeCharacter0xf121', [], 'Display', None],
  [ 0xf122, 'UnicodeCharacter0xf122', [], 'Display', None],
  [ 0xf123, 'UnicodeCharacter0xf123', [], 'Display', None],
  [ 0xf124, 'UnicodeCharacter0xf124', [], 'Display', None],
  [ 0xf125, 'UnicodeCharacter0xf125', [], 'Display', None],
  [ 0xf126, 'UnicodeCharacter0xf126', [], 'Display', None],
  [ 0xf127, 'UnicodeCharacter0xf127', [], 'Display', None],
  [ 0xf128, 'UnicodeCharacter0xf128', [], 'Display', None],
  [ 0xf129, 'UnicodeCharacter0xf129', [], 'Display', None],
  [ 0xf12a, 'UnicodeCharacter0xf12a', [], 'Display', None],
  [ 0xf12b, 'UnicodeCharacter0xf12b', [], 'Display', None],
  [ 0xf12c, 'UnicodeCharacter0xf12c', [], 'Display', None],
  [ 0xf12d, 'UnicodeCharacter0xf12d', [], 'Display', None],
  [ 0xf12e, 'UnicodeCharacter0xf12e', [], 'Display', None],
  [ 0xf12f, 'UnicodeCharacter0xf12f', [], 'Display', None],
  [ 0xf130, 'UnicodeCharacter0xf130', [], 'Display', None],
  [ 0xf131, 'UnicodeCharacter0xf131', [], 'Display', None],
  [ 0xf132, 'UnicodeCharacter0xf132', [], 'Display', None],
  [ 0xf133, 'UnicodeCharacter0xf133', [], 'Display', None],
  [ 0xf134, 'UnicodeCharacter0xf134', [], 'Display', None],
  [ 0xf135, 'UnicodeCharacter0xf135', [], 'Display', None],
  [ 0xf136, 'UnicodeCharacter0xf136', [], 'Display', None],
  [ 0xf137, 'UnicodeCharacter0xf137', [], 'Display', None],
  [ 0xf138, 'UnicodeCharacter0xf138', [], 'Display', None],
  [ 0xf139, 'UnicodeCharacter0xf139', [], 'Display', None],
  [ 0xf13a, 'UnicodeCharacter0xf13a', [], 'Display', None],
  [ 0xf13b, 'UnicodeCharacter0xf13b', [], 'Display', None],
  [ 0xf13c, 'UnicodeCharacter0xf13c', [], 'Display', None],
  [ 0xf13d, 'UnicodeCharacter0xf13d', [], 'Display', None],
  [ 0xf13e, 'UnicodeCharacter0xf13e', [], 'Display', None],
  [ 0xf13f, 'UnicodeCharacter0xf13f', [], 'Display', None],
  [ 0xf140, 'UnicodeCharacter0xf140', [], 'Display', None],
  [ 0xf141, 'UnicodeCharacter0xf141', [], 'Display', None],
  [ 0xf142, 'UnicodeCharacter0xf142', [], 'Display', None],
  [ 0xf143, 'UnicodeCharacter0xf143', [], 'Display', None],
  [ 0xf144, 'UnicodeCharacter0xf144', [], 'Display', None],
  [ 0xf145, 'UnicodeCharacter0xf145', [], 'Display', None],
  [ 0xf146, 'UnicodeCharacter0xf146', [], 'Display', None],
  [ 0xf147, 'UnicodeCharacter0xf147', [], 'Display', None],
  [ 0xf148, 'UnicodeCharacter0xf148', [], 'Display', None],
  [ 0xf149, 'UnicodeCharacter0xf149', [], 'Display', None],
  [ 0xf14a, 'UnicodeCharacter0xf14a', [], 'Display', None],
  [ 0xf14b, 'UnicodeCharacter0xf14b', [], 'Display', None],
  [ 0xf14c, 'UnicodeCharacter0xf14c', [], 'Display', None],
  [ 0xf14d, 'UnicodeCharacter0xf14d', [], 'Display', None],
  [ 0xf14e, 'UnicodeCharacter0xf14e', [], 'Display', None],
  [ 0xf14f, 'UnicodeCharacter0xf14f', [], 'Display', None],
  [ 0xf150, 'UnicodeCharacter0xf150', [], 'Display', None],
  [ 0xf151, 'UnicodeCharacter0xf151', [], 'Display', None],
  [ 0xf152, 'UnicodeCharacter0xf152', [], 'Display', None],
  [ 0xf153, 'UnicodeCharacter0xf153', [], 'Display', None],
  [ 0xf154, 'UnicodeCharacter0xf154', [], 'Display', None],
  [ 0xf155, 'UnicodeCharacter0xf155', [], 'Display', None],
  [ 0xf156, 'UnicodeCharacter0xf156', [], 'Display', None],
  [ 0xf157, 'UnicodeCharacter0xf157', [], 'Display', None],
  [ 0xf158, 'UnicodeCharacter0xf158', [], 'Display', None],
  [ 0xf159, 'UnicodeCharacter0xf159', [], 'Display', None],
  [ 0xf15a, 'UnicodeCharacter0xf15a', [], 'Display', None],
  [ 0xf15b, 'UnicodeCharacter0xf15b', [], 'Display', None],
  [ 0xf15c, 'UnicodeCharacter0xf15c', [], 'Display', None],
  [ 0xf15d, 'UnicodeCharacter0xf15d', [], 'Display', None],
  [ 0xf15e, 'UnicodeCharacter0xf15e', [], 'Display', None],
  [ 0xf15f, 'UnicodeCharacter0xf15f', [], 'Display', None],
  [ 0xf160, 'UnicodeCharacter0xf160', [], 'Display', None],
  [ 0xf161, 'UnicodeCharacter0xf161', [], 'Display', None],
  [ 0xf162, 'UnicodeCharacter0xf162', [], 'Display', None],
  [ 0xf163, 'UnicodeCharacter0xf163', [], 'Display', None],
  [ 0xf164, 'UnicodeCharacter0xf164', [], 'Display', None],
  [ 0xf165, 'UnicodeCharacter0xf165', [], 'Display', None],
  [ 0xf166, 'UnicodeCharacter0xf166', [], 'Display', None],
  [ 0xf167, 'UnicodeCharacter0xf167', [], 'Display', None],
  [ 0xf168, 'UnicodeCharacter0xf168', [], 'Display', None],
  [ 0xf169, 'UnicodeCharacter0xf169', [], 'Display', None],
  [ 0xf16a, 'UnicodeCharacter0xf16a', [], 'Display', None],
  [ 0xf16b, 'UnicodeCharacter0xf16b', [], 'Display', None],
  [ 0xf16c, 'UnicodeCharacter0xf16c', [], 'Display', None],
  [ 0xf16d, 'UnicodeCharacter0xf16d', [], 'Display', None],
  [ 0xf16e, 'UnicodeCharacter0xf16e', [], 'Display', None],
  [ 0xf16f, 'UnicodeCharacter0xf16f', [], 'Display', None],
  [ 0xf170, 'UnicodeCharacter0xf170', [], 'Display', None],
  [ 0xf171, 'UnicodeCharacter0xf171', [], 'Display', None],
  [ 0xf172, 'UnicodeCharacter0xf172', [], 'Display', None],
  [ 0xf173, 'UnicodeCharacter0xf173', [], 'Display', None],
  [ 0xf174, 'UnicodeCharacter0xf174', [], 'Display', None],
  [ 0xf175, 'UnicodeCharacter0xf175', [], 'Display', None],
  [ 0xf176, 'UnicodeCharacter0xf176', [], 'Display', None],
  [ 0xf177, 'UnicodeCharacter0xf177', [], 'Display', None],
  [ 0xf178, 'UnicodeCharacter0xf178', [], 'Display', None],
  [ 0xf179, 'UnicodeCharacter0xf179', [], 'Display', None],
  [ 0xf17a, 'UnicodeCharacter0xf17a', [], 'Display', None],
  [ 0xf17b, 'UnicodeCharacter0xf17b', [], 'Display', None],
  [ 0xf17c, 'UnicodeCharacter0xf17c', [], 'Display', None],
  [ 0xf17d, 'UnicodeCharacter0xf17d', [], 'Display', None],
  [ 0xf17e, 'UnicodeCharacter0xf17e', [], 'Display', None],
  [ 0xf17f, 'UnicodeCharacter0xf17f', [], 'Display', None],
  [ 0xf180, 'UnicodeCharacter0xf180', [], 'Display', None],
  [ 0xf181, 'UnicodeCharacter0xf181', [], 'Display', None],
  [ 0xf182, 'UnicodeCharacter0xf182', [], 'Display', None],
  [ 0xf183, 'UnicodeCharacter0xf183', [], 'Display', None],
  [ 0xf184, 'UnicodeCharacter0xf184', [], 'Display', None],
  [ 0xf185, 'UnicodeCharacter0xf185', [], 'Display', None],
  [ 0xf186, 'UnicodeCharacter0xf186', [], 'Display', None],
  [ 0xf187, 'UnicodeCharacter0xf187', [], 'Display', None],
  [ 0xf188, 'UnicodeCharacter0xf188', [], 'Display', None],
  [ 0xf189, 'UnicodeCharacter0xf189', [], 'Display', None],
  [ 0xf18a, 'UnicodeCharacter0xf18a', [], 'Display', None],
  [ 0xf18b, 'UnicodeCharacter0xf18b', [], 'Display', None],
  [ 0xf18c, 'UnicodeCharacter0xf18c', [], 'Display', None],
  [ 0xf18d, 'UnicodeCharacter0xf18d', [], 'Display', None],
  [ 0xf18e, 'UnicodeCharacter0xf18e', [], 'Display', None],
  [ 0xf18f, 'UnicodeCharacter0xf18f', [], 'Display', None],
  [ 0xf190, 'UnicodeCharacter0xf190', [], 'Display', None],
  [ 0xf191, 'UnicodeCharacter0xf191', [], 'Display', None],
  [ 0xf192, 'UnicodeCharacter0xf192', [], 'Display', None],
  [ 0xf193, 'UnicodeCharacter0xf193', [], 'Display', None],
  [ 0xf194, 'UnicodeCharacter0xf194', [], 'Display', None],
  [ 0xf195, 'UnicodeCharacter0xf195', [], 'Display', None],
  [ 0xf196, 'UnicodeCharacter0xf196', [], 'Display', None],
  [ 0xf197, 'UnicodeCharacter0xf197', [], 'Display', None],
  [ 0xf198, 'UnicodeCharacter0xf198', [], 'Display', None],
  [ 0xf199, 'UnicodeCharacter0xf199', [], 'Display', None],
  [ 0xf19a, 'UnicodeCharacter0xf19a', [], 'Display', None],
  [ 0xf19b, 'UnicodeCharacter0xf19b', [], 'Display', None],
  [ 0xf19c, 'UnicodeCharacter0xf19c', [], 'Display', None],
  [ 0xf19d, 'UnicodeCharacter0xf19d', [], 'Display', None],
  [ 0xf19e, 'UnicodeCharacter0xf19e', [], 'Display', None],
  [ 0xf19f, 'UnicodeCharacter0xf19f', [], 'Display', None],
  [ 0xf1a0, 'UnicodeCharacter0xf1a0', [], 'Display', None],
  [ 0xf1a1, 'UnicodeCharacter0xf1a1', [], 'Display', None],
  [ 0xf1a2, 'UnicodeCharacter0xf1a2', [], 'Display', None],
  [ 0xf1a3, 'UnicodeCharacter0xf1a3', [], 'Display', None],
  [ 0xf1a4, 'UnicodeCharacter0xf1a4', [], 'Display', None],
  [ 0xf1a5, 'UnicodeCharacter0xf1a5', [], 'Display', None],
  [ 0xf1a6, 'UnicodeCharacter0xf1a6', [], 'Display', None],
  [ 0xf1a7, 'UnicodeCharacter0xf1a7', [], 'Display', None],
  [ 0xf1a8, 'UnicodeCharacter0xf1a8', [], 'Display', None],
  [ 0xf1a9, 'UnicodeCharacter0xf1a9', [], 'Display', None],
  [ 0xf1aa, 'UnicodeCharacter0xf1aa', [], 'Display', None],
  [ 0xf1ab, 'UnicodeCharacter0xf1ab', [], 'Display', None],
  [ 0xf1ac, 'UnicodeCharacter0xf1ac', [], 'Display', None],
  [ 0xf1ad, 'UnicodeCharacter0xf1ad', [], 'Display', None],
  [ 0xf1ae, 'UnicodeCharacter0xf1ae', [], 'Display', None],
  [ 0xf1af, 'UnicodeCharacter0xf1af', [], 'Display', None],
  [ 0xf1b0, 'UnicodeCharacter0xf1b0', [], 'Display', None],
  [ 0xf1b1, 'UnicodeCharacter0xf1b1', [], 'Display', None],
  [ 0xf1b2, 'UnicodeCharacter0xf1b2', [], 'Display', None],
  [ 0xf1b3, 'UnicodeCharacter0xf1b3', [], 'Display', None],
  [ 0xf1b4, 'UnicodeCharacter0xf1b4', [], 'Display', None],
  [ 0xf1b5, 'UnicodeCharacter0xf1b5', [], 'Display', None],
  [ 0xf1b6, 'UnicodeCharacter0xf1b6', [], 'Display', None],
  [ 0xf1b7, 'UnicodeCharacter0xf1b7', [], 'Display', None],
  [ 0xf1b8, 'UnicodeCharacter0xf1b8', [], 'Display', None],
  [ 0xf1b9, 'UnicodeCharacter0xf1b9', [], 'Display', None],
  [ 0xf1ba, 'UnicodeCharacter0xf1ba', [], 'Display', None],
  [ 0xf1bb, 'UnicodeCharacter0xf1bb', [], 'Display', None],
  [ 0xf1bc, 'UnicodeCharacter0xf1bc', [], 'Display', None],
  [ 0xf1bd, 'UnicodeCharacter0xf1bd', [], 'Display', None],
  [ 0xf1be, 'UnicodeCharacter0xf1be', [], 'Display', None],
  [ 0xf1bf, 'UnicodeCharacter0xf1bf', [], 'Display', None],
  [ 0xf1c0, 'UnicodeCharacter0xf1c0', [], 'Display', None],
  [ 0xf1c1, 'UnicodeCharacter0xf1c1', [], 'Display', None],
  [ 0xf1c2, 'UnicodeCharacter0xf1c2', [], 'Display', None],
  [ 0xf1c3, 'UnicodeCharacter0xf1c3', [], 'Display', None],
  [ 0xf1c4, 'UnicodeCharacter0xf1c4', [], 'Display', None],
  [ 0xf1c5, 'UnicodeCharacter0xf1c5', [], 'Display', None],
  [ 0xf1c6, 'UnicodeCharacter0xf1c6', [], 'Display', None],
  [ 0xf1c7, 'UnicodeCharacter0xf1c7', [], 'Display', None],
  [ 0xf1c8, 'UnicodeCharacter0xf1c8', [], 'Display', None],
  [ 0xf1c9, 'UnicodeCharacter0xf1c9', [], 'Display', None],
  [ 0xf1ca, 'UnicodeCharacter0xf1ca', [], 'Display', None],
  [ 0xf1cb, 'UnicodeCharacter0xf1cb', [], 'Display', None],
  [ 0xf1cc, 'UnicodeCharacter0xf1cc', [], 'Display', None],
  [ 0xf1cd, 'UnicodeCharacter0xf1cd', [], 'Display', None],
  [ 0xf1ce, 'UnicodeCharacter0xf1ce', [], 'Display', None],
  [ 0xf1cf, 'UnicodeCharacter0xf1cf', [], 'Display', None],
  [ 0xf1d0, 'UnicodeCharacter0xf1d0', [], 'Display', None],
  [ 0xf1d1, 'UnicodeCharacter0xf1d1', [], 'Display', None],
  [ 0xf1d2, 'UnicodeCharacter0xf1d2', [], 'Display', None],
  [ 0xf1d3, 'UnicodeCharacter0xf1d3', [], 'Display', None],
  [ 0xf1d4, 'UnicodeCharacter0xf1d4', [], 'Display', None],
  [ 0xf1d5, 'UnicodeCharacter0xf1d5', [], 'Display', None],
  [ 0xf1d6, 'UnicodeCharacter0xf1d6', [], 'Display', None],
  [ 0xf1d7, 'UnicodeCharacter0xf1d7', [], 'Display', None],
  [ 0xf1d8, 'UnicodeCharacter0xf1d8', [], 'Display', None],
  [ 0xf1d9, 'UnicodeCharacter0xf1d9', [], 'Display', None],
  [ 0xf1da, 'UnicodeCharacter0xf1da', [], 'Display', None],
  [ 0xf1db, 'UnicodeCharacter0xf1db', [], 'Display', None],
  [ 0xf1dc, 'UnicodeCharacter0xf1dc', [], 'Display', None],
  [ 0xf1dd, 'UnicodeCharacter0xf1dd', [], 'Display', None],
  [ 0xf1de, 'UnicodeCharacter0xf1de', [], 'Display', None],
  [ 0xf1df, 'UnicodeCharacter0xf1df', [], 'Display', None],
  [ 0xf1e0, 'UnicodeCharacter0xf1e0', [], 'Display', None],
  [ 0xf1e1, 'UnicodeCharacter0xf1e1', [], 'Display', None],
  [ 0xf1e2, 'UnicodeCharacter0xf1e2', [], 'Display', None],
  [ 0xf1e3, 'UnicodeCharacter0xf1e3', [], 'Display', None],
  [ 0xf1e4, 'UnicodeCharacter0xf1e4', [], 'Display', None],
  [ 0xf1e5, 'UnicodeCharacter0xf1e5', [], 'Display', None],
  [ 0xf1e6, 'UnicodeCharacter0xf1e6', [], 'Display', None],
  [ 0xf1e7, 'UnicodeCharacter0xf1e7', [], 'Display', None],
  [ 0xf1e8, 'UnicodeCharacter0xf1e8', [], 'Display', None],
  [ 0xf1e9, 'UnicodeCharacter0xf1e9', [], 'Display', None],
  [ 0xf1ea, 'UnicodeCharacter0xf1ea', [], 'Display', None],
  [ 0xf1eb, 'UnicodeCharacter0xf1eb', [], 'Display', None],
  [ 0xf1ec, 'UnicodeCharacter0xf1ec', [], 'Display', None],
  [ 0xf1ed, 'UnicodeCharacter0xf1ed', [], 'Display', None],
  [ 0xf1ee, 'UnicodeCharacter0xf1ee', [], 'Display', None],
  [ 0xf1ef, 'UnicodeCharacter0xf1ef', [], 'Display', None],
  [ 0xf1f0, 'UnicodeCharacter0xf1f0', [], 'Display', None],
  [ 0xf1f1, 'UnicodeCharacter0xf1f1', [], 'Display', None],
  [ 0xf1f2, 'UnicodeCharacter0xf1f2', [], 'Display', None],
  [ 0xf1f3, 'UnicodeCharacter0xf1f3', [], 'Display', None],
  [ 0xf1f4, 'UnicodeCharacter0xf1f4', [], 'Display', None],
  [ 0xf1f5, 'UnicodeCharacter0xf1f5', [], 'Display', None],
  [ 0xf1f6, 'UnicodeCharacter0xf1f6', [], 'Display', None],
  [ 0xf1f7, 'UnicodeCharacter0xf1f7', [], 'Display', None],
  [ 0xf1f8, 'UnicodeCharacter0xf1f8', [], 'Display', None],
  [ 0xf1f9, 'UnicodeCharacter0xf1f9', [], 'Display', None],
  [ 0xf1fa, 'UnicodeCharacter0xf1fa', [], 'Display', None],
  [ 0xf1fb, 'UnicodeCharacter0xf1fb', [], 'Display', None],
  [ 0xf1fc, 'UnicodeCharacter0xf1fc', [], 'Display', None],
  [ 0xf1fd, 'UnicodeCharacter0xf1fd', [], 'Display', None],
  [ 0xf1fe, 'UnicodeCharacter0xf1fe', [], 'Display', None],
  [ 0xf1ff, 'UnicodeCharacter0xf1ff', [], 'Display', None],
  [ 0xf200, 'UnicodeCharacter0xf200', [], 'Display', None],
  [ 0xf201, 'UnicodeCharacter0xf201', [], 'Display', None],
  [ 0xf202, 'UnicodeCharacter0xf202', [], 'Display', None],
  [ 0xf203, 'UnicodeCharacter0xf203', [], 'Display', None],
  [ 0xf204, 'UnicodeCharacter0xf204', [], 'Display', None],
  [ 0xf205, 'UnicodeCharacter0xf205', [], 'Display', None],
  [ 0xf206, 'UnicodeCharacter0xf206', [], 'Display', None],
  [ 0xf207, 'UnicodeCharacter0xf207', [], 'Display', None],
  [ 0xf208, 'UnicodeCharacter0xf208', [], 'Display', None],
  [ 0xf209, 'UnicodeCharacter0xf209', [], 'Display', None],
  [ 0xf20a, 'UnicodeCharacter0xf20a', [], 'Display', None],
  [ 0xf20b, 'UnicodeCharacter0xf20b', [], 'Display', None],
  [ 0xf20c, 'UnicodeCharacter0xf20c', [], 'Display', None],
  [ 0xf20d, 'UnicodeCharacter0xf20d', [], 'Display', None],
  [ 0xf20e, 'UnicodeCharacter0xf20e', [], 'Display', None],
  [ 0xf20f, 'UnicodeCharacter0xf20f', [], 'Display', None],
  [ 0xf210, 'UnicodeCharacter0xf210', [], 'Display', None],
  [ 0xf211, 'UnicodeCharacter0xf211', [], 'Display', None],
  [ 0xf212, 'UnicodeCharacter0xf212', [], 'Display', None],
  [ 0xf213, 'UnicodeCharacter0xf213', [], 'Display', None],
  [ 0xf214, 'UnicodeCharacter0xf214', [], 'Display', None],
  [ 0xf215, 'UnicodeCharacter0xf215', [], 'Display', None],
  [ 0xf216, 'UnicodeCharacter0xf216', [], 'Display', None],
  [ 0xf217, 'UnicodeCharacter0xf217', [], 'Display', None],
  [ 0xf218, 'UnicodeCharacter0xf218', [], 'Display', None],
  [ 0xf219, 'UnicodeCharacter0xf219', [], 'Display', None],
  [ 0xf21a, 'UnicodeCharacter0xf21a', [], 'Display', None],
  [ 0xf21b, 'UnicodeCharacter0xf21b', [], 'Display', None],
  [ 0xf21c, 'UnicodeCharacter0xf21c', [], 'Display', None],
  [ 0xf21d, 'UnicodeCharacter0xf21d', [], 'Display', None],
  [ 0xf21e, 'UnicodeCharacter0xf21e', [], 'Display', None],
  [ 0xf21f, 'UnicodeCharacter0xf21f', [], 'Display', None],
  [ 0xf220, 'UnicodeCharacter0xf220', [], 'Display', None],
  [ 0xf221, 'UnicodeCharacter0xf221', [], 'Display', None],
  [ 0xf222, 'UnicodeCharacter0xf222', [], 'Display', None],
  [ 0xf223, 'UnicodeCharacter0xf223', [], 'Display', None],
  [ 0xf224, 'UnicodeCharacter0xf224', [], 'Display', None],
  [ 0xf225, 'UnicodeCharacter0xf225', [], 'Display', None],
  [ 0xf226, 'UnicodeCharacter0xf226', [], 'Display', None],
  [ 0xf227, 'UnicodeCharacter0xf227', [], 'Display', None],
  [ 0xf228, 'UnicodeCharacter0xf228', [], 'Display', None],
  [ 0xf229, 'UnicodeCharacter0xf229', [], 'Display', None],
  [ 0xf22a, 'UnicodeCharacter0xf22a', [], 'Display', None],
  [ 0xf22b, 'UnicodeCharacter0xf22b', [], 'Display', None],
  [ 0xf22c, 'UnicodeCharacter0xf22c', [], 'Display', None],
  [ 0xf22d, 'UnicodeCharacter0xf22d', [], 'Display', None],
  [ 0xf22e, 'UnicodeCharacter0xf22e', [], 'Display', None],
  [ 0xf22f, 'UnicodeCharacter0xf22f', [], 'Display', None],
  [ 0xf230, 'UnicodeCharacter0xf230', [], 'Display', None],
  [ 0xf231, 'UnicodeCharacter0xf231', [], 'Display', None],
  [ 0xf232, 'UnicodeCharacter0xf232', [], 'Display', None],
  [ 0xf233, 'UnicodeCharacter0xf233', [], 'Display', None],
  [ 0xf234, 'UnicodeCharacter0xf234', [], 'Display', None],
  [ 0xf235, 'UnicodeCharacter0xf235', [], 'Display', None],
  [ 0xf236, 'UnicodeCharacter0xf236', [], 'Display', None],
  [ 0xf237, 'UnicodeCharacter0xf237', [], 'Display', None],
  [ 0xf238, 'UnicodeCharacter0xf238', [], 'Display', None],
  [ 0xf239, 'UnicodeCharacter0xf239', [], 'Display', None],
  [ 0xf23a, 'UnicodeCharacter0xf23a', [], 'Display', None],
  [ 0xf23b, 'UnicodeCharacter0xf23b', [], 'Display', None],
  [ 0xf23c, 'UnicodeCharacter0xf23c', [], 'Display', None],
  [ 0xf23d, 'UnicodeCharacter0xf23d', [], 'Display', None],
  [ 0xf23e, 'UnicodeCharacter0xf23e', [], 'Display', None],
  [ 0xf23f, 'UnicodeCharacter0xf23f', [], 'Display', None],
  [ 0xf240, 'UnicodeCharacter0xf240', [], 'Display', None],
  [ 0xf241, 'UnicodeCharacter0xf241', [], 'Display', None],
  [ 0xf242, 'UnicodeCharacter0xf242', [], 'Display', None],
  [ 0xf243, 'UnicodeCharacter0xf243', [], 'Display', None],
  [ 0xf244, 'UnicodeCharacter0xf244', [], 'Display', None],
  [ 0xf245, 'UnicodeCharacter0xf245', [], 'Display', None],
  [ 0xf246, 'UnicodeCharacter0xf246', [], 'Display', None],
  [ 0xf247, 'UnicodeCharacter0xf247', [], 'Display', None],
  [ 0xf248, 'UnicodeCharacter0xf248', [], 'Display', None],
  [ 0xf249, 'UnicodeCharacter0xf249', [], 'Display', None],
  [ 0xf24a, 'UnicodeCharacter0xf24a', [], 'Display', None],
  [ 0xf24b, 'UnicodeCharacter0xf24b', [], 'Display', None],
  [ 0xf24c, 'UnicodeCharacter0xf24c', [], 'Display', None],
  [ 0xf24d, 'UnicodeCharacter0xf24d', [], 'Display', None],
  [ 0xf24e, 'UnicodeCharacter0xf24e', [], 'Display', None],
  [ 0xf24f, 'UnicodeCharacter0xf24f', [], 'Display', None],
  [ 0xf250, 'UnicodeCharacter0xf250', [], 'Display', None],
  [ 0xf251, 'UnicodeCharacter0xf251', [], 'Display', None],
  [ 0xf252, 'UnicodeCharacter0xf252', [], 'Display', None],
  [ 0xf253, 'UnicodeCharacter0xf253', [], 'Display', None],
  [ 0xf254, 'UnicodeCharacter0xf254', [], 'Display', None],
  [ 0xf255, 'UnicodeCharacter0xf255', [], 'Display', None],
  [ 0xf256, 'UnicodeCharacter0xf256', [], 'Display', None],
  [ 0xf257, 'UnicodeCharacter0xf257', [], 'Display', None],
  [ 0xf258, 'UnicodeCharacter0xf258', [], 'Display', None],
  [ 0xf259, 'UnicodeCharacter0xf259', [], 'Display', None],
  [ 0xf25a, 'UnicodeCharacter0xf25a', [], 'Display', None],
  [ 0xf25b, 'UnicodeCharacter0xf25b', [], 'Display', None],
  [ 0xf25c, 'UnicodeCharacter0xf25c', [], 'Display', None],
  [ 0xf25d, 'UnicodeCharacter0xf25d', [], 'Display', None],
  [ 0xf25e, 'UnicodeCharacter0xf25e', [], 'Display', None],
  [ 0xf25f, 'UnicodeCharacter0xf25f', [], 'Display', None],
  [ 0xf260, 'UnicodeCharacter0xf260', [], 'Display', None],
  [ 0xf261, 'UnicodeCharacter0xf261', [], 'Display', None],
  [ 0xf262, 'UnicodeCharacter0xf262', [], 'Display', None],
  [ 0xf263, 'UnicodeCharacter0xf263', [], 'Display', None],
  [ 0xf264, 'UnicodeCharacter0xf264', [], 'Display', None],
  [ 0xf265, 'UnicodeCharacter0xf265', [], 'Display', None],
  [ 0xf266, 'UnicodeCharacter0xf266', [], 'Display', None],
  [ 0xf267, 'UnicodeCharacter0xf267', [], 'Display', None],
  [ 0xf268, 'UnicodeCharacter0xf268', [], 'Display', None],
  [ 0xf269, 'UnicodeCharacter0xf269', [], 'Display', None],
  [ 0xf26a, 'UnicodeCharacter0xf26a', [], 'Display', None],
  [ 0xf26b, 'UnicodeCharacter0xf26b', [], 'Display', None],
  [ 0xf26c, 'UnicodeCharacter0xf26c', [], 'Display', None],
  [ 0xf26d, 'UnicodeCharacter0xf26d', [], 'Display', None],
  [ 0xf26e, 'UnicodeCharacter0xf26e', [], 'Display', None],
  [ 0xf26f, 'UnicodeCharacter0xf26f', [], 'Display', None],
  [ 0xf270, 'UnicodeCharacter0xf270', [], 'Display', None],
  [ 0xf271, 'UnicodeCharacter0xf271', [], 'Display', None],
  [ 0xf272, 'UnicodeCharacter0xf272', [], 'Display', None],
  [ 0xf273, 'UnicodeCharacter0xf273', [], 'Display', None],
  [ 0xf274, 'UnicodeCharacter0xf274', [], 'Display', None],
  [ 0xf275, 'UnicodeCharacter0xf275', [], 'Display', None],
  [ 0xf276, 'UnicodeCharacter0xf276', [], 'Display', None],
  [ 0xf277, 'UnicodeCharacter0xf277', [], 'Display', None],
  [ 0xf278, 'UnicodeCharacter0xf278', [], 'Display', None],
  [ 0xf279, 'UnicodeCharacter0xf279', [], 'Display', None],
  [ 0xf27a, 'UnicodeCharacter0xf27a', [], 'Display', None],
  [ 0xf27b, 'UnicodeCharacter0xf27b', [], 'Display', None],
  [ 0xf27c, 'UnicodeCharacter0xf27c', [], 'Display', None],
  [ 0xf27d, 'UnicodeCharacter0xf27d', [], 'Display', None],
  [ 0xf27e, 'UnicodeCharacter0xf27e', [], 'Display', None],
  [ 0xf27f, 'UnicodeCharacter0xf27f', [], 'Display', None],
  [ 0xf280, 'UnicodeCharacter0xf280', [], 'Display', None],
  [ 0xf281, 'UnicodeCharacter0xf281', [], 'Display', None],
  [ 0xf282, 'UnicodeCharacter0xf282', [], 'Display', None],
  [ 0xf283, 'UnicodeCharacter0xf283', [], 'Display', None],
  [ 0xf284, 'UnicodeCharacter0xf284', [], 'Display', None],
  [ 0xf285, 'UnicodeCharacter0xf285', [], 'Display', None],
  [ 0xf286, 'UnicodeCharacter0xf286', [], 'Display', None],
  [ 0xf287, 'UnicodeCharacter0xf287', [], 'Display', None],
  [ 0xf288, 'UnicodeCharacter0xf288', [], 'Display', None],
  [ 0xf289, 'UnicodeCharacter0xf289', [], 'Display', None],
  [ 0xf28a, 'UnicodeCharacter0xf28a', [], 'Display', None],
  [ 0xf28b, 'UnicodeCharacter0xf28b', [], 'Display', None],
  [ 0xf28c, 'UnicodeCharacter0xf28c', [], 'Display', None],
  [ 0xf28d, 'UnicodeCharacter0xf28d', [], 'Display', None],
  [ 0xf28e, 'UnicodeCharacter0xf28e', [], 'Display', None],
  [ 0xf28f, 'UnicodeCharacter0xf28f', [], 'Display', None],
  [ 0xf290, 'UnicodeCharacter0xf290', [], 'Display', None],
  [ 0xf291, 'UnicodeCharacter0xf291', [], 'Display', None],
  [ 0xf292, 'UnicodeCharacter0xf292', [], 'Display', None],
  [ 0xf293, 'UnicodeCharacter0xf293', [], 'Display', None],
  [ 0xf294, 'UnicodeCharacter0xf294', [], 'Display', None],
  [ 0xf295, 'UnicodeCharacter0xf295', [], 'Display', None],
  [ 0xf296, 'UnicodeCharacter0xf296', [], 'Display', None],
  [ 0xf297, 'UnicodeCharacter0xf297', [], 'Display', None],
  [ 0xf298, 'UnicodeCharacter0xf298', [], 'Display', None],
  [ 0xf299, 'UnicodeCharacter0xf299', [], 'Display', None],
  [ 0xf29a, 'UnicodeCharacter0xf29a', [], 'Display', None],
  [ 0xf29b, 'UnicodeCharacter0xf29b', [], 'Display', None],
  [ 0xf29c, 'UnicodeCharacter0xf29c', [], 'Display', None],
  [ 0xf29d, 'UnicodeCharacter0xf29d', [], 'Display', None],
  [ 0xf29e, 'UnicodeCharacter0xf29e', [], 'Display', None],
  [ 0xf29f, 'UnicodeCharacter0xf29f', [], 'Display', None],
  [ 0xf2a0, 'UnicodeCharacter0xf2a0', [], 'Display', None],
  [ 0xf2a1, 'UnicodeCharacter0xf2a1', [], 'Display', None],
  [ 0xf2a2, 'UnicodeCharacter0xf2a2', [], 'Display', None],
  [ 0xf2a3, 'UnicodeCharacter0xf2a3', [], 'Display', None],
  [ 0xf2a4, 'UnicodeCharacter0xf2a4', [], 'Display', None],
  [ 0xf2a5, 'UnicodeCharacter0xf2a5', [], 'Display', None],
  [ 0xf2a6, 'UnicodeCharacter0xf2a6', [], 'Display', None],
  [ 0xf2a7, 'UnicodeCharacter0xf2a7', [], 'Display', None],
  [ 0xf2a8, 'UnicodeCharacter0xf2a8', [], 'Display', None],
  [ 0xf2a9, 'UnicodeCharacter0xf2a9', [], 'Display', None],
  [ 0xf2aa, 'UnicodeCharacter0xf2aa', [], 'Display', None],
  [ 0xf2ab, 'UnicodeCharacter0xf2ab', [], 'Display', None],
  [ 0xf2ac, 'UnicodeCharacter0xf2ac', [], 'Display', None],
  [ 0xf2ad, 'UnicodeCharacter0xf2ad', [], 'Display', None],
  [ 0xf2ae, 'UnicodeCharacter0xf2ae', [], 'Display', None],
  [ 0xf2af, 'UnicodeCharacter0xf2af', [], 'Display', None],
  [ 0xf2b0, 'UnicodeCharacter0xf2b0', [], 'Display', None],
  [ 0xf2b1, 'UnicodeCharacter0xf2b1', [], 'Display', None],
  [ 0xf2b2, 'UnicodeCharacter0xf2b2', [], 'Display', None],
  [ 0xf2b3, 'UnicodeCharacter0xf2b3', [], 'Display', None],
  [ 0xf2b4, 'UnicodeCharacter0xf2b4', [], 'Display', None],
  [ 0xf2b5, 'UnicodeCharacter0xf2b5', [], 'Display', None],
  [ 0xf2b6, 'UnicodeCharacter0xf2b6', [], 'Display', None],
  [ 0xf2b7, 'UnicodeCharacter0xf2b7', [], 'Display', None],
  [ 0xf2b8, 'UnicodeCharacter0xf2b8', [], 'Display', None],
  [ 0xf2b9, 'UnicodeCharacter0xf2b9', [], 'Display', None],
  [ 0xf2ba, 'UnicodeCharacter0xf2ba', [], 'Display', None],
  [ 0xf2bb, 'UnicodeCharacter0xf2bb', [], 'Display', None],
  [ 0xf2bc, 'UnicodeCharacter0xf2bc', [], 'Display', None],
  [ 0xf2bd, 'UnicodeCharacter0xf2bd', [], 'Display', None],
  [ 0xf2be, 'UnicodeCharacter0xf2be', [], 'Display', None],
  [ 0xf2bf, 'UnicodeCharacter0xf2bf', [], 'Display', None],
  [ 0xf2c0, 'UnicodeCharacter0xf2c0', [], 'Display', None],
  [ 0xf2c1, 'UnicodeCharacter0xf2c1', [], 'Display', None],
  [ 0xf2c2, 'UnicodeCharacter0xf2c2', [], 'Display', None],
  [ 0xf2c3, 'UnicodeCharacter0xf2c3', [], 'Display', None],
  [ 0xf2c4, 'UnicodeCharacter0xf2c4', [], 'Display', None],
  [ 0xf2c5, 'UnicodeCharacter0xf2c5', [], 'Display', None],
  [ 0xf2c6, 'UnicodeCharacter0xf2c6', [], 'Display', None],
  [ 0xf2c7, 'UnicodeCharacter0xf2c7', [], 'Display', None],
  [ 0xf2c8, 'UnicodeCharacter0xf2c8', [], 'Display', None],
  [ 0xf2c9, 'UnicodeCharacter0xf2c9', [], 'Display', None],
  [ 0xf2ca, 'UnicodeCharacter0xf2ca', [], 'Display', None],
  [ 0xf2cb, 'UnicodeCharacter0xf2cb', [], 'Display', None],
  [ 0xf2cc, 'UnicodeCharacter0xf2cc', [], 'Display', None],
  [ 0xf2cd, 'UnicodeCharacter0xf2cd', [], 'Display', None],
  [ 0xf2ce, 'UnicodeCharacter0xf2ce', [], 'Display', None],
  [ 0xf2cf, 'UnicodeCharacter0xf2cf', [], 'Display', None],
  [ 0xf2d0, 'UnicodeCharacter0xf2d0', [], 'Display', None],
  [ 0xf2d1, 'UnicodeCharacter0xf2d1', [], 'Display', None],
  [ 0xf2d2, 'UnicodeCharacter0xf2d2', [], 'Display', None],
  [ 0xf2d3, 'UnicodeCharacter0xf2d3', [], 'Display', None],
  [ 0xf2d4, 'UnicodeCharacter0xf2d4', [], 'Display', None],
  [ 0xf2d5, 'UnicodeCharacter0xf2d5', [], 'Display', None],
  [ 0xf2d6, 'UnicodeCharacter0xf2d6', [], 'Display', None],
  [ 0xf2d7, 'UnicodeCharacter0xf2d7', [], 'Display', None],
  [ 0xf2d8, 'UnicodeCharacter0xf2d8', [], 'Display', None],
  [ 0xf2d9, 'UnicodeCharacter0xf2d9', [], 'Display', None],
  [ 0xf2da, 'UnicodeCharacter0xf2da', [], 'Display', None],
  [ 0xf2db, 'UnicodeCharacter0xf2db', [], 'Display', None],
  [ 0xf2dc, 'UnicodeCharacter0xf2dc', [], 'Display', None],
  [ 0xf2dd, 'UnicodeCharacter0xf2dd', [], 'Display', None],
  [ 0xf2de, 'UnicodeCharacter0xf2de', [], 'Display', None],
  [ 0xf2df, 'UnicodeCharacter0xf2df', [], 'Display', None],
  [ 0xf2e0, 'UnicodeCharacter0xf2e0', [], 'Display', None],
  [ 0xf2e1, 'UnicodeCharacter0xf2e1', [], 'Display', None],
  [ 0xf2e2, 'UnicodeCharacter0xf2e2', [], 'Display', None],
  [ 0xf2e3, 'UnicodeCharacter0xf2e3', [], 'Display', None],
  [ 0xf2e4, 'UnicodeCharacter0xf2e4', [], 'Display', None],
  [ 0xf2e5, 'UnicodeCharacter0xf2e5', [], 'Display', None],
  [ 0xf2e6, 'UnicodeCharacter0xf2e6', [], 'Display', None],
  [ 0xf2e7, 'UnicodeCharacter0xf2e7', [], 'Display', None],
  [ 0xf2e8, 'UnicodeCharacter0xf2e8', [], 'Display', None],
  [ 0xf2e9, 'UnicodeCharacter0xf2e9', [], 'Display', None],
  [ 0xf2ea, 'UnicodeCharacter0xf2ea', [], 'Display', None],
  [ 0xf2eb, 'UnicodeCharacter0xf2eb', [], 'Display', None],
  [ 0xf2ec, 'UnicodeCharacter0xf2ec', [], 'Display', None],
  [ 0xf2ed, 'UnicodeCharacter0xf2ed', [], 'Display', None],
  [ 0xf2ee, 'UnicodeCharacter0xf2ee', [], 'Display', None],
  [ 0xf2ef, 'UnicodeCharacter0xf2ef', [], 'Display', None],
  [ 0xf2f0, 'UnicodeCharacter0xf2f0', [], 'Display', None],
  [ 0xf2f1, 'UnicodeCharacter0xf2f1', [], 'Display', None],
  [ 0xf2f2, 'UnicodeCharacter0xf2f2', [], 'Display', None],
  [ 0xf2f3, 'UnicodeCharacter0xf2f3', [], 'Display', None],
  [ 0xf2f4, 'UnicodeCharacter0xf2f4', [], 'Display', None],
  [ 0xf2f5, 'UnicodeCharacter0xf2f5', [], 'Display', None],
  [ 0xf2f6, 'UnicodeCharacter0xf2f6', [], 'Display', None],
  [ 0xf2f7, 'UnicodeCharacter0xf2f7', [], 'Display', None],
  [ 0xf2f8, 'UnicodeCharacter0xf2f8', [], 'Display', None],
  [ 0xf2f9, 'UnicodeCharacter0xf2f9', [], 'Display', None],
  [ 0xf2fa, 'UnicodeCharacter0xf2fa', [], 'Display', None],
  [ 0xf2fb, 'UnicodeCharacter0xf2fb', [], 'Display', None],
  [ 0xf2fc, 'UnicodeCharacter0xf2fc', [], 'Display', None],
  [ 0xf2fd, 'UnicodeCharacter0xf2fd', [], 'Display', None],
  [ 0xf2fe, 'UnicodeCharacter0xf2fe', [], 'Display', None],
  [ 0xf2ff, 'UnicodeCharacter0xf2ff', [], 'Display', None],
  [ 0xf300, 'UnicodeCharacter0xf300', [], 'Display', None],
  [ 0xf301, 'UnicodeCharacter0xf301', [], 'Display', None],
  [ 0xf302, 'UnicodeCharacter0xf302', [], 'Display', None],
  [ 0xf303, 'UnicodeCharacter0xf303', [], 'Display', None],
  [ 0xf304, 'UnicodeCharacter0xf304', [], 'Display', None],
  [ 0xf305, 'UnicodeCharacter0xf305', [], 'Display', None],
  [ 0xf306, 'UnicodeCharacter0xf306', [], 'Display', None],
  [ 0xf307, 'UnicodeCharacter0xf307', [], 'Display', None],
  [ 0xf308, 'UnicodeCharacter0xf308', [], 'Display', None],
  [ 0xf309, 'UnicodeCharacter0xf309', [], 'Display', None],
  [ 0xf30a, 'UnicodeCharacter0xf30a', [], 'Display', None],
  [ 0xf30b, 'UnicodeCharacter0xf30b', [], 'Display', None],
  [ 0xf30c, 'UnicodeCharacter0xf30c', [], 'Display', None],
  [ 0xf30d, 'UnicodeCharacter0xf30d', [], 'Display', None],
  [ 0xf30e, 'UnicodeCharacter0xf30e', [], 'Display', None],
  [ 0xf30f, 'UnicodeCharacter0xf30f', [], 'Display', None],
  [ 0xf310, 'UnicodeCharacter0xf310', [], 'Display', None],
  [ 0xf311, 'UnicodeCharacter0xf311', [], 'Display', None],
  [ 0xf312, 'UnicodeCharacter0xf312', [], 'Display', None],
  [ 0xf313, 'UnicodeCharacter0xf313', [], 'Display', None],
  [ 0xf314, 'UnicodeCharacter0xf314', [], 'Display', None],
  [ 0xf315, 'UnicodeCharacter0xf315', [], 'Display', None],
  [ 0xf316, 'UnicodeCharacter0xf316', [], 'Display', None],
  [ 0xf317, 'UnicodeCharacter0xf317', [], 'Display', None],
  [ 0xf318, 'UnicodeCharacter0xf318', [], 'Display', None],
  [ 0xf319, 'UnicodeCharacter0xf319', [], 'Display', None],
  [ 0xf31a, 'UnicodeCharacter0xf31a', [], 'Display', None],
  [ 0xf31b, 'UnicodeCharacter0xf31b', [], 'Display', None],
  [ 0xf31c, 'UnicodeCharacter0xf31c', [], 'Display', None],
  [ 0xf31d, 'UnicodeCharacter0xf31d', [], 'Display', None],
  [ 0xf31e, 'UnicodeCharacter0xf31e', [], 'Display', None],
  [ 0xf31f, 'UnicodeCharacter0xf31f', [], 'Display', None],
  [ 0xf320, 'UnicodeCharacter0xf320', [], 'Display', None],
  [ 0xf321, 'UnicodeCharacter0xf321', [], 'Display', None],
  [ 0xf322, 'UnicodeCharacter0xf322', [], 'Display', None],
  [ 0xf323, 'UnicodeCharacter0xf323', [], 'Display', None],
  [ 0xf324, 'UnicodeCharacter0xf324', [], 'Display', None],
  [ 0xf325, 'UnicodeCharacter0xf325', [], 'Display', None],
  [ 0xf326, 'UnicodeCharacter0xf326', [], 'Display', None],
  [ 0xf327, 'UnicodeCharacter0xf327', [], 'Display', None],
  [ 0xf328, 'UnicodeCharacter0xf328', [], 'Display', None],
  [ 0xf329, 'UnicodeCharacter0xf329', [], 'Display', None],
  [ 0xf32a, 'UnicodeCharacter0xf32a', [], 'Display', None],
  [ 0xf32b, 'UnicodeCharacter0xf32b', [], 'Display', None],
  [ 0xf32c, 'UnicodeCharacter0xf32c', [], 'Display', None],
  [ 0xf32d, 'UnicodeCharacter0xf32d', [], 'Display', None],
  [ 0xf32e, 'UnicodeCharacter0xf32e', [], 'Display', None],
  [ 0xf32f, 'UnicodeCharacter0xf32f', [], 'Display', None],
  [ 0xf330, 'UnicodeCharacter0xf330', [], 'Display', None],
  [ 0xf331, 'UnicodeCharacter0xf331', [], 'Display', None],
  [ 0xf332, 'UnicodeCharacter0xf332', [], 'Display', None],
  [ 0xf333, 'UnicodeCharacter0xf333', [], 'Display', None],
  [ 0xf334, 'UnicodeCharacter0xf334', [], 'Display', None],
  [ 0xf335, 'UnicodeCharacter0xf335', [], 'Display', None],
  [ 0xf336, 'UnicodeCharacter0xf336', [], 'Display', None],
  [ 0xf337, 'UnicodeCharacter0xf337', [], 'Display', None],
  [ 0xf338, 'UnicodeCharacter0xf338', [], 'Display', None],
  [ 0xf339, 'UnicodeCharacter0xf339', [], 'Display', None],
  [ 0xf33a, 'UnicodeCharacter0xf33a', [], 'Display', None],
  [ 0xf33b, 'UnicodeCharacter0xf33b', [], 'Display', None],
  [ 0xf33c, 'UnicodeCharacter0xf33c', [], 'Display', None],
  [ 0xf33d, 'UnicodeCharacter0xf33d', [], 'Display', None],
  [ 0xf33e, 'UnicodeCharacter0xf33e', [], 'Display', None],
  [ 0xf33f, 'UnicodeCharacter0xf33f', [], 'Display', None],
  [ 0xf340, 'UnicodeCharacter0xf340', [], 'Display', None],
  [ 0xf341, 'UnicodeCharacter0xf341', [], 'Display', None],
  [ 0xf342, 'UnicodeCharacter0xf342', [], 'Display', None],
  [ 0xf343, 'UnicodeCharacter0xf343', [], 'Display', None],
  [ 0xf344, 'UnicodeCharacter0xf344', [], 'Display', None],
  [ 0xf345, 'UnicodeCharacter0xf345', [], 'Display', None],
  [ 0xf346, 'UnicodeCharacter0xf346', [], 'Display', None],
  [ 0xf347, 'UnicodeCharacter0xf347', [], 'Display', None],
  [ 0xf348, 'UnicodeCharacter0xf348', [], 'Display', None],
  [ 0xf349, 'UnicodeCharacter0xf349', [], 'Display', None],
  [ 0xf34a, 'UnicodeCharacter0xf34a', [], 'Display', None],
  [ 0xf34b, 'UnicodeCharacter0xf34b', [], 'Display', None],
  [ 0xf34c, 'UnicodeCharacter0xf34c', [], 'Display', None],
  [ 0xf34d, 'UnicodeCharacter0xf34d', [], 'Display', None],
  [ 0xf34e, 'UnicodeCharacter0xf34e', [], 'Display', None],
  [ 0xf34f, 'UnicodeCharacter0xf34f', [], 'Display', None],
  [ 0xf350, 'UnicodeCharacter0xf350', [], 'Display', None],
  [ 0xf351, 'UnicodeCharacter0xf351', [], 'Display', None],
  [ 0xf352, 'UnicodeCharacter0xf352', [], 'Display', None],
  [ 0xf353, 'UnicodeCharacter0xf353', [], 'Display', None],
  [ 0xf354, 'UnicodeCharacter0xf354', [], 'Display', None],
  [ 0xf355, 'UnicodeCharacter0xf355', [], 'Display', None],
  [ 0xf356, 'UnicodeCharacter0xf356', [], 'Display', None],
  [ 0xf357, 'UnicodeCharacter0xf357', [], 'Display', None],
  [ 0xf358, 'UnicodeCharacter0xf358', [], 'Display', None],
  [ 0xf359, 'UnicodeCharacter0xf359', [], 'Display', None],
  [ 0xf35a, 'UnicodeCharacter0xf35a', [], 'Display', None],
  [ 0xf35b, 'UnicodeCharacter0xf35b', [], 'Display', None],
  [ 0xf35c, 'UnicodeCharacter0xf35c', [], 'Display', None],
  [ 0xf35d, 'UnicodeCharacter0xf35d', [], 'Display', None],
  [ 0xf35e, 'UnicodeCharacter0xf35e', [], 'Display', None],
  [ 0xf35f, 'UnicodeCharacter0xf35f', [], 'Display', None],
  [ 0xf360, 'UnicodeCharacter0xf360', [], 'Display', None],
  [ 0xf361, 'UnicodeCharacter0xf361', [], 'Display', None],
  [ 0xf362, 'UnicodeCharacter0xf362', [], 'Display', None],
  [ 0xf363, 'UnicodeCharacter0xf363', [], 'Display', None],
  [ 0xf364, 'UnicodeCharacter0xf364', [], 'Display', None],
  [ 0xf365, 'UnicodeCharacter0xf365', [], 'Display', None],
  [ 0xf366, 'UnicodeCharacter0xf366', [], 'Display', None],
  [ 0xf367, 'UnicodeCharacter0xf367', [], 'Display', None],
  [ 0xf368, 'UnicodeCharacter0xf368', [], 'Display', None],
  [ 0xf369, 'UnicodeCharacter0xf369', [], 'Display', None],
  [ 0xf36a, 'UnicodeCharacter0xf36a', [], 'Display', None],
  [ 0xf36b, 'UnicodeCharacter0xf36b', [], 'Display', None],
  [ 0xf36c, 'UnicodeCharacter0xf36c', [], 'Display', None],
  [ 0xf36d, 'UnicodeCharacter0xf36d', [], 'Display', None],
  [ 0xf36e, 'UnicodeCharacter0xf36e', [], 'Display', None],
  [ 0xf36f, 'UnicodeCharacter0xf36f', [], 'Display', None],
  [ 0xf370, 'UnicodeCharacter0xf370', [], 'Display', None],
  [ 0xf371, 'UnicodeCharacter0xf371', [], 'Display', None],
  [ 0xf372, 'UnicodeCharacter0xf372', [], 'Display', None],
  [ 0xf373, 'UnicodeCharacter0xf373', [], 'Display', None],
  [ 0xf374, 'UnicodeCharacter0xf374', [], 'Display', None],
  [ 0xf375, 'UnicodeCharacter0xf375', [], 'Display', None],
  [ 0xf376, 'UnicodeCharacter0xf376', [], 'Display', None],
  [ 0xf377, 'UnicodeCharacter0xf377', [], 'Display', None],
  [ 0xf378, 'UnicodeCharacter0xf378', [], 'Display', None],
  [ 0xf379, 'UnicodeCharacter0xf379', [], 'Display', None],
  [ 0xf37a, 'UnicodeCharacter0xf37a', [], 'Display', None],
  [ 0xf37b, 'UnicodeCharacter0xf37b', [], 'Display', None],
  [ 0xf37c, 'UnicodeCharacter0xf37c', [], 'Display', None],
  [ 0xf37d, 'UnicodeCharacter0xf37d', [], 'Display', None],
  [ 0xf37e, 'UnicodeCharacter0xf37e', [], 'Display', None],
  [ 0xf37f, 'UnicodeCharacter0xf37f', [], 'Display', None],
  [ 0xf380, 'UnicodeCharacter0xf380', [], 'Display', None],
  [ 0xf381, 'UnicodeCharacter0xf381', [], 'Display', None],
  [ 0xf382, 'UnicodeCharacter0xf382', [], 'Display', None],
  [ 0xf383, 'UnicodeCharacter0xf383', [], 'Display', None],
  [ 0xf384, 'UnicodeCharacter0xf384', [], 'Display', None],
  [ 0xf385, 'UnicodeCharacter0xf385', [], 'Display', None],
  [ 0xf386, 'UnicodeCharacter0xf386', [], 'Display', None],
  [ 0xf387, 'UnicodeCharacter0xf387', [], 'Display', None],
  [ 0xf388, 'UnicodeCharacter0xf388', [], 'Display', None],
  [ 0xf389, 'UnicodeCharacter0xf389', [], 'Display', None],
  [ 0xf38a, 'UnicodeCharacter0xf38a', [], 'Display', None],
  [ 0xf38b, 'UnicodeCharacter0xf38b', [], 'Display', None],
  [ 0xf38c, 'UnicodeCharacter0xf38c', [], 'Display', None],
  [ 0xf38d, 'UnicodeCharacter0xf38d', [], 'Display', None],
  [ 0xf38e, 'UnicodeCharacter0xf38e', [], 'Display', None],
  [ 0xf38f, 'UnicodeCharacter0xf38f', [], 'Display', None],
  [ 0xf390, 'UnicodeCharacter0xf390', [], 'Display', None],
  [ 0xf391, 'UnicodeCharacter0xf391', [], 'Display', None],
  [ 0xf392, 'UnicodeCharacter0xf392', [], 'Display', None],
  [ 0xf393, 'UnicodeCharacter0xf393', [], 'Display', None],
  [ 0xf394, 'UnicodeCharacter0xf394', [], 'Display', None],
  [ 0xf395, 'UnicodeCharacter0xf395', [], 'Display', None],
  [ 0xf396, 'UnicodeCharacter0xf396', [], 'Display', None],
  [ 0xf397, 'UnicodeCharacter0xf397', [], 'Display', None],
  [ 0xf398, 'UnicodeCharacter0xf398', [], 'Display', None],
  [ 0xf399, 'UnicodeCharacter0xf399', [], 'Display', None],
  [ 0xf39a, 'UnicodeCharacter0xf39a', [], 'Display', None],
  [ 0xf39b, 'UnicodeCharacter0xf39b', [], 'Display', None],
  [ 0xf39c, 'UnicodeCharacter0xf39c', [], 'Display', None],
  [ 0xf39d, 'UnicodeCharacter0xf39d', [], 'Display', None],
  [ 0xf39e, 'UnicodeCharacter0xf39e', [], 'Display', None],
  [ 0xf39f, 'UnicodeCharacter0xf39f', [], 'Display', None],
  [ 0xf3a0, 'UnicodeCharacter0xf3a0', [], 'Display', None],
  [ 0xf3a1, 'UnicodeCharacter0xf3a1', [], 'Display', None],
  [ 0xf3a2, 'UnicodeCharacter0xf3a2', [], 'Display', None],
  [ 0xf3a3, 'UnicodeCharacter0xf3a3', [], 'Display', None],
  [ 0xf3a4, 'UnicodeCharacter0xf3a4', [], 'Display', None],
  [ 0xf3a5, 'UnicodeCharacter0xf3a5', [], 'Display', None],
  [ 0xf3a6, 'UnicodeCharacter0xf3a6', [], 'Display', None],
  [ 0xf3a7, 'UnicodeCharacter0xf3a7', [], 'Display', None],
  [ 0xf3a8, 'UnicodeCharacter0xf3a8', [], 'Display', None],
  [ 0xf3a9, 'UnicodeCharacter0xf3a9', [], 'Display', None],
  [ 0xf3aa, 'UnicodeCharacter0xf3aa', [], 'Display', None],
  [ 0xf3ab, 'UnicodeCharacter0xf3ab', [], 'Display', None],
  [ 0xf3ac, 'UnicodeCharacter0xf3ac', [], 'Display', None],
  [ 0xf3ad, 'UnicodeCharacter0xf3ad', [], 'Display', None],
  [ 0xf3ae, 'UnicodeCharacter0xf3ae', [], 'Display', None],
  [ 0xf3af, 'UnicodeCharacter0xf3af', [], 'Display', None],
  [ 0xf3b0, 'UnicodeCharacter0xf3b0', [], 'Display', None],
  [ 0xf3b1, 'UnicodeCharacter0xf3b1', [], 'Display', None],
  [ 0xf3b2, 'UnicodeCharacter0xf3b2', [], 'Display', None],
  [ 0xf3b3, 'UnicodeCharacter0xf3b3', [], 'Display', None],
  [ 0xf3b4, 'UnicodeCharacter0xf3b4', [], 'Display', None],
  [ 0xf3b5, 'UnicodeCharacter0xf3b5', [], 'Display', None],
  [ 0xf3b6, 'UnicodeCharacter0xf3b6', [], 'Display', None],
  [ 0xf3b7, 'UnicodeCharacter0xf3b7', [], 'Display', None],
  [ 0xf3b8, 'UnicodeCharacter0xf3b8', [], 'Display', None],
  [ 0xf3b9, 'UnicodeCharacter0xf3b9', [], 'Display', None],
  [ 0xf3ba, 'UnicodeCharacter0xf3ba', [], 'Display', None],
  [ 0xf3bb, 'UnicodeCharacter0xf3bb', [], 'Display', None],
  [ 0xf3bc, 'UnicodeCharacter0xf3bc', [], 'Display', None],
  [ 0xf3bd, 'UnicodeCharacter0xf3bd', [], 'Display', None],
  [ 0xf3be, 'UnicodeCharacter0xf3be', [], 'Display', None],
  [ 0xf3bf, 'UnicodeCharacter0xf3bf', [], 'Display', None],
  [ 0xf3c0, 'UnicodeCharacter0xf3c0', [], 'Display', None],
  [ 0xf3c1, 'UnicodeCharacter0xf3c1', [], 'Display', None],
  [ 0xf3c2, 'UnicodeCharacter0xf3c2', [], 'Display', None],
  [ 0xf3c3, 'UnicodeCharacter0xf3c3', [], 'Display', None],
  [ 0xf3c4, 'UnicodeCharacter0xf3c4', [], 'Display', None],
  [ 0xf3c5, 'UnicodeCharacter0xf3c5', [], 'Display', None],
  [ 0xf3c6, 'UnicodeCharacter0xf3c6', [], 'Display', None],
  [ 0xf3c7, 'UnicodeCharacter0xf3c7', [], 'Display', None],
  [ 0xf3c8, 'UnicodeCharacter0xf3c8', [], 'Display', None],
  [ 0xf3c9, 'UnicodeCharacter0xf3c9', [], 'Display', None],
  [ 0xf3ca, 'UnicodeCharacter0xf3ca', [], 'Display', None],
  [ 0xf3cb, 'UnicodeCharacter0xf3cb', [], 'Display', None],
  [ 0xf3cc, 'UnicodeCharacter0xf3cc', [], 'Display', None],
  [ 0xf3cd, 'UnicodeCharacter0xf3cd', [], 'Display', None],
  [ 0xf3ce, 'UnicodeCharacter0xf3ce', [], 'Display', None],
  [ 0xf3cf, 'UnicodeCharacter0xf3cf', [], 'Display', None],
  [ 0xf3d0, 'UnicodeCharacter0xf3d0', [], 'Display', None],
  [ 0xf3d1, 'UnicodeCharacter0xf3d1', [], 'Display', None],
  [ 0xf3d2, 'UnicodeCharacter0xf3d2', [], 'Display', None],
  [ 0xf3d3, 'UnicodeCharacter0xf3d3', [], 'Display', None],
  [ 0xf3d4, 'UnicodeCharacter0xf3d4', [], 'Display', None],
  [ 0xf3d5, 'UnicodeCharacter0xf3d5', [], 'Display', None],
  [ 0xf3d6, 'UnicodeCharacter0xf3d6', [], 'Display', None],
  [ 0xf3d7, 'UnicodeCharacter0xf3d7', [], 'Display', None],
  [ 0xf3d8, 'UnicodeCharacter0xf3d8', [], 'Display', None],
  [ 0xf3d9, 'UnicodeCharacter0xf3d9', [], 'Display', None],
  [ 0xf3da, 'UnicodeCharacter0xf3da', [], 'Display', None],
  [ 0xf3db, 'UnicodeCharacter0xf3db', [], 'Display', None],
  [ 0xf3dc, 'UnicodeCharacter0xf3dc', [], 'Display', None],
  [ 0xf3dd, 'UnicodeCharacter0xf3dd', [], 'Display', None],
  [ 0xf3de, 'UnicodeCharacter0xf3de', [], 'Display', None],
  [ 0xf3df, 'UnicodeCharacter0xf3df', [], 'Display', None],
  [ 0xf3e0, 'UnicodeCharacter0xf3e0', [], 'Display', None],
  [ 0xf3e1, 'UnicodeCharacter0xf3e1', [], 'Display', None],
  [ 0xf3e2, 'UnicodeCharacter0xf3e2', [], 'Display', None],
  [ 0xf3e3, 'UnicodeCharacter0xf3e3', [], 'Display', None],
  [ 0xf3e4, 'UnicodeCharacter0xf3e4', [], 'Display', None],
  [ 0xf3e5, 'UnicodeCharacter0xf3e5', [], 'Display', None],
  [ 0xf3e6, 'UnicodeCharacter0xf3e6', [], 'Display', None],
  [ 0xf3e7, 'UnicodeCharacter0xf3e7', [], 'Display', None],
  [ 0xf3e8, 'UnicodeCharacter0xf3e8', [], 'Display', None],
  [ 0xf3e9, 'UnicodeCharacter0xf3e9', [], 'Display', None],
  [ 0xf3ea, 'UnicodeCharacter0xf3ea', [], 'Display', None],
  [ 0xf3eb, 'UnicodeCharacter0xf3eb', [], 'Display', None],
  [ 0xf3ec, 'UnicodeCharacter0xf3ec', [], 'Display', None],
  [ 0xf3ed, 'UnicodeCharacter0xf3ed', [], 'Display', None],
  [ 0xf3ee, 'UnicodeCharacter0xf3ee', [], 'Display', None],
  [ 0xf3ef, 'UnicodeCharacter0xf3ef', [], 'Display', None],
  [ 0xf3f0, 'UnicodeCharacter0xf3f0', [], 'Display', None],
  [ 0xf3f1, 'UnicodeCharacter0xf3f1', [], 'Display', None],
  [ 0xf3f2, 'UnicodeCharacter0xf3f2', [], 'Display', None],
  [ 0xf3f3, 'UnicodeCharacter0xf3f3', [], 'Display', None],
  [ 0xf3f4, 'UnicodeCharacter0xf3f4', [], 'Display', None],
  [ 0xf3f5, 'UnicodeCharacter0xf3f5', [], 'Display', None],
  [ 0xf3f6, 'UnicodeCharacter0xf3f6', [], 'Display', None],
  [ 0xf3f7, 'UnicodeCharacter0xf3f7', [], 'Display', None],
  [ 0xf3f8, 'UnicodeCharacter0xf3f8', [], 'Display', None],
  [ 0xf3f9, 'UnicodeCharacter0xf3f9', [], 'Display', None],
  [ 0xf3fa, 'UnicodeCharacter0xf3fa', [], 'Display', None],
  [ 0xf3fb, 'UnicodeCharacter0xf3fb', [], 'Display', None],
  [ 0xf3fc, 'UnicodeCharacter0xf3fc', [], 'Display', None],
  [ 0xf3fd, 'UnicodeCharacter0xf3fd', [], 'Display', None],
  [ 0xf3fe, 'UnicodeCharacter0xf3fe', [], 'Display', None],
  [ 0xf3ff, 'UnicodeCharacter0xf3ff', [], 'Display', None],
  [ 0xf400, 'UnicodeCharacter0xf400', [], 'Display', None],
  [ 0xf401, 'UnicodeCharacter0xf401', [], 'Display', None],
  [ 0xf402, 'UnicodeCharacter0xf402', [], 'Display', None],
  [ 0xf403, 'UnicodeCharacter0xf403', [], 'Display', None],
  [ 0xf404, 'UnicodeCharacter0xf404', [], 'Display', None],
  [ 0xf405, 'UnicodeCharacter0xf405', [], 'Display', None],
  [ 0xf406, 'UnicodeCharacter0xf406', [], 'Display', None],
  [ 0xf407, 'UnicodeCharacter0xf407', [], 'Display', None],
  [ 0xf408, 'UnicodeCharacter0xf408', [], 'Display', None],
  [ 0xf409, 'UnicodeCharacter0xf409', [], 'Display', None],
  [ 0xf40a, 'UnicodeCharacter0xf40a', [], 'Display', None],
  [ 0xf40b, 'UnicodeCharacter0xf40b', [], 'Display', None],
  [ 0xf40c, 'UnicodeCharacter0xf40c', [], 'Display', None],
  [ 0xf40d, 'UnicodeCharacter0xf40d', [], 'Display', None],
  [ 0xf40e, 'UnicodeCharacter0xf40e', [], 'Display', None],
  [ 0xf40f, 'UnicodeCharacter0xf40f', [], 'Display', None],
  [ 0xf410, 'UnicodeCharacter0xf410', [], 'Display', None],
  [ 0xf411, 'UnicodeCharacter0xf411', [], 'Display', None],
  [ 0xf412, 'UnicodeCharacter0xf412', [], 'Display', None],
  [ 0xf413, 'UnicodeCharacter0xf413', [], 'Display', None],
  [ 0xf414, 'UnicodeCharacter0xf414', [], 'Display', None],
  [ 0xf415, 'UnicodeCharacter0xf415', [], 'Display', None],
  [ 0xf416, 'UnicodeCharacter0xf416', [], 'Display', None],
  [ 0xf417, 'UnicodeCharacter0xf417', [], 'Display', None],
  [ 0xf418, 'UnicodeCharacter0xf418', [], 'Display', None],
  [ 0xf419, 'UnicodeCharacter0xf419', [], 'Display', None],
  [ 0xf41a, 'UnicodeCharacter0xf41a', [], 'Display', None],
  [ 0xf41b, 'UnicodeCharacter0xf41b', [], 'Display', None],
  [ 0xf41c, 'UnicodeCharacter0xf41c', [], 'Display', None],
  [ 0xf41d, 'UnicodeCharacter0xf41d', [], 'Display', None],
  [ 0xf41e, 'UnicodeCharacter0xf41e', [], 'Display', None],
  [ 0xf41f, 'UnicodeCharacter0xf41f', [], 'Display', None],
  [ 0xf420, 'UnicodeCharacter0xf420', [], 'Display', None],
  [ 0xf421, 'UnicodeCharacter0xf421', [], 'Display', None],
  [ 0xf422, 'UnicodeCharacter0xf422', [], 'Display', None],
  [ 0xf423, 'UnicodeCharacter0xf423', [], 'Display', None],
  [ 0xf424, 'UnicodeCharacter0xf424', [], 'Display', None],
  [ 0xf425, 'UnicodeCharacter0xf425', [], 'Display', None],
  [ 0xf426, 'UnicodeCharacter0xf426', [], 'Display', None],
  [ 0xf427, 'UnicodeCharacter0xf427', [], 'Display', None],
  [ 0xf428, 'UnicodeCharacter0xf428', [], 'Display', None],
  [ 0xf429, 'UnicodeCharacter0xf429', [], 'Display', None],
  [ 0xf42a, 'UnicodeCharacter0xf42a', [], 'Display', None],
  [ 0xf42b, 'UnicodeCharacter0xf42b', [], 'Display', None],
  [ 0xf42c, 'UnicodeCharacter0xf42c', [], 'Display', None],
  [ 0xf42d, 'UnicodeCharacter0xf42d', [], 'Display', None],
  [ 0xf42e, 'UnicodeCharacter0xf42e', [], 'Display', None],
  [ 0xf42f, 'UnicodeCharacter0xf42f', [], 'Display', None],
  [ 0xf430, 'UnicodeCharacter0xf430', [], 'Display', None],
  [ 0xf431, 'UnicodeCharacter0xf431', [], 'Display', None],
  [ 0xf432, 'UnicodeCharacter0xf432', [], 'Display', None],
  [ 0xf433, 'UnicodeCharacter0xf433', [], 'Display', None],
  [ 0xf434, 'UnicodeCharacter0xf434', [], 'Display', None],
  [ 0xf435, 'UnicodeCharacter0xf435', [], 'Display', None],
  [ 0xf436, 'UnicodeCharacter0xf436', [], 'Display', None],
  [ 0xf437, 'UnicodeCharacter0xf437', [], 'Display', None],
  [ 0xf438, 'UnicodeCharacter0xf438', [], 'Display', None],
  [ 0xf439, 'UnicodeCharacter0xf439', [], 'Display', None],
  [ 0xf43a, 'UnicodeCharacter0xf43a', [], 'Display', None],
  [ 0xf43b, 'UnicodeCharacter0xf43b', [], 'Display', None],
  [ 0xf43c, 'UnicodeCharacter0xf43c', [], 'Display', None],
  [ 0xf43d, 'UnicodeCharacter0xf43d', [], 'Display', None],
  [ 0xf43e, 'UnicodeCharacter0xf43e', [], 'Display', None],
  [ 0xf43f, 'UnicodeCharacter0xf43f', [], 'Display', None],
  [ 0xf440, 'UnicodeCharacter0xf440', [], 'Display', None],
  [ 0xf441, 'UnicodeCharacter0xf441', [], 'Display', None],
  [ 0xf442, 'UnicodeCharacter0xf442', [], 'Display', None],
  [ 0xf443, 'UnicodeCharacter0xf443', [], 'Display', None],
  [ 0xf444, 'UnicodeCharacter0xf444', [], 'Display', None],
  [ 0xf445, 'UnicodeCharacter0xf445', [], 'Display', None],
  [ 0xf446, 'UnicodeCharacter0xf446', [], 'Display', None],
  [ 0xf447, 'UnicodeCharacter0xf447', [], 'Display', None],
  [ 0xf448, 'UnicodeCharacter0xf448', [], 'Display', None],
  [ 0xf449, 'UnicodeCharacter0xf449', [], 'Display', None],
  [ 0xf44a, 'UnicodeCharacter0xf44a', [], 'Display', None],
  [ 0xf44b, 'UnicodeCharacter0xf44b', [], 'Display', None],
  [ 0xf44c, 'UnicodeCharacter0xf44c', [], 'Display', None],
  [ 0xf44d, 'UnicodeCharacter0xf44d', [], 'Display', None],
  [ 0xf44e, 'UnicodeCharacter0xf44e', [], 'Display', None],
  [ 0xf44f, 'UnicodeCharacter0xf44f', [], 'Display', None],
  [ 0xf450, 'UnicodeCharacter0xf450', [], 'Display', None],
  [ 0xf451, 'UnicodeCharacter0xf451', [], 'Display', None],
  [ 0xf452, 'UnicodeCharacter0xf452', [], 'Display', None],
  [ 0xf453, 'UnicodeCharacter0xf453', [], 'Display', None],
  [ 0xf454, 'UnicodeCharacter0xf454', [], 'Display', None],
  [ 0xf455, 'UnicodeCharacter0xf455', [], 'Display', None],
  [ 0xf456, 'UnicodeCharacter0xf456', [], 'Display', None],
  [ 0xf457, 'UnicodeCharacter0xf457', [], 'Display', None],
  [ 0xf458, 'UnicodeCharacter0xf458', [], 'Display', None],
  [ 0xf459, 'UnicodeCharacter0xf459', [], 'Display', None],
  [ 0xf45a, 'UnicodeCharacter0xf45a', [], 'Display', None],
  [ 0xf45b, 'UnicodeCharacter0xf45b', [], 'Display', None],
  [ 0xf45c, 'UnicodeCharacter0xf45c', [], 'Display', None],
  [ 0xf45d, 'UnicodeCharacter0xf45d', [], 'Display', None],
  [ 0xf45e, 'UnicodeCharacter0xf45e', [], 'Display', None],
  [ 0xf45f, 'UnicodeCharacter0xf45f', [], 'Display', None],
  [ 0xf460, 'UnicodeCharacter0xf460', [], 'Display', None],
  [ 0xf461, 'UnicodeCharacter0xf461', [], 'Display', None],
  [ 0xf462, 'UnicodeCharacter0xf462', [], 'Display', None],
  [ 0xf463, 'UnicodeCharacter0xf463', [], 'Display', None],
  [ 0xf464, 'UnicodeCharacter0xf464', [], 'Display', None],
  [ 0xf465, 'UnicodeCharacter0xf465', [], 'Display', None],
  [ 0xf466, 'UnicodeCharacter0xf466', [], 'Display', None],
  [ 0xf467, 'UnicodeCharacter0xf467', [], 'Display', None],
  [ 0xf468, 'UnicodeCharacter0xf468', [], 'Display', None],
  [ 0xf469, 'UnicodeCharacter0xf469', [], 'Display', None],
  [ 0xf46a, 'UnicodeCharacter0xf46a', [], 'Display', None],
  [ 0xf46b, 'UnicodeCharacter0xf46b', [], 'Display', None],
  [ 0xf46c, 'UnicodeCharacter0xf46c', [], 'Display', None],
  [ 0xf46d, 'UnicodeCharacter0xf46d', [], 'Display', None],
  [ 0xf46e, 'UnicodeCharacter0xf46e', [], 'Display', None],
  [ 0xf46f, 'UnicodeCharacter0xf46f', [], 'Display', None],
  [ 0xf470, 'UnicodeCharacter0xf470', [], 'Display', None],
  [ 0xf471, 'UnicodeCharacter0xf471', [], 'Display', None],
  [ 0xf472, 'UnicodeCharacter0xf472', [], 'Display', None],
  [ 0xf473, 'UnicodeCharacter0xf473', [], 'Display', None],
  [ 0xf474, 'UnicodeCharacter0xf474', [], 'Display', None],
  [ 0xf475, 'UnicodeCharacter0xf475', [], 'Display', None],
  [ 0xf476, 'UnicodeCharacter0xf476', [], 'Display', None],
  [ 0xf477, 'UnicodeCharacter0xf477', [], 'Display', None],
  [ 0xf478, 'UnicodeCharacter0xf478', [], 'Display', None],
  [ 0xf479, 'UnicodeCharacter0xf479', [], 'Display', None],
  [ 0xf47a, 'UnicodeCharacter0xf47a', [], 'Display', None],
  [ 0xf47b, 'UnicodeCharacter0xf47b', [], 'Display', None],
  [ 0xf47c, 'UnicodeCharacter0xf47c', [], 'Display', None],
  [ 0xf47d, 'UnicodeCharacter0xf47d', [], 'Display', None],
  [ 0xf47e, 'UnicodeCharacter0xf47e', [], 'Display', None],
  [ 0xf47f, 'UnicodeCharacter0xf47f', [], 'Display', None],
  [ 0xf480, 'UnicodeCharacter0xf480', [], 'Display', None],
  [ 0xf481, 'UnicodeCharacter0xf481', [], 'Display', None],
  [ 0xf482, 'UnicodeCharacter0xf482', [], 'Display', None],
  [ 0xf483, 'UnicodeCharacter0xf483', [], 'Display', None],
  [ 0xf484, 'UnicodeCharacter0xf484', [], 'Display', None],
  [ 0xf485, 'UnicodeCharacter0xf485', [], 'Display', None],
  [ 0xf486, 'UnicodeCharacter0xf486', [], 'Display', None],
  [ 0xf487, 'UnicodeCharacter0xf487', [], 'Display', None],
  [ 0xf488, 'UnicodeCharacter0xf488', [], 'Display', None],
  [ 0xf489, 'UnicodeCharacter0xf489', [], 'Display', None],
  [ 0xf48a, 'UnicodeCharacter0xf48a', [], 'Display', None],
  [ 0xf48b, 'UnicodeCharacter0xf48b', [], 'Display', None],
  [ 0xf48c, 'UnicodeCharacter0xf48c', [], 'Display', None],
  [ 0xf48d, 'UnicodeCharacter0xf48d', [], 'Display', None],
  [ 0xf48e, 'UnicodeCharacter0xf48e', [], 'Display', None],
  [ 0xf48f, 'UnicodeCharacter0xf48f', [], 'Display', None],
  [ 0xf490, 'UnicodeCharacter0xf490', [], 'Display', None],
  [ 0xf491, 'UnicodeCharacter0xf491', [], 'Display', None],
  [ 0xf492, 'UnicodeCharacter0xf492', [], 'Display', None],
  [ 0xf493, 'UnicodeCharacter0xf493', [], 'Display', None],
  [ 0xf494, 'UnicodeCharacter0xf494', [], 'Display', None],
  [ 0xf495, 'UnicodeCharacter0xf495', [], 'Display', None],
  [ 0xf496, 'UnicodeCharacter0xf496', [], 'Display', None],
  [ 0xf497, 'UnicodeCharacter0xf497', [], 'Display', None],
  [ 0xf498, 'UnicodeCharacter0xf498', [], 'Display', None],
  [ 0xf499, 'UnicodeCharacter0xf499', [], 'Display', None],
  [ 0xf49a, 'UnicodeCharacter0xf49a', [], 'Display', None],
  [ 0xf49b, 'UnicodeCharacter0xf49b', [], 'Display', None],
  [ 0xf49c, 'UnicodeCharacter0xf49c', [], 'Display', None],
  [ 0xf49d, 'UnicodeCharacter0xf49d', [], 'Display', None],
  [ 0xf49e, 'UnicodeCharacter0xf49e', [], 'Display', None],
  [ 0xf49f, 'UnicodeCharacter0xf49f', [], 'Display', None],
  [ 0xf4a0, 'UnicodeCharacter0xf4a0', [], 'Display', None],
  [ 0xf4a1, 'UnicodeCharacter0xf4a1', [], 'Display', None],
  [ 0xf4a2, 'UnicodeCharacter0xf4a2', [], 'Display', None],
  [ 0xf4a3, 'UnicodeCharacter0xf4a3', [], 'Display', None],
  [ 0xf4a4, 'UnicodeCharacter0xf4a4', [], 'Display', None],
  [ 0xf4a5, 'UnicodeCharacter0xf4a5', [], 'Display', None],
  [ 0xf4a6, 'UnicodeCharacter0xf4a6', [], 'Display', None],
  [ 0xf4a7, 'UnicodeCharacter0xf4a7', [], 'Display', None],
  [ 0xf4a8, 'UnicodeCharacter0xf4a8', [], 'Display', None],
  [ 0xf4a9, 'UnicodeCharacter0xf4a9', [], 'Display', None],
  [ 0xf4aa, 'UnicodeCharacter0xf4aa', [], 'Display', None],
  [ 0xf4ab, 'UnicodeCharacter0xf4ab', [], 'Display', None],
  [ 0xf4ac, 'UnicodeCharacter0xf4ac', [], 'Display', None],
  [ 0xf4ad, 'UnicodeCharacter0xf4ad', [], 'Display', None],
  [ 0xf4ae, 'UnicodeCharacter0xf4ae', [], 'Display', None],
  [ 0xf4af, 'UnicodeCharacter0xf4af', [], 'Display', None],
  [ 0xf4b0, 'UnicodeCharacter0xf4b0', [], 'Display', None],
  [ 0xf4b1, 'UnicodeCharacter0xf4b1', [], 'Display', None],
  [ 0xf4b2, 'UnicodeCharacter0xf4b2', [], 'Display', None],
  [ 0xf4b3, 'UnicodeCharacter0xf4b3', [], 'Display', None],
  [ 0xf4b4, 'UnicodeCharacter0xf4b4', [], 'Display', None],
  [ 0xf4b5, 'UnicodeCharacter0xf4b5', [], 'Display', None],
  [ 0xf4b6, 'UnicodeCharacter0xf4b6', [], 'Display', None],
  [ 0xf4b7, 'UnicodeCharacter0xf4b7', [], 'Display', None],
  [ 0xf4b8, 'UnicodeCharacter0xf4b8', [], 'Display', None],
  [ 0xf4b9, 'UnicodeCharacter0xf4b9', [], 'Display', None],
  [ 0xf4ba, 'UnicodeCharacter0xf4ba', [], 'Display', None],
  [ 0xf4bb, 'UnicodeCharacter0xf4bb', [], 'Display', None],
  [ 0xf4bc, 'UnicodeCharacter0xf4bc', [], 'Display', None],
  [ 0xf4bd, 'UnicodeCharacter0xf4bd', [], 'Display', None],
  [ 0xf4be, 'UnicodeCharacter0xf4be', [], 'Display', None],
  [ 0xf4bf, 'UnicodeCharacter0xf4bf', [], 'Display', None],
  [ 0xf4c0, 'UnicodeCharacter0xf4c0', [], 'Display', None],
  [ 0xf4c1, 'UnicodeCharacter0xf4c1', [], 'Display', None],
  [ 0xf4c2, 'UnicodeCharacter0xf4c2', [], 'Display', None],
  [ 0xf4c3, 'UnicodeCharacter0xf4c3', [], 'Display', None],
  [ 0xf4c4, 'UnicodeCharacter0xf4c4', [], 'Display', None],
  [ 0xf4c5, 'UnicodeCharacter0xf4c5', [], 'Display', None],
  [ 0xf4c6, 'UnicodeCharacter0xf4c6', [], 'Display', None],
  [ 0xf4c7, 'UnicodeCharacter0xf4c7', [], 'Display', None],
  [ 0xf4c8, 'UnicodeCharacter0xf4c8', [], 'Display', None],
  [ 0xf4c9, 'UnicodeCharacter0xf4c9', [], 'Display', None],
  [ 0xf4ca, 'UnicodeCharacter0xf4ca', [], 'Display', None],
  [ 0xf4cb, 'UnicodeCharacter0xf4cb', [], 'Display', None],
  [ 0xf4cc, 'UnicodeCharacter0xf4cc', [], 'Display', None],
  [ 0xf4cd, 'UnicodeCharacter0xf4cd', [], 'Display', None],
  [ 0xf4ce, 'UnicodeCharacter0xf4ce', [], 'Display', None],
  [ 0xf4cf, 'UnicodeCharacter0xf4cf', [], 'Display', None],
  [ 0xf4d0, 'UnicodeCharacter0xf4d0', [], 'Display', None],
  [ 0xf4d1, 'UnicodeCharacter0xf4d1', [], 'Display', None],
  [ 0xf4d2, 'UnicodeCharacter0xf4d2', [], 'Display', None],
  [ 0xf4d3, 'UnicodeCharacter0xf4d3', [], 'Display', None],
  [ 0xf4d4, 'UnicodeCharacter0xf4d4', [], 'Display', None],
  [ 0xf4d5, 'UnicodeCharacter0xf4d5', [], 'Display', None],
  [ 0xf4d6, 'UnicodeCharacter0xf4d6', [], 'Display', None],
  [ 0xf4d7, 'UnicodeCharacter0xf4d7', [], 'Display', None],
  [ 0xf4d8, 'UnicodeCharacter0xf4d8', [], 'Display', None],
  [ 0xf4d9, 'UnicodeCharacter0xf4d9', [], 'Display', None],
  [ 0xf4da, 'UnicodeCharacter0xf4da', [], 'Display', None],
  [ 0xf4db, 'UnicodeCharacter0xf4db', [], 'Display', None],
  [ 0xf4dc, 'UnicodeCharacter0xf4dc', [], 'Display', None],
  [ 0xf4dd, 'UnicodeCharacter0xf4dd', [], 'Display', None],
  [ 0xf4de, 'UnicodeCharacter0xf4de', [], 'Display', None],
  [ 0xf4df, 'UnicodeCharacter0xf4df', [], 'Display', None],
  [ 0xf4e0, 'UnicodeCharacter0xf4e0', [], 'Display', None],
  [ 0xf4e1, 'UnicodeCharacter0xf4e1', [], 'Display', None],
  [ 0xf4e2, 'UnicodeCharacter0xf4e2', [], 'Display', None],
  [ 0xf4e3, 'UnicodeCharacter0xf4e3', [], 'Display', None],
  [ 0xf4e4, 'UnicodeCharacter0xf4e4', [], 'Display', None],
  [ 0xf4e5, 'UnicodeCharacter0xf4e5', [], 'Display', None],
  [ 0xf4e6, 'UnicodeCharacter0xf4e6', [], 'Display', None],
  [ 0xf4e7, 'UnicodeCharacter0xf4e7', [], 'Display', None],
  [ 0xf4e8, 'UnicodeCharacter0xf4e8', [], 'Display', None],
  [ 0xf4e9, 'UnicodeCharacter0xf4e9', [], 'Display', None],
  [ 0xf4ea, 'UnicodeCharacter0xf4ea', [], 'Display', None],
  [ 0xf4eb, 'UnicodeCharacter0xf4eb', [], 'Display', None],
  [ 0xf4ec, 'UnicodeCharacter0xf4ec', [], 'Display', None],
  [ 0xf4ed, 'UnicodeCharacter0xf4ed', [], 'Display', None],
  [ 0xf4ee, 'UnicodeCharacter0xf4ee', [], 'Display', None],
  [ 0xf4ef, 'UnicodeCharacter0xf4ef', [], 'Display', None],
  [ 0xf4f0, 'UnicodeCharacter0xf4f0', [], 'Display', None],
  [ 0xf4f1, 'UnicodeCharacter0xf4f1', [], 'Display', None],
  [ 0xf4f2, 'UnicodeCharacter0xf4f2', [], 'Display', None],
  [ 0xf4f3, 'UnicodeCharacter0xf4f3', [], 'Display', None],
  [ 0xf4f4, 'UnicodeCharacter0xf4f4', [], 'Display', None],
  [ 0xf4f5, 'UnicodeCharacter0xf4f5', [], 'Display', None],
  [ 0xf4f6, 'UnicodeCharacter0xf4f6', [], 'Display', None],
  [ 0xf4f7, 'UnicodeCharacter0xf4f7', [], 'Display', None],
  [ 0xf4f8, 'UnicodeCharacter0xf4f8', [], 'Display', None],
  [ 0xf4f9, 'UnicodeCharacter0xf4f9', [], 'Display', None],
  [ 0xf4fa, 'UnicodeCharacter0xf4fa', [], 'Display', None],
  [ 0xf4fb, 'UnicodeCharacter0xf4fb', [], 'Display', None],
  [ 0xf4fc, 'UnicodeCharacter0xf4fc', [], 'Display', None],
  [ 0xf4fd, 'UnicodeCharacter0xf4fd', [], 'Display', None],
  [ 0xf4fe, 'UnicodeCharacter0xf4fe', [], 'Display', None],
  [ 0xf4ff, 'UnicodeCharacter0xf4ff', [], 'Display', None],
  [ 0xf500, 'UnicodeCharacter0xf500', [], 'Display', None],
  [ 0xf501, 'UnicodeCharacter0xf501', [], 'Display', None],
  [ 0xf502, 'UnicodeCharacter0xf502', [], 'Display', None],
  [ 0xf503, 'UnicodeCharacter0xf503', [], 'Display', None],
  [ 0xf504, 'UnicodeCharacter0xf504', [], 'Display', None],
  [ 0xf505, 'UnicodeCharacter0xf505', [], 'Display', None],
  [ 0xf506, 'UnicodeCharacter0xf506', [], 'Display', None],
  [ 0xf507, 'UnicodeCharacter0xf507', [], 'Display', None],
  [ 0xf508, 'UnicodeCharacter0xf508', [], 'Display', None],
  [ 0xf509, 'UnicodeCharacter0xf509', [], 'Display', None],
  [ 0xf50a, 'UnicodeCharacter0xf50a', [], 'Display', None],
  [ 0xf50b, 'UnicodeCharacter0xf50b', [], 'Display', None],
  [ 0xf50c, 'UnicodeCharacter0xf50c', [], 'Display', None],
  [ 0xf50d, 'UnicodeCharacter0xf50d', [], 'Display', None],
  [ 0xf50e, 'UnicodeCharacter0xf50e', [], 'Display', None],
  [ 0xf50f, 'UnicodeCharacter0xf50f', [], 'Display', None],
  [ 0xf510, 'UnicodeCharacter0xf510', [], 'Display', None],
  [ 0xf511, 'UnicodeCharacter0xf511', [], 'Display', None],
  [ 0xf512, 'UnicodeCharacter0xf512', [], 'Display', None],
  [ 0xf513, 'UnicodeCharacter0xf513', [], 'Display', None],
  [ 0xf514, 'UnicodeCharacter0xf514', [], 'Display', None],
  [ 0xf515, 'UnicodeCharacter0xf515', [], 'Display', None],
  [ 0xf516, 'UnicodeCharacter0xf516', [], 'Display', None],
  [ 0xf517, 'UnicodeCharacter0xf517', [], 'Display', None],
  [ 0xf518, 'UnicodeCharacter0xf518', [], 'Display', None],
  [ 0xf519, 'UnicodeCharacter0xf519', [], 'Display', None],
  [ 0xf51a, 'UnicodeCharacter0xf51a', [], 'Display', None],
  [ 0xf51b, 'UnicodeCharacter0xf51b', [], 'Display', None],
  [ 0xf51c, 'UnicodeCharacter0xf51c', [], 'Display', None],
  [ 0xf51d, 'UnicodeCharacter0xf51d', [], 'Display', None],
  [ 0xf51e, 'UnicodeCharacter0xf51e', [], 'Display', None],
  [ 0xf51f, 'UnicodeCharacter0xf51f', [], 'Display', None],
  [ 0xf520, 'UnicodeCharacter0xf520', [], 'Display', None],
  [ 0xf521, 'UnicodeCharacter0xf521', [], 'Display', None],
  [ 0xf522, 'UnicodeCharacter0xf522', [], 'Display', None],
  [ 0xf523, 'UnicodeCharacter0xf523', [], 'Display', None],
  [ 0xf524, 'UnicodeCharacter0xf524', [], 'Display', None],
  [ 0xf525, 'UnicodeCharacter0xf525', [], 'Display', None],
  [ 0xf526, 'UnicodeCharacter0xf526', [], 'Display', None],
  [ 0xf527, 'UnicodeCharacter0xf527', [], 'Display', None],
  [ 0xf528, 'UnicodeCharacter0xf528', [], 'Display', None],
  [ 0xf529, 'UnicodeCharacter0xf529', [], 'Display', None],
  [ 0xf52a, 'UnicodeCharacter0xf52a', [], 'Display', None],
  [ 0xf52b, 'UnicodeCharacter0xf52b', [], 'Display', None],
  [ 0xf52c, 'UnicodeCharacter0xf52c', [], 'Display', None],
  [ 0xf52d, 'UnicodeCharacter0xf52d', [], 'Display', None],
  [ 0xf52e, 'UnicodeCharacter0xf52e', [], 'Display', None],
  [ 0xf52f, 'UnicodeCharacter0xf52f', [], 'Display', None],
  [ 0xf530, 'UnicodeCharacter0xf530', [], 'Display', None],
  [ 0xf531, 'UnicodeCharacter0xf531', [], 'Display', None],
  [ 0xf532, 'UnicodeCharacter0xf532', [], 'Display', None],
  [ 0xf533, 'UnicodeCharacter0xf533', [], 'Display', None],
  [ 0xf534, 'UnicodeCharacter0xf534', [], 'Display', None],
  [ 0xf535, 'UnicodeCharacter0xf535', [], 'Display', None],
  [ 0xf536, 'UnicodeCharacter0xf536', [], 'Display', None],
  [ 0xf537, 'UnicodeCharacter0xf537', [], 'Display', None],
  [ 0xf538, 'UnicodeCharacter0xf538', [], 'Display', None],
  [ 0xf539, 'UnicodeCharacter0xf539', [], 'Display', None],
  [ 0xf53a, 'UnicodeCharacter0xf53a', [], 'Display', None],
  [ 0xf53b, 'UnicodeCharacter0xf53b', [], 'Display', None],
  [ 0xf53c, 'UnicodeCharacter0xf53c', [], 'Display', None],
  [ 0xf53d, 'UnicodeCharacter0xf53d', [], 'Display', None],
  [ 0xf53e, 'UnicodeCharacter0xf53e', [], 'Display', None],
  [ 0xf53f, 'UnicodeCharacter0xf53f', [], 'Display', None],
  [ 0xf540, 'UnicodeCharacter0xf540', [], 'Display', None],
  [ 0xf541, 'UnicodeCharacter0xf541', [], 'Display', None],
  [ 0xf542, 'UnicodeCharacter0xf542', [], 'Display', None],
  [ 0xf543, 'UnicodeCharacter0xf543', [], 'Display', None],
  [ 0xf544, 'UnicodeCharacter0xf544', [], 'Display', None],
  [ 0xf545, 'UnicodeCharacter0xf545', [], 'Display', None],
  [ 0xf546, 'UnicodeCharacter0xf546', [], 'Display', None],
  [ 0xf547, 'UnicodeCharacter0xf547', [], 'Display', None],
  [ 0xf548, 'UnicodeCharacter0xf548', [], 'Display', None],
  [ 0xf549, 'UnicodeCharacter0xf549', [], 'Display', None],
  [ 0xf54a, 'UnicodeCharacter0xf54a', [], 'Display', None],
  [ 0xf54b, 'UnicodeCharacter0xf54b', [], 'Display', None],
  [ 0xf54c, 'UnicodeCharacter0xf54c', [], 'Display', None],
  [ 0xf54d, 'UnicodeCharacter0xf54d', [], 'Display', None],
  [ 0xf54e, 'UnicodeCharacter0xf54e', [], 'Display', None],
  [ 0xf54f, 'UnicodeCharacter0xf54f', [], 'Display', None],
  [ 0xf550, 'UnicodeCharacter0xf550', [], 'Display', None],
  [ 0xf551, 'UnicodeCharacter0xf551', [], 'Display', None],
  [ 0xf552, 'UnicodeCharacter0xf552', [], 'Display', None],
  [ 0xf553, 'UnicodeCharacter0xf553', [], 'Display', None],
  [ 0xf554, 'UnicodeCharacter0xf554', [], 'Display', None],
  [ 0xf555, 'UnicodeCharacter0xf555', [], 'Display', None],
  [ 0xf556, 'UnicodeCharacter0xf556', [], 'Display', None],
  [ 0xf557, 'UnicodeCharacter0xf557', [], 'Display', None],
  [ 0xf558, 'UnicodeCharacter0xf558', [], 'Display', None],
  [ 0xf559, 'UnicodeCharacter0xf559', [], 'Display', None],
  [ 0xf55a, 'UnicodeCharacter0xf55a', [], 'Display', None],
  [ 0xf55b, 'UnicodeCharacter0xf55b', [], 'Display', None],
  [ 0xf55c, 'UnicodeCharacter0xf55c', [], 'Display', None],
  [ 0xf55d, 'UnicodeCharacter0xf55d', [], 'Display', None],
  [ 0xf55e, 'UnicodeCharacter0xf55e', [], 'Display', None],
  [ 0xf55f, 'UnicodeCharacter0xf55f', [], 'Display', None],
  [ 0xf560, 'UnicodeCharacter0xf560', [], 'Display', None],
  [ 0xf561, 'UnicodeCharacter0xf561', [], 'Display', None],
  [ 0xf562, 'UnicodeCharacter0xf562', [], 'Display', None],
  [ 0xf563, 'UnicodeCharacter0xf563', [], 'Display', None],
  [ 0xf564, 'UnicodeCharacter0xf564', [], 'Display', None],
  [ 0xf565, 'UnicodeCharacter0xf565', [], 'Display', None],
  [ 0xf566, 'UnicodeCharacter0xf566', [], 'Display', None],
  [ 0xf567, 'UnicodeCharacter0xf567', [], 'Display', None],
  [ 0xf568, 'UnicodeCharacter0xf568', [], 'Display', None],
  [ 0xf569, 'UnicodeCharacter0xf569', [], 'Display', None],
  [ 0xf56a, 'UnicodeCharacter0xf56a', [], 'Display', None],
  [ 0xf56b, 'UnicodeCharacter0xf56b', [], 'Display', None],
  [ 0xf56c, 'UnicodeCharacter0xf56c', [], 'Display', None],
  [ 0xf56d, 'UnicodeCharacter0xf56d', [], 'Display', None],
  [ 0xf56e, 'UnicodeCharacter0xf56e', [], 'Display', None],
  [ 0xf56f, 'UnicodeCharacter0xf56f', [], 'Display', None],
  [ 0xf570, 'UnicodeCharacter0xf570', [], 'Display', None],
  [ 0xf571, 'UnicodeCharacter0xf571', [], 'Display', None],
  [ 0xf572, 'UnicodeCharacter0xf572', [], 'Display', None],
  [ 0xf573, 'UnicodeCharacter0xf573', [], 'Display', None],
  [ 0xf574, 'UnicodeCharacter0xf574', [], 'Display', None],
  [ 0xf575, 'UnicodeCharacter0xf575', [], 'Display', None],
  [ 0xf576, 'UnicodeCharacter0xf576', [], 'Display', None],
  [ 0xf577, 'UnicodeCharacter0xf577', [], 'Display', None],
  [ 0xf578, 'UnicodeCharacter0xf578', [], 'Display', None],
  [ 0xf579, 'UnicodeCharacter0xf579', [], 'Display', None],
  [ 0xf57a, 'UnicodeCharacter0xf57a', [], 'Display', None],
  [ 0xf57b, 'UnicodeCharacter0xf57b', [], 'Display', None],
  [ 0xf57c, 'UnicodeCharacter0xf57c', [], 'Display', None],
  [ 0xf57d, 'UnicodeCharacter0xf57d', [], 'Display', None],
  [ 0xf57e, 'UnicodeCharacter0xf57e', [], 'Display', None],
  [ 0xf57f, 'UnicodeCharacter0xf57f', [], 'Display', None],
  [ 0xf580, 'UnicodeCharacter0xf580', [], 'Display', None],
  [ 0xf581, 'UnicodeCharacter0xf581', [], 'Display', None],
  [ 0xf582, 'UnicodeCharacter0xf582', [], 'Display', None],
  [ 0xf583, 'UnicodeCharacter0xf583', [], 'Display', None],
  [ 0xf584, 'UnicodeCharacter0xf584', [], 'Display', None],
  [ 0xf585, 'UnicodeCharacter0xf585', [], 'Display', None],
  [ 0xf586, 'UnicodeCharacter0xf586', [], 'Display', None],
  [ 0xf587, 'UnicodeCharacter0xf587', [], 'Display', None],
  [ 0xf588, 'UnicodeCharacter0xf588', [], 'Display', None],
  [ 0xf589, 'UnicodeCharacter0xf589', [], 'Display', None],
  [ 0xf58a, 'UnicodeCharacter0xf58a', [], 'Display', None],
  [ 0xf58b, 'UnicodeCharacter0xf58b', [], 'Display', None],
  [ 0xf58c, 'UnicodeCharacter0xf58c', [], 'Display', None],
  [ 0xf58d, 'UnicodeCharacter0xf58d', [], 'Display', None],
  [ 0xf58e, 'UnicodeCharacter0xf58e', [], 'Display', None],
  [ 0xf58f, 'UnicodeCharacter0xf58f', [], 'Display', None],
  [ 0xf590, 'UnicodeCharacter0xf590', [], 'Display', None],
  [ 0xf591, 'UnicodeCharacter0xf591', [], 'Display', None],
  [ 0xf592, 'UnicodeCharacter0xf592', [], 'Display', None],
  [ 0xf593, 'UnicodeCharacter0xf593', [], 'Display', None],
  [ 0xf594, 'UnicodeCharacter0xf594', [], 'Display', None],
  [ 0xf595, 'UnicodeCharacter0xf595', [], 'Display', None],
  [ 0xf596, 'UnicodeCharacter0xf596', [], 'Display', None],
  [ 0xf597, 'UnicodeCharacter0xf597', [], 'Display', None],
  [ 0xf598, 'UnicodeCharacter0xf598', [], 'Display', None],
  [ 0xf599, 'UnicodeCharacter0xf599', [], 'Display', None],
  [ 0xf59a, 'UnicodeCharacter0xf59a', [], 'Display', None],
  [ 0xf59b, 'UnicodeCharacter0xf59b', [], 'Display', None],
  [ 0xf59c, 'UnicodeCharacter0xf59c', [], 'Display', None],
  [ 0xf59d, 'UnicodeCharacter0xf59d', [], 'Display', None],
  [ 0xf59e, 'UnicodeCharacter0xf59e', [], 'Display', None],
  [ 0xf59f, 'UnicodeCharacter0xf59f', [], 'Display', None],
  [ 0xf5a0, 'UnicodeCharacter0xf5a0', [], 'Display', None],
  [ 0xf5a1, 'UnicodeCharacter0xf5a1', [], 'Display', None],
  [ 0xf5a2, 'UnicodeCharacter0xf5a2', [], 'Display', None],
  [ 0xf5a3, 'UnicodeCharacter0xf5a3', [], 'Display', None],
  [ 0xf5a4, 'UnicodeCharacter0xf5a4', [], 'Display', None],
  [ 0xf5a5, 'UnicodeCharacter0xf5a5', [], 'Display', None],
  [ 0xf5a6, 'UnicodeCharacter0xf5a6', [], 'Display', None],
  [ 0xf5a7, 'UnicodeCharacter0xf5a7', [], 'Display', None],
  [ 0xf5a8, 'UnicodeCharacter0xf5a8', [], 'Display', None],
  [ 0xf5a9, 'UnicodeCharacter0xf5a9', [], 'Display', None],
  [ 0xf5aa, 'UnicodeCharacter0xf5aa', [], 'Display', None],
  [ 0xf5ab, 'UnicodeCharacter0xf5ab', [], 'Display', None],
  [ 0xf5ac, 'UnicodeCharacter0xf5ac', [], 'Display', None],
  [ 0xf5ad, 'UnicodeCharacter0xf5ad', [], 'Display', None],
  [ 0xf5ae, 'UnicodeCharacter0xf5ae', [], 'Display', None],
  [ 0xf5af, 'UnicodeCharacter0xf5af', [], 'Display', None],
  [ 0xf5b0, 'UnicodeCharacter0xf5b0', [], 'Display', None],
  [ 0xf5b1, 'UnicodeCharacter0xf5b1', [], 'Display', None],
  [ 0xf5b2, 'UnicodeCharacter0xf5b2', [], 'Display', None],
  [ 0xf5b3, 'UnicodeCharacter0xf5b3', [], 'Display', None],
  [ 0xf5b4, 'UnicodeCharacter0xf5b4', [], 'Display', None],
  [ 0xf5b5, 'UnicodeCharacter0xf5b5', [], 'Display', None],
  [ 0xf5b6, 'UnicodeCharacter0xf5b6', [], 'Display', None],
  [ 0xf5b7, 'UnicodeCharacter0xf5b7', [], 'Display', None],
  [ 0xf5b8, 'UnicodeCharacter0xf5b8', [], 'Display', None],
  [ 0xf5b9, 'UnicodeCharacter0xf5b9', [], 'Display', None],
  [ 0xf5ba, 'UnicodeCharacter0xf5ba', [], 'Display', None],
  [ 0xf5bb, 'UnicodeCharacter0xf5bb', [], 'Display', None],
  [ 0xf5bc, 'UnicodeCharacter0xf5bc', [], 'Display', None],
  [ 0xf5bd, 'UnicodeCharacter0xf5bd', [], 'Display', None],
  [ 0xf5be, 'UnicodeCharacter0xf5be', [], 'Display', None],
  [ 0xf5bf, 'UnicodeCharacter0xf5bf', [], 'Display', None],
  [ 0xf5c0, 'UnicodeCharacter0xf5c0', [], 'Display', None],
  [ 0xf5c1, 'UnicodeCharacter0xf5c1', [], 'Display', None],
  [ 0xf5c2, 'UnicodeCharacter0xf5c2', [], 'Display', None],
  [ 0xf5c3, 'UnicodeCharacter0xf5c3', [], 'Display', None],
  [ 0xf5c4, 'UnicodeCharacter0xf5c4', [], 'Display', None],
  [ 0xf5c5, 'UnicodeCharacter0xf5c5', [], 'Display', None],
  [ 0xf5c6, 'UnicodeCharacter0xf5c6', [], 'Display', None],
  [ 0xf5c7, 'UnicodeCharacter0xf5c7', [], 'Display', None],
  [ 0xf5c8, 'UnicodeCharacter0xf5c8', [], 'Display', None],
  [ 0xf5c9, 'UnicodeCharacter0xf5c9', [], 'Display', None],
  [ 0xf5ca, 'UnicodeCharacter0xf5ca', [], 'Display', None],
  [ 0xf5cb, 'UnicodeCharacter0xf5cb', [], 'Display', None],
  [ 0xf5cc, 'UnicodeCharacter0xf5cc', [], 'Display', None],
  [ 0xf5cd, 'UnicodeCharacter0xf5cd', [], 'Display', None],
  [ 0xf5ce, 'UnicodeCharacter0xf5ce', [], 'Display', None],
  [ 0xf5cf, 'UnicodeCharacter0xf5cf', [], 'Display', None],
  [ 0xf5d0, 'UnicodeCharacter0xf5d0', [], 'Display', None],
  [ 0xf5d1, 'UnicodeCharacter0xf5d1', [], 'Display', None],
  [ 0xf5d2, 'UnicodeCharacter0xf5d2', [], 'Display', None],
  [ 0xf5d3, 'UnicodeCharacter0xf5d3', [], 'Display', None],
  [ 0xf5d4, 'UnicodeCharacter0xf5d4', [], 'Display', None],
  [ 0xf5d5, 'UnicodeCharacter0xf5d5', [], 'Display', None],
  [ 0xf5d6, 'UnicodeCharacter0xf5d6', [], 'Display', None],
  [ 0xf5d7, 'UnicodeCharacter0xf5d7', [], 'Display', None],
  [ 0xf5d8, 'UnicodeCharacter0xf5d8', [], 'Display', None],
  [ 0xf5d9, 'UnicodeCharacter0xf5d9', [], 'Display', None],
  [ 0xf5da, 'UnicodeCharacter0xf5da', [], 'Display', None],
  [ 0xf5db, 'UnicodeCharacter0xf5db', [], 'Display', None],
  [ 0xf5dc, 'UnicodeCharacter0xf5dc', [], 'Display', None],
  [ 0xf5dd, 'UnicodeCharacter0xf5dd', [], 'Display', None],
  [ 0xf5de, 'UnicodeCharacter0xf5de', [], 'Display', None],
  [ 0xf5df, 'UnicodeCharacter0xf5df', [], 'Display', None],
  [ 0xf5e0, 'UnicodeCharacter0xf5e0', [], 'Display', None],
  [ 0xf5e1, 'UnicodeCharacter0xf5e1', [], 'Display', None],
  [ 0xf5e2, 'UnicodeCharacter0xf5e2', [], 'Display', None],
  [ 0xf5e3, 'UnicodeCharacter0xf5e3', [], 'Display', None],
  [ 0xf5e4, 'UnicodeCharacter0xf5e4', [], 'Display', None],
  [ 0xf5e5, 'UnicodeCharacter0xf5e5', [], 'Display', None],
  [ 0xf5e6, 'UnicodeCharacter0xf5e6', [], 'Display', None],
  [ 0xf5e7, 'UnicodeCharacter0xf5e7', [], 'Display', None],
  [ 0xf5e8, 'UnicodeCharacter0xf5e8', [], 'Display', None],
  [ 0xf5e9, 'UnicodeCharacter0xf5e9', [], 'Display', None],
  [ 0xf5ea, 'UnicodeCharacter0xf5ea', [], 'Display', None],
  [ 0xf5eb, 'UnicodeCharacter0xf5eb', [], 'Display', None],
  [ 0xf5ec, 'UnicodeCharacter0xf5ec', [], 'Display', None],
  [ 0xf5ed, 'UnicodeCharacter0xf5ed', [], 'Display', None],
  [ 0xf5ee, 'UnicodeCharacter0xf5ee', [], 'Display', None],
  [ 0xf5ef, 'UnicodeCharacter0xf5ef', [], 'Display', None],
  [ 0xf5f0, 'UnicodeCharacter0xf5f0', [], 'Display', None],
  [ 0xf5f1, 'UnicodeCharacter0xf5f1', [], 'Display', None],
  [ 0xf5f2, 'UnicodeCharacter0xf5f2', [], 'Display', None],
  [ 0xf5f3, 'UnicodeCharacter0xf5f3', [], 'Display', None],
  [ 0xf5f4, 'UnicodeCharacter0xf5f4', [], 'Display', None],
  [ 0xf5f5, 'UnicodeCharacter0xf5f5', [], 'Display', None],
  [ 0xf5f6, 'UnicodeCharacter0xf5f6', [], 'Display', None],
  [ 0xf5f7, 'UnicodeCharacter0xf5f7', [], 'Display', None],
  [ 0xf5f8, 'UnicodeCharacter0xf5f8', [], 'Display', None],
  [ 0xf5f9, 'UnicodeCharacter0xf5f9', [], 'Display', None],
  [ 0xf5fa, 'UnicodeCharacter0xf5fa', [], 'Display', None],
  [ 0xf5fb, 'UnicodeCharacter0xf5fb', [], 'Display', None],
  [ 0xf5fc, 'UnicodeCharacter0xf5fc', [], 'Display', None],
  [ 0xf5fd, 'UnicodeCharacter0xf5fd', [], 'Display', None],
  [ 0xf5fe, 'UnicodeCharacter0xf5fe', [], 'Display', None],
  [ 0xf5ff, 'UnicodeCharacter0xf5ff', [], 'Display', None],
  [ 0xf600, 'UnicodeCharacter0xf600', [], 'Display', None],
  [ 0xf601, 'UnicodeCharacter0xf601', [], 'Display', None],
  [ 0xf602, 'UnicodeCharacter0xf602', [], 'Display', None],
  [ 0xf603, 'UnicodeCharacter0xf603', [], 'Display', None],
  [ 0xf604, 'UnicodeCharacter0xf604', [], 'Display', None],
  [ 0xf605, 'UnicodeCharacter0xf605', [], 'Display', None],
  [ 0xf606, 'UnicodeCharacter0xf606', [], 'Display', None],
  [ 0xf607, 'UnicodeCharacter0xf607', [], 'Display', None],
  [ 0xf608, 'UnicodeCharacter0xf608', [], 'Display', None],
  [ 0xf609, 'UnicodeCharacter0xf609', [], 'Display', None],
  [ 0xf60a, 'UnicodeCharacter0xf60a', [], 'Display', None],
  [ 0xf60b, 'UnicodeCharacter0xf60b', [], 'Display', None],
  [ 0xf60c, 'UnicodeCharacter0xf60c', [], 'Display', None],
  [ 0xf60d, 'UnicodeCharacter0xf60d', [], 'Display', None],
  [ 0xf60e, 'UnicodeCharacter0xf60e', [], 'Display', None],
  [ 0xf60f, 'UnicodeCharacter0xf60f', [], 'Display', None],
  [ 0xf610, 'UnicodeCharacter0xf610', [], 'Display', None],
  [ 0xf611, 'UnicodeCharacter0xf611', [], 'Display', None],
  [ 0xf612, 'UnicodeCharacter0xf612', [], 'Display', None],
  [ 0xf613, 'UnicodeCharacter0xf613', [], 'Display', None],
  [ 0xf614, 'UnicodeCharacter0xf614', [], 'Display', None],
  [ 0xf615, 'UnicodeCharacter0xf615', [], 'Display', None],
  [ 0xf616, 'UnicodeCharacter0xf616', [], 'Display', None],
  [ 0xf617, 'UnicodeCharacter0xf617', [], 'Display', None],
  [ 0xf618, 'UnicodeCharacter0xf618', [], 'Display', None],
  [ 0xf619, 'UnicodeCharacter0xf619', [], 'Display', None],
  [ 0xf61a, 'UnicodeCharacter0xf61a', [], 'Display', None],
  [ 0xf61b, 'UnicodeCharacter0xf61b', [], 'Display', None],
  [ 0xf61c, 'UnicodeCharacter0xf61c', [], 'Display', None],
  [ 0xf61d, 'UnicodeCharacter0xf61d', [], 'Display', None],
  [ 0xf61e, 'UnicodeCharacter0xf61e', [], 'Display', None],
  [ 0xf61f, 'UnicodeCharacter0xf61f', [], 'Display', None],
  [ 0xf620, 'UnicodeCharacter0xf620', [], 'Display', None],
  [ 0xf621, 'UnicodeCharacter0xf621', [], 'Display', None],
  [ 0xf622, 'UnicodeCharacter0xf622', [], 'Display', None],
  [ 0xf623, 'UnicodeCharacter0xf623', [], 'Display', None],
  [ 0xf624, 'UnicodeCharacter0xf624', [], 'Display', None],
  [ 0xf625, 'UnicodeCharacter0xf625', [], 'Display', None],
  [ 0xf626, 'UnicodeCharacter0xf626', [], 'Display', None],
  [ 0xf627, 'UnicodeCharacter0xf627', [], 'Display', None],
  [ 0xf628, 'UnicodeCharacter0xf628', [], 'Display', None],
  [ 0xf629, 'UnicodeCharacter0xf629', [], 'Display', None],
  [ 0xf62a, 'UnicodeCharacter0xf62a', [], 'Display', None],
  [ 0xf62b, 'UnicodeCharacter0xf62b', [], 'Display', None],
  [ 0xf62c, 'UnicodeCharacter0xf62c', [], 'Display', None],
  [ 0xf62d, 'UnicodeCharacter0xf62d', [], 'Display', None],
  [ 0xf62e, 'UnicodeCharacter0xf62e', [], 'Display', None],
  [ 0xf62f, 'UnicodeCharacter0xf62f', [], 'Display', None],
  [ 0xf630, 'UnicodeCharacter0xf630', [], 'Display', None],
  [ 0xf631, 'UnicodeCharacter0xf631', [], 'Display', None],
  [ 0xf632, 'UnicodeCharacter0xf632', [], 'Display', None],
  [ 0xf633, 'UnicodeCharacter0xf633', [], 'Display', None],
  [ 0xf634, 'UnicodeCharacter0xf634', [], 'Display', None],
  [ 0xf635, 'UnicodeCharacter0xf635', [], 'Display', None],
  [ 0xf636, 'UnicodeCharacter0xf636', [], 'Display', None],
  [ 0xf637, 'UnicodeCharacter0xf637', [], 'Display', None],
  [ 0xf638, 'UnicodeCharacter0xf638', [], 'Display', None],
  [ 0xf639, 'UnicodeCharacter0xf639', [], 'Display', None],
  [ 0xf63a, 'UnicodeCharacter0xf63a', [], 'Display', None],
  [ 0xf63b, 'UnicodeCharacter0xf63b', [], 'Display', None],
  [ 0xf63c, 'UnicodeCharacter0xf63c', [], 'Display', None],
  [ 0xf63d, 'UnicodeCharacter0xf63d', [], 'Display', None],
  [ 0xf63e, 'UnicodeCharacter0xf63e', [], 'Display', None],
  [ 0xf63f, 'UnicodeCharacter0xf63f', [], 'Display', None],
  [ 0xf640, 'UnicodeCharacter0xf640', [], 'Display', None],
  [ 0xf641, 'UnicodeCharacter0xf641', [], 'Display', None],
  [ 0xf642, 'UnicodeCharacter0xf642', [], 'Display', None],
  [ 0xf643, 'UnicodeCharacter0xf643', [], 'Display', None],
  [ 0xf644, 'UnicodeCharacter0xf644', [], 'Display', None],
  [ 0xf645, 'UnicodeCharacter0xf645', [], 'Display', None],
  [ 0xf646, 'UnicodeCharacter0xf646', [], 'Display', None],
  [ 0xf647, 'UnicodeCharacter0xf647', [], 'Display', None],
  [ 0xf648, 'UnicodeCharacter0xf648', [], 'Display', None],
  [ 0xf649, 'UnicodeCharacter0xf649', [], 'Display', None],
  [ 0xf64a, 'UnicodeCharacter0xf64a', [], 'Display', None],
  [ 0xf64b, 'UnicodeCharacter0xf64b', [], 'Display', None],
  [ 0xf64c, 'UnicodeCharacter0xf64c', [], 'Display', None],
  [ 0xf64d, 'UnicodeCharacter0xf64d', [], 'Display', None],
  [ 0xf64e, 'UnicodeCharacter0xf64e', [], 'Display', None],
  [ 0xf64f, 'UnicodeCharacter0xf64f', [], 'Display', None],
  [ 0xf650, 'UnicodeCharacter0xf650', [], 'Display', None],
  [ 0xf651, 'UnicodeCharacter0xf651', [], 'Display', None],
  [ 0xf652, 'UnicodeCharacter0xf652', [], 'Display', None],
  [ 0xf653, 'UnicodeCharacter0xf653', [], 'Display', None],
  [ 0xf654, 'UnicodeCharacter0xf654', [], 'Display', None],
  [ 0xf655, 'UnicodeCharacter0xf655', [], 'Display', None],
  [ 0xf656, 'UnicodeCharacter0xf656', [], 'Display', None],
  [ 0xf657, 'UnicodeCharacter0xf657', [], 'Display', None],
  [ 0xf658, 'UnicodeCharacter0xf658', [], 'Display', None],
  [ 0xf659, 'UnicodeCharacter0xf659', [], 'Display', None],
  [ 0xf65a, 'UnicodeCharacter0xf65a', [], 'Display', None],
  [ 0xf65b, 'UnicodeCharacter0xf65b', [], 'Display', None],
  [ 0xf65c, 'UnicodeCharacter0xf65c', [], 'Display', None],
  [ 0xf65d, 'UnicodeCharacter0xf65d', [], 'Display', None],
  [ 0xf65e, 'UnicodeCharacter0xf65e', [], 'Display', None],
  [ 0xf65f, 'UnicodeCharacter0xf65f', [], 'Display', None],
  [ 0xf660, 'UnicodeCharacter0xf660', [], 'Display', None],
  [ 0xf661, 'UnicodeCharacter0xf661', [], 'Display', None],
  [ 0xf662, 'UnicodeCharacter0xf662', [], 'Display', None],
  [ 0xf663, 'UnicodeCharacter0xf663', [], 'Display', None],
  [ 0xf664, 'UnicodeCharacter0xf664', [], 'Display', None],
  [ 0xf665, 'UnicodeCharacter0xf665', [], 'Display', None],
  [ 0xf666, 'UnicodeCharacter0xf666', [], 'Display', None],
  [ 0xf667, 'UnicodeCharacter0xf667', [], 'Display', None],
  [ 0xf668, 'UnicodeCharacter0xf668', [], 'Display', None],
  [ 0xf669, 'UnicodeCharacter0xf669', [], 'Display', None],
  [ 0xf66a, 'UnicodeCharacter0xf66a', [], 'Display', None],
  [ 0xf66b, 'UnicodeCharacter0xf66b', [], 'Display', None],
  [ 0xf66c, 'UnicodeCharacter0xf66c', [], 'Display', None],
  [ 0xf66d, 'UnicodeCharacter0xf66d', [], 'Display', None],
  [ 0xf66e, 'UnicodeCharacter0xf66e', [], 'Display', None],
  [ 0xf66f, 'UnicodeCharacter0xf66f', [], 'Display', None],
  [ 0xf670, 'UnicodeCharacter0xf670', [], 'Display', None],
  [ 0xf671, 'UnicodeCharacter0xf671', [], 'Display', None],
  [ 0xf672, 'UnicodeCharacter0xf672', [], 'Display', None],
  [ 0xf673, 'UnicodeCharacter0xf673', [], 'Display', None],
  [ 0xf674, 'UnicodeCharacter0xf674', [], 'Display', None],
  [ 0xf675, 'UnicodeCharacter0xf675', [], 'Display', None],
  [ 0xf676, 'UnicodeCharacter0xf676', [], 'Display', None],
  [ 0xf677, 'UnicodeCharacter0xf677', [], 'Display', None],
  [ 0xf678, 'UnicodeCharacter0xf678', [], 'Display', None],
  [ 0xf679, 'UnicodeCharacter0xf679', [], 'Display', None],
  [ 0xf67a, 'UnicodeCharacter0xf67a', [], 'Display', None],
  [ 0xf67b, 'UnicodeCharacter0xf67b', [], 'Display', None],
  [ 0xf67c, 'UnicodeCharacter0xf67c', [], 'Display', None],
  [ 0xf67d, 'UnicodeCharacter0xf67d', [], 'Display', None],
  [ 0xf67e, 'UnicodeCharacter0xf67e', [], 'Display', None],
  [ 0xf67f, 'UnicodeCharacter0xf67f', [], 'Display', None],
  [ 0xf680, 'UnicodeCharacter0xf680', [], 'Display', None],
  [ 0xf681, 'UnicodeCharacter0xf681', [], 'Display', None],
  [ 0xf682, 'UnicodeCharacter0xf682', [], 'Display', None],
  [ 0xf683, 'UnicodeCharacter0xf683', [], 'Display', None],
  [ 0xf684, 'UnicodeCharacter0xf684', [], 'Display', None],
  [ 0xf685, 'UnicodeCharacter0xf685', [], 'Display', None],
  [ 0xf686, 'UnicodeCharacter0xf686', [], 'Display', None],
  [ 0xf687, 'UnicodeCharacter0xf687', [], 'Display', None],
  [ 0xf688, 'UnicodeCharacter0xf688', [], 'Display', None],
  [ 0xf689, 'UnicodeCharacter0xf689', [], 'Display', None],
  [ 0xf68a, 'UnicodeCharacter0xf68a', [], 'Display', None],
  [ 0xf68b, 'UnicodeCharacter0xf68b', [], 'Display', None],
  [ 0xf68c, 'UnicodeCharacter0xf68c', [], 'Display', None],
  [ 0xf68d, 'UnicodeCharacter0xf68d', [], 'Display', None],
  [ 0xf68e, 'UnicodeCharacter0xf68e', [], 'Display', None],
  [ 0xf68f, 'UnicodeCharacter0xf68f', [], 'Display', None],
  [ 0xf690, 'UnicodeCharacter0xf690', [], 'Display', None],
  [ 0xf691, 'UnicodeCharacter0xf691', [], 'Display', None],
  [ 0xf692, 'UnicodeCharacter0xf692', [], 'Display', None],
  [ 0xf693, 'UnicodeCharacter0xf693', [], 'Display', None],
  [ 0xf694, 'UnicodeCharacter0xf694', [], 'Display', None],
  [ 0xf695, 'UnicodeCharacter0xf695', [], 'Display', None],
  [ 0xf696, 'UnicodeCharacter0xf696', [], 'Display', None],
  [ 0xf697, 'UnicodeCharacter0xf697', [], 'Display', None],
  [ 0xf698, 'UnicodeCharacter0xf698', [], 'Display', None],
  [ 0xf699, 'UnicodeCharacter0xf699', [], 'Display', None],
  [ 0xf69a, 'UnicodeCharacter0xf69a', [], 'Display', None],
  [ 0xf69b, 'UnicodeCharacter0xf69b', [], 'Display', None],
  [ 0xf69c, 'UnicodeCharacter0xf69c', [], 'Display', None],
  [ 0xf69d, 'UnicodeCharacter0xf69d', [], 'Display', None],
  [ 0xf69e, 'UnicodeCharacter0xf69e', [], 'Display', None],
  [ 0xf69f, 'UnicodeCharacter0xf69f', [], 'Display', None],
  [ 0xf6a0, 'UnicodeCharacter0xf6a0', [], 'Display', None],
  [ 0xf6a1, 'UnicodeCharacter0xf6a1', [], 'Display', None],
  [ 0xf6a2, 'UnicodeCharacter0xf6a2', [], 'Display', None],
  [ 0xf6a3, 'UnicodeCharacter0xf6a3', [], 'Display', None],
  [ 0xf6a4, 'UnicodeCharacter0xf6a4', [], 'Display', None],
  [ 0xf6a5, 'UnicodeCharacter0xf6a5', [], 'Display', None],
  [ 0xf6a6, 'UnicodeCharacter0xf6a6', [], 'Display', None],
  [ 0xf6a7, 'UnicodeCharacter0xf6a7', [], 'Display', None],
  [ 0xf6a8, 'UnicodeCharacter0xf6a8', [], 'Display', None],
  [ 0xf6a9, 'UnicodeCharacter0xf6a9', [], 'Display', None],
  [ 0xf6aa, 'UnicodeCharacter0xf6aa', [], 'Display', None],
  [ 0xf6ab, 'UnicodeCharacter0xf6ab', [], 'Display', None],
  [ 0xf6ac, 'UnicodeCharacter0xf6ac', [], 'Display', None],
  [ 0xf6ad, 'UnicodeCharacter0xf6ad', [], 'Display', None],
  [ 0xf6ae, 'UnicodeCharacter0xf6ae', [], 'Display', None],
  [ 0xf6af, 'UnicodeCharacter0xf6af', [], 'Display', None],
  [ 0xf6b0, 'UnicodeCharacter0xf6b0', [], 'Display', None],
  [ 0xf6b1, 'UnicodeCharacter0xf6b1', [], 'Display', None],
  [ 0xf6b2, 'UnicodeCharacter0xf6b2', [], 'Display', None],
  [ 0xf6b3, 'UnicodeCharacter0xf6b3', [], 'Display', None],
  [ 0xf6b4, 'UnicodeCharacter0xf6b4', [], 'Display', None],
  [ 0xf6b5, 'UnicodeCharacter0xf6b5', [], 'Display', None],
  [ 0xf6b6, 'UnicodeCharacter0xf6b6', [], 'Display', None],
  [ 0xf6b7, 'UnicodeCharacter0xf6b7', [], 'Display', None],
  [ 0xf6b8, 'UnicodeCharacter0xf6b8', [], 'Display', None],
  [ 0xf6b9, 'UnicodeCharacter0xf6b9', [], 'Display', None],
  [ 0xf6ba, 'UnicodeCharacter0xf6ba', [], 'Display', None],
  [ 0xf6bb, 'UnicodeCharacter0xf6bb', [], 'Display', None],
  [ 0xf6bc, 'UnicodeCharacter0xf6bc', [], 'Display', None],
  [ 0xf6bd, 'UnicodeCharacter0xf6bd', [], 'Display', None],
  [ 0xf6be, 'UnicodeCharacter0xf6be', [], 'Display', None],
  [ 0xf6bf, 'UnicodeCharacter0xf6bf', [], 'Display', None],
  [ 0xf6c0, 'UnicodeCharacter0xf6c0', [], 'Display', None],
  [ 0xf6c1, 'UnicodeCharacter0xf6c1', [], 'Display', None],
  [ 0xf6c2, 'UnicodeCharacter0xf6c2', [], 'Display', None],
  [ 0xf6c3, 'UnicodeCharacter0xf6c3', [], 'Display', None],
  [ 0xf6c4, 'UnicodeCharacter0xf6c4', [], 'Display', None],
  [ 0xf6c5, 'UnicodeCharacter0xf6c5', [], 'Display', None],
  [ 0xf6c6, 'UnicodeCharacter0xf6c6', [], 'Display', None],
  [ 0xf6c7, 'UnicodeCharacter0xf6c7', [], 'Display', None],
  [ 0xf6c8, 'UnicodeCharacter0xf6c8', [], 'Display', None],
  [ 0xf6c9, 'UnicodeCharacter0xf6c9', [], 'Display', None],
  [ 0xf6ca, 'UnicodeCharacter0xf6ca', [], 'Display', None],
  [ 0xf6cb, 'UnicodeCharacter0xf6cb', [], 'Display', None],
  [ 0xf6cc, 'UnicodeCharacter0xf6cc', [], 'Display', None],
  [ 0xf6cd, 'UnicodeCharacter0xf6cd', [], 'Display', None],
  [ 0xf6ce, 'UnicodeCharacter0xf6ce', [], 'Display', None],
  [ 0xf6cf, 'UnicodeCharacter0xf6cf', [], 'Display', None],
  [ 0xf6d0, 'UnicodeCharacter0xf6d0', [], 'Display', None],
  [ 0xf6d1, 'UnicodeCharacter0xf6d1', [], 'Display', None],
  [ 0xf6d2, 'UnicodeCharacter0xf6d2', [], 'Display', None],
  [ 0xf6d3, 'UnicodeCharacter0xf6d3', [], 'Display', None],
  [ 0xf6d4, 'UnicodeCharacter0xf6d4', [], 'Display', None],
  [ 0xf6d5, 'UnicodeCharacter0xf6d5', [], 'Display', None],
  [ 0xf6d6, 'UnicodeCharacter0xf6d6', [], 'Display', None],
  [ 0xf6d7, 'UnicodeCharacter0xf6d7', [], 'Display', None],
  [ 0xf6d8, 'UnicodeCharacter0xf6d8', [], 'Display', None],
  [ 0xf6d9, 'UnicodeCharacter0xf6d9', [], 'Display', None],
  [ 0xf6da, 'UnicodeCharacter0xf6da', [], 'Display', None],
  [ 0xf6db, 'UnicodeCharacter0xf6db', [], 'Display', None],
  [ 0xf6dc, 'UnicodeCharacter0xf6dc', [], 'Display', None],
  [ 0xf6dd, 'UnicodeCharacter0xf6dd', [], 'Display', None],
  [ 0xf6de, 'UnicodeCharacter0xf6de', [], 'Display', None],
  [ 0xf6df, 'UnicodeCharacter0xf6df', [], 'Display', None],
  [ 0xf6e0, 'UnicodeCharacter0xf6e0', [], 'Display', None],
  [ 0xf6e1, 'UnicodeCharacter0xf6e1', [], 'Display', None],
  [ 0xf6e2, 'UnicodeCharacter0xf6e2', [], 'Display', None],
  [ 0xf6e3, 'UnicodeCharacter0xf6e3', [], 'Display', None],
  [ 0xf6e4, 'UnicodeCharacter0xf6e4', [], 'Display', None],
  [ 0xf6e5, 'UnicodeCharacter0xf6e5', [], 'Display', None],
  [ 0xf6e6, 'UnicodeCharacter0xf6e6', [], 'Display', None],
  [ 0xf6e7, 'UnicodeCharacter0xf6e7', [], 'Display', None],
  [ 0xf6e8, 'UnicodeCharacter0xf6e8', [], 'Display', None],
  [ 0xf6e9, 'UnicodeCharacter0xf6e9', [], 'Display', None],
  [ 0xf6ea, 'UnicodeCharacter0xf6ea', [], 'Display', None],
  [ 0xf6eb, 'UnicodeCharacter0xf6eb', [], 'Display', None],
  [ 0xf6ec, 'UnicodeCharacter0xf6ec', [], 'Display', None],
  [ 0xf6ed, 'UnicodeCharacter0xf6ed', [], 'Display', None],
  [ 0xf6ee, 'UnicodeCharacter0xf6ee', [], 'Display', None],
  [ 0xf6ef, 'UnicodeCharacter0xf6ef', [], 'Display', None],
  [ 0xf6f0, 'UnicodeCharacter0xf6f0', [], 'Display', None],
  [ 0xf6f1, 'UnicodeCharacter0xf6f1', [], 'Display', None],
  [ 0xf6f2, 'UnicodeCharacter0xf6f2', [], 'Display', None],
  [ 0xf6f3, 'UnicodeCharacter0xf6f3', [], 'Display', None],
  [ 0xf6f4, 'UnicodeCharacter0xf6f4', [], 'Display', None],
  [ 0xf6f5, 'UnicodeCharacter0xf6f5', [], 'Display', None],
  [ 0xf6f6, 'UnicodeCharacter0xf6f6', [], 'Display', None],
  [ 0xf6f7, 'UnicodeCharacter0xf6f7', [], 'Display', None],
  [ 0xf6f8, 'UnicodeCharacter0xf6f8', [], 'Display', None],
  [ 0xf6f9, 'UnicodeCharacter0xf6f9', [], 'Display', None],
  [ 0xf6fa, 'UnicodeCharacter0xf6fa', [], 'Display', None],
  [ 0xf6fb, 'UnicodeCharacter0xf6fb', [], 'Display', None],
  [ 0xf6fc, 'UnicodeCharacter0xf6fc', [], 'Display', None],
  [ 0xf6fd, 'UnicodeCharacter0xf6fd', [], 'Display', None],
  [ 0xf6fe, 'UnicodeCharacter0xf6fe', [], 'Display', None],
  [ 0xf6ff, 'UnicodeCharacter0xf6ff', [], 'Display', None],
  [ 0xf700, 'UnicodeCharacter0xf700', [], 'Display', None],
  [ 0xf701, 'UnicodeCharacter0xf701', [], 'Display', None],
  [ 0xf702, 'UnicodeCharacter0xf702', [], 'Display', None],
  [ 0xf703, 'UnicodeCharacter0xf703', [], 'Display', None],
  [ 0xf704, 'UnicodeCharacter0xf704', [], 'Display', None],
  [ 0xf705, 'UnicodeCharacter0xf705', [], 'Display', None],
  [ 0xf706, 'UnicodeCharacter0xf706', [], 'Display', None],
  [ 0xf707, 'UnicodeCharacter0xf707', [], 'Display', None],
  [ 0xf708, 'UnicodeCharacter0xf708', [], 'Display', None],
  [ 0xf709, 'UnicodeCharacter0xf709', [], 'Display', None],
  [ 0xf70a, 'UnicodeCharacter0xf70a', [], 'Display', None],
  [ 0xf70b, 'UnicodeCharacter0xf70b', [], 'Display', None],
  [ 0xf70c, 'UnicodeCharacter0xf70c', [], 'Display', None],
  [ 0xf70d, 'UnicodeCharacter0xf70d', [], 'Display', None],
  [ 0xf70e, 'UnicodeCharacter0xf70e', [], 'Display', None],
  [ 0xf70f, 'UnicodeCharacter0xf70f', [], 'Display', None],
  [ 0xf710, 'UnicodeCharacter0xf710', [], 'Display', None],
  [ 0xf711, 'UnicodeCharacter0xf711', [], 'Display', None],
  [ 0xf712, 'UnicodeCharacter0xf712', [], 'Display', None],
  [ 0xf713, 'UnicodeCharacter0xf713', [], 'Display', None],
  [ 0xf714, 'UnicodeCharacter0xf714', [], 'Display', None],
  [ 0xf715, 'UnicodeCharacter0xf715', [], 'Display', None],
  [ 0xf716, 'UnicodeCharacter0xf716', [], 'Display', None],
  [ 0xf717, 'UnicodeCharacter0xf717', [], 'Display', None],
  [ 0xf718, 'UnicodeCharacter0xf718', [], 'Display', None],
  [ 0xf719, 'UnicodeCharacter0xf719', [], 'Display', None],
  [ 0xf71a, 'UnicodeCharacter0xf71a', [], 'Display', None],
  [ 0xf71b, 'UnicodeCharacter0xf71b', [], 'Display', None],
  [ 0xf71c, 'UnicodeCharacter0xf71c', [], 'Display', None],
  [ 0xf71d, 'UnicodeCharacter0xf71d', [], 'Display', None],
  [ 0xf71e, 'UnicodeCharacter0xf71e', [], 'Display', None],
  [ 0xf71f, 'UnicodeCharacter0xf71f', [], 'Display', None],
  [ 0xf720, 'UnicodeCharacter0xf720', [], 'Display', None],
  [ 0xf721, 'UnicodeCharacter0xf721', [], 'Display', None],
  [ 0xf722, 'UnicodeCharacter0xf722', [], 'Display', None],
  [ 0xf723, 'UnicodeCharacter0xf723', [], 'Display', None],
  [ 0xf724, 'UnicodeCharacter0xf724', [], 'Display', None],
  [ 0xf725, 'UnicodeCharacter0xf725', [], 'Display', None],
  [ 0xf726, 'UnicodeCharacter0xf726', [], 'Display', None],
  [ 0xf727, 'UnicodeCharacter0xf727', [], 'Display', None],
  [ 0xf728, 'UnicodeCharacter0xf728', [], 'Display', None],
  [ 0xf729, 'UnicodeCharacter0xf729', [], 'Display', None],
  [ 0xf72a, 'UnicodeCharacter0xf72a', [], 'Display', None],
  [ 0xf72b, 'UnicodeCharacter0xf72b', [], 'Display', None],
  [ 0xf72c, 'UnicodeCharacter0xf72c', [], 'Display', None],
  [ 0xf72d, 'UnicodeCharacter0xf72d', [], 'Display', None],
  [ 0xf72e, 'UnicodeCharacter0xf72e', [], 'Display', None],
  [ 0xf72f, 'UnicodeCharacter0xf72f', [], 'Display', None],
  [ 0xf730, 'UnicodeCharacter0xf730', [], 'Display', None],
  [ 0xf731, 'UnicodeCharacter0xf731', [], 'Display', None],
  [ 0xf732, 'UnicodeCharacter0xf732', [], 'Display', None],
  [ 0xf733, 'UnicodeCharacter0xf733', [], 'Display', None],
  [ 0xf734, 'UnicodeCharacter0xf734', [], 'Display', None],
  [ 0xf735, 'UnicodeCharacter0xf735', [], 'Display', None],
  [ 0xf736, 'UnicodeCharacter0xf736', [], 'Display', None],
  [ 0xf737, 'UnicodeCharacter0xf737', [], 'Display', None],
  [ 0xf738, 'UnicodeCharacter0xf738', [], 'Display', None],
  [ 0xf739, 'UnicodeCharacter0xf739', [], 'Display', None],
  [ 0xf73a, 'UnicodeCharacter0xf73a', [], 'Display', None],
  [ 0xf73b, 'UnicodeCharacter0xf73b', [], 'Display', None],
  [ 0xf73c, 'UnicodeCharacter0xf73c', [], 'Display', None],
  [ 0xf73d, 'UnicodeCharacter0xf73d', [], 'Display', None],
  [ 0xf73e, 'UnicodeCharacter0xf73e', [], 'Display', None],
  [ 0xf73f, 'UnicodeCharacter0xf73f', [], 'Display', None],
  [ 0xf740, 'UnicodeCharacter0xf740', [], 'Display', None],
  [ 0xf741, 'UnicodeCharacter0xf741', [], 'Display', None],
  [ 0xf742, 'UnicodeCharacter0xf742', [], 'Display', None],
  [ 0xf743, 'UnicodeCharacter0xf743', [], 'Display', None],
  [ 0xf744, 'UnicodeCharacter0xf744', [], 'Display', None],
  [ 0xf745, 'UnicodeCharacter0xf745', [], 'Display', None],
  [ 0xf746, 'UnicodeCharacter0xf746', [], 'Display', None],
  [ 0xf747, 'UnicodeCharacter0xf747', [], 'Display', None],
  [ 0xf748, 'UnicodeCharacter0xf748', [], 'Display', None],
  [ 0xf749, 'UnicodeCharacter0xf749', [], 'Display', None],
  [ 0xf74a, 'UnicodeCharacter0xf74a', [], 'Display', None],
  [ 0xf74b, 'UnicodeCharacter0xf74b', [], 'Display', None],
  [ 0xf74c, 'UnicodeCharacter0xf74c', [], 'Display', None],
  [ 0xf74d, 'UnicodeCharacter0xf74d', [], 'Display', None],
  [ 0xf74e, 'UnicodeCharacter0xf74e', [], 'Display', None],
  [ 0xf74f, 'UnicodeCharacter0xf74f', [], 'Display', None],
  [ 0xf750, 'UnicodeCharacter0xf750', [], 'Display', None],
  [ 0xf751, 'UnicodeCharacter0xf751', [], 'Display', None],
  [ 0xf752, 'UnicodeCharacter0xf752', [], 'Display', None],
  [ 0xf753, 'UnicodeCharacter0xf753', [], 'Display', None],
  [ 0xf754, 'UnicodeCharacter0xf754', [], 'Display', None],
  [ 0xf755, 'UnicodeCharacter0xf755', [], 'Display', None],
  [ 0xf756, 'UnicodeCharacter0xf756', [], 'Display', None],
  [ 0xf757, 'UnicodeCharacter0xf757', [], 'Display', None],
  [ 0xf758, 'UnicodeCharacter0xf758', [], 'Display', None],
  [ 0xf759, 'UnicodeCharacter0xf759', [], 'Display', None],
  [ 0xf75a, 'UnicodeCharacter0xf75a', [], 'Display', None],
  [ 0xf75b, 'UnicodeCharacter0xf75b', [], 'Display', None],
  [ 0xf75c, 'UnicodeCharacter0xf75c', [], 'Display', None],
  [ 0xf75d, 'UnicodeCharacter0xf75d', [], 'Display', None],
  [ 0xf75e, 'UnicodeCharacter0xf75e', [], 'Display', None],
  [ 0xf75f, 'UnicodeCharacter0xf75f', [], 'Display', None],
  [ 0xf760, 'UnicodeCharacter0xf760', [], 'Display', None],
  [ 0xf761, 'UnicodeCharacter0xf761', [], 'Display', None],
  [ 0xf762, 'UnicodeCharacter0xf762', [], 'Display', None],
  [ 0xf763, 'UnicodeCharacter0xf763', [], 'Display', None],
  [ 0xf764, 'UnicodeCharacter0xf764', [], 'Display', None],
  [ 0xf765, 'UnicodeCharacter0xf765', [], 'Display', None],
  [ 0xf766, 'UnicodeCharacter0xf766', [], 'Display', None],
  [ 0xf767, 'UnicodeCharacter0xf767', [], 'Display', None],
  [ 0xf768, 'UnicodeCharacter0xf768', [], 'Display', None],
  [ 0xf769, 'UnicodeCharacter0xf769', [], 'Display', None],
  [ 0xf76a, 'UnicodeCharacter0xf76a', [], 'Display', None],
  [ 0xf76b, 'UnicodeCharacter0xf76b', [], 'Display', None],
  [ 0xf76c, 'UnicodeCharacter0xf76c', [], 'Display', None],
  [ 0xf76d, 'UnicodeCharacter0xf76d', [], 'Display', None],
  [ 0xf76e, 'UnicodeCharacter0xf76e', [], 'Display', None],
  [ 0xf76f, 'UnicodeCharacter0xf76f', [], 'Display', None],
  [ 0xf770, 'UnicodeCharacter0xf770', [], 'Display', None],
  [ 0xf771, 'UnicodeCharacter0xf771', [], 'Display', None],
  [ 0xf772, 'UnicodeCharacter0xf772', [], 'Display', None],
  [ 0xf773, 'UnicodeCharacter0xf773', [], 'Display', None],
  [ 0xf774, 'UnicodeCharacter0xf774', [], 'Display', None],
  [ 0xf775, 'UnicodeCharacter0xf775', [], 'Display', None],
  [ 0xf776, 'UnicodeCharacter0xf776', [], 'Display', None],
  [ 0xf777, 'UnicodeCharacter0xf777', [], 'Display', None],
  [ 0xf778, 'UnicodeCharacter0xf778', [], 'Display', None],
  [ 0xf779, 'UnicodeCharacter0xf779', [], 'Display', None],
  [ 0xf77a, 'UnicodeCharacter0xf77a', [], 'Display', None],
  [ 0xf77b, 'UnicodeCharacter0xf77b', [], 'Display', None],
  [ 0xf77c, 'UnicodeCharacter0xf77c', [], 'Display', None],
  [ 0xf77d, 'UnicodeCharacter0xf77d', [], 'Display', None],
  [ 0xf77e, 'UnicodeCharacter0xf77e', [], 'Display', None],
  [ 0xf77f, 'UnicodeCharacter0xf77f', [], 'Display', None],
  [ 0xf780, 'UnicodeCharacter0xf780', [], 'Display', None],
  [ 0xf781, 'UnicodeCharacter0xf781', [], 'Display', None],
  [ 0xf782, 'UnicodeCharacter0xf782', [], 'Display', None],
  [ 0xf783, 'UnicodeCharacter0xf783', [], 'Display', None],
  [ 0xf784, 'UnicodeCharacter0xf784', [], 'Display', None],
  [ 0xf785, 'UnicodeCharacter0xf785', [], 'Display', None],
  [ 0xf786, 'UnicodeCharacter0xf786', [], 'Display', None],
  [ 0xf787, 'UnicodeCharacter0xf787', [], 'Display', None],
  [ 0xf788, 'UnicodeCharacter0xf788', [], 'Display', None],
  [ 0xf789, 'UnicodeCharacter0xf789', [], 'Display', None],
  [ 0xf78a, 'UnicodeCharacter0xf78a', [], 'Display', None],
  [ 0xf78b, 'UnicodeCharacter0xf78b', [], 'Display', None],
  [ 0xf78c, 'UnicodeCharacter0xf78c', [], 'Display', None],
  [ 0xf78d, 'UnicodeCharacter0xf78d', [], 'Display', None],
  [ 0xf78e, 'UnicodeCharacter0xf78e', [], 'Display', None],
  [ 0xf78f, 'UnicodeCharacter0xf78f', [], 'Display', None],
  [ 0xf790, 'UnicodeCharacter0xf790', [], 'Display', None],
  [ 0xf791, 'UnicodeCharacter0xf791', [], 'Display', None],
  [ 0xf792, 'UnicodeCharacter0xf792', [], 'Display', None],
  [ 0xf793, 'UnicodeCharacter0xf793', [], 'Display', None],
  [ 0xf794, 'UnicodeCharacter0xf794', [], 'Display', None],
  [ 0xf795, 'UnicodeCharacter0xf795', [], 'Display', None],
  [ 0xf796, 'UnicodeCharacter0xf796', [], 'Display', None],
  [ 0xf797, 'UnicodeCharacter0xf797', [], 'Display', None],
  [ 0xf798, 'UnicodeCharacter0xf798', [], 'Display', None],
  [ 0xf799, 'UnicodeCharacter0xf799', [], 'Display', None],
  [ 0xf79a, 'UnicodeCharacter0xf79a', [], 'Display', None],
  [ 0xf79b, 'UnicodeCharacter0xf79b', [], 'Display', None],
  [ 0xf79c, 'UnicodeCharacter0xf79c', [], 'Display', None],
  [ 0xf79d, 'UnicodeCharacter0xf79d', [], 'Display', None],
  [ 0xf79e, 'UnicodeCharacter0xf79e', [], 'Display', None],
  [ 0xf79f, 'UnicodeCharacter0xf79f', [], 'Display', None],
  [ 0xf7a0, 'UnicodeCharacter0xf7a0', [], 'Display', None],
  [ 0xf7a1, 'UnicodeCharacter0xf7a1', [], 'Display', None],
  [ 0xf7a2, 'UnicodeCharacter0xf7a2', [], 'Display', None],
  [ 0xf7a3, 'UnicodeCharacter0xf7a3', [], 'Display', None],
  [ 0xf7a4, 'UnicodeCharacter0xf7a4', [], 'Display', None],
  [ 0xf7a5, 'UnicodeCharacter0xf7a5', [], 'Display', None],
  [ 0xf7a6, 'UnicodeCharacter0xf7a6', [], 'Display', None],
  [ 0xf7a7, 'UnicodeCharacter0xf7a7', [], 'Display', None],
  [ 0xf7a8, 'UnicodeCharacter0xf7a8', [], 'Display', None],
  [ 0xf7a9, 'UnicodeCharacter0xf7a9', [], 'Display', None],
  [ 0xf7aa, 'UnicodeCharacter0xf7aa', [], 'Display', None],
  [ 0xf7ab, 'UnicodeCharacter0xf7ab', [], 'Display', None],
  [ 0xf7ac, 'UnicodeCharacter0xf7ac', [], 'Display', None],
  [ 0xf7ad, 'UnicodeCharacter0xf7ad', [], 'Display', None],
  [ 0xf7ae, 'UnicodeCharacter0xf7ae', [], 'Display', None],
  [ 0xf7af, 'UnicodeCharacter0xf7af', [], 'Display', None],
  [ 0xf7b0, 'UnicodeCharacter0xf7b0', [], 'Display', None],
  [ 0xf7b1, 'UnicodeCharacter0xf7b1', [], 'Display', None],
  [ 0xf7b2, 'UnicodeCharacter0xf7b2', [], 'Display', None],
  [ 0xf7b3, 'UnicodeCharacter0xf7b3', [], 'Display', None],
  [ 0xf7b4, 'UnicodeCharacter0xf7b4', [], 'Display', None],
  [ 0xf7b5, 'UnicodeCharacter0xf7b5', [], 'Display', None],
  [ 0xf7b6, 'UnicodeCharacter0xf7b6', [], 'Display', None],
  [ 0xf7b7, 'UnicodeCharacter0xf7b7', [], 'Display', None],
  [ 0xf7b8, 'UnicodeCharacter0xf7b8', [], 'Display', None],
  [ 0xf7b9, 'UnicodeCharacter0xf7b9', [], 'Display', None],
  [ 0xf7ba, 'UnicodeCharacter0xf7ba', [], 'Display', None],
  [ 0xf7bb, 'UnicodeCharacter0xf7bb', [], 'Display', None],
  [ 0xf7bc, 'UnicodeCharacter0xf7bc', [], 'Display', None],
  [ 0xf7bd, 'UnicodeCharacter0xf7bd', [], 'Display', None],
  [ 0xf7be, 'UnicodeCharacter0xf7be', [], 'Display', None],
  [ 0xf7bf, 'UnicodeCharacter0xf7bf', [], 'Display', None],
  [ 0xf7c0, 'UnicodeCharacter0xf7c0', [], 'Display', None],
  [ 0xf7c1, 'UnicodeCharacter0xf7c1', [], 'Display', None],
  [ 0xf7c2, 'UnicodeCharacter0xf7c2', [], 'Display', None],
  [ 0xf7c3, 'UnicodeCharacter0xf7c3', [], 'Display', None],
  [ 0xf7c4, 'UnicodeCharacter0xf7c4', [], 'Display', None],
  [ 0xf7c5, 'UnicodeCharacter0xf7c5', [], 'Display', None],
  [ 0xf7c6, 'UnicodeCharacter0xf7c6', [], 'Display', None],
  [ 0xf7c7, 'UnicodeCharacter0xf7c7', [], 'Display', None],
  [ 0xf7c8, 'UnicodeCharacter0xf7c8', [], 'Display', None],
  [ 0xf7c9, 'UnicodeCharacter0xf7c9', [], 'Display', None],
  [ 0xf7ca, 'UnicodeCharacter0xf7ca', [], 'Display', None],
  [ 0xf7cb, 'UnicodeCharacter0xf7cb', [], 'Display', None],
  [ 0xf7cc, 'UnicodeCharacter0xf7cc', [], 'Display', None],
  [ 0xf7cd, 'UnicodeCharacter0xf7cd', [], 'Display', None],
  [ 0xf7ce, 'UnicodeCharacter0xf7ce', [], 'Display', None],
  [ 0xf7cf, 'UnicodeCharacter0xf7cf', [], 'Display', None],
  [ 0xf7d0, 'UnicodeCharacter0xf7d0', [], 'Display', None],
  [ 0xf7d1, 'UnicodeCharacter0xf7d1', [], 'Display', None],
  [ 0xf7d2, 'UnicodeCharacter0xf7d2', [], 'Display', None],
  [ 0xf7d3, 'UnicodeCharacter0xf7d3', [], 'Display', None],
  [ 0xf7d4, 'UnicodeCharacter0xf7d4', [], 'Display', None],
  [ 0xf7d5, 'UnicodeCharacter0xf7d5', [], 'Display', None],
  [ 0xf7d6, 'UnicodeCharacter0xf7d6', [], 'Display', None],
  [ 0xf7d7, 'UnicodeCharacter0xf7d7', [], 'Display', None],
  [ 0xf7d8, 'UnicodeCharacter0xf7d8', [], 'Display', None],
  [ 0xf7d9, 'UnicodeCharacter0xf7d9', [], 'Display', None],
  [ 0xf7da, 'UnicodeCharacter0xf7da', [], 'Display', None],
  [ 0xf7db, 'UnicodeCharacter0xf7db', [], 'Display', None],
  [ 0xf7dc, 'UnicodeCharacter0xf7dc', [], 'Display', None],
  [ 0xf7dd, 'UnicodeCharacter0xf7dd', [], 'Display', None],
  [ 0xf7de, 'UnicodeCharacter0xf7de', [], 'Display', None],
  [ 0xf7df, 'UnicodeCharacter0xf7df', [], 'Display', None],
  [ 0xf7e0, 'UnicodeCharacter0xf7e0', [], 'Display', None],
  [ 0xf7e1, 'UnicodeCharacter0xf7e1', [], 'Display', None],
  [ 0xf7e2, 'UnicodeCharacter0xf7e2', [], 'Display', None],
  [ 0xf7e3, 'UnicodeCharacter0xf7e3', [], 'Display', None],
  [ 0xf7e4, 'UnicodeCharacter0xf7e4', [], 'Display', None],
  [ 0xf7e5, 'UnicodeCharacter0xf7e5', [], 'Display', None],
  [ 0xf7e6, 'UnicodeCharacter0xf7e6', [], 'Display', None],
  [ 0xf7e7, 'UnicodeCharacter0xf7e7', [], 'Display', None],
  [ 0xf7e8, 'UnicodeCharacter0xf7e8', [], 'Display', None],
  [ 0xf7e9, 'UnicodeCharacter0xf7e9', [], 'Display', None],
  [ 0xf7ea, 'UnicodeCharacter0xf7ea', [], 'Display', None],
  [ 0xf7eb, 'UnicodeCharacter0xf7eb', [], 'Display', None],
  [ 0xf7ec, 'UnicodeCharacter0xf7ec', [], 'Display', None],
  [ 0xf7ed, 'UnicodeCharacter0xf7ed', [], 'Display', None],
  [ 0xf7ee, 'UnicodeCharacter0xf7ee', [], 'Display', None],
  [ 0xf7ef, 'UnicodeCharacter0xf7ef', [], 'Display', None],
  [ 0xf7f0, 'UnicodeCharacter0xf7f0', [], 'Display', None],
  [ 0xf7f1, 'UnicodeCharacter0xf7f1', [], 'Display', None],
  [ 0xf7f2, 'UnicodeCharacter0xf7f2', [], 'Display', None],
  [ 0xf7f3, 'UnicodeCharacter0xf7f3', [], 'Display', None],
  [ 0xf7f4, 'UnicodeCharacter0xf7f4', [], 'Display', None],
  [ 0xf7f5, 'UnicodeCharacter0xf7f5', [], 'Display', None],
  [ 0xf7f6, 'UnicodeCharacter0xf7f6', [], 'Display', None],
  [ 0xf7f7, 'UnicodeCharacter0xf7f7', [], 'Display', None],
  [ 0xf7f8, 'UnicodeCharacter0xf7f8', [], 'Display', None],
  [ 0xf7f9, 'UnicodeCharacter0xf7f9', [], 'Display', None],
  [ 0xf7fa, 'UnicodeCharacter0xf7fa', [], 'Display', None],
  [ 0xf7fb, 'UnicodeCharacter0xf7fb', [], 'Display', None],
  [ 0xf7fc, 'UnicodeCharacter0xf7fc', [], 'Display', None],
  [ 0xf7fd, 'UnicodeCharacter0xf7fd', [], 'Display', None],
  [ 0xf7fe, 'UnicodeCharacter0xf7fe', [], 'Display', None],
  [ 0xf7ff, 'UnicodeCharacter0xf7ff', [], 'Display', None],
  [ 0xf800, 'UnicodeCharacter0xf800', [], 'Display', None],
  [ 0xf801, 'UnicodeCharacter0xf801', [], 'Display', None],
  [ 0xf802, 'UnicodeCharacter0xf802', [], 'Display', None],
  [ 0xf803, 'UnicodeCharacter0xf803', [], 'Display', None],
  [ 0xf804, 'UnicodeCharacter0xf804', [], 'Display', None],
  [ 0xf805, 'UnicodeCharacter0xf805', [], 'Display', None],
  [ 0xf806, 'UnicodeCharacter0xf806', [], 'Display', None],
  [ 0xf807, 'UnicodeCharacter0xf807', [], 'Display', None],
  [ 0xf808, 'UnicodeCharacter0xf808', [], 'Display', None],
  [ 0xf809, 'UnicodeCharacter0xf809', [], 'Display', None],
  [ 0xf80a, 'UnicodeCharacter0xf80a', [], 'Display', None],
  [ 0xf80b, 'UnicodeCharacter0xf80b', [], 'Display', None],
  [ 0xf80c, 'UnicodeCharacter0xf80c', [], 'Display', None],
  [ 0xf80d, 'UnicodeCharacter0xf80d', [], 'Display', None],
  [ 0xf80e, 'UnicodeCharacter0xf80e', [], 'Display', None],
  [ 0xf80f, 'UnicodeCharacter0xf80f', [], 'Display', None],
  [ 0xf810, 'UnicodeCharacter0xf810', [], 'Display', None],
  [ 0xf811, 'UnicodeCharacter0xf811', [], 'Display', None],
  [ 0xf812, 'UnicodeCharacter0xf812', [], 'Display', None],
  [ 0xf813, 'UnicodeCharacter0xf813', [], 'Display', None],
  [ 0xf814, 'UnicodeCharacter0xf814', [], 'Display', None],
  [ 0xf815, 'UnicodeCharacter0xf815', [], 'Display', None],
  [ 0xf816, 'UnicodeCharacter0xf816', [], 'Display', None],
  [ 0xf817, 'UnicodeCharacter0xf817', [], 'Display', None],
  [ 0xf818, 'UnicodeCharacter0xf818', [], 'Display', None],
  [ 0xf819, 'UnicodeCharacter0xf819', [], 'Display', None],
  [ 0xf81a, 'UnicodeCharacter0xf81a', [], 'Display', None],
  [ 0xf81b, 'UnicodeCharacter0xf81b', [], 'Display', None],
  [ 0xf81c, 'UnicodeCharacter0xf81c', [], 'Display', None],
  [ 0xf81d, 'UnicodeCharacter0xf81d', [], 'Display', None],
  [ 0xf81e, 'UnicodeCharacter0xf81e', [], 'Display', None],
  [ 0xf81f, 'UnicodeCharacter0xf81f', [], 'Display', None],
  [ 0xf820, 'UnicodeCharacter0xf820', [], 'Display', None],
  [ 0xf821, 'UnicodeCharacter0xf821', [], 'Display', None],
  [ 0xf822, 'UnicodeCharacter0xf822', [], 'Display', None],
  [ 0xf823, 'UnicodeCharacter0xf823', [], 'Display', None],
  [ 0xf824, 'UnicodeCharacter0xf824', [], 'Display', None],
  [ 0xf825, 'UnicodeCharacter0xf825', [], 'Display', None],
  [ 0xf826, 'UnicodeCharacter0xf826', [], 'Display', None],
  [ 0xf827, 'UnicodeCharacter0xf827', [], 'Display', None],
  [ 0xf828, 'UnicodeCharacter0xf828', [], 'Display', None],
  [ 0xf829, 'UnicodeCharacter0xf829', [], 'Display', None],
  [ 0xf82a, 'UnicodeCharacter0xf82a', [], 'Display', None],
  [ 0xf82b, 'UnicodeCharacter0xf82b', [], 'Display', None],
  [ 0xf82c, 'UnicodeCharacter0xf82c', [], 'Display', None],
  [ 0xf82d, 'UnicodeCharacter0xf82d', [], 'Display', None],
  [ 0xf82e, 'UnicodeCharacter0xf82e', [], 'Display', None],
  [ 0xf82f, 'UnicodeCharacter0xf82f', [], 'Display', None],
  [ 0xf830, 'UnicodeCharacter0xf830', [], 'Display', None],
  [ 0xf831, 'UnicodeCharacter0xf831', [], 'Display', None],
  [ 0xf832, 'UnicodeCharacter0xf832', [], 'Display', None],
  [ 0xf833, 'UnicodeCharacter0xf833', [], 'Display', None],
  [ 0xf834, 'UnicodeCharacter0xf834', [], 'Display', None],
  [ 0xf835, 'UnicodeCharacter0xf835', [], 'Display', None],
  [ 0xf836, 'UnicodeCharacter0xf836', [], 'Display', None],
  [ 0xf837, 'UnicodeCharacter0xf837', [], 'Display', None],
  [ 0xf838, 'UnicodeCharacter0xf838', [], 'Display', None],
  [ 0xf839, 'UnicodeCharacter0xf839', [], 'Display', None],
  [ 0xf83a, 'UnicodeCharacter0xf83a', [], 'Display', None],
  [ 0xf83b, 'UnicodeCharacter0xf83b', [], 'Display', None],
  [ 0xf83c, 'UnicodeCharacter0xf83c', [], 'Display', None],
  [ 0xf83d, 'UnicodeCharacter0xf83d', [], 'Display', None],
  [ 0xf83e, 'UnicodeCharacter0xf83e', [], 'Display', None],
  [ 0xf83f, 'UnicodeCharacter0xf83f', [], 'Display', None],
  [ 0xf840, 'UnicodeCharacter0xf840', [], 'Display', None],
  [ 0xf841, 'UnicodeCharacter0xf841', [], 'Display', None],
  [ 0xf842, 'UnicodeCharacter0xf842', [], 'Display', None],
  [ 0xf843, 'UnicodeCharacter0xf843', [], 'Display', None],
  [ 0xf844, 'UnicodeCharacter0xf844', [], 'Display', None],
  [ 0xf845, 'UnicodeCharacter0xf845', [], 'Display', None],
  [ 0xf846, 'UnicodeCharacter0xf846', [], 'Display', None],
  [ 0xf847, 'UnicodeCharacter0xf847', [], 'Display', None],
  [ 0xf848, 'UnicodeCharacter0xf848', [], 'Display', None],
  [ 0xf849, 'UnicodeCharacter0xf849', [], 'Display', None],
  [ 0xf84a, 'UnicodeCharacter0xf84a', [], 'Display', None],
  [ 0xf84b, 'UnicodeCharacter0xf84b', [], 'Display', None],
  [ 0xf84c, 'UnicodeCharacter0xf84c', [], 'Display', None],
  [ 0xf84d, 'UnicodeCharacter0xf84d', [], 'Display', None],
  [ 0xf84e, 'UnicodeCharacter0xf84e', [], 'Display', None],
  [ 0xf84f, 'UnicodeCharacter0xf84f', [], 'Display', None],
  [ 0xf850, 'UnicodeCharacter0xf850', [], 'Display', None],
  [ 0xf851, 'UnicodeCharacter0xf851', [], 'Display', None],
  [ 0xf852, 'UnicodeCharacter0xf852', [], 'Display', None],
  [ 0xf853, 'UnicodeCharacter0xf853', [], 'Display', None],
  [ 0xf854, 'UnicodeCharacter0xf854', [], 'Display', None],
  [ 0xf855, 'UnicodeCharacter0xf855', [], 'Display', None],
  [ 0xf856, 'UnicodeCharacter0xf856', [], 'Display', None],
  [ 0xf857, 'UnicodeCharacter0xf857', [], 'Display', None],
  [ 0xf858, 'UnicodeCharacter0xf858', [], 'Display', None],
  [ 0xf859, 'UnicodeCharacter0xf859', [], 'Display', None],
  [ 0xf85a, 'UnicodeCharacter0xf85a', [], 'Display', None],
  [ 0xf85b, 'UnicodeCharacter0xf85b', [], 'Display', None],
  [ 0xf85c, 'UnicodeCharacter0xf85c', [], 'Display', None],
  [ 0xf85d, 'UnicodeCharacter0xf85d', [], 'Display', None],
  [ 0xf85e, 'UnicodeCharacter0xf85e', [], 'Display', None],
  [ 0xf85f, 'UnicodeCharacter0xf85f', [], 'Display', None],
  [ 0xf860, 'UnicodeCharacter0xf860', [], 'Display', None],
  [ 0xf861, 'UnicodeCharacter0xf861', [], 'Display', None],
  [ 0xf862, 'UnicodeCharacter0xf862', [], 'Display', None],
  [ 0xf863, 'UnicodeCharacter0xf863', [], 'Display', None],
  [ 0xf864, 'UnicodeCharacter0xf864', [], 'Display', None],
  [ 0xf865, 'UnicodeCharacter0xf865', [], 'Display', None],
  [ 0xf866, 'UnicodeCharacter0xf866', [], 'Display', None],
  [ 0xf867, 'UnicodeCharacter0xf867', [], 'Display', None],
  [ 0xf868, 'UnicodeCharacter0xf868', [], 'Display', None],
  [ 0xf869, 'UnicodeCharacter0xf869', [], 'Display', None],
  [ 0xf86a, 'UnicodeCharacter0xf86a', [], 'Display', None],
  [ 0xf86b, 'UnicodeCharacter0xf86b', [], 'Display', None],
  [ 0xf86c, 'UnicodeCharacter0xf86c', [], 'Display', None],
  [ 0xf86d, 'UnicodeCharacter0xf86d', [], 'Display', None],
  [ 0xf86e, 'UnicodeCharacter0xf86e', [], 'Display', None],
  [ 0xf86f, 'UnicodeCharacter0xf86f', [], 'Display', None],
  [ 0xf870, 'UnicodeCharacter0xf870', [], 'Display', None],
  [ 0xf871, 'UnicodeCharacter0xf871', [], 'Display', None],
  [ 0xf872, 'UnicodeCharacter0xf872', [], 'Display', None],
  [ 0xf873, 'UnicodeCharacter0xf873', [], 'Display', None],
  [ 0xf874, 'UnicodeCharacter0xf874', [], 'Display', None],
  [ 0xf875, 'UnicodeCharacter0xf875', [], 'Display', None],
  [ 0xf876, 'UnicodeCharacter0xf876', [], 'Display', None],
  [ 0xf877, 'UnicodeCharacter0xf877', [], 'Display', None],
  [ 0xf878, 'UnicodeCharacter0xf878', [], 'Display', None],
  [ 0xf879, 'UnicodeCharacter0xf879', [], 'Display', None],
  [ 0xf87a, 'UnicodeCharacter0xf87a', [], 'Display', None],
  [ 0xf87b, 'UnicodeCharacter0xf87b', [], 'Display', None],
  [ 0xf87c, 'UnicodeCharacter0xf87c', [], 'Display', None],
  [ 0xf87d, 'UnicodeCharacter0xf87d', [], 'Display', None],
  [ 0xf87e, 'UnicodeCharacter0xf87e', [], 'Display', None],
  [ 0xf87f, 'UnicodeCharacter0xf87f', [], 'Display', None],
  [ 0xf880, 'UnicodeCharacter0xf880', [], 'Display', None],
  [ 0xf881, 'UnicodeCharacter0xf881', [], 'Display', None],
  [ 0xf882, 'UnicodeCharacter0xf882', [], 'Display', None],
  [ 0xf883, 'UnicodeCharacter0xf883', [], 'Display', None],
  [ 0xf884, 'UnicodeCharacter0xf884', [], 'Display', None],
  [ 0xf885, 'UnicodeCharacter0xf885', [], 'Display', None],
  [ 0xf886, 'UnicodeCharacter0xf886', [], 'Display', None],
  [ 0xf887, 'UnicodeCharacter0xf887', [], 'Display', None],
  [ 0xf888, 'UnicodeCharacter0xf888', [], 'Display', None],
  [ 0xf889, 'UnicodeCharacter0xf889', [], 'Display', None],
  [ 0xf88a, 'UnicodeCharacter0xf88a', [], 'Display', None],
  [ 0xf88b, 'UnicodeCharacter0xf88b', [], 'Display', None],
  [ 0xf88c, 'UnicodeCharacter0xf88c', [], 'Display', None],
  [ 0xf88d, 'UnicodeCharacter0xf88d', [], 'Display', None],
  [ 0xf88e, 'UnicodeCharacter0xf88e', [], 'Display', None],
  [ 0xf88f, 'UnicodeCharacter0xf88f', [], 'Display', None],
  [ 0xf890, 'UnicodeCharacter0xf890', [], 'Display', None],
  [ 0xf891, 'UnicodeCharacter0xf891', [], 'Display', None],
  [ 0xf892, 'UnicodeCharacter0xf892', [], 'Display', None],
  [ 0xf893, 'UnicodeCharacter0xf893', [], 'Display', None],
  [ 0xf894, 'UnicodeCharacter0xf894', [], 'Display', None],
  [ 0xf895, 'UnicodeCharacter0xf895', [], 'Display', None],
  [ 0xf896, 'UnicodeCharacter0xf896', [], 'Display', None],
  [ 0xf897, 'UnicodeCharacter0xf897', [], 'Display', None],
  [ 0xf898, 'UnicodeCharacter0xf898', [], 'Display', None],
  [ 0xf899, 'UnicodeCharacter0xf899', [], 'Display', None],
  [ 0xf89a, 'UnicodeCharacter0xf89a', [], 'Display', None],
  [ 0xf89b, 'UnicodeCharacter0xf89b', [], 'Display', None],
  [ 0xf89c, 'UnicodeCharacter0xf89c', [], 'Display', None],
  [ 0xf89d, 'UnicodeCharacter0xf89d', [], 'Display', None],
  [ 0xf89e, 'UnicodeCharacter0xf89e', [], 'Display', None],
  [ 0xf89f, 'UnicodeCharacter0xf89f', [], 'Display', None],
  [ 0xf8a0, 'UnicodeCharacter0xf8a0', [], 'Display', None],
  [ 0xf8a1, 'UnicodeCharacter0xf8a1', [], 'Display', None],
  [ 0xf8a2, 'UnicodeCharacter0xf8a2', [], 'Display', None],
  [ 0xf8a3, 'UnicodeCharacter0xf8a3', [], 'Display', None],
  [ 0xf8a4, 'UnicodeCharacter0xf8a4', [], 'Display', None],
  [ 0xf8a5, 'UnicodeCharacter0xf8a5', [], 'Display', None],
  [ 0xf8a6, 'UnicodeCharacter0xf8a6', [], 'Display', None],
  [ 0xf8a7, 'UnicodeCharacter0xf8a7', [], 'Display', None],
  [ 0xf8a8, 'UnicodeCharacter0xf8a8', [], 'Display', None],
  [ 0xf8a9, 'UnicodeCharacter0xf8a9', [], 'Display', None],
  [ 0xf8aa, 'UnicodeCharacter0xf8aa', [], 'Display', None],
  [ 0xf8ab, 'UnicodeCharacter0xf8ab', [], 'Display', None],
  [ 0xf8ac, 'UnicodeCharacter0xf8ac', [], 'Display', None],
  [ 0xf8ad, 'UnicodeCharacter0xf8ad', [], 'Display', None],
  [ 0xf8ae, 'UnicodeCharacter0xf8ae', [], 'Display', None],
  [ 0xf8af, 'UnicodeCharacter0xf8af', [], 'Display', None],
  [ 0xf8b0, 'UnicodeCharacter0xf8b0', [], 'Display', None],
  [ 0xf8b1, 'UnicodeCharacter0xf8b1', [], 'Display', None],
  [ 0xf8b2, 'UnicodeCharacter0xf8b2', [], 'Display', None],
  [ 0xf8b3, 'UnicodeCharacter0xf8b3', [], 'Display', None],
  [ 0xf8b4, 'UnicodeCharacter0xf8b4', [], 'Display', None],
  [ 0xf8b5, 'UnicodeCharacter0xf8b5', [], 'Display', None],
  [ 0xf8b6, 'UnicodeCharacter0xf8b6', [], 'Display', None],
  [ 0xf8b7, 'UnicodeCharacter0xf8b7', [], 'Display', None],
  [ 0xf8b8, 'UnicodeCharacter0xf8b8', [], 'Display', None],
  [ 0xf8b9, 'UnicodeCharacter0xf8b9', [], 'Display', None],
  [ 0xf8ba, 'UnicodeCharacter0xf8ba', [], 'Display', None],
  [ 0xf8bb, 'UnicodeCharacter0xf8bb', [], 'Display', None],
  [ 0xf8bc, 'UnicodeCharacter0xf8bc', [], 'Display', None],
  [ 0xf8bd, 'UnicodeCharacter0xf8bd', [], 'Display', None],
  [ 0xf8be, 'UnicodeCharacter0xf8be', [], 'Display', None],
  [ 0xf8bf, 'UnicodeCharacter0xf8bf', [], 'Display', None],
  [ 0xf8c0, 'UnicodeCharacter0xf8c0', [], 'Display', None],
  [ 0xf8c1, 'UnicodeCharacter0xf8c1', [], 'Display', None],
  [ 0xf8c2, 'UnicodeCharacter0xf8c2', [], 'Display', None],
  [ 0xf8c3, 'UnicodeCharacter0xf8c3', [], 'Display', None],
  [ 0xf8c4, 'UnicodeCharacter0xf8c4', [], 'Display', None],
  [ 0xf8c5, 'UnicodeCharacter0xf8c5', [], 'Display', None],
  [ 0xf8c6, 'UnicodeCharacter0xf8c6', [], 'Display', None],
  [ 0xf8c7, 'UnicodeCharacter0xf8c7', [], 'Display', None],
  [ 0xf8c8, 'UnicodeCharacter0xf8c8', [], 'Display', None],
  [ 0xf8c9, 'UnicodeCharacter0xf8c9', [], 'Display', None],
  [ 0xf8ca, 'UnicodeCharacter0xf8ca', [], 'Display', None],
  [ 0xf8cb, 'UnicodeCharacter0xf8cb', [], 'Display', None],
  [ 0xf8cc, 'UnicodeCharacter0xf8cc', [], 'Display', None],
  [ 0xf8cd, 'UnicodeCharacter0xf8cd', [], 'Display', None],
  [ 0xf8ce, 'UnicodeCharacter0xf8ce', [], 'Display', None],
  [ 0xf8cf, 'UnicodeCharacter0xf8cf', [], 'Display', None],
  [ 0xf8d0, 'UnicodeCharacter0xf8d0', [], 'Display', None],
  [ 0xf8d1, 'UnicodeCharacter0xf8d1', [], 'Display', None],
  [ 0xf8d2, 'UnicodeCharacter0xf8d2', [], 'Display', None],
  [ 0xf8d3, 'UnicodeCharacter0xf8d3', [], 'Display', None],
  [ 0xf8d4, 'UnicodeCharacter0xf8d4', [], 'Display', None],
  [ 0xf8d5, 'UnicodeCharacter0xf8d5', [], 'Display', None],
  [ 0xf8d6, 'UnicodeCharacter0xf8d6', [], 'Display', None],
  [ 0xf8d7, 'UnicodeCharacter0xf8d7', [], 'Display', None],
  [ 0xf8d8, 'UnicodeCharacter0xf8d8', [], 'Display', None],
  [ 0xf8d9, 'UnicodeCharacter0xf8d9', [], 'Display', None],
  [ 0xf8da, 'UnicodeCharacter0xf8da', [], 'Display', None],
  [ 0xf8db, 'UnicodeCharacter0xf8db', [], 'Display', None],
  [ 0xf8dc, 'UnicodeCharacter0xf8dc', [], 'Display', None],
  [ 0xf8dd, 'UnicodeCharacter0xf8dd', [], 'Display', None],
  [ 0xf8de, 'UnicodeCharacter0xf8de', [], 'Display', None],
  [ 0xf8df, 'UnicodeCharacter0xf8df', [], 'Display', None],
  [ 0xf8e0, 'UnicodeCharacter0xf8e0', [], 'Display', None],
  [ 0xf8e1, 'UnicodeCharacter0xf8e1', [], 'Display', None],
  [ 0xf8e2, 'UnicodeCharacter0xf8e2', [], 'Display', None],
  [ 0xf8e3, 'UnicodeCharacter0xf8e3', [], 'Display', None],
  [ 0xf8e4, 'UnicodeCharacter0xf8e4', [], 'Display', None],
  [ 0xf8e5, 'UnicodeCharacter0xf8e5', [], 'Display', None],
  [ 0xf8e6, 'UnicodeCharacter0xf8e6', [], 'Display', None],
  [ 0xf8e7, 'UnicodeCharacter0xf8e7', [], 'Display', None],
  [ 0xf8e8, 'UnicodeCharacter0xf8e8', [], 'Display', None],
  [ 0xf8e9, 'UnicodeCharacter0xf8e9', [], 'Display', None],
  [ 0xf8ea, 'UnicodeCharacter0xf8ea', [], 'Display', None],
  [ 0xf8eb, 'UnicodeCharacter0xf8eb', [], 'Display', None],
  [ 0xf8ec, 'UnicodeCharacter0xf8ec', [], 'Display', None],
  [ 0xf8ed, 'UnicodeCharacter0xf8ed', [], 'Display', None],
  [ 0xf8ee, 'UnicodeCharacter0xf8ee', [], 'Display', None],
  [ 0xf8ef, 'UnicodeCharacter0xf8ef', [], 'Display', None],
  [ 0xf8f0, 'UnicodeCharacter0xf8f0', [], 'Display', None],
  [ 0xf8f1, 'UnicodeCharacter0xf8f1', [], 'Display', None],
  [ 0xf8f2, 'UnicodeCharacter0xf8f2', [], 'Display', None],
  [ 0xf8f3, 'UnicodeCharacter0xf8f3', [], 'Display', None],
  [ 0xf8f4, 'UnicodeCharacter0xf8f4', [], 'Display', None],
  [ 0xf8f5, 'UnicodeCharacter0xf8f5', [], 'Display', None],
  [ 0xf8f6, 'UnicodeCharacter0xf8f6', [], 'Display', None],
  [ 0xf8f7, 'UnicodeCharacter0xf8f7', [], 'Display', None],
  [ 0xf8f8, 'UnicodeCharacter0xf8f8', [], 'Display', None],
  [ 0xf8f9, 'UnicodeCharacter0xf8f9', [], 'Display', None],
  [ 0xf8fa, 'UnicodeCharacter0xf8fa', [], 'Display', None],
  [ 0xf8fb, 'UnicodeCharacter0xf8fb', [], 'Display', None],
  [ 0xf8fc, 'UnicodeCharacter0xf8fc', [], 'Display', None],
  [ 0xf8fd, 'UnicodeCharacter0xf8fd', [], 'Display', None],
  [ 0xf8fe, 'UnicodeCharacter0xf8fe', [], 'Display', None],
  [ 0xf8ff, 'UnicodeCharacter0xf8ff', [], 'Display', None],
  [ 0xf900, 'CjkCompatibilityIdeographf900', [], 'Display', None],
  [ 0xf901, 'CjkCompatibilityIdeographf901', [], 'Display', None],
  [ 0xf902, 'CjkCompatibilityIdeographf902', [], 'Display', None],
  [ 0xf903, 'CjkCompatibilityIdeographf903', [], 'Display', None],
  [ 0xf904, 'CjkCompatibilityIdeographf904', [], 'Display', None],
  [ 0xf905, 'CjkCompatibilityIdeographf905', [], 'Display', None],
  [ 0xf906, 'CjkCompatibilityIdeographf906', [], 'Display', None],
  [ 0xf907, 'CjkCompatibilityIdeographf907', [], 'Display', None],
  [ 0xf908, 'CjkCompatibilityIdeographf908', [], 'Display', None],
  [ 0xf909, 'CjkCompatibilityIdeographf909', [], 'Display', None],
  [ 0xf90a, 'CjkCompatibilityIdeographf90a', [], 'Display', None],
  [ 0xf90b, 'CjkCompatibilityIdeographf90b', [], 'Display', None],
  [ 0xf90c, 'CjkCompatibilityIdeographf90c', [], 'Display', None],
  [ 0xf90d, 'CjkCompatibilityIdeographf90d', [], 'Display', None],
  [ 0xf90e, 'CjkCompatibilityIdeographf90e', [], 'Display', None],
  [ 0xf90f, 'CjkCompatibilityIdeographf90f', [], 'Display', None],
  [ 0xf910, 'CjkCompatibilityIdeographf910', [], 'Display', None],
  [ 0xf911, 'CjkCompatibilityIdeographf911', [], 'Display', None],
  [ 0xf912, 'CjkCompatibilityIdeographf912', [], 'Display', None],
  [ 0xf913, 'CjkCompatibilityIdeographf913', [], 'Display', None],
  [ 0xf914, 'CjkCompatibilityIdeographf914', [], 'Display', None],
  [ 0xf915, 'CjkCompatibilityIdeographf915', [], 'Display', None],
  [ 0xf916, 'CjkCompatibilityIdeographf916', [], 'Display', None],
  [ 0xf917, 'CjkCompatibilityIdeographf917', [], 'Display', None],
  [ 0xf918, 'CjkCompatibilityIdeographf918', [], 'Display', None],
  [ 0xf919, 'CjkCompatibilityIdeographf919', [], 'Display', None],
  [ 0xf91a, 'CjkCompatibilityIdeographf91a', [], 'Display', None],
  [ 0xf91b, 'CjkCompatibilityIdeographf91b', [], 'Display', None],
  [ 0xf91c, 'CjkCompatibilityIdeographf91c', [], 'Display', None],
  [ 0xf91d, 'CjkCompatibilityIdeographf91d', [], 'Display', None],
  [ 0xf91e, 'CjkCompatibilityIdeographf91e', [], 'Display', None],
  [ 0xf91f, 'CjkCompatibilityIdeographf91f', [], 'Display', None],
  [ 0xf920, 'CjkCompatibilityIdeographf920', [], 'Display', None],
  [ 0xf921, 'CjkCompatibilityIdeographf921', [], 'Display', None],
  [ 0xf922, 'CjkCompatibilityIdeographf922', [], 'Display', None],
  [ 0xf923, 'CjkCompatibilityIdeographf923', [], 'Display', None],
  [ 0xf924, 'CjkCompatibilityIdeographf924', [], 'Display', None],
  [ 0xf925, 'CjkCompatibilityIdeographf925', [], 'Display', None],
  [ 0xf926, 'CjkCompatibilityIdeographf926', [], 'Display', None],
  [ 0xf927, 'CjkCompatibilityIdeographf927', [], 'Display', None],
  [ 0xf928, 'CjkCompatibilityIdeographf928', [], 'Display', None],
  [ 0xf929, 'CjkCompatibilityIdeographf929', [], 'Display', None],
  [ 0xf92a, 'CjkCompatibilityIdeographf92a', [], 'Display', None],
  [ 0xf92b, 'CjkCompatibilityIdeographf92b', [], 'Display', None],
  [ 0xf92c, 'CjkCompatibilityIdeographf92c', [], 'Display', None],
  [ 0xf92d, 'CjkCompatibilityIdeographf92d', [], 'Display', None],
  [ 0xf92e, 'CjkCompatibilityIdeographf92e', [], 'Display', None],
  [ 0xf92f, 'CjkCompatibilityIdeographf92f', [], 'Display', None],
  [ 0xf930, 'CjkCompatibilityIdeographf930', [], 'Display', None],
  [ 0xf931, 'CjkCompatibilityIdeographf931', [], 'Display', None],
  [ 0xf932, 'CjkCompatibilityIdeographf932', [], 'Display', None],
  [ 0xf933, 'CjkCompatibilityIdeographf933', [], 'Display', None],
  [ 0xf934, 'CjkCompatibilityIdeographf934', [], 'Display', None],
  [ 0xf935, 'CjkCompatibilityIdeographf935', [], 'Display', None],
  [ 0xf936, 'CjkCompatibilityIdeographf936', [], 'Display', None],
  [ 0xf937, 'CjkCompatibilityIdeographf937', [], 'Display', None],
  [ 0xf938, 'CjkCompatibilityIdeographf938', [], 'Display', None],
  [ 0xf939, 'CjkCompatibilityIdeographf939', [], 'Display', None],
  [ 0xf93a, 'CjkCompatibilityIdeographf93a', [], 'Display', None],
  [ 0xf93b, 'CjkCompatibilityIdeographf93b', [], 'Display', None],
  [ 0xf93c, 'CjkCompatibilityIdeographf93c', [], 'Display', None],
  [ 0xf93d, 'CjkCompatibilityIdeographf93d', [], 'Display', None],
  [ 0xf93e, 'CjkCompatibilityIdeographf93e', [], 'Display', None],
  [ 0xf93f, 'CjkCompatibilityIdeographf93f', [], 'Display', None],
  [ 0xf940, 'CjkCompatibilityIdeographf940', [], 'Display', None],
  [ 0xf941, 'CjkCompatibilityIdeographf941', [], 'Display', None],
  [ 0xf942, 'CjkCompatibilityIdeographf942', [], 'Display', None],
  [ 0xf943, 'CjkCompatibilityIdeographf943', [], 'Display', None],
  [ 0xf944, 'CjkCompatibilityIdeographf944', [], 'Display', None],
  [ 0xf945, 'CjkCompatibilityIdeographf945', [], 'Display', None],
  [ 0xf946, 'CjkCompatibilityIdeographf946', [], 'Display', None],
  [ 0xf947, 'CjkCompatibilityIdeographf947', [], 'Display', None],
  [ 0xf948, 'CjkCompatibilityIdeographf948', [], 'Display', None],
  [ 0xf949, 'CjkCompatibilityIdeographf949', [], 'Display', None],
  [ 0xf94a, 'CjkCompatibilityIdeographf94a', [], 'Display', None],
  [ 0xf94b, 'CjkCompatibilityIdeographf94b', [], 'Display', None],
  [ 0xf94c, 'CjkCompatibilityIdeographf94c', [], 'Display', None],
  [ 0xf94d, 'CjkCompatibilityIdeographf94d', [], 'Display', None],
  [ 0xf94e, 'CjkCompatibilityIdeographf94e', [], 'Display', None],
  [ 0xf94f, 'CjkCompatibilityIdeographf94f', [], 'Display', None],
  [ 0xf950, 'CjkCompatibilityIdeographf950', [], 'Display', None],
  [ 0xf951, 'CjkCompatibilityIdeographf951', [], 'Display', None],
  [ 0xf952, 'CjkCompatibilityIdeographf952', [], 'Display', None],
  [ 0xf953, 'CjkCompatibilityIdeographf953', [], 'Display', None],
  [ 0xf954, 'CjkCompatibilityIdeographf954', [], 'Display', None],
  [ 0xf955, 'CjkCompatibilityIdeographf955', [], 'Display', None],
  [ 0xf956, 'CjkCompatibilityIdeographf956', [], 'Display', None],
  [ 0xf957, 'CjkCompatibilityIdeographf957', [], 'Display', None],
  [ 0xf958, 'CjkCompatibilityIdeographf958', [], 'Display', None],
  [ 0xf959, 'CjkCompatibilityIdeographf959', [], 'Display', None],
  [ 0xf95a, 'CjkCompatibilityIdeographf95a', [], 'Display', None],
  [ 0xf95b, 'CjkCompatibilityIdeographf95b', [], 'Display', None],
  [ 0xf95c, 'CjkCompatibilityIdeographf95c', [], 'Display', None],
  [ 0xf95d, 'CjkCompatibilityIdeographf95d', [], 'Display', None],
  [ 0xf95e, 'CjkCompatibilityIdeographf95e', [], 'Display', None],
  [ 0xf95f, 'CjkCompatibilityIdeographf95f', [], 'Display', None],
  [ 0xf960, 'CjkCompatibilityIdeographf960', [], 'Display', None],
  [ 0xf961, 'CjkCompatibilityIdeographf961', [], 'Display', None],
  [ 0xf962, 'CjkCompatibilityIdeographf962', [], 'Display', None],
  [ 0xf963, 'CjkCompatibilityIdeographf963', [], 'Display', None],
  [ 0xf964, 'CjkCompatibilityIdeographf964', [], 'Display', None],
  [ 0xf965, 'CjkCompatibilityIdeographf965', [], 'Display', None],
  [ 0xf966, 'CjkCompatibilityIdeographf966', [], 'Display', None],
  [ 0xf967, 'CjkCompatibilityIdeographf967', [], 'Display', None],
  [ 0xf968, 'CjkCompatibilityIdeographf968', [], 'Display', None],
  [ 0xf969, 'CjkCompatibilityIdeographf969', [], 'Display', None],
  [ 0xf96a, 'CjkCompatibilityIdeographf96a', [], 'Display', None],
  [ 0xf96b, 'CjkCompatibilityIdeographf96b', [], 'Display', None],
  [ 0xf96c, 'CjkCompatibilityIdeographf96c', [], 'Display', None],
  [ 0xf96d, 'CjkCompatibilityIdeographf96d', [], 'Display', None],
  [ 0xf96e, 'CjkCompatibilityIdeographf96e', [], 'Display', None],
  [ 0xf96f, 'CjkCompatibilityIdeographf96f', [], 'Display', None],
  [ 0xf970, 'CjkCompatibilityIdeographf970', [], 'Display', None],
  [ 0xf971, 'CjkCompatibilityIdeographf971', [], 'Display', None],
  [ 0xf972, 'CjkCompatibilityIdeographf972', [], 'Display', None],
  [ 0xf973, 'CjkCompatibilityIdeographf973', [], 'Display', None],
  [ 0xf974, 'CjkCompatibilityIdeographf974', [], 'Display', None],
  [ 0xf975, 'CjkCompatibilityIdeographf975', [], 'Display', None],
  [ 0xf976, 'CjkCompatibilityIdeographf976', [], 'Display', None],
  [ 0xf977, 'CjkCompatibilityIdeographf977', [], 'Display', None],
  [ 0xf978, 'CjkCompatibilityIdeographf978', [], 'Display', None],
  [ 0xf979, 'CjkCompatibilityIdeographf979', [], 'Display', None],
  [ 0xf97a, 'CjkCompatibilityIdeographf97a', [], 'Display', None],
  [ 0xf97b, 'CjkCompatibilityIdeographf97b', [], 'Display', None],
  [ 0xf97c, 'CjkCompatibilityIdeographf97c', [], 'Display', None],
  [ 0xf97d, 'CjkCompatibilityIdeographf97d', [], 'Display', None],
  [ 0xf97e, 'CjkCompatibilityIdeographf97e', [], 'Display', None],
  [ 0xf97f, 'CjkCompatibilityIdeographf97f', [], 'Display', None],
  [ 0xf980, 'CjkCompatibilityIdeographf980', [], 'Display', None],
  [ 0xf981, 'CjkCompatibilityIdeographf981', [], 'Display', None],
  [ 0xf982, 'CjkCompatibilityIdeographf982', [], 'Display', None],
  [ 0xf983, 'CjkCompatibilityIdeographf983', [], 'Display', None],
  [ 0xf984, 'CjkCompatibilityIdeographf984', [], 'Display', None],
  [ 0xf985, 'CjkCompatibilityIdeographf985', [], 'Display', None],
  [ 0xf986, 'CjkCompatibilityIdeographf986', [], 'Display', None],
  [ 0xf987, 'CjkCompatibilityIdeographf987', [], 'Display', None],
  [ 0xf988, 'CjkCompatibilityIdeographf988', [], 'Display', None],
  [ 0xf989, 'CjkCompatibilityIdeographf989', [], 'Display', None],
  [ 0xf98a, 'CjkCompatibilityIdeographf98a', [], 'Display', None],
  [ 0xf98b, 'CjkCompatibilityIdeographf98b', [], 'Display', None],
  [ 0xf98c, 'CjkCompatibilityIdeographf98c', [], 'Display', None],
  [ 0xf98d, 'CjkCompatibilityIdeographf98d', [], 'Display', None],
  [ 0xf98e, 'CjkCompatibilityIdeographf98e', [], 'Display', None],
  [ 0xf98f, 'CjkCompatibilityIdeographf98f', [], 'Display', None],
  [ 0xf990, 'CjkCompatibilityIdeographf990', [], 'Display', None],
  [ 0xf991, 'CjkCompatibilityIdeographf991', [], 'Display', None],
  [ 0xf992, 'CjkCompatibilityIdeographf992', [], 'Display', None],
  [ 0xf993, 'CjkCompatibilityIdeographf993', [], 'Display', None],
  [ 0xf994, 'CjkCompatibilityIdeographf994', [], 'Display', None],
  [ 0xf995, 'CjkCompatibilityIdeographf995', [], 'Display', None],
  [ 0xf996, 'CjkCompatibilityIdeographf996', [], 'Display', None],
  [ 0xf997, 'CjkCompatibilityIdeographf997', [], 'Display', None],
  [ 0xf998, 'CjkCompatibilityIdeographf998', [], 'Display', None],
  [ 0xf999, 'CjkCompatibilityIdeographf999', [], 'Display', None],
  [ 0xf99a, 'CjkCompatibilityIdeographf99a', [], 'Display', None],
  [ 0xf99b, 'CjkCompatibilityIdeographf99b', [], 'Display', None],
  [ 0xf99c, 'CjkCompatibilityIdeographf99c', [], 'Display', None],
  [ 0xf99d, 'CjkCompatibilityIdeographf99d', [], 'Display', None],
  [ 0xf99e, 'CjkCompatibilityIdeographf99e', [], 'Display', None],
  [ 0xf99f, 'CjkCompatibilityIdeographf99f', [], 'Display', None],
  [ 0xf9a0, 'CjkCompatibilityIdeographf9a0', [], 'Display', None],
  [ 0xf9a1, 'CjkCompatibilityIdeographf9a1', [], 'Display', None],
  [ 0xf9a2, 'CjkCompatibilityIdeographf9a2', [], 'Display', None],
  [ 0xf9a3, 'CjkCompatibilityIdeographf9a3', [], 'Display', None],
  [ 0xf9a4, 'CjkCompatibilityIdeographf9a4', [], 'Display', None],
  [ 0xf9a5, 'CjkCompatibilityIdeographf9a5', [], 'Display', None],
  [ 0xf9a6, 'CjkCompatibilityIdeographf9a6', [], 'Display', None],
  [ 0xf9a7, 'CjkCompatibilityIdeographf9a7', [], 'Display', None],
  [ 0xf9a8, 'CjkCompatibilityIdeographf9a8', [], 'Display', None],
  [ 0xf9a9, 'CjkCompatibilityIdeographf9a9', [], 'Display', None],
  [ 0xf9aa, 'CjkCompatibilityIdeographf9aa', [], 'Display', None],
  [ 0xf9ab, 'CjkCompatibilityIdeographf9ab', [], 'Display', None],
  [ 0xf9ac, 'CjkCompatibilityIdeographf9ac', [], 'Display', None],
  [ 0xf9ad, 'CjkCompatibilityIdeographf9ad', [], 'Display', None],
  [ 0xf9ae, 'CjkCompatibilityIdeographf9ae', [], 'Display', None],
  [ 0xf9af, 'CjkCompatibilityIdeographf9af', [], 'Display', None],
  [ 0xf9b0, 'CjkCompatibilityIdeographf9b0', [], 'Display', None],
  [ 0xf9b1, 'CjkCompatibilityIdeographf9b1', [], 'Display', None],
  [ 0xf9b2, 'CjkCompatibilityIdeographf9b2', [], 'Display', None],
  [ 0xf9b3, 'CjkCompatibilityIdeographf9b3', [], 'Display', None],
  [ 0xf9b4, 'CjkCompatibilityIdeographf9b4', [], 'Display', None],
  [ 0xf9b5, 'CjkCompatibilityIdeographf9b5', [], 'Display', None],
  [ 0xf9b6, 'CjkCompatibilityIdeographf9b6', [], 'Display', None],
  [ 0xf9b7, 'CjkCompatibilityIdeographf9b7', [], 'Display', None],
  [ 0xf9b8, 'CjkCompatibilityIdeographf9b8', [], 'Display', None],
  [ 0xf9b9, 'CjkCompatibilityIdeographf9b9', [], 'Display', None],
  [ 0xf9ba, 'CjkCompatibilityIdeographf9ba', [], 'Display', None],
  [ 0xf9bb, 'CjkCompatibilityIdeographf9bb', [], 'Display', None],
  [ 0xf9bc, 'CjkCompatibilityIdeographf9bc', [], 'Display', None],
  [ 0xf9bd, 'CjkCompatibilityIdeographf9bd', [], 'Display', None],
  [ 0xf9be, 'CjkCompatibilityIdeographf9be', [], 'Display', None],
  [ 0xf9bf, 'CjkCompatibilityIdeographf9bf', [], 'Display', None],
  [ 0xf9c0, 'CjkCompatibilityIdeographf9c0', [], 'Display', None],
  [ 0xf9c1, 'CjkCompatibilityIdeographf9c1', [], 'Display', None],
  [ 0xf9c2, 'CjkCompatibilityIdeographf9c2', [], 'Display', None],
  [ 0xf9c3, 'CjkCompatibilityIdeographf9c3', [], 'Display', None],
  [ 0xf9c4, 'CjkCompatibilityIdeographf9c4', [], 'Display', None],
  [ 0xf9c5, 'CjkCompatibilityIdeographf9c5', [], 'Display', None],
  [ 0xf9c6, 'CjkCompatibilityIdeographf9c6', [], 'Display', None],
  [ 0xf9c7, 'CjkCompatibilityIdeographf9c7', [], 'Display', None],
  [ 0xf9c8, 'CjkCompatibilityIdeographf9c8', [], 'Display', None],
  [ 0xf9c9, 'CjkCompatibilityIdeographf9c9', [], 'Display', None],
  [ 0xf9ca, 'CjkCompatibilityIdeographf9ca', [], 'Display', None],
  [ 0xf9cb, 'CjkCompatibilityIdeographf9cb', [], 'Display', None],
  [ 0xf9cc, 'CjkCompatibilityIdeographf9cc', [], 'Display', None],
  [ 0xf9cd, 'CjkCompatibilityIdeographf9cd', [], 'Display', None],
  [ 0xf9ce, 'CjkCompatibilityIdeographf9ce', [], 'Display', None],
  [ 0xf9cf, 'CjkCompatibilityIdeographf9cf', [], 'Display', None],
  [ 0xf9d0, 'CjkCompatibilityIdeographf9d0', [], 'Display', None],
  [ 0xf9d1, 'CjkCompatibilityIdeographf9d1', [], 'Display', None],
  [ 0xf9d2, 'CjkCompatibilityIdeographf9d2', [], 'Display', None],
  [ 0xf9d3, 'CjkCompatibilityIdeographf9d3', [], 'Display', None],
  [ 0xf9d4, 'CjkCompatibilityIdeographf9d4', [], 'Display', None],
  [ 0xf9d5, 'CjkCompatibilityIdeographf9d5', [], 'Display', None],
  [ 0xf9d6, 'CjkCompatibilityIdeographf9d6', [], 'Display', None],
  [ 0xf9d7, 'CjkCompatibilityIdeographf9d7', [], 'Display', None],
  [ 0xf9d8, 'CjkCompatibilityIdeographf9d8', [], 'Display', None],
  [ 0xf9d9, 'CjkCompatibilityIdeographf9d9', [], 'Display', None],
  [ 0xf9da, 'CjkCompatibilityIdeographf9da', [], 'Display', None],
  [ 0xf9db, 'CjkCompatibilityIdeographf9db', [], 'Display', None],
  [ 0xf9dc, 'CjkCompatibilityIdeographf9dc', [], 'Display', None],
  [ 0xf9dd, 'CjkCompatibilityIdeographf9dd', [], 'Display', None],
  [ 0xf9de, 'CjkCompatibilityIdeographf9de', [], 'Display', None],
  [ 0xf9df, 'CjkCompatibilityIdeographf9df', [], 'Display', None],
  [ 0xf9e0, 'CjkCompatibilityIdeographf9e0', [], 'Display', None],
  [ 0xf9e1, 'CjkCompatibilityIdeographf9e1', [], 'Display', None],
  [ 0xf9e2, 'CjkCompatibilityIdeographf9e2', [], 'Display', None],
  [ 0xf9e3, 'CjkCompatibilityIdeographf9e3', [], 'Display', None],
  [ 0xf9e4, 'CjkCompatibilityIdeographf9e4', [], 'Display', None],
  [ 0xf9e5, 'CjkCompatibilityIdeographf9e5', [], 'Display', None],
  [ 0xf9e6, 'CjkCompatibilityIdeographf9e6', [], 'Display', None],
  [ 0xf9e7, 'CjkCompatibilityIdeographf9e7', [], 'Display', None],
  [ 0xf9e8, 'CjkCompatibilityIdeographf9e8', [], 'Display', None],
  [ 0xf9e9, 'CjkCompatibilityIdeographf9e9', [], 'Display', None],
  [ 0xf9ea, 'CjkCompatibilityIdeographf9ea', [], 'Display', None],
  [ 0xf9eb, 'CjkCompatibilityIdeographf9eb', [], 'Display', None],
  [ 0xf9ec, 'CjkCompatibilityIdeographf9ec', [], 'Display', None],
  [ 0xf9ed, 'CjkCompatibilityIdeographf9ed', [], 'Display', None],
  [ 0xf9ee, 'CjkCompatibilityIdeographf9ee', [], 'Display', None],
  [ 0xf9ef, 'CjkCompatibilityIdeographf9ef', [], 'Display', None],
  [ 0xf9f0, 'CjkCompatibilityIdeographf9f0', [], 'Display', None],
  [ 0xf9f1, 'CjkCompatibilityIdeographf9f1', [], 'Display', None],
  [ 0xf9f2, 'CjkCompatibilityIdeographf9f2', [], 'Display', None],
  [ 0xf9f3, 'CjkCompatibilityIdeographf9f3', [], 'Display', None],
  [ 0xf9f4, 'CjkCompatibilityIdeographf9f4', [], 'Display', None],
  [ 0xf9f5, 'CjkCompatibilityIdeographf9f5', [], 'Display', None],
  [ 0xf9f6, 'CjkCompatibilityIdeographf9f6', [], 'Display', None],
  [ 0xf9f7, 'CjkCompatibilityIdeographf9f7', [], 'Display', None],
  [ 0xf9f8, 'CjkCompatibilityIdeographf9f8', [], 'Display', None],
  [ 0xf9f9, 'CjkCompatibilityIdeographf9f9', [], 'Display', None],
  [ 0xf9fa, 'CjkCompatibilityIdeographf9fa', [], 'Display', None],
  [ 0xf9fb, 'CjkCompatibilityIdeographf9fb', [], 'Display', None],
  [ 0xf9fc, 'CjkCompatibilityIdeographf9fc', [], 'Display', None],
  [ 0xf9fd, 'CjkCompatibilityIdeographf9fd', [], 'Display', None],
  [ 0xf9fe, 'CjkCompatibilityIdeographf9fe', [], 'Display', None],
  [ 0xf9ff, 'CjkCompatibilityIdeographf9ff', [], 'Display', None],
  [ 0xfa00, 'CjkCompatibilityIdeographfa00', [], 'Display', None],
  [ 0xfa01, 'CjkCompatibilityIdeographfa01', [], 'Display', None],
  [ 0xfa02, 'CjkCompatibilityIdeographfa02', [], 'Display', None],
  [ 0xfa03, 'CjkCompatibilityIdeographfa03', [], 'Display', None],
  [ 0xfa04, 'CjkCompatibilityIdeographfa04', [], 'Display', None],
  [ 0xfa05, 'CjkCompatibilityIdeographfa05', [], 'Display', None],
  [ 0xfa06, 'CjkCompatibilityIdeographfa06', [], 'Display', None],
  [ 0xfa07, 'CjkCompatibilityIdeographfa07', [], 'Display', None],
  [ 0xfa08, 'CjkCompatibilityIdeographfa08', [], 'Display', None],
  [ 0xfa09, 'CjkCompatibilityIdeographfa09', [], 'Display', None],
  [ 0xfa0a, 'CjkCompatibilityIdeographfa0a', [], 'Display', None],
  [ 0xfa0b, 'CjkCompatibilityIdeographfa0b', [], 'Display', None],
  [ 0xfa0c, 'CjkCompatibilityIdeographfa0c', [], 'Display', None],
  [ 0xfa0d, 'CjkCompatibilityIdeographfa0d', [], 'Display', None],
  [ 0xfa0e, 'CjkCompatibilityIdeographfa0e', [], 'Display', None],
  [ 0xfa0f, 'CjkCompatibilityIdeographfa0f', [], 'Display', None],
  [ 0xfa10, 'CjkCompatibilityIdeographfa10', [], 'Display', None],
  [ 0xfa11, 'CjkCompatibilityIdeographfa11', [], 'Display', None],
  [ 0xfa12, 'CjkCompatibilityIdeographfa12', [], 'Display', None],
  [ 0xfa13, 'CjkCompatibilityIdeographfa13', [], 'Display', None],
  [ 0xfa14, 'CjkCompatibilityIdeographfa14', [], 'Display', None],
  [ 0xfa15, 'CjkCompatibilityIdeographfa15', [], 'Display', None],
  [ 0xfa16, 'CjkCompatibilityIdeographfa16', [], 'Display', None],
  [ 0xfa17, 'CjkCompatibilityIdeographfa17', [], 'Display', None],
  [ 0xfa18, 'CjkCompatibilityIdeographfa18', [], 'Display', None],
  [ 0xfa19, 'CjkCompatibilityIdeographfa19', [], 'Display', None],
  [ 0xfa1a, 'CjkCompatibilityIdeographfa1a', [], 'Display', None],
  [ 0xfa1b, 'CjkCompatibilityIdeographfa1b', [], 'Display', None],
  [ 0xfa1c, 'CjkCompatibilityIdeographfa1c', [], 'Display', None],
  [ 0xfa1d, 'CjkCompatibilityIdeographfa1d', [], 'Display', None],
  [ 0xfa1e, 'CjkCompatibilityIdeographfa1e', [], 'Display', None],
  [ 0xfa1f, 'CjkCompatibilityIdeographfa1f', [], 'Display', None],
  [ 0xfa20, 'CjkCompatibilityIdeographfa20', [], 'Display', None],
  [ 0xfa21, 'CjkCompatibilityIdeographfa21', [], 'Display', None],
  [ 0xfa22, 'CjkCompatibilityIdeographfa22', [], 'Display', None],
  [ 0xfa23, 'CjkCompatibilityIdeographfa23', [], 'Display', None],
  [ 0xfa24, 'CjkCompatibilityIdeographfa24', [], 'Display', None],
  [ 0xfa25, 'CjkCompatibilityIdeographfa25', [], 'Display', None],
  [ 0xfa26, 'CjkCompatibilityIdeographfa26', [], 'Display', None],
  [ 0xfa27, 'CjkCompatibilityIdeographfa27', [], 'Display', None],
  [ 0xfa28, 'CjkCompatibilityIdeographfa28', [], 'Display', None],
  [ 0xfa29, 'CjkCompatibilityIdeographfa29', [], 'Display', None],
  [ 0xfa2a, 'CjkCompatibilityIdeographfa2a', [], 'Display', None],
  [ 0xfa2b, 'CjkCompatibilityIdeographfa2b', [], 'Display', None],
  [ 0xfa2c, 'CjkCompatibilityIdeographfa2c', [], 'Display', None],
  [ 0xfa2d, 'CjkCompatibilityIdeographfa2d', [], 'Display', None],
  [ 0xfa2e, 'UnicodeCharacter0xfa2e', [], 'Display', None],
  [ 0xfa2f, 'UnicodeCharacter0xfa2f', [], 'Display', None],
  [ 0xfa30, 'CjkCompatibilityIdeographfa30', [], 'Display', None],
  [ 0xfa31, 'CjkCompatibilityIdeographfa31', [], 'Display', None],
  [ 0xfa32, 'CjkCompatibilityIdeographfa32', [], 'Display', None],
  [ 0xfa33, 'CjkCompatibilityIdeographfa33', [], 'Display', None],
  [ 0xfa34, 'CjkCompatibilityIdeographfa34', [], 'Display', None],
  [ 0xfa35, 'CjkCompatibilityIdeographfa35', [], 'Display', None],
  [ 0xfa36, 'CjkCompatibilityIdeographfa36', [], 'Display', None],
  [ 0xfa37, 'CjkCompatibilityIdeographfa37', [], 'Display', None],
  [ 0xfa38, 'CjkCompatibilityIdeographfa38', [], 'Display', None],
  [ 0xfa39, 'CjkCompatibilityIdeographfa39', [], 'Display', None],
  [ 0xfa3a, 'CjkCompatibilityIdeographfa3a', [], 'Display', None],
  [ 0xfa3b, 'CjkCompatibilityIdeographfa3b', [], 'Display', None],
  [ 0xfa3c, 'CjkCompatibilityIdeographfa3c', [], 'Display', None],
  [ 0xfa3d, 'CjkCompatibilityIdeographfa3d', [], 'Display', None],
  [ 0xfa3e, 'CjkCompatibilityIdeographfa3e', [], 'Display', None],
  [ 0xfa3f, 'CjkCompatibilityIdeographfa3f', [], 'Display', None],
  [ 0xfa40, 'CjkCompatibilityIdeographfa40', [], 'Display', None],
  [ 0xfa41, 'CjkCompatibilityIdeographfa41', [], 'Display', None],
  [ 0xfa42, 'CjkCompatibilityIdeographfa42', [], 'Display', None],
  [ 0xfa43, 'CjkCompatibilityIdeographfa43', [], 'Display', None],
  [ 0xfa44, 'CjkCompatibilityIdeographfa44', [], 'Display', None],
  [ 0xfa45, 'CjkCompatibilityIdeographfa45', [], 'Display', None],
  [ 0xfa46, 'CjkCompatibilityIdeographfa46', [], 'Display', None],
  [ 0xfa47, 'CjkCompatibilityIdeographfa47', [], 'Display', None],
  [ 0xfa48, 'CjkCompatibilityIdeographfa48', [], 'Display', None],
  [ 0xfa49, 'CjkCompatibilityIdeographfa49', [], 'Display', None],
  [ 0xfa4a, 'CjkCompatibilityIdeographfa4a', [], 'Display', None],
  [ 0xfa4b, 'CjkCompatibilityIdeographfa4b', [], 'Display', None],
  [ 0xfa4c, 'CjkCompatibilityIdeographfa4c', [], 'Display', None],
  [ 0xfa4d, 'CjkCompatibilityIdeographfa4d', [], 'Display', None],
  [ 0xfa4e, 'CjkCompatibilityIdeographfa4e', [], 'Display', None],
  [ 0xfa4f, 'CjkCompatibilityIdeographfa4f', [], 'Display', None],
  [ 0xfa50, 'CjkCompatibilityIdeographfa50', [], 'Display', None],
  [ 0xfa51, 'CjkCompatibilityIdeographfa51', [], 'Display', None],
  [ 0xfa52, 'CjkCompatibilityIdeographfa52', [], 'Display', None],
  [ 0xfa53, 'CjkCompatibilityIdeographfa53', [], 'Display', None],
  [ 0xfa54, 'CjkCompatibilityIdeographfa54', [], 'Display', None],
  [ 0xfa55, 'CjkCompatibilityIdeographfa55', [], 'Display', None],
  [ 0xfa56, 'CjkCompatibilityIdeographfa56', [], 'Display', None],
  [ 0xfa57, 'CjkCompatibilityIdeographfa57', [], 'Display', None],
  [ 0xfa58, 'CjkCompatibilityIdeographfa58', [], 'Display', None],
  [ 0xfa59, 'CjkCompatibilityIdeographfa59', [], 'Display', None],
  [ 0xfa5a, 'CjkCompatibilityIdeographfa5a', [], 'Display', None],
  [ 0xfa5b, 'CjkCompatibilityIdeographfa5b', [], 'Display', None],
  [ 0xfa5c, 'CjkCompatibilityIdeographfa5c', [], 'Display', None],
  [ 0xfa5d, 'CjkCompatibilityIdeographfa5d', [], 'Display', None],
  [ 0xfa5e, 'CjkCompatibilityIdeographfa5e', [], 'Display', None],
  [ 0xfa5f, 'CjkCompatibilityIdeographfa5f', [], 'Display', None],
  [ 0xfa60, 'CjkCompatibilityIdeographfa60', [], 'Display', None],
  [ 0xfa61, 'CjkCompatibilityIdeographfa61', [], 'Display', None],
  [ 0xfa62, 'CjkCompatibilityIdeographfa62', [], 'Display', None],
  [ 0xfa63, 'CjkCompatibilityIdeographfa63', [], 'Display', None],
  [ 0xfa64, 'CjkCompatibilityIdeographfa64', [], 'Display', None],
  [ 0xfa65, 'CjkCompatibilityIdeographfa65', [], 'Display', None],
  [ 0xfa66, 'CjkCompatibilityIdeographfa66', [], 'Display', None],
  [ 0xfa67, 'CjkCompatibilityIdeographfa67', [], 'Display', None],
  [ 0xfa68, 'CjkCompatibilityIdeographfa68', [], 'Display', None],
  [ 0xfa69, 'CjkCompatibilityIdeographfa69', [], 'Display', None],
  [ 0xfa6a, 'CjkCompatibilityIdeographfa6a', [], 'Display', None],
  [ 0xfa6b, 'UnicodeCharacter0xfa6b', [], 'Display', None],
  [ 0xfa6c, 'UnicodeCharacter0xfa6c', [], 'Display', None],
  [ 0xfa6d, 'UnicodeCharacter0xfa6d', [], 'Display', None],
  [ 0xfa6e, 'UnicodeCharacter0xfa6e', [], 'Display', None],
  [ 0xfa6f, 'UnicodeCharacter0xfa6f', [], 'Display', None],
  [ 0xfa70, 'CjkCompatibilityIdeographfa70', [], 'Display', None],
  [ 0xfa71, 'CjkCompatibilityIdeographfa71', [], 'Display', None],
  [ 0xfa72, 'CjkCompatibilityIdeographfa72', [], 'Display', None],
  [ 0xfa73, 'CjkCompatibilityIdeographfa73', [], 'Display', None],
  [ 0xfa74, 'CjkCompatibilityIdeographfa74', [], 'Display', None],
  [ 0xfa75, 'CjkCompatibilityIdeographfa75', [], 'Display', None],
  [ 0xfa76, 'CjkCompatibilityIdeographfa76', [], 'Display', None],
  [ 0xfa77, 'CjkCompatibilityIdeographfa77', [], 'Display', None],
  [ 0xfa78, 'CjkCompatibilityIdeographfa78', [], 'Display', None],
  [ 0xfa79, 'CjkCompatibilityIdeographfa79', [], 'Display', None],
  [ 0xfa7a, 'CjkCompatibilityIdeographfa7a', [], 'Display', None],
  [ 0xfa7b, 'CjkCompatibilityIdeographfa7b', [], 'Display', None],
  [ 0xfa7c, 'CjkCompatibilityIdeographfa7c', [], 'Display', None],
  [ 0xfa7d, 'CjkCompatibilityIdeographfa7d', [], 'Display', None],
  [ 0xfa7e, 'CjkCompatibilityIdeographfa7e', [], 'Display', None],
  [ 0xfa7f, 'CjkCompatibilityIdeographfa7f', [], 'Display', None],
  [ 0xfa80, 'CjkCompatibilityIdeographfa80', [], 'Display', None],
  [ 0xfa81, 'CjkCompatibilityIdeographfa81', [], 'Display', None],
  [ 0xfa82, 'CjkCompatibilityIdeographfa82', [], 'Display', None],
  [ 0xfa83, 'CjkCompatibilityIdeographfa83', [], 'Display', None],
  [ 0xfa84, 'CjkCompatibilityIdeographfa84', [], 'Display', None],
  [ 0xfa85, 'CjkCompatibilityIdeographfa85', [], 'Display', None],
  [ 0xfa86, 'CjkCompatibilityIdeographfa86', [], 'Display', None],
  [ 0xfa87, 'CjkCompatibilityIdeographfa87', [], 'Display', None],
  [ 0xfa88, 'CjkCompatibilityIdeographfa88', [], 'Display', None],
  [ 0xfa89, 'CjkCompatibilityIdeographfa89', [], 'Display', None],
  [ 0xfa8a, 'CjkCompatibilityIdeographfa8a', [], 'Display', None],
  [ 0xfa8b, 'CjkCompatibilityIdeographfa8b', [], 'Display', None],
  [ 0xfa8c, 'CjkCompatibilityIdeographfa8c', [], 'Display', None],
  [ 0xfa8d, 'CjkCompatibilityIdeographfa8d', [], 'Display', None],
  [ 0xfa8e, 'CjkCompatibilityIdeographfa8e', [], 'Display', None],
  [ 0xfa8f, 'CjkCompatibilityIdeographfa8f', [], 'Display', None],
  [ 0xfa90, 'CjkCompatibilityIdeographfa90', [], 'Display', None],
  [ 0xfa91, 'CjkCompatibilityIdeographfa91', [], 'Display', None],
  [ 0xfa92, 'CjkCompatibilityIdeographfa92', [], 'Display', None],
  [ 0xfa93, 'CjkCompatibilityIdeographfa93', [], 'Display', None],
  [ 0xfa94, 'CjkCompatibilityIdeographfa94', [], 'Display', None],
  [ 0xfa95, 'CjkCompatibilityIdeographfa95', [], 'Display', None],
  [ 0xfa96, 'CjkCompatibilityIdeographfa96', [], 'Display', None],
  [ 0xfa97, 'CjkCompatibilityIdeographfa97', [], 'Display', None],
  [ 0xfa98, 'CjkCompatibilityIdeographfa98', [], 'Display', None],
  [ 0xfa99, 'CjkCompatibilityIdeographfa99', [], 'Display', None],
  [ 0xfa9a, 'CjkCompatibilityIdeographfa9a', [], 'Display', None],
  [ 0xfa9b, 'CjkCompatibilityIdeographfa9b', [], 'Display', None],
  [ 0xfa9c, 'CjkCompatibilityIdeographfa9c', [], 'Display', None],
  [ 0xfa9d, 'CjkCompatibilityIdeographfa9d', [], 'Display', None],
  [ 0xfa9e, 'CjkCompatibilityIdeographfa9e', [], 'Display', None],
  [ 0xfa9f, 'CjkCompatibilityIdeographfa9f', [], 'Display', None],
  [ 0xfaa0, 'CjkCompatibilityIdeographfaa0', [], 'Display', None],
  [ 0xfaa1, 'CjkCompatibilityIdeographfaa1', [], 'Display', None],
  [ 0xfaa2, 'CjkCompatibilityIdeographfaa2', [], 'Display', None],
  [ 0xfaa3, 'CjkCompatibilityIdeographfaa3', [], 'Display', None],
  [ 0xfaa4, 'CjkCompatibilityIdeographfaa4', [], 'Display', None],
  [ 0xfaa5, 'CjkCompatibilityIdeographfaa5', [], 'Display', None],
  [ 0xfaa6, 'CjkCompatibilityIdeographfaa6', [], 'Display', None],
  [ 0xfaa7, 'CjkCompatibilityIdeographfaa7', [], 'Display', None],
  [ 0xfaa8, 'CjkCompatibilityIdeographfaa8', [], 'Display', None],
  [ 0xfaa9, 'CjkCompatibilityIdeographfaa9', [], 'Display', None],
  [ 0xfaaa, 'CjkCompatibilityIdeographfaaa', [], 'Display', None],
  [ 0xfaab, 'CjkCompatibilityIdeographfaab', [], 'Display', None],
  [ 0xfaac, 'CjkCompatibilityIdeographfaac', [], 'Display', None],
  [ 0xfaad, 'CjkCompatibilityIdeographfaad', [], 'Display', None],
  [ 0xfaae, 'CjkCompatibilityIdeographfaae', [], 'Display', None],
  [ 0xfaaf, 'CjkCompatibilityIdeographfaaf', [], 'Display', None],
  [ 0xfab0, 'CjkCompatibilityIdeographfab0', [], 'Display', None],
  [ 0xfab1, 'CjkCompatibilityIdeographfab1', [], 'Display', None],
  [ 0xfab2, 'CjkCompatibilityIdeographfab2', [], 'Display', None],
  [ 0xfab3, 'CjkCompatibilityIdeographfab3', [], 'Display', None],
  [ 0xfab4, 'CjkCompatibilityIdeographfab4', [], 'Display', None],
  [ 0xfab5, 'CjkCompatibilityIdeographfab5', [], 'Display', None],
  [ 0xfab6, 'CjkCompatibilityIdeographfab6', [], 'Display', None],
  [ 0xfab7, 'CjkCompatibilityIdeographfab7', [], 'Display', None],
  [ 0xfab8, 'CjkCompatibilityIdeographfab8', [], 'Display', None],
  [ 0xfab9, 'CjkCompatibilityIdeographfab9', [], 'Display', None],
  [ 0xfaba, 'CjkCompatibilityIdeographfaba', [], 'Display', None],
  [ 0xfabb, 'CjkCompatibilityIdeographfabb', [], 'Display', None],
  [ 0xfabc, 'CjkCompatibilityIdeographfabc', [], 'Display', None],
  [ 0xfabd, 'CjkCompatibilityIdeographfabd', [], 'Display', None],
  [ 0xfabe, 'CjkCompatibilityIdeographfabe', [], 'Display', None],
  [ 0xfabf, 'CjkCompatibilityIdeographfabf', [], 'Display', None],
  [ 0xfac0, 'CjkCompatibilityIdeographfac0', [], 'Display', None],
  [ 0xfac1, 'CjkCompatibilityIdeographfac1', [], 'Display', None],
  [ 0xfac2, 'CjkCompatibilityIdeographfac2', [], 'Display', None],
  [ 0xfac3, 'CjkCompatibilityIdeographfac3', [], 'Display', None],
  [ 0xfac4, 'CjkCompatibilityIdeographfac4', [], 'Display', None],
  [ 0xfac5, 'CjkCompatibilityIdeographfac5', [], 'Display', None],
  [ 0xfac6, 'CjkCompatibilityIdeographfac6', [], 'Display', None],
  [ 0xfac7, 'CjkCompatibilityIdeographfac7', [], 'Display', None],
  [ 0xfac8, 'CjkCompatibilityIdeographfac8', [], 'Display', None],
  [ 0xfac9, 'CjkCompatibilityIdeographfac9', [], 'Display', None],
  [ 0xfaca, 'CjkCompatibilityIdeographfaca', [], 'Display', None],
  [ 0xfacb, 'CjkCompatibilityIdeographfacb', [], 'Display', None],
  [ 0xfacc, 'CjkCompatibilityIdeographfacc', [], 'Display', None],
  [ 0xfacd, 'CjkCompatibilityIdeographfacd', [], 'Display', None],
  [ 0xface, 'CjkCompatibilityIdeographface', [], 'Display', None],
  [ 0xfacf, 'CjkCompatibilityIdeographfacf', [], 'Display', None],
  [ 0xfad0, 'CjkCompatibilityIdeographfad0', [], 'Display', None],
  [ 0xfad1, 'CjkCompatibilityIdeographfad1', [], 'Display', None],
  [ 0xfad2, 'CjkCompatibilityIdeographfad2', [], 'Display', None],
  [ 0xfad3, 'CjkCompatibilityIdeographfad3', [], 'Display', None],
  [ 0xfad4, 'CjkCompatibilityIdeographfad4', [], 'Display', None],
  [ 0xfad5, 'CjkCompatibilityIdeographfad5', [], 'Display', None],
  [ 0xfad6, 'CjkCompatibilityIdeographfad6', [], 'Display', None],
  [ 0xfad7, 'CjkCompatibilityIdeographfad7', [], 'Display', None],
  [ 0xfad8, 'CjkCompatibilityIdeographfad8', [], 'Display', None],
  [ 0xfad9, 'CjkCompatibilityIdeographfad9', [], 'Display', None],
  [ 0xfada, 'UnicodeCharacter0xfada', [], 'Display', None],
  [ 0xfadb, 'UnicodeCharacter0xfadb', [], 'Display', None],
  [ 0xfadc, 'UnicodeCharacter0xfadc', [], 'Display', None],
  [ 0xfadd, 'UnicodeCharacter0xfadd', [], 'Display', None],
  [ 0xfade, 'UnicodeCharacter0xfade', [], 'Display', None],
  [ 0xfadf, 'UnicodeCharacter0xfadf', [], 'Display', None],
  [ 0xfae0, 'UnicodeCharacter0xfae0', [], 'Display', None],
  [ 0xfae1, 'UnicodeCharacter0xfae1', [], 'Display', None],
  [ 0xfae2, 'UnicodeCharacter0xfae2', [], 'Display', None],
  [ 0xfae3, 'UnicodeCharacter0xfae3', [], 'Display', None],
  [ 0xfae4, 'UnicodeCharacter0xfae4', [], 'Display', None],
  [ 0xfae5, 'UnicodeCharacter0xfae5', [], 'Display', None],
  [ 0xfae6, 'UnicodeCharacter0xfae6', [], 'Display', None],
  [ 0xfae7, 'UnicodeCharacter0xfae7', [], 'Display', None],
  [ 0xfae8, 'UnicodeCharacter0xfae8', [], 'Display', None],
  [ 0xfae9, 'UnicodeCharacter0xfae9', [], 'Display', None],
  [ 0xfaea, 'UnicodeCharacter0xfaea', [], 'Display', None],
  [ 0xfaeb, 'UnicodeCharacter0xfaeb', [], 'Display', None],
  [ 0xfaec, 'UnicodeCharacter0xfaec', [], 'Display', None],
  [ 0xfaed, 'UnicodeCharacter0xfaed', [], 'Display', None],
  [ 0xfaee, 'UnicodeCharacter0xfaee', [], 'Display', None],
  [ 0xfaef, 'UnicodeCharacter0xfaef', [], 'Display', None],
  [ 0xfaf0, 'UnicodeCharacter0xfaf0', [], 'Display', None],
  [ 0xfaf1, 'UnicodeCharacter0xfaf1', [], 'Display', None],
  [ 0xfaf2, 'UnicodeCharacter0xfaf2', [], 'Display', None],
  [ 0xfaf3, 'UnicodeCharacter0xfaf3', [], 'Display', None],
  [ 0xfaf4, 'UnicodeCharacter0xfaf4', [], 'Display', None],
  [ 0xfaf5, 'UnicodeCharacter0xfaf5', [], 'Display', None],
  [ 0xfaf6, 'UnicodeCharacter0xfaf6', [], 'Display', None],
  [ 0xfaf7, 'UnicodeCharacter0xfaf7', [], 'Display', None],
  [ 0xfaf8, 'UnicodeCharacter0xfaf8', [], 'Display', None],
  [ 0xfaf9, 'UnicodeCharacter0xfaf9', [], 'Display', None],
  [ 0xfafa, 'UnicodeCharacter0xfafa', [], 'Display', None],
  [ 0xfafb, 'UnicodeCharacter0xfafb', [], 'Display', None],
  [ 0xfafc, 'UnicodeCharacter0xfafc', [], 'Display', None],
  [ 0xfafd, 'UnicodeCharacter0xfafd', [], 'Display', None],
  [ 0xfafe, 'UnicodeCharacter0xfafe', [], 'Display', None],
  [ 0xfaff, 'UnicodeCharacter0xfaff', [], 'Display', None],
  [ 0xfb00, 'LatinSmallLigatureFf', [], 'Display', None],
  [ 0xfb01, 'LatinSmallLigatureFi', [], 'Display', None],
  [ 0xfb02, 'LatinSmallLigatureFl', [], 'Display', None],
  [ 0xfb03, 'LatinSmallLigatureFfi', [], 'Display', None],
  [ 0xfb04, 'LatinSmallLigatureFfl', [], 'Display', None],
  [ 0xfb05, 'LatinSmallLigatureLongST', [], 'Display', None],
  [ 0xfb06, 'LatinSmallLigatureSt', [], 'Display', None],
  [ 0xfb07, 'UnicodeCharacter0xfb07', [], 'Display', None],
  [ 0xfb08, 'UnicodeCharacter0xfb08', [], 'Display', None],
  [ 0xfb09, 'UnicodeCharacter0xfb09', [], 'Display', None],
  [ 0xfb0a, 'UnicodeCharacter0xfb0a', [], 'Display', None],
  [ 0xfb0b, 'UnicodeCharacter0xfb0b', [], 'Display', None],
  [ 0xfb0c, 'UnicodeCharacter0xfb0c', [], 'Display', None],
  [ 0xfb0d, 'UnicodeCharacter0xfb0d', [], 'Display', None],
  [ 0xfb0e, 'UnicodeCharacter0xfb0e', [], 'Display', None],
  [ 0xfb0f, 'UnicodeCharacter0xfb0f', [], 'Display', None],
  [ 0xfb10, 'UnicodeCharacter0xfb10', [], 'Display', None],
  [ 0xfb11, 'UnicodeCharacter0xfb11', [], 'Display', None],
  [ 0xfb12, 'UnicodeCharacter0xfb12', [], 'Display', None],
  [ 0xfb13, 'ArmenianSmallLigatureMenNow', [], 'Display', None],
  [ 0xfb14, 'ArmenianSmallLigatureMenEch', [], 'Display', None],
  [ 0xfb15, 'ArmenianSmallLigatureMenIni', [], 'Display', None],
  [ 0xfb16, 'ArmenianSmallLigatureVewNow', [], 'Display', None],
  [ 0xfb17, 'ArmenianSmallLigatureMenXeh', [], 'Display', None],
  [ 0xfb18, 'UnicodeCharacter0xfb18', [], 'Display', None],
  [ 0xfb19, 'UnicodeCharacter0xfb19', [], 'Display', None],
  [ 0xfb1a, 'UnicodeCharacter0xfb1a', [], 'Display', None],
  [ 0xfb1b, 'UnicodeCharacter0xfb1b', [], 'Display', None],
  [ 0xfb1c, 'UnicodeCharacter0xfb1c', [], 'Display', None],
  [ 0xfb1d, 'HebrewLetterYodWithHiriq', [], 'Display', None],
  [ 0xfb1e, 'HebrewPointJudeospanishVarika', [], 'Display', None],
  [ 0xfb1f, 'HebrewLigatureYiddishYodYodPatah', [], 'Display', None],
  [ 0xfb20, 'HebrewLetterAlternativeAyin', [], 'Display', None],
  [ 0xfb21, 'HebrewLetterWideAlef', [], 'Display', None],
  [ 0xfb22, 'HebrewLetterWideDalet', [], 'Display', None],
  [ 0xfb23, 'HebrewLetterWideHe', [], 'Display', None],
  [ 0xfb24, 'HebrewLetterWideKaf', [], 'Display', None],
  [ 0xfb25, 'HebrewLetterWideLamed', [], 'Display', None],
  [ 0xfb26, 'HebrewLetterWideFinalMem', [], 'Display', None],
  [ 0xfb27, 'HebrewLetterWideResh', [], 'Display', None],
  [ 0xfb28, 'HebrewLetterWideTav', [], 'Display', None],
  [ 0xfb29, 'HebrewLetterAlternativePlusSign', [], 'Display', None],
  [ 0xfb2a, 'HebrewLetterShinWithShinDot', [], 'Display', None],
  [ 0xfb2b, 'HebrewLetterShinWithSinDot', [], 'Display', None],
  [ 0xfb2c, 'HebrewLetterShinWithDageshAndShinDot', [], 'Display', None],
  [ 0xfb2d, 'HebrewLetterShinWithDageshAndSinDot', [], 'Display', None],
  [ 0xfb2e, 'HebrewLetterAlefWithPatah', [], 'Display', None],
  [ 0xfb2f, 'HebrewLetterAlefWithQamats', [], 'Display', None],
  [ 0xfb30, 'HebrewLetterAlefWithMapiq', [], 'Display', None],
  [ 0xfb31, 'HebrewLetterBetWithDagesh', [], 'Display', None],
  [ 0xfb32, 'HebrewLetterGimelWithDagesh', [], 'Display', None],
  [ 0xfb33, 'HebrewLetterDaletWithDagesh', [], 'Display', None],
  [ 0xfb34, 'HebrewLetterHeWithMapiq', [], 'Display', None],
  [ 0xfb35, 'HebrewLetterVavWithDagesh', [], 'Display', None],
  [ 0xfb36, 'HebrewLetterZayinWithDagesh', [], 'Display', None],
  [ 0xfb37, 'UnicodeCharacter0xfb37', [], 'Display', None],
  [ 0xfb38, 'HebrewLetterTetWithDagesh', [], 'Display', None],
  [ 0xfb39, 'HebrewLetterYodWithDagesh', [], 'Display', None],
  [ 0xfb3a, 'HebrewLetterFinalKafWithDagesh', [], 'Display', None],
  [ 0xfb3b, 'HebrewLetterKafWithDagesh', [], 'Display', None],
  [ 0xfb3c, 'HebrewLetterLamedWithDagesh', [], 'Display', None],
  [ 0xfb3d, 'UnicodeCharacter0xfb3d', [], 'Display', None],
  [ 0xfb3e, 'HebrewLetterMemWithDagesh', [], 'Display', None],
  [ 0xfb3f, 'UnicodeCharacter0xfb3f', [], 'Display', None],
  [ 0xfb40, 'HebrewLetterNunWithDagesh', [], 'Display', None],
  [ 0xfb41, 'HebrewLetterSamekhWithDagesh', [], 'Display', None],
  [ 0xfb42, 'UnicodeCharacter0xfb42', [], 'Display', None],
  [ 0xfb43, 'HebrewLetterFinalPeWithDagesh', [], 'Display', None],
  [ 0xfb44, 'HebrewLetterPeWithDagesh', [], 'Display', None],
  [ 0xfb45, 'UnicodeCharacter0xfb45', [], 'Display', None],
  [ 0xfb46, 'HebrewLetterTsadiWithDagesh', [], 'Display', None],
  [ 0xfb47, 'HebrewLetterQofWithDagesh', [], 'Display', None],
  [ 0xfb48, 'HebrewLetterReshWithDagesh', [], 'Display', None],
  [ 0xfb49, 'HebrewLetterShinWithDagesh', [], 'Display', None],
  [ 0xfb4a, 'HebrewLetterTavWithDagesh', [], 'Display', None],
  [ 0xfb4b, 'HebrewLetterVavWithHolam', [], 'Display', None],
  [ 0xfb4c, 'HebrewLetterBetWithRafe', [], 'Display', None],
  [ 0xfb4d, 'HebrewLetterKafWithRafe', [], 'Display', None],
  [ 0xfb4e, 'HebrewLetterPeWithRafe', [], 'Display', None],
  [ 0xfb4f, 'HebrewLigatureAlefLamed', [], 'Display', None],
  [ 0xfb50, 'ArabicLetterAlefWaslaIsolatedForm', [], 'Display', None],
  [ 0xfb51, 'ArabicLetterAlefWaslaFinalForm', [], 'Display', None],
  [ 0xfb52, 'ArabicLetterBeehIsolatedForm', [], 'Display', None],
  [ 0xfb53, 'ArabicLetterBeehFinalForm', [], 'Display', None],
  [ 0xfb54, 'ArabicLetterBeehInitialForm', [], 'Display', None],
  [ 0xfb55, 'ArabicLetterBeehMedialForm', [], 'Display', None],
  [ 0xfb56, 'ArabicLetterPehIsolatedForm', [], 'Display', None],
  [ 0xfb57, 'ArabicLetterPehFinalForm', [], 'Display', None],
  [ 0xfb58, 'ArabicLetterPehInitialForm', [], 'Display', None],
  [ 0xfb59, 'ArabicLetterPehMedialForm', [], 'Display', None],
  [ 0xfb5a, 'ArabicLetterBehehIsolatedForm', [], 'Display', None],
  [ 0xfb5b, 'ArabicLetterBehehFinalForm', [], 'Display', None],
  [ 0xfb5c, 'ArabicLetterBehehInitialForm', [], 'Display', None],
  [ 0xfb5d, 'ArabicLetterBehehMedialForm', [], 'Display', None],
  [ 0xfb5e, 'ArabicLetterTtehehIsolatedForm', [], 'Display', None],
  [ 0xfb5f, 'ArabicLetterTtehehFinalForm', [], 'Display', None],
  [ 0xfb60, 'ArabicLetterTtehehInitialForm', [], 'Display', None],
  [ 0xfb61, 'ArabicLetterTtehehMedialForm', [], 'Display', None],
  [ 0xfb62, 'ArabicLetterTehehIsolatedForm', [], 'Display', None],
  [ 0xfb63, 'ArabicLetterTehehFinalForm', [], 'Display', None],
  [ 0xfb64, 'ArabicLetterTehehInitialForm', [], 'Display', None],
  [ 0xfb65, 'ArabicLetterTehehMedialForm', [], 'Display', None],
  [ 0xfb66, 'ArabicLetterTtehIsolatedForm', [], 'Display', None],
  [ 0xfb67, 'ArabicLetterTtehFinalForm', [], 'Display', None],
  [ 0xfb68, 'ArabicLetterTtehInitialForm', [], 'Display', None],
  [ 0xfb69, 'ArabicLetterTtehMedialForm', [], 'Display', None],
  [ 0xfb6a, 'ArabicLetterVehIsolatedForm', [], 'Display', None],
  [ 0xfb6b, 'ArabicLetterVehFinalForm', [], 'Display', None],
  [ 0xfb6c, 'ArabicLetterVehInitialForm', [], 'Display', None],
  [ 0xfb6d, 'ArabicLetterVehMedialForm', [], 'Display', None],
  [ 0xfb6e, 'ArabicLetterPehehIsolatedForm', [], 'Display', None],
  [ 0xfb6f, 'ArabicLetterPehehFinalForm', [], 'Display', None],
  [ 0xfb70, 'ArabicLetterPehehInitialForm', [], 'Display', None],
  [ 0xfb71, 'ArabicLetterPehehMedialForm', [], 'Display', None],
  [ 0xfb72, 'ArabicLetterDyehIsolatedForm', [], 'Display', None],
  [ 0xfb73, 'ArabicLetterDyehFinalForm', [], 'Display', None],
  [ 0xfb74, 'ArabicLetterDyehInitialForm', [], 'Display', None],
  [ 0xfb75, 'ArabicLetterDyehMedialForm', [], 'Display', None],
  [ 0xfb76, 'ArabicLetterNyehIsolatedForm', [], 'Display', None],
  [ 0xfb77, 'ArabicLetterNyehFinalForm', [], 'Display', None],
  [ 0xfb78, 'ArabicLetterNyehInitialForm', [], 'Display', None],
  [ 0xfb79, 'ArabicLetterNyehMedialForm', [], 'Display', None],
  [ 0xfb7a, 'ArabicLetterTchehIsolatedForm', [], 'Display', None],
  [ 0xfb7b, 'ArabicLetterTchehFinalForm', [], 'Display', None],
  [ 0xfb7c, 'ArabicLetterTchehInitialForm', [], 'Display', None],
  [ 0xfb7d, 'ArabicLetterTchehMedialForm', [], 'Display', None],
  [ 0xfb7e, 'ArabicLetterTchehehIsolatedForm', [], 'Display', None],
  [ 0xfb7f, 'ArabicLetterTchehehFinalForm', [], 'Display', None],
  [ 0xfb80, 'ArabicLetterTchehehInitialForm', [], 'Display', None],
  [ 0xfb81, 'ArabicLetterTchehehMedialForm', [], 'Display', None],
  [ 0xfb82, 'ArabicLetterDdahalIsolatedForm', [], 'Display', None],
  [ 0xfb83, 'ArabicLetterDdahalFinalForm', [], 'Display', None],
  [ 0xfb84, 'ArabicLetterDahalIsolatedForm', [], 'Display', None],
  [ 0xfb85, 'ArabicLetterDahalFinalForm', [], 'Display', None],
  [ 0xfb86, 'ArabicLetterDulIsolatedForm', [], 'Display', None],
  [ 0xfb87, 'ArabicLetterDulFinalForm', [], 'Display', None],
  [ 0xfb88, 'ArabicLetterDdalIsolatedForm', [], 'Display', None],
  [ 0xfb89, 'ArabicLetterDdalFinalForm', [], 'Display', None],
  [ 0xfb8a, 'ArabicLetterJehIsolatedForm', [], 'Display', None],
  [ 0xfb8b, 'ArabicLetterJehFinalForm', [], 'Display', None],
  [ 0xfb8c, 'ArabicLetterRrehIsolatedForm', [], 'Display', None],
  [ 0xfb8d, 'ArabicLetterRrehFinalForm', [], 'Display', None],
  [ 0xfb8e, 'ArabicLetterKehehIsolatedForm', [], 'Display', None],
  [ 0xfb8f, 'ArabicLetterKehehFinalForm', [], 'Display', None],
  [ 0xfb90, 'ArabicLetterKehehInitialForm', [], 'Display', None],
  [ 0xfb91, 'ArabicLetterKehehMedialForm', [], 'Display', None],
  [ 0xfb92, 'ArabicLetterGafIsolatedForm', [], 'Display', None],
  [ 0xfb93, 'ArabicLetterGafFinalForm', [], 'Display', None],
  [ 0xfb94, 'ArabicLetterGafInitialForm', [], 'Display', None],
  [ 0xfb95, 'ArabicLetterGafMedialForm', [], 'Display', None],
  [ 0xfb96, 'ArabicLetterGuehIsolatedForm', [], 'Display', None],
  [ 0xfb97, 'ArabicLetterGuehFinalForm', [], 'Display', None],
  [ 0xfb98, 'ArabicLetterGuehInitialForm', [], 'Display', None],
  [ 0xfb99, 'ArabicLetterGuehMedialForm', [], 'Display', None],
  [ 0xfb9a, 'ArabicLetterNgoehIsolatedForm', [], 'Display', None],
  [ 0xfb9b, 'ArabicLetterNgoehFinalForm', [], 'Display', None],
  [ 0xfb9c, 'ArabicLetterNgoehInitialForm', [], 'Display', None],
  [ 0xfb9d, 'ArabicLetterNgoehMedialForm', [], 'Display', None],
  [ 0xfb9e, 'ArabicLetterNoonGhunnaIsolatedForm', [], 'Display', None],
  [ 0xfb9f, 'ArabicLetterNoonGhunnaFinalForm', [], 'Display', None],
  [ 0xfba0, 'ArabicLetterRnoonIsolatedForm', [], 'Display', None],
  [ 0xfba1, 'ArabicLetterRnoonFinalForm', [], 'Display', None],
  [ 0xfba2, 'ArabicLetterRnoonInitialForm', [], 'Display', None],
  [ 0xfba3, 'ArabicLetterRnoonMedialForm', [], 'Display', None],
  [ 0xfba4, 'ArabicLetterHehWithYehAboveIsolatedForm', [], 'Display', None],
  [ 0xfba5, 'ArabicLetterHehWithYehAboveFinalForm', [], 'Display', None],
  [ 0xfba6, 'ArabicLetterHehGoalIsolatedForm', [], 'Display', None],
  [ 0xfba7, 'ArabicLetterHehGoalFinalForm', [], 'Display', None],
  [ 0xfba8, 'ArabicLetterHehGoalInitialForm', [], 'Display', None],
  [ 0xfba9, 'ArabicLetterHehGoalMedialForm', [], 'Display', None],
  [ 0xfbaa, 'ArabicLetterHehDoachashmeeIsolatedForm', [], 'Display', None],
  [ 0xfbab, 'ArabicLetterHehDoachashmeeFinalForm', [], 'Display', None],
  [ 0xfbac, 'ArabicLetterHehDoachashmeeInitialForm', [], 'Display', None],
  [ 0xfbad, 'ArabicLetterHehDoachashmeeMedialForm', [], 'Display', None],
  [ 0xfbae, 'ArabicLetterYehBarreeIsolatedForm', [], 'Display', None],
  [ 0xfbaf, 'ArabicLetterYehBarreeFinalForm', [], 'Display', None],
  [ 0xfbb0, 'ArabicLetterYehBarreeWithHamzaAboveIsolatedForm', [], 'Display', None],
  [ 0xfbb1, 'ArabicLetterYehBarreeWithHamzaAboveFinalForm', [], 'Display', None],
  [ 0xfbb2, 'UnicodeCharacter0xfbb2', [], 'Display', None],
  [ 0xfbb3, 'UnicodeCharacter0xfbb3', [], 'Display', None],
  [ 0xfbb4, 'UnicodeCharacter0xfbb4', [], 'Display', None],
  [ 0xfbb5, 'UnicodeCharacter0xfbb5', [], 'Display', None],
  [ 0xfbb6, 'UnicodeCharacter0xfbb6', [], 'Display', None],
  [ 0xfbb7, 'UnicodeCharacter0xfbb7', [], 'Display', None],
  [ 0xfbb8, 'UnicodeCharacter0xfbb8', [], 'Display', None],
  [ 0xfbb9, 'UnicodeCharacter0xfbb9', [], 'Display', None],
  [ 0xfbba, 'UnicodeCharacter0xfbba', [], 'Display', None],
  [ 0xfbbb, 'UnicodeCharacter0xfbbb', [], 'Display', None],
  [ 0xfbbc, 'UnicodeCharacter0xfbbc', [], 'Display', None],
  [ 0xfbbd, 'UnicodeCharacter0xfbbd', [], 'Display', None],
  [ 0xfbbe, 'UnicodeCharacter0xfbbe', [], 'Display', None],
  [ 0xfbbf, 'UnicodeCharacter0xfbbf', [], 'Display', None],
  [ 0xfbc0, 'UnicodeCharacter0xfbc0', [], 'Display', None],
  [ 0xfbc1, 'UnicodeCharacter0xfbc1', [], 'Display', None],
  [ 0xfbc2, 'UnicodeCharacter0xfbc2', [], 'Display', None],
  [ 0xfbc3, 'UnicodeCharacter0xfbc3', [], 'Display', None],
  [ 0xfbc4, 'UnicodeCharacter0xfbc4', [], 'Display', None],
  [ 0xfbc5, 'UnicodeCharacter0xfbc5', [], 'Display', None],
  [ 0xfbc6, 'UnicodeCharacter0xfbc6', [], 'Display', None],
  [ 0xfbc7, 'UnicodeCharacter0xfbc7', [], 'Display', None],
  [ 0xfbc8, 'UnicodeCharacter0xfbc8', [], 'Display', None],
  [ 0xfbc9, 'UnicodeCharacter0xfbc9', [], 'Display', None],
  [ 0xfbca, 'UnicodeCharacter0xfbca', [], 'Display', None],
  [ 0xfbcb, 'UnicodeCharacter0xfbcb', [], 'Display', None],
  [ 0xfbcc, 'UnicodeCharacter0xfbcc', [], 'Display', None],
  [ 0xfbcd, 'UnicodeCharacter0xfbcd', [], 'Display', None],
  [ 0xfbce, 'UnicodeCharacter0xfbce', [], 'Display', None],
  [ 0xfbcf, 'UnicodeCharacter0xfbcf', [], 'Display', None],
  [ 0xfbd0, 'UnicodeCharacter0xfbd0', [], 'Display', None],
  [ 0xfbd1, 'UnicodeCharacter0xfbd1', [], 'Display', None],
  [ 0xfbd2, 'UnicodeCharacter0xfbd2', [], 'Display', None],
  [ 0xfbd3, 'ArabicLetterNgIsolatedForm', [], 'Display', None],
  [ 0xfbd4, 'ArabicLetterNgFinalForm', [], 'Display', None],
  [ 0xfbd5, 'ArabicLetterNgInitialForm', [], 'Display', None],
  [ 0xfbd6, 'ArabicLetterNgMedialForm', [], 'Display', None],
  [ 0xfbd7, 'ArabicLetterUIsolatedForm', [], 'Display', None],
  [ 0xfbd8, 'ArabicLetterUFinalForm', [], 'Display', None],
  [ 0xfbd9, 'ArabicLetterOeIsolatedForm', [], 'Display', None],
  [ 0xfbda, 'ArabicLetterOeFinalForm', [], 'Display', None],
  [ 0xfbdb, 'ArabicLetterYuIsolatedForm', [], 'Display', None],
  [ 0xfbdc, 'ArabicLetterYuFinalForm', [], 'Display', None],
  [ 0xfbdd, 'ArabicLetterUWithHamzaAboveIsolatedForm', [], 'Display', None],
  [ 0xfbde, 'ArabicLetterVeIsolatedForm', [], 'Display', None],
  [ 0xfbdf, 'ArabicLetterVeFinalForm', [], 'Display', None],
  [ 0xfbe0, 'ArabicLetterKirghizOeIsolatedForm', [], 'Display', None],
  [ 0xfbe1, 'ArabicLetterKirghizOeFinalForm', [], 'Display', None],
  [ 0xfbe2, 'ArabicLetterKirghizYuIsolatedForm', [], 'Display', None],
  [ 0xfbe3, 'ArabicLetterKirghizYuFinalForm', [], 'Display', None],
  [ 0xfbe4, 'ArabicLetterEIsolatedForm', [], 'Display', None],
  [ 0xfbe5, 'ArabicLetterEFinalForm', [], 'Display', None],
  [ 0xfbe6, 'ArabicLetterEInitialForm', [], 'Display', None],
  [ 0xfbe7, 'ArabicLetterEMedialForm', [], 'Display', None],
  [ 0xfbe8, 'ArabicLetterUighurKazakhKirghizAlefMaksuraInitialForm', [], 'Display', None],
  [ 0xfbe9, 'ArabicLetterUighurKazakhKirghizAlefMaksuraMedialForm', [], 'Display', None],
  [ 0xfbea, 'ArabicLigatureYehWithHamzaAboveWithAlefIsolatedForm', [], 'Display', None],
  [ 0xfbeb, 'ArabicLigatureYehWithHamzaAboveWithAlefFinalForm', [], 'Display', None],
  [ 0xfbec, 'ArabicLigatureYehWithHamzaAboveWithAeIsolatedForm', [], 'Display', None],
  [ 0xfbed, 'ArabicLigatureYehWithHamzaAboveWithAeFinalForm', [], 'Display', None],
  [ 0xfbee, 'ArabicLigatureYehWithHamzaAboveWithWawIsolatedForm', [], 'Display', None],
  [ 0xfbef, 'ArabicLigatureYehWithHamzaAboveWithWawFinalForm', [], 'Display', None],
  [ 0xfbf0, 'ArabicLigatureYehWithHamzaAboveWithUIsolatedForm', [], 'Display', None],
  [ 0xfbf1, 'ArabicLigatureYehWithHamzaAboveWithUFinalForm', [], 'Display', None],
  [ 0xfbf2, 'ArabicLigatureYehWithHamzaAboveWithOeIsolatedForm', [], 'Display', None],
  [ 0xfbf3, 'ArabicLigatureYehWithHamzaAboveWithOeFinalForm', [], 'Display', None],
  [ 0xfbf4, 'ArabicLigatureYehWithHamzaAboveWithYuIsolatedForm', [], 'Display', None],
  [ 0xfbf5, 'ArabicLigatureYehWithHamzaAboveWithYuFinalForm', [], 'Display', None],
  [ 0xfbf6, 'ArabicLigatureYehWithHamzaAboveWithEIsolatedForm', [], 'Display', None],
  [ 0xfbf7, 'ArabicLigatureYehWithHamzaAboveWithEFinalForm', [], 'Display', None],
  [ 0xfbf8, 'ArabicLigatureYehWithHamzaAboveWithEInitialForm', [], 'Display', None],
  [ 0xfbf9, 'ArabicLigatureUighurKirghizYehWithHamzaAboveWithAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfbfa, 'ArabicLigatureUighurKirghizYehWithHamzaAboveWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfbfb, 'ArabicLigatureUighurKirghizYehWithHamzaAboveWithAlefMaksuraInitialForm', [], 'Display', None],
  [ 0xfbfc, 'ArabicLetterFarsiYehIsolatedForm', [], 'Display', None],
  [ 0xfbfd, 'ArabicLetterFarsiYehFinalForm', [], 'Display', None],
  [ 0xfbfe, 'ArabicLetterFarsiYehInitialForm', [], 'Display', None],
  [ 0xfbff, 'ArabicLetterFarsiYehMedialForm', [], 'Display', None],
  [ 0xfc00, 'ArabicLigatureYehWithHamzaAboveWithJeemIsolatedForm', [], 'Display', None],
  [ 0xfc01, 'ArabicLigatureYehWithHamzaAboveWithHahIsolatedForm', [], 'Display', None],
  [ 0xfc02, 'ArabicLigatureYehWithHamzaAboveWithMeemIsolatedForm', [], 'Display', None],
  [ 0xfc03, 'ArabicLigatureYehWithHamzaAboveWithAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfc04, 'ArabicLigatureYehWithHamzaAboveWithYehIsolatedForm', [], 'Display', None],
  [ 0xfc05, 'ArabicLigatureBehWithJeemIsolatedForm', [], 'Display', None],
  [ 0xfc06, 'ArabicLigatureBehWithHahIsolatedForm', [], 'Display', None],
  [ 0xfc07, 'ArabicLigatureBehWithKhahIsolatedForm', [], 'Display', None],
  [ 0xfc08, 'ArabicLigatureBehWithMeemIsolatedForm', [], 'Display', None],
  [ 0xfc09, 'ArabicLigatureBehWithAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfc0a, 'ArabicLigatureBehWithYehIsolatedForm', [], 'Display', None],
  [ 0xfc0b, 'ArabicLigatureTehWithJeemIsolatedForm', [], 'Display', None],
  [ 0xfc0c, 'ArabicLigatureTehWithHahIsolatedForm', [], 'Display', None],
  [ 0xfc0d, 'ArabicLigatureTehWithKhahIsolatedForm', [], 'Display', None],
  [ 0xfc0e, 'ArabicLigatureTehWithMeemIsolatedForm', [], 'Display', None],
  [ 0xfc0f, 'ArabicLigatureTehWithAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfc10, 'ArabicLigatureTehWithYehIsolatedForm', [], 'Display', None],
  [ 0xfc11, 'ArabicLigatureThehWithJeemIsolatedForm', [], 'Display', None],
  [ 0xfc12, 'ArabicLigatureThehWithMeemIsolatedForm', [], 'Display', None],
  [ 0xfc13, 'ArabicLigatureThehWithAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfc14, 'ArabicLigatureThehWithYehIsolatedForm', [], 'Display', None],
  [ 0xfc15, 'ArabicLigatureJeemWithHahIsolatedForm', [], 'Display', None],
  [ 0xfc16, 'ArabicLigatureJeemWithMeemIsolatedForm', [], 'Display', None],
  [ 0xfc17, 'ArabicLigatureHahWithJeemIsolatedForm', [], 'Display', None],
  [ 0xfc18, 'ArabicLigatureHahWithMeemIsolatedForm', [], 'Display', None],
  [ 0xfc19, 'ArabicLigatureKhahWithJeemIsolatedForm', [], 'Display', None],
  [ 0xfc1a, 'ArabicLigatureKhahWithHahIsolatedForm', [], 'Display', None],
  [ 0xfc1b, 'ArabicLigatureKhahWithMeemIsolatedForm', [], 'Display', None],
  [ 0xfc1c, 'ArabicLigatureSeenWithJeemIsolatedForm', [], 'Display', None],
  [ 0xfc1d, 'ArabicLigatureSeenWithHahIsolatedForm', [], 'Display', None],
  [ 0xfc1e, 'ArabicLigatureSeenWithKhahIsolatedForm', [], 'Display', None],
  [ 0xfc1f, 'ArabicLigatureSeenWithMeemIsolatedForm', [], 'Display', None],
  [ 0xfc20, 'ArabicLigatureSadWithHahIsolatedForm', [], 'Display', None],
  [ 0xfc21, 'ArabicLigatureSadWithMeemIsolatedForm', [], 'Display', None],
  [ 0xfc22, 'ArabicLigatureDadWithJeemIsolatedForm', [], 'Display', None],
  [ 0xfc23, 'ArabicLigatureDadWithHahIsolatedForm', [], 'Display', None],
  [ 0xfc24, 'ArabicLigatureDadWithKhahIsolatedForm', [], 'Display', None],
  [ 0xfc25, 'ArabicLigatureDadWithMeemIsolatedForm', [], 'Display', None],
  [ 0xfc26, 'ArabicLigatureTahWithHahIsolatedForm', [], 'Display', None],
  [ 0xfc27, 'ArabicLigatureTahWithMeemIsolatedForm', [], 'Display', None],
  [ 0xfc28, 'ArabicLigatureZahWithMeemIsolatedForm', [], 'Display', None],
  [ 0xfc29, 'ArabicLigatureAinWithJeemIsolatedForm', [], 'Display', None],
  [ 0xfc2a, 'ArabicLigatureAinWithMeemIsolatedForm', [], 'Display', None],
  [ 0xfc2b, 'ArabicLigatureGhainWithJeemIsolatedForm', [], 'Display', None],
  [ 0xfc2c, 'ArabicLigatureGhainWithMeemIsolatedForm', [], 'Display', None],
  [ 0xfc2d, 'ArabicLigatureFehWithJeemIsolatedForm', [], 'Display', None],
  [ 0xfc2e, 'ArabicLigatureFehWithHahIsolatedForm', [], 'Display', None],
  [ 0xfc2f, 'ArabicLigatureFehWithKhahIsolatedForm', [], 'Display', None],
  [ 0xfc30, 'ArabicLigatureFehWithMeemIsolatedForm', [], 'Display', None],
  [ 0xfc31, 'ArabicLigatureFehWithAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfc32, 'ArabicLigatureFehWithYehIsolatedForm', [], 'Display', None],
  [ 0xfc33, 'ArabicLigatureQafWithHahIsolatedForm', [], 'Display', None],
  [ 0xfc34, 'ArabicLigatureQafWithMeemIsolatedForm', [], 'Display', None],
  [ 0xfc35, 'ArabicLigatureQafWithAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfc36, 'ArabicLigatureQafWithYehIsolatedForm', [], 'Display', None],
  [ 0xfc37, 'ArabicLigatureKafWithAlefIsolatedForm', [], 'Display', None],
  [ 0xfc38, 'ArabicLigatureKafWithJeemIsolatedForm', [], 'Display', None],
  [ 0xfc39, 'ArabicLigatureKafWithHahIsolatedForm', [], 'Display', None],
  [ 0xfc3a, 'ArabicLigatureKafWithKhahIsolatedForm', [], 'Display', None],
  [ 0xfc3b, 'ArabicLigatureKafWithLamIsolatedForm', [], 'Display', None],
  [ 0xfc3c, 'ArabicLigatureKafWithMeemIsolatedForm', [], 'Display', None],
  [ 0xfc3d, 'ArabicLigatureKafWithAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfc3e, 'ArabicLigatureKafWithYehIsolatedForm', [], 'Display', None],
  [ 0xfc3f, 'ArabicLigatureLamWithJeemIsolatedForm', [], 'Display', None],
  [ 0xfc40, 'ArabicLigatureLamWithHahIsolatedForm', [], 'Display', None],
  [ 0xfc41, 'ArabicLigatureLamWithKhahIsolatedForm', [], 'Display', None],
  [ 0xfc42, 'ArabicLigatureLamWithMeemIsolatedForm', [], 'Display', None],
  [ 0xfc43, 'ArabicLigatureLamWithAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfc44, 'ArabicLigatureLamWithYehIsolatedForm', [], 'Display', None],
  [ 0xfc45, 'ArabicLigatureMeemWithJeemIsolatedForm', [], 'Display', None],
  [ 0xfc46, 'ArabicLigatureMeemWithHahIsolatedForm', [], 'Display', None],
  [ 0xfc47, 'ArabicLigatureMeemWithKhahIsolatedForm', [], 'Display', None],
  [ 0xfc48, 'ArabicLigatureMeemWithMeemIsolatedForm', [], 'Display', None],
  [ 0xfc49, 'ArabicLigatureMeemWithAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfc4a, 'ArabicLigatureMeemWithYehIsolatedForm', [], 'Display', None],
  [ 0xfc4b, 'ArabicLigatureNoonWithJeemIsolatedForm', [], 'Display', None],
  [ 0xfc4c, 'ArabicLigatureNoonWithHahIsolatedForm', [], 'Display', None],
  [ 0xfc4d, 'ArabicLigatureNoonWithKhahIsolatedForm', [], 'Display', None],
  [ 0xfc4e, 'ArabicLigatureNoonWithMeemIsolatedForm', [], 'Display', None],
  [ 0xfc4f, 'ArabicLigatureNoonWithAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfc50, 'ArabicLigatureNoonWithYehIsolatedForm', [], 'Display', None],
  [ 0xfc51, 'ArabicLigatureHehWithJeemIsolatedForm', [], 'Display', None],
  [ 0xfc52, 'ArabicLigatureHehWithMeemIsolatedForm', [], 'Display', None],
  [ 0xfc53, 'ArabicLigatureHehWithAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfc54, 'ArabicLigatureHehWithYehIsolatedForm', [], 'Display', None],
  [ 0xfc55, 'ArabicLigatureYehWithJeemIsolatedForm', [], 'Display', None],
  [ 0xfc56, 'ArabicLigatureYehWithHahIsolatedForm', [], 'Display', None],
  [ 0xfc57, 'ArabicLigatureYehWithKhahIsolatedForm', [], 'Display', None],
  [ 0xfc58, 'ArabicLigatureYehWithMeemIsolatedForm', [], 'Display', None],
  [ 0xfc59, 'ArabicLigatureYehWithAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfc5a, 'ArabicLigatureYehWithYehIsolatedForm', [], 'Display', None],
  [ 0xfc5b, 'ArabicLigatureThalWithSuperscriptAlefIsolatedForm', [], 'Display', None],
  [ 0xfc5c, 'ArabicLigatureRehWithSuperscriptAlefIsolatedForm', [], 'Display', None],
  [ 0xfc5d, 'ArabicLigatureAlefMaksuraWithSuperscriptAlefIsolatedForm', [], 'Display', None],
  [ 0xfc5e, 'ArabicLigatureShaddaWithDammatanIsolatedForm', [], 'Display', None],
  [ 0xfc5f, 'ArabicLigatureShaddaWithKasratanIsolatedForm', [], 'Display', None],
  [ 0xfc60, 'ArabicLigatureShaddaWithFathaIsolatedForm', [], 'Display', None],
  [ 0xfc61, 'ArabicLigatureShaddaWithDammaIsolatedForm', [], 'Display', None],
  [ 0xfc62, 'ArabicLigatureShaddaWithKasraIsolatedForm', [], 'Display', None],
  [ 0xfc63, 'ArabicLigatureShaddaWithSuperscriptAlefIsolatedForm', [], 'Display', None],
  [ 0xfc64, 'ArabicLigatureYehWithHamzaAboveWithRehFinalForm', [], 'Display', None],
  [ 0xfc65, 'ArabicLigatureYehWithHamzaAboveWithZainFinalForm', [], 'Display', None],
  [ 0xfc66, 'ArabicLigatureYehWithHamzaAboveWithMeemFinalForm', [], 'Display', None],
  [ 0xfc67, 'ArabicLigatureYehWithHamzaAboveWithNoonFinalForm', [], 'Display', None],
  [ 0xfc68, 'ArabicLigatureYehWithHamzaAboveWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfc69, 'ArabicLigatureYehWithHamzaAboveWithYehFinalForm', [], 'Display', None],
  [ 0xfc6a, 'ArabicLigatureBehWithRehFinalForm', [], 'Display', None],
  [ 0xfc6b, 'ArabicLigatureBehWithZainFinalForm', [], 'Display', None],
  [ 0xfc6c, 'ArabicLigatureBehWithMeemFinalForm', [], 'Display', None],
  [ 0xfc6d, 'ArabicLigatureBehWithNoonFinalForm', [], 'Display', None],
  [ 0xfc6e, 'ArabicLigatureBehWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfc6f, 'ArabicLigatureBehWithYehFinalForm', [], 'Display', None],
  [ 0xfc70, 'ArabicLigatureTehWithRehFinalForm', [], 'Display', None],
  [ 0xfc71, 'ArabicLigatureTehWithZainFinalForm', [], 'Display', None],
  [ 0xfc72, 'ArabicLigatureTehWithMeemFinalForm', [], 'Display', None],
  [ 0xfc73, 'ArabicLigatureTehWithNoonFinalForm', [], 'Display', None],
  [ 0xfc74, 'ArabicLigatureTehWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfc75, 'ArabicLigatureTehWithYehFinalForm', [], 'Display', None],
  [ 0xfc76, 'ArabicLigatureThehWithRehFinalForm', [], 'Display', None],
  [ 0xfc77, 'ArabicLigatureThehWithZainFinalForm', [], 'Display', None],
  [ 0xfc78, 'ArabicLigatureThehWithMeemFinalForm', [], 'Display', None],
  [ 0xfc79, 'ArabicLigatureThehWithNoonFinalForm', [], 'Display', None],
  [ 0xfc7a, 'ArabicLigatureThehWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfc7b, 'ArabicLigatureThehWithYehFinalForm', [], 'Display', None],
  [ 0xfc7c, 'ArabicLigatureFehWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfc7d, 'ArabicLigatureFehWithYehFinalForm', [], 'Display', None],
  [ 0xfc7e, 'ArabicLigatureQafWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfc7f, 'ArabicLigatureQafWithYehFinalForm', [], 'Display', None],
  [ 0xfc80, 'ArabicLigatureKafWithAlefFinalForm', [], 'Display', None],
  [ 0xfc81, 'ArabicLigatureKafWithLamFinalForm', [], 'Display', None],
  [ 0xfc82, 'ArabicLigatureKafWithMeemFinalForm', [], 'Display', None],
  [ 0xfc83, 'ArabicLigatureKafWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfc84, 'ArabicLigatureKafWithYehFinalForm', [], 'Display', None],
  [ 0xfc85, 'ArabicLigatureLamWithMeemFinalForm', [], 'Display', None],
  [ 0xfc86, 'ArabicLigatureLamWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfc87, 'ArabicLigatureLamWithYehFinalForm', [], 'Display', None],
  [ 0xfc88, 'ArabicLigatureMeemWithAlefFinalForm', [], 'Display', None],
  [ 0xfc89, 'ArabicLigatureMeemWithMeemFinalForm', [], 'Display', None],
  [ 0xfc8a, 'ArabicLigatureNoonWithRehFinalForm', [], 'Display', None],
  [ 0xfc8b, 'ArabicLigatureNoonWithZainFinalForm', [], 'Display', None],
  [ 0xfc8c, 'ArabicLigatureNoonWithMeemFinalForm', [], 'Display', None],
  [ 0xfc8d, 'ArabicLigatureNoonWithNoonFinalForm', [], 'Display', None],
  [ 0xfc8e, 'ArabicLigatureNoonWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfc8f, 'ArabicLigatureNoonWithYehFinalForm', [], 'Display', None],
  [ 0xfc90, 'ArabicLigatureAlefMaksuraWithSuperscriptAlefFinalForm', [], 'Display', None],
  [ 0xfc91, 'ArabicLigatureYehWithRehFinalForm', [], 'Display', None],
  [ 0xfc92, 'ArabicLigatureYehWithZainFinalForm', [], 'Display', None],
  [ 0xfc93, 'ArabicLigatureYehWithMeemFinalForm', [], 'Display', None],
  [ 0xfc94, 'ArabicLigatureYehWithNoonFinalForm', [], 'Display', None],
  [ 0xfc95, 'ArabicLigatureYehWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfc96, 'ArabicLigatureYehWithYehFinalForm', [], 'Display', None],
  [ 0xfc97, 'ArabicLigatureYehWithHamzaAboveWithJeemInitialForm', [], 'Display', None],
  [ 0xfc98, 'ArabicLigatureYehWithHamzaAboveWithHahInitialForm', [], 'Display', None],
  [ 0xfc99, 'ArabicLigatureYehWithHamzaAboveWithKhahInitialForm', [], 'Display', None],
  [ 0xfc9a, 'ArabicLigatureYehWithHamzaAboveWithMeemInitialForm', [], 'Display', None],
  [ 0xfc9b, 'ArabicLigatureYehWithHamzaAboveWithHehInitialForm', [], 'Display', None],
  [ 0xfc9c, 'ArabicLigatureBehWithJeemInitialForm', [], 'Display', None],
  [ 0xfc9d, 'ArabicLigatureBehWithHahInitialForm', [], 'Display', None],
  [ 0xfc9e, 'ArabicLigatureBehWithKhahInitialForm', [], 'Display', None],
  [ 0xfc9f, 'ArabicLigatureBehWithMeemInitialForm', [], 'Display', None],
  [ 0xfca0, 'ArabicLigatureBehWithHehInitialForm', [], 'Display', None],
  [ 0xfca1, 'ArabicLigatureTehWithJeemInitialForm', [], 'Display', None],
  [ 0xfca2, 'ArabicLigatureTehWithHahInitialForm', [], 'Display', None],
  [ 0xfca3, 'ArabicLigatureTehWithKhahInitialForm', [], 'Display', None],
  [ 0xfca4, 'ArabicLigatureTehWithMeemInitialForm', [], 'Display', None],
  [ 0xfca5, 'ArabicLigatureTehWithHehInitialForm', [], 'Display', None],
  [ 0xfca6, 'ArabicLigatureThehWithMeemInitialForm', [], 'Display', None],
  [ 0xfca7, 'ArabicLigatureJeemWithHahInitialForm', [], 'Display', None],
  [ 0xfca8, 'ArabicLigatureJeemWithMeemInitialForm', [], 'Display', None],
  [ 0xfca9, 'ArabicLigatureHahWithJeemInitialForm', [], 'Display', None],
  [ 0xfcaa, 'ArabicLigatureHahWithMeemInitialForm', [], 'Display', None],
  [ 0xfcab, 'ArabicLigatureKhahWithJeemInitialForm', [], 'Display', None],
  [ 0xfcac, 'ArabicLigatureKhahWithMeemInitialForm', [], 'Display', None],
  [ 0xfcad, 'ArabicLigatureSeenWithJeemInitialForm', [], 'Display', None],
  [ 0xfcae, 'ArabicLigatureSeenWithHahInitialForm', [], 'Display', None],
  [ 0xfcaf, 'ArabicLigatureSeenWithKhahInitialForm', [], 'Display', None],
  [ 0xfcb0, 'ArabicLigatureSeenWithMeemInitialForm', [], 'Display', None],
  [ 0xfcb1, 'ArabicLigatureSadWithHahInitialForm', [], 'Display', None],
  [ 0xfcb2, 'ArabicLigatureSadWithKhahInitialForm', [], 'Display', None],
  [ 0xfcb3, 'ArabicLigatureSadWithMeemInitialForm', [], 'Display', None],
  [ 0xfcb4, 'ArabicLigatureDadWithJeemInitialForm', [], 'Display', None],
  [ 0xfcb5, 'ArabicLigatureDadWithHahInitialForm', [], 'Display', None],
  [ 0xfcb6, 'ArabicLigatureDadWithKhahInitialForm', [], 'Display', None],
  [ 0xfcb7, 'ArabicLigatureDadWithMeemInitialForm', [], 'Display', None],
  [ 0xfcb8, 'ArabicLigatureTahWithHahInitialForm', [], 'Display', None],
  [ 0xfcb9, 'ArabicLigatureZahWithMeemInitialForm', [], 'Display', None],
  [ 0xfcba, 'ArabicLigatureAinWithJeemInitialForm', [], 'Display', None],
  [ 0xfcbb, 'ArabicLigatureAinWithMeemInitialForm', [], 'Display', None],
  [ 0xfcbc, 'ArabicLigatureGhainWithJeemInitialForm', [], 'Display', None],
  [ 0xfcbd, 'ArabicLigatureGhainWithMeemInitialForm', [], 'Display', None],
  [ 0xfcbe, 'ArabicLigatureFehWithJeemInitialForm', [], 'Display', None],
  [ 0xfcbf, 'ArabicLigatureFehWithHahInitialForm', [], 'Display', None],
  [ 0xfcc0, 'ArabicLigatureFehWithKhahInitialForm', [], 'Display', None],
  [ 0xfcc1, 'ArabicLigatureFehWithMeemInitialForm', [], 'Display', None],
  [ 0xfcc2, 'ArabicLigatureQafWithHahInitialForm', [], 'Display', None],
  [ 0xfcc3, 'ArabicLigatureQafWithMeemInitialForm', [], 'Display', None],
  [ 0xfcc4, 'ArabicLigatureKafWithJeemInitialForm', [], 'Display', None],
  [ 0xfcc5, 'ArabicLigatureKafWithHahInitialForm', [], 'Display', None],
  [ 0xfcc6, 'ArabicLigatureKafWithKhahInitialForm', [], 'Display', None],
  [ 0xfcc7, 'ArabicLigatureKafWithLamInitialForm', [], 'Display', None],
  [ 0xfcc8, 'ArabicLigatureKafWithMeemInitialForm', [], 'Display', None],
  [ 0xfcc9, 'ArabicLigatureLamWithJeemInitialForm', [], 'Display', None],
  [ 0xfcca, 'ArabicLigatureLamWithHahInitialForm', [], 'Display', None],
  [ 0xfccb, 'ArabicLigatureLamWithKhahInitialForm', [], 'Display', None],
  [ 0xfccc, 'ArabicLigatureLamWithMeemInitialForm', [], 'Display', None],
  [ 0xfccd, 'ArabicLigatureLamWithHehInitialForm', [], 'Display', None],
  [ 0xfcce, 'ArabicLigatureMeemWithJeemInitialForm', [], 'Display', None],
  [ 0xfccf, 'ArabicLigatureMeemWithHahInitialForm', [], 'Display', None],
  [ 0xfcd0, 'ArabicLigatureMeemWithKhahInitialForm', [], 'Display', None],
  [ 0xfcd1, 'ArabicLigatureMeemWithMeemInitialForm', [], 'Display', None],
  [ 0xfcd2, 'ArabicLigatureNoonWithJeemInitialForm', [], 'Display', None],
  [ 0xfcd3, 'ArabicLigatureNoonWithHahInitialForm', [], 'Display', None],
  [ 0xfcd4, 'ArabicLigatureNoonWithKhahInitialForm', [], 'Display', None],
  [ 0xfcd5, 'ArabicLigatureNoonWithMeemInitialForm', [], 'Display', None],
  [ 0xfcd6, 'ArabicLigatureNoonWithHehInitialForm', [], 'Display', None],
  [ 0xfcd7, 'ArabicLigatureHehWithJeemInitialForm', [], 'Display', None],
  [ 0xfcd8, 'ArabicLigatureHehWithMeemInitialForm', [], 'Display', None],
  [ 0xfcd9, 'ArabicLigatureHehWithSuperscriptAlefInitialForm', [], 'Display', None],
  [ 0xfcda, 'ArabicLigatureYehWithJeemInitialForm', [], 'Display', None],
  [ 0xfcdb, 'ArabicLigatureYehWithHahInitialForm', [], 'Display', None],
  [ 0xfcdc, 'ArabicLigatureYehWithKhahInitialForm', [], 'Display', None],
  [ 0xfcdd, 'ArabicLigatureYehWithMeemInitialForm', [], 'Display', None],
  [ 0xfcde, 'ArabicLigatureYehWithHehInitialForm', [], 'Display', None],
  [ 0xfcdf, 'ArabicLigatureYehWithHamzaAboveWithMeemMedialForm', [], 'Display', None],
  [ 0xfce0, 'ArabicLigatureYehWithHamzaAboveWithHehMedialForm', [], 'Display', None],
  [ 0xfce1, 'ArabicLigatureBehWithMeemMedialForm', [], 'Display', None],
  [ 0xfce2, 'ArabicLigatureBehWithHehMedialForm', [], 'Display', None],
  [ 0xfce3, 'ArabicLigatureTehWithMeemMedialForm', [], 'Display', None],
  [ 0xfce4, 'ArabicLigatureTehWithHehMedialForm', [], 'Display', None],
  [ 0xfce5, 'ArabicLigatureThehWithMeemMedialForm', [], 'Display', None],
  [ 0xfce6, 'ArabicLigatureThehWithHehMedialForm', [], 'Display', None],
  [ 0xfce7, 'ArabicLigatureSeenWithMeemMedialForm', [], 'Display', None],
  [ 0xfce8, 'ArabicLigatureSeenWithHehMedialForm', [], 'Display', None],
  [ 0xfce9, 'ArabicLigatureSheenWithMeemMedialForm', [], 'Display', None],
  [ 0xfcea, 'ArabicLigatureSheenWithHehMedialForm', [], 'Display', None],
  [ 0xfceb, 'ArabicLigatureKafWithLamMedialForm', [], 'Display', None],
  [ 0xfcec, 'ArabicLigatureKafWithMeemMedialForm', [], 'Display', None],
  [ 0xfced, 'ArabicLigatureLamWithMeemMedialForm', [], 'Display', None],
  [ 0xfcee, 'ArabicLigatureNoonWithMeemMedialForm', [], 'Display', None],
  [ 0xfcef, 'ArabicLigatureNoonWithHehMedialForm', [], 'Display', None],
  [ 0xfcf0, 'ArabicLigatureYehWithMeemMedialForm', [], 'Display', None],
  [ 0xfcf1, 'ArabicLigatureYehWithHehMedialForm', [], 'Display', None],
  [ 0xfcf2, 'ArabicLigatureShaddaWithFathaMedialForm', [], 'Display', None],
  [ 0xfcf3, 'ArabicLigatureShaddaWithDammaMedialForm', [], 'Display', None],
  [ 0xfcf4, 'ArabicLigatureShaddaWithKasraMedialForm', [], 'Display', None],
  [ 0xfcf5, 'ArabicLigatureTahWithAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfcf6, 'ArabicLigatureTahWithYehIsolatedForm', [], 'Display', None],
  [ 0xfcf7, 'ArabicLigatureAinWithAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfcf8, 'ArabicLigatureAinWithYehIsolatedForm', [], 'Display', None],
  [ 0xfcf9, 'ArabicLigatureGhainWithAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfcfa, 'ArabicLigatureGhainWithYehIsolatedForm', [], 'Display', None],
  [ 0xfcfb, 'ArabicLigatureSeenWithAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfcfc, 'ArabicLigatureSeenWithYehIsolatedForm', [], 'Display', None],
  [ 0xfcfd, 'ArabicLigatureSheenWithAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfcfe, 'ArabicLigatureSheenWithYehIsolatedForm', [], 'Display', None],
  [ 0xfcff, 'ArabicLigatureHahWithAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfd00, 'ArabicLigatureHahWithYehIsolatedForm', [], 'Display', None],
  [ 0xfd01, 'ArabicLigatureJeemWithAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfd02, 'ArabicLigatureJeemWithYehIsolatedForm', [], 'Display', None],
  [ 0xfd03, 'ArabicLigatureKhahWithAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfd04, 'ArabicLigatureKhahWithYehIsolatedForm', [], 'Display', None],
  [ 0xfd05, 'ArabicLigatureSadWithAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfd06, 'ArabicLigatureSadWithYehIsolatedForm', [], 'Display', None],
  [ 0xfd07, 'ArabicLigatureDadWithAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfd08, 'ArabicLigatureDadWithYehIsolatedForm', [], 'Display', None],
  [ 0xfd09, 'ArabicLigatureSheenWithJeemIsolatedForm', [], 'Display', None],
  [ 0xfd0a, 'ArabicLigatureSheenWithHahIsolatedForm', [], 'Display', None],
  [ 0xfd0b, 'ArabicLigatureSheenWithKhahIsolatedForm', [], 'Display', None],
  [ 0xfd0c, 'ArabicLigatureSheenWithMeemIsolatedForm', [], 'Display', None],
  [ 0xfd0d, 'ArabicLigatureSheenWithRehIsolatedForm', [], 'Display', None],
  [ 0xfd0e, 'ArabicLigatureSeenWithRehIsolatedForm', [], 'Display', None],
  [ 0xfd0f, 'ArabicLigatureSadWithRehIsolatedForm', [], 'Display', None],
  [ 0xfd10, 'ArabicLigatureDadWithRehIsolatedForm', [], 'Display', None],
  [ 0xfd11, 'ArabicLigatureTahWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfd12, 'ArabicLigatureTahWithYehFinalForm', [], 'Display', None],
  [ 0xfd13, 'ArabicLigatureAinWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfd14, 'ArabicLigatureAinWithYehFinalForm', [], 'Display', None],
  [ 0xfd15, 'ArabicLigatureGhainWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfd16, 'ArabicLigatureGhainWithYehFinalForm', [], 'Display', None],
  [ 0xfd17, 'ArabicLigatureSeenWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfd18, 'ArabicLigatureSeenWithYehFinalForm', [], 'Display', None],
  [ 0xfd19, 'ArabicLigatureSheenWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfd1a, 'ArabicLigatureSheenWithYehFinalForm', [], 'Display', None],
  [ 0xfd1b, 'ArabicLigatureHahWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfd1c, 'ArabicLigatureHahWithYehFinalForm', [], 'Display', None],
  [ 0xfd1d, 'ArabicLigatureJeemWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfd1e, 'ArabicLigatureJeemWithYehFinalForm', [], 'Display', None],
  [ 0xfd1f, 'ArabicLigatureKhahWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfd20, 'ArabicLigatureKhahWithYehFinalForm', [], 'Display', None],
  [ 0xfd21, 'ArabicLigatureSadWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfd22, 'ArabicLigatureSadWithYehFinalForm', [], 'Display', None],
  [ 0xfd23, 'ArabicLigatureDadWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfd24, 'ArabicLigatureDadWithYehFinalForm', [], 'Display', None],
  [ 0xfd25, 'ArabicLigatureSheenWithJeemFinalForm', [], 'Display', None],
  [ 0xfd26, 'ArabicLigatureSheenWithHahFinalForm', [], 'Display', None],
  [ 0xfd27, 'ArabicLigatureSheenWithKhahFinalForm', [], 'Display', None],
  [ 0xfd28, 'ArabicLigatureSheenWithMeemFinalForm', [], 'Display', None],
  [ 0xfd29, 'ArabicLigatureSheenWithRehFinalForm', [], 'Display', None],
  [ 0xfd2a, 'ArabicLigatureSeenWithRehFinalForm', [], 'Display', None],
  [ 0xfd2b, 'ArabicLigatureSadWithRehFinalForm', [], 'Display', None],
  [ 0xfd2c, 'ArabicLigatureDadWithRehFinalForm', [], 'Display', None],
  [ 0xfd2d, 'ArabicLigatureSheenWithJeemInitialForm', [], 'Display', None],
  [ 0xfd2e, 'ArabicLigatureSheenWithHahInitialForm', [], 'Display', None],
  [ 0xfd2f, 'ArabicLigatureSheenWithKhahInitialForm', [], 'Display', None],
  [ 0xfd30, 'ArabicLigatureSheenWithMeemInitialForm', [], 'Display', None],
  [ 0xfd31, 'ArabicLigatureSeenWithHehInitialForm', [], 'Display', None],
  [ 0xfd32, 'ArabicLigatureSheenWithHehInitialForm', [], 'Display', None],
  [ 0xfd33, 'ArabicLigatureTahWithMeemInitialForm', [], 'Display', None],
  [ 0xfd34, 'ArabicLigatureSeenWithJeemMedialForm', [], 'Display', None],
  [ 0xfd35, 'ArabicLigatureSeenWithHahMedialForm', [], 'Display', None],
  [ 0xfd36, 'ArabicLigatureSeenWithKhahMedialForm', [], 'Display', None],
  [ 0xfd37, 'ArabicLigatureSheenWithJeemMedialForm', [], 'Display', None],
  [ 0xfd38, 'ArabicLigatureSheenWithHahMedialForm', [], 'Display', None],
  [ 0xfd39, 'ArabicLigatureSheenWithKhahMedialForm', [], 'Display', None],
  [ 0xfd3a, 'ArabicLigatureTahWithMeemMedialForm', [], 'Display', None],
  [ 0xfd3b, 'ArabicLigatureZahWithMeemMedialForm', [], 'Display', None],
  [ 0xfd3c, 'ArabicLigatureAlefWithFathatanFinalForm', [], 'Display', None],
  [ 0xfd3d, 'ArabicLigatureAlefWithFathatanIsolatedForm', [], 'Display', None],
  [ 0xfd3e, 'OrnateLeftParenthesis', [], 'Display', None],
  [ 0xfd3f, 'OrnateRightParenthesis', [], 'Display', None],
  [ 0xfd40, 'UnicodeCharacter0xfd40', [], 'Display', None],
  [ 0xfd41, 'UnicodeCharacter0xfd41', [], 'Display', None],
  [ 0xfd42, 'UnicodeCharacter0xfd42', [], 'Display', None],
  [ 0xfd43, 'UnicodeCharacter0xfd43', [], 'Display', None],
  [ 0xfd44, 'UnicodeCharacter0xfd44', [], 'Display', None],
  [ 0xfd45, 'UnicodeCharacter0xfd45', [], 'Display', None],
  [ 0xfd46, 'UnicodeCharacter0xfd46', [], 'Display', None],
  [ 0xfd47, 'UnicodeCharacter0xfd47', [], 'Display', None],
  [ 0xfd48, 'UnicodeCharacter0xfd48', [], 'Display', None],
  [ 0xfd49, 'UnicodeCharacter0xfd49', [], 'Display', None],
  [ 0xfd4a, 'UnicodeCharacter0xfd4a', [], 'Display', None],
  [ 0xfd4b, 'UnicodeCharacter0xfd4b', [], 'Display', None],
  [ 0xfd4c, 'UnicodeCharacter0xfd4c', [], 'Display', None],
  [ 0xfd4d, 'UnicodeCharacter0xfd4d', [], 'Display', None],
  [ 0xfd4e, 'UnicodeCharacter0xfd4e', [], 'Display', None],
  [ 0xfd4f, 'UnicodeCharacter0xfd4f', [], 'Display', None],
  [ 0xfd50, 'ArabicLigatureTehWithJeemWithMeemInitialForm', [], 'Display', None],
  [ 0xfd51, 'ArabicLigatureTehWithHahWithJeemFinalForm', [], 'Display', None],
  [ 0xfd52, 'ArabicLigatureTehWithHahWithJeemInitialForm', [], 'Display', None],
  [ 0xfd53, 'ArabicLigatureTehWithHahWithMeemInitialForm', [], 'Display', None],
  [ 0xfd54, 'ArabicLigatureTehWithKhahWithMeemInitialForm', [], 'Display', None],
  [ 0xfd55, 'ArabicLigatureTehWithMeemWithJeemInitialForm', [], 'Display', None],
  [ 0xfd56, 'ArabicLigatureTehWithMeemWithHahInitialForm', [], 'Display', None],
  [ 0xfd57, 'ArabicLigatureTehWithMeemWithKhahInitialForm', [], 'Display', None],
  [ 0xfd58, 'ArabicLigatureJeemWithMeemWithHahFinalForm', [], 'Display', None],
  [ 0xfd59, 'ArabicLigatureJeemWithMeemWithHahInitialForm', [], 'Display', None],
  [ 0xfd5a, 'ArabicLigatureHahWithMeemWithYehFinalForm', [], 'Display', None],
  [ 0xfd5b, 'ArabicLigatureHahWithMeemWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfd5c, 'ArabicLigatureSeenWithHahWithJeemInitialForm', [], 'Display', None],
  [ 0xfd5d, 'ArabicLigatureSeenWithJeemWithHahInitialForm', [], 'Display', None],
  [ 0xfd5e, 'ArabicLigatureSeenWithJeemWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfd5f, 'ArabicLigatureSeenWithMeemWithHahFinalForm', [], 'Display', None],
  [ 0xfd60, 'ArabicLigatureSeenWithMeemWithHahInitialForm', [], 'Display', None],
  [ 0xfd61, 'ArabicLigatureSeenWithMeemWithJeemInitialForm', [], 'Display', None],
  [ 0xfd62, 'ArabicLigatureSeenWithMeemWithMeemFinalForm', [], 'Display', None],
  [ 0xfd63, 'ArabicLigatureSeenWithMeemWithMeemInitialForm', [], 'Display', None],
  [ 0xfd64, 'ArabicLigatureSadWithHahWithHahFinalForm', [], 'Display', None],
  [ 0xfd65, 'ArabicLigatureSadWithHahWithHahInitialForm', [], 'Display', None],
  [ 0xfd66, 'ArabicLigatureSadWithMeemWithMeemFinalForm', [], 'Display', None],
  [ 0xfd67, 'ArabicLigatureSheenWithHahWithMeemFinalForm', [], 'Display', None],
  [ 0xfd68, 'ArabicLigatureSheenWithHahWithMeemInitialForm', [], 'Display', None],
  [ 0xfd69, 'ArabicLigatureSheenWithJeemWithYehFinalForm', [], 'Display', None],
  [ 0xfd6a, 'ArabicLigatureSheenWithMeemWithKhahFinalForm', [], 'Display', None],
  [ 0xfd6b, 'ArabicLigatureSheenWithMeemWithKhahInitialForm', [], 'Display', None],
  [ 0xfd6c, 'ArabicLigatureSheenWithMeemWithMeemFinalForm', [], 'Display', None],
  [ 0xfd6d, 'ArabicLigatureSheenWithMeemWithMeemInitialForm', [], 'Display', None],
  [ 0xfd6e, 'ArabicLigatureDadWithHahWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfd6f, 'ArabicLigatureDadWithKhahWithMeemFinalForm', [], 'Display', None],
  [ 0xfd70, 'ArabicLigatureDadWithKhahWithMeemInitialForm', [], 'Display', None],
  [ 0xfd71, 'ArabicLigatureTahWithMeemWithHahFinalForm', [], 'Display', None],
  [ 0xfd72, 'ArabicLigatureTahWithMeemWithHahInitialForm', [], 'Display', None],
  [ 0xfd73, 'ArabicLigatureTahWithMeemWithMeemInitialForm', [], 'Display', None],
  [ 0xfd74, 'ArabicLigatureTahWithMeemWithYehFinalForm', [], 'Display', None],
  [ 0xfd75, 'ArabicLigatureAinWithJeemWithMeemFinalForm', [], 'Display', None],
  [ 0xfd76, 'ArabicLigatureAinWithMeemWithMeemFinalForm', [], 'Display', None],
  [ 0xfd77, 'ArabicLigatureAinWithMeemWithMeemInitialForm', [], 'Display', None],
  [ 0xfd78, 'ArabicLigatureAinWithMeemWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfd79, 'ArabicLigatureGhainWithMeemWithMeemFinalForm', [], 'Display', None],
  [ 0xfd7a, 'ArabicLigatureGhainWithMeemWithYehFinalForm', [], 'Display', None],
  [ 0xfd7b, 'ArabicLigatureGhainWithMeemWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfd7c, 'ArabicLigatureFehWithKhahWithMeemFinalForm', [], 'Display', None],
  [ 0xfd7d, 'ArabicLigatureFehWithKhahWithMeemInitialForm', [], 'Display', None],
  [ 0xfd7e, 'ArabicLigatureQafWithMeemWithHahFinalForm', [], 'Display', None],
  [ 0xfd7f, 'ArabicLigatureQafWithMeemWithMeemFinalForm', [], 'Display', None],
  [ 0xfd80, 'ArabicLigatureLamWithHahWithMeemFinalForm', [], 'Display', None],
  [ 0xfd81, 'ArabicLigatureLamWithHahWithYehFinalForm', [], 'Display', None],
  [ 0xfd82, 'ArabicLigatureLamWithHahWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfd83, 'ArabicLigatureLamWithJeemWithJeemInitialForm', [], 'Display', None],
  [ 0xfd84, 'ArabicLigatureLamWithJeemWithJeemFinalForm', [], 'Display', None],
  [ 0xfd85, 'ArabicLigatureLamWithKhahWithMeemFinalForm', [], 'Display', None],
  [ 0xfd86, 'ArabicLigatureLamWithKhahWithMeemInitialForm', [], 'Display', None],
  [ 0xfd87, 'ArabicLigatureLamWithMeemWithHahFinalForm', [], 'Display', None],
  [ 0xfd88, 'ArabicLigatureLamWithMeemWithHahInitialForm', [], 'Display', None],
  [ 0xfd89, 'ArabicLigatureMeemWithHahWithJeemInitialForm', [], 'Display', None],
  [ 0xfd8a, 'ArabicLigatureMeemWithHahWithMeemInitialForm', [], 'Display', None],
  [ 0xfd8b, 'ArabicLigatureMeemWithHahWithYehFinalForm', [], 'Display', None],
  [ 0xfd8c, 'ArabicLigatureMeemWithJeemWithHahInitialForm', [], 'Display', None],
  [ 0xfd8d, 'ArabicLigatureMeemWithJeemWithMeemInitialForm', [], 'Display', None],
  [ 0xfd8e, 'ArabicLigatureMeemWithKhahWithJeemInitialForm', [], 'Display', None],
  [ 0xfd8f, 'ArabicLigatureMeemWithKhahWithMeemInitialForm', [], 'Display', None],
  [ 0xfd90, 'UnicodeCharacter0xfd90', [], 'Display', None],
  [ 0xfd91, 'UnicodeCharacter0xfd91', [], 'Display', None],
  [ 0xfd92, 'ArabicLigatureMeemWithJeemWithKhahInitialForm', [], 'Display', None],
  [ 0xfd93, 'ArabicLigatureHehWithMeemWithJeemInitialForm', [], 'Display', None],
  [ 0xfd94, 'ArabicLigatureHehWithMeemWithMeemInitialForm', [], 'Display', None],
  [ 0xfd95, 'ArabicLigatureNoonWithHahWithMeemInitialForm', [], 'Display', None],
  [ 0xfd96, 'ArabicLigatureNoonWithHahWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfd97, 'ArabicLigatureNoonWithJeemWithMeemFinalForm', [], 'Display', None],
  [ 0xfd98, 'ArabicLigatureNoonWithJeemWithMeemInitialForm', [], 'Display', None],
  [ 0xfd99, 'ArabicLigatureNoonWithJeemWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfd9a, 'ArabicLigatureNoonWithMeemWithYehFinalForm', [], 'Display', None],
  [ 0xfd9b, 'ArabicLigatureNoonWithMeemWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfd9c, 'ArabicLigatureYehWithMeemWithMeemFinalForm', [], 'Display', None],
  [ 0xfd9d, 'ArabicLigatureYehWithMeemWithMeemInitialForm', [], 'Display', None],
  [ 0xfd9e, 'ArabicLigatureBehWithKhahWithYehFinalForm', [], 'Display', None],
  [ 0xfd9f, 'ArabicLigatureTehWithJeemWithYehFinalForm', [], 'Display', None],
  [ 0xfda0, 'ArabicLigatureTehWithJeemWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfda1, 'ArabicLigatureTehWithKhahWithYehFinalForm', [], 'Display', None],
  [ 0xfda2, 'ArabicLigatureTehWithKhahWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfda3, 'ArabicLigatureTehWithMeemWithYehFinalForm', [], 'Display', None],
  [ 0xfda4, 'ArabicLigatureTehWithMeemWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfda5, 'ArabicLigatureJeemWithMeemWithYehFinalForm', [], 'Display', None],
  [ 0xfda6, 'ArabicLigatureJeemWithHahWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfda7, 'ArabicLigatureJeemWithMeemWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfda8, 'ArabicLigatureSeenWithKhahWithAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfda9, 'ArabicLigatureSadWithHahWithYehFinalForm', [], 'Display', None],
  [ 0xfdaa, 'ArabicLigatureSheenWithHahWithYehFinalForm', [], 'Display', None],
  [ 0xfdab, 'ArabicLigatureDadWithHahWithYehFinalForm', [], 'Display', None],
  [ 0xfdac, 'ArabicLigatureLamWithJeemWithYehFinalForm', [], 'Display', None],
  [ 0xfdad, 'ArabicLigatureLamWithMeemWithYehFinalForm', [], 'Display', None],
  [ 0xfdae, 'ArabicLigatureYehWithHahWithYehFinalForm', [], 'Display', None],
  [ 0xfdaf, 'ArabicLigatureYehWithJeemWithYehFinalForm', [], 'Display', None],
  [ 0xfdb0, 'ArabicLigatureYehWithMeemWithYehFinalForm', [], 'Display', None],
  [ 0xfdb1, 'ArabicLigatureMeemWithMeemWithYehFinalForm', [], 'Display', None],
  [ 0xfdb2, 'ArabicLigatureQafWithMeemWithYehFinalForm', [], 'Display', None],
  [ 0xfdb3, 'ArabicLigatureNoonWithHahWithYehFinalForm', [], 'Display', None],
  [ 0xfdb4, 'ArabicLigatureQafWithMeemWithHahInitialForm', [], 'Display', None],
  [ 0xfdb5, 'ArabicLigatureLamWithHahWithMeemInitialForm', [], 'Display', None],
  [ 0xfdb6, 'ArabicLigatureAinWithMeemWithYehFinalForm', [], 'Display', None],
  [ 0xfdb7, 'ArabicLigatureKafWithMeemWithYehFinalForm', [], 'Display', None],
  [ 0xfdb8, 'ArabicLigatureNoonWithJeemWithHahInitialForm', [], 'Display', None],
  [ 0xfdb9, 'ArabicLigatureMeemWithKhahWithYehFinalForm', [], 'Display', None],
  [ 0xfdba, 'ArabicLigatureLamWithJeemWithMeemInitialForm', [], 'Display', None],
  [ 0xfdbb, 'ArabicLigatureKafWithMeemWithMeemFinalForm', [], 'Display', None],
  [ 0xfdbc, 'ArabicLigatureLamWithJeemWithMeemFinalForm', [], 'Display', None],
  [ 0xfdbd, 'ArabicLigatureNoonWithJeemWithHahFinalForm', [], 'Display', None],
  [ 0xfdbe, 'ArabicLigatureJeemWithHahWithYehFinalForm', [], 'Display', None],
  [ 0xfdbf, 'ArabicLigatureHahWithJeemWithYehFinalForm', [], 'Display', None],
  [ 0xfdc0, 'ArabicLigatureMeemWithJeemWithYehFinalForm', [], 'Display', None],
  [ 0xfdc1, 'ArabicLigatureFehWithMeemWithYehFinalForm', [], 'Display', None],
  [ 0xfdc2, 'ArabicLigatureBehWithHahWithYehFinalForm', [], 'Display', None],
  [ 0xfdc3, 'ArabicLigatureKafWithMeemWithMeemInitialForm', [], 'Display', None],
  [ 0xfdc4, 'ArabicLigatureAinWithJeemWithMeemInitialForm', [], 'Display', None],
  [ 0xfdc5, 'ArabicLigatureSadWithMeemWithMeemInitialForm', [], 'Display', None],
  [ 0xfdc6, 'ArabicLigatureSeenWithKhahWithYehFinalForm', [], 'Display', None],
  [ 0xfdc7, 'ArabicLigatureNoonWithJeemWithYehFinalForm', [], 'Display', None],
  [ 0xfdc8, 'UnicodeCharacter0xfdc8', [], 'Display', None],
  [ 0xfdc9, 'UnicodeCharacter0xfdc9', [], 'Display', None],
  [ 0xfdca, 'UnicodeCharacter0xfdca', [], 'Display', None],
  [ 0xfdcb, 'UnicodeCharacter0xfdcb', [], 'Display', None],
  [ 0xfdcc, 'UnicodeCharacter0xfdcc', [], 'Display', None],
  [ 0xfdcd, 'UnicodeCharacter0xfdcd', [], 'Display', None],
  [ 0xfdce, 'UnicodeCharacter0xfdce', [], 'Display', None],
  [ 0xfdcf, 'UnicodeCharacter0xfdcf', [], 'Display', None],
  [ 0xfdd0, 'UnicodeCharacter0xfdd0', [], 'Display', None],
  [ 0xfdd1, 'UnicodeCharacter0xfdd1', [], 'Display', None],
  [ 0xfdd2, 'UnicodeCharacter0xfdd2', [], 'Display', None],
  [ 0xfdd3, 'UnicodeCharacter0xfdd3', [], 'Display', None],
  [ 0xfdd4, 'UnicodeCharacter0xfdd4', [], 'Display', None],
  [ 0xfdd5, 'UnicodeCharacter0xfdd5', [], 'Display', None],
  [ 0xfdd6, 'UnicodeCharacter0xfdd6', [], 'Display', None],
  [ 0xfdd7, 'UnicodeCharacter0xfdd7', [], 'Display', None],
  [ 0xfdd8, 'UnicodeCharacter0xfdd8', [], 'Display', None],
  [ 0xfdd9, 'UnicodeCharacter0xfdd9', [], 'Display', None],
  [ 0xfdda, 'UnicodeCharacter0xfdda', [], 'Display', None],
  [ 0xfddb, 'UnicodeCharacter0xfddb', [], 'Display', None],
  [ 0xfddc, 'UnicodeCharacter0xfddc', [], 'Display', None],
  [ 0xfddd, 'UnicodeCharacter0xfddd', [], 'Display', None],
  [ 0xfdde, 'UnicodeCharacter0xfdde', [], 'Display', None],
  [ 0xfddf, 'UnicodeCharacter0xfddf', [], 'Display', None],
  [ 0xfde0, 'UnicodeCharacter0xfde0', [], 'Display', None],
  [ 0xfde1, 'UnicodeCharacter0xfde1', [], 'Display', None],
  [ 0xfde2, 'UnicodeCharacter0xfde2', [], 'Display', None],
  [ 0xfde3, 'UnicodeCharacter0xfde3', [], 'Display', None],
  [ 0xfde4, 'UnicodeCharacter0xfde4', [], 'Display', None],
  [ 0xfde5, 'UnicodeCharacter0xfde5', [], 'Display', None],
  [ 0xfde6, 'UnicodeCharacter0xfde6', [], 'Display', None],
  [ 0xfde7, 'UnicodeCharacter0xfde7', [], 'Display', None],
  [ 0xfde8, 'UnicodeCharacter0xfde8', [], 'Display', None],
  [ 0xfde9, 'UnicodeCharacter0xfde9', [], 'Display', None],
  [ 0xfdea, 'UnicodeCharacter0xfdea', [], 'Display', None],
  [ 0xfdeb, 'UnicodeCharacter0xfdeb', [], 'Display', None],
  [ 0xfdec, 'UnicodeCharacter0xfdec', [], 'Display', None],
  [ 0xfded, 'UnicodeCharacter0xfded', [], 'Display', None],
  [ 0xfdee, 'UnicodeCharacter0xfdee', [], 'Display', None],
  [ 0xfdef, 'UnicodeCharacter0xfdef', [], 'Display', None],
  [ 0xfdf0, 'ArabicLigatureSallaUsedAsKoranicStopSignIsolatedForm', [], 'Display', None],
  [ 0xfdf1, 'ArabicLigatureQalaUsedAsKoranicStopSignIsolatedForm', [], 'Display', None],
  [ 0xfdf2, 'ArabicLigatureAllahIsolatedForm', [], 'Display', None],
  [ 0xfdf3, 'ArabicLigatureAkbarIsolatedForm', [], 'Display', None],
  [ 0xfdf4, 'ArabicLigatureMohammadIsolatedForm', [], 'Display', None],
  [ 0xfdf5, 'ArabicLigatureSalamIsolatedForm', [], 'Display', None],
  [ 0xfdf6, 'ArabicLigatureRasoulIsolatedForm', [], 'Display', None],
  [ 0xfdf7, 'ArabicLigatureAlayheIsolatedForm', [], 'Display', None],
  [ 0xfdf8, 'ArabicLigatureWasallamIsolatedForm', [], 'Display', None],
  [ 0xfdf9, 'ArabicLigatureSallaIsolatedForm', [], 'Display', None],
  [ 0xfdfa, 'ArabicLigatureSallallahouAlayheWasallam', [], 'Display', None],
  [ 0xfdfb, 'ArabicLigatureJallajalalouhou', [], 'Display', None],
  [ 0xfdfc, 'RialSign', [], 'Display', None],
  [ 0xfdfd, 'ArabicLigatureBismillahArrahmanArraheem', [], 'Display', None],
  [ 0xfdfe, 'UnicodeCharacter0xfdfe', [], 'Display', None],
  [ 0xfdff, 'UnicodeCharacter0xfdff', [], 'Display', None],
  [ 0xfe00, 'VariationSelector1', [], 'Display', None],
  [ 0xfe01, 'VariationSelector2', [], 'Display', None],
  [ 0xfe02, 'VariationSelector3', [], 'Display', None],
  [ 0xfe03, 'VariationSelector4', [], 'Display', None],
  [ 0xfe04, 'VariationSelector5', [], 'Display', None],
  [ 0xfe05, 'VariationSelector6', [], 'Display', None],
  [ 0xfe06, 'VariationSelector7', [], 'Display', None],
  [ 0xfe07, 'VariationSelector8', [], 'Display', None],
  [ 0xfe08, 'VariationSelector9', [], 'Display', None],
  [ 0xfe09, 'VariationSelector10', [], 'Display', None],
  [ 0xfe0a, 'VariationSelector11', [], 'Display', None],
  [ 0xfe0b, 'VariationSelector12', [], 'Display', None],
  [ 0xfe0c, 'VariationSelector13', [], 'Display', None],
  [ 0xfe0d, 'VariationSelector14', [], 'Display', None],
  [ 0xfe0e, 'VariationSelector15', [], 'Display', None],
  [ 0xfe0f, 'VariationSelector16', [], 'Display', None],
  [ 0xfe10, 'PresentationFormForVerticalComma', [], 'Display', None],
  [ 0xfe11, 'PresentationFormForVerticalIdeographicComma', [], 'Display', None],
  [ 0xfe12, 'PresentationFormForVerticalIdeographicFullStop', [], 'Display', None],
  [ 0xfe13, 'PresentationFormForVerticalColon', [], 'Display', None],
  [ 0xfe14, 'PresentationFormForVerticalSemicolon', [], 'Display', None],
  [ 0xfe15, 'PresentationFormForVerticalExclamationMark', [], 'Display', None],
  [ 0xfe16, 'PresentationFormForVerticalQuestionMark', [], 'Display', None],
  [ 0xfe17, 'PresentationFormForVerticalLeftWhiteLenticularBracket', [], 'Display', None],
  [ 0xfe18, 'PresentationFormForVerticalRightWhiteLenticularBrakcet', [], 'Display', None],
  [ 0xfe19, 'PresentationFormForVerticalHorizontalEllipsis', [], 'Display', None],
  [ 0xfe1a, 'UnicodeCharacter0xfe1a', [], 'Display', None],
  [ 0xfe1b, 'UnicodeCharacter0xfe1b', [], 'Display', None],
  [ 0xfe1c, 'UnicodeCharacter0xfe1c', [], 'Display', None],
  [ 0xfe1d, 'UnicodeCharacter0xfe1d', [], 'Display', None],
  [ 0xfe1e, 'UnicodeCharacter0xfe1e', [], 'Display', None],
  [ 0xfe1f, 'UnicodeCharacter0xfe1f', [], 'Display', None],
  [ 0xfe20, 'CombiningLigatureLeftHalf', [], 'Display', None],
  [ 0xfe21, 'CombiningLigatureRightHalf', [], 'Display', None],
  [ 0xfe22, 'CombiningDoubleTildeLeftHalf', [], 'Display', None],
  [ 0xfe23, 'CombiningDoubleTildeRightHalf', [], 'Display', None],
  [ 0xfe24, 'CombiningMacronLeftHalf', [], 'Display', None],
  [ 0xfe25, 'CombiningMacronRightHalf', [], 'Display', None],
  [ 0xfe26, 'CombiningConjoiningMacron', [], 'Display', None],
  [ 0xfe27, 'UnicodeCharacter0xfe27', [], 'Display', None],
  [ 0xfe28, 'UnicodeCharacter0xfe28', [], 'Display', None],
  [ 0xfe29, 'UnicodeCharacter0xfe29', [], 'Display', None],
  [ 0xfe2a, 'UnicodeCharacter0xfe2a', [], 'Display', None],
  [ 0xfe2b, 'UnicodeCharacter0xfe2b', [], 'Display', None],
  [ 0xfe2c, 'UnicodeCharacter0xfe2c', [], 'Display', None],
  [ 0xfe2d, 'UnicodeCharacter0xfe2d', [], 'Display', None],
  [ 0xfe2e, 'UnicodeCharacter0xfe2e', [], 'Display', None],
  [ 0xfe2f, 'UnicodeCharacter0xfe2f', [], 'Display', None],
  [ 0xfe30, 'PresentationFormForVerticalTwoDotLeader', [], 'Display', None],
  [ 0xfe31, 'PresentationFormForVerticalEmDash', [], 'Display', None],
  [ 0xfe32, 'PresentationFormForVerticalEnDash', [], 'Display', None],
  [ 0xfe33, 'PresentationFormForVerticalLowLine', [], 'Display', None],
  [ 0xfe34, 'PresentationFormForVerticalWavyLowLine', [], 'Display', None],
  [ 0xfe35, 'PresentationFormForVerticalLeftParenthesis', [], 'Display', None],
  [ 0xfe36, 'PresentationFormForVerticalRightParenthesis', [], 'Display', None],
  [ 0xfe37, 'PresentationFormForVerticalLeftCurlyBracket', [], 'Display', None],
  [ 0xfe38, 'PresentationFormForVerticalRightCurlyBracket', [], 'Display', None],
  [ 0xfe39, 'PresentationFormForVerticalLeftTortoiseShellBracket', [], 'Display', None],
  [ 0xfe3a, 'PresentationFormForVerticalRightTortoiseShellBracket', [], 'Display', None],
  [ 0xfe3b, 'PresentationFormForVerticalLeftBlackLenticularBracket', [], 'Display', None],
  [ 0xfe3c, 'PresentationFormForVerticalRightBlackLenticularBracket', [], 'Display', None],
  [ 0xfe3d, 'PresentationFormForVerticalLeftDoubleAngleBracket', [], 'Display', None],
  [ 0xfe3e, 'PresentationFormForVerticalRightDoubleAngleBracket', [], 'Display', None],
  [ 0xfe3f, 'PresentationFormForVerticalLeftAngleBracket', [], 'Display', None],
  [ 0xfe40, 'PresentationFormForVerticalRightAngleBracket', [], 'Display', None],
  [ 0xfe41, 'PresentationFormForVerticalLeftCornerBracket', [], 'Display', None],
  [ 0xfe42, 'PresentationFormForVerticalRightCornerBracket', [], 'Display', None],
  [ 0xfe43, 'PresentationFormForVerticalLeftWhiteCornerBracket', [], 'Display', None],
  [ 0xfe44, 'PresentationFormForVerticalRightWhiteCornerBracket', [], 'Display', None],
  [ 0xfe45, 'SesameDot', [], 'Display', None],
  [ 0xfe46, 'WhiteSesameDot', [], 'Display', None],
  [ 0xfe47, 'PresentationFormForVerticalLeftSquareBracket', [], 'Display', None],
  [ 0xfe48, 'PresentationFormForVerticalRightSquareBracket', [], 'Display', None],
  [ 0xfe49, 'DashedOverline', [], 'Display', None],
  [ 0xfe4a, 'CentrelineOverline', [], 'Display', None],
  [ 0xfe4b, 'WavyOverline', [], 'Display', None],
  [ 0xfe4c, 'DoubleWavyOverline', [], 'Display', None],
  [ 0xfe4d, 'DashedLowLine', [], 'Display', None],
  [ 0xfe4e, 'CentrelineLowLine', [], 'Display', None],
  [ 0xfe4f, 'WavyLowLine', [], 'Display', None],
  [ 0xfe50, 'SmallComma', [], 'Display', None],
  [ 0xfe51, 'SmallIdeographicComma', [], 'Display', None],
  [ 0xfe52, 'SmallFullStop', [], 'Display', None],
  [ 0xfe53, 'UnicodeCharacter0xfe53', [], 'Display', None],
  [ 0xfe54, 'SmallSemicolon', [], 'Display', None],
  [ 0xfe55, 'SmallColon', [], 'Display', None],
  [ 0xfe56, 'SmallQuestionMark', [], 'Display', None],
  [ 0xfe57, 'SmallExclamationMark', [], 'Display', None],
  [ 0xfe58, 'SmallEmDash', [], 'Display', None],
  [ 0xfe59, 'SmallLeftParenthesis', [], 'Display', None],
  [ 0xfe5a, 'SmallRightParenthesis', [], 'Display', None],
  [ 0xfe5b, 'SmallLeftCurlyBracket', [], 'Display', None],
  [ 0xfe5c, 'SmallRightCurlyBracket', [], 'Display', None],
  [ 0xfe5d, 'SmallLeftTortoiseShellBracket', [], 'Display', None],
  [ 0xfe5e, 'SmallRightTortoiseShellBracket', [], 'Display', None],
  [ 0xfe5f, 'SmallNumberSign', [], 'Display', None],
  [ 0xfe60, 'SmallAmpersand', [], 'Display', None],
  [ 0xfe61, 'SmallAsterisk', [], 'Display', None],
  [ 0xfe62, 'SmallPlusSign', [], 'Display', None],
  [ 0xfe63, 'SmallHyphenminus', [], 'Display', None],
  [ 0xfe64, 'SmallLessthanSign', [], 'Display', None],
  [ 0xfe65, 'SmallGreaterthanSign', [], 'Display', None],
  [ 0xfe66, 'SmallEqualsSign', [], 'Display', None],
  [ 0xfe67, 'UnicodeCharacter0xfe67', [], 'Display', None],
  [ 0xfe68, 'SmallReverseSolidus', [], 'Display', None],
  [ 0xfe69, 'SmallDollarSign', [], 'Display', None],
  [ 0xfe6a, 'SmallPercentSign', [], 'Display', None],
  [ 0xfe6b, 'SmallCommercialAt', [], 'Display', None],
  [ 0xfe6c, 'UnicodeCharacter0xfe6c', [], 'Display', None],
  [ 0xfe6d, 'UnicodeCharacter0xfe6d', [], 'Display', None],
  [ 0xfe6e, 'UnicodeCharacter0xfe6e', [], 'Display', None],
  [ 0xfe6f, 'UnicodeCharacter0xfe6f', [], 'Display', None],
  [ 0xfe70, 'ArabicFathatanIsolatedForm', [], 'Display', None],
  [ 0xfe71, 'ArabicTatweelWithFathatanAbove', [], 'Display', None],
  [ 0xfe72, 'ArabicDammatanIsolatedForm', [], 'Display', None],
  [ 0xfe73, 'ArabicTailFragment', [], 'Display', None],
  [ 0xfe74, 'ArabicKasratanIsolatedForm', [], 'Display', None],
  [ 0xfe75, 'UnicodeCharacter0xfe75', [], 'Display', None],
  [ 0xfe76, 'ArabicFathaIsolatedForm', [], 'Display', None],
  [ 0xfe77, 'ArabicFathaMedialForm', [], 'Display', None],
  [ 0xfe78, 'ArabicDammaIsolatedForm', [], 'Display', None],
  [ 0xfe79, 'ArabicDammaMedialForm', [], 'Display', None],
  [ 0xfe7a, 'ArabicKasraIsolatedForm', [], 'Display', None],
  [ 0xfe7b, 'ArabicKasraMedialForm', [], 'Display', None],
  [ 0xfe7c, 'ArabicShaddaIsolatedForm', [], 'Display', None],
  [ 0xfe7d, 'ArabicShaddaMedialForm', [], 'Display', None],
  [ 0xfe7e, 'ArabicSukunIsolatedForm', [], 'Display', None],
  [ 0xfe7f, 'ArabicSukunMedialForm', [], 'Display', None],
  [ 0xfe80, 'ArabicLetterHamzaIsolatedForm', [], 'Display', None],
  [ 0xfe81, 'ArabicLetterAlefWithMaddaAboveIsolatedForm', [], 'Display', None],
  [ 0xfe82, 'ArabicLetterAlefWithMaddaAboveFinalForm', [], 'Display', None],
  [ 0xfe83, 'ArabicLetterAlefWithHamzaAboveIsolatedForm', [], 'Display', None],
  [ 0xfe84, 'ArabicLetterAlefWithHamzaAboveFinalForm', [], 'Display', None],
  [ 0xfe85, 'ArabicLetterWawWithHamzaAboveIsolatedForm', [], 'Display', None],
  [ 0xfe86, 'ArabicLetterWawWithHamzaAboveFinalForm', [], 'Display', None],
  [ 0xfe87, 'ArabicLetterAlefWithHamzaBelowIsolatedForm', [], 'Display', None],
  [ 0xfe88, 'ArabicLetterAlefWithHamzaBelowFinalForm', [], 'Display', None],
  [ 0xfe89, 'ArabicLetterYehWithHamzaAboveIsolatedForm', [], 'Display', None],
  [ 0xfe8a, 'ArabicLetterYehWithHamzaAboveFinalForm', [], 'Display', None],
  [ 0xfe8b, 'ArabicLetterYehWithHamzaAboveInitialForm', [], 'Display', None],
  [ 0xfe8c, 'ArabicLetterYehWithHamzaAboveMedialForm', [], 'Display', None],
  [ 0xfe8d, 'ArabicLetterAlefIsolatedForm', [], 'Display', None],
  [ 0xfe8e, 'ArabicLetterAlefFinalForm', [], 'Display', None],
  [ 0xfe8f, 'ArabicLetterBehIsolatedForm', [], 'Display', None],
  [ 0xfe90, 'ArabicLetterBehFinalForm', [], 'Display', None],
  [ 0xfe91, 'ArabicLetterBehInitialForm', [], 'Display', None],
  [ 0xfe92, 'ArabicLetterBehMedialForm', [], 'Display', None],
  [ 0xfe93, 'ArabicLetterTehMarbutaIsolatedForm', [], 'Display', None],
  [ 0xfe94, 'ArabicLetterTehMarbutaFinalForm', [], 'Display', None],
  [ 0xfe95, 'ArabicLetterTehIsolatedForm', [], 'Display', None],
  [ 0xfe96, 'ArabicLetterTehFinalForm', [], 'Display', None],
  [ 0xfe97, 'ArabicLetterTehInitialForm', [], 'Display', None],
  [ 0xfe98, 'ArabicLetterTehMedialForm', [], 'Display', None],
  [ 0xfe99, 'ArabicLetterThehIsolatedForm', [], 'Display', None],
  [ 0xfe9a, 'ArabicLetterThehFinalForm', [], 'Display', None],
  [ 0xfe9b, 'ArabicLetterThehInitialForm', [], 'Display', None],
  [ 0xfe9c, 'ArabicLetterThehMedialForm', [], 'Display', None],
  [ 0xfe9d, 'ArabicLetterJeemIsolatedForm', [], 'Display', None],
  [ 0xfe9e, 'ArabicLetterJeemFinalForm', [], 'Display', None],
  [ 0xfe9f, 'ArabicLetterJeemInitialForm', [], 'Display', None],
  [ 0xfea0, 'ArabicLetterJeemMedialForm', [], 'Display', None],
  [ 0xfea1, 'ArabicLetterHahIsolatedForm', [], 'Display', None],
  [ 0xfea2, 'ArabicLetterHahFinalForm', [], 'Display', None],
  [ 0xfea3, 'ArabicLetterHahInitialForm', [], 'Display', None],
  [ 0xfea4, 'ArabicLetterHahMedialForm', [], 'Display', None],
  [ 0xfea5, 'ArabicLetterKhahIsolatedForm', [], 'Display', None],
  [ 0xfea6, 'ArabicLetterKhahFinalForm', [], 'Display', None],
  [ 0xfea7, 'ArabicLetterKhahInitialForm', [], 'Display', None],
  [ 0xfea8, 'ArabicLetterKhahMedialForm', [], 'Display', None],
  [ 0xfea9, 'ArabicLetterDalIsolatedForm', [], 'Display', None],
  [ 0xfeaa, 'ArabicLetterDalFinalForm', [], 'Display', None],
  [ 0xfeab, 'ArabicLetterThalIsolatedForm', [], 'Display', None],
  [ 0xfeac, 'ArabicLetterThalFinalForm', [], 'Display', None],
  [ 0xfead, 'ArabicLetterRehIsolatedForm', [], 'Display', None],
  [ 0xfeae, 'ArabicLetterRehFinalForm', [], 'Display', None],
  [ 0xfeaf, 'ArabicLetterZainIsolatedForm', [], 'Display', None],
  [ 0xfeb0, 'ArabicLetterZainFinalForm', [], 'Display', None],
  [ 0xfeb1, 'ArabicLetterSeenIsolatedForm', [], 'Display', None],
  [ 0xfeb2, 'ArabicLetterSeenFinalForm', [], 'Display', None],
  [ 0xfeb3, 'ArabicLetterSeenInitialForm', [], 'Display', None],
  [ 0xfeb4, 'ArabicLetterSeenMedialForm', [], 'Display', None],
  [ 0xfeb5, 'ArabicLetterSheenIsolatedForm', [], 'Display', None],
  [ 0xfeb6, 'ArabicLetterSheenFinalForm', [], 'Display', None],
  [ 0xfeb7, 'ArabicLetterSheenInitialForm', [], 'Display', None],
  [ 0xfeb8, 'ArabicLetterSheenMedialForm', [], 'Display', None],
  [ 0xfeb9, 'ArabicLetterSadIsolatedForm', [], 'Display', None],
  [ 0xfeba, 'ArabicLetterSadFinalForm', [], 'Display', None],
  [ 0xfebb, 'ArabicLetterSadInitialForm', [], 'Display', None],
  [ 0xfebc, 'ArabicLetterSadMedialForm', [], 'Display', None],
  [ 0xfebd, 'ArabicLetterDadIsolatedForm', [], 'Display', None],
  [ 0xfebe, 'ArabicLetterDadFinalForm', [], 'Display', None],
  [ 0xfebf, 'ArabicLetterDadInitialForm', [], 'Display', None],
  [ 0xfec0, 'ArabicLetterDadMedialForm', [], 'Display', None],
  [ 0xfec1, 'ArabicLetterTahIsolatedForm', [], 'Display', None],
  [ 0xfec2, 'ArabicLetterTahFinalForm', [], 'Display', None],
  [ 0xfec3, 'ArabicLetterTahInitialForm', [], 'Display', None],
  [ 0xfec4, 'ArabicLetterTahMedialForm', [], 'Display', None],
  [ 0xfec5, 'ArabicLetterZahIsolatedForm', [], 'Display', None],
  [ 0xfec6, 'ArabicLetterZahFinalForm', [], 'Display', None],
  [ 0xfec7, 'ArabicLetterZahInitialForm', [], 'Display', None],
  [ 0xfec8, 'ArabicLetterZahMedialForm', [], 'Display', None],
  [ 0xfec9, 'ArabicLetterAinIsolatedForm', [], 'Display', None],
  [ 0xfeca, 'ArabicLetterAinFinalForm', [], 'Display', None],
  [ 0xfecb, 'ArabicLetterAinInitialForm', [], 'Display', None],
  [ 0xfecc, 'ArabicLetterAinMedialForm', [], 'Display', None],
  [ 0xfecd, 'ArabicLetterGhainIsolatedForm', [], 'Display', None],
  [ 0xfece, 'ArabicLetterGhainFinalForm', [], 'Display', None],
  [ 0xfecf, 'ArabicLetterGhainInitialForm', [], 'Display', None],
  [ 0xfed0, 'ArabicLetterGhainMedialForm', [], 'Display', None],
  [ 0xfed1, 'ArabicLetterFehIsolatedForm', [], 'Display', None],
  [ 0xfed2, 'ArabicLetterFehFinalForm', [], 'Display', None],
  [ 0xfed3, 'ArabicLetterFehInitialForm', [], 'Display', None],
  [ 0xfed4, 'ArabicLetterFehMedialForm', [], 'Display', None],
  [ 0xfed5, 'ArabicLetterQafIsolatedForm', [], 'Display', None],
  [ 0xfed6, 'ArabicLetterQafFinalForm', [], 'Display', None],
  [ 0xfed7, 'ArabicLetterQafInitialForm', [], 'Display', None],
  [ 0xfed8, 'ArabicLetterQafMedialForm', [], 'Display', None],
  [ 0xfed9, 'ArabicLetterKafIsolatedForm', [], 'Display', None],
  [ 0xfeda, 'ArabicLetterKafFinalForm', [], 'Display', None],
  [ 0xfedb, 'ArabicLetterKafInitialForm', [], 'Display', None],
  [ 0xfedc, 'ArabicLetterKafMedialForm', [], 'Display', None],
  [ 0xfedd, 'ArabicLetterLamIsolatedForm', [], 'Display', None],
  [ 0xfede, 'ArabicLetterLamFinalForm', [], 'Display', None],
  [ 0xfedf, 'ArabicLetterLamInitialForm', [], 'Display', None],
  [ 0xfee0, 'ArabicLetterLamMedialForm', [], 'Display', None],
  [ 0xfee1, 'ArabicLetterMeemIsolatedForm', [], 'Display', None],
  [ 0xfee2, 'ArabicLetterMeemFinalForm', [], 'Display', None],
  [ 0xfee3, 'ArabicLetterMeemInitialForm', [], 'Display', None],
  [ 0xfee4, 'ArabicLetterMeemMedialForm', [], 'Display', None],
  [ 0xfee5, 'ArabicLetterNoonIsolatedForm', [], 'Display', None],
  [ 0xfee6, 'ArabicLetterNoonFinalForm', [], 'Display', None],
  [ 0xfee7, 'ArabicLetterNoonInitialForm', [], 'Display', None],
  [ 0xfee8, 'ArabicLetterNoonMedialForm', [], 'Display', None],
  [ 0xfee9, 'ArabicLetterHehIsolatedForm', [], 'Display', None],
  [ 0xfeea, 'ArabicLetterHehFinalForm', [], 'Display', None],
  [ 0xfeeb, 'ArabicLetterHehInitialForm', [], 'Display', None],
  [ 0xfeec, 'ArabicLetterHehMedialForm', [], 'Display', None],
  [ 0xfeed, 'ArabicLetterWawIsolatedForm', [], 'Display', None],
  [ 0xfeee, 'ArabicLetterWawFinalForm', [], 'Display', None],
  [ 0xfeef, 'ArabicLetterAlefMaksuraIsolatedForm', [], 'Display', None],
  [ 0xfef0, 'ArabicLetterAlefMaksuraFinalForm', [], 'Display', None],
  [ 0xfef1, 'ArabicLetterYehIsolatedForm', [], 'Display', None],
  [ 0xfef2, 'ArabicLetterYehFinalForm', [], 'Display', None],
  [ 0xfef3, 'ArabicLetterYehInitialForm', [], 'Display', None],
  [ 0xfef4, 'ArabicLetterYehMedialForm', [], 'Display', None],
  [ 0xfef5, 'ArabicLigatureLamWithAlefWithMaddaAboveIsolatedForm', [], 'Display', None],
  [ 0xfef6, 'ArabicLigatureLamWithAlefWithMaddaAboveFinalForm', [], 'Display', None],
  [ 0xfef7, 'ArabicLigatureLamWithAlefWithHamzaAboveIsolatedForm', [], 'Display', None],
  [ 0xfef8, 'ArabicLigatureLamWithAlefWithHamzaAboveFinalForm', [], 'Display', None],
  [ 0xfef9, 'ArabicLigatureLamWithAlefWithHamzaBelowIsolatedForm', [], 'Display', None],
  [ 0xfefa, 'ArabicLigatureLamWithAlefWithHamzaBelowFinalForm', [], 'Display', None],
  [ 0xfefb, 'ArabicLigatureLamWithAlefIsolatedForm', [], 'Display', None],
  [ 0xfefc, 'ArabicLigatureLamWithAlefFinalForm', [], 'Display', None],
  [ 0xfefd, 'UnicodeCharacter0xfefd', [], 'Display', None],
  [ 0xfefe, 'UnicodeCharacter0xfefe', [], 'Display', None],
  [ 0xfeff, 'ZeroWidthNobreakSpace', [], 'Display', None],
  [ 0xff00, 'UnicodeCharacter0xff00', [], 'Display', None],
  [ 0xff01, 'FullwidthExclamationMark', [], 'Display', None],
  [ 0xff02, 'FullwidthQuotationMark', [], 'Display', None],
  [ 0xff03, 'FullwidthNumberSign', [], 'Display', None],
  [ 0xff04, 'FullwidthDollarSign', [], 'Display', None],
  [ 0xff05, 'FullwidthPercentSign', [], 'Display', None],
  [ 0xff06, 'FullwidthAmpersand', [], 'Display', None],
  [ 0xff07, 'FullwidthApostrophe', [], 'Display', None],
  [ 0xff08, 'FullwidthLeftParenthesis', [], 'Display', None],
  [ 0xff09, 'FullwidthRightParenthesis', [], 'Display', None],
  [ 0xff0a, 'FullwidthAsterisk', [], 'Display', None],
  [ 0xff0b, 'FullwidthPlusSign', [], 'Display', None],
  [ 0xff0c, 'FullwidthComma', [], 'Display', None],
  [ 0xff0d, 'FullwidthHyphenminus', [], 'Display', None],
  [ 0xff0e, 'FullwidthFullStop', [], 'Display', None],
  [ 0xff0f, 'FullwidthSolidus', [], 'Display', None],
  [ 0xff10, 'FullwidthDigitZero', [], 'Display', None],
  [ 0xff11, 'FullwidthDigitOne', [], 'Display', None],
  [ 0xff12, 'FullwidthDigitTwo', [], 'Display', None],
  [ 0xff13, 'FullwidthDigitThree', [], 'Display', None],
  [ 0xff14, 'FullwidthDigitFour', [], 'Display', None],
  [ 0xff15, 'FullwidthDigitFive', [], 'Display', None],
  [ 0xff16, 'FullwidthDigitSix', [], 'Display', None],
  [ 0xff17, 'FullwidthDigitSeven', [], 'Display', None],
  [ 0xff18, 'FullwidthDigitEight', [], 'Display', None],
  [ 0xff19, 'FullwidthDigitNine', [], 'Display', None],
  [ 0xff1a, 'FullwidthColon', [], 'Display', None],
  [ 0xff1b, 'FullwidthSemicolon', [], 'Display', None],
  [ 0xff1c, 'FullwidthLessthanSign', [], 'Display', None],
  [ 0xff1d, 'FullwidthEqualsSign', [], 'Display', None],
  [ 0xff1e, 'FullwidthGreaterthanSign', [], 'Display', None],
  [ 0xff1f, 'FullwidthQuestionMark', [], 'Display', None],
  [ 0xff20, 'FullwidthCommercialAt', [], 'Display', None],
  [ 0xff21, 'FullwidthLatinCapitalLetterA', [], 'Display', None],
  [ 0xff22, 'FullwidthLatinCapitalLetterB', [], 'Display', None],
  [ 0xff23, 'FullwidthLatinCapitalLetterC', [], 'Display', None],
  [ 0xff24, 'FullwidthLatinCapitalLetterD', [], 'Display', None],
  [ 0xff25, 'FullwidthLatinCapitalLetterE', [], 'Display', None],
  [ 0xff26, 'FullwidthLatinCapitalLetterF', [], 'Display', None],
  [ 0xff27, 'FullwidthLatinCapitalLetterG', [], 'Display', None],
  [ 0xff28, 'FullwidthLatinCapitalLetterH', [], 'Display', None],
  [ 0xff29, 'FullwidthLatinCapitalLetterI', [], 'Display', None],
  [ 0xff2a, 'FullwidthLatinCapitalLetterJ', [], 'Display', None],
  [ 0xff2b, 'FullwidthLatinCapitalLetterK', [], 'Display', None],
  [ 0xff2c, 'FullwidthLatinCapitalLetterL', [], 'Display', None],
  [ 0xff2d, 'FullwidthLatinCapitalLetterM', [], 'Display', None],
  [ 0xff2e, 'FullwidthLatinCapitalLetterN', [], 'Display', None],
  [ 0xff2f, 'FullwidthLatinCapitalLetterO', [], 'Display', None],
  [ 0xff30, 'FullwidthLatinCapitalLetterP', [], 'Display', None],
  [ 0xff31, 'FullwidthLatinCapitalLetterQ', [], 'Display', None],
  [ 0xff32, 'FullwidthLatinCapitalLetterR', [], 'Display', None],
  [ 0xff33, 'FullwidthLatinCapitalLetterS', [], 'Display', None],
  [ 0xff34, 'FullwidthLatinCapitalLetterT', [], 'Display', None],
  [ 0xff35, 'FullwidthLatinCapitalLetterU', [], 'Display', None],
  [ 0xff36, 'FullwidthLatinCapitalLetterV', [], 'Display', None],
  [ 0xff37, 'FullwidthLatinCapitalLetterW', [], 'Display', None],
  [ 0xff38, 'FullwidthLatinCapitalLetterX', [], 'Display', None],
  [ 0xff39, 'FullwidthLatinCapitalLetterY', [], 'Display', None],
  [ 0xff3a, 'FullwidthLatinCapitalLetterZ', [], 'Display', None],
  [ 0xff3b, 'FullwidthLeftSquareBracket', [], 'Display', None],
  [ 0xff3c, 'FullwidthReverseSolidus', [], 'Display', None],
  [ 0xff3d, 'FullwidthRightSquareBracket', [], 'Display', None],
  [ 0xff3e, 'FullwidthCircumflexAccent', [], 'Display', None],
  [ 0xff3f, 'FullwidthLowLine', [], 'Display', None],
  [ 0xff40, 'FullwidthGraveAccent', [], 'Display', None],
  [ 0xff41, 'FullwidthLatinSmallLetterA', [], 'Display', None],
  [ 0xff42, 'FullwidthLatinSmallLetterB', [], 'Display', None],
  [ 0xff43, 'FullwidthLatinSmallLetterC', [], 'Display', None],
  [ 0xff44, 'FullwidthLatinSmallLetterD', [], 'Display', None],
  [ 0xff45, 'FullwidthLatinSmallLetterE', [], 'Display', None],
  [ 0xff46, 'FullwidthLatinSmallLetterF', [], 'Display', None],
  [ 0xff47, 'FullwidthLatinSmallLetterG', [], 'Display', None],
  [ 0xff48, 'FullwidthLatinSmallLetterH', [], 'Display', None],
  [ 0xff49, 'FullwidthLatinSmallLetterI', [], 'Display', None],
  [ 0xff4a, 'FullwidthLatinSmallLetterJ', [], 'Display', None],
  [ 0xff4b, 'FullwidthLatinSmallLetterK', [], 'Display', None],
  [ 0xff4c, 'FullwidthLatinSmallLetterL', [], 'Display', None],
  [ 0xff4d, 'FullwidthLatinSmallLetterM', [], 'Display', None],
  [ 0xff4e, 'FullwidthLatinSmallLetterN', [], 'Display', None],
  [ 0xff4f, 'FullwidthLatinSmallLetterO', [], 'Display', None],
  [ 0xff50, 'FullwidthLatinSmallLetterP', [], 'Display', None],
  [ 0xff51, 'FullwidthLatinSmallLetterQ', [], 'Display', None],
  [ 0xff52, 'FullwidthLatinSmallLetterR', [], 'Display', None],
  [ 0xff53, 'FullwidthLatinSmallLetterS', [], 'Display', None],
  [ 0xff54, 'FullwidthLatinSmallLetterT', [], 'Display', None],
  [ 0xff55, 'FullwidthLatinSmallLetterU', [], 'Display', None],
  [ 0xff56, 'FullwidthLatinSmallLetterV', [], 'Display', None],
  [ 0xff57, 'FullwidthLatinSmallLetterW', [], 'Display', None],
  [ 0xff58, 'FullwidthLatinSmallLetterX', [], 'Display', None],
  [ 0xff59, 'FullwidthLatinSmallLetterY', [], 'Display', None],
  [ 0xff5a, 'FullwidthLatinSmallLetterZ', [], 'Display', None],
  [ 0xff5b, 'FullwidthLeftCurlyBracket', [], 'Display', None],
  [ 0xff5c, 'FullwidthVerticalLine', [], 'Display', None],
  [ 0xff5d, 'FullwidthRightCurlyBracket', [], 'Display', None],
  [ 0xff5e, 'FullwidthTilde', [], 'Display', None],
  [ 0xff5f, 'FullwidthLeftWhiteParenthesis', [], 'Display', None],
  [ 0xff60, 'FullwidthRightWhiteParenthesis', [], 'Display', None],
  [ 0xff61, 'HalfwidthIdeographicFullStop', [], 'Display', None],
  [ 0xff62, 'HalfwidthLeftCornerBracket', [], 'Display', None],
  [ 0xff63, 'HalfwidthRightCornerBracket', [], 'Display', None],
  [ 0xff64, 'HalfwidthIdeographicComma', [], 'Display', None],
  [ 0xff65, 'HalfwidthKatakanaMiddleDot', [], 'Display', None],
  [ 0xff66, 'HalfwidthKatakanaLetterWo', [], 'Display', None],
  [ 0xff67, 'HalfwidthKatakanaLetterSmallA', [], 'Display', None],
  [ 0xff68, 'HalfwidthKatakanaLetterSmallI', [], 'Display', None],
  [ 0xff69, 'HalfwidthKatakanaLetterSmallU', [], 'Display', None],
  [ 0xff6a, 'HalfwidthKatakanaLetterSmallE', [], 'Display', None],
  [ 0xff6b, 'HalfwidthKatakanaLetterSmallO', [], 'Display', None],
  [ 0xff6c, 'HalfwidthKatakanaLetterSmallYa', [], 'Display', None],
  [ 0xff6d, 'HalfwidthKatakanaLetterSmallYu', [], 'Display', None],
  [ 0xff6e, 'HalfwidthKatakanaLetterSmallYo', [], 'Display', None],
  [ 0xff6f, 'HalfwidthKatakanaLetterSmallTu', [], 'Display', None],
  [ 0xff70, 'HalfwidthKatakanahiraganaProlongedSoundMark', [], 'Display', None],
  [ 0xff71, 'HalfwidthKatakanaLetterA', [], 'Display', None],
  [ 0xff72, 'HalfwidthKatakanaLetterI', [], 'Display', None],
  [ 0xff73, 'HalfwidthKatakanaLetterU', [], 'Display', None],
  [ 0xff74, 'HalfwidthKatakanaLetterE', [], 'Display', None],
  [ 0xff75, 'HalfwidthKatakanaLetterO', [], 'Display', None],
  [ 0xff76, 'HalfwidthKatakanaLetterKa', [], 'Display', None],
  [ 0xff77, 'HalfwidthKatakanaLetterKi', [], 'Display', None],
  [ 0xff78, 'HalfwidthKatakanaLetterKu', [], 'Display', None],
  [ 0xff79, 'HalfwidthKatakanaLetterKe', [], 'Display', None],
  [ 0xff7a, 'HalfwidthKatakanaLetterKo', [], 'Display', None],
  [ 0xff7b, 'HalfwidthKatakanaLetterSa', [], 'Display', None],
  [ 0xff7c, 'HalfwidthKatakanaLetterSi', [], 'Display', None],
  [ 0xff7d, 'HalfwidthKatakanaLetterSu', [], 'Display', None],
  [ 0xff7e, 'HalfwidthKatakanaLetterSe', [], 'Display', None],
  [ 0xff7f, 'HalfwidthKatakanaLetterSo', [], 'Display', None],
  [ 0xff80, 'HalfwidthKatakanaLetterTa', [], 'Display', None],
  [ 0xff81, 'HalfwidthKatakanaLetterTi', [], 'Display', None],
  [ 0xff82, 'HalfwidthKatakanaLetterTu', [], 'Display', None],
  [ 0xff83, 'HalfwidthKatakanaLetterTe', [], 'Display', None],
  [ 0xff84, 'HalfwidthKatakanaLetterTo', [], 'Display', None],
  [ 0xff85, 'HalfwidthKatakanaLetterNa', [], 'Display', None],
  [ 0xff86, 'HalfwidthKatakanaLetterNi', [], 'Display', None],
  [ 0xff87, 'HalfwidthKatakanaLetterNu', [], 'Display', None],
  [ 0xff88, 'HalfwidthKatakanaLetterNe', [], 'Display', None],
  [ 0xff89, 'HalfwidthKatakanaLetterNo', [], 'Display', None],
  [ 0xff8a, 'HalfwidthKatakanaLetterHa', [], 'Display', None],
  [ 0xff8b, 'HalfwidthKatakanaLetterHi', [], 'Display', None],
  [ 0xff8c, 'HalfwidthKatakanaLetterHu', [], 'Display', None],
  [ 0xff8d, 'HalfwidthKatakanaLetterHe', [], 'Display', None],
  [ 0xff8e, 'HalfwidthKatakanaLetterHo', [], 'Display', None],
  [ 0xff8f, 'HalfwidthKatakanaLetterMa', [], 'Display', None],
  [ 0xff90, 'HalfwidthKatakanaLetterMi', [], 'Display', None],
  [ 0xff91, 'HalfwidthKatakanaLetterMu', [], 'Display', None],
  [ 0xff92, 'HalfwidthKatakanaLetterMe', [], 'Display', None],
  [ 0xff93, 'HalfwidthKatakanaLetterMo', [], 'Display', None],
  [ 0xff94, 'HalfwidthKatakanaLetterYa', [], 'Display', None],
  [ 0xff95, 'HalfwidthKatakanaLetterYu', [], 'Display', None],
  [ 0xff96, 'HalfwidthKatakanaLetterYo', [], 'Display', None],
  [ 0xff97, 'HalfwidthKatakanaLetterRa', [], 'Display', None],
  [ 0xff98, 'HalfwidthKatakanaLetterRi', [], 'Display', None],
  [ 0xff99, 'HalfwidthKatakanaLetterRu', [], 'Display', None],
  [ 0xff9a, 'HalfwidthKatakanaLetterRe', [], 'Display', None],
  [ 0xff9b, 'HalfwidthKatakanaLetterRo', [], 'Display', None],
  [ 0xff9c, 'HalfwidthKatakanaLetterWa', [], 'Display', None],
  [ 0xff9d, 'HalfwidthKatakanaLetterN', [], 'Display', None],
  [ 0xff9e, 'HalfwidthKatakanaVoicedSoundMark', [], 'Display', None],
  [ 0xff9f, 'HalfwidthKatakanaSemivoicedSoundMark', [], 'Display', None],
  [ 0xffa0, 'HalfwidthHangulFiller', [], 'Display', None],
  [ 0xffa1, 'HalfwidthHangulLetterKiyeok', [], 'Display', None],
  [ 0xffa2, 'HalfwidthHangulLetterSsangkiyeok', [], 'Display', None],
  [ 0xffa3, 'HalfwidthHangulLetterKiyeoksios', [], 'Display', None],
  [ 0xffa4, 'HalfwidthHangulLetterNieun', [], 'Display', None],
  [ 0xffa5, 'HalfwidthHangulLetterNieuncieuc', [], 'Display', None],
  [ 0xffa6, 'HalfwidthHangulLetterNieunhieuh', [], 'Display', None],
  [ 0xffa7, 'HalfwidthHangulLetterTikeut', [], 'Display', None],
  [ 0xffa8, 'HalfwidthHangulLetterSsangtikeut', [], 'Display', None],
  [ 0xffa9, 'HalfwidthHangulLetterRieul', [], 'Display', None],
  [ 0xffaa, 'HalfwidthHangulLetterRieulkiyeok', [], 'Display', None],
  [ 0xffab, 'HalfwidthHangulLetterRieulmieum', [], 'Display', None],
  [ 0xffac, 'HalfwidthHangulLetterRieulpieup', [], 'Display', None],
  [ 0xffad, 'HalfwidthHangulLetterRieulsios', [], 'Display', None],
  [ 0xffae, 'HalfwidthHangulLetterRieulthieuth', [], 'Display', None],
  [ 0xffaf, 'HalfwidthHangulLetterRieulphieuph', [], 'Display', None],
  [ 0xffb0, 'HalfwidthHangulLetterRieulhieuh', [], 'Display', None],
  [ 0xffb1, 'HalfwidthHangulLetterMieum', [], 'Display', None],
  [ 0xffb2, 'HalfwidthHangulLetterPieup', [], 'Display', None],
  [ 0xffb3, 'HalfwidthHangulLetterSsangpieup', [], 'Display', None],
  [ 0xffb4, 'HalfwidthHangulLetterPieupsios', [], 'Display', None],
  [ 0xffb5, 'HalfwidthHangulLetterSios', [], 'Display', None],
  [ 0xffb6, 'HalfwidthHangulLetterSsangsios', [], 'Display', None],
  [ 0xffb7, 'HalfwidthHangulLetterIeung', [], 'Display', None],
  [ 0xffb8, 'HalfwidthHangulLetterCieuc', [], 'Display', None],
  [ 0xffb9, 'HalfwidthHangulLetterSsangcieuc', [], 'Display', None],
  [ 0xffba, 'HalfwidthHangulLetterChieuch', [], 'Display', None],
  [ 0xffbb, 'HalfwidthHangulLetterKhieukh', [], 'Display', None],
  [ 0xffbc, 'HalfwidthHangulLetterThieuth', [], 'Display', None],
  [ 0xffbd, 'HalfwidthHangulLetterPhieuph', [], 'Display', None],
  [ 0xffbe, 'HalfwidthHangulLetterHieuh', [], 'Display', None],
  [ 0xffbf, 'UnicodeCharacter0xffbf', [], 'Display', None],
  [ 0xffc0, 'UnicodeCharacter0xffc0', [], 'Display', None],
  [ 0xffc1, 'UnicodeCharacter0xffc1', [], 'Display', None],
  [ 0xffc2, 'HalfwidthHangulLetterA', [], 'Display', None],
  [ 0xffc3, 'HalfwidthHangulLetterAe', [], 'Display', None],
  [ 0xffc4, 'HalfwidthHangulLetterYa', [], 'Display', None],
  [ 0xffc5, 'HalfwidthHangulLetterYae', [], 'Display', None],
  [ 0xffc6, 'HalfwidthHangulLetterEo', [], 'Display', None],
  [ 0xffc7, 'HalfwidthHangulLetterE', [], 'Display', None],
  [ 0xffc8, 'UnicodeCharacter0xffc8', [], 'Display', None],
  [ 0xffc9, 'UnicodeCharacter0xffc9', [], 'Display', None],
  [ 0xffca, 'HalfwidthHangulLetterYeo', [], 'Display', None],
  [ 0xffcb, 'HalfwidthHangulLetterYe', [], 'Display', None],
  [ 0xffcc, 'HalfwidthHangulLetterO', [], 'Display', None],
  [ 0xffcd, 'HalfwidthHangulLetterWa', [], 'Display', None],
  [ 0xffce, 'HalfwidthHangulLetterWae', [], 'Display', None],
  [ 0xffcf, 'HalfwidthHangulLetterOe', [], 'Display', None],
  [ 0xffd0, 'UnicodeCharacter0xffd0', [], 'Display', None],
  [ 0xffd1, 'UnicodeCharacter0xffd1', [], 'Display', None],
  [ 0xffd2, 'HalfwidthHangulLetterYo', [], 'Display', None],
  [ 0xffd3, 'HalfwidthHangulLetterU', [], 'Display', None],
  [ 0xffd4, 'HalfwidthHangulLetterWeo', [], 'Display', None],
  [ 0xffd5, 'HalfwidthHangulLetterWe', [], 'Display', None],
  [ 0xffd6, 'HalfwidthHangulLetterWi', [], 'Display', None],
  [ 0xffd7, 'HalfwidthHangulLetterYu', [], 'Display', None],
  [ 0xffd8, 'UnicodeCharacter0xffd8', [], 'Display', None],
  [ 0xffd9, 'UnicodeCharacter0xffd9', [], 'Display', None],
  [ 0xffda, 'HalfwidthHangulLetterEu', [], 'Display', None],
  [ 0xffdb, 'HalfwidthHangulLetterYi', [], 'Display', None],
  [ 0xffdc, 'HalfwidthHangulLetterI', [], 'Display', None],
  [ 0xffdd, 'UnicodeCharacter0xffdd', [], 'Display', None],
  [ 0xffde, 'UnicodeCharacter0xffde', [], 'Display', None],
  [ 0xffdf, 'UnicodeCharacter0xffdf', [], 'Display', None],
  [ 0xffe0, 'FullwidthCentSign', [], 'Display', None],
  [ 0xffe1, 'FullwidthPoundSign', [], 'Display', None],
  [ 0xffe2, 'FullwidthNotSign', [], 'Display', None],
  [ 0xffe3, 'FullwidthMacron', [], 'Display', None],
  [ 0xffe4, 'FullwidthBrokenBar', [], 'Display', None],
  [ 0xffe5, 'FullwidthYenSign', [], 'Display', None],
  [ 0xffe6, 'FullwidthWonSign', [], 'Display', None],
  [ 0xffe7, 'UnicodeCharacter0xffe7', [], 'Display', None],
  [ 0xffe8, 'HalfwidthFormsLightVertical', [], 'Display', None],
  [ 0xffe9, 'HalfwidthLeftwardsArrow', [], 'Display', None],
  [ 0xffea, 'HalfwidthUpwardsArrow', [], 'Display', None],
  [ 0xffeb, 'HalfwidthRightwardsArrow', [], 'Display', None],
  [ 0xffec, 'HalfwidthDownwardsArrow', [], 'Display', None],
  [ 0xffed, 'HalfwidthBlackSquare', [], 'Display', None],
  [ 0xffee, 'HalfwidthWhiteCircle', [], 'Display', None],
  [ 0xffef, 'UnicodeCharacter0xffef', [], 'Display', None],
  [ 0xfff0, 'UnicodeCharacter0xfff0', [], 'Display', None],
  [ 0xfff1, 'UnicodeCharacter0xfff1', [], 'Display', None],
  [ 0xfff2, 'UnicodeCharacter0xfff2', [], 'Display', None],
  [ 0xfff3, 'UnicodeCharacter0xfff3', [], 'Display', None],
  [ 0xfff4, 'UnicodeCharacter0xfff4', [], 'Display', None],
  [ 0xfff5, 'UnicodeCharacter0xfff5', [], 'Display', None],
  [ 0xfff6, 'UnicodeCharacter0xfff6', [], 'Display', None],
  [ 0xfff7, 'UnicodeCharacter0xfff7', [], 'Display', None],
  [ 0xfff8, 'UnicodeCharacter0xfff8', [], 'Display', None],
  [ 0xfff9, 'InterlinearAnnotationAnchor', [], 'Display', None],
  [ 0xfffa, 'InterlinearAnnotationSeparator', [], 'Display', None],
  [ 0xfffb, 'InterlinearAnnotationTerminator', [], 'Display', None],
  [ 0xfffc, 'ObjectReplacementCharacter', [], 'Display', None],
  [ 0xfffd, 'ReplacementCharacter', [], 'Display', None],
  [ 0xfffe, 'UnicodeCharacter0xfffe', [], 'Display', None],
  [ 0xffff, 'UnicodeCharacter0xffff', [], 'Display', None],
]
