# This file is autogenerated from the Table...
# Make modifications to symbolConversionTable.py, then 
#   regenerate this file with symbolConversionDictWriter.py 

infixOperatorNames = [
'MultiplicationSign',
'DivisionSign',
'FractionSlash',
'ElementOf',
'NotAnElementOf',
'DotPlus',
'RingOperator',
'BulletOperator',
'Intersection',
'Union',
'DotMinus',
'CircledPlus',
'CircledMinus',
'CircledTimes',
'CircledDivisionSlash',
'CircledDotOperator',
'DotOperator',
'StarOperator',
'DivisionTimes',
'mult',
'multiply',
'div',
'division',
'divisionsign',
'frac',
'/',
'elementof',
'elem',
'el',
'notelementof',
'notelem',
'.+',
'dotplus',
'ring',
'bullet',
'intersect',
'inter',
'union',
'un',
'.-',
'dotminus',
'c+',
'c-',
'c*',
'c/',
'c.',
'dot',
'.',
'star',
'/*']

NameToInfixAstSubstitute = {
'el': '|',
'CircledTimes': '*',
'c*': '*',
'Union': '|',
'FractionSlash': '*',
'/': '*',
'DotMinus': '+',
'c/': '*',
'ring': '*',
'DivisionSign': '*',
'RingOperator': '*',
'.+': '+',
'CircledMinus': '+',
'.-': '+',
'MultiplicationSign': '*',
'union': '|',
'elementof': '|',
'DotPlus': '+',
'.': '*',
'divisionsign': '*',
'intersect': '|',
'Intersection': '|',
'BulletOperator': '*',
'notelem': '|',
'CircledDivisionSlash': '*',
'division': '*',
'frac': '*',
'c.': '*',
'c-': '+',
'DivisionTimes': '*',
'DotOperator': '*',
'dotminus': '+',
'multiply': '*',
'StarOperator': '*',
'mult': '*',
'CircledDotOperator': '*',
'/*': '*',
'bullet': '*',
'ElementOf': '|',
'CircledPlus': '+',
'elem': '|',
'dotplus': '+',
'NotAnElementOf': '|',
'c+': '+',
'un': '|',
'dot': '*',
'div': '*',
'inter': '|',
'star': '*',
'notelementof': '|'}

ToName = {
u'\u2483': 'ParenthesizedNumberSixteen',
u'\uc302': 'HangulSyllableSsalm',
u'\ue785': 'UnicodeCharacter0xe785',
u'\u4906': 'CjkUnifiedIdeograph4906',
u'\u6d89': 'CjkUnifiedIdeograph6d89',
u'\u920c': 'CjkUnifiedIdeograph920c',
u'\ub68f': 'HangulSyllableDdyolh',
u'\u1810': 'MongolianDigitZero',
u'\u3c93': 'CjkUnifiedIdeograph3c93',
u'\udb12': 'UnicodeCharacter0xdb12',
u'\uff95': 'HalfwidthKatakanaLetterYu',
u'\u6116': 'CjkUnifiedIdeograph6116',
u'\u8599': 'CjkUnifiedIdeograph8599',
u'\u0b9d': 'UnicodeCharacter0xb9d',
u'\uaa1c': 'ChamLetterPha',
u'\uce9f': 'HangulSyllableKaelh',
u'\u3020': 'PostalMarkFace',
u'\u54a3': 'CjkUnifiedIdeograph54a3',
u'\uf322': 'UnicodeCharacter0xf322',
u'\u7926': 'CjkUnifiedIdeograph7926',
u'\u9da9': 'CjkUnifiedIdeograph9da9',
u'\u23ad': 'RightCurlyBracketLowerHook',
u'\uc22c': 'HangulSyllableSuss',
u'\ue6af': 'UnicodeCharacter0xe6af',
u'\u4830': 'CjkUnifiedIdeograph4830',
u'\u6cb3': 'CjkUnifiedIdeograph6cb3',
u'\u9136': 'CjkUnifiedIdeograph9136',
u'\ub5b9': 'HangulSyllableDdeot',
u'\u173a': 'UnicodeCharacter0x173a',
u'\u3bbd': 'CjkUnifiedIdeograph3bbd',
u'\uda3c': 'UnicodeCharacter0xda3c',
u'\ufebf': 'ArabicLetterDadInitialForm',
u'\u6040': 'CjkUnifiedIdeograph6040',
u'\u84c3': 'CjkUnifiedIdeograph84c3',
u'\u0ac7': 'GujaratiVowelSignE',
u'\ua946': 'RejangLetterA',
u'\ucdc9': 'HangulSyllableCweot',
u'\u2f4a': 'KangxiRadicalTree',
u'\u53cd': 'CjkUnifiedIdeograph53cd',
u'\uf24c': 'UnicodeCharacter0xf24c',
u'\u7850': 'CjkUnifiedIdeograph7850',
u'\u9cd3': 'CjkUnifiedIdeograph9cd3',
u'\u22d7': 'GreaterthanWithDot',
u'\uc156': 'HangulSyllableSyeogg',
u'\ue5d9': 'UnicodeCharacter0xe5d9',
u'\u475a': 'CjkUnifiedIdeograph475a',
u'\u6bdd': 'CjkUnifiedIdeograph6bdd',
u'\u9060': 'CjkUnifiedIdeograph9060',
u'\ub4e3': 'HangulSyllableDeud',
u'\u1664': 'CanadianSyllabicsCarrierChee',
u'\u3ae7': 'CjkUnifiedIdeograph3ae7',
u'\ud966': 'UnicodeCharacter0xd966',
u'\ufde9': 'UnicodeCharacter0xfde9',
u'\u5f6a': 'CjkUnifiedIdeograph5f6a',
u'\u83ed': 'CjkUnifiedIdeograph83ed',
u'\u09f1': 'BengaliLetterRaWithLowerDiagonal',
u'\ua870': 'PhagspaLetterAspiratedFa',
u'\uccf3': 'HangulSyllableCyed',
u'\u2e74': 'UnicodeCharacter0x2e74',
u'\u52f7': 'CjkUnifiedIdeograph52f7',
u'\uf176': 'UnicodeCharacter0xf176',
u'\u777a': 'CjkUnifiedIdeograph777a',
u'\u9bfd': 'CjkUnifiedIdeograph9bfd',
u'\ua101': 'YiSyllableDix',
u'\uc080': 'HangulSyllableBbyils',
u'\u2705': 'UnicodeCharacter0x2705',
u'\u4684': 'CjkUnifiedIdeograph4684',
u'\uea07': 'UnicodeCharacter0xea07',
u'\u700b': 'CjkUnifiedIdeograph700b',
u'\u8f8a': 'CjkUnifiedIdeograph8f8a',
u'\u158e': 'CanadianSyllabicsNgaai',
u'\ub911': 'HangulSyllableRweolt',
u'\ud890': 'UnicodeCharacter0xd890',
u'\u3f15': 'CjkUnifiedIdeograph3f15',
u'\u5e94': 'CjkUnifiedIdeograph5e94',
u'\u881b': 'CjkUnifiedIdeograph881b',
u'\ua79a': 'UnicodeCharacter0xa79a',
u'\u0e1f': 'ThaiCharacterFoFan',
u'\u2d9e': 'UnicodeCharacter0x2d9e',
u'\ud121': 'HangulSyllableTyaelt',
u'\uf0a0': 'UnicodeCharacter0xf0a0',
u'\u5725': 'CjkUnifiedIdeograph5725',
u'\u76a4': 'CjkUnifiedIdeograph76a4',
u'\ua02b': 'YiSyllableBep',
u'\ubfaa': 'HangulSyllableBboej',
u'\u262f': 'YinYang',
u'\u45ae': 'CjkUnifiedIdeograph45ae',
u'\ue931': 'UnicodeCharacter0xe931',
u'\u6f35': 'CjkUnifiedIdeograph6f35',
u'\u8eb4': 'CjkUnifiedIdeograph8eb4',
u'\u14b8': 'CanadianSyllabicsMwaa',
u'\ub83b': 'HangulSyllableRyeoc',
u'\ud7ba': 'UnicodeCharacter0xd7ba',
u'\u3e3f': 'CjkUnifiedIdeograph3e3f',
u'\u5dbe': 'CjkUnifiedIdeograph5dbe',
u'\u8745': 'CjkUnifiedIdeograph8745',
u'\ua6c4': 'UnicodeCharacter0xa6c4',
u'\u0d49': 'UnicodeCharacter0xd49',
u'\u2cc8': 'CopticCapitalLetterAkhmimicKhei',
u'\ud04b': 'HangulSyllableKwic',
u'\uefca': 'UnicodeCharacter0xefca',
u'\u564f': 'CjkUnifiedIdeograph564f',
u'\u75ce': 'CjkUnifiedIdeograph75ce',
u'\u9f55': 'CjkUnifiedIdeograph9f55',
u'\ubed4': 'HangulSyllableBbeon',
u'\u2559': 'BoxDrawingsUpDoubleAndRightSingle',
u'\u44d8': 'CjkUnifiedIdeograph44d8',
u'\ue85b': 'UnicodeCharacter0xe85b',
u'\u6e5f': 'CjkUnifiedIdeograph6e5f',
u'\u8dde': 'CjkUnifiedIdeograph8dde',
u'\u13e2': 'CherokeeLetterTlv',
u'\ub765': 'HangulSyllableDdinj',
u'\ud6e4': 'HangulSyllableHweon',
u'\u3d69': 'CjkUnifiedIdeograph3d69',
u'\u5ce8': 'CjkUnifiedIdeograph5ce8',
u'\u866f': 'CjkUnifiedIdeograph866f',
u'\ua5ee': 'VaiSyllableMgbe',
u'\u0c73': 'UnicodeCharacter0xc73',
u'\u2bf2': 'UnicodeCharacter0x2bf2',
u'\ucf75': 'HangulSyllableKwanj',
u'\ueef4': 'UnicodeCharacter0xeef4',
u'\u5579': 'CjkUnifiedIdeograph5579',
u'\u74f8': 'CjkUnifiedIdeograph74f8',
u'\u9e7f': 'CjkUnifiedIdeograph9e7f',
u'\ubdfe': 'HangulSyllableByulp',
u'\uc806': 'HangulSyllableJeonh',
u'\uec89': 'UnicodeCharacter0xec89',
u'\u8d08': 'CjkUnifiedIdeograph8d08',
u'\ub18b': 'HangulSyllableNos',
u'\u4e0a': 'CjkUnifiedIdeograph4e0a',
u'\u728d': 'CjkUnifiedIdeograph728d',
u'\u130c': 'EthiopicSyllableGee',
u'\u378f': 'CjkUnifiedIdeograph378f',
u'\ue016': 'UnicodeCharacter0xe016',
u'\ua518': 'VaiSyllableZee',
u'\uc99b': 'HangulSyllableJeus',
u'\u661a': 'CjkUnifiedIdeograph661a',
u'\u8a9d': 'CjkUnifiedIdeograph8a9d',
u'\u2b1c': 'WhiteLargeSquare',
u'\u4f9f': 'CjkUnifiedIdeograph4f9f',
u'\u10a1': 'GeorgianCapitalLetterBan',
u'\uf826': 'UnicodeCharacter0xf826',
u'\ubd28': 'HangulSyllableBwak',
u'\ue1ab': 'UnicodeCharacter0xe1ab',
u'\u7e2a': 'CjkUnifiedIdeograph7e2a',
u'\ua2ad': 'YiSyllableZyr',
u'\u432c': 'CjkUnifiedIdeograph432c',
u'\u67af': 'CjkUnifiedIdeograph67af',
u'\u042e': 'CyrillicCapitalLetterYu',
u'\u28b1': 'BraillePatternDots1568',
u'\ud538': 'HangulSyllablePyik',
u'\uf9bb': 'CjkCompatibilityIdeographf9bb',
u'\u963a': 'CjkUnifiedIdeograph963a',
u'\ubabd': 'HangulSyllableMong',
u'\u5b3c': 'CjkUnifiedIdeograph5b3c',
u'\u7fbf': 'CjkUnifiedIdeograph7fbf',
u'\u1c3e': 'LepchaPunctuationTshookCerwa',
u'\u40c1': 'CjkUnifiedIdeograph40c1',
u'\u05c3': 'HebrewPunctuationSofPasuq',
u'\ued48': 'UnicodeCharacter0xed48',
u'\uae4a': 'HangulSyllableGip',
u'\ud2cd': 'HangulSyllableTeung',
u'\u734c': 'CjkUnifiedIdeograph734c',
u'\u97cf': 'CjkUnifiedIdeograph97cf',
u'\u344e': 'CjkUnifiedIdeograph344e',
u'\u58d1': 'CjkUnifiedIdeograph58d1',
u'\u1dd3': 'CombiningLatinSmallLetterFlattenedOpenAAbove',
u'\uc65a': 'HangulSyllableWap',
u'\ueadd': 'UnicodeCharacter0xeadd',
u'\u8b5c': 'CjkUnifiedIdeograph8b5c',
u'\uafdf': 'HangulSyllableGgweolb',
u'\u4c5e': 'CjkUnifiedIdeograph4c5e',
u'\u70e1': 'CjkUnifiedIdeograph70e1',
u'\u1160': 'HangulJungseongFiller',
u'\u35e3': 'CjkUnifiedIdeograph35e3',
u'\ude6a': 'UnicodeCharacter0xde6a',
u'\ua36c': 'YiSyllableChop',
u'\uc7ef': 'HangulSyllableJyaelb',
u'\u646e': 'CjkUnifiedIdeograph646e',
u'\u88f1': 'CjkUnifiedIdeograph88f1',
u'\u2970': 'RightDoubleArrowWithRoundedHead',
u'\u4df3': 'HexagramForTheKeepingStillMountain',
u'\u0ef5': 'UnicodeCharacter0xef5',
u'\uf67a': 'UnicodeCharacter0xf67a',
u'\ubb7c': 'HangulSyllableMwem',
u'\udfff': 'UnicodeCharacter0xdfff',
u'\u7c7e': 'CjkUnifiedIdeograph7c7e',
u'\u4180': 'CjkUnifiedIdeograph4180',
u'\ue503': 'UnicodeCharacter0xe503',
u'\u0282': 'LatinSmallLetterSWithHook',
u'\ua605': 'VaiSyllableNggen',
u'\u6b07': 'CjkUnifiedIdeograph6b07',
u'\u2c09': 'GlagoliticCapitalLetterIzhe',
u'\ud38c': 'HangulSyllablePeom',
u'\u948e': 'CjkUnifiedIdeograph948e',
u'\u5990': 'CjkUnifiedIdeograph5990',
u'\ufd13': 'ArabicLigatureAinWithAlefMaksuraFinalForm',
u'\u1a92': 'UnicodeCharacter0x1a92',
u'\ube15': 'HangulSyllableBeulg',
u'\u8317': 'CjkUnifiedIdeograph8317',
u'\u4419': 'CjkUnifiedIdeograph4419',
u'\u091b': 'DevanagariLetterCha',
u'\ueb9c': 'UnicodeCharacter0xeb9c',
u'\uac9e': 'HangulSyllableGebs',
u'\u71a0': 'CjkUnifiedIdeograph71a0',
u'\u32a2': 'CircledIdeographCopy',
u'\ud625': 'HangulSyllableHyelg',
u'\u9b27': 'CjkUnifiedIdeograph9b27',
u'\u5c29': 'CjkUnifiedIdeograph5c29',
u'\u212b': 'AngstromSign',
u'\uc4ae': 'HangulSyllableSswebs',
u'\u89b0': 'CjkUnifiedIdeograph89b0',
u'\u4ab2': 'CjkUnifiedIdeograph4ab2',
u'\uee35': 'UnicodeCharacter0xee35',
u'\u0fb4': 'TibetanSubjoinedLetterSha',
u'\ub337': 'HangulSyllableDyah',
u'\u7439': 'CjkUnifiedIdeograph7439',
u'\u393b': 'CjkUnifiedIdeograph393b',
u'\udcbe': 'UnicodeCharacter0xdcbe',
u'\ua1c0': 'YiSyllableLax',
u'\u62c2': 'CjkUnifiedIdeograph62c2',
u'\u27c4': 'OpenSuperset',
u'\ucb47': 'HangulSyllableJjyoh',
u'\u8c49': 'CjkUnifiedIdeograph8c49',
u'\u514b': 'CjkUnifiedIdeograph514b',
u'\u124d': 'EthiopicSyllableQwe',
u'\uf4ce': 'UnicodeCharacter0xf4ce',
u'\ub9d0': 'HangulSyllableMal',
u'\u7ad2': 'CjkUnifiedIdeograph7ad2',
u'\u3fd4': 'CjkUnifiedIdeograph3fd4',
u'\ue357': 'UnicodeCharacter0xe357',
u'\xd6': 'LatinCapitalLetterOWithDiaeresis',
u'\ua459': 'YiSyllableNyut',
u'\u695b': 'CjkUnifiedIdeograph695b',
u'\u2a5d': 'LogicalOrWithHorizontalDash',
u'\ud1e0': 'HangulSyllableTwael',
u'\u92e2': 'CjkUnifiedIdeograph92e2',
u'\u57e4': 'CjkUnifiedIdeograph57e4',
u'\ufb67': 'ArabicLetterTtehFinalForm',
u'\u18e6': 'UnicodeCharacter0x18e6',
u'\ubc69': 'HangulSyllableByaeg',
u'\u816b': 'CjkUnifiedIdeograph816b',
u'\u426d': 'CjkUnifiedIdeograph426d',
u'\u076f': 'ArabicLetterHahWithSmallArabicLetterTahAndTwoDots',
u'\ue9f0': 'UnicodeCharacter0xe9f0',
u'\uaaf2': 'UnicodeCharacter0xaaf2',
u'\u6ff4': 'CjkUnifiedIdeograph6ff4',
u'\u30f6': 'KatakanaLetterSmallKe',
u'\ud479': 'HangulSyllablePug',
u'\u997b': 'CjkUnifiedIdeograph997b',
u'\u5a7d': 'CjkUnifiedIdeograph5a7d',
u'\u1f7f': 'UnicodeCharacter0x1f7f',
u'\u6381': 'CjkUnifiedIdeograph6381',
u'\u8404': 'CjkUnifiedIdeograph8404',
u'\ua887': 'SaurashtraLetterUu',
u'\u0e08': 'ThaiCharacterChoChan',
u'\u328b': 'CircledIdeographFire',
u'\ucd0a': 'HangulSyllableCogg',
u'\uf18d': 'UnicodeCharacter0xf18d',
u'\u570e': 'CjkUnifiedIdeograph570e',
u'\u7b91': 'CjkUnifiedIdeograph7b91',
u'\u9c14': 'CjkUnifiedIdeograph9c14',
u'\uc097': 'HangulSyllableBbid',
u'\u2618': 'Shamrock',
u'\u4a9b': 'CjkUnifiedIdeograph4a9b',
u'\ue51a': 'UnicodeCharacter0xe51a',
u'\u6f1e': 'CjkUnifiedIdeograph6f1e',
u'\u93a1': 'CjkUnifiedIdeograph93a1',
u'\u15a5': 'CanadianSyllabicsLhaa',
u'\ub424': 'HangulSyllableDoels',
u'\ud8a7': 'UnicodeCharacter0xd8a7',
u'\u3e28': 'CjkUnifiedIdeograph3e28',
u'\u62ab': 'CjkUnifiedIdeograph62ab',
u'\ufd2a': 'ArabicLigatureSeenWithRehFinalForm',
u'\u872e': 'CjkUnifiedIdeograph872e',
u'\uabb1': 'UnicodeCharacter0xabb1',
u'\u0932': 'DevanagariLetterLa',
u'\u2db5': 'EthiopicSyllableZze',
u'\ucc34': 'HangulSyllableCals',
u'\uf0b7': 'UnicodeCharacter0xf0b7',
u'\u5638': 'CjkUnifiedIdeograph5638',
u'\u7abb': 'CjkUnifiedIdeograph7abb',
u'\u9f3e': 'CjkUnifiedIdeograph9f3e',
u'\uc3c1': 'HangulSyllableSsyenj',
u'\u2142': 'TurnedSansserifCapitalL',
u'\u45c5': 'CjkUnifiedIdeograph45c5',
u'\ue444': 'UnicodeCharacter0xe444',
u'\u6e48': 'CjkUnifiedIdeograph6e48',
u'\u92cb': 'CjkUnifiedIdeograph92cb',
u'\u14cf': 'CanadianSyllabicsNaskapiNwaa',
u'\ub74e': 'HangulSyllableDdyilm',
u'\udbd1': 'UnicodeCharacter0xdbd1',
u'\u3952': 'CjkUnifiedIdeograph3952',
u'\u5dd5': 'CjkUnifiedIdeograph5dd5',
u'\ufc54': 'ArabicLigatureHehWithYehIsolatedForm',
u'\u8658': 'CjkUnifiedIdeograph8658',
u'\uaadb': 'UnicodeCharacter0xaadb',
u'\u085c': 'UnicodeCharacter0x85c',
u'\u2cdf': 'CopticSmallLetterOldNubianNgi',
u'\ucf5e': 'HangulSyllableKolm',
u'\uf3e1': 'UnicodeCharacter0xf3e1',
u'\u5162': 'CjkUnifiedIdeograph5162',
u'\u75e5': 'CjkUnifiedIdeograph75e5',
u'\u9e68': 'CjkUnifiedIdeograph9e68',
u'\uc2eb': 'HangulSyllableSilh',
u'\u206c': 'InhibitArabicFormShaping',
u'\u44ef': 'CjkUnifiedIdeograph44ef',
u'\ue76e': 'UnicodeCharacter0xe76e',
u'\u6972': 'CjkUnifiedIdeograph6972',
u'\u8df5': 'CjkUnifiedIdeograph8df5',
u'\u17f9': 'KhmerSymbolLekAttakPrambuon',
u'\ub678': 'HangulSyllableDdoess',
u'\udafb': 'UnicodeCharacter0xdafb',
u'\u387c': 'CjkUnifiedIdeograph387c',
u'\u5cff': 'CjkUnifiedIdeograph5cff',
u'\uff7e': 'HalfwidthKatakanaLetterSe',
u'\u6603': 'CjkUnifiedIdeograph6603',
u'\u8182': 'CjkUnifiedIdeograph8182',
u'\u0b86': 'TamilLetterAa',
u'\uaf09': 'HangulSyllableGgyeong',
u'\uce88': 'HangulSyllableKass',
u'\u310d': 'BopomofoLetterG',
u'\u508c': 'CjkUnifiedIdeograph508c',
u'\uf40f': 'UnicodeCharacter0xf40f',
u'\u7e13': 'CjkUnifiedIdeograph7e13',
u'\u9992': 'CjkUnifiedIdeograph9992',
u'\u2396': 'DecimalSeparatorKeySymbol',
u'\uc719': 'HangulSyllableWing',
u'\ue698': 'UnicodeCharacter0xe698',
u'\u491d': 'CjkUnifiedIdeograph491d',
u'\u689c': 'CjkUnifiedIdeograph689c',
u'\u9623': 'CjkUnifiedIdeograph9623',
u'\ub1a2': 'HangulSyllableNwalp',
u'\u1827': 'MongolianLetterEe',
u'\u3ba6': 'CjkUnifiedIdeograph3ba6',
u'\udf29': 'UnicodeCharacter0xdf29',
u'\ufea8': 'ArabicLetterKhahMedialForm',
u'\u612d': 'CjkUnifiedIdeograph612d',
u'\u80ac': 'CjkUnifiedIdeograph80ac',
u'\u0ab0': 'GujaratiLetterRa',
u'\uae33': 'HangulSyllableGigs',
u'\uc9b2': 'HangulSyllableJyilp',
u'\u3037': 'IdeographicTelegraphLineFeedSeparatorSymbol',
u'\u53b6': 'CjkUnifiedIdeograph53b6',
u'\uf739': 'UnicodeCharacter0xf739',
u'\u793d': 'CjkUnifiedIdeograph793d',
u'\u98bc': 'CjkUnifiedIdeograph98bc',
u'\u0341': 'CombiningAcuteToneMark',
u'\u22c0': 'NaryLogicalAnd',
u'\uc643': 'HangulSyllableWags',
u'\ue1c2': 'UnicodeCharacter0xe1c2',
u'\u4847': 'CjkUnifiedIdeograph4847',
u'\u6bc6': 'CjkUnifiedIdeograph6bc6',
u'\u914d': 'CjkUnifiedIdeograph914d',
u'\ub0cc': 'HangulSyllableNaek',
u'\u1b51': 'BalineseDigitOne',
u'\u3ad0': 'CjkUnifiedIdeograph3ad0',
u'\ude53': 'UnicodeCharacter0xde53',
u'\uf9d2': 'CjkCompatibilityIdeographf9d2',
u'\u6057': 'CjkUnifiedIdeograph6057',
u'\u83d6': 'CjkUnifiedIdeograph83d6',
u'\u05da': 'HebrewLetterFinalKaf',
u'\ua95d': 'UnicodeCharacter0xa95d',
u'\uc8dc': 'HangulSyllableJoek',
u'\u3361': 'IdeographicTelegraphSymbolForHourNine',
u'\u52e0': 'CjkUnifiedIdeograph52e0',
u'\uf663': 'UnicodeCharacter0xf663',
u'\u7867': 'CjkUnifiedIdeograph7867',
u'\u9be6': 'CjkUnifiedIdeograph9be6',
u'\u026b': 'LatinSmallLetterLWithMiddleTilde',
u'\u1dea': 'UnicodeCharacter0x1dea',
u'\uc16d': 'HangulSyllableSyeot',
u'\ue0ec': 'UnicodeCharacter0xe0ec',
u'\u4b71': 'CjkUnifiedIdeograph4b71',
u'\u6af0': 'CjkUnifiedIdeograph6af0',
u'\u9077': 'CjkUnifiedIdeograph9077',
u'\ub3f6': 'HangulSyllableDwaj',
u'\u1a7b': 'UnicodeCharacter0x1a7b',
u'\u35fa': 'CjkUnifiedIdeograph35fa',
u'\ud97d': 'UnicodeCharacter0xd97d',
u'\uf8fc': 'UnicodeCharacter0xf8fc',
u'\ue281': 'UnicodeCharacter0xe281',
u'\u8300': 'CjkUnifiedIdeograph8300',
u'\ua783': 'LatinSmallLetterInsularR',
u'\u4002': 'CjkUnifiedIdeograph4002',
u'\u6485': 'CjkUnifiedIdeograph6485',
u'\u0504': 'CyrillicCapitalLetterKomiZje',
u'\u2987': 'ZNotationLeftImageBracket',
u'\ud60e': 'HangulSyllableHyeolp',
u'\ufa91': 'CjkCompatibilityIdeographfa91',
u'\u9b10': 'CjkUnifiedIdeograph9b10',
u'\ubf93': 'HangulSyllableBbwaeh',
u'\u5812': 'CjkUnifiedIdeograph5812',
u'\u7c95': 'CjkUnifiedIdeograph7c95',
u'\u1d14': 'LatinSmallLetterTurnedOe',
u'\u4197': 'CjkUnifiedIdeograph4197',
u'\u0699': 'ArabicLetterRehWithFourDotsAbove',
u'\uee1e': 'UnicodeCharacter0xee1e',
u'\ub320': 'HangulSyllableDyan',
u'\ud7a3': 'HangulSyllableHih',
u'\u7022': 'CjkUnifiedIdeograph7022',
u'\u94a5': 'CjkUnifiedIdeograph94a5',
u'\u3524': 'CjkUnifiedIdeograph3524',
u'\u59a7': 'CjkUnifiedIdeograph59a7',
u'\u1ea9': 'LatinSmallLetterAWithCircumflexAndHookAbove',
u'\ucb30': 'HangulSyllableJjyon',
u'\uefb3': 'UnicodeCharacter0xefb3',
u'\u8832': 'CjkUnifiedIdeograph8832',
u'\uacb5': 'HangulSyllableGyeolt',
u'\u4d34': 'CjkUnifiedIdeograph4d34',
u'\u71b7': 'CjkUnifiedIdeograph71b7',
u'\u1236': 'EthiopicSyllableSo',
u'\u36b9': 'CjkUnifiedIdeograph36b9',
u'\ue340': 'UnicodeCharacter0xe340',
u'\ua042': 'YiSyllablePap',
u'\uc4c5': 'HangulSyllableSswilt',
u'\u6544': 'CjkUnifiedIdeograph6544',
u'\u89c7': 'CjkUnifiedIdeograph89c7',
u'\u2a46': 'UnionAboveIntersection',
u'\u4ec9': 'CjkUnifiedIdeograph4ec9',
u'\u13cb': 'CherokeeLetterQuv',
u'\ufb50': 'ArabicLetterAlefWaslaIsolatedForm',
u'\ub852': 'HangulSyllableRyebs',
u'\udcd5': 'UnicodeCharacter0xdcd5',
u'\u7d54': 'CjkUnifiedIdeograph7d54',
u'\ua1d7': 'YiSyllableLyp',
u'\u4256': 'CjkUnifiedIdeograph4256',
u'\u66d9': 'CjkUnifiedIdeograph66d9',
u'\u0758': 'ArabicLetterHahWithThreeDotsPointingUpwardsBelow',
u'\u2bdb': 'UnicodeCharacter0x2bdb',
u'\ud062': 'HangulSyllableKyubs',
u'\uf4e5': 'UnicodeCharacter0xf4e5',
u'\u9564': 'CjkUnifiedIdeograph9564',
u'\ub9e7': 'HangulSyllableMaegs',
u'\u5a66': 'CjkUnifiedIdeograph5a66',
u'\u7ee9': 'CjkUnifiedIdeograph7ee9',
u'\u1f68': 'GreekCapitalLetterOmegaWithPsili',
u'\u43eb': 'CjkUnifiedIdeograph43eb',
u'\xed': 'LatinSmallLetterIWithAcute',
u'\ue872': 'UnicodeCharacter0xe872',
u'\uad74': 'HangulSyllableGul',
u'\ud1f7': 'HangulSyllableToegs',
u'\u7276': 'CjkUnifiedIdeograph7276',
u'\u96f9': 'CjkUnifiedIdeograph96f9',
u'\u3778': 'CjkUnifiedIdeograph3778',
u'\u5bfb': 'CjkUnifiedIdeograph5bfb',
u'\u18fd': 'UnicodeCharacter0x18fd',
u'\u2201': 'Complement',
u'\uc584': 'HangulSyllableYal',
u'\u8a86': 'CjkUnifiedIdeograph8a86',
u'\u4f88': 'CjkUnifiedIdeograph4f88',
u'\uf30b': 'UnicodeCharacter0xf30b',
u'\u0c8a': 'KannadaLetterUu',
u'\ub00d': 'HangulSyllableGgwig',
u'\u750f': 'CjkUnifiedIdeograph750f',
u'\u3a11': 'CjkUnifiedIdeograph3a11',
u'\udd94': 'UnicodeCharacter0xdd94',
u'\ua296': 'YiSyllableZa',
u'\u6798': 'CjkUnifiedIdeograph6798',
u'\u249a': 'NumberNineteenFullStop',
u'\uc81d': 'HangulSyllableJeg',
u'\u8d1f': 'CjkUnifiedIdeograph8d1f',
u'\u5221': 'CjkUnifiedIdeograph5221',
u'\u1723': 'HanunooLetterKa',
u'\uf5a4': 'UnicodeCharacter0xf5a4',
u'\ubaa6': 'HangulSyllableMyep',
u'\u7fa8': 'CjkUnifiedIdeograph7fa8',
u'\u3caa': 'CjkUnifiedIdeograph3caa',
u'\ue02d': 'UnicodeCharacter0xe02d',
u'\u01ac': 'LatinCapitalLetterTWithHook',
u'\ua52f': 'VaiSyllableMgbi',
u'\u6a31': 'CjkUnifiedIdeograph6a31',
u'\u2f33': 'KangxiRadicalShortThread',
u'\ud2b6': 'HangulSyllableTyup',
u'\u97b8': 'CjkUnifiedIdeograph97b8',
u'\u54ba': 'CjkUnifiedIdeograph54ba',
u'\uf83d': 'UnicodeCharacter0xf83d',
u'\u19bc': 'NewTaiLueVowelSignUy',
u'\ubd3f': 'HangulSyllableBwaes',
u'\u8241': 'CjkUnifiedIdeograph8241',
u'\u4743': 'CjkUnifiedIdeograph4743',
u'\u0445': 'CyrillicSmallLetterHa',
u'\ueac6': 'UnicodeCharacter0xeac6',
u'\uafc8': 'HangulSyllableGgum',
u'\u6cca': 'CjkUnifiedIdeograph6cca',
u'\u31cc': 'CjkStrokeHpwg',
u'\ud54f': 'HangulSyllablePis',
u'\u9a51': 'CjkUnifiedIdeograph9a51',
u'\u5f53': 'CjkUnifiedIdeograph5f53',
u'\u1c55': 'OlChikiDigitFive',
u'\uc7d8': 'HangulSyllableJyam',
u'\u84da': 'CjkUnifiedIdeograph84da',
u'\u49dc': 'CjkUnifiedIdeograph49dc',
u'\ued5f': 'UnicodeCharacter0xed5f',
u'\u0ede': 'UnicodeCharacter0xede',
u'\ub261': 'HangulSyllableNwilg',
u'\u7763': 'CjkUnifiedIdeograph7763',
u'\u3465': 'CjkUnifiedIdeograph3465',
u'\udfe8': 'UnicodeCharacter0xdfe8',
u'\u9cea': 'CjkUnifiedIdeograph9cea',
u'\u61ec': 'CjkUnifiedIdeograph61ec',
u'\u26ee': 'UnicodeCharacter0x26ee',
u'\uca71': 'HangulSyllableJjelg',
u'\u8f73': 'CjkUnifiedIdeograph8f73',
u'\u4c75': 'CjkUnifiedIdeograph4c75',
u'\u1177': 'HangulJungseongAu',
u'\uf7f8': 'UnicodeCharacter0xf7f8',
u'\ub4fa': 'HangulSyllableDyigg',
u'\u79fc': 'CjkUnifiedIdeograph79fc',
u'\u3efe': 'CjkUnifiedIdeograph3efe',
u'\u0400': 'CyrillicCapitalLetterIeWithGrave',
u'\u2083': 'SubscriptThree',
u'\uc702': 'HangulSyllableWep',
u'\ue385': 'UnicodeCharacter0xe385',
u'\u4d06': 'CjkUnifiedIdeograph4d06',
u'\u6989': 'CjkUnifiedIdeograph6989',
u'\u960c': 'CjkUnifiedIdeograph960c',
u'\ub28f': 'HangulSyllableNyuh',
u'\u1c10': 'LepchaLetterPha',
u'\u3893': 'CjkUnifiedIdeograph3893',
u'\udf12': 'UnicodeCharacter0xdf12',
u'\ufb95': 'ArabicLetterGafMedialForm',
u'\u6516': 'CjkUnifiedIdeograph6516',
u'\u8199': 'CjkUnifiedIdeograph8199',
u'\u0f9d': 'TibetanSubjoinedLetterDdha',
u'\uae1c': 'HangulSyllableGyil',
u'\uca9f': 'HangulSyllableJjyeoh',
u'\u3420': 'CjkUnifiedIdeograph3420',
u'\u50a3': 'CjkUnifiedIdeograph50a3',
u'\uf722': 'UnicodeCharacter0xf722',
u'\u7d26': 'CjkUnifiedIdeograph7d26',
u'\u99a9': 'CjkUnifiedIdeograph99a9',
u'\u27ad': 'HeavyLowerRightshadowedWhiteRightwardsArrow',
u'\uc62c': 'HangulSyllableOl',
u'\ue2af': 'UnicodeCharacter0xe2af',
u'\u4c30': 'CjkUnifiedIdeograph4c30',
u'\u68b3': 'CjkUnifiedIdeograph68b3',
u'\u9536': 'CjkUnifiedIdeograph9536',
u'\ub1b9': 'HangulSyllableNwaelg',
u'\u133a': 'EthiopicSyllableTsi',
u'\u3fbd': 'CjkUnifiedIdeograph3fbd',
u'\ude3c': 'UnicodeCharacter0xde3c',
u'\ufabf': 'CjkCompatibilityIdeographfabf',
u'\u6440': 'CjkUnifiedIdeograph6440',
u'\u80c3': 'CjkUnifiedIdeograph80c3',
u'\u0ec7': 'UnicodeCharacter0xec7',
u'\uad46': 'HangulSyllableGoebs',
u'\uc9c9': 'HangulSyllableJilg',
u'\u2b4a': 'LeftwardsArrowAboveAlmostEqualTo',
u'\u57cd': 'CjkUnifiedIdeograph57cd',
u'\uf64c': 'UnicodeCharacter0xf64c',
u'\u7c50': 'CjkUnifiedIdeograph7c50',
u'\u98d3': 'CjkUnifiedIdeograph98d3',
u'\u26d7': 'UnicodeCharacter0x26d7',
u'\uc556': 'HangulSyllableAbs',
u'\ue1d9': 'UnicodeCharacter0xe1d9',
u'\u435a': 'CjkUnifiedIdeograph435a',
u'\u6fdd': 'CjkUnifiedIdeograph6fdd',
u'\u9460': 'CjkUnifiedIdeograph9460',
u'\ub0e3': 'HangulSyllableNyas',
u'\u1264': 'EthiopicSyllableBee',
u'\u3ee7': 'CjkUnifiedIdeograph3ee7',
u'\udd66': 'UnicodeCharacter0xdd66',
u'\uf9e9': 'CjkCompatibilityIdeographf9e9',
u'\u5b6a': 'CjkUnifiedIdeograph5b6a',
u'\u87ed': 'CjkUnifiedIdeograph87ed',
u'\u0df1': 'UnicodeCharacter0xdf1',
u'\uac70': 'HangulSyllableGeo',
u'\uc8f3': 'HangulSyllableJyos',
u'\u2a74': 'DoubleColonEqual',
u'\u56f7': 'CjkUnifiedIdeograph56f7',
u'\uf576': 'UnicodeCharacter0xf576',
u'\u737a': 'CjkUnifiedIdeograph737a',
u'\u9ffd': 'UnicodeCharacter0x9ffd',
u'\ua501': 'VaiSyllableEen',
u'\uc480': 'HangulSyllableSsweo',
u'\u2305': 'Projective',
u'\u4284': 'CjkUnifiedIdeograph4284',
u'\uee07': 'UnicodeCharacter0xee07',
u'\u740b': 'CjkUnifiedIdeograph740b',
u'\u8b8a': 'CjkUnifiedIdeograph8b8a',
u'\u118e': 'HangulJungseongYua',
u'\ubd11': 'HangulSyllableBwag',
u'\udc90': 'UnicodeCharacter0xdc90',
u'\u3b15': 'CjkUnifiedIdeograph3b15',
u'\u5a94': 'CjkUnifiedIdeograph5a94',
u'\u8c1b': 'CjkUnifiedIdeograph8c1b',
u'\ua39a': 'YiSyllableNrop',
u'\u0a1f': 'GurmukhiLetterTta',
u'\u299e': 'AngleWithSInside',
u'\ud521': 'HangulSyllablePyig',
u'\uf4a0': 'UnicodeCharacter0xf4a0',
u'\u5325': 'CjkUnifiedIdeograph5325',
u'\u72a4': 'CjkUnifiedIdeograph72a4',
u'\ua42b': 'YiSyllableJjurx',
u'\ubbaa': 'HangulSyllableMyunh',
u'\u222f': 'SurfaceIntegral',
u'\u41ae': 'CjkUnifiedIdeograph41ae',
u'\ued31': 'UnicodeCharacter0xed31',
u'\u6b35': 'CjkUnifiedIdeograph6b35',
u'\u8ab4': 'CjkUnifiedIdeograph8ab4',
u'\u10b8': 'GeorgianCapitalLetterShin',
u'\ubc3b': 'HangulSyllableBaelb',
u'\ud3ba': 'HangulSyllablePyeonh',
u'\u3a3f': 'CjkUnifiedIdeograph3a3f',
u'\u59be': 'CjkUnifiedIdeograph59be',
u'\u8345': 'CjkUnifiedIdeograph8345',
u'\ua2c4': 'YiSyllableCut',
u'\u0949': 'DevanagariVowelSignCandraO',
u'\u28c8': 'BraillePatternDots478',
u'\ud44b': 'HangulSyllablePoelb',
u'\uebca': 'UnicodeCharacter0xebca',
u'\u524f': 'CjkUnifiedIdeograph524f',
u'\u71ce': 'CjkUnifiedIdeograph71ce',
u'\u9b55': 'CjkUnifiedIdeograph9b55',
u'\ubad4': 'HangulSyllableMwam',
u'\u2159': 'VulgarFractionOneSixth',
u'\u40d8': 'CjkUnifiedIdeograph40d8',
u'\uec5b': 'UnicodeCharacter0xec5b',
u'\u6a5f': 'CjkUnifiedIdeograph6a5f',
u'\u89de': 'CjkUnifiedIdeograph89de',
u'\u17e2': 'KhmerDigitTwo',
u'\ub365': 'HangulSyllableDeob',
u'\ud2e4': 'HangulSyllableTyim',
u'\u3969': 'CjkUnifiedIdeograph3969',
u'\u58e8': 'CjkUnifiedIdeograph58e8',
u'\u826f': 'CjkUnifiedIdeograph826f',
u'\ua1ee': 'YiSyllableGet',
u'\u0873': 'UnicodeCharacter0x873',
u'\u2ff2': 'IdeographicDescriptionCharacterLeftToMiddleAndRight',
u'\ucb75': 'HangulSyllableJjweob',
u'\ueaf4': 'UnicodeCharacter0xeaf4',
u'\u5179': 'CjkUnifiedIdeograph5179',
u'\u70f8': 'CjkUnifiedIdeograph70f8',
u'\u9a7f': 'CjkUnifiedIdeograph9a7f',
u'\ub9fe': 'HangulSyllableMaep',
u'\ucc06': 'HangulSyllableJjyij',
u'\ue889': 'UnicodeCharacter0xe889',
u'\u8908': 'CjkUnifiedIdeograph8908',
u'\ub58b': 'HangulSyllableDdyaed',
u'\u4a0a': 'CjkUnifiedIdeograph4a0a',
u'\u768d': 'CjkUnifiedIdeograph768d',
u'\u170c': 'TagalogLetterYa',
u'\u338f': 'SquareKg',
u'\ue416': 'UnicodeCharacter0xe416',
u'\ua118': 'YiSyllableDurx',
u'\ucd9b': 'HangulSyllableCud',
u'\u621a': 'CjkUnifiedIdeograph621a',
u'\u8e9d': 'CjkUnifiedIdeograph8e9d',
u'\u2f1c': 'KangxiRadicalAgain',
u'\u4b9f': 'CjkUnifiedIdeograph4b9f',
u'\u14a1': 'CanadianSyllabicsC',
u'\ufc26': 'ArabicLigatureTahWithHahIsolatedForm',
u'\ub928': 'HangulSyllableRwel',
u'\ue5ab': 'UnicodeCharacter0xe5ab',
u'\u7a2a': 'CjkUnifiedIdeograph7a2a',
u'\ua6ad': 'UnicodeCharacter0xa6ad',
u'\u472c': 'CjkUnifiedIdeograph472c',
u'\u63af': 'CjkUnifiedIdeograph63af',
u'\u2cb1': 'CopticSmallLetterOou',
u'\ud138': 'HangulSyllableTeol',
u'\ufdbb': 'ArabicLigatureKafWithMeemWithMeemFinalForm',
u'\u923a': 'CjkUnifiedIdeograph923a',
u'\ubebd': 'HangulSyllableBbyaelg',
u'\u5f3c': 'CjkUnifiedIdeograph5f3c',
u'\u7bbf': 'CjkUnifiedIdeograph7bbf',
u'\u183e': 'MongolianLetterHaa',
u'\u44c1': 'CjkUnifiedIdeograph44c1',
u'\u01c3': 'LatinLetterRetroflexClick',
u'\ue948': 'UnicodeCharacter0xe948',
u'\uaa4a': 'ChamLetterFinalL',
u'\ud6cd': 'HangulSyllableHulg',
u'\u774c': 'CjkUnifiedIdeograph774c',
u'\u93cf': 'CjkUnifiedIdeograph93cf',
u'\u304e': 'HiraganaLetterGi',
u'\u5cd1': 'CjkUnifiedIdeograph5cd1',
u'\u19d3': 'NewTaiLueDigitThree',
u'\uc25a': 'HangulSyllableSwelm',
u'\ueedd': 'UnicodeCharacter0xeedd',
u'\u8f5c': 'CjkUnifiedIdeograph8f5c',
u'\uabdf': 'UnicodeCharacter0xabdf',
u'\u485e': 'CjkUnifiedIdeograph485e',
u'\u74e1': 'CjkUnifiedIdeograph74e1',
u'\u1560': 'CanadianSyllabicsThi',
u'\u31e3': 'CjkStrokeQ',
u'\uda6a': 'UnicodeCharacter0xda6a',
u'\ua76c': 'LatinCapitalLetterIs',
u'\uc3ef': 'HangulSyllableSsoc',
u'\u606e': 'CjkUnifiedIdeograph606e',
u'\u8cf1': 'CjkUnifiedIdeograph8cf1',
u'\u2d70': 'UnicodeCharacter0x2d70',
u'\u49f3': 'CjkUnifiedIdeograph49f3',
u'\u0af5': 'UnicodeCharacter0xaf5',
u'\uf27a': 'UnicodeCharacter0xf27a',
u'\ubf7c': 'HangulSyllableBbwaen',
u'\udbff': 'UnicodeCharacter0xdbff',
u'\u787e': 'CjkUnifiedIdeograph787e',
u'\u4580': 'CjkUnifiedIdeograph4580',
u'\ue103': 'UnicodeCharacter0xe103',
u'\u0682': 'ArabicLetterHahWithTwoDotsVerticalAbove',
u'\ua205': 'YiSyllableKuop',
u'\u6f07': 'CjkUnifiedIdeograph6f07',
u'\u2809': 'BraillePatternDots14',
u'\ud78c': 'HangulSyllableHin',
u'\u908e': 'CjkUnifiedIdeograph908e',
u'\u5d90': 'CjkUnifiedIdeograph5d90',
u'\uf913': 'CjkCompatibilityIdeographf913',
u'\u1e92': 'LatinCapitalLetterZWithDotBelow',
u'\uba15': 'HangulSyllableMyang',
u'\u8717': 'CjkUnifiedIdeograph8717',
u'\u4019': 'CjkUnifiedIdeograph4019',
u'\u0d1b': 'MalayalamLetterCha',
u'\uef9c': 'UnicodeCharacter0xef9c',
u'\ua89e': 'SaurashtraLetterDda',
u'\u75a0': 'CjkUnifiedIdeograph75a0',
u'\u36a2': 'CjkUnifiedIdeograph36a2',
u'\ud225': 'HangulSyllableTyong',
u'\u9f27': 'CjkUnifiedIdeograph9f27',
u'\u5829': 'CjkUnifiedIdeograph5829',
u'\u252b': 'BoxDrawingsHeavyVerticalAndLeft',
u'\uc0ae': 'HangulSyllableSagg',
u'\u8db0': 'CjkUnifiedIdeograph8db0',
u'\u4eb2': 'CjkUnifiedIdeograph4eb2',
u'\uea35': 'UnicodeCharacter0xea35',
u'\u0bb4': 'TamilLetterLlla',
u'\ub737': 'HangulSyllableDdeulh',
u'\u7039': 'CjkUnifiedIdeograph7039',
u'\u3d3b': 'CjkUnifiedIdeograph3d3b',
u'\ud8be': 'UnicodeCharacter0xd8be',
u'\ua5c0': 'VaiSyllableWon',
u'\u66c2': 'CjkUnifiedIdeograph66c2',
u'\u23c4': 'DentistrySymbolLightDownAndHorizontalWithTriangle',
u'\ucf47': 'HangulSyllableKyelh',
u'\u8849': 'CjkUnifiedIdeograph8849',
u'\u554b': 'CjkUnifiedIdeograph554b',
u'\u164d': 'CanadianSyllabicsCarrierDza',
u'\uf0ce': 'UnicodeCharacter0xf0ce',
u'\ubdd0': 'HangulSyllableBwek',
u'\u7ed2': 'CjkUnifiedIdeograph7ed2',
u'\u3bd4': 'CjkUnifiedIdeograph3bd4',
u'\ue757': 'UnicodeCharacter0xe757',
u'\u04d6': 'CyrillicCapitalLetterIeWithBreve',
u'\ua059': 'YiSyllableBbip',
u'\u6d5b': 'CjkUnifiedIdeograph6d5b',
u'\u2e5d': 'UnicodeCharacter0x2e5d',
u'\ud5e0': 'HangulSyllableHeok',
u'\u96e2': 'CjkUnifiedIdeograph96e2',
u'\u53e4': 'CjkUnifiedIdeograph53e4',
u'\uff67': 'HalfwidthKatakanaLetterSmallA',
u'\u1ce6': 'UnicodeCharacter0x1ce6',
u'\ub869': 'HangulSyllableRolt',
u'\u856b': 'CjkUnifiedIdeograph856b',
u'\u466d': 'CjkUnifiedIdeograph466d',
u'\u036f': 'CombiningLatinSmallLetterX',
u'\uedf0': 'UnicodeCharacter0xedf0',
u'\uaef2': 'HangulSyllableGgep',
u'\u6bf4': 'CjkUnifiedIdeograph6bf4',
u'\u34f6': 'CjkUnifiedIdeograph34f6',
u'\ud079': 'HangulSyllableKeult',
u'\u9d7b': 'CjkUnifiedIdeograph9d7b',
u'\u5e7d': 'CjkUnifiedIdeograph5e7d',
u'\u1b7f': 'UnicodeCharacter0x1b7f',
u'\u6781': 'CjkUnifiedIdeograph6781',
u'\u8004': 'CjkUnifiedIdeograph8004',
u'\uac87': 'HangulSyllableGeoc',
u'\u0a08': 'GurmukhiLetterIi',
u'\u368b': 'CjkUnifiedIdeograph368b',
u'\uc90a': 'HangulSyllableJulp',
u'\uf58d': 'UnicodeCharacter0xf58d',
u'\u530e': 'CjkUnifiedIdeograph530e',
u'\u7f91': 'CjkUnifiedIdeograph7f91',
u'\u9814': 'CjkUnifiedIdeograph9814',
u'\uc497': 'HangulSyllableSsweoc',
u'\u2218': 'RingOperator',
u'\u4e9b': 'CjkUnifiedIdeograph4e9b',
u'\ue11a': 'UnicodeCharacter0xe11a',
u'\u6b1e': 'CjkUnifiedIdeograph6b1e',
u'\u97a1': 'CjkUnifiedIdeograph97a1',
u'\u11a5': 'UnicodeCharacter0x11a5',
u'\ub024': 'HangulSyllableGgwik',
u'\udca7': 'UnicodeCharacter0xdca7',
u'\u3a28': 'CjkUnifiedIdeograph3a28',
u'\u66ab': 'CjkUnifiedIdeograph66ab',
u'\uf92a': 'CjkCompatibilityIdeographf92a',
u'\u832e': 'CjkUnifiedIdeograph832e',
u'\uafb1': 'HangulSyllableGgyong',
u'\u0d32': 'MalayalamLetterLa',
u'\u29b5': 'CircleWithHorizontalBar',
u'\uc834': 'HangulSyllableJek',
u'\uf4b7': 'UnicodeCharacter0xf4b7',
u'\u5238': 'CjkUnifiedIdeograph5238',
u'\u7ebb': 'CjkUnifiedIdeograph7ebb',
u'\u9b3e': 'CjkUnifiedIdeograph9b3e',
u'\uc7c1': 'HangulSyllableJaeng',
u'\u2542': 'BoxDrawingsVerticalHeavyAndHorizontalLight',
u'\u41c5': 'CjkUnifiedIdeograph41c5',
u'\ue044': 'UnicodeCharacter0xe044',
u'\u6a48': 'CjkUnifiedIdeograph6a48',
u'\u96cb': 'CjkUnifiedIdeograph96cb',
u'\u10cf': 'UnicodeCharacter0x10cf',
u'\ub34e': 'HangulSyllableDyaej',
u'\udfd1': 'UnicodeCharacter0xdfd1',
u'\u3d52': 'CjkUnifiedIdeograph3d52',
u'\u59d5': 'CjkUnifiedIdeograph59d5',
u'\uf854': 'UnicodeCharacter0xf854',
u'\u8258': 'CjkUnifiedIdeograph8258',
u'\uaedb': 'HangulSyllableGgegs',
u'\u0c5c': 'UnicodeCharacter0xc5c',
u'\u28df': 'BraillePatternDots1234578',
u'\ucb5e': 'HangulSyllableJjuj',
u'\uf7e1': 'UnicodeCharacter0xf7e1',
u'\u5562': 'CjkUnifiedIdeograph5562',
u'\u71e5': 'CjkUnifiedIdeograph71e5',
u'\u9a68': 'CjkUnifiedIdeograph9a68',
u'\uc6eb': 'HangulSyllableWegs',
u'\u246c': 'CircledNumberThirteen',
u'\u40ef': 'CjkUnifiedIdeograph40ef',
u'\ue36e': 'UnicodeCharacter0xe36e',
u'\u6d72': 'CjkUnifiedIdeograph6d72',
u'\u89f5': 'CjkUnifiedIdeograph89f5',
u'\u13f9': 'UnicodeCharacter0x13f9',
u'\ub278': 'HangulSyllableNyun',
u'\udefb': 'UnicodeCharacter0xdefb',
u'\u3c7c': 'CjkUnifiedIdeograph3c7c',
u'\u58ff': 'CjkUnifiedIdeograph58ff',
u'\ufb7e': 'ArabicLetterTchehehIsolatedForm',
u'\u6203': 'CjkUnifiedIdeograph6203',
u'\u8582': 'CjkUnifiedIdeograph8582',
u'\u0f86': 'TibetanSignLciRtags',
u'\uab09': 'UnicodeCharacter0xab09',
u'\uca88': 'HangulSyllableJjyeon',
u'\u350d': 'CjkUnifiedIdeograph350d',
u'\u548c': 'CjkUnifiedIdeograph548c',
u'\uf00f': 'UnicodeCharacter0xf00f',
u'\u7a13': 'CjkUnifiedIdeograph7a13',
u'\u9d92': 'CjkUnifiedIdeograph9d92',
u'\u0417': 'CyrillicCapitalLetterZe',
u'\u2796': 'UnicodeCharacter0x2796',
u'\uc319': 'HangulSyllableSsaenj',
u'\ue298': 'UnicodeCharacter0xe298',
u'\u4d1d': 'CjkUnifiedIdeograph4d1d',
u'\u6c9c': 'CjkUnifiedIdeograph6c9c',
u'\u9223': 'CjkUnifiedIdeograph9223',
u'\ub5a2': 'HangulSyllableDdeogg',
u'\u1c27': 'LepchaVowelSignI',
u'\u3fa6': 'CjkUnifiedIdeograph3fa6',
u'\udb29': 'UnicodeCharacter0xdb29',
u'\ufaa8': 'CjkCompatibilityIdeographfaa8',
u'\u652d': 'CjkUnifiedIdeograph652d',
u'\u84ac': 'CjkUnifiedIdeograph84ac',
u'\u0eb0': 'LaoVowelSignA',
u'\uaa33': 'ChamConsonantSignYa',
u'\ucdb2': 'HangulSyllableCweogg',
u'\u3437': 'CjkUnifiedIdeograph3437',
u'\u57b6': 'CjkUnifiedIdeograph57b6',
u'\uf339': 'UnicodeCharacter0xf339',
u'\u7d3d': 'CjkUnifiedIdeograph7d3d',
u'\u9cbc': 'CjkUnifiedIdeograph9cbc',
u'\u0741': 'SyriacQushshaya',
u'\u26c0': 'WhiteDraughtsMan',
u'\uc243': 'HangulSyllableSweolh',
u'\ue5c2': 'UnicodeCharacter0xe5c2',
u'\u4c47': 'CjkUnifiedIdeograph4c47',
u'\u6fc6': 'CjkUnifiedIdeograph6fc6',
u'\u954d': 'CjkUnifiedIdeograph954d',
u'\ub4cc': 'HangulSyllableDyuls',
u'\u1f51': 'GreekSmallLetterUpsilonWithDasia',
u'\u3ed0': 'CjkUnifiedIdeograph3ed0',
u'\uda53': 'UnicodeCharacter0xda53',
u'\ufdd2': 'UnicodeCharacter0xfdd2',
u'\u6457': 'CjkUnifiedIdeograph6457',
u'\u87d6': 'CjkUnifiedIdeograph87d6',
u'\u01da': 'LatinSmallLetterUWithDiaeresisAndCaron',
u'\uad5d': 'HangulSyllableGyolt',
u'\uccdc': 'HangulSyllableCyeols',
u'\u3761': 'CjkUnifiedIdeograph3761',
u'\u56e0': 'CjkUnifiedIdeograph56e0',
u'\uf263': 'UnicodeCharacter0xf263',
u'\u7c67': 'CjkUnifiedIdeograph7c67',
u'\u9fe6': 'UnicodeCharacter0x9fe6',
u'\u066b': 'ArabicDecimalSeparator',
u'\u19ea': 'KhmerSymbolDapKoet',
u'\uc56d': 'HangulSyllableAelt',
u'\ue4ec': 'UnicodeCharacter0xe4ec',
u'\u4f71': 'CjkUnifiedIdeograph4f71',
u'\u6ef0': 'CjkUnifiedIdeograph6ef0',
u'\u9477': 'CjkUnifiedIdeograph9477',
u'\ub7f6': 'HangulSyllableReolm',
u'\u1e7b': 'LatinSmallLetterUWithMacronAndDiaeresis',
u'\u31fa': 'KatakanaLetterSmallMu',
u'\udd7d': 'UnicodeCharacter0xdd7d',
u'\ufcfc': 'ArabicLigatureSeenWithYehIsolatedForm',
u'\ue681': 'UnicodeCharacter0xe681',
u'\u8700': 'CjkUnifiedIdeograph8700',
u'\ua383': 'YiSyllableRrop',
u'\u4402': 'CjkUnifiedIdeograph4402',
u'\u6085': 'CjkUnifiedIdeograph6085',
u'\u0104': 'LatinCapitalLetterAWithOgonek',
u'\u2d87': 'EthiopicSyllableCoa',
u'\ud20e': 'HangulSyllableToep',
u'\ufe91': 'ArabicLetterBehInitialForm',
u'\u9f10': 'CjkUnifiedIdeograph9f10',
u'\ubb93': 'HangulSyllableMwilb',
u'\u5c12': 'CjkUnifiedIdeograph5c12',
u'\u7895': 'CjkUnifiedIdeograph7895',
u'\u1914': 'LimbuLetterMa',
u'\u4597': 'CjkUnifiedIdeograph4597',
u'\u0299': 'LatinLetterSmallCapitalB',
u'\uea1e': 'UnicodeCharacter0xea1e',
u'\ub720': 'HangulSyllableDdyuss',
u'\ud3a3': 'HangulSyllablePelb',
u'\u7422': 'CjkUnifiedIdeograph7422',
u'\u90a5': 'CjkUnifiedIdeograph90a5',
u'\u3124': 'BopomofoLetterAng',
u'\u5da7': 'CjkUnifiedIdeograph5da7',
u'\u1aa9': 'UnicodeCharacter0x1aa9',
u'\ucf30': 'HangulSyllableKyeoss',
u'\uebb3': 'UnicodeCharacter0xebb3',
u'\u8c32': 'CjkUnifiedIdeograph8c32',
u'\ua8b5': 'SaurashtraVowelSignAa',
u'\u4934': 'CjkUnifiedIdeograph4934',
u'\u75b7': 'CjkUnifiedIdeograph75b7',
u'\u1636': 'CanadianSyllabicsCarrierTlhe',
u'\u32b9': 'CircledNumberFortyFour',
u'\ue740': 'UnicodeCharacter0xe740',
u'\ua442': 'YiSyllableNjurx',
u'\uc0c5': 'HangulSyllableSat',
u'\u6144': 'CjkUnifiedIdeograph6144',
u'\u8dc7': 'CjkUnifiedIdeograph8dc7',
u'\u2e46': 'UnicodeCharacter0x2e46',
u'\u4ac9': 'CjkUnifiedIdeograph4ac9',
u'\u17cb': 'KhmerSignBantoc',
u'\uff50': 'FullwidthLatinSmallLetterP',
u'\ubc52': 'HangulSyllableByanh',
u'\ud8d5': 'UnicodeCharacter0xd8d5',
u'\u7954': 'CjkUnifiedIdeograph7954',
u'\ua5d7': 'VaiSyllableCo',
u'\u4656': 'CjkUnifiedIdeograph4656',
u'\u62d9': 'CjkUnifiedIdeograph62d9',
u'\u0358': 'CombiningDotAboveRight',
u'\u2fdb': 'UnicodeCharacter0x2fdb',
u'\ud462': 'HangulSyllablePyonh',
u'\uf0e5': 'UnicodeCharacter0xf0e5',
u'\u9164': 'CjkUnifiedIdeograph9164',
u'\ubde7': 'HangulSyllableBwis',
u'\u5e66': 'CjkUnifiedIdeograph5e66',
u'\u7ae9': 'CjkUnifiedIdeograph7ae9',
u'\u1b68': 'BalineseMusicalSymbolDeung',
u'\u47eb': 'CjkUnifiedIdeograph47eb',
u'\u04ed': 'CyrillicSmallLetterEWithDiaeresis',
u'\uec72': 'UnicodeCharacter0xec72',
u'\ua974': 'UnicodeCharacter0xa974',
u'\ud5f7': 'HangulSyllableHes',
u'\u7676': 'CjkUnifiedIdeograph7676',
u'\u92f9': 'CjkUnifiedIdeograph92f9',
u'\u3378': 'SquareDmSquared',
u'\u5ffb': 'CjkUnifiedIdeograph5ffb',
u'\u1cfd': 'UnicodeCharacter0x1cfd',
u'\u2601': 'Cloud',
u'\uc184': 'HangulSyllableSyess',
u'\u8e86': 'CjkUnifiedIdeograph8e86',
u'\u4b88': 'CjkUnifiedIdeograph4b88',
u'\uf70b': 'UnicodeCharacter0xf70b',
u'\u088a': 'UnicodeCharacter0x88a',
u'\ub40d': 'HangulSyllableDwaeb',
u'\u710f': 'CjkUnifiedIdeograph710f',
u'\u3e11': 'CjkUnifiedIdeograph3e11',
u'\ud994': 'UnicodeCharacter0xd994',
u'\ua696': 'CyrillicCapitalLetterShwe',
u'\u6398': 'CjkUnifiedIdeograph6398',
u'\u209a': 'UnicodeCharacter0x209a',
u'\ucc1d': 'HangulSyllableJjib',
u'\u891f': 'CjkUnifiedIdeograph891f',
u'\u5621': 'CjkUnifiedIdeograph5621',
u'\u1323': 'EthiopicSyllableThaa',
u'\uf1a4': 'UnicodeCharacter0xf1a4',
u'\ubea6': 'HangulSyllableBbyalp',
u'\u7ba8': 'CjkUnifiedIdeograph7ba8',
u'\u38aa': 'CjkUnifiedIdeograph38aa',
u'\ue42d': 'UnicodeCharacter0xe42d',
u'\u05ac': 'HebrewAccentIluy',
u'\ua12f': 'YiSyllableTep',
u'\u6e31': 'CjkUnifiedIdeograph6e31',
u'\u2b33': 'LongLeftwardsSquiggleArrow',
u'\ud6b6': 'HangulSyllableHyolp',
u'\u93b8': 'CjkUnifiedIdeograph93b8',
u'\u50ba': 'CjkUnifiedIdeograph50ba',
u'\ufc3d': 'ArabicLigatureKafWithAlefMaksuraIsolatedForm',
u'\u1dbc': 'ModifierLetterSmallZWithRetroflexHook',
u'\ub93f': 'HangulSyllableRwigs',
u'\u8641': 'CjkUnifiedIdeograph8641',
u'\u4343': 'CjkUnifiedIdeograph4343',
u'\ueec6': 'UnicodeCharacter0xeec6',
u'\uabc8': 'UnicodeCharacter0xabc8',
u'\u68ca': 'CjkUnifiedIdeograph68ca',
u'\u35cc': 'CjkUnifiedIdeograph35cc',
u'\ud14f': 'HangulSyllableTegs',
u'\u9e51': 'CjkUnifiedIdeograph9e51',
u'\u5b53': 'CjkUnifiedIdeograph5b53',
u'\u1855': 'MongolianLetterTodoYa',
u'\uc3d8': 'HangulSyllableSso',
u'\u80da': 'CjkUnifiedIdeograph80da',
u'\u4ddc': 'HexagramForTheAbysmalWater',
u'\ue95f': 'UnicodeCharacter0xe95f',
u'\u0ade': 'UnicodeCharacter0xade',
u'\ub661': 'HangulSyllableDdwaet',
u'\u7363': 'CjkUnifiedIdeograph7363',
u'\u3065': 'HiraganaLetterDu',
u'\udbe8': 'UnicodeCharacter0xdbe8',
u'\u98ea': 'CjkUnifiedIdeograph98ea',
u'\u65ec': 'CjkUnifiedIdeograph65ec',
u'\u22ee': 'VerticalEllipsis',
u'\uce71': 'HangulSyllableCit',
u'\u8b73': 'CjkUnifiedIdeograph8b73',
u'\u4875': 'CjkUnifiedIdeograph4875',
u'\u1577': 'CanadianSyllabicsNunavikHo',
u'\uf3f8': 'UnicodeCharacter0xf3f8',
u'\ub0fa': 'HangulSyllableNyaelp',
u'\u7dfc': 'CjkUnifiedIdeograph7dfc',
u'\u3afe': 'CjkUnifiedIdeograph3afe',
u'\u0800': 'UnicodeCharacter0x800',
u'\u2c83': 'CopticSmallLetterVida',
u'\ucb02': 'HangulSyllableJjwaelp',
u'\uef85': 'UnicodeCharacter0xef85',
u'\u4106': 'CjkUnifiedIdeograph4106',
u'\u6589': 'CjkUnifiedIdeograph6589',
u'\u9a0c': 'CjkUnifiedIdeograph9a0c',
u'\ube8f': 'HangulSyllableBbaes',
u'\u1010': 'MyanmarLetterTa',
u'\u3493': 'CjkUnifiedIdeograph3493',
u'\ud312': 'HangulSyllablePanh',
u'\uf795': 'UnicodeCharacter0xf795',
u'\u6916': 'CjkUnifiedIdeograph6916',
u'\u8d99': 'CjkUnifiedIdeograph8d99',
u'\u039d': 'GreekCapitalLetterNu',
u'\ua21c': 'YiSyllableGga',
u'\uc69f': 'HangulSyllableYolb',
u'\u3820': 'CjkUnifiedIdeograph3820',
u'\u5ca3': 'CjkUnifiedIdeograph5ca3',
u'\ufb22': 'HebrewLetterWideDalet',
u'\u7126': 'CjkUnifiedIdeograph7126',
u'\u95a9': 'CjkUnifiedIdeograph95a9',
u'\u2bad': 'UnicodeCharacter0x2bad',
u'\uca2c': 'HangulSyllableJjyak',
u'\ueeaf': 'UnicodeCharacter0xeeaf',
u'\u4030': 'CjkUnifiedIdeograph4030',
u'\u64b3': 'CjkUnifiedIdeograph64b3',
u'\u9936': 'CjkUnifiedIdeograph9936',
u'\ubdb9': 'HangulSyllableBweg',
u'\u1f3a': 'GreekCapitalLetterIotaWithPsiliAndVaria',
u'\u33bd': 'SquareMw',
u'\ud23c': 'HangulSyllableTum',
u'\uf6bf': 'UnicodeCharacter0xf6bf',
u'\u6840': 'CjkUnifiedIdeograph6840',
u'\u8cc3': 'CjkUnifiedIdeograph8cc3',
u'\u02c7': 'Caron',
u'\ua146': 'YiSyllableDdo',
u'\uc5c9': 'HangulSyllableEong',
u'\u274a': 'EightTeardropspokedPropellerAsterisk',
u'\u5bcd': 'CjkUnifiedIdeograph5bcd',
u'\ufa4c': 'CjkCompatibilityIdeographfa4c',
u'\u7050': 'CjkUnifiedIdeograph7050',
u'\u94d3': 'CjkUnifiedIdeograph94d3',
u'\u2ad7': 'SupersetBesideSubset',
u'\uc956': 'HangulSyllableJwinh',
u'\uedd9': 'UnicodeCharacter0xedd9',
u'\u4f5a': 'CjkUnifiedIdeograph4f5a',
u'\u63dd': 'CjkUnifiedIdeograph63dd',
u'\u9860': 'CjkUnifiedIdeograph9860',
u'\ubce3': 'HangulSyllableByelb',
u'\u1e64': 'LatinCapitalLetterSWithAcuteAndDotAbove',
u'\u32e7': 'CircledKatakanaNe',
u'\ud166': 'HangulSyllableTep',
u'\uf5e9': 'UnicodeCharacter0xf5e9',
u'\u576a': 'CjkUnifiedIdeograph576a',
u'\u8bed': 'CjkUnifiedIdeograph8bed',
u'\u01f1': 'LatinCapitalLetterDz',
u'\ua070': 'YiSyllableBburx',
u'\uc4f3': 'HangulSyllableSseugs',
u'\u2674': 'RecyclingSymbolForType2Plastics',
u'\u5af7': 'CjkUnifiedIdeograph5af7',
u'\uf976': 'CjkCompatibilityIdeographf976',
u'\u7f7a': 'CjkUnifiedIdeograph7f7a',
u'\u93fd': 'CjkUnifiedIdeograph93fd',
u'\ua901': 'KayahLiDigitOne',
u'\uc880': 'HangulSyllableJom',
u'\u2f05': 'KangxiRadicalHook',
u'\u4e84': 'CjkUnifiedIdeograph4e84',
u'\ue207': 'UnicodeCharacter0xe207',
u'\u780b': 'CjkUnifiedIdeograph780b',
u'\u878a': 'CjkUnifiedIdeograph878a',
u'\u1d8e': 'LatinSmallLetterZWithPalatalHook',
u'\ub111': 'HangulSyllableNeolg',
u'\ud090': 'HangulSyllableKyil',
u'\u3715': 'CjkUnifiedIdeograph3715',
u'\u5694': 'CjkUnifiedIdeograph5694',
u'\u801b': 'CjkUnifiedIdeograph801b',
u'\uaf9a': 'HangulSyllableGgoep',
u'\u061f': 'ArabicQuestionMark',
u'\u259e': 'QuadrantUpperRightAndLowerLeft',
u'\ud921': 'UnicodeCharacter0xd921',
u'\uf8a0': 'UnicodeCharacter0xf8a0',
u'\u5f25': 'CjkUnifiedIdeograph5f25',
u'\u7ea4': 'CjkUnifiedIdeograph7ea4',
u'\ua82b': 'SylotiNagriPoetryMark4',
u'\ub7aa': 'HangulSyllableRaebs',
u'\u2e2f': 'VerticalTilde',
u'\u4dae': 'CjkUnifiedIdeograph4dae',
u'\ue131': 'UnicodeCharacter0xe131',
u'\u6735': 'CjkUnifiedIdeograph6735',
u'\u86b4': 'CjkUnifiedIdeograph86b4',
u'\u1cb8': 'UnicodeCharacter0x1cb8',
u'\ub03b': 'HangulSyllableGgyus',
u'\udfba': 'UnicodeCharacter0xdfba',
u'\u363f': 'CjkUnifiedIdeograph363f',
u'\u55be': 'CjkUnifiedIdeograph55be',
u'\u8f45': 'CjkUnifiedIdeograph8f45',
u'\uaec4': 'HangulSyllableGgeol',
u'\u0549': 'ArmenianCapitalLetterCha',
u'\u24c8': 'CircledLatinCapitalLetterS',
u'\ud84b': 'UnicodeCharacter0xd84b',
u'\ue7ca': 'UnicodeCharacter0xe7ca',
u'\u5e4f': 'CjkUnifiedIdeograph5e4f',
u'\u7dce': 'CjkUnifiedIdeograph7dce',
u'\u9755': 'CjkUnifiedIdeograph9755',
u'\ub6d4': 'HangulSyllableDdwe',
u'\u2d59': 'TifinaghLetterYas',
u'\u4cd8': 'CjkUnifiedIdeograph4cd8',
u'\ue05b': 'UnicodeCharacter0xe05b',
u'\u665f': 'CjkUnifiedIdeograph665f',
u'\u85de': 'CjkUnifiedIdeograph85de',
u'\u1be2': 'UnicodeCharacter0x1be2',
u'\ubf65': 'HangulSyllableBbwalg',
u'\udee4': 'UnicodeCharacter0xdee4',
u'\u3569': 'CjkUnifiedIdeograph3569',
u'\u54e8': 'CjkUnifiedIdeograph54e8',
u'\u8e6f': 'CjkUnifiedIdeograph8e6f',
u'\uadee': 'HangulSyllableGyubs',
u'\u0473': 'CyrillicSmallLetterFita',
u'\u23f2': 'UnicodeCharacter0x23f2',
u'\uc775': 'HangulSyllableIg',
u'\ue6f4': 'UnicodeCharacter0xe6f4',
u'\u5d79': 'CjkUnifiedIdeograph5d79',
u'\u7cf8': 'CjkUnifiedIdeograph7cf8',
u'\u967f': 'CjkUnifiedIdeograph967f',
u'\ub5fe': 'HangulSyllableDdyelm',
u'\uc006': 'HangulSyllableBbwegg',
u'\ue489': 'UnicodeCharacter0xe489',
u'\u8508': 'CjkUnifiedIdeograph8508',
u'\ub98b': 'HangulSyllableReuc',
u'\u460a': 'CjkUnifiedIdeograph460a',
u'\u7a8d': 'CjkUnifiedIdeograph7a8d',
u'\u1b0c': 'BalineseLetterRaRepaTedung',
u'\u3f8f': 'CjkUnifiedIdeograph3f8f',
u'\ue816': 'UnicodeCharacter0xe816',
u'\uad18': 'HangulSyllableGwae',
u'\uc19b': 'HangulSyllableSolh',
u'\u6e1a': 'CjkUnifiedIdeograph6e1a',
u'\u829d': 'CjkUnifiedIdeograph829d',
u'\u231c': 'TopLeftCorner',
u'\u479f': 'CjkUnifiedIdeograph479f',
u'\u18a1': 'MongolianLetterManchuAliGaliDha',
u'\uf026': 'UnicodeCharacter0xf026',
u'\ub528': 'HangulSyllableDiss',
u'\ue9ab': 'UnicodeCharacter0xe9ab',
u'\u762a': 'CjkUnifiedIdeograph762a',
u'\uaaad': 'UnicodeCharacter0xaaad',
u'\u4b2c': 'CjkUnifiedIdeograph4b2c',
u'\u6faf': 'CjkUnifiedIdeograph6faf',
u'\u0c2e': 'TeluguLetterMa',
u'\u20b1': 'PesoSign',
u'\udd38': 'UnicodeCharacter0xdd38',
u'\uf1bb': 'UnicodeCharacter0xf1bb',
u'\u9e3a': 'CjkUnifiedIdeograph9e3a',
u'\ub2bd': 'HangulSyllableNyib',
u'\u533c': 'CjkUnifiedIdeograph533c',
u'\u77bf': 'CjkUnifiedIdeograph77bf',
u'\u143e': 'CanadianSyllabicsPwii',
u'\u48c1': 'CjkUnifiedIdeograph48c1',
u'\u0dc3': 'SinhalaLetterDantajaSayanna',
u'\ue548': 'UnicodeCharacter0xe548',
u'\ua64a': 'CyrillicCapitalLetterMonographUk',
u'\udacd': 'UnicodeCharacter0xdacd',
u'\u7b4c': 'CjkUnifiedIdeograph7b4c',
u'\u9fcf': 'UnicodeCharacter0x9fcf',
u'\u3c4e': 'CjkUnifiedIdeograph3c4e',
u'\u50d1': 'CjkUnifiedIdeograph50d1',
u'\u15d3': 'CanadianSyllabicsCarrierWee',
u'\uce5a': 'HangulSyllableCigg',
u'\ue2dd': 'UnicodeCharacter0xe2dd',
u'\u835c': 'CjkUnifiedIdeograph835c',
u'\ua7df': 'UnicodeCharacter0xa7df',
u'\u445e': 'CjkUnifiedIdeograph445e',
u'\u78e1': 'CjkUnifiedIdeograph78e1',
u'\u1960': 'TaiLeLetterKha',
u'\u3de3': 'CjkUnifiedIdeograph3de3',
u'\ud66a': 'HangulSyllableHwaj',
u'\uab6c': 'UnicodeCharacter0xab6c',
u'\ucfef': 'HangulSyllableKulh',
u'\u6c6e': 'CjkUnifiedIdeograph6c6e',
u'\u80f1': 'CjkUnifiedIdeograph80f1',
u'\u2170': 'SmallRomanNumeralOne',
u'\u45f3': 'CjkUnifiedIdeograph45f3',
u'\u06f5': 'ExtendedArabicindicDigitFive',
u'\ufe7a': 'ArabicKasraIsolatedForm',
u'\ub37c': 'HangulSyllableDels',
u'\ud7ff': 'UnicodeCharacter0xd7ff',
u'\u747e': 'CjkUnifiedIdeograph747e',
u'\u4980': 'CjkUnifiedIdeograph4980',
u'\ued03': 'UnicodeCharacter0xed03',
u'\u0a82': 'GujaratiSignAnusvara',
u'\uae05': 'HangulSyllableGeult',
u'\u6307': 'CjkUnifiedIdeograph6307',
u'\u2409': 'SymbolForHorizontalTabulation',
u'\udb8c': 'UnicodeCharacter0xdb8c',
u'\u9c8e': 'CjkUnifiedIdeograph9c8e',
u'\u5190': 'CjkUnifiedIdeograph5190',
u'\uf513': 'UnicodeCharacter0xf513',
u'\u1292': 'EthiopicSyllableNi',
u'\ub615': 'HangulSyllableDdonj',
u'\u8b17': 'CjkUnifiedIdeograph8b17',
u'\u4c19': 'CjkUnifiedIdeograph4c19',
u'\u011b': 'LatinSmallLetterEWithCaron',
u'\ue39c': 'UnicodeCharacter0xe39c',
u'\ua49e': 'YiRadicalPut',
u'\u79a0': 'CjkUnifiedIdeograph79a0',
u'\u3aa2': 'CjkUnifiedIdeograph3aa2',
u'\ude25': 'UnicodeCharacter0xde25',
u'\u9327': 'CjkUnifiedIdeograph9327',
u'\u5429': 'CjkUnifiedIdeograph5429',
u'\u292b': 'RisingDiagonalCrossingFallingDiagonal',
u'\uccae': 'HangulSyllableCeoj',
u'\u81b0': 'CjkUnifiedIdeograph81b0',
u'\u42b2': 'CjkUnifiedIdeograph42b2',
u'\ue635': 'UnicodeCharacter0xe635',
u'\u07b4': 'UnicodeCharacter0x7b4',
u'\ubb37': 'HangulSyllableMugs',
u'\u7c39': 'CjkUnifiedIdeograph7c39',
u'\u313b': 'HangulLetterRieulmieum',
u'\ud4be': 'HangulSyllablePwelp',
u'\ua9c0': 'UnicodeCharacter0xa9c0',
u'\u6ac2': 'CjkUnifiedIdeograph6ac2',
u'\u2fc4': 'KangxiRadicalSalt',
u'\uc347': 'HangulSyllableSsyac',
u'\u8449': 'CjkUnifiedIdeograph8449',
u'\u594b': 'CjkUnifiedIdeograph594b',
u'\u1a4d': 'UnicodeCharacter0x1a4d',
u'\ufcce': 'ArabicLigatureMeemWithJeemInitialForm',
u'\ub1d0': 'HangulSyllableNoen',
u'\u72d2': 'CjkUnifiedIdeograph72d2',
u'\u37d4': 'CjkUnifiedIdeograph37d4',
u'\ueb57': 'UnicodeCharacter0xeb57',
u'\u08d6': 'UnicodeCharacter0x8d6',
u'\uac59': 'HangulSyllableGyaenj',
u'\u615b': 'CjkUnifiedIdeograph615b',
u'\u225d': 'EqualToByDefinition',
u'\ud9e0': 'UnicodeCharacter0xd9e0',
u'\u9ae2': 'CjkUnifiedIdeograph9ae2',
u'\u5fe4': 'CjkUnifiedIdeograph5fe4',
u'\uf367': 'UnicodeCharacter0xf367',
u'\u10e6': 'GeorgianLetterGhan',
u'\ub469': 'HangulSyllableDut',
u'\u896b': 'CjkUnifiedIdeograph896b',
u'\u4a6d': 'CjkUnifiedIdeograph4a6d',
u'\u0f6f': 'UnicodeCharacter0xf6f',
u'\ue1f0': 'UnicodeCharacter0xe1f0',
u'\ua2f2': 'YiSyllableNzie',
u'\u67f4': 'CjkUnifiedIdeograph67f4',
u'\u38f6': 'CjkUnifiedIdeograph38f6',
u'\udc79': 'UnicodeCharacter0xdc79',
u'\u917b': 'CjkUnifiedIdeograph917b',
u'\u527d': 'CjkUnifiedIdeograph527d',
u'\u177f': 'UnicodeCharacter0x177f',
u'\u6b81': 'CjkUnifiedIdeograph6b81',
u'\u8c04': 'CjkUnifiedIdeograph8c04',
u'\ua087': 'YiSyllableNbu',
u'\u0608': 'ArabicRay',
u'\u3a8b': 'CjkUnifiedIdeograph3a8b',
u'\uc50a': 'HangulSyllableSseup',
u'\uf98d': 'CjkCompatibilityIdeographf98d',
u'\u5f0e': 'CjkUnifiedIdeograph5f0e',
u'\u7391': 'CjkUnifiedIdeograph7391',
u'\u9414': 'CjkUnifiedIdeograph9414',
u'\uc897': 'HangulSyllableJwalb',
u'\u2e18': 'InvertedInterrobang',
u'\u429b': 'CjkUnifiedIdeograph429b',
u'\ued1a': 'UnicodeCharacter0xed1a',
u'\u671e': 'CjkUnifiedIdeograph671e',
u'\u9ba1': 'CjkUnifiedIdeograph9ba1',
u'\u1da5': 'ModifierLetterSmallIota',
u'\ubc24': 'HangulSyllableBam',
u'\ud0a7': 'HangulSyllableKigs',
u'\u3628': 'CjkUnifiedIdeograph3628',
u'\u6aab': 'CjkUnifiedIdeograph6aab',
u'\uf52a': 'UnicodeCharacter0xf52a',
u'\u8f2e': 'CjkUnifiedIdeograph8f2e',
u'\ua3b1': 'YiSyllableShuop',
u'\u0132': 'LatinCapitalLigatureIj',
u'\u25b5': 'WhiteUppointingSmallTriangle',
u'\uc434': 'HangulSyllableSsoel',
u'\uf8b7': 'UnicodeCharacter0xf8b7',
u'\u5e38': 'CjkUnifiedIdeograph5e38',
u'\u72bb': 'CjkUnifiedIdeograph72bb',
u'\u973e': 'CjkUnifiedIdeograph973e',
u'\ucbc1': 'HangulSyllableJjyulg',
u'\u2942': 'RightwardsArrowAboveShortLeftwardsArrow',
u'\u4dc5': 'HexagramForConflict',
u'\uec44': 'UnicodeCharacter0xec44',
u'\u6648': 'CjkUnifiedIdeograph6648',
u'\u9acb': 'CjkUnifiedIdeograph9acb',
u'\u1ccf': 'UnicodeCharacter0x1ccf',
u'\ubf4e': 'HangulSyllableBbolp',
u'\ud3d1': 'HangulSyllablePyeg',
u'\u3152': 'HangulLetterYae',
u'\u55d5': 'CjkUnifiedIdeograph55d5',
u'\uf454': 'UnicodeCharacter0xf454',
u'\u8e58': 'CjkUnifiedIdeograph8e58',
u'\ua2db': 'YiSyllableZzap',
u'\u24df': 'CircledLatinSmallLetterP',
u'\uc75e': 'HangulSyllableYinh',
u'\ufbe1': 'ArabicLetterKirghizOeFinalForm',
u'\u5962': 'CjkUnifiedIdeograph5962',
u'\u7de5': 'CjkUnifiedIdeograph7de5',
u'\u9668': 'CjkUnifiedIdeograph9668',
u'\ucaeb': 'HangulSyllableJjwas',
u'\u286c': 'BraillePatternDots3467',
u'\u4cef': 'CjkUnifiedIdeograph4cef',
u'\uef6e': 'UnicodeCharacter0xef6e',
u'\u6172': 'CjkUnifiedIdeograph6172',
u'\u85f5': 'CjkUnifiedIdeograph85f5',
u'\u1ff9': 'GreekCapitalLetterOmicronWithOxia',
u'\ube78': 'HangulSyllableBbak',
u'\ud2fb': 'HangulSyllableTilb',
u'\u307c': 'HiraganaLetterBo',
u'\u54ff': 'CjkUnifiedIdeograph54ff',
u'\uf77e': 'UnicodeCharacter0xf77e',
u'\u6e03': 'CjkUnifiedIdeograph6e03',
u'\u8982': 'CjkUnifiedIdeograph8982',
u'\u0386': 'GreekCapitalLetterAlphaWithTonos',
u'\ua709': 'ModifierLetterHighDottedToneBar',
u'\uc688': 'HangulSyllableOem',
u'\u390d': 'CjkUnifiedIdeograph390d',
u'\u588c': 'CjkUnifiedIdeograph588c',
u'\ufc0f': 'ArabicLigatureTehWithAlefMaksuraIsolatedForm',
u'\u7613': 'CjkUnifiedIdeograph7613',
u'\u9192': 'CjkUnifiedIdeograph9192',
u'\u0817': 'UnicodeCharacter0x817',
u'\u2b96': 'UnicodeCharacter0x2b96',
u'\ucf19': 'HangulSyllableKet',
u'\uee98': 'UnicodeCharacter0xee98',
u'\u411d': 'CjkUnifiedIdeograph411d',
u'\u609c': 'CjkUnifiedIdeograph609c',
u'\u9e23': 'CjkUnifiedIdeograph9e23',
u'\ub9a2': 'HangulSyllableRyibs',
u'\u1027': 'MyanmarLetterE',
u'\u33a6': 'SquareKmCubed',
u'\ud729': 'HangulSyllableHwib',
u'\uf6a8': 'UnicodeCharacter0xf6a8',
u'\u692d': 'CjkUnifiedIdeograph692d',
u'\u88ac': 'CjkUnifiedIdeograph88ac',
u'\u02b0': 'ModifierLetterSmallH',
u'\ua633': 'UnicodeCharacter0xa633',
u'\uc1b2': 'HangulSyllableSwalm',
u'\u3837': 'CjkUnifiedIdeograph3837',
u'\u5bb6': 'CjkUnifiedIdeograph5bb6',
u'\uff39': 'FullwidthLatinCapitalLetterY',
u'\u713d': 'CjkUnifiedIdeograph713d',
u'\u90bc': 'CjkUnifiedIdeograph90bc',
u'\u0b41': 'OriyaVowelSignU',
u'\u2ac0': 'SupersetWithPlusSignBelow',
u'\uce43': 'HangulSyllableCyid',
u'\ue9c2': 'UnicodeCharacter0xe9c2',
u'\u4047': 'CjkUnifiedIdeograph4047',
u'\u63c6': 'CjkUnifiedIdeograph63c6',
u'\u994d': 'CjkUnifiedIdeograph994d',
u'\ub8cc': 'HangulSyllableRyo',
u'\u1351': 'EthiopicSyllablePu',
u'\u32d0': 'CircledKatakanaA',
u'\ud653': 'HangulSyllableHoh',
u'\uf1d2': 'UnicodeCharacter0xf1d2',
u'\u6857': 'CjkUnifiedIdeograph6857',
u'\u8bd6': 'CjkUnifiedIdeograph8bd6',
u'\u0dda': 'SinhalaVowelSignDigaKombuva',
u'\ua15d': 'YiSyllableNdo',
u'\uc0dc': 'HangulSyllableSaess',
u'\u3b61': 'CjkUnifiedIdeograph3b61',
u'\u5ae0': 'CjkUnifiedIdeograph5ae0',
u'\ufe63': 'SmallHyphenminus',
u'\u7067': 'CjkUnifiedIdeograph7067',
u'\u93e6': 'CjkUnifiedIdeograph93e6',
u'\u0a6b': 'GurmukhiDigitFive',
u'\u15ea': 'CanadianSyllabicsCarrierPe',
u'\uc96d': 'HangulSyllableJyug',
u'\ue8ec': 'UnicodeCharacter0xe8ec',
u'\u4371': 'CjkUnifiedIdeograph4371',
u'\u62f0': 'CjkUnifiedIdeograph62f0',
u'\u9877': 'CjkUnifiedIdeograph9877',
u'\ubbf6': 'HangulSyllableMyip',
u'\u127b': 'EthiopicSyllableCaa',
u'\u3dfa': 'CjkUnifiedIdeograph3dfa',
u'\ud17d': 'HangulSyllableTyeong',
u'\uf0fc': 'UnicodeCharacter0xf0fc',
u'\uea81': 'UnicodeCharacter0xea81',
u'\u8b00': 'CjkUnifiedIdeograph8b00',
u'\uaf83': 'HangulSyllableGgoegs',
u'\u4802': 'CjkUnifiedIdeograph4802',
u'\u6c85': 'CjkUnifiedIdeograph6c85',
u'\u0d04': 'UnicodeCharacter0xd04',
u'\u2187': 'RomanNumeralFiftyThousand',
u'\ude0e': 'UnicodeCharacter0xde0e',
u'\uf291': 'UnicodeCharacter0xf291',
u'\u9310': 'CjkUnifiedIdeograph9310',
u'\ub793': 'HangulSyllableRac',
u'\u5012': 'CjkUnifiedIdeograph5012',
u'\u7495': 'CjkUnifiedIdeograph7495',
u'\u1514': 'CanadianSyllabicsShoo',
u'\u4997': 'CjkUnifiedIdeograph4997',
u'\u0e99': 'LaoLetterNo',
u'\ue61e': 'UnicodeCharacter0xe61e',
u'\ubb20': 'HangulSyllableMyol',
u'\udfa3': 'UnicodeCharacter0xdfa3',
u'\u7822': 'CjkUnifiedIdeograph7822',
u'\u9ca5': 'CjkUnifiedIdeograph9ca5',
u'\u3d24': 'CjkUnifiedIdeograph3d24',
u'\u51a7': 'CjkUnifiedIdeograph51a7',
u'\u16a9': 'RunicLetterOsO',
u'\uc330': 'HangulSyllableSsya',
u'\ue7b3': 'UnicodeCharacter0xe7b3',
u'\u8032': 'CjkUnifiedIdeograph8032',
u'\ua4b5': 'YiRadicalJjy',
u'\u4534': 'CjkUnifiedIdeograph4534',
u'\u79b7': 'CjkUnifiedIdeograph79b7',
u'\u1a36': 'UnicodeCharacter0x1a36',
u'\u3eb9': 'CjkUnifiedIdeograph3eb9',
u'\ueb40': 'UnicodeCharacter0xeb40',
u'\ua842': 'PhagspaLetterGa',
u'\uccc5': 'HangulSyllableCeb',
u'\u6d44': 'CjkUnifiedIdeograph6d44',
u'\u81c7': 'CjkUnifiedIdeograph81c7',
u'\u2246': 'ApproximatelyButNotActuallyEqualTo',
u'\u46c9': 'CjkUnifiedIdeograph46c9',
u'\u1bcb': 'UnicodeCharacter0x1bcb',
u'\uf350': 'UnicodeCharacter0xf350',
u'\ub052': 'HangulSyllableGgeulp',
u'\ud4d5': 'HangulSyllablePwilg',
u'\u7554': 'CjkUnifiedIdeograph7554',
u'\ua9d7': 'UnicodeCharacter0xa9d7',
u'\u4a56': 'CjkUnifiedIdeograph4a56',
u'\u6ed9': 'CjkUnifiedIdeograph6ed9',
u'\u0f58': 'TibetanLetterMa',
u'\u23db': 'Fuse',
u'\ud862': 'UnicodeCharacter0xd862',
u'\ufce5': 'ArabicLigatureThehWithMeemMedialForm',
u'\u9d64': 'CjkUnifiedIdeograph9d64',
u'\ub1e7': 'HangulSyllableNoeh',
u'\u5266': 'CjkUnifiedIdeograph5266',
u'\u76e9': 'CjkUnifiedIdeograph76e9',
u'\u1768': 'TagbanwaLetterNa',
u'\u4beb': 'CjkUnifiedIdeograph4beb',
u'\u08ed': 'UnicodeCharacter0x8ed',
u'\ue072': 'UnicodeCharacter0xe072',
u'\ua574': 'VaiSyllableWoo',
u'\ud9f7': 'UnicodeCharacter0xd9f7',
u'\u7a76': 'CjkUnifiedIdeograph7a76',
u'\u9ef9': 'CjkUnifiedIdeograph9ef9',
u'\u3f78': 'CjkUnifiedIdeograph3f78',
u'\u53fb': 'CjkUnifiedIdeograph53fb',
u'\u10fd': 'UnicodeCharacter0x10fd',
u'\u2a01': 'NaryCircledPlusOperator',
u'\ucd84': 'HangulSyllableCyols',
u'\u8286': 'CjkUnifiedIdeograph8286',
u'\u4788': 'CjkUnifiedIdeograph4788',
u'\ufb0b': 'UnicodeCharacter0xfb0b',
u'\u048a': 'CyrillicCapitalLetterShortIWithTail',
u'\ub80d': 'HangulSyllableRenj',
u'\u7d0f': 'CjkUnifiedIdeograph7d0f',
u'\u3211': 'ParenthesizedHangulRieulA',
u'\ud594': 'HangulSyllableHyan',
u'\uaa96': 'UnicodeCharacter0xaa96',
u'\u6f98': 'CjkUnifiedIdeograph6f98',
u'\u2c9a': 'CopticCapitalLetterNi',
u'\uc01d': 'HangulSyllableBbwet',
u'\u851f': 'CjkUnifiedIdeograph851f',
u'\u5a21': 'CjkUnifiedIdeograph5a21',
u'\u1f23': 'GreekSmallLetterEtaWithDasiaAndVaria',
u'\ufda4': 'ArabicLigatureTehWithMeemWithAlefMaksuraFinalForm',
u'\ub2a6': 'HangulSyllableNeuj',
u'\u77a8': 'CjkUnifiedIdeograph77a8',
u'\u34aa': 'CjkUnifiedIdeograph34aa',
u'\ue82d': 'UnicodeCharacter0xe82d',
u'\u09ac': 'BengaliLetterBa',
u'\uad2f': 'HangulSyllableGwaec',
u'\u6231': 'CjkUnifiedIdeograph6231',
u'\u2733': 'EightSpokedAsterisk',
u'\udab6': 'UnicodeCharacter0xdab6',
u'\u9fb8': 'CjkUnifiedIdeograph9fb8',
u'\u5cba': 'CjkUnifiedIdeograph5cba',
u'\uf03d': 'UnicodeCharacter0xf03d',
u'\u11bc': 'HangulJongseongIeung',
u'\ub53f': 'HangulSyllableDdalh',
u'\u8a41': 'CjkUnifiedIdeograph8a41',
u'\u4f43': 'CjkUnifiedIdeograph4f43',
u'\u0c45': 'UnicodeCharacter0xc45',
u'\ue2c6': 'UnicodeCharacter0xe2c6',
u'\ua7c8': 'UnicodeCharacter0xa7c8',
u'\u64ca': 'CjkUnifiedIdeograph64ca',
u'\u39cc': 'CjkUnifiedIdeograph39cc',
u'\udd4f': 'UnicodeCharacter0xdd4f',
u'\u9251': 'CjkUnifiedIdeograph9251',
u'\u5753': 'CjkUnifiedIdeograph5753',
u'\u1455': 'CanadianSyllabicsTa',
u'\ucfd8': 'HangulSyllableKyoss',
u'\u8cda': 'CjkUnifiedIdeograph8cda',
u'\u41dc': 'CjkUnifiedIdeograph41dc',
u'\ue55f': 'UnicodeCharacter0xe55f',
u'\u06de': 'ArabicStartOfRubElHizb',
u'\uba61': 'HangulSyllableMelt',
u'\u7f63': 'CjkUnifiedIdeograph7f63',
u'\u3c65': 'CjkUnifiedIdeograph3c65',
u'\ud7e8': 'UnicodeCharacter0xd7e8',
u'\u94ea': 'CjkUnifiedIdeograph94ea',
u'\u69ec': 'CjkUnifiedIdeograph69ec',
u'\u2eee': 'CjkRadicalCsimplifiedTooth',
u'\uc271': 'HangulSyllableSwinj',
u'\u8773': 'CjkUnifiedIdeograph8773',
u'\u4475': 'CjkUnifiedIdeograph4475',
u'\u1977': 'UnicodeCharacter0x1977',
u'\ufff8': 'UnicodeCharacter0xfff8',
u'\ubcfa': 'HangulSyllableBonh',
u'\u71fc': 'CjkUnifiedIdeograph71fc',
u'\u36fe': 'CjkUnifiedIdeograph36fe',
u'\u0c00': 'UnicodeCharacter0xc00',
u'\u2883': 'BraillePatternDots128',
u'\ucf02': 'HangulSyllableKegg',
u'\ueb85': 'UnicodeCharacter0xeb85',
u'\u4506': 'CjkUnifiedIdeograph4506',
u'\u6189': 'CjkUnifiedIdeograph6189',
u'\u9e0c': 'CjkUnifiedIdeograph9e0c',
u'\uba8f': 'HangulSyllableMyegs',
u'\u1410': 'CanadianSyllabicsWii',
u'\u3093': 'HiraganaLetterN',
u'\ud712': 'HangulSyllableHwej',
u'\uf395': 'UnicodeCharacter0xf395',
u'\u6d16': 'CjkUnifiedIdeograph6d16',
u'\u8999': 'CjkUnifiedIdeograph8999',
u'\u079d': 'ThaanaLetterSheenu',
u'\ua61c': 'VaiSymbolKung',
u'\uc29f': 'HangulSyllableSyuc',
u'\u3c20': 'CjkUnifiedIdeograph3c20',
u'\u58a3': 'CjkUnifiedIdeograph58a3',
u'\uff22': 'FullwidthLatinCapitalLetterB',
u'\u7526': 'CjkUnifiedIdeograph7526',
u'\u91a9': 'CjkUnifiedIdeograph91a9',
u'\u2fad': 'KangxiRadicalBlue',
u'\uce2c': 'HangulSyllableCeuls',
u'\ueaaf': 'UnicodeCharacter0xeaaf',
u'\u4430': 'CjkUnifiedIdeograph4430',
u'\u60b3': 'CjkUnifiedIdeograph60b3',
u'\u9d36': 'CjkUnifiedIdeograph9d36',
u'\ub9b9': 'HangulSyllableRilt',
u'\u1b3a': 'BalineseVowelSignRaRepa',
u'\u37bd': 'CjkUnifiedIdeograph37bd',
u'\ud63c': 'HangulSyllableHon',
u'\uf2bf': 'UnicodeCharacter0xf2bf',
u'\u6c40': 'CjkUnifiedIdeograph6c40',
u'\u88c3': 'CjkUnifiedIdeograph88c3',
u'\u06c7': 'ArabicLetterU',
u'\ua546': 'VaiSyllableMi',
u'\uc1c9': 'HangulSyllableSwaenj',
u'\u234a': 'AplFunctionalSymbolDownTackUnderbar',
u'\u5fcd': 'CjkUnifiedIdeograph5fcd',
u'\ufe4c': 'DoubleWavyOverline',
u'\u7450': 'CjkUnifiedIdeograph7450',
u'\u90d3': 'CjkUnifiedIdeograph90d3',
u'\u2ed7': 'CjkRadicalRain',
u'\ucd56': 'HangulSyllableCwaej',
u'\ue9d9': 'UnicodeCharacter0xe9d9',
u'\u4b5a': 'CjkUnifiedIdeograph4b5a',
u'\u67dd': 'CjkUnifiedIdeograph67dd',
u'\u9c60': 'CjkUnifiedIdeograph9c60',
u'\ub8e3': 'HangulSyllableRyoc',
u'\u1a64': 'UnicodeCharacter0x1a64',
u'\u36e7': 'CjkUnifiedIdeograph36e7',
u'\ud566': 'HangulSyllableHalp',
u'\uf1e9': 'UnicodeCharacter0xf1e9',
u'\u536a': 'CjkUnifiedIdeograph536a',
u'\u8fed': 'CjkUnifiedIdeograph8fed',
u'\u05f1': 'HebrewLigatureYiddishVavYod',
u'\ua470': 'YiSyllableXyr',
u'\uc0f3': 'HangulSyllableSyalh',
u'\u2274': 'NeitherLessthanNorEquivalentTo',
u'\u5ef7': 'CjkUnifiedIdeograph5ef7',
u'\ufd76': 'ArabicLigatureAinWithMeemWithMeemFinalForm',
u'\u7b7a': 'CjkUnifiedIdeograph7b7a',
u'\u97fd': 'CjkUnifiedIdeograph97fd',
u'\uad01': 'HangulSyllableGwanj',
u'\ucc80': 'HangulSyllableCyaen',
u'\u2b05': 'LeftwardsBlackArrow',
u'\u4a84': 'CjkUnifiedIdeograph4a84',
u'\ue607': 'UnicodeCharacter0xe607',
u'\u7c0b': 'CjkUnifiedIdeograph7c0b',
u'\u838a': 'CjkUnifiedIdeograph838a',
u'\u198e': 'NewTaiLueLetterHighTa',
u'\ub511': 'HangulSyllableDyit',
u'\ud490': 'HangulSyllablePuk',
u'\u3315': 'SquareKiroguramu',
u'\u5294': 'CjkUnifiedIdeograph5294',
u'\u841b': 'CjkUnifiedIdeograph841b',
u'\uab9a': 'UnicodeCharacter0xab9a',
u'\u021f': 'LatinSmallLetterHWithCaron',
u'\u219e': 'LeftwardsTwoHeadedArrow',
u'\udd21': 'UnicodeCharacter0xdd21',
u'\ufca0': 'ArabicLigatureBehWithHehInitialForm',
u'\u5b25': 'CjkUnifiedIdeograph5b25',
u'\u7aa4': 'CjkUnifiedIdeograph7aa4',
u'\uac2b': 'HangulSyllableGaelh',
u'\ub3aa': 'HangulSyllableDyegg',
u'\u2a2f': 'VectorOrCrossProduct',
u'\u49ae': 'CjkUnifiedIdeograph49ae',
u'\ue531': 'UnicodeCharacter0xe531',
u'\u6335': 'CjkUnifiedIdeograph6335',
u'\u82b4': 'CjkUnifiedIdeograph82b4',
u'\u18b8': 'UnicodeCharacter0x18b8',
u'\ub43b': 'HangulSyllableDyod',
u'\udbba': 'UnicodeCharacter0xdbba',
u'\u323f': 'ParenthesizedIdeographAlliance',
u'\u51be': 'CjkUnifiedIdeograph51be',
u'\u8b45': 'CjkUnifiedIdeograph8b45',
u'\uaac4': 'UnicodeCharacter0xaac4',
u'\u0149': 'LatinSmallLetterNPrecededByApostrophe',
u'\u20c8': 'UnicodeCharacter0x20c8',
u'\udc4b': 'UnicodeCharacter0xdc4b',
u'\ue3ca': 'UnicodeCharacter0xe3ca',
u'\u5a4f': 'CjkUnifiedIdeograph5a4f',
u'\u79ce': 'CjkUnifiedIdeograph79ce',
u'\u9355': 'CjkUnifiedIdeograph9355',
u'\ub2d4': 'HangulSyllableNils',
u'\u2959': 'DownwardsHarpoonWithBarbLeftToBar',
u'\u48d8': 'CjkUnifiedIdeograph48d8',
u'\ue45b': 'UnicodeCharacter0xe45b',
u'\u625f': 'CjkUnifiedIdeograph625f',
u'\u81de': 'CjkUnifiedIdeograph81de',
u'\u1fe2': 'GreekSmallLetterUpsilonWithDialytikaAndVaria',
u'\ubb65': 'HangulSyllableMweong',
u'\udae4': 'UnicodeCharacter0xdae4',
u'\u3169': 'HangulLetterRieulkiyeoksios',
u'\u50e8': 'CjkUnifiedIdeograph50e8',
u'\u8a6f': 'CjkUnifiedIdeograph8a6f',
u'\ua9ee': 'UnicodeCharacter0xa9ee',
u'\u27f2': 'AnticlockwiseGappedCircleArrow',
u'\uc375': 'HangulSyllableSseolt',
u'\ue2f4': 'UnicodeCharacter0xe2f4',
u'\u5979': 'CjkUnifiedIdeograph5979',
u'\u78f8': 'CjkUnifiedIdeograph78f8',
u'\u927f': 'CjkUnifiedIdeograph927f',
u'\ub1fe': 'HangulSyllableNyoj',
u'\uc406': 'HangulSyllableSswabs',
u'\ue089': 'UnicodeCharacter0xe089',
u'\u8108': 'CjkUnifiedIdeograph8108',
u'\ubd8b': 'HangulSyllableBulb',
u'\u420a': 'CjkUnifiedIdeograph420a',
u'\u7e8d': 'CjkUnifiedIdeograph7e8d',
u'\u1f0c': 'GreekCapitalLetterAlphaWithPsiliAndOxia',
u'\u3b8f': 'CjkUnifiedIdeograph3b8f',
u'\uec16': 'UnicodeCharacter0xec16',
u'\ua918': 'KayahLiLetterDa',
u'\uc59b': 'HangulSyllableYaegs',
u'\u6a1a': 'CjkUnifiedIdeograph6a1a',
u'\u869d': 'CjkUnifiedIdeograph869d',
u'\u271c': 'HeavyOpenCentreCross',
u'\u439f': 'CjkUnifiedIdeograph439f',
u'\u1ca1': 'UnicodeCharacter0x1ca1',
u'\uf426': 'UnicodeCharacter0xf426',
u'\ub128': 'HangulSyllableNen',
u'\uedab': 'UnicodeCharacter0xedab',
u'\u722a': 'CjkUnifiedIdeograph722a',
u'\uaead': 'HangulSyllableGgyaelt',
u'\u4f2c': 'CjkUnifiedIdeograph4f2c',
u'\u6baf': 'CjkUnifiedIdeograph6baf',
u'\u082e': 'UnicodeCharacter0x82e',
u'\u24b1': 'ParenthesizedLatinSmallLetterV',
u'\ud938': 'UnicodeCharacter0xd938',
u'\uf5bb': 'UnicodeCharacter0xf5bb',
u'\u9a3a': 'CjkUnifiedIdeograph9a3a',
u'\ub6bd': 'HangulSyllableDdweonj',
u'\u573c': 'CjkUnifiedIdeograph573c',
u'\u73bf': 'CjkUnifiedIdeograph73bf',
u'\u103e': 'MyanmarConsonantSignMedialHa',
u'\u4cc1': 'CjkUnifiedIdeograph4cc1',
u'\u09c3': 'BengaliVowelSignVocalicR',
u'\ue148': 'UnicodeCharacter0xe148',
u'\ua24a': 'YiSyllableHxiet',
u'\udecd': 'UnicodeCharacter0xdecd',
u'\u7f4c': 'CjkUnifiedIdeograph7f4c',
u'\u9bcf': 'CjkUnifiedIdeograph9bcf',
u'\u384e': 'CjkUnifiedIdeograph384e',
u'\u54d1': 'CjkUnifiedIdeograph54d1',
u'\u11d3': 'HangulJongseongRieulpieupsios',
u'\uca5a': 'HangulSyllableJjeolp',
u'\ue6dd': 'UnicodeCharacter0xe6dd',
u'\u875c': 'CjkUnifiedIdeograph875c',
u'\ua3df': 'YiSyllableRyr',
u'\u405e': 'CjkUnifiedIdeograph405e',
u'\u7ce1': 'CjkUnifiedIdeograph7ce1',
u'\u1d60': 'ModifierLetterSmallGreekPhi',
u'\u39e3': 'CjkUnifiedIdeograph39e3',
u'\ud26a': 'HangulSyllableTwenh',
u'\uaf6c': 'HangulSyllableGgwael',
u'\ucbef': 'HangulSyllableJjeuh',
u'\u686e': 'CjkUnifiedIdeograph686e',
u'\u84f1': 'CjkUnifiedIdeograph84f1',
u'\u2570': 'BoxDrawingsLightArcUpAndRight',
u'\u41f3': 'CjkUnifiedIdeograph41f3',
u'\u02f5': 'ModifierLetterMiddleDoubleGraveAccent',
u'\ufa7a': 'CjkCompatibilityIdeographfa7a',
u'\ub77c': 'HangulSyllableRa',
u'\ud3ff': 'HangulSyllablePos',
u'\u707e': 'CjkUnifiedIdeograph707e',
u'\u4d80': 'CjkUnifiedIdeograph4d80',
u'\ue903': 'UnicodeCharacter0xe903',
u'\u0e82': 'LaoLetterKhoSung',
u'\uaa05': 'ChamLetterO',
u'\u6707': 'CjkUnifiedIdeograph6707',
u'\u2009': 'ThinSpace',
u'\udf8c': 'UnicodeCharacter0xdf8c',
u'\u988e': 'CjkUnifiedIdeograph988e',
u'\u5590': 'CjkUnifiedIdeograph5590',
u'\uf113': 'UnicodeCharacter0xf113',
u'\u1692': 'OghamLetterUr',
u'\ub215': 'HangulSyllableNub',
u'\u8f17': 'CjkUnifiedIdeograph8f17',
u'\u4819': 'CjkUnifiedIdeograph4819',
u'\u051b': 'CyrillicSmallLetterQa',
u'\ue79c': 'UnicodeCharacter0xe79c',
u'\ua09e': 'YiSyllableHmuop',
u'\u7da0': 'CjkUnifiedIdeograph7da0',
u'\u3ea2': 'CjkUnifiedIdeograph3ea2',
u'\uda25': 'UnicodeCharacter0xda25',
u'\u9727': 'CjkUnifiedIdeograph9727',
u'\u5029': 'CjkUnifiedIdeograph5029',
u'\u2d2b': 'UnicodeCharacter0x2d2b',
u'\uc8ae': 'HangulSyllableJwaenh',
u'\u85b0': 'CjkUnifiedIdeograph85b0',
u'\u46b2': 'CjkUnifiedIdeograph46b2',
u'\ue235': 'UnicodeCharacter0xe235',
u'\u03b4': 'GreekSmallLetterDelta',
u'\ubf37': 'HangulSyllableBbyes',
u'\u7839': 'CjkUnifiedIdeograph7839',
u'\u353b': 'CjkUnifiedIdeograph353b',
u'\ud0be': 'HangulSyllableKip',
u'\uadc0': 'HangulSyllableGwi',
u'\u6ec2': 'CjkUnifiedIdeograph6ec2',
u'\u2bc4': 'UnicodeCharacter0x2bc4',
u'\uc747': 'HangulSyllableEulb',
u'\u8049': 'CjkUnifiedIdeograph8049',
u'\u5d4b': 'CjkUnifiedIdeograph5d4b',
u'\u1e4d': 'LatinSmallLetterOWithTildeAndAcute',
u'\uf8ce': 'UnicodeCharacter0xf8ce',
u'\ub5d0': 'HangulSyllableDdess',
u'\u76d2': 'CjkUnifiedIdeograph76d2',
u'\u33d4': 'SquareMbSmall',
u'\uef57': 'UnicodeCharacter0xef57',
u'\u0cd6': 'KannadaAiLengthMark',
u'\ua859': 'PhagspaLetterLa',
u'\u655b': 'CjkUnifiedIdeograph655b',
u'\u265d': 'BlackChessBishop',
u'\udde0': 'UnicodeCharacter0xdde0',
u'\u9ee2': 'CjkUnifiedIdeograph9ee2',
u'\u5be4': 'CjkUnifiedIdeograph5be4',
u'\uf767': 'UnicodeCharacter0xf767',
u'\u14e6': 'CanadianSyllabicsLwa',
u'\ub069': 'HangulSyllableGgyilg',
u'\u8d6b': 'CjkUnifiedIdeograph8d6b',
u'\u4e6d': 'CjkUnifiedIdeograph4e6d',
u'\u0b6f': 'OriyaDigitNine',
u'\ue5f0': 'UnicodeCharacter0xe5f0',
u'\ua6f2': 'UnicodeCharacter0xa6f2',
u'\u63f4': 'CjkUnifiedIdeograph63f4',
u'\u3cf6': 'CjkUnifiedIdeograph3cf6',
u'\ud879': 'UnicodeCharacter0xd879',
u'\u957b': 'CjkUnifiedIdeograph957b',
u'\u567d': 'CjkUnifiedIdeograph567d',
u'\u137f': 'UnicodeCharacter0x137f',
u'\u6f81': 'CjkUnifiedIdeograph6f81',
u'\u8804': 'CjkUnifiedIdeograph8804',
u'\ua487': 'YiSyllableYyt',
u'\u0208': 'LatinCapitalLetterIWithDoubleGrave',
u'\u3e8b': 'CjkUnifiedIdeograph3e8b',
u'\uc10a': 'HangulSyllableSyaelm',
u'\ufd8d': 'ArabicLigatureMeemWithJeemWithMeemInitialForm',
u'\u5b0e': 'CjkUnifiedIdeograph5b0e',
u'\u7791': 'CjkUnifiedIdeograph7791',
u'\u9014': 'CjkUnifiedIdeograph9014',
u'\ucc97': 'HangulSyllableCyaeh',
u'\u2a18': 'IntegralWithTimesSign',
u'\u469b': 'CjkUnifiedIdeograph469b',
u'\ue91a': 'UnicodeCharacter0xe91a',
u'\u631e': 'CjkUnifiedIdeograph631e',
u'\u9fa1': 'CjkUnifiedIdeograph9fa1',
u'\u19a5': 'NewTaiLueLetterLowBa',
u'\ub824': 'HangulSyllableRyeo',
u'\ud4a7': 'HangulSyllablePweos',
u'\u3228': 'ParenthesizedIdeographNine',
u'\u6eab': 'CjkUnifiedIdeograph6eab',
u'\uf12a': 'UnicodeCharacter0xf12a',
u'\u8b2e': 'CjkUnifiedIdeograph8b2e',
u'\ua7b1': 'UnicodeCharacter0xa7b1',
u'\u0532': 'ArmenianCapitalLetterBen',
u'\u21b5': 'DownwardsArrowWithCornerLeftwards',
u'\uc034': 'HangulSyllableBbwiss',
u'\ufcb7': 'ArabicLigatureDadWithMeemInitialForm',
u'\u5a38': 'CjkUnifiedIdeograph5a38',
u'\u76bb': 'CjkUnifiedIdeograph76bb',
u'\u933e': 'CjkUnifiedIdeograph933e',
u'\ucfc1': 'HangulSyllableKoet',
u'\u2d42': 'TifinaghLetterTuaregYah',
u'\u49c5': 'CjkUnifiedIdeograph49c5',
u'\ue844': 'UnicodeCharacter0xe844',
u'\u6248': 'CjkUnifiedIdeograph6248',
u'\u9ecb': 'CjkUnifiedIdeograph9ecb',
u'\u18cf': 'UnicodeCharacter0x18cf',
u'\ubb4e': 'HangulSyllableMup',
u'\ud7d1': 'UnicodeCharacter0xd7d1',
u'\u3552': 'CjkUnifiedIdeograph3552',
u'\u51d5': 'CjkUnifiedIdeograph51d5',
u'\uf054': 'UnicodeCharacter0xf054',
u'\u8a58': 'CjkUnifiedIdeograph8a58',
u'\ua6db': 'UnicodeCharacter0xa6db',
u'\u045c': 'CyrillicSmallLetterKje',
u'\u20df': 'CombiningEnclosingDiamond',
u'\uc35e': 'HangulSyllableSsyaebs',
u'\uffe1': 'FullwidthPoundSign',
u'\u5d62': 'CjkUnifiedIdeograph5d62',
u'\u79e5': 'CjkUnifiedIdeograph79e5',
u'\u9268': 'CjkUnifiedIdeograph9268',
u'\uceeb': 'HangulSyllableKeod',
u'\u2c6c': 'LatinSmallLetterZWithDescender',
u'\u48ef': 'CjkUnifiedIdeograph48ef',
u'\ueb6e': 'UnicodeCharacter0xeb6e',
u'\u6572': 'CjkUnifiedIdeograph6572',
u'\u81f5': 'CjkUnifiedIdeograph81f5',
u'\u1bf9': 'UnicodeCharacter0x1bf9',
u'\uba78': 'HangulSyllableMyeol',
u'\ud6fb': 'HangulSyllableHweoh',
u'\u347c': 'CjkUnifiedIdeograph347c',
u'\u50ff': 'CjkUnifiedIdeograph50ff',
u'\uf37e': 'UnicodeCharacter0xf37e',
u'\u6a03': 'CjkUnifiedIdeograph6a03',
u'\u8d82': 'CjkUnifiedIdeograph8d82',
u'\u0786': 'ThaanaLetterKaafu',
u'\ua309': 'YiSyllableSit',
u'\uc288': 'HangulSyllableSyu',
u'\u3d0d': 'CjkUnifiedIdeograph3d0d',
u'\u5c8c': 'CjkUnifiedIdeograph5c8c',
u'\uf80f': 'UnicodeCharacter0xf80f',
u'\u7213': 'CjkUnifiedIdeograph7213',
u'\u9592': 'CjkUnifiedIdeograph9592',
u'\u0c17': 'TeluguLetterGa',
u'\u2f96': 'KangxiRadicalBean',
u'\ucb19': 'HangulSyllableJjoelg',
u'\uea98': 'UnicodeCharacter0xea98',
u'\u451d': 'CjkUnifiedIdeograph451d',
u'\u649c': 'CjkUnifiedIdeograph649c',
u'\u9a23': 'CjkUnifiedIdeograph9a23',
u'\ubda2': 'HangulSyllableBweonh',
u'\u1427': 'CanadianSyllabicsFinalMiddleDot',
u'\u37a6': 'CjkUnifiedIdeograph37a6',
u'\ud329': 'HangulSyllablePaeg',
u'\uf2a8': 'UnicodeCharacter0xf2a8',
u'\u6d2d': 'CjkUnifiedIdeograph6d2d',
u'\u8cac': 'CjkUnifiedIdeograph8cac',
u'\u06b0': 'ArabicLetterGafWithRing',
u'\ua233': 'YiSyllableMgax',
u'\uc5b2': 'HangulSyllableYaep',
u'\u3c37': 'CjkUnifiedIdeograph3c37',
u'\u5fb6': 'CjkUnifiedIdeograph5fb6',
u'\ufb39': 'HebrewLetterYodWithDagesh',
u'\u753d': 'CjkUnifiedIdeograph753d',
u'\u94bc': 'CjkUnifiedIdeograph94bc',
u'\u0f41': 'TibetanLetterKha',
u'\u2ec0': 'CjkRadicalGrassThree',
u'\uca43': 'HangulSyllableJjyaes',
u'\uedc2': 'UnicodeCharacter0xedc2',
u'\u4447': 'CjkUnifiedIdeograph4447',
u'\u67c6': 'CjkUnifiedIdeograph67c6',
u'\u9d4d': 'CjkUnifiedIdeograph9d4d',
u'\ubccc': 'HangulSyllableByeom',
u'\u1751': 'BuhidLetterHa',
u'\u36d0': 'CjkUnifiedIdeograph36d0',
u'\ud253': 'HangulSyllableTweolb',
u'\uf5d2': 'UnicodeCharacter0xf5d2',
u'\u6c57': 'CjkUnifiedIdeograph6c57',
u'\u8fd6': 'CjkUnifiedIdeograph8fd6',
u'\u09da': 'UnicodeCharacter0x9da',
u'\ua55d': 'VaiSyllableDhha',
u'\uc4dc': 'HangulSyllableSsyul',
u'\u3f61': 'CjkUnifiedIdeograph3f61',
u'\u5ee0': 'CjkUnifiedIdeograph5ee0',
u'\ufa63': 'CjkCompatibilityIdeographfa63',
u'\u7467': 'CjkUnifiedIdeograph7467',
u'\u97e6': 'CjkUnifiedIdeograph97e6',
u'\u0e6b': 'UnicodeCharacter0xe6b',
u'\u11ea': 'HangulJongseongSiospieup',
u'\ucd6d': 'HangulSyllableCoeb',
u'\uecec': 'UnicodeCharacter0xecec',
u'\u4771': 'CjkUnifiedIdeograph4771',
u'\u66f0': 'CjkUnifiedIdeograph66f0',
u'\u9c77': 'CjkUnifiedIdeograph9c77',
u'\ubff6': 'HangulSyllableBbweolp',
u'\u167b': 'UnicodeCharacter0x167b',
u'\u39fa': 'CjkUnifiedIdeograph39fa',
u'\ud57d': 'HangulSyllableHaelg',
u'\uf4fc': 'UnicodeCharacter0xf4fc',
u'\uee81': 'UnicodeCharacter0xee81',
u'\u8f00': 'CjkUnifiedIdeograph8f00',
u'\uab83': 'UnicodeCharacter0xab83',
u'\u4c02': 'CjkUnifiedIdeograph4c02',
u'\u6885': 'CjkUnifiedIdeograph6885',
u'\u0904': 'DevanagariLetterShortA',
u'\u2587': 'LowerSevenEighthsBlock',
u'\uda0e': 'UnicodeCharacter0xda0e',
u'\uf691': 'UnicodeCharacter0xf691',
u'\u9710': 'CjkUnifiedIdeograph9710',
u'\ub393': 'HangulSyllableDyeod',
u'\u5412': 'CjkUnifiedIdeograph5412',
u'\u7095': 'CjkUnifiedIdeograph7095',
u'\u1114': 'HangulChoseongSsangnieun',
u'\u4d97': 'CjkUnifiedIdeograph4d97',
u'\u0a99': 'GujaratiLetterNga',
u'\ue21e': 'UnicodeCharacter0xe21e',
u'\ubf20': 'HangulSyllableBbyeok',
u'\udba3': 'UnicodeCharacter0xdba3',
u'\u7c22': 'CjkUnifiedIdeograph7c22',
u'\u98a5': 'CjkUnifiedIdeograph98a5',
u'\u3924': 'CjkUnifiedIdeograph3924',
u'\u55a7': 'CjkUnifiedIdeograph55a7',
u'\u12a9': 'EthiopicSyllableKu',
u'\uc730': 'HangulSyllableYum',
u'\ue3b3': 'UnicodeCharacter0xe3b3',
u'\u8432': 'CjkUnifiedIdeograph8432',
u'\ua0b5': 'YiSyllableMat',
u'\u4134': 'CjkUnifiedIdeograph4134',
u'\u7db7': 'CjkUnifiedIdeograph7db7',
u'\u1e36': 'LatinCapitalLetterLWithDotBelow',
u'\u3ab9': 'CjkUnifiedIdeograph3ab9',
u'\uef40': 'UnicodeCharacter0xef40',
u'\uac42': 'HangulSyllableGyalm',
u'\uc8c5': 'HangulSyllableJoeg',
u'\u6944': 'CjkUnifiedIdeograph6944',
u'\u85c7': 'CjkUnifiedIdeograph85c7',
u'\u2646': 'Neptune',
u'\u42c9': 'CjkUnifiedIdeograph42c9',
u'\u1fcb': 'GreekCapitalLetterEtaWithOxia',
u'\uf750': 'UnicodeCharacter0xf750',
u'\ub452': 'HangulSyllableDugg',
u'\ud0d5': 'HangulSyllableTang',
u'\u7154': 'CjkUnifiedIdeograph7154',
u'\uadd7': 'HangulSyllableGwic',
u'\u4e56': 'CjkUnifiedIdeograph4e56',
u'\u6ad9': 'CjkUnifiedIdeograph6ad9',
u'\u0b58': 'UnicodeCharacter0xb58',
u'\u27db': 'LeftAndRightTack',
u'\udc62': 'UnicodeCharacter0xdc62',
u'\uf8e5': 'UnicodeCharacter0xf8e5',
u'\u9964': 'CjkUnifiedIdeograph9964',
u'\ub5e7': 'HangulSyllableDdyeolh',
u'\u5666': 'CjkUnifiedIdeograph5666',
u'\u72e9': 'CjkUnifiedIdeograph72e9',
u'\u1368': 'EthiopicParagraphSeparator',
u'\u4feb': 'CjkUnifiedIdeograph4feb',
u'\u0ced': 'KannadaDigitSeven',
u'\ue472': 'UnicodeCharacter0xe472',
u'\ua174': 'YiSyllableHnuox',
u'\uddf7': 'UnicodeCharacter0xddf7',
u'\u7e76': 'CjkUnifiedIdeograph7e76',
u'\u9af9': 'CjkUnifiedIdeograph9af9',
u'\u3b78': 'CjkUnifiedIdeograph3b78',
u'\u57fb': 'CjkUnifiedIdeograph57fb',
u'\u14fd': 'CanadianSyllabicsWestcreeSwo',
u'\u2e01': 'RightAngleDottedSubstitutionMarker',
u'\uc984': 'HangulSyllableJyuk',
u'\u8686': 'CjkUnifiedIdeograph8686',
u'\u4388': 'CjkUnifiedIdeograph4388',
u'\uff0b': 'FullwidthPlusSign',
u'\x8a': 'UnicodeCharacter0x8a',
u'\ubc0d': 'HangulSyllableMing',
u'\u790f': 'CjkUnifiedIdeograph790f',
u'\u3611': 'CjkUnifiedIdeograph3611',
u'\ud194': 'HangulSyllableTyem',
u'\uae96': 'HangulSyllableGgyabs',
u'\u6b98': 'CjkUnifiedIdeograph6b98',
u'\u289a': 'BraillePatternDots2458',
u'\uc41d': 'HangulSyllableSswaelt',
u'\u811f': 'CjkUnifiedIdeograph811f',
u'\u5e21': 'CjkUnifiedIdeograph5e21',
u'\u1b23': 'BalineseLetterTaTawa',
u'\uf9a4': 'CjkCompatibilityIdeographf9a4',
u'\ub6a6': 'HangulSyllableDdulm',
u'\u73a8': 'CjkUnifiedIdeograph73a8',
u'\u30aa': 'KatakanaLetterO',
u'\uec2d': 'UnicodeCharacter0xec2d',
u'\u0dac': 'SinhalaLetterSanyakaDdayanna',
u'\ua92f': 'KayahLiSignShya',
u'\u6631': 'CjkUnifiedIdeograph6631',
u'\u2333': 'Slope',
u'\udeb6': 'UnicodeCharacter0xdeb6',
u'\u9bb8': 'CjkUnifiedIdeograph9bb8',
u'\u58ba': 'CjkUnifiedIdeograph58ba',
u'\uf43d': 'UnicodeCharacter0xf43d',
u'\u15bc': 'CanadianSyllabicsBlackfootKe',
u'\ub13f': 'HangulSyllableNeh',
u'\u8e41': 'CjkUnifiedIdeograph8e41',
u'\u4b43': 'CjkUnifiedIdeograph4b43',
u'\u0845': 'UnicodeCharacter0x845',
u'\ue6c6': 'UnicodeCharacter0xe6c6',
u'\ua3c8': 'YiSyllableRa',
u'\u60ca': 'CjkUnifiedIdeograph60ca',
u'\u3dcc': 'CjkUnifiedIdeograph3dcc',
u'\ud94f': 'UnicodeCharacter0xd94f',
u'\u9651': 'CjkUnifiedIdeograph9651',
u'\u5353': 'CjkUnifiedIdeograph5353',
u'\u1055': 'MyanmarLetterVocalicLl',
u'\ucbd8': 'HangulSyllableJjeun',
u'\u88da': 'CjkUnifiedIdeograph88da',
u'\u45dc': 'CjkUnifiedIdeograph45dc',
u'\ue15f': 'UnicodeCharacter0xe15f',
u'\u02de': 'ModifierLetterRhoticHook',
u'\ube61': 'HangulSyllableBbag',
u'\u7b63': 'CjkUnifiedIdeograph7b63',
u'\u3865': 'CjkUnifiedIdeograph3865',
u'\ud3e8': 'HangulSyllablePyek',
u'\u90ea': 'CjkUnifiedIdeograph90ea',
u'\u6dec': 'CjkUnifiedIdeograph6dec',
u'\u2aee': 'DoesNotDivideWithReversedNegationSlash',
u'\uc671': 'HangulSyllableWaeng',
u'\u8373': 'CjkUnifiedIdeograph8373',
u'\u4075': 'CjkUnifiedIdeograph4075',
u'\u1d77': 'LatinSmallLetterTurnedG',
u'\ufbf8': 'ArabicLigatureYehWithHamzaAboveWithEInitialForm',
u'\ub8fa': 'HangulSyllableRubs',
u'\u75fc': 'CjkUnifiedIdeograph75fc',
u'\u32fe': 'CircledKatakanaWo',
u'\u1000': 'MyanmarLetterKa',
u'\u3483': 'CjkUnifiedIdeograph3483',
u'\ud302': 'HangulSyllableTibs',
u'\uf785': 'UnicodeCharacter0xf785',
u'\u5906': 'CjkUnifiedIdeograph5906',
u'\u7d89': 'CjkUnifiedIdeograph7d89',
u'\u820c': 'CjkUnifiedIdeograph820c',
u'\ua68f': 'CyrillicSmallLetterTswe',
u'\u0810': 'UnicodeCharacter0x810',
u'\u2c93': 'CopticSmallLetterIauda',
u'\ucb12': 'HangulSyllableJjoegg',
u'\uef95': 'UnicodeCharacter0xef95',
u'\u7116': 'CjkUnifiedIdeograph7116',
u'\u9599': 'CjkUnifiedIdeograph9599',
u'\u1b9d': 'SundaneseLetterWa',
u'\uba1c': 'HangulSyllableMyae',
u'\ude9f': 'UnicodeCharacter0xde9f',
u'\u2020': 'Dagger',
u'\u44a3': 'CjkUnifiedIdeograph44a3',
u'\ue322': 'UnicodeCharacter0xe322',
u'\u6926': 'CjkUnifiedIdeograph6926',
u'\u8da9': 'CjkUnifiedIdeograph8da9',
u'\u33ad': 'SquareRad',
u'\ud22c': 'HangulSyllableTu',
u'\uf6af': 'UnicodeCharacter0xf6af',
u'\u5830': 'CjkUnifiedIdeograph5830',
u'\u7cb3': 'CjkUnifiedIdeograph7cb3',
u'\u8136': 'CjkUnifiedIdeograph8136',
u'\ua5b9': 'VaiSyllableNyu',
u'\u073a': 'SyriacHbasaAbove',
u'\u2bbd': 'UnicodeCharacter0x2bbd',
u'\uca3c': 'HangulSyllableJjyaels',
u'\ueebf': 'UnicodeCharacter0xeebf',
u'\u7040': 'CjkUnifiedIdeograph7040',
u'\u94c3': 'CjkUnifiedIdeograph94c3',
u'\u1ac7': 'UnicodeCharacter0x1ac7',
u'\ub946': 'HangulSyllableRwilm',
u'\uddc9': 'UnicodeCharacter0xddc9',
u'\u3f4a': 'CjkUnifiedIdeograph3f4a',
u'\u43cd': 'CjkUnifiedIdeograph43cd',
u'\ue24c': 'UnicodeCharacter0xe24c',
u'\u6850': 'CjkUnifiedIdeograph6850',
u'\u8cd3': 'CjkUnifiedIdeograph8cd3',
u'\u32d7': 'CircledKatakanaKu',
u'\ud156': 'HangulSyllableTelm',
u'\uf5d9': 'UnicodeCharacter0xf5d9',
u'\u575a': 'CjkUnifiedIdeograph575a',
u'\u7bdd': 'CjkUnifiedIdeograph7bdd',
u'\u8060': 'CjkUnifiedIdeograph8060',
u'\ua4e3': 'UnicodeCharacter0xa4e3',
u'\u0664': 'ArabicindicDigitFour',
u'\u2ae7': 'ShortDownTackWithOverbar',
u'\uc966': 'HangulSyllableJwij',
u'\uede9': 'UnicodeCharacter0xede9',
u'\u4f6a': 'CjkUnifiedIdeograph4f6a',
u'\u93ed': 'CjkUnifiedIdeograph93ed',
u'\u19f1': 'KhmerSymbolMuoyRoc',
u'\ub870': 'HangulSyllableRoss',
u'\udcf3': 'UnicodeCharacter0xdcf3',
u'\u3e74': 'CjkUnifiedIdeograph3e74',
u'\u42f7': 'CjkUnifiedIdeograph42f7',
u'\ue176': 'UnicodeCharacter0xe176',
u'\u677a': 'CjkUnifiedIdeograph677a',
u'\u8bfd': 'CjkUnifiedIdeograph8bfd',
u'\ub101': 'HangulSyllableNyaeng',
u'\ud080': 'HangulSyllableKeuss',
u'\u3705': 'CjkUnifiedIdeograph3705',
u'\u5684': 'CjkUnifiedIdeograph5684',
u'\ufa07': 'CjkCompatibilityIdeographfa07',
u'\u600b': 'CjkUnifiedIdeograph600b',
u'\u9f8a': 'CjkUnifiedIdeograph9f8a',
u'\u058e': 'UnicodeCharacter0x58e',
u'\ua911': 'KayahLiLetterNya',
u'\uc890': 'HangulSyllableJwan',
u'\u2f15': 'KangxiRadicalRightOpenBox',
u'\u4e94': 'CjkUnifiedIdeograph4e94',
u'\u981b': 'CjkUnifiedIdeograph981b',
u'\ub79a': 'HangulSyllableRaegg',
u'\u1e1f': 'LatinSmallLetterFWithDotAbove',
u'\u3d9e': 'CjkUnifiedIdeograph3d9e',
u'\uc121': 'HangulSyllableSeonj',
u'\ue0a0': 'UnicodeCharacter0xe0a0',
u'\u4725': 'CjkUnifiedIdeograph4725',
u'\u66a4': 'CjkUnifiedIdeograph66a4',
u'\ub02b': 'HangulSyllableGgyugs',
u'\uafaa': 'HangulSyllableGgyolp',
u'\u362f': 'CjkUnifiedIdeograph362f',
u'\u55ae': 'CjkUnifiedIdeograph55ae',
u'\uf931': 'CjkCompatibilityIdeographf931',
u'\u7f35': 'CjkUnifiedIdeograph7f35',
u'\u9eb4': 'CjkUnifiedIdeograph9eb4',
u'\u04b8': 'CyrillicCapitalLetterCheWithVerticalStroke',
u'\ua83b': 'UnicodeCharacter0xa83b',
u'\uc7ba': 'HangulSyllableJaelp',
u'\u2e3f': 'UnicodeCharacter0x2e3f',
u'\u4dbe': 'UnicodeCharacter0x4dbe',
u'\u9745': 'CjkUnifiedIdeograph9745',
u'\ub6c4': 'HangulSyllableDdweols',
u'\u1d49': 'ModifierLetterSmallE',
u'\u3cc8': 'CjkUnifiedIdeograph3cc8',
u'\uc04b': 'HangulSyllableBbyulh',
u'\uffca': 'HalfwidthHangulLetterYeo',
u'\u464f': 'CjkUnifiedIdeograph464f',
u'\u65ce': 'CjkUnifiedIdeograph65ce',
u'\u8f55': 'CjkUnifiedIdeograph8f55',
u'\uaed4': 'HangulSyllableGgeok',
u'\u3559': 'CjkUnifiedIdeograph3559',
u'\u54d8': 'CjkUnifiedIdeograph54d8',
u'\uf85b': 'UnicodeCharacter0xf85b',
u'\u7e5f': 'CjkUnifiedIdeograph7e5f',
u'\u9dde': 'CjkUnifiedIdeograph9dde',
u'\u03e2': 'CopticCapitalLetterShei',
u'\ua765': 'LatinSmallLetterThornWithStroke',
u'\uc6e4': 'HangulSyllableWeok',
u'\u2d69': 'UnicodeCharacter0x2d69',
u'\u4ce8': 'CjkUnifiedIdeograph4ce8',
u'\u966f': 'CjkUnifiedIdeograph966f',
u'\ub5ee': 'HangulSyllableDdyeoj',
u'\u1c73': 'OlChikiLetterLo',
u'\u3bf2': 'CjkUnifiedIdeograph3bf2',
u'\udf75': 'UnicodeCharacter0xdf75',
u'\ufef4': 'ArabicLetterYehMedialForm',
u'\u4579': 'CjkUnifiedIdeograph4579',
u'\u64f8': 'CjkUnifiedIdeograph64f8',
u'\u8e7f': 'CjkUnifiedIdeograph8e7f',
u'\uadfe': 'HangulSyllableGeunh',
u'\ud806': 'UnicodeCharacter0xd806',
u'\ufc89': 'ArabicLigatureMeemWithMeemFinalForm',
u'\u9d08': 'CjkUnifiedIdeograph9d08',
u'\ua18b': 'YiSyllableNox',
u'\u5e0a': 'CjkUnifiedIdeograph5e0a',
u'\u628d': 'CjkUnifiedIdeograph628d',
u'\u030c': 'CombiningCaron',
u'\u278f': 'DingbatNegativeCircledSansserifDigitSix',
u'\uf016': 'UnicodeCharacter0xf016',
u'\ub518': 'HangulSyllableDin',
u'\ud99b': 'UnicodeCharacter0xd99b',
u'\u761a': 'CjkUnifiedIdeograph761a',
u'\u9a9d': 'CjkUnifiedIdeograph9a9d',
u'\u3b1c': 'CjkUnifiedIdeograph3b1c',
u'\u5f9f': 'CjkUnifiedIdeograph5f9f',
u'\xa1': 'InvertedExclamationMark',
u'\ue826': 'UnicodeCharacter0xe826',
u'\uad28': 'HangulSyllableGwaem',
u'\uf1ab': 'UnicodeCharacter0xf1ab',
u'\u6e2a': 'CjkUnifiedIdeograph6e2a',
u'\ub2ad': 'HangulSyllableNyig',
u'\u532c': 'CjkUnifiedIdeograph532c',
u'\u77af': 'CjkUnifiedIdeograph77af',
u'\u142e': 'CanadianSyllabicsAn',
u'\u38b1': 'CjkUnifiedIdeograph38b1',
u'\uc538': 'HangulSyllableSsim',
u'\ue9bb': 'UnicodeCharacter0xe9bb',
u'\u863a': 'CjkUnifiedIdeograph863a',
u'\uaabd': 'UnicodeCharacter0xaabd',
u'\u4b3c': 'CjkUnifiedIdeograph4b3c',
u'\u6fbf': 'CjkUnifiedIdeograph6fbf',
u'\u0c3e': 'TeluguVowelSignAa',
u'\u50c1': 'CjkUnifiedIdeograph50c1',
u'\u15c3': 'CanadianSyllabicsSayisiHa',
u'\ufd48': 'UnicodeCharacter0xfd48',
u'\ube4a': 'HangulSyllableBinh',
u'\uc2cd': 'HangulSyllableSyilt',
u'\u634c': 'CjkUnifiedIdeograph634c',
u'\u87cf': 'CjkUnifiedIdeograph87cf',
u'\u244e': 'UnicodeCharacter0x244e',
u'\u48d1': 'CjkUnifiedIdeograph48d1',
u'\u0dd3': 'SinhalaVowelSignDigaIspilla',
u'\ud65a': 'HangulSyllableHwanh',
u'\ufadd': 'UnicodeCharacter0xfadd',
u'\u9b5c': 'CjkUnifiedIdeograph9b5c',
u'\ubfdf': 'HangulSyllableBbus',
u'\u5c5e': 'CjkUnifiedIdeograph5c5e',
u'\u60e1': 'CjkUnifiedIdeograph60e1',
u'\u0160': 'LatinCapitalLetterSWithCaron',
u'\u25e3': 'BlackLowerLeftTriangle',
u'\uce6a': 'HangulSyllableCibs',
u'\ub36c': 'HangulSyllableDeok',
u'\ud7ef': 'UnicodeCharacter0xd7ef',
u'\u746e': 'CjkUnifiedIdeograph746e',
u'\u98f1': 'CjkUnifiedIdeograph98f1',
u'\u3970': 'CjkUnifiedIdeograph3970',
u'\u5df3': 'CjkUnifiedIdeograph5df3',
u'\u1ef5': 'LatinSmallLetterYWithDotBelow',
u'\ue67a': 'UnicodeCharacter0xe67a',
u'\uab7c': 'UnicodeCharacter0xab7c',
u'\ucfff': 'HangulSyllableKweogs',
u'\u6c7e': 'CjkUnifiedIdeograph6c7e',
u'\u5180': 'CjkUnifiedIdeograph5180',
u'\uf503': 'UnicodeCharacter0xf503',
u'\u1282': 'EthiopicSyllableXi',
u'\ub605': 'HangulSyllableDdyeb',
u'\u7b07': 'CjkUnifiedIdeograph7b07',
u'\u3c09': 'CjkUnifiedIdeograph3c09',
u'\uc38c': 'HangulSyllableSsel',
u'\u848e': 'CjkUnifiedIdeograph848e',
u'\u4990': 'CjkUnifiedIdeograph4990',
u'\ued13': 'UnicodeCharacter0xed13',
u'\u0a92': 'UnicodeCharacter0xa92',
u'\uae15': 'HangulSyllableGyig',
u'\u9317': 'CjkUnifiedIdeograph9317',
u'\u5419': 'CjkUnifiedIdeograph5419',
u'\u191b': 'LimbuLetterSa',
u'\ufb9c': 'ArabicLetterNgoehInitialForm',
u'\ubc9e': 'HangulSyllableBeop',
u'\u61a0': 'CjkUnifiedIdeograph61a0',
u'\u22a2': 'RightTack',
u'\uc625': 'HangulSyllableOg',
u'\u8b27': 'CjkUnifiedIdeograph8b27',
u'\u4c29': 'CjkUnifiedIdeograph4c29',
u'\u312b': 'BopomofoLetterNg',
u'\ud4ae': 'HangulSyllablePweop',
u'\u99b0': 'CjkUnifiedIdeograph99b0',
u'\u5ab2': 'CjkUnifiedIdeograph5ab2',
u'\ufe35': 'PresentationFormForVerticalLeftParenthesis',
u'\u1fb4': 'GreekSmallLetterAlphaWithOxiaAndYpogegrammeni',
u'\ua337': 'YiSyllableSso',
u'\u6439': 'CjkUnifiedIdeograph6439',
u'\u293b': 'BottomArcAnticlockwiseArrow',
u'\uccbe': 'HangulSyllableCelm',
u'\ub1c0': 'HangulSyllableNwaem',
u'\u72c2': 'CjkUnifiedIdeograph72c2',
u'\u37c4': 'CjkUnifiedIdeograph37c4',
u'\udb47': 'UnicodeCharacter0xdb47',
u'\u9c49': 'CjkUnifiedIdeograph9c49',
u'\u414b': 'CjkUnifiedIdeograph414b',
u'\u024d': 'LatinSmallLetterRWithStroke',
u'\ue4ce': 'UnicodeCharacter0xe4ce',
u'\ua9d0': 'UnicodeCharacter0xa9d0',
u'\u6ad2': 'CjkUnifiedIdeograph6ad2',
u'\u2fd4': 'KangxiRadicalTurtle',
u'\uf357': 'UnicodeCharacter0xf357',
u'\u10d6': 'GeorgianLetterZen',
u'\ub459': 'HangulSyllableDulg',
u'\u795b': 'CjkUnifiedIdeograph795b',
u'\u3a5d': 'CjkUnifiedIdeograph3a5d',
u'\uc1e0': 'HangulSyllableSoe',
u'\u82e2': 'CjkUnifiedIdeograph82e2',
u'\u47e4': 'CjkUnifiedIdeograph47e4',
u'\ueb67': 'UnicodeCharacter0xeb67',
u'\u08e6': 'UnicodeCharacter0x8e6',
u'\uac69': 'HangulSyllableGyaeng',
u'\u916b': 'CjkUnifiedIdeograph916b',
u'\u526d': 'CjkUnifiedIdeograph526d',
u'\u176f': 'TagbanwaLetterWa',
u'\uf9f0': 'CjkCompatibilityIdeographf9f0',
u'\ubaf2': 'HangulSyllableMwaebs',
u'\u7ff4': 'CjkUnifiedIdeograph7ff4',
u'\u20f6': 'UnicodeCharacter0x20f6',
u'\uc479': 'HangulSyllableSsung',
u'\u897b': 'CjkUnifiedIdeograph897b',
u'\u4a7d': 'CjkUnifiedIdeograph4a7d',
u'\u0f7f': 'TibetanSignRnamBcad',
u'\u7381': 'CjkUnifiedIdeograph7381',
u'\u9404': 'CjkUnifiedIdeograph9404',
u'\ub887': 'HangulSyllableRwalh',
u'\u1e08': 'LatinCapitalLetterCWithCedillaAndAcute',
u'\u228b': 'SupersetOfWithNotEqualTo',
u'\udd0a': 'UnicodeCharacter0xdd0a',
u'\ue18d': 'UnicodeCharacter0xe18d',
u'\u470e': 'CjkUnifiedIdeograph470e',
u'\u6b91': 'CjkUnifiedIdeograph6b91',
u'\u8c14': 'CjkUnifiedIdeograph8c14',
u'\ud097': 'HangulSyllableKyilh',
u'\u3618': 'CjkUnifiedIdeograph3618',
u'\u5a9b': 'CjkUnifiedIdeograph5a9b',
u'\uf51a': 'UnicodeCharacter0xf51a',
u'\u7f1e': 'CjkUnifiedIdeograph7f1e',
u'\u83a1': 'CjkUnifiedIdeograph83a1',
u'\u05a5': 'HebrewAccentMerkha',
u'\ua424': 'YiSyllableJjox',
u'\uc8a7': 'HangulSyllableJwah',
u'\u2e28': 'LeftDoubleParenthesis',
u'\u72ab': 'CjkUnifiedIdeograph72ab',
u'\ued2a': 'UnicodeCharacter0xed2a',
u'\u972e': 'CjkUnifiedIdeograph972e',
u'\ubbb1': 'HangulSyllableMyult',
u'\u1932': 'LimbuSmallLetterAnusvara',
u'\u3db5': 'CjkUnifiedIdeograph3db5',
u'\udc34': 'UnicodeCharacter0xdc34',
u'\ue0b7': 'UnicodeCharacter0xe0b7',
u'\u4638': 'CjkUnifiedIdeograph4638',
u'\u6abb': 'CjkUnifiedIdeograph6abb',
u'\u8f3e': 'CjkUnifiedIdeograph8f3e',
u'\ud3c1': 'HangulSyllablePyeolt',
u'\u3142': 'HangulLetterPieup',
u'\u55c5': 'CjkUnifiedIdeograph55c5',
u'\uf444': 'UnicodeCharacter0xf444',
u'\u7e48': 'CjkUnifiedIdeograph7e48',
u'\u82cb': 'CjkUnifiedIdeograph82cb',
u'\u04cf': 'CyrillicSmallLetterPalochka',
u'\ua74e': 'LatinCapitalLetterOo',
u'\ucbd1': 'HangulSyllableJjyut',
u'\u2952': 'LeftwardsHarpoonWithBarbUpToBar',
u'\u4dd5': 'HexagramForGrace',
u'\uec54': 'UnicodeCharacter0xec54',
u'\u9658': 'CjkUnifiedIdeograph9658',
u'\ubadb': 'HangulSyllableMwac',
u'\u185c': 'MongolianLetterTodoDza',
u'\u3cdf': 'CjkUnifiedIdeograph3cdf',
u'\udf5e': 'UnicodeCharacter0xdf5e',
u'\ue3e1': 'UnicodeCharacter0xe3e1',
u'\u4162': 'CjkUnifiedIdeograph4162',
u'\u65e5': 'CjkUnifiedIdeograph65e5',
u'\u8e68': 'CjkUnifiedIdeograph8e68',
u'\ud2eb': 'HangulSyllableTyic',
u'\u306c': 'HiraganaLetterNu',
u'\u54ef': 'CjkUnifiedIdeograph54ef',
u'\uf76e': 'UnicodeCharacter0xf76e',
u'\u7972': 'CjkUnifiedIdeograph7972',
u'\u9df5': 'CjkUnifiedIdeograph9df5',
u'\u07f9': 'NkoExclamationMark',
u'\ua678': 'UnicodeCharacter0xa678',
u'\ucafb': 'HangulSyllableJjwaed',
u'\u287c': 'BraillePatternDots34567',
u'\u4cff': 'CjkUnifiedIdeograph4cff',
u'\uef7e': 'UnicodeCharacter0xef7e',
u'\u7603': 'CjkUnifiedIdeograph7603',
u'\u9182': 'CjkUnifiedIdeograph9182',
u'\u1b86': 'SundaneseLetterAe',
u'\ubf09': 'HangulSyllableBbyeog',
u'\ude88': 'UnicodeCharacter0xde88',
u'\u210d': 'DoublestruckCapitalH',
u'\u408c': 'CjkUnifiedIdeograph408c',
u'\ue40f': 'UnicodeCharacter0xe40f',
u'\u6e13': 'CjkUnifiedIdeograph6e13',
u'\u8992': 'CjkUnifiedIdeograph8992',
u'\u1017': 'MyanmarLetterBa',
u'\u3396': 'SquareMl',
u'\ud719': 'HangulSyllableHwig',
u'\uf698': 'UnicodeCharacter0xf698',
u'\u591d': 'CjkUnifiedIdeograph591d',
u'\u789c': 'CjkUnifiedIdeograph789c',
u'\u8623': 'CjkUnifiedIdeograph8623',
u'\ua1a2': 'YiSyllableHluox',
u'\u0827': 'UnicodeCharacter0x827',
u'\u2ba6': 'UnicodeCharacter0x2ba6',
u'\ucf29': 'HangulSyllableKyeolt',
u'\ueea8': 'UnicodeCharacter0xeea8',
u'\u712d': 'CjkUnifiedIdeograph712d',
u'\u90ac': 'CjkUnifiedIdeograph90ac',
u'\u1ab0': 'UnicodeCharacter0x1ab0',
u'\ube33': 'HangulSyllableByilb',
u'\ud9b2': 'UnicodeCharacter0xd9b2',
u'\u2037': 'ReversedTriplePrime',
u'\u43b6': 'CjkUnifiedIdeograph43b6',
u'\ue739': 'UnicodeCharacter0xe739',
u'\u693d': 'CjkUnifiedIdeograph693d',
u'\u88bc': 'CjkUnifiedIdeograph88bc',
u'\u1341': 'EthiopicSyllableTzu',
u'\u32c0': 'IdeographicTelegraphSymbolForJanuary',
u'\ud643': 'HangulSyllableHolb',
u'\uf1c2': 'UnicodeCharacter0xf1c2',
u'\u5847': 'CjkUnifiedIdeograph5847',
u'\u7bc6': 'CjkUnifiedIdeograph7bc6',
u'\u814d': 'CjkUnifiedIdeograph814d',
u'\ua0cc': 'YiSyllableMyp',
u'\u0b51': 'UnicodeCharacter0xb51',
u'\u2ad0': 'ClosedSuperset',
u'\uce53': 'HangulSyllableCyic',
u'\ue9d2': 'UnicodeCharacter0xe9d2',
u'\u7057': 'CjkUnifiedIdeograph7057',
u'\u93d6': 'CjkUnifiedIdeograph93d6',
u'\u15da': 'CanadianSyllabicsCarrierHwi',
u'\ub95d': 'HangulSyllableRyunj',
u'\ud8dc': 'UnicodeCharacter0xd8dc',
u'\u2361': 'AplFunctionalSymbolUpTackDiaeresis',
u'\u42e0': 'CjkUnifiedIdeograph42e0',
u'\ue663': 'UnicodeCharacter0xe663',
u'\u6867': 'CjkUnifiedIdeograph6867',
u'\u8be6': 'CjkUnifiedIdeograph8be6',
u'\u126b': 'EthiopicSyllableVaa',
u'\u0dea': 'UnicodeCharacter0xdea',
u'\ud16d': 'HangulSyllableTyeonj',
u'\uf0ec': 'UnicodeCharacter0xf0ec',
u'\u5b71': 'CjkUnifiedIdeograph5b71',
u'\u7af0': 'CjkUnifiedIdeograph7af0',
u'\u8077': 'CjkUnifiedIdeograph8077',
u'\ua3f6': 'YiSyllableJyt',
u'\u0a7b': 'UnicodeCharacter0xa7b',
u'\u25fa': 'LowerLeftTriangle',
u'\uc97d': 'HangulSyllableJyub',
u'\ue8fc': 'UnicodeCharacter0xe8fc',
u'\uf281': 'UnicodeCharacter0xf281',
u'\u9300': 'CjkUnifiedIdeograph9300',
u'\ub783': 'HangulSyllableRad',
u'\u5002': 'CjkUnifiedIdeograph5002',
u'\u7485': 'CjkUnifiedIdeograph7485',
u'\u1504': 'CanadianSyllabicsNaskapiSwaa',
u'\u3987': 'CjkUnifiedIdeograph3987',
u'\uc60e': 'HangulSyllableYenh',
u'\uea91': 'UnicodeCharacter0xea91',
u'\u8b10': 'CjkUnifiedIdeograph8b10',
u'\uaf93': 'HangulSyllableGgoes',
u'\u4812': 'CjkUnifiedIdeograph4812',
u'\u6c95': 'CjkUnifiedIdeograph6c95',
u'\u0d14': 'MalayalamLetterAu',
u'\u5197': 'CjkUnifiedIdeograph5197',
u'\u1699': 'OghamLetterEamhancholl',
u'\ufe1e': 'UnicodeCharacter0xfe1e',
u'\ua320': 'YiSyllableSu',
u'\uc7a3': 'HangulSyllableJas',
u'\u6022': 'CjkUnifiedIdeograph6022',
u'\u84a5': 'CjkUnifiedIdeograph84a5',
u'\u2524': 'BoxDrawingsLightVerticalAndLeft',
u'\u49a7': 'CjkUnifiedIdeograph49a7',
u'\u0ea9': 'UnicodeCharacter0xea9',
u'\udb30': 'UnicodeCharacter0xdb30',
u'\uffb3': 'HalfwidthHangulLetterSsangpieup',
u'\u9832': 'CjkUnifiedIdeograph9832',
u'\ubcb5': 'HangulSyllableBeng',
u'\u5d34': 'CjkUnifiedIdeograph5d34',
u'\u61b7': 'CjkUnifiedIdeograph61b7',
u'\u0236': 'LatinSmallLetterTWithCurl',
u'\u26b9': 'Sextile',
u'\uf340': 'UnicodeCharacter0xf340',
u'\ub042': 'HangulSyllableGgyup',
u'\ud4c5': 'HangulSyllablePweng',
u'\u7544': 'CjkUnifiedIdeograph7544',
u'\u99c7': 'CjkUnifiedIdeograph99c7',
u'\u3a46': 'CjkUnifiedIdeograph3a46',
u'\u5ec9': 'CjkUnifiedIdeograph5ec9',
u'\u03cb': 'GreekSmallLetterUpsilonWithDialytika',
u'\ueb50': 'UnicodeCharacter0xeb50',
u'\ua852': 'PhagspaLetterDza',
u'\uccd5': 'HangulSyllableCyeonj',
u'\u6d54': 'CjkUnifiedIdeograph6d54',
u'\ub1d7': 'HangulSyllableNoelb',
u'\u5256': 'CjkUnifiedIdeograph5256',
u'\u76d9': 'CjkUnifiedIdeograph76d9',
u'\u1758': 'UnicodeCharacter0x1758',
u'\u3bdb': 'CjkUnifiedIdeograph3bdb',
u'\uc062': 'HangulSyllableBbeulm',
u'\ue4e5': 'UnicodeCharacter0xe4e5',
u'\u8564': 'CjkUnifiedIdeograph8564',
u'\ua9e7': 'UnicodeCharacter0xa9e7',
u'\u4a66': 'CjkUnifiedIdeograph4a66',
u'\u6ee9': 'CjkUnifiedIdeograph6ee9',
u'\u0f68': 'TibetanLetterA',
u'\u53eb': 'CjkUnifiedIdeograph53eb',
u'\u10ed': 'GeorgianLetterChar',
u'\uf872': 'UnicodeCharacter0xf872',
u'\ubd74': 'HangulSyllableByom',
u'\uc1f7': 'HangulSyllableSoec',
u'\u6276': 'CjkUnifiedIdeograph6276',
u'\u86f9': 'CjkUnifiedIdeograph86f9',
u'\u2778': 'DingbatNegativeCircledDigitThree',
u'\u4bfb': 'CjkUnifiedIdeograph4bfb',
u'\u08fd': 'UnicodeCharacter0x8fd',
u'\u3201': 'ParenthesizedHangulNieun',
u'\ud584': 'HangulSyllableHaem',
u'\u9a86': 'CjkUnifiedIdeograph9a86',
u'\u5f88': 'CjkUnifiedIdeograph5f88',
u'\ue30b': 'UnicodeCharacter0xe30b',
u'\u1c8a': 'UnicodeCharacter0x1c8a',
u'\ua00d': 'YiSyllableUo',
u'\u650f': 'CjkUnifiedIdeograph650f',
u'\u2a11': 'AnticlockwiseIntegration',
u'\ucd94': 'HangulSyllableCu',
u'\ub296': 'HangulSyllableNeunh',
u'\u7798': 'CjkUnifiedIdeograph7798',
u'\u349a': 'CjkUnifiedIdeograph349a',
u'\ud81d': 'UnicodeCharacter0xd81d',
u'\u9d1f': 'CjkUnifiedIdeograph9d1f',
u'\u4221': 'CjkUnifiedIdeograph4221',
u'\u0723': 'SyriacLetterSemkath',
u'\ue5a4': 'UnicodeCharacter0xe5a4',
u'\uaaa6': 'UnicodeCharacter0xaaa6',
u'\u6fa8': 'CjkUnifiedIdeograph6fa8',
u'\u2caa': 'CopticCapitalLetterFi',
u'\uf02d': 'UnicodeCharacter0xf02d',
u'\u11ac': 'HangulJongseongNieuncieuc',
u'\ub52f': 'HangulSyllableDih',
u'\u7a31': 'CjkUnifiedIdeograph7a31',
u'\u3f33': 'CjkUnifiedIdeograph3f33',
u'\uc2b6': 'HangulSyllableSeubs',
u'\u87b8': 'CjkUnifiedIdeograph87b8',
u'\u44ba': 'CjkUnifiedIdeograph44ba',
u'\ue83d': 'UnicodeCharacter0xe83d',
u'\u09bc': 'BengaliSignNukta',
u'\uad3f': 'HangulSyllableGoelb',
u'\u9241': 'CjkUnifiedIdeograph9241',
u'\u5743': 'CjkUnifiedIdeograph5743',
u'\u1445': 'CanadianSyllabicsWestcreePwa',
u'\ufac6': 'CjkCompatibilityIdeographfac6',
u'\ubfc8': 'HangulSyllableBbyok',
u'\u7cca': 'CjkUnifiedIdeograph7cca',
u'\u21cc': 'RightwardsHarpoonOverLeftwardsHarpoon',
u'\uc54f': 'HangulSyllableAlb',
u'\u8a51': 'CjkUnifiedIdeograph8a51',
u'\u4f53': 'CjkUnifiedIdeograph4f53',
u'\u0c55': 'TeluguLengthMark',
u'\ud7d8': 'UnicodeCharacter0xd7d8',
u'\u94da': 'CjkUnifiedIdeograph94da',
u'\u59dc': 'CjkUnifiedIdeograph59dc',
u'\ufd5f': 'ArabicLigatureSeenWithMeemWithHahFinalForm',
u'\u1ede': 'LatinCapitalLetterOWithHornAndHookAbove',
u'\ua261': 'YiSyllableNgax',
u'\u6763': 'CjkUnifiedIdeograph6763',
u'\u2465': 'CircledDigitSix',
u'\ucfe8': 'HangulSyllableKul',
u'\u8cea': 'CjkUnifiedIdeograph8cea',
u'\u71ec': 'CjkUnifiedIdeograph71ec',
u'\u36ee': 'CjkUnifiedIdeograph36ee',
u'\uda71': 'UnicodeCharacter0xda71',
u'\u9f73': 'CjkUnifiedIdeograph9f73',
u'\u5c75': 'CjkUnifiedIdeograph5c75',
u'\u0177': 'LatinSmallLetterYWithCircumflex',
u'\ue7f8': 'UnicodeCharacter0xe7f8',
u'\ua4fa': 'UnicodeCharacter0xa4fa',
u'\u69fc': 'CjkUnifiedIdeograph69fc',
u'\u2efe': 'UnicodeCharacter0x2efe',
u'\u1400': 'UnicodeCharacter0x1400',
u'\u3083': 'HiraganaLetterSmallYa',
u'\ud702': 'HangulSyllableHwenh',
u'\uf385': 'UnicodeCharacter0xf385',
u'\u5d06': 'CjkUnifiedIdeograph5d06',
u'\u7989': 'CjkUnifiedIdeograph7989',
u'\u860c': 'CjkUnifiedIdeograph860c',
u'\ua28f': 'YiSyllableZi',
u'\u0c10': 'TeluguLetterAi',
u'\u2893': 'BraillePatternDots1258',
u'\ucf12': 'HangulSyllableKebs',
u'\ueb95': 'UnicodeCharacter0xeb95',
u'\u7516': 'CjkUnifiedIdeograph7516',
u'\u9199': 'CjkUnifiedIdeograph9199',
u'\u1f9d': 'GreekCapitalLetterEtaWithDasiaAndOxiaAndProsgegrammeni',
u'\ube1c': 'HangulSyllableBeum',
u'\uda9f': 'UnicodeCharacter0xda9f',
u'\u2420': 'SymbolForSpace',
u'\u40a3': 'CjkUnifiedIdeograph40a3',
u'\ue722': 'UnicodeCharacter0xe722',
u'\u6d26': 'CjkUnifiedIdeograph6d26',
u'\u89a9': 'CjkUnifiedIdeograph89a9',
u'\u37ad': 'CjkUnifiedIdeograph37ad',
u'\ud62c': 'HangulSyllableHyem',
u'\uf2af': 'UnicodeCharacter0xf2af',
u'\u5c30': 'CjkUnifiedIdeograph5c30',
u'\u78b3': 'CjkUnifiedIdeograph78b3',
u'\u8536': 'CjkUnifiedIdeograph8536',
u'\ua1b9': 'YiSyllableLi',
u'\u033a': 'CombiningInvertedBridgeBelow',
u'\u2fbd': 'KangxiRadicalHair',
u'\uce3c': 'HangulSyllableCyi',
u'\ueabf': 'UnicodeCharacter0xeabf',
u'\u7440': 'CjkUnifiedIdeograph7440',
u'\u90c3': 'CjkUnifiedIdeograph90c3',
u'\u1ec7': 'LatinSmallLetterEWithCircumflexAndDotBelow',
u'\ubd46': 'HangulSyllableBwaep',
u'\ud9c9': 'UnicodeCharacter0xd9c9',
u'\u3b4a': 'CjkUnifiedIdeograph3b4a',
u'\u47cd': 'CjkUnifiedIdeograph47cd',
u'\ue64c': 'UnicodeCharacter0xe64c',
u'\u6c50': 'CjkUnifiedIdeograph6c50',
u'\u88d3': 'CjkUnifiedIdeograph88d3',
u'\u36d7': 'CjkUnifiedIdeograph36d7',
u'\ud556': 'HangulSyllablePip',
u'\uf1d9': 'UnicodeCharacter0xf1d9',
u'\u535a': 'CjkUnifiedIdeograph535a',
u'\u7fdd': 'CjkUnifiedIdeograph7fdd',
u'\u8460': 'CjkUnifiedIdeograph8460',
u'\ua0e3': 'YiSyllableVix',
u'\u0264': 'LatinSmallLetterRamsHorn',
u'\u2ee7': 'CjkRadicalCsimplifiedSalt',
u'\ucd66': 'HangulSyllableCoelm',
u'\ue9e9': 'UnicodeCharacter0xe9e9',
u'\u4b6a': 'CjkUnifiedIdeograph4b6a',
u'\u97ed': 'CjkUnifiedIdeograph97ed',
u'\u1df1': 'UnicodeCharacter0x1df1',
u'\ubc70': 'HangulSyllableByael',
u'\ud8f3': 'UnicodeCharacter0xd8f3',
u'\u3a74': 'CjkUnifiedIdeograph3a74',
u'\u46f7': 'CjkUnifiedIdeograph46f7',
u'\ue576': 'UnicodeCharacter0xe576',
u'\u637a': 'CjkUnifiedIdeograph637a',
u'\u8ffd': 'CjkUnifiedIdeograph8ffd',
u'\ub501': 'HangulSyllableDyilg',
u'\ud480': 'HangulSyllablePul',
u'\u3305': 'SquareInti',
u'\u5284': 'CjkUnifiedIdeograph5284',
u'\ufe07': 'VariationSelector8',
u'\u640b': 'CjkUnifiedIdeograph640b',
u'\u9b8a': 'CjkUnifiedIdeograph9b8a',
u'\u018e': 'LatinCapitalLetterReversedE',
u'\uad11': 'HangulSyllableGwang',
u'\ucc90': 'HangulSyllableCyaess',
u'\u2b15': 'SquareWithLowerLeftDiagonalHalfBlack',
u'\u4a94': 'CjkUnifiedIdeograph4a94',
u'\u9c1b': 'CjkUnifiedIdeograph9c1b',
u'\ub39a': 'HangulSyllableDyeolp',
u'\u1a1f': 'BugineseEndOfSection',
u'\u399e': 'CjkUnifiedIdeograph399e',
u'\uc521': 'HangulSyllableSsying',
u'\ue4a0': 'UnicodeCharacter0xe4a0',
u'\u4325': 'CjkUnifiedIdeograph4325',
u'\u62a4': 'CjkUnifiedIdeograph62a4',
u'\ub42b': 'HangulSyllableDoes',
u'\uabaa': 'UnicodeCharacter0xabaa',
u'\u322f': 'ParenthesizedIdeographEarth',
u'\u51ae': 'CjkUnifiedIdeograph51ae',
u'\ufd31': 'ArabicLigatureSeenWithHehInitialForm',
u'\u7b35': 'CjkUnifiedIdeograph7b35',
u'\u9ab4': 'CjkUnifiedIdeograph9ab4',
u'\xb8': 'Cedilla',
u'\uac3b': 'HangulSyllableGyags',
u'\uc3ba': 'HangulSyllableSsyeop',
u'\u2a3f': 'AmalgamationOrCoproduct',
u'\u49be': 'CjkUnifiedIdeograph49be',
u'\u9345': 'CjkUnifiedIdeograph9345',
u'\ub2c4': 'HangulSyllableNyik',
u'\u1949': 'LimbuDigitThree',
u'\u38c8': 'CjkUnifiedIdeograph38c8',
u'\uc44b': 'HangulSyllableSsyogs',
u'\ufbca': 'UnicodeCharacter0xfbca',
u'\u424f': 'CjkUnifiedIdeograph424f',
u'\u61ce': 'CjkUnifiedIdeograph61ce',
u'\u8b55': 'CjkUnifiedIdeograph8b55',
u'\uaad4': 'UnicodeCharacter0xaad4',
u'\u3159': 'HangulLetterWae',
u'\u50d8': 'CjkUnifiedIdeograph50d8',
u'\ufc5b': 'ArabicLigatureThalWithSuperscriptAlefIsolatedForm',
u'\u7a5f': 'CjkUnifiedIdeograph7a5f',
u'\u99de': 'CjkUnifiedIdeograph99de',
u'\u07e2': 'NkoLetterNya',
u'\ua365': 'YiSyllableChuot',
u'\uc2e4': 'HangulSyllableSil',
u'\u2969': 'RightwardsHarpoonWithBarbDownAboveLeftwardsHarpoonWithBarbDown',
u'\u48e8': 'CjkUnifiedIdeograph48e8',
u'\u926f': 'CjkUnifiedIdeograph926f',
u'\ub1ee': 'HangulSyllableNyonh',
u'\u1873': 'MongolianLetterManchuI',
u'\u3ff2': 'CjkUnifiedIdeograph3ff2',
u'\udb75': 'UnicodeCharacter0xdb75',
u'\ufaf4': 'UnicodeCharacter0xfaf4',
u'\u4179': 'CjkUnifiedIdeograph4179',
u'\u60f8': 'CjkUnifiedIdeograph60f8',
u'\u8a7f': 'CjkUnifiedIdeograph8a7f',
u'\ua9fe': 'UnicodeCharacter0xa9fe',
u'\udc06': 'UnicodeCharacter0xdc06',
u'\uf889': 'UnicodeCharacter0xf889',
u'\u9908': 'CjkUnifiedIdeograph9908',
u'\ua58b': 'VaiSyllableCoo',
u'\u5a0a': 'CjkUnifiedIdeograph5a0a',
u'\u668d': 'CjkUnifiedIdeograph668d',
u'\u070c': 'SyriacHarkleanMetobelus',
u'\u238f': 'OpencircuitoutputHtypeSymbol',
u'\uf416': 'UnicodeCharacter0xf416',
u'\ub118': 'HangulSyllableNeom',
u'\udd9b': 'UnicodeCharacter0xdd9b',
u'\u721a': 'CjkUnifiedIdeograph721a',
u'\u9e9d': 'CjkUnifiedIdeograph9e9d',
u'\u3f1c': 'CjkUnifiedIdeograph3f1c',
u'\u5b9f': 'CjkUnifiedIdeograph5b9f',
u'\u04a1': 'CyrillicSmallLetterBashkirKa',
u'\uec26': 'UnicodeCharacter0xec26',
u'\ua928': 'KayahLiVowelU',
u'\uf5ab': 'UnicodeCharacter0xf5ab',
u'\u6a2a': 'CjkUnifiedIdeograph6a2a',
u'\ub6ad': 'HangulSyllableDdub',
u'\u572c': 'CjkUnifiedIdeograph572c',
u'\u73af': 'CjkUnifiedIdeograph73af',
u'\u102e': 'MyanmarVowelSignIi',
u'\u3cb1': 'CjkUnifiedIdeograph3cb1',
u'\uc138': 'HangulSyllableSe',
u'\uedbb': 'UnicodeCharacter0xedbb',
u'\u823a': 'CjkUnifiedIdeograph823a',
u'\uaebd': 'HangulSyllableGgeog',
u'\u4f3c': 'CjkUnifiedIdeograph4f3c',
u'\u6bbf': 'CjkUnifiedIdeograph6bbf',
u'\u083e': 'UnicodeCharacter0x83e',
u'\u54c1': 'CjkUnifiedIdeograph54c1',
u'\u11c3': 'HangulJongseongKiyeokrieul',
u'\uf948': 'CjkCompatibilityIdeographf948',
u'\uba4a': 'HangulSyllableMeobs',
u'\uc6cd': 'HangulSyllableWeog',
u'\u674c': 'CjkUnifiedIdeograph674c',
u'\u83cf': 'CjkUnifiedIdeograph83cf',
u'\u204e': 'LowAsterisk',
u'\u4cd1': 'CjkUnifiedIdeograph4cd1',
u'\u09d3': 'UnicodeCharacter0x9d3',
u'\ud25a': 'HangulSyllableTweobs',
u'\ufedd': 'ArabicLetterLamIsolatedForm',
u'\u9f5c': 'CjkUnifiedIdeograph9f5c',
u'\ubbdf': 'HangulSyllableMyigs',
u'\u585e': 'CjkUnifiedIdeograph585e',
u'\u64e1': 'CjkUnifiedIdeograph64e1',
u'\u0560': 'UnicodeCharacter0x560',
u'\u21e3': 'DownwardsDashedArrow',
u'\uca6a': 'HangulSyllableJjegg',
u'\ub76c': 'HangulSyllableDdils',
u'\ud3ef': 'HangulSyllablePogs',
u'\u706e': 'CjkUnifiedIdeograph706e',
u'\u9cf1': 'CjkUnifiedIdeograph9cf1',
u'\u3d70': 'CjkUnifiedIdeograph3d70',
u'\u59f3': 'CjkUnifiedIdeograph59f3',
u'\u1af5': 'UnicodeCharacter0x1af5',
u'\ue27a': 'UnicodeCharacter0xe27a',
u'\uaf7c': 'HangulSyllableGgwaek',
u'\ucbff': 'HangulSyllableJjyilh',
u'\u687e': 'CjkUnifiedIdeograph687e',
u'\u5580': 'CjkUnifiedIdeograph5580',
u'\uf103': 'UnicodeCharacter0xf103',
u'\u1682': 'OghamLetterLuis',
u'\ub205': 'HangulSyllableNug',
u'\u7f07': 'CjkUnifiedIdeograph7f07',
u'\u3809': 'CjkUnifiedIdeograph3809',
u'\uc78c': 'HangulSyllableIk',
u'\u808e': 'CjkUnifiedIdeograph808e',
u'\u4d90': 'CjkUnifiedIdeograph4d90',
u'\ue913': 'UnicodeCharacter0xe913',
u'\u0e92': 'UnicodeCharacter0xe92',
u'\uaa15': 'ChamLetterDa',
u'\u9717': 'CjkUnifiedIdeograph9717',
u'\u5019': 'CjkUnifiedIdeograph5019',
u'\u1d1b': 'LatinLetterSmallCapitalT',
u'\uff9c': 'HalfwidthKatakanaLetterWa',
u'\ub89e': 'HangulSyllableRwaelm',
u'\u65a0': 'CjkUnifiedIdeograph65a0',
u'\u26a2': 'DoubledFemaleSign',
u'\uc225': 'HangulSyllableSult',
u'\u8f27': 'CjkUnifiedIdeograph8f27',
u'\u4829': 'CjkUnifiedIdeograph4829',
u'\u352b': 'CjkUnifiedIdeograph352b',
u'\ud0ae': 'HangulSyllableKilm',
u'\u9db0': 'CjkUnifiedIdeograph9db0',
u'\u5eb2': 'CjkUnifiedIdeograph5eb2',
u'\ufa35': 'CjkCompatibilityIdeographfa35',
u'\u1bb4': 'SundaneseDigitFour',
u'\ua737': 'LatinSmallLetterAu',
u'\u6039': 'CjkUnifiedIdeograph6039',
u'\u2d3b': 'TifinaghLetterYey',
u'\uc8be': 'HangulSyllableJwaej',
u'\ub5c0': 'HangulSyllableDden',
u'\u76c2': 'CjkUnifiedIdeograph76c2',
u'\u33c4': 'SquareCc',
u'\udf47': 'UnicodeCharacter0xdf47',
u'\u9849': 'CjkUnifiedIdeograph9849',
u'\u454b': 'CjkUnifiedIdeograph454b',
u'\u064d': 'ArabicKasratan',
u'\ue0ce': 'UnicodeCharacter0xe0ce',
u'\uadd0': 'HangulSyllableGwim',
u'\u6ed2': 'CjkUnifiedIdeograph6ed2',
u'\u2bd4': 'UnicodeCharacter0x2bd4',
u'\uf757': 'UnicodeCharacter0xf757',
u'\u14d6': 'CanadianSyllabicsLii',
u'\ub059': 'HangulSyllableGgeung',
u'\u7d5b': 'CjkUnifiedIdeograph7d5b',
u'\u3e5d': 'CjkUnifiedIdeograph3e5d',
u'\uc5e0': 'HangulSyllableEm',
u'\u86e2': 'CjkUnifiedIdeograph86e2',
u'\u43e4': 'CjkUnifiedIdeograph43e4',
u'\uef67': 'UnicodeCharacter0xef67',
u'\u0ce6': 'KannadaDigitZero',
u'\ua869': 'PhagspaLetterTta',
u'\u956b': 'CjkUnifiedIdeograph956b',
u'\u566d': 'CjkUnifiedIdeograph566d',
u'\u136f': 'EthiopicDigitSeven',
u'\ufdf0': 'ArabicLigatureSallaUsedAsKoranicStopSignIsolatedForm',
u'\ubef2': 'HangulSyllableBbenh',
u'\u7bf4': 'CjkUnifiedIdeograph7bf4',
u'\u24f6': 'DoubleCircledDigitTwo',
u'\uc079': 'HangulSyllableBbyinj',
u'\u8d7b': 'CjkUnifiedIdeograph8d7b',
u'\u4e7d': 'CjkUnifiedIdeograph4e7d',
u'\u0b7f': 'UnicodeCharacter0xb7f',
u'\u7781': 'CjkUnifiedIdeograph7781',
u'\u9004': 'CjkUnifiedIdeograph9004',
u'\ubc87': 'HangulSyllableBeogs',
u'\u1a08': 'BugineseLetterTa',
u'\u268b': 'MonogramForYin',
u'\ud90a': 'UnicodeCharacter0xd90a',
u'\ue58d': 'UnicodeCharacter0xe58d',
u'\u430e': 'CjkUnifiedIdeograph430e',
u'\u6f91': 'CjkUnifiedIdeograph6f91',
u'\u8814': 'CjkUnifiedIdeograph8814',
u'\ud497': 'HangulSyllablePweogs',
u'\u3218': 'ParenthesizedHangulKhieukhA',
u'\u5e9b': 'CjkUnifiedIdeograph5e9b',
u'\uf11a': 'UnicodeCharacter0xf11a',
u'\u7b1e': 'CjkUnifiedIdeograph7b1e',
u'\u87a1': 'CjkUnifiedIdeograph87a1',
u'\u01a5': 'LatinSmallLetterPWithHook',
u'\ua024': 'YiSyllableBuop',
u'\ucca7': 'HangulSyllableCeolh',
u'\u2a28': 'PlusSignWithBlackTriangle',
u'\u76ab': 'CjkUnifiedIdeograph76ab',
u'\ue92a': 'UnicodeCharacter0xe92a',
u'\u932e': 'CjkUnifiedIdeograph932e',
u'\ubfb1': 'HangulSyllableBbyog',
u'\u1d32': 'ModifierLetterCapitalReversedE',
u'\u39b5': 'CjkUnifiedIdeograph39b5',
u'\ud834': 'UnicodeCharacter0xd834',
u'\ue4b7': 'UnicodeCharacter0xe4b7',
u'\u4238': 'CjkUnifiedIdeograph4238',
u'\u6ebb': 'CjkUnifiedIdeograph6ebb',
u'\u8b3e': 'CjkUnifiedIdeograph8b3e',
u'\ud7c1': 'UnicodeCharacter0xd7c1',
u'\u3542': 'CjkUnifiedIdeograph3542',
u'\u51c5': 'CjkUnifiedIdeograph51c5',
u'\uf044': 'UnicodeCharacter0xf044',
u'\u7a48': 'CjkUnifiedIdeograph7a48',
u'\u86cb': 'CjkUnifiedIdeograph86cb',
u'\xcf': 'LatinCapitalLetterIWithDiaeresis',
u'\ua34e': 'YiSyllableZhox',
u'\ucfd1': 'HangulSyllableKyolt',
u'\u2d52': 'TifinaghLetterYap',
u'\u49d5': 'CjkUnifiedIdeograph49d5',
u'\ue854': 'UnicodeCharacter0xe854',
u'\u9258': 'CjkUnifiedIdeograph9258',
u'\ubedb': 'HangulSyllableBbeolb',
u'\u1c5c': 'OlChikiLetterAg',
u'\u38df': 'CjkUnifiedIdeograph38df',
u'\udb5e': 'UnicodeCharacter0xdb5e',
u'\ue7e1': 'UnicodeCharacter0xe7e1',
u'\u4562': 'CjkUnifiedIdeograph4562',
u'\u61e5': 'CjkUnifiedIdeograph61e5',
u'\u8a68': 'CjkUnifiedIdeograph8a68',
u'\ud6eb': 'HangulSyllableHweolb',
u'\u346c': 'CjkUnifiedIdeograph346c',
u'\u50ef': 'CjkUnifiedIdeograph50ef',
u'\uf36e': 'UnicodeCharacter0xf36e',
u'\u7d72': 'CjkUnifiedIdeograph7d72',
u'\u99f5': 'CjkUnifiedIdeograph99f5',
u'\u03f9': 'GreekCapitalLunateSigmaSymbol',
u'\ua278': 'YiSyllableHuop',
u'\ucefb': 'HangulSyllableKeoc',
u'\u2c7c': 'LatinSubscriptSmallLetterJ',
u'\u48ff': 'CjkUnifiedIdeograph48ff',
u'\ueb7e': 'UnicodeCharacter0xeb7e',
u'\u7203': 'CjkUnifiedIdeograph7203',
u'\u9582': 'CjkUnifiedIdeograph9582',
u'\u1f86': 'GreekSmallLetterAlphaWithPsiliAndPerispomeniAndYpogegrammeni',
u'\ubb09': 'HangulSyllableMoelt',
u'\uda88': 'UnicodeCharacter0xda88',
u'\u250d': 'BoxDrawingsDownLightAndRightHeavy',
u'\u448c': 'CjkUnifiedIdeograph448c',
u'\ue00f': 'UnicodeCharacter0xe00f',
u'\u6a13': 'CjkUnifiedIdeograph6a13',
u'\u8d92': 'CjkUnifiedIdeograph8d92',
u'\u1417': 'CanadianSyllabicsWa',
u'\u3796': 'CjkUnifiedIdeograph3796',
u'\ud319': 'HangulSyllablePalt',
u'\uf298': 'UnicodeCharacter0xf298',
u'\u5d1d': 'CjkUnifiedIdeograph5d1d',
u'\u7c9c': 'CjkUnifiedIdeograph7c9c',
u'\u8223': 'CjkUnifiedIdeograph8223',
u'\ua5a2': 'VaiSyllableFu',
u'\u0c27': 'TeluguLetterDha',
u'\u2fa6': 'KangxiRadicalGold',
u'\ucb29': 'HangulSyllableJjoet',
u'\ueaa8': 'UnicodeCharacter0xeaa8',
u'\u752d': 'CjkUnifiedIdeograph752d',
u'\u94ac': 'CjkUnifiedIdeograph94ac',
u'\u1eb0': 'LatinCapitalLetterAWithBreveAndGrave',
u'\uba33': 'HangulSyllableMyaec',
u'\uddb2': 'UnicodeCharacter0xddb2',
u'\u2437': 'UnicodeCharacter0x2437',
u'\u47b6': 'CjkUnifiedIdeograph47b6',
u'\ue339': 'UnicodeCharacter0xe339',
u'\u6d3d': 'CjkUnifiedIdeograph6d3d',
u'\u8cbc': 'CjkUnifiedIdeograph8cbc',
u'\u1741': 'BuhidLetterI',
u'\u36c0': 'CjkUnifiedIdeograph36c0',
u'\ud243': 'HangulSyllableTuc',
u'\uf5c2': 'UnicodeCharacter0xf5c2',
u'\u5c47': 'CjkUnifiedIdeograph5c47',
u'\u7fc6': 'CjkUnifiedIdeograph7fc6',
u'\u854d': 'CjkUnifiedIdeograph854d',
u'\ua4cc': 'UnicodeCharacter0xa4cc',
u'\u0f51': 'TibetanLetterDa',
u'\u2ed0': 'CjkRadicalCsimplifiedGold',
u'\uca53': 'HangulSyllableJjeod',
u'\uedd2': 'UnicodeCharacter0xedd2',
u'\u7457': 'CjkUnifiedIdeograph7457',
u'\u97d6': 'CjkUnifiedIdeograph97d6',
u'\u11da': 'HangulJongseongMieumkiyeok',
u'\ubd5d': 'HangulSyllableBoeng',
u'\udcdc': 'UnicodeCharacter0xdcdc',
u'\u2761': 'CurvedStemParagraphSignOrnament',
u'\u46e0': 'CjkUnifiedIdeograph46e0',
u'\ue263': 'UnicodeCharacter0xe263',
u'\u6c67': 'CjkUnifiedIdeograph6c67',
u'\u8fe6': 'CjkUnifiedIdeograph8fe6',
u'\u166b': 'CanadianSyllabicsCarrierTtsi',
u'\u09ea': 'BengaliDigitFour',
u'\ud56d': 'HangulSyllableHang',
u'\uf4ec': 'UnicodeCharacter0xf4ec',
u'\u5f71': 'CjkUnifiedIdeograph5f71',
u'\u7ef0': 'CjkUnifiedIdeograph7ef0',
u'\u8477': 'CjkUnifiedIdeograph8477',
u'\ua7f6': 'UnicodeCharacter0xa7f6',
u'\u0e7b': 'UnicodeCharacter0xe7b',
u'\u21fa': 'LeftwardsArrowWithDoubleVerticalStroke',
u'\ucd7d': 'HangulSyllableCyonj',
u'\uecfc': 'UnicodeCharacter0xecfc',
u'\uf681': 'UnicodeCharacter0xf681',
u'\u9700': 'CjkUnifiedIdeograph9700',
u'\ub383': 'HangulSyllableDes',
u'\u5402': 'CjkUnifiedIdeograph5402',
u'\u7085': 'CjkUnifiedIdeograph7085',
u'\u1104': 'HangulChoseongSsangtikeut',
u'\u3d87': 'CjkUnifiedIdeograph3d87',
u'\uc20e': 'HangulSyllableSyobs',
u'\uee91': 'UnicodeCharacter0xee91',
u'\u8f10': 'CjkUnifiedIdeograph8f10',
u'\uab93': 'UnicodeCharacter0xab93',
u'\u4c12': 'CjkUnifiedIdeograph4c12',
u'\u6895': 'CjkUnifiedIdeograph6895',
u'\u0914': 'DevanagariLetterAu',
u'\u5597': 'CjkUnifiedIdeograph5597',
u'\u1299': 'EthiopicSyllableNyu',
u'\ufa1e': 'CjkCompatibilityIdeographfa1e',
u'\ua720': 'ModifierLetterStressAndHighTone',
u'\uc3a3': 'HangulSyllableSsyeogs',
u'\u6422': 'CjkUnifiedIdeograph6422',
u'\u80a5': 'CjkUnifiedIdeograph80a5',
u'\u2124': 'DoublestruckCapitalZ',
u'\u4da7': 'CjkUnifiedIdeograph4da7',
u'\u0aa9': 'UnicodeCharacter0xaa9',
u'\udf30': 'UnicodeCharacter0xdf30',
u'\ufbb3': 'UnicodeCharacter0xfbb3',
u'\u9c32': 'CjkUnifiedIdeograph9c32',
u'\ub8b5': 'HangulSyllableRoenj',
u'\u5934': 'CjkUnifiedIdeograph5934',
u'\u65b7': 'CjkUnifiedIdeograph65b7',
u'\u0636': 'ArabicLetterDad',
u'\u22b9': 'HermitianConjugateMatrix',
u'\uf740': 'UnicodeCharacter0xf740',
u'\ub442': 'HangulSyllableDyolp',
u'\ud0c5': 'HangulSyllableTanj',
u'\u7144': 'CjkUnifiedIdeograph7144',
u'\u9dc7': 'CjkUnifiedIdeograph9dc7',
u'\u3e46': 'CjkUnifiedIdeograph3e46',
u'\u5ac9': 'CjkUnifiedIdeograph5ac9',
u'\u07cb': 'NkoLetterEe',
u'\uef50': 'UnicodeCharacter0xef50',
u'\uac52': 'HangulSyllableGyap',
u'\uc8d5': 'HangulSyllableJoeb',
u'\u6954': 'CjkUnifiedIdeograph6954',
u'\ub5d7': 'HangulSyllableDdeh',
u'\u5656': 'CjkUnifiedIdeograph5656',
u'\u72d9': 'CjkUnifiedIdeograph72d9',
u'\u1358': 'EthiopicSyllableRya',
u'\u3fdb': 'CjkUnifiedIdeograph3fdb',
u'\uc462': 'HangulSyllableSsyop',
u'\ue0e5': 'UnicodeCharacter0xe0e5',
u'\u8164': 'CjkUnifiedIdeograph8164',
u'\uade7': 'HangulSyllableGyulb',
u'\u4e66': 'CjkUnifiedIdeograph4e66',
u'\u6ae9': 'CjkUnifiedIdeograph6ae9',
u'\u0b68': 'OriyaDigitTwo',
u'\u57eb': 'CjkUnifiedIdeograph57eb',
u'\u14ed': 'CanadianSyllabicsSe',
u'\ufc72': 'ArabicLigatureTehWithMeemFinalForm',
u'\ub974': 'HangulSyllableReu',
u'\uc5f7': 'HangulSyllableYeolb',
u'\u6676': 'CjkUnifiedIdeograph6676',
u'\u82f9': 'CjkUnifiedIdeograph82f9',
u'\u2378': 'AplFunctionalSymbolIotaUnderbar',
u'\u4ffb': 'CjkUnifiedIdeograph4ffb',
u'\u0cfd': 'UnicodeCharacter0xcfd',
u'\u3601': 'CjkUnifiedIdeograph3601',
u'\ud184': 'HangulSyllableTye',
u'\u9e86': 'CjkUnifiedIdeograph9e86',
u'\u5b88': 'CjkUnifiedIdeograph5b88',
u'\ue70b': 'UnicodeCharacter0xe70b',
u'\u188a': 'MongolianLetterAliGaliNga',
u'\ua40d': 'YiSyllableQux',
u'\u610f': 'CjkUnifiedIdeograph610f',
u'\u2e11': 'ReversedForkedParagraphos',
u'\uc994': 'HangulSyllableJeuls',
u'\ub696': 'HangulSyllableDdyoj',
u'\u7398': 'CjkUnifiedIdeograph7398',
u'\u309a': 'CombiningKatakanahiraganaSemivoicedSoundMark',
u'\udc1d': 'UnicodeCharacter0xdc1d',
u'\u991f': 'CjkUnifiedIdeograph991f',
u'\u4621': 'CjkUnifiedIdeograph4621',
u'\u0323': 'CombiningDotBelow',
u'\ue1a4': 'UnicodeCharacter0xe1a4',
u'\uaea6': 'HangulSyllableGgyaenh',
u'\u6ba8': 'CjkUnifiedIdeograph6ba8',
u'\u28aa': 'BraillePatternDots2468',
u'\uf42d': 'UnicodeCharacter0xf42d',
u'\u15ac': 'CanadianSyllabicsThcreeTha',
u'\ub12f': 'HangulSyllableNelb',
u'\u7e31': 'CjkUnifiedIdeograph7e31',
u'\u3b33': 'CjkUnifiedIdeograph3b33',
u'\uc6b6': 'HangulSyllableUnh',
u'\u83b8': 'CjkUnifiedIdeograph83b8',
u'\u40ba': 'CjkUnifiedIdeograph40ba',
u'\uec3d': 'UnicodeCharacter0xec3d',
u'\u0dbc': 'UnicodeCharacter0xdbc',
u'\ua93f': 'RejangLetterYa',
u'\u9641': 'CjkUnifiedIdeograph9641',
u'\u5343': 'CjkUnifiedIdeograph5343',
u'\u1045': 'MyanmarDigitFive',
u'\ufec6': 'ArabicLetterZahFinalForm',
u'\ubbc8': 'HangulSyllableMeul',
u'\u78ca': 'CjkUnifiedIdeograph78ca',
u'\u25cc': 'DottedCircle',
u'\uc14f': 'HangulSyllableSec',
u'\u8e51': 'CjkUnifiedIdeograph8e51',
u'\u4b53': 'CjkUnifiedIdeograph4b53',
u'\u0855': 'UnicodeCharacter0x855',
u'\ud3d8': 'HangulSyllablePyel',
u'\u90da': 'CjkUnifiedIdeograph90da',
u'\u5ddc': 'CjkUnifiedIdeograph5ddc',
u'\uf95f': 'CjkCompatibilityIdeographf95f',
u'\u1ade': 'UnicodeCharacter0x1ade',
u'\ua661': 'UnicodeCharacter0xa661',
u'\u6363': 'CjkUnifiedIdeograph6363',
u'\u2065': 'UnicodeCharacter0x2065',
u'\ucbe8': 'HangulSyllableJjeuss',
u'\u88ea': 'CjkUnifiedIdeograph88ea',
u'\u75ec': 'CjkUnifiedIdeograph75ec',
u'\u32ee': 'CircledKatakanaMa',
u'\ude71': 'UnicodeCharacter0xde71',
u'\u9b73': 'CjkUnifiedIdeograph9b73',
u'\u5875': 'CjkUnifiedIdeograph5875',
u'\u0577': 'ArmenianSmallLetterSha',
u'\ue3f8': 'UnicodeCharacter0xe3f8',
u'\ua0fa': 'YiSyllableVyt',
u'\u6dfc': 'CjkUnifiedIdeograph6dfc',
u'\u2afe': 'WhiteVerticalBar',
u'\u1800': 'MongolianBirga',
u'\u3c83': 'CjkUnifiedIdeograph3c83',
u'\udb02': 'UnicodeCharacter0xdb02',
u'\uff85': 'HalfwidthKatakanaLetterNa',
u'\u5106': 'CjkUnifiedIdeograph5106',
u'\u7589': 'CjkUnifiedIdeograph7589',
u'\u8a0c': 'CjkUnifiedIdeograph8a0c',
u'\uae8f': 'HangulSyllableGgyalb',
u'\u2493': 'NumberTwelveFullStop',
u'\uc312': 'HangulSyllableSsap',
u'\ue795': 'UnicodeCharacter0xe795',
u'\u7916': 'CjkUnifiedIdeograph7916',
u'\u9d99': 'CjkUnifiedIdeograph9d99',
u'\u139d': 'UnicodeCharacter0x139d',
u'\ub21c': 'HangulSyllableNuk',
u'\ud69f': 'HangulSyllableHoes',
u'\u2820': 'BraillePatternDots6',
u'\u4ca3': 'CjkUnifiedIdeograph4ca3',
u'\ueb22': 'UnicodeCharacter0xeb22',
u'\u6126': 'CjkUnifiedIdeograph6126',
u'\u85a9': 'CjkUnifiedIdeograph85a9',
u'\u3bad': 'CjkUnifiedIdeograph3bad',
u'\uda2c': 'UnicodeCharacter0xda2c',
u'\ufeaf': 'ArabicLetterZainIsolatedForm',
u'\u5030': 'CjkUnifiedIdeograph5030',
u'\u74b3': 'CjkUnifiedIdeograph74b3',
u'\u8936': 'CjkUnifiedIdeograph8936',
u'\uadb9': 'HangulSyllableGweng',
u'\u0f3a': 'TibetanMarkGugRtagsGyon',
u'\u23bd': 'HorizontalScanLine9',
u'\uc23c': 'HangulSyllableSweol',
u'\ue6bf': 'UnicodeCharacter0xe6bf',
u'\u7840': 'CjkUnifiedIdeograph7840',
u'\u9cc3': 'CjkUnifiedIdeograph9cc3',
u'\u12c7': 'UnicodeCharacter0x12c7',
u'\ub146': 'HangulSyllableNyeonh',
u'\ud5c9': 'HangulSyllableHeog',
u'\u374a': 'CjkUnifiedIdeograph374a',
u'\u4bcd': 'CjkUnifiedIdeograph4bcd',
u'\uea4c': 'UnicodeCharacter0xea4c',
u'\u6050': 'CjkUnifiedIdeograph6050',
u'\u84d3': 'CjkUnifiedIdeograph84d3',
u'\u3ad7': 'CjkUnifiedIdeograph3ad7',
u'\ud956': 'UnicodeCharacter0xd956',
u'\ufdd9': 'UnicodeCharacter0xfdd9',
u'\u5f5a': 'CjkUnifiedIdeograph5f5a',
u'\u73dd': 'CjkUnifiedIdeograph73dd',
u'\u8860': 'CjkUnifiedIdeograph8860',
u'\uace3': 'HangulSyllableGogs',
u'\u0e64': 'UnicodeCharacter0xe64',
u'\u22e7': 'GreaterthanButNotEquivalentTo',
u'\uc166': 'HangulSyllableSyeobs',
u'\ue5e9': 'UnicodeCharacter0xe5e9',
u'\u476a': 'CjkUnifiedIdeograph476a',
u'\u9bed': 'CjkUnifiedIdeograph9bed',
u'\u11f1': 'HangulJongseongYesieungsios',
u'\ub070': 'HangulSyllableGgyim',
u'\ud4f3': 'HangulSyllablePyulb',
u'\u3674': 'CjkUnifiedIdeograph3674',
u'\u4af7': 'CjkUnifiedIdeograph4af7',
u'\ue976': 'UnicodeCharacter0xe976',
u'\u6f7a': 'CjkUnifiedIdeograph6f7a',
u'\u83fd': 'CjkUnifiedIdeograph83fd',
u'\ub901': 'HangulSyllableRut',
u'\ud880': 'UnicodeCharacter0xd880',
u'\u3f05': 'CjkUnifiedIdeograph3f05',
u'\u5e84': 'CjkUnifiedIdeograph5e84',
u'\uf207': 'UnicodeCharacter0xf207',
u'\u680b': 'CjkUnifiedIdeograph680b',
u'\u978a': 'CjkUnifiedIdeograph978a',
u'\u0d8e': 'SinhalaLetterIruuyanna',
u'\ua111': 'YiSyllableDex',
u'\uc090': 'HangulSyllableBbi',
u'\u2715': 'MultiplicationX',
u'\u4694': 'CjkUnifiedIdeograph4694',
u'\u901b': 'CjkUnifiedIdeograph901b',
u'\ubf9a': 'HangulSyllableBboenh',
u'\u161f': 'CanadianSyllabicsCarrierJjee',
u'\u359e': 'CjkUnifiedIdeograph359e',
u'\uc921': 'HangulSyllableJweolg',
u'\ue8a0': 'UnicodeCharacter0xe8a0',
u'\u4f25': 'CjkUnifiedIdeograph4f25',
u'\u6ea4': 'CjkUnifiedIdeograph6ea4',
u'\ub82b': 'HangulSyllableRyeod',
u'\ua7aa': 'UnicodeCharacter0xa7aa',
u'\u3e2f': 'CjkUnifiedIdeograph3e2f',
u'\u5dae': 'CjkUnifiedIdeograph5dae',
u'\uf131': 'UnicodeCharacter0xf131',
u'\u7735': 'CjkUnifiedIdeograph7735',
u'\u96b4': 'CjkUnifiedIdeograph96b4',
u'\u0cb8': 'KannadaLetterSa',
u'\ua03b': 'YiSyllablePip',
u'\ucfba': 'HangulSyllableKoebs',
u'\u263f': 'Mercury',
u'\u45be': 'CjkUnifiedIdeograph45be',
u'\u9f45': 'CjkUnifiedIdeograph9f45',
u'\ubec4': 'HangulSyllableBbyaem',
u'\u1549': 'CanadianSyllabicsRoo',
u'\u34c8': 'CjkUnifiedIdeograph34c8',
u'\uc84b': 'HangulSyllableJyeos',
u'\uf7ca': 'UnicodeCharacter0xf7ca',
u'\u4e4f': 'CjkUnifiedIdeograph4e4f',
u'\u6dce': 'CjkUnifiedIdeograph6dce',
u'\u8755': 'CjkUnifiedIdeograph8755',
u'\ua6d4': 'UnicodeCharacter0xa6d4',
u'\u3d59': 'CjkUnifiedIdeograph3d59',
u'\u5cd8': 'CjkUnifiedIdeograph5cd8',
u'\uf05b': 'UnicodeCharacter0xf05b',
u'\u765f': 'CjkUnifiedIdeograph765f',
u'\u95de': 'CjkUnifiedIdeograph95de',
u'\u0be2': 'UnicodeCharacter0xbe2',
u'\uaf65': 'HangulSyllableGgwaeg',
u'\ucee4': 'HangulSyllableKeo',
u'\u2569': 'BoxDrawingsDoubleUpAndHorizontal',
u'\u44e8': 'CjkUnifiedIdeograph44e8',
u'\u9e6f': 'CjkUnifiedIdeograph9e6f',
u'\ubdee': 'HangulSyllableBwip',
u'\u1473': 'CanadianSyllabicsKaa',
u'\u33f2': 'IdeographicTelegraphSymbolForDayNineteen',
u'\ud775': 'HangulSyllableHyilg',
u'\uf6f4': 'UnicodeCharacter0xf6f4',
u'\u4d79': 'CjkUnifiedIdeograph4d79',
u'\u6cf8': 'CjkUnifiedIdeograph6cf8',
u'\u867f': 'CjkUnifiedIdeograph867f',
u'\ua5fe': 'VaiSyllableZhe',
u'\ud006': 'HangulSyllableKweolm',
u'\uf489': 'UnicodeCharacter0xf489',
u'\u9508': 'CjkUnifiedIdeograph9508',
u'\ua98b': 'UnicodeCharacter0xa98b',
u'\u560a': 'CjkUnifiedIdeograph560a',
u'\u6a8d': 'CjkUnifiedIdeograph6a8d',
u'\u0b0c': 'OriyaLetterVocalicL',
u'\u2f8f': 'KangxiRadicalWalkEnclosure',
u'\uf816': 'UnicodeCharacter0xf816',
u'\ubd18': 'HangulSyllableBwal',
u'\ud19b': 'HangulSyllableTyec',
u'\u7e1a': 'CjkUnifiedIdeograph7e1a',
u'\u929d': 'CjkUnifiedIdeograph929d',
u'\u331c': 'SquareKeesu',
u'\u579f': 'CjkUnifiedIdeograph579f',
u'\u08a1': 'UnicodeCharacter0x8a1',
u'\ue026': 'UnicodeCharacter0xe026',
u'\ua528': 'VaiSyllableWi',
u'\uf9ab': 'CjkCompatibilityIdeographf9ab',
u'\u662a': 'CjkUnifiedIdeograph662a',
u'\ubaad': 'HangulSyllableMonj',
u'\u5b2c': 'CjkUnifiedIdeograph5b2c',
u'\u7faf': 'CjkUnifiedIdeograph7faf',
u'\u1c2e': 'LepchaConsonantSignM',
u'\u30b1': 'KatakanaLetterKe',
u'\ucd38': 'HangulSyllableCwass',
u'\ue1bb': 'UnicodeCharacter0xe1bb',
u'\u8e3a': 'CjkUnifiedIdeograph8e3a',
u'\ua2bd': 'YiSyllableCot',
u'\u433c': 'CjkUnifiedIdeograph433c',
u'\u67bf': 'CjkUnifiedIdeograph67bf',
u'\u043e': 'CyrillicSmallLetterO',
u'\u58c1': 'CjkUnifiedIdeograph58c1',
u'\u1dc3': 'CombiningSuspensionMark',
u'\uf548': 'UnicodeCharacter0xf548',
u'\ub64a': 'HangulSyllableDdwaegg',
u'\ucacd': 'HangulSyllableJjob',
u'\u6b4c': 'CjkUnifiedIdeograph6b4c',
u'\u8fcf': 'CjkUnifiedIdeograph8fcf',
u'\u2c4e': 'GlagoliticSmallLetterSha',
u'\u40d1': 'CjkUnifiedIdeograph40d1',
u'\u05d3': 'HebrewLetterDalet',
u'\ude5a': 'UnicodeCharacter0xde5a',
u'\uf2dd': 'UnicodeCharacter0xf2dd',
u'\u935c': 'CjkUnifiedIdeograph935c',
u'\ub7df': 'HangulSyllableRyaelh',
u'\u545e': 'CjkUnifiedIdeograph545e',
u'\u68e1': 'CjkUnifiedIdeograph68e1',
u'\u0960': 'DevanagariLetterVocalicRr',
u'\u2de3': 'CombiningCyrillicLetterDe',
u'\uc66a': 'HangulSyllableWaelp',
u'\ubb6c': 'HangulSyllableMwe',
u'\udfef': 'UnicodeCharacter0xdfef',
u'\u7c6e': 'CjkUnifiedIdeograph7c6e',
u'\u90f1': 'CjkUnifiedIdeograph90f1',
u'\u3170': 'HangulLetterMieumpansios',
u'\u55f3': 'CjkUnifiedIdeograph55f3',
u'\u16f5': 'UnicodeCharacter0x16f5',
u'\uee7a': 'UnicodeCharacter0xee7a',
u'\ua37c': 'YiSyllableRrax',
u'\uc7ff': 'HangulSyllableJyaeh',
u'\u647e': 'CjkUnifiedIdeograph647e',
u'\u5980': 'CjkUnifiedIdeograph5980',
u'\ufd03': 'ArabicLigatureKhahWithAlefMaksuraIsolatedForm',
u'\u1a82': 'UnicodeCharacter0x1a82',
u'\ube05': 'HangulSyllableByung',
u'\u7307': 'CjkUnifiedIdeograph7307',
u'\u3409': 'CjkUnifiedIdeograph3409',
u'\ucb8c': 'HangulSyllableJjwels',
u'\u8c8e': 'CjkUnifiedIdeograph8c8e',
u'\u4190': 'CjkUnifiedIdeograph4190',
u'\ue513': 'UnicodeCharacter0xe513',
u'\u0292': 'LatinSmallLetterEzh',
u'\ua615': 'VaiSymbolTing',
u'\u9b17': 'CjkUnifiedIdeograph9b17',
u'\u5c19': 'CjkUnifiedIdeograph5c19',
u'\u111b': 'HangulChoseongKapyeounrieul',
u'\uf39c': 'UnicodeCharacter0xf39c',
u'\ub49e': 'HangulSyllableDwej',
u'\u69a0': 'CjkUnifiedIdeograph69a0',
u'\u2aa2': 'DoubleNestedGreaterthan',
u'\uce25': 'HangulSyllableCeunj',
u'\u8327': 'CjkUnifiedIdeograph8327',
u'\u4429': 'CjkUnifiedIdeograph4429',
u'\u392b': 'CjkUnifiedIdeograph392b',
u'\udcae': 'UnicodeCharacter0xdcae',
u'\u91b0': 'CjkUnifiedIdeograph91b0',
u'\u52b2': 'CjkUnifiedIdeograph52b2',
u'\uf635': 'UnicodeCharacter0xf635',
u'\u17b4': 'KhmerVowelInherentAq',
u'\uab37': 'UnicodeCharacter0xab37',
u'\u6c39': 'CjkUnifiedIdeograph6c39',
u'\u213b': 'FacsimileSign',
u'\uc4be': 'HangulSyllableSswinh',
u'\ub9c0': 'HangulSyllableRiss',
u'\u7ac2': 'CjkUnifiedIdeograph7ac2',
u'\u3fc4': 'CjkUnifiedIdeograph3fc4',
u'\ud347': 'HangulSyllablePyags',
u'\u9449': 'CjkUnifiedIdeograph9449',
u'\u494b': 'CjkUnifiedIdeograph494b',
u'\u0a4d': 'GurmukhiSignVirama',
u'\uecce': 'UnicodeCharacter0xecce',
u'\ua1d0': 'YiSyllableLu',
u'\u62d2': 'CjkUnifiedIdeograph62d2',
u'\u27d4': 'UpperLeftCornerWithDot',
u'\ufb57': 'ArabicLetterPehFinalForm',
u'\u18d6': 'UnicodeCharacter0x18d6',
u'\ubc59': 'HangulSyllableByalt',
u'\u715b': 'CjkUnifiedIdeograph715b',
u'\u325d': 'CircledNumberThirtyThree',
u'\uc9e0': 'HangulSyllableJjan',
u'\u8ae2': 'CjkUnifiedIdeograph8ae2',
u'\u4fe4': 'CjkUnifiedIdeograph4fe4',
u'\ue367': 'UnicodeCharacter0xe367',
u'\xe6': 'LatinSmallLetterAe',
u'\ua469': 'YiSyllableXo',
u'\u996b': 'CjkUnifiedIdeograph996b',
u'\u5a6d': 'CjkUnifiedIdeograph5a6d',
u'\u1f6f': 'GreekCapitalLetterOmegaWithDasiaAndPerispomeni',
u'\uf1f0': 'UnicodeCharacter0xf1f0',
u'\ub2f2': 'HangulSyllableDalp',
u'\u77f4': 'CjkUnifiedIdeograph77f4',
u'\u28f6': 'BraillePatternDots235678',
u'\ucc79': 'HangulSyllableCyat',
u'\u817b': 'CjkUnifiedIdeograph817b',
u'\u427d': 'CjkUnifiedIdeograph427d',
u'\u077f': 'ArabicLetterKafWithTwoDotsAbove',
u'\u7b81': 'CjkUnifiedIdeograph7b81',
u'\u9c04': 'CjkUnifiedIdeograph9c04',
u'\ub087': 'HangulSyllableGgilb',
u'\u1608': 'CanadianSyllabicsCarrierMu',
u'\u2a8b': 'LessthanAboveDoublelineEqualAboveGreaterthan',
u'\ud50a': 'HangulSyllablePeunh',
u'\ue98d': 'UnicodeCharacter0xe98d',
u'\u4f0e': 'CjkUnifiedIdeograph4f0e',
u'\u6391': 'CjkUnifiedIdeograph6391',
u'\u8414': 'CjkUnifiedIdeograph8414',
u'\ud897': 'UnicodeCharacter0xd897',
u'\u3e18': 'CjkUnifiedIdeograph3e18',
u'\u529b': 'CjkUnifiedIdeograph529b',
u'\ufd1a': 'ArabicLigatureSheenWithYehFinalForm',
u'\u771e': 'CjkUnifiedIdeograph771e',
u'\u8ba1': 'CjkUnifiedIdeograph8ba1',
u'\u0da5': 'SinhalaLetterTaalujaSanyoogaNaaksikyaya',
u'\uac24': 'HangulSyllableGael',
u'\uc0a7': 'HangulSyllableBbic',
u'\u2628': 'CrossOfLorraine',
u'\u7aab': 'CjkUnifiedIdeograph7aab',
u'\ue52a': 'UnicodeCharacter0xe52a',
u'\u9f2e': 'CjkUnifiedIdeograph9f2e',
u'\ub3b1': 'HangulSyllableDyelg',
u'\u1132': 'HangulChoseongSiospieup',
u'\u35b5': 'CjkUnifiedIdeograph35b5',
u'\ud434': 'HangulSyllablePwaem',
u'\ue8b7': 'UnicodeCharacter0xe8b7',
u'\u4e38': 'CjkUnifiedIdeograph4e38',
u'\u62bb': 'CjkUnifiedIdeograph62bb',
u'\u873e': 'CjkUnifiedIdeograph873e',
u'\udbc1': 'UnicodeCharacter0xdbc1',
u'\u3942': 'CjkUnifiedIdeograph3942',
u'\u5dc5': 'CjkUnifiedIdeograph5dc5',
u'\ufc44': 'ArabicLigatureLamWithYehIsolatedForm',
u'\u7648': 'CjkUnifiedIdeograph7648',
u'\u8acb': 'CjkUnifiedIdeograph8acb',
u'\u0ccf': 'UnicodeCharacter0xccf',
u'\uaf4e': 'HangulSyllableGgwanh',
u'\uc3d1': 'HangulSyllableSsyeng',
u'\u2152': 'UnicodeCharacter0x2152',
u'\u45d5': 'CjkUnifiedIdeograph45d5',
u'\ue454': 'UnicodeCharacter0xe454',
u'\u9e58': 'CjkUnifiedIdeograph9e58',
u'\ub2db': 'HangulSyllableNis',
u'\u105c': 'MyanmarLetterMonBba',
u'\u34df': 'CjkUnifiedIdeograph34df',
u'\ud75e': 'HangulSyllableHeulp',
u'\uebe1': 'UnicodeCharacter0xebe1',
u'\u4962': 'CjkUnifiedIdeograph4962',
u'\u6de5': 'CjkUnifiedIdeograph6de5',
u'\u8668': 'CjkUnifiedIdeograph8668',
u'\udaeb': 'UnicodeCharacter0xdaeb',
u'\u386c': 'CjkUnifiedIdeograph386c',
u'\u5cef': 'CjkUnifiedIdeograph5cef',
u'\uff6e': 'HalfwidthKatakanaLetterSmallYo',
u'\u7172': 'CjkUnifiedIdeograph7172',
u'\u95f5': 'CjkUnifiedIdeograph95f5',
u'\u0ff9': 'UnicodeCharacter0xff9',
u'\uae78': 'HangulSyllableGgaem',
u'\uc2fb': 'HangulSyllableSsags',
u'\u207c': 'SuperscriptEqualsSign',
u'\u44ff': 'CjkUnifiedIdeograph44ff',
u'\ue77e': 'UnicodeCharacter0xe77e',
u'\u7e03': 'CjkUnifiedIdeograph7e03',
u'\u9982': 'CjkUnifiedIdeograph9982',
u'\u1386': 'EthiopicSyllableBwee',
u'\ub709': 'HangulSyllableDdwit',
u'\ud688': 'HangulSyllableHwaek',
u'\u290d': 'RightwardsDoubleDashArrow',
u'\u488c': 'CjkUnifiedIdeograph488c',
u'\uec0f': 'UnicodeCharacter0xec0f',
u'\u6613': 'CjkUnifiedIdeograph6613',
u'\u8192': 'CjkUnifiedIdeograph8192',
u'\u1817': 'MongolianDigitSeven',
u'\u3b96': 'CjkUnifiedIdeograph3b96',
u'\udf19': 'UnicodeCharacter0xdf19',
u'\ufe98': 'ArabicLetterTehMedialForm',
u'\u511d': 'CjkUnifiedIdeograph511d',
u'\u709c': 'CjkUnifiedIdeograph709c',
u'\u8e23': 'CjkUnifiedIdeograph8e23',
u'\ua9a2': 'UnicodeCharacter0xa9a2',
u'\u23a6': 'RightSquareBracketLowerCorner',
u'\uc729': 'HangulSyllableYulg',
u'\ue6a8': 'UnicodeCharacter0xe6a8',
u'\u792d': 'CjkUnifiedIdeograph792d',
u'\u98ac': 'CjkUnifiedIdeograph98ac',
u'\u12b0': 'EthiopicSyllableKwa',
u'\ub633': 'HangulSyllableDdwad',
u'\ud1b2': 'HangulSyllableTobs',
u'\u2837': 'BraillePatternDots12356',
u'\u4bb6': 'CjkUnifiedIdeograph4bb6',
u'\uef39': 'UnicodeCharacter0xef39',
u'\u613d': 'CjkUnifiedIdeograph613d',
u'\u80bc': 'CjkUnifiedIdeograph80bc',
u'\u1b41': 'BalineseVowelSignTalingRepaTedung',
u'\u3ac0': 'CjkUnifiedIdeograph3ac0',
u'\ude43': 'UnicodeCharacter0xde43',
u'\uf9c2': 'CjkCompatibilityIdeographf9c2',
u'\u5047': 'CjkUnifiedIdeograph5047',
u'\u73c6': 'CjkUnifiedIdeograph73c6',
u'\u894d': 'CjkUnifiedIdeograph894d',
u'\ua8cc': 'UnicodeCharacter0xa8cc',
u'\u0351': 'CombiningLeftHalfRingAbove',
u'\u22d0': 'DoubleSubset',
u'\uc653': 'HangulSyllableWas',
u'\ue1d2': 'UnicodeCharacter0xe1d2',
u'\u7857': 'CjkUnifiedIdeograph7857',
u'\u9bd6': 'CjkUnifiedIdeograph9bd6',
u'\u1dda': 'CombiningLatinSmallLetterG',
u'\ub15d': 'HangulSyllableNyeg',
u'\ud0dc': 'HangulSyllableTae',
u'\u2b61': 'UnicodeCharacter0x2b61',
u'\u4ae0': 'CjkUnifiedIdeograph4ae0',
u'\uee63': 'UnicodeCharacter0xee63',
u'\u6067': 'CjkUnifiedIdeograph6067',
u'\u83e6': 'CjkUnifiedIdeograph83e6',
u'\u1a6b': 'UnicodeCharacter0x1a6b',
u'\u05ea': 'HebrewLetterTav',
u'\ud96d': 'UnicodeCharacter0xd96d',
u'\uf8ec': 'UnicodeCharacter0xf8ec',
u'\u5371': 'CjkUnifiedIdeograph5371',
u'\u72f0': 'CjkUnifiedIdeograph72f0',
u'\u8877': 'CjkUnifiedIdeograph8877',
u'\uabf6': 'UnicodeCharacter0xabf6',
u'\u027b': 'LatinSmallLetterTurnedRWithHook',
u'\u2dfa': 'CombiningCyrillicLetterYat',
u'\uc17d': 'HangulSyllableSyelt',
u'\ue0fc': 'UnicodeCharacter0xe0fc',
u'\ufa81': 'CjkCompatibilityIdeographfa81',
u'\u9b00': 'CjkUnifiedIdeograph9b00',
u'\ubf83': 'HangulSyllableBbwaelb',
u'\u5802': 'CjkUnifiedIdeograph5802',
u'\u7c85': 'CjkUnifiedIdeograph7c85',
u'\u1d04': 'LatinLetterSmallCapitalC',
u'\u3187': 'HangulLetterYoya',
u'\uce0e': 'HangulSyllableCyulm',
u'\ue291': 'UnicodeCharacter0xe291',
u'\u8310': 'CjkUnifiedIdeograph8310',
u'\ua793': 'UnicodeCharacter0xa793',
u'\u4012': 'CjkUnifiedIdeograph4012',
u'\u6495': 'CjkUnifiedIdeograph6495',
u'\u0514': 'CyrillicCapitalLetterLha',
u'\u5997': 'CjkUnifiedIdeograph5997',
u'\u1e99': 'LatinSmallLetterYWithRingAbove',
u'\uf61e': 'UnicodeCharacter0xf61e',
u'\uab20': 'UnicodeCharacter0xab20',
u'\ucfa3': 'HangulSyllableKwaec',
u'\u6822': 'CjkUnifiedIdeograph6822',
u'\u8ca5': 'CjkUnifiedIdeograph8ca5',
u'\u2d24': 'GeorgianSmallLetterHar',
u'\u41a7': 'CjkUnifiedIdeograph41a7',
u'\u06a9': 'ArabicLetterKeheh',
u'\ud330': 'HangulSyllablePael',
u'\uf7b3': 'UnicodeCharacter0xf7b3',
u'\u9032': 'CjkUnifiedIdeograph9032',
u'\ub4b5': 'HangulSyllableDwib',
u'\u5534': 'CjkUnifiedIdeograph5534',
u'\u69b7': 'CjkUnifiedIdeograph69b7',
u'\u0a36': 'GurmukhiLetterSha',
u'\u2eb9': 'CjkRadicalOld',
u'\ufb40': 'HebrewLetterNunWithDagesh',
u'\ub842': 'HangulSyllableRyegg',
u'\udcc5': 'UnicodeCharacter0xdcc5',
u'\u7d44': 'CjkUnifiedIdeograph7d44',
u'\u91c7': 'CjkUnifiedIdeograph91c7',
u'\u3246': 'UnicodeCharacter0x3246',
u'\u56c9': 'CjkUnifiedIdeograph56c9',
u'\u0bcb': 'TamilVowelSignOo',
u'\ue350': 'UnicodeCharacter0xe350',
u'\ua052': 'YiSyllablePy',
u'\uc4d5': 'HangulSyllableSsyug',
u'\u6554': 'CjkUnifiedIdeograph6554',
u'\ub9d7': 'HangulSyllableMalh',
u'\u5a56': 'CjkUnifiedIdeograph5a56',
u'\u7ed9': 'CjkUnifiedIdeograph7ed9',
u'\u1f58': 'UnicodeCharacter0x1f58',
u'\u33db': 'SquareSr',
u'\uc862': 'HangulSyllableJyelp',
u'\uece5': 'UnicodeCharacter0xece5',
u'\u8d64': 'CjkUnifiedIdeograph8d64',
u'\ua1e7': 'YiSyllableGuox',
u'\u4266': 'CjkUnifiedIdeograph4266',
u'\u66e9': 'CjkUnifiedIdeograph66e9',
u'\u0768': 'ArabicLetterNoonWithSmallTah',
u'\u5beb': 'CjkUnifiedIdeograph5beb',
u'\u18ed': 'UnicodeCharacter0x18ed',
u'\uf072': 'UnicodeCharacter0xf072',
u'\ub574': 'HangulSyllableDdyals',
u'\uc9f7': 'HangulSyllableJjah',
u'\u6a76': 'CjkUnifiedIdeograph6a76',
u'\u8ef9': 'CjkUnifiedIdeograph8ef9',
u'\u2f78': 'KangxiRadicalJar',
u'\u43fb': 'CjkUnifiedIdeograph43fb',
u'\xfd': 'LatinSmallLetterYWithAcute',
u'\u3a01': 'CjkUnifiedIdeograph3a01',
u'\udd84': 'UnicodeCharacter0xdd84',
u'\u9286': 'CjkUnifiedIdeograph9286',
u'\u5788': 'CjkUnifiedIdeograph5788',
u'\ueb0b': 'UnicodeCharacter0xeb0b',
u'\u148a': 'CanadianSyllabicsCaai',
u'\ua80d': 'SylotiNagriLetterCho',
u'\u6d0f': 'CjkUnifiedIdeograph6d0f',
u'\u2211': 'NarySummation',
u'\uc594': 'HangulSyllableYak',
u'\uba96': 'HangulSyllableMyelm',
u'\u7f98': 'CjkUnifiedIdeograph7f98',
u'\u3c9a': 'CjkUnifiedIdeograph3c9a',
u'\ud01d': 'HangulSyllableKwenj',
u'\u951f': 'CjkUnifiedIdeograph951f',
u'\u4a21': 'CjkUnifiedIdeograph4a21',
u'\u0f23': 'TibetanDigitThree',
u'\ueda4': 'UnicodeCharacter0xeda4',
u'\ua2a6': 'YiSyllableZurx',
u'\u67a8': 'CjkUnifiedIdeograph67a8',
u'\u24aa': 'ParenthesizedLatinSmallLetterO',
u'\uf82d': 'UnicodeCharacter0xf82d',
u'\u19ac': 'UnicodeCharacter0x19ac',
u'\ubd2f': 'HangulSyllableBwaegs',
u'\u7231': 'CjkUnifiedIdeograph7231',
u'\u3733': 'CjkUnifiedIdeograph3733',
u'\ucab6': 'HangulSyllableJjyej',
u'\u8fb8': 'CjkUnifiedIdeograph8fb8',
u'\u4cba': 'CjkUnifiedIdeograph4cba',
u'\ue03d': 'UnicodeCharacter0xe03d',
u'\u01bc': 'LatinCapitalLetterToneFive',
u'\ua53f': 'VaiSyllableCi',
u'\u9a41': 'CjkUnifiedIdeograph9a41',
u'\u5f43': 'CjkUnifiedIdeograph5f43',
u'\u1c45': 'LepchaDigitFive',
u'\uf2c6': 'UnicodeCharacter0xf2c6',
u'\ub7c8': 'HangulSyllableRyass',
u'\u74ca': 'CjkUnifiedIdeograph74ca',
u'\u29cc': 'SInTriangle',
u'\ucd4f': 'HangulSyllableCwaelh',
u'\u8251': 'CjkUnifiedIdeograph8251',
u'\u4753': 'CjkUnifiedIdeograph4753',
u'\u0455': 'CyrillicSmallLetterDze',
u'\udfd8': 'UnicodeCharacter0xdfd8',
u'\u9cda': 'CjkUnifiedIdeograph9cda',
u'\u51dc': 'CjkUnifiedIdeograph51dc',
u'\uf55f': 'UnicodeCharacter0xf55f',
u'\u16de': 'RunicLetterDagazDaegD',
u'\uaa61': 'UnicodeCharacter0xaa61',
u'\u6f63': 'CjkUnifiedIdeograph6f63',
u'\u2c65': 'LatinSmallLetterAWithStroke',
u'\uc7e8': 'HangulSyllableJyaen',
u'\u84ea': 'CjkUnifiedIdeograph84ea',
u'\u79ec': 'CjkUnifiedIdeograph79ec',
u'\u3eee': 'CjkUnifiedIdeograph3eee',
u'\ud271': 'HangulSyllableTwelt',
u'\u9773': 'CjkUnifiedIdeograph9773',
u'\u5475': 'CjkUnifiedIdeograph5475',
u'\u0977': 'UnicodeCharacter0x977',
u'\ueff8': 'UnicodeCharacter0xeff8',
u'\uacfa': 'HangulSyllableGop',
u'\u61fc': 'CjkUnifiedIdeograph61fc',
u'\u26fe': 'UnicodeCharacter0x26fe',
u'\u1c00': 'LepchaLetterKa',
u'\u3883': 'CjkUnifiedIdeograph3883',
u'\udf02': 'UnicodeCharacter0xdf02',
u'\ufb85': 'ArabicLetterDahalFinalForm',
u'\u5506': 'CjkUnifiedIdeograph5506',
u'\u7189': 'CjkUnifiedIdeograph7189',
u'\u8e0c': 'CjkUnifiedIdeograph8e0c',
u'\uaa8f': 'UnicodeCharacter0xaa8f',
u'\u0410': 'CyrillicCapitalLetterA',
u'\u2093': 'LatinSubscriptSmallLetterX',
u'\uc712': 'HangulSyllableWilp',
u'\ue395': 'UnicodeCharacter0xe395',
u'\u7d16': 'CjkUnifiedIdeograph7d16',
u'\u9999': 'CjkUnifiedIdeograph9999',
u'\u179d': 'KhmerLetterSha',
u'\ub61c': 'HangulSyllableDdols',
u'\ud29f': 'HangulSyllableTyugs',
u'\u2c20': 'GlagoliticCapitalLetterYeri',
u'\u48a3': 'CjkUnifiedIdeograph48a3',
u'\uef22': 'UnicodeCharacter0xef22',
u'\u6526': 'CjkUnifiedIdeograph6526',
u'\u81a9': 'CjkUnifiedIdeograph81a9',
u'\u3fad': 'CjkUnifiedIdeograph3fad',
u'\ude2c': 'UnicodeCharacter0xde2c',
u'\ufaaf': 'CjkCompatibilityIdeographfaaf',
u'\u5430': 'CjkUnifiedIdeograph5430',
u'\u70b3': 'CjkUnifiedIdeograph70b3',
u'\u8d36': 'CjkUnifiedIdeograph8d36',
u'\ua9b9': 'UnicodeCharacter0xa9b9',
u'\u0b3a': 'UnicodeCharacter0xb3a',
u'\u27bd': 'HeavyWedgetailedRightwardsArrow',
u'\uc63c': 'HangulSyllableOk',
u'\ue2bf': 'UnicodeCharacter0xe2bf',
u'\u7c40': 'CjkUnifiedIdeograph7c40',
u'\u98c3': 'CjkUnifiedIdeograph98c3',
u'\u16c7': 'RunicLetterIwazEoh',
u'\ub546': 'HangulSyllableDdaj',
u'\ud1c9': 'HangulSyllableTwalt',
u'\u334a': 'SquareMiribaaru',
u'\u4fcd': 'CjkUnifiedIdeograph4fcd',
u'\uee4c': 'UnicodeCharacter0xee4c',
u'\u6450': 'CjkUnifiedIdeograph6450',
u'\u80d3': 'CjkUnifiedIdeograph80d3',
u'\u3ed7': 'CjkUnifiedIdeograph3ed7',
u'\udd56': 'UnicodeCharacter0xdd56',
u'\uf9d9': 'CjkCompatibilityIdeographf9d9',
u'\u5b5a': 'CjkUnifiedIdeograph5b5a',
u'\u77dd': 'CjkUnifiedIdeograph77dd',
u'\u8c60': 'CjkUnifiedIdeograph8c60',
u'\ua8e3': 'UnicodeCharacter0xa8e3',
u'\u0a64': 'UnicodeCharacter0xa64',
u'\u26e7': 'UnicodeCharacter0x26e7',
u'\uc566': 'HangulSyllableAenh',
u'\ue1e9': 'UnicodeCharacter0xe1e9',
u'\u436a': 'CjkUnifiedIdeograph436a',
u'\u9fed': 'UnicodeCharacter0x9fed',
u'\u15f1': 'CanadianSyllabicsCarrierGe',
u'\ub470': 'HangulSyllableDweon',
u'\ud0f3': 'HangulSyllableTaec',
u'\u3274': 'CircledHangulSiosA',
u'\u4ef7': 'CjkUnifiedIdeograph4ef7',
u'\ued76': 'UnicodeCharacter0xed76',
u'\u6b7a': 'CjkUnifiedIdeograph6b7a',
u'\u87fd': 'CjkUnifiedIdeograph87fd',
u'\ubd01': 'HangulSyllableBolt',
u'\udc80': 'UnicodeCharacter0xdc80',
u'\u3b05': 'CjkUnifiedIdeograph3b05',
u'\u5a84': 'CjkUnifiedIdeograph5a84',
u'\uf607': 'UnicodeCharacter0xf607',
u'\u6c0b': 'CjkUnifiedIdeograph6c0b',
u'\u938a': 'CjkUnifiedIdeograph938a',
u'\u098e': 'UnicodeCharacter0x98e',
u'\ua511': 'VaiSyllableDhhee',
u'\uc490': 'HangulSyllableSsweom',
u'\u2315': 'TelephoneRecorder',
u'\u4294': 'CjkUnifiedIdeograph4294',
u'\u941b': 'CjkUnifiedIdeograph941b',
u'\ubb9a': 'HangulSyllableMwibs',
u'\u121f': 'EthiopicSyllableMwa',
u'\u319e': 'IdeographicAnnotationEarthMark',
u'\ucd21': 'HangulSyllableCot',
u'\ueca0': 'UnicodeCharacter0xeca0',
u'\u4b25': 'CjkUnifiedIdeograph4b25',
u'\u6aa4': 'CjkUnifiedIdeograph6aa4',
u'\ubc2b': 'HangulSyllableBac',
u'\ua3aa': 'YiSyllableNryr',
u'\u3a2f': 'CjkUnifiedIdeograph3a2f',
u'\u59ae': 'CjkUnifiedIdeograph59ae',
u'\uf531': 'UnicodeCharacter0xf531',
u'\u7335': 'CjkUnifiedIdeograph7335',
u'\u92b4': 'CjkUnifiedIdeograph92b4',
u'\u08b8': 'UnicodeCharacter0x8b8',
u'\ua43b': 'YiSyllableNjot',
u'\ucbba': 'HangulSyllableJjyugg',
u'\u223f': 'SineWave',
u'\u41be': 'CjkUnifiedIdeograph41be',
u'\u9b45': 'CjkUnifiedIdeograph9b45',
u'\ubac4': 'HangulSyllableMwa',
u'\u1149': 'HangulChoseongIeungchieuch',
u'\u30c8': 'KatakanaLetterTo',
u'\ucc4b': 'HangulSyllableCaed',
u'\uf3ca': 'UnicodeCharacter0xf3ca',
u'\u4a4f': 'CjkUnifiedIdeograph4a4f',
u'\u69ce': 'CjkUnifiedIdeograph69ce',
u'\u8355': 'CjkUnifiedIdeograph8355',
u'\ua2d4': 'YiSyllableZziet',
u'\u3959': 'CjkUnifiedIdeograph3959',
u'\u58d8': 'CjkUnifiedIdeograph58d8',
u'\uf45b': 'UnicodeCharacter0xf45b',
u'\u725f': 'CjkUnifiedIdeograph725f',
u'\u91de': 'CjkUnifiedIdeograph91de',
u'\u0fe2': 'UnicodeCharacter0xfe2',
u'\uab65': 'UnicodeCharacter0xab65',
u'\ucae4': 'HangulSyllableJjwals',
u'\u2169': 'RomanNumeralTen',
u'\u40e8': 'CjkUnifiedIdeograph40e8',
u'\u9a6f': 'CjkUnifiedIdeograph9a6f',
u'\ub9ee': 'HangulSyllableMaelm',
u'\u1073': 'MyanmarVowelSignKayahU',
u'\u37f2': 'CjkUnifiedIdeograph37f2',
u'\ud375': 'HangulSyllablePyaeng',
u'\uf2f4': 'UnicodeCharacter0xf2f4',
u'\u4979': 'CjkUnifiedIdeograph4979',
u'\u68f8': 'CjkUnifiedIdeograph68f8',
u'\u827f': 'CjkUnifiedIdeograph827f',
u'\ua1fe': 'YiSyllableKiep',
u'\ud406': 'HangulSyllablePop',
u'\uf089': 'UnicodeCharacter0xf089',
u'\u9108': 'CjkUnifiedIdeograph9108',
u'\uad8b': 'HangulSyllableGweogs',
u'\u520a': 'CjkUnifiedIdeograph520a',
u'\u6e8d': 'CjkUnifiedIdeograph6e8d',
u'\u0f0c': 'TibetanMarkDelimiterTshegBstar',
u'\u2b8f': 'UnicodeCharacter0x2b8f',
u'\ufc16': 'ArabicLigatureJeemWithMeemIsolatedForm',
u'\ub918': 'HangulSyllableRweoss',
u'\ud59b': 'HangulSyllableHyalb',
u'\u7a1a': 'CjkUnifiedIdeograph7a1a',
u'\u969d': 'CjkUnifiedIdeograph969d',
u'\u371c': 'CjkUnifiedIdeograph371c',
u'\u539f': 'CjkUnifiedIdeograph539f',
u'\u0ca1': 'KannadaLetterDda',
u'\ue426': 'UnicodeCharacter0xe426',
u'\ua128': 'YiSyllableTuop',
u'\ufdab': 'ArabicLigatureDadWithHahWithYehFinalForm',
u'\u622a': 'CjkUnifiedIdeograph622a',
u'\ubead': 'HangulSyllableBbyang',
u'\u5f2c': 'CjkUnifiedIdeograph5f2c',
u'\u7baf': 'CjkUnifiedIdeograph7baf',
u'\u182e': 'MongolianLetterMa',
u'\u34b1': 'CjkUnifiedIdeograph34b1',
u'\uc938': 'HangulSyllableJwen',
u'\ue5bb': 'UnicodeCharacter0xe5bb',
u'\u8a3a': 'CjkUnifiedIdeograph8a3a',
u'\ua6bd': 'UnicodeCharacter0xa6bd',
u'\u473c': 'CjkUnifiedIdeograph473c',
u'\u63bf': 'CjkUnifiedIdeograph63bf',
u'\u5cc1': 'CjkUnifiedIdeograph5cc1',
u'\u19c3': 'NewTaiLueLetterFinalN',
u'\uf148': 'UnicodeCharacter0xf148',
u'\ub24a': 'HangulSyllableNwelp',
u'\ucecd': 'HangulSyllableKyaenj',
u'\u6f4c': 'CjkUnifiedIdeograph6f4c',
u'\u8bcf': 'CjkUnifiedIdeograph8bcf',
u'\u284e': 'BraillePatternDots2347',
u'\u44d1': 'CjkUnifiedIdeograph44d1',
u'\u01d3': 'LatinCapitalLetterUWithCaron',
u'\uda5a': 'UnicodeCharacter0xda5a',
u'\uf6dd': 'UnicodeCharacter0xf6dd',
u'\u975c': 'CjkUnifiedIdeograph975c',
u'\ub3df': 'HangulSyllableDoh',
u'\u505e': 'CjkUnifiedIdeograph505e',
u'\u6ce1': 'CjkUnifiedIdeograph6ce1',
u'\u0d60': 'MalayalamLetterVocalicRr',
u'\u29e3': 'EqualsSignAndSlantedParallel',
u'\uc26a': 'HangulSyllableSwep',
u'\ubf6c': 'HangulSyllableBbwam',
u'\udbef': 'UnicodeCharacter0xdbef',
u'\u786e': 'CjkUnifiedIdeograph786e',
u'\u94f1': 'CjkUnifiedIdeograph94f1',
u'\u3570': 'CjkUnifiedIdeograph3570',
u'\u51f3': 'CjkUnifiedIdeograph51f3',
u'\u12f5': 'EthiopicSyllableDe',
u'\uea7a': 'UnicodeCharacter0xea7a',
u'\ua77c': 'LatinSmallLetterInsularF',
u'\uc3ff': 'HangulSyllableSswalb',
u'\u607e': 'CjkUnifiedIdeograph607e',
u'\u5d80': 'CjkUnifiedIdeograph5d80',
u'\uf903': 'CjkCompatibilityIdeographf903',
u'\u1e82': 'LatinCapitalLetterWWithAcute',
u'\uba05': 'HangulSyllableMyanj',
u'\u7707': 'CjkUnifiedIdeograph7707',
u'\u3009': 'RightAngleBracket',
u'\ucf8c': 'HangulSyllableKwae',
u'\u888e': 'CjkUnifiedIdeograph888e',
u'\u4590': 'CjkUnifiedIdeograph4590',
u'\ue113': 'UnicodeCharacter0xe113',
u'\u0692': 'ArabicLetterRehWithSmallV',
u'\ua215': 'YiSyllableGgix',
u'\u9f17': 'CjkUnifiedIdeograph9f17',
u'\u5819': 'CjkUnifiedIdeograph5819',
u'\u151b': 'CanadianSyllabicsShwii',
u'\uf79c': 'UnicodeCharacter0xf79c',
u'\ub09e': 'HangulSyllableNanh',
u'\u6da0': 'CjkUnifiedIdeograph6da0',
u'\u2ea2': 'CjkRadicalWaterTwo',
u'\uca25': 'HangulSyllableJjyab',
u'\u8727': 'CjkUnifiedIdeograph8727',
u'\u4029': 'CjkUnifiedIdeograph4029',
u'\u3d2b': 'CjkUnifiedIdeograph3d2b',
u'\ud8ae': 'UnicodeCharacter0xd8ae',
u'\u95b0': 'CjkUnifiedIdeograph95b0',
u'\u56b2': 'CjkUnifiedIdeograph56b2',
u'\uf235': 'UnicodeCharacter0xf235',
u'\u13b4': 'CherokeeLetterLe',
u'\uaf37': 'HangulSyllableGgolb',
u'\u6839': 'CjkUnifiedIdeograph6839',
u'\u253b': 'BoxDrawingsHeavyUpAndHorizontal',
u'\uc0be': 'HangulSyllableSabs',
u'\ubdc0': 'HangulSyllableBwel',
u'\u7ec2': 'CjkUnifiedIdeograph7ec2',
u'\u3bc4': 'CjkUnifiedIdeograph3bc4',
u'\ud747': 'HangulSyllableHyus',
u'\u9049': 'CjkUnifiedIdeograph9049',
u'\u4d4b': 'CjkUnifiedIdeograph4d4b',
u'\u0e4d': 'ThaiCharacterNikhahit',
u'\ue8ce': 'UnicodeCharacter0xe8ce',
u'\ua5d0': 'VaiSyllableRo',
u'\u66d2': 'CjkUnifiedIdeograph66d2',
u'\u23d4': 'MetricalLongOverTwoShorts',
u'\uff57': 'FullwidthLatinSmallLetterW',
u'\u1cd6': 'UnicodeCharacter0x1cd6',
u'\ub859': 'HangulSyllableRyet',
u'\u755b': 'CjkUnifiedIdeograph755b',
u'\u365d': 'CjkUnifiedIdeograph365d',
u'\ucde0': 'HangulSyllableCwess',
u'\u8ee2': 'CjkUnifiedIdeograph8ee2',
u'\u4be4': 'CjkUnifiedIdeograph4be4',
u'\ue767': 'UnicodeCharacter0xe767',
u'\u04e6': 'CyrillicCapitalLetterOWithDiaeresis',
u'\ua069': 'YiSyllableBbex',
u'\u9d6b': 'CjkUnifiedIdeograph9d6b',
u'\u5e6d': 'CjkUnifiedIdeograph5e6d',
u'\u1b6f': 'BalineseMusicalSymbolCombiningJegogan',
u'\uf5f0': 'UnicodeCharacter0xf5f0',
u'\ub6f2': 'HangulSyllableDdwigg',
u'\u73f4': 'CjkUnifiedIdeograph73f4',
u'\u2cf6': 'UnicodeCharacter0x2cf6',
u'\uc879': 'HangulSyllableJolg',
u'\u857b': 'CjkUnifiedIdeograph857b',
u'\u467d': 'CjkUnifiedIdeograph467d',
u'\u037f': 'UnicodeCharacter0x37f',
u'\u7f81': 'CjkUnifiedIdeograph7f81',
u'\u9804': 'CjkUnifiedIdeograph9804',
u'\ub487': 'HangulSyllableDweoh',
u'\u1208': 'EthiopicSyllableLa',
u'\u2e8b': 'CjkRadicalSeal',
u'\ud10a': 'HangulSyllableTyabs',
u'\ued8d': 'UnicodeCharacter0xed8d',
u'\u4b0e': 'CjkUnifiedIdeograph4b0e',
u'\u6791': 'CjkUnifiedIdeograph6791',
u'\u8014': 'CjkUnifiedIdeograph8014',
u'\udc97': 'UnicodeCharacter0xdc97',
u'\u3a18': 'CjkUnifiedIdeograph3a18',
u'\u569b': 'CjkUnifiedIdeograph569b',
u'\uf91a': 'CjkCompatibilityIdeographf91a',
u'\u731e': 'CjkUnifiedIdeograph731e',
u'\u8fa1': 'CjkUnifiedIdeograph8fa1',
u'\u09a5': 'BengaliLetterTha',
u'\ua824': 'SylotiNagriVowelSignI',
u'\uc4a7': 'HangulSyllableSswelb',
u'\u2228': 'LogicalOr',
u'\u7eab': 'CjkUnifiedIdeograph7eab',
u'\ue12a': 'UnicodeCharacter0xe12a',
u'\u9b2e': 'CjkUnifiedIdeograph9b2e',
u'\ub7b1': 'HangulSyllableRaet',
u'\u1532': 'CanadianSyllabicsWestcreeYwi',
u'\u31b5': 'BopomofoFinalLetterT',
u'\ud034': 'HangulSyllableKwi',
u'\uecb7': 'UnicodeCharacter0xecb7',
u'\u4a38': 'CjkUnifiedIdeograph4a38',
u'\u66bb': 'CjkUnifiedIdeograph66bb',
u'\u833e': 'CjkUnifiedIdeograph833e',
u'\udfc1': 'UnicodeCharacter0xdfc1',
u'\u3d42': 'CjkUnifiedIdeograph3d42',
u'\u59c5': 'CjkUnifiedIdeograph59c5',
u'\uf844': 'UnicodeCharacter0xf844',
u'\u7248': 'CjkUnifiedIdeograph7248',
u'\u8ecb': 'CjkUnifiedIdeograph8ecb',
u'\u08cf': 'UnicodeCharacter0x8cf',
u'\uab4e': 'UnicodeCharacter0xab4e',
u'\uc7d1': 'HangulSyllableJyalg',
u'\u2552': 'BoxDrawingsDownSingleAndRightDouble',
u'\u41d5': 'CjkUnifiedIdeograph41d5',
u'\ue054': 'UnicodeCharacter0xe054',
u'\u9a58': 'CjkUnifiedIdeograph9a58',
u'\ub6db': 'HangulSyllableDdwed',
u'\u145c': 'CanadianSyllabicsWestcreeTwii',
u'\u30df': 'KatakanaLetterMi',
u'\ud35e': 'HangulSyllablePyap',
u'\uefe1': 'UnicodeCharacter0xefe1',
u'\u4d62': 'CjkUnifiedIdeograph4d62',
u'\u69e5': 'CjkUnifiedIdeograph69e5',
u'\u8268': 'CjkUnifiedIdeograph8268',
u'\udeeb': 'UnicodeCharacter0xdeeb',
u'\u3c6c': 'CjkUnifiedIdeograph3c6c',
u'\u58ef': 'CjkUnifiedIdeograph58ef',
u'\ufb6e': 'ArabicLetterPehehIsolatedForm',
u'\u7572': 'CjkUnifiedIdeograph7572',
u'\u91f5': 'CjkUnifiedIdeograph91f5',
u'\u0bf9': 'TamilRupeeSign',
u'\uaa78': 'UnicodeCharacter0xaa78',
u'\uc6fb': 'HangulSyllableWes',
u'\u247c': 'ParenthesizedDigitNine',
u'\u40ff': 'CjkUnifiedIdeograph40ff',
u'\ue37e': 'UnicodeCharacter0xe37e',
u'\u7a03': 'CjkUnifiedIdeograph7a03',
u'\u9d82': 'CjkUnifiedIdeograph9d82',
u'\u1786': 'KhmerLetterCha',
u'\ub309': 'HangulSyllableDaelg',
u'\ud288': 'HangulSyllableTwil',
u'\u2d0d': 'GeorgianSmallLetterOn',
u'\u4c8c': 'CjkUnifiedIdeograph4c8c',
u'\ue80f': 'UnicodeCharacter0xe80f',
u'\u6213': 'CjkUnifiedIdeograph6213',
u'\u8592': 'CjkUnifiedIdeograph8592',
u'\u1c17': 'LepchaLetterTsa',
u'\u3f96': 'CjkUnifiedIdeograph3f96',
u'\udb19': 'UnicodeCharacter0xdb19',
u'\ufa98': 'CjkCompatibilityIdeographfa98',
u'\u551d': 'CjkUnifiedIdeograph551d',
u'\u749c': 'CjkUnifiedIdeograph749c',
u'\u8a23': 'CjkUnifiedIdeograph8a23',
u'\uada2': 'HangulSyllableGweop',
u'\u0427': 'CyrillicCapitalLetterChe',
u'\u27a6': 'HeavyBlackCurvedUpwardsAndRightwardsArrow',
u'\uc329': 'HangulSyllableSsaeng',
u'\ue2a8': 'UnicodeCharacter0xe2a8',
u'\u7d2d': 'CjkUnifiedIdeograph7d2d',
u'\u9cac': 'CjkUnifiedIdeograph9cac',
u'\u16b0': 'RunicLetterOn',
u'\ub233': 'HangulSyllableNweos',
u'\ud5b2': 'HangulSyllableHyaenh',
u'\u2c37': 'GlagoliticSmallLetterDzelo',
u'\u4fb6': 'CjkUnifiedIdeograph4fb6',
u'\ueb39': 'UnicodeCharacter0xeb39',
u'\u653d': 'CjkUnifiedIdeograph653d',
u'\u84bc': 'CjkUnifiedIdeograph84bc',
u'\u1f41': 'GreekSmallLetterOmicronWithDasia',
u'\u3ec0': 'CjkUnifiedIdeograph3ec0',
u'\uda43': 'UnicodeCharacter0xda43',
u'\ufdc2': 'ArabicLigatureBehWithHahWithYehFinalForm',
u'\u5447': 'CjkUnifiedIdeograph5447',
u'\u77c6': 'CjkUnifiedIdeograph77c6',
u'\u8d4d': 'CjkUnifiedIdeograph8d4d',
u'\uaccc': 'HangulSyllableGyel',
u'\u0751': 'ArabicLetterBehWithDotBelowAndThreeDotsAbove',
u'\u26d0': 'UnicodeCharacter0x26d0',
u'\uc253': 'HangulSyllableSwegs',
u'\ue5d2': 'UnicodeCharacter0xe5d2',
u'\u7c57': 'CjkUnifiedIdeograph7c57',
u'\u9fd6': 'UnicodeCharacter0x9fd6',
u'\u19da': 'UnicodeCharacter0x19da',
u'\ub55d': 'HangulSyllableDdaeb',
u'\ud4dc': 'HangulSyllablePwim',
u'\u2f61': 'KangxiRadicalTile',
u'\u4ee0': 'CjkUnifiedIdeograph4ee0',
u'\uea63': 'UnicodeCharacter0xea63',
u'\u6467': 'CjkUnifiedIdeograph6467',
u'\u87e6': 'CjkUnifiedIdeograph87e6',
u'\u1e6b': 'LatinSmallLetterTWithDotAbove',
u'\u01ea': 'LatinCapitalLetterOWithOgonek',
u'\udd6d': 'UnicodeCharacter0xdd6d',
u'\ufcec': 'ArabicLigatureKafWithMeemMedialForm',
u'\u5771': 'CjkUnifiedIdeograph5771',
u'\u76f0': 'CjkUnifiedIdeograph76f0',
u'\u8c77': 'CjkUnifiedIdeograph8c77',
u'\uaff6': 'HangulSyllableGgwenh',
u'\u067b': 'ArabicLetterBeeh',
u'\u29fa': 'DoublePlus',
u'\uc57d': 'HangulSyllableYag',
u'\ue4fc': 'UnicodeCharacter0xe4fc',
u'\ufe81': 'ArabicLetterAlefWithMaddaAboveIsolatedForm',
u'\u9f00': 'CjkUnifiedIdeograph9f00',
u'\ubb83': 'HangulSyllableMwec',
u'\u5c02': 'CjkUnifiedIdeograph5c02',
u'\u7885': 'CjkUnifiedIdeograph7885',
u'\u1904': 'LimbuLetterGha',
u'\u3587': 'CjkUnifiedIdeograph3587',
u'\uca0e': 'HangulSyllableJjaej',
u'\ue691': 'UnicodeCharacter0xe691',
u'\u8710': 'CjkUnifiedIdeograph8710',
u'\ua393': 'YiSyllableRryr',
u'\u4412': 'CjkUnifiedIdeograph4412',
u'\u6095': 'CjkUnifiedIdeograph6095',
u'\u0114': 'LatinCapitalLetterEWithBreve',
u'\u5d97': 'CjkUnifiedIdeograph5d97',
u'\u1a99': 'UnicodeCharacter0x1a99',
u'\uf21e': 'UnicodeCharacter0xf21e',
u'\uaf20': 'HangulSyllableGgyem',
u'\ucba3': 'HangulSyllableJjwid',
u'\u6c22': 'CjkUnifiedIdeograph6c22',
u'\u88a5': 'CjkUnifiedIdeograph88a5',
u'\u2924': 'NorthEastArrowWithHook',
u'\u45a7': 'CjkUnifiedIdeograph45a7',
u'\u02a9': 'LatinSmallLetterFengDigraph',
u'\ud730': 'HangulSyllableHwik',
u'\uf3b3': 'UnicodeCharacter0xf3b3',
u'\u9432': 'CjkUnifiedIdeograph9432',
u'\ub0b5': 'HangulSyllableNaeg',
u'\u5134': 'CjkUnifiedIdeograph5134',
u'\u6db7': 'CjkUnifiedIdeograph6db7',
u'\u0e36': 'ThaiCharacterSaraUe',
u'\u2ab9': 'PrecedesAboveNotAlmostEqualTo',
u'\uff40': 'FullwidthGraveAccent',
u'\ubc42': 'HangulSyllableBaebs',
u'\ud8c5': 'UnicodeCharacter0xd8c5',
u'\u7944': 'CjkUnifiedIdeograph7944',
u'\u95c7': 'CjkUnifiedIdeograph95c7',
u'\u3646': 'CjkUnifiedIdeograph3646',
u'\u52c9': 'CjkUnifiedIdeograph52c9',
u'\u0fcb': 'TibetanSymbolNorBuGsumKhyil',
u'\ue750': 'UnicodeCharacter0xe750',
u'\ua452': 'YiSyllableNyuox',
u'\uc0d5': 'HangulSyllableSaelt',
u'\u6154': 'CjkUnifiedIdeograph6154',
u'\ubdd7': 'HangulSyllableBwigs',
u'\u5e56': 'CjkUnifiedIdeograph5e56',
u'\u7ad9': 'CjkUnifiedIdeograph7ad9',
u'\u1b58': 'BalineseDigitEight',
u'\u37db': 'CjkUnifiedIdeograph37db',
u'\ucc62': 'HangulSyllableCyagg',
u'\ue8e5': 'UnicodeCharacter0xe8e5',
u'\u8964': 'CjkUnifiedIdeograph8964',
u'\ua5e7': 'VaiSyllableWen',
u'\u4666': 'CjkUnifiedIdeograph4666',
u'\u62e9': 'CjkUnifiedIdeograph62e9',
u'\u0368': 'CombiningLatinSmallLetterC',
u'\u5feb': 'CjkUnifiedIdeograph5feb',
u'\u1ced': 'UnicodeCharacter0x1ced',
u'\uf472': 'UnicodeCharacter0xf472',
u'\ub174': 'HangulSyllableNyek',
u'\ucdf7': 'HangulSyllableCwilh',
u'\u6e76': 'CjkUnifiedIdeograph6e76',
u'\u8af9': 'CjkUnifiedIdeograph8af9',
u'\u2b78': 'UnicodeCharacter0x2b78',
u'\u47fb': 'CjkUnifiedIdeograph47fb',
u'\u04fd': 'CyrillicSmallLetterHaWithHook',
u'\u3e01': 'CjkUnifiedIdeograph3e01',
u'\ud984': 'UnicodeCharacter0xd984',
u'\u9686': 'CjkUnifiedIdeograph9686',
u'\u5388': 'CjkUnifiedIdeograph5388',
u'\uef0b': 'UnicodeCharacter0xef0b',
u'\u108a': 'MyanmarSignShanTone6',
u'\uac0d': 'HangulSyllableGalt',
u'\u690f': 'CjkUnifiedIdeograph690f',
u'\u2611': 'BallotBoxWithCheck',
u'\uc194': 'HangulSyllableSol',
u'\ube96': 'HangulSyllableBbaep',
u'\u7b98': 'CjkUnifiedIdeograph7b98',
u'\u389a': 'CjkUnifiedIdeograph389a',
u'\ud41d': 'HangulSyllablePwang',
u'\u911f': 'CjkUnifiedIdeograph911f',
u'\u4e21': 'CjkUnifiedIdeograph4e21',
u'\u0b23': 'OriyaLetterNna',
u'\ue9a4': 'UnicodeCharacter0xe9a4',
u'\ua6a6': 'UnicodeCharacter0xa6a6',
u'\u63a8': 'CjkUnifiedIdeograph63a8',
u'\u20aa': 'NewSheqelSign',
u'\ufc2d': 'ArabicLigatureFehWithJeemIsolatedForm',
u'\u1dac': 'ModifierLetterSmallMWithHook',
u'\ub92f': 'HangulSyllableRwelh',
u'\u7631': 'CjkUnifiedIdeograph7631',
u'\u3333': 'SquareHuiito',
u'\uceb6': 'HangulSyllableKyalm',
u'\u8bb8': 'CjkUnifiedIdeograph8bb8',
u'\u48ba': 'CjkUnifiedIdeograph48ba',
u'\ue43d': 'UnicodeCharacter0xe43d',
u'\u05bc': 'HebrewPointDageshOrMapiq',
u'\ua13f': 'YiSyllableDda',
u'\u9e41': 'CjkUnifiedIdeograph9e41',
u'\u5b43': 'CjkUnifiedIdeograph5b43',
u'\u1845': 'MongolianLetterTodoI',
u'\uf6c6': 'UnicodeCharacter0xf6c6',
u'\ub3c8': 'HangulSyllableDon',
u'\u70ca': 'CjkUnifiedIdeograph70ca',
u'\u2dcc': 'EthiopicSyllableKyee',
u'\uc94f': 'HangulSyllableJweh',
u'\u8651': 'CjkUnifiedIdeograph8651',
u'\u4353': 'CjkUnifiedIdeograph4353',
u'\udbd8': 'UnicodeCharacter0xdbd8',
u'\u98da': 'CjkUnifiedIdeograph98da',
u'\u55dc': 'CjkUnifiedIdeograph55dc',
u'\uf15f': 'UnicodeCharacter0xf15f',
u'\u12de': 'EthiopicSyllableZo',
u'\uae61': 'HangulSyllableGgang',
u'\u6b63': 'CjkUnifiedIdeograph6b63',
u'\u2865': 'BraillePatternDots1367',
u'\uc3e8': 'HangulSyllableSsom',
u'\u80ea': 'CjkUnifiedIdeograph80ea',
u'\u7dec': 'CjkUnifiedIdeograph7dec',
u'\u3aee': 'CjkUnifiedIdeograph3aee',
u'\ud671': 'HangulSyllableHwaeg',
u'\u9373': 'CjkUnifiedIdeograph9373',
u'\u5075': 'CjkUnifiedIdeograph5075',
u'\u0d77': 'UnicodeCharacter0xd77',
u'\uebf8': 'UnicodeCharacter0xebf8',
u'\ua8fa': 'UnicodeCharacter0xa8fa',
u'\u65fc': 'CjkUnifiedIdeograph65fc',
u'\u22fe': 'SmallContainsWithOverbar',
u'\u2000': 'EnQuad',
u'\u0483': 'CombiningCyrillicTitlo',
u'\ue302': 'UnicodeCharacter0xe302',
u'\uc785': 'HangulSyllableIb',
u'\u6906': 'CjkUnifiedIdeograph6906',
u'\u4d89': 'CjkUnifiedIdeograph4d89',
u'\ub20c': 'HangulSyllableNul',
u'\u968f': 'CjkUnifiedIdeograph968f',
u'\u3810': 'CjkUnifiedIdeograph3810',
u'\u1c93': 'UnicodeCharacter0x1c93',
u'\ufb12': 'UnicodeCharacter0xfb12',
u'\udf95': 'UnicodeCharacter0xdf95',
u'\u4116': 'CjkUnifiedIdeograph4116',
u'\ua599': 'VaiSyllableWu',
u'\u2b9d': 'UnicodeCharacter0x2b9d',
u'\u8a1c': 'CjkUnifiedIdeograph8a1c',
u'\uee9f': 'UnicodeCharacter0xee9f',
u'\u1020': 'MyanmarLetterLla',
u'\u74a3': 'CjkUnifiedIdeograph74a3',
u'\ud322': 'HangulSyllablePaj',
u'\u5926': 'CjkUnifiedIdeograph5926',
u'\ubda9': 'HangulSyllableBweolt',
u'\u03ad': 'GreekSmallLetterEpsilonWithTonos',
u'\ue22c': 'UnicodeCharacter0xe22c',
u'\uc6af': 'HangulSyllableYoh',
u'\u6830': 'CjkUnifiedIdeograph6830',
u'\u4cb3': 'CjkUnifiedIdeograph4cb3',
u'\ub136': 'HangulSyllableNebs',
u'\u95b9': 'CjkUnifiedIdeograph95b9',
u'\u373a': 'CjkUnifiedIdeograph373a',
u'\u1bbd': 'UnicodeCharacter0x1bbd',
u'\ufa3c': 'CjkCompatibilityIdeographfa3c',
u'\udebf': 'UnicodeCharacter0xdebf',
u'\u4040': 'CjkUnifiedIdeograph4040',
u'\ua4c3': 'YiRadicalChe',
u'\u2ac7': 'SubsetOfAboveTildeOperator',
u'\u8946': 'CjkUnifiedIdeograph8946',
u'\uedc9': 'UnicodeCharacter0xedc9',
u'\u0f4a': 'TibetanLetterTta',
u'\u73cd': 'CjkUnifiedIdeograph73cd',
u'\ud24c': 'HangulSyllableTweon',
u'\u5850': 'CjkUnifiedIdeograph5850',
u'\ubcd3': 'HangulSyllableByeoc',
u'\u02d7': 'ModifierLetterMinusSign',
u'\ue156': 'UnicodeCharacter0xe156',
u'\uc5d9': 'HangulSyllableElg',
u'\u675a': 'CjkUnifiedIdeograph675a',
u'\u4bdd': 'CjkUnifiedIdeograph4bdd',
u'\ub060': 'HangulSyllableGgyi',
u'\u94e3': 'CjkUnifiedIdeograph94e3',
u'\u3664': 'CjkUnifiedIdeograph3664',
u'\u1ae7': 'UnicodeCharacter0x1ae7',
u'\uf966': 'CjkCompatibilityIdeographf966',
u'\udde9': 'UnicodeCharacter0xdde9',
u'\u7f6a': 'CjkUnifiedIdeograph7f6a',
u'\ua3ed': 'YiSyllableJox',
u'\u29f1': 'ErrorbarredBlackDiamond',
u'\u8870': 'CjkUnifiedIdeograph8870',
u'\uecf3': 'UnicodeCharacter0xecf3',
u'\u0e74': 'UnicodeCharacter0xe74',
u'\u72f7': 'CjkUnifiedIdeograph72f7',
u'\ud176': 'HangulSyllableTyeolp',
u'\u577a': 'CjkUnifiedIdeograph577a',
u'\ubbfd': 'HangulSyllableMinj',
u'\u8101': 'CjkUnifiedIdeograph8101',
u'\ue080': 'UnicodeCharacter0xe080',
u'\u0705': 'SyriacHorizontalColon',
u'\u6684': 'CjkUnifiedIdeograph6684',
u'\uca07': 'HangulSyllableJjaelh',
u'\u500b': 'CjkUnifiedIdeograph500b',
u'\uaf8a': 'HangulSyllableGgoelm',
u'\u358e': 'CjkUnifiedIdeograph358e',
u'\u9911': 'CjkUnifiedIdeograph9911',
u'\uf890': 'UnicodeCharacter0xf890',
u'\u1f15': 'GreekSmallLetterEpsilonWithDasiaAndOxia',
u'\u7e94': 'CjkUnifiedIdeograph7e94',
u'\ua81b': 'SylotiNagriLetterBo',
u'\u879a': 'CjkUnifiedIdeograph879a',
u'\u2e1f': 'TildeWithDotBelow',
u'\u0d9e': 'SinhalaLetterKantajaNaasikyaya',
u'\uf121': 'UnicodeCharacter0xf121',
u'\ud0a0': 'HangulSyllableKyik',
u'\u7725': 'CjkUnifiedIdeograph7725',
u'\u56a4': 'CjkUnifiedIdeograph56a4',
u'\u802b': 'CjkUnifiedIdeograph802b',
u'\u9faa': 'CjkUnifiedIdeograph9faa',
u'\u062f': 'ArabicLetterDal',
u'\u65ae': 'CjkUnifiedIdeograph65ae',
u'\uc931': 'HangulSyllableJweot',
u'\u4f35': 'CjkUnifiedIdeograph4f35',
u'\uaeb4': 'HangulSyllableGgyaess',
u'\u34b8': 'CjkUnifiedIdeograph34b8',
u'\u983b': 'CjkUnifiedIdeograph983b',
u'\uf7ba': 'UnicodeCharacter0xf7ba',
u'\u1e3f': 'LatinSmallLetterMWithAcute',
u'\u7dbe': 'CjkUnifiedIdeograph7dbe',
u'\ua745': 'LatinSmallLetterKWithStrokeAndDiagonalStroke',
u'\u86c4': 'CjkUnifiedIdeograph86c4',
u'\u2d49': 'TifinaghLetterYi',
u'\u0cc8': 'KannadaVowelSignAi',
u'\uf04b': 'UnicodeCharacter0xf04b',
u'\ucfca': 'HangulSyllableKyonh',
u'\u764f': 'CjkUnifiedIdeograph764f',
u'\u55ce': 'CjkUnifiedIdeograph55ce',
u'\ubf55': 'HangulSyllableBbong',
u'\u9ed4': 'CjkUnifiedIdeograph9ed4',
u'\u0559': 'ArmenianModifierLetterLeftHalfRing',
u'\u64d8': 'CjkUnifiedIdeograph64d8',
u'\uc85b': 'HangulSyllableJyed',
u'\u4e5f': 'CjkUnifiedIdeograph4e5f',
u'\uadde': 'HangulSyllableGyugg',
u'\u33e2': 'IdeographicTelegraphSymbolForDayThree',
u'\u9765': 'CjkUnifiedIdeograph9765',
u'\uf6e4': 'UnicodeCharacter0xf6e4',
u'\u1d69': 'GreekSubscriptSmallLetterPhi',
u'\u7ce8': 'CjkUnifiedIdeograph7ce8',
u'\ua66f': 'CombiningCyrillicVzmet',
u'\u85ee': 'CjkUnifiedIdeograph85ee',
u'\u2c73': 'LatinSmallLetterWWithHook',
u'\u0bf2': 'TamilNumberOneThousand',
u'\uef75': 'UnicodeCharacter0xef75',
u'\ucef4': 'HangulSyllableKeom',
u'\u7579': 'CjkUnifiedIdeograph7579',
u'\u54f8': 'CjkUnifiedIdeograph54f8',
u'\ube7f': 'HangulSyllableBbaegs',
u'\u9dfe': 'CjkUnifiedIdeograph9dfe',
u'\ue806': 'UnicodeCharacter0xe806',
u'\ucc89': 'HangulSyllableCyaelt',
u'\uad08': 'HangulSyllableGwals',
u'\u918b': 'CjkUnifiedIdeograph918b',
u'\u6e0a': 'CjkUnifiedIdeograph6e0a',
u'\u528d': 'CjkUnifiedIdeograph528d',
u'\u330c': 'SquareKaratto',
u'\u178f': 'KhmerLetterTa',
u'\uc016': 'HangulSyllableBbwebs',
u'\u8518': 'CjkUnifiedIdeograph8518',
u'\ue99b': 'UnicodeCharacter0xe99b',
u'\u461a': 'CjkUnifiedIdeograph461a',
u'\uaa9d': 'UnicodeCharacter0xaa9d',
u'\u0b1c': 'OriyaLetterJa',
u'\u6f9f': 'CjkUnifiedIdeograph6f9f',
u'\u30a1': 'KatakanaLetterSmallA',
u'\ud826': 'UnicodeCharacter0xd826',
u'\u9d28': 'CjkUnifiedIdeograph9d28',
u'\uc1ab': 'HangulSyllableSwags',
u'\u5e2a': 'CjkUnifiedIdeograph5e2a',
u'\u82ad': 'CjkUnifiedIdeograph82ad',
u'\u632c': 'CjkUnifiedIdeograph632c',
u'\u47af': 'CjkUnifiedIdeograph47af',
u'\u242e': 'UnicodeCharacter0x242e',
u'\u08b1': 'UnicodeCharacter0x8b1',
u'\uf538': 'UnicodeCharacter0xf538',
u'\ud9bb': 'UnicodeCharacter0xd9bb',
u'\ub63a': 'HangulSyllableDdwalp',
u'\u9abd': 'CjkUnifiedIdeograph9abd',
u'\u7b3c': 'CjkUnifiedIdeograph7b3c',
u'\u5fbf': 'CjkUnifiedIdeograph5fbf',
u'\u3c3e': 'CjkUnifiedIdeograph3c3e',
u'\u60c1': 'CjkUnifiedIdeograph60c1',
u'\u25c3': 'WhiteLeftpointingSmallTriangle',
u'\ucd48': 'HangulSyllableCwael',
u'\u8e4a': 'CjkUnifiedIdeograph8e4a',
u'\uf2cd': 'UnicodeCharacter0xf2cd',
u'\u534c': 'CjkUnifiedIdeograph534c',
u'\ub7cf': 'HangulSyllableRyah',
u'\u144e': 'CanadianSyllabicsTi',
u'\u78d1': 'CjkUnifiedIdeograph78d1',
u'\u3dd3': 'CjkUnifiedIdeograph3dd3',
u'\ue65a': 'UnicodeCharacter0xe65a',
u'\ucadd': 'HangulSyllableJjwanj',
u'\uab5c': 'UnicodeCharacter0xab5c',
u'\u8fdf': 'CjkUnifiedIdeograph8fdf',
u'\u6c5e': 'CjkUnifiedIdeograph6c5e',
u'\u50e1': 'CjkUnifiedIdeograph50e1',
u'\u3160': 'HangulLetterYu',
u'\u15e3': 'CanadianSyllabicsCarrierTto',
u'\ufe6a': 'SmallPercentSign',
u'\u836c': 'CjkUnifiedIdeograph836c',
u'\ue7ef': 'UnicodeCharacter0xe7ef',
u'\u446e': 'CjkUnifiedIdeograph446e',
u'\ua8f1': 'UnicodeCharacter0xa8f1',
u'\u0970': 'DevanagariAbbreviationSign',
u'\u6df3': 'CjkUnifiedIdeograph6df3',
u'\u2ef5': 'UnicodeCharacter0x2ef5',
u'\ud67a': 'HangulSyllableHwaelm',
u'\u9b7c': 'CjkUnifiedIdeograph9b7c',
u'\uffff': 'UnicodeCharacter0xffff',
u'\u5c7e': 'CjkUnifiedIdeograph5c7e',
u'\u6180': 'CjkUnifiedIdeograph6180',
u'\uc503': 'HangulSyllableSseus',
u'\u2282': 'SubsetOf',
u'\u8605': 'CjkUnifiedIdeograph8605',
u'\u4b07': 'CjkUnifiedIdeograph4b07',
u'\u0c09': 'TeluguLetterU',
u'\uf38c': 'UnicodeCharacter0xf38c',
u'\ub48e': 'HangulSyllableDwenh',
u'\u7990': 'CjkUnifiedIdeograph7990',
u'\udd13': 'UnicodeCharacter0xdd13',
u'\u3a92': 'CjkUnifiedIdeograph3a92',
u'\u9e15': 'CjkUnifiedIdeograph9e15',
u'\ua317': 'YiSyllableSot',
u'\u6419': 'CjkUnifiedIdeograph6419',
u'\u291b': 'LeftwardsDoubleArrowtail',
u'\ucb9c': 'HangulSyllableJjwi',
u'\u8c9e': 'CjkUnifiedIdeograph8c9e',
u'\u51a0': 'CjkUnifiedIdeograph51a0',
u'\u12a2': 'EthiopicSyllableGlottalI',
u'\uf625': 'UnicodeCharacter0xf625',
u'\ubb27': 'HangulSyllableMyolh',
u'\u7c29': 'CjkUnifiedIdeograph7c29',
u'\u012b': 'LatinSmallLetterIWithMacron',
u'\ue4ae': 'UnicodeCharacter0xe4ae',
u'\ua9b0': 'UnicodeCharacter0xa9b0',
u'\u6ab2': 'CjkUnifiedIdeograph6ab2',
u'\uce35': 'HangulSyllableCeung',
u'\u2fb4': 'KangxiRadicalLeaf',
u'\u9337': 'CjkUnifiedIdeograph9337',
u'\u5439': 'CjkUnifiedIdeograph5439',
u'\u193b': 'LimbuSignSai',
u'\ufcbe': 'ArabicLigatureFehWithJeemInitialForm',
u'\u81c0': 'CjkUnifiedIdeograph81c0',
u'\u42c2': 'CjkUnifiedIdeograph42c2',
u'\u07c4': 'NkoDigitFour',
u'\ueb47': 'UnicodeCharacter0xeb47',
u'\uac49': 'HangulSyllableGyab',
u'\u714b': 'CjkUnifiedIdeograph714b',
u'\u324d': 'UnicodeCharacter0x324d',
u'\ud4ce': 'HangulSyllablePwigg',
u'\u99d0': 'CjkUnifiedIdeograph99d0',
u'\u5ad2': 'CjkUnifiedIdeograph5ad2',
u'\u1fd4': 'UnicodeCharacter0x1fd4',
u'\uc357': 'HangulSyllableSsyaelb',
u'\u20d6': 'CombiningLeftArrowAbove',
u'\u8459': 'CjkUnifiedIdeograph8459',
u'\u495b': 'CjkUnifiedIdeograph495b',
u'\u0a5d': 'UnicodeCharacter0xa5d',
u'\uf1e0': 'UnicodeCharacter0xf1e0',
u'\ub2e2': 'HangulSyllableNip',
u'\u77e4': 'CjkUnifiedIdeograph77e4',
u'\udb67': 'UnicodeCharacter0xdb67',
u'\u38e6': 'CjkUnifiedIdeograph38e6',
u'\u9c69': 'CjkUnifiedIdeograph9c69',
u'\ua16b': 'YiSyllableHnip',
u'\u626d': 'CjkUnifiedIdeograph626d',
u'\u276f': 'HeavyRightpointingAngleQuotationMarkOrnament',
u'\uc9f0': 'HangulSyllableJjass',
u'\u8af2': 'CjkUnifiedIdeograph8af2',
u'\u4ff4': 'CjkUnifiedIdeograph4ff4',
u'\u10f6': 'GeorgianLetterFi',
u'\uf479': 'UnicodeCharacter0xf479',
u'\ub97b': 'HangulSyllableReud',
u'\u7a7d': 'CjkUnifiedIdeograph7a7d',
u'\u3f7f': 'CjkUnifiedIdeograph3f7f',
u'\u4381': 'CjkUnifiedIdeograph4381',
u'\ua404': 'YiSyllableQuot',
u'\u8887': 'CjkUnifiedIdeograph8887',
u'\u2e08': 'DottedTranspositionMarker',
u'\u128b': 'EthiopicSyllableXwaa',
u'\ued0a': 'UnicodeCharacter0xed0a',
u'\ud18d': 'HangulSyllableTyelg',
u'\u770e': 'CjkUnifiedIdeograph770e',
u'\u5b91': 'CjkUnifiedIdeograph5b91',
u'\ubc14': 'HangulSyllableBa',
u'\ue097': 'UnicodeCharacter0xe097',
u'\u0618': 'ArabicSmallFatha',
u'\u6a9b': 'CjkUnifiedIdeograph6a9b',
u'\uc51a': 'HangulSyllableSsyilp',
u'\u4f1e': 'CjkUnifiedIdeograph4f1e',
u'\ub3a1': 'HangulSyllableDyeong',
u'\u35a5': 'CjkUnifiedIdeograph35a5',
u'\u9424': 'CjkUnifiedIdeograph9424',
u'\uf8a7': 'UnicodeCharacter0xf8a7',
u'\u1e28': 'LatinCapitalLetterHWithCedilla',
u'\u42ab': 'CjkUnifiedIdeograph42ab',
u'\udd2a': 'UnicodeCharacter0xdd2a',
u'\ua72e': 'LatinCapitalLetterCuatrilloWithComma',
u'\u8bb1': 'CjkUnifiedIdeograph8bb1',
u'\u2932': 'NorthWestArrowCrossingNorthEastArrow',
u'\u0db5': 'SinhalaLetterMahaapraanaPayanna',
u'\uec34': 'UnicodeCharacter0xec34',
u'\ud0b7': 'HangulSyllableKis',
u'\u7638': 'CjkUnifiedIdeograph7638',
u'\u5abb': 'CjkUnifiedIdeograph5abb',
u'\ubf3e': 'HangulSyllableBbyep',
u'\ue3c1': 'UnicodeCharacter0xe3c1',
u'\u0142': 'LatinSmallLetterLWithStroke',
u'\u65c5': 'CjkUnifiedIdeograph65c5',
u'\uc444': 'HangulSyllableSsoek',
u'\u4e48': 'CjkUnifiedIdeograph4e48',
u'\ub2cb': 'HangulSyllableNigs',
u'\u34cf': 'CjkUnifiedIdeograph34cf',
u'\u974e': 'CjkUnifiedIdeograph974e',
u'\ufbd1': 'UnicodeCharacter0xfbd1',
u'\u1952': 'TaiLeLetterNga',
u'\u7dd5': 'CjkUnifiedIdeograph7dd5',
u'\udc54': 'UnicodeCharacter0xdc54',
u'\ua658': 'CyrillicCapitalLetterClosedLittleYus',
u'\u8adb': 'CjkUnifiedIdeograph8adb',
u'\u285c': 'BraillePatternDots3457',
u'\u0cdf': 'UnicodeCharacter0xcdf',
u'\uef5e': 'UnicodeCharacter0xef5e',
u'\ud3e1': 'HangulSyllablePyeb',
u'\u7162': 'CjkUnifiedIdeograph7162',
u'\u55e5': 'CjkUnifiedIdeograph55e5',
u'\ube68': 'HangulSyllableBbal',
u'\ue2eb': 'UnicodeCharacter0xe2eb',
u'\u64ef': 'CjkUnifiedIdeograph64ef',
u'\uc76e': 'HangulSyllableYij',
u'\u4972': 'CjkUnifiedIdeograph4972',
u'\uadf5': 'HangulSyllableGyut',
u'\u37f9': 'CjkUnifiedIdeograph37f9',
u'\u9678': 'CjkUnifiedIdeograph9678',
u'\ufafb': 'UnicodeCharacter0xfafb',
u'\u187c': 'UnicodeCharacter0x187c',
u'\u7cff': 'CjkUnifiedIdeograph7cff',
u'\udf7e': 'UnicodeCharacter0xdf7e',
u'\u4603': 'CjkUnifiedIdeograph4603',
u'\ua182': 'YiSyllableNie',
u'\u2b86': 'UnicodeCharacter0x2b86',
u'\u8f09': 'CjkUnifiedIdeograph8f09',
u'\uee88': 'UnicodeCharacter0xee88',
u'\u110d': 'HangulChoseongSsangcieuc',
u'\u708c': 'CjkUnifiedIdeograph708c',
u'\ud40f': 'HangulSyllablePwad',
u'\u5e13': 'CjkUnifiedIdeograph5e13',
u'\ub992': 'HangulSyllableRyigg',
u'\u2017': 'DoubleLowLine',
u'\u0396': 'GreekCapitalLetterZeta',
u'\ue719': 'UnicodeCharacter0xe719',
u'\uc698': 'HangulSyllableYon',
u'\u691d': 'CjkUnifiedIdeograph691d',
u'\u489c': 'CjkUnifiedIdeograph489c',
u'\ub623': 'HangulSyllableDdos',
u'\u91a2': 'CjkUnifiedIdeograph91a2',
u'\u3827': 'CjkUnifiedIdeograph3827',
u'\u1ba6': 'SundaneseVowelSignPanaelaeng',
u'\uff29': 'FullwidthLatinCapitalLetterI',
u'\udea8': 'UnicodeCharacter0xdea8',
u'\u412d': 'CjkUnifiedIdeograph412d',
u'\ua0ac': 'YiSyllableHmyrx',
u'\u2ab0': 'SucceedsAboveSinglelineEqualsSign',
u'\u8e33': 'CjkUnifiedIdeograph8e33',
u'\ue9b2': 'UnicodeCharacter0xe9b2',
u'\u1037': 'MyanmarSignDotBelow',
u'\u73b6': 'CjkUnifiedIdeograph73b6',
u'\ud739': 'HangulSyllableHyunj',
u'\u593d': 'CjkUnifiedIdeograph593d',
u'\ub8bc': 'HangulSyllableRoels',
u'\u2341': 'AplFunctionalSymbolQuadSlash',
u'\u02c0': 'ModifierLetterGlottalStop',
u'\ue643': 'UnicodeCharacter0xe643',
u'\uc1c2': 'HangulSyllableSwap',
u'\u6847': 'CjkUnifiedIdeograph6847',
u'\u4bc6': 'CjkUnifiedIdeograph4bc6',
u'\ub14d': 'HangulSyllableNyeolt',
u'\u90cc': 'CjkUnifiedIdeograph90cc',
u'\u3b51': 'CjkUnifiedIdeograph3b51',
u'\u1ad0': 'UnicodeCharacter0x1ad0',
u'\ufe53': 'UnicodeCharacter0xfe53',
u'\ud9d2': 'UnicodeCharacter0xd9d2',
u'\u4057': 'CjkUnifiedIdeograph4057',
u'\ua3d6': 'YiSyllableRu',
u'\u25da': 'UpperHalfInverseWhiteCircle',
u'\u895d': 'CjkUnifiedIdeograph895d',
u'\ue8dc': 'UnicodeCharacter0xe8dc',
u'\u1361': 'EthiopicWordspace',
u'\u72e0': 'CjkUnifiedIdeograph72e0',
u'\ud663': 'HangulSyllableHwalh',
u'\u5867': 'CjkUnifiedIdeograph5867',
u'\ubbe6': 'HangulSyllableMyilm',
u'\u226b': 'MuchGreaterthan',
u'\u3dea': 'CjkUnifiedIdeograph3dea',
u'\ue16d': 'UnicodeCharacter0xe16d',
u'\uc0ec': 'HangulSyllableSyal',
u'\u6b71': 'CjkUnifiedIdeograph6b71',
u'\u4af0': 'CjkUnifiedIdeograph4af0',
u'\ub077': 'HangulSyllableGgyic',
u'\u93f6': 'CjkUnifiedIdeograph93f6',
u'\u3a7b': 'CjkUnifiedIdeograph3a7b',
u'\u15fa': 'CanadianSyllabicsCarrierKha',
u'\uf97d': 'CjkCompatibilityIdeographf97d',
u'\ud8fc': 'UnicodeCharacter0xd8fc',
u'\uc281': 'HangulSyllableSwing',
u'\ua300': 'YiSyllableNzup',
u'\u8783': 'CjkUnifiedIdeograph8783',
u'\u6002': 'CjkUnifiedIdeograph6002',
u'\u4485': 'CjkUnifiedIdeograph4485',
u'\u2504': 'BoxDrawingsLightTripleDashHorizontal',
u'\u0987': 'BengaliLetterI',
u'\uf60e': 'UnicodeCharacter0xf60e',
u'\uda91': 'UnicodeCharacter0xda91',
u'\ubb10': 'HangulSyllableMoess',
u'\u9f93': 'CjkUnifiedIdeograph9f93',
u'\u7812': 'CjkUnifiedIdeograph7812',
u'\u5c95': 'CjkUnifiedIdeograph5c95',
u'\u3d14': 'CjkUnifiedIdeograph3d14',
u'\u6197': 'CjkUnifiedIdeograph6197',
u'\u2699': 'Gear',
u'\uce1e': 'HangulSyllableCyup',
u'\u9320': 'CjkUnifiedIdeograph9320',
u'\uf7a3': 'UnicodeCharacter0xf7a3',
u'\u5022': 'CjkUnifiedIdeograph5022',
u'\ub4a5': 'HangulSyllableDwig',
u'\u1524': 'CanadianSyllabicsWestcreeShwaa',
u'\u79a7': 'CjkUnifiedIdeograph79a7',
u'\u3ea9': 'CjkUnifiedIdeograph3ea9',
u'\ueb30': 'UnicodeCharacter0xeb30',
u'\ucfb3': 'HangulSyllableKoelb',
u'\ua832': 'UnicodeCharacter0xa832',
u'\u8cb5': 'CjkUnifiedIdeograph8cb5',
u'\u6d34': 'CjkUnifiedIdeograph6d34',
u'\u51b7': 'CjkUnifiedIdeograph51b7',
u'\u3236': 'ParenthesizedIdeographFinancial',
u'\u16b9': 'RunicLetterWunjoWynnW',
u'\uc340': 'HangulSyllableSsyam',
u'\u8042': 'CjkUnifiedIdeograph8042',
u'\ue4c5': 'UnicodeCharacter0xe4c5',
u'\u4544': 'CjkUnifiedIdeograph4544',
u'\ua9c7': 'UnicodeCharacter0xa9c7',
u'\u0a46': 'UnicodeCharacter0xa46',
u'\u6ec9': 'CjkUnifiedIdeograph6ec9',
u'\u33cb': 'SquareHp',
u'\udb50': 'UnicodeCharacter0xdb50',
u'\u9852': 'CjkUnifiedIdeograph9852',
u'\ufcd5': 'ArabicLigatureNoonWithMeemInitialForm',
u'\u5d54': 'CjkUnifiedIdeograph5d54',
u'\u81d7': 'CjkUnifiedIdeograph81d7',
u'\u6256': 'CjkUnifiedIdeograph6256',
u'\u46d9': 'CjkUnifiedIdeograph46d9',
u'\u2758': 'LightVerticalBar',
u'\u0bdb': 'UnicodeCharacter0xbdb',
u'\uf062': 'UnicodeCharacter0xf062',
u'\ud4e5': 'HangulSyllablePwit',
u'\ub564': 'HangulSyllableDdaek',
u'\u99e7': 'CjkUnifiedIdeograph99e7',
u'\u7a66': 'CjkUnifiedIdeograph7a66',
u'\u5ee9': 'CjkUnifiedIdeograph5ee9',
u'\u3f68': 'CjkUnifiedIdeograph3f68',
u'\u63eb': 'CjkUnifiedIdeograph63eb',
u'\u20ed': 'CombiningLeftwardsHarpoonWithBarbDownwards',
u'\uc872': 'HangulSyllableJogg',
u'\u8d74': 'CjkUnifiedIdeograph8d74',
u'\uf1f7': 'UnicodeCharacter0xf1f7',
u'\u5276': 'CjkUnifiedIdeograph5276',
u'\ub6f9': 'HangulSyllableDdwilg',
u'\u1778': 'UnicodeCharacter0x1778',
u'\u7bfb': 'CjkUnifiedIdeograph7bfb',
u'\u38fd': 'CjkUnifiedIdeograph38fd',
u'\u0201': 'LatinSmallLetterAWithDoubleGrave',
u'\ue584': 'UnicodeCharacter0xe584',
u'\uaa86': 'UnicodeCharacter0xaa86',
u'\u6f88': 'CjkUnifiedIdeograph6f88',
u'\ud30b': 'HangulSyllableTih',
u'\u2c8a': 'CopticCapitalLetterSou',
u'\u900d': 'CjkUnifiedIdeograph900d',
u'\u550f': 'CjkUnifiedIdeograph550f',
u'\u1a11': 'BugineseLetterRa',
u'\ufd94': 'ArabicLigatureHehWithMeemWithMeemInitialForm',
u'\u8296': 'CjkUnifiedIdeograph8296',
u'\u4798': 'CjkUnifiedIdeograph4798',
u'\u049a': 'CyrillicCapitalLetterKaWithDescender',
u'\ue81d': 'UnicodeCharacter0xe81d',
u'\uad1f': 'HangulSyllableGwaed',
u'\u7221': 'CjkUnifiedIdeograph7221',
u'\u3723': 'CjkUnifiedIdeograph3723',
u'\ud5a4': 'HangulSyllableHyass',
u'\u9aa6': 'CjkUnifiedIdeograph9aa6',
u'\u5fa8': 'CjkUnifiedIdeograph5fa8',
u'\u1caa': 'UnicodeCharacter0x1caa',
u'\uc02d': 'HangulSyllableBbwilt',
u'\u21ac': 'RightwardsArrowWithLoop',
u'\u852f': 'CjkUnifiedIdeograph852f',
u'\u4a31': 'CjkUnifiedIdeograph4a31',
u'\u0f33': 'TibetanDigitHalfZero',
u'\uf2b6': 'UnicodeCharacter0xf2b6',
u'\ub7b8': 'HangulSyllableRyan',
u'\u74ba': 'CjkUnifiedIdeograph74ba',
u'\ud83d': 'UnicodeCharacter0xd83d',
u'\u39bc': 'CjkUnifiedIdeograph39bc',
u'\u9d3f': 'CjkUnifiedIdeograph9d3f',
u'\ua241': 'YiSyllableMgux',
u'\u6743': 'CjkUnifiedIdeograph6743',
u'\u2445': 'OcrBowTie',
u'\ucac6': 'HangulSyllableJjolm',
u'\u8fc8': 'CjkUnifiedIdeograph8fc8',
u'\u4cca': 'CjkUnifiedIdeograph4cca',
u'\u11cc': 'HangulJongseongRieulkiyeoksios',
u'\uf54f': 'UnicodeCharacter0xf54f',
u'\uba51': 'HangulSyllableMeot',
u'\u7f53': 'CjkUnifiedIdeograph7f53',
u'\u3c55': 'CjkUnifiedIdeograph3c55',
u'\ue7d8': 'UnicodeCharacter0xe7d8',
u'\ua4da': 'UnicodeCharacter0xa4da',
u'\u69dc': 'CjkUnifiedIdeograph69dc',
u'\ucd5f': 'HangulSyllableCoegs',
u'\u2ede': 'CjkRadicalEatTwo',
u'\u9261': 'CjkUnifiedIdeograph9261',
u'\u5763': 'CjkUnifiedIdeograph5763',
u'\u1465': 'CanadianSyllabicsNaskapiTwaa',
u'\uffe8': 'HalfwidthFormsLightVertical',
u'\ubcea': 'HangulSyllableByebs',
u'\u41ec': 'CjkUnifiedIdeograph41ec',
u'\u06ee': 'ArabicLetterDalWithInvertedV',
u'\uea71': 'UnicodeCharacter0xea71',
u'\uaf73': 'HangulSyllableGgwaelh',
u'\u6c75': 'CjkUnifiedIdeograph6c75',
u'\u3177': 'HangulLetterPieupthieuth',
u'\ud7f8': 'UnicodeCharacter0xd7f8',
u'\u94fa': 'CjkUnifiedIdeograph94fa',
u'\u59fc': 'CjkUnifiedIdeograph59fc',
u'\u1efe': 'LatinCapitalLetterYWithLoop',
u'\u2400': 'SymbolForNull',
u'\x83': 'UnicodeCharacter0x83',
u'\ue702': 'UnicodeCharacter0xe702',
u'\uc385': 'HangulSyllableSseg',
u'\u6d06': 'CjkUnifiedIdeograph6d06',
u'\u4989': 'CjkUnifiedIdeograph4989',
u'\ub60c': 'HangulSyllableDdyek',
u'\u928f': 'CjkUnifiedIdeograph928f',
u'\u3c10': 'CjkUnifiedIdeograph3c10',
u'\u1893': 'MongolianLetterAliGaliPha',
u'\uff12': 'FullwidthDigitTwo',
u'\udb95': 'UnicodeCharacter0xdb95',
u'\u4516': 'CjkUnifiedIdeograph4516',
u'\ua199': 'YiSyllableHli',
u'\u2f9d': 'KangxiRadicalBody',
u'\u8e1c': 'CjkUnifiedIdeograph8e1c',
u'\uea9f': 'UnicodeCharacter0xea9f',
u'\u1420': 'CanadianSyllabicsFinalGrave',
u'\u70a3': 'CjkUnifiedIdeograph70a3',
u'\ud722': 'HangulSyllableHwilm',
u'\u5d26': 'CjkUnifiedIdeograph5d26',
u'\ub9a9': 'HangulSyllableRyit',
u'\u07ad': 'ThaanaEybeyfili',
u'\ue62c': 'UnicodeCharacter0xe62c',
u'\uc2af': 'HangulSyllableSeulb',
u'\u6c30': 'CjkUnifiedIdeograph6c30',
u'\u48b3': 'CjkUnifiedIdeograph48b3',
u'\ub536': 'HangulSyllableDdanh',
u'\u91b9': 'CjkUnifiedIdeograph91b9',
u'\u333a': 'SquarePensu',
u'\u1fbd': 'GreekKoronis',
u'\ufe3c': 'PresentationFormForVerticalRightBlackLenticularBracket',
u'\udabf': 'UnicodeCharacter0xdabf',
u'\u4440': 'CjkUnifiedIdeograph4440',
u'\ua0c3': 'YiSyllableMut',
u'\u2ec7': 'CjkRadicalHorn',
u'\u8d46': 'CjkUnifiedIdeograph8d46',
u'\ue9c9': 'UnicodeCharacter0xe9c9',
u'\u0b4a': 'UnicodeCharacter0xb4a',
u'\u77cd': 'CjkUnifiedIdeograph77cd',
u'\ud64c': 'HangulSyllableHoss',
u'\u5c50': 'CjkUnifiedIdeograph5c50',
u'\ub8d3': 'HangulSyllableRyod',
u'\u06d7': 'ArabicSmallHighLigatureQafWithLamWithAlefMaksura',
u'\ue556': 'UnicodeCharacter0xe556',
u'\uc1d9': 'HangulSyllableSwaeng',
u'\u635a': 'CjkUnifiedIdeograph635a',
u'\u4fdd': 'CjkUnifiedIdeograph4fdd',
u'\ub460': 'HangulSyllableDum',
u'\u90e3': 'CjkUnifiedIdeograph90e3',
u'\u3264': 'CircledHangulMieum',
u'\u1ee7': 'LatinSmallLetterUWithHookAbove',
u'\ufd66': 'ArabicLigatureSadWithMeemWithMeemFinalForm',
u'\ud9e9': 'UnicodeCharacter0xd9e9',
u'\u7b6a': 'CjkUnifiedIdeograph7b6a',
u'\ua7ed': 'UnicodeCharacter0xa7ed',
u'\u2df1': 'CombiningCyrillicLetterChe',
u'\u8c70': 'CjkUnifiedIdeograph8c70',
u'\ue8f3': 'UnicodeCharacter0xe8f3',
u'\u0a74': 'GurmukhiEkOnkar',
u'\u76f7': 'CjkUnifiedIdeograph76f7',
u'\ud576': 'HangulSyllableHaegg',
u'\u537a': 'CjkUnifiedIdeograph537a',
u'\ubffd': 'HangulSyllableBbweong',
u'\u8501': 'CjkUnifiedIdeograph8501',
u'\ue480': 'UnicodeCharacter0xe480',
u'\u0305': 'CombiningOverline',
u'\u6284': 'CjkUnifiedIdeograph6284',
u'\uce07': 'HangulSyllableCyugs',
u'\u540b': 'CjkUnifiedIdeograph540b',
u'\uab8a': 'UnicodeCharacter0xab8a',
u'\u318e': 'HangulLetterAraeae',
u'\u9d11': 'CjkUnifiedIdeograph9d11',
u'\ufc90': 'ArabicLigatureAlefMaksuraWithSuperscriptAlefFinalForm',
u'\u1b15': 'BalineseLetterGa',
u'\u7a94': 'CjkUnifiedIdeograph7a94',
u'\uac1b': 'HangulSyllableGah',
u'\u839a': 'CjkUnifiedIdeograph839a',
u'\u2a1f': 'ZNotationSchemaComposition',
u'\u099e': 'BengaliLetterNya',
u'\uf521': 'UnicodeCharacter0xf521',
u'\ud4a0': 'HangulSyllablePweols',
u'\u7325': 'CjkUnifiedIdeograph7325',
u'\u52a4': 'CjkUnifiedIdeograph52a4',
u'\u842b': 'CjkUnifiedIdeograph842b',
u'\u9baa': 'CjkUnifiedIdeograph9baa',
u'\u022f': 'LatinSmallLetterOWithDotAbove',
u'\u61ae': 'CjkUnifiedIdeograph61ae',
u'\ucd31': 'HangulSyllableCwalt',
u'\u4b35': 'CjkUnifiedIdeograph4b35',
u'\uaab4': 'UnicodeCharacter0xaab4',
u'\u30b8': 'KatakanaLetterZi',
u'\u9c3b': 'CjkUnifiedIdeograph9c3b',
u'\uf3ba': 'UnicodeCharacter0xf3ba',
u'\u1a3f': 'UnicodeCharacter0x1a3f',
u'\u79be': 'CjkUnifiedIdeograph79be',
u'\ua345': 'YiSyllableSsyr',
u'\u82c4': 'CjkUnifiedIdeograph82c4',
u'\u2949': 'UpwardsTwoheadedArrowFromSmallCircle',
u'\u08c8': 'UnicodeCharacter0x8c8',
u'\uf44b': 'UnicodeCharacter0xf44b',
u'\ucbca': 'HangulSyllableJjyubs',
u'\u724f': 'CjkUnifiedIdeograph724f',
u'\u51ce': 'CjkUnifiedIdeograph51ce',
u'\ubb55': 'HangulSyllableMweonj',
u'\u9ad4': 'CjkUnifiedIdeograph9ad4',
u'\u0159': 'LatinSmallLetterRWithCaron',
u'\u60d8': 'CjkUnifiedIdeograph60d8',
u'\ucc5b': 'HangulSyllableCaec',
u'\u4a5f': 'CjkUnifiedIdeograph4a5f',
u'\ua9de': 'UnicodeCharacter0xa9de',
u'\u37e2': 'CjkUnifiedIdeograph37e2',
u'\u9365': 'CjkUnifiedIdeograph9365',
u'\uf2e4': 'UnicodeCharacter0xf2e4',
u'\u1969': 'TaiLeLetterO',
u'\u78e8': 'CjkUnifiedIdeograph78e8',
u'\ua26f': 'YiSyllableHiex',
u'\u81ee': 'CjkUnifiedIdeograph81ee',
u'\u2873': 'BraillePatternDots12567',
u'\u0ff2': 'UnicodeCharacter0xff2',
u'\ueb75': 'UnicodeCharacter0xeb75',
u'\ucaf4': 'HangulSyllableJjwae',
u'\u7179': 'CjkUnifiedIdeograph7179',
u'\u50f8': 'CjkUnifiedIdeograph50f8',
u'\uba7f': 'HangulSyllableMyeolh',
u'\u99fe': 'CjkUnifiedIdeograph99fe',
u'\uec06': 'UnicodeCharacter0xec06',
u'\uc889': 'HangulSyllableJot',
u'\ua908': 'KayahLiDigitEight',
u'\u958b': 'CjkUnifiedIdeograph958b',
u'\u6a0a': 'CjkUnifiedIdeograph6a0a',
u'\u568d': 'CjkUnifiedIdeograph568d',
u'\u370c': 'CjkUnifiedIdeograph370c',
u'\u138f': 'EthiopicSyllablePwe',
u'\uc416': 'HangulSyllableSswaenh',
u'\u8118': 'CjkUnifiedIdeograph8118',
u'\ued9b': 'UnicodeCharacter0xed9b',
u'\u421a': 'CjkUnifiedIdeograph421a',
u'\uae9d': 'HangulSyllableGgyat',
u'\u0f1c': 'TibetanSignRdelDkarGsum',
u'\u6b9f': 'CjkUnifiedIdeograph6b9f',
u'\u34a1': 'CjkUnifiedIdeograph34a1',
u'\udc26': 'UnicodeCharacter0xdc26',
u'\u9928': 'CjkUnifiedIdeograph9928',
u'\uc5ab': 'HangulSyllableYaes',
u'\u5a2a': 'CjkUnifiedIdeograph5a2a',
u'\u86ad': 'CjkUnifiedIdeograph86ad',
u'\u672c': 'CjkUnifiedIdeograph672c',
u'\u43af': 'CjkUnifiedIdeograph43af',
u'\u202e': 'RighttoleftOverride',
u'\u0cb1': 'KannadaLetterRra',
u'\uf138': 'UnicodeCharacter0xf138',
u'\uddbb': 'UnicodeCharacter0xddbb',
u'\ub23a': 'HangulSyllableNweop',
u'\u9ebd': 'CjkUnifiedIdeograph9ebd',
u'\u7f3c': 'CjkUnifiedIdeograph7f3c',
u'\u5bbf': 'CjkUnifiedIdeograph5bbf',
u'\u383e': 'CjkUnifiedIdeograph383e',
u'\u64c1': 'CjkUnifiedIdeograph64c1',
u'\u21c3': 'DownwardsHarpoonWithBarbLeftwards',
u'\uc948': 'HangulSyllableJwess',
u'\u8a4a': 'CjkUnifiedIdeograph8a4a',
u'\uf6cd': 'UnicodeCharacter0xf6cd',
u'\u574c': 'CjkUnifiedIdeograph574c',
u'\ub3cf': 'HangulSyllableDolb',
u'\u104e': 'MyanmarSymbolAforementioned',
u'\u7cd1': 'CjkUnifiedIdeograph7cd1',
u'\u39d3': 'CjkUnifiedIdeograph39d3',
u'\ue25a': 'UnicodeCharacter0xe25a',
u'\ucedd': 'HangulSyllableKyaeng',
u'\uaf5c': 'HangulSyllableGgwass',
u'\u8bdf': 'CjkUnifiedIdeograph8bdf',
u'\u685e': 'CjkUnifiedIdeograph685e',
u'\u54e1': 'CjkUnifiedIdeograph54e1',
u'\u3560': 'CjkUnifiedIdeograph3560',
u'\u11e3': 'HangulJongseongPieuprieul',
u'\ufa6a': 'CjkCompatibilityIdeographfa6a',
u'\u876c': 'CjkUnifiedIdeograph876c',
u'\ue3ef': 'UnicodeCharacter0xe3ef',
u'\u406e': 'CjkUnifiedIdeograph406e',
u'\uacf1': 'HangulSyllableGob',
u'\u0d70': 'MalayalamNumberTen',
u'\u69f3': 'CjkUnifiedIdeograph69f3',
u'\u2af5': 'TripleVerticalBarWithHorizontalStroke',
u'\ud27a': 'HangulSyllableTwej',
u'\u9f7c': 'CjkUnifiedIdeograph9f7c',
u'\ufbff': 'ArabicLetterFarsiYehMedialForm',
u'\u587e': 'CjkUnifiedIdeograph587e',
u'\u6580': 'CjkUnifiedIdeograph6580',
u'\uc103': 'HangulSyllableSyaegs',
u'\u2682': 'DieFace3',
u'\u8205': 'CjkUnifiedIdeograph8205',
u'\u4f07': 'CjkUnifiedIdeograph4f07',
u'\u0809': 'UnicodeCharacter0x809',
u'\uf78c': 'UnicodeCharacter0xf78c',
u'\ub08e': 'HangulSyllableGgibs',
u'\u7d90': 'CjkUnifiedIdeograph7d90',
u'\ud913': 'UnicodeCharacter0xd913',
u'\u3e92': 'CjkUnifiedIdeograph3e92',
u'\u9a15': 'CjkUnifiedIdeograph9a15',
u'\ua717': 'ModifierLetterDotVerticalBar',
u'\u6019': 'CjkUnifiedIdeograph6019',
u'\u2d1b': 'GeorgianSmallLetterJil',
u'\ucf9c': 'HangulSyllableKwaem',
u'\u889e': 'CjkUnifiedIdeograph889e',
u'\u55a0': 'CjkUnifiedIdeograph55a0',
u'\u16a2': 'RunicLetterUruzUrU',
u'\uf225': 'UnicodeCharacter0xf225',
u'\ubf27': 'HangulSyllableBbyegs',
u'\u7829': 'CjkUnifiedIdeograph7829',
u'\u052b': 'UnicodeCharacter0x52b',
u'\ue0ae': 'UnicodeCharacter0xe0ae',
u'\uadb0': 'HangulSyllableGwels',
u'\u6eb2': 'CjkUnifiedIdeograph6eb2',
u'\uca35': 'HangulSyllableJjyaenj',
u'\u2bb4': 'UnicodeCharacter0x2bb4',
u'\u9737': 'CjkUnifiedIdeograph9737',
u'\u5039': 'CjkUnifiedIdeograph5039',
u'\u1d3b': 'ModifierLetterCapitalReversedN',
u'\uf8be': 'UnicodeCharacter0xf8be',
u'\u85c0': 'CjkUnifiedIdeograph85c0',
u'\u46c2': 'CjkUnifiedIdeograph46c2',
u'\u03c4': 'GreekSmallLetterTau',
u'\uef47': 'UnicodeCharacter0xef47',
u'\ua849': 'PhagspaLetterTha',
u'\u754b': 'CjkUnifiedIdeograph754b',
u'\u364d': 'CjkUnifiedIdeograph364d',
u'\ud0ce': 'HangulSyllableTalp',
u'\u9dd0': 'CjkUnifiedIdeograph9dd0',
u'\u5ed2': 'CjkUnifiedIdeograph5ed2',
u'\u1bd4': 'UnicodeCharacter0x1bd4',
u'\uc757': 'HangulSyllableEuh',
u'\u24d6': 'CircledLatinSmallLetterG',
u'\u8059': 'CjkUnifiedIdeograph8059',
u'\u4d5b': 'CjkUnifiedIdeograph4d5b',
u'\u0e5d': 'UnicodeCharacter0xe5d',
u'\uf5e0': 'UnicodeCharacter0xf5e0',
u'\ub6e2': 'HangulSyllableDdwelp',
u'\u73e4': 'CjkUnifiedIdeograph73e4',
u'\udf67': 'UnicodeCharacter0xdf67',
u'\u3ce6': 'CjkUnifiedIdeograph3ce6',
u'\u9869': 'CjkUnifiedIdeograph9869',
u'\ua56b': 'VaiSyllableKan',
u'\u666d': 'CjkUnifiedIdeograph666d',
u'\u236f': 'AplFunctionalSymbolQuadNotEqual',
u'\ucdf0': 'HangulSyllableCwil',
u'\u8ef2': 'CjkUnifiedIdeograph8ef2',
u'\u4bf4': 'CjkUnifiedIdeograph4bf4',
u'\u14f6': 'CanadianSyllabicsSwe',
u'\uf079': 'UnicodeCharacter0xf079',
u'\ubd7b': 'HangulSyllableByoc',
u'\u7e7d': 'CjkUnifiedIdeograph7e7d',
u'\u3b7f': 'CjkUnifiedIdeograph3b7f',
u'\u4781': 'CjkUnifiedIdeograph4781',
u'\ua004': 'YiSyllableIet',
u'\u8c87': 'CjkUnifiedIdeograph8c87',
u'\u2a08': 'TwoLogicalOrOperator',
u'\u168b': 'OghamLetterMuin',
u'\ue90a': 'UnicodeCharacter0xe90a',
u'\ud58d': 'HangulSyllableHaet',
u'\u730e': 'CjkUnifiedIdeograph730e',
u'\u5f91': 'CjkUnifiedIdeograph5f91',
u'\ub814': 'HangulSyllableRels',
u'\ue497': 'UnicodeCharacter0xe497',
u'\u0218': 'LatinCapitalLetterSWithCommaBelow',
u'\u6e9b': 'CjkUnifiedIdeograph6e9b',
u'\uc11a': 'HangulSyllableSyaep',
u'\u4b1e': 'CjkUnifiedIdeograph4b1e',
u'\ub7a1': 'HangulSyllableRaelg',
u'\u31a5': 'BopomofoLetterEnn',
u'\u9024': 'CjkUnifiedIdeograph9024',
u'\ufca7': 'ArabicLigatureJeemWithHahInitialForm',
u'\u1a28': 'UnicodeCharacter0x1a28',
u'\u46ab': 'CjkUnifiedIdeograph46ab',
u'\ud92a': 'UnicodeCharacter0xd92a',
u'\ua32e': 'YiSyllableSsiex',
u'\u8fb1': 'CjkUnifiedIdeograph8fb1',
u'\u2d32': 'TifinaghLetterYabh',
u'\u09b5': 'UnicodeCharacter0x9b5',
u'\ue834': 'UnicodeCharacter0xe834',
u'\ud4b7': 'HangulSyllablePwed',
u'\u7238': 'CjkUnifiedIdeograph7238',
u'\u5ebb': 'CjkUnifiedIdeograph5ebb',
u'\ubb3e': 'HangulSyllableMulm',
u'\ue7c1': 'UnicodeCharacter0xe7c1',
u'\u0542': 'ArmenianCapitalLetterGhad',
u'\u61c5': 'CjkUnifiedIdeograph61c5',
u'\uc044': 'HangulSyllableBbyul',
u'\u4a48': 'CjkUnifiedIdeograph4a48',
u'\ub6cb': 'HangulSyllableDdweos',
u'\u30cf': 'KatakanaLetterHa',
u'\u934e': 'CjkUnifiedIdeograph934e',
u'\uffd1': 'UnicodeCharacter0xffd1',
u'\u1d52': 'ModifierLetterSmallO',
u'\u79d5': 'CjkUnifiedIdeograph79d5',
u'\ud854': 'UnicodeCharacter0xd854',
u'\ua258': 'YiSyllableHxo',
u'\u8edb': 'CjkUnifiedIdeograph8edb',
u'\u2c5c': 'GlagoliticSmallLetterShtapic',
u'\u08df': 'UnicodeCharacter0x8df',
u'\ueb5e': 'UnicodeCharacter0xeb5e',
u'\ud7e1': 'UnicodeCharacter0xd7e1',
u'\u7562': 'CjkUnifiedIdeograph7562',
u'\u51e5': 'CjkUnifiedIdeograph51e5',
u'\uba68': 'HangulSyllableMess',
u'\ue6eb': 'UnicodeCharacter0xe6eb',
u'\u046c': 'CyrillicCapitalLetterIotifiedBigYus',
u'\u60ef': 'CjkUnifiedIdeograph60ef',
u'\uc36e': 'HangulSyllableSseonh',
u'\u4d72': 'CjkUnifiedIdeograph4d72',
u'\ua9f5': 'UnicodeCharacter0xa9f5',
u'\u33f9': 'IdeographicTelegraphSymbolForDayTwentysix',
u'\u9278': 'CjkUnifiedIdeograph9278',
u'\ufefb': 'ArabicLigatureLamWithAlefIsolatedForm',
u'\u1c7c': 'OlChikiPhaarkaa',
u'\u78ff': 'CjkUnifiedIdeograph78ff',
u'\udb7e': 'UnicodeCharacter0xdb7e',
u'\u4203': 'CjkUnifiedIdeograph4203',
u'\ua582': 'VaiSyllableDhhoo',
u'\u2f86': 'KangxiRadicalTongue',
u'\u8b09': 'CjkUnifiedIdeograph8b09',
u'\uea88': 'UnicodeCharacter0xea88',
u'\u150d': 'CanadianSyllabicsNaskapiStwa',
u'\u748c': 'CjkUnifiedIdeograph748c',
u'\ud00f': 'HangulSyllableKweos',
u'\u5a13': 'CjkUnifiedIdeograph5a13',
u'\ubd92': 'HangulSyllableBubs',
u'\u2417': 'SymbolForEndOfTransmissionBlock',
u'\u0796': 'ThaanaLetterJaviyani',
u'\ue319': 'UnicodeCharacter0xe319',
u'\uc298': 'HangulSyllableSyum',
u'\u6d1d': 'CjkUnifiedIdeograph6d1d',
u'\u4c9c': 'CjkUnifiedIdeograph4c9c',
u'\ub223': 'HangulSyllableNweogs',
u'\u95a2': 'CjkUnifiedIdeograph95a2',
u'\u3c27': 'CjkUnifiedIdeograph3c27',
u'\u1fa6': 'GreekSmallLetterOmegaWithPsiliAndPerispomeniAndYpogegrammeni',
u'\ufb29': 'HebrewLetterAlternativePlusSign',
u'\udaa8': 'UnicodeCharacter0xdaa8',
u'\u452d': 'CjkUnifiedIdeograph452d',
u'\ua4ac': 'YiRadicalPyt',
u'\u2eb0': 'CjkRadicalCsimplifiedSilk',
u'\u8a33': 'CjkUnifiedIdeograph8a33',
u'\uedb2': 'UnicodeCharacter0xedb2',
u'\u1437': 'CanadianSyllabicsCarrierHi',
u'\u77b6': 'CjkUnifiedIdeograph77b6',
u'\ud339': 'HangulSyllablePaeb',
u'\u5d3d': 'CjkUnifiedIdeograph5d3d',
u'\ubcbc': 'HangulSyllableByeo',
u'\u2741': 'EightPetalledOutlinedBlackFlorette',
u'\u06c0': 'ArabicLetterHehWithYehAbove',
u'\ue243': 'UnicodeCharacter0xe243',
u'\uc5c2': 'HangulSyllableEolp',
u'\u6c47': 'CjkUnifiedIdeograph6c47',
u'\u4fc6': 'CjkUnifiedIdeograph4fc6',
u'\ub54d': 'HangulSyllableDdaeg',
u'\u94cc': 'CjkUnifiedIdeograph94cc',
u'\u3f51': 'CjkUnifiedIdeograph3f51',
u'\u1ed0': 'LatinCapitalLetterOWithCircumflexAndAcute',
u'\ufa53': 'CjkCompatibilityIdeographfa53',
u'\uddd2': 'UnicodeCharacter0xddd2',
u'\u4457': 'CjkUnifiedIdeograph4457',
u'\ua7d6': 'UnicodeCharacter0xa7d6',
u'\u21da': 'LeftwardsTripleArrow',
u'\u8d5d': 'CjkUnifiedIdeograph8d5d',
u'\uecdc': 'UnicodeCharacter0xecdc',
u'\u1761': 'TagbanwaLetterI',
u'\u76e0': 'CjkUnifiedIdeograph76e0',
u'\ud263': 'HangulSyllableTweoh',
u'\u5c67': 'CjkUnifiedIdeograph5c67',
u'\ubfe6': 'HangulSyllableBbup',
u'\u266b': 'BeamedEighthNotes',
u'\u39ea': 'CjkUnifiedIdeograph39ea',
u'\ue56d': 'UnicodeCharacter0xe56d',
u'\uc4ec': 'HangulSyllableSsyuk',
u'\u6f71': 'CjkUnifiedIdeograph6f71',
u'\u4ef0': 'CjkUnifiedIdeograph4ef0',
u'\ub477': 'HangulSyllableDweolb',
u'\u97f6': 'CjkUnifiedIdeograph97f6',
u'\u3e7b': 'CjkUnifiedIdeograph3e7b',
u'\u11fa': 'UnicodeCharacter0x11fa',
u'\ufd7d': 'ArabicLigatureFehWithKhahWithMeemInitialForm',
u'\udcfc': 'UnicodeCharacter0xdcfc',
u'\uc681': 'HangulSyllableOelg',
u'\ua700': 'ModifierLetterChineseToneYinPing',
u'\u8383': 'CjkUnifiedIdeograph8383',
u'\u6402': 'CjkUnifiedIdeograph6402',
u'\u4085': 'CjkUnifiedIdeograph4085',
u'\u2104': 'CentreLineSymbol',
u'\u0d87': 'SinhalaLetterAeyanna',
u'\uf20e': 'UnicodeCharacter0xf20e',
u'\ude91': 'UnicodeCharacter0xde91',
u'\ubf10': 'HangulSyllableBbyeol',
u'\u9b93': 'CjkUnifiedIdeograph9b93',
u'\u7c12': 'CjkUnifiedIdeograph7c12',
u'\u5895': 'CjkUnifiedIdeograph5895',
u'\u3914': 'CjkUnifiedIdeograph3914',
u'\u6597': 'CjkUnifiedIdeograph6597',
u'\u2299': 'CircledDotOperator',
u'\uca1e': 'HangulSyllableJjyalm',
u'\u9720': 'CjkUnifiedIdeograph9720',
u'\uf3a3': 'UnicodeCharacter0xf3a3',
u'\u5422': 'CjkUnifiedIdeograph5422',
u'\ub0a5': 'HangulSyllableNalt',
u'\u1124': 'HangulChoseongPieupsiospieup',
u'\u7da7': 'CjkUnifiedIdeograph7da7',
u'\u3aa9': 'CjkUnifiedIdeograph3aa9',
u'\uef30': 'UnicodeCharacter0xef30',
u'\ucbb3': 'HangulSyllableJjwic',
u'\uac32': 'HangulSyllableGaej',
u'\u88b5': 'CjkUnifiedIdeograph88b5',
u'\u6934': 'CjkUnifiedIdeograph6934',
u'\u55b7': 'CjkUnifiedIdeograph55b7',
u'\u3636': 'CjkUnifiedIdeograph3636',
u'\u12b9': 'EthiopicSyllableKxu',
u'\uc740': 'HangulSyllableEun',
u'\u8442': 'CjkUnifiedIdeograph8442',
u'\ue0c5': 'UnicodeCharacter0xe0c5',
u'\u4144': 'CjkUnifiedIdeograph4144',
u'\uadc7': 'HangulSyllableGwid',
u'\u0e46': 'ThaiCharacterMaiyamok',
u'\u6ac9': 'CjkUnifiedIdeograph6ac9',
u'\u37cb': 'CjkUnifiedIdeograph37cb',
u'\udf50': 'UnicodeCharacter0xdf50',
u'\u9c52': 'CjkUnifiedIdeograph9c52',
u'\uf8d5': 'UnicodeCharacter0xf8d5',
u'\u5954': 'CjkUnifiedIdeograph5954',
u'\u85d7': 'CjkUnifiedIdeograph85d7',
u'\u6656': 'CjkUnifiedIdeograph6656',
u'\u42d9': 'CjkUnifiedIdeograph42d9',
u'\u2358': 'AplFunctionalSymbolQuoteUnderbar',
u'\u0fdb': 'UnicodeCharacter0xfdb',
u'\uf462': 'UnicodeCharacter0xf462',
u'\ud0e5': 'HangulSyllableTaelg',
u'\ub164': 'HangulSyllableNyel',
u'\u9de7': 'CjkUnifiedIdeograph9de7',
u'\u7e66': 'CjkUnifiedIdeograph7e66',
u'\u5ae9': 'CjkUnifiedIdeograph5ae9',
u'\u3b68': 'CjkUnifiedIdeograph3b68',
u'\u67eb': 'CjkUnifiedIdeograph67eb',
u'\u24ed': 'NegativeCircledNumberThirteen',
u'\ucc72': 'HangulSyllableCyabs',
u'\u8974': 'CjkUnifiedIdeograph8974',
u'\uf5f7': 'UnicodeCharacter0xf5f7',
u'\u5676': 'CjkUnifiedIdeograph5676',
u'\ub2f9': 'HangulSyllableDang',
u'\u1378': 'EthiopicNumberSeventy',
u'\u7ffb': 'CjkUnifiedIdeograph7ffb',
u'\u3cfd': 'CjkUnifiedIdeograph3cfd',
u'\u0601': 'ArabicSignSanah',
u'\ue184': 'UnicodeCharacter0xe184',
u'\uae86': 'HangulSyllableGgyagg',
u'\u6b88': 'CjkUnifiedIdeograph6b88',
u'\ud70b': 'HangulSyllableHwelh',
u'\u288a': 'BraillePatternDots248',
u'\u940d': 'CjkUnifiedIdeograph940d',
u'\u510f': 'CjkUnifiedIdeograph510f',
u'\u1e11': 'LatinSmallLetterDWithCedilla',
u'\uf994': 'CjkCompatibilityIdeographf994',
u'\u8696': 'CjkUnifiedIdeograph8696',
u'\u4398': 'CjkUnifiedIdeograph4398',
u'\x9a': 'UnicodeCharacter0x9a',
u'\uec1d': 'UnicodeCharacter0xec1d',
u'\ua91f': 'KayahLiLetterHa',
u'\u7621': 'CjkUnifiedIdeograph7621',
u'\u3323': 'SquareSento',
u'\ud1a4': 'HangulSyllableTon',
u'\u9ea6': 'CjkUnifiedIdeograph9ea6',
u'\u5ba8': 'CjkUnifiedIdeograph5ba8',
u'\u18aa': 'MongolianLetterManchuAliGaliLha',
u'\uc42d': 'HangulSyllableSsoeg',
u'\u25ac': 'BlackRectangle',
u'\u812f': 'CjkUnifiedIdeograph812f',
u'\u4e31': 'CjkUnifiedIdeograph4e31',
u'\u0b33': 'OriyaLetterLla',
u'\uf6b6': 'UnicodeCharacter0xf6b6',
u'\ub3b8': 'HangulSyllableDyem',
u'\u70ba': 'CjkUnifiedIdeograph70ba',
u'\udc3d': 'UnicodeCharacter0xdc3d',
u'\u3dbc': 'CjkUnifiedIdeograph3dbc',
u'\u993f': 'CjkUnifiedIdeograph993f',
u'\ua641': 'CyrillicSmallLetterZemlya',
u'\u6343': 'CjkUnifiedIdeograph6343',
u'\u2045': 'LeftSquareBracketWithQuill',
u'\ucec6': 'HangulSyllableKyap',
u'\u8bc8': 'CjkUnifiedIdeograph8bc8',
u'\u48ca': 'CjkUnifiedIdeograph48ca',
u'\u15cc': 'CanadianSyllabicsCarrierRe',
u'\uf14f': 'UnicodeCharacter0xf14f',
u'\ube51': 'HangulSyllableBilt',
u'\u7b53': 'CjkUnifiedIdeograph7b53',
u'\u3855': 'CjkUnifiedIdeograph3855',
u'\ue3d8': 'UnicodeCharacter0xe3d8',
u'\ua0da': 'YiSyllableFu',
u'\u6ddc': 'CjkUnifiedIdeograph6ddc',
u'\uc95f': 'HangulSyllableJwilh',
u'\u2ade': 'ShortLeftTack',
u'\u9661': 'CjkUnifiedIdeograph9661',
u'\u5363': 'CjkUnifiedIdeograph5363',
u'\u1065': 'MyanmarLetterWesternPwoKarenTha',
u'\ufbe8': 'ArabicLetterUighurKazakhKirghizAlefMaksuraInitialForm',
u'\ub8ea': 'HangulSyllableRugg',
u'\u45ec': 'CjkUnifiedIdeograph45ec',
u'\u02ee': 'ModifierLetterDoubleApostrophe',
u'\uee71': 'UnicodeCharacter0xee71',
u'\uab73': 'UnicodeCharacter0xab73',
u'\u6875': 'CjkUnifiedIdeograph6875',
u'\u3577': 'CjkUnifiedIdeograph3577',
u'\ud3f8': 'HangulSyllablePols',
u'\u90fa': 'CjkUnifiedIdeograph90fa',
u'\u5dfc': 'CjkUnifiedIdeograph5dfc',
u'\u1afe': 'UnicodeCharacter0x1afe',
u'\u2800': 'BraillePatternBlank',
u'\u0c83': 'KannadaSignVisarga',
u'\ueb02': 'UnicodeCharacter0xeb02',
u'\ucf85': 'HangulSyllableKwang',
u'\u6106': 'CjkUnifiedIdeograph6106',
u'\u4589': 'CjkUnifiedIdeograph4589',
u'\uba0c': 'HangulSyllableMyals',
u'\u9e8f': 'CjkUnifiedIdeograph9e8f',
u'\u3010': 'LeftBlackLenticularBracket',
u'\u1493': 'CanadianSyllabicsWestcreeCwe',
u'\uf312': 'UnicodeCharacter0xf312',
u'\ud795': 'HangulSyllableHilt',
u'\u4916': 'CjkUnifiedIdeograph4916',
u'\uad99': 'HangulSyllableGweob',
u'\u239d': 'LeftParenthesisLowerHook',
u'\u821c': 'CjkUnifiedIdeograph821c',
u'\ue69f': 'UnicodeCharacter0xe69f',
u'\u1820': 'MongolianLetterA',
u'\u7ca3': 'CjkUnifiedIdeograph7ca3',
u'\udb22': 'UnicodeCharacter0xdb22',
u'\u5126': 'CjkUnifiedIdeograph5126',
u'\ub5a9': 'HangulSyllableDdeolg',
u'\u0bad': 'UnicodeCharacter0xbad',
u'\uea2c': 'UnicodeCharacter0xea2c',
u'\uceaf': 'HangulSyllableKyags',
u'\u6030': 'CjkUnifiedIdeograph6030',
u'\u44b3': 'CjkUnifiedIdeograph44b3',
u'\ub936': 'HangulSyllableRwej',
u'\u9db9': 'CjkUnifiedIdeograph9db9',
u'\u3f3a': 'CjkUnifiedIdeograph3f3a',
u'\u13bd': 'CherokeeLetterMu',
u'\uf23c': 'UnicodeCharacter0xf23c',
u'\ud6bf': 'HangulSyllableHyoc',
u'\u4840': 'CjkUnifiedIdeograph4840',
u'\uacc3': 'HangulSyllableGyeoh',
u'\u22c7': 'DivisionTimes',
u'\u8146': 'CjkUnifiedIdeograph8146',
u'\ue5c9': 'UnicodeCharacter0xe5c9',
u'\u074a': 'SyriacBarrekh',
u'\u7bcd': 'CjkUnifiedIdeograph7bcd',
u'\uda4c': 'UnicodeCharacter0xda4c',
u'\u5050': 'CjkUnifiedIdeograph5050',
u'\ub4d3': 'HangulSyllableDyus',
u'\u0ad7': 'UnicodeCharacter0xad7',
u'\ue956': 'UnicodeCharacter0xe956',
u'\ucdd9': 'HangulSyllableCwelt',
u'\u6f5a': 'CjkUnifiedIdeograph6f5a',
u'\u43dd': 'CjkUnifiedIdeograph43dd',
u'\ub860': 'HangulSyllableRon',
u'\u9ce3': 'CjkUnifiedIdeograph9ce3',
u'\u3e64': 'CjkUnifiedIdeograph3e64',
u'\u12e7': 'EthiopicSyllableZhwa',
u'\uf166': 'UnicodeCharacter0xf166',
u'\ud5e9': 'HangulSyllableHenj',
u'\u776a': 'CjkUnifiedIdeograph776a',
u'\uabed': 'UnicodeCharacter0xabed',
u'\u21f1': 'NorthWestArrowToCorner',
u'\u8070': 'CjkUnifiedIdeograph8070',
u'\ue4f3': 'UnicodeCharacter0xe4f3',
u'\u0674': 'ArabicLetterHighHamza',
u'\u7af7': 'CjkUnifiedIdeograph7af7',
u'\ud976': 'UnicodeCharacter0xd976',
u'\u5f7a': 'CjkUnifiedIdeograph5f7a',
u'\ub3fd': 'HangulSyllableDwaeg',
u'\u8901': 'CjkUnifiedIdeograph8901',
u'\ue880': 'UnicodeCharacter0xe880',
u'\u0f05': 'TibetanMarkClosingYigMgoSgabMa',
u'\u6e84': 'CjkUnifiedIdeograph6e84',
u'\uc207': 'HangulSyllableSyolb',
u'\u580b': 'CjkUnifiedIdeograph580b',
u'\ua78a': 'ModifierLetterShortEqualsSign',
u'\u3d8e': 'CjkUnifiedIdeograph3d8e',
u'\u9111': 'CjkUnifiedIdeograph9111',
u'\uf090': 'UnicodeCharacter0xf090',
u'\u1715': 'UnicodeCharacter0x1715',
u'\u7694': 'CjkUnifiedIdeograph7694',
u'\ua01b': 'YiSyllableBiex',
u'\u8f9a': 'CjkUnifiedIdeograph8f9a',
u'\u261f': 'WhiteDownPointingIndex',
u'\u059e': 'HebrewAccentGershayim',
u'\uf921': 'CjkCompatibilityIdeographf921',
u'\ud8a0': 'UnicodeCharacter0xd8a0',
u'\u7f25': 'CjkUnifiedIdeograph7f25',
u'\u5ea4': 'CjkUnifiedIdeograph5ea4',
u'\u882b': 'CjkUnifiedIdeograph882b',
u'\u97aa': 'CjkUnifiedIdeograph97aa',
u'\u0e2f': 'ThaiCharacterPaiyannoi',
u'\u6dae': 'CjkUnifiedIdeograph6dae',
u'\uc131': 'HangulSyllableSeong',
u'\u4735': 'CjkUnifiedIdeograph4735',
u'\ua6b4': 'UnicodeCharacter0xa6b4',
u'\u3cb8': 'CjkUnifiedIdeograph3cb8',
u'\u903b': 'CjkUnifiedIdeograph903b',
u'\uffba': 'HalfwidthHangulLetterChieuch',
u'\u163f': 'CanadianSyllabicsCarrierTla',
u'\u75be': 'CjkUnifiedIdeograph75be',
u'\uaf45': 'HangulSyllableGgot',
u'\u8ec4': 'CjkUnifiedIdeograph8ec4',
u'\u2549': 'BoxDrawingsRightLightAndLeftVerticalHeavy',
u'\u04c8': 'CyrillicSmallLetterEnWithHook',
u'\uf84b': 'UnicodeCharacter0xf84b',
u'\uc7ca': 'HangulSyllableJyagg',
u'\u7e4f': 'CjkUnifiedIdeograph7e4f',
u'\u5dce': 'CjkUnifiedIdeograph5dce',
u'\ub755': 'HangulSyllableDdyib',
u'\u96d4': 'CjkUnifiedIdeograph96d4',
u'\u0d59': 'UnicodeCharacter0xd59',
u'\u6cd8': 'CjkUnifiedIdeograph6cd8',
u'\uc05b': 'HangulSyllableBbeugs',
u'\u465f': 'CjkUnifiedIdeograph465f',
u'\ua5de': 'VaiSyllableMo',
u'\u3be2': 'CjkUnifiedIdeograph3be2',
u'\u9f65': 'CjkUnifiedIdeograph9f65',
u'\ufee4': 'ArabicLetterMeemMedialForm',
u'\u1569': 'CanadianSyllabicsWestcreeThwaa',
u'\u74e8': 'CjkUnifiedIdeograph74e8',
u'\uae6f': 'HangulSyllableGgaed',
u'\u8dee': 'CjkUnifiedIdeograph8dee',
u'\u2473': 'CircledNumberTwenty',
u'\u03f2': 'GreekLunateSigmaSymbol',
u'\ue775': 'UnicodeCharacter0xe775',
u'\uc6f4': 'HangulSyllableWels',
u'\u7d79': 'CjkUnifiedIdeograph7d79',
u'\u5cf8': 'CjkUnifiedIdeograph5cf8',
u'\ub67f': 'HangulSyllableDdoeh',
u'\u95fe': 'CjkUnifiedIdeograph95fe',
u'\ue006': 'UnicodeCharacter0xe006',
u'\uc489': 'HangulSyllableSsweolg',
u'\ua508': 'VaiSyllableMbee',
u'\u998b': 'CjkUnifiedIdeograph998b',
u'\u660a': 'CjkUnifiedIdeograph660a',
u'\u5a8d': 'CjkUnifiedIdeograph5a8d',
u'\u3b0c': 'CjkUnifiedIdeograph3b0c',
u'\u1f8f': 'GreekCapitalLetterAlphaWithDasiaAndPerispomeniAndProsgegrammeni',
u'\uc816': 'HangulSyllableJeoj',
u'\u8d18': 'CjkUnifiedIdeograph8d18',
u'\ue19b': 'UnicodeCharacter0xe19b',
u'\u4e1a': 'CjkUnifiedIdeograph4e1a',
u'\ua29d': 'YiSyllableZo',
u'\u031c': 'CombiningLeftHalfRingBelow',
u'\u679f': 'CjkUnifiedIdeograph679f',
u'\u38a1': 'CjkUnifiedIdeograph38a1',
u'\ud026': 'HangulSyllableKwelp',
u'\u9528': 'CjkUnifiedIdeograph9528',
u'\uc9ab': 'HangulSyllableJyid',
u'\u562a': 'CjkUnifiedIdeograph562a',
u'\u8aad': 'CjkUnifiedIdeograph8aad',
u'\u6b2c': 'CjkUnifiedIdeograph6b2c',
u'\u4faf': 'CjkUnifiedIdeograph4faf',
u'\u2c2e': 'GlagoliticCapitalLetterLatinateMyslite',
u'\xb1': 'PlusminusSign',
u'\ufd38': 'ArabicLigatureSheenWithHahMedialForm',
u'\ud1bb': 'HangulSyllableToh',
u'\ube3a': 'HangulSyllableByibs',
u'\u92bd': 'CjkUnifiedIdeograph92bd',
u'\u733c': 'CjkUnifiedIdeograph733c',
u'\u57bf': 'CjkUnifiedIdeograph57bf',
u'\u343e': 'CjkUnifiedIdeograph343e',
u'\u68c1': 'CjkUnifiedIdeograph68c1',
u'\u2dc3': 'EthiopicSyllableQyaa',
u'\uc548': 'HangulSyllableAn',
u'\u864a': 'CjkUnifiedIdeograph864a',
u'\ufacd': 'CjkCompatibilityIdeographfacd',
u'\u5b4c': 'CjkUnifiedIdeograph5b4c',
u'\ubfcf': 'HangulSyllableBbugs',
u'\u1c4e': 'LepchaLetterTtha',
u'\u70d1': 'CjkUnifiedIdeograph70d1',
u'\u35d3': 'CjkUnifiedIdeograph35d3',
u'\uee5a': 'UnicodeCharacter0xee5a',
u'\uc2dd': 'HangulSyllableSig',
u'\ua35c': 'YiSyllableZhyx',
u'\u87df': 'CjkUnifiedIdeograph87df',
u'\u645e': 'CjkUnifiedIdeograph645e',
u'\u58e1': 'CjkUnifiedIdeograph58e1',
u'\u3960': 'CjkUnifiedIdeograph3960',
u'\u1de3': 'CombiningLatinSmallLetterRRotunda',
u'\uf66a': 'UnicodeCharacter0xf66a',
u'\u8b6c': 'CjkUnifiedIdeograph8b6c',
u'\uefef': 'UnicodeCharacter0xefef',
u'\u4c6e': 'CjkUnifiedIdeograph4c6e',
u'\ua0f1': 'YiSyllableVop',
u'\u0170': 'LatinCapitalLetterUWithDoubleAcute',
u'\u65f3': 'CjkUnifiedIdeograph65f3',
u'\u26f5': 'UnicodeCharacter0x26f5',
u'\ude7a': 'UnicodeCharacter0xde7a',
u'\u937c': 'CjkUnifiedIdeograph937c',
u'\uf7ff': 'UnicodeCharacter0xf7ff',
u'\u547e': 'CjkUnifiedIdeograph547e',
u'\u6980': 'CjkUnifiedIdeograph6980',
u'\ucd03': 'HangulSyllableCyec',
u'\u2a82': 'GreaterthanOrSlantedEqualToWithDotAbove',
u'\u8e05': 'CjkUnifiedIdeograph8e05',
u'\u4307': 'CjkUnifiedIdeograph4307',
u'\u0409': 'CyrillicCapitalLetterLje',
u'\ufb8c': 'ArabicLetterRrehIsolatedForm',
u'\ubc8e': 'HangulSyllableBeolm',
u'\u7190': 'CjkUnifiedIdeograph7190',
u'\ud513': 'HangulSyllablePeulh',
u'\u3292': 'CircledIdeographHave',
u'\u9615': 'CjkUnifiedIdeograph9615',
u'\uab17': 'UnicodeCharacter0xab17',
u'\u6c19': 'CjkUnifiedIdeograph6c19',
u'\u211b': 'ScriptCapitalR',
u'\uc39c': 'HangulSyllableSsek',
u'\u849e': 'CjkUnifiedIdeograph849e',
u'\u59a0': 'CjkUnifiedIdeograph59a0',
u'\u1aa2': 'UnicodeCharacter0x1aa2',
u'\ufe25': 'CombiningMacronRightHalf',
u'\ub327': 'HangulSyllableDyalb',
u'\u7429': 'CjkUnifiedIdeograph7429',
u'\u092b': 'DevanagariLetterPha',
u'\uecae': 'UnicodeCharacter0xecae',
u'\ua1b0': 'YiSyllableHlur',
u'\u62b2': 'CjkUnifiedIdeograph62b2',
u'\uc635': 'HangulSyllableOb',
u'\u27b4': 'BlackfeatheredSouthEastArrow',
u'\u9b37': 'CjkUnifiedIdeograph9b37',
u'\u5c39': 'CjkUnifiedIdeograph5c39',
u'\u113b': 'HangulChoseongSioshieuh',
u'\uf4be': 'UnicodeCharacter0xf4be',
u'\u89c0': 'CjkUnifiedIdeograph89c0',
u'\u4ac2': 'CjkUnifiedIdeograph4ac2',
u'\u0fc4': 'TibetanSymbolDrilBu',
u'\ue347': 'UnicodeCharacter0xe347',
u'\ua449': 'YiSyllableNjyr',
u'\u794b': 'CjkUnifiedIdeograph794b',
u'\u3a4d': 'CjkUnifiedIdeograph3a4d',
u'\udcce': 'UnicodeCharacter0xdcce',
u'\u91d0': 'CjkUnifiedIdeograph91d0',
u'\u52d2': 'CjkUnifiedIdeograph52d2',
u'\u17d4': 'KhmerSignKhan',
u'\ucb57': 'HangulSyllableJjulh',
u'\u28d6': 'BraillePatternDots23578',
u'\u8c59': 'CjkUnifiedIdeograph8c59',
u'\u415b': 'CjkUnifiedIdeograph415b',
u'\u025d': 'LatinSmallLetterReversedOpenEWithHook',
u'\uf9e0': 'CjkCompatibilityIdeographf9e0',
u'\ubae2': 'HangulSyllableMwaegg',
u'\u7fe4': 'CjkUnifiedIdeograph7fe4',
u'\ud367': 'HangulSyllablePyaed',
u'\u30e6': 'KatakanaLetterYu',
u'\u9469': 'CjkUnifiedIdeograph9469',
u'\ua96b': 'UnicodeCharacter0xa96b',
u'\u6a6d': 'CjkUnifiedIdeograph6a6d',
u'\u2f6f': 'KangxiRadicalStone',
u'\uc1f0': 'HangulSyllableSoem',
u'\u82f2': 'CjkUnifiedIdeograph82f2',
u'\u47f4': 'CjkUnifiedIdeograph47f4',
u'\u18f6': 'UnicodeCharacter0x18f6',
u'\ufc79': 'ArabicLigatureThehWithNoonFinalForm',
u'\ub17b': 'HangulSyllableNogs',
u'\u727d': 'CjkUnifiedIdeograph727d',
u'\u377f': 'CjkUnifiedIdeograph377f',
u'\u4b81': 'CjkUnifiedIdeograph4b81',
u'\uac04': 'HangulSyllableGan',
u'\u8087': 'CjkUnifiedIdeograph8087',
u'\u2608': 'Thunderstorm',
u'\u1a8b': 'UnicodeCharacter0x1a8b',
u'\ue50a': 'UnicodeCharacter0xe50a',
u'\ud98d': 'UnicodeCharacter0xd98d',
u'\u7f0e': 'CjkUnifiedIdeograph7f0e',
u'\u5391': 'CjkUnifiedIdeograph5391',
u'\ub414': 'HangulSyllableDwaek',
u'\ue897': 'UnicodeCharacter0xe897',
u'\u0e18': 'ThaiCharacterThoThong',
u'\u629b': 'CjkUnifiedIdeograph629b',
u'\ucd1a': 'HangulSyllableCobs',
u'\u471e': 'CjkUnifiedIdeograph471e',
u'\ubba1': 'HangulSyllableMwit',
u'\u3da5': 'CjkUnifiedIdeograph3da5',
u'\u9c24': 'CjkUnifiedIdeograph9c24',
u'\uf0a7': 'UnicodeCharacter0xf0a7',
u'\u1628': 'CanadianSyllabicsCarrierDlu',
u'\u4aab': 'CjkUnifiedIdeograph4aab',
u'\ud52a': 'HangulSyllablePyilm',
u'\uaf2e': 'HangulSyllableGgogg',
u'\u83b1': 'CjkUnifiedIdeograph83b1',
u'\u2132': 'TurnedCapitalF',
u'\u05b5': 'HebrewPointTsere',
u'\ue434': 'UnicodeCharacter0xe434',
u'\ud8b7': 'UnicodeCharacter0xd8b7',
u'\u7e38': 'CjkUnifiedIdeograph7e38',
u'\u52bb': 'CjkUnifiedIdeograph52bb',
u'\ub73e': 'HangulSyllableDdeuj',
u'\uebc1': 'UnicodeCharacter0xebc1',
u'\u0942': 'DevanagariVowelSignUu',
u'\u6dc5': 'CjkUnifiedIdeograph6dc5',
u'\ucc44': 'HangulSyllableCae',
u'\u4648': 'CjkUnifiedIdeograph4648',
u'\ubacb': 'HangulSyllableMwad',
u'\u3ccf': 'CjkUnifiedIdeograph3ccf',
u'\u9f4e': 'CjkUnifiedIdeograph9f4e',
u'\uf3d1': 'UnicodeCharacter0xf3d1',
u'\u1152': 'HangulChoseongChieuchkhieukh',
u'\u75d5': 'CjkUnifiedIdeograph75d5',
u'\ud454': 'HangulSyllablePoess',
u'\uae58': 'HangulSyllableGgals',
u'\u82db': 'CjkUnifiedIdeograph82db',
u'\u205c': 'DottedCross',
u'\u04df': 'CyrillicSmallLetterZeWithDiaeresis',
u'\ue75e': 'UnicodeCharacter0xe75e',
u'\udbe1': 'UnicodeCharacter0xdbe1',
u'\u7962': 'CjkUnifiedIdeograph7962',
u'\u5de5': 'CjkUnifiedIdeograph5de5',
u'\ub668': 'HangulSyllableDdoen',
u'\ueaeb': 'UnicodeCharacter0xeaeb',
u'\u086c': 'UnicodeCharacter0x86c',
u'\u6cef': 'CjkUnifiedIdeograph6cef',
u'\ucf6e': 'HangulSyllableKop',
u'\u4172': 'CjkUnifiedIdeograph4172',
u'\ua5f5': 'VaiSyllableDhe',
u'\u3ff9': 'CjkUnifiedIdeograph3ff9',
u'\u9e78': 'CjkUnifiedIdeograph9e78',
u'\uf2fb': 'UnicodeCharacter0xf2fb',
u'\u107c': 'MyanmarLetterShanNa',
u'\u74ff': 'CjkUnifiedIdeograph74ff',
u'\ud77e': 'HangulSyllableHyibs',
u'\u4e03': 'CjkUnifiedIdeograph4e03',
u'\ua982': 'UnicodeCharacter0xa982',
u'\u2386': 'EnterSymbol',
u'\u8709': 'CjkUnifiedIdeograph8709',
u'\ue688': 'UnicodeCharacter0xe688',
u'\u190d': 'LimbuLetterDa',
u'\u788c': 'CjkUnifiedIdeograph788c',
u'\udc0f': 'UnicodeCharacter0xdc0f',
u'\u5613': 'CjkUnifiedIdeograph5613',
u'\ub192': 'HangulSyllableNop',
u'\u2817': 'BraillePatternDots1235',
u'\u0b96': 'UnicodeCharacter0xb96',
u'\uef19': 'UnicodeCharacter0xef19',
u'\uce98': 'HangulSyllableKael',
u'\u611d': 'CjkUnifiedIdeograph611d',
u'\u409c': 'CjkUnifiedIdeograph409c',
u'\ube23': 'HangulSyllableBeuc',
u'\u99a2': 'CjkUnifiedIdeograph99a2',
u'\u3027': 'HangzhouNumeralSeven',
u'\u13a6': 'CherokeeLetterGa',
u'\uf729': 'UnicodeCharacter0xf729',
u'\ud6a8': 'HangulSyllableHyo',
u'\u492d': 'CjkUnifiedIdeograph492d',
u'\ua8ac': 'SaurashtraLetterRa',
u'\u22b0': 'PrecedesUnderRelation',
u'\u8633': 'CjkUnifiedIdeograph8633',
u'\ue1b2': 'UnicodeCharacter0xe1b2',
u'\u1837': 'MongolianLetterRa',
u'\u7bb6': 'CjkUnifiedIdeograph7bb6',
u'\udf39': 'UnicodeCharacter0xdf39',
u'\u513d': 'CjkUnifiedIdeograph513d',
u'\ub0bc': 'HangulSyllableNael',
u'\u2b41': 'ReverseTildeOperatorAboveLeftwardsArrow',
u'\u0ac0': 'GujaratiVowelSignIi',
u'\uee43': 'UnicodeCharacter0xee43',
u'\uc9c2': 'HangulSyllableJigg',
u'\u6047': 'CjkUnifiedIdeograph6047',
u'\u43c6': 'CjkUnifiedIdeograph43c6',
u'\ub94d': 'HangulSyllableRwib',
u'\u98cc': 'CjkUnifiedIdeograph98cc',
u'\u3351': 'SquareRittoru',
u'\u12d0': 'EthiopicSyllablePharyngealA',
u'\uf653': 'UnicodeCharacter0xf653',
u'\ud1d2': 'HangulSyllableTwaj',
u'\u4857': 'CjkUnifiedIdeograph4857',
u'\uabd6': 'UnicodeCharacter0xabd6',
u'\u2dda': 'EthiopicSyllableGyi',
u'\u815d': 'CjkUnifiedIdeograph815d',
u'\ue0dc': 'UnicodeCharacter0xe0dc',
u'\u1b61': 'BalineseMusicalSymbolDong',
u'\u7ae0': 'CjkUnifiedIdeograph7ae0',
u'\ude63': 'UnicodeCharacter0xde63',
u'\u5067': 'CjkUnifiedIdeograph5067',
u'\ub3e6': 'HangulSyllableDwanh',
u'\u2a6b': 'TildeOperatorWithRisingDots',
u'\u35ea': 'CjkUnifiedIdeograph35ea',
u'\ue96d': 'UnicodeCharacter0xe96d',
u'\uc8ec': 'HangulSyllableJyols',
u'\u6371': 'CjkUnifiedIdeograph6371',
u'\u42f0': 'CjkUnifiedIdeograph42f0',
u'\ub877': 'HangulSyllableRoh',
u'\u9bf6': 'CjkUnifiedIdeograph9bf6',
u'\u327b': 'CircledHangulHieuhA',
u'\u1dfa': 'UnicodeCharacter0x1dfa',
u'\uf17d': 'UnicodeCharacter0xf17d',
u'\ud0fc': 'HangulSyllableTyan',
u'\uca81': 'HangulSyllableJjet',
u'\uab00': 'UnicodeCharacter0xab00',
u'\u8f83': 'CjkUnifiedIdeograph8f83',
u'\u6802': 'CjkUnifiedIdeograph6802',
u'\u4c85': 'CjkUnifiedIdeograph4c85',
u'\u2d04': 'GeorgianSmallLetterEn',
u'\u0187': 'LatinCapitalLetterCWithHook',
u'\ufe0e': 'VariationSelector15',
u'\ud291': 'HangulSyllableTwib',
u'\ub310': 'HangulSyllableDaem',
u'\u9793': 'CjkUnifiedIdeograph9793',
u'\u7012': 'CjkUnifiedIdeograph7012',
u'\u5495': 'CjkUnifiedIdeograph5495',
u'\u3514': 'CjkUnifiedIdeograph3514',
u'\u6997': 'CjkUnifiedIdeograph6997',
u'\u2e99': 'CjkRadicalRap',
u'\uc61e': 'HangulSyllableYej',
u'\u9b20': 'CjkUnifiedIdeograph9b20',
u'\uffa3': 'HalfwidthHangulLetterKiyeoksios',
u'\u5822': 'CjkUnifiedIdeograph5822',
u'\ubca5': 'HangulSyllableBenj',
u'\u1d24': 'LatinLetterVoicedLaryngealSpirant',
u'\u71a7': 'CjkUnifiedIdeograph71a7',
u'\u36a9': 'CjkUnifiedIdeograph36a9',
u'\ue330': 'UnicodeCharacter0xe330',
u'\uc7b3': 'HangulSyllableJaed',
u'\ua032': 'YiSyllableByt',
u'\u84b5': 'CjkUnifiedIdeograph84b5',
u'\u6534': 'CjkUnifiedIdeograph6534',
u'\u59b7': 'CjkUnifiedIdeograph59b7',
u'\u3a36': 'CjkUnifiedIdeograph3a36',
u'\u1eb9': 'LatinSmallLetterEWithDotBelow',
u'\ucb40': 'HangulSyllableJjyoss',
u'\u8842': 'CjkUnifiedIdeograph8842',
u'\uecc5': 'UnicodeCharacter0xecc5',
u'\u4d44': 'CjkUnifiedIdeograph4d44',
u'\ua1c7': 'YiSyllableLot',
u'\u0246': 'LatinCapitalLetterEWithStroke',
u'\u66c9': 'CjkUnifiedIdeograph66c9',
u'\u3bcb': 'CjkUnifiedIdeograph3bcb',
u'\ud350': 'HangulSyllablePyals',
u'\u9052': 'CjkUnifiedIdeograph9052',
u'\uf4d5': 'UnicodeCharacter0xf4d5',
u'\u5554': 'CjkUnifiedIdeograph5554',
u'\u89d7': 'CjkUnifiedIdeograph89d7',
u'\u6a56': 'CjkUnifiedIdeograph6a56',
u'\u4ed9': 'CjkUnifiedIdeograph4ed9',
u'\u2f58': 'KangxiRadicalDoubleX',
u'\u03db': 'GreekSmallLetterStigma',
u'\uf862': 'UnicodeCharacter0xf862',
u'\udce5': 'UnicodeCharacter0xdce5',
u'\ubd64': 'HangulSyllableByo',
u'\u91e7': 'CjkUnifiedIdeograph91e7',
u'\u7266': 'CjkUnifiedIdeograph7266',
u'\u56e9': 'CjkUnifiedIdeograph56e9',
u'\u3768': 'CjkUnifiedIdeograph3768',
u'\u6beb': 'CjkUnifiedIdeograph6beb',
u'\u28ed': 'BraillePatternDots134678',
u'\uc072': 'HangulSyllableBbeup',
u'\u8574': 'CjkUnifiedIdeograph8574',
u'\uf9f7': 'CjkCompatibilityIdeographf9f7',
u'\u5a76': 'CjkUnifiedIdeograph5a76',
u'\ubef9': 'HangulSyllableBbelt',
u'\u1f78': 'GreekSmallLetterOmicronWithVaria',
u'\u73fb': 'CjkUnifiedIdeograph73fb',
u'\u30fd': 'KatakanaIterationMark',
u'\u0a01': 'GurmukhiSignAdakBindi',
u'\ued84': 'UnicodeCharacter0xed84',
u'\ua286': 'YiSyllableWuop',
u'\u6788': 'CjkUnifiedIdeograph6788',
u'\udb0b': 'UnicodeCharacter0xdb0b',
u'\u248a': 'DigitThreeFullStop',
u'\u980d': 'CjkUnifiedIdeograph980d',
u'\u5d0f': 'CjkUnifiedIdeograph5d0f',
u'\u1211': 'EthiopicSyllableHhu',
u'\uf594': 'UnicodeCharacter0xf594',
u'\u8a96': 'CjkUnifiedIdeograph8a96',
u'\u4f98': 'CjkUnifiedIdeograph4f98',
u'\u0c9a': 'KannadaLetterCa',
u'\ue01d': 'UnicodeCharacter0xe01d',
u'\ua51f': 'VaiSyllableNggee',
u'\u7a21': 'CjkUnifiedIdeograph7a21',
u'\u3f23': 'CjkUnifiedIdeograph3f23',
u'\udda4': 'UnicodeCharacter0xdda4',
u'\u92a6': 'CjkUnifiedIdeograph92a6',
u'\u57a8': 'CjkUnifiedIdeograph57a8',
u'\u14aa': 'CanadianSyllabicsMa',
u'\uc82d': 'HangulSyllableJeb',
u'\u29ac': 'MeasuredAngleWithOpenArmEndingInArrowPointingRightAndUp',
u'\u8d2f': 'CjkUnifiedIdeograph8d2f',
u'\u4231': 'CjkUnifiedIdeograph4231',
u'\u0733': 'SyriacZqaphaAbove',
u'\ufab6': 'CjkCompatibilityIdeographfab6',
u'\ubfb8': 'HangulSyllableBbyol',
u'\u7cba': 'CjkUnifiedIdeograph7cba',
u'\ud03d': 'HangulSyllableKwilg',
u'\u31bc': 'UnicodeCharacter0x31bc',
u'\u953f': 'CjkUnifiedIdeograph953f',
u'\uaa41': 'ChamLetterFinalG',
u'\u6f43': 'CjkUnifiedIdeograph6f43',
u'\u2c45': 'GlagoliticSmallLetterTvrido',
u'\uc2c6': 'HangulSyllableSyinh',
u'\u87c8': 'CjkUnifiedIdeograph87c8',
u'\u44ca': 'CjkUnifiedIdeograph44ca',
u'\u19cc': 'UnicodeCharacter0x19cc',
u'\ufd4f': 'UnicodeCharacter0xfd4f',
u'\ub251': 'HangulSyllableNweng',
u'\u7753': 'CjkUnifiedIdeograph7753',
u'\u3455': 'CjkUnifiedIdeograph3455',
u'\uefd8': 'UnicodeCharacter0xefd8',
u'\uacda': 'HangulSyllableGyej',
u'\u61dc': 'CjkUnifiedIdeograph61dc',
u'\uc55f': 'HangulSyllableAh',
u'\u26de': 'UnicodeCharacter0x26de',
u'\u9a61': 'CjkUnifiedIdeograph9a61',
u'\u5f63': 'CjkUnifiedIdeograph5f63',
u'\u1c65': 'OlChikiLetterIs',
u'\uf7e8': 'UnicodeCharacter0xf7e8',
u'\ub4ea': 'HangulSyllableDeulp',
u'\u49ec': 'CjkUnifiedIdeograph49ec',
u'\u0eee': 'UnicodeCharacter0xeee',
u'\ue271': 'UnicodeCharacter0xe271',
u'\ua773': 'LatinSmallLetterMum',
u'\u6475': 'CjkUnifiedIdeograph6475',
u'\u3977': 'CjkUnifiedIdeograph3977',
u'\udff8': 'UnicodeCharacter0xdff8',
u'\u9cfa': 'CjkUnifiedIdeograph9cfa',
u'\u51fc': 'CjkUnifiedIdeograph51fc',
u'\u16fe': 'UnicodeCharacter0x16fe',
u'\u2c00': 'GlagoliticCapitalLetterAzu',
u'\u0883': 'UnicodeCharacter0x883',
u'\uef02': 'UnicodeCharacter0xef02',
u'\ucb85': 'HangulSyllableJjwenj',
u'\u6506': 'CjkUnifiedIdeograph6506',
u'\u4189': 'CjkUnifiedIdeograph4189',
u'\ube0c': 'HangulSyllableBeu',
u'\u9a8f': 'CjkUnifiedIdeograph9a8f',
u'\u3410': 'CjkUnifiedIdeograph3410',
u'\u1093': 'MyanmarShanDigitThree',
u'\uf712': 'UnicodeCharacter0xf712',
u'\ud395': 'HangulSyllablePeot',
u'\u4d16': 'CjkUnifiedIdeograph4d16',
u'\ua999': 'UnicodeCharacter0xa999',
u'\u279d': 'TriangleheadedRightwardsArrow',
u'\u861c': 'CjkUnifiedIdeograph861c',
u'\ue29f': 'UnicodeCharacter0xe29f',
u'\u1c20': 'LepchaLetterSa',
u'\u78a3': 'CjkUnifiedIdeograph78a3',
u'\udf22': 'UnicodeCharacter0xdf22',
u'\u5526': 'CjkUnifiedIdeograph5526',
u'\ub1a9': 'HangulSyllableNwang',
u'\u0fad': 'TibetanSubjoinedLetterWa',
u'\uee2c': 'UnicodeCharacter0xee2c',
u'\ucaaf': 'HangulSyllableJjyelh',
u'\u6430': 'CjkUnifiedIdeograph6430',
u'\u40b3': 'CjkUnifiedIdeograph40b3',
u'\ubd36': 'HangulSyllableBwaelm',
u'\u99b9': 'CjkUnifiedIdeograph99b9',
u'\u3b3a': 'CjkUnifiedIdeograph3b3a',
u'\u17bd': 'KhmerVowelSignUa',
u'\uf63c': 'UnicodeCharacter0xf63c',
u'\ud2bf': 'HangulSyllableTeud',
u'\u4c40': 'CjkUnifiedIdeograph4c40',
u'\ua8c3': 'SaurashtraVowelSignAu',
u'\u26c7': 'UnicodeCharacter0x26c7',
u'\u8546': 'CjkUnifiedIdeograph8546',
u'\ue1c9': 'UnicodeCharacter0xe1c9',
u'\u034a': 'CombiningNotTildeAbove',
u'\u7fcd': 'CjkUnifiedIdeograph7fcd',
u'\ude4c': 'UnicodeCharacter0xde4c',
u'\u5450': 'CjkUnifiedIdeograph5450',
u'\ub0d3': 'HangulSyllableNyags',
u'\u0ed7': 'LaoDigitSeven',
u'\ued56': 'UnicodeCharacter0xed56',
u'\uc9d9': 'HangulSyllableJit',
u'\u6b5a': 'CjkUnifiedIdeograph6b5a',
u'\u47dd': 'CjkUnifiedIdeograph47dd',
u'\ubc60': 'HangulSyllableByass',
u'\u98e3': 'CjkUnifiedIdeograph98e3',
u'\u3a64': 'CjkUnifiedIdeograph3a64',
u'\u16e7': 'RunicLetterShorttwigyr',
u'\uf566': 'UnicodeCharacter0xf566',
u'\ud1e9': 'HangulSyllableTwaeb',
u'\u736a': 'CjkUnifiedIdeograph736a',
u'\uafed': 'HangulSyllableGgweot',
u'\u25f1': 'WhiteSquareWithLowerLeftQuadrant',
u'\u8470': 'CjkUnifiedIdeograph8470',
u'\ue0f3': 'UnicodeCharacter0xe0f3',
u'\u0274': 'LatinLetterSmallCapitalN',
u'\u7ef7': 'CjkUnifiedIdeograph7ef7',
u'\udd76': 'UnicodeCharacter0xdd76',
u'\u5b7a': 'CjkUnifiedIdeograph5b7a',
u'\ub7fd': 'HangulSyllableReob',
u'\u8d01': 'CjkUnifiedIdeograph8d01',
u'\uec80': 'UnicodeCharacter0xec80',
u'\u0b05': 'OriyaLetterA',
u'\u6a84': 'CjkUnifiedIdeograph6a84',
u'\uc607': 'HangulSyllableYeoh',
u'\u5c0b': 'CjkUnifiedIdeograph5c0b',
u'\ua38a': 'YiSyllableRru',
u'\u398e': 'CjkUnifiedIdeograph398e',
u'\u9511': 'CjkUnifiedIdeograph9511',
u'\uf490': 'UnicodeCharacter0xf490',
u'\u1315': 'EthiopicSyllableGwe',
u'\u7294': 'CjkUnifiedIdeograph7294',
u'\ua41b': 'YiSyllableJjip',
u'\u8b9a': 'CjkUnifiedIdeograph8b9a',
u'\u221f': 'RightAngle',
u'\u019e': 'LatinSmallLetterNWithLongRightLeg',
u'\ufd21': 'ArabicLigatureSadWithAlefMaksuraFinalForm',
u'\udca0': 'UnicodeCharacter0xdca0',
u'\u7b25': 'CjkUnifiedIdeograph7b25',
u'\u5aa4': 'CjkUnifiedIdeograph5aa4',
u'\u8c2b': 'CjkUnifiedIdeograph8c2b',
u'\u93aa': 'CjkUnifiedIdeograph93aa',
u'\u0a2f': 'GurmukhiLetterYa',
u'\u69ae': 'CjkUnifiedIdeograph69ae',
u'\uc531': 'HangulSyllableSsilg',
u'\u4335': 'CjkUnifiedIdeograph4335',
u'\ua2b4': 'YiSyllableCie',
u'\u38b8': 'CjkUnifiedIdeograph38b8',
u'\u943b': 'CjkUnifiedIdeograph943b',
u'\ufbba': 'UnicodeCharacter0xfbba',
u'\u123f': 'EthiopicSyllableShwa',
u'\u71be': 'CjkUnifiedIdeograph71be',
u'\uab45': 'UnicodeCharacter0xab45',
u'\u8ac4': 'CjkUnifiedIdeograph8ac4',
u'\u2149': 'DoublestruckItalicSmallJ',
u'\xc8': 'LatinCapitalLetterEWithGrave',
u'\ufc4b': 'ArabicLigatureNoonWithJeemIsolatedForm',
u'\uc3ca': 'HangulSyllableSsyelp',
u'\u7a4f': 'CjkUnifiedIdeograph7a4f',
u'\u59ce': 'CjkUnifiedIdeograph59ce',
u'\ub355': 'HangulSyllableDeog',
u'\u92d4': 'CjkUnifiedIdeograph92d4',
u'\u0959': 'DevanagariLetterKhha',
u'\u68d8': 'CjkUnifiedIdeograph68d8',
u'\uc45b': 'HangulSyllableSsyos',
u'\u425f': 'CjkUnifiedIdeograph425f',
u'\ua1de': 'YiSyllableGiet',
u'\u3fe2': 'CjkUnifiedIdeograph3fe2',
u'\u9b65': 'CjkUnifiedIdeograph9b65',
u'\ufae4': 'UnicodeCharacter0xfae4',
u'\u1169': 'HangulJungseongO',
u'\u70e8': 'CjkUnifiedIdeograph70e8',
u'\uaa6f': 'UnicodeCharacter0xaa6f',
u'\u89ee': 'CjkUnifiedIdeograph89ee',
u'\u2073': 'UnicodeCharacter0x2073',
u'\u07f2': 'NkoCombiningNasalizationMark',
u'\ue375': 'UnicodeCharacter0xe375',
u'\uc2f4': 'HangulSyllableSik',
u'\u7979': 'CjkUnifiedIdeograph7979',
u'\u58f8': 'CjkUnifiedIdeograph58f8',
u'\ub27f': 'HangulSyllableNyulb',
u'\u91fe': 'CjkUnifiedIdeograph91fe',
u'\ue406': 'UnicodeCharacter0xe406',
u'\uc089': 'HangulSyllableBbying',
u'\ua108': 'YiSyllableDax',
u'\u9d8b': 'CjkUnifiedIdeograph9d8b',
u'\u620a': 'CjkUnifiedIdeograph620a',
u'\u5e8d': 'CjkUnifiedIdeograph5e8d',
u'\u3f0c': 'CjkUnifiedIdeograph3f0c',
u'\u1b8f': 'SundaneseLetterJa',
u'\ucc16': 'HangulSyllableJjilm',
u'\u8918': 'CjkUnifiedIdeograph8918',
u'\ue59b': 'UnicodeCharacter0xe59b',
u'\u4a1a': 'CjkUnifiedIdeograph4a1a',
u'\ua69d': 'UnicodeCharacter0xa69d',
u'\u071c': 'SyriacLetterTethGarshuni',
u'\u639f': 'CjkUnifiedIdeograph639f',
u'\u3ca1': 'CjkUnifiedIdeograph3ca1',
u'\ud426': 'HangulSyllablePwaegg',
u'\u9128': 'CjkUnifiedIdeograph9128',
u'\ucdab': 'HangulSyllableCuc',
u'\u522a': 'CjkUnifiedIdeograph522a',
u'\u8ead': 'CjkUnifiedIdeograph8ead',
u'\u6f2c': 'CjkUnifiedIdeograph6f2c',
u'\u4baf': 'CjkUnifiedIdeograph4baf',
u'\u282e': 'BraillePatternDots2346',
u'\u04b1': 'CyrillicSmallLetterStraightUWithStroke',
u'\uf938': 'CjkCompatibilityIdeographf938',
u'\ud5bb': 'HangulSyllableHyaelh',
u'\uba3a': 'HangulSyllableMeogg',
u'\u96bd': 'CjkUnifiedIdeograph96bd',
u'\u773c': 'CjkUnifiedIdeograph773c',
u'\u53bf': 'CjkUnifiedIdeograph53bf',
u'\u303e': 'IdeographicVariationIndicator',
u'\u6cc1': 'CjkUnifiedIdeograph6cc1',
u'\u29c3': 'CircleWithTwoHorizontalStrokesToTheRight',
u'\uc148': 'HangulSyllableSem',
u'\u824a': 'CjkUnifiedIdeograph824a',
u'\ufecd': 'ArabicLetterGhainIsolatedForm',
u'\u5f4c': 'CjkUnifiedIdeograph5f4c',
u'\ubbcf': 'HangulSyllableMeulh',
u'\u184e': 'MongolianLetterTodoGa',
u'\u74d1': 'CjkUnifiedIdeograph74d1',
u'\u31d3': 'CjkStrokeSp',
u'\uea5a': 'UnicodeCharacter0xea5a',
u'\uc6dd': 'HangulSyllableWeob',
u'\ua75c': 'LatinCapitalLetterRumRotunda',
u'\u83df': 'CjkUnifiedIdeograph83df',
u'\u605e': 'CjkUnifiedIdeograph605e',
u'\u5ce1': 'CjkUnifiedIdeograph5ce1',
u'\u3d60': 'CjkUnifiedIdeograph3d60',
u'\u19e3': 'KhmerSymbolBeiKoet',
u'\uf26a': 'UnicodeCharacter0xf26a',
u'\u8f6c': 'CjkUnifiedIdeograph8f6c',
u'\uebef': 'UnicodeCharacter0xebef',
u'\u486e': 'CjkUnifiedIdeograph486e',
u'\ua4f1': 'UnicodeCharacter0xa4f1',
u'\u0570': 'ArmenianSmallLetterHo',
u'\u61f3': 'CjkUnifiedIdeograph61f3',
u'\u22f5': 'ElementOfWithDotAbove',
u'\uda7a': 'UnicodeCharacter0xda7a',
u'\u977c': 'CjkUnifiedIdeograph977c',
u'\uf3ff': 'UnicodeCharacter0xf3ff',
u'\u507e': 'CjkUnifiedIdeograph507e',
u'\u6d80': 'CjkUnifiedIdeograph6d80',
u'\uc903': 'HangulSyllableJud',
u'\u2e82': 'CjkRadicalSecondOne',
u'\u8a05': 'CjkUnifiedIdeograph8a05',
u'\u4707': 'CjkUnifiedIdeograph4707',
u'\uff8c': 'HalfwidthKatakanaLetterHu',
u'\ub88e': 'HangulSyllableRwaj',
u'\u7590': 'CjkUnifiedIdeograph7590',
u'\ud113': 'HangulSyllableTyah',
u'\u3692': 'CjkUnifiedIdeograph3692',
u'\u9215': 'CjkUnifiedIdeograph9215',
u'\uaf17': 'HangulSyllableGgyed',
u'\u6819': 'CjkUnifiedIdeograph6819',
u'\u251b': 'BoxDrawingsHeavyUpAndLeft',
u'\uc79c': 'HangulSyllableJals',
u'\u809e': 'CjkUnifiedIdeograph809e',
u'\u5da0': 'CjkUnifiedIdeograph5da0',
u'\u1ea2': 'LatinCapitalLetterAWithHookAbove',
u'\ufa25': 'CjkCompatibilityIdeographfa25',
u'\ub727': 'HangulSyllableDdyuh',
u'\u7029': 'CjkUnifiedIdeograph7029',
u'\u0d2b': 'MalayalamLetterPha',
u'\ue8ae': 'UnicodeCharacter0xe8ae',
u'\ua5b0': 'VaiSyllableCu',
u'\u66b2': 'CjkUnifiedIdeograph66b2',
u'\uc235': 'HangulSyllableSweog',
u'\u23b4': 'TopSquareBracket',
u'\u9f37': 'CjkUnifiedIdeograph9f37',
u'\u5839': 'CjkUnifiedIdeograph5839',
u'\u153b': 'CanadianSyllabicsYwaa',
u'\uf0be': 'UnicodeCharacter0xf0be',
u'\u8dc0': 'CjkUnifiedIdeograph8dc0',
u'\u4ec2': 'CjkUnifiedIdeograph4ec2',
u'\u0bc4': 'UnicodeCharacter0xbc4',
u'\ue747': 'UnicodeCharacter0xe747',
u'\ua049': 'YiSyllablePop',
u'\u7d4b': 'CjkUnifiedIdeograph7d4b',
u'\u3e4d': 'CjkUnifiedIdeograph3e4d',
u'\ud8ce': 'UnicodeCharacter0xd8ce',
u'\u95d0': 'CjkUnifiedIdeograph95d0',
u'\u56d2': 'CjkUnifiedIdeograph56d2',
u'\u13d4': 'CherokeeLetterTa',
u'\ucf57': 'HangulSyllableKogs',
u'\u2cd6': 'CopticCapitalLetterOldCopticGangia',
u'\u8859': 'CjkUnifiedIdeograph8859',
u'\u455b': 'CjkUnifiedIdeograph455b',
u'\u065d': 'ArabicReversedDamma',
u'\ufde0': 'UnicodeCharacter0xfde0',
u'\ubee2': 'HangulSyllableBbeobs',
u'\u7be4': 'CjkUnifiedIdeograph7be4',
u'\ud767': 'HangulSyllableHeuc',
u'\u34e6': 'CjkUnifiedIdeograph34e6',
u'\u9069': 'CjkUnifiedIdeograph9069',
u'\uad6b': 'HangulSyllableGyoh',
u'\u6e6d': 'CjkUnifiedIdeograph6e6d',
u'\u2b6f': 'UnicodeCharacter0x2b6f',
u'\uc5f0': 'HangulSyllableYeon',
u'\u86f2': 'CjkUnifiedIdeograph86f2',
u'\u43f4': 'CjkUnifiedIdeograph43f4',
u'\u1cf6': 'UnicodeCharacter0x1cf6',
u'\uf879': 'UnicodeCharacter0xf879',
u'\ub57b': 'HangulSyllableDdyas',
u'\u767d': 'CjkUnifiedIdeograph767d',
u'\u337f': 'SquareCorporation',
u'\u4f81': 'CjkUnifiedIdeograph4f81',
u'\ua804': 'SylotiNagriLetterE',
u'\u8487': 'CjkUnifiedIdeograph8487',
u'\u2208': 'ElementOf',
u'\u1e8b': 'LatinSmallLetterXWithDotAbove',
u'\ue10a': 'UnicodeCharacter0xe10a',
u'\udd8d': 'UnicodeCharacter0xdd8d',
u'\u7b0e': 'CjkUnifiedIdeograph7b0e',
u'\u5791': 'CjkUnifiedIdeograph5791',
u'\ub014': 'HangulSyllableGgwil',
u'\uec97': 'UnicodeCharacter0xec97',
u'\u0a18': 'GurmukhiLetterGha',
u'\u669b': 'CjkUnifiedIdeograph669b',
u'\uc91a': 'HangulSyllableJweogg',
u'\u431e': 'CjkUnifiedIdeograph431e',
u'\ubfa1': 'HangulSyllableBboelt',
u'\u39a5': 'CjkUnifiedIdeograph39a5',
u'\u9824': 'CjkUnifiedIdeograph9824',
u'\uf4a7': 'UnicodeCharacter0xf4a7',
u'\u1228': 'EthiopicSyllableRa',
u'\u4eab': 'CjkUnifiedIdeograph4eab',
u'\ud12a': 'HangulSyllableTyaej',
u'\uab2e': 'UnicodeCharacter0xab2e',
u'\u87b1': 'CjkUnifiedIdeograph87b1',
u'\u2532': 'BoxDrawingsLeftLightAndRightDownHeavy',
u'\u01b5': 'LatinCapitalLetterZWithStroke',
u'\ue034': 'UnicodeCharacter0xe034',
u'\udcb7': 'UnicodeCharacter0xdcb7',
u'\u7a38': 'CjkUnifiedIdeograph7a38',
u'\u56bb': 'CjkUnifiedIdeograph56bb',
u'\ub33e': 'HangulSyllableDyaenh',
u'\uefc1': 'UnicodeCharacter0xefc1',
u'\u0d42': 'MalayalamVowelSignUu',
u'\u69c5': 'CjkUnifiedIdeograph69c5',
u'\uc844': 'HangulSyllableJyeols',
u'\u4248': 'CjkUnifiedIdeograph4248',
u'\ubecb': 'HangulSyllableBbyaec',
u'\u38cf': 'CjkUnifiedIdeograph38cf',
u'\u9b4e': 'CjkUnifiedIdeograph9b4e',
u'\uf7d1': 'UnicodeCharacter0xf7d1',
u'\u1552': 'CanadianSyllabicsMedialR',
u'\u71d5': 'CjkUnifiedIdeograph71d5',
u'\ud054': 'HangulSyllableKyun',
u'\uaa58': 'ChamDigitEight',
u'\u86db': 'CjkUnifiedIdeograph86db',
u'\u245c': 'UnicodeCharacter0x245c',
u'\xdf': 'LatinSmallLetterSharpS',
u'\ue35e': 'UnicodeCharacter0xe35e',
u'\udfe1': 'UnicodeCharacter0xdfe1',
u'\u7d62': 'CjkUnifiedIdeograph7d62',
u'\u59e5': 'CjkUnifiedIdeograph59e5',
u'\ub268': 'HangulSyllableNwim',
u'\ueeeb': 'UnicodeCharacter0xeeeb',
u'\u0c6c': 'TeluguDigitSix',
u'\u68ef': 'CjkUnifiedIdeograph68ef',
u'\ucb6e': 'HangulSyllableJjweolm',
u'\u4572': 'CjkUnifiedIdeograph4572',
u'\ua1f5': 'YiSyllableGup',
u'\u3bf9': 'CjkUnifiedIdeograph3bf9',
u'\u9a78': 'CjkUnifiedIdeograph9a78',
u'\uf6fb': 'UnicodeCharacter0xf6fb',
u'\u147c': 'CanadianSyllabicsKwoo',
u'\u70ff': 'CjkUnifiedIdeograph70ff',
u'\ud37e': 'HangulSyllablePeogg',
u'\u4a03': 'CjkUnifiedIdeograph4a03',
u'\uad82': 'HangulSyllableGuj',
u'\u2786': 'DingbatCircledSansserifDigitSeven',
u'\u8309': 'CjkUnifiedIdeograph8309',
u'\ue288': 'UnicodeCharacter0xe288',
u'\u1d0d': 'LatinLetterSmallCapitalM',
u'\u7c8c': 'CjkUnifiedIdeograph7c8c',
u'\ud80f': 'UnicodeCharacter0xd80f',
u'\u5213': 'CjkUnifiedIdeograph5213',
u'\ub592': 'HangulSyllableDdyaelp',
u'\u2c17': 'GlagoliticCapitalLetterFritu',
u'\u0f96': 'TibetanSubjoinedLetterCha',
u'\ueb19': 'UnicodeCharacter0xeb19',
u'\uca98': 'HangulSyllableJjyeoss',
u'\u651d': 'CjkUnifiedIdeograph651d',
u'\u449c': 'CjkUnifiedIdeograph449c',
u'\uba23': 'HangulSyllableMyaed',
u'\u9da2': 'CjkUnifiedIdeograph9da2',
u'\u3427': 'CjkUnifiedIdeograph3427',
u'\u17a6': 'KhmerIndependentVowelQii',
u'\uf329': 'UnicodeCharacter0xf329',
u'\ud2a8': 'HangulSyllableTyuls',
u'\u4d2d': 'CjkUnifiedIdeograph4d2d',
u'\uacac': 'HangulSyllableGyeon',
u'\u26b0': 'Coffin',
u'\u8233': 'CjkUnifiedIdeograph8233',
u'\ue5b2': 'UnicodeCharacter0xe5b2',
u'\u1c37': 'LepchaSignNukta',
u'\u7fb6': 'CjkUnifiedIdeograph7fb6',
u'\udb39': 'UnicodeCharacter0xdb39',
u'\u553d': 'CjkUnifiedIdeograph553d',
u'\ub4bc': 'HangulSyllableDwik',
u'\u2f41': 'KangxiRadicalRap',
u'\u0ec0': 'LaoVowelSignE',
u'\uea43': 'UnicodeCharacter0xea43',
u'\ucdc2': 'HangulSyllableCweobs',
u'\u6447': 'CjkUnifiedIdeograph6447',
u'\u47c6': 'CjkUnifiedIdeograph47c6',
u'\ubd4d': 'HangulSyllableBoenj',
u'\u9ccc': 'CjkUnifiedIdeograph9ccc',
u'\u3751': 'CjkUnifiedIdeograph3751',
u'\u16d0': 'RunicLetterShorttwigtyrT',
u'\uf253': 'UnicodeCharacter0xf253',
u'\ud5d2': 'HangulSyllableHeolm',
u'\u4c57': 'CjkUnifiedIdeograph4c57',
u'\uafd6': 'HangulSyllableGgweogg',
u'\u29da': 'LeftDoubleWigglyFence',
u'\u855d': 'CjkUnifiedIdeograph855d',
u'\ue4dc': 'UnicodeCharacter0xe4dc',
u'\u1f61': 'GreekSmallLetterOmegaWithDasia',
u'\u7ee0': 'CjkUnifiedIdeograph7ee0',
u'\uda63': 'UnicodeCharacter0xda63',
u'\u5467': 'CjkUnifiedIdeograph5467',
u'\ub7e6': 'HangulSyllableRyaej',
u'\u2e6b': 'UnicodeCharacter0x2e6b',
u'\u31ea': 'UnicodeCharacter0x31ea',
u'\ued6d': 'UnicodeCharacter0xed6d',
u'\uccec': 'HangulSyllableCye',
u'\u6771': 'CjkUnifiedIdeograph6771',
u'\u46f0': 'CjkUnifiedIdeograph46f0',
u'\ubc77': 'HangulSyllableByaelh',
u'\u9ff6': 'UnicodeCharacter0x9ff6',
u'\u367b': 'CjkUnifiedIdeograph367b',
u'\u19fa': 'KhmerSymbolDapRoc',
u'\uf57d': 'UnicodeCharacter0xf57d',
u'\ud4fc': 'HangulSyllablePyuss',
u'\uce81': 'HangulSyllableKalt',
u'\uaf00': 'HangulSyllableGgyeols',
u'\u8b83': 'CjkUnifiedIdeograph8b83',
u'\u6c02': 'CjkUnifiedIdeograph6c02',
u'\u4885': 'CjkUnifiedIdeograph4885',
u'\u2904': 'LeftRightDoubleArrowWithVerticalStroke',
u'\u0587': 'ArmenianSmallLigatureEchYiwn',
u'\ufa0e': 'CjkCompatibilityIdeographfa0e',
u'\ud691': 'HangulSyllableHoenj',
u'\ub710': 'HangulSyllableDdyun',
u'\u9393': 'CjkUnifiedIdeograph9393',
u'\u7412': 'CjkUnifiedIdeograph7412',
u'\u5095': 'CjkUnifiedIdeograph5095',
u'\u3114': 'BopomofoLetterCh',
u'\u6d97': 'CjkUnifiedIdeograph6d97',
u'\u2a99': 'DoublelineEqualToOrLessthan',
u'\uc21e': 'HangulSyllableSunh',
u'\u9f20': 'CjkUnifiedIdeograph9f20',
u'\ufba3': 'ArabicLetterRnoonMedialForm',
u'\u5c22': 'CjkUnifiedIdeograph5c22',
u'\ub8a5': 'HangulSyllableRwaeb',
u'\u1924': 'LimbuVowelSignAi',
u'\u75a7': 'CjkUnifiedIdeograph75a7',
u'\u32a9': 'CircledIdeographMedicine',
u'\ue730': 'UnicodeCharacter0xe730',
u'\uc3b3': 'HangulSyllableSsyeos',
u'\ua432': 'YiSyllableNjix',
u'\u80b5': 'CjkUnifiedIdeograph80b5',
u'\u6134': 'CjkUnifiedIdeograph6134',
u'\u5db7': 'CjkUnifiedIdeograph5db7',
u'\u3e36': 'CjkUnifiedIdeograph3e36',
u'\u1ab9': 'UnicodeCharacter0x1ab9',
u'\ucf40': 'HangulSyllableKyel',
u'\u8c42': 'CjkUnifiedIdeograph8c42',
u'\ue8c5': 'UnicodeCharacter0xe8c5',
u'\u4944': 'CjkUnifiedIdeograph4944',
u'\ua5c7': 'VaiSyllableGbo',
u'\u0646': 'ArabicLetterNoon',
u'\u62c9': 'CjkUnifiedIdeograph62c9',
u'\u3fcb': 'CjkUnifiedIdeograph3fcb',
u'\ud750': 'HangulSyllableHeu',
u'\u9452': 'CjkUnifiedIdeograph9452',
u'\uf0d5': 'UnicodeCharacter0xf0d5',
u'\u5154': 'CjkUnifiedIdeograph5154',
u'\u8dd7': 'CjkUnifiedIdeograph8dd7',
u'\u6e56': 'CjkUnifiedIdeograph6e56',
u'\u4ad9': 'CjkUnifiedIdeograph4ad9',
u'\u2b58': 'UnicodeCharacter0x2b58',
u'\u07db': 'NkoLetterSa',
u'\ufc62': 'ArabicLigatureShaddaWithKasraIsolatedForm',
u'\ud8e5': 'UnicodeCharacter0xd8e5',
u'\ub964': 'HangulSyllableRyuls',
u'\u95e7': 'CjkUnifiedIdeograph95e7',
u'\u7666': 'CjkUnifiedIdeograph7666',
u'\u52e9': 'CjkUnifiedIdeograph52e9',
u'\u3368': 'IdeographicTelegraphSymbolForHourSixteen',
u'\u6feb': 'CjkUnifiedIdeograph6feb',
u'\u2ced': 'UnicodeCharacter0x2ced',
u'\uc472': 'HangulSyllableSsulp',
u'\u8174': 'CjkUnifiedIdeograph8174',
u'\ufdf7': 'ArabicLigatureAlayheIsolatedForm',
u'\u5e76': 'CjkUnifiedIdeograph5e76',
u'\ubaf9': 'HangulSyllableMwaet',
u'\u1b78': 'BalineseMusicalSymbolLefthandOpenPang',
u'\u77fb': 'CjkUnifiedIdeograph77fb',
u'\u34fd': 'CjkUnifiedIdeograph34fd',
u'\u0e01': 'ThaiCharacterKoKai',
u'\ue984': 'UnicodeCharacter0xe984',
u'\ua686': 'CyrillicCapitalLetterCche',
u'\u6388': 'CjkUnifiedIdeograph6388',
u'\udf0b': 'UnicodeCharacter0xdf0b',
u'\u208a': 'SubscriptPlusSign',
u'\u9c0d': 'CjkUnifiedIdeograph9c0d',
u'\u590f': 'CjkUnifiedIdeograph590f',
u'\u1611': 'CanadianSyllabicsCarrierYee',
u'\uf194': 'UnicodeCharacter0xf194',
u'\u8e96': 'CjkUnifiedIdeograph8e96',
u'\u4b98': 'CjkUnifiedIdeograph4b98',
u'\u089a': 'UnicodeCharacter0x89a',
u'\ue41d': 'UnicodeCharacter0xe41d',
u'\ua11f': 'YiSyllableTie',
u'\u7e21': 'CjkUnifiedIdeograph7e21',
u'\u3b23': 'CjkUnifiedIdeograph3b23',
u'\ud9a4': 'UnicodeCharacter0xd9a4',
u'\u96a6': 'CjkUnifiedIdeograph96a6',
u'\u53a8': 'CjkUnifiedIdeograph53a8',
u'\u10aa': 'GeorgianCapitalLetterLas',
u'\ucc2d': 'HangulSyllableCanj',
u'\u2dac': 'EthiopicSyllableCcee',
u'\u892f': 'CjkUnifiedIdeograph892f',
u'\u4631': 'CjkUnifiedIdeograph4631',
u'\u0333': 'CombiningDoubleLowLine',
u'\ufeb6': 'ArabicLetterSheenFinalForm',
u'\ubbb8': 'HangulSyllableMyuss',
u'\u78ba': 'CjkUnifiedIdeograph78ba',
u'\ud43d': 'HangulSyllablePwaet',
u'\u35bc': 'CjkUnifiedIdeograph35bc',
u'\u913f': 'CjkUnifiedIdeograph913f',
u'\uae41': 'HangulSyllableGib',
u'\u6b43': 'CjkUnifiedIdeograph6b43',
u'\u2845': 'BraillePatternDots137',
u'\uc6c6': 'HangulSyllableUj',
u'\u83c8': 'CjkUnifiedIdeograph83c8',
u'\u40ca': 'CjkUnifiedIdeograph40ca',
u'\u1dcc': 'CombiningMacronbreve',
u'\uf94f': 'CjkCompatibilityIdeographf94f',
u'\ub651': 'HangulSyllableDdwaelg',
u'\u7353': 'CjkUnifiedIdeograph7353',
u'\u3055': 'HiraganaLetterSa',
u'\uebd8': 'UnicodeCharacter0xebd8',
u'\ua8da': 'UnicodeCharacter0xa8da',
u'\u65dc': 'CjkUnifiedIdeograph65dc',
u'\uc15f': 'HangulSyllableSyeolb',
u'\u22de': 'EqualToOrPrecedes',
u'\u9e61': 'CjkUnifiedIdeograph9e61',
u'\u5b63': 'CjkUnifiedIdeograph5b63',
u'\u1865': 'MongolianLetterSibeHa',
u'\uf3e8': 'UnicodeCharacter0xf3e8',
u'\ub0ea': 'HangulSyllableNyap',
u'\u4dec': 'HexagramForGatheringTogether',
u'\u0aee': 'GujaratiDigitEight',
u'\ue671': 'UnicodeCharacter0xe671',
u'\ua373': 'YiSyllableChup',
u'\u6075': 'CjkUnifiedIdeograph6075',
u'\u3d77': 'CjkUnifiedIdeograph3d77',
u'\udbf8': 'UnicodeCharacter0xdbf8',
u'\u98fa': 'CjkUnifiedIdeograph98fa',
u'\u55fc': 'CjkUnifiedIdeograph55fc',
u'\u12fe': 'EthiopicSyllableDdo',
u'\u3000': 'IdeographicSpace',
u'\u1483': 'CanadianSyllabicsK',
u'\uf302': 'UnicodeCharacter0xf302',
u'\ud785': 'HangulSyllableHyit',
u'\u7906': 'CjkUnifiedIdeograph7906',
u'\u5d89': 'CjkUnifiedIdeograph5d89',
u'\ua20c': 'YiSyllableKe',
u'\u868f': 'CjkUnifiedIdeograph868f',
u'\u2810': 'BraillePatternDots5',
u'\u0c93': 'KannadaLetterOo',
u'\ueb12': 'UnicodeCharacter0xeb12',
u'\ucf95': 'HangulSyllableKwaelg',
u'\u5116': 'CjkUnifiedIdeograph5116',
u'\ub599': 'HangulSyllableDdyaeng',
u'\u3b9d': 'CjkUnifiedIdeograph3b9d',
u'\u9a1c': 'CjkUnifiedIdeograph9a1c',
u'\ufe9f': 'ArabicLetterJeemInitialForm',
u'\u64a3': 'CjkUnifiedIdeograph64a3',
u'\uc322': 'HangulSyllableSsaelp',
u'\u4926': 'CjkUnifiedIdeograph4926',
u'\uada9': 'HangulSyllableGwenj',
u'\u13ad': 'CherokeeLetterHa',
u'\uf22c': 'UnicodeCharacter0xf22c',
u'\ud6af': 'HangulSyllableHyod',
u'\u7830': 'CjkUnifiedIdeograph7830',
u'\u5cb3': 'CjkUnifiedIdeograph5cb3',
u'\ua136': 'YiSyllableDdit',
u'\u85b9': 'CjkUnifiedIdeograph85b9',
u'\u273a': 'SixteenPointedAsterisk',
u'\u0bbd': 'UnicodeCharacter0xbbd',
u'\uea3c': 'UnicodeCharacter0xea3c',
u'\ucebf': 'HangulSyllableKyas',
u'\u5040': 'CjkUnifiedIdeograph5040',
u'\ub4c3': 'HangulSyllableDyugs',
u'\u3ac7': 'CjkUnifiedIdeograph3ac7',
u'\u9946': 'CjkUnifiedIdeograph9946',
u'\ufdc9': 'UnicodeCharacter0xfdc9',
u'\u1f4a': 'GreekCapitalLetterOmicronWithPsiliAndVaria',
u'\u63cd': 'CjkUnifiedIdeograph63cd',
u'\uc24c': 'HangulSyllableSweok',
u'\u4850': 'CjkUnifiedIdeograph4850',
u'\uacd3': 'HangulSyllableGyelh',
u'\u12d7': 'UnicodeCharacter0x12d7',
u'\uf156': 'UnicodeCharacter0xf156',
u'\ud5d9': 'HangulSyllableHeob',
u'\u775a': 'CjkUnifiedIdeograph775a',
u'\u5bdd': 'CjkUnifiedIdeograph5bdd',
u'\ua060': 'YiSyllableBba',
u'\u84e3': 'CjkUnifiedIdeograph84e3',
u'\u2664': 'WhiteSpadeSuit',
u'\u0ae7': 'GujaratiDigitOne',
u'\ue966': 'UnicodeCharacter0xe966',
u'\ucde9': 'HangulSyllableCwig',
u'\u6f6a': 'CjkUnifiedIdeograph6f6a',
u'\ub3ed': 'HangulSyllableDwalt',
u'\u39f1': 'CjkUnifiedIdeograph39f1',
u'\u9870': 'CjkUnifiedIdeograph9870',
u'\ufcf3': 'ArabicLigatureShaddaWithDammaMedialForm',
u'\u1e74': 'LatinCapitalLetterUWithTildeBelow',
u'\u62f7': 'CjkUnifiedIdeograph62f7',
u'\uc176': 'HangulSyllableSyenh',
u'\u477a': 'CjkUnifiedIdeograph477a',
u'\uabfd': 'UnicodeCharacter0xabfd',
u'\u9101': 'CjkUnifiedIdeograph9101',
u'\uf080': 'UnicodeCharacter0xf080',
u'\u1705': 'TagalogLetterNga',
u'\u7684': 'CjkUnifiedIdeograph7684',
u'\uda07': 'UnicodeCharacter0xda07',
u'\u400b': 'CjkUnifiedIdeograph400b',
u'\ubf8a': 'HangulSyllableBbwaebs',
u'\u258e': 'LeftOneQuarterBlock',
u'\u8911': 'CjkUnifiedIdeograph8911',
u'\ue890': 'UnicodeCharacter0xe890',
u'\u0f15': 'TibetanLogotypeSignChadRtags',
u'\u6e94': 'CjkUnifiedIdeograph6e94',
u'\ub81b': 'HangulSyllableRes',
u'\u979a': 'CjkUnifiedIdeograph979a',
u'\u3e1f': 'CjkUnifiedIdeograph3e1f',
u'\u1d9e': 'ModifierLetterSmallEth',
u'\ue121': 'UnicodeCharacter0xe121',
u'\uc0a0': 'HangulSyllableBbim',
u'\u6725': 'CjkUnifiedIdeograph6725',
u'\u46a4': 'CjkUnifiedIdeograph46a4',
u'\u902b': 'CjkUnifiedIdeograph902b',
u'\u8faa': 'CjkUnifiedIdeograph8faa',
u'\u162f': 'CanadianSyllabicsCarrierLho',
u'\u75ae': 'CjkUnifiedIdeograph75ae',
u'\ud931': 'UnicodeCharacter0xd931',
u'\u5f35': 'CjkUnifiedIdeograph5f35',
u'\ubeb4': 'HangulSyllableBbyae',
u'\u24b8': 'CircledLatinCapitalLetterC',
u'\u883b': 'CjkUnifiedIdeograph883b',
u'\ue7ba': 'UnicodeCharacter0xe7ba',
u'\u0e3f': 'ThaiCurrencySymbolBaht',
u'\u6dbe': 'CjkUnifiedIdeograph6dbe',
u'\ub745': 'HangulSyllableDdyig',
u'\u96c4': 'CjkUnifiedIdeograph96c4',
u'\u3d49': 'CjkUnifiedIdeograph3d49',
u'\u1cc8': 'UnicodeCharacter0x1cc8',
u'\ue04b': 'UnicodeCharacter0xe04b',
u'\udfca': 'UnicodeCharacter0xdfca',
u'\u664f': 'CjkUnifiedIdeograph664f',
u'\u45ce': 'CjkUnifiedIdeograph45ce',
u'\uaf55': 'HangulSyllableGgwalt',
u'\u8ed4': 'CjkUnifiedIdeograph8ed4',
u'\u1559': 'CanadianSyllabicsFa',
u'\u74d8': 'CjkUnifiedIdeograph74d8',
u'\ud85b': 'UnicodeCharacter0xd85b',
u'\u5e5f': 'CjkUnifiedIdeograph5e5f',
u'\ubdde': 'HangulSyllableBwilm',
u'\u23e2': 'WhiteTrapezium',
u'\u8765': 'CjkUnifiedIdeograph8765',
u'\ue6e4': 'UnicodeCharacter0xe6e4',
u'\u0d69': 'MalayalamDigitThree',
u'\u6ce8': 'CjkUnifiedIdeograph6ce8',
u'\ub66f': 'HangulSyllableDdoelb',
u'\u95ee': 'CjkUnifiedIdeograph95ee',
u'\u3c73': 'CjkUnifiedIdeograph3c73',
u'\u1bf2': 'UnicodeCharacter0x1bf2',
u'\uff75': 'HalfwidthKatakanaLetterO',
u'\udef4': 'UnicodeCharacter0xdef4',
u'\u6579': 'CjkUnifiedIdeograph6579',
u'\u44f8': 'CjkUnifiedIdeograph44f8',
u'\uae7f': 'HangulSyllableGgaec',
u'\u8dfe': 'CjkUnifiedIdeograph8dfe',
u'\uf806': 'UnicodeCharacter0xf806',
u'\udc89': 'UnicodeCharacter0xdc89',
u'\ubd08': 'HangulSyllableBoss',
u'\u818b': 'CjkUnifiedIdeograph818b',
u'\u7e0a': 'CjkUnifiedIdeograph7e0a',
u'\u428d': 'CjkUnifiedIdeograph428d',
u'\u230c': 'BottomRightCrop',
u'\u078f': 'ThaanaLetterGnaviyani',
u'\ud016': 'HangulSyllableKweop',
u'\u9518': 'CjkUnifiedIdeograph9518',
u'\uf99b': 'CjkCompatibilityIdeographf99b',
u'\u561a': 'CjkUnifiedIdeograph561a',
u'\uba9d': 'HangulSyllableMyeb',
u'\u1b1c': 'BalineseLetterNya',
u'\u7f9f': 'CjkUnifiedIdeograph7f9f',
u'\u20a1': 'ColonSign',
u'\uc826': 'HangulSyllableJelm',
u'\u8d28': 'CjkUnifiedIdeograph8d28',
u'\ud1ab': 'HangulSyllableTolb',
u'\u4e2a': 'CjkUnifiedIdeograph4e2a',
u'\u92ad': 'CjkUnifiedIdeograph92ad',
u'\u732c': 'CjkUnifiedIdeograph732c',
u'\u57af': 'CjkUnifiedIdeograph57af',
u'\u342e': 'CjkUnifiedIdeograph342e',
u'\u18b1': 'UnicodeCharacter0x18b1',
u'\ue538': 'UnicodeCharacter0xe538',
u'\uc9bb': 'HangulSyllableJyic',
u'\ua63a': 'UnicodeCharacter0xa63a',
u'\u8abd': 'CjkUnifiedIdeograph8abd',
u'\u6b3c': 'CjkUnifiedIdeograph6b3c',
u'\u4fbf': 'CjkUnifiedIdeograph4fbf',
u'\u2c3e': 'GlagoliticSmallLetterLjudije',
u'\u70c1': 'CjkUnifiedIdeograph70c1',
u'\u35c3': 'CjkUnifiedIdeograph35c3',
u'\udd48': 'UnicodeCharacter0xdd48',
u'\u9e4a': 'CjkUnifiedIdeograph9e4a',
u'\ue2cd': 'UnicodeCharacter0xe2cd',
u'\u434c': 'CjkUnifiedIdeograph434c',
u'\ua7cf': 'UnicodeCharacter0xa7cf',
u'\u044e': 'CyrillicSmallLetterYu',
u'\u68d1': 'CjkUnifiedIdeograph68d1',
u'\u2dd3': 'EthiopicSyllableXyaa',
u'\uf65a': 'UnicodeCharacter0xf65a',
u'\udadd': 'UnicodeCharacter0xdadd',
u'\ubb5c': 'HangulSyllableMweols',
u'\u9fdf': 'UnicodeCharacter0x9fdf',
u'\u7c5e': 'CjkUnifiedIdeograph7c5e',
u'\u40e1': 'CjkUnifiedIdeograph40e1',
u'\u2160': 'RomanNumeralOne',
u'\u05e3': 'HebrewLetterFinalPe',
u'\uee6a': 'UnicodeCharacter0xee6a',
u'\u936c': 'CjkUnifiedIdeograph936c',
u'\uf7ef': 'UnicodeCharacter0xf7ef',
u'\u546e': 'CjkUnifiedIdeograph546e',
u'\ub8f1': 'HangulSyllableRulg',
u'\u1970': 'TaiLeLetterTone2',
u'\u7df3': 'CjkUnifiedIdeograph7df3',
u'\u3ef5': 'CjkUnifiedIdeograph3ef5',
u'\uc67a': 'HangulSyllableOegg',
u'\u8b7c': 'CjkUnifiedIdeograph8b7c',
u'\uefff': 'UnicodeCharacter0xefff',
u'\u4c7e': 'CjkUnifiedIdeograph4c7e',
u'\u7180': 'CjkUnifiedIdeograph7180',
u'\ud503': 'HangulSyllablePyuh',
u'\u3282': 'CircledIdeographThree',
u'\u9605': 'CjkUnifiedIdeograph9605',
u'\u5b07': 'CjkUnifiedIdeograph5b07',
u'\u1c09': 'LepchaLetterNya',
u'\ue38c': 'UnicodeCharacter0xe38c',
u'\ua48e': 'UnicodeCharacter0xa48e',
u'\u6990': 'CjkUnifiedIdeograph6990',
u'\ucd13': 'HangulSyllableColb',
u'\u2a92': 'GreaterthanAboveLessthanAboveDoublelineEqual',
u'\u8e15': 'CjkUnifiedIdeograph8e15',
u'\ub317': 'HangulSyllableDaec',
u'\u7419': 'CjkUnifiedIdeograph7419',
u'\u391b': 'CjkUnifiedIdeograph391b',
u'\udb9c': 'UnicodeCharacter0xdb9c',
u'\u9c9e': 'CjkUnifiedIdeograph9c9e',
u'\u41a0': 'CjkUnifiedIdeograph41a0',
u'\u02a2': 'LatinLetterReversedGlottalStopWithStroke',
u'\ue625': 'UnicodeCharacter0xe625',
u'\uab27': 'UnicodeCharacter0xab27',
u'\u6c29': 'CjkUnifiedIdeograph6c29',
u'\u112b': 'HangulChoseongKapyeounpieup',
u'\uf4ae': 'UnicodeCharacter0xf4ae',
u'\ub9b0': 'HangulSyllableRin',
u'\u7ab2': 'CjkUnifiedIdeograph7ab2',
u'\ude35': 'UnicodeCharacter0xde35',
u'\u3fb4': 'CjkUnifiedIdeograph3fb4',
u'\u8337': 'CjkUnifiedIdeograph8337',
u'\u4439': 'CjkUnifiedIdeograph4439',
u'\u093b': 'UnicodeCharacter0x93b',
u'\uecbe': 'UnicodeCharacter0xecbe',
u'\u91c0': 'CjkUnifiedIdeograph91c0',
u'\u52c2': 'CjkUnifiedIdeograph52c2',
u'\u17c4': 'KhmerVowelSignOo',
u'\ufb47': 'HebrewLetterQofWithDagesh',
u'\ubc49': 'HangulSyllableBaet',
u'\u614b': 'CjkUnifiedIdeograph614b',
u'\u224d': 'EquivalentTo',
u'\uc4ce': 'HangulSyllableSswij',
u'\u89d0': 'CjkUnifiedIdeograph89d0',
u'\u4ad2': 'CjkUnifiedIdeograph4ad2',
u'\u0fd4': 'TibetanMarkClosingBrdaRnyingYigMgoSgabMa',
u'\ud357': 'HangulSyllablePyas',
u'\u30d6': 'KatakanaLetterBu',
u'\u9459': 'CjkUnifiedIdeograph9459',
u'\u595b': 'CjkUnifiedIdeograph595b',
u'\u1a5d': 'UnicodeCharacter0x1a5d',
u'\ue1e0': 'UnicodeCharacter0xe1e0',
u'\ua2e2': 'YiSyllableZzux',
u'\u67e4': 'CjkUnifiedIdeograph67e4',
u'\ucb67': 'HangulSyllableJjweogs',
u'\u28e6': 'BraillePatternDots23678',
u'\u8c69': 'CjkUnifiedIdeograph8c69',
u'\ub16b': 'HangulSyllableNyelh',
u'\u726d': 'CjkUnifiedIdeograph726d',
u'\u376f': 'CjkUnifiedIdeograph376f',
u'\ud9f0': 'UnicodeCharacter0xd9f0',
u'\u9af2': 'CjkUnifiedIdeograph9af2',
u'\u5ff4': 'CjkUnifiedIdeograph5ff4',
u'\xf6': 'LatinSmallLetterOWithDiaeresis',
u'\ue479': 'UnicodeCharacter0xe479',
u'\ua97b': 'UnicodeCharacter0xa97b',
u'\u6a7d': 'CjkUnifiedIdeograph6a7d',
u'\u2f7f': 'KangxiRadicalEar',
u'\u5381': 'CjkUnifiedIdeograph5381',
u'\ub404': 'HangulSyllableDwael',
u'\u9887': 'CjkUnifiedIdeograph9887',
u'\u3e08': 'CjkUnifiedIdeograph3e08',
u'\u028b': 'LatinSmallLetterVWithHook',
u'\ufd0a': 'ArabicLigatureSheenWithHahIsolatedForm',
u'\uc18d': 'HangulSyllableSog',
u'\u670e': 'CjkUnifiedIdeograph670e',
u'\u4b91': 'CjkUnifiedIdeograph4b91',
u'\uac14': 'HangulSyllableGass',
u'\uf097': 'UnicodeCharacter0xf097',
u'\u1618': 'CanadianSyllabicsCarrierJee',
u'\u7a9b': 'CjkUnifiedIdeograph7a9b',
u'\ud51a': 'HangulSyllablePeuj',
u'\u5f1e': 'CjkUnifiedIdeograph5f1e',
u'\ua3a1': 'YiSyllableNru',
u'\u25a5': 'SquareWithVerticalFill',
u'\u8424': 'CjkUnifiedIdeograph8424',
u'\ue8a7': 'UnicodeCharacter0xe8a7',
u'\u0e28': 'ThaiCharacterSoSala',
u'\u52ab': 'CjkUnifiedIdeograph52ab',
u'\ucd2a': 'HangulSyllableCwanh',
u'\ub72e': 'HangulSyllableDdeunh',
u'\u9bb1': 'CjkUnifiedIdeograph9bb1',
u'\u3932': 'CjkUnifiedIdeograph3932',
u'\u1db5': 'ModifierLetterSmallTWithPalatalHook',
u'\ufc34': 'ArabicLigatureQafWithMeemIsolatedForm',
u'\uc0b7': 'HangulSyllableSalb',
u'\u6638': 'CjkUnifiedIdeograph6638',
u'\u4abb': 'CjkUnifiedIdeograph4abb',
u'\uaf3e': 'HangulSyllableGgobs',
u'\uf3c1': 'UnicodeCharacter0xf3c1',
u'\u1142': 'HangulChoseongIeungtikeut',
u'\u75c5': 'CjkUnifiedIdeograph75c5',
u'\ud444': 'HangulSyllablePoen',
u'\u5e48': 'CjkUnifiedIdeograph5e48',
u'\ua2cb': 'YiSyllableCyx',
u'\u24cf': 'CircledLatinCapitalLetterZ',
u'\u874e': 'CjkUnifiedIdeograph874e',
u'\uebd1': 'UnicodeCharacter0xebd1',
u'\u0952': 'DevanagariStressSignAnudatta',
u'\u6dd5': 'CjkUnifiedIdeograph6dd5',
u'\ucc54': 'HangulSyllableCaem',
u'\ub658': 'HangulSyllableDdwaem',
u'\u9adb': 'CjkUnifiedIdeograph9adb',
u'\u385c': 'CjkUnifiedIdeograph385c',
u'\u1cdf': 'UnicodeCharacter0x1cdf',
u'\uff5e': 'FullwidthTilde',
u'\uc3e1': 'HangulSyllableSsolg',
u'\u6162': 'CjkUnifiedIdeograph6162',
u'\u45e5': 'CjkUnifiedIdeograph45e5',
u'\uae68': 'HangulSyllableGgae',
u'\uf2eb': 'UnicodeCharacter0xf2eb',
u'\u106c': 'MyanmarSignWesternPwoKarenTone4',
u'\u74ef': 'CjkUnifiedIdeograph74ef',
u'\ud76e': 'HangulSyllableHyigg',
u'\u5972': 'CjkUnifiedIdeograph5972',
u'\ubdf5': 'HangulSyllableByunj',
u'\u27f9': 'LongRightwardsDoubleArrow',
u'\u8678': 'CjkUnifiedIdeograph8678',
u'\ueafb': 'UnicodeCharacter0xeafb',
u'\u087c': 'UnicodeCharacter0x87c',
u'\u6cff': 'CjkUnifiedIdeograph6cff',
u'\ucf7e': 'HangulSyllableKwalp',
u'\u5603': 'CjkUnifiedIdeograph5603',
u'\ub182': 'HangulSyllableNolm',
u'\u3b86': 'CjkUnifiedIdeograph3b86',
u'\u9f09': 'CjkUnifiedIdeograph9f09',
u'\ufe88': 'ArabicLetterAlefWithHamzaBelowFinalForm',
u'\u010d': 'LatinSmallLetterCWithCaron',
u'\u608c': 'CjkUnifiedIdeograph608c',
u'\uc40f': 'HangulSyllableSswah',
u'\u4e13': 'CjkUnifiedIdeograph4e13',
u'\ua992': 'UnicodeCharacter0xa992',
u'\u3017': 'RightWhiteLenticularBracket',
u'\u1396': 'EthiopicTonalMarkChiret',
u'\uf719': 'UnicodeCharacter0xf719',
u'\ud698': 'HangulSyllableHoels',
u'\u791d': 'CjkUnifiedIdeograph791d',
u'\u589c': 'CjkUnifiedIdeograph589c',
u'\ua623': 'VaiDigitThree',
u'\u81a2': 'CjkUnifiedIdeograph81a2',
u'\u2827': 'BraillePatternDots1236',
u'\u0ba6': 'UnicodeCharacter0xba6',
u'\uef29': 'UnicodeCharacter0xef29',
u'\ucea8': 'HangulSyllableKaek',
u'\u512d': 'CjkUnifiedIdeograph512d',
u'\ub0ac': 'HangulSyllableNass',
u'\u3ab0': 'CjkUnifiedIdeograph3ab0',
u'\u9e33': 'CjkUnifiedIdeograph9e33',
u'\uf9b2': 'CjkCompatibilityIdeographf9b2',
u'\u63b6': 'CjkUnifiedIdeograph63b6',
u'\uc739': 'HangulSyllableYut',
u'\u493d': 'CjkUnifiedIdeograph493d',
u'\ua8bc': 'SaurashtraVowelSignVocalicL',
u'\u3341': 'SquareHooru',
u'\u12c0': 'EthiopicSyllableKxwa',
u'\uf643': 'UnicodeCharacter0xf643',
u'\ud1c2': 'HangulSyllableTwanh',
u'\u7847': 'CjkUnifiedIdeograph7847',
u'\u5bc6': 'CjkUnifiedIdeograph5bc6',
u'\ua14d': 'YiSyllableDdu',
u'\u80cc': 'CjkUnifiedIdeograph80cc',
u'\u2b51': 'BlackSmallStar',
u'\u0ad0': 'GujaratiOm',
u'\uee53': 'UnicodeCharacter0xee53',
u'\uc9d2': 'HangulSyllableJibs',
u'\u5057': 'CjkUnifiedIdeograph5057',
u'\ub3d6': 'HangulSyllableDobs',
u'\u35da': 'CjkUnifiedIdeograph35da',
u'\u995d': 'CjkUnifiedIdeograph995d',
u'\uf8dc': 'UnicodeCharacter0xf8dc',
u'\u0361': 'CombiningDoubleInvertedBreve',
u'\u62e0': 'CjkUnifiedIdeograph62e0',
u'\uc663': 'HangulSyllableWaed',
u'\u4867': 'CjkUnifiedIdeograph4867',
u'\uabe6': 'UnicodeCharacter0xabe6',
u'\u326b': 'CircledHangulThieuth',
u'\u2dea': 'CombiningCyrillicLetterO',
u'\uf16d': 'UnicodeCharacter0xf16d',
u'\ud0ec': 'HangulSyllableTaem',
u'\u7b71': 'CjkUnifiedIdeograph7b71',
u'\u5af0': 'CjkUnifiedIdeograph5af0',
u'\ua077': 'YiSyllableNbix',
u'\u83f6': 'CjkUnifiedIdeograph83f6',
u'\u2a7b': 'LessthanWithQuestionMarkAbove',
u'\u05fa': 'UnicodeCharacter0x5fa',
u'\ue97d': 'UnicodeCharacter0xe97d',
u'\uc8fc': 'HangulSyllableJu',
u'\ud281': 'HangulSyllableTwig',
u'\ub300': 'HangulSyllableDae',
u'\u9783': 'CjkUnifiedIdeograph9783',
u'\u7002': 'CjkUnifiedIdeograph7002',
u'\u5485': 'CjkUnifiedIdeograph5485',
u'\u3504': 'CjkUnifiedIdeograph3504',
u'\u1987': 'NewTaiLueLetterLowNga',
u'\ue60e': 'UnicodeCharacter0xe60e',
u'\uca91': 'HangulSyllableJjyeolt',
u'\uab10': 'UnicodeCharacter0xab10',
u'\u8f93': 'CjkUnifiedIdeograph8f93',
u'\u6812': 'CjkUnifiedIdeograph6812',
u'\u4c95': 'CjkUnifiedIdeograph4c95',
u'\u2d14': 'GeorgianSmallLetterPhar',
u'\u7197': 'CjkUnifiedIdeograph7197',
u'\u3699': 'CjkUnifiedIdeograph3699',
u'\ude1e': 'UnicodeCharacter0xde1e',
u'\u8320': 'CjkUnifiedIdeograph8320',
u'\ue7a3': 'UnicodeCharacter0xe7a3',
u'\u4022': 'CjkUnifiedIdeograph4022',
u'\ua4a5': 'YiRadicalBur',
u'\u0524': 'UnicodeCharacter0x524',
u'\u69a7': 'CjkUnifiedIdeograph69a7',
u'\u2ea9': 'CjkRadicalJade',
u'\ufb30': 'HebrewLetterAlefWithMapiq',
u'\udfb3': 'UnicodeCharacter0xdfb3',
u'\ub832': 'HangulSyllableRyeolp',
u'\u9cb5': 'CjkUnifiedIdeograph9cb5',
u'\u7d34': 'CjkUnifiedIdeograph7d34',
u'\u41b7': 'CjkUnifiedIdeograph41b7',
u'\u2236': 'Ratio',
u'\u06b9': 'ArabicLetterNoonWithDotBelow',
u'\ud340': 'HangulSyllablePaek',
u'\u9042': 'CjkUnifiedIdeograph9042',
u'\uf4c5': 'UnicodeCharacter0xf4c5',
u'\u5544': 'CjkUnifiedIdeograph5544',
u'\ub9c7': 'HangulSyllableRih',
u'\u1a46': 'UnicodeCharacter0x1a46',
u'\u7ec9': 'CjkUnifiedIdeograph7ec9',
u'\u23cb': 'DentistrySymbolLightVerticalAndTopLeft',
u'\ucb50': 'HangulSyllableJjul',
u'\u8852': 'CjkUnifiedIdeograph8852',
u'\uecd5': 'UnicodeCharacter0xecd5',
u'\u4d54': 'CjkUnifiedIdeograph4d54',
u'\u91d7': 'CjkUnifiedIdeograph91d7',
u'\u7256': 'CjkUnifiedIdeograph7256',
u'\u56d9': 'CjkUnifiedIdeograph56d9',
u'\u3758': 'CjkUnifiedIdeograph3758',
u'\u1bdb': 'UnicodeCharacter0x1bdb',
u'\ue062': 'UnicodeCharacter0xe062',
u'\uc4e5': 'HangulSyllableSsyub',
u'\ua564': 'VaiSyllableZa',
u'\u89e7': 'CjkUnifiedIdeograph89e7',
u'\u6a66': 'CjkUnifiedIdeograph6a66',
u'\u4ee9': 'CjkUnifiedIdeograph4ee9',
u'\u2f68': 'KangxiRadicalDottedTent',
u'\u73eb': 'CjkUnifiedIdeograph73eb',
u'\u30ed': 'KatakanaLetterRo',
u'\ud872': 'UnicodeCharacter0xd872',
u'\u9d74': 'CjkUnifiedIdeograph9d74',
u'\ue1f7': 'UnicodeCharacter0xe1f7',
u'\u4276': 'CjkUnifiedIdeograph4276',
u'\ua6f9': 'UnicodeCharacter0xa6f9',
u'\u0778': 'ArabicLetterWawWithExtendedArabicindicDigitTwoAbove',
u'\u6bfb': 'CjkUnifiedIdeograph6bfb',
u'\u28fd': 'BraillePatternDots1345678',
u'\u1201': 'EthiopicSyllableHu',
u'\uf584': 'UnicodeCharacter0xf584',
u'\uba86': 'HangulSyllableMyeoj',
u'\u7f88': 'CjkUnifiedIdeograph7f88',
u'\uc30b': 'HangulSyllableSsas',
u'\u3c8a': 'CjkUnifiedIdeograph3c8a',
u'\u800d': 'CjkUnifiedIdeograph800d',
u'\u450f': 'CjkUnifiedIdeograph450f',
u'\u0a11': 'UnicodeCharacter0xa11',
u'\ued94': 'UnicodeCharacter0xed94',
u'\u9296': 'CjkUnifiedIdeograph9296',
u'\u5798': 'CjkUnifiedIdeograph5798',
u'\u149a': 'CanadianSyllabicsCwoo',
u'\uf81d': 'UnicodeCharacter0xf81d',
u'\ubd1f': 'HangulSyllableBwalh',
u'\u6221': 'CjkUnifiedIdeograph6221',
u'\u2723': 'FourBalloonspokedAsterisk',
u'\uc5a4': 'HangulSyllableYaels',
u'\u8aa6': 'CjkUnifiedIdeograph8aa6',
u'\u4fa8': 'CjkUnifiedIdeograph4fa8',
u'\u0caa': 'KannadaLetterPa',
u'\ud02d': 'HangulSyllableKweng',
u'\u31ac': 'BopomofoLetterIm',
u'\u952f': 'CjkUnifiedIdeograph952f',
u'\u5a31': 'CjkUnifiedIdeograph5a31',
u'\u1f33': 'GreekSmallLetterIotaWithDasiaAndVaria',
u'\ue2b6': 'UnicodeCharacter0xe2b6',
u'\ua7b8': 'UnicodeCharacter0xa7b8',
u'\u64ba': 'CjkUnifiedIdeograph64ba',
u'\uc83d': 'HangulSyllableJyeonj',
u'\u29bc': 'CircledAnticlockwiserotatedDivisionSign',
u'\u8d3f': 'CjkUnifiedIdeograph8d3f',
u'\ub241': 'HangulSyllableNwenj',
u'\u7743': 'CjkUnifiedIdeograph7743',
u'\u3445': 'CjkUnifiedIdeograph3445',
u'\udac6': 'UnicodeCharacter0xdac6',
u'\u9fc8': 'UnicodeCharacter0x9fc8',
u'\u5cca': 'CjkUnifiedIdeograph5cca',
u'\u01cc': 'LatinSmallLetterNj',
u'\ue54f': 'UnicodeCharacter0xe54f',
u'\uaa51': 'ChamDigitOne',
u'\u6f53': 'CjkUnifiedIdeograph6f53',
u'\u2c55': 'GlagoliticSmallLetterSmallYusWithTail',
u'\uf7d8': 'UnicodeCharacter0xf7d8',
u'\ub4da': 'HangulSyllableDyup',
u'\u79dc': 'CjkUnifiedIdeograph79dc',
u'\udd5f': 'UnicodeCharacter0xdd5f',
u'\u3ede': 'CjkUnifiedIdeograph3ede',
u'\u8261': 'CjkUnifiedIdeograph8261',
u'\u4763': 'CjkUnifiedIdeograph4763',
u'\u0465': 'CyrillicSmallLetterIotifiedE',
u'\uefe8': 'UnicodeCharacter0xefe8',
u'\uacea': 'HangulSyllableGolm',
u'\u51ec': 'CjkUnifiedIdeograph51ec',
u'\u16ee': 'RunicArlaugSymbol',
u'\ufa71': 'CjkCompatibilityIdeographfa71',
u'\ubf73': 'HangulSyllableBbwac',
u'\u7c75': 'CjkUnifiedIdeograph7c75',
u'\u2177': 'SmallRomanNumeralEight',
u'\uc7f8': 'HangulSyllableJyaess',
u'\u84fa': 'CjkUnifiedIdeograph84fa',
u'\u49fc': 'CjkUnifiedIdeograph49fc',
u'\u0efe': 'UnicodeCharacter0xefe',
u'\u3400': 'CjkUnifiedIdeograph3400',
u'\u1083': 'MyanmarVowelSignShanAa',
u'\uf702': 'UnicodeCharacter0xf702',
u'\ud385': 'HangulSyllablePeolg',
u'\u7d06': 'CjkUnifiedIdeograph7d06',
u'\u5989': 'CjkUnifiedIdeograph5989',
u'\ua60c': 'VaiSyllableLengthener',
u'\u828f': 'CjkUnifiedIdeograph828f',
u'\u2c10': 'GlagoliticCapitalLetterNashi',
u'\u0893': 'UnicodeCharacter0x893',
u'\uef12': 'UnicodeCharacter0xef12',
u'\ucb95': 'HangulSyllableJjweng',
u'\u5516': 'CjkUnifiedIdeograph5516',
u'\ub199': 'HangulSyllableNwanj',
u'\u3f9d': 'CjkUnifiedIdeograph3f9d',
u'\u9e1c': 'CjkUnifiedIdeograph9e1c',
u'\ufa9f': 'CjkCompatibilityIdeographfa9f',
u'\u0420': 'CyrillicCapitalLetterEr',
u'\u60a3': 'CjkUnifiedIdeograph60a3',
u'\uc722': 'HangulSyllableYugg',
u'\u4d26': 'CjkUnifiedIdeograph4d26',
u'\ua9a9': 'UnicodeCharacter0xa9a9',
u'\u17ad': 'KhmerIndependentVowelLy',
u'\uf62c': 'UnicodeCharacter0xf62c',
u'\ud2af': 'HangulSyllableTyus',
u'\u7c30': 'CjkUnifiedIdeograph7c30',
u'\u58b3': 'CjkUnifiedIdeograph58b3',
u'\ua536': 'VaiSyllableDhhi',
u'\u81b9': 'CjkUnifiedIdeograph81b9',
u'\u233a': 'AplFunctionalSymbolQuadDiamond',
u'\u0fbd': 'UnicodeCharacter0xfbd',
u'\uee3c': 'UnicodeCharacter0xee3c',
u'\ucabf': 'HangulSyllableJjogs',
u'\u5440': 'CjkUnifiedIdeograph5440',
u'\ub0c3': 'HangulSyllableNaelh',
u'\u3ec7': 'CjkUnifiedIdeograph3ec7',
u'\u9d46': 'CjkUnifiedIdeograph9d46',
u'\uf9c9': 'CjkCompatibilityIdeographf9c9',
u'\u1b4a': 'BalineseLetterZalSasak',
u'\u67cd': 'CjkUnifiedIdeograph67cd',
u'\uc64c': 'HangulSyllableWals',
u'\u4c50': 'CjkUnifiedIdeograph4c50',
u'\ua8d3': 'SaurashtraDigitThree',
u'\u16d7': 'RunicLetterMannazManM',
u'\uf556': 'UnicodeCharacter0xf556',
u'\ud1d9': 'HangulSyllableTwaeg',
u'\u735a': 'CjkUnifiedIdeograph735a',
u'\u5fdd': 'CjkUnifiedIdeograph5fdd',
u'\ua460': 'YiSyllableXip',
u'\u80e3': 'CjkUnifiedIdeograph80e3',
u'\u2264': 'LessthanOrEqualTo',
u'\u0ee7': 'UnicodeCharacter0xee7',
u'\ued66': 'UnicodeCharacter0xed66',
u'\uc9e9': 'HangulSyllableJjalt',
u'\u6b6a': 'CjkUnifiedIdeograph6b6a',
u'\ub7ed': 'HangulSyllableReog',
u'\u3df1': 'CjkUnifiedIdeograph3df1',
u'\u9c70': 'CjkUnifiedIdeograph9c70',
u'\uf8f3': 'UnicodeCharacter0xf8f3',
u'\u1a74': 'UnicodeCharacter0x1a74',
u'\u66f7': 'CjkUnifiedIdeograph66f7',
u'\uc576': 'HangulSyllableAej',
u'\u437a': 'CjkUnifiedIdeograph437a',
u'\uaffd': 'HangulSyllableGgwelt',
u'\u9501': 'CjkUnifiedIdeograph9501',
u'\uf480': 'UnicodeCharacter0xf480',
u'\u1305': 'EthiopicSyllableJe',
u'\u7284': 'CjkUnifiedIdeograph7284',
u'\ude07': 'UnicodeCharacter0xde07',
u'\u440b': 'CjkUnifiedIdeograph440b',
u'\ubb8a': 'HangulSyllableMwigg',
u'\u218e': 'UnicodeCharacter0x218e',
u'\u8d11': 'CjkUnifiedIdeograph8d11',
u'\uec90': 'UnicodeCharacter0xec90',
u'\u0b15': 'OriyaLetterKa',
u'\u6a94': 'CjkUnifiedIdeograph6a94',
u'\ubc1b': 'HangulSyllableBad',
u'\u939a': 'CjkUnifiedIdeograph939a',
u'\u3a1f': 'CjkUnifiedIdeograph3a1f',
u'\u199e': 'NewTaiLueLetterLowVa',
u'\ue521': 'UnicodeCharacter0xe521',
u'\uc4a0': 'HangulSyllableSswen',
u'\u6325': 'CjkUnifiedIdeograph6325',
u'\u42a4': 'CjkUnifiedIdeograph42a4',
u'\u942b': 'CjkUnifiedIdeograph942b',
u'\u8baa': 'CjkUnifiedIdeograph8baa',
u'\u122f': 'EthiopicSyllableRwa',
u'\u71ae': 'CjkUnifiedIdeograph71ae',
u'\udd31': 'UnicodeCharacter0xdd31',
u'\u5b35': 'CjkUnifiedIdeograph5b35',
u'\ubab4': 'HangulSyllableMols',
u'\u20b8': 'UnicodeCharacter0x20b8',
u'\u8c3b': 'CjkUnifiedIdeograph8c3b',
u'\ue3ba': 'UnicodeCharacter0xe3ba',
u'\u0a3f': 'GurmukhiVowelSignI',
u'\u69be': 'CjkUnifiedIdeograph69be',
u'\ub345': 'HangulSyllableDyaelt',
u'\u92c4': 'CjkUnifiedIdeograph92c4',
u'\u3949': 'CjkUnifiedIdeograph3949',
u'\u18c8': 'UnicodeCharacter0x18c8',
u'\ue44b': 'UnicodeCharacter0xe44b',
u'\udbca': 'UnicodeCharacter0xdbca',
u'\u624f': 'CjkUnifiedIdeograph624f',
u'\u41ce': 'CjkUnifiedIdeograph41ce',
u'\uab55': 'UnicodeCharacter0xab55',
u'\u8ad4': 'CjkUnifiedIdeograph8ad4',
u'\u1159': 'HangulChoseongYeorinhieuh',
u'\u70d8': 'CjkUnifiedIdeograph70d8',
u'\udc5b': 'UnicodeCharacter0xdc5b',
u'\u5a5f': 'CjkUnifiedIdeograph5a5f',
u'\ub9de': 'HangulSyllableMaj',
u'\u27e2': 'WhiteConcavesidedDiamondWithLeftwardsTick',
u'\u8365': 'CjkUnifiedIdeograph8365',
u'\ue2e4': 'UnicodeCharacter0xe2e4',
u'\u0969': 'DevanagariDigitThree',
u'\u68e8': 'CjkUnifiedIdeograph68e8',
u'\ub26f': 'HangulSyllableNwic',
u'\u91ee': 'CjkUnifiedIdeograph91ee',
u'\u3873': 'CjkUnifiedIdeograph3873',
u'\u1ff2': 'GreekSmallLetterOmegaWithVariaAndYpogegrammeni',
u'\ufb75': 'ArabicLetterDyehMedialForm',
u'\udaf4': 'UnicodeCharacter0xdaf4',
u'\u6179': 'CjkUnifiedIdeograph6179',
u'\u40f8': 'CjkUnifiedIdeograph40f8',
u'\uaa7f': 'UnicodeCharacter0xaa7f',
u'\u89fe': 'CjkUnifiedIdeograph89fe',
u'\ufc06': 'ArabicLigatureBehWithHahIsolatedForm',
u'\ud889': 'UnicodeCharacter0xd889',
u'\ub908': 'HangulSyllableRweon',
u'\u858b': 'CjkUnifiedIdeograph858b',
u'\u7a0a': 'CjkUnifiedIdeograph7a0a',
u'\u468d': 'CjkUnifiedIdeograph468d',
u'\u270c': 'VictoryHand',
u'\u038f': 'GreekCapitalLetterOmegaWithTonos',
u'\ud416': 'HangulSyllablePwalp',
u'\u9118': 'CjkUnifiedIdeograph9118',
u'\ufd9b': 'ArabicLigatureNoonWithMeemWithAlefMaksuraFinalForm',
u'\u521a': 'CjkUnifiedIdeograph521a',
u'\ube9d': 'HangulSyllableBbyanj',
u'\u1f1c': 'GreekCapitalLetterEpsilonWithPsiliAndOxia',
u'\u7b9f': 'CjkUnifiedIdeograph7b9f',
u'\u24a1': 'ParenthesizedLatinSmallLetterF',
u'\ucc26': 'HangulSyllableJjip',
u'\u8928': 'CjkUnifiedIdeograph8928',
u'\ud5ab': 'HangulSyllableHyah',
u'\u4a2a': 'CjkUnifiedIdeograph4a2a',
u'\u96ad': 'CjkUnifiedIdeograph96ad',
u'\u772c': 'CjkUnifiedIdeograph772c',
u'\u53af': 'CjkUnifiedIdeograph53af',
u'\u302e': 'HangulSingleDotToneMark',
u'\u1cb1': 'UnicodeCharacter0x1cb1',
u'\ue138': 'UnicodeCharacter0xe138',
u'\ucdbb': 'HangulSyllableCweolb',
u'\ua23a': 'YiSyllableMgox',
u'\u8ebd': 'CjkUnifiedIdeograph8ebd',
u'\u6f3c': 'CjkUnifiedIdeograph6f3c',
u'\u4bbf': 'CjkUnifiedIdeograph4bbf',
u'\u283e': 'BraillePatternDots23456',
u'\u74c1': 'CjkUnifiedIdeograph74c1',
u'\u31c3': 'CjkStrokeBxg',
u'\ud948': 'UnicodeCharacter0xd948',
u'\u9a4a': 'CjkUnifiedIdeograph9a4a',
u'\ue6cd': 'UnicodeCharacter0xe6cd',
u'\u474c': 'CjkUnifiedIdeograph474c',
u'\ua3cf': 'YiSyllableRo',
u'\u6cd1': 'CjkUnifiedIdeograph6cd1',
u'\u29d3': 'BlackBowtie',
u'\uf25a': 'UnicodeCharacter0xf25a',
u'\udedd': 'UnicodeCharacter0xdedd',
u'\ubf5c': 'HangulSyllableBbwa',
u'\u9bdf': 'CjkUnifiedIdeograph9bdf',
u'\u785e': 'CjkUnifiedIdeograph785e',
u'\u44e1': 'CjkUnifiedIdeograph44e1',
u'\u2560': 'BoxDrawingsDoubleVerticalAndRight',
u'\u01e3': 'LatinSmallLetterAeWithMacron',
u'\uea6a': 'UnicodeCharacter0xea6a',
u'\u976c': 'CjkUnifiedIdeograph976c',
u'\uf3ef': 'UnicodeCharacter0xf3ef',
u'\u506e': 'CjkUnifiedIdeograph506e',
u'\ubcf1': 'HangulSyllableByet',
u'\u1d70': 'LatinSmallLetterNWithMiddleTilde',
u'\u79f3': 'CjkUnifiedIdeograph79f3',
u'\u3af5': 'CjkUnifiedIdeograph3af5',
u'\uc27a': 'HangulSyllableSwilp',
u'\u8f7c': 'CjkUnifiedIdeograph8f7c',
u'\uebff': 'UnicodeCharacter0xebff',
u'\u487e': 'CjkUnifiedIdeograph487e',
u'\u7580': 'CjkUnifiedIdeograph7580',
u'\ud103': 'HangulSyllableTyalb',
u'\u3682': 'CjkUnifiedIdeograph3682',
u'\u9205': 'CjkUnifiedIdeograph9205',
u'\u5f07': 'CjkUnifiedIdeograph5f07',
u'\u1809': 'MongolianManchuFullStop',
u'\ue78c': 'UnicodeCharacter0xe78c',
u'\ua08e': 'YiSyllableNbyp',
u'\u6d90': 'CjkUnifiedIdeograph6d90',
u'\uc913': 'HangulSyllableJuc',
u'\u2e92': 'CjkRadicalSnake',
u'\u8a15': 'CjkUnifiedIdeograph8a15',
u'\ub717': 'HangulSyllableDdyulb',
u'\u7019': 'CjkUnifiedIdeograph7019',
u'\u3d1b': 'CjkUnifiedIdeograph3d1b',
u'\udf9c': 'UnicodeCharacter0xdf9c',
u'\u989e': 'CjkUnifiedIdeograph989e',
u'\u45a0': 'CjkUnifiedIdeograph45a0',
u'\u06a2': 'ArabicLetterFehWithDotMovedBelow',
u'\ue225': 'UnicodeCharacter0xe225',
u'\uaf27': 'HangulSyllableGgyec',
u'\u6829': 'CjkUnifiedIdeograph6829',
u'\u152b': 'CanadianSyllabicsYoo',
u'\uf0ae': 'UnicodeCharacter0xf0ae',
u'\ubdb0': 'HangulSyllableBweoss',
u'\u7eb2': 'CjkUnifiedIdeograph7eb2',
u'\uda35': 'UnicodeCharacter0xda35',
u'\u3bb4': 'CjkUnifiedIdeograph3bb4',
u'\u8737': 'CjkUnifiedIdeograph8737',
u'\u4039': 'CjkUnifiedIdeograph4039',
u'\u0d3b': 'UnicodeCharacter0xd3b',
u'\ue8be': 'UnicodeCharacter0xe8be',
u'\u95c0': 'CjkUnifiedIdeograph95c0',
u'\u56c2': 'CjkUnifiedIdeograph56c2',
u'\u13c4': 'CherokeeLetterNu',
u'\uff47': 'FullwidthLatinSmallLetterG',
u'\ub849': 'HangulSyllableRyelg',
u'\u654b': 'CjkUnifiedIdeograph654b',
u'\u264d': 'Virgo',
u'\uc0ce': 'HangulSyllableSaenh',
u'\u8dd0': 'CjkUnifiedIdeograph8dd0',
u'\u4ed2': 'CjkUnifiedIdeograph4ed2',
u'\u0bd4': 'UnicodeCharacter0xbd4',
u'\ud757': 'HangulSyllableHeud',
u'\u34d6': 'CjkUnifiedIdeograph34d6',
u'\u9059': 'CjkUnifiedIdeograph9059',
u'\u5d5b': 'CjkUnifiedIdeograph5d5b',
u'\u1e5d': 'LatinSmallLetterRWithDotBelowAndMacron',
u'\ue5e0': 'UnicodeCharacter0xe5e0',
u'\ua6e2': 'UnicodeCharacter0xa6e2',
u'\u63e4': 'CjkUnifiedIdeograph63e4',
u'\ucf67': 'HangulSyllableKos',
u'\u2ce6': 'CopticSymbolPiRo',
u'\u8869': 'CjkUnifiedIdeograph8869',
u'\ub56b': 'HangulSyllableDdyags',
u'\u766d': 'CjkUnifiedIdeograph766d',
u'\u336f': 'IdeographicTelegraphSymbolForHourTwentythree',
u'\uddf0': 'UnicodeCharacter0xddf0',
u'\u9ef2': 'CjkUnifiedIdeograph9ef2',
u'\u5bf4': 'CjkUnifiedIdeograph5bf4',
u'\u04f6': 'CyrillicCapitalLetterGheWithDescender',
u'\ue079': 'UnicodeCharacter0xe079',
u'\uad7b': 'HangulSyllableGulh',
u'\u6e7d': 'CjkUnifiedIdeograph6e7d',
u'\u2b7f': 'UnicodeCharacter0x2b7f',
u'\u5781': 'CjkUnifiedIdeograph5781',
u'\ub004': 'HangulSyllableGgwess',
u'\u9c87': 'CjkUnifiedIdeograph9c87',
u'\u3a08': 'CjkUnifiedIdeograph3a08',
u'\u068b': 'ArabicLetterDalWithDotBelowAndSmallTah',
u'\uf90a': 'CjkCompatibilityIdeographf90a',
u'\uc58d': 'HangulSyllableYab',
u'\u630e': 'CjkUnifiedIdeograph630e',
u'\u4f91': 'CjkUnifiedIdeograph4f91',
u'\ua814': 'SylotiNagriLetterTo',
u'\uf497': 'UnicodeCharacter0xf497',
u'\u1218': 'EthiopicSyllableMa',
u'\u7e9b': 'CjkUnifiedIdeograph7e9b',
u'\ud11a': 'HangulSyllableTyaenh',
u'\u5b1e': 'CjkUnifiedIdeograph5b1e',
u'\ua7a1': 'UnicodeCharacter0xa7a1',
u'\u21a5': 'UpwardsArrowFromBar',
u'\u8024': 'CjkUnifiedIdeograph8024',
u'\ueca7': 'UnicodeCharacter0xeca7',
u'\u0a28': 'GurmukhiLetterNa',
u'\u56ab': 'CjkUnifiedIdeograph56ab',
u'\uc92a': 'HangulSyllableJweobs',
u'\ub32e': 'HangulSyllableDyabs',
u'\u9fb1': 'CjkUnifiedIdeograph9fb1',
u'\u3d32': 'CjkUnifiedIdeograph3d32',
u'\u19b5': 'NewTaiLueVowelSignE',
u'\uf834': 'UnicodeCharacter0xf834',
u'\uc4b7': 'HangulSyllableSsweh',
u'\u6238': 'CjkUnifiedIdeograph6238',
u'\u4ebb': 'CjkUnifiedIdeograph4ebb',
u'\uab3e': 'UnicodeCharacter0xab3e',
u'\uf7c1': 'UnicodeCharacter0xf7c1',
u'\u1542': 'CanadianSyllabicsRe',
u'\u71c5': 'CjkUnifiedIdeograph71c5',
u'\ud044': 'HangulSyllableKwim',
u'\u5a48': 'CjkUnifiedIdeograph5a48',
u'\ua6cb': 'UnicodeCharacter0xa6cb',
u'\u20cf': 'UnicodeCharacter0x20cf',
u'\u834e': 'CjkUnifiedIdeograph834e',
u'\uefd1': 'UnicodeCharacter0xefd1',
u'\u0d52': 'UnicodeCharacter0xd52',
u'\u69d5': 'CjkUnifiedIdeograph69d5',
u'\uc854': 'HangulSyllableJye',
u'\ub258': 'HangulSyllableNwi',
u'\u9edb': 'CjkUnifiedIdeograph9edb',
u'\u3c5c': 'CjkUnifiedIdeograph3c5c',
u'\u18df': 'UnicodeCharacter0x18df',
u'\ufb5e': 'ArabicLetterTtehehIsolatedForm',
u'\uc7e1': 'HangulSyllableJyat',
u'\u6562': 'CjkUnifiedIdeograph6562',
u'\u41e5': 'CjkUnifiedIdeograph41e5',
u'\uaa68': 'UnicodeCharacter0xaa68',
u'\uf6eb': 'UnicodeCharacter0xf6eb',
u'\u146c': 'CanadianSyllabicsKaai',
u'\u70ef': 'CjkUnifiedIdeograph70ef',
u'\ud36e': 'HangulSyllablePyaelp',
u'\u5d72': 'CjkUnifiedIdeograph5d72',
u'\ub9f5': 'HangulSyllableMaeb',
u'\u23f9': 'UnicodeCharacter0x23f9',
u'\u8278': 'CjkUnifiedIdeograph8278',
u'\ueefb': 'UnicodeCharacter0xeefb',
u'\u0c7c': 'TeluguFractionDigitOneForEvenPowersOfFour',
u'\u68ff': 'CjkUnifiedIdeograph68ff',
u'\ucb7e': 'HangulSyllableJjweop',
u'\u5203': 'CjkUnifiedIdeograph5203',
u'\ub582': 'HangulSyllableDdyap',
u'\u3f86': 'CjkUnifiedIdeograph3f86',
u'\u9b09': 'CjkUnifiedIdeograph9b09',
u'\ufa88': 'CjkCompatibilityIdeographfa88',
u'\u050d': 'CyrillicSmallLetterKomiSje',
u'\u648c': 'CjkUnifiedIdeograph648c',
u'\uc00f': 'HangulSyllableBbwelb',
u'\u4a13': 'CjkUnifiedIdeograph4a13',
u'\uad92': 'HangulSyllableGweolm',
u'\u3417': 'CjkUnifiedIdeograph3417',
u'\u1796': 'KhmerLetterPo',
u'\uf319': 'UnicodeCharacter0xf319',
u'\ud298': 'HangulSyllableTwik',
u'\u7d1d': 'CjkUnifiedIdeograph7d1d',
u'\u5c9c': 'CjkUnifiedIdeograph5c9c',
u'\ua223': 'YiSyllableGgox',
u'\u85a2': 'CjkUnifiedIdeograph85a2',
u'\u2c27': 'GlagoliticCapitalLetterIotatedSmallYus',
u'\u0fa6': 'TibetanSubjoinedLetterBa',
u'\ueb29': 'UnicodeCharacter0xeb29',
u'\ucaa8': 'HangulSyllableJjyel',
u'\u552d': 'CjkUnifiedIdeograph552d',
u'\ub4ac': 'HangulSyllableDwil',
u'\u3eb0': 'CjkUnifiedIdeograph3eb0',
u'\u9a33': 'CjkUnifiedIdeograph9a33',
u'\ufdb2': 'ArabicLigatureQafWithMeemWithYehFinalForm',
u'\u0437': 'CyrillicSmallLetterZe',
u'\u67b6': 'CjkUnifiedIdeograph67b6',
u'\uc339': 'HangulSyllableSsyalg',
u'\u4d3d': 'CjkUnifiedIdeograph4d3d',
u'\uacbc': 'HangulSyllableGyeoss',
u'\u3741': 'CjkUnifiedIdeograph3741',
u'\u16c0': 'RunicLetterDottedn',
u'\uf243': 'UnicodeCharacter0xf243',
u'\ud5c2': 'HangulSyllableHyaej',
u'\u7c47': 'CjkUnifiedIdeograph7c47',
u'\u5fc6': 'CjkUnifiedIdeograph5fc6',
u'\ua54d': 'VaiSyllableHan',
u'\u84cc': 'CjkUnifiedIdeograph84cc',
u'\u2f51': 'KangxiRadicalFur',
u'\u0ed0': 'LaoDigitZero',
u'\uea53': 'UnicodeCharacter0xea53',
u'\ucdd2': 'HangulSyllableCwenh',
u'\u5457': 'CjkUnifiedIdeograph5457',
u'\ub7d6': 'HangulSyllableRyaenh',
u'\u31da': 'CjkStrokeSg',
u'\u9d5d': 'CjkUnifiedIdeograph9d5d',
u'\ufcdc': 'ArabicLigatureYehWithKhahInitialForm',
u'\u0761': 'ArabicLetterFehWithThreeDotsPointingUpwardsBelow',
u'\u66e0': 'CjkUnifiedIdeograph66e0',
u'\uc263': 'HangulSyllableSwes',
u'\u4c67': 'CjkUnifiedIdeograph4c67',
u'\uafe6': 'HangulSyllableGgweobs',
u'\u366b': 'CjkUnifiedIdeograph366b',
u'\u29ea': 'BlackDiamondWithDownArrow',
u'\uf56d': 'UnicodeCharacter0xf56d',
u'\ud4ec': 'HangulSyllablePyun',
u'\u7f71': 'CjkUnifiedIdeograph7f71',
u'\u5ef0': 'CjkUnifiedIdeograph5ef0',
u'\ua477': 'YiSyllableYie',
u'\u87f6': 'CjkUnifiedIdeograph87f6',
u'\u2e7b': 'UnicodeCharacter0x2e7b',
u'\u01fa': 'LatinCapitalLetterAWithRingAboveAndAcute',
u'\ued7d': 'UnicodeCharacter0xed7d',
u'\uccfc': 'HangulSyllableCyem',
u'\ud681': 'HangulSyllableHwaeb',
u'\ub700': 'HangulSyllableDdwim',
u'\u9383': 'CjkUnifiedIdeograph9383',
u'\u7402': 'CjkUnifiedIdeograph7402',
u'\u5085': 'CjkUnifiedIdeograph5085',
u'\u3104': 'UnicodeCharacter0x3104',
u'\u1d87': 'LatinSmallLetterNWithPalatalHook',
u'\ue20e': 'UnicodeCharacter0xe20e',
u'\uce91': 'HangulSyllableKaeg',
u'\uaf10': 'HangulSyllableGgye',
u'\u8b93': 'CjkUnifiedIdeograph8b93',
u'\u6c12': 'CjkUnifiedIdeograph6c12',
u'\u4895': 'CjkUnifiedIdeograph4895',
u'\u2914': 'RightwardsArrowWithTailWithVerticalStroke',
u'\u7597': 'CjkUnifiedIdeograph7597',
u'\u3299': 'CircledIdeographSecret',
u'\uda1e': 'UnicodeCharacter0xda1e',
u'\u8720': 'CjkUnifiedIdeograph8720',
u'\ue3a3': 'UnicodeCharacter0xe3a3',
u'\u4422': 'CjkUnifiedIdeograph4422',
u'\ua0a5': 'YiSyllableHmu',
u'\u0124': 'LatinCapitalLetterHWithCircumflex',
u'\u6da7': 'CjkUnifiedIdeograph6da7',
u'\u2aa9': 'GreaterthanClosedByCurveAboveSlantedEqual',
u'\uff30': 'FullwidthLatinCapitalLetterP',
u'\udbb3': 'UnicodeCharacter0xdbb3',
u'\ubc32': 'HangulSyllableBaegg',
u'\u98b5': 'CjkUnifiedIdeograph98b5',
u'\u7934': 'CjkUnifiedIdeograph7934',
u'\u45b7': 'CjkUnifiedIdeograph45b7',
u'\u2636': 'TrigramForMountain',
u'\u02b9': 'ModifierLetterPrime',
u'\ud740': 'HangulSyllableHyuls',
u'\u9442': 'CjkUnifiedIdeograph9442',
u'\uf0c5': 'UnicodeCharacter0xf0c5',
u'\u5144': 'CjkUnifiedIdeograph5144',
u'\ubdc7': 'HangulSyllableBwelh',
u'\u1e46': 'LatinCapitalLetterNWithDotBelow',
u'\u7ac9': 'CjkUnifiedIdeograph7ac9',
u'\u27cb': 'UnicodeCharacter0x27cb',
u'\ucf50': 'HangulSyllableKyek',
u'\u8c52': 'CjkUnifiedIdeograph8c52',
u'\ue8d5': 'UnicodeCharacter0xe8d5',
u'\u4954': 'CjkUnifiedIdeograph4954',
u'\u95d7': 'CjkUnifiedIdeograph95d7',
u'\u7656': 'CjkUnifiedIdeograph7656',
u'\u52d9': 'CjkUnifiedIdeograph52d9',
u'\u3358': 'IdeographicTelegraphSymbolForHourZero',
u'\u1fdb': 'GreekCapitalLetterIotaWithOxia',
u'\ue462': 'UnicodeCharacter0xe462',
u'\uc0e5': 'HangulSyllableSyag',
u'\ua164': 'YiSyllableNdu',
u'\u8de7': 'CjkUnifiedIdeograph8de7',
u'\u6e66': 'CjkUnifiedIdeograph6e66',
u'\u4ae9': 'CjkUnifiedIdeograph4ae9',
u'\u2b68': 'UnicodeCharacter0x2b68',
u'\u77eb': 'CjkUnifiedIdeograph77eb',
u'\u34ed': 'CjkUnifiedIdeograph34ed',
u'\udc72': 'UnicodeCharacter0xdc72',
u'\u9974': 'CjkUnifiedIdeograph9974',
u'\ue5f7': 'UnicodeCharacter0xe5f7',
u'\u4676': 'CjkUnifiedIdeograph4676',
u'\ua2f9': 'YiSyllableNzuo',
u'\u0378': 'UnicodeCharacter0x378',
u'\u6ffb': 'CjkUnifiedIdeograph6ffb',
u'\u2cfd': 'CopticFractionOneHalf',
u'\u1601': 'CanadianSyllabicsCarrierKk',
u'\uf184': 'UnicodeCharacter0xf184',
u'\ube86': 'HangulSyllableBbaelm',
u'\u7b88': 'CjkUnifiedIdeograph7b88',
u'\uc70b': 'HangulSyllableWid',
u'\u388a': 'CjkUnifiedIdeograph388a',
u'\u840d': 'CjkUnifiedIdeograph840d',
u'\u410f': 'CjkUnifiedIdeograph410f',
u'\u0e11': 'ThaiCharacterThoNangmontho',
u'\ue994': 'UnicodeCharacter0xe994',
u'\u9696': 'CjkUnifiedIdeograph9696',
u'\u5398': 'CjkUnifiedIdeograph5398',
u'\u109a': 'UnicodeCharacter0x109a',
u'\ufc1d': 'ArabicLigatureSeenWithHahIsolatedForm',
u'\ub91f': 'HangulSyllableRweoh',
u'\u6621': 'CjkUnifiedIdeograph6621',
u'\u2323': 'Smile',
u'\uc1a4': 'HangulSyllableSok',
u'\u8ea6': 'CjkUnifiedIdeograph8ea6',
u'\u4ba8': 'CjkUnifiedIdeograph4ba8',
u'\u08aa': 'UnicodeCharacter0x8aa',
u'\ud42d': 'HangulSyllablePwaelg',
u'\u35ac': 'CjkUnifiedIdeograph35ac',
u'\u912f': 'CjkUnifiedIdeograph912f',
u'\u5e31': 'CjkUnifiedIdeograph5e31',
u'\u1b33': 'BalineseLetterHa',
u'\ue6b6': 'UnicodeCharacter0xe6b6',
u'\ua3b8': 'YiSyllableShe',
u'\u60ba': 'CjkUnifiedIdeograph60ba',
u'\ucc3d': 'HangulSyllableCang',
u'\u2dbc': 'EthiopicSyllableCchee',
u'\u893f': 'CjkUnifiedIdeograph893f',
u'\ub641': 'HangulSyllableDdwang',
u'\u7343': 'CjkUnifiedIdeograph7343',
u'\u3045': 'HiraganaLetterSmallU',
u'\udec6': 'UnicodeCharacter0xdec6',
u'\u9bc8': 'CjkUnifiedIdeograph9bc8',
u'\u58ca': 'CjkUnifiedIdeograph58ca',
u'\u05cc': 'UnicodeCharacter0x5cc',
u'\ue14f': 'UnicodeCharacter0xe14f',
u'\uae51': 'HangulSyllableGganj',
u'\u6b53': 'CjkUnifiedIdeograph6b53',
u'\u2855': 'BraillePatternDots1357',
u'\uf3d8': 'UnicodeCharacter0xf3d8',
u'\ub0da': 'HangulSyllableNyalm',
u'\u7ddc': 'CjkUnifiedIdeograph7ddc',
u'\ud95f': 'UnicodeCharacter0xd95f',
u'\u3ade': 'CjkUnifiedIdeograph3ade',
u'\u8661': 'CjkUnifiedIdeograph8661',
u'\u4363': 'CjkUnifiedIdeograph4363',
u'\uebe8': 'UnicodeCharacter0xebe8',
u'\ua8ea': 'UnicodeCharacter0xa8ea',
u'\u55ec': 'CjkUnifiedIdeograph55ec',
u'\u12ee': 'EthiopicSyllableYo',
u'\ufe71': 'ArabicTatweelWithFathatanAbove',
u'\ubb73': 'HangulSyllableMwed',
u'\u7875': 'CjkUnifiedIdeograph7875',
u'\u2577': 'BoxDrawingsLightDown',
u'\uc3f8': 'HangulSyllableSswan',
u'\u80fa': 'CjkUnifiedIdeograph80fa',
u'\u4dfc': 'HexagramForInnerTruth',
u'\u0afe': 'UnicodeCharacter0xafe',
u'\u3800': 'CjkUnifiedIdeograph3800',
u'\u1c83': 'UnicodeCharacter0x1c83',
u'\ufb02': 'LatinSmallLigatureFl',
u'\udf85': 'UnicodeCharacter0xdf85',
u'\u7106': 'CjkUnifiedIdeograph7106',
u'\u5589': 'CjkUnifiedIdeograph5589',
u'\uaa0c': 'ChamLetterCha',
u'\u8e8f': 'CjkUnifiedIdeograph8e8f',
u'\u2010': 'Hyphen',
u'\u0493': 'CyrillicSmallLetterGheWithStroke',
u'\ue312': 'UnicodeCharacter0xe312',
u'\uc795': 'HangulSyllableJanj',
u'\u5916': 'CjkUnifiedIdeograph5916',
u'\ubd99': 'HangulSyllableBut',
u'\u339d': 'SquareCm',
u'\u921c': 'CjkUnifiedIdeograph921c',
u'\uf69f': 'UnicodeCharacter0xf69f',
u'\u0820': 'UnicodeCharacter0x820',
u'\u6ca3': 'CjkUnifiedIdeograph6ca3',
u'\ucb22': 'HangulSyllableJjoebs',
u'\u4126': 'CjkUnifiedIdeograph4126',
u'\ua5a9': 'VaiSyllableRu',
u'\u1bad': 'UnicodeCharacter0x1bad',
u'\ufa2c': 'CjkCompatibilityIdeographfa2c',
u'\udeaf': 'UnicodeCharacter0xdeaf',
u'\u7030': 'CjkUnifiedIdeograph7030',
u'\u54b3': 'CjkUnifiedIdeograph54b3',
u'\ua936': 'RejangLetterPa',
u'\u8db9': 'CjkUnifiedIdeograph8db9',
u'\u2f3a': 'KangxiRadicalBristle',
u'\u03bd': 'GreekSmallLetterNu',
u'\ue23c': 'UnicodeCharacter0xe23c',
u'\uc6bf': 'HangulSyllableUlh',
u'\u5840': 'CjkUnifiedIdeograph5840',
u'\ubcc3': 'HangulSyllableByeod',
u'\u32c7': 'IdeographicTelegraphSymbolForAugust',
u'\u9146': 'CjkUnifiedIdeograph9146',
u'\uf5c9': 'UnicodeCharacter0xf5c9',
u'\u174a': 'BuhidLetterBa',
u'\u6bcd': 'CjkUnifiedIdeograph6bcd',
u'\uca4c': 'HangulSyllableJjeo',
u'\u4050': 'CjkUnifiedIdeograph4050',
u'\ua4d3': 'UnicodeCharacter0xa4d3',
u'\u1ad7': 'UnicodeCharacter0x1ad7',
u'\uf956': 'CjkCompatibilityIdeographf956',
u'\uddd9': 'UnicodeCharacter0xddd9',
u'\u7f5a': 'CjkUnifiedIdeograph7f5a',
u'\u53dd': 'CjkUnifiedIdeograph53dd',
u'\ua860': 'PhagspaLetterE',
u'\u8ce3': 'CjkUnifiedIdeograph8ce3',
u'\u2e64': 'UnicodeCharacter0x2e64',
u'\u02e7': 'ModifierLetterMidToneBar',
u'\ue166': 'UnicodeCharacter0xe166',
u'\uc5e9': 'HangulSyllableEt',
u'\u676a': 'CjkUnifiedIdeograph676a',
u'\ubbed': 'HangulSyllableMyib',
u'\u31f1': 'KatakanaLetterSmallSi',
u'\u9070': 'CjkUnifiedIdeograph9070',
u'\uf4f3': 'UnicodeCharacter0xf4f3',
u'\u1674': 'CanadianSyllabicsNngoo',
u'\u6af7': 'CjkUnifiedIdeograph6af7',
u'\uc976': 'HangulSyllableJyulm',
u'\u4f7a': 'CjkUnifiedIdeograph4f7a',
u'\ua3fd': 'YiSyllableQix',
u'\u9901': 'CjkUnifiedIdeograph9901',
u'\uf880': 'UnicodeCharacter0xf880',
u'\u1f05': 'GreekSmallLetterAlphaWithDasiaAndOxia',
u'\u7e84': 'CjkUnifiedIdeograph7e84',
u'\ud207': 'HangulSyllableToes',
u'\u480b': 'CjkUnifiedIdeograph480b',
u'\ub78a': 'HangulSyllableRalp',
u'\u2d8e': 'EthiopicSyllableJoa',
u'\u8111': 'CjkUnifiedIdeograph8111',
u'\ue090': 'UnicodeCharacter0xe090',
u'\u0715': 'SyriacLetterDalath',
u'\u6694': 'CjkUnifiedIdeograph6694',
u'\ub01b': 'HangulSyllableGgwilh',
u'\u9f9a': 'CjkUnifiedIdeograph9f9a',
u'\u361f': 'CjkUnifiedIdeograph361f',
u'\u159e': 'CanadianSyllabicsWoodscreeTha',
u'\ue921': 'UnicodeCharacter0xe921',
u'\uc8a0': 'HangulSyllableJwass',
u'\u6f25': 'CjkUnifiedIdeograph6f25',
u'\u4ea4': 'CjkUnifiedIdeograph4ea4',
u'\u982b': 'CjkUnifiedIdeograph982b',
u'\u87aa': 'CjkUnifiedIdeograph87aa',
u'\u1e2f': 'LatinSmallLetterIWithDiaeresisAndAcute',
u'\u7dae': 'CjkUnifiedIdeograph7dae',
u'\ud131': 'HangulSyllableTeog',
u'\u5735': 'CjkUnifiedIdeograph5735',
u'\ub6b4': 'HangulSyllableDduk',
u'\u2cb8': 'CopticCapitalLetterDialectpKapa',
u'\u803b': 'CjkUnifiedIdeograph803b',
u'\uefba': 'UnicodeCharacter0xefba',
u'\u063f': 'ArabicLetterFarsiYehWithThreeDotsAbove',
u'\u65be': 'CjkUnifiedIdeograph65be',
u'\ubf45': 'HangulSyllableBbonj',
u'\u9ec4': 'CjkUnifiedIdeograph9ec4',
u'\u3549': 'CjkUnifiedIdeograph3549',
u'\u14c8': 'CanadianSyllabicsNaa',
u'\ue84b': 'UnicodeCharacter0xe84b',
u'\ud7ca': 'UnicodeCharacter0xd7ca',
u'\u6e4f': 'CjkUnifiedIdeograph6e4f',
u'\u4dce': 'HexagramForModesty',
u'\ua755': 'LatinSmallLetterPWithSquirrelTail',
u'\u86d4': 'CjkUnifiedIdeograph86d4',
u'\u1d59': 'ModifierLetterSmallSidewaysU',
u'\u7cd8': 'CjkUnifiedIdeograph7cd8',
u'\ud05b': 'HangulSyllableKyulb',
u'\u565f': 'CjkUnifiedIdeograph565f',
u'\ub5de': 'HangulSyllableDdyeonh',
u'\u2be2': 'UnicodeCharacter0x2be2',
u'\u8f65': 'CjkUnifiedIdeograph8f65',
u'\ueee4': 'UnicodeCharacter0xeee4',
u'\u0569': 'ArmenianSmallLetterTo',
u'\u64e8': 'CjkUnifiedIdeograph64e8',
u'\ube6f': 'HangulSyllableBbalh',
u'\u9dee': 'CjkUnifiedIdeograph9dee',
u'\u3473': 'CjkUnifiedIdeograph3473',
u'\u13f2': 'CherokeeLetterYo',
u'\uf775': 'UnicodeCharacter0xf775',
u'\ud6f4': 'HangulSyllableHweoss',
u'\u6d79': 'CjkUnifiedIdeograph6d79',
u'\u4cf8': 'CjkUnifiedIdeograph4cf8',
u'\ua67f': 'CyrillicPayerok',
u'\u85fe': 'CjkUnifiedIdeograph85fe',
u'\uf006': 'UnicodeCharacter0xf006',
u'\ud489': 'HangulSyllablePub',
u'\ub508': 'HangulSyllableDyim',
u'\u898b': 'CjkUnifiedIdeograph898b',
u'\u760a': 'CjkUnifiedIdeograph760a',
u'\u4a8d': 'CjkUnifiedIdeograph4a8d',
u'\u2b0c': 'LeftRightBlackArrow',
u'\u0f8f': 'UnicodeCharacter0xf8f',
u'\ud816': 'UnicodeCharacter0xd816',
u'\u9d18': 'CjkUnifiedIdeograph9d18',
u'\uf19b': 'UnicodeCharacter0xf19b',
u'\u5e1a': 'CjkUnifiedIdeograph5e1a',
u'\ub29d': 'HangulSyllableNeult',
u'\u131c': 'EthiopicSyllableGgee',
u'\u779f': 'CjkUnifiedIdeograph779f',
u'\u28a1': 'BraillePatternDots168',
u'\uc026': 'HangulSyllableBbwinh',
u'\u8528': 'CjkUnifiedIdeograph8528',
u'\ud9ab': 'UnicodeCharacter0xd9ab',
u'\u462a': 'CjkUnifiedIdeograph462a',
u'\u9aad': 'CjkUnifiedIdeograph9aad',
u'\u7b2c': 'CjkUnifiedIdeograph7b2c',
u'\u5faf': 'CjkUnifiedIdeograph5faf',
u'\u3c2e': 'CjkUnifiedIdeograph3c2e',
u'\u10b1': 'GeorgianCapitalLetterSan',
u'\ued38': 'UnicodeCharacter0xed38',
u'\uc1bb': 'HangulSyllableSwas',
u'\uae3a': 'HangulSyllableGilm',
u'\u82bd': 'CjkUnifiedIdeograph82bd',
u'\u633c': 'CjkUnifiedIdeograph633c',
u'\u47bf': 'CjkUnifiedIdeograph47bf',
u'\u243e': 'UnicodeCharacter0x243e',
u'\u78c1': 'CjkUnifiedIdeograph78c1',
u'\u3dc3': 'CjkUnifiedIdeograph3dc3',
u'\ud548': 'HangulSyllablePils',
u'\u964a': 'CjkUnifiedIdeograph964a',
u'\ueacd': 'UnicodeCharacter0xeacd',
u'\u4b4c': 'CjkUnifiedIdeograph4b4c',
u'\uafcf': 'HangulSyllableGguc',
u'\u0c4e': 'UnicodeCharacter0xc4e',
u'\u60d1': 'CjkUnifiedIdeograph60d1',
u'\u25d3': 'CircleWithUpperHalfBlack',
u'\ufe5a': 'SmallRightParenthesis',
u'\ud2dd': 'HangulSyllableTyilg',
u'\ub35c': 'HangulSyllableDeol',
u'\u97df': 'CjkUnifiedIdeograph97df',
u'\u745e': 'CjkUnifiedIdeograph745e',
u'\u48e1': 'CjkUnifiedIdeograph48e1',
u'\u2960': 'UpwardsHarpoonWithBarbLeftFromBar',
u'\u0de3': 'UnicodeCharacter0xde3',
u'\ue66a': 'UnicodeCharacter0xe66a',
u'\u9b6c': 'CjkUnifiedIdeograph9b6c',
u'\uffef': 'UnicodeCharacter0xffef',
u'\u5c6e': 'CjkUnifiedIdeograph5c6e',
u'\ub0f1': 'HangulSyllableNyaenj',
u'\u1170': 'HangulJungseongWe',
u'\u75f3': 'CjkUnifiedIdeograph75f3',
u'\u36f5': 'CjkUnifiedIdeograph36f5',
u'\uce7a': 'HangulSyllableKanh',
u'\u837c': 'CjkUnifiedIdeograph837c',
u'\ue7ff': 'UnicodeCharacter0xe7ff',
u'\u447e': 'CjkUnifiedIdeograph447e',
u'\u7980': 'CjkUnifiedIdeograph7980',
u'\udd03': 'UnicodeCharacter0xdd03',
u'\u3a82': 'CjkUnifiedIdeograph3a82',
u'\u9e05': 'CjkUnifiedIdeograph9e05',
u'\u5307': 'CjkUnifiedIdeograph5307',
u'\u1409': 'CanadianSyllabicsCarrierI',
u'\ueb8c': 'UnicodeCharacter0xeb8c',
u'\uac8e': 'HangulSyllableGegg',
u'\u6190': 'CjkUnifiedIdeograph6190',
u'\uc513': 'HangulSyllableSsyid',
u'\u2292': 'SquareOriginalOfOrEqualTo',
u'\u8615': 'CjkUnifiedIdeograph8615',
u'\ubb17': 'HangulSyllableMoeh',
u'\u7c19': 'CjkUnifiedIdeograph7c19',
u'\u311b': 'BopomofoLetterO',
u'\ud39c': 'HangulSyllablePen',
u'\u949e': 'CjkUnifiedIdeograph949e',
u'\u49a0': 'CjkUnifiedIdeograph49a0',
u'\u0aa2': 'GujaratiLetterDdha',
u'\uee25': 'UnicodeCharacter0xee25',
u'\ua327': 'YiSyllableSyp',
u'\u6429': 'CjkUnifiedIdeograph6429',
u'\u192b': 'LimbuSubjoinedLetterWa',
u'\ufcae': 'ArabicLigatureSeenWithHahInitialForm',
u'\ub1b0': 'HangulSyllableNwae',
u'\u72b2': 'CjkUnifiedIdeograph72b2',
u'\ud635': 'HangulSyllableHyet',
u'\u37b4': 'CjkUnifiedIdeograph37b4',
u'\u8b37': 'CjkUnifiedIdeograph8b37',
u'\u4c39': 'CjkUnifiedIdeograph4c39',
u'\u013b': 'LatinCapitalLetterLWithCedilla',
u'\ue4be': 'UnicodeCharacter0xe4be',
u'\u99c0': 'CjkUnifiedIdeograph99c0',
u'\u5ac2': 'CjkUnifiedIdeograph5ac2',
u'\u1fc4': 'GreekSmallLetterEtaWithOxiaAndYpogegrammeni',
u'\uf347': 'UnicodeCharacter0xf347',
u'\ub449': 'HangulSyllableDyong',
u'\u694b': 'CjkUnifiedIdeograph694b',
u'\u2a4d': 'ClosedIntersectionWithSerifs',
u'\uccce': 'HangulSyllableCep',
u'\u81d0': 'CjkUnifiedIdeograph81d0',
u'\u42d2': 'CjkUnifiedIdeograph42d2',
u'\u07d4': 'NkoLetterPa',
u'\udb57': 'UnicodeCharacter0xdb57',
u'\u38d6': 'CjkUnifiedIdeograph38d6',
u'\u9c59': 'CjkUnifiedIdeograph9c59',
u'\u515b': 'CjkUnifiedIdeograph515b',
u'\u125d': 'EthiopicSyllableQhwe',
u'\ue9e0': 'UnicodeCharacter0xe9e0',
u'\uaae2': 'UnicodeCharacter0xaae2',
u'\u6fe4': 'CjkUnifiedIdeograph6fe4',
u'\uc367': 'HangulSyllableSsyaeh',
u'\u20e6': 'CombiningDoubleVerticalStrokeOverlay',
u'\u8469': 'CjkUnifiedIdeograph8469',
u'\ub96b': 'HangulSyllableRyus',
u'\u7a6d': 'CjkUnifiedIdeograph7a6d',
u'\u3f6f': 'CjkUnifiedIdeograph3f6f',
u'\ud1f0': 'HangulSyllableTwaek',
u'\u92f2': 'CjkUnifiedIdeograph92f2',
u'\u57f4': 'CjkUnifiedIdeograph57f4',
u'\u08f6': 'UnicodeCharacter0x8f6',
u'\uec79': 'UnicodeCharacter0xec79',
u'\ua17b': 'YiSyllableHnep',
u'\u627d': 'CjkUnifiedIdeograph627d',
u'\u277f': 'DingbatNegativeCircledNumberTen',
u'\u5b81': 'CjkUnifiedIdeograph5b81',
u'\ubc04': 'HangulSyllableMils',
u'\u9087': 'CjkUnifiedIdeograph9087',
u'\u3608': 'CjkUnifiedIdeograph3608',
u'\u0a8b': 'GujaratiLetterVocalicR',
u'\uf50a': 'UnicodeCharacter0xf50a',
u'\uc98d': 'HangulSyllableJeunj',
u'\u6f0e': 'CjkUnifiedIdeograph6f0e',
u'\u4391': 'CjkUnifiedIdeograph4391',
u'\ua414': 'YiSyllableQy',
u'\uf897': 'UnicodeCharacter0xf897',
u'\u1e18': 'LatinCapitalLetterEWithCircumflexBelow',
u'\u729b': 'CjkUnifiedIdeograph729b',
u'\udd1a': 'UnicodeCharacter0xdd1a',
u'\u571e': 'CjkUnifiedIdeograph571e',
u'\uaba1': 'UnicodeCharacter0xaba1',
u'\u2da5': 'EthiopicSyllableSse',
u'\u8c24': 'CjkUnifiedIdeograph8c24',
u'\ue0a7': 'UnicodeCharacter0xe0a7',
u'\u0628': 'ArabicLetterBeh',
u'\u5aab': 'CjkUnifiedIdeograph5aab',
u'\uc52a': 'HangulSyllableSsigg',
u'\ubf2e': 'HangulSyllableBbyelm',
u'\u93b1': 'CjkUnifiedIdeograph93b1',
u'\u3132': 'HangulLetterSsangkiyeok',
u'\u15b5': 'CanadianSyllabicsBlackfootWi',
u'\uf434': 'UnicodeCharacter0xf434',
u'\uc8b7': 'HangulSyllableJwaelh',
u'\u6e38': 'CjkUnifiedIdeograph6e38',
u'\u42bb': 'CjkUnifiedIdeograph42bb',
u'\ua73e': 'LatinCapitalLetterReversedCWithDot',
u'\ufbc1': 'UnicodeCharacter0xfbc1',
u'\u1942': 'UnicodeCharacter0x1942',
u'\u7dc5': 'CjkUnifiedIdeograph7dc5',
u'\udc44': 'UnicodeCharacter0xdc44',
u'\u5648': 'CjkUnifiedIdeograph5648',
u'\uaacb': 'UnicodeCharacter0xaacb',
u'\u2ccf': 'CopticSmallLetterOldCopticHa',
u'\u8f4e': 'CjkUnifiedIdeograph8f4e',
u'\ue3d1': 'UnicodeCharacter0xe3d1',
u'\u0152': 'LatinCapitalLigatureOe',
u'\u65d5': 'CjkUnifiedIdeograph65d5',
u'\uc454': 'HangulSyllableSsyols',
u'\ube58': 'HangulSyllableBiss',
u'\u92db': 'CjkUnifiedIdeograph92db',
u'\u305c': 'HiraganaLetterZe',
u'\u14df': 'CanadianSyllabicsWestcreeLwi',
u'\uf75e': 'UnicodeCharacter0xf75e',
u'\ucbe1': 'HangulSyllableJjeult',
u'\u6962': 'CjkUnifiedIdeograph6962',
u'\u4de5': 'HexagramForOpposition',
u'\ua668': 'CyrillicCapitalLetterMonocularO',
u'\ufaeb': 'UnicodeCharacter0xfaeb',
u'\u186c': 'MongolianLetterSibeGaa',
u'\u7cef': 'CjkUnifiedIdeograph7cef',
u'\udf6e': 'UnicodeCharacter0xdf6e',
u'\u5172': 'CjkUnifiedIdeograph5172',
u'\ub5f5': 'HangulSyllableDdyeg',
u'\u2ff9': 'IdeographicDescriptionCharacterSurroundFromUpperRight',
u'\u8e78': 'CjkUnifiedIdeograph8e78',
u'\ue2fb': 'UnicodeCharacter0xe2fb',
u'\u64ff': 'CjkUnifiedIdeograph64ff',
u'\uc77e': 'HangulSyllableIlm',
u'\u5e03': 'CjkUnifiedIdeograph5e03',
u'\ub982': 'HangulSyllableReulp',
u'\u3386': 'SquareMb',
u'\u9709': 'CjkUnifiedIdeograph9709',
u'\uf688': 'UnicodeCharacter0xf688',
u'\u090d': 'DevanagariLetterCandraE',
u'\u688c': 'CjkUnifiedIdeograph688c',
u'\ucc0f': 'HangulSyllableJjigs',
u'\u4613': 'CjkUnifiedIdeograph4613',
u'\ua192': 'YiSyllableNux',
u'\u3817': 'CjkUnifiedIdeograph3817',
u'\u1b96': 'SundaneseLetterFa',
u'\uff19': 'FullwidthDigitNine',
u'\ude98': 'UnicodeCharacter0xde98',
u'\u711d': 'CjkUnifiedIdeograph711d',
u'\u509c': 'CjkUnifiedIdeograph509c',
u'\uae23': 'HangulSyllableGyilh',
u'\u89a2': 'CjkUnifiedIdeograph89a2',
u'\u2027': 'HyphenationPoint',
u'\u03a6': 'GreekCapitalLetterPhi',
u'\ue729': 'UnicodeCharacter0xe729',
u'\uc6a8': 'HangulSyllableYoss',
u'\u592d': 'CjkUnifiedIdeograph592d',
u'\ub8ac': 'HangulSyllableRwaek',
u'\u32b0': 'CircledIdeographNight',
u'\u9633': 'CjkUnifiedIdeograph9633',
u'\uf1b2': 'UnicodeCharacter0xf1b2',
u'\u0837': 'UnicodeCharacter0x837',
u'\u6bb6': 'CjkUnifiedIdeograph6bb6',
u'\ucf39': 'HangulSyllableKyeg',
u'\u413d': 'CjkUnifiedIdeograph413d',
u'\ua0bc': 'YiSyllableMuop',
u'\u3b41': 'CjkUnifiedIdeograph3b41',
u'\u1ac0': 'UnicodeCharacter0x1ac0',
u'\ufe43': 'PresentationFormForVerticalLeftWhiteCornerBracket',
u'\ud9c2': 'UnicodeCharacter0xd9c2',
u'\u7047': 'CjkUnifiedIdeograph7047',
u'\u53c6': 'CjkUnifiedIdeograph53c6',
u'\ua94d': 'RejangVowelSignEu',
u'\u88cc': 'CjkUnifiedIdeograph88cc',
u'\u2351': 'AplFunctionalSymbolUpTackOverbar',
u'\u02d0': 'ModifierLetterTriangularColon',
u'\ue653': 'UnicodeCharacter0xe653',
u'\uc1d2': 'HangulSyllableSwaelp',
u'\u5857': 'CjkUnifiedIdeograph5857',
u'\ubbd6': 'HangulSyllableMeuj',
u'\u3dda': 'CjkUnifiedIdeograph3dda',
u'\u915d': 'CjkUnifiedIdeograph915d',
u'\uf0dc': 'UnicodeCharacter0xf0dc',
u'\u0b61': 'OriyaLetterVocalicLl',
u'\u6ae0': 'CjkUnifiedIdeograph6ae0',
u'\uce63': 'HangulSyllableCilb',
u'\u4067': 'CjkUnifiedIdeograph4067',
u'\ua3e6': 'YiSyllableJie',
u'\u3a6b': 'CjkUnifiedIdeograph3a6b',
u'\u25ea': 'SquareWithLowerRightDiagonalHalfBlack',
u'\uf96d': 'CjkCompatibilityIdeographf96d',
u'\ud8ec': 'UnicodeCharacter0xd8ec',
u'\u7371': 'CjkUnifiedIdeograph7371',
u'\u52f0': 'CjkUnifiedIdeograph52f0',
u'\ua877': 'PhagspaMarkDoubleShad',
u'\u8bf6': 'CjkUnifiedIdeograph8bf6',
u'\u227b': 'Succeeds',
u'\u0dfa': 'UnicodeCharacter0xdfa',
u'\ue17d': 'UnicodeCharacter0xe17d',
u'\uc0fc': 'HangulSyllableSyak',
u'\uda81': 'UnicodeCharacter0xda81',
u'\ubb00': 'HangulSyllableMoen',
u'\u9f83': 'CjkUnifiedIdeograph9f83',
u'\u7802': 'CjkUnifiedIdeograph7802',
u'\u5c85': 'CjkUnifiedIdeograph5c85',
u'\u3d04': 'CjkUnifiedIdeograph3d04',
u'\u1187': 'HangulJungseongYoo',
u'\uee0e': 'UnicodeCharacter0xee0e',
u'\uc291': 'HangulSyllableSyulg',
u'\ua310': 'YiSyllableSat',
u'\u8793': 'CjkUnifiedIdeograph8793',
u'\u6012': 'CjkUnifiedIdeograph6012',
u'\u4495': 'CjkUnifiedIdeograph4495',
u'\u2514': 'BoxDrawingsLightUpAndRight',
u'\u7997': 'CjkUnifiedIdeograph7997',
u'\u3e99': 'CjkUnifiedIdeograph3e99',
u'\ud61e': 'HangulSyllableHyegg',
u'\u8b20': 'CjkUnifiedIdeograph8b20',
u'\uefa3': 'UnicodeCharacter0xefa3',
u'\u4822': 'CjkUnifiedIdeograph4822',
u'\uaca5': 'HangulSyllableGet',
u'\u0d24': 'MalayalamLetterTa',
u'\u61a7': 'CjkUnifiedIdeograph61a7',
u'\u26a9': 'HorizontalMaleWithStrokeSign',
u'\uf330': 'UnicodeCharacter0xf330',
u'\ud7b3': 'UnicodeCharacter0xd7b3',
u'\ub032': 'HangulSyllableGgyulm',
u'\u94b5': 'CjkUnifiedIdeograph94b5',
u'\u7534': 'CjkUnifiedIdeograph7534',
u'\u49b7': 'CjkUnifiedIdeograph49b7',
u'\u2a36': 'CircledMultiplicationSignWithCircumflexAccent',
u'\u0eb9': 'LaoVowelSignUu',
u'\udb40': 'UnicodeCharacter0xdb40',
u'\u9842': 'CjkUnifiedIdeograph9842',
u'\ufcc5': 'ArabicLigatureKafWithHahInitialForm',
u'\u5d44': 'CjkUnifiedIdeograph5d44',
u'\ub1c7': 'HangulSyllableNwaec',
u'\u1246': 'EthiopicSyllableQo',
u'\u76c9': 'CjkUnifiedIdeograph76c9',
u'\u2bcb': 'UnicodeCharacter0x2bcb',
u'\uc350': 'HangulSyllableSsyaen',
u'\u8052': 'CjkUnifiedIdeograph8052',
u'\ue4d5': 'UnicodeCharacter0xe4d5',
u'\u4554': 'CjkUnifiedIdeograph4554',
u'\u99d7': 'CjkUnifiedIdeograph99d7',
u'\u7a56': 'CjkUnifiedIdeograph7a56',
u'\u5ed9': 'CjkUnifiedIdeograph5ed9',
u'\u3f58': 'CjkUnifiedIdeograph3f58',
u'\u13db': 'CherokeeLetterDv',
u'\ue862': 'UnicodeCharacter0xe862',
u'\ucce5': 'HangulSyllableCyeong',
u'\uad64': 'HangulSyllableGyoss',
u'\u81e7': 'CjkUnifiedIdeograph81e7',
u'\u6266': 'CjkUnifiedIdeograph6266',
u'\u46e9': 'CjkUnifiedIdeograph46e9',
u'\u2768': 'MediumLeftParenthesisOrnament',
u'\u7beb': 'CjkUnifiedIdeograph7beb',
u'\u38ed': 'CjkUnifiedIdeograph38ed',
u'\ud072': 'HangulSyllableKeunh',
u'\u9574': 'CjkUnifiedIdeograph9574',
u'\ue9f7': 'UnicodeCharacter0xe9f7',
u'\u4a76': 'CjkUnifiedIdeograph4a76',
u'\uaef9': 'HangulSyllableGgyeonj',
u'\u0f78': 'TibetanVowelSignVocalicL',
u'\u63fb': 'CjkUnifiedIdeograph63fb',
u'\u20fd': 'UnicodeCharacter0x20fd',
u'\u1a01': 'BugineseLetterGa',
u'\ufd84': 'ArabicLigatureLamWithJeemWithJeemFinalForm',
u'\ub286': 'HangulSyllableNyubs',
u'\u7788': 'CjkUnifiedIdeograph7788',
u'\ucb0b': 'HangulSyllableJjwaec',
u'\u348a': 'CjkUnifiedIdeograph348a',
u'\u880d': 'CjkUnifiedIdeograph880d',
u'\u4d0f': 'CjkUnifiedIdeograph4d0f',
u'\u0211': 'LatinSmallLetterRWithDoubleGrave',
u'\ue594': 'UnicodeCharacter0xe594',
u'\u9a96': 'CjkUnifiedIdeograph9a96',
u'\u5f98': 'CjkUnifiedIdeograph5f98',
u'\u1c9a': 'UnicodeCharacter0x1c9a',
u'\uf01d': 'UnicodeCharacter0xf01d',
u'\ub51f': 'HangulSyllableDilb',
u'\u6a21': 'CjkUnifiedIdeograph6a21',
u'\u2f23': 'KangxiRadicalEvening',
u'\ucda4': 'HangulSyllableCum',
u'\u82a6': 'CjkUnifiedIdeograph82a6',
u'\u47a8': 'CjkUnifiedIdeograph47a8',
u'\u04aa': 'CyrillicCapitalLetterEsWithDescender',
u'\ud82d': 'UnicodeCharacter0xd82d',
u'\u39ac': 'CjkUnifiedIdeograph39ac',
u'\u9d2f': 'CjkUnifiedIdeograph9d2f',
u'\u5231': 'CjkUnifiedIdeograph5231',
u'\u1733': 'HanunooVowelSignU',
u'\ueab6': 'UnicodeCharacter0xeab6',
u'\uafb8': 'HangulSyllableGgu',
u'\u6cba': 'CjkUnifiedIdeograph6cba',
u'\uc03d': 'HangulSyllableBbyug',
u'\u21bc': 'LeftwardsHarpoonWithBarbUpwards',
u'\u853f': 'CjkUnifiedIdeograph853f',
u'\uba41': 'HangulSyllableMeolg',
u'\u7f43': 'CjkUnifiedIdeograph7f43',
u'\u3c45': 'CjkUnifiedIdeograph3c45',
u'\ud2c6': 'HangulSyllableTeulp',
u'\u97c8': 'CjkUnifiedIdeograph97c8',
u'\u54ca': 'CjkUnifiedIdeograph54ca',
u'\u09cc': 'BengaliVowelSignAu',
u'\ued4f': 'UnicodeCharacter0xed4f',
u'\ua251': 'YiSyllableHxap',
u'\u6753': 'CjkUnifiedIdeograph6753',
u'\u2455': 'UnicodeCharacter0x2455',
u'\uffd8': 'UnicodeCharacter0xffd8',
u'\ubcda': 'HangulSyllableByegg',
u'\u71dc': 'CjkUnifiedIdeograph71dc',
u'\ud55f': 'HangulSyllableHad',
u'\u36de': 'CjkUnifiedIdeograph36de',
u'\u8a61': 'CjkUnifiedIdeograph8a61',
u'\u4f63': 'CjkUnifiedIdeograph4f63',
u'\u0c65': 'UnicodeCharacter0xc65',
u'\ue7e8': 'UnicodeCharacter0xe7e8',
u'\ua4ea': 'UnicodeCharacter0xa4ea',
u'\u59ec': 'CjkUnifiedIdeograph59ec',
u'\u1eee': 'LatinCapitalLetterUWithHornAndTilde',
u'\uf271': 'UnicodeCharacter0xf271',
u'\ub773': 'HangulSyllableDdis',
u'\u7475': 'CjkUnifiedIdeograph7475',
u'\u2977': 'LeftwardsArrowThroughLessthan',
u'\ucff8': 'HangulSyllableKuk',
u'\u8cfa': 'CjkUnifiedIdeograph8cfa',
u'\u41fc': 'CjkUnifiedIdeograph41fc',
u'\u06fe': 'ArabicSignSindhiPostpositionMen',
u'\u3c00': 'CjkUnifiedIdeograph3c00',
u'\u1883': 'MongolianLetterAliGaliUbadama',
u'\uff02': 'FullwidthQuotationMark',
u'\udb85': 'UnicodeCharacter0xdb85',
u'\u7506': 'CjkUnifiedIdeograph7506',
u'\u5189': 'CjkUnifiedIdeograph5189',
u'\uae0c': 'HangulSyllableGeuss',
u'\u8a8f': 'CjkUnifiedIdeograph8a8f',
u'\u2410': 'SymbolForDataLinkEscape',
u'\x93': 'UnicodeCharacter0x93',
u'\ue712': 'UnicodeCharacter0xe712',
u'\uc395': 'HangulSyllableSseb',
u'\u5d16': 'CjkUnifiedIdeograph5d16',
u'\ub999': 'HangulSyllableRyilg',
u'\u379d': 'CjkUnifiedIdeograph379d',
u'\u961c': 'CjkUnifiedIdeograph961c',
u'\uf29f': 'UnicodeCharacter0xf29f',
u'\u0c20': 'TeluguLetterTtha',
u'\u68a3': 'CjkUnifiedIdeograph68a3',
u'\ucf22': 'HangulSyllableKyeonh',
u'\u4526': 'CjkUnifiedIdeograph4526',
u'\ua1a9': 'YiSyllableHle',
u'\u1fad': 'GreekCapitalLetterOmegaWithDasiaAndOxiaAndProsgegrammeni',
u'\ufe2c': 'UnicodeCharacter0xfe2c',
u'\udaaf': 'UnicodeCharacter0xdaaf',
u'\u7430': 'CjkUnifiedIdeograph7430',
u'\u50b3': 'CjkUnifiedIdeograph50b3',
u'\uad36': 'HangulSyllableGoegg',
u'\u89b9': 'CjkUnifiedIdeograph89b9',
u'\u2b3a': 'LeftwardsArrowWithTailWithDoubleVerticalStroke',
u'\u07bd': 'UnicodeCharacter0x7bd',
u'\ue63c': 'UnicodeCharacter0xe63c',
u'\uc2bf': 'HangulSyllableSeuh',
u'\u5c40': 'CjkUnifiedIdeograph5c40',
u'\ub8c3': 'HangulSyllableRoes',
u'\u36c7': 'CjkUnifiedIdeograph36c7',
u'\u9546': 'CjkUnifiedIdeograph9546',
u'\uf1c9': 'UnicodeCharacter0xf1c9',
u'\u134a': 'EthiopicSyllableFi',
u'\u6fcd': 'CjkUnifiedIdeograph6fcd',
u'\uce4c': 'HangulSyllableCyim',
u'\u4450': 'CjkUnifiedIdeograph4450',
u'\ua0d3': 'YiSyllableFa',
u'\u1ed7': 'LatinSmallLetterOWithCircumflexAndTilde',
u'\ufd56': 'ArabicLigatureTehWithMeemWithHahInitialForm',
u'\ud9d9': 'UnicodeCharacter0xd9d9',
u'\u7b5a': 'CjkUnifiedIdeograph7b5a',
u'\u57dd': 'CjkUnifiedIdeograph57dd',
u'\uac60': 'HangulSyllableGyaels',
u'\u88e3': 'CjkUnifiedIdeograph88e3',
u'\u2a64': 'ZNotationDomainAntirestriction',
u'\u06e7': 'ArabicSmallHighYeh',
u'\ue566': 'UnicodeCharacter0xe566',
u'\uc1e9': 'HangulSyllableSoelg',
u'\u636a': 'CjkUnifiedIdeograph636a',
u'\ubfed': 'HangulSyllableBbweonj',
u'\u35f1': 'CjkUnifiedIdeograph35f1',
u'\u9470': 'CjkUnifiedIdeograph9470',
u'\uf0f3': 'UnicodeCharacter0xf0f3',
u'\u1274': 'EthiopicSyllableTee',
u'\u6ef7': 'CjkUnifiedIdeograph6ef7',
u'\ucd76': 'HangulSyllableCoep',
u'\u4b7a': 'CjkUnifiedIdeograph4b7a',
u'\ua7fd': 'LatinEpigraphicLetterInvertedM',
u'\u9d01': 'CjkUnifiedIdeograph9d01',
u'\ufc80': 'ArabicLigatureKafWithAlefFinalForm',
u'\u1b05': 'BalineseLetterAkara',
u'\u7a84': 'CjkUnifiedIdeograph7a84',
u'\ud607': 'HangulSyllableHyeod',
u'\u4c0b': 'CjkUnifiedIdeograph4c0b',
u'\ub38a': 'HangulSyllableDep',
u'\u298e': 'RightSquareBracketWithTickInBottomCorner',
u'\u8511': 'CjkUnifiedIdeograph8511',
u'\ue490': 'UnicodeCharacter0xe490',
u'\u0315': 'CombiningCommaAboveRight',
u'\u6294': 'CjkUnifiedIdeograph6294',
u'\ub41b': 'HangulSyllableDoegs',
u'\u9b9a': 'CjkUnifiedIdeograph9b9a',
u'\u321f': 'UnicodeCharacter0x321f',
u'\u119e': 'HangulJungseongAraea',
u'\ued21': 'UnicodeCharacter0xed21',
u'\ucca0': 'HangulSyllableCeol',
u'\u6b25': 'CjkUnifiedIdeograph6b25',
u'\u4aa4': 'CjkUnifiedIdeograph4aa4',
u'\u9c2b': 'CjkUnifiedIdeograph9c2b',
u'\u83aa': 'CjkUnifiedIdeograph83aa',
u'\u1a2f': 'UnicodeCharacter0x1a2f',
u'\u79ae': 'CjkUnifiedIdeograph79ae',
u'\ud531': 'HangulSyllablePyib',
u'\u5335': 'CjkUnifiedIdeograph5335',
u'\ub2b4': 'HangulSyllableNyil',
u'\u28b8': 'BraillePatternDots4568',
u'\u843b': 'CjkUnifiedIdeograph843b',
u'\uebba': 'UnicodeCharacter0xebba',
u'\u023f': 'LatinSmallLetterSWithSwashTail',
u'\u61be': 'CjkUnifiedIdeograph61be',
u'\ubb45': 'HangulSyllableMub',
u'\u9ac4': 'CjkUnifiedIdeograph9ac4',
u'\u3149': 'HangulLetterSsangcieuc',
u'\u10c8': 'UnicodeCharacter0x10c8',
u'\uec4b': 'UnicodeCharacter0xec4b',
u'\ud3ca': 'HangulSyllablePyeoj',
u'\u6a4f': 'CjkUnifiedIdeograph6a4f',
u'\u49ce': 'CjkUnifiedIdeograph49ce',
u'\ua355': 'YiSyllableZhut',
u'\u82d4': 'CjkUnifiedIdeograph82d4',
u'\u1959': 'TaiLeLetterPa',
u'\u78d8': 'CjkUnifiedIdeograph78d8',
u'\ud45b': 'HangulSyllablePoeh',
u'\u525f': 'CjkUnifiedIdeograph525f',
u'\ub1de': 'HangulSyllableNoebs',
u'\u2fe2': 'UnicodeCharacter0x2fe2',
u'\u8b65': 'CjkUnifiedIdeograph8b65',
u'\ueae4': 'UnicodeCharacter0xeae4',
u'\u0169': 'LatinSmallLetterUWithTilde',
u'\u60e8': 'CjkUnifiedIdeograph60e8',
u'\uba6f': 'HangulSyllableMeh',
u'\u99ee': 'CjkUnifiedIdeograph99ee',
u'\u3073': 'HiraganaLetterBi',
u'\u17f2': 'KhmerSymbolLekAttakPii',
u'\uf375': 'UnicodeCharacter0xf375',
u'\ud2f4': 'HangulSyllableTin',
u'\u6979': 'CjkUnifiedIdeograph6979',
u'\u48f8': 'CjkUnifiedIdeograph48f8',
u'\ua27f': 'YiSyllableHep',
u'\u81fe': 'CjkUnifiedIdeograph81fe',
u'\uf406': 'UnicodeCharacter0xf406',
u'\ud089': 'HangulSyllableKyig',
u'\ub108': 'HangulSyllableNeo',
u'\u8d8b': 'CjkUnifiedIdeograph8d8b',
u'\u720a': 'CjkUnifiedIdeograph720a',
u'\u4e8d': 'CjkUnifiedIdeograph4e8d',
u'\u2f0c': 'KangxiRadicalDownBox',
u'\u0b8f': 'TamilLetterEe',
u'\udc16': 'UnicodeCharacter0xdc16',
u'\u9918': 'CjkUnifiedIdeograph9918',
u'\uf59b': 'UnicodeCharacter0xf59b',
u'\u5a1a': 'CjkUnifiedIdeograph5a1a',
u'\ub69d': 'HangulSyllableDdug',
u'\u171c': 'UnicodeCharacter0x171c',
u'\u739f': 'CjkUnifiedIdeograph739f',
u'\u2ca1': 'CopticSmallLetterPi',
u'\uc426': 'HangulSyllableSswaej',
u'\u8128': 'CjkUnifiedIdeograph8128',
u'\uddab': 'UnicodeCharacter0xddab',
u'\u422a': 'CjkUnifiedIdeograph422a',
u'\u9ead': 'CjkUnifiedIdeograph9ead',
u'\u7f2c': 'CjkUnifiedIdeograph7f2c',
u'\u5baf': 'CjkUnifiedIdeograph5baf',
u'\u382e': 'CjkUnifiedIdeograph382e',
u'\u14b1': 'CanadianSyllabicsWestcreeMwii',
u'\ue938': 'UnicodeCharacter0xe938',
u'\uc5bb': 'HangulSyllableEod',
u'\uaa3a': 'UnicodeCharacter0xaa3a',
u'\u86bd': 'CjkUnifiedIdeograph86bd',
u'\u673c': 'CjkUnifiedIdeograph673c',
u'\u43bf': 'CjkUnifiedIdeograph43bf',
u'\u203e': 'Overline',
u'\u7cc1': 'CjkUnifiedIdeograph7cc1',
u'\u39c3': 'CjkUnifiedIdeograph39c3',
u'\ud148': 'HangulSyllableTeok',
u'\u924a': 'CjkUnifiedIdeograph924a',
u'\ueecd': 'UnicodeCharacter0xeecd',
u'\u4f4c': 'CjkUnifiedIdeograph4f4c',
u'\uabcf': 'UnicodeCharacter0xabcf',
u'\u084e': 'UnicodeCharacter0x84e',
u'\u64d1': 'CjkUnifiedIdeograph64d1',
u'\u21d3': 'DownwardsDoubleArrow',
u'\ufa5a': 'CjkCompatibilityIdeographfa5a',
u'\ud6dd': 'HangulSyllableHut',
u'\ub75c': 'HangulSyllableDdyik',
u'\u93df': 'CjkUnifiedIdeograph93df',
u'\u705e': 'CjkUnifiedIdeograph705e',
u'\u4ce1': 'CjkUnifiedIdeograph4ce1',
u'\u2d60': 'TifinaghLetterYav',
u'\u09e3': 'BengaliVowelSignVocalicLl',
u'\ue26a': 'UnicodeCharacter0xe26a',
u'\u9f6c': 'CjkUnifiedIdeograph9f6c',
u'\ufbef': 'ArabicLigatureYehWithHamzaAboveWithWawFinalForm',
u'\u586e': 'CjkUnifiedIdeograph586e',
u'\ub4f1': 'HangulSyllableDeung',
u'\u1570': 'CanadianSyllabicsTye',
u'\u71f3': 'CjkUnifiedIdeograph71f3',
u'\u32f5': 'CircledKatakanaYo',
u'\uca7a': 'HangulSyllableJjebs',
u'\u877c': 'CjkUnifiedIdeograph877c',
u'\ue3ff': 'UnicodeCharacter0xe3ff',
u'\u407e': 'CjkUnifiedIdeograph407e',
u'\u7d80': 'CjkUnifiedIdeograph7d80',
u'\ud903': 'UnicodeCharacter0xd903',
u'\u3e82': 'CjkUnifiedIdeograph3e82',
u'\u9a05': 'CjkUnifiedIdeograph9a05',
u'\u5707': 'CjkUnifiedIdeograph5707',
u'\u1009': 'MyanmarLetterNya',
u'\uef8c': 'UnicodeCharacter0xef8c',
u'\ua88e': 'SaurashtraLetterAi',
u'\u6590': 'CjkUnifiedIdeograph6590',
u'\uc113': 'HangulSyllableSyaes',
u'\u2692': 'HammerAndPick',
u'\u8215': 'CjkUnifiedIdeograph8215',
u'\ubf17': 'HangulSyllableBbyeolh',
u'\u7819': 'CjkUnifiedIdeograph7819',
u'\u351b': 'CjkUnifiedIdeograph351b',
u'\ud79c': 'HangulSyllableHiss',
u'\u909e': 'CjkUnifiedIdeograph909e',
u'\u4da0': 'CjkUnifiedIdeograph4da0',
u'\u0ea2': 'LaoLetterYo',
u'\uea25': 'UnicodeCharacter0xea25',
u'\ua727': 'LatinSmallLetterHeng',
u'\u6029': 'CjkUnifiedIdeograph6029',
u'\u1d2b': 'CyrillicLetterSmallCapitalEl',
u'\uf8ae': 'UnicodeCharacter0xf8ae',
u'\ub5b0': 'HangulSyllableDdeom',
u'\u76b2': 'CjkUnifiedIdeograph76b2',
u'\ud235': 'HangulSyllableTulg',
u'\u33b4': 'SquarePv',
u'\u8f37': 'CjkUnifiedIdeograph8f37',
u'\u4839': 'CjkUnifiedIdeograph4839',
u'\u053b': 'ArmenianCapitalLetterIni',
u'\ue0be': 'UnicodeCharacter0xe0be',
u'\u9dc0': 'CjkUnifiedIdeograph9dc0',
u'\u5ec2': 'CjkUnifiedIdeograph5ec2',
u'\u1bc4': 'UnicodeCharacter0x1bc4',
u'\uf747': 'UnicodeCharacter0xf747',
u'\ub049': 'HangulSyllableGgeunj',
u'\u6d4b': 'CjkUnifiedIdeograph6d4b',
u'\u2e4d': 'UnicodeCharacter0x2e4d',
u'\uc8ce': 'HangulSyllableJoelm',
u'\u85d0': 'CjkUnifiedIdeograph85d0',
u'\u46d2': 'CjkUnifiedIdeograph46d2',
u'\u03d4': 'GreekUpsilonWithDiaeresisAndHookSymbol',
u'\udf57': 'UnicodeCharacter0xdf57',
u'\u3cd6': 'CjkUnifiedIdeograph3cd6',
u'\u9859': 'CjkUnifiedIdeograph9859',
u'\u555b': 'CjkUnifiedIdeograph555b',
u'\u165d': 'CanadianSyllabicsCarrierTse',
u'\uede0': 'UnicodeCharacter0xede0',
u'\uaee2': 'HangulSyllableGgelm',
u'\u6be4': 'CjkUnifiedIdeograph6be4',
u'\uc767': 'HangulSyllableYilh',
u'\u24e6': 'CircledLatinSmallLetterW',
u'\u8069': 'CjkUnifiedIdeograph8069',
u'\ubd6b': 'HangulSyllableByod',
u'\u7e6d': 'CjkUnifiedIdeograph7e6d',
u'\u3b6f': 'CjkUnifiedIdeograph3b6f',
u'\ud5f0': 'HangulSyllableHels',
u'\u96f2': 'CjkUnifiedIdeograph96f2',
u'\u53f4': 'CjkUnifiedIdeograph53f4',
u'\u0cf6': 'UnicodeCharacter0xcf6',
u'\ue879': 'UnicodeCharacter0xe879',
u'\ua57b': 'VaiSyllableMgboo',
u'\u667d': 'CjkUnifiedIdeograph667d',
u'\u237f': 'VerticalLineWithMiddleDot',
u'\u5f81': 'CjkUnifiedIdeograph5f81',
u'\ub804': 'HangulSyllableReok',
u'\u9487': 'CjkUnifiedIdeograph9487',
u'\u3208': 'ParenthesizedHangulCieuc',
u'\u0e8b': 'UnicodeCharacter0xe8b',
u'\uf10a': 'UnicodeCharacter0xf10a',
u'\ucd8d': 'HangulSyllableCyong',
u'\u6b0e': 'CjkUnifiedIdeograph6b0e',
u'\u4791': 'CjkUnifiedIdeograph4791',
u'\ua014': 'YiSyllableE',
u'\ufc97': 'ArabicLigatureYehWithHamzaAboveWithJeemInitialForm',
u'\u1a18': 'BugineseVowelSignU',
u'\u769b': 'CjkUnifiedIdeograph769b',
u'\ud91a': 'UnicodeCharacter0xd91a',
u'\u531e': 'CjkUnifiedIdeograph531e',
u'\uafa1': 'HangulSyllableGgyonj',
u'\u29a5': 'ReversedAngleWithUnderbar',
u'\u8824': 'CjkUnifiedIdeograph8824',
u'\ue4a7': 'UnicodeCharacter0xe4a7',
u'\u0228': 'LatinCapitalLetterEWithCedilla',
u'\u5eab': 'CjkUnifiedIdeograph5eab',
u'\uc12a': 'HangulSyllableSeolp',
u'\ubb2e': 'HangulSyllableMyoj',
u'\u97b1': 'CjkUnifiedIdeograph97b1',
u'\u3532': 'CjkUnifiedIdeograph3532',
u'\u11b5': 'HangulJongseongRieulphieuph',
u'\uf034': 'UnicodeCharacter0xf034',
u'\uccb7': 'HangulSyllableCegs',
u'\u6a38': 'CjkUnifiedIdeograph6a38',
u'\u46bb': 'CjkUnifiedIdeograph46bb',
u'\ua33e': 'YiSyllableSsu',
u'\uffc1': 'UnicodeCharacter0xffc1',
u'\u1d42': 'ModifierLetterCapitalW',
u'\u79c5': 'CjkUnifiedIdeograph79c5',
u'\ud844': 'UnicodeCharacter0xd844',
u'\u5248': 'CjkUnifiedIdeograph5248',
u'\uaecb': 'HangulSyllableGgeolh',
u'\u28cf': 'BraillePatternDots123478',
u'\u8b4e': 'CjkUnifiedIdeograph8b4e',
u'\ue7d1': 'UnicodeCharacter0xe7d1',
u'\u0552': 'ArmenianCapitalLetterYiwn',
u'\u61d5': 'CjkUnifiedIdeograph61d5',
u'\uc054': 'HangulSyllableBbyuk',
u'\uba58': 'HangulSyllableMen',
u'\u96db': 'CjkUnifiedIdeograph96db',
u'\u345c': 'CjkUnifiedIdeograph345c',
u'\u10df': 'GeorgianLetterZhar',
u'\uf35e': 'UnicodeCharacter0xf35e',
u'\ucfe1': 'HangulSyllableKug',
u'\u6d62': 'CjkUnifiedIdeograph6d62',
u'\u49e5': 'CjkUnifiedIdeograph49e5',
u'\ua268': 'YiSyllableNgox',
u'\ufeeb': 'ArabicLetterHehInitialForm',
u'\u1c6c': 'OlChikiLetterUnn',
u'\u78ef': 'CjkUnifiedIdeograph78ef',
u'\udb6e': 'UnicodeCharacter0xdb6e',
u'\u5572': 'CjkUnifiedIdeograph5572',
u'\ub1f5': 'HangulSyllableNyolt',
u'\u2bf9': 'UnicodeCharacter0x2bf9',
u'\u8a78': 'CjkUnifiedIdeograph8a78',
u'\ue6fb': 'UnicodeCharacter0xe6fb',
u'\u047c': 'CyrillicCapitalLetterOmegaWithTitlo',
u'\u60ff': 'CjkUnifiedIdeograph60ff',
u'\uc37e': 'HangulSyllableSseoj',
u'\u5a03': 'CjkUnifiedIdeograph5a03',
u'\ubd82': 'HangulSyllableBugg',
u'\u3786': 'CjkUnifiedIdeograph3786',
u'\u9309': 'CjkUnifiedIdeograph9309',
u'\uf288': 'UnicodeCharacter0xf288',
u'\u0d0d': 'UnicodeCharacter0xd0d',
u'\u6c8c': 'CjkUnifiedIdeograph6c8c',
u'\uc80f': 'HangulSyllableJeolh',
u'\u4213': 'CjkUnifiedIdeograph4213',
u'\ua592': 'VaiSyllableMoo',
u'\u3c17': 'CjkUnifiedIdeograph3c17',
u'\u1f96': 'GreekSmallLetterEtaWithPsiliAndPerispomeniAndYpogegrammeni',
u'\ufb19': 'UnicodeCharacter0xfb19',
u'\uda98': 'UnicodeCharacter0xda98',
u'\u751d': 'CjkUnifiedIdeograph751d',
u'\u549c': 'CjkUnifiedIdeograph549c',
u'\uaa23': 'ChamLetterRa',
u'\u8da2': 'CjkUnifiedIdeograph8da2',
u'\u2427': 'UnicodeCharacter0x2427',
u'\u07a6': 'ThaanaAbafili',
u'\ue329': 'UnicodeCharacter0xe329',
u'\uc2a8': 'HangulSyllableSeun',
u'\u5d2d': 'CjkUnifiedIdeograph5d2d',
u'\ubcac': 'HangulSyllableBels',
u'\u36b0': 'CjkUnifiedIdeograph36b0',
u'\u9233': 'CjkUnifiedIdeograph9233',
u'\uf5b2': 'UnicodeCharacter0xf5b2',
u'\u0c37': 'TeluguLetterSsa',
u'\u6fb6': 'CjkUnifiedIdeograph6fb6',
u'\ucb39': 'HangulSyllableJjyolt',
u'\u453d': 'CjkUnifiedIdeograph453d',
u'\ua4bc': 'YiRadicalCy',
u'\u3f41': 'CjkUnifiedIdeograph3f41',
u'\u1ec0': 'LatinCapitalLetterEWithCircumflexAndGrave',
u'\ufa43': 'CjkCompatibilityIdeographfa43',
u'\uddc2': 'UnicodeCharacter0xddc2',
u'\u7447': 'CjkUnifiedIdeograph7447',
u'\u57c6': 'CjkUnifiedIdeograph57c6',
u'\uad4d': 'HangulSyllableGoet',
u'\u8ccc': 'CjkUnifiedIdeograph8ccc',
u'\u2751': 'LowerRightShadowedWhiteSquare',
u'\u06d0': 'ArabicLetterE',
u'\ue253': 'UnicodeCharacter0xe253',
u'\uc5d2': 'HangulSyllableEgg',
u'\u5c57': 'CjkUnifiedIdeograph5c57',
u'\ubfd6': 'HangulSyllableBbulm',
u'\u39da': 'CjkUnifiedIdeograph39da',
u'\u955d': 'CjkUnifiedIdeograph955d',
u'\uf4dc': 'UnicodeCharacter0xf4dc',
u'\u0f61': 'TibetanLetterYa',
u'\u6ee0': 'CjkUnifiedIdeograph6ee0',
u'\uca63': 'HangulSyllableJjeoc',
u'\u4467': 'CjkUnifiedIdeograph4467',
u'\ua7e6': 'UnicodeCharacter0xa7e6',
u'\u3e6b': 'CjkUnifiedIdeograph3e6b',
u'\u21ea': 'UpwardsWhiteArrowFromBar',
u'\ufd6d': 'ArabicLigatureSheenWithMeemWithMeemInitialForm',
u'\udcec': 'UnicodeCharacter0xdcec',
u'\u7771': 'CjkUnifiedIdeograph7771',
u'\u56f0': 'CjkUnifiedIdeograph56f0',
u'\uac77': 'HangulSyllableGeod',
u'\u8ff6': 'CjkUnifiedIdeograph8ff6',
u'\u267b': 'BlackUniversalRecyclingSymbol',
u'\u09fa': 'BengaliIsshar',
u'\ue57d': 'UnicodeCharacter0xe57d',
u'\uc4fc': 'HangulSyllableSseuls',
u'\ude81': 'UnicodeCharacter0xde81',
u'\ubf00': 'HangulSyllableBbess',
u'\u9b83': 'CjkUnifiedIdeograph9b83',
u'\u7c02': 'CjkUnifiedIdeograph7c02',
u'\u5885': 'CjkUnifiedIdeograph5885',
u'\u3904': 'CjkUnifiedIdeograph3904',
u'\u1587': 'CanadianSyllabicsTlhi',
u'\uea0e': 'UnicodeCharacter0xea0e',
u'\uc691': 'HangulSyllableOet',
u'\ua710': 'ModifierLetterLowDottedLeftstemToneBar',
u'\u8393': 'CjkUnifiedIdeograph8393',
u'\u6412': 'CjkUnifiedIdeograph6412',
u'\u4095': 'CjkUnifiedIdeograph4095',
u'\u2114': 'LBBarSymbol',
u'\u7d97': 'CjkUnifiedIdeograph7d97',
u'\u3a99': 'CjkUnifiedIdeograph3a99',
u'\ud21e': 'HangulSyllableTyolp',
u'\u8f20': 'CjkUnifiedIdeograph8f20',
u'\ueba3': 'UnicodeCharacter0xeba3',
u'\u4c22': 'CjkUnifiedIdeograph4c22',
u'\ua8a5': 'SaurashtraLetterNa',
u'\u0924': 'DevanagariLetterTa',
u'\u65a7': 'CjkUnifiedIdeograph65a7',
u'\u22a9': 'Forces',
u'\uf730': 'UnicodeCharacter0xf730',
u'\ud3b3': 'HangulSyllablePeh',
u'\ub432': 'HangulSyllableDoep',
u'\u90b5': 'CjkUnifiedIdeograph90b5',
u'\u7134': 'CjkUnifiedIdeograph7134',
u'\u4db7': 'UnicodeCharacter0x4db7',
u'\u2e36': 'UnicodeCharacter0x2e36',
u'\u0ab9': 'GujaratiLetterHa',
u'\udf40': 'UnicodeCharacter0xdf40',
u'\u9c42': 'CjkUnifiedIdeograph9c42',
u'\uf8c5': 'UnicodeCharacter0xf8c5',
u'\u5944': 'CjkUnifiedIdeograph5944',
u'\ub5c7': 'HangulSyllableDdelb',
u'\u1646': 'CanadianSyllabicsCarrierZ',
u'\u72c9': 'CjkUnifiedIdeograph72c9',
u'\u2fcb': 'KangxiRadicalEmbroidery',
u'\uc750': 'HangulSyllableEuss',
u'\u8452': 'CjkUnifiedIdeograph8452',
u'\ue0d5': 'UnicodeCharacter0xe0d5',
u'\u4154': 'CjkUnifiedIdeograph4154',
u'\u9dd7': 'CjkUnifiedIdeograph9dd7',
u'\u7e56': 'CjkUnifiedIdeograph7e56',
u'\u5ad9': 'CjkUnifiedIdeograph5ad9',
u'\u3b58': 'CjkUnifiedIdeograph3b58',
u'\u17db': 'KhmerCurrencySymbolRiel',
u'\uec62': 'UnicodeCharacter0xec62',
u'\uc8e5': 'HangulSyllableJyonj',
u'\ua964': 'UnicodeCharacter0xa964',
u'\u85e7': 'CjkUnifiedIdeograph85e7',
u'\u6666': 'CjkUnifiedIdeograph6666',
u'\u42e9': 'CjkUnifiedIdeograph42e9',
u'\u2368': 'AplFunctionalSymbolTildeDiaeresis',
u'\u7feb': 'CjkUnifiedIdeograph7feb',
u'\u3ced': 'CjkUnifiedIdeograph3ced',
u'\ud472': 'HangulSyllablePyoj',
u'\u9174': 'CjkUnifiedIdeograph9174',
u'\uedf7': 'UnicodeCharacter0xedf7',
u'\u4e76': 'CjkUnifiedIdeograph4e76',
u'\uaaf9': 'UnicodeCharacter0xaaf9',
u'\u0b78': 'UnicodeCharacter0xb78',
u'\u67fb': 'CjkUnifiedIdeograph67fb',
u'\u24fd': 'DoubleCircledDigitNine',
u'\u1e01': 'LatinSmallLetterAWithRingBelow',
u'\uf984': 'CjkCompatibilityIdeographf984',
u'\ub686': 'HangulSyllableDdyonh',
u'\u7388': 'CjkUnifiedIdeograph7388',
u'\ucf0b': 'HangulSyllableKelb',
u'\u308a': 'HiraganaLetterRi',
u'\u8c0d': 'CjkUnifiedIdeograph8c0d',
u'\u490f': 'CjkUnifiedIdeograph490f',
u'\u0611': 'ArabicSignAlayheAssallam',
u'\ue194': 'UnicodeCharacter0xe194',
u'\u9e96': 'CjkUnifiedIdeograph9e96',
u'\u5b98': 'CjkUnifiedIdeograph5b98',
u'\u189a': 'MongolianLetterManchuAliGaliGha',
u'\uf41d': 'UnicodeCharacter0xf41d',
u'\ub11f': 'HangulSyllableNeoc',
u'\u6e21': 'CjkUnifiedIdeograph6e21',
u'\u2b23': 'HorizontalBlackHexagon',
u'\uc9a4': 'HangulSyllableJyi',
u'\u86a6': 'CjkUnifiedIdeograph86a6',
u'\u43a8': 'CjkUnifiedIdeograph43a8',
u'\xaa': 'FeminineOrdinalIndicator',
u'\udc2d': 'UnicodeCharacter0xdc2d',
u'\u3dac': 'CjkUnifiedIdeograph3dac',
u'\u992f': 'CjkUnifiedIdeograph992f',
u'\u5631': 'CjkUnifiedIdeograph5631',
u'\u1333': 'EthiopicSyllablePhaa',
u'\ueeb6': 'UnicodeCharacter0xeeb6',
u'\uabb8': 'UnicodeCharacter0xabb8',
u'\u68ba': 'CjkUnifiedIdeograph68ba',
u'\uc43d': 'HangulSyllableSsoeb',
u'\u25bc': 'BlackDownpointingTriangle',
u'\u813f': 'CjkUnifiedIdeograph813f',
u'\ube41': 'HangulSyllableByit',
u'\u7b43': 'CjkUnifiedIdeograph7b43',
u'\u3845': 'CjkUnifiedIdeograph3845',
u'\ud6c6': 'HangulSyllableHugg',
u'\u93c8': 'CjkUnifiedIdeograph93c8',
u'\u50ca': 'CjkUnifiedIdeograph50ca',
u'\u0dcc': 'UnicodeCharacter0xdcc',
u'\ue94f': 'UnicodeCharacter0xe94f',
u'\ua651': 'CyrillicSmallLetterYeruWithBackYer',
u'\u6353': 'CjkUnifiedIdeograph6353',
u'\u2055': 'FlowerPunctuationMark',
u'\ufbd8': 'ArabicLetterUFinalForm',
u'\ub8da': 'HangulSyllableRyolp',
u'\u75dc': 'CjkUnifiedIdeograph75dc',
u'\ud15f': 'HangulSyllableTes',
u'\u32de': 'CircledKatakanaSo',
u'\u8e61': 'CjkUnifiedIdeograph8e61',
u'\u4b63': 'CjkUnifiedIdeograph4b63',
u'\u0865': 'UnicodeCharacter0x865',
u'\ue3e8': 'UnicodeCharacter0xe3e8',
u'\ua0ea': 'YiSyllableVat',
u'\u5dec': 'CjkUnifiedIdeograph5dec',
u'\u1aee': 'UnicodeCharacter0x1aee',
u'\uf671': 'UnicodeCharacter0xf671',
u'\ub373': 'HangulSyllableDegs',
u'\u7075': 'CjkUnifiedIdeograph7075',
u'\u2d77': 'UnicodeCharacter0x2d77',
u'\ucbf8': 'HangulSyllableJjyil',
u'\u88fa': 'CjkUnifiedIdeograph88fa',
u'\u45fc': 'CjkUnifiedIdeograph45fc',
u'\u02fe': 'ModifierLetterOpenShelf',
u'\u4000': 'CjkUnifiedIdeograph4000',
u'\u6483': 'CjkUnifiedIdeograph6483',
u'\u8302': 'CjkUnifiedIdeograph8302',
u'\ua785': 'LatinSmallLetterInsularS',
u'\u0906': 'DevanagariLetterAa',
u'\u2d89': 'EthiopicSyllableNyoa',
u'\ud20c': 'HangulSyllableToek',
u'\uf68f': 'UnicodeCharacter0xf68f',
u'\u5810': 'CjkUnifiedIdeograph5810',
u'\u7c93': 'CjkUnifiedIdeograph7c93',
u'\u9b12': 'CjkUnifiedIdeograph9b12',
u'\ubf95': 'HangulSyllableBboeg',
u'\u2116': 'NumeroSign',
u'\uc599': 'HangulSyllableYaeg',
u'\u4b9d': 'CjkUnifiedIdeograph4b9d',
u'\uea1c': 'UnicodeCharacter0xea1c',
u'\u8e9f': 'CjkUnifiedIdeograph8e9f',
u'\u7020': 'CjkUnifiedIdeograph7020',
u'\u14a3': 'CanadianSyllabicsMe',
u'\ub322': 'HangulSyllableDyanh',
u'\u3926': 'CjkUnifiedIdeograph3926',
u'\udda9': 'UnicodeCharacter0xdda9',
u'\u63ad': 'CjkUnifiedIdeograph63ad',
u'\u822c': 'CjkUnifiedIdeograph822c',
u'\ua6af': 'UnicodeCharacter0xa6af',
u'\u0830': 'UnicodeCharacter0x830',
u'\u2cb3': 'CopticSmallLetterDialectpAlef',
u'\ud136': 'HangulSyllableTeonh',
u'\uf5b9': 'UnicodeCharacter0xf5b9',
u'\u573a': 'CjkUnifiedIdeograph573a',
u'\u7bbd': 'CjkUnifiedIdeograph7bbd',
u'\u9a3c': 'CjkUnifiedIdeograph9a3c',
u'\ubebf': 'HangulSyllableBbyaelb',
u'\u2040': 'CharacterTie',
u'\uc4c3': 'HangulSyllableSswilb',
u'\u4ac7': 'CjkUnifiedIdeograph4ac7',
u'\ue946': 'UnicodeCharacter0xe946',
u'\u8dc9': 'CjkUnifiedIdeograph8dc9',
u'\u6f4a': 'CjkUnifiedIdeograph6f4a',
u'\u13cd': 'CherokeeLetterS',
u'\ub24c': 'HangulSyllableNwem',
u'\u3850': 'CjkUnifiedIdeograph3850',
u'\udcd3': 'UnicodeCharacter0xdcd3',
u'\u62d7': 'CjkUnifiedIdeograph62d7',
u'\u8156': 'CjkUnifiedIdeograph8156',
u'\ua5d9': 'VaiSyllableNjo',
u'\u075a': 'ArabicLetterDalWithInvertedSmallVBelow',
u'\u2bdd': 'UnicodeCharacter0x2bdd',
u'\ud060': 'HangulSyllableKyum',
u'\uf4e3': 'UnicodeCharacter0xf4e3',
u'\u5664': 'CjkUnifiedIdeograph5664',
u'\u7ae7': 'CjkUnifiedIdeograph7ae7',
u'\u9966': 'CjkUnifiedIdeograph9966',
u'\ubde9': 'HangulSyllableBwing',
u'\u1f6a': 'GreekCapitalLetterOmegaWithPsiliAndVaria',
u'\uc3ed': 'HangulSyllableSsong',
u'\u49f1': 'CjkUnifiedIdeograph49f1',
u'\ue870': 'UnicodeCharacter0xe870',
u'\u8cf3': 'CjkUnifiedIdeograph8cf3',
u'\u6e74': 'CjkUnifiedIdeograph6e74',
u'\u12f7': 'EthiopicSyllableDwa',
u'\ub176': 'HangulSyllableNyep',
u'\u377a': 'CjkUnifiedIdeograph377a',
u'\udbfd': 'UnicodeCharacter0xdbfd',
u'\ue101': 'UnicodeCharacter0xe101',
u'\u8080': 'CjkUnifiedIdeograph8080',
u'\u6705': 'CjkUnifiedIdeograph6705',
u'\u0684': 'ArabicLetterDyeh',
u'\uaa07': 'ChamLetterKha',
u'\u300b': 'RightDoubleAngleBracket',
u'\ucf8a': 'HangulSyllableKwap',
u'\u558e': 'CjkUnifiedIdeograph558e',
u'\uf911': 'CjkCompatibilityIdeographf911',
u'\u9890': 'CjkUnifiedIdeograph9890',
u'\u7f15': 'CjkUnifiedIdeograph7f15',
u'\u1e94': 'LatinCapitalLetterZWithLineBelow',
u'\uc81b': 'HangulSyllableJeoh',
u'\ue79a': 'UnicodeCharacter0xe79a',
u'\u4e1f': 'CjkUnifiedIdeograph4e1f',
u'\u6d9e': 'CjkUnifiedIdeograph6d9e',
u'\u9121': 'CjkUnifiedIdeograph9121',
u'\ub0a0': 'HangulSyllableNal',
u'\u1725': 'HanunooLetterNga',
u'\u36a4': 'CjkUnifiedIdeograph36a4',
u'\ue02b': 'UnicodeCharacter0xe02b',
u'\uffaa': 'HalfwidthHangulLetterRieulkiyeok',
u'\u662f': 'CjkUnifiedIdeograph662f',
u'\u05ae': 'HebrewAccentZinor',
u'\ua931': 'RejangLetterGa',
u'\u2f35': 'KangxiRadicalLongStride',
u'\uceb4': 'HangulSyllableKyal',
u'\u54b8': 'CjkUnifiedIdeograph54b8',
u'\uf83b': 'UnicodeCharacter0xf83b',
u'\u97ba': 'CjkUnifiedIdeograph97ba',
u'\u7e3f': 'CjkUnifiedIdeograph7e3f',
u'\u1dbe': 'ModifierLetterSmallEzh',
u'\uc745': 'HangulSyllableEulg',
u'\ue6c4': 'UnicodeCharacter0xe6c4',
u'\u4d49': 'CjkUnifiedIdeograph4d49',
u'\u6cc8': 'CjkUnifiedIdeograph6cc8',
u'\u904b': 'CjkUnifiedIdeograph904b',
u'\uafca': 'HangulSyllableGgubs',
u'\u164f': 'CanadianSyllabicsCarrierSo',
u'\u35ce': 'CjkUnifiedIdeograph35ce',
u'\udf55': 'UnicodeCharacter0xdf55',
u'\ufed4': 'ArabicLetterFehMedialForm',
u'\u6559': 'CjkUnifiedIdeograph6559',
u'\u04d8': 'CyrillicCapitalLetterSchwa',
u'\ua85b': 'PhagspaLetterSa',
u'\u2e5f': 'UnicodeCharacter0x2e5f',
u'\ucdde': 'HangulSyllableCwebs',
u'\u53e2': 'CjkUnifiedIdeograph53e2',
u'\uf765': 'UnicodeCharacter0xf765',
u'\u96e4': 'CjkUnifiedIdeograph96e4',
u'\u7d69': 'CjkUnifiedIdeograph7d69',
u'\u1ce8': 'UnicodeCharacter0x1ce8',
u'\uc66f': 'HangulSyllableWaes',
u'\ue5ee': 'UnicodeCharacter0xe5ee',
u'\u4c73': 'CjkUnifiedIdeograph4c73',
u'\u6bf2': 'CjkUnifiedIdeograph6bf2',
u'\u8f75': 'CjkUnifiedIdeograph8f75',
u'\uaef4': 'HangulSyllableGgyeo',
u'\u1579': 'CanadianSyllabicsNunavikHa',
u'\u34f8': 'CjkUnifiedIdeograph34f8',
u'\ude7f': 'UnicodeCharacter0xde7f',
u'\ufdfe': 'UnicodeCharacter0xfdfe',
u'\u8806': 'CjkUnifiedIdeograph8806',
u'\uac89': 'HangulSyllableGeot',
u'\ucd08': 'HangulSyllableCo',
u'\uf18b': 'UnicodeCharacter0xf18b',
u'\u0e0a': 'ThaiCharacterChoChang',
u'\u328d': 'CircledIdeographWood',
u'\u530c': 'CjkUnifiedIdeograph530c',
u'\u778f': 'CjkUnifiedIdeograph778f',
u'\ua016': 'YiSyllableBit',
u'\ue518': 'UnicodeCharacter0xe518',
u'\u899b': 'CjkUnifiedIdeograph899b',
u'\u261a': 'BlackLeftPointingIndex',
u'\uca9d': 'HangulSyllableJjyeot',
u'\u6b1c': 'CjkUnifiedIdeograph6b1c',
u'\u0f9f': 'TibetanSubjoinedLetterTa',
u'\u50a1': 'CjkUnifiedIdeograph50a1',
u'\ub826': 'HangulSyllableRyeogg',
u'\ufd28': 'ArabicLigatureSheenWithMeemFinalForm',
u'\ua1ab': 'YiSyllableHlut',
u'\u3e2a': 'CjkUnifiedIdeograph3e2a',
u'\ue2ad': 'UnicodeCharacter0xe2ad',
u'\u032c': 'CombiningCaronBelow',
u'\u27af': 'NotchedLowerRightshadowedWhiteRightwardsArrow',
u'\u442e': 'CjkUnifiedIdeograph442e',
u'\u68b1': 'CjkUnifiedIdeograph68b1',
u'\u9538': 'CjkUnifiedIdeograph9538',
u'\ub9bb': 'HangulSyllableRilh',
u'\ud63a': 'HangulSyllableHogg',
u'\ufabd': 'CjkCompatibilityIdeographfabd',
u'\u1b3c': 'BalineseVowelSignLaLenga',
u'\u3fbf': 'CjkUnifiedIdeograph3fbf',
u'\u5c3e': 'CjkUnifiedIdeograph5c3e',
u'\xc1': 'LatinCapitalLetterAWithAcute',
u'\u45c3': 'CjkUnifiedIdeograph45c3',
u'\uad48': 'HangulSyllableGoess',
u'\uee4a': 'UnicodeCharacter0xee4a',
u'\u92cd': 'CjkUnifiedIdeograph92cd',
u'\u334c': 'SquareMegaton',
u'\ud7cf': 'UnicodeCharacter0xd7cf',
u'\u744e': 'CjkUnifiedIdeograph744e',
u'\u18d1': 'UnicodeCharacter0x18d1',
u'\u5dd3': 'CjkUnifiedIdeograph5dd3',
u'\u865a': 'CjkUnifiedIdeograph865a',
u'\uaadd': 'UnicodeCharacter0xaadd',
u'\ucb5c': 'HangulSyllableJjuss',
u'\uefdf': 'UnicodeCharacter0xefdf',
u'\u0c5e': 'UnicodeCharacter0xc5e',
u'\u30e1': 'KatakanaLetterMe',
u'\u5160': 'CjkUnifiedIdeograph5160',
u'\u75e3': 'CjkUnifiedIdeograph75e3',
u'\u9e6a': 'CjkUnifiedIdeograph9e6a',
u'\ue36c': 'UnicodeCharacter0xe36c',
u'\u87ef': 'CjkUnifiedIdeograph87ef',
u'\u246e': 'CircledNumberFifteen',
u'\uc8f1': 'HangulSyllableJyob',
u'\u6970': 'CjkUnifiedIdeograph6970',
u'\u0df3': 'SinhalaVowelSignDigaGayanukitta',
u'\u4ef5': 'CjkUnifiedIdeograph4ef5',
u'\ub67a': 'HangulSyllableDdoej',
u'\ufb7c': 'ArabicLetterTchehInitialForm',
u'\u9fff': 'UnicodeCharacter0x9fff',
u'\u3c7e': 'CjkUnifiedIdeograph3c7e',
u'\u0180': 'LatinSmallLetterBWithStroke',
u'\ua503': 'VaiSyllableWee',
u'\u4282': 'CjkUnifiedIdeograph4282',
u'\ue605': 'UnicodeCharacter0xe605',
u'\u2b07': 'DownwardsBlackArrow',
u'\u6c09': 'CjkUnifiedIdeograph6c09',
u'\u938c': 'CjkUnifiedIdeograph938c',
u'\ud48e': 'HangulSyllablePuj',
u'\u1990': 'NewTaiLueLetterHighNa',
u'\ubd13': 'HangulSyllableBwags',
u'\u5a92': 'CjkUnifiedIdeograph5a92',
u'\ufe15': 'PresentationFormForVerticalExclamationMark',
u'\uc317': 'HangulSyllableSsaegs',
u'\u0419': 'CyrillicCapitalLetterShortI',
u'\u491b': 'CjkUnifiedIdeograph491b',
u'\uab9c': 'UnicodeCharacter0xab9c',
u'\uec9e': 'UnicodeCharacter0xec9e',
u'\u31a0': 'BopomofoLetterBu',
u'\u72a2': 'CjkUnifiedIdeograph72a2',
u'\u9625': 'CjkUnifiedIdeograph9625',
u'\udb27': 'UnicodeCharacter0xdb27',
u'\u1c29': 'LepchaVowelSignOo',
u'\u612b': 'CjkUnifiedIdeograph612b',
u'\u84ae': 'CjkUnifiedIdeograph84ae',
u'\uc9b0': 'HangulSyllableJyils',
u'\u0ab2': 'GujaratiLetterLa',
u'\uae35': 'HangulSyllableGinj',
u'\u4fb4': 'CjkUnifiedIdeograph4fb4',
u'\uf337': 'UnicodeCharacter0xf337',
u'\u3439': 'CjkUnifiedIdeograph3439',
u'\u793b': 'CjkUnifiedIdeograph793b',
u'\u9cbe': 'CjkUnifiedIdeograph9cbe',
u'\ue1c0': 'UnicodeCharacter0xe1c0',
u'\u22c2': 'NaryIntersection',
u'\u67c4': 'CjkUnifiedIdeograph67c4',
u'\u8b47': 'CjkUnifiedIdeograph8b47',
u'\ucc49': 'HangulSyllableCaenj',
u'\u114b': 'HangulChoseongIeungphieuph',
u'\u524d': 'CjkUnifiedIdeograph524d',
u'\ub4ce': 'HangulSyllableDyulp',
u'\uf9d0': 'CjkCompatibilityIdeographf9d0',
u'\u3ad2': 'CjkUnifiedIdeograph3ad2',
u'\u7fd4': 'CjkUnifiedIdeograph7fd4',
u'\ua357': 'YiSyllableZhu',
u'\u40d6': 'CjkUnifiedIdeograph40d6',
u'\ue459': 'UnicodeCharacter0xe459',
u'\u295b': 'RightwardsHarpoonWithBarbUpFromBar',
u'\u6a5d': 'CjkUnifiedIdeograph6a5d',
u'\u91e0': 'CjkUnifiedIdeograph91e0',
u'\ud2e2': 'HangulSyllableTyilp',
u'\u17e4': 'KhmerDigitFour',
u'\ubb67': 'HangulSyllableMweoc',
u'\u58e6': 'CjkUnifiedIdeograph58e6',
u'\ufc69': 'ArabicLigatureYehWithHamzaAboveWithYehFinalForm',
u'\uc16b': 'HangulSyllableSyeoc',
u'\u026d': 'LatinSmallLetterLWithRetroflexHook',
u'\u476f': 'CjkUnifiedIdeograph476f',
u'\ua9f0': 'UnicodeCharacter0xa9f0',
u'\ueaf2': 'UnicodeCharacter0xeaf2',
u'\u2ff4': 'IdeographicDescriptionCharacterFullSurround',
u'\u70f6': 'CjkUnifiedIdeograph70f6',
u'\u9479': 'CjkUnifiedIdeograph9479',
u'\ud97b': 'UnicodeCharacter0xd97b',
u'\u1a7d': 'UnicodeCharacter0x1a7d',
u'\u5f7f': 'CjkUnifiedIdeograph5f7f',
u'\u2381': 'ContinuousUnderlineSymbol',
u'\uc404': 'HangulSyllableSswam',
u'\ue887': 'UnicodeCharacter0xe887',
u'\u4e08': 'CjkUnifiedIdeograph4e08',
u'\u728b': 'CjkUnifiedIdeograph728b',
u'\u8d0a': 'CjkUnifiedIdeograph8d0a',
u'\ub18d': 'HangulSyllableNong',
u'\u170e': 'TagalogLetterLa',
u'\u3b91': 'CjkUnifiedIdeograph3b91',
u'\udc14': 'UnicodeCharacter0xdc14',
u'\u8097': 'CjkUnifiedIdeograph8097',
u'\u6618': 'CjkUnifiedIdeograph6618',
u'\u0a9b': 'GujaratiLetterCha',
u'\ua51a': 'VaiSyllableCee',
u'\u2f1e': 'KangxiRadicalEnclosure',
u'\ud3a1': 'HangulSyllablePelg',
u'\u55a5': 'CjkUnifiedIdeograph55a5',
u'\uf424': 'UnicodeCharacter0xf424',
u'\u98a7': 'CjkUnifiedIdeograph98a7',
u'\u7e28': 'CjkUnifiedIdeograph7e28',
u'\u22ab': 'DoubleVerticalBarDoubleRightTurnstile',
u'\ubd2a': 'HangulSyllableBwap',
u'\uc72e': 'HangulSyllableYulp',
u'\uebb1': 'UnicodeCharacter0xebb1',
u'\u4932': 'CjkUnifiedIdeograph4932',
u'\u6db5': 'CjkUnifiedIdeograph6db5',
u'\u8c34': 'CjkUnifiedIdeograph8c34',
u'\ub0b7': 'HangulSyllableNaegs',
u'\u1638': 'CanadianSyllabicsCarrierTlhi',
u'\u3abb': 'CjkUnifiedIdeograph3abb',
u'\udf3e': 'UnicodeCharacter0xdf3e',
u'\u83c1': 'CjkUnifiedIdeograph83c1',
u'\u6142': 'CjkUnifiedIdeograph6142',
u'\u05c5': 'HebrewMarkLowerDot',
u'\ua444': 'YiSyllableNjyt',
u'\u2e48': 'UnicodeCharacter0x2e48',
u'\ud2cb': 'HangulSyllableTeus',
u'\u54cf': 'CjkUnifiedIdeograph54cf',
u'\uf74e': 'UnicodeCharacter0xf74e',
u'\u9bd1': 'CjkUnifiedIdeograph9bd1',
u'\u7952': 'CjkUnifiedIdeograph7952',
u'\u1dd5': 'CombiningLatinSmallLetterAo',
u'\ubc54': 'HangulSyllableByal',
u'\uc658': 'HangulSyllableWak',
u'\ueadb': 'UnicodeCharacter0xeadb',
u'\u485c': 'CjkUnifiedIdeograph485c',
u'\u6cdf': 'CjkUnifiedIdeograph6cdf',
u'\u8f5e': 'CjkUnifiedIdeograph8f5e',
u'\ub3e1': 'HangulSyllableDwag',
u'\u1162': 'HangulJungseongAe',
u'\u35e5': 'CjkUnifiedIdeograph35e5',
u'\ude68': 'UnicodeCharacter0xde68',
u'\u82eb': 'CjkUnifiedIdeograph82eb',
u'\u606c': 'CjkUnifiedIdeograph606c',
u'\u04ef': 'CyrillicSmallLetterUWithMacron',
u'\ua76e': 'LatinCapitalLetterCon',
u'\u2972': 'TildeOperatorAboveRightwardsArrow',
u'\ucdf5': 'HangulSyllableCwilt',
u'\u57f9': 'CjkUnifiedIdeograph57f9',
u'\uf678': 'UnicodeCharacter0xf678',
u'\u9afb': 'CjkUnifiedIdeograph9afb',
u'\u787c': 'CjkUnifiedIdeograph787c',
u'\u1cff': 'UnicodeCharacter0x1cff',
u'\ubf7e': 'HangulSyllableBbwaenh',
u'\u2603': 'Snowman',
u'\uc182': 'HangulSyllableSyebs',
u'\u4b86': 'CjkUnifiedIdeograph4b86',
u'\uef09': 'UnicodeCharacter0xef09',
u'\u8e88': 'CjkUnifiedIdeograph8e88',
u'\u710d': 'CjkUnifiedIdeograph710d',
u'\u108c': 'MyanmarSignShanCouncilTone3',
u'\ub40f': 'HangulSyllableDwaes',
u'\u3e13': 'CjkUnifiedIdeograph3e13',
u'\ud992': 'UnicodeCharacter0xd992',
u'\u4017': 'CjkUnifiedIdeograph4017',
u'\u6396': 'CjkUnifiedIdeograph6396',
u'\u8719': 'CjkUnifiedIdeograph8719',
u'\ua698': 'UnicodeCharacter0xa698',
u'\u091d': 'DevanagariLetterJha',
u'\u289c': 'BraillePatternDots3458',
u'\ud623': 'HangulSyllableHyed',
u'\uf1a2': 'UnicodeCharacter0xf1a2',
u'\u5827': 'CjkUnifiedIdeograph5827',
u'\u7ba6': 'CjkUnifiedIdeograph7ba6',
u'\u9f29': 'CjkUnifiedIdeograph9f29',
u'\ubea8': 'HangulSyllableBbyam',
u'\u212d': 'BlackletterCapitalC',
u'\uc0ac': 'HangulSyllableSa',
u'\u4ab0': 'CjkUnifiedIdeograph4ab0',
u'\uee33': 'UnicodeCharacter0xee33',
u'\u89b2': 'CjkUnifiedIdeograph89b2',
u'\u7037': 'CjkUnifiedIdeograph7037',
u'\u13b6': 'CherokeeLetterLo',
u'\ub739': 'HangulSyllableDdeub',
u'\u393d': 'CjkUnifiedIdeograph393d',
u'\ud8bc': 'UnicodeCharacter0xd8bc',
u'\u4341': 'CjkUnifiedIdeograph4341',
u'\u62c0': 'CjkUnifiedIdeograph62c0',
u'\u8643': 'CjkUnifiedIdeograph8643',
u'\ua1c2': 'YiSyllableLap',
u'\u0847': 'UnicodeCharacter0x847',
u'\u2bc6': 'UnicodeCharacter0x2bc6',
u'\ud14d': 'HangulSyllableTeg',
u'\uf0cc': 'UnicodeCharacter0xf0cc',
u'\u5b51': 'CjkUnifiedIdeograph5b51',
u'\u7ad0': 'CjkUnifiedIdeograph7ad0',
u'\u9e53': 'CjkUnifiedIdeograph9e53',
u'\ub9d2': 'HangulSyllableMalm',
u'\u2057': 'QuadruplePrime',
u'\uc3d6': 'HangulSyllableSsyep',
u'\u45da': 'CjkUnifiedIdeograph45da',
u'\ue95d': 'UnicodeCharacter0xe95d',
u'\u88dc': 'CjkUnifiedIdeograph88dc',
u'\u7361': 'CjkUnifiedIdeograph7361',
u'\u12e0': 'EthiopicSyllableZha',
u'\ub663': 'HangulSyllableDdwaeh',
u'\u3867': 'CjkUnifiedIdeograph3867',
u'\udbe6': 'UnicodeCharacter0xdbe6',
u'\u426b': 'CjkUnifiedIdeograph426b',
u'\u5dea': 'CjkUnifiedIdeograph5dea',
u'\u816d': 'CjkUnifiedIdeograph816d',
u'\ua0ec': 'YiSyllableVa',
u'\u0b71': 'OriyaLetterWa',
u'\u2af0': 'VerticalLineWithCircleBelow',
u'\ud077': 'HangulSyllableKeulb',
u'\uf3f6': 'UnicodeCharacter0xf3f6',
u'\u5a7b': 'CjkUnifiedIdeograph5a7b',
u'\u75fa': 'CjkUnifiedIdeograph75fa',
u'\u997d': 'CjkUnifiedIdeograph997d',
u'\ub8fc': 'HangulSyllableRuss',
u'\ua281': 'YiSyllableWax',
u'\uc300': 'HangulSyllableSsal',
u'\ue783': 'UnicodeCharacter0xe783',
u'\u2485': 'ParenthesizedNumberEighteen',
u'\u4504': 'CjkUnifiedIdeograph4504',
u'\u6987': 'CjkUnifiedIdeograph6987',
u'\u960e': 'CjkUnifiedIdeograph960e',
u'\uba91': 'HangulSyllableMyenj',
u'\udb10': 'UnicodeCharacter0xdb10',
u'\uff93': 'HalfwidthKatakanaLetterMo',
u'\u1812': 'MongolianDigitTwo',
u'\u3c95': 'CjkUnifiedIdeograph3c95',
u'\u5d14': 'CjkUnifiedIdeograph5d14',
u'\u0197': 'LatinCapitalLetterIWithStroke',
u'\u4699': 'CjkUnifiedIdeograph4699',
u'\uae1e': 'HangulSyllableGyilm',
u'\uf320': 'UnicodeCharacter0xf320',
u'\u97a3': 'CjkUnifiedIdeograph97a3',
u'\u3022': 'HangzhouNumeralTwo',
u'\ud4a5': 'HangulSyllablePweob',
u'\u7524': 'CjkUnifiedIdeograph7524',
u'\u19a7': 'NewTaiLueLetterHighXva',
u'\u5ea9': 'CjkUnifiedIdeograph5ea9',
u'\u8b30': 'CjkUnifiedIdeograph8b30',
u'\uafb3': 'HangulSyllableGgyoc',
u'\uc832': 'HangulSyllableJej',
u'\uecb5': 'UnicodeCharacter0xecb5',
u'\u0d34': 'MalayalamLetterLlla',
u'\u31b7': 'BopomofoFinalLetterH',
u'\u5236': 'CjkUnifiedIdeograph5236',
u'\u76b9': 'CjkUnifiedIdeograph76b9',
u'\ua340': 'YiSyllableSsyt',
u'\ue042': 'UnicodeCharacter0xe042',
u'\u84c5': 'CjkUnifiedIdeograph84c5',
u'\u2544': 'BoxDrawingsRightUpHeavyAndLeftDownLight',
u'\uc9c7': 'HangulSyllableJid',
u'\u6a46': 'CjkUnifiedIdeograph6a46',
u'\u0ec9': 'LaoToneMaiTho',
u'\u53cb': 'CjkUnifiedIdeograph53cb',
u'\ubb50': 'HangulSyllableMweo',
u'\uf852': 'UnicodeCharacter0xf852',
u'\u9cd5': 'CjkUnifiedIdeograph9cd5',
u'\u3d54': 'CjkUnifiedIdeograph3d54',
u'\ue1d7': 'UnicodeCharacter0xe1d7',
u'\u0256': 'LatinSmallLetterDWithTail',
u'\u26d9': 'UnicodeCharacter0x26d9',
u'\u4758': 'CjkUnifiedIdeograph4758',
u'\u6bdb': 'CjkUnifiedIdeograph6bdb',
u'\u9062': 'CjkUnifiedIdeograph9062',
u'\ub4e5': 'HangulSyllableDeulg',
u'\ud564': 'HangulSyllableHals',
u'\uf9e7': 'CjkCompatibilityIdeographf9e7',
u'\u1a66': 'UnicodeCharacter0x1a66',
u'\u3ee9': 'CjkUnifiedIdeograph3ee9',
u'\u5f68': 'CjkUnifiedIdeograph5f68',
u'\u03eb': 'CopticSmallLetterGangia',
u'\u40ed': 'CjkUnifiedIdeograph40ed',
u'\ua872': 'PhagspaSuperfixedLetterRa',
u'\ued74': 'UnicodeCharacter0xed74',
u'\u91f7': 'CjkUnifiedIdeograph91f7',
u'\u3276': 'CircledHangulCieucA',
u'\ud6f9': 'HangulSyllableHweot',
u'\u7778': 'CjkUnifiedIdeograph7778',
u'\u1bfb': 'UnicodeCharacter0x1bfb',
u'\u58fd': 'CjkUnifiedIdeograph58fd',
u'\u6201': 'CjkUnifiedIdeograph6201',
u'\u8584': 'CjkUnifiedIdeograph8584',
u'\uca86': 'HangulSyllableJjyeogg',
u'\u0f88': 'TibetanSignLceTsaCan',
u'\ub30b': 'HangulSyllableDaelb',
u'\u4c8a': 'CjkUnifiedIdeograph4c8a',
u'\uf00d': 'UnicodeCharacter0xf00d',
u'\u350f': 'CjkUnifiedIdeograph350f',
u'\u7a11': 'CjkUnifiedIdeograph7a11',
u'\u9d94': 'CjkUnifiedIdeograph9d94',
u'\ue296': 'UnicodeCharacter0xe296',
u'\u2798': 'HeavySouthEastArrow',
u'\u649a': 'CjkUnifiedIdeograph649a',
u'\u881d': 'CjkUnifiedIdeograph881d',
u'\ucd1f': 'HangulSyllableCoc',
u'\u1221': 'EthiopicSyllableSzu',
u'\u5723': 'CjkUnifiedIdeograph5723',
u'\ub5a4': 'HangulSyllableDdeon',
u'\ufaa6': 'CjkCompatibilityIdeographfaa6',
u'\u3fa8': 'CjkUnifiedIdeograph3fa8',
u'\u7caa': 'CjkUnifiedIdeograph7caa',
u'\ua02d': 'YiSyllableBux',
u'\u41ac': 'CjkUnifiedIdeograph41ac',
u'\ue52f': 'UnicodeCharacter0xe52f',
u'\u2a31': 'MultiplicationSignWithUnderbar',
u'\u6f33': 'CjkUnifiedIdeograph6f33',
u'\u92b6': 'CjkUnifiedIdeograph92b6',
u'\ud7b8': 'UnicodeCharacter0xd7b8',
u'\u14ba': 'CanadianSyllabicsNaskapiMwaa',
u'\ub83d': 'HangulSyllableRyeot',
u'\u59bc': 'CjkUnifiedIdeograph59bc',
u'\ufd3f': 'OrnateRightParenthesis',
u'\uc241': 'HangulSyllableSweolt',
u'\u0743': 'SyriacTwoVerticalDotsAbove',
u'\u4445': 'CjkUnifiedIdeograph4445',
u'\uaac6': 'UnicodeCharacter0xaac6',
u'\uefc8': 'UnicodeCharacter0xefc8',
u'\u2cca': 'CopticCapitalLetterDialectpHori',
u'\u71cc': 'CjkUnifiedIdeograph71cc',
u'\u954f': 'CjkUnifiedIdeograph954f',
u'\uda51': 'UnicodeCharacter0xda51',
u'\u1f53': 'GreekSmallLetterUpsilonWithDasiaAndVaria',
u'\u5c55': 'CjkUnifiedIdeograph5c55',
u'\u87d8': 'CjkUnifiedIdeograph87d8',
u'\uc4da': 'HangulSyllableSsyunh',
u'\u09dc': 'BengaliLetterRra',
u'\uad5f': 'HangulSyllableGyolh',
u'\u4ede': 'CjkUnifiedIdeograph4ede',
u'\uf261': 'UnicodeCharacter0xf261',
u'\u3763': 'CjkUnifiedIdeograph3763',
u'\u7465': 'CjkUnifiedIdeograph7465',
u'\u9fe8': 'UnicodeCharacter0x9fe8',
u'\udcea': 'UnicodeCharacter0xdcea',
u'\u21ec': 'UpwardsWhiteArrowOnPedestalWithHorizontalBar',
u'\u66ee': 'CjkUnifiedIdeograph66ee',
u'\u8a71': 'CjkUnifiedIdeograph8a71',
u'\ucf73': 'HangulSyllableKwags',
u'\u0c75': 'UnicodeCharacter0xc75',
u'\u5177': 'CjkUnifiedIdeograph5177',
u'\ub7f8': 'HangulSyllableReols',
u'\uf4fa': 'UnicodeCharacter0xf4fa',
u'\u39fc': 'CjkUnifiedIdeograph39fc',
u'\u7efe': 'CjkUnifiedIdeograph7efe',
u'\u4400': 'CjkUnifiedIdeograph4400',
u'\u6083': 'CjkUnifiedIdeograph6083',
u'\u8702': 'CjkUnifiedIdeograph8702',
u'\ua385': 'YiSyllableRrex',
u'\u0d06': 'MalayalamLetterAa',
u'\u2989': 'ZNotationLeftBindingBracket',
u'\ud60c': 'HangulSyllableHyeols',
u'\uf28f': 'UnicodeCharacter0xf28f',
u'\u5c10': 'CjkUnifiedIdeograph5c10',
u'\u7893': 'CjkUnifiedIdeograph7893',
u'\u9f12': 'CjkUnifiedIdeograph9f12',
u'\ubb95': 'HangulSyllableMwilt',
u'\u2516': 'BoxDrawingsUpHeavyAndRightLight',
u'\uc199': 'HangulSyllableSolt',
u'\u4f9d': 'CjkUnifiedIdeograph4f9d',
u'\uee1c': 'UnicodeCharacter0xee1c',
u'\u8a9f': 'CjkUnifiedIdeograph8a9f',
u'\u7420': 'CjkUnifiedIdeograph7420',
u'\u10a3': 'GeorgianCapitalLetterDon',
u'\ub722': 'HangulSyllableDdyuj',
u'\u3d26': 'CjkUnifiedIdeograph3d26',
u'\ud9a9': 'UnicodeCharacter0xd9a9',
u'\u67ad': 'CjkUnifiedIdeograph67ad',
u'\u862c': 'CjkUnifiedIdeograph862c',
u'\ua2af': 'YiSyllableCix',
u'\u0c30': 'TeluguLetterRa',
u'\u28b3': 'BraillePatternDots12568',
u'\ud536': 'HangulSyllablePyij',
u'\uf1b9': 'UnicodeCharacter0xf1b9',
u'\u533a': 'CjkUnifiedIdeograph533a',
u'\u7fbd': 'CjkUnifiedIdeograph7fbd',
u'\u9e3c': 'CjkUnifiedIdeograph9e3c',
u'\ubabf': 'HangulSyllableMoc',
u'\u2440': 'OcrHook',
u'\uc0c3': 'HangulSyllableSac',
u'\u4ec7': 'CjkUnifiedIdeograph4ec7',
u'\ued46': 'UnicodeCharacter0xed46',
u'\u89c9': 'CjkUnifiedIdeograph89c9',
u'\u6b4a': 'CjkUnifiedIdeograph6b4a',
u'\u17cd': 'KhmerSignToandakhiat',
u'\ub64c': 'HangulSyllableDdwaen',
u'\u3c50': 'CjkUnifiedIdeograph3c50',
u'\ud8d3': 'UnicodeCharacter0xd8d3',
u'\u66d7': 'CjkUnifiedIdeograph66d7',
u'\u8556': 'CjkUnifiedIdeograph8556',
u'\ua1d9': 'YiSyllableLyr',
u'\u035a': 'CombiningDoubleRingBelow',
u'\u2fdd': 'UnicodeCharacter0x2fdd',
u'\ud460': 'HangulSyllablePyon',
u'\uf0e3': 'UnicodeCharacter0xf0e3',
u'\u5264': 'CjkUnifiedIdeograph5264',
u'\u7ee7': 'CjkUnifiedIdeograph7ee7',
u'\u9d66': 'CjkUnifiedIdeograph9d66',
u'\ub9e9': 'HangulSyllableMaenj',
u'\u1b6a': 'BalineseMusicalSymbolDangGede',
u'\uc7ed': 'HangulSyllableJyaelg',
u'\u4df1': 'HexagramForTheCauldron',
u'\uec70': 'UnicodeCharacter0xec70',
u'\u88f3': 'CjkUnifiedIdeograph88f3',
u'\u6a74': 'CjkUnifiedIdeograph6a74',
u'\u16f7': 'UnicodeCharacter0x16f7',
u'\ub576': 'HangulSyllableDdyalp',
u'\u337a': 'SquareIu',
u'\udffd': 'UnicodeCharacter0xdffd',
u'\ue501': 'UnicodeCharacter0xe501',
u'\u8480': 'CjkUnifiedIdeograph8480',
u'\u6305': 'CjkUnifiedIdeograph6305',
u'\u0284': 'LatinSmallLetterDotlessJWithStrokeAndHook',
u'\uae07': 'HangulSyllableGeulh',
u'\u340b': 'CjkUnifiedIdeograph340b',
u'\ucb8a': 'HangulSyllableJjwelm',
u'\u518e': 'CjkUnifiedIdeograph518e',
u'\ufd11': 'ArabicLigatureTahWithAlefMaksuraFinalForm',
u'\u9c90': 'CjkUnifiedIdeograph9c90',
u'\u7b15': 'CjkUnifiedIdeograph7b15',
u'\u1a94': 'UnicodeCharacter0x1a94',
u'\ucc1b': 'HangulSyllableJjilh',
u'\ue39a': 'UnicodeCharacter0xe39a',
u'\u4a1f': 'CjkUnifiedIdeograph4a1f',
u'\u699e': 'CjkUnifiedIdeograph699e',
u'\u9521': 'CjkUnifiedIdeograph9521',
u'\ub4a0': 'HangulSyllableDwek',
u'\u1325': 'EthiopicSyllableThe',
u'\u32a4': 'CircledIdeographHigh',
u'\ue42b': 'UnicodeCharacter0xe42b',
u'\ufbaa': 'ArabicLetterHehDoachashmeeIsolatedForm',
u'\u622f': 'CjkUnifiedIdeograph622f',
u'\u01ae': 'LatinCapitalLetterTWithRetroflexHook',
u'\uad31': 'HangulSyllableGwaet',
u'\u2b35': 'LeftwardsTwoheadedArrowWithDoubleVerticalStroke',
u'\ucab4': 'HangulSyllableJjyess',
u'\u50b8': 'CjkUnifiedIdeograph50b8',
u'\ufc3b': 'ArabicLigatureKafWithLamIsolatedForm',
u'\u93ba': 'CjkUnifiedIdeograph93ba',
u'\u7a3f': 'CjkUnifiedIdeograph7a3f',
u'\u19be': 'NewTaiLueVowelSignOay',
u'\uc345': 'HangulSyllableSsyang',
u'\ue2c4': 'UnicodeCharacter0xe2c4',
u'\u4949': 'CjkUnifiedIdeograph4949',
u'\u68c8': 'CjkUnifiedIdeograph68c8',
u'\u944b': 'CjkUnifiedIdeograph944b',
u'\uabca': 'UnicodeCharacter0xabca',
u'\u124f': 'UnicodeCharacter0x124f',
u'\u31ce': 'CjkStrokeHzzz',
u'\udb55': 'UnicodeCharacter0xdb55',
u'\ufad4': 'CjkCompatibilityIdeographfad4',
u'\u6159': 'CjkUnifiedIdeograph6159',
u'\xd8': 'LatinCapitalLetterOWithStroke',
u'\uac5b': 'HangulSyllableGyaed',
u'\u2a5f': 'LogicalAndWithUnderbar',
u'\uc9de': 'HangulSyllableJjagg',
u'\u57e2': 'CjkUnifiedIdeograph57e2',
u'\uf365': 'UnicodeCharacter0xf365',
u'\u92e4': 'CjkUnifiedIdeograph92e4',
u'\u7969': 'CjkUnifiedIdeograph7969',
u'\u18e8': 'UnicodeCharacter0x18e8',
u'\uc26f': 'HangulSyllableSwigs',
u'\ue1ee': 'UnicodeCharacter0xe1ee',
u'\u4873': 'CjkUnifiedIdeograph4873',
u'\u6ff2': 'CjkUnifiedIdeograph6ff2',
u'\u8b75': 'CjkUnifiedIdeograph8b75',
u'\uaaf4': 'UnicodeCharacter0xaaf4',
u'\u1179': 'HangulJungseongYayo',
u'\u30f8': 'KatakanaLetterVi',
u'\uda7f': 'UnicodeCharacter0xda7f',
u'\uf9fe': 'CjkCompatibilityIdeographf9fe',
u'\u8c06': 'CjkUnifiedIdeograph8c06',
u'\ua889': 'SaurashtraLetterVocalicRr',
u'\uc908': 'HangulSyllableJuls',
u'\uf58b': 'UnicodeCharacter0xf58b',
u'\u0a0a': 'GurmukhiLetterUu',
u'\u368d': 'CjkUnifiedIdeograph368d',
u'\u570c': 'CjkUnifiedIdeograph570c',
u'\u738f': 'CjkUnifiedIdeograph738f',
u'\ua416': 'YiSyllableQyrx',
u'\ue118': 'UnicodeCharacter0xe118',
u'\u8d9b': 'CjkUnifiedIdeograph8d9b',
u'\u221a': 'SquareRoot',
u'\uce9d': 'HangulSyllableKaelt',
u'\u6f1c': 'CjkUnifiedIdeograph6f1c',
u'\u0b9f': 'TamilLetterTta',
u'\u54a1': 'CjkUnifiedIdeograph54a1',
u'\ubc26': 'HangulSyllableBabs',
u'\uf928': 'CjkCompatibilityIdeographf928',
u'\ua5ab': 'VaiSyllableNdu',
u'\u3a2a': 'CjkUnifiedIdeograph3a2a',
u'\ue6ad': 'UnicodeCharacter0xe6ad',
u'\u072c': 'SyriacLetterTaw',
u'\u23af': 'HorizontalLineExtension',
u'\u402e': 'CjkUnifiedIdeograph402e',
u'\u6cb1': 'CjkUnifiedIdeograph6cb1',
u'\u9138': 'CjkUnifiedIdeograph9138',
u'\ubdbb': 'HangulSyllableBwegs',
u'\ud23a': 'HangulSyllableTulp',
u'\ufebd': 'ArabicLetterDadIsolatedForm',
u'\u1f3c': 'GreekCapitalLetterIotaWithPsiliAndOxia',
u'\u3bbf': 'CjkUnifiedIdeograph3bbf',
u'\u583e': 'CjkUnifiedIdeograph583e',
u'\u04c1': 'CyrillicCapitalLetterZheWithBreve',
u'\u41c3': 'CjkUnifiedIdeograph41c3',
u'\ua948': 'RejangVowelSignU',
u'\uea4a': 'UnicodeCharacter0xea4a',
u'\u96cd': 'CjkUnifiedIdeograph96cd',
u'\u374c': 'CjkUnifiedIdeograph374c',
u'\ud3cf': 'HangulSyllablePyeoh',
u'\u704e': 'CjkUnifiedIdeograph704e',
u'\u1cd1': 'UnicodeCharacter0x1cd1',
u'\u59d3': 'CjkUnifiedIdeograph59d3',
u'\u825a': 'CjkUnifiedIdeograph825a',
u'\uaedd': 'HangulSyllableGgenj',
u'\ucf5c': 'HangulSyllableKol',
u'\uebdf': 'UnicodeCharacter0xebdf',
u'\u085e': 'UnicodeCharacter0x85e',
u'\u34e1': 'CjkUnifiedIdeograph34e1',
u'\u5560': 'CjkUnifiedIdeograph5560',
u'\u71e3': 'CjkUnifiedIdeograph71e3',
u'\u9a6a': 'CjkUnifiedIdeograph9a6a',
u'\ue76c': 'UnicodeCharacter0xe76c',
u'\u83ef': 'CjkUnifiedIdeograph83ef',
u'\u206e': 'NationalDigitShapes',
u'\uccf1': 'HangulSyllableCyenj',
u'\u6d70': 'CjkUnifiedIdeograph6d70',
u'\u09f3': 'BengaliRupeeSign',
u'\u4af5': 'CjkUnifiedIdeograph4af5',
u'\ub27a': 'HangulSyllableNyunh',
u'\uff7c': 'HalfwidthKatakanaLetterSi',
u'\u9bff': 'CjkUnifiedIdeograph9bff',
u'\u387e': 'CjkUnifiedIdeograph387e',
u'\u0580': 'ArmenianSmallLetterReh',
u'\ua103': 'YiSyllableDip',
u'\u4682': 'CjkUnifiedIdeograph4682',
u'\ue205': 'UnicodeCharacter0xe205',
u'\u2f07': 'KangxiRadicalLid',
u'\u6809': 'CjkUnifiedIdeograph6809',
u'\u978c': 'CjkUnifiedIdeograph978c',
u'\ud08e': 'HangulSyllableKyinh',
u'\u1d90': 'LatinSmallLetterAlphaWithRetroflexHook',
u'\ub913': 'HangulSyllableRweolh',
u'\u5e92': 'CjkUnifiedIdeograph5e92',
u'\ufa15': 'CjkCompatibilityIdeographfa15',
u'\uc717': 'HangulSyllableWis',
u'\u4d1b': 'CjkUnifiedIdeograph4d1b',
u'\uaf9c': 'HangulSyllableGgyo',
u'\ue89e': 'UnicodeCharacter0xe89e',
u'\u35a0': 'CjkUnifiedIdeograph35a0',
u'\u76a2': 'CjkUnifiedIdeograph76a2',
u'\u9225': 'CjkUnifiedIdeograph9225',
u'\udf27': 'UnicodeCharacter0xdf27',
u'\u1829': 'MongolianLetterAng',
u'\u652b': 'CjkUnifiedIdeograph652b',
u'\u80ae': 'CjkUnifiedIdeograph80ae',
u'\ucdb0': 'HangulSyllableCweo',
u'\u0eb2': 'LaoVowelSignAa',
u'\uaa35': 'ChamConsonantSignLa',
u'\u4bb4': 'CjkUnifiedIdeograph4bb4',
u'\uf737': 'UnicodeCharacter0xf737',
u'\u3039': 'HangzhouNumeralTwenty',
u'\u7d3b': 'CjkUnifiedIdeograph7d3b',
u'\u98be': 'CjkUnifiedIdeograph98be',
u'\ue5c0': 'UnicodeCharacter0xe5c0',
u'\u26c2': 'BlackDraughtsMan',
u'\u63c4': 'CjkUnifiedIdeograph63c4',
u'\u8f47': 'CjkUnifiedIdeograph8f47',
u'\uc849': 'HangulSyllableJyeob',
u'\u154b': 'CanadianSyllabicsRa',
u'\u564d': 'CjkUnifiedIdeograph564d',
u'\ub0ce': 'HangulSyllableNaep',
u'\ufdd0': 'UnicodeCharacter0xfdd0',
u'\u3ed2': 'CjkUnifiedIdeograph3ed2',
u'\u7bd4': 'CjkUnifiedIdeograph7bd4',
u'\ua757': 'LatinSmallLetterQWithStrokeThroughDescender',
u'\u44d6': 'CjkUnifiedIdeograph44d6',
u'\ue059': 'UnicodeCharacter0xe059',
u'\u2d5b': 'TifinaghLetterYash',
u'\u6e5d': 'CjkUnifiedIdeograph6e5d',
u'\u95e0': 'CjkUnifiedIdeograph95e0',
u'\ud6e2': 'HangulSyllableHweogg',
u'\u13e4': 'CherokeeLetterTse',
u'\ubf67': 'HangulSyllableBbwalb',
u'\u5ce6': 'CjkUnifiedIdeograph5ce6',
u'\uf869': 'UnicodeCharacter0xf869',
u'\uc56b': 'HangulSyllableAelb',
u'\u066d': 'ArabicFivePointedStar',
u'\u436f': 'CjkUnifiedIdeograph436f',
u'\uadf0': 'HangulSyllableGyuss',
u'\ueef2': 'UnicodeCharacter0xeef2',
u'\u2bf4': 'UnicodeCharacter0x2bf4',
u'\u74f6': 'CjkUnifiedIdeograph74f6',
u'\u9079': 'CjkUnifiedIdeograph9079',
u'\udd7b': 'UnicodeCharacter0xdd7b',
u'\u1e7d': 'LatinSmallLetterVWithTilde',
u'\u5b7f': 'CjkUnifiedIdeograph5b7f',
u'\u2781': 'DingbatCircledSansserifDigitTwo',
u'\uc004': 'HangulSyllableBbwe',
u'\uec87': 'UnicodeCharacter0xec87',
u'\u4a08': 'CjkUnifiedIdeograph4a08',
u'\u768b': 'CjkUnifiedIdeograph768b',
u'\u890a': 'CjkUnifiedIdeograph890a',
u'\ub58d': 'HangulSyllableDdyaelg',
u'\u130e': 'EthiopicSyllableGo',
u'\u3f91': 'CjkUnifiedIdeograph3f91',
u'\ud814': 'UnicodeCharacter0xd814',
u'\u8497': 'CjkUnifiedIdeograph8497',
u'\u6218': 'CjkUnifiedIdeograph6218',
u'\u0e9b': 'LaoLetterPo',
u'\ua11a': 'YiSyllableTit',
u'\u2b1e': 'WhiteVerySmallSquare',
u'\ud7a1': 'HangulSyllableHit',
u'\u51a5': 'CjkUnifiedIdeograph51a5',
u'\uf024': 'UnicodeCharacter0xf024',
u'\u9ca7': 'CjkUnifiedIdeograph9ca7',
u'\u7a28': 'CjkUnifiedIdeograph7a28',
u'\u26ab': 'MediumBlackCircle',
u'\ub92a': 'HangulSyllableRwelm',
u'\uc32e': 'HangulSyllableSsaep',
u'\uefb1': 'UnicodeCharacter0xefb1',
u'\u4d32': 'CjkUnifiedIdeograph4d32',
u'\u69b5': 'CjkUnifiedIdeograph69b5',
u'\u8834': 'CjkUnifiedIdeograph8834',
u'\ub4b7': 'HangulSyllableDwis',
u'\u1238': 'EthiopicSyllableSha',
u'\u3ebb': 'CjkUnifiedIdeograph3ebb',
u'\udb3e': 'UnicodeCharacter0xdb3e',
u'\u87c1': 'CjkUnifiedIdeograph87c1',
u'\u6542': 'CjkUnifiedIdeograph6542',
u'\u01c5': 'LatinCapitalLetterDWithSmallLetterZWithCaron',
u'\ua044': 'YiSyllablePuo',
u'\u2a48': 'UnionAboveBarAboveIntersection',
u'\ud6cb': 'HangulSyllableHud',
u'\u50cf': 'CjkUnifiedIdeograph50cf',
u'\uf34e': 'UnicodeCharacter0xf34e',
u'\u9fd1': 'UnicodeCharacter0x9fd1',
u'\u7d52': 'CjkUnifiedIdeograph7d52',
u'\u19d5': 'NewTaiLueDigitFive',
u'\ub854': 'HangulSyllableRyess',
u'\uc258': 'HangulSyllableSwel',
u'\ueedb': 'UnicodeCharacter0xeedb',
u'\u4c5c': 'CjkUnifiedIdeograph4c5c',
u'\u68df': 'CjkUnifiedIdeograph68df',
u'\u8b5e': 'CjkUnifiedIdeograph8b5e',
u'\ub7e1': 'HangulSyllableRyaeb',
u'\u1562': 'CanadianSyllabicsThii',
u'\u31e5': 'UnicodeCharacter0x31e5',
u'\uda68': 'UnicodeCharacter0xda68',
u'\u86eb': 'CjkUnifiedIdeograph86eb',
u'\u646c': 'CjkUnifiedIdeograph646c',
u'\xef': 'LatinSmallLetterIWithDiaeresis',
u'\ua36e': 'YiSyllableChex',
u'\u2d72': 'UnicodeCharacter0x2d72',
u'\uc9f5': 'HangulSyllableJjat',
u'\u53f9': 'CjkUnifiedIdeograph53f9',
u'\uf278': 'UnicodeCharacter0xf278',
u'\u9efb': 'CjkUnifiedIdeograph9efb',
u'\u7c7c': 'CjkUnifiedIdeograph7c7c',
u'\u18ff': 'UnicodeCharacter0x18ff',
u'\ubb7e': 'HangulSyllableMwebs',
u'\u2203': 'ThereExists',
u'\uc582': 'HangulSyllableYanh',
u'\u4f86': 'CjkUnifiedIdeograph4f86',
u'\ueb09': 'UnicodeCharacter0xeb09',
u'\u8a88': 'CjkUnifiedIdeograph8a88',
u'\u750d': 'CjkUnifiedIdeograph750d',
u'\u148c': 'CanadianSyllabicsCii',
u'\ub00f': 'HangulSyllableGgwigs',
u'\u3a13': 'CjkUnifiedIdeograph3a13',
u'\udd92': 'UnicodeCharacter0xdd92',
u'\u4417': 'CjkUnifiedIdeograph4417',
u'\u6796': 'CjkUnifiedIdeograph6796',
u'\u8319': 'CjkUnifiedIdeograph8319',
u'\ua298': 'YiSyllableZuox',
u'\u0d1d': 'MalayalamLetterJha',
u'\u2c9c': 'CopticCapitalLetterKsi',
u'\ud223': 'HangulSyllableTyos',
u'\uf5a2': 'UnicodeCharacter0xf5a2',
u'\u5c27': 'CjkUnifiedIdeograph5c27',
u'\u7fa6': 'CjkUnifiedIdeograph7fa6',
u'\u9b29': 'CjkUnifiedIdeograph9b29',
u'\ubaa8': 'HangulSyllableMo',
u'\u252d': 'BoxDrawingsLeftHeavyAndRightDownLight',
u'\uc4ac': 'HangulSyllableSswem',
u'\u4eb0': 'CjkUnifiedIdeograph4eb0',
u'\uea33': 'UnicodeCharacter0xea33',
u'\u8db2': 'CjkUnifiedIdeograph8db2',
u'\u7437': 'CjkUnifiedIdeograph7437',
u'\u17b6': 'KhmerVowelSignAa',
u'\ub339': 'HangulSyllableDyaeg',
u'\u3d3d': 'CjkUnifiedIdeograph3d3d',
u'\udcbc': 'UnicodeCharacter0xdcbc',
u'\u4741': 'CjkUnifiedIdeograph4741',
u'\u66c0': 'CjkUnifiedIdeograph66c0',
u'\u8243': 'CjkUnifiedIdeograph8243',
u'\ua5c2': 'VaiSyllableBho',
u'\u0c47': 'TeluguVowelSignEe',
u'\u2fc6': 'KangxiRadicalWheat',
u'\ud54d': 'HangulSyllablePib',
u'\uf4cc': 'UnicodeCharacter0xf4cc',
u'\u5f51': 'CjkUnifiedIdeograph5f51',
u'\u7ed0': 'CjkUnifiedIdeograph7ed0',
u'\u9a53': 'CjkUnifiedIdeograph9a53',
u'\ubdd2': 'HangulSyllableBwep',
u'\u2457': 'UnicodeCharacter0x2457',
u'\uc7d6': 'HangulSyllableJyalp',
u'\u41da': 'CjkUnifiedIdeograph41da',
u'\ued5d': 'UnicodeCharacter0xed5d',
u'\u8cdc': 'CjkUnifiedIdeograph8cdc',
u'\u7761': 'CjkUnifiedIdeograph7761',
u'\u16e0': 'RunicLetterEar',
u'\ub263': 'HangulSyllableNwilb',
u'\u3c67': 'CjkUnifiedIdeograph3c67',
u'\udfe6': 'UnicodeCharacter0xdfe6',
u'\u466b': 'CjkUnifiedIdeograph466b',
u'\u59ea': 'CjkUnifiedIdeograph59ea',
u'\u856d': 'CjkUnifiedIdeograph856d',
u'\ua4ec': 'UnicodeCharacter0xa4ec',
u'\u0f71': 'TibetanVowelSignAa',
u'\u2ef0': 'CjkRadicalCsimplifiedDragon',
u'\ud477': 'HangulSyllablePyoh',
u'\uf7f6': 'UnicodeCharacter0xf7f6',
u'\u5e7b': 'CjkUnifiedIdeograph5e7b',
u'\u71fa': 'CjkUnifiedIdeograph71fa',
u'\u9d7d': 'CjkUnifiedIdeograph9d7d',
u'\ubcfc': 'HangulSyllableBol',
u'\ua681': 'CyrillicSmallLetterDwe',
u'\uc700': 'HangulSyllableWek',
u'\ue383': 'UnicodeCharacter0xe383',
u'\u0402': 'CyrillicCapitalLetterDje',
u'\u2085': 'SubscriptFive',
u'\u4104': 'CjkUnifiedIdeograph4104',
u'\u6d87': 'CjkUnifiedIdeograph6d87',
u'\u920e': 'CjkUnifiedIdeograph920e',
u'\ube91': 'HangulSyllableBbaeng',
u'\udf10': 'UnicodeCharacter0xdf10',
u'\ufb93': 'ArabicLetterGafFinalForm',
u'\u1c12': 'LepchaLetterFla',
u'\u3895': 'CjkUnifiedIdeograph3895',
u'\u5914': 'CjkUnifiedIdeograph5914',
u'\u0597': 'HebrewAccentRevia',
u'\u4299': 'CjkUnifiedIdeograph4299',
u'\uaa1e': 'ChamLetterBha',
u'\uf720': 'UnicodeCharacter0xf720',
u'\u93a3': 'CjkUnifiedIdeograph93a3',
u'\u3422': 'CjkUnifiedIdeograph3422',
u'\ud0a5': 'HangulSyllableKig',
u'\u7124': 'CjkUnifiedIdeograph7124',
u'\u1da7': 'ModifierLetterSmallCapitalIWithStroke',
u'\u5aa9': 'CjkUnifiedIdeograph5aa9',
u'\u8f30': 'CjkUnifiedIdeograph8f30',
u'\uabb3': 'UnicodeCharacter0xabb3',
u'\ucc32': 'HangulSyllableCalm',
u'\ue8b5': 'UnicodeCharacter0xe8b5',
u'\u0934': 'DevanagariLetterLlla',
u'\u35b7': 'CjkUnifiedIdeograph35b7',
u'\u5636': 'CjkUnifiedIdeograph5636',
u'\u72b9': 'CjkUnifiedIdeograph72b9',
u'\ua740': 'LatinCapitalLetterKWithStroke',
u'\ue442': 'UnicodeCharacter0xe442',
u'\u80c5': 'CjkUnifiedIdeograph80c5',
u'\u2144': 'TurnedSansserifCapitalY',
u'\ucdc7': 'HangulSyllableCweoc',
u'\u6e46': 'CjkUnifiedIdeograph6e46',
u'\u0ac9': 'GujaratiVowelSignCandraO',
u'\u57cb': 'CjkUnifiedIdeograph57cb',
u'\ubf50': 'HangulSyllableBbom',
u'\ufc52': 'ArabicLigatureHehWithMeemIsolatedForm',
u'\u98d5': 'CjkUnifiedIdeograph98d5',
u'\u3954': 'CjkUnifiedIdeograph3954',
u'\ue5d7': 'UnicodeCharacter0xe5d7',
u'\u0656': 'ArabicSubscriptAlef',
u'\u22d9': 'VeryMuchGreaterthan',
u'\u4358': 'CjkUnifiedIdeograph4358',
u'\u6fdb': 'CjkUnifiedIdeograph6fdb',
u'\u9462': 'CjkUnifiedIdeograph9462',
u'\ub0e5': 'HangulSyllableNyang',
u'\ud164': 'HangulSyllableTek',
u'\ufde7': 'UnicodeCharacter0xfde7',
u'\u1e66': 'LatinCapitalLetterSWithCaronAndDotAbove',
u'\u3ae9': 'CjkUnifiedIdeograph3ae9',
u'\u5b68': 'CjkUnifiedIdeograph5b68',
u'\u07eb': 'NkoCombiningShortHighTone',
u'\u44ed': 'CjkUnifiedIdeograph44ed',
u'\uac72': 'HangulSyllableGeogg',
u'\ue974': 'UnicodeCharacter0xe974',
u'\u95f7': 'CjkUnifiedIdeograph95f7',
u'\u3676': 'CjkUnifiedIdeograph3676',
u'\ud2f9': 'HangulSyllableTilg',
u'\u7378': 'CjkUnifiedIdeograph7378',
u'\u1ffb': 'GreekCapitalLetterOmegaWithOxia',
u'\u5cfd': 'CjkUnifiedIdeograph5cfd',
u'\u6601': 'CjkUnifiedIdeograph6601',
u'\u8184': 'CjkUnifiedIdeograph8184',
u'\uce86': 'HangulSyllableKabs',
u'\u0b88': 'TamilLetterIi',
u'\ub70b': 'HangulSyllableDdwih',
u'\u488a': 'CjkUnifiedIdeograph488a',
u'\uf40d': 'UnicodeCharacter0xf40d',
u'\u310f': 'BopomofoLetterH',
u'\u7e11': 'CjkUnifiedIdeograph7e11',
u'\u9994': 'CjkUnifiedIdeograph9994',
u'\ue696': 'UnicodeCharacter0xe696',
u'\u2398': 'NextPage',
u'\u609a': 'CjkUnifiedIdeograph609a',
u'\u8c1d': 'CjkUnifiedIdeograph8c1d',
u'\uc91f': 'HangulSyllableJweod',
u'\u1621': 'CanadianSyllabicsCarrierJja',
u'\u5323': 'CjkUnifiedIdeograph5323',
u'\ub1a4': 'HangulSyllableNwam',
u'\ufea6': 'ArabicLetterKhahFinalForm',
u'\u3ba8': 'CjkUnifiedIdeograph3ba8',
u'\u78aa': 'CjkUnifiedIdeograph78aa',
u'\ua42d': 'YiSyllableJjyt',
u'\u45ac': 'CjkUnifiedIdeograph45ac',
u'\ue12f': 'UnicodeCharacter0xe12f',
u'\u2e31': 'UnicodeCharacter0x2e31',
u'\u6b33': 'CjkUnifiedIdeograph6b33',
u'\u96b6': 'CjkUnifiedIdeograph96b6',
u'\ud3b8': 'HangulSyllablePyeon',
u'\u10ba': 'GeorgianCapitalLetterCan',
u'\ubc3d': 'HangulSyllableBaelt',
u'\u5dbc': 'CjkUnifiedIdeograph5dbc',
u'\uf93f': 'CjkCompatibilityIdeographf93f',
u'\uc641': 'HangulSyllableWag',
u'\u0343': 'CombiningGreekKoronis',
u'\u4045': 'CjkUnifiedIdeograph4045',
u'\uaec6': 'HangulSyllableGgeolm',
u'\uebc8': 'UnicodeCharacter0xebc8',
u'\u28ca': 'BraillePatternDots2478',
u'\u75cc': 'CjkUnifiedIdeograph75cc',
u'\u914f': 'CjkUnifiedIdeograph914f',
u'\ude51': 'UnicodeCharacter0xde51',
u'\u1b53': 'BalineseDigitThree',
u'\u5855': 'CjkUnifiedIdeograph5855',
u'\u83d8': 'CjkUnifiedIdeograph83d8',
u'\uc0da': 'HangulSyllableSaebs',
u'\u0ddc': 'SinhalaVowelSignKombuvaHaaAelapilla',
u'\ua95f': 'RejangSectionMark',
u'\u4ade': 'CjkUnifiedIdeograph4ade',
u'\uf661': 'UnicodeCharacter0xf661',
u'\u3363': 'IdeographicTelegraphSymbolForHourEleven',
u'\u7065': 'CjkUnifiedIdeograph7065',
u'\u9be8': 'CjkUnifiedIdeograph9be8',
u'\ud8ea': 'UnicodeCharacter0xd8ea',
u'\u25ec': 'WhiteUppointingTriangleWithDot',
u'\u62ee': 'CjkUnifiedIdeograph62ee',
u'\u8e71': 'CjkUnifiedIdeograph8e71',
u'\ucb73': 'HangulSyllableJjweolh',
u'\u0875': 'UnicodeCharacter0x875',
u'\u5577': 'CjkUnifiedIdeograph5577',
u'\ub3f8': 'HangulSyllableDwak',
u'\uf0fa': 'UnicodeCharacter0xf0fa',
u'\u3dfc': 'CjkUnifiedIdeograph3dfc',
u'\u7afe': 'CjkUnifiedIdeograph7afe',
u'\u4800': 'CjkUnifiedIdeograph4800',
u'\u6c83': 'CjkUnifiedIdeograph6c83',
u'\u8b02': 'CjkUnifiedIdeograph8b02',
u'\uaf85': 'HangulSyllableGgoenj',
u'\u0106': 'LatinCapitalLetterCWithAcute',
u'\u2589': 'LeftSevenEighthsBlock',
u'\uda0c': 'UnicodeCharacter0xda0c',
u'\ufe8f': 'ArabicLetterBehIsolatedForm',
u'\u5010': 'CjkUnifiedIdeograph5010',
u'\u7493': 'CjkUnifiedIdeograph7493',
u'\u9312': 'CjkUnifiedIdeograph9312',
u'\ub795': 'HangulSyllableRat',
u'\u2916': 'RightwardsTwoheadedArrowWithTail',
u'\ucd99': 'HangulSyllableCunj',
u'\u439d': 'CjkUnifiedIdeograph439d',
u'\ue21c': 'UnicodeCharacter0xe21c',
u'\u869f': 'CjkUnifiedIdeograph869f',
u'\u7820': 'CjkUnifiedIdeograph7820',
u'\u1ca3': 'UnicodeCharacter0x1ca3',
u'\ubb22': 'HangulSyllableMyolm',
u'\u3126': 'BopomofoLetterEr',
u'\ud5a9': 'HangulSyllableHyat',
u'\u6bad': 'CjkUnifiedIdeograph6bad',
u'\u8a2c': 'CjkUnifiedIdeograph8a2c',
u'\uaeaf': 'HangulSyllableGgyaelh',
u'\u24b3': 'ParenthesizedLatinSmallLetterX',
u'\ud936': 'UnicodeCharacter0xd936',
u'\ufdb9': 'ArabicLigatureMeemWithKhahWithYehFinalForm',
u'\u5f3a': 'CjkUnifiedIdeograph5f3a',
u'\u73bd': 'CjkUnifiedIdeograph73bd',
u'\u923c': 'CjkUnifiedIdeograph923c',
u'\ub6bf': 'HangulSyllableDdweod',
u'\u2840': 'BraillePatternDots7',
u'\uccc3': 'HangulSyllableCelh',
u'\u42c7': 'CjkUnifiedIdeograph42c7',
u'\ue146': 'UnicodeCharacter0xe146',
u'\u85c9': 'CjkUnifiedIdeograph85c9',
u'\u674a': 'CjkUnifiedIdeograph674a',
u'\u1bcd': 'UnicodeCharacter0x1bcd',
u'\uba4c': 'HangulSyllableMeoss',
u'\u3050': 'HiraganaLetterGu',
u'\ud4d3': 'HangulSyllablePwid',
u'\u6ad7': 'CjkUnifiedIdeograph6ad7',
u'\u8956': 'CjkUnifiedIdeograph8956',
u'\uadd9': 'HangulSyllableGwit',
u'\u0f5a': 'TibetanLetterTsha',
u'\u23dd': 'BottomParenthesis',
u'\ud860': 'UnicodeCharacter0xd860',
u'\ufce3': 'ArabicLigatureTehWithMeemMedialForm',
u'\u5e64': 'CjkUnifiedIdeograph5e64',
u'\u72e7': 'CjkUnifiedIdeograph72e7',
u'\u9166': 'CjkUnifiedIdeograph9166',
u'\ub5e9': 'HangulSyllableDdyeob',
u'\u176a': 'TagbanwaLetterBa',
u'\ucbed': 'HangulSyllableJjeut',
u'\u41f1': 'CjkUnifiedIdeograph41f1',
u'\ue070': 'UnicodeCharacter0xe070',
u'\u84f3': 'CjkUnifiedIdeograph84f3',
u'\u6674': 'CjkUnifiedIdeograph6674',
u'\u1af7': 'UnicodeCharacter0x1af7',
u'\ub976': 'HangulSyllableReugg',
u'\u3f7a': 'CjkUnifiedIdeograph3f7a',
u'\ud3fd': 'HangulSyllablePob',
u'\ue901': 'UnicodeCharacter0xe901',
u'\u8880': 'CjkUnifiedIdeograph8880',
u'\u6f05': 'CjkUnifiedIdeograph6f05',
u'\u0e84': 'LaoLetterKhoTam',
u'\ua207': 'YiSyllableKox',
u'\u380b': 'CjkUnifiedIdeograph380b',
u'\uc78a': 'HangulSyllableIj',
u'\u5d8e': 'CjkUnifiedIdeograph5d8e',
u'\uf111': 'UnicodeCharacter0xf111',
u'\u9090': 'CjkUnifiedIdeograph9090',
u'\u7715': 'CjkUnifiedIdeograph7715',
u'\u1694': 'OghamLetterIodhadh',
u'\uc01b': 'HangulSyllableBbwec',
u'\uef9a': 'UnicodeCharacter0xef9a',
u'\u461f': 'CjkUnifiedIdeograph461f',
u'\u659e': 'CjkUnifiedIdeograph659e',
u'\u9921': 'CjkUnifiedIdeograph9921',
u'\ub8a0': 'HangulSyllableRwaels',
u'\u1f25': 'GreekSmallLetterEtaWithDasiaAndOxia',
u'\u3ea4': 'CjkUnifiedIdeograph3ea4',
u'\ue82b': 'UnicodeCharacter0xe82b',
u'\uf7aa': 'UnicodeCharacter0xf7aa',
u'\u6e2f': 'CjkUnifiedIdeograph6e2f',
u'\u0dae': 'SinhalaLetterMahaapraanaTayanna',
u'\ua131': 'YiSyllableTux',
u'\u2735': 'EightPointedPinwheelStar',
u'\uc6b4': 'HangulSyllableUn',
u'\u5cb8': 'CjkUnifiedIdeograph5cb8',
u'\uf03b': 'UnicodeCharacter0xf03b',
u'\u9fba': 'CjkUnifiedIdeograph9fba',
u'\u763f': 'CjkUnifiedIdeograph763f',
u'\u15be': 'CanadianSyllabicsBlackfootKo',
u'\ucf45': 'HangulSyllableKyelt',
u'\ueec4': 'UnicodeCharacter0xeec4',
u'\u4549': 'CjkUnifiedIdeograph4549',
u'\u64c8': 'CjkUnifiedIdeograph64c8',
u'\u984b': 'CjkUnifiedIdeograph984b',
u'\ua7ca': 'UnicodeCharacter0xa7ca',
u'\u1e4f': 'LatinSmallLetterOWithTildeAndDiaeresis',
u'\u3dce': 'CjkUnifiedIdeograph3dce',
u'\ud755': 'HangulSyllableHeunj',
u'\uf6d4': 'UnicodeCharacter0xf6d4',
u'\u6d59': 'CjkUnifiedIdeograph6d59',
u'\u0cd8': 'UnicodeCharacter0xcd8',
u'\ua05b': 'YiSyllableBbiex',
u'\u265f': 'BlackChessPawn',
u'\uc5de': 'HangulSyllableElp',
u'\u5be2': 'CjkUnifiedIdeograph5be2',
u'\uff65': 'HalfwidthKatakanaMiddleDot',
u'\u9ee4': 'CjkUnifiedIdeograph9ee4',
u'\u7569': 'CjkUnifiedIdeograph7569',
u'\u14e8': 'CanadianSyllabicsLwaa',
u'\uce6f': 'HangulSyllableCic',
u'\uedee': 'UnicodeCharacter0xedee',
u'\u4473': 'CjkUnifiedIdeograph4473',
u'\u63f2': 'CjkUnifiedIdeograph63f2',
u'\u8775': 'CjkUnifiedIdeograph8775',
u'\ua6f4': 'UnicodeCharacter0xa6f4',
u'\u1d79': 'LatinSmallLetterInsularG',
u'\u3cf8': 'CjkUnifiedIdeograph3cf8',
u'\ud67f': 'HangulSyllableHwaelh',
u'\uf5fe': 'UnicodeCharacter0xf5fe',
u'\u8006': 'CjkUnifiedIdeograph8006',
u'\ua489': 'YiSyllableYy',
u'\uc508': 'HangulSyllableSseuk',
u'\uf98b': 'CjkCompatibilityIdeographf98b',
u'\u060a': 'ArabicindicPerTenThousandSign',
u'\u3a8d': 'CjkUnifiedIdeograph3a8d',
u'\u5b0c': 'CjkUnifiedIdeograph5b0c',
u'\u7f8f': 'CjkUnifiedIdeograph7f8f',
u'\ua816': 'SylotiNagriLetterDo',
u'\ued18': 'UnicodeCharacter0xed18',
u'\u819b': 'CjkUnifiedIdeograph819b',
u'\u2e1a': 'HyphenWithDiaeresis',
u'\uc29d': 'HangulSyllableSyung',
u'\u631c': 'CjkUnifiedIdeograph631c',
u'\u079f': 'ThaanaLetterDaadhu',
u'\u58a1': 'CjkUnifiedIdeograph58a1',
u'\ub026': 'HangulSyllableGgwip',
u'\uf528': 'UnicodeCharacter0xf528',
u'\ua9ab': 'UnicodeCharacter0xa9ab',
u'\u362a': 'CjkUnifiedIdeograph362a',
u'\ueaad': 'UnicodeCharacter0xeaad',
u'\u0b2c': 'OriyaLetterBa',
u'\u2faf': 'KangxiRadicalFace',
u'\u4c2e': 'CjkUnifiedIdeograph4c2e',
u'\u60b1': 'CjkUnifiedIdeograph60b1',
u'\u9d38': 'CjkUnifiedIdeograph9d38',
u'\ub1bb': 'HangulSyllableNwaelb',
u'\ude3a': 'UnicodeCharacter0xde3a',
u'\uf2bd': 'UnicodeCharacter0xf2bd',
u'\u133c': 'EthiopicSyllableTsee',
u'\u37bf': 'CjkUnifiedIdeograph37bf',
u'\u543e': 'CjkUnifiedIdeograph543e',
u'\u08c1': 'UnicodeCharacter0x8c1',
u'\u4dc3': 'HexagramForYouthfulFolly',
u'\ua548': 'VaiSyllableNyi',
u'\ue64a': 'UnicodeCharacter0xe64a',
u'\u9acd': 'CjkUnifiedIdeograph9acd',
u'\u3b4c': 'CjkUnifiedIdeograph3b4c',
u'\udfcf': 'UnicodeCharacter0xdfcf',
u'\u7c4e': 'CjkUnifiedIdeograph7c4e',
u'\u10d1': 'GeorgianLetterBan',
u'\u55d3': 'CjkUnifiedIdeograph55d3',
u'\u8e5a': 'CjkUnifiedIdeograph8e5a',
u'\ua2dd': 'YiSyllableZzo',
u'\uc35c': 'HangulSyllableSsyaem',
u'\ue7df': 'UnicodeCharacter0xe7df',
u'\u045e': 'CyrillicSmallLetterShortU',
u'\u38e1': 'CjkUnifiedIdeograph38e1',
u'\u5960': 'CjkUnifiedIdeograph5960',
u'\u7de3': 'CjkUnifiedIdeograph7de3',
u'\u966a': 'CjkUnifiedIdeograph966a',
u'\ueb6c': 'UnicodeCharacter0xeb6c',
u'\u8fef': 'CjkUnifiedIdeograph8fef',
u'\u2c6e': 'LatinCapitalLetterMWithHook',
u'\uc0f1': 'HangulSyllableSyalt',
u'\u6170': 'CjkUnifiedIdeograph6170',
u'\u05f3': 'HebrewPunctuationGeresh',
u'\u46f5': 'CjkUnifiedIdeograph46f5',
u'\ube7a': 'HangulSyllableBbap',
u'\uf37c': 'UnicodeCharacter0xf37c',
u'\u97ff': 'CjkUnifiedIdeograph97ff',
u'\u347e': 'CjkUnifiedIdeograph347e',
u'\u0980': 'UnicodeCharacter0x980',
u'\uad03': 'HangulSyllableGwad',
u'\u4a82': 'CjkUnifiedIdeograph4a82',
u'\uee05': 'UnicodeCharacter0xee05',
u'\u2307': 'WavyLine',
u'\u6409': 'CjkUnifiedIdeograph6409',
u'\u9b8c': 'CjkUnifiedIdeograph9b8c',
u'\udc8e': 'UnicodeCharacter0xdc8e',
u'\u1190': 'HangulJungseongYue',
u'\ub513': 'HangulSyllableDyih',
u'\u5292': 'CjkUnifiedIdeograph5292',
u'\uf615': 'UnicodeCharacter0xf615',
u'\ucb17': 'HangulSyllableJjoed',
u'\u0c19': 'TeluguLetterNga',
u'\u411b': 'CjkUnifiedIdeograph411b',
u'\ua39c': 'YiSyllableNrex',
u'\ue49e': 'UnicodeCharacter0xe49e',
u'\u39a0': 'CjkUnifiedIdeograph39a0',
u'\u7aa2': 'CjkUnifiedIdeograph7aa2',
u'\u9e25': 'CjkUnifiedIdeograph9e25',
u'\ud327': 'HangulSyllablePah',
u'\u1429': 'CanadianSyllabicsFinalPlus',
u'\u692b': 'CjkUnifiedIdeograph692b',
u'\u8cae': 'CjkUnifiedIdeograph8cae',
u'\uc1b0': 'HangulSyllableSwal',
u'\u02b2': 'ModifierLetterSmallJ',
u'\ua635': 'UnicodeCharacter0xa635',
u'\u47b4': 'CjkUnifiedIdeograph47b4',
u'\ufb37': 'UnicodeCharacter0xfb37',
u'\u3c39': 'CjkUnifiedIdeograph3c39',
u'\u713b': 'CjkUnifiedIdeograph713b',
u'\u94be': 'CjkUnifiedIdeograph94be',
u'\ue9c0': 'UnicodeCharacter0xe9c0',
u'\u2ac2': 'SupersetWithMultiplicationSignBelow',
u'\u6fc4': 'CjkUnifiedIdeograph6fc4',
u'\u8347': 'CjkUnifiedIdeograph8347',
u'\uc449': 'HangulSyllableSsyog',
u'\u194b': 'LimbuDigitFive',
u'\u5a4d': 'CjkUnifiedIdeograph5a4d',
u'\ubcce': 'HangulSyllableByeobs',
u'\uf1d0': 'UnicodeCharacter0xf1d0',
u'\u32d2': 'CircledKatakanaU',
u'\u77d4': 'CjkUnifiedIdeograph77d4',
u'\uab57': 'UnicodeCharacter0xab57',
u'\u48d6': 'CjkUnifiedIdeograph48d6',
u'\uec59': 'UnicodeCharacter0xec59',
u'\u215b': 'VulgarFractionOneEighth',
u'\u625d': 'CjkUnifiedIdeograph625d',
u'\u99e0': 'CjkUnifiedIdeograph99e0',
u'\udae2': 'UnicodeCharacter0xdae2',
u'\u1fe4': 'GreekSmallLetterRhoWithPsili',
u'\ub367': 'HangulSyllableDeos',
u'\u50e6': 'CjkUnifiedIdeograph50e6',
u'\uf469': 'UnicodeCharacter0xf469',
u'\uc96b': 'HangulSyllableJwih',
u'\u0a6d': 'GurmukhiDigitSeven',
u'\u4f6f': 'CjkUnifiedIdeograph4f6f',
u'\ua1f0': 'YiSyllableGe',
u'\ue2f2': 'UnicodeCharacter0xe2f2',
u'\u27f4': 'RightArrowWithCircledPlus',
u'\u78f6': 'CjkUnifiedIdeograph78f6',
u'\u9c79': 'CjkUnifiedIdeograph9c79',
u'\ud17b': 'HangulSyllableTyeos',
u'\u127d': 'EthiopicSyllableCe',
u'\u577f': 'CjkUnifiedIdeograph577f',
u'\u2b81': 'UnicodeCharacter0x2b81',
u'\ucc04': 'HangulSyllableJjyiss',
u'\ue087': 'UnicodeCharacter0xe087',
u'\u4608': 'CjkUnifiedIdeograph4608',
u'\u7a8b': 'CjkUnifiedIdeograph7a8b',
u'\u850a': 'CjkUnifiedIdeograph850a',
u'\ub98d': 'HangulSyllableReut',
u'\u1f0e': 'GreekCapitalLetterAlphaWithPsiliAndPerispomeni',
u'\u3391': 'SquareKhz',
u'\ud414': 'HangulSyllablePwals',
u'\u8897': 'CjkUnifiedIdeograph8897',
u'\u6e18': 'CjkUnifiedIdeograph6e18',
u'\u029b': 'LatinLetterSmallCapitalGWithHook',
u'\uad1a': 'HangulSyllableGwaegg',
u'\u271e': 'ShadowedWhiteLatinCross',
u'\udba1': 'UnicodeCharacter0xdba1',
u'\u5da5': 'CjkUnifiedIdeograph5da5',
u'\ufc24': 'ArabicLigatureDadWithKhahIsolatedForm',
u'\u90a7': 'CjkUnifiedIdeograph90a7',
u'\u7628': 'CjkUnifiedIdeograph7628',
u'\u2aab': 'LargerThan',
u'\ub52a': 'HangulSyllableDij',
u'\ucf2e': 'HangulSyllableKyeobs',
u'\ue3b1': 'UnicodeCharacter0xe3b1',
u'\u4132': 'CjkUnifiedIdeograph4132',
u'\u65b5': 'CjkUnifiedIdeograph65b5',
u'\u8434': 'CjkUnifiedIdeograph8434',
u'\ub8b7': 'HangulSyllableRoed',
u'\u1e38': 'LatinCapitalLetterLWithDotBelowAndMacron',
u'\u32bb': 'CircledNumberFortySix',
u'\ud73e': 'HangulSyllableHyulm',
u'\u8bc1': 'CjkUnifiedIdeograph8bc1',
u'\u6942': 'CjkUnifiedIdeograph6942',
u'\u0dc5': 'SinhalaLetterMuurdhajaLayanna',
u'\uac44': 'HangulSyllableGyals',
u'\u2648': 'Aries',
u'\udacb': 'UnicodeCharacter0xdacb',
u'\u5ccf': 'CjkUnifiedIdeograph5ccf',
u'\uff4e': 'FullwidthLatinSmallLetterN',
u'\u93d1': 'CjkUnifiedIdeograph93d1',
u'\u7152': 'CjkUnifiedIdeograph7152',
u'\u15d5': 'CanadianSyllabicsCarrierWa',
u'\ub454': 'HangulSyllableDun',
u'\uce58': 'HangulSyllableCi',
u'\ue2db': 'UnicodeCharacter0xe2db',
u'\u405c': 'CjkUnifiedIdeograph405c',
u'\u64df': 'CjkUnifiedIdeograph64df',
u'\u875e': 'CjkUnifiedIdeograph875e',
u'\ubbe1': 'HangulSyllableMyinj',
u'\u1962': 'TaiLeLetterNa',
u'\u3de5': 'CjkUnifiedIdeograph3de5',
u'\ud668': 'HangulSyllableHwass',
u'\u8aeb': 'CjkUnifiedIdeograph8aeb',
u'\u686c': 'CjkUnifiedIdeograph686c',
u'\u0cef': 'KannadaDigitNine',
u'\uaf6e': 'HangulSyllableGgwaelm',
u'\u2172': 'SmallRomanNumeralThree',
u'\uc5f5': 'HangulSyllableYeolg',
u'\u5ff9': 'CjkUnifiedIdeograph5ff9',
u'\ufe78': 'ArabicDammaIsolatedForm',
u'\u92fb': 'CjkUnifiedIdeograph92fb',
u'\u707c': 'CjkUnifiedIdeograph707c',
u'\u14ff': 'CanadianSyllabicsWestcreeSwoo',
u'\ub77e': 'HangulSyllableRagg',
u'\u2e03': 'RightSubstitutionBracket',
u'\uc982': 'HangulSyllableJyuj',
u'\u4386': 'CjkUnifiedIdeograph4386',
u'\ue709': 'UnicodeCharacter0xe709',
u'\u8688': 'CjkUnifiedIdeograph8688',
u'\u790d': 'CjkUnifiedIdeograph790d',
u'\u188c': 'MongolianLetterAliGaliTta',
u'\ubc0f': 'HangulSyllableMic',
u'\u3613': 'CjkUnifiedIdeograph3613',
u'\ud192': 'HangulSyllableTyelp',
u'\u4817': 'CjkUnifiedIdeograph4817',
u'\u6b96': 'CjkUnifiedIdeograph6b96',
u'\u8f19': 'CjkUnifiedIdeograph8f19',
u'\uae98': 'HangulSyllableGgyass',
u'\u011d': 'LatinSmallLetterGWithCircumflex',
u'\u209c': 'UnicodeCharacter0x209c',
u'\ude23': 'UnicodeCharacter0xde23',
u'\uf9a2': 'CjkCompatibilityIdeographf9a2',
u'\u5027': 'CjkUnifiedIdeograph5027',
u'\u73a6': 'CjkUnifiedIdeograph73a6',
u'\u9729': 'CjkUnifiedIdeograph9729',
u'\ub6a8': 'HangulSyllableDduls',
u'\u292d': 'SouthEastArrowCrossingNorthEastArrow',
u'\uc8ac': 'HangulSyllableJwaen',
u'\u42b0': 'CjkUnifiedIdeograph42b0',
u'\ue633': 'UnicodeCharacter0xe633',
u'\u81b2': 'CjkUnifiedIdeograph81b2',
u'\u7837': 'CjkUnifiedIdeograph7837',
u'\u1bb6': 'SundaneseDigitSix',
u'\ubf39': 'HangulSyllableBbyeng',
u'\u313d': 'HangulLetterRieulsios',
u'\ud0bc': 'HangulSyllableKik',
u'\u4b41': 'CjkUnifiedIdeograph4b41',
u'\u6ac0': 'CjkUnifiedIdeograph6ac0',
u'\u8e43': 'CjkUnifiedIdeograph8e43',
u'\ua9c2': 'UnicodeCharacter0xa9c2',
u'\u23c6': 'DentistrySymbolLightVerticalAndWave',
u'\ud94d': 'UnicodeCharacter0xd94d',
u'\uf8cc': 'UnicodeCharacter0xf8cc',
u'\u5351': 'CjkUnifiedIdeograph5351',
u'\u72d0': 'CjkUnifiedIdeograph72d0',
u'\u9653': 'CjkUnifiedIdeograph9653',
u'\ub1d2': 'HangulSyllableNoenh',
u'\u2857': 'BraillePatternDots12357',
u'\ucbd6': 'HangulSyllableJjeugg',
u'\u4dda': 'HexagramForMouthCorners',
u'\ue15d': 'UnicodeCharacter0xe15d',
u'\u80dc': 'CjkUnifiedIdeograph80dc',
u'\u7b61': 'CjkUnifiedIdeograph7b61',
u'\u1ae0': 'UnicodeCharacter0x1ae0',
u'\ube63': 'HangulSyllableBbags',
u'\u3067': 'HiraganaLetterDe',
u'\ud3e6': 'HangulSyllablePyej',
u'\u4a6b': 'CjkUnifiedIdeograph4a6b',
u'\u55ea': 'CjkUnifiedIdeograph55ea',
u'\u896d': 'CjkUnifiedIdeograph896d',
u'\ua8ec': 'UnicodeCharacter0xa8ec',
u'\u0371': 'GreekSmallLetterHeta',
u'\u22f0': 'UpRightDiagonalEllipsis',
u'\ud877': 'UnicodeCharacter0xd877',
u'\ufbf6': 'ArabicLigatureYehWithHamzaAboveWithEIsolatedForm',
u'\u527b': 'CjkUnifiedIdeograph527b',
u'\u7dfa': 'CjkUnifiedIdeograph7dfa',
u'\u917d': 'CjkUnifiedIdeograph917d',
u'\ub0fc': 'HangulSyllableNyaem',
u'\uaa81': 'UnicodeCharacter0xaa81',
u'\ucb00': 'HangulSyllableJjwaels',
u'\uef83': 'UnicodeCharacter0xef83',
u'\u0802': 'UnicodeCharacter0x802',
u'\u2c85': 'CopticSmallLetterGamma',
u'\u4d04': 'CjkUnifiedIdeograph4d04',
u'\u6187': 'CjkUnifiedIdeograph6187',
u'\u9e0e': 'CjkUnifiedIdeograph9e0e',
u'\ub291': 'HangulSyllableNeug',
u'\ud310': 'HangulSyllablePan',
u'\uf793': 'UnicodeCharacter0xf793',
u'\u1012': 'MyanmarLetterDa',
u'\u3495': 'CjkUnifiedIdeograph3495',
u'\u5514': 'CjkUnifiedIdeograph5514',
u'\u0997': 'BengaliLetterGa',
u'\u4e99': 'CjkUnifiedIdeograph4e99',
u'\ua61e': 'VaiSymbolDoo',
u'\ufb20': 'HebrewLetterAlternativeAyin',
u'\u9fa3': 'CjkUnifiedIdeograph9fa3',
u'\u3822': 'CjkUnifiedIdeograph3822',
u'\udca5': 'UnicodeCharacter0xdca5',
u'\u7d24': 'CjkUnifiedIdeograph7d24',
u'\u11a7': 'UnicodeCharacter0x11a7',
u'\u56a9': 'CjkUnifiedIdeograph56a9',
u'\u8330': 'CjkUnifiedIdeograph8330',
u'\ua7b3': 'UnicodeCharacter0xa7b3',
u'\uc032': 'HangulSyllableBbwibs',
u'\ue4b5': 'UnicodeCharacter0xe4b5',
u'\u0534': 'ArmenianCapitalLetterDa',
u'\u39b7': 'CjkUnifiedIdeograph39b7',
u'\u5a36': 'CjkUnifiedIdeograph5a36',
u'\u7eb9': 'CjkUnifiedIdeograph7eb9',
u'\uab40': 'UnicodeCharacter0xab40',
u'\ue842': 'UnicodeCharacter0xe842',
u'\u8cc5': 'CjkUnifiedIdeograph8cc5',
u'\u2d44': 'TifinaghLetterYaa',
u'\uc1c7': 'HangulSyllableSwaegs',
u'\u6246': 'CjkUnifiedIdeograph6246',
u'\u06c9': 'ArabicLetterKirghizYu',
u'\u5bcb': 'CjkUnifiedIdeograph5bcb',
u'\ub350': 'HangulSyllableDyaek',
u'\uf052': 'UnicodeCharacter0xf052',
u'\u94d5': 'CjkUnifiedIdeograph94d5',
u'\u3554': 'CjkUnifiedIdeograph3554',
u'\ue9d7': 'UnicodeCharacter0xe9d7',
u'\u0a56': 'UnicodeCharacter0xa56',
u'\u2ed9': 'CjkRadicalCsimplifiedTannedLeather',
u'\u4f58': 'CjkUnifiedIdeograph4f58',
u'\u63db': 'CjkUnifiedIdeograph63db',
u'\u9862': 'CjkUnifiedIdeograph9862',
u'\ubce5': 'HangulSyllableByelt',
u'\udd64': 'UnicodeCharacter0xdd64',
u'\uf1e7': 'UnicodeCharacter0xf1e7',
u'\u1266': 'EthiopicSyllableBo',
u'\u36e9': 'CjkUnifiedIdeograph36e9',
u'\u5768': 'CjkUnifiedIdeograph5768',
u'\u0beb': 'TamilDigitFive',
u'\u48ed': 'CjkUnifiedIdeograph48ed',
u'\ua072': 'YiSyllableBbyt',
u'\ue574': 'UnicodeCharacter0xe574',
u'\u99f7': 'CjkUnifiedIdeograph99f7',
u'\u3a76': 'CjkUnifiedIdeograph3a76',
u'\udef9': 'UnicodeCharacter0xdef9',
u'\u7f78': 'CjkUnifiedIdeograph7f78',
u'\u13fb': 'UnicodeCharacter0x13fb',
u'\u50fd': 'CjkUnifiedIdeograph50fd',
u'\u6a01': 'CjkUnifiedIdeograph6a01',
u'\u8d84': 'CjkUnifiedIdeograph8d84',
u'\uc286': 'HangulSyllableSwip',
u'\u0788': 'ThaanaLetterVaavu',
u'\ubb0b': 'HangulSyllableMoelh',
u'\u448a': 'CjkUnifiedIdeograph448a',
u'\uf80d': 'UnicodeCharacter0xf80d',
u'\u3d0f': 'CjkUnifiedIdeograph3d0f',
u'\u7211': 'CjkUnifiedIdeograph7211',
u'\u9594': 'CjkUnifiedIdeograph9594',
u'\uea96': 'UnicodeCharacter0xea96',
u'\u2f98': 'KangxiRadicalBadger',
u'\u6c9a': 'CjkUnifiedIdeograph6c9a',
u'\u801d': 'CjkUnifiedIdeograph801d',
u'\uc51f': 'HangulSyllableSsyis',
u'\u1a21': 'UnicodeCharacter0x1a21',
u'\u5f23': 'CjkUnifiedIdeograph5f23',
u'\ubda4': 'HangulSyllableBweol',
u'\uf2a6': 'UnicodeCharacter0xf2a6',
u'\u37a8': 'CjkUnifiedIdeograph37a8',
u'\u74aa': 'CjkUnifiedIdeograph74aa',
u'\ua82d': 'UnicodeCharacter0xa82d',
u'\u49ac': 'CjkUnifiedIdeograph49ac',
u'\ued2f': 'UnicodeCharacter0xed2f',
u'\u2231': 'ClockwiseIntegral',
u'\u6733': 'CjkUnifiedIdeograph6733',
u'\u9ab6': 'CjkUnifiedIdeograph9ab6',
u'\udfb8': 'UnicodeCharacter0xdfb8',
u'\u1cba': 'UnicodeCharacter0x1cba',
u'\ub03d': 'HangulSyllableGgyung',
u'\u51bc': 'CjkUnifiedIdeograph51bc',
u'\uf53f': 'UnicodeCharacter0xf53f',
u'\uca41': 'HangulSyllableJjyaeb',
u'\u0f43': 'TibetanLetterGha',
u'\u4c45': 'CjkUnifiedIdeograph4c45',
u'\ua2c6': 'YiSyllableCu',
u'\ue7c8': 'UnicodeCharacter0xe7c8',
u'\u24ca': 'CircledLatinCapitalLetterU',
u'\u79cc': 'CjkUnifiedIdeograph79cc',
u'\u9d4f': 'CjkUnifiedIdeograph9d4f',
u'\ud251': 'HangulSyllableTweolg',
u'\u1753': 'BuhidVowelSignU',
u'\u5455': 'CjkUnifiedIdeograph5455',
u'\u8fd8': 'CjkUnifiedIdeograph8fd8',
u'\uccda': 'HangulSyllableCyeolm',
u'\u01dc': 'LatinSmallLetterUWithDiaeresisAndGrave',
u'\ua55f': 'VaiSyllableRa',
u'\u46de': 'CjkUnifiedIdeograph46de',
u'\ufa61': 'CjkCompatibilityIdeographfa61',
u'\u3f63': 'CjkUnifiedIdeograph3f63',
u'\u7c65': 'CjkUnifiedIdeograph7c65',
u'\u97e8': 'CjkUnifiedIdeograph97e8',
u'\ud4ea': 'HangulSyllablePyugg',
u'\u29ec': 'WhiteCircleWithDownArrow',
u'\u6eee': 'CjkUnifiedIdeograph6eee',
u'\u8271': 'CjkUnifiedIdeograph8271',
u'\uc773': 'HangulSyllableYih',
u'\u0475': 'CyrillicSmallLetterIzhitsa',
u'\u5977': 'CjkUnifiedIdeograph5977',
u'\ubff8': 'HangulSyllableBbweom',
u'\ufcfa': 'ArabicLigatureGhainWithYehIsolatedForm',
u'\u31fc': 'KatakanaLetterSmallRi',
u'\u76fe': 'CjkUnifiedIdeograph76fe',
u'\u4c00': 'CjkUnifiedIdeograph4c00',
u'\u6883': 'CjkUnifiedIdeograph6883',
u'\u8f02': 'CjkUnifiedIdeograph8f02',
u'\uab85': 'UnicodeCharacter0xab85',
u'\u0506': 'CyrillicCapitalLetterKomiDzje',
u'\u2189': 'UnicodeCharacter0x2189',
u'\ude0c': 'UnicodeCharacter0xde0c',
u'\ufa8f': 'CjkCompatibilityIdeographfa8f',
u'\u5410': 'CjkUnifiedIdeograph5410',
u'\u7093': 'CjkUnifiedIdeograph7093',
u'\u9712': 'CjkUnifiedIdeograph9712',
u'\ub395': 'HangulSyllableDyeolg',
u'\u2d16': 'GeorgianSmallLetterGhan',
u'\uc999': 'HangulSyllableJeub',
u'\u479d': 'CjkUnifiedIdeograph479d',
u'\ue61c': 'UnicodeCharacter0xe61c',
u'\u829f': 'CjkUnifiedIdeograph829f',
u'\u7c20': 'CjkUnifiedIdeograph7c20',
u'\u18a3': 'MongolianLetterManchuAliGaliCya',
u'\ubf22': 'HangulSyllableBbyeop',
u'\u3526': 'CjkUnifiedIdeograph3526',
u'\ud1a9': 'HangulSyllableTolg',
u'\u6fad': 'CjkUnifiedIdeograph6fad',
u'\u8e2c': 'CjkUnifiedIdeograph8e2c',
u'\uaaaf': 'UnicodeCharacter0xaaaf',
u'\u0430': 'CyrillicSmallLetterA',
u'\u20b3': 'AustralSign',
u'\udd36': 'UnicodeCharacter0xdd36',
u'\uf9b9': 'CjkCompatibilityIdeographf9b9',
u'\u5b3a': 'CjkUnifiedIdeograph5b3a',
u'\u77bd': 'CjkUnifiedIdeograph77bd',
u'\u963c': 'CjkUnifiedIdeograph963c',
u'\ub2bf': 'HangulSyllableNyis',
u'\u2c40': 'GlagoliticSmallLetterNashi',
u'\uc8c3': 'HangulSyllableJwaeh',
u'\u46c7': 'CjkUnifiedIdeograph46c7',
u'\ue546': 'UnicodeCharacter0xe546',
u'\u81c9': 'CjkUnifiedIdeograph81c9',
u'\u634a': 'CjkUnifiedIdeograph634a',
u'\u1fcd': 'GreekPsiliAndVaria',
u'\ube4c': 'HangulSyllableBil',
u'\u3450': 'CjkUnifiedIdeograph3450',
u'\ud0d3': 'HangulSyllableTas',
u'\u6ed7': 'CjkUnifiedIdeograph6ed7',
u'\u8d56': 'CjkUnifiedIdeograph8d56',
u'\ua9d9': 'UnicodeCharacter0xa9d9',
u'\u0b5a': 'UnicodeCharacter0xb5a',
u'\u27dd': 'LongRightTack',
u'\udc60': 'UnicodeCharacter0xdc60',
u'\uf8e3': 'UnicodeCharacter0xf8e3',
u'\u5a64': 'CjkUnifiedIdeograph5a64',
u'\u76e7': 'CjkUnifiedIdeograph76e7',
u'\u9566': 'CjkUnifiedIdeograph9566',
u'\ub1e9': 'HangulSyllableNyog',
u'\u136a': 'EthiopicDigitTwo',
u'\ucfed': 'HangulSyllableKult',
u'\u45f1': 'CjkUnifiedIdeograph45f1',
u'\ue470': 'UnicodeCharacter0xe470',
u'\u80f3': 'CjkUnifiedIdeograph80f3',
u'\u6274': 'CjkUnifiedIdeograph6274',
u'\u1ef7': 'LatinSmallLetterYWithHookAbove',
u'\ubd76': 'HangulSyllableByobs',
u'\u3b7a': 'CjkUnifiedIdeograph3b7a',
u'\ud7fd': 'UnicodeCharacter0xd7fd',
u'\ued01': 'UnicodeCharacter0xed01',
u'\u8c80': 'CjkUnifiedIdeograph8c80',
u'\u6b05': 'CjkUnifiedIdeograph6b05',
u'\u0a84': 'UnicodeCharacter0xa84',
u'\ua607': 'VaiSyllableGen',
u'\u3c0b': 'CjkUnifiedIdeograph3c0b',
u'\uc38a': 'HangulSyllableSsenh',
u'\u598e': 'CjkUnifiedIdeograph598e',
u'\uf511': 'UnicodeCharacter0xf511',
u'\u9490': 'CjkUnifiedIdeograph9490',
u'\u7315': 'CjkUnifiedIdeograph7315',
u'\u1294': 'EthiopicSyllableNee',
u'\uc41b': 'HangulSyllableSswaelb',
u'\ueb9a': 'UnicodeCharacter0xeb9a',
u'\u421f': 'CjkUnifiedIdeograph421f',
u'\u619e': 'CjkUnifiedIdeograph619e',
u'\u9d21': 'CjkUnifiedIdeograph9d21',
u'\ubca0': 'HangulSyllableBe',
u'\u1b25': 'BalineseLetterDaMadu',
u'\u3aa4': 'CjkUnifiedIdeograph3aa4',
u'\uec2b': 'UnicodeCharacter0xec2b',
u'\uf3aa': 'UnicodeCharacter0xf3aa',
u'\u6a2f': 'CjkUnifiedIdeograph6a2f',
u'\u09ae': 'BengaliLetterMa',
u'\ua531': 'VaiSyllableFi',
u'\u2335': 'Countersink',
u'\uc2b4': 'HangulSyllableSeum',
u'\u58b8': 'CjkUnifiedIdeograph58b8',
u'\uf43b': 'UnicodeCharacter0xf43b',
u'\u9bba': 'CjkUnifiedIdeograph9bba',
u'\u723f': 'CjkUnifiedIdeograph723f',
u'\u11be': 'HangulJongseongChieuch',
u'\ucb45': 'HangulSyllableJjyot',
u'\ueac4': 'UnicodeCharacter0xeac4',
u'\u4149': 'CjkUnifiedIdeograph4149',
u'\u60c8': 'CjkUnifiedIdeograph60c8',
u'\u9c4b': 'CjkUnifiedIdeograph9c4b',
u'\ua3ca': 'YiSyllableRuox',
u'\u1a4f': 'UnicodeCharacter0x1a4f',
u'\u39ce': 'CjkUnifiedIdeograph39ce',
u'\ud355': 'HangulSyllablePyab',
u'\uf2d4': 'UnicodeCharacter0xf2d4',
u'\u6959': 'CjkUnifiedIdeograph6959',
u'\u08d8': 'UnicodeCharacter0x8d8',
u'\ua45b': 'YiSyllableNyu',
u'\u225f': 'QuestionedEqualTo',
u'\uc1de': 'HangulSyllableSwaep',
u'\u5fe2': 'CjkUnifiedIdeograph5fe2',
u'\ufb65': 'ArabicLetterTehehMedialForm',
u'\u9ae4': 'CjkUnifiedIdeograph9ae4',
u'\u7169': 'CjkUnifiedIdeograph7169',
u'\u10e8': 'GeorgianLetterShin',
u'\uca6f': 'HangulSyllableJjed',
u'\ue9ee': 'UnicodeCharacter0xe9ee',
u'\u4073': 'CjkUnifiedIdeograph4073',
u'\u67f2': 'CjkUnifiedIdeograph67f2',
u'\u8375': 'CjkUnifiedIdeograph8375',
u'\ua2f4': 'YiSyllableNzat',
u'\u1979': 'UnicodeCharacter0x1979',
u'\u38f8': 'CjkUnifiedIdeograph38f8',
u'\ud27f': 'HangulSyllableTweh',
u'\uf1fe': 'UnicodeCharacter0xf1fe',
u'\u8406': 'CjkUnifiedIdeograph8406',
u'\ua089': 'YiSyllableNburx',
u'\uc108': 'HangulSyllableSyael',
u'\ufd8b': 'ArabicLigatureMeemWithHahWithYehFinalForm',
u'\u020a': 'LatinCapitalLetterIWithInvertedBreve',
u'\u3e8d': 'CjkUnifiedIdeograph3e8d',
u'\u5f0c': 'CjkUnifiedIdeograph5f0c',
u'\u7b8f': 'CjkUnifiedIdeograph7b8f',
u'\uac16': 'HangulSyllableGaj',
u'\ue918': 'UnicodeCharacter0xe918',
u'\u859b': 'CjkUnifiedIdeograph859b',
u'\u2a1a': 'IntegralWithUnion',
u'\uc69d': 'HangulSyllableYolg',
u'\u671c': 'CjkUnifiedIdeograph671c',
u'\u039f': 'GreekCapitalLetterOmicron',
u'\u5ca1': 'CjkUnifiedIdeograph5ca1',
u'\ub426': 'HangulSyllableDoelp',
u'\uf128': 'UnicodeCharacter0xf128',
u'\uadab': 'HangulSyllableGwed',
u'\u322a': 'ParenthesizedIdeographMoon',
u'\ueead': 'UnicodeCharacter0xeead',
u'\u0f2c': 'TibetanDigitHalfThree',
u'\u2baf': 'UnicodeCharacter0x2baf',
u'\u482e': 'CjkUnifiedIdeograph482e',
u'\u64b1': 'CjkUnifiedIdeograph64b1',
u'\u9938': 'CjkUnifiedIdeograph9938',
u'\ub5bb': 'HangulSyllableDdeoh',
u'\uda3a': 'UnicodeCharacter0xda3a',
u'\uf6bd': 'UnicodeCharacter0xf6bd',
u'\u173c': 'UnicodeCharacter0x173c',
u'\u33bf': 'SquareMwMega',
u'\u503e': 'CjkUnifiedIdeograph503e',
u'\u0cc1': 'KannadaVowelSignU',
u'\u49c3': 'CjkUnifiedIdeograph49c3',
u'\ua148': 'YiSyllableDdex',
u'\ue24a': 'UnicodeCharacter0xe24a',
u'\u9ecd': 'CjkUnifiedIdeograph9ecd',
u'\u3f4c': 'CjkUnifiedIdeograph3f4c',
u'\udbcf': 'UnicodeCharacter0xdbcf',
u'\u784e': 'CjkUnifiedIdeograph784e',
u'\u14d1': 'CanadianSyllabicsCarrierNg',
u'\u51d3': 'CjkUnifiedIdeograph51d3',
u'\u8a5a': 'CjkUnifiedIdeograph8a5a',
u'\ua6dd': 'UnicodeCharacter0xa6dd',
u'\uc75c': 'HangulSyllableYin',
u'\ue3df': 'UnicodeCharacter0xe3df',
u'\u3ce1': 'CjkUnifiedIdeograph3ce1',
u'\u5d60': 'CjkUnifiedIdeograph5d60',
u'\u79e3': 'CjkUnifiedIdeograph79e3',
u'\u926a': 'CjkUnifiedIdeograph926a',
u'\uef6c': 'UnicodeCharacter0xef6c',
u'\u8bef': 'CjkUnifiedIdeograph8bef',
u'\u286e': 'BraillePatternDots23467',
u'\uc4f1': 'HangulSyllableSseug',
u'\u6570': 'CjkUnifiedIdeograph6570',
u'\u01f3': 'LatinSmallLetterDz',
u'\u42f5': 'CjkUnifiedIdeograph42f5',
u'\uba7a': 'HangulSyllableMyeolm',
u'\uf77c': 'UnicodeCharacter0xf77c',
u'\u93ff': 'CjkUnifiedIdeograph93ff',
u'\u307e': 'HiraganaLetterMa',
u'\u0d80': 'UnicodeCharacter0xd80',
u'\ua903': 'KayahLiDigitThree',
u'\u4e82': 'CjkUnifiedIdeograph4e82',
u'\uea05': 'UnicodeCharacter0xea05',
u'\u2707': 'TapeDrive',
u'\u6009': 'CjkUnifiedIdeograph6009',
u'\u9f8c': 'CjkUnifiedIdeograph9f8c',
u'\ud88e': 'UnicodeCharacter0xd88e',
u'\u1590': 'CanadianSyllabicsNgii',
u'\ub113': 'HangulSyllableNeolb',
u'\u5692': 'CjkUnifiedIdeograph5692',
u'\uf215': 'UnicodeCharacter0xf215',
u'\ucf17': 'HangulSyllableKec',
u'\u0819': 'UnicodeCharacter0x819',
u'\u451b': 'CjkUnifiedIdeograph451b',
u'\ua79c': 'UnicodeCharacter0xa79c',
u'\ue09e': 'UnicodeCharacter0xe09e',
u'\u3da0': 'CjkUnifiedIdeograph3da0',
u'\u7ea2': 'CjkUnifiedIdeograph7ea2',
u'\u9a25': 'CjkUnifiedIdeograph9a25',
u'\ud727': 'HangulSyllableHwilh',
u'\u1029': 'MyanmarLetterO',
u'\u6d2b': 'CjkUnifiedIdeograph6d2b',
u'\u88ae': 'CjkUnifiedIdeograph88ae',
u'\uc5b0': 'HangulSyllableYaek',
u'\u06b2': 'ArabicLetterGafWithTwoDotsBelow',
u'\ua235': 'YiSyllableMgap',
u'\u43b4': 'CjkUnifiedIdeograph43b4',
u'\uff37': 'FullwidthLatinCapitalLetterW',
u'\u3839': 'CjkUnifiedIdeograph3839',
u'\u753b': 'CjkUnifiedIdeograph753b',
u'\u90be': 'CjkUnifiedIdeograph90be',
u'\uedc0': 'UnicodeCharacter0xedc0',
u'\u2ec2': 'CjkRadicalClothes',
u'\u6bc4': 'CjkUnifiedIdeograph6bc4',
u'\u8747': 'CjkUnifiedIdeograph8747',
u'\uc049': 'HangulSyllableBbyult',
u'\u1d4b': 'ModifierLetterSmallOpenE',
u'\u5e4d': 'CjkUnifiedIdeograph5e4d',
u'\ub8ce': 'HangulSyllableRyogg',
u'\uf5d0': 'UnicodeCharacter0xf5d0',
u'\u36d2': 'CjkUnifiedIdeograph36d2',
u'\u73d4': 'CjkUnifiedIdeograph73d4',
u'\uaf57': 'HangulSyllableGgwalh',
u'\u4cd6': 'CjkUnifiedIdeograph4cd6',
u'\ue859': 'UnicodeCharacter0xe859',
u'\u255b': 'BoxDrawingsUpSingleAndLeftDouble',
u'\u665d': 'CjkUnifiedIdeograph665d',
u'\u9de0': 'CjkUnifiedIdeograph9de0',
u'\udee2': 'UnicodeCharacter0xdee2',
u'\u1be4': 'UnicodeCharacter0x1be4',
u'\ub767': 'HangulSyllableDdid',
u'\u54e6': 'CjkUnifiedIdeograph54e6',
u'\uf069': 'UnicodeCharacter0xf069',
u'\ucd6b': 'HangulSyllableCoelh',
u'\u0e6d': 'UnicodeCharacter0xe6d',
u'\u4b6f': 'CjkUnifiedIdeograph4b6f',
u'\ua5f0': 'VaiSyllableGben',
u'\ue6f2': 'UnicodeCharacter0xe6f2',
u'\u23f4': 'UnicodeCharacter0x23f4',
u'\u7cf6': 'CjkUnifiedIdeograph7cf6',
u'\u9879': 'CjkUnifiedIdeograph9879',
u'\ud57b': 'HangulSyllableHaed',
u'\u167d': 'UnicodeCharacter0x167d',
u'\u537f': 'CjkUnifiedIdeograph537f',
u'\u2f81': 'KangxiRadicalMeat',
u'\uc804': 'HangulSyllableJeon',
u'\ue487': 'UnicodeCharacter0xe487',
u'\u4208': 'CjkUnifiedIdeograph4208',
u'\u7e8b': 'CjkUnifiedIdeograph7e8b',
u'\u810a': 'CjkUnifiedIdeograph810a',
u'\ubd8d': 'HangulSyllableBult',
u'\u1b0e': 'BalineseLetterLaLengaTedung',
u'\u3791': 'CjkUnifiedIdeograph3791',
u'\ud014': 'HangulSyllableKweok',
u'\u8c97': 'CjkUnifiedIdeograph8c97',
u'\u6a18': 'CjkUnifiedIdeograph6a18',
u'\u069b': 'ArabicLetterSeenWithThreeDotsBelow',
u'\ua91a': 'KayahLiLetterRa',
u'\u231e': 'BottomLeftCorner',
u'\udfa1': 'UnicodeCharacter0xdfa1',
u'\u59a5': 'CjkUnifiedIdeograph59a5',
u'\uf824': 'UnicodeCharacter0xf824',
u'\u94a7': 'CjkUnifiedIdeograph94a7',
u'\u7228': 'CjkUnifiedIdeograph7228',
u'\u2eab': 'CjkRadicalEye',
u'\ub12a': 'HangulSyllableNenh',
u'\ucb2e': 'HangulSyllableJjyogg',
u'\ue7b1': 'UnicodeCharacter0xe7b1',
u'\u4532': 'CjkUnifiedIdeograph4532',
u'\u61b5': 'CjkUnifiedIdeograph61b5',
u'\u8034': 'CjkUnifiedIdeograph8034',
u'\ubcb7': 'HangulSyllableBec',
u'\u1a38': 'UnicodeCharacter0x1a38',
u'\u36bb': 'CjkUnifiedIdeograph36bb',
u'\ud33e': 'HangulSyllablePaej',
u'\u8fc1': 'CjkUnifiedIdeograph8fc1',
u'\u6d42': 'CjkUnifiedIdeograph6d42',
u'\u09c5': 'UnicodeCharacter0x9c5',
u'\ua844': 'PhagspaLetterCa',
u'\u2248': 'AlmostEqualTo',
u'\udecb': 'UnicodeCharacter0xdecb',
u'\u58cf': 'CjkUnifiedIdeograph58cf',
u'\ufb4e': 'HebrewLetterPeWithRafe',
u'\u97d1': 'CjkUnifiedIdeograph97d1',
u'\u7552': 'CjkUnifiedIdeograph7552',
u'\u11d5': 'HangulJongseongRieulkapyeounpieup',
u'\ub054': 'HangulSyllableGgeum',
u'\uca58': 'HangulSyllableJjeols',
u'\ue6db': 'UnicodeCharacter0xe6db',
u'\u445c': 'CjkUnifiedIdeograph445c',
u'\u60df': 'CjkUnifiedIdeograph60df',
u'\u835e': 'CjkUnifiedIdeograph835e',
u'\ubfe1': 'HangulSyllableBbung',
u'\u1d62': 'LatinSubscriptSmallLetterI',
u'\u39e5': 'CjkUnifiedIdeograph39e5',
u'\ud268': 'HangulSyllableTwen',
u'\u8eeb': 'CjkUnifiedIdeograph8eeb',
u'\u6c6c': 'CjkUnifiedIdeograph6c6c',
u'\u08ef': 'UnicodeCharacter0x8ef',
u'\uab6e': 'UnicodeCharacter0xab6e',
u'\u2572': 'BoxDrawingsLightDiagonalUpperLeftToLowerRight',
u'\uc1f5': 'HangulSyllableSoeng',
u'\u5bf9': 'CjkUnifiedIdeograph5bf9',
u'\ufa78': 'CjkCompatibilityIdeographfa78',
u'\u96fb': 'CjkUnifiedIdeograph96fb',
u'\u747c': 'CjkUnifiedIdeograph747c',
u'\u10ff': 'UnicodeCharacter0x10ff',
u'\ub37e': 'HangulSyllableDelp',
u'\u2a03': 'NaryUnionOperatorWithDot',
u'\ucd82': 'HangulSyllableCyolm',
u'\u4786': 'CjkUnifiedIdeograph4786',
u'\ue309': 'UnicodeCharacter0xe309',
u'\u8288': 'CjkUnifiedIdeograph8288',
u'\u7d0d': 'CjkUnifiedIdeograph7d0d',
u'\u1c8c': 'UnicodeCharacter0x1c8c',
u'\ub80f': 'HangulSyllableRed',
u'\u3213': 'ParenthesizedHangulPieupA',
u'\ud592': 'HangulSyllableHyagg',
u'\u4c17': 'CjkUnifiedIdeograph4c17',
u'\u6f96': 'CjkUnifiedIdeograph6f96',
u'\u8b19': 'CjkUnifiedIdeograph8b19',
u'\uaa98': 'UnicodeCharacter0xaa98',
u'\u051d': 'CyrillicSmallLetterWe',
u'\u249c': 'ParenthesizedLatinSmallLetterA',
u'\uda23': 'UnicodeCharacter0xda23',
u'\ufda2': 'ArabicLigatureTehWithKhahWithAlefMaksuraFinalForm',
u'\u5427': 'CjkUnifiedIdeograph5427',
u'\u77a6': 'CjkUnifiedIdeograph77a6',
u'\u9329': 'CjkUnifiedIdeograph9329',
u'\ub2a8': 'HangulSyllableNeuk',
u'\u2d2d': 'UnicodeCharacter0x2d2d',
u'\uccac': 'HangulSyllableCeoss',
u'\u46b0': 'CjkUnifiedIdeograph46b0',
u'\ue233': 'UnicodeCharacter0xe233',
u'\u85b2': 'CjkUnifiedIdeograph85b2',
u'\u7c37': 'CjkUnifiedIdeograph7c37',
u'\u1fb6': 'GreekSmallLetterAlphaWithPerispomeni',
u'\ubb39': 'HangulSyllableMunj',
u'\u353d': 'CjkUnifiedIdeograph353d',
u'\ud4bc': 'HangulSyllablePwels',
u'\u4f41': 'CjkUnifiedIdeograph4f41',
u'\u6ec0': 'CjkUnifiedIdeograph6ec0',
u'\u8a43': 'CjkUnifiedIdeograph8a43',
u'\uadc2': 'HangulSyllableGwigg',
u'\u0447': 'CyrillicSmallLetterChe',
u'\u27c6': 'RightSshapedBagDelimiter',
u'\udd4d': 'UnicodeCharacter0xdd4d',
u'\ufccc': 'ArabicLigatureLamWithMeemInitialForm',
u'\u5751': 'CjkUnifiedIdeograph5751',
u'\u76d0': 'CjkUnifiedIdeograph76d0',
u'\u9253': 'CjkUnifiedIdeograph9253',
u'\ub5d2': 'HangulSyllableDdej',
u'\u2c57': 'GlagoliticSmallLetterIotatedSmallYus',
u'\ucfd6': 'HangulSyllableKyobs',
u'\u49da': 'CjkUnifiedIdeograph49da',
u'\ue55d': 'UnicodeCharacter0xe55d',
u'\u84dc': 'CjkUnifiedIdeograph84dc',
u'\u7f61': 'CjkUnifiedIdeograph7f61',
u'\u1ee0': 'LatinCapitalLetterOWithHornAndTilde',
u'\uba63': 'HangulSyllableMelh',
u'\u3467': 'CjkUnifiedIdeograph3467',
u'\ud7e6': 'UnicodeCharacter0xd7e6',
u'\u4e6b': 'CjkUnifiedIdeograph4e6b',
u'\u51ea': 'CjkUnifiedIdeograph51ea',
u'\u8d6d': 'CjkUnifiedIdeograph8d6d',
u'\uacec': 'HangulSyllableGols',
u'\u0771': 'ArabicLetterRehWithSmallArabicLetterTahAndTwoDots',
u'\u26f0': 'UnicodeCharacter0x26f0',
u'\udc77': 'UnicodeCharacter0xdc77',
u'\ufff6': 'UnicodeCharacter0xfff6',
u'\u567b': 'CjkUnifiedIdeograph567b',
u'\u79fa': 'CjkUnifiedIdeograph79fa',
u'\u957d': 'CjkUnifiedIdeograph957d',
u'\ub4fc': 'HangulSyllableDyin',
u'\uae81': 'HangulSyllableGgaet',
u'\ucf00': 'HangulSyllableKe',
u'\ueb83': 'UnicodeCharacter0xeb83',
u'\u0c02': 'TeluguSignAnusvara',
u'\u2885': 'BraillePatternDots138',
u'\u4904': 'CjkUnifiedIdeograph4904',
u'\u6587': 'CjkUnifiedIdeograph6587',
u'\u9a0e': 'CjkUnifiedIdeograph9a0e',
u'\ub691': 'HangulSyllableDdyob',
u'\ud710': 'HangulSyllableHwess',
u'\uf393': 'UnicodeCharacter0xf393',
u'\u1412': 'CanadianSyllabicsWo',
u'\u3095': 'HiraganaLetterSmallKa',
u'\u5114': 'CjkUnifiedIdeograph5114',
u'\u0d97': 'UnicodeCharacter0xd97',
u'\u4a99': 'CjkUnifiedIdeograph4a99',
u'\ua21e': 'YiSyllableGguot',
u'\uff20': 'FullwidthCommercialAt',
u'\u9ba3': 'CjkUnifiedIdeograph9ba3',
u'\u3c22': 'CjkUnifiedIdeograph3c22',
u'\ud8a5': 'UnicodeCharacter0xd8a5',
u'\u7924': 'CjkUnifiedIdeograph7924',
u'\u15a7': 'CanadianSyllabicsThcreeThe',
u'\u52a9': 'CjkUnifiedIdeograph52a9',
u'\u8730': 'CjkUnifiedIdeograph8730',
u'\ua3b3': 'YiSyllableShox',
u'\uc432': 'HangulSyllableSsoenh',
u'\ue0b5': 'UnicodeCharacter0xe0b5',
u'\u0134': 'LatinCapitalLetterJWithCircumflex',
u'\u3db7': 'CjkUnifiedIdeograph3db7',
u'\u5e36': 'CjkUnifiedIdeograph5e36',
u'\u7ab9': 'CjkUnifiedIdeograph7ab9',
u'\uaf40': 'HangulSyllableGgoss',
u'\uec42': 'UnicodeCharacter0xec42',
u'\u88c5': 'CjkUnifiedIdeograph88c5',
u'\u2944': 'ShortRightwardsArrowAboveLeftwardsArrow',
u'\uc5c7': 'HangulSyllableEos',
u'\u6646': 'CjkUnifiedIdeograph6646',
u'\u02c9': 'ModifierLetterMacron',
u'\u5fcb': 'CjkUnifiedIdeograph5fcb',
u'\ub750': 'HangulSyllableDdyils',
u'\uf452': 'UnicodeCharacter0xf452',
u'\u90d5': 'CjkUnifiedIdeograph90d5',
u'\u3154': 'HangulLetterE',
u'\uedd7': 'UnicodeCharacter0xedd7',
u'\u0e56': 'ThaiDigitSix',
u'\u2ad9': 'ElementOfOpeningDownwards',
u'\u4b58': 'CjkUnifiedIdeograph4b58',
u'\u67db': 'CjkUnifiedIdeograph67db',
u'\u9c62': 'CjkUnifiedIdeograph9c62',
u'\ub8e5': 'HangulSyllableRyot',
u'\ud964': 'UnicodeCharacter0xd964',
u'\uf5e7': 'UnicodeCharacter0xf5e7',
u'\u1666': 'CanadianSyllabicsCarrierCha',
u'\u32e9': 'CircledKatakanaHa',
u'\u5368': 'CjkUnifiedIdeograph5368',
u'\u0feb': 'UnicodeCharacter0xfeb',
u'\u4ced': 'CjkUnifiedIdeograph4ced',
u'\ua472': 'YiSyllableYix',
u'\ue174': 'UnicodeCharacter0xe174',
u'\u9df7': 'CjkUnifiedIdeograph9df7',
u'\u3e76': 'CjkUnifiedIdeograph3e76',
u'\udaf9': 'UnicodeCharacter0xdaf9',
u'\u7b78': 'CjkUnifiedIdeograph7b78',
u'\u17fb': 'UnicodeCharacter0x17fb',
u'\u54fd': 'CjkUnifiedIdeograph54fd',
u'\u6e01': 'CjkUnifiedIdeograph6e01',
u'\u8984': 'CjkUnifiedIdeograph8984',
u'\uc686': 'HangulSyllableOelp',
u'\u0388': 'GreekCapitalLetterEpsilonWithTonos',
u'\ubf0b': 'HangulSyllableBbyeogs',
u'\u408a': 'CjkUnifiedIdeograph408a',
u'\ufc0d': 'ArabicLigatureTehWithKhahIsolatedForm',
u'\u390f': 'CjkUnifiedIdeograph390f',
u'\u7611': 'CjkUnifiedIdeograph7611',
u'\u9194': 'CjkUnifiedIdeograph9194',
u'\uee96': 'UnicodeCharacter0xee96',
u'\u2b98': 'UnicodeCharacter0x2b98',
u'\u689a': 'CjkUnifiedIdeograph689a',
u'\u841d': 'CjkUnifiedIdeograph841d',
u'\uc11f': 'HangulSyllableSeogs',
u'\u1e21': 'LatinSmallLetterGWithMacron',
u'\u5b23': 'CjkUnifiedIdeograph5b23',
u'\ub9a4': 'HangulSyllableRyiss',
u'\uf6a6': 'UnicodeCharacter0xf6a6',
u'\u33a8': 'SquareMOverSSquared',
u'\u70aa': 'CjkUnifiedIdeograph70aa',
u'\uac2d': 'HangulSyllableGaeb',
u'\u4dac': 'CjkUnifiedIdeograph4dac',
u'\ue92f': 'UnicodeCharacter0xe92f',
u'\u2631': 'TrigramForLake',
u'\u6333': 'CjkUnifiedIdeograph6333',
u'\u9eb6': 'CjkUnifiedIdeograph9eb6',
u'\udbb8': 'UnicodeCharacter0xdbb8',
u'\u18ba': 'UnicodeCharacter0x18ba',
u'\ub43d': 'HangulSyllableDyolg',
u'\u55bc': 'CjkUnifiedIdeograph55bc',
u'\uf13f': 'UnicodeCharacter0xf13f',
u'\uce41': 'HangulSyllableCyinj',
u'\u0b43': 'OriyaVowelSignVocalicR',
u'\u4845': 'CjkUnifiedIdeograph4845',
u'\ua6c6': 'UnicodeCharacter0xa6c6',
u'\ue3c8': 'UnicodeCharacter0xe3c8',
u'\u20ca': 'UnicodeCharacter0x20ca',
u'\u7dcc': 'CjkUnifiedIdeograph7dcc',
u'\u994f': 'CjkUnifiedIdeograph994f',
u'\ud651': 'HangulSyllableHot',
u'\u1353': 'EthiopicSyllablePaa',
u'\u5055': 'CjkUnifiedIdeograph5055',
u'\u8bd8': 'CjkUnifiedIdeograph8bd8',
u'\uc8da': 'HangulSyllableJoej',
u'\u05dc': 'HebrewLetterLamed',
u'\ua15f': 'YiSyllableNdex',
u'\u42de': 'CjkUnifiedIdeograph42de',
u'\ufe61': 'SmallAsterisk',
u'\u3b63': 'CjkUnifiedIdeograph3b63',
u'\u7865': 'CjkUnifiedIdeograph7865',
u'\u93e8': 'CjkUnifiedIdeograph93e8',
u'\ud0ea': 'HangulSyllableTaelp',
u'\u2dec': 'CombiningCyrillicLetterEr',
u'\u6aee': 'CjkUnifiedIdeograph6aee',
u'\u8671': 'CjkUnifiedIdeograph8671',
u'\uc373': 'HangulSyllableSseolb',
u'\u5d77': 'CjkUnifiedIdeograph5d77',
u'\ubbf8': 'HangulSyllableMi',
u'\uf8fa': 'UnicodeCharacter0xf8fa',
u'\u35fc': 'CjkUnifiedIdeograph35fc',
u'\u72fe': 'CjkUnifiedIdeograph72fe',
u'\u5000': 'CjkUnifiedIdeograph5000',
u'\u7483': 'CjkUnifiedIdeograph7483',
u'\u9302': 'CjkUnifiedIdeograph9302',
u'\ub785': 'HangulSyllableRalg',
u'\u1906': 'LimbuLetterCa',
u'\u3d89': 'CjkUnifiedIdeograph3d89',
u'\uc20c': 'HangulSyllableSyom',
u'\ue68f': 'UnicodeCharacter0xe68f',
u'\u4810': 'CjkUnifiedIdeograph4810',
u'\u6c93': 'CjkUnifiedIdeograph6c93',
u'\u8b12': 'CjkUnifiedIdeograph8b12',
u'\uaf95': 'HangulSyllableGgoeng',
u'\u3116': 'BopomofoLetterR',
u'\ud599': 'HangulSyllableHyalg',
u'\u5b9d': 'CjkUnifiedIdeograph5b9d',
u'\ufa1c': 'CjkCompatibilityIdeographfa1c',
u'\u9e9f': 'CjkUnifiedIdeograph9e9f',
u'\u6020': 'CjkUnifiedIdeograph6020',
u'\u04a3': 'CyrillicSmallLetterEnWithDescender',
u'\ua322': 'YiSyllableSurx',
u'\u2926': 'SouthWestArrowWithHook',
u'\ucda9': 'HangulSyllableCung',
u'\u73ad': 'CjkUnifiedIdeograph73ad',
u'\u922c': 'CjkUnifiedIdeograph922c',
u'\ub6af': 'HangulSyllableDdus',
u'\u1830': 'MongolianLetterSa',
u'\u3cb3': 'CjkUnifiedIdeograph3cb3',
u'\uc136': 'HangulSyllableSeop',
u'\ue5b9': 'UnicodeCharacter0xe5b9',
u'\u473a': 'CjkUnifiedIdeograph473a',
u'\u6bbd': 'CjkUnifiedIdeograph6bbd',
u'\u8a3c': 'CjkUnifiedIdeograph8a3c',
u'\uaebf': 'HangulSyllableGgeogs',
u'\u3040': 'UnicodeCharacter0x3040',
u'\ud4c3': 'HangulSyllablePwes',
u'\u5ac7': 'CjkUnifiedIdeograph5ac7',
u'\uf946': 'CjkCompatibilityIdeographf946',
u'\u9dc9': 'CjkUnifiedIdeograph9dc9',
u'\u7f4a': 'CjkUnifiedIdeograph7f4a',
u'\u03cd': 'GreekSmallLetterUpsilonWithTonos',
u'\ua24c': 'YiSyllableHxie',
u'\u2850': 'BraillePatternDots57',
u'\uccd3': 'HangulSyllableCyeogs',
u'\u72d7': 'CjkUnifiedIdeograph72d7',
u'\u9156': 'CjkUnifiedIdeograph9156',
u'\ub5d9': 'HangulSyllableDdyeog',
u'\u175a': 'UnicodeCharacter0x175a',
u'\u3bdd': 'CjkUnifiedIdeograph3bdd',
u'\uc060': 'HangulSyllableBbeul',
u'\ue4e3': 'UnicodeCharacter0xe4e3',
u'\u4664': 'CjkUnifiedIdeograph4664',
u'\u6ae7': 'CjkUnifiedIdeograph6ae7',
u'\u8966': 'CjkUnifiedIdeograph8966',
u'\uade9': 'HangulSyllableGyult',
u'\u0f6a': 'TibetanLetterFixedformRa',
u'\ud3ed': 'HangulSyllablePog',
u'\u59f1': 'CjkUnifiedIdeograph59f1',
u'\uf870': 'UnicodeCharacter0xf870',
u'\u9cf3': 'CjkUnifiedIdeograph9cf3',
u'\u7e74': 'CjkUnifiedIdeograph7e74',
u'\u02f7': 'ModifierLetterLowTilde',
u'\ua176': 'YiSyllableHnot',
u'\u277a': 'DingbatNegativeCircledDigitFive',
u'\ucbfd': 'HangulSyllableJjyilt',
u'\uf101': 'UnicodeCharacter0xf101',
u'\u9080': 'CjkUnifiedIdeograph9080',
u'\u7705': 'CjkUnifiedIdeograph7705',
u'\u1684': 'OghamLetterSail',
u'\uba07': 'HangulSyllableMyad',
u'\u200b': 'ZeroWidthSpace',
u'\udf8a': 'UnicodeCharacter0xdf8a',
u'\u458e': 'CjkUnifiedIdeograph458e',
u'\ue911': 'UnicodeCharacter0xe911',
u'\u8890': 'CjkUnifiedIdeograph8890',
u'\u6f15': 'CjkUnifiedIdeograph6f15',
u'\u0e94': 'LaoLetterDo',
u'\ud81b': 'UnicodeCharacter0xd81b',
u'\uf79a': 'UnicodeCharacter0xf79a',
u'\u5e1f': 'CjkUnifiedIdeograph5e1f',
u'\u7d9e': 'CjkUnifiedIdeograph7d9e',
u'\u8121': 'CjkUnifiedIdeograph8121',
u'\ua0a0': 'YiSyllableHmox',
u'\u0725': 'SyriacLetterE',
u'\u26a4': 'InterlockedFemaleAndMaleSign',
u'\uf02b': 'UnicodeCharacter0xf02b',
u'\uefaa': 'UnicodeCharacter0xefaa',
u'\u762f': 'CjkUnifiedIdeograph762f',
u'\u15ae': 'CanadianSyllabicsThcreeTh',
u'\ub931': 'HangulSyllableRweb',
u'\u3f35': 'CjkUnifiedIdeograph3f35',
u'\udeb4': 'UnicodeCharacter0xdeb4',
u'\u44b8': 'CjkUnifiedIdeograph44b8',
u'\ue83b': 'UnicodeCharacter0xe83b',
u'\u87ba': 'CjkUnifiedIdeograph87ba',
u'\u6e3f': 'CjkUnifiedIdeograph6e3f',
u'\u0dbe': 'UnicodeCharacter0xdbe',
u'\ud745': 'HangulSyllableHyub',
u'\uf6c4': 'UnicodeCharacter0xf6c4',
u'\u5d49': 'CjkUnifiedIdeograph5d49',
u'\u7cc8': 'CjkUnifiedIdeograph7cc8',
u'\u804b': 'CjkUnifiedIdeograph804b',
u'\ubfca': 'HangulSyllableBbyop',
u'\u064f': 'ArabicDamma',
u'\u25ce': 'Bullseye',
u'\ucf55': 'HangulSyllableKog',
u'\ueed4': 'UnicodeCharacter0xeed4',
u'\u7559': 'CjkUnifiedIdeograph7559',
u'\u14d8': 'CanadianSyllabicsLoo',
u'\ub85b': 'HangulSyllableRyeh',
u'\u3e5f': 'CjkUnifiedIdeograph3e5f',
u'\uddde': 'UnicodeCharacter0xddde',
u'\u43e2': 'CjkUnifiedIdeograph43e2',
u'\ue765': 'UnicodeCharacter0xe765',
u'\u86e4': 'CjkUnifiedIdeograph86e4',
u'\u6d69': 'CjkUnifiedIdeograph6d69',
u'\u0ce8': 'KannadaDigitTwo',
u'\ud66f': 'HangulSyllableHwah',
u'\uf5ee': 'UnicodeCharacter0xf5ee',
u'\u5c73': 'CjkUnifiedIdeograph5c73',
u'\u7bf2': 'CjkUnifiedIdeograph7bf2',
u'\u9f75': 'CjkUnifiedIdeograph9f75',
u'\ubef4': 'HangulSyllableBbel',
u'\u0579': 'ArmenianSmallLetterCha',
u'\u24f8': 'DoubleCircledDigitFour',
u'\uce7f': 'HangulSyllableKalb',
u'\uedfe': 'UnicodeCharacter0xedfe',
u'\u9806': 'CjkUnifiedIdeograph9806',
u'\ubc89': 'HangulSyllableBeonj',
u'\udd08': 'UnicodeCharacter0xdd08',
u'\ue18b': 'UnicodeCharacter0xe18b',
u'\u1e0a': 'LatinCapitalLetterDWithDotAbove',
u'\u228d': 'MultisetMultiplication',
u'\u430c': 'CjkUnifiedIdeograph430c',
u'\u678f': 'CjkUnifiedIdeograph678f',
u'\ub016': 'HangulSyllableGgwilm',
u'\uf518': 'UnicodeCharacter0xf518',
u'\u999b': 'CjkUnifiedIdeograph999b',
u'\u361a': 'CjkUnifiedIdeograph361a',
u'\uda9d': 'UnicodeCharacter0xda9d',
u'\u7b1c': 'CjkUnifiedIdeograph7b1c',
u'\u1f9f': 'GreekCapitalLetterEtaWithDasiaAndPerispomeniAndProsgegrammeni',
u'\u40a1': 'CjkUnifiedIdeograph40a1',
u'\ua826': 'SylotiNagriVowelSignE',
u'\ued28': 'UnicodeCharacter0xed28',
u'\ub1ab': 'HangulSyllableNwac',
u'\u2e2a': 'TwoDotsOverOneDotPunctuation',
u'\uf2ad': 'UnicodeCharacter0xf2ad',
u'\u132c': 'EthiopicSyllableChee',
u'\u37af': 'CjkUnifiedIdeograph37af',
u'\u542e': 'CjkUnifiedIdeograph542e',
u'\u78b1': 'CjkUnifiedIdeograph78b1',
u'\u8538': 'CjkUnifiedIdeograph8538',
u'\ua9bb': 'UnicodeCharacter0xa9bb',
u'\uc63a': 'HangulSyllableOj',
u'\ueabd': 'UnicodeCharacter0xeabd',
u'\u0b3c': 'OriyaSignNukta',
u'\u2fbf': 'KangxiRadicalSacrificialWine',
u'\u4c3e': 'CjkUnifiedIdeograph4c3e',
u'\u10c1': 'GeorgianCapitalLetterHe',
u'\u55c3': 'CjkUnifiedIdeograph55c3',
u'\ubd48': 'HangulSyllableBoe',
u'\ufe4a': 'CentrelineOverline',
u'\u82cd': 'CjkUnifiedIdeograph82cd',
u'\u234c': 'AplFunctionalSymbolQuadDownCaret',
u'\uc7cf': 'HangulSyllableJyad',
u'\u644e': 'CjkUnifiedIdeograph644e',
u'\u08d1': 'UnicodeCharacter0x8d1',
u'\u4dd3': 'HexagramForContemplation',
u'\u965a': 'CjkUnifiedIdeograph965a',
u'\ubadd': 'HangulSyllableMwat',
u'\udb5c': 'UnicodeCharacter0xdb5c',
u'\uffdf': 'UnicodeCharacter0xffdf',
u'\u1c5e': 'OlChikiLetterAl',
u'\u20e1': 'CombiningLeftRightArrowAbove',
u'\u4160': 'CjkUnifiedIdeograph4160',
u'\u65e3': 'CjkUnifiedIdeograph65e3',
u'\u8e6a': 'CjkUnifiedIdeograph8e6a',
u'\uf36c': 'UnicodeCharacter0xf36c',
u'\u97ef': 'CjkUnifiedIdeograph97ef',
u'\u346e': 'CjkUnifiedIdeograph346e',
u'\ud8f1': 'UnicodeCharacter0xd8f1',
u'\u7970': 'CjkUnifiedIdeograph7970',
u'\u1df3': 'UnicodeCharacter0x1df3',
u'\u5ef5': 'CjkUnifiedIdeograph5ef5',
u'\ua67a': 'UnicodeCharacter0xa67a',
u'\ueb7c': 'UnicodeCharacter0xeb7c',
u'\u8fff': 'CjkUnifiedIdeograph8fff',
u'\u2c7e': 'UnicodeCharacter0x2c7e',
u'\u1180': 'HangulJungseongOe',
u'\ub503': 'HangulSyllableDyilb',
u'\u5282': 'CjkUnifiedIdeograph5282',
u'\uf605': 'UnicodeCharacter0xf605',
u'\u3b07': 'CjkUnifiedIdeograph3b07',
u'\u7c09': 'CjkUnifiedIdeograph7c09',
u'\u838c': 'CjkUnifiedIdeograph838c',
u'\uc48e': 'HangulSyllableSsweolp',
u'\u0990': 'BengaliLetterAi',
u'\uad13': 'HangulSyllableGwac',
u'\u4a92': 'CjkUnifiedIdeograph4a92',
u'\uee15': 'UnicodeCharacter0xee15',
u'\ud317': 'HangulSyllablePalb',
u'\u1419': 'CanadianSyllabicsWaa',
u'\u591b': 'CjkUnifiedIdeograph591b',
u'\ubb9c': 'HangulSyllableMwiss',
u'\ufc9e': 'ArabicLigatureBehWithKhahInitialForm',
u'\u21a0': 'RightwardsTwoHeadedArrow',
u'\u62a2': 'CjkUnifiedIdeograph62a2',
u'\u8625': 'CjkUnifiedIdeograph8625',
u'\ucb27': 'HangulSyllableJjoec',
u'\u0c29': 'UnicodeCharacter0xc29',
u'\u712b': 'CjkUnifiedIdeograph712b',
u'\u94ae': 'CjkUnifiedIdeograph94ae',
u'\ud9b0': 'UnicodeCharacter0xd9b0',
u'\u1ab2': 'UnicodeCharacter0x1ab2',
u'\ube35': 'HangulSyllableByilt',
u'\u5fb4': 'CjkUnifiedIdeograph5fb4',
u'\ue337': 'UnicodeCharacter0xe337',
u'\u2439': 'UnicodeCharacter0x2439',
u'\u693b': 'CjkUnifiedIdeograph693b',
u'\u8cbe': 'CjkUnifiedIdeograph8cbe',
u'\uf1c0': 'UnicodeCharacter0xf1c0',
u'\u32c2': 'IdeographicTelegraphSymbolForMarch',
u'\u77c4': 'CjkUnifiedIdeograph77c4',
u'\u9b47': 'CjkUnifiedIdeograph9b47',
u'\udc49': 'UnicodeCharacter0xdc49',
u'\u014b': 'LatinSmallLetterEng',
u'\u424d': 'CjkUnifiedIdeograph424d',
u'\ua4ce': 'UnicodeCharacter0xa4ce',
u'\ue9d0': 'UnicodeCharacter0xe9d0',
u'\u2ad2': 'ClosedSupersetOrEqualTo',
u'\u6fd4': 'CjkUnifiedIdeograph6fd4',
u'\ub357': 'HangulSyllableDeogs',
u'\u50d6': 'CjkUnifiedIdeograph50d6',
u'\uf459': 'UnicodeCharacter0xf459',
u'\u395b': 'CjkUnifiedIdeograph395b',
u'\u7a5d': 'CjkUnifiedIdeograph7a5d',
u'\u81e0': 'CjkUnifiedIdeograph81e0',
u'\uc2e2': 'HangulSyllableSinh',
u'\u07e4': 'NkoLetterHa',
u'\uab67': 'UnicodeCharacter0xab67',
u'\u48e6': 'CjkUnifiedIdeograph48e6',
u'\uec69': 'UnicodeCharacter0xec69',
u'\ud16b': 'HangulSyllableTyeogs',
u'\u126d': 'EthiopicSyllableVe',
u'\u576f': 'CjkUnifiedIdeograph576f',
u'\ub9f0': 'HangulSyllableMaels',
u'\ufaf2': 'UnicodeCharacter0xfaf2',
u'\u3ff4': 'CjkUnifiedIdeograph3ff4',
u'\u60f6': 'CjkUnifiedIdeograph60f6',
u'\u8479': 'CjkUnifiedIdeograph8479',
u'\uc97b': 'HangulSyllableJyulh',
u'\u0a7d': 'UnicodeCharacter0xa7d',
u'\u4f7f': 'CjkUnifiedIdeograph4f7f',
u'\u3381': 'SquareNa',
u'\ud404': 'HangulSyllablePok',
u'\uf887': 'UnicodeCharacter0xf887',
u'\u5e08': 'CjkUnifiedIdeograph5e08',
u'\u628b': 'CjkUnifiedIdeograph628b',
u'\u9d0a': 'CjkUnifiedIdeograph9d0a',
u'\ua18d': 'YiSyllableNop',
u'\u070e': 'UnicodeCharacter0x70e',
u'\u2b91': 'UnicodeCharacter0x2b91',
u'\ucc14': 'HangulSyllableJjil',
u'\u9097': 'CjkUnifiedIdeograph9097',
u'\u7618': 'CjkUnifiedIdeograph7618',
u'\u1a9b': 'UnicodeCharacter0x1a9b',
u'\ub51a': 'HangulSyllableDinh',
u'\u3f1e': 'CjkUnifiedIdeograph3f1e',
u'\uc3a1': 'HangulSyllableSsyeog',
u'\u45a5': 'CjkUnifiedIdeograph45a5',
u'\ue424': 'UnicodeCharacter0xe424',
u'\u88a7': 'CjkUnifiedIdeograph88a7',
u'\u6e28': 'CjkUnifiedIdeograph6e28',
u'\u32ab': 'CircledIdeographStudy',
u'\uad2a': 'HangulSyllableGwaebs',
u'\ud72e': 'HangulSyllableHwij',
u'\ufbb1': 'ArabicLetterYehBarreeWithHamzaAboveFinalForm',
u'\u5932': 'CjkUnifiedIdeograph5932',
u'\u7db5': 'CjkUnifiedIdeograph7db5',
u'\u9c34': 'CjkUnifiedIdeograph9c34',
u'\ua0b7': 'YiSyllableMa',
u'\u0638': 'ArabicLetterZah',
u'\u2abb': 'DoublePrecedes',
u'\ucf3e': 'HangulSyllableKyenh',
u'\u93c1': 'CjkUnifiedIdeograph93c1',
u'\u7142': 'CjkUnifiedIdeograph7142',
u'\u15c5': 'CanadianSyllabicsCarrierGho',
u'\ub444': 'HangulSyllableDyom',
u'\u3e48': 'CjkUnifiedIdeograph3e48',
u'\uc2cb': 'HangulSyllableSyilb',
u'\u44cf': 'CjkUnifiedIdeograph44cf',
u'\ue74e': 'UnicodeCharacter0xe74e',
u'\u8bd1': 'CjkUnifiedIdeograph8bd1',
u'\u6952': 'CjkUnifiedIdeograph6952',
u'\u0dd5': 'UnicodeCharacter0xdd5',
u'\uac54': 'HangulSyllableGyae',
u'\ud658': 'HangulSyllableHwan',
u'\ufadb': 'UnicodeCharacter0xfadb',
u'\u585c': 'CjkUnifiedIdeograph585c',
u'\u7cdf': 'CjkUnifiedIdeograph7cdf',
u'\u9f5e': 'CjkUnifiedIdeograph9f5e',
u'\ua3e1': 'YiSyllableJix',
u'\u0162': 'LatinCapitalLetterTWithCedilla',
u'\u25e5': 'BlackUpperRightTriangle',
u'\uce68': 'HangulSyllableCim',
u'\u92eb': 'CjkUnifiedIdeograph92eb',
u'\u706c': 'CjkUnifiedIdeograph706c',
u'\u14ef': 'CanadianSyllabicsSi',
u'\ub76e': 'HangulSyllableDdilp',
u'\u3972': 'CjkUnifiedIdeograph3972',
u'\uddf5': 'UnicodeCharacter0xddf5',
u'\u47f9': 'CjkUnifiedIdeograph47f9',
u'\ue678': 'UnicodeCharacter0xe678',
u'\u8afb': 'CjkUnifiedIdeograph8afb',
u'\u687c': 'CjkUnifiedIdeograph687c',
u'\u0cff': 'UnicodeCharacter0xcff',
u'\uaf7e': 'HangulSyllableGgwaep',
u'\u3603': 'CjkUnifiedIdeograph3603',
u'\ud182': 'HangulSyllableTyeop',
u'\u5b86': 'CjkUnifiedIdeograph5b86',
u'\uff09': 'FullwidthRightParenthesis',
u'\u9e88': 'CjkUnifiedIdeograph9e88',
u'\u610d': 'CjkUnifiedIdeograph610d',
u'\x8c': 'UnicodeCharacter0x8c',
u'\ua40f': 'YiSyllableQup',
u'\u2e13': 'DottedObelos',
u'\uc992': 'HangulSyllableJeulm',
u'\u5017': 'CjkUnifiedIdeograph5017',
u'\u7396': 'CjkUnifiedIdeograph7396',
u'\u9719': 'CjkUnifiedIdeograph9719',
u'\ub698': 'HangulSyllableDdyok',
u'\u191d': 'UnicodeCharacter0x191d',
u'\u389c': 'CjkUnifiedIdeograph389c',
u'\uc623': 'HangulSyllableYeh',
u'\ue1a2': 'UnicodeCharacter0xe1a2',
u'\u4827': 'CjkUnifiedIdeograph4827',
u'\u6ba6': 'CjkUnifiedIdeograph6ba6',
u'\u8f29': 'CjkUnifiedIdeograph8f29',
u'\uaea8': 'HangulSyllableGgyael',
u'\u312d': 'BopomofoLetterIh',
u'\ud0ac': 'HangulSyllableKil',
u'\u5ab0': 'CjkUnifiedIdeograph5ab0',
u'\ufe33': 'PresentationFormForVerticalLowLine',
u'\u99b2': 'CjkUnifiedIdeograph99b2',
u'\u6037': 'CjkUnifiedIdeograph6037',
u'\u03b6': 'GreekSmallLetterZeta',
u'\ua739': 'LatinSmallLetterAv',
u'\u293d': 'TopArcAnticlockwiseArrowWithPlus',
u'\uc8bc': 'HangulSyllableJwaess',
u'\u5341': 'CjkUnifiedIdeograph5341',
u'\u72c0': 'CjkUnifiedIdeograph72c0',
u'\u9643': 'CjkUnifiedIdeograph9643',
u'\ub1c2': 'HangulSyllableNwaebs',
u'\u1847': 'MongolianLetterTodoU',
u'\u3bc6': 'CjkUnifiedIdeograph3bc6',
u'\uc14d': 'HangulSyllableSeng',
u'\ue0cc': 'UnicodeCharacter0xe0cc',
u'\u4b51': 'CjkUnifiedIdeograph4b51',
u'\u6ad0': 'CjkUnifiedIdeograph6ad0',
u'\u8e53': 'CjkUnifiedIdeograph8e53',
u'\ua9d2': 'UnicodeCharacter0xa9d2',
u'\u3057': 'HiraganaLetterSi',
u'\ud3d6': 'HangulSyllablePyenh',
u'\u55da': 'CjkUnifiedIdeograph55da',
u'\uf95d': 'CjkCompatibilityIdeographf95d',
u'\u98dc': 'CjkUnifiedIdeograph98dc',
u'\u6361': 'CjkUnifiedIdeograph6361',
u'\u02e0': 'ModifierLetterSmallGamma',
u'\ua663': 'CyrillicSmallLetterSoftDe',
u'\u2867': 'BraillePatternDots12367',
u'\ucbe6': 'HangulSyllableJjeubs',
u'\u526b': 'CjkUnifiedIdeograph526b',
u'\u4dea': 'HexagramForBreakthrough',
u'\u916d': 'CjkUnifiedIdeograph916d',
u'\ub0ec': 'HangulSyllableNyae',
u'\u1b71': 'BalineseMusicalSymbolCombiningKempliWithJegogan',
u'\u3af0': 'CjkUnifiedIdeograph3af0',
u'\uc077': 'HangulSyllableBbyigs',
u'\ue3f6': 'UnicodeCharacter0xe3f6',
u'\u4a7b': 'CjkUnifiedIdeograph4a7b',
u'\u65fa': 'CjkUnifiedIdeograph65fa',
u'\u897d': 'CjkUnifiedIdeograph897d',
u'\ua8fc': 'UnicodeCharacter0xa8fc',
u'\ub281': 'HangulSyllableNyult',
u'\ud300': 'HangulSyllableTim',
u'\uf783': 'UnicodeCharacter0xf783',
u'\u1002': 'MyanmarLetterGa',
u'\u3485': 'CjkUnifiedIdeograph3485',
u'\u5504': 'CjkUnifiedIdeograph5504',
u'\u7987': 'CjkUnifiedIdeograph7987',
u'\u860e': 'CjkUnifiedIdeograph860e',
u'\uaa91': 'UnicodeCharacter0xaa91',
u'\ucb10': 'HangulSyllableJjoe',
u'\uef93': 'UnicodeCharacter0xef93',
u'\u0812': 'UnicodeCharacter0x812',
u'\u2c95': 'CopticSmallLetterKapa',
u'\u4d14': 'CjkUnifiedIdeograph4d14',
u'\u1197': 'HangulJungseongYiu',
u'\u5699': 'CjkUnifiedIdeograph5699',
u'\ube1e': 'HangulSyllableBeubs',
u'\ue320': 'UnicodeCharacter0xe320',
u'\u87a3': 'CjkUnifiedIdeograph87a3',
u'\u2022': 'Bullet',
u'\uc4a5': 'HangulSyllableSswelg',
u'\u6524': 'CjkUnifiedIdeograph6524',
u'\u09a7': 'BengaliLetterDha',
u'\u4ea9': 'CjkUnifiedIdeograph4ea9',
u'\u9b30': 'CjkUnifiedIdeograph9b30',
u'\ubfb3': 'HangulSyllableBbyogs',
u'\ud832': 'UnicodeCharacter0xd832',
u'\ufcb5': 'ArabicLigatureDadWithHahInitialForm',
u'\u1d34': 'ModifierLetterCapitalH',
u'\u21b7': 'ClockwiseTopSemicircleArrow',
u'\u4236': 'CjkUnifiedIdeograph4236',
u'\u66b9': 'CjkUnifiedIdeograph66b9',
u'\ub340': 'HangulSyllableDyael',
u'\uf042': 'UnicodeCharacter0xf042',
u'\u94c5': 'CjkUnifiedIdeograph94c5',
u'\u3544': 'CjkUnifiedIdeograph3544',
u'\ud9c7': 'UnicodeCharacter0xd9c7',
u'\u7a46': 'CjkUnifiedIdeograph7a46',
u'\u1ec9': 'LatinSmallLetterIWithHookAbove',
u'\u43cb': 'CjkUnifiedIdeograph43cb',
u'\uab50': 'UnicodeCharacter0xab50',
u'\ue852': 'UnicodeCharacter0xe852',
u'\u8cd5': 'CjkUnifiedIdeograph8cd5',
u'\u2d54': 'TifinaghLetterYar',
u'\uf1d7': 'UnicodeCharacter0xf1d7',
u'\u1256': 'EthiopicSyllableQho',
u'\u36d9': 'CjkUnifiedIdeograph36d9',
u'\u5758': 'CjkUnifiedIdeograph5758',
u'\u7bdb': 'CjkUnifiedIdeograph7bdb',
u'\u8062': 'CjkUnifiedIdeograph8062',
u'\ua4e5': 'UnicodeCharacter0xa4e5',
u'\uc564': 'HangulSyllableAen',
u'\ue9e7': 'UnicodeCharacter0xe9e7',
u'\u0a66': 'GurmukhiDigitZero',
u'\u2ee9': 'CjkRadicalSimplifiedYellow',
u'\u4f68': 'CjkUnifiedIdeograph4f68',
u'\u13eb': 'CherokeeLetterWi',
u'\u50ed': 'CjkUnifiedIdeograph50ed',
u'\ub872': 'HangulSyllableRoj',
u'\ufd74': 'ArabicLigatureTahWithMeemWithYehFinalForm',
u'\u81f7': 'CjkUnifiedIdeograph81f7',
u'\u2276': 'LessthanOrGreaterthan',
u'\uc6f9': 'HangulSyllableWeb',
u'\u6778': 'CjkUnifiedIdeograph6778',
u'\u0bfb': 'UnicodeCharacter0xbfb',
u'\u48fd': 'CjkUnifiedIdeograph48fd',
u'\u7201': 'CjkUnifiedIdeograph7201',
u'\u9584': 'CjkUnifiedIdeograph9584',
u'\uda86': 'UnicodeCharacter0xda86',
u'\u1f88': 'GreekCapitalLetterAlphaWithPsiliAndProsgegrammeni',
u'\ua30b': 'YiSyllableSi',
u'\u5c8a': 'CjkUnifiedIdeograph5c8a',
u'\ue00d': 'UnicodeCharacter0xe00d',
u'\u250f': 'BoxDrawingsHeavyDownAndRight',
u'\u6a11': 'CjkUnifiedIdeograph6a11',
u'\u8d94': 'CjkUnifiedIdeograph8d94',
u'\uf296': 'UnicodeCharacter0xf296',
u'\u3798': 'CjkUnifiedIdeograph3798',
u'\u749a': 'CjkUnifiedIdeograph749a',
u'\u981d': 'CjkUnifiedIdeograph981d',
u'\udd1f': 'UnicodeCharacter0xdd1f',
u'\u0221': 'LatinSmallLetterDWithCurl',
u'\u4723': 'CjkUnifiedIdeograph4723',
u'\ua5a4': 'VaiSyllableTu',
u'\ueaa6': 'UnicodeCharacter0xeaa6',
u'\u2fa8': 'KangxiRadicalGate',
u'\u6caa': 'CjkUnifiedIdeograph6caa',
u'\ub02d': 'HangulSyllableGgyunj',
u'\u51ac': 'CjkUnifiedIdeograph51ac',
u'\uf52f': 'UnicodeCharacter0xf52f',
u'\u3a31': 'CjkUnifiedIdeograph3a31',
u'\u7f33': 'CjkUnifiedIdeograph7f33',
u'\u82b6': 'CjkUnifiedIdeograph82b6',
u'\uc7b8': 'HangulSyllableJaels',
u'\u04ba': 'CyrillicCapitalLetterShha',
u'\ua83d': 'UnicodeCharacter0xa83d',
u'\u49bc': 'CjkUnifiedIdeograph49bc',
u'\ued3f': 'UnicodeCharacter0xed3f',
u'\ud241': 'HangulSyllableTung',
u'\u1743': 'BuhidLetterKa',
u'\u5445': 'CjkUnifiedIdeograph5445',
u'\ubac6': 'HangulSyllableMwagg',
u'\uffc8': 'UnicodeCharacter0xffc8',
u'\u3cca': 'CjkUnifiedIdeograph3cca',
u'\u61cc': 'CjkUnifiedIdeograph61cc',
u'\u854f': 'CjkUnifiedIdeograph854f',
u'\uca51': 'HangulSyllableJjeonj',
u'\u0f53': 'TibetanLetterNa',
u'\u4c55': 'CjkUnifiedIdeograph4c55',
u'\u97d8': 'CjkUnifiedIdeograph97d8',
u'\ud4da': 'HangulSyllablePwilp',
u'\u19dc': 'UnicodeCharacter0x19dc',
u'\ubd5f': 'HangulSyllableBoec',
u'\u5ede': 'CjkUnifiedIdeograph5ede',
u'\ue261': 'UnicodeCharacter0xe261',
u'\u2763': 'HeavyHeartExclamationMarkOrnament',
u'\u6465': 'CjkUnifiedIdeograph6465',
u'\u8fe8': 'CjkUnifiedIdeograph8fe8',
u'\uccea': 'HangulSyllableCyeop',
u'\u31ec': 'UnicodeCharacter0x31ec',
u'\u76ee': 'CjkUnifiedIdeograph76ee',
u'\u9a71': 'CjkUnifiedIdeograph9a71',
u'\udf73': 'UnicodeCharacter0xdf73',
u'\u1c75': 'OlChikiLetterOb',
u'\u4177': 'CjkUnifiedIdeograph4177',
u'\ua7f8': 'UnicodeCharacter0xa7f8',
u'\ue4fa': 'UnicodeCharacter0xe4fa',
u'\u29fc': 'LeftpointingCurvedAngleBracket',
u'\u6efe': 'CjkUnifiedIdeograph6efe',
u'\u5400': 'CjkUnifiedIdeograph5400',
u'\u7083': 'CjkUnifiedIdeograph7083',
u'\u9702': 'CjkUnifiedIdeograph9702',
u'\ub385': 'HangulSyllableDeng',
u'\u1d06': 'LatinLetterSmallCapitalEth',
u'\u3989': 'CjkUnifiedIdeograph3989',
u'\uc60c': 'HangulSyllableYen',
u'\ue28f': 'UnicodeCharacter0xe28f',
u'\u4c10': 'CjkUnifiedIdeograph4c10',
u'\u6893': 'CjkUnifiedIdeograph6893',
u'\u8f12': 'CjkUnifiedIdeograph8f12',
u'\uab95': 'UnicodeCharacter0xab95',
u'\u3516': 'CjkUnifiedIdeograph3516',
u'\ud199': 'HangulSyllableTyeng',
u'\u5f9d': 'CjkUnifiedIdeograph5f9d',
u'\ufe1c': 'UnicodeCharacter0xfe1c',
u'\u9a9f': 'CjkUnifiedIdeograph9a9f',
u'\u6420': 'CjkUnifiedIdeograph6420',
u'\xa3': 'PoundSign',
u'\ua722': 'LatinCapitalLetterEgyptologicalAlef',
u'\u2d26': 'UnicodeCharacter0x2d26',
u'\uc9a9': 'HangulSyllableJyinj',
u'\u77ad': 'CjkUnifiedIdeograph77ad',
u'\u962c': 'CjkUnifiedIdeograph962c',
u'\ub2af': 'HangulSyllableNyigs',
u'\u1c30': 'LepchaConsonantSignN',
u'\u38b3': 'CjkUnifiedIdeograph38b3',
u'\uc536': 'HangulSyllableSsilp',
u'\ue1b9': 'UnicodeCharacter0xe1b9',
u'\u433a': 'CjkUnifiedIdeograph433a',
u'\u6fbd': 'CjkUnifiedIdeograph6fbd',
u'\u8e3c': 'CjkUnifiedIdeograph8e3c',
u'\uaabf': 'UnicodeCharacter0xaabf',
u'\u3440': 'CjkUnifiedIdeograph3440',
u'\ud0c3': 'HangulSyllableTags',
u'\u5ec7': 'CjkUnifiedIdeograph5ec7',
u'\ufd46': 'UnicodeCharacter0xfd46',
u'\u99c9': 'CjkUnifiedIdeograph99c9',
u'\u7b4a': 'CjkUnifiedIdeograph7b4a',
u'\u07cd': 'NkoLetterE',
u'\ua64c': 'CyrillicCapitalLetterBroadOmega',
u'\u2c50': 'GlagoliticSmallLetterYeri',
u'\uc8d3': 'HangulSyllableJoelh',
u'\u76d7': 'CjkUnifiedIdeograph76d7',
u'\u9556': 'CjkUnifiedIdeograph9556',
u'\ub1d9': 'HangulSyllableNoelt',
u'\u135a': 'EthiopicSyllableFya',
u'\u3fdd': 'CjkUnifiedIdeograph3fdd',
u'\uc460': 'HangulSyllableSsyok',
u'\ue0e3': 'UnicodeCharacter0xe0e3',
u'\u4264': 'CjkUnifiedIdeograph4264',
u'\u6ee7': 'CjkUnifiedIdeograph6ee7',
u'\u8d66': 'CjkUnifiedIdeograph8d66',
u'\ua9e9': 'UnicodeCharacter0xa9e9',
u'\u0b6a': 'OriyaDigitFour',
u'\ud7ed': 'UnicodeCharacter0xd7ed',
u'\u5df1': 'CjkUnifiedIdeograph5df1',
u'\ufc70': 'ArabicLigatureTehWithRehFinalForm',
u'\u98f3': 'CjkUnifiedIdeograph98f3',
u'\u7a74': 'CjkUnifiedIdeograph7a74',
u'\u06f7': 'ExtendedArabicindicDigitSeven',
u'\ua576': 'VaiSyllablePoo',
u'\u237a': 'AplFunctionalSymbolAlpha',
u'\ucffd': 'HangulSyllableKweog',
u'\uf501': 'UnicodeCharacter0xf501',
u'\u9480': 'CjkUnifiedIdeograph9480',
u'\u7305': 'CjkUnifiedIdeograph7305',
u'\u1284': 'EthiopicSyllableXee',
u'\ube07': 'HangulSyllableByuc',
u'\u240b': 'SymbolForVerticalTabulation',
u'\udb8a': 'UnicodeCharacter0xdb8a',
u'\u418e': 'CjkUnifiedIdeograph418e',
u'\ued11': 'UnicodeCharacter0xed11',
u'\u8c90': 'CjkUnifiedIdeograph8c90',
u'\u6b15': 'CjkUnifiedIdeograph6b15',
u'\u0a94': 'GujaratiLetterAu',
u'\udc1b': 'UnicodeCharacter0xdc1b',
u'\uf39a': 'UnicodeCharacter0xf39a',
u'\u5a1f': 'CjkUnifiedIdeograph5a1f',
u'\u799e': 'CjkUnifiedIdeograph799e',
u'\u8521': 'CjkUnifiedIdeograph8521',
u'\ua4a0': 'YiRadicalTat',
u'\u0325': 'CombiningRingBelow',
u'\u22a4': 'DownTack',
u'\uf42b': 'UnicodeCharacter0xf42b',
u'\uebaa': 'UnicodeCharacter0xebaa',
u'\u722f': 'CjkUnifiedIdeograph722f',
u'\u11ae': 'HangulJongseongTikeut',
u'\ubd31': 'HangulSyllableBwaenj',
u'\u3b35': 'CjkUnifiedIdeograph3b35',
u'\udab4': 'UnicodeCharacter0xdab4',
u'\u40b8': 'CjkUnifiedIdeograph40b8',
u'\uec3b': 'UnicodeCharacter0xec3b',
u'\u83ba': 'CjkUnifiedIdeograph83ba',
u'\u6a3f': 'CjkUnifiedIdeograph6a3f',
u'\u09be': 'BengaliVowelSignAa',
u'\ud345': 'HangulSyllablePyag',
u'\uf2c4': 'UnicodeCharacter0xf2c4',
u'\u5949': 'CjkUnifiedIdeograph5949',
u'\u78c8': 'CjkUnifiedIdeograph78c8',
u'\u844b': 'CjkUnifiedIdeograph844b',
u'\ubbca': 'HangulSyllableMeulm',
u'\u024f': 'LatinSmallLetterYWithStroke',
u'\u21ce': 'LeftRightDoubleArrowWithStroke',
u'\ucb55': 'HangulSyllableJjult',
u'\uead4': 'UnicodeCharacter0xead4',
u'\u7159': 'CjkUnifiedIdeograph7159',
u'\u10d8': 'GeorgianLetterIn',
u'\ubc5b': 'HangulSyllableByalh',
u'\u3a5f': 'CjkUnifiedIdeograph3a5f',
u'\ud9de': 'UnicodeCharacter0xd9de',
u'\u47e2': 'CjkUnifiedIdeograph47e2',
u'\ue365': 'UnicodeCharacter0xe365',
u'\u82e4': 'CjkUnifiedIdeograph82e4',
u'\u6969': 'CjkUnifiedIdeograph6969',
u'\u08e8': 'UnicodeCharacter0x8e8',
u'\ud26f': 'HangulSyllableTwelb',
u'\uf1ee': 'UnicodeCharacter0xf1ee',
u'\u5873': 'CjkUnifiedIdeograph5873',
u'\u7ff2': 'CjkUnifiedIdeograph7ff2',
u'\u9b75': 'CjkUnifiedIdeograph9b75',
u'\ubaf4': 'HangulSyllableMwaess',
u'\u0179': 'LatinCapitalLetterZWithAcute',
u'\u20f8': 'UnicodeCharacter0x20f8',
u'\uca7f': 'HangulSyllableJjec',
u'\ue9fe': 'UnicodeCharacter0xe9fe',
u'\u9c06': 'CjkUnifiedIdeograph9c06',
u'\ub889': 'HangulSyllableRwab',
u'\ud908': 'UnicodeCharacter0xd908',
u'\ue58b': 'UnicodeCharacter0xe58b',
u'\u1a0a': 'BugineseLetterNa',
u'\u268d': 'DigramForLesserYin',
u'\u470c': 'CjkUnifiedIdeograph470c',
u'\u638f': 'CjkUnifiedIdeograph638f',
u'\ub416': 'HangulSyllableDwaep',
u'\uf118': 'UnicodeCharacter0xf118',
u'\u9d9b': 'CjkUnifiedIdeograph9d9b',
u'\u321a': 'ParenthesizedHangulPhieuphA',
u'\ude9d': 'UnicodeCharacter0xde9d',
u'\u7f1c': 'CjkUnifiedIdeograph7f1c',
u'\u1b9f': 'SundaneseLetterXa',
u'\u44a1': 'CjkUnifiedIdeograph44a1',
u'\uac26': 'HangulSyllableGaelm',
u'\ue928': 'UnicodeCharacter0xe928',
u'\ub5ab': 'HangulSyllableDdeolb',
u'\u2a2a': 'MinusSignWithDotBelow',
u'\uf6ad': 'UnicodeCharacter0xf6ad',
u'\u172c': 'HanunooLetterYa',
u'\u33af': 'SquareRadOverSSquared',
u'\u502e': 'CjkUnifiedIdeograph502e',
u'\u7cb1': 'CjkUnifiedIdeograph7cb1',
u'\u8138': 'CjkUnifiedIdeograph8138',
u'\uadbb': 'HangulSyllableGwec',
u'\uc23a': 'HangulSyllableSweonh',
u'\ueebd': 'UnicodeCharacter0xeebd',
u'\u0f3c': 'TibetanMarkAngKhangGyon',
u'\u2bbf': 'UnicodeCharacter0x2bbf',
u'\u483e': 'CjkUnifiedIdeograph483e',
u'\u14c1': 'CanadianSyllabicsNaai',
u'\u51c3': 'CjkUnifiedIdeograph51c3',
u'\ub948': 'HangulSyllableRwils',
u'\ufa4a': 'CjkCompatibilityIdeographfa4a',
u'\u86cd': 'CjkUnifiedIdeograph86cd',
u'\u274c': 'UnicodeCharacter0x274c',
u'\uc3cf': 'HangulSyllableSsyes',
u'\u604e': 'CjkUnifiedIdeograph604e',
u'\u0cd1': 'UnicodeCharacter0xcd1',
u'\u49d3': 'CjkUnifiedIdeograph49d3',
u'\u925a': 'CjkUnifiedIdeograph925a',
u'\ubedd': 'HangulSyllableBbeolt',
u'\udf5c': 'UnicodeCharacter0xdf5c',
u'\ufbdf': 'ArabicLetterVeFinalForm',
u'\u185e': 'MongolianLetterSibeI',
u'\u24e1': 'CircledLatinSmallLetterR',
u'\u4560': 'CjkUnifiedIdeograph4560',
u'\u61e3': 'CjkUnifiedIdeograph61e3',
u'\u8a6a': 'CjkUnifiedIdeograph8a6a',
u'\uf76c': 'UnicodeCharacter0xf76c',
u'\u93ef': 'CjkUnifiedIdeograph93ef',
u'\u306e': 'HiraganaLetterNo',
u'\udcf1': 'UnicodeCharacter0xdcf1',
u'\u7d70': 'CjkUnifiedIdeograph7d70',
u'\u19f3': 'KhmerSymbolBeiRoc',
u'\u5af5': 'CjkUnifiedIdeograph5af5',
u'\ua27a': 'YiSyllableHox',
u'\uef7c': 'UnicodeCharacter0xef7c',
u'\u8bff': 'CjkUnifiedIdeograph8bff',
u'\u287e': 'BraillePatternDots234567',
u'\u1580': 'CanadianSyllabicsQii',
u'\ub103': 'HangulSyllableNyaec',
u'\u5682': 'CjkUnifiedIdeograph5682',
u'\uf205': 'UnicodeCharacter0xf205',
u'\u3f07': 'CjkUnifiedIdeograph3f07',
u'\u7809': 'CjkUnifiedIdeograph7809',
u'\u878c': 'CjkUnifiedIdeograph878c',
u'\uc08e': 'HangulSyllableBbyip',
u'\u0d90': 'SinhalaLetterIluuyanna',
u'\ua913': 'KayahLiLetterHta',
u'\u4e92': 'CjkUnifiedIdeograph4e92',
u'\uea15': 'UnicodeCharacter0xea15',
u'\ud717': 'HangulSyllableHweh',
u'\u1019': 'MyanmarLetterMa',
u'\u5d1b': 'CjkUnifiedIdeograph5d1b',
u'\ubf9c': 'HangulSyllableBboel',
u'\uf89e': 'UnicodeCharacter0xf89e',
u'\u25a0': 'BlackSquare',
u'\u66a2': 'CjkUnifiedIdeograph66a2',
u'\u8225': 'CjkUnifiedIdeograph8225',
u'\ucf27': 'HangulSyllableKyeolb',
u'\u0829': 'UnicodeCharacter0x829',
u'\u752b': 'CjkUnifiedIdeograph752b',
u'\u90ae': 'CjkUnifiedIdeograph90ae',
u'\uddb0': 'UnicodeCharacter0xddb0',
u'\u1eb2': 'LatinCapitalLetterAWithBreveAndHookAbove',
u'\uba35': 'HangulSyllableMyaet',
u'\u5bb4': 'CjkUnifiedIdeograph5bb4',
u'\ue737': 'UnicodeCharacter0xe737',
u'\u2039': 'SingleLeftpointingAngleQuotationMark',
u'\u6d3b': 'CjkUnifiedIdeograph6d3b',
u'\u88be': 'CjkUnifiedIdeograph88be',
u'\uf5c0': 'UnicodeCharacter0xf5c0',
u'\u36c2': 'CjkUnifiedIdeograph36c2',
u'\u73c4': 'CjkUnifiedIdeograph73c4',
u'\u9f47': 'CjkUnifiedIdeograph9f47',
u'\ud849': 'UnicodeCharacter0xd849',
u'\u054b': 'ArmenianCapitalLetterJheh',
u'\u464d': 'CjkUnifiedIdeograph464d',
u'\ua0ce': 'YiSyllableFix',
u'\uedd0': 'UnicodeCharacter0xedd0',
u'\u2ed2': 'CjkRadicalLongTwo',
u'\u6bd4': 'CjkUnifiedIdeograph6bd4',
u'\ub757': 'HangulSyllableDdyis',
u'\u54d6': 'CjkUnifiedIdeograph54d6',
u'\uf059': 'UnicodeCharacter0xf059',
u'\u3d5b': 'CjkUnifiedIdeograph3d5b',
u'\u7e5d': 'CjkUnifiedIdeograph7e5d',
u'\u85e0': 'CjkUnifiedIdeograph85e0',
u'\uc6e2': 'HangulSyllableWeoj',
u'\u03e4': 'CopticCapitalLetterFei',
u'\uaf67': 'HangulSyllableGgwaegs',
u'\u4ce6': 'CjkUnifiedIdeograph4ce6',
u'\ue869': 'UnicodeCharacter0xe869',
u'\ud56b': 'HangulSyllableHas',
u'\u166d': 'CanadianSyllabicsChiSign',
u'\u536f': 'CjkUnifiedIdeograph536f',
u'\ubdf0': 'HangulSyllableByu',
u'\ufef2': 'ArabicLetterYehFinalForm',
u'\u3bf4': 'CjkUnifiedIdeograph3bf4',
u'\u64f6': 'CjkUnifiedIdeograph64f6',
u'\u8079': 'CjkUnifiedIdeograph8079',
u'\ucd7b': 'HangulSyllableCyogs',
u'\u0e7d': 'UnicodeCharacter0xe7d',
u'\u4b7f': 'CjkUnifiedIdeograph4b7f',
u'\u3781': 'CjkUnifiedIdeograph3781',
u'\ud004': 'HangulSyllableKweol',
u'\ufc87': 'ArabicLigatureLamWithYehFinalForm',
u'\u5a08': 'CjkUnifiedIdeograph5a08',
u'\u668b': 'CjkUnifiedIdeograph668b',
u'\u990a': 'CjkUnifiedIdeograph990a',
u'\ua58d': 'VaiSyllableNjoo',
u'\u030e': 'CombiningDoubleVerticalLineAbove',
u'\u2f91': 'KangxiRadicalWest',
u'\uc814': 'HangulSyllableJeoss',
u'\u9497': 'CjkUnifiedIdeograph9497',
u'\u7218': 'CjkUnifiedIdeograph7218',
u'\u1e9b': 'LatinSmallLetterLongSWithDotAbove',
u'\ub11a': 'HangulSyllableNeobs',
u'\u3b1e': 'CjkUnifiedIdeograph3b1e',
u'\uc7a1': 'HangulSyllableJab',
u'\u41a5': 'CjkUnifiedIdeograph41a5',
u'\ue024': 'UnicodeCharacter0xe024',
u'\u8ca7': 'CjkUnifiedIdeograph8ca7',
u'\u6a28': 'CjkUnifiedIdeograph6a28',
u'\u36ab': 'CjkUnifiedIdeograph36ab',
u'\ua92a': 'KayahLiVowelO',
u'\ud32e': 'HangulSyllablePaenh',
u'\uffb1': 'HalfwidthHangulLetterMieum',
u'\u5d32': 'CjkUnifiedIdeograph5d32',
u'\u79b5': 'CjkUnifiedIdeograph79b5',
u'\u9834': 'CjkUnifiedIdeograph9834',
u'\ua4b7': 'YiRadicalJjie',
u'\u0238': 'LatinSmallLetterDbDigraph',
u'\u2ebb': 'CjkRadicalBrushTwo',
u'\ucb3e': 'HangulSyllableJjyobs',
u'\u97c1': 'CjkUnifiedIdeograph97c1',
u'\u7542': 'CjkUnifiedIdeograph7542',
u'\u11c5': 'HangulJongseongNieunkiyeok',
u'\ub044': 'HangulSyllableGgeu',
u'\u3a48': 'CjkUnifiedIdeograph3a48',
u'\uc6cb': 'HangulSyllableUh',
u'\u40cf': 'CjkUnifiedIdeograph40cf',
u'\ue34e': 'UnicodeCharacter0xe34e',
u'\u8fd1': 'CjkUnifiedIdeograph8fd1',
u'\u6d52': 'CjkUnifiedIdeograph6d52',
u'\u09d5': 'UnicodeCharacter0x9d5',
u'\ua854': 'PhagspaLetterZha',
u'\ud258': 'HangulSyllableTweom',
u'\ufedb': 'ArabicLetterKafInitialForm',
u'\u5c5c': 'CjkUnifiedIdeograph5c5c',
u'\u78df': 'CjkUnifiedIdeograph78df',
u'\u9b5e': 'CjkUnifiedIdeograph9b5e',
u'\ua7e1': 'UnicodeCharacter0xa7e1',
u'\u0562': 'ArmenianSmallLetterBen',
u'\u21e5': 'RightwardsArrowToBar',
u'\uca68': 'HangulSyllableJje',
u'\u96eb': 'CjkUnifiedIdeograph96eb',
u'\u746c': 'CjkUnifiedIdeograph746c',
u'\u10ef': 'GeorgianLetterJhan',
u'\ub36e': 'HangulSyllableDeop',
u'\u3d72': 'CjkUnifiedIdeograph3d72',
u'\ud9f5': 'UnicodeCharacter0xd9f5',
u'\u43f9': 'CjkUnifiedIdeograph43f9',
u'\ue278': 'UnicodeCharacter0xe278',
u'\u8efb': 'CjkUnifiedIdeograph8efb',
u'\u6c7c': 'CjkUnifiedIdeograph6c7c',
u'\u08ff': 'UnicodeCharacter0x8ff',
u'\uab7e': 'UnicodeCharacter0xab7e',
u'\u3203': 'ParenthesizedHangulRieul',
u'\ud582': 'HangulSyllableHaelp',
u'\u5f86': 'CjkUnifiedIdeograph5f86',
u'\ufb09': 'UnicodeCharacter0xfb09',
u'\u9a88': 'CjkUnifiedIdeograph9a88',
u'\u650d': 'CjkUnifiedIdeograph650d',
u'\u048c': 'CyrillicCapitalLetterSemisoftSign',
u'\ua00f': 'YiSyllableOt',
u'\u2a13': 'LineIntegrationWithSemicircularPathAroundPole',
u'\ucd92': 'HangulSyllableCyop',
u'\u5417': 'CjkUnifiedIdeograph5417',
u'\u7796': 'CjkUnifiedIdeograph7796',
u'\u9319': 'CjkUnifiedIdeograph9319',
u'\ub298': 'HangulSyllableNeul',
u'\u1d1d': 'LatinSmallLetterSidewaysU',
u'\u3c9c': 'CjkUnifiedIdeograph3c9c',
u'\uc223': 'HangulSyllableSulb',
u'\ue5a2': 'UnicodeCharacter0xe5a2',
u'\u4c27': 'CjkUnifiedIdeograph4c27',
u'\u6fa6': 'CjkUnifiedIdeograph6fa6',
u'\u8b29': 'CjkUnifiedIdeograph8b29',
u'\uaaa8': 'UnicodeCharacter0xaaa8',
u'\u352d': 'CjkUnifiedIdeograph352d',
u'\ud4ac': 'HangulSyllablePweok',
u'\u5eb0': 'CjkUnifiedIdeograph5eb0',
u'\ufa33': 'CjkCompatibilityIdeographfa33',
u'\u9db2': 'CjkUnifiedIdeograph9db2',
u'\u6437': 'CjkUnifiedIdeograph6437',
u'\u07b6': 'UnicodeCharacter0x7b6',
u'\ua339': 'YiSyllableSsex',
u'\u2d3d': 'TifinaghLetterYak',
u'\uccbc': 'HangulSyllableCel',
u'\u5741': 'CjkUnifiedIdeograph5741',
u'\u76c0': 'CjkUnifiedIdeograph76c0',
u'\u9243': 'CjkUnifiedIdeograph9243',
u'\ub5c2': 'HangulSyllableDdenh',
u'\u1c47': 'LepchaDigitSeven',
u'\u3fc6': 'CjkUnifiedIdeograph3fc6',
u'\uc54d': 'HangulSyllableAlg',
u'\ue4cc': 'UnicodeCharacter0xe4cc',
u'\u4f51': 'CjkUnifiedIdeograph4f51',
u'\u6ed0': 'CjkUnifiedIdeograph6ed0',
u'\u8a53': 'CjkUnifiedIdeograph8a53',
u'\uadd2': 'HangulSyllableGwibs',
u'\u3457': 'CjkUnifiedIdeograph3457',
u'\ud7d6': 'UnicodeCharacter0xd7d6',
u'\u51da': 'CjkUnifiedIdeograph51da',
u'\ufd5d': 'ArabicLigatureSeenWithJeemWithHahInitialForm',
u'\u9cdc': 'CjkUnifiedIdeograph9cdc',
u'\u6761': 'CjkUnifiedIdeograph6761',
u'\u06e0': 'ArabicSmallHighUprightRectangularZero',
u'\ua263': 'YiSyllableNgap',
u'\u2c67': 'LatinCapitalLetterHWithDescender',
u'\ucfe6': 'HangulSyllableKunh',
u'\u566b': 'CjkUnifiedIdeograph566b',
u'\u49ea': 'CjkUnifiedIdeograph49ea',
u'\u956d': 'CjkUnifiedIdeograph956d',
u'\ub4ec': 'HangulSyllableDeum',
u'\u1f71': 'GreekSmallLetterAlphaWithOxia',
u'\u3ef0': 'CjkUnifiedIdeograph3ef0',
u'\uc477': 'HangulSyllableSsus',
u'\ue7f6': 'UnicodeCharacter0xe7f6',
u'\u4e7b': 'CjkUnifiedIdeograph4e7b',
u'\u61fa': 'CjkUnifiedIdeograph61fa',
u'\u8d7d': 'CjkUnifiedIdeograph8d7d',
u'\uacfc': 'HangulSyllableGwa',
u'\ub681': 'HangulSyllableDdyog',
u'\ud700': 'HangulSyllableHwen',
u'\uf383': 'UnicodeCharacter0xf383',
u'\u1402': 'CanadianSyllabicsAai',
u'\u3085': 'HiraganaLetterSmallYu',
u'\u5104': 'CjkUnifiedIdeograph5104',
u'\u7d87': 'CjkUnifiedIdeograph7d87',
u'\u820e': 'CjkUnifiedIdeograph820e',
u'\uae91': 'HangulSyllableGgyalt',
u'\ucf10': 'HangulSyllableKem',
u'\ueb93': 'UnicodeCharacter0xeb93',
u'\u0c12': 'TeluguLetterO',
u'\u2895': 'BraillePatternDots1358',
u'\u4914': 'CjkUnifiedIdeograph4914',
u'\u1597': 'CanadianSyllabicsSayisiShe',
u'\u5299': 'CjkUnifiedIdeograph5299',
u'\uba1e': 'HangulSyllableMyaegg',
u'\ue720': 'UnicodeCharacter0xe720',
u'\u83a3': 'CjkUnifiedIdeograph83a3',
u'\u2422': 'BlankSymbol',
u'\uc0a5': 'HangulSyllableBbing',
u'\u6124': 'CjkUnifiedIdeograph6124',
u'\u0da7': 'SinhalaLetterAlpapraanaTtayanna',
u'\u4aa9': 'CjkUnifiedIdeograph4aa9',
u'\u9f30': 'CjkUnifiedIdeograph9f30',
u'\ubbb3': 'HangulSyllableMyulh',
u'\udc32': 'UnicodeCharacter0xdc32',
u'\uf8b5': 'UnicodeCharacter0xf8b5',
u'\u1934': 'LimbuSmallLetterNa',
u'\u25b7': 'WhiteRightpointingTriangle',
u'\u4636': 'CjkUnifiedIdeograph4636',
u'\u62b9': 'CjkUnifiedIdeograph62b9',
u'\ub740': 'HangulSyllableDdeuk',
u'\uf442': 'UnicodeCharacter0xf442',
u'\u90c5': 'CjkUnifiedIdeograph90c5',
u'\u3144': 'HangulLetterPieupsios',
u'\uddc7': 'UnicodeCharacter0xddc7',
u'\u7e46': 'CjkUnifiedIdeograph7e46',
u'\u1ac9': 'UnicodeCharacter0x1ac9',
u'\u47cb': 'CjkUnifiedIdeograph47cb',
u'\uaf50': 'HangulSyllableGgwal',
u'\uec52': 'UnicodeCharacter0xec52',
u'\u88d5': 'CjkUnifiedIdeograph88d5',
u'\u2954': 'UpwardsHarpoonWithBarbRightToBar',
u'\uf5d7': 'UnicodeCharacter0xf5d7',
u'\u1656': 'CanadianSyllabicsCarrierShe',
u'\u32d9': 'CircledKatakanaKo',
u'\u5358': 'CjkUnifiedIdeograph5358',
u'\u7fdb': 'CjkUnifiedIdeograph7fdb',
u'\u8462': 'CjkUnifiedIdeograph8462',
u'\ua0e5': 'YiSyllableVip',
u'\uc164': 'HangulSyllableSyeom',
u'\uede7': 'UnicodeCharacter0xede7',
u'\u0e66': 'UnicodeCharacter0xe66',
u'\u2ae9': 'ShortUpTackAboveShortDownTack',
u'\u4b68': 'CjkUnifiedIdeograph4b68',
u'\u17eb': 'UnicodeCharacter0x17eb',
u'\u54ed': 'CjkUnifiedIdeograph54ed',
u'\ubc72': 'HangulSyllableByaelm',
u'\uf974': 'CjkCompatibilityIdeographf974',
u'\u85f7': 'CjkUnifiedIdeograph85f7',
u'\u2676': 'RecyclingSymbolForType4Plastics',
u'\uc2f9': 'HangulSyllableSsag',
u'\u6378': 'CjkUnifiedIdeograph6378',
u'\u0ffb': 'UnicodeCharacter0xffb',
u'\u4cfd': 'CjkUnifiedIdeograph4cfd',
u'\u7601': 'CjkUnifiedIdeograph7601',
u'\u9184': 'CjkUnifiedIdeograph9184',
u'\ude86': 'UnicodeCharacter0xde86',
u'\u1b88': 'SundaneseLetterE',
u'\ua70b': 'ModifierLetterLowDottedToneBar',
u'\u588a': 'CjkUnifiedIdeograph588a',
u'\ue40d': 'UnicodeCharacter0xe40d',
u'\u210f': 'PlanckConstantOverTwoPi',
u'\u6e11': 'CjkUnifiedIdeograph6e11',
u'\u8994': 'CjkUnifiedIdeograph8994',
u'\uf696': 'UnicodeCharacter0xf696',
u'\u3398': 'SquareKl',
u'\u709a': 'CjkUnifiedIdeograph709a',
u'\u9c1d': 'CjkUnifiedIdeograph9c1d',
u'\ud91f': 'UnicodeCharacter0xd91f',
u'\u0621': 'ArabicLetterHamza',
u'\u4323': 'CjkUnifiedIdeograph4323',
u'\ua1a4': 'YiSyllableHluop',
u'\ueea6': 'UnicodeCharacter0xeea6',
u'\u2ba8': 'UnicodeCharacter0x2ba8',
u'\u68aa': 'CjkUnifiedIdeograph68aa',
u'\ub42d': 'HangulSyllableDoeng',
u'\u55ac': 'CjkUnifiedIdeograph55ac',
u'\uf12f': 'UnicodeCharacter0xf12f',
u'\u3e31': 'CjkUnifiedIdeograph3e31',
u'\u7b33': 'CjkUnifiedIdeograph7b33',
u'\u86b6': 'CjkUnifiedIdeograph86b6',
u'\uc3b8': 'HangulSyllableSsyeok',
u'\xba': 'MasculineOrdinalIndicator',
u'\uac3d': 'HangulSyllableGyanj',
u'\u4dbc': 'UnicodeCharacter0x4dbc',
u'\ue93f': 'UnicodeCharacter0xe93f',
u'\ud641': 'HangulSyllableHolg',
u'\u1343': 'EthiopicSyllableTzaa',
u'\u5045': 'CjkUnifiedIdeograph5045',
u'\ubec6': 'HangulSyllableBbyaebs',
u'\ufbc8': 'UnicodeCharacter0xfbc8',
u'\u38ca': 'CjkUnifiedIdeograph38ca',
u'\u65cc': 'CjkUnifiedIdeograph65cc',
u'\u814f': 'CjkUnifiedIdeograph814f',
u'\uce51': 'HangulSyllableCying',
u'\u0b53': 'UnicodeCharacter0xb53',
u'\u4855': 'CjkUnifiedIdeograph4855',
u'\u93d8': 'CjkUnifiedIdeograph93d8',
u'\ud0da': 'HangulSyllableTap',
u'\u1ddc': 'CombiningLatinSmallLetterK',
u'\ub95f': 'HangulSyllableRyud',
u'\u5ade': 'CjkUnifiedIdeograph5ade',
u'\ue661': 'UnicodeCharacter0xe661',
u'\u2363': 'AplFunctionalSymbolStarDiaeresis',
u'\u6065': 'CjkUnifiedIdeograph6065',
u'\u8be8': 'CjkUnifiedIdeograph8be8',
u'\uc8ea': 'HangulSyllableJyolm',
u'\u35ec': 'CjkUnifiedIdeograph35ec',
u'\u72ee': 'CjkUnifiedIdeograph72ee',
u'\u9e71': 'CjkUnifiedIdeograph9e71',
u'\udb73': 'UnicodeCharacter0xdb73',
u'\u1875': 'MongolianLetterManchuRa',
u'\u4577': 'CjkUnifiedIdeograph4577',
u'\ua3f8': 'YiSyllableJy',
u'\ue0fa': 'UnicodeCharacter0xe0fa',
u'\u2dfc': 'CombiningCyrillicLetterIotifiedA',
u'\u6afe': 'CjkUnifiedIdeograph6afe',
u'\u5800': 'CjkUnifiedIdeograph5800',
u'\u7c83': 'CjkUnifiedIdeograph7c83',
u'\u9b02': 'CjkUnifiedIdeograph9b02',
u'\ubf85': 'HangulSyllableBbwaelt',
u'\u1106': 'HangulChoseongMieum',
u'\u3589': 'CjkUnifiedIdeograph3589',
u'\uca0c': 'HangulSyllableJjaess',
u'\uee8f': 'UnicodeCharacter0xee8f',
u'\u4010': 'CjkUnifiedIdeograph4010',
u'\u6493': 'CjkUnifiedIdeograph6493',
u'\u8312': 'CjkUnifiedIdeograph8312',
u'\ua795': 'UnicodeCharacter0xa795',
u'\u3916': 'CjkUnifiedIdeograph3916',
u'\udd99': 'UnicodeCharacter0xdd99',
u'\u539d': 'CjkUnifiedIdeograph539d',
u'\uf21c': 'UnicodeCharacter0xf21c',
u'\u969f': 'CjkUnifiedIdeograph969f',
u'\u6820': 'CjkUnifiedIdeograph6820',
u'\u0ca3': 'KannadaLetterNna',
u'\uab22': 'UnicodeCharacter0xab22',
u'\u2126': 'OhmSign',
u'\uc5a9': 'HangulSyllableYaeb',
u'\u7bad': 'CjkUnifiedIdeograph7bad',
u'\u9a2c': 'CjkUnifiedIdeograph9a2c',
u'\ubeaf': 'HangulSyllableBbyac',
u'\u1030': 'MyanmarVowelSignUu',
u'\u34b3': 'CjkUnifiedIdeograph34b3',
u'\uc936': 'HangulSyllableJwegg',
u'\uedb9': 'UnicodeCharacter0xedb9',
u'\u4f3a': 'CjkUnifiedIdeograph4f3a',
u'\u63bd': 'CjkUnifiedIdeograph63bd',
u'\u823c': 'CjkUnifiedIdeograph823c',
u'\ua6bf': 'UnicodeCharacter0xa6bf',
u'\u3840': 'CjkUnifiedIdeograph3840',
u'\udcc3': 'UnicodeCharacter0xdcc3',
u'\u52c7': 'CjkUnifiedIdeograph52c7',
u'\uf146': 'UnicodeCharacter0xf146',
u'\u95c9': 'CjkUnifiedIdeograph95c9',
u'\u774a': 'CjkUnifiedIdeograph774a',
u'\u0bcd': 'TamilSignVirama',
u'\uaa4c': 'ChamConsonantSignFinalM',
u'\u2050': 'CloseUp',
u'\uc4d3': 'HangulSyllableSswih',
u'\u7ad7': 'CjkUnifiedIdeograph7ad7',
u'\u9956': 'CjkUnifiedIdeograph9956',
u'\ubdd9': 'HangulSyllableBwinj',
u'\u1f5a': 'UnicodeCharacter0x1f5a',
u'\u33dd': 'SquareWb',
u'\uc860': 'HangulSyllableJyels',
u'\uece3': 'UnicodeCharacter0xece3',
u'\u4e64': 'CjkUnifiedIdeograph4e64',
u'\u62e7': 'CjkUnifiedIdeograph62e7',
u'\u8166': 'CjkUnifiedIdeograph8166',
u'\ua5e9': 'VaiSyllableBhe',
u'\u076a': 'ArabicLetterLamWithBar',
u'\udbed': 'UnicodeCharacter0xdbed',
u'\u51f1': 'CjkUnifiedIdeograph51f1',
u'\uf070': 'UnicodeCharacter0xf070',
u'\u94f3': 'CjkUnifiedIdeograph94f3',
u'\u7674': 'CjkUnifiedIdeograph7674',
u'\u0af7': 'UnicodeCharacter0xaf7',
u'\ua976': 'UnicodeCharacter0xa976',
u'\u2f7a': 'KangxiRadicalSheep',
u'\uc3fd': 'HangulSyllableSswalg',
u'\uf901': 'CjkCompatibilityIdeographf901',
u'\u9880': 'CjkUnifiedIdeograph9880',
u'\u7f05': 'CjkUnifiedIdeograph7f05',
u'\u1e84': 'LatinCapitalLetterWWithDiaeresis',
u'\ub207': 'HangulSyllableNugs',
u'\u280b': 'BraillePatternDots124',
u'\ud78a': 'HangulSyllableHigg',
u'\u4d8e': 'CjkUnifiedIdeograph4d8e',
u'\ue111': 'UnicodeCharacter0xe111',
u'\u8090': 'CjkUnifiedIdeograph8090',
u'\u6715': 'CjkUnifiedIdeograph6715',
u'\u0694': 'ArabicLetterRehWithDotBelow',
u'\ud01b': 'HangulSyllableKwegs',
u'\uff9a': 'HalfwidthKatakanaLetterRe',
u'\u561f': 'CjkUnifiedIdeograph561f',
u'\u759e': 'CjkUnifiedIdeograph759e',
u'\u8921': 'CjkUnifiedIdeograph8921',
u'\ua8a0': 'SaurashtraLetterNna',
u'\u0f25': 'TibetanDigitFive',
u'\u2ea4': 'CjkRadicalPawOne',
u'\uf82b': 'UnicodeCharacter0xf82b',
u'\ue7aa': 'UnicodeCharacter0xe7aa',
u'\u7e2f': 'CjkUnifiedIdeograph7e2f',
u'\u1dae': 'ModifierLetterSmallNWithLeftHook',
u'\ub131': 'HangulSyllableNelt',
u'\u3735': 'CjkUnifiedIdeograph3735',
u'\ud6b4': 'HangulSyllableHyols',
u'\u4cb8': 'CjkUnifiedIdeograph4cb8',
u'\ue03b': 'UnicodeCharacter0xe03b',
u'\u8fba': 'CjkUnifiedIdeograph8fba',
u'\u663f': 'CjkUnifiedIdeograph663f',
u'\u05be': 'HebrewPunctuationMaqaf',
u'\udf45': 'UnicodeCharacter0xdf45',
u'\ufec4': 'ArabicLetterTahMedialForm',
u'\u5549': 'CjkUnifiedIdeograph5549',
u'\u74c8': 'CjkUnifiedIdeograph74c8',
u'\u884b': 'CjkUnifiedIdeograph884b',
u'\ub7ca': 'HangulSyllableRyaj',
u'\u0e4f': 'ThaiCharacterFongman',
u'\u2dce': 'EthiopicSyllableKyo',
u'\uc755': 'HangulSyllableEut',
u'\ue6d4': 'UnicodeCharacter0xe6d4',
u'\u7d59': 'CjkUnifiedIdeograph7d59',
u'\u1cd8': 'UnicodeCharacter0x1cd8',
u'\ub05b': 'HangulSyllableGgeuc',
u'\u365f': 'CjkUnifiedIdeograph365f',
u'\ud5de': 'HangulSyllableHeoj',
u'\u4be2': 'CjkUnifiedIdeograph4be2',
u'\uef65': 'UnicodeCharacter0xef65',
u'\u8ee4': 'CjkUnifiedIdeograph8ee4',
u'\u6569': 'CjkUnifiedIdeograph6569',
u'\u04e8': 'CyrillicCapitalLetterBarredO',
u'\ude6f': 'UnicodeCharacter0xde6f',
u'\ufdee': 'UnicodeCharacter0xfdee',
u'\u5473': 'CjkUnifiedIdeograph5473',
u'\u73f2': 'CjkUnifiedIdeograph73f2',
u'\u9775': 'CjkUnifiedIdeograph9775',
u'\ub6f4': 'HangulSyllableDdwin',
u'\u0d79': 'MalayalamDateMark',
u'\u2cf8': 'UnicodeCharacter0x2cf8',
u'\uc67f': 'HangulSyllableOed',
u'\ue5fe': 'UnicodeCharacter0xe5fe',
u'\u9006': 'CjkUnifiedIdeograph9006',
u'\ub489': 'HangulSyllableDweg',
u'\ud508': 'HangulSyllablePeun',
u'\ue98b': 'UnicodeCharacter0xe98b',
u'\u160a': 'CanadianSyllabicsCarrierMe',
u'\u2a8d': 'LessthanAboveSimilarOrEqual',
u'\u4b0c': 'CjkUnifiedIdeograph4b0c',
u'\u6f8f': 'CjkUnifiedIdeograph6f8f',
u'\ub816': 'HangulSyllableRelp',
u'\ufd18': 'ArabicLigatureSeenWithYehFinalForm',
u'\u919b': 'CjkUnifiedIdeograph919b',
u'\u3e1a': 'CjkUnifiedIdeograph3e1a',
u'\ud29d': 'HangulSyllableTyug',
u'\u731c': 'CjkUnifiedIdeograph731c',
u'\u179f': 'KhmerLetterSa',
u'\u48a1': 'CjkUnifiedIdeograph48a1',
u'\ua026': 'YiSyllableBox',
u'\ue528': 'UnicodeCharacter0xe528',
u'\ub9ab': 'HangulSyllableRyih',
u'\u262a': 'StarAndCrescent',
u'\ufaad': 'CjkCompatibilityIdeographfaad',
u'\u1b2c': 'BalineseLetterYa',
u'\u3faf': 'CjkUnifiedIdeograph3faf',
u'\u5c2e': 'CjkUnifiedIdeograph5c2e',
u'\u70b1': 'CjkUnifiedIdeograph70b1',
u'\u8d38': 'CjkUnifiedIdeograph8d38',
u'\ua1bb': 'YiSyllableLiet',
u'\uce3a': 'HangulSyllableCeup',
u'\ue2bd': 'UnicodeCharacter0xe2bd',
u'\u033c': 'CombiningSeagullBelow',
u'\u27bf': 'UnicodeCharacter0x27bf',
u'\u443e': 'CjkUnifiedIdeograph443e',
u'\u18c1': 'UnicodeCharacter0x18c1',
u'\u5dc3': 'CjkUnifiedIdeograph5dc3',
u'\ub548': 'HangulSyllableDdak',
u'\uf64a': 'UnicodeCharacter0xf64a',
u'\u8acd': 'CjkUnifiedIdeograph8acd',
u'\u2b4c': 'RightwardsArrowAboveReverseTildeOperator',
u'\ucfcf': 'HangulSyllableKyolb',
u'\u6c4e': 'CjkUnifiedIdeograph6c4e',
u'\xd1': 'LatinCapitalLetterNWithTilde',
u'\u45d3': 'CjkUnifiedIdeograph45d3',
u'\u9e5a': 'CjkUnifiedIdeograph9e5a',
u'\ub2dd': 'HangulSyllableNing',
u'\ud35c': 'HangulSyllablePyak',
u'\uf7df': 'UnicodeCharacter0xf7df',
u'\u145e': 'CanadianSyllabicsWestcreeTwo',
u'\u28e1': 'BraillePatternDots1678',
u'\u4960': 'CjkUnifiedIdeograph4960',
u'\u6de3': 'CjkUnifiedIdeograph6de3',
u'\u866a': 'CjkUnifiedIdeograph866a',
u'\ufb6c': 'ArabicLetterVehInitialForm',
u'\u9fef': 'UnicodeCharacter0x9fef',
u'\u3c6e': 'CjkUnifiedIdeograph3c6e',
u'\ud0f1': 'HangulSyllableTaeng',
u'\u7170': 'CjkUnifiedIdeograph7170',
u'\u15f3': 'CanadianSyllabicsCarrierGi',
u'\u56f5': 'CjkUnifiedIdeograph56f5',
u'\uae7a': 'HangulSyllableGgaebs',
u'\ue37c': 'UnicodeCharacter0xe37c',
u'\u87ff': 'CjkUnifiedIdeograph87ff',
u'\u247e': 'ParenthesizedNumberEleven',
u'\u1980': 'NewTaiLueLetterHighQa',
u'\ubd03': 'HangulSyllableBolh',
u'\u5a82': 'CjkUnifiedIdeograph5a82',
u'\ufe05': 'VariationSelector6',
u'\u3307': 'SquareEsukuudo',
u'\u7409': 'CjkUnifiedIdeograph7409',
u'\u8b8c': 'CjkUnifiedIdeograph8b8c',
u'\ucc8e': 'HangulSyllableCyaebs',
u'\u0190': 'LatinCapitalLetterOpenE',
u'\ua513': 'VaiSyllableRee',
u'\u4292': 'CjkUnifiedIdeograph4292',
u'\ue615': 'UnicodeCharacter0xe615',
u'\udb17': 'UnicodeCharacter0xdb17',
u'\u1c19': 'LepchaLetterDza',
u'\u511b': 'CjkUnifiedIdeograph511b',
u'\ub39c': 'HangulSyllableDyeom',
u'\uf49e': 'UnicodeCharacter0xf49e',
u'\u29a0': 'SphericalAngleOpeningLeft',
u'\u6aa2': 'CjkUnifiedIdeograph6aa2',
u'\u8e25': 'CjkUnifiedIdeograph8e25',
u'\uc327': 'HangulSyllableSsaes',
u'\u0429': 'CyrillicCapitalLetterShcha',
u'\u792b': 'CjkUnifiedIdeograph792b',
u'\u9cae': 'CjkUnifiedIdeograph9cae',
u'\ud1b0': 'HangulSyllableTom',
u'\u12b2': 'EthiopicSyllableKwi',
u'\ub635': 'HangulSyllableDdwalg',
u'\u57b4': 'CjkUnifiedIdeograph57b4',
u'\ueb37': 'UnicodeCharacter0xeb37',
u'\u2c39': 'GlagoliticSmallLetterIzhe',
u'\u613b': 'CjkUnifiedIdeograph613b',
u'\u84be': 'CjkUnifiedIdeograph84be',
u'\uf9c0': 'CjkCompatibilityIdeographf9c0',
u'\u3ac2': 'CjkUnifiedIdeograph3ac2',
u'\u7fc4': 'CjkUnifiedIdeograph7fc4',
u'\u9347': 'CjkUnifiedIdeograph9347',
u'\ud449': 'HangulSyllablePoelg',
u'\u094b': 'DevanagariVowelSignO',
u'\u4a4d': 'CjkUnifiedIdeograph4a4d',
u'\uacce': 'HangulSyllableGyelm',
u'\ue1d0': 'UnicodeCharacter0xe1d0',
u'\u22d2': 'DoubleIntersection',
u'\u67d4': 'CjkUnifiedIdeograph67d4',
u'\ubb57': 'HangulSyllableMweod',
u'\u58d6': 'CjkUnifiedIdeograph58d6',
u'\ufc59': 'ArabicLigatureYehWithAlefMaksuraIsolatedForm',
u'\u315b': 'HangulLetterYo',
u'\u725d': 'CjkUnifiedIdeograph725d',
u'\u89e0': 'CjkUnifiedIdeograph89e0',
u'\ucae2': 'HangulSyllableJjwalm',
u'\u0fe4': 'UnicodeCharacter0xfe4',
u'\ua367': 'YiSyllableChuo',
u'\u40e6': 'CjkUnifiedIdeograph40e6',
u'\ue469': 'UnicodeCharacter0xe469',
u'\ud96b': 'UnicodeCharacter0xd96b',
u'\u1a6d': 'UnicodeCharacter0x1a6d',
u'\u5f6f': 'CjkUnifiedIdeograph5f6f',
u'\ub1f0': 'HangulSyllableNyol',
u'\uf2f2': 'UnicodeCharacter0xf2f2',
u'\u37f4': 'CjkUnifiedIdeograph37f4',
u'\u68f6': 'CjkUnifiedIdeograph68f6',
u'\u8c79': 'CjkUnifiedIdeograph8c79',
u'\uc17b': 'HangulSyllableSyelb',
u'\u027d': 'LatinSmallLetterRWithTail',
u'\u477f': 'CjkUnifiedIdeograph477f',
u'\u3b81': 'CjkUnifiedIdeograph3b81',
u'\udc04': 'UnicodeCharacter0xdc04',
u'\uf087': 'UnicodeCharacter0xf087',
u'\u5608': 'CjkUnifiedIdeograph5608',
u'\u6a8b': 'CjkUnifiedIdeograph6a8b',
u'\u950a': 'CjkUnifiedIdeograph950a',
u'\ua98d': 'UnicodeCharacter0xa98d',
u'\u0f0e': 'TibetanMarkNyisShad',
u'\u2391': 'PassivepulldownoutputSymbol',
u'\uc414': 'HangulSyllableSswaen',
u'\u9897': 'CjkUnifiedIdeograph9897',
u'\u7e18': 'CjkUnifiedIdeograph7e18',
u'\u129b': 'EthiopicSyllableNyaa',
u'\ubd1a': 'HangulSyllableBwalm',
u'\u371e': 'CjkUnifiedIdeograph371e',
u'\ucba1': 'HangulSyllableJjwinj',
u'\u4da5': 'CjkUnifiedIdeograph4da5',
u'\uec24': 'UnicodeCharacter0xec24',
u'\u80a7': 'CjkUnifiedIdeograph80a7',
u'\u6628': 'CjkUnifiedIdeograph6628',
u'\u3aab': 'CjkUnifiedIdeograph3aab',
u'\ua52a': 'VaiSyllablePi',
u'\udf2e': 'UnicodeCharacter0xdf2e',
u'\uf3b1': 'UnicodeCharacter0xf3b1',
u'\u5132': 'CjkUnifiedIdeograph5132',
u'\u75b5': 'CjkUnifiedIdeograph75b5',
u'\u9434': 'CjkUnifiedIdeograph9434',
u'\ua8b7': 'SaurashtraVowelSignIi',
u'\u0e38': 'ThaiCharacterSaraU',
u'\u22bb': 'Xor',
u'\uc73e': 'HangulSyllableEugg',
u'\u9bc1': 'CjkUnifiedIdeograph9bc1',
u'\u7942': 'CjkUnifiedIdeograph7942',
u'\u1dc5': 'CombiningGravemacron',
u'\ubc44': 'HangulSyllableBaess',
u'\u3648': 'CjkUnifiedIdeograph3648',
u'\ucacb': 'HangulSyllableJjolh',
u'\u4ccf': 'CjkUnifiedIdeograph4ccf',
u'\uef4e': 'UnicodeCharacter0xef4e',
u'\u83d1': 'CjkUnifiedIdeograph83d1',
u'\u6152': 'CjkUnifiedIdeograph6152',
u'\u05d5': 'HebrewLetterVav',
u'\ua454': 'YiSyllableNyuop',
u'\ude58': 'UnicodeCharacter0xde58',
u'\uf2db': 'UnicodeCharacter0xf2db',
u'\u505c': 'CjkUnifiedIdeograph505c',
u'\u74df': 'CjkUnifiedIdeograph74df',
u'\u975e': 'CjkUnifiedIdeograph975e',
u'\uabe1': 'UnicodeCharacter0xabe1',
u'\u0962': 'DevanagariVowelSignVocalicL',
u'\u2de5': 'CombiningCyrillicLetterZe',
u'\uc668': 'HangulSyllableWaels',
u'\u9aeb': 'CjkUnifiedIdeograph9aeb',
u'\u786c': 'CjkUnifiedIdeograph786c',
u'\u1cef': 'UnicodeCharacter0x1cef',
u'\ubf6e': 'HangulSyllableBbwabs',
u'\u3172': 'HangulLetterPieupkiyeok',
u'\ud5f5': 'HangulSyllableHeb',
u'\u4ff9': 'CjkUnifiedIdeograph4ff9',
u'\uee78': 'UnicodeCharacter0xee78',
u'\u82fb': 'CjkUnifiedIdeograph82fb',
u'\u607c': 'CjkUnifiedIdeograph607c',
u'\u04ff': 'CyrillicSmallLetterHaWithStroke',
u'\ua77e': 'LatinCapitalLetterTurnedInsularG',
u'\u3e03': 'CjkUnifiedIdeograph3e03',
u'\ud982': 'UnicodeCharacter0xd982',
u'\u5386': 'CjkUnifiedIdeograph5386',
u'\uf709': 'UnicodeCharacter0xf709',
u'\u9688': 'CjkUnifiedIdeograph9688',
u'\u690d': 'CjkUnifiedIdeograph690d',
u'\u088c': 'UnicodeCharacter0x88c',
u'\uac0f': 'HangulSyllableGalh',
u'\u2613': 'Saltire',
u'\uc192': 'HangulSyllableSonh',
u'\u5817': 'CjkUnifiedIdeograph5817',
u'\u7b96': 'CjkUnifiedIdeograph7b96',
u'\u9f19': 'CjkUnifiedIdeograph9f19',
u'\ube98': 'HangulSyllableBbya',
u'\u111d': 'HangulChoseongKapyeounmieum',
u'\u309c': 'KatakanahiraganaSemivoicedSoundMark',
u'\uce23': 'HangulSyllableCeugs',
u'\ue9a2': 'UnicodeCharacter0xe9a2',
u'\u4027': 'CjkUnifiedIdeograph4027',
u'\u63a6': 'CjkUnifiedIdeograph63a6',
u'\u8729': 'CjkUnifiedIdeograph8729',
u'\ua6a8': 'UnicodeCharacter0xa6a8',
u'\u392d': 'CjkUnifiedIdeograph392d',
u'\ud8ac': 'UnicodeCharacter0xd8ac',
u'\u52b0': 'CjkUnifiedIdeograph52b0',
u'\uf633': 'UnicodeCharacter0xf633',
u'\u91b2': 'CjkUnifiedIdeograph91b2',
u'\u6837': 'CjkUnifiedIdeograph6837',
u'\u0bb6': 'TamilLetterSha',
u'\uaf39': 'HangulSyllableGgolt',
u'\u213d': 'DoublestruckSmallGamma',
u'\uc0bc': 'HangulSyllableSam',
u'\u5b41': 'CjkUnifiedIdeograph5b41',
u'\u7ac0': 'CjkUnifiedIdeograph7ac0',
u'\u9e43': 'CjkUnifiedIdeograph9e43',
u'\ub9c2': 'HangulSyllableRij',
u'\u1047': 'MyanmarDigitSeven',
u'\u33c6': 'SquareCOverKg',
u'\uc94d': 'HangulSyllableJwet',
u'\ue8cc': 'UnicodeCharacter0xe8cc',
u'\u4351': 'CjkUnifiedIdeograph4351',
u'\u62d0': 'CjkUnifiedIdeograph62d0',
u'\u8653': 'CjkUnifiedIdeograph8653',
u'\ua1d2': 'YiSyllableLurx',
u'\u3857': 'CjkUnifiedIdeograph3857',
u'\udbd6': 'UnicodeCharacter0xdbd6',
u'\u5dda': 'CjkUnifiedIdeograph5dda',
u'\uf15d': 'UnicodeCharacter0xf15d',
u'\u90dc': 'CjkUnifiedIdeograph90dc',
u'\u6b61': 'CjkUnifiedIdeograph6b61',
u'\u0ae0': 'GujaratiLetterVocalicRr',
u'\uae63': 'HangulSyllableGgac',
u'\u2067': 'UnicodeCharacter0x2067',
u'\uc3e6': 'HangulSyllableSsolp',
u'\u5a6b': 'CjkUnifiedIdeograph5a6b',
u'\u45ea': 'CjkUnifiedIdeograph45ea',
u'\u996d': 'CjkUnifiedIdeograph996d',
u'\ub8ec': 'HangulSyllableRun',
u'\u1371': 'EthiopicDigitNine',
u'\u32f0': 'CircledKatakanaMu',
u'\uc877': 'HangulSyllableJod',
u'\uebf6': 'UnicodeCharacter0xebf6',
u'\u427b': 'CjkUnifiedIdeograph427b',
u'\u6dfa': 'CjkUnifiedIdeograph6dfa',
u'\u817d': 'CjkUnifiedIdeograph817d',
u'\ua0fc': 'YiSyllableVy',
u'\uba81': 'HangulSyllableMyeob',
u'\udb00': 'UnicodeCharacter0xdb00',
u'\uff83': 'HalfwidthKatakanaLetterTe',
u'\u1802': 'MongolianComma',
u'\u3c85': 'CjkUnifiedIdeograph3c85',
u'\u5d04': 'CjkUnifiedIdeograph5d04',
u'\u7187': 'CjkUnifiedIdeograph7187',
u'\u8e0e': 'CjkUnifiedIdeograph8e0e',
u'\ua291': 'YiSyllableZiex',
u'\uc310': 'HangulSyllableSsak',
u'\ue793': 'UnicodeCharacter0xe793',
u'\u2495': 'NumberFourteenFullStop',
u'\u4514': 'CjkUnifiedIdeograph4514',
u'\u1997': 'NewTaiLueLetterLowPa',
u'\u5e99': 'CjkUnifiedIdeograph5e99',
u'\ub61e': 'HangulSyllableDdolp',
u'\ueb20': 'UnicodeCharacter0xeb20',
u'\u8fa3': 'CjkUnifiedIdeograph8fa3',
u'\u2822': 'BraillePatternDots26',
u'\ucca5': 'HangulSyllableCeolt',
u'\u6d24': 'CjkUnifiedIdeograph6d24',
u'\u01a7': 'LatinCapitalLetterToneTwo',
u'\u46a9': 'CjkUnifiedIdeograph46a9',
u'\u9330': 'CjkUnifiedIdeograph9330',
u'\ub7b3': 'HangulSyllableRaeh',
u'\ud032': 'HangulSyllableKwep',
u'\uf4b5': 'UnicodeCharacter0xf4b5',
u'\u1534': 'CanadianSyllabicsWestcreeYwii',
u'\u29b7': 'CircledParallel',
u'\u4a36': 'CjkUnifiedIdeograph4a36',
u'\u6eb9': 'CjkUnifiedIdeograph6eb9',
u'\ubb40': 'HangulSyllableMuls',
u'\uf842': 'UnicodeCharacter0xf842',
u'\u9cc5': 'CjkUnifiedIdeograph9cc5',
u'\u3d44': 'CjkUnifiedIdeograph3d44',
u'\ud1c7': 'HangulSyllableTwalb',
u'\u7246': 'CjkUnifiedIdeograph7246',
u'\u16c9': 'RunicLetterAlgizEolhx',
u'\u4bcb': 'CjkUnifiedIdeograph4bcb',
u'\ua350': 'YiSyllableZhop',
u'\ue052': 'UnicodeCharacter0xe052',
u'\u84d5': 'CjkUnifiedIdeograph84d5',
u'\u2554': 'BoxDrawingsDoubleDownAndRight',
u'\uf9d7': 'CjkCompatibilityIdeographf9d7',
u'\u1a56': 'UnicodeCharacter0x1a56',
u'\u3ed9': 'CjkUnifiedIdeograph3ed9',
u'\u5f58': 'CjkUnifiedIdeograph5f58',
u'\u73db': 'CjkUnifiedIdeograph73db',
u'\u8862': 'CjkUnifiedIdeograph8862',
u'\uace5': 'HangulSyllableGonj',
u'\ucd64': 'HangulSyllableCoel',
u'\ue1e7': 'UnicodeCharacter0xe1e7',
u'\u0266': 'LatinSmallLetterHWithHook',
u'\u26e9': 'UnicodeCharacter0x26e9',
u'\u4768': 'CjkUnifiedIdeograph4768',
u'\u1beb': 'UnicodeCharacter0x1beb',
u'\u58ed': 'CjkUnifiedIdeograph58ed',
u'\ub072': 'HangulSyllableGgyibs',
u'\uf574': 'UnicodeCharacter0xf574',
u'\u89f7': 'CjkUnifiedIdeograph89f7',
u'\u2a76': 'ThreeConsecutiveEqualsSigns',
u'\ucef9': 'HangulSyllableKeong',
u'\u6f78': 'CjkUnifiedIdeograph6f78',
u'\u03fb': 'GreekSmallLetterSan',
u'\u40fd': 'CjkUnifiedIdeograph40fd',
u'\u7a01': 'CjkUnifiedIdeograph7a01',
u'\u9d84': 'CjkUnifiedIdeograph9d84',
u'\ud286': 'HangulSyllableTwinh',
u'\u1788': 'KhmerLetterCho',
u'\uab0b': 'UnicodeCharacter0xab0b',
u'\u548a': 'CjkUnifiedIdeograph548a',
u'\ue80d': 'UnicodeCharacter0xe80d',
u'\u2d0f': 'GeorgianSmallLetterZhar',
u'\u6211': 'CjkUnifiedIdeograph6211',
u'\u8594': 'CjkUnifiedIdeograph8594',
u'\ufa96': 'CjkCompatibilityIdeographfa96',
u'\u3f98': 'CjkUnifiedIdeograph3f98',
u'\u7c9a': 'CjkUnifiedIdeograph7c9a',
u'\u901d': 'CjkUnifiedIdeograph901d',
u'\ud51f': 'HangulSyllablePeuh',
u'\u0a21': 'GurmukhiLetterDda',
u'\u4f23': 'CjkUnifiedIdeograph4f23',
u'\uada4': 'HangulSyllableGwe',
u'\ue2a6': 'UnicodeCharacter0xe2a6',
u'\u27a8': 'HeavyConcavepointedBlackRightwardsArrow',
u'\u64aa': 'CjkUnifiedIdeograph64aa',
u'\ub82d': 'HangulSyllableRyeolg',
u'\u59ac': 'CjkUnifiedIdeograph59ac',
u'\ufd2f': 'ArabicLigatureSheenWithKhahInitialForm',
u'\u3231': 'ParenthesizedIdeographStock',
u'\u7733': 'CjkUnifiedIdeograph7733',
u'\u8ab6': 'CjkUnifiedIdeograph8ab6',
u'\ucfb8': 'HangulSyllableKoem',
u'\u0cba': 'UnicodeCharacter0xcba',
u'\ua03d': 'YiSyllablePie',
u'\u41bc': 'CjkUnifiedIdeograph41bc',
u'\ue53f': 'UnicodeCharacter0xe53f',
u'\uda41': 'UnicodeCharacter0xda41',
u'\u1f43': 'GreekSmallLetterOmicronWithDasiaAndVaria',
u'\u5c45': 'CjkUnifiedIdeograph5c45',
u'\ub2c6': 'HangulSyllableNyip',
u'\uf7c8': 'UnicodeCharacter0xf7c8',
u'\u34ca': 'CjkUnifiedIdeograph34ca',
u'\u69cc': 'CjkUnifiedIdeograph69cc',
u'\u8d4f': 'CjkUnifiedIdeograph8d4f',
u'\uc251': 'HangulSyllableSweg',
u'\u0753': 'ArabicLetterBehWithThreeDotsPointingUpwardsBelowAndTwoDotsAbove',
u'\u4455': 'CjkUnifiedIdeograph4455',
u'\u9fd8': 'UnicodeCharacter0x9fd8',
u'\udcda': 'UnicodeCharacter0xdcda',
u'\u11dc': 'HangulJongseongMieumpieup',
u'\ub55f': 'HangulSyllableDdaes',
u'\u56de': 'CjkUnifiedIdeograph56de',
u'\uea61': 'UnicodeCharacter0xea61',
u'\u2f63': 'KangxiRadicalLife',
u'\u6c65': 'CjkUnifiedIdeograph6c65',
u'\u87e8': 'CjkUnifiedIdeograph87e8',
u'\uc4ea': 'HangulSyllableSsyuj',
u'\u39ec': 'CjkUnifiedIdeograph39ec',
u'\u7eee': 'CjkUnifiedIdeograph7eee',
u'\u9271': 'CjkUnifiedIdeograph9271',
u'\ud773': 'HangulSyllableHyid',
u'\u1475': 'CanadianSyllabicsWestcreeKwe',
u'\u4977': 'CjkUnifiedIdeograph4977',
u'\uaff8': 'HangulSyllableGgwel',
u'\uecfa': 'UnicodeCharacter0xecfa',
u'\u21fc': 'LeftRightArrowWithDoubleVerticalStroke',
u'\u66fe': 'CjkUnifiedIdeograph66fe',
u'\u5c00': 'CjkUnifiedIdeograph5c00',
u'\u7883': 'CjkUnifiedIdeograph7883',
u'\u9f02': 'CjkUnifiedIdeograph9f02',
u'\ubb85': 'HangulSyllableMwet',
u'\u1506': 'CanadianSyllabicsAthapascanS',
u'\u3189': 'HangulLetterYoi',
u'\uce0c': 'HangulSyllableCyul',
u'\uea8f': 'UnicodeCharacter0xea8f',
u'\u4410': 'CjkUnifiedIdeograph4410',
u'\u6093': 'CjkUnifiedIdeograph6093',
u'\u8712': 'CjkUnifiedIdeograph8712',
u'\ua395': 'YiSyllableNrax',
u'\u3d16': 'CjkUnifiedIdeograph3d16',
u'\ud999': 'UnicodeCharacter0xd999',
u'\u579d': 'CjkUnifiedIdeograph579d',
u'\uf61c': 'UnicodeCharacter0xf61c',
u'\u929f': 'CjkUnifiedIdeograph929f',
u'\u6c20': 'CjkUnifiedIdeograph6c20',
u'\u08a3': 'UnicodeCharacter0x8a3',
u'\uaf22': 'HangulSyllableGgyebs',
u'\u2526': 'BoxDrawingsUpHeavyAndLeftDownLight',
u'\uc1a9': 'HangulSyllableSwag',
u'\u7fad': 'CjkUnifiedIdeograph7fad',
u'\u9e2c': 'CjkUnifiedIdeograph9e2c',
u'\ubaaf': 'HangulSyllableMod',
u'\u1430': 'CanadianSyllabicsPaai',
u'\u30b3': 'KatakanaLetterKo',
u'\ucd36': 'HangulSyllableCwabs',
u'\ue9b9': 'UnicodeCharacter0xe9b9',
u'\u4b3a': 'CjkUnifiedIdeograph4b3a',
u'\u67bd': 'CjkUnifiedIdeograph67bd',
u'\u863c': 'CjkUnifiedIdeograph863c',
u'\ua2bf': 'YiSyllableCo',
u'\u3c40': 'CjkUnifiedIdeograph3c40',
u'\ud8c3': 'UnicodeCharacter0xd8c3',
u'\u56c7': 'CjkUnifiedIdeograph56c7',
u'\uf546': 'UnicodeCharacter0xf546',
u'\u91c9': 'CjkUnifiedIdeograph91c9',
u'\u734a': 'CjkUnifiedIdeograph734a',
u'\u0fcd': 'UnicodeCharacter0xfcd',
u'\uae4c': 'HangulSyllableGga',
u'\u2450': 'UnicodeCharacter0x2450',
u'\uc0d3': 'HangulSyllableSaelb',
u'\u7ed7': 'CjkUnifiedIdeograph7ed7',
u'\u9d56': 'CjkUnifiedIdeograph9d56',
u'\ub9d9': 'HangulSyllableMab',
u'\u1b5a': 'BalinesePanti',
u'\u37dd': 'CjkUnifiedIdeograph37dd',
u'\ucc60': 'HangulSyllableCya',
u'\ue8e3': 'UnicodeCharacter0xe8e3',
u'\u4a64': 'CjkUnifiedIdeograph4a64',
u'\u66e7': 'CjkUnifiedIdeograph66e7',
u'\u8566': 'CjkUnifiedIdeograph8566',
u'\ua1e9': 'YiSyllableGuop',
u'\u036a': 'CombiningLatinSmallLetterH',
u'\udfed': 'UnicodeCharacter0xdfed',
u'\u55f1': 'CjkUnifiedIdeograph55f1',
u'\uf470': 'UnicodeCharacter0xf470',
u'\u90f3': 'CjkUnifiedIdeograph90f3',
u'\u7274': 'CjkUnifiedIdeograph7274',
u'\u0ef7': 'UnicodeCharacter0xef7',
u'\uad76': 'HangulSyllableGulm',
u'\u2b7a': 'UnicodeCharacter0x2b7a',
u'\uc7fd': 'HangulSyllableJyaet',
u'\ufd01': 'ArabicLigatureJeemWithAlefMaksuraIsolatedForm',
u'\u9c80': 'CjkUnifiedIdeograph9c80',
u'\u7b05': 'CjkUnifiedIdeograph7b05',
u'\u1a84': 'UnicodeCharacter0x1a84',
u'\ub607': 'HangulSyllableDdyes',
u'\u2c0b': 'GlagoliticCapitalLetterI',
u'\ud38a': 'HangulSyllablePeolp',
u'\u498e': 'CjkUnifiedIdeograph498e',
u'\ue511': 'UnicodeCharacter0xe511',
u'\u8490': 'CjkUnifiedIdeograph8490',
u'\u6315': 'CjkUnifiedIdeograph6315',
u'\u0294': 'LatinLetterGlottalStop',
u'\ud41b': 'HangulSyllablePwas',
u'\ufb9a': 'ArabicLetterNgoehIsolatedForm',
u'\u521f': 'CjkUnifiedIdeograph521f',
u'\u719e': 'CjkUnifiedIdeograph719e',
u'\u8d21': 'CjkUnifiedIdeograph8d21',
u'\uaca0': 'HangulSyllableGess',
u'\u0b25': 'OriyaLetterTha',
u'\u2aa4': 'GreaterthanOverlappingLessthan',
u'\ufc2b': 'ArabicLigatureGhainWithJeemIsolatedForm',
u'\ue3aa': 'UnicodeCharacter0xe3aa',
u'\u7a2f': 'CjkUnifiedIdeograph7a2f',
u'\u19ae': 'UnicodeCharacter0x19ae',
u'\ub531': 'HangulSyllableDdag',
u'\u3335': 'SquareHuran',
u'\ud2b4': 'HangulSyllableTyuk',
u'\u48b8': 'CjkUnifiedIdeograph48b8',
u'\ue43b': 'UnicodeCharacter0xe43b',
u'\u8bba': 'CjkUnifiedIdeograph8bba',
u'\u623f': 'CjkUnifiedIdeograph623f',
u'\u01be': 'LatinLetterInvertedGlottalStopWithStroke',
u'\udb45': 'UnicodeCharacter0xdb45',
u'\ufac4': 'CjkCompatibilityIdeographfac4',
u'\u5149': 'CjkUnifiedIdeograph5149',
u'\u70c8': 'CjkUnifiedIdeograph70c8',
u'\u8c4b': 'CjkUnifiedIdeograph8c4b',
u'\ub3ca': 'HangulSyllableDonh',
u'\u0a4f': 'UnicodeCharacter0xa4f',
u'\u29ce': 'RightTriangleAboveLeftTriangle',
u'\uc355': 'HangulSyllableSsyaelg',
u'\ue2d4': 'UnicodeCharacter0xe2d4',
u'\u7959': 'CjkUnifiedIdeograph7959',
u'\u18d8': 'UnicodeCharacter0x18d8',
u'\ub45b': 'HangulSyllableDulb',
u'\u325f': 'CircledNumberThirtyFive',
u'\ud1de': 'HangulSyllableTwaenh',
u'\u4fe2': 'CjkUnifiedIdeograph4fe2',
u'\ueb65': 'UnicodeCharacter0xeb65',
u'\u8ae4': 'CjkUnifiedIdeograph8ae4',
u'\u6169': 'CjkUnifiedIdeograph6169',
u'\xe8': 'LatinSmallLetterEWithGrave',
u'\uda6f': 'UnicodeCharacter0xda6f',
u'\uf9ee': 'CjkCompatibilityIdeographf9ee',
u'\u5073': 'CjkUnifiedIdeograph5073',
u'\u77f2': 'CjkUnifiedIdeograph77f2',
u'\u9375': 'CjkUnifiedIdeograph9375',
u'\ub2f4': 'HangulSyllableDam',
u'\u0979': 'UnicodeCharacter0x979',
u'\u28f8': 'BraillePatternDots45678',
u'\uc27f': 'HangulSyllableSwis',
u'\ue1fe': 'UnicodeCharacter0xe1fe',
u'\u9406': 'CjkUnifiedIdeograph9406',
u'\ub089': 'HangulSyllableGgilt',
u'\ud108': 'HangulSyllableTyam',
u'\ued8b': 'UnicodeCharacter0xed8b',
u'\u120a': 'EthiopicSyllableLi',
u'\u2e8d': 'CjkRadicalSmallTwo',
u'\u4f0c': 'CjkUnifiedIdeograph4f0c',
u'\u6b8f': 'CjkUnifiedIdeograph6b8f',
u'\ubc16': 'HangulSyllableBagg',
u'\uf918': 'CjkCompatibilityIdeographf918',
u'\u959b': 'CjkUnifiedIdeograph959b',
u'\u3a1a': 'CjkUnifiedIdeograph3a1a',
u'\ud69d': 'HangulSyllableHoeb',
u'\u771c': 'CjkUnifiedIdeograph771c',
u'\u139f': 'UnicodeCharacter0x139f',
u'\u4ca1': 'CjkUnifiedIdeograph4ca1',
u'\ua426': 'YiSyllableJjop',
u'\ue128': 'UnicodeCharacter0xe128',
u'\ubdab': 'HangulSyllableBweolh',
u'\u222a': 'Union',
u'\ufead': 'ArabicLetterRehIsolatedForm',
u'\u1f2c': 'GreekCapitalLetterEtaWithPsiliAndOxia',
u'\u3baf': 'CjkUnifiedIdeograph3baf',
u'\u582e': 'CjkUnifiedIdeograph582e',
u'\u74b1': 'CjkUnifiedIdeograph74b1',
u'\u8938': 'CjkUnifiedIdeograph8938',
u'\ua5bb': 'VaiSyllableOn',
u'\uca3a': 'HangulSyllableJjyaelm',
u'\ue6bd': 'UnicodeCharacter0xe6bd',
u'\u073c': 'SyriacHbasaesasaDotted',
u'\u23bf': 'DentistrySymbolLightVerticalAndBottomRight',
u'\u403e': 'CjkUnifiedIdeograph403e',
u'\u1cc1': 'UnicodeCharacter0x1cc1',
u'\u59c3': 'CjkUnifiedIdeograph59c3',
u'\ub148': 'HangulSyllableNyeol',
u'\uf24a': 'UnicodeCharacter0xf24a',
u'\u8ecd': 'CjkUnifiedIdeograph8ecd',
u'\u2f4c': 'KangxiRadicalStop',
u'\ucbcf': 'HangulSyllableJjyuc',
u'\u684e': 'CjkUnifiedIdeograph684e',
u'\u04d1': 'CyrillicSmallLetterAWithBreve',
u'\u41d3': 'CjkUnifiedIdeograph41d3',
u'\u9a5a': 'CjkUnifiedIdeograph9a5a',
u'\ub6dd': 'HangulSyllableDdwelg',
u'\ud75c': 'HangulSyllableHeuls',
u'\uf3df': 'UnicodeCharacter0xf3df',
u'\u105e': 'MyanmarConsonantSignMonMedialNa',
u'\u2ce1': 'CopticSmallLetterOldNubianNyi',
u'\u4d60': 'CjkUnifiedIdeograph4d60',
u'\u69e3': 'CjkUnifiedIdeograph69e3',
u'\u826a': 'CjkUnifiedIdeograph826a',
u'\uff6c': 'HalfwidthKatakanaLetterSmallYa',
u'\u9bef': 'CjkUnifiedIdeograph9bef',
u'\u386e': 'CjkUnifiedIdeograph386e',
u'\ud4f1': 'HangulSyllablePyulg',
u'\u7570': 'CjkUnifiedIdeograph7570',
u'\u11f3': 'HangulJongseongPhieuphpieup',
u'\u52f5': 'CjkUnifiedIdeograph52f5',
u'\uaa7a': 'UnicodeCharacter0xaa7a',
u'\ue77c': 'UnicodeCharacter0xe77c',
u'\u83ff': 'CjkUnifiedIdeograph83ff',
u'\u207e': 'SuperscriptRightParenthesis',
u'\u1d80': 'LatinSmallLetterBWithPalatalHook',
u'\ub903': 'HangulSyllableRuh',
u'\u5e82': 'CjkUnifiedIdeograph5e82',
u'\ufa05': 'CjkCompatibilityIdeographfa05',
u'\u3707': 'CjkUnifiedIdeograph3707',
u'\u7009': 'CjkUnifiedIdeograph7009',
u'\u8f8c': 'CjkUnifiedIdeograph8f8c',
u'\uc88e': 'HangulSyllableJwagg',
u'\u0590': 'UnicodeCharacter0x590',
u'\ua113': 'YiSyllableDep',
u'\u4692': 'CjkUnifiedIdeograph4692',
u'\ue215': 'UnicodeCharacter0xe215',
u'\udf17': 'UnicodeCharacter0xdf17',
u'\u1819': 'MongolianDigitNine',
u'\u551b': 'CjkUnifiedIdeograph551b',
u'\ub79c': 'HangulSyllableRaen',
u'\uf09e': 'UnicodeCharacter0xf09e',
u'\u2da0': 'EthiopicSyllableSsa',
u'\u6ea2': 'CjkUnifiedIdeograph6ea2',
u'\u8a25': 'CjkUnifiedIdeograph8a25',
u'\uc727': 'HangulSyllableYud',
u'\u7d2b': 'CjkUnifiedIdeograph7d2b',
u'\u98ae': 'CjkUnifiedIdeograph98ae',
u'\ud5b0': 'HangulSyllableHyaen',
u'\u16b2': 'RunicLetterKauna',
u'\ub235': 'HangulSyllableNweong',
u'\u53b4': 'CjkUnifiedIdeograph53b4',
u'\uef37': 'UnicodeCharacter0xef37',
u'\u2839': 'BraillePatternDots1456',
u'\u653b': 'CjkUnifiedIdeograph653b',
u'\u80be': 'CjkUnifiedIdeograph80be',
u'\ufdc0': 'ArabicLigatureMeemWithJeemWithYehFinalForm',
u'\u3ec2': 'CjkUnifiedIdeograph3ec2',
u'\u7bc4': 'CjkUnifiedIdeograph7bc4',
u'\u9747': 'CjkUnifiedIdeograph9747',
u'\ud049': 'HangulSyllableKwing',
u'\u0d4b': 'MalayalamVowelSignOo',
u'\u4e4d': 'CjkUnifiedIdeograph4e4d',
u'\ua8ce': 'SaurashtraDanda',
u'\ue5d0': 'UnicodeCharacter0xe5d0',
u'\u26d2': 'UnicodeCharacter0x26d2',
u'\u63d4': 'CjkUnifiedIdeograph63d4',
u'\ubf57': 'HangulSyllableBboc',
u'\u5cd6': 'CjkUnifiedIdeograph5cd6',
u'\uf859': 'UnicodeCharacter0xf859',
u'\u355b': 'CjkUnifiedIdeograph355b',
u'\u765d': 'CjkUnifiedIdeograph765d',
u'\u8de0': 'CjkUnifiedIdeograph8de0',
u'\ucee2': 'HangulSyllableKyaep',
u'\u0be4': 'UnicodeCharacter0xbe4',
u'\ua767': 'LatinSmallLetterThornWithStrokeThroughDescender',
u'\u44e6': 'CjkUnifiedIdeograph44e6',
u'\ue069': 'UnicodeCharacter0xe069',
u'\udd6b': 'UnicodeCharacter0xdd6b',
u'\u1e6d': 'LatinSmallLetterTWithDotBelow',
u'\u5b6f': 'CjkUnifiedIdeograph5b6f',
u'\ub5f0': 'HangulSyllableDdyeok',
u'\uf6f2': 'UnicodeCharacter0xf6f2',
u'\u33f4': 'IdeographicTelegraphSymbolForDayTwentyone',
u'\u6cf6': 'CjkUnifiedIdeograph6cf6',
u'\u8879': 'CjkUnifiedIdeograph8879',
u'\uc57b': 'HangulSyllableAeh',
u'\u067d': 'ArabicLetterTehWithThreeDotsAboveDownwards',
u'\u437f': 'CjkUnifiedIdeograph437f',
u'\u3f81': 'CjkUnifiedIdeograph3f81',
u'\ud804': 'UnicodeCharacter0xd804',
u'\uf487': 'UnicodeCharacter0xf487',
u'\u5208': 'CjkUnifiedIdeograph5208',
u'\u6e8b': 'CjkUnifiedIdeograph6e8b',
u'\u910a': 'CjkUnifiedIdeograph910a',
u'\uad8d': 'HangulSyllableGweonj',
u'\u0b0e': 'UnicodeCharacter0xb0e',
u'\u2791': 'DingbatNegativeCircledSansserifDigitEight',
u'\uc014': 'HangulSyllableBbwem',
u'\u9c97': 'CjkUnifiedIdeograph9c97',
u'\u7a18': 'CjkUnifiedIdeograph7a18',
u'\u169b': 'OghamFeatherMark',
u'\ub91a': 'HangulSyllableRweoj',
u'\u331e': 'SquareKoopo',
u'\ucfa1': 'HangulSyllableKwaeng',
u'\u49a5': 'CjkUnifiedIdeograph49a5',
u'\ue824': 'UnicodeCharacter0xe824',
u'\u84a7': 'CjkUnifiedIdeograph84a7',
u'\u6228': 'CjkUnifiedIdeograph6228',
u'\u3eab': 'CjkUnifiedIdeograph3eab',
u'\ua12a': 'YiSyllableTox',
u'\udb2e': 'UnicodeCharacter0xdb2e',
u'\uf7b1': 'UnicodeCharacter0xf7b1',
u'\u5532': 'CjkUnifiedIdeograph5532',
u'\u71b5': 'CjkUnifiedIdeograph71b5',
u'\u9034': 'CjkUnifiedIdeograph9034',
u'\uacb7': 'HangulSyllableGyeolh',
u'\u0a38': 'GurmukhiLetterSa',
u'\u26bb': 'Quincunx',
u'\uc33e': 'HangulSyllableSsyalp',
u'\u9fc1': 'UnicodeCharacter0x9fc1',
u'\u7d42': 'CjkUnifiedIdeograph7d42',
u'\u19c5': 'NewTaiLueLetterFinalK',
u'\ub844': 'HangulSyllableRyen',
u'\u3248': 'UnicodeCharacter0x3248',
u'\ucecb': 'HangulSyllableKyaegs',
u'\u48cf': 'CjkUnifiedIdeograph48cf',
u'\ueb4e': 'UnicodeCharacter0xeb4e',
u'\u87d1': 'CjkUnifiedIdeograph87d1',
u'\u6552': 'CjkUnifiedIdeograph6552',
u'\u01d5': 'LatinCapitalLetterUWithDiaeresisAndMacron',
u'\ua054': 'YiSyllablePyrx',
u'\uda58': 'UnicodeCharacter0xda58',
u'\uf6db': 'UnicodeCharacter0xf6db',
u'\u545c': 'CjkUnifiedIdeograph545c',
u'\u70df': 'CjkUnifiedIdeograph70df',
u'\u935e': 'CjkUnifiedIdeograph935e',
u'\uafe1': 'HangulSyllableGgweolt',
u'\u0d62': 'MalayalamVowelSignVocalicL',
u'\u29e5': 'IdenticalToAndSlantedParallel',
u'\uc268': 'HangulSyllableSwek',
u'\u9eeb': 'CjkUnifiedIdeograph9eeb',
u'\u7c6c': 'CjkUnifiedIdeograph7c6c',
u'\u18ef': 'UnicodeCharacter0x18ef',
u'\ubb6e': 'HangulSyllableMwegg',
u'\u3572': 'CjkUnifiedIdeograph3572',
u'\ud1f5': 'HangulSyllableToeg',
u'\u4bf9': 'CjkUnifiedIdeograph4bf9',
u'\uea78': 'UnicodeCharacter0xea78',
u'\u86fb': 'CjkUnifiedIdeograph86fb',
u'\u647c': 'CjkUnifiedIdeograph647c',
u'\xff': 'LatinSmallLetterYWithDiaeresis',
u'\ua37e': 'YiSyllableRruox',
u'\u3a03': 'CjkUnifiedIdeograph3a03',
u'\udd82': 'UnicodeCharacter0xdd82',
u'\u5786': 'CjkUnifiedIdeograph5786',
u'\uf309': 'UnicodeCharacter0xf309',
u'\u9288': 'CjkUnifiedIdeograph9288',
u'\u6d0d': 'CjkUnifiedIdeograph6d0d',
u'\u0c8c': 'KannadaLetterVocalicL',
u'\ua80f': 'SylotiNagriLetterJho',
u'\u2213': 'MinusorplusSign',
u'\uc592': 'HangulSyllableYaj',
u'\u5c17': 'CjkUnifiedIdeograph5c17',
u'\u7f96': 'CjkUnifiedIdeograph7f96',
u'\u9b19': 'CjkUnifiedIdeograph9b19',
u'\uba98': 'HangulSyllableMyels',
u'\u151d': 'CanadianSyllabicsShwo',
u'\u349c': 'CjkUnifiedIdeograph349c',
u'\uca23': 'HangulSyllableJjyalh',
u'\ueda2': 'UnicodeCharacter0xeda2',
u'\u4427': 'CjkUnifiedIdeograph4427',
u'\u67a6': 'CjkUnifiedIdeograph67a6',
u'\u8329': 'CjkUnifiedIdeograph8329',
u'\ua2a8': 'YiSyllableZyt',
u'\u3d2d': 'CjkUnifiedIdeograph3d2d',
u'\udcac': 'UnicodeCharacter0xdcac',
u'\u56b0': 'CjkUnifiedIdeograph56b0',
u'\uf233': 'UnicodeCharacter0xf233',
u'\u95b2': 'CjkUnifiedIdeograph95b2',
u'\u6c37': 'CjkUnifiedIdeograph6c37',
u'\u0fb6': 'TibetanSubjoinedLetterSa',
u'\uab39': 'UnicodeCharacter0xab39',
u'\u253d': 'BoxDrawingsLeftHeavyAndRightVerticalLight',
u'\uc4bc': 'HangulSyllableSswin',
u'\u5f41': 'CjkUnifiedIdeograph5f41',
u'\u7ec0': 'CjkUnifiedIdeograph7ec0',
u'\u9a43': 'CjkUnifiedIdeograph9a43',
u'\ubdc2': 'HangulSyllableBwelm',
u'\u1447': 'CanadianSyllabicsWestcreePwaa',
u'\u37c6': 'CjkUnifiedIdeograph37c6',
u'\ucd4d': 'HangulSyllableCwaelt',
u'\ueccc': 'UnicodeCharacter0xeccc',
u'\u4751': 'CjkUnifiedIdeograph4751',
u'\u66d0': 'CjkUnifiedIdeograph66d0',
u'\u8253': 'CjkUnifiedIdeograph8253',
u'\ua5d2': 'VaiSyllableNdo',
u'\u3c57': 'CjkUnifiedIdeograph3c57',
u'\udfd6': 'UnicodeCharacter0xdfd6',
u'\u59da': 'CjkUnifiedIdeograph59da',
u'\uf55d': 'UnicodeCharacter0xf55d',
u'\u94dc': 'CjkUnifiedIdeograph94dc',
u'\u6f61': 'CjkUnifiedIdeograph6f61',
u'\u0ee0': 'UnicodeCharacter0xee0',
u'\uaa63': 'UnicodeCharacter0xaa63',
u'\u2467': 'CircledDigitEight',
u'\uc7e6': 'HangulSyllableJyaegg',
u'\u5e6b': 'CjkUnifiedIdeograph5e6b',
u'\u41ea': 'CjkUnifiedIdeograph41ea',
u'\u9d6d': 'CjkUnifiedIdeograph9d6d',
u'\ubcec': 'HangulSyllableByess',
u'\u1771': 'UnicodeCharacter0x1771',
u'\u36f0': 'CjkUnifiedIdeograph36f0',
u'\ucc77': 'HangulSyllableCyac',
u'\ueff6': 'UnicodeCharacter0xeff6',
u'\u467b': 'CjkUnifiedIdeograph467b',
u'\u69fa': 'CjkUnifiedIdeograph69fa',
u'\u857d': 'CjkUnifiedIdeograph857d',
u'\ua4fc': 'UnicodeCharacter0xa4fc',
u'\ube81': 'HangulSyllableBbaenj',
u'\udf00': 'UnicodeCharacter0xdf00',
u'\ufb83': 'ArabicLetterDdahalFinalForm',
u'\u1c02': 'LepchaLetterKha',
u'\u3885': 'CjkUnifiedIdeograph3885',
u'\u5904': 'CjkUnifiedIdeograph5904',
u'\u7587': 'CjkUnifiedIdeograph7587',
u'\u8a0e': 'CjkUnifiedIdeograph8a0e',
u'\ua691': 'CyrillicSmallLetterTsse',
u'\uc710': 'HangulSyllableWils',
u'\ue393': 'UnicodeCharacter0xe393',
u'\u0412': 'CyrillicCapitalLetterVe',
u'\u2095': 'UnicodeCharacter0x2095',
u'\u4114': 'CjkUnifiedIdeograph4114',
u'\u1d97': 'LatinSmallLetterOpenOWithRetroflexHook',
u'\u5a99': 'CjkUnifiedIdeograph5a99',
u'\ub21e': 'HangulSyllableNup',
u'\uef20': 'UnicodeCharacter0xef20',
u'\u8ba3': 'CjkUnifiedIdeograph8ba3',
u'\u2c22': 'GlagoliticCapitalLetterSpideryHa',
u'\uc8a5': 'HangulSyllableJwat',
u'\u6924': 'CjkUnifiedIdeograph6924',
u'\u05a7': 'HebrewAccentDarga',
u'\u42a9': 'CjkUnifiedIdeograph42a9',
u'\u9730': 'CjkUnifiedIdeograph9730',
u'\ub3b3': 'HangulSyllableDyelb',
u'\ud432': 'HangulSyllablePwaelp',
u'\uf0b5': 'UnicodeCharacter0xf0b5',
u'\u1134': 'HangulChoseongSiosssangsios',
u'\u2db7': 'UnicodeCharacter0x2db7',
u'\u4e36': 'CjkUnifiedIdeograph4e36',
u'\u6ab9': 'CjkUnifiedIdeograph6ab9',
u'\ubf40': 'HangulSyllableBbo',
u'\ufc42': 'ArabicLigatureLamWithMeemIsolatedForm',
u'\u98c5': 'CjkUnifiedIdeograph98c5',
u'\u3944': 'CjkUnifiedIdeograph3944',
u'\ud5c7': 'HangulSyllableHyaeh',
u'\u7646': 'CjkUnifiedIdeograph7646',
u'\u12c9': 'EthiopicSyllableWu',
u'\u4fcb': 'CjkUnifiedIdeograph4fcb',
u'\ua750': 'LatinCapitalLetterPWithStrokeThroughDescender',
u'\ue452': 'UnicodeCharacter0xe452',
u'\u80d5': 'CjkUnifiedIdeograph80d5',
u'\u2154': 'VulgarFractionTwoThirds',
u'\ufdd7': 'UnicodeCharacter0xfdd7',
u'\u1e56': 'LatinCapitalLetterPWithDotAbove',
u'\u3ad9': 'CjkUnifiedIdeograph3ad9',
u'\u5b58': 'CjkUnifiedIdeograph5b58',
u'\u77db': 'CjkUnifiedIdeograph77db',
u'\u8c62': 'CjkUnifiedIdeograph8c62',
u'\ua8e5': 'UnicodeCharacter0xa8e5',
u'\uc964': 'HangulSyllableJwiss',
u'\ue5e7': 'UnicodeCharacter0xe5e7',
u'\u0666': 'ArabicindicDigitSix',
u'\u22e9': 'SucceedsButNotEquivalentTo',
u'\u4368': 'CjkUnifiedIdeograph4368',
u'\u1feb': 'GreekCapitalLetterUpsilonWithOxia',
u'\u5ced': 'CjkUnifiedIdeograph5ced',
u'\ub472': 'HangulSyllableDweonh',
u'\uf174': 'UnicodeCharacter0xf174',
u'\u8df7': 'CjkUnifiedIdeograph8df7',
u'\u2e76': 'UnicodeCharacter0x2e76',
u'\ucaf9': 'HangulSyllableJjwaenj',
u'\u6b78': 'CjkUnifiedIdeograph6b78',
u'\u07fb': 'UnicodeCharacter0x7fb',
u'\u44fd': 'CjkUnifiedIdeograph44fd',
u'\u7e01': 'CjkUnifiedIdeograph7e01',
u'\u9984': 'CjkUnifiedIdeograph9984',
u'\ud686': 'HangulSyllableHwaej',
u'\u1388': 'EthiopicSyllableSebatbeitFwa',
u'\uaf0b': 'HangulSyllableGgyeoc',
u'\u508a': 'CjkUnifiedIdeograph508a',
u'\uec0d': 'UnicodeCharacter0xec0d',
u'\u290f': 'RightwardsTripleDashArrow',
u'\u6611': 'CjkUnifiedIdeograph6611',
u'\u8194': 'CjkUnifiedIdeograph8194',
u'\ufe96': 'ArabicLetterTehFinalForm',
u'\u3b98': 'CjkUnifiedIdeograph3b98',
u'\u789a': 'CjkUnifiedIdeograph789a',
u'\u941d': 'CjkUnifiedIdeograph941d',
u'\ud11f': 'HangulSyllableTyaelb',
u'\u0e21': 'ThaiCharacterMoMa',
u'\u4b23': 'CjkUnifiedIdeograph4b23',
u'\ua9a4': 'UnicodeCharacter0xa9a4',
u'\ue6a6': 'UnicodeCharacter0xe6a6',
u'\u23a8': 'LeftCurlyBracketMiddlePiece',
u'\u60aa': 'CjkUnifiedIdeograph60aa',
u'\ubc2d': 'HangulSyllableBat',
u'\u5dac': 'CjkUnifiedIdeograph5dac',
u'\uf92f': 'CjkCompatibilityIdeographf92f',
u'\u3631': 'CjkUnifiedIdeograph3631',
u'\u7333': 'CjkUnifiedIdeograph7333',
u'\u8eb6': 'CjkUnifiedIdeograph8eb6',
u'\ucbb8': 'HangulSyllableJjyu',
u'\u08ba': 'UnicodeCharacter0x8ba',
u'\ua43d': 'YiSyllableNjo',
u'\u45bc': 'CjkUnifiedIdeograph45bc',
u'\ue13f': 'UnicodeCharacter0xe13f',
u'\ude41': 'UnicodeCharacter0xde41',
u'\u1b43': 'BalineseVowelSignPepetTedung',
u'\u5845': 'CjkUnifiedIdeograph5845',
u'\ub6c6': 'HangulSyllableDdweolp',
u'\uf3c8': 'UnicodeCharacter0xf3c8',
u'\u30ca': 'KatakanaLetterNa',
u'\u6dcc': 'CjkUnifiedIdeograph6dcc',
u'\u894f': 'CjkUnifiedIdeograph894f',
u'\uc651': 'HangulSyllableWab',
u'\u0353': 'CombiningXBelow',
u'\u4055': 'CjkUnifiedIdeograph4055',
u'\u9bd8': 'CjkUnifiedIdeograph9bd8',
u'\ud8da': 'UnicodeCharacter0xd8da',
u'\u15dc': 'CanadianSyllabicsCarrierThu',
u'\ub15f': 'HangulSyllableNyegs',
u'\u52de': 'CjkUnifiedIdeograph52de',
u'\uee61': 'UnicodeCharacter0xee61',
u'\u2b63': 'UnicodeCharacter0x2b63',
u'\u6865': 'CjkUnifiedIdeograph6865',
u'\u83e8': 'CjkUnifiedIdeograph83e8',
u'\uc0ea': 'HangulSyllableSyanh',
u'\u3dec': 'CjkUnifiedIdeograph3dec',
u'\u7aee': 'CjkUnifiedIdeograph7aee',
u'\u9671': 'CjkUnifiedIdeograph9671',
u'\ud373': 'HangulSyllablePyaes',
u'\u1075': 'MyanmarLetterShanKa',
u'\u4d77': 'CjkUnifiedIdeograph4d77',
u'\uabf8': 'UnicodeCharacter0xabf8',
u'\ue8fa': 'UnicodeCharacter0xe8fa',
u'\u25fc': 'BlackMediumSquare',
u'\u62fe': 'CjkUnifiedIdeograph62fe',
u'\u6000': 'CjkUnifiedIdeograph6000',
u'\u4483': 'CjkUnifiedIdeograph4483',
u'\ua302': 'YiSyllableNzur',
u'\u8785': 'CjkUnifiedIdeograph8785',
u'\u2906': 'LeftwardsDoubleArrowFromBar',
u'\u0d89': 'SinhalaLetterIyanna',
u'\uf20c': 'UnicodeCharacter0xf20c',
u'\ud68f': 'HangulSyllableHoegs',
u'\u7810': 'CjkUnifiedIdeograph7810',
u'\u5c93': 'CjkUnifiedIdeograph5c93',
u'\ubb12': 'HangulSyllableMoej',
u'\u9f95': 'CjkUnifiedIdeograph9f95',
u'\u0116': 'LatinCapitalLetterEWithDotAbove',
u'\ue599': 'UnicodeCharacter0xe599',
u'\u6b9d': 'CjkUnifiedIdeograph6b9d',
u'\uca1c': 'HangulSyllableJjyal',
u'\uae9f': 'HangulSyllableGgyah',
u'\u5020': 'CjkUnifiedIdeograph5020',
u'\u34a3': 'CjkUnifiedIdeograph34a3',
u'\u9322': 'CjkUnifiedIdeograph9322',
u'\u1926': 'LimbuVowelSignAu',
u'\ufda9': 'ArabicLigatureSadWithHahWithYehFinalForm',
u'\u43ad': 'CjkUnifiedIdeograph43ad',
u'\ua22c': 'YiSyllableGgu',
u'\u86af': 'CjkUnifiedIdeograph86af',
u'\u2830': 'BraillePatternDots56',
u'\u0cb3': 'KannadaLetterLla',
u'\uf136': 'UnicodeCharacter0xf136',
u'\ud5b9': 'HangulSyllableHyaelt',
u'\u773a': 'CjkUnifiedIdeograph773a',
u'\u5bbd': 'CjkUnifiedIdeograph5bbd',
u'\uba3c': 'HangulSyllableMeon',
u'\u9ebf': 'CjkUnifiedIdeograph9ebf',
u'\ue4c3': 'UnicodeCharacter0xe4c3',
u'\u6ac7': 'CjkUnifiedIdeograph6ac7',
u'\uc946': 'HangulSyllableJwebs',
u'\uadc9': 'HangulSyllableGwilg',
u'\u4f4a': 'CjkUnifiedIdeograph4f4a',
u'\u33cd': 'SquareKk',
u'\u924c': 'CjkUnifiedIdeograph924c',
u'\u1850': 'MongolianLetterTodoTa',
u'\ufcd3': 'ArabicLigatureNoonWithHahInitialForm',
u'\u42d7': 'CjkUnifiedIdeograph42d7',
u'\ua156': 'YiSyllableNdie',
u'\u85d9': 'CjkUnifiedIdeograph85d9',
u'\u275a': 'HeavyVerticalBar',
u'\u0bdd': 'UnicodeCharacter0xbdd',
u'\uf060': 'UnicodeCharacter0xf060',
u'\ud4e3': 'HangulSyllablePwic',
u'\u7664': 'CjkUnifiedIdeograph7664',
u'\u5ae7': 'CjkUnifiedIdeograph5ae7',
u'\ub966': 'HangulSyllableRyulp',
u'\u9de9': 'CjkUnifiedIdeograph9de9',
u'\u3f6a': 'CjkUnifiedIdeograph3f6a',
u'\ue3ed': 'UnicodeCharacter0xe3ed',
u'\u69f1': 'CjkUnifiedIdeograph69f1',
u'\uc870': 'HangulSyllableJo',
u'\uacf3': 'HangulSyllableGos',
u'\u4e74': 'CjkUnifiedIdeograph4e74',
u'\u32f7': 'CircledKatakanaRi',
u'\u9176': 'CjkUnifiedIdeograph9176',
u'\u177a': 'UnicodeCharacter0x177a',
u'\ufbfd': 'ArabicLetterFarsiYehFinalForm',
u'\uc101': 'HangulSyllableSyaeg',
u'\ua080': 'YiSyllableNbap',
u'\u4705': 'CjkUnifiedIdeograph4705',
u'\u2684': 'DieFace5',
u'\u8a07': 'CjkUnifiedIdeograph8a07',
u'\u100b': 'MyanmarLetterTta',
u'\uef8a': 'UnicodeCharacter0xef8a',
u'\u758e': 'CjkUnifiedIdeograph758e',
u'\ud911': 'UnicodeCharacter0xd911',
u'\ub890': 'HangulSyllableRwak',
u'\u5f15': 'CjkUnifiedIdeograph5f15',
u'\u3e94': 'CjkUnifiedIdeograph3e94',
u'\ue81b': 'UnicodeCharacter0xe81b',
u'\uc79a': 'HangulSyllableJalm',
u'\u6e1f': 'CjkUnifiedIdeograph6e1f',
u'\u4d9e': 'CjkUnifiedIdeograph4d9e',
u'\ub121': 'HangulSyllableNeot',
u'\u90a0': 'CjkUnifiedIdeograph90a0',
u'\u3725': 'CjkUnifiedIdeograph3725',
u'\u16a4': 'RunicLetterY',
u'\uc02b': 'HangulSyllableBbwilb',
u'\udfaa': 'UnicodeCharacter0xdfaa',
u'\u462f': 'CjkUnifiedIdeograph462f',
u'\u25ae': 'BlackVerticalRectangle',
u'\u8931': 'CjkUnifiedIdeograph8931',
u'\u0f35': 'TibetanMarkNgasBzungNyiZla',
u'\ueeb4': 'UnicodeCharacter0xeeb4',
u'\u74b8': 'CjkUnifiedIdeograph74b8',
u'\ud83b': 'UnicodeCharacter0xd83b',
u'\ub7ba': 'HangulSyllableRyanh',
u'\u5e3f': 'CjkUnifiedIdeograph5e3f',
u'\u3dbe': 'CjkUnifiedIdeograph3dbe',
u'\ue745': 'UnicodeCharacter0xe745',
u'\uc6c4': 'HangulSyllableUss',
u'\u6d49': 'CjkUnifiedIdeograph6d49',
u'\u4cc8': 'CjkUnifiedIdeograph4cc8',
u'\ub04b': 'HangulSyllableGgeud',
u'\u8fca': 'CjkUnifiedIdeograph8fca',
u'\u364f': 'CjkUnifiedIdeograph364f',
u'\u15ce': 'CanadianSyllabicsCarrierRi',
u'\uff55': 'FullwidthLatinSmallLetterU',
u'\uded4': 'UnicodeCharacter0xded4',
u'\u4559': 'CjkUnifiedIdeograph4559',
u'\u24d8': 'CircledLatinSmallLetterI',
u'\u885b': 'CjkUnifiedIdeograph885b',
u'\u0e5f': 'UnicodeCharacter0xe5f',
u'\uedde': 'UnicodeCharacter0xedde',
u'\u73e2': 'CjkUnifiedIdeograph73e2',
u'\ud765': 'HangulSyllableHeung',
u'\ub6e4': 'HangulSyllableDdwem',
u'\u5d69': 'CjkUnifiedIdeograph5d69',
u'\u3ce8': 'CjkUnifiedIdeograph3ce8',
u'\ue66f': 'UnicodeCharacter0xe66f',
u'\uc5ee': 'HangulSyllableYeogg',
u'\u6c73': 'CjkUnifiedIdeograph6c73',
u'\u4bf2': 'CjkUnifiedIdeograph4bf2',
u'\uaf75': 'HangulSyllableGgwaeb',
u'\u8ef4': 'CjkUnifiedIdeograph8ef4',
u'\u3579': 'CjkUnifiedIdeograph3579',
u'\u14f8': 'CanadianSyllabicsSwi',
u'\ufe7f': 'ArabicSukunMedialForm',
u'\uddfe': 'UnicodeCharacter0xddfe',
u'\ua806': 'SylotiNagriSignHasanta',
u'\u8c89': 'CjkUnifiedIdeograph8c89',
u'\ued08': 'UnicodeCharacter0xed08',
u'\ud18b': 'HangulSyllableTyed',
u'\u2e0a': 'RightTranspositionBracket',
u'\u128d': 'EthiopicSyllableXwe',
u'\u730c': 'CjkUnifiedIdeograph730c',
u'\u578f': 'CjkUnifiedIdeograph578f',
u'\u8016': 'CjkUnifiedIdeograph8016',
u'\uc518': 'HangulSyllableSsyils',
u'\ua99b': 'UnicodeCharacter0xa99b',
u'\u061a': 'ArabicSmallKasra',
u'\uea9d': 'UnicodeCharacter0xea9d',
u'\u4b1c': 'CjkUnifiedIdeograph4b1c',
u'\u2f9f': 'KangxiRadicalBitter',
u'\u70a1': 'CjkUnifiedIdeograph70a1',
u'\u9826': 'CjkUnifiedIdeograph9826',
u'\udd28': 'UnicodeCharacter0xdd28',
u'\u81ab': 'CjkUnifiedIdeograph81ab',
u'\u1e2a': 'LatinCapitalLetterHWithBreveBelow',
u'\uc2ad': 'HangulSyllableSeulg',
u'\u232c': 'BenzeneRing',
u'\u07af': 'ThaanaOaboafili',
u'\u642e': 'CjkUnifiedIdeograph642e',
u'\u48b1': 'CjkUnifiedIdeograph48b1',
u'\ub538': 'HangulSyllableDdal',
u'\u99bb': 'CjkUnifiedIdeograph99bb',
u'\uf63a': 'UnicodeCharacter0xf63a',
u'\udabd': 'UnicodeCharacter0xdabd',
u'\u3b3c': 'CjkUnifiedIdeograph3b3c',
u'\u1fbf': 'GreekPsili',
u'\u7c3e': 'CjkUnifiedIdeograph7c3e',
u'\u20c1': 'UnicodeCharacter0x20c1',
u'\u65c3': 'CjkUnifiedIdeograph65c3',
u'\u8d48': 'CjkUnifiedIdeograph8d48',
u'\uce4a': 'HangulSyllableCyilp',
u'\ub2cd': 'HangulSyllableNinj',
u'\u134c': 'EthiopicSyllableFee',
u'\uf7cf': 'UnicodeCharacter0xf7cf',
u'\u544e': 'CjkUnifiedIdeograph544e',
u'\u38d1': 'CjkUnifiedIdeograph38d1',
u'\u7dd3': 'CjkUnifiedIdeograph7dd3',
u'\ua65a': 'CyrillicCapitalLetterBlendedYus',
u'\u8add': 'CjkUnifiedIdeograph8add',
u'\ueb5c': 'UnicodeCharacter0xeb5c',
u'\ucfdf': 'HangulSyllableKyoh',
u'\u2c5e': 'GlagoliticSmallLetterLatinateMyslite',
u'\u10e1': 'GeorgianLetterSan',
u'\u7160': 'CjkUnifiedIdeograph7160',
u'\u55e3': 'CjkUnifiedIdeograph55e3',
u'\ube6a': 'HangulSyllableBbalm',
u'\uc36c': 'HangulSyllableSseon',
u'\ua7ef': 'UnicodeCharacter0xa7ef',
u'\u046e': 'CyrillicCapitalLetterKsi',
u'\ue8f1': 'UnicodeCharacter0xe8f1',
u'\u4970': 'CjkUnifiedIdeograph4970',
u'\u2df3': 'CombiningCyrillicLetterShcha',
u'\u6ef5': 'CjkUnifiedIdeograph6ef5',
u'\u967a': 'CjkUnifiedIdeograph967a',
u'\udb7c': 'UnicodeCharacter0xdb7c',
u'\ubfff': 'HangulSyllableBbweoc',
u'\u1c7e': 'OlChikiPunctuationMucaad',
u'\u2180': 'RomanNumeralOneThousandCD',
u'\u8503': 'CjkUnifiedIdeograph8503',
u'\u6282': 'CjkUnifiedIdeograph6282',
u'\uc605': 'HangulSyllableYeot',
u'\u0b07': 'OriyaLetterI',
u'\u4c09': 'CjkUnifiedIdeograph4c09',
u'\ub38c': 'HangulSyllableDyeo',
u'\uf48e': 'UnicodeCharacter0xf48e',
u'\u3990': 'CjkUnifiedIdeograph3990',
u'\u9d13': 'CjkUnifiedIdeograph9d13',
u'\u7a92': 'CjkUnifiedIdeograph7a92',
u'\ude15': 'UnicodeCharacter0xde15',
u'\ue317': 'UnicodeCharacter0xe317',
u'\u2419': 'SymbolForEndOfMedium',
u'\u691b': 'CjkUnifiedIdeograph691b',
u'\u8b9c': 'CjkUnifiedIdeograph8b9c',
u'\ucc9e': 'HangulSyllableCeonh',
u'\u11a0': 'HangulJungseongAraeau',
u'\u52a2': 'CjkUnifiedIdeograph52a2',
u'\ub625': 'HangulSyllableDdong',
u'\ufb27': 'HebrewLetterWideResh',
u'\u3c29': 'CjkUnifiedIdeograph3c29',
u'\u412b': 'CjkUnifiedIdeograph412b',
u'\ua4ae': 'YiRadicalYit',
u'\ue9b0': 'UnicodeCharacter0xe9b0',
u'\u2ab2': 'SucceedsAboveSinglelineNotEqualTo',
u'\u8e35': 'CjkUnifiedIdeograph8e35',
u'\u6fb4': 'CjkUnifiedIdeograph6fb4',
u'\ud337': 'HangulSyllablePaelh',
u'\u1439': 'CanadianSyllabicsPaa',
u'\u593b': 'CjkUnifiedIdeograph593b',
u'\ubcbe': 'HangulSyllableByeogg',
u'\uc1c0': 'HangulSyllableSwak',
u'\u02c2': 'ModifierLetterLeftArrowhead',
u'\u47c4': 'CjkUnifiedIdeograph47c4',
u'\uab47': 'UnicodeCharacter0xab47',
u'\uec49': 'UnicodeCharacter0xec49',
u'\u314b': 'HangulLetterKhieukh',
u'\u724d': 'CjkUnifiedIdeograph724d',
u'\u94ce': 'CjkUnifiedIdeograph94ce',
u'\ud9d0': 'UnicodeCharacter0xd9d0',
u'\u1ad2': 'UnicodeCharacter0x1ad2',
u'\u5fd4': 'CjkUnifiedIdeograph5fd4',
u'\u8357': 'CjkUnifiedIdeograph8357',
u'\u60d6': 'CjkUnifiedIdeograph60d6',
u'\uc459': 'HangulSyllableSsyob',
u'\u095b': 'DevanagariLetterZa',
u'\u4a5d': 'CjkUnifiedIdeograph4a5d',
u'\ub1e0': 'HangulSyllableNoess',
u'\uf2e2': 'UnicodeCharacter0xf2e2',
u'\u37e4': 'CjkUnifiedIdeograph37e4',
u'\u9b67': 'CjkUnifiedIdeograph9b67',
u'\u78e6': 'CjkUnifiedIdeograph78e6',
u'\udc69': 'UnicodeCharacter0xdc69',
u'\ue16b': 'UnicodeCharacter0xe16b',
u'\u226d': 'NotEquivalentTo',
u'\u676f': 'CjkUnifiedIdeograph676f',
u'\u89f0': 'CjkUnifiedIdeograph89f0',
u'\ucaf2': 'HangulSyllableJjwap',
u'\u0ff4': 'UnicodeCharacter0xff4',
u'\u50f6': 'CjkUnifiedIdeograph50f6',
u'\ub479': 'HangulSyllableDweolt',
u'\uf97b': 'CjkCompatibilityIdeographf97b',
u'\u3a7d': 'CjkUnifiedIdeograph3a7d',
u'\u7f7f': 'CjkUnifiedIdeograph7f7f',
u'\u0381': 'UnicodeCharacter0x381',
u'\ue404': 'UnicodeCharacter0xe404',
u'\uc887': 'HangulSyllableJoc',
u'\u6e08': 'CjkUnifiedIdeograph6e08',
u'\u528b': 'CjkUnifiedIdeograph528b',
u'\uad0a': 'HangulSyllableGwalp',
u'\u918d': 'CjkUnifiedIdeograph918d',
u'\u370e': 'CjkUnifiedIdeograph370e',
u'\u1b91': 'SundaneseLetterNya',
u'\ufc14': 'ArabicLigatureThehWithYehIsolatedForm',
u'\ua097': 'YiSyllableHmiep',
u'\u4618': 'CjkUnifiedIdeograph4618',
u'\u2a9b': 'DoublelineSlantedEqualToOrLessthan',
u'\u851a': 'CjkUnifiedIdeograph851a',
u'\u0f1e': 'TibetanSignRdelNagGnyis',
u'\uf3a1': 'UnicodeCharacter0xf3a1',
u'\u75a5': 'CjkUnifiedIdeograph75a5',
u'\ud424': 'HangulSyllablePwae',
u'\ub8a7': 'HangulSyllableRwaes',
u'\u5e28': 'CjkUnifiedIdeograph5e28',
u'\u02ab': 'LatinSmallLetterLzDigraph',
u'\u9d2a': 'CjkUnifiedIdeograph9d2a',
u'\ue72e': 'UnicodeCharacter0xe72e',
u'\ucbb1': 'HangulSyllableJjwing',
u'\u6932': 'CjkUnifiedIdeograph6932',
u'\u4db5': 'CjkUnifiedIdeograph4db5',
u'\uac34': 'HangulSyllableGaek',
u'\u90b7': 'CjkUnifiedIdeograph90b7',
u'\u3638': 'CjkUnifiedIdeograph3638',
u'\u1abb': 'UnicodeCharacter0x1abb',
u'\uff3e': 'FullwidthCircumflexAccent',
u'\ua3c1': 'YiSyllableShyx',
u'\u4142': 'CjkUnifiedIdeograph4142',
u'\u25c5': 'WhiteLeftpointingPointer',
u'\u8444': 'CjkUnifiedIdeograph8444',
u'\u0e48': 'ThaiCharacterMaiEk',
u'\uf2cb': 'UnicodeCharacter0xf2cb',
u'\u74cf': 'CjkUnifiedIdeograph74cf',
u'\ud74e': 'HangulSyllableHyup',
u'\ubbd1': 'HangulSyllableMeub',
u'\u5952': 'CjkUnifiedIdeograph5952',
u'\u3dd5': 'CjkUnifiedIdeograph3dd5',
u'\u9c54': 'CjkUnifiedIdeograph9c54',
u'\ue658': 'UnicodeCharacter0xe658',
u'\ucadb': 'HangulSyllableJjwags',
u'\u685c': 'CjkUnifiedIdeograph685c',
u'\u4cdf': 'CjkUnifiedIdeograph4cdf',
u'\uaf5e': 'HangulSyllableGgwaj',
u'\u93e1': 'CjkUnifiedIdeograph93e1',
u'\u3162': 'HangulLetterYi',
u'\u15e5': 'CanadianSyllabicsCarrierTtee',
u'\ufe68': 'SmallReverseSolidus',
u'\ua2eb': 'YiSyllableZzyrx',
u'\u406c': 'CjkUnifiedIdeograph406c',
u'\u24ef': 'NegativeCircledNumberFifteen',
u'\u876e': 'CjkUnifiedIdeograph876e',
u'\u0972': 'DevanagariLetterCandraA',
u'\uedf5': 'UnicodeCharacter0xedf5',
u'\u77f9': 'CjkUnifiedIdeograph77f9',
u'\ud678': 'HangulSyllableHwael',
u'\ubafb': 'HangulSyllableMwaeh',
u'\u587c': 'CjkUnifiedIdeograph587c',
u'\u3cff': 'CjkUnifiedIdeograph3cff',
u'\u9f7e': 'CjkUnifiedIdeograph9f7e',
u'\u0603': 'ArabicSignSafha',
u'\ue182': 'UnicodeCharacter0xe182',
u'\u6b86': 'CjkUnifiedIdeograph6b86',
u'\ucf09': 'HangulSyllableKelg',
u'\uae88': 'HangulSyllableGgyan',
u'\u510d': 'CjkUnifiedIdeograph510d',
u'\u308c': 'HiraganaLetterRe',
u'\u940f': 'CjkUnifiedIdeograph940f',
u'\u1e13': 'LatinSmallLetterDWithCircumflexBelow',
u'\uf992': 'CjkCompatibilityIdeographf992',
u'\u6017': 'CjkUnifiedIdeograph6017',
u'\u4396': 'CjkUnifiedIdeograph4396',
u'\ua719': 'ModifierLetterDotHorizontalBar',
u'\u8698': 'CjkUnifiedIdeograph8698',
u'\u291d': 'LeftwardsArrowToBlackDiamond',
u'\u089c': 'UnicodeCharacter0x89c',
u'\uf623': 'UnicodeCharacter0xf623',
u'\ud1a2': 'HangulSyllableTogg',
u'\u7827': 'CjkUnifiedIdeograph7827',
u'\u5ba6': 'CjkUnifiedIdeograph5ba6',
u'\ubf29': 'HangulSyllableBbyenj',
u'\u9ea8': 'CjkUnifiedIdeograph9ea8',
u'\u012d': 'LatinSmallLetterIWithBreve',
u'\ue0ac': 'UnicodeCharacter0xe0ac',
u'\u6ab0': 'CjkUnifiedIdeograph6ab0',
u'\uce33': 'HangulSyllableCeus',
u'\ua9b2': 'UnicodeCharacter0xa9b2',
u'\u5037': 'CjkUnifiedIdeograph5037',
u'\u33b6': 'SquareMuV',
u'\u9739': 'CjkUnifiedIdeograph9739',
u'\u193d': 'UnicodeCharacter0x193d',
u'\uf8bc': 'UnicodeCharacter0xf8bc',
u'\u6341': 'CjkUnifiedIdeograph6341',
u'\u42c0': 'CjkUnifiedIdeograph42c0',
u'\ua643': 'CyrillicSmallLetterDzelo',
u'\u81c2': 'CjkUnifiedIdeograph81c2',
u'\u2847': 'BraillePatternDots1237',
u'\u0bc6': 'TamilVowelSignE',
u'\uf14d': 'UnicodeCharacter0xf14d',
u'\ud0cc': 'HangulSyllableTals',
u'\u7b51': 'CjkUnifiedIdeograph7b51',
u'\u5ad0': 'CjkUnifiedIdeograph5ad0',
u'\ube53': 'HangulSyllableBilh',
u'\u99d2': 'CjkUnifiedIdeograph99d2',
u'\ue3d6': 'UnicodeCharacter0xe3d6',
u'\u65da': 'CjkUnifiedIdeograph65da',
u'\uc95d': 'HangulSyllableJwilt',
u'\ua8dc': 'UnicodeCharacter0xa8dc',
u'\u5361': 'CjkUnifiedIdeograph5361',
u'\u32e0': 'CircledKatakanaTi',
u'\u9663': 'CjkUnifiedIdeograph9663',
u'\u1867': 'MongolianLetterSibeSha',
u'\ufbe6': 'ArabicLetterEInitialForm',
u'\u626b': 'CjkUnifiedIdeograph626b',
u'\u7dea': 'CjkUnifiedIdeograph7dea',
u'\ua16d': 'YiSyllableHniex',
u'\u80ec': 'CjkUnifiedIdeograph80ec',
u'\u2b71': 'UnicodeCharacter0x2b71',
u'\u0af0': 'UnicodeCharacter0xaf0',
u'\uf077': 'UnicodeCharacter0xf077',
u'\ud3f6': 'HangulSyllablePolm',
u'\u7a7b': 'CjkUnifiedIdeograph7a7b',
u'\u55fa': 'CjkUnifiedIdeograph55fa',
u'\ub97d': 'HangulSyllableReulg',
u'\u98fc': 'CjkUnifiedIdeograph98fc',
u'\u8281': 'CjkUnifiedIdeograph8281',
u'\ue300': 'UnicodeCharacter0xe300',
u'\uc783': 'HangulSyllableIlh',
u'\u2002': 'EnSpace',
u'\u0485': 'CombiningCyrillicDasiaPneumata',
u'\u6504': 'CjkUnifiedIdeograph6504',
u'\u4987': 'CjkUnifiedIdeograph4987',
u'\ub60e': 'HangulSyllableDdyep',
u'\u9a91': 'CjkUnifiedIdeograph9a91',
u'\ufb10': 'UnicodeCharacter0xfb10',
u'\udf93': 'UnicodeCharacter0xdf93',
u'\u3812': 'CjkUnifiedIdeograph3812',
u'\u1c95': 'UnicodeCharacter0x1c95',
u'\u7d14': 'CjkUnifiedIdeograph7d14',
u'\u2197': 'NorthEastArrow',
u'\u6699': 'CjkUnifiedIdeograph6699',
u'\u8e1e': 'CjkUnifiedIdeograph8e1e',
u'\ud320': 'HangulSyllablePass',
u'\ub7a3': 'HangulSyllableRaelb',
u'\u1022': 'MyanmarLetterShanA',
u'\uf4a5': 'UnicodeCharacter0xf4a5',
u'\u5524': 'CjkUnifiedIdeograph5524',
u'\u39a7': 'CjkUnifiedIdeograph39a7',
u'\u7ea9': 'CjkUnifiedIdeograph7ea9',
u'\uab30': 'UnicodeCharacter0xab30',
u'\u8fb3': 'CjkUnifiedIdeograph8fb3',
u'\ue832': 'UnicodeCharacter0xe832',
u'\uccb5': 'HangulSyllableCeg',
u'\u2d34': 'TifinaghLetterYaghh',
u'\u11b7': 'HangulJongseongMieum',
u'\u7236': 'CjkUnifiedIdeograph7236',
u'\u56b9': 'CjkUnifiedIdeograph56b9',
u'\u8340': 'CjkUnifiedIdeograph8340',
u'\uc042': 'HangulSyllableBbyunh',
u'\ua4c5': 'YiRadicalNbie',
u'\u0544': 'ArmenianCapitalLetterMen',
u'\ue9c7': 'UnicodeCharacter0xe9c7',
u'\u4a46': 'CjkUnifiedIdeograph4a46',
u'\u2ec9': 'CjkRadicalCsimplifiedShell',
u'\u73cb': 'CjkUnifiedIdeograph73cb',
u'\u9b50': 'CjkUnifiedIdeograph9b50',
u'\ud852': 'UnicodeCharacter0xd852',
u'\ubcd5': 'HangulSyllableByeot',
u'\u1d54': 'ModifierLetterSmallTopHalfO',
u'\uc1d7': 'HangulSyllableSwaes',
u'\u2256': 'RingInEqualTo',
u'\u06d9': 'ArabicSmallHighLamAlef',
u'\u6758': 'CjkUnifiedIdeograph6758',
u'\u4bdb': 'CjkUnifiedIdeograph4bdb',
u'\ub062': 'HangulSyllableGgyigg',
u'\u94e5': 'CjkUnifiedIdeograph94e5',
u'\uf564': 'UnicodeCharacter0xf564',
u'\ud9e7': 'UnicodeCharacter0xd9e7',
u'\u3a66': 'CjkUnifiedIdeograph3a66',
u'\u1ee9': 'LatinSmallLetterUWithHornAndAcute',
u'\u7f68': 'CjkUnifiedIdeograph7f68',
u'\u23eb': 'UnicodeCharacter0x23eb',
u'\u60ed': 'CjkUnifiedIdeograph60ed',
u'\u8872': 'CjkUnifiedIdeograph8872',
u'\ucd74': 'HangulSyllableCoek',
u'\ub1f7': 'HangulSyllableNyolh',
u'\u1276': 'EthiopicSyllableTo',
u'\uf6f9': 'UnicodeCharacter0xf6f9',
u'\u5778': 'CjkUnifiedIdeograph5778',
u'\u3bfb': 'CjkUnifiedIdeograph3bfb',
u'\u78fd': 'CjkUnifiedIdeograph78fd',
u'\u4201': 'CjkUnifiedIdeograph4201',
u'\ua584': 'VaiSyllableRoo',
u'\uea86': 'UnicodeCharacter0xea86',
u'\u2f88': 'KangxiRadicalBoat',
u'\u930b': 'CjkUnifiedIdeograph930b',
u'\u6c8a': 'CjkUnifiedIdeograph6c8a',
u'\ud00d': 'HangulSyllableKweob',
u'\u150f': 'CanadianSyllabicsNaskapiScwa',
u'\u5a11': 'CjkUnifiedIdeograph5a11',
u'\ubd94': 'HangulSyllableBuss',
u'\uc296': 'HangulSyllableSyulp',
u'\u0798': 'ThaanaLetterTtaa',
u'\u449a': 'CjkUnifiedIdeograph449a',
u'\ua81d': 'SylotiNagriLetterMo',
u'\ued1f': 'UnicodeCharacter0xed1f',
u'\u3221': 'ParenthesizedIdeographTwo',
u'\u7723': 'CjkUnifiedIdeograph7723',
u'\u95a4': 'CjkUnifiedIdeograph95a4',
u'\udaa6': 'UnicodeCharacter0xdaa6',
u'\u1fa8': 'GreekCapitalLetterOmegaWithPsiliAndProsgegrammeni',
u'\u5caa': 'CjkUnifiedIdeograph5caa',
u'\u802d': 'CjkUnifiedIdeograph802d',
u'\u61ac': 'CjkUnifiedIdeograph61ac',
u'\uc52f': 'HangulSyllableSsid',
u'\u0a31': 'UnicodeCharacter0xa31',
u'\u4f33': 'CjkUnifiedIdeograph4f33',
u'\ub2b6': 'HangulSyllableNyilm',
u'\uf7b8': 'UnicodeCharacter0xf7b8',
u'\u34ba': 'CjkUnifiedIdeograph34ba',
u'\u983d': 'CjkUnifiedIdeograph983d',
u'\u79bc': 'CjkUnifiedIdeograph79bc',
u'\udd3f': 'UnicodeCharacter0xdd3f',
u'\ue241': 'UnicodeCharacter0xe241',
u'\u2743': 'HeavyTeardropspokedPinwheelAsterisk',
u'\u6445': 'CjkUnifiedIdeograph6445',
u'\u8ac6': 'CjkUnifiedIdeograph8ac6',
u'\ucfc8': 'HangulSyllableKyon',
u'\u0cca': 'KannadaVowelSignO',
u'\u51cc': 'CjkUnifiedIdeograph51cc',
u'\ub54f': 'HangulSyllableDdaegs',
u'\ufa51': 'CjkCompatibilityIdeographfa51',
u'\u3f53': 'CjkUnifiedIdeograph3f53',
u'\u7c55': 'CjkUnifiedIdeograph7c55',
u'\ua7d8': 'UnicodeCharacter0xa7d8',
u'\ue4da': 'UnicodeCharacter0xe4da',
u'\u29dc': 'IncompleteInfinity',
u'\u8d5f': 'CjkUnifiedIdeograph8d5f',
u'\u6ede': 'CjkUnifiedIdeograph6ede',
u'\ud261': 'HangulSyllableTweot',
u'\u1763': 'TagbanwaLetterKa',
u'\u5465': 'CjkUnifiedIdeograph5465',
u'\ubfe8': 'HangulSyllableBbweo',
u'\ufcea': 'ArabicLigatureSheenWithHehMedialForm',
u'\u01ec': 'LatinCapitalLetterOWithOgonekAndMacron',
u'\u46ee': 'CjkUnifiedIdeograph46ee',
u'\uaa71': 'UnicodeCharacter0xaa71',
u'\uef73': 'UnicodeCharacter0xef73',
u'\u2c75': 'LatinCapitalLetterHalfH',
u'\u7177': 'CjkUnifiedIdeograph7177',
u'\u97f8': 'CjkUnifiedIdeograph97f8',
u'\ud4fa': 'HangulSyllablePyubs',
u'\u19fc': 'KhmerSymbolDappiiRoc',
u'\u5efe': 'CjkUnifiedIdeograph5efe',
u'\u6400': 'CjkUnifiedIdeograph6400',
u'\u4083': 'CjkUnifiedIdeograph4083',
u'\ua702': 'ModifierLetterChineseToneYinShang',
u'\u8385': 'CjkUnifiedIdeograph8385',
u'\u2d06': 'GeorgianSmallLetterZen',
u'\u0989': 'BengaliLetterU',
u'\uf60c': 'UnicodeCharacter0xf60c',
u'\ud28f': 'HangulSyllableTwilh',
u'\u7c10': 'CjkUnifiedIdeograph7c10',
u'\u5893': 'CjkUnifiedIdeograph5893',
u'\ubf12': 'HangulSyllableBbyeolm',
u'\u9b95': 'CjkUnifiedIdeograph9b95',
u'\u0516': 'CyrillicCapitalLetterRha',
u'\ue199': 'UnicodeCharacter0xe199',
u'\u6f9d': 'CjkUnifiedIdeograph6f9d',
u'\uce1c': 'HangulSyllableCyuk',
u'\uaa9f': 'UnicodeCharacter0xaa9f',
u'\u5420': 'CjkUnifiedIdeograph5420',
u'\u30a3': 'KatakanaLetterSmallI',
u'\u9722': 'CjkUnifiedIdeograph9722',
u'\u1d26': 'GreekLetterSmallCapitalGamma',
u'\uf9a9': 'CjkCompatibilityIdeographf9a9',
u'\u47ad': 'CjkUnifiedIdeograph47ad',
u'\ua62c': 'UnicodeCharacter0xa62c',
u'\u82af': 'CjkUnifiedIdeograph82af',
u'\u2c30': 'GlagoliticSmallLetterAzu',
u'\u08b3': 'UnicodeCharacter0x8b3',
u'\uf536': 'UnicodeCharacter0xf536',
u'\ud1b9': 'HangulSyllableTot',
u'\u733a': 'CjkUnifiedIdeograph733a',
u'\u5fbd': 'CjkUnifiedIdeograph5fbd',
u'\ube3c': 'HangulSyllableByiss',
u'\u9abf': 'CjkUnifiedIdeograph9abf',
u'\u0440': 'CyrillicSmallLetterEr',
u'\ue0c3': 'UnicodeCharacter0xe0c3',
u'\u6ec7': 'CjkUnifiedIdeograph6ec7',
u'\ucd46': 'HangulSyllableCwaenh',
u'\ua9c9': 'UnicodeCharacter0xa9c9',
u'\u4b4a': 'CjkUnifiedIdeograph4b4a',
u'\u37cd': 'CjkUnifiedIdeograph37cd',
u'\u964c': 'CjkUnifiedIdeograph964c',
u'\u1c50': 'OlChikiDigitZero',
u'\uf8d3': 'UnicodeCharacter0xf8d3',
u'\u46d7': 'CjkUnifiedIdeograph46d7',
u'\ua556': 'VaiSyllableMgba',
u'\u81d9': 'CjkUnifiedIdeograph81d9',
u'\u235a': 'AplFunctionalSymbolDiamondUnderbar',
u'\u0fdd': 'UnicodeCharacter0xfdd',
u'\uf460': 'UnicodeCharacter0xf460',
u'\ud0e3': 'HangulSyllableTaed',
u'\u7264': 'CjkUnifiedIdeograph7264',
u'\u5ee7': 'CjkUnifiedIdeograph5ee7',
u'\ubd66': 'HangulSyllableByogg',
u'\u99e9': 'CjkUnifiedIdeograph99e9',
u'\u3b6a': 'CjkUnifiedIdeograph3b6a',
u'\ue7ed': 'UnicodeCharacter0xe7ed',
u'\u6df1': 'CjkUnifiedIdeograph6df1',
u'\ucc70': 'HangulSyllableCyam',
u'\ua8f3': 'UnicodeCharacter0xa8f3',
u'\u4a74': 'CjkUnifiedIdeograph4a74',
u'\u36f7': 'CjkUnifiedIdeograph36f7',
u'\u9576': 'CjkUnifiedIdeograph9576',
u'\u137a': 'EthiopicNumberNinety',
u'\ufffd': 'ReplacementCharacter',
u'\uc501': 'HangulSyllableSseub',
u'\ua480': 'YiSyllableYop',
u'\u4305': 'CjkUnifiedIdeograph4305',
u'\u2284': 'NotASubsetOf',
u'\u8e07': 'CjkUnifiedIdeograph8e07',
u'\u140b': 'CanadianSyllabicsAa',
u'\ueb8a': 'UnicodeCharacter0xeb8a',
u'\u718e': 'CjkUnifiedIdeograph718e',
u'\udd11': 'UnicodeCharacter0xdd11',
u'\ubc90': 'HangulSyllableBeols',
u'\u5b15': 'CjkUnifiedIdeograph5b15',
u'\u3a94': 'CjkUnifiedIdeograph3a94',
u'\uec1b': 'UnicodeCharacter0xec1b',
u'\uc39a': 'HangulSyllableSsej',
u'\u6a1f': 'CjkUnifiedIdeograph6a1f',
u'\u499e': 'CjkUnifiedIdeograph499e',
u'\ub521': 'HangulSyllableDilt',
u'\u94a0': 'CjkUnifiedIdeograph94a0',
u'\u3325': 'SquareDesi',
u'\u12a4': 'EthiopicSyllableGlottalEe',
u'\uc42b': 'HangulSyllableSswaeh',
u'\udbaa': 'UnicodeCharacter0xdbaa',
u'\u422f': 'CjkUnifiedIdeograph422f',
u'\u21ae': 'LeftRightArrowWithStroke',
u'\u8d31': 'CjkUnifiedIdeograph8d31',
u'\u0b35': 'OriyaLetterVa',
u'\ueab4': 'UnicodeCharacter0xeab4',
u'\u70b8': 'CjkUnifiedIdeograph70b8',
u'\udc3b': 'UnicodeCharacter0xdc3b',
u'\ub3ba': 'HangulSyllableDyebs',
u'\u5a3f': 'CjkUnifiedIdeograph5a3f',
u'\u39be': 'CjkUnifiedIdeograph39be',
u'\ue345': 'UnicodeCharacter0xe345',
u'\uc2c4': 'HangulSyllableSyin',
u'\u6949': 'CjkUnifiedIdeograph6949',
u'\u48c8': 'CjkUnifiedIdeograph48c8',
u'\ub44b': 'HangulSyllableDyoc',
u'\u8bca': 'CjkUnifiedIdeograph8bca',
u'\u324f': 'UnicodeCharacter0x324f',
u'\u11ce': 'HangulJongseongRieultikeut',
u'\ufb55': 'ArabicLetterBeehMedialForm',
u'\udad4': 'UnicodeCharacter0xdad4',
u'\u4159': 'CjkUnifiedIdeograph4159',
u'\u20d8': 'CombiningRingOverlay',
u'\u8c5b': 'CjkUnifiedIdeograph8c5b',
u'\u0a5f': 'UnicodeCharacter0xa5f',
u'\ue9de': 'UnicodeCharacter0xe9de',
u'\u77e2': 'CjkUnifiedIdeograph77e2',
u'\ud365': 'HangulSyllablePyaenj',
u'\ub2e4': 'HangulSyllableDa',
u'\u5969': 'CjkUnifiedIdeograph5969',
u'\u38e8': 'CjkUnifiedIdeograph38e8',
u'\ue26f': 'UnicodeCharacter0xe26f',
u'\uc1ee': 'HangulSyllableSoelp',
u'\u6873': 'CjkUnifiedIdeograph6873',
u'\u4ff2': 'CjkUnifiedIdeograph4ff2',
u'\uab75': 'UnicodeCharacter0xab75',
u'\u8af4': 'CjkUnifiedIdeograph8af4',
u'\u3179': 'HangulLetterKapyeounssangpieup',
u'\u10f8': 'GeorgianLetterElifi',
u'\ufa7f': 'CjkCompatibilityIdeographfa7f',
u'\ud9fe': 'UnicodeCharacter0xd9fe',
u'\uac06': 'HangulSyllableGanh',
u'\u8889': 'CjkUnifiedIdeograph8889',
u'\ue908': 'UnicodeCharacter0xe908',
u'\ud58b': 'HangulSyllableHaec',
u'\u2a0a': 'ModuloTwoSum',
u'\u168d': 'OghamLetterNgeadal',
u'\u770c': 'CjkUnifiedIdeograph770c',
u'\u538f': 'CjkUnifiedIdeograph538f',
u'\u8416': 'CjkUnifiedIdeograph8416',
u'\uc118': 'HangulSyllableSyaek',
u'\uad9b': 'HangulSyllableGweos',
u'\u021a': 'LatinCapitalLetterTWithCommaBelow',
u'\uee9d': 'UnicodeCharacter0xee9d',
u'\u4f1c': 'CjkUnifiedIdeograph4f1c',
u'\u2b9f': 'UnicodeCharacter0x2b9f',
u'\u74a1': 'CjkUnifiedIdeograph74a1',
u'\u9c26': 'CjkUnifiedIdeograph9c26',
u'\ud928': 'UnicodeCharacter0xd928',
u'\u85ab': 'CjkUnifiedIdeograph85ab',
u'\u1a2a': 'UnicodeCharacter0x1a2a',
u'\uc6ad': 'HangulSyllableYot',
u'\u272c': 'BlackCentreWhiteStar',
u'\u03af': 'GreekSmallLetterIotaWithTonos',
u'\u602e': 'CjkUnifiedIdeograph602e',
u'\u4cb1': 'CjkUnifiedIdeograph4cb1',
u'\ub138': 'HangulSyllableNess',
u'\u9dbb': 'CjkUnifiedIdeograph9dbb',
u'\uf23a': 'UnicodeCharacter0xf23a',
u'\udebd': 'UnicodeCharacter0xdebd',
u'\u3f3c': 'CjkUnifiedIdeograph3f3c',
u'\u1bbf': 'UnicodeCharacter0x1bbf',
u'\u783e': 'CjkUnifiedIdeograph783e',
u'\u24c1': 'CircledLatinCapitalLetterL',
u'\u61c3': 'CjkUnifiedIdeograph61c3',
u'\u8948': 'CjkUnifiedIdeograph8948',
u'\uca4a': 'HangulSyllableJjyaep',
u'\ub6cd': 'HangulSyllableDdweong',
u'\u174c': 'BuhidLetterYa',
u'\uf3cf': 'UnicodeCharacter0xf3cf',
u'\u504e': 'CjkUnifiedIdeograph504e',
u'\u3cd1': 'CjkUnifiedIdeograph3cd1',
u'\u79d3': 'CjkUnifiedIdeograph79d3',
u'\ua25a': 'YiSyllableHxex',
u'\u8edd': 'CjkUnifiedIdeograph8edd',
u'\uef5c': 'UnicodeCharacter0xef5c',
u'\ucbdf': 'HangulSyllableJjeulb',
u'\u285e': 'BraillePatternDots23457',
u'\u14e1': 'CanadianSyllabicsWestcreeLwii',
u'\u7560': 'CjkUnifiedIdeograph7560',
u'\u51e3': 'CjkUnifiedIdeograph51e3',
u'\uba6a': 'HangulSyllableMej',
u'\uc76c': 'HangulSyllableYiss',
u'\ua3ef': 'YiSyllableJop',
u'\uecf1': 'UnicodeCharacter0xecf1',
u'\u4d70': 'CjkUnifiedIdeograph4d70',
u'\u29f3': 'ErrorbarredBlackCircle',
u'\u6af5': 'CjkUnifiedIdeograph6af5',
u'\u927a': 'CjkUnifiedIdeograph927a',
u'\udf7c': 'UnicodeCharacter0xdf7c',
u'\ubbff': 'HangulSyllableMid',
u'\u187e': 'UnicodeCharacter0x187e',
u'\u2580': 'UpperHalfBlock',
u'\u8103': 'CjkUnifiedIdeograph8103',
u'\u6682': 'CjkUnifiedIdeograph6682',
u'\uc205': 'HangulSyllableSyolg',
u'\u0f07': 'TibetanMarkYigMgoTshegShadMa',
u'\u4809': 'CjkUnifiedIdeograph4809',
u'\ub78c': 'HangulSyllableRam',
u'\uf08e': 'UnicodeCharacter0xf08e',
u'\u3d90': 'CjkUnifiedIdeograph3d90',
u'\u9913': 'CjkUnifiedIdeograph9913',
u'\u7e92': 'CjkUnifiedIdeograph7e92',
u'\uda15': 'UnicodeCharacter0xda15',
u'\ue717': 'UnicodeCharacter0xe717',
u'\u2019': 'RightSingleQuotationMark',
u'\u6d1b': 'CjkUnifiedIdeograph6d1b',
u'\u8f9c': 'CjkUnifiedIdeograph8f9c',
u'\uc89e': 'HangulSyllableJwabs',
u'\u15a0': 'CanadianSyllabicsLhi',
u'\u56a2': 'CjkUnifiedIdeograph56a2',
u'\ub225': 'HangulSyllableNweonj',
u'\uff27': 'FullwidthLatinCapitalLetterG',
u'\u3829': 'CjkUnifiedIdeograph3829',
u'\u452b': 'CjkUnifiedIdeograph452b',
u'\ua0ae': 'YiSyllableMit',
u'\uedb0': 'UnicodeCharacter0xedb0',
u'\u2eb2': 'CjkRadicalNetTwo',
u'\u8a35': 'CjkUnifiedIdeograph8a35',
u'\u6bb4': 'CjkUnifiedIdeograph6bb4',
u'\ud737': 'HangulSyllableHyugs',
u'\u1039': 'MyanmarSignVirama',
u'\u5d3b': 'CjkUnifiedIdeograph5d3b',
u'\ub8be': 'HangulSyllableRoelp',
u'\uc5c0': 'HangulSyllableEols',
u'\u06c2': 'ArabicLetterHehGoalWithHamzaAbove',
u'\u43c4': 'CjkUnifiedIdeograph43c4',
u'\uaf47': 'HangulSyllableGgoh',
u'\ue849': 'UnicodeCharacter0xe849',
u'\u354b': 'CjkUnifiedIdeograph354b',
u'\u764d': 'CjkUnifiedIdeograph764d',
u'\u90ce': 'CjkUnifiedIdeograph90ce',
u'\uddd0': 'UnicodeCharacter0xddd0',
u'\u1ed2': 'LatinCapitalLetterOWithCircumflexAndGrave',
u'\u5bd4': 'CjkUnifiedIdeograph5bd4',
u'\u8757': 'CjkUnifiedIdeograph8757',
u'\u64d6': 'CjkUnifiedIdeograph64d6',
u'\uc059': 'HangulSyllableBbeug',
u'\u0d5b': 'UnicodeCharacter0xd5b',
u'\u4e5d': 'CjkUnifiedIdeograph4e5d',
u'\ub5e0': 'HangulSyllableDdyeol',
u'\uf6e2': 'UnicodeCharacter0xf6e2',
u'\u33e4': 'IdeographicTelegraphSymbolForDayFive',
u'\u9f67': 'CjkUnifiedIdeograph9f67',
u'\u7ce6': 'CjkUnifiedIdeograph7ce6',
u'\ud869': 'UnicodeCharacter0xd869',
u'\ue56b': 'UnicodeCharacter0xe56b',
u'\u266d': 'MusicFlatSign',
u'\u636f': 'CjkUnifiedIdeograph636f',
u'\u8df0': 'CjkUnifiedIdeograph8df0',
u'\ucef2': 'HangulSyllableKeolp',
u'\u0bf4': 'TamilMonthSign',
u'\u54f6': 'CjkUnifiedIdeograph54f6',
u'\ub079': 'HangulSyllableGgyit',
u'\ufd7b': 'ArabicLigatureGhainWithMeemWithAlefMaksuraFinalForm',
u'\u3e7d': 'CjkUnifiedIdeograph3e7d',
u'\u7b7f': 'CjkUnifiedIdeograph7b7f',
u'\u0781': 'ThaanaLetterShaviyani',
u'\ue004': 'UnicodeCharacter0xe004',
u'\ucc87': 'HangulSyllableCyaelb',
u'\u6a08': 'CjkUnifiedIdeograph6a08',
u'\u568b': 'CjkUnifiedIdeograph568b',
u'\ua90a': 'KayahLiLetterKa',
u'\u958d': 'CjkUnifiedIdeograph958d',
u'\u330e': 'SquareGaron',
u'\u1f91': 'GreekSmallLetterEtaWithDasiaAndYpogegrammeni',
u'\uf814': 'UnicodeCharacter0xf814',
u'\ua497': 'YiRadicalGep',
u'\u4218': 'CjkUnifiedIdeograph4218',
u'\u2e9b': 'CjkRadicalChoke',
u'\u811a': 'CjkUnifiedIdeograph811a',
u'\u0b1e': 'OriyaLetterNya',
u'\uf7a1': 'UnicodeCharacter0xf7a1',
u'\u71a5': 'CjkUnifiedIdeograph71a5',
u'\ud024': 'HangulSyllableKwels',
u'\ubca7': 'HangulSyllableBed',
u'\u5a28': 'CjkUnifiedIdeograph5a28',
u'\u06ab': 'ArabicLetterKafWithRing',
u'\u992a': 'CjkUnifiedIdeograph992a',
u'\ue32e': 'UnicodeCharacter0xe32e',
u'\ucfb1': 'HangulSyllableKoelg',
u'\u6d32': 'CjkUnifiedIdeograph6d32',
u'\u49b5': 'CjkUnifiedIdeograph49b5',
u'\ua834': 'UnicodeCharacter0xa834',
u'\u94b7': 'CjkUnifiedIdeograph94b7',
u'\u3238': 'ParenthesizedIdeographLabor',
u'\u1ebb': 'LatinSmallLetterEWithHookAbove',
u'\ufb3e': 'HebrewLetterMemWithDagesh',
u'\ua7c1': 'UnicodeCharacter0xa7c1',
u'\u4542': 'CjkUnifiedIdeograph4542',
u'\u21c5': 'UpwardsArrowLeftwardsOfDownwardsArrow',
u'\u8044': 'CjkUnifiedIdeograph8044',
u'\u0a48': 'GurmukhiVowelSignAi',
u'\uf6cb': 'UnicodeCharacter0xf6cb',
u'\u70cf': 'CjkUnifiedIdeograph70cf',
u'\ud34e': 'HangulSyllablePyalm',
u'\ubfd1': 'HangulSyllableBbunj',
u'\u5d52': 'CjkUnifiedIdeograph5d52',
u'\u39d5': 'CjkUnifiedIdeograph39d5',
u'\u9854': 'CjkUnifiedIdeograph9854',
u'\ue258': 'UnicodeCharacter0xe258',
u'\ucedb': 'HangulSyllableKyaes',
u'\u6c5c': 'CjkUnifiedIdeograph6c5c',
u'\u48df': 'CjkUnifiedIdeograph48df',
u'\uab5e': 'UnicodeCharacter0xab5e',
u'\u97e1': 'CjkUnifiedIdeograph97e1',
u'\u3562': 'CjkUnifiedIdeograph3562',
u'\u11e5': 'HangulJongseongPieuphieuh',
u'\ufa68': 'CjkCompatibilityIdeographfa68',
u'\ua6eb': 'UnicodeCharacter0xa6eb',
u'\u446c': 'CjkUnifiedIdeograph446c',
u'\u20ef': 'CombiningRightArrowBelow',
u'\u836e': 'CjkUnifiedIdeograph836e',
u'\u0d72': 'MalayalamNumberOneThousand',
u'\ue9f5': 'UnicodeCharacter0xe9f5',
u'\u73f9': 'CjkUnifiedIdeograph73f9',
u'\ud278': 'HangulSyllableTwess',
u'\ubefb': 'HangulSyllableBbelh',
u'\u5c7c': 'CjkUnifiedIdeograph5c7c',
u'\u38ff': 'CjkUnifiedIdeograph38ff',
u'\u9b7e': 'CjkUnifiedIdeograph9b7e',
u'\u0203': 'LatinSmallLetterAWithInvertedBreve',
u'\ue582': 'UnicodeCharacter0xe582',
u'\u6f86': 'CjkUnifiedIdeograph6f86',
u'\ucb09': 'HangulSyllableJjwaeng',
u'\uaa88': 'UnicodeCharacter0xaa88',
u'\u550d': 'CjkUnifiedIdeograph550d',
u'\u348c': 'CjkUnifiedIdeograph348c',
u'\u900f': 'CjkUnifiedIdeograph900f',
u'\u1a13': 'BugineseLetterVa',
u'\ufd92': 'ArabicLigatureMeemWithJeemWithKhahInitialForm',
u'\u6417': 'CjkUnifiedIdeograph6417',
u'\u4796': 'CjkUnifiedIdeograph4796',
u'\ua319': 'YiSyllableSo',
u'\u8298': 'CjkUnifiedIdeograph8298',
u'\u2d1d': 'GeorgianSmallLetterChar',
u'\u0c9c': 'KannadaLetterJa',
u'\uf223': 'UnicodeCharacter0xf223',
u'\ud5a2': 'HangulSyllableHyabs',
u'\u7c27': 'CjkUnifiedIdeograph7c27',
u'\u5fa6': 'CjkUnifiedIdeograph5fa6',
u'\ubb29': 'HangulSyllableMyob',
u'\u9aa8': 'CjkUnifiedIdeograph9aa8',
u'\u052d': 'UnicodeCharacter0x52d',
u'\ue4ac': 'UnicodeCharacter0xe4ac',
u'\u6eb0': 'CjkUnifiedIdeograph6eb0',
u'\uca33': 'HangulSyllableJjyaegs',
u'\uadb2': 'HangulSyllableGwelp',
u'\u5437': 'CjkUnifiedIdeograph5437',
u'\u37b6': 'CjkUnifiedIdeograph37b6',
u'\u9339': 'CjkUnifiedIdeograph9339',
u'\u1d3d': 'ModifierLetterCapitalOu',
u'\ufcbc': 'ArabicLigatureGhainWithJeemInitialForm',
u'\u6741': 'CjkUnifiedIdeograph6741',
u'\u46c0': 'CjkUnifiedIdeograph46c0',
u'\ua243': 'YiSyllableMgup',
u'\u85c2': 'CjkUnifiedIdeograph85c2',
u'\u2c47': 'GlagoliticSmallLetterFritu',
u'\u0fc6': 'TibetanSymbolPadmaGdan',
u'\uf54d': 'UnicodeCharacter0xf54d',
u'\ud4cc': 'HangulSyllablePwi',
u'\u7f51': 'CjkUnifiedIdeograph7f51',
u'\u5ed0': 'CjkUnifiedIdeograph5ed0',
u'\uba53': 'HangulSyllableMeoh',
u'\u9dd2': 'CjkUnifiedIdeograph9dd2',
u'\u0457': 'CyrillicSmallLetterYi',
u'\ue7d6': 'UnicodeCharacter0xe7d6',
u'\u61da': 'CjkUnifiedIdeograph61da',
u'\ucd5d': 'HangulSyllableCoeg',
u'\uacdc': 'HangulSyllableGyek',
u'\u5761': 'CjkUnifiedIdeograph5761',
u'\u36e0': 'CjkUnifiedIdeograph36e0',
u'\u9263': 'CjkUnifiedIdeograph9263',
u'\u1c67': 'OlChikiLetterIny',
u'\uffe6': 'FullwidthWonSign',
u'\u666b': 'CjkUnifiedIdeograph666b',
u'\u79ea': 'CjkUnifiedIdeograph79ea',
u'\ua56d': 'VaiSyllableGa',
u'\u84ec': 'CjkUnifiedIdeograph84ec',
u'\u2f71': 'KangxiRadicalTrack',
u'\u0ef0': 'UnicodeCharacter0xef0',
u'\uf477': 'UnicodeCharacter0xf477',
u'\ud7f6': 'UnicodeCharacter0xd7f6',
u'\u7e7b': 'CjkUnifiedIdeograph7e7b',
u'\u51fa': 'CjkUnifiedIdeograph51fa',
u'\ubd7d': 'HangulSyllableByot',
u'\u9cfc': 'CjkUnifiedIdeograph9cfc',
u'\u8681': 'CjkUnifiedIdeograph8681',
u'\ue700': 'UnicodeCharacter0xe700',
u'\uc383': 'HangulSyllableSseoh',
u'\u2402': 'SymbolForStartOfText',
u'\x85': 'UnicodeCharacter0x85',
u'\u6104': 'CjkUnifiedIdeograph6104',
u'\u4d87': 'CjkUnifiedIdeograph4d87',
u'\ub20e': 'HangulSyllableNulm',
u'\u9e91': 'CjkUnifiedIdeograph9e91',
u'\uff10': 'FullwidthDigitZero',
u'\udb93': 'UnicodeCharacter0xdb93',
u'\u3c12': 'CjkUnifiedIdeograph3c12',
u'\u1895': 'MongolianLetterAliGaliZha',
u'\u7914': 'CjkUnifiedIdeograph7914',
u'\u2597': 'QuadrantLowerRight',
u'\u6299': 'CjkUnifiedIdeograph6299',
u'\u8a1e': 'CjkUnifiedIdeograph8a1e',
u'\ud720': 'HangulSyllableHwil',
u'\ub3a3': 'HangulSyllableDyeoc',
u'\u1422': 'CanadianSyllabicsFinalTopHalfRing',
u'\uf0a5': 'UnicodeCharacter0xf0a5',
u'\u5124': 'CjkUnifiedIdeograph5124',
u'\u3da7': 'CjkUnifiedIdeograph3da7',
u'\u7aa9': 'CjkUnifiedIdeograph7aa9',
u'\uaf30': 'HangulSyllableGgon',
u'\u8bb3': 'CjkUnifiedIdeograph8bb3',
u'\uec32': 'UnicodeCharacter0xec32',
u'\uc8b5': 'HangulSyllableJwaelt',
u'\u2934': 'ArrowPointingRightwardsThenCurvingUpwards',
u'\u15b7': 'CanadianSyllabicsBlackfootWa',
u'\u7636': 'CjkUnifiedIdeograph7636',
u'\u52b9': 'CjkUnifiedIdeograph52b9',
u'\u8740': 'CjkUnifiedIdeograph8740',
u'\uc442': 'HangulSyllableSsoej',
u'\ua0c5': 'YiSyllableMu',
u'\u0144': 'LatinSmallLetterNWithAcute',
u'\uedc7': 'UnicodeCharacter0xedc7',
u'\u4e46': 'CjkUnifiedIdeograph4e46',
u'\u2ac9': 'SubsetOfAboveAlmostEqualTo',
u'\u77cb': 'CjkUnifiedIdeograph77cb',
u'\u9f50': 'CjkUnifiedIdeograph9f50',
u'\udc52': 'UnicodeCharacter0xdc52',
u'\ub8d5': 'HangulSyllableRyolg',
u'\u1954': 'TaiLeLetterSa',
u'\uc5d7': 'HangulSyllableEd',
u'\u2656': 'WhiteChessRook',
u'\u02d9': 'DotAbove',
u'\u6358': 'CjkUnifiedIdeograph6358',
u'\u4fdb': 'CjkUnifiedIdeograph4fdb',
u'\ub462': 'HangulSyllableDubs',
u'\u90e5': 'CjkUnifiedIdeograph90e5',
u'\uf164': 'UnicodeCharacter0xf164',
u'\udde7': 'UnicodeCharacter0xdde7',
u'\u3e66': 'CjkUnifiedIdeograph3e66',
u'\u1ae9': 'UnicodeCharacter0x1ae9',
u'\u7b68': 'CjkUnifiedIdeograph7b68',
u'\u27eb': 'MathematicalRightDoubleAngleBracket',
u'\u64ed': 'CjkUnifiedIdeograph64ed',
u'\u8c72': 'CjkUnifiedIdeograph8c72',
u'\uc974': 'HangulSyllableJyul',
u'\ub5f7': 'HangulSyllableDdyegs',
u'\u1676': 'CanadianSyllabicsNngaa',
u'\uf2f9': 'UnicodeCharacter0xf2f9',
u'\u5378': 'CjkUnifiedIdeograph5378',
u'\u3ffb': 'CjkUnifiedIdeograph3ffb',
u'\u7cfd': 'CjkUnifiedIdeograph7cfd',
u'\u4601': 'CjkUnifiedIdeograph4601',
u'\ua184': 'YiSyllableNax',
u'\uee86': 'UnicodeCharacter0xee86',
u'\u2b88': 'UnicodeCharacter0x2b88',
u'\u970b': 'CjkUnifiedIdeograph970b',
u'\u688a': 'CjkUnifiedIdeograph688a',
u'\ud40d': 'HangulSyllablePwanj',
u'\u110f': 'HangulChoseongKhieukh',
u'\u5e11': 'CjkUnifiedIdeograph5e11',
u'\ub994': 'HangulSyllableRyin',
u'\uc696': 'HangulSyllableYogg',
u'\u0398': 'GreekCapitalLetterTheta',
u'\u409a': 'CjkUnifiedIdeograph409a',
u'\uac1d': 'HangulSyllableGaeg',
u'\ue91f': 'UnicodeCharacter0xe91f',
u'\u3621': 'CjkUnifiedIdeograph3621',
u'\u7323': 'CjkUnifiedIdeograph7323',
u'\u91a4': 'CjkUnifiedIdeograph91a4',
u'\udea6': 'UnicodeCharacter0xdea6',
u'\u1ba8': 'SundaneseVowelSignPamepet',
u'\u58aa': 'CjkUnifiedIdeograph58aa',
u'\u842d': 'CjkUnifiedIdeograph842d',
u'\u65ac': 'CjkUnifiedIdeograph65ac',
u'\uc12f': 'HangulSyllableSeos',
u'\u0e31': 'ThaiCharacterMaiHanakat',
u'\u4b33': 'CjkUnifiedIdeograph4b33',
u'\ub6b6': 'HangulSyllableDdup',
u'\uf3b8': 'UnicodeCharacter0xf3b8',
u'\u30ba': 'KatakanaLetterZu',
u'\u9c3d': 'CjkUnifiedIdeograph9c3d',
u'\u7dbc': 'CjkUnifiedIdeograph7dbc',
u'\ud93f': 'UnicodeCharacter0xd93f',
u'\ue641': 'UnicodeCharacter0xe641',
u'\u2343': 'AplFunctionalSymbolQuadLessthan',
u'\u6045': 'CjkUnifiedIdeograph6045',
u'\u8ec6': 'CjkUnifiedIdeograph8ec6',
u'\ucbc8': 'HangulSyllableJjyum',
u'\u08ca': 'UnicodeCharacter0x8ca',
u'\u55cc': 'CjkUnifiedIdeograph55cc',
u'\ub14f': 'HangulSyllableNyeolh',
u'\ufe51': 'SmallIdeographicComma',
u'\u3b53': 'CjkUnifiedIdeograph3b53',
u'\u7855': 'CjkUnifiedIdeograph7855',
u'\ua3d8': 'YiSyllableRurx',
u'\ue0da': 'UnicodeCharacter0xe0da',
u'\u2ddc': 'EthiopicSyllableGyee',
u'\u895f': 'CjkUnifiedIdeograph895f',
u'\u6ade': 'CjkUnifiedIdeograph6ade',
u'\ud661': 'HangulSyllableHwalt',
u'\u1363': 'EthiopicComma',
u'\u5065': 'CjkUnifiedIdeograph5065',
u'\ubbe8': 'HangulSyllableMyils',
u'\uf8ea': 'UnicodeCharacter0xf8ea',
u'\u05ec': 'UnicodeCharacter0x5ec',
u'\u42ee': 'CjkUnifiedIdeograph42ee',
u'\uae71': 'HangulSyllableGgaelg',
u'\ueb73': 'UnicodeCharacter0xeb73',
u'\u2875': 'BraillePatternDots13567',
u'\u7577': 'CjkUnifiedIdeograph7577',
u'\u93f8': 'CjkUnifiedIdeograph93f8',
u'\ud0fa': 'HangulSyllableTyagg',
u'\u1dfc': 'UnicodeCharacter0x1dfc',
u'\u5afe': 'CjkUnifiedIdeograph5afe',
u'\u6800': 'CjkUnifiedIdeograph6800',
u'\u4c83': 'CjkUnifiedIdeograph4c83',
u'\uab02': 'UnicodeCharacter0xab02',
u'\u8f85': 'CjkUnifiedIdeograph8f85',
u'\u2106': 'CadaUna',
u'\u0589': 'ArmenianFullStop',
u'\ufa0c': 'CjkCompatibilityIdeographfa0c',
u'\ude8f': 'UnicodeCharacter0xde8f',
u'\u7010': 'CjkUnifiedIdeograph7010',
u'\u5493': 'CjkUnifiedIdeograph5493',
u'\ub312': 'HangulSyllableDaebs',
u'\u9795': 'CjkUnifiedIdeograph9795',
u'\u0916': 'DevanagariLetterKha',
u'\ued99': 'UnicodeCharacter0xed99',
u'\u639d': 'CjkUnifiedIdeograph639d',
u'\uc21c': 'HangulSyllableSun',
u'\ua69f': 'UnicodeCharacter0xa69f',
u'\u5820': 'CjkUnifiedIdeograph5820',
u'\u3ca3': 'CjkUnifiedIdeograph3ca3',
u'\u9b22': 'CjkUnifiedIdeograph9b22',
u'\u1126': 'HangulChoseongPieupsioscieuc',
u'\uf5a9': 'UnicodeCharacter0xf5a9',
u'\u4bad': 'CjkUnifiedIdeograph4bad',
u'\uaa2c': 'ChamVowelSignEi',
u'\u8eaf': 'CjkUnifiedIdeograph8eaf',
u'\u2030': 'PerMilleSign',
u'\u04b3': 'CyrillicSmallLetterHaWithDescender',
u'\uf936': 'CjkCompatibilityIdeographf936',
u'\uddb9': 'UnicodeCharacter0xddb9',
u'\u7f3a': 'CjkUnifiedIdeograph7f3a',
u'\u53bd': 'CjkUnifiedIdeograph53bd',
u'\ub23c': 'HangulSyllableNwe',
u'\u96bf': 'CjkUnifiedIdeograph96bf',
u'\u0840': 'UnicodeCharacter0x840',
u'\uecc3': 'UnicodeCharacter0xecc3',
u'\u62c7': 'CjkUnifiedIdeograph62c7',
u'\uc146': 'HangulSyllableSelp',
u'\ua5c9': 'VaiSyllableFo',
u'\u474a': 'CjkUnifiedIdeograph474a',
u'\u3bcd': 'CjkUnifiedIdeograph3bcd',
u'\u9a4c': 'CjkUnifiedIdeograph9a4c',
u'\u1050': 'MyanmarLetterSha',
u'\uf4d3': 'UnicodeCharacter0xf4d3',
u'\u4ad7': 'CjkUnifiedIdeograph4ad7',
u'\ua956': 'UnicodeCharacter0xa956',
u'\u8dd9': 'CjkUnifiedIdeograph8dd9',
u'\u2f5a': 'KangxiRadicalSlice',
u'\u03dd': 'GreekSmallLetterDigamma',
u'\uf860': 'UnicodeCharacter0xf860',
u'\udce3': 'UnicodeCharacter0xdce3',
u'\u7e64': 'CjkUnifiedIdeograph7e64',
u'\u52e7': 'CjkUnifiedIdeograph52e7',
u'\ub166': 'HangulSyllableNyelm',
u'\u95e9': 'CjkUnifiedIdeograph95e9',
u'\u376a': 'CjkUnifiedIdeograph376a',
u'\uebed': 'UnicodeCharacter0xebed',
u'\u61f1': 'CjkUnifiedIdeograph61f1',
u'\uc070': 'HangulSyllableBbeuk',
u'\ua4f3': 'UnicodeCharacter0xa4f3',
u'\u4674': 'CjkUnifiedIdeograph4674',
u'\u3af7': 'CjkUnifiedIdeograph3af7',
u'\u9976': 'CjkUnifiedIdeograph9976',
u'\u1f7a': 'GreekSmallLetterUpsilonWithVaria',
u'\uf3fd': 'UnicodeCharacter0xf3fd',
u'\uc901': 'HangulSyllableJunj',
u'\ua880': 'SaurashtraSignAnusvara',
u'\u4f05': 'CjkUnifiedIdeograph4f05',
u'\u2e84': 'CjkRadicalSecondThree',
u'\u8207': 'CjkUnifiedIdeograph8207',
u'\u180b': 'MongolianFreeVariationSelectorOne',
u'\ue78a': 'UnicodeCharacter0xe78a',
u'\u7d8e': 'CjkUnifiedIdeograph7d8e',
u'\ud111': 'HangulSyllableTyat',
u'\ub090': 'HangulSyllableGgiss',
u'\u5715': 'CjkUnifiedIdeograph5715',
u'\u3694': 'CjkUnifiedIdeograph3694',
u'\ue01b': 'UnicodeCharacter0xe01b',
u'\ucf9a': 'HangulSyllableKwaelp',
u'\u661f': 'CjkUnifiedIdeograph661f',
u'\u459e': 'CjkUnifiedIdeograph459e',
u'\ub921': 'HangulSyllableRweg',
u'\u98a0': 'CjkUnifiedIdeograph98a0',
u'\u3f25': 'CjkUnifiedIdeograph3f25',
u'\u1ea4': 'LatinCapitalLetterAWithCircumflexAndAcute',
u'\uc82b': 'HangulSyllableJelh',
u'\ud7aa': 'UnicodeCharacter0xd7aa',
u'\u4e2f': 'CjkUnifiedIdeograph4e2f',
u'\u2dae': 'EthiopicSyllableCco',
u'\u8131': 'CjkUnifiedIdeograph8131',
u'\u0735': 'SyriacZqaphaDotted',
u'\ue6b4': 'UnicodeCharacter0xe6b4',
u'\u7cb8': 'CjkUnifiedIdeograph7cb8',
u'\ud03b': 'HangulSyllableKwid',
u'\ubfba': 'HangulSyllableBbyolm',
u'\u563f': 'CjkUnifiedIdeograph563f',
u'\u35be': 'CjkUnifiedIdeograph35be',
u'\uef45': 'UnicodeCharacter0xef45',
u'\ucec4': 'HangulSyllableKyak',
u'\u6549': 'CjkUnifiedIdeograph6549',
u'\u44c8': 'CjkUnifiedIdeograph44c8',
u'\ub84b': 'HangulSyllableRyelb',
u'\u87ca': 'CjkUnifiedIdeograph87ca',
u'\u3e4f': 'CjkUnifiedIdeograph3e4f',
u'\u1dce': 'CombiningOgonekAbove',
u'\uf755': 'UnicodeCharacter0xf755',
u'\ud6d4': 'HangulSyllableHum',
u'\u4d59': 'CjkUnifiedIdeograph4d59',
u'\u2cd8': 'CopticCapitalLetterOldCopticDja',
u'\u805b': 'CjkUnifiedIdeograph805b',
u'\u065f': 'UnicodeCharacter0x65f',
u'\ue5de': 'UnicodeCharacter0xe5de',
u'\u7be2': 'CjkUnifiedIdeograph7be2',
u'\udf65': 'UnicodeCharacter0xdf65',
u'\ubee4': 'HangulSyllableBbeoss',
u'\u5569': 'CjkUnifiedIdeograph5569',
u'\u34e8': 'CjkUnifiedIdeograph34e8',
u'\uee6f': 'UnicodeCharacter0xee6f',
u'\ucdee': 'HangulSyllableCwinh',
u'\u6473': 'CjkUnifiedIdeograph6473',
u'\u43f2': 'CjkUnifiedIdeograph43f2',
u'\ua775': 'LatinSmallLetterRum',
u'\u86f4': 'CjkUnifiedIdeograph86f4',
u'\u3d79': 'CjkUnifiedIdeograph3d79',
u'\u1cf8': 'UnicodeCharacter0x1cf8',
u'\uf67f': 'UnicodeCharacter0xf67f',
u'\ud5fe': 'HangulSyllableHep',
u'\ua006': 'YiSyllableIe',
u'\u8489': 'CjkUnifiedIdeograph8489',
u'\ue508': 'UnicodeCharacter0xe508',
u'\ud98b': 'UnicodeCharacter0xd98b',
u'\u260a': 'AscendingNode',
u'\u1a8d': 'UnicodeCharacter0x1a8d',
u'\u7b0c': 'CjkUnifiedIdeograph7b0c',
u'\u5f8f': 'CjkUnifiedIdeograph5f8f',
u'\u8816': 'CjkUnifiedIdeograph8816',
u'\ucd18': 'HangulSyllableCom',
u'\ua19b': 'YiSyllableHliex',
u'\u0e1a': 'ThaiCharacterBoBaimai',
u'\ue29d': 'UnicodeCharacter0xe29d',
u'\u431c': 'CjkUnifiedIdeograph431c',
u'\u279f': 'DashedTriangleheadedRightwardsArrow',
u'\u78a1': 'CjkUnifiedIdeograph78a1',
u'\u9026': 'CjkUnifiedIdeograph9026',
u'\ud528': 'HangulSyllablePyil',
u'\u89ab': 'CjkUnifiedIdeograph89ab',
u'\u162a': 'CanadianSyllabicsCarrierDle',
u'\ucaad': 'HangulSyllableJjyelt',
u'\u2b2c': 'BlackHorizontalEllipse',
u'\u0faf': 'TibetanSubjoinedLetterZa',
u'\u6c2e': 'CjkUnifiedIdeograph6c2e',
u'\u40b1': 'CjkUnifiedIdeograph40b1',
u'\ubd38': 'HangulSyllableBwaels',
u'\u91bb': 'CjkUnifiedIdeograph91bb',
u'\ufe3a': 'PresentationFormForVerticalRightTortoiseShellBracket',
u'\ud2bd': 'HangulSyllableTeunj',
u'\u333c': 'SquareBeeta',
u'\u17bf': 'KhmerVowelSignYa',
u'\u743e': 'CjkUnifiedIdeograph743e',
u'\u28c1': 'BraillePatternDots178',
u'\u6dc3': 'CjkUnifiedIdeograph6dc3',
u'\u8548': 'CjkUnifiedIdeograph8548',
u'\uc64a': 'HangulSyllableWalm',
u'\ubacd': 'HangulSyllableMwalg',
u'\u1b4c': 'UnicodeCharacter0x1b4c',
u'\uffcf': 'HalfwidthHangulLetterOe',
u'\u5c4e': 'CjkUnifiedIdeograph5c4e',
u'\u30d1': 'KatakanaLetterPa',
u'\u75d3': 'CjkUnifiedIdeograph75d3',
u'\uae5a': 'HangulSyllableGgalp',
u'\u82dd': 'CjkUnifiedIdeograph82dd',
u'\ue35c': 'UnicodeCharacter0xe35c',
u'\uc7df': 'HangulSyllableJyac',
u'\u245e': 'UnicodeCharacter0x245e',
u'\u18e1': 'UnicodeCharacter0x18e1',
u'\u7960': 'CjkUnifiedIdeograph7960',
u'\u5de3': 'CjkUnifiedIdeograph5de3',
u'\ub66a': 'HangulSyllableDdoenh',
u'\ucb6c': 'HangulSyllableJjweol',
u'\uafef': 'HangulSyllableGgweoh',
u'\u0c6e': 'TeluguDigitEight',
u'\ue0f1': 'UnicodeCharacter0xe0f1',
u'\u4170': 'CjkUnifiedIdeograph4170',
u'\u25f3': 'WhiteSquareWithUpperRightQuadrant',
u'\u66f5': 'CjkUnifiedIdeograph66f5',
u'\u9e7a': 'CjkUnifiedIdeograph9e7a',
u'\ud37c': 'HangulSyllablePeo',
u'\ub7ff': 'HangulSyllableReos',
u'\u147e': 'CanadianSyllabicsKwa',
u'\u2980': 'TripleVerticalBarDelimiter',
u'\u8d03': 'CjkUnifiedIdeograph8d03',
u'\u6a82': 'CjkUnifiedIdeograph6a82',
u'\uce05': 'HangulSyllableCyug',
u'\u0307': 'CombiningDotAbove',
u'\u4409': 'CjkUnifiedIdeograph4409',
u'\ubb8c': 'HangulSyllableMwin',
u'\ufc8e': 'ArabicLigatureNoonWithAlefMaksuraFinalForm',
u'\u3190': 'IdeographicAnnotationLinkingMark',
u'\u9513': 'CjkUnifiedIdeograph9513',
u'\u7292': 'CjkUnifiedIdeograph7292',
u'\ud615': 'HangulSyllableHyeong',
u'\ueb17': 'UnicodeCharacter0xeb17',
u'\u2c19': 'GlagoliticCapitalLetterOtu',
u'\u611b': 'CjkUnifiedIdeograph611b',
u'\u839c': 'CjkUnifiedIdeograph839c',
u'\uc49e': 'HangulSyllableSswegg',
u'\u19a0': 'NewTaiLueLetterHighHa',
u'\u5aa2': 'CjkUnifiedIdeograph5aa2',
u'\ube25': 'HangulSyllableBeut',
u'\uf327': 'UnicodeCharacter0xf327',
u'\u3429': 'CjkUnifiedIdeograph3429',
u'\u492b': 'CjkUnifiedIdeograph492b',
u'\uacae': 'HangulSyllableGyeonh',
u'\ue1b0': 'UnicodeCharacter0xe1b0',
u'\u22b2': 'NormalSubgroupOf',
u'\u8635': 'CjkUnifiedIdeograph8635',
u'\u67b4': 'CjkUnifiedIdeograph67b4',
u'\udb37': 'UnicodeCharacter0xdb37',
u'\u1c39': 'UnicodeCharacter0x1c39',
u'\u513b': 'CjkUnifiedIdeograph513b',
u'\ub4be': 'HangulSyllableDwip',
u'\uc9c0': 'HangulSyllableJi',
u'\u0ac2': 'GujaratiVowelSignUu',
u'\u4fc4': 'CjkUnifiedIdeograph4fc4',
u'\ua347': 'YiSyllableZhax',
u'\ue449': 'UnicodeCharacter0xe449',
u'\u394b': 'CjkUnifiedIdeograph394b',
u'\u7a4d': 'CjkUnifiedIdeograph7a4d',
u'\u9cce': 'CjkUnifiedIdeograph9cce',
u'\ud1d0': 'HangulSyllableTwass',
u'\u12d2': 'EthiopicSyllablePharyngealI',
u'\u57d4': 'CjkUnifiedIdeograph57d4',
u'\u8b57': 'CjkUnifiedIdeograph8b57',
u'\u68d6': 'CjkUnifiedIdeograph68d6',
u'\ucc59': 'HangulSyllableCaeng',
u'\u015b': 'LatinSmallLetterSWithAcute',
u'\u425d': 'CjkUnifiedIdeograph425d',
u'\ub9e0': 'HangulSyllableMak',
u'\ufae2': 'UnicodeCharacter0xfae2',
u'\u3fe4': 'CjkUnifiedIdeograph3fe4',
u'\u9367': 'CjkUnifiedIdeograph9367',
u'\u70e6': 'CjkUnifiedIdeograph70e6',
u'\ud469': 'HangulSyllablePyolt',
u'\ue96b': 'UnicodeCharacter0xe96b',
u'\u2a6d': 'CongruentWithDotAbove',
u'\u6f6f': 'CjkUnifiedIdeograph6f6f',
u'\u81f0': 'CjkUnifiedIdeograph81f0',
u'\uc2f2': 'HangulSyllableSij',
u'\u07f4': 'NkoHighToneApostrophe',
u'\u58f6': 'CjkUnifiedIdeograph58f6',
u'\ubc79': 'HangulSyllableByaeb',
u'\uf17b': 'UnicodeCharacter0xf17b',
u'\u327d': 'CircledKoreanCharacterJueui',
u'\u777f': 'CjkUnifiedIdeograph777f',
u'\u0b81': 'UnicodeCharacter0xb81',
u'\uec04': 'UnicodeCharacter0xec04',
u'\uc087': 'HangulSyllableBbyis',
u'\u6608': 'CjkUnifiedIdeograph6608',
u'\u5a8b': 'CjkUnifiedIdeograph5a8b',
u'\ua50a': 'VaiSyllableMgbee',
u'\u998d': 'CjkUnifiedIdeograph998d',
u'\u3f0e': 'CjkUnifiedIdeograph3f0e',
u'\u1391': 'EthiopicTonalMarkDeret',
u'\uf414': 'UnicodeCharacter0xf414',
u'\ua897': 'SaurashtraLetterCa',
u'\u4e18': 'CjkUnifiedIdeograph4e18',
u'\u229b': 'CircledAsteriskOperator',
u'\u8d1a': 'CjkUnifiedIdeograph8d1a',
u'\u071e': 'SyriacLetterYudhHe',
u'\ufba1': 'ArabicLetterRnoonFinalForm',
u'\u7da5': 'CjkUnifiedIdeograph7da5',
u'\udc24': 'UnicodeCharacter0xdc24',
u'\ub0a7': 'HangulSyllableNalh',
u'\u5628': 'CjkUnifiedIdeograph5628',
u'\u0aab': 'GujaratiLetterPha',
u'\u952a': 'CjkUnifiedIdeograph952a',
u'\uef2e': 'UnicodeCharacter0xef2e',
u'\uc3b1': 'HangulSyllableSsyeob',
u'\u6132': 'CjkUnifiedIdeograph6132',
u'\u45b5': 'CjkUnifiedIdeograph45b5',
u'\ua434': 'YiSyllableNjip',
u'\u98b7': 'CjkUnifiedIdeograph98b7',
u'\u3e38': 'CjkUnifiedIdeograph3e38',
u'\u12bb': 'EthiopicSyllableKxaa',
u'\uf73e': 'UnicodeCharacter0xf73e',
u'\uabc1': 'UnicodeCharacter0xabc1',
u'\u4942': 'CjkUnifiedIdeograph4942',
u'\u2dc5': 'EthiopicSyllableQye',
u'\u8c44': 'CjkUnifiedIdeograph8c44',
u'\u0648': 'ArabicLetterWaw',
u'\ufacb': 'CjkCompatibilityIdeographfacb',
u'\u7ccf': 'CjkUnifiedIdeograph7ccf',
u'\udf4e': 'UnicodeCharacter0xdf4e',
u'\ub3d1': 'HangulSyllableDolt',
u'\u5152': 'CjkUnifiedIdeograph5152',
u'\u35d5': 'CjkUnifiedIdeograph35d5',
u'\u9454': 'CjkUnifiedIdeograph9454',
u'\uee58': 'UnicodeCharacter0xee58',
u'\uc2db': 'HangulSyllableSyih',
u'\u605c': 'CjkUnifiedIdeograph605c',
u'\u44df': 'CjkUnifiedIdeograph44df',
u'\ua75e': 'LatinCapitalLetterVWithDiagonalStroke',
u'\u9be1': 'CjkUnifiedIdeograph9be1',
u'\u3962': 'CjkUnifiedIdeograph3962',
u'\u1de5': 'CombiningLatinSmallLetterLongS',
u'\uf668': 'UnicodeCharacter0xf668',
u'\uaaeb': 'UnicodeCharacter0xaaeb',
u'\u486c': 'CjkUnifiedIdeograph486c',
u'\u2cef': 'UnicodeCharacter0x2cef',
u'\u8f6e': 'CjkUnifiedIdeograph8f6e',
u'\u0172': 'LatinCapitalLetterUWithOgonek',
u'\ue5f5': 'UnicodeCharacter0xe5f5',
u'\u7ff9': 'CjkUnifiedIdeograph7ff9',
u'\ude78': 'UnicodeCharacter0xde78',
u'\ub2fb': 'HangulSyllableDac',
u'\u507c': 'CjkUnifiedIdeograph507c',
u'\u34ff': 'CjkUnifiedIdeograph34ff',
u'\u977e': 'CjkUnifiedIdeograph977e',
u'\u0e03': 'ThaiCharacterKhoKhuat',
u'\ue982': 'UnicodeCharacter0xe982',
u'\u6386': 'CjkUnifiedIdeograph6386',
u'\uc709': 'HangulSyllableWinj',
u'\ua688': 'CyrillicCapitalLetterDzze',
u'\u590d': 'CjkUnifiedIdeograph590d',
u'\u388c': 'CjkUnifiedIdeograph388c',
u'\u9c0f': 'CjkUnifiedIdeograph9c0f',
u'\u1613': 'CanadianSyllabicsCarrierYa',
u'\uf192': 'UnicodeCharacter0xf192',
u'\u6817': 'CjkUnifiedIdeograph6817',
u'\u4b96': 'CjkUnifiedIdeograph4b96',
u'\uaf19': 'HangulSyllableGgyelg',
u'\u8e98': 'CjkUnifiedIdeograph8e98',
u'\u211d': 'DoublestruckCapitalR',
u'\x9c': 'UnicodeCharacter0x9c',
u'\ufe23': 'CombiningDoubleTildeRightHalf',
u'\ud9a2': 'UnicodeCharacter0xd9a2',
u'\u7027': 'CjkUnifiedIdeograph7027',
u'\u53a6': 'CjkUnifiedIdeograph53a6',
u'\ub729': 'HangulSyllableDdeug',
u'\u96a8': 'CjkUnifiedIdeograph96a8',
u'\u092d': 'DevanagariLetterBha',
u'\ue8ac': 'UnicodeCharacter0xe8ac',
u'\u62b0': 'CjkUnifiedIdeograph62b0',
u'\uc633': 'HangulSyllableOlh',
u'\ua1b2': 'YiSyllableHlyx',
u'\u5837': 'CjkUnifiedIdeograph5837',
u'\u3bb6': 'CjkUnifiedIdeograph3bb6',
u'\u9f39': 'CjkUnifiedIdeograph9f39',
u'\u113d': 'HangulChoseongChitueumssangsios',
u'\uf0bc': 'UnicodeCharacter0xf0bc',
u'\u6b41': 'CjkUnifiedIdeograph6b41',
u'\u4ac0': 'CjkUnifiedIdeograph4ac0',
u'\uae43': 'HangulSyllableGis',
u'\u89c2': 'CjkUnifiedIdeograph89c2',
u'\u2047': 'DoubleQuestionMark',
u'\u03c6': 'GreekSmallLetterPhi',
u'\uf94d': 'CjkCompatibilityIdeographf94d',
u'\ud8cc': 'UnicodeCharacter0xd8cc',
u'\u7351': 'CjkUnifiedIdeograph7351',
u'\u52d0': 'CjkUnifiedIdeograph52d0',
u'\ub653': 'HangulSyllableDdwaelb',
u'\u91d2': 'CjkUnifiedIdeograph91d2',
u'\u0857': 'UnicodeCharacter0x857',
u'\uebd6': 'UnicodeCharacter0xebd6',
u'\u6dda': 'CjkUnifiedIdeograph6dda',
u'\uc15d': 'HangulSyllableSyeolg',
u'\ua0dc': 'YiSyllableFurx',
u'\u5b61': 'CjkUnifiedIdeograph5b61',
u'\u3ae0': 'CjkUnifiedIdeograph3ae0',
u'\u9e63': 'CjkUnifiedIdeograph9e63',
u'\u1067': 'MyanmarVowelSignWesternPwoKarenEu',
u'\uf3e6': 'UnicodeCharacter0xf3e6',
u'\u6a6b': 'CjkUnifiedIdeograph6a6b',
u'\u75ea': 'CjkUnifiedIdeograph75ea',
u'\ua96d': 'UnicodeCharacter0xa96d',
u'\u88ec': 'CjkUnifiedIdeograph88ec',
u'\u2371': 'AplFunctionalSymbolDownCaretTilde',
u'\u02f0': 'ModifierLetterLowUpArrowhead',
u'\uf877': 'UnicodeCharacter0xf877',
u'\udbf6': 'UnicodeCharacter0xdbf6',
u'\u727b': 'CjkUnifiedIdeograph727b',
u'\u5dfa': 'CjkUnifiedIdeograph5dfa',
u'\ub17d': 'HangulSyllableNonj',
u'\u90fc': 'CjkUnifiedIdeograph90fc',
u'\u8a81': 'CjkUnifiedIdeograph8a81',
u'\ueb00': 'UnicodeCharacter0xeb00',
u'\ucf83': 'HangulSyllableKwas',
u'\u2802': 'BraillePatternDots2',
u'\u0c85': 'KannadaLetterA',
u'\u6d04': 'CjkUnifiedIdeograph6d04',
u'\u4187': 'CjkUnifiedIdeograph4187',
u'\ube0e': 'HangulSyllableBeugg',
u'\u9291': 'CjkUnifiedIdeograph9291',
u'\uf310': 'UnicodeCharacter0xf310',
u'\ud793': 'HangulSyllableHilb',
u'\u3012': 'PostalMark',
u'\u1495': 'CanadianSyllabicsWestcreeCwi',
u'\u7514': 'CjkUnifiedIdeograph7514',
u'\u2997': 'LeftBlackTortoiseShellBracket',
u'\u6e99': 'CjkUnifiedIdeograph6e99',
u'\u861e': 'CjkUnifiedIdeograph861e',
u'\udb20': 'UnicodeCharacter0xdb20',
u'\ubfa3': 'HangulSyllableBboelh',
u'\u1822': 'MongolianLetterI',
u'\ufca5': 'ArabicLigatureTehWithHehInitialForm',
u'\u5d24': 'CjkUnifiedIdeograph5d24',
u'\u31a7': 'BopomofoLetterOnn',
u'\u76a9': 'CjkUnifiedIdeograph76a9',
u'\ua330': 'YiSyllableSsiep',
u'\u87b3': 'CjkUnifiedIdeograph87b3',
u'\ue032': 'UnicodeCharacter0xe032',
u'\uc4b5': 'HangulSyllableSswet',
u'\u2534': 'BoxDrawingsLightUpAndHorizontal',
u'\u19b7': 'NewTaiLueVowelSignO',
u'\u7a36': 'CjkUnifiedIdeograph7a36',
u'\u5eb9': 'CjkUnifiedIdeograph5eb9',
u'\u8b40': 'CjkUnifiedIdeograph8b40',
u'\uc842': 'HangulSyllableJyeolm',
u'\uacc5': 'HangulSyllableGyeg',
u'\u0d44': 'MalayalamVowelSignVocalicRr',
u'\ue1c7': 'UnicodeCharacter0xe1c7',
u'\u4246': 'CjkUnifiedIdeograph4246',
u'\u26c9': 'UnicodeCharacter0x26c9',
u'\u7bcb': 'CjkUnifiedIdeograph7bcb',
u'\u9350': 'CjkUnifiedIdeograph9350',
u'\ud052': 'HangulSyllableKyugg',
u'\ub4d5': 'HangulSyllableDyung',
u'\u1554': 'CanadianSyllabicsFaai',
u'\uc9d7': 'HangulSyllableJic',
u'\u2a56': 'TwoIntersectingLogicalOr',
u'\u0ed9': 'LaoDigitNine',
u'\u6f58': 'CjkUnifiedIdeograph6f58',
u'\u43db': 'CjkUnifiedIdeograph43db',
u'\ub862': 'HangulSyllableRonh',
u'\u9ce5': 'CjkUnifiedIdeograph9ce5',
u'\ufd64': 'ArabicLigatureSadWithHahWithHahFinalForm',
u'\ud1e7': 'HangulSyllableTwaelh',
u'\u3266': 'CircledHangulSios',
u'\u16e9': 'RunicLetterQ',
u'\u7768': 'CjkUnifiedIdeograph7768',
u'\u2beb': 'UnicodeCharacter0x2beb',
u'\u68ed': 'CjkUnifiedIdeograph68ed',
u'\u8072': 'CjkUnifiedIdeograph8072',
u'\uc574': 'HangulSyllableAess',
u'\ub9f7': 'HangulSyllableMaes',
u'\u1a76': 'UnicodeCharacter0x1a76',
u'\ufef9': 'ArabicLigatureLamWithAlefWithHamzaBelowIsolatedForm',
u'\u5f78': 'CjkUnifiedIdeograph5f78',
u'\u33fb': 'IdeographicTelegraphSymbolForDayTwentyeight',
u'\u70fd': 'CjkUnifiedIdeograph70fd',
u'\u4a01': 'CjkUnifiedIdeograph4a01',
u'\uad84': 'HangulSyllableGuk',
u'\ue286': 'UnicodeCharacter0xe286',
u'\u2788': 'DingbatCircledSansserifDigitNine',
u'\u9b0b': 'CjkUnifiedIdeograph9b0b',
u'\u648a': 'CjkUnifiedIdeograph648a',
u'\ud80d': 'UnicodeCharacter0xd80d',
u'\u1d0f': 'LatinLetterSmallCapitalO',
u'\u5211': 'CjkUnifiedIdeograph5211',
u'\ub594': 'HangulSyllableDdyaem',
u'\uca96': 'HangulSyllableJjyeobs',
u'\u0f98': 'UnicodeCharacter0xf98',
u'\u4c9a': 'CjkUnifiedIdeograph4c9a',
u'\ua01d': 'YiSyllableBiep',
u'\ue51f': 'UnicodeCharacter0xe51f',
u'\u3a21': 'CjkUnifiedIdeograph3a21',
u'\u7f23': 'CjkUnifiedIdeograph7f23',
u'\u9da4': 'CjkUnifiedIdeograph9da4',
u'\ud2a6': 'HangulSyllableTyulm',
u'\u17a8': 'KhmerIndependentVowelQuk',
u'\u54aa': 'CjkUnifiedIdeograph54aa',
u'\u882d': 'CjkUnifiedIdeograph882d',
u'\u69ac': 'CjkUnifiedIdeograph69ac',
u'\ucd2f': 'HangulSyllableCwalb',
u'\u0231': 'LatinSmallLetterOWithDotAboveAndMacron',
u'\u4733': 'CjkUnifiedIdeograph4733',
u'\ubab6': 'HangulSyllableMolp',
u'\uffb8': 'HalfwidthHangulLetterCieuc',
u'\u3cba': 'CjkUnifiedIdeograph3cba',
u'\u903d': 'CjkUnifiedIdeograph903d',
u'\u71bc': 'CjkUnifiedIdeograph71bc',
u'\ud53f': 'HangulSyllablePigs',
u'\uea41': 'UnicodeCharacter0xea41',
u'\u2f43': 'KangxiRadicalDipper',
u'\u6c45': 'CjkUnifiedIdeograph6c45',
u'\u82c6': 'CjkUnifiedIdeograph82c6',
u'\uc7c8': 'HangulSyllableJya',
u'\u04ca': 'CyrillicSmallLetterEnWithTail',
u'\u59cc': 'CjkUnifiedIdeograph59cc',
u'\ubd4f': 'HangulSyllableBoed',
u'\uf251': 'UnicodeCharacter0xf251',
u'\u3753': 'CjkUnifiedIdeograph3753',
u'\u7455': 'CjkUnifiedIdeograph7455',
u'\uafd8': 'HangulSyllableGgweon',
u'\uecda': 'UnicodeCharacter0xecda',
u'\u21dc': 'LeftwardsSquiggleArrow',
u'\u855f': 'CjkUnifiedIdeograph855f',
u'\u66de': 'CjkUnifiedIdeograph66de',
u'\uda61': 'UnicodeCharacter0xda61',
u'\u1f63': 'GreekSmallLetterOmegaWithDasiaAndVaria',
u'\u5c65': 'CjkUnifiedIdeograph5c65',
u'\ub7e8': 'HangulSyllableRyaek',
u'\uf4ea': 'UnicodeCharacter0xf4ea',
u'\u09ec': 'BengaliDigitSix',
u'\u4eee': 'CjkUnifiedIdeograph4eee',
u'\ua271': 'YiSyllableHat',
u'\ue773': 'UnicodeCharacter0xe773',
u'\u2475': 'ParenthesizedDigitTwo',
u'\u7977': 'CjkUnifiedIdeograph7977',
u'\u9ff8': 'UnicodeCharacter0x9ff8',
u'\udcfa': 'UnicodeCharacter0xdcfa',
u'\u11fc': 'UnicodeCharacter0x11fc',
u'\u56fe': 'CjkUnifiedIdeograph56fe',
u'\u6c00': 'CjkUnifiedIdeograph6c00',
u'\u4883': 'CjkUnifiedIdeograph4883',
u'\uaf02': 'HangulSyllableGgyeolp',
u'\u8b85': 'CjkUnifiedIdeograph8b85',
u'\u2506': 'BoxDrawingsLightTripleDashVertical',
u'\u0189': 'LatinCapitalLetterAfricanD',
u'\ufe0c': 'VariationSelector13',
u'\uda8f': 'UnicodeCharacter0xda8f',
u'\u7410': 'CjkUnifiedIdeograph7410',
u'\u5093': 'CjkUnifiedIdeograph5093',
u'\ub712': 'HangulSyllableDdyunh',
u'\u9395': 'CjkUnifiedIdeograph9395',
u'\u0d16': 'MalayalamLetterKha',
u'\ue999': 'UnicodeCharacter0xe999',
u'\u679d': 'CjkUnifiedIdeograph679d',
u'\uc61c': 'HangulSyllableYess',
u'\ua29f': 'YiSyllableZex',
u'\u5c20': 'CjkUnifiedIdeograph5c20',
u'\u38a3': 'CjkUnifiedIdeograph38a3',
u'\u9f22': 'CjkUnifiedIdeograph9f22',
u'\u1526': 'CanadianSyllabicsYe',
u'\uf1a9': 'UnicodeCharacter0xf1a9',
u'\u4fad': 'CjkUnifiedIdeograph4fad',
u'\uae2c': 'HangulSyllableGyik',
u'\u8aaf': 'CjkUnifiedIdeograph8aaf',
u'\u2430': 'UnicodeCharacter0x2430',
u'\xb3': 'SuperscriptThree',
u'\ufd36': 'ArabicLigatureSeenWithKhahMedialForm',
u'\ud9b9': 'UnicodeCharacter0xd9b9',
u'\u7b3a': 'CjkUnifiedIdeograph7b3a',
u'\u57bd': 'CjkUnifiedIdeograph57bd',
u'\ub63c': 'HangulSyllableDdwam',
u'\u92bf': 'CjkUnifiedIdeograph92bf',
u'\u0c40': 'TeluguVowelSignIi',
u'\ue8c3': 'UnicodeCharacter0xe8c3',
u'\u66c7': 'CjkUnifiedIdeograph66c7',
u'\uc546': 'HangulSyllableAgg',
u'\ua1c9': 'YiSyllableLo',
u'\u434a': 'CjkUnifiedIdeograph434a',
u'\u3fcd': 'CjkUnifiedIdeograph3fcd',
u'\u9e4c': 'CjkUnifiedIdeograph9e4c',
u'\u1450': 'CanadianSyllabicsTo',
u'\uf0d3': 'UnicodeCharacter0xf0d3',
u'\u4ed7': 'CjkUnifiedIdeograph4ed7',
u'\uad56': 'HangulSyllableGyonh',
u'\u89d9': 'CjkUnifiedIdeograph89d9',
u'\u2b5a': 'UnicodeCharacter0x2b5a',
u'\u07dd': 'NkoLetterFa',
u'\ufc60': 'ArabicLigatureShaddaWithFathaIsolatedForm',
u'\ud8e3': 'UnicodeCharacter0xd8e3',
u'\u7a64': 'CjkUnifiedIdeograph7a64',
u'\u56e7': 'CjkUnifiedIdeograph56e7',
u'\ub566': 'HangulSyllableDdaep',
u'\u91e9': 'CjkUnifiedIdeograph91e9',
u'\u336a': 'IdeographicTelegraphSymbolForHourEighteen',
u'\uefed': 'UnicodeCharacter0xefed',
u'\u65f1': 'CjkUnifiedIdeograph65f1',
u'\uc470': 'HangulSyllableSsuls',
u'\ua0f3': 'YiSyllableVep',
u'\u4274': 'CjkUnifiedIdeograph4274',
u'\u3ef7': 'CjkUnifiedIdeograph3ef7',
u'\u9d76': 'CjkUnifiedIdeograph9d76',
u'\u1b7a': 'BalineseMusicalSymbolLefthandClosedPlak',
u'\uf7fd': 'UnicodeCharacter0xf7fd',
u'\ucd01': 'HangulSyllableCyeng',
u'\uac80': 'HangulSyllableGeom',
u'\u4b05': 'CjkUnifiedIdeograph4b05',
u'\u2a84': 'GreaterthanOrSlantedEqualToWithDotAboveLeft',
u'\u8607': 'CjkUnifiedIdeograph8607',
u'\u1c0b': 'LepchaLetterTha',
u'\ue38a': 'UnicodeCharacter0xe38a',
u'\u798e': 'CjkUnifiedIdeograph798e',
u'\ud511': 'HangulSyllablePeult',
u'\ub490': 'HangulSyllableDwel',
u'\u5315': 'CjkUnifiedIdeograph5315',
u'\u3294': 'CircledIdeographName',
u'\ue41b': 'UnicodeCharacter0xe41b',
u'\ucb9a': 'HangulSyllableJjwep',
u'\u621f': 'CjkUnifiedIdeograph621f',
u'\u419e': 'CjkUnifiedIdeograph419e',
u'\ubd21': 'HangulSyllableBwab',
u'\u9ca0': 'CjkUnifiedIdeograph9ca0',
u'\u3b25': 'CjkUnifiedIdeograph3b25',
u'\u1aa4': 'UnicodeCharacter0x1aa4',
u'\ucc2b': 'HangulSyllableCags',
u'\ud3aa': 'HangulSyllablePebs',
u'\u4a2f': 'CjkUnifiedIdeograph4a2f',
u'\u29ae': 'MeasuredAngleWithOpenArmEndingInArrowPointingRightAndDown',
u'\u8531': 'CjkUnifiedIdeograph8531',
u'\u0335': 'CombiningShortStrokeOverlay',
u'\ue2b4': 'UnicodeCharacter0xe2b4',
u'\u78b8': 'CjkUnifiedIdeograph78b8',
u'\ud43b': 'HangulSyllablePwaec',
u'\ubbba': 'HangulSyllableMyuj',
u'\u523f': 'CjkUnifiedIdeograph523f',
u'\u31be': 'UnicodeCharacter0x31be',
u'\ueb45': 'UnicodeCharacter0xeb45',
u'\ucac4': 'HangulSyllableJjol',
u'\u6149': 'CjkUnifiedIdeograph6149',
u'\u40c8': 'CjkUnifiedIdeograph40c8',
u'\ubc4b': 'HangulSyllableBaeh',
u'\u83ca': 'CjkUnifiedIdeograph83ca',
u'\u3a4f': 'CjkUnifiedIdeograph3a4f',
u'\u19ce': 'UnicodeCharacter0x19ce',
u'\uf355': 'UnicodeCharacter0xf355',
u'\ud2d4': 'HangulSyllableTyi',
u'\u4959': 'CjkUnifiedIdeograph4959',
u'\u28d8': 'BraillePatternDots4578',
u'\u845b': 'CjkUnifiedIdeograph845b',
u'\u025f': 'LatinSmallLetterDotlessJWithStroke',
u'\ue1de': 'UnicodeCharacter0xe1de',
u'\u7fe2': 'CjkUnifiedIdeograph7fe2',
u'\udb65': 'UnicodeCharacter0xdb65',
u'\ubae4': 'HangulSyllableMwaen',
u'\u5169': 'CjkUnifiedIdeograph5169',
u'\u30e8': 'KatakanaLetterYo',
u'\uea6f': 'UnicodeCharacter0xea6f',
u'\uc9ee': 'HangulSyllableJjabs',
u'\u6073': 'CjkUnifiedIdeograph6073',
u'\u47f2': 'CjkUnifiedIdeograph47f2',
u'\ua375': 'YiSyllableChur',
u'\u82f4': 'CjkUnifiedIdeograph82f4',
u'\u3979': 'CjkUnifiedIdeograph3979',
u'\u18f8': 'UnicodeCharacter0x18f8',
u'\uf27f': 'UnicodeCharacter0xf27f',
u'\ud1fe': 'HangulSyllableToelm',
u'\ua406': 'YiSyllableQuo',
u'\u8089': 'CjkUnifiedIdeograph8089',
u'\ue108': 'UnicodeCharacter0xe108',
u'\udd8b': 'UnicodeCharacter0xdd8b',
u'\u220a': 'SmallElementOf',
u'\u1e8d': 'LatinSmallLetterXWithDiaeresis',
u'\u7f0c': 'CjkUnifiedIdeograph7f0c',
u'\u5b8f': 'CjkUnifiedIdeograph5b8f',
u'\u8c16': 'CjkUnifiedIdeograph8c16',
u'\uc918': 'HangulSyllableJweo',
u'\ua59b': 'VaiSyllablePu',
u'\u0a1a': 'GurmukhiLetterCa',
u'\ue69d': 'UnicodeCharacter0xe69d',
u'\u471c': 'CjkUnifiedIdeograph471c',
u'\u239f': 'RightParenthesisExtension',
u'\u7ca1': 'CjkUnifiedIdeograph7ca1',
u'\u9426': 'CjkUnifiedIdeograph9426',
u'\ud128': 'HangulSyllableTyaess',
u'\u8dab': 'CjkUnifiedIdeograph8dab',
u'\u122a': 'EthiopicSyllableRi',
u'\ucead': 'HangulSyllableKyag',
u'\u2f2c': 'KangxiRadicalSprout',
u'\u0baf': 'TamilLetterYa',
u'\u682e': 'CjkUnifiedIdeograph682e',
u'\u44b1': 'CjkUnifiedIdeograph44b1',
u'\ub938': 'HangulSyllableRwek',
u'\u95bb': 'CjkUnifiedIdeograph95bb',
u'\ufa3a': 'CjkCompatibilityIdeographfa3a',
u'\ud6bd': 'HangulSyllableHyong',
u'\u373c': 'CjkUnifiedIdeograph373c',
u'\u13bf': 'CherokeeLetterHna',
u'\u703e': 'CjkUnifiedIdeograph703e',
u'\u2cc1': 'CopticSmallLetterSampi',
u'\u69c3': 'CjkUnifiedIdeograph69c3',
u'\u8148': 'CjkUnifiedIdeograph8148',
u'\uc24a': 'HangulSyllableSweoj',
u'\ubecd': 'HangulSyllableBbyaet',
u'\u1f4c': 'GreekCapitalLetterOmicronWithPsiliAndOxia',
u'\ufbcf': 'UnicodeCharacter0xfbcf',
u'\u584e': 'CjkUnifiedIdeograph584e',
u'\u34d1': 'CjkUnifiedIdeograph34d1',
u'\u71d3': 'CjkUnifiedIdeograph71d3',
u'\uaa5a': 'UnicodeCharacter0xaa5a',
u'\u86dd': 'CjkUnifiedIdeograph86dd',
u'\ue75c': 'UnicodeCharacter0xe75c',
u'\uc3df': 'HangulSyllableSsod',
u'\u205e': 'VerticalFourDots',
u'\u1ce1': 'UnicodeCharacter0x1ce1',
u'\u7d60': 'CjkUnifiedIdeograph7d60',
u'\u59e3': 'CjkUnifiedIdeograph59e3',
u'\ub26a': 'HangulSyllableNwibs',
u'\ucf6c': 'HangulSyllableKok',
u'\uabef': 'UnicodeCharacter0xabef',
u'\u086e': 'UnicodeCharacter0x86e',
u'\ue4f1': 'UnicodeCharacter0xe4f1',
u'\u4570': 'CjkUnifiedIdeograph4570',
u'\u21f3': 'UpDownWhiteArrow',
u'\u62f5': 'CjkUnifiedIdeograph62f5',
u'\u9a7a': 'CjkUnifiedIdeograph9a7a',
u'\ud77c': 'HangulSyllableHyim',
u'\ub3ff': 'HangulSyllableDwaegs',
u'\u107e': 'MyanmarLetterShanFa',
u'\u2d80': 'EthiopicSyllableLoa',
u'\u8903': 'CjkUnifiedIdeograph8903',
u'\u6e82': 'CjkUnifiedIdeograph6e82',
u'\uca05': 'HangulSyllableJjaelt',
u'\u0707': 'SyriacColonSkewedRight',
u'\u4009': 'CjkUnifiedIdeograph4009',
u'\ubf8c': 'HangulSyllableBbwaess',
u'\uf88e': 'UnicodeCharacter0xf88e',
u'\u3590': 'CjkUnifiedIdeograph3590',
u'\u9113': 'CjkUnifiedIdeograph9113',
u'\u7692': 'CjkUnifiedIdeograph7692',
u'\ud215': 'HangulSyllableTyonj',
u'\uef17': 'UnicodeCharacter0xef17',
u'\u2819': 'BraillePatternDots145',
u'\u651b': 'CjkUnifiedIdeograph651b',
u'\u879c': 'CjkUnifiedIdeograph879c',
u'\uc09e': 'HangulSyllableBbilp',
u'\u1da0': 'ModifierLetterSmallF',
u'\u5ea2': 'CjkUnifiedIdeograph5ea2',
u'\uba25': 'HangulSyllableMyaelg',
u'\uf727': 'UnicodeCharacter0xf727',
u'\u3029': 'HangzhouNumeralNine',
u'\u4d2b': 'CjkUnifiedIdeograph4d2b',
u'\ua8ae': 'SaurashtraLetterVa',
u'\ue5b0': 'UnicodeCharacter0xe5b0',
u'\u26b2': 'Neuter',
u'\u8235': 'CjkUnifiedIdeograph8235',
u'\u63b4': 'CjkUnifiedIdeograph63b4',
u'\udf37': 'UnicodeCharacter0xdf37',
u'\u1839': 'MongolianLetterFa',
u'\u553b': 'CjkUnifiedIdeograph553b',
u'\ub0be': 'HangulSyllableNaelm',
u'\ucdc0': 'HangulSyllableCweom',
u'\u0ec2': 'LaoVowelSignO',
u'\u4bc4': 'CjkUnifiedIdeograph4bc4',
u'\ua747': 'LatinSmallLetterBrokenL',
u'\ue049': 'UnicodeCharacter0xe049',
u'\u3d4b': 'CjkUnifiedIdeograph3d4b',
u'\u7e4d': 'CjkUnifiedIdeograph7e4d',
u'\u98ce': 'CjkUnifiedIdeograph98ce',
u'\ud5d0': 'HangulSyllableHeol',
u'\u16d2': 'RunicLetterBerkananBeorcBjarkanB',
u'\u53d4': 'CjkUnifiedIdeograph53d4',
u'\u8f57': 'CjkUnifiedIdeograph8f57',
u'\u6cd6': 'CjkUnifiedIdeograph6cd6',
u'\uc859': 'HangulSyllableJyenj',
u'\u055b': 'ArmenianEmphasisMark',
u'\u465d': 'CjkUnifiedIdeograph465d',
u'\ubde0': 'HangulSyllableBwils',
u'\ufee2': 'ArabicLetterMeemFinalForm',
u'\u3be4': 'CjkUnifiedIdeograph3be4',
u'\u9767': 'CjkUnifiedIdeograph9767',
u'\u74e6': 'CjkUnifiedIdeograph74e6',
u'\ud069': 'HangulSyllableKyut',
u'\ued6b': 'UnicodeCharacter0xed6b',
u'\u2e6d': 'UnicodeCharacter0x2e6d',
u'\u6b6f': 'CjkUnifiedIdeograph6b6f',
u'\u85f0': 'CjkUnifiedIdeograph85f0',
u'\uc6f2': 'HangulSyllableWelm',
u'\u03f4': 'GreekCapitalThetaSymbol',
u'\u5cf6': 'CjkUnifiedIdeograph5cf6',
u'\ub879': 'HangulSyllableRwag',
u'\uf57b': 'UnicodeCharacter0xf57b',
u'\u367d': 'CjkUnifiedIdeograph367d',
u'\u737f': 'CjkUnifiedIdeograph737f',
u'\u0f81': 'TibetanVowelSignReversedIi',
u'\ue804': 'UnicodeCharacter0xe804',
u'\uc487': 'HangulSyllableSsweod',
u'\u6208': 'CjkUnifiedIdeograph6208',
u'\u5e8b': 'CjkUnifiedIdeograph5e8b',
u'\ua10a': 'YiSyllableDap',
u'\u9d8d': 'CjkUnifiedIdeograph9d8d',
u'\u3b0e': 'CjkUnifiedIdeograph3b0e',
u'\u1791': 'KhmerLetterTo',
u'\uf014': 'UnicodeCharacter0xf014',
u'\uac97': 'HangulSyllableGelb',
u'\u4a18': 'CjkUnifiedIdeograph4a18',
u'\u269b': 'AtomSymbol',
u'\u891a': 'CjkUnifiedIdeograph891a',
u'\u031e': 'CombiningDownTackBelow',
u'\uffa1': 'HalfwidthHangulLetterKiyeok',
u'\u79a5': 'CjkUnifiedIdeograph79a5',
u'\ud824': 'UnicodeCharacter0xd824',
u'\ub4a7': 'HangulSyllableDwigs',
u'\u5228': 'CjkUnifiedIdeograph5228',
u'\u0eab': 'LaoLetterHoSung',
u'\u912a': 'CjkUnifiedIdeograph912a',
u'\ueb2e': 'UnicodeCharacter0xeb2e',
u'\uc7b1': 'HangulSyllableJaenj',
u'\u6532': 'CjkUnifiedIdeograph6532',
u'\u41b5': 'CjkUnifiedIdeograph41b5',
u'\ua034': 'YiSyllableBy',
u'\u9cb7': 'CjkUnifiedIdeograph9cb7',
u'\u3a38': 'CjkUnifiedIdeograph3a38',
u'\u16bb': 'RunicLetterHaeglH',
u'\uf33e': 'UnicodeCharacter0xf33e',
u'\uafc1': 'HangulSyllableGgulg',
u'\u4d42': 'CjkUnifiedIdeograph4d42',
u'\u29c5': 'SquaredFallingDiagonalSlash',
u'\u8844': 'CjkUnifiedIdeograph8844',
u'\u0248': 'LatinCapitalLetterJWithStroke',
u'\ufecb': 'ArabicLetterAinInitialForm',
u'\u78cf': 'CjkUnifiedIdeograph78cf',
u'\udb4e': 'UnicodeCharacter0xdb4e',
u'\ub7d1': 'HangulSyllableRyaeg',
u'\u5552': 'CjkUnifiedIdeograph5552',
u'\u31d5': 'CjkStrokeHz',
u'\u9054': 'CjkUnifiedIdeograph9054',
u'\uea58': 'UnicodeCharacter0xea58',
u'\uc6db': 'HangulSyllableWeolh',
u'\u645c': 'CjkUnifiedIdeograph645c',
u'\u40df': 'CjkUnifiedIdeograph40df',
u'\ua35e': 'YiSyllableZhyp',
u'\u9fe1': 'UnicodeCharacter0x9fe1',
u'\u3d62': 'CjkUnifiedIdeograph3d62',
u'\u19e5': 'KhmerSymbolPramKoet',
u'\uf268': 'UnicodeCharacter0xf268',
u'\uaeeb': 'HangulSyllableGges',
u'\u4c6c': 'CjkUnifiedIdeograph4c6c',
u'\u28ef': 'BraillePatternDots1234678',
u'\u8b6e': 'CjkUnifiedIdeograph8b6e',
u'\u0572': 'ArmenianSmallLetterGhad',
u'\ue1f5': 'UnicodeCharacter0xe1f5',
u'\u7bf9': 'CjkUnifiedIdeograph7bf9',
u'\uda78': 'UnicodeCharacter0xda78',
u'\ub6fb': 'HangulSyllableDdwilb',
u'\u547c': 'CjkUnifiedIdeograph547c',
u'\u30ff': 'KatakanaDigraphKoto',
u'\u937e': 'CjkUnifiedIdeograph937e',
u'\u0a03': 'GurmukhiSignVisarga',
u'\ued82': 'UnicodeCharacter0xed82',
u'\u6786': 'CjkUnifiedIdeograph6786',
u'\uc309': 'HangulSyllableSsab',
u'\ua288': 'YiSyllableWo',
u'\u5d0d': 'CjkUnifiedIdeograph5d0d',
u'\u3c8c': 'CjkUnifiedIdeograph3c8c',
u'\u980f': 'CjkUnifiedIdeograph980f',
u'\u1213': 'EthiopicSyllableHhaa',
u'\uf592': 'UnicodeCharacter0xf592',
u'\u6c17': 'CjkUnifiedIdeograph6c17',
u'\u4f96': 'CjkUnifiedIdeograph4f96',
u'\uab19': 'UnicodeCharacter0xab19',
u'\u8a98': 'CjkUnifiedIdeograph8a98',
u'\u251d': 'BoxDrawingsVerticalLightAndRightHeavy',
u'\u049c': 'CyrillicCapitalLetterKaWithVerticalStroke',
u'\ufa23': 'CjkCompatibilityIdeographfa23',
u'\udda2': 'UnicodeCharacter0xdda2',
u'\u7427': 'CjkUnifiedIdeograph7427',
u'\u57a6': 'CjkUnifiedIdeograph57a6',
u'\ub329': 'HangulSyllableDyalt',
u'\u92a8': 'CjkUnifiedIdeograph92a8',
u'\u0d2d': 'MalayalamLetterBha',
u'\uecac': 'UnicodeCharacter0xecac',
u'\u66b0': 'CjkUnifiedIdeograph66b0',
u'\uc233': 'HangulSyllableSuh',
u'\ua5b2': 'VaiSyllableNju',
u'\u5c37': 'CjkUnifiedIdeograph5c37',
u'\u3fb6': 'CjkUnifiedIdeograph3fb6',
u'\u9b39': 'CjkUnifiedIdeograph9b39',
u'\u153d': 'CanadianSyllabicsNaskapiYwaa',
u'\uf4bc': 'UnicodeCharacter0xf4bc',
u'\u6f41': 'CjkUnifiedIdeograph6f41',
u'\u4ec0': 'CjkUnifiedIdeograph4ec0',
u'\uaa43': 'ChamConsonantSignFinalNg',
u'\u8dc2': 'CjkUnifiedIdeograph8dc2',
u'\u2447': 'OcrAmountOfCheck',
u'\u07c6': 'NkoDigitSix',
u'\ufd4d': 'UnicodeCharacter0xfd4d',
u'\udccc': 'UnicodeCharacter0xdccc',
u'\u7751': 'CjkUnifiedIdeograph7751',
u'\u56d0': 'CjkUnifiedIdeograph56d0',
u'\ub253': 'HangulSyllableNwec',
u'\u95d2': 'CjkUnifiedIdeograph95d2',
u'\u0c57': 'UnicodeCharacter0xc57',
u'\uefd6': 'UnicodeCharacter0xefd6',
u'\u69da': 'CjkUnifiedIdeograph69da',
u'\uc55d': 'HangulSyllableAt',
u'\ua4dc': 'UnicodeCharacter0xa4dc',
u'\u5f61': 'CjkUnifiedIdeograph5f61',
u'\u3ee0': 'CjkUnifiedIdeograph3ee0',
u'\u9a63': 'CjkUnifiedIdeograph9a63',
u'\u1467': 'CanadianSyllabicsTte',
u'\uf7e6': 'UnicodeCharacter0xf7e6',
u'\u6e6b': 'CjkUnifiedIdeograph6e6b',
u'\u71ea': 'CjkUnifiedIdeograph71ea',
u'\uad6d': 'HangulSyllableGug',
u'\u8cec': 'CjkUnifiedIdeograph8cec',
u'\u2771': 'HeavyRightpointingAngleBracketOrnament',
u'\u06f0': 'ExtendedArabicindicDigitZero',
u'\ufc77': 'ArabicLigatureThehWithZainFinalForm',
u'\udff6': 'UnicodeCharacter0xdff6',
u'\u767b': 'CjkUnifiedIdeograph767b',
u'\u59fa': 'CjkUnifiedIdeograph59fa',
u'\ub57d': 'HangulSyllableDdyang',
u'\u94fc': 'CjkUnifiedIdeograph94fc',
u'\u8e81': 'CjkUnifiedIdeograph8e81',
u'\uef00': 'UnicodeCharacter0xef00',
u'\ucb83': 'HangulSyllableJjwegs',
u'\u2c02': 'GlagoliticCapitalLetterVede',
u'\u0885': 'UnicodeCharacter0x885',
u'\u6904': 'CjkUnifiedIdeograph6904',
u'\u4587': 'CjkUnifiedIdeograph4587',
u'\uba0e': 'HangulSyllableMyalp',
u'\u9691': 'CjkUnifiedIdeograph9691',
u'\uf710': 'UnicodeCharacter0xf710',
u'\ud393': 'HangulSyllablePeoc',
u'\u3412': 'CjkUnifiedIdeograph3412',
u'\u1095': 'MyanmarShanDigitFive',
u'\u7114': 'CjkUnifiedIdeograph7114',
u'\u2d97': 'UnicodeCharacter0x2d97',
u'\u6a99': 'CjkUnifiedIdeograph6a99',
u'\u821e': 'CjkUnifiedIdeograph821e',
u'\udf20': 'UnicodeCharacter0xdf20',
u'\ubba3': 'HangulSyllableMwih',
u'\u1c22': 'LepchaLetterWa',
u'\uf8a5': 'UnicodeCharacter0xf8a5',
u'\u5924': 'CjkUnifiedIdeograph5924',
u'\u35a7': 'CjkUnifiedIdeograph35a7',
u'\u72a9': 'CjkUnifiedIdeograph72a9',
u'\ua730': 'LatinLetterSmallCapitalF',
u'\u83b3': 'CjkUnifiedIdeograph83b3',
u'\ue432': 'UnicodeCharacter0xe432',
u'\uc0b5': 'HangulSyllableSalg',
u'\u2134': 'ScriptSmallO',
u'\u1db7': 'ModifierLetterSmallUpsilon',
u'\u7e36': 'CjkUnifiedIdeograph7e36',
u'\u5ab9': 'CjkUnifiedIdeograph5ab9',
u'\u8f40': 'CjkUnifiedIdeograph8f40',
u'\ucc42': 'HangulSyllableCap',
u'\ua8c5': 'UnicodeCharacter0xa8c5',
u'\u0944': 'DevanagariVowelSignVocalicRr',
u'\ue5c7': 'UnicodeCharacter0xe5c7',
u'\u4646': 'CjkUnifiedIdeograph4646',
u'\u22c9': 'LeftNormalFactorSemidirectProduct',
u'\u7fcb': 'CjkUnifiedIdeograph7fcb',
u'\u9750': 'CjkUnifiedIdeograph9750',
u'\ud452': 'HangulSyllablePoebs',
u'\ub0d5': 'HangulSyllableNyanj',
u'\u1154': 'HangulChoseongChitueumchieuch',
u'\ucdd7': 'HangulSyllableCwelb',
u'\u2e56': 'UnicodeCharacter0x2e56',
u'\u0ad9': 'UnicodeCharacter0xad9',
u'\u6b58': 'CjkUnifiedIdeograph6b58',
u'\u47db': 'CjkUnifiedIdeograph47db',
u'\ubc62': 'HangulSyllableByaj',
u'\u98e5': 'CjkUnifiedIdeograph98e5',
u'\uf964': 'CjkCompatibilityIdeographf964',
u'\ud5e7': 'HangulSyllableHegs',
u'\u3666': 'CjkUnifiedIdeograph3666',
u'\u12e9': 'EthiopicSyllableYu',
u'\u7368': 'CjkUnifiedIdeograph7368',
u'\u2feb': 'UnicodeCharacter0x2feb',
u'\u6ced': 'CjkUnifiedIdeograph6ced',
u'\u8472': 'CjkUnifiedIdeograph8472',
u'\uc174': 'HangulSyllableSyen',
u'\ubdf7': 'HangulSyllableByud',
u'\u1e76': 'LatinCapitalLetterUWithCircumflexBelow',
u'\ufaf9': 'UnicodeCharacter0xfaf9',
u'\u5b78': 'CjkUnifiedIdeograph5b78',
u'\u37fb': 'CjkUnifiedIdeograph37fb',
u'\u74fd': 'CjkUnifiedIdeograph74fd',
u'\u4e01': 'CjkUnifiedIdeograph4e01',
u'\ua984': 'UnicodeCharacter0xa984',
u'\ue686': 'UnicodeCharacter0xe686',
u'\u2388': 'HelmSymbol',
u'\u9f0b': 'CjkUnifiedIdeograph9f0b',
u'\u608a': 'CjkUnifiedIdeograph608a',
u'\udc0d': 'UnicodeCharacter0xdc0d',
u'\u190f': 'LimbuLetterNa',
u'\u5611': 'CjkUnifiedIdeograph5611',
u'\ub194': 'HangulSyllableNwa',
u'\uce96': 'HangulSyllableKaenh',
u'\u0b98': 'UnicodeCharacter0xb98',
u'\u489a': 'CjkUnifiedIdeograph489a',
u'\ua41d': 'YiSyllableJjiex',
u'\ue11f': 'UnicodeCharacter0xe11f',
u'\u3e21': 'CjkUnifiedIdeograph3e21',
u'\u7b23': 'CjkUnifiedIdeograph7b23',
u'\u99a4': 'CjkUnifiedIdeograph99a4',
u'\ud6a6': 'HangulSyllableHoep',
u'\u13a8': 'CherokeeLetterGe',
u'\u50aa': 'CjkUnifiedIdeograph50aa',
u'\u8c2d': 'CjkUnifiedIdeograph8c2d',
u'\u6dac': 'CjkUnifiedIdeograph6dac',
u'\uc92f': 'HangulSyllableJweoc',
u'\u0631': 'ArabicLetterReh',
u'\u4333': 'CjkUnifiedIdeograph4333',
u'\ubeb6': 'HangulSyllableBbyaegg',
u'\ufbb8': 'UnicodeCharacter0xfbb8',
u'\u38ba': 'CjkUnifiedIdeograph38ba',
u'\u943d': 'CjkUnifiedIdeograph943d',
u'\u75bc': 'CjkUnifiedIdeograph75bc',
u'\ud13f': 'HangulSyllableTeolh',
u'\uee41': 'UnicodeCharacter0xee41',
u'\u2b43': 'RightwardsArrowThroughGreaterthan',
u'\u6845': 'CjkUnifiedIdeograph6845',
u'\u86c6': 'CjkUnifiedIdeograph86c6',
u'\uc3c8': 'HangulSyllableSsyels',
u'\xca': 'LatinCapitalLetterEWithCircumflex',
u'\u5dcc': 'CjkUnifiedIdeograph5dcc',
u'\ub94f': 'HangulSyllableRwis',
u'\uf651': 'UnicodeCharacter0xf651',
u'\u3353': 'SquareRupii',
u'\u7055': 'CjkUnifiedIdeograph7055',
u'\uabd8': 'UnicodeCharacter0xabd8',
u'\ue8da': 'UnicodeCharacter0xe8da',
u'\u25dc': 'UpperLeftQuadrantCircularArc',
u'\u815f': 'CjkUnifiedIdeograph815f',
u'\u62de': 'CjkUnifiedIdeograph62de',
u'\ude61': 'UnicodeCharacter0xde61',
u'\u1b63': 'BalineseMusicalSymbolDung',
u'\u5865': 'CjkUnifiedIdeograph5865',
u'\ub3e8': 'HangulSyllableDwal',
u'\uf0ea': 'UnicodeCharacter0xf0ea',
u'\u0dec': 'UnicodeCharacter0xdec',
u'\u4aee': 'CjkUnifiedIdeograph4aee',
u'\ua671': 'CombiningCyrillicHundredMillionsSign',
u'\ue373': 'UnicodeCharacter0xe373',
u'\u2075': 'SuperscriptFive',
u'\u7d77': 'CjkUnifiedIdeograph7d77',
u'\u9bf8': 'CjkUnifiedIdeograph9bf8',
u'\ud8fa': 'UnicodeCharacter0xd8fa',
u'\u15fc': 'CanadianSyllabicsCarrierKko',
u'\u52fe': 'CjkUnifiedIdeograph52fe',
u'\u7000': 'CjkUnifiedIdeograph7000',
u'\u5483': 'CjkUnifiedIdeograph5483',
u'\ub302': 'HangulSyllableDaegg',
u'\u9785': 'CjkUnifiedIdeograph9785',
u'\u3906': 'CjkUnifiedIdeograph3906',
u'\u1d89': 'LatinSmallLetterRWithPalatalHook',
u'\ue20c': 'UnicodeCharacter0xe20c',
u'\uc68f': 'HangulSyllableOec',
u'\u6810': 'CjkUnifiedIdeograph6810',
u'\u4c93': 'CjkUnifiedIdeograph4c93',
u'\uab12': 'UnicodeCharacter0xab12',
u'\u8f95': 'CjkUnifiedIdeograph8f95',
u'\u1116': 'HangulChoseongNieunpieup',
u'\uf599': 'UnicodeCharacter0xf599',
u'\u7b9d': 'CjkUnifiedIdeograph7b9d',
u'\uda1c': 'UnicodeCharacter0xda1c',
u'\ube9f': 'HangulSyllableBbyad',
u'\u4020': 'CjkUnifiedIdeograph4020',
u'\u24a3': 'ParenthesizedLatinSmallLetterH',
u'\u8322': 'CjkUnifiedIdeograph8322',
u'\u0926': 'DevanagariLetterDa',
u'\ueda9': 'UnicodeCharacter0xeda9',
u'\u53ad': 'CjkUnifiedIdeograph53ad',
u'\ub22c': 'HangulSyllableNweols',
u'\u96af': 'CjkUnifiedIdeograph96af',
u'\u3830': 'CjkUnifiedIdeograph3830',
u'\u1cb3': 'UnicodeCharacter0x1cb3',
u'\ue136': 'UnicodeCharacter0xe136',
u'\uc5b9': 'HangulSyllableEonj',
u'\u673a': 'CjkUnifiedIdeograph673a',
u'\u4bbd': 'CjkUnifiedIdeograph4bbd',
u'\uaa3c': 'UnicodeCharacter0xaa3c',
u'\u8ebf': 'CjkUnifiedIdeograph8ebf',
u'\u1040': 'MyanmarDigitZero',
u'\uf4c3': 'UnicodeCharacter0xf4c3',
u'\u7ac7': 'CjkUnifiedIdeograph7ac7',
u'\ud946': 'UnicodeCharacter0xd946',
u'\ubdc9': 'HangulSyllableBweb',
u'\u5f4a': 'CjkUnifiedIdeograph5f4a',
u'\u23cd': 'SquareFoot',
u'\u824c': 'CjkUnifiedIdeograph824c',
u'\u0850': 'UnicodeCharacter0x850',
u'\uecd3': 'UnicodeCharacter0xecd3',
u'\u52d7': 'CjkUnifiedIdeograph52d7',
u'\ub156': 'HangulSyllableNyeoj',
u'\u95d9': 'CjkUnifiedIdeograph95d9',
u'\u375a': 'CjkUnifiedIdeograph375a',
u'\u1bdd': 'UnicodeCharacter0x1bdd',
u'\ue060': 'UnicodeCharacter0xe060',
u'\uc4e3': 'HangulSyllableSsyulh',
u'\u6664': 'CjkUnifiedIdeograph6664',
u'\u4ae7': 'CjkUnifiedIdeograph4ae7',
u'\ua966': 'UnicodeCharacter0xa966',
u'\u8de9': 'CjkUnifiedIdeograph8de9',
u'\u2f6a': 'KangxiRadicalSkin',
u'\uf3ed': 'UnicodeCharacter0xf3ed',
u'\u79f1': 'CjkUnifiedIdeograph79f1',
u'\ud870': 'UnicodeCharacter0xd870',
u'\ubcf3': 'HangulSyllableByeh',
u'\u5e74': 'CjkUnifiedIdeograph5e74',
u'\u22f7': 'SmallElementOfWithOverbar',
u'\u8176': 'CjkUnifiedIdeograph8176',
u'\u077a': 'ArabicLetterYehBarreeWithExtendedArabicindicDigitTwoAbove',
u'\uebfd': 'UnicodeCharacter0xebfd',
u'\ud101': 'HangulSyllableTyalg',
u'\ub080': 'HangulSyllableGgin',
u'\u5705': 'CjkUnifiedIdeograph5705',
u'\u3684': 'CjkUnifiedIdeograph3684',
u'\u9a07': 'CjkUnifiedIdeograph9a07',
u'\uff8a': 'HalfwidthKatakanaLetterHa',
u'\u658e': 'CjkUnifiedIdeograph658e',
u'\uc911': 'HangulSyllableJung',
u'\ua890': 'SaurashtraLetterOo',
u'\u4f15': 'CjkUnifiedIdeograph4f15',
u'\u2e94': 'CjkRadicalSnoutOne',
u'\uf81b': 'UnicodeCharacter0xf81b',
u'\ud79a': 'HangulSyllableHibs',
u'\u7e1f': 'CjkUnifiedIdeograph7e1f',
u'\u5d9e': 'CjkUnifiedIdeograph5d9e',
u'\ua121': 'YiSyllableTat',
u'\u80a0': 'CjkUnifiedIdeograph80a0',
u'\u2725': 'FourClubspokedAsterisk',
u'\u06a4': 'ArabicLetterVeh',
u'\ud02b': 'HangulSyllableKwes',
u'\ucfaa': 'HangulSyllableKoegg',
u'\u562f': 'CjkUnifiedIdeograph562f',
u'\u35ae': 'CjkUnifiedIdeograph35ae',
u'\u9931': 'CjkUnifiedIdeograph9931',
u'\u1f35': 'GreekSmallLetterIotaWithDasiaAndOxia',
u'\ufeb4': 'ArabicLetterSeenMedialForm',
u'\u64b8': 'CjkUnifiedIdeograph64b8',
u'\uc83b': 'HangulSyllableJyeogs',
u'\ua7ba': 'UnicodeCharacter0xa7ba',
u'\u4e3f': 'CjkUnifiedIdeograph4e3f',
u'\u2dbe': 'EthiopicSyllableCcho',
u'\uf745': 'UnicodeCharacter0xf745',
u'\ud6c4': 'HangulSyllableHu',
u'\u7d49': 'CjkUnifiedIdeograph7d49',
u'\u5cc8': 'CjkUnifiedIdeograph5cc8',
u'\ua04b': 'YiSyllablePux',
u'\u9fca': 'UnicodeCharacter0x9fca',
u'\u264f': 'Scorpius',
u'\u05ce': 'UnicodeCharacter0x5ce',
u'\uef55': 'UnicodeCharacter0xef55',
u'\uced4': 'HangulSyllableKyaels',
u'\u5559': 'CjkUnifiedIdeograph5559',
u'\u34d8': 'CjkUnifiedIdeograph34d8',
u'\u985b': 'CjkUnifiedIdeograph985b',
u'\u1e5f': 'LatinSmallLetterRWithLineBelow',
u'\ufdde': 'UnicodeCharacter0xfdde',
u'\u63e2': 'CjkUnifiedIdeograph63e2',
u'\uc765': 'HangulSyllableYilt',
u'\ua6e4': 'UnicodeCharacter0xa6e4',
u'\u4d69': 'CjkUnifiedIdeograph4d69',
u'\u2ce8': 'CopticSymbolTauRo',
u'\uf66f': 'UnicodeCharacter0xf66f',
u'\ud5ee': 'HangulSyllableHelm',
u'\u7c73': 'CjkUnifiedIdeograph7c73',
u'\u5bf2': 'CjkUnifiedIdeograph5bf2',
u'\ubf75': 'HangulSyllableBbwat',
u'\u9ef4': 'CjkUnifiedIdeograph9ef4',
u'\u2579': 'BoxDrawingsHeavyUp',
u'\u04f8': 'CyrillicCapitalLetterYeruWithDiaeresis',
u'\uee7f': 'UnicodeCharacter0xee7f',
u'\ucdfe': 'HangulSyllableCwij',
u'\ub806': 'HangulSyllableReop',
u'\u9c89': 'CjkUnifiedIdeograph9c89',
u'\ufd08': 'ArabicLigatureDadWithYehIsolatedForm',
u'\uc18b': 'HangulSyllableSyeh',
u'\u3e0a': 'CjkUnifiedIdeograph3e0a',
u'\u028d': 'LatinSmallLetterTurnedW',
u'\u630c': 'CjkUnifiedIdeograph630c',
u'\u478f': 'CjkUnifiedIdeograph478f',
u'\u9016': 'CjkUnifiedIdeograph9016',
u'\ud518': 'HangulSyllablePeuss',
u'\ub99b': 'HangulSyllableRyilb',
u'\u161a': 'CanadianSyllabicsSayisiJi',
u'\ufa9d': 'CjkCompatibilityIdeographfa9d',
u'\u5b1c': 'CjkUnifiedIdeograph5b1c',
u'\u3f9f': 'CjkUnifiedIdeograph3f9f',
u'\u60a1': 'CjkUnifiedIdeograph60a1',
u'\u8826': 'CjkUnifiedIdeograph8826',
u'\ucd28': 'HangulSyllableCwan',
u'\u91ab': 'CjkUnifiedIdeograph91ab',
u'\u0e2a': 'ThaiCharacterSoSua',
u'\ud2ad': 'HangulSyllableTyub',
u'\u332c': 'SquarePaatu',
u'\u17af': 'KhmerIndependentVowelQe',
u'\u742e': 'CjkUnifiedIdeograph742e',
u'\u58b1': 'CjkUnifiedIdeograph58b1',
u'\ua538': 'VaiSyllableRi',
u'\u89bb': 'CjkUnifiedIdeograph89bb',
u'\ue63a': 'UnicodeCharacter0xe63a',
u'\ucabd': 'HangulSyllableJjog',
u'\u2b3c': 'LeftwardsTwoheadedArrowWithTailWithVerticalStroke',
u'\u0fbf': 'TibetanKuRuKhaBzhiMigCan',
u'\u6c3e': 'CjkUnifiedIdeograph6c3e',
u'\u30c1': 'KatakanaLetterTi',
u'\u75c3': 'CjkUnifiedIdeograph75c3',
u'\u9d48': 'CjkUnifiedIdeograph9d48',
u'\ude4a': 'UnicodeCharacter0xde4a',
u'\ua2cd': 'YiSyllableCyp',
u'\u034c': 'CombiningAlmostEqualToAbove',
u'\ue7cf': 'UnicodeCharacter0xe7cf',
u'\u444e': 'CjkUnifiedIdeograph444e',
u'\u28d1': 'BraillePatternDots1578',
u'\u6dd3': 'CjkUnifiedIdeograph6dd3',
u'\ub65a': 'HangulSyllableDdwaebs',
u'\u9add': 'CjkUnifiedIdeograph9add',
u'\ufb5c': 'ArabicLetterBehehInitialForm',
u'\udfdf': 'UnicodeCharacter0xdfdf',
u'\u3c5e': 'CjkUnifiedIdeograph3c5e',
u'\xe1': 'LatinSmallLetterAWithAcute',
u'\u6160': 'CjkUnifiedIdeograph6160',
u'\u45e3': 'CjkUnifiedIdeograph45e3',
u'\uae6a': 'HangulSyllableGgaegg',
u'\ud36c': 'HangulSyllablePyaels',
u'\ub7ef': 'HangulSyllableReogs',
u'\u146e': 'CanadianSyllabicsKii',
u'\uf8f1': 'UnicodeCharacter0xf8f1',
u'\u5970': 'CjkUnifiedIdeograph5970',
u'\u3df3': 'CjkUnifiedIdeograph3df3',
u'\u7ef5': 'CjkUnifiedIdeograph7ef5',
u'\u867a': 'CjkUnifiedIdeograph867a',
u'\ucb7c': 'HangulSyllableJjweok',
u'\uafff': 'HangulSyllableGgwelh',
u'\u0c7e': 'TeluguFractionDigitThreeForEvenPowersOfFour',
u'\u3180': 'HangulLetterSsangieung',
u'\u9503': 'CjkUnifiedIdeograph9503',
u'\u7282': 'CjkUnifiedIdeograph7282',
u'\ud605': 'HangulSyllableHyeonj',
u'\u1b07': 'BalineseLetterIkara',
u'\u5c09': 'CjkUnifiedIdeograph5c09',
u'\ua38c': 'YiSyllableRrurx',
u'\ue48e': 'UnicodeCharacter0xe48e',
u'\u2990': 'RightSquareBracketWithTickInTopCorner',
u'\u8d13': 'CjkUnifiedIdeograph8d13',
u'\u6a92': 'CjkUnifiedIdeograph6a92',
u'\uce15': 'HangulSyllableCyub',
u'\uf317': 'UnicodeCharacter0xf317',
u'\u3419': 'CjkUnifiedIdeograph3419',
u'\u791b': 'CjkUnifiedIdeograph791b',
u'\u9b9c': 'CjkUnifiedIdeograph9b9c',
u'\udc9e': 'UnicodeCharacter0xdc9e',
u'\u01a0': 'LatinCapitalLetterOWithHorn',
u'\u42a2': 'CjkUnifiedIdeograph42a2',
u'\ua625': 'VaiDigitFive',
u'\ueb27': 'UnicodeCharacter0xeb27',
u'\u2c29': 'GlagoliticCapitalLetterIotatedBigYus',
u'\u512b': 'CjkUnifiedIdeograph512b',
u'\ub4ae': 'HangulSyllableDwilm',
u'\uf9b0': 'CjkCompatibilityIdeographf9b0',
u'\u3ab2': 'CjkUnifiedIdeograph3ab2',
u'\u9e35': 'CjkUnifiedIdeograph9e35',
u'\u7fb4': 'CjkUnifiedIdeograph7fb4',
u'\uc337': 'HangulSyllableSsyad',
u'\u0439': 'CyrillicSmallLetterShortI',
u'\u493b': 'CjkUnifiedIdeograph493b',
u'\uacbe': 'HangulSyllableGyeoj',
u'\ud1c0': 'HangulSyllableTwan',
u'\u12c2': 'EthiopicSyllableKxwi',
u'\u57c4': 'CjkUnifiedIdeograph57c4',
u'\ubb47': 'HangulSyllableMus',
u'\ufc49': 'ArabicLigatureMeemWithAlefMaksuraIsolatedForm',
u'\u214b': 'TurnedAmpersand',
u'\u624d': 'CjkUnifiedIdeograph624d',
u'\u84ce': 'CjkUnifiedIdeograph84ce',
u'\uc9d0': 'HangulSyllableJim',
u'\u0ad2': 'UnicodeCharacter0xad2',
u'\u4fd4': 'CjkUnifiedIdeograph4fd4',
u'\u9357': 'CjkUnifiedIdeograph9357',
u'\u70d6': 'CjkUnifiedIdeograph70d6',
u'\ud459': 'HangulSyllablePoet',
u'\u195b': 'TaiLeLetterMa',
u'\u5a5d': 'CjkUnifiedIdeograph5a5d',
u'\ua1e0': 'YiSyllableGie',
u'\ue2e2': 'UnicodeCharacter0xe2e2',
u'\u27e4': 'WhiteSquareWithLeftwardsTick',
u'\u8b67': 'CjkUnifiedIdeograph8b67',
u'\u68e6': 'CjkUnifiedIdeograph68e6',
u'\ucc69': 'HangulSyllableCyalg',
u'\uf16b': 'UnicodeCharacter0xf16b',
u'\u326d': 'CircledHangulHieuh',
u'\u776f': 'CjkUnifiedIdeograph776f',
u'\u99f0': 'CjkUnifiedIdeograph99f0',
u'\udaf2': 'UnicodeCharacter0xdaf2',
u'\u1ff4': 'GreekSmallLetterOmegaWithOxiaAndYpogegrammeni',
u'\u40f6': 'CjkUnifiedIdeograph40f6',
u'\ua479': 'YiSyllableYuot',
u'\ue97b': 'UnicodeCharacter0xe97b',
u'\u2a7d': 'LessthanOrSlantedEqualTo',
u'\u6f7f': 'CjkUnifiedIdeograph6f7f',
u'\u1381': 'EthiopicSyllableMwi',
u'\uf404': 'UnicodeCharacter0xf404',
u'\ud887': 'UnicodeCharacter0xd887',
u'\u7e08': 'CjkUnifiedIdeograph7e08',
u'\u428b': 'CjkUnifiedIdeograph428b',
u'\ubd0a': 'HangulSyllableBoj',
u'\u818d': 'CjkUnifiedIdeograph818d',
u'\u270e': 'LowerRightPencil',
u'\u0b91': 'UnicodeCharacter0xb91',
u'\uec14': 'UnicodeCharacter0xec14',
u'\ub097': 'HangulSyllableGgih',
u'\u5618': 'CjkUnifiedIdeograph5618',
u'\u3a9b': 'CjkUnifiedIdeograph3a9b',
u'\u951a': 'CjkUnifiedIdeograph951a',
u'\u1f1e': 'UnicodeCharacter0x1f1e',
u'\ue3a1': 'UnicodeCharacter0xe3a1',
u'\u65a5': 'CjkUnifiedIdeograph65a5',
u'\uc424': 'HangulSyllableSswaess',
u'\ua8a7': 'SaurashtraLetterPha',
u'\u4e28': 'CjkUnifiedIdeograph4e28',
u'\u12ab': 'EthiopicSyllableKaa',
u'\u8d2a': 'CjkUnifiedIdeograph8d2a',
u'\uf72e': 'UnicodeCharacter0xf72e',
u'\udbb1': 'UnicodeCharacter0xdbb1',
u'\u7932': 'CjkUnifiedIdeograph7932',
u'\u5db5': 'CjkUnifiedIdeograph5db5',
u'\ubc34': 'HangulSyllableBaen',
u'\u80b7': 'CjkUnifiedIdeograph80b7',
u'\u2638': 'WheelOfDharma',
u'\u0abb': 'UnicodeCharacter0xabb',
u'\uef3e': 'UnicodeCharacter0xef3e',
u'\ub3c1': 'HangulSyllableDyet',
u'\u5142': 'CjkUnifiedIdeograph5142',
u'\u35c5': 'CjkUnifiedIdeograph35c5',
u'\u9444': 'CjkUnifiedIdeograph9444',
u'\u1e48': 'LatinCapitalLetterNWithLineBelow',
u'\ue2cb': 'UnicodeCharacter0xe2cb',
u'\u64cf': 'CjkUnifiedIdeograph64cf',
u'\uc74e': 'HangulSyllableEubs',
u'\uabd1': 'UnicodeCharacter0xabd1',
u'\u4952': 'CjkUnifiedIdeograph4952',
u'\u2dd5': 'EthiopicSyllableXye',
u'\u8c54': 'CjkUnifiedIdeograph8c54',
u'\uf658': 'UnicodeCharacter0xf658',
u'\udadb': 'UnicodeCharacter0xdadb',
u'\u785c': 'CjkUnifiedIdeograph785c',
u'\u5cdf': 'CjkUnifiedIdeograph5cdf',
u'\ubf5e': 'HangulSyllableBbwagg',
u'\u83e1': 'CjkUnifiedIdeograph83e1',
u'\u2162': 'RomanNumeralThree',
u'\u05e5': 'HebrewLetterFinalTsadi',
u'\uee68': 'UnicodeCharacter0xee68',
u'\ub2eb': 'HangulSyllableDad',
u'\u506c': 'CjkUnifiedIdeograph506c',
u'\u34ef': 'CjkUnifiedIdeograph34ef',
u'\u976e': 'CjkUnifiedIdeograph976e',
u'\u1972': 'TaiLeLetterTone4',
u'\ufdf5': 'ArabicLigatureSalamIsolatedForm',
u'\u67f9': 'CjkUnifiedIdeograph67f9',
u'\uc678': 'HangulSyllableOe',
u'\uaafb': 'UnicodeCharacter0xaafb',
u'\u487c': 'CjkUnifiedIdeograph487c',
u'\u2cff': 'CopticMorphologicalDivider',
u'\u8f7e': 'CjkUnifiedIdeograph8f7e',
u'\u1603': 'CanadianSyllabicsCarrierNo',
u'\uf182': 'UnicodeCharacter0xf182',
u'\u7b86': 'CjkUnifiedIdeograph7b86',
u'\udf09': 'UnicodeCharacter0xdf09',
u'\ube88': 'HangulSyllableBbaels',
u'\u410d': 'CjkUnifiedIdeograph410d',
u'\u208c': 'SubscriptEqualsSign',
u'\u840f': 'CjkUnifiedIdeograph840f',
u'\u0e13': 'ThaiCharacterNoNen',
u'\ue992': 'UnicodeCharacter0xe992',
u'\u7017': 'CjkUnifiedIdeograph7017',
u'\u5396': 'CjkUnifiedIdeograph5396',
u'\ub719': 'HangulSyllableDdyult',
u'\u9698': 'CjkUnifiedIdeograph9698',
u'\u391d': 'CjkUnifiedIdeograph391d',
u'\u189c': 'MongolianLetterManchuAliGaliCa',
u'\ue623': 'UnicodeCharacter0xe623',
u'\uc1a2': 'HangulSyllableSoj',
u'\u6827': 'CjkUnifiedIdeograph6827',
u'\u4ba6': 'CjkUnifiedIdeograph4ba6',
u'\uaf29': 'HangulSyllableGgyet',
u'\u8ea8': 'CjkUnifiedIdeograph8ea8',
u'\u112d': 'HangulChoseongSioskiyeok',
u'\uf0ac': 'UnicodeCharacter0xf0ac',
u'\u7ab0': 'CjkUnifiedIdeograph7ab0',
u'\ude33': 'UnicodeCharacter0xde33',
u'\ub9b2': 'HangulSyllableRinh',
u'\u4037': 'CjkUnifiedIdeograph4037',
u'\u23b6': 'BottomSquareBracketOverTopSquareBracket',
u'\u8739': 'CjkUnifiedIdeograph8739',
u'\u093d': 'DevanagariSignAvagraha',
u'\ue8bc': 'UnicodeCharacter0xe8bc',
u'\u7341': 'CjkUnifiedIdeograph7341',
u'\u52c0': 'CjkUnifiedIdeograph52c0',
u'\ub643': 'HangulSyllableDdwac',
u'\u91c2': 'CjkUnifiedIdeograph91c2',
u'\u3847': 'CjkUnifiedIdeograph3847',
u'\u1bc6': 'UnicodeCharacter0x1bc6',
u'\ue14d': 'UnicodeCharacter0xe14d',
u'\uc0cc': 'HangulSyllableSaen',
u'\u6b51': 'CjkUnifiedIdeograph6b51',
u'\u4ad0': 'CjkUnifiedIdeograph4ad0',
u'\uae53': 'HangulSyllableGgad',
u'\u89d2': 'CjkUnifiedIdeograph89d2',
u'\u1057': 'MyanmarVowelSignVocalicRr',
u'\uf3d6': 'UnicodeCharacter0xf3d6',
u'\u75da': 'CjkUnifiedIdeograph75da',
u'\ud95d': 'UnicodeCharacter0xd95d',
u'\ub8dc': 'HangulSyllableRyom',
u'\u4361': 'CjkUnifiedIdeograph4361',
u'\u22e0': 'DoesNotPrecedeOrEqual',
u'\u8663': 'CjkUnifiedIdeograph8663',
u'\u0867': 'UnicodeCharacter0x867',
u'\uebe6': 'UnicodeCharacter0xebe6',
u'\u726b': 'CjkUnifiedIdeograph726b',
u'\u6dea': 'CjkUnifiedIdeograph6dea',
u'\ub16d': 'HangulSyllableNyeb',
u'\u90ec': 'CjkUnifiedIdeograph90ec',
u'\u3b71': 'CjkUnifiedIdeograph3b71',
u'\u1af0': 'UnicodeCharacter0x1af0',
u'\ue077': 'UnicodeCharacter0xe077',
u'\uc3f6': 'HangulSyllableSswagg',
u'\u6a7b': 'CjkUnifiedIdeograph6a7b',
u'\u45fa': 'CjkUnifiedIdeograph45fa',
u'\ua97d': 'UnicodeCharacter0xa97d',
u'\u88fc': 'CjkUnifiedIdeograph88fc',
u'\u9281': 'CjkUnifiedIdeograph9281',
u'\uf300': 'UnicodeCharacter0xf300',
u'\ud783': 'HangulSyllableHyic',
u'\u3002': 'IdeographicFullStop',
u'\u1485': 'CanadianSyllabicsSouthslaveyKeh',
u'\u7504': 'CjkUnifiedIdeograph7504',
u'\u5987': 'CjkUnifiedIdeograph5987',
u'\ua60e': 'VaiFullStop',
u'\u8a91': 'CjkUnifiedIdeograph8a91',
u'\ueb10': 'UnicodeCharacter0xeb10',
u'\ucf93': 'HangulSyllableKwaed',
u'\u2812': 'BraillePatternDots25',
u'\u0c95': 'KannadaLetterKa',
u'\u6d14': 'CjkUnifiedIdeograph6d14',
u'\u3197': 'IdeographicAnnotationMiddleMark',
u'\u7699': 'CjkUnifiedIdeograph7699',
u'\u9e1e': 'CjkUnifiedIdeograph9e1e',
u'\uc320': 'HangulSyllableSsaels',
u'\ua7a3': 'UnicodeCharacter0xa7a3',
u'\ue4a5': 'UnicodeCharacter0xe4a5',
u'\u4524': 'CjkUnifiedIdeograph4524',
u'\u29a7': 'ObliqueAngleOpeningDown',
u'\u6ea9': 'CjkUnifiedIdeograph6ea9',
u'\ubb30': 'HangulSyllableMyok',
u'\u9fb3': 'CjkUnifiedIdeograph9fb3',
u'\uf832': 'UnicodeCharacter0xf832',
u'\udcb5': 'UnicodeCharacter0xdcb5',
u'\u3d34': 'CjkUnifiedIdeograph3d34',
u'\u01b7': 'LatinCapitalLetterEzh',
u'\u6236': 'CjkUnifiedIdeograph6236',
u'\u46b9': 'CjkUnifiedIdeograph46b9',
u'\u9340': 'CjkUnifiedIdeograph9340',
u'\ud042': 'HangulSyllableKwilp',
u'\ub4c5': 'HangulSyllableDyunj',
u'\u1544': 'CanadianSyllabicsWestcreeLe',
u'\uf9c7': 'CjkCompatibilityIdeographf9c7',
u'\u5a46': 'CjkUnifiedIdeograph5a46',
u'\u3ec9': 'CjkUnifiedIdeograph3ec9',
u'\u63cb': 'CjkUnifiedIdeograph63cb',
u'\u8b50': 'CjkUnifiedIdeograph8b50',
u'\uc852': 'HangulSyllableJyeop',
u'\uacd5': 'HangulSyllableGyeb',
u'\u0d54': 'UnicodeCharacter0xd54',
u'\ud1d7': 'HangulSyllableTwah',
u'\u3256': 'CircledNumberTwentySix',
u'\u16d9': 'RunicLetterShorttwigmadrM',
u'\u7758': 'CjkUnifiedIdeograph7758',
u'\u5bdb': 'CjkUnifiedIdeograph5bdb',
u'\ua062': 'YiSyllableBbuox',
u'\u84e5': 'CjkUnifiedIdeograph84e5',
u'\ue564': 'UnicodeCharacter0xe564',
u'\uc9e7': 'HangulSyllableJjalb',
u'\u2a66': 'EqualsSignWithDotBelow',
u'\u0ee9': 'UnicodeCharacter0xee9',
u'\u6f68': 'CjkUnifiedIdeograph6f68',
u'\u33eb': 'IdeographicTelegraphSymbolForDayTwelve',
u'\u70ed': 'CjkUnifiedIdeograph70ed',
u'\u9872': 'CjkUnifiedIdeograph9872',
u'\udd74': 'UnicodeCharacter0xdd74',
u'\ua1f7': 'YiSyllableGur',
u'\u0276': 'LatinLetterSmallCapitalOe',
u'\ue6f9': 'UnicodeCharacter0xe6f9',
u'\u4778': 'CjkUnifiedIdeograph4778',
u'\u2bfb': 'UnicodeCharacter0x2bfb',
u'\u68fd': 'CjkUnifiedIdeograph68fd',
u'\u5201': 'CjkUnifiedIdeograph5201',
u'\ub584': 'HangulSyllableDdyae',
u'\ufa86': 'CjkCompatibilityIdeographfa86',
u'\u3f88': 'CjkUnifiedIdeograph3f88',
u'\u830b': 'CjkUnifiedIdeograph830b',
u'\u7c8a': 'CjkUnifiedIdeograph7c8a',
u'\uc00d': 'HangulSyllableBbwelg',
u'\u050f': 'CyrillicSmallLetterKomiTje',
u'\u4a11': 'CjkUnifiedIdeograph4a11',
u'\uad94': 'HangulSyllableGweols',
u'\ud296': 'HangulSyllableTwij',
u'\u1798': 'KhmerLetterMo',
u'\u549a': 'CjkUnifiedIdeograph549a',
u'\ub81d': 'HangulSyllableReng',
u'\ufd1f': 'ArabicLigatureKhahWithAlefMaksuraFinalForm',
u'\u2221': 'MeasuredAngle',
u'\u6723': 'CjkUnifiedIdeograph6723',
u'\u85a4': 'CjkUnifiedIdeograph85a4',
u'\ucaa6': 'HangulSyllableJjyenh',
u'\u0fa8': 'TibetanSubjoinedLetterMa',
u'\u4caa': 'CjkUnifiedIdeograph4caa',
u'\u902d': 'CjkUnifiedIdeograph902d',
u'\u71ac': 'CjkUnifiedIdeograph71ac',
u'\ud52f': 'HangulSyllablePyilh',
u'\u1a31': 'UnicodeCharacter0x1a31',
u'\u5f33': 'CjkUnifiedIdeograph5f33',
u'\ua2b6': 'YiSyllableCat',
u'\ue7b8': 'UnicodeCharacter0xe7b8',
u'\u24ba': 'CircledLatinCapitalLetterE',
u'\u883d': 'CjkUnifiedIdeograph883d',
u'\u69bc': 'CjkUnifiedIdeograph69bc',
u'\ucd3f': 'HangulSyllableCwah',
u'\uf241': 'UnicodeCharacter0xf241',
u'\u3743': 'CjkUnifiedIdeograph3743',
u'\u7445': 'CjkUnifiedIdeograph7445',
u'\u9ac6': 'CjkUnifiedIdeograph9ac6',
u'\udfc8': 'UnicodeCharacter0xdfc8',
u'\u1cca': 'UnicodeCharacter0x1cca',
u'\u41cc': 'CjkUnifiedIdeograph41cc',
u'\ua54f': 'VaiSyllableWan',
u'\uea51': 'UnicodeCharacter0xea51',
u'\u2f53': 'KangxiRadicalSteam',
u'\u6c55': 'CjkUnifiedIdeograph6c55',
u'\ub7d8': 'HangulSyllableRyael',
u'\uf4da': 'UnicodeCharacter0xf4da',
u'\u39dc': 'CjkUnifiedIdeograph39dc',
u'\u9d5f': 'CjkUnifiedIdeograph9d5f',
u'\u7ede': 'CjkUnifiedIdeograph7ede',
u'\uc261': 'HangulSyllableSweb',
u'\u0763': 'ArabicLetterKehehWithThreeDotsAbove',
u'\u4465': 'CjkUnifiedIdeograph4465',
u'\uafe8': 'HangulSyllableGgweoss',
u'\uecea': 'UnicodeCharacter0xecea',
u'\u11ec': 'HangulJongseongIeungkiyeok',
u'\u56ee': 'CjkUnifiedIdeograph56ee',
u'\uba71': 'HangulSyllableMyeog',
u'\uff73': 'HalfwidthKatakanaLetterU',
u'\u3c75': 'CjkUnifiedIdeograph3c75',
u'\u6177': 'CjkUnifiedIdeograph6177',
u'\u87f8': 'CjkUnifiedIdeograph87f8',
u'\uc4fa': 'HangulSyllableSseulm',
u'\u09fc': 'UnicodeCharacter0x9fc',
u'\u4efe': 'CjkUnifiedIdeograph4efe',
u'\u7400': 'CjkUnifiedIdeograph7400',
u'\u5083': 'CjkUnifiedIdeograph5083',
u'\ub702': 'HangulSyllableDdwibs',
u'\u9385': 'CjkUnifiedIdeograph9385',
u'\u3d06': 'CjkUnifiedIdeograph3d06',
u'\u1989': 'NewTaiLueLetterHighSa',
u'\ue60c': 'UnicodeCharacter0xe60c',
u'\uc28f': 'HangulSyllableSyud',
u'\u6c10': 'CjkUnifiedIdeograph6c10',
u'\u4893': 'CjkUnifiedIdeograph4893',
u'\uaf12': 'HangulSyllableGgyegg',
u'\u8b95': 'CjkUnifiedIdeograph8b95',
u'\u1516': 'CanadianSyllabicsShaa',
u'\uf199': 'UnicodeCharacter0xf199',
u'\u7f9d': 'CjkUnifiedIdeograph7f9d',
u'\ude1c': 'UnicodeCharacter0xde1c',
u'\uba9f': 'HangulSyllableMyes',
u'\u4420': 'CjkUnifiedIdeograph4420',
u'\u20a3': 'FrenchFrancSign',
u'\u8722': 'CjkUnifiedIdeograph8722',
u'\u0d26': 'MalayalamLetterDa',
u'\ue9a9': 'UnicodeCharacter0xe9a9',
u'\u57ad': 'CjkUnifiedIdeograph57ad',
u'\ub62c': 'HangulSyllableDdwa',
u'\u92af': 'CjkUnifiedIdeograph92af',
u'\u3c30': 'CjkUnifiedIdeograph3c30',
u'\u18b3': 'UnicodeCharacter0x18b3',
u'\ue536': 'UnicodeCharacter0xe536',
u'\uc1b9': 'HangulSyllableSwab',
u'\u633a': 'CjkUnifiedIdeograph633a',
u'\u4fbd': 'CjkUnifiedIdeograph4fbd',
u'\uae3c': 'HangulSyllableGils',
u'\u8abf': 'CjkUnifiedIdeograph8abf',
u'\u1440': 'CanadianSyllabicsPwo',
u'\uf0c3': 'UnicodeCharacter0xf0c3',
u'\u7ec7': 'CjkUnifiedIdeograph7ec7',
u'\udd46': 'UnicodeCharacter0xdd46',
u'\ub9c9': 'HangulSyllableMag',
u'\u5b4a': 'CjkUnifiedIdeograph5b4a',
u'\u27cd': 'UnicodeCharacter0x27cd',
u'\u864c': 'CjkUnifiedIdeograph864c',
u'\u0c50': 'UnicodeCharacter0xc50',
u'\ue8d3': 'UnicodeCharacter0xe8d3',
u'\u56d7': 'CjkUnifiedIdeograph56d7',
u'\ub556': 'HangulSyllableDdaelm',
u'\u91d9': 'CjkUnifiedIdeograph91d9',
u'\u335a': 'IdeographicTelegraphSymbolForHourTwo',
u'\u1fdd': 'GreekDasiaAndVaria',
u'\ue460': 'UnicodeCharacter0xe460',
u'\uc0e3': 'HangulSyllableSaeh',
u'\u6264': 'CjkUnifiedIdeograph6264',
u'\u4ee7': 'CjkUnifiedIdeograph4ee7',
u'\uad66': 'HangulSyllableGyoj',
u'\u89e9': 'CjkUnifiedIdeograph89e9',
u'\u2b6a': 'UnicodeCharacter0x2b6a',
u'\uf7ed': 'UnicodeCharacter0xf7ed',
u'\u7df1': 'CjkUnifiedIdeograph7df1',
u'\udc70': 'UnicodeCharacter0xdc70',
u'\ub8f3': 'HangulSyllableRulb',
u'\u5a74': 'CjkUnifiedIdeograph5a74',
u'\u26f7': 'UnicodeCharacter0x26f7',
u'\u8576': 'CjkUnifiedIdeograph8576',
u'\u037a': 'GreekYpogegrammeni',
u'\ueffd': 'UnicodeCharacter0xeffd',
u'\ud501': 'HangulSyllablePyut',
u'\ub480': 'HangulSyllableDweoss',
u'\u5305': 'CjkUnifiedIdeograph5305',
u'\u3284': 'CircledIdeographFive',
u'\u9e07': 'CjkUnifiedIdeograph9e07',
u'\u040b': 'CyrillicCapitalLetterTshe',
u'\ufb8a': 'ArabicLetterJehIsolatedForm',
u'\u618e': 'CjkUnifiedIdeograph618e',
u'\ucd11': 'HangulSyllableColg',
u'\uac90': 'HangulSyllableGen',
u'\u4b15': 'CjkUnifiedIdeograph4b15',
u'\u2a94': 'GreaterthanAboveSlantedEqualAboveLessthanAboveSlantedEqual',
u'\ufc1b': 'ArabicLigatureKhahWithMeemIsolatedForm',
u'\ud39a': 'HangulSyllablePegg',
u'\u7a1f': 'CjkUnifiedIdeograph7a1f',
u'\u599e': 'CjkUnifiedIdeograph599e',
u'\ua521': 'VaiSyllableMee',
u'\u84a0': 'CjkUnifiedIdeograph84a0',
u'\u2325': 'OptionKey',
u'\u02a4': 'LatinSmallLetterDezhDigraph',
u'\ud42b': 'HangulSyllablePwaed',
u'\ucbaa': 'HangulSyllableJjwilp',
u'\u522f': 'CjkUnifiedIdeograph522f',
u'\u31ae': 'BopomofoLetterAinn',
u'\u9d31': 'CjkUnifiedIdeograph9d31',
u'\u1b35': 'BalineseVowelSignTedung',
u'\ufab4': 'CjkCompatibilityIdeographfab4',
u'\u60b8': 'CjkUnifiedIdeograph60b8',
u'\ucc3b': 'HangulSyllableCas',
u'\ua3ba': 'YiSyllableShut',
u'\u4a3f': 'CjkUnifiedIdeograph4a3f',
u'\u29be': 'CircledWhiteBullet',
u'\uf345': 'UnicodeCharacter0xf345',
u'\ud2c4': 'HangulSyllableTeuls',
u'\u7949': 'CjkUnifiedIdeograph7949',
u'\u58c8': 'CjkUnifiedIdeograph58c8',
u'\ua44b': 'YiSyllableNyix',
u'\u9bca': 'CjkUnifiedIdeograph9bca',
u'\u224f': 'DifferenceBetween',
u'\u01ce': 'LatinSmallLetterAWithCaron',
u'\ueb55': 'UnicodeCharacter0xeb55',
u'\ucad4': 'HangulSyllableJjok',
u'\u5159': 'CjkUnifiedIdeograph5159',
u'\u30d8': 'KatakanaLetterHe',
u'\u9c5b': 'CjkUnifiedIdeograph9c5b',
u'\u1a5f': 'UnicodeCharacter0x1a5f',
u'\uf9de': 'CjkCompatibilityIdeographf9de',
u'\u67e2': 'CjkUnifiedIdeograph67e2',
u'\uc365': 'HangulSyllableSsyaet',
u'\ua2e4': 'YiSyllableZzup',
u'\u4969': 'CjkUnifiedIdeograph4969',
u'\u28e8': 'BraillePatternDots4678',
u'\uf26f': 'UnicodeCharacter0xf26f',
u'\ud1ee': 'HangulSyllableTwaej',
u'\u7873': 'CjkUnifiedIdeograph7873',
u'\u5ff2': 'CjkUnifiedIdeograph5ff2',
u'\ubb75': 'HangulSyllableMwelg',
u'\u9af4': 'CjkUnifiedIdeograph9af4',
u'\u2179': 'SmallRomanNumeralTen',
u'\xf8': 'LatinSmallLetterOWithStroke',
u'\uea7f': 'UnicodeCharacter0xea7f',
u'\uc9fe': 'HangulSyllableJjaenh',
u'\ubc06': 'HangulSyllableMilp',
u'\u9889': 'CjkUnifiedIdeograph9889',
u'\uf908': 'CjkCompatibilityIdeographf908',
u'\uc58b': 'HangulSyllableYalh',
u'\u3a0a': 'CjkUnifiedIdeograph3a0a',
u'\u068d': 'ArabicLetterDdahal',
u'\u670c': 'CjkUnifiedIdeograph670c',
u'\u438f': 'CjkUnifiedIdeograph438f',
u'\u9416': 'CjkUnifiedIdeograph9416',
u'\ud118': 'HangulSyllableTyaen',
u'\ubd9b': 'HangulSyllableBuh',
u'\u121a': 'EthiopicSyllableMi',
u'\ufe9d': 'ArabicLetterJeemIsolatedForm',
u'\u5f1c': 'CjkUnifiedIdeograph5f1c',
u'\u3b9f': 'CjkUnifiedIdeograph3b9f',
u'\u64a1': 'CjkUnifiedIdeograph64a1',
u'\u8c26': 'CjkUnifiedIdeograph8c26',
u'\uc928': 'HangulSyllableJweom',
u'\u95ab': 'CjkUnifiedIdeograph95ab',
u'\u0a2a': 'GurmukhiLetterPa',
u'\ud6ad': 'HangulSyllableHyonj',
u'\u372c': 'CjkUnifiedIdeograph372c',
u'\u13af': 'CherokeeLetterHi',
u'\u702e': 'CjkUnifiedIdeograph702e',
u'\u5cb1': 'CjkUnifiedIdeograph5cb1',
u'\ua138': 'YiSyllableDdi',
u'\u8dbb': 'CjkUnifiedIdeograph8dbb',
u'\ue23a': 'UnicodeCharacter0xe23a',
u'\ucebd': 'HangulSyllableKyab',
u'\u2f3c': 'KangxiRadicalHeart',
u'\u0bbf': 'TamilVowelSignI',
u'\u683e': 'CjkUnifiedIdeograph683e',
u'\u34c1': 'CjkUnifiedIdeograph34c1',
u'\u71c3': 'CjkUnifiedIdeograph71c3',
u'\u9948': 'CjkUnifiedIdeograph9948',
u'\uda4a': 'UnicodeCharacter0xda4a',
u'\ua6cd': 'UnicodeCharacter0xa6cd',
u'\u074c': 'UnicodeCharacter0x74c',
u'\ue3cf': 'UnicodeCharacter0xe3cf',
u'\u404e': 'CjkUnifiedIdeograph404e',
u'\u2cd1': 'CopticSmallLetterLshapedHa',
u'\u69d3': 'CjkUnifiedIdeograph69d3',
u'\ub25a': 'HangulSyllableNwigg',
u'\u9edd': 'CjkUnifiedIdeograph9edd',
u'\uff5c': 'FullwidthVerticalLine',
u'\udbdf': 'UnicodeCharacter0xdbdf',
u'\u385e': 'CjkUnifiedIdeograph385e',
u'\u04e1': 'CyrillicSmallLetterAbkhasianDze',
u'\u6560': 'CjkUnifiedIdeograph6560',
u'\u41e3': 'CjkUnifiedIdeograph41e3',
u'\uaa6a': 'UnicodeCharacter0xaa6a',
u'\ud76c': 'HangulSyllableHyi',
u'\ub3ef': 'HangulSyllableDwalh',
u'\u106e': 'MyanmarLetterEasternPwoKarenNna',
u'\ufcf1': 'ArabicLigatureYehWithHehMedialForm',
u'\u5d70': 'CjkUnifiedIdeograph5d70',
u'\u39f3': 'CjkUnifiedIdeograph39f3',
u'\u7af5': 'CjkUnifiedIdeograph7af5',
u'\u827a': 'CjkUnifiedIdeograph827a',
u'\ucf7c': 'HangulSyllableKwals',
u'\uabff': 'UnicodeCharacter0xabff',
u'\u087e': 'UnicodeCharacter0x87e',
u'\u3580': 'CjkUnifiedIdeograph3580',
u'\u9103': 'CjkUnifiedIdeograph9103',
u'\u7682': 'CjkUnifiedIdeograph7682',
u'\ud205': 'HangulSyllableToeb',
u'\u1f07': 'GreekSmallLetterAlphaWithDasiaAndPerispomeni',
u'\u5809': 'CjkUnifiedIdeograph5809',
u'\ua78c': 'LatinSmallLetterSaltillo',
u'\ue08e': 'UnicodeCharacter0xe08e',
u'\u2d90': 'EthiopicSyllableChoa',
u'\u8913': 'CjkUnifiedIdeograph8913',
u'\u6e92': 'CjkUnifiedIdeograph6e92',
u'\uca15': 'HangulSyllableJjyag',
u'\uf717': 'UnicodeCharacter0xf717',
u'\u3019': 'RightWhiteTortoiseShellBracket',
u'\u7d1b': 'CjkUnifiedIdeograph7d1b',
u'\u9f9c': 'CjkUnifiedIdeograph9f9c',
u'\ud89e': 'UnicodeCharacter0xd89e',
u'\u05a0': 'HebrewAccentTelishaGedola',
u'\u46a2': 'CjkUnifiedIdeograph46a2',
u'\ua225': 'YiSyllableGgop',
u'\uef27': 'UnicodeCharacter0xef27',
u'\u2829': 'BraillePatternDots146',
u'\u552b': 'CjkUnifiedIdeograph552b',
u'\ub0ae': 'HangulSyllableNaj',
u'\ufdb0': 'ArabicLigatureYehWithMeemWithYehFinalForm',
u'\u3eb2': 'CjkUnifiedIdeograph3eb2',
u'\u9a35': 'CjkUnifiedIdeograph9a35',
u'\u7bb4': 'CjkUnifiedIdeograph7bb4',
u'\uc737': 'HangulSyllableYuc',
u'\u4d3b': 'CjkUnifiedIdeograph4d3b',
u'\ua8be': 'SaurashtraVowelSignE',
u'\ud5c0': 'HangulSyllableHyaess',
u'\u16c2': 'RunicLetterE',
u'\u53c4': 'CjkUnifiedIdeograph53c4',
u'\ubf47': 'HangulSyllableBbod',
u'\uf849': 'UnicodeCharacter0xf849',
u'\u254b': 'BoxDrawingsHeavyVerticalAndHorizontal',
u'\u664d': 'CjkUnifiedIdeograph664d',
u'\u80ce': 'CjkUnifiedIdeograph80ce',
u'\ucdd0': 'HangulSyllableCwen',
u'\u0ed2': 'LaoDigitTwo',
u'\u4bd4': 'CjkUnifiedIdeograph4bd4',
u'\u9757': 'CjkUnifiedIdeograph9757',
u'\u74d6': 'CjkUnifiedIdeograph74d6',
u'\ud059': 'HangulSyllableKyulg',
u'\u1d5b': 'ModifierLetterSmallV',
u'\u5e5d': 'CjkUnifiedIdeograph5e5d',
u'\ua5e0': 'VaiSyllableNyo',
u'\ue6e2': 'UnicodeCharacter0xe6e2',
u'\u23e4': 'Straightness',
u'\u8f67': 'CjkUnifiedIdeograph8f67',
u'\u6ce6': 'CjkUnifiedIdeograph6ce6',
u'\uc869': 'HangulSyllableJyeng',
u'\uf56b': 'UnicodeCharacter0xf56b',
u'\u366d': 'CjkUnifiedIdeograph366d',
u'\u736f': 'CjkUnifiedIdeograph736f',
u'\u9df0': 'CjkUnifiedIdeograph9df0',
u'\udef2': 'UnicodeCharacter0xdef2',
u'\u1bf4': 'UnicodeCharacter0x1bf4',
u'\u44f6': 'CjkUnifiedIdeograph44f6',
u'\ua079': 'YiSyllableNbip',
u'\ued7b': 'UnicodeCharacter0xed7b',
u'\u2e7d': 'UnicodeCharacter0x2e7d',
u'\u6b7f': 'CjkUnifiedIdeograph6b7f',
u'\u1781': 'KhmerLetterKha',
u'\uf004': 'UnicodeCharacter0xf004',
u'\udc87': 'UnicodeCharacter0xdc87',
u'\u7a08': 'CjkUnifiedIdeograph7a08',
u'\u468b': 'CjkUnifiedIdeograph468b',
u'\ub90a': 'HangulSyllableRweonh',
u'\u858d': 'CjkUnifiedIdeograph858d',
u'\u230e': 'TopRightCrop',
u'\u0f91': 'TibetanSubjoinedLetterKha',
u'\ue814': 'UnicodeCharacter0xe814',
u'\ub497': 'HangulSyllableDwelh',
u'\u5218': 'CjkUnifiedIdeograph5218',
u'\u3e9b': 'CjkUnifiedIdeograph3e9b',
u'\u911a': 'CjkUnifiedIdeograph911a',
u'\u1b1e': 'BalineseLetterTaMurdaMahaprana',
u'\ue7a1': 'UnicodeCharacter0xe7a1',
u'\u61a5': 'CjkUnifiedIdeograph61a5',
u'\uc024': 'HangulSyllableBbwin',
u'\uaca7': 'HangulSyllableGeh',
u'\u4a28': 'CjkUnifiedIdeograph4a28',
u'\u16ab': 'RunicLetterAesc',
u'\u892a': 'CjkUnifiedIdeograph892a',
u'\uf32e': 'UnicodeCharacter0xf32e',
u'\udfb1': 'UnicodeCharacter0xdfb1',
u'\u7d32': 'CjkUnifiedIdeograph7d32',
u'\u59b5': 'CjkUnifiedIdeograph59b5',
u'\ub834': 'HangulSyllableRyeom',
u'\u84b7': 'CjkUnifiedIdeograph84b7',
u'\u2238': 'DotMinus',
u'\u0ebb': 'LaoVowelSignMaiKon',
u'\ueb3e': 'UnicodeCharacter0xeb3e',
u'\ub7c1': 'HangulSyllableRyalt',
u'\u5542': 'CjkUnifiedIdeograph5542',
u'\u31c5': 'CjkStrokeHzz',
u'\u9044': 'CjkUnifiedIdeograph9044',
u'\u1a48': 'UnicodeCharacter0x1a48',
u'\ue6cb': 'UnicodeCharacter0xe6cb',
u'\u60cf': 'CjkUnifiedIdeograph60cf',
u'\uc34e': 'HangulSyllableSsyaegg',
u'\uafd1': 'HangulSyllableGgut',
u'\u4d52': 'CjkUnifiedIdeograph4d52',
u'\u29d5': 'TimesWithRightHalfBlack',
u'\u8854': 'CjkUnifiedIdeograph8854',
u'\uf258': 'UnicodeCharacter0xf258',
u'\udedb': 'UnicodeCharacter0xdedb',
u'\u7c5c': 'CjkUnifiedIdeograph7c5c',
u'\u58df': 'CjkUnifiedIdeograph58df',
u'\ubb5e': 'HangulSyllableMweolp',
u'\u87e1': 'CjkUnifiedIdeograph87e1',
u'\u2562': 'BoxDrawingsVerticalDoubleAndLeftSingle',
u'\u01e5': 'LatinSmallLetterGWithStroke',
u'\uea68': 'UnicodeCharacter0xea68',
u'\ub6eb': 'HangulSyllableDdwec',
u'\u546c': 'CjkUnifiedIdeograph546c',
u'\u30ef': 'KatakanaLetterWa',
u'\u936e': 'CjkUnifiedIdeograph936e',
u'\u1d72': 'LatinSmallLetterRWithMiddleTilde',
u'\uf9f5': 'CjkCompatibilityIdeographf9f5',
u'\u63f9': 'CjkUnifiedIdeograph63f9',
u'\uc278': 'HangulSyllableSwils',
u'\uaefb': 'HangulSyllableGgyeod',
u'\u4c7c': 'CjkUnifiedIdeograph4c7c',
u'\u28ff': 'BraillePatternDots12345678',
u'\u8b7e': 'CjkUnifiedIdeograph8b7e',
u'\u1203': 'EthiopicSyllableHaa',
u'\uf582': 'UnicodeCharacter0xf582',
u'\u7f86': 'CjkUnifiedIdeograph7f86',
u'\udb09': 'UnicodeCharacter0xdb09',
u'\uba88': 'HangulSyllableMyeok',
u'\u450d': 'CjkUnifiedIdeograph450d',
u'\u248c': 'DigitFiveFullStop',
u'\u800f': 'CjkUnifiedIdeograph800f',
u'\u0a13': 'GurmukhiLetterOo',
u'\ued92': 'UnicodeCharacter0xed92',
u'\u7417': 'CjkUnifiedIdeograph7417',
u'\u5796': 'CjkUnifiedIdeograph5796',
u'\ub319': 'HangulSyllableDaet',
u'\u9298': 'CjkUnifiedIdeograph9298',
u'\u3d1d': 'CjkUnifiedIdeograph3d1d',
u'\u1c9c': 'UnicodeCharacter0x1c9c',
u'\ue223': 'UnicodeCharacter0xe223',
u'\uc5a2': 'HangulSyllableYaelm',
u'\u6c27': 'CjkUnifiedIdeograph6c27',
u'\u4fa6': 'CjkUnifiedIdeograph4fa6',
u'\uab29': 'UnicodeCharacter0xab29',
u'\u8aa8': 'CjkUnifiedIdeograph8aa8',
u'\u152d': 'CanadianSyllabicsYa',
u'\uf4ac': 'UnicodeCharacter0xf4ac',
u'\u7eb0': 'CjkUnifiedIdeograph7eb0',
u'\uda33': 'UnicodeCharacter0xda33',
u'\ubdb2': 'HangulSyllableBweoj',
u'\u4437': 'CjkUnifiedIdeograph4437',
u'\u27b6': 'BlackfeatheredNorthEastArrow',
u'\u8339': 'CjkUnifiedIdeograph8339',
u'\u0d3d': 'MalayalamSignAvagraha',
u'\uecbc': 'UnicodeCharacter0xecbc',
u'\u7741': 'CjkUnifiedIdeograph7741',
u'\u56c0': 'CjkUnifiedIdeograph56c0',
u'\ub243': 'HangulSyllableNwed',
u'\u95c2': 'CjkUnifiedIdeograph95c2',
u'\u3c47': 'CjkUnifiedIdeograph3c47',
u'\u1fc6': 'GreekSmallLetterEtaWithPerispomeni',
u'\ue54d': 'UnicodeCharacter0xe54d',
u'\uc4cc': 'HangulSyllableSswiss',
u'\u6f51': 'CjkUnifiedIdeograph6f51',
u'\u4ed0': 'CjkUnifiedIdeograph4ed0',
u'\uaa53': 'ChamDigitThree',
u'\u8dd2': 'CjkUnifiedIdeograph8dd2',
u'\u1457': 'CanadianSyllabicsTwe',
u'\uf7d6': 'UnicodeCharacter0xf7d6',
u'\u71da': 'CjkUnifiedIdeograph71da',
u'\udd5d': 'UnicodeCharacter0xdd5d',
u'\ubcdc': 'HangulSyllableByen',
u'\u4761': 'CjkUnifiedIdeograph4761',
u'\u26e0': 'UnicodeCharacter0x26e0',
u'\u8263': 'CjkUnifiedIdeograph8263',
u'\u0c67': 'TeluguDigitOne',
u'\uefe6': 'UnicodeCharacter0xefe6',
u'\u766b': 'CjkUnifiedIdeograph766b',
u'\u69ea': 'CjkUnifiedIdeograph69ea',
u'\ub56d': 'HangulSyllableDdyanj',
u'\u94ec': 'CjkUnifiedIdeograph94ec',
u'\u3f71': 'CjkUnifiedIdeograph3f71',
u'\u1ef0': 'LatinCapitalLetterUWithHornAndDotBelow',
u'\ue477': 'UnicodeCharacter0xe477',
u'\uc7f6': 'HangulSyllableJyaebs',
u'\u6e7b': 'CjkUnifiedIdeograph6e7b',
u'\u41fa': 'CjkUnifiedIdeograph41fa',
u'\uad7d': 'HangulSyllableGub',
u'\u8cfc': 'CjkUnifiedIdeograph8cfc',
u'\u9681': 'CjkUnifiedIdeograph9681',
u'\uf700': 'UnicodeCharacter0xf700',
u'\ud383': 'HangulSyllablePeod',
u'\u3402': 'CjkUnifiedIdeograph3402',
u'\u1085': 'MyanmarVowelSignShanEAbove',
u'\u7104': 'CjkUnifiedIdeograph7104',
u'\u5d87': 'CjkUnifiedIdeograph5d87',
u'\ua20e': 'YiSyllableKut',
u'\u8e91': 'CjkUnifiedIdeograph8e91',
u'\uef10': 'UnicodeCharacter0xef10',
u'\ucb93': 'HangulSyllableJjwes',
u'\u2c12': 'GlagoliticCapitalLetterPokoji',
u'\u0895': 'UnicodeCharacter0x895',
u'\u6914': 'CjkUnifiedIdeograph6914',
u'\u3597': 'CjkUnifiedIdeograph3597',
u'\u7299': 'CjkUnifiedIdeograph7299',
u'\u9a1e': 'CjkUnifiedIdeograph9a1e',
u'\uc720': 'HangulSyllableYu',
u'\ua3a3': 'YiSyllableNrurx',
u'\u0422': 'CyrillicCapitalLetterTe',
u'\ue0a5': 'UnicodeCharacter0xe0a5',
u'\u4124': 'CjkUnifiedIdeograph4124',
u'\u2da7': 'UnicodeCharacter0x2da7',
u'\u6aa9': 'CjkUnifiedIdeograph6aa9',
u'\ubf30': 'HangulSyllableBbyels',
u'\u9bb3': 'CjkUnifiedIdeograph9bb3',
u'\ufc32': 'ArabicLigatureFehWithYehIsolatedForm',
u'\ud8b5': 'UnicodeCharacter0xd8b5',
u'\u3934': 'CjkUnifiedIdeograph3934',
u'\u05b7': 'HebrewPointPatah',
u'\u6636': 'CjkUnifiedIdeograph6636',
u'\u42b9': 'CjkUnifiedIdeograph42b9',
u'\u9740': 'CjkUnifiedIdeograph9740',
u'\ud442': 'HangulSyllablePoegg',
u'\ub0c5': 'HangulSyllableNaeb',
u'\u1144': 'HangulChoseongIeungpieup',
u'\ufdc7': 'ArabicLigatureNoonWithJeemWithYehFinalForm',
u'\u5e46': 'CjkUnifiedIdeograph5e46',
u'\u3ac9': 'CjkUnifiedIdeograph3ac9',
u'\u67cb': 'CjkUnifiedIdeograph67cb',
u'\u8f50': 'CjkUnifiedIdeograph8f50',
u'\ucc52': 'HangulSyllableCaelp',
u'\ua8d5': 'SaurashtraDigitFive',
u'\u0954': 'DevanagariAcuteAccent',
u'\ud5d7': 'HangulSyllableHeolh',
u'\u3656': 'CjkUnifiedIdeograph3656',
u'\u12d9': 'EthiopicSyllableZu',
u'\u7358': 'CjkUnifiedIdeograph7358',
u'\u5fdb': 'CjkUnifiedIdeograph5fdb',
u'\ua462': 'YiSyllableXiex',
u'\u80e5': 'CjkUnifiedIdeograph80e5',
u'\ue164': 'UnicodeCharacter0xe164',
u'\ucde7': 'HangulSyllableCweh',
u'\u2e66': 'UnicodeCharacter0x2e66',
u'\u0ae9': 'GujaratiDigitThree',
u'\u6b68': 'CjkUnifiedIdeograph6b68',
u'\u37eb': 'CjkUnifiedIdeograph37eb',
u'\u74ed': 'CjkUnifiedIdeograph74ed',
u'\u9c72': 'CjkUnifiedIdeograph9c72',
u'\ud974': 'UnicodeCharacter0xd974',
u'\ua5f7': 'VaiSyllableLe',
u'\u0676': 'ArabicLetterHighHamzaWaw',
u'\ue2f9': 'UnicodeCharacter0xe2f9',
u'\u4378': 'CjkUnifiedIdeograph4378',
u'\u2ffb': 'IdeographicDescriptionCharacterOverlaid',
u'\u6cfd': 'CjkUnifiedIdeograph6cfd',
u'\u5601': 'CjkUnifiedIdeograph5601',
u'\ub184': 'HangulSyllableNols',
u'\ufe86': 'ArabicLetterWawWithHamzaAboveFinalForm',
u'\u3b88': 'CjkUnifiedIdeograph3b88',
u'\u870b': 'CjkUnifiedIdeograph870b',
u'\u788a': 'CjkUnifiedIdeograph788a',
u'\uc40d': 'HangulSyllableSswat',
u'\u010f': 'LatinSmallLetterDWithCaron',
u'\u4e11': 'CjkUnifiedIdeograph4e11',
u'\ua994': 'UnicodeCharacter0xa994',
u'\ud696': 'HangulSyllableHoelm',
u'\u1398': 'EthiopicTonalMarkDerethidet',
u'\u509a': 'CjkUnifiedIdeograph509a',
u'\ubc1d': 'HangulSyllableBalg',
u'\uf91f': 'CjkCompatibilityIdeographf91f',
u'\u2621': 'CautionSign',
u'\u6323': 'CjkUnifiedIdeograph6323',
u'\u81a4': 'CjkUnifiedIdeograph81a4',
u'\ucea6': 'HangulSyllableKaej',
u'\u0ba8': 'TamilLetterNa',
u'\u48aa': 'CjkUnifiedIdeograph48aa',
u'\u942d': 'CjkUnifiedIdeograph942d',
u'\u75ac': 'CjkUnifiedIdeograph75ac',
u'\ud12f': 'HangulSyllableTyaeh',
u'\u1e31': 'LatinSmallLetterKWithAcute',
u'\u5b33': 'CjkUnifiedIdeograph5b33',
u'\ua6b6': 'UnicodeCharacter0xa6b6',
u'\ue3b8': 'UnicodeCharacter0xe3b8',
u'\u20ba': 'UnicodeCharacter0x20ba',
u'\u8c3d': 'CjkUnifiedIdeograph8c3d',
u'\u6dbc': 'CjkUnifiedIdeograph6dbc',
u'\uc93f': 'HangulSyllableJwelb',
u'\uf641': 'UnicodeCharacter0xf641',
u'\u3343': 'SquareMaikuro',
u'\u7045': 'CjkUnifiedIdeograph7045',
u'\u9ec6': 'CjkUnifiedIdeograph9ec6',
u'\udbc8': 'UnicodeCharacter0xdbc8',
u'\u18ca': 'UnicodeCharacter0x18ca',
u'\u45cc': 'CjkUnifiedIdeograph45cc',
u'\ua14f': 'YiSyllableDdurx',
u'\uee51': 'UnicodeCharacter0xee51',
u'\u2b53': 'BlackRightpointingPentagon',
u'\u6855': 'CjkUnifiedIdeograph6855',
u'\ub3d8': 'HangulSyllableDoss',
u'\uf0da': 'UnicodeCharacter0xf0da',
u'\u3ddc': 'CjkUnifiedIdeograph3ddc',
u'\u995f': 'CjkUnifiedIdeograph995f',
u'\u7ade': 'CjkUnifiedIdeograph7ade',
u'\uc661': 'HangulSyllableWaenj',
u'\u0363': 'CombiningLatinSmallLetterA',
u'\u4065': 'CjkUnifiedIdeograph4065',
u'\uabe8': 'UnicodeCharacter0xabe8',
u'\ue8ea': 'UnicodeCharacter0xe8ea',
u'\u15ec': 'CanadianSyllabicsCarrierPi',
u'\u52ee': 'CjkUnifiedIdeograph52ee',
u'\ube71': 'HangulSyllableBbab',
u'\ufb73': 'ArabicLetterDyehFinalForm',
u'\u3875': 'CjkUnifiedIdeograph3875',
u'\u6577': 'CjkUnifiedIdeograph6577',
u'\u83f8': 'CjkUnifiedIdeograph83f8',
u'\uc0fa': 'HangulSyllableSyaj',
u'\u0dfc': 'UnicodeCharacter0xdfc',
u'\u4afe': 'CjkUnifiedIdeograph4afe',
u'\u7800': 'CjkUnifiedIdeograph7800',
u'\u5c83': 'CjkUnifiedIdeograph5c83',
u'\ubb02': 'HangulSyllableMoenh',
u'\u9f85': 'CjkUnifiedIdeograph9f85',
u'\u3106': 'BopomofoLetterP',
u'\u1589': 'CanadianSyllabicsTlha',
u'\uea0c': 'UnicodeCharacter0xea0c',
u'\uce8f': 'HangulSyllableKah',
u'\u6010': 'CjkUnifiedIdeograph6010',
u'\u4493': 'CjkUnifiedIdeograph4493',
u'\ua312': 'YiSyllableSa',
u'\u8795': 'CjkUnifiedIdeograph8795',
u'\u1916': 'LimbuLetterRa',
u'\ufd99': 'ArabicLigatureNoonWithJeemWithAlefMaksuraFinalForm',
u'\u739d': 'CjkUnifiedIdeograph739d',
u'\ud21c': 'HangulSyllableTyols',
u'\ub69f': 'HangulSyllableDdugs',
u'\u4820': 'CjkUnifiedIdeograph4820',
u'\u2ca3': 'CopticSmallLetterRo',
u'\u8b22': 'CjkUnifiedIdeograph8b22',
u'\u0126': 'LatinCapitalLetterHWithStroke',
u'\ue5a9': 'UnicodeCharacter0xe5a9',
u'\u5bad': 'CjkUnifiedIdeograph5bad',
u'\uba2c': 'HangulSyllableMyaem',
u'\u9eaf': 'CjkUnifiedIdeograph9eaf',
u'\u3030': 'WavyDash',
u'\u14b3': 'CanadianSyllabicsWestcreeMwo',
u'\ue936': 'UnicodeCharacter0xe936',
u'\ucdb9': 'HangulSyllableCweolg',
u'\u6f3a': 'CjkUnifiedIdeograph6f3a',
u'\u43bd': 'CjkUnifiedIdeograph43bd',
u'\ua23c': 'YiSyllableMgop',
u'\u86bf': 'CjkUnifiedIdeograph86bf',
u'\u1840': 'MongolianLetterLha',
u'\ufcc3': 'ArabicLigatureQafWithMeemInitialForm',
u'\u72c7': 'CjkUnifiedIdeograph72c7',
u'\ud146': 'HangulSyllableTeoj',
u'\ub5c9': 'HangulSyllableDdelt',
u'\u574a': 'CjkUnifiedIdeograph574a',
u'\u2bcd': 'UnicodeCharacter0x2bcd',
u'\u8a4c': 'CjkUnifiedIdeograph8a4c',
u'\ue4d3': 'UnicodeCharacter0xe4d3',
u'\u5ad7': 'CjkUnifiedIdeograph5ad7',
u'\ub956': 'HangulSyllableRwip',
u'\u9dd9': 'CjkUnifiedIdeograph9dd9',
u'\u3f5a': 'CjkUnifiedIdeograph3f5a',
u'\u13dd': 'CherokeeLetterTla',
u'\ue860': 'UnicodeCharacter0xe860',
u'\ucce3': 'HangulSyllableCyeos',
u'\u6e64': 'CjkUnifiedIdeograph6e64',
u'\u42e7': 'CjkUnifiedIdeograph42e7',
u'\ua166': 'YiSyllableNdurx',
u'\u85e9': 'CjkUnifiedIdeograph85e9',
u'\u276a': 'MediumFlattenedLeftParenthesisOrnament',
u'\ufbed': 'ArabicLigatureYehWithHamzaAboveWithAeFinalForm',
u'\u71f1': 'CjkUnifiedIdeograph71f1',
u'\ud070': 'HangulSyllableKeun',
u'\ub4f3': 'HangulSyllableDeuc',
u'\u5674': 'CjkUnifiedIdeograph5674',
u'\u2af7': 'TripleNestedLessthan',
u'\u8976': 'CjkUnifiedIdeograph8976',
u'\u0f7a': 'TibetanVowelSignE',
u'\ue3fd': 'UnicodeCharacter0xe3fd',
u'\ud901': 'UnicodeCharacter0xd901',
u'\ub880': 'HangulSyllableRwal',
u'\u5f05': 'CjkUnifiedIdeograph5f05',
u'\u3e84': 'CjkUnifiedIdeograph3e84',
u'\u9207': 'CjkUnifiedIdeograph9207',
u'\u080b': 'UnicodeCharacter0x80b',
u'\uf78a': 'UnicodeCharacter0xf78a',
u'\u6d8e': 'CjkUnifiedIdeograph6d8e',
u'\uc111': 'HangulSyllableSyaeb',
u'\ua090': 'YiSyllableNbyr',
u'\u4715': 'CjkUnifiedIdeograph4715',
u'\u2694': 'CrossedSwords',
u'\uf01b': 'UnicodeCharacter0xf01b',
u'\udf9a': 'UnicodeCharacter0xdf9a',
u'\u761f': 'CjkUnifiedIdeograph761f',
u'\u559e': 'CjkUnifiedIdeograph559e',
u'\ua921': 'KayahLiLetterCa',
u'\u88a0': 'CjkUnifiedIdeograph88a0',
u'\u2f25': 'KangxiRadicalWoman',
u'\u0ea4': 'UnicodeCharacter0xea4',
u'\ud82b': 'UnicodeCharacter0xd82b',
u'\uc7aa': 'HangulSyllableJap',
u'\u5e2f': 'CjkUnifiedIdeograph5e2f',
u'\u3dae': 'CjkUnifiedIdeograph3dae',
u'\u9131': 'CjkUnifiedIdeograph9131',
u'\u1735': 'PhilippineSinglePunctuation',
u'\uf6b4': 'UnicodeCharacter0xf6b4',
u'\u6cb8': 'CjkUnifiedIdeograph6cb8',
u'\uc03b': 'HangulSyllableBbwih',
u'\uafba': 'HangulSyllableGgugg',
u'\u463f': 'CjkUnifiedIdeograph463f',
u'\u25be': 'BlackDownpointingSmallTriangle',
u'\uff45': 'FullwidthLatinSmallLetterE',
u'\udec4': 'UnicodeCharacter0xdec4',
u'\u7549': 'CjkUnifiedIdeograph7549',
u'\u54c8': 'CjkUnifiedIdeograph54c8',
u'\ua84b': 'PhagspaLetterNa',
u'\u97ca': 'CjkUnifiedIdeograph97ca',
u'\u2e4f': 'UnicodeCharacter0x2e4f',
u'\u0dce': 'UnicodeCharacter0xdce',
u'\ue755': 'UnicodeCharacter0xe755',
u'\uc6d4': 'HangulSyllableWeol',
u'\u5d59': 'CjkUnifiedIdeograph5d59',
u'\u3cd8': 'CjkUnifiedIdeograph3cd8',
u'\u905b': 'CjkUnifiedIdeograph905b',
u'\u165f': 'CanadianSyllabicsCarrierTsi',
u'\uf5de': 'UnicodeCharacter0xf5de',
u'\u6be2': 'CjkUnifiedIdeograph6be2',
u'\ucf65': 'HangulSyllableKob',
u'\uaee4': 'HangulSyllableGgels',
u'\u4569': 'CjkUnifiedIdeograph4569',
u'\u24e8': 'CircledLatinSmallLetterY',
u'\ufe6f': 'UnicodeCharacter0xfe6f',
u'\uddee': 'UnicodeCharacter0xddee',
u'\u7473': 'CjkUnifiedIdeograph7473',
u'\u53f2': 'CjkUnifiedIdeograph53f2',
u'\ub775': 'HangulSyllableDding',
u'\u96f4': 'CjkUnifiedIdeograph96f4',
u'\u2d79': 'UnicodeCharacter0x2d79',
u'\u0cf8': 'UnicodeCharacter0xcf8',
u'\ue67f': 'UnicodeCharacter0xe67f',
u'\uc5fe': 'HangulSyllableYeobs',
u'\ub006': 'HangulSyllableGgwej',
u'\u9489': 'CjkUnifiedIdeograph9489',
u'\uf508': 'UnicodeCharacter0xf508',
u'\uc98b': 'HangulSyllableJeugs',
u'\u360a': 'CjkUnifiedIdeograph360a',
u'\u0a8d': 'GujaratiVowelCandraE',
u'\u6b0c': 'CjkUnifiedIdeograph6b0c',
u'\u4f8f': 'CjkUnifiedIdeograph4f8f',
u'\u9816': 'CjkUnifiedIdeograph9816',
u'\udd18': 'UnicodeCharacter0xdd18',
u'\ub19b': 'HangulSyllableNwad',
u'\u1e1a': 'LatinCapitalLetterEWithTildeBelow',
u'\uf29d': 'UnicodeCharacter0xf29d',
u'\u531c': 'CjkUnifiedIdeograph531c',
u'\u379f': 'CjkUnifiedIdeograph379f',
u'\u68a1': 'CjkUnifiedIdeograph68a1',
u'\u8026': 'CjkUnifiedIdeograph8026',
u'\uc528': 'HangulSyllableSsi',
u'\u99ab': 'CjkUnifiedIdeograph99ab',
u'\u062a': 'ArabicLetterTeh',
u'\udaad': 'UnicodeCharacter0xdaad',
u'\u3b2c': 'CjkUnifiedIdeograph3b2c',
u'\u1faf': 'GreekCapitalLetterOmegaWithDasiaAndPerispomeniAndProsgegrammeni',
u'\u7c2e': 'CjkUnifiedIdeograph7c2e',
u'\u50b1': 'CjkUnifiedIdeograph50b1',
u'\uad38': 'HangulSyllableGoen',
u'\u81bb': 'CjkUnifiedIdeograph81bb',
u'\uee3a': 'UnicodeCharacter0xee3a',
u'\uc2bd': 'HangulSyllableSeut',
u'\u233c': 'AplFunctionalSymbolQuadCircle',
u'\u07bf': 'UnicodeCharacter0x7bf',
u'\u643e': 'CjkUnifiedIdeograph643e',
u'\u38c1': 'CjkUnifiedIdeograph38c1',
u'\u7dc3': 'CjkUnifiedIdeograph7dc3',
u'\u9548': 'CjkUnifiedIdeograph9548',
u'\ud64a': 'HangulSyllableHobs',
u'\uaacd': 'UnicodeCharacter0xaacd',
u'\u0b4c': 'OriyaVowelSignAu',
u'\uefcf': 'UnicodeCharacter0xefcf',
u'\u4c4e': 'CjkUnifiedIdeograph4c4e',
u'\u20d1': 'CombiningRightHarpoonAbove',
u'\u65d3': 'CjkUnifiedIdeograph65d3',
u'\ube5a': 'HangulSyllableBij',
u'\u92dd': 'CjkUnifiedIdeograph92dd',
u'\uf35c': 'UnicodeCharacter0xf35c',
u'\ud7df': 'UnicodeCharacter0xd7df',
u'\u345e': 'CjkUnifiedIdeograph345e',
u'\u08e1': 'UnicodeCharacter0x8e1',
u'\u6960': 'CjkUnifiedIdeograph6960',
u'\u4de3': 'HexagramForDarkeningOfTheLight',
u'\ua66a': 'CyrillicCapitalLetterBinocularO',
u'\udb6c': 'UnicodeCharacter0xdb6c',
u'\ubfef': 'HangulSyllableBbweod',
u'\u1c6e': 'OlChikiLetterLe',
u'\uf0f1': 'UnicodeCharacter0xf0f1',
u'\u5170': 'CjkUnifiedIdeograph5170',
u'\u35f3': 'CjkUnifiedIdeograph35f3',
u'\u76f5': 'CjkUnifiedIdeograph76f5',
u'\u8e7a': 'CjkUnifiedIdeograph8e7a',
u'\uc37c': 'HangulSyllableSseoss',
u'\ua7ff': 'LatinEpigraphicLetterArchaicM',
u'\u047e': 'CyrillicCapitalLetterOt',
u'\u3980': 'CjkUnifiedIdeograph3980',
u'\u9d03': 'CjkUnifiedIdeograph9d03',
u'\u7a82': 'CjkUnifiedIdeograph7a82',
u'\ude05': 'UnicodeCharacter0xde05',
u'\u1307': 'EthiopicSyllableJwa',
u'\u5409': 'CjkUnifiedIdeograph5409',
u'\uab8c': 'UnicodeCharacter0xab8c',
u'\uec8e': 'UnicodeCharacter0xec8e',
u'\u2190': 'LeftwardsArrow',
u'\u8513': 'CjkUnifiedIdeograph8513',
u'\u6292': 'CjkUnifiedIdeograph6292',
u'\uc615': 'HangulSyllableYelt',
u'\ufb17': 'ArmenianSmallLigatureMenXeh',
u'\u3c19': 'CjkUnifiedIdeograph3c19',
u'\u711b': 'CjkUnifiedIdeograph711b',
u'\u939c': 'CjkUnifiedIdeograph939c',
u'\ud49e': 'HangulSyllablePweolm',
u'\u09a0': 'BengaliLetterTtha',
u'\u4aa2': 'CjkUnifiedIdeograph4aa2',
u'\uae25': 'HangulSyllableGyib',
u'\ue327': 'UnicodeCharacter0xe327',
u'\u2429': 'UnicodeCharacter0x2429',
u'\u592b': 'CjkUnifiedIdeograph592b',
u'\ubcae': 'HangulSyllableBelp',
u'\uf1b0': 'UnicodeCharacter0xf1b0',
u'\u32b2': 'CircledNumberThirtySeven',
u'\u9635': 'CjkUnifiedIdeograph9635',
u'\u77b4': 'CjkUnifiedIdeograph77b4',
u'\ucb37': 'HangulSyllableJjyolb',
u'\u0c39': 'TeluguLetterHa',
u'\u413b': 'CjkUnifiedIdeograph413b',
u'\ua4be': 'YiRadicalCip',
u'\ud9c0': 'UnicodeCharacter0xd9c0',
u'\u1ac2': 'UnicodeCharacter0x1ac2',
u'\u5fc4': 'CjkUnifiedIdeograph5fc4',
u'\ub347': 'HangulSyllableDyaelh',
u'\uf449': 'UnicodeCharacter0xf449',
u'\u294b': 'LeftBarbDownRightBarbUpHarpoon',
u'\u6a4d': 'CjkUnifiedIdeograph6a4d',
u'\u8cce': 'CjkUnifiedIdeograph8cce',
u'\uc1d0': 'HangulSyllableSwaels',
u'\u02d2': 'ModifierLetterCentredRightHalfRing',
u'\u47d4': 'CjkUnifiedIdeograph47d4',
u'\u9b57': 'CjkUnifiedIdeograph9b57',
u'\u78d6': 'CjkUnifiedIdeograph78d6',
u'\udc59': 'UnicodeCharacter0xdc59',
u'\u115b': 'UnicodeCharacter0x115b',
u'\u525d': 'CjkUnifiedIdeograph525d',
u'\ua9e0': 'UnicodeCharacter0xa9e0',
u'\ueae2': 'UnicodeCharacter0xeae2',
u'\u2fe4': 'UnicodeCharacter0x2fe4',
u'\u8367': 'CjkUnifiedIdeograph8367',
u'\u60e6': 'CjkUnifiedIdeograph60e6',
u'\uc469': 'HangulSyllableSsunj',
u'\uf96b': 'CjkCompatibilityIdeographf96b',
u'\u3a6d': 'CjkUnifiedIdeograph3a6d',
u'\u7f6f': 'CjkUnifiedIdeograph7f6f',
u'\u91f0': 'CjkUnifiedIdeograph91f0',
u'\ud2f2': 'HangulSyllableTigg',
u'\u17f4': 'KhmerSymbolLekAttakBuon',
u'\u48f6': 'CjkUnifiedIdeograph48f6',
u'\uac79': 'HangulSyllableGeolg',
u'\ue17b': 'UnicodeCharacter0xe17b',
u'\u227d': 'SucceedsOrEqualTo',
u'\u677f': 'CjkUnifiedIdeograph677f',
u'\u1b81': 'SundaneseSignPanglayar',
u'\ufc04': 'ArabicLigatureYehWithHamzaAboveWithYehIsolatedForm',
u'\ud087': 'HangulSyllableKeuh',
u'\u7608': 'CjkUnifiedIdeograph7608',
u'\u4a8b': 'CjkUnifiedIdeograph4a8b',
u'\ub50a': 'HangulSyllableDyibs',
u'\u898d': 'CjkUnifiedIdeograph898d',
u'\u2f0e': 'KangxiRadicalIce',
u'\u0391': 'GreekCapitalLetterAlpha',
u'\ue414': 'UnicodeCharacter0xe414',
u'\ub897': 'HangulSyllableRwaegs',
u'\u5e18': 'CjkUnifiedIdeograph5e18',
u'\u329b': 'CircledIdeographFemale',
u'\u9d1a': 'CjkUnifiedIdeograph9d1a',
u'\u171e': 'UnicodeCharacter0x171e',
u'\ueba1': 'UnicodeCharacter0xeba1',
u'\u6da5': 'CjkUnifiedIdeograph6da5',
u'\ucc24': 'HangulSyllableJjik',
u'\ua0a7': 'YiSyllableHmurx',
u'\u4628': 'CjkUnifiedIdeograph4628',
u'\u1aab': 'UnicodeCharacter0x1aab',
u'\u852a': 'CjkUnifiedIdeograph852a',
u'\uff2e': 'FullwidthLatinCapitalLetterN',
u'\ud3b1': 'HangulSyllablePet',
u'\u7132': 'CjkUnifiedIdeograph7132',
u'\u55b5': 'CjkUnifiedIdeograph55b5',
u'\ub434': 'HangulSyllableDyo',
u'\u88b7': 'CjkUnifiedIdeograph88b7',
u'\u2e38': 'UnicodeCharacter0x2e38',
u'\u02bb': 'ModifierLetterTurnedComma',
u'\ue73e': 'UnicodeCharacter0xe73e',
u'\ubbc1': 'HangulSyllableMeug',
u'\u5942': 'CjkUnifiedIdeograph5942',
u'\u3dc5': 'CjkUnifiedIdeograph3dc5',
u'\u9c44': 'CjkUnifiedIdeograph9c44',
u'\u1648': 'CanadianSyllabicsCarrierDzu',
u'\ueacb': 'UnicodeCharacter0xeacb',
u'\u6ccf': 'CjkUnifiedIdeograph6ccf',
u'\ucf4e': 'HangulSyllableKyej',
u'\ua3d1': 'YiSyllableRex',
u'\u4152': 'CjkUnifiedIdeograph4152',
u'\u25d5': 'CircleWithAllButUpperLeftQuadrantBlack',
u'\u8454': 'CjkUnifiedIdeograph8454',
u'\ufe58': 'SmallEmDash',
u'\ud2db': 'HangulSyllableTyid',
u'\u705c': 'CjkUnifiedIdeograph705c',
u'\u54df': 'CjkUnifiedIdeograph54df',
u'\ub75e': 'HangulSyllableDdyip',
u'\u8be1': 'CjkUnifiedIdeograph8be1',
u'\u2962': 'LeftwardsHarpoonWithBarbUpAboveLeftwardsHarpoonWithBarbDown',
u'\u0de5': 'UnicodeCharacter0xde5',
u'\ue668': 'UnicodeCharacter0xe668',
u'\ubaeb': 'HangulSyllableMwaelb',
u'\u586c': 'CjkUnifiedIdeograph586c',
u'\u3cef': 'CjkUnifiedIdeograph3cef',
u'\u9f6e': 'CjkUnifiedIdeograph9f6e',
u'\u1172': 'HangulJungseongYu',
u'\uf5f5': 'UnicodeCharacter0xf5f5',
u'\u6ff9': 'CjkUnifiedIdeograph6ff9',
u'\uce78': 'HangulSyllableKan',
u'\ua2fb': 'YiSyllableNzop',
u'\u407c': 'CjkUnifiedIdeograph407c',
u'\u24ff': 'NegativeCircledDigitZero',
u'\u877e': 'CjkUnifiedIdeograph877e',
u'\u1e03': 'LatinSmallLetterBWithDotAbove',
u'\uf982': 'CjkCompatibilityIdeographf982',
u'\u7386': 'CjkUnifiedIdeograph7386',
u'\ud709': 'HangulSyllableHwelt',
u'\ub688': 'HangulSyllableDdyol',
u'\u490d': 'CjkUnifiedIdeograph490d',
u'\u288c': 'BraillePatternDots348',
u'\u8c0f': 'CjkUnifiedIdeograph8c0f',
u'\u0613': 'ArabicSignRadiAllahouAnhu',
u'\ue192': 'UnicodeCharacter0xe192',
u'\u7817': 'CjkUnifiedIdeograph7817',
u'\u5b96': 'CjkUnifiedIdeograph5b96',
u'\ubf19': 'HangulSyllableBbyeob',
u'\u9e98': 'CjkUnifiedIdeograph9e98',
u'\u311d': 'BopomofoLetterEh',
u'\u109c': 'UnicodeCharacter0x109c',
u'\uee23': 'UnicodeCharacter0xee23',
u'\uc9a2': 'HangulSyllableJeup',
u'\u6027': 'CjkUnifiedIdeograph6027',
u'\u43a6': 'CjkUnifiedIdeograph43a6',
u'\ua729': 'LatinSmallLetterTz',
u'\u86a8': 'CjkUnifiedIdeograph86a8',
u'\u192d': 'UnicodeCharacter0x192d',
u'\uf8ac': 'UnicodeCharacter0xf8ac',
u'\u72b0': 'CjkUnifiedIdeograph72b0',
u'\ud633': 'HangulSyllableHyec',
u'\ub1b2': 'HangulSyllableNwaegg',
u'\u4837': 'CjkUnifiedIdeograph4837',
u'\u2bb6': 'UnicodeCharacter0x2bb6',
u'\u8f39': 'CjkUnifiedIdeograph8f39',
u'\u013d': 'LatinCapitalLetterLWithCaron',
u'\ue0bc': 'UnicodeCharacter0xe0bc',
u'\u7b41': 'CjkUnifiedIdeograph7b41',
u'\u5ac0': 'CjkUnifiedIdeograph5ac0',
u'\ube43': 'HangulSyllableByih',
u'\u99c2': 'CjkUnifiedIdeograph99c2',
u'\u3047': 'HiraganaLetterSmallE',
u'\u13c6': 'CherokeeLetterQua',
u'\ue94d': 'UnicodeCharacter0xe94d',
u'\uc8cc': 'HangulSyllableJoel',
u'\u6351': 'CjkUnifiedIdeograph6351',
u'\u42d0': 'CjkUnifiedIdeograph42d0',
u'\ua653': 'CyrillicSmallLetterIotifiedYat',
u'\u81d2': 'CjkUnifiedIdeograph81d2',
u'\u1857': 'MongolianLetterTodoKa',
u'\ufbd6': 'ArabicLetterNgMedialForm',
u'\u7dda': 'CjkUnifiedIdeograph7dda',
u'\ud15d': 'HangulSyllableTeb',
u'\ub0dc': 'HangulSyllableNyals',
u'\u4b61': 'CjkUnifiedIdeograph4b61',
u'\u2ae0': 'ShortUpTack',
u'\u8e63': 'CjkUnifiedIdeograph8e63',
u'\ue3e6': 'UnicodeCharacter0xe3e6',
u'\u7a6b': 'CjkUnifiedIdeograph7a6b',
u'\u65ea': 'CjkUnifiedIdeograph65ea',
u'\ub96d': 'HangulSyllableRyung',
u'\u98ec': 'CjkUnifiedIdeograph98ec',
u'\u3371': 'SquareHpa',
u'\u12f0': 'EthiopicSyllableDa',
u'\ue877': 'UnicodeCharacter0xe877',
u'\ucbf6': 'HangulSyllableJjyinh',
u'\u627b': 'CjkUnifiedIdeograph627b',
u'\u4dfa': 'HexagramForDispersion',
u'\ua17d': 'YiSyllableNit',
u'\u80fc': 'CjkUnifiedIdeograph80fc',
u'\u9a81': 'CjkUnifiedIdeograph9a81',
u'\ufb00': 'LatinSmallLigatureFf',
u'\udf83': 'UnicodeCharacter0xdf83',
u'\u3802': 'CjkUnifiedIdeograph3802',
u'\u1c85': 'UnicodeCharacter0x1c85',
u'\u7d04': 'CjkUnifiedIdeograph7d04',
u'\u5187': 'CjkUnifiedIdeograph5187',
u'\uae0e': 'HangulSyllableGeuj',
u'\u8291': 'CjkUnifiedIdeograph8291',
u'\ue310': 'UnicodeCharacter0xe310',
u'\uc793': 'HangulSyllableJags',
u'\u2012': 'FigureDash',
u'\u0495': 'CyrillicSmallLetterGheWithMiddleHook',
u'\u6514': 'CjkUnifiedIdeograph6514',
u'\u3997': 'CjkUnifiedIdeograph3997',
u'\u7e99': 'CjkUnifiedIdeograph7e99',
u'\u961e': 'CjkUnifiedIdeograph961e',
u'\ucb20': 'HangulSyllableJjoem',
u'\uafa3': 'HangulSyllableGgyod',
u'\u0822': 'UnicodeCharacter0x822',
u'\ueca5': 'UnicodeCharacter0xeca5',
u'\u4d24': 'CjkUnifiedIdeograph4d24',
u'\u21a7': 'DownwardsArrowFromBar',
u'\u66a9': 'CjkUnifiedIdeograph66a9',
u'\ub330': 'HangulSyllableDyass',
u'\u97b3': 'CjkUnifiedIdeograph97b3',
u'\uf032': 'UnicodeCharacter0xf032',
u'\ud4b5': 'HangulSyllablePwenj',
u'\u3534': 'CjkUnifiedIdeograph3534',
u'\u09b7': 'BengaliLetterSsa',
u'\u6a36': 'CjkUnifiedIdeograph6a36',
u'\u4eb9': 'CjkUnifiedIdeograph4eb9',
u'\u9b40': 'CjkUnifiedIdeograph9b40',
u'\ud842': 'UnicodeCharacter0xd842',
u'\ubcc5': 'HangulSyllableByeolg',
u'\u1d44': 'ModifierLetterSmallTurnedA',
u'\uf1c7': 'UnicodeCharacter0xf1c7',
u'\u5246': 'CjkUnifiedIdeograph5246',
u'\u36c9': 'CjkUnifiedIdeograph36c9',
u'\u6bcb': 'CjkUnifiedIdeograph6bcb',
u'\u8350': 'CjkUnifiedIdeograph8350',
u'\uc052': 'HangulSyllableBbyuj',
u'\ua4d5': 'UnicodeCharacter0xa4d5',
u'\u0554': 'ArmenianCapitalLetterKeh',
u'\ud9d7': 'UnicodeCharacter0xd9d7',
u'\u3a56': 'CjkUnifiedIdeograph3a56',
u'\u1ed9': 'LatinSmallLetterOWithCircumflexAndDotBelow',
u'\u7f58': 'CjkUnifiedIdeograph7f58',
u'\u53db': 'CjkUnifiedIdeograph53db',
u'\ua862': 'PhagspaLetterQa',
u'\u8ce5': 'CjkUnifiedIdeograph8ce5',
u'\ued64': 'UnicodeCharacter0xed64',
u'\uc1e7': 'HangulSyllableSoed',
u'\u2266': 'LessthanOverEqualTo',
u'\u06e9': 'ArabicPlaceOfSajdah',
u'\u6768': 'CjkUnifiedIdeograph6768',
u'\u3beb': 'CjkUnifiedIdeograph3beb',
u'\u78ed': 'CjkUnifiedIdeograph78ed',
u'\u9072': 'CjkUnifiedIdeograph9072',
u'\ud574': 'HangulSyllableHae',
u'\ua9f7': 'UnicodeCharacter0xa9f7',
u'\u0a76': 'UnicodeCharacter0xa76',
u'\ueef9': 'UnicodeCharacter0xeef9',
u'\u4f78': 'CjkUnifiedIdeograph4f78',
u'\u23fb': 'UnicodeCharacter0x23fb',
u'\u60fd': 'CjkUnifiedIdeograph60fd',
u'\u5a01': 'CjkUnifiedIdeograph5a01',
u'\ubd84': 'HangulSyllableBun',
u'\uf286': 'UnicodeCharacter0xf286',
u'\u3788': 'CjkUnifiedIdeograph3788',
u'\u8b0b': 'CjkUnifiedIdeograph8b0b',
u'\u748a': 'CjkUnifiedIdeograph748a',
u'\uc80d': 'HangulSyllableJeolt',
u'\u0d0f': 'MalayalamLetterEe',
u'\u4211': 'CjkUnifiedIdeograph4211',
u'\ua594': 'VaiSyllableNyoo',
u'\uda96': 'UnicodeCharacter0xda96',
u'\u1f98': 'GreekCapitalLetterEtaWithPsiliAndProsgegrammeni',
u'\u5c9a': 'CjkUnifiedIdeograph5c9a',
u'\ub01d': 'HangulSyllableGgwib',
u'\uf51f': 'UnicodeCharacter0xf51f',
u'\u2a21': 'ZNotationSchemaProjection',
u'\u6f23': 'CjkUnifiedIdeograph6f23',
u'\u8da4': 'CjkUnifiedIdeograph8da4',
u'\uc2a6': 'HangulSyllableSeugg',
u'\u07a8': 'ThaanaIbifili',
u'\u44aa': 'CjkUnifiedIdeograph44aa',
u'\u982d': 'CjkUnifiedIdeograph982d',
u'\u79ac': 'CjkUnifiedIdeograph79ac',
u'\udd2f': 'UnicodeCharacter0xdd2f',
u'\u1231': 'EthiopicSyllableSu',
u'\u5733': 'CjkUnifiedIdeograph5733',
u'\uaab6': 'UnicodeCharacter0xaab6',
u'\uefb8': 'UnicodeCharacter0xefb8',
u'\u2cba': 'CopticCapitalLetterDialectpNi',
u'\u803d': 'CjkUnifiedIdeograph803d',
u'\u61bc': 'CjkUnifiedIdeograph61bc',
u'\uc53f': 'HangulSyllableSsic',
u'\ufa41': 'CjkCompatibilityIdeographfa41',
u'\u3f43': 'CjkUnifiedIdeograph3f43',
u'\u7c45': 'CjkUnifiedIdeograph7c45',
u'\u92c6': 'CjkUnifiedIdeograph92c6',
u'\ud7c8': 'UnicodeCharacter0xd7c8',
u'\u14ca': 'CanadianSyllabicsWestcreeNwe',
u'\u49cc': 'CjkUnifiedIdeograph49cc',
u'\uad4f': 'HangulSyllableGoeh',
u'\ue251': 'UnicodeCharacter0xe251',
u'\u2753': 'UnicodeCharacter0x2753',
u'\u6455': 'CjkUnifiedIdeograph6455',
u'\ubfd8': 'HangulSyllableBbuls',
u'\ufcda': 'ArabicLigatureYehWithJeemInitialForm',
u'\u31dc': 'CjkStrokePz',
u'\u955f': 'CjkUnifiedIdeograph955f',
u'\u76de': 'CjkUnifiedIdeograph76de',
u'\uca61': 'HangulSyllableJjeong',
u'\u0f63': 'TibetanLetterLa',
u'\u4c65': 'CjkUnifiedIdeograph4c65',
u'\ua7e8': 'UnicodeCharacter0xa7e8',
u'\ue4ea': 'UnicodeCharacter0xe4ea',
u'\u19ec': 'KhmerSymbolDappiiKoet',
u'\u5eee': 'CjkUnifiedIdeograph5eee',
u'\ub271': 'HangulSyllableNwit',
u'\uf773': 'UnicodeCharacter0xf773',
u'\u3475': 'CjkUnifiedIdeograph3475',
u'\u6977': 'CjkUnifiedIdeograph6977',
u'\u8ff8': 'CjkUnifiedIdeograph8ff8',
u'\uccfa': 'HangulSyllableCyelp',
u'\u01fc': 'LatinCapitalLetterAeWithAcute',
u'\u46fe': 'CjkUnifiedIdeograph46fe',
u'\u7c00': 'CjkUnifiedIdeograph7c00',
u'\u5883': 'CjkUnifiedIdeograph5883',
u'\ubf02': 'HangulSyllableBbej',
u'\u9b85': 'CjkUnifiedIdeograph9b85',
u'\u3506': 'CjkUnifiedIdeograph3506',
u'\u1189': 'HangulJungseongUa',
u'\uee0c': 'UnicodeCharacter0xee0c',
u'\uca8f': 'HangulSyllableJjyeolb',
u'\u6410': 'CjkUnifiedIdeograph6410',
u'\u4093': 'CjkUnifiedIdeograph4093',
u'\ua712': 'ModifierLetterExtrahighLeftstemToneBar',
u'\u8395': 'CjkUnifiedIdeograph8395',
u'\u1d16': 'LatinSmallLetterTopHalfO',
u'\uf999': 'CjkCompatibilityIdeographf999',
u'\u779d': 'CjkUnifiedIdeograph779d',
u'\ud61c': 'HangulSyllableHye',
u'\ub29f': 'HangulSyllableNeulh',
u'\u4c20': 'CjkUnifiedIdeograph4c20',
u'\u28a3': 'BraillePatternDots1268',
u'\u8f22': 'CjkUnifiedIdeograph8f22',
u'\u0526': 'UnicodeCharacter0x526',
u'\ue1a9': 'UnicodeCharacter0xe1a9',
u'\u5fad': 'CjkUnifiedIdeograph5fad',
u'\ube2c': 'HangulSyllableByin',
u'\u9aaf': 'CjkUnifiedIdeograph9aaf',
u'\u3430': 'CjkUnifiedIdeograph3430',
u'\u10b3': 'GeorgianCapitalLetterUn',
u'\ued36': 'UnicodeCharacter0xed36',
u'\uc9b9': 'HangulSyllableJying',
u'\u6b3a': 'CjkUnifiedIdeograph6b3a',
u'\u47bd': 'CjkUnifiedIdeograph47bd',
u'\ua63c': 'UnicodeCharacter0xa63c',
u'\u82bf': 'CjkUnifiedIdeograph82bf',
u'\u1c40': 'LepchaDigitZero',
u'\uf8c3': 'UnicodeCharacter0xf8c3',
u'\u76c7': 'CjkUnifiedIdeograph76c7',
u'\ud546': 'HangulSyllablePilm',
u'\ub1c9': 'HangulSyllableNwaet',
u'\u534a': 'CjkUnifiedIdeograph534a',
u'\u2fcd': 'KangxiRadicalTripod',
u'\u8e4c': 'CjkUnifiedIdeograph8e4c',
u'\u0450': 'CyrillicSmallLetterIeWithGrave',
u'\ue0d3': 'UnicodeCharacter0xe0d3',
u'\u5ed7': 'CjkUnifiedIdeograph5ed7',
u'\ubd56': 'HangulSyllableBoelp',
u'\u99d9': 'CjkUnifiedIdeograph99d9',
u'\u3b5a': 'CjkUnifiedIdeograph3b5a',
u'\u17dd': 'KhmerSignAtthacan',
u'\uec60': 'UnicodeCharacter0xec60',
u'\uc8e3': 'HangulSyllableJyogs',
u'\u6a64': 'CjkUnifiedIdeograph6a64',
u'\u46e7': 'CjkUnifiedIdeograph46e7',
u'\ua566': 'VaiSyllableCa',
u'\u81e9': 'CjkUnifiedIdeograph81e9',
u'\u236a': 'AplFunctionalSymbolCommaBar',
u'\uffed': 'HalfwidthBlackSquare',
u'\u75f1': 'CjkUnifiedIdeograph75f1',
u'\ud470': 'HangulSyllablePyoss',
u'\ub0f3': 'HangulSyllableNyaed',
u'\u5274': 'CjkUnifiedIdeograph5274',
u'\u2ef7': 'UnicodeCharacter0x2ef7',
u'\u8d76': 'CjkUnifiedIdeograph8d76',
u'\u0b7a': 'UnicodeCharacter0xb7a',
u'\ue7fd': 'UnicodeCharacter0xe7fd',
u'\udd01': 'UnicodeCharacter0xdd01',
u'\ubc80': 'HangulSyllableByaek',
u'\u5b05': 'CjkUnifiedIdeograph5b05',
u'\u3a84': 'CjkUnifiedIdeograph3a84',
u'\u9607': 'CjkUnifiedIdeograph9607',
u'\u0c0b': 'TeluguLetterVocalicR',
u'\uf38a': 'UnicodeCharacter0xf38a',
u'\u698e': 'CjkUnifiedIdeograph698e',
u'\uc511': 'HangulSyllableSsyinj',
u'\ua490': 'YiRadicalQot',
u'\u4315': 'CjkUnifiedIdeograph4315',
u'\u2294': 'SquareCup',
u'\uf41b': 'UnicodeCharacter0xf41b',
u'\udb9a': 'UnicodeCharacter0xdb9a',
u'\u721f': 'CjkUnifiedIdeograph721f',
u'\u519e': 'CjkUnifiedIdeograph519e',
u'\uad21': 'HangulSyllableGwaelg',
u'\u8ca0': 'CjkUnifiedIdeograph8ca0',
u'\u2b25': 'BlackMediumDiamond',
u'\u0aa4': 'GujaratiLetterTa',
u'\udc2b': 'UnicodeCharacter0xdc2b',
u'\uc3aa': 'HangulSyllableSsyeolm',
u'\u5a2f': 'CjkUnifiedIdeograph5a2f',
u'\u39ae': 'CjkUnifiedIdeograph39ae',
u'\u9531': 'CjkUnifiedIdeograph9531',
u'\u1335': 'EthiopicSyllablePhe',
u'\uf2b4': 'UnicodeCharacter0xf2b4',
u'\u68b8': 'CjkUnifiedIdeograph68b8',
u'\uc43b': 'HangulSyllableSsoelh',
u'\uabba': 'UnicodeCharacter0xabba',
u'\u423f': 'CjkUnifiedIdeograph423f',
u'\u21be': 'UpwardsHarpoonWithBarbRightwards',
u'\ufb45': 'UnicodeCharacter0xfb45',
u'\udac4': 'UnicodeCharacter0xdac4',
u'\u7149': 'CjkUnifiedIdeograph7149',
u'\u50c8': 'CjkUnifiedIdeograph50c8',
u'\uac4b': 'HangulSyllableGyas',
u'\u93ca': 'CjkUnifiedIdeograph93ca',
u'\u2a4f': 'DoubleSquareUnion',
u'\u09ce': 'BengaliLetterKhandaTa',
u'\ue355': 'UnicodeCharacter0xe355',
u'\uc2d4': 'HangulSyllableSyiss',
u'\u5959': 'CjkUnifiedIdeograph5959',
u'\u38d8': 'CjkUnifiedIdeograph38d8',
u'\u945b': 'CjkUnifiedIdeograph945b',
u'\u125f': 'UnicodeCharacter0x125f',
u'\uf1de': 'UnicodeCharacter0xf1de',
u'\u6fe2': 'CjkUnifiedIdeograph6fe2',
u'\ucb65': 'HangulSyllableJjweog',
u'\uaae4': 'UnicodeCharacter0xaae4',
u'\u4169': 'CjkUnifiedIdeograph4169',
u'\u20e8': 'CombiningTripleUnderdot',
u'\ufa6f': 'UnicodeCharacter0xfa6f',
u'\ud9ee': 'UnicodeCharacter0xd9ee',
u'\u7073': 'CjkUnifiedIdeograph7073',
u'\u57f2': 'CjkUnifiedIdeograph57f2',
u'\ub375': 'HangulSyllableDenj',
u'\u92f4': 'CjkUnifiedIdeograph92f4',
u'\u2979': 'SubsetAboveRightwardsArrow',
u'\u08f8': 'UnicodeCharacter0x8f8',
u'\ue27f': 'UnicodeCharacter0xe27f',
u'\uc1fe': 'HangulSyllableSyogg',
u'\ub406': 'HangulSyllableDwaelm',
u'\u9089': 'CjkUnifiedIdeograph9089',
u'\uf108': 'UnicodeCharacter0xf108',
u'\ucd8b': 'HangulSyllableCyos',
u'\u320a': 'ParenthesizedHangulKhieukh',
u'\u0e8d': 'LaoLetterNyo',
u'\u6f0c': 'CjkUnifiedIdeograph6f0c',
u'\u4b8f': 'CjkUnifiedIdeograph4b8f',
u'\u9c16': 'CjkUnifiedIdeograph9c16',
u'\ud918': 'UnicodeCharacter0xd918',
u'\ub59b': 'HangulSyllableDdyaec',
u'\u1a1a': 'BugineseVowelSignO',
u'\uf69d': 'UnicodeCharacter0xf69d',
u'\u571c': 'CjkUnifiedIdeograph571c',
u'\u339f': 'SquareMmSquared',
u'\u6ca1': 'CjkUnifiedIdeograph6ca1',
u'\u8426': 'CjkUnifiedIdeograph8426',
u'\uc128': 'HangulSyllableSeols',
u'\u9dab': 'CjkUnifiedIdeograph9dab',
u'\u022a': 'LatinCapitalLetterOWithDiaeresisAndMacron',
u'\udead': 'UnicodeCharacter0xdead',
u'\u3f2c': 'CjkUnifiedIdeograph3f2c',
u'\u1baf': 'SundaneseLetterSya',
u'\u782e': 'CjkUnifiedIdeograph782e',
u'\u54b1': 'CjkUnifiedIdeograph54b1',
u'\ua938': 'RejangLetterMa',
u'\u85bb': 'CjkUnifiedIdeograph85bb',
u'\uea3a': 'UnicodeCharacter0xea3a',
u'\uc6bd': 'HangulSyllableUlt',
u'\u273c': 'OpenCentreTeardropspokedAsterisk',
u'\u03bf': 'GreekSmallLetterOmicron',
u'\u603e': 'CjkUnifiedIdeograph603e',
u'\u3cc1': 'CjkUnifiedIdeograph3cc1',
u'\u79c3': 'CjkUnifiedIdeograph79c3',
u'\u9148': 'CjkUnifiedIdeograph9148',
u'\ud24a': 'HangulSyllableTweogg',
u'\uaecd': 'HangulSyllableGgeob',
u'\u0f4c': 'TibetanLetterDda',
u'\uebcf': 'UnicodeCharacter0xebcf',
u'\u484e': 'CjkUnifiedIdeograph484e',
u'\u24d1': 'CircledLatinSmallLetterB',
u'\u61d3': 'CjkUnifiedIdeograph61d3',
u'\uba5a': 'HangulSyllableMenh',
u'\u96dd': 'CjkUnifiedIdeograph96dd',
u'\uf75c': 'UnicodeCharacter0xf75c',
u'\ud3df': 'HangulSyllablePyelh',
u'\u305e': 'HiraganaLetterZo',
u'\u0ce1': 'KannadaLetterVocalicLl',
u'\u6d60': 'CjkUnifiedIdeograph6d60',
u'\u49e3': 'CjkUnifiedIdeograph49e3',
u'\ua26a': 'YiSyllableNgop',
u'\udf6c': 'UnicodeCharacter0xdf6c',
u'\ubbef': 'HangulSyllableMyis',
u'\u186e': 'MongolianLetterSibeTsa',
u'\uf4f1': 'UnicodeCharacter0xf4f1',
u'\u5570': 'CjkUnifiedIdeograph5570',
u'\u31f3': 'KatakanaLetterSmallTo',
u'\u72f5': 'CjkUnifiedIdeograph72f5',
u'\u8a7a': 'CjkUnifiedIdeograph8a7a',
u'\uc77c': 'HangulSyllableIl',
u'\ua3ff': 'YiSyllableQip',
u'\u3d80': 'CjkUnifiedIdeograph3d80',
u'\u9903': 'CjkUnifiedIdeograph9903',
u'\u7e82': 'CjkUnifiedIdeograph7e82',
u'\uda05': 'UnicodeCharacter0xda05',
u'\u1707': 'TagalogLetterDa',
u'\u5009': 'CjkUnifiedIdeograph5009',
u'\uaf8c': 'HangulSyllableGgoels',
u'\ue88e': 'UnicodeCharacter0xe88e',
u'\u2590': 'RightHalfBlock',
u'\u8113': 'CjkUnifiedIdeograph8113',
u'\u6692': 'CjkUnifiedIdeograph6692',
u'\uc215': 'HangulSyllableSyot',
u'\uff17': 'FullwidthDigitSeven',
u'\u3819': 'CjkUnifiedIdeograph3819',
u'\u751b': 'CjkUnifiedIdeograph751b',
u'\u979c': 'CjkUnifiedIdeograph979c',
u'\ud09e': 'HangulSyllableKyij',
u'\u0da0': 'SinhalaLetterAlpapraanaCayanna',
u'\u4ea2': 'CjkUnifiedIdeograph4ea2',
u'\uaa25': 'ChamLetterVa',
u'\ue727': 'UnicodeCharacter0xe727',
u'\u2029': 'ParagraphSeparator',
u'\u5d2b': 'CjkUnifiedIdeograph5d2b',
u'\ub8ae': 'HangulSyllableRwaep',
u'\uf5b0': 'UnicodeCharacter0xf5b0',
u'\u36b2': 'CjkUnifiedIdeograph36b2',
u'\u9235': 'CjkUnifiedIdeograph9235',
u'\u73b4': 'CjkUnifiedIdeograph73b4',
u'\ucf37': 'HangulSyllableKyeoh',
u'\u0839': 'UnicodeCharacter0x839',
u'\u453b': 'CjkUnifiedIdeograph453b',
u'\ua0be': 'YiSyllableMox',
u'\uddc0': 'UnicodeCharacter0xddc0',
u'\u1ec2': 'LatinCapitalLetterEWithCircumflexAndHookAbove',
u'\u5bc4': 'CjkUnifiedIdeograph5bc4',
u'\ub747': 'HangulSyllableDdyigs',
u'\uf049': 'UnicodeCharacter0xf049',
u'\u2d4b': 'TifinaghLetterAhaggarYazh',
u'\u6e4d': 'CjkUnifiedIdeograph6e4d',
u'\u88ce': 'CjkUnifiedIdeograph88ce',
u'\uc5d0': 'HangulSyllableE',
u'\u06d2': 'ArabicLetterYehBarree',
u'\u43d4': 'CjkUnifiedIdeograph43d4',
u'\u9f57': 'CjkUnifiedIdeograph9f57',
u'\u7cd6': 'CjkUnifiedIdeograph7cd6',
u'\ud859': 'UnicodeCharacter0xd859',
u'\u155b': 'CanadianSyllabicsFwaa',
u'\u565d': 'CjkUnifiedIdeograph565d',
u'\uade0': 'HangulSyllableGyun',
u'\ueee2': 'UnicodeCharacter0xeee2',
u'\u2be4': 'UnicodeCharacter0x2be4',
u'\u8767': 'CjkUnifiedIdeograph8767',
u'\u64e6': 'CjkUnifiedIdeograph64e6',
u'\uc069': 'HangulSyllableBbeub',
u'\ufd6b': 'ArabicLigatureSheenWithMeemWithKhahInitialForm',
u'\u3e6d': 'CjkUnifiedIdeograph3e6d',
u'\u7b6f': 'CjkUnifiedIdeograph7b6f',
u'\u95f0': 'CjkUnifiedIdeograph95f0',
u'\ud6f2': 'HangulSyllableHweobs',
u'\u13f4': 'CherokeeLetterYv',
u'\u4cf6': 'CjkUnifiedIdeograph4cf6',
u'\ua879': 'UnicodeCharacter0xa879',
u'\ue57b': 'UnicodeCharacter0xe57b',
u'\u267d': 'PartiallyrecycledPaperSymbol',
u'\u637f': 'CjkUnifiedIdeograph637f',
u'\u1f81': 'GreekSmallLetterAlphaWithDasiaAndYpogegrammeni',
u'\uf804': 'UnicodeCharacter0xf804',
u'\ud487': 'HangulSyllablePulh',
u'\u7208': 'CjkUnifiedIdeograph7208',
u'\u4e8b': 'CjkUnifiedIdeograph4e8b',
u'\ub10a': 'HangulSyllableNeogg',
u'\u8d8d': 'CjkUnifiedIdeograph8d8d',
u'\u2b0e': 'RightwardsArrowWithTipDownwards',
u'\u0791': 'ThaanaLetterDaviyani',
u'\ue014': 'UnicodeCharacter0xe014',
u'\ubc97': 'HangulSyllableBeos',
u'\u5a18': 'CjkUnifiedIdeograph5a18',
u'\u369b': 'CjkUnifiedIdeograph369b',
u'\u991a': 'CjkUnifiedIdeograph991a',
u'\u131e': 'EthiopicSyllableGgo',
u'\uefa1': 'UnicodeCharacter0xefa1',
u'\u69a5': 'CjkUnifiedIdeograph69a5',
u'\uc824': 'HangulSyllableJel',
u'\ua4a7': 'YiRadicalNyop',
u'\u4228': 'CjkUnifiedIdeograph4228',
u'\u1eab': 'LatinSmallLetterAWithCircumflexAndTilde',
u'\u812a': 'CjkUnifiedIdeograph812a',
u'\ufb2e': 'HebrewLetterAlefWithPatah',
u'\ud7b1': 'UnicodeCharacter0xd7b1',
u'\u7532': 'CjkUnifiedIdeograph7532',
u'\u51b5': 'CjkUnifiedIdeograph51b5',
u'\ub034': 'HangulSyllableGgyuls',
u'\u8cb7': 'CjkUnifiedIdeograph8cb7',
u'\u2a38': 'CircledDivisionSign',
u'\u06bb': 'ArabicLetterRnoon',
u'\ue33e': 'UnicodeCharacter0xe33e',
u'\ubfc1': 'HangulSyllableBbyob',
u'\u5d42': 'CjkUnifiedIdeograph5d42',
u'\u39c5': 'CjkUnifiedIdeograph39c5',
u'\u9844': 'CjkUnifiedIdeograph9844',
u'\u1248': 'EthiopicSyllableQwa',
u'\ueecb': 'UnicodeCharacter0xeecb',
u'\u68cf': 'CjkUnifiedIdeograph68cf',
u'\ucb4e': 'HangulSyllableJjunh',
u'\ua7d1': 'UnicodeCharacter0xa7d1',
u'\u4552': 'CjkUnifiedIdeograph4552',
u'\u21d5': 'UpDownDoubleArrow',
u'\u8054': 'CjkUnifiedIdeograph8054',
u'\ufa58': 'CjkCompatibilityIdeographfa58',
u'\ud6db': 'HangulSyllableHuc',
u'\u745c': 'CjkUnifiedIdeograph745c',
u'\u50df': 'CjkUnifiedIdeograph50df',
u'\ub35e': 'HangulSyllableDeolm',
u'\u8fe1': 'CjkUnifiedIdeograph8fe1',
u'\u2d62': 'TifinaghLetterYay',
u'\u09e5': 'UnicodeCharacter0x9e5',
u'\ue268': 'UnicodeCharacter0xe268',
u'\ubeeb': 'HangulSyllableBbeoh',
u'\u5c6c': 'CjkUnifiedIdeograph5c6c',
u'\u38ef': 'CjkUnifiedIdeograph38ef',
u'\u9b6e': 'CjkUnifiedIdeograph9b6e',
u'\u1572': 'CanadianSyllabicsTyo',
u'\uf1f5': 'UnicodeCharacter0xf1f5',
u'\u6bf9': 'CjkUnifiedIdeograph6bf9',
u'\uca78': 'HangulSyllableJjem',
u'\ua6fb': 'UnicodeCharacter0xa6fb',
u'\u447c': 'CjkUnifiedIdeograph447c',
u'\u20ff': 'UnicodeCharacter0x20ff',
u'\u837e': 'CjkUnifiedIdeograph837e',
u'\u1a03': 'BugineseLetterNgka',
u'\ufd82': 'ArabicLigatureLamWithHahWithAlefMaksuraFinalForm',
u'\u7786': 'CjkUnifiedIdeograph7786',
u'\ud309': 'HangulSyllableTit',
u'\ub288': 'HangulSyllableNyuss',
u'\u4d0d': 'CjkUnifiedIdeograph4d0d',
u'\u2c8c': 'CopticCapitalLetterZata',
u'\u880f': 'CjkUnifiedIdeograph880f',
u'\u0213': 'LatinSmallLetterRWithInvertedBreve',
u'\ue592': 'UnicodeCharacter0xe592',
u'\u7c17': 'CjkUnifiedIdeograph7c17',
u'\u5f96': 'CjkUnifiedIdeograph5f96',
u'\ubb19': 'HangulSyllableMyog',
u'\u9a98': 'CjkUnifiedIdeograph9a98',
u'\u351d': 'CjkUnifiedIdeograph351d',
u'\u149c': 'CanadianSyllabicsCwa',
u'\uea23': 'UnicodeCharacter0xea23',
u'\ucda2': 'HangulSyllableCulp',
u'\u6427': 'CjkUnifiedIdeograph6427',
u'\u47a6': 'CjkUnifiedIdeograph47a6',
u'\ua329': 'YiSyllableSyr',
u'\u82a8': 'CjkUnifiedIdeograph82a8',
u'\u1d2d': 'ModifierLetterCapitalAe',
u'\ufcac': 'ArabicLigatureKhahWithMeemInitialForm',
u'\u76b0': 'CjkUnifiedIdeograph76b0',
u'\ud233': 'HangulSyllableTud',
u'\ub5b2': 'HangulSyllableDdeobs',
u'\u4c37': 'CjkUnifiedIdeograph4c37',
u'\u2fb6': 'KangxiRadicalFly',
u'\u8b39': 'CjkUnifiedIdeograph8b39',
u'\u053d': 'ArmenianCapitalLetterXeh',
u'\ue4bc': 'UnicodeCharacter0xe4bc',
u'\u7f41': 'CjkUnifiedIdeograph7f41',
u'\u5ec0': 'CjkUnifiedIdeograph5ec0',
u'\uba43': 'HangulSyllableMeolb',
u'\u9dc2': 'CjkUnifiedIdeograph9dc2',
u'\u3447': 'CjkUnifiedIdeograph3447',
u'\u17c6': 'KhmerSignNikahit',
u'\ued4d': 'UnicodeCharacter0xed4d',
u'\ucccc': 'HangulSyllableCek',
u'\u6751': 'CjkUnifiedIdeograph6751',
u'\u46d0': 'CjkUnifiedIdeograph46d0',
u'\ua253': 'YiSyllableHxuox',
u'\u85d2': 'CjkUnifiedIdeograph85d2',
u'\u1c57': 'OlChikiDigitSeven',
u'\uffd6': 'HalfwidthHangulLetterWi',
u'\u79da': 'CjkUnifiedIdeograph79da',
u'\ud55d': 'HangulSyllableHanj',
u'\ub4dc': 'HangulSyllableDeu',
u'\u4f61': 'CjkUnifiedIdeograph4f61',
u'\u2ee0': 'CjkRadicalCsimplifiedEat',
u'\u8a63': 'CjkUnifiedIdeograph8a63',
u'\u0467': 'CyrillicSmallLetterLittleYus',
u'\ue7e6': 'UnicodeCharacter0xe7e6',
u'\u7e6b': 'CjkUnifiedIdeograph7e6b',
u'\u61ea': 'CjkUnifiedIdeograph61ea',
u'\ubd6d': 'HangulSyllableByolg',
u'\u9cec': 'CjkUnifiedIdeograph9cec',
u'\u3771': 'CjkUnifiedIdeograph3771',
u'\u16f0': 'RunicBelgthorSymbol',
u'\uec77': 'UnicodeCharacter0xec77',
u'\ucff6': 'HangulSyllableKuj',
u'\u667b': 'CjkUnifiedIdeograph667b',
u'\u49fa': 'CjkUnifiedIdeograph49fa',
u'\ua57d': 'VaiSyllableFoo',
u'\u84fc': 'CjkUnifiedIdeograph84fc',
u'\u9e81': 'CjkUnifiedIdeograph9e81',
u'\uff00': 'UnicodeCharacter0xff00',
u'\udb83': 'UnicodeCharacter0xdb83',
u'\u3c02': 'CjkUnifiedIdeograph3c02',
u'\u1885': 'MongolianLetterAliGaliBaluda',
u'\u7904': 'CjkUnifiedIdeograph7904',
u'\u5587': 'CjkUnifiedIdeograph5587',
u'\uaa0e': 'ChamLetterJa',
u'\u8691': 'CjkUnifiedIdeograph8691',
u'\ue710': 'UnicodeCharacter0xe710',
u'\uc393': 'HangulSyllableSselh',
u'\u2412': 'SymbolForDeviceControlTwo',
u'\x95': 'UnicodeCharacter0x95',
u'\u6114': 'CjkUnifiedIdeograph6114',
u'\u3d97': 'CjkUnifiedIdeograph3d97',
u'\u7a99': 'CjkUnifiedIdeograph7a99',
u'\u921e': 'CjkUnifiedIdeograph921e',
u'\ucf20': 'HangulSyllableKyeon',
u'\uaba3': 'UnicodeCharacter0xaba3',
u'\u0c22': 'TeluguLetterDdha',
u'\ue8a5': 'UnicodeCharacter0xe8a5',
u'\u4924': 'CjkUnifiedIdeograph4924',
u'\u25a7': 'SquareWithUpperLeftToLowerRightFill',
u'\u62a9': 'CjkUnifiedIdeograph62a9',
u'\ub730': 'HangulSyllableDdeul',
u'\u93b3': 'CjkUnifiedIdeograph93b3',
u'\uf432': 'UnicodeCharacter0xf432',
u'\ud0b5': 'HangulSyllableKib',
u'\u3134': 'HangulLetterNieun',
u'\u0db7': 'SinhalaLetterMahaapraanaBayanna',
u'\u6e36': 'CjkUnifiedIdeograph6e36',
u'\u4ab9': 'CjkUnifiedIdeograph4ab9',
u'\u9f40': 'CjkUnifiedIdeograph9f40',
u'\udc42': 'UnicodeCharacter0xdc42',
u'\ub8c5': 'HangulSyllableRoeng',
u'\u1944': 'LimbuExclamationMark',
u'\uf5c7': 'UnicodeCharacter0xf5c7',
u'\u5646': 'CjkUnifiedIdeograph5646',
u'\u32c9': 'IdeographicTelegraphSymbolForOctober',
u'\u6fcb': 'CjkUnifiedIdeograph6fcb',
u'\u8750': 'CjkUnifiedIdeograph8750',
u'\uc452': 'HangulSyllableSsyolm',
u'\ua0d5': 'YiSyllableFox',
u'\u0154': 'LatinCapitalLetterRWithAcute',
u'\uddd7': 'UnicodeCharacter0xddd7',
u'\u3e56': 'CjkUnifiedIdeograph3e56',
u'\u1ad9': 'UnicodeCharacter0x1ad9',
u'\u7b58': 'CjkUnifiedIdeograph7b58',
u'\u57db': 'CjkUnifiedIdeograph57db',
u'\uac62': 'HangulSyllableGyaelp',
u'\u88e5': 'CjkUnifiedIdeograph88e5',
u'\ue964': 'UnicodeCharacter0xe964',
u'\uc5e7': 'HangulSyllableEc',
u'\u2666': 'BlackDiamondSuit',
u'\u02e9': 'ModifierLetterExtralowToneBar',
u'\u6368': 'CjkUnifiedIdeograph6368',
u'\u3feb': 'CjkUnifiedIdeograph3feb',
u'\u7ced': 'CjkUnifiedIdeograph7ced',
u'\u9472': 'CjkUnifiedIdeograph9472',
u'\ud174': 'HangulSyllableTyeols',
u'\uadf7': 'HangulSyllableGyuh',
u'\u0e76': 'UnicodeCharacter0xe76',
u'\ueaf9': 'UnicodeCharacter0xeaf9',
u'\u4b78': 'CjkUnifiedIdeograph4b78',
u'\u27fb': 'LongLeftwardsArrowFromBar',
u'\u64fd': 'CjkUnifiedIdeograph64fd',
u'\u5e01': 'CjkUnifiedIdeograph5e01',
u'\ub984': 'HangulSyllableReum',
u'\uf686': 'UnicodeCharacter0xf686',
u'\u3388': 'SquareCal',
u'\u8f0b': 'CjkUnifiedIdeograph8f0b',
u'\u708a': 'CjkUnifiedIdeograph708a',
u'\ucc0d': 'HangulSyllableJjig',
u'\u090f': 'DevanagariLetterE',
u'\u4611': 'CjkUnifiedIdeograph4611',
u'\ua194': 'YiSyllableNup',
u'\ude96': 'UnicodeCharacter0xde96',
u'\u1b98': 'SundaneseLetterBa',
u'\u589a': 'CjkUnifiedIdeograph589a',
u'\ub41d': 'HangulSyllableDoenj',
u'\uf11f': 'UnicodeCharacter0xf11f',
u'\u2e21': 'RightVerticalBarWithQuill',
u'\u6b23': 'CjkUnifiedIdeograph6b23',
u'\u89a4': 'CjkUnifiedIdeograph89a4',
u'\uc6a6': 'HangulSyllableYobs',
u'\u03a8': 'GreekCapitalLetterPsi',
u'\u40aa': 'CjkUnifiedIdeograph40aa',
u'\u9c2d': 'CjkUnifiedIdeograph9c2d',
u'\u7dac': 'CjkUnifiedIdeograph7dac',
u'\ud92f': 'UnicodeCharacter0xd92f',
u'\u1631': 'CanadianSyllabicsCarrierLhee',
u'\u5333': 'CjkUnifiedIdeograph5333',
u'\uaeb6': 'HangulSyllableGgyaej',
u'\uebb8': 'UnicodeCharacter0xebb8',
u'\u28ba': 'BraillePatternDots24568',
u'\u843d': 'CjkUnifiedIdeograph843d',
u'\u65bc': 'CjkUnifiedIdeograph65bc',
u'\uc13f': 'HangulSyllableSed',
u'\ufe41': 'PresentationFormForVerticalLeftCornerBracket',
u'\u3b43': 'CjkUnifiedIdeograph3b43',
u'\u7845': 'CjkUnifiedIdeograph7845',
u'\u96c6': 'CjkUnifiedIdeograph96c6',
u'\ud3c8': 'HangulSyllablePyeoss',
u'\u10ca': 'UnicodeCharacter0x10ca',
u'\u4dcc': 'HexagramForFellowship',
u'\ua94f': 'RejangConsonantSignNg',
u'\ue651': 'UnicodeCharacter0xe651',
u'\u2353': 'AplFunctionalSymbolQuadUpCaret',
u'\u6055': 'CjkUnifiedIdeograph6055',
u'\ubbd8': 'HangulSyllableMeuk',
u'\uf8da': 'UnicodeCharacter0xf8da',
u'\u35dc': 'CjkUnifiedIdeograph35dc',
u'\u915f': 'CjkUnifiedIdeograph915f',
u'\u72de': 'CjkUnifiedIdeograph72de',
u'\uce61': 'HangulSyllableCilg',
u'\u0b63': 'OriyaVowelSignVocalicLl',
u'\u4865': 'CjkUnifiedIdeograph4865',
u'\ua3e8': 'YiSyllableJuot',
u'\ue0ea': 'UnicodeCharacter0xe0ea',
u'\u1dec': 'UnicodeCharacter0x1dec',
u'\u5aee': 'CjkUnifiedIdeograph5aee',
u'\ub671': 'HangulSyllableDdoelt',
u'\uf373': 'UnicodeCharacter0xf373',
u'\u3075': 'HiraganaLetterHu',
u'\u6d77': 'CjkUnifiedIdeograph6d77',
u'\u8bf8': 'CjkUnifiedIdeograph8bf8',
u'\uc8fa': 'HangulSyllableJyop',
u'\u05fc': 'UnicodeCharacter0x5fc',
u'\u42fe': 'CjkUnifiedIdeograph42fe',
u'\u8000': 'CjkUnifiedIdeograph8000',
u'\ua483': 'YiSyllableYu',
u'\u4302': 'CjkUnifiedIdeograph4302',
u'\u6785': 'CjkUnifiedIdeograph6785',
u'\uc906': 'HangulSyllableJulm',
u'\ued89': 'UnicodeCharacter0xed89',
u'\u120c': 'EthiopicSyllableLee',
u'\u368f': 'CjkUnifiedIdeograph368f',
u'\u9810': 'CjkUnifiedIdeograph9810',
u'\ubc93': 'HangulSyllableBeolh',
u'\u5b12': 'CjkUnifiedIdeograph5b12',
u'\u7f95': 'CjkUnifiedIdeograph7f95',
u'\ue116': 'UnicodeCharacter0xe116',
u'\u0599': 'HebrewAccentPashta',
u'\u8b9d': 'CjkUnifiedIdeograph8b9d',
u'\u2a1c': 'IntegralWithUnderbar',
u'\u4e9f': 'CjkUnifiedIdeograph4e9f',
u'\ub020': 'HangulSyllableGgwiss',
u'\ud4a3': 'HangulSyllablePweolh',
u'\u7322': 'CjkUnifiedIdeograph7322',
u'\uf926': 'CjkCompatibilityIdeographf926',
u'\u1da9': 'ModifierLetterSmallLWithRetroflexHook',
u'\ua3ad': 'YiSyllableSha',
u'\u422c': 'CjkUnifiedIdeograph422c',
u'\u66af': 'CjkUnifiedIdeograph66af',
u'\uc830': 'HangulSyllableJess',
u'\uecb3': 'UnicodeCharacter0xecb3',
u'\u1136': 'HangulChoseongSioscieuc',
u'\u35b9': 'CjkUnifiedIdeograph35b9',
u'\u973a': 'CjkUnifiedIdeograph973a',
u'\ubbbd': 'HangulSyllableMyut',
u'\u5a3c': 'CjkUnifiedIdeograph5a3c',
u'\u7ebf': 'CjkUnifiedIdeograph7ebf',
u'\ue040': 'UnicodeCharacter0xe040',
u'\u04c3': 'CyrillicCapitalLetterKaWithHook',
u'\u8ac7': 'CjkUnifiedIdeograph8ac7',
u'\u2946': 'LeftwardsArrowWithPlusBelow',
u'\u4dc9': 'HexagramForTreading',
u'\uaf4a': 'HangulSyllableGgwagg',
u'\ud3cd': 'HangulSyllablePyeot',
u'\u724c': 'CjkUnifiedIdeograph724c',
u'\uf850': 'UnicodeCharacter0xf850',
u'\u1cd3': 'UnicodeCharacter0x1cd3',
u'\ua2d7': 'YiSyllableZziep',
u'\u4156': 'CjkUnifiedIdeograph4156',
u'\u65d9': 'CjkUnifiedIdeograph65d9',
u'\uc75a': 'HangulSyllableYigg',
u'\uebdd': 'UnicodeCharacter0xebdd',
u'\u1060': 'MyanmarConsonantSignMonMedialLa',
u'\u34e3': 'CjkUnifiedIdeograph34e3',
u'\u9664': 'CjkUnifiedIdeograph9664',
u'\ubae7': 'HangulSyllableMwaed',
u'\u5966': 'CjkUnifiedIdeograph5966',
u'\u7de9': 'CjkUnifiedIdeograph7de9',
u'\udf6a': 'UnicodeCharacter0xdf6a',
u'\u03ed': 'CopticSmallLetterShima',
u'\u89f1': 'CjkUnifiedIdeograph89f1',
u'\u2870': 'BraillePatternDots567',
u'\u4cf3': 'CjkUnifiedIdeograph4cf3',
u'\uae74': 'HangulSyllableGgaels',
u'\ud2f7': 'HangulSyllableTid',
u'\u7176': 'CjkUnifiedIdeograph7176',
u'\uf77a': 'UnicodeCharacter0xf77a',
u'\u1bfd': 'UnicodeCharacter0x1bfd',
u'\u2101': 'AddressedToTheSubject',
u'\u4080': 'CjkUnifiedIdeograph4080',
u'\ua705': 'ModifierLetterChineseToneYangQu',
u'\uc684': 'HangulSyllableOels',
u'\u6a07': 'CjkUnifiedIdeograph6a07',
u'\uf00b': 'UnicodeCharacter0xf00b',
u'\u0f8a': 'TibetanSignGruCanRgyings',
u'\u958e': 'CjkUnifiedIdeograph958e',
u'\u3911': 'CjkUnifiedIdeograph3911',
u'\u5890': 'CjkUnifiedIdeograph5890',
u'\ubf15': 'HangulSyllableBbyeolt',
u'\ude94': 'UnicodeCharacter0xde94',
u'\u081b': 'UnicodeCharacter0x81b',
u'\u279a': 'HeavyNorthEastArrow',
u'\u8e1f': 'CjkUnifiedIdeograph8e1f',
u'\uad9e': 'HangulSyllableGweoj',
u'\u5121': 'CjkUnifiedIdeograph5121',
u'\u70a0': 'CjkUnifiedIdeograph70a0',
u'\ud725': 'HangulSyllableHwilt',
u'\uf6a4': 'UnicodeCharacter0xf6a4',
u'\u202b': 'RighttoleftEmbedding',
u'\u3faa': 'CjkUnifiedIdeograph3faa',
u'\ua62f': 'UnicodeCharacter0xa62f',
u'\uc5ae': 'HangulSyllableYaej',
u'\u6931': 'CjkUnifiedIdeograph6931',
u'\uef35': 'UnicodeCharacter0xef35',
u'\u0eb4': 'LaoVowelSignI',
u'\u94b8': 'CjkUnifiedIdeograph94b8',
u'\u383b': 'CjkUnifiedIdeograph383b',
u'\u57ba': 'CjkUnifiedIdeograph57ba',
u'\ube3f': 'HangulSyllableByic',
u'\uddbe': 'UnicodeCharacter0xddbe',
u'\u0745': 'SyriacThreeDotsAbove',
u'\u26c4': 'UnicodeCharacter0x26c4',
u'\u8d49': 'CjkUnifiedIdeograph8d49',
u'\uacc8': 'HangulSyllableGyen',
u'\u504b': 'CjkUnifiedIdeograph504b',
u'\u6fca': 'CjkUnifiedIdeograph6fca',
u'\ud64f': 'HangulSyllableHoc',
u'\uf5ce': 'UnicodeCharacter0xf5ce',
u'\u1f55': 'GreekSmallLetterUpsilonWithDasiaAndOxia',
u'\u3ed4': 'CjkUnifiedIdeograph3ed4',
u'\ua559': 'VaiSyllableVa',
u'\uc4d8': 'HangulSyllableSsyun',
u'\u685b': 'CjkUnifiedIdeograph685b',
u'\uee5f': 'UnicodeCharacter0xee5f',
u'\u0dde': 'SinhalaVowelSignKombuvaHaaGayanukitta',
u'\u93e2': 'CjkUnifiedIdeograph93e2',
u'\u3765': 'CjkUnifiedIdeograph3765',
u'\u56e4': 'CjkUnifiedIdeograph56e4',
u'\ubd69': 'HangulSyllableByonj',
u'\udce8': 'UnicodeCharacter0xdce8',
u'\u066f': 'ArabicLetterDotlessQaf',
u'\u25ee': 'UppointingTriangleWithRightHalfBlack',
u'\u8c73': 'CjkUnifiedIdeograph8c73',
u'\uabf2': 'UnicodeCharacter0xabf2',
u'\u4f75': 'CjkUnifiedIdeograph4f75',
u'\u6ef4': 'CjkUnifiedIdeograph6ef4',
u'\ud579': 'HangulSyllableHaenj',
u'\uf4f8': 'UnicodeCharacter0xf4f8',
u'\u1e7f': 'LatinSmallLetterVWithDotBelow',
u'\u3dfe': 'CjkUnifiedIdeograph3dfe',
u'\u4806': 'CjkUnifiedIdeograph4806',
u'\u6c89': 'CjkUnifiedIdeograph6c89',
u'\u0d08': 'MalayalamLetterIi',
u'\u318b': 'HangulLetterYuye',
u'\uce0a': 'HangulSyllableCyunh',
u'\uf28d': 'UnicodeCharacter0xf28d',
u'\u930c': 'CjkUnifiedIdeograph930c',
u'\ub78f': 'HangulSyllableRas',
u'\u6016': 'CjkUnifiedIdeograph6016',
u'\u2518': 'BoxDrawingsLightUpAndLeft',
u'\u499b': 'CjkUnifiedIdeograph499b',
u'\ue61a': 'UnicodeCharacter0xe61a',
u'\u0a9d': 'GujaratiLetterJha',
u'\uab1c': 'UnicodeCharacter0xab1c',
u'\ucf9f': 'HangulSyllableKwaes',
u'\u90a1': 'CjkUnifiedIdeograph90a1',
u'\u7826': 'CjkUnifiedIdeograph7826',
u'\u3d28': 'CjkUnifiedIdeograph3d28',
u'\u61ab': 'CjkUnifiedIdeograph61ab',
u'\ufe2a': 'UnicodeCharacter0xfe2a',
u'\u22ad': 'NotTrue',
u'\uc32c': 'HangulSyllableSsaek',
u'\ue7af': 'UnicodeCharacter0xe7af',
u'\u842e': 'CjkUnifiedIdeograph842e',
u'\ua8b1': 'SaurashtraLetterSa',
u'\u5538': 'CjkUnifiedIdeograph5538',
u'\u79bb': 'CjkUnifiedIdeograph79bb',
u'\u163a': 'CanadianSyllabicsCarrierTlu',
u'\u3abd': 'CjkUnifiedIdeograph3abd',
u'\udb3c': 'UnicodeCharacter0xdb3c',
u'\uffbf': 'UnicodeCharacter0xffbf',
u'\u9c3e': 'CjkUnifiedIdeograph9c3e',
u'\uc0c1': 'HangulSyllableSang',
u'\u85c3': 'CjkUnifiedIdeograph85c3',
u'\u6d48': 'CjkUnifiedIdeograph6d48',
u'\u2e4a': 'UnicodeCharacter0x2e4a',
u'\u52cd': 'CjkUnifiedIdeograph52cd',
u'\uf34c': 'UnicodeCharacter0xf34c',
u'\u17cf': 'KhmerSignAhsda',
u'\ub44e': 'HangulSyllableDyop',
u'\ud8d1': 'UnicodeCharacter0xd8d1',
u'\u9dd3': 'CjkUnifiedIdeograph9dd3',
u'\u465a': 'CjkUnifiedIdeograph465a',
u'\u6add': 'CjkUnifiedIdeograph6add',
u'\u0b5c': 'OriyaLetterRra',
u'\u2fdf': 'UnicodeCharacter0x2fdf',
u'\ucc5e': 'HangulSyllableCaep',
u'\uf0e1': 'UnicodeCharacter0xf0e1',
u'\u9160': 'CjkUnifiedIdeograph9160',
u'\ub5e3': 'HangulSyllableDdyeolb',
u'\u5e6a': 'CjkUnifiedIdeograph5e6a',
u'\u236c': 'AplFunctionalSymbolZilde',
u'\u47ef': 'CjkUnifiedIdeograph47ef',
u'\ue46e': 'UnicodeCharacter0xe46e',
u'\u08f1': 'UnicodeCharacter0x8f1',
u'\ua970': 'UnicodeCharacter0xa970',
u'\ucdf3': 'HangulSyllableCwilb',
u'\u8ef5': 'CjkUnifiedIdeograph8ef5',
u'\u767a': 'CjkUnifiedIdeograph767a',
u'\u3b7c': 'CjkUnifiedIdeograph3b7c',
u'\u5fff': 'CjkUnifiedIdeograph5fff',
u'\ufc7e': 'ArabicLigatureQafWithAlefMaksuraFinalForm',
u'\uc180': 'HangulSyllableSyem',
u'\u6503': 'CjkUnifiedIdeograph6503',
u'\u8282': 'CjkUnifiedIdeograph8282',
u'\u2605': 'BlackStar',
u'\ueb07': 'UnicodeCharacter0xeb07',
u'\uac09': 'HangulSyllableGalg',
u'\u538c': 'CjkUnifiedIdeograph538c',
u'\u148e': 'CanadianSyllabicsCoo',
u'\ud990': 'UnicodeCharacter0xd990',
u'\u7d13': 'CjkUnifiedIdeograph7d13',
u'\u9a92': 'CjkUnifiedIdeograph9a92',
u'\u3e15': 'CjkUnifiedIdeograph3e15',
u'\u0317': 'CombiningAcuteAccentBelow',
u'\uc419': 'HangulSyllableSswaelg',
u'\u891b': 'CjkUnifiedIdeograph891b',
u'\u6b9c': 'CjkUnifiedIdeograph6b9c',
u'\u2c9e': 'CopticCapitalLetterO',
u'\uf1a0': 'UnicodeCharacter0xf1a0',
u'\ub2a2': 'HangulSyllableNeubs',
u'\u5625': 'CjkUnifiedIdeograph5625',
u'\u1b27': 'BalineseLetterPa',
u'\udc29': 'UnicodeCharacter0xdc29',
u'\ua12b': 'YiSyllableTo',
u'\u44ae': 'CjkUnifiedIdeograph44ae',
u'\u09b0': 'BengaliLetterRa',
u'\ucab2': 'HangulSyllableJjyebs',
u'\u6e35': 'CjkUnifiedIdeograph6e35',
u'\u8fb4': 'CjkUnifiedIdeograph8fb4',
u'\u3337': 'SquarePeso',
u'\uf439': 'UnicodeCharacter0xf439',
u'\ub93b': 'HangulSyllableRweh',
u'\u5cbe': 'CjkUnifiedIdeograph5cbe',
u'\u21c0': 'RightwardsHarpoonWithBarbUpwards',
u'\ue2c2': 'UnicodeCharacter0xe2c2',
u'\ua7c4': 'UnicodeCharacter0xa7c4',
u'\u4b47': 'CjkUnifiedIdeograph4b47',
u'\u0c49': 'UnicodeCharacter0xc49',
u'\ud14b': 'HangulSyllableTeoh',
u'\u924d': 'CjkUnifiedIdeograph924d',
u'\u74ce': 'CjkUnifiedIdeograph74ce',
u'\u39d0': 'CjkUnifiedIdeograph39d0',
u'\ufad2': 'CjkCompatibilityIdeographfad2',
u'\ubfd4': 'HangulSyllableBbul',
u'\u6357': 'CjkUnifiedIdeograph6357',
u'\u80d6': 'CjkUnifiedIdeograph80d6',
u'\u2459': 'UnicodeCharacter0x2459',
u'\ue95b': 'UnicodeCharacter0xe95b',
u'\uaa5d': 'ChamPunctuationDanda',
u'\u51e0': 'CjkUnifiedIdeograph51e0',
u'\u12e2': 'EthiopicSyllableZhi',
u'\ud7e4': 'UnicodeCharacter0xd7e4',
u'\u7b67': 'CjkUnifiedIdeograph7b67',
u'\u98e6': 'CjkUnifiedIdeograph98e6',
u'\u3c69': 'CjkUnifiedIdeograph3c69',
u'\u016b': 'LatinSmallLetterUWithMacron',
u'\uc26d': 'HangulSyllableSwig',
u'\u876f': 'CjkUnifiedIdeograph876f',
u'\u69f0': 'CjkUnifiedIdeograph69f0',
u'\u2af2': 'ParallelWithHorizontalStroke',
u'\ueff4': 'UnicodeCharacter0xeff4',
u'\ub0f6': 'HangulSyllableNyaelm',
u'\u5479': 'CjkUnifiedIdeograph5479',
u'\u197b': 'UnicodeCharacter0x197b',
u'\uda7d': 'UnicodeCharacter0xda7d',
u'\u9f7f': 'CjkUnifiedIdeograph9f7f',
u'\ue381': 'UnicodeCharacter0xe381',
u'\u0404': 'CyrillicCapitalLetterUkrainianIe',
u'\u2887': 'BraillePatternDots1238',
u'\u8e08': 'CjkUnifiedIdeograph8e08',
u'\ub28b': 'HangulSyllableNyuc',
u'\u4d0a': 'CjkUnifiedIdeograph4d0a',
u'\u718d': 'CjkUnifiedIdeograph718d',
u'\ud70e': 'HangulSyllableHwebs',
u'\ufb91': 'ArabicLetterKehehMedialForm',
u'\u1c14': 'LepchaLetterBla',
u'\u4097': 'CjkUnifiedIdeograph4097',
u'\ua618': 'VaiSymbolFaa',
u'\uca9b': 'HangulSyllableJjyeoc',
u'\u651a': 'CjkUnifiedIdeograph651a',
u'\uef1e': 'UnicodeCharacter0xef1e',
u'\u13a1': 'CherokeeLetterE',
u'\u95a5': 'CjkUnifiedIdeograph95a5',
u'\u3424': 'CjkUnifiedIdeograph3424',
u'\u58a7': 'CjkUnifiedIdeograph58a7',
u'\ube28': 'HangulSyllableByi',
u'\ue2ab': 'UnicodeCharacter0xe2ab',
u'\u7d2a': 'CjkUnifiedIdeograph7d2a',
u'\u072e': 'SyriacLetterPersianGhamal',
u'\u2bb1': 'UnicodeCharacter0x2bb1',
u'\u8932': 'CjkUnifiedIdeograph8932',
u'\uadb5': 'HangulSyllableGweb',
u'\u4c34': 'CjkUnifiedIdeograph4c34',
u'\u70b7': 'CjkUnifiedIdeograph70b7',
u'\ud638': 'HangulSyllableHo',
u'\ufabb': 'CjkCompatibilityIdeographfabb',
u'\u1f3e': 'GreekCapitalLetterIotaWithPsiliAndPerispomeni',
u'\u43c1': 'CjkUnifiedIdeograph43c1',
u'\ua142': 'YiSyllableDduo',
u'\uc5c5': 'HangulSyllableEob',
u'\u6444': 'CjkUnifiedIdeograph6444',
u'\uee48': 'UnicodeCharacter0xee48',
u'\u12cb': 'EthiopicSyllableWaa',
u'\u94cf': 'CjkUnifiedIdeograph94cf',
u'\u374e': 'CjkUnifiedIdeograph374e',
u'\u5bd1': 'CjkUnifiedIdeograph5bd1',
u'\ub952': 'HangulSyllableRwij',
u'\uddd5': 'UnicodeCharacter0xddd5',
u'\u7c54': 'CjkUnifiedIdeograph7c54',
u'\u0658': 'ArabicMarkNoonGhunna',
u'\u2adb': 'TransversalIntersection',
u'\u885c': 'CjkUnifiedIdeograph885c',
u'\uacdf': 'HangulSyllableGyeh',
u'\u4f5e': 'CjkUnifiedIdeograph4f5e',
u'\u73e1': 'CjkUnifiedIdeograph73e1',
u'\ud162': 'HangulSyllableTej',
u'\uf5e5': 'UnicodeCharacter0xf5e5',
u'\u1e68': 'LatinCapitalLetterSWithDotBelowAndDotAbove',
u'\u42eb': 'CjkUnifiedIdeograph42eb',
u'\ua06c': 'YiSyllableBbut',
u'\uc4ef': 'HangulSyllableSsyuh',
u'\u676e': 'CjkUnifiedIdeograph676e',
u'\ue972': 'UnicodeCharacter0xe972',
u'\u0df5': 'UnicodeCharacter0xdf5',
u'\u97f9': 'CjkUnifiedIdeograph97f9',
u'\u3678': 'CjkUnifiedIdeograph3678',
u'\u5afb': 'CjkUnifiedIdeograph5afb',
u'\ub87c': 'HangulSyllableRwan',
u'\udcff': 'UnicodeCharacter0xdcff',
u'\u7f7e': 'CjkUnifiedIdeograph7f7e',
u'\ue603': 'UnicodeCharacter0xe603',
u'\u0182': 'LatinCapitalLetterBWithTopbar',
u'\u8b86': 'CjkUnifiedIdeograph8b86',
u'\u2f09': 'KangxiRadicalLegs',
u'\u4e88': 'CjkUnifiedIdeograph4e88',
u'\ub10d': 'HangulSyllableNeonj',
u'\ud08c': 'HangulSyllableKyin',
u'\u740f': 'CjkUnifiedIdeograph740f',
u'\ufe13': 'PresentationFormForVerticalColon',
u'\u1992': 'NewTaiLueLetterLowTha',
u'\u8017': 'CjkUnifiedIdeograph8017',
u'\ua396': 'YiSyllableNra',
u'\u4719': 'CjkUnifiedIdeograph4719',
u'\u6698': 'CjkUnifiedIdeograph6698',
u'\uc91d': 'HangulSyllableJweonj',
u'\ue89c': 'UnicodeCharacter0xe89c',
u'\u1623': 'CanadianSyllabicsCarrierLo',
u'\u31a2': 'BopomofoLetterJi',
u'\u9827': 'CjkUnifiedIdeograph9827',
u'\ubba6': 'HangulSyllableMyugg',
u'\u5f29': 'CjkUnifiedIdeograph5f29',
u'\u7ea8': 'CjkUnifiedIdeograph7ea8',
u'\ue12d': 'UnicodeCharacter0xe12d',
u'\xac': 'NotSign',
u'\u8ab0': 'CjkUnifiedIdeograph8ab0',
u'\u2e33': 'UnicodeCharacter0x2e33',
u'\u49b2': 'CjkUnifiedIdeograph49b2',
u'\ub037': 'HangulSyllableGgyulh',
u'\ud3b6': 'HangulSyllablePyeogg',
u'\u7739': 'CjkUnifiedIdeograph7739',
u'\uf93d': 'CjkCompatibilityIdeographf93d',
u'\u18bc': 'UnicodeCharacter0x18bc',
u'\u8341': 'CjkUnifiedIdeograph8341',
u'\ua2c0': 'YiSyllableCop',
u'\u4643': 'CjkUnifiedIdeograph4643',
u'\u61c2': 'CjkUnifiedIdeograph61c2',
u'\uc847': 'HangulSyllableJyeolh',
u'\uebc6': 'UnicodeCharacter0xebc6',
u'\u114d': 'HangulChoseongCieucieung',
u'\u30cc': 'KatakanaLetterNu',
u'\u9b51': 'CjkUnifiedIdeograph9b51',
u'\ubad0': 'HangulSyllableMwals',
u'\u5e53': 'CjkUnifiedIdeograph5e53',
u'\u79d2': 'CjkUnifiedIdeograph79d2',
u'\ue057': 'UnicodeCharacter0xe057',
u'\u03d6': 'GreekPiSymbol',
u'\u85da': 'CjkUnifiedIdeograph85da',
u'\u295d': 'DownwardsHarpoonWithBarbRightFromBar',
u'\u48dc': 'CjkUnifiedIdeograph48dc',
u'\ub361': 'HangulSyllableDeolt',
u'\ud2e0': 'HangulSyllableTyils',
u'\u7663': 'CjkUnifiedIdeograph7663',
u'\uf867': 'UnicodeCharacter0xf867',
u'\u1be6': 'UnicodeCharacter0x1be6',
u'\u826b': 'CjkUnifiedIdeograph826b',
u'\u9dea': 'CjkUnifiedIdeograph9dea',
u'\u416d': 'CjkUnifiedIdeograph416d',
u'\u60ec': 'CjkUnifiedIdeograph60ec',
u'\ucb71': 'HangulSyllableJjweolt',
u'\ueaf0': 'UnicodeCharacter0xeaf0',
u'\u1077': 'MyanmarLetterShanGa',
u'\u33f6': 'IdeographicTelegraphSymbolForDayTwentythree',
u'\u9a7b': 'CjkUnifiedIdeograph9a7b',
u'\ub5fa': 'HangulSyllableDdyenh',
u'\u597d': 'CjkUnifiedIdeograph597d',
u'\u78fc': 'CjkUnifiedIdeograph78fc',
u'\u6281': 'CjkUnifiedIdeograph6281',
u'\u0300': 'CombiningGraveAccent',
u'\u2783': 'DingbatCircledSansserifDigitFour',
u'\uc002': 'HangulSyllableBbweop',
u'\ue485': 'UnicodeCharacter0xe485',
u'\u8504': 'CjkUnifiedIdeograph8504',
u'\ua987': 'UnicodeCharacter0xa987',
u'\u560e': 'CjkUnifiedIdeograph560e',
u'\u7a91': 'CjkUnifiedIdeograph7a91',
u'\u1b10': 'BalineseLetterAikara',
u'\u3f93': 'CjkUnifiedIdeograph3f93',
u'\ud812': 'UnicodeCharacter0xd812',
u'\ufc95': 'ArabicLigatureYehWithAlefMaksuraFinalForm',
u'\u9d14': 'CjkUnifiedIdeograph9d14',
u'\uc197': 'HangulSyllableSolb',
u'\u8699': 'CjkUnifiedIdeograph8699',
u'\u6e1e': 'CjkUnifiedIdeograph6e1e',
u'\u3320': 'SquareSantiimu',
u'\u57a3': 'CjkUnifiedIdeograph57a3',
u'\uf022': 'UnicodeCharacter0xf022',
u'\u14a5': 'CanadianSyllabicsMi',
u'\ub524': 'HangulSyllableDim',
u'\ud9a7': 'UnicodeCharacter0xd9a7',
u'\u9ea9': 'CjkUnifiedIdeograph9ea9',
u'\u4b30': 'CjkUnifiedIdeograph4b30',
u'\u6fb3': 'CjkUnifiedIdeograph6fb3',
u'\u0832': 'UnicodeCharacter0x832',
u'\u2cb5': 'CopticSmallLetterOldCopticAin',
u'\ucd34': 'HangulSyllableCwam',
u'\uf1b7': 'UnicodeCharacter0xf1b7',
u'\u9236': 'CjkUnifiedIdeograph9236',
u'\ub6b9': 'HangulSyllableDdweog',
u'\u6340': 'CjkUnifiedIdeograph6340',
u'\u2042': 'Asterism',
u'\u44c5': 'CjkUnifiedIdeograph44c5',
u'\ue544': 'UnicodeCharacter0xe544',
u'\u09c7': 'BengaliVowelSignE',
u'\uaa46': 'ChamLetterFinalN',
u'\ucec9': 'HangulSyllableKyaeg',
u'\u93cb': 'CjkUnifiedIdeograph93cb',
u'\u7b50': 'CjkUnifiedIdeograph7b50',
u'\u3852': 'CjkUnifiedIdeograph3852',
u'\u5cd5': 'CjkUnifiedIdeograph5cd5',
u'\ufd54': 'ArabicLigatureTehWithKhahWithMeemInitialForm',
u'\u21d7': 'NorthEastDoubleArrow',
u'\uc256': 'HangulSyllableSwenh',
u'\ue6d9': 'UnicodeCharacter0xe6d9',
u'\u8758': 'CjkUnifiedIdeograph8758',
u'\uabdb': 'UnicodeCharacter0xabdb',
u'\u5062': 'CjkUnifiedIdeograph5062',
u'\u74e5': 'CjkUnifiedIdeograph74e5',
u'\u1564': 'CanadianSyllabicsTho',
u'\u39e7': 'CjkUnifiedIdeograph39e7',
u'\uda66': 'UnicodeCharacter0xda66',
u'\ufee9': 'ArabicLetterHehIsolatedForm',
u'\u9f68': 'CjkUnifiedIdeograph9f68',
u'\uc3eb': 'HangulSyllableSsos',
u'\u80ed': 'CjkUnifiedIdeograph80ed',
u'\u6872': 'CjkUnifiedIdeograph6872',
u'\u2d74': 'UnicodeCharacter0x2d74',
u'\u51f7': 'CjkUnifiedIdeograph51f7',
u'\uf276': 'UnicodeCharacter0xf276',
u'\u16f9': 'UnicodeCharacter0x16f9',
u'\ub778': 'HangulSyllableDdik',
u'\udbfb': 'UnicodeCharacter0xdbfb',
u'\u98fd': 'CjkUnifiedIdeograph98fd',
u'\ua201': 'YiSyllableKa',
u'\u4584': 'CjkUnifiedIdeograph4584',
u'\u0a86': 'GujaratiLetterAa',
u'\ucf88': 'HangulSyllableKwak',
u'\u730b': 'CjkUnifiedIdeograph730b',
u'\u8c8a': 'CjkUnifiedIdeograph8c8a',
u'\u300d': 'RightCornerBracket',
u'\uf50f': 'UnicodeCharacter0xf50f',
u'\uba11': 'HangulSyllableMyab',
u'\u5d94': 'CjkUnifiedIdeograph5d94',
u'\u2296': 'CircledMinus',
u'\ue798': 'UnicodeCharacter0xe798',
u'\ua49a': 'YiRadicalLyr',
u'\u481d': 'CjkUnifiedIdeograph481d',
u'\u0d1f': 'MalayalamLetterTta',
u'\ud221': 'HangulSyllableTyob',
u'\u9723': 'CjkUnifiedIdeograph9723',
u'\u75a4': 'CjkUnifiedIdeograph75a4',
u'\u3aa6': 'CjkUnifiedIdeograph3aa6',
u'\uffa8': 'HalfwidthHangulLetterSsangtikeut',
u'\ubcaa': 'HangulSyllableBelm',
u'\u602d': 'CjkUnifiedIdeograph602d',
u'\u81ac': 'CjkUnifiedIdeograph81ac',
u'\u252f': 'BoxDrawingsDownLightAndHorizontalHeavy',
u'\uea31': 'UnicodeCharacter0xea31',
u'\uaf33': 'HangulSyllableGgod',
u'\u52b6': 'CjkUnifiedIdeograph52b6',
u'\u17b8': 'KhmerVowelSignIi',
u'\ud4ba': 'HangulSyllablePwelm',
u'\u783d': 'CjkUnifiedIdeograph783d',
u'\u99bc': 'CjkUnifiedIdeograph99bc',
u'\u3d3f': 'CjkUnifiedIdeograph3d3f',
u'\u0241': 'LatinCapitalLetterGlottalStop',
u'\uc743': 'HangulSyllableEud',
u'\u8445': 'CjkUnifiedIdeograph8445',
u'\u6ac6': 'CjkUnifiedIdeograph6ac6',
u'\u2fc8': 'KangxiRadicalYellow',
u'\uecca': 'UnicodeCharacter0xecca',
u'\ub1cc': 'HangulSyllableNoe',
u'\u554f': 'CjkUnifiedIdeograph554f',
u'\u1a51': 'UnicodeCharacter0x1a51',
u'\udf53': 'UnicodeCharacter0xdf53',
u'\u9c55': 'CjkUnifiedIdeograph9c55',
u'\u47d8': 'CjkUnifiedIdeograph47d8',
u'\u04da': 'CyrillicCapitalLetterSchwaWithDiaeresis',
u'\uc9dc': 'HangulSyllableJja',
u'\u6d5f': 'CjkUnifiedIdeograph6d5f',
u'\u8ede': 'CjkUnifiedIdeograph8ede',
u'\u3261': 'CircledHangulNieun',
u'\uf763': 'UnicodeCharacter0xf763',
u'\ub465': 'HangulSyllableDung',
u'\u5fe8': 'CjkUnifiedIdeograph5fe8',
u'\u1cea': 'UnicodeCharacter0x1cea',
u'\ue1ec': 'UnicodeCharacter0xe1ec',
u'\ua6ee': 'UnicodeCharacter0xa6ee',
u'\u4a71': 'CjkUnifiedIdeograph4a71',
u'\u0f73': 'TibetanVowelSignIi',
u'\ucc75': 'HangulSyllableCyang',
u'\u9177': 'CjkUnifiedIdeograph9177',
u'\u77f8': 'CjkUnifiedIdeograph77f8',
u'\u34fa': 'CjkUnifiedIdeograph34fa',
u'\uf9fc': 'CjkCompatibilityIdeographf9fc',
u'\ubefe': 'HangulSyllableBbebs',
u'\u8400': 'CjkUnifiedIdeograph8400',
u'\ua083': 'YiSyllableNbo',
u'\u4702': 'CjkUnifiedIdeograph4702',
u'\u6385': 'CjkUnifiedIdeograph6385',
u'\ucd06': 'HangulSyllableCyep',
u'\ue989': 'UnicodeCharacter0xe989',
u'\u160c': 'CanadianSyllabicsCarrierMi',
u'\u328f': 'CircledIdeographEarth',
u'\u9c10': 'CjkUnifiedIdeograph9c10',
u'\ub893': 'HangulSyllableRwah',
u'\u5f12': 'CjkUnifiedIdeograph5f12',
u'\u7b95': 'CjkUnifiedIdeograph7b95',
u'\ue516': 'UnicodeCharacter0xe516',
u'\u0199': 'LatinSmallLetterKWithHook',
u'\u8f9d': 'CjkUnifiedIdeograph8f9d',
u'\u2e1c': 'LeftLowParaphraseBracket',
u'\u4a9f': 'CjkUnifiedIdeograph4a9f',
u'\ub420': 'HangulSyllableDoel',
u'\ud0a3': 'HangulSyllableKyih',
u'\u7722': 'CjkUnifiedIdeograph7722',
u'\ufd26': 'ArabicLigatureSheenWithHahFinalForm',
u'\u19a9': 'NewTaiLueLetterLowXva',
u'\ua7ad': 'UnicodeCharacter0xa7ad',
u'\u462c': 'CjkUnifiedIdeograph462c',
u'\u62af': 'CjkUnifiedIdeograph62af',
u'\ucc30': 'HangulSyllableCal',
u'\ue8b3': 'UnicodeCharacter0xe8b3',
u'\u1536': 'CanadianSyllabicsWestcreeYwo',
u'\u31b9': 'UnicodeCharacter0x31b9',
u'\u933a': 'CjkUnifiedIdeograph933a',
u'\ubfbd': 'HangulSyllableBbyolt',
u'\u5e3c': 'CjkUnifiedIdeograph5e3c',
u'\u7abf': 'CjkUnifiedIdeograph7abf',
u'\ue440': 'UnicodeCharacter0xe440',
u'\xc3': 'LatinCapitalLetterAWithTilde',
u'\u8ec7': 'CjkUnifiedIdeograph8ec7',
u'\u2d46': 'TifinaghLetterTuaregYakh',
u'\u49c9': 'CjkUnifiedIdeograph49c9',
u'\uab4a': 'UnicodeCharacter0xab4a',
u'\ud7cd': 'UnicodeCharacter0xd7cd',
u'\u764c': 'CjkUnifiedIdeograph764c',
u'\ufc50': 'ArabicLigatureNoonWithYehIsolatedForm',
u'\u18d3': 'UnicodeCharacter0x18d3',
u'\ua6d7': 'UnicodeCharacter0xa6d7',
u'\u4556': 'CjkUnifiedIdeograph4556',
u'\u61d9': 'CjkUnifiedIdeograph61d9',
u'\uc35a': 'HangulSyllableSsyaelp',
u'\uefdd': 'UnicodeCharacter0xefdd',
u'\u1460': 'CanadianSyllabicsWestcreeTwoo',
u'\u30e3': 'KatakanaLetterSmallYa',
u'\u9264': 'CjkUnifiedIdeograph9264',
u'\ubee7': 'HangulSyllableBbeoc',
u'\u5d66': 'CjkUnifiedIdeograph5d66',
u'\u79e9': 'CjkUnifiedIdeograph79e9',
u'\udb6a': 'UnicodeCharacter0xdb6a',
u'\u07ed': 'NkoCombiningShortRisingTone',
u'\u8df1': 'CjkUnifiedIdeograph8df1',
u'\u2c70': 'UnicodeCharacter0x2c70',
u'\u48f3': 'CjkUnifiedIdeograph48f3',
u'\uaa74': 'UnicodeCharacter0xaa74',
u'\ud6f7': 'HangulSyllableHweoc',
u'\u7576': 'CjkUnifiedIdeograph7576',
u'\uf37a': 'UnicodeCharacter0xf37a',
u'\u1ffd': 'GreekOxia',
u'\u2501': 'BoxDrawingsHeavyHorizontal',
u'\u4480': 'CjkUnifiedIdeograph4480',
u'\ua305': 'YiSyllableNzy',
u'\uc284': 'HangulSyllableSwik',
u'\u6e07': 'CjkUnifiedIdeograph6e07',
u'\uf40b': 'UnicodeCharacter0xf40b',
u'\u0b8a': 'TamilLetterUu',
u'\u918e': 'CjkUnifiedIdeograph918e',
u'\u3d11': 'CjkUnifiedIdeograph3d11',
u'\u5c90': 'CjkUnifiedIdeograph5c90',
u'\ubb15': 'HangulSyllableMoet',
u'\uda94': 'UnicodeCharacter0xda94',
u'\u0c1b': 'TeluguLetterCha',
u'\u239a': 'ClearScreenSymbol',
u'\u8a1f': 'CjkUnifiedIdeograph8a1f',
u'\ua99e': 'UnicodeCharacter0xa99e',
u'\u5521': 'CjkUnifiedIdeograph5521',
u'\u74a0': 'CjkUnifiedIdeograph74a0',
u'\ud325': 'HangulSyllablePat',
u'\uf2a4': 'UnicodeCharacter0xf2a4',
u'\u242b': 'UnicodeCharacter0x242b',
u'\u3baa': 'CjkUnifiedIdeograph3baa',
u'\ua22f': 'YiSyllableGgur',
u'\uc1ae': 'HangulSyllableSwanh',
u'\u6d31': 'CjkUnifiedIdeograph6d31',
u'\ueb35': 'UnicodeCharacter0xeb35',
u'\u0ab4': 'UnicodeCharacter0xab4',
u'\u90b8': 'CjkUnifiedIdeograph90b8',
u'\u3c3b': 'CjkUnifiedIdeograph3c3b',
u'\u53ba': 'CjkUnifiedIdeograph53ba',
u'\uba3f': 'HangulSyllableMeod',
u'\ud9be': 'UnicodeCharacter0xd9be',
u'\u0345': 'CombiningGreekYpogegrammeni',
u'\u22c4': 'DiamondOperator',
u'\u8949': 'CjkUnifiedIdeograph8949',
u'\ua8c8': 'UnicodeCharacter0xa8c8',
u'\u544b': 'CjkUnifiedIdeograph544b',
u'\u6bca': 'CjkUnifiedIdeograph6bca',
u'\ud24f': 'HangulSyllableTweod',
u'\uf1ce': 'UnicodeCharacter0xf1ce',
u'\u1b55': 'BalineseDigitFive',
u'\u3ad4': 'CjkUnifiedIdeograph3ad4',
u'\ua159': 'YiSyllableNda',
u'\uc0d8': 'HangulSyllableSaem',
u'\u6c5b': 'CjkUnifiedIdeograph6c5b',
u'\uea5f': 'UnicodeCharacter0xea5f',
u'\u09de': 'UnicodeCharacter0x9de',
u'\u97e2': 'CjkUnifiedIdeograph97e2',
u'\u3365': 'IdeographicTelegraphSymbolForHourThirteen',
u'\u52e4': 'CjkUnifiedIdeograph52e4',
u'\ub969': 'HangulSyllableRyub',
u'\ud8e8': 'UnicodeCharacter0xd8e8',
u'\u026f': 'LatinSmallLetterTurnedM',
u'\u21ee': 'UpwardsWhiteDoubleArrow',
u'\u8873': 'CjkUnifiedIdeograph8873',
u'\uaff2': 'HangulSyllableGgwegg',
u'\u4b75': 'CjkUnifiedIdeograph4b75',
u'\u6af4': 'CjkUnifiedIdeograph6af4',
u'\ud179': 'HangulSyllableTyeob',
u'\uf0f8': 'UnicodeCharacter0xf0f8',
u'\u1a7f': 'UnicodeCharacter0x1a7f',
u'\u39fe': 'CjkUnifiedIdeograph39fe',
u'\u4c06': 'CjkUnifiedIdeograph4c06',
u'\u6889': 'CjkUnifiedIdeograph6889',
u'\u0908': 'DevanagariLetterIi',
u'\u358b': 'CjkUnifiedIdeograph358b',
u'\uca0a': 'HangulSyllableJjaebs',
u'\uf68d': 'UnicodeCharacter0xf68d',
u'\u970c': 'CjkUnifiedIdeograph970c',
u'\ub38f': 'HangulSyllableDyeogs',
u'\u6416': 'CjkUnifiedIdeograph6416',
u'\u2118': 'ScriptCapitalP',
u'\u4d9b': 'CjkUnifiedIdeograph4d9b',
u'\ue21a': 'UnicodeCharacter0xe21a',
u'\u0e9d': 'LaoLetterFoTam',
u'\uaf1c': 'HangulSyllableGgyels',
u'\ucb9f': 'HangulSyllableJjwigs',
u'\u94a1': 'CjkUnifiedIdeograph94a1',
u'\u7c26': 'CjkUnifiedIdeograph7c26',
u'\u3928': 'CjkUnifiedIdeograph3928',
u'\u65ab': 'CjkUnifiedIdeograph65ab',
u'\ufa2a': 'CjkCompatibilityIdeographfa2a',
u'\u26ad': 'MarriageSymbol',
u'\uc72c': 'HangulSyllableYuls',
u'\ue3af': 'UnicodeCharacter0xe3af',
u'\u802e': 'CjkUnifiedIdeograph802e',
u'\uacb1': 'HangulSyllableGyeolg',
u'\u5138': 'CjkUnifiedIdeograph5138',
u'\u7dbb': 'CjkUnifiedIdeograph7dbb',
u'\u123a': 'EthiopicSyllableShi',
u'\u3ebd': 'CjkUnifiedIdeograph3ebd',
u'\udf3c': 'UnicodeCharacter0xdf3c',
u'\ufbbf': 'UnicodeCharacter0xfbbf',
u'\u983e': 'CjkUnifiedIdeograph983e',
u'\uc4c1': 'HangulSyllableSswilg',
u'\u81c3': 'CjkUnifiedIdeograph81c3',
u'\u6948': 'CjkUnifiedIdeograph6948',
u'\u2a4a': 'UnionBesideAndJoinedWithUnion',
u'\u56cd': 'CjkUnifiedIdeograph56cd',
u'\uf74c': 'UnicodeCharacter0xf74c',
u'\u13cf': 'CherokeeLetterSi',
u'\ub04e': 'HangulSyllableGgeulm',
u'\udcd1': 'UnicodeCharacter0xdcd1',
u'\u99d3': 'CjkUnifiedIdeograph99d3',
u'\u425a': 'CjkUnifiedIdeograph425a',
u'\u6edd': 'CjkUnifiedIdeograph6edd',
u'\u0f5c': 'TibetanLetterDzha',
u'\u2bdf': 'UnicodeCharacter0x2bdf',
u'\uc85e': 'HangulSyllableJyelm',
u'\uf4e1': 'UnicodeCharacter0xf4e1',
u'\u9560': 'CjkUnifiedIdeograph9560',
u'\ub1e3': 'HangulSyllableNoec',
u'\u5a6a': 'CjkUnifiedIdeograph5a6a',
u'\u276c': 'MediumLeftpointingAngleBracketOrnament',
u'\u43ef': 'CjkUnifiedIdeograph43ef',
u'\ue06e': 'UnicodeCharacter0xe06e',
u'\u0cf1': 'KannadaSignJihvamuliya',
u'\uad70': 'HangulSyllableGun',
u'\uc9f3': 'HangulSyllableJjac',
u'\u8af5': 'CjkUnifiedIdeograph8af5',
u'\u727a': 'CjkUnifiedIdeograph727a',
u'\u3f7c': 'CjkUnifiedIdeograph3f7c',
u'\u5bff': 'CjkUnifiedIdeograph5bff',
u'\uf87e': 'UnicodeCharacter0xf87e',
u'\uc580': 'HangulSyllableYan',
u'\u6103': 'CjkUnifiedIdeograph6103',
u'\u8682': 'CjkUnifiedIdeograph8682',
u'\u2205': 'EmptySet',
u'\uef07': 'UnicodeCharacter0xef07',
u'\ua809': 'SylotiNagriLetterGo',
u'\u578c': 'CjkUnifiedIdeograph578c',
u'\u108e': 'MyanmarLetterRumaiPalaungFa',
u'\udd90': 'UnicodeCharacter0xdd90',
u'\u7913': 'CjkUnifiedIdeograph7913',
u'\u9e92': 'CjkUnifiedIdeograph9e92',
u'\u3a15': 'CjkUnifiedIdeograph3a15',
u'\u0717': 'SyriacLetterHe',
u'\uc019': 'HangulSyllableBbweng',
u'\u8d1b': 'CjkUnifiedIdeograph8d1b',
u'\u6f9c': 'CjkUnifiedIdeograph6f9c',
u'\u289e': 'BraillePatternDots23458',
u'\uf5a0': 'UnicodeCharacter0xf5a0',
u'\ub6a2': 'HangulSyllableDdunh',
u'\u5225': 'CjkUnifiedIdeograph5225',
u'\u1f27': 'GreekSmallLetterEtaWithDasiaAndPerispomeni',
u'\ud829': 'UnicodeCharacter0xd829',
u'\ua52b': 'VaiSyllableBhi',
u'\u40ae': 'CjkUnifiedIdeograph40ae',
u'\u0db0': 'SinhalaLetterMahaapraanaDayanna',
u'\uceb2': 'HangulSyllableKyanh',
u'\u6a35': 'CjkUnifiedIdeograph6a35',
u'\u8bb4': 'CjkUnifiedIdeograph8bb4',
u'\u3737': 'CjkUnifiedIdeograph3737',
u'\uf039': 'UnicodeCharacter0xf039',
u'\ubd3b': 'HangulSyllableBwaelh',
u'\u58be': 'CjkUnifiedIdeograph58be',
u'\u25c0': 'BlackLeftpointingTriangle',
u'\ue6c2': 'UnicodeCharacter0xe6c2',
u'\ua3c4': 'YiSyllableShyrx',
u'\u4f47': 'CjkUnifiedIdeograph4f47',
u'\u0849': 'UnicodeCharacter0x849',
u'\ud54b': 'HangulSyllablePilh',
u'\u964d': 'CjkUnifiedIdeograph964d',
u'\u70ce': 'CjkUnifiedIdeograph70ce',
u'\u3dd0': 'CjkUnifiedIdeograph3dd0',
u'\ufed2': 'ArabicLetterFehFinalForm',
u'\ubbd4': 'HangulSyllableMeuss',
u'\u6757': 'CjkUnifiedIdeograph6757',
u'\u84d6': 'CjkUnifiedIdeograph84d6',
u'\u2059': 'FiveDotPunctuation',
u'\ued5b': 'UnicodeCharacter0xed5b',
u'\uae5d': 'HangulSyllableGgab',
u'\u55e0': 'CjkUnifiedIdeograph55e0',
u'\u16e2': 'RunicLetterCweorth',
u'\ud3e4': 'HangulSyllablePyess',
u'\u7f67': 'CjkUnifiedIdeograph7f67',
u'\u9ce6': 'CjkUnifiedIdeograph9ce6',
u'\u3869': 'CjkUnifiedIdeograph3869',
u'\u056b': 'ArmenianSmallLetterIni',
u'\uc66d': 'HangulSyllableWaeb',
u'\u836f': 'CjkUnifiedIdeograph836f',
u'\u6df0': 'CjkUnifiedIdeograph6df0',
u'\u2ef2': 'CjkRadicalJsimplifiedTurtle',
u'\uebf4': 'UnicodeCharacter0xebf4',
u'\ub4f6': 'HangulSyllableDeup',
u'\u5079': 'CjkUnifiedIdeograph5079',
u'\u1d7b': 'LatinSmallCapitalLetterIWithStroke',
u'\ude7d': 'UnicodeCharacter0xde7d',
u'\u9b7f': 'CjkUnifiedIdeograph9b7f',
u'\ue781': 'UnicodeCharacter0xe781',
u'\u2c87': 'CopticSmallLetterDalda',
u'\u8a08': 'CjkUnifiedIdeograph8a08',
u'\ub68b': 'HangulSyllableDdyolb',
u'\u490a': 'CjkUnifiedIdeograph490a',
u'\u758d': 'CjkUnifiedIdeograph758d',
u'\ud30e': 'HangulSyllablePagg',
u'\uff91': 'HalfwidthKatakanaLetterMu',
u'\u1814': 'MongolianDigitFour',
u'\u4497': 'CjkUnifiedIdeograph4497',
u'\ua218': 'YiSyllableGgie',
u'\uce9b': 'HangulSyllableKaelb',
u'\u611a': 'CjkUnifiedIdeograph611a',
u'\ueb1e': 'UnicodeCharacter0xeb1e',
u'\u17a1': 'KhmerLetterLa',
u'\u91a5': 'CjkUnifiedIdeograph91a5',
u'\u3024': 'HangzhouNumeralFour',
u'\u5ca7': 'CjkUnifiedIdeograph5ca7',
u'\uba28': 'HangulSyllableMyaels',
u'\ue6ab': 'UnicodeCharacter0xe6ab',
u'\u792a': 'CjkUnifiedIdeograph792a',
u'\u032e': 'CombiningBreveBelow',
u'\u2fb1': 'KangxiRadicalTannedLeather',
u'\u8d32': 'CjkUnifiedIdeograph8d32',
u'\ua9b5': 'UnicodeCharacter0xa9b5',
u'\u4834': 'CjkUnifiedIdeograph4834',
u'\u74b7': 'CjkUnifiedIdeograph74b7',
u'\ud238': 'HangulSyllableTuls',
u'\ufebb': 'ArabicLetterSadInitialForm',
u'\u1b3e': 'BalineseVowelSignTaling',
u'\u47c1': 'CjkUnifiedIdeograph47c1',
u'\ua542': 'VaiSyllableYi',
u'\uc1c5': 'HangulSyllableSwaeg',
u'\u6044': 'CjkUnifiedIdeograph6044',
u'\uea48': 'UnicodeCharacter0xea48',
u'\u16cb': 'RunicLetterSigelLongbranchsolS',
u'\u90cf': 'CjkUnifiedIdeograph90cf',
u'\u334e': 'SquareYaado',
u'\u5fd1': 'CjkUnifiedIdeograph5fd1',
u'\ubd52': 'HangulSyllableBoelm',
u'\ud9d5': 'UnicodeCharacter0xd9d5',
u'\u7854': 'CjkUnifiedIdeograph7854',
u'\u0258': 'LatinSmallLetterReversedE',
u'\u2edb': 'CjkRadicalCsimplifiedWind',
u'\u8c5c': 'CjkUnifiedIdeograph8c5c',
u'\ua8df': 'UnicodeCharacter0xa8df',
u'\u4b5e': 'CjkUnifiedIdeograph4b5e',
u'\u77e1': 'CjkUnifiedIdeograph77e1',
u'\ud562': 'HangulSyllableHalm',
u'\uf1e5': 'UnicodeCharacter0xf1e5',
u'\u1a68': 'UnicodeCharacter0x1a68',
u'\u46eb': 'CjkUnifiedIdeograph46eb',
u'\ua46c': 'YiSyllableXyx',
u'\uc0ef': 'HangulSyllableSyalb',
u'\u636e': 'CjkUnifiedIdeograph636e',
u'\ued72': 'UnicodeCharacter0xed72',
u'\u09f5': 'BengaliCurrencyNumeratorTwo',
u'\u93f9': 'CjkUnifiedIdeograph93f9',
u'\u3278': 'CircledHangulKhieukhA',
u'\u5efb': 'CjkUnifiedIdeograph5efb',
u'\ubc7c': 'HangulSyllableByaess',
u'\ud8ff': 'UnicodeCharacter0xd8ff',
u'\u7b7e': 'CjkUnifiedIdeograph7b7e',
u'\ue203': 'UnicodeCharacter0xe203',
u'\u0582': 'ArmenianSmallLetterYiwn',
u'\u8f86': 'CjkUnifiedIdeograph8f86',
u'\u2b09': 'NorthWestBlackArrow',
u'\u4a88': 'CjkUnifiedIdeograph4a88',
u'\ub50d': 'HangulSyllableDying',
u'\ud48c': 'HangulSyllablePuss',
u'\u700f': 'CjkUnifiedIdeograph700f',
u'\ufa13': 'CjkCompatibilityIdeographfa13',
u'\u1d92': 'LatinSmallLetterEWithRetroflexHook',
u'\u8417': 'CjkUnifiedIdeograph8417',
u'\ua796': 'UnicodeCharacter0xa796',
u'\u4319': 'CjkUnifiedIdeograph4319',
u'\u6298': 'CjkUnifiedIdeograph6298',
u'\ucd1d': 'HangulSyllableCong',
u'\uec9c': 'UnicodeCharacter0xec9c',
u'\u1223': 'EthiopicSyllableSzaa',
u'\u35a2': 'CjkUnifiedIdeograph35a2',
u'\u9c27': 'CjkUnifiedIdeograph9c27',
u'\ubfa6': 'HangulSyllableBboebs',
u'\u5b29': 'CjkUnifiedIdeograph5b29',
u'\u7aa8': 'CjkUnifiedIdeograph7aa8',
u'\ue52d': 'UnicodeCharacter0xe52d',
u'\u04ac': 'CyrillicCapitalLetterTeWithDescender',
u'\u8eb0': 'CjkUnifiedIdeograph8eb0',
u'\u2a33': 'SmashProduct',
u'\u4db2': 'CjkUnifiedIdeograph4db2',
u'\ub437': 'HangulSyllableDyogs',
u'\ud7b6': 'UnicodeCharacter0xd7b6',
u'\u7339': 'CjkUnifiedIdeograph7339',
u'\ufd3d': 'ArabicLigatureAlefWithFathatanIsolatedForm',
u'\u1cbc': 'UnicodeCharacter0x1cbc',
u'\u8741': 'CjkUnifiedIdeograph8741',
u'\ua6c0': 'UnicodeCharacter0xa6c0',
u'\u4243': 'CjkUnifiedIdeograph4243',
u'\u65c2': 'CjkUnifiedIdeograph65c2',
u'\ucc47': 'HangulSyllableCaegs',
u'\uefc6': 'UnicodeCharacter0xefc6',
u'\u154d': 'CanadianSyllabicsWestcreeLa',
u'\u34cc': 'CjkUnifiedIdeograph34cc',
u'\u9f51': 'CjkUnifiedIdeograph9f51',
u'\ubed0': 'HangulSyllableBbeo',
u'\u5a53': 'CjkUnifiedIdeograph5a53',
u'\u7dd2': 'CjkUnifiedIdeograph7dd2',
u'\ue457': 'UnicodeCharacter0xe457',
u'\u07d6': 'NkoLetterJa',
u'\u81da': 'CjkUnifiedIdeograph81da',
u'\u2d5d': 'TifinaghLetterYath',
u'\u4cdc': 'CjkUnifiedIdeograph4cdc',
u'\ub761': 'HangulSyllableDdig',
u'\ud6e0': 'HangulSyllableHweo',
u'\u7263': 'CjkUnifiedIdeograph7263',
u'\ufc67': 'ArabicLigatureYehWithHamzaAboveWithNoonFinalForm',
u'\u1fe6': 'GreekSmallLetterUpsilonWithPerispomeni',
u'\u866b': 'CjkUnifiedIdeograph866b',
u'\u99ea': 'CjkUnifiedIdeograph99ea',
u'\u456d': 'CjkUnifiedIdeograph456d',
u'\u64ec': 'CjkUnifiedIdeograph64ec',
u'\ucf71': 'HangulSyllableKwag',
u'\ueef0': 'UnicodeCharacter0xeef0',
u'\u1477': 'CanadianSyllabicsWestcreeKwi',
u'\u37f6': 'CjkUnifiedIdeograph37f6',
u'\u9e7b': 'CjkUnifiedIdeograph9e7b',
u'\ub1fa': 'HangulSyllableNyobs',
u'\u5d7d': 'CjkUnifiedIdeograph5d7d',
u'\u7cfc': 'CjkUnifiedIdeograph7cfc',
u'\u6681': 'CjkUnifiedIdeograph6681',
u'\u0700': 'SyriacEndOfParagraph',
u'\u2383': 'EmphasisSymbol',
u'\uc402': 'HangulSyllableSswalp',
u'\ue085': 'UnicodeCharacter0xe085',
u'\u8104': 'CjkUnifiedIdeograph8104',
u'\uad87': 'HangulSyllableGuh',
u'\u520e': 'CjkUnifiedIdeograph520e',
u'\u7e91': 'CjkUnifiedIdeograph7e91',
u'\u1f10': 'GreekSmallLetterEpsilonWithPsili',
u'\u3b93': 'CjkUnifiedIdeograph3b93',
u'\udc12': 'UnicodeCharacter0xdc12',
u'\uf895': 'UnicodeCharacter0xf895',
u'\u9914': 'CjkUnifiedIdeograph9914',
u'\uc597': 'HangulSyllableYah',
u'\u8299': 'CjkUnifiedIdeograph8299',
u'\u6a1e': 'CjkUnifiedIdeograph6a1e',
u'\u3720': 'CjkUnifiedIdeograph3720',
u'\u53a3': 'CjkUnifiedIdeograph53a3',
u'\uf422': 'UnicodeCharacter0xf422',
u'\u10a5': 'GeorgianCapitalLetterVin',
u'\ub124': 'HangulSyllableNe',
u'\udda7': 'UnicodeCharacter0xdda7',
u'\u9aa9': 'CjkUnifiedIdeograph9aa9',
u'\u4f30': 'CjkUnifiedIdeograph4f30',
u'\u6bb3': 'CjkUnifiedIdeograph6bb3',
u'\u0c32': 'TeluguLetterLa',
u'\u28b5': 'BraillePatternDots13568',
u'\uc934': 'HangulSyllableJwe',
u'\uf5b7': 'UnicodeCharacter0xf5b7',
u'\u9636': 'CjkUnifiedIdeograph9636',
u'\ub2b9': 'HangulSyllableNyilt',
u'\u6740': 'CjkUnifiedIdeograph6740',
u'\u2442': 'OcrFork',
u'\u40c5': 'CjkUnifiedIdeograph40c5',
u'\ue144': 'UnicodeCharacter0xe144',
u'\u0dc7': 'UnicodeCharacter0xdc7',
u'\uae46': 'HangulSyllableGij',
u'\ucac9': 'HangulSyllableJjolt',
u'\u97cb': 'CjkUnifiedIdeograph97cb',
u'\u7f50': 'CjkUnifiedIdeograph7f50',
u'\u3c52': 'CjkUnifiedIdeograph3c52',
u'\u58d5': 'CjkUnifiedIdeograph58d5',
u'\uf954': 'CjkCompatibilityIdeographf954',
u'\u25d7': 'RightHalfBlackCircle',
u'\uc656': 'HangulSyllableWaj',
u'\ue2d9': 'UnicodeCharacter0xe2d9',
u'\u8358': 'CjkUnifiedIdeograph8358',
u'\uafdb': 'HangulSyllableGgweod',
u'\u5462': 'CjkUnifiedIdeograph5462',
u'\u70e5': 'CjkUnifiedIdeograph70e5',
u'\u1164': 'HangulJungseongYae',
u'\u3de7': 'CjkUnifiedIdeograph3de7',
u'\ude66': 'UnicodeCharacter0xde66',
u'\ufae9': 'UnicodeCharacter0xfae9',
u'\u9b68': 'CjkUnifiedIdeograph9b68',
u'\uc7eb': 'HangulSyllableJyaed',
u'\u84ed': 'CjkUnifiedIdeograph84ed',
u'\u6c72': 'CjkUnifiedIdeograph6c72',
u'\u2974': 'RightwardsArrowAboveTildeOperator',
u'\u55f7': 'CjkUnifiedIdeograph55f7',
u'\uf676': 'UnicodeCharacter0xf676',
u'\u12f9': 'EthiopicSyllableDdu',
u'\ub378': 'HangulSyllableDel',
u'\udffb': 'UnicodeCharacter0xdffb',
u'\u9cfd': 'CjkUnifiedIdeograph9cfd',
u'\ua601': 'VaiSyllableNje',
u'\u4184': 'CjkUnifiedIdeograph4184',
u'\u0e86': 'UnicodeCharacter0xe86',
u'\ucb88': 'HangulSyllableJjwel',
u'\u770b': 'CjkUnifiedIdeograph770b',
u'\u888a': 'CjkUnifiedIdeograph888a',
u'\u340d': 'CjkUnifiedIdeograph340d',
u'\uf10f': 'UnicodeCharacter0xf10f',
u'\ube11': 'HangulSyllableBeunj',
u'\u5994': 'CjkUnifiedIdeograph5994',
u'\u2696': 'Scales',
u'\ue398': 'UnicodeCharacter0xe398',
u'\ua09a': 'YiSyllableHma',
u'\u4c1d': 'CjkUnifiedIdeograph4c1d',
u'\u091f': 'DevanagariLetterTta',
u'\ud621': 'HangulSyllableHyenj',
u'\u9323': 'CjkUnifiedIdeograph9323',
u'\u71a4': 'CjkUnifiedIdeograph71a4',
u'\u3ea6': 'CjkUnifiedIdeograph3ea6',
u'\ufba8': 'ArabicLetterHehGoalInitialForm',
u'\ub8aa': 'HangulSyllableRwaej',
u'\u642d': 'CjkUnifiedIdeograph642d',
u'\u85ac': 'CjkUnifiedIdeograph85ac',
u'\u212f': 'ScriptSmallE',
u'\uee31': 'UnicodeCharacter0xee31',
u'\uab33': 'UnicodeCharacter0xab33',
u'\u56b6': 'CjkUnifiedIdeograph56b6',
u'\u13b8': 'CherokeeLetterLv',
u'\ud0ba': 'HangulSyllableKij',
u'\u7c3d': 'CjkUnifiedIdeograph7c3d',
u'\u9dbc': 'CjkUnifiedIdeograph9dbc',
u'\u393f': 'CjkUnifiedIdeograph393f',
u'\u0641': 'ArabicLetterFeh',
u'\uc343': 'HangulSyllableSsyas',
u'\u8045': 'CjkUnifiedIdeograph8045',
u'\u6ec6': 'CjkUnifiedIdeograph6ec6',
u'\u2bc8': 'UnicodeCharacter0x2bc8',
u'\ue8ca': 'UnicodeCharacter0xe8ca',
u'\ub5cc': 'HangulSyllableDdem',
u'\u514f': 'CjkUnifiedIdeograph514f',
u'\u1e51': 'LatinSmallLetterOWithMacronAndGrave',
u'\udb53': 'UnicodeCharacter0xdb53',
u'\u9855': 'CjkUnifiedIdeograph9855',
u'\u43d8': 'CjkUnifiedIdeograph43d8',
u'\xda': 'LatinCapitalLetterUWithAcute',
u'\ucddc': 'HangulSyllableCwem',
u'\u695f': 'CjkUnifiedIdeograph695f',
u'\u8ade': 'CjkUnifiedIdeograph8ade',
u'\u3661': 'CjkUnifiedIdeograph3661',
u'\uf363': 'UnicodeCharacter0xf363',
u'\ub065': 'HangulSyllableGgyinj',
u'\u5be8': 'CjkUnifiedIdeograph5be8',
u'\u18ea': 'UnicodeCharacter0x18ea',
u'\ue5ec': 'UnicodeCharacter0xe5ec',
u'\ua2ee': 'YiSyllableNzix',
u'\u4e71': 'CjkUnifiedIdeograph4e71',
u'\u0b73': 'UnicodeCharacter0xb73',
u'\uc875': 'HangulSyllableJonj',
u'\u9577': 'CjkUnifiedIdeograph9577',
u'\u73f8': 'CjkUnifiedIdeograph73f8',
u'\u30fa': 'KatakanaLetterVo',
u'\ufdfc': 'RialSign',
u'\ubafe': 'HangulSyllableMoegg',
u'\u8800': 'CjkUnifiedIdeograph8800',
u'\uac83': 'HangulSyllableGeos',
u'\u4b02': 'CjkUnifiedIdeograph4b02',
u'\u6f85': 'CjkUnifiedIdeograph6f85',
u'\uc106': 'HangulSyllableSyaenh',
u'\ue589': 'UnicodeCharacter0xe589',
u'\u1a0c': 'BugineseLetterCa',
u'\u3e8f': 'CjkUnifiedIdeograph3e8f',
u'\u9010': 'CjkUnifiedIdeograph9010',
u'\ub493': 'HangulSyllableDwelb',
u'\u5312': 'CjkUnifiedIdeograph5312',
u'\u7795': 'CjkUnifiedIdeograph7795',
u'\ue916': 'UnicodeCharacter0xe916',
u'\u0d99': 'UnicodeCharacter0xd99',
u'\u839d': 'CjkUnifiedIdeograph839d',
u'\u221c': 'FourthRoot',
u'\u469f': 'CjkUnifiedIdeograph469f',
u'\ub820': 'HangulSyllableRek',
u'\udca3': 'UnicodeCharacter0xdca3',
u'\u7b22': 'CjkUnifiedIdeograph7b22',
u'\uf126': 'UnicodeCharacter0xf126',
u'\u15a9': 'CanadianSyllabicsThcreeThii',
u'\uabad': 'UnicodeCharacter0xabad',
u'\u4a2c': 'CjkUnifiedIdeograph4a2c',
u'\u6eaf': 'CjkUnifiedIdeograph6eaf',
u'\uc030': 'HangulSyllableBbwim',
u'\ue4b3': 'UnicodeCharacter0xe4b3',
u'\u1936': 'LimbuSmallLetterMa',
u'\u3db9': 'CjkUnifiedIdeograph3db9',
u'\u9f3a': 'CjkUnifiedIdeograph9f3a',
u'\ub3bd': 'HangulSyllableDyeng',
u'\u523c': 'CjkUnifiedIdeograph523c',
u'\u76bf': 'CjkUnifiedIdeograph76bf',
u'\ue840': 'UnicodeCharacter0xe840',
u'\u0cc3': 'KannadaVowelSignVocalicR',
u'\u82c7': 'CjkUnifiedIdeograph82c7',
u'\u2146': 'DoublestruckItalicSmallD',
u'\u45c9': 'CjkUnifiedIdeograph45c9',
u'\ua74a': 'LatinCapitalLetterOWithLongStrokeOverlay',
u'\udbcd': 'UnicodeCharacter0xdbcd',
u'\u7a4c': 'CjkUnifiedIdeograph7a4c',
u'\uf050': 'UnicodeCharacter0xf050',
u'\u14d3': 'CanadianSyllabicsLe',
u'\uaad7': 'UnicodeCharacter0xaad7',
u'\u4956': 'CjkUnifiedIdeograph4956',
u'\u6dd9': 'CjkUnifiedIdeograph6dd9',
u'\ucf5a': 'HangulSyllableKonh',
u'\ue3dd': 'UnicodeCharacter0xe3dd',
u'\u1860': 'MongolianLetterSibeUe',
u'\u3ce3': 'CjkUnifiedIdeograph3ce3',
u'\u9e64': 'CjkUnifiedIdeograph9e64',
u'\ub2e7': 'HangulSyllableDags',
u'\u5166': 'CjkUnifiedIdeograph5166',
u'\u75e9': 'CjkUnifiedIdeograph75e9',
u'\ud76a': 'HangulSyllableHeup',
u'\u0bed': 'TamilDigitSeven',
u'\u81f1': 'CjkUnifiedIdeograph81f1',
u'\u2070': 'SuperscriptZero',
u'\u44f3': 'CjkUnifiedIdeograph44f3',
u'\ua674': 'UnicodeCharacter0xa674',
u'\udaf7': 'UnicodeCharacter0xdaf7',
u'\u7976': 'CjkUnifiedIdeograph7976',
u'\uff7a': 'HalfwidthKatakanaLetterKo',
u'\u13fd': 'UnicodeCharacter0x13fd',
u'\u2901': 'RightwardsTwoheadedArrowWithDoubleVerticalStroke',
u'\u4880': 'CjkUnifiedIdeograph4880',
u'\uaf05': 'HangulSyllableGgyeob',
u'\uce84': 'HangulSyllableKam',
u'\u6207': 'CjkUnifiedIdeograph6207',
u'\uf80b': 'UnicodeCharacter0xf80b',
u'\u078a': 'ThaanaLetterFaafu',
u'\u9d8e': 'CjkUnifiedIdeograph9d8e',
u'\u3111': 'BopomofoLetterQ',
u'\u5090': 'CjkUnifiedIdeograph5090',
u'\ub715': 'HangulSyllableDdyulg',
u'\ud694': 'HangulSyllableHoel',
u'\u2f9a': 'KangxiRadicalRed',
u'\u861f': 'CjkUnifiedIdeograph861f',
u'\ua59e': 'VaiSyllableMbu',
u'\u5921': 'CjkUnifiedIdeograph5921',
u'\u78a0': 'CjkUnifiedIdeograph78a0',
u'\udf25': 'UnicodeCharacter0xdf25',
u'\ufea4': 'ArabicLetterHahMedialForm',
u'\u282b': 'BraillePatternDots1246',
u'\u37aa': 'CjkUnifiedIdeograph37aa',
u'\uae2f': 'HangulSyllableGyih',
u'\ucdae': 'HangulSyllableCup',
u'\u6131': 'CjkUnifiedIdeograph6131',
u'\ue735': 'UnicodeCharacter0xe735',
u'\u06b4': 'ArabicLetterGafWithThreeDotsAbove',
u'\u9cb8': 'CjkUnifiedIdeograph9cb8',
u'\u303b': 'VerticalIdeographicIterationMark',
u'\u5fba': 'CjkUnifiedIdeograph5fba',
u'\ub63f': 'HangulSyllableDdwas',
u'\ud5be': 'HangulSyllableHyaebs',
u'\u0f45': 'TibetanLetterCa',
u'\u2ec4': 'CjkRadicalWestTwo',
u'\u8549': 'CjkUnifiedIdeograph8549',
u'\ua4c8': 'UnicodeCharacter0xa4c8',
u'\u584b': 'CjkUnifiedIdeograph584b',
u'\u67ca': 'CjkUnifiedIdeograph67ca',
u'\ude4f': 'UnicodeCharacter0xde4f',
u'\ufdce': 'UnicodeCharacter0xfdce',
u'\u1755': 'UnicodeCharacter0x1755',
u'\u36d4': 'CjkUnifiedIdeograph36d4',
u'\uad59': 'HangulSyllableGyolg',
u'\uccd8': 'HangulSyllableCyeol',
u'\u605b': 'CjkUnifiedIdeograph605b',
u'\ue65f': 'UnicodeCharacter0xe65f',
u'\u05de': 'HebrewLetterMem',
u'\u9be2': 'CjkUnifiedIdeograph9be2',
u'\u3f65': 'CjkUnifiedIdeograph3f65',
u'\u5ee4': 'CjkUnifiedIdeograph5ee4',
u'\ub569': 'HangulSyllableDdyag',
u'\ud4e8': 'HangulSyllablePyu',
u'\u0e6f': 'UnicodeCharacter0xe6f',
u'\u2dee': 'CombiningCyrillicLetterTe',
u'\u8473': 'CjkUnifiedIdeograph8473',
u'\ua3f2': 'YiSyllableJu',
u'\u4775': 'CjkUnifiedIdeograph4775',
u'\u66f4': 'CjkUnifiedIdeograph66f4',
u'\udd79': 'UnicodeCharacter0xdd79',
u'\ufcf8': 'ArabicLigatureAinWithYehIsolatedForm',
u'\u167f': 'UnicodeCharacter0x167f',
u'\u35fe': 'CjkUnifiedIdeograph35fe',
u'\u4006': 'CjkUnifiedIdeograph4006',
u'\u6489': 'CjkUnifiedIdeograph6489',
u'\u0508': 'CyrillicCapitalLetterKomiLje',
u'\u398b': 'CjkUnifiedIdeograph398b',
u'\uc60a': 'HangulSyllableYegg',
u'\ufa8d': 'CjkCompatibilityIdeographfa8d',
u'\u9b0c': 'CjkUnifiedIdeograph9b0c',
u'\ubf8f': 'HangulSyllableBbwaec',
u'\u6816': 'CjkUnifiedIdeograph6816',
u'\u2d18': 'GeorgianSmallLetterShin',
u'\u419b': 'CjkUnifiedIdeograph419b',
u'\uee1a': 'UnicodeCharacter0xee1a',
u'\u029d': 'LatinSmallLetterJWithCrossedtail',
u'\ua31c': 'YiSyllableSe',
u'\uc79f': 'HangulSyllableJalh',
u'\u98a1': 'CjkUnifiedIdeograph98a1',
u'\u7026': 'CjkUnifiedIdeograph7026',
u'\u3528': 'CjkUnifiedIdeograph3528',
u'\u69ab': 'CjkUnifiedIdeograph69ab',
u'\uf62a': 'UnicodeCharacter0xf62a',
u'\u2aad': 'LargerThanOrEqualTo',
u'\ucb2c': 'HangulSyllableJjyo',
u'\uefaf': 'UnicodeCharacter0xefaf',
u'\u8c2e': 'CjkUnifiedIdeograph8c2e',
u'\ua0b1': 'YiSyllableMip',
u'\u5d38': 'CjkUnifiedIdeograph5d38',
u'\u71bb': 'CjkUnifiedIdeograph71bb',
u'\u1e3a': 'LatinCapitalLetterLWithLineBelow',
u'\u32bd': 'CircledNumberFortyEight',
u'\ud33c': 'HangulSyllablePaess',
u'\uf7bf': 'UnicodeCharacter0xf7bf',
u'\u943e': 'CjkUnifiedIdeograph943e',
u'\uc8c1': 'HangulSyllableJwaet',
u'\u8dc3': 'CjkUnifiedIdeograph8dc3',
u'\u6548': 'CjkUnifiedIdeograph6548',
u'\u264a': 'Gemini',
u'\u5acd': 'CjkUnifiedIdeograph5acd',
u'\ufb4c': 'HebrewLetterBetWithRafe',
u'\u1fcf': 'GreekPsiliAndPerispomeni',
u'\ubc4e': 'HangulSyllableByagg',
u'\ud0d1': 'HangulSyllableTab',
u'\u95d3': 'CjkUnifiedIdeograph95d3',
u'\u4e5a': 'CjkUnifiedIdeograph4e5a',
u'\u62dd': 'CjkUnifiedIdeograph62dd',
u'\u035c': 'CombiningDoubleBreveBelow',
u'\u27df': 'UpTackWithCircleAbove',
u'\uc45e': 'HangulSyllableSsyoj',
u'\uf8e1': 'UnicodeCharacter0xf8e1',
u'\u9960': 'CjkUnifiedIdeograph9960',
u'\ubde3': 'HangulSyllableBwilh',
u'\u566a': 'CjkUnifiedIdeograph566a',
u'\u2b6c': 'UnicodeCharacter0x2b6c',
u'\u4fef': 'CjkUnifiedIdeograph4fef',
u'\uec6e': 'UnicodeCharacter0xec6e',
u'\xf1': 'LatinSmallLetterNWithTilde',
u'\ua170': 'YiSyllableHnat',
u'\uc5f3': 'HangulSyllableYeod',
u'\u86f5': 'CjkUnifiedIdeograph86f5',
u'\u7e7a': 'CjkUnifiedIdeograph7e7a',
u'\u337c': 'SquareEraNameSyouwa',
u'\u57ff': 'CjkUnifiedIdeograph57ff',
u'\uf47e': 'UnicodeCharacter0xf47e',
u'\uc980': 'HangulSyllableJyuss',
u'\u6d03': 'CjkUnifiedIdeograph6d03',
u'\u8a82': 'CjkUnifiedIdeograph8a82',
u'\u2e05': 'RightDottedSubstitutionBracket',
u'\ue307': 'UnicodeCharacter0xe307',
u'\ua409': 'YiSyllableQox',
u'\u5b8c': 'CjkUnifiedIdeograph5b8c',
u'\u1c8e': 'UnicodeCharacter0x1c8e',
u'\ud190': 'HangulSyllableTyels',
u'\u7513': 'CjkUnifiedIdeograph7513',
u'\u9292': 'CjkUnifiedIdeograph9292',
u'\u3615': 'CjkUnifiedIdeograph3615',
u'\u0b17': 'OriyaLetterGa',
u'\ucc19': 'HangulSyllableJjilt',
u'\u811b': 'CjkUnifiedIdeograph811b',
u'\u639c': 'CjkUnifiedIdeograph639c',
u'\u249e': 'ParenthesizedLatinSmallLetterC',
u'\uf9a0': 'CjkCompatibilityIdeographf9a0',
u'\ubaa2': 'HangulSyllableMyej',
u'\u5e25': 'CjkUnifiedIdeograph5e25',
u'\u1327': 'EthiopicSyllableThwa',
u'\ud429': 'HangulSyllablePwaenj',
u'\ua92b': 'KayahLiTonePlophu',
u'\u4cae': 'CjkUnifiedIdeograph4cae',
u'\u01b0': 'LatinSmallLetterUWithHorn',
u'\uc2b2': 'HangulSyllableSeulp',
u'\u6635': 'CjkUnifiedIdeograph6635',
u'\u87b4': 'CjkUnifiedIdeograph87b4',
u'\u3b37': 'CjkUnifiedIdeograph3b37',
u'\ufc39': 'ArabicLigatureKafWithHahIsolatedForm',
u'\ub13b': 'HangulSyllableNec',
u'\u54be': 'CjkUnifiedIdeograph54be',
u'\u29c0': 'CircledLessthan',
u'\ueac2': 'UnicodeCharacter0xeac2',
u'\uafc4': 'HangulSyllableGguls',
u'\u4347': 'CjkUnifiedIdeograph4347',
u'\u0449': 'CyrillicSmallLetterShcha',
u'\ud94b': 'UnicodeCharacter0xd94b',
u'\u9a4d': 'CjkUnifiedIdeograph9a4d',
u'\u7cce': 'CjkUnifiedIdeograph7cce',
u'\u31d0': 'CjkStrokeH',
u'\uf2d2': 'UnicodeCharacter0xf2d2',
u'\ub7d4': 'HangulSyllableRyaen',
u'\u6b57': 'CjkUnifiedIdeograph6b57',
u'\u88d6': 'CjkUnifiedIdeograph88d6',
u'\u2c59': 'GlagoliticSmallLetterIotatedBigYus',
u'\ue15b': 'UnicodeCharacter0xe15b',
u'\ua25d': 'YiSyllableNgiex',
u'\u59e0': 'CjkUnifiedIdeograph59e0',
u'\u1ae2': 'UnicodeCharacter0x1ae2',
u'\udfe4': 'UnicodeCharacter0xdfe4',
u'\u7367': 'CjkUnifiedIdeograph7367',
u'\u90e6': 'CjkUnifiedIdeograph90e6',
u'\u3469': 'CjkUnifiedIdeograph3469',
u'\u096b': 'DevanagariDigitFive',
u'\uca6d': 'HangulSyllableJjenj',
u'\u8f6f': 'CjkUnifiedIdeograph8f6f',
u'\u61f0': 'CjkUnifiedIdeograph61f0',
u'\u22f2': 'ElementOfWithLongHorizontalStroke',
u'\ue7f4': 'UnicodeCharacter0xe7f4',
u'\ub8f6': 'HangulSyllableRulp',
u'\u5c79': 'CjkUnifiedIdeograph5c79',
u'\u117b': 'HangulJungseongEou',
u'\ud27d': 'HangulSyllableTwet',
u'\u977f': 'CjkUnifiedIdeograph977f',
u'\ueb81': 'UnicodeCharacter0xeb81',
u'\u0c04': 'UnicodeCharacter0xc04',
u'\u2087': 'SubscriptSeven',
u'\u8608': 'CjkUnifiedIdeograph8608',
u'\uba8b': 'HangulSyllableMyeoh',
u'\u450a': 'CjkUnifiedIdeograph450a',
u'\u798d': 'CjkUnifiedIdeograph798d',
u'\udf0e': 'UnicodeCharacter0xdf0e',
u'\uf391': 'UnicodeCharacter0xf391',
u'\u1414': 'CanadianSyllabicsWoo',
u'\u4897': 'CjkUnifiedIdeograph4897',
u'\uae18': 'HangulSyllableGyin',
u'\uc29b': 'HangulSyllableSyus',
u'\u6d1a': 'CjkUnifiedIdeograph6d1a',
u'\ue71e': 'UnicodeCharacter0xe71e',
u'\u1ba1': 'SundaneseConsonantSignPamingkal',
u'\u9da5': 'CjkUnifiedIdeograph9da5',
u'\u3c24': 'CjkUnifiedIdeograph3c24',
u'\u50a7': 'CjkUnifiedIdeograph50a7',
u'\ub628': 'HangulSyllableDdok',
u'\ueaab': 'UnicodeCharacter0xeaab',
u'\u752a': 'CjkUnifiedIdeograph752a',
u'\u0f2e': 'TibetanDigitHalfFive',
u'\u23b1': 'UpperRightOrLowerLeftCurlyBracketSection',
u'\u8132': 'CjkUnifiedIdeograph8132',
u'\ua5b5': 'VaiSyllableNggu',
u'\u4434': 'CjkUnifiedIdeograph4434',
u'\u78b7': 'CjkUnifiedIdeograph78b7',
u'\ude38': 'UnicodeCharacter0xde38',
u'\uf2bb': 'UnicodeCharacter0xf2bb',
u'\u173e': 'UnicodeCharacter0x173e',
u'\u4bc1': 'CjkUnifiedIdeograph4bc1',
u'\ua942': 'RejangLetterMba',
u'\ucdc5': 'HangulSyllableCweong',
u'\u6c44': 'CjkUnifiedIdeograph6c44',
u'\ue648': 'UnicodeCharacter0xe648',
u'\u1acb': 'UnicodeCharacter0x1acb',
u'\u9ccf': 'CjkUnifiedIdeograph9ccf',
u'\u3f4e': 'CjkUnifiedIdeograph3f4e',
u'\u53d1': 'CjkUnifiedIdeograph53d1',
u'\ub152': 'HangulSyllableNyeobs',
u'\ud5d5': 'HangulSyllableHeolt',
u'\u7454': 'CjkUnifiedIdeograph7454',
u'\u0e58': 'ThaiDigitEight',
u'\u22db': 'GreaterthanEqualToOrLessthan',
u'\u805c': 'CjkUnifiedIdeograph805c',
u'\ua4df': 'UnicodeCharacter0xa4df',
u'\u475e': 'CjkUnifiedIdeograph475e',
u'\u7be1': 'CjkUnifiedIdeograph7be1',
u'\ud962': 'UnicodeCharacter0xd962',
u'\ufde5': 'UnicodeCharacter0xfde5',
u'\u1668': 'CanadianSyllabicsCarrierTtso',
u'\u4aeb': 'CjkUnifiedIdeograph4aeb',
u'\ua86c': 'PhagspaLetterNna',
u'\uccef': 'HangulSyllableCyegs',
u'\u6f6e': 'CjkUnifiedIdeograph6f6e',
u'\ue172': 'UnicodeCharacter0xe172',
u'\u05f5': 'UnicodeCharacter0x5f5',
u'\u9ff9': 'UnicodeCharacter0x9ff9',
u'\u3e78': 'CjkUnifiedIdeograph3e78',
u'\u52fb': 'CjkUnifiedIdeograph52fb',
u'\ub07c': 'HangulSyllableGgi',
u'\ud4ff': 'HangulSyllablePyuc',
u'\u777e': 'CjkUnifiedIdeograph777e',
u'\uee03': 'UnicodeCharacter0xee03',
u'\u0982': 'BengaliSignAnusvara',
u'\u8386': 'CjkUnifiedIdeograph8386',
u'\u2709': 'Envelope',
u'\u4688': 'CjkUnifiedIdeograph4688',
u'\ub90d': 'HangulSyllableRweolg',
u'\ud88c': 'UnicodeCharacter0xd88c',
u'\u7c0f': 'CjkUnifiedIdeograph7c0f',
u'\uf613': 'UnicodeCharacter0xf613',
u'\u1192': 'HangulJungseongYuye',
u'\u8817': 'CjkUnifiedIdeograph8817',
u'\uab96': 'UnicodeCharacter0xab96',
u'\u4f19': 'CjkUnifiedIdeograph4f19',
u'\u6e98': 'CjkUnifiedIdeograph6e98',
u'\uc11d': 'HangulSyllableSeog',
u'\ue09c': 'UnicodeCharacter0xe09c',
u'\u1e23': 'LatinSmallLetterHWithDotAbove',
u'\u39a2': 'CjkUnifiedIdeograph39a2',
u'\u9027': 'CjkUnifiedIdeograph9027',
u'\ub3a6': 'HangulSyllableDyeop',
u'\u5729': 'CjkUnifiedIdeograph5729',
u'\u76a8': 'CjkUnifiedIdeograph76a8',
u'\ue92d': 'UnicodeCharacter0xe92d',
u'\u08ac': 'UnicodeCharacter0x8ac',
u'\u82b0': 'CjkUnifiedIdeograph82b0',
u'\u2633': 'TrigramForThunder',
u'\u41b2': 'CjkUnifiedIdeograph41b2',
u'\ub837': 'HangulSyllableRyeos',
u'\udbb6': 'UnicodeCharacter0xdbb6',
u'\u7f39': 'CjkUnifiedIdeograph7f39',
u'\uf13d': 'UnicodeCharacter0xf13d',
u'\u10bc': 'GeorgianCapitalLetterCil',
u'\u8b41': 'CjkUnifiedIdeograph8b41',
u'\uaac0': 'UnicodeCharacter0xaac0',
u'\u4e43': 'CjkUnifiedIdeograph4e43',
u'\u69c2': 'CjkUnifiedIdeograph69c2',
u'\uc047': 'HangulSyllableBbyulb',
u'\ue3c6': 'UnicodeCharacter0xe3c6',
u'\u194d': 'LimbuDigitSeven',
u'\u38cc': 'CjkUnifiedIdeograph38cc',
u'\u9351': 'CjkUnifiedIdeograph9351',
u'\ub2d0': 'HangulSyllableNil',
u'\u5653': 'CjkUnifiedIdeograph5653',
u'\u71d2': 'CjkUnifiedIdeograph71d2',
u'\ue857': 'UnicodeCharacter0xe857',
u'\u0bd6': 'UnicodeCharacter0xbd6',
u'\u8dda': 'CjkUnifiedIdeograph8dda',
u'\u215d': 'VulgarFractionFiveEighths',
u'\u40dc': 'CjkUnifiedIdeograph40dc',
u'\ubb61': 'HangulSyllableMweob',
u'\udae0': 'UnicodeCharacter0xdae0',
u'\u7e63': 'CjkUnifiedIdeograph7e63',
u'\uf067': 'UnicodeCharacter0xf067',
u'\u13e6': 'CherokeeLetterTso',
u'\u8a6b': 'CjkUnifiedIdeograph8a6b',
u'\u95ea': 'CjkUnifiedIdeograph95ea',
u'\u496d': 'CjkUnifiedIdeograph496d',
u'\u68ec': 'CjkUnifiedIdeograph68ec',
u'\uc371': 'HangulSyllableSseolg',
u'\ue2f0': 'UnicodeCharacter0xe2f0',
u'\u1877': 'MongolianLetterManchuZha',
u'\u3bf6': 'CjkUnifiedIdeograph3bf6',
u'\u927b': 'CjkUnifiedIdeograph927b',
u'\ubdfa': 'HangulSyllableByulm',
u'\u517d': 'CjkUnifiedIdeograph517d',
u'\u70fc': 'CjkUnifiedIdeograph70fc',
u'\u6a81': 'CjkUnifiedIdeograph6a81',
u'\u0b00': 'UnicodeCharacter0xb00',
u'\u2f83': 'KangxiRadicalSelf',
u'\uc802': 'HangulSyllableJeogg',
u'\uec85': 'UnicodeCharacter0xec85',
u'\u8d04': 'CjkUnifiedIdeograph8d04',
u'\ua187': 'YiSyllableNuox',
u'\u5e0e': 'CjkUnifiedIdeograph5e0e',
u'\u7291': 'CjkUnifiedIdeograph7291',
u'\u1310': 'EthiopicSyllableGwa',
u'\u3793': 'CjkUnifiedIdeograph3793',
u'\ud012': 'HangulSyllableKweoj',
u'\uf495': 'UnicodeCharacter0xf495',
u'\u9514': 'CjkUnifiedIdeograph9514',
u'\uc997': 'HangulSyllableJeulh',
u'\u8e99': 'CjkUnifiedIdeograph8e99',
u'\u661e': 'CjkUnifiedIdeograph661e',
u'\u3b20': 'CjkUnifiedIdeograph3b20',
u'\u5fa3': 'CjkUnifiedIdeograph5fa3',
u'\uf822': 'UnicodeCharacter0xf822',
u'\u1ca5': 'UnicodeCharacter0x1ca5',
u'\ubd24': 'HangulSyllableBwass',
u'\ud1a7': 'HangulSyllableTod',
u'\u96a9': 'CjkUnifiedIdeograph96a9',
u'\u4330': 'CjkUnifiedIdeograph4330',
u'\u67b3': 'CjkUnifiedIdeograph67b3',
u'\u24b5': 'ParenthesizedLatinSmallLetterZ',
u'\uc534': 'HangulSyllableSsils',
u'\uf9b7': 'CjkCompatibilityIdeographf9b7',
u'\u9a36': 'CjkUnifiedIdeograph9a36',
u'\ubeb9': 'HangulSyllableBbyaenj',
u'\u6b40': 'CjkUnifiedIdeograph6b40',
u'\u2842': 'BraillePatternDots27',
u'\u4cc5': 'CjkUnifiedIdeograph4cc5',
u'\ued44': 'UnicodeCharacter0xed44',
u'\u01c7': 'LatinCapitalLetterLj',
u'\ua246': 'YiSyllableHxit',
u'\uc6c9': 'HangulSyllableUt',
u'\u9bcb': 'CjkUnifiedIdeograph9bcb',
u'\u7350': 'CjkUnifiedIdeograph7350',
u'\u3052': 'HiraganaLetterGe',
u'\u54d5': 'CjkUnifiedIdeograph54d5',
u'\uf554': 'UnicodeCharacter0xf554',
u'\u29d7': 'BlackHourglass',
u'\uca56': 'HangulSyllableJjeolm',
u'\ueed9': 'UnicodeCharacter0xeed9',
u'\u8f58': 'CjkUnifiedIdeograph8f58',
u'\ua3db': 'YiSyllableRyx',
u'\u5862': 'CjkUnifiedIdeograph5862',
u'\u7ce5': 'CjkUnifiedIdeograph7ce5',
u'\u1d64': 'LatinSubscriptSmallLetterU',
u'\u31e7': 'UnicodeCharacter0x31e7',
u'\ud266': 'HangulSyllableTwegg',
u'\uf6e9': 'UnicodeCharacter0xf6e9',
u'\u9768': 'CjkUnifiedIdeograph9768',
u'\ucbeb': 'HangulSyllableJjeuc',
u'\u88ed': 'CjkUnifiedIdeograph88ed',
u'\u6072': 'CjkUnifiedIdeograph6072',
u'\u2574': 'BoxDrawingsLightLeft',
u'\u59f7': 'CjkUnifiedIdeograph59f7',
u'\ufa76': 'CjkCompatibilityIdeographfa76',
u'\u1ef9': 'LatinSmallLetterYWithTilde',
u'\ubf78': 'HangulSyllableBbwae',
u'\ud3fb': 'HangulSyllablePolh',
u'\u90fd': 'CjkUnifiedIdeograph90fd',
u'\uaa01': 'ChamLetterI',
u'\u4d84': 'CjkUnifiedIdeograph4d84',
u'\u0286': 'LatinSmallLetterEshWithCurl',
u'\uc788': 'HangulSyllableIss',
u'\u7b0b': 'CjkUnifiedIdeograph7b0b',
u'\u848a': 'CjkUnifiedIdeograph848a',
u'\u380d': 'CjkUnifiedIdeograph380d',
u'\ufd0f': 'ArabicLigatureSadWithRehIsolatedForm',
u'\ub211': 'HangulSyllableNult',
u'\u5594': 'CjkUnifiedIdeograph5594',
u'\u2a96': 'SlantedEqualToOrGreaterthan',
u'\uef98': 'UnicodeCharacter0xef98',
u'\uac9a': 'HangulSyllableGelp',
u'\u401d': 'CjkUnifiedIdeograph401d',
u'\u051f': 'CyrillicSmallLetterAleutKa',
u'\uda21': 'UnicodeCharacter0xda21',
u'\u9f23': 'CjkUnifiedIdeograph9f23',
u'\u7da4': 'CjkUnifiedIdeograph7da4',
u'\u32a6': 'CircledIdeographLow',
u'\uf7a8': 'UnicodeCharacter0xf7a8',
u'\ub4aa': 'HangulSyllableDwinh',
u'\u682d': 'CjkUnifiedIdeograph682d',
u'\u89ac': 'CjkUnifiedIdeograph89ac',
u'\u2d2f': 'UnicodeCharacter0x2d2f',
u'\ue231': 'UnicodeCharacter0xe231',
u'\ua733': 'LatinSmallLetterAa',
u'\u5ab6': 'CjkUnifiedIdeograph5ab6',
u'\u1fb8': 'GreekCapitalLetterAlphaWithVrachy',
u'\udcba': 'UnicodeCharacter0xdcba',
u'\u703d': 'CjkUnifiedIdeograph703d',
u'\u91bc': 'CjkUnifiedIdeograph91bc',
u'\u353f': 'CjkUnifiedIdeograph353f',
u'\u0a41': 'GurmukhiVowelSignU',
u'\ucf43': 'HangulSyllableKyelb',
u'\u8c45': 'CjkUnifiedIdeograph8c45',
u'\u62c6': 'CjkUnifiedIdeograph62c6',
u'\u27c8': 'ReverseSolidusPrecedingSubset',
u'\ue4ca': 'UnicodeCharacter0xe4ca',
u'\ub9cc': 'HangulSyllableMan',
u'\u5d4f': 'CjkUnifiedIdeograph5d4f',
u'\u1251': 'EthiopicSyllableQhu',
u'\ud753': 'HangulSyllableHeugs',
u'\u9455': 'CjkUnifiedIdeograph9455',
u'\u4fd8': 'CjkUnifiedIdeograph4fd8',
u'\u0cda': 'UnicodeCharacter0xcda',
u'\uc1dc': 'HangulSyllableSwaek',
u'\u655f': 'CjkUnifiedIdeograph655f',
u'\u86de': 'CjkUnifiedIdeograph86de',
u'\u3a61': 'CjkUnifiedIdeograph3a61',
u'\uff63': 'HalfwidthRightCornerBracket',
u'\ubc65': 'HangulSyllableByat',
u'\u57e8': 'CjkUnifiedIdeograph57e8',
u'\u14ea': 'CanadianSyllabicsL',
u'\ue9ec': 'UnicodeCharacter0xe9ec',
u'\uaeee': 'HangulSyllableGgej',
u'\u4271': 'CjkUnifiedIdeograph4271',
u'\u0773': 'ArabicLetterAlefWithExtendedArabicindicDigitTwoAbove',
u'\uc475': 'HangulSyllableSsub',
u'\u9977': 'CjkUnifiedIdeograph9977',
u'\u7ff8': 'CjkUnifiedIdeograph7ff8',
u'\u3cfa': 'CjkUnifiedIdeograph3cfa',
u'\uf1fc': 'UnicodeCharacter0xf1fc',
u'\ub6fe': 'HangulSyllableDdwilp',
u'\u8c00': 'CjkUnifiedIdeograph8c00',
u'\ua883': 'SaurashtraLetterAa',
u'\u4f02': 'CjkUnifiedIdeograph4f02',
u'\u6b85': 'CjkUnifiedIdeograph6b85',
u'\uc506': 'HangulSyllableSseuj',
u'\ue189': 'UnicodeCharacter0xe189',
u'\u1e0c': 'LatinCapitalLetterDWithDotBelow',
u'\u3a8f': 'CjkUnifiedIdeograph3a8f',
u'\u9410': 'CjkUnifiedIdeograph9410',
u'\ub093': 'HangulSyllableGgic',
u'\u5712': 'CjkUnifiedIdeograph5712',
u'\u7395': 'CjkUnifiedIdeograph7395',
u'\ued16': 'UnicodeCharacter0xed16',
u'\u0999': 'BengaliLetterNga',
u'\u879d': 'CjkUnifiedIdeograph879d',
u'\u261c': 'WhiteLeftPointingIndex',
u'\u429f': 'CjkUnifiedIdeograph429f',
u'\ubc20': 'HangulSyllableBals',
u'\ud8a3': 'UnicodeCharacter0xd8a3',
u'\u7f22': 'CjkUnifiedIdeograph7f22',
u'\uf526': 'UnicodeCharacter0xf526',
u'\u11a9': 'HangulJongseongSsangkiyeok',
u'\uafad': 'HangulSyllableGgyob',
u'\u4e2c': 'CjkUnifiedIdeograph4e2c',
u'\u6aaf': 'CjkUnifiedIdeograph6aaf',
u'\uc430': 'HangulSyllableSsoen',
u'\ue0b3': 'UnicodeCharacter0xe0b3',
u'\u1d36': 'ModifierLetterCapitalJ',
u'\u39b9': 'CjkUnifiedIdeograph39b9',
u'\u9b3a': 'CjkUnifiedIdeograph9b3a',
u'\ub7bd': 'HangulSyllableRyalg',
u'\u563c': 'CjkUnifiedIdeograph563c',
u'\u72bf': 'CjkUnifiedIdeograph72bf',
u'\uec40': 'UnicodeCharacter0xec40',
u'\u08c3': 'UnicodeCharacter0x8c3',
u'\u86c7': 'CjkUnifiedIdeograph86c7',
u'\u2546': 'BoxDrawingsRightDownHeavyAndLeftUpLight',
u'\u41c9': 'CjkUnifiedIdeograph41c9',
u'\ua34a': 'YiSyllableZhuox',
u'\udfcd': 'UnicodeCharacter0xdfcd',
u'\u7e4c': 'CjkUnifiedIdeograph7e4c',
u'\uf450': 'UnicodeCharacter0xf450',
u'\u10d3': 'GeorgianLetterDon',
u'\uaed7': 'HangulSyllableGgeoh',
u'\u4d56': 'CjkUnifiedIdeograph4d56',
u'\u69d9': 'CjkUnifiedIdeograph69d9',
u'\ucb5a': 'HangulSyllableJjubs',
u'\ue7dd': 'UnicodeCharacter0xe7dd',
u'\u1c60': 'OlChikiLetterAak',
u'\u38e3': 'CjkUnifiedIdeograph38e3',
u'\u9a64': 'CjkUnifiedIdeograph9a64',
u'\ub6e7': 'HangulSyllableDdwes',
u'\u5566': 'CjkUnifiedIdeograph5566',
u'\u71e9': 'CjkUnifiedIdeograph71e9',
u'\ud36a': 'HangulSyllablePyaelm',
u'\u0fed': 'UnicodeCharacter0xfed',
u'\u85f1': 'CjkUnifiedIdeograph85f1',
u'\u2470': 'CircledNumberSeventeen',
u'\u40f3': 'CjkUnifiedIdeograph40f3',
u'\ua274': 'YiSyllableHap',
u'\udef7': 'UnicodeCharacter0xdef7',
u'\u7d76': 'CjkUnifiedIdeograph7d76',
u'\ufb7a': 'ArabicLetterTchehIsolatedForm',
u'\u17fd': 'UnicodeCharacter0x17fd',
u'\u2d01': 'GeorgianSmallLetterBan',
u'\u4c80': 'CjkUnifiedIdeograph4c80',
u'\uab05': 'UnicodeCharacter0xab05',
u'\uca84': 'HangulSyllableJjyeo',
u'\u6607': 'CjkUnifiedIdeograph6607',
u'\ufc0b': 'ArabicLigatureTehWithJeemIsolatedForm',
u'\u038a': 'GreekCapitalLetterIotaWithTonos',
u'\u998e': 'CjkUnifiedIdeograph998e',
u'\u3511': 'CjkUnifiedIdeograph3511',
u'\u5490': 'CjkUnifiedIdeograph5490',
u'\ub315': 'HangulSyllableDaeng',
u'\ud294': 'HangulSyllableTwiss',
u'\u041b': 'CyrillicCapitalLetterEl',
u'\u2b9a': 'UnicodeCharacter0x2b9a',
u'\u821f': 'CjkUnifiedIdeograph821f',
u'\ua19e': 'YiSyllableHlat',
u'\u5d21': 'CjkUnifiedIdeograph5d21',
u'\u7ca0': 'CjkUnifiedIdeograph7ca0',
u'\udb25': 'UnicodeCharacter0xdb25',
u'\ufaa4': 'CjkCompatibilityIdeographfaa4',
u'\u2c2b': 'GlagoliticCapitalLetterIzhitsa',
u'\u33aa': 'SquareKpa',
u'\uaa2f': 'ChamVowelSignO',
u'\uc9ae': 'HangulSyllableJyilm',
u'\u6531': 'CjkUnifiedIdeograph6531',
u'\ue335': 'UnicodeCharacter0xe335',
u'\u02b4': 'ModifierLetterSmallTurnedR',
u'\u98b8': 'CjkUnifiedIdeograph98b8',
u'\u343b': 'CjkUnifiedIdeograph343b',
u'\u5bba': 'CjkUnifiedIdeograph5bba',
u'\ub23f': 'HangulSyllableNwegs',
u'\ud1be': 'HangulSyllableTwagg',
u'\u0b45': 'UnicodeCharacter0xb45',
u'\u2ac4': 'SupersetOfOrEqualToWithDotAbove',
u'\u8149': 'CjkUnifiedIdeograph8149',
u'\ua0c8': 'YiSyllableMur',
u'\u5c4b': 'CjkUnifiedIdeograph5c4b',
u'\u63ca': 'CjkUnifiedIdeograph63ca',
u'\uda4f': 'UnicodeCharacter0xda4f',
u'\uf9ce': 'CjkCompatibilityIdeographf9ce',
u'\u1355': 'EthiopicSyllablePe',
u'\u32d4': 'CircledKatakanaO',
u'\ua959': 'UnicodeCharacter0xa959',
u'\uc8d8': 'HangulSyllableJoess',
u'\u645b': 'CjkUnifiedIdeograph645b',
u'\ue25f': 'UnicodeCharacter0xe25f',
u'\u01de': 'LatinCapitalLetterAWithDiaeresisAndMacron',
u'\u9fe2': 'UnicodeCharacter0x9fe2',
u'\u3b65': 'CjkUnifiedIdeograph3b65',
u'\u5ae4': 'CjkUnifiedIdeograph5ae4',
u'\ub169': 'HangulSyllableNyelt',
u'\ud0e8': 'HangulSyllableTaels',
u'\u0a6f': 'GurmukhiDigitNine',
u'\u29ee': 'ErrorbarredWhiteSquare',
u'\u8073': 'CjkUnifiedIdeograph8073',
u'\ua7f2': 'UnicodeCharacter0xa7f2',
u'\u4375': 'CjkUnifiedIdeograph4375',
u'\u62f4': 'CjkUnifiedIdeograph62f4',
u'\ud979': 'UnicodeCharacter0xd979',
u'\uf8f8': 'UnicodeCharacter0xf8f8',
u'\u127f': 'EthiopicSyllableCwa',
u'\u31fe': 'KatakanaLetterSmallRe',
u'\u4406': 'CjkUnifiedIdeograph4406',
u'\u6089': 'CjkUnifiedIdeograph6089',
u'\u0108': 'LatinCapitalLetterCWithCircumflex',
u'\u3d8b': 'CjkUnifiedIdeograph3d8b',
u'\uc20a': 'HangulSyllableSyolp',
u'\ufe8d': 'ArabicLetterAlefIsolatedForm',
u'\u9f0c': 'CjkUnifiedIdeograph9f0c',
u'\ubb8f': 'HangulSyllableMwid',
u'\u6c16': 'CjkUnifiedIdeograph6c16',
u'\u2918': 'RightwardsTwoheadedArrowWithTailWithDoubleVerticalStroke',
u'\u459b': 'CjkUnifiedIdeograph459b',
u'\uea1a': 'UnicodeCharacter0xea1a',
u'\u069d': 'ArabicLetterSadWithTwoDotsBelow',
u'\ua71c': 'ModifierLetterRaisedDownArrow',
u'\uc39f': 'HangulSyllableSseh',
u'\u9ca1': 'CjkUnifiedIdeograph9ca1',
u'\u7426': 'CjkUnifiedIdeograph7426',
u'\u3128': 'BopomofoLetterU',
u'\u6dab': 'CjkUnifiedIdeograph6dab',
u'\uf22a': 'UnicodeCharacter0xf22a',
u'\u2ead': 'CjkRadicalSpiritTwo',
u'\ucf2c': 'HangulSyllableKyeom',
u'\uebaf': 'UnicodeCharacter0xebaf',
u'\u882e': 'CjkUnifiedIdeograph882e',
u'\ua4b1': 'YiRadicalVep',
u'\u5938': 'CjkUnifiedIdeograph5938',
u'\u75bb': 'CjkUnifiedIdeograph75bb',
u'\u1a3a': 'UnicodeCharacter0x1a3a',
u'\u36bd': 'CjkUnifiedIdeograph36bd',
u'\ud73c': 'HangulSyllableHyul',
u'\uf3bf': 'UnicodeCharacter0xf3bf',
u'\u903e': 'CjkUnifiedIdeograph903e',
u'\uccc1': 'HangulSyllableCelt',
u'\u89c3': 'CjkUnifiedIdeograph89c3',
u'\u6148': 'CjkUnifiedIdeograph6148',
u'\u224a': 'AlmostEqualOrEqualTo',
u'\u5ecd': 'CjkUnifiedIdeograph5ecd',
u'\uff4c': 'FullwidthLatinSmallLetterL',
u'\u1bcf': 'UnicodeCharacter0x1bcf',
u'\ub84e': 'HangulSyllableRyelp',
u'\ud4d1': 'HangulSyllablePwinj',
u'\u91d3': 'CjkUnifiedIdeograph91d3',
u'\u4a5a': 'CjkUnifiedIdeograph4a5a',
u'\u66dd': 'CjkUnifiedIdeograph66dd',
u'\u075c': 'ArabicLetterSeenWithFourDotsAbove',
u'\u23df': 'BottomCurlyBracket',
u'\uc05e': 'HangulSyllableBbeunh',
u'\ufce1': 'ArabicLigatureBehWithMeemMedialForm',
u'\u9d60': 'CjkUnifiedIdeograph9d60',
u'\ub9e3': 'HangulSyllableMah',
u'\u526a': 'CjkUnifiedIdeograph526a',
u'\u2f6c': 'KangxiRadicalEye',
u'\u4bef': 'CjkUnifiedIdeograph4bef',
u'\ue86e': 'UnicodeCharacter0xe86e',
u'\u04f1': 'CyrillicSmallLetterUWithDiaeresis',
u'\ua570': 'VaiSyllableNya',
u'\uc1f3': 'HangulSyllableSoes',
u'\u82f5': 'CjkUnifiedIdeograph82f5',
u'\u7a7a': 'CjkUnifiedIdeograph7a7a',
u'\u377c': 'CjkUnifiedIdeograph377c',
u'\u53ff': 'CjkUnifiedIdeograph53ff',
u'\uf07e': 'UnicodeCharacter0xf07e',
u'\ucd80': 'HangulSyllableCyol',
u'\u6903': 'CjkUnifiedIdeograph6903',
u'\u8e82': 'CjkUnifiedIdeograph8e82',
u'\u2a05': 'NarySquareIntersectionOperator',
u'\ue707': 'UnicodeCharacter0xe707',
u'\ua009': 'YiSyllableAx',
u'\u5f8c': 'CjkUnifiedIdeograph5f8c',
u'\u188e': 'MongolianLetterAliGaliDda',
u'\ud590': 'HangulSyllableHya',
u'\u7113': 'CjkUnifiedIdeograph7113',
u'\u9692': 'CjkUnifiedIdeograph9692',
u'\u3215': 'ParenthesizedHangulIeungA',
u'\u0f17': 'TibetanAstrologicalSignSgraGcanCharRtags',
u'\uc819': 'HangulSyllableJeot',
u'\u851b': 'CjkUnifiedIdeograph851b',
u'\u679c': 'CjkUnifiedIdeograph679c',
u'\u209e': 'UnicodeCharacter0x209e',
u'\ufda0': 'ArabicLigatureTehWithJeemWithAlefMaksuraFinalForm',
u'\ubea2': 'HangulSyllableBbyalm',
u'\u5a25': 'CjkUnifiedIdeograph5a25',
u'\u1727': 'HanunooLetterDa',
u'\ud029': 'HangulSyllableKweb',
u'\uad2b': 'HangulSyllableGwaes',
u'\u48ae': 'CjkUnifiedIdeograph48ae',
u'\u05b0': 'HebrewPointSheva',
u'\uc6b2': 'HangulSyllableUgg',
u'\u6235': 'CjkUnifiedIdeograph6235',
u'\u83b4': 'CjkUnifiedIdeograph83b4',
u'\u3f37': 'CjkUnifiedIdeograph3f37',
u'\uf839': 'UnicodeCharacter0xf839',
u'\ub53b': 'HangulSyllableDdalb',
u'\u50be': 'CjkUnifiedIdeograph50be',
u'\u2dc0': 'EthiopicSyllableQya',
u'\ueec2': 'UnicodeCharacter0xeec2',
u'\uabc4': 'UnicodeCharacter0xabc4',
u'\u4747': 'CjkUnifiedIdeograph4747',
u'\udd4b': 'UnicodeCharacter0xdd4b',
u'\u9e4d': 'CjkUnifiedIdeograph9e4d',
u'\u78ce': 'CjkUnifiedIdeograph78ce',
u'\u35d0': 'CjkUnifiedIdeograph35d0',
u'\uf6d2': 'UnicodeCharacter0xf6d2',
u'\ub3d4': 'HangulSyllableDom',
u'\u6f57': 'CjkUnifiedIdeograph6f57',
u'\u8cd6': 'CjkUnifiedIdeograph8cd6',
u'\u2859': 'BraillePatternDots1457',
u'\ue55b': 'UnicodeCharacter0xe55b',
u'\ua65d': 'CyrillicSmallLetterIotifiedClosedLittleYus',
u'\u5de0': 'CjkUnifiedIdeograph5de0',
u'\u1ee2': 'LatinCapitalLetterOWithHornAndDotBelow',
u'\udbe4': 'UnicodeCharacter0xdbe4',
u'\u7767': 'CjkUnifiedIdeograph7767',
u'\u94e6': 'CjkUnifiedIdeograph94e6',
u'\u3069': 'HiraganaLetterDo',
u'\u0d6b': 'MalayalamDigitFive',
u'\uce6d': 'HangulSyllableCing',
u'\u8b6f': 'CjkUnifiedIdeograph8b6f',
u'\u65f0': 'CjkUnifiedIdeograph65f0',
u'\u26f2': 'UnicodeCharacter0x26f2',
u'\ue3f4': 'UnicodeCharacter0xe3f4',
u'\ubcf6': 'HangulSyllableBogg',
u'\u5879': 'CjkUnifiedIdeograph5879',
u'\u157b': 'CanadianSyllabicsNunavikH',
u'\ud67d': 'HangulSyllableHwaelt',
u'\u937f': 'CjkUnifiedIdeograph937f',
u'\uef81': 'UnicodeCharacter0xef81',
u'\u0804': 'UnicodeCharacter0x804',
u'\u2487': 'ParenthesizedNumberTwenty',
u'\u8208': 'CjkUnifiedIdeograph8208',
u'\ube8b': 'HangulSyllableBbaelh',
u'\u410a': 'CjkUnifiedIdeograph410a',
u'\u7d8d': 'CjkUnifiedIdeograph7d8d',
u'\udb0e': 'UnicodeCharacter0xdb0e',
u'\uf791': 'UnicodeCharacter0xf791',
u'\u1014': 'MyanmarLetterNa',
u'\u4c97': 'CjkUnifiedIdeograph4c97',
u'\uaa18': 'ChamLetterNa',
u'\uc69b': 'HangulSyllableYod',
u'\u691a': 'CjkUnifiedIdeograph691a',
u'\ue31e': 'UnicodeCharacter0xe31e',
u'\u1fa1': 'GreekSmallLetterOmegaWithDasiaAndYpogegrammeni',
u'\u99a5': 'CjkUnifiedIdeograph99a5',
u'\u3824': 'CjkUnifiedIdeograph3824',
u'\u54a7': 'CjkUnifiedIdeograph54a7',
u'\ub228': 'HangulSyllableNweol',
u'\ueeab': 'UnicodeCharacter0xeeab',
u'\u712a': 'CjkUnifiedIdeograph712a',
u'\u0b2e': 'OriyaLetterMa',
u'\u27b1': 'NotchedUpperRightshadowedWhiteRightwardsArrow',
u'\u8532': 'CjkUnifiedIdeograph8532',
u'\ua1b5': 'YiSyllableHlyrx',
u'\u4034': 'CjkUnifiedIdeograph4034',
u'\u7cb7': 'CjkUnifiedIdeograph7cb7',
u'\uda38': 'UnicodeCharacter0xda38',
u'\uf6bb': 'UnicodeCharacter0xf6bb',
u'\u133e': 'EthiopicSyllableTso',
u'\u4fc1': 'CjkUnifiedIdeograph4fc1',
u'\uad42': 'HangulSyllableGoelp',
u'\uc9c5': 'HangulSyllableJinj',
u'\u6844': 'CjkUnifiedIdeograph6844',
u'\ue248': 'UnicodeCharacter0xe248',
u'\u1ecb': 'LatinSmallLetterIWithDotBelow',
u'\u98cf': 'CjkUnifiedIdeograph98cf',
u'\u3b4e': 'CjkUnifiedIdeograph3b4e',
u'\u57d1': 'CjkUnifiedIdeograph57d1',
u'\ub552': 'HangulSyllableDdaenh',
u'\ud1d5': 'HangulSyllableTwat',
u'\u7054': 'CjkUnifiedIdeograph7054',
u'\u0a58': 'UnicodeCharacter0xa58',
u'\u26db': 'UnicodeCharacter0x26db',
u'\u845c': 'CjkUnifiedIdeograph845c',
u'\ua0df': 'YiSyllableFyx',
u'\u435e': 'CjkUnifiedIdeograph435e',
u'\u7fe1': 'CjkUnifiedIdeograph7fe1',
u'\udd62': 'UnicodeCharacter0xdd62',
u'\uf9e5': 'CjkCompatibilityIdeographf9e5',
u'\u1268': 'EthiopicSyllableVa',
u'\u4eeb': 'CjkUnifiedIdeograph4eeb',
u'\uac6c': 'HangulSyllableGyaek',
u'\uc8ef': 'HangulSyllableJyolh',
u'\u6b6e': 'CjkUnifiedIdeograph6b6e',
u'\ue572': 'UnicodeCharacter0xe572',
u'\u01f5': 'LatinSmallLetterGWithAcute',
u'\u9bf9': 'CjkUnifiedIdeograph9bf9',
u'\u3a78': 'CjkUnifiedIdeograph3a78',
u'\u56fb': 'CjkUnifiedIdeograph56fb',
u'\ub47c': 'HangulSyllableDweom',
u'\ud0ff': 'HangulSyllableTyad',
u'\u737e': 'CjkUnifiedIdeograph737e',
u'\uea03': 'UnicodeCharacter0xea03',
u'\u0d82': 'SinhalaSignAnusvaraya',
u'\u8786': 'CjkUnifiedIdeograph8786',
u'\u2309': 'RightCeiling',
u'\u4288': 'CjkUnifiedIdeograph4288',
u'\ubd0d': 'HangulSyllableBot',
u'\udc8c': 'UnicodeCharacter0xdc8c',
u'\u780f': 'CjkUnifiedIdeograph780f',
u'\uf213': 'UnicodeCharacter0xf213',
u'\u1592': 'CanadianSyllabicsNgoo',
u'\u8c17': 'CjkUnifiedIdeograph8c17',
u'\uaf96': 'HangulSyllableGgoej',
u'\u4b19': 'CjkUnifiedIdeograph4b19',
u'\u6a98': 'CjkUnifiedIdeograph6a98',
u'\uc51d': 'HangulSyllableSsyib',
u'\ue49c': 'UnicodeCharacter0xe49c',
u'\u1a23': 'UnicodeCharacter0x1a23',
u'\u3da2': 'CjkUnifiedIdeograph3da2',
u'\u9427': 'CjkUnifiedIdeograph9427',
u'\ub7a6': 'HangulSyllableRaelp',
u'\u5329': 'CjkUnifiedIdeograph5329',
u'\u72a8': 'CjkUnifiedIdeograph72a8',
u'\ued2d': 'UnicodeCharacter0xed2d',
u'\u0cac': 'KannadaLetterBa',
u'\u86b0': 'CjkUnifiedIdeograph86b0',
u'\u2233': 'AnticlockwiseContourIntegral',
u'\u45b2': 'CjkUnifiedIdeograph45b2',
u'\ubc37': 'HangulSyllableBaed',
u'\udfb6': 'UnicodeCharacter0xdfb6',
u'\u7b39': 'CjkUnifiedIdeograph7b39',
u'\uf53d': 'UnicodeCharacter0xf53d',
u'\u14bc': 'CanadianSyllabicsWestcreeM',
u'\u8f41': 'CjkUnifiedIdeograph8f41',
u'\uaec0': 'HangulSyllableGgeon',
u'\u4a43': 'CjkUnifiedIdeograph4a43',
u'\u6dc2': 'CjkUnifiedIdeograph6dc2',
u'\uc447': 'HangulSyllableSsoeh',
u'\ue7c6': 'UnicodeCharacter0xe7c6',
u'\u1d4d': 'ModifierLetterSmallG',
u'\u3ccc': 'CjkUnifiedIdeograph3ccc',
u'\u9751': 'CjkUnifiedIdeograph9751',
u'\ub6d0': 'HangulSyllableDdweok',
u'\u5253': 'CjkUnifiedIdeograph5253',
u'\u75d2': 'CjkUnifiedIdeograph75d2',
u'\uec57': 'UnicodeCharacter0xec57',
u'\u0fd6': 'UnicodeCharacter0xfd6',
u'\u89da': 'CjkUnifiedIdeograph89da',
u'\u255d': 'BoxDrawingsDoubleUpAndLeft',
u'\u44dc': 'CjkUnifiedIdeograph44dc',
u'\ubf61': 'HangulSyllableBbwanj',
u'\udee0': 'UnicodeCharacter0xdee0',
u'\u7a63': 'CjkUnifiedIdeograph7a63',
u'\uf467': 'UnicodeCharacter0xf467',
u'\u17e6': 'KhmerDigitSix',
u'\u8e6b': 'CjkUnifiedIdeograph8e6b',
u'\u91ea': 'CjkUnifiedIdeograph91ea',
u'\u4d6d': 'CjkUnifiedIdeograph4d6d',
u'\u6cec': 'CjkUnifiedIdeograph6cec',
u'\uc771': 'HangulSyllableYit',
u'\ue6f0': 'UnicodeCharacter0xe6f0',
u'\u1c77': 'OlChikiLetterOh',
u'\u3ff6': 'CjkUnifiedIdeograph3ff6',
u'\u967b': 'CjkUnifiedIdeograph967b',
u'\ub9fa': 'HangulSyllableMaej',
u'\u557d': 'CjkUnifiedIdeograph557d',
u'\u74fc': 'CjkUnifiedIdeograph74fc',
u'\u6e81': 'CjkUnifiedIdeograph6e81',
u'\u0f00': 'TibetanSyllableOm',
u'\u2b83': 'UnicodeCharacter0x2b83',
u'\ucc02': 'HangulSyllableJjyibs',
u'\ue885': 'UnicodeCharacter0xe885',
u'\u8904': 'CjkUnifiedIdeograph8904',
u'\ua587': 'VaiSyllableSoo',
u'\u5a0e': 'CjkUnifiedIdeograph5a0e',
u'\u7691': 'CjkUnifiedIdeograph7691',
u'\u1710': 'TagalogLetterSa',
u'\u3393': 'SquareGhz',
u'\ud412': 'HangulSyllablePwalm',
u'\uf095': 'UnicodeCharacter0xf095',
u'\u9114': 'CjkUnifiedIdeograph9114',
u'\ucd97': 'HangulSyllableCugs',
u'\u8a99': 'CjkUnifiedIdeograph8a99',
u'\u621e': 'CjkUnifiedIdeograph621e',
u'\u3f20': 'CjkUnifiedIdeograph3f20',
u'\u5ba3': 'CjkUnifiedIdeograph5ba3',
u'\ufc22': 'ArabicLigatureDadWithJeemIsolatedForm',
u'\u18a5': 'MongolianLetterManchuAliGaliZa',
u'\ub924': 'HangulSyllableRwen',
u'\ud5a7': 'HangulSyllableHyac',
u'\u92a9': 'CjkUnifiedIdeograph92a9',
u'\u4730': 'CjkUnifiedIdeograph4730',
u'\u63b3': 'CjkUnifiedIdeograph63b3',
u'\u0432': 'CyrillicSmallLetterVe',
u'\u20b5': 'CediSign',
u'\uc134': 'HangulSyllableSeok',
u'\ufdb7': 'ArabicLigatureKafWithMeemWithYehFinalForm',
u'\u9e36': 'CjkUnifiedIdeograph9e36',
u'\ubab9': 'HangulSyllableMob',
u'\u6f40': 'CjkUnifiedIdeograph6f40',
u'\u2c42': 'GlagoliticSmallLetterPokoji',
u'\u48c5': 'CjkUnifiedIdeograph48c5',
u'\ue944': 'UnicodeCharacter0xe944',
u'\u05c7': 'HebrewPointQamatsQatan',
u'\ua646': 'CyrillicCapitalLetterIota',
u'\uc2c9': 'HangulSyllableSyilg',
u'\u9fcb': 'UnicodeCharacter0x9fcb',
u'\u7750': 'CjkUnifiedIdeograph7750',
u'\u3452': 'CjkUnifiedIdeograph3452',
u'\u50d5': 'CjkUnifiedIdeograph50d5',
u'\uf154': 'UnicodeCharacter0xf154',
u'\u2dd7': 'UnicodeCharacter0x2dd7',
u'\uce56': 'HangulSyllableCyip',
u'\uead9': 'UnicodeCharacter0xead9',
u'\u8b58': 'CjkUnifiedIdeograph8b58',
u'\ua7db': 'UnicodeCharacter0xa7db',
u'\u5c62': 'CjkUnifiedIdeograph5c62',
u'\u78e5': 'CjkUnifiedIdeograph78e5',
u'\u1964': 'TaiLeLetterI',
u'\u35e7': 'CjkUnifiedIdeograph35e7',
u'\ud666': 'HangulSyllableHwabs',
u'\uf2e9': 'UnicodeCharacter0xf2e9',
u'\u9368': 'CjkUnifiedIdeograph9368',
u'\ucfeb': 'HangulSyllableKulb',
u'\u8ced': 'CjkUnifiedIdeograph8ced',
u'\u6472': 'CjkUnifiedIdeograph6472',
u'\u2174': 'SmallRomanNumeralFive',
u'\u5df7': 'CjkUnifiedIdeograph5df7',
u'\ufe76': 'ArabicFathaIsolatedForm',
u'\u1af9': 'UnicodeCharacter0x1af9',
u'\ubb78': 'HangulSyllableMwels',
u'\ud7fb': 'UnicodeCharacter0xd7fb',
u'\u94fd': 'CjkUnifiedIdeograph94fd',
u'\uae01': 'HangulSyllableGeulg',
u'\u4984': 'CjkUnifiedIdeograph4984',
u'\u0686': 'ArabicLetterTcheh',
u'\uc388': 'HangulSyllableSsen',
u'\u7f0b': 'CjkUnifiedIdeograph7f0b',
u'\u808a': 'CjkUnifiedIdeograph808a',
u'\u3c0d': 'CjkUnifiedIdeograph3c0d',
u'\uf90f': 'CjkCompatibilityIdeographf90f',
u'\ub611': 'HangulSyllableDdog',
u'\u5194': 'CjkUnifiedIdeograph5194',
u'\u2e96': 'CjkRadicalHeartOne',
u'\ueb98': 'UnicodeCharacter0xeb98',
u'\ua89a': 'SaurashtraLetterJha',
u'\u441d': 'CjkUnifiedIdeograph441d',
u'\u011f': 'LatinSmallLetterGWithBreve',
u'\ude21': 'UnicodeCharacter0xde21',
u'\u9b23': 'CjkUnifiedIdeograph9b23',
u'\u79a4': 'CjkUnifiedIdeograph79a4',
u'\u36a6': 'CjkUnifiedIdeograph36a6',
u'\uf3a8': 'UnicodeCharacter0xf3a8',
u'\ub0aa': 'HangulSyllableNabs',
u'\u6c2d': 'CjkUnifiedIdeograph6c2d',
u'\u8dac': 'CjkUnifiedIdeograph8dac',
u'\u292f': 'FallingDiagonalCrossingNorthEastArrow',
u'\ue631': 'UnicodeCharacter0xe631',
u'\ua333': 'YiSyllableSsa',
u'\u5eb6': 'CjkUnifiedIdeograph5eb6',
u'\u1bb8': 'SundaneseDigitEight',
u'\ud8ba': 'UnicodeCharacter0xd8ba',
u'\u743d': 'CjkUnifiedIdeograph743d',
u'\u95bc': 'CjkUnifiedIdeograph95bc',
u'\u313f': 'HangulLetterRieulphieuph',
u'\u0e41': 'ThaiCharacterSaraAe',
u'\ucb43': 'HangulSyllableJjyoc',
u'\u8845': 'CjkUnifiedIdeograph8845',
u'\u66c6': 'CjkUnifiedIdeograph66c6',
u'\u23c8': 'DentistrySymbolLightUpAndHorizontalWithWave',
u'\ue0ca': 'UnicodeCharacter0xe0ca',
u'\ubdcc': 'HangulSyllableBwess',
u'\u594f': 'CjkUnifiedIdeograph594f',
u'\u1651': 'CanadianSyllabicsCarrierSee',
u'\ud353': 'HangulSyllablePyalh',
u'\u9055': 'CjkUnifiedIdeograph9055',
u'\u4bd8': 'CjkUnifiedIdeograph4bd8',
u'\u08da': 'UnicodeCharacter0x8da',
u'\uc5dc': 'HangulSyllableEls',
u'\u615f': 'CjkUnifiedIdeograph615f',
u'\u82de': 'CjkUnifiedIdeograph82de',
u'\u3e61': 'CjkUnifiedIdeograph3e61',
u'\ufb63': 'ArabicLetterTehehFinalForm',
u'\ub865': 'HangulSyllableRolg',
u'\u53e8': 'CjkUnifiedIdeograph53e8',
u'\u10ea': 'GeorgianLetterCan',
u'\uedec': 'UnicodeCharacter0xedec',
u'\uaaee': 'UnicodeCharacter0xaaee',
u'\u4671': 'CjkUnifiedIdeograph4671',
u'\u0373': 'GreekSmallLetterArchaicSampi',
u'\uc075': 'HangulSyllableBbyig',
u'\u9d77': 'CjkUnifiedIdeograph9d77',
u'\u7bf8': 'CjkUnifiedIdeograph7bf8',
u'\u38fa': 'CjkUnifiedIdeograph38fa',
u'\uf5fc': 'UnicodeCharacter0xf5fc',
u'\ub2fe': 'HangulSyllableDap',
u'\u9000': 'CjkUnifiedIdeograph9000',
u'\ub483': 'HangulSyllableDweoc',
u'\u5302': 'CjkUnifiedIdeograph5302',
u'\u7785': 'CjkUnifiedIdeograph7785',
u'\ud906': 'UnicodeCharacter0xd906',
u'\ufd89': 'ArabicLigatureMeemWithHahWithJeemInitialForm',
u'\u020c': 'LatinCapitalLetterOWithDoubleGrave',
u'\u268f': 'DigramForGreaterYin',
u'\u8810': 'CjkUnifiedIdeograph8810',
u'\uac93': 'HangulSyllableGed',
u'\u4b12': 'CjkUnifiedIdeograph4b12',
u'\u6f95': 'CjkUnifiedIdeograph6f95',
u'\uf116': 'UnicodeCharacter0xf116',
u'\u1599': 'CanadianSyllabicsSayisiSho',
u'\u9b9d': 'CjkUnifiedIdeograph9b9d',
u'\u3a1c': 'CjkUnifiedIdeograph3a1c',
u'\u5e9f': 'CjkUnifiedIdeograph5e9f',
u'\ua020': 'YiSyllableBa',
u'\uc4a3': 'HangulSyllableSswed',
u'\u6322': 'CjkUnifiedIdeograph6322',
u'\ue926': 'UnicodeCharacter0xe926',
u'\u0da9': 'SinhalaLetterAlpapraanaDdayanna',
u'\ub3ad': 'HangulSyllableDyenj',
u'\u522c': 'CjkUnifiedIdeograph522c',
u'\u76af': 'CjkUnifiedIdeograph76af',
u'\ud830': 'UnicodeCharacter0xd830',
u'\ufcb3': 'ArabicLigatureSadWithMeemInitialForm',
u'\u0136': 'LatinCapitalLetterKWithCedilla',
u'\u25b9': 'WhiteRightpointingSmallTriangle',
u'\u873a': 'CjkUnifiedIdeograph873a',
u'\uabbd': 'UnicodeCharacter0xabbd',
u'\u4a3c': 'CjkUnifiedIdeograph4a3c',
u'\u6ebf': 'CjkUnifiedIdeograph6ebf',
u'\uf040': 'UnicodeCharacter0xf040',
u'\u14c3': 'CanadianSyllabicsNii',
u'\u9ac7': 'CjkUnifiedIdeograph9ac7',
u'\u3946': 'CjkUnifiedIdeograph3946',
u'\u5dc9': 'CjkUnifiedIdeograph5dc9',
u'\ubf4a': 'HangulSyllableBbolm',
u'\uc3cd': 'HangulSyllableSsyeb',
u'\u624c': 'CjkUnifiedIdeograph624c',
u'\ue850': 'UnicodeCharacter0xe850',
u'\u0cd3': 'UnicodeCharacter0xcd3',
u'\ub2d7': 'HangulSyllableNilh',
u'\u5156': 'CjkUnifiedIdeograph5156',
u'\u75d9': 'CjkUnifiedIdeograph75d9',
u'\ud75a': 'HangulSyllableHeulm',
u'\ufbdd': 'ArabicLetterUWithHamzaAboveIsolatedForm',
u'\u24e3': 'CircledLatinSmallLetterT',
u'\u8664': 'CjkUnifiedIdeograph8664',
u'\uaae7': 'UnicodeCharacter0xaae7',
u'\u4966': 'CjkUnifiedIdeograph4966',
u'\u6de9': 'CjkUnifiedIdeograph6de9',
u'\ucf6a': 'HangulSyllableKoj',
u'\u13ed': 'CherokeeLetterWu',
u'\u99f1': 'CjkUnifiedIdeograph99f1',
u'\u3870': 'CjkUnifiedIdeograph3870',
u'\u5cf3': 'CjkUnifiedIdeograph5cf3',
u'\ube74': 'HangulSyllableBbass',
u'\uc2f7': 'HangulSyllableSih',
u'\u6176': 'CjkUnifiedIdeograph6176',
u'\ue77a': 'UnicodeCharacter0xe77a',
u'\u0bfd': 'UnicodeCharacter0xbfd',
u'\u3101': 'UnicodeCharacter0x3101',
u'\u5080': 'CjkUnifiedIdeograph5080',
u'\ub705': 'HangulSyllableDdwing',
u'\ud684': 'HangulSyllableHwaess',
u'\u7a07': 'CjkUnifiedIdeograph7a07',
u'\ue00b': 'UnicodeCharacter0xe00b',
u'\u1f8a': 'GreekCapitalLetterAlphaWithPsiliAndVariaAndProsgegrammeni',
u'\u858e': 'CjkUnifiedIdeograph858e',
u'\u2911': 'RightwardsArrowWithDottedStem',
u'\u4890': 'CjkUnifiedIdeograph4890',
u'\uaf15': 'HangulSyllableGgyenj',
u'\uce94': 'HangulSyllableKaen',
u'\u181b': 'UnicodeCharacter0x181b',
u'\u379a': 'CjkUnifiedIdeograph379a',
u'\u9e1f': 'CjkUnifiedIdeograph9e1f',
u'\ubd9e': 'HangulSyllableBweogg',
u'\u4121': 'CjkUnifiedIdeograph4121',
u'\u60a0': 'CjkUnifiedIdeograph60a0',
u'\uc725': 'HangulSyllableYunj',
u'\ue6a4': 'UnicodeCharacter0xe6a4',
u'\u302b': 'IdeographicRisingToneMark',
u'\u2faa': 'KangxiRadicalSlave',
u'\ub62f': 'HangulSyllableDdwags',
u'\ud5ae': 'HangulSyllableHyaegg',
u'\u7931': 'CjkUnifiedIdeograph7931',
u'\uff35': 'FullwidthLatinCapitalLetterU',
u'\u1eb4': 'LatinCapitalLetterAWithBreveAndTilde',
u'\u84b8': 'CjkUnifiedIdeograph84b8',
u'\u283b': 'BraillePatternDots12456',
u'\u47ba': 'CjkUnifiedIdeograph47ba',
u'\uae3f': 'HangulSyllableGilh',
u'\ucdbe': 'HangulSyllableCweolp',
u'\u1745': 'BuhidLetterNga',
u'\u36c4': 'CjkUnifiedIdeograph36c4',
u'\u9d49': 'CjkUnifiedIdeograph9d49',
u'\ubcc8': 'HangulSyllableByeols',
u'\u404b': 'CjkUnifiedIdeograph404b',
u'\u7fca': 'CjkUnifiedIdeograph7fca',
u'\uc64f': 'HangulSyllableWalh',
u'\ue5ce': 'UnicodeCharacter0xe5ce',
u'\u0f55': 'TibetanLetterPha',
u'\u2ed4': 'CjkRadicalCsimplifiedGate',
u'\ub559': 'HangulSyllableDdaelt',
u'\ud4d8': 'HangulSyllablePwils',
u'\u785b': 'CjkUnifiedIdeograph785b',
u'\ufe5f': 'SmallNumberSign',
u'\u1dde': 'CombiningLatinLetterSmallCapitalL',
u'\u83e2': 'CjkUnifiedIdeograph83e2',
u'\u2765': 'RotatedHeavyBlackHeartBullet',
u'\u46e4': 'CjkUnifiedIdeograph46e4',
u'\uad69': 'HangulSyllableGyot',
u'\ucce8': 'HangulSyllableCyeok',
u'\u166f': 'CanadianSyllabicsQai',
u'\u35ee': 'CjkUnifiedIdeograph35ee',
u'\u9c73': 'CjkUnifiedIdeograph9c73',
u'\ubbf2': 'HangulSyllableMyij',
u'\u5f75': 'CjkUnifiedIdeograph5f75',
u'\u7ef4': 'CjkUnifiedIdeograph7ef4',
u'\uc579': 'HangulSyllableAet',
u'\ue4f8': 'UnicodeCharacter0xe4f8',
u'\u0e7f': 'UnicodeCharacter0xe7f',
u'\u2dfe': 'CombiningCyrillicLetterBigYus',
u'\u5806': 'CjkUnifiedIdeograph5806',
u'\u7c89': 'CjkUnifiedIdeograph7c89',
u'\u1d08': 'LatinSmallLetterTurnedOpenE',
u'\u218b': 'UnicodeCharacter0x218b',
u'\ude0a': 'UnicodeCharacter0xde0a',
u'\ue28d': 'UnicodeCharacter0xe28d',
u'\u830c': 'CjkUnifiedIdeograph830c',
u'\ua78f': 'UnicodeCharacter0xa78f',
u'\u7016': 'CjkUnifiedIdeograph7016',
u'\u3518': 'CjkUnifiedIdeograph3518',
u'\u599b': 'CjkUnifiedIdeograph599b',
u'\uf61a': 'UnicodeCharacter0xf61a',
u'\u1a9d': 'UnicodeCharacter0x1a9d',
u'\ubb1c': 'HangulSyllableMyon',
u'\udf9f': 'UnicodeCharacter0xdf9f',
u'\u80a1': 'CjkUnifiedIdeograph80a1',
u'\u6826': 'CjkUnifiedIdeograph6826',
u'\u2d28': 'UnicodeCharacter0x2d28',
u'\u71ab': 'CjkUnifiedIdeograph71ab',
u'\uee2a': 'UnicodeCharacter0xee2a',
u'\u32ad': 'CircledIdeographEnterprise',
u'\ud32c': 'HangulSyllablePaen',
u'\uf7af': 'UnicodeCharacter0xf7af',
u'\u942e': 'CjkUnifiedIdeograph942e',
u'\ub8b1': 'HangulSyllableRoeg',
u'\u4538': 'CjkUnifiedIdeograph4538',
u'\u69bb': 'CjkUnifiedIdeograph69bb',
u'\u063a': 'ArabicLetterGhain',
u'\u2abd': 'SubsetWithDot',
u'\ucb3c': 'HangulSyllableJjyom',
u'\uefbf': 'UnicodeCharacter0xefbf',
u'\u8c3e': 'CjkUnifiedIdeograph8c3e',
u'\ud0c1': 'HangulSyllableTag',
u'\u95c3': 'CjkUnifiedIdeograph95c3',
u'\u7d48': 'CjkUnifiedIdeograph7d48',
u'\u3e4a': 'CjkUnifiedIdeograph3e4a',
u'\u42cd': 'CjkUnifiedIdeograph42cd',
u'\ue34c': 'UnicodeCharacter0xe34c',
u'\u07cf': 'NkoLetterOo',
u'\ua44e': 'YiSyllableNyiet',
u'\uc8d1': 'HangulSyllableJoelt',
u'\u8dd3': 'CjkUnifiedIdeograph8dd3',
u'\u565a': 'CjkUnifiedIdeograph565a',
u'\u7add': 'CjkUnifiedIdeograph7add',
u'\u1b5c': 'BalineseWindu',
u'\u3fdf': 'CjkUnifiedIdeograph3fdf',
u'\udc5e': 'UnicodeCharacter0xdc5e',
u'\ue0e1': 'UnicodeCharacter0xe0e1',
u'\u8160': 'CjkUnifiedIdeograph8160',
u'\ua5e3': 'VaiSyllableNgen',
u'\u4e6a': 'CjkUnifiedIdeograph4e6a',
u'\u336c': 'IdeographicTelegraphSymbolForHourTwenty',
u'\u57ef': 'CjkUnifiedIdeograph57ef',
u'\uf46e': 'UnicodeCharacter0xf46e',
u'\u18f1': 'UnicodeCharacter0x18f1',
u'\ub970': 'HangulSyllableRyuk',
u'\uddf3': 'UnicodeCharacter0xddf3',
u'\u9ef5': 'CjkUnifiedIdeograph9ef5',
u'\u667a': 'CjkUnifiedIdeograph667a',
u'\u2b7c': 'UnicodeCharacter0x2b7c',
u'\u4fff': 'CjkUnifiedIdeograph4fff',
u'\uec7e': 'UnicodeCharacter0xec7e',
u'\ud180': 'HangulSyllableTyeok',
u'\u7503': 'CjkUnifiedIdeograph7503',
u'\u9282': 'CjkUnifiedIdeograph9282',
u'\u3605': 'CjkUnifiedIdeograph3605',
u'\ufb07': 'UnicodeCharacter0xfb07',
u'\ubc09': 'HangulSyllableMib',
u'\u438c': 'CjkUnifiedIdeograph438c',
u'\u048e': 'CyrillicCapitalLetterErWithTick',
u'\uc990': 'HangulSyllableJeul',
u'\u6d13': 'CjkUnifiedIdeograph6d13',
u'\u8a92': 'CjkUnifiedIdeograph8a92',
u'\u2e15': 'UpwardsAncora',
u'\u1317': 'UnicodeCharacter0x1317',
u'\ud419': 'HangulSyllablePwab',
u'\u991b': 'CjkUnifiedIdeograph991b',
u'\u7b9c': 'CjkUnifiedIdeograph7b9c',
u'\u3c9e': 'CjkUnifiedIdeograph3c9e',
u'\ue1a0': 'UnicodeCharacter0xe1a0',
u'\ua2a2': 'YiSyllableZut',
u'\u4625': 'CjkUnifiedIdeograph4625',
u'\u0b27': 'OriyaLetterDha',
u'\ucc29': 'HangulSyllableCag',
u'\ub12b': 'HangulSyllableNed',
u'\u54ae': 'CjkUnifiedIdeograph54ae',
u'\u19b0': 'NewTaiLueVowelSignVowelShortener',
u'\udab2': 'UnicodeCharacter0xdab2',
u'\u7e35': 'CjkUnifiedIdeograph7e35',
u'\u9fb4': 'CjkUnifiedIdeograph9fb4',
u'\u2337': 'AplFunctionalSymbolSquishQuad',
u'\ue439': 'UnicodeCharacter0xe439',
u'\ua93b': 'RejangLetterNya',
u'\u4cbe': 'CjkUnifiedIdeograph4cbe',
u'\u31c0': 'CjkStrokeT',
u'\uf2c2': 'UnicodeCharacter0xf2c2',
u'\ub7c4': 'HangulSyllableRyam',
u'\u5b47': 'CjkUnifiedIdeograph5b47',
u'\u1c49': 'LepchaDigitNine',
u'\uc14b': 'HangulSyllableSes',
u'\u824d': 'CjkUnifiedIdeograph824d',
u'\u64ce': 'CjkUnifiedIdeograph64ce',
u'\u29d0': 'VerticalBarBesideRightTriangle',
u'\uead2': 'UnicodeCharacter0xead2',
u'\uafd4': 'HangulSyllableGgweo',
u'\u7357': 'CjkUnifiedIdeograph7357',
u'\u90d6': 'CjkUnifiedIdeograph90d6',
u'\u3459': 'CjkUnifiedIdeograph3459',
u'\uf95b': 'CjkCompatibilityIdeographf95b',
u'\uba5d': 'HangulSyllableMelg',
u'\u41e0': 'CjkUnifiedIdeograph41e0',
u'\u02e2': 'ModifierLetterSmallS',
u'\uc7e4': 'HangulSyllableJyae',
u'\u6b67': 'CjkUnifiedIdeograph6b67',
u'\u88e6': 'CjkUnifiedIdeograph88e6',
u'\u2c69': 'LatinCapitalLetterKWithDescender',
u'\u116b': 'HangulJungseongWae',
u'\ud26d': 'HangulSyllableTwelg',
u'\u976f': 'CjkUnifiedIdeograph976f',
u'\u79f0': 'CjkUnifiedIdeograph79f0',
u'\u3af2': 'CjkUnifiedIdeograph3af2',
u'\ufff4': 'UnicodeCharacter0xfff4',
u'\ua0f6': 'YiSyllableVu',
u'\u4479': 'CjkUnifiedIdeograph4479',
u'\u097b': 'DevanagariLetterGga',
u'\uca7d': 'HangulSyllableJjeng',
u'\u8f7f': 'CjkUnifiedIdeograph8f7f',
u'\uf381': 'UnicodeCharacter0xf381',
u'\u1404': 'CanadianSyllabicsIi',
u'\u3887': 'CjkUnifiedIdeograph3887',
u'\u9e08': 'CjkUnifiedIdeograph9e08',
u'\ua28b': 'YiSyllableWe',
u'\u5d0a': 'CjkUnifiedIdeograph5d0a',
u'\u618d': 'CjkUnifiedIdeograph618d',
u'\uc70e': 'HangulSyllableWilm',
u'\ueb91': 'UnicodeCharacter0xeb91',
u'\u0c14': 'TeluguLetterAu',
u'\u5097': 'CjkUnifiedIdeograph5097',
u'\ub618': 'HangulSyllableDdol',
u'\uda9b': 'UnicodeCharacter0xda9b',
u'\u751a': 'CjkUnifiedIdeograph751a',
u'\uff1e': 'FullwidthGreaterthanSign',
u'\u03a1': 'GreekCapitalLetterRho',
u'\u85a5': 'CjkUnifiedIdeograph85a5',
u'\u2424': 'SymbolForNewline',
u'\u48a7': 'CjkUnifiedIdeograph48a7',
u'\uae28': 'HangulSyllableGyiss',
u'\uf2ab': 'UnicodeCharacter0xf2ab',
u'\u6d2a': 'CjkUnifiedIdeograph6d2a',
u'\u172e': 'HanunooLetterLa',
u'\u3bb1': 'CjkUnifiedIdeograph3bb1',
u'\u9932': 'CjkUnifiedIdeograph9932',
u'\ubdb5': 'HangulSyllableBweot',
u'\u5c34': 'CjkUnifiedIdeograph5c34',
u'\u60b7': 'CjkUnifiedIdeograph60b7',
u'\uc638': 'HangulSyllableOss',
u'\ueabb': 'UnicodeCharacter0xeabb',
u'\u0f3e': 'TibetanSignYarTshes',
u'\u53c1': 'CjkUnifiedIdeograph53c1',
u'\ub142': 'HangulSyllableNyeogg',
u'\ud5c5': 'HangulSyllableHyaet',
u'\u7444': 'CjkUnifiedIdeograph7444',
u'\ufe48': 'PresentationFormForVerticalRightSquareBracket',
u'\u02cb': 'ModifierLetterGraveAccent',
u'\u84cf': 'CjkUnifiedIdeograph84cf',
u'\u274e': 'UnicodeCharacter0x274e',
u'\u4bd1': 'CjkUnifiedIdeograph4bd1',
u'\ua952': 'RejangConsonantSignH',
u'\ucdd5': 'HangulSyllableCwelg',
u'\u6c54': 'CjkUnifiedIdeograph6c54',
u'\u1658': 'CanadianSyllabicsCarrierShi',
u'\u3adb': 'CjkUnifiedIdeograph3adb',
u'\u985c': 'CjkUnifiedIdeograph985c',
u'\ubcdf': 'HangulSyllableByed',
u'\u5f5e': 'CjkUnifiedIdeograph5f5e',
u'\u63e1': 'CjkUnifiedIdeograph63e1',
u'\uc162': 'HangulSyllableSyeolp',
u'\ue5e5': 'UnicodeCharacter0xe5e5',
u'\u0e68': 'UnicodeCharacter0xe68',
u'\u52eb': 'CjkUnifiedIdeograph52eb',
u'\ub06c': 'HangulSyllableGgyils',
u'\ud4ef': 'HangulSyllablePyud',
u'\u776e': 'CjkUnifiedIdeograph776e',
u'\uf972': 'CjkCompatibilityIdeographf972',
u'\u1df5': 'UnicodeCharacter0x1df5',
u'\u87f9': 'CjkUnifiedIdeograph87f9',
u'\u2678': 'RecyclingSymbolForType6Plastics',
u'\u4afb': 'CjkUnifiedIdeograph4afb',
u'\ua87c': 'UnicodeCharacter0xa87c',
u'\uccff': 'HangulSyllableCyes',
u'\u6f7e': 'CjkUnifiedIdeograph6f7e',
u'\uf603': 'UnicodeCharacter0xf603',
u'\u1182': 'HangulJungseongOo',
u'\u9b86': 'CjkUnifiedIdeograph9b86',
u'\u3f09': 'CjkUnifiedIdeograph3f09',
u'\u5e88': 'CjkUnifiedIdeograph5e88',
u'\ua10d': 'YiSyllableDot',
u'\uc08c': 'HangulSyllableBbyik',
u'\u640f': 'CjkUnifiedIdeograph640f',
u'\uee13': 'UnicodeCharacter0xee13',
u'\u0992': 'UnicodeCharacter0x992',
u'\u9017': 'CjkUnifiedIdeograph9017',
u'\ub396': 'HangulSyllableDyeolm',
u'\u5719': 'CjkUnifiedIdeograph5719',
u'\u7698': 'CjkUnifiedIdeograph7698',
u'\ud91d': 'UnicodeCharacter0xd91d',
u'\uf89c': 'UnicodeCharacter0xf89c',
u'\u0623': 'ArabicLetterAlefWithHamzaAbove',
u'\u21a2': 'LeftwardsArrowWithTail',
u'\u8827': 'CjkUnifiedIdeograph8827',
u'\uaba6': 'UnicodeCharacter0xaba6',
u'\u4f29': 'CjkUnifiedIdeograph4f29',
u'\u6ea8': 'CjkUnifiedIdeograph6ea8',
u'\uf12d': 'UnicodeCharacter0xf12d',
u'\u10ac': 'GeorgianCapitalLetterNar',
u'\u9ab0': 'CjkUnifiedIdeograph9ab0',
u'\u3e33': 'CjkUnifiedIdeograph3e33',
u'\u59b2': 'CjkUnifiedIdeograph59b2',
u'\ua037': 'YiSyllableByr',
u'\uc3b6': 'HangulSyllableSsyeoj',
u'\u6739': 'CjkUnifiedIdeograph6739',
u'\ue93d': 'UnicodeCharacter0xe93d',
u'\u08bc': 'UnicodeCharacter0x8bc',
u'\u9341': 'CjkUnifiedIdeograph9341',
u'\ub2c0': 'HangulSyllableNyiss',
u'\u5643': 'CjkUnifiedIdeograph5643',
u'\u71c2': 'CjkUnifiedIdeograph71c2',
u'\ud847': 'UnicodeCharacter0xd847',
u'\ufbc6': 'UnicodeCharacter0xfbc6',
u'\u014d': 'LatinSmallLetterOWithMacron',
u'\u20cc': 'UnicodeCharacter0x20cc',
u'\u8b51': 'CjkUnifiedIdeograph8b51',
u'\uaad0': 'UnicodeCharacter0xaad0',
u'\u4e53': 'CjkUnifiedIdeograph4e53',
u'\u69d2': 'CjkUnifiedIdeograph69d2',
u'\uf057': 'UnicodeCharacter0xf057',
u'\u13d6': 'CherokeeLetterTe',
u'\u95da': 'CjkUnifiedIdeograph95da',
u'\u395d': 'CjkUnifiedIdeograph395d',
u'\u58dc': 'CjkUnifiedIdeograph58dc',
u'\ua361': 'YiSyllableChat',
u'\uc2e0': 'HangulSyllableSin',
u'\u6663': 'CjkUnifiedIdeograph6663',
u'\ue867': 'UnicodeCharacter0xe867',
u'\u0be6': 'TamilDigitZero',
u'\u926b': 'CjkUnifiedIdeograph926b',
u'\u8dea': 'CjkUnifiedIdeograph8dea',
u'\u516d': 'CjkUnifiedIdeograph516d',
u'\u70ec': 'CjkUnifiedIdeograph70ec',
u'\udb71': 'UnicodeCharacter0xdb71',
u'\ufaf0': 'UnicodeCharacter0xfaf0',
u'\u23f6': 'UnicodeCharacter0x23f6',
u'\u8a7b': 'CjkUnifiedIdeograph8a7b',
u'\ua5fa': 'VaiSyllableNde',
u'\u497d': 'CjkUnifiedIdeograph497d',
u'\u68fc': 'CjkUnifiedIdeograph68fc',
u'\u7281': 'CjkUnifiedIdeograph7281',
u'\u1300': 'EthiopicSyllableJa',
u'\u3783': 'CjkUnifiedIdeograph3783',
u'\ud002': 'HangulSyllableKweonh',
u'\uf485': 'UnicodeCharacter0xf485',
u'\u9504': 'CjkUnifiedIdeograph9504',
u'\ub987': 'HangulSyllableReus',
u'\u460e': 'CjkUnifiedIdeograph460e',
u'\u6a91': 'CjkUnifiedIdeograph6a91',
u'\u0b10': 'OriyaLetterAi',
u'\u2f93': 'KangxiRadicalHorn',
u'\uc812': 'HangulSyllableJeobs',
u'\uec95': 'UnicodeCharacter0xec95',
u'\u8d14': 'CjkUnifiedIdeograph8d14',
u'\ud197': 'HangulSyllableTyes',
u'\u9699': 'CjkUnifiedIdeograph9699',
u'\u7e1e': 'CjkUnifiedIdeograph7e1e',
u'\u2320': 'TopHalfIntegral',
u'\u47a3': 'CjkUnifiedIdeograph47a3',
u'\ue022': 'UnicodeCharacter0xe022',
u'\u04a5': 'CyrillicSmallLigatureEnGhe',
u'\ua524': 'VaiSyllableI',
u'\uc9a7': 'HangulSyllableJyigs',
u'\u8ea9': 'CjkUnifiedIdeograph8ea9',
u'\u5b30': 'CjkUnifiedIdeograph5b30',
u'\u7fb3': 'CjkUnifiedIdeograph7fb3',
u'\u1832': 'MongolianLetterTa',
u'\u3cb5': 'CjkUnifiedIdeograph3cb5',
u'\udd34': 'UnicodeCharacter0xdd34',
u'\ue1b7': 'UnicodeCharacter0xe1b7',
u'\u8236': 'CjkUnifiedIdeograph8236',
u'\ua6b9': 'UnicodeCharacter0xa6b9',
u'\u7340': 'CjkUnifiedIdeograph7340',
u'\u3042': 'HiraganaLetterA',
u'\u54c5': 'CjkUnifiedIdeograph54c5',
u'\uf544': 'UnicodeCharacter0xf544',
u'\u19c7': 'NewTaiLueLetterFinalB',
u'\uba46': 'HangulSyllableMeolp',
u'\udec9': 'UnicodeCharacter0xdec9',
u'\u83cb': 'CjkUnifiedIdeograph83cb',
u'\u6b50': 'CjkUnifiedIdeograph6b50',
u'\u2852': 'BraillePatternDots257',
u'\u4cd5': 'CjkUnifiedIdeograph4cd5',
u'\ued54': 'UnicodeCharacter0xed54',
u'\u31d7': 'CjkStrokeSz',
u'\ud256': 'HangulSyllableTweolp',
u'\uf6d9': 'UnicodeCharacter0xf6d9',
u'\u9758': 'CjkUnifiedIdeograph9758',
u'\ubbdb': 'HangulSyllableMeuh',
u'\u4062': 'CjkUnifiedIdeograph4062',
u'\u64e5': 'CjkUnifiedIdeograph64e5',
u'\u0564': 'ArmenianSmallLetterDa',
u'\u29e7': 'Thermodynamic',
u'\uca66': 'HangulSyllableJjeop',
u'\ueee9': 'UnicodeCharacter0xeee9',
u'\u8f68': 'CjkUnifiedIdeograph8f68',
u'\ud3eb': 'HangulSyllablePyeh',
u'\u90ed': 'CjkUnifiedIdeograph90ed',
u'\u7872': 'CjkUnifiedIdeograph7872',
u'\u3d74': 'CjkUnifiedIdeograph3d74',
u'\u41f7': 'CjkUnifiedIdeograph41f7',
u'\ue276': 'UnicodeCharacter0xe276',
u'\u06f9': 'ExtendedArabicindicDigitNine',
u'\ua778': 'LatinSmallLetterUm',
u'\ucbfb': 'HangulSyllableJjyilb',
u'\u88fd': 'CjkUnifiedIdeograph88fd',
u'\ub201': 'HangulSyllableNyot',
u'\u5584': 'CjkUnifiedIdeograph5584',
u'\u1a86': 'UnicodeCharacter0x1a86',
u'\udf88': 'UnicodeCharacter0xdf88',
u'\u630b': 'CjkUnifiedIdeograph630b',
u'\u9c8a': 'CjkUnifiedIdeograph9c8a',
u'\u200d': 'ZeroWidthJoiner',
u'\ue50f': 'UnicodeCharacter0xe50f',
u'\uaa11': 'ChamLetterNha',
u'\u4d94': 'CjkUnifiedIdeograph4d94',
u'\u3296': 'CircledIdeographFinancial',
u'\uf798': 'UnicodeCharacter0xf798',
u'\ub49a': 'HangulSyllableDwebs',
u'\u581d': 'CjkUnifiedIdeograph581d',
u'\u1d1f': 'LatinSmallLetterSidewaysTurnedM',
u'\uc221': 'HangulSyllableSulg',
u'\u8723': 'CjkUnifiedIdeograph8723',
u'\u65a4': 'CjkUnifiedIdeograph65a4',
u'\u2aa6': 'LessthanClosedByCurve',
u'\uefa8': 'UnicodeCharacter0xefa8',
u'\uacaa': 'HangulSyllableGyeogg',
u'\u702d': 'CjkUnifiedIdeograph702d',
u'\u91ac': 'CjkUnifiedIdeograph91ac',
u'\u352f': 'CjkUnifiedIdeograph352f',
u'\ufa31': 'CjkCompatibilityIdeographfa31',
u'\ubf33': 'HangulSyllableBbyelh',
u'\u42b6': 'CjkUnifiedIdeograph42b6',
u'\u07b8': 'UnicodeCharacter0x7b8',
u'\uc4ba': 'HangulSyllableSswigg',
u'\u683d': 'CjkUnifiedIdeograph683d',
u'\u89bc': 'CjkUnifiedIdeograph89bc',
u'\u2d3f': 'TifinaghLetterYakhh',
u'\u1241': 'EthiopicSyllableQu',
u'\ud743': 'HangulSyllableHyulh',
u'\u9445': 'CjkUnifiedIdeograph9445',
u'\u7ac6': 'CjkUnifiedIdeograph7ac6',
u'\u3fc8': 'CjkUnifiedIdeograph3fc8',
u'\ufcca': 'ArabicLigatureLamWithHahInitialForm',
u'\ua1cc': 'YiSyllableLe',
u'\u454f': 'CjkUnifiedIdeograph454f',
u'\u0a51': 'GurmukhiSignUdaat',
u'\ucf53': 'HangulSyllableKyeh',
u'\u8c55': 'CjkUnifiedIdeograph8c55',
u'\u57d8': 'CjkUnifiedIdeograph57d8',
u'\u14da': 'CanadianSyllabicsLa',
u'\ud9dc': 'UnicodeCharacter0xd9dc',
u'\u7d5f': 'CjkUnifiedIdeograph7d5f',
u'\u9ede': 'CjkUnifiedIdeograph9ede',
u'\u2261': 'IdenticalTo',
u'\ue763': 'UnicodeCharacter0xe763',
u'\ua465': 'YiSyllableXuox',
u'\u4fe8': 'CjkUnifiedIdeograph4fe8',
u'\u0cea': 'KannadaDigitFour',
u'\uf1ec': 'UnicodeCharacter0xf1ec',
u'\ub6ee': 'HangulSyllableDdwep',
u'\u5a71': 'CjkUnifiedIdeograph5a71',
u'\u1f73': 'GreekSmallLetterEpsilonWithOxia',
u'\udc75': 'UnicodeCharacter0xdc75',
u'\u8177': 'CjkUnifiedIdeograph8177',
u'\u67f8': 'CjkUnifiedIdeograph67f8',
u'\u24fa': 'DoubleCircledDigitSix',
u'\ue9fc': 'UnicodeCharacter0xe9fc',
u'\uaefe': 'HangulSyllableGgyeolm',
u'\u9400': 'CjkUnifiedIdeograph9400',
u'\ub083': 'HangulSyllableGgid',
u'\u5702': 'CjkUnifiedIdeograph5702',
u'\u7385': 'CjkUnifiedIdeograph7385',
u'\udd06': 'UnicodeCharacter0xdd06',
u'\uf989': 'CjkCompatibilityIdeographf989',
u'\u060c': 'ArabicComma',
u'\u228f': 'SquareImageOf',
u'\u8c10': 'CjkUnifiedIdeograph8c10',
u'\ua893': 'SaurashtraLetterKha',
u'\u4f12': 'CjkUnifiedIdeograph4f12',
u'\u6b95': 'CjkUnifiedIdeograph6b95',
u'\uf516': 'UnicodeCharacter0xf516',
u'\u1199': 'HangulJungseongIya',
u'\u9f9d': 'CjkUnifiedIdeograph9f9d',
u'\u3e1c': 'CjkUnifiedIdeograph3e1c',
u'\u5a9f': 'CjkUnifiedIdeograph5a9f',
u'\ua420': 'YiSyllableJjuox',
u'\uc0a3': 'HangulSyllableBbis',
u'\u6722': 'CjkUnifiedIdeograph6722',
u'\ued26': 'UnicodeCharacter0xed26',
u'\u09a9': 'UnicodeCharacter0x9a9',
u'\ub7ad': 'HangulSyllableRaeng',
u'\u562c': 'CjkUnifiedIdeograph562c',
u'\u72af': 'CjkUnifiedIdeograph72af',
u'\udc30': 'UnicodeCharacter0xdc30',
u'\uf8b3': 'UnicodeCharacter0xf8b3',
u'\u0536': 'ArmenianCapitalLetterZa',
u'\u21b9': 'LeftwardsArrowToBarOverRightwardsArrowToBar',
u'\u833a': 'CjkUnifiedIdeograph833a',
u'\uafbd': 'HangulSyllableGgunj',
u'\u4e3c': 'CjkUnifiedIdeograph4e3c',
u'\u6abf': 'CjkUnifiedIdeograph6abf',
u'\uf440': 'UnicodeCharacter0xf440',
u'\u10c3': 'GeorgianCapitalLetterWe',
u'\u9ec7': 'CjkUnifiedIdeograph9ec7',
u'\u3d46': 'CjkUnifiedIdeograph3d46',
u'\u59c9': 'CjkUnifiedIdeograph59c9',
u'\ubb4a': 'HangulSyllableMuj',
u'\uc7cd': 'HangulSyllableJyanj',
u'\u664c': 'CjkUnifiedIdeograph664c',
u'\uec50': 'UnicodeCharacter0xec50',
u'\u08d3': 'UnicodeCharacter0x8d3',
u'\ub6d7': 'HangulSyllableDdwegs',
u'\u5556': 'CjkUnifiedIdeograph5556',
u'\u71d9': 'CjkUnifiedIdeograph71d9',
u'\ud35a': 'HangulSyllablePyaj',
u'\uffdd': 'UnicodeCharacter0xffdd',
u'\u0460': 'CyrillicCapitalLetterOmega',
u'\u20e3': 'CombiningEnclosingKeycap',
u'\u8264': 'CjkUnifiedIdeograph8264',
u'\uaee7': 'HangulSyllableGgelh',
u'\u4d66': 'CjkUnifiedIdeograph4d66',
u'\u69e9': 'CjkUnifiedIdeograph69e9',
u'\ucb6a': 'HangulSyllableJjweonh',
u'\u17ed': 'UnicodeCharacter0x17ed',
u'\u9df1': 'CjkUnifiedIdeograph9df1',
u'\u3c70': 'CjkUnifiedIdeograph3c70',
u'\u58f3': 'CjkUnifiedIdeograph58f3',
u'\uba74': 'HangulSyllableMyeon',
u'\uc6f7': 'HangulSyllableWelh',
u'\u6576': 'CjkUnifiedIdeograph6576',
u'\ue37a': 'UnicodeCharacter0xe37a',
u'\u0ffd': 'UnicodeCharacter0xffd',
u'\u3501': 'CjkUnifiedIdeograph3501',
u'\u5480': 'CjkUnifiedIdeograph5480',
u'\ub305': 'HangulSyllableDaenj',
u'\ud284': 'HangulSyllableTwin',
u'\u7e07': 'CjkUnifiedIdeograph7e07',
u'\ue40b': 'UnicodeCharacter0xe40b',
u'\u1b8a': 'SundaneseLetterKa',
u'\u818e': 'CjkUnifiedIdeograph818e',
u'\u2d11': 'GeorgianSmallLetterSan',
u'\u4c90': 'CjkUnifiedIdeograph4c90',
u'\uab15': 'UnicodeCharacter0xab15',
u'\uca94': 'HangulSyllableJjyeom',
u'\u1c1b': 'LepchaLetterRa',
u'\u339a': 'SquareNm',
u'\u9a1f': 'CjkUnifiedIdeograph9a1f',
u'\ub99e': 'HangulSyllableRyilp',
u'\u4521': 'CjkUnifiedIdeograph4521',
u'\u64a0': 'CjkUnifiedIdeograph64a0',
u'\uc325': 'HangulSyllableSsaeb',
u'\ue2a4': 'UnicodeCharacter0xe2a4',
u'\u342b': 'CjkUnifiedIdeograph342b',
u'\u2baa': 'UnicodeCharacter0x2baa',
u'\ub22f': 'HangulSyllableNweolh',
u'\ud1ae': 'HangulSyllableTolp',
u'\u7d31': 'CjkUnifiedIdeograph7d31',
u'\ufb35': 'HebrewLetterVavWithDagesh',
u'\u1ab4': 'UnicodeCharacter0x1ab4',
u'\u80b8': 'CjkUnifiedIdeograph80b8',
u'\u2c3b': 'GlagoliticSmallLetterI',
u'\u43ba': 'CjkUnifiedIdeograph43ba',
u'\uaa3f': 'UnicodeCharacter0xaa3f',
u'\uc9be': 'HangulSyllableJyip',
u'\u1345': 'EthiopicSyllableTze',
u'\u32c4': 'IdeographicTelegraphSymbolForMay',
u'\u9949': 'CjkUnifiedIdeograph9949',
u'\ub8c8': 'HangulSyllableRoek',
u'\u444b': 'CjkUnifiedIdeograph444b',
u'\u7bca': 'CjkUnifiedIdeograph7bca',
u'\uc24f': 'HangulSyllableSweoh',
u'\ue1ce': 'UnicodeCharacter0xe1ce',
u'\u0b55': 'UnicodeCharacter0xb55',
u'\u2ad4': 'SupersetAboveSubset',
u'\ub159': 'HangulSyllableNyeot',
u'\ud0d8': 'HangulSyllableTak',
u'\u7c5b': 'CjkUnifiedIdeograph7c5b',
u'\ufa5f': 'CjkCompatibilityIdeographfa5f',
u'\u19de': 'NewTaiLueSignLae',
u'\u87e2': 'CjkUnifiedIdeograph87e2',
u'\u2365': 'AplFunctionalSymbolCircleDiaeresis',
u'\u42e4': 'CjkUnifiedIdeograph42e4',
u'\ua969': 'UnicodeCharacter0xa969',
u'\uc8e8': 'HangulSyllableJyol',
u'\u126f': 'EthiopicSyllableVwa',
u'\u31ee': 'UnicodeCharacter0x31ee',
u'\u9873': 'CjkUnifiedIdeograph9873',
u'\ubff2': 'HangulSyllableBbweolm',
u'\u5b75': 'CjkUnifiedIdeograph5b75',
u'\u7af4': 'CjkUnifiedIdeograph7af4',
u'\uc179': 'HangulSyllableSyelg',
u'\ue0f8': 'UnicodeCharacter0xe0f8',
u'\u0a7f': 'UnicodeCharacter0xa7f',
u'\u29fe': 'Tiny',
u'\u5c06': 'CjkUnifiedIdeograph5c06',
u'\u7889': 'CjkUnifiedIdeograph7889',
u'\u1908': 'LimbuLetterJa',
u'\u258b': 'LeftFiveEighthsBlock',
u'\uda0a': 'UnicodeCharacter0xda0a',
u'\ue68d': 'UnicodeCharacter0xe68d',
u'\u870c': 'CjkUnifiedIdeograph870c',
u'\ua38f': 'YiSyllableRryx',
u'\u7416': 'CjkUnifiedIdeograph7416',
u'\u3118': 'BopomofoLetterC',
u'\u5d9b': 'CjkUnifiedIdeograph5d9b',
u'\uf21a': 'UnicodeCharacter0xf21a',
u'\u1e9d': 'LatinSmallLetterLongSWithHighStroke',
u'\ubf1c': 'HangulSyllableBbyeoss',
u'\udb9f': 'UnicodeCharacter0xdb9f',
u'\u84a1': 'CjkUnifiedIdeograph84a1',
u'\u6c26': 'CjkUnifiedIdeograph6c26',
u'\u2928': 'NorthEastArrowAndSouthEastArrow',
u'\u75ab': 'CjkUnifiedIdeograph75ab',
u'\uea2a': 'UnicodeCharacter0xea2a',
u'\u36ad': 'CjkUnifiedIdeograph36ad',
u'\ud72c': 'HangulSyllableHwiss',
u'\uf3af': 'UnicodeCharacter0xf3af',
u'\u902e': 'CjkUnifiedIdeograph902e',
u'\ubcb1': 'HangulSyllableBeb',
u'\u4138': 'CjkUnifiedIdeograph4138',
u'\u6dbb': 'CjkUnifiedIdeograph6dbb',
u'\u023a': 'LatinCapitalLetterAWithStroke',
u'\u2ebd': 'CjkRadicalMortar',
u'\ucf3c': 'HangulSyllableKyen',
u'\uebbf': 'UnicodeCharacter0xebbf',
u'\u883e': 'CjkUnifiedIdeograph883e',
u'\ud4c1': 'HangulSyllablePweb',
u'\u91c3': 'CjkUnifiedIdeograph91c3',
u'\u7948': 'CjkUnifiedIdeograph7948',
u'\u3a4a': 'CjkUnifiedIdeograph3a4a',
u'\u46cd': 'CjkUnifiedIdeograph46cd',
u'\ue74c': 'UnicodeCharacter0xe74c',
u'\u03cf': 'GreekCapitalKaiSymbol',
u'\ua04e': 'YiSyllablePurx',
u'\uccd1': 'HangulSyllableCyeog',
u'\u89d3': 'CjkUnifiedIdeograph89d3',
u'\u525a': 'CjkUnifiedIdeograph525a',
u'\u7edd': 'CjkUnifiedIdeograph7edd',
u'\u1f5c': 'UnicodeCharacter0x1f5c',
u'\u3bdf': 'CjkUnifiedIdeograph3bdf',
u'\ud85e': 'UnicodeCharacter0xd85e',
u'\ue4e1': 'UnicodeCharacter0xe4e1',
u'\u8560': 'CjkUnifiedIdeograph8560',
u'\ua1e3': 'YiSyllableGax',
u'\u4a6a': 'CjkUnifiedIdeograph4a6a',
u'\u376c': 'CjkUnifiedIdeograph376c',
u'\u53ef': 'CjkUnifiedIdeograph53ef',
u'\uf06e': 'UnicodeCharacter0xf06e',
u'\u1cf1': 'UnicodeCharacter0x1cf1',
u'\ubd70': 'HangulSyllableByols',
u'\ud9f3': 'UnicodeCharacter0xd9f3',
u'\u9af5': 'CjkUnifiedIdeograph9af5',
u'\u627a': 'CjkUnifiedIdeograph627a',
u'\u2f7c': 'KangxiRadicalOld',
u'\u4bff': 'CjkUnifiedIdeograph4bff',
u'\ue87e': 'UnicodeCharacter0xe87e',
u'\ud580': 'HangulSyllableHaels',
u'\u7103': 'CjkUnifiedIdeograph7103',
u'\u9682': 'CjkUnifiedIdeograph9682',
u'\u3205': 'ParenthesizedHangulPieup',
u'\uff07': 'FullwidthApostrophe',
u'\ub809': 'HangulSyllableReg',
u'\u478c': 'CjkUnifiedIdeograph478c',
u'\x8e': 'UnicodeCharacter0x8e',
u'\ucd90': 'HangulSyllableCyok',
u'\u6913': 'CjkUnifiedIdeograph6913',
u'\u8e92': 'CjkUnifiedIdeograph8e92',
u'\u2a15': 'IntegralAroundAPointOperator',
u'\u1717': 'UnicodeCharacter0x1717',
u'\ud019': 'HangulSyllableKweg',
u'\u9d1b': 'CjkUnifiedIdeograph9d1b',
u'\u7f9c': 'CjkUnifiedIdeograph7f9c',
u'\u389e': 'CjkUnifiedIdeograph389e',
u'\ue5a0': 'UnicodeCharacter0xe5a0',
u'\ua6a2': 'UnicodeCharacter0xa6a2',
u'\u4225': 'CjkUnifiedIdeograph4225',
u'\u0f27': 'TibetanDigitSeven',
u'\uc829': 'HangulSyllableJelt',
u'\ub52b': 'HangulSyllableDic',
u'\u50ae': 'CjkUnifiedIdeograph50ae',
u'\u1db0': 'ModifierLetterSmallCapitalN',
u'\udeb2': 'UnicodeCharacter0xdeb2',
u'\u7a35': 'CjkUnifiedIdeograph7a35',
u'\u9bb4': 'CjkUnifiedIdeograph9bb4',
u'\u2737': 'EightPointedRectilinearBlackStar',
u'\ue039': 'UnicodeCharacter0xe039',
u'\uad3b': 'HangulSyllableGoed',
u'\u48be': 'CjkUnifiedIdeograph48be',
u'\u35c0': 'CjkUnifiedIdeograph35c0',
u'\uf6c2': 'UnicodeCharacter0xf6c2',
u'\ub3c4': 'HangulSyllableDo',
u'\u5f47': 'CjkUnifiedIdeograph5f47',
u'\u1849': 'MongolianLetterTodoUe',
u'\uc54b': 'HangulSyllableAd',
u'\u864d': 'CjkUnifiedIdeograph864d',
u'\u60ce': 'CjkUnifiedIdeograph60ce',
u'\u2dd0': 'EthiopicSyllableXya',
u'\ueed2': 'UnicodeCharacter0xeed2',
u'\uabd4': 'UnicodeCharacter0xabd4',
u'\u7757': 'CjkUnifiedIdeograph7757',
u'\u94d6': 'CjkUnifiedIdeograph94d6',
u'\u3059': 'HiraganaLetterSu',
u'\ufd5b': 'ArabicLigatureHahWithMeemWithAlefMaksuraFinalForm',
u'\ube5d': 'HangulSyllableBit',
u'\u45e0': 'CjkUnifiedIdeograph45e0',
u'\u06e2': 'ArabicSmallHighMeemIsolatedForm',
u'\uc3e4': 'HangulSyllableSsols',
u'\u6f67': 'CjkUnifiedIdeograph6f67',
u'\u8ce6': 'CjkUnifiedIdeograph8ce6',
u'\u2869': 'BraillePatternDots1467',
u'\u156b': 'CanadianSyllabicsTthe',
u'\ud66d': 'HangulSyllableHwat',
u'\u936f': 'CjkUnifiedIdeograph936f',
u'\u7df0': 'CjkUnifiedIdeograph7df0',
u'\u3ef2': 'CjkUnifiedIdeograph3ef2',
u'\ufbf4': 'ArabicLigatureYehWithHamzaAboveWithYuIsolatedForm',
u'\ua4f6': 'UnicodeCharacter0xa4f6',
u'\u4079': 'CjkUnifiedIdeograph4079',
u'\u0d7b': 'MalayalamLetterChilluN',
u'\uce7d': 'HangulSyllableKalg',
u'\u8b7f': 'CjkUnifiedIdeograph8b7f',
u'\uf781': 'UnicodeCharacter0xf781',
u'\u1004': 'MyanmarLetterNga',
u'\u3c87': 'CjkUnifiedIdeograph3c87',
u'\u9a08': 'CjkUnifiedIdeograph9a08',
u'\ua68b': 'CyrillicSmallLetterTeWithMiddleHook',
u'\u590a': 'CjkUnifiedIdeograph590a',
u'\u658d': 'CjkUnifiedIdeograph658d',
u'\uc30e': 'HangulSyllableSsaj',
u'\uef91': 'UnicodeCharacter0xef91',
u'\u0814': 'UnicodeCharacter0x814',
u'\u5497': 'CjkUnifiedIdeograph5497',
u'\ub218': 'HangulSyllableNuss',
u'\ude9b': 'UnicodeCharacter0xde9b',
u'\u711a': 'CjkUnifiedIdeograph711a',
u'\ufb1e': 'HebrewPointJudeospanishVarika',
u'\u07a1': 'ThaanaLetterZo',
u'\u81a5': 'CjkUnifiedIdeograph81a5',
u'\u2024': 'OneDotLeader',
u'\u4ca7': 'CjkUnifiedIdeograph4ca7',
u'\uaa28': 'ChamLetterHa',
u'\uf6ab': 'UnicodeCharacter0xf6ab',
u'\u692a': 'CjkUnifiedIdeograph692a',
u'\u132e': 'EthiopicSyllableCho',
u'\u3fb1': 'CjkUnifiedIdeograph3fb1',
u'\u9d32': 'CjkUnifiedIdeograph9d32',
u'\ub9b5': 'HangulSyllableRilg',
u'\u5834': 'CjkUnifiedIdeograph5834',
u'\u64b7': 'CjkUnifiedIdeograph64b7',
u'\uc238': 'HangulSyllableSweon',
u'\ueebb': 'UnicodeCharacter0xeebb',
u'\u0b3e': 'OriyaVowelSignAa',
u'\u57c1': 'CjkUnifiedIdeograph57c1',
u'\ub542': 'HangulSyllableDdabs',
u'\ud1c5': 'HangulSyllableTwalg',
u'\u7044': 'CjkUnifiedIdeograph7044',
u'\ufa48': 'CjkCompatibilityIdeographfa48',
u'\u06cb': 'ArabicLetterVe',
u'\u80cf': 'CjkUnifiedIdeograph80cf',
u'\u234e': 'AplFunctionalSymbolDownTackJot',
u'\u4fd1': 'CjkUnifiedIdeograph4fd1',
u'\uad52': 'HangulSyllableGyogg',
u'\uc9d5': 'HangulSyllableJing',
u'\u6854': 'CjkUnifiedIdeograph6854',
u'\u1258': 'EthiopicSyllableQhwa',
u'\u3edb': 'CjkUnifiedIdeograph3edb',
u'\u9c5c': 'CjkUnifiedIdeograph9c5c',
u'\ub8df': 'HangulSyllableRyos',
u'\u5b5e': 'CjkUnifiedIdeograph5b5e',
u'\u67e1': 'CjkUnifiedIdeograph67e1',
u'\uc562': 'HangulSyllableAegg',
u'\ue1e5': 'UnicodeCharacter0xe1e5',
u'\u0a68': 'GurmukhiDigitTwo',
u'\u56eb': 'CjkUnifiedIdeograph56eb',
u'\ub46c': 'HangulSyllableDweo',
u'\ud0ef': 'HangulSyllableTaes',
u'\u736e': 'CjkUnifiedIdeograph736e',
u'\ufd72': 'ArabicLigatureTahWithMeemWithHahInitialForm',
u'\u19f5': 'KhmerSymbolPramRoc',
u'\u83f9': 'CjkUnifiedIdeograph83f9',
u'\u2278': 'NeitherLessthanNorGreaterthan',
u'\u4efb': 'CjkUnifiedIdeograph4efb',
u'\uac7c': 'HangulSyllableGeols',
u'\uc8ff': 'HangulSyllableJugs',
u'\u6b7e': 'CjkUnifiedIdeograph6b7e',
u'\uf203': 'UnicodeCharacter0xf203',
u'\u1582': 'CanadianSyllabicsQoo',
u'\u9f86': 'CjkUnifiedIdeograph9f86',
u'\u3b09': 'CjkUnifiedIdeograph3b09',
u'\u5a88': 'CjkUnifiedIdeograph5a88',
u'\ua50d': 'VaiSyllableVee',
u'\uc48c': 'HangulSyllableSsweols',
u'\u600f': 'CjkUnifiedIdeograph600f',
u'\uea13': 'UnicodeCharacter0xea13',
u'\u0d92': 'SinhalaLetterEeyanna',
u'\u9417': 'CjkUnifiedIdeograph9417',
u'\ub796': 'HangulSyllableRap',
u'\u5319': 'CjkUnifiedIdeograph5319',
u'\u7298': 'CjkUnifiedIdeograph7298',
u'\udd1d': 'UnicodeCharacter0xdd1d',
u'\ufc9c': 'ArabicLigatureBehWithJeemInitialForm',
u'\u0223': 'LatinSmallLetterOu',
u'\u25a2': 'WhiteSquareWithRoundedCorners',
u'\u8c27': 'CjkUnifiedIdeograph8c27',
u'\uafa6': 'HangulSyllableGgyolm',
u'\u4b29': 'CjkUnifiedIdeograph4b29',
u'\u6aa8': 'CjkUnifiedIdeograph6aa8',
u'\uf52d': 'UnicodeCharacter0xf52d',
u'\u14ac': 'CanadianSyllabicsMwe',
u'\u9eb0': 'CjkUnifiedIdeograph9eb0',
u'\u3a33': 'CjkUnifiedIdeograph3a33',
u'\u5db2': 'CjkUnifiedIdeograph5db2',
u'\ua437': 'YiSyllableNjie',
u'\uc7b6': 'HangulSyllableJaelm',
u'\u6339': 'CjkUnifiedIdeograph6339',
u'\ued3d': 'UnicodeCharacter0xed3d',
u'\u0cbc': 'KannadaSignNukta',
u'\u9741': 'CjkUnifiedIdeograph9741',
u'\ub6c0': 'HangulSyllableDdweol',
u'\u5243': 'CjkUnifiedIdeograph5243',
u'\u75c2': 'CjkUnifiedIdeograph75c2',
u'\udc47': 'UnicodeCharacter0xdc47',
u'\uffc6': 'HalfwidthHangulLetterEo',
u'\u054d': 'ArmenianCapitalLetterSeh',
u'\u24cc': 'CircledLatinCapitalLetterW',
u'\u8f51': 'CjkUnifiedIdeograph8f51',
u'\uaed0': 'HangulSyllableGgeoss',
u'\u4a53': 'CjkUnifiedIdeograph4a53',
u'\u6dd2': 'CjkUnifiedIdeograph6dd2',
u'\uf457': 'UnicodeCharacter0xf457',
u'\u17d6': 'KhmerSignCamnucPiiKuuh',
u'\u91da': 'CjkUnifiedIdeograph91da',
u'\u3d5d': 'CjkUnifiedIdeograph3d5d',
u'\u5cdc': 'CjkUnifiedIdeograph5cdc',
u'\ua761': 'LatinSmallLetterVy',
u'\uc6e0': 'HangulSyllableWeoss',
u'\u6263': 'CjkUnifiedIdeograph6263',
u'\uec67': 'UnicodeCharacter0xec67',
u'\u0fe6': 'UnicodeCharacter0xfe6',
u'\u966b': 'CjkUnifiedIdeograph966b',
u'\u89ea': 'CjkUnifiedIdeograph89ea',
u'\u556d': 'CjkUnifiedIdeograph556d',
u'\u74ec': 'CjkUnifiedIdeograph74ec',
u'\udf71': 'UnicodeCharacter0xdf71',
u'\ufef0': 'ArabicLetterAlefMaksuraFinalForm',
u'\u0477': 'CyrillicSmallLetterIzhitsaWithDoubleGraveAccent',
u'\u27f6': 'LongRightwardsArrow',
u'\u8e7b': 'CjkUnifiedIdeograph8e7b',
u'\ua1fa': 'YiSyllableKi',
u'\u4d7d': 'CjkUnifiedIdeograph4d7d',
u'\u6cfc': 'CjkUnifiedIdeograph6cfc',
u'\u7681': 'CjkUnifiedIdeograph7681',
u'\u1700': 'TagalogLetterA',
u'\u3383': 'SquareMa',
u'\ud402': 'HangulSyllablePoj',
u'\uf085': 'UnicodeCharacter0xf085',
u'\u9104': 'CjkUnifiedIdeograph9104',
u'\ubd87': 'HangulSyllableBud',
u'\u420e': 'CjkUnifiedIdeograph420e',
u'\u6e91': 'CjkUnifiedIdeograph6e91',
u'\u0f10': 'TibetanMarkNyisTshegShad',
u'\u2b93': 'UnicodeCharacter0x2b93',
u'\ucc12': 'HangulSyllableJjinh',
u'\ue895': 'UnicodeCharacter0xe895',
u'\u8914': 'CjkUnifiedIdeograph8914',
u'\ud597': 'HangulSyllableHyad',
u'\u9299': 'CjkUnifiedIdeograph9299',
u'\u7a1e': 'CjkUnifiedIdeograph7a1e',
u'\u2720': 'MalteseCross',
u'\u43a3': 'CjkUnifiedIdeograph43a3',
u'\ue422': 'UnicodeCharacter0xe422',
u'\xa5': 'YenSign',
u'\ua124': 'YiSyllableTap',
u'\ucda7': 'HangulSyllableCus',
u'\u8aa9': 'CjkUnifiedIdeograph8aa9',
u'\u5f30': 'CjkUnifiedIdeograph5f30',
u'\u7bb3': 'CjkUnifiedIdeograph7bb3',
u'\u1c32': 'LepchaConsonantSignR',
u'\u38b5': 'CjkUnifiedIdeograph38b5',
u'\ud934': 'UnicodeCharacter0xd934',
u'\ue5b7': 'UnicodeCharacter0xe5b7',
u'\u8636': 'CjkUnifiedIdeograph8636',
u'\ua2b9': 'YiSyllableCap',
u'\u7740': 'CjkUnifiedIdeograph7740',
u'\u3442': 'CjkUnifiedIdeograph3442',
u'\u50c5': 'CjkUnifiedIdeograph50c5',
u'\uf144': 'UnicodeCharacter0xf144',
u'\u1dc7': 'CombiningAcutemacron',
u'\ube46': 'HangulSyllableBigg',
u'\udac9': 'UnicodeCharacter0xdac9',
u'\u87cb': 'CjkUnifiedIdeograph87cb',
u'\u6f50': 'CjkUnifiedIdeograph6f50',
u'\u2c52': 'GlagoliticSmallLetterSpideryHa',
u'\u48d5': 'CjkUnifiedIdeograph48d5',
u'\ue954': 'UnicodeCharacter0xe954',
u'\u35d7': 'CjkUnifiedIdeograph35d7',
u'\ud656': 'HangulSyllableHwagg',
u'\uf2d9': 'UnicodeCharacter0xf2d9',
u'\u9358': 'CjkUnifiedIdeograph9358',
u'\ubfdb': 'HangulSyllableBbulh',
u'\u4462': 'CjkUnifiedIdeograph4462',
u'\u60e5': 'CjkUnifiedIdeograph60e5',
u'\u0164': 'LatinCapitalLetterTWithCaron',
u'\u2de7': 'CombiningCyrillicLetterEl',
u'\uce66': 'HangulSyllableCilp',
u'\ueae9': 'UnicodeCharacter0xeae9',
u'\u8b68': 'CjkUnifiedIdeograph8b68',
u'\ud7eb': 'UnicodeCharacter0xd7eb',
u'\u94ed': 'CjkUnifiedIdeograph94ed',
u'\u7c72': 'CjkUnifiedIdeograph7c72',
u'\u3974': 'CjkUnifiedIdeograph3974',
u'\u45f7': 'CjkUnifiedIdeograph45f7',
u'\ue676': 'UnicodeCharacter0xe676',
u'\u02f9': 'ModifierLetterBeginHighTone',
u'\ua378': 'YiSyllableChy',
u'\ucffb': 'HangulSyllableKuh',
u'\u8cfd': 'CjkUnifiedIdeograph8cfd',
u'\ub601': 'HangulSyllableDdyelt',
u'\u5184': 'CjkUnifiedIdeograph5184',
u'\u1e86': 'LatinCapitalLetterWWithDotAbove',
u'\udb88': 'UnicodeCharacter0xdb88',
u'\u670b': 'CjkUnifiedIdeograph670b',
u'\u988a': 'CjkUnifiedIdeograph988a',
u'\u240d': 'SymbolForCarriageReturn',
u'\ue10f': 'UnicodeCharacter0xe10f',
u'\uae11': 'HangulSyllableGeut',
u'\u4994': 'CjkUnifiedIdeograph4994',
u'\u3696': 'CjkUnifiedIdeograph3696',
u'\uf398': 'UnicodeCharacter0xf398',
u'\ub09a': 'HangulSyllableNagg',
u'\u5c1d': 'CjkUnifiedIdeograph5c1d',
u'\u191f': 'UnicodeCharacter0x191f',
u'\uc621': 'HangulSyllableYet',
u'\u8323': 'CjkUnifiedIdeograph8323',
u'\u61a4': 'CjkUnifiedIdeograph61a4',
u'\u2ea6': 'CjkRadicalSimplifiedHalfTreeTrunk',
u'\ueba8': 'UnicodeCharacter0xeba8',
u'\ua8aa': 'SaurashtraLetterMa',
u'\u742d': 'CjkUnifiedIdeograph742d',
u'\u95ac': 'CjkUnifiedIdeograph95ac',
u'\u312f': 'UnicodeCharacter0x312f',
u'\ufe31': 'PresentationFormForVerticalEmDash',
u'\ubb33': 'HangulSyllableMyoh',
u'\u46b6': 'CjkUnifiedIdeograph46b6',
u'\u03b8': 'GreekSmallLetterTheta',
u'\uc0ba': 'HangulSyllableSalp',
u'\u6c3d': 'CjkUnifiedIdeograph6c3d',
u'\u8dbc': 'CjkUnifiedIdeograph8dbc',
u'\u293f': 'LowerLeftSemicircularAnticlockwiseArrow',
u'\u1641': 'CanadianSyllabicsCarrierZo',
u'\ud343': 'HangulSyllablePaeh',
u'\u9045': 'CjkUnifiedIdeograph9045',
u'\u7ec6': 'CjkUnifiedIdeograph7ec6',
u'\u3bc8': 'CjkUnifiedIdeograph3bc8',
u'\uf8ca': 'UnicodeCharacter0xf8ca',
u'\ua5cc': 'VaiSyllableTho',
u'\u414f': 'CjkUnifiedIdeograph414f',
u'\u0e51': 'ThaiDigitOne',
u'\ucb53': 'HangulSyllableJjulb',
u'\u8855': 'CjkUnifiedIdeograph8855',
u'\u53d8': 'CjkUnifiedIdeograph53d8',
u'\u10da': 'GeorgianLetterLas',
u'\udddc': 'UnicodeCharacter0xdddc',
u'\u795f': 'CjkUnifiedIdeograph795f',
u'\u9ade': 'CjkUnifiedIdeograph9ade',
u'\u2661': 'WhiteHeartSuit',
u'\ue363': 'UnicodeCharacter0xe363',
u'\ua065': 'YiSyllableBbot',
u'\u4be8': 'CjkUnifiedIdeograph4be8',
u'\u08ea': 'UnicodeCharacter0x8ea',
u'\uf5ec': 'UnicodeCharacter0xf5ec',
u'\ub2ee': 'HangulSyllableDalm',
u'\u5e71': 'CjkUnifiedIdeograph5e71',
u'\u1b73': 'BalineseMusicalSymbolCombiningGong',
u'\ud875': 'UnicodeCharacter0xd875',
u'\u8577': 'CjkUnifiedIdeograph8577',
u'\u63f8': 'CjkUnifiedIdeograph63f8',
u'\u20fa': 'UnicodeCharacter0x20fa',
u'\uedfc': 'UnicodeCharacter0xedfc',
u'\uaafe': 'UnicodeCharacter0xaafe',
u'\u9800': 'CjkUnifiedIdeograph9800',
u'\ubc83': 'HangulSyllableByaeh',
u'\u5b02': 'CjkUnifiedIdeograph5b02',
u'\u7f85': 'CjkUnifiedIdeograph7f85',
u'\ud106': 'HangulSyllableTyalp',
u'\uf589': 'UnicodeCharacter0xf589',
u'\u0a0c': 'UnicodeCharacter0xa0c',
u'\u2e8f': 'CjkRadicalLameTwo',
u'\u8010': 'CjkUnifiedIdeograph8010',
u'\ua493': 'YiRadicalNyip',
u'\u4312': 'CjkUnifiedIdeograph4312',
u'\u6795': 'CjkUnifiedIdeograph6795',
u'\uf916': 'CjkCompatibilityIdeographf916',
u'\u1d99': 'LatinSmallLetterUWithRetroflexHook',
u'\u939d': 'CjkUnifiedIdeograph939d',
u'\u321c': 'ParenthesizedHangulCieucU',
u'\u569f': 'CjkUnifiedIdeograph569f',
u'\ua820': 'SylotiNagriLetterRro',
u'\ucca3': 'HangulSyllableCeolb',
u'\u6b22': 'CjkUnifiedIdeograph6b22',
u'\ue126': 'UnicodeCharacter0xe126',
u'\u05a9': 'HebrewAccentTelishaQetana',
u'\ubbad': 'HangulSyllableMyulg',
u'\u5a2c': 'CjkUnifiedIdeograph5a2c',
u'\u7eaf': 'CjkUnifiedIdeograph7eaf',
u'\ud030': 'HangulSyllableKwek',
u'\uf4b3': 'UnicodeCharacter0xf4b3',
u'\u0936': 'DevanagariLetterSha',
u'\u2db9': 'EthiopicSyllableCchu',
u'\u8f3a': 'CjkUnifiedIdeograph8f3a',
u'\ua3bd': 'YiSyllableShup',
u'\u423c': 'CjkUnifiedIdeograph423c',
u'\u66bf': 'CjkUnifiedIdeograph66bf',
u'\uf840': 'UnicodeCharacter0xf840',
u'\u1cc3': 'UnicodeCharacter0x1cc3',
u'\u92c7': 'CjkUnifiedIdeograph92c7',
u'\u3146': 'HangulLetterSsangsios',
u'\u55c9': 'CjkUnifiedIdeograph55c9',
u'\ub74a': 'HangulSyllableDdyinh',
u'\ucbcd': 'HangulSyllableJjyung',
u'\u6a4c': 'CjkUnifiedIdeograph6a4c',
u'\ue050': 'UnicodeCharacter0xe050',
u'\u04d3': 'CyrillicSmallLetterAWithDiaeresis',
u'\ubad7': 'HangulSyllableMwas',
u'\u5956': 'CjkUnifiedIdeograph5956',
u'\u7dd9': 'CjkUnifiedIdeograph7dd9',
u'\udf5a': 'UnicodeCharacter0xdf5a',
u'\uf3dd': 'UnicodeCharacter0xf3dd',
u'\u0860': 'UnicodeCharacter0x860',
u'\u2ce3': 'CopticSmallLetterOldNubianWau',
u'\u8e64': 'CjkUnifiedIdeograph8e64',
u'\ua2e7': 'YiSyllableZzyt',
u'\u4166': 'CjkUnifiedIdeograph4166',
u'\u65e9': 'CjkUnifiedIdeograph65e9',
u'\uc76a': 'HangulSyllableYibs',
u'\u1bed': 'UnicodeCharacter0x1bed',
u'\u91f1': 'CjkUnifiedIdeograph91f1',
u'\u3070': 'HiraganaLetterBa',
u'\u54f3': 'CjkUnifiedIdeograph54f3',
u'\ub674': 'HangulSyllableDdoem',
u'\ucaf7': 'HangulSyllableJjwaegs',
u'\u6976': 'CjkUnifiedIdeograph6976',
u'\uef7a': 'UnicodeCharacter0xef7a',
u'\u03fd': 'GreekCapitalReversedLunateSigmaSymbol',
u'\u3901': 'CjkUnifiedIdeograph3901',
u'\u5880': 'CjkUnifiedIdeograph5880',
u'\ubf05': 'HangulSyllableBbet',
u'\ude84': 'UnicodeCharacter0xde84',
u'\u7207': 'CjkUnifiedIdeograph7207',
u'\ue80b': 'UnicodeCharacter0xe80b',
u'\u178a': 'KhmerLetterDa',
u'\u8d8e': 'CjkUnifiedIdeograph8d8e',
u'\u2111': 'BlackletterCapitalI',
u'\u4090': 'CjkUnifiedIdeograph4090',
u'\ua715': 'ModifierLetterLowLeftstemToneBar',
u'\uc694': 'HangulSyllableYo',
u'\u101b': 'MyanmarLetterRa',
u'\u3f9a': 'CjkUnifiedIdeograph3f9a',
u'\u961f': 'CjkUnifiedIdeograph961f',
u'\ub59e': 'HangulSyllableDdyaep',
u'\u4921': 'CjkUnifiedIdeograph4921',
u'\u68a0': 'CjkUnifiedIdeograph68a0',
u'\ucf25': 'HangulSyllableKyeolg',
u'\ueea4': 'UnicodeCharacter0xeea4',
u'\u382b': 'CjkUnifiedIdeograph382b',
u'\u27aa': 'LeftshadedWhiteRightwardsArrow',
u'\ube2f': 'HangulSyllableByid',
u'\uddae': 'UnicodeCharacter0xddae',
u'\u7131': 'CjkUnifiedIdeograph7131',
u'\uf735': 'UnicodeCharacter0xf735',
u'\u16b4': 'RunicLetterKaunK',
u'\u8cb8': 'CjkUnifiedIdeograph8cb8',
u'\u203b': 'ReferenceMark',
u'\u4fba': 'CjkUnifiedIdeograph4fba',
u'\ua63f': 'UnicodeCharacter0xa63f',
u'\uc5be': 'HangulSyllableEolm',
u'\u1f45': 'GreekSmallLetterOmicronWithDasiaAndOxia',
u'\u3ec4': 'CjkUnifiedIdeograph3ec4',
u'\u9549': 'CjkUnifiedIdeograph9549',
u'\ub4c8': 'HangulSyllableDyul',
u'\u484b': 'CjkUnifiedIdeograph484b',
u'\u77ca': 'CjkUnifiedIdeograph77ca',
u'\uce4f': 'HangulSyllableCyis',
u'\uedce': 'UnicodeCharacter0xedce',
u'\u0755': 'ArabicLetterBehWithInvertedSmallVBelow',
u'\u26d4': 'UnicodeCharacter0x26d4',
u'\ubd59': 'HangulSyllableBoeb',
u'\udcd8': 'UnicodeCharacter0xdcd8',
u'\u705b': 'CjkUnifiedIdeograph705b',
u'\uf65f': 'UnicodeCharacter0xf65f',
u'\u15de': 'CanadianSyllabicsCarrierThe',
u'\u8be2': 'CjkUnifiedIdeograph8be2',
u'\u2f65': 'KangxiRadicalField',
u'\u4ee4': 'CjkUnifiedIdeograph4ee4',
u'\ua569': 'VaiSyllableYa',
u'\uc4e8': 'HangulSyllableSsyuss',
u'\u1e6f': 'LatinSmallLetterTWithLineBelow',
u'\u3dee': 'CjkUnifiedIdeograph3dee',
u'\u9473': 'CjkUnifiedIdeograph9473',
u'\ub3f2': 'HangulSyllableDwabs',
u'\u5775': 'CjkUnifiedIdeograph5775',
u'\u76f4': 'CjkUnifiedIdeograph76f4',
u'\ucd79': 'HangulSyllableCyog',
u'\uecf8': 'UnicodeCharacter0xecf8',
u'\u067f': 'ArabicLetterTeheh',
u'\u25fe': 'BlackMediumSmallSquare',
u'\u5006': 'CjkUnifiedIdeograph5006',
u'\u7489': 'CjkUnifiedIdeograph7489',
u'\u1508': 'CanadianSyllabicsBlackfootS',
u'\u298b': 'LeftSquareBracketWithUnderbar',
u'\ud60a': 'HangulSyllableHyeolm',
u'\uea8d': 'UnicodeCharacter0xea8d',
u'\u8b0c': 'CjkUnifiedIdeograph8b0c',
u'\uaf8f': 'HangulSyllableGgoelh',
u'\u7816': 'CjkUnifiedIdeograph7816',
u'\u3d18': 'CjkUnifiedIdeograph3d18',
u'\u519b': 'CjkUnifiedIdeograph519b',
u'\ufe1a': 'UnicodeCharacter0xfe1a',
u'\u129d': 'EthiopicSyllableNye',
u'\ub31c': 'HangulSyllableDya',
u'\ud79f': 'HangulSyllableHic',
u'\u88a1': 'CjkUnifiedIdeograph88a1',
u'\u6026': 'CjkUnifiedIdeograph6026',
u'\u2528': 'BoxDrawingsVerticalHeavyAndLeftLight',
u'\u79ab': 'CjkUnifiedIdeograph79ab',
u'\ue62a': 'UnicodeCharacter0xe62a',
u'\u3aad': 'CjkUnifiedIdeograph3aad',
u'\udb2c': 'UnicodeCharacter0xdb2c',
u'\uffaf': 'HalfwidthHangulLetterRieulphieuph',
u'\u9c2e': 'CjkUnifiedIdeograph9c2e',
u'\ub0b1': 'HangulSyllableNat',
u'\u4d38': 'CjkUnifiedIdeograph4d38',
u'\u61bb': 'CjkUnifiedIdeograph61bb',
u'\u0e3a': 'ThaiCharacterPhinthu',
u'\u22bd': 'Nor',
u'\uc33c': 'HangulSyllableSsyals',
u'\ue7bf': 'UnicodeCharacter0xe7bf',
u'\u843e': 'CjkUnifiedIdeograph843e',
u'\ud8c1': 'UnicodeCharacter0xd8c1',
u'\u9dc3': 'CjkUnifiedIdeograph9dc3',
u'\u7548': 'CjkUnifiedIdeograph7548',
u'\u364a': 'CjkUnifiedIdeograph364a',
u'\u4acd': 'CjkUnifiedIdeograph4acd',
u'\ueb4c': 'UnicodeCharacter0xeb4c',
u'\u0fcf': 'TibetanSignRdelNagGsum',
u'\uac4e': 'HangulSyllableGyaj',
u'\uc0d1': 'HangulSyllableSaelg',
u'\u85d3': 'CjkUnifiedIdeograph85d3',
u'\u5e5a': 'CjkUnifiedIdeograph5e5a',
u'\u72dd': 'CjkUnifiedIdeograph72dd',
u'\u135c': 'UnicodeCharacter0x135c',
u'\u37df': 'CjkUnifiedIdeograph37df',
u'\ud45e': 'HangulSyllablePyogg',
u'\ue8e1': 'UnicodeCharacter0xe8e1',
u'\u8960': 'CjkUnifiedIdeograph8960',
u'\uade3': 'HangulSyllableGyud',
u'\u466a': 'CjkUnifiedIdeograph466a',
u'\u3b6c': 'CjkUnifiedIdeograph3b6c',
u'\u5fef': 'CjkUnifiedIdeograph5fef',
u'\ufc6e': 'ArabicLigatureBehWithAlefMaksuraFinalForm',
u'\u10f1': 'GeorgianLetterHe',
u'\ub170': 'HangulSyllableNyess',
u'\ud5f3': 'HangulSyllableHelh',
u'\u96f5': 'CjkUnifiedIdeograph96f5',
u'\u6e7a': 'CjkUnifiedIdeograph6e7a',
u'\u237c': 'RightAngleWithDownwardsZigzagArrow',
u'\u47ff': 'CjkUnifiedIdeograph47ff',
u'\ue47e': 'UnicodeCharacter0xe47e',
u'\ud980': 'UnicodeCharacter0xd980',
u'\u7d03': 'CjkUnifiedIdeograph7d03',
u'\u9a82': 'CjkUnifiedIdeograph9a82',
u'\u3e05': 'CjkUnifiedIdeograph3e05',
u'\uf307': 'UnicodeCharacter0xf307',
u'\ub409': 'HangulSyllableDwaelt',
u'\u4b8c': 'CjkUnifiedIdeograph4b8c',
u'\u0c8e': 'KannadaLetterE',
u'\uc190': 'HangulSyllableSon',
u'\u6513': 'CjkUnifiedIdeograph6513',
u'\u8292': 'CjkUnifiedIdeograph8292',
u'\u2615': 'HotBeverage',
u'\u1b17': 'BalineseLetterNga',
u'\udc19': 'UnicodeCharacter0xdc19',
u'\u911b': 'CjkUnifiedIdeograph911b',
u'\u739c': 'CjkUnifiedIdeograph739c',
u'\u349e': 'CjkUnifiedIdeograph349e',
u'\ue9a0': 'UnicodeCharacter0xe9a0',
u'\uaaa2': 'UnicodeCharacter0xaaa2',
u'\u4e25': 'CjkUnifiedIdeograph4e25',
u'\u0327': 'CombiningCedilla',
u'\uc429': 'HangulSyllableSswaet',
u'\ub92b': 'HangulSyllableRwelb',
u'\u5cae': 'CjkUnifiedIdeograph5cae',
u'\u11b0': 'HangulJongseongRieulkiyeok',
u'\ud2b2': 'HangulSyllableTyuj',
u'\u7635': 'CjkUnifiedIdeograph7635',
u'\u97b4': 'CjkUnifiedIdeograph97b4',
u'\u2b37': 'LeftwardsTwoheadedTripleDashArrow',
u'\uec39': 'UnicodeCharacter0xec39',
u'\ua13b': 'YiSyllableDdie',
u'\u44be': 'CjkUnifiedIdeograph44be',
u'\u39c0': 'CjkUnifiedIdeograph39c0',
u'\ufac2': 'CjkCompatibilityIdeographfac2',
u'\ubfc4': 'HangulSyllableBbyoss',
u'\u5347': 'CjkUnifiedIdeograph5347',
u'\u1449': 'CanadianSyllabicsP',
u'\uc94b': 'HangulSyllableJwec',
u'\u8a4d': 'CjkUnifiedIdeograph8a4d',
u'\u6cce': 'CjkUnifiedIdeograph6cce',
u'\u21d0': 'LeftwardsDoubleArrow',
u'\ue2d2': 'UnicodeCharacter0xe2d2',
u'\ua7d4': 'UnicodeCharacter0xa7d4',
u'\u7b57': 'CjkUnifiedIdeograph7b57',
u'\u98d6': 'CjkUnifiedIdeograph98d6',
u'\u3c59': 'CjkUnifiedIdeograph3c59',
u'\uf15b': 'UnicodeCharacter0xf15b',
u'\ub25d': 'HangulSyllableNwinj',
u'\u49e0': 'CjkUnifiedIdeograph49e0',
u'\u0ae2': 'GujaratiVowelSignVocalicL',
u'\ucfe4': 'HangulSyllableKun',
u'\u6367': 'CjkUnifiedIdeograph6367',
u'\u80e6': 'CjkUnifiedIdeograph80e6',
u'\u2469': 'CircledNumberTen',
u'\u196b': 'TaiLeLetterE',
u'\uda6d': 'UnicodeCharacter0xda6d',
u'\u9f6f': 'CjkUnifiedIdeograph9f6f',
u'\u71f0': 'CjkUnifiedIdeograph71f0',
u'\u32f2': 'CircledKatakanaMo',
u'\uf7f4': 'UnicodeCharacter0xf7f4',
u'\ua8f6': 'UnicodeCharacter0xa8f6',
u'\u4c79': 'CjkUnifiedIdeograph4c79',
u'\u017b': 'LatinCapitalLetterZWithDotAbove',
u'\uc27d': 'HangulSyllableSwib',
u'\u877f': 'CjkUnifiedIdeograph877f',
u'\ufb81': 'ArabicLetterTchehehMedialForm',
u'\u1c04': 'LepchaLetterGla',
u'\u3087': 'HiraganaLetterSmallYo',
u'\u9608': 'CjkUnifiedIdeograph9608',
u'\uaa8b': 'UnicodeCharacter0xaa8b',
u'\u550a': 'CjkUnifiedIdeograph550a',
u'\u698d': 'CjkUnifiedIdeograph698d',
u'\ucf0e': 'HangulSyllableKelp',
u'\ue391': 'UnicodeCharacter0xe391',
u'\u0414': 'CyrillicCapitalLetterDe',
u'\u5897': 'CjkUnifiedIdeograph5897',
u'\ube18': 'HangulSyllableBeuls',
u'\ud29b': 'HangulSyllableTwih',
u'\u7d1a': 'CjkUnifiedIdeograph7d1a',
u'\uf71e': 'UnicodeCharacter0xf71e',
u'\u0ba1': 'UnicodeCharacter0xba1',
u'\u8da5': 'CjkUnifiedIdeograph8da5',
u'\u2c24': 'GlagoliticCapitalLetterSmallYus',
u'\u40a7': 'CjkUnifiedIdeograph40a7',
u'\ua628': 'VaiDigitEight',
u'\ufaab': 'CjkCompatibilityIdeographfaab',
u'\u652a': 'CjkUnifiedIdeograph652a',
u'\u1f2e': 'GreekCapitalLetterEtaWithPsiliAndPerispomeni',
u'\u33b1': 'SquareNs',
u'\u9132': 'CjkUnifiedIdeograph9132',
u'\ub5b5': 'HangulSyllableDdeong',
u'\u5434': 'CjkUnifiedIdeograph5434',
u'\u68b7': 'CjkUnifiedIdeograph68b7',
u'\uce38': 'HangulSyllableCeuk',
u'\ue2bb': 'UnicodeCharacter0xe2bb',
u'\u073e': 'SyriacEsasaBelow',
u'\u5bc1': 'CjkUnifiedIdeograph5bc1',
u'\ub942': 'HangulSyllableRwinh',
u'\uddc5': 'UnicodeCharacter0xddc5',
u'\u7c44': 'CjkUnifiedIdeograph7c44',
u'\uf648': 'UnicodeCharacter0xf648',
u'\u0acb': 'GujaratiVowelSignO',
u'\u8ccf': 'CjkUnifiedIdeograph8ccf',
u'\u2f4e': 'KangxiRadicalWeapon',
u'\u43d1': 'CjkUnifiedIdeograph43d1',
u'\ua152': 'YiSyllableNdix',
u'\uc5d5': 'HangulSyllableEnj',
u'\u6454': 'CjkUnifiedIdeograph6454',
u'\u1e58': 'LatinCapitalLetterRWithDotAbove',
u'\u32db': 'CircledKatakanaSi',
u'\u905c': 'CjkUnifiedIdeograph905c',
u'\ub4df': 'HangulSyllableDeugs',
u'\u575e': 'CjkUnifiedIdeograph575e',
u'\u6be1': 'CjkUnifiedIdeograph6be1',
u'\uc962': 'HangulSyllableJwibs',
u'\uede5': 'UnicodeCharacter0xede5',
u'\u0668': 'ArabicindicDigitEight',
u'\u5aeb': 'CjkUnifiedIdeograph5aeb',
u'\ub86c': 'HangulSyllableRom',
u'\udcef': 'UnicodeCharacter0xdcef',
u'\u7f6e': 'CjkUnifiedIdeograph7f6e',
u'\uf172': 'UnicodeCharacter0xf172',
u'\u15f5': 'CanadianSyllabicsCarrierKhu',
u'\u8ff9': 'CjkUnifiedIdeograph8ff9',
u'\u2e78': 'UnicodeCharacter0x2e78',
u'\u42fb': 'CjkUnifiedIdeograph42fb',
u'\ua07c': 'YiSyllableNbiep',
u'\uc4ff': 'HangulSyllableSseulh',
u'\u677e': 'CjkUnifiedIdeograph677e',
u'\ufe03': 'VariationSelector4',
u'\u1982': 'NewTaiLueLetterHighKa',
u'\u9386': 'CjkUnifiedIdeograph9386',
u'\u3709': 'CjkUnifiedIdeograph3709',
u'\u5688': 'CjkUnifiedIdeograph5688',
u'\ua90d': 'KayahLiLetterNga',
u'\uc88c': 'HangulSyllableJwa',
u'\u6c0f': 'CjkUnifiedIdeograph6c0f',
u'\ue613': 'UnicodeCharacter0xe613',
u'\u0192': 'LatinSmallLetterFWithHook',
u'\u9817': 'CjkUnifiedIdeograph9817',
u'\ubb96': 'HangulSyllableMwilp',
u'\u5f19': 'CjkUnifiedIdeograph5f19',
u'\u7e98': 'CjkUnifiedIdeograph7e98',
u'\ud11d': 'HangulSyllableTyaelg',
u'\uf09c': 'UnicodeCharacter0xf09c',
u'\u0e23': 'ThaiCharacterRoRua',
u'\u29a2': 'TurnedAngle',
u'\u8027': 'CjkUnifiedIdeograph8027',
u'\ua3a6': 'YiSyllableNryx',
u'\u4729': 'CjkUnifiedIdeograph4729',
u'\u66a8': 'CjkUnifiedIdeograph66a8',
u'\uf92d': 'CjkCompatibilityIdeographf92d',
u'\u18ac': 'UnicodeCharacter0x18ac',
u'\u92b0': 'CjkUnifiedIdeograph92b0',
u'\u3633': 'CjkUnifiedIdeograph3633',
u'\u51b2': 'CjkUnifiedIdeograph51b2',
u'\ua837': 'UnicodeCharacter0xa837',
u'\ucbb6': 'HangulSyllableJjwip',
u'\u6f39': 'CjkUnifiedIdeograph6f39',
u'\ue13d': 'UnicodeCharacter0xe13d',
u'\xbc': 'VulgarFractionOneQuarter',
u'\u9b41': 'CjkUnifiedIdeograph9b41',
u'\ubac0': 'HangulSyllableMok',
u'\u5e43': 'CjkUnifiedIdeograph5e43',
u'\u79c2': 'CjkUnifiedIdeograph79c2',
u'\ud047': 'HangulSyllableKwis',
u'\uf3c6': 'UnicodeCharacter0xf3c6',
u'\u094d': 'DevanagariSignVirama',
u'\u28cc': 'BraillePatternDots3478',
u'\u8351': 'CjkUnifiedIdeograph8351',
u'\ua2d0': 'YiSyllableZzit',
u'\u4653': 'CjkUnifiedIdeograph4653',
u'\u61d2': 'CjkUnifiedIdeograph61d2',
u'\uf857': 'UnicodeCharacter0xf857',
u'\u1bd6': 'UnicodeCharacter0x1bd6',
u'\u9dda': 'CjkUnifiedIdeograph9dda',
u'\u315d': 'HangulLetterWeo',
u'\u50dc': 'CjkUnifiedIdeograph50dc',
u'\uab61': 'UnicodeCharacter0xab61',
u'\ucae0': 'HangulSyllableJjwal',
u'\u6e63': 'CjkUnifiedIdeograph6e63',
u'\ue067': 'UnicodeCharacter0xe067',
u'\u03e6': 'CopticCapitalLetterKhei',
u'\u9a6b': 'CjkUnifiedIdeograph9a6b',
u'\u85ea': 'CjkUnifiedIdeograph85ea',
u'\u596d': 'CjkUnifiedIdeograph596d',
u'\u78ec': 'CjkUnifiedIdeograph78ec',
u'\ud371': 'HangulSyllablePyaeb',
u'\uf2f0': 'UnicodeCharacter0xf2f0',
u'\u0877': 'UnicodeCharacter0x877',
u'\u2bf6': 'UnicodeCharacter0x2bf6',
u'\u827b': 'CjkUnifiedIdeograph827b',
u'\uadfa': 'HangulSyllableGeugg',
u'\u417d': 'CjkUnifiedIdeograph417d',
u'\u60fc': 'CjkUnifiedIdeograph60fc',
u'\u7a81': 'CjkUnifiedIdeograph7a81',
u'\u1b00': 'BalineseSignUluRicem',
u'\u3f83': 'CjkUnifiedIdeograph3f83',
u'\ud802': 'UnicodeCharacter0xd802',
u'\ufc85': 'ArabicLigatureLamWithMeemFinalForm',
u'\u9d04': 'CjkUnifiedIdeograph9d04',
u'\ub187': 'HangulSyllableNolh',
u'\u4e0e': 'CjkUnifiedIdeograph4e0e',
u'\u6291': 'CjkUnifiedIdeograph6291',
u'\u0310': 'CombiningCandrabindu',
u'\u2793': 'DingbatNegativeCircledSansserifNumberTen',
u'\uc012': 'HangulSyllableBbwelp',
u'\ue495': 'UnicodeCharacter0xe495',
u'\u8514': 'CjkUnifiedIdeograph8514',
u'\ud997': 'UnicodeCharacter0xd997',
u'\u9e99': 'CjkUnifiedIdeograph9e99',
u'\u761e': 'CjkUnifiedIdeograph761e',
u'\u2b20': 'WhitePentagon',
u'\u4fa3': 'CjkUnifiedIdeograph4fa3',
u'\ue822': 'UnicodeCharacter0xe822',
u'\u0ca5': 'KannadaLetterTha',
u'\uad24': 'HangulSyllableGwaels',
u'\uc1a7': 'HangulSyllableSoh',
u'\u86a9': 'CjkUnifiedIdeograph86a9',
u'\u5330': 'CjkUnifiedIdeograph5330',
u'\u77b3': 'CjkUnifiedIdeograph77b3',
u'\u1032': 'MyanmarVowelSignAi',
u'\u34b5': 'CjkUnifiedIdeograph34b5',
u'\ud534': 'HangulSyllablePyiss',
u'\ue9b7': 'UnicodeCharacter0xe9b7',
u'\u8a36': 'CjkUnifiedIdeograph8a36',
u'\uaeb9': 'HangulSyllableGgyaet',
u'\u7b40': 'CjkUnifiedIdeograph7b40',
u'\u3842': 'CjkUnifiedIdeograph3842',
u'\u5cc5': 'CjkUnifiedIdeograph5cc5',
u'\ufd44': 'UnicodeCharacter0xfd44',
u'\u11c7': 'HangulJongseongNieunsios',
u'\ub246': 'HangulSyllableNwelm',
u'\ud6c9': 'HangulSyllableHunj',
u'\u8bcb': 'CjkUnifiedIdeograph8bcb',
u'\u6350': 'CjkUnifiedIdeograph6350',
u'\u2052': 'CommercialMinusSign',
u'\u44d5': 'CjkUnifiedIdeograph44d5',
u'\ue554': 'UnicodeCharacter0xe554',
u'\u39d7': 'CjkUnifiedIdeograph39d7',
u'\uda56': 'UnicodeCharacter0xda56',
u'\ufed9': 'ArabicLetterKafIsolatedForm',
u'\u9f58': 'CjkUnifiedIdeograph9f58',
u'\ub3db': 'HangulSyllableDoc',
u'\u4862': 'CjkUnifiedIdeograph4862',
u'\u6ce5': 'CjkUnifiedIdeograph6ce5',
u'\u0d64': 'UnicodeCharacter0xd64',
u'\u21e7': 'UpwardsWhiteArrow',
u'\uc266': 'HangulSyllableSwej',
u'\ue6e9': 'UnicodeCharacter0xe6e9',
u'\u8768': 'CjkUnifiedIdeograph8768',
u'\udbeb': 'UnicodeCharacter0xdbeb',
u'\u98ed': 'CjkUnifiedIdeograph98ed',
u'\u7072': 'CjkUnifiedIdeograph7072',
u'\u3574': 'CjkUnifiedIdeograph3574',
u'\u49f7': 'CjkUnifiedIdeograph49f7',
u'\uea76': 'UnicodeCharacter0xea76',
u'\u0ef9': 'UnicodeCharacter0xef9',
u'\uaf78': 'HangulSyllableGgwaess',
u'\uc3fb': 'HangulSyllableSswad',
u'\u80fd': 'CjkUnifiedIdeograph80fd',
u'\uba01': 'HangulSyllableMyag',
u'\u5d84': 'CjkUnifiedIdeograph5d84',
u'\u1286': 'EthiopicSyllableXo',
u'\ud788': 'HangulSyllableHi',
u'\u6b0b': 'CjkUnifiedIdeograph6b0b',
u'\u948a': 'CjkUnifiedIdeograph948a',
u'\u280d': 'BraillePatternDots134',
u'\ued0f': 'UnicodeCharacter0xed0f',
u'\ua211': 'YiSyllableKup',
u'\u4594': 'CjkUnifiedIdeograph4594',
u'\u3a96': 'CjkUnifiedIdeograph3a96',
u'\uff98': 'HalfwidthKatakanaLetterRi',
u'\ubc9a': 'HangulSyllableBeoj',
u'\u501d': 'CjkUnifiedIdeograph501d',
u'\u151f': 'CanadianSyllabicsShwoo',
u'\uca21': 'HangulSyllableJjyalt',
u'\u8f23': 'CjkUnifiedIdeograph8f23',
u'\u6da4': 'CjkUnifiedIdeograph6da4',
u'\u22a6': 'Assertion',
u'\ue7a8': 'UnicodeCharacter0xe7a8',
u'\ua4aa': 'YiRadicalJjut',
u'\u782d': 'CjkUnifiedIdeograph782d',
u'\u99ac': 'CjkUnifiedIdeograph99ac',
u'\u3d2f': 'CjkUnifiedIdeograph3d2f',
u'\uf231': 'UnicodeCharacter0xf231',
u'\ub733': 'HangulSyllableDdeulb',
u'\u4ab6': 'CjkUnifiedIdeograph4ab6',
u'\u0fb8': 'TibetanSubjoinedLetterA',
u'\uccba': 'HangulSyllableCenh',
u'\u603d': 'CjkUnifiedIdeograph603d',
u'\u81bc': 'CjkUnifiedIdeograph81bc',
u'\u253f': 'BoxDrawingsVerticalLightAndHorizontalHeavy',
u'\u1a41': 'UnicodeCharacter0x1a41',
u'\udf43': 'UnicodeCharacter0xdf43',
u'\u9c45': 'CjkUnifiedIdeograph9c45',
u'\u72c6': 'CjkUnifiedIdeograph72c6',
u'\u37c8': 'CjkUnifiedIdeograph37c8',
u'\uf4ca': 'UnicodeCharacter0xf4ca',
u'\ua9cc': 'UnicodeCharacter0xa9cc',
u'\u4d4f': 'CjkUnifiedIdeograph4d4f',
u'\u0251': 'LatinSmallLetterAlpha',
u'\uc753': 'HangulSyllableEuc',
u'\u8455': 'CjkUnifiedIdeograph8455',
u'\u5fd8': 'CjkUnifiedIdeograph5fd8',
u'\u1cda': 'UnicodeCharacter0x1cda',
u'\ud1dc': 'HangulSyllableTwaen',
u'\u755f': 'CjkUnifiedIdeograph755f',
u'\u96de': 'CjkUnifiedIdeograph96de',
u'\u2a61': 'SmallVeeWithUnderbar',
u'\uef63': 'UnicodeCharacter0xef63',
u'\uac65': 'HangulSyllableGyaeb',
u'\u47e8': 'CjkUnifiedIdeograph47e8',
u'\u04ea': 'CyrillicCapitalLetterBarredOWithDiaeresis',
u'\uf9ec': 'CjkCompatibilityIdeographf9ec',
u'\ubeee': 'HangulSyllableBbegg',
u'\u5271': 'CjkUnifiedIdeograph5271',
u'\u1773': 'TagbanwaVowelSignU',
u'\ud475': 'HangulSyllablePyot',
u'\u8977': 'CjkUnifiedIdeograph8977',
u'\u6ff8': 'CjkUnifiedIdeograph6ff8',
u'\u2cfa': 'CopticOldNubianDirectQuestionMark',
u'\ue1fc': 'UnicodeCharacter0xe1fc',
u'\ua6fe': 'UnicodeCharacter0xa6fe',
u'\u9c00': 'CjkUnifiedIdeograph9c00',
u'\ub883': 'HangulSyllableRwalb',
u'\u5f02': 'CjkUnifiedIdeograph5f02',
u'\u7b85': 'CjkUnifiedIdeograph7b85',
u'\ud506': 'HangulSyllablePeugg',
u'\uf189': 'UnicodeCharacter0xf189',
u'\u0e0c': 'ThaiCharacterChoChoe',
u'\u2a8f': 'LessthanAboveSimilarAboveGreaterthan',
u'\u8410': 'CjkUnifiedIdeograph8410',
u'\ua093': 'YiSyllableHmi',
u'\u4712': 'CjkUnifiedIdeograph4712',
u'\u6395': 'CjkUnifiedIdeograph6395',
u'\ufd16': 'ArabicLigatureGhainWithYehFinalForm',
u'\u1999': 'NewTaiLueLetterLowMa',
u'\u979d': 'CjkUnifiedIdeograph979d',
u'\u361c': 'CjkUnifiedIdeograph361c',
u'\u529f': 'CjkUnifiedIdeograph529f',
u'\uac20': 'HangulSyllableGaen',
u'\uc8a3': 'HangulSyllableJwac',
u'\u6f22': 'CjkUnifiedIdeograph6f22',
u'\ue526': 'UnicodeCharacter0xe526',
u'\u01a9': 'LatinCapitalLetterEsh',
u'\ubfad': 'HangulSyllableBboet',
u'\u5e2c': 'CjkUnifiedIdeograph5e2c',
u'\u7aaf': 'CjkUnifiedIdeograph7aaf',
u'\ud430': 'HangulSyllablePwaels',
u'\uf0b3': 'UnicodeCharacter0xf0b3',
u'\u0d36': 'MalayalamLetterSha',
u'\u29b9': 'CircledPerpendicular',
u'\u8b3a': 'CjkUnifiedIdeograph8b3a',
u'\ua7bd': 'UnicodeCharacter0xa7bd',
u'\u463c': 'CjkUnifiedIdeograph463c',
u'\u62bf': 'CjkUnifiedIdeograph62bf',
u'\ufc40': 'ArabicLigatureLamWithHahIsolatedForm',
u'\u18c3': 'UnicodeCharacter0x18c3',
u'\u96c7': 'CjkUnifiedIdeograph96c7',
u'\u3546': 'CjkUnifiedIdeograph3546',
u'\u51c9': 'CjkUnifiedIdeograph51c9',
u'\ub34a': 'HangulSyllableDyaebs',
u'\ucfcd': 'HangulSyllableKyolg',
u'\u6e4c': 'CjkUnifiedIdeograph6e4c',
u'\ue450': 'UnicodeCharacter0xe450',
u'\xd3': 'LatinCapitalLetterOWithAcute',
u'\ubed7': 'HangulSyllableBbeod',
u'\u5d56': 'CjkUnifiedIdeograph5d56',
u'\u79d9': 'CjkUnifiedIdeograph79d9',
u'\udb5a': 'UnicodeCharacter0xdb5a',
u'\uf7dd': 'UnicodeCharacter0xf7dd',
u'\u0c60': 'TeluguLetterVocalicRr',
u'\u28e3': 'BraillePatternDots12678',
u'\u8a64': 'CjkUnifiedIdeograph8a64',
u'\ua6e7': 'UnicodeCharacter0xa6e7',
u'\u4566': 'CjkUnifiedIdeograph4566',
u'\u61e9': 'CjkUnifiedIdeograph61e9',
u'\uc36a': 'HangulSyllableSseogg',
u'\u1fed': 'GreekDialytikaAndVaria',
u'\u95f1': 'CjkUnifiedIdeograph95f1',
u'\u3470': 'CjkUnifiedIdeograph3470',
u'\u50f3': 'CjkUnifiedIdeograph50f3',
u'\ub274': 'HangulSyllableNyu',
u'\ucef7': 'HangulSyllableKeos',
u'\u6d76': 'CjkUnifiedIdeograph6d76',
u'\ueb7a': 'UnicodeCharacter0xeb7a',
u'\u07fd': 'UnicodeCharacter0x7fd',
u'\u3d01': 'CjkUnifiedIdeograph3d01',
u'\u5c80': 'CjkUnifiedIdeograph5c80',
u'\ubb05': 'HangulSyllableMoelg',
u'\uda84': 'UnicodeCharacter0xda84',
u'\u7607': 'CjkUnifiedIdeograph7607',
u'\uec0b': 'UnicodeCharacter0xec0b',
u'\u138a': 'EthiopicSyllableFwee',
u'\u898e': 'CjkUnifiedIdeograph898e',
u'\u2511': 'BoxDrawingsDownLightAndLeftHeavy',
u'\u4490': 'CjkUnifiedIdeograph4490',
u'\ua315': 'YiSyllableSuo',
u'\uc294': 'HangulSyllableSyuls',
u'\u141b': 'CanadianSyllabicsNaskapiWaa',
u'\u3b9a': 'CjkUnifiedIdeograph3b9a',
u'\u921f': 'CjkUnifiedIdeograph921f',
u'\ub19e': 'HangulSyllableNwalm',
u'\u4d21': 'CjkUnifiedIdeograph4d21',
u'\u6ca0': 'CjkUnifiedIdeograph6ca0',
u'\ucb25': 'HangulSyllableJjoeng',
u'\ueaa4': 'UnicodeCharacter0xeaa4',
u'\u3c2b': 'CjkUnifiedIdeograph3c2b',
u'\u23aa': 'CurlyBracketExtension',
u'\uba2f': 'HangulSyllableMyaes',
u'\ud9ae': 'UnicodeCharacter0xd9ae',
u'\u7531': 'CjkUnifiedIdeograph7531',
u'\uf335': 'UnicodeCharacter0xf335',
u'\u12b4': 'EthiopicSyllableKwee',
u'\u88b8': 'CjkUnifiedIdeograph88b8',
u'\u243b': 'UnicodeCharacter0x243b',
u'\u4bba': 'CjkUnifiedIdeograph4bba',
u'\ua23f': 'YiSyllableMgep',
u'\uc1be': 'HangulSyllableSwaj',
u'\u1b45': 'BalineseLetterKafSasak',
u'\u3ac4': 'CjkUnifiedIdeograph3ac4',
u'\u9149': 'CjkUnifiedIdeograph9149',
u'\ub0c8': 'HangulSyllableNaess',
u'\u4c4b': 'CjkUnifiedIdeograph4c4b',
u'\u73ca': 'CjkUnifiedIdeograph73ca',
u'\uca4f': 'HangulSyllableJjeogs',
u'\ue9ce': 'UnicodeCharacter0xe9ce',
u'\u0355': 'CombiningRightArrowheadBelow',
u'\u22d4': 'Pitchfork',
u'\ub959': 'HangulSyllableRyug',
u'\ud8d8': 'UnicodeCharacter0xd8d8',
u'\u745b': 'CjkUnifiedIdeograph745b',
u'\uf25f': 'UnicodeCharacter0xf25f',
u'\u11de': 'HangulJongseongMieumssangsios',
u'\u8fe2': 'CjkUnifiedIdeograph8fe2',
u'\u2b65': 'UnicodeCharacter0x2b65',
u'\u4ae4': 'CjkUnifiedIdeograph4ae4',
u'\ua169': 'YiSyllableHnix',
u'\uc0e8': 'HangulSyllableSyan',
u'\u1a6f': 'UnicodeCharacter0x1a6f',
u'\u39ee': 'CjkUnifiedIdeograph39ee',
u'\u9073': 'CjkUnifiedIdeograph9073',
u'\ub7f2': 'HangulSyllableReonh',
u'\u5375': 'CjkUnifiedIdeograph5375',
u'\u72f4': 'CjkUnifiedIdeograph72f4',
u'\uc979': 'HangulSyllableJyult',
u'\ue8f8': 'UnicodeCharacter0xe8f8',
u'\u027f': 'LatinSmallLetterReversedRWithFishhook',
u'\u21fe': 'RightwardsOpenheadedArrow',
u'\u5406': 'CjkUnifiedIdeograph5406',
u'\u7089': 'CjkUnifiedIdeograph7089',
u'\u1108': 'HangulChoseongSsangpieup',
u'\u2d8b': 'EthiopicSyllableZoa',
u'\ud20a': 'HangulSyllableToej',
u'\uee8d': 'UnicodeCharacter0xee8d',
u'\u8f0c': 'CjkUnifiedIdeograph8f0c',
u'\uab8f': 'UnicodeCharacter0xab8f',
u'\u7c16': 'CjkUnifiedIdeograph7c16',
u'\u3918': 'CjkUnifiedIdeograph3918',
u'\u559b': 'CjkUnifiedIdeograph559b',
u'\ufa1a': 'CjkCompatibilityIdeographfa1a',
u'\u169d': 'UnicodeCharacter0x169d',
u'\ub71c': 'HangulSyllableDdyum',
u'\ud39f': 'HangulSyllablePed',
u'\u8ca1': 'CjkUnifiedIdeograph8ca1',
u'\u6426': 'CjkUnifiedIdeograph6426',
u'\u2128': 'BlackletterCapitalZ',
u'\u7dab': 'CjkUnifiedIdeograph7dab',
u'\ue22a': 'UnicodeCharacter0xe22a',
u'\u3ead': 'CjkUnifiedIdeograph3ead',
u'\udf2c': 'UnicodeCharacter0xdf2c',
u'\ufbaf': 'ArabicLetterYehBarreeFinalForm',
u'\u982e': 'CjkUnifiedIdeograph982e',
u'\ub4b1': 'HangulSyllableDwilt',
u'\u4938': 'CjkUnifiedIdeograph4938',
u'\u65bb': 'CjkUnifiedIdeograph65bb',
u'\u0a3a': 'UnicodeCharacter0xa3a',
u'\u26bd': 'UnicodeCharacter0x26bd',
u'\uc73c': 'HangulSyllableEu',
u'\ue3bf': 'UnicodeCharacter0xe3bf',
u'\u803e': 'CjkUnifiedIdeograph803e',
u'\udcc1': 'UnicodeCharacter0xdcc1',
u'\u99c3': 'CjkUnifiedIdeograph99c3',
u'\u7148': 'CjkUnifiedIdeograph7148',
u'\u324a': 'UnicodeCharacter0x324a',
u'\u4ecd': 'CjkUnifiedIdeograph4ecd',
u'\uef4c': 'UnicodeCharacter0xef4c',
u'\u0bcf': 'UnicodeCharacter0xbcf',
u'\ua84e': 'PhagspaLetterBa',
u'\uc4d1': 'HangulSyllableSswit',
u'\u81d3': 'CjkUnifiedIdeograph81d3',
u'\u5a5a': 'CjkUnifiedIdeograph5a5a',
u'\u76dd': 'CjkUnifiedIdeograph76dd',
u'\u175c': 'UnicodeCharacter0x175c',
u'\u33df': 'SquareAOverM',
u'\ud05e': 'HangulSyllableKyulp',
u'\uece1': 'UnicodeCharacter0xece1',
u'\u8d60': 'CjkUnifiedIdeograph8d60',
u'\ua9e3': 'UnicodeCharacter0xa9e3',
u'\u426a': 'CjkUnifiedIdeograph426a',
u'\u3f6c': 'CjkUnifiedIdeograph3f6c',
u'\u5bef': 'CjkUnifiedIdeograph5bef',
u'\uf86e': 'UnicodeCharacter0xf86e',
u'\u14f1': 'CanadianSyllabicsSo',
u'\ub570': 'HangulSyllableDdyal',
u'\ud1f3': 'HangulSyllableTwaeh',
u'\u92f5': 'CjkUnifiedIdeograph92f5',
u'\u6a7a': 'CjkUnifiedIdeograph6a7a',
u'\u277c': 'DingbatNegativeCircledDigitSeven',
u'\u43ff': 'CjkUnifiedIdeograph43ff',
u'\ue07e': 'UnicodeCharacter0xe07e',
u'\udd80': 'UnicodeCharacter0xdd80',
u'\u7903': 'CjkUnifiedIdeograph7903',
u'\u9e82': 'CjkUnifiedIdeograph9e82',
u'\u3a05': 'CjkUnifiedIdeograph3a05',
u'\uf707': 'UnicodeCharacter0xf707',
u'\ub009': 'HangulSyllableGgwet',
u'\u4f8c': 'CjkUnifiedIdeograph4f8c',
u'\u088e': 'UnicodeCharacter0x88e',
u'\uc590': 'HangulSyllableYass',
u'\u6113': 'CjkUnifiedIdeograph6113',
u'\u8692': 'CjkUnifiedIdeograph8692',
u'\u2215': 'DivisionSlash',
u'\u1f17': 'UnicodeCharacter0x1f17',
u'\ud819': 'UnicodeCharacter0xd819',
u'\u951b': 'CjkUnifiedIdeograph951b',
u'\u779c': 'CjkUnifiedIdeograph779c',
u'\u309e': 'HiraganaVoicedIterationMark',
u'\ueda0': 'UnicodeCharacter0xeda0',
u'\uaea2': 'HangulSyllableGgyaegg',
u'\u4a25': 'CjkUnifiedIdeograph4a25',
u'\u0727': 'SyriacLetterReversedPe',
u'\uc029': 'HangulSyllableBbwilg',
u'\ubd2b': 'HangulSyllableBwah',
u'\u58ae': 'CjkUnifiedIdeograph58ae',
u'\u15b0': 'CanadianSyllabicsBlackfootE',
u'\ud6b2': 'HangulSyllableHyolm',
u'\u7235': 'CjkUnifiedIdeograph7235',
u'\u93b4': 'CjkUnifiedIdeograph93b4',
u'\u2f37': 'KangxiRadicalShoot',
u'\ue839': 'UnicodeCharacter0xe839',
u'\ua53b': 'VaiSyllableSi',
u'\u40be': 'CjkUnifiedIdeograph40be',
u'\u3dc0': 'CjkUnifiedIdeograph3dc0',
u'\ufec2': 'ArabicLetterTahFinalForm',
u'\ubbc4': 'HangulSyllableMeun',
u'\u5747': 'CjkUnifiedIdeograph5747',
u'\u1049': 'MyanmarDigitNine',
u'\ucd4b': 'HangulSyllableCwaelb',
u'\u8e4d': 'CjkUnifiedIdeograph8e4d',
u'\u68ce': 'CjkUnifiedIdeograph68ce',
u'\u25d0': 'CircleWithLeftHalfBlack',
u'\ue6d2': 'UnicodeCharacter0xe6d2',
u'\ua3d4': 'YiSyllableRut',
u'\u7f57': 'CjkUnifiedIdeograph7f57',
u'\u9cd6': 'CjkUnifiedIdeograph9cd6',
u'\u3859': 'CjkUnifiedIdeograph3859',
u'\uf55b': 'UnicodeCharacter0xf55b',
u'\ub65d': 'HangulSyllableDdwaeng',
u'\u4de0': 'HexagramForRetreat',
u'\u0ee2': 'UnicodeCharacter0xee2',
u'\ucbe4': 'HangulSyllableJjeum',
u'\u6767': 'CjkUnifiedIdeograph6767',
u'\u84e6': 'CjkUnifiedIdeograph84e6',
u'\u2069': 'UnicodeCharacter0x2069',
u'\u1d6b': 'LatinSmallLetterUe',
u'\ude6d': 'UnicodeCharacter0xde6d',
u'\u9b6f': 'CjkUnifiedIdeograph9b6f',
u'\u75f0': 'CjkUnifiedIdeograph75f0',
u'\u36f2': 'CjkUnifiedIdeograph36f2',
u'\uf3f4': 'UnicodeCharacter0xf3f4',
u'\uacf6': 'HangulSyllableGoj',
u'\u4879': 'CjkUnifiedIdeograph4879',
u'\u057b': 'ArmenianSmallLetterJheh',
u'\uc67d': 'HangulSyllableOenj',
u'\u837f': 'CjkUnifiedIdeograph837f',
u'\uff81': 'HalfwidthKatakanaLetterTi',
u'\u1804': 'MongolianColon',
u'\u3487': 'CjkUnifiedIdeograph3487',
u'\u9208': 'CjkUnifiedIdeograph9208',
u'\uae8b': 'HangulSyllableGgyad',
u'\u510a': 'CjkUnifiedIdeograph510a',
u'\u6d8d': 'CjkUnifiedIdeograph6d8d',
u'\ucb0e': 'HangulSyllableJjwaep',
u'\ue791': 'UnicodeCharacter0xe791',
u'\u5c97': 'CjkUnifiedIdeograph5c97',
u'\uba18': 'HangulSyllableMyak',
u'\ud69b': 'HangulSyllableHoelh',
u'\u791a': 'CjkUnifiedIdeograph791a',
u'\uf31e': 'UnicodeCharacter0xf31e',
u'\u0fa1': 'TibetanSubjoinedLetterDa',
u'\u89a5': 'CjkUnifiedIdeograph89a5',
u'\u2824': 'BraillePatternDots36',
u'\u44a7': 'CjkUnifiedIdeograph44a7',
u'\ua228': 'YiSyllableGge',
u'\ufeab': 'ArabicLetterThalIsolatedForm',
u'\u612a': 'CjkUnifiedIdeograph612a',
u'\u1b2e': 'BalineseLetterLa',
u'\u37b1': 'CjkUnifiedIdeograph37b1',
u'\u9532': 'CjkUnifiedIdeograph9532',
u'\ub1b5': 'HangulSyllableNwaenj',
u'\u5034': 'CjkUnifiedIdeograph5034',
u'\u6cb7': 'CjkUnifiedIdeograph6cb7',
u'\uca38': 'HangulSyllableJjyael',
u'\ue6bb': 'UnicodeCharacter0xe6bb',
u'\u033e': 'CombiningVerticalTilde',
u'\u5fc1': 'CjkUnifiedIdeograph5fc1',
u'\ubd42': 'HangulSyllableBwaej',
u'\ud9c5': 'UnicodeCharacter0xd9c5',
u'\u7844': 'CjkUnifiedIdeograph7844',
u'\uf248': 'UnicodeCharacter0xf248',
u'\u0ecb': 'LaoToneMaiCatawa',
u'\u88cf': 'CjkUnifiedIdeograph88cf',
u'\u2b4e': 'UnicodeCharacter0x2b4e',
u'\u47d1': 'CjkUnifiedIdeograph47d1',
u'\ua552': 'VaiSyllableBa',
u'\uc1d5': 'HangulSyllableSwaeb',
u'\u6054': 'CjkUnifiedIdeograph6054',
u'\u1a58': 'UnicodeCharacter0x1a58',
u'\u36db': 'CjkUnifiedIdeograph36db',
u'\u945c': 'CjkUnifiedIdeograph945c',
u'\ub0df': 'HangulSyllableNyalh',
u'\u535e': 'CjkUnifiedIdeograph535e',
u'\u6fe1': 'CjkUnifiedIdeograph6fe1',
u'\ucd62': 'HangulSyllableCoenh',
u'\ue9e5': 'UnicodeCharacter0xe9e5',
u'\u0268': 'LatinSmallLetterIWithStroke',
u'\u5eeb': 'CjkUnifiedIdeograph5eeb',
u'\ubc6c': 'HangulSyllableByaen',
u'\ud8ef': 'UnicodeCharacter0xd8ef',
u'\u7b6e': 'CjkUnifiedIdeograph7b6e',
u'\uf572': 'UnicodeCharacter0xf572',
u'\u11f5': 'HangulJongseongHieuhnieun',
u'\u8bf9': 'CjkUnifiedIdeograph8bf9',
u'\u2a78': 'EquivalentWithFourDotsAbove',
u'\u46fb': 'CjkUnifiedIdeograph46fb',
u'\ua47c': 'YiSyllableYuop',
u'\uc0ff': 'HangulSyllableSyah',
u'\u637e': 'CjkUnifiedIdeograph637e',
u'\ufa03': 'CjkCompatibilityIdeographfa03',
u'\u1d82': 'LatinSmallLetterFWithPalatalHook',
u'\u9786': 'CjkUnifiedIdeograph9786',
u'\u3309': 'SquareOnsu',
u'\u5288': 'CjkUnifiedIdeograph5288',
u'\uad0d': 'HangulSyllableGwab',
u'\ucc8c': 'HangulSyllableCyaem',
u'\u680f': 'CjkUnifiedIdeograph680f',
u'\ue213': 'UnicodeCharacter0xe213',
u'\u0592': 'HebrewAccentSegol',
u'\u9c17': 'CjkUnifiedIdeograph9c17',
u'\ubf96': 'HangulSyllableBboegg',
u'\u5b19': 'CjkUnifiedIdeograph5b19',
u'\u7a98': 'CjkUnifiedIdeograph7a98',
u'\ud51d': 'HangulSyllablePeut',
u'\uf49c': 'UnicodeCharacter0xf49c',
u'\u0a23': 'GurmukhiLetterNna',
u'\u2da2': 'EthiopicSyllableSsi',
u'\u8427': 'CjkUnifiedIdeograph8427',
u'\ua7a6': 'UnicodeCharacter0xa7a6',
u'\u4329': 'CjkUnifiedIdeograph4329',
u'\u62a8': 'CjkUnifiedIdeograph62a8',
u'\ufd2d': 'ArabicLigatureSheenWithJeemInitialForm',
u'\u1cac': 'UnicodeCharacter0x1cac',
u'\u96b0': 'CjkUnifiedIdeograph96b0',
u'\u3233': 'ParenthesizedIdeographSociety',
u'\u55b2': 'CjkUnifiedIdeograph55b2',
u'\uac37': 'HangulSyllableGaeh',
u'\ucfb6': 'HangulSyllableKoelp',
u'\u6b39': 'CjkUnifiedIdeograph6b39',
u'\ue53d': 'UnicodeCharacter0xe53d',
u'\u04bc': 'CyrillicCapitalLetterAbkhasianChe',
u'\u9f41': 'CjkUnifiedIdeograph9f41',
u'\ubec0': 'HangulSyllableBbyaels',
u'\u5a43': 'CjkUnifiedIdeograph5a43',
u'\u7dc2': 'CjkUnifiedIdeograph7dc2',
u'\ud447': 'HangulSyllablePoed',
u'\uf7c6': 'UnicodeCharacter0xf7c6',
u'\u0d4d': 'MalayalamSignVirama',
u'\u2ccc': 'CopticCapitalLetterOldCopticHori',
u'\u8751': 'CjkUnifiedIdeograph8751',
u'\ua6d0': 'UnicodeCharacter0xa6d0',
u'\u4253': 'CjkUnifiedIdeograph4253',
u'\u65d2': 'CjkUnifiedIdeograph65d2',
u'\ufc57': 'ArabicLigatureYehWithKhahIsolatedForm',
u'\u1fd6': 'GreekSmallLetterIotaWithPerispomeni',
u'\u99da': 'CjkUnifiedIdeograph99da',
u'\u355d': 'CjkUnifiedIdeograph355d',
u'\u54dc': 'CjkUnifiedIdeograph54dc',
u'\uaf61': 'HangulSyllableGgwat',
u'\ucee0': 'HangulSyllableKyaek',
u'\u6a63': 'CjkUnifiedIdeograph6a63',
u'\ue467': 'UnicodeCharacter0xe467',
u'\u07e6': 'NkoLetterYa',
u'\u9e6b': 'CjkUnifiedIdeograph9e6b',
u'\u81ea': 'CjkUnifiedIdeograph81ea',
u'\u5d6d': 'CjkUnifiedIdeograph5d6d',
u'\u7cec': 'CjkUnifiedIdeograph7cec',
u'\ud771': 'HangulSyllableHyinj',
u'\uf6f0': 'UnicodeCharacter0xf6f0',
u'\u0c77': 'UnicodeCharacter0xc77',
u'\u2ff6': 'IdeographicDescriptionCharacterSurroundFromBelow',
u'\u867b': 'CjkUnifiedIdeograph867b',
u'\ua9fa': 'UnicodeCharacter0xa9fa',
u'\u457d': 'CjkUnifiedIdeograph457d',
u'\u64fc': 'CjkUnifiedIdeograph64fc',
u'\u7e81': 'CjkUnifiedIdeograph7e81',
u'\u1f00': 'GreekSmallLetterAlphaWithPsili',
u'\u3b83': 'CjkUnifiedIdeograph3b83',
u'\udc02': 'UnicodeCharacter0xdc02',
u'\uf885': 'UnicodeCharacter0xf885',
u'\u9904': 'CjkUnifiedIdeograph9904',
u'\ub587': 'HangulSyllableDdyaegs',
u'\u4a0e': 'CjkUnifiedIdeograph4a0e',
u'\u6691': 'CjkUnifiedIdeograph6691',
u'\u0710': 'SyriacLetterAlaph',
u'\u2393': 'DirectCurrentSymbolFormTwo',
u'\uc412': 'HangulSyllableSswaegg',
u'\ue095': 'UnicodeCharacter0xe095',
u'\u8114': 'CjkUnifiedIdeograph8114',
u'\udd97': 'UnicodeCharacter0xdd97',
u'\u9a99': 'CjkUnifiedIdeograph9a99',
u'\u721e': 'CjkUnifiedIdeograph721e',
u'\u2f20': 'KangxiRadicalScholar',
u'\u4ba3': 'CjkUnifiedIdeograph4ba3',
u'\uec22': 'UnicodeCharacter0xec22',
u'\u08a5': 'UnicodeCharacter0x8a5',
u'\ua924': 'KayahLiLetterI',
u'\uc5a7': 'HangulSyllableYaelh',
u'\u82a9': 'CjkUnifiedIdeograph82a9',
u'\u5730': 'CjkUnifiedIdeograph5730',
u'\u73b3': 'CjkUnifiedIdeograph73b3',
u'\u1432': 'CanadianSyllabicsPii',
u'\u30b5': 'KatakanaLetterSa',
u'\ud134': 'HangulSyllableTeon',
u'\uedb7': 'UnicodeCharacter0xedb7',
u'\u8e36': 'CjkUnifiedIdeograph8e36',
u'\uaab9': 'UnicodeCharacter0xaab9',
u'\u7f40': 'CjkUnifiedIdeograph7f40',
u'\u3c42': 'CjkUnifiedIdeograph3c42',
u'\u58c5': 'CjkUnifiedIdeograph58c5',
u'\uf944': 'CjkCompatibilityIdeographf944',
u'\u15c7': 'CanadianSyllabicsCarrierGhee',
u'\ub646': 'HangulSyllableDdwap',
u'\ud2c9': 'HangulSyllableTeub',
u'\u8fcb': 'CjkUnifiedIdeograph8fcb',
u'\u6750': 'CjkUnifiedIdeograph6750',
u'\u2452': 'UnicodeCharacter0x2452',
u'\u40d5': 'CjkUnifiedIdeograph40d5',
u'\ue154': 'UnicodeCharacter0xe154',
u'\u3dd7': 'CjkUnifiedIdeograph3dd7',
u'\ude56': 'UnicodeCharacter0xde56',
u'\ufad9': 'CjkCompatibilityIdeographfad9',
u'\u9b58': 'CjkUnifiedIdeograph9b58',
u'\ub7db': 'HangulSyllableRyaelb',
u'\u4c62': 'CjkUnifiedIdeograph4c62',
u'\u68e5': 'CjkUnifiedIdeograph68e5',
u'\u0964': 'DevanagariDanda',
u'\u25e7': 'SquareWithLeftHalfBlack',
u'\uc666': 'HangulSyllableWaelm',
u'\ue2e9': 'UnicodeCharacter0xe2e9',
u'\u8368': 'CjkUnifiedIdeograph8368',
u'\udfeb': 'UnicodeCharacter0xdfeb',
u'\u9ced': 'CjkUnifiedIdeograph9ced',
u'\u7472': 'CjkUnifiedIdeograph7472',
u'\u3174': 'HangulLetterPieupsioskiyeok',
u'\u4df7': 'HexagramForTheWanderer',
u'\uee76': 'UnicodeCharacter0xee76',
u'\u0af9': 'UnicodeCharacter0xaf9',
u'\uab78': 'UnicodeCharacter0xab78',
u'\uc7fb': 'HangulSyllableJyaec',
u'\u84fd': 'CjkUnifiedIdeograph84fd',
u'\ube01': 'HangulSyllableByub',
u'\u5984': 'CjkUnifiedIdeograph5984',
u'\u1686': 'OghamLetterUath',
u'\ud388': 'HangulSyllablePeols',
u'\u6f0b': 'CjkUnifiedIdeograph6f0b',
u'\u908a': 'CjkUnifiedIdeograph908a',
u'\u2c0d': 'GlagoliticCapitalLetterKako',
u'\ue90f': 'UnicodeCharacter0xe90f',
u'\ua611': 'VaiSyllableNdoleKa',
u'\u4194': 'CjkUnifiedIdeograph4194',
u'\u3e96': 'CjkUnifiedIdeograph3e96',
u'\ufb98': 'ArabicLetterGuehInitialForm',
u'\ub89a': 'HangulSyllableRwaenh',
u'\u541d': 'CjkUnifiedIdeograph541d',
u'\u111f': 'HangulChoseongPieupnieun',
u'\uce21': 'HangulSyllableCeug',
u'\u8b23': 'CjkUnifiedIdeograph8b23',
u'\u69a4': 'CjkUnifiedIdeograph69a4',
u'\u26a6': 'MaleWithStrokeSign',
u'\ue3a8': 'UnicodeCharacter0xe3a8',
u'\ua0aa': 'YiSyllableHmy',
u'\u7c2d': 'CjkUnifiedIdeograph7c2d',
u'\u9dac': 'CjkUnifiedIdeograph9dac',
u'\u392f': 'CjkUnifiedIdeograph392f',
u'\uf631': 'UnicodeCharacter0xf631',
u'\ub333': 'HangulSyllableDyac',
u'\u4eb6': 'CjkUnifiedIdeograph4eb6',
u'\u0bb8': 'TamilLetterSa',
u'\uc8ba': 'HangulSyllableJwaebs',
u'\u643d': 'CjkUnifiedIdeograph643d',
u'\u85bc': 'CjkUnifiedIdeograph85bc',
u'\u213f': 'DoublestruckCapitalPi',
u'\u1e41': 'LatinSmallLetterMWithDotAbove',
u'\udb43': 'UnicodeCharacter0xdb43',
u'\u9845': 'CjkUnifiedIdeograph9845',
u'\u76c6': 'CjkUnifiedIdeograph76c6',
u'\u33c8': 'SquareDb',
u'\uf0ca': 'UnicodeCharacter0xf0ca',
u'\uadcc': 'HangulSyllableGwils',
u'\u494f': 'CjkUnifiedIdeograph494f',
u'\u0651': 'ArabicShadda',
u'\uc353': 'HangulSyllableSsyaed',
u'\u8055': 'CjkUnifiedIdeograph8055',
u'\u5bd8': 'CjkUnifiedIdeograph5bd8',
u'\u18da': 'UnicodeCharacter0x18da',
u'\ud5dc': 'HangulSyllableHeoss',
u'\u715f': 'CjkUnifiedIdeograph715f',
u'\u92de': 'CjkUnifiedIdeograph92de',
u'\u2e61': 'UnicodeCharacter0x2e61',
u'\ueb63': 'UnicodeCharacter0xeb63',
u'\ua865': 'PhagspaLetterGga',
u'\u43e8': 'CjkUnifiedIdeograph43e8',
u'\xea': 'LatinSmallLetterEWithCircumflex',
u'\ufdec': 'UnicodeCharacter0xfdec',
u'\ubaee': 'HangulSyllableMwaelp',
u'\u5671': 'CjkUnifiedIdeograph5671',
u'\u1373': 'EthiopicNumberTwenty',
u'\ud075': 'HangulSyllableKeulg',
u'\u8d77': 'CjkUnifiedIdeograph8d77',
u'\u6bf8': 'CjkUnifiedIdeograph6bf8',
u'\u28fa': 'BraillePatternDots245678',
u'\ue5fc': 'UnicodeCharacter0xe5fc',
u'\ua2fe': 'YiSyllableNzux',
u'\ua000': 'YiSyllableIt',
u'\u8483': 'CjkUnifiedIdeograph8483',
u'\u6302': 'CjkUnifiedIdeograph6302',
u'\u4785': 'CjkUnifiedIdeograph4785',
u'\ue906': 'UnicodeCharacter0xe906',
u'\ucd89': 'HangulSyllableCyob',
u'\u320c': 'ParenthesizedHangulPhieuph',
u'\u168f': 'OghamLetterRuis',
u'\ub810': 'HangulSyllableRel',
u'\u9c93': 'CjkUnifiedIdeograph9c93',
u'\u7b12': 'CjkUnifiedIdeograph7b12',
u'\u5f95': 'CjkUnifiedIdeograph5f95',
u'\uc116': 'HangulSyllableSyaej',
u'\u2599': 'QuadrantUpperLeftAndLowerLeftAndLowerRight',
u'\uab9d': 'UnicodeCharacter0xab9d',
u'\u0a1c': 'GurmukhiLetterJa',
u'\u6e9f': 'CjkUnifiedIdeograph6e9f',
u'\u9020': 'CjkUnifiedIdeograph9020',
u'\uf4a3': 'UnicodeCharacter0xf4a3',
u'\u5322': 'CjkUnifiedIdeograph5322',
u'\ud926': 'UnicodeCharacter0xd926',
u'\u3da9': 'CjkUnifiedIdeograph3da9',
u'\u83ad': 'CjkUnifiedIdeograph83ad',
u'\u622c': 'CjkUnifiedIdeograph622c',
u'\u46af': 'CjkUnifiedIdeograph46af',
u'\ue830': 'UnicodeCharacter0xe830',
u'\uccb3': 'HangulSyllableCeoh',
u'\u3136': 'HangulLetterNieunhieuh',
u'\u15b9': 'CanadianSyllabicsBlackfootNi',
u'\ub73a': 'HangulSyllableDdeubs',
u'\u9bbd': 'CjkUnifiedIdeograph9bbd',
u'\u7a3c': 'CjkUnifiedIdeograph7a3c',
u'\u5ebf': 'CjkUnifiedIdeograph5ebf',
u'\uc040': 'HangulSyllableBbyun',
u'\u24c3': 'CircledLatinCapitalLetterN',
u'\uaac7': 'UnicodeCharacter0xaac7',
u'\u0946': 'DevanagariVowelSignShortE',
u'\u6dc9': 'CjkUnifiedIdeograph6dc9',
u'\u8f4a': 'CjkUnifiedIdeograph8f4a',
u'\uf3cd': 'UnicodeCharacter0xf3cd',
u'\u524c': 'CjkUnifiedIdeograph524c',
u'\ud850': 'UnicodeCharacter0xd850',
u'\u3cd3': 'CjkUnifiedIdeograph3cd3',
u'\u82d7': 'CjkUnifiedIdeograph82d7',
u'\u6156': 'CjkUnifiedIdeograph6156',
u'\u45d9': 'CjkUnifiedIdeograph45d9',
u'\ue75a': 'UnicodeCharacter0xe75a',
u'\ucbdd': 'HangulSyllableJjeulg',
u'\u3060': 'HiraganaLetterDa',
u'\u14e3': 'CanadianSyllabicsWestcreeLwo',
u'\ub664': 'HangulSyllableDdoe',
u'\u9ae7': 'CjkUnifiedIdeograph9ae7',
u'\u7966': 'CjkUnifiedIdeograph7966',
u'\u5de9': 'CjkUnifiedIdeograph5de9',
u'\uff6a': 'HalfwidthKatakanaLetterSmallE',
u'\u23ed': 'UnicodeCharacter0x23ed',
u'\ua9f1': 'UnicodeCharacter0xa9f1',
u'\u0870': 'UnicodeCharacter0x870',
u'\u6cf3': 'CjkUnifiedIdeograph6cf3',
u'\u8e74': 'CjkUnifiedIdeograph8e74',
u'\uf2f7': 'UnicodeCharacter0xf2f7',
u'\u5176': 'CjkUnifiedIdeograph5176',
u'\ud77a': 'HangulSyllableHyilp',
u'\u3bfd': 'CjkUnifiedIdeograph3bfd',
u'\u0101': 'LatinSmallLetterAWithMacron',
u'\u6080': 'CjkUnifiedIdeograph6080',
u'\u8705': 'CjkUnifiedIdeograph8705',
u'\ue684': 'UnicodeCharacter0xe684',
u'\u4a07': 'CjkUnifiedIdeograph4a07',
u'\ud00b': 'HangulSyllableKweolh',
u'\u2f8a': 'KangxiRadicalColor',
u'\ub58e': 'HangulSyllableDdyaelm',
u'\u1911': 'LimbuLetterPha',
u'\u7890': 'CjkUnifiedIdeograph7890',
u'\u9f15': 'CjkUnifiedIdeograph9f15',
u'\ufe94': 'ArabicLetterTehMarbutaFinalForm',
u'\u281b': 'BraillePatternDots1245',
u'\u079a': 'ThaanaLetterKhaa',
u'\uae1f': 'HangulSyllableGyilb',
u'\u8d9e': 'CjkUnifiedIdeograph8d9e',
u'\u7121': 'CjkUnifiedIdeograph7121',
u'\u50a0': 'CjkUnifiedIdeograph50a0',
u'\uf725': 'UnicodeCharacter0xf725',
u'\ud6a4': 'HangulSyllableHoek',
u'\u1faa': 'GreekCapitalLetterOmegaWithPsiliAndVariaAndProsgegrammeni',
u'\u862f': 'CjkUnifiedIdeograph862f',
u'\ue5ae': 'UnicodeCharacter0xe5ae',
u'\u4931': 'CjkUnifiedIdeograph4931',
u'\ucf35': 'HangulSyllableKyeot',
u'\u2eb4': 'CjkRadicalNetFour',
u'\ub4b8': 'HangulSyllableDwiss',
u'\u183b': 'MongolianLetterKha',
u'\u77ba': 'CjkUnifiedIdeograph77ba',
u'\u9e3f': 'CjkUnifiedIdeograph9e3f',
u'\ufdbe': 'ArabicLigatureJeemWithHahWithYehFinalForm',
u'\u2745': 'TightTrifoliateSnowflake',
u'\u06c4': 'ArabicLetterWawWithRing',
u'\uad49': 'HangulSyllableGoeng',
u'\u8cc8': 'CjkUnifiedIdeograph8cc8',
u'\u704b': 'CjkUnifiedIdeograph704b',
u'\u4fca': 'CjkUnifiedIdeograph4fca',
u'\uf64f': 'UnicodeCharacter0xf64f',
u'\ud5ce': 'HangulSyllableHeonh',
u'\u3f55': 'CjkUnifiedIdeograph3f55',
u'\u1ed4': 'LatinCapitalLetterOWithCircumflexAndHookAbove',
u'\u8559': 'CjkUnifiedIdeograph8559',
u'\ue4d8': 'UnicodeCharacter0xe4d8',
u'\u485b': 'CjkUnifiedIdeograph485b',
u'\uce5f': 'HangulSyllableCid',
u'\u2dde': 'EthiopicSyllableGyo',
u'\ub3e2': 'HangulSyllableDwagg',
u'\u1765': 'TagbanwaLetterNga',
u'\u76e4': 'CjkUnifiedIdeograph76e4',
u'\u9d69': 'CjkUnifiedIdeograph9d69',
u'\ufce8': 'ArabicLigatureSeenWithHehMedialForm',
u'\u266f': 'MusicSharpSign',
u'\u05ee': 'UnicodeCharacter0x5ee',
u'\uac73': 'HangulSyllableGeogs',
u'\u8bf2': 'CjkUnifiedIdeograph8bf2',
u'\u6f75': 'CjkUnifiedIdeograph6f75',
u'\u4ef4': 'CjkUnifiedIdeograph4ef4',
u'\uf579': 'UnicodeCharacter0xf579',
u'\ud4f8': 'HangulSyllablePyum',
u'\u3e7f': 'CjkUnifiedIdeograph3e7f',
u'\u1dfe': 'CombiningLeftArrowheadAbove',
u'\u6806': 'CjkUnifiedIdeograph6806',
u'\u4c89': 'CjkUnifiedIdeograph4c89',
u'\u2d08': 'GeorgianSmallLetterIn',
u'\u118b': 'HangulJungseongUeoeu',
u'\uee0a': 'UnicodeCharacter0xee0a',
u'\ud28d': 'HangulSyllableTwilt',
u'\ub30c': 'HangulSyllableDaels',
u'\u978f': 'CjkUnifiedIdeograph978f',
u'\u4016': 'CjkUnifiedIdeograph4016',
u'\u0518': 'CyrillicCapitalLetterYae',
u'\u699b': 'CjkUnifiedIdeograph699b',
u'\uc61a': 'HangulSyllableYebs',
u'\u2a9d': 'SimilarOrLessthan',
u'\u8b1c': 'CjkUnifiedIdeograph8b1c',
u'\uef9f': 'UnicodeCharacter0xef9f',
u'\ub0a1': 'HangulSyllableNalg',
u'\u5826': 'CjkUnifiedIdeograph5826',
u'\u1d28': 'GreekLetterSmallCapitalPi',
u'\u41ab': 'CjkUnifiedIdeograph41ab',
u'\ude2a': 'UnicodeCharacter0xde2a',
u'\u02ad': 'LatinLetterBidentalPercussive',
u'\ue32c': 'UnicodeCharacter0xe32c',
u'\uc7af': 'HangulSyllableJaegs',
u'\ua42e': 'YiSyllableJjyx',
u'\u88b1': 'CjkUnifiedIdeograph88b1',
u'\u7538': 'CjkUnifiedIdeograph7538',
u'\u59bb': 'CjkUnifiedIdeograph59bb',
u'\u363a': 'CjkUnifiedIdeograph363a',
u'\u1abd': 'UnicodeCharacter0x1abd',
u'\ufb3c': 'HebrewLetterLamedWithDagesh',
u'\udfbf': 'UnicodeCharacter0xdfbf',
u'\ubc3e': 'HangulSyllableBaelp',
u'\ue0c1': 'UnicodeCharacter0xe0c1',
u'\ua5c3': 'VaiSyllableBo',
u'\u4d48': 'CjkUnifiedIdeograph4d48',
u'\u0e4a': 'ThaiCharacterMaiTri',
u'\u72cd': 'CjkUnifiedIdeograph72cd',
u'\ud34c': 'HangulSyllablePyal',
u'\u37cf': 'CjkUnifiedIdeograph37cf',
u'\u944e': 'CjkUnifiedIdeograph944e',
u'\uf8d1': 'UnicodeCharacter0xf8d1',
u'\ubdd3': 'HangulSyllableBweh',
u'\u665a': 'CjkUnifiedIdeograph665a',
u'\u4add': 'CjkUnifiedIdeograph4add',
u'\u2b5c': 'UnicodeCharacter0x2b5c',
u'\u0fdf': 'UnicodeCharacter0xfdf',
u'\uec5e': 'UnicodeCharacter0xec5e',
u'\ud0e1': 'HangulSyllableTaenj',
u'\ub160': 'HangulSyllableNyen',
u'\u95e3': 'CjkUnifiedIdeograph95e3',
u'\u7e6a': 'CjkUnifiedIdeograph7e6a',
u'\u036c': 'CombiningLatinSmallLetterR',
u'\u67ef': 'CjkUnifiedIdeograph67ef',
u'\uc46e': 'HangulSyllableSsulm',
u'\u28f1': 'BraillePatternDots15678',
u'\u8970': 'CjkUnifiedIdeograph8970',
u'\uedf3': 'UnicodeCharacter0xedf3',
u'\uaef5': 'HangulSyllableGgyeog',
u'\u567a': 'CjkUnifiedIdeograph567a',
u'\u1b7c': 'BalineseMusicalSymbolLefthandOpenPing',
u'\u7fff': 'CjkUnifiedIdeograph7fff',
u'\udc7e': 'UnicodeCharacter0xdc7e',
u'\ue180': 'UnicodeCharacter0xe180',
u'\u4503': 'CjkUnifiedIdeograph4503',
u'\ua282': 'YiSyllableWa',
u'\u0605': 'UnicodeCharacter0x605',
u'\ucb07': 'HangulSyllableJjwaes',
u'\u8c09': 'CjkUnifiedIdeograph8c09',
u'\u738c': 'CjkUnifiedIdeograph738c',
u'\u348e': 'CjkUnifiedIdeograph348e',
u'\uf990': 'CjkCompatibilityIdeographf990',
u'\u5d13': 'CjkUnifiedIdeograph5d13',
u'\uba92': 'HangulSyllableMyenh',
u'\u1e15': 'LatinSmallLetterEWithMacronAndGrave',
u'\u2317': 'ViewdataSquare',
u'\ue419': 'UnicodeCharacter0xe419',
u'\ua91b': 'KayahLiLetterYa',
u'\u4b9c': 'CjkUnifiedIdeograph4b9c',
u'\u0c9e': 'KannadaLetterNya',
u'\ud1a0': 'HangulSyllableTo',
u'\u92a2': 'CjkUnifiedIdeograph92a2',
u'\u7625': 'CjkUnifiedIdeograph7625',
u'\u3b27': 'CjkUnifiedIdeograph3b27',
u'\ufc29': 'ArabicLigatureAinWithJeemIsolatedForm',
u'\u812b': 'CjkUnifiedIdeograph812b',
u'\u64ae': 'CjkUnifiedIdeograph64ae',
u'\u29b0': 'ReversedEmptySet',
u'\ueab2': 'UnicodeCharacter0xeab2',
u'\u4e35': 'CjkUnifiedIdeograph4e35',
u'\uafb4': 'HangulSyllableGgyok',
u'\u1337': 'EthiopicSyllablePhwa',
u'\ud439': 'HangulSyllablePwaeng',
u'\u993b': 'CjkUnifiedIdeograph993b',
u'\u7cbe': 'CjkUnifiedIdeograph7cbe',
u'\u01c0': 'LatinLetterDentalClick',
u'\uc2c2': 'HangulSyllableSyigg',
u'\u87c4': 'CjkUnifiedIdeograph87c4',
u'\u6b47': 'CjkUnifiedIdeograph6b47',
u'\u2c49': 'GlagoliticSmallLetterOtu',
u'\uf14b': 'UnicodeCharacter0xf14b',
u'\ub24d': 'HangulSyllableNweb',
u'\u54ce': 'CjkUnifiedIdeograph54ce',
u'\u19d0': 'NewTaiLueDigitZero',
u'\udad2': 'UnicodeCharacter0xdad2',
u'\u9fd4': 'UnicodeCharacter0x9fd4',
u'\u4357': 'CjkUnifiedIdeograph4357',
u'\ua0d6': 'YiSyllableFo',
u'\u0459': 'CyrillicSmallLetterLje',
u'\uc95b': 'HangulSyllableJwilb',
u'\u8a5d': 'CjkUnifiedIdeograph8a5d',
u'\u71e0': 'CjkUnifiedIdeograph71e0',
u'\u32e2': 'CircledKatakanaTe',
u'\uf7e4': 'UnicodeCharacter0xf7e4',
u'\u5b67': 'CjkUnifiedIdeograph5b67',
u'\ub8e6': 'HangulSyllableRyop',
u'\u1c69': 'OlChikiLetterLu',
u'\u216b': 'RomanNumeralTwelve',
u'\ue26d': 'UnicodeCharacter0xe26d',
u'\ua76f': 'LatinSmallLetterCon',
u'\u49f0': 'CjkUnifiedIdeograph49f0',
u'\u0af2': 'UnicodeCharacter0xaf2',
u'\ucff4': 'HangulSyllableKuss',
u'\u90f6': 'CjkUnifiedIdeograph90f6',
u'\u7479': 'CjkUnifiedIdeograph7479',
u'\u397b': 'CjkUnifiedIdeograph397b',
u'\ufa7d': 'CjkCompatibilityIdeographfa7d',
u'\ubf7f': 'HangulSyllableBbwaed',
u'\uc381': 'HangulSyllableSseot',
u'\u2404': 'SymbolForEndOfTransmission',
u'\u0887': 'UnicodeCharacter0x887',
u'\uae08': 'HangulSyllableGeum',
u'\u928b': 'CjkUnifiedIdeograph928b',
u'\u6d0a': 'CjkUnifiedIdeograph6d0a',
u'\u518d': 'CjkUnifiedIdeograph518d',
u'\uf70e': 'UnicodeCharacter0xf70e',
u'\udb91': 'UnicodeCharacter0xdb91',
u'\u3c14': 'CjkUnifiedIdeograph3c14',
u'\u6097': 'CjkUnifiedIdeograph6097',
u'\u8618': 'CjkUnifiedIdeograph8618',
u'\uea9b': 'UnicodeCharacter0xea9b',
u'\u451a': 'CjkUnifiedIdeograph451a',
u'\ucf1e': 'HangulSyllableKyeogg',
u'\u33a1': 'SquareMSquared',
u'\ub5a5': 'HangulSyllableDdeonj',
u'\u1424': 'CanadianSyllabicsFinalRing',
u'\u78a7': 'CjkUnifiedIdeograph78a7',
u'\u9e28': 'CjkUnifiedIdeograph9e28',
u'\uc2ab': 'HangulSyllableSeud',
u'\u5d2a': 'CjkUnifiedIdeograph5d2a',
u'\u272e': 'HeavyOutlinedBlackStar',
u'\u0bb1': 'TamilLetterRra',
u'\ua932': 'RejangLetterNga',
u'\u8db5': 'CjkUnifiedIdeograph8db5',
u'\u6c34': 'CjkUnifiedIdeograph6c34',
u'\u50b7': 'CjkUnifiedIdeograph50b7',
u'\uf638': 'UnicodeCharacter0xf638',
u'\udabb': 'UnicodeCharacter0xdabb',
u'\u3f3e': 'CjkUnifiedIdeograph3f3e',
u'\u63c1': 'CjkUnifiedIdeograph63c1',
u'\u8142': 'CjkUnifiedIdeograph8142',
u'\ue5c5': 'UnicodeCharacter0xe5c5',
u'\u4444': 'CjkUnifiedIdeograph4444',
u'\uce48': 'HangulSyllableCyils',
u'\u32cb': 'IdeographicTelegraphSymbolForDecember',
u'\ub4cf': 'HangulSyllableDyulh',
u'\u174e': 'BuhidLetterLa',
u'\u7bd1': 'CjkUnifiedIdeograph7bd1',
u'\u9952': 'CjkUnifiedIdeograph9952',
u'\ufdd5': 'UnicodeCharacter0xfdd5',
u'\u5c54': 'CjkUnifiedIdeograph5c54',
u'\u2658': 'WhiteChessKnight',
u'\u0adb': 'UnicodeCharacter0xadb',
u'\ua85c': 'PhagspaLetterHa',
u'\u8cdf': 'CjkUnifiedIdeograph8cdf',
u'\u6f5e': 'CjkUnifiedIdeograph6f5e',
u'\u53e1': 'CjkUnifiedIdeograph53e1',
u'\uf162': 'UnicodeCharacter0xf162',
u'\ud5e5': 'HangulSyllableHeg',
u'\u3e68': 'CjkUnifiedIdeograph3e68',
u'\u62eb': 'CjkUnifiedIdeograph62eb',
u'\u806c': 'CjkUnifiedIdeograph806c',
u'\ue4ef': 'UnicodeCharacter0xe4ef',
u'\u476e': 'CjkUnifiedIdeograph476e',
u'\uc972': 'HangulSyllableJyunh',
u'\u2df5': 'CombiningCyrillicLetterEste',
u'\ub7f9': 'HangulSyllableReolt',
u'\u1678': 'UnicodeCharacter0x1678',
u'\u7afb': 'CjkUnifiedIdeograph7afb',
u'\u987c': 'CjkUnifiedIdeograph987c',
u'\ufcff': 'ArabicLigatureHahWithAlefMaksuraIsolatedForm',
u'\u5f7e': 'CjkUnifiedIdeograph5f7e',
u'\uc603': 'HangulSyllableYeoc',
u'\u2182': 'RomanNumeralTenThousand',
u'\uab86': 'UnicodeCharacter0xab86',
u'\u0f09': 'TibetanMarkBskurYigMgo',
u'\u6e88': 'CjkUnifiedIdeograph6e88',
u'\u910d': 'CjkUnifiedIdeograph910d',
u'\uf08c': 'UnicodeCharacter0xf08c',
u'\u540f': 'CjkUnifiedIdeograph540f',
u'\ude13': 'UnicodeCharacter0xde13',
u'\u3992': 'CjkUnifiedIdeograph3992',
u'\ua017': 'YiSyllableBix',
u'\u8396': 'CjkUnifiedIdeograph8396',
u'\u6719': 'CjkUnifiedIdeograph6719',
u'\u4698': 'CjkUnifiedIdeograph4698',
u'\ue91d': 'UnicodeCharacter0xe91d',
u'\uc89c': 'HangulSyllableJwam',
u'\u3623': 'CjkUnifiedIdeograph3623',
u'\u11a2': 'HangulJungseongSsangaraea',
u'\ub827': 'HangulSyllableRyeogs',
u'\u9ba6': 'CjkUnifiedIdeograph9ba6',
u'\u7f29': 'CjkUnifiedIdeograph7f29',
u'\u5ea8': 'CjkUnifiedIdeograph5ea8',
u'\uc12d': 'HangulSyllableSeob',
u'\u20ac': 'EuroSign',
u'\uaab0': 'UnicodeCharacter0xaab0',
u'\u0e33': 'ThaiCharacterSaraAm',
u'\u69b2': 'CjkUnifiedIdeograph69b2',
u'\u9037': 'CjkUnifiedIdeograph9037',
u'\uf3b6': 'UnicodeCharacter0xf3b6',
u'\u5739': 'CjkUnifiedIdeograph5739',
u'\ud93d': 'UnicodeCharacter0xd93d',
u'\u38bc': 'CjkUnifiedIdeograph38bc',
u'\ua341': 'YiSyllableSsyx',
u'\u82c0': 'CjkUnifiedIdeograph82c0',
u'\u6643': 'CjkUnifiedIdeograph6643',
u'\u41c2': 'CjkUnifiedIdeograph41c2',
u'\ue847': 'UnicodeCharacter0xe847',
u'\ucbc6': 'HangulSyllableJjyulp',
u'\u314d': 'HangulLetterPhieuph',
u'\u10cc': 'UnicodeCharacter0x10cc',
u'\ubb51': 'HangulSyllableMweog',
u'\u9ad0': 'CjkUnifiedIdeograph9ad0',
u'\u7e53': 'CjkUnifiedIdeograph7e53',
u'\u59d2': 'CjkUnifiedIdeograph59d2',
u'\uc057': 'HangulSyllableBbyuh',
u'\u23d6': 'MetricalTwoShortsJoined',
u'\ua5da': 'VaiSyllableYo',
u'\u095d': 'DevanagariLetterRha',
u'\u68dc': 'CjkUnifiedIdeograph68dc',
u'\u9361': 'CjkUnifiedIdeograph9361',
u'\uf2e0': 'UnicodeCharacter0xf2e0',
u'\u5663': 'CjkUnifiedIdeograph5663',
u'\ud867': 'UnicodeCharacter0xd867',
u'\u3be6': 'CjkUnifiedIdeograph3be6',
u'\ua26b': 'YiSyllableNgex',
u'\ubdea': 'HangulSyllableBwij',
u'\u616d': 'CjkUnifiedIdeograph616d',
u'\u40ec': 'CjkUnifiedIdeograph40ec',
u'\ueb71': 'UnicodeCharacter0xeb71',
u'\ucaf0': 'HangulSyllableJjwak',
u'\u3077': 'HiraganaLetterPu',
u'\u13f6': 'UnicodeCharacter0x13f6',
u'\uba7b': 'HangulSyllableMyeolb',
u'\u95fa': 'CjkUnifiedIdeograph95fa',
u'\u797d': 'CjkUnifiedIdeograph797d',
u'\u58fc': 'CjkUnifiedIdeograph58fc',
u'\u4281': 'CjkUnifiedIdeograph4281',
u'\u2300': 'DiameterSign',
u'\u0783': 'ThaanaLetterRaa',
u'\ue002': 'UnicodeCharacter0xe002',
u'\uc485': 'HangulSyllableSsweonj',
u'\ua504': 'VaiSyllableWeen',
u'\u8987': 'CjkUnifiedIdeograph8987',
u'\u760e': 'CjkUnifiedIdeograph760e',
u'\u5a91': 'CjkUnifiedIdeograph5a91',
u'\u3b10': 'CjkUnifiedIdeograph3b10',
u'\u1f93': 'GreekSmallLetterEtaWithDasiaAndVariaAndYpogegrammeni',
u'\uf812': 'UnicodeCharacter0xf812',
u'\udc95': 'UnicodeCharacter0xdc95',
u'\ubd14': 'HangulSyllableBwan',
u'\ue197': 'UnicodeCharacter0xe197',
u'\ua699': 'UnicodeCharacter0xa699',
u'\u4e1e': 'CjkUnifiedIdeograph4e1e',
u'\u1320': 'EthiopicSyllableTha',
u'\u77a3': 'CjkUnifiedIdeograph77a3',
u'\ud022': 'HangulSyllableKwelm',
u'\u34a5': 'CjkUnifiedIdeograph34a5',
u'\u9524': 'CjkUnifiedIdeograph9524',
u'\uf9a7': 'CjkCompatibilityIdeographf9a7',
u'\ubea9': 'HangulSyllableBbyab',
u'\u6b30': 'CjkUnifiedIdeograph6b30',
u'\u4fb3': 'CjkUnifiedIdeograph4fb3',
u'\u2832': 'BraillePatternDots256',
u'\u0cb5': 'KannadaLetterVa',
u'\ued34': 'UnicodeCharacter0xed34',
u'\ud1b7': 'HangulSyllableToc',
u'\ub236': 'HangulSyllableNweoj',
u'\u96b9': 'CjkUnifiedIdeograph96b9',
u'\u4340': 'CjkUnifiedIdeograph4340',
u'\u64c5': 'CjkUnifiedIdeograph64c5',
u'\uc544': 'HangulSyllableA',
u'\u29c7': 'SquaredSmallCircle',
u'\u8a46': 'CjkUnifiedIdeograph8a46',
u'\ueec9': 'UnicodeCharacter0xeec9',
u'\ub3cb': 'HangulSyllableDod',
u'\u5b50': 'CjkUnifiedIdeograph5b50',
u'\u1852': 'MongolianLetterTodoCha',
u'\u7cd5': 'CjkUnifiedIdeograph7cd5',
u'\udd54': 'UnicodeCharacter0xdd54',
u'\u01d7': 'LatinCapitalLetterUWithDiaeresisAndAcute',
u'\ue256': 'UnicodeCharacter0xe256',
u'\uc6d9': 'HangulSyllableWeolt',
u'\ua758': 'LatinCapitalLetterQWithDiagonalStroke',
u'\u8bdb': 'CjkUnifiedIdeograph8bdb',
u'\u7062': 'CjkUnifiedIdeograph7062',
u'\u54e5': 'CjkUnifiedIdeograph54e5',
u'\u3564': 'CjkUnifiedIdeograph3564',
u'\u19e7': 'KhmerSymbolPrampiiKoet',
u'\ufa66': 'CjkCompatibilityIdeographfa66',
u'\udee9': 'UnicodeCharacter0xdee9',
u'\ubf68': 'HangulSyllableBbwals',
u'\ue3eb': 'UnicodeCharacter0xe3eb',
u'\ua0ed': 'YiSyllableVap',
u'\u4872': 'CjkUnifiedIdeograph4872',
u'\u0d74': 'MalayalamFractionOneHalf',
u'\u71f7': 'CjkUnifiedIdeograph71f7',
u'\ud276': 'HangulSyllableTwebs',
u'\u36f9': 'CjkUnifiedIdeograph36f9',
u'\u9778': 'CjkUnifiedIdeograph9778',
u'\ufbfb': 'ArabicLigatureUighurKirghizYehWithHamzaAboveWithAlefMaksuraInitialForm',
u'\ub8fd': 'HangulSyllableRung',
u'\u8201': 'CjkUnifiedIdeograph8201',
u'\u6584': 'CjkUnifiedIdeograph6584',
u'\u2a86': 'GreaterthanOrApproximate',
u'\uef88': 'UnicodeCharacter0xef88',
u'\u530b': 'CjkUnifiedIdeograph530b',
u'\uac8a': 'HangulSyllableGeop',
u'\u100d': 'MyanmarLetterDda',
u'\ud50f': 'HangulSyllablePeulb',
u'\u9a11': 'CjkUnifiedIdeograph9a11',
u'\u7d94': 'CjkUnifiedIdeograph7d94',
u'\u0296': 'LatinLetterInvertedGlottalStop',
u'\uc798': 'HangulSyllableJal',
u'\u849a': 'CjkUnifiedIdeograph849a',
u'\u681d': 'CjkUnifiedIdeograph681d',
u'\u2d1f': 'GeorgianSmallLetterJhan',
u'\uf221': 'UnicodeCharacter0xf221',
u'\ub723': 'HangulSyllableDdyuc',
u'\u55a4': 'CjkUnifiedIdeograph55a4',
u'\u1aa6': 'UnicodeCharacter0x1aa6',
u'\udfa8': 'UnicodeCharacter0xdfa8',
u'\u9caa': 'CjkUnifiedIdeograph9caa',
u'\u402d': 'CjkUnifiedIdeograph402d',
u'\ua1ac': 'YiSyllableHlux',
u'\u052f': 'UnicodeCharacter0x52f',
u'\uca31': 'HangulSyllableJjyaeg',
u'\u8f33': 'CjkUnifiedIdeograph8f33',
u'\u72b6': 'CjkUnifiedIdeograph72b6',
u'\u37b8': 'CjkUnifiedIdeograph37b8',
u'\uf4ba': 'UnicodeCharacter0xf4ba',
u'\u583d': 'CjkUnifiedIdeograph583d',
u'\ub9bc': 'HangulSyllableRim',
u'\u1d3f': 'ModifierLetterCapitalR',
u'\u2241': 'NotTilde',
u'\ue743': 'UnicodeCharacter0xe743',
u'\ua445': 'YiSyllableNjyx',
u'\u4ac6': 'CjkUnifiedIdeograph4ac6',
u'\u0fc8': 'TibetanSymbolPhurPa',
u'\uccca': 'HangulSyllableCej',
u'\u91cc': 'CjkUnifiedIdeograph91cc',
u'\u754f': 'CjkUnifiedIdeograph754f',
u'\u3a51': 'CjkUnifiedIdeograph3a51',
u'\uff53': 'FullwidthLatinSmallLetterS',
u'\ubc55': 'HangulSyllableByalg',
u'\u67d8': 'CjkUnifiedIdeograph67d8',
u'\u24da': 'CircledLatinSmallLetterK',
u'\ue9dc': 'UnicodeCharacter0xe9dc',
u'\u4d5f': 'CjkUnifiedIdeograph4d5f',
u'\uaede': 'HangulSyllableGgenh',
u'\u1261': 'EthiopicSyllableBu',
u'\ud763': 'HangulSyllableHeus',
u'\u9465': 'CjkUnifiedIdeograph9465',
u'\u7fe8': 'CjkUnifiedIdeograph7fe8',
u'\u3cea': 'CjkUnifiedIdeograph3cea',
u'\uc1ec': 'HangulSyllableSoels',
u'\u86ee': 'CjkUnifiedIdeograph86ee',
u'\u6a71': 'CjkUnifiedIdeograph6a71',
u'\u2f73': 'KangxiRadicalCave',
u'\uec75': 'UnicodeCharacter0xec75',
u'\ub177': 'HangulSyllableNyeh',
u'\u57f8': 'CjkUnifiedIdeograph57f8',
u'\u14fa': 'CanadianSyllabicsSwii',
u'\ud9fc': 'UnicodeCharacter0xd9fc',
u'\u9efe': 'CjkUnifiedIdeograph9efe',
u'\ua400': 'YiSyllableQiet',
u'\u8083': 'CjkUnifiedIdeograph8083',
u'\u6702': 'CjkUnifiedIdeograph6702',
u'\u4385': 'CjkUnifiedIdeograph4385',
u'\ued06': 'UnicodeCharacter0xed06',
u'\uc989': 'HangulSyllableJeug',
u'\u360c': 'CjkUnifiedIdeograph360c',
u'\u128f': 'UnicodeCharacter0x128f',
u'\ubc10': 'HangulSyllableMik',
u'\u9893': 'CjkUnifiedIdeograph9893',
u'\u7f12': 'CjkUnifiedIdeograph7f12',
u'\u5b95': 'CjkUnifiedIdeograph5b95',
u'\uc516': 'HangulSyllableSsyilm',
u'\u2199': 'SouthWestArrow',
u'\uaf9d': 'HangulSyllableGgyog',
u'\u0e1c': 'ThaiCharacterPhoPhung',
u'\u6a9f': 'CjkUnifiedIdeograph6a9f',
u'\u9420': 'CjkUnifiedIdeograph9420',
u'\uf0a3': 'UnicodeCharacter0xf0a3',
u'\u5722': 'CjkUnifiedIdeograph5722',
u'\udd26': 'UnicodeCharacter0xdd26',
u'\u39a9': 'CjkUnifiedIdeograph39a9',
u'\u87ad': 'CjkUnifiedIdeograph87ad',
u'\u662c': 'CjkUnifiedIdeograph662c',
u'\u42af': 'CjkUnifiedIdeograph42af',
u'\uec30': 'UnicodeCharacter0xec30',
u'\uc8b3': 'HangulSyllableJwaelb',
u'\u3536': 'CjkUnifiedIdeograph3536',
u'\u11b9': 'HangulJongseongPieupsios',
u'\ub33a': 'HangulSyllableDyaegg',
u'\u9fbd': 'UnicodeCharacter0x9fbd',
u'\u7e3c': 'CjkUnifiedIdeograph7e3c',
u'\u5abf': 'CjkUnifiedIdeograph5abf',
u'\uc440': 'HangulSyllableSsoess',
u'\u20c3': 'UnicodeCharacter0x20c3',
u'\uaec7': 'HangulSyllableGgeolb',
u'\u0d46': 'MalayalamVowelSignE',
u'\u69c9': 'CjkUnifiedIdeograph69c9',
u'\u8b4a': 'CjkUnifiedIdeograph8b4a',
u'\uf7cd': 'UnicodeCharacter0xf7cd',
u'\u564c': 'CjkUnifiedIdeograph564c',
u'\udc50': 'UnicodeCharacter0xdc50',
u'\u38d3': 'CjkUnifiedIdeograph38d3',
u'\u86d7': 'CjkUnifiedIdeograph86d7',
u'\u6556': 'CjkUnifiedIdeograph6556',
u'\u41d9': 'CjkUnifiedIdeograph41d9',
u'\ue35a': 'UnicodeCharacter0xe35a',
u'\ucfdd': 'HangulSyllableKyot',
u'\u3460': 'CjkUnifiedIdeograph3460',
u'\u10e3': 'GeorgianLetterUn',
u'\ub264': 'HangulSyllableNwils',
u'\u9ee7': 'CjkUnifiedIdeograph9ee7',
u'\u7d66': 'CjkUnifiedIdeograph7d66',
u'\u59e9': 'CjkUnifiedIdeograph59e9',
u'\ufb6a': 'ArabicLetterVehIsolatedForm',
u'\u27ed': 'MathematicalRightWhiteTortoiseShellBracket',
u'\uadf1': 'HangulSyllableGyung',
u'\u0c70': 'UnicodeCharacter0xc70',
u'\u68f3': 'CjkUnifiedIdeograph68f3',
u'\u8a74': 'CjkUnifiedIdeograph8a74',
u'\uf6f7': 'UnicodeCharacter0xf6f7',
u'\u5576': 'CjkUnifiedIdeograph5576',
u'\ud37a': 'HangulSyllablePyaep',
u'\u3ffd': 'CjkUnifiedIdeograph3ffd',
u'\u0501': 'CyrillicSmallLetterKomiDe',
u'\u6480': 'CjkUnifiedIdeograph6480',
u'\u8305': 'CjkUnifiedIdeograph8305',
u'\ue284': 'UnicodeCharacter0xe284',
u'\u4e07': 'CjkUnifiedIdeograph4e07',
u'\ud40b': 'HangulSyllablePwags',
u'\u2b8a': 'UnicodeCharacter0x2b8a',
u'\ub18e': 'HangulSyllableNoj',
u'\u1d11': 'LatinSmallLetterSidewaysO',
u'\u7c90': 'CjkUnifiedIdeograph7c90',
u'\u9b15': 'CjkUnifiedIdeograph9b15',
u'\ufa94': 'CjkCompatibilityIdeographfa94',
u'\u2c1b': 'GlagoliticCapitalLetterShta',
u'\u039a': 'GreekCapitalLetterKappa',
u'\uaa1f': 'ChamLetterMue',
u'\u899e': 'CjkUnifiedIdeograph899e',
u'\u7521': 'CjkUnifiedIdeograph7521',
u'\u54a0': 'CjkUnifiedIdeograph54a0',
u'\uf325': 'UnicodeCharacter0xf325',
u'\ud2a4': 'HangulSyllableTyul',
u'\u042b': 'CyrillicCapitalLetterYeru',
u'\u1baa': 'SundaneseSignPamaaeh',
u'\u822f': 'CjkUnifiedIdeograph822f',
u'\ue1ae': 'UnicodeCharacter0xe1ae',
u'\u4d31': 'CjkUnifiedIdeograph4d31',
u'\ucb35': 'HangulSyllableJjyolg',
u'\u2ab4': 'SucceedsAboveEqualsSign',
u'\ub0b8': 'HangulSyllableNaen',
u'\u1c3b': 'LepchaPunctuationTarol',
u'\u73ba': 'CjkUnifiedIdeograph73ba',
u'\u9a3f': 'CjkUnifiedIdeograph9a3f',
u'\uf9be': 'CjkCompatibilityIdeographf9be',
u'\u2345': 'AplFunctionalSymbolLeftwardsVane',
u'\u02c4': 'ModifierLetterUpArrowhead',
u'\ua949': 'RejangVowelSignE',
u'\u88c8': 'CjkUnifiedIdeograph88c8',
u'\u744b': 'CjkUnifiedIdeograph744b',
u'\u4bca': 'CjkUnifiedIdeograph4bca',
u'\uf24f': 'UnicodeCharacter0xf24f',
u'\ud1ce': 'HangulSyllableTwabs',
u'\u3b55': 'CjkUnifiedIdeograph3b55',
u'\u1ad4': 'UnicodeCharacter0x1ad4',
u'\u8159': 'CjkUnifiedIdeograph8159',
u'\ue0d8': 'UnicodeCharacter0xe0d8',
u'\u4c5b': 'CjkUnifiedIdeograph4c5b',
u'\uca5f': 'HangulSyllableJjeos',
u'\u29de': 'InfinityNegatedWithVerticalBar',
u'\ub7e2': 'HangulSyllableRyaebs',
u'\u1365': 'EthiopicColon',
u'\u72e4': 'CjkUnifiedIdeograph72e4',
u'\u9969': 'CjkUnifiedIdeograph9969',
u'\uf8e8': 'UnicodeCharacter0xf8e8',
u'\u226f': 'NotGreaterthan',
u'\u01ee': 'LatinCapitalLetterEzhWithCaron',
u'\ua873': 'PhagspaLetterCandrabindu',
u'\u8ff2': 'CjkUnifiedIdeograph8ff2',
u'\u6b75': 'CjkUnifiedIdeograph6b75',
u'\u4af4': 'CjkUnifiedIdeograph4af4',
u'\uf179': 'UnicodeCharacter0xf179',
u'\ud0f8': 'HangulSyllableTya',
u'\u3a7f': 'CjkUnifiedIdeograph3a7f',
u'\u19fe': 'KhmerSymbolDapbuonRoc',
u'\u6c06': 'CjkUnifiedIdeograph6c06',
u'\u4889': 'CjkUnifiedIdeograph4889',
u'\u2908': 'DownwardsArrowWithHorizontalStroke',
u'\u158b': 'CanadianSyllabicsWestcreeRi',
u'\uea0a': 'UnicodeCharacter0xea0a',
u'\ud68d': 'HangulSyllableHoeg',
u'\ub70c': 'HangulSyllableDdyu',
u'\u938f': 'CjkUnifiedIdeograph938f',
u'\u4416': 'CjkUnifiedIdeograph4416',
u'\u0118': 'LatinCapitalLetterEWithOgonek',
u'\u6d9b': 'CjkUnifiedIdeograph6d9b',
u'\uc21a': 'HangulSyllableSugg',
u'\u2e9d': 'CjkRadicalMoon',
u'\u8f1c': 'CjkUnifiedIdeograph8f1c',
u'\ueb9f': 'UnicodeCharacter0xeb9f',
u'\ub4a1': 'HangulSyllableDwet',
u'\u5c26': 'CjkUnifiedIdeograph5c26',
u'\u1928': 'LimbuVowelSignO',
u'\u45ab': 'CjkUnifiedIdeograph45ab',
u'\uda2a': 'UnicodeCharacter0xda2a',
u'\u06ad': 'ArabicLetterNg',
u'\ue72c': 'UnicodeCharacter0xe72c',
u'\uc3af': 'HangulSyllableSsyeolh',
u'\ua02e': 'YiSyllableBu',
u'\u8cb1': 'CjkUnifiedIdeograph8cb1',
u'\u7138': 'CjkUnifiedIdeograph7138',
u'\u5dbb': 'CjkUnifiedIdeograph5dbb',
u'\u323a': 'ParenthesizedIdeographCall',
u'\u1ebd': 'LatinSmallLetterEWithTilde',
u'\uff3c': 'FullwidthReverseSolidus',
u'\udbbf': 'UnicodeCharacter0xdbbf',
u'\ub83e': 'HangulSyllableRyeop',
u'\ue4c1': 'UnicodeCharacter0xe4c1',
u'\ua1c3': 'YiSyllableLuot',
u'\u4948': 'CjkUnifiedIdeograph4948',
u'\u0a4a': 'UnicodeCharacter0xa4a',
u'\u76cd': 'CjkUnifiedIdeograph76cd',
u'\ud74c': 'HangulSyllableHyuk',
u'\u33cf': 'SquareKt',
u'\u904e': 'CjkUnifiedIdeograph904e',
u'\ufcd1': 'ArabicLigatureMeemWithMeemInitialForm',
u'\ub9d3': 'HangulSyllableMalb',
u'\u625a': 'CjkUnifiedIdeograph625a',
u'\u4edd': 'CjkUnifiedIdeograph4edd',
u'\u2f5c': 'KangxiRadicalCow',
u'\u0bdf': 'UnicodeCharacter0xbdf',
u'\ue85e': 'UnicodeCharacter0xe85e',
u'\ud4e1': 'HangulSyllablePwing',
u'\ub560': 'HangulSyllableDdaess',
u'\u91e3': 'CjkUnifiedIdeograph91e3',
u'\u7a6a': 'CjkUnifiedIdeograph7a6a',
u'\u076c': 'ArabicLetterRehWithHamzaAbove',
u'\u63ef': 'CjkUnifiedIdeograph63ef',
u'\uc06e': 'HangulSyllableBbeuj',
u'\u2cf1': 'UnicodeCharacter0x2cf1',
u'\u8d70': 'CjkUnifiedIdeograph8d70',
u'\ue9f3': 'UnicodeCharacter0xe9f3',
u'\uaaf5': 'UnicodeCharacter0xaaf5',
u'\u527a': 'CjkUnifiedIdeograph527a',
u'\u1f7c': 'GreekSmallLetterOmegaWithVaria',
u'\u7bff': 'CjkUnifiedIdeograph7bff',
u'\ud87e': 'UnicodeCharacter0xd87e',
u'\ue580': 'UnicodeCharacter0xe580',
u'\u4103': 'CjkUnifiedIdeograph4103',
u'\ua682': 'CyrillicCapitalLetterDzwe',
u'\u0205': 'LatinSmallLetterEWithDoubleGrave',
u'\ucf07': 'HangulSyllableKed',
u'\u8809': 'CjkUnifiedIdeograph8809',
u'\u778c': 'CjkUnifiedIdeograph778c',
u'\u308e': 'HiraganaLetterSmallWa',
u'\ufd90': 'UnicodeCharacter0xfd90',
u'\u5913': 'CjkUnifiedIdeograph5913',
u'\ube92': 'HangulSyllableBbaej',
u'\u1a15': 'BugineseLetterA',
u'\u2717': 'BallotX',
u'\ue019': 'UnicodeCharacter0xe019',
u'\uad1b': 'HangulSyllableGwaegs',
u'\u4f9c': 'CjkUnifiedIdeograph4f9c',
u'\u089e': 'UnicodeCharacter0x89e',
u'\ud5a0': 'HangulSyllableHyam',
u'\u96a2': 'CjkUnifiedIdeograph96a2',
u'\u7225': 'CjkUnifiedIdeograph7225',
u'\u3f27': 'CjkUnifiedIdeograph3f27',
u'\uf829': 'UnicodeCharacter0xf829',
u'\u852b': 'CjkUnifiedIdeograph852b',
u'\u60ae': 'CjkUnifiedIdeograph60ae',
u'\u2db0': 'EthiopicSyllableZza',
u'\ueeb2': 'UnicodeCharacter0xeeb2',
u'\u4a35': 'CjkUnifiedIdeograph4a35',
u'\uabb4': 'UnicodeCharacter0xabb4',
u'\u1737': 'UnicodeCharacter0x1737',
u'\ud039': 'HangulSyllableKwinj',
u'\u9d3b': 'CjkUnifiedIdeograph9d3b',
u'\u78be': 'CjkUnifiedIdeograph78be',
u'\u05c0': 'HebrewPunctuationPaseq',
u'\uc6c2': 'HangulSyllableUbs',
u'\u83c4': 'CjkUnifiedIdeograph83c4',
u'\u6f47': 'CjkUnifiedIdeograph6f47',
u'\u2849': 'BraillePatternDots147',
u'\uf54b': 'UnicodeCharacter0xf54b',
u'\ub64d': 'HangulSyllableDdwaenj',
u'\u50ce': 'CjkUnifiedIdeograph50ce',
u'\u1dd0': 'CombiningIsBelow',
u'\uded2': 'UnicodeCharacter0xded2',
u'\u9bd4': 'CjkUnifiedIdeograph9bd4',
u'\u4757': 'CjkUnifiedIdeograph4757',
u'\ua4d6': 'UnicodeCharacter0xa4d6',
u'\ucd5b': 'HangulSyllableCwaeh',
u'\u8e5d': 'CjkUnifiedIdeograph8e5d',
u'\u75e0': 'CjkUnifiedIdeograph75e0',
u'\u36e2': 'CjkUnifiedIdeograph36e2',
u'\uf3e4': 'UnicodeCharacter0xf3e4',
u'\u5f67': 'CjkUnifiedIdeograph5f67',
u'\ubce6': 'HangulSyllableByelp',
u'\u1869': 'MongolianLetterSibeDa',
u'\u256b': 'BoxDrawingsVerticalDoubleAndHorizontalSingle',
u'\ue66d': 'UnicodeCharacter0xe66d',
u'\ua36f': 'YiSyllableChe',
u'\u4df0': 'HexagramForRevolution',
u'\u0ef2': 'UnicodeCharacter0xef2',
u'\ucbf4': 'HangulSyllableJjyin',
u'\u94f6': 'CjkUnifiedIdeograph94f6',
u'\u7079': 'CjkUnifiedIdeograph7079',
u'\u3d7b': 'CjkUnifiedIdeograph3d7b',
u'\ufe7d': 'ArabicShaddaMedialForm',
u'\ubb7f': 'HangulSyllableMwes',
u'\uc781': 'HangulSyllableIlt',
u'\u2004': 'ThreeperemSpace',
u'\u0c87': 'KannadaLetterI',
u'\uaa08': 'ChamLetterGa',
u'\u968b': 'CjkUnifiedIdeograph968b',
u'\u690a': 'CjkUnifiedIdeograph690a',
u'\u558d': 'CjkUnifiedIdeograph558d',
u'\uf30e': 'UnicodeCharacter0xf30e',
u'\udf91': 'UnicodeCharacter0xdf91',
u'\u3814': 'CjkUnifiedIdeograph3814',
u'\u6497': 'CjkUnifiedIdeograph6497',
u'\u8218': 'CjkUnifiedIdeograph8218',
u'\uee9b': 'UnicodeCharacter0xee9b',
u'\u411a': 'CjkUnifiedIdeograph411a',
u'\ucb1e': 'HangulSyllableJjoelp',
u'\u37a1': 'CjkUnifiedIdeograph37a1',
u'\ub1a5': 'HangulSyllableNwab',
u'\u1024': 'MyanmarLetterIi',
u'\u7ca7': 'CjkUnifiedIdeograph7ca7',
u'\u9a28': 'CjkUnifiedIdeograph9a28',
u'\uc6ab': 'HangulSyllableYoc',
u'\u592a': 'CjkUnifiedIdeograph592a',
u'\u232e': 'AllAroundprofile',
u'\u0fb1': 'TibetanSubjoinedLetterYa',
u'\uad32': 'HangulSyllableGwaep',
u'\u89b5': 'CjkUnifiedIdeograph89b5',
u'\u6834': 'CjkUnifiedIdeograph6834',
u'\u54b7': 'CjkUnifiedIdeograph54b7',
u'\uf238': 'UnicodeCharacter0xf238',
u'\udebb': 'UnicodeCharacter0xdebb',
u'\u3b3e': 'CjkUnifiedIdeograph3b3e',
u'\u67c1': 'CjkUnifiedIdeograph67c1',
u'\u8542': 'CjkUnifiedIdeograph8542',
u'\ue1c5': 'UnicodeCharacter0xe1c5',
u'\u4044': 'CjkUnifiedIdeograph4044',
u'\uca48': 'HangulSyllableJjyaek',
u'\u36cb': 'CjkUnifiedIdeograph36cb',
u'\ub0cf': 'HangulSyllableNaeh',
u'\u134e': 'EthiopicSyllableFo',
u'\u7fd1': 'CjkUnifiedIdeograph7fd1',
u'\u9d52': 'CjkUnifiedIdeograph9d52',
u'\uf9d5': 'CjkCompatibilityIdeographf9d5',
u'\u5854': 'CjkUnifiedIdeograph5854',
u'\u2258': 'CorrespondsTo',
u'\u0edb': 'UnicodeCharacter0xedb',
u'\uac5c': 'HangulSyllableGyael',
u'\u88df': 'CjkUnifiedIdeograph88df',
u'\u6b5e': 'CjkUnifiedIdeograph6b5e',
u'\u57e1': 'CjkUnifiedIdeograph57e1',
u'\uf562': 'UnicodeCharacter0xf562',
u'\ud1e5': 'HangulSyllableTwaelt',
u'\u3a68': 'CjkUnifiedIdeograph3a68',
u'\u66eb': 'CjkUnifiedIdeograph66eb',
u'\u846c': 'CjkUnifiedIdeograph846c',
u'\ue0ef': 'UnicodeCharacter0xe0ef',
u'\u436e': 'CjkUnifiedIdeograph436e',
u'\ucd72': 'HangulSyllableCoej',
u'\u29f5': 'ReverseSolidusOperator',
u'\ub3f9': 'HangulSyllableDwat',
u'\u1278': 'EthiopicSyllableCa',
u'\u7efb': 'CjkUnifiedIdeograph7efb',
u'\u9c7c': 'CjkUnifiedIdeograph9c7c',
u'\uf8ff': 'UnicodeCharacter0xf8ff',
u'\u5b7e': 'CjkUnifiedIdeograph5b7e',
u'\uc203': 'HangulSyllableSyod',
u'\u2582': 'LowerOneQuarterBlock',
u'\uaf86': 'HangulSyllableGgoenh',
u'\u0b09': 'OriyaLetterU',
u'\u6a88': 'CjkUnifiedIdeograph6a88',
u'\u950d': 'CjkUnifiedIdeograph950d',
u'\uf48c': 'UnicodeCharacter0xf48c',
u'\u500f': 'CjkUnifiedIdeograph500f',
u'\uda13': 'UnicodeCharacter0xda13',
u'\u3d92': 'CjkUnifiedIdeograph3d92',
u'\ua417': 'YiSyllableQyr',
u'\u8796': 'CjkUnifiedIdeograph8796',
u'\u6319': 'CjkUnifiedIdeograph6319',
u'\u4298': 'CjkUnifiedIdeograph4298',
u'\ued1d': 'UnicodeCharacter0xed1d',
u'\ucc9c': 'HangulSyllableCeon',
u'\u3223': 'ParenthesizedIdeographFour',
u'\u15a2': 'CanadianSyllabicsLho',
u'\ubc27': 'HangulSyllableBas',
u'\u9fa6': 'CjkUnifiedIdeograph9fa6',
u'\u7b29': 'CjkUnifiedIdeograph7b29',
u'\u5aa8': 'CjkUnifiedIdeograph5aa8',
u'\uc52d': 'HangulSyllableSsinj',
u'\u24ac': 'ParenthesizedLatinSmallLetterQ',
u'\uaeb0': 'HangulSyllableGgyaem',
u'\u0a33': 'GurmukhiLetterLla',
u'\u6db2': 'CjkUnifiedIdeograph6db2',
u'\u9437': 'CjkUnifiedIdeograph9437',
u'\uf7b6': 'UnicodeCharacter0xf7b6',
u'\u5339': 'CjkUnifiedIdeograph5339',
u'\udd3d': 'UnicodeCharacter0xdd3d',
u'\u3cbc': 'CjkUnifiedIdeograph3cbc',
u'\ua741': 'LatinSmallLetterKWithStroke',
u'\u86c0': 'CjkUnifiedIdeograph86c0',
u'\u6243': 'CjkUnifiedIdeograph6243',
u'\u45c2': 'CjkUnifiedIdeograph45c2',
u'\uec47': 'UnicodeCharacter0xec47',
u'\ucfc6': 'HangulSyllableKyogg',
u'\u354d': 'CjkUnifiedIdeograph354d',
u'\u14cc': 'CanadianSyllabicsWestcreeNwa',
u'\ubf51': 'HangulSyllableBbob',
u'\u9ed0': 'CjkUnifiedIdeograph9ed0',
u'\u7a53': 'CjkUnifiedIdeograph7a53',
u'\u5dd2': 'CjkUnifiedIdeograph5dd2',
u'\uc457': 'HangulSyllableSsyolh',
u'\u27d6': 'RightOuterJoin',
u'\ua1da': 'YiSyllableGit',
u'\u0d5d': 'UnicodeCharacter0xd5d',
u'\u6cdc': 'CjkUnifiedIdeograph6cdc',
u'\u9761': 'CjkUnifiedIdeograph9761',
u'\uf6e0': 'UnicodeCharacter0xf6e0',
u'\u5263': 'CjkUnifiedIdeograph5263',
u'\udc67': 'UnicodeCharacter0xdc67',
u'\u3fe6': 'CjkUnifiedIdeograph3fe6',
u'\ua66b': 'CyrillicSmallLetterBinocularO',
u'\ub9ea': 'HangulSyllableMaenh',
u'\u656d': 'CjkUnifiedIdeograph656d',
u'\u44ec': 'CjkUnifiedIdeograph44ec',
u'\uef71': 'UnicodeCharacter0xef71',
u'\ucef0': 'HangulSyllableKeols',
u'\u3477': 'CjkUnifiedIdeograph3477',
u'\u17f6': 'KhmerSymbolLekAttakPrammuoy',
u'\ube7b': 'HangulSyllableBbah',
u'\u91fa': 'CjkUnifiedIdeograph91fa',
u'\u7d7d': 'CjkUnifiedIdeograph7d7d',
u'\u5cfc': 'CjkUnifiedIdeograph5cfc',
u'\u4681': 'CjkUnifiedIdeograph4681',
u'\u2700': 'UnicodeCharacter0x2700',
u'\u0383': 'UnicodeCharacter0x383',
u'\ue402': 'UnicodeCharacter0xe402',
u'\uc085': 'HangulSyllableBbyib',
u'\ua104': 'YiSyllableDiex',
u'\u8d87': 'CjkUnifiedIdeograph8d87',
u'\u720e': 'CjkUnifiedIdeograph720e',
u'\u5e91': 'CjkUnifiedIdeograph5e91',
u'\u3f10': 'CjkUnifiedIdeograph3f10',
u'\u1b93': 'SundaneseLetterDa',
u'\ufc12': 'ArabicLigatureThehWithMeemIsolatedForm',
u'\ud895': 'UnicodeCharacter0xd895',
u'\ub914': 'HangulSyllableRweom',
u'\ue597': 'UnicodeCharacter0xe597',
u'\ua299': 'YiSyllableZuo',
u'\u4a1e': 'CjkUnifiedIdeograph4a1e',
u'\u1720': 'HanunooLetterA',
u'\u73a3': 'CjkUnifiedIdeograph73a3',
u'\ud422': 'HangulSyllablePwap',
u'\u30a5': 'KatakanaLetterSmallU',
u'\u9124': 'CjkUnifiedIdeograph9124',
u'\ufda7': 'ArabicLigatureJeemWithMeemWithAlefMaksuraFinalForm',
u'\ubaa9': 'HangulSyllableMog',
u'\u6f30': 'CjkUnifiedIdeograph6f30',
u'\u4bb3': 'CjkUnifiedIdeograph4bb3',
u'\u2c32': 'GlagoliticSmallLetterVede',
u'\u08b5': 'UnicodeCharacter0x8b5',
u'\ue934': 'UnicodeCharacter0xe934',
u'\ud5b7': 'HangulSyllableHyaelb',
u'\ub636': 'HangulSyllableDdwalm',
u'\u92b9': 'CjkUnifiedIdeograph92b9',
u'\u4740': 'CjkUnifiedIdeograph4740',
u'\u0442': 'CyrillicSmallLetterTe',
u'\u60c5': 'CjkUnifiedIdeograph60c5',
u'\uc144': 'HangulSyllableSels',
u'\u2dc7': 'UnicodeCharacter0x2dc7',
u'\u8e46': 'CjkUnifiedIdeograph8e46',
u'\ueac9': 'UnicodeCharacter0xeac9',
u'\ub7cb': 'HangulSyllableRyac',
u'\u5f50': 'CjkUnifiedIdeograph5f50',
u'\u1c52': 'OlChikiDigitTwo',
u'\u78d5': 'CjkUnifiedIdeograph78d5',
u'\ud954': 'UnicodeCharacter0xd954',
u'\u05d7': 'HebrewLetterHet',
u'\ue656': 'UnicodeCharacter0xe656',
u'\uc2d9': 'HangulSyllableSyit',
u'\ua358': 'YiSyllableZhup',
u'\u8fdb': 'CjkUnifiedIdeograph8fdb',
u'\u7462': 'CjkUnifiedIdeograph7462',
u'\u50e5': 'CjkUnifiedIdeograph50e5',
u'\u3164': 'HangulFiller',
u'\u1de7': 'UnicodeCharacter0x1de7',
u'\ufe66': 'SmallEqualsSign',
u'\udae9': 'UnicodeCharacter0xdae9',
u'\ubb68': 'HangulSyllableMweok',
u'\ue7eb': 'UnicodeCharacter0xe7eb',
u'\ua4ed': 'UnicodeCharacter0xa4ed',
u'\u4c72': 'CjkUnifiedIdeograph4c72',
u'\u0974': 'UnicodeCharacter0x974',
u'\u75f7': 'CjkUnifiedIdeograph75f7',
u'\ud676': 'HangulSyllableHwaenh',
u'\u32f9': 'CircledKatakanaRe',
u'\u9378': 'CjkUnifiedIdeograph9378',
u'\ufffb': 'InterlinearAnnotationTerminator',
u'\ubcfd': 'HangulSyllableBolg',
u'\u8601': 'CjkUnifiedIdeograph8601',
u'\u6184': 'CjkUnifiedIdeograph6184',
u'\u2e86': 'CjkRadicalBox',
u'\ueb88': 'UnicodeCharacter0xeb88',
u'\u570b': 'CjkUnifiedIdeograph570b',
u'\ua88a': 'SaurashtraLetterVocalicL',
u'\u140d': 'CanadianSyllabicsWestcreeWe',
u'\ud10f': 'HangulSyllableTyac',
u'\u9e11': 'CjkUnifiedIdeograph9e11',
u'\u7994': 'CjkUnifiedIdeograph7994',
u'\u0696': 'ArabicLetterRehWithDotBelowAndDotAbove',
u'\uc398': 'HangulSyllableSsess',
u'\u809a': 'CjkUnifiedIdeograph809a',
u'\u6c1d': 'CjkUnifiedIdeograph6c1d',
u'\u291f': 'LeftwardsArrowFromBarToBlackDiamond',
u'\uf621': 'UnicodeCharacter0xf621',
u'\ub323': 'HangulSyllableDyad',
u'\u51a4': 'CjkUnifiedIdeograph51a4',
u'\u1ea6': 'LatinCapitalLetterAWithCircumflexAndGrave',
u'\udba8': 'UnicodeCharacter0xdba8',
u'\u98aa': 'CjkUnifiedIdeograph98aa',
u'\u442d': 'CjkUnifiedIdeograph442d',
u'\ua5ac': 'VaiSyllableSu',
u'\u012f': 'LatinSmallLetterIWithOgonek',
u'\uce31': 'HangulSyllableCeub',
u'\u8b33': 'CjkUnifiedIdeograph8b33',
u'\u76b6': 'CjkUnifiedIdeograph76b6',
u'\u33b8': 'SquareKv',
u'\uf0ba': 'UnicodeCharacter0xf0ba',
u'\u5c3d': 'CjkUnifiedIdeograph5c3d',
u'\ubdbc': 'HangulSyllableBwen',
u'\u193f': 'UnicodeCharacter0x193f',
u'\u2641': 'Earth',
u'\ue343': 'UnicodeCharacter0xe343',
u'\ua045': 'YiSyllablePuop',
u'\u4ec6': 'CjkUnifiedIdeograph4ec6',
u'\u0bc8': 'TamilVowelSignAi',
u'\uc8ca': 'HangulSyllableJoenh',
u'\u95cc': 'CjkUnifiedIdeograph95cc',
u'\u714f': 'CjkUnifiedIdeograph714f',
u'\u3e51': 'CjkUnifiedIdeograph3e51',
u'\ufb53': 'ArabicLetterBeehFinalForm',
u'\ub855': 'HangulSyllableRyeng',
u'\u63d8': 'CjkUnifiedIdeograph63d8',
u'\u20da': 'CombiningAnticlockwiseRingOverlay',
u'\ueddc': 'UnicodeCharacter0xeddc',
u'\u495f': 'CjkUnifiedIdeograph495f',
u'\uaade': 'UnicodeCharacter0xaade',
u'\u1661': 'CanadianSyllabicsCarrierChu',
u'\ud363': 'HangulSyllablePyaegs',
u'\u9065': 'CjkUnifiedIdeograph9065',
u'\u7be8': 'CjkUnifiedIdeograph7be8',
u'\u38ea': 'CjkUnifiedIdeograph38ea',
u'\uc5ec': 'HangulSyllableYeo',
u'\u82ee': 'CjkUnifiedIdeograph82ee',
u'\u6e71': 'CjkUnifiedIdeograph6e71',
u'\u2b73': 'UnicodeCharacter0x2b73',
u'\ue875': 'UnicodeCharacter0xe875',
u'\ub577': 'HangulSyllableDdyalh',
u'\u53f8': 'CjkUnifiedIdeograph53f8',
u'\u10fa': 'GeorgianLetterAin',
u'\uddfc': 'UnicodeCharacter0xddfc',
u'\u9afe': 'CjkUnifiedIdeograph9afe',
u'\ua800': 'SylotiNagriLetterA',
u'\u8c83': 'CjkUnifiedIdeograph8c83',
u'\u6b02': 'CjkUnifiedIdeograph6b02',
u'\u4f85': 'CjkUnifiedIdeograph4f85',
u'\ue106': 'UnicodeCharacter0xe106',
u'\uc589': 'HangulSyllableYalt',
u'\u3a0c': 'CjkUnifiedIdeograph3a0c',
u'\u1e8f': 'LatinSmallLetterYWithDotAbove',
u'\ub010': 'HangulSyllableGgwin',
u'\u9493': 'CjkUnifiedIdeograph9493',
u'\u7312': 'CjkUnifiedIdeograph7312',
u'\u5795': 'CjkUnifiedIdeograph5795',
u'\uc916': 'HangulSyllableJup',
u'\u2d99': 'UnicodeCharacter0x2d99',
u'\ua39d': 'YiSyllableNre',
u'\u021c': 'LatinCapitalLetterYogh',
u'\u669f': 'CjkUnifiedIdeograph669f',
u'\u9820': 'CjkUnifiedIdeograph9820',
u'\ufca3': 'ArabicLigatureTehWithKhahInitialForm',
u'\u5b22': 'CjkUnifiedIdeograph5b22',
u'\ud126': 'HangulSyllableTyaebs',
u'\u35a9': 'CjkUnifiedIdeograph35a9',
u'\u8bad': 'CjkUnifiedIdeograph8bad',
u'\u6a2c': 'CjkUnifiedIdeograph6a2c',
u'\u4eaf': 'CjkUnifiedIdeograph4eaf',
u'\ue030': 'UnicodeCharacter0xe030',
u'\uc4b3': 'HangulSyllableSswec',
u'\u3936': 'CjkUnifiedIdeograph3936',
u'\u1db9': 'ModifierLetterSmallVWithHook',
u'\ubf3a': 'HangulSyllableBbyej',
u'\u93bd': 'CjkUnifiedIdeograph93bd',
u'\u723c': 'CjkUnifiedIdeograph723c',
u'\u56bf': 'CjkUnifiedIdeograph56bf',
u'\uc840': 'HangulSyllableJyeol',
u'\u2cc3': 'CopticSmallLetterCrossedShei',
u'\ua2c7': 'YiSyllableCup',
u'\u0146': 'LatinSmallLetterNWithCedilla',
u'\u65c9': 'CjkUnifiedIdeograph65c9',
u'\u874a': 'CjkUnifiedIdeograph874a',
u'\ufbcd': 'UnicodeCharacter0xfbcd',
u'\u5a4c': 'CjkUnifiedIdeograph5a4c',
u'\ud050': 'HangulSyllableKyu',
u'\u34d3': 'CjkUnifiedIdeograph34d3',
u'\u8ad7': 'CjkUnifiedIdeograph8ad7',
u'\u6956': 'CjkUnifiedIdeograph6956',
u'\u4dd9': 'HexagramForGreatTaming',
u'\uef5a': 'UnicodeCharacter0xef5a',
u'\uc3dd': 'HangulSyllableSsonj',
u'\u3860': 'CjkUnifiedIdeograph3860',
u'\u1ce3': 'UnicodeCharacter0x1ce3',
u'\ube64': 'HangulSyllableBban',
u'\u92e7': 'CjkUnifiedIdeograph92e7',
u'\u7166': 'CjkUnifiedIdeograph7166',
u'\u55e9': 'CjkUnifiedIdeograph55e9',
u'\uf76a': 'UnicodeCharacter0xf76a',
u'\u2bed': 'UnicodeCharacter0x2bed',
u'\ua1f1': 'YiSyllableGep',
u'\u64f3': 'CjkUnifiedIdeograph64f3',
u'\u8674': 'CjkUnifiedIdeograph8674',
u'\ufaf7': 'UnicodeCharacter0xfaf7',
u'\u5976': 'CjkUnifiedIdeograph5976',
u'\udf7a': 'UnicodeCharacter0xdf7a',
u'\u33fd': 'IdeographicTelegraphSymbolForDayThirty',
u'\u0901': 'DevanagariSignCandrabindu',
u'\u6880': 'CjkUnifiedIdeograph6880',
u'\u8f05': 'CjkUnifiedIdeograph8f05',
u'\uee84': 'UnicodeCharacter0xee84',
u'\u4207': 'CjkUnifiedIdeograph4207',
u'\ud80b': 'UnicodeCharacter0xd80b',
u'\u278a': 'DingbatNegativeCircledSansserifDigitOne',
u'\ubd8e': 'HangulSyllableBulp',
u'\u1111': 'HangulChoseongPhieuph',
u'\u7090': 'CjkUnifiedIdeograph7090',
u'\u9715': 'CjkUnifiedIdeograph9715',
u'\uf694': 'UnicodeCharacter0xf694',
u'\u201b': 'SingleHighreversed9QuotationMark',
u'\u0f9a': 'TibetanSubjoinedLetterTta',
u'\ua61f': 'VaiSymbolJong',
u'\u859e': 'CjkUnifiedIdeograph859e',
u'\u7921': 'CjkUnifiedIdeograph7921',
u'\u58a0': 'CjkUnifiedIdeograph58a0',
u'\uff25': 'FullwidthLatinCapitalLetterE',
u'\udea4': 'UnicodeCharacter0xdea4',
u'\u082b': 'UnicodeCharacter0x82b',
u'\u17aa': 'KhmerIndependentVowelQuuv',
u'\u8e2f': 'CjkUnifiedIdeograph8e2f',
u'\uedae': 'UnicodeCharacter0xedae',
u'\u4131': 'CjkUnifiedIdeograph4131',
u'\uc735': 'HangulSyllableYung',
u'\u26b4': 'Pallas',
u'\ubcb8': 'HangulSyllableBek',
u'\u103b': 'MyanmarConsonantSignMedialYa',
u'\u7fba': 'CjkUnifiedIdeograph7fba',
u'\u963f': 'CjkUnifiedIdeograph963f',
u'\uf5be': 'UnicodeCharacter0xf5be',
u'\u2f45': 'KangxiRadicalSquare',
u'\u0ec4': 'LaoVowelSignAi',
u'\ua549': 'VaiSyllableA',
u'\u84c8': 'CjkUnifiedIdeograph84c8',
u'\u784b': 'CjkUnifiedIdeograph784b',
u'\u47ca': 'CjkUnifiedIdeograph47ca',
u'\ufe4f': 'WavyLowLine',
u'\uddce': 'UnicodeCharacter0xddce',
u'\u3755': 'CjkUnifiedIdeograph3755',
u'\u16d4': 'RunicLetterDottedp',
u'\u8d59': 'CjkUnifiedIdeograph8d59',
u'\uecd8': 'UnicodeCharacter0xecd8',
u'\u405b': 'CjkUnifiedIdeograph405b',
u'\uc65f': 'HangulSyllableWaegs',
u'\u25de': 'LowerRightQuadrantCircularArc',
u'\ubbe2': 'HangulSyllableMyinh',
u'\u1f65': 'GreekSmallLetterOmegaWithDasiaAndOxia',
u'\u7ee4': 'CjkUnifiedIdeograph7ee4',
u'\u9569': 'CjkUnifiedIdeograph9569',
u'\uf4e8': 'UnicodeCharacter0xf4e8',
u'\u2e6f': 'UnicodeCharacter0x2e6f',
u'\u0dee': 'UnicodeCharacter0xdee',
u'\ua473': 'YiSyllableYi',
u'\u83f2': 'CjkUnifiedIdeograph83f2',
u'\u6775': 'CjkUnifiedIdeograph6775',
u'\u46f4': 'CjkUnifiedIdeograph46f4',
u'\ufd79': 'ArabicLigatureGhainWithMeemWithMeemFinalForm',
u'\udcf8': 'UnicodeCharacter0xdcf8',
u'\u367f': 'CjkUnifiedIdeograph367f',
u'\u15fe': 'CanadianSyllabicsCarrierKkee',
u'\u6006': 'CjkUnifiedIdeograph6006',
u'\u4489': 'CjkUnifiedIdeograph4489',
u'\u2508': 'BoxDrawingsLightQuadrupleDashHorizontal',
u'\u198b': 'NewTaiLueLetterLowTsa',
u'\ue60a': 'UnicodeCharacter0xe60a',
u'\uda8d': 'UnicodeCharacter0xda8d',
u'\ubb0c': 'HangulSyllableMoem',
u'\u9f8f': 'CjkUnifiedIdeograph9f8f',
u'\u4816': 'CjkUnifiedIdeograph4816',
u'\u0d18': 'MalayalamLetterGha',
u'\u619b': 'CjkUnifiedIdeograph619b',
u'\uce1a': 'HangulSyllableCyuj',
u'\u229d': 'CircledDash',
u'\u831c': 'CjkUnifiedIdeograph831c',
u'\ue79f': 'UnicodeCharacter0xe79f',
u'\ub8a1': 'HangulSyllableRwaelt',
u'\u5026': 'CjkUnifiedIdeograph5026',
u'\u1528': 'CanadianSyllabicsYi',
u'\u49ab': 'CjkUnifiedIdeograph49ab',
u'\ud62a': 'HangulSyllableHyelp',
u'\u0aad': 'GujaratiLetterBha',
u'\ueb2c': 'UnicodeCharacter0xeb2c',
u'\ucfaf': 'HangulSyllableKoed',
u'\uac2e': 'HangulSyllableGaebs',
u'\u80b1': 'CjkUnifiedIdeograph80b1',
u'\u7d38': 'CjkUnifiedIdeograph7d38',
u'\u51bb': 'CjkUnifiedIdeograph51bb',
u'\u3e3a': 'CjkUnifiedIdeograph3e3a',
u'\u12bd': 'EthiopicSyllableKxe',
u'\uf33c': 'UnicodeCharacter0xf33c',
u'\ud7bf': 'UnicodeCharacter0xd7bf',
u'\ub43e': 'HangulSyllableDyolm',
u'\ue8c1': 'UnicodeCharacter0xe8c1',
u'\uadc3': 'HangulSyllableGwigs',
u'\u4548': 'CjkUnifiedIdeograph4548',
u'\u064a': 'ArabicLetterYeh',
u'\u7acd': 'CjkUnifiedIdeograph7acd',
u'\udb4c': 'UnicodeCharacter0xdb4c',
u'\u3fcf': 'CjkUnifiedIdeograph3fcf',
u'\u9c4e': 'CjkUnifiedIdeograph9c4e',
u'\uf0d1': 'UnicodeCharacter0xf0d1',
u'\ub5d3': 'HangulSyllableDdec',
u'\u6e5a': 'CjkUnifiedIdeograph6e5a',
u'\u42dd': 'CjkUnifiedIdeograph42dd',
u'\u235c': 'AplFunctionalSymbolCircleUnderbar',
u'\u07df': 'NkoLetterLa',
u'\ue45e': 'UnicodeCharacter0xe45e',
u'\ud8e1': 'UnicodeCharacter0xd8e1',
u'\ub960': 'HangulSyllableRyul',
u'\u9de3': 'CjkUnifiedIdeograph9de3',
u'\u766a': 'CjkUnifiedIdeograph766a',
u'\u0b6c': 'OriyaDigitSix',
u'\u6fef': 'CjkUnifiedIdeograph6fef',
u'\ucc6e': 'HangulSyllableCyalp',
u'\u20f1': 'UnicodeCharacter0x20f1',
u'\u8170': 'CjkUnifiedIdeograph8170',
u'\ue5f3': 'UnicodeCharacter0xe5f3',
u'\ua6f5': 'UnicodeCharacter0xa6f5',
u'\u5e7a': 'CjkUnifiedIdeograph5e7a',
u'\u137c': 'EthiopicNumberTenThousand',
u'\u77ff': 'CjkUnifiedIdeograph77ff',
u'\ud47e': 'HangulSyllablePunh',
u'\ue980': 'UnicodeCharacter0xe980',
u'\u4d03': 'CjkUnifiedIdeograph4d03',
u'\uaa82': 'UnicodeCharacter0xaa82',
u'\u0e05': 'ThaiCharacterKhoKhon',
u'\uc307': 'HangulSyllableSsalh',
u'\u8409': 'CjkUnifiedIdeograph8409',
u'\u7b8c': 'CjkUnifiedIdeograph7b8c',
u'\u3c8e': 'CjkUnifiedIdeograph3c8e',
u'\uf190': 'UnicodeCharacter0xf190',
u'\u5513': 'CjkUnifiedIdeograph5513',
u'\ub292': 'HangulSyllableNeugg',
u'\u1615': 'CanadianSyllabicsSayisiJu',
u'\u2b17': 'DiamondWithRightHalfBlack',
u'\uec19': 'UnicodeCharacter0xec19',
u'\ua11b': 'YiSyllableTix',
u'\u439c': 'CjkUnifiedIdeograph439c',
u'\u049e': 'CyrillicCapitalLetterKaWithStroke',
u'\ud9a0': 'UnicodeCharacter0xd9a0',
u'\u9aa2': 'CjkUnifiedIdeograph9aa2',
u'\u7e25': 'CjkUnifiedIdeograph7e25',
u'\u3327': 'SquareTon',
u'\uf429': 'UnicodeCharacter0xf429',
u'\u892b': 'CjkUnifiedIdeograph892b',
u'\u6cae': 'CjkUnifiedIdeograph6cae',
u'\u21b0': 'UpwardsArrowWithTipLeftwards',
u'\ue2b2': 'UnicodeCharacter0xe2b2',
u'\u4635': 'CjkUnifiedIdeograph4635',
u'\ua7b4': 'UnicodeCharacter0xa7b4',
u'\u1b37': 'BalineseVowelSignUluSari',
u'\udc39': 'UnicodeCharacter0xdc39',
u'\u913b': 'CjkUnifiedIdeograph913b',
u'\u74be': 'CjkUnifiedIdeograph74be',
u'\u09c0': 'BengaliVowelSignIi',
u'\ucac2': 'HangulSyllableJjonh',
u'\u8fc4': 'CjkUnifiedIdeograph8fc4',
u'\u6347': 'CjkUnifiedIdeograph6347',
u'\u2449': 'OcrCustomerAccountNumber',
u'\uf94b': 'CjkCompatibilityIdeographf94b',
u'\uba4d': 'HangulSyllableMeong',
u'\u5cce': 'CjkUnifiedIdeograph5cce',
u'\u11d0': 'HangulJongseongSsangrieul',
u'\ud2d2': 'HangulSyllableTeup',
u'\u97d4': 'CjkUnifiedIdeograph97d4',
u'\u4b57': 'CjkUnifiedIdeograph4b57',
u'\ua8d6': 'SaurashtraDigitSix',
u'\u0c59': 'TeluguLetterDza',
u'\uc15b': 'HangulSyllableSyeod',
u'\u825d': 'CjkUnifiedIdeograph825d',
u'\u79e0': 'CjkUnifiedIdeograph79e0',
u'\u3ae2': 'CjkUnifiedIdeograph3ae2',
u'\uffe4': 'FullwidthBrokenBar',
u'\u5367': 'CjkUnifiedIdeograph5367',
u'\ub0e6': 'HangulSyllableNyaj',
u'\u1469': 'CanadianSyllabicsTto',
u'\u296b': 'LeftwardsHarpoonWithBarbDownBelowLongDash',
u'\uea6d': 'UnicodeCharacter0xea6d',
u'\uaf6f': 'HangulSyllableGgwaelb',
u'\u41f0': 'CjkUnifiedIdeograph41f0',
u'\u02f2': 'ModifierLetterLowRightArrowhead',
u'\uc7f4': 'HangulSyllableJyaem',
u'\u98f6': 'CjkUnifiedIdeograph98f6',
u'\u7c79': 'CjkUnifiedIdeograph7c79',
u'\u317b': 'HangulLetterSiosnieun',
u'\uf27d': 'UnicodeCharacter0xf27d',
u'\ub77f': 'HangulSyllableRags',
u'\ucb81': 'HangulSyllableJjweg',
u'\u2c04': 'GlagoliticCapitalLetterDobro',
u'\x87': 'UnicodeCharacter0x87',
u'\ua608': 'VaiSyllableMe',
u'\u9a8b': 'CjkUnifiedIdeograph9a8b',
u'\u650a': 'CjkUnifiedIdeograph650a',
u'\u598d': 'CjkUnifiedIdeograph598d',
u'\uff0e': 'FullwidthFullStop',
u'\ud391': 'HangulSyllablePeong',
u'\u3414': 'CjkUnifiedIdeograph3414',
u'\u6897': 'CjkUnifiedIdeograph6897',
u'\u8e18': 'CjkUnifiedIdeograph8e18',
u'\ue29b': 'UnicodeCharacter0xe29b',
u'\u4d1a': 'CjkUnifiedIdeograph4d1a',
u'\uc71e': 'HangulSyllableWip',
u'\u3ba1': 'CjkUnifiedIdeograph3ba1',
u'\ubda5': 'HangulSyllableBweolg',
u'\u1c24': 'LepchaSubjoinedLetterYa',
u'\u70a7': 'CjkUnifiedIdeograph70a7',
u'\u9628': 'CjkUnifiedIdeograph9628',
u'\ucaab': 'HangulSyllableJjyelb',
u'\u552a': 'CjkUnifiedIdeograph552a',
u'\u2f2e': 'KangxiRadicalRiver',
u'\u03b1': 'GreekSmallLetterAlpha',
u'\ua132': 'YiSyllableTu',
u'\u85b5': 'CjkUnifiedIdeograph85b5',
u'\u6434': 'CjkUnifiedIdeograph6434',
u'\u58b7': 'CjkUnifiedIdeograph58b7',
u'\ufe38': 'PresentationFormForVerticalRightCurlyBracket',
u'\ud2bb': 'HangulSyllableTeugs',
u'\u373e': 'CjkUnifiedIdeograph373e',
u'\u6bc1': 'CjkUnifiedIdeograph6bc1',
u'\u8942': 'CjkUnifiedIdeograph8942',
u'\uedc5': 'UnicodeCharacter0xedc5',
u'\u4c44': 'CjkUnifiedIdeograph4c44',
u'\uc648': 'HangulSyllableWal',
u'\u3acb': 'CjkUnifiedIdeograph3acb',
u'\ubccf': 'HangulSyllableByeos',
u'\u1f4e': 'UnicodeCharacter0x1f4e',
u'\u73d1': 'CjkUnifiedIdeograph73d1',
u'\u9152': 'CjkUnifiedIdeograph9152',
u'\uf5d5': 'UnicodeCharacter0xf5d5',
u'\u5454': 'CjkUnifiedIdeograph5454',
u'\u2e58': 'UnicodeCharacter0x2e58',
u'\u02db': 'Ogonek',
u'\ua05c': 'YiSyllableBbie',
u'\u84df': 'CjkUnifiedIdeograph84df',
u'\u675e': 'CjkUnifiedIdeograph675e',
u'\u5be1': 'CjkUnifiedIdeograph5be1',
u'\uf962': 'CjkCompatibilityIdeographf962',
u'\udde5': 'UnicodeCharacter0xdde5',
u'\u3668': 'CjkUnifiedIdeograph3668',
u'\u6aeb': 'CjkUnifiedIdeograph6aeb',
u'\u886c': 'CjkUnifiedIdeograph886c',
u'\uecef': 'UnicodeCharacter0xecef',
u'\u4f6e': 'CjkUnifiedIdeograph4f6e',
u'\uc172': 'HangulSyllableSyegg',
u'\u25f5': 'WhiteCircleWithLowerLeftQuadrant',
u'\ubff9': 'HangulSyllableBbweob',
u'\u1e78': 'LatinCapitalLetterUWithTildeAndAcute',
u'\u72fb': 'CjkUnifiedIdeograph72fb',
u'\u907c': 'CjkUnifiedIdeograph907c',
u'\uf4ff': 'UnicodeCharacter0xf4ff',
u'\u577e': 'CjkUnifiedIdeograph577e',
u'\uce03': 'HangulSyllableCwih',
u'\u2982': 'ZNotationTypeColon',
u'\ua386': 'YiSyllableRre',
u'\u0709': 'SyriacSublinearColonSkewedRight',
u'\u6688': 'CjkUnifiedIdeograph6688',
u'\u990d': 'CjkUnifiedIdeograph990d',
u'\uf88c': 'UnicodeCharacter0xf88c',
u'\u5c0f': 'CjkUnifiedIdeograph5c0f',
u'\ud613': 'HangulSyllableHyeos',
u'\u3192': 'IdeographicAnnotationOneMark',
u'\ua817': 'SylotiNagriLetterDho',
u'\u8b96': 'CjkUnifiedIdeograph8b96',
u'\u6f19': 'CjkUnifiedIdeograph6f19',
u'\u4e98': 'CjkUnifiedIdeograph4e98',
u'\ue11d': 'UnicodeCharacter0xe11d',
u'\uc09c': 'HangulSyllableBbils',
u'\u3e23': 'CjkUnifiedIdeograph3e23',
u'\u19a2': 'NewTaiLueLetterHighBa',
u'\ub027': 'HangulSyllableGgwih',
u'\u93a6': 'CjkUnifiedIdeograph93a6',
u'\u7729': 'CjkUnifiedIdeograph7729',
u'\u56a8': 'CjkUnifiedIdeograph56a8',
u'\uc92d': 'HangulSyllableJweong',
u'\u28ac': 'BraillePatternDots3468',
u'\ua2b0': 'YiSyllableCi',
u'\u0633': 'ArabicLetterSeen',
u'\u61b2': 'CjkUnifiedIdeograph61b2',
u'\u9837': 'CjkUnifiedIdeograph9837',
u'\ufbb6': 'UnicodeCharacter0xfbb6',
u'\u5f39': 'CjkUnifiedIdeograph5f39',
u'\ud13d': 'HangulSyllableTeolt',
u'\u30bc': 'KatakanaLetterZe',
u'\uab41': 'UnicodeCharacter0xab41',
u'\u8ac0': 'CjkUnifiedIdeograph8ac0',
u'\u6e43': 'CjkUnifiedIdeograph6e43',
u'\u49c2': 'CjkUnifiedIdeograph49c2',
u'\ue047': 'UnicodeCharacter0xe047',
u'\uc3c6': 'HangulSyllableSsyelm',
u'\u394d': 'CjkUnifiedIdeograph394d',
u'\u18cc': 'UnicodeCharacter0x18cc',
u'\ub351': 'HangulSyllableDyaet',
u'\u92d0': 'CjkUnifiedIdeograph92d0',
u'\u7653': 'CjkUnifiedIdeograph7653',
u'\u51d2': 'CjkUnifiedIdeograph51d2',
u'\uc857': 'HangulSyllableJyegs',
u'\u2bd6': 'UnicodeCharacter0x2bd6',
u'\uadda': 'HangulSyllableGwip',
u'\u015d': 'LatinSmallLetterSWithCircumflex',
u'\u60dc': 'CjkUnifiedIdeograph60dc',
u'\u9b61': 'CjkUnifiedIdeograph9b61',
u'\ufae0': 'UnicodeCharacter0xfae0',
u'\u5e63': 'CjkUnifiedIdeograph5e63',
u'\ud067': 'HangulSyllableKyuc',
u'\u33e6': 'IdeographicTelegraphSymbolForDaySeven',
u'\uaa6b': 'UnicodeCharacter0xaa6b',
u'\ub5ea': 'HangulSyllableDdyeobs',
u'\u696d': 'CjkUnifiedIdeograph696d',
u'\u48ec': 'CjkUnifiedIdeograph48ec',
u'\ue371': 'UnicodeCharacter0xe371',
u'\uc2f0': 'HangulSyllableSiss',
u'\u3877': 'CjkUnifiedIdeograph3877',
u'\u1bf6': 'UnicodeCharacter0x1bf6',
u'\ub27b': 'HangulSyllableNyud',
u'\u9dfa': 'CjkUnifiedIdeograph9dfa',
u'\u717d': 'CjkUnifiedIdeograph717d',
u'\u50fc': 'CjkUnifiedIdeograph50fc',
u'\u4a81': 'CjkUnifiedIdeograph4a81',
u'\u2b00': 'NorthEastWhiteArrow',
u'\u0f83': 'TibetanSignSnaLdan',
u'\ue802': 'UnicodeCharacter0xe802',
u'\ucc85': 'HangulSyllableCyaelg',
u'\uad04': 'HangulSyllableGwal',
u'\u8187': 'CjkUnifiedIdeograph8187',
u'\u7e0e': 'CjkUnifiedIdeograph7e0e',
u'\u5291': 'CjkUnifiedIdeograph5291',
u'\u3310': 'SquareGiga',
u'\u1793': 'KhmerLetterNo',
u'\uf012': 'UnicodeCharacter0xf012',
u'\ud495': 'HangulSyllablePweog',
u'\ub514': 'HangulSyllableDi',
u'\ue997': 'UnicodeCharacter0xe997',
u'\uae99': 'HangulSyllableGgyang',
u'\u461e': 'CjkUnifiedIdeograph461e',
u'\u1b20': 'BalineseLetterDaMurdaMahaprana',
u'\u7fa3': 'CjkUnifiedIdeograph7fa3',
u'\ud822': 'UnicodeCharacter0xd822',
u'\u3ca5': 'CjkUnifiedIdeograph3ca5',
u'\u9d24': 'CjkUnifiedIdeograph9d24',
u'\uf1a7': 'UnicodeCharacter0xf1a7',
u'\ub6a9': 'HangulSyllableDdult',
u'\u6330': 'CjkUnifiedIdeograph6330',
u'\u47b3': 'CjkUnifiedIdeograph47b3',
u'\u2032': 'Prime',
u'\u04b5': 'CyrillicSmallLigatureTeTse',
u'\ue534': 'UnicodeCharacter0xe534',
u'\ud9b7': 'UnicodeCharacter0xd9b7',
u'\uba36': 'HangulSyllableMyaep',
u'\u9eb9': 'CjkUnifiedIdeograph9eb9',
u'\u4b40': 'CjkUnifiedIdeograph4b40',
u'\u0842': 'UnicodeCharacter0x842',
u'\u6cc5': 'CjkUnifiedIdeograph6cc5',
u'\ucd44': 'HangulSyllableCwaen',
u'\u21c7': 'LeftwardsPairedArrows',
u'\u8246': 'CjkUnifiedIdeograph8246',
u'\ue6c9': 'UnicodeCharacter0xe6c9',
u'\ubbcb': 'HangulSyllableMeulb',
u'\u5350': 'CjkUnifiedIdeograph5350',
u'\u1052': 'MyanmarLetterVocalicR',
u'\u74d5': 'CjkUnifiedIdeograph74d5',
u'\ud554': 'HangulSyllablePik',
u'\u09d7': 'BengaliAuLengthMark',
u'\uea56': 'UnicodeCharacter0xea56',
u'\uced9': 'HangulSyllableKyaeb',
u'\uaf58': 'HangulSyllableGgwam',
u'\u83db': 'CjkUnifiedIdeograph83db',
u'\u7862': 'CjkUnifiedIdeograph7862',
u'\u5ce5': 'CjkUnifiedIdeograph5ce5',
u'\u3d64': 'CjkUnifiedIdeograph3d64',
u'\u11e7': 'HangulJongseongSioskiyeok',
u'\uf266': 'UnicodeCharacter0xf266',
u'\ud6e9': 'HangulSyllableHweolg',
u'\ub768': 'HangulSyllableDdil',
u'\uebeb': 'UnicodeCharacter0xebeb',
u'\ua8ed': 'UnicodeCharacter0xa8ed',
u'\u4072': 'CjkUnifiedIdeograph4072',
u'\u0574': 'ArmenianSmallLetterMen',
u'\u79f7': 'CjkUnifiedIdeograph79f7',
u'\uda76': 'UnicodeCharacter0xda76',
u'\u3ef9': 'CjkUnifiedIdeograph3ef9',
u'\u9f78': 'CjkUnifiedIdeograph9f78',
u'\uf3fb': 'UnicodeCharacter0xf3fb',
u'\ub0fd': 'HangulSyllableNyaeb',
u'\u8a01': 'CjkUnifiedIdeograph8a01',
u'\u6d84': 'CjkUnifiedIdeograph6d84',
u'\u2286': 'SubsetOfOrEqualTo',
u'\ue788': 'UnicodeCharacter0xe788',
u'\u5b0b': 'CjkUnifiedIdeograph5b0b',
u'\ua48a': 'YiSyllableYyp',
u'\u180d': 'MongolianFreeVariationSelectorThree',
u'\udd0f': 'UnicodeCharacter0xdd0f',
u'\u9211': 'CjkUnifiedIdeograph9211',
u'\u7594': 'CjkUnifiedIdeograph7594',
u'\u0a96': 'GujaratiLetterKha',
u'\ucf98': 'HangulSyllableKwaels',
u'\u8c9a': 'CjkUnifiedIdeograph8c9a',
u'\u601d': 'CjkUnifiedIdeograph601d',
u'\u251f': 'BoxDrawingsDownHeavyAndRightUpLight',
u'\ufa21': 'CjkCompatibilityIdeographfa21',
u'\ubf23': 'HangulSyllableBbyeoh',
u'\u5da4': 'CjkUnifiedIdeograph5da4',
u'\u12a6': 'EthiopicSyllableGlottalO',
u'\ud7a8': 'UnicodeCharacter0xd7a8',
u'\u94aa': 'CjkUnifiedIdeograph94aa',
u'\u482d': 'CjkUnifiedIdeograph482d',
u'\ua9ac': 'UnicodeCharacter0xa9ac',
u'\u0d2f': 'MalayalamLetterYa',
u'\uc231': 'HangulSyllableSut',
u'\u8733': 'CjkUnifiedIdeograph8733',
u'\u7ab6': 'CjkUnifiedIdeograph7ab6',
u'\u3fb8': 'CjkUnifiedIdeograph3fb8',
u'\ufcba': 'ArabicLigatureAinWithJeemInitialForm',
u'\u503d': 'CjkUnifiedIdeograph503d',
u'\ub1bc': 'HangulSyllableNwaels',
u'\u153f': 'CanadianSyllabicsBiblecreeY',
u'\u2a41': 'UnionWithMinusSign',
u'\uef43': 'UnicodeCharacter0xef43',
u'\uac45': 'HangulSyllableGyalt',
u'\u42c6': 'CjkUnifiedIdeograph42c6',
u'\u07c8': 'NkoDigitEight',
u'\uc4ca': 'HangulSyllableSswibs',
u'\u99cc': 'CjkUnifiedIdeograph99cc',
u'\u7d4f': 'CjkUnifiedIdeograph7d4f',
u'\u3251': 'CircledNumberTwentyOne',
u'\uf753': 'UnicodeCharacter0xf753',
u'\ub455': 'HangulSyllableDunj',
u'\u6fd8': 'CjkUnifiedIdeograph6fd8',
u'\u2cda': 'CopticCapitalLetterOldCopticShima',
u'\ue1dc': 'UnicodeCharacter0xe1dc',
u'\u455f': 'CjkUnifiedIdeograph455f',
u'\ua6de': 'UnicodeCharacter0xa6de',
u'\u1a61': 'UnicodeCharacter0x1a61',
u'\udf63': 'UnicodeCharacter0xdf63',
u'\u9c65': 'CjkUnifiedIdeograph9c65',
u'\u77e8': 'CjkUnifiedIdeograph77e8',
u'\u34ea': 'CjkUnifiedIdeograph34ea',
u'\uc9ec': 'HangulSyllableJjam',
u'\u8eee': 'CjkUnifiedIdeograph8eee',
u'\u6271': 'CjkUnifiedIdeograph6271',
u'\u2773': 'LightRightTortoiseShellBracketOrnament',
u'\ue475': 'UnicodeCharacter0xe475',
u'\ub977': 'HangulSyllableReugs',
u'\u5ff8': 'CjkUnifiedIdeograph5ff8',
u'\u1cfa': 'UnicodeCharacter0x1cfa',
u'\ud1fc': 'HangulSyllableToel',
u'\u96fe': 'CjkUnifiedIdeograph96fe',
u'\uac00': 'HangulSyllableGa',
u'\u8883': 'CjkUnifiedIdeograph8883',
u'\u6f02': 'CjkUnifiedIdeograph6f02',
u'\u4b85': 'CjkUnifiedIdeograph4b85',
u'\ue506': 'UnicodeCharacter0xe506',
u'\uc189': 'HangulSyllableSyet',
u'\u3e0c': 'CjkUnifiedIdeograph3e0c',
u'\u1a8f': 'UnicodeCharacter0x1a8f',
u'\ub410': 'HangulSyllableDwaess',
u'\u9093': 'CjkUnifiedIdeograph9093',
u'\u7712': 'CjkUnifiedIdeograph7712',
u'\u5395': 'CjkUnifiedIdeograph5395',
u'\ucd16': 'HangulSyllableColp',
u'\u2999': 'DottedFence',
u'\ua79d': 'UnicodeCharacter0xa79d',
u'\u061c': 'UnicodeCharacter0x61c',
u'\u629f': 'CjkUnifiedIdeograph629f',
u'\u9c20': 'CjkUnifiedIdeograph9c20',
u'\uf8a3': 'UnicodeCharacter0xf8a3',
u'\u5f22': 'CjkUnifiedIdeograph5f22',
u'\ud526': 'HangulSyllablePyinh',
u'\u31a9': 'BopomofoLetterAnn',
u'\u8fad': 'CjkUnifiedIdeograph8fad',
u'\u6e2c': 'CjkUnifiedIdeograph6e2c',
u'\u4aaf': 'CjkUnifiedIdeograph4aaf',
u'\ue430': 'UnicodeCharacter0xe430',
u'\uc0b3': 'HangulSyllableSad',
u'\u3d36': 'CjkUnifiedIdeograph3d36',
u'\u19b9': 'NewTaiLueVowelSignUe',
u'\ubb3a': 'HangulSyllableMunh',
u'\u97bd': 'CjkUnifiedIdeograph97bd',
u'\u763c': 'CjkUnifiedIdeograph763c',
u'\u52bf': 'CjkUnifiedIdeograph52bf',
u'\ucc40': 'HangulSyllableCak',
u'\u28c3': 'BraillePatternDots1278',
u'\ua6c7': 'UnicodeCharacter0xa6c7',
u'\u0546': 'ArmenianCapitalLetterNow',
u'\u61c9': 'CjkUnifiedIdeograph61c9',
u'\u834a': 'CjkUnifiedIdeograph834a',
u'\uffcd': 'HalfwidthHangulLetterWa',
u'\u5e4c': 'CjkUnifiedIdeograph5e4c',
u'\ud450': 'HangulSyllablePoem',
u'\u30d3': 'KatakanaLetterBi',
u'\u8ed7': 'CjkUnifiedIdeograph8ed7',
u'\u6d56': 'CjkUnifiedIdeograph6d56',
u'\u49d9': 'CjkUnifiedIdeograph49d9',
u'\ueb5a': 'UnicodeCharacter0xeb5a',
u'\uc7dd': 'HangulSyllableJyang',
u'\u3c60': 'CjkUnifiedIdeograph3c60',
u'\u18e3': 'UnicodeCharacter0x18e3',
u'\uba64': 'HangulSyllableMem',
u'\u96e7': 'CjkUnifiedIdeograph96e7',
u'\u7566': 'CjkUnifiedIdeograph7566',
u'\u51e9': 'CjkUnifiedIdeograph51e9',
u'\uf36a': 'UnicodeCharacter0xf36a',
u'\u2fed': 'UnicodeCharacter0x2fed',
u'\ua5f1': 'VaiSyllableFe',
u'\u0470': 'CyrillicCapitalLetterPsi',
u'\u60f3': 'CjkUnifiedIdeograph60f3',
u'\u8274': 'CjkUnifiedIdeograph8274',
u'\ufef7': 'ArabicLigatureLamWithAlefWithHamzaAboveIsolatedForm',
u'\u5d76': 'CjkUnifiedIdeograph5d76',
u'\udb7a': 'UnicodeCharacter0xdb7a',
u'\u37fd': 'CjkUnifiedIdeograph37fd',
u'\u0d01': 'UnicodeCharacter0xd01',
u'\u6c80': 'CjkUnifiedIdeograph6c80',
u'\u8b05': 'CjkUnifiedIdeograph8b05',
u'\uea84': 'UnicodeCharacter0xea84',
u'\u4607': 'CjkUnifiedIdeograph4607',
u'\udc0b': 'UnicodeCharacter0xdc0b',
u'\u238a': 'CircledTriangleDown',
u'\ub98e': 'HangulSyllableReup',
u'\u1511': 'CanadianSyllabicsShi',
u'\u7490': 'CjkUnifiedIdeograph7490',
u'\u9315': 'CjkUnifiedIdeograph9315',
u'\uf294': 'UnicodeCharacter0xf294',
u'\u241b': 'SymbolForEscape',
u'\u0b9a': 'TamilLetterCa',
u'\ua21f': 'YiSyllableGguox',
u'\u819e': 'CjkUnifiedIdeograph819e',
u'\u7d21': 'CjkUnifiedIdeograph7d21',
u'\u5ca0': 'CjkUnifiedIdeograph5ca0',
u'\ufb25': 'HebrewLetterWideLamed',
u'\udaa4': 'UnicodeCharacter0xdaa4',
u'\u0c2b': 'TeluguLetterPha',
u'\u13aa': 'CherokeeLetterGo',
u'\u8a2f': 'CjkUnifiedIdeograph8a2f',
u'\ue9ae': 'UnicodeCharacter0xe9ae',
u'\u4531': 'CjkUnifiedIdeograph4531',
u'\uc335': 'HangulSyllableSsyanj',
u'\u22b4': 'NormalSubgroupOfOrEqualTo',
u'\ub8b8': 'HangulSyllableRoel',
u'\u143b': 'CanadianSyllabicsWestcreePwe',
u'\u7bba': 'CjkUnifiedIdeograph7bba',
u'\u923f': 'CjkUnifiedIdeograph923f',
u'\uf1be': 'UnicodeCharacter0xf1be',
u'\u2b45': 'LeftwardsQuadrupleArrow',
u'\u0ac4': 'GujaratiVowelSignVocalicRr',
u'\ua149': 'YiSyllableDde',
u'\u80c8': 'CjkUnifiedIdeograph80c8',
u'\u7c4b': 'CjkUnifiedIdeograph7c4b',
u'\u43ca': 'CjkUnifiedIdeograph43ca',
u'\ufa4f': 'CjkCompatibilityIdeographfa4f',
u'\ud9ce': 'UnicodeCharacter0xd9ce',
u'\u3355': 'SquareRemu',
u'\u12d4': 'EthiopicSyllablePharyngealEe',
u'\u8959': 'CjkUnifiedIdeograph8959',
u'\ue8d8': 'UnicodeCharacter0xe8d8',
u'\u445b': 'CjkUnifiedIdeograph445b',
u'\uc25f': 'HangulSyllableSwelh',
u'\u21de': 'UpwardsArrowWithDoubleStroke',
u'\ubfe2': 'HangulSyllableBbuj',
u'\u1b65': 'BalineseMusicalSymbolDangSurang',
u'\u7ae4': 'CjkUnifiedIdeograph7ae4',
u'\u9169': 'CjkUnifiedIdeograph9169',
u'\uf0e8': 'UnicodeCharacter0xf0e8',
u'\u2a6f': 'AlmostEqualToWithCircumflexAccent',
u'\u09ee': 'BengaliDigitEight',
u'\ua073': 'YiSyllableBbyx',
u'\u87f2': 'CjkUnifiedIdeograph87f2',
u'\u6375': 'CjkUnifiedIdeograph6375',
u'\u42f4': 'CjkUnifiedIdeograph42f4',
u'\uf979': 'CjkCompatibilityIdeographf979',
u'\ud8f8': 'UnicodeCharacter0xd8f8',
u'\u327f': 'KoreanStandardSymbol',
u'\u11fe': 'UnicodeCharacter0x11fe',
u'\u6406': 'CjkUnifiedIdeograph6406',
u'\u4089': 'CjkUnifiedIdeograph4089',
u'\u2108': 'Scruple',
u'\u1d8b': 'LatinSmallLetterEshWithPalatalHook',
u'\ue20a': 'UnicodeCharacter0xe20a',
u'\ude8d': 'UnicodeCharacter0xde8d',
u'\ubf0c': 'HangulSyllableBbyeon',
u'\u9b8f': 'CjkUnifiedIdeograph9b8f',
u'\u4c16': 'CjkUnifiedIdeograph4c16',
u'\u0918': 'DevanagariLetterGha',
u'\u659b': 'CjkUnifiedIdeograph659b',
u'\uca1a': 'HangulSyllableJjyanh',
u'\u269d': 'OutlinedWhiteStar',
u'\u871c': 'CjkUnifiedIdeograph871c',
u'\ue39f': 'UnicodeCharacter0xe39f',
u'\ubca1': 'HangulSyllableBeg',
u'\u5426': 'CjkUnifiedIdeograph5426',
u'\u1128': 'HangulChoseongPieupchieuch',
u'\u4dab': 'CjkUnifiedIdeograph4dab',
u'\ud22a': 'HangulSyllableTyop',
u'\u0ead': 'LaoLetterO',
u'\uef2c': 'UnicodeCharacter0xef2c',
u'\ucbaf': 'HangulSyllableJjwis',
u'\ua82e': 'UnicodeCharacter0xa82e',
u'\u84b1': 'CjkUnifiedIdeograph84b1',
u'\u7938': 'CjkUnifiedIdeograph7938',
u'\u55bb': 'CjkUnifiedIdeograph55bb',
u'\u3a3a': 'CjkUnifiedIdeograph3a3a',
u'\u16bd': 'RunicLetterShorttwighagallH',
u'\uf73c': 'UnicodeCharacter0xf73c',
u'\ud3bf': 'HangulSyllablePyeolb',
u'\ub03e': 'HangulSyllableGgyuj',
u'\uecc1': 'UnicodeCharacter0xecc1',
u'\ua9c3': 'UnicodeCharacter0xa9c3',
u'\u4148': 'CjkUnifiedIdeograph4148',
u'\u024a': 'LatinCapitalLetterSmallQWithHookTail',
u'\u7ecd': 'CjkUnifiedIdeograph7ecd',
u'\udf4c': 'UnicodeCharacter0xdf4c',
u'\u3bcf': 'CjkUnifiedIdeograph3bcf',
u'\u984e': 'CjkUnifiedIdeograph984e',
u'\uf4d1': 'UnicodeCharacter0xf4d1',
u'\ub1d3': 'HangulSyllableNoed',
u'\u6a5a': 'CjkUnifiedIdeograph6a5a',
u'\u46dd': 'CjkUnifiedIdeograph46dd',
u'\u275c': 'HeavySingleCommaQuotationMarkOrnament',
u'\u03df': 'GreekSmallLetterKoppa',
u'\ue05e': 'UnicodeCharacter0xe05e',
u'\udce1': 'UnicodeCharacter0xdce1',
u'\ubd60': 'HangulSyllableBoek',
u'\u99e3': 'CjkUnifiedIdeograph99e3',
u'\u726a': 'CjkUnifiedIdeograph726a',
u'\u0f6c': 'TibetanLetterRra',
u'\u6bef': 'CjkUnifiedIdeograph6bef',
u'\uc86e': 'HangulSyllableJyep',
u'\u24f1': 'NegativeCircledNumberSeventeen',
u'\u8570': 'CjkUnifiedIdeograph8570',
u'\ue1f3': 'UnicodeCharacter0xe1f3',
u'\ua2f5': 'YiSyllableNzax',
u'\u5a7a': 'CjkUnifiedIdeograph5a7a',
u'\u177c': 'UnicodeCharacter0x177c',
u'\u73ff': 'CjkUnifiedIdeograph73ff',
u'\ud07e': 'HangulSyllableKeubs',
u'\ued80': 'UnicodeCharacter0xed80',
u'\u4903': 'CjkUnifiedIdeograph4903',
u'\uae82': 'HangulSyllableGgaep',
u'\u0a05': 'GurmukhiLetterA',
u'\uc707': 'HangulSyllableWigs',
u'\u8009': 'CjkUnifiedIdeograph8009',
u'\u7f8c': 'CjkUnifiedIdeograph7f8c',
u'\u388e': 'CjkUnifiedIdeograph388e',
u'\uf590': 'UnicodeCharacter0xf590',
u'\u5113': 'CjkUnifiedIdeograph5113',
u'\ub692': 'HangulSyllableDdyobs',
u'\u1215': 'EthiopicSyllableHhe',
u'\u2f17': 'KangxiRadicalTen',
u'\ue819': 'UnicodeCharacter0xe819',
u'\ua51b': 'VaiSyllableJee',
u'\u479c': 'CjkUnifiedIdeograph479c',
u'\x9e': 'UnicodeCharacter0x9e',
u'\udda0': 'UnicodeCharacter0xdda0',
u'\u9ea2': 'CjkUnifiedIdeograph9ea2',
u'\u7a25': 'CjkUnifiedIdeograph7a25',
u'\u3727': 'CjkUnifiedIdeograph3727',
u'\uf029': 'UnicodeCharacter0xf029',
u'\u8d2b': 'CjkUnifiedIdeograph8d2b',
u'\u68ae': 'CjkUnifiedIdeograph68ae',
u'\u25b0': 'BlackParallelogram',
u'\ue6b2': 'UnicodeCharacter0xe6b2',
u'\u4235': 'CjkUnifiedIdeograph4235',
u'\ua3b4': 'YiSyllableSho',
u'\u1f37': 'GreekSmallLetterIotaWithDasiaAndPerispomeni',
u'\ud839': 'UnicodeCharacter0xd839',
u'\u953b': 'CjkUnifiedIdeograph953b',
u'\u70be': 'CjkUnifiedIdeograph70be',
u'\u0dc0': 'SinhalaLetterVayanna',
u'\ucec2': 'HangulSyllableKyaj',
u'\u8bc4': 'CjkUnifiedIdeograph8bc4',
u'\u6747': 'CjkUnifiedIdeograph6747',
u'\u2049': 'ExclamationQuestionMark',
u'\ufd4b': 'UnicodeCharacter0xfd4b',
u'\ube4d': 'HangulSyllableBilg',
u'\u58ce': 'CjkUnifiedIdeograph58ce',
u'\u15d0': 'CanadianSyllabicsCarrierWu',
u'\ud6d2': 'HangulSyllableHulp',
u'\u93d4': 'CjkUnifiedIdeograph93d4',
u'\u4f57': 'CjkUnifiedIdeograph4f57',
u'\uacd6': 'HangulSyllableGyebs',
u'\u0859': 'UnicodeCharacter0x859',
u'\uc55b': 'HangulSyllableAc',
u'\u865d': 'CjkUnifiedIdeograph865d',
u'\u7de0': 'CjkUnifiedIdeograph7de0',
u'\u3ee2': 'CjkUnifiedIdeograph3ee2',
u'\ufbe4': 'ArabicLetterEIsolatedForm',
u'\u5767': 'CjkUnifiedIdeograph5767',
u'\ub4e6': 'HangulSyllableDeulm',
u'\u1069': 'MyanmarSignWesternPwoKarenTone1',
u'\u2d6b': 'UnicodeCharacter0x2d6b',
u'\uee6d': 'UnicodeCharacter0xee6d',
u'\uab6f': 'UnicodeCharacter0xab6f',
u'\u45f0': 'CjkUnifiedIdeograph45f0',
u'\u06f2': 'ExtendedArabicindicDigitTwo',
u'\uc3f4': 'HangulSyllableSswa',
u'\u9cf6': 'CjkUnifiedIdeograph9cf6',
u'\u7879': 'CjkUnifiedIdeograph7879',
u'\u357b': 'CjkUnifiedIdeograph357b',
u'\uf67d': 'UnicodeCharacter0xf67d',
u'\ub37f': 'HangulSyllableDelh',
u'\ucf81': 'HangulSyllableKwab',
u'\u2804': 'BraillePatternDots3',
u'\u0487': 'CombiningCyrillicPokrytie',
u'\ua208': 'YiSyllableKo',
u'\u9e8b': 'CjkUnifiedIdeograph9e8b',
u'\u610a': 'CjkUnifiedIdeograph610a',
u'\u5d8d': 'CjkUnifiedIdeograph5d8d',
u'\ufb0e': 'UnicodeCharacter0xfb0e',
u'\ud791': 'HangulSyllableHilg',
u'\u3014': 'LeftTortoiseShellBracket',
u'\u6c97': 'CjkUnifiedIdeograph6c97',
u'\u8a18': 'CjkUnifiedIdeograph8a18',
u'\ue69b': 'UnicodeCharacter0xe69b',
u'\u491a': 'CjkUnifiedIdeograph491a',
u'\uc31e': 'HangulSyllableSsaelm',
u'\u3fa1': 'CjkUnifiedIdeograph3fa1',
u'\ub9a5': 'HangulSyllableRying',
u'\u1824': 'MongolianLetterU',
u'\u74a7': 'CjkUnifiedIdeograph74a7',
u'\u9228': 'CjkUnifiedIdeograph9228',
u'\uceab': 'HangulSyllableKaeh',
u'\u512a': 'CjkUnifiedIdeograph512a',
u'\u2b2e': 'BlackVerticalEllipse',
u'\u07b1': 'ThaanaLetterNaa',
u'\ua532': 'VaiSyllableVi',
u'\u81b5': 'CjkUnifiedIdeograph81b5',
u'\u6034': 'CjkUnifiedIdeograph6034',
u'\u5cb7': 'CjkUnifiedIdeograph5cb7',
u'\ufa38': 'CjkCompatibilityIdeographfa38',
u'\ud6bb': 'HangulSyllableHyos',
u'\u333e': 'SquareBoruto',
u'\u6fc1': 'CjkUnifiedIdeograph6fc1',
u'\u8d42': 'CjkUnifiedIdeograph8d42',
u'\ue9c5': 'UnicodeCharacter0xe9c5',
u'\u4844': 'CjkUnifiedIdeograph4844',
u'\uc248': 'HangulSyllableSweoss',
u'\u3ecb': 'CjkUnifiedIdeograph3ecb',
u'\ub8cf': 'HangulSyllableRyogs',
u'\u1b4e': 'UnicodeCharacter0x1b4e',
u'\u77d1': 'CjkUnifiedIdeograph77d1',
u'\u9552': 'CjkUnifiedIdeograph9552',
u'\uf1d5': 'UnicodeCharacter0xf1d5',
u'\u5054': 'CjkUnifiedIdeograph5054',
u'\u2a58': 'SlopingLargeAnd',
u'\u06db': 'ArabicSmallHighThreeDots',
u'\ua45c': 'YiSyllableNyup',
u'\u80df': 'CjkUnifiedIdeograph80df',
u'\u635e': 'CjkUnifiedIdeograph635e',
u'\u5fe1': 'CjkUnifiedIdeograph5fe1',
u'\ufd62': 'ArabicLigatureSeenWithMeemWithMeemFinalForm',
u'\ud9e5': 'UnicodeCharacter0xd9e5',
u'\u3268': 'CircledHangulCieuc',
u'\u6eeb': 'CjkUnifiedIdeograph6eeb',
u'\u8c6c': 'CjkUnifiedIdeograph8c6c',
u'\ue8ef': 'UnicodeCharacter0xe8ef',
u'\u4b6e': 'CjkUnifiedIdeograph4b6e',
u'\uc572': 'HangulSyllableAebs',
u'\u21f5': 'DownwardsArrowLeftwardsOfUpwardsArrow',
u'\ubbf9': 'HangulSyllableMig',
u'\u1a78': 'UnicodeCharacter0x1a78',
u'\u76fb': 'CjkUnifiedIdeograph76fb',
u'\u947c': 'CjkUnifiedIdeograph947c',
u'\uf0ff': 'UnicodeCharacter0xf0ff',
u'\u537e': 'CjkUnifiedIdeograph537e',
u'\uca03': 'HangulSyllableJjaelb',
u'\u2d82': 'EthiopicSyllableRoa',
u'\ua786': 'LatinCapitalLetterInsularT',
u'\u0309': 'CombiningHookAbove',
u'\u6288': 'CjkUnifiedIdeograph6288',
u'\u9d0d': 'CjkUnifiedIdeograph9d0d',
u'\ufc8c': 'ArabicLigatureNoonWithMeemFinalForm',
u'\u580f': 'CjkUnifiedIdeograph580f',
u'\ud213': 'HangulSyllableTyogs',
u'\u3592': 'CjkUnifiedIdeograph3592',
u'\uac17': 'HangulSyllableGac',
u'\u8f96': 'CjkUnifiedIdeograph8f96',
u'\u6b19': 'CjkUnifiedIdeograph6b19',
u'\u4a98': 'CjkUnifiedIdeograph4a98',
u'\ue51d': 'UnicodeCharacter0xe51d',
u'\uc49c': 'HangulSyllableSswe',
u'\u3a23': 'CjkUnifiedIdeograph3a23',
u'\u1da2': 'ModifierLetterSmallScriptG',
u'\ub427': 'HangulSyllableDoelh',
u'\u97a6': 'CjkUnifiedIdeograph97a6',
u'\u7329': 'CjkUnifiedIdeograph7329',
u'\u52a8': 'CjkUnifiedIdeograph52a8',
u'\ucd2d': 'HangulSyllableCwalg',
u'\u2cac': 'CopticCapitalLetterKhi',
u'\ua6b0': 'UnicodeCharacter0xa6b0',
u'\u0233': 'LatinSmallLetterYWithMacron',
u'\u65b2': 'CjkUnifiedIdeograph65b2',
u'\u9c37': 'CjkUnifiedIdeograph9c37',
u'\uffb6': 'HalfwidthHangulLetterSsangsios',
u'\u5b39': 'CjkUnifiedIdeograph5b39',
u'\ud53d': 'HangulSyllablePig',
u'\u34bc': 'CjkUnifiedIdeograph34bc',
u'\uaf41': 'HangulSyllableGgong',
u'\u8ec0': 'CjkUnifiedIdeograph8ec0',
u'\u6a43': 'CjkUnifiedIdeograph6a43',
u'\u4dc2': 'HexagramForDifficultyAtTheBeginning',
u'\ue447': 'UnicodeCharacter0xe447',
u'\uc7c6': 'HangulSyllableJaep',
u'\u3d4d': 'CjkUnifiedIdeograph3d4d',
u'\u1ccc': 'UnicodeCharacter0x1ccc',
u'\ub751': 'HangulSyllableDdyilt',
u'\u96d0': 'CjkUnifiedIdeograph96d0',
u'\u7253': 'CjkUnifiedIdeograph7253',
u'\u55d2': 'CjkUnifiedIdeograph55d2',
u'\ucc57': 'HangulSyllableCaes',
u'\u2fd6': 'UnicodeCharacter0x2fd6',
u'\ua9da': 'UnicodeCharacter0xa9da',
u'\u055d': 'ArmenianComma',
u'\u64dc': 'CjkUnifiedIdeograph64dc',
u'\u9f61': 'CjkUnifiedIdeograph9f61',
u'\ufee0': 'ArabicLetterLamMedialForm',
u'\u5a63': 'CjkUnifiedIdeograph5a63',
u'\ud467': 'HangulSyllablePyolb',
u'\u37e6': 'CjkUnifiedIdeograph37e6',
u'\uae6b': 'HangulSyllableGgaegs',
u'\ub1ea': 'HangulSyllableNyogg',
u'\u6d6d': 'CjkUnifiedIdeograph6d6d',
u'\u4cec': 'CjkUnifiedIdeograph4cec',
u'\ue771': 'UnicodeCharacter0xe771',
u'\uc6f0': 'HangulSyllableWel',
u'\u3c77': 'CjkUnifiedIdeograph3c77',
u'\u1ff6': 'GreekSmallLetterOmegaWithPerispomeni',
u'\ub67b': 'HangulSyllableDdoec',
u'\u99fa': 'CjkUnifiedIdeograph99fa',
u'\u757d': 'CjkUnifiedIdeograph757d',
u'\u54fc': 'CjkUnifiedIdeograph54fc',
u'\u4e81': 'CjkUnifiedIdeograph4e81',
u'\u2f00': 'KangxiRadicalOne',
u'\u0b83': 'TamilSignVisarga',
u'\uec02': 'UnicodeCharacter0xec02',
u'\uc885': 'HangulSyllableJong',
u'\ua904': 'KayahLiDigitFour',
u'\u8587': 'CjkUnifiedIdeograph8587',
u'\u7a0e': 'CjkUnifiedIdeograph7a0e',
u'\u5691': 'CjkUnifiedIdeograph5691',
u'\u3710': 'CjkUnifiedIdeograph3710',
u'\u1393': 'EthiopicTonalMarkShortRikrik',
u'\uf412': 'UnicodeCharacter0xf412',
u'\ud095': 'HangulSyllableKyilt',
u'\ub114': 'HangulSyllableNeols',
u'\ued97': 'UnicodeCharacter0xed97',
u'\uaa99': 'UnicodeCharacter0xaa99',
u'\u421e': 'CjkUnifiedIdeograph421e',
u'\u1f20': 'GreekSmallLetterEtaWithPsili',
u'\u7ba3': 'CjkUnifiedIdeograph7ba3',
u'\udc22': 'UnicodeCharacter0xdc22',
u'\u38a5': 'CjkUnifiedIdeograph38a5',
u'\u9924': 'CjkUnifiedIdeograph9924',
u'\uf5a7': 'UnicodeCharacter0xf5a7',
u'\ub2a9': 'HangulSyllableNeut',
u'\u6730': 'CjkUnifiedIdeograph6730',
u'\u43b3': 'CjkUnifiedIdeograph43b3',
u'\u2432': 'UnicodeCharacter0x2432',
u'\xb5': 'MicroSign',
u'\ue134': 'UnicodeCharacter0xe134',
u'\uddb7': 'UnicodeCharacter0xddb7',
u'\ube36': 'HangulSyllableByilp',
u'\u9ab9': 'CjkUnifiedIdeograph9ab9',
u'\u4f40': 'CjkUnifiedIdeograph4f40',
u'\u0c42': 'TeluguVowelSignUu',
u'\u68c5': 'CjkUnifiedIdeograph68c5',
u'\uc944': 'HangulSyllableJwem',
u'\u25c7': 'WhiteDiamond',
u'\u8646': 'CjkUnifiedIdeograph8646',
u'\ue2c9': 'UnicodeCharacter0xe2c9',
u'\ubfcb': 'HangulSyllableBbyoh',
u'\u5750': 'CjkUnifiedIdeograph5750',
u'\u1452': 'CanadianSyllabicsYcreeToo',
u'\u70d5': 'CjkUnifiedIdeograph70d5',
u'\ud154': 'HangulSyllableTel',
u'\u0dd7': 'UnicodeCharacter0xdd7',
u'\uee56': 'UnicodeCharacter0xee56',
u'\ucad9': 'HangulSyllableJjwag',
u'\uab58': 'UnicodeCharacter0xab58',
u'\u87db': 'CjkUnifiedIdeograph87db',
u'\u7c62': 'CjkUnifiedIdeograph7c62',
u'\u58e5': 'CjkUnifiedIdeograph58e5',
u'\u3964': 'CjkUnifiedIdeograph3964',
u'\u15e7': 'CanadianSyllabicsCarrierTta',
u'\uf666': 'UnicodeCharacter0xf666',
u'\ud2e9': 'HangulSyllableTying',
u'\ub368': 'HangulSyllableDeoss',
u'\uefeb': 'UnicodeCharacter0xefeb',
u'\uaced': 'HangulSyllableGolt',
u'\u4472': 'CjkUnifiedIdeograph4472',
u'\u0174': 'LatinCapitalLetterWWithCircumflex',
u'\u7df7': 'CjkUnifiedIdeograph7df7',
u'\ude76': 'UnicodeCharacter0xde76',
u'\u3af9': 'CjkUnifiedIdeograph3af9',
u'\u9b78': 'CjkUnifiedIdeograph9b78',
u'\uf7fb': 'UnicodeCharacter0xf7fb',
u'\ub4fd': 'HangulSyllableDyinj',
u'\u8e01': 'CjkUnifiedIdeograph8e01',
u'\u6984': 'CjkUnifiedIdeograph6984',
u'\u2686': 'WhiteCircleWithDotRight',
u'\ue388': 'UnicodeCharacter0xe388',
u'\u5f0b': 'CjkUnifiedIdeograph5f0b',
u'\ua08a': 'YiSyllableNbur',
u'\u1c0d': 'LepchaLetterNa',
u'\ud90f': 'UnicodeCharacter0xd90f',
u'\u9611': 'CjkUnifiedIdeograph9611',
u'\u7194': 'CjkUnifiedIdeograph7194',
u'\u0e96': 'LaoLetterThoSung',
u'\ucb98': 'HangulSyllableJjwek',
u'\u889a': 'CjkUnifiedIdeograph889a',
u'\u641d': 'CjkUnifiedIdeograph641d',
u'\u211f': 'Response',
u'\ufe21': 'CombiningLigatureRightHalf',
u'\ubb23': 'HangulSyllableMyolb',
u'\u59a4': 'CjkUnifiedIdeograph59a4',
u'\u16a6': 'RunicLetterThurisazThursThorn',
u'\ud3a8': 'HangulSyllablePem',
u'\u90aa': 'CjkUnifiedIdeograph90aa',
u'\u4c2d': 'CjkUnifiedIdeograph4c2d',
u'\uadac': 'HangulSyllableGwel',
u'\u092f': 'DevanagariLetterYa',
u'\uc631': 'HangulSyllableOlt',
u'\u8333': 'CjkUnifiedIdeograph8333',
u'\u7eb6': 'CjkUnifiedIdeograph7eb6',
u'\u3bb8': 'CjkUnifiedIdeograph3bb8',
u'\uf8ba': 'UnicodeCharacter0xf8ba',
u'\u543d': 'CjkUnifiedIdeograph543d',
u'\ub5bc': 'HangulSyllableDde',
u'\u113f': 'HangulChoseongCeongchieumssangsios',
u'\u2e41': 'UnicodeCharacter0x2e41',
u'\ueb43': 'UnicodeCharacter0xeb43',
u'\ua845': 'PhagspaLetterCha',
u'\u46c6': 'CjkUnifiedIdeograph46c6',
u'\u03c8': 'GreekSmallLetterPsi',
u'\uc0ca': 'HangulSyllableSaegg',
u'\u9dcc': 'CjkUnifiedIdeograph9dcc',
u'\u794f': 'CjkUnifiedIdeograph794f',
u'\u3651': 'CjkUnifiedIdeograph3651',
u'\uf353': 'UnicodeCharacter0xf353',
u'\ub055': 'HangulSyllableGgeub',
u'\u6bd8': 'CjkUnifiedIdeograph6bd8',
u'\u28da': 'BraillePatternDots24578',
u'\ue5dc': 'UnicodeCharacter0xe5dc',
u'\u415f': 'CjkUnifiedIdeograph415f',
u'\ua2de': 'YiSyllableZzop',
u'\u1e61': 'LatinSmallLetterSWithDotAbove',
u'\udb63': 'UnicodeCharacter0xdb63',
u'\u9865': 'CjkUnifiedIdeograph9865',
u'\u73e8': 'CjkUnifiedIdeograph73e8',
u'\u30ea': 'KatakanaLetterRi',
u'\ucdec': 'HangulSyllableCwin',
u'\u8aee': 'CjkUnifiedIdeograph8aee',
u'\u6671': 'CjkUnifiedIdeograph6671',
u'\u2373': 'AplFunctionalSymbolIota',
u'\ue075': 'UnicodeCharacter0xe075',
u'\ubd77': 'HangulSyllableByos',
u'\u5bf8': 'CjkUnifiedIdeograph5bf8',
u'\u18fa': 'UnicodeCharacter0x18fa',
u'\ud5fc': 'HangulSyllableHek',
u'\u92fe': 'CjkUnifiedIdeograph92fe',
u'\ub000': 'HangulSyllableGgwem',
u'\u9483': 'CjkUnifiedIdeograph9483',
u'\u7302': 'CjkUnifiedIdeograph7302',
u'\u5785': 'CjkUnifiedIdeograph5785',
u'\uf906': 'CjkCompatibilityIdeographf906',
u'\udd89': 'UnicodeCharacter0xdd89',
u'\u220c': 'DoesNotContainAsMember',
u'\u068f': 'ArabicLetterDalWithThreeDotsAboveDownwards',
u'\ua810': 'SylotiNagriLetterTto',
u'\u8c93': 'CjkUnifiedIdeograph8c93',
u'\u6b12': 'CjkUnifiedIdeograph6b12',
u'\u4f95': 'CjkUnifiedIdeograph4f95',
u'\ud116': 'HangulSyllableTyaegg',
u'\u3599': 'CjkUnifiedIdeograph3599',
u'\ubb9d': 'HangulSyllableMwing',
u'\u1a1c': 'UnicodeCharacter0x1a1c',
u'\u7e9f': 'CjkUnifiedIdeograph7e9f',
u'\u8020': 'CjkUnifiedIdeograph8020',
u'\ue4a3': 'UnicodeCharacter0xe4a3',
u'\u4322': 'CjkUnifiedIdeograph4322',
u'\uc926': 'HangulSyllableJweolp',
u'\u2da9': 'EthiopicSyllableCcu',
u'\u93ad': 'CjkUnifiedIdeograph93ad',
u'\u722c': 'CjkUnifiedIdeograph722c',
u'\u56af': 'CjkUnifiedIdeograph56af',
u'\uf830': 'UnicodeCharacter0xf830',
u'\udcb3': 'UnicodeCharacter0xdcb3',
u'\u2136': 'BetSymbol',
u'\u05b9': 'HebrewPointHolam',
u'\ua73a': 'LatinCapitalLetterAvWithHorizontalBar',
u'\u8bbd': 'CjkUnifiedIdeograph8bbd',
u'\u6a3c': 'CjkUnifiedIdeograph6a3c',
u'\u4ebf': 'CjkUnifiedIdeograph4ebf',
u'\ud040': 'HangulSyllableKwils',
u'\u34c3': 'CjkUnifiedIdeograph34c3',
u'\ubac7': 'HangulSyllableMwags',
u'\u1946': 'LimbuDigitZero',
u'\u7dc9': 'CjkUnifiedIdeograph7dc9',
u'\u9f4a': 'CjkUnifiedIdeograph9f4a',
u'\ue3cd': 'UnicodeCharacter0xe3cd',
u'\u424c': 'CjkUnifiedIdeograph424c',
u'\uc850': 'HangulSyllableJyeok',
u'\u2cd3': 'CopticSmallLetterOldCopticHei',
u'\u92d7': 'CjkUnifiedIdeograph92d7',
u'\u7156': 'CjkUnifiedIdeograph7156',
u'\u55d9': 'CjkUnifiedIdeograph55d9',
u'\uf75a': 'UnicodeCharacter0xf75a',
u'\udbdd': 'UnicodeCharacter0xdbdd',
u'\u2060': 'WordJoiner',
u'\u04e3': 'CyrillicSmallLetterIWithMacron',
u'\ua664': 'CyrillicCapitalLetterSoftEl',
u'\u8ae7': 'CjkUnifiedIdeograph8ae7',
u'\u6966': 'CjkUnifiedIdeograph6966',
u'\u4de9': 'HexagramForIncrease',
u'\uef6a': 'UnicodeCharacter0xef6a',
u'\u33ed': 'IdeographicTelegraphSymbolForDayFourteen',
u'\ub9f1': 'HangulSyllableMaelt',
u'\u1870': 'MongolianLetterSibeRaa',
u'\u7cf3': 'CjkUnifiedIdeograph7cf3',
u'\u9e74': 'CjkUnifiedIdeograph9e74',
u'\ue2f7': 'UnicodeCharacter0xe2f7',
u'\u4176': 'CjkUnifiedIdeograph4176',
u'\uc77a': 'HangulSyllableInh',
u'\u2bfd': 'UnicodeCharacter0x2bfd',
u'\u1101': 'HangulChoseongSsangkiyeok',
u'\u7080': 'CjkUnifiedIdeograph7080',
u'\u9705': 'CjkUnifiedIdeograph9705',
u'\uf684': 'UnicodeCharacter0xf684',
u'\u5a07': 'CjkUnifiedIdeograph5a07',
u'\uc00b': 'HangulSyllableBbwed',
u'\u3f8a': 'CjkUnifiedIdeograph3f8a',
u'\ua58e': 'VaiSyllableYoo',
u'\u0911': 'DevanagariLetterCandraO',
u'\u6890': 'CjkUnifiedIdeograph6890',
u'\u8f15': 'CjkUnifiedIdeograph8f15',
u'\uee94': 'UnicodeCharacter0xee94',
u'\u381b': 'CjkUnifiedIdeograph381b',
u'\u179a': 'KhmerLetterRo',
u'\ube1f': 'HangulSyllableBeus',
u'\u9d9e': 'CjkUnifiedIdeograph9d9e',
u'\u6121': 'CjkUnifiedIdeograph6121',
u'\u40a0': 'CjkUnifiedIdeograph40a0',
u'\ue725': 'UnicodeCharacter0xe725',
u'\uc6a4': 'HangulSyllableYom',
u'\u102b': 'MyanmarVowelSignTallAa',
u'\u0faa': 'TibetanSubjoinedLetterTsha',
u'\u962f': 'CjkUnifiedIdeograph962f',
u'\uf5ae': 'UnicodeCharacter0xf5ae',
u'\u5931': 'CjkUnifiedIdeograph5931',
u'\udf35': 'UnicodeCharacter0xdf35',
u'\u3eb4': 'CjkUnifiedIdeograph3eb4',
u'\ua4b8': 'YiRadicalWo',
u'\u083b': 'UnicodeCharacter0x83b',
u'\u67ba': 'CjkUnifiedIdeograph67ba',
u'\u8e3f': 'CjkUnifiedIdeograph8e3f',
u'\uedbe': 'UnicodeCharacter0xedbe',
u'\u3745': 'CjkUnifiedIdeograph3745',
u'\u16c4': 'RunicLetterGer',
u'\ubd49': 'HangulSyllableBoeg',
u'\u9cc8': 'CjkUnifiedIdeograph9cc8',
u'\u604b': 'CjkUnifiedIdeograph604b',
u'\u5fca': 'CjkUnifiedIdeograph5fca',
u'\ue64f': 'UnicodeCharacter0xe64f',
u'\uc5ce': 'HangulSyllableEop',
u'\u2f55': 'KangxiRadicalFire',
u'\u0ed4': 'LaoDigitFour',
u'\u9559': 'CjkUnifiedIdeograph9559',
u'\uf4d8': 'UnicodeCharacter0xf4d8',
u'\u585b': 'CjkUnifiedIdeograph585b',
u'\ude5f': 'UnicodeCharacter0xde5f',
u'\u3dde': 'CjkUnifiedIdeograph3dde',
u'\ua3e2': 'YiSyllableJi',
u'\u0765': 'ArabicLetterMeemWithDotAbove',
u'\u66e4': 'CjkUnifiedIdeograph66e4',
u'\u8d69': 'CjkUnifiedIdeograph8d69',
u'\uece8': 'UnicodeCharacter0xece8',
u'\u366f': 'CjkUnifiedIdeograph366f',
u'\u15ee': 'CanadianSyllabicsCarrierP',
u'\ubc73': 'HangulSyllableByaelb',
u'\u9bf2': 'CjkUnifiedIdeograph9bf2',
u'\u7f75': 'CjkUnifiedIdeograph7f75',
u'\u5ef4': 'CjkUnifiedIdeograph5ef4',
u'\ue579': 'UnicodeCharacter0xe579',
u'\uc4f8': 'HangulSyllableSseul',
u'\u2e7f': 'UnicodeCharacter0x2e7f',
u'\u0dfe': 'UnicodeCharacter0xdfe',
u'\u7806': 'CjkUnifiedIdeograph7806',
u'\u5c89': 'CjkUnifiedIdeograph5c89',
u'\u3d08': 'CjkUnifiedIdeograph3d08',
u'\u018b': 'LatinCapitalLetterDWithTopbar',
u'\ufe0a': 'VariationSelector11',
u'\uc28d': 'HangulSyllableSyunj',
u'\ua30c': 'YiSyllableSip',
u'\u878f': 'CjkUnifiedIdeograph878f',
u'\u5016': 'CjkUnifiedIdeograph5016',
u'\u1518': 'CanadianSyllabicsWestcreeShwe',
u'\u799b': 'CjkUnifiedIdeograph799b',
u'\ud61a': 'HangulSyllableHyeop',
u'\u3a9d': 'CjkUnifiedIdeograph3a9d',
u'\u9b1c': 'CjkUnifiedIdeograph9b1c',
u'\uff9f': 'HalfwidthKatakanaSemivoicedSoundMark',
u'\ua0a1': 'YiSyllableHmo',
u'\u4826': 'CjkUnifiedIdeograph4826',
u'\u0d28': 'MalayalamLetterNa',
u'\u51ab': 'CjkUnifiedIdeograph51ab',
u'\uce2a': 'HangulSyllableCeulm',
u'\u12ad': 'EthiopicSyllableKe',
u'\uf32c': 'UnicodeCharacter0xf32c',
u'\ud7af': 'UnicodeCharacter0xd7af',
u'\ub42e': 'HangulSyllableDoej',
u'\u98b1': 'CjkUnifiedIdeograph98b1',
u'\u6538': 'CjkUnifiedIdeograph6538',
u'\u49bb': 'CjkUnifiedIdeograph49bb',
u'\u263a': 'WhiteSmilingFace',
u'\u0abd': 'GujaratiSignAvagraha',
u'\ueb3c': 'UnicodeCharacter0xeb3c',
u'\ucfbf': 'HangulSyllableKoec',
u'\uac3e': 'HangulSyllableGyanh',
u'\uf0c1': 'UnicodeCharacter0xf0c1',
u'\ub5c3': 'HangulSyllableDded',
u'\u5d48': 'CjkUnifiedIdeograph5d48',
u'\u1e4a': 'LatinCapitalLetterNWithCircumflexBelow',
u'\u62cd': 'CjkUnifiedIdeograph62cd',
u'\uc34c': 'HangulSyllableSsyae',
u'\u27cf': 'UnicodeCharacter0x27cf',
u'\u844e': 'CjkUnifiedIdeograph844e',
u'\ue8d1': 'UnicodeCharacter0xe8d1',
u'\uadd3': 'HangulSyllableGwis',
u'\u765a': 'CjkUnifiedIdeograph765a',
u'\u5add': 'CjkUnifiedIdeograph5add',
u'\u3b5c': 'CjkUnifiedIdeograph3b5c',
u'\u1fdf': 'GreekDasiaAndPerispomeni',
u'\ufc5e': 'ArabicLigatureShaddaWithDammatanIsolatedForm',
u'\uc0e1': 'HangulSyllableSaet',
u'\ua160': 'YiSyllableNde',
u'\u85e3': 'CjkUnifiedIdeograph85e3',
u'\u6e6a': 'CjkUnifiedIdeograph6e6a',
u'\u136c': 'EthiopicDigitFour',
u'\u77ef': 'CjkUnifiedIdeograph77ef',
u'\ud46e': 'HangulSyllablePyobs',
u'\u38f1': 'CjkUnifiedIdeograph38f1',
u'\u9970': 'CjkUnifiedIdeograph9970',
u'\ufdf3': 'ArabicLigatureAkbarIsolatedForm',
u'\ubef5': 'HangulSyllableBbelg',
u'\u467a': 'CjkUnifiedIdeograph467a',
u'\u0b7c': 'UnicodeCharacter0xb7c',
u'\u6fff': 'CjkUnifiedIdeograph6fff',
u'\ucc7e': 'HangulSyllableCyaegg',
u'\uf180': 'UnicodeCharacter0xf180',
u'\u5503': 'CjkUnifiedIdeograph5503',
u'\ub282': 'HangulSyllableNyulp',
u'\u1605': 'CanadianSyllabicsCarrierNee',
u'\udb07': 'UnicodeCharacter0xdb07',
u'\u9c09': 'CjkUnifiedIdeograph9c09',
u'\u638c': 'CjkUnifiedIdeograph638c',
u'\u248e': 'DigitSevenFullStop',
u'\ue990': 'UnicodeCharacter0xe990',
u'\u4d13': 'CjkUnifiedIdeograph4d13',
u'\uaa92': 'UnicodeCharacter0xaa92',
u'\u0e15': 'ThaiCharacterToTao',
u'\u3317': 'SquareKirowatto',
u'\uf419': 'UnicodeCharacter0xf419',
u'\ub91b': 'HangulSyllableRweoc',
u'\u5b9c': 'CjkUnifiedIdeograph5b9c',
u'\u1c9e': 'UnicodeCharacter0x1c9e',
u'\uc1a0': 'HangulSyllableSoss',
u'\u82a2': 'CjkUnifiedIdeograph82a2',
u'\u6625': 'CjkUnifiedIdeograph6625',
u'\u2b27': 'BlackMediumLozenge',
u'\uec29': 'UnicodeCharacter0xec29',
u'\u912b': 'CjkUnifiedIdeograph912b',
u'\u74ae': 'CjkUnifiedIdeograph74ae',
u'\u39b0': 'CjkUnifiedIdeograph39b0',
u'\ufab2': 'CjkCompatibilityIdeographfab2',
u'\u5e35': 'CjkUnifiedIdeograph5e35',
u'\ubfb4': 'HangulSyllableBbyon',
u'\u0337': 'CombiningShortSolidusOverlay',
u'\uc439': 'HangulSyllableSsoelt',
u'\u893b': 'CjkUnifiedIdeograph893b',
u'\u6cbe': 'CjkUnifiedIdeograph6cbe',
u'\u11c0': 'HangulJongseongThieuth',
u'\ud2c2': 'HangulSyllableTeulm',
u'\u97c4': 'CjkUnifiedIdeograph97c4',
u'\u7b47': 'CjkUnifiedIdeograph7b47',
u'\u3c49': 'CjkUnifiedIdeograph3c49',
u'\ue14b': 'UnicodeCharacter0xe14b',
u'\ua24d': 'YiSyllableHxiep',
u'\u44ce': 'CjkUnifiedIdeograph44ce',
u'\u09d0': 'UnicodeCharacter0x9d0',
u'\ucad2': 'HangulSyllableJjoj',
u'\u8fd4': 'CjkUnifiedIdeograph8fd4',
u'\u5357': 'CjkUnifiedIdeograph5357',
u'\ub0d6': 'HangulSyllableNyanh',
u'\u1459': 'CanadianSyllabicsTwi',
u'\ud95b': 'UnicodeCharacter0xd95b',
u'\u9a5d': 'CjkUnifiedIdeograph9a5d',
u'\u61e0': 'CjkUnifiedIdeograph61e0',
u'\u22e2': 'NotSquareImageOfOrEqualTo',
u'\ue7e4': 'UnicodeCharacter0xe7e4',
u'\u4b67': 'CjkUnifiedIdeograph4b67',
u'\ua8e6': 'UnicodeCharacter0xa8e6',
u'\u0c69': 'TeluguDigitThree',
u'\u316b': 'HangulLetterRieulpieupsios',
u'\uf26d': 'UnicodeCharacter0xf26d',
u'\ub76f': 'HangulSyllableDdilh',
u'\u59f0': 'CjkUnifiedIdeograph59f0',
u'\u1af2': 'UnicodeCharacter0x1af2',
u'\udff4': 'UnicodeCharacter0xdff4',
u'\u80f6': 'CjkUnifiedIdeograph80f6',
u'\u6479': 'CjkUnifiedIdeograph6479',
u'\u297b': 'SupersetAboveLeftwardsArrow',
u'\uea7d': 'UnicodeCharacter0xea7d',
u'\uaf7f': 'HangulSyllableGgwaeh',
u'\ud381': 'HangulSyllablePeonj',
u'\u3404': 'CjkUnifiedIdeograph3404',
u'\u1887': 'MongolianLetterAliGaliA',
u'\ube08': 'HangulSyllableByuk',
u'\u828b': 'CjkUnifiedIdeograph828b',
u'\u7d0a': 'CjkUnifiedIdeograph7d0a',
u'\u418d': 'CjkUnifiedIdeograph418d',
u'\ue70e': 'UnicodeCharacter0xe70e',
u'\ucb91': 'HangulSyllableJjweb',
u'\u2c14': 'GlagoliticCapitalLetterSlovo',
u'\u7097': 'CjkUnifiedIdeograph7097',
u'\u9618': 'CjkUnifiedIdeograph9618',
u'\ufa9b': 'CjkCompatibilityIdeographfa9b',
u'\u551a': 'CjkUnifiedIdeograph551a',
u'\udf1e': 'UnicodeCharacter0xdf1e',
u'\u23a1': 'LeftSquareBracketUpperCorner',
u'\ua5a5': 'VaiSyllableThu',
u'\u0424': 'CyrillicCapitalLetterEf',
u'\u68a7': 'CjkUnifiedIdeograph68a7',
u'\u8e28': 'CjkUnifiedIdeograph8e28',
u'\ud2ab': 'HangulSyllableTyulh',
u'\u4d2a': 'CjkUnifiedIdeograph4d2a',
u'\u372e': 'CjkUnifiedIdeograph372e',
u'\u1bb1': 'SundaneseDigitOne',
u'\ub932': 'HangulSyllableRwebs',
u'\u9db5': 'CjkUnifiedIdeograph9db5',
u'\u7c34': 'CjkUnifiedIdeograph7c34',
u'\u40b7': 'CjkUnifiedIdeograph40b7',
u'\ue638': 'UnicodeCharacter0xe638',
u'\ucabb': 'HangulSyllableJjyeh',
u'\u2f3e': 'KangxiRadicalDoor',
u'\u73c1': 'CjkUnifiedIdeograph73c1',
u'\u9142': 'CjkUnifiedIdeograph9142',
u'\uf5c5': 'UnicodeCharacter0xf5c5',
u'\u5444': 'CjkUnifiedIdeograph5444',
u'\ude48': 'UnicodeCharacter0xde48',
u'\u22cb': 'LeftSemidirectProduct',
u'\ua4cf': 'UnicodeCharacter0xa4cf',
u'\u074e': 'SyriacLetterSogdianKhaph',
u'\u6bd1': 'CjkUnifiedIdeograph6bd1',
u'\u8952': 'CjkUnifiedIdeograph8952',
u'\uedd5': 'UnicodeCharacter0xedd5',
u'\u4c54': 'CjkUnifiedIdeograph4c54',
u'\u3658': 'CjkUnifiedIdeograph3658',
u'\u1adb': 'UnicodeCharacter0x1adb',
u'\ub85c': 'HangulSyllableRo',
u'\u9cdf': 'CjkUnifiedIdeograph9cdf',
u'\u7f5e': 'CjkUnifiedIdeograph7f5e',
u'\u43e1': 'CjkUnifiedIdeograph43e1',
u'\ue162': 'UnicodeCharacter0xe162',
u'\uc5e5': 'HangulSyllableEng',
u'\u2e68': 'UnicodeCharacter0x2e68',
u'\u72eb': 'CjkUnifiedIdeograph72eb',
u'\u906c': 'CjkUnifiedIdeograph906c',
u'\uf4ef': 'UnicodeCharacter0xf4ef',
u'\u576e': 'CjkUnifiedIdeograph576e',
u'\ud972': 'UnicodeCharacter0xd972',
u'\u3df5': 'CjkUnifiedIdeograph3df5',
u'\ua7f9': 'UnicodeCharacter0xa7f9',
u'\u0678': 'ArabicLetterHighHamzaYeh',
u'\u6afb': 'CjkUnifiedIdeograph6afb',
u'\u887c': 'CjkUnifiedIdeograph887c',
u'\uecff': 'UnicodeCharacter0xecff',
u'\u4f7e': 'CjkUnifiedIdeograph4f7e',
u'\ud603': 'HangulSyllableHyeogs',
u'\u3182': 'HangulLetterYesieungsios',
u'\ubb86': 'HangulSyllableMwep',
u'\u1f09': 'GreekCapitalLetterAlphaWithDasia',
u'\u7e88': 'CjkUnifiedIdeograph7e88',
u'\u810d': 'CjkUnifiedIdeograph810d',
u'\ue08c': 'UnicodeCharacter0xe08c',
u'\u440f': 'CjkUnifiedIdeograph440f',
u'\uce13': 'HangulSyllableCyulh',
u'\u2992': 'RightAngleBracketWithDot',
u'\ub017': 'HangulSyllableGgwilb',
u'\u9396': 'CjkUnifiedIdeograph9396',
u'\u7719': 'CjkUnifiedIdeograph7719',
u'\u5698': 'CjkUnifiedIdeograph5698',
u'\uf91d': 'CjkCompatibilityIdeographf91d',
u'\ud89c': 'UnicodeCharacter0xd89c',
u'\u2623': 'BiohazardSign',
u'\u01a2': 'LatinCapitalLetterOi',
u'\ua827': 'SylotiNagriVowelSignOo',
u'\u8ba6': 'CjkUnifiedIdeograph8ba6',
u'\u6f29': 'CjkUnifiedIdeograph6f29',
u'\u4ea8': 'CjkUnifiedIdeograph4ea8',
u'\ud12d': 'HangulSyllableTyaet',
u'\u30ac': 'KatakanaLetterGa',
u'\ubab0': 'HangulSyllableMol',
u'\u1e33': 'LatinSmallLetterKWithDotBelow',
u'\u79b2': 'CjkUnifiedIdeograph79b2',
u'\u8037': 'CjkUnifiedIdeograph8037',
u'\ue3b6': 'UnicodeCharacter0xe3b6',
u'\u4739': 'CjkUnifiedIdeograph4739',
u'\uc93d': 'HangulSyllableJwelg',
u'\u28bc': 'BraillePatternDots34568',
u'\ub341': 'HangulSyllableDyaelg',
u'\u92c0': 'CjkUnifiedIdeograph92c0',
u'\u7643': 'CjkUnifiedIdeograph7643',
u'\u51c2': 'CjkUnifiedIdeograph51c2',
u'\uf847': 'UnicodeCharacter0xf847',
u'\udbc6': 'UnicodeCharacter0xdbc6',
u'\u214d': 'Aktieselskab',
u'\xcc': 'LatinCapitalLetterIWithGrave',
u'\uab51': 'UnicodeCharacter0xab51',
u'\u8ad0': 'CjkUnifiedIdeograph8ad0',
u'\u6e53': 'CjkUnifiedIdeograph6e53',
u'\u49d2': 'CjkUnifiedIdeograph49d2',
u'\ud057': 'HangulSyllableKyud',
u'\u33d6': 'SquareMol',
u'\ub5da': 'HangulSyllableDdyeogg',
u'\u195d': 'TaiLeLetterVa',
u'\u78dc': 'CjkUnifiedIdeograph78dc',
u'\u8361': 'CjkUnifiedIdeograph8361',
u'\ue2e0': 'UnicodeCharacter0xe2e0',
u'\u4663': 'CjkUnifiedIdeograph4663',
u'\uc867': 'HangulSyllableJyes',
u'\u2be6': 'UnicodeCharacter0x2be6',
u'\ub26b': 'HangulSyllableNwis',
u'\uadea': 'HangulSyllableGyulp',
u'\u716d': 'CjkUnifiedIdeograph716d',
u'\u50ec': 'CjkUnifiedIdeograph50ec',
u'\ufb71': 'ArabicLetterPehehMedialForm',
u'\udaf0': 'UnicodeCharacter0xdaf0',
u'\u2077': 'SuperscriptSeven',
u'\u03f6': 'GreekReversedLunateEpsilonSymbol',
u'\uaa7b': 'UnicodeCharacter0xaa7b',
u'\u85fa': 'CjkUnifiedIdeograph85fa',
u'\u697d': 'CjkUnifiedIdeograph697d',
u'\u48fc': 'CjkUnifiedIdeograph48fc',
u'\u5281': 'CjkUnifiedIdeograph5281',
u'\u3300': 'SquareApaato',
u'\u1783': 'KhmerLetterKho',
u'\uf002': 'UnicodeCharacter0xf002',
u'\ud485': 'HangulSyllablePult',
u'\ub504': 'HangulSyllableDyils',
u'\u9987': 'CjkUnifiedIdeograph9987',
u'\u660e': 'CjkUnifiedIdeograph660e',
u'\u4a91': 'CjkUnifiedIdeograph4a91',
u'\u2b10': 'LeftwardsArrowWithTipDownwards',
u'\u0f93': 'TibetanSubjoinedLetterGha',
u'\ue812': 'UnicodeCharacter0xe812',
u'\ucc95': 'HangulSyllableCyaet',
u'\uad14': 'HangulSyllableGwak',
u'\uf197': 'UnicodeCharacter0xf197',
u'\ub699': 'HangulSyllableDdyot',
u'\u5e1e': 'CjkUnifiedIdeograph5e1e',
u'\u0320': 'CombiningMinusSignBelow',
u'\u67a3': 'CjkUnifiedIdeograph67a3',
u'\uc022': 'HangulSyllableBbwigg',
u'\u24a5': 'ParenthesizedLatinSmallLetterJ',
u'\u8524': 'CjkUnifiedIdeograph8524',
u'\ue9a7': 'UnicodeCharacter0xe9a7',
u'\uaea9': 'HangulSyllableGgyaelg',
u'\u7b30': 'CjkUnifiedIdeograph7b30',
u'\u5fb3': 'CjkUnifiedIdeograph5fb3',
u'\u3832': 'CjkUnifiedIdeograph3832',
u'\u1cb5': 'UnicodeCharacter0x1cb5',
u'\ufd34': 'ArabicLigatureSeenWithJeemMedialForm',
u'\uc1b7': 'HangulSyllableSwalh',
u'\ua236': 'YiSyllableMguox',
u'\u86b9': 'CjkUnifiedIdeograph86b9',
u'\u5340': 'CjkUnifiedIdeograph5340',
u'\u1042': 'MyanmarDigitTwo',
u'\u74c5': 'CjkUnifiedIdeograph74c5',
u'\ud544': 'HangulSyllablePil',
u'\u39c7': 'CjkUnifiedIdeograph39c7',
u'\u9a46': 'CjkUnifiedIdeograph9a46',
u'\ufec9': 'ArabicLetterAinIsolatedForm',
u'\ua3cb': 'YiSyllableRuo',
u'\u4b50': 'CjkUnifiedIdeograph4b50',
u'\u0852': 'UnicodeCharacter0x852',
u'\u6cd5': 'CjkUnifiedIdeograph6cd5',
u'\ucd54': 'HangulSyllableCwaess',
u'\u11d7': 'HangulJongseongRieulpansios',
u'\uf256': 'UnicodeCharacter0xf256',
u'\ud6d9': 'HangulSyllableHung',
u'\ub758': 'HangulSyllableDdyiss',
u'\u9bdb': 'CjkUnifiedIdeograph9bdb',
u'\u6062': 'CjkUnifiedIdeograph6062',
u'\u44e5': 'CjkUnifiedIdeograph44e5',
u'\u2564': 'BoxDrawingsDownSingleAndHorizontalDouble',
u'\u09e7': 'BengaliDigitOne',
u'\uea66': 'UnicodeCharacter0xea66',
u'\ucee9': 'HangulSyllableKeonj',
u'\uaf68': 'HangulSyllableGgwaen',
u'\uf3eb': 'UnicodeCharacter0xf3eb',
u'\ub0ed': 'HangulSyllableNyaeg',
u'\u5872': 'CjkUnifiedIdeograph5872',
u'\u1d74': 'LatinSmallLetterSWithMiddleTilde',
u'\u61f7': 'CjkUnifiedIdeograph61f7',
u'\uc276': 'HangulSyllableSwilm',
u'\u26f9': 'UnicodeCharacter0x26f9',
u'\u8778': 'CjkUnifiedIdeograph8778',
u'\uebfb': 'UnicodeCharacter0xebfb',
u'\ua8fd': 'UnicodeCharacter0xa8fd',
u'\u9201': 'CjkUnifiedIdeograph9201',
u'\u7584': 'CjkUnifiedIdeograph7584',
u'\u3a86': 'CjkUnifiedIdeograph3a86',
u'\uff88': 'HalfwidthKatakanaLetterNe',
u'\u430b': 'CjkUnifiedIdeograph430b',
u'\ubc8a': 'HangulSyllableBeonh',
u'\uc50f': 'HangulSyllableSsyigs',
u'\u8a11': 'CjkUnifiedIdeograph8a11',
u'\u6d94': 'CjkUnifiedIdeograph6d94',
u'\u1296': 'EthiopicSyllableNo',
u'\ud798': 'HangulSyllableHim',
u'\u949a': 'CjkUnifiedIdeograph949a',
u'\u781d': 'CjkUnifiedIdeograph781d',
u'\u3d1f': 'CjkUnifiedIdeograph3d1f',
u'\ue221': 'UnicodeCharacter0xe221',
u'\ua723': 'LatinSmallLetterEgyptologicalAlef',
u'\u45a4': 'CjkUnifiedIdeograph45a4',
u'\u0aa6': 'GujaratiLetterDa',
u'\ucfa8': 'HangulSyllableKoe',
u'\u8caa': 'CjkUnifiedIdeograph8caa',
u'\u502d': 'CjkUnifiedIdeograph502d',
u'\ub1ac': 'HangulSyllableNwak',
u'\u152f': 'CanadianSyllabicsYwe',
u'\uda31': 'UnicodeCharacter0xda31',
u'\u9f33': 'CjkUnifiedIdeograph9f33',
u'\u62b6': 'CjkUnifiedIdeograph62b6',
u'\u27b8': 'HeavyBlackfeatheredRightwardsArrow',
u'\ue4ba': 'UnicodeCharacter0xe4ba',
u'\u483d': 'CjkUnifiedIdeograph483d',
u'\ua9bc': 'UnicodeCharacter0xa9bc',
u'\u0d3f': 'MalayalamVowelSignI',
u'\u3241': 'ParenthesizedIdeographRest',
u'\uf743': 'UnicodeCharacter0xf743',
u'\ub445': 'HangulSyllableDyob',
u'\u5ac6': 'CjkUnifiedIdeograph5ac6',
u'\u1fc8': 'GreekCapitalLetterEpsilonWithVaria',
u'\udcca': 'UnicodeCharacter0xdcca',
u'\u81cc': 'CjkUnifiedIdeograph81cc',
u'\u654f': 'CjkUnifiedIdeograph654f',
u'\u2a51': 'LogicalAndWithDotAbove',
u'\uef53': 'UnicodeCharacter0xef53',
u'\uac55': 'HangulSyllableGyaeg',
u'\u77d8': 'CjkUnifiedIdeograph77d8',
u'\u34da': 'CjkUnifiedIdeograph34da',
u'\uf9dc': 'CjkCompatibilityIdeographf9dc',
u'\u5d5f': 'CjkUnifiedIdeograph5d5f',
u'\ubede': 'HangulSyllableBbeolp',
u'\u0261': 'LatinSmallLetterScriptG',
u'\uc763': 'HangulSyllableYilb',
u'\u8465': 'CjkUnifiedIdeograph8465',
u'\u6fe8': 'CjkUnifiedIdeograph6fe8',
u'\u2cea': 'CopticSymbolShimaSima',
u'\ud1ec': 'HangulSyllableTwaess',
u'\u96ee': 'CjkUnifiedIdeograph96ee',
u'\u7a71': 'CjkUnifiedIdeograph7a71',
u'\u3f73': 'CjkUnifiedIdeograph3f73',
u'\ufc75': 'ArabicLigatureTehWithYehFinalForm',
u'\ua177': 'YiSyllableHnox',
u'\u47f8': 'CjkUnifiedIdeograph47f8',
u'\u04fa': 'CyrillicCapitalLetterGheWithStrokeAndHook',
u'\uc9fc': 'HangulSyllableJjaen',
u'\u8efe': 'CjkUnifiedIdeograph8efe',
u'\ub400': 'HangulSyllableDwaen',
u'\u9083': 'CjkUnifiedIdeograph9083',
u'\u7702': 'CjkUnifiedIdeograph7702',
u'\u5385': 'CjkUnifiedIdeograph5385',
u'\ufd06': 'ArabicLigatureSadWithYehIsolatedForm',
u'\ud989': 'UnicodeCharacter0xd989',
u'\u260c': 'Conjunction',
u'\u028f': 'LatinLetterSmallCapitalY',
u'\uac10': 'HangulSyllableGam',
u'\u8893': 'CjkUnifiedIdeograph8893',
u'\u6f12': 'CjkUnifiedIdeograph6f12',
u'\u4b95': 'CjkUnifiedIdeograph4b95',
u'\ud516': 'HangulSyllablePeubs',
u'\u3199': 'IdeographicAnnotationFirstMark',
u'\ubf9d': 'HangulSyllableBboelg',
u'\u1e1c': 'LatinCapitalLetterEWithCedillaAndBreve',
u'\u7a9f': 'CjkUnifiedIdeograph7a9f',
u'\u8420': 'CjkUnifiedIdeograph8420',
u'\ue0a3': 'UnicodeCharacter0xe0a3',
u'\u4722': 'CjkUnifiedIdeograph4722',
u'\ucd26': 'HangulSyllableCwagg',
u'\u29a9': 'MeasuredAngleWithOpenArmEndingInArrowPointingUpAndLeft',
u'\u97ad': 'CjkUnifiedIdeograph97ad',
u'\u762c': 'CjkUnifiedIdeograph762c',
u'\u52af': 'CjkUnifiedIdeograph52af',
u'\ufc30': 'ArabicLigatureFehWithMeemIsolatedForm',
u'\ud8b3': 'UnicodeCharacter0xd8b3',
u'\u2536': 'BoxDrawingsRightHeavyAndLeftUpLight',
u'\u01b9': 'LatinSmallLetterEzhReversed',
u'\ua33a': 'YiSyllableSse',
u'\u8fbd': 'CjkUnifiedIdeograph8fbd',
u'\u6e3c': 'CjkUnifiedIdeograph6e3c',
u'\u4abf': 'CjkUnifiedIdeograph4abf',
u'\ud440': 'HangulSyllablePoe',
u'\u30c3': 'KatakanaLetterSmallTu',
u'\ubec7': 'HangulSyllableBbyaes',
u'\u1d46': 'ModifierLetterSmallTurnedAe',
u'\u79c9': 'CjkUnifiedIdeograph79c9',
u'\u9b4a': 'CjkUnifiedIdeograph9b4a',
u'\ue7cd': 'UnicodeCharacter0xe7cd',
u'\u464c': 'CjkUnifiedIdeograph464c',
u'\ucc50': 'HangulSyllableCaels',
u'\u28d3': 'BraillePatternDots12578',
u'\u96d7': 'CjkUnifiedIdeograph96d7',
u'\u7556': 'CjkUnifiedIdeograph7556',
u'\u51d9': 'CjkUnifiedIdeograph51d9',
u'\uf35a': 'UnicodeCharacter0xf35a',
u'\udfdd': 'UnicodeCharacter0xdfdd',
u'\u2460': 'CircledDigitOne',
u'\xe3': 'LatinSmallLetterAWithTilde',
u'\ua264': 'YiSyllableNguot',
u'\u8ee7': 'CjkUnifiedIdeograph8ee7',
u'\u6d66': 'CjkUnifiedIdeograph6d66',
u'\u49e9': 'CjkUnifiedIdeograph49e9',
u'\ueb6a': 'UnicodeCharacter0xeb6a',
u'\u37ed': 'CjkUnifiedIdeograph37ed',
u'\ubdf1': 'HangulSyllableByug',
u'\u1c70': 'OlChikiLetterEdd',
u'\u78f3': 'CjkUnifiedIdeograph78f3',
u'\u9a74': 'CjkUnifiedIdeograph9a74',
u'\ue6f7': 'UnicodeCharacter0xe6f7',
u'\u4576': 'CjkUnifiedIdeograph4576',
u'\uc37a': 'HangulSyllableSseobs',
u'\u2ffd': 'UnicodeCharacter0x2ffd',
u'\u1501': 'CanadianSyllabicsWestcreeSwa',
u'\u7480': 'CjkUnifiedIdeograph7480',
u'\u9305': 'CjkUnifiedIdeograph9305',
u'\uf284': 'UnicodeCharacter0xf284',
u'\u5e07': 'CjkUnifiedIdeograph5e07',
u'\uc40b': 'HangulSyllableSswac',
u'\u3b8a': 'CjkUnifiedIdeograph3b8a',
u'\ua18e': 'YiSyllableNex',
u'\u0d11': 'UnicodeCharacter0xd11',
u'\u6c90': 'CjkUnifiedIdeograph6c90',
u'\u8b15': 'CjkUnifiedIdeograph8b15',
u'\uea94': 'UnicodeCharacter0xea94',
u'\u3c1b': 'CjkUnifiedIdeograph3c1b',
u'\u139a': 'UnicodeCharacter0x139a',
u'\uba1f': 'HangulSyllableMyaegs',
u'\u999e': 'CjkUnifiedIdeograph999e',
u'\u6521': 'CjkUnifiedIdeograph6521',
u'\u44a0': 'CjkUnifiedIdeograph44a0',
u'\ue325': 'UnicodeCharacter0xe325',
u'\uc2a4': 'HangulSyllableSeu',
u'\u142b': 'CanadianSyllabicsEn',
u'\u0baa': 'TamilLetterPa',
u'\u922f': 'CjkUnifiedIdeograph922f',
u'\uf1ae': 'UnicodeCharacter0xf1ae',
u'\u5d31': 'CjkUnifiedIdeograph5d31',
u'\udb35': 'UnicodeCharacter0xdb35',
u'\u3ab4': 'CjkUnifiedIdeograph3ab4',
u'\ua0b8': 'YiSyllableMap',
u'\u0c3b': 'UnicodeCharacter0xc3b',
u'\u63ba': 'CjkUnifiedIdeograph63ba',
u'\u8a3f': 'CjkUnifiedIdeograph8a3f',
u'\ue9be': 'UnicodeCharacter0xe9be',
u'\u3345': 'SquareMahha',
u'\u12c4': 'EthiopicSyllableKxwee',
u'\ub949': 'HangulSyllableRwilt',
u'\u98c8': 'CjkUnifiedIdeograph98c8',
u'\u644b': 'CjkUnifiedIdeograph644b',
u'\u5bca': 'CjkUnifiedIdeograph5bca',
u'\ue24f': 'UnicodeCharacter0xe24f',
u'\uc1ce': 'HangulSyllableSwaelm',
u'\u2b55': 'UnicodeCharacter0x2b55',
u'\u0ad4': 'UnicodeCharacter0xad4',
u'\u9159': 'CjkUnifiedIdeograph9159',
u'\uf0d8': 'UnicodeCharacter0xf0d8',
u'\u5c5b': 'CjkUnifiedIdeograph5c5b',
u'\uda5f': 'UnicodeCharacter0xda5f',
u'\u39de': 'CjkUnifiedIdeograph39de',
u'\ua7e2': 'UnicodeCharacter0xa7e2',
u'\u0365': 'CombiningLatinSmallLetterI',
u'\u62e4': 'CjkUnifiedIdeograph62e4',
u'\u8969': 'CjkUnifiedIdeograph8969',
u'\ue8e8': 'UnicodeCharacter0xe8e8',
u'\u326f': 'CircledHangulNieunA',
u'\u11ee': 'HangulJongseongSsangieung',
u'\ub873': 'HangulSyllableRoc',
u'\u9ff2': 'UnicodeCharacter0x9ff2',
u'\u7b75': 'CjkUnifiedIdeograph7b75',
u'\u5af4': 'CjkUnifiedIdeograph5af4',
u'\ue179': 'UnicodeCharacter0xe179',
u'\uc0f8': 'HangulSyllableSyass',
u'\u2a7f': 'LessthanOrSlantedEqualToWithDotInside',
u'\u09fe': 'UnicodeCharacter0x9fe',
u'\u7c06': 'CjkUnifiedIdeograph7c06',
u'\u5889': 'CjkUnifiedIdeograph5889',
u'\u3908': 'CjkUnifiedIdeograph3908',
u'\u058b': 'UnicodeCharacter0x58b',
u'\ufa0a': 'CjkCompatibilityIdeographfa0a',
u'\uc68d': 'HangulSyllableOeng',
u'\ua70c': 'ModifierLetterExtralowDottedToneBar',
u'\u838f': 'CjkUnifiedIdeograph838f',
u'\u5416': 'CjkUnifiedIdeograph5416',
u'\u1118': 'HangulChoseongRieulnieun',
u'\u7d9b': 'CjkUnifiedIdeograph7d9b',
u'\ud21a': 'HangulSyllableTyolm',
u'\u3e9d': 'CjkUnifiedIdeograph3e9d',
u'\u9f1c': 'CjkUnifiedIdeograph9f1c',
u'\ufb9f': 'ArabicLetterNoonGhunnaFinalForm',
u'\ua4a1': 'YiRadicalGa',
u'\u4c26': 'CjkUnifiedIdeograph4c26',
u'\u0928': 'DevanagariLetterNa',
u'\u55ab': 'CjkUnifiedIdeograph55ab',
u'\uca2a': 'HangulSyllableJjyaj',
u'\u16ad': 'RunicLetterShorttwigossO',
u'\uf72c': 'UnicodeCharacter0xf72c',
u'\ud3af': 'HangulSyllablePec',
u'\ub02e': 'HangulSyllableGgyunh',
u'\u9cb1': 'CjkUnifiedIdeograph9cb1',
u'\u6138': 'CjkUnifiedIdeograph6138',
u'\u4dbb': 'UnicodeCharacter0x4dbb',
u'\u223a': 'GeometricProportion',
u'\u0ebd': 'LaoSemivowelSignNyo',
u'\uef3c': 'UnicodeCharacter0xef3c',
u'\ucbbf': 'HangulSyllableJjyud',
u'\ua83e': 'UnicodeCharacter0xa83e',
u'\uf4c1': 'UnicodeCharacter0xf4c1',
u'\ub1c3': 'HangulSyllableNwaes',
u'\u5948': 'CjkUnifiedIdeograph5948',
u'\u1a4a': 'UnicodeCharacter0x1a4a',
u'\u66cd': 'CjkUnifiedIdeograph66cd',
u'\uc74c': 'HangulSyllableEum',
u'\u23cf': 'EjectSymbol',
u'\u804e': 'CjkUnifiedIdeograph804e',
u'\uecd1': 'UnicodeCharacter0xecd1',
u'\ua9d3': 'UnicodeCharacter0xa9d3',
u'\u725a': 'CjkUnifiedIdeograph725a',
u'\u5edd': 'CjkUnifiedIdeograph5edd',
u'\u3f5c': 'CjkUnifiedIdeograph3f5c',
u'\u1bdf': 'UnicodeCharacter0x1bdf',
u'\uf85e': 'UnicodeCharacter0xf85e',
u'\uc4e1': 'HangulSyllableSsyult',
u'\ua560': 'VaiSyllableDa',
u'\u81e3': 'CjkUnifiedIdeograph81e3',
u'\u6a6a': 'CjkUnifiedIdeograph6a6a',
u'\u176c': 'TagbanwaLetterYa',
u'\u73ef': 'CjkUnifiedIdeograph73ef',
u'\ud06e': 'HangulSyllableKeugg',
u'\u3cf1': 'CjkUnifiedIdeograph3cf1',
u'\u9d70': 'CjkUnifiedIdeograph9d70',
u'\uf9f3': 'CjkCompatibilityIdeographf9f3',
u'\ubaf5': 'HangulSyllableMwaeng',
u'\u427a': 'CjkUnifiedIdeograph427a',
u'\u0f7c': 'TibetanVowelSignO',
u'\u6bff': 'CjkUnifiedIdeograph6bff',
u'\uc87e': 'HangulSyllableJolp',
u'\uf580': 'UnicodeCharacter0xf580',
u'\u5103': 'CjkUnifiedIdeograph5103',
u'\ub682': 'HangulSyllableDdyogg',
u'\u1205': 'EthiopicSyllableHe',
u'\udf07': 'UnicodeCharacter0xdf07',
u'\u9809': 'CjkUnifiedIdeograph9809',
u'\u678c': 'CjkUnifiedIdeograph678c',
u'\u208e': 'SubscriptRightParenthesis',
u'\ued90': 'UnicodeCharacter0xed90',
u'\u4913': 'CjkUnifiedIdeograph4913',
u'\uae92': 'HangulSyllableGgyalp',
u'\u0a15': 'GurmukhiLetterKa',
u'\u3717': 'CjkUnifiedIdeograph3717',
u'\uf019': 'UnicodeCharacter0xf019',
u'\ubd1b': 'HangulSyllableBwalb',
u'\u5f9c': 'CjkUnifiedIdeograph5f9c',
u'\u189e': 'MongolianLetterManchuAliGaliTta',
u'\uc5a0': 'HangulSyllableYael',
u'\u86a2': 'CjkUnifiedIdeograph86a2',
u'\u6225': 'CjkUnifiedIdeograph6225',
u'\u2f27': 'KangxiRadicalRoof',
u'\ue829': 'UnicodeCharacter0xe829',
u'\u952b': 'CjkUnifiedIdeograph952b',
u'\u70ae': 'CjkUnifiedIdeograph70ae',
u'\u3db0': 'CjkUnifiedIdeograph3db0',
u'\ufeb2': 'ArabicLetterSeenFinalForm',
u'\u5a35': 'CjkUnifiedIdeograph5a35',
u'\ubbb4': 'HangulSyllableMyum',
u'\u0737': 'SyriacRbasaBelow',
u'\uc039': 'HangulSyllableBbwit',
u'\u8d3b': 'CjkUnifiedIdeograph8d3b',
u'\u68be': 'CjkUnifiedIdeograph68be',
u'\u15c0': 'CanadianSyllabicsSayisiHe',
u'\ud6c2': 'HangulSyllableHyop',
u'\u93c4': 'CjkUnifiedIdeograph93c4',
u'\u7f47': 'CjkUnifiedIdeograph7f47',
u'\u3849': 'CjkUnifiedIdeograph3849',
u'\ue54b': 'UnicodeCharacter0xe54b',
u'\ua64d': 'CyrillicSmallLetterBroadOmega',
u'\u40ce': 'CjkUnifiedIdeograph40ce',
u'\u0dd0': 'SinhalaVowelSignKettiAedapilla',
u'\uced2': 'HangulSyllableKyaelm',
u'\u8bd4': 'CjkUnifiedIdeograph8bd4',
u'\u5757': 'CjkUnifiedIdeograph5757',
u'\ub4d6': 'HangulSyllableDyuj',
u'\u1059': 'MyanmarVowelSignVocalicLl',
u'\udd5b': 'UnicodeCharacter0xdd5b',
u'\u9e5d': 'CjkUnifiedIdeograph9e5d',
u'\u65e0': 'CjkUnifiedIdeograph65e0',
u'\u26e2': 'UnicodeCharacter0x26e2',
u'\ue3e4': 'UnicodeCharacter0xe3e4',
u'\u4f67': 'CjkUnifiedIdeograph4f67',
u'\uace6': 'HangulSyllableGonh',
u'\u0869': 'UnicodeCharacter0x869',
u'\u356b': 'CjkUnifiedIdeograph356b',
u'\uf66d': 'UnicodeCharacter0xf66d',
u'\ub36f': 'HangulSyllableDeoh',
u'\u5df0': 'CjkUnifiedIdeograph5df0',
u'\u1ef2': 'LatinCapitalLetterYWithGrave',
u'\udbf4': 'UnicodeCharacter0xdbf4',
u'\u84f6': 'CjkUnifiedIdeograph84f6',
u'\u6079': 'CjkUnifiedIdeograph6079',
u'\u2d7b': 'UnicodeCharacter0x2d7b',
u'\uee7d': 'UnicodeCharacter0xee7d',
u'\uab7f': 'UnicodeCharacter0xab7f',
u'\ud781': 'HangulSyllableHying',
u'\u3004': 'JapaneseIndustrialStandardSymbol',
u'\u1c87': 'UnicodeCharacter0x1c87',
u'\uba08': 'HangulSyllableMyal',
u'\u868b': 'CjkUnifiedIdeograph868b',
u'\u790a': 'CjkUnifiedIdeograph790a',
u'\u458d': 'CjkUnifiedIdeograph458d',
u'\ue30e': 'UnicodeCharacter0xe30e',
u'\ucf91': 'HangulSyllableKwaenj',
u'\u2814': 'BraillePatternDots35',
u'\u7497': 'CjkUnifiedIdeograph7497',
u'\u9218': 'CjkUnifiedIdeograph9218',
u'\ufe9b': 'ArabicLetterThehInitialForm',
u'\u511a': 'CjkUnifiedIdeograph511a',
u'\udb1e': 'UnicodeCharacter0xdb1e',
u'\u27a1': 'BlackRightwardsArrow',
u'\ua1a5': 'YiSyllableHlox',
u'\u6ca7': 'CjkUnifiedIdeograph6ca7',
u'\u8a28': 'CjkUnifiedIdeograph8a28',
u'\ud6ab': 'HangulSyllableHyogs',
u'\u492a': 'CjkUnifiedIdeograph492a',
u'\u332e': 'SquarePiasutoru',
u'\u1fb1': 'GreekSmallLetterAlphaWithMacron',
u'\ubd32': 'HangulSyllableBwaenh',
u'\u99b5': 'CjkUnifiedIdeograph99b5',
u'\u7834': 'CjkUnifiedIdeograph7834',
u'\u44b7': 'CjkUnifiedIdeograph44b7',
u'\ue238': 'UnicodeCharacter0xe238',
u'\ucebb': 'HangulSyllableKyalh',
u'\u2b3e': 'LeftwardsArrowThroughX',
u'\u77c1': 'CjkUnifiedIdeograph77c1',
u'\u9542': 'CjkUnifiedIdeograph9542',
u'\uf1c5': 'UnicodeCharacter0xf1c5',
u'\u5044': 'CjkUnifiedIdeograph5044',
u'\uda48': 'UnicodeCharacter0xda48',
u'\u26cb': 'UnicodeCharacter0x26cb',
u'\ua0cf': 'YiSyllableFi',
u'\u034e': 'CombiningUpwardsArrowBelow',
u'\u6fd1': 'CjkUnifiedIdeograph6fd1',
u'\u8d52': 'CjkUnifiedIdeograph8d52',
u'\ue9d5': 'UnicodeCharacter0xe9d5',
u'\u4854': 'CjkUnifiedIdeograph4854',
u'\u3258': 'CircledNumberTwentyEight',
u'\u1edb': 'LatinSmallLetterOWithHornAndAcute',
u'\ubc5c': 'HangulSyllableByam',
u'\u98df': 'CjkUnifiedIdeograph98df',
u'\u7b5e': 'CjkUnifiedIdeograph7b5e',
u'\u47e1': 'CjkUnifiedIdeograph47e1',
u'\ue562': 'UnicodeCharacter0xe562',
u'\uc1e5': 'HangulSyllableSoenj',
u'\u2a68': 'TripleHorizontalBarWithDoubleVerticalStroke',
u'\u76eb': 'CjkUnifiedIdeograph76eb',
u'\u946c': 'CjkUnifiedIdeograph946c',
u'\uf0ef': 'UnicodeCharacter0xf0ef',
u'\u536e': 'CjkUnifiedIdeograph536e',
u'\udd72': 'UnicodeCharacter0xdd72',
u'\u39f5': 'CjkUnifiedIdeograph39f5',
u'\ua3f9': 'YiSyllableJyp',
u'\u0278': 'LatinSmallLetterPhi',
u'\u6efb': 'CjkUnifiedIdeograph6efb',
u'\u8c7c': 'CjkUnifiedIdeograph8c7c',
u'\ue8ff': 'UnicodeCharacter0xe8ff',
u'\u4b7e': 'CjkUnifiedIdeograph4b7e',
u'\ud203': 'HangulSyllableToelh',
u'\u3582': 'CjkUnifiedIdeograph3582',
u'\ubf86': 'HangulSyllableBbwaelp',
u'\u1b09': 'BalineseLetterUkara',
u'\u7a88': 'CjkUnifiedIdeograph7a88',
u'\u850d': 'CjkUnifiedIdeograph850d',
u'\ue48c': 'UnicodeCharacter0xe48c',
u'\u400f': 'CjkUnifiedIdeograph400f',
u'\uca13': 'HangulSyllableJjaeh',
u'\u2d92': 'EthiopicSyllablePoa',
u'\ub417': 'HangulSyllableDwaeh',
u'\u9796': 'CjkUnifiedIdeograph9796',
u'\u7319': 'CjkUnifiedIdeograph7319',
u'\u5298': 'CjkUnifiedIdeograph5298',
u'\ufd1d': 'ArabicLigatureJeemWithAlefMaksuraFinalForm',
u'\udc9c': 'UnicodeCharacter0xdc9c',
u'\u2223': 'Divides',
u'\u05a2': 'HebrewAccentAtnahHafukh',
u'\uac27': 'HangulSyllableGaelb',
u'\u8fa6': 'CjkUnifiedIdeograph8fa6',
u'\u6b29': 'CjkUnifiedIdeograph6b29',
u'\u4aa8': 'CjkUnifiedIdeograph4aa8',
u'\ud52d': 'HangulSyllablePyilt',
u'\u34ac': 'CjkUnifiedIdeograph34ac',
u'\ubeb0': 'HangulSyllableBbyak',
u'\u1a33': 'UnicodeCharacter0x1a33',
u'\u7db2': 'CjkUnifiedIdeograph7db2',
u'\u8437': 'CjkUnifiedIdeograph8437',
u'\ue7b6': 'UnicodeCharacter0xe7b6',
u'\u4339': 'CjkUnifiedIdeograph4339',
u'\ucd3d': 'HangulSyllableCwat',
u'\u2cbc': 'CopticCapitalLetterCryptogrammicNi',
u'\ub741': 'HangulSyllableDdeut',
u'\u96c0': 'CjkUnifiedIdeograph96c0',
u'\u7243': 'CjkUnifiedIdeograph7243',
u'\u55c2': 'CjkUnifiedIdeograph55c2',
u'\ufc47': 'ArabicLigatureMeemWithKhahIsolatedForm',
u'\udfc6': 'UnicodeCharacter0xdfc6',
u'\u254d': 'BoxDrawingsHeavyDoubleDashHorizontal',
u'\u04cc': 'CyrillicSmallLetterKhakassianChe',
u'\uaf51': 'HangulSyllableGgwalg',
u'\u8ed0': 'CjkUnifiedIdeograph8ed0',
u'\u6a53': 'CjkUnifiedIdeograph6a53',
u'\u4dd2': 'HexagramForApproach',
u'\ud457': 'HangulSyllablePoec',
u'\u37d6': 'CjkUnifiedIdeograph37d6',
u'\ub1da': 'HangulSyllableNoelp',
u'\u1d5d': 'ModifierLetterSmallBeta',
u'\u7cdc': 'CjkUnifiedIdeograph7cdc',
u'\u8761': 'CjkUnifiedIdeograph8761',
u'\ue6e0': 'UnicodeCharacter0xe6e0',
u'\u4263': 'CjkUnifiedIdeograph4263',
u'\ucc67': 'HangulSyllableCyad',
u'\u2fe6': 'UnicodeCharacter0x2fe6',
u'\ub66b': 'HangulSyllableDdoed',
u'\ua9ea': 'UnicodeCharacter0xa9ea',
u'\u756d': 'CjkUnifiedIdeograph756d',
u'\u54ec': 'CjkUnifiedIdeograph54ec',
u'\uff71': 'HalfwidthKatakanaLetterA',
u'\udef0': 'UnicodeCharacter0xdef0',
u'\u2477': 'ParenthesizedDigitFour',
u'\u07f6': 'NkoSymbolOoDennen',
u'\uae7b': 'HangulSyllableGgaes',
u'\u81fa': 'CjkUnifiedIdeograph81fa',
u'\u6d7d': 'CjkUnifiedIdeograph6d7d',
u'\u4cfc': 'CjkUnifiedIdeograph4cfc',
u'\u5681': 'CjkUnifiedIdeograph5681',
u'\u3700': 'CjkUnifiedIdeograph3700',
u'\u1383': 'EthiopicSyllableMwe',
u'\uf402': 'UnicodeCharacter0xf402',
u'\ud085': 'HangulSyllableKeut',
u'\ub104': 'HangulSyllableNyaek',
u'\u9d87': 'CjkUnifiedIdeograph9d87',
u'\u620e': 'CjkUnifiedIdeograph620e',
u'\u4e91': 'CjkUnifiedIdeograph4e91',
u'\u2f10': 'KangxiRadicalOpenBox',
u'\u0b93': 'TamilLetterOo',
u'\uec12': 'UnicodeCharacter0xec12',
u'\uc895': 'HangulSyllableJwalg',
u'\ua914': 'KayahLiLetterNa',
u'\uf597': 'UnicodeCharacter0xf597',
u'\ub299': 'HangulSyllableNeulg',
u'\u5a1e': 'CjkUnifiedIdeograph5a1e',
u'\u0720': 'SyriacLetterLamadh',
u'\u63a3': 'CjkUnifiedIdeograph63a3',
u'\uc422': 'HangulSyllableSswaebs',
u'\u20a5': 'MillSign',
u'\u8124': 'CjkUnifiedIdeograph8124',
u'\ueda7': 'UnicodeCharacter0xeda7',
u'\uaaa9': 'UnicodeCharacter0xaaa9',
u'\u7f30': 'CjkUnifiedIdeograph7f30',
u'\u5bb3': 'CjkUnifiedIdeograph5bb3',
u'\u3c32': 'CjkUnifiedIdeograph3c32',
u'\u18b5': 'UnicodeCharacter0x18b5',
u'\uf934': 'CjkCompatibilityIdeographf934',
u'\uc5b7': 'HangulSyllableEogs',
u'\ua636': 'UnicodeCharacter0xa636',
u'\u82b9': 'CjkUnifiedIdeograph82b9',
u'\u5740': 'CjkUnifiedIdeograph5740',
u'\u1442': 'CanadianSyllabicsPwoo',
u'\u70c5': 'CjkUnifiedIdeograph70c5',
u'\ud144': 'HangulSyllableTeoss',
u'\u3dc7': 'CjkUnifiedIdeograph3dc7',
u'\u9e46': 'CjkUnifiedIdeograph9e46',
u'\ufac9': 'CjkCompatibilityIdeographfac9',
u'\ua7cb': 'UnicodeCharacter0xa7cb',
u'\u4f50': 'CjkUnifiedIdeograph4f50',
u'\u0c52': 'UnicodeCharacter0xc52',
u'\u68d5': 'CjkUnifiedIdeograph68d5',
u'\uc954': 'HangulSyllableJwin',
u'\u15d7': 'CanadianSyllabicsCarrierHwo',
u'\uf656': 'UnicodeCharacter0xf656',
u'\ud2d9': 'HangulSyllableTyinj',
u'\ub358': 'HangulSyllableDeon',
u'\u9fdb': 'UnicodeCharacter0x9fdb',
u'\u6462': 'CjkUnifiedIdeograph6462',
u'\u40e5': 'CjkUnifiedIdeograph40e5',
u'\u2164': 'RomanNumeralFive',
u'\u0de7': 'UnicodeCharacter0xde7',
u'\uee66': 'UnicodeCharacter0xee66',
u'\ucae9': 'HangulSyllableJjwab',
u'\uab68': 'UnicodeCharacter0xab68',
u'\uf7eb': 'UnicodeCharacter0xf7eb',
u'\ub4ed': 'HangulSyllableDeub',
u'\u5c72': 'CjkUnifiedIdeograph5c72',
u'\u1974': 'TaiLeLetterTone6',
u'\u65f7': 'CjkUnifiedIdeograph65f7',
u'\uc676': 'HangulSyllableWaep',
u'\u22f9': 'ElementOfWithTwoHorizontalStrokes',
u'\u8378': 'CjkUnifiedIdeograph8378',
u'\ueffb': 'UnicodeCharacter0xeffb',
u'\uacfd': 'HangulSyllableGwag',
u'\u9601': 'CjkUnifiedIdeograph9601',
u'\u7184': 'CjkUnifiedIdeograph7184',
u'\u3e86': 'CjkUnifiedIdeograph3e86',
u'\ufb88': 'ArabicLetterDdalIsolatedForm',
u'\u470b': 'CjkUnifiedIdeograph470b',
u'\ub88a': 'HangulSyllableRwabs',
u'\u040d': 'CyrillicCapitalLetterIWithGrave',
u'\uc10f': 'HangulSyllableSyaelh',
u'\u8e11': 'CjkUnifiedIdeograph8e11',
u'\u6994': 'CjkUnifiedIdeograph6994',
u'\u1696': 'OghamLetterOr',
u'\ud398': 'HangulSyllablePe',
u'\u909a': 'CjkUnifiedIdeograph909a',
u'\u7c1d': 'CjkUnifiedIdeograph7c1d',
u'\u391f': 'CjkUnifiedIdeograph391f',
u'\ue621': 'UnicodeCharacter0xe621',
u'\ua323': 'YiSyllableSur',
u'\u41a4': 'CjkUnifiedIdeograph41a4',
u'\u0ea6': 'UnicodeCharacter0xea6',
u'\ucba8': 'HangulSyllableJjwils',
u'\u88aa': 'CjkUnifiedIdeograph88aa',
u'\u542d': 'CjkUnifiedIdeograph542d',
u'\ub5ac': 'HangulSyllableDdeols',
u'\u112f': 'HangulChoseongSiostikeut',
u'\ude31': 'UnicodeCharacter0xde31',
u'\u9b33': 'CjkUnifiedIdeograph9b33',
u'\u66b6': 'CjkUnifiedIdeograph66b6',
u'\u23b8': 'LeftVerticalBoxLine',
u'\ue0ba': 'UnicodeCharacter0xe0ba',
u'\u4c3d': 'CjkUnifiedIdeograph4c3d',
u'\uadbc': 'HangulSyllableGwek',
u'\u093f': 'DevanagariVowelSignI',
u'\u3641': 'CjkUnifiedIdeograph3641',
u'\uf343': 'UnicodeCharacter0xf343',
u'\ub045': 'HangulSyllableGgeug',
u'\u5ec6': 'CjkUnifiedIdeograph5ec6',
u'\u1bc8': 'UnicodeCharacter0x1bc8',
u'\ud8ca': 'UnicodeCharacter0xd8ca',
u'\u85cc': 'CjkUnifiedIdeograph85cc',
u'\u614f': 'CjkUnifiedIdeograph614f',
u'\u2e51': 'UnicodeCharacter0x2e51',
u'\ueb53': 'UnicodeCharacter0xeb53',
u'\ua855': 'PhagspaLetterZa',
u'\u73d8': 'CjkUnifiedIdeograph73d8',
u'\u30da': 'KatakanaLetterPe',
u'\ufddc': 'UnicodeCharacter0xfddc',
u'\u595f': 'CjkUnifiedIdeograph595f',
u'\ubade': 'HangulSyllableMwap',
u'\u0661': 'ArabicindicDigitOne',
u'\uc363': 'HangulSyllableSsyaec',
u'\u8065': 'CjkUnifiedIdeograph8065',
u'\u6be8': 'CjkUnifiedIdeograph6be8',
u'\u28ea': 'BraillePatternDots24678',
u'\ud5ec': 'HangulSyllableHel',
u'\u92ee': 'CjkUnifiedIdeograph92ee',
u'\u7e71': 'CjkUnifiedIdeograph7e71',
u'\u3b73': 'CjkUnifiedIdeograph3b73',
u'\uf875': 'UnicodeCharacter0xf875',
u'\ua577': 'VaiSyllableBhoo',
u'\u43f8': 'CjkUnifiedIdeograph43f8',
u'\xfa': 'LatinSmallLetterUWithAcute',
u'\ucdfc': 'HangulSyllableCwiss',
u'\u8afe': 'CjkUnifiedIdeograph8afe',
u'\ub800': 'HangulSyllableReoss',
u'\u9c83': 'CjkUnifiedIdeograph9c83',
u'\u7b02': 'CjkUnifiedIdeograph7b02',
u'\u5f85': 'CjkUnifiedIdeograph5f85',
u'\uf106': 'UnicodeCharacter0xf106',
u'\ud589': 'HangulSyllableHaeng',
u'\u2a0c': 'QuadrupleIntegralOperator',
u'\u0e8f': 'UnicodeCharacter0xe8f',
u'\ua010': 'YiSyllableOx',
u'\u8493': 'CjkUnifiedIdeograph8493',
u'\u6312': 'CjkUnifiedIdeograph6312',
u'\u4795': 'CjkUnifiedIdeograph4795',
u'\ud916': 'UnicodeCharacter0xd916',
u'\u3d99': 'CjkUnifiedIdeograph3d99',
u'\ub39d': 'HangulSyllableDyeob',
u'\u121c': 'EthiopicSyllableMee',
u'\u769f': 'CjkUnifiedIdeograph769f',
u'\u8820': 'CjkUnifiedIdeograph8820',
u'\ueca3': 'UnicodeCharacter0xeca3',
u'\u4b22': 'CjkUnifiedIdeograph4b22',
u'\uc126': 'HangulSyllableSeolm',
u'\u25a9': 'SquareWithDiagonalCrosshatchFill',
u'\u9bad': 'CjkUnifiedIdeograph9bad',
u'\u7a2c': 'CjkUnifiedIdeograph7a2c',
u'\u5eaf': 'CjkUnifiedIdeograph5eaf',
u'\uf030': 'UnicodeCharacter0xf030',
u'\ud4b3': 'HangulSyllablePwegs',
u'\u2936': 'ArrowPointingDownwardsThenCurvingLeftwards',
u'\u0db9': 'SinhalaLetterAmbaBayanna',
u'\uaf3a': 'HangulSyllableGgolp',
u'\u83bd': 'CjkUnifiedIdeograph83bd',
u'\u623c': 'CjkUnifiedIdeograph623c',
u'\u46bf': 'CjkUnifiedIdeograph46bf',
u'\ud840': 'UnicodeCharacter0xd840',
u'\u3cc3': 'CjkUnifiedIdeograph3cc3',
u'\ub2c7': 'HangulSyllableNyih',
u'\u1146': 'HangulChoseongIeungpansios',
u'\u75c9': 'CjkUnifiedIdeograph75c9',
u'\u974a': 'CjkUnifiedIdeograph974a',
u'\uebcd': 'UnicodeCharacter0xebcd',
u'\u4a4c': 'CjkUnifiedIdeograph4a4c',
u'\uc050': 'HangulSyllableBbyuss',
u'\u24d3': 'CircledLatinSmallLetterD',
u'\u9ad7': 'CjkUnifiedIdeograph9ad7',
u'\u7956': 'CjkUnifiedIdeograph7956',
u'\u5dd9': 'CjkUnifiedIdeograph5dd9',
u'\uff5a': 'FullwidthLatinSmallLetterZ',
u'\ud3dd': 'HangulSyllablePyelt',
u'\u2860': 'BraillePatternDots67',
u'\u0ce3': 'KannadaVowelSignVocalicLl',
u'\uae64': 'HangulSyllableGgak',
u'\u82e7': 'CjkUnifiedIdeograph82e7',
u'\u6166': 'CjkUnifiedIdeograph6166',
u'\u45e9': 'CjkUnifiedIdeograph45e9',
u'\ue76a': 'UnicodeCharacter0xe76a',
u'\u3bed': 'CjkUnifiedIdeograph3bed',
u'\ub1f1': 'HangulSyllableNyolg',
u'\u1070': 'MyanmarLetterEasternPwoKarenGhwa',
u'\u74f3': 'CjkUnifiedIdeograph74f3',
u'\u9674': 'CjkUnifiedIdeograph9674',
u'\ueaf7': 'UnicodeCharacter0xeaf7',
u'\u4976': 'CjkUnifiedIdeograph4976',
u'\ucf7a': 'HangulSyllableKwalm',
u'\u23fd': 'UnicodeCharacter0x23fd',
u'\u1901': 'LimbuLetterKa',
u'\u7880': 'CjkUnifiedIdeograph7880',
u'\u9f05': 'CjkUnifiedIdeograph9f05',
u'\ufe84': 'ArabicLetterAlefWithHamzaAboveFinalForm',
u'\u5207': 'CjkUnifiedIdeograph5207',
u'\uc80b': 'HangulSyllableJeolb',
u'\u378a': 'CjkUnifiedIdeograph378a',
u'\uad8e': 'HangulSyllableGweonh',
u'\u0111': 'LatinSmallLetterDWithStroke',
u'\u6090': 'CjkUnifiedIdeograph6090',
u'\u8715': 'CjkUnifiedIdeograph8715',
u'\ue694': 'UnicodeCharacter0xe694',
u'\u301b': 'RightWhiteSquareBracket',
u'\u1f9a': 'GreekCapitalLetterEtaWithPsiliAndVariaAndProsgegrammeni',
u'\ub61f': 'HangulSyllableDdolh',
u'\u959e': 'CjkUnifiedIdeograph959e',
u'\u6921': 'CjkUnifiedIdeograph6921',
u'\u48a0': 'CjkUnifiedIdeograph48a0',
u'\uef25': 'UnicodeCharacter0xef25',
u'\ucea4': 'HangulSyllableKaess',
u'\u182b': 'MongolianLetterPa',
u'\u07aa': 'ThaanaUbufili',
u'\u9e2f': 'CjkUnifiedIdeograph9e2f',
u'\ufdae': 'ArabicLigatureYehWithHahWithYehFinalForm',
u'\u5131': 'CjkUnifiedIdeograph5131',
u'\ud735': 'HangulSyllableHyug',
u'\u36b4': 'CjkUnifiedIdeograph36b4',
u'\uacb8': 'HangulSyllableGyeom',
u'\u6fba': 'CjkUnifiedIdeograph6fba',
u'\u863f': 'CjkUnifiedIdeograph863f',
u'\ue5be': 'UnicodeCharacter0xe5be',
u'\u3f45': 'CjkUnifiedIdeograph3f45',
u'\u1ec4': 'LatinCapitalLetterEWithCircumflexAndTilde',
u'\ub549': 'HangulSyllableDdat',
u'\u94c8': 'CjkUnifiedIdeograph94c8',
u'\u684b': 'CjkUnifiedIdeograph684b',
u'\u57ca': 'CjkUnifiedIdeograph57ca',
u'\uee4f': 'UnicodeCharacter0xee4f',
u'\ucdce': 'HangulSyllableCwegg',
u'\u2755': 'UnicodeCharacter0x2755',
u'\u06d4': 'ArabicFullStop',
u'\u9d59': 'CjkUnifiedIdeograph9d59',
u'\ufcd8': 'ArabicLigatureHehWithMeemInitialForm',
u'\u505b': 'CjkUnifiedIdeograph505b',
u'\ud65f': 'HangulSyllableHwalb',
u'\u35de': 'CjkUnifiedIdeograph35de',
u'\uabe2': 'UnicodeCharacter0xabe2',
u'\u0f65': 'TibetanLetterSsa',
u'\u6ee4': 'CjkUnifiedIdeograph6ee4',
u'\u8569': 'CjkUnifiedIdeograph8569',
u'\ue4e8': 'UnicodeCharacter0xe4e8',
u'\u3e6f': 'CjkUnifiedIdeograph3e6f',
u'\u1dee': 'UnicodeCharacter0x1dee',
u'\ub473': 'HangulSyllableDweod',
u'\u93f2': 'CjkUnifiedIdeograph93f2',
u'\u7775': 'CjkUnifiedIdeograph7775',
u'\u56f4': 'CjkUnifiedIdeograph56f4',
u'\ued79': 'UnicodeCharacter0xed79',
u'\uccf8': 'HangulSyllableCyels',
u'\u267f': 'WheelchairSymbol',
u'\u05fe': 'UnicodeCharacter0x5fe',
u'\u7006': 'CjkUnifiedIdeograph7006',
u'\u5489': 'CjkUnifiedIdeograph5489',
u'\u3508': 'CjkUnifiedIdeograph3508',
u'\u098b': 'BengaliLetterVocalicR',
u'\uf60a': 'UnicodeCharacter0xf60a',
u'\uca8d': 'HangulSyllableJjyeolg',
u'\uab0c': 'UnicodeCharacter0xab0c',
u'\u8f8f': 'CjkUnifiedIdeograph8f8f',
u'\u5816': 'CjkUnifiedIdeograph5816',
u'\u1d18': 'LatinLetterSmallCapitalP',
u'\u719b': 'CjkUnifiedIdeograph719b',
u'\ude1a': 'UnicodeCharacter0xde1a',
u'\u329d': 'CircledIdeographExcellent',
u'\u931c': 'CjkUnifiedIdeograph931c',
u'\uf79f': 'UnicodeCharacter0xf79f',
u'\ua8a1': 'SaurashtraLetterTa',
u'\u4026': 'CjkUnifiedIdeograph4026',
u'\u0528': 'UnicodeCharacter0x528',
u'\u59ab': 'CjkUnifiedIdeograph59ab',
u'\uc62a': 'HangulSyllableOnh',
u'\u1aad': 'UnicodeCharacter0x1aad',
u'\ufb2c': 'HebrewLetterShinWithDageshAndShinDot',
u'\udfaf': 'UnicodeCharacter0xdfaf',
u'\ubc2e': 'HangulSyllableBap',
u'\u90b1': 'CjkUnifiedIdeograph90b1',
u'\u6d38': 'CjkUnifiedIdeograph6d38',
u'\u41bb': 'CjkUnifiedIdeograph41bb',
u'\u2e3a': 'UnicodeCharacter0x2e3a',
u'\u02bd': 'ModifierLetterReversedComma',
u'\ue33c': 'UnicodeCharacter0xe33c',
u'\uc7bf': 'HangulSyllableJaes',
u'\ua43e': 'YiSyllableNjop',
u'\uf8c1': 'UnicodeCharacter0xf8c1',
u'\ubdc3': 'HangulSyllableBwelb',
u'\u5548': 'CjkUnifiedIdeograph5548',
u'\u164a': 'CanadianSyllabicsCarrierDze',
u'\u6acd': 'CjkUnifiedIdeograph6acd',
u'\ucb4c': 'HangulSyllableJjun',
u'\u2fcf': 'KangxiRadicalRat',
u'\u8c4e': 'CjkUnifiedIdeograph8c4e',
u'\ue0d1': 'UnicodeCharacter0xe0d1',
u'\ua5d3': 'VaiSyllableSo',
u'\u7e5a': 'CjkUnifiedIdeograph7e5a',
u'\u52dd': 'CjkUnifiedIdeograph52dd',
u'\u335c': 'IdeographicTelegraphSymbolForHourFour',
u'\u17df': 'UnicodeCharacter0x17df',
u'\uf45e': 'UnicodeCharacter0xf45e',
u'\uc8e1': 'HangulSyllableJyog',
u'\ua960': 'UnicodeCharacter0xa960',
u'\u8de3': 'CjkUnifiedIdeograph8de3',
u'\u666a': 'CjkUnifiedIdeograph666a',
u'\u1b6c': 'BalineseMusicalSymbolCombiningEndep',
u'\u7fef': 'CjkUnifiedIdeograph7fef',
u'\udc6e': 'UnicodeCharacter0xdc6e',
u'\u30f1': 'KatakanaLetterWe',
u'\u9170': 'CjkUnifiedIdeograph9170',
u'\uf5f3': 'UnicodeCharacter0xf5f3',
u'\ub6f5': 'HangulSyllableDdwinj',
u'\u4e7a': 'CjkUnifiedIdeograph4e7a',
u'\u037c': 'GreekSmallDottedLunateSigmaSymbol',
u'\u67ff': 'CjkUnifiedIdeograph67ff',
u'\uc47e': 'HangulSyllableSsup',
u'\uf980': 'CjkCompatibilityIdeographf980',
u'\u5d03': 'CjkUnifiedIdeograph5d03',
u'\uba82': 'HangulSyllableMyeobs',
u'\u1e05': 'LatinSmallLetterBWithDotBelow',
u'\ud307': 'HangulSyllableTic',
u'\u9409': 'CjkUnifiedIdeograph9409',
u'\u6b8c': 'CjkUnifiedIdeograph6b8c',
u'\u2c8e': 'CopticCapitalLetterHate',
u'\ue190': 'UnicodeCharacter0xe190',
u'\u4513': 'CjkUnifiedIdeograph4513',
u'\ua292': 'YiSyllableZie',
u'\u0615': 'ArabicSmallHighTah',
u'\u3b17': 'CjkUnifiedIdeograph3b17',
u'\ufc19': 'ArabicLigatureKhahWithJeemIsolatedForm',
u'\ub11b': 'HangulSyllableNeos',
u'\u539c': 'CjkUnifiedIdeograph539c',
u'\u149e': 'CanadianSyllabicsCwaa',
u'\uc9a0': 'HangulSyllableJeuk',
u'\u8aa2': 'CjkUnifiedIdeograph8aa2',
u'\u6e25': 'CjkUnifiedIdeograph6e25',
u'\u2327': 'XInARectangleBox',
u'\ue429': 'UnicodeCharacter0xe429',
u'\u992b': 'CjkUnifiedIdeograph992b',
u'\u7cae': 'CjkUnifiedIdeograph7cae',
u'\u31b0': 'BopomofoLetterAm',
u'\uf2b2': 'UnicodeCharacter0xf2b2',
u'\u5635': 'CjkUnifiedIdeograph5635',
u'\ub7b4': 'HangulSyllableRya',
u'\u0b37': 'OriyaLetterSsa',
u'\ucc39': 'HangulSyllableCab',
u'\u813b': 'CjkUnifiedIdeograph813b',
u'\u64be': 'CjkUnifiedIdeograph64be',
u'\u19c0': 'NewTaiLueVowelSignIy',
u'\udac2': 'UnicodeCharacter0xdac2',
u'\u9fc4': 'UnicodeCharacter0x9fc4',
u'\u7347': 'CjkUnifiedIdeograph7347',
u'\u3449': 'CjkUnifiedIdeograph3449',
u'\ue94b': 'UnicodeCharacter0xe94b',
u'\uaa4d': 'ChamConsonantSignFinalH',
u'\u4cce': 'CjkUnifiedIdeograph4cce',
u'\u01d0': 'LatinSmallLetterIWithCaron',
u'\uc2d2': 'HangulSyllableSyibs',
u'\u87d4': 'CjkUnifiedIdeograph87d4',
u'\u5b57': 'CjkUnifiedIdeograph5b57',
u'\ub8d6': 'HangulSyllableRyolm',
u'\u1c59': 'OlChikiDigitNine',
u'\ud15b': 'HangulSyllableTelh',
u'\u925d': 'CjkUnifiedIdeograph925d',
u'\u69e0': 'CjkUnifiedIdeograph69e0',
u'\u2ae2': 'VerticalBarTripleRightTurnstile',
u'\uefe4': 'UnicodeCharacter0xefe4',
u'\u4367': 'CjkUnifiedIdeograph4367',
u'\ua0e6': 'YiSyllableViet',
u'\u0469': 'CyrillicSmallLetterIotifiedLittleYus',
u'\u396b': 'CjkUnifiedIdeograph396b',
u'\ufa6d': 'UnicodeCharacter0xfa6d',
u'\ubf6f': 'HangulSyllableBbwas',
u'\u51f0': 'CjkUnifiedIdeograph51f0',
u'\u12f2': 'EthiopicSyllableDi',
u'\ud7f4': 'UnicodeCharacter0xd7f4',
u'\u88f6': 'CjkUnifiedIdeograph88f6',
u'\u6c79': 'CjkUnifiedIdeograph6c79',
u'\u217b': 'SmallRomanNumeralTwelve',
u'\ue27d': 'UnicodeCharacter0xe27d',
u'\ua77f': 'LatinSmallLetterTurnedInsularG',
u'\udb81': 'UnicodeCharacter0xdb81',
u'\u3c04': 'CjkUnifiedIdeograph3c04',
u'\u1087': 'MyanmarSignShanTone2',
u'\ub608': 'HangulSyllableDdyess',
u'\u8a8b': 'CjkUnifiedIdeograph8a8b',
u'\u750a': 'CjkUnifiedIdeograph750a',
u'\u498d': 'CjkUnifiedIdeograph498d',
u'\uef0e': 'UnicodeCharacter0xef0e',
u'\uc391': 'HangulSyllableSselt',
u'\u2414': 'SymbolForDeviceControlFour',
u'\u7897': 'CjkUnifiedIdeograph7897',
u'\u9e18': 'CjkUnifiedIdeograph9e18',
u'\uf29b': 'UnicodeCharacter0xf29b',
u'\u5d1a': 'CjkUnifiedIdeograph5d1a',
u'\ud71e': 'HangulSyllableHwinh',
u'\u2ba1': 'UnicodeCharacter0x2ba1',
u'\uada5': 'HangulSyllableGweg',
u'\u0c24': 'TeluguLetterTa',
u'\u60a7': 'CjkUnifiedIdeograph60a7',
u'\u8628': 'CjkUnifiedIdeograph8628',
u'\udaab': 'UnicodeCharacter0xdaab',
u'\u452a': 'CjkUnifiedIdeograph452a',
u'\u3f2e': 'CjkUnifiedIdeograph3f2e',
u'\u13b1': 'CherokeeLetterHu',
u'\ub132': 'HangulSyllableNelp',
u'\u95b5': 'CjkUnifiedIdeograph95b5',
u'\u7434': 'CjkUnifiedIdeograph7434',
u'\u48b7': 'CjkUnifiedIdeograph48b7',
u'\uee38': 'UnicodeCharacter0xee38',
u'\uc2bb': 'HangulSyllableSeuc',
u'\u273e': 'SixPetalledBlackAndWhiteFlorette',
u'\u7bc1': 'CjkUnifiedIdeograph7bc1',
u'\u9942': 'CjkUnifiedIdeograph9942',
u'\ufdc5': 'ArabicLigatureSadWithMeemWithMeemInitialForm',
u'\u5c44': 'CjkUnifiedIdeograph5c44',
u'\ud648': 'HangulSyllableHom',
u'\u2acb': 'SubsetOfAboveNotEqualTo',
u'\uaccf': 'HangulSyllableGyelb',
u'\u0f4e': 'TibetanLetterNna',
u'\u63d1': 'CjkUnifiedIdeograph63d1',
u'\u8152': 'CjkUnifiedIdeograph8152',
u'\ue5d5': 'UnicodeCharacter0xe5d5',
u'\u4454': 'CjkUnifiedIdeograph4454',
u'\u3e58': 'CjkUnifiedIdeograph3e58',
u'\u12db': 'EthiopicSyllableZaa',
u'\ub05c': 'HangulSyllableGgeuk',
u'\u94df': 'CjkUnifiedIdeograph94df',
u'\u775e': 'CjkUnifiedIdeograph775e',
u'\u4be1': 'CjkUnifiedIdeograph4be1',
u'\ue962': 'UnicodeCharacter0xe962',
u'\ucde5': 'HangulSyllableCwet',
u'\u2668': 'HotSprings',
u'\u7aeb': 'CjkUnifiedIdeograph7aeb',
u'\u986c': 'CjkUnifiedIdeograph986c',
u'\ufcef': 'ArabicLigatureNoonWithHehMedialForm',
u'\u5f6e': 'CjkUnifiedIdeograph5f6e',
u'\ud172': 'HangulSyllableTyeolm',
u'\u35f5': 'CjkUnifiedIdeograph35f5',
u'\uaff9': 'HangulSyllableGgwelg',
u'\u0e78': 'UnicodeCharacter0xe78',
u'\u62fb': 'CjkUnifiedIdeograph62fb',
u'\u807c': 'CjkUnifiedIdeograph807c',
u'\ue4ff': 'UnicodeCharacter0xe4ff',
u'\u477e': 'CjkUnifiedIdeograph477e',
u'\ude03': 'UnicodeCharacter0xde03',
u'\u3982': 'CjkUnifiedIdeograph3982',
u'\ub386': 'HangulSyllableDej',
u'\u1709': 'TagalogLetterPa',
u'\u7688': 'CjkUnifiedIdeograph7688',
u'\u890d': 'CjkUnifiedIdeograph890d',
u'\ue88c': 'UnicodeCharacter0xe88c',
u'\u4c0f': 'CjkUnifiedIdeograph4c0f',
u'\uc613': 'HangulSyllableYelb',
u'\u2192': 'RightwardsArrow',
u'\ub817': 'HangulSyllableRelh',
u'\u9b96': 'CjkUnifiedIdeograph9b96',
u'\u7f19': 'CjkUnifiedIdeograph7f19',
u'\u5e98': 'CjkUnifiedIdeograph5e98',
u'\uf11d': 'UnicodeCharacter0xf11d',
u'\ud09c': 'HangulSyllableKyiss',
u'\u2e23': 'TopRightHalfBracket',
u'\u09a2': 'BengaliLetterDdha',
u'\ua027': 'YiSyllableBo',
u'\u83a6': 'CjkUnifiedIdeograph83a6',
u'\u6729': 'CjkUnifiedIdeograph6729',
u'\u46a8': 'CjkUnifiedIdeograph46a8',
u'\ud92d': 'UnicodeCharacter0xd92d',
u'\u38ac': 'CjkUnifiedIdeograph38ac',
u'\ub2b0': 'HangulSyllableNyin',
u'\u1633': 'CanadianSyllabicsCarrierLha',
u'\u71b2': 'CjkUnifiedIdeograph71b2',
u'\u8837': 'CjkUnifiedIdeograph8837',
u'\uebb6': 'UnicodeCharacter0xebb6',
u'\u4f39': 'CjkUnifiedIdeograph4f39',
u'\uc13d': 'HangulSyllableSenj',
u'\u20bc': 'UnicodeCharacter0x20bc',
u'\ubb41': 'HangulSyllableMult',
u'\u9ac0': 'CjkUnifiedIdeograph9ac0',
u'\u7e43': 'CjkUnifiedIdeograph7e43',
u'\u59c2': 'CjkUnifiedIdeograph59c2',
u'\uf047': 'UnicodeCharacter0xf047',
u'\ud3c6': 'HangulSyllablePyeobs',
u'\u294d': 'UpBarbLeftDownBarbRightHarpoon',
u'\u08cc': 'UnicodeCharacter0x8cc',
u'\ua351': 'YiSyllableZhet',
u'\u82d0': 'CjkUnifiedIdeograph82d0',
u'\u6653': 'CjkUnifiedIdeograph6653',
u'\u41d2': 'CjkUnifiedIdeograph41d2',
u'\ud857': 'UnicodeCharacter0xd857',
u'\u3bd6': 'CjkUnifiedIdeograph3bd6',
u'\ubdda': 'HangulSyllableBwinh',
u'\u115d': 'UnicodeCharacter0x115d',
u'\u70dc': 'CjkUnifiedIdeograph70dc',
u'\u8b61': 'CjkUnifiedIdeograph8b61',
u'\ueae0': 'UnicodeCharacter0xeae0',
u'\u4e63': 'CjkUnifiedIdeograph4e63',
u'\uc067': 'HangulSyllableBbeulh',
u'\u23e6': 'AcCurrent',
u'\uba6b': 'HangulSyllableMec',
u'\ua5ea': 'VaiSyllableBe',
u'\u796d': 'CjkUnifiedIdeograph796d',
u'\u58ec': 'CjkUnifiedIdeograph58ec',
u'\uf371': 'UnicodeCharacter0xf371',
u'\ud2f0': 'HangulSyllableTi',
u'\u2877': 'BraillePatternDots123567',
u'\u0bf6': 'TamilDebitSign',
u'\ua27b': 'YiSyllableHo',
u'\u8dfa': 'CjkUnifiedIdeograph8dfa',
u'\u617d': 'CjkUnifiedIdeograph617d',
u'\u40fc': 'CjkUnifiedIdeograph40fc',
u'\u5a81': 'CjkUnifiedIdeograph5a81',
u'\u3b00': 'CjkUnifiedIdeograph3b00',
u'\u1f83': 'GreekSmallLetterAlphaWithDasiaAndVariaAndYpogegrammeni',
u'\uf802': 'UnicodeCharacter0xf802',
u'\udc85': 'UnicodeCharacter0xdc85',
u'\ubd04': 'HangulSyllableBom',
u'\u9187': 'CjkUnifiedIdeograph9187',
u'\u6e0e': 'CjkUnifiedIdeograph6e0e',
u'\u4291': 'CjkUnifiedIdeograph4291',
u'\u2310': 'ReversedNotSign',
u'\u0793': 'ThaanaLetterTaviyani',
u'\ue012': 'UnicodeCharacter0xe012',
u'\uc495': 'HangulSyllableSsweong',
u'\ua514': 'VaiSyllableDee',
u'\uf997': 'CjkCompatibilityIdeographf997',
u'\ube99': 'HangulSyllableBbyag',
u'\u561e': 'CjkUnifiedIdeograph561e',
u'\u0b20': 'OriyaLetterTtha',
u'\u6fa3': 'CjkUnifiedIdeograph6fa3',
u'\uc822': 'HangulSyllableJenh',
u'\u2ca5': 'CopticSmallLetterSima',
u'\u8d24': 'CjkUnifiedIdeograph8d24',
u'\ue1a7': 'UnicodeCharacter0xe1a7',
u'\ua6a9': 'UnicodeCharacter0xa6a9',
u'\u7330': 'CjkUnifiedIdeograph7330',
u'\u57b3': 'CjkUnifiedIdeograph57b3',
u'\u3032': 'VerticalKanaRepeatWithVoicedSoundMark',
u'\u14b5': 'CanadianSyllabicsWestcreeMwoo',
u'\uf534': 'UnicodeCharacter0xf534',
u'\uc9b7': 'HangulSyllableJyis',
u'\uaa36': 'ChamConsonantSignWa',
u'\u8eb9': 'CjkUnifiedIdeograph8eb9',
u'\u5b40': 'CjkUnifiedIdeograph5b40',
u'\u1842': 'MongolianLetterChi',
u'\u7cc5': 'CjkUnifiedIdeograph7cc5',
u'\udd44': 'UnicodeCharacter0xdd44',
u'\u31c7': 'CjkStrokeHp',
u'\u9246': 'CjkUnifiedIdeograph9246',
u'\uf6c9': 'UnicodeCharacter0xf6c9',
u'\uabcb': 'UnicodeCharacter0xabcb',
u'\u4350': 'CjkUnifiedIdeograph4350',
u'\u64d5': 'CjkUnifiedIdeograph64d5',
u'\uc554': 'HangulSyllableAm',
u'\u19d7': 'NewTaiLueDigitSeven',
u'\ufa56': 'CjkCompatibilityIdeographfa56',
u'\uded9': 'UnicodeCharacter0xded9',
u'\ubf58': 'HangulSyllableBbok',
u'\u93db': 'CjkUnifiedIdeograph93db',
u'\u6862': 'CjkUnifiedIdeograph6862',
u'\u4ce5': 'CjkUnifiedIdeograph4ce5',
u'\u2d64': 'TifinaghLetterTawellemetYaz',
u'\u01e7': 'LatinSmallLetterGWithCaron',
u'\ue266': 'UnicodeCharacter0xe266',
u'\uc6e9': 'HangulSyllableWeg',
u'\ua768': 'LatinCapitalLetterVend',
u'\ufbeb': 'ArabicLigatureYehWithHamzaAboveWithAlefFinalForm',
u'\ub8ed': 'HangulSyllableRunj',
u'\u5072': 'CjkUnifiedIdeograph5072',
u'\u1574': 'CanadianSyllabicsNunavikHe',
u'\u69f7': 'CjkUnifiedIdeograph69f7',
u'\uca76': 'HangulSyllableJjelp',
u'\u2ef9': 'UnicodeCharacter0x2ef9',
u'\u8f78': 'CjkUnifiedIdeograph8f78',
u'\ue3fb': 'UnicodeCharacter0xe3fb',
u'\ua0fd': 'YiSyllableVyp',
u'\u9a01': 'CjkUnifiedIdeograph9a01',
u'\u7d84': 'CjkUnifiedIdeograph7d84',
u'\u3286': 'CircledIdeographSeven',
u'\uf788': 'UnicodeCharacter0xf788',
u'\u4b0b': 'CjkUnifiedIdeograph4b0b',
u'\ub48a': 'HangulSyllableDwegg',
u'\u080d': 'UnicodeCharacter0x80d',
u'\ucd0f': 'HangulSyllableCod',
u'\u8211': 'CjkUnifiedIdeograph8211',
u'\u6594': 'CjkUnifiedIdeograph6594',
u'\u1a96': 'UnicodeCharacter0x1a96',
u'\udf98': 'UnicodeCharacter0xdf98',
u'\u9c9a': 'CjkUnifiedIdeograph9c9a',
u'\u701d': 'CjkUnifiedIdeograph701d',
u'\u351f': 'CjkUnifiedIdeograph351f',
u'\uea21': 'UnicodeCharacter0xea21',
u'\uaf23': 'HangulSyllableGgyes',
u'\u4da4': 'CjkUnifiedIdeograph4da4',
u'\u02a6': 'LatinSmallLetterTsDigraph',
u'\uc7a8': 'HangulSyllableJak',
u'\u84aa': 'CjkUnifiedIdeograph84aa',
u'\u582d': 'CjkUnifiedIdeograph582d',
u'\ub9ac': 'HangulSyllableRi',
u'\u1d2f': 'ModifierLetterCapitalBarredB',
u'\ud231': 'HangulSyllableTunj',
u'\u9733': 'CjkUnifiedIdeograph9733',
u'\u6ab6': 'CjkUnifiedIdeograph6ab6',
u'\u2fb8': 'KangxiRadicalHead',
u'\uecba': 'UnicodeCharacter0xecba',
u'\u403d': 'CjkUnifiedIdeograph403d',
u'\ua1bc': 'YiSyllableLiex',
u'\u053f': 'ArmenianCapitalLetterKen',
u'\u3a41': 'CjkUnifiedIdeograph3a41',
u'\uff43': 'FullwidthLatinSmallLetterC',
u'\ubc45': 'HangulSyllableBaeng',
u'\u52c6': 'CjkUnifiedIdeograph52c6',
u'\u17c8': 'KhmerSignYuukaleapintu',
u'\ud4ca': 'HangulSyllablePwep',
u'\u89cc': 'CjkUnifiedIdeograph89cc',
u'\u6d4f': 'CjkUnifiedIdeograph6d4f',
u'\u2251': 'GeometricallyEqualTo',
u'\ue753': 'UnicodeCharacter0xe753',
u'\ua455': 'YiSyllableNyot',
u'\u7fd8': 'CjkUnifiedIdeograph7fd8',
u'\u3cda': 'CjkUnifiedIdeograph3cda',
u'\uf1dc': 'UnicodeCharacter0xf1dc',
u'\u555f': 'CjkUnifiedIdeograph555f',
u'\ub6de': 'HangulSyllableDdwelm',
u'\u0a61': 'UnicodeCharacter0xa61',
u'\ucf63': 'HangulSyllableKolh',
u'\u8c65': 'CjkUnifiedIdeograph8c65',
u'\u67e8': 'CjkUnifiedIdeograph67e8',
u'\u24ea': 'CircledDigitZero',
u'\ud9ec': 'UnicodeCharacter0xd9ec',
u'\u9eee': 'CjkUnifiedIdeograph9eee',
u'\u7271': 'CjkUnifiedIdeograph7271',
u'\u3773': 'CjkUnifiedIdeograph3773',
u'\uf475': 'UnicodeCharacter0xf475',
u'\ua977': 'UnicodeCharacter0xa977',
u'\u4ff8': 'CjkUnifiedIdeograph4ff8',
u'\u0cfa': 'UnicodeCharacter0xcfa',
u'\uc1fc': 'HangulSyllableSyo',
u'\u86fe': 'CjkUnifiedIdeograph86fe',
u'\ubc00': 'HangulSyllableMil',
u'\u9883': 'CjkUnifiedIdeograph9883',
u'\u7f02': 'CjkUnifiedIdeograph7f02',
u'\u5b85': 'CjkUnifiedIdeograph5b85',
u'\uf506': 'UnicodeCharacter0xf506',
u'\ud189': 'HangulSyllableTyenj',
u'\u2e0c': 'LeftRaisedOmissionBracket',
u'\u0a8f': 'GujaratiLetterE',
u'\ua410': 'YiSyllableQurx',
u'\u8093': 'CjkUnifiedIdeograph8093',
u'\u6712': 'CjkUnifiedIdeograph6712',
u'\u4395': 'CjkUnifiedIdeograph4395',
u'\udd16': 'UnicodeCharacter0xdd16',
u'\u3999': 'CjkUnifiedIdeograph3999',
u'\ub79d': 'HangulSyllableRaenj',
u'\u161c': 'CanadianSyllabicsCarrierJju',
u'\u729f': 'CjkUnifiedIdeograph729f',
u'\u8c20': 'CjkUnifiedIdeograph8c20',
u'\ue8a3': 'UnicodeCharacter0xe8a3',
u'\u4f22': 'CjkUnifiedIdeograph4f22',
u'\uc526': 'HangulSyllableSsyip',
u'\u21a9': 'LeftwardsArrowWithHook',
u'\u9fad': 'CjkUnifiedIdeograph9fad',
u'\u7e2c': 'CjkUnifiedIdeograph7e2c',
u'\u5aaf': 'CjkUnifiedIdeograph5aaf',
u'\uf430': 'UnicodeCharacter0xf430',
u'\ud0b3': 'HangulSyllableKilh',
u'\u2d36': 'TifinaghLetterYaj',
u'\u09b9': 'BengaliLetterHa',
u'\uab3a': 'UnicodeCharacter0xab3a',
u'\u87bd': 'CjkUnifiedIdeograph87bd',
u'\u663c': 'CjkUnifiedIdeograph663c',
u'\u42bf': 'CjkUnifiedIdeograph42bf',
u'\udc40': 'UnicodeCharacter0xdc40',
u'\u38c3': 'CjkUnifiedIdeograph38c3',
u'\ub6c7': 'HangulSyllableDdweolh',
u'\u1546': 'CanadianSyllabicsRi',
u'\u71c9': 'CjkUnifiedIdeograph71c9',
u'\u934a': 'CjkUnifiedIdeograph934a',
u'\uefcd': 'UnicodeCharacter0xefcd',
u'\u4e4c': 'CjkUnifiedIdeograph4e4c',
u'\uc450': 'HangulSyllableSsyol',
u'\u20d3': 'CombiningShortVerticalLineOverlay',
u'\u9ed7': 'CjkUnifiedIdeograph9ed7',
u'\u7d56': 'CjkUnifiedIdeograph7d56',
u'\u59d9': 'CjkUnifiedIdeograph59d9',
u'\ufb5a': 'ArabicLetterBehehIsolatedForm',
u'\ud7dd': 'UnicodeCharacter0xd7dd',
u'\u2c60': 'LatinCapitalLetterLWithDoubleBar',
u'\u08e3': 'UnicodeCharacter0x8e3',
u'\uaa64': 'UnicodeCharacter0xaa64',
u'\u86e7': 'CjkUnifiedIdeograph86e7',
u'\u6566': 'CjkUnifiedIdeograph6566',
u'\u41e9': 'CjkUnifiedIdeograph41e9',
u'\ue36a': 'UnicodeCharacter0xe36a',
u'\u3fed': 'CjkUnifiedIdeograph3fed',
u'\ub5f1': 'HangulSyllableDdyeot',
u'\u1470': 'CanadianSyllabicsKoo',
u'\u70f3': 'CjkUnifiedIdeograph70f3',
u'\u9274': 'CjkUnifiedIdeograph9274',
u'\ueef7': 'UnicodeCharacter0xeef7',
u'\u4d76': 'CjkUnifiedIdeograph4d76',
u'\ucb7a': 'HangulSyllableJjweoj',
u'\u27fd': 'LongLeftwardsDoubleArrowFromBar',
u'\u1d01': 'LatinLetterSmallCapitalAe',
u'\u7c80': 'CjkUnifiedIdeograph7c80',
u'\u9b05': 'CjkUnifiedIdeograph9b05',
u'\ufa84': 'CjkCompatibilityIdeographfa84',
u'\u5607': 'CjkUnifiedIdeograph5607',
u'\ucc0b': 'HangulSyllableJjyih',
u'\u338a': 'SquarePf',
u'\ua98e': 'UnicodeCharacter0xa98e',
u'\u0511': 'CyrillicSmallLetterReversedZe',
u'\u6490': 'CjkUnifiedIdeograph6490',
u'\u8315': 'CjkUnifiedIdeograph8315',
u'\ue294': 'UnicodeCharacter0xe294',
u'\u341b': 'CjkUnifiedIdeograph341b',
u'\u1b9a': 'SundaneseLetterYa',
u'\ub21f': 'HangulSyllableNuh',
u'\u919e': 'CjkUnifiedIdeograph919e',
u'\u6d21': 'CjkUnifiedIdeograph6d21',
u'\u4ca0': 'CjkUnifiedIdeograph4ca0',
u'\ueb25': 'UnicodeCharacter0xeb25',
u'\ucaa4': 'HangulSyllableJjyen',
u'\u1c2b': 'LepchaVowelSignUu',
u'\u03aa': 'GreekCapitalLetterIotaWithDialytika',
u'\u9a2f': 'CjkUnifiedIdeograph9a2f',
u'\uf9ae': 'CjkCompatibilityIdeographf9ae',
u'\u5531': 'CjkUnifiedIdeograph5531',
u'\ud335': 'HangulSyllablePaelt',
u'\u32b4': 'CircledNumberThirtyNine',
u'\ua8b8': 'SaurashtraVowelSignU',
u'\u043b': 'CyrillicSmallLetterEl',
u'\u6bba': 'CjkUnifiedIdeograph6bba',
u'\u823f': 'CjkUnifiedIdeograph823f',
u'\ue1be': 'UnicodeCharacter0xe1be',
u'\u3b45': 'CjkUnifiedIdeograph3b45',
u'\u1ac4': 'UnicodeCharacter0x1ac4',
u'\ub149': 'HangulSyllableNyeolg',
u'\u90c8': 'CjkUnifiedIdeograph90c8',
u'\u6c4b': 'CjkUnifiedIdeograph6c4b',
u'\u53ca': 'CjkUnifiedIdeograph53ca',
u'\uea4f': 'UnicodeCharacter0xea4f',
u'\uc9ce': 'HangulSyllableJilp',
u'\u2355': 'AplFunctionalSymbolUpTackJot',
u'\u02d4': 'ModifierLetterUpTack',
u'\u9959': 'CjkUnifiedIdeograph9959',
u'\uf8d8': 'UnicodeCharacter0xf8d8',
u'\u545b': 'CjkUnifiedIdeograph545b',
u'\ud25f': 'HangulSyllableTweoc',
u'\u31de': 'CjkStrokeSzz',
u'\uafe2': 'HangulSyllableGgweolp',
u'\u0b65': 'UnicodeCharacter0xb65',
u'\u6ae4': 'CjkUnifiedIdeograph6ae4',
u'\u8169': 'CjkUnifiedIdeograph8169',
u'\ue0e8': 'UnicodeCharacter0xe0e8',
u'\u3a6f': 'CjkUnifiedIdeograph3a6f',
u'\u19ee': 'KhmerSymbolDapbuonKoet',
u'\ub073': 'HangulSyllableGgyis',
u'\u97f2': 'CjkUnifiedIdeograph97f2',
u'\u7375': 'CjkUnifiedIdeograph7375',
u'\u52f4': 'CjkUnifiedIdeograph52f4',
u'\ue979': 'UnicodeCharacter0xe979',
u'\uc8f8': 'HangulSyllableJyok',
u'\u227f': 'SucceedsOrEquivalentTo',
u'\u01fe': 'LatinCapitalLetterOWithStrokeAndAcute',
u'\u7406': 'CjkUnifiedIdeograph7406',
u'\u5089': 'CjkUnifiedIdeograph5089',
u'\u3108': 'BopomofoLetterF',
u'\u0d8b': 'SinhalaLetterUyanna',
u'\uf20a': 'UnicodeCharacter0xf20a',
u'\uce8d': 'HangulSyllableKat',
u'\uaf0c': 'HangulSyllableGgyeok',
u'\u8b8f': 'CjkUnifiedIdeograph8b8f',
u'\u5c16': 'CjkUnifiedIdeograph5c16',
u'\u1918': 'LimbuLetterWa',
u'\u759b': 'CjkUnifiedIdeograph759b',
u'\uda1a': 'UnicodeCharacter0xda1a',
u'\u369d': 'CjkUnifiedIdeograph369d',
u'\u971c': 'CjkUnifiedIdeograph971c',
u'\uf39f': 'UnicodeCharacter0xf39f',
u'\uaca1': 'HangulSyllableGeng',
u'\u4426': 'CjkUnifiedIdeograph4426',
u'\u0128': 'LatinCapitalLetterIWithTilde',
u'\u5dab': 'CjkUnifiedIdeograph5dab',
u'\uc22a': 'HangulSyllableSubs',
u'\u1ead': 'LatinSmallLetterAWithCircumflexAndDotBelow',
u'\uff2c': 'FullwidthLatinCapitalLetterL',
u'\udbaf': 'UnicodeCharacter0xdbaf',
u'\ub82e': 'HangulSyllableRyeolm',
u'\u94b1': 'CjkUnifiedIdeograph94b1',
u'\u6938': 'CjkUnifiedIdeograph6938',
u'\u45bb': 'CjkUnifiedIdeograph45bb',
u'\u2a3a': 'MinusSignInTriangle',
u'\u06bd': 'ArabicLetterNoonWithThreeDotsAbove',
u'\ue73c': 'UnicodeCharacter0xe73c',
u'\uc3bf': 'HangulSyllableSsyegs',
u'\ua03e': 'YiSyllablePiep',
u'\ufcc1': 'ArabicLigatureFehWithMeemInitialForm',
u'\ub9c3': 'HangulSyllableRic',
u'\u5148': 'CjkUnifiedIdeograph5148',
u'\u124a': 'EthiopicSyllableQwi',
u'\u6ecd': 'CjkUnifiedIdeograph6ecd',
u'\ucf4c': 'HangulSyllableKyess',
u'\u2bcf': 'UnicodeCharacter0x2bcf',
u'\u884e': 'CjkUnifiedIdeograph884e',
u'\ue4d1': 'UnicodeCharacter0xe4d1',
u'\ua1d3': 'YiSyllableLur',
u'\u7a5a': 'CjkUnifiedIdeograph7a5a',
u'\u56dd': 'CjkUnifiedIdeograph56dd',
u'\u375c': 'CjkUnifiedIdeograph375c',
u'\u13df': 'CherokeeLetterTli',
u'\uf05e': 'UnicodeCharacter0xf05e',
u'\ucce1': 'HangulSyllableCyeob',
u'\uad60': 'HangulSyllableGyom',
u'\u89e3': 'CjkUnifiedIdeograph89e3',
u'\u626a': 'CjkUnifiedIdeograph626a',
u'\u1f6c': 'GreekCapitalLetterOmegaWithPsiliAndOxia',
u'\u7bef': 'CjkUnifiedIdeograph7bef',
u'\ud86e': 'UnicodeCharacter0xd86e',
u'\u34f1': 'CjkUnifiedIdeograph34f1',
u'\u9570': 'CjkUnifiedIdeograph9570',
u'\uf1f3': 'UnicodeCharacter0xf1f3',
u'\ub2f5': 'HangulSyllableDab',
u'\u4a7a': 'CjkUnifiedIdeograph4a7a',
u'\u077c': 'ArabicLetterHahWithExtendedArabicindicDigitFourBelow',
u'\u63ff': 'CjkUnifiedIdeograph63ff',
u'\uc07e': 'HangulSyllableBbyilm',
u'\ufd80': 'ArabicLigatureLamWithHahWithMeemFinalForm',
u'\u5903': 'CjkUnifiedIdeograph5903',
u'\ube82': 'HangulSyllableBbaenh',
u'\u1a05': 'BugineseLetterBa',
u'\ud707': 'HangulSyllableHwelb',
u'\u9009': 'CjkUnifiedIdeograph9009',
u'\u6f8c': 'CjkUnifiedIdeograph6f8c',
u'\u288e': 'BraillePatternDots2348',
u'\ue590': 'UnicodeCharacter0xe590',
u'\u4113': 'CjkUnifiedIdeograph4113',
u'\ua692': 'CyrillicCapitalLetterTche',
u'\u0215': 'LatinSmallLetterUWithDoubleGrave',
u'\u3f17': 'CjkUnifiedIdeograph3f17',
u'\uf819': 'UnicodeCharacter0xf819',
u'\ub51b': 'HangulSyllableDid',
u'\u579c': 'CjkUnifiedIdeograph579c',
u'\u109e': 'MyanmarSymbolShanOne',
u'\ucda0': 'HangulSyllableCuls',
u'\u8ea2': 'CjkUnifiedIdeograph8ea2',
u'\u6a25': 'CjkUnifiedIdeograph6a25',
u'\u2727': 'WhiteFourPointedStar',
u'\ue029': 'UnicodeCharacter0xe029',
u'\u9d2b': 'CjkUnifiedIdeograph9d2b',
u'\u78ae': 'CjkUnifiedIdeograph78ae',
u'\u35b0': 'CjkUnifiedIdeograph35b0',
u'\uf6b2': 'UnicodeCharacter0xf6b2',
u'\u5235': 'CjkUnifiedIdeograph5235',
u'\ub3b4': 'HangulSyllableDyels',
u'\u0f37': 'TibetanMarkNgasBzungSgorRtags',
u'\uc839': 'HangulSyllableJyeog',
u'\u853b': 'CjkUnifiedIdeograph853b',
u'\u60be': 'CjkUnifiedIdeograph60be',
u'\u1dc0': 'CombiningDottedGraveAccent',
u'\udec2': 'UnicodeCharacter0xdec2',
u'\u9bc4': 'CjkUnifiedIdeograph9bc4',
u'\u7747': 'CjkUnifiedIdeograph7747',
u'\u3049': 'HiraganaLetterSmallO',
u'\ued4b': 'UnicodeCharacter0xed4b',
u'\uae4d': 'HangulSyllableGgag',
u'\u48ce': 'CjkUnifiedIdeograph48ce',
u'\u05d0': 'HebrewLetterAlef',
u'\uc6d2': 'HangulSyllableWeonh',
u'\u83d4': 'CjkUnifiedIdeograph83d4',
u'\u5f57': 'CjkUnifiedIdeograph5f57',
u'\ubcd6': 'HangulSyllableByeop',
u'\u1859': 'MongolianLetterTodoHaa',
u'\ud55b': 'HangulSyllableHags',
u'\u965d': 'CjkUnifiedIdeograph965d',
u'\u6de0': 'CjkUnifiedIdeograph6de0',
u'\u2ee2': 'CjkRadicalCsimplifiedHorse',
u'\uebe4': 'UnicodeCharacter0xebe4',
u'\u4767': 'CjkUnifiedIdeograph4767',
u'\ua4e6': 'UnicodeCharacter0xa4e6',
u'\u3d6b': 'CjkUnifiedIdeograph3d6b',
u'\ufe6d': 'UnicodeCharacter0xfe6d',
u'\ubb6f': 'HangulSyllableMwegs',
u'\u55f0': 'CjkUnifiedIdeograph55f0',
u'\u16f2': 'UnicodeCharacter0x16f2',
u'\ud3f4': 'HangulSyllablePol',
u'\u8cf6': 'CjkUnifiedIdeograph8cf6',
u'\u6879': 'CjkUnifiedIdeograph6879',
u'\u257b': 'BoxDrawingsHeavyDown',
u'\ue67d': 'UnicodeCharacter0xe67d',
u'\ua37f': 'YiSyllableRruo',
u'\udf81': 'UnicodeCharacter0xdf81',
u'\u3804': 'CjkUnifiedIdeograph3804',
u'\u1487': 'CanadianSyllabicsSouthslaveyKoh',
u'\ub208': 'HangulSyllableNun',
u'\u8e8b': 'CjkUnifiedIdeograph8e8b',
u'\u710a': 'CjkUnifiedIdeograph710a',
u'\u4d8d': 'CjkUnifiedIdeograph4d8d',
u'\ueb0e': 'UnicodeCharacter0xeb0e',
u'\uc791': 'HangulSyllableJag',
u'\u2014': 'EmDash',
u'\u7c97': 'CjkUnifiedIdeograph7c97',
u'\u9a18': 'CjkUnifiedIdeograph9a18',
u'\uf69b': 'UnicodeCharacter0xf69b',
u'\u591a': 'CjkUnifiedIdeograph591a',
u'\ud31e': 'HangulSyllablePabs',
u'\u2fa1': 'KangxiRadicalWalk',
u'\ua9a5': 'UnicodeCharacter0xa9a5',
u'\u0824': 'UnicodeCharacter0x824',
u'\u64a7': 'CjkUnifiedIdeograph64a7',
u'\u8228': 'CjkUnifiedIdeograph8228',
u'\udeab': 'UnicodeCharacter0xdeab',
u'\u412a': 'CjkUnifiedIdeograph412a',
u'\u3b2e': 'CjkUnifiedIdeograph3b2e',
u'\u17b1': 'KhmerIndependentVowelQooTypeOne',
u'\ub532': 'HangulSyllableDdagg',
u'\u91b5': 'CjkUnifiedIdeograph91b5',
u'\u7034': 'CjkUnifiedIdeograph7034',
u'\u4cb7': 'CjkUnifiedIdeograph4cb7',
u'\uea38': 'UnicodeCharacter0xea38',
u'\uc6bb': 'HangulSyllableUlb',
u'\u233e': 'AplFunctionalSymbolCircleJot',
u'\u7fc1': 'CjkUnifiedIdeograph7fc1',
u'\u9d42': 'CjkUnifiedIdeograph9d42',
u'\uf9c5': 'CjkCompatibilityIdeographf9c5',
u'\u5844': 'CjkUnifiedIdeograph5844',
u'\ud248': 'HangulSyllableTweo',
u'\u2ecb': 'CjkRadicalCsimplifiedCart',
u'\ua8cf': 'SaurashtraDoubleDanda',
u'\u0b4e': 'UnicodeCharacter0xb4e',
u'\u67d1': 'CjkUnifiedIdeograph67d1',
u'\u8552': 'CjkUnifiedIdeograph8552',
u'\ue1d5': 'UnicodeCharacter0xe1d5',
u'\u4054': 'CjkUnifiedIdeograph4054',
u'\u3a58': 'CjkUnifiedIdeograph3a58',
u'\u16db': 'RunicLetterDottedl',
u'\ub45c': 'HangulSyllableDuls',
u'\u90df': 'CjkUnifiedIdeograph90df',
u'\u735e': 'CjkUnifiedIdeograph735e',
u'\u4fe1': 'CjkUnifiedIdeograph4fe1',
u'\ued62': 'UnicodeCharacter0xed62',
u'\uc9e5': 'HangulSyllableJjalg',
u'\u2268': 'LessthanButNotEqualTo',
u'\u7eeb': 'CjkUnifiedIdeograph7eeb',
u'\u9c6c': 'CjkUnifiedIdeograph9c6c',
u'\uf8ef': 'UnicodeCharacter0xf8ef',
u'\u5b6e': 'CjkUnifiedIdeograph5b6e',
u'\ud572': 'HangulSyllableHap',
u'\u31f5': 'KatakanaLetterSmallHa',
u'\uabf9': 'UnicodeCharacter0xabf9',
u'\u0a78': 'UnicodeCharacter0xa78',
u'\u66fb': 'CjkUnifiedIdeograph66fb',
u'\u847c': 'CjkUnifiedIdeograph847c',
u'\ue0ff': 'UnicodeCharacter0xe0ff',
u'\u437e': 'CjkUnifiedIdeograph437e',
u'\uda03': 'UnicodeCharacter0xda03',
u'\u3d82': 'CjkUnifiedIdeograph3d82',
u'\ub786': 'HangulSyllableRalm',
u'\u1309': 'EthiopicSyllableGu',
u'\u7288': 'CjkUnifiedIdeograph7288',
u'\u8d0d': 'CjkUnifiedIdeograph8d0d',
u'\uec8c': 'UnicodeCharacter0xec8c',
u'\u480f': 'CjkUnifiedIdeograph480f',
u'\uc213': 'HangulSyllableSyoc',
u'\u2592': 'MediumShade',
u'\ubc17': 'HangulSyllableBags',
u'\u9f96': 'CjkUnifiedIdeograph9f96',
u'\u7b19': 'CjkUnifiedIdeograph7b19',
u'\u5a98': 'CjkUnifiedIdeograph5a98',
u'\uf51d': 'UnicodeCharacter0xf51d',
u'\ud49c': 'HangulSyllablePweol',
u'\u2a23': 'PlusSignWithCircumflexAccentAbove',
u'\u0da2': 'SinhalaLetterAlpapraanaJayanna',
u'\ua427': 'YiSyllableJjut',
u'\u87a6': 'CjkUnifiedIdeograph87a6',
u'\u6329': 'CjkUnifiedIdeograph6329',
u'\u42a8': 'CjkUnifiedIdeograph42a8',
u'\udd2d': 'UnicodeCharacter0xdd2d',
u'\u3cac': 'CjkUnifiedIdeograph3cac',
u'\ub6b0': 'HangulSyllableDduss',
u'\u1233': 'EthiopicSyllableSaa',
u'\u75b2': 'CjkUnifiedIdeograph75b2',
u'\u8c37': 'CjkUnifiedIdeograph8c37',
u'\uefb6': 'UnicodeCharacter0xefb6',
u'\u4b39': 'CjkUnifiedIdeograph4b39',
u'\uc53d': 'HangulSyllableSsing',
u'\u24bc': 'CircledLatinCapitalLetterG',
u'\ubf41': 'HangulSyllableBbog',
u'\u9ec0': 'CjkUnifiedIdeograph9ec0',
u'\u7a43': 'CjkUnifiedIdeograph7a43',
u'\u5dc2': 'CjkUnifiedIdeograph5dc2',
u'\uf447': 'UnicodeCharacter0xf447',
u'\ud7c6': 'UnicodeCharacter0xd7c6',
u'\u2d4d': 'TifinaghLetterYal',
u'\u0ccc': 'KannadaVowelSignAu',
u'\ua751': 'LatinSmallLetterPWithStrokeThroughDescender',
u'\u86d0': 'CjkUnifiedIdeograph86d0',
u'\u6253': 'CjkUnifiedIdeograph6253',
u'\u45d2': 'CjkUnifiedIdeograph45d2',
u'\udc57': 'UnicodeCharacter0xdc57',
u'\u3fd6': 'CjkUnifiedIdeograph3fd6',
u'\ub9da': 'HangulSyllableMabs',
u'\u155d': 'CanadianSyllabicsF',
u'\u74dc': 'CjkUnifiedIdeograph74dc',
u'\u8f61': 'CjkUnifiedIdeograph8f61',
u'\ueee0': 'UnicodeCharacter0xeee0',
u'\u4a63': 'CjkUnifiedIdeograph4a63',
u'\uc467': 'HangulSyllableSsugs',
u'\u27e6': 'MathematicalLeftWhiteSquareBracket',
u'\ube6b': 'HangulSyllableBbalb',
u'\ua1ea': 'YiSyllableGot',
u'\u7d6d': 'CjkUnifiedIdeograph7d6d',
u'\u5cec': 'CjkUnifiedIdeograph5cec',
u'\uf771': 'UnicodeCharacter0xf771',
u'\ud6f0': 'HangulSyllableHweom',
u'\u2c77': 'LatinSmallLetterTaillessPhi',
u'\u0ff6': 'UnicodeCharacter0xff6',
u'\ua67b': 'UnicodeCharacter0xa67b',
u'\u89fa': 'CjkUnifiedIdeograph89fa',
u'\u657d': 'CjkUnifiedIdeograph657d',
u'\u44fc': 'CjkUnifiedIdeograph44fc',
u'\u5e81': 'CjkUnifiedIdeograph5e81',
u'\u3f00': 'CjkUnifiedIdeograph3f00',
u'\u1b83': 'SundaneseLetterA',
u'\ufc02': 'ArabicLigatureYehWithHamzaAboveWithMeemIsolatedForm',
u'\ud885': 'UnicodeCharacter0xd885',
u'\ub904': 'HangulSyllableRweo',
u'\u9587': 'CjkUnifiedIdeograph9587',
u'\u6a0e': 'CjkUnifiedIdeograph6a0e',
u'\u4691': 'CjkUnifiedIdeograph4691',
u'\u2710': 'UpperRightPencil',
u'\u0393': 'GreekCapitalLetterGamma',
u'\ue412': 'UnicodeCharacter0xe412',
u'\uc095': 'HangulSyllableBbinj',
u'\ua114': 'YiSyllableDut',
u'\ufd97': 'ArabicLigatureNoonWithJeemWithMeemFinalForm',
u'\uba99': 'HangulSyllableMyelt',
u'\u521e': 'CjkUnifiedIdeograph521e',
u'\u0f20': 'TibetanDigitZero',
u'\u6ba3': 'CjkUnifiedIdeograph6ba3',
u'\ucc22': 'HangulSyllableJjij',
u'\u28a5': 'BraillePatternDots1368',
u'\u8924': 'CjkUnifiedIdeograph8924',
u'\ue5a7': 'UnicodeCharacter0xe5a7',
u'\ua2a9': 'YiSyllableZyx',
u'\u7730': 'CjkUnifiedIdeograph7730',
u'\u53b3': 'CjkUnifiedIdeograph53b3',
u'\u3432': 'CjkUnifiedIdeograph3432',
u'\u10b5': 'GeorgianCapitalLetterKhar',
u'\uf134': 'UnicodeCharacter0xf134',
u'\ucdb7': 'HangulSyllableCweod',
u'\uae36': 'HangulSyllableGinh',
u'\u8ab9': 'CjkUnifiedIdeograph8ab9',
u'\u5f40': 'CjkUnifiedIdeograph5f40',
u'\u1c42': 'LepchaDigitTwo',
u'\u78c5': 'CjkUnifiedIdeograph78c5',
u'\ud944': 'UnicodeCharacter0xd944',
u'\u35c7': 'CjkUnifiedIdeograph35c7',
u'\u9646': 'CjkUnifiedIdeograph9646',
u'\uf2c9': 'UnicodeCharacter0xf2c9',
u'\uafcb': 'HangulSyllableGgus',
u'\u4750': 'CjkUnifiedIdeograph4750',
u'\u0452': 'CyrillicSmallLetterDje',
u'\u60d5': 'CjkUnifiedIdeograph60d5',
u'\uc154': 'HangulSyllableSyeo',
u'\u1dd7': 'CombiningLatinSmallLetterCCedilla',
u'\ufe56': 'SmallQuestionMark',
u'\udad9': 'UnicodeCharacter0xdad9',
u'\ubb58': 'HangulSyllableMweol',
u'\u97db': 'CjkUnifiedIdeograph97db',
u'\u6c62': 'CjkUnifiedIdeograph6c62',
u'\u48e5': 'CjkUnifiedIdeograph48e5',
u'\u2964': 'RightwardsHarpoonWithBarbUpAboveRightwardsHarpoonWithBarbDown',
u'\u05e7': 'HebrewLetterQof',
u'\ue666': 'UnicodeCharacter0xe666',
u'\uc2e9': 'HangulSyllableSilt',
u'\ua368': 'YiSyllableChuop',
u'\uffeb': 'HalfwidthRightwardsArrow',
u'\ubced': 'HangulSyllableByeng',
u'\u5472': 'CjkUnifiedIdeograph5472',
u'\u1174': 'HangulJungseongYi',
u'\u6df7': 'CjkUnifiedIdeograph6df7',
u'\uce76': 'HangulSyllableKagg',
u'\u2af9': 'DoublelineSlantedLessthanOrEqualTo',
u'\u8b78': 'CjkUnifiedIdeograph8b78',
u'\ue7fb': 'UnicodeCharacter0xe7fb',
u'\ua4fd': 'UnicodeCharacter0xa4fd',
u'\u9e01': 'CjkUnifiedIdeograph9e01',
u'\u7984': 'CjkUnifiedIdeograph7984',
u'\u3686': 'CjkUnifiedIdeograph3686',
u'\uf388': 'UnicodeCharacter0xf388',
u'\u4f0b': 'CjkUnifiedIdeograph4f0b',
u'\ub08a': 'HangulSyllableGgilp',
u'\u0c0d': 'UnicodeCharacter0xc0d',
u'\uc90f': 'HangulSyllableJus',
u'\u8611': 'CjkUnifiedIdeograph8611',
u'\u6194': 'CjkUnifiedIdeograph6194',
u'\u1e96': 'LatinSmallLetterHWithLineBelow',
u'\udb98': 'UnicodeCharacter0xdb98',
u'\u989a': 'CjkUnifiedIdeograph989a',
u'\u741d': 'CjkUnifiedIdeograph741d',
u'\u311f': 'BopomofoLetterEi',
u'\uee21': 'UnicodeCharacter0xee21',
u'\uab23': 'UnicodeCharacter0xab23',
u'\u49a4': 'CjkUnifiedIdeograph49a4',
u'\u06a6': 'ArabicLetterPeheh',
u'\uc3a8': 'HangulSyllableSsyeol',
u'\u80aa': 'CjkUnifiedIdeograph80aa',
u'\u5c2d': 'CjkUnifiedIdeograph5c2d',
u'\ubdac': 'HangulSyllableBweom',
u'\u192f': 'UnicodeCharacter0x192f',
u'\ud631': 'HangulSyllableHyeng',
u'\u9333': 'CjkUnifiedIdeograph9333',
u'\u6eb6': 'CjkUnifiedIdeograph6eb6',
u'\u2bb8': 'UnicodeCharacter0x2bb8',
u'\ue8ba': 'UnicodeCharacter0xe8ba',
u'\u443d': 'CjkUnifiedIdeograph443d',
u'\ua5bc': 'VaiSyllableNgon',
u'\u013f': 'LatinCapitalLetterLWithMiddleDot',
u'\u3e41': 'CjkUnifiedIdeograph3e41',
u'\ufb43': 'HebrewLetterFinalPeWithDagesh',
u'\ub845': 'HangulSyllableRyenj',
u'\u56c6': 'CjkUnifiedIdeograph56c6',
u'\u13c8': 'CherokeeLetterQui',
u'\ud0ca': 'HangulSyllableTalm',
u'\u8dcc': 'CjkUnifiedIdeograph8dcc',
u'\u694f': 'CjkUnifiedIdeograph694f',
u'\u2651': 'Capricorn',
u'\ue353': 'UnicodeCharacter0xe353',
u'\ua055': 'YiSyllablePyr',
u'\u7bd8': 'CjkUnifiedIdeograph7bd8',
u'\u38da': 'CjkUnifiedIdeograph38da',
u'\uf5dc': 'UnicodeCharacter0xf5dc',
u'\u515f': 'CjkUnifiedIdeograph515f',
u'\ub2de': 'HangulSyllableNij',
u'\u0e61': 'UnicodeCharacter0xe61',
u'\ucb63': 'HangulSyllableJjuh',
u'\u8865': 'CjkUnifiedIdeograph8865',
u'\u63e8': 'CjkUnifiedIdeograph63e8',
u'\u20ea': 'CombiningLeftwardsArrowOverlay',
u'\uddec': 'UnicodeCharacter0xddec',
u'\u9aee': 'CjkUnifiedIdeograph9aee',
u'\u7671': 'CjkUnifiedIdeograph7671',
u'\u3373': 'SquareAu',
u'\uf075': 'UnicodeCharacter0xf075',
u'\uad77': 'HangulSyllableGulb',
u'\u4bf8': 'CjkUnifiedIdeograph4bf8',
u'\u08fa': 'UnicodeCharacter0x8fa',
u'\uc5fc': 'HangulSyllableYeom',
u'\u82fe': 'CjkUnifiedIdeograph82fe',
u'\uc000': 'HangulSyllableBbweok',
u'\ue483': 'UnicodeCharacter0xe483',
u'\u0302': 'CombiningCircumflexAccent',
u'\u2785': 'DingbatCircledSansserifDigitSix',
u'\u8906': 'CjkUnifiedIdeograph8906',
u'\uad89': 'HangulSyllableGweog',
u'\u520c': 'CjkUnifiedIdeograph520c',
u'\u768f': 'CjkUnifiedIdeograph768f',
u'\ud810': 'UnicodeCharacter0xd810',
u'\ufc93': 'ArabicLigatureYehWithMeemFinalForm',
u'\u1b12': 'BalineseLetterOkaraTedung',
u'\u3f95': 'CjkUnifiedIdeograph3f95',
u'\ua116': 'YiSyllableDu',
u'\u4599': 'CjkUnifiedIdeograph4599',
u'\ucb9d': 'HangulSyllableJjwig',
u'\u6a1c': 'CjkUnifiedIdeograph6a1c',
u'\u0e9f': 'LaoLetterFoSung',
u'\uf020': 'UnicodeCharacter0xf020',
u'\u94a3': 'CjkUnifiedIdeograph94a3',
u'\u3322': 'SquareSenti',
u'\ub926': 'HangulSyllableRwenh',
u'\u5da9': 'CjkUnifiedIdeograph5da9',
u'\ue3ad': 'UnicodeCharacter0xe3ad',
u'\u022c': 'LatinCapitalLetterOWithTildeAndMacron',
u'\u26af': 'UnmarriedPartnershipSymbol',
u'\u8830': 'CjkUnifiedIdeograph8830',
u'\uacb3': 'HangulSyllableGyeolb',
u'\u5136': 'CjkUnifiedIdeograph5136',
u'\u75b9': 'CjkUnifiedIdeograph75b9',
u'\ud73a': 'HangulSyllableHyunh',
u'\ufbbd': 'UnicodeCharacter0xfbbd',
u'\u1a3c': 'UnicodeCharacter0x1a3c',
u'\u3ebf': 'CjkUnifiedIdeograph3ebf',
u'\ua040': 'YiSyllablePax',
u'\u44c3': 'CjkUnifiedIdeograph44c3',
u'\ucac7': 'HangulSyllableJjolb',
u'\u6946': 'CjkUnifiedIdeograph6946',
u'\u0dc9': 'UnicodeCharacter0xdc9',
u'\uef4a': 'UnicodeCharacter0xef4a',
u'\u93cd': 'CjkUnifiedIdeograph93cd',
u'\u324c': 'UnicodeCharacter0x324c',
u'\ub850': 'HangulSyllableRyem',
u'\u5cd3': 'CjkUnifiedIdeograph5cd3',
u'\ue2d7': 'UnicodeCharacter0xe2d7',
u'\u0156': 'LatinCapitalLetterRWithCedilla',
u'\u25d9': 'InverseWhiteCircle',
u'\u875a': 'CjkUnifiedIdeograph875a',
u'\uabdd': 'UnicodeCharacter0xabdd',
u'\u5060': 'CjkUnifiedIdeograph5060',
u'\u74e3': 'CjkUnifiedIdeograph74e3',
u'\ud664': 'HangulSyllableHwam',
u'\ufae7': 'UnicodeCharacter0xfae7',
u'\u1966': 'TaiLeLetterEh',
u'\u3de9': 'CjkUnifiedIdeograph3de9',
u'\u9f6a': 'CjkUnifiedIdeograph9f6a',
u'\u43ed': 'CjkUnifiedIdeograph43ed',
u'\uc9f1': 'HangulSyllableJjang',
u'\u6870': 'CjkUnifiedIdeograph6870',
u'\u0cf3': 'UnicodeCharacter0xcf3',
u'\uee74': 'UnicodeCharacter0xee74',
u'\u92f7': 'CjkUnifiedIdeograph92f7',
u'\u3176': 'HangulLetterPieupcieuc',
u'\ub77a': 'HangulSyllableDdip',
u'\u5bfd': 'CjkUnifiedIdeograph5bfd',
u'\u6101': 'CjkUnifiedIdeograph6101',
u'\x80': 'UnicodeCharacter0x80',
u'\ue705': 'UnicodeCharacter0xe705',
u'\u8684': 'CjkUnifiedIdeograph8684',
u'\u2a07': 'TwoLogicalAndOperator',
u'\ub00b': 'HangulSyllableGgweh',
u'\u4f8a': 'CjkUnifiedIdeograph4f8a',
u'\ud58e': 'HangulSyllableHaep',
u'\u7911': 'CjkUnifiedIdeograph7911',
u'\u1890': 'MongolianLetterAliGaliTa',
u'\uff15': 'FullwidthDigitFive',
u'\u9e94': 'CjkUnifiedIdeograph9e94',
u'\u481b': 'CjkUnifiedIdeograph481b',
u'\u679a': 'CjkUnifiedIdeograph679a',
u'\uce1f': 'HangulSyllableCyuh',
u'\ued9e': 'UnicodeCharacter0xed9e',
u'\u1121': 'HangulChoseongPieupsios',
u'\u30a0': 'KatakanahiraganaDoubleHyphen',
u'\u9725': 'CjkUnifiedIdeograph9725',
u'\ub6a4': 'HangulSyllableDdul',
u'\u602b': 'CjkUnifiedIdeograph602b',
u'\u7faa': 'CjkUnifiedIdeograph7faa',
u'\ue62f': 'UnicodeCharacter0xe62f',
u'\u85ae': 'CjkUnifiedIdeograph85ae',
u'\u2931': 'NorthEastArrowCrossingNorthWestArrow',
u'\uaf35': 'HangulSyllableGgolg',
u'\u4eb4': 'CjkUnifiedIdeograph4eb4',
u'\ud4b8': 'HangulSyllablePwel',
u'\u783b': 'CjkUnifiedIdeograph783b',
u'\u17ba': 'KhmerVowelSignYy',
u'\ufe3f': 'PresentationFormForVerticalLeftAngleBracket',
u'\u9dbe': 'CjkUnifiedIdeograph9dbe',
u'\u4745': 'CjkUnifiedIdeograph4745',
u'\u66c4': 'CjkUnifiedIdeograph66c4',
u'\ucd49': 'HangulSyllableCwaelg',
u'\uecc8': 'UnicodeCharacter0xecc8',
u'\u104b': 'MyanmarSignSection',
u'\u2fca': 'KangxiRadicalBlack',
u'\u964f': 'CjkUnifiedIdeograph964f',
u'\ub5ce': 'HangulSyllableDdebs',
u'\u5f55': 'CjkUnifiedIdeograph5f55',
u'\u7ed4': 'CjkUnifiedIdeograph7ed4',
u'\ue559': 'UnicodeCharacter0xe559',
u'\u84d8': 'CjkUnifiedIdeograph84d8',
u'\u285b': 'BraillePatternDots12457',
u'\uae5f': 'HangulSyllableGgas',
u'\u4dde': 'HexagramForInfluence',
u'\ud3e2': 'HangulSyllablePyebs',
u'\u7765': 'CjkUnifiedIdeograph7765',
u'\u16e4': 'RunicLetterCealc',
u'\ufd69': 'ArabicLigatureSheenWithJeemWithYehFinalForm',
u'\u9ce8': 'CjkUnifiedIdeograph9ce8',
u'\u466f': 'CjkUnifiedIdeograph466f',
u'\u65ee': 'CjkUnifiedIdeograph65ee',
u'\ucc73': 'HangulSyllableCyas',
u'\uebf2': 'UnicodeCharacter0xebf2',
u'\u0f75': 'TibetanVowelSignUu',
u'\u2ef4': 'UnicodeCharacter0x2ef4',
u'\u9579': 'CjkUnifiedIdeograph9579',
u'\ub4f8': 'HangulSyllableDyi',
u'\u5e7f': 'CjkUnifiedIdeograph5e7f',
u'\u7dfe': 'CjkUnifiedIdeograph7dfe',
u'\u0806': 'UnicodeCharacter0x806',
u'\u2c89': 'CopticSmallLetterEie',
u'\u4d08': 'CjkUnifiedIdeograph4d08',
u'\u718b': 'CjkUnifiedIdeograph718b',
u'\u8e0a': 'CjkUnifiedIdeograph8e0a',
u'\ub28d': 'HangulSyllableNyut',
u'\ud30c': 'HangulSyllablePa',
u'\uf78f': 'UnicodeCharacter0xf78f',
u'\u2016': 'DoubleVerticalLine',
u'\u6518': 'CjkUnifiedIdeograph6518',
u'\u099b': 'BengaliLetterCha',
u'\ua61a': 'VaiSymbolDang',
u'\u4a9d': 'CjkUnifiedIdeograph4a9d',
u'\ueb1c': 'UnicodeCharacter0xeb1c',
u'\u8f9f': 'CjkUnifiedIdeograph8f9f',
u'\ud0a1': 'HangulSyllableKyit',
u'\u3826': 'CjkUnifiedIdeograph3826',
u'\u7d28': 'CjkUnifiedIdeograph7d28',
u'\u21ab': 'LeftwardsArrowWithLoop',
u'\ube2a': 'HangulSyllableByigg',
u'\u62ad': 'CjkUnifiedIdeograph62ad',
u'\u832c': 'CjkUnifiedIdeograph832c',
u'\ua7af': 'UnicodeCharacter0xa7af',
u'\uc42e': 'HangulSyllableSsoegg',
u'\ue8b1': 'UnicodeCharacter0xe8b1',
u'\u1538': 'CanadianSyllabicsWestcreeYwoo',
u'\u39bb': 'CjkUnifiedIdeograph39bb',
u'\u563a': 'CjkUnifiedIdeograph563a',
u'\u7abd': 'CjkUnifiedIdeograph7abd',
u'\u9b3c': 'CjkUnifiedIdeograph9b3c',
u'\ubfbf': 'HangulSyllableBbyolh',
u'\udc3e': 'UnicodeCharacter0xdc3e',
u'\u80c1': 'CjkUnifiedIdeograph80c1',
u'\uc5c3': 'HangulSyllableEolh',
u'\u2d48': 'TifinaghLetterTuaregYaq',
u'\u6e4a': 'CjkUnifiedIdeograph6e4a',
u'\u12cd': 'EthiopicSyllableWe',
u'\ub34c': 'HangulSyllableDyaess',
u'\u57cf': 'CjkUnifiedIdeograph57cf',
u'\uf44e': 'UnicodeCharacter0xf44e',
u'\u98d1': 'CjkUnifiedIdeograph98d1',
u'\uddd3': 'UnicodeCharacter0xddd3',
u'\u065a': 'ArabicVowelSignSmallVAbove',
u'\u2add': 'Nonforking',
u'\u4b5c': 'CjkUnifiedIdeograph4b5c',
u'\u6fdf': 'CjkUnifiedIdeograph6fdf',
u'\u8c5e': 'CjkUnifiedIdeograph8c5e',
u'\ub0e1': 'HangulSyllableNyab',
u'\ud160': 'HangulSyllableTess',
u'\uf5e3': 'UnicodeCharacter0xf5e3',
u'\u1e6a': 'LatinCapitalLetterTWithDotAbove',
u'\u636c': 'CjkUnifiedIdeograph636c',
u'\u07ef': 'NkoCombiningLongHighTone',
u'\ua46e': 'YiSyllableXyp',
u'\u48f1': 'CjkUnifiedIdeograph48f1',
u'\ue970': 'UnicodeCharacter0xe970',
u'\u8df3': 'CjkUnifiedIdeograph8df3',
u'\ucef5': 'HangulSyllableKeob',
u'\u367a': 'CjkUnifiedIdeograph367a',
u'\u7b7c': 'CjkUnifiedIdeograph7b7c',
u'\u1fff': 'UnicodeCharacter0x1fff',
u'\ubc7e': 'HangulSyllableByaej',
u'\u8180': 'CjkUnifiedIdeograph8180',
u'\u2503': 'BoxDrawingsHeavyVertical',
u'\uc282': 'HangulSyllableSwij',
u'\u6605': 'CjkUnifiedIdeograph6605',
u'\uab07': 'UnicodeCharacter0xab07',
u'\uec09': 'UnicodeCharacter0xec09',
u'\u138c': 'EthiopicSyllableSebatbeitPwa',
u'\u548e': 'CjkUnifiedIdeograph548e',
u'\u9990': 'CjkUnifiedIdeograph9990',
u'\u3d13': 'CjkUnifiedIdeograph3d13',
u'\uda92': 'UnicodeCharacter0xda92',
u'\u7e15': 'CjkUnifiedIdeograph7e15',
u'\u4317': 'CjkUnifiedIdeograph4317',
u'\u8419': 'CjkUnifiedIdeograph8419',
u'\uc91b': 'HangulSyllableJweogs',
u'\u2b9c': 'UnicodeCharacter0x2b9c',
u'\u6c9e': 'CjkUnifiedIdeograph6c9e',
u'\ub1a0': 'HangulSyllableNwals',
u'\uf2a2': 'UnicodeCharacter0xf2a2',
u'\u1625': 'CanadianSyllabicsCarrierLee',
u'\u5b27': 'CjkUnifiedIdeograph5b27',
u'\u9c29': 'CjkUnifiedIdeograph9c29',
u'\ue12b': 'UnicodeCharacter0xe12b',
u'\u04ae': 'CyrillicCapitalLetterStraightU',
u'\u49b0': 'CjkUnifiedIdeograph49b0',
u'\u8ab2': 'CjkUnifiedIdeograph8ab2',
u'\u2e35': 'UnicodeCharacter0x2e35',
u'\ucfb4': 'HangulSyllableKoels',
u'\u7337': 'CjkUnifiedIdeograph7337',
u'\ub439': 'HangulSyllableDyonj',
u'\uf93b': 'CjkCompatibilityIdeographf93b',
u'\u1cbe': 'UnicodeCharacter0x1cbe',
u'\u61c0': 'CjkUnifiedIdeograph61c0',
u'\ua2c2': 'YiSyllableCe',
u'\ue7c4': 'UnicodeCharacter0xe7c4',
u'\u0b47': 'OriyaVowelSignE',
u'\u4c49': 'CjkUnifiedIdeograph4c49',
u'\u914b': 'CjkUnifiedIdeograph914b',
u'\ud24d': 'HangulSyllableTweonj',
u'\u34ce': 'CjkUnifiedIdeograph34ce',
u'\u79d0': 'CjkUnifiedIdeograph79d0',
u'\ubad2': 'HangulSyllableMwalp',
u'\uffd4': 'HalfwidthHangulLetterWeo',
u'\u2357': 'AplFunctionalSymbolQuadDownwardsArrow',
u'\uc0d6': 'HangulSyllableSaelp',
u'\u6459': 'CjkUnifiedIdeograph6459',
u'\ua95b': 'UnicodeCharacter0xa95b',
u'\uea5d': 'UnicodeCharacter0xea5d',
u'\u11e0': 'HangulJongseongMieumchieuch',
u'\u52e2': 'CjkUnifiedIdeograph52e2',
u'\u97e4': 'CjkUnifiedIdeograph97e4',
u'\u3b67': 'CjkUnifiedIdeograph3b67',
u'\ud8e6': 'UnicodeCharacter0xd8e6',
u'\u7c69': 'CjkUnifiedIdeograph7c69',
u'\u416b': 'CjkUnifiedIdeograph416b',
u'\u826d': 'CjkUnifiedIdeograph826d',
u'\uc76f': 'HangulSyllableYic',
u'\u29f0': 'ErrorbarredWhiteDiamond',
u'\u6af2': 'CjkUnifiedIdeograph6af2',
u'\uaff4': 'HangulSyllableGgwen',
u'\uf0f6': 'UnicodeCharacter0xf0f6',
u'\u1479': 'CanadianSyllabicsWestcreeKwii',
u'\u597b': 'CjkUnifiedIdeograph597b',
u'\u9a7d': 'CjkUnifiedIdeograph9a7d',
u'\udf7f': 'UnicodeCharacter0xdf7f',
u'\ua381': 'YiSyllableRrox',
u'\u4404': 'CjkUnifiedIdeograph4404',
u'\u6887': 'CjkUnifiedIdeograph6887',
u'\uce08': 'HangulSyllableCyun',
u'\uf28b': 'UnicodeCharacter0xf28b',
u'\u0d0a': 'MalayalamLetterUu',
u'\u318d': 'HangulLetterAraea',
u'\u970e': 'CjkUnifiedIdeograph970e',
u'\ubb91': 'HangulSyllableMwilg',
u'\u5c14': 'CjkUnifiedIdeograph5c14',
u'\x97': 'UnicodeCharacter0x97',
u'\ue618': 'UnicodeCharacter0xe618',
u'\u8a9b': 'CjkUnifiedIdeograph8a9b',
u'\u251a': 'BoxDrawingsUpHeavyAndLeftLight',
u'\uaf1e': 'HangulSyllableGgyelp',
u'\u53a1': 'CjkUnifiedIdeograph53a1',
u'\ud5a5': 'HangulSyllableHyang',
u'\u7424': 'CjkUnifiedIdeograph7424',
u'\u18a7': 'MongolianLetterAliGaliHalfYa',
u'\ufe28': 'UnicodeCharacter0xfe28',
u'\ua2ab': 'YiSyllableZyp',
u'\u3d2a': 'CjkUnifiedIdeograph3d2a',
u'\u472e': 'CjkUnifiedIdeograph472e',
u'\u6bb1': 'CjkUnifiedIdeograph6bb1',
u'\uc932': 'HangulSyllableJweop',
u'\uedb5': 'UnicodeCharacter0xedb5',
u'\u0c34': 'UnicodeCharacter0xc34',
u'\u30b7': 'KatakanaLetterSi',
u'\u9638': 'CjkUnifiedIdeograph9638',
u'\ubabb': 'HangulSyllableMos',
u'\u5f3e': 'CjkUnifiedIdeograph5f3e',
u'\u03c1': 'GreekSmallLetterRho',
u'\ue142': 'UnicodeCharacter0xe142',
u'\u85c5': 'CjkUnifiedIdeograph85c5',
u'\u2444': 'OcrBeltBuckle',
u'\uae48': 'HangulSyllableGik',
u'\u52cb': 'CjkUnifiedIdeograph52cb',
u'\ud4cf': 'HangulSyllablePwigs',
u'\u774e': 'CjkUnifiedIdeograph774e',
u'\u1bd1': 'UnicodeCharacter0x1bd1',
u'\uf952': 'CjkCompatibilityIdeographf952',
u'\u9dd5': 'CjkUnifiedIdeograph9dd5',
u'\u3c54': 'CjkUnifiedIdeograph3c54',
u'\u4658': 'CjkUnifiedIdeograph4658',
u'\u6adb': 'CjkUnifiedIdeograph6adb',
u'\uc85c': 'HangulSyllableJyel',
u'\uecdf': 'UnicodeCharacter0xecdf',
u'\u0f5e': 'TibetanLetterZha',
u'\u33e1': 'IdeographicTelegraphSymbolForDayTwo',
u'\u9162': 'CjkUnifiedIdeograph9162',
u'\ub5e5': 'HangulSyllableDdyeolt',
u'\u5e68': 'CjkUnifiedIdeograph5e68',
u'\u02eb': 'ModifierLetterYangDepartingToneMark',
u'\ue06c': 'UnicodeCharacter0xe06c',
u'\u84ef': 'CjkUnifiedIdeograph84ef',
u'\u276e': 'HeavyLeftpointingAngleQuotationMarkOrnament',
u'\ua972': 'UnicodeCharacter0xa972',
u'\u4df5': 'HexagramForTheMarryingMaiden',
u'\ud7f9': 'UnicodeCharacter0xd7f9',
u'\u7678': 'CjkUnifiedIdeograph7678',
u'\u1afb': 'UnicodeCharacter0x1afb',
u'\uf87c': 'UnicodeCharacter0xf87c',
u'\u9cff': 'CjkUnifiedIdeograph9cff',
u'\u3f7e': 'CjkUnifiedIdeograph3f7e',
u'\ua603': 'VaiSyllableKe',
u'\u4182': 'CjkUnifiedIdeograph4182',
u'\ucb86': 'HangulSyllableJjwenh',
u'\u6f09': 'CjkUnifiedIdeograph6f09',
u'\u0e88': 'LaoLetterCo',
u'\uf10d': 'UnicodeCharacter0xf10d',
u'\u908c': 'CjkUnifiedIdeograph908c',
u'\u340f': 'CjkUnifiedIdeograph340f',
u'\ube13': 'HangulSyllableBeud',
u'\u5992': 'CjkUnifiedIdeograph5992',
u'\uc017': 'HangulSyllableBbwes',
u'\ue396': 'UnicodeCharacter0xe396',
u'\u0719': 'SyriacLetterZain',
u'\u2698': 'Flower',
u'\u891d': 'CjkUnifiedIdeograph891d',
u'\ua89c': 'SaurashtraLetterTta',
u'\u5623': 'CjkUnifiedIdeograph5623',
u'\u71a2': 'CjkUnifiedIdeograph71a2',
u'\ud827': 'UnicodeCharacter0xd827',
u'\ufba6': 'ArabicLetterHehGoalIsolatedForm',
u'\u1f29': 'GreekCapitalLetterEtaWithDasia',
u'\u3ea8': 'CjkUnifiedIdeograph3ea8',
u'\ua12d': 'YiSyllableTex',
u'\u40ac': 'CjkUnifiedIdeograph40ac',
u'\ucab0': 'HangulSyllableJjyem',
u'\u6e33': 'CjkUnifiedIdeograph6e33',
u'\u09b2': 'BengaliLetterLa',
u'\uf037': 'UnicodeCharacter0xf037',
u'\u93b6': 'CjkUnifiedIdeograph93b6',
u'\u3739': 'CjkUnifiedIdeograph3739',
u'\ub93d': 'HangulSyllableRwig',
u'\u58bc': 'CjkUnifiedIdeograph58bc',
u'\uc341': 'HangulSyllableSsyab',
u'\ue2c0': 'UnicodeCharacter0xe2c0',
u'\u0643': 'ArabicLetterKaf',
u'\u21c2': 'DownwardsHarpoonWithBarbRightwards',
u'\u8847': 'CjkUnifiedIdeograph8847',
u'\uabc6': 'UnicodeCharacter0xabc6',
u'\u514d': 'CjkUnifiedIdeograph514d',
u'\u70cc': 'CjkUnifiedIdeograph70cc',
u'\udb51': 'UnicodeCharacter0xdb51',
u'\ufad0': 'CjkCompatibilityIdeographfad0',
u'\u1e53': 'LatinSmallLetterOWithMacronAndAcute',
u'\u39d2': 'CjkUnifiedIdeograph39d2',
u'\ua057': 'YiSyllableBbix',
u'\u43d6': 'CjkUnifiedIdeograph43d6',
u'\uc5da': 'HangulSyllableElm',
u'\u695d': 'CjkUnifiedIdeograph695d',
u'\u08dc': 'UnicodeCharacter0x8dc',
u'\uf361': 'UnicodeCharacter0xf361',
u'\u92e0': 'CjkUnifiedIdeograph92e0',
u'\u3663': 'CjkUnifiedIdeograph3663',
u'\ub867': 'HangulSyllableRolb',
u'\u5be6': 'CjkUnifiedIdeograph5be6',
u'\uc26b': 'HangulSyllableSweh',
u'\uddea': 'UnicodeCharacter0xddea',
u'\u016d': 'LatinSmallLetterUWithBreve',
u'\u20ec': 'CombiningRightwardsHarpoonWithBarbDownwards',
u'\u8b71': 'CjkUnifiedIdeograph8b71',
u'\uaaf0': 'UnicodeCharacter0xaaf0',
u'\u5077': 'CjkUnifiedIdeograph5077',
u'\u73f6': 'CjkUnifiedIdeograph73f6',
u'\uda7b': 'UnicodeCharacter0xda7b',
u'\uf5fa': 'UnicodeCharacter0xf5fa',
u'\u197d': 'UnicodeCharacter0x197d',
u'\u38fc': 'CjkUnifiedIdeograph38fc',
u'\u2281': 'DoesNotSucceed',
u'\u4300': 'CjkUnifiedIdeograph4300',
u'\u6783': 'CjkUnifiedIdeograph6783',
u'\u8002': 'CjkUnifiedIdeograph8002',
u'\ua485': 'YiSyllableYurx',
u'\uc504': 'HangulSyllableSseuss',
u'\ue987': 'UnicodeCharacter0xe987',
u'\u160e': 'CanadianSyllabicsCarrierYu',
u'\u3a91': 'CjkUnifiedIdeograph3a91',
u'\u5b10': 'CjkUnifiedIdeograph5b10',
u'\u7f93': 'CjkUnifiedIdeograph7f93',
u'\u9812': 'CjkUnifiedIdeograph9812',
u'\ubc95': 'HangulSyllableBeob',
u'\udd14': 'UnicodeCharacter0xdd14',
u'\u8197': 'CjkUnifiedIdeograph8197',
u'\uc699': 'HangulSyllableYonj',
u'\u2e1e': 'TildeWithDotAbove',
u'\u7320': 'CjkUnifiedIdeograph7320',
u'\u17a3': 'KhmerIndependentVowelQaq',
u'\ub022': 'HangulSyllableGgwij',
u'\u54a5': 'CjkUnifiedIdeograph54a5',
u'\uf524': 'UnicodeCharacter0xf524',
u'\u99a7': 'CjkUnifiedIdeograph99a7',
u'\udea9': 'UnicodeCharacter0xdea9',
u'\u0b30': 'OriyaLetterRa',
u'\u2fb3': 'KangxiRadicalSound',
u'\u4832': 'CjkUnifiedIdeograph4832',
u'\u6cb5': 'CjkUnifiedIdeograph6cb5',
u'\u8d34': 'CjkUnifiedIdeograph8d34',
u'\ub1b7': 'HangulSyllableNwaed',
u'\ud236': 'HangulSyllableTulm',
u'\uf6b9': 'UnicodeCharacter0xf6b9',
u'\u2340': 'AplFunctionalSymbolBackslashBar',
u'\u6042': 'CjkUnifiedIdeograph6042',
u'\u04c5': 'CyrillicCapitalLetterElWithTail',
u'\ua544': 'VaiSyllableNggi',
u'\u49c7': 'CjkUnifiedIdeograph49c7',
u'\uea46': 'UnicodeCharacter0xea46',
u'\u8ec9': 'CjkUnifiedIdeograph8ec9',
u'\ud3cb': 'HangulSyllablePyeoc',
u'\u3b50': 'CjkUnifiedIdeograph3b50',
u'\u7852': 'CjkUnifiedIdeograph7852',
u'\u1cd5': 'UnicodeCharacter0x1cd5',
u'\ubd54': 'HangulSyllableBoels',
u'\u61d7': 'CjkUnifiedIdeograph61d7',
u'\u8256': 'CjkUnifiedIdeograph8256',
u'\ua6d9': 'UnicodeCharacter0xa6d9',
u'\uc758': 'HangulSyllableYi',
u'\uebdb': 'UnicodeCharacter0xebdb',
u'\u1062': 'MyanmarVowelSignSgawKarenEu',
u'\u34e5': 'CjkUnifiedIdeograph34e5',
u'\u5564': 'CjkUnifiedIdeograph5564',
u'\u79e7': 'CjkUnifiedIdeograph79e7',
u'\u9a66': 'CjkUnifiedIdeograph9a66',
u'\ubee9': 'HangulSyllableBbeot',
u'\udf68': 'UnicodeCharacter0xdf68',
u'\u83eb': 'CjkUnifiedIdeograph83eb',
u'\uc0ed': 'HangulSyllableSyalg',
u'\u2872': 'BraillePatternDots2567',
u'\u6d74': 'CjkUnifiedIdeograph6d74',
u'\u11f7': 'HangulJongseongHieuhmieum',
u'\ub276': 'HangulSyllableNyugg',
u'\u56f9': 'CjkUnifiedIdeograph56f9',
u'\uf778': 'UnicodeCharacter0xf778',
u'\u9bfb': 'CjkUnifiedIdeograph9bfb',
u'\ud8fd': 'UnicodeCharacter0xd8fd',
u'\ue201': 'UnicodeCharacter0xe201',
u'\u0584': 'ArmenianSmallLetterKeh',
u'\u4a86': 'CjkUnifiedIdeograph4a86',
u'\u8f88': 'CjkUnifiedIdeograph8f88',
u'\u330b': 'SquareKairi',
u'\ucc8a': 'HangulSyllableCyaelp',
u'\u700d': 'CjkUnifiedIdeograph700d',
u'\ub50f': 'HangulSyllableDyic',
u'\ufa11': 'CjkCompatibilityIdeographfa11',
u'\u1d94': 'LatinSmallLetterReversedOpenEWithRetroflexHook',
u'\u6296': 'CjkUnifiedIdeograph6296',
u'\ua798': 'UnicodeCharacter0xa798',
u'\ue49a': 'UnicodeCharacter0xe49a',
u'\u081d': 'UnicodeCharacter0x81d',
u'\u4d1f': 'CjkUnifiedIdeograph4d1f',
u'\u9221': 'CjkUnifiedIdeograph9221',
u'\ud723': 'HangulSyllableHwilb',
u'\u35a4': 'CjkUnifiedIdeograph35a4',
u'\u7aa6': 'CjkUnifiedIdeograph7aa6',
u'\ubfa8': 'HangulSyllableBboess',
u'\ufcaa': 'ArabicLigatureHahWithMeemInitialForm',
u'\u202d': 'LefttorightOverride',
u'\uc1ac': 'HangulSyllableSwan',
u'\u652f': 'CjkUnifiedIdeograph652f',
u'\uaa31': 'ChamVowelSignAu',
u'\uef33': 'UnicodeCharacter0xef33',
u'\u12b6': 'UnicodeCharacter0x12b6',
u'\u57b8': 'CjkUnifiedIdeograph57b8',
u'\u94ba': 'CjkUnifiedIdeograph94ba',
u'\u383d': 'CjkUnifiedIdeograph383d',
u'\ud9bc': 'UnicodeCharacter0xd9bc',
u'\u7d3f': 'CjkUnifiedIdeograph7d3f',
u'\u4241': 'CjkUnifiedIdeograph4241',
u'\u8743': 'CjkUnifiedIdeograph8743',
u'\uc445': 'HangulSyllableSsoet',
u'\u2ac6': 'SupersetOfAboveEqualsSign',
u'\u6fc8': 'CjkUnifiedIdeograph6fc8',
u'\uacca': 'HangulSyllableGyenh',
u'\uf1cc': 'UnicodeCharacter0xf1cc',
u'\u154f': 'CanadianSyllabicsWestcreeRwaa',
u'\u5a51': 'CjkUnifiedIdeograph5a51',
u'\u9f53': 'CjkUnifiedIdeograph9f53',
u'\udc55': 'UnicodeCharacter0xdc55',
u'\u07d8': 'NkoLetterDa',
u'\u44da': 'CjkUnifiedIdeograph44da',
u'\u89dc': 'CjkUnifiedIdeograph89dc',
u'\u2d5f': 'TifinaghLetterYatt',
u'\ucede': 'HangulSyllableKyaej',
u'\u7261': 'CjkUnifiedIdeograph7261',
u'\ub763': 'HangulSyllableDdigs',
u'\uf465': 'UnicodeCharacter0xf465',
u'\u1fe8': 'GreekCapitalLetterUpsilonWithVrachy',
u'\u5cea': 'CjkUnifiedIdeograph5cea',
u'\ua1ec': 'YiSyllableGo',
u'\ue6ee': 'UnicodeCharacter0xe6ee',
u'\u0a71': 'GurmukhiAddak',
u'\u4f73': 'CjkUnifiedIdeograph4f73',
u'\u8c75': 'CjkUnifiedIdeograph8c75',
u'\ud177': 'HangulSyllableTyeolh',
u'\u37f8': 'CjkUnifiedIdeograph37f8',
u'\u74fa': 'CjkUnifiedIdeograph74fa',
u'\ub9fc': 'HangulSyllableMaek',
u'\ufefe': 'UnicodeCharacter0xfefe',
u'\uc400': 'HangulSyllableSswals',
u'\ue083': 'UnicodeCharacter0xe083',
u'\u0702': 'SyriacSublinearFullStop',
u'\u2385': 'WhiteSquareWithCentreVerticalLine',
u'\u8d06': 'CjkUnifiedIdeograph8d06',
u'\ua989': 'UnicodeCharacter0xa989',
u'\u560c': 'CjkUnifiedIdeograph560c',
u'\u728f': 'CjkUnifiedIdeograph728f',
u'\udc10': 'UnicodeCharacter0xdc10',
u'\uf893': 'UnicodeCharacter0xf893',
u'\u1f12': 'GreekSmallLetterEpsilonWithPsiliAndVaria',
u'\u3b95': 'CjkUnifiedIdeograph3b95',
u'\ua516': 'VaiSyllableSee',
u'\u4199': 'CjkUnifiedIdeograph4199',
u'\ucf9d': 'HangulSyllableKwaeb',
u'\u6e1c': 'CjkUnifiedIdeograph6e1c',
u'\u0a9f': 'GujaratiLetterTta',
u'\uf420': 'UnicodeCharacter0xf420',
u'\u90a3': 'CjkUnifiedIdeograph90a3',
u'\u3722': 'CjkUnifiedIdeograph3722',
u'\ubd26': 'HangulSyllableBwaj',
u'\u59a9': 'CjkUnifiedIdeograph59a9',
u'\ue7ad': 'UnicodeCharacter0xe7ad',
u'\u062c': 'ArabicLetterJeem',
u'\u22af': 'NegatedDoubleVerticalBarDoubleRightTurnstile',
u'\u8c30': 'CjkUnifiedIdeograph8c30',
u'\ua8b3': 'SaurashtraLetterLla',
u'\u5536': 'CjkUnifiedIdeograph5536',
u'\u71b9': 'CjkUnifiedIdeograph71b9',
u'\ud33a': 'HangulSyllablePaebs',
u'\uffbd': 'HalfwidthHangulLetterPhieuph',
u'\u1e3c': 'LatinCapitalLetterLWithCircumflexBelow',
u'\u3abf': 'CjkUnifiedIdeograph3abf',
u'\ua440': 'YiSyllableNju',
u'\u40c3': 'CjkUnifiedIdeograph40c3',
u'\ucec7': 'HangulSyllableKyah',
u'\u6d46': 'CjkUnifiedIdeograph6d46',
u'\u09c9': 'UnicodeCharacter0x9c9',
u'\ueb4a': 'UnicodeCharacter0xeb4a',
u'\u97cd': 'CjkUnifiedIdeograph97cd',
u'\u364c': 'CjkUnifiedIdeograph364c',
u'\ubc50': 'HangulSyllableByan',
u'\u58d3': 'CjkUnifiedIdeograph58d3',
u'\ue6d7': 'UnicodeCharacter0xe6d7',
u'\u0556': 'ArmenianCapitalLetterFeh',
u'\u21d9': 'SouthWestDoubleArrow',
u'\u835a': 'CjkUnifiedIdeograph835a',
u'\uafdd': 'HangulSyllableGgweolg',
u'\u5460': 'CjkUnifiedIdeograph5460',
u'\u70e3': 'CjkUnifiedIdeograph70e3',
u'\ud264': 'HangulSyllableTwe',
u'\ufee7': 'ArabicLetterNoonInitialForm',
u'\u1d66': 'GreekSubscriptSmallLetterBeta',
u'\u39e9': 'CjkUnifiedIdeograph39e9',
u'\u9b6a': 'CjkUnifiedIdeograph9b6a',
u'\u47ed': 'CjkUnifiedIdeograph47ed',
u'\ucdf1': 'HangulSyllableCwilg',
u'\u6c70': 'CjkUnifiedIdeograph6c70',
u'\u08f3': 'UnicodeCharacter0x8f3',
u'\uea74': 'UnicodeCharacter0xea74',
u'\u96f7': 'CjkUnifiedIdeograph96f7',
u'\u3576': 'CjkUnifiedIdeograph3576',
u'\ub37a': 'HangulSyllableDelm',
u'\u5ffd': 'CjkUnifiedIdeograph5ffd',
u'\u6501': 'CjkUnifiedIdeograph6501',
u'\u0480': 'CyrillicCapitalLetterKoppa',
u'\ue305': 'UnicodeCharacter0xe305',
u'\u8284': 'CjkUnifiedIdeograph8284',
u'\u2e07': 'RaisedDottedInterpolationMarker',
u'\ub40b': 'HangulSyllableDwaelh',
u'\u4b8a': 'CjkUnifiedIdeograph4b8a',
u'\ud18e': 'HangulSyllableTyelm',
u'\u7d11': 'CjkUnifiedIdeograph7d11',
u'\u1c90': 'UnicodeCharacter0x1c90',
u'\ufb15': 'ArmenianSmallLigatureMenIni',
u'\u9a94': 'CjkUnifiedIdeograph9a94',
u'\u4c1b': 'CjkUnifiedIdeograph4c1b',
u'\u639a': 'CjkUnifiedIdeograph639a',
u'\uca1f': 'HangulSyllableJjyalb',
u'\ue99e': 'UnicodeCharacter0xe99e',
u'\u1521': 'CanadianSyllabicsShwa',
u'\u34a0': 'CjkUnifiedIdeograph34a0',
u'\u9325': 'CjkUnifiedIdeograph9325',
u'\ub2a4': 'HangulSyllableNeuss',
u'\u642b': 'CjkUnifiedIdeograph642b',
u'\u7baa': 'CjkUnifiedIdeograph7baa',
u'\ue22f': 'UnicodeCharacter0xe22f',
u'\u81ae': 'CjkUnifiedIdeograph81ae',
u'\u2d31': 'TifinaghLetterYab',
u'\uab35': 'UnicodeCharacter0xab35',
u'\u4ab4': 'CjkUnifiedIdeograph4ab4',
u'\ud0b8': 'HangulSyllableKiss',
u'\u7c3b': 'CjkUnifiedIdeograph7c3b',
u'\u13ba': 'CherokeeLetterMe',
u'\ufa3f': 'CjkCompatibilityIdeographfa3f',
u'\u99be': 'CjkUnifiedIdeograph99be',
u'\u4345': 'CjkUnifiedIdeograph4345',
u'\u62c4': 'CjkUnifiedIdeograph62c4',
u'\uc949': 'HangulSyllableJweng',
u'\ue8c8': 'UnicodeCharacter0xe8c8',
u'\u144b': 'CanadianSyllabicsCarrierH',
u'\u2bca': 'UnicodeCharacter0x2bca',
u'\u924f': 'CjkUnifiedIdeograph924f',
u'\ub1ce': 'HangulSyllableNoegg',
u'\u5b55': 'CjkUnifiedIdeograph5b55',
u'\u7ad4': 'CjkUnifiedIdeograph7ad4',
u'\ue159': 'UnicodeCharacter0xe159',
u'\u80d8': 'CjkUnifiedIdeograph80d8',
u'\u2c5b': 'GlagoliticSmallLetterIzhitsa',
u'\uaa5f': 'ChamPunctuationTripleDanda',
u'\u49de': 'CjkUnifiedIdeograph49de',
u'\ud7e2': 'UnicodeCharacter0xd7e2',
u'\u7365': 'CjkUnifiedIdeograph7365',
u'\u12e4': 'EthiopicSyllableZhee',
u'\uf969': 'CjkCompatibilityIdeographf969',
u'\u98e8': 'CjkUnifiedIdeograph98e8',
u'\u426f': 'CjkUnifiedIdeograph426f',
u'\u61ee': 'CjkUnifiedIdeograph61ee',
u'\uc873': 'HangulSyllableJogs',
u'\ueff2': 'UnicodeCharacter0xeff2',
u'\u0b75': 'UnicodeCharacter0xb75',
u'\u2af4': 'TripleVerticalBarBinaryRelation',
u'\u9179': 'CjkUnifiedIdeograph9179',
u'\ub0f8': 'HangulSyllableNyaels',
u'\u5a7f': 'CjkUnifiedIdeograph5a7f',
u'\u79fe': 'CjkUnifiedIdeograph79fe',
u'\u0c06': 'TeluguLetterAa',
u'\u2889': 'BraillePatternDots148',
u'\u4908': 'CjkUnifiedIdeograph4908',
u'\u758b': 'CjkUnifiedIdeograph758b',
u'\u8a0a': 'CjkUnifiedIdeograph8a0a',
u'\ub68d': 'HangulSyllableDdyolt',
u'\ud70c': 'HangulSyllableHwem',
u'\uf38f': 'UnicodeCharacter0xf38f',
u'\u2416': 'SymbolForSynchronousIdle',
u'\u6118': 'CjkUnifiedIdeograph6118',
u'\u0d9b': 'SinhalaLetterMahaapraanaKayanna',
u'\ua21a': 'YiSyllableGgat',
u'\u4e9d': 'CjkUnifiedIdeograph4e9d',
u'\uef1c': 'UnicodeCharacter0xef1c',
u'\u8b9f': 'CjkUnifiedIdeograph8b9f',
u'\ud4a1': 'HangulSyllablePweolt',
u'\u3c26': 'CjkUnifiedIdeograph3c26',
u'\u7928': 'CjkUnifiedIdeograph7928',
u'\u25ab': 'WhiteSmallSquare',
u'\uba2a': 'HangulSyllableMyaelp',
u'\u66ad': 'CjkUnifiedIdeograph66ad',
u'\u872c': 'CjkUnifiedIdeograph872c',
u'\ua3af': 'YiSyllableShuox',
u'\uc02e': 'HangulSyllableBbwilp',
u'\uecb1': 'UnicodeCharacter0xecb1',
u'\u1138': 'HangulChoseongSioskhieukh',
u'\u3dbb': 'CjkUnifiedIdeograph3dbb',
u'\u523a': 'CjkUnifiedIdeograph523a',
u'\u7ebd': 'CjkUnifiedIdeograph7ebd',
u'\u9f3c': 'CjkUnifiedIdeograph9f3c',
u'\ubbbf': 'HangulSyllableMyuh',
u'\ud83e': 'UnicodeCharacter0xd83e',
u'\u84c1': 'CjkUnifiedIdeograph84c1',
u'\uc1c3': 'HangulSyllableSwah',
u'\u2948': 'LeftRightArrowThroughSmallCircle',
u'\u6a4a': 'CjkUnifiedIdeograph6a4a',
u'\u16cd': 'RunicLetterC',
u'\ub74c': 'HangulSyllableDdyil',
u'\u53cf': 'CjkUnifiedIdeograph53cf',
u'\uf04e': 'UnicodeCharacter0xf04e',
u'\u9cd1': 'CjkUnifiedIdeograph9cd1',
u'\ud9d3': 'UnicodeCharacter0xd9d3',
u'\u025a': 'LatinSmallLetterSchwaWithHook',
u'\u2edd': 'CjkRadicalEatOne',
u'\u4f5c': 'CjkUnifiedIdeograph4f5c',
u'\u6bdf': 'CjkUnifiedIdeograph6bdf',
u'\u885e': 'CjkUnifiedIdeograph885e',
u'\ub4e1': 'HangulSyllableDeunj',
u'\ud560': 'HangulSyllableHal',
u'\uf1e3': 'UnicodeCharacter0xf1e3',
u'\u1a6a': 'UnicodeCharacter0x1a6a',
u'\u676c': 'CjkUnifiedIdeograph676c',
u'\u03ef': 'CopticSmallLetterDei',
u'\ua06e': 'YiSyllableBbu',
u'\u4cf1': 'CjkUnifiedIdeograph4cf1',
u'\ued70': 'UnicodeCharacter0xed70',
u'\u89f3': 'CjkUnifiedIdeograph89f3',
u'\ucaf5': 'HangulSyllableJjwaeg',
u'\u327a': 'CircledHangulPhieuphA',
u'\u7f7c': 'CjkUnifiedIdeograph7f7c',
u'\u1bff': 'UnicodeCharacter0x1bff',
u'\ub87e': 'HangulSyllableRwanh',
u'\u8580': 'CjkUnifiedIdeograph8580',
u'\u2103': 'DegreeCelsius',
u'\uc682': 'HangulSyllableOelm',
u'\u6205': 'CjkUnifiedIdeograph6205',
u'\uaf07': 'HangulSyllableGgyeos',
u'\ue809': 'UnicodeCharacter0xe809',
u'\u178c': 'KhmerLetterDo',
u'\u508e': 'CjkUnifiedIdeograph508e',
u'\u9d90': 'CjkUnifiedIdeograph9d90',
u'\u3913': 'CjkUnifiedIdeograph3913',
u'\ude92': 'UnicodeCharacter0xde92',
u'\u7a15': 'CjkUnifiedIdeograph7a15',
u'\u4717': 'CjkUnifiedIdeograph4717',
u'\u8019': 'CjkUnifiedIdeograph8019',
u'\ucd1b': 'HangulSyllableCos',
u'\u2f9c': 'KangxiRadicalFoot',
u'\u689e': 'CjkUnifiedIdeograph689e',
u'\ub5a0': 'HangulSyllableDdeo',
u'\uf6a2': 'UnicodeCharacter0xf6a2',
u'\u1225': 'EthiopicSyllableSze',
u'\u5f27': 'CjkUnifiedIdeograph5f27',
u'\u9829': 'CjkUnifiedIdeograph9829',
u'\ue52b': 'UnicodeCharacter0xe52b',
u'\xae': 'RegisteredSign',
u'\u4db0': 'CjkUnifiedIdeograph4db0',
u'\u8eb2': 'CjkUnifiedIdeograph8eb2',
u'\u2a35': 'MultiplicationSignInRightHalfCircle',
u'\ucbb4': 'HangulSyllableJjwik',
u'\u7737': 'CjkUnifiedIdeograph7737',
u'\ub039': 'HangulSyllableGgyub',
u'\ufd3b': 'ArabicLigatureZahWithMeemMedialForm',
u'\u18be': 'UnicodeCharacter0x18be',
u'\u65c0': 'CjkUnifiedIdeograph65c0',
u'\ua6c2': 'UnicodeCharacter0xa6c2',
u'\ue3c4': 'UnicodeCharacter0xe3c4',
u'\u0f47': 'TibetanLetterJa',
u'\u4849': 'CjkUnifiedIdeograph4849',
u'\u954b': 'CjkUnifiedIdeograph954b',
u'\ud64d': 'HangulSyllableHong',
u'\u30ce': 'KatakanaLetterNo',
u'\u7dd0': 'CjkUnifiedIdeograph7dd0',
u'\ubed2': 'HangulSyllableBbeogg',
u'\ufbd4': 'ArabicLetterNgFinalForm',
u'\u2757': 'UnicodeCharacter0x2757',
u'\uc4d6': 'HangulSyllableSsyugg',
u'\u6059': 'CjkUnifiedIdeograph6059',
u'\uad5b': 'HangulSyllableGyolb',
u'\uee5d': 'UnicodeCharacter0xee5d',
u'\u15e0': 'CanadianSyllabicsCarrierThi',
u'\u56e2': 'CjkUnifiedIdeograph56e2',
u'\u93e4': 'CjkUnifiedIdeograph93e4',
u'\u3f67': 'CjkUnifiedIdeograph3f67',
u'\udce6': 'UnicodeCharacter0xdce6',
u'\u7869': 'CjkUnifiedIdeograph7869',
u'\u456b': 'CjkUnifiedIdeograph456b',
u'\u866d': 'CjkUnifiedIdeograph866d',
u'\uc36f': 'HangulSyllableSseod',
u'\u2df0': 'CombiningCyrillicLetterTse',
u'\u6ef2': 'CjkUnifiedIdeograph6ef2',
u'\uabf4': 'UnicodeCharacter0xabf4',
u'\uf4f6': 'UnicodeCharacter0xf4f6',
u'\u1079': 'MyanmarLetterShanZa',
u'\u5d7b': 'CjkUnifiedIdeograph5d7b',
u'\u9e7d': 'CjkUnifiedIdeograph9e7d',
u'\udb7f': 'UnicodeCharacter0xdb7f',
u'\ua781': 'LatinSmallLetterTurnedL',
u'\u4004': 'CjkUnifiedIdeograph4004',
u'\u6c87': 'CjkUnifiedIdeograph6c87',
u'\uca08': 'HangulSyllableJjaem',
u'\uf68b': 'UnicodeCharacter0xf68b',
u'\u090a': 'DevanagariLetterUu',
u'\u358d': 'CjkUnifiedIdeograph358d',
u'\u930e': 'CjkUnifiedIdeograph930e',
u'\ubf91': 'HangulSyllableBbwaet',
u'\u5814': 'CjkUnifiedIdeograph5814',
u'\u0497': 'CyrillicSmallLetterZheWithDescender',
u'\ue218': 'UnicodeCharacter0xe218',
u'\u8e9b': 'CjkUnifiedIdeograph8e9b',
u'\u211a': 'DoublestruckCapitalQ',
u'\uab1e': 'UnicodeCharacter0xab1e',
u'\u57a1': 'CjkUnifiedIdeograph57a1',
u'\ud1a5': 'HangulSyllableTonj',
u'\u7024': 'CjkUnifiedIdeograph7024',
u'\u1ca7': 'UnicodeCharacter0x1ca7',
u'\ufa28': 'CjkCompatibilityIdeographfa28',
u'\ua6ab': 'UnicodeCharacter0xa6ab',
u'\u392a': 'CjkUnifiedIdeograph392a',
u'\u432e': 'CjkUnifiedIdeograph432e',
u'\u6fb1': 'CjkUnifiedIdeograph6fb1',
u'\ucd32': 'HangulSyllableCwalp',
u'\ue9b5': 'UnicodeCharacter0xe9b5',
u'\u0834': 'UnicodeCharacter0x834',
u'\u34b7': 'CjkUnifiedIdeograph34b7',
u'\u9238': 'CjkUnifiedIdeograph9238',
u'\ubebb': 'HangulSyllableBbyaed',
u'\u5b3e': 'CjkUnifiedIdeograph5b3e',
u'\u07c1': 'NkoDigitOne',
u'\ue542': 'UnicodeCharacter0xe542',
u'\u81c5': 'CjkUnifiedIdeograph81c5',
u'\u2044': 'FractionSlash',
u'\uaa48': 'ChamLetterFinalY',
u'\u56cb': 'CjkUnifiedIdeograph56cb',
u'\ud0cf': 'HangulSyllableTalh',
u'\u734e': 'CjkUnifiedIdeograph734e',
u'\u1fd1': 'GreekSmallLetterIotaWithMacron',
u'\ufd52': 'ArabicLigatureTehWithHahWithJeemInitialForm',
u'\u99d5': 'CjkUnifiedIdeograph99d5',
u'\u3854': 'CjkUnifiedIdeograph3854',
u'\u4258': 'CjkUnifiedIdeograph4258',
u'\u6edb': 'CjkUnifiedIdeograph6edb',
u'\ucc5c': 'HangulSyllableCaek',
u'\ue8df': 'UnicodeCharacter0xe8df',
u'\u0b5e': 'UnicodeCharacter0xb5e',
u'\u37e1': 'CjkUnifiedIdeograph37e1',
u'\u9562': 'CjkUnifiedIdeograph9562',
u'\ub1e5': 'HangulSyllableNoet',
u'\u5a68': 'CjkUnifiedIdeograph5a68',
u'\u06eb': 'ArabicEmptyCentreHighStop',
u'\ue46c': 'UnicodeCharacter0xe46c',
u'\u80ef': 'CjkUnifiedIdeograph80ef',
u'\u236e': 'AplFunctionalSymbolSemicolonUnderbar',
u'\uad72': 'HangulSyllableGunh',
u'\u49f5': 'CjkUnifiedIdeograph49f5',
u'\ud3f9': 'HangulSyllablePolt',
u'\u7278': 'CjkUnifiedIdeograph7278',
u'\u1efb': 'LatinSmallLetterMiddlewelshLl',
u'\ufc7c': 'ArabicLigatureFehWithAlefMaksuraFinalForm',
u'\u98ff': 'CjkUnifiedIdeograph98ff',
u'\u3b7e': 'CjkUnifiedIdeograph3b7e',
u'\ua203': 'YiSyllableKuox',
u'\u4582': 'CjkUnifiedIdeograph4582',
u'\ucf86': 'HangulSyllableKwaj',
u'\u6b09': 'CjkUnifiedIdeograph6b09',
u'\u0a88': 'GujaratiLetterIi',
u'\uf50d': 'UnicodeCharacter0xf50d',
u'\u948c': 'CjkUnifiedIdeograph948c',
u'\u300f': 'RightWhiteCornerBracket',
u'\uba13': 'HangulSyllableMyas',
u'\u5d92': 'CjkUnifiedIdeograph5d92',
u'\uc417': 'HangulSyllableSswaed',
u'\ue796': 'UnicodeCharacter0xe796',
u'\u0319': 'CombiningRightTackBelow',
u'\u2298': 'CircledDivisionSlash',
u'\u8d1d': 'CjkUnifiedIdeograph8d1d',
u'\uac9c': 'HangulSyllableGem',
u'\u5223': 'CjkUnifiedIdeograph5223',
u'\u75a2': 'CjkUnifiedIdeograph75a2',
u'\udc27': 'UnicodeCharacter0xdc27',
u'\uffa6': 'HalfwidthHangulLetterNieunhieuh',
u'\u1b29': 'BalineseLetterBa',
u'\u3aa8': 'CjkUnifiedIdeograph3aa8',
u'\ua52d': 'VaiSyllableMbi',
u'\u44ac': 'CjkUnifiedIdeograph44ac',
u'\uceb0': 'HangulSyllableKyan',
u'\u6a33': 'CjkUnifiedIdeograph6a33',
u'\u0db2': 'UnicodeCharacter0xdb2',
u'\uf437': 'UnicodeCharacter0xf437',
u'\u97b6': 'CjkUnifiedIdeograph97b6',
u'\u3339': 'SquareHerutu',
u'\ubd3d': 'HangulSyllableBwaeb',
u'\u5cbc': 'CjkUnifiedIdeograph5cbc',
u'\uc741': 'HangulSyllableEunj',
u'\ue6c0': 'UnicodeCharacter0xe6c0',
u'\u0243': 'LatinCapitalLetterBWithStroke',
u'\u25c2': 'BlackLeftpointingSmallTriangle',
u'\u8c47': 'CjkUnifiedIdeograph8c47',
u'\uafc6': 'HangulSyllableGgulp',
u'\u554d': 'CjkUnifiedIdeograph554d',
u'\u74cc': 'CjkUnifiedIdeograph74cc',
u'\udf51': 'UnicodeCharacter0xdf51',
u'\ufed0': 'ArabicLetterGhainMedialForm',
u'\u1a53': 'UnicodeCharacter0x1a53',
u'\u3dd2': 'CjkUnifiedIdeograph3dd2',
u'\ua457': 'YiSyllableNyo',
u'\u47d6': 'CjkUnifiedIdeograph47d6',
u'\uc1da': 'HangulSyllableSwaej',
u'\u6d5d': 'CjkUnifiedIdeograph6d5d',
u'\u0cdc': 'UnicodeCharacter0xcdc',
u'\uf761': 'UnicodeCharacter0xf761',
u'\u96e0': 'CjkUnifiedIdeograph96e0',
u'\u3263': 'CircledHangulRieul',
u'\ubc67': 'HangulSyllableByah',
u'\u5fe6': 'CjkUnifiedIdeograph5fe6',
u'\uc66b': 'HangulSyllableWaelh',
u'\ud9ea': 'UnicodeCharacter0xd9ea',
u'\u056d': 'ArmenianSmallLetterXeh',
u'\u24ec': 'NegativeCircledNumberTwelve',
u'\u8f71': 'CjkUnifiedIdeograph8f71',
u'\uaef0': 'HangulSyllableGgek',
u'\u5477': 'CjkUnifiedIdeograph5477',
u'\u77f6': 'CjkUnifiedIdeograph77f6',
u'\ude7b': 'UnicodeCharacter0xde7b',
u'\uf1fa': 'UnicodeCharacter0xf1fa',
u'\u1d7d': 'LatinSmallLetterPWithStroke',
u'\u3cfc': 'CjkUnifiedIdeograph3cfc',
u'\u2681': 'DieFace2',
u'\u4700': 'CjkUnifiedIdeograph4700',
u'\u6383': 'CjkUnifiedIdeograph6383',
u'\u8402': 'CjkUnifiedIdeograph8402',
u'\ua085': 'YiSyllableNbut',
u'\uc104': 'HangulSyllableSyaen',
u'\ued87': 'UnicodeCharacter0xed87',
u'\u120e': 'EthiopicSyllableLo',
u'\u3e91': 'CjkUnifiedIdeograph3e91',
u'\u5f10': 'CjkUnifiedIdeograph5f10',
u'\u7b93': 'CjkUnifiedIdeograph7b93',
u'\u9c12': 'CjkUnifiedIdeograph9c12',
u'\ub895': 'HangulSyllableRwaeg',
u'\ud914': 'UnicodeCharacter0xd914',
u'\u8597': 'CjkUnifiedIdeograph8597',
u'\uc299': 'HangulSyllableSyub',
u'\u2a1e': 'LargeLeftTriangleOperator',
u'\u7720': 'CjkUnifiedIdeograph7720',
u'\u13a3': 'CherokeeLetterO',
u'\ub422': 'HangulSyllableDoelm',
u'\u50a5': 'CjkUnifiedIdeograph50a5',
u'\uf124': 'UnicodeCharacter0xf124',
u'\u9da7': 'CjkUnifiedIdeograph9da7',
u'\udaa9': 'UnicodeCharacter0xdaa9',
u'\u0f30': 'TibetanDigitHalfSeven',
u'\u2bb3': 'UnicodeCharacter0x2bb3',
u'\u4c32': 'CjkUnifiedIdeograph4c32',
u'\u68b5': 'CjkUnifiedIdeograph68b5',
u'\u8934': 'CjkUnifiedIdeograph8934',
u'\ub5b7': 'HangulSyllableDdeoc',
u'\ud636': 'HangulSyllableHyep',
u'\uf2b9': 'UnicodeCharacter0xf2b9',
u'\u2740': 'WhiteFlorette',
u'\u6442': 'CjkUnifiedIdeograph6442',
u'\xc5': 'LatinCapitalLetterAWithRingAbove',
u'\ua144': 'YiSyllableDdot',
u'\u4dc7': 'HexagramForHoldingTogether',
u'\uee46': 'UnicodeCharacter0xee46',
u'\u8ac9': 'CjkUnifiedIdeograph8ac9',
u'\ud7cb': 'UnicodeCharacter0xd7cb',
u'\u3f50': 'CjkUnifiedIdeograph3f50',
u'\u7c52': 'CjkUnifiedIdeograph7c52',
u'\u18d5': 'UnicodeCharacter0x18d5',
u'\ub954': 'HangulSyllableRwik',
u'\u65d7': 'CjkUnifiedIdeograph65d7',
u'\u8656': 'CjkUnifiedIdeograph8656',
u'\ua2d9': 'YiSyllableZzax',
u'\uc358': 'HangulSyllableSsyaels',
u'\uefdb': 'UnicodeCharacter0xefdb',
u'\u1462': 'CanadianSyllabicsWestcreeTwa',
u'\u30e5': 'KatakanaLetterSmallYu',
u'\u5164': 'CjkUnifiedIdeograph5164',
u'\u7de7': 'CjkUnifiedIdeograph7de7',
u'\u9e66': 'CjkUnifiedIdeograph9e66',
u'\ubae9': 'HangulSyllableMwaelg',
u'\udb68': 'UnicodeCharacter0xdb68',
u'\u87eb': 'CjkUnifiedIdeograph87eb',
u'\uc4ed': 'HangulSyllableSsyut',
u'\u2c72': 'LatinCapitalLetterWWithHook',
u'\u6974': 'CjkUnifiedIdeograph6974',
u'\u15f7': 'CanadianSyllabicsCarrierKhe',
u'\ub676': 'HangulSyllableDdoebs',
u'\u52f9': 'CjkUnifiedIdeograph52f9',
u'\uf378': 'UnicodeCharacter0xf378',
u'\u9ffb': 'UnicodeCharacter0x9ffb',
u'\udcfd': 'UnicodeCharacter0xdcfd',
u'\ue601': 'UnicodeCharacter0xe601',
u'\u0184': 'LatinCapitalLetterToneSix',
u'\u4e86': 'CjkUnifiedIdeograph4e86',
u'\u8b88': 'CjkUnifiedIdeograph8b88',
u'\u370b': 'CjkUnifiedIdeograph370b',
u'\uc88a': 'HangulSyllableJop',
u'\u740d': 'CjkUnifiedIdeograph740d',
u'\ub10f': 'HangulSyllableNeod',
u'\ufe11': 'PresentationFormForVerticalIdeographicComma',
u'\u1994': 'NewTaiLueLetterHighPa',
u'\u6696': 'CjkUnifiedIdeograph6696',
u'\ua398': 'YiSyllableNrox',
u'\ue09a': 'UnicodeCharacter0xe09a',
u'\u0c1d': 'TeluguLetterJha',
u'\u491f': 'CjkUnifiedIdeograph491f',
u'\u9621': 'CjkUnifiedIdeograph9621',
u'\ud323': 'HangulSyllablePac',
u'\u31a4': 'BopomofoLetterEe',
u'\u7ea6': 'CjkUnifiedIdeograph7ea6',
u'\ubba8': 'HangulSyllableMyun',
u'\uf8aa': 'UnicodeCharacter0xf8aa',
u'\u242d': 'UnicodeCharacter0x242d',
u'\uc5ac': 'HangulSyllableYaess',
u'\u612f': 'CjkUnifiedIdeograph612f',
u'\uae31': 'HangulSyllableGig',
u'\ueb33': 'UnicodeCharacter0xeb33',
u'\u16b6': 'RunicLetterEng',
u'\u53b8': 'CjkUnifiedIdeograph53b8',
u'\u90ba': 'CjkUnifiedIdeograph90ba',
u'\u3c3d': 'CjkUnifiedIdeograph3c3d',
u'\uddbc': 'UnicodeCharacter0xddbc',
u'\u793f': 'CjkUnifiedIdeograph793f',
u'\u4641': 'CjkUnifiedIdeograph4641',
u'\u8343': 'CjkUnifiedIdeograph8343',
u'\uc045': 'HangulSyllableBbyulg',
u'\u2ec6': 'CjkRadicalSimplifiedHorn',
u'\u6bc8': 'CjkUnifiedIdeograph6bc8',
u'\ua8ca': 'UnicodeCharacter0xa8ca',
u'\uf5cc': 'UnicodeCharacter0xf5cc',
u'\u114f': 'HangulChoseongChitueumssangcieuc',
u'\u5e51': 'CjkUnifiedIdeograph5e51',
u'\u9b53': 'CjkUnifiedIdeograph9b53',
u'\ud855': 'UnicodeCharacter0xd855',
u'\u03d8': 'GreekLetterArchaicKoppa',
u'\u40da': 'CjkUnifiedIdeograph40da',
u'\u8ddc': 'CjkUnifiedIdeograph8ddc',
u'\u295f': 'RightwardsHarpoonWithBarbDownFromBar',
u'\ucade': 'HangulSyllableJjwanh',
u'\u7661': 'CjkUnifiedIdeograph7661',
u'\ub363': 'HangulSyllableDeolh',
u'\uf065': 'UnicodeCharacter0xf065',
u'\u1be8': 'UnicodeCharacter0x1be8',
u'\u58ea': 'CjkUnifiedIdeograph58ea',
u'\ua5ec': 'VaiSyllableKpe',
u'\ue2ee': 'UnicodeCharacter0xe2ee',
u'\u0e71': 'UnicodeCharacter0xe71',
u'\u4b73': 'CjkUnifiedIdeograph4b73',
u'\u8875': 'CjkUnifiedIdeograph8875',
u'\ud577': 'HangulSyllableHaegs',
u'\u33f8': 'IdeographicTelegraphSymbolForDayTwentyfive',
u'\u70fa': 'CjkUnifiedIdeograph70fa',
u'\ubdfc': 'HangulSyllableByuls',
u'\ufafe': 'UnicodeCharacter0xfafe',
u'\uc800': 'HangulSyllableJeo',
u'\uec83': 'UnicodeCharacter0xec83',
u'\u0b02': 'OriyaSignAnusvara',
u'\u2f85': 'KangxiRadicalMortar',
u'\u8106': 'CjkUnifiedIdeograph8106',
u'\ua589': 'VaiSyllableZoo',
u'\u5a0c': 'CjkUnifiedIdeograph5a0c',
u'\u7e8f': 'CjkUnifiedIdeograph7e8f',
u'\ud010': 'HangulSyllableKweoss',
u'\uf493': 'UnicodeCharacter0xf493',
u'\u1312': 'EthiopicSyllableGwi',
u'\u3795': 'CjkUnifiedIdeograph3795',
u'\ua916': 'KayahLiLetterPha',
u'\u4d99': 'CjkUnifiedIdeograph4d99',
u'\uc39d': 'HangulSyllableSset',
u'\u621c': 'CjkUnifiedIdeograph621c',
u'\u069f': 'ArabicLetterTahWithThreeDotsAbove',
u'\uf820': 'UnicodeCharacter0xf820',
u'\u9ca3': 'CjkUnifiedIdeograph9ca3',
u'\u3b22': 'CjkUnifiedIdeograph3b22',
u'\ub126': 'HangulSyllableNegg',
u'\u55a9': 'CjkUnifiedIdeograph55a9',
u'\uebad': 'UnicodeCharacter0xebad',
u'\u0a2c': 'GurmukhiLetterBa',
u'\u2eaf': 'CjkRadicalSilk',
u'\u8030': 'CjkUnifiedIdeograph8030',
u'\ua4b3': 'YiRadicalJo',
u'\u5936': 'CjkUnifiedIdeograph5936',
u'\u7db9': 'CjkUnifiedIdeograph7db9',
u'\udf3a': 'UnicodeCharacter0xdf3a',
u'\uf3bd': 'UnicodeCharacter0xf3bd',
u'\u123c': 'EthiopicSyllableShee',
u'\u36bf': 'CjkUnifiedIdeograph36bf',
u'\ua840': 'PhagspaLetterKa',
u'\u4cc3': 'CjkUnifiedIdeograph4cc3',
u'\uc2c7': 'HangulSyllableSyid',
u'\u6146': 'CjkUnifiedIdeograph6146',
u'\u05c9': 'UnicodeCharacter0x5c9',
u'\ue74a': 'UnicodeCharacter0xe74a',
u'\u9bcd': 'CjkUnifiedIdeograph9bcd',
u'\u3a4c': 'CjkUnifiedIdeograph3a4c',
u'\ub050': 'HangulSyllableGgeuls',
u'\u54d3': 'CjkUnifiedIdeograph54d3',
u'\uead7': 'UnicodeCharacter0xead7',
u'\u0956': 'UnicodeCharacter0x956',
u'\u2dd9': 'EthiopicSyllableGyu',
u'\u8f5a': 'CjkUnifiedIdeograph8f5a',
u'\ua3dd': 'YiSyllableRyp',
u'\u5860': 'CjkUnifiedIdeograph5860',
u'\u7ce3': 'CjkUnifiedIdeograph7ce3',
u'\ude64': 'UnicodeCharacter0xde64',
u'\uf2e7': 'UnicodeCharacter0xf2e7',
u'\u1166': 'HangulJungseongE',
u'\u35e9': 'CjkUnifiedIdeograph35e9',
u'\u976a': 'CjkUnifiedIdeograph976a',
u'\u4bed': 'CjkUnifiedIdeograph4bed',
u'\uc1f1': 'HangulSyllableSoeb',
u'\u6070': 'CjkUnifiedIdeograph6070',
u'\u04f3': 'CyrillicSmallLetterUWithDoubleAcute',
u'\ue674': 'UnicodeCharacter0xe674',
u'\u9af7': 'CjkUnifiedIdeograph9af7',
u'\u3976': 'CjkUnifiedIdeograph3976',
u'\ubf7a': 'HangulSyllableBbwaegg',
u'\u53fd': 'CjkUnifiedIdeograph53fd',
u'\u6901': 'CjkUnifiedIdeograph6901',
u'\u0880': 'UnicodeCharacter0x880',
u'\uef05': 'UnicodeCharacter0xef05',
u'\u8e84': 'CjkUnifiedIdeograph8e84',
u'\u2207': 'Nabla',
u'\ub80b': 'HangulSyllableRegs',
u'\u478a': 'CjkUnifiedIdeograph478a',
u'\udd8e': 'UnicodeCharacter0xdd8e',
u'\u7111': 'CjkUnifiedIdeograph7111',
u'\u1090': 'MyanmarShanDigitZero',
u'\uf715': 'UnicodeCharacter0xf715',
u'\u9694': 'CjkUnifiedIdeograph9694',
u'\u401b': 'CjkUnifiedIdeograph401b',
u'\u6f9a': 'CjkUnifiedIdeograph6f9a',
u'\uc61f': 'HangulSyllableYec',
u'\ue59e': 'UnicodeCharacter0xe59e',
u'\u1921': 'LimbuVowelSignI',
u'\u38a0': 'CjkUnifiedIdeograph38a0',
u'\u9f25': 'CjkUnifiedIdeograph9f25',
u'\ubea4': 'HangulSyllableBbyals',
u'\u682b': 'CjkUnifiedIdeograph682b',
u'\u77aa': 'CjkUnifiedIdeograph77aa',
u'\uee2f': 'UnicodeCharacter0xee2f',
u'\u8dae': 'CjkUnifiedIdeograph8dae',
u'\u2131': 'ScriptCapitalF',
u'\ua735': 'LatinSmallLetterAo',
u'\u46b4': 'CjkUnifiedIdeograph46b4',
u'\udcb8': 'UnicodeCharacter0xdcb8',
u'\u703b': 'CjkUnifiedIdeograph703b',
u'\u1fba': 'GreekCapitalLetterAlphaWithVaria',
u'\uf63f': 'UnicodeCharacter0xf63f',
u'\u95be': 'CjkUnifiedIdeograph95be',
u'\u4f45': 'CjkUnifiedIdeograph4f45',
u'\u6ec4': 'CjkUnifiedIdeograph6ec4',
u'\uc549': 'HangulSyllableAnj',
u'\ue4c8': 'UnicodeCharacter0xe4c8',
u'\u184b': 'MongolianLetterTodoBa',
u'\u27ca': 'VerticalBarWithHorizontalStroke',
u'\u9e4f': 'CjkUnifiedIdeograph9e4f',
u'\ubdce': 'HangulSyllableBwej',
u'\u5755': 'CjkUnifiedIdeograph5755',
u'\u76d4': 'CjkUnifiedIdeograph76d4',
u'\ued59': 'UnicodeCharacter0xed59',
u'\u8cd8': 'CjkUnifiedIdeograph8cd8',
u'\u205b': 'FourDotMark',
u'\ua65f': 'CyrillicSmallLetterYn',
u'\u45de': 'CjkUnifiedIdeograph45de',
u'\udbe2': 'UnicodeCharacter0xdbe2',
u'\u7f65': 'CjkUnifiedIdeograph7f65',
u'\u1ee4': 'LatinCapitalLetterUWithDotBelow',
u'\uf569': 'UnicodeCharacter0xf569',
u'\u94e8': 'CjkUnifiedIdeograph94e8',
u'\u4e6f': 'CjkUnifiedIdeograph4e6f',
u'\u6dee': 'CjkUnifiedIdeograph6dee',
u'\uc473': 'HangulSyllableSsulh',
u'\ue3f2': 'UnicodeCharacter0xe3f2',
u'\u0775': 'ArabicLetterFarsiYehWithExtendedArabicindicDigitTwoAbove',
u'\u26f4': 'UnicodeCharacter0x26f4',
u'\u9d79': 'CjkUnifiedIdeograph9d79',
u'\ubcf8': 'HangulSyllableBon',
u'\u567f': 'CjkUnifiedIdeograph567f',
u'\u75fe': 'CjkUnifiedIdeograph75fe',
u'\u2489': 'DigitTwoFullStop',
u'\u4508': 'CjkUnifiedIdeograph4508',
u'\u798b': 'CjkUnifiedIdeograph798b',
u'\u860a': 'CjkUnifiedIdeograph860a',
u'\uba8d': 'HangulSyllableMyeg',
u'\udb0c': 'UnicodeCharacter0xdb0c',
u'\uff8f': 'HalfwidthKatakanaLetterMa',
u'\u2816': 'BraillePatternDots235',
u'\u6d18': 'CjkUnifiedIdeograph6d18',
u'\u019b': 'LatinSmallLetterLambdaWithStroke',
u'\uae1a': 'HangulSyllableGyinh',
u'\u429d': 'CjkUnifiedIdeograph429d',
u'\ue31c': 'UnicodeCharacter0xe31c',
u'\u879f': 'CjkUnifiedIdeograph879f',
u'\ud8a1': 'UnicodeCharacter0xd8a1',
u'\u3026': 'HangzhouNumeralSix',
u'\u7528': 'CjkUnifiedIdeograph7528',
u'\u29ab': 'MeasuredAngleWithOpenArmEndingInArrowPointingDownAndLeft',
u'\ub62a': 'HangulSyllableDdop',
u'\u6aad': 'CjkUnifiedIdeograph6aad',
u'\u8b2c': 'CjkUnifiedIdeograph8b2c',
u'\uafaf': 'HangulSyllableGgyos',
u'\ucc2e': 'HangulSyllableCanh',
u'\ue0b1': 'UnicodeCharacter0xe0b1',
u'\u1d38': 'ModifierLetterCapitalL',
u'\u31bb': 'UnicodeCharacter0x31bb',
u'\u5e3a': 'CjkUnifiedIdeograph5e3a',
u'\u72bd': 'CjkUnifiedIdeograph72bd',
u'\u933c': 'CjkUnifiedIdeograph933c',
u'\ub7bf': 'HangulSyllableRyalb',
u'\ud43e': 'HangulSyllablePwaep',
u'\u88c1': 'CjkUnifiedIdeograph88c1',
u'\ucdc3': 'HangulSyllableCweos',
u'\u2548': 'BoxDrawingsUpLightAndDownHorizontalHeavy',
u'\u664a': 'CjkUnifiedIdeograph664a',
u'\u1acd': 'UnicodeCharacter0x1acd',
u'\ubb4c': 'HangulSyllableMuk',
u'\u5fcf': 'CjkUnifiedIdeograph5fcf',
u'\ufc4e': 'ArabicLigatureNoonWithMeemIsolatedForm',
u'\u90d1': 'CjkUnifiedIdeograph90d1',
u'\ud5d3': 'HangulSyllableHeolb',
u'\u0e5a': 'ThaiCharacterAngkhankhu',
u'\u22dd': 'EqualToOrGreaterthan',
u'\u435c': 'CjkUnifiedIdeograph435c',
u'\u67df': 'CjkUnifiedIdeograph67df',
u'\u845e': 'CjkUnifiedIdeograph845e',
u'\ub8e1': 'HangulSyllableRyong',
u'\ud960': 'UnicodeCharacter0xd960',
u'\ufde3': 'UnicodeCharacter0xfde3',
u'\u166a': 'CanadianSyllabicsCarrierTtsee',
u'\u6b6c': 'CjkUnifiedIdeograph6b6c',
u'\u0fef': 'UnicodeCharacter0xfef',
u'\uac6e': 'HangulSyllableGyaep',
u'\u40f1': 'CjkUnifiedIdeograph40f1',
u'\ue170': 'UnicodeCharacter0xe170',
u'\u85f3': 'CjkUnifiedIdeograph85f3',
u'\uc6f5': 'HangulSyllableWelt',
u'\u3e7a': 'CjkUnifiedIdeograph3e7a',
u'\u737c': 'CjkUnifiedIdeograph737c',
u'\u17ff': 'UnicodeCharacter0x17ff',
u'\ub47e': 'HangulSyllableDweobs',
u'\u8980': 'CjkUnifiedIdeograph8980',
u'\u2d03': 'GeorgianSmallLetterDon',
u'\uca82': 'HangulSyllableJjep',
u'\u6e05': 'CjkUnifiedIdeograph6e05',
u'\ua307': 'YiSyllableNzyrx',
u'\ue409': 'UnicodeCharacter0xe409',
u'\u1b8c': 'SundaneseLetterGa',
u'\u5c8e': 'CjkUnifiedIdeograph5c8e',
u'\u9190': 'CjkUnifiedIdeograph9190',
u'\u3513': 'CjkUnifiedIdeograph3513',
u'\ud292': 'HangulSyllableTwibs',
u'\u7615': 'CjkUnifiedIdeograph7615',
u'\u4b17': 'CjkUnifiedIdeograph4b17',
u'\u8c19': 'CjkUnifiedIdeograph8c19',
u'\uc11b': 'HangulSyllableSyaeh',
u'\u239c': 'LeftParenthesisExtension',
u'\u649e': 'CjkUnifiedIdeograph649e',
u'\ub9a0': 'HangulSyllableRyim',
u'\ufaa2': 'CjkCompatibilityIdeographfaa2',
u'\u1e25': 'LatinSmallLetterHWithDotBelow',
u'\u5327': 'CjkUnifiedIdeograph5327',
u'\u9429': 'CjkUnifiedIdeograph9429',
u'\ue92b': 'UnicodeCharacter0xe92b',
u'\u0cae': 'KannadaLetterMa',
u'\u41b0': 'CjkUnifiedIdeograph41b0',
u'\u82b2': 'CjkUnifiedIdeograph82b2',
u'\u2635': 'TrigramForWater',
u'\uc7b4': 'HangulSyllableJael',
u'\u7b37': 'CjkUnifiedIdeograph7b37',
u'\ubc39': 'HangulSyllableBaelg',
u'\uf13b': 'UnicodeCharacter0xf13b',
u'\u14be': 'CanadianSyllabicsAthapascanM',
u'\u69c0': 'CjkUnifiedIdeograph69c0',
u'\uaac2': 'UnicodeCharacter0xaac2',
u'\uefc4': 'UnicodeCharacter0xefc4',
u'\u0347': 'CombiningEqualsSignBelow',
u'\u4449': 'CjkUnifiedIdeograph4449',
u'\u994b': 'CjkUnifiedIdeograph994b',
u'\uda4d': 'UnicodeCharacter0xda4d',
u'\u3cce': 'CjkUnifiedIdeograph3cce',
u'\u71d0': 'CjkUnifiedIdeograph71d0',
u'\ub2d2': 'HangulSyllableNilm',
u'\uf7d4': 'UnicodeCharacter0xf7d4',
u'\u2b57': 'UnicodeCharacter0x2b57',
u'\uc8d6': 'HangulSyllableJoebs',
u'\u6c59': 'CjkUnifiedIdeograph6c59',
u'\ua15b': 'YiSyllableNdot',
u'\ue25d': 'UnicodeCharacter0xe25d',
u'\u19e0': 'KhmerSymbolPathamasat',
u'\u5ae2': 'CjkUnifiedIdeograph5ae2',
u'\u9fe4': 'UnicodeCharacter0x9fe4',
u'\u3367': 'IdeographicTelegraphSymbolForHourFifteen',
u'\ud0e6': 'HangulSyllableTaelm',
u'\u7469': 'CjkUnifiedIdeograph7469',
u'\u496b': 'CjkUnifiedIdeograph496b',
u'\u8a6d': 'CjkUnifiedIdeograph8a6d',
u'\ucf6f': 'HangulSyllableKoh',
u'\u21f0': 'RightwardsWhiteArrowFromWall',
u'\u62f2': 'CjkUnifiedIdeograph62f2',
u'\ua7f4': 'UnicodeCharacter0xa7f4',
u'\uf8f6': 'UnicodeCharacter0xf8f6',
u'\u1c79': 'OlChikiGaahlaaTtuddaag',
u'\u517b': 'CjkUnifiedIdeograph517b',
u'\u927d': 'CjkUnifiedIdeograph927d',
u'\ud77f': 'HangulSyllableHyis',
u'\uab81': 'UnicodeCharacter0xab81',
u'\u4c04': 'CjkUnifiedIdeograph4c04',
u'\u6087': 'CjkUnifiedIdeograph6087',
u'\uc608': 'HangulSyllableYe',
u'\ufa8b': 'CjkCompatibilityIdeographfa8b',
u'\u050a': 'CyrillicCapitalLetterKomiNje',
u'\u398d': 'CjkUnifiedIdeograph398d',
u'\u9f0e': 'CjkUnifiedIdeograph9f0e',
u'\ub391': 'HangulSyllableDyeonj',
u'\u5414': 'CjkUnifiedIdeograph5414',
u'\u0897': 'UnicodeCharacter0x897',
u'\uee18': 'UnicodeCharacter0xee18',
u'\u829b': 'CjkUnifiedIdeograph829b',
u'\u2d1a': 'GeorgianSmallLetterCan',
u'\ua71e': 'ModifierLetterRaisedInvertedExclamationMark',
u'\u5ba1': 'CjkUnifiedIdeograph5ba1',
u'\udda5': 'UnicodeCharacter0xdda5',
u'\u7c24': 'CjkUnifiedIdeograph7c24',
u'\u10a7': 'GeorgianCapitalLetterTan',
u'\uf628': 'UnicodeCharacter0xf628',
u'\uaaab': 'UnicodeCharacter0xaaab',
u'\u352a': 'CjkUnifiedIdeograph352a',
u'\u4f2e': 'CjkUnifiedIdeograph4f2e',
u'\u63b1': 'CjkUnifiedIdeograph63b1',
u'\uc132': 'HangulSyllableSeoj',
u'\ue5b5': 'UnicodeCharacter0xe5b5',
u'\u0434': 'CyrillicSmallLetterDe',
u'\u38b7': 'CjkUnifiedIdeograph38b7',
u'\u9e38': 'CjkUnifiedIdeograph9e38',
u'\ub2bb': 'HangulSyllableNyilh',
u'\u573e': 'CjkUnifiedIdeograph573e',
u'\u0bc1': 'TamilVowelSignU',
u'\ue942': 'UnicodeCharacter0xe942',
u'\u8dc5': 'CjkUnifiedIdeograph8dc5',
u'\u2c44': 'GlagoliticSmallLetterSlovo',
u'\ua648': 'CyrillicCapitalLetterDjerv',
u'\u5acb': 'CjkUnifiedIdeograph5acb',
u'\udccf': 'UnicodeCharacter0xdccf',
u'\u7f4e': 'CjkUnifiedIdeograph7f4e',
u'\u13d1': 'CherokeeLetterSu',
u'\uf152': 'UnicodeCharacter0xf152',
u'\u95d5': 'CjkUnifiedIdeograph95d5',
u'\u3454': 'CjkUnifiedIdeograph3454',
u'\u4e58': 'CjkUnifiedIdeograph4e58',
u'\u62db': 'CjkUnifiedIdeograph62db',
u'\uc05c': 'HangulSyllableBbeun',
u'\ue4df': 'UnicodeCharacter0xe4df',
u'\u075e': 'ArabicLetterAinWithThreeDotsPointingDownwardsAbove',
u'\u3be1': 'CjkUnifiedIdeograph3be1',
u'\u9962': 'CjkUnifiedIdeograph9962',
u'\ubde5': 'HangulSyllableBwib',
u'\u5668': 'CjkUnifiedIdeograph5668',
u'\u0aeb': 'GujaratiDigitFive',
u'\ue86c': 'UnicodeCharacter0xe86c',
u'\u8cef': 'CjkUnifiedIdeograph8cef',
u'\u2f6e': 'KangxiRadicalArrow',
u'\ua172': 'YiSyllableHna',
u'\u45f5': 'CjkUnifiedIdeograph45f5',
u'\udff9': 'UnicodeCharacter0xdff9',
u'\u7e78': 'CjkUnifiedIdeograph7e78',
u'\u12fb': 'EthiopicSyllableDdaa',
u'\uf07c': 'UnicodeCharacter0xf07c',
u'\u94ff': 'CjkUnifiedIdeograph94ff',
u'\u377e': 'CjkUnifiedIdeograph377e',
u'\uae03': 'HangulSyllableGeulb',
u'\u4982': 'CjkUnifiedIdeograph4982',
u'\uc386': 'HangulSyllableSsegg',
u'\u6709': 'CjkUnifiedIdeograph6709',
u'\u0688': 'ArabicLetterDdal',
u'\uf90d': 'CjkCompatibilityIdeographf90d',
u'\u988c': 'CjkUnifiedIdeograph988c',
u'\u3c0f': 'CjkUnifiedIdeograph3c0f',
u'\ub613': 'HangulSyllableDdogs',
u'\u5192': 'CjkUnifiedIdeograph5192',
u'\uc817': 'HangulSyllableJeoc',
u'\ueb96': 'UnicodeCharacter0xeb96',
u'\u0f19': 'TibetanAstrologicalSignSdongTshugs',
u'\u2e98': 'CjkRadicalHand',
u'\u811d': 'CjkUnifiedIdeograph811d',
u'\ua09c': 'YiSyllableHmuox',
u'\u5e23': 'CjkUnifiedIdeograph5e23',
u'\u79a2': 'CjkUnifiedIdeograph79a2',
u'\ud027': 'HangulSyllableKwelh',
u'\uf3a6': 'UnicodeCharacter0xf3a6',
u'\u1729': 'HanunooLetterPa',
u'\u36a8': 'CjkUnifiedIdeograph36a8',
u'\ua92d': 'KayahLiToneCalyaPlophu',
u'\u48ac': 'CjkUnifiedIdeograph48ac',
u'\uc2b0': 'HangulSyllableSeuls',
u'\u6633': 'CjkUnifiedIdeograph6633',
u'\u01b2': 'LatinCapitalLetterVWithHook',
u'\uf837': 'UnicodeCharacter0xf837',
u'\u9bb6': 'CjkUnifiedIdeograph9bb6',
u'\u3f39': 'CjkUnifiedIdeograph3f39',
u'\ub13d': 'HangulSyllableNet',
u'\u50bc': 'CjkUnifiedIdeograph50bc',
u'\ucb41': 'HangulSyllableJjyong',
u'\ueac0': 'UnicodeCharacter0xeac0',
u'\u0e43': 'ThaiCharacterSaraAiMaimuan',
u'\u29c2': 'CircleWithSmallCircleToTheRight',
u'\u8047': 'CjkUnifiedIdeograph8047',
u'\ua3c6': 'YiSyllableRat',
u'\u594d': 'CjkUnifiedIdeograph594d',
u'\u78cc': 'CjkUnifiedIdeograph78cc',
u'\ud351': 'HangulSyllablePyalt',
u'\uf2d0': 'UnicodeCharacter0xf2d0',
u'\u1653': 'CanadianSyllabicsCarrierSa',
u'\u31d2': 'CjkStrokeP',
u'\ua857': 'PhagspaLetterYa',
u'\u4bd6': 'CjkUnifiedIdeograph4bd6',
u'\ucdda': 'HangulSyllableCwelp',
u'\u615d': 'CjkUnifiedIdeograph615d',
u'\xdc': 'LatinCapitalLetterUWithDiaeresis',
u'\ufb61': 'ArabicLetterTtehehMedialForm',
u'\u9ae0': 'CjkUnifiedIdeograph9ae0',
u'\u3e63': 'CjkUnifiedIdeograph3e63',
u'\ub067': 'HangulSyllableGgyid',
u'\u53e6': 'CjkUnifiedIdeograph53e6',
u'\uca6b': 'HangulSyllableJjegs',
u'\ud5ea': 'HangulSyllableHenh',
u'\u096d': 'DevanagariDigitSeven',
u'\u28ec': 'BraillePatternDots34678',
u'\u8371': 'CjkUnifiedIdeograph8371',
u'\ua2f0': 'YiSyllableNzip',
u'\u5877': 'CjkUnifiedIdeograph5877',
u'\u7bf6': 'CjkUnifiedIdeograph7bf6',
u'\ud27b': 'HangulSyllableTwec',
u'\ufdfa': 'ArabicLigatureSallallahouAlayheWasallam',
u'\u117d': 'HangulJungseongYeoo',
u'\u30fc': 'KatakanahiraganaProlongedSoundMark',
u'\u2a81': 'LessthanOrSlantedEqualToWithDotAbove',
u'\u4b00': 'CjkUnifiedIdeograph4b00',
u'\u6f83': 'CjkUnifiedIdeograph6f83',
u'\u8802': 'CjkUnifiedIdeograph8802',
u'\uac85': 'HangulSyllableGeong',
u'\ucd04': 'HangulSyllableCyek',
u'\ue187': 'UnicodeCharacter0xe187',
u'\u1e0e': 'LatinCapitalLetterDWithLineBelow',
u'\u3291': 'CircledIdeographStock',
u'\u5310': 'CjkUnifiedIdeograph5310',
u'\u7793': 'CjkUnifiedIdeograph7793',
u'\u9012': 'CjkUnifiedIdeograph9012',
u'\ub495': 'HangulSyllableDwelt',
u'\ud514': 'HangulSyllablePeum',
u'\u8997': 'CjkUnifiedIdeograph8997',
u'\uce99': 'HangulSyllableKaelg',
u'\u261e': 'WhiteRightPointingIndex',
u'\u7b20': 'CjkUnifiedIdeograph7b20',
u'\u1fa3': 'GreekSmallLetterOmegaWithDasiaAndVariaAndYpogegrammeni',
u'\ub822': 'HangulSyllableRep',
u'\u5ca5': 'CjkUnifiedIdeograph5ca5',
u'\ufd24': 'ArabicLigatureDadWithYehFinalForm',
u'\u91a7': 'CjkUnifiedIdeograph91a7',
u'\ud6a9': 'HangulSyllableHyog',
u'\u0330': 'CombiningTildeBelow',
u'\u27b3': 'WhitefeatheredRightwardsArrow',
u'\u4032': 'CjkUnifiedIdeograph4032',
u'\u64b5': 'CjkUnifiedIdeograph64b5',
u'\u8534': 'CjkUnifiedIdeograph8534',
u'\ub9b7': 'HangulSyllableRilb',
u'\uda36': 'UnicodeCharacter0xda36',
u'\ufeb9': 'ArabicLetterSadIsolatedForm',
u'\u2b40': 'EqualsSignAboveLeftwardsArrow',
u'\u6842': 'CjkUnifiedIdeograph6842',
u'\u0cc5': 'UnicodeCharacter0xcc5',
u'\uad44': 'HangulSyllableGoem',
u'\u41c7': 'CjkUnifiedIdeograph41c7',
u'\ue246': 'UnicodeCharacter0xe246',
u'\u86c9': 'CjkUnifiedIdeograph86c9',
u'\udbcb': 'UnicodeCharacter0xdbcb',
u'\u3350': 'SquareYuan',
u'\u7052': 'CjkUnifiedIdeograph7052',
u'\u14d5': 'CanadianSyllabicsLi',
u'\ub554': 'HangulSyllableDdael',
u'\u69d7': 'CjkUnifiedIdeograph69d7',
u'\u8a56': 'CjkUnifiedIdeograph8a56',
u'\uaed9': 'HangulSyllableGgeg',
u'\ucf58': 'HangulSyllableKon',
u'\ue3db': 'UnicodeCharacter0xe3db',
u'\u1862': 'MongolianLetterSibeAng',
u'\u3ce5': 'CjkUnifiedIdeograph3ce5',
u'\u5d64': 'CjkUnifiedIdeograph5d64',
u'\u71e7': 'CjkUnifiedIdeograph71e7',
u'\u9266': 'CjkUnifiedIdeograph9266',
u'\ub6e9': 'HangulSyllableDdweng',
u'\ud768': 'HangulSyllableHeuk',
u'\u8beb': 'CjkUnifiedIdeograph8beb',
u'\uc8ed': 'HangulSyllableJyolt',
u'\u2072': 'UnicodeCharacter0x2072',
u'\u6574': 'CjkUnifiedIdeograph6574',
u'\u19f7': 'KhmerSymbolPrampiiRoc',
u'\uba76': 'HangulSyllableMyeonh',
u'\u5ef9': 'CjkUnifiedIdeograph5ef9',
u'\uff78': 'HalfwidthKatakanaLetterKu',
u'\u93fb': 'CjkUnifiedIdeograph93fb',
u'\ud0fd': 'HangulSyllableTyanj',
u'\uea01': 'UnicodeCharacter0xea01',
u'\u0d84': 'UnicodeCharacter0xd84',
u'\u4286': 'CjkUnifiedIdeograph4286',
u'\u8788': 'CjkUnifiedIdeograph8788',
u'\u3b0b': 'CjkUnifiedIdeograph3b0b',
u'\uc48a': 'HangulSyllableSsweolm',
u'\u780d': 'CjkUnifiedIdeograph780d',
u'\ubd0f': 'HangulSyllableBoh',
u'\uf211': 'UnicodeCharacter0xf211',
u'\u1594': 'CanadianSyllabicsNgaa',
u'\u6a96': 'CjkUnifiedIdeograph6a96',
u'\uaf98': 'HangulSyllableGgoek',
u'\uec9a': 'UnicodeCharacter0xec9a',
u'\u451f': 'CjkUnifiedIdeograph451f',
u'\u9a21': 'CjkUnifiedIdeograph9a21',
u'\udf23': 'UnicodeCharacter0xdf23',
u'\u3da4': 'CjkUnifiedIdeograph3da4',
u'\u72a6': 'CjkUnifiedIdeograph72a6',
u'\ub7a8': 'HangulSyllableRaem',
u'\uf4aa': 'UnicodeCharacter0xf4aa',
u'\u282d': 'BraillePatternDots1346',
u'\uc9ac': 'HangulSyllableJyil',
u'\u6d2f': 'CjkUnifiedIdeograph6d2f',
u'\ua231': 'YiSyllableMgie',
u'\ue733': 'UnicodeCharacter0xe733',
u'\u1ab6': 'UnicodeCharacter0x1ab6',
u'\u5fb8': 'CjkUnifiedIdeograph5fb8',
u'\u9cba': 'CjkUnifiedIdeograph9cba',
u'\u303d': 'PartAlternationMark',
u'\ud1bc': 'HangulSyllableTwa',
u'\u753f': 'CjkUnifiedIdeograph753f',
u'\u4a41': 'CjkUnifiedIdeograph4a41',
u'\u8f43': 'CjkUnifiedIdeograph8f43',
u'\ucc45': 'HangulSyllableCaeg',
u'\u22c6': 'StarOperator',
u'\u67c8': 'CjkUnifiedIdeograph67c8',
u'\ua4ca': 'UnicodeCharacter0xa4ca',
u'\uf9cc': 'CjkCompatibilityIdeographf9cc',
u'\u1d4f': 'ModifierLetterSmallK',
u'\u5251': 'CjkUnifiedIdeograph5251',
u'\u9753': 'CjkUnifiedIdeograph9753',
u'\ud455': 'HangulSyllablePoeng',
u'\u0fd8': 'UnicodeCharacter0xfd8',
u'\u4cda': 'CjkUnifiedIdeograph4cda',
u'\u81dc': 'CjkUnifiedIdeograph81dc',
u'\u255f': 'BoxDrawingsVerticalDoubleAndRightSingle',
u'\uc6de': 'HangulSyllableWeobs',
u'\u7a61': 'CjkUnifiedIdeograph7a61',
u'\ubf63': 'HangulSyllableBbwad',
u'\ufc65': 'ArabicLigatureYehWithHamzaAboveWithZainFinalForm',
u'\u17e8': 'KhmerDigitEight',
u'\u54ea': 'CjkUnifiedIdeograph54ea',
u'\ua9ec': 'UnicodeCharacter0xa9ec',
u'\ueeee': 'UnicodeCharacter0xeeee',
u'\u0271': 'LatinSmallLetterMWithHook',
u'\u4773': 'CjkUnifiedIdeograph4773',
u'\u8475': 'CjkUnifiedIdeograph8475',
u'\ud977': 'UnicodeCharacter0xd977',
u'\u3ff8': 'CjkUnifiedIdeograph3ff8',
u'\u7cfa': 'CjkUnifiedIdeograph7cfa',
u'\ub1fc': 'HangulSyllableNyoss',
u'\uf6fe': 'UnicodeCharacter0xf6fe',
u'\ucc00': 'HangulSyllableJjyim',
u'\ue883': 'UnicodeCharacter0xe883',
u'\u0f02': 'TibetanMarkGterYigMgoUmRnamBcadMa',
u'\u2b85': 'UnicodeCharacter0x2b85',
u'\u8506': 'CjkUnifiedIdeograph8506',
u'\ua189': 'YiSyllableNuop',
u'\u5e0c': 'CjkUnifiedIdeograph5e0c',
u'\u7a8f': 'CjkUnifiedIdeograph7a8f',
u'\ud410': 'HangulSyllablePwal',
u'\uf093': 'UnicodeCharacter0xf093',
u'\u1712': 'TagalogVowelSignI',
u'\u3395': 'SquareMuL',
u'\uad16': 'HangulSyllableGwap',
u'\u4999': 'CjkUnifiedIdeograph4999',
u'\uc79d': 'HangulSyllableJalt',
u'\u661c': 'CjkUnifiedIdeograph661c',
u'\u029f': 'LatinLetterSmallCapitalL',
u'\ufc20': 'ArabicLigatureSadWithHahIsolatedForm',
u'\u98a3': 'CjkUnifiedIdeograph98a3',
u'\u3f22': 'CjkUnifiedIdeograph3f22',
u'\ub526': 'HangulSyllableDibs',
u'\u51a9': 'CjkUnifiedIdeograph51a9',
u'\uefad': 'UnicodeCharacter0xefad',
u'\u0e2c': 'ThaiCharacterLoChula',
u'\u2aaf': 'PrecedesAboveSinglelineEqualsSign',
u'\u8430': 'CjkUnifiedIdeograph8430',
u'\ua0b3': 'YiSyllableMie',
u'\u5d36': 'CjkUnifiedIdeograph5d36',
u'\u79b9': 'CjkUnifiedIdeograph79b9',
u'\udb3a': 'UnicodeCharacter0xdb3a',
u'\uf7bd': 'UnicodeCharacter0xf7bd',
u'\u163c': 'CanadianSyllabicsCarrierTle',
u'\u32bf': 'CircledNumberFifty',
u'\uac40': 'HangulSyllableGyal',
u'\u48c3': 'CjkUnifiedIdeograph48c3',
u'\uc6c7': 'HangulSyllableUc',
u'\u6546': 'CjkUnifiedIdeograph6546',
u'\u01c9': 'LatinSmallLetterLj',
u'\ue34a': 'UnicodeCharacter0xe34a',
u'\u9fcd': 'UnicodeCharacter0x9fcd',
u'\u3e4c': 'CjkUnifiedIdeograph3e4c',
u'\ub450': 'HangulSyllableDu',
u'\u50d3': 'CjkUnifiedIdeograph50d3',
u'\ueed7': 'UnicodeCharacter0xeed7',
u'\u0d56': 'UnicodeCharacter0xd56',
u'\u29d9': 'RightWigglyFence',
u'\u8b5a': 'CjkUnifiedIdeograph8b5a',
u'\ua7dd': 'UnicodeCharacter0xa7dd',
u'\u5c60': 'CjkUnifiedIdeograph5c60',
u'\u78e3': 'CjkUnifiedIdeograph78e3',
u'\uda64': 'UnicodeCharacter0xda64',
u'\uf6e7': 'UnicodeCharacter0xf6e7',
u'\u1566': 'CanadianSyllabicsTha',
u'\u31e9': 'UnicodeCharacter0x31e9',
u'\u936a': 'CjkUnifiedIdeograph936a',
u'\u4fed': 'CjkUnifiedIdeograph4fed',
u'\uc5f1': 'HangulSyllableYeonj',
u'\u6470': 'CjkUnifiedIdeograph6470',
u'\xf3': 'LatinSmallLetterOWithAcute',
u'\ue274': 'UnicodeCharacter0xe274',
u'\u9ef7': 'CjkUnifiedIdeograph9ef7',
u'\u3d76': 'CjkUnifiedIdeograph3d76',
u'\ubb7a': 'HangulSyllableMwelp',
u'\u57fd': 'CjkUnifiedIdeograph57fd',
u'\u6d01': 'CjkUnifiedIdeograph6d01',
u'\u0c80': 'UnicodeCharacter0xc80',
u'\ueb05': 'UnicodeCharacter0xeb05',
u'\u8a84': 'CjkUnifiedIdeograph8a84',
u'\u2607': 'Lightning',
u'\ubc0b': 'HangulSyllableMis',
u'\u438a': 'CjkUnifiedIdeograph438a',
u'\ud98e': 'UnicodeCharacter0xd98e',
u'\u7511': 'CjkUnifiedIdeograph7511',
u'\u1490': 'CanadianSyllabicsCa',
u'\uf315': 'UnicodeCharacter0xf315',
u'\u9294': 'CjkUnifiedIdeograph9294',
u'\u441b': 'CjkUnifiedIdeograph441b',
u'\u6b9a': 'CjkUnifiedIdeograph6b9a',
u'\uc21f': 'HangulSyllableSud',
u'\ue19e': 'UnicodeCharacter0xe19e',
u'\u1d21': 'LatinLetterSmallCapitalW',
u'\u3ca0': 'CjkUnifiedIdeograph3ca0',
u'\u9b25': 'CjkUnifiedIdeograph9b25',
u'\ubaa4': 'HangulSyllableMyek',
u'\u6c2b': 'CjkUnifiedIdeograph6c2b',
u'\u73aa': 'CjkUnifiedIdeograph73aa',
u'\uea2f': 'UnicodeCharacter0xea2f',
u'\u89ae': 'CjkUnifiedIdeograph89ae',
u'\u2531': 'BoxDrawingsRightLightAndLeftDownHeavy',
u'\ua335': 'YiSyllableSsot',
u'\u42b4': 'CjkUnifiedIdeograph42b4',
u'\ud8b8': 'UnicodeCharacter0xd8b8',
u'\u743b': 'CjkUnifiedIdeograph743b',
u'\u1bba': 'UnicodeCharacter0x1bba',
u'\uf23f': 'UnicodeCharacter0xf23f',
u'\u91be': 'CjkUnifiedIdeograph91be',
u'\u4b45': 'CjkUnifiedIdeograph4b45',
u'\u6ac4': 'CjkUnifiedIdeograph6ac4',
u'\uc149': 'HangulSyllableSeb',
u'\ue0c8': 'UnicodeCharacter0xe0c8',
u'\u1c4b': 'UnicodeCharacter0x1c4b',
u'\u23ca': 'DentistrySymbolLightUpAndHorizontal',
u'\u9a4f': 'CjkUnifiedIdeograph9a4f',
u'\ub9ce': 'HangulSyllableManh',
u'\u5355': 'CjkUnifiedIdeograph5355',
u'\u72d4': 'CjkUnifiedIdeograph72d4',
u'\ue959': 'UnicodeCharacter0xe959',
u'\u88d8': 'CjkUnifiedIdeograph88d8',
u'\u245b': 'UnicodeCharacter0x245b',
u'\ua25f': 'YiSyllableNgiep',
u'\u41de': 'CjkUnifiedIdeograph41de',
u'\udfe2': 'UnicodeCharacter0xdfe2',
u'\u7b65': 'CjkUnifiedIdeograph7b65',
u'\u1ae4': 'UnicodeCharacter0x1ae4',
u'\uf169': 'UnicodeCharacter0xf169',
u'\u90e8': 'CjkUnifiedIdeograph90e8',
u'\u4a6f': 'CjkUnifiedIdeograph4a6f',
u'\u69ee': 'CjkUnifiedIdeograph69ee',
u'\uc073': 'HangulSyllableBbeuh',
u'\ue7f2': 'UnicodeCharacter0xe7f2',
u'\u0375': 'GreekLowerNumeralSign',
u'\u22f4': 'SmallElementOfWithVerticalBarAtEndOfHorizontalStroke',
u'\u9979': 'CjkUnifiedIdeograph9979',
u'\ub8f8': 'HangulSyllableRum',
u'\u527f': 'CjkUnifiedIdeograph527f',
u'\u71fe': 'CjkUnifiedIdeograph71fe',
u'\u0406': 'CyrillicCapitalLetterByelorussianukrainianI',
u'\u2089': 'SubscriptNine',
u'\u4108': 'CjkUnifiedIdeograph4108',
u'\u7d8b': 'CjkUnifiedIdeograph7d8b',
u'\u820a': 'CjkUnifiedIdeograph820a',
u'\ube8d': 'HangulSyllableBbaeb',
u'\udf0c': 'UnicodeCharacter0xdf0c',
u'\ufb8f': 'ArabicLetterKehehFinalForm',
u'\u2c16': 'GlagoliticCapitalLetterUku',
u'\u6918': 'CjkUnifiedIdeograph6918',
u'\u059b': 'HebrewAccentTevir',
u'\uaa1a': 'ChamLetterPa',
u'\u469d': 'CjkUnifiedIdeograph469d',
u'\ue71c': 'UnicodeCharacter0xe71c',
u'\u839f': 'CjkUnifiedIdeograph839f',
u'\udca1': 'UnicodeCharacter0xdca1',
u'\u3426': 'CjkUnifiedIdeograph3426',
u'\u7128': 'CjkUnifiedIdeograph7128',
u'\u2dab': 'EthiopicSyllableCcaa',
u'\ub22a': 'HangulSyllableNweolm',
u'\u6ead': 'CjkUnifiedIdeograph6ead',
u'\u8f2c': 'CjkUnifiedIdeograph8f2c',
u'\uabaf': 'UnicodeCharacter0xabaf',
u'\uc82e': 'HangulSyllableJebs',
u'\ue4b1': 'UnicodeCharacter0xe4b1',
u'\u1938': 'LimbuSmallLetterLa',
u'\u35bb': 'CjkUnifiedIdeograph35bb',
u'\u5a3a': 'CjkUnifiedIdeograph5a3a',
u'\u76bd': 'CjkUnifiedIdeograph76bd',
u'\u973c': 'CjkUnifiedIdeograph973c',
u'\ub3bf': 'HangulSyllableDyec',
u'\ud03e': 'HangulSyllableKwilm',
u'\u8cc1': 'CjkUnifiedIdeograph8cc1',
u'\uc9c3': 'HangulSyllableJigs',
u'\u2148': 'DoublestruckItalicSmallI',
u'\u624a': 'CjkUnifiedIdeograph624a',
u'\u1ecd': 'LatinSmallLetterOWithDotBelow',
u'\ubf4c': 'HangulSyllableBbols',
u'\u5bcf': 'CjkUnifiedIdeograph5bcf',
u'\uf84e': 'UnicodeCharacter0xf84e',
u'\u94d1': 'CjkUnifiedIdeograph94d1',
u'\ud1d3': 'HangulSyllableTwac',
u'\u0a5a': 'GurmukhiLetterGhha',
u'\u26dd': 'UnicodeCharacter0x26dd',
u'\u475c': 'CjkUnifiedIdeograph475c',
u'\u63df': 'CjkUnifiedIdeograph63df',
u'\u805e': 'CjkUnifiedIdeograph805e',
u'\ubce1': 'HangulSyllableByelg',
u'\udd60': 'UnicodeCharacter0xdd60',
u'\uf9e3': 'CjkCompatibilityIdeographf9e3',
u'\u126a': 'EthiopicSyllableVi',
u'\u6f6c': 'CjkUnifiedIdeograph6f6c',
u'\u0bef': 'TamilDigitNine',
u'\ua86e': 'PhagspaLetterVoicelessSha',
u'\u44f1': 'CjkUnifiedIdeograph44f1',
u'\ue570': 'UnicodeCharacter0xe570',
u'\u81f3': 'CjkUnifiedIdeograph81f3',
u'\uc2f5': 'HangulSyllableSit',
u'\u3a7a': 'CjkUnifiedIdeograph3a7a',
u'\u777c': 'CjkUnifiedIdeograph777c',
u'\u13ff': 'UnicodeCharacter0x13ff',
u'\ub07e': 'HangulSyllableGgigg',
u'\u8d80': 'CjkUnifiedIdeograph8d80',
u'\u2903': 'RightwardsDoubleArrowWithVerticalStroke',
u'\uce82': 'HangulSyllableKalp',
u'\u6a05': 'CjkUnifiedIdeograph6a05',
u'\ua707': 'ModifierLetterChineseToneYangRu',
u'\ue009': 'UnicodeCharacter0xe009',
u'\u1f8c': 'GreekCapitalLetterAlphaWithPsiliAndOxiaAndProsgegrammeni',
u'\u588e': 'CjkUnifiedIdeograph588e',
u'\u9590': 'CjkUnifiedIdeograph9590',
u'\u3113': 'BopomofoLetterZh',
u'\ud692': 'HangulSyllableHoenh',
u'\u7215': 'CjkUnifiedIdeograph7215',
u'\u4f17': 'CjkUnifiedIdeograph4f17',
u'\u8819': 'CjkUnifiedIdeograph8819',
u'\uc51b': 'HangulSyllableSsyilh',
u'\u279c': 'HeavyRoundtippedRightwardsArrow',
u'\u609e': 'CjkUnifiedIdeograph609e',
u'\ubda0': 'HangulSyllableBweon',
u'\ufea2': 'ArabicLetterHahFinalForm',
u'\u1a25': 'UnicodeCharacter0x1a25',
u'\u5727': 'CjkUnifiedIdeograph5727',
u'\u9029': 'CjkUnifiedIdeograph9029',
u'\ued2b': 'UnicodeCharacter0xed2b',
u'\u08ae': 'UnicodeCharacter0x8ae',
u'\u45b0': 'CjkUnifiedIdeograph45b0',
u'\u86b2': 'CjkUnifiedIdeograph86b2',
u'\u2235': 'Because',
u'\uc3b4': 'HangulSyllableSsyeoss',
u'\u7f37': 'CjkUnifiedIdeograph7f37',
u'\ub839': 'HangulSyllableRyeong',
u'\uf53b': 'UnicodeCharacter0xf53b',
u'\u10be': 'GeorgianCapitalLetterXan',
u'\u6dc0': 'CjkUnifiedIdeograph6dc0',
u'\uaec2': 'HangulSyllableGgeonh',
u'\uebc4': 'UnicodeCharacter0xebc4',
u'\u0747': 'SyriacObliqueLineAbove',
u'\u4049': 'CjkUnifiedIdeograph4049',
u'\u9d4b': 'CjkUnifiedIdeograph9d4b',
u'\ude4d': 'UnicodeCharacter0xde4d',
u'\u38ce': 'CjkUnifiedIdeograph38ce',
u'\u75d0': 'CjkUnifiedIdeograph75d0',
u'\ub6d2': 'HangulSyllableDdweop',
u'\uf3d4': 'UnicodeCharacter0xf3d4',
u'\u2f57': 'KangxiRadicalFather',
u'\uccd6': 'HangulSyllableCyeonh',
u'\u6859': 'CjkUnifiedIdeograph6859',
u'\ua55b': 'VaiSyllableTha',
u'\ue65d': 'UnicodeCharacter0xe65d',
u'\u1de0': 'CombiningLatinSmallLetterN',
u'\u5ee2': 'CjkUnifiedIdeograph5ee2',
u'\u9be4': 'CjkUnifiedIdeograph9be4',
u'\u3767': 'CjkUnifiedIdeograph3767',
u'\ud4e6': 'HangulSyllablePwip',
u'\u7069': 'CjkUnifiedIdeograph7069',
u'\u4d6b': 'CjkUnifiedIdeograph4d6b',
u'\u8e6d': 'CjkUnifiedIdeograph8e6d',
u'\ucb6f': 'HangulSyllableJjweolb',
u'\u25f0': 'WhiteSquareWithUpperLeftQuadrant',
u'\u66f2': 'CjkUnifiedIdeograph66f2',
u'\ua3f4': 'YiSyllableJurx',
u'\ufcf6': 'ArabicLigatureTahWithYehIsolatedForm',
u'\u1879': 'UnicodeCharacter0x1879',
u'\u557b': 'CjkUnifiedIdeograph557b',
u'\u967d': 'CjkUnifiedIdeograph967d',
u'\ud37f': 'HangulSyllablePeogs',
u'\uaf81': 'HangulSyllableGgoeg',
u'\u4804': 'CjkUnifiedIdeograph4804',
u'\u6487': 'CjkUnifiedIdeograph6487',
u'\uc208': 'HangulSyllableSyols',
u'\ufe8b': 'ArabicLetterYehWithHamzaAboveInitialForm',
u'\u010a': 'LatinCapitalLetterCWithDotAbove',
u'\u3d8d': 'CjkUnifiedIdeograph3d8d',
u'\u9b0e': 'CjkUnifiedIdeograph9b0e',
u'\ub791': 'HangulSyllableRang',
u'\u5014': 'CjkUnifiedIdeograph5014',
u'\u0c97': 'KannadaLetterGa',
u'\uea18': 'UnicodeCharacter0xea18',
u'\u869b': 'CjkUnifiedIdeograph869b',
u'\u291a': 'RightwardsArrowtail',
u'\ua31e': 'YiSyllableSut',
u'\u5fa1': 'CjkUnifiedIdeograph5fa1',
u'\ud9a5': 'UnicodeCharacter0xd9a5',
u'\u7824': 'CjkUnifiedIdeograph7824',
u'\u14a7': 'CanadianSyllabicsMo',
u'\uf228': 'UnicodeCharacter0xf228',
u'\uaeab': 'HangulSyllableGgyaelb',
u'\u312a': 'BopomofoLetterV',
u'\u4b2e': 'CjkUnifiedIdeograph4b2e',
u'\u67b1': 'CjkUnifiedIdeograph67b1',
u'\uc532': 'HangulSyllableSsilm',
u'\ue1b5': 'UnicodeCharacter0xe1b5',
u'\u3cb7': 'CjkUnifiedIdeograph3cb7',
u'\u9a38': 'CjkUnifiedIdeograph9a38',
u'\ub6bb': 'HangulSyllableDdweogs',
u'\u533e': 'CjkUnifiedIdeograph533e',
u'\u0fc1': 'TibetanCantillationSignLightBeat',
u'\ued42': 'UnicodeCharacter0xed42',
u'\u89c5': 'CjkUnifiedIdeograph89c5',
u'\u2844': 'BraillePatternDots37',
u'\ua248': 'YiSyllableHxi',
u'\u5ecb': 'CjkUnifiedIdeograph5ecb',
u'\ud8cf': 'UnicodeCharacter0xd8cf',
u'\u7b4e': 'CjkUnifiedIdeograph7b4e',
u'\u17d1': 'KhmerSignViriam',
u'\uf552': 'UnicodeCharacter0xf552',
u'\u91d5': 'CjkUnifiedIdeograph91d5',
u'\u3054': 'HiraganaLetterGo',
u'\u4a58': 'CjkUnifiedIdeograph4a58',
u'\u66db': 'CjkUnifiedIdeograph66db',
u'\uc45c': 'HangulSyllableSsyoss',
u'\ue0df': 'UnicodeCharacter0xe0df',
u'\u035e': 'CombiningDoubleMacron',
u'\u3fe1': 'CjkUnifiedIdeograph3fe1',
u'\u9d62': 'CjkUnifiedIdeograph9d62',
u'\ub9e5': 'HangulSyllableMaeg',
u'\u5268': 'CjkUnifiedIdeograph5268',
u'\u0eeb': 'UnicodeCharacter0xeeb',
u'\uec6c': 'UnicodeCharacter0xec6c',
u'\u88ef': 'CjkUnifiedIdeograph88ef',
u'\u2b6e': 'UnicodeCharacter0x2b6e',
u'\ua572': 'VaiSyllableOon',
u'\u41f5': 'CjkUnifiedIdeograph41f5',
u'\udbf9': 'UnicodeCharacter0xdbf9',
u'\u7a78': 'CjkUnifiedIdeograph7a78',
u'\u16fb': 'UnicodeCharacter0x16fb',
u'\uf47c': 'UnicodeCharacter0xf47c',
u'\u90ff': 'CjkUnifiedIdeograph90ff',
u'\u337e': 'SquareEraNameMeizi',
u'\uaa03': 'ChamLetterE',
u'\u4d82': 'CjkUnifiedIdeograph4d82',
u'\uc786': 'HangulSyllableIbs',
u'\u6309': 'CjkUnifiedIdeograph6309',
u'\u0288': 'LatinSmallLetterTWithRetroflexHook',
u'\ufd0d': 'ArabicLigatureSheenWithRehIsolatedForm',
u'\u9c8c': 'CjkUnifiedIdeograph9c8c',
u'\u380f': 'CjkUnifiedIdeograph380f',
u'\ub213': 'HangulSyllableNulh',
u'\u5592': 'CjkUnifiedIdeograph5592',
u'\ucc17': 'HangulSyllableJjilb',
u'\uef96': 'UnicodeCharacter0xef96',
u'\u0b19': 'OriyaLetterNga',
u'\u2a98': 'SlantedEqualToOrGreaterthanWithDotInside',
u'\u851d': 'CjkUnifiedIdeograph851d',
u'\ua49c': 'YiRadicalMop',
u'\u5a23': 'CjkUnifiedIdeograph5a23',
u'\u7da2': 'CjkUnifiedIdeograph7da2',
u'\ud427': 'HangulSyllablePwaegs',
u'\uf7a6': 'UnicodeCharacter0xf7a6',
u'\u1329': 'EthiopicSyllableChu',
u'\u32a8': 'CircledIdeographRight',
u'\uad2d': 'HangulSyllableGwaeng',
u'\u4cac': 'CjkUnifiedIdeograph4cac',
u'\uc6b0': 'HangulSyllableU',
u'\u6233': 'CjkUnifiedIdeograph6233',
u'\u05b2': 'HebrewPointHatafPatah',
u'\ufc37': 'ArabicLigatureKafWithAlefIsolatedForm',
u'\u9fb6': 'CjkUnifiedIdeograph9fb6',
u'\u3b39': 'CjkUnifiedIdeograph3b39',
u'\ub53d': 'HangulSyllableDdalt',
u'\u54bc': 'CjkUnifiedIdeograph54bc',
u'\ucf41': 'HangulSyllableKyelg',
u'\ueec0': 'UnicodeCharacter0xeec0',
u'\u0a43': 'UnicodeCharacter0xa43',
u'\u2dc2': 'EthiopicSyllableQyi',
u'\u8447': 'CjkUnifiedIdeograph8447',
u'\ua7c6': 'UnicodeCharacter0xa7c6',
u'\u5d4d': 'CjkUnifiedIdeograph5d4d',
u'\u7ccc': 'CjkUnifiedIdeograph7ccc',
u'\ud751': 'HangulSyllableHeug',
u'\uf6d0': 'UnicodeCharacter0xf6d0',
u'\u1253': 'EthiopicSyllableQhaa',
u'\u35d2': 'CjkUnifiedIdeograph35d2',
u'\uac57': 'HangulSyllableGyaegs',
u'\u4fd6': 'CjkUnifiedIdeograph4fd6',
u'\uc9da': 'HangulSyllableJip',
u'\u655d': 'CjkUnifiedIdeograph655d',
u'\u04dc': 'CyrillicCapitalLetterZheWithDiaeresis',
u'\uff61': 'HalfwidthIdeographicFullStop',
u'\u9ee0': 'CjkUnifiedIdeograph9ee0',
u'\u3a63': 'CjkUnifiedIdeograph3a63',
u'\ub467': 'HangulSyllableDuc',
u'\u57e6': 'CjkUnifiedIdeograph57e6',
u'\uce6b': 'HangulSyllableCis',
u'\ud1ea': 'HangulSyllableTwaebs',
u'\u0d6d': 'MalayalamDigitSeven',
u'\u2cec': 'UnicodeCharacter0x2cec',
u'\u8771': 'CjkUnifiedIdeograph8771',
u'\ua6f0': 'UnicodeCharacter0xa6f0',
u'\u5c77': 'CjkUnifiedIdeograph5c77',
u'\u7ff6': 'CjkUnifiedIdeograph7ff6',
u'\ud67b': 'HangulSyllableHwaelb',
u'\uf9fa': 'CjkCompatibilityIdeographf9fa',
u'\u157d': 'CanadianSyllabicsHk',
u'\u34fc': 'CjkUnifiedIdeograph34fc',
u'\u2e81': 'CjkRadicalCliff',
u'\u4f00': 'CjkUnifiedIdeograph4f00',
u'\u6b83': 'CjkUnifiedIdeograph6b83',
u'\u8c02': 'CjkUnifiedIdeograph8c02',
u'\ua885': 'SaurashtraLetterIi',
u'\uc904': 'HangulSyllableJul',
u'\ue587': 'UnicodeCharacter0xe587',
u'\u1a0e': 'BugineseLetterNya',
u'\u3691': 'CjkUnifiedIdeograph3691',
u'\u5710': 'CjkUnifiedIdeograph5710',
u'\u7393': 'CjkUnifiedIdeograph7393',
u'\u9412': 'CjkUnifiedIdeograph9412',
u'\ub095': 'HangulSyllableGgit',
u'\ud114': 'HangulSyllableTyae',
u'\u8d97': 'CjkUnifiedIdeograph8d97',
u'\uca99': 'HangulSyllableJjyeong',
u'\u221e': 'Infinity',
u'\u7f20': 'CjkUnifiedIdeograph7f20',
u'\u1ba3': 'SundaneseConsonantSignPanyiku',
u'\ubc22': 'HangulSyllableBalp',
u'\u58a5': 'CjkUnifiedIdeograph58a5',
u'\uf924': 'CjkCompatibilityIdeographf924',
u'\u95a7': 'CjkUnifiedIdeograph95a7',
u'\ud2a9': 'HangulSyllableTyult',
u'\u0730': 'SyriacPthahaAbove',
u'\u23b3': 'SummationBottom',
u'\u4432': 'CjkUnifiedIdeograph4432',
u'\u60b5': 'CjkUnifiedIdeograph60b5',
u'\u8134': 'CjkUnifiedIdeograph8134',
u'\ubdb7': 'HangulSyllableBweoh',
u'\ude36': 'UnicodeCharacter0xde36',
u'\ufab9': 'CjkCompatibilityIdeographfab9',
u'\u2f40': 'KangxiRadicalBranch',
u'\u6c42': 'CjkUnifiedIdeograph6c42',
u'\u08c5': 'UnicodeCharacter0x8c5',
u'\ua944': 'RejangLetterNda',
u'\u45c7': 'CjkUnifiedIdeograph45c7',
u'\ue646': 'UnicodeCharacter0xe646',
u'\u82c9': 'CjkUnifiedIdeograph82c9',
u'\udfcb': 'UnicodeCharacter0xdfcb',
u'\u3750': 'CjkUnifiedIdeograph3750',
u'\u7452': 'CjkUnifiedIdeograph7452',
u'\u10d5': 'GeorgianLetterVin',
u'\ub154': 'HangulSyllableNyeoss',
u'\u6dd7': 'CjkUnifiedIdeograph6dd7',
u'\u8e56': 'CjkUnifiedIdeograph8e56',
u'\uaad9': 'UnicodeCharacter0xaad9',
u'\ucb58': 'HangulSyllableJjum',
u'\ue7db': 'UnicodeCharacter0xe7db',
u'\u1c62': 'OlChikiLetterAam',
u'\u38e5': 'CjkUnifiedIdeograph38e5',
u'\u5964': 'CjkUnifiedIdeograph5964',
u'\u75e7': 'CjkUnifiedIdeograph75e7',
u'\u9666': 'CjkUnifiedIdeograph9666',
u'\ub2e9': 'HangulSyllableDanj',
u'\ud368': 'HangulSyllablePyael',
u'\u8feb': 'CjkUnifiedIdeograph8feb',
u'\ucced': 'HangulSyllableCyeg',
u'\u2472': 'CircledNumberNineteen',
u'\u6174': 'CjkUnifiedIdeograph6174',
u'\u1df7': 'UnicodeCharacter0x1df7',
u'\ube76': 'HangulSyllableBbaj',
u'\u5af9': 'CjkUnifiedIdeograph5af9',
u'\ufb78': 'ArabicLetterNyehInitialForm',
u'\u97fb': 'CjkUnifiedIdeograph97fb',
u'\ud4fd': 'HangulSyllablePyung',
u'\uee01': 'UnicodeCharacter0xee01',
u'\u0984': 'UnicodeCharacter0x984',
u'\u4686': 'CjkUnifiedIdeograph4686',
u'\u8388': 'CjkUnifiedIdeograph8388',
u'\u3f0b': 'CjkUnifiedIdeograph3f0b',
u'\uc08a': 'HangulSyllableBbyij',
u'\u7c0d': 'CjkUnifiedIdeograph7c0d',
u'\ub90f': 'HangulSyllableRweolb',
u'\uf611': 'UnicodeCharacter0xf611',
u'\u1194': 'HangulJungseongYui',
u'\u6e96': 'CjkUnifiedIdeograph6e96',
u'\uab98': 'UnicodeCharacter0xab98',
u'\ue89a': 'UnicodeCharacter0xe89a',
u'\u041d': 'CyrillicCapitalLetterEn',
u'\u411f': 'CjkUnifiedIdeograph411f',
u'\u9e21': 'CjkUnifiedIdeograph9e21',
u'\udb23': 'UnicodeCharacter0xdb23',
u'\u39a4': 'CjkUnifiedIdeograph39a4',
u'\u76a6': 'CjkUnifiedIdeograph76a6',
u'\ub3a8': 'HangulSyllableDye',
u'\uf0aa': 'UnicodeCharacter0xf0aa',
u'\u2c2d': 'GlagoliticCapitalLetterTrokutastiA',
u'\ucdac': 'HangulSyllableCuk',
u'\u692f': 'CjkUnifiedIdeograph692f',
u'\ua631': 'UnicodeCharacter0xa631',
u'\ue333': 'UnicodeCharacter0xe333',
u'\u1eb6': 'LatinCapitalLetterAWithBreveAndDotBelow',
u'\u5bb8': 'CjkUnifiedIdeograph5bb8',
u'\u98ba': 'CjkUnifiedIdeograph98ba',
u'\u343d': 'CjkUnifiedIdeograph343d',
u'\ud5bc': 'HangulSyllableHyaem',
u'\u713f': 'CjkUnifiedIdeograph713f',
u'\u4e41': 'CjkUnifiedIdeograph4e41',
u'\u8b43': 'CjkUnifiedIdeograph8b43',
u'\uc845': 'HangulSyllableJyeolt',
u'\u26c6': 'UnicodeCharacter0x26c6',
u'\u63c8': 'CjkUnifiedIdeograph63c8',
u'\ua0ca': 'YiSyllableMyx',
u'\ufdcc': 'UnicodeCharacter0xfdcc',
u'\u194f': 'LimbuDigitNine',
u'\u5651': 'CjkUnifiedIdeograph5651',
u'\u9353': 'CjkUnifiedIdeograph9353',
u'\ud055': 'HangulSyllableKyunj',
u'\u0bd8': 'UnicodeCharacter0xbd8',
u'\u48da': 'CjkUnifiedIdeograph48da',
u'\u85dc': 'CjkUnifiedIdeograph85dc',
u'\u215f': 'FractionNumeratorOne',
u'\uc2de': 'HangulSyllableSigg',
u'\u7e61': 'CjkUnifiedIdeograph7e61',
u'\ubb63': 'HangulSyllableMweos',
u'\uf865': 'UnicodeCharacter0xf865',
u'\u13e8': 'CherokeeLetterTsv',
u'\u50ea': 'CjkUnifiedIdeograph50ea',
u'\uadec': 'HangulSyllableGyum',
u'\ueaee': 'UnicodeCharacter0xeaee',
u'\u0671': 'ArabicLetterAlefWasla',
u'\u4373': 'CjkUnifiedIdeograph4373',
u'\u8075': 'CjkUnifiedIdeograph8075',
u'\udd77': 'UnicodeCharacter0xdd77',
u'\u3bf8': 'CjkUnifiedIdeograph3bf8',
u'\u78fa': 'CjkUnifiedIdeograph78fa',
u'\ub5fc': 'HangulSyllableDdyel',
u'\uf2fe': 'UnicodeCharacter0xf2fe',
u'\ud000': 'HangulSyllableKweon',
u'\uf483': 'UnicodeCharacter0xf483',
u'\u1302': 'EthiopicSyllableJi',
u'\u3785': 'CjkUnifiedIdeograph3785',
u'\u9906': 'CjkUnifiedIdeograph9906',
u'\ubd89': 'HangulSyllableBulg',
u'\u420c': 'CjkUnifiedIdeograph420c',
u'\u668f': 'CjkUnifiedIdeograph668f',
u'\uc810': 'HangulSyllableJeom',
u'\uec93': 'UnicodeCharacter0xec93',
u'\u0b12': 'UnicodeCharacter0xb12',
u'\u2f95': 'KangxiRadicalValley',
u'\ub116': 'HangulSyllableNeolp',
u'\u5599': 'CjkUnifiedIdeograph5599',
u'\udb9d': 'UnicodeCharacter0xdb9d',
u'\u7a1c': 'CjkUnifiedIdeograph7a1c',
u'\u1e9f': 'LatinSmallLetterDelta',
u'\ue020': 'UnicodeCharacter0xe020',
u'\u84a3': 'CjkUnifiedIdeograph84a3',
u'\u2322': 'Frown',
u'\ua926': 'KayahLiVowelUe',
u'\u4da9': 'CjkUnifiedIdeograph4da9',
u'\uf3ad': 'UnicodeCharacter0xf3ad',
u'\u122c': 'EthiopicSyllableRee',
u'\u36af': 'CjkUnifiedIdeograph36af',
u'\u9830': 'CjkUnifiedIdeograph9830',
u'\ubcb3': 'HangulSyllableBes',
u'\u4136': 'CjkUnifiedIdeograph4136',
u'\u65b9': 'CjkUnifiedIdeograph65b9',
u'\uc73a': 'HangulSyllableYup',
u'\uebbd': 'UnicodeCharacter0xebbd',
u'\u0a3c': 'GurmukhiSignNukta',
u'\u2ebf': 'CjkRadicalGrassTwo',
u'\ub040': 'HangulSyllableGgyuk',
u'\u54c3': 'CjkUnifiedIdeograph54c3',
u'\udac7': 'UnicodeCharacter0xdac7',
u'\u7946': 'CjkUnifiedIdeograph7946',
u'\u1dc9': 'CombiningAcutegraveacute',
u'\uff4a': 'FullwidthLatinSmallLetterJ',
u'\u83cd': 'CjkUnifiedIdeograph83cd',
u'\u224c': 'AllEqualTo',
u'\ua850': 'PhagspaLetterTsa',
u'\u4cd3': 'CjkUnifiedIdeograph4cd3',
u'\uf2d7': 'UnicodeCharacter0xf2d7',
u'\u1156': 'HangulChoseongPhieuphpieup',
u'\u35d9': 'CjkUnifiedIdeograph35d9',
u'\u975a': 'CjkUnifiedIdeograph975a',
u'\ubbdd': 'HangulSyllableMyig',
u'\u4060': 'CjkUnifiedIdeograph4060',
u'\u64e3': 'CjkUnifiedIdeograph64e3',
u'\uc664': 'HangulSyllableWael',
u'\ueae7': 'UnicodeCharacter0xeae7',
u'\u0966': 'DevanagariDigitZero',
u'\u2de9': 'CombiningCyrillicLetterEn',
u'\u8f6a': 'CjkUnifiedIdeograph8f6a',
u'\u53ed': 'CjkUnifiedIdeograph53ed',
u'\ud9f1': 'UnicodeCharacter0xd9f1',
u'\u7870': 'CjkUnifiedIdeograph7870',
u'\u1cf3': 'UnicodeCharacter0x1cf3',
u'\ufe74': 'ArabicKasratanIsolatedForm',
u'\u82f7': 'CjkUnifiedIdeograph82f7',
u'\u2176': 'SmallRomanNumeralSeven',
u'\ua77a': 'LatinSmallLetterInsularD',
u'\u4bfd': 'CjkUnifiedIdeograph4bfd',
u'\u7101': 'CjkUnifiedIdeograph7101',
u'\u1080': 'MyanmarLetterShanTha',
u'\uf705': 'UnicodeCharacter0xf705',
u'\u9684': 'CjkUnifiedIdeograph9684',
u'\u3a07': 'CjkUnifiedIdeograph3a07',
u'\ua00b': 'YiSyllableAp',
u'\u5f8a': 'CjkUnifiedIdeograph5f8a',
u'\uc58e': 'HangulSyllableYabs',
u'\u6911': 'CjkUnifiedIdeograph6911',
u'\u0890': 'UnicodeCharacter0x890',
u'\uef15': 'UnicodeCharacter0xef15',
u'\u8e94': 'CjkUnifiedIdeograph8e94',
u'\u581b': 'CjkUnifiedIdeograph581b',
u'\u779a': 'CjkUnifiedIdeograph779a',
u'\ude1f': 'UnicodeCharacter0xde1f',
u'\ufd9e': 'ArabicLigatureBehWithKhahWithYehFinalForm',
u'\u0121': 'LatinSmallLetterGWithDotAbove',
u'\u20a0': 'EurocurrencySign',
u'\u8725': 'CjkUnifiedIdeograph8725',
u'\ua6a4': 'UnicodeCharacter0xa6a4',
u'\u702b': 'CjkUnifiedIdeograph702b',
u'\u6faa': 'CjkUnifiedIdeograph6faa',
u'\uf62f': 'UnicodeCharacter0xf62f',
u'\u95ae': 'CjkUnifiedIdeograph95ae',
u'\u3931': 'CjkUnifiedIdeograph3931',
u'\ubf35': 'HangulSyllableBbyeb',
u'\u5eb4': 'CjkUnifiedIdeograph5eb4',
u'\uc4b8': 'HangulSyllableSswi',
u'\u683b': 'CjkUnifiedIdeograph683b',
u'\u07ba': 'UnicodeCharacter0x7ba',
u'\uee3f': 'UnicodeCharacter0xee3f',
u'\u8dbe': 'CjkUnifiedIdeograph8dbe',
u'\u5745': 'CjkUnifiedIdeograph5745',
u'\u76c4': 'CjkUnifiedIdeograph76c4',
u'\udd49': 'UnicodeCharacter0xdd49',
u'\ufcc8': 'ArabicLigatureKafWithMeemInitialForm',
u'\u3fca': 'CjkUnifiedIdeograph3fca',
u'\u864f': 'CjkUnifiedIdeograph864f',
u'\ua5ce': 'VaiSyllableDhho',
u'\u4f55': 'CjkUnifiedIdeograph4f55',
u'\u6ed4': 'CjkUnifiedIdeograph6ed4',
u'\uf559': 'UnicodeCharacter0xf559',
u'\u94d8': 'CjkUnifiedIdeograph94d8',
u'\u385b': 'CjkUnifiedIdeograph385b',
u'\ube5f': 'HangulSyllableBih',
u'\u5dde': 'CjkUnifiedIdeograph5dde',
u'\uc3e2': 'HangulSyllableSsolm',
u'\u6765': 'CjkUnifiedIdeograph6765',
u'\u06e4': 'ArabicSmallHighMadda',
u'\ued69': 'UnicodeCharacter0xed69',
u'\u8ce8': 'CjkUnifiedIdeograph8ce8',
u'\u566f': 'CjkUnifiedIdeograph566f',
u'\u75ee': 'CjkUnifiedIdeograph75ee',
u'\udc73': 'UnicodeCharacter0xdc73',
u'\ufbf2': 'ArabicLigatureYehWithHamzaAboveWithOeIsolatedForm',
u'\u1f75': 'GreekSmallLetterEtaWithOxia',
u'\u3ef4': 'CjkUnifiedIdeograph3ef4',
u'\u8579': 'CjkUnifiedIdeograph8579',
u'\ua4f8': 'UnicodeCharacter0xa4f8',
u'\u4e7f': 'CjkUnifiedIdeograph4e7f',
u'\u6dfe': 'CjkUnifiedIdeograph6dfe',
u'\u1806': 'MongolianTodoSoftHyphen',
u'\u3c89': 'CjkUnifiedIdeograph3c89',
u'\u5d08': 'CjkUnifiedIdeograph5d08',
u'\u618b': 'CjkUnifiedIdeograph618b',
u'\u9e0a': 'CjkUnifiedIdeograph9e0a',
u'\ua28d': 'YiSyllableZit',
u'\uc30c': 'HangulSyllableSsass',
u'\ue78f': 'UnicodeCharacter0xe78f',
u'\u3016': 'LeftWhiteLenticularBracket',
u'\u7518': 'CjkUnifiedIdeograph7518',
u'\u199b': 'NewTaiLueLetterHighVa',
u'\ub61a': 'HangulSyllableDdolm',
u'\u5a9d': 'CjkUnifiedIdeograph5a9d',
u'\ufb1c': 'UnicodeCharacter0xfb1c',
u'\u9f9f': 'CjkUnifiedIdeograph9f9f',
u'\uc0a1': 'HangulSyllableBbib',
u'\u2826': 'BraillePatternDots236',
u'\u6d28': 'CjkUnifiedIdeograph6d28',
u'\u31ab': 'BopomofoLetterUnn',
u'\uae2a': 'HangulSyllableGyij',
u'\u72ad': 'CjkUnifiedIdeograph72ad',
u'\u932c': 'CjkUnifiedIdeograph932c',
u'\ub7af': 'HangulSyllableRaec',
u'\ud42e': 'HangulSyllablePwaelm',
u'\uf8b1': 'UnicodeCharacter0xf8b1',
u'\u0538': 'ArmenianCapitalLetterEt',
u'\u29bb': 'CircleWithSuperimposedX',
u'\u463a': 'CjkUnifiedIdeograph463a',
u'\u6abd': 'CjkUnifiedIdeograph6abd',
u'\u8b3c': 'CjkUnifiedIdeograph8b3c',
u'\uafbf': 'HangulSyllableGgud',
u'\ucc3e': 'HangulSyllableCaj',
u'\u90c1': 'CjkUnifiedIdeograph90c1',
u'\ud5c3': 'HangulSyllableHyaec',
u'\u3d48': 'CjkUnifiedIdeograph3d48',
u'\u7e4a': 'CjkUnifiedIdeograph7e4a',
u'\u02cd': 'ModifierLetterLowMacron',
u'\ua34c': 'YiSyllableZhuop',
u'\u47cf': 'CjkUnifiedIdeograph47cf',
u'\ue44e': 'UnicodeCharacter0xe44e',
u'\u88d1': 'CjkUnifiedIdeograph88d1',
u'\ucdd3': 'HangulSyllableCwed',
u'\u165a': 'CanadianSyllabicsCarrierSh',
u'\u3add': 'CjkUnifiedIdeograph3add',
u'\u5b5c': 'CjkUnifiedIdeograph5b5c',
u'\u7fdf': 'CjkUnifiedIdeograph7fdf',
u'\u9c5e': 'CjkUnifiedIdeograph9c5e',
u'\ua0e1': 'YiSyllableFyp',
u'\uc160': 'HangulSyllableSyeols',
u'\ue5e3': 'UnicodeCharacter0xe5e3',
u'\u0e6a': 'UnicodeCharacter0xe6a',
u'\u736c': 'CjkUnifiedIdeograph736c',
u'\u17ef': 'UnicodeCharacter0x17ef',
u'\ub46e': 'HangulSyllableDweogg',
u'\u58f1': 'CjkUnifiedIdeograph58f1',
u'\uf970': 'CjkCompatibilityIdeographf970',
u'\u9df3': 'CjkUnifiedIdeograph9df3',
u'\udef5': 'UnicodeCharacter0xdef5',
u'\u267a': 'RecyclingSymbolForGenericMaterials',
u'\u6b7c': 'CjkUnifiedIdeograph6b7c',
u'\u0fff': 'UnicodeCharacter0xfff',
u'\uac7e': 'HangulSyllableGeolp',
u'\u9180': 'CjkUnifiedIdeograph9180',
u'\u3503': 'CjkUnifiedIdeograph3503',
u'\ud282': 'HangulSyllableTwigg',
u'\u7605': 'CjkUnifiedIdeograph7605',
u'\ubb07': 'HangulSyllableMoelb',
u'\ufc09': 'ArabicLigatureBehWithAlefMaksuraIsolatedForm',
u'\u038c': 'GreekCapitalLetterOmicronWithTonos',
u'\u448e': 'CjkUnifiedIdeograph448e',
u'\u8990': 'CjkUnifiedIdeograph8990',
u'\u2d13': 'GeorgianSmallLetterUn',
u'\uca92': 'HangulSyllableJjyeolp',
u'\u6e15': 'CjkUnifiedIdeograph6e15',
u'\u5317': 'CjkUnifiedIdeograph5317',
u'\u9419': 'CjkUnifiedIdeograph9419',
u'\ud91b': 'UnicodeCharacter0xd91b',
u'\u3b9c': 'CjkUnifiedIdeograph3b9c',
u'\u7c9e': 'CjkUnifiedIdeograph7c9e',
u'\ua1a0': 'YiSyllableHla',
u'\ue2a2': 'UnicodeCharacter0xe2a2',
u'\u0625': 'ArabicLetterAlefWithHamzaBelow',
u'\u4b27': 'CjkUnifiedIdeograph4b27',
u'\u8c29': 'CjkUnifiedIdeograph8c29',
u'\uf12b': 'UnicodeCharacter0xf12b',
u'\u14ae': 'CanadianSyllabicsMwi',
u'\u59b0': 'CjkUnifiedIdeograph59b0',
u'\u9ab2': 'CjkUnifiedIdeograph9ab2',
u'\u3e35': 'CjkUnifiedIdeograph3e35',
u'\udfb4': 'UnicodeCharacter0xdfb4',
u'\u6337': 'CjkUnifiedIdeograph6337',
u'\ua439': 'YiSyllableNjuox',
u'\ue93b': 'UnicodeCharacter0xe93b',
u'\u0cbe': 'KannadaVowelSignAa',
u'\u71c0': 'CjkUnifiedIdeograph71c0',
u'\ub2c2': 'HangulSyllableNyij',
u'\uf7c4': 'UnicodeCharacter0xf7c4',
u'\u1b47': 'BalineseLetterTzirSasak',
u'\u5c49': 'CjkUnifiedIdeograph5c49',
u'\u814b': 'CjkUnifiedIdeograph814b',
u'\uc24d': 'HangulSyllableSweot',
u'\u24ce': 'CircledLatinCapitalLetterY',
u'\u69d0': 'CjkUnifiedIdeograph69d0',
u'\uaad2': 'UnicodeCharacter0xaad2',
u'\uefd4': 'UnicodeCharacter0xefd4',
u'\u3357': 'SquareWatto',
u'\ud0d6': 'HangulSyllableTaj',
u'\u7459': 'CjkUnifiedIdeograph7459',
u'\ub95b': 'HangulSyllableRyugs',
u'\ufa5d': 'CjkCompatibilityIdeographfa5d',
u'\u01e0': 'LatinCapitalLetterAWithDotAboveAndMacron',
u'\u42e2': 'CjkUnifiedIdeograph42e2',
u'\u87e4': 'CjkUnifiedIdeograph87e4',
u'\u2b67': 'UnicodeCharacter0x2b67',
u'\uc8e6': 'HangulSyllableJyonh',
u'\u6c69': 'CjkUnifiedIdeograph6c69',
u'\u516b': 'CjkUnifiedIdeograph516b',
u'\u926d': 'CjkUnifiedIdeograph926d',
u'\ud76f': 'HangulSyllableHyigs',
u'\u39f0': 'CjkUnifiedIdeograph39f0',
u'\u7af2': 'CjkUnifiedIdeograph7af2',
u'\ubff4': 'HangulSyllableBbweols',
u'\ue0f6': 'UnicodeCharacter0xe0f6',
u'\u0479': 'CyrillicSmallLetterUk',
u'\u497b': 'CjkUnifiedIdeograph497b',
u'\u8a7d': 'CjkUnifiedIdeograph8a7d',
u'\ucf7f': 'HangulSyllableKwalh',
u'\ub381': 'HangulSyllableDeb',
u'\u5404': 'CjkUnifiedIdeograph5404',
u'\u7887': 'CjkUnifiedIdeograph7887',
u'\ude08': 'UnicodeCharacter0xde08',
u'\ue28b': 'UnicodeCharacter0xe28b',
u'\u1d0a': 'LatinLetterSmallCapitalJ',
u'\u218d': 'UnicodeCharacter0x218d',
u'\u870e': 'CjkUnifiedIdeograph870e',
u'\uab91': 'UnicodeCharacter0xab91',
u'\u4c14': 'CjkUnifiedIdeograph4c14',
u'\u1097': 'MyanmarShanDigitSeven',
u'\uf618': 'UnicodeCharacter0xf618',
u'\u9a9b': 'CjkUnifiedIdeograph9a9b',
u'\u351a': 'CjkUnifiedIdeograph351a',
u'\ubf1e': 'HangulSyllableBbyeoj',
u'\u43a1': 'CjkUnifiedIdeograph43a1',
u'\uc5a5': 'HangulSyllableYaelt',
u'\u6424': 'CjkUnifiedIdeograph6424',
u'\u08a7': 'UnicodeCharacter0x8a7',
u'\uee28': 'UnicodeCharacter0xee28',
u'\ub2ab': 'HangulSyllableNeuh',
u'\u2d2a': 'UnicodeCharacter0x2d2a',
u'\u572e': 'CjkUnifiedIdeograph572e',
u'\u7bb1': 'CjkUnifiedIdeograph7bb1',
u'\ud932': 'UnicodeCharacter0xd932',
u'\ufdb5': 'ArabicLigatureLamWithHahWithMeemInitialForm',
u'\u1c34': 'LepchaConsonantSignNyindo',
u'\u20b7': 'UnicodeCharacter0x20b7',
u'\u8638': 'CjkUnifiedIdeograph8638',
u'\uaabb': 'UnicodeCharacter0xaabb',
u'\u4f3e': 'CjkUnifiedIdeograph4f3e',
u'\u13c1': 'CherokeeLetterNe',
u'\uf142': 'UnicodeCharacter0xf142',
u'\u95c5': 'CjkUnifiedIdeograph95c5',
u'\u3444': 'CjkUnifiedIdeograph3444',
u'\ube48': 'HangulSyllableBin',
u'\u42cb': 'CjkUnifiedIdeograph42cb',
u'\uc4cf': 'HangulSyllableSswic',
u'\u674e': 'CjkUnifiedIdeograph674e',
u'\u0bd1': 'UnicodeCharacter0xbd1',
u'\ue952': 'UnicodeCharacter0xe952',
u'\u8dd5': 'CjkUnifiedIdeograph8dd5',
u'\u2c54': 'GlagoliticSmallLetterSmallYus',
u'\u5658': 'CjkUnifiedIdeograph5658',
u'\u7adb': 'CjkUnifiedIdeograph7adb',
u'\ud85c': 'UnicodeCharacter0xd85c',
u'\ufcdf': 'ArabicLigatureYehWithHamzaAboveWithMeemMedialForm',
u'\u1f5e': 'UnicodeCharacter0x1f5e',
u'\u23e1': 'BottomTortoiseShellBracket',
u'\u8162': 'CjkUnifiedIdeograph8162',
u'\ua5e5': 'VaiSyllableHen',
u'\u4e68': 'CjkUnifiedIdeograph4e68',
u'\u12eb': 'EthiopicSyllableYaa',
u'\uf06c': 'UnicodeCharacter0xf06c',
u'\u94ef': 'CjkUnifiedIdeograph94ef',
u'\u376e': 'CjkUnifiedIdeograph376e',
u'\ub972': 'HangulSyllableRyup',
u'\u5df5': 'CjkUnifiedIdeograph5df5',
u'\uc7f9': 'HangulSyllableJyaeng',
u'\u6678': 'CjkUnifiedIdeograph6678',
u'\u0afb': 'UnicodeCharacter0xafb',
u'\ue87c': 'UnicodeCharacter0xe87c',
u'\u8cff': 'CjkUnifiedIdeograph8cff',
u'\u2f7e': 'KangxiRadicalPlow',
u'\ub603': 'HangulSyllableDdyelh',
u'\u5182': 'CjkUnifiedIdeograph5182',
u'\udb86': 'UnicodeCharacter0xdb86',
u'\u7f09': 'CjkUnifiedIdeograph7f09',
u'\u1e88': 'LatinCapitalLetterWWithDotBelow',
u'\ue10d': 'UnicodeCharacter0xe10d',
u'\u808c': 'CjkUnifiedIdeograph808c',
u'\u240f': 'SymbolForShiftIn',
u'\uae13': 'HangulSyllableGeuh',
u'\u4992': 'CjkUnifiedIdeograph4992',
u'\ud017': 'HangulSyllableKweoh',
u'\uf396': 'UnicodeCharacter0xf396',
u'\u1719': 'UnicodeCharacter0x1719',
u'\u3698': 'CjkUnifiedIdeograph3698',
u'\u991d': 'CjkUnifiedIdeograph991d',
u'\ub89c': 'HangulSyllableRwael',
u'\u4623': 'CjkUnifiedIdeograph4623',
u'\u61a2': 'CjkUnifiedIdeograph61a2',
u'\uc827': 'HangulSyllableJelb',
u'\ueba6': 'UnicodeCharacter0xeba6',
u'\u0f29': 'TibetanDigitNine',
u'\u2ea8': 'CjkRadicalDog',
u'\ub12d': 'HangulSyllableNelg',
u'\u50ac': 'CjkUnifiedIdeograph50ac',
u'\udab0': 'UnicodeCharacter0xdab0',
u'\u7e33': 'CjkUnifiedIdeograph7e33',
u'\u19b2': 'NewTaiLueVowelSignIi',
u'\ue037': 'UnicodeCharacter0xe037',
u'\u83b6': 'CjkUnifiedIdeograph83b6',
u'\u2739': 'TwelvePointedBlackStar',
u'\ua93d': 'RejangLetterRa',
u'\u48bc': 'CjkUnifiedIdeograph48bc',
u'\ud341': 'HangulSyllablePaet',
u'\uf2c0': 'UnicodeCharacter0xf2c0',
u'\u1643': 'CanadianSyllabicsCarrierZee',
u'\u31c2': 'CjkStrokeXg',
u'\u9847': 'CjkUnifiedIdeograph9847',
u'\ubbc6': 'HangulSyllableMeunh',
u'\u414d': 'CjkUnifiedIdeograph414d',
u'\u60cc': 'CjkUnifiedIdeograph60cc',
u'\ucb51': 'HangulSyllableJjulg',
u'\uead0': 'UnicodeCharacter0xead0',
u'\u0e53': 'ThaiDigitThree',
u'\u29d2': 'BowtieWithRightHalfBlack',
u'\ub057': 'HangulSyllableGgeus',
u'\u53d6': 'CjkUnifiedIdeograph53d6',
u'\ud5da': 'HangulSyllableHeobs',
u'\u795d': 'CjkUnifiedIdeograph795d',
u'\u18dc': 'UnicodeCharacter0x18dc',
u'\ue361': 'UnicodeCharacter0xe361',
u'\u82e0': 'CjkUnifiedIdeograph82e0',
u'\u2663': 'BlackClubSuit',
u'\ua867': 'PhagspaSubjoinedLetterWa',
u'\u4be6': 'CjkUnifiedIdeograph4be6',
u'\ud26b': 'HangulSyllableTwed',
u'\ucdea': 'HangulSyllableCwigg',
u'\u116d': 'HangulJungseongYo',
u'\u30ec': 'KatakanaLetterRe',
u'\u9b71': 'CjkUnifiedIdeograph9b71',
u'\ubaf0': 'HangulSyllableMwaem',
u'\u4077': 'CjkUnifiedIdeograph4077',
u'\u63f6': 'CjkUnifiedIdeograph63f6',
u'\uca7b': 'HangulSyllableJjes',
u'\ue5fa': 'UnicodeCharacter0xe5fa',
u'\u097d': 'DevanagariLetterGlottalStop',
u'\u28fc': 'BraillePatternDots345678',
u'\u3281': 'CircledIdeographTwo',
u'\u5300': 'CjkUnifiedIdeograph5300',
u'\u7783': 'CjkUnifiedIdeograph7783',
u'\u9002': 'CjkUnifiedIdeograph9002',
u'\ub485': 'HangulSyllableDweot',
u'\ud504': 'HangulSyllablePeu',
u'\uf987': 'CjkCompatibilityIdeographf987',
u'\u060e': 'ArabicPoeticVerseSign',
u'\u2a91': 'LessthanAboveGreaterthanAboveDoublelineEqual',
u'\u4b10': 'CjkUnifiedIdeograph4b10',
u'\u6f93': 'CjkUnifiedIdeograph6f93',
u'\u8812': 'CjkUnifiedIdeograph8812',
u'\uac95': 'HangulSyllableGelg',
u'\ucd14': 'HangulSyllableCols',
u'\u9197': 'CjkUnifiedIdeograph9197',
u'\ud699': 'HangulSyllableHoelt',
u'\u3e1e': 'CjkUnifiedIdeograph3e1e',
u'\u6320': 'CjkUnifiedIdeograph6320',
u'\u07a3': 'ThaanaLetterGhainu',
u'\ua022': 'YiSyllableBuox',
u'\u44a5': 'CjkUnifiedIdeograph44a5',
u'\ue524': 'UnicodeCharacter0xe524',
u'\u89a7': 'CjkUnifiedIdeograph89a7',
u'\ucea9': 'HangulSyllableKaet',
u'\u1b30': 'BalineseLetterSaSaga',
u'\u3fb3': 'CjkUnifiedIdeograph3fb3',
u'\u5832': 'CjkUnifiedIdeograph5832',
u'\u7cb5': 'CjkUnifiedIdeograph7cb5',
u'\u9d34': 'CjkUnifiedIdeograph9d34',
u'\ua1b7': 'YiSyllableLit',
u'\uc236': 'HangulSyllableSweogg',
u'\ue6b9': 'UnicodeCharacter0xe6b9',
u'\u3340': 'SquarePondo',
u'\u7042': 'CjkUnifiedIdeograph7042',
u'\u14c5': 'CanadianSyllabicsNoo',
u'\ub544': 'HangulSyllableDdass',
u'\u59c7': 'CjkUnifiedIdeograph59c7',
u'\ufa46': 'CjkCompatibilityIdeographfa46',
u'\u9ec9': 'CjkUnifiedIdeograph9ec9',
u'\uc3cb': 'HangulSyllableSsyelh',
u'\u2b50': 'WhiteMediumStar',
u'\u6852': 'CjkUnifiedIdeograph6852',
u'\u0cd5': 'KannadaLengthMark',
u'\uad54': 'HangulSyllableGyon',
u'\u71d7': 'CjkUnifiedIdeograph71d7',
u'\u9256': 'CjkUnifiedIdeograph9256',
u'\ub6d9': 'HangulSyllableDdwenj',
u'\ud758': 'HangulSyllableHeul',
u'\ufbdb': 'ArabicLetterYuIsolatedForm',
u'\u24e5': 'CircledLatinSmallLetterV',
u'\u4564': 'CjkUnifiedIdeograph4564',
u'\u69e7': 'CjkUnifiedIdeograph69e7',
u'\u8a66': 'CjkUnifiedIdeograph8a66',
u'\uaee9': 'HangulSyllableGgeb',
u'\ucf68': 'HangulSyllableKoss',
u'\u93eb': 'CjkUnifiedIdeograph93eb',
u'\ud0ed': 'HangulSyllableTaeb',
u'\u3872': 'CjkUnifiedIdeograph3872',
u'\u7d74': 'CjkUnifiedIdeograph7d74',
u'\u01f7': 'LatinCapitalLetterWynn',
u'\ua276': 'YiSyllableHuox',
u'\u46f9': 'CjkUnifiedIdeograph46f9',
u'\ue778': 'UnicodeCharacter0xe778',
u'\u8bfb': 'CjkUnifiedIdeograph8bfb',
u'\uc8fd': 'HangulSyllableJug',
u'\uf201': 'UnicodeCharacter0xf201',
u'\u1584': 'CanadianSyllabicsQaa',
u'\u5a86': 'CjkUnifiedIdeograph5a86',
u'\u9f88': 'CjkUnifiedIdeograph9f88',
u'\u230b': 'RightFloor',
u'\udc8a': 'UnicodeCharacter0xdc8a',
u'\u600d': 'CjkUnifiedIdeograph600d',
u'\ua50f': 'VaiSyllableThee',
u'\uea11': 'UnicodeCharacter0xea11',
u'\u0d94': 'SinhalaLetterOyanna',
u'\u7296': 'CjkUnifiedIdeograph7296',
u'\ub798': 'HangulSyllableRae',
u'\uf49a': 'UnicodeCharacter0xf49a',
u'\u181d': 'UnicodeCharacter0x181d',
u'\u5d1f': 'CjkUnifiedIdeograph5d1f',
u'\u8221': 'CjkUnifiedIdeograph8221',
u'\uc723': 'HangulSyllableYugs',
u'\u25a4': 'SquareWithHorizontalFill',
u'\u6aa6': 'CjkUnifiedIdeograph6aa6',
u'\uafa8': 'HangulSyllableGgyols',
u'\uecaa': 'UnicodeCharacter0xecaa',
u'\u302d': 'IdeographicEnteringToneMark',
u'\ud1ac': 'HangulSyllableTols',
u'\u752f': 'CjkUnifiedIdeograph752f',
u'\uba31': 'HangulSyllableMyaeng',
u'\uff33': 'FullwidthLatinCapitalLetterS',
u'\u02b6': 'ModifierLetterSmallCapitalInvertedR',
u'\u47b8': 'CjkUnifiedIdeograph47b8',
u'\u84ba': 'CjkUnifiedIdeograph84ba',
u'\u283d': 'BraillePatternDots13456',
u'\uc9bc': 'HangulSyllableJyik',
u'\u6d3f': 'CjkUnifiedIdeograph6d3f',
u'\u5241': 'CjkUnifiedIdeograph5241',
u'\u9743': 'CjkUnifiedIdeograph9743',
u'\ud445': 'HangulSyllablePoenj',
u'\u3ac6': 'CjkUnifiedIdeograph3ac6',
u'\u7fc8': 'CjkUnifiedIdeograph7fc8',
u'\ubcca': 'HangulSyllableByeolp',
u'\ue1cc': 'UnicodeCharacter0xe1cc',
u'\u054f': 'ArmenianCapitalLetterTiwn',
u'\u4a51': 'CjkUnifiedIdeograph4a51',
u'\u8f53': 'CjkUnifiedIdeograph8f53',
u'\ucc55': 'HangulSyllableCaeb',
u'\u17d8': 'KhmerSignBeyyal',
u'\u54da': 'CjkUnifiedIdeograph54da',
u'\u99dc': 'CjkUnifiedIdeograph99dc',
u'\u3d5f': 'CjkUnifiedIdeograph3d5f',
u'\udede': 'UnicodeCharacter0xdede',
u'\u6261': 'CjkUnifiedIdeograph6261',
u'\ua763': 'LatinSmallLetterVisigothicZ',
u'\ue465': 'UnicodeCharacter0xe465',
u'\u0fe8': 'UnicodeCharacter0xfe8',
u'\u4cea': 'CjkUnifiedIdeograph4cea',
u'\ub1ec': 'HangulSyllableNyon',
u'\uf6ee': 'UnicodeCharacter0xf6ee',
u'\u1a71': 'UnicodeCharacter0x1a71',
u'\u5f73': 'CjkUnifiedIdeograph5f73',
u'\u9c75': 'CjkUnifiedIdeograph9c75',
u'\uc177': 'HangulSyllableSyed',
u'\u27f8': 'LongLeftwardsDoubleArrow',
u'\u64fa': 'CjkUnifiedIdeograph64fa',
u'\ua9fc': 'UnicodeCharacter0xa9fc',
u'\ueefe': 'UnicodeCharacter0xeefe',
u'\ud400': 'HangulSyllablePoss',
u'\uf083': 'UnicodeCharacter0xf083',
u'\u1702': 'TagalogLetterU',
u'\u3385': 'SquareKb',
u'\u9d06': 'CjkUnifiedIdeograph9d06',
u'\ub989': 'HangulSyllableReung',
u'\u460c': 'CjkUnifiedIdeograph460c',
u'\u628f': 'CjkUnifiedIdeograph628f',
u'\ucc10': 'HangulSyllableJjin',
u'\ue893': 'UnicodeCharacter0xe893',
u'\u0f12': 'TibetanMarkRgyaGramShad',
u'\u2b95': 'UnicodeCharacter0x2b95',
u'\ub516': 'HangulSyllableDigg',
u'\u5199': 'CjkUnifiedIdeograph5199',
u'\udf9d': 'UnicodeCharacter0xdf9d',
u'\u7e1c': 'CjkUnifiedIdeograph7e1c',
u'\u1a9f': 'UnicodeCharacter0x1a9f',
u'\ue420': 'UnicodeCharacter0xe420',
u'\u80a3': 'CjkUnifiedIdeograph80a3',
u'\u2722': 'FourTeardropspokedAsterisk',
u'\uad26': 'HangulSyllableGwaelp',
u'\u49a9': 'CjkUnifiedIdeograph49a9',
u'\uf7ad': 'UnicodeCharacter0xf7ad',
u'\u162c': 'CanadianSyllabicsCarrierDli',
u'\u32af': 'CircledIdeographAlliance',
u'\u9c30': 'CjkUnifiedIdeograph9c30',
u'\ub8b3': 'HangulSyllableRoegs',
u'\u4536': 'CjkUnifiedIdeograph4536',
u'\u61b9': 'CjkUnifiedIdeograph61b9',
u'\uc33a': 'HangulSyllableSsyalm',
u'\uefbd': 'UnicodeCharacter0xefbd',
u'\u0e3c': 'UnicodeCharacter0xe3c',
u'\u2abf': 'SubsetWithPlusSignBelow',
u'\ub440': 'HangulSyllableDyols',
u'\u50c3': 'CjkUnifiedIdeograph50c3',
u'\udec7': 'UnicodeCharacter0xdec7',
u'\u7d46': 'CjkUnifiedIdeograph7d46',
u'\u19c9': 'NewTaiLueToneMark2',
u'\ufb4a': 'HebrewLetterTavWithDagesh',
u'\u87cd': 'CjkUnifiedIdeograph87cd',
u'\u264c': 'Leo',
u'\uac50': 'HangulSyllableGyak',
u'\u48d3': 'CjkUnifiedIdeograph48d3',
u'\uf6d7': 'UnicodeCharacter0xf6d7',
u'\u1556': 'CanadianSyllabicsFii',
u'\u31d9': 'CjkStrokeSt',
u'\u935a': 'CjkUnifiedIdeograph935a',
u'\ubfdd': 'HangulSyllableBbub',
u'\u4460': 'CjkUnifiedIdeograph4460',
u'\u60e3': 'CjkUnifiedIdeograph60e3',
u'\uc264': 'HangulSyllableSwess',
u'\ueee7': 'UnicodeCharacter0xeee7',
u'\u0d66': 'MalayalamDigitZero',
u'\u29e9': 'DownpointingTriangleWithRightHalfBlack',
u'\u8b6a': 'CjkUnifiedIdeograph8b6a',
u'\u57ed': 'CjkUnifiedIdeograph57ed',
u'\uddf1': 'UnicodeCharacter0xddf1',
u'\u7c70': 'CjkUnifiedIdeograph7c70',
u'\u18f3': 'UnicodeCharacter0x18f3',
u'\ufa74': 'CjkCompatibilityIdeographfa74',
u'\u86f7': 'CjkUnifiedIdeograph86f7',
u'\u2576': 'BoxDrawingsLightRight',
u'\ua37a': 'YiSyllableChyrx',
u'\u4ffd': 'CjkUnifiedIdeograph4ffd',
u'\u7501': 'CjkUnifiedIdeograph7501',
u'\u1480': 'CanadianSyllabicsKwaa',
u'\uf305': 'UnicodeCharacter0xf305',
u'\u9284': 'CjkUnifiedIdeograph9284',
u'\u3e07': 'CjkUnifiedIdeograph3e07',
u'\ua40b': 'YiSyllableQop',
u'\u5b8a': 'CjkUnifiedIdeograph5b8a',
u'\uc18e': 'HangulSyllableSogg',
u'\u6d11': 'CjkUnifiedIdeograph6d11',
u'\u0c90': 'KannadaLetterAi',
u'\ueb15': 'UnicodeCharacter0xeb15',
u'\u8a94': 'CjkUnifiedIdeograph8a94',
u'\u5c1b': 'CjkUnifiedIdeograph5c1b',
u'\u739a': 'CjkUnifiedIdeograph739a',
u'\uda1f': 'UnicodeCharacter0xda1f',
u'\uf99e': 'CjkCompatibilityIdeographf99e',
u'\u0521': 'CyrillicSmallLetterElWithMiddleHook',
u'\u24a0': 'ParenthesizedLatinSmallLetterE',
u'\u8325': 'CjkUnifiedIdeograph8325',
u'\ua2a4': 'YiSyllableZu',
u'\u742b': 'CjkUnifiedIdeograph742b',
u'\u6baa': 'CjkUnifiedIdeograph6baa',
u'\uf22f': 'UnicodeCharacter0xf22f',
u'\u91ae': 'CjkUnifiedIdeograph91ae',
u'\u3d31': 'CjkUnifiedIdeograph3d31',
u'\ubb35': 'HangulSyllableMug',
u'\u5ab4': 'CjkUnifiedIdeograph5ab4',
u'\uc0b8': 'HangulSyllableSals',
u'\u6c3b': 'CjkUnifiedIdeograph6c3b',
u'\u03ba': 'GreekSmallLetterKappa',
u'\uea3f': 'UnicodeCharacter0xea3f',
u'\u89be': 'CjkUnifiedIdeograph89be',
u'\u5345': 'CjkUnifiedIdeograph5345',
u'\u72c4': 'CjkUnifiedIdeograph72c4',
u'\ud949': 'UnicodeCharacter0xd949',
u'\uf8c8': 'UnicodeCharacter0xf8c8',
u'\u044b': 'CyrillicSmallLetterYeru',
u'\u3bca': 'CjkUnifiedIdeograph3bca',
u'\u824f': 'CjkUnifiedIdeograph824f',
u'\ua1ce': 'YiSyllableLut',
u'\u4b55': 'CjkUnifiedIdeograph4b55',
u'\u6ad4': 'CjkUnifiedIdeograph6ad4',
u'\uf159': 'UnicodeCharacter0xf159',
u'\u90d8': 'CjkUnifiedIdeograph90d8',
u'\u3c5b': 'CjkUnifiedIdeograph3c5b',
u'\uba5f': 'HangulSyllableMelb',
u'\u59de': 'CjkUnifiedIdeograph59de',
u'\uc7e2': 'HangulSyllableJyap',
u'\u6365': 'CjkUnifiedIdeograph6365',
u'\u02e4': 'ModifierLetterSmallReversedGlottalStop',
u'\ue969': 'UnicodeCharacter0xe969',
u'\u88e8': 'CjkUnifiedIdeograph88e8',
u'\u526f': 'CjkUnifiedIdeograph526f',
u'\u71ee': 'CjkUnifiedIdeograph71ee',
u'\ud873': 'UnicodeCharacter0xd873',
u'\ufff2': 'UnicodeCharacter0xfff2',
u'\u1b75': 'BalineseMusicalSymbolRighthandOpenDag',
u'\u3af4': 'CjkUnifiedIdeograph3af4',
u'\u8179': 'CjkUnifiedIdeograph8179',
u'\ua0f8': 'YiSyllableVurx',
u'\u4a7f': 'CjkUnifiedIdeograph4a7f',
u'\u69fe': 'CjkUnifiedIdeograph69fe',
u'\u1c06': 'LepchaLetterCa',
u'\u3889': 'CjkUnifiedIdeograph3889',
u'\u5908': 'CjkUnifiedIdeograph5908',
u'\u658b': 'CjkUnifiedIdeograph658b',
u'\u9a0a': 'CjkUnifiedIdeograph9a0a',
u'\ua68d': 'CyrillicSmallLetterTwe',
u'\uc70c': 'HangulSyllableWil',
u'\ue38f': 'UnicodeCharacter0xe38f',
u'\u3416': 'CjkUnifiedIdeograph3416',
u'\u7118': 'CjkUnifiedIdeograph7118',
u'\u1d9b': 'ModifierLetterSmallTurnedAlpha',
u'\ub21a': 'HangulSyllableNuj',
u'\u5e9d': 'CjkUnifiedIdeograph5e9d',
u'\uff1c': 'FullwidthLessthanSign',
u'\u9b9f': 'CjkUnifiedIdeograph9b9f',
u'\uc4a1': 'HangulSyllableSswenj',
u'\u2c26': 'GlagoliticCapitalLetterYo',
u'\u6928': 'CjkUnifiedIdeograph6928',
u'\u35ab': 'CjkUnifiedIdeograph35ab',
u'\uaa2a': 'ChamVowelSignI',
u'\u76ad': 'CjkUnifiedIdeograph76ad',
u'\u972c': 'CjkUnifiedIdeograph972c',
u'\ub3af': 'HangulSyllableDyed',
u'\ud02e': 'HangulSyllableKwej',
u'\ufcb1': 'ArabicLigatureSadWithHahInitialForm',
u'\u0138': 'LatinSmallLetterKra',
u'\u2dbb': 'EthiopicSyllableCchaa',
u'\u423a': 'CjkUnifiedIdeograph423a',
u'\u6ebd': 'CjkUnifiedIdeograph6ebd',
u'\u8f3c': 'CjkUnifiedIdeograph8f3c',
u'\uabbf': 'UnicodeCharacter0xabbf',
u'\uc83e': 'HangulSyllableJyeonh',
u'\u94c1': 'CjkUnifiedIdeograph94c1',
u'\ud1c3': 'HangulSyllableTwad',
u'\u3948': 'CjkUnifiedIdeograph3948',
u'\u7a4a': 'CjkUnifiedIdeograph7a4a',
u'\u06cd': 'ArabicLetterYehWithTail',
u'\ua74c': 'LatinCapitalLetterOWithLoop',
u'\u43cf': 'CjkUnifiedIdeograph43cf',
u'\ue04e': 'UnicodeCharacter0xe04e',
u'\u8cd1': 'CjkUnifiedIdeograph8cd1',
u'\uc9d3': 'HangulSyllableJis',
u'\u125a': 'EthiopicSyllableQhwi',
u'\u3edd': 'CjkUnifiedIdeograph3edd',
u'\u5f5c': 'CjkUnifiedIdeograph5f5c',
u'\u7bdf': 'CjkUnifiedIdeograph7bdf',
u'\u985e': 'CjkUnifiedIdeograph985e',
u'\ua4e1': 'UnicodeCharacter0xa4e1',
u'\uc560': 'HangulSyllableAe',
u'\ue1e3': 'UnicodeCharacter0xe1e3',
u'\u0a6a': 'GurmukhiDigitFour',
u'\u776c': 'CjkUnifiedIdeograph776c',
u'\u13ef': 'CherokeeLetterYa',
u'\ub06e': 'HangulSyllableGgyilp',
u'\u5cf1': 'CjkUnifiedIdeograph5cf1',
u'\ufd70': 'ArabicLigatureDadWithKhahWithMeemInitialForm',
u'\u99f3': 'CjkUnifiedIdeograph99f3',
u'\udaf5': 'UnicodeCharacter0xdaf5',
u'\u227a': 'Precedes',
u'\u6f7c': 'CjkUnifiedIdeograph6f7c',
u'\u0bff': 'UnicodeCharacter0xbff',
u'\ua87e': 'UnicodeCharacter0xa87e',
u'\u9580': 'CjkUnifiedIdeograph9580',
u'\u3103': 'UnicodeCharacter0x3103',
u'\ud682': 'HangulSyllableHwaebs',
u'\u7205': 'CjkUnifiedIdeograph7205',
u'\ubf07': 'HangulSyllableBbeh',
u'\uf809': 'UnicodeCharacter0xf809',
u'\u078c': 'ThaanaLetterThaa',
u'\u408e': 'CjkUnifiedIdeograph408e',
u'\u8d90': 'CjkUnifiedIdeograph8d90',
u'\u2913': 'DownwardsArrowToBar',
u'\uce92': 'HangulSyllableKaegg',
u'\u6a15': 'CjkUnifiedIdeograph6a15',
u'\u5717': 'CjkUnifiedIdeograph5717',
u'\u9019': 'CjkUnifiedIdeograph9019',
u'\udd1b': 'UnicodeCharacter0xdd1b',
u'\u3f9c': 'CjkUnifiedIdeograph3f9c',
u'\u789e': 'CjkUnifiedIdeograph789e',
u'\ua5a0': 'VaiSyllableMgbu',
u'\ue6a2': 'UnicodeCharacter0xe6a2',
u'\u0225': 'LatinSmallLetterZWithHook',
u'\u4f27': 'CjkUnifiedIdeograph4f27',
u'\u8829': 'CjkUnifiedIdeograph8829',
u'\uf52b': 'UnicodeCharacter0xf52b',
u'\u10ae': 'GeorgianCapitalLetterPar',
u'\u5db0': 'CjkUnifiedIdeograph5db0',
u'\u9eb2': 'CjkUnifiedIdeograph9eb2',
u'\u3a35': 'CjkUnifiedIdeograph3a35',
u'\udbb4': 'UnicodeCharacter0xdbb4',
u'\u6737': 'CjkUnifiedIdeograph6737',
u'\ua039': 'YiSyllablePix',
u'\ued3b': 'UnicodeCharacter0xed3b',
u'\u08be': 'UnicodeCharacter0x8be',
u'\u75c0': 'CjkUnifiedIdeograph75c0',
u'\ub6c2': 'HangulSyllableDdweolm',
u'\uf3c4': 'UnicodeCharacter0xf3c4',
u'\u1f47': 'UnicodeCharacter0x1f47',
u'\u5849': 'CjkUnifiedIdeograph5849',
u'\u854b': 'CjkUnifiedIdeograph854b',
u'\uc64d': 'HangulSyllableWalt',
u'\u20ce': 'UnicodeCharacter0x20ce',
u'\u6dd0': 'CjkUnifiedIdeograph6dd0',
u'\uaed2': 'HangulSyllableGgeoj',
u'\uebd4': 'UnicodeCharacter0xebd4',
u'\u3757': 'CjkUnifiedIdeograph3757',
u'\ud4d6': 'HangulSyllablePwilm',
u'\u7059': 'CjkUnifiedIdeograph7059',
u'\ubd5b': 'HangulSyllableBoes',
u'\ufe5d': 'SmallLeftTortoiseShellBracket',
u'\u05e0': 'HebrewLetterNun',
u'\u46e2': 'CjkUnifiedIdeograph46e2',
u'\u83e4': 'CjkUnifiedIdeograph83e4',
u'\u2f67': 'KangxiRadicalSickness',
u'\ucce6': 'HangulSyllableCyeoj',
u'\u6869': 'CjkUnifiedIdeograph6869',
u'\u556b': 'CjkUnifiedIdeograph556b',
u'\u966d': 'CjkUnifiedIdeograph966d',
u'\ud36f': 'HangulSyllablePyaelh',
u'\u3df0': 'CjkUnifiedIdeograph3df0',
u'\u7ef2': 'CjkUnifiedIdeograph7ef2',
u'\ubbf4': 'HangulSyllableMyik',
u'\ue4f6': 'UnicodeCharacter0xe4f6',
u'\u4d7b': 'CjkUnifiedIdeograph4d7b',
u'\u8e7d': 'CjkUnifiedIdeograph8e7d',
u'\ucb7f': 'HangulSyllableJjweoh',
u'\ub781': 'HangulSyllableRanj',
u'\u5004': 'CjkUnifiedIdeograph5004',
u'\u7c87': 'CjkUnifiedIdeograph7c87',
u'\uda08': 'UnicodeCharacter0xda08',
u'\ue68b': 'UnicodeCharacter0xe68b',
u'\u190a': 'LimbuLetterYan',
u'\u258d': 'LeftThreeEighthsBlock',
u'\u830e': 'CjkUnifiedIdeograph830e',
u'\uaf91': 'HangulSyllableGgoeb',
u'\u4814': 'CjkUnifiedIdeograph4814',
u'\u1497': 'CanadianSyllabicsWestcreeCwii',
u'\uf218': 'UnicodeCharacter0xf218',
u'\u9e9b': 'CjkUnifiedIdeograph9e9b',
u'\u311a': 'BopomofoLetterA',
u'\ubb1e': 'HangulSyllableMyonh',
u'\u47a1': 'CjkUnifiedIdeograph47a1',
u'\uc1a5': 'HangulSyllableSot',
u'\u6024': 'CjkUnifiedIdeograph6024',
u'\u0ca7': 'KannadaLetterDha',
u'\uea28': 'UnicodeCharacter0xea28',
u'\ub6ab': 'HangulSyllableDdulh',
u'\u292a': 'SouthWestArrowAndNorthWestArrow',
u'\u532e': 'CjkUnifiedIdeograph532e',
u'\u7fb1': 'CjkUnifiedIdeograph7fb1',
u'\udd32': 'UnicodeCharacter0xdd32',
u'\uf9b5': 'CjkCompatibilityIdeographf9b5',
u'\u1834': 'MongolianLetterCha',
u'\u24b7': 'CircledLatinCapitalLetterB',
u'\u8238': 'CjkUnifiedIdeograph8238',
u'\uaebb': 'HangulSyllableGgyaeh',
u'\u4b3e': 'CjkUnifiedIdeograph4b3e',
u'\u17c1': 'KhmerVowelSignE',
u'\uf542': 'UnicodeCharacter0xf542',
u'\u91c5': 'CjkUnifiedIdeograph91c5',
u'\u3044': 'HiraganaLetterI',
u'\uba48': 'HangulSyllableMeom',
u'\u46cb': 'CjkUnifiedIdeograph46cb',
u'\uc0cf': 'HangulSyllableSaed',
u'\u634e': 'CjkUnifiedIdeograph634e',
u'\u0fd1': 'TibetanMarkMnyamYigGiMgoRgyan',
u'\ued52': 'UnicodeCharacter0xed52',
u'\u89d5': 'CjkUnifiedIdeograph89d5',
u'\u2854': 'BraillePatternDots357',
u'\u5258': 'CjkUnifiedIdeograph5258',
u'\u7edb': 'CjkUnifiedIdeograph7edb',
u'\udc5c': 'UnicodeCharacter0xdc5c',
u'\uf8df': 'UnicodeCharacter0xf8df',
u'\u1b5e': 'BalineseCarikSiki',
u'\u27e1': 'WhiteConcavesidedDiamond',
u'\u8562': 'CjkUnifiedIdeograph8562',
u'\ua1e5': 'YiSyllableGap',
u'\u4a68': 'CjkUnifiedIdeograph4a68',
u'\u16eb': 'RunicSinglePunctuation',
u'\uf46c': 'UnicodeCharacter0xf46c',
u'\u90ef': 'CjkUnifiedIdeograph90ef',
u'\u336e': 'IdeographicTelegraphSymbolForHourTwentytwo',
u'\ubd72': 'HangulSyllableByolp',
u'\u59f5': 'CjkUnifiedIdeograph59f5',
u'\uc3f9': 'HangulSyllableSswanj',
u'\u6278': 'CjkUnifiedIdeograph6278',
u'\u0efb': 'UnicodeCharacter0xefb',
u'\uec7c': 'UnicodeCharacter0xec7c',
u'\u88ff': 'CjkUnifiedIdeograph88ff',
u'\u2b7e': 'UnicodeCharacter0x2b7e',
u'\ub203': 'HangulSyllableNyoh',
u'\u5582': 'CjkUnifiedIdeograph5582',
u'\udf86': 'UnicodeCharacter0xdf86',
u'\u7b09': 'CjkUnifiedIdeograph7b09',
u'\u1a88': 'UnicodeCharacter0x1a88',
u'\ue50d': 'UnicodeCharacter0xe50d',
u'\u848c': 'CjkUnifiedIdeograph848c',
u'\u200f': 'RighttoleftMark',
u'\uaa13': 'ChamLetterTa',
u'\u4d92': 'CjkUnifiedIdeograph4d92',
u'\ud417': 'HangulSyllablePwalh',
u'\uf796': 'UnicodeCharacter0xf796',
u'\u1319': 'EthiopicSyllableGgu',
u'\u3298': 'CircledIdeographLabor',
u'\u9d1d': 'CjkUnifiedIdeograph9d1d',
u'\ubc9c': 'HangulSyllableBeok',
u'\u4223': 'CjkUnifiedIdeograph4223',
u'\u65a2': 'CjkUnifiedIdeograph65a2',
u'\ucc27': 'HangulSyllableJjih',
u'\uefa6': 'UnicodeCharacter0xefa6',
u'\u0b29': 'UnicodeCharacter0xb29',
u'\u2aa8': 'LessthanClosedByCurveAboveSlantedEqual',
u'\ub52d': 'HangulSyllableDit',
u'\u54ac': 'CjkUnifiedIdeograph54ac',
u'\udeb0': 'UnicodeCharacter0xdeb0',
u'\u7a33': 'CjkUnifiedIdeograph7a33',
u'\u1db2': 'ModifierLetterSmallPhi',
u'\ue437': 'UnicodeCharacter0xe437',
u'\u87b6': 'CjkUnifiedIdeograph87b6',
u'\u2339': 'AplFunctionalSymbolQuadDivide',
u'\uad3d': 'HangulSyllableGoelg',
u'\u4cbc': 'CjkUnifiedIdeograph4cbc',
u'\ud741': 'HangulSyllableHyult',
u'\uf6c0': 'UnicodeCharacter0xf6c0',
u'\u1243': 'EthiopicSyllableQaa',
u'\u35c2': 'CjkUnifiedIdeograph35c2',
u'\u9c47': 'CjkUnifiedIdeograph9c47',
u'\ubfc6': 'HangulSyllableBbyoj',
u'\u454d': 'CjkUnifiedIdeograph454d',
u'\u64cc': 'CjkUnifiedIdeograph64cc',
u'\ucf51': 'HangulSyllableKyet',
u'\ueed0': 'UnicodeCharacter0xeed0',
u'\u0a53': 'UnicodeCharacter0xa53',
u'\u2dd2': 'EthiopicSyllableXyi',
u'\ub457': 'HangulSyllableDud',
u'\u57d6': 'CjkUnifiedIdeograph57d6',
u'\ud1da': 'HangulSyllableTwaegg',
u'\u7d5d': 'CjkUnifiedIdeograph7d5d',
u'\u1cdc': 'UnicodeCharacter0x1cdc',
u'\ue761': 'UnicodeCharacter0xe761',
u'\u86e0': 'CjkUnifiedIdeograph86e0',
u'\u2263': 'StrictlyEquivalentTo',
u'\uac67': 'HangulSyllableGyaes',
u'\u4fe6': 'CjkUnifiedIdeograph4fe6',
u'\ud66b': 'HangulSyllableHwac',
u'\uc9ea': 'HangulSyllableJjalp',
u'\u156d': 'CanadianSyllabicsTtho',
u'\u34ec': 'CjkUnifiedIdeograph34ec',
u'\u9f71': 'CjkUnifiedIdeograph9f71',
u'\ubef0': 'HangulSyllableBben',
u'\u4477': 'CjkUnifiedIdeograph4477',
u'\u67f6': 'CjkUnifiedIdeograph67f6',
u'\uce7b': 'HangulSyllableKad',
u'\ue1fa': 'UnicodeCharacter0xe1fa',
u'\u0d7d': 'MalayalamLetterChilluL',
u'\u2cfc': 'CopticOldNubianVerseDivider',
u'\u3681': 'CjkUnifiedIdeograph3681',
u'\u5700': 'CjkUnifiedIdeograph5700',
u'\u7383': 'CjkUnifiedIdeograph7383',
u'\u9402': 'CjkUnifiedIdeograph9402',
u'\ub085': 'HangulSyllableGgilg',
u'\ud104': 'HangulSyllableTyals',
u'\ufd87': 'ArabicLigatureLamWithMeemWithHahFinalForm',
u'\u020e': 'LatinCapitalLetterOWithInvertedBreve',
u'\u2e91': 'CjkRadicalLameFour',
u'\u4f10': 'CjkUnifiedIdeograph4f10',
u'\u6b93': 'CjkUnifiedIdeograph6b93',
u'\u8c12': 'CjkUnifiedIdeograph8c12',
u'\ua895': 'SaurashtraLetterGha',
u'\uc914': 'HangulSyllableJuk',
u'\u9597': 'CjkUnifiedIdeograph9597',
u'\ud299': 'HangulSyllableTwit',
u'\u3a1e': 'CjkUnifiedIdeograph3a1e',
u'\u6720': 'CjkUnifiedIdeograph6720',
u'\u03a3': 'GreekCapitalLetterSigma',
u'\ua422': 'YiSyllableJjuop',
u'\u40a5': 'CjkUnifiedIdeograph40a5',
u'\ue124': 'UnicodeCharacter0xe124',
u'\u8da7': 'CjkUnifiedIdeograph8da7',
u'\ucaa9': 'HangulSyllableJjyelg',
u'\u1f30': 'GreekSmallLetterIotaWithPsili',
u'\u3bb3': 'CjkUnifiedIdeograph3bb3',
u'\u5c32': 'CjkUnifiedIdeograph5c32',
u'\u78b5': 'CjkUnifiedIdeograph78b5',
u'\u9934': 'CjkUnifiedIdeograph9934',
u'\ua5b7': 'VaiSyllableMu',
u'\uc636': 'HangulSyllableObs',
u'\ue2b9': 'UnicodeCharacter0xe2b9',
u'\u3740': 'CjkUnifiedIdeograph3740',
u'\u7442': 'CjkUnifiedIdeograph7442',
u'\u10c5': 'GeorgianCapitalLetterHoe',
u'\ub144': 'HangulSyllableNyeon',
u'\u5dc7': 'CjkUnifiedIdeograph5dc7',
u'\ufe46': 'WhiteSesameDot',
u'\u9ac9': 'CjkUnifiedIdeograph9ac9',
u'\uc7cb': 'HangulSyllableJyags',
u'\u2f50': 'KangxiRadicalCompare',
u'\u6c52': 'CjkUnifiedIdeograph6c52',
u'\u08d5': 'UnicodeCharacter0x8d5',
u'\ua954': 'UnicodeCharacter0xa954',
u'\u75d7': 'CjkUnifiedIdeograph75d7',
u'\u9656': 'CjkUnifiedIdeograph9656',
u'\ub2d9': 'HangulSyllableNib',
u'\ud358': 'HangulSyllablePyass',
u'\uffdb': 'HalfwidthHangulLetterYi',
u'\u0462': 'CyrillicCapitalLetterYat',
u'\u20e5': 'CombiningReverseSolidusOverlay',
u'\u4164': 'CjkUnifiedIdeograph4164',
u'\u6de7': 'CjkUnifiedIdeograph6de7',
u'\u8e66': 'CjkUnifiedIdeograph8e66',
u'\uaae9': 'UnicodeCharacter0xaae9',
u'\ucb68': 'HangulSyllableJjweon',
u'\u97eb': 'CjkUnifiedIdeograph97eb',
u'\ud4ed': 'HangulSyllablePyunj',
u'\u3c72': 'CjkUnifiedIdeograph3c72',
u'\u7974': 'CjkUnifiedIdeograph7974',
u'\u05f7': 'UnicodeCharacter0x5f7',
u'\ua676': 'UnicodeCharacter0xa676',
u'\u42f9': 'CjkUnifiedIdeograph42f9',
u'\ue378': 'UnicodeCharacter0xe378',
u'\u8ffb': 'CjkUnifiedIdeograph8ffb',
u'\uccfd': 'HangulSyllableCyeb',
u'\uf601': 'UnicodeCharacter0xf601',
u'\u1184': 'HangulJungseongYoya',
u'\u5e86': 'CjkUnifiedIdeograph5e86',
u'\u9b88': 'CjkUnifiedIdeograph9b88',
u'\u270b': 'UnicodeCharacter0x270b',
u'\ud88a': 'UnicodeCharacter0xd88a',
u'\u640d': 'CjkUnifiedIdeograph640d',
u'\ua10f': 'YiSyllableDo',
u'\uee11': 'UnicodeCharacter0xee11',
u'\u0994': 'BengaliLetterAu',
u'\u7696': 'CjkUnifiedIdeograph7696',
u'\ub398': 'HangulSyllableDyeols',
u'\uf09a': 'UnicodeCharacter0xf09a',
u'\u1c1d': 'LepchaLetterHa',
u'\u591f': 'CjkUnifiedIdeograph591f',
u'\u8621': 'CjkUnifiedIdeograph8621',
u'\uc323': 'HangulSyllableSsaelh',
u'\u21a4': 'LeftwardsArrowFromBar',
u'\u6ea6': 'CjkUnifiedIdeograph6ea6',
u'\uaba8': 'UnicodeCharacter0xaba8',
u'\ue8aa': 'UnicodeCharacter0xe8aa',
u'\u342d': 'CjkUnifiedIdeograph342d',
u'\ud5ac': 'HangulSyllableHyae',
u'\u712f': 'CjkUnifiedIdeograph712f',
u'\ube31': 'HangulSyllableByilg',
u'\ufb33': 'HebrewLetterDaletWithDagesh',
u'\u06b6': 'ArabicLetterLamWithDotAbove',
u'\u43b8': 'CjkUnifiedIdeograph43b8',
u'\u80ba': 'CjkUnifiedIdeograph80ba',
u'\u2c3d': 'GlagoliticSmallLetterKako',
u'\ucdbc': 'HangulSyllableCweols',
u'\u693f': 'CjkUnifiedIdeograph693f',
u'\u5641': 'CjkUnifiedIdeograph5641',
u'\u9343': 'CjkUnifiedIdeograph9343',
u'\ud045': 'HangulSyllableKwib',
u'\u3ec6': 'CjkUnifiedIdeograph3ec6',
u'\u7bc8': 'CjkUnifiedIdeograph7bc8',
u'\ub8ca': 'HangulSyllableRoep',
u'\ue5cc': 'UnicodeCharacter0xe5cc',
u'\u014f': 'LatinSmallLetterOWithBreve',
u'\u4e51': 'CjkUnifiedIdeograph4e51',
u'\u8b53': 'CjkUnifiedIdeograph8b53',
u'\uc855': 'HangulSyllableJyeg',
u'\u13d8': 'CherokeeLetterTi',
u'\u50da': 'CjkUnifiedIdeograph50da',
u'\u9ddc': 'CjkUnifiedIdeograph9ddc',
u'\u395f': 'CjkUnifiedIdeograph395f',
u'\udade': 'UnicodeCharacter0xdade',
u'\u6661': 'CjkUnifiedIdeograph6661',
u'\ua363': 'YiSyllableCha',
u'\ue065': 'UnicodeCharacter0xe065',
u'\u0be8': 'TamilDigitTwo',
u'\u48ea': 'CjkUnifiedIdeograph48ea',
u'\ub5ec': 'HangulSyllableDdyeoss',
u'\uf2ee': 'UnicodeCharacter0xf2ee',
u'\u1e71': 'LatinSmallLetterTWithCircumflexBelow',
u'\u5b73': 'CjkUnifiedIdeograph5b73',
u'\u9875': 'CjkUnifiedIdeograph9875',
u'\uc577': 'HangulSyllableAec',
u'\u23f8': 'UnicodeCharacter0x23f8',
u'\u60fa': 'CjkUnifiedIdeograph60fa',
u'\uadfc': 'HangulSyllableGeun',
u'\ueafe': 'UnicodeCharacter0xeafe',
u'\ud800': 'UnicodeCharacter0xd800',
u'\ufc83': 'ArabicLigatureKafWithAlefMaksuraFinalForm',
u'\u1b02': 'BalineseSignCecek',
u'\u3f85': 'CjkUnifiedIdeograph3f85',
u'\u9106': 'CjkUnifiedIdeograph9106',
u'\ub589': 'HangulSyllableDdyaenj',
u'\u4a0c': 'CjkUnifiedIdeograph4a0c',
u'\u6e8f': 'CjkUnifiedIdeograph6e8f',
u'\uc010': 'HangulSyllableBbwels',
u'\ue493': 'UnicodeCharacter0xe493',
u'\u0312': 'CombiningTurnedCommaAbove',
u'\u2795': 'UnicodeCharacter0x2795',
u'\ub916': 'HangulSyllableRweobs',
u'\u5d99': 'CjkUnifiedIdeograph5d99',
u'\ud39d': 'HangulSyllablePenj',
u'\u721c': 'CjkUnifiedIdeograph721c',
u'\u169f': 'UnicodeCharacter0x169f',
u'\ue820': 'UnicodeCharacter0xe820',
u'\u8ca3': 'CjkUnifiedIdeograph8ca3',
u'\u2b22': 'BlackHexagon',
u'\ua126': 'YiSyllableTuox',
u'\u45a9': 'CjkUnifiedIdeograph45a9',
u'\ufbad': 'ArabicLetterHehDoachashmeeMedialForm',
u'\u1a2c': 'UnicodeCharacter0x1a2c',
u'\u3eaf': 'CjkUnifiedIdeograph3eaf',
u'\u9030': 'CjkUnifiedIdeograph9030',
u'\ub4b3': 'HangulSyllableDwilh',
u'\u4936': 'CjkUnifiedIdeograph4936',
u'\u6db9': 'CjkUnifiedIdeograph6db9',
u'\ucf3a': 'HangulSyllableKyegg',
u'\ue3bd': 'UnicodeCharacter0xe3bd',
u'\u023c': 'LatinSmallLetterCWithStroke',
u'\u26bf': 'UnicodeCharacter0x26bf',
u'\ub840': 'HangulSyllableRye',
u'\u5cc3': 'CjkUnifiedIdeograph5cc3',
u'\ud2c7': 'HangulSyllableTeulh',
u'\u7146': 'CjkUnifiedIdeograph7146',
u'\u15c9': 'CanadianSyllabicsCarrierGha',
u'\uf74a': 'UnicodeCharacter0xf74a',
u'\u8bcd': 'CjkUnifiedIdeograph8bcd',
u'\u2a4c': 'ClosedUnionWithSerifs',
u'\ua050': 'YiSyllablePyt',
u'\u44d3': 'CjkUnifiedIdeograph44d3',
u'\ufad7': 'CjkCompatibilityIdeographfad7',
u'\u1956': 'TaiLeLetterTa',
u'\u3dd9': 'CjkUnifiedIdeograph3dd9',
u'\u9f5a': 'CjkUnifiedIdeograph9f5a',
u'\ub3dd': 'HangulSyllableDot',
u'\u4860': 'CjkUnifiedIdeograph4860',
u'\u6ce3': 'CjkUnifiedIdeograph6ce3',
u'\uce64': 'HangulSyllableCils',
u'\ue2e7': 'UnicodeCharacter0xe2e7',
u'\u0166': 'LatinCapitalLetterTWithStroke',
u'\u25e9': 'SquareWithUpperLeftDiagonalHalfBlack',
u'\u876a': 'CjkUnifiedIdeograph876a',
u'\u5bed': 'CjkUnifiedIdeograph5bed',
u'\ud1f1': 'HangulSyllableTwaet',
u'\u7070': 'CjkUnifiedIdeograph7070',
u'\u14f3': 'CanadianSyllabicsYcreeSoo',
u'\uf674': 'UnicodeCharacter0xf674',
u'\u8af7': 'CjkUnifiedIdeograph8af7',
u'\u2976': 'LessthanAboveLeftwardsArrow',
u'\uaf7a': 'HangulSyllableGgwaej',
u'\u43fd': 'CjkUnifiedIdeograph43fd',
u'\u7901': 'CjkUnifiedIdeograph7901',
u'\u1880': 'MongolianLetterAliGaliAnusvaraOne',
u'\uff05': 'FullwidthPercentSign',
u'\u9e84': 'CjkUnifiedIdeograph9e84',
u'\u3207': 'ParenthesizedHangulIeung',
u'\ua80b': 'SylotiNagriSignAnusvara',
u'\u578a': 'CjkUnifiedIdeograph578a',
u'\ucd8e': 'HangulSyllableCyoj',
u'\u6111': 'CjkUnifiedIdeograph6111',
u'\x90': 'UnicodeCharacter0x90',
u'\ue715': 'UnicodeCharacter0xe715',
u'\u8694': 'CjkUnifiedIdeograph8694',
u'\u501b': 'CjkUnifiedIdeograph501b',
u'\u7f9a': 'CjkUnifiedIdeograph7f9a',
u'\ud61f': 'HangulSyllableHyegs',
u'\uf59e': 'UnicodeCharacter0xf59e',
u'\u0921': 'DevanagariLetterDda',
u'\u28a0': 'BraillePatternDots68',
u'\u8f25': 'CjkUnifiedIdeograph8f25',
u'\uaea4': 'HangulSyllableGgyaen',
u'\u782b': 'CjkUnifiedIdeograph782b',
u'\u67aa': 'CjkUnifiedIdeograph67aa',
u'\ufe2f': 'UnicodeCharacter0xfe2f',
u'\u9dae': 'CjkUnifiedIdeograph9dae',
u'\u3131': 'HangulLetterKiyeok',
u'\ub735': 'HangulSyllableDdeult',
u'\u56b4': 'CjkUnifiedIdeograph56b4',
u'\uccb8': 'HangulSyllableCen',
u'\u603b': 'CjkUnifiedIdeograph603b',
u'\u0fba': 'TibetanSubjoinedLetterFixedformWa',
u'\ue63f': 'UnicodeCharacter0xe63f',
u'\u85be': 'CjkUnifiedIdeograph85be',
u'\u5f45': 'CjkUnifiedIdeograph5f45',
u'\u7ec4': 'CjkUnifiedIdeograph7ec4',
u'\ud549': 'HangulSyllablePilt',
u'\uf4c8': 'UnicodeCharacter0xf4c8',
u'\u084b': 'UnicodeCharacter0x84b',
u'\u37ca': 'CjkUnifiedIdeograph37ca',
u'\u8e4f': 'CjkUnifiedIdeograph8e4f',
u'\uadce': 'HangulSyllableGwilp',
u'\u4755': 'CjkUnifiedIdeograph4755',
u'\u66d4': 'CjkUnifiedIdeograph66d4',
u'\ufd59': 'ArabicLigatureJeemWithMeemWithHahInitialForm',
u'\u9cd8': 'CjkUnifiedIdeograph9cd8',
u'\u305b': 'HiraganaLetterSe',
u'\ub65f': 'HangulSyllableDdwaec',
u'\u55de': 'CjkUnifiedIdeograph55de',
u'\ucbe2': 'HangulSyllableJjeulp',
u'\u6f65': 'CjkUnifiedIdeograph6f65',
u'\u0ee4': 'UnicodeCharacter0xee4',
u'\ue569': 'UnicodeCharacter0xe569',
u'\u84e8': 'CjkUnifiedIdeograph84e8',
u'\u5e6f': 'CjkUnifiedIdeograph5e6f',
u'\u7dee': 'CjkUnifiedIdeograph7dee',
u'\ud473': 'HangulSyllablePyoc',
u'\uf3f2': 'UnicodeCharacter0xf3f2',
u'\u1775': 'UnicodeCharacter0x1775',
u'\u36f4': 'CjkUnifiedIdeograph36f4',
u'\u8d79': 'CjkUnifiedIdeograph8d79',
u'\uacf8': 'HangulSyllableGok',
u'\u467f': 'CjkUnifiedIdeograph467f',
u'\u65fe': 'CjkUnifiedIdeograph65fe',
u'\u1006': 'MyanmarLetterCha',
u'\u3489': 'CjkUnifiedIdeograph3489',
u'\u5508': 'CjkUnifiedIdeograph5508',
u'\u698b': 'CjkUnifiedIdeograph698b',
u'\u960a': 'CjkUnifiedIdeograph960a',
u'\uaa8d': 'UnicodeCharacter0xaa8d',
u'\ucb0c': 'HangulSyllableJjwaek',
u'\uef8f': 'UnicodeCharacter0xef8f',
u'\u3816': 'CjkUnifiedIdeograph3816',
u'\u7d18': 'CjkUnifiedIdeograph7d18',
u'\u119b': 'HangulJungseongIu',
u'\ube1a': 'HangulSyllableBeulp',
u'\u529d': 'CjkUnifiedIdeograph529d',
u'\uf31c': 'UnicodeCharacter0xf31c',
u'\u979f': 'CjkUnifiedIdeograph979f',
u'\uc8a1': 'HangulSyllableJwang',
u'\u2026': 'HorizontalEllipsis',
u'\u6528': 'CjkUnifiedIdeograph6528',
u'\u39ab': 'CjkUnifiedIdeograph39ab',
u'\ua62a': 'VaiSyllableNdoleMa',
u'\u7aad': 'CjkUnifiedIdeograph7aad',
u'\u9b2c': 'CjkUnifiedIdeograph9b2c',
u'\ubfaf': 'HangulSyllableBboeh',
u'\udc2e': 'UnicodeCharacter0xdc2e',
u'\uf0b1': 'UnicodeCharacter0xf0b1',
u'\u0d38': 'MalayalamLetterSa',
u'\u21bb': 'ClockwiseOpenCircleArrow',
u'\u4e3a': 'CjkUnifiedIdeograph4e3a',
u'\u62bd': 'CjkUnifiedIdeograph62bd',
u'\u833c': 'CjkUnifiedIdeograph833c',
u'\ua7bf': 'UnicodeCharacter0xa7bf',
u'\uc43e': 'HangulSyllableSsoebs',
u'\u98c1': 'CjkUnifiedIdeograph98c1',
u'\uddc3': 'UnicodeCharacter0xddc3',
u'\u3548': 'CjkUnifiedIdeograph3548',
u'\u764a': 'CjkUnifiedIdeograph764a',
u'\u0acd': 'GujaratiSignVirama',
u'\uab4c': 'UnicodeCharacter0xab4c',
u'\u4fcf': 'CjkUnifiedIdeograph4fcf',
u'\uec4e': 'UnicodeCharacter0xec4e',
u'\u80d1': 'CjkUnifiedIdeograph80d1',
u'\uc5d3': 'HangulSyllableEgs',
u'\u1e5a': 'LatinCapitalLetterRWithDotBelow',
u'\u32dd': 'CircledKatakanaSe',
u'\u535c': 'CjkUnifiedIdeograph535c',
u'\u77df': 'CjkUnifiedIdeograph77df',
u'\u945e': 'CjkUnifiedIdeograph945e',
u'\ua8e1': 'UnicodeCharacter0xa8e1',
u'\uc960': 'HangulSyllableJwim',
u'\uede3': 'UnicodeCharacter0xede3',
u'\u066a': 'ArabicPercentSign',
u'\u7b6c': 'CjkUnifiedIdeograph7b6c',
u'\u1fef': 'GreekVaria',
u'\ubc6e': 'HangulSyllableByaenh',
u'\u50f1': 'CjkUnifiedIdeograph50f1',
u'\uf170': 'UnicodeCharacter0xf170',
u'\u95f3': 'CjkUnifiedIdeograph95f3',
u'\ud6f5': 'HangulSyllableHweong',
u'\u2e7a': 'UnicodeCharacter0x2e7a',
u'\u637c': 'CjkUnifiedIdeograph637c',
u'\u07ff': 'UnicodeCharacter0x7ff',
u'\ua47e': 'YiSyllableYox',
u'\u9980': 'CjkUnifiedIdeograph9980',
u'\u3d03': 'CjkUnifiedIdeograph3d03',
u'\uda82': 'UnicodeCharacter0xda82',
u'\u7e05': 'CjkUnifiedIdeograph7e05',
u'\ub307': 'HangulSyllableDaed',
u'\uf409': 'UnicodeCharacter0xf409',
u'\u0b8c': 'UnicodeCharacter0xb8c',
u'\u4c8e': 'CjkUnifiedIdeograph4c8e',
u'\u8190': 'CjkUnifiedIdeograph8190',
u'\u2513': 'BoxDrawingsHeavyDownAndLeft',
u'\uc292': 'HangulSyllableSyulm',
u'\u6615': 'CjkUnifiedIdeograph6615',
u'\u5b17': 'CjkUnifiedIdeograph5b17',
u'\u9c19': 'CjkUnifiedIdeograph9c19',
u'\ud11b': 'HangulSyllableTyaed',
u'\u339c': 'SquareMm',
u'\u749e': 'CjkUnifiedIdeograph749e',
u'\ua9a0': 'UnicodeCharacter0xa9a0',
u'\ueaa2': 'UnicodeCharacter0xeaa2',
u'\u0e25': 'ThaiCharacterLoLing',
u'\u4327': 'CjkUnifiedIdeograph4327',
u'\u8429': 'CjkUnifiedIdeograph8429',
u'\uf92b': 'CjkCompatibilityIdeographf92b',
u'\u1cae': 'UnicodeCharacter0x1cae',
u'\u51b0': 'CjkUnifiedIdeograph51b0',
u'\u92b2': 'CjkUnifiedIdeograph92b2',
u'\u3635': 'CjkUnifiedIdeograph3635',
u'\ud7b4': 'UnicodeCharacter0xd7b4',
u'\u6b37': 'CjkUnifiedIdeograph6b37',
u'\uac39': 'HangulSyllableGyag',
u'\ue13b': 'UnicodeCharacter0xe13b',
u'\u04be': 'CyrillicCapitalLetterAbkhasianCheWithDescender',
u'\u79c0': 'CjkUnifiedIdeograph79c0',
u'\ubac2': 'HangulSyllableMop',
u'\uffc4': 'HalfwidthHangulLetterYa',
u'\u1347': 'EthiopicSyllableTzoa',
u'\u5449': 'CjkUnifiedIdeograph5449',
u'\u894b': 'CjkUnifiedIdeograph894b',
u'\uca4d': 'HangulSyllableJjeog',
u'\u2cce': 'CopticCapitalLetterOldCopticHa',
u'\u61d0': 'CjkUnifiedIdeograph61d0',
u'\ua2d2': 'YiSyllableZzi',
u'\ue7d4': 'UnicodeCharacter0xe7d4',
u'\u3b57': 'CjkUnifiedIdeograph3b57',
u'\ud8d6': 'UnicodeCharacter0xd8d6',
u'\u7c59': 'CjkUnifiedIdeograph7c59',
u'\ub15b': 'HangulSyllableNyeoh',
u'\uf25d': 'UnicodeCharacter0xf25d',
u'\u09e0': 'BengaliLetterVocalicRr',
u'\u4ae2': 'CjkUnifiedIdeograph4ae2',
u'\u8fe4': 'CjkUnifiedIdeograph8fe4',
u'\u2367': 'AplFunctionalSymbolLeftShoeStile',
u'\uc0e6': 'HangulSyllableSyagg',
u'\u6469': 'CjkUnifiedIdeograph6469',
u'\u596b': 'CjkUnifiedIdeograph596b',
u'\u9a6d': 'CjkUnifiedIdeograph9a6d',
u'\udf6f': 'UnicodeCharacter0xdf6f',
u'\u31f0': 'KatakanaLetterSmallKu',
u'\u72f2': 'CjkUnifiedIdeograph72f2',
u'\ub7f4': 'HangulSyllableReol',
u'\ue8f6': 'UnicodeCharacter0xe8f6',
u'\u0c79': 'TeluguFractionDigitOneForOddPowersOfFour',
u'\u417b': 'CjkUnifiedIdeograph417b',
u'\u827d': 'CjkUnifiedIdeograph827d',
u'\uc77f': 'HangulSyllableIlb',
u'\ubb81': 'HangulSyllableMweng',
u'\u5c04': 'CjkUnifiedIdeograph5c04',
u'\u7087': 'CjkUnifiedIdeograph7087',
u'\ud608': 'HangulSyllableHyeol',
u'\uea8b': 'UnicodeCharacter0xea8b',
u'\u150a': 'CanadianSyllabicsNaskapiSkw',
u'\u298d': 'LeftSquareBracketWithTickInTopCorner',
u'\u8f0e': 'CjkUnifiedIdeograph8f0e',
u'\ua391': 'YiSyllableRryp',
u'\u4414': 'CjkUnifiedIdeograph4414',
u'\u1897': 'MongolianLetterAliGaliAh',
u'\ufe18': 'PresentationFormForVerticalRightWhiteLenticularBrakcet',
u'\u929b': 'CjkUnifiedIdeograph929b',
u'\u3d1a': 'CjkUnifiedIdeograph3d1a',
u'\ub71e': 'HangulSyllableDdyubs',
u'\u4ba1': 'CjkUnifiedIdeograph4ba1',
u'\ucda5': 'HangulSyllableCub',
u'\u6c24': 'CjkUnifiedIdeograph6c24',
u'\xa7': 'SectionSign',
u'\ue628': 'UnicodeCharacter0xe628',
u'\ubaab': 'HangulSyllableMogs',
u'\u252a': 'BoxDrawingsUpLightAndLeftDownHeavy',
u'\u5f2e': 'CjkUnifiedIdeograph5f2e',
u'\u73b1': 'CjkUnifiedIdeograph73b1',
u'\ud132': 'HangulSyllableTeogg',
u'\uf5b5': 'UnicodeCharacter0xf5b5',
u'\u1434': 'CanadianSyllabicsPoo',
u'\u28b7': 'BraillePatternDots123568',
u'\u8e38': 'CjkUnifiedIdeograph8e38',
u'\ua2bb': 'YiSyllableCuo',
u'\u473e': 'CjkUnifiedIdeograph473e',
u'\u1bc1': 'UnicodeCharacter0x1bc1',
u'\uf942': 'CjkCompatibilityIdeographf942',
u'\u9dc5': 'CjkUnifiedIdeograph9dc5',
u'\u3c44': 'CjkUnifiedIdeograph3c44',
u'\ub648': 'HangulSyllableDdwae',
u'\u4acb': 'CjkUnifiedIdeograph4acb',
u'\ucccf': 'HangulSyllableCeh',
u'\u6f4e': 'CjkUnifiedIdeograph6f4e',
u'\u03d1': 'GreekThetaSymbol',
u'\ue152': 'UnicodeCharacter0xe152',
u'\u85d5': 'CjkUnifiedIdeograph85d5',
u'\u2454': 'UnicodeCharacter0x2454',
u'\u5e58': 'CjkUnifiedIdeograph5e58',
u'\u72db': 'CjkUnifiedIdeograph72db',
u'\ud05c': 'HangulSyllableKyuls',
u'\uf4df': 'UnicodeCharacter0xf4df',
u'\u175e': 'UnicodeCharacter0x175e',
u'\u2be1': 'UnicodeCharacter0x2be1',
u'\u8962': 'CjkUnifiedIdeograph8962',
u'\uade5': 'HangulSyllableGyulg',
u'\u4668': 'CjkUnifiedIdeograph4668',
u'\u1aeb': 'UnicodeCharacter0x1aeb',
u'\uf86c': 'UnicodeCharacter0xf86c',
u'\u9cef': 'CjkUnifiedIdeograph9cef',
u'\u3f6e': 'CjkUnifiedIdeograph3f6e',
u'\ub172': 'HangulSyllableNyej',
u'\u55f5': 'CjkUnifiedIdeograph55f5',
u'\ucff9': 'HangulSyllableKut',
u'\u6e78': 'CjkUnifiedIdeograph6e78',
u'\u02fb': 'ModifierLetterBeginLowTone',
u'\ue07c': 'UnicodeCharacter0xe07c',
u'\u84ff': 'CjkUnifiedIdeograph84ff',
u'\u277e': 'DingbatNegativeCircledDigitNine',
u'\ube03': 'HangulSyllableByus',
u'\u5982': 'CjkUnifiedIdeograph5982',
u'\ud386': 'HangulSyllablePeolm',
u'\u7709': 'CjkUnifiedIdeograph7709',
u'\u1688': 'OghamLetterTinne',
u'\ue90d': 'UnicodeCharacter0xe90d',
u'\u888c': 'CjkUnifiedIdeograph888c',
u'\u2c0f': 'GlagoliticCapitalLetterMyslite',
u'\ua613': 'VaiSymbolFeeng',
u'\u4192': 'CjkUnifiedIdeograph4192',
u'\ud817': 'UnicodeCharacter0xd817',
u'\ufb96': 'ArabicLetterGuehIsolatedForm',
u'\u1f19': 'GreekCapitalLetterEpsilonWithDasia',
u'\u3e98': 'CjkUnifiedIdeograph3e98',
u'\u911d': 'CjkUnifiedIdeograph911d',
u'\ub09c': 'HangulSyllableNan',
u'\u4e23': 'CjkUnifiedIdeograph4e23',
u'\u69a2': 'CjkUnifiedIdeograph69a2',
u'\uc027': 'HangulSyllableBbwid',
u'\ue3a6': 'UnicodeCharacter0xe3a6',
u'\u0729': 'SyriacLetterQaph',
u'\u26a8': 'VerticalMaleWithStrokeSign',
u'\ub92d': 'HangulSyllableRwelt',
u'\u58ac': 'CjkUnifiedIdeograph58ac',
u'\ud2b0': 'HangulSyllableTyuss',
u'\u7633': 'CjkUnifiedIdeograph7633',
u'\u11b2': 'HangulJongseongRieulpieup',
u'\ue837': 'UnicodeCharacter0xe837',
u'\u8bb6': 'CjkUnifiedIdeograph8bb6',
u'\u2f39': 'KangxiRadicalSnout',
u'\ua13d': 'YiSyllableDdat',
u'\u40bc': 'CjkUnifiedIdeograph40bc',
u'\udb41': 'UnicodeCharacter0xdb41',
u'\ufac0': 'CjkCompatibilityIdeographfac0',
u'\u1e43': 'LatinSmallLetterMWithDotBelow',
u'\u39c2': 'CjkUnifiedIdeograph39c2',
u'\u9047': 'CjkUnifiedIdeograph9047',
u'\ub3c6': 'HangulSyllableDogg',
u'\u494d': 'CjkUnifiedIdeograph494d',
u'\u68cc': 'CjkUnifiedIdeograph68cc',
u'\uc351': 'HangulSyllableSsyaenj',
u'\ue2d0': 'UnicodeCharacter0xe2d0',
u'\u0653': 'ArabicMaddahAbove',
u'\u21d2': 'RightwardsDoubleArrow',
u'\ub857': 'HangulSyllableRyec',
u'\u5bd6': 'CjkUnifiedIdeograph5bd6',
u'\uddda': 'UnicodeCharacter0xddda',
u'\u715d': 'CjkUnifiedIdeograph715d',
u'\u10dc': 'GeorgianLetterNar',
u'\ueb61': 'UnicodeCharacter0xeb61',
u'\u8ae0': 'CjkUnifiedIdeograph8ae0',
u'\u2e63': 'UnicodeCharacter0x2e63',
u'\ua067': 'YiSyllableBbo',
u'\u43e6': 'CjkUnifiedIdeograph43e6',
u'\uda6b': 'UnicodeCharacter0xda6b',
u'\uc5ea': 'HangulSyllableEp',
u'\u196d': 'TaiLeLetterAi',
u'\u38ec': 'CjkUnifiedIdeograph38ec',
u'\u9371': 'CjkUnifiedIdeograph9371',
u'\ub2f0': 'HangulSyllableDals',
u'\u4877': 'CjkUnifiedIdeograph4877',
u'\u6bf6': 'CjkUnifiedIdeograph6bf6',
u'\uc27b': 'HangulSyllableSwilh',
u'\uedfa': 'UnicodeCharacter0xedfa',
u'\u017d': 'LatinCapitalLetterZWithCaron',
u'\u20fc': 'UnicodeCharacter0x20fc',
u'\u3a81': 'CjkUnifiedIdeograph3a81',
u'\u5b00': 'CjkUnifiedIdeograph5b00',
u'\u7f83': 'CjkUnifiedIdeograph7f83',
u'\u9802': 'CjkUnifiedIdeograph9802',
u'\ubc85': 'HangulSyllableBeog',
u'\udd04': 'UnicodeCharacter0xdd04',
u'\uf187': 'UnicodeCharacter0xf187',
u'\u0e0e': 'ThaiCharacterDoChada',
u'\u2291': 'SquareImageOfOrEqualTo',
u'\u4310': 'CjkUnifiedIdeograph4310',
u'\u6793': 'CjkUnifiedIdeograph6793',
u'\u8012': 'CjkUnifiedIdeograph8012',
u'\ua495': 'YiRadicalSsi',
u'\uc514': 'HangulSyllableSsyil',
u'\u9997': 'CjkUnifiedIdeograph9997',
u'\ude99': 'UnicodeCharacter0xde99',
u'\u361e': 'CjkUnifiedIdeograph361e',
u'\u6b20': 'CjkUnifiedIdeograph6b20',
u'\u0fa3': 'TibetanSubjoinedLetterNa',
u'\ua822': 'SylotiNagriLetterHo',
u'\u4ca5': 'CjkUnifiedIdeograph4ca5',
u'\ued24': 'UnicodeCharacter0xed24',
u'\u81a7': 'CjkUnifiedIdeograph81a7',
u'\uc6a9': 'HangulSyllableYong',
u'\u1330': 'EthiopicSyllablePha',
u'\u37b3': 'CjkUnifiedIdeograph37b3',
u'\u5032': 'CjkUnifiedIdeograph5032',
u'\u74b5': 'CjkUnifiedIdeograph74b5',
u'\u9534': 'CjkUnifiedIdeograph9534',
u'\ua9b7': 'UnicodeCharacter0xa9b7',
u'\uca36': 'HangulSyllableJjyaenh',
u'\ueeb9': 'UnicodeCharacter0xeeb9',
u'\u3b40': 'CjkUnifiedIdeograph3b40',
u'\u7842': 'CjkUnifiedIdeograph7842',
u'\u1cc5': 'UnicodeCharacter0x1cc5',
u'\ubd44': 'HangulSyllableBwaek',
u'\u51c7': 'CjkUnifiedIdeograph51c7',
u'\uf246': 'UnicodeCharacter0xf246',
u'\u96c9': 'CjkUnifiedIdeograph96c9',
u'\ucbcb': 'HangulSyllableJjyus',
u'\u2350': 'AplFunctionalSymbolQuadUpwardsArrow',
u'\u6052': 'CjkUnifiedIdeograph6052',
u'\u04d5': 'CyrillicSmallLigatureAIe',
u'\ua554': 'VaiSyllableKpa',
u'\u79d7': 'CjkUnifiedIdeograph79d7',
u'\u9a56': 'CjkUnifiedIdeograph9a56',
u'\ubed9': 'HangulSyllableBbeolg',
u'\udf58': 'UnicodeCharacter0xdf58',
u'\uf3db': 'UnicodeCharacter0xf3db',
u'\u0862': 'UnicodeCharacter0x862',
u'\u2ce5': 'CopticSymbolMiRo',
u'\u4d64': 'CjkUnifiedIdeograph4d64',
u'\u61e7': 'CjkUnifiedIdeograph61e7',
u'\u8266': 'CjkUnifiedIdeograph8266',
u'\ua6e9': 'UnicodeCharacter0xa6e9',
u'\uc768': 'HangulSyllableYim',
u'\u9beb': 'CjkUnifiedIdeograph9beb',
u'\ud8ed': 'UnicodeCharacter0xd8ed',
u'\u3072': 'HiraganaLetterHi',
u'\u7574': 'CjkUnifiedIdeograph7574',
u'\u09f7': 'BengaliCurrencyNumeratorFour',
u'\uaa76': 'UnicodeCharacter0xaa76',
u'\u4ef9': 'CjkUnifiedIdeograph4ef9',
u'\uef78': 'UnicodeCharacter0xef78',
u'\u83fb': 'CjkUnifiedIdeograph83fb',
u'\uc0fd': 'HangulSyllableSyat',
u'\ufa01': 'CjkCompatibilityIdeographfa01',
u'\u1d84': 'LatinSmallLetterKWithPalatalHook',
u'\u5286': 'CjkUnifiedIdeograph5286',
u'\u9788': 'CjkUnifiedIdeograph9788',
u'\u2b0b': 'SouthWestBlackArrow',
u'\ud48a': 'HangulSyllablePubs',
u'\u680d': 'CjkUnifiedIdeograph680d',
u'\uad0f': 'HangulSyllableGwas',
u'\ue211': 'UnicodeCharacter0xe211',
u'\u0594': 'HebrewAccentZaqefQatan',
u'\u7a96': 'CjkUnifiedIdeograph7a96',
u'\ubf98': 'HangulSyllableBboen',
u'\ufc9a': 'ArabicLigatureYehWithHamzaAboveWithMeemInitialForm',
u'\u101d': 'MyanmarLetterWa',
u'\u551f': 'CjkUnifiedIdeograph551f',
u'\u8a21': 'CjkUnifiedIdeograph8a21',
u'\ucf23': 'HangulSyllableKyeod',
u'\u2da4': 'EthiopicSyllableSsee',
u'\u62a6': 'CjkUnifiedIdeograph62a6',
u'\ua7a8': 'UnicodeCharacter0xa7a8',
u'\ue4aa': 'UnicodeCharacter0xe4aa',
u'\u382d': 'CjkUnifiedIdeograph382d',
u'\ud9ac': 'UnicodeCharacter0xd9ac',
u'\u7d2f': 'CjkUnifiedIdeograph7d2f',
u'\ub231': 'HangulSyllableNweob',
u'\uf733': 'UnicodeCharacter0xf733',
u'\u0ab6': 'GujaratiLetterSha',
u'\u4fb8': 'CjkUnifiedIdeograph4fb8',
u'\u8cba': 'CjkUnifiedIdeograph8cba',
u'\u203d': 'Interrobang',
u'\uc1bc': 'HangulSyllableSwass',
u'\u653f': 'CjkUnifiedIdeograph653f',
u'\u5a41': 'CjkUnifiedIdeograph5a41',
u'\u9f43': 'CjkUnifiedIdeograph9f43',
u'\udc45': 'UnicodeCharacter0xdc45',
u'\u32c6': 'IdeographicTelegraphSymbolForJuly',
u'\u77c8': 'CjkUnifiedIdeograph77c8',
u'\ub4ca': 'HangulSyllableDyulm',
u'\ue9cc': 'UnicodeCharacter0xe9cc',
u'\u0d4f': 'UnicodeCharacter0xd4f',
u'\u4251': 'CjkUnifiedIdeograph4251',
u'\u8753': 'CjkUnifiedIdeograph8753',
u'\uc455': 'HangulSyllableSsyolt',
u'\u1fd8': 'GreekCapitalLetterIotaWithVrachy',
u'\u5cda': 'CjkUnifiedIdeograph5cda',
u'\u91dc': 'CjkUnifiedIdeograph91dc',
u'\u355f': 'CjkUnifiedIdeograph355f',
u'\ud6de': 'HangulSyllableHup',
u'\u6a61': 'CjkUnifiedIdeograph6a61',
u'\uaf63': 'HangulSyllableGgwah',
u'\uec65': 'UnicodeCharacter0xec65',
u'\u07e8': 'NkoLetterJonaJa',
u'\u44ea': 'CjkUnifiedIdeograph44ea',
u'\ub9ec': 'HangulSyllableMael',
u'\ufeee': 'ArabicLetterWawFinalForm',
u'\u1271': 'EthiopicSyllableTu',
u'\u5773': 'CjkUnifiedIdeograph5773',
u'\u9475': 'CjkUnifiedIdeograph9475',
u'\uc977': 'HangulSyllableJyulb',
u'\u2ff8': 'IdeographicDescriptionCharacterSurroundFromUpperLeft',
u'\u6cfa': 'CjkUnifiedIdeograph6cfa',
u'\ua1fc': 'YiSyllableKiex',
u'\ue6fe': 'UnicodeCharacter0xe6fe',
u'\udc00': 'UnicodeCharacter0xdc00',
u'\uf883': 'UnicodeCharacter0xf883',
u'\u1f02': 'GreekSmallLetterAlphaWithPsiliAndVaria',
u'\u3b85': 'CjkUnifiedIdeograph3b85',
u'\u9506': 'CjkUnifiedIdeograph9506',
u'\ub189': 'HangulSyllableNob',
u'\u4e0c': 'CjkUnifiedIdeograph4e0c',
u'\u6a8f': 'CjkUnifiedIdeograph6a8f',
u'\uc410': 'HangulSyllableSswae',
u'\ue093': 'UnicodeCharacter0xe093',
u'\u0712': 'SyriacLetterBeth',
u'\u2395': 'AplFunctionalSymbolQuad',
u'\ubd16': 'HangulSyllableBwanh',
u'\u5999': 'CjkUnifiedIdeograph5999',
u'\ud79d': 'HangulSyllableHing',
u'\u761c': 'CjkUnifiedIdeograph761c',
u'\u129f': 'EthiopicSyllableNywa',
u'\uec20': 'UnicodeCharacter0xec20',
u'\u88a3': 'CjkUnifiedIdeograph88a3',
u'\u2f22': 'KangxiRadicalGoSlowly',
u'\ua526': 'VaiSyllableHi',
u'\u41a9': 'CjkUnifiedIdeograph41a9',
u'\uffad': 'HalfwidthHangulLetterRieulsios',
u'\u1e2c': 'LatinCapitalLetterIWithTildeBelow',
u'\u3aaf': 'CjkUnifiedIdeograph3aaf',
u'\u9430': 'CjkUnifiedIdeograph9430',
u'\ub0b3': 'HangulSyllableNah',
u'\u4d36': 'CjkUnifiedIdeograph4d36',
u'\u69b9': 'CjkUnifiedIdeograph69b9',
u'\ucb3a': 'HangulSyllableJjyolp',
u'\ue7bd': 'UnicodeCharacter0xe7bd',
u'\u063c': 'ArabicLetterKehehWithThreeDotsBelow',
u'\u22bf': 'RightTriangle',
u'\ubc40': 'HangulSyllableBaem',
u'\u58c3': 'CjkUnifiedIdeograph58c3',
u'\ud6c7': 'HangulSyllableHugs',
u'\u7546': 'CjkUnifiedIdeograph7546',
u'\u11c9': 'HangulJongseongNieunthieuth',
u'\uf34a': 'UnicodeCharacter0xf34a',
u'\u8fcd': 'CjkUnifiedIdeograph8fcd',
u'\u2e4c': 'UnicodeCharacter0x2e4c',
u'\ua450': 'YiSyllableNyie',
u'\u40d3': 'CjkUnifiedIdeograph40d3',
u'\ufed7': 'ArabicLetterQafInitialForm',
u'\u1d56': 'ModifierLetterSmallP',
u'\u39d9': 'CjkUnifiedIdeograph39d9',
u'\u9b5a': 'CjkUnifiedIdeograph9b5a',
u'\ub7dd': 'HangulSyllableRyaelt',
u'\u4c60': 'CjkUnifiedIdeograph4c60',
u'\u68e3': 'CjkUnifiedIdeograph68e3',
u'\uca64': 'HangulSyllableJjeok',
u'\ue6e7': 'UnicodeCharacter0xe6e7',
u'\u0566': 'ArmenianSmallLetterZa',
u'\u21e9': 'DownwardsWhiteArrow',
u'\u836a': 'CjkUnifiedIdeograph836a',
u'\u5fed': 'CjkUnifiedIdeograph5fed',
u'\ud5f1': 'HangulSyllableHelt',
u'\u7470': 'CjkUnifiedIdeograph7470',
u'\u10f3': 'GeorgianLetterWe',
u'\uf274': 'UnicodeCharacter0xf274',
u'\u8ef7': 'CjkUnifiedIdeograph8ef7',
u'\u2d76': 'UnicodeCharacter0x2d76',
u'\uab7a': 'UnicodeCharacter0xab7a',
u'\u47fd': 'CjkUnifiedIdeograph47fd',
u'\u7d01': 'CjkUnifiedIdeograph7d01',
u'\u1c80': 'UnicodeCharacter0x1c80',
u'\ufb05': 'LatinSmallLigatureLongST',
u'\u9a84': 'CjkUnifiedIdeograph9a84',
u'\u3607': 'CjkUnifiedIdeograph3607',
u'\uac0b': 'HangulSyllableGalb',
u'\u538a': 'CjkUnifiedIdeograph538a',
u'\uc98e': 'HangulSyllableJeunh',
u'\u6511': 'CjkUnifiedIdeograph6511',
u'\u0490': 'CyrillicCapitalLetterGheWithUpturn',
u'\ue315': 'UnicodeCharacter0xe315',
u'\u8294': 'CjkUnifiedIdeograph8294',
u'\u541b': 'CjkUnifiedIdeograph541b',
u'\u7b9a': 'CjkUnifiedIdeograph7b9a',
u'\ud21f': 'HangulSyllableTyolh',
u'\uf19e': 'UnicodeCharacter0xf19e',
u'\u0d21': 'MalayalamLetterDda',
u'\u2ca0': 'CopticCapitalLetterPi',
u'\u8b25': 'CjkUnifiedIdeograph8b25',
u'\uaaa4': 'UnicodeCharacter0xaaa4',
u'\u7c2b': 'CjkUnifiedIdeograph7c2b',
u'\u63aa': 'CjkUnifiedIdeograph63aa',
u'\ufa2f': 'UnicodeCharacter0xfa2f',
u'\u99ae': 'CjkUnifiedIdeograph99ae',
u'\u3531': 'CjkUnifiedIdeograph3531',
u'\ub335': 'HangulSyllableDyat',
u'\u52b4': 'CjkUnifiedIdeograph52b4',
u'\uc8b8': 'HangulSyllableJwaem',
u'\u643b': 'CjkUnifiedIdeograph643b',
u'\u0bba': 'UnicodeCharacter0xbba',
u'\ue23f': 'UnicodeCharacter0xe23f',
u'\u81be': 'CjkUnifiedIdeograph81be',
u'\u5b45': 'CjkUnifiedIdeograph5b45',
u'\u7ac4': 'CjkUnifiedIdeograph7ac4',
u'\ud149': 'HangulSyllableTeot',
u'\uf0c8': 'UnicodeCharacter0xf0c8',
u'\u0c4b': 'TeluguVowelSignOo',
u'\u33ca': 'SquareHa',
u'\u8a4f': 'CjkUnifiedIdeograph8a4f',
u'\ua9ce': 'UnicodeCharacter0xa9ce',
u'\u4355': 'CjkUnifiedIdeograph4355',
u'\u62d4': 'CjkUnifiedIdeograph62d4',
u'\uf959': 'CjkCompatibilityIdeographf959',
u'\u98d8': 'CjkUnifiedIdeograph98d8',
u'\u345b': 'CjkUnifiedIdeograph345b',
u'\ub25f': 'HangulSyllableNwid',
u'\u51de': 'CjkUnifiedIdeograph51de',
u'\ucfe2': 'HangulSyllableKugg',
u'\u6b65': 'CjkUnifiedIdeograph6b65',
u'\u0ae4': 'UnicodeCharacter0xae4',
u'\ue169': 'UnicodeCharacter0xe169',
u'\u80e8': 'CjkUnifiedIdeograph80e8',
u'\u5a6f': 'CjkUnifiedIdeograph5a6f',
u'\u79ee': 'CjkUnifiedIdeograph79ee',
u'\ud073': 'HangulSyllableKeud',
u'\uf7f2': 'UnicodeCharacter0xf7f2',
u'\u1375': 'EthiopicNumberForty',
u'\u32f4': 'CircledKatakanaYu',
u'\u8979': 'CjkUnifiedIdeograph8979',
u'\ua8f8': 'UnicodeCharacter0xa8f8',
u'\u427f': 'CjkUnifiedIdeograph427f',
u'\u61fe': 'CjkUnifiedIdeograph61fe',
u'\u1406': 'CanadianSyllabicsOo',
u'\u3089': 'HiraganaLetterRa',
u'\u5108': 'CjkUnifiedIdeograph5108',
u'\u6d8b': 'CjkUnifiedIdeograph6d8b',
u'\u920a': 'CjkUnifiedIdeograph920a',
u'\uae8d': 'HangulSyllableGgyalg',
u'\ucf0c': 'HangulSyllableKels',
u'\ueb8f': 'UnicodeCharacter0xeb8f',
u'\u3c16': 'CjkUnifiedIdeograph3c16',
u'\u7918': 'CjkUnifiedIdeograph7918',
u'\u159b': 'CanadianSyllabicsWoodscreeThe',
u'\uba1a': 'HangulSyllableMyap',
u'\u569d': 'CjkUnifiedIdeograph569d',
u'\uf71c': 'UnicodeCharacter0xf71c',
u'\u939f': 'CjkUnifiedIdeograph939f',
u'\ucca1': 'HangulSyllableCeolg',
u'\u2426': 'SymbolForSubstituteFormTwo',
u'\u6128': 'CjkUnifiedIdeograph6128',
u'\u3dab': 'CjkUnifiedIdeograph3dab',
u'\ua22a': 'YiSyllableGgut',
u'\u7ead': 'CjkUnifiedIdeograph7ead',
u'\u9f2c': 'CjkUnifiedIdeograph9f2c',
u'\ubbaf': 'HangulSyllableMyulb',
u'\ud82e': 'UnicodeCharacter0xd82e',
u'\uf4b1': 'UnicodeCharacter0xf4b1',
u'\u0938': 'DevanagariLetterSa',
u'\u25bb': 'WhiteRightpointingPointer',
u'\u4a3a': 'CjkUnifiedIdeograph4a3a',
u'\u66bd': 'CjkUnifiedIdeograph66bd',
u'\u873c': 'CjkUnifiedIdeograph873c',
u'\ua3bf': 'YiSyllableShur',
u'\uc03e': 'HangulSyllableBbyugg',
u'\u9cc1': 'CjkUnifiedIdeograph9cc1',
u'\ud9c3': 'UnicodeCharacter0xd9c3',
u'\u3148': 'HangulLetterCieuc',
u'\u724a': 'CjkUnifiedIdeograph724a',
u'\u0ecd': 'LaoNiggahita',
u'\uaf4c': 'HangulSyllableGgwan',
u'\u4bcf': 'CjkUnifiedIdeograph4bcf',
u'\ue84e': 'UnicodeCharacter0xe84e',
u'\u84d1': 'CjkUnifiedIdeograph84d1',
u'\uc1d3': 'HangulSyllableSwaelh',
u'\u1a5a': 'UnicodeCharacter0x1a5a',
u'\u36dd': 'CjkUnifiedIdeograph36dd',
u'\u575c': 'CjkUnifiedIdeograph575c',
u'\u73df': 'CjkUnifiedIdeograph73df',
u'\u905e': 'CjkUnifiedIdeograph905e',
u'\uace1': 'HangulSyllableGog',
u'\ucd60': 'HangulSyllableCoen',
u'\ue9e3': 'UnicodeCharacter0xe9e3',
u'\u026a': 'LatinLetterSmallCapitalI',
u'\u7f6c': 'CjkUnifiedIdeograph7f6c',
u'\u1bef': 'UnicodeCharacter0x1bef',
u'\ub86e': 'HangulSyllableRobs',
u'\u54f1': 'CjkUnifiedIdeograph54f1',
u'\uf570': 'UnicodeCharacter0xf570',
u'\u91f3': 'CjkUnifiedIdeograph91f3',
u'\ud2f5': 'HangulSyllableTinj',
u'\u2a7a': 'GreaterthanWithCircleInside',
u'\u677c': 'CjkUnifiedIdeograph677c',
u'\u03ff': 'GreekCapitalReversedDottedLunateSigmaSymbol',
u'\ua07e': 'YiSyllableNbax',
u'\u9d80': 'CjkUnifiedIdeograph9d80',
u'\u3903': 'CjkUnifiedIdeograph3903',
u'\ude82': 'UnicodeCharacter0xde82',
u'\u7a05': 'CjkUnifiedIdeograph7a05',
u'\ub707': 'HangulSyllableDdwic',
u'\uf009': 'UnicodeCharacter0xf009',
u'\u0f8c': 'UnicodeCharacter0xf8c',
u'\u488e': 'CjkUnifiedIdeograph488e',
u'\u8590': 'CjkUnifiedIdeograph8590',
u'\u2113': 'ScriptSmallL',
u'\uc692': 'HangulSyllableOep',
u'\u6215': 'CjkUnifiedIdeograph6215',
u'\u5f17': 'CjkUnifiedIdeograph5f17',
u'\u9819': 'CjkUnifiedIdeograph9819',
u'\ud51b': 'HangulSyllablePeuc',
u'\u379c': 'CjkUnifiedIdeograph379c',
u'\u709e': 'CjkUnifiedIdeograph709e',
u'\uada0': 'HangulSyllableGweok',
u'\ueea2': 'UnicodeCharacter0xeea2',
u'\u0a25': 'GurmukhiLetterTha',
u'\u4727': 'CjkUnifiedIdeograph4727',
u'\u8029': 'CjkUnifiedIdeograph8029',
u'\ufd2b': 'ArabicLigatureSadWithRehFinalForm',
u'\u18ae': 'UnicodeCharacter0x18ae',
u'\u55b0': 'CjkUnifiedIdeograph55b0',
u'\u96b2': 'CjkUnifiedIdeograph96b2',
u'\u3235': 'ParenthesizedIdeographSpecial',
u'\ud3b4': 'HangulSyllablePyeo',
u'\u6f37': 'CjkUnifiedIdeograph6f37',
u'\ua839': 'UnicodeCharacter0xa839',
u'\ue53b': 'UnicodeCharacter0xe53b',
u'\xbe': 'VulgarFractionThreeQuarters',
u'\u7dc0': 'CjkUnifiedIdeograph7dc0',
u'\ubec2': 'HangulSyllableBbyaelp',
u'\ufbc4': 'UnicodeCharacter0xfbc4',
u'\u1747': 'BuhidLetterDa',
u'\u5049': 'CjkUnifiedIdeograph5049',
u'\u8d4b': 'CjkUnifiedIdeograph8d4b',
u'\uce4d': 'HangulSyllableCyib',
u'\u28ce': 'BraillePatternDots23478',
u'\u65d0': 'CjkUnifiedIdeograph65d0',
u'\ua6d2': 'UnicodeCharacter0xa6d2',
u'\ue3d4': 'UnicodeCharacter0xe3d4',
u'\u3f57': 'CjkUnifiedIdeograph3f57',
u'\udcd6': 'UnicodeCharacter0xdcd6',
u'\u7859': 'CjkUnifiedIdeograph7859',
u'\ub55b': 'HangulSyllableDdaelh',
u'\uf65d': 'UnicodeCharacter0xf65d',
u'\u0de0': 'UnicodeCharacter0xde0',
u'\u4ee2': 'CjkUnifiedIdeograph4ee2',
u'\u8be4': 'CjkUnifiedIdeograph8be4',
u'\u2767': 'RotatedFloralHeartBullet',
u'\uc4e6': 'HangulSyllableSsyubs',
u'\u6069': 'CjkUnifiedIdeograph6069',
u'\u5d6b': 'CjkUnifiedIdeograph5d6b',
u'\u9e6d': 'CjkUnifiedIdeograph9e6d',
u'\udb6f': 'UnicodeCharacter0xdb6f',
u'\u35f0': 'CjkUnifiedIdeograph35f0',
u'\u76f2': 'CjkUnifiedIdeograph76f2',
u'\ub3f4': 'HangulSyllableDwass',
u'\uecf6': 'UnicodeCharacter0xecf6',
u'\u0879': 'UnicodeCharacter0x879',
u'\u457b': 'CjkUnifiedIdeograph457b',
u'\u867d': 'CjkUnifiedIdeograph867d',
u'\uc37f': 'HangulSyllableSseoc',
u'\ubf81': 'HangulSyllableBbwaelg',
u'\u5804': 'CjkUnifiedIdeograph5804',
u'\u7487': 'CjkUnifiedIdeograph7487',
u'\ud208': 'HangulSyllableToess',
u'\uee8b': 'UnicodeCharacter0xee8b',
u'\u110a': 'HangulChoseongSsangsios',
u'\u2d8d': 'EthiopicSyllableDdoa',
u'\u8b0e': 'CjkUnifiedIdeograph8b0e',
u'\ua791': 'UnicodeCharacter0xa791',
u'\u4014': 'CjkUnifiedIdeograph4014',
u'\u1c97': 'UnicodeCharacter0x1c97',
u'\ufa18': 'CjkCompatibilityIdeographfa18',
u'\u969b': 'CjkUnifiedIdeograph969b',
u'\u391a': 'CjkUnifiedIdeograph391a',
u'\ub31e': 'HangulSyllableDyagg',
u'\u4fa1': 'CjkUnifiedIdeograph4fa1',
u'\uc9a5': 'HangulSyllableJyig',
u'\u6824': 'CjkUnifiedIdeograph6824',
u'\u04a7': 'CyrillicSmallLetterPeWithMiddleHook',
u'\ue228': 'UnicodeCharacter0xe228',
u'\ubeab': 'HangulSyllableBbyas',
u'\u212a': 'KelvinSign',
u'\u5b2e': 'CjkUnifiedIdeograph5b2e',
u'\u77b1': 'CjkUnifiedIdeograph77b1',
u'\ud532': 'HangulSyllablePyibs',
u'\uf1b5': 'UnicodeCharacter0xf1b5',
u'\u1034': 'MyanmarVowelSignMonO',
u'\u2cb7': 'CopticSmallLetterCryptogrammicEie',
u'\u8a38': 'CjkUnifiedIdeograph8a38',
u'\ua6bb': 'UnicodeCharacter0xa6bb',
u'\u433e': 'CjkUnifiedIdeograph433e',
u'\u1fc1': 'GreekDialytikaAndPerispomeni',
u'\ufd42': 'UnicodeCharacter0xfd42',
u'\u99c5': 'CjkUnifiedIdeograph99c5',
u'\u3844': 'CjkUnifiedIdeograph3844',
u'\ub248': 'HangulSyllableNwels',
u'\u4ecb': 'CjkUnifiedIdeograph4ecb',
u'\uc8cf': 'HangulSyllableJoelb',
u'\u6b4e': 'CjkUnifiedIdeograph6b4e',
u'\u07d1': 'NkoLetterDagbasinna',
u'\ue552': 'UnicodeCharacter0xe552',
u'\u81d5': 'CjkUnifiedIdeograph81d5',
u'\u2054': 'InvertedUndertie',
u'\u5a58': 'CjkUnifiedIdeograph5a58',
u'\u76db': 'CjkUnifiedIdeograph76db',
u'\ud45c': 'HangulSyllablePyo',
u'\uf0df': 'UnicodeCharacter0xf0df',
u'\u135e': 'UnicodeCharacter0x135e',
u'\u2fe1': 'UnicodeCharacter0x2fe1',
u'\u8d62': 'CjkUnifiedIdeograph8d62',
u'\ua9e5': 'UnicodeCharacter0xa9e5',
u'\u4268': 'CjkUnifiedIdeograph4268',
u'\u1eeb': 'LatinSmallLetterUWithHornAndGrave',
u'\ufc6c': 'ArabicLigatureBehWithMeemFinalForm',
u'\u98ef': 'CjkUnifiedIdeograph98ef',
u'\u3b6e': 'CjkUnifiedIdeograph3b6e',
u'\ub572': 'HangulSyllableDdyalm',
u'\u51f5': 'CjkUnifiedIdeograph51f5',
u'\ucbf9': 'HangulSyllableJjyilg',
u'\u6a78': 'CjkUnifiedIdeograph6a78',
u'\u06fb': 'ArabicLetterDadWithDotBelow',
u'\ue47c': 'UnicodeCharacter0xe47c',
u'\u80ff': 'CjkUnifiedIdeograph80ff',
u'\u237e': 'BellSymbol',
u'\uba03': 'HangulSyllableMyags',
u'\u5d82': 'CjkUnifiedIdeograph5d82',
u'\ud786': 'HangulSyllableHyip',
u'\u7309': 'CjkUnifiedIdeograph7309',
u'\u1288': 'EthiopicSyllableXwa',
u'\ued0d': 'UnicodeCharacter0xed0d',
u'\u8c8c': 'CjkUnifiedIdeograph8c8c',
u'\u280f': 'BraillePatternDots1234',
u'\ua213': 'YiSyllableKur',
u'\u4592': 'CjkUnifiedIdeograph4592',
u'\udc17': 'UnicodeCharacter0xdc17',
u'\uff96': 'HalfwidthKatakanaLetterYo',
u'\u1b19': 'BalineseLetterCaLaca',
u'\u3a98': 'CjkUnifiedIdeograph3a98',
u'\u951d': 'CjkUnifiedIdeograph951d',
u'\ub49c': 'HangulSyllableDwess',
u'\u4a23': 'CjkUnifiedIdeograph4a23',
u'\u6da2': 'CjkUnifiedIdeograph6da2',
u'\uc427': 'HangulSyllableSswaec',
u'\ue7a6': 'UnicodeCharacter0xe7a6',
u'\u0329': 'CombiningVerticalLineBelow',
u'\u22a8': 'True',
u'\ubd2d': 'HangulSyllableBwaeg',
u'\u5cac': 'CjkUnifiedIdeograph5cac',
u'\ud6b0': 'HangulSyllableHyol',
u'\u7233': 'CjkUnifiedIdeograph7233',
u'\u15b2': 'CanadianSyllabicsBlackfootO',
u'\uec37': 'UnicodeCharacter0xec37',
u'\u8fb6': 'CjkUnifiedIdeograph8fb6',
u'\u2b39': 'LeftwardsArrowWithTailWithVerticalStroke',
u'\ua53d': 'VaiSyllableZi',
u'\u44bc': 'CjkUnifiedIdeograph44bc',
u'\udf41': 'UnicodeCharacter0xdf41',
u'\ufec0': 'ArabicLetterDadMedialForm',
u'\u1a43': 'UnicodeCharacter0x1a43',
u'\u3dc2': 'CjkUnifiedIdeograph3dc2',
u'\u9447': 'CjkUnifiedIdeograph9447',
u'\ub7c6': 'HangulSyllableRyabs',
u'\u4d4d': 'CjkUnifiedIdeograph4d4d',
u'\u6ccc': 'CjkUnifiedIdeograph6ccc',
u'\uc751': 'HangulSyllableEung',
u'\ue6d0': 'UnicodeCharacter0xe6d0',
u'\u0253': 'LatinSmallLetterBWithHook',
u'\u25d2': 'CircleWithLowerHalfBlack',
u'\ubc57': 'HangulSyllableByalb',
u'\u5fd6': 'CjkUnifiedIdeograph5fd6',
u'\ud9da': 'UnicodeCharacter0xd9da',
u'\u755d': 'CjkUnifiedIdeograph755d',
u'\u14dc': 'CanadianSyllabicsLwe',
u'\uef61': 'UnicodeCharacter0xef61',
u'\u8ee0': 'CjkUnifiedIdeograph8ee0',
u'\u2a63': 'LogicalOrWithDoubleUnderbar',
u'\ua467': 'YiSyllableXot',
u'\u47e6': 'CjkUnifiedIdeograph47e6',
u'\ude6b': 'UnicodeCharacter0xde6b',
u'\uc1ea': 'HangulSyllableSoelm',
u'\u1d6d': 'LatinSmallLetterDWithMiddleTilde',
u'\u3cec': 'CjkUnifiedIdeograph3cec',
u'\u9771': 'CjkUnifiedIdeograph9771',
u'\ub6f0': 'HangulSyllableDdwi',
u'\u4c77': 'CjkUnifiedIdeograph4c77',
u'\u6ff6': 'CjkUnifiedIdeograph6ff6',
u'\uc67b': 'HangulSyllableOegs',
u'\ue9fa': 'UnicodeCharacter0xe9fa',
u'\u057d': 'ArmenianSmallLetterSeh',
u'\u24fc': 'DoubleCircledDigitEight',
u'\u3e81': 'CjkUnifiedIdeograph3e81',
u'\u5f00': 'CjkUnifiedIdeograph5f00',
u'\u7b83': 'CjkUnifiedIdeograph7b83',
u'\u9c02': 'CjkUnifiedIdeograph9c02',
u'\ub885': 'HangulSyllableRwalt',
u'\ud904': 'UnicodeCharacter0xd904',
u'\uf587': 'UnicodeCharacter0xf587',
u'\u0a0e': 'UnicodeCharacter0xa0e',
u'\u2691': 'BlackFlag',
u'\u4710': 'CjkUnifiedIdeograph4710',
u'\u6393': 'CjkUnifiedIdeograph6393',
u'\u8412': 'CjkUnifiedIdeograph8412',
u'\ua095': 'YiSyllableHmiex',
u'\uc114': 'HangulSyllableSyaess',
u'\u9d97': 'CjkUnifiedIdeograph9d97',
u'\uda99': 'UnicodeCharacter0xda99',
u'\u321e': 'ParenthesizedKoreanCharacterOHu',
u'\u6f20': 'CjkUnifiedIdeograph6f20',
u'\u0ba3': 'TamilLetterNna',
u'\uac22': 'HangulSyllableGaenh',
u'\u48a5': 'CjkUnifiedIdeograph48a5',
u'\ue924': 'UnicodeCharacter0xe924',
u'\u85a7': 'CjkUnifiedIdeograph85a7',
u'\uc2a9': 'HangulSyllableSeunj',
u'\u1730': 'HanunooLetterSa',
u'\u33b3': 'SquareMs',
u'\u5432': 'CjkUnifiedIdeograph5432',
u'\u70b5': 'CjkUnifiedIdeograph70b5',
u'\u9134': 'CjkUnifiedIdeograph9134',
u'\uadb7': 'HangulSyllableGwes',
u'\uce36': 'HangulSyllableCeuj',
u'\ueab9': 'UnicodeCharacter0xeab9',
u'\u3f40': 'CjkUnifiedIdeograph3f40',
u'\u7c42': 'CjkUnifiedIdeograph7c42',
u'\u18c5': 'UnicodeCharacter0x18c5',
u'\ub944': 'HangulSyllableRwil',
u'\u55c7': 'CjkUnifiedIdeograph55c7',
u'\uf646': 'UnicodeCharacter0xf646',
u'\u92c9': 'CjkUnifiedIdeograph92c9',
u'\ucfcb': 'HangulSyllableKyod',
u'\u2750': 'UpperRightDropshadowedWhiteSquare',
u'\u6452': 'CjkUnifiedIdeograph6452',
u'\xd5': 'LatinCapitalLetterOWithTilde',
u'\ua154': 'YiSyllableNdip',
u'\u7dd7': 'CjkUnifiedIdeograph7dd7',
u'\u9e56': 'CjkUnifiedIdeograph9e56',
u'\ubad9': 'HangulSyllableMwang',
u'\udb58': 'UnicodeCharacter0xdb58',
u'\uf7db': 'UnicodeCharacter0xf7db',
u'\u0c62': 'TeluguVowelSignVocalicL',
u'\u28e5': 'BraillePatternDots13678',
u'\u4964': 'CjkUnifiedIdeograph4964',
u'\u65e7': 'CjkUnifiedIdeograph65e7',
u'\u8666': 'CjkUnifiedIdeograph8666',
u'\ua2e9': 'YiSyllableZzy',
u'\uc368': 'HangulSyllableSseo',
u'\u9feb': 'UnicodeCharacter0x9feb',
u'\udced': 'UnicodeCharacter0xdced',
u'\u3472': 'CjkUnifiedIdeograph3472',
u'\u7174': 'CjkUnifiedIdeograph7174',
u'\u0df7': 'UnicodeCharacter0xdf7',
u'\uae76': 'HangulSyllableGgaelp',
u'\u4af9': 'CjkUnifiedIdeograph4af9',
u'\ueb78': 'UnicodeCharacter0xeb78',
u'\u87fb': 'CjkUnifiedIdeograph87fb',
u'\uc4fd': 'HangulSyllableSseult',
u'\ufe01': 'VariationSelector2',
u'\u1984': 'NewTaiLueLetterHighNga',
u'\u5686': 'CjkUnifiedIdeograph5686',
u'\u9388': 'CjkUnifiedIdeograph9388',
u'\u2f0b': 'KangxiRadicalEight',
u'\ud08a': 'HangulSyllableKyigg',
u'\u6c0d': 'CjkUnifiedIdeograph6c0d',
u'\ua90f': 'KayahLiLetterSha',
u'\ue611': 'UnicodeCharacter0xe611',
u'\u0194': 'LatinCapitalLetterGamma',
u'\u7e96': 'CjkUnifiedIdeograph7e96',
u'\ubb98': 'HangulSyllableMwim',
u'\uf89a': 'UnicodeCharacter0xf89a',
u'\u141d': 'CanadianSyllabicsYcreeW',
u'\u511f': 'CjkUnifiedIdeograph511f',
u'\u8e21': 'CjkUnifiedIdeograph8e21',
u'\ucb23': 'HangulSyllableJjoes',
u'\u29a4': 'AngleWithUnderbar',
u'\u66a6': 'CjkUnifiedIdeograph66a6',
u'\ua3a8': 'YiSyllableNryp',
u'\ue0aa': 'UnicodeCharacter0xe0aa',
u'\u3c2d': 'CjkUnifiedIdeograph3c2d',
u'\uddac': 'UnicodeCharacter0xddac',
u'\u792f': 'CjkUnifiedIdeograph792f',
u'\ub631': 'HangulSyllableDdwanj',
u'\uf333': 'UnicodeCharacter0xf333',
u'\u0eb6': 'LaoVowelSignY',
u'\u4bb8': 'CjkUnifiedIdeograph4bb8',
u'\u88ba': 'CjkUnifiedIdeograph88ba',
u'\u243d': 'UnicodeCharacter0x243d',
u'\uc5bc': 'HangulSyllableEol',
u'\u613f': 'CjkUnifiedIdeograph613f',
u'\u5e41': 'CjkUnifiedIdeograph5e41',
u'\u9b43': 'CjkUnifiedIdeograph9b43',
u'\ud845': 'UnicodeCharacter0xd845',
u'\u36c6': 'CjkUnifiedIdeograph36c6',
u'\u73c8': 'CjkUnifiedIdeograph73c8',
u'\ub0ca': 'HangulSyllableNaej',
u'\uedcc': 'UnicodeCharacter0xedcc',
u'\u094f': 'UnicodeCharacter0x94f',
u'\u4651': 'CjkUnifiedIdeograph4651',
u'\u8353': 'CjkUnifiedIdeograph8353',
u'\uc055': 'HangulSyllableBbyut',
u'\u1bd8': 'UnicodeCharacter0x1bd8',
u'\u58da': 'CjkUnifiedIdeograph58da',
u'\u95dc': 'CjkUnifiedIdeograph95dc',
u'\u315f': 'HangulLetterWi',
u'\ud2de': 'HangulSyllableTyilm',
u'\u6e61': 'CjkUnifiedIdeograph6e61',
u'\uab63': 'UnicodeCharacter0xab63',
u'\ue865': 'UnicodeCharacter0xe865',
u'\u03e8': 'CopticCapitalLetterHori',
u'\u40ea': 'CjkUnifiedIdeograph40ea',
u'\ubdec': 'HangulSyllableBwik',
u'\ufaee': 'UnicodeCharacter0xfaee',
u'\u1671': 'CanadianSyllabicsNngi',
u'\u5373': 'CjkUnifiedIdeograph5373',
u'\u9075': 'CjkUnifiedIdeograph9075',
u'\ucd77': 'HangulSyllableCoeh',
u'\u2bf8': 'UnicodeCharacter0x2bf8',
u'\u68fa': 'CjkUnifiedIdeograph68fa',
u'\ua5fc': 'VaiSyllableShe',
u'\ue2fe': 'UnicodeCharacter0xe2fe',
u'\ue000': 'UnicodeCharacter0xe000',
u'\uc483': 'HangulSyllableSsweogs',
u'\u2302': 'House',
u'\u0785': 'ThaanaLetterLhaviyani',
u'\ua906': 'KayahLiDigitSix',
u'\u8d89': 'CjkUnifiedIdeograph8d89',
u'\u720c': 'CjkUnifiedIdeograph720c',
u'\u568f': 'CjkUnifiedIdeograph568f',
u'\uf810': 'UnicodeCharacter0xf810',
u'\udc93': 'UnicodeCharacter0xdc93',
u'\u3b12': 'CjkUnifiedIdeograph3b12',
u'\u1f95': 'GreekSmallLetterEtaWithDasiaAndOxiaAndYpogegrammeni',
u'\u8116': 'CjkUnifiedIdeograph8116',
u'\u6599': 'CjkUnifiedIdeograph6599',
u'\ueb9d': 'UnicodeCharacter0xeb9d',
u'\u4a1c': 'CjkUnifiedIdeograph4a1c',
u'\u2e9f': 'CjkRadicalMother',
u'\ud020': 'HangulSyllableKwel',
u'\ub4a3': 'HangulSyllableDweh',
u'\u1322': 'EthiopicSyllableThi',
u'\u9926': 'CjkUnifiedIdeograph9926',
u'\u7da9': 'CjkUnifiedIdeograph7da9',
u'\uc3ad': 'HangulSyllableSsyeolt',
u'\u222c': 'DoubleIntegral',
u'\u06af': 'ArabicLetterGaf',
u'\ua830': 'UnicodeCharacter0xa830',
u'\u8cb3': 'CjkUnifiedIdeograph8cb3',
u'\u7136': 'CjkUnifiedIdeograph7136',
u'\u55b9': 'CjkUnifiedIdeograph55b9',
u'\uf73a': 'UnicodeCharacter0xf73a',
u'\udbbd': 'UnicodeCharacter0xdbbd',
u'\u3a3c': 'CjkUnifiedIdeograph3a3c',
u'\u1ebf': 'LatinSmallLetterEWithCircumflexAndAcute',
u'\u8040': 'CjkUnifiedIdeograph8040',
u'\u64c3': 'CjkUnifiedIdeograph64c3',
u'\ueac7': 'UnicodeCharacter0xeac7',
u'\u4946': 'CjkUnifiedIdeograph4946',
u'\u2dc9': 'EthiopicSyllableKyu',
u'\ucf4a': 'HangulSyllableKyebs',
u'\ub3cd': 'HangulSyllableDolg',
u'\u124c': 'EthiopicSyllableQwee',
u'\u9850': 'CjkUnifiedIdeograph9850',
u'\u7cd3': 'CjkUnifiedIdeograph7cd3',
u'\uc2d7': 'HangulSyllableSyic',
u'\u2156': 'VulgarFractionTwoFifths',
u'\u05d9': 'HebrewLetterYod',
u'\ua75a': 'LatinCapitalLetterRRotunda',
u'\u8bdd': 'CjkUnifiedIdeograph8bdd',
u'\u7060': 'CjkUnifiedIdeograph7060',
u'\u54e3': 'CjkUnifiedIdeograph54e3',
u'\uf664': 'UnicodeCharacter0xf664',
u'\udae7': 'UnicodeCharacter0xdae7',
u'\u3966': 'CjkUnifiedIdeograph3966',
u'\u1de9': 'UnicodeCharacter0x1de9',
u'\ubf6a': 'HangulSyllableBbwalp',
u'\u63ed': 'CjkUnifiedIdeograph63ed',
u'\ue9f1': 'UnicodeCharacter0xe9f1',
u'\u4870': 'CjkUnifiedIdeograph4870',
u'\u2cf3': 'UnicodeCharacter0x2cf3',
u'\uce74': 'HangulSyllableKa',
u'\ub2f7': 'HangulSyllableDas',
u'\u1176': 'HangulJungseongAo',
u'\u977a': 'CjkUnifiedIdeograph977a',
u'\u7bfd': 'CjkUnifiedIdeograph7bfd',
u'\u4101': 'CjkUnifiedIdeograph4101',
u'\u2080': 'SubscriptZero',
u'\uc705': 'HangulSyllableWig',
u'\ua684': 'CyrillicCapitalLetterZhwe',
u'\u0a07': 'GurmukhiLetterI',
u'\u900b': 'CjkUnifiedIdeograph900b',
u'\u6f8a': 'CjkUnifiedIdeograph6f8a',
u'\uf58e': 'UnicodeCharacter0xf58e',
u'\u5911': 'CjkUnifiedIdeograph5911',
u'\u3890': 'CjkUnifiedIdeograph3890',
u'\udf15': 'UnicodeCharacter0xdf15',
u'\ube94': 'HangulSyllableBbaek',
u'\u681b': 'CjkUnifiedIdeograph681b',
u'\u479a': 'CjkUnifiedIdeograph479a',
u'\uee1f': 'UnicodeCharacter0xee1f',
u'\ucd9e': 'HangulSyllableCulm',
u'\u3121': 'BopomofoLetterOu',
u'\u10a0': 'GeorgianCapitalLetterAn',
u'\ub725': 'HangulSyllableDdyut',
u'\u96a4': 'CjkUnifiedIdeograph96a4',
u'\u402b': 'CjkUnifiedIdeograph402b',
u'\u5faa': 'CjkUnifiedIdeograph5faa',
u'\uc62f': 'HangulSyllableOlb',
u'\ua5ae': 'VaiSyllableZu',
u'\u0931': 'DevanagariLetterRra',
u'\u8f35': 'CjkUnifiedIdeograph8f35',
u'\u6eb4': 'CjkUnifiedIdeograph6eb4',
u'\uf4b8': 'UnicodeCharacter0xf4b8',
u'\u583b': 'CjkUnifiedIdeograph583b',
u'\u37ba': 'CjkUnifiedIdeograph37ba',
u'\ude3f': 'UnicodeCharacter0xde3f',
u'\ubdbe': 'HangulSyllableBwenh',
u'\u6745': 'CjkUnifiedIdeograph6745',
u'\u46c4': 'CjkUnifiedIdeograph46c4',
u'\ued49': 'UnicodeCharacter0xed49',
u'\uccc8': 'HangulSyllableCess',
u'\u304b': 'HiraganaLetterKa',
u'\u0fca': 'TibetanSymbolNorBuNyisKhyil',
u'\ub64f': 'HangulSyllableDdwaed',
u'\u95ce': 'CjkUnifiedIdeograph95ce',
u'\u7f55': 'CjkUnifiedIdeograph7f55',
u'\u5ed4': 'CjkUnifiedIdeograph5ed4',
u'\uc559': 'HangulSyllableAng',
u'\ua4d8': 'UnicodeCharacter0xa4d8',
u'\u085b': 'UnicodeCharacter0x85b',
u'\u8e5f': 'CjkUnifiedIdeograph8e5f',
u'\u6dde': 'CjkUnifiedIdeograph6dde',
u'\uf3e2': 'UnicodeCharacter0xf3e2',
u'\u5765': 'CjkUnifiedIdeograph5765',
u'\u36e4': 'CjkUnifiedIdeograph36e4',
u'\udd69': 'UnicodeCharacter0xdd69',
u'\ubce8': 'HangulSyllableByem',
u'\u666f': 'CjkUnifiedIdeograph666f',
u'\u45ee': 'CjkUnifiedIdeograph45ee',
u'\uec73': 'UnicodeCharacter0xec73',
u'\ucbf2': 'HangulSyllableJjyigg',
u'\u2f75': 'KangxiRadicalBamboo',
u'\u0ef4': 'UnicodeCharacter0xef4',
u'\ub579': 'HangulSyllableDdyab',
u'\u94f8': 'CjkUnifiedIdeograph94f8',
u'\u7e7f': 'CjkUnifiedIdeograph7e7f',
u'\u5dfe': 'CjkUnifiedIdeograph5dfe',
u'\u2806': 'BraillePatternDots23',
u'\u0c89': 'KannadaLetterU',
u'\u6d08': 'CjkUnifiedIdeograph6d08',
u'\u518b': 'CjkUnifiedIdeograph518b',
u'\uae0a': 'HangulSyllableGeubs',
u'\u928d': 'CjkUnifiedIdeograph928d',
u'\uf30c': 'UnicodeCharacter0xf30c',
u'\ud78f': 'HangulSyllableHid',
u'\u4518': 'CjkUnifiedIdeograph4518',
u'\u299b': 'MeasuredAngleOpeningLeft',
u'\u861a': 'CjkUnifiedIdeograph861a',
u'\u6a9d': 'CjkUnifiedIdeograph6a9d',
u'\ucb1c': 'HangulSyllableJjoels',
u'\uaf9f': 'HangulSyllableGgyogs',
u'\uf0a1': 'UnicodeCharacter0xf0a1',
u'\u1826': 'MongolianLetterUe',
u'\u5d28': 'CjkUnifiedIdeograph5d28',
u'\u01ab': 'LatinSmallLetterTWithPalatalHook',
u'\u9e2a': 'CjkUnifiedIdeograph9e2a',
u'\u42ad': 'CjkUnifiedIdeograph42ad',
u'\ua32c': 'YiSyllableSsi',
u'\u87af': 'CjkUnifiedIdeograph87af',
u'\ue42e': 'UnicodeCharacter0xe42e',
u'\uc8b1': 'HangulSyllableJwaelg',
u'\u3538': 'CjkUnifiedIdeograph3538',
u'\u19bb': 'NewTaiLueVowelSignAay',
u'\u763a': 'CjkUnifiedIdeograph763a',
u'\u5abd': 'CjkUnifiedIdeograph5abd',
u'\ubb3c': 'HangulSyllableMul',
u'\u9fbf': 'UnicodeCharacter0x9fbf',
u'\ufc3e': 'ArabicLigatureKafWithYehIsolatedForm',
u'\ua0c1': 'YiSyllableMex',
u'\ue5c3': 'UnicodeCharacter0xe5c3',
u'\u0d48': 'MalayalamVowelSignAi',
u'\u4e4a': 'CjkUnifiedIdeograph4e4a',
u'\u32cd': 'SquareErg',
u'\u934c': 'CjkUnifiedIdeograph934c',
u'\u77cf': 'CjkUnifiedIdeograph77cf',
u'\ud44e': 'HangulSyllablePoelp',
u'\ub8d1': 'HangulSyllableRyonj',
u'\ufdd3': 'UnicodeCharacter0xfdd3',
u'\u265a': 'BlackChessKing',
u'\u0add': 'UnicodeCharacter0xadd',
u'\u6b5c': 'CjkUnifiedIdeograph6b5c',
u'\u4fdf': 'CjkUnifiedIdeograph4fdf',
u'\uac5e': 'HangulSyllableGyaelm',
u'\u90e1': 'CjkUnifiedIdeograph90e1',
u'\uf160': 'UnicodeCharacter0xf160',
u'\ud5e3': 'HangulSyllableHeoh',
u'\u3e6a': 'CjkUnifiedIdeograph3e6a',
u'\u436c': 'CjkUnifiedIdeograph436c',
u'\u27ef': 'MathematicalRightFlattenedParenthesis',
u'\u846e': 'CjkUnifiedIdeograph846e',
u'\u68f1': 'CjkUnifiedIdeograph68f1',
u'\uc970': 'HangulSyllableJyun',
u'\uadf3': 'HangulSyllableGyuc',
u'\ueef5': 'UnicodeCharacter0xeef5',
u'\u167a': 'UnicodeCharacter0x167a',
u'\u5b7c': 'CjkUnifiedIdeograph5b7c',
u'\u3fff': 'CjkUnifiedIdeograph3fff',
u'\u9c7e': 'CjkUnifiedIdeograph9c7e',
u'\ua180': 'YiSyllableNip',
u'\u0503': 'CyrillicSmallLetterKomiDje',
u'\ue282': 'UnicodeCharacter0xe282',
u'\u4605': 'CjkUnifiedIdeograph4605',
u'\u8b07': 'CjkUnifiedIdeograph8b07',
u'\ucc09': 'HangulSyllableJjyit',
u'\u338c': 'SquareMuF',
u'\u748e': 'CjkUnifiedIdeograph748e',
u'\ub990': 'HangulSyllableRyi',
u'\u1d13': 'LatinSmallLetterSidewaysOWithStroke',
u'\ufa92': 'CjkCompatibilityIdeographfa92',
u'\u5e15': 'CjkUnifiedIdeograph5e15',
u'\u6317': 'CjkUnifiedIdeograph6317',
u'\ua419': 'YiSyllableJjix',
u'\ue91b': 'UnicodeCharacter0xe91b',
u'\u0b9c': 'TamilLetterJa',
u'\u4c9e': 'CjkUnifiedIdeograph4c9e',
u'\u91a0': 'CjkUnifiedIdeograph91a0',
u'\ud2a2': 'HangulSyllableTyunh',
u'\u3625': 'CjkUnifiedIdeograph3625',
u'\u7b27': 'CjkUnifiedIdeograph7b27',
u'\ubc29': 'HangulSyllableBang',
u'\uc12b': 'HangulSyllableSeolh',
u'\u24ae': 'ParenthesizedLatinSmallLetterS',
u'\u69b0': 'CjkUnifiedIdeograph69b0',
u'\uaab2': 'UnicodeCharacter0xaab2',
u'\u0e35': 'ThaiCharacterSaraIi',
u'\uefb4': 'UnicodeCharacter0xefb4',
u'\u5337': 'CjkUnifiedIdeograph5337',
u'\u9439': 'CjkUnifiedIdeograph9439',
u'\ud93b': 'UnicodeCharacter0xd93b',
u'\u3cbe': 'CjkUnifiedIdeograph3cbe',
u'\u41c0': 'CjkUnifiedIdeograph41c0',
u'\u82c2': 'CjkUnifiedIdeograph82c2',
u'\uc7c4': 'HangulSyllableJaek',
u'\u2b47': 'ReverseTildeOperatorAboveRightwardsArrow',
u'\u6c49': 'CjkUnifiedIdeograph6c49',
u'\ub14b': 'HangulSyllableNyeolb',
u'\uf24d': 'UnicodeCharacter0xf24d',
u'\u14ce': 'CanadianSyllabicsWestcreeNwaa',
u'\u59d0': 'CjkUnifiedIdeograph59d0',
u'\u9ad2': 'CjkUnifiedIdeograph9ad2',
u'\udfd4': 'UnicodeCharacter0xdfd4',
u'\u0357': 'CombiningRightHalfRingAbove',
u'\ue0d6': 'UnicodeCharacter0xe0d6',
u'\u4459': 'CjkUnifiedIdeograph4459',
u'\u895b': 'CjkUnifiedIdeograph895b',
u'\uca5d': 'HangulSyllableJjeob',
u'\u31e0': 'CjkStrokeHxwg',
u'\u72e2': 'CjkUnifiedIdeograph72e2',
u'\ub7e4': 'HangulSyllableRyaess',
u'\u1b67': 'BalineseMusicalSymbolDaeng',
u'\uf8e6': 'UnicodeCharacter0xf8e6',
u'\u5c69': 'CjkUnifiedIdeograph5c69',
u'\u616b': 'CjkUnifiedIdeograph616b',
u'\ua26d': 'YiSyllableNgep',
u'\ue76f': 'UnicodeCharacter0xe76f',
u'\u09f0': 'BengaliLetterRaWithMiddleDiagonal',
u'\u4af2': 'CjkUnifiedIdeograph4af2',
u'\u8ff4': 'CjkUnifiedIdeograph8ff4',
u'\ud0f6': 'HangulSyllableTaep',
u'\u3479': 'CjkUnifiedIdeograph3479',
u'\u797b': 'CjkUnifiedIdeograph797b',
u'\uba7d': 'HangulSyllableMyeolt',
u'\uff7f': 'HalfwidthKatakanaLetterSo',
u'\u8381': 'CjkUnifiedIdeograph8381',
u'\u6404': 'CjkUnifiedIdeograph6404',
u'\u4887': 'CjkUnifiedIdeograph4887',
u'\uee08': 'UnicodeCharacter0xee08',
u'\ud28b': 'HangulSyllableTwilb',
u'\u2d0a': 'GeorgianSmallLetterLas',
u'\u118d': 'HangulJungseongUu',
u'\ub70e': 'HangulSyllableDdyugg',
u'\u9b91': 'CjkUnifiedIdeograph9b91',
u'\u7c14': 'CjkUnifiedIdeograph7c14',
u'\u2097': 'UnicodeCharacter0x2097',
u'\uc618': 'HangulSyllableYem',
u'\uaa9b': 'UnicodeCharacter0xaa9b',
u'\u051a': 'CyrillicCapitalLetterQa',
u'\u8f1e': 'CjkUnifiedIdeograph8f1e',
u'\u73a1': 'CjkUnifiedIdeograph73a1',
u'\uf5a5': 'UnicodeCharacter0xf5a5',
u'\u5424': 'CjkUnifiedIdeograph5424',
u'\u38a7': 'CjkUnifiedIdeograph38a7',
u'\ude28': 'UnicodeCharacter0xde28',
u'\u82ab': 'CjkUnifiedIdeograph82ab',
u'\u1d2a': 'GreekLetterSmallCapitalPsi',
u'\u672e': 'CjkUnifiedIdeograph672e',
u'\u4bb1': 'CjkUnifiedIdeograph4bb1',
u'\ue932': 'UnicodeCharacter0xe932',
u'\ucdb5': 'HangulSyllableCweonj',
u'\u2c34': 'GlagoliticSmallLetterDobro',
u'\u10b7': 'GeorgianCapitalLetterQar',
u'\ub638': 'HangulSyllableDdwals',
u'\u9abb': 'CjkUnifiedIdeograph9abb',
u'\u7f3e': 'CjkUnifiedIdeograph7f3e',
u'\u23c1': 'DentistrySymbolLightDownAndHorizontalWithCircle',
u'\uc142': 'HangulSyllableSelm',
u'\ua5c5': 'VaiSyllableKpo',
u'\u0444': 'CyrillicSmallLetterEf',
u'\u8e48': 'CjkUnifiedIdeograph8e48',
u'\u72cb': 'CjkUnifiedIdeograph72cb',
u'\uf4cf': 'UnicodeCharacter0xf4cf',
u'\u574e': 'CjkUnifiedIdeograph574e',
u'\u3bd1': 'CjkUnifiedIdeograph3bd1',
u'\ud952': 'UnicodeCharacter0xd952',
u'\ubdd5': 'HangulSyllableBwig',
u'\u1c54': 'OlChikiDigitFour',
u'\u6658': 'CjkUnifiedIdeograph6658',
u'\u4adb': 'CjkUnifiedIdeograph4adb',
u'\ue85c': 'UnicodeCharacter0xe85c',
u'\uccdf': 'HangulSyllableCyeolh',
u'\u2f5e': 'KangxiRadicalProfound',
u'\u13e1': 'CherokeeLetterTlu',
u'\ub162': 'HangulSyllableNyenh',
u'\u95e5': 'CjkUnifiedIdeograph95e5',
u'\u7e68': 'CjkUnifiedIdeograph7e68',
u'\u22eb': 'DoesNotContainAsNormalSubgroup',
u'\uc06c': 'HangulSyllableBbeuss',
u'\ua4ef': 'UnicodeCharacter0xa4ef',
u'\u076e': 'ArabicLetterHahWithSmallArabicLetterTahBelow',
u'\u8972': 'CjkUnifiedIdeograph8972',
u'\u6df5': 'CjkUnifiedIdeograph6df5',
u'\uf7f9': 'UnicodeCharacter0xf7f9',
u'\u5678': 'CjkUnifiedIdeograph5678',
u'\u3afb': 'CjkUnifiedIdeograph3afb',
u'\ud87c': 'UnicodeCharacter0xd87c',
u'\ubcff': 'HangulSyllableBolb',
u'\u1f7e': 'UnicodeCharacter0x1f7e',
u'\u8603': 'CjkUnifiedIdeograph8603',
u'\u6182': 'CjkUnifiedIdeograph6182',
u'\ueb86': 'UnicodeCharacter0xeb86',
u'\u4f09': 'CjkUnifiedIdeograph4f09',
u'\u2e88': 'CjkRadicalKnifeOne',
u'\ud10d': 'HangulSyllableTyang',
u'\ub08c': 'HangulSyllableGgim',
u'\u140f': 'CanadianSyllabicsWestcreeWi',
u'\u9e13': 'CjkUnifiedIdeograph9e13',
u'\u7992': 'CjkUnifiedIdeograph7992',
u'\ue017': 'UnicodeCharacter0xe017',
u'\uc396': 'HangulSyllableSsebs',
u'\u2719': 'OutlinedGreekCross',
u'\u0698': 'ArabicLetterJeh',
u'\ua91d': 'KayahLiLetterWa',
u'\u889c': 'CjkUnifiedIdeograph889c',
u'\u7623': 'CjkUnifiedIdeograph7623',
u'\u51a2': 'CjkUnifiedIdeograph51a2',
u'\uf827': 'UnicodeCharacter0xf827',
u'\udba6': 'UnicodeCharacter0xdba6',
u'\u3f29': 'CjkUnifiedIdeograph3f29',
u'\u1ea8': 'LatinCapitalLetterAWithCircumflexAndHookAbove',
u'\u812d': 'CjkUnifiedIdeograph812d',
u'\u60ac': 'CjkUnifiedIdeograph60ac',
u'\ueab0': 'UnicodeCharacter0xeab0',
u'\u4e33': 'CjkUnifiedIdeograph4e33',
u'\u29b2': 'EmptySetWithSmallCircleAbove',
u'\ud037': 'HangulSyllableKwigs',
u'\ub3b6': 'HangulSyllableDyelp',
u'\u1739': 'UnicodeCharacter0x1739',
u'\u993d': 'CjkUnifiedIdeograph993d',
u'\u78bc': 'CjkUnifiedIdeograph78bc',
u'\ue341': 'UnicodeCharacter0xe341',
u'\uc2c0': 'HangulSyllableSyi',
u'\u2643': 'Jupiter',
u'\u01c2': 'LatinLetterAlveolarClick',
u'\ua847': 'PhagspaLetterNya',
u'\u8bc6': 'CjkUnifiedIdeograph8bc6',
u'\u714d': 'CjkUnifiedIdeograph714d',
u'\u50cc': 'CjkUnifiedIdeograph50cc',
u'\ufb51': 'ArabicLetterAlefWaslaFinalForm',
u'\udad0': 'UnicodeCharacter0xdad0',
u'\u3e53': 'CjkUnifiedIdeograph3e53',
u'\u19d2': 'NewTaiLueDigitTwo',
u'\u8057': 'CjkUnifiedIdeograph8057',
u'\u63d6': 'CjkUnifiedIdeograph63d6',
u'\ue5da': 'UnicodeCharacter0xe5da',
u'\u495d': 'CjkUnifiedIdeograph495d',
u'\u28dc': 'BraillePatternDots34578',
u'\ud361': 'HangulSyllablePyaeg',
u'\ub2e0': 'HangulSyllableNik',
u'\u1663': 'CanadianSyllabicsCarrierChe',
u'\u9867': 'CjkUnifiedIdeograph9867',
u'\u7be6': 'CjkUnifiedIdeograph7be6',
u'\ue26b': 'UnicodeCharacter0xe26b',
u'\ufdea': 'UnicodeCharacter0xfdea',
u'\u216d': 'RomanNumeralOneHundred',
u'\xec': 'LatinSmallLetterIWithGrave',
u'\uab71': 'UnicodeCharacter0xab71',
u'\u8af0': 'CjkUnifiedIdeograph8af0',
u'\u7077': 'CjkUnifiedIdeograph7077',
u'\u53f6': 'CjkUnifiedIdeograph53f6',
u'\ufa7b': 'CjkCompatibilityIdeographfa7b',
u'\ud5fa': 'HangulSyllableHej',
u'\u397d': 'CjkUnifiedIdeograph397d',
u'\u18fc': 'UnicodeCharacter0x18fc',
u'\u0281': 'LatinLetterSmallCapitalInvertedR',
u'\u6300': 'CjkUnifiedIdeograph6300',
u'\u4783': 'CjkUnifiedIdeograph4783',
u'\ua002': 'YiSyllableI',
u'\u8485': 'CjkUnifiedIdeograph8485',
u'\ue504': 'UnicodeCharacter0xe504',
u'\uc987': 'HangulSyllableJyuh',
u'\u360e': 'CjkUnifiedIdeograph360e',
u'\u1a91': 'UnicodeCharacter0x1a91',
u'\u7b10': 'CjkUnifiedIdeograph7b10',
u'\u5f93': 'CjkUnifiedIdeograph5f93',
u'\ub812': 'HangulSyllableRelm',
u'\u9c95': 'CjkUnifiedIdeograph9c95',
u'\ufd14': 'ArabicLigatureAinWithYehFinalForm',
u'\ua197': 'YiSyllableHlit',
u'\ue699': 'UnicodeCharacter0xe699',
u'\u0e1e': 'ThaiCharacterPhoPhan',
u'\u5320': 'CjkUnifiedIdeograph5320',
u'\u37a3': 'CjkUnifiedIdeograph37a3',
u'\u9022': 'CjkUnifiedIdeograph9022',
u'\u74a5': 'CjkUnifiedIdeograph74a5',
u'\ud524': 'HangulSyllablePyin',
u'\ub9a7': 'HangulSyllableRyic',
u'\ufea9': 'ArabicLetterDalIsolatedForm',
u'\u2b30': 'LeftArrowWithSmallCircle',
u'\u0fb3': 'TibetanSubjoinedLetterLa',
u'\u6832': 'CjkUnifiedIdeograph6832',
u'\u4cb5': 'CjkUnifiedIdeograph4cb5',
u'\uad34': 'HangulSyllableGoe',
u'\u91b7': 'CjkUnifiedIdeograph91b7',
u'\uf236': 'UnicodeCharacter0xf236',
u'\ud6b9': 'HangulSyllableHyob',
u'\u0340': 'CombiningGraveToneMark',
u'\u4042': 'CjkUnifiedIdeograph4042',
u'\u24c5': 'CircledLatinCapitalLetterP',
u'\u8544': 'CjkUnifiedIdeograph8544',
u'\u69c7': 'CjkUnifiedIdeograph69c7',
u'\uca46': 'HangulSyllableJjyaej',
u'\uaec9': 'HangulSyllableGgeolt',
u'\uf3cb': 'UnicodeCharacter0xf3cb',
u'\u1b50': 'BalineseDigitZero',
u'\u5852': 'CjkUnifiedIdeograph5852',
u'\u3cd5': 'CjkUnifiedIdeograph3cd5',
u'\u9d54': 'CjkUnifiedIdeograph9d54',
u'\u41d7': 'CjkUnifiedIdeograph41d7',
u'\ua256': 'YiSyllableHxot',
u'\u86d9': 'CjkUnifiedIdeograph86d9',
u'\ue758': 'UnicodeCharacter0xe758',
u'\ucbdb': 'HangulSyllableJjeud',
u'\u3062': 'HiraganaLetterDi',
u'\u14e5': 'CanadianSyllabicsWestcreeLwoo',
u'\u7564': 'CjkUnifiedIdeograph7564',
u'\u59e7': 'CjkUnifiedIdeograph59e7',
u'\uba66': 'HangulSyllableMebs',
u'\u9ee9': 'CjkUnifiedIdeograph9ee9',
u'\uff68': 'HalfwidthKatakanaLetterSmallI',
u'\ua3eb': 'YiSyllableJuop',
u'\ue0ed': 'UnicodeCharacter0xe0ed',
u'\u0872': 'UnicodeCharacter0x872',
u'\u4d74': 'CjkUnifiedIdeograph4d74',
u'\u31f7': 'KatakanaLetterSmallHu',
u'\u9276': 'CjkUnifiedIdeograph9276',
u'\u76f9': 'CjkUnifiedIdeograph76f9',
u'\ud778': 'HangulSyllableHyils',
u'\ubbfb': 'HangulSyllableMigs',
u'\uf8fd': 'UnicodeCharacter0xf8fd',
u'\uc201': 'HangulSyllableSyonj',
u'\u2584': 'LowerHalfBlock',
u'\u6a86': 'CjkUnifiedIdeograph6a86',
u'\uaf88': 'HangulSyllableGgoel',
u'\u130b': 'EthiopicSyllableGaa',
u'\uec8a': 'UnicodeCharacter0xec8a',
u'\u500d': 'CjkUnifiedIdeograph500d',
u'\u950f': 'CjkUnifiedIdeograph950f',
u'\uda11': 'UnicodeCharacter0xda11',
u'\u3d94': 'CjkUnifiedIdeograph3d94',
u'\u4296': 'CjkUnifiedIdeograph4296',
u'\u8798': 'CjkUnifiedIdeograph8798',
u'\uc49a': 'HangulSyllableSsweop',
u'\u281d': 'BraillePatternDots1345',
u'\u6d1f': 'CjkUnifiedIdeograph6d1f',
u'\ub221': 'HangulSyllableNweog',
u'\uf723': 'UnicodeCharacter0xf723',
u'\u15a4': 'CanadianSyllabicsLha',
u'\u5aa6': 'CjkUnifiedIdeograph5aa6',
u'\u9fa8': 'CjkUnifiedIdeograph9fa8',
u'\udcaa': 'UnicodeCharacter0xdcaa',
u'\ue1ac': 'UnicodeCharacter0xe1ac',
u'\u452f': 'CjkUnifiedIdeograph452f',
u'\u8a31': 'CjkUnifiedIdeograph8a31',
u'\ucf33': 'HangulSyllableKyeoc',
u'\u32b6': 'CircledNumberFortyOne',
u'\u77b8': 'CjkUnifiedIdeograph77b8',
u'\ub4ba': 'HangulSyllableDwij',
u'\u183d': 'MongolianLetterZa',
u'\uf9bc': 'CjkCompatibilityIdeographf9bc',
u'\u5d3f': 'CjkUnifiedIdeograph5d3f',
u'\u6241': 'CjkUnifiedIdeograph6241',
u'\ua743': 'LatinSmallLetterKWithDiagonalStroke',
u'\ue445': 'UnicodeCharacter0xe445',
u'\u0ac6': 'UnicodeCharacter0xac6',
u'\u4fc8': 'CjkUnifiedIdeograph4fc8',
u'\u8cca': 'CjkUnifiedIdeograph8cca',
u'\ud1cc': 'HangulSyllableTwam',
u'\u354f': 'CjkUnifiedIdeograph354f',
u'\u7a51': 'CjkUnifiedIdeograph7a51',
u'\ubf53': 'HangulSyllableBbos',
u'\ufc55': 'ArabicLigatureYehWithJeemIsolatedForm',
u'\u27d8': 'LargeUpTack',
u'\u64da': 'CjkUnifiedIdeograph64da',
u'\ua9dc': 'UnicodeCharacter0xa9dc',
u'\u0d5f': 'UnicodeCharacter0xd5f',
u'\ueede': 'UnicodeCharacter0xeede',
u'\u5261': 'CjkUnifiedIdeograph5261',
u'\u9763': 'CjkUnifiedIdeograph9763',
u'\ud465': 'HangulSyllablePyolg',
u'\u3fe8': 'CjkUnifiedIdeograph3fe8',
u'\u7cea': 'CjkUnifiedIdeograph7cea',
u'\u81ec': 'CjkUnifiedIdeograph81ec',
u'\uc6ee': 'HangulSyllableWenh',
u'\u2a71': 'EqualsSignAbovePlusSign',
u'\u6f73': 'CjkUnifiedIdeograph6f73',
u'\uac75': 'HangulSyllableGeonj',
u'\uf177': 'UnicodeCharacter0xf177',
u'\u17f8': 'KhmerSymbolLekAttakPrambei',
u'\u54fa': 'CjkUnifiedIdeograph54fa',
u'\u99fc': 'CjkUnifiedIdeograph99fc',
u'\udefe': 'UnicodeCharacter0xdefe',
u'\ue400': 'UnicodeCharacter0xe400',
u'\uc083': 'HangulSyllableBbyilh',
u'\u2702': 'BlackScissors',
u'\u0385': 'GreekDialytikaTonos',
u'\uad06': 'HangulSyllableGwalm',
u'\u8989': 'CjkUnifiedIdeograph8989',
u'\u760c': 'CjkUnifiedIdeograph760c',
u'\u528f': 'CjkUnifiedIdeograph528f',
u'\ufc10': 'ArabicLigatureTehWithYehIsolatedForm',
u'\ud893': 'UnicodeCharacter0xd893',
u'\u3f12': 'CjkUnifiedIdeograph3f12',
u'\u1b95': 'SundaneseLetterPa',
u'\u8516': 'CjkUnifiedIdeograph8516',
u'\u6199': 'CjkUnifiedIdeograph6199',
u'\uef9d': 'UnicodeCharacter0xef9d',
u'\u4e1c': 'CjkUnifiedIdeograph4e1c',
u'\u2a9f': 'SimilarAboveLessthanAboveEqualsSign',
u'\ud420': 'HangulSyllablePwak',
u'\ub0a3': 'HangulSyllableNalb',
u'\u1722': 'HanunooLetterU',
u'\u9d26': 'CjkUnifiedIdeograph9d26',
u'\u79a9': 'CjkUnifiedIdeograph79a9',
u'\uc7ad': 'HangulSyllableJaeg',
u'\u262c': 'AdiShakti',
u'\u02af': 'LatinSmallLetterTurnedHWithFishhookAndTail',
u'\uac30': 'HangulSyllableGaess',
u'\u88b3': 'CjkUnifiedIdeograph88b3',
u'\u7536': 'CjkUnifiedIdeograph7536',
u'\u51b9': 'CjkUnifiedIdeograph51b9',
u'\uf33a': 'UnicodeCharacter0xf33a',
u'\udfbd': 'UnicodeCharacter0xdfbd',
u'\u3e3c': 'CjkUnifiedIdeograph3e3c',
u'\u1abf': 'UnicodeCharacter0x1abf',
u'\u8440': 'CjkUnifiedIdeograph8440',
u'\u60c3': 'CjkUnifiedIdeograph60c3',
u'\ueec7': 'UnicodeCharacter0xeec7',
u'\u4d46': 'CjkUnifiedIdeograph4d46',
u'\u29c9': 'TwoJoinedSquares',
u'\ucb4a': 'HangulSyllableJjugg',
u'\ub7cd': 'HangulSyllableRyat',
u'\u164c': 'CanadianSyllabicsCarrierDzi',
u'\u9c50': 'CjkUnifiedIdeograph9c50',
u'\u78d3': 'CjkUnifiedIdeograph78d3',
u'\uc6d7': 'HangulSyllableWeolb',
u'\u2556': 'BoxDrawingsDownDoubleAndLeftSingle',
u'\u01d9': 'LatinCapitalLetterUWithDiaeresisAndCaron',
u'\ua35a': 'YiSyllableZhur',
u'\u8fdd': 'CjkUnifiedIdeograph8fdd',
u'\u7460': 'CjkUnifiedIdeograph7460',
u'\u50e3': 'CjkUnifiedIdeograph50e3',
u'\uf264': 'UnicodeCharacter0xf264',
u'\udee7': 'UnicodeCharacter0xdee7',
u'\u3d66': 'CjkUnifiedIdeograph3d66',
u'\u19e9': 'KhmerSymbolPrambuonKoet',
u'\ubb6a': 'HangulSyllableMweop',
u'\u67ed': 'CjkUnifiedIdeograph67ed',
u'\uedf1': 'UnicodeCharacter0xedf1',
u'\u4c70': 'CjkUnifiedIdeograph4c70',
u'\u28f3': 'BraillePatternDots125678',
u'\uca74': 'HangulSyllableJjels',
u'\ub6f7': 'HangulSyllableDdwid',
u'\u1576': 'CanadianSyllabicsNunavikHii',
u'\u937a': 'CjkUnifiedIdeograph937a',
u'\u7ffd': 'CjkUnifiedIdeograph7ffd',
u'\u4501': 'CjkUnifiedIdeograph4501',
u'\u2480': 'ParenthesizedNumberThirteen',
u'\uc305': 'HangulSyllableSsalt',
u'\ua284': 'YiSyllableWuox',
u'\u0e07': 'ThaiCharacterNgoNgu',
u'\u940b': 'CjkUnifiedIdeograph940b',
u'\u6b8a': 'CjkUnifiedIdeograph6b8a',
u'\uf18e': 'UnicodeCharacter0xf18e',
u'\u5d11': 'CjkUnifiedIdeograph5d11',
u'\u3c90': 'CjkUnifiedIdeograph3c90',
u'\udb15': 'UnicodeCharacter0xdb15',
u'\uba94': 'HangulSyllableMyel',
u'\u6c1b': 'CjkUnifiedIdeograph6c1b',
u'\u439a': 'CjkUnifiedIdeograph439a',
u'\uea1f': 'UnicodeCharacter0xea1f',
u'\uc99e': 'HangulSyllableJeuj',
u'\u3521': 'CjkUnifiedIdeograph3521',
u'\u14a0': 'CanadianSyllabicsNaskapiCwaa',
u'\ub325': 'HangulSyllableDyalg',
u'\u92a4': 'CjkUnifiedIdeograph92a4',
u'\u442b': 'CjkUnifiedIdeograph442b',
u'\u5baa': 'CjkUnifiedIdeograph5baa',
u'\uc22f': 'HangulSyllableSuc',
u'\ua1ae': 'YiSyllableHlup',
u'\u0d31': 'MalayalamLetterRra',
u'\u8b35': 'CjkUnifiedIdeograph8b35',
u'\u6ab4': 'CjkUnifiedIdeograph6ab4',
u'\uf0b8': 'UnicodeCharacter0xf0b8',
u'\u5c3b': 'CjkUnifiedIdeograph5c3b',
u'\u33ba': 'SquarePw',
u'\uda3f': 'UnicodeCharacter0xda3f',
u'\ub9be': 'HangulSyllableRibs',
u'\u6345': 'CjkUnifiedIdeograph6345',
u'\u42c4': 'CjkUnifiedIdeograph42c4',
u'\ue949': 'UnicodeCharacter0xe949',
u'\uc8c8': 'HangulSyllableJoen',
u'\u344b': 'CjkUnifiedIdeograph344b',
u'\u0bca': 'TamilVowelSignO',
u'\ub24f': 'HangulSyllableNwes',
u'\u91ce': 'CjkUnifiedIdeograph91ce',
u'\u7b55': 'CjkUnifiedIdeograph7b55',
u'\u5ad4': 'CjkUnifiedIdeograph5ad4',
u'\uc159': 'HangulSyllableSyeonj',
u'\ua0d8': 'YiSyllableFut',
u'\u0c5b': 'UnicodeCharacter0xc5b',
u'\u8a5f': 'CjkUnifiedIdeograph8a5f',
u'\u69de': 'CjkUnifiedIdeograph69de',
u'\uf7e2': 'UnicodeCharacter0xf7e2',
u'\u5365': 'CjkUnifiedIdeograph5365',
u'\u32e4': 'CircledKatakanaNa',
u'\ud969': 'UnicodeCharacter0xd969',
u'\ub8e8': 'HangulSyllableRu',
u'\u626f': 'CjkUnifiedIdeograph626f',
u'\u41ee': 'CjkUnifiedIdeograph41ee',
u'\ue873': 'UnicodeCharacter0xe873',
u'\ucff2': 'HangulSyllableKubs',
u'\u2b75': 'UnicodeCharacter0x2b75',
u'\u0af4': 'UnicodeCharacter0xaf4',
u'\ub179': 'HangulSyllableNog',
u'\u90f8': 'CjkUnifiedIdeograph90f8',
u'\u7a7f': 'CjkUnifiedIdeograph7a7f',
u'\u59fe': 'CjkUnifiedIdeograph59fe',
u'\u2c06': 'GlagoliticCapitalLetterZhivete',
u'\u0889': 'UnicodeCharacter0x889',
u'\u6908': 'CjkUnifiedIdeograph6908',
u'\u558b': 'CjkUnifiedIdeograph558b',
u'\uaa0a': 'ChamLetterNgue',
u'\u968d': 'CjkUnifiedIdeograph968d',
u'\uf70c': 'UnicodeCharacter0xf70c',
u'\ud38f': 'HangulSyllablePeos',
u'\u0416': 'CyrillicCapitalLetterZhe',
u'\u4118': 'CjkUnifiedIdeograph4118',
u'\u2d9b': 'UnicodeCharacter0x2d9b',
u'\u821a': 'CjkUnifiedIdeograph821a',
u'\u6e9d': 'CjkUnifiedIdeograph6e9d',
u'\ucf1c': 'HangulSyllableKyeo',
u'\uab9f': 'UnicodeCharacter0xab9f',
u'\uf4a1': 'UnicodeCharacter0xf4a1',
u'\u1c26': 'LepchaVowelSignAa',
u'\u5928': 'CjkUnifiedIdeograph5928',
u'\u05ab': 'HebrewAccentOle',
u'\u9a2a': 'CjkUnifiedIdeograph9a2a',
u'\u46ad': 'CjkUnifiedIdeograph46ad',
u'\ua72c': 'LatinCapitalLetterCuatrillo',
u'\u83af': 'CjkUnifiedIdeograph83af',
u'\ue02e': 'UnicodeCharacter0xe02e',
u'\uccb1': 'HangulSyllableCeot',
u'\u3138': 'HangulLetterSsangtikeut',
u'\u1dbb': 'ModifierLetterSmallZ',
u'\u723a': 'CjkUnifiedIdeograph723a',
u'\u5ebd': 'CjkUnifiedIdeograph5ebd',
u'\ubf3c': 'HangulSyllableBbyek',
u'\u9bbf': 'CjkUnifiedIdeograph9bbf',
u'\uf83e': 'UnicodeCharacter0xf83e',
u'\ua4c1': 'YiRadicalZur',
u'\ue1c3': 'UnicodeCharacter0xe1c3',
u'\u0948': 'DevanagariVowelSignAi',
u'\u4a4a': 'CjkUnifiedIdeograph4a4a',
u'\u36cd': 'CjkUnifiedIdeograph36cd',
u'\u974c': 'CjkUnifiedIdeograph974c',
u'\u73cf': 'CjkUnifiedIdeograph73cf',
u'\ud04e': 'HangulSyllableKwip',
u'\ubcd1': 'HangulSyllableByeong',
u'\uf9d3': 'CjkCompatibilityIdeographf9d3',
u'\u225a': 'EquiangularTo',
u'\u0edd': 'LaoHoMo',
u'\u6f5c': 'CjkUnifiedIdeograph6f5c',
u'\u4bdf': 'CjkUnifiedIdeograph4bdf',
u'\ua85e': 'PhagspaLetterI',
u'\u94e1': 'CjkUnifiedIdeograph94e1',
u'\uf560': 'UnicodeCharacter0xf560',
u'\ud1e3': 'HangulSyllableTwaelb',
u'\u3a6a': 'CjkUnifiedIdeograph3a6a',
u'\u476c': 'CjkUnifiedIdeograph476c',
u'\u23ef': 'UnicodeCharacter0x23ef',
u'\u806e': 'CjkUnifiedIdeograph806e',
u'\u6cf1': 'CjkUnifiedIdeograph6cf1',
u'\ucd70': 'HangulSyllableCoess',
u'\ua9f3': 'UnicodeCharacter0xa9f3',
u'\ueaf5': 'UnicodeCharacter0xeaf5',
u'\u127a': 'EthiopicSyllableCi',
u'\u5f7c': 'CjkUnifiedIdeograph5f7c',
u'\u3bff': 'CjkUnifiedIdeograph3bff',
u'\u987e': 'CjkUnifiedIdeograph987e',
u'\ua580': 'VaiSyllableThoo',
u'\u0103': 'LatinSmallLetterAWithBreve',
u'\ue682': 'UnicodeCharacter0xe682',
u'\u4205': 'CjkUnifiedIdeograph4205',
u'\u8f07': 'CjkUnifiedIdeograph8f07',
u'\uc809': 'HangulSyllableJeolg',
u'\u378c': 'CjkUnifiedIdeograph378c',
u'\u708e': 'CjkUnifiedIdeograph708e',
u'\ubd90': 'HangulSyllableBum',
u'\u1913': 'LimbuLetterBha',
u'\ufe92': 'ArabicLetterBehMedialForm',
u'\u5a15': 'CjkUnifiedIdeograph5a15',
u'\u6717': 'CjkUnifiedIdeograph6717',
u'\ua019': 'YiSyllableBip',
u'\ued1b': 'UnicodeCharacter0xed1b',
u'\u0f9c': 'TibetanSubjoinedLetterDda',
u'\u489e': 'CjkUnifiedIdeograph489e',
u'\u95a0': 'CjkUnifiedIdeograph95a0',
u'\ud6a2': 'HangulSyllableHoej',
u'\u3225': 'ParenthesizedIdeographSix',
u'\u7f27': 'CjkUnifiedIdeograph7f27',
u'\ub829': 'HangulSyllableRyeonj',
u'\uc52b': 'HangulSyllableSsigs',
u'\u20ae': 'TugrikSign',
u'\u6db0': 'CjkUnifiedIdeograph6db0',
u'\uaeb2': 'HangulSyllableGgyaebs',
u'\u0a35': 'GurmukhiLetterVa',
u'\uebb4': 'UnicodeCharacter0xebb4',
u'\u5737': 'CjkUnifiedIdeograph5737',
u'\u9039': 'CjkUnifiedIdeograph9039',
u'\udd3b': 'UnicodeCharacter0xdd3b',
u'\u38be': 'CjkUnifiedIdeograph38be',
u'\u45c0': 'CjkUnifiedIdeograph45c0',
u'\u86c2': 'CjkUnifiedIdeograph86c2',
u'\uc3c4': 'HangulSyllableSsyel',
u'\u2f47': 'KangxiRadicalSun',
u'\u6849': 'CjkUnifiedIdeograph6849',
u'\ub54b': 'HangulSyllableDdah',
u'\uf64d': 'UnicodeCharacter0xf64d',
u'\u10ce': 'UnicodeCharacter0x10ce',
u'\u5dd0': 'CjkUnifiedIdeograph5dd0',
u'\u9ed2': 'CjkUnifiedIdeograph9ed2',
u'\udbd4': 'UnicodeCharacter0xdbd4',
u'\u0757': 'ArabicLetterHahWithTwoDotsAbove',
u'\ue4d6': 'UnicodeCharacter0xe4d6',
u'\u4059': 'CjkUnifiedIdeograph4059',
u'\u8d5b': 'CjkUnifiedIdeograph8d5b',
u'\uce5d': 'HangulSyllableCinj',
u'\u35e0': 'CjkUnifiedIdeograph35e0',
u'\u76e2': 'CjkUnifiedIdeograph76e2',
u'\ub3e4': 'HangulSyllableDwan',
u'\u1f67': 'GreekSmallLetterOmegaWithDasiaAndPerispomeni',
u'\ufce6': 'ArabicLigatureThehWithHehMedialForm',
u'\u5869': 'CjkUnifiedIdeograph5869',
u'\u656b': 'CjkUnifiedIdeograph656b',
u'\ua66d': 'CyrillicSmallLetterDoubleMonocularO',
u'\ue36f': 'UnicodeCharacter0xe36f',
u'\u0df0': 'UnicodeCharacter0xdf0',
u'\u4ef2': 'CjkUnifiedIdeograph4ef2',
u'\u8bf4': 'CjkUnifiedIdeograph8bf4',
u'\ud4f6': 'HangulSyllablePyulp',
u'\u3079': 'HiraganaLetterBe',
u'\u7d7b': 'CjkUnifiedIdeograph7d7b',
u'\ube7d': 'HangulSyllableBbaeg',
u'\ufb7f': 'ArabicLetterTchehehFinalForm',
u'\u8781': 'CjkUnifiedIdeograph8781',
u'\u6004': 'CjkUnifiedIdeograph6004',
u'\u4c87': 'CjkUnifiedIdeograph4c87',
u'\uea08': 'UnicodeCharacter0xea08',
u'\ud68b': 'HangulSyllableHwaeh',
u'\u290a': 'UpwardsTripleArrow',
u'\u158d': 'CanadianSyllabicsWestcreeRa',
u'\ub30e': 'HangulSyllableDaelp',
u'\u9f91': 'CjkUnifiedIdeograph9f91',
u'\u7814': 'CjkUnifiedIdeograph7814',
u'\u2497': 'NumberSixteenFullStop',
u'\uc218': 'HangulSyllableSu',
u'\uae9b': 'HangulSyllableGgyac',
u'\u011a': 'LatinCapitalLetterEWithCaron',
u'\u8b1e': 'CjkUnifiedIdeograph8b1e',
u'\u77a1': 'CjkUnifiedIdeograph77a1',
u'\uf1a5': 'UnicodeCharacter0xf1a5',
u'\u5024': 'CjkUnifiedIdeograph5024',
u'\u3ca7': 'CjkUnifiedIdeograph3ca7',
u'\uda28': 'UnicodeCharacter0xda28',
u'\u86ab': 'CjkUnifiedIdeograph86ab',
u'\u192a': 'LimbuSubjoinedLetterRa',
u'\u632e': 'CjkUnifiedIdeograph632e',
u'\u4fb1': 'CjkUnifiedIdeograph4fb1',
u'\ued32': 'UnicodeCharacter0xed32',
u'\uc9b5': 'HangulSyllableJyib',
u'\u2834': 'BraillePatternDots356',
u'\u14b7': 'CanadianSyllabicsWestcreeMwa',
u'\ub238': 'HangulSyllableNweok',
u'\u9ebb': 'CjkUnifiedIdeograph9ebb',
u'\u7b3e': 'CjkUnifiedIdeograph7b3e',
u'\u27c1': 'WhiteTriangleContainingSmallWhiteTriangle',
u'\uc542': 'HangulSyllableSsip',
u'\ua1c5': 'YiSyllableLuo',
u'\u8a48': 'CjkUnifiedIdeograph8a48',
u'\u76cb': 'CjkUnifiedIdeograph76cb',
u'\uf0cf': 'UnicodeCharacter0xf0cf',
u'\u534e': 'CjkUnifiedIdeograph534e',
u'\u3fd1': 'CjkUnifiedIdeograph3fd1',
u'\udd52': 'UnicodeCharacter0xdd52',
u'\ub9d5': 'HangulSyllableMalt',
u'\u1854': 'MongolianLetterTodoTsa',
u'\u6258': 'CjkUnifiedIdeograph6258',
u'\u4edb': 'CjkUnifiedIdeograph4edb',
u'\uec5c': 'UnicodeCharacter0xec5c',
u'\uc8df': 'HangulSyllableJoeh',
u'\u2b5e': 'UnicodeCharacter0x2b5e',
u'\u17e1': 'KhmerDigitOne',
u'\ub562': 'HangulSyllableDdaej',
u'\u91e5': 'CjkUnifiedIdeograph91e5',
u'\u7a68': 'CjkUnifiedIdeograph7a68',
u'\u26eb': 'UnicodeCharacter0x26eb',
u'\uc46c': 'HangulSyllableSsul',
u'\ua0ef': 'YiSyllableVox',
u'\u036e': 'CombiningLatinSmallLetterV',
u'\u8d72': 'CjkUnifiedIdeograph8d72',
u'\u69f5': 'CjkUnifiedIdeograph69f5',
u'\uf3f9': 'UnicodeCharacter0xf3f9',
u'\u5278': 'CjkUnifiedIdeograph5278',
u'\u3efb': 'CjkUnifiedIdeograph3efb',
u'\udc7c': 'UnicodeCharacter0xdc7c',
u'\ub8ff': 'HangulSyllableRuc',
u'\u1b7e': 'UnicodeCharacter0x1b7e',
u'\u8203': 'CjkUnifiedIdeograph8203',
u'\u6582': 'CjkUnifiedIdeograph6582',
u'\uef86': 'UnicodeCharacter0xef86',
u'\u4b09': 'CjkUnifiedIdeograph4b09',
u'\u2a88': 'GreaterthanAndSinglelineNotEqualTo',
u'\ud50d': 'HangulSyllablePeulg',
u'\ub48c': 'HangulSyllableDwen',
u'\u100f': 'MyanmarLetterNna',
u'\u9a13': 'CjkUnifiedIdeograph9a13',
u'\u7d92': 'CjkUnifiedIdeograph7d92',
u'\ue417': 'UnicodeCharacter0xe417',
u'\uc796': 'HangulSyllableJanh',
u'\u2319': 'TurnedNotSign',
u'\u0298': 'LatinLetterBilabialClick',
u'\uad1d': 'HangulSyllableGwaenj',
u'\u8c9c': 'CjkUnifiedIdeograph8c9c',
u'\u7223': 'CjkUnifiedIdeograph7223',
u'\u55a2': 'CjkUnifiedIdeograph55a2',
u'\ufc27': 'ArabicLigatureTahWithMeemIsolatedForm',
u'\udfa6': 'UnicodeCharacter0xdfa6',
u'\u3b29': 'CjkUnifiedIdeograph3b29',
u'\u1aa8': 'UnicodeCharacter0x1aa8',
u'\u852d': 'CjkUnifiedIdeograph852d',
u'\u64ac': 'CjkUnifiedIdeograph64ac',
u'\ueeb0': 'UnicodeCharacter0xeeb0',
u'\u4a33': 'CjkUnifiedIdeograph4a33',
u'\u2db2': 'EthiopicSyllableZzi',
u'\ud437': 'HangulSyllablePwaes',
u'\ub7b6': 'HangulSyllableRyagg',
u'\u1339': 'EthiopicSyllableTsu',
u'\u9d3d': 'CjkUnifiedIdeograph9d3d',
u'\u7cbc': 'CjkUnifiedIdeograph7cbc',
u'\ue741': 'UnicodeCharacter0xe741',
u'\uc6c0': 'HangulSyllableUm',
u'\u2243': 'AsymptoticallyEqualTo',
u'\u05c2': 'HebrewPointSinDot',
u'\uac47': 'HangulSyllableGyalh',
u'\u8fc6': 'CjkUnifiedIdeograph8fc6',
u'\u754d': 'CjkUnifiedIdeograph754d',
u'\u54cc': 'CjkUnifiedIdeograph54cc',
u'\uff51': 'FullwidthLatinSmallLetterQ',
u'\uded0': 'UnicodeCharacter0xded0',
u'\u3a53': 'CjkUnifiedIdeograph3a53',
u'\u1dd2': 'CombiningUsAbove',
u'\u8457': 'CjkUnifiedIdeograph8457',
u'\u67d6': 'CjkUnifiedIdeograph67d6',
u'\ue1da': 'UnicodeCharacter0xe1da',
u'\u4d5d': 'CjkUnifiedIdeograph4d5d',
u'\u2cdc': 'CopticCapitalLetterOldNubianShima',
u'\ud761': 'HangulSyllableHeub',
u'\ub6e0': 'HangulSyllableDdwels',
u'\u1263': 'EthiopicSyllableBaa',
u'\u9c67': 'CjkUnifiedIdeograph9c67',
u'\u7fe6': 'CjkUnifiedIdeograph7fe6',
u'\ue66b': 'UnicodeCharacter0xe66b',
u'\uf9ea': 'CjkCompatibilityIdeographf9ea',
u'\u256d': 'BoxDrawingsLightArcDownAndRight',
u'\u04ec': 'CyrillicCapitalLetterEWithDiaeresis',
u'\uaf71': 'HangulSyllableGgwaelt',
u'\u8ef0': 'CjkUnifiedIdeograph8ef0',
u'\u7477': 'CjkUnifiedIdeograph7477',
u'\u57f6': 'CjkUnifiedIdeograph57f6',
u'\ufe7b': 'ArabicKasraMedialForm',
u'\ud1fa': 'HangulSyllableToenh',
u'\u3d7d': 'CjkUnifiedIdeograph3d7d',
u'\u1cfc': 'UnicodeCharacter0x1cfc',
u'\u0681': 'ArabicLetterHahWithHamzaAbove',
u'\u6700': 'CjkUnifiedIdeograph6700',
u'\u4383': 'CjkUnifiedIdeograph4383',
u'\ua402': 'YiSyllableQie',
u'\u8085': 'CjkUnifiedIdeograph8085',
u'\ue104': 'UnicodeCharacter0xe104',
u'\ucd87': 'HangulSyllableCyolh',
u'\u320e': 'ParenthesizedHangulKiyeokA',
u'\u1e91': 'LatinSmallLetterZWithCircumflex',
u'\u7f10': 'CjkUnifiedIdeograph7f10',
u'\u5b93': 'CjkUnifiedIdeograph5b93',
u'\ubc12': 'HangulSyllableMip',
u'\u9895': 'CjkUnifiedIdeograph9895',
u'\uf914': 'CjkCompatibilityIdeographf914',
u'\ua597': 'VaiSyllableHu',
u'\ue299': 'UnicodeCharacter0xe299',
u'\u0a1e': 'GurmukhiLetterNya',
u'\u5720': 'CjkUnifiedIdeograph5720',
u'\u33a3': 'SquareMmCubed',
u'\u9422': 'CjkUnifiedIdeograph9422',
u'\u70a5': 'CjkUnifiedIdeograph70a5',
u'\ud124': 'HangulSyllableTyaem',
u'\ubda7': 'HangulSyllableBweolb',
u'\ufaa9': 'CjkCompatibilityIdeographfaa9',
u'\u2f30': 'KangxiRadicalOneself',
u'\u0bb3': 'TamilLetterLla',
u'\u6c32': 'CjkUnifiedIdeograph6c32',
u'\u48b5': 'CjkUnifiedIdeograph48b5',
u'\ua934': 'RejangLetterDa',
u'\u95b7': 'CjkUnifiedIdeograph95b7',
u'\uf636': 'UnicodeCharacter0xf636',
u'\ud2b9': 'HangulSyllableTeug',
u'\u0740': 'SyriacFeminineDot',
u'\u4442': 'CjkUnifiedIdeograph4442',
u'\u20c5': 'UnicodeCharacter0x20c5',
u'\u8144': 'CjkUnifiedIdeograph8144',
u'\u6dc7': 'CjkUnifiedIdeograph6dc7',
u'\uce46': 'HangulSyllableCyilm',
u'\uaac9': 'UnicodeCharacter0xaac9',
u'\uf7cb': 'UnicodeCharacter0xf7cb',
u'\u1f50': 'GreekSmallLetterUpsilonWithPsili',
u'\u5c52': 'CjkUnifiedIdeograph5c52',
u'\u38d5': 'CjkUnifiedIdeograph38d5',
u'\u9954': 'CjkUnifiedIdeograph9954',
u'\u45d7': 'CjkUnifiedIdeograph45d7',
u'\ua656': 'CyrillicCapitalLetterIotifiedA',
u'\u82d9': 'CjkUnifiedIdeograph82d9',
u'\ue358': 'UnicodeCharacter0xe358',
u'\ucfdb': 'HangulSyllableKyoc',
u'\u3462': 'CjkUnifiedIdeograph3462',
u'\u10e5': 'GeorgianLetterKhar',
u'\u7164': 'CjkUnifiedIdeograph7164',
u'\u5de7': 'CjkUnifiedIdeograph5de7',
u'\ube66': 'HangulSyllableBbanh',
u'\u9ae9': 'CjkUnifiedIdeograph9ae9',
u'\ufb68': 'ArabicLetterTtehInitialForm',
u'\ua7eb': 'UnicodeCharacter0xa7eb',
u'\ue4ed': 'UnicodeCharacter0xe4ed',
u'\u0c72': 'UnicodeCharacter0xc72',
u'\u4974': 'CjkUnifiedIdeograph4974',
u'\u35f7': 'CjkUnifiedIdeograph35f7',
u'\u9676': 'CjkUnifiedIdeograph9676',
u'\u72f9': 'CjkUnifiedIdeograph72f9',
u'\ud378': 'HangulSyllablePyaek',
u'\ubffb': 'HangulSyllableBbweos',
u'\ufcfd': 'ArabicLigatureSheenWithAlefMaksuraIsolatedForm',
u'\uc601': 'HangulSyllableYeong',
u'\u2184': 'LatinSmallLetterReversedC',
u'\u6e86': 'CjkUnifiedIdeograph6e86',
u'\uab88': 'UnicodeCharacter0xab88',
u'\u170b': 'TagalogLetterMa',
u'\ue88a': 'UnicodeCharacter0xe88a',
u'\u540d': 'CjkUnifiedIdeograph540d',
u'\u910f': 'CjkUnifiedIdeograph910f',
u'\ude11': 'UnicodeCharacter0xde11',
u'\u3994': 'CjkUnifiedIdeograph3994',
u'\u4696': 'CjkUnifiedIdeograph4696',
u'\u8398': 'CjkUnifiedIdeograph8398',
u'\uc09a': 'HangulSyllableBbilm',
u'\u2c1d': 'GlagoliticCapitalLetterChrivi',
u'\u691f': 'CjkUnifiedIdeograph691f',
u'\ub621': 'HangulSyllableDdob',
u'\uf323': 'UnicodeCharacter0xf323',
u'\u11a4': 'UnicodeCharacter0x11a4',
u'\u5ea6': 'CjkUnifiedIdeograph5ea6',
u'\u9ba8': 'CjkUnifiedIdeograph9ba8',
u'\ud8aa': 'UnicodeCharacter0xd8aa',
u'\u042d': 'CyrillicCapitalLetterE',
u'\ue5ac': 'UnicodeCharacter0xe5ac',
u'\u412f': 'CjkUnifiedIdeograph412f',
u'\u8e31': 'CjkUnifiedIdeograph8e31',
u'\ucb33': 'HangulSyllableJjyod',
u'\u36b6': 'CjkUnifiedIdeograph36b6',
u'\u73b8': 'CjkUnifiedIdeograph73b8',
u'\ub0ba': 'HangulSyllableNaenh',
u'\u1c3d': 'LepchaPunctuationCerwa',
u'\ufdbc': 'ArabicLigatureLamWithJeemWithMeemFinalForm',
u'\u593f': 'CjkUnifiedIdeograph593f',
u'\u6641': 'CjkUnifiedIdeograph6641',
u'\ua343': 'YiSyllableSsyp',
u'\ue045': 'UnicodeCharacter0xe045',
u'\u0ec6': 'LaoKoLa',
u'\u4bc8': 'CjkUnifiedIdeograph4bc8',
u'\u88ca': 'CjkUnifiedIdeograph88ca',
u'\ud5cc': 'HangulSyllableHeon',
u'\u314f': 'HangulLetterA',
u'\u7e51': 'CjkUnifiedIdeograph7e51',
u'\ubb53': 'HangulSyllableMweogs',
u'\uf855': 'UnicodeCharacter0xf855',
u'\u23d8': 'MetricalTetraseme',
u'\u60da': 'CjkUnifiedIdeograph60da',
u'\uaddc': 'HangulSyllableGyu',
u'\u095f': 'DevanagariLetterYya',
u'\ueade': 'UnicodeCharacter0xeade',
u'\u5661': 'CjkUnifiedIdeograph5661',
u'\u9363': 'CjkUnifiedIdeograph9363',
u'\ud065': 'HangulSyllableKyung',
u'\u3be8': 'CjkUnifiedIdeograph3be8',
u'\u78ea': 'CjkUnifiedIdeograph78ea',
u'\u85ec': 'CjkUnifiedIdeograph85ec',
u'\uc2ee': 'HangulSyllableSibs',
u'\u2e71': 'UnicodeCharacter0x2e71',
u'\u6b73': 'CjkUnifiedIdeograph6b73',
u'\ua875': 'PhagspaDoubleHeadMark',
u'\uf577': 'UnicodeCharacter0xf577',
u'\u13f8': 'UnicodeCharacter0x13f8',
u'\u50fa': 'CjkUnifiedIdeograph50fa',
u'\u9dfc': 'CjkUnifiedIdeograph9dfc',
u'\udafe': 'UnicodeCharacter0xdafe',
u'\ue800': 'UnicodeCharacter0xe800',
u'\ucc83': 'HangulSyllableCyaed',
u'\u2b02': 'SouthEastWhiteArrow',
u'\u0f85': 'TibetanMarkPaluta',
u'\ua106': 'YiSyllableDiep',
u'\u8589': 'CjkUnifiedIdeograph8589',
u'\u7a0c': 'CjkUnifiedIdeograph7a0c',
u'\u5e8f': 'CjkUnifiedIdeograph5e8f',
u'\uf010': 'UnicodeCharacter0xf010',
u'\ud493': 'HangulSyllablePuh',
u'\u3312': 'SquareKyurii',
u'\u1795': 'KhmerLetterPha',
u'\u8916': 'CjkUnifiedIdeograph8916',
u'\u6d99': 'CjkUnifiedIdeograph6d99',
u'\ue39d': 'UnicodeCharacter0xe39d',
u'\u421c': 'CjkUnifiedIdeograph421c',
u'\u269f': 'UnicodeCharacter0x269f',
u'\ud820': 'UnicodeCharacter0xd820',
u'\ubca3': 'HangulSyllableBegs',
u'\u1b22': 'BalineseLetterTa',
u'\u9126': 'CjkUnifiedIdeograph9126',
u'\u75a9': 'CjkUnifiedIdeograph75a9',
u'\ucbad': 'HangulSyllableJjwib',
u'\u2a2c': 'MinusSignWithRisingDots',
u'\u0eaf': 'LaoEllipsis',
u'\ua030': 'YiSyllableBurx',
u'\u84b3': 'CjkUnifiedIdeograph84b3',
u'\u7936': 'CjkUnifiedIdeograph7936',
u'\u5db9': 'CjkUnifiedIdeograph5db9',
u'\uff3a': 'FullwidthLatinCapitalLetterZ',
u'\ud3bd': 'HangulSyllablePyeolg',
u'\u323c': 'ParenthesizedIdeographSupervise',
u'\u16bf': 'RunicLetterShorttwignaudN',
u'\u8840': 'CjkUnifiedIdeograph8840',
u'\u6cc3': 'CjkUnifiedIdeograph6cc3',
u'\ue2c7': 'UnicodeCharacter0xe2c7',
u'\u4146': 'CjkUnifiedIdeograph4146',
u'\u25c9': 'Fisheye',
u'\uc74a': 'HangulSyllableEulp',
u'\ubbcd': 'HangulSyllableMeult',
u'\u1a4c': 'UnicodeCharacter0x1a4c',
u'\u9050': 'CjkUnifiedIdeograph9050',
u'\u74d3': 'CjkUnifiedIdeograph74d3',
u'\ucad7': 'HangulSyllableJjoh',
u'\u2956': 'LeftwardsHarpoonWithBarbDownToBar',
u'\u0dd9': 'SinhalaVowelSignKombuva',
u'\uaf5a': 'HangulSyllableGgwabs',
u'\u83dd': 'CjkUnifiedIdeograph83dd',
u'\u7860': 'CjkUnifiedIdeograph7860',
u'\u5ce3': 'CjkUnifiedIdeograph5ce3',
u'\ufe64': 'SmallLessthanSign',
u'\ud2e7': 'HangulSyllableTyis',
u'\u3166': 'HangulLetterNieuntikeut',
u'\u15e9': 'CanadianSyllabicsCarrierPo',
u'\ub76a': 'HangulSyllableDdilm',
u'\u6bed': 'CjkUnifiedIdeograph6bed',
u'\ue1f1': 'UnicodeCharacter0xe1f1',
u'\u4070': 'CjkUnifiedIdeograph4070',
u'\u24f3': 'NegativeCircledNumberNineteen',
u'\uc674': 'HangulSyllableWaek',
u'\ubaf7': 'HangulSyllableMwaec',
u'\u1976': 'UnicodeCharacter0x1976',
u'\u9f7a': 'CjkUnifiedIdeograph9f7a',
u'\u73fd': 'CjkUnifiedIdeograph73fd',
u'\u4901': 'CjkUnifiedIdeograph4901',
u'\u2880': 'BraillePatternDots8',
u'\ucf05': 'HangulSyllableKenj',
u'\uae84': 'HangulSyllableGgya',
u'\u0207': 'LatinSmallLetterEWithInvertedBreve',
u'\u980b': 'CjkUnifiedIdeograph980b',
u'\u678a': 'CjkUnifiedIdeograph678a',
u'\ufd8e': 'ArabicLigatureMeemWithKhahWithJeemInitialForm',
u'\u5111': 'CjkUnifiedIdeograph5111',
u'\u3090': 'HiraganaLetterWi',
u'\ud715': 'HangulSyllableHwet',
u'\ub694': 'HangulSyllableDdyoss',
u'\u601b': 'CjkUnifiedIdeograph601b',
u'\u4f9a': 'CjkUnifiedIdeograph4f9a',
u'\ue61f': 'UnicodeCharacter0xe61f',
u'\uc59e': 'HangulSyllableYaenh',
u'\u3921': 'CjkUnifiedIdeograph3921',
u'\u18a0': 'MongolianLetterManchuAliGaliTa',
u'\ubf25': 'HangulSyllableBbyeg',
u'\u9ea4': 'CjkUnifiedIdeograph9ea4',
u'\u482b': 'CjkUnifiedIdeograph482b',
u'\u57aa': 'CjkUnifiedIdeograph57aa',
u'\uce2f': 'HangulSyllableCeulh',
u'\uadae': 'HangulSyllableGwelm',
u'\u0131': 'LatinSmallLetterDotlessI',
u'\u8735': 'CjkUnifiedIdeograph8735',
u'\u66b4': 'CjkUnifiedIdeograph66b4',
u'\ufcb8': 'ArabicLigatureTahWithHahInitialForm',
u'\u503b': 'CjkUnifiedIdeograph503b',
u'\u3fba': 'CjkUnifiedIdeograph3fba',
u'\ud63f': 'HangulSyllableHod',
u'\ub5be': 'HangulSyllableDdegg',
u'\u6f45': 'CjkUnifiedIdeograph6f45',
u'\u4ec4': 'CjkUnifiedIdeograph4ec4',
u'\ue549': 'UnicodeCharacter0xe549',
u'\uc4c8': 'HangulSyllableSswim',
u'\u384b': 'CjkUnifiedIdeograph384b',
u'\u07ca': 'NkoLetterA',
u'\ube4f': 'HangulSyllableBilb',
u'\u9dce': 'CjkUnifiedIdeograph9dce',
u'\u7755': 'CjkUnifiedIdeograph7755',
u'\u56d4': 'CjkUnifiedIdeograph56d4',
u'\ucd59': 'HangulSyllableCwaet',
u'\uacd8': 'HangulSyllableGyess',
u'\u865f': 'CjkUnifiedIdeograph865f',
u'\u65de': 'CjkUnifiedIdeograph65de',
u'\ufbe2': 'ArabicLetterKirghizYuIsolatedForm',
u'\u5f65': 'CjkUnifiedIdeograph5f65',
u'\u3ee4': 'CjkUnifiedIdeograph3ee4',
u'\ud569': 'HangulSyllableHab',
u'\ub4e8': 'HangulSyllableDeuls',
u'\u6e6f': 'CjkUnifiedIdeograph6e6f',
u'\u4dee': 'HexagramForOppression',
u'\ue473': 'UnicodeCharacter0xe473',
u'\uc3f2': 'HangulSyllableSsop',
u'\u2775': 'MediumRightCurlyBracketOrnament',
u'\u06f4': 'ExtendedArabicindicDigitFour',
u'\ubd79': 'HangulSyllableByong',
u'\u9cf8': 'CjkUnifiedIdeograph9cf8',
u'\u767f': 'CjkUnifiedIdeograph767f',
u'\u55fe': 'CjkUnifiedIdeograph55fe',
u'\u2006': 'SixperemSpace',
u'\u0489': 'CombiningCyrillicMillionsSign',
u'\u6508': 'CjkUnifiedIdeograph6508',
u'\u598b': 'CjkUnifiedIdeograph598b',
u'\ua60a': 'VaiSyllableNye',
u'\u9a8d': 'CjkUnifiedIdeograph9a8d',
u'\ufb0c': 'UnicodeCharacter0xfb0c',
u'\udf8f': 'UnicodeCharacter0xdf8f',
u'\u0816': 'UnicodeCharacter0x816',
u'\u4d18': 'CjkUnifiedIdeograph4d18',
u'\u219b': 'RightwardsArrowWithStroke',
u'\u8e1a': 'CjkUnifiedIdeograph8e1a',
u'\u629d': 'CjkUnifiedIdeograph629d',
u'\uc31c': 'HangulSyllableSsael',
u'\ua79f': 'UnicodeCharacter0xa79f',
u'\uf8a1': 'UnicodeCharacter0xf8a1',
u'\u1026': 'MyanmarLetterUu',
u'\u5528': 'CjkUnifiedIdeograph5528',
u'\u09ab': 'BengaliLetterPha',
u'\u962a': 'CjkUnifiedIdeograph962a',
u'\u4aad': 'CjkUnifiedIdeograph4aad',
u'\uab2c': 'UnicodeCharacter0xab2c',
u'\u8faf': 'CjkUnifiedIdeograph8faf',
u'\uec2e': 'UnicodeCharacter0xec2e',
u'\uc0b1': 'HangulSyllableSanj',
u'\u3d38': 'CjkUnifiedIdeograph3d38',
u'\u11bb': 'HangulJongseongSsangsios',
u'\u7e3a': 'CjkUnifiedIdeograph7e3a',
u'\u52bd': 'CjkUnifiedIdeograph52bd',
u'\ub33c': 'HangulSyllableDyaen',
u'\u97bf': 'CjkUnifiedIdeograph97bf',
u'\uf43e': 'UnicodeCharacter0xf43e',
u'\ua8c1': 'SaurashtraVowelSignO',
u'\uedc3': 'UnicodeCharacter0xedc3',
u'\u0548': 'ArmenianCapitalLetterVo',
u'\u464a': 'CjkUnifiedIdeograph464a',
u'\u3acd': 'CjkUnifiedIdeograph3acd',
u'\u9b4c': 'CjkUnifiedIdeograph9b4c',
u'\u7fcf': 'CjkUnifiedIdeograph7fcf',
u'\udc4e': 'UnicodeCharacter0xdc4e',
u'\ub0d1': 'HangulSyllableNyag',
u'\uf5d3': 'UnicodeCharacter0xf5d3',
u'\u2e5a': 'UnicodeCharacter0x2e5a',
u'\u02dd': 'DoubleAcuteAccent',
u'\u635c': 'CjkUnifiedIdeograph635c',
u'\u47df': 'CjkUnifiedIdeograph47df',
u'\ua45e': 'YiSyllableXix',
u'\u98e1': 'CjkUnifiedIdeograph98e1',
u'\uf960': 'CjkCompatibilityIdeographf960',
u'\udde3': 'UnicodeCharacter0xdde3',
u'\u366a': 'CjkUnifiedIdeograph366a',
u'\u4b6c': 'CjkUnifiedIdeograph4b6c',
u'\u2fef': 'UnicodeCharacter0x2fef',
u'\u8c6e': 'CjkUnifiedIdeograph8c6e',
u'\u60f1': 'CjkUnifiedIdeograph60f1',
u'\uc170': 'HangulSyllableSye',
u'\ua5f3': 'VaiSyllableTe',
u'\ue6f5': 'UnicodeCharacter0xe6f5',
u'\u1e7a': 'LatinCapitalLetterUWithMacronAndDiaeresis',
u'\u537c': 'CjkUnifiedIdeograph537c',
u'\u37ff': 'CjkUnifiedIdeograph37ff',
u'\u947e': 'CjkUnifiedIdeograph947e',
u'\ua980': 'UnicodeCharacter0xa980',
u'\u0d03': 'MalayalamSignVisarga',
u'\uea82': 'UnicodeCharacter0xea82',
u'\u4e05': 'CjkUnifiedIdeograph4e05',
u'\u8307': 'CjkUnifiedIdeograph8307',
u'\uc409': 'HangulSyllableSswang',
u'\u3b8c': 'CjkUnifiedIdeograph3b8c',
u'\u7c8e': 'CjkUnifiedIdeograph7c8e',
u'\ub190': 'HangulSyllableNok',
u'\u1513': 'CanadianSyllabicsSho',
u'\uf292': 'UnicodeCharacter0xf292',
u'\u5615': 'CjkUnifiedIdeograph5615',
u'\u6b17': 'CjkUnifiedIdeograph6b17',
u'\uac19': 'HangulSyllableGat',
u'\ue11b': 'UnicodeCharacter0xe11b',
u'\u039c': 'GreekCapitalLetterMu',
u'\u449e': 'CjkUnifiedIdeograph449e',
u'\u99a0': 'CjkUnifiedIdeograph99a0',
u'\udaa2': 'UnicodeCharacter0xdaa2',
u'\u3e25': 'CjkUnifiedIdeograph3e25',
u'\u7327': 'CjkUnifiedIdeograph7327',
u'\ub429': 'HangulSyllableDoeb',
u'\uc92b': 'HangulSyllableJweos',
u'\u2cae': 'CopticCapitalLetterPsi',
u'\u61b0': 'CjkUnifiedIdeograph61b0',
u'\ua2b2': 'YiSyllableCiet',
u'\u0635': 'ArabicLetterSad',
u'\ue7b4': 'UnicodeCharacter0xe7b4',
u'\u5b37': 'CjkUnifiedIdeograph5b37',
u'\u9c39': 'CjkUnifiedIdeograph9c39',
u'\ud13b': 'HangulSyllableTeolb',
u'\u34be': 'CjkUnifiedIdeograph34be',
u'\u49c0': 'CjkUnifiedIdeograph49c0',
u'\u8ac2': 'CjkUnifiedIdeograph8ac2',
u'\ucfc4': 'HangulSyllableKyo',
u'\u2347': 'AplFunctionalSymbolQuadLeftwardsArrow',
u'\u6449': 'CjkUnifiedIdeograph6449',
u'\ub94b': 'HangulSyllableRwilh',
u'\ufa4d': 'CjkCompatibilityIdeographfa4d',
u'\u1cce': 'UnicodeCharacter0x1cce',
u'\u51d0': 'CjkUnifiedIdeograph51d0',
u'\u92d2': 'CjkUnifiedIdeograph92d2',
u'\ud7d4': 'UnicodeCharacter0xd7d4',
u'\u0b57': 'OriyaAuLengthMark',
u'\ue8d6': 'UnicodeCharacter0xe8d6',
u'\u4c59': 'CjkUnifiedIdeograph4c59',
u'\u815b': 'CjkUnifiedIdeograph815b',
u'\uc25d': 'HangulSyllableSwelt',
u'\u39e0': 'CjkUnifiedIdeograph39e0',
u'\u7ae2': 'CjkUnifiedIdeograph7ae2',
u'\ubfe4': 'HangulSyllableBbuk',
u'\u1367': 'EthiopicQuestionMark',
u'\uf0e6': 'UnicodeCharacter0xf0e6',
u'\u5469': 'CjkUnifiedIdeograph5469',
u'\u696b': 'CjkUnifiedIdeograph696b',
u'\uaa6d': 'UnicodeCharacter0xaa6d',
u'\uef6f': 'UnicodeCharacter0xef6f',
u'\u01f0': 'LatinSmallLetterJWithCaron',
u'\u42f2': 'CjkUnifiedIdeograph42f2',
u'\u87f4': 'CjkUnifiedIdeograph87f4',
u'\ud8f6': 'UnicodeCharacter0xd8f6',
u'\u3c79': 'CjkUnifiedIdeograph3c79',
u'\u717b': 'CjkUnifiedIdeograph717b',
u'\ub27d': 'HangulSyllableNyulg',
u'\uf77f': 'UnicodeCharacter0xf77f',
u'\u8b81': 'CjkUnifiedIdeograph8b81',
u'\u6c04': 'CjkUnifiedIdeograph6c04',
u'\u4087': 'CjkUnifiedIdeograph4087',
u'\ue608': 'UnicodeCharacter0xe608',
u'\uda8b': 'UnicodeCharacter0xda8b',
u'\u250a': 'BoxDrawingsLightQuadrupleDashVertical',
u'\u198d': 'NewTaiLueLetterLowYa',
u'\ubf0e': 'HangulSyllableBbyeonh',
u'\u9391': 'CjkUnifiedIdeograph9391',
u'\u7414': 'CjkUnifiedIdeograph7414',
u'\u2897': 'BraillePatternDots12358',
u'\uce18': 'HangulSyllableCyuss',
u'\ua29b': 'YiSyllableZot',
u'\u0d1a': 'MalayalamLetterCa',
u'\u871e': 'CjkUnifiedIdeograph871e',
u'\u7ba1': 'CjkUnifiedIdeograph7ba1',
u'\ufda5': 'ArabicLigatureJeemWithMeemWithYehFinalForm',
u'\u5c24': 'CjkUnifiedIdeograph5c24',
u'\u30a7': 'KatakanaLetterSmallE',
u'\ud628': 'HangulSyllableHyels',
u'\u8aab': 'CjkUnifiedIdeograph8aab',
u'\u152a': 'CanadianSyllabicsYo',
u'\u6f2e': 'CjkUnifiedIdeograph6f2e',
u'\u43b1': 'CjkUnifiedIdeograph43b1',
u'\ue132': 'UnicodeCharacter0xe132',
u'\uc5b5': 'HangulSyllableEog',
u'\u2434': 'UnicodeCharacter0x2434',
u'\u18b7': 'UnicodeCharacter0x18b7',
u'\ube38': 'HangulSyllableByim',
u'\u92bb': 'CjkUnifiedIdeograph92bb',
u'\u773e': 'CjkUnifiedIdeograph773e',
u'\u2bc1': 'UnicodeCharacter0x2bc1',
u'\uc942': 'HangulSyllableJwelp',
u'\uadc5': 'HangulSyllableGwinj',
u'\u0c44': 'TeluguVowelSignVocalicRr',
u'\u8648': 'CjkUnifiedIdeograph8648',
u'\u7acb': 'CjkUnifiedIdeograph7acb',
u'\ufccf': 'ArabicLigatureMeemWithHahInitialForm',
u'\u5f4e': 'CjkUnifiedIdeograph5f4e',
u'\u33d1': 'SquareLn',
u'\ud152': 'HangulSyllableTenh',
u'\ub5d5': 'HangulSyllableDdet',
u'\u1454': 'CanadianSyllabicsCarrierDi',
u'\u6e58': 'CjkUnifiedIdeograph6e58',
u'\u42db': 'CjkUnifiedIdeograph42db',
u'\ue05c': 'UnicodeCharacter0xe05c',
u'\uc4df': 'HangulSyllableSsyulb',
u'\u275e': 'HeavyDoubleCommaQuotationMarkOrnament',
u'\u1be1': 'UnicodeCharacter0x1be1',
u'\ub962': 'HangulSyllableRyulm',
u'\u9de5': 'CjkUnifiedIdeograph9de5',
u'\u7668': 'CjkUnifiedIdeograph7668',
u'\u2aeb': 'DoubleUpTack',
u'\uc86c': 'HangulSyllableJyek',
u'\uacef': 'HangulSyllableGolh',
u'\u0f6e': 'UnicodeCharacter0xf6e',
u'\u8172': 'CjkUnifiedIdeograph8172',
u'\u65f5': 'CjkUnifiedIdeograph65f5',
u'\ufff9': 'InterlinearAnnotationAnchor',
u'\u5e78': 'CjkUnifiedIdeograph5e78',
u'\u32fb': 'CircledKatakanaWa',
u'\ud07c': 'HangulSyllableKeum',
u'\ub4ff': 'HangulSyllableDyid',
u'\u177e': 'UnicodeCharacter0x177e',
u'\u8e03': 'CjkUnifiedIdeograph8e03',
u'\u6982': 'CjkUnifiedIdeograph6982',
u'\ue386': 'UnicodeCharacter0xe386',
u'\u4709': 'CjkUnifiedIdeograph4709',
u'\u2688': 'BlackCircleWithWhiteDotRight',
u'\ud90d': 'UnicodeCharacter0xd90d',
u'\ub88c': 'HangulSyllableRwass',
u'\u1c0f': 'LepchaLetterPla',
u'\u9613': 'CjkUnifiedIdeograph9613',
u'\u7192': 'CjkUnifiedIdeograph7192',
u'\ue817': 'UnicodeCharacter0xe817',
u'\ucb96': 'HangulSyllableJjwej',
u'\u2f19': 'KangxiRadicalSeal',
u'\u0e98': 'UnicodeCharacter0xe98',
u'\ua11d': 'YiSyllableTip',
u'\u809c': 'CjkUnifiedIdeograph809c',
u'\u7e23': 'CjkUnifiedIdeograph7e23',
u'\u59a2': 'CjkUnifiedIdeograph59a2',
u'\uf027': 'UnicodeCharacter0xf027',
u'\ud3a6': 'HangulSyllablePelp',
u'\u3729': 'CjkUnifiedIdeograph3729',
u'\u16a8': 'RunicLetterAnsuzA',
u'\u892d': 'CjkUnifiedIdeograph892d',
u'\u68ac': 'CjkUnifiedIdeograph68ac',
u'\ue2b0': 'UnicodeCharacter0xe2b0',
u'\u4633': 'CjkUnifiedIdeograph4633',
u'\u21b2': 'DownwardsArrowWithTipLeftwards',
u'\ud837': 'UnicodeCharacter0xd837',
u'\ubbb6': 'HangulSyllableMyubs',
u'\u1f39': 'GreekCapitalLetterIotaWithDasia',
u'\u913d': 'CjkUnifiedIdeograph913d',
u'\u70bc': 'CjkUnifiedIdeograph70bc',
u'\ueb41': 'UnicodeCharacter0xeb41',
u'\ucac0': 'HangulSyllableJjon',
u'\u2e43': 'UnicodeCharacter0x2e43',
u'\u09c2': 'BengaliVowelSignUu',
u'\ua047': 'YiSyllablePox',
u'\u83c6': 'CjkUnifiedIdeograph83c6',
u'\u794d': 'CjkUnifiedIdeograph794d',
u'\u58cc': 'CjkUnifiedIdeograph58cc',
u'\uf351': 'UnicodeCharacter0xf351',
u'\ud2d0': 'HangulSyllableTeuk',
u'\u3653': 'CjkUnifiedIdeograph3653',
u'\u11d2': 'HangulJongseongRieulmieumsios',
u'\u8857': 'CjkUnifiedIdeograph8857',
u'\u6bd6': 'CjkUnifiedIdeograph6bd6',
u'\uedda': 'UnicodeCharacter0xedda',
u'\u415d': 'CjkUnifiedIdeograph415d',
u'\u20dc': 'CombiningFourDotsAbove',
u'\udb61': 'UnicodeCharacter0xdb61',
u'\ubae0': 'HangulSyllableMwae',
u'\u1e63': 'LatinSmallLetterSWithDotBelow',
u'\u9067': 'CjkUnifiedIdeograph9067',
u'\u73e6': 'CjkUnifiedIdeograph73e6',
u'\uea6b': 'UnicodeCharacter0xea6b',
u'\uf5ea': 'UnicodeCharacter0xf5ea',
u'\u296d': 'RightwardsHarpoonWithBarbDownBelowLongDash',
u'\u08ec': 'UnicodeCharacter0x8ec',
u'\ua371': 'YiSyllableChux',
u'\u82f0': 'CjkUnifiedIdeograph82f0',
u'\u7877': 'CjkUnifiedIdeograph7877',
u'\u5bf6': 'CjkUnifiedIdeograph5bf6',
u'\uf27b': 'UnicodeCharacter0xf27b',
u'\uddfa': 'UnicodeCharacter0xddfa',
u'\u317d': 'HangulLetterSiospieup',
u'\u10fc': 'ModifierLetterGeorgianNar',
u'\u0a81': 'GujaratiSignCandrabindu',
u'\u6b00': 'CjkUnifiedIdeograph6b00',
u'\u4f83': 'CjkUnifiedIdeograph4f83',
u'\ua802': 'SylotiNagriSignDvisvara',
u'\u8c85': 'CjkUnifiedIdeograph8c85',
u'\ued04': 'UnicodeCharacter0xed04',
u'\uc187': 'HangulSyllableSyec',
u'\u3e0e': 'CjkUnifiedIdeograph3e0e',
u'\u1291': 'EthiopicSyllableNu',
u'\u7310': 'CjkUnifiedIdeograph7310',
u'\u5793': 'CjkUnifiedIdeograph5793',
u'\ub012': 'HangulSyllableGgwinh',
u'\u9495': 'CjkUnifiedIdeograph9495',
u'\uf514': 'UnicodeCharacter0xf514',
u'\ua997': 'UnicodeCharacter0xa997',
u'\uee99': 'UnicodeCharacter0xee99',
u'\u061e': 'ArabicTripleDotPunctuationMark',
u'\u5b20': 'CjkUnifiedIdeograph5b20',
u'\u3fa3': 'CjkUnifiedIdeograph3fa3',
u'\u9822': 'CjkUnifiedIdeograph9822',
u'\u7ca5': 'CjkUnifiedIdeograph7ca5',
u'\udd24': 'UnicodeCharacter0xdd24',
u'\ub1a7': 'HangulSyllableNwas',
u'\uf6a9': 'UnicodeCharacter0xf6a9',
u'\u2330': 'TotalRunout',
u'\u07b3': 'UnicodeCharacter0x7b3',
u'\u6032': 'CjkUnifiedIdeograph6032',
u'\u44b5': 'CjkUnifiedIdeograph44b5',
u'\ua534': 'VaiSyllableThi',
u'\u99b7': 'CjkUnifiedIdeograph99b7',
u'\ufa36': 'CjkCompatibilityIdeographfa36',
u'\udeb9': 'UnicodeCharacter0xdeb9',
u'\u0b40': 'OriyaVowelSignIi',
u'\u4842': 'CjkUnifiedIdeograph4842',
u'\u2cc5': 'CopticSmallLetterOldCopticShei',
u'\u8d44': 'CjkUnifiedIdeograph8d44',
u'\u61c7': 'CjkUnifiedIdeograph61c7',
u'\uc246': 'HangulSyllableSweobs',
u'\ua6c9': 'UnicodeCharacter0xa6c9',
u'\ufbcb': 'UnicodeCharacter0xfbcb',
u'\u1350': 'EthiopicSyllablePa',
u'\u5052': 'CjkUnifiedIdeograph5052',
u'\u34d5': 'CjkUnifiedIdeograph34d5',
u'\u9554': 'CjkUnifiedIdeograph9554',
u'\u49d7': 'CjkUnifiedIdeograph49d7',
u'\uaa56': 'ChamDigitSix',
u'\u8ed9': 'CjkUnifiedIdeograph8ed9',
u'\uef58': 'UnicodeCharacter0xef58',
u'\uc3db': 'HangulSyllableSsogs',
u'\u3862': 'CjkUnifiedIdeograph3862',
u'\u1ce5': 'UnicodeCharacter0x1ce5',
u'\u7d64': 'CjkUnifiedIdeograph7d64',
u'\u51e7': 'CjkUnifiedIdeograph51e7',
u'\ub266': 'HangulSyllableNwilp',
u'\u96e9': 'CjkUnifiedIdeograph96e9',
u'\uf768': 'UnicodeCharacter0xf768',
u'\uabeb': 'UnicodeCharacter0xabeb',
u'\ue8ed': 'UnicodeCharacter0xe8ed',
u'\u4574': 'CjkUnifiedIdeograph4574',
u'\u39f7': 'CjkUnifiedIdeograph39f7',
u'\u9a76': 'CjkUnifiedIdeograph9a76',
u'\u7ef9': 'CjkUnifiedIdeograph7ef9',
u'\udf78': 'UnicodeCharacter0xdf78',
u'\ub3fb': 'HangulSyllableDwah',
u'\uf0fd': 'UnicodeCharacter0xf0fd',
u'\uca01': 'HangulSyllableJjaelg',
u'\u2d84': 'EthiopicSyllableShoa',
u'\u6286': 'CjkUnifiedIdeograph6286',
u'\ua788': 'ModifierLetterLowCircumflexAccent',
u'\u1b0b': 'BalineseLetterRaRepa',
u'\ue48a': 'UnicodeCharacter0xe48a',
u'\u580d': 'CjkUnifiedIdeograph580d',
u'\u9d0f': 'CjkUnifiedIdeograph9d0f',
u'\ud211': 'HangulSyllableTyog',
u'\u3594': 'CjkUnifiedIdeograph3594',
u'\u4a96': 'CjkUnifiedIdeograph4a96',
u'\u8f98': 'CjkUnifiedIdeograph8f98',
u'\ucc9a': 'HangulSyllableCeogg',
u'\u201d': 'RightDoubleQuotationMark',
u'\u651f': 'CjkUnifiedIdeograph651f',
u'\uba21': 'HangulSyllableMyaenj',
u'\uff23': 'FullwidthLatinCapitalLetterC',
u'\u1da4': 'ModifierLetterSmallIWithStroke',
u'\u52a6': 'CjkUnifiedIdeograph52a6',
u'\u97a8': 'CjkUnifiedIdeograph97a8',
u'\ud4aa': 'HangulSyllablePweoj',
u'\u082d': 'UnicodeCharacter0x82d',
u'\ue9ac': 'UnicodeCharacter0xe9ac',
u'\u4d2f': 'CjkUnifiedIdeograph4d2f',
u'\u8231': 'CjkUnifiedIdeograph8231',
u'\uc733': 'HangulSyllableYus',
u'\u3ab6': 'CjkUnifiedIdeograph3ab6',
u'\u7fb8': 'CjkUnifiedIdeograph7fb8',
u'\ubcba': 'HangulSyllableBep',
u'\u103d': 'MyanmarConsonantSignMedialWa',
u'\uf1bc': 'UnicodeCharacter0xf1bc',
u'\u553f': 'CjkUnifiedIdeograph553f',
u'\u6a41': 'CjkUnifiedIdeograph6a41',
u'\uaf43': 'HangulSyllableGgoc',
u'\uec45': 'UnicodeCharacter0xec45',
u'\u02c6': 'ModifierLetterCircumflexAccent',
u'\u47c8': 'CjkUnifiedIdeograph47c8',
u'\u84ca': 'CjkUnifiedIdeograph84ca',
u'\ud9cc': 'UnicodeCharacter0xd9cc',
u'\u3d4f': 'CjkUnifiedIdeograph3d4f',
u'\u7251': 'CjkUnifiedIdeograph7251',
u'\ub753': 'HangulSyllableDdyilh',
u'\uf455': 'UnicodeCharacter0xf455',
u'\u2fd8': 'UnicodeCharacter0x2fd8',
u'\u6cda': 'CjkUnifiedIdeograph6cda',
u'\ua1dc': 'YiSyllableGi',
u'\u055f': 'ArmenianAbbreviationMark',
u'\ue6de': 'UnicodeCharacter0xe6de',
u'\u5a61': 'CjkUnifiedIdeograph5a61',
u'\u9f63': 'CjkUnifiedIdeograph9f63',
u'\udc65': 'UnicodeCharacter0xdc65',
u'\u37e8': 'CjkUnifiedIdeograph37e8',
u'\u74ea': 'CjkUnifiedIdeograph74ea',
u'\u89ec': 'CjkUnifiedIdeograph89ec',
u'\uceee': 'HangulSyllableKeolm',
u'\u2271': 'NeitherGreaterthanNorEqualTo',
u'\u6773': 'CjkUnifiedIdeograph6773',
u'\ua475': 'YiSyllableYiet',
u'\uf977': 'CjkCompatibilityIdeographf977',
u'\u1ff8': 'GreekCapitalLetterOmicronWithVaria',
u'\u5cfa': 'CjkUnifiedIdeograph5cfa',
u'\u91fc': 'CjkUnifiedIdeograph91fc',
u'\ud6fe': 'HangulSyllableHwegg',
u'\uec00': 'UnicodeCharacter0xec00',
u'\uc883': 'HangulSyllableJos',
u'\u2f02': 'KangxiRadicalDot',
u'\u0b85': 'TamilLetterA',
u'\ua506': 'VaiSyllableBhee',
u'\u8189': 'CjkUnifiedIdeograph8189',
u'\u7e0c': 'CjkUnifiedIdeograph7e0c',
u'\u5a8f': 'CjkUnifiedIdeograph5a8f',
u'\uf410': 'UnicodeCharacter0xf410',
u'\ud093': 'HangulSyllableKyilb',
u'\u3712': 'CjkUnifiedIdeograph3712',
u'\u1395': 'EthiopicTonalMarkKenat',
u'\u8d16': 'CjkUnifiedIdeograph8d16',
u'\u6999': 'CjkUnifiedIdeograph6999',
u'\ue79d': 'UnicodeCharacter0xe79d',
u'\u461c': 'CjkUnifiedIdeograph461c',
u'\u229f': 'SquaredMinus',
u'\udc20': 'UnicodeCharacter0xdc20',
u'\ub8a3': 'HangulSyllableRwaelh',
u'\u1f22': 'GreekSmallLetterEtaWithPsiliAndVaria',
u'\u9526': 'CjkUnifiedIdeograph9526',
u'\u71a9': 'CjkUnifiedIdeograph71a9',
u'\ucfad': 'HangulSyllableKoenj',
u'\u2e2c': 'SquaredFourDotPunctuation',
u'\u0aaf': 'GujaratiLetterYa',
u'\ua430': 'YiSyllableJjyp',
u'\u80b3': 'CjkUnifiedIdeograph80b3',
u'\u7d36': 'CjkUnifiedIdeograph7d36',
u'\u59b9': 'CjkUnifiedIdeograph59b9',
u'\ufb3a': 'HebrewLetterFinalKafWithDagesh',
u'\ud7bd': 'UnicodeCharacter0xd7bd',
u'\u363c': 'CjkUnifiedIdeograph363c',
u'\u12bf': 'UnicodeCharacter0x12bf',
u'\u8c40': 'CjkUnifiedIdeograph8c40',
u'\u68c3': 'CjkUnifiedIdeograph68c3',
u'\ue6c7': 'UnicodeCharacter0xe6c7',
u'\u4546': 'CjkUnifiedIdeograph4546',
u'\u21c9': 'RightwardsPairedArrows',
u'\uc34a': 'HangulSyllableSsyap',
u'\ubfcd': 'HangulSyllableBbug',
u'\u1e4c': 'LatinCapitalLetterOWithTildeAndAcute',
u'\u9450': 'CjkUnifiedIdeograph9450',
u'\u70d3': 'CjkUnifiedIdeograph70d3',
u'\uced7': 'HangulSyllableKyaelh',
u'\u2d56': 'TifinaghLetterYagh',
u'\u09d9': 'UnicodeCharacter0x9d9',
u'\uab5a': 'UnicodeCharacter0xab5a',
u'\u87dd': 'CjkUnifiedIdeograph87dd',
u'\u7c60': 'CjkUnifiedIdeograph7c60',
u'\u58e3': 'CjkUnifiedIdeograph58e3',
u'\ufa64': 'CjkCompatibilityIdeographfa64',
u'\ud6e7': 'HangulSyllableHweod',
u'\u3566': 'CjkUnifiedIdeograph3566',
u'\u11e9': 'HangulJongseongSiosrieul',
u'\ub36a': 'HangulSyllableDeoj',
u'\u6fed': 'CjkUnifiedIdeograph6fed',
u'\ue5f1': 'UnicodeCharacter0xe5f1',
u'\u4470': 'CjkUnifiedIdeograph4470',
u'\u20f3': 'UnicodeCharacter0x20f3',
u'\uc274': 'HangulSyllableSwil',
u'\ubef7': 'HangulSyllableBbelb',
u'\u1d76': 'LatinSmallLetterZWithMiddleTilde',
u'\u9b7a': 'CjkUnifiedIdeograph9b7a',
u'\u77fd': 'CjkUnifiedIdeograph77fd',
u'\u4d01': 'CjkUnifiedIdeograph4d01',
u'\u2c80': 'CopticCapitalLetterAlfa',
u'\ucb05': 'HangulSyllableJjwaeb',
u'\uaa84': 'UnicodeCharacter0xaa84',
u'\u0607': 'ArabicindicFourthRoot',
u'\u9c0b': 'CjkUnifiedIdeograph9c0b',
u'\u638a': 'CjkUnifiedIdeograph638a',
u'\uf98e': 'CjkCompatibilityIdeographf98e',
u'\u5511': 'CjkUnifiedIdeograph5511',
u'\u3490': 'CjkUnifiedIdeograph3490',
u'\ud315': 'HangulSyllablePalg',
u'\ub294': 'HangulSyllableNeun',
u'\u641b': 'CjkUnifiedIdeograph641b',
u'\u4b9a': 'CjkUnifiedIdeograph4b9a',
u'\ue21f': 'UnicodeCharacter0xe21f',
u'\uc19e': 'HangulSyllableSobs',
u'\u3d21': 'CjkUnifiedIdeograph3d21',
u'\u1ca0': 'UnicodeCharacter0x1ca0',
u'\ubb25': 'HangulSyllableMyolt',
u'\u9aa4': 'CjkUnifiedIdeograph9aa4',
u'\u4c2b': 'CjkUnifiedIdeograph4c2b',
u'\u53aa': 'CjkUnifiedIdeograph53aa',
u'\uca2f': 'HangulSyllableJjyah',
u'\ua9ae': 'UnicodeCharacter0xa9ae',
u'\u0531': 'ArmenianCapitalLetterAyb',
u'\u8335': 'CjkUnifiedIdeograph8335',
u'\u62b4': 'CjkUnifiedIdeograph62b4',
u'\uf8b8': 'UnicodeCharacter0xf8b8',
u'\u543b': 'CjkUnifiedIdeograph543b',
u'\u3bba': 'CjkUnifiedIdeograph3bba',
u'\ud23f': 'HangulSyllableTus',
u'\ub1be': 'HangulSyllableNwaelp',
u'\u6b45': 'CjkUnifiedIdeograph6b45',
u'\u4ac4': 'CjkUnifiedIdeograph4ac4',
u'\ue149': 'UnicodeCharacter0xe149',
u'\uc0c8': 'HangulSyllableSae',
u'\u3c4b': 'CjkUnifiedIdeograph3c4b',
u'\u03ca': 'GreekSmallLetterIotaWithDialytika',
u'\uba4f': 'HangulSyllableMeoc',
u'\u99ce': 'CjkUnifiedIdeograph99ce',
u'\u7355': 'CjkUnifiedIdeograph7355',
u'\u52d4': 'CjkUnifiedIdeograph52d4',
u'\uc959': 'HangulSyllableJwilg',
u'\ua8d8': 'SaurashtraDigitEight',
u'\u045b': 'CyrillicSmallLetterTshe',
u'\u825f': 'CjkUnifiedIdeograph825f',
u'\u61de': 'CjkUnifiedIdeograph61de',
u'\uffe2': 'FullwidthNotSign',
u'\u5b65': 'CjkUnifiedIdeograph5b65',
u'\u3ae4': 'CjkUnifiedIdeograph3ae4',
u'\ud169': 'HangulSyllableTyeog',
u'\ub0e8': 'HangulSyllableNyak',
u'\u6a6f': 'CjkUnifiedIdeograph6a6f',
u'\u49ee': 'CjkUnifiedIdeograph49ee',
u'\ue073': 'UnicodeCharacter0xe073',
u'\uc7f2': 'HangulSyllableJyaelp',
u'\u2375': 'AplFunctionalSymbolOmega',
u'\u02f4': 'ModifierLetterMiddleGraveAccent',
u'\ub979': 'HangulSyllableReunj',
u'\u98f8': 'CjkUnifiedIdeograph98f8',
u'\u727f': 'CjkUnifiedIdeograph727f',
u'\u51fe': 'CjkUnifiedIdeograph51fe',
u'\u2406': 'SymbolForAcknowledge',
u'\x89': 'UnicodeCharacter0x89',
u'\u6108': 'CjkUnifiedIdeograph6108',
u'\u5d8b': 'CjkUnifiedIdeograph5d8b',
u'\ua20a': 'YiSyllableKet',
u'\u9e8d': 'CjkUnifiedIdeograph9e8d',
u'\uff0c': 'FullwidthComma',
u'\udb8f': 'UnicodeCharacter0xdb8f',
u'\u0c16': 'TeluguLetterKha',
u'\u4918': 'CjkUnifiedIdeograph4918',
u'\u259b': 'QuadrantUpperLeftAndUpperRightAndLowerLeft',
u'\u8a1a': 'CjkUnifiedIdeograph8a1a',
u'\u669d': 'CjkUnifiedIdeograph669d',
u'\uc71c': 'HangulSyllableWik',
u'\ua39f': 'YiSyllableNrut',
u'\ufca1': 'ArabicLigatureTehWithJeemInitialForm',
u'\u1426': 'CanadianSyllabicsFinalDoubleShortVerticalStrokes',
u'\u5128': 'CjkUnifiedIdeograph5128',
u'\u0dab': 'SinhalaLetterMuurdhajaNayanna',
u'\u922a': 'CjkUnifiedIdeograph922a',
u'\u4ead': 'CjkUnifiedIdeograph4ead',
u'\uaf2c': 'HangulSyllableGgo',
u'\u8baf': 'CjkUnifiedIdeograph8baf',
u'\ue82e': 'UnicodeCharacter0xe82e',
u'\uc4b1': 'HangulSyllableSsweng',
u'\u3938': 'CjkUnifiedIdeograph3938',
u'\u15bb': 'CanadianSyllabicsBlackfootNa',
u'\u7a3a': 'CjkUnifiedIdeograph7a3a',
u'\u56bd': 'CjkUnifiedIdeograph56bd',
u'\ub73c': 'HangulSyllableDdeuss',
u'\u93bf': 'CjkUnifiedIdeograph93bf',
u'\uf03e': 'UnicodeCharacter0xf03e',
u'\uacc1': 'HangulSyllableGyeot',
u'\ue9c3': 'UnicodeCharacter0xe9c3',
u'\u0148': 'LatinSmallLetterNWithCaron',
u'\u424a': 'CjkUnifiedIdeograph424a',
u'\u3ecd': 'CjkUnifiedIdeograph3ecd',
u'\u9f4c': 'CjkUnifiedIdeograph9f4c',
u'\u7bcf': 'CjkUnifiedIdeograph7bcf',
u'\ud84e': 'UnicodeCharacter0xd84e',
u'\ub4d1': 'HangulSyllableDyub',
u'\uf1d3': 'UnicodeCharacter0xf1d3',
u'\u2a5a': 'LogicalAndWithMiddleStem',
u'\u06dd': 'ArabicEndOfAyah',
u'\u675c': 'CjkUnifiedIdeograph675c',
u'\u43df': 'CjkUnifiedIdeograph43df',
u'\ua05e': 'YiSyllableBbat',
u'\u9ce1': 'CjkUnifiedIdeograph9ce1',
u'\ufd60': 'ArabicLigatureSeenWithMeemWithHahInitialForm',
u'\ud9e3': 'UnicodeCharacter0xd9e3',
u'\u326a': 'CircledHangulKhieukh',
u'\u4f6c': 'CjkUnifiedIdeograph4f6c',
u'\u2bef': 'UnicodeCharacter0x2bef',
u'\u886e': 'CjkUnifiedIdeograph886e',
u'\u64f1': 'CjkUnifiedIdeograph64f1',
u'\uc570': 'HangulSyllableAem',
u'\ua1f3': 'YiSyllableGux',
u'\ue2f5': 'UnicodeCharacter0xe2f5',
u'\u1a7a': 'UnicodeCharacter0x1a7a',
u'\u577c': 'CjkUnifiedIdeograph577c',
u'\u33ff': 'SquareGal',
u'\u907e': 'CjkUnifiedIdeograph907e',
u'\uad80': 'HangulSyllableGuss',
u'\u0903': 'DevanagariSignVisarga',
u'\uee82': 'UnicodeCharacter0xee82',
u'\u4a05': 'CjkUnifiedIdeograph4a05',
u'\u8707': 'CjkUnifiedIdeograph8707',
u'\uc009': 'HangulSyllableBbwenj',
u'\u3f8c': 'CjkUnifiedIdeograph3f8c',
u'\u788e': 'CjkUnifiedIdeograph788e',
u'\ub590': 'HangulSyllableDdyaels',
u'\u1113': 'HangulChoseongNieunkiyeok',
u'\uf692': 'UnicodeCharacter0xf692',
u'\u5215': 'CjkUnifiedIdeograph5215',
u'\u6f17': 'CjkUnifiedIdeograph6f17',
u'\ua819': 'SylotiNagriLetterPo',
u'\ue51b': 'UnicodeCharacter0xe51b',
u'\u079c': 'ThaanaLetterZaa',
u'\u409e': 'CjkUnifiedIdeograph409e',
u'\u9da0': 'CjkUnifiedIdeograph9da0',
u'\udea2': 'UnicodeCharacter0xdea2',
u'\u3a25': 'CjkUnifiedIdeograph3a25',
u'\u7727': 'CjkUnifiedIdeograph7727',
u'\ub029': 'HangulSyllableGgyug',
u'\ucd2b': 'HangulSyllableCwad',
u'\u28ae': 'BraillePatternDots23468',
u'\u65b0': 'CjkUnifiedIdeograph65b0',
u'\ua6b2': 'UnicodeCharacter0xa6b2',
u'\u0235': 'LatinSmallLetterNWithCurl',
u'\ue3b4': 'UnicodeCharacter0xe3b4',
u'\u5f37': 'CjkUnifiedIdeograph5f37',
u'\u9839': 'CjkUnifiedIdeograph9839',
u'\ud53b': 'HangulSyllablePyih',
u'\u30be': 'KatakanaLetterZo',
u'\u4dc0': 'HexagramForTheCreativeHeaven',
u'\u8ec2': 'CjkUnifiedIdeograph8ec2',
u'\ucbc4': 'HangulSyllableJjyuls',
u'\u2747': 'Sparkle',
u'\u6049': 'CjkUnifiedIdeograph6049',
u'\ubd4b': 'HangulSyllableBoegs',
u'\ufe4d': 'DashedLowLine',
u'\u18ce': 'UnicodeCharacter0x18ce',
u'\u55d0': 'CjkUnifiedIdeograph55d0',
u'\u96d2': 'CjkUnifiedIdeograph96d2',
u'\ud3d4': 'HangulSyllablePyen',
u'\u0f57': 'TibetanLetterBha',
u'\uecd6': 'UnicodeCharacter0xecd6',
u'\u4859': 'CjkUnifiedIdeograph4859',
u'\u855b': 'CjkUnifiedIdeograph855b',
u'\uc65d': 'HangulSyllableWaeg',
u'\u3de0': 'CjkUnifiedIdeograph3de0',
u'\u7ee2': 'CjkUnifiedIdeograph7ee2',
u'\ubbe4': 'HangulSyllableMyil',
u'\u1767': 'TagbanwaLetterDa',
u'\uf4e6': 'UnicodeCharacter0xf4e6',
u'\u5069': 'CjkUnifiedIdeograph5069',
u'\u6d6b': 'CjkUnifiedIdeograph6d6b',
u'\uae6d': 'HangulSyllableGgaenj',
u'\ueb6f': 'UnicodeCharacter0xeb6f',
u'\u05f0': 'HebrewLigatureYiddishDoubleVav',
u'\u46f2': 'CjkUnifiedIdeograph46f2',
u'\u83f4': 'CjkUnifiedIdeograph83f4',
u'\udcf6': 'UnicodeCharacter0xdcf6',
u'\u3879': 'CjkUnifiedIdeograph3879',
u'\u757b': 'CjkUnifiedIdeograph757b',
u'\ub67d': 'HangulSyllableDdoet',
u'\uf37f': 'UnicodeCharacter0xf37f',
u'\u8f81': 'CjkUnifiedIdeograph8f81',
u'\u6804': 'CjkUnifiedIdeograph6804',
u'\u4487': 'CjkUnifiedIdeograph4487',
u'\ue208': 'UnicodeCharacter0xe208',
u'\ude8b': 'UnicodeCharacter0xde8b',
u'\u210a': 'ScriptSmallG',
u'\u1d8d': 'LatinSmallLetterXWithPalatalHook',
u'\ubb0e': 'HangulSyllableMoebs',
u'\u9791': 'CjkUnifiedIdeograph9791',
u'\u7014': 'CjkUnifiedIdeograph7014',
u'\u2c97': 'CopticSmallLetterLaula',
u'\uca18': 'HangulSyllableJjyan',
u'\ua69b': 'UnicodeCharacter0xa69b',
u'\u091a': 'DevanagariLetterCa',
u'\u831e': 'CjkUnifiedIdeograph831e',
u'\u7fa1': 'CjkUnifiedIdeograph7fa1',
u'\uf9a5': 'CjkCompatibilityIdeographf9a5',
u'\u5824': 'CjkUnifiedIdeograph5824',
u'\u34a7': 'CjkUnifiedIdeograph34a7',
u'\ud228': 'HangulSyllableTyok',
u'\u8eab': 'CjkUnifiedIdeograph8eab',
u'\u112a': 'HangulChoseongPieupphieuph',
u'\u6b2e': 'CjkUnifiedIdeograph6b2e',
u'\u47b1': 'CjkUnifiedIdeograph47b1',
u'\ue532': 'UnicodeCharacter0xe532',
u'\uc1b5': 'HangulSyllableSwalt',
u'\u2034': 'TriplePrime',
u'\u1cb7': 'UnicodeCharacter0x1cb7',
u'\uba38': 'HangulSyllableMeo',
u'\u96bb': 'CjkUnifiedIdeograph96bb',
u'\u733e': 'CjkUnifiedIdeograph733e',
u'\u2fc1': 'KangxiRadicalGhost',
u'\ucd42': 'HangulSyllableCwaegg',
u'\ua9c5': 'UnicodeCharacter0xa9c5',
u'\u0844': 'UnicodeCharacter0x844',
u'\u8248': 'CjkUnifiedIdeograph8248',
u'\u7ecb': 'CjkUnifiedIdeograph7ecb',
u'\uf8cf': 'UnicodeCharacter0xf8cf',
u'\u5b4e': 'CjkUnifiedIdeograph5b4e',
u'\u37d1': 'CjkUnifiedIdeograph37d1',
u'\ud552': 'HangulSyllablePij',
u'\ub1d5': 'HangulSyllableNoelg',
u'\u1054': 'MyanmarLetterVocalicL',
u'\u6a58': 'CjkUnifiedIdeograph6a58',
u'\u46db': 'CjkUnifiedIdeograph46db',
u'\ue45c': 'UnicodeCharacter0xe45c',
u'\uc0df': 'HangulSyllableSaec',
u'\u235e': 'AplFunctionalSymbolQuoteQuad',
u'\u1fe1': 'GreekSmallLetterUpsilonWithMacron',
u'\ubd62': 'HangulSyllableBoep',
u'\u99e5': 'CjkUnifiedIdeograph99e5',
u'\u7268': 'CjkUnifiedIdeograph7268',
u'\u2eeb': 'CjkRadicalJsimplifiedEven',
u'\ucc6c': 'HangulSyllableCyals',
u'\ua8ef': 'UnicodeCharacter0xa8ef',
u'\u0b6e': 'OriyaDigitEight',
u'\u8572': 'CjkUnifiedIdeograph8572',
u'\u61f5': 'CjkUnifiedIdeograph61f5',
u'\ufbf9': 'ArabicLigatureUighurKirghizYehWithHamzaAboveWithAlefMaksuraIsolatedForm',
u'\u5a78': 'CjkUnifiedIdeograph5a78',
u'\u36fb': 'CjkUnifiedIdeograph36fb',
u'\ud47c': 'HangulSyllablePun',
u'\ub0ff': 'HangulSyllableNyaes',
u'\u137e': 'UnicodeCharacter0x137e',
u'\u8a03': 'CjkUnifiedIdeograph8a03',
u'\u6d82': 'CjkUnifiedIdeograph6d82',
u'\ue786': 'UnicodeCharacter0xe786',
u'\u4309': 'CjkUnifiedIdeograph4309',
u'\u2288': 'NeitherASubsetOfNorEqualTo',
u'\udd0d': 'UnicodeCharacter0xdd0d',
u'\ubc8c': 'HangulSyllableBeol',
u'\u180f': 'UnicodeCharacter0x180f',
u'\u9213': 'CjkUnifiedIdeograph9213',
u'\u7592': 'CjkUnifiedIdeograph7592',
u'\uec17': 'UnicodeCharacter0xec17',
u'\ucf96': 'HangulSyllableKwaelm',
u'\u2b19': 'DiamondWithBottomHalfBlack',
u'\u0a98': 'GujaratiLetterGha',
u'\ua51d': 'VaiSyllableYee',
u'\u849c': 'CjkUnifiedIdeograph849c',
u'\u7a23': 'CjkUnifiedIdeograph7a23',
u'\u5da2': 'CjkUnifiedIdeograph5da2',
u'\uf427': 'UnicodeCharacter0xf427',
u'\ud7a6': 'UnicodeCharacter0xd7a6',
u'\u3329': 'SquareNotto',
u'\u12a8': 'EthiopicSyllableKa',
u'\u8d2d': 'CjkUnifiedIdeograph8d2d',
u'\u6cac': 'CjkUnifiedIdeograph6cac',
u'\ue6b0': 'UnicodeCharacter0xe6b0',
u'\u4233': 'CjkUnifiedIdeograph4233',
u'\u25b2': 'BlackUppointingTriangle',
u'\udc37': 'UnicodeCharacter0xdc37',
u'\ubfb6': 'HangulSyllableBbyonh',
u'\u1b39': 'BalineseVowelSignSukuIlut',
u'\u953d': 'CjkUnifiedIdeograph953d',
u'\u74bc': 'CjkUnifiedIdeograph74bc',
u'\uef41': 'UnicodeCharacter0xef41',
u'\ucec0': 'HangulSyllableKyass',
u'\u2a43': 'IntersectionWithOverbar',
u'\u0dc2': 'SinhalaLetterMuurdhajaSayanna',
u'\ua447': 'YiSyllableNjyp',
u'\u87c6': 'CjkUnifiedIdeograph87c6',
u'\u7d4d': 'CjkUnifiedIdeograph7d4d',
u'\u5ccc': 'CjkUnifiedIdeograph5ccc',
u'\uf751': 'UnicodeCharacter0xf751',
u'\ud6d0': 'HangulSyllableHuls',
u'\u3253': 'CircledNumberTwentyThree',
u'\u15d2': 'CanadianSyllabicsCarrierWe',
u'\u8c57': 'CjkUnifiedIdeograph8c57',
u'\u6fd6': 'CjkUnifiedIdeograph6fd6',
u'\ue9da': 'UnicodeCharacter0xe9da',
u'\u455d': 'CjkUnifiedIdeograph455d',
u'\u24dc': 'CircledLatinSmallLetterM',
u'\udf61': 'UnicodeCharacter0xdf61',
u'\ubee0': 'HangulSyllableBbeom',
u'\u1a63': 'UnicodeCharacter0x1a63',
u'\u9467': 'CjkUnifiedIdeograph9467',
u'\u77e6': 'CjkUnifiedIdeograph77e6',
u'\uee6b': 'UnicodeCharacter0xee6b',
u'\uf1ea': 'UnicodeCharacter0xf1ea',
u'\u2d6d': 'UnicodeCharacter0x2d6d',
u'\u0cec': 'KannadaDigitSix',
u'\ua771': 'LatinSmallLetterDum',
u'\u86f0': 'CjkUnifiedIdeograph86f0',
u'\u7c77': 'CjkUnifiedIdeograph7c77',
u'\u5ff6': 'CjkUnifiedIdeograph5ff6',
u'\uf67b': 'UnicodeCharacter0xf67b',
u'\ud9fa': 'UnicodeCharacter0xd9fa',
u'\u357d': 'CjkUnifiedIdeograph357d',
u'\u14fc': 'CanadianSyllabicsSwo',
u'\u0e81': 'LaoLetterKo',
u'\u6f00': 'CjkUnifiedIdeograph6f00',
u'\u4b83': 'CjkUnifiedIdeograph4b83',
u'\uac02': 'HangulSyllableGagg',
u'\u8885': 'CjkUnifiedIdeograph8885',
u'\ue904': 'UnicodeCharacter0xe904',
u'\uc587': 'HangulSyllableYalb',
u'\u3a0e': 'CjkUnifiedIdeograph3a0e',
u'\u1691': 'OghamLetterOnn',
u'\u7710': 'CjkUnifiedIdeograph7710',
u'\u5393': 'CjkUnifiedIdeograph5393',
u'\ub412': 'HangulSyllableDwaej',
u'\u9095': 'CjkUnifiedIdeograph9095',
u'\uf114': 'UnicodeCharacter0xf114',
u'\uad97': 'HangulSyllableGweolh',
u'\uea99': 'UnicodeCharacter0xea99',
u'\u021e': 'LatinCapitalLetterHWithCaron',
u'\u5f20': 'CjkUnifiedIdeograph5f20',
u'\u3ba3': 'CjkUnifiedIdeograph3ba3',
u'\u9c22': 'CjkUnifiedIdeograph9c22',
u'\u78a5': 'CjkUnifiedIdeograph78a5',
u'\ud924': 'UnicodeCharacter0xd924',
u'\ub5a7': 'HangulSyllableDdeod',
u'\uf2a9': 'UnicodeCharacter0xf2a9',
u'\u2730': 'ShadowedWhiteStar',
u'\u03b3': 'GreekSmallLetterGamma',
u'\u6432': 'CjkUnifiedIdeograph6432',
u'\u40b5': 'CjkUnifiedIdeograph40b5',
u'\ua134': 'YiSyllableTurx',
u'\u9db7': 'CjkUnifiedIdeograph9db7',
u'\ufe36': 'PresentationFormForVerticalRightParenthesis',
u'\udab9': 'UnicodeCharacter0xdab9',
u'\u0f40': 'TibetanLetterKa',
u'\u4c42': 'CjkUnifiedIdeograph4c42',
u'\u28c5': 'BraillePatternDots1378',
u'\u8944': 'CjkUnifiedIdeograph8944',
u'\u65c7': 'CjkUnifiedIdeograph65c7',
u'\uc646': 'HangulSyllableWanh',
u'\ua2c9': 'YiSyllableCur',
u'\uffcb': 'HalfwidthHangulLetterYe',
u'\u1750': 'BuhidLetterSa',
u'\u5452': 'CjkUnifiedIdeograph5452',
u'\u30d5': 'KatakanaLetterHu',
u'\u9154': 'CjkUnifiedIdeograph9154',
u'\u4dd7': 'HexagramForReturn',
u'\uae56': 'HangulSyllableGgalm',
u'\u8ad9': 'CjkUnifiedIdeograph8ad9',
u'\ueb58': 'UnicodeCharacter0xeb58',
u'\uc7db': 'HangulSyllableJyas',
u'\u3c62': 'CjkUnifiedIdeograph3c62',
u'\u18e5': 'UnicodeCharacter0x18e5',
u'\u7964': 'CjkUnifiedIdeograph7964',
u'\u55e7': 'CjkUnifiedIdeograph55e7',
u'\ub666': 'HangulSyllableDdoegg',
u'\u92e9': 'CjkUnifiedIdeograph92e9',
u'\uf368': 'UnicodeCharacter0xf368',
u'\uafeb': 'HangulSyllableGgweoc',
u'\ueced': 'UnicodeCharacter0xeced',
u'\u0472': 'CyrillicCapitalLetterFita',
u'\u4174': 'CjkUnifiedIdeograph4174',
u'\u3df7': 'CjkUnifiedIdeograph3df7',
u'\u9e76': 'CjkUnifiedIdeograph9e76',
u'\u7af9': 'CjkUnifiedIdeograph7af9',
u'\udb78': 'UnicodeCharacter0xdb78',
u'\ub7fb': 'HangulSyllableReolh',
u'\uf4fd': 'UnicodeCharacter0xf4fd',
u'\uce01': 'HangulSyllableCwit',
u'\u2984': 'RightWhiteCurlyBracket',
u'\u6686': 'CjkUnifiedIdeograph6686',
u'\ua388': 'YiSyllableRrut',
u'\u1f0b': 'GreekCapitalLetterAlphaWithDasiaAndVaria',
u'\ue08a': 'UnicodeCharacter0xe08a',
u'\u5c0d': 'CjkUnifiedIdeograph5c0d',
u'\u990f': 'CjkUnifiedIdeograph990f',
u'\ud611': 'HangulSyllableHyeob',
u'\u3194': 'IdeographicAnnotationThreeMark',
u'\u4e96': 'CjkUnifiedIdeograph4e96',
u'\u8b98': 'CjkUnifiedIdeograph8b98',
u'\uc89a': 'HangulSyllableJwalp',
u'\u241d': 'SymbolForGroupSeparator',
u'\u611f': 'CjkUnifiedIdeograph611f',
u'\ube21': 'HangulSyllableBeung',
u'\ufb23': 'HebrewLetterWideHe',
u'\u19a4': 'NewTaiLueLetterLowDa',
u'\u56a6': 'CjkUnifiedIdeograph56a6',
u'\u93a8': 'CjkUnifiedIdeograph93a8',
u'\ud0aa': 'HangulSyllableKinh',
u'\u0c2d': 'TeluguLetterBha',
u'\uedac': 'UnicodeCharacter0xedac',
u'\u492f': 'CjkUnifiedIdeograph492f',
u'\u8631': 'CjkUnifiedIdeograph8631',
u'\uc333': 'HangulSyllableSsyags',
u'\u3eb6': 'CjkUnifiedIdeograph3eb6',
u'\u7bb8': 'CjkUnifiedIdeograph7bb8',
u'\ub8ba': 'HangulSyllableRoelm',
u'\u143d': 'CanadianSyllabicsWestcreePwi',
u'\uf5bc': 'UnicodeCharacter0xf5bc',
u'\u513f': 'CjkUnifiedIdeograph513f',
u'\u6e41': 'CjkUnifiedIdeograph6e41',
u'\uab43': 'UnicodeCharacter0xab43',
u'\ue845': 'UnicodeCharacter0xe845',
u'\u06c6': 'ArabicLetterOe',
u'\u43c8': 'CjkUnifiedIdeograph43c8',
u'\u80ca': 'CjkUnifiedIdeograph80ca',
u'\uddcc': 'UnicodeCharacter0xddcc',
u'\u394f': 'CjkUnifiedIdeograph394f',
u'\u7651': 'CjkUnifiedIdeograph7651',
u'\ub353': 'HangulSyllableDyaeh',
u'\uf055': 'UnicodeCharacter0xf055',
u'\u2bd8': 'UnicodeCharacter0x2bd8',
u'\u68da': 'CjkUnifiedIdeograph68da',
u'\ua5dc': 'VaiSyllableNggo',
u'\u015f': 'LatinSmallLetterSWithCedilla',
u'\ue2de': 'UnicodeCharacter0xe2de',
u'\u5e61': 'CjkUnifiedIdeograph5e61',
u'\u9b63': 'CjkUnifiedIdeograph9b63',
u'\ud865': 'UnicodeCharacter0xd865',
u'\u33e8': 'IdeographicTelegraphSymbolForDayNine',
u'\u70ea': 'CjkUnifiedIdeograph70ea',
u'\u8dec': 'CjkUnifiedIdeograph8dec',
u'\ucaee': 'HangulSyllableJjwaj',
u'\u2671': 'EastSyriacCross',
u'\u6373': 'CjkUnifiedIdeograph6373',
u'\ua075': 'YiSyllableBbyp',
u'\ufd77': 'ArabicLigatureAinWithMeemWithMeemInitialForm',
u'\u1bf8': 'UnicodeCharacter0x1bf8',
u'\u58fa': 'CjkUnifiedIdeograph58fa',
u'\u95fc': 'CjkUnifiedIdeograph95fc',
u'\ud2fe': 'HangulSyllableTilp',
u'\uf000': 'UnicodeCharacter0xf000',
u'\ud483': 'HangulSyllablePulb',
u'\u3302': 'SquareAnpea',
u'\u1785': 'KhmerLetterCa',
u'\ub906': 'HangulSyllableRweogg',
u'\u9d89': 'CjkUnifiedIdeograph9d89',
u'\u620c': 'CjkUnifiedIdeograph620c',
u'\u468f': 'CjkUnifiedIdeograph468f',
u'\ue810': 'UnicodeCharacter0xe810',
u'\ucc93': 'HangulSyllableCyaec',
u'\u2b12': 'SquareWithTopHalfBlack',
u'\u0f95': 'TibetanSubjoinedLetterCa',
u'\u9116': 'CjkUnifiedIdeograph9116',
u'\u7599': 'CjkUnifiedIdeograph7599',
u'\ufb9d': 'ArabicLetterNgoehMedialForm',
u'\u5a1c': 'CjkUnifiedIdeograph5a1c',
u'\u3e9f': 'CjkUnifiedIdeograph3e9f',
u'\uc020': 'HangulSyllableBbwi',
u'\ua4a3': 'YiRadicalCyt',
u'\u0322': 'CombiningRetroflexHookBelow',
u'\u8926': 'CjkUnifiedIdeograph8926',
u'\u6da9': 'CjkUnifiedIdeograph6da9',
u'\ud3ad': 'HangulSyllablePeng',
u'\u322c': 'ParenthesizedIdeographWater',
u'\u16af': 'RunicLetterOe',
u'\ub830': 'HangulSyllableRyeols',
u'\u9cb3': 'CjkUnifiedIdeograph9cb3',
u'\u6136': 'CjkUnifiedIdeograph6136',
u'\u45b9': 'CjkUnifiedIdeograph45b9',
u'\ue73a': 'UnicodeCharacter0xe73a',
u'\ucbbd': 'HangulSyllableJjyunj',
u'\u2a3c': 'InteriorProduct',
u'\u0ebf': 'UnicodeCharacter0xebf',
u'\u9040': 'CjkUnifiedIdeograph9040',
u'\u74c3': 'CjkUnifiedIdeograph74c3',
u'\ufac7': 'CjkCompatibilityIdeographfac7',
u'\u5946': 'CjkUnifiedIdeograph5946',
u'\u3dc9': 'CjkUnifiedIdeograph3dc9',
u'\udf4a': 'UnicodeCharacter0xdf4a',
u'\ua3cd': 'YiSyllableRot',
u'\u024c': 'LatinCapitalLetterRWithStroke',
u'\u8850': 'CjkUnifiedIdeograph8850',
u'\u6cd3': 'CjkUnifiedIdeograph6cd3',
u'\ud2d7': 'HangulSyllableTyigs',
u'\u3156': 'HangulLetterYe',
u'\u15d9': 'CanadianSyllabicsCarrierHwee',
u'\ub75a': 'HangulSyllableDdyij',
u'\u9bdd': 'CjkUnifiedIdeograph9bdd',
u'\u6060': 'CjkUnifiedIdeograph6060',
u'\u44e3': 'CjkUnifiedIdeograph44e3',
u'\ue664': 'UnicodeCharacter0xe664',
u'\ucae7': 'HangulSyllableJjwalh',
u'\u2966': 'LeftwardsHarpoonWithBarbUpAboveRightwardsHarpoonWithBarbUp',
u'\u0de9': 'UnicodeCharacter0xde9',
u'\uaf6a': 'HangulSyllableGgwaenh',
u'\u73ed': 'CjkUnifiedIdeograph73ed',
u'\uf9f1': 'CjkCompatibilityIdeographf9f1',
u'\u5870': 'CjkUnifiedIdeograph5870',
u'\u3cf3': 'CjkUnifiedIdeograph3cf3',
u'\ude74': 'UnicodeCharacter0xde74',
u'\ua2f7': 'YiSyllableNzap',
u'\u0176': 'LatinCapitalLetterYWithCircumflex',
u'\u877a': 'CjkUnifiedIdeograph877a',
u'\u6bfd': 'CjkUnifiedIdeograph6bfd',
u'\u5101': 'CjkUnifiedIdeograph5101',
u'\u3080': 'HiraganaLetterMu',
u'\ud705': 'HangulSyllableHwelg',
u'\ub684': 'HangulSyllableDdyon',
u'\u1a07': 'BugineseLetterMpa',
u'\u800b': 'CjkUnifiedIdeograph800b',
u'\u7f8a': 'CjkUnifiedIdeograph7f8a',
u'\ue58e': 'UnicodeCharacter0xe58e',
u'\u4911': 'CjkUnifiedIdeograph4911',
u'\u2890': 'BraillePatternDots58',
u'\ucf15': 'HangulSyllableKeng',
u'\uae94': 'HangulSyllableGgyam',
u'\u781b': 'CjkUnifiedIdeograph781b',
u'\u579a': 'CjkUnifiedIdeograph579a',
u'\ufe1f': 'UnicodeCharacter0xfe1f',
u'\udd9e': 'UnicodeCharacter0xdd9e',
u'\u2121': 'TelephoneSign',
u'\xa0': 'NobreakSpace',
u'\ua725': 'LatinSmallLetterEgyptologicalAin',
u'\u86a4': 'CjkUnifiedIdeograph86a4',
u'\u502b': 'CjkUnifiedIdeograph502b',
u'\u4faa': 'CjkUnifiedIdeograph4faa',
u'\ud62f': 'HangulSyllableHyes',
u'\ub5ae': 'HangulSyllableDdeolp',
u'\u1931': 'LimbuSmallLetterNga',
u'\u9f35': 'CjkUnifiedIdeograph9f35',
u'\u7eb4': 'CjkUnifiedIdeograph7eb4',
u'\ue4b8': 'UnicodeCharacter0xe4b8',
u'\u483b': 'CjkUnifiedIdeograph483b',
u'\u27ba': 'TeardropbarbedRightwardsArrow',
u'\uce3f': 'HangulSyllableCyigs',
u'\uadbe': 'HangulSyllableGwep',
u'\u7745': 'CjkUnifiedIdeograph7745',
u'\u56c4': 'CjkUnifiedIdeograph56c4',
u'\ufd49': 'UnicodeCharacter0xfd49',
u'\udcc8': 'UnicodeCharacter0xdcc8',
u'\u204b': 'ReversedPilcrowSign',
u'\u1fca': 'GreekCapitalLetterEtaWithVaria',
u'\ua64f': 'CyrillicSmallLetterNeutralYer',
u'\u85ce': 'CjkUnifiedIdeograph85ce',
u'\u6f55': 'CjkUnifiedIdeograph6f55',
u'\u4ed4': 'CjkUnifiedIdeograph4ed4',
u'\ud559': 'HangulSyllableHag',
u'\ub4d8': 'HangulSyllableDyuk',
u'\u185b': 'MongolianLetterTodoNia',
u'\u9e5f': 'CjkUnifiedIdeograph9e5f',
u'\u7dde': 'CjkUnifiedIdeograph7dde',
u'\ue3e2': 'UnicodeCharacter0xe3e2',
u'\u4765': 'CjkUnifiedIdeograph4765',
u'\u26e4': 'UnicodeCharacter0x26e4',
u'\ucd69': 'HangulSyllableCoelt',
u'\uace8': 'HangulSyllableGol',
u'\u766f': 'CjkUnifiedIdeograph766f',
u'\u55ee': 'CjkUnifiedIdeograph55ee',
u'\ufc73': 'ArabicLigatureTehWithNoonFinalForm',
u'\udbf2': 'UnicodeCharacter0xdbf2',
u'\u3f75': 'CjkUnifiedIdeograph3f75',
u'\u1ef4': 'LatinCapitalLetterYWithDotBelow',
u'\ua579': 'VaiSyllableMboo',
u'\u84f8': 'CjkUnifiedIdeograph84f8',
u'\u6e7f': 'CjkUnifiedIdeograph6e7f',
u'\u4dfe': 'HexagramForAfterCompletion',
u'\u3806': 'CjkUnifiedIdeograph3806',
u'\u1c89': 'UnicodeCharacter0x1c89',
u'\u7d08': 'CjkUnifiedIdeograph7d08',
u'\u418b': 'CjkUnifiedIdeograph418b',
u'\ube0a': 'HangulSyllableByup',
u'\u828d': 'CjkUnifiedIdeograph828d',
u'\ue30c': 'UnicodeCharacter0xe30c',
u'\uc78f': 'HangulSyllableIh',
u'\u1016': 'MyanmarLetterPha',
u'\u5518': 'CjkUnifiedIdeograph5518',
u'\u399b': 'CjkUnifiedIdeograph399b',
u'\u961a': 'CjkUnifiedIdeograph961a',
u'\u7a9d': 'CjkUnifiedIdeograph7a9d',
u'\udb1c': 'UnicodeCharacter0xdb1c',
u'\ubf9f': 'HangulSyllableBboelb',
u'\ue0a1': 'UnicodeCharacter0xe0a1',
u'\u0826': 'UnicodeCharacter0x826',
u'\u4d28': 'CjkUnifiedIdeograph4d28',
u'\u11ab': 'HangulJongseongNieun',
u'\u8e2a': 'CjkUnifiedIdeograph8e2a',
u'\u52ad': 'CjkUnifiedIdeograph52ad',
u'\ub32c': 'HangulSyllableDyam',
u'\u97af': 'CjkUnifiedIdeograph97af',
u'\uf42e': 'UnicodeCharacter0xf42e',
u'\ud8b1': 'UnicodeCharacter0xd8b1',
u'\u2538': 'BoxDrawingsUpHeavyAndHorizontalLight',
u'\u09bb': 'UnicodeCharacter0x9bb',
u'\u663a': 'CjkUnifiedIdeograph663a',
u'\u4abd': 'CjkUnifiedIdeograph4abd',
u'\uab3c': 'UnicodeCharacter0xab3c',
u'\u8fbf': 'CjkUnifiedIdeograph8fbf',
u'\uec3e': 'UnicodeCharacter0xec3e',
u'\ub0c1': 'HangulSyllableNaelt',
u'\uf5c3': 'UnicodeCharacter0xf5c3',
u'\u1d48': 'ModifierLetterSmallD',
u'\u5e4a': 'CjkUnifiedIdeograph5e4a',
u'\u22cd': 'ReversedTildeEquals',
u'\u834c': 'CjkUnifiedIdeograph834c',
u'\u67cf': 'CjkUnifiedIdeograph67cf',
u'\uc44e': 'HangulSyllableSsyonh',
u'\ua8d1': 'SaurashtraDigitOne',
u'\uedd3': 'UnicodeCharacter0xedd3',
u'\u365a': 'CjkUnifiedIdeograph365a',
u'\u1add': 'UnicodeCharacter0x1add',
u'\u7b5c': 'CjkUnifiedIdeograph7b5c',
u'\u5fdf': 'CjkUnifiedIdeograph5fdf',
u'\ubc5e': 'HangulSyllableByabs',
u'\u80e1': 'CjkUnifiedIdeograph80e1',
u'\ue160': 'UnicodeCharacter0xe160',
u'\uc5e3': 'HangulSyllableEs',
u'\u2e6a': 'UnicodeCharacter0x2e6a',
u'\u536c': 'CjkUnifiedIdeograph536c',
u'\u37ef': 'CjkUnifiedIdeograph37ef',
u'\u946e': 'CjkUnifiedIdeograph946e',
u'\u78f1': 'CjkUnifiedIdeograph78f1',
u'\ud970': 'UnicodeCharacter0xd970',
u'\ubdf3': 'HangulSyllableByugs',
u'\ufef5': 'ArabicLigatureLamWithAlefWithMaddaAboveIsolatedForm',
u'\u067a': 'ArabicLetterTteheh',
u'\u4b7c': 'CjkUnifiedIdeograph4b7c',
u'\u2fff': 'UnicodeCharacter0x2fff',
u'\u8c7e': 'CjkUnifiedIdeograph8c7e',
u'\ub180': 'HangulSyllableNol',
u'\u1503': 'CanadianSyllabicsWestcreeSwaa',
u'\uf282': 'UnicodeCharacter0xf282',
u'\u5605': 'CjkUnifiedIdeograph5605',
u'\u9b07': 'CjkUnifiedIdeograph9b07',
u'\udc09': 'UnicodeCharacter0xdc09',
u'\u238c': 'UndoSymbol',
u'\u648e': 'CjkUnifiedIdeograph648e',
u'\ua990': 'UnicodeCharacter0xa990',
u'\u0d13': 'MalayalamLetterOo',
u'\uea92': 'UnicodeCharacter0xea92',
u'\u4e15': 'CjkUnifiedIdeograph4e15',
u'\u7317': 'CjkUnifiedIdeograph7317',
u'\ub419': 'HangulSyllableDoeg',
u'\uf91b': 'CjkCompatibilityIdeographf91b',
u'\u1b9c': 'SundaneseLetterLa',
u'\u5c9e': 'CjkUnifiedIdeograph5c9e',
u'\u81a0': 'CjkUnifiedIdeograph81a0',
u'\uc2a2': 'HangulSyllableSyup',
u'\u2625': 'Ankh',
u'\u6b27': 'CjkUnifiedIdeograph6b27',
u'\uac29': 'HangulSyllableGaelt',
u'\ud12b': 'HangulSyllableTyaec',
u'\u34ae': 'CjkUnifiedIdeograph34ae',
u'\u79b0': 'CjkUnifiedIdeograph79b0',
u'\ubab2': 'HangulSyllableMolm',
u'\u1e35': 'LatinSmallLetterKWithLineBelow',
u'\uffb4': 'HalfwidthHangulLetterPieupsios',
u'\u4337': 'CjkUnifiedIdeograph4337',
u'\u8439': 'CjkUnifiedIdeograph8439',
u'\uc93b': 'HangulSyllableJwed',
u'\u2cbe': 'CopticCapitalLetterOldCopticOou',
u'\u51c0': 'CjkUnifiedIdeograph51c0',
u'\u92c2': 'CjkUnifiedIdeograph92c2',
u'\ud7c4': 'UnicodeCharacter0xd7c4',
u'\u3b47': 'CjkUnifiedIdeograph3b47',
u'\u7c49': 'CjkUnifiedIdeograph7c49',
u'\ua14b': 'YiSyllableDdut',
u'\ue24d': 'UnicodeCharacter0xe24d',
u'\u04ce': 'CyrillicSmallLetterEmWithTail',
u'\u49d0': 'CjkUnifiedIdeograph49d0',
u'\u8ad2': 'CjkUnifiedIdeograph8ad2',
u'\ucfd4': 'HangulSyllableKyom',
u'\u1357': 'EthiopicSyllablePwa',
u'\uf0d6': 'UnicodeCharacter0xf0d6',
u'\u5459': 'CjkUnifiedIdeograph5459',
u'\u995b': 'CjkUnifiedIdeograph995b',
u'\uda5d': 'UnicodeCharacter0xda5d',
u'\u21e0': 'LeftwardsDashedArrow',
u'\u62e2': 'CjkUnifiedIdeograph62e2',
u'\ua7e4': 'UnicodeCharacter0xa7e4',
u'\u0b67': 'OriyaDigitOne',
u'\ue8e6': 'UnicodeCharacter0xe8e6',
u'\u4c69': 'CjkUnifiedIdeograph4c69',
u'\u716b': 'CjkUnifiedIdeograph716b',
u'\ub26d': 'HangulSyllableNwing',
u'\uf76f': 'UnicodeCharacter0xf76f',
u'\u19f0': 'KhmerSymbolTuteyasat',
u'\u5af2': 'CjkUnifiedIdeograph5af2',
u'\u9ff4': 'UnicodeCharacter0x9ff4',
u'\uc0f6': 'HangulSyllableSyabs',
u'\u2479': 'ParenthesizedDigitSix',
u'\u697b': 'CjkUnifiedIdeograph697b',
u'\uaa7d': 'UnicodeCharacter0xaa7d',
u'\uef7f': 'UnicodeCharacter0xef7f',
u'\u9381': 'CjkUnifiedIdeograph9381',
u'\u7404': 'CjkUnifiedIdeograph7404',
u'\u5887': 'CjkUnifiedIdeograph5887',
u'\ufe08': 'VariationSelector9',
u'\uc28b': 'HangulSyllableSyugs',
u'\u3d0a': 'CjkUnifiedIdeograph3d0a',
u'\u018d': 'LatinSmallLetterTurnedDelta',
u'\ua70e': 'ModifierLetterHighDottedLeftstemToneBar',
u'\u8b91': 'CjkUnifiedIdeograph8b91',
u'\u6c14': 'CjkUnifiedIdeograph6c14',
u'\u3097': 'UnicodeCharacter0x3097',
u'\ud618': 'HangulSyllableHyeok',
u'\uba9b': 'HangulSyllableMyelh',
u'\u151a': 'CanadianSyllabicsWestcreeShwi',
u'\u9f1e': 'CjkUnifiedIdeograph9f1e',
u'\u63a1': 'CjkUnifiedIdeograph63a1',
u'\ue5a5': 'UnicodeCharacter0xe5a5',
u'\u4424': 'CjkUnifiedIdeograph4424',
u'\u28a7': 'BraillePatternDots12368',
u'\uce28': 'HangulSyllableCeul',
u'\u92ab': 'CjkUnifiedIdeograph92ab',
u'\u0d2a': 'MalayalamLetterPa',
u'\u772e': 'CjkUnifiedIdeograph772e',
u'\u5bb1': 'CjkUnifiedIdeograph5bb1',
u'\uf932': 'CjkCompatibilityIdeographf932',
u'\uddb5': 'UnicodeCharacter0xddb5',
u'\u3c34': 'CjkUnifiedIdeograph3c34',
u'\xb7': 'MiddleDot',
u'\ua638': 'UnicodeCharacter0xa638',
u'\u8abb': 'CjkUnifiedIdeograph8abb',
u'\u6f3e': 'CjkUnifiedIdeograph6f3e',
u'\u33c1': 'SquareMOhm',
u'\ud142': 'HangulSyllableTeobs',
u'\ub5c5': 'HangulSyllableDdelg',
u'\u1444': 'CanadianSyllabicsPwa',
u'\u9e48': 'CjkUnifiedIdeograph9e48',
u'\u62cb': 'CjkUnifiedIdeograph62cb',
u'\ue4cf': 'UnicodeCharacter0xe4cf',
u'\u474e': 'CjkUnifiedIdeograph474e',
u'\u2bd1': 'UnicodeCharacter0x2bd1',
u'\uc952': 'HangulSyllableJwigg',
u'\uadd5': 'HangulSyllableGwing',
u'\u0c54': 'UnicodeCharacter0xc54',
u'\u7658': 'CjkUnifiedIdeograph7658',
u'\u5adb': 'CjkUnifiedIdeograph5adb',
u'\uf85c': 'UnicodeCharacter0xf85c',
u'\udcdf': 'UnicodeCharacter0xdcdf',
u'\u3f5e': 'CjkUnifiedIdeograph3f5e',
u'\u03e1': 'GreekSmallLetterSampi',
u'\ua162': 'YiSyllableNdut',
u'\u85e5': 'CjkUnifiedIdeograph85e5',
u'\u6e68': 'CjkUnifiedIdeograph6e68',
u'\u32eb': 'CircledKatakanaHu',
u'\ud06c': 'HangulSyllableKeu',
u'\ub4ef': 'HangulSyllableDeus',
u'\u176e': 'TagbanwaLetterLa',
u'\u9972': 'CjkUnifiedIdeograph9972',
u'\u7df5': 'CjkUnifiedIdeograph7df5',
u'\ue7f9': 'UnicodeCharacter0xe7f9',
u'\u4678': 'CjkUnifiedIdeograph4678',
u'\u2afb': 'TripleSolidusBinaryRelation',
u'\uc87c': 'HangulSyllableJols',
u'\uacff': 'HangulSyllableGwags',
u'\u0f7e': 'TibetanSignRjesSuNgaRo',
u'\u9603': 'CjkUnifiedIdeograph9603',
u'\u7182': 'CjkUnifiedIdeograph7182',
u'\ufb86': 'ArabicLetterDulIsolatedForm',
u'\u5f09': 'CjkUnifiedIdeograph5f09',
u'\u3e88': 'CjkUnifiedIdeograph3e88',
u'\uc10d': 'HangulSyllableSyaelt',
u'\ua08c': 'YiSyllableNbyx',
u'\u040f': 'CyrillicCapitalLetterDzhe',
u'\u8e13': 'CjkUnifiedIdeograph8e13',
u'\u6992': 'CjkUnifiedIdeograph6992',
u'\uf017': 'UnicodeCharacter0xf017',
u'\ud396': 'HangulSyllablePeop',
u'\u3719': 'CjkUnifiedIdeograph3719',
u'\u1698': 'OghamLetterIfin',
u'\ub91d': 'HangulSyllableRweot',
u'\u989c': 'CjkUnifiedIdeograph989c',
u'\u6623': 'CjkUnifiedIdeograph6623',
u'\u41a2': 'CjkUnifiedIdeograph41a2',
u'\ue827': 'UnicodeCharacter0xe827',
u'\ucba6': 'HangulSyllableJjwilm',
u'\u2f29': 'KangxiRadicalSmall',
u'\u0ea8': 'UnicodeCharacter0xea8',
u'\u912d': 'CjkUnifiedIdeograph912d',
u'\u70ac': 'CjkUnifiedIdeograph70ac',
u'\ufab0': 'CjkCompatibilityIdeographfab0',
u'\u5e33': 'CjkUnifiedIdeograph5e33',
u'\u39b2': 'CjkUnifiedIdeograph39b2',
u'\uc037': 'HangulSyllableBbwic',
u'\ua3b6': 'YiSyllableShet',
u'\u0739': 'SyriacDottedZlamaAngular',
u'\u893d': 'CjkUnifiedIdeograph893d',
u'\u68bc': 'CjkUnifiedIdeograph68bc',
u'\uf341': 'UnicodeCharacter0xf341',
u'\ud2c0': 'HangulSyllableTeul',
u'\u3643': 'CjkUnifiedIdeograph3643',
u'\u11c2': 'HangulJongseongHieuh',
u'\ub847': 'HangulSyllableRyed',
u'\u9bc6': 'CjkUnifiedIdeograph9bc6',
u'\u614d': 'CjkUnifiedIdeograph614d',
u'\u40cc': 'CjkUnifiedIdeograph40cc',
u'\ueb51': 'UnicodeCharacter0xeb51',
u'\ucad0': 'HangulSyllableJjoss',
u'\u2e53': 'UnicodeCharacter0x2e53',
u'\u09d2': 'UnicodeCharacter0x9d2',
u'\u9057': 'CjkUnifiedIdeograph9057',
u'\u73d6': 'CjkUnifiedIdeograph73d6',
u'\uf5da': 'UnicodeCharacter0xf5da',
u'\u595d': 'CjkUnifiedIdeograph595d',
u'\u38dc': 'CjkUnifiedIdeograph38dc',
u'\uc361': 'HangulSyllableSsyaeng',
u'\ua2e0': 'YiSyllableZze',
u'\u0663': 'ArabicindicDigitThree',
u'\u8867': 'CjkUnifiedIdeograph8867',
u'\u6be6': 'CjkUnifiedIdeograph6be6',
u'\uf26b': 'UnicodeCharacter0xf26b',
u'\uedea': 'UnicodeCharacter0xedea',
u'\u316d': 'HangulLetterRieulyeorinhieuh',
u'\u10ec': 'GeorgianLetterCil',
u'\ubb71': 'HangulSyllableMwenj',
u'\u9af0': 'CjkUnifiedIdeograph9af0',
u'\u6077': 'CjkUnifiedIdeograph6077',
u'\u43f6': 'CjkUnifiedIdeograph43f6',
u'\uea7b': 'UnicodeCharacter0xea7b',
u'\uc5fa': 'HangulSyllableYeolp',
u'\u297d': 'RightFishTail',
u'\u08fc': 'UnicodeCharacter0x8fc',
u'\u1281': 'EthiopicSyllableXu',
u'\u7300': 'CjkUnifiedIdeograph7300',
u'\u5783': 'CjkUnifiedIdeograph5783',
u'\ub002': 'HangulSyllableGgwebs',
u'\u9485': 'CjkUnifiedIdeograph9485',
u'\uf504': 'UnicodeCharacter0xf504',
u'\ud987': 'UnicodeCharacter0xd987',
u'\u260e': 'BlackTelephone',
u'\u0a91': 'GujaratiVowelCandraO',
u'\u6b10': 'CjkUnifiedIdeograph6b10',
u'\u4f93': 'CjkUnifiedIdeograph4f93',
u'\ua812': 'SylotiNagriLetterDdo',
u'\u8c95': 'CjkUnifiedIdeograph8c95',
u'\ued14': 'UnicodeCharacter0xed14',
u'\ub197': 'HangulSyllableNwags',
u'\uf699': 'UnicodeCharacter0xf699',
u'\u1e1e': 'LatinCapitalLetterFWithDotAbove',
u'\u4320': 'CjkUnifiedIdeograph4320',
u'\u27a3': 'ThreedBottomlightedRightwardsArrowhead',
u'\u8022': 'CjkUnifiedIdeograph8022',
u'\u64a5': 'CjkUnifiedIdeograph64a5',
u'\uc524': 'HangulSyllableSsyik',
u'\ua9a7': 'UnicodeCharacter0xa9a7',
u'\ueea9': 'UnicodeCharacter0xeea9',
u'\u3b30': 'CjkUnifiedIdeograph3b30',
u'\u1fb3': 'GreekSmallLetterAlphaWithYpogegrammeni',
u'\u7832': 'CjkUnifiedIdeograph7832',
u'\u5cb5': 'CjkUnifiedIdeograph5cb5',
u'\ubd34': 'HangulSyllableBwael',
u'\u81b7': 'CjkUnifiedIdeograph81b7',
u'\ue236': 'UnicodeCharacter0xe236',
u'\uc6b9': 'HangulSyllableUlg',
u'\u1340': 'EthiopicSyllableTza',
u'\u5042': 'CjkUnifiedIdeograph5042',
u'\u34c5': 'CjkUnifiedIdeograph34c5',
u'\u9544': 'CjkUnifiedIdeograph9544',
u'\u79c7': 'CjkUnifiedIdeograph79c7',
u'\uda46': 'UnicodeCharacter0xda46',
u'\ubec9': 'HangulSyllableBbyaeng',
u'\ue3cb': 'UnicodeCharacter0xe3cb',
u'\u0b50': 'UnicodeCharacter0xb50',
u'\u4852': 'CjkUnifiedIdeograph4852',
u'\u2cd5': 'CopticSmallLetterOldCopticHat',
u'\u8d54': 'CjkUnifiedIdeograph8d54',
u'\u51d7': 'CjkUnifiedIdeograph51d7',
u'\ub256': 'HangulSyllableNwep',
u'\u96d9': 'CjkUnifiedIdeograph96d9',
u'\uf758': 'UnicodeCharacter0xf758',
u'\udbdb': 'UnicodeCharacter0xdbdb',
u'\u2062': 'InvisibleTimes',
u'\u04e5': 'CyrillicSmallLetterIWithDiaeresis',
u'\u6564': 'CjkUnifiedIdeograph6564',
u'\u49e7': 'CjkUnifiedIdeograph49e7',
u'\uaa66': 'UnicodeCharacter0xaa66',
u'\u8ee9': 'CjkUnifiedIdeograph8ee9',
u'\uef68': 'UnicodeCharacter0xef68',
u'\ub3eb': 'HangulSyllableDwalb',
u'\uf0ed': 'UnicodeCharacter0xf0ed',
u'\u1872': 'MongolianLetterSibeZha',
u'\u5d74': 'CjkUnifiedIdeograph5d74',
u'\u21f7': 'LeftwardsArrowWithVerticalStroke',
u'\u8276': 'CjkUnifiedIdeograph8276',
u'\u66f9': 'CjkUnifiedIdeograph66f9',
u'\uc778': 'HangulSyllableIn',
u'\uabfb': 'UnicodeCharacter0xabfb',
u'\ue8fd': 'UnicodeCharacter0xe8fd',
u'\ud201': 'HangulSyllableToelt',
u'\u3584': 'CjkUnifiedIdeograph3584',
u'\u7a86': 'CjkUnifiedIdeograph7a86',
u'\ubf88': 'HangulSyllableBbwaem',
u'\u030b': 'CombiningDoubleAcuteAccent',
u'\ufc8a': 'ArabicLigatureNoonWithRehFinalForm',
u'\u400d': 'CjkUnifiedIdeograph400d',
u'\u850f': 'CjkUnifiedIdeograph850f',
u'\uca11': 'HangulSyllableJjaet',
u'\u2d94': 'EthiopicSyllableGgwi',
u'\u5296': 'CjkUnifiedIdeograph5296',
u'\u9798': 'CjkUnifiedIdeograph9798',
u'\ud49a': 'HangulSyllablePweonh',
u'\u381d': 'CjkUnifiedIdeograph381d',
u'\u7d1f': 'CjkUnifiedIdeograph7d1f',
u'\ua221': 'YiSyllableGguop',
u'\ue723': 'UnicodeCharacter0xe723',
u'\u05a4': 'HebrewAccentMahapakh',
u'\u4aa6': 'CjkUnifiedIdeograph4aa6',
u'\u8fa8': 'CjkUnifiedIdeograph8fa8',
u'\uccaa': 'HangulSyllableCeobs',
u'\u102d': 'MyanmarVowelSignI',
u'\uf1ac': 'UnicodeCharacter0xf1ac',
u'\u552f': 'CjkUnifiedIdeograph552f',
u'\u9a31': 'CjkUnifiedIdeograph9a31',
u'\udf33': 'UnicodeCharacter0xdf33',
u'\u22b6': 'OriginalOf',
u'\u67b8': 'CjkUnifiedIdeograph67b8',
u'\ua4ba': 'YiRadicalShur',
u'\u083d': 'UnicodeCharacter0x83d',
u'\ue9bc': 'UnicodeCharacter0xe9bc',
u'\u4d3f': 'CjkUnifiedIdeograph4d3f',
u'\u7241': 'CjkUnifiedIdeograph7241',
u'\ub743': 'HangulSyllableDdeuh',
u'\uf445': 'UnicodeCharacter0xf445',
u'\u1ac6': 'UnicodeCharacter0x1ac6',
u'\u5fc8': 'CjkUnifiedIdeograph5fc8',
u'\u9cca': 'CjkUnifiedIdeograph9cca',
u'\uc1cc': 'HangulSyllableSwael',
u'\u254f': 'BoxDrawingsHeavyDoubleDashVertical',
u'\u6a51': 'CjkUnifiedIdeograph6a51',
u'\uaf53': 'HangulSyllableGgwalb',
u'\uec55': 'UnicodeCharacter0xec55',
u'\u37d8': 'CjkUnifiedIdeograph37d8',
u'\u74da': 'CjkUnifiedIdeograph74da',
u'\ub9dc': 'HangulSyllableMass',
u'\u1d5f': 'ModifierLetterSmallDelta',
u'\ufede': 'ArabicLetterLamFinalForm',
u'\u4261': 'CjkUnifiedIdeograph4261',
u'\u8763': 'CjkUnifiedIdeograph8763',
u'\uc465': 'HangulSyllableSsug',
u'\u2fe8': 'UnicodeCharacter0x2fe8',
u'\u6cea': 'CjkUnifiedIdeograph6cea',
u'\u91ec': 'CjkUnifiedIdeograph91ec',
u'\ud6ee': 'HangulSyllableHweolp',
u'\u3a71': 'CjkUnifiedIdeograph3a71',
u'\u7f73': 'CjkUnifiedIdeograph7f73',
u'\ubc75': 'HangulSyllableByaelt',
u'\ue177': 'UnicodeCharacter0xe177',
u'\u07f8': 'NkoComma',
u'\u44fa': 'CjkUnifiedIdeograph44fa',
u'\u89fc': 'CjkUnifiedIdeograph89fc',
u'\ucefe': 'HangulSyllableKeop',
u'\uf400': 'UnicodeCharacter0xf400',
u'\ud083': 'HangulSyllableKeuc',
u'\u3702': 'CjkUnifiedIdeograph3702',
u'\u1385': 'EthiopicSyllableBwi',
u'\ubd06': 'HangulSyllableBobs',
u'\u9989': 'CjkUnifiedIdeograph9989',
u'\u660c': 'CjkUnifiedIdeograph660c',
u'\u428f': 'CjkUnifiedIdeograph428f',
u'\uec10': 'UnicodeCharacter0xec10',
u'\uc893': 'HangulSyllableJwad',
u'\u2f12': 'KangxiRadicalPower',
u'\u0b95': 'TamilLetterKa',
u'\u9516': 'CjkUnifiedIdeograph9516',
u'\u7199': 'CjkUnifiedIdeograph7199',
u'\uff9d': 'HalfwidthKatakanaLetterN',
u'\u5e1c': 'CjkUnifiedIdeograph5e1c',
u'\u3a9f': 'CjkUnifiedIdeograph3a9f',
u'\uc420': 'HangulSyllableSswaem',
u'\ua0a3': 'YiSyllableHmut',
u'\u0722': 'SyriacLetterNun',
u'\u8d26': 'CjkUnifiedIdeograph8d26',
u'\u69a9': 'CjkUnifiedIdeograph69a9',
u'\ud7ad': 'UnicodeCharacter0xd7ad',
u'\u362c': 'CjkUnifiedIdeograph362c',
u'\u12af': 'EthiopicSyllableKoa',
u'\ubc30': 'HangulSyllableBae',
u'\u98b3': 'CjkUnifiedIdeograph98b3',
u'\u6536': 'CjkUnifiedIdeograph6536',
u'\u41b9': 'CjkUnifiedIdeograph41b9',
u'\ue33a': 'UnicodeCharacter0xe33a',
u'\ucfbd': 'HangulSyllableKoeng',
u'\u2e3c': 'UnicodeCharacter0x2e3c',
u'\u0abf': 'GujaratiVowelSignI',
u'\u9440': 'CjkUnifiedIdeograph9440',
u'\u70c3': 'CjkUnifiedIdeograph70c3',
u'\ufec7': 'ArabicLetterZahInitialForm',
u'\u5d46': 'CjkUnifiedIdeograph5d46',
u'\u39c9': 'CjkUnifiedIdeograph39c9',
u'\udb4a': 'UnicodeCharacter0xdb4a',
u'\ua7cd': 'UnicodeCharacter0xa7cd',
u'\u064c': 'ArabicDammatan',
u'\u8c50': 'CjkUnifiedIdeograph8c50',
u'\u68d3': 'CjkUnifiedIdeograph68d3',
u'\ud6d7': 'HangulSyllableHus',
u'\u3556': 'CjkUnifiedIdeograph3556',
u'\u11d9': 'HangulJongseongRieulyeorinhieuh',
u'\ub35a': 'HangulSyllableDeonh',
u'\u9fdd': 'UnicodeCharacter0x9fdd',
u'\u6460': 'CjkUnifiedIdeograph6460',
u'\u40e3': 'CjkUnifiedIdeograph40e3',
u'\ue264': 'UnicodeCharacter0xe264',
u'\ucee7': 'HangulSyllableKeogs',
u'\u2d66': 'UnicodeCharacter0x2d66',
u'\u09e9': 'BengaliDigitThree',
u'\uab6a': 'UnicodeCharacter0xab6a',
u'\u77ed': 'CjkUnifiedIdeograph77ed',
u'\ufdf1': 'ArabicLigatureQalaUsedAsKoranicStopSignIsolatedForm',
u'\u5c70': 'CjkUnifiedIdeograph5c70',
u'\u38f3': 'CjkUnifiedIdeograph38f3',
u'\uda74': 'UnicodeCharacter0xda74',
u'\ua6f7': 'UnicodeCharacter0xa6f7',
u'\u0576': 'ArmenianSmallLetterNow',
u'\u837a': 'CjkUnifiedIdeograph837a',
u'\u6ffd': 'CjkUnifiedIdeograph6ffd',
u'\u5501': 'CjkUnifiedIdeograph5501',
u'\u3480': 'CjkUnifiedIdeograph3480',
u'\ud305': 'HangulSyllableTing',
u'\ub284': 'HangulSyllableNyum',
u'\u1e07': 'LatinSmallLetterBWithLineBelow',
u'\u840b': 'CjkUnifiedIdeograph840b',
u'\u7b8a': 'CjkUnifiedIdeograph7b8a',
u'\ue18e': 'UnicodeCharacter0xe18e',
u'\u4d11': 'CjkUnifiedIdeograph4d11',
u'\u2c90': 'CopticCapitalLetterThethe',
u'\ucb15': 'HangulSyllableJjoenj',
u'\uaa94': 'UnicodeCharacter0xaa94',
u'\u7c1b': 'CjkUnifiedIdeograph7c1b',
u'\u539a': 'CjkUnifiedIdeograph539a',
u'\ufa1f': 'CjkCompatibilityIdeographfa1f',
u'\ud99e': 'UnicodeCharacter0xd99e',
u'\u2521': 'BoxDrawingsDownLightAndRightUpHeavy',
u'\u04a0': 'CyrillicCapitalLetterBashkirKa',
u'\ua325': 'YiSyllableSyx',
u'\u82a4': 'CjkUnifiedIdeograph82a4',
u'\u542b': 'CjkUnifiedIdeograph542b',
u'\u4baa': 'CjkUnifiedIdeograph4baa',
u'\ud22f': 'HangulSyllableTugs',
u'\ub1ae': 'HangulSyllableNwap',
u'\u1d31': 'ModifierLetterCapitalE',
u'\u9b35': 'CjkUnifiedIdeograph9b35',
u'\u7ab4': 'CjkUnifiedIdeograph7ab4',
u'\ue0b8': 'UnicodeCharacter0xe0b8',
u'\u4c3b': 'CjkUnifiedIdeograph4c3b',
u'\u23ba': 'HorizontalScanLine1',
u'\uca3f': 'HangulSyllableJjyaelh',
u'\ua9be': 'UnicodeCharacter0xa9be',
u'\u7345': 'CjkUnifiedIdeograph7345',
u'\u52c4': 'CjkUnifiedIdeograph52c4',
u'\uf949': 'CjkCompatibilityIdeographf949',
u'\ud8c8': 'UnicodeCharacter0xd8c8',
u'\u244b': 'UnicodeCharacter0x244b',
u'\u1bca': 'UnicodeCharacter0x1bca',
u'\ua24f': 'YiSyllableHxax',
u'\u81ce': 'CjkUnifiedIdeograph81ce',
u'\u6b55': 'CjkUnifiedIdeograph6b55',
u'\u4ad4': 'CjkUnifiedIdeograph4ad4',
u'\ud159': 'HangulSyllableTelt',
u'\ub0d8': 'HangulSyllableNyal',
u'\u1c5b': 'OlChikiLetterAt',
u'\u9a5f': 'CjkUnifiedIdeograph9a5f',
u'\u79de': 'CjkUnifiedIdeograph79de',
u'\ue7e2': 'UnicodeCharacter0xe7e2',
u'\u4365': 'CjkUnifiedIdeograph4365',
u'\u22e4': 'SquareImageOfOrNotEqualTo',
u'\uc969': 'HangulSyllableJwit',
u'\ua8e8': 'UnicodeCharacter0xa8e8',
u'\u726f': 'CjkUnifiedIdeograph726f',
u'\u51ee': 'CjkUnifiedIdeograph51ee',
u'\uf873': 'UnicodeCharacter0xf873',
u'\udff2': 'UnicodeCharacter0xdff2',
u'\u3b75': 'CjkUnifiedIdeograph3b75',
u'\u1af4': 'UnicodeCharacter0x1af4',
u'\ua179': 'YiSyllableHnex',
u'\u80f8': 'CjkUnifiedIdeograph80f8',
u'\u6a7f': 'CjkUnifiedIdeograph6a7f',
u'\u49fe': 'CjkUnifiedIdeograph49fe',
u'\u3c06': 'CjkUnifiedIdeograph3c06',
u'\u1889': 'MongolianLetterAliGaliKa',
u'\u7908': 'CjkUnifiedIdeograph7908',
u'\u458b': 'CjkUnifiedIdeograph458b',
u'\uba0a': 'HangulSyllableMyalm',
u'\u868d': 'CjkUnifiedIdeograph868d',
u'\ue70c': 'UnicodeCharacter0xe70c',
u'\uc38f': 'HangulSyllableSselb',
u'\u1416': 'CanadianSyllabicsNaskapiWoo',
u'\u5118': 'CjkUnifiedIdeograph5118',
u'\u3d9b': 'CjkUnifiedIdeograph3d9b',
u'\u921a': 'CjkUnifiedIdeograph921a',
u'\u7e9d': 'CjkUnifiedIdeograph7e9d',
u'\udf1c': 'UnicodeCharacter0xdf1c',
u'\ubb9f': 'HangulSyllableMwic',
u'\ue4a1': 'UnicodeCharacter0xe4a1',
u'\u0c26': 'TeluguLetterDa',
u'\u4928': 'CjkUnifiedIdeograph4928',
u'\u15ab': 'CanadianSyllabicsThcreeThoo',
u'\u8a2a': 'CjkUnifiedIdeograph8a2a',
u'\u56ad': 'CjkUnifiedIdeograph56ad',
u'\ub72c': 'HangulSyllableDdeun',
u'\u93af': 'CjkUnifiedIdeograph93af',
u'\uf02e': 'UnicodeCharacter0xf02e',
u'\udcb1': 'UnicodeCharacter0xdcb1',
u'\u2138': 'DaletSymbol',
u'\u0dbb': 'SinhalaLetterRayanna',
u'\u623a': 'CjkUnifiedIdeograph623a',
u'\u4ebd': 'CjkUnifiedIdeograph4ebd',
u'\uaf3c': 'HangulSyllableGgom',
u'\u8bbf': 'CjkUnifiedIdeograph8bbf',
u'\ue83e': 'UnicodeCharacter0xe83e',
u'\ub4c1': 'HangulSyllableDyug',
u'\uf1c3': 'UnicodeCharacter0xf1c3',
u'\u1948': 'LimbuDigitTwo',
u'\u5a4a': 'CjkUnifiedIdeograph5a4a',
u'\u26cd': 'UnicodeCharacter0x26cd',
u'\u874c': 'CjkUnifiedIdeograph874c',
u'\u63cf': 'CjkUnifiedIdeograph63cf',
u'\uc04e': 'HangulSyllableBbyubs',
u'\uacd1': 'HangulSyllableGyelt',
u'\ue9d3': 'UnicodeCharacter0xe9d3',
u'\u325a': 'CircledNumberThirty',
u'\u1edd': 'LatinSmallLetterOWithHornAndGrave',
u'\u7f5c': 'CjkUnifiedIdeograph7f5c',
u'\u5bdf': 'CjkUnifiedIdeograph5bdf',
u'\ub85e': 'HangulSyllableRogg',
u'\u84e1': 'CjkUnifiedIdeograph84e1',
u'\ue560': 'UnicodeCharacter0xe560',
u'\uc1e3': 'HangulSyllableSoegs',
u'\u2a6a': 'TildeOperatorWithDotAbove',
u'\u576c': 'CjkUnifiedIdeograph576c',
u'\u33ef': 'IdeographicTelegraphSymbolForDaySixteen',
u'\u906e': 'CjkUnifiedIdeograph906e',
u'\u7cf1': 'CjkUnifiedIdeograph7cf1',
u'\udd70': 'UnicodeCharacter0xdd70',
u'\ub9f3': 'HangulSyllableMaelh',
u'\ufaf5': 'UnicodeCharacter0xfaf5',
u'\u027a': 'LatinSmallLetterTurnedRWithLongLeg',
u'\u4f7c': 'CjkUnifiedIdeograph4f7c',
u'\u2bff': 'UnicodeCharacter0x2bff',
u'\u887e': 'CjkUnifiedIdeograph887e',
u'\ub580': 'HangulSyllableDdyak',
u'\u1103': 'HangulChoseongTikeut',
u'\uf682': 'UnicodeCharacter0xf682',
u'\u5205': 'CjkUnifiedIdeograph5205',
u'\u9f07': 'CjkUnifiedIdeograph9f07',
u'\ud809': 'UnicodeCharacter0xd809',
u'\u278c': 'DingbatNegativeCircledSansserifDigitThree',
u'\u608e': 'CjkUnifiedIdeograph608e',
u'\uad90': 'HangulSyllableGweol',
u'\u0913': 'DevanagariLetterO',
u'\uee92': 'UnicodeCharacter0xee92',
u'\u4a15': 'CjkUnifiedIdeograph4a15',
u'\u7717': 'CjkUnifiedIdeograph7717',
u'\ub019': 'HangulSyllableGgwilt',
u'\ufd1b': 'ArabicLigatureHahWithAlefMaksuraFinalForm',
u'\u1f9c': 'GreekCapitalLetterEtaWithPsiliAndOxiaAndProsgegrammeni',
u'\u589e': 'CjkUnifiedIdeograph589e',
u'\u85a0': 'CjkUnifiedIdeograph85a0',
u'\uc6a2': 'HangulSyllableYolp',
u'\u2225': 'ParallelTo',
u'\u6f27': 'CjkUnifiedIdeograph6f27',
u'\ua829': 'SylotiNagriPoetryMark2',
u'\ud52b': 'HangulSyllablePyilb',
u'\u30ae': 'KatakanaLetterGi',
u'\u7db0': 'CjkUnifiedIdeograph7db0',
u'\ubeb2': 'HangulSyllableBbyap',
u'\u1a35': 'UnicodeCharacter0x1a35',
u'\ufbb4': 'UnicodeCharacter0xfbb4',
u'\u4737': 'CjkUnifiedIdeograph4737',
u'\u8039': 'CjkUnifiedIdeograph8039',
u'\ucd3b': 'HangulSyllableCwac',
u'\u28be': 'BraillePatternDots234568',
u'\u55c0': 'CjkUnifiedIdeograph55c0',
u'\u96c2': 'CjkUnifiedIdeograph96c2',
u'\ud3c4': 'HangulSyllablePyeom',
u'\u3f47': 'CjkUnifiedIdeograph3f47',
u'\u7849': 'CjkUnifiedIdeograph7849',
u'\ua54b': 'VaiSyllableNgan',
u'\ue64d': 'UnicodeCharacter0xe64d',
u'\xce': 'LatinCapitalLetterIWithCircumflex',
u'\u4dd0': 'HexagramForFollowing',
u'\u8ed2': 'CjkUnifiedIdeograph8ed2',
u'\ucbd4': 'HangulSyllableJjeu',
u'\u1757': 'UnicodeCharacter0x1757',
u'\uf4d6': 'UnicodeCharacter0xf4d6',
u'\u5059': 'CjkUnifiedIdeograph5059',
u'\u9d5b': 'CjkUnifiedIdeograph9d5b',
u'\ude5d': 'UnicodeCharacter0xde5d',
u'\u25e0': 'UpperHalfCircle',
u'\u66e2': 'CjkUnifiedIdeograph66e2',
u'\ua3e4': 'YiSyllableJiet',
u'\u0f67': 'TibetanLetterHa',
u'\uece6': 'UnicodeCharacter0xece6',
u'\u4869': 'CjkUnifiedIdeograph4869',
u'\u756b': 'CjkUnifiedIdeograph756b',
u'\ub66d': 'HangulSyllableDdoelg',
u'\uf36f': 'UnicodeCharacter0xf36f',
u'\u1df0': 'UnicodeCharacter0x1df0',
u'\u5ef2': 'CjkUnifiedIdeograph5ef2',
u'\u9bf4': 'CjkUnifiedIdeograph9bf4',
u'\uc4f6': 'HangulSyllableSseunh',
u'\u2079': 'SuperscriptNine',
u'\u6d7b': 'CjkUnifiedIdeograph6d7b',
u'\uae7d': 'HangulSyllableGgaeng',
u'\ueb7f': 'UnicodeCharacter0xeb7f',
u'\u9781': 'CjkUnifiedIdeograph9781',
u'\u7004': 'CjkUnifiedIdeograph7004',
u'\u5c87': 'CjkUnifiedIdeograph5c87',
u'\ufa08': 'CjkCompatibilityIdeographfa08',
u'\uc68b': 'HangulSyllableOes',
u'\u390a': 'CjkUnifiedIdeograph390a',
u'\u058d': 'UnicodeCharacter0x58d',
u'\ua30e': 'YiSyllableSie',
u'\u8f91': 'CjkUnifiedIdeograph8f91',
u'\u6814': 'CjkUnifiedIdeograph6814',
u'\u3497': 'CjkUnifiedIdeograph3497',
u'\ud218': 'HangulSyllableTyol',
u'\ube9b': 'HangulSyllableBbyags',
u'\u111a': 'HangulChoseongRieulhieuh',
u'\u9b1e': 'CjkUnifiedIdeograph9b1e',
u'\u67a1': 'CjkUnifiedIdeograph67a1',
u'\ue1a5': 'UnicodeCharacter0xe1a5',
u'\u4024': 'CjkUnifiedIdeograph4024',
u'\u2ca7': 'CopticSmallLetterTau',
u'\uca28': 'HangulSyllableJjyass',
u'\u96ab': 'CjkUnifiedIdeograph96ab',
u'\u092a': 'DevanagariLetterPa',
u'\u732e': 'CjkUnifiedIdeograph732e',
u'\u5fb1': 'CjkUnifiedIdeograph5fb1',
u'\ufd32': 'ArabicLigatureSheenWithHehInitialForm',
u'\ud9b5': 'UnicodeCharacter0xd9b5',
u'\u3834': 'CjkUnifiedIdeograph3834',
u'\u04b7': 'CyrillicSmallLetterCheWithDescender',
u'\ua238': 'YiSyllableMguop',
u'\u8ebb': 'CjkUnifiedIdeograph8ebb',
u'\u6b3e': 'CjkUnifiedIdeograph6b3e',
u'\u37c1': 'CjkUnifiedIdeograph37c1',
u'\ud542': 'HangulSyllablePinh',
u'\ub1c5': 'HangulSyllableNwaeng',
u'\u1044': 'MyanmarDigitFour',
u'\u9a48': 'CjkUnifiedIdeograph9a48',
u'\u66cb': 'CjkUnifiedIdeograph66cb',
u'\ue0cf': 'UnicodeCharacter0xe0cf',
u'\u434e': 'CjkUnifiedIdeograph434e',
u'\u2fd1': 'KangxiRadicalEven',
u'\ucd52': 'HangulSyllableCwaebs',
u'\ua9d5': 'UnicodeCharacter0xa9d5',
u'\u0854': 'UnicodeCharacter0x854',
u'\u7258': 'CjkUnifiedIdeograph7258',
u'\u5edb': 'CjkUnifiedIdeograph5edb',
u'\ufc5c': 'ArabicLigatureRehWithSuperscriptAlefIsolatedForm',
u'\ud8df': 'UnicodeCharacter0xd8df',
u'\u3b5e': 'CjkUnifiedIdeograph3b5e',
u'\u07e1': 'NkoLetterMa',
u'\ua562': 'VaiSyllableSa',
u'\u81e5': 'CjkUnifiedIdeograph81e5',
u'\u6a68': 'CjkUnifiedIdeograph6a68',
u'\u36eb': 'CjkUnifiedIdeograph36eb',
u'\ud46c': 'HangulSyllablePyom',
u'\ub0ef': 'HangulSyllableNyaegs',
u'\u136e': 'EthiopicDigitSix',
u'\u9d72': 'CjkUnifiedIdeograph9d72',
u'\u79f5': 'CjkUnifiedIdeograph79f5',
u'\ue3f9': 'UnicodeCharacter0xe3f9',
u'\u4278': 'CjkUnifiedIdeograph4278',
u'\u2efb': 'UnicodeCharacter0x2efb',
u'\ucc7c': 'HangulSyllableCyae',
u'\ua8ff': 'UnicodeCharacter0xa8ff',
u'\u0b7e': 'UnicodeCharacter0xb7e',
u'\u9203': 'CjkUnifiedIdeograph9203',
u'\u7582': 'CjkUnifiedIdeograph7582',
u'\uff86': 'HalfwidthKatakanaLetterNi',
u'\u5b09': 'CjkUnifiedIdeograph5b09',
u'\u3a88': 'CjkUnifiedIdeograph3a88',
u'\uc50d': 'HangulSyllableSsyig',
u'\ua48c': 'YiSyllableYyr',
u'\u8a13': 'CjkUnifiedIdeograph8a13',
u'\u6d92': 'CjkUnifiedIdeograph6d92',
u'\uf417': 'UnicodeCharacter0xf417',
u'\ud796': 'HangulSyllableHilp',
u'\u3319': 'SquareGuramuton',
u'\u1298': 'EthiopicSyllableNya',
u'\ubd1d': 'HangulSyllableBwalt',
u'\u9c9c': 'CjkUnifiedIdeograph9c9c',
u'\u6223': 'CjkUnifiedIdeograph6223',
u'\u45a2': 'CjkUnifiedIdeograph45a2',
u'\uec27': 'UnicodeCharacter0xec27',
u'\ucfa6': 'HangulSyllableKwaep',
u'\u2b29': 'BlackSmallDiamond',
u'\u0aa8': 'GujaratiLetterNa',
u'\u952d': 'CjkUnifiedIdeograph952d',
u'\u74ac': 'CjkUnifiedIdeograph74ac',
u'\ufeb0': 'ArabicLetterZainFinalForm',
u'\u5a33': 'CjkUnifiedIdeograph5a33',
u'\u3db2': 'CjkUnifiedIdeograph3db2',
u'\uc437': 'HangulSyllableSsoelb',
u'\ua7b6': 'UnicodeCharacter0xa7b6',
u'\u0339': 'CombiningRightHalfRingBelow',
u'\u8d3d': 'CjkUnifiedIdeograph8d3d',
u'\u6cbc': 'CjkUnifiedIdeograph6cbc',
u'\uf741': 'UnicodeCharacter0xf741',
u'\ud6c0': 'HangulSyllableHyok',
u'\u3243': 'ParenthesizedIdeographReach',
u'\u15c2': 'CanadianSyllabicsSayisiHo',
u'\ubc47': 'HangulSyllableBaec',
u'\u9fc6': 'UnicodeCharacter0x9fc6',
u'\u654d': 'CjkUnifiedIdeograph654d',
u'\u44cc': 'CjkUnifiedIdeograph44cc',
u'\uef51': 'UnicodeCharacter0xef51',
u'\uced0': 'HangulSyllableKyael',
u'\u2a53': 'DoubleLogicalAnd',
u'\u0dd2': 'SinhalaVowelSignKettiIspilla',
u'\u9457': 'CjkUnifiedIdeograph9457',
u'\u77d6': 'CjkUnifiedIdeograph77d6',
u'\uf1da': 'UnicodeCharacter0xf1da',
u'\u5d5d': 'CjkUnifiedIdeograph5d5d',
u'\u3cdc': 'CjkUnifiedIdeograph3cdc',
u'\uc761': 'HangulSyllableYilg',
u'\ua6e0': 'UnicodeCharacter0xa6e0',
u'\u0263': 'LatinSmallLetterGamma',
u'\u8c67': 'CjkUnifiedIdeograph8c67',
u'\u6fe6': 'CjkUnifiedIdeograph6fe6',
u'\uf66b': 'UnicodeCharacter0xf66b',
u'\ue9ea': 'UnicodeCharacter0xe9ea',
u'\u356d': 'CjkUnifiedIdeograph356d',
u'\u14ec': 'CanadianSyllabicsMedialL',
u'\ubf71': 'HangulSyllableBbwang',
u'\u9ef0': 'CjkUnifiedIdeograph9ef0',
u'\u6477': 'CjkUnifiedIdeograph6477',
u'\u47f6': 'CjkUnifiedIdeograph47f6',
u'\uee7b': 'UnicodeCharacter0xee7b',
u'\uc1fa': 'HangulSyllableSoep',
u'\u2d7d': 'UnicodeCharacter0x2d7d',
u'\u0cfc': 'UnicodeCharacter0xcfc',
u'\u1681': 'OghamLetterBeith',
u'\u7700': 'CjkUnifiedIdeograph7700',
u'\u5383': 'CjkUnifiedIdeograph5383',
u'\ub402': 'HangulSyllableDwaenh',
u'\u9085': 'CjkUnifiedIdeograph9085',
u'\uf104': 'UnicodeCharacter0xf104',
u'\udd87': 'UnicodeCharacter0xdd87',
u'\u220e': 'EndOfProof',
u'\u0e91': 'UnicodeCharacter0xe91',
u'\u6f10': 'CjkUnifiedIdeograph6f10',
u'\u4b93': 'CjkUnifiedIdeograph4b93',
u'\uac12': 'HangulSyllableGabs',
u'\u8895': 'CjkUnifiedIdeograph8895',
u'\ue914': 'UnicodeCharacter0xe914',
u'\ub597': 'HangulSyllableDdyaes',
u'\uf299': 'UnicodeCharacter0xf299',
u'\u1a1e': 'BuginesePallawa',
u'\u4720': 'CjkUnifiedIdeograph4720',
u'\u23a3': 'LeftSquareBracketLowerCorner',
u'\u8422': 'CjkUnifiedIdeograph8422',
u'\u60a5': 'CjkUnifiedIdeograph60a5',
u'\uc124': 'HangulSyllableSeol',
u'\uada7': 'HangulSyllableGwegs',
u'\ueaa9': 'UnicodeCharacter0xeaa9',
u'\u3f30': 'CjkUnifiedIdeograph3f30',
u'\u1bb3': 'SundaneseDigitThree',
u'\u7c32': 'CjkUnifiedIdeograph7c32',
u'\u58b5': 'CjkUnifiedIdeograph58b5',
u'\ub934': 'HangulSyllableRwess',
u'\u85b7': 'CjkUnifiedIdeograph85b7',
u'\ue636': 'UnicodeCharacter0xe636',
u'\uc2b9': 'HangulSyllableSeung',
u'\u1740': 'BuhidLetterA',
u'\u5442': 'CjkUnifiedIdeograph5442',
u'\u30c5': 'KatakanaLetterDu',
u'\u9144': 'CjkUnifiedIdeograph9144',
u'\u7dc7': 'CjkUnifiedIdeograph7dc7',
u'\ude46': 'UnicodeCharacter0xde46',
u'\ubac9': 'HangulSyllableMwanj',
u'\ue7cb': 'UnicodeCharacter0xe7cb',
u'\u0f50': 'TibetanLetterTha',
u'\u4c52': 'CjkUnifiedIdeograph4c52',
u'\u28d5': 'BraillePatternDots13578',
u'\u8954': 'CjkUnifiedIdeograph8954',
u'\u55d7': 'CjkUnifiedIdeograph55d7',
u'\ub656': 'HangulSyllableDdwaelp',
u'\u92d9': 'CjkUnifiedIdeograph92d9',
u'\uf358': 'UnicodeCharacter0xf358',
u'\udfdb': 'UnicodeCharacter0xdfdb',
u'\u2462': 'CircledDigitThree',
u'\xe5': 'LatinSmallLetterAWithRingAbove',
u'\u6164': 'CjkUnifiedIdeograph6164',
u'\u4de7': 'HexagramForDeliverance',
u'\uae66': 'HangulSyllableGgap',
u'\u8ae9': 'CjkUnifiedIdeograph8ae9',
u'\ueb68': 'UnicodeCharacter0xeb68',
u'\ub7eb': 'HangulSyllableRyaeh',
u'\uf4ed': 'UnicodeCharacter0xf4ed',
u'\u1c72': 'OlChikiLetterErr',
u'\u5974': 'CjkUnifiedIdeograph5974',
u'\u25f7': 'WhiteCircleWithUpperRightQuadrant',
u'\u8676': 'CjkUnifiedIdeograph8676',
u'\u62f9': 'CjkUnifiedIdeograph62f9',
u'\uc378': 'HangulSyllableSseom',
u'\uaffb': 'HangulSyllableGgwelb',
u'\uecfd': 'UnicodeCharacter0xecfd',
u'\ud601': 'HangulSyllableHyeog',
u'\u3184': 'HangulLetterKapyeounphieuph',
u'\u7e86': 'CjkUnifiedIdeograph7e86',
u'\ubb88': 'HangulSyllableMwi',
u'\u070b': 'SyriacHarkleanObelus',
u'\uf88a': 'UnicodeCharacter0xf88a',
u'\u440d': 'CjkUnifiedIdeograph440d',
u'\u810f': 'CjkUnifiedIdeograph810f',
u'\uce11': 'HangulSyllableCyult',
u'\u2994': 'RightArcGreaterthanBracket',
u'\u5696': 'CjkUnifiedIdeograph5696',
u'\u9398': 'CjkUnifiedIdeograph9398',
u'\ud09a': 'HangulSyllableKyibs',
u'\u3c1d': 'CjkUnifiedIdeograph3c1d',
u'\u791f': 'CjkUnifiedIdeograph791f',
u'\ua621': 'VaiDigitOne',
u'\ue323': 'UnicodeCharacter0xe323',
u'\u01a4': 'LatinCapitalLetterPWithHook',
u'\u4ea6': 'CjkUnifiedIdeograph4ea6',
u'\u8ba8': 'CjkUnifiedIdeograph8ba8',
u'\uc8aa': 'HangulSyllableJwaegg',
u'\u142d': 'CanadianSyllabicsOn',
u'\uf5ac': 'UnicodeCharacter0xf5ac',
u'\u512f': 'CjkUnifiedIdeograph512f',
u'\u9e31': 'CjkUnifiedIdeograph9e31',
u'\udb33': 'UnicodeCharacter0xdb33',
u'\u26b6': 'Vesta',
u'\u63b8': 'CjkUnifiedIdeograph63b8',
u'\ua0ba': 'YiSyllableMuox',
u'\u0c3d': 'TeluguSignAvagraha',
u'\uedbc': 'UnicodeCharacter0xedbc',
u'\u493f': 'CjkUnifiedIdeograph493f',
u'\u7641': 'CjkUnifiedIdeograph7641',
u'\ub343': 'HangulSyllableDyaelb',
u'\uf045': 'UnicodeCharacter0xf045',
u'\u1ec6': 'LatinCapitalLetterEWithCircumflexAndDotBelow',
u'\u5bc8': 'CjkUnifiedIdeograph5bc8',
u'\u98ca': 'CjkUnifiedIdeograph98ca',
u'\uc5cc': 'HangulSyllableEok',
u'\u214f': 'SymbolForSamaritanSource',
u'\u6e51': 'CjkUnifiedIdeograph6e51',
u'\uab53': 'UnicodeCharacter0xab53',
u'\ue855': 'UnicodeCharacter0xe855',
u'\u33d8': 'SquarePm',
u'\u70da': 'CjkUnifiedIdeograph70da',
u'\ubddc': 'HangulSyllableBwil',
u'\u195f': 'TaiLeLetterQa',
u'\ufade': 'UnicodeCharacter0xfade',
u'\u4661': 'CjkUnifiedIdeograph4661',
u'\u8363': 'CjkUnifiedIdeograph8363',
u'\uc065': 'HangulSyllableBbeult',
u'\u2be8': 'UnicodeCharacter0x2be8',
u'\u68ea': 'CjkUnifiedIdeograph68ea',
u'\u95ec': 'CjkUnifiedIdeograph95ec',
u'\ud2ee': 'HangulSyllableTyip',
u'\u3e71': 'CjkUnifiedIdeograph3e71',
u'\u7b73': 'CjkUnifiedIdeograph7b73',
u'\ub875': 'HangulSyllableRot',
u'\ue577': 'UnicodeCharacter0xe577',
u'\u03f8': 'GreekSmallLetterSho',
u'\u40fa': 'CjkUnifiedIdeograph40fa',
u'\u8dfc': 'CjkUnifiedIdeograph8dfc',
u'\ucafe': 'HangulSyllableJjwaelm',
u'\uf800': 'UnicodeCharacter0xf800',
u'\udc83': 'UnicodeCharacter0xdc83',
u'\u3b02': 'CjkUnifiedIdeograph3b02',
u'\u1f85': 'GreekSmallLetterAlphaWithDasiaAndOxiaAndYpogegrammeni',
u'\ub106': 'HangulSyllableNyaep',
u'\u9589': 'CjkUnifiedIdeograph9589',
u'\u6a0c': 'CjkUnifiedIdeograph6a0c',
u'\u4e8f': 'CjkUnifiedIdeograph4e8f',
u'\ue010': 'UnicodeCharacter0xe010',
u'\uc493': 'HangulSyllableSsweos',
u'\u2312': 'Arc',
u'\u0795': 'ThaanaLetterPaviyani',
u'\u9916': 'CjkUnifiedIdeograph9916',
u'\u7d99': 'CjkUnifiedIdeograph7d99',
u'\uf39d': 'UnicodeCharacter0xf39d',
u'\u521c': 'CjkUnifiedIdeograph521c',
u'\u369f': 'CjkUnifiedIdeograph369f',
u'\uc820': 'HangulSyllableJen',
u'\uaca3': 'HangulSyllableGec',
u'\u0b22': 'OriyaLetterDdha',
u'\u8126': 'CjkUnifiedIdeograph8126',
u'\u65a9': 'CjkUnifiedIdeograph65a9',
u'\udbad': 'UnicodeCharacter0xdbad',
u'\u3a2c': 'CjkUnifiedIdeograph3a2c',
u'\u1eaf': 'LatinSmallLetterAWithBreveAndAcute',
u'\ub030': 'HangulSyllableGgyul',
u'\u94b3': 'CjkUnifiedIdeograph94b3',
u'\u6936': 'CjkUnifiedIdeograph6936',
u'\u4db9': 'UnicodeCharacter0x4db9',
u'\uef3a': 'UnicodeCharacter0xef3a',
u'\uc3bd': 'HangulSyllableSsyeg',
u'\u223c': 'TildeOperator',
u'\u06bf': 'ArabicLetterTchehWithDotAbove',
u'\u9840': 'CjkUnifiedIdeograph9840',
u'\u7cc3': 'CjkUnifiedIdeograph7cc3',
u'\uf2c7': 'UnicodeCharacter0xf2c7',
u'\u5146': 'CjkUnifiedIdeograph5146',
u'\u35c9': 'CjkUnifiedIdeograph35c9',
u'\ud74a': 'HangulSyllableHyuj',
u'\uabcd': 'UnicodeCharacter0xabcd',
u'\u0a4c': 'GurmukhiVowelSignAu',
u'\u8050': 'CjkUnifiedIdeograph8050',
u'\u64d3': 'CjkUnifiedIdeograph64d3',
u'\udad7': 'UnicodeCharacter0xdad7',
u'\u3956': 'CjkUnifiedIdeograph3956',
u'\u1dd9': 'CombiningLatinSmallLetterEth',
u'\ubf5a': 'HangulSyllableBbop',
u'\u93dd': 'CjkUnifiedIdeograph93dd',
u'\u6860': 'CjkUnifiedIdeograph6860',
u'\u4ce3': 'CjkUnifiedIdeograph4ce3',
u'\uee64': 'UnicodeCharacter0xee64',
u'\uc2e7': 'HangulSyllableSilb',
u'\u2166': 'RomanNumeralSeven',
u'\u05e9': 'HebrewLetterShin',
u'\ua76a': 'LatinCapitalLetterEt',
u'\u7bed': 'CjkUnifiedIdeograph7bed',
u'\uf1f1': 'UnicodeCharacter0xf1f1',
u'\u5070': 'CjkUnifiedIdeograph5070',
u'\u34f3': 'CjkUnifiedIdeograph34f3',
u'\ud674': 'HangulSyllableHwaen',
u'\uaaf7': 'UnicodeCharacter0xaaf7',
u'\u0976': 'UnicodeCharacter0x976',
u'\u8f7a': 'CjkUnifiedIdeograph8f7a',
u'\u63fd': 'CjkUnifiedIdeograph63fd',
u'\u5901': 'CjkUnifiedIdeograph5901',
u'\u3880': 'CjkUnifiedIdeograph3880',
u'\udf05': 'UnicodeCharacter0xdf05',
u'\ube84': 'HangulSyllableBbael',
u'\u1207': 'EthiopicSyllableHoa',
u'\u880b': 'CjkUnifiedIdeograph880b',
u'\u778a': 'CjkUnifiedIdeograph778a',
u'\ued8e': 'UnicodeCharacter0xed8e',
u'\u4111': 'CjkUnifiedIdeograph4111',
u'\u2090': 'LatinSubscriptSmallLetterA',
u'\uc715': 'HangulSyllableWib',
u'\ua694': 'CyrillicCapitalLetterHwe',
u'\u701b': 'CjkUnifiedIdeograph701b',
u'\u5f9a': 'CjkUnifiedIdeograph5f9a',
u'\uf61f': 'UnicodeCharacter0xf61f',
u'\ud59e': 'HangulSyllableHyalp',
u'\u2921': 'NorthWestAndSouthEastArrow',
u'\u08a0': 'UnicodeCharacter0x8a0',
u'\uaf25': 'HangulSyllableGgyeng',
u'\u8ea4': 'CjkUnifiedIdeograph8ea4',
u'\u582b': 'CjkUnifiedIdeograph582b',
u'\u47aa': 'CjkUnifiedIdeograph47aa',
u'\ude2f': 'UnicodeCharacter0xde2f',
u'\ubdae': 'HangulSyllableBweobs',
u'\u1131': 'HangulChoseongSiosmieum',
u'\u9735': 'CjkUnifiedIdeograph9735',
u'\u76b4': 'CjkUnifiedIdeograph76b4',
u'\uecb8': 'UnicodeCharacter0xecb8',
u'\u403b': 'CjkUnifiedIdeograph403b',
u'\u2fba': 'KangxiRadicalHorse',
u'\uc63f': 'HangulSyllableOh',
u'\ua5be': 'VaiSyllableHon',
u'\u7f45': 'CjkUnifiedIdeograph7f45',
u'\u5ec4': 'CjkUnifiedIdeograph5ec4',
u'\uf549': 'UnicodeCharacter0xf549',
u'\ud4c8': 'HangulSyllablePwek',
u'\u284b': 'BraillePatternDots1247',
u'\u17ca': 'KhmerSignTriisap',
u'\uae4f': 'HangulSyllableGgags',
u'\u8dce': 'CjkUnifiedIdeograph8dce',
u'\u6755': 'CjkUnifiedIdeograph6755',
u'\u46d4': 'CjkUnifiedIdeograph46d4',
u'\udd59': 'UnicodeCharacter0xdd59',
u'\ubcd8': 'HangulSyllableBye',
u'\u105b': 'MyanmarLetterMonJha',
u'\u965f': 'CjkUnifiedIdeograph965f',
u'\u75de': 'CjkUnifiedIdeograph75de',
u'\uebe2': 'UnicodeCharacter0xebe2',
u'\u4f65': 'CjkUnifiedIdeograph4f65',
u'\u2ee4': 'CjkRadicalGhost',
u'\uc569': 'HangulSyllableAelg',
u'\ua4e8': 'UnicodeCharacter0xa4e8',
u'\u7e6f': 'CjkUnifiedIdeograph7e6f',
u'\u5dee': 'CjkUnifiedIdeograph5dee',
u'\uf473': 'UnicodeCharacter0xf473',
u'\ud3f2': 'HangulSyllablePonh',
u'\u3775': 'CjkUnifiedIdeograph3775',
u'\u16f4': 'UnicodeCharacter0x16f4',
u'\uad79': 'HangulSyllableGult',
u'\u8cf8': 'CjkUnifiedIdeograph8cf8',
u'\u667f': 'CjkUnifiedIdeograph667f',
u'\u45fe': 'CjkUnifiedIdeograph45fe',
u'\u3006': 'IdeographicClosingMark',
u'\u1489': 'CanadianSyllabicsCe',
u'\u7508': 'CjkUnifiedIdeograph7508',
u'\u498b': 'CjkUnifiedIdeograph498b',
u'\ub60a': 'HangulSyllableDdyej',
u'\u8a8d': 'CjkUnifiedIdeograph8a8d',
u'\ueb0c': 'UnicodeCharacter0xeb0c',
u'\ucf8f': 'HangulSyllableKwaegs',
u'\u1816': 'MongolianDigitSix',
u'\u5d18': 'CjkUnifiedIdeograph5d18',
u'\u319b': 'IdeographicAnnotationThirdMark',
u'\u9e1a': 'CjkUnifiedIdeograph9e1a',
u'\u729d': 'CjkUnifiedIdeograph729d',
u'\ud31c': 'HangulSyllablePam',
u'\ub79f': 'HangulSyllableRaed',
u'\ue8a1': 'UnicodeCharacter0xe8a1',
u'\u4528': 'CjkUnifiedIdeograph4528',
u'\u19ab': 'UnicodeCharacter0x19ab',
u'\u862a': 'CjkUnifiedIdeograph862a',
u'\u5aad': 'CjkUnifiedIdeograph5aad',
u'\ubb2c': 'HangulSyllableMyoss',
u'\u9faf': 'CjkUnifiedIdeograph9faf',
u'\ufc2e': 'ArabicLigatureFehWithHahIsolatedForm',
u'\ud0b1': 'HangulSyllableKilt',
u'\u2d38': 'TifinaghLetterYadh',
u'\u01bb': 'LatinLetterTwoWithStroke',
u'\u6e3a': 'CjkUnifiedIdeograph6e3a',
u'\u42bd': 'CjkUnifiedIdeograph42bd',
u'\ua33c': 'YiSyllableSsut',
u'\u87bf': 'CjkUnifiedIdeograph87bf',
u'\ue43e': 'UnicodeCharacter0xe43e',
u'\ub8c1': 'HangulSyllableRoeb',
u'\ufdc3': 'ArabicLigatureKafWithMeemWithMeemInitialForm',
u'\u1548': 'CanadianSyllabicsRo',
u'\u564a': 'CjkUnifiedIdeograph564a',
u'\u2acd': 'SquareLeftOpenBoxOperator',
u'\u8b4c': 'CjkUnifiedIdeograph8b4c',
u'\u6fcf': 'CjkUnifiedIdeograph6fcf',
u'\ucc4e': 'HangulSyllableCaelm',
u'\ua0d1': 'YiSyllableFat',
u'\ue5d3': 'UnicodeCharacter0xe5d3',
u'\u3e5a': 'CjkUnifiedIdeograph3e5a',
u'\u12dd': 'EthiopicSyllableZe',
u'\u735c': 'CjkUnifiedIdeograph735c',
u'\u57df': 'CjkUnifiedIdeograph57df',
u'\ub45e': 'HangulSyllableDulp',
u'\u88e1': 'CjkUnifiedIdeograph88e1',
u'\ue960': 'UnicodeCharacter0xe960',
u'\ucde3': 'HangulSyllableCwec',
u'\u266a': 'EighthNote',
u'\u5b6c': 'CjkUnifiedIdeograph5b6c',
u'\u3fef': 'CjkUnifiedIdeograph3fef',
u'\u9c6e': 'CjkUnifiedIdeograph9c6e',
u'\u70f1': 'CjkUnifiedIdeograph70f1',
u'\ud170': 'HangulSyllableTyeol',
u'\ub5f3': 'HangulSyllableDdyeoh',
u'\uf6f5': 'UnicodeCharacter0xf6f5',
u'\u0e7a': 'UnicodeCharacter0xe7a',
u'\u437c': 'CjkUnifiedIdeograph437c',
u'\u27ff': 'LongRightwardsSquiggleArrow',
u'\u847e': 'CjkUnifiedIdeograph847e',
u'\ub980': 'HangulSyllableReuls',
u'\u1d03': 'LatinLetterSmallCapitalBarredB',
u'\ufa82': 'CjkCompatibilityIdeographfa82',
u'\u5e05': 'CjkUnifiedIdeograph5e05',
u'\u9307': 'CjkUnifiedIdeograph9307',
u'\ud409': 'HangulSyllablePwag',
u'\u2b8c': 'UnicodeCharacter0x2b8c',
u'\u6c8e': 'CjkUnifiedIdeograph6c8e',
u'\ua190': 'YiSyllableNep',
u'\u0513': 'CyrillicSmallLetterElWithHook',
u'\ue292': 'UnicodeCharacter0xe292',
u'\u4615': 'CjkUnifiedIdeograph4615',
u'\u7b17': 'CjkUnifiedIdeograph7b17',
u'\ubc19': 'HangulSyllableBanj',
u'\uf11b': 'UnicodeCharacter0xf11b',
u'\u139c': 'UnicodeCharacter0x139c',
u'\u549e': 'CjkUnifiedIdeograph549e',
u'\u89a0': 'CjkUnifiedIdeograph89a0',
u'\ucaa2': 'HangulSyllableJjyegg',
u'\u2e25': 'BottomRightHalfBracket',
u'\u6327': 'CjkUnifiedIdeograph6327',
u'\ua429': 'YiSyllableJju',
u'\ud92b': 'UnicodeCharacter0xd92b',
u'\u3cae': 'CjkUnifiedIdeograph3cae',
u'\u71b0': 'CjkUnifiedIdeograph71b0',
u'\ub2b2': 'HangulSyllableNyinh',
u'\u1635': 'CanadianSyllabicsCarrierTlho',
u'\uf7b4': 'UnicodeCharacter0xf7b4',
u'\u4b37': 'CjkUnifiedIdeograph4b37',
u'\u8c39': 'CjkUnifiedIdeograph8c39',
u'\uc13b': 'HangulSyllableSegs',
u'\u24be': 'CircledLatinCapitalLetterI',
u'\u59c0': 'CjkUnifiedIdeograph59c0',
u'\u9ac2': 'CjkUnifiedIdeograph9ac2',
u'\udfc4': 'UnicodeCharacter0xdfc4',
u'\u3347': 'SquareMansyon',
u'\u7449': 'CjkUnifiedIdeograph7449',
u'\ua94b': 'RejangVowelSignO',
u'\uea4d': 'UnicodeCharacter0xea4d',
u'\u0cce': 'UnicodeCharacter0xcce',
u'\u41d0': 'CjkUnifiedIdeograph41d0',
u'\u82d2': 'CjkUnifiedIdeograph82d2',
u'\uc7d4': 'HangulSyllableJyals',
u'\u1b57': 'BalineseDigitSeven',
u'\uf8d6': 'UnicodeCharacter0xf8d6',
u'\u5c59': 'CjkUnifiedIdeograph5c59',
u'\u915b': 'CjkUnifiedIdeograph915b',
u'\ud25d': 'HangulSyllableTweong',
u'\u29e0': 'SquareWithContouredOutline',
u'\u6ae2': 'CjkUnifiedIdeograph6ae2',
u'\uafe4': 'HangulSyllableGgweom',
u'\u0367': 'CombiningLatinSmallLetterU',
u'\ue0e6': 'UnicodeCharacter0xe0e6',
u'\u4469': 'CjkUnifiedIdeograph4469',
u'\u796b': 'CjkUnifiedIdeograph796b',
u'\uba6d': 'HangulSyllableMet',
u'\uff6f': 'HalfwidthKatakanaLetterSmallTu',
u'\u11f0': 'HangulJongseongYesieung',
u'\u52f2': 'CjkUnifiedIdeograph52f2',
u'\u97f4': 'CjkUnifiedIdeograph97f4',
u'\uc8f6': 'HangulSyllableJyoj',
u'\u2c79': 'LatinSmallLetterTurnedRWithTail',
u'\u617b': 'CjkUnifiedIdeograph617b',
u'\ua27d': 'YiSyllableHex',
u'\ue77f': 'UnicodeCharacter0xe77f',
u'\u9b81': 'CjkUnifiedIdeograph9b81',
u'\u7c04': 'CjkUnifiedIdeograph7c04',
u'\u5087': 'CjkUnifiedIdeograph5087',
u'\uf608': 'UnicodeCharacter0xf608',
u'\uca8b': 'HangulSyllableJjyeod',
u'\u350a': 'CjkUnifiedIdeograph350a',
u'\u098d': 'UnicodeCharacter0x98d',
u'\uaf0e': 'HangulSyllableGgyeop',
u'\u8391': 'CjkUnifiedIdeograph8391',
u'\u6414': 'CjkUnifiedIdeograph6414',
u'\u3897': 'CjkUnifiedIdeograph3897',
u'\ude18': 'UnicodeCharacter0xde18',
u'\ub29b': 'HangulSyllableNeulb',
u'\u1d1a': 'LatinLetterSmallCapitalTurnedR',
u'\u971e': 'CjkUnifiedIdeograph971e',
u'\u6ba1': 'CjkUnifiedIdeograph6ba1',
u'\ueda5': 'UnicodeCharacter0xeda5',
u'\u4c24': 'CjkUnifiedIdeograph4c24',
u'\u20a7': 'PesetaSign',
u'\uc628': 'HangulSyllableOn',
u'\u9aab': 'CjkUnifiedIdeograph9aab',
u'\u052a': 'UnicodeCharacter0x52a',
u'\u7f2e': 'CjkUnifiedIdeograph7f2e',
u'\u53b1': 'CjkUnifiedIdeograph53b1',
u'\uf132': 'UnicodeCharacter0xf132',
u'\ud5b5': 'HangulSyllableHyaelg',
u'\u3434': 'CjkUnifiedIdeograph3434',
u'\u08b7': 'UnicodeCharacter0x8b7',
u'\uae38': 'HangulSyllableGil',
u'\u82bb': 'CjkUnifiedIdeograph82bb',
u'\u673e': 'CjkUnifiedIdeograph673e',
u'\u3bc1': 'CjkUnifiedIdeograph3bc1',
u'\ud942': 'UnicodeCharacter0xd942',
u'\ubdc5': 'HangulSyllableBwelt',
u'\u1c44': 'LepchaDigitFour',
u'\u9648': 'CjkUnifiedIdeograph9648',
u'\u6acb': 'CjkUnifiedIdeograph6acb',
u'\ueccf': 'UnicodeCharacter0xeccf',
u'\u4f4e': 'CjkUnifiedIdeograph4f4e',
u'\u23d1': 'MetricalBreve',
u'\uc152': 'HangulSyllableSep',
u'\ua5d5': 'VaiSyllableZo',
u'\u0454': 'CyrillicSmallLetterUkrainianIe',
u'\u7e58': 'CjkUnifiedIdeograph7e58',
u'\u52db': 'CjkUnifiedIdeograph52db',
u'\uf05c': 'UnicodeCharacter0xf05c',
u'\ud4df': 'HangulSyllablePwis',
u'\u375e': 'CjkUnifiedIdeograph375e',
u'\u0be1': 'UnicodeCharacter0xbe1',
u'\ua962': 'UnicodeCharacter0xa962',
u'\u8de5': 'CjkUnifiedIdeograph8de5',
u'\u6668': 'CjkUnifiedIdeograph6668',
u'\u3aeb': 'CjkUnifiedIdeograph3aeb',
u'\ud86c': 'UnicodeCharacter0xd86c',
u'\ubcef': 'HangulSyllableByec',
u'\u1f6e': 'GreekCapitalLetterOmegaWithPsiliAndPerispomeni',
u'\u9172': 'CjkUnifiedIdeograph9172',
u'\u75f5': 'CjkUnifiedIdeograph75f5',
u'\ueff9': 'UnicodeCharacter0xeff9',
u'\u4e78': 'CjkUnifiedIdeograph4e78',
u'\u22fb': 'ContainsWithVerticalBarAtEndOfHorizontalStroke',
u'\uc07c': 'HangulSyllableBbyil',
u'\ua4ff': 'UnicodeCharacter0xa4ff',
u'\u077e': 'ArabicLetterSeenWithInvertedV',
u'\u9e03': 'CjkUnifiedIdeograph9e03',
u'\u7982': 'CjkUnifiedIdeograph7982',
u'\uf386': 'UnicodeCharacter0xf386',
u'\u5709': 'CjkUnifiedIdeograph5709',
u'\u3688': 'CjkUnifiedIdeograph3688',
u'\uc90d': 'HangulSyllableJub',
u'\ua88c': 'SaurashtraLetterE',
u'\u0c0f': 'TeluguLetterEe',
u'\u8613': 'CjkUnifiedIdeograph8613',
u'\u6192': 'CjkUnifiedIdeograph6192',
u'\uf817': 'UnicodeCharacter0xf817',
u'\udb96': 'UnicodeCharacter0xdb96',
u'\u3f19': 'CjkUnifiedIdeograph3f19',
u'\u1e98': 'LatinSmallLetterWWithRingAbove',
u'\ub11d': 'HangulSyllableNeong',
u'\u909c': 'CjkUnifiedIdeograph909c',
u'\u6e23': 'CjkUnifiedIdeograph6e23',
u'\u49a2': 'CjkUnifiedIdeograph49a2',
u'\ue027': 'UnicodeCharacter0xe027',
u'\uc3a6': 'HangulSyllableSsyeonh',
u'\u2729': 'StressOutlinedWhiteStar',
u'\u06a8': 'ArabicLetterQafWithThreeDotsAbove',
u'\u992d': 'CjkUnifiedIdeograph992d',
u'\u78ac': 'CjkUnifiedIdeograph78ac',
u'\uf2b0': 'UnicodeCharacter0xf2b0',
u'\u5633': 'CjkUnifiedIdeograph5633',
u'\u31b2': 'BopomofoLetterOng',
u'\uc837': 'HangulSyllableJeh',
u'\uabb6': 'UnicodeCharacter0xabb6',
u'\u0f39': 'TibetanMarkTsaPhru',
u'\u813d': 'CjkUnifiedIdeograph813d',
u'\u60bc': 'CjkUnifiedIdeograph60bc',
u'\ufb41': 'HebrewLetterSamekhWithDagesh',
u'\udac0': 'UnicodeCharacter0xdac0',
u'\u3e43': 'CjkUnifiedIdeograph3e43',
u'\u19c2': 'NewTaiLueLetterFinalNg',
u'\ub047': 'HangulSyllableGgeugs',
u'\u93c6': 'CjkUnifiedIdeograph93c6',
u'\u694d': 'CjkUnifiedIdeograph694d',
u'\u48cc': 'CjkUnifiedIdeograph48cc',
u'\ue351': 'UnicodeCharacter0xe351',
u'\uc2d0': 'HangulSyllableSyim',
u'\u2653': 'Pisces',
u'\u01d2': 'LatinSmallLetterOWithCaron',
u'\u9857': 'CjkUnifiedIdeograph9857',
u'\u7bd6': 'CjkUnifiedIdeograph7bd6',
u'\ufdda': 'UnicodeCharacter0xfdda',
u'\u515d': 'CjkUnifiedIdeograph515d',
u'\u30dc': 'KatakanaLetterBo',
u'\ucb61': 'HangulSyllableJjut',
u'\uaae0': 'UnicodeCharacter0xaae0',
u'\u0e63': 'UnicodeCharacter0xe63',
u'\u8067': 'CjkUnifiedIdeograph8067',
u'\u63e6': 'CjkUnifiedIdeograph63e6',
u'\ufa6b': 'UnicodeCharacter0xfa6b',
u'\ue5ea': 'UnicodeCharacter0xe5ea',
u'\u396d': 'CjkUnifiedIdeograph396d',
u'\u18ec': 'UnicodeCharacter0x18ec',
u'\ub371': 'HangulSyllableDeg',
u'\u92f0': 'CjkUnifiedIdeograph92f0',
u'\u6877': 'CjkUnifiedIdeograph6877',
u'\u4bf6': 'CjkUnifiedIdeograph4bf6',
u'\ue27b': 'UnicodeCharacter0xe27b',
u'\ucdfa': 'HangulSyllableCwibs',
u'\u217d': 'SmallRomanNumeralOneHundred',
u'\xfc': 'LatinSmallLetterUWithDiaeresis',
u'\u1a81': 'UnicodeCharacter0x1a81',
u'\u7b00': 'CjkUnifiedIdeograph7b00',
u'\u5f83': 'CjkUnifiedIdeograph5f83',
u'\ub802': 'HangulSyllableReoj',
u'\u9c85': 'CjkUnifiedIdeograph9c85',
u'\ufd04': 'ArabicLigatureKhahWithYehIsolatedForm',
u'\ud187': 'HangulSyllableTyegs',
u'\u2e0e': 'EditorialCoronis',
u'\u0291': 'LatinSmallLetterZWithCurl',
u'\u6310': 'CjkUnifiedIdeograph6310',
u'\u4793': 'CjkUnifiedIdeograph4793',
u'\ua012': 'YiSyllableOp',
u'\u8495': 'CjkUnifiedIdeograph8495',
u'\ue514': 'UnicodeCharacter0xe514',
u'\ub997': 'HangulSyllableRyid',
u'\ufe99': 'ArabicLetterThehIsolatedForm',
u'\u161e': 'CanadianSyllabicsCarrierJje',
u'\u4b20': 'CjkUnifiedIdeograph4b20',
u'\u2fa3': 'KangxiRadicalWine',
u'\u8822': 'CjkUnifiedIdeograph8822',
u'\u6ca5': 'CjkUnifiedIdeograph6ca5',
u'\ucd24': 'HangulSyllableCwa',
u'\ua1a7': 'YiSyllableHlop',
u'\ue6a9': 'UnicodeCharacter0xe6a9',
u'\u3330': 'SquarePiko',
u'\u17b3': 'KhmerIndependentVowelQau',
u'\u7032': 'CjkUnifiedIdeograph7032',
u'\u54b5': 'CjkUnifiedIdeograph54b5',
u'\ub534': 'HangulSyllableDdan',
u'\u89b7': 'CjkUnifiedIdeograph89b7',
u'\uea36': 'UnicodeCharacter0xea36',
u'\uceb9': 'HangulSyllableKyalt',
u'\u1b40': 'BalineseVowelSignTalingTedung',
u'\u5842': 'CjkUnifiedIdeograph5842',
u'\u3cc5': 'CjkUnifiedIdeograph3cc5',
u'\u9d44': 'CjkUnifiedIdeograph9d44',
u'\u71c7': 'CjkUnifiedIdeograph71c7',
u'\ud246': 'HangulSyllableTup',
u'\ub6c9': 'HangulSyllableDdweob',
u'\uebcb': 'UnicodeCharacter0xebcb',
u'\u0350': 'CombiningRightArrowheadAbove',
u'\u4052': 'CjkUnifiedIdeograph4052',
u'\u24d5': 'CircledLatinSmallLetterF',
u'\u8554': 'CjkUnifiedIdeograph8554',
u'\u59d7': 'CjkUnifiedIdeograph59d7',
u'\uba56': 'HangulSyllableMegg',
u'\u9ed9': 'CjkUnifiedIdeograph9ed9',
u'\uff58': 'FullwidthLatinSmallLetterX',
u'\ud3db': 'HangulSyllablePyelb',
u'\u2862': 'BraillePatternDots267',
u'\u0ce5': 'UnicodeCharacter0xce5',
u'\u6d64': 'CjkUnifiedIdeograph6d64',
u'\u41e7': 'CjkUnifiedIdeograph41e7',
u'\ua266': 'YiSyllableNguo',
u'\u86e9': 'CjkUnifiedIdeograph86e9',
u'\ue768': 'UnicodeCharacter0xe768',
u'\ubbeb': 'HangulSyllableMyilh',
u'\uf8ed': 'UnicodeCharacter0xf8ed',
u'\u1072': 'MyanmarVowelSignKayahOe',
u'\u5574': 'CjkUnifiedIdeograph5574',
u'\u29f7': 'ReverseSolidusWithHorizontalStroke',
u'\u8a76': 'CjkUnifiedIdeograph8a76',
u'\u6ef9': 'CjkUnifiedIdeograph6ef9',
u'\ucf78': 'HangulSyllableKwal',
u'\ua3fb': 'YiSyllableJyr',
u'\ue0fd': 'UnicodeCharacter0xe0fd',
u'\uda01': 'UnicodeCharacter0xda01',
u'\u3d84': 'CjkUnifiedIdeograph3d84',
u'\u7286': 'CjkUnifiedIdeograph7286',
u'\ub788': 'HangulSyllableRals',
u'\u0b0b': 'OriyaLetterVocalicR',
u'\uf48a': 'UnicodeCharacter0xf48a',
u'\u480d': 'CjkUnifiedIdeograph480d',
u'\u8d0f': 'CjkUnifiedIdeograph8d0f',
u'\uc211': 'HangulSyllableSyong',
u'\u2594': 'UpperOneEighthBlock',
u'\u5a96': 'CjkUnifiedIdeograph5a96',
u'\u9f98': 'CjkUnifiedIdeograph9f98',
u'\udc9a': 'UnicodeCharacter0xdc9a',
u'\u301d': 'ReversedDoublePrimeQuotationMark',
u'\u751f': 'CjkUnifiedIdeograph751f',
u'\uaa21': 'ChamLetterBba',
u'\uef23': 'UnicodeCharacter0xef23',
u'\u0da4': 'SinhalaLetterTaalujaNaasikyaya',
u'\u42a6': 'CjkUnifiedIdeograph42a6',
u'\u87a8': 'CjkUnifiedIdeograph87a8',
u'\uc4aa': 'HangulSyllableSswelp',
u'\u182d': 'MongolianLetterGa',
u'\uf9ac': 'CjkCompatibilityIdeographf9ac',
u'\u5d2f': 'CjkUnifiedIdeograph5d2f',
u'\u9231': 'CjkUnifiedIdeograph9231',
u'\ud733': 'HangulSyllableHwih',
u'\u2ab6': 'SucceedsAboveNotEqualTo',
u'\u6fb8': 'CjkUnifiedIdeograph6fb8',
u'\uacba': 'HangulSyllableGyeobs',
u'\ue1bc': 'UnicodeCharacter0xe1bc',
u'\u453f': 'CjkUnifiedIdeograph453f',
u'\u7a41': 'CjkUnifiedIdeograph7a41',
u'\ubf43': 'HangulSyllableBbogs',
u'\ufc45': 'ArabicLigatureMeemWithJeemIsolatedForm',
u'\u12c6': 'UnicodeCharacter0x12c6',
u'\u57c8': 'CjkUnifiedIdeograph57c8',
u'\u94ca': 'CjkUnifiedIdeograph94ca',
u'\uc9cc': 'HangulSyllableJils',
u'\u2d4f': 'TifinaghLetterYan',
u'\u6251': 'CjkUnifiedIdeograph6251',
u'\ua753': 'LatinSmallLetterPWithFlourish',
u'\ue455': 'UnicodeCharacter0xe455',
u'\u3fd8': 'CjkUnifiedIdeograph3fd8',
u'\u7cda': 'CjkUnifiedIdeograph7cda',
u'\ub1dc': 'HangulSyllableNoem',
u'\u155f': 'CanadianSyllabicsNcreeThe',
u'\uf6de': 'UnicodeCharacter0xf6de',
u'\u4a61': 'CjkUnifiedIdeograph4a61',
u'\u8f63': 'CjkUnifiedIdeograph8f63',
u'\ucc65': 'HangulSyllableCyanj',
u'\u27e8': 'MathematicalLeftAngleBracket',
u'\u64ea': 'CjkUnifiedIdeograph64ea',
u'\u99ec': 'CjkUnifiedIdeograph99ec',
u'\udeee': 'UnicodeCharacter0xdeee',
u'\u3271': 'CircledHangulRieulA',
u'\u7773': 'CjkUnifiedIdeograph7773',
u'\ub475': 'HangulSyllableDweolg',
u'\ue977': 'UnicodeCharacter0xe977',
u'\u0ff8': 'UnicodeCharacter0xff8',
u'\u4cfa': 'CjkUnifiedIdeograph4cfa',
u'\u81fc': 'CjkUnifiedIdeograph81fc',
u'\uc6fe': 'HangulSyllableWej',
u'\ufc00': 'ArabicLigatureYehWithHamzaAboveWithJeemIsolatedForm',
u'\ud883': 'UnicodeCharacter0xd883',
u'\u3f02': 'CjkUnifiedIdeograph3f02',
u'\u1b85': 'SundaneseLetterU',
u'\ub506': 'HangulSyllableDyilp',
u'\u9189': 'CjkUnifiedIdeograph9189',
u'\u6e0c': 'CjkUnifiedIdeograph6e0c',
u'\u4a8f': 'CjkUnifiedIdeograph4a8f',
u'\ue410': 'UnicodeCharacter0xe410',
u'\uc093': 'HangulSyllableBbigs',
u'\u2712': 'BlackNib',
u'\u0395': 'GreekCapitalLetterEpsilon',
u'\u9d16': 'CjkUnifiedIdeograph9d16',
u'\u7999': 'CjkUnifiedIdeograph7999',
u'\uf79d': 'UnicodeCharacter0xf79d',
u'\u561c': 'CjkUnifiedIdeograph561c',
u'\u329f': 'CircledIdeographAttention',
u'\ucc20': 'HangulSyllableJjiss',
u'\ua8a3': 'SaurashtraLetterDa',
u'\u0f22': 'TibetanDigitTwo',
u'\u8526': 'CjkUnifiedIdeograph8526',
u'\u61a9': 'CjkUnifiedIdeograph61a9',
u'\udfad': 'UnicodeCharacter0xdfad',
u'\u3e2c': 'CjkUnifiedIdeograph3e2c',
u'\u1aaf': 'UnicodeCharacter0x1aaf',
u'\ub430': 'HangulSyllableDoek',
u'\u90b3': 'CjkUnifiedIdeograph90b3',
u'\u6d36': 'CjkUnifiedIdeograph6d36',
u'\u49b9': 'CjkUnifiedIdeograph49b9',
u'\ueb3a': 'UnicodeCharacter0xeb3a',
u'\uc7bd': 'HangulSyllableJaeb',
u'\u263c': 'WhiteSunWithRays',
u'\u02bf': 'ModifierLetterLeftHalfRing',
u'\u9c40': 'CjkUnifiedIdeograph9c40',
u'\u78c3': 'CjkUnifiedIdeograph78c3',
u'\uf6c7': 'UnicodeCharacter0xf6c7',
u'\u5546': 'CjkUnifiedIdeograph5546',
u'\u31c9': 'CjkStrokeSzwg',
u'\ud34a': 'HangulSyllablePyanh',
u'\uafcd': 'HangulSyllableGgung',
u'\u0e4c': 'ThaiCharacterThanthakhat',
u'\u8450': 'CjkUnifiedIdeograph8450',
u'\u60d3': 'CjkUnifiedIdeograph60d3',
u'\uded7': 'UnicodeCharacter0xded7',
u'\u3d56': 'CjkUnifiedIdeograph3d56',
u'\u19d9': 'NewTaiLueDigitNine',
u'\ubb5a': 'HangulSyllableMweolm',
u'\u97dd': 'CjkUnifiedIdeograph97dd',
u'\u6c60': 'CjkUnifiedIdeograph6c60',
u'\u48e3': 'CjkUnifiedIdeograph48e3',
u'\uea64': 'UnicodeCharacter0xea64',
u'\uc6e7': 'HangulSyllableWeoh',
u'\u2566': 'BoxDrawingsDoubleDownAndHorizontal',
u'\u01e9': 'LatinSmallLetterKWithCaron',
u'\ua36a': 'YiSyllableChox',
u'\u7fed': 'CjkUnifiedIdeograph7fed',
u'\uf5f1': 'UnicodeCharacter0xf5f1',
u'\u5470': 'CjkUnifiedIdeograph5470',
u'\u30f3': 'KatakanaLetterN',
u'\ud274': 'HangulSyllableTwem',
u'\uaef7': 'HangulSyllableGgyeogs',
u'\u0d76': 'UnicodeCharacter0xd76',
u'\u8b7a': 'CjkUnifiedIdeograph8b7a',
u'\u67fd': 'CjkUnifiedIdeograph67fd',
u'\u5d01': 'CjkUnifiedIdeograph5d01',
u'\u3c80': 'CjkUnifiedIdeograph3c80',
u'\udb05': 'UnicodeCharacter0xdb05',
u'\uba84': 'HangulSyllableMyeoss',
u'\u1607': 'CanadianSyllabicsCarrierNa',
u'\u8c0b': 'CjkUnifiedIdeograph8c0b',
u'\u738a': 'CjkUnifiedIdeograph738a',
u'\ue98e': 'UnicodeCharacter0xe98e',
u'\u4511': 'CjkUnifiedIdeograph4511',
u'\u2490': 'DigitNineFullStop',
u'\uc315': 'HangulSyllableSsaeg',
u'\ua294': 'YiSyllableZat',
u'\u741b': 'CjkUnifiedIdeograph741b',
u'\u5b9a': 'CjkUnifiedIdeograph5b9a',
u'\uf21f': 'UnicodeCharacter0xf21f',
u'\ud19e': 'HangulSyllableTyep',
u'\u2d21': 'GeorgianSmallLetterHe',
u'\u0ca0': 'KannadaLetterTtha',
u'\uab25': 'UnicodeCharacter0xab25',
u'\u8aa4': 'CjkUnifiedIdeograph8aa4',
u'\u5c2b': 'CjkUnifiedIdeograph5c2b',
u'\u43aa': 'CjkUnifiedIdeograph43aa',
u'\uda2f': 'UnicodeCharacter0xda2f',
u'\ub9ae': 'HangulSyllableRigg',
u'\u1531': 'CanadianSyllabicsYwi',
u'\u9335': 'CjkUnifiedIdeograph9335',
u'\u72b4': 'CjkUnifiedIdeograph72b4',
u'\ue8b8': 'UnicodeCharacter0xe8b8',
u'\u443b': 'CjkUnifiedIdeograph443b',
u'\u2bba': 'UnicodeCharacter0x2bba',
u'\uc23f': 'HangulSyllableSweolb',
u'\ua1be': 'YiSyllableLiep',
u'\u7b45': 'CjkUnifiedIdeograph7b45',
u'\u5ac4': 'CjkUnifiedIdeograph5ac4',
u'\uf149': 'UnicodeCharacter0xf149',
u'\ud0c8': 'HangulSyllableTal',
u'\u2c4b': 'GlagoliticSmallLetterShta',
u'\u13ca': 'CherokeeLetterQuu',
u'\uaa4f': 'UnicodeCharacter0xaa4f',
u'\u89ce': 'CjkUnifiedIdeograph89ce',
u'\u6355': 'CjkUnifiedIdeograph6355',
u'\u42d4': 'CjkUnifiedIdeograph42d4',
u'\ud959': 'UnicodeCharacter0xd959',
u'\ub8d8': 'HangulSyllableRyols',
u'\u145b': 'CanadianSyllabicsTwii',
u'\u925f': 'CjkUnifiedIdeograph925f',
u'\u71de': 'CjkUnifiedIdeograph71de',
u'\uefe2': 'UnicodeCharacter0xefe2',
u'\u4b65': 'CjkUnifiedIdeograph4b65',
u'\u2ae4': 'VerticalBarDoubleLeftTurnstile',
u'\uc169': 'HangulSyllableSyeong',
u'\ua0e8': 'YiSyllableVie',
u'\u7a6f': 'CjkUnifiedIdeograph7a6f',
u'\u59ee': 'CjkUnifiedIdeograph59ee',
u'\uf073': 'UnicodeCharacter0xf073',
u'\ud7f2': 'UnicodeCharacter0xd7f2',
u'\u3375': 'SquareOv',
u'\u12f4': 'EthiopicSyllableDee',
u'\ua979': 'UnicodeCharacter0xa979',
u'\u88f8': 'CjkUnifiedIdeograph88f8',
u'\u627f': 'CjkUnifiedIdeograph627f',
u'\u41fe': 'CjkUnifiedIdeograph41fe',
u'\u3406': 'CjkUnifiedIdeograph3406',
u'\u1089': 'MyanmarSignShanTone5',
u'\u7108': 'CjkUnifiedIdeograph7108',
u'\u4d8b': 'CjkUnifiedIdeograph4d8b',
u'\ub20a': 'HangulSyllableNunh',
u'\u8e8d': 'CjkUnifiedIdeograph8e8d',
u'\uef0c': 'UnicodeCharacter0xef0c',
u'\ucb8f': 'HangulSyllableJjwelh',
u'\u1c16': 'LepchaLetterMla',
u'\u5918': 'CjkUnifiedIdeograph5918',
u'\u359b': 'CjkUnifiedIdeograph359b',
u'\u9a1a': 'CjkUnifiedIdeograph9a1a',
u'\u769d': 'CjkUnifiedIdeograph769d',
u'\ud71c': 'HangulSyllableHwin',
u'\ub39f': 'HangulSyllableDyeos',
u'\ueca1': 'UnicodeCharacter0xeca1',
u'\u0426': 'CyrillicCapitalLetterTse',
u'\u4128': 'CjkUnifiedIdeograph4128',
u'\u1dab': 'ModifierLetterSmallCapitalL',
u'\u822a': 'CjkUnifiedIdeograph822a',
u'\u5ead': 'CjkUnifiedIdeograph5ead',
u'\ubf2c': 'HangulSyllableBbyel',
u'\u9baf': 'CjkUnifiedIdeograph9baf',
u'\uf82e': 'UnicodeCharacter0xf82e',
u'\ud4b1': 'HangulSyllablePweg',
u'\u2938': 'RightsideArcClockwiseArrow',
u'\u05bb': 'HebrewPointQubuts',
u'\u6a3a': 'CjkUnifiedIdeograph6a3a',
u'\u46bd': 'CjkUnifiedIdeograph46bd',
u'\ua73c': 'LatinCapitalLetterAy',
u'\u83bf': 'CjkUnifiedIdeograph83bf',
u'\ue03e': 'UnicodeCharacter0xe03e',
u'\ubcc1': 'HangulSyllableByeonj',
u'\uf9c3': 'CjkCompatibilityIdeographf9c3',
u'\u1148': 'HangulChoseongIeungcieuc',
u'\u524a': 'CjkUnifiedIdeograph524a',
u'\u2ecd': 'CjkRadicalWalkOne',
u'\u8f4c': 'CjkUnifiedIdeograph8f4c',
u'\u6bcf': 'CjkUnifiedIdeograph6bcf',
u'\uc84e': 'HangulSyllableJyeoj',
u'\ua4d1': 'UnicodeCharacter0xa4d1',
u'\ue1d3': 'UnicodeCharacter0xe1d3',
u'\u3a5a': 'CjkUnifiedIdeograph3a5a',
u'\u16dd': 'RunicLetterIng',
u'\u775c': 'CjkUnifiedIdeograph775c',
u'\u53df': 'CjkUnifiedIdeograph53df',
u'\ub05e': 'HangulSyllableGgeup',
u'\u8ce1': 'CjkUnifiedIdeograph8ce1',
u'\ued60': 'UnicodeCharacter0xed60',
u'\uc9e3': 'HangulSyllableJjad',
u'\u226a': 'MuchLessthan',
u'\u5f6c': 'CjkUnifiedIdeograph5f6c',
u'\u3bef': 'CjkUnifiedIdeograph3bef',
u'\u986e': 'CjkUnifiedIdeograph986e',
u'\u74f1': 'CjkUnifiedIdeograph74f1',
u'\ud570': 'HangulSyllableHak',
u'\ub1f3': 'HangulSyllableNyolb',
u'\uf2f5': 'UnicodeCharacter0xf2f5',
u'\u0a7a': 'UnicodeCharacter0xa7a',
u'\u477c': 'CjkUnifiedIdeograph477c',
u'\u23ff': 'UnicodeCharacter0x23ff',
u'\u807e': 'CjkUnifiedIdeograph807e',
u'\ubd80': 'HangulSyllableBu',
u'\u1903': 'LimbuLetterGa',
u'\ufe82': 'ArabicLetterAlefWithMaddaAboveFinalForm',
u'\u5a05': 'CjkUnifiedIdeograph5a05',
u'\u9707': 'CjkUnifiedIdeograph9707',
u'\ud009': 'HangulSyllableKweolt',
u'\u2f8c': 'KangxiRadicalTiger',
u'\u688e': 'CjkUnifiedIdeograph688e',
u'\ua590': 'VaiSyllableNggoo',
u'\u0113': 'LatinSmallLetterEWithMacron',
u'\ue692': 'UnicodeCharacter0xe692',
u'\u4215': 'CjkUnifiedIdeograph4215',
u'\u7f17': 'CjkUnifiedIdeograph7f17',
u'\ub819': 'HangulSyllableReb',
u'\uf51b': 'UnicodeCharacter0xf51b',
u'\u179c': 'KhmerLetterVo',
u'\u509e': 'CjkUnifiedIdeograph509e',
u'\u8da0': 'CjkUnifiedIdeograph8da0',
u'\ucea2': 'HangulSyllableKaebs',
u'\u2a25': 'PlusSignWithDotBelow',
u'\u6727': 'CjkUnifiedIdeograph6727',
u'\ua029': 'YiSyllableBex',
u'\udd2b': 'UnicodeCharacter0xdd2b',
u'\u38ae': 'CjkUnifiedIdeograph38ae',
u'\u75b0': 'CjkUnifiedIdeograph75b0',
u'\ub6b2': 'HangulSyllableDduj',
u'\u1235': 'EthiopicSyllableSe',
u'\uf3b4': 'UnicodeCharacter0xf3b4',
u'\u4f37': 'CjkUnifiedIdeograph4f37',
u'\u8839': 'CjkUnifiedIdeograph8839',
u'\uc53b': 'HangulSyllableSsis',
u'\u20be': 'UnicodeCharacter0x20be',
u'\u5dc0': 'CjkUnifiedIdeograph5dc0',
u'\u9ec2': 'CjkUnifiedIdeograph9ec2',
u'\udbc4': 'UnicodeCharacter0xdbc4',
u'\u3747': 'CjkUnifiedIdeograph3747',
u'\u7049': 'CjkUnifiedIdeograph7049',
u'\uad4b': 'HangulSyllableGoec',
u'\uee4d': 'UnicodeCharacter0xee4d',
u'\u08ce': 'UnicodeCharacter0x8ce',
u'\u45d0': 'CjkUnifiedIdeograph45d0',
u'\u86d2': 'CjkUnifiedIdeograph86d2',
u'\uc3d4': 'HangulSyllableSsyek',
u'\u1f57': 'GreekSmallLetterUpsilonWithDasiaAndPerispomeni',
u'\ufcd6': 'ArabicLigatureNoonWithHehInitialForm',
u'\u5859': 'CjkUnifiedIdeograph5859',
u'\u955b': 'CjkUnifiedIdeograph955b',
u'\ud65d': 'HangulSyllableHwalg',
u'\u2de0': 'CombiningCyrillicLetterBe',
u'\u6ee2': 'CjkUnifiedIdeograph6ee2',
u'\uabe4': 'UnicodeCharacter0xabe4',
u'\u0767': 'ArabicLetterNoonWithTwoDotsBelow',
u'\ue4e6': 'UnicodeCharacter0xe4e6',
u'\u4069': 'CjkUnifiedIdeograph4069',
u'\u7d6b': 'CjkUnifiedIdeograph7d6b',
u'\ube6d': 'HangulSyllableBbalt',
u'\ufb6f': 'ArabicLetterPehehFinalForm',
u'\u15f0': 'CanadianSyllabicsCarrierGo',
u'\u56f2': 'CjkUnifiedIdeograph56f2',
u'\u93f4': 'CjkUnifiedIdeograph93f4',
u'\uccf6': 'HangulSyllableCyelm',
u'\u2879': 'BraillePatternDots14567',
u'\u657b': 'CjkUnifiedIdeograph657b',
u'\ua67d': 'CombiningCyrillicPayerok',
u'\ue37f': 'UnicodeCharacter0xe37f',
u'\u9f81': 'CjkUnifiedIdeograph9f81',
u'\u7804': 'CjkUnifiedIdeograph7804',
u'\u5487': 'CjkUnifiedIdeograph5487',
u'\uf208': 'UnicodeCharacter0xf208',
u'\uce8b': 'HangulSyllableKac',
u'\u310a': 'BopomofoLetterT',
u'\u0d8d': 'SinhalaLetterIruyanna',
u'\uab0e': 'UnicodeCharacter0xab0e',
u'\u8791': 'CjkUnifiedIdeograph8791',
u'\u6014': 'CjkUnifiedIdeograph6014',
u'\u3c97': 'CjkUnifiedIdeograph3c97',
u'\uda18': 'UnicodeCharacter0xda18',
u'\ub69b': 'HangulSyllableDdyoh',
u'\u191a': 'LimbuLetterSsa',
u'\u931e': 'CjkUnifiedIdeograph931e',
u'\u6fa1': 'CjkUnifiedIdeograph6fa1',
u'\ue9a5': 'UnicodeCharacter0xe9a5',
u'\u4824': 'CjkUnifiedIdeograph4824',
u'\u24a7': 'ParenthesizedLatinSmallLetterL',
u'\uc228': 'HangulSyllableSum',
u'\u9eab': 'CjkUnifiedIdeograph9eab',
u'\u012a': 'LatinCapitalLetterIWithMacron',
u'\u7b2e': 'CjkUnifiedIdeograph7b2e',
u'\u57b1': 'CjkUnifiedIdeograph57b1',
u'\uf532': 'UnicodeCharacter0xf532',
u'\ud1b5': 'HangulSyllableTong',
u'\u3034': 'VerticalKanaRepeatWithVoicedSoundMarkUpperHalf',
u'\u0cb7': 'KannadaLetterSsa',
u'\uaa38': 'UnicodeCharacter0xaa38',
u'\u86bb': 'CjkUnifiedIdeograph86bb',
u'\u633e': 'CjkUnifiedIdeograph633e',
u'\u3fc1': 'CjkUnifiedIdeograph3fc1',
u'\udd42': 'UnicodeCharacter0xdd42',
u'\ub9c5': 'HangulSyllableRit',
u'\u1844': 'MongolianLetterTodoE',
u'\u9248': 'CjkUnifiedIdeograph9248',
u'\u6ecb': 'CjkUnifiedIdeograph6ecb',
u'\ue8cf': 'UnicodeCharacter0xe8cf',
u'\u4b4e': 'CjkUnifiedIdeograph4b4e',
u'\u27d1': 'AndWithDot',
u'\uc552': 'HangulSyllableAlp',
u'\ua1d5': 'YiSyllableLyx',
u'\u7a58': 'CjkUnifiedIdeograph7a58',
u'\u56db': 'CjkUnifiedIdeograph56db',
u'\uf45c': 'UnicodeCharacter0xf45c',
u'\ud0df': 'HangulSyllableTaegs',
u'\u335e': 'IdeographicTelegraphSymbolForHourSix',
u'\u0fe1': 'UnicodeCharacter0xfe1',
u'\uad62': 'HangulSyllableGyobs',
u'\u89e5': 'CjkUnifiedIdeograph89e5',
u'\u6268': 'CjkUnifiedIdeograph6268',
u'\u3eeb': 'CjkUnifiedIdeograph3eeb',
u'\udc6c': 'UnicodeCharacter0xdc6c',
u'\ub8ef': 'HangulSyllableRud',
u'\u1b6e': 'BalineseMusicalSymbolCombiningKempli',
u'\u9572': 'CjkUnifiedIdeograph9572',
u'\u71f5': 'CjkUnifiedIdeograph71f5',
u'\uebf9': 'UnicodeCharacter0xebf9',
u'\u4a78': 'CjkUnifiedIdeograph4a78',
u'\u26fb': 'UnicodeCharacter0x26fb',
u'\uc47c': 'HangulSyllableSsuk',
u'\ua0ff': 'YiSyllableVyr',
u'\u037e': 'GreekQuestionMark',
u'\u9a03': 'CjkUnifiedIdeograph9a03',
u'\u7d82': 'CjkUnifiedIdeograph7d82',
u'\uf786': 'UnicodeCharacter0xf786',
u'\u5309': 'CjkUnifiedIdeograph5309',
u'\u3288': 'CircledIdeographNine',
u'\ucd0d': 'HangulSyllableConj',
u'\uac8c': 'HangulSyllableGe',
u'\u080f': 'UnicodeCharacter0x80f',
u'\u8213': 'CjkUnifiedIdeograph8213',
u'\u6592': 'CjkUnifiedIdeograph6592',
u'\ufc17': 'ArabicLigatureHahWithJeemIsolatedForm',
u'\udf96': 'UnicodeCharacter0xdf96',
u'\u3b19': 'CjkUnifiedIdeograph3b19',
u'\u1a98': 'UnicodeCharacter0x1a98',
u'\ub51d': 'HangulSyllableDilg',
u'\u949c': 'CjkUnifiedIdeograph949c',
u'\u6a23': 'CjkUnifiedIdeograph6a23',
u'\u4da2': 'CjkUnifiedIdeograph4da2',
u'\ue427': 'UnicodeCharacter0xe427',
u'\uc7a6': 'HangulSyllableJaj',
u'\u2329': 'LeftpointingAngleBracket',
u'\u02a8': 'LatinSmallLetterTcDigraphWithCurl',
u'\u9d2d': 'CjkUnifiedIdeograph9d2d',
u'\u7cac': 'CjkUnifiedIdeograph7cac',
u'\uf6b0': 'UnicodeCharacter0xf6b0',
u'\u5233': 'CjkUnifiedIdeograph5233',
u'\u35b2': 'CjkUnifiedIdeograph35b2',
u'\ucc37': 'HangulSyllableCalh',
u'\uafb6': 'HangulSyllableGgyop',
u'\u0b39': 'OriyaLetterHa',
u'\u853d': 'CjkUnifiedIdeograph853d',
u'\u64bc': 'CjkUnifiedIdeograph64bc',
u'\uff41': 'FullwidthLatinSmallLetterA',
u'\udec0': 'UnicodeCharacter0xdec0',
u'\u3a43': 'CjkUnifiedIdeograph3a43',
u'\u1dc2': 'CombiningSnakeBelow',
u'\ub447': 'HangulSyllableDyos',
u'\u97c6': 'CjkUnifiedIdeograph97c6',
u'\u6d4d': 'CjkUnifiedIdeograph6d4d',
u'\u4ccc': 'CjkUnifiedIdeograph4ccc',
u'\ue751': 'UnicodeCharacter0xe751',
u'\uc6d0': 'HangulSyllableWeon',
u'\u2253': 'ImageOfOrApproximatelyEqualTo',
u'\u05d2': 'HebrewLetterGimel',
u'\u9c57': 'CjkUnifiedIdeograph9c57',
u'\u7fd6': 'CjkUnifiedIdeograph7fd6',
u'\uf9da': 'CjkCompatibilityIdeographf9da',
u'\u555d': 'CjkUnifiedIdeograph555d',
u'\u34dc': 'CjkUnifiedIdeograph34dc',
u'\ucf61': 'HangulSyllableKolt',
u'\uaee0': 'HangulSyllableGgel',
u'\u0a63': 'UnicodeCharacter0xa63',
u'\u8467': 'CjkUnifiedIdeograph8467',
u'\u67e6': 'CjkUnifiedIdeograph67e6',
u'\ufe6b': 'SmallCommercialAt',
u'\ue1ea': 'UnicodeCharacter0xe1ea',
u'\u3d6d': 'CjkUnifiedIdeograph3d6d',
u'\u1cec': 'UnicodeCharacter0x1cec',
u'\ub771': 'HangulSyllableDdib',
u'\u96f0': 'CjkUnifiedIdeograph96f0',
u'\u6c77': 'CjkUnifiedIdeograph6c77',
u'\u4ff6': 'CjkUnifiedIdeograph4ff6',
u'\ue67b': 'UnicodeCharacter0xe67b',
u'\uc9fa': 'HangulSyllableJjaegg',
u'\u257d': 'BoxDrawingsLightUpAndHeavyDown',
u'\u04fc': 'CyrillicCapitalLetterHaWithHook',
u'\u1e81': 'LatinSmallLetterWWithGrave',
u'\u7f00': 'CjkUnifiedIdeograph7f00',
u'\u5b83': 'CjkUnifiedIdeograph5b83',
u'\ubc02': 'HangulSyllableMilm',
u'\u9885': 'CjkUnifiedIdeograph9885',
u'\uf904': 'CjkCompatibilityIdeographf904',
u'\ud587': 'HangulSyllableHaes',
u'\u2a0e': 'IntegralWithDoubleStroke',
u'\u0691': 'ArabicLetterRreh',
u'\u6710': 'CjkUnifiedIdeograph6710',
u'\u4393': 'CjkUnifiedIdeograph4393',
u'\ua412': 'YiSyllableQyt',
u'\u8095': 'CjkUnifiedIdeograph8095',
u'\ue114': 'UnicodeCharacter0xe114',
u'\ubd97': 'HangulSyllableBuc',
u'\ufa99': 'CjkCompatibilityIdeographfa99',
u'\u121e': 'EthiopicSyllableMo',
u'\u4f20': 'CjkUnifiedIdeograph4f20',
u'\u2ba3': 'UnicodeCharacter0x2ba3',
u'\u8c22': 'CjkUnifiedIdeograph8c22',
u'\u68a5': 'CjkUnifiedIdeograph68a5',
u'\uc924': 'HangulSyllableJweols',
u'\ua5a7': 'VaiSyllableDhhu',
u'\ue2a9': 'UnicodeCharacter0xe2a9',
u'\u3730': 'CjkUnifiedIdeograph3730',
u'\u13b3': 'CherokeeLetterLa',
u'\u7432': 'CjkUnifiedIdeograph7432',
u'\u50b5': 'CjkUnifiedIdeograph50b5',
u'\ub134': 'HangulSyllableNem',
u'\u8db7': 'CjkUnifiedIdeograph8db7',
u'\uee36': 'UnicodeCharacter0xee36',
u'\ucab9': 'HangulSyllableJjyet',
u'\u1f40': 'GreekSmallLetterOmicronWithPsili',
u'\u5c42': 'CjkUnifiedIdeograph5c42',
u'\u38c5': 'CjkUnifiedIdeograph38c5',
u'\u9944': 'CjkUnifiedIdeograph9944',
u'\u75c7': 'CjkUnifiedIdeograph75c7',
u'\ud646': 'HangulSyllableHolp',
u'\ub2c9': 'HangulSyllableNig',
u'\uefcb': 'UnicodeCharacter0xefcb',
u'\u0750': 'ArabicLetterBehWithThreeDotsHorizontallyBelow',
u'\u4452': 'CjkUnifiedIdeograph4452',
u'\u20d5': 'CombiningClockwiseArrowAbove',
u'\u8154': 'CjkUnifiedIdeograph8154',
u'\u5dd7': 'CjkUnifiedIdeograph5dd7',
u'\ube56': 'HangulSyllableBibs',
u'\u9ad9': 'CjkUnifiedIdeograph9ad9',
u'\ufb58': 'ArabicLetterPehInitialForm',
u'\ud7db': 'UnicodeCharacter0xd7db',
u'\u2c62': 'LatinCapitalLetterLWithMiddleTilde',
u'\u08e5': 'UnicodeCharacter0x8e5',
u'\u6964': 'CjkUnifiedIdeograph6964',
u'\u45e7': 'CjkUnifiedIdeograph45e7',
u'\ua666': 'CyrillicCapitalLetterSoftEm',
u'\u82e9': 'CjkUnifiedIdeograph82e9',
u'\ue368': 'UnicodeCharacter0xe368',
u'\ubfeb': 'HangulSyllableBbweogs',
u'\ufced': 'ArabicLigatureLamWithMeemMedialForm',
u'\u1472': 'CanadianSyllabicsKa',
u'\u5174': 'CjkUnifiedIdeograph5174',
u'\u2df7': 'CombiningCyrillicLetterIe',
u'\u8e76': 'CjkUnifiedIdeograph8e76',
u'\u6af9': 'CjkUnifiedIdeograph6af9',
u'\ucb78': 'HangulSyllableJjweoss',
u'\ua7fb': 'LatinEpigraphicLetterReversedF',
u'\ue4fd': 'UnicodeCharacter0xe4fd',
u'\ude01': 'UnicodeCharacter0xde01',
u'\u3984': 'CjkUnifiedIdeograph3984',
u'\u7686': 'CjkUnifiedIdeograph7686',
u'\ub388': 'HangulSyllableDek',
u'\u0f0b': 'TibetanMarkIntersyllabicTsheg',
u'\uf08a': 'UnicodeCharacter0xf08a',
u'\u4c0d': 'CjkUnifiedIdeograph4c0d',
u'\u890f': 'CjkUnifiedIdeograph890f',
u'\uc611': 'HangulSyllableYelg',
u'\u2194': 'LeftRightArrow',
u'\u5e96': 'CjkUnifiedIdeograph5e96',
u'\u9b98': 'CjkUnifiedIdeograph9b98',
u'\ud89a': 'UnicodeCharacter0xd89a',
u'\u341d': 'CjkUnifiedIdeograph341d',
u'\u711f': 'CjkUnifiedIdeograph711f',
u'\uae21': 'HangulSyllableGyilt',
u'\ueb23': 'UnicodeCharacter0xeb23',
u'\u09a4': 'BengaliLetterTa',
u'\u46a6': 'CjkUnifiedIdeograph46a6',
u'\u83a8': 'CjkUnifiedIdeograph83a8',
u'\uc0aa': 'HangulSyllableBbip',
u'\u1c2d': 'LepchaConsonantSignK',
u'\ufdac': 'ArabicLigatureLamWithJeemWithYehFinalForm',
u'\u592f': 'CjkUnifiedIdeograph592f',
u'\u9631': 'CjkUnifiedIdeograph9631',
u'\ud333': 'HangulSyllablePaelb',
u'\u2eb6': 'CjkRadicalSheep',
u'\u6bb8': 'CjkUnifiedIdeograph6bb8',
u'\ua8ba': 'SaurashtraVowelSignVocalicR',
u'\u043d': 'CyrillicSmallLetterEn',
u'\ue5bc': 'UnicodeCharacter0xe5bc',
u'\u413f': 'CjkUnifiedIdeograph413f',
u'\u7e41': 'CjkUnifiedIdeograph7e41',
u'\ubb43': 'HangulSyllableMulh',
u'\uf845': 'UnicodeCharacter0xf845',
u'\u16c6': 'RunicLetterShorttwigarA',
u'\u53c8': 'CjkUnifiedIdeograph53c8',
u'\u90ca': 'CjkUnifiedIdeograph90ca',
u'\ucdcc': 'HangulSyllableCwe',
u'\u294f': 'UpBarbRightDownBarbRightHarpoon',
u'\u6651': 'CjkUnifiedIdeograph6651',
u'\ua353': 'YiSyllableZhe',
u'\ue055': 'UnicodeCharacter0xe055',
u'\u3bd8': 'CjkUnifiedIdeograph3bd8',
u'\u78da': 'CjkUnifiedIdeograph78da',
u'\ub5dc': 'HangulSyllableDdyeon',
u'\u115f': 'HangulChoseongFiller',
u'\uf2de': 'UnicodeCharacter0xf2de',
u'\u4e61': 'CjkUnifiedIdeograph4e61',
u'\u8b63': 'CjkUnifiedIdeograph8b63',
u'\uc865': 'HangulSyllableJyeb',
u'\u23e8': 'UnicodeCharacter0x23e8',
u'\u60ea': 'CjkUnifiedIdeograph60ea',
u'\u9dec': 'CjkUnifiedIdeograph9dec',
u'\udaee': 'UnicodeCharacter0xdaee',
u'\u3671': 'CjkUnifiedIdeograph3671',
u'\u7373': 'CjkUnifiedIdeograph7373',
u'\ub075': 'HangulSyllableGgying',
u'\ued77': 'UnicodeCharacter0xed77',
u'\u0bf8': 'TamilAsAboveSign',
u'\u48fa': 'CjkUnifiedIdeograph48fa',
u'\u85fc': 'CjkUnifiedIdeograph85fc',
u'\uc2fe': 'HangulSyllableSsanh',
u'\u6181': 'CjkUnifiedIdeograph6181',
u'\u8604': 'CjkUnifiedIdeograph8604',
u'\uaa87': 'UnicodeCharacter0xaa87',
u'\u0c08': 'TeluguLetterIi',
u'\u308b': 'HiraganaLetterRu',
u'\ucf0a': 'HangulSyllableKelm',
u'\uf38d': 'UnicodeCharacter0xf38d',
u'\u550e': 'CjkUnifiedIdeograph550e',
u'\u7991': 'CjkUnifiedIdeograph7991',
u'\u9e14': 'CjkUnifiedIdeograph9e14',
u'\uc297': 'HangulSyllableSyulh',
u'\u2418': 'SymbolForCancel',
u'\u489b': 'CjkUnifiedIdeograph489b',
u'\ue71a': 'UnicodeCharacter0xe71a',
u'\u6d1e': 'CjkUnifiedIdeograph6d1e',
u'\u91a1': 'CjkUnifiedIdeograph91a1',
u'\u17a5': 'KhmerIndependentVowelQi',
u'\ub624': 'HangulSyllableDdoss',
u'\udaa7': 'UnicodeCharacter0xdaa7',
u'\u3c28': 'CjkUnifiedIdeograph3c28',
u'\u60ab': 'CjkUnifiedIdeograph60ab',
u'\uff2a': 'FullwidthLatinCapitalLetterJ',
u'\u852e': 'CjkUnifiedIdeograph852e',
u'\ua9b1': 'UnicodeCharacter0xa9b1',
u'\u0b32': 'OriyaLetterLa',
u'\u2fb5': 'KangxiRadicalWind',
u'\uce34': 'HangulSyllableCeuss',
u'\uf2b7': 'UnicodeCharacter0xf2b7',
u'\u5438': 'CjkUnifiedIdeograph5438',
u'\u78bb': 'CjkUnifiedIdeograph78bb',
u'\u9d3e': 'CjkUnifiedIdeograph9d3e',
u'\uc1c1': 'HangulSyllableSwat',
u'\u2342': 'AplFunctionalSymbolQuadBackslash',
u'\u47c5': 'CjkUnifiedIdeograph47c5',
u'\ue644': 'UnicodeCharacter0xe644',
u'\u6c48': 'CjkUnifiedIdeograph6c48',
u'\u90cb': 'CjkUnifiedIdeograph90cb',
u'\u16cf': 'RunicLetterTiwazTirTyrT',
u'\ub54e': 'HangulSyllableDdaegg',
u'\ud9d1': 'UnicodeCharacter0xd9d1',
u'\u3b52': 'CjkUnifiedIdeograph3b52',
u'\u5fd5': 'CjkUnifiedIdeograph5fd5',
u'\ufe54': 'SmallSemicolon',
u'\u8458': 'CjkUnifiedIdeograph8458',
u'\ua8db': 'UnicodeCharacter0xa8db',
u'\u0a5c': 'GurmukhiLetterRra',
u'\u2edf': 'CjkRadicalEatThree',
u'\ucd5e': 'HangulSyllableCoegg',
u'\uf1e1': 'UnicodeCharacter0xf1e1',
u'\u5362': 'CjkUnifiedIdeograph5362',
u'\u77e5': 'CjkUnifiedIdeograph77e5',
u'\u9c68': 'CjkUnifiedIdeograph9c68',
u'\uc0eb': 'HangulSyllableSyad',
u'\u226c': 'Between',
u'\u46ef': 'CjkUnifiedIdeograph46ef',
u'\ue56e': 'UnicodeCharacter0xe56e',
u'\u6b72': 'CjkUnifiedIdeograph6b72',
u'\u8ff5': 'CjkUnifiedIdeograph8ff5',
u'\u15f9': 'CanadianSyllabicsCarrierKhi',
u'\ub478': 'HangulSyllableDweols',
u'\ud8fb': 'UnicodeCharacter0xd8fb',
u'\u3a7c': 'CjkUnifiedIdeograph3a7c',
u'\u5eff': 'CjkUnifiedIdeograph5eff',
u'\ufd7e': 'ArabicLigatureQafWithMeemWithHahFinalForm',
u'\u6403': 'CjkUnifiedIdeograph6403',
u'\u8382': 'CjkUnifiedIdeograph8382',
u'\u0986': 'BengaliLetterAa',
u'\uad09': 'HangulSyllableGwalt',
u'\ucc88': 'HangulSyllableCyaels',
u'\u330d': 'SquareKarorii',
u'\u528c': 'CjkUnifiedIdeograph528c',
u'\uf60f': 'UnicodeCharacter0xf60f',
u'\u7c13': 'CjkUnifiedIdeograph7c13',
u'\u9b92': 'CjkUnifiedIdeograph9b92',
u'\u0217': 'LatinSmallLetterUWithInvertedBreve',
u'\u2196': 'NorthWestArrow',
u'\uc519': 'HangulSyllableSsyilt',
u'\ue498': 'UnicodeCharacter0xe498',
u'\u4b1d': 'CjkUnifiedIdeograph4b1d',
u'\u6a9c': 'CjkUnifiedIdeograph6a9c',
u'\u9423': 'CjkUnifiedIdeograph9423',
u'\ub3a2': 'HangulSyllableDyeoj',
u'\u1a27': 'UnicodeCharacter0x1a27',
u'\u39a6': 'CjkUnifiedIdeograph39a6',
u'\udd29': 'UnicodeCharacter0xdd29',
u'\ufca8': 'ArabicLigatureJeemWithMeemInitialForm',
u'\u632d': 'CjkUnifiedIdeograph632d',
u'\u82ac': 'CjkUnifiedIdeograph82ac',
u'\u08b0': 'UnicodeCharacter0x8b0',
u'\uac33': 'HangulSyllableGaec',
u'\ucbb2': 'HangulSyllableJjwij',
u'\u3237': 'ParenthesizedIdeographCongratulation',
u'\u51b6': 'CjkUnifiedIdeograph51b6',
u'\uf539': 'UnicodeCharacter0xf539',
u'\u7b3d': 'CjkUnifiedIdeograph7b3d',
u'\u9abc': 'CjkUnifiedIdeograph9abc',
u'\u0141': 'LatinCapitalLetterLWithStroke',
u'\u20c0': 'UnicodeCharacter0x20c0',
u'\uc443': 'HangulSyllableSsoec',
u'\ue3c2': 'UnicodeCharacter0xe3c2',
u'\u4a47': 'CjkUnifiedIdeograph4a47',
u'\u69c6': 'CjkUnifiedIdeograph69c6',
u'\u934d': 'CjkUnifiedIdeograph934d',
u'\ub2cc': 'HangulSyllableNin',
u'\u1951': 'TaiLeLetterXa',
u'\u38d0': 'CjkUnifiedIdeograph38d0',
u'\udc53': 'UnicodeCharacter0xdc53',
u'\ufbd2': 'UnicodeCharacter0xfbd2',
u'\u6257': 'CjkUnifiedIdeograph6257',
u'\u81d6': 'CjkUnifiedIdeograph81d6',
u'\u07da': 'NkoLetterRra',
u'\uab5d': 'UnicodeCharacter0xab5d',
u'\ucadc': 'HangulSyllableJjwan',
u'\u3161': 'HangulLetterEu',
u'\u50e0': 'CjkUnifiedIdeograph50e0',
u'\uf463': 'UnicodeCharacter0xf463',
u'\u7a67': 'CjkUnifiedIdeograph7a67',
u'\u99e6': 'CjkUnifiedIdeograph99e6',
u'\u1fea': 'GreekCapitalLetterUpsilonWithVaria',
u'\uc36d': 'HangulSyllableSseonj',
u'\ue2ec': 'UnicodeCharacter0xe2ec',
u'\u4971': 'CjkUnifiedIdeograph4971',
u'\u68f0': 'CjkUnifiedIdeograph68f0',
u'\u9277': 'CjkUnifiedIdeograph9277',
u'\ub1f6': 'HangulSyllableNyolp',
u'\u187b': 'UnicodeCharacter0x187b',
u'\u37fa': 'CjkUnifiedIdeograph37fa',
u'\udb7d': 'UnicodeCharacter0xdb7d',
u'\ufafc': 'UnicodeCharacter0xfafc',
u'\ue081': 'UnicodeCharacter0xe081',
u'\u8100': 'CjkUnifiedIdeograph8100',
u'\ua583': 'VaiSyllableLoo',
u'\u4202': 'CjkUnifiedIdeograph4202',
u'\u6685': 'CjkUnifiedIdeograph6685',
u'\u0704': 'SyriacSublinearColon',
u'\u2b87': 'UnicodeCharacter0x2b87',
u'\ud40e': 'HangulSyllablePwanh',
u'\uf891': 'UnicodeCharacter0xf891',
u'\u9910': 'CjkUnifiedIdeograph9910',
u'\ubd93': 'HangulSyllableBus',
u'\u5a12': 'CjkUnifiedIdeograph5a12',
u'\u7e95': 'CjkUnifiedIdeograph7e95',
u'\u1f14': 'GreekSmallLetterEpsilonWithPsiliAndOxia',
u'\u4397': 'CjkUnifiedIdeograph4397',
u'\u0499': 'CyrillicSmallLetterZeWithDescender',
u'\uec1e': 'UnicodeCharacter0xec1e',
u'\ub120': 'HangulSyllableNeok',
u'\ud5a3': 'HangulSyllableHyas',
u'\u7222': 'CjkUnifiedIdeograph7222',
u'\u96a5': 'CjkUnifiedIdeograph96a5',
u'\u3724': 'CjkUnifiedIdeograph3724',
u'\u5ba7': 'CjkUnifiedIdeograph5ba7',
u'\u1ca9': 'UnicodeCharacter0x1ca9',
u'\uc930': 'HangulSyllableJweok',
u'\uedb3': 'UnicodeCharacter0xedb3',
u'\u8a32': 'CjkUnifiedIdeograph8a32',
u'\uaeb5': 'HangulSyllableGgyaeng',
u'\u4f34': 'CjkUnifiedIdeograph4f34',
u'\u73b7': 'CjkUnifiedIdeograph73b7',
u'\u1036': 'MyanmarSignAnusvara',
u'\u34b9': 'CjkUnifiedIdeograph34b9',
u'\ue140': 'UnicodeCharacter0xe140',
u'\ua242': 'YiSyllableMgu',
u'\uc6c5': 'HangulSyllableUng',
u'\u6744': 'CjkUnifiedIdeograph6744',
u'\u8bc7': 'CjkUnifiedIdeograph8bc7',
u'\u2846': 'BraillePatternDots237',
u'\u4cc9': 'CjkUnifiedIdeograph4cc9',
u'\u11cb': 'HangulJongseongTikeutrieul',
u'\uf950': 'CjkCompatibilityIdeographf950',
u'\uba52': 'HangulSyllableMeop',
u'\uded5': 'UnicodeCharacter0xded5',
u'\u7f54': 'CjkUnifiedIdeograph7f54',
u'\ua3d7': 'YiSyllableRup',
u'\u4056': 'CjkUnifiedIdeograph4056',
u'\u64d9': 'CjkUnifiedIdeograph64d9',
u'\u0558': 'UnicodeCharacter0x558',
u'\u29db': 'RightDoubleWigglyFence',
u'\ud262': 'HangulSyllableTweop',
u'\uf6e5': 'UnicodeCharacter0xf6e5',
u'\u9764': 'CjkUnifiedIdeograph9764',
u'\ubbe7': 'HangulSyllableMyilb',
u'\u5866': 'CjkUnifiedIdeograph5866',
u'\u7ce9': 'CjkUnifiedIdeograph7ce9',
u'\u1d68': 'GreekSubscriptSmallLetterRho',
u'\u41eb': 'CjkUnifiedIdeograph41eb',
u'\u02ed': 'ModifierLetterUnaspirated',
u'\uea72': 'UnicodeCharacter0xea72',
u'\uaf74': 'HangulSyllableGgwaem',
u'\ud3f7': 'HangulSyllablePolb',
u'\u7076': 'CjkUnifiedIdeograph7076',
u'\u94f9': 'CjkUnifiedIdeograph94f9',
u'\u3578': 'CjkUnifiedIdeograph3578',
u'\u59fb': 'CjkUnifiedIdeograph59fb',
u'\u1afd': 'UnicodeCharacter0x1afd',
u'\u2001': 'EmQuad',
u'\uc784': 'HangulSyllableIm',
u'\u8886': 'CjkUnifiedIdeograph8886',
u'\u4d88': 'CjkUnifiedIdeograph4d88',
u'\uf10b': 'UnicodeCharacter0xf10b',
u'\u0e8a': 'LaoLetterSoTam',
u'\ub20d': 'HangulSyllableNulg',
u'\u770f': 'CjkUnifiedIdeograph770f',
u'\u3811': 'CjkUnifiedIdeograph3811',
u'\udf94': 'UnicodeCharacter0xdf94',
u'\ua096': 'YiSyllableHmie',
u'\u6598': 'CjkUnifiedIdeograph6598',
u'\u269a': 'StaffOfHermes',
u'\uca1d': 'HangulSyllableJjyalg',
u'\u8f1f': 'CjkUnifiedIdeograph8f1f',
u'\u5021': 'CjkUnifiedIdeograph5021',
u'\u1523': 'CanadianSyllabicsShwaa',
u'\uf7a4': 'UnicodeCharacter0xf7a4',
u'\ub8a6': 'HangulSyllableRwaebs',
u'\u7da8': 'CjkUnifiedIdeograph7da8',
u'\u3eaa': 'CjkUnifiedIdeograph3eaa',
u'\ue22d': 'UnicodeCharacter0xe22d',
u'\u03ac': 'GreekSmallLetterAlphaWithTonos',
u'\ua72f': 'LatinSmallLetterCuatrilloWithComma',
u'\u6831': 'CjkUnifiedIdeograph6831',
u'\u2d33': 'TifinaghLetterYag',
u'\ud0b6': 'HangulSyllableKibs',
u'\u95b8': 'CjkUnifiedIdeograph95b8',
u'\u56ba': 'CjkUnifiedIdeograph56ba',
u'\ufa3d': 'CjkCompatibilityIdeographfa3d',
u'\u1bbc': 'UnicodeCharacter0x1bbc',
u'\ubf3f': 'HangulSyllableBbyeh',
u'\u8041': 'CjkUnifiedIdeograph8041',
u'\u4543': 'CjkUnifiedIdeograph4543',
u'\u0645': 'ArabicLetterMeem',
u'\ue8c6': 'UnicodeCharacter0xe8c6',
u'\uadc8': 'HangulSyllableGwil',
u'\u6eca': 'CjkUnifiedIdeograph6eca',
u'\u33cc': 'SquareIn',
u'\ud74f': 'HangulSyllableHyuh',
u'\u9851': 'CjkUnifiedIdeograph9851',
u'\u5d53': 'CjkUnifiedIdeograph5d53',
u'\u1e55': 'LatinSmallLetterPWithAcute',
u'\uc5d8': 'HangulSyllableEl',
u'\u86da': 'CjkUnifiedIdeograph86da',
u'\u4bdc': 'CjkUnifiedIdeograph4bdc',
u'\uef5f': 'UnicodeCharacter0xef5f',
u'\u0cde': 'KannadaLetterFa',
u'\ub061': 'HangulSyllableGgyig',
u'\u7563': 'CjkUnifiedIdeograph7563',
u'\u3665': 'CjkUnifiedIdeograph3665',
u'\udde8': 'UnicodeCharacter0xdde8',
u'\u9eea': 'CjkUnifiedIdeograph9eea',
u'\u63ec': 'CjkUnifiedIdeograph63ec',
u'\u24ee': 'NegativeCircledNumberFourteen',
u'\uc871': 'HangulSyllableJog',
u'\u8d73': 'CjkUnifiedIdeograph8d73',
u'\u4e75': 'CjkUnifiedIdeograph4e75',
u'\u1377': 'EthiopicNumberSixty',
u'\uf5f8': 'UnicodeCharacter0xf5f8',
u'\ub6fa': 'HangulSyllableDdwilm',
u'\u7bfc': 'CjkUnifiedIdeograph7bfc',
u'\u3cfe': 'CjkUnifiedIdeograph3cfe',
u'\u0200': 'LatinCapitalLetterAWithDoubleGrave',
u'\u2683': 'DieFace4',
u'\uc102': 'HangulSyllableSyaegg',
u'\ue585': 'UnicodeCharacter0xe585',
u'\u4b06': 'CjkUnifiedIdeograph4b06',
u'\u6f89': 'CjkUnifiedIdeograph6f89',
u'\u900c': 'CjkUnifiedIdeograph900c',
u'\ub48f': 'HangulSyllableDwed',
u'\u1a10': 'BugineseLetterYa',
u'\u3e93': 'CjkUnifiedIdeograph3e93',
u'\ud912': 'UnicodeCharacter0xd912',
u'\ufd95': 'ArabicLigatureNoonWithHahWithMeemInitialForm',
u'\u6316': 'CjkUnifiedIdeograph6316',
u'\u8799': 'CjkUnifiedIdeograph8799',
u'\u099d': 'BengaliLetterJha',
u'\ua81c': 'SylotiNagriLetterBho',
u'\ucc9f': 'HangulSyllableCeod',
u'\u3220': 'ParenthesizedIdeographOne',
u'\u56a3': 'CjkUnifiedIdeograph56a3',
u'\uf122': 'UnicodeCharacter0xf122',
u'\u7b26': 'CjkUnifiedIdeograph7b26',
u'\u9fa9': 'CjkUnifiedIdeograph9fa9',
u'\u21ad': 'LeftRightWaveArrow',
u'\uc02c': 'HangulSyllableBbwils',
u'\ue4af': 'UnicodeCharacter0xe4af',
u'\u4a30': 'CjkUnifiedIdeograph4a30',
u'\u6eb3': 'CjkUnifiedIdeograph6eb3',
u'\u9336': 'CjkUnifiedIdeograph9336',
u'\ub7b9': 'HangulSyllableRyanj',
u'\u153a': 'CanadianSyllabicsWestcreeYwa',
u'\u39bd': 'CjkUnifiedIdeograph39bd',
u'\ud83c': 'UnicodeCharacter0xd83c',
u'\ufcbf': 'ArabicLigatureFehWithHahInitialForm',
u'\u6240': 'CjkUnifiedIdeograph6240',
u'\u86c3': 'CjkUnifiedIdeograph86c3',
u'\u08c7': 'UnicodeCharacter0x8c7',
u'\uab46': 'UnicodeCharacter0xab46',
u'\ucfc9': 'HangulSyllableKyonj',
u'\u2d4a': 'TifinaghLetterYazh',
u'\u51cd': 'CjkUnifiedIdeograph51cd',
u'\uf04c': 'UnicodeCharacter0xf04c',
u'\u7a50': 'CjkUnifiedIdeograph7a50',
u'\u9ed3': 'CjkUnifiedIdeograph9ed3',
u'\u20d7': 'CombiningRightArrowAbove',
u'\uc356': 'HangulSyllableSsyaelm',
u'\ue7d9': 'UnicodeCharacter0xe7d9',
u'\u455a': 'CjkUnifiedIdeograph455a',
u'\u69dd': 'CjkUnifiedIdeograph69dd',
u'\u9260': 'CjkUnifiedIdeograph9260',
u'\ub6e3': 'HangulSyllableDdwelh',
u'\u1464': 'CanadianSyllabicsWestcreeTwaa',
u'\u38e7': 'CjkUnifiedIdeograph38e7',
u'\udb66': 'UnicodeCharacter0xdb66',
u'\uffe9': 'HalfwidthLeftwardsArrow',
u'\u5d6a': 'CjkUnifiedIdeograph5d6a',
u'\u81ed': 'CjkUnifiedIdeograph81ed',
u'\u0bf1': 'TamilNumberOneHundred',
u'\uaa70': 'UnicodeCharacter0xaa70',
u'\ucef3': 'HangulSyllableKeolh',
u'\u2c74': 'LatinSmallLetterVWithCurl',
u'\u50f7': 'CjkUnifiedIdeograph50f7',
u'\uf376': 'UnicodeCharacter0xf376',
u'\u757a': 'CjkUnifiedIdeograph757a',
u'\u99fd': 'CjkUnifiedIdeograph99fd',
u'\ua301': 'YiSyllableNzurx',
u'\uc280': 'HangulSyllableSwiss',
u'\u2505': 'BoxDrawingsHeavyTripleDashHorizontal',
u'\u4484': 'CjkUnifiedIdeograph4484',
u'\ue807': 'UnicodeCharacter0xe807',
u'\u720b': 'CjkUnifiedIdeograph720b',
u'\u8d8a': 'CjkUnifiedIdeograph8d8a',
u'\u178e': 'KhmerLetterNno',
u'\ubb11': 'HangulSyllableMoeng',
u'\uda90': 'UnicodeCharacter0xda90',
u'\u3d15': 'CjkUnifiedIdeograph3d15',
u'\u5c94': 'CjkUnifiedIdeograph5c94',
u'\u8a1b': 'CjkUnifiedIdeograph8a1b',
u'\ua59a': 'VaiSyllableWun',
u'\u0c1f': 'TeluguLetterTta',
u'\u2f9e': 'KangxiRadicalCart',
u'\ud321': 'HangulSyllablePang',
u'\uf2a0': 'UnicodeCharacter0xf2a0',
u'\u5525': 'CjkUnifiedIdeograph5525',
u'\u74a4': 'CjkUnifiedIdeograph74a4',
u'\ua22b': 'YiSyllableGgux',
u'\ubdaa': 'HangulSyllableBweolp',
u'\u242f': 'UnicodeCharacter0x242f',
u'\u47ae': 'CjkUnifiedIdeograph47ae',
u'\ueb31': 'UnicodeCharacter0xeb31',
u'\u6d35': 'CjkUnifiedIdeograph6d35',
u'\u8cb4': 'CjkUnifiedIdeograph8cb4',
u'\u16b8': 'RunicLetterGar',
u'\uba3b': 'HangulSyllableMeogs',
u'\ud5ba': 'HangulSyllableHyaelp',
u'\u3c3f': 'CjkUnifiedIdeograph3c3f',
u'\u5fbe': 'CjkUnifiedIdeograph5fbe',
u'\u8545': 'CjkUnifiedIdeograph8545',
u'\ua4c4': 'YiRadicalZziet',
u'\u0f49': 'TibetanLetterNya',
u'\u2ec8': 'CjkRadicalCsimplifiedSpeech',
u'\ud24b': 'HangulSyllableTweogs',
u'\uedca': 'UnicodeCharacter0xedca',
u'\u544f': 'CjkUnifiedIdeograph544f',
u'\u77ce': 'CjkUnifiedIdeograph77ce',
u'\u9d55': 'CjkUnifiedIdeograph9d55',
u'\ubcd4': 'HangulSyllableByeok',
u'\u2759': 'MediumVerticalBar',
u'\u46d8': 'CjkUnifiedIdeograph46d8',
u'\uea5b': 'UnicodeCharacter0xea5b',
u'\u6c5f': 'CjkUnifiedIdeograph6c5f',
u'\u8fde': 'CjkUnifiedIdeograph8fde',
u'\u11e2': 'HangulJongseongKapyeounmieum',
u'\ub565': 'HangulSyllableDdaet',
u'\ud4e4': 'HangulSyllablePwik',
u'\u3f69': 'CjkUnifiedIdeograph3f69',
u'\u5ee8': 'CjkUnifiedIdeograph5ee8',
u'\u846f': 'CjkUnifiedIdeograph846f',
u'\ua7ee': 'UnicodeCharacter0xa7ee',
u'\u0e73': 'UnicodeCharacter0xe73',
u'\u29f2': 'ErrorbarredWhiteCircle',
u'\ucd75': 'HangulSyllableCoet',
u'\uecf4': 'UnicodeCharacter0xecf4',
u'\u5779': 'CjkUnifiedIdeograph5779',
u'\u76f8': 'CjkUnifiedIdeograph76f8',
u'\u9c7f': 'CjkUnifiedIdeograph9c7f',
u'\ubffe': 'HangulSyllableBbweoj',
u'\uca06': 'HangulSyllableJjaelp',
u'\uee89': 'UnicodeCharacter0xee89',
u'\u8f08': 'CjkUnifiedIdeograph8f08',
u'\ub38b': 'HangulSyllableDeh',
u'\u4c0a': 'CjkUnifiedIdeograph4c0a',
u'\u708d': 'CjkUnifiedIdeograph708d',
u'\u110c': 'HangulChoseongCieuc',
u'\u358f': 'CjkUnifiedIdeograph358f',
u'\ue216': 'UnicodeCharacter0xe216',
u'\ua718': 'ModifierLetterDotSlash',
u'\ucb9b': 'HangulSyllableJjweh',
u'\u641a': 'CjkUnifiedIdeograph641a',
u'\u889d': 'CjkUnifiedIdeograph889d',
u'\u291c': 'RightwardsDoubleArrowtail',
u'\u4d9f': 'CjkUnifiedIdeograph4d9f',
u'\u12a1': 'EthiopicSyllableGlottalU',
u'\ufa26': 'CjkCompatibilityIdeographfa26',
u'\ubf28': 'HangulSyllableBbyen',
u'\ue3ab': 'UnicodeCharacter0xe3ab',
u'\u7c2a': 'CjkUnifiedIdeograph7c2a',
u'\ua0ad': 'YiSyllableHmyr',
u'\u412c': 'CjkUnifiedIdeograph412c',
u'\u65af': 'CjkUnifiedIdeograph65af',
u'\u062e': 'ArabicLetterKhah',
u'\u2ab1': 'PrecedesAboveSinglelineNotEqualTo',
u'\ud738': 'HangulSyllableHyun',
u'\ufbbb': 'UnicodeCharacter0xfbbb',
u'\u943a': 'CjkUnifiedIdeograph943a',
u'\ub8bd': 'HangulSyllableRoelt',
u'\u593c': 'CjkUnifiedIdeograph593c',
u'\u7dbf': 'CjkUnifiedIdeograph7dbf',
u'\u1e3e': 'LatinCapitalLetterMWithAcute',
u'\u42c1': 'CjkUnifiedIdeograph42c1',
u'\u07c3': 'NkoDigitThree',
u'\uef48': 'UnicodeCharacter0xef48',
u'\uac4a': 'HangulSyllableGyabs',
u'\ud0cd': 'HangulSyllableTalt',
u'\u714c': 'CjkUnifiedIdeograph714c',
u'\u95cf': 'CjkUnifiedIdeograph95cf',
u'\u364e': 'CjkUnifiedIdeograph364e',
u'\u5ad1': 'CjkUnifiedIdeograph5ad1',
u'\u1fd3': 'GreekSmallLetterIotaWithDialytikaAndOxia',
u'\uc45a': 'HangulSyllableSsyobs',
u'\ue8dd': 'UnicodeCharacter0xe8dd',
u'\u895c': 'CjkUnifiedIdeograph895c',
u'\uaddf': 'HangulSyllableGyugs',
u'\u4e5e': 'CjkUnifiedIdeograph4e5e',
u'\u72e1': 'CjkUnifiedIdeograph72e1',
u'\u1360': 'EthiopicSectionMark',
u'\u37e3': 'CjkUnifiedIdeograph37e3',
u'\udc6a': 'UnicodeCharacter0xdc6a',
u'\ua16c': 'YiSyllableHniet',
u'\uc5ef': 'HangulSyllableYeogs',
u'\u666e': 'CjkUnifiedIdeograph666e',
u'\u8af1': 'CjkUnifiedIdeograph8af1',
u'\u2b70': 'UnicodeCharacter0x2b70',
u'\u4ff3': 'CjkUnifiedIdeograph4ff3',
u'\u0cf5': 'UnicodeCharacter0xcf5',
u'\uf47a': 'UnicodeCharacter0xf47a',
u'\ub97c': 'HangulSyllableReul',
u'\uddff': 'UnicodeCharacter0xddff',
u'\u7e7e': 'CjkUnifiedIdeograph7e7e',
u'\u4380': 'CjkUnifiedIdeograph4380',
u'\ue703': 'UnicodeCharacter0xe703',
u'\x82': 'UnicodeCharacter0x82',
u'\ua405': 'YiSyllableQuox',
u'\u6907': 'CjkUnifiedIdeograph6907',
u'\u2e09': 'LeftTranspositionBracket',
u'\ud18c': 'HangulSyllableTyel',
u'\u968e': 'CjkUnifiedIdeograph968e',
u'\u5b90': 'CjkUnifiedIdeograph5b90',
u'\uff13': 'FullwidthDigitThree',
u'\u1892': 'MongolianLetterAliGaliPa',
u'\ubc15': 'HangulSyllableBag',
u'\u8117': 'CjkUnifiedIdeograph8117',
u'\u4619': 'CjkUnifiedIdeograph4619',
u'\u0b1b': 'OriyaLetterCha',
u'\ue99c': 'UnicodeCharacter0xe99c',
u'\uae9e': 'HangulSyllableGgyap',
u'\u73a0': 'CjkUnifiedIdeograph73a0',
u'\u30a2': 'KatakanaLetterA',
u'\ud425': 'HangulSyllablePwaeg',
u'\u9927': 'CjkUnifiedIdeograph9927',
u'\u5e29': 'CjkUnifiedIdeograph5e29',
u'\u232b': 'EraseToTheLeft',
u'\uc6ae': 'HangulSyllableYop',
u'\u8bb0': 'CjkUnifiedIdeograph8bb0',
u'\u48b2': 'CjkUnifiedIdeograph48b2',
u'\uec35': 'UnicodeCharacter0xec35',
u'\u0db4': 'SinhalaLetterAlpapraanaPayanna',
u'\ub137': 'HangulSyllableNes',
u'\u7639': 'CjkUnifiedIdeograph7639',
u'\u3b3b': 'CjkUnifiedIdeograph3b3b',
u'\udebe': 'UnicodeCharacter0xdebe',
u'\ua3c0': 'YiSyllableShyt',
u'\u60c2': 'CjkUnifiedIdeograph60c2',
u'\u25c4': 'BlackLeftpointingPointer',
u'\uc947': 'HangulSyllableJwes',
u'\u8e49': 'CjkUnifiedIdeograph8e49',
u'\u534b': 'CjkUnifiedIdeograph534b',
u'\u104d': 'MyanmarSymbolCompleted',
u'\uf6ce': 'UnicodeCharacter0xf6ce',
u'\ubbd0': 'HangulSyllableMeum',
u'\u78d2': 'CjkUnifiedIdeograph78d2',
u'\u3dd4': 'CjkUnifiedIdeograph3dd4',
u'\ue157': 'UnicodeCharacter0xe157',
u'\u02d6': 'ModifierLetterPlusSign',
u'\ua659': 'CyrillicSmallLetterClosedLittleYus',
u'\u6b5b': 'CjkUnifiedIdeograph6b5b',
u'\u285d': 'BraillePatternDots13457',
u'\ud3e0': 'HangulSyllablePyem',
u'\u90e2': 'CjkUnifiedIdeograph90e2',
u'\u55e4': 'CjkUnifiedIdeograph55e4',
u'\uf967': 'CjkCompatibilityIdeographf967',
u'\u1ae6': 'UnicodeCharacter0x1ae6',
u'\ube69': 'HangulSyllableBbalg',
u'\u836b': 'CjkUnifiedIdeograph836b',
u'\u406d': 'CjkUnifiedIdeograph406d',
u'\u056f': 'ArmenianSmallLetterKen',
u'\uebf0': 'UnicodeCharacter0xebf0',
u'\ua8f2': 'UnicodeCharacter0xa8f2',
u'\u6df4': 'CjkUnifiedIdeograph6df4',
u'\u32f6': 'CircledKatakanaRa',
u'\ud679': 'HangulSyllableHwaelg',
u'\u9b7b': 'CjkUnifiedIdeograph9b7b',
u'\u587d': 'CjkUnifiedIdeograph587d',
u'\u1d7f': 'LatinSmallLetterUpsilonWithStroke',
u'\u6581': 'CjkUnifiedIdeograph6581',
u'\u8204': 'CjkUnifiedIdeograph8204',
u'\uae87': 'HangulSyllableGgyags',
u'\u0808': 'UnicodeCharacter0x808',
u'\u348b': 'CjkUnifiedIdeograph348b',
u'\ucb0a': 'HangulSyllableJjwaej',
u'\uf78d': 'UnicodeCharacter0xf78d',
u'\u510e': 'CjkUnifiedIdeograph510e',
u'\u7d91': 'CjkUnifiedIdeograph7d91',
u'\u9a14': 'CjkUnifiedIdeograph9a14',
u'\uc697': 'HangulSyllableYogs',
u'\u2018': 'LeftSingleQuotationMark',
u'\u4c9b': 'CjkUnifiedIdeograph4c9b',
u'\ue31a': 'UnicodeCharacter0xe31a',
u'\u691e': 'CjkUnifiedIdeograph691e',
u'\u95a1': 'CjkUnifiedIdeograph95a1',
u'\u13a5': 'CherokeeLetterV',
u'\ub224': 'HangulSyllableNweon',
u'\udea7': 'UnicodeCharacter0xdea7',
u'\u3828': 'CjkUnifiedIdeograph3828',
u'\u64ab': 'CjkUnifiedIdeograph64ab',
u'\ufb2a': 'HebrewLetterShinWithShinDot',
u'\u812e': 'CjkUnifiedIdeograph812e',
u'\uadb1': 'HangulSyllableGwelt',
u'\u0f32': 'TibetanDigitHalfNine',
u'\u2bb5': 'UnicodeCharacter0x2bb5',
u'\uca34': 'HangulSyllableJjyaen',
u'\uf6b7': 'UnicodeCharacter0xf6b7',
u'\u5038': 'CjkUnifiedIdeograph5038',
u'\u7cbb': 'CjkUnifiedIdeograph7cbb',
u'\u993e': 'CjkUnifiedIdeograph993e',
u'\uc5c1': 'HangulSyllableEolt',
u'\u2742': 'CircledOpenCentreEightPointedStar',
u'\u43c5': 'CjkUnifiedIdeograph43c5',
u'\ue244': 'UnicodeCharacter0xe244',
u'\u6848': 'CjkUnifiedIdeograph6848',
u'\u94cb': 'CjkUnifiedIdeograph94cb',
u'\u12cf': 'EthiopicSyllableWoa',
u'\ub14e': 'HangulSyllableNyeolp',
u'\uddd1': 'UnicodeCharacter0xddd1',
u'\u3f52': 'CjkUnifiedIdeograph3f52',
u'\u5bd5': 'CjkUnifiedIdeograph5bd5',
u'\ufa54': 'CjkCompatibilityIdeographfa54',
u'\u8058': 'CjkUnifiedIdeograph8058',
u'\uacdb': 'HangulSyllableGyec',
u'\u0e5c': 'UnicodeCharacter0xe5c',
u'\u2adf': 'ShortDownTack',
u'\uc95e': 'HangulSyllableJwilp',
u'\uf5e1': 'UnicodeCharacter0xf5e1',
u'\u5762': 'CjkUnifiedIdeograph5762',
u'\u73e5': 'CjkUnifiedIdeograph73e5',
u'\u9868': 'CjkUnifiedIdeograph9868',
u'\uc4eb': 'HangulSyllableSsyuc',
u'\u266c': 'BeamedSixteenthNotes',
u'\u42ef': 'CjkUnifiedIdeograph42ef',
u'\ue16e': 'UnicodeCharacter0xe16e',
u'\u6f72': 'CjkUnifiedIdeograph6f72',
u'\u8bf5': 'CjkUnifiedIdeograph8bf5',
u'\u11f9': 'HangulJongseongYeorinhieuh',
u'\ub078': 'HangulSyllableGgyik',
u'\udcfb': 'UnicodeCharacter0xdcfb',
u'\u3e7c': 'CjkUnifiedIdeograph3e7c',
u'\u5aff': 'CjkUnifiedIdeograph5aff',
u'\uf97e': 'CjkCompatibilityIdeographf97e',
u'\u6003': 'CjkUnifiedIdeograph6003',
u'\u8782': 'CjkUnifiedIdeograph8782',
u'\u0d86': 'SinhalaLetterAayanna',
u'\ua909': 'KayahLiDigitNine',
u'\uc888': 'HangulSyllableJok',
u'\u370d': 'CjkUnifiedIdeograph370d',
u'\u568c': 'CjkUnifiedIdeograph568c',
u'\uf20f': 'UnicodeCharacter0xf20f',
u'\u7813': 'CjkUnifiedIdeograph7813',
u'\u9f92': 'CjkUnifiedIdeograph9f92',
u'\u0617': 'ArabicSmallHighZain',
u'\u2596': 'QuadrantLowerLeft',
u'\uc119': 'HangulSyllableSyaet',
u'\ue098': 'UnicodeCharacter0xe098',
u'\u4f1d': 'CjkUnifiedIdeograph4f1d',
u'\u6e9c': 'CjkUnifiedIdeograph6e9c',
u'\u9023': 'CjkUnifiedIdeograph9023',
u'\ub7a2': 'HangulSyllableRaelm',
u'\u1e27': 'LatinSmallLetterHWithDiaeresis',
u'\u3da6': 'CjkUnifiedIdeograph3da6',
u'\ud929': 'UnicodeCharacter0xd929',
u'\uf8a8': 'UnicodeCharacter0xf8a8',
u'\u672d': 'CjkUnifiedIdeograph672d',
u'\u86ac': 'CjkUnifiedIdeograph86ac',
u'\u0cb0': 'KannadaLetterRa',
u'\ua833': 'UnicodeCharacter0xa833',
u'\ucfb2': 'HangulSyllableKoelm',
u'\u3637': 'CjkUnifiedIdeograph3637',
u'\u55b6': 'CjkUnifiedIdeograph55b6',
u'\uf139': 'UnicodeCharacter0xf139',
u'\u7f3d': 'CjkUnifiedIdeograph7f3d',
u'\u9ebc': 'CjkUnifiedIdeograph9ebc',
u'\u0541': 'ArmenianCapitalLetterJa',
u'\u24c0': 'CircledLatinCapitalLetterK',
u'\uc043': 'HangulSyllableBbyud',
u'\ue7c2': 'UnicodeCharacter0xe7c2',
u'\u4e47': 'CjkUnifiedIdeograph4e47',
u'\u6dc6': 'CjkUnifiedIdeograph6dc6',
u'\u974d': 'CjkUnifiedIdeograph974d',
u'\ub6cc': 'HangulSyllableDdweoss',
u'\u1d51': 'ModifierLetterSmallEng',
u'\u3cd0': 'CjkUnifiedIdeograph3cd0',
u'\ud853': 'UnicodeCharacter0xd853',
u'\uffd2': 'HalfwidthHangulLetterYo',
u'\u6657': 'CjkUnifiedIdeograph6657',
u'\u85d6': 'CjkUnifiedIdeograph85d6',
u'\u03da': 'GreekLetterStigma',
u'\uaf5d': 'HangulSyllableGgwang',
u'\ucedc': 'HangulSyllableKyaess',
u'\u3561': 'CjkUnifiedIdeograph3561',
u'\u54e0': 'CjkUnifiedIdeograph54e0',
u'\uf063': 'UnicodeCharacter0xf063',
u'\u7e67': 'CjkUnifiedIdeograph7e67',
u'\u9de6': 'CjkUnifiedIdeograph9de6',
u'\u046b': 'CyrillicSmallLetterBigYus',
u'\u1bea': 'UnicodeCharacter0x1bea',
u'\uc76d': 'HangulSyllableYing',
u'\ue6ec': 'UnicodeCharacter0xe6ec',
u'\u4d71': 'CjkUnifiedIdeograph4d71',
u'\u6cf0': 'CjkUnifiedIdeograph6cf0',
u'\u9677': 'CjkUnifiedIdeograph9677',
u'\ub5f6': 'HangulSyllableDdyegg',
u'\u1c7b': 'OlChikiRelaa',
u'\u33fa': 'IdeographicTelegraphSymbolForDayTwentyseven',
u'\udf7d': 'UnicodeCharacter0xdf7d',
u'\ufefc': 'ArabicLigatureLamWithAlefFinalForm',
u'\ue481': 'UnicodeCharacter0xe481',
u'\u8500': 'CjkUnifiedIdeograph8500',
u'\ua183': 'YiSyllableNiep',
u'\u4602': 'CjkUnifiedIdeograph4602',
u'\u6285': 'CjkUnifiedIdeograph6285',
u'\u0304': 'CombiningMacron',
u'\u2f87': 'KangxiRadicalOppose',
u'\ud00e': 'HangulSyllableKweobs',
u'\ufc91': 'ArabicLigatureYehWithRehFinalForm',
u'\u9d10': 'CjkUnifiedIdeograph9d10',
u'\ub993': 'HangulSyllableRyigs',
u'\u5e12': 'CjkUnifiedIdeograph5e12',
u'\u7a95': 'CjkUnifiedIdeograph7a95',
u'\u1b14': 'BalineseLetterKaMahaprana',
u'\u4797': 'CjkUnifiedIdeograph4797',
u'\x99': 'UnicodeCharacter0x99',
u'\ue81e': 'UnicodeCharacter0xe81e',
u'\ub520': 'HangulSyllableDils',
u'\ud1a3': 'HangulSyllableTogs',
u'\u7622': 'CjkUnifiedIdeograph7622',
u'\u92a5': 'CjkUnifiedIdeograph92a5',
u'\u3324': 'SquareDaasu',
u'\u5fa7': 'CjkUnifiedIdeograph5fa7',
u'\u18a9': 'MongolianLetterAliGaliDagalga',
u'\ucd30': 'HangulSyllableCwals',
u'\ue9b3': 'UnicodeCharacter0xe9b3',
u'\u8e32': 'CjkUnifiedIdeograph8e32',
u'\uaab5': 'UnicodeCharacter0xaab5',
u'\u4b34': 'CjkUnifiedIdeograph4b34',
u'\u77b7': 'CjkUnifiedIdeograph77b7',
u'\u1436': 'CanadianSyllabicsCarrierHee',
u'\u30b9': 'KatakanaLetterSu',
u'\ue540': 'UnicodeCharacter0xe540',
u'\ua642': 'CyrillicCapitalLetterDzelo',
u'\uc2c5': 'HangulSyllableSyinj',
u'\u6344': 'CjkUnifiedIdeograph6344',
u'\u8fc7': 'CjkUnifiedIdeograph8fc7',
u'\u2c46': 'GlagoliticSmallLetterUku',
u'\u48c9': 'CjkUnifiedIdeograph48c9',
u'\u15cb': 'CanadianSyllabicsCarrierRo',
u'\ufd50': 'ArabicLigatureTehWithJeemWithMeemInitialForm',
u'\ube52': 'HangulSyllableBilp',
u'\udad5': 'UnicodeCharacter0xdad5',
u'\u7b54': 'CjkUnifiedIdeograph7b54',
u'\ua7d7': 'UnicodeCharacter0xa7d7',
u'\u4456': 'CjkUnifiedIdeograph4456',
u'\u60d9': 'CjkUnifiedIdeograph60d9',
u'\u0158': 'LatinCapitalLetterRWithCaron',
u'\u2ddb': 'EthiopicSyllableGyaa',
u'\ud662': 'HangulSyllableHwalp',
u'\uf2e5': 'UnicodeCharacter0xf2e5',
u'\u9364': 'CjkUnifiedIdeograph9364',
u'\ubfe7': 'HangulSyllableBbuh',
u'\u5c66': 'CjkUnifiedIdeograph5c66',
u'\u78e9': 'CjkUnifiedIdeograph78e9',
u'\u1968': 'TaiLeLetterOo',
u'\u45eb': 'CjkUnifiedIdeograph45eb',
u'\u06ed': 'ArabicSmallLowMeem',
u'\uee72': 'UnicodeCharacter0xee72',
u'\uab74': 'UnicodeCharacter0xab74',
u'\ud7f7': 'UnicodeCharacter0xd7f7',
u'\u7476': 'CjkUnifiedIdeograph7476',
u'\u90f9': 'CjkUnifiedIdeograph90f9',
u'\u3178': 'HangulLetterKapyeounpieup',
u'\u5dfb': 'CjkUnifiedIdeograph5dfb',
u'\u1efd': 'LatinSmallLetterMiddlewelshV',
u'\u2401': 'SymbolForStartOfHeading',
u'\uc384': 'HangulSyllableSse',
u'\u8c86': 'CjkUnifiedIdeograph8c86',
u'\u4988': 'CjkUnifiedIdeograph4988',
u'\uf50b': 'UnicodeCharacter0xf50b',
u'\u0a8a': 'GujaratiLetterUu',
u'\ub60d': 'HangulSyllableDdyet',
u'\u730f': 'CjkUnifiedIdeograph730f',
u'\u3c11': 'CjkUnifiedIdeograph3c11',
u'\udb94': 'UnicodeCharacter0xdb94',
u'\ua496': 'YiRadicalGgop',
u'\u6198': 'CjkUnifiedIdeograph6198',
u'\u229a': 'CircledRingOperator',
u'\uce1d': 'HangulSyllableCyut',
u'\u8b1f': 'CjkUnifiedIdeograph8b1f',
u'\u5421': 'CjkUnifiedIdeograph5421',
u'\u1123': 'HangulChoseongPieupsiostikeut',
u'\uf3a4': 'UnicodeCharacter0xf3a4',
u'\ubca6': 'HangulSyllableBenh',
u'\u79a8': 'CjkUnifiedIdeograph79a8',
u'\u3aaa': 'CjkUnifiedIdeograph3aaa',
u'\ue62d': 'UnicodeCharacter0xe62d',
u'\u07ac': 'ThaanaEbefili',
u'\ua32f': 'YiSyllableSsie',
u'\u6c31': 'CjkUnifiedIdeograph6c31',
u'\u2933': 'WaveArrowPointingDirectlyRight',
u'\ud4b6': 'HangulSyllablePwenh',
u'\u91b8': 'CjkUnifiedIdeograph91b8',
u'\u52ba': 'CjkUnifiedIdeograph52ba',
u'\ufe3d': 'PresentationFormForVerticalLeftDoubleAngleBracket',
u'\u1fbc': 'GreekCapitalLetterAlphaWithProsgegrammeni',
u'\ubb3f': 'HangulSyllableMulb',
u'\u8441': 'CjkUnifiedIdeograph8441',
u'\u4143': 'CjkUnifiedIdeograph4143',
u'\u0245': 'LatinCapitalLetterTurnedV',
u'\uecc6': 'UnicodeCharacter0xecc6',
u'\ua9c8': 'UnicodeCharacter0xa9c8',
u'\u6aca': 'CjkUnifiedIdeograph6aca',
u'\u37cc': 'CjkUnifiedIdeograph37cc',
u'\ud34f': 'HangulSyllablePyalb',
u'\u9c51': 'CjkUnifiedIdeograph9c51',
u'\u5953': 'CjkUnifiedIdeograph5953',
u'\u1a55': 'UnicodeCharacter0x1a55',
u'\uc1d8': 'HangulSyllableSwaess',
u'\u82da': 'CjkUnifiedIdeograph82da',
u'\u4fdc': 'CjkUnifiedIdeograph4fdc',
u'\ueb5f': 'UnicodeCharacter0xeb5f',
u'\u08de': 'UnicodeCharacter0x8de',
u'\ub461': 'HangulSyllableDub',
u'\u7163': 'CjkUnifiedIdeograph7163',
u'\u3265': 'CircledHangulPieup',
u'\ud9e8': 'UnicodeCharacter0xd9e8',
u'\u9aea': 'CjkUnifiedIdeograph9aea',
u'\u67ec': 'CjkUnifiedIdeograph67ec',
u'\u20ee': 'CombiningLeftArrowBelow',
u'\ucc71': 'HangulSyllableCyab',
u'\u8973': 'CjkUnifiedIdeograph8973',
u'\u4a75': 'CjkUnifiedIdeograph4a75',
u'\u1777': 'UnicodeCharacter0x1777',
u'\uf1f8': 'UnicodeCharacter0xf1f8',
u'\ub2fa': 'HangulSyllableDaj',
u'\u7ffc': 'CjkUnifiedIdeograph7ffc',
u'\u38fe': 'CjkUnifiedIdeograph38fe',
u'\u0600': 'ArabicNumberSign',
u'\u2283': 'SupersetOf',
u'\uc502': 'HangulSyllableSseubs',
u'\ue185': 'UnicodeCharacter0xe185',
u'\u4f06': 'CjkUnifiedIdeograph4f06',
u'\u6b89': 'CjkUnifiedIdeograph6b89',
u'\u940c': 'CjkUnifiedIdeograph940c',
u'\ub08f': 'HangulSyllableGgis',
u'\u1e10': 'LatinCapitalLetterDWithCedilla',
u'\u3a93': 'CjkUnifiedIdeograph3a93',
u'\udd12': 'UnicodeCharacter0xdd12',
u'\uf995': 'CjkCompatibilityIdeographf995',
u'\u6716': 'CjkUnifiedIdeograph6716',
u'\u8399': 'CjkUnifiedIdeograph8399',
u'\u0d9d': 'SinhalaLetterMahaapraanaGayanna',
u'\uac1c': 'HangulSyllableGae',
u'\uc89f': 'HangulSyllableJwas',
u'\u3620': 'CjkUnifiedIdeograph3620',
u'\u52a3': 'CjkUnifiedIdeograph52a3',
u'\uf522': 'UnicodeCharacter0xf522',
u'\u7f26': 'CjkUnifiedIdeograph7f26',
u'\u9ba9': 'CjkUnifiedIdeograph9ba9',
u'\u25ad': 'WhiteRectangle',
u'\uc42c': 'HangulSyllableSsoe',
u'\ue0af': 'UnicodeCharacter0xe0af',
u'\u4e30': 'CjkUnifiedIdeograph4e30',
u'\u6ab3': 'CjkUnifiedIdeograph6ab3',
u'\u9736': 'CjkUnifiedIdeograph9736',
u'\ub3b9': 'HangulSyllableDyeb',
u'\u113a': 'HangulChoseongSiosphieuph',
u'\u3dbd': 'CjkUnifiedIdeograph3dbd',
u'\udc3c': 'UnicodeCharacter0xdc3c',
u'\uf8bf': 'UnicodeCharacter0xf8bf',
u'\u6640': 'CjkUnifiedIdeograph6640',
u'\u82c3': 'CjkUnifiedIdeograph82c3',
u'\u0cc7': 'KannadaVowelSignEe',
u'\uaf46': 'HangulSyllableGgop',
u'\ucbc9': 'HangulSyllableJjyub',
u'\u294a': 'LeftBarbUpRightBarbDownHarpoon',
u'\u55cd': 'CjkUnifiedIdeograph55cd',
u'\uf44c': 'UnicodeCharacter0xf44c',
u'\u7e50': 'CjkUnifiedIdeograph7e50',
u'\u9ad3': 'CjkUnifiedIdeograph9ad3',
u'\u24d7': 'CircledLatinSmallLetterH',
u'\uc756': 'HangulSyllableEup',
u'\ue3d9': 'UnicodeCharacter0xe3d9',
u'\u415a': 'CjkUnifiedIdeograph415a',
u'\u6ddd': 'CjkUnifiedIdeograph6ddd',
u'\u9660': 'CjkUnifiedIdeograph9660',
u'\ub2e3': 'HangulSyllableNih',
u'\u1064': 'MyanmarToneMarkSgawKarenKePho',
u'\u3ce7': 'CjkUnifiedIdeograph3ce7',
u'\udf66': 'UnicodeCharacter0xdf66',
u'\ufbe9': 'ArabicLetterUighurKazakhKirghizAlefMaksuraMedialForm',
u'\u596a': 'CjkUnifiedIdeograph596a',
u'\u85ed': 'CjkUnifiedIdeograph85ed',
u'\u0ff1': 'UnicodeCharacter0xff1',
u'\uae70': 'HangulSyllableGgael',
u'\ucaf3': 'HangulSyllableJjwah',
u'\u2874': 'BraillePatternDots3567',
u'\u54f7': 'CjkUnifiedIdeograph54f7',
u'\uf776': 'UnicodeCharacter0xf776',
u'\u717a': 'CjkUnifiedIdeograph717a',
u'\u9dfd': 'CjkUnifiedIdeograph9dfd',
u'\ua701': 'ModifierLetterChineseToneYangPing',
u'\uc680': 'HangulSyllableOel',
u'\u2105': 'CareOf',
u'\u4084': 'CjkUnifiedIdeograph4084',
u'\uec07': 'UnicodeCharacter0xec07',
u'\u760b': 'CjkUnifiedIdeograph760b',
u'\u898a': 'CjkUnifiedIdeograph898a',
u'\u138e': 'EthiopicSyllablePwee',
u'\ubf11': 'HangulSyllableBbyeolg',
u'\ude90': 'UnicodeCharacter0xde90',
u'\u3915': 'CjkUnifiedIdeograph3915',
u'\u5894': 'CjkUnifiedIdeograph5894',
u'\u8e1b': 'CjkUnifiedIdeograph8e1b',
u'\ua19a': 'YiSyllableHlip',
u'\u081f': 'UnicodeCharacter0x81f',
u'\u2b9e': 'UnicodeCharacter0x2b9e',
u'\ud721': 'HangulSyllableHwilg',
u'\uf6a0': 'UnicodeCharacter0xf6a0',
u'\u5125': 'CjkUnifiedIdeograph5125',
u'\u70a4': 'CjkUnifiedIdeograph70a4',
u'\ua62b': 'VaiSyllableNdoleDo',
u'\ub9aa': 'HangulSyllableRyip',
u'\u202f': 'NarrowNobreakSpace',
u'\u43ae': 'CjkUnifiedIdeograph43ae',
u'\uef31': 'UnicodeCharacter0xef31',
u'\u6935': 'CjkUnifiedIdeograph6935',
u'\u88b4': 'CjkUnifiedIdeograph88b4',
u'\u12b8': 'EthiopicSyllableKxa',
u'\ube3b': 'HangulSyllableByis',
u'\ud1ba': 'HangulSyllableTop',
u'\u383f': 'CjkUnifiedIdeograph383f',
u'\u5bbe': 'CjkUnifiedIdeograph5bbe',
u'\u8145': 'CjkUnifiedIdeograph8145',
u'\ua0c4': 'YiSyllableMux',
u'\u0b49': 'UnicodeCharacter0xb49',
u'\u2ac8': 'SupersetOfAboveTildeOperator',
u'\ud64b': 'HangulSyllableHos',
u'\ue9ca': 'UnicodeCharacter0xe9ca',
u'\u504f': 'CjkUnifiedIdeograph504f',
u'\u73ce': 'CjkUnifiedIdeograph73ce',
u'\u9955': 'CjkUnifiedIdeograph9955',
u'\ub8d4': 'HangulSyllableRyol',
u'\u2359': 'AplFunctionalSymbolDeltaUnderbar',
u'\u42d8': 'CjkUnifiedIdeograph42d8',
u'\uee5b': 'UnicodeCharacter0xee5b',
u'\u685f': 'CjkUnifiedIdeograph685f',
u'\u8bde': 'CjkUnifiedIdeograph8bde',
u'\u15e2': 'CanadianSyllabicsCarrierTtu',
u'\ub165': 'HangulSyllableNyelg',
u'\ud0e4': 'HangulSyllableTael',
u'\u3b69': 'CjkUnifiedIdeograph3b69',
u'\u5ae8': 'CjkUnifiedIdeograph5ae8',
u'\u806f': 'CjkUnifiedIdeograph806f',
u'\ua3ee': 'YiSyllableJo',
u'\u0a73': 'GurmukhiUra',
u'\u2df2': 'CombiningCyrillicLetterSha',
u'\uc975': 'HangulSyllableJyulg',
u'\ue8f4': 'UnicodeCharacter0xe8f4',
u'\u5379': 'CjkUnifiedIdeograph5379',
u'\u72f8': 'CjkUnifiedIdeograph72f8',
u'\u987f': 'CjkUnifiedIdeograph987f',
u'\ubbfe': 'HangulSyllableMinh',
u'\uce06': 'HangulSyllableCyugg',
u'\uea89': 'UnicodeCharacter0xea89',
u'\u8b08': 'CjkUnifiedIdeograph8b08',
u'\ub78b': 'HangulSyllableRalh',
u'\u480a': 'CjkUnifiedIdeograph480a',
u'\u748d': 'CjkUnifiedIdeograph748d',
u'\u150c': 'CanadianSyllabicsNaskapiSpwa',
u'\u318f': 'UnicodeCharacter0x318f',
u'\ue616': 'UnicodeCharacter0xe616',
u'\ua318': 'YiSyllableSox',
u'\ucf9b': 'HangulSyllableKwaelh',
u'\u601a': 'CjkUnifiedIdeograph601a',
u'\u8c9d': 'CjkUnifiedIdeograph8c9d',
u'\u2d1c': 'GeorgianSmallLetterCil',
u'\u499f': 'CjkUnifiedIdeograph499f',
u'\u16a1': 'RunicLetterV',
u'\ufe26': 'CombiningConjoiningMacron',
u'\ubb28': 'HangulSyllableMyom',
u'\ue7ab': 'UnicodeCharacter0xe7ab',
u'\u782a': 'CjkUnifiedIdeograph782a',
u'\ua4ad': 'YiRadicalHmo',
u'\u452c': 'CjkUnifiedIdeograph452c',
u'\u61af': 'CjkUnifiedIdeograph61af',
u'\u022e': 'LatinCapitalLetterOWithDotAbove',
u'\u2eb1': 'CjkRadicalNetOne',
u'\ud338': 'HangulSyllablePaem',
u'\uffbb': 'HalfwidthHangulLetterKhieukh',
u'\u903a': 'CjkUnifiedIdeograph903a',
u'\ubcbd': 'HangulSyllableByeog',
u'\u5d3c': 'CjkUnifiedIdeograph5d3c',
u'\u79bf': 'CjkUnifiedIdeograph79bf',
u'\u1a3e': 'UnicodeCharacter0x1a3e',
u'\u46c1': 'CjkUnifiedIdeograph46c1',
u'\u03c3': 'GreekSmallLetterSigma',
u'\ueb48': 'UnicodeCharacter0xeb48',
u'\ua84a': 'PhagspaLetterDa',
u'\ud4cd': 'HangulSyllablePwig',
u'\u754c': 'CjkUnifiedIdeograph754c',
u'\u91cf': 'CjkUnifiedIdeograph91cf',
u'\u324e': 'UnicodeCharacter0x324e',
u'\u5ed1': 'CjkUnifiedIdeograph5ed1',
u'\u1bd3': 'UnicodeCharacter0x1bd3',
u'\uc05a': 'HangulSyllableBbeugg',
u'\uecdd': 'UnicodeCharacter0xecdd',
u'\u8d5c': 'CjkUnifiedIdeograph8d5c',
u'\ua9df': 'UnicodeCharacter0xa9df',
u'\u4a5e': 'CjkUnifiedIdeograph4a5e',
u'\u76e1': 'CjkUnifiedIdeograph76e1',
u'\u1760': 'TagbanwaLetterA',
u'\u33e3': 'IdeographicTelegraphSymbolForDayFour',
u'\ud86a': 'UnicodeCharacter0xd86a',
u'\ua56c': 'VaiSyllableNgga',
u'\uc1ef': 'HangulSyllableSoelh',
u'\u626e': 'CjkUnifiedIdeograph626e',
u'\u8ef1': 'CjkUnifiedIdeograph8ef1',
u'\u2f70': 'KangxiRadicalSpirit',
u'\u4bf3': 'CjkUnifiedIdeograph4bf3',
u'\u08f5': 'UnicodeCharacter0x8f5',
u'\uf07a': 'UnicodeCharacter0xf07a',
u'\ubd7c': 'HangulSyllableByok',
u'\ud9ff': 'UnicodeCharacter0xd9ff',
u'\u7a7e': 'CjkUnifiedIdeograph7a7e',
u'\u4780': 'CjkUnifiedIdeograph4780',
u'\ue303': 'UnicodeCharacter0xe303',
u'\u0482': 'CyrillicThousandsSign',
u'\ua005': 'YiSyllableIex',
u'\u6d07': 'CjkUnifiedIdeograph6d07',
u'\u2a09': 'NaryTimesOperator',
u'\ud58c': 'HangulSyllableHaek',
u'\u928e': 'CjkUnifiedIdeograph928e',
u'\u5f90': 'CjkUnifiedIdeograph5f90',
u'\ufb13': 'ArmenianSmallLigatureMenNow',
u'\u1c92': 'UnicodeCharacter0x1c92',
u'\ub815': 'HangulSyllableRelt',
u'\u8517': 'CjkUnifiedIdeograph8517',
u'\u4219': 'CjkUnifiedIdeograph4219',
u'\u0f1b': 'TibetanSignRdelDkarGnyis',
u'\ued9c': 'UnicodeCharacter0xed9c',
u'\uaa9e': 'UnicodeCharacter0xaa9e',
u'\u77a0': 'CjkUnifiedIdeograph77a0',
u'\u34a2': 'CjkUnifiedIdeograph34a2',
u'\ud025': 'HangulSyllableKwelt',
u'\u9d27': 'CjkUnifiedIdeograph9d27',
u'\u5a29': 'CjkUnifiedIdeograph5a29',
u'\u272b': 'OpenCentreBlackStar',
u'\uc2ae': 'HangulSyllableSeulm',
u'\u8fb0': 'CjkUnifiedIdeograph8fb0',
u'\u4cb2': 'CjkUnifiedIdeograph4cb2',
u'\ue835': 'UnicodeCharacter0xe835',
u'\u09b4': 'UnicodeCharacter0x9b4',
u'\ub537': 'HangulSyllableDdad',
u'\u7239': 'CjkUnifiedIdeograph7239',
u'\u3f3b': 'CjkUnifiedIdeograph3f3b',
u'\udabe': 'UnicodeCharacter0xdabe',
u'\ua7c0': 'UnicodeCharacter0xa7c0',
u'\u64c2': 'CjkUnifiedIdeograph64c2',
u'\u21c4': 'RightwardsArrowOverLeftwardsArrow',
u'\ucd47': 'HangulSyllableCwaed',
u'\u8a49': 'CjkUnifiedIdeograph8a49',
u'\u574b': 'CjkUnifiedIdeograph574b',
u'\u144d': 'CanadianSyllabicsTaai',
u'\uf2ce': 'UnicodeCharacter0xf2ce',
u'\ubfd0': 'HangulSyllableBbun',
u'\u7cd2': 'CjkUnifiedIdeograph7cd2',
u'\u39d4': 'CjkUnifiedIdeograph39d4',
u'\ue557': 'UnicodeCharacter0xe557',
u'\u06d6': 'ArabicSmallHighLigatureSadWithLamWithAlefMaksura',
u'\ua259': 'YiSyllableHxop',
u'\u6f5b': 'CjkUnifiedIdeograph6f5b',
u'\u2c5d': 'GlagoliticSmallLetterTrokutastiA',
u'\ud7e0': 'UnicodeCharacter0xd7e0',
u'\u94e2': 'CjkUnifiedIdeograph94e2',
u'\u51e4': 'CjkUnifiedIdeograph51e4',
u'\ufd67': 'ArabicLigatureSheenWithHahWithMeemFinalForm',
u'\u1ee6': 'LatinCapitalLetterUWithHookAbove',
u'\uba69': 'HangulSyllableMeng',
u'\u876b': 'CjkUnifiedIdeograph876b',
u'\u446d': 'CjkUnifiedIdeograph446d',
u'\u016f': 'LatinSmallLetterUWithRingAbove',
u'\ueff0': 'UnicodeCharacter0xeff0',
u'\uacf2': 'HangulSyllableGobs',
u'\u69f4': 'CjkUnifiedIdeograph69f4',
u'\u36f6': 'CjkUnifiedIdeograph36f6',
u'\ud279': 'HangulSyllableTweng',
u'\u9f7b': 'CjkUnifiedIdeograph9f7b',
u'\u5c7d': 'CjkUnifiedIdeograph5c7d',
u'\u197f': 'UnicodeCharacter0x197f',
u'\u6981': 'CjkUnifiedIdeograph6981',
u'\u8e04': 'CjkUnifiedIdeograph8e04',
u'\ua287': 'YiSyllableWox',
u'\u0408': 'CyrillicCapitalLetterJe',
u'\u388b': 'CjkUnifiedIdeograph388b',
u'\uc70a': 'HangulSyllableWinh',
u'\ufb8d': 'ArabicLetterRrehFinalForm',
u'\u5d0e': 'CjkUnifiedIdeograph5d0e',
u'\u7191': 'CjkUnifiedIdeograph7191',
u'\u9614': 'CjkUnifiedIdeograph9614',
u'\uca97': 'HangulSyllableJjyeos',
u'\u2c18': 'GlagoliticCapitalLetterHeru',
u'\u409b': 'CjkUnifiedIdeograph409b',
u'\uef1a': 'UnicodeCharacter0xef1a',
u'\u651e': 'CjkUnifiedIdeograph651e',
u'\u99a1': 'CjkUnifiedIdeograph99a1',
u'\u1fa5': 'GreekSmallLetterOmegaWithDasiaAndOxiaAndYpogegrammeni',
u'\ube24': 'HangulSyllableBeuk',
u'\ud2a7': 'HangulSyllableTyulb',
u'\u3428': 'CjkUnifiedIdeograph3428',
u'\u68ab': 'CjkUnifiedIdeograph68ab',
u'\uf72a': 'UnicodeCharacter0xf72a',
u'\u8d2e': 'CjkUnifiedIdeograph8d2e',
u'\ua1b1': 'YiSyllableHlyt',
u'\u0332': 'CombiningLowLine',
u'\u27b5': 'BlackfeatheredRightwardsArrow',
u'\uc634': 'HangulSyllableOm',
u'\ufab7': 'CjkCompatibilityIdeographfab7',
u'\u5c38': 'CjkUnifiedIdeograph5c38',
u'\u70bb': 'CjkUnifiedIdeograph70bb',
u'\u953e': 'CjkUnifiedIdeograph953e',
u'\uc9c1': 'HangulSyllableJig',
u'\u2b42': 'LeftwardsArrowAboveReverseAlmostEqualTo',
u'\u4fc5': 'CjkUnifiedIdeograph4fc5',
u'\uee44': 'UnicodeCharacter0xee44',
u'\u6448': 'CjkUnifiedIdeograph6448',
u'\u98cb': 'CjkUnifiedIdeograph98cb',
u'\u1ecf': 'LatinSmallLetterOWithHookAbove',
u'\ubd4e': 'HangulSyllableBoenh',
u'\ud1d1': 'HangulSyllableTwang',
u'\u3352': 'SquareRira',
u'\u57d5': 'CjkUnifiedIdeograph57d5',
u'\uf654': 'UnicodeCharacter0xf654',
u'\u8c58': 'CjkUnifiedIdeograph8c58',
u'\ua0db': 'YiSyllableFup',
u'\u025c': 'LatinSmallLetterReversedOpenE',
u'\u26df': 'UnicodeCharacter0x26df',
u'\uc55e': 'HangulSyllableAp',
u'\uf9e1': 'CjkCompatibilityIdeographf9e1',
u'\u5b62': 'CjkUnifiedIdeograph5b62',
u'\u7fe5': 'CjkUnifiedIdeograph7fe5',
u'\u9468': 'CjkUnifiedIdeograph9468',
u'\uc8eb': 'HangulSyllableJyolb',
u'\u2a6c': 'SimilarMinusSimilar',
u'\u4eef': 'CjkUnifiedIdeograph4eef',
u'\ued6e': 'UnicodeCharacter0xed6e',
u'\u6372': 'CjkUnifiedIdeograph6372',
u'\u87f5': 'CjkUnifiedIdeograph87f5',
u'\u1df9': 'UnicodeCharacter0x1df9',
u'\ubc78': 'HangulSyllableByaem',
u'\ud0fb': 'HangulSyllableTyags',
u'\u327c': 'CircledKoreanCharacterChamko',
u'\u56ff': 'CjkUnifiedIdeograph56ff',
u'\uf57e': 'UnicodeCharacter0xf57e',
u'\u6c03': 'CjkUnifiedIdeograph6c03',
u'\u8b82': 'CjkUnifiedIdeograph8b82',
u'\u0186': 'LatinCapitalLetterOpenO',
u'\ua509': 'VaiSyllableKpee',
u'\uc488': 'HangulSyllableSsweol',
u'\u3b0d': 'CjkUnifiedIdeograph3b0d',
u'\u5a8c': 'CjkUnifiedIdeograph5a8c',
u'\ufe0f': 'VariationSelector16',
u'\u7413': 'CjkUnifiedIdeograph7413',
u'\u9392': 'CjkUnifiedIdeograph9392',
u'\u0a17': 'GurmukhiLetterGa',
u'\u2996': 'DoubleRightArcLessthanBracket',
u'\ucd19': 'HangulSyllableCob',
u'\uec98': 'UnicodeCharacter0xec98',
u'\u431d': 'CjkUnifiedIdeograph431d',
u'\u629c': 'CjkUnifiedIdeograph629c',
u'\u9c23': 'CjkUnifiedIdeograph9c23',
u'\ubba2': 'HangulSyllableMwip',
u'\u1227': 'EthiopicSyllableSzwa',
u'\u31a6': 'BopomofoLetterOo',
u'\ud529': 'HangulSyllablePyilg',
u'\uf4a8': 'UnicodeCharacter0xf4a8',
u'\u6b2d': 'CjkUnifiedIdeograph6b2d',
u'\u8aac': 'CjkUnifiedIdeograph8aac',
u'\xb0': 'DegreeSign',
u'\ua433': 'YiSyllableNji',
u'\uc3b2': 'HangulSyllableSsyeobs',
u'\u3a37': 'CjkUnifiedIdeograph3a37',
u'\u59b6': 'CjkUnifiedIdeograph59b6',
u'\ufd39': 'ArabicLigatureSheenWithKhahMedialForm',
u'\u733d': 'CjkUnifiedIdeograph733d',
u'\u92bc': 'CjkUnifiedIdeograph92bc',
u'\u0941': 'DevanagariVowelSignU',
u'\u28c0': 'BraillePatternDots78',
u'\ucc43': 'HangulSyllableCah',
u'\uebc2': 'UnicodeCharacter0xebc2',
u'\u4247': 'CjkUnifiedIdeograph4247',
u'\u61c6': 'CjkUnifiedIdeograph61c6',
u'\u9b4d': 'CjkUnifiedIdeograph9b4d',
u'\ubacc': 'HangulSyllableMwal',
u'\u1151': 'HangulChoseongCeongchieumssangcieuc',
u'\u30d0': 'KatakanaLetterBa',
u'\ud453': 'HangulSyllablePoes',
u'\uf3d2': 'UnicodeCharacter0xf3d2',
u'\u6a57': 'CjkUnifiedIdeograph6a57',
u'\u89d6': 'CjkUnifiedIdeograph89d6',
u'\u0fda': 'UnicodeCharacter0xfda',
u'\ua35d': 'YiSyllableZhy',
u'\uc2dc': 'HangulSyllableSi',
u'\u3961': 'CjkUnifiedIdeograph3961',
u'\u58e0': 'CjkUnifiedIdeograph58e0',
u'\ufc63': 'ArabicLigatureShaddaWithSuperscriptAlefIsolatedForm',
u'\u7267': 'CjkUnifiedIdeograph7267',
u'\u91e6': 'CjkUnifiedIdeograph91e6',
u'\u086b': 'UnicodeCharacter0x86b',
u'\u17ea': 'UnicodeCharacter0x17ea',
u'\ucb6d': 'HangulSyllableJjweolg',
u'\ueaec': 'UnicodeCharacter0xeaec',
u'\u4171': 'CjkUnifiedIdeograph4171',
u'\u60f0': 'CjkUnifiedIdeograph60f0',
u'\u9a77': 'CjkUnifiedIdeograph9a77',
u'\ub9f6': 'HangulSyllableMaebs',
u'\u107b': 'MyanmarLetterShanDa',
u'\u3ffa': 'CjkUnifiedIdeograph3ffa',
u'\ud37d': 'HangulSyllablePeog',
u'\uf2fc': 'UnicodeCharacter0xf2fc',
u'\ue881': 'UnicodeCharacter0xe881',
u'\u8900': 'CjkUnifiedIdeograph8900',
u'\uad83': 'HangulSyllableGuc',
u'\u4a02': 'CjkUnifiedIdeograph4a02',
u'\u6e85': 'CjkUnifiedIdeograph6e85',
u'\u0f04': 'TibetanMarkInitialYigMgoMdunMa',
u'\u2387': 'AlternativeKeySymbol',
u'\udc0e': 'UnicodeCharacter0xdc0e',
u'\uf091': 'UnicodeCharacter0xf091',
u'\u9110': 'CjkUnifiedIdeograph9110',
u'\ub593': 'HangulSyllableDdyaelh',
u'\u5212': 'CjkUnifiedIdeograph5212',
u'\u7695': 'CjkUnifiedIdeograph7695',
u'\u1714': 'TagalogSignVirama',
u'\u4b97': 'CjkUnifiedIdeograph4b97',
u'\u0c99': 'KannadaLetterNga',
u'\ue41e': 'UnicodeCharacter0xe41e',
u'\ub920': 'HangulSyllableRwe',
u'\udda3': 'UnicodeCharacter0xdda3',
u'\u7a22': 'CjkUnifiedIdeograph7a22',
u'\u9ea5': 'CjkUnifiedIdeograph9ea5',
u'\u3f24': 'CjkUnifiedIdeograph3f24',
u'\u53a7': 'CjkUnifiedIdeograph53a7',
u'\u14a9': 'CanadianSyllabicsYcreeMoo',
u'\uc130': 'HangulSyllableSeoss',
u'\ue5b3': 'UnicodeCharacter0xe5b3',
u'\u8232': 'CjkUnifiedIdeograph8232',
u'\ua6b5': 'UnicodeCharacter0xa6b5',
u'\u4734': 'CjkUnifiedIdeograph4734',
u'\u7bb7': 'CjkUnifiedIdeograph7bb7',
u'\u1836': 'MongolianLetterYa',
u'\u3cb9': 'CjkUnifiedIdeograph3cb9',
u'\ue940': 'UnicodeCharacter0xe940',
u'\uaa42': 'ChamLetterFinalNg',
u'\ucec5': 'HangulSyllableKyat',
u'\u6f44': 'CjkUnifiedIdeograph6f44',
u'\u83c7': 'CjkUnifiedIdeograph83c7',
u'\u2046': 'RightSquareBracketWithQuill',
u'\u44c9': 'CjkUnifiedIdeograph44c9',
u'\u19cb': 'UnicodeCharacter0x19cb',
u'\uf150': 'UnicodeCharacter0xf150',
u'\ub252': 'HangulSyllableNwej',
u'\ud6d5': 'HangulSyllableHub',
u'\u7754': 'CjkUnifiedIdeograph7754',
u'\uabd7': 'UnicodeCharacter0xabd7',
u'\u4856': 'CjkUnifiedIdeograph4856',
u'\u6cd9': 'CjkUnifiedIdeograph6cd9',
u'\u0d58': 'UnicodeCharacter0xd58',
u'\u21db': 'RightwardsTripleArrow',
u'\uda62': 'UnicodeCharacter0xda62',
u'\ufee5': 'ArabicLetterNoonIsolatedForm',
u'\u9f64': 'CjkUnifiedIdeograph9f64',
u'\ub3e7': 'HangulSyllableDwad',
u'\u5066': 'CjkUnifiedIdeograph5066',
u'\u74e9': 'CjkUnifiedIdeograph74e9',
u'\u1568': 'CanadianSyllabicsThwaa',
u'\u49eb': 'CjkUnifiedIdeograph49eb',
u'\u0aed': 'GujaratiDigitSeven',
u'\ue272': 'UnicodeCharacter0xe272',
u'\ua774': 'LatinSmallLetterNum',
u'\udbf7': 'UnicodeCharacter0xdbf7',
u'\u7876': 'CjkUnifiedIdeograph7876',
u'\u9cf9': 'CjkUnifiedIdeograph9cf9',
u'\u3d78': 'CjkUnifiedIdeograph3d78',
u'\u51fb': 'CjkUnifiedIdeograph51fb',
u'\u12fd': 'EthiopicSyllableDde',
u'\u2801': 'BraillePatternDots1',
u'\ucf84': 'HangulSyllableKwass',
u'\u8086': 'CjkUnifiedIdeograph8086',
u'\u4588': 'CjkUnifiedIdeograph4588',
u'\uf90b': 'CjkCompatibilityIdeographf90b',
u'\u068a': 'ArabicLetterDalWithDotBelow',
u'\uba0d': 'HangulSyllableMyalt',
u'\u7f0f': 'CjkUnifiedIdeograph7f0f',
u'\u3011': 'RightBlackLenticularBracket',
u'\ud794': 'HangulSyllableHils',
u'\ua896': 'SaurashtraLetterNga',
u'\u6d98': 'CjkUnifiedIdeograph6d98',
u'\u2e9a': 'UnicodeCharacter0x2e9a',
u'\uc21d': 'HangulSyllableSunj',
u'\u871f': 'CjkUnifiedIdeograph871f',
u'\u5821': 'CjkUnifiedIdeograph5821',
u'\u1d23': 'LatinLetterSmallCapitalEzh',
u'\uffa4': 'HalfwidthHangulLetterNieun',
u'\ub0a6': 'HangulSyllableNalp',
u'\u75a8': 'CjkUnifiedIdeograph75a8',
u'\u36aa': 'CjkUnifiedIdeograph36aa',
u'\uea2d': 'UnicodeCharacter0xea2d',
u'\u0bac': 'UnicodeCharacter0xbac',
u'\uaf2f': 'HangulSyllableGgogs',
u'\u6031': 'CjkUnifiedIdeograph6031',
u'\u2533': 'BoxDrawingsHeavyDownAndHorizontal',
u'\ud8b6': 'UnicodeCharacter0xd8b6',
u'\u9db8': 'CjkUnifiedIdeograph9db8',
u'\u5eba': 'CjkUnifiedIdeograph5eba',
u'\uf23d': 'UnicodeCharacter0xf23d',
u'\u13bc': 'CherokeeLetterMo',
u'\ub73f': 'HangulSyllableDdeuc',
u'\u8841': 'CjkUnifiedIdeograph8841',
u'\u4d43': 'CjkUnifiedIdeograph4d43',
u'\u0e45': 'ThaiCharacterLakkhangyao',
u'\ue0c6': 'UnicodeCharacter0xe0c6',
u'\ua5c8': 'VaiSyllableGbon',
u'\u66ca': 'CjkUnifiedIdeograph66ca',
u'\u3bcc': 'CjkUnifiedIdeograph3bcc',
u'\udf4f': 'UnicodeCharacter0xdf4f',
u'\u9051': 'CjkUnifiedIdeograph9051',
u'\u5553': 'CjkUnifiedIdeograph5553',
u'\u1655': 'CanadianSyllabicsCarrierSho',
u'\ucdd8': 'HangulSyllableCwels',
u'\u8eda': 'CjkUnifiedIdeograph8eda',
u'\u43dc': 'CjkUnifiedIdeograph43dc',
u'\ue75f': 'UnicodeCharacter0xe75f',
u'\u04de': 'CyrillicCapitalLetterZeWithDiaeresis',
u'\ub861': 'HangulSyllableRonj',
u'\u7d63': 'CjkUnifiedIdeograph7d63',
u'\u3e65': 'CjkUnifiedIdeograph3e65',
u'\ud5e8': 'HangulSyllableHen',
u'\u96ea': 'CjkUnifiedIdeograph96ea',
u'\u6bec': 'CjkUnifiedIdeograph6bec',
u'\u2cee': 'UnicodeCharacter0x2cee',
u'\uc071': 'HangulSyllableBbeut',
u'\u8573': 'CjkUnifiedIdeograph8573',
u'\u4675': 'CjkUnifiedIdeograph4675',
u'\u1b77': 'BalineseMusicalSymbolRighthandClosedTak',
u'\ufdf8': 'ArabicLigatureWasallamIsolatedForm',
u'\ubefa': 'HangulSyllableBbelp',
u'\u73fc': 'CjkUnifiedIdeograph73fc',
u'\u34fe': 'CjkUnifiedIdeograph34fe',
u'\u0a00': 'UnicodeCharacter0xa00',
u'\u2e83': 'CjkRadicalSecondTwo',
u'\uc902': 'HangulSyllableJunh',
u'\ued85': 'UnicodeCharacter0xed85',
u'\u4306': 'CjkUnifiedIdeograph4306',
u'\u6789': 'CjkUnifiedIdeograph6789',
u'\u980c': 'CjkUnifiedIdeograph980c',
u'\ubc8f': 'HangulSyllableBeolb',
u'\u1210': 'EthiopicSyllableHha',
u'\u3693': 'CjkUnifiedIdeograph3693',
u'\ud112': 'HangulSyllableTyap',
u'\uf595': 'UnicodeCharacter0xf595',
u'\u6b16': 'CjkUnifiedIdeograph6b16',
u'\u8f99': 'CjkUnifiedIdeograph8f99',
u'\u019d': 'LatinCapitalLetterNWithLeftHook',
u'\ua01c': 'YiSyllableBie',
u'\uc49f': 'HangulSyllableSswegs',
u'\u3a20': 'CjkUnifiedIdeograph3a20',
u'\u5ea3': 'CjkUnifiedIdeograph5ea3',
u'\uf922': 'CjkCompatibilityIdeographf922',
u'\u7326': 'CjkUnifiedIdeograph7326',
u'\u97a9': 'CjkUnifiedIdeograph97a9',
u'\u29ad': 'MeasuredAngleWithOpenArmEndingInArrowPointingLeftAndUp',
u'\uc82c': 'HangulSyllableJem',
u'\uecaf': 'UnicodeCharacter0xecaf',
u'\u4230': 'CjkUnifiedIdeograph4230',
u'\u66b3': 'CjkUnifiedIdeograph66b3',
u'\u9b36': 'CjkUnifiedIdeograph9b36',
u'\ubfb9': 'HangulSyllableBbyolg',
u'\u1d3a': 'ModifierLetterCapitalN',
u'\u31bd': 'UnicodeCharacter0x31bd',
u'\ud03c': 'HangulSyllableKwil',
u'\uf4bf': 'UnicodeCharacter0xf4bf',
u'\u6a40': 'CjkUnifiedIdeograph6a40',
u'\u8ec3': 'CjkUnifiedIdeograph8ec3',
u'\xc7': 'LatinCapitalLetterCWithCedilla',
u'\ua346': 'YiSyllableZhat',
u'\uc7c9': 'HangulSyllableJyag',
u'\u254a': 'BoxDrawingsLeftLightAndRightVerticalHeavy',
u'\u59cd': 'CjkUnifiedIdeograph59cd',
u'\uf84c': 'UnicodeCharacter0xf84c',
u'\u7250': 'CjkUnifiedIdeograph7250',
u'\u96d3': 'CjkUnifiedIdeograph96d3',
u'\u28d7': 'BraillePatternDots123578',
u'\ucb56': 'HangulSyllableJjulp',
u'\uefd9': 'UnicodeCharacter0xefd9',
u'\u4d5a': 'CjkUnifiedIdeograph4d5a',
u'\u61dd': 'CjkUnifiedIdeograph61dd',
u'\u9a60': 'CjkUnifiedIdeograph9a60',
u'\ubee3': 'HangulSyllableBbeos',
u'\u1c64': 'OlChikiLetterLi',
u'\u30e7': 'KatakanaLetterSmallYo',
u'\ud366': 'HangulSyllablePyaenh',
u'\uf7e9': 'UnicodeCharacter0xf7e9',
u'\u556a': 'CjkUnifiedIdeograph556a',
u'\u89ed': 'CjkUnifiedIdeograph89ed',
u'\u03f1': 'GreekRhoSymbol',
u'\ua270': 'YiSyllableHie',
u'\uc6f3': 'HangulSyllableWelb',
u'\u2474': 'ParenthesizedDigitOne',
u'\u58f7': 'CjkUnifiedIdeograph58f7',
u'\ufb76': 'ArabicLetterNyehIsolatedForm',
u'\u7d7a': 'CjkUnifiedIdeograph7d7a',
u'\u91fd': 'CjkUnifiedIdeograph91fd',
u'\uab01': 'UnicodeCharacter0xab01',
u'\uca80': 'HangulSyllableJjek',
u'\u2d05': 'GeorgianSmallLetterVin',
u'\u4c84': 'CjkUnifiedIdeograph4c84',
u'\ue007': 'UnicodeCharacter0xe007',
u'\u7a0b': 'CjkUnifiedIdeograph7a0b',
u'\u858a': 'CjkUnifiedIdeograph858a',
u'\u1f8e': 'GreekCapitalLetterAlphaWithPsiliAndPerispomeniAndProsgegrammeni',
u'\ub311': 'HangulSyllableDaeb',
u'\ud290': 'HangulSyllableTwim',
u'\u3515': 'CjkUnifiedIdeograph3515',
u'\u5494': 'CjkUnifiedIdeograph5494',
u'\u821b': 'CjkUnifiedIdeograph821b',
u'\uad9a': 'HangulSyllableGweobs',
u'\u041f': 'CyrillicCapitalLetterPe',
u'\u279e': 'HeavyTriangleheadedRightwardsArrow',
u'\udb21': 'UnicodeCharacter0xdb21',
u'\ufaa0': 'CjkCompatibilityIdeographfaa0',
u'\u5d25': 'CjkUnifiedIdeograph5d25',
u'\u7ca4': 'CjkUnifiedIdeograph7ca4',
u'\uaa2b': 'ChamVowelSignIi',
u'\ub5aa': 'HangulSyllableDdeolm',
u'\u2c2f': 'UnicodeCharacter0x2c2f',
u'\u4fae': 'CjkUnifiedIdeograph4fae',
u'\ue331': 'UnicodeCharacter0xe331',
u'\u6535': 'CjkUnifiedIdeograph6535',
u'\u84b4': 'CjkUnifiedIdeograph84b4',
u'\u1eb8': 'LatinCapitalLetterEWithDotBelow',
u'\ub23b': 'HangulSyllableNweoh',
u'\uddba': 'UnicodeCharacter0xddba',
u'\u343f': 'CjkUnifiedIdeograph343f',
u'\u57be': 'CjkUnifiedIdeograph57be',
u'\u8d45': 'CjkUnifiedIdeograph8d45',
u'\uacc4': 'HangulSyllableGye',
u'\u0749': 'SyriacMusic',
u'\u26c8': 'UnicodeCharacter0x26c8',
u'\uda4b': 'UnicodeCharacter0xda4b',
u'\ue5ca': 'UnicodeCharacter0xe5ca',
u'\u5c4f': 'CjkUnifiedIdeograph5c4f',
u'\u7fce': 'CjkUnifiedIdeograph7fce',
u'\u9555': 'CjkUnifiedIdeograph9555',
u'\ub4d4': 'HangulSyllableDyuss',
u'\u2f59': 'KangxiRadicalHalfTreeTrunk',
u'\u4ed8': 'CjkUnifiedIdeograph4ed8',
u'\ue25b': 'UnicodeCharacter0xe25b',
u'\u645f': 'CjkUnifiedIdeograph645f',
u'\u87de': 'CjkUnifiedIdeograph87de',
u'\u19e2': 'KhmerSymbolPiiKoet',
u'\ubd65': 'HangulSyllableByog',
u'\udce4': 'UnicodeCharacter0xdce4',
u'\u3769': 'CjkUnifiedIdeograph3769',
u'\u56e8': 'CjkUnifiedIdeograph56e8',
u'\u8c6f': 'CjkUnifiedIdeograph8c6f',
u'\uafee': 'HangulSyllableGgweop',
u'\u0673': 'ArabicLetterAlefWithWavyHamzaBelow',
u'\u21f2': 'SouthEastArrowToCorner',
u'\uc575': 'HangulSyllableAeng',
u'\ue4f4': 'UnicodeCharacter0xe4f4',
u'\u5f79': 'CjkUnifiedIdeograph5f79',
u'\u7ef8': 'CjkUnifiedIdeograph7ef8',
u'\u947f': 'CjkUnifiedIdeograph947f',
u'\ub7fe': 'HangulSyllableReobs',
u'\uc206': 'HangulSyllableSyolm',
u'\ue689': 'UnicodeCharacter0xe689',
u'\u8708': 'CjkUnifiedIdeograph8708',
u'\ubb8b': 'HangulSyllableMwigs',
u'\u440a': 'CjkUnifiedIdeograph440a',
u'\u788d': 'CjkUnifiedIdeograph788d',
u'\u190c': 'LimbuLetterTha',
u'\u3d8f': 'CjkUnifiedIdeograph3d8f',
u'\uea16': 'UnicodeCharacter0xea16',
u'\uaf18': 'HangulSyllableGgyel',
u'\uc39b': 'HangulSyllableSsec',
u'\u6c1a': 'CjkUnifiedIdeograph6c1a',
u'\u809d': 'CjkUnifiedIdeograph809d',
u'\u211c': 'BlackletterCapitalR',
u'\u459f': 'CjkUnifiedIdeograph459f',
u'\u1aa1': 'UnicodeCharacter0x1aa1',
u'\uf226': 'UnicodeCharacter0xf226',
u'\ub728': 'HangulSyllableDdeu',
u'\uebab': 'UnicodeCharacter0xebab',
u'\u742a': 'CjkUnifiedIdeograph742a',
u'\ua8ad': 'SaurashtraLetterLa',
u'\u492c': 'CjkUnifiedIdeograph492c',
u'\u6daf': 'CjkUnifiedIdeograph6daf',
u'\u0e2e': 'ThaiCharacterHoNokhuk',
u'\u22b1': 'SucceedsUnderRelation',
u'\udf38': 'UnicodeCharacter0xdf38',
u'\uf3bb': 'UnicodeCharacter0xf3bb',
u'\u9c3a': 'CjkUnifiedIdeograph9c3a',
u'\ub0bd': 'HangulSyllableNaelg',
u'\u513c': 'CjkUnifiedIdeograph513c',
u'\u75bf': 'CjkUnifiedIdeograph75bf',
u'\u163e': 'CanadianSyllabicsCarrierTli',
u'\u4ac1': 'CjkUnifiedIdeograph4ac1',
u'\u0fc3': 'TibetanCantillationSignSbubChal',
u'\ue748': 'UnicodeCharacter0xe748',
u'\ua44a': 'YiSyllableNyit',
u'\ud8cd': 'UnicodeCharacter0xd8cd',
u'\u794c': 'CjkUnifiedIdeograph794c',
u'\u9dcf': 'CjkUnifiedIdeograph9dcf',
u'\u3e4e': 'CjkUnifiedIdeograph3e4e',
u'\u52d1': 'CjkUnifiedIdeograph52d1',
u'\u17d3': 'KhmerSignBathamasat',
u'\ucc5a': 'HangulSyllableCaej',
u'\ue0dd': 'UnicodeCharacter0xe0dd',
u'\u815c': 'CjkUnifiedIdeograph815c',
u'\ua5df': 'VaiSyllableNo',
u'\u465e': 'CjkUnifiedIdeograph465e',
u'\u7ae1': 'CjkUnifiedIdeograph7ae1',
u'\u1b60': 'BalinesePameneng',
u'\u3fe3': 'CjkUnifiedIdeograph3fe3',
u'\ud46a': 'HangulSyllablePyolp',
u'\ua96c': 'UnicodeCharacter0xa96c',
u'\ucdef': 'HangulSyllableCwid',
u'\u6e6e': 'CjkUnifiedIdeograph6e6e',
u'\u82f1': 'CjkUnifiedIdeograph82f1',
u'\u2370': 'AplFunctionalSymbolQuadQuestion',
u'\u47f3': 'CjkUnifiedIdeograph47f3',
u'\u04f5': 'CyrillicSmallLetterCheWithDiaeresis',
u'\ufc7a': 'ArabicLigatureThehWithAlefMaksuraFinalForm',
u'\ub17c': 'HangulSyllableNon',
u'\ud5ff': 'HangulSyllableHeh',
u'\u767e': 'CjkUnifiedIdeograph767e',
u'\u4b80': 'CjkUnifiedIdeograph4b80',
u'\uef03': 'UnicodeCharacter0xef03',
u'\u0882': 'UnicodeCharacter0x882',
u'\uac05': 'HangulSyllableGanj',
u'\u6107': 'CjkUnifiedIdeograph6107',
u'\u2609': 'Sun',
u'\ud98c': 'UnicodeCharacter0xd98c',
u'\u9e8e': 'CjkUnifiedIdeograph9e8e',
u'\u5390': 'CjkUnifiedIdeograph5390',
u'\uf713': 'UnicodeCharacter0xf713',
u'\u1092': 'MyanmarShanDigitTwo',
u'\ub415': 'HangulSyllableDwaet',
u'\u8917': 'CjkUnifiedIdeograph8917',
u'\u4e19': 'CjkUnifiedIdeograph4e19',
u'\u031b': 'CombiningHorn',
u'\ue19c': 'UnicodeCharacter0xe19c',
u'\ua69e': 'UnicodeCharacter0xa69e',
u'\u7ba0': 'CjkUnifiedIdeograph7ba0',
u'\u38a2': 'CjkUnifiedIdeograph38a2',
u'\udc25': 'UnicodeCharacter0xdc25',
u'\u9127': 'CjkUnifiedIdeograph9127',
u'\u5629': 'CjkUnifiedIdeograph5629',
u'\u2b2b': 'WhiteSmallLozenge',
u'\uceae': 'HangulSyllableKyagg',
u'\u83b0': 'CjkUnifiedIdeograph83b0',
u'\u40b2': 'CjkUnifiedIdeograph40b2',
u'\ue435': 'UnicodeCharacter0xe435',
u'\u05b4': 'HebrewPointHiriq',
u'\ub937': 'HangulSyllableRwec',
u'\u7e39': 'CjkUnifiedIdeograph7e39',
u'\u333b': 'SquarePeezi',
u'\ud6be': 'HangulSyllableHyoj',
u'\uabc0': 'UnicodeCharacter0xabc0',
u'\u68c2': 'CjkUnifiedIdeograph68c2',
u'\u2dc4': 'EthiopicSyllableQyee',
u'\uc147': 'HangulSyllableSelh',
u'\u8649': 'CjkUnifiedIdeograph8649',
u'\u5b4b': 'CjkUnifiedIdeograph5b4b',
u'\u184d': 'MongolianLetterTodoQa',
u'\ufece': 'ArabicLetterGhainFinalForm',
u'\ub3d0': 'HangulSyllableDols',
u'\u70d2': 'CjkUnifiedIdeograph70d2',
u'\u35d4': 'CjkUnifiedIdeograph35d4',
u'\ue957': 'UnicodeCharacter0xe957',
u'\u0ad6': 'UnicodeCharacter0xad6',
u'\uae59': 'HangulSyllableGgalt',
u'\u635b': 'CjkUnifiedIdeograph635b',
u'\u205d': 'Tricolon',
u'\udbe0': 'UnicodeCharacter0xdbe0',
u'\u98e2': 'CjkUnifiedIdeograph98e2',
u'\u5de4': 'CjkUnifiedIdeograph5de4',
u'\uf167': 'UnicodeCharacter0xf167',
u'\u12e6': 'EthiopicSyllableZho',
u'\ub669': 'HangulSyllableDdoenj',
u'\u8b6b': 'CjkUnifiedIdeograph8b6b',
u'\u486d': 'CjkUnifiedIdeograph486d',
u'\u0d6f': 'MalayalamDigitNine',
u'\ue3f0': 'UnicodeCharacter0xe3f0',
u'\ua0f2': 'YiSyllableVex',
u'\u65f4': 'CjkUnifiedIdeograph65f4',
u'\u3af6': 'CjkUnifiedIdeograph3af6',
u'\ude79': 'UnicodeCharacter0xde79',
u'\u937b': 'CjkUnifiedIdeograph937b',
u'\u507d': 'CjkUnifiedIdeograph507d',
u'\u157f': 'CanadianSyllabicsQi',
u'\u6d81': 'CjkUnifiedIdeograph6d81',
u'\u8a04': 'CjkUnifiedIdeograph8a04',
u'\ua687': 'CyrillicSmallLetterCche',
u'\u3c8b': 'CjkUnifiedIdeograph3c8b',
u'\uc30a': 'HangulSyllableSsabs',
u'\uff8d': 'HalfwidthKatakanaLetterHe',
u'\u590e': 'CjkUnifiedIdeograph590e',
u'\u7591': 'CjkUnifiedIdeograph7591',
u'\u9214': 'CjkUnifiedIdeograph9214',
u'\uce97': 'HangulSyllableKaed',
u'\u2818': 'BraillePatternDots45',
u'\u449b': 'CjkUnifiedIdeograph449b',
u'\ueb1a': 'UnicodeCharacter0xeb1a',
u'\u611e': 'CjkUnifiedIdeograph611e',
u'\u9da1': 'CjkUnifiedIdeograph9da1',
u'\u1ba5': 'SundaneseVowelSignPanyuku',
u'\uba24': 'HangulSyllableMyael',
u'\ud6a7': 'HangulSyllableHoeh',
u'\u3028': 'HangzhouNumeralEight',
u'\u6cab': 'CjkUnifiedIdeograph6cab',
u'\uf32a': 'UnicodeCharacter0xf32a',
u'\u892e': 'CjkUnifiedIdeograph892e',
u'\ua5b1': 'VaiSyllableJu',
u'\u0732': 'SyriacPthahaDotted',
u'\u23b5': 'BottomSquareBracket',
u'\uc234': 'HangulSyllableSweo',
u'\ufeb7': 'ArabicLetterSheenInitialForm',
u'\u5838': 'CjkUnifiedIdeograph5838',
u'\u74bb': 'CjkUnifiedIdeograph74bb',
u'\u913e': 'CjkUnifiedIdeograph913e',
u'\ucdc1': 'HangulSyllableCweob',
u'\u2f42': 'KangxiRadicalScript',
u'\u4bc5': 'CjkUnifiedIdeograph4bc5',
u'\uea44': 'UnicodeCharacter0xea44',
u'\u6048': 'CjkUnifiedIdeograph6048',
u'\u9ccb': 'CjkUnifiedIdeograph9ccb',
u'\u1acf': 'UnicodeCharacter0x1acf',
u'\ub94e': 'HangulSyllableRwibs',
u'\ud5d1': 'HangulSyllableHeolg',
u'\u3752': 'CjkUnifiedIdeograph3752',
u'\u53d5': 'CjkUnifiedIdeograph53d5',
u'\uf254': 'UnicodeCharacter0xf254',
u'\u8858': 'CjkUnifiedIdeograph8858',
u'\ua4db': 'UnicodeCharacter0xa4db',
u'\u065c': 'ArabicVowelSignDotBelow',
u'\u22df': 'EqualToOrSucceeds',
u'\uc15e': 'HangulSyllableSyeolm',
u'\ufde1': 'UnicodeCharacter0xfde1',
u'\u5f62': 'CjkUnifiedIdeograph5f62',
u'\u7be5': 'CjkUnifiedIdeograph7be5',
u'\u9068': 'CjkUnifiedIdeograph9068',
u'\ucceb': 'HangulSyllableCyeoh',
u'\u2e6c': 'UnicodeCharacter0x2e6c',
u'\u4aef': 'CjkUnifiedIdeograph4aef',
u'\ue96e': 'UnicodeCharacter0xe96e',
u'\u6772': 'CjkUnifiedIdeograph6772',
u'\u83f5': 'CjkUnifiedIdeograph83f5',
u'\u19f9': 'KhmerSymbolPrambuonRoc',
u'\ub878': 'HangulSyllableRwa',
u'\ud4fb': 'HangulSyllablePyus',
u'\u367c': 'CjkUnifiedIdeograph367c',
u'\u52ff': 'CjkUnifiedIdeograph52ff',
u'\uf17e': 'UnicodeCharacter0xf17e',
u'\u6803': 'CjkUnifiedIdeograph6803',
u'\u8f82': 'CjkUnifiedIdeograph8f82',
u'\u0586': 'ArmenianSmallLetterFeh',
u'\ua109': 'YiSyllableDa',
u'\uc088': 'HangulSyllableBbyiss',
u'\u3f0d': 'CjkUnifiedIdeograph3f0d',
u'\u5e8c': 'CjkUnifiedIdeograph5e8c',
u'\ufa0f': 'CjkCompatibilityIdeographfa0f',
u'\u7013': 'CjkUnifiedIdeograph7013',
u'\u9792': 'CjkUnifiedIdeograph9792',
u'\u0e17': 'ThaiCharacterThoThahan',
u'\u2d96': 'EthiopicSyllableGgwe',
u'\uc919': 'HangulSyllableJweog',
u'\ue898': 'UnicodeCharacter0xe898',
u'\u471d': 'CjkUnifiedIdeograph471d',
u'\u669c': 'CjkUnifiedIdeograph669c',
u'\u9823': 'CjkUnifiedIdeograph9823',
u'\ubfa2': 'HangulSyllableBboelp',
u'\u1627': 'CanadianSyllabicsCarrierLa',
u'\u35a6': 'CjkUnifiedIdeograph35a6',
u'\ud129': 'HangulSyllableTyaeng',
u'\uf0a8': 'UnicodeCharacter0xf0a8',
u'\u6f2d': 'CjkUnifiedIdeograph6f2d',
u'\u8eac': 'CjkUnifiedIdeograph8eac',
u'\u04b0': 'CyrillicCapitalLetterStraightUWithStroke',
u'\ua033': 'YiSyllableByx',
u'\uc7b2': 'HangulSyllableJaenh',
u'\u3e37': 'CjkUnifiedIdeograph3e37',
u'\u5db6': 'CjkUnifiedIdeograph5db6',
u'\uf939': 'CjkCompatibilityIdeographf939',
u'\u773d': 'CjkUnifiedIdeograph773d',
u'\u96bc': 'CjkUnifiedIdeograph96bc',
u'\u0d41': 'MalayalamVowelSignU',
u'\u2cc0': 'CopticCapitalLetterSampi',
u'\uc843': 'HangulSyllableJyeolb',
u'\uefc2': 'UnicodeCharacter0xefc2',
u'\u4647': 'CjkUnifiedIdeograph4647',
u'\u65c6': 'CjkUnifiedIdeograph65c6',
u'\u9f4d': 'CjkUnifiedIdeograph9f4d',
u'\ubecc': 'HangulSyllableBbyaek',
u'\u1551': 'CanadianSyllabicsWestcreeR',
u'\u34d0': 'CjkUnifiedIdeograph34d0',
u'\ud053': 'HangulSyllableKyugs',
u'\uf7d2': 'UnicodeCharacter0xf7d2',
u'\u6e57': 'CjkUnifiedIdeograph6e57',
u'\u8dd6': 'CjkUnifiedIdeograph8dd6',
u'\u0bda': 'UnicodeCharacter0xbda',
u'\ua75d': 'LatinSmallLetterRumRotunda',
u'\uc6dc': 'HangulSyllableWeom',
u'\u3d61': 'CjkUnifiedIdeograph3d61',
u'\u5ce0': 'CjkUnifiedIdeograph5ce0',
u'\uf863': 'UnicodeCharacter0xf863',
u'\u7667': 'CjkUnifiedIdeograph7667',
u'\u95e6': 'CjkUnifiedIdeograph95e6',
u'\u0c6b': 'TeluguDigitFive',
u'\u13ea': 'CherokeeLetterWe',
u'\ucf6d': 'HangulSyllableKot',
u'\ueeec': 'UnicodeCharacter0xeeec',
u'\u4571': 'CjkUnifiedIdeograph4571',
u'\u64f0': 'CjkUnifiedIdeograph64f0',
u'\u9e77': 'CjkUnifiedIdeograph9e77',
u'\ubdf6': 'HangulSyllableByunh',
u'\u147b': 'CanadianSyllabicsWestcreeKwo',
u'\u3bfa': 'CjkUnifiedIdeograph3bfa',
u'\ud77d': 'HangulSyllableHyib',
u'\uf6fc': 'UnicodeCharacter0xf6fc',
u'\uec81': 'UnicodeCharacter0xec81',
u'\u8d00': 'CjkUnifiedIdeograph8d00',
u'\ua983': 'UnicodeCharacter0xa983',
u'\u4e02': 'CjkUnifiedIdeograph4e02',
u'\u6a85': 'CjkUnifiedIdeograph6a85',
u'\u0b04': 'UnicodeCharacter0xb04',
u'\u2787': 'DingbatCircledSansserifDigitEight',
u'\ud80e': 'UnicodeCharacter0xd80e',
u'\uf491': 'UnicodeCharacter0xf491',
u'\u9510': 'CjkUnifiedIdeograph9510',
u'\ub193': 'HangulSyllableNoh',
u'\u5612': 'CjkUnifiedIdeograph5612',
u'\u7295': 'CjkUnifiedIdeograph7295',
u'\u1314': 'EthiopicSyllableGwee',
u'\u4f97': 'CjkUnifiedIdeograph4f97',
u'\u0899': 'UnicodeCharacter0x899',
u'\ue01e': 'UnicodeCharacter0xe01e',
u'\ubd20': 'HangulSyllableBwam',
u'\ud9a3': 'UnicodeCharacter0xd9a3',
u'\u7e22': 'CjkUnifiedIdeograph7e22',
u'\u9aa5': 'CjkUnifiedIdeograph9aa5',
u'\u3b24': 'CjkUnifiedIdeograph3b24',
u'\u57a7': 'CjkUnifiedIdeograph57a7',
u'\u10a9': 'GeorgianCapitalLetterKan',
u'\uc530': 'HangulSyllableSsil',
u'\ue1b3': 'UnicodeCharacter0xe1b3',
u'\u8632': 'CjkUnifiedIdeograph8632',
u'\ua2b5': 'YiSyllableCiep',
u'\u4334': 'CjkUnifiedIdeograph4334',
u'\u7fb7': 'CjkUnifiedIdeograph7fb7',
u'\u1c36': 'LepchaSignRan',
u'\u38b9': 'CjkUnifiedIdeograph38b9',
u'\ued40': 'UnicodeCharacter0xed40',
u'\uae42': 'HangulSyllableGibs',
u'\ucac5': 'HangulSyllableJjolg',
u'\u6b44': 'CjkUnifiedIdeograph6b44',
u'\u87c7': 'CjkUnifiedIdeograph87c7',
u'\u2446': 'OcrBranchBankIdentification',
u'\u40c9': 'CjkUnifiedIdeograph40c9',
u'\u1dcb': 'CombiningBrevemacron',
u'\uf550': 'UnicodeCharacter0xf550',
u'\ub652': 'HangulSyllableDdwaelm',
u'\ud2d5': 'HangulSyllableTyig',
u'\u7354': 'CjkUnifiedIdeograph7354',
u'\uafd7': 'HangulSyllableGgweogs',
u'\u4c56': 'CjkUnifiedIdeograph4c56',
u'\u68d9': 'CjkUnifiedIdeograph68d9',
u'\u0958': 'DevanagariLetterQa',
u'\u25db': 'LowerHalfInverseWhiteCircle',
u'\ude62': 'UnicodeCharacter0xde62',
u'\ufae5': 'UnicodeCharacter0xfae5',
u'\u9b64': 'CjkUnifiedIdeograph9b64',
u'\ub7e7': 'HangulSyllableRyaec',
u'\u5466': 'CjkUnifiedIdeograph5466',
u'\u70e9': 'CjkUnifiedIdeograph70e9',
u'\u1168': 'HangulJungseongYe',
u'\u4deb': 'HexagramForComingToMeet',
u'\u0eed': 'UnicodeCharacter0xeed',
u'\ue672': 'UnicodeCharacter0xe672',
u'\ua374': 'YiSyllableChurx',
u'\udff7': 'UnicodeCharacter0xdff7',
u'\u7c76': 'CjkUnifiedIdeograph7c76',
u'\u98f9': 'CjkUnifiedIdeograph98f9',
u'\u3978': 'CjkUnifiedIdeograph3978',
u'\u55fb': 'CjkUnifiedIdeograph55fb',
u'\u16fd': 'UnicodeCharacter0x16fd',
u'\u2c01': 'GlagoliticCapitalLetterBuky',
u'\ucb84': 'HangulSyllableJjwen',
u'\u8486': 'CjkUnifiedIdeograph8486',
u'\u4188': 'CjkUnifiedIdeograph4188',
u'\ufd0b': 'ArabicLigatureSheenWithKhahIsolatedForm',
u'\u028a': 'LatinSmallLetterUpsilon',
u'\ube0d': 'HangulSyllableBeug',
u'\u7b0f': 'CjkUnifiedIdeograph7b0f',
u'\u3411': 'CjkUnifiedIdeograph3411',
u'\ud394': 'HangulSyllablePeok',
u'\uac96': 'HangulSyllableGelm',
u'\u6998': 'CjkUnifiedIdeograph6998',
u'\u2a9a': 'DoublelineEqualToOrGreaterthan',
u'\uc61d': 'HangulSyllableYeng',
u'\u831f': 'CjkUnifiedIdeograph831f',
u'\u5c21': 'CjkUnifiedIdeograph5c21',
u'\u1923': 'LimbuVowelSignEe',
u'\ufba4': 'ArabicLetterHehWithYehAboveIsolatedForm',
u'\ub4a6': 'HangulSyllableDwigg',
u'\u71a8': 'CjkUnifiedIdeograph71a8',
u'\u32aa': 'CircledIdeographReligion',
u'\uee2d': 'UnicodeCharacter0xee2d',
u'\u0fac': 'TibetanSubjoinedLetterDzha',
u'\uab2f': 'UnicodeCharacter0xab2f',
u'\u6431': 'CjkUnifiedIdeograph6431',
u'\u2133': 'ScriptCapitalM',
u'\udcb6': 'UnicodeCharacter0xdcb6',
u'\u99b8': 'CjkUnifiedIdeograph99b8',
u'\u5aba': 'CjkUnifiedIdeograph5aba',
u'\uf63d': 'UnicodeCharacter0xf63d',
u'\u17bc': 'KhmerVowelSignUu',
u'\ub33f': 'HangulSyllableDyaed',
u'\u8c41': 'CjkUnifiedIdeograph8c41',
u'\u4943': 'CjkUnifiedIdeograph4943',
u'\u0a45': 'UnicodeCharacter0xa45',
u'\ue4c6': 'UnicodeCharacter0xe4c6',
u'\ua1c8': 'YiSyllableLox',
u'\u62ca': 'CjkUnifiedIdeograph62ca',
u'\u3fcc': 'CjkUnifiedIdeograph3fcc',
u'\udb4f': 'UnicodeCharacter0xdb4f',
u'\u9451': 'CjkUnifiedIdeograph9451',
u'\u5153': 'CjkUnifiedIdeograph5153',
u'\u1255': 'EthiopicSyllableQhe',
u'\uc9d8': 'HangulSyllableJik',
u'\u8ada': 'CjkUnifiedIdeograph8ada',
u'\u47dc': 'CjkUnifiedIdeograph47dc',
u'\ue35f': 'UnicodeCharacter0xe35f',
u'\xde': 'LatinCapitalLetterThorn',
u'\ubc61': 'HangulSyllableByang',
u'\u7963': 'CjkUnifiedIdeograph7963',
u'\u3a65': 'CjkUnifiedIdeograph3a65',
u'\ud1e8': 'HangulSyllableTwaem',
u'\u92ea': 'CjkUnifiedIdeograph92ea',
u'\u6fec': 'CjkUnifiedIdeograph6fec',
u'\u28ee': 'BraillePatternDots234678',
u'\uc471': 'HangulSyllableSsult',
u'\u8173': 'CjkUnifiedIdeograph8173',
u'\u4275': 'CjkUnifiedIdeograph4275',
u'\u1f77': 'GreekSmallLetterIotaWithOxia',
u'\uf9f8': 'CjkCompatibilityIdeographf9f8',
u'\ubafa': 'HangulSyllableMwaep',
u'\u77fc': 'CjkUnifiedIdeograph77fc',
u'\u30fe': 'KatakanaVoicedIterationMark',
u'\u0e00': 'UnicodeCharacter0xe00',
u'\u2a83': 'LessthanOrSlantedEqualToWithDotAboveRight',
u'\ucd02': 'HangulSyllableCyej',
u'\ue985': 'UnicodeCharacter0xe985',
u'\u4706': 'CjkUnifiedIdeograph4706',
u'\u6389': 'CjkUnifiedIdeograph6389',
u'\u9c0c': 'CjkUnifiedIdeograph9c0c',
u'\ub88f': 'HangulSyllableRwac',
u'\u1610': 'CanadianSyllabicsCarrierYe',
u'\u3293': 'CircledIdeographSociety',
u'\ud512': 'HangulSyllablePeulp',
u'\uf195': 'UnicodeCharacter0xf195',
u'\u6f16': 'CjkUnifiedIdeograph6f16',
u'\u8b99': 'CjkUnifiedIdeograph8b99',
u'\u059d': 'HebrewAccentGereshMuqdam',
u'\ua41c': 'YiSyllableJjiet',
u'\uc09f': 'HangulSyllableBbilh',
u'\u3e20': 'CjkUnifiedIdeograph3e20',
u'\u5aa3': 'CjkUnifiedIdeograph5aa3',
u'\ufd22': 'ArabicLigatureSadWithYehFinalForm',
u'\u7726': 'CjkUnifiedIdeograph7726',
u'\u93a9': 'CjkUnifiedIdeograph93a9',
u'\u2dad': 'EthiopicSyllableCce',
u'\ucc2c': 'HangulSyllableCan',
u'\ue8af': 'UnicodeCharacter0xe8af',
u'\u4630': 'CjkUnifiedIdeograph4630',
u'\u62b3': 'CjkUnifiedIdeograph62b3',
u'\u9f36': 'CjkUnifiedIdeograph9f36',
u'\ubbb9': 'HangulSyllableMyung',
u'\u193a': 'LimbuSignKemphreng',
u'\u35bd': 'CjkUnifiedIdeograph35bd',
u'\ud43c': 'HangulSyllablePwaek',
u'\uf0bf': 'UnicodeCharacter0xf0bf',
u'\u6e40': 'CjkUnifiedIdeograph6e40',
u'\u8ac3': 'CjkUnifiedIdeograph8ac3',
u'\u04c7': 'CyrillicCapitalLetterEnWithHook',
u'\ua746': 'LatinCapitalLetterBrokenL',
u'\uc3c9': 'HangulSyllableSsyelt',
u'\u214a': 'PropertyLine',
u'\u5dcd': 'CjkUnifiedIdeograph5dcd',
u'\ufc4c': 'ArabicLigatureNoonWithHahIsolatedForm',
u'\u7650': 'CjkUnifiedIdeograph7650',
u'\u92d3': 'CjkUnifiedIdeograph92d3',
u'\u2cd7': 'CopticSmallLetterOldCopticGangia',
u'\ucf56': 'HangulSyllableKogg',
u'\uebd9': 'UnicodeCharacter0xebd9',
u'\u495a': 'CjkUnifiedIdeograph495a',
u'\u65dd': 'CjkUnifiedIdeograph65dd',
u'\u9e60': 'CjkUnifiedIdeograph9e60',
u'\ubae3': 'HangulSyllableMwaegs',
u'\u1864': 'MongolianLetterSibeGa',
u'\u34e7': 'CjkUnifiedIdeograph34e7',
u'\ud766': 'HangulSyllableHeuj',
u'\uf3e9': 'UnicodeCharacter0xf3e9',
u'\u516a': 'CjkUnifiedIdeograph516a',
u'\u8ded': 'CjkUnifiedIdeograph8ded',
u'\u07f1': 'NkoCombiningLongRisingTone',
u'\ua670': 'CombiningCyrillicTenMillionsSign',
u'\uc2f3': 'HangulSyllableSic',
u'\u2074': 'SuperscriptFour',
u'\u5cf7': 'CjkUnifiedIdeograph5cf7',
u'\uff76': 'HalfwidthKatakanaLetterKa',
u'\u797a': 'CjkUnifiedIdeograph797a',
u'\u95fd': 'CjkUnifiedIdeograph95fd',
u'\uaf01': 'HangulSyllableGgyeolt',
u'\uce80': 'HangulSyllableKals',
u'\u2905': 'RightwardsTwoheadedArrowFromBar',
u'\u4884': 'CjkUnifiedIdeograph4884',
u'\ue407': 'UnicodeCharacter0xe407',
u'\u7e0b': 'CjkUnifiedIdeograph7e0b',
u'\u818a': 'CjkUnifiedIdeograph818a',
u'\u1b8e': 'SundaneseLetterCa',
u'\ub711': 'HangulSyllableDdyunj',
u'\ud690': 'HangulSyllableHoen',
u'\u3115': 'BopomofoLetterSh',
u'\u5094': 'CjkUnifiedIdeograph5094',
u'\u861b': 'CjkUnifiedIdeograph861b',
u'\ua99a': 'UnicodeCharacter0xa99a',
u'\u239e': 'RightParenthesisUpperHook',
u'\udf21': 'UnicodeCharacter0xdf21',
u'\ufea0': 'ArabicLetterJeemMedialForm',
u'\u5925': 'CjkUnifiedIdeograph5925',
u'\u78a4': 'CjkUnifiedIdeograph78a4',
u'\uae2b': 'HangulSyllableGyic',
u'\ub1aa': 'HangulSyllableNwaj',
u'\u282f': 'BraillePatternDots12346',
u'\u4bae': 'CjkUnifiedIdeograph4bae',
u'\ue731': 'UnicodeCharacter0xe731',
u'\u6135': 'CjkUnifiedIdeograph6135',
u'\u80b4': 'CjkUnifiedIdeograph80b4',
u'\u1ab8': 'UnicodeCharacter0x1ab8',
u'\ub63b': 'HangulSyllableDdwalh',
u'\ud9ba': 'UnicodeCharacter0xd9ba',
u'\u303f': 'IdeographicHalfFillSpace',
u'\u53be': 'CjkUnifiedIdeograph53be',
u'\u8945': 'CjkUnifiedIdeograph8945',
u'\ua8c4': 'SaurashtraSignVirama',
u'\u0349': 'CombiningLeftAngleBelow',
u'\u22c8': 'Bowtie',
u'\ude4b': 'UnicodeCharacter0xde4b',
u'\ue1ca': 'UnicodeCharacter0xe1ca',
u'\u584f': 'CjkUnifiedIdeograph584f',
u'\u7bce': 'CjkUnifiedIdeograph7bce',
u'\u9155': 'CjkUnifiedIdeograph9155',
u'\ub0d4': 'HangulSyllableNyan',
u'\u2b59': 'UnicodeCharacter0x2b59',
u'\u4ad8': 'CjkUnifiedIdeograph4ad8',
u'\ue65b': 'UnicodeCharacter0xe65b',
u'\u605f': 'CjkUnifiedIdeograph605f',
u'\u83de': 'CjkUnifiedIdeograph83de',
u'\u1de2': 'CombiningLatinLetterSmallCapitalR',
u'\ub965': 'HangulSyllableRyult',
u'\ud8e4': 'UnicodeCharacter0xd8e4',
u'\u3369': 'IdeographicTelegraphSymbolForHourSeventeen',
u'\u52e8': 'CjkUnifiedIdeograph52e8',
u'\u886f': 'CjkUnifiedIdeograph886f',
u'\uabee': 'UnicodeCharacter0xabee',
u'\u0273': 'LatinSmallLetterNWithRetroflexHook',
u'\u25f2': 'WhiteSquareWithLowerRightQuadrant',
u'\uc175': 'HangulSyllableSyenj',
u'\ue0f4': 'UnicodeCharacter0xe0f4',
u'\u5b79': 'CjkUnifiedIdeograph5b79',
u'\u7af8': 'CjkUnifiedIdeograph7af8',
u'\u907f': 'CjkUnifiedIdeograph907f',
u'\ub3fe': 'HangulSyllableDwaegg',
u'\uc606': 'HangulSyllableYeop',
u'\ue289': 'UnicodeCharacter0xe289',
u'\u8308': 'CjkUnifiedIdeograph8308',
u'\ubf8b': 'HangulSyllableBbwaes',
u'\u400a': 'CjkUnifiedIdeograph400a',
u'\u7c8d': 'CjkUnifiedIdeograph7c8d',
u'\u1d0c': 'LatinLetterSmallCapitalLWithStroke',
u'\u398f': 'CjkUnifiedIdeograph398f',
u'\uee16': 'UnicodeCharacter0xee16',
u'\uab18': 'UnicodeCharacter0xab18',
u'\uc79b': 'HangulSyllableJalb',
u'\u681a': 'CjkUnifiedIdeograph681a',
u'\u849d': 'CjkUnifiedIdeograph849d',
u'\u251c': 'BoxDrawingsLightVerticalAndRight',
u'\u419f': 'CjkUnifiedIdeograph419f',
u'\u1ea1': 'LatinSmallLetterAWithDotBelow',
u'\uf626': 'UnicodeCharacter0xf626',
u'\ub328': 'HangulSyllableDyals',
u'\uefab': 'UnicodeCharacter0xefab',
u'\u702a': 'CjkUnifiedIdeograph702a',
u'\uacad': 'HangulSyllableGyeonj',
u'\u4d2c': 'CjkUnifiedIdeograph4d2c',
u'\u69af': 'CjkUnifiedIdeograph69af',
u'\u0a2e': 'GurmukhiLetterMa',
u'\u26b1': 'FuneralUrn',
u'\udb38': 'UnicodeCharacter0xdb38',
u'\uf7bb': 'UnicodeCharacter0xf7bb',
u'\u983a': 'CjkUnifiedIdeograph983a',
u'\ub4bd': 'HangulSyllableDwit',
u'\u553c': 'CjkUnifiedIdeograph553c',
u'\u71bf': 'CjkUnifiedIdeograph71bf',
u'\u123e': 'EthiopicSyllableSho',
u'\u4ec1': 'CjkUnifiedIdeograph4ec1',
u'\u0bc3': 'UnicodeCharacter0xbc3',
u'\ue348': 'UnicodeCharacter0xe348',
u'\ua04a': 'YiSyllablePut',
u'\udccd': 'UnicodeCharacter0xdccd',
u'\u7d4c': 'CjkUnifiedIdeograph7d4c',
u'\u99cf': 'CjkUnifiedIdeograph99cf',
u'\u3a4e': 'CjkUnifiedIdeograph3a4e',
u'\u56d1': 'CjkUnifiedIdeograph56d1',
u'\u13d3': 'CherokeeLetterDa',
u'\uc85a': 'HangulSyllableJyenh',
u'\ue4dd': 'UnicodeCharacter0xe4dd',
u'\u855c': 'CjkUnifiedIdeograph855c',
u'\ua1df': 'YiSyllableGiex',
u'\u425e': 'CjkUnifiedIdeograph425e',
u'\u7ee1': 'CjkUnifiedIdeograph7ee1',
u'\u1f60': 'GreekSmallLetterOmegaWithPsili',
u'\u3be3': 'CjkUnifiedIdeograph3be3',
u'\ud06a': 'HangulSyllableKyup',
u'\uad6c': 'HangulSyllableGu',
u'\uc9ef': 'HangulSyllableJjas',
u'\u6a6e': 'CjkUnifiedIdeograph6a6e',
u'\u86f1': 'CjkUnifiedIdeograph86f1',
u'\u2770': 'HeavyLeftpointingAngleBracketOrnament',
u'\u43f3': 'CjkUnifiedIdeograph43f3',
u'\xf5': 'LatinSmallLetterOWithTilde',
u'\uf87a': 'UnicodeCharacter0xf87a',
u'\ub57c': 'HangulSyllableDdyass',
u'\ud1ff': 'HangulSyllableToelb',
u'\u727e': 'CjkUnifiedIdeograph727e',
u'\u4f80': 'CjkUnifiedIdeograph4f80',
u'\ueb03': 'UnicodeCharacter0xeb03',
u'\u0c82': 'KannadaSignAnusvara',
u'\ua805': 'SylotiNagriLetterO',
u'\u6507': 'CjkUnifiedIdeograph6507',
u'\u2209': 'NotAnElementOf',
u'\udd8c': 'UnicodeCharacter0xdd8c',
u'\u9a8e': 'CjkUnifiedIdeograph9a8e',
u'\u5790': 'CjkUnifiedIdeograph5790',
u'\uf313': 'UnicodeCharacter0xf313',
u'\u1492': 'CanadianSyllabicsCwe',
u'\ub015': 'HangulSyllableGgwilg',
u'\u8d17': 'CjkUnifiedIdeograph8d17',
u'\u4a19': 'CjkUnifiedIdeograph4a19',
u'\u071b': 'SyriacLetterTeth',
u'\ue59c': 'UnicodeCharacter0xe59c',
u'\ua29e': 'YiSyllableZop',
u'\u7fa0': 'CjkUnifiedIdeograph7fa0',
u'\u3ca2': 'CjkUnifiedIdeograph3ca2',
u'\ud825': 'UnicodeCharacter0xd825',
u'\u9527': 'CjkUnifiedIdeograph9527',
u'\u5229': 'CjkUnifiedIdeograph5229',
u'\u2f2b': 'KangxiRadicalCorpse',
u'\ucaae': 'HangulSyllableJjyelp',
u'\u87b0': 'CjkUnifiedIdeograph87b0',
u'\u44b2': 'CjkUnifiedIdeograph44b2',
u'\ue035': 'UnicodeCharacter0xe035',
u'\u01b4': 'LatinSmallLetterYWithHook',
u'\ubd37': 'HangulSyllableBwaelb',
u'\u7a39': 'CjkUnifiedIdeograph7a39',
u'\u373b': 'CjkUnifiedIdeograph373b',
u'\ud2be': 'HangulSyllableTeunh',
u'\uafc0': 'HangulSyllableGgul',
u'\u6cc2': 'CjkUnifiedIdeograph6cc2',
u'\u29c4': 'SquaredRisingDiagonalSlash',
u'\uc547': 'HangulSyllableAgs',
u'\u8249': 'CjkUnifiedIdeograph8249',
u'\u5f4b': 'CjkUnifiedIdeograph5f4b',
u'\u1c4d': 'LepchaLetterTta',
u'\uface': 'CjkCompatibilityIdeographface',
u'\ub7d0': 'HangulSyllableRyae',
u'\u74d2': 'CjkUnifiedIdeograph74d2',
u'\u31d4': 'CjkStrokeD',
u'\ued57': 'UnicodeCharacter0xed57',
u'\u0ed6': 'LaoDigitSix',
u'\uaa59': 'ChamDigitNine',
u'\u675b': 'CjkUnifiedIdeograph675b',
u'\u245d': 'UnicodeCharacter0x245d',
u'\udfe0': 'UnicodeCharacter0xdfe0',
u'\u9ce2': 'CjkUnifiedIdeograph9ce2',
u'\u59e4': 'CjkUnifiedIdeograph59e4',
u'\uf567': 'UnicodeCharacter0xf567',
u'\u16e6': 'RunicLetterLongbranchyr',
u'\ub269': 'HangulSyllableNwib',
u'\u8f6b': 'CjkUnifiedIdeograph8f6b',
u'\u4c6d': 'CjkUnifiedIdeograph4c6d',
u'\u096f': 'DevanagariDigitNine',
u'\ue7f0': 'UnicodeCharacter0xe7f0',
u'\ua4f2': 'UnicodeCharacter0xa4f2',
u'\u61f4': 'CjkUnifiedIdeograph61f4',
u'\u3ef6': 'CjkUnifiedIdeograph3ef6',
u'\uda79': 'UnicodeCharacter0xda79',
u'\u977b': 'CjkUnifiedIdeograph977b',
u'\u547d': 'CjkUnifiedIdeograph547d',
u'\u117f': 'HangulJungseongOeo',
u'\u7181': 'CjkUnifiedIdeograph7181',
u'\u9604': 'CjkUnifiedIdeograph9604',
u'\uba87': 'HangulSyllableMyeoc',
u'\u1c08': 'LepchaLetterJa',
u'\u208b': 'SubscriptMinus',
u'\udf0a': 'UnicodeCharacter0xdf0a',
u'\ue38d': 'UnicodeCharacter0xe38d',
u'\u450e': 'CjkUnifiedIdeograph450e',
u'\u6991': 'CjkUnifiedIdeograph6991',
u'\u8e14': 'CjkUnifiedIdeograph8e14',
u'\ud297': 'HangulSyllableTwic',
u'\u3418': 'CjkUnifiedIdeograph3418',
u'\u589b': 'CjkUnifiedIdeograph589b',
u'\uf71a': 'UnicodeCharacter0xf71a',
u'\u7d1e': 'CjkUnifiedIdeograph7d1e',
u'\u81a1': 'CjkUnifiedIdeograph81a1',
u'\u07a5': 'ThaanaLetterWaavu',
u'\ua624': 'VaiDigitFour',
u'\ucaa7': 'HangulSyllableJjyed',
u'\u2c28': 'GlagoliticCapitalLetterBigYus',
u'\u70ab': 'CjkUnifiedIdeograph70ab',
u'\uef2a': 'UnicodeCharacter0xef2a',
u'\u952e': 'CjkUnifiedIdeograph952e',
u'\ub9b1': 'HangulSyllableRinj',
u'\u1b32': 'BalineseLetterSa',
u'\u3fb5': 'CjkUnifiedIdeograph3fb5',
u'\ude34': 'UnicodeCharacter0xde34',
u'\ue2b7': 'UnicodeCharacter0xe2b7',
u'\u4438': 'CjkUnifiedIdeograph4438',
u'\u68bb': 'CjkUnifiedIdeograph68bb',
u'\u8d3e': 'CjkUnifiedIdeograph8d3e',
u'\ud1c1': 'HangulSyllableTwanj',
u'\u3342': 'SquareHoon',
u'\u57c5': 'CjkUnifiedIdeograph57c5',
u'\uf644': 'UnicodeCharacter0xf644',
u'\u7c48': 'CjkUnifiedIdeograph7c48',
u'\u80cb': 'CjkUnifiedIdeograph80cb',
u'\u06cf': 'ArabicLetterWawWithDotAbove',
u'\ua54e': 'VaiSyllableWa',
u'\uc9d1': 'HangulSyllableJib',
u'\u2b52': 'WhiteSmallStar',
u'\u4fd5': 'CjkUnifiedIdeograph4fd5',
u'\uee54': 'UnicodeCharacter0xee54',
u'\u9458': 'CjkUnifiedIdeograph9458',
u'\ub8db': 'HangulSyllableRyolh',
u'\u1a5c': 'UnicodeCharacter0x1a5c',
u'\u3edf': 'CjkUnifiedIdeograph3edf',
u'\udd5e': 'UnicodeCharacter0xdd5e',
u'\ue1e1': 'UnicodeCharacter0xe1e1',
u'\u4362': 'CjkUnifiedIdeograph4362',
u'\u67e5': 'CjkUnifiedIdeograph67e5',
u'\u8c68': 'CjkUnifiedIdeograph8c68',
u'\ud0eb': 'HangulSyllableTaelh',
u'\u326c': 'CircledHangulPhieuph',
u'\u56ef': 'CjkUnifiedIdeograph56ef',
u'\uf56e': 'UnicodeCharacter0xf56e',
u'\u7b72': 'CjkUnifiedIdeograph7b72',
u'\u9ff5': 'UnicodeCharacter0x9ff5',
u'\u05f9': 'UnicodeCharacter0x5f9',
u'\ua478': 'YiSyllableYiep',
u'\uc8fb': 'HangulSyllableJyoh',
u'\u2a7c': 'GreaterthanWithQuestionMarkAbove',
u'\u4eff': 'CjkUnifiedIdeograph4eff',
u'\ued7e': 'UnicodeCharacter0xed7e',
u'\u7403': 'CjkUnifiedIdeograph7403',
u'\u9382': 'CjkUnifiedIdeograph9382',
u'\u1986': 'NewTaiLueLetterLowXa',
u'\ubd09': 'HangulSyllableBong',
u'\udc88': 'UnicodeCharacter0xdc88',
u'\u230d': 'BottomLeftCrop',
u'\u428c': 'CjkUnifiedIdeograph428c',
u'\ue60f': 'UnicodeCharacter0xe60f',
u'\u6c13': 'CjkUnifiedIdeograph6c13',
u'\u8b92': 'CjkUnifiedIdeograph8b92',
u'\u1217': 'EthiopicSyllableHhwa',
u'\u3196': 'IdeographicAnnotationTopMark',
u'\ud519': 'HangulSyllablePeung',
u'\uf498': 'UnicodeCharacter0xf498',
u'\u5b1d': 'CjkUnifiedIdeograph5b1d',
u'\u7a9c': 'CjkUnifiedIdeograph7a9c',
u'\u8423': 'CjkUnifiedIdeograph8423',
u'\ua3a2': 'YiSyllableNrup',
u'\u0a27': 'GurmukhiLetterDha',
u'\u29a6': 'ObliqueAngleOpeningUp',
u'\ucd29': 'HangulSyllableCwanj',
u'\ueca8': 'UnicodeCharacter0xeca8',
u'\u732d': 'CjkUnifiedIdeograph732d',
u'\u92ac': 'CjkUnifiedIdeograph92ac',
u'\u18b0': 'UnicodeCharacter0x18b0',
u'\ubc33': 'HangulSyllableBaegs',
u'\udbb2': 'UnicodeCharacter0xdbb2',
u'\u2237': 'Proportion',
u'\u41b6': 'CjkUnifiedIdeograph41b6',
u'\ue539': 'UnicodeCharacter0xe539',
u'\u6b3d': 'CjkUnifiedIdeograph6b3d',
u'\u8abc': 'CjkUnifiedIdeograph8abc',
u'\u1141': 'HangulChoseongIeungkiyeok',
u'\u30c0': 'KatakanaLetterDa',
u'\ud443': 'HangulSyllablePoegs',
u'\uf3c2': 'UnicodeCharacter0xf3c2',
u'\u5a47': 'CjkUnifiedIdeograph5a47',
u'\u79c6': 'CjkUnifiedIdeograph79c6',
u'\u834d': 'CjkUnifiedIdeograph834d',
u'\ua2cc': 'YiSyllableCy',
u'\u0951': 'DevanagariStressSignUdatta',
u'\u28d0': 'BraillePatternDots578',
u'\ucc53': 'HangulSyllableCaelh',
u'\uebd2': 'UnicodeCharacter0xebd2',
u'\u7257': 'CjkUnifiedIdeograph7257',
u'\u91d6': 'CjkUnifiedIdeograph91d6',
u'\u17da': 'KhmerSignKoomuut',
u'\ubb5d': 'HangulSyllableMweolt',
u'\udadc': 'UnicodeCharacter0xdadc',
u'\u2161': 'RomanNumeralTwo',
u'\u40e0': 'CjkUnifiedIdeograph40e0',
u'\ue463': 'UnicodeCharacter0xe463',
u'\u6a67': 'CjkUnifiedIdeograph6a67',
u'\u89e6': 'CjkUnifiedIdeograph89e6',
u'\u106b': 'MyanmarSignWesternPwoKarenTone3',
u'\u0fea': 'UnicodeCharacter0xfea',
u'\ud36d': 'HangulSyllablePyaelt',
u'\uf2ec': 'UnicodeCharacter0xf2ec',
u'\u5971': 'CjkUnifiedIdeograph5971',
u'\u78f0': 'CjkUnifiedIdeograph78f0',
u'\u8277': 'CjkUnifiedIdeograph8277',
u'\ua1f6': 'YiSyllableGurx',
u'\u087b': 'UnicodeCharacter0x87b',
u'\u27fa': 'LongLeftRightDoubleArrow',
u'\ucb7d': 'HangulSyllableJjweot',
u'\ueafc': 'UnicodeCharacter0xeafc',
u'\uf081': 'UnicodeCharacter0xf081',
u'\u9100': 'CjkUnifiedIdeograph9100',
u'\ub583': 'HangulSyllableDdyah',
u'\u5202': 'CjkUnifiedIdeograph5202',
u'\u7685': 'CjkUnifiedIdeograph7685',
u'\u1704': 'TagalogLetterGa',
u'\u3b87': 'CjkUnifiedIdeograph3b87',
u'\uc40e': 'HangulSyllableSswap',
u'\ue891': 'UnicodeCharacter0xe891',
u'\u8910': 'CjkUnifiedIdeograph8910',
u'\uad93': 'HangulSyllableGweolb',
u'\u4a12': 'CjkUnifiedIdeograph4a12',
u'\u6e95': 'CjkUnifiedIdeograph6e95',
u'\u0f14': 'TibetanMarkGterTsheg',
u'\u5397': 'CjkUnifiedIdeograph5397',
u'\u1499': 'CanadianSyllabicsWestcreeCwo',
u'\ufc1e': 'ArabicLigatureSeenWithKhahIsolatedForm',
u'\ua120': 'YiSyllableTiep',
u'\uc5a3': 'HangulSyllableYaelb',
u'\u6222': 'CjkUnifiedIdeograph6222',
u'\u86a5': 'CjkUnifiedIdeograph86a5',
u'\u2724': 'HeavyFourBalloonspokedAsterisk',
u'\u4ba7': 'CjkUnifiedIdeograph4ba7',
u'\u0ca9': 'UnicodeCharacter0xca9',
u'\ud930': 'UnicodeCharacter0xd930',
u'\ufdb3': 'ArabicLigatureNoonWithHahWithYehFinalForm',
u'\u9a32': 'CjkUnifiedIdeograph9a32',
u'\ubeb5': 'HangulSyllableBbyaeg',
u'\u5f34': 'CjkUnifiedIdeograph5f34',
u'\u63b7': 'CjkUnifiedIdeograph63b7',
u'\u24b9': 'CircledLatinCapitalLetterD',
u'\uf140': 'UnicodeCharacter0xf140',
u'\ub242': 'HangulSyllableNwenh',
u'\ud6c5': 'HangulSyllableHug',
u'\u7744': 'CjkUnifiedIdeograph7744',
u'\u9bc7': 'CjkUnifiedIdeograph9bc7',
u'\u3846': 'CjkUnifiedIdeograph3846',
u'\u5cc9': 'CjkUnifiedIdeograph5cc9',
u'\u01cb': 'LatinCapitalLetterNWithSmallLetterJ',
u'\ue950': 'UnicodeCharacter0xe950',
u'\uaa52': 'ChamDigitTwo',
u'\uced5': 'HangulSyllableKyaelt',
u'\u6f54': 'CjkUnifiedIdeograph6f54',
u'\ub3d7': 'HangulSyllableDos',
u'\u5056': 'CjkUnifiedIdeograph5056',
u'\u74d9': 'CjkUnifiedIdeograph74d9',
u'\u1558': 'CanadianSyllabicsFoo',
u'\u39db': 'CjkUnifiedIdeograph39db',
u'\uc262': 'HangulSyllableSwebs',
u'\ue6e5': 'UnicodeCharacter0xe6e5',
u'\u8764': 'CjkUnifiedIdeograph8764',
u'\uabe7': 'UnicodeCharacter0xabe7',
u'\u4866': 'CjkUnifiedIdeograph4866',
u'\u6ce9': 'CjkUnifiedIdeograph6ce9',
u'\u0d68': 'MalayalamDigitTwo',
u'\u51eb': 'CjkUnifiedIdeograph51eb',
u'\u12ed': 'EthiopicSyllableYe',
u'\ufa72': 'CjkCompatibilityIdeographfa72',
u'\ubf74': 'HangulSyllableBbwak',
u'\uc3f7': 'HangulSyllableSswags',
u'\u6076': 'CjkUnifiedIdeograph6076',
u'\u84f9': 'CjkUnifiedIdeograph84f9',
u'\u2578': 'BoxDrawingsHeavyLeft',
u'\u49fb': 'CjkUnifiedIdeograph49fb',
u'\u0afd': 'UnicodeCharacter0xafd',
u'\u3001': 'IdeographicComma',
u'\ud784': 'HangulSyllableHyik',
u'\u9886': 'CjkUnifiedIdeograph9886',
u'\u5d88': 'CjkUnifiedIdeograph5d88',
u'\ue10b': 'UnicodeCharacter0xe10b',
u'\u1e8a': 'LatinCapitalLetterXWithDotAbove',
u'\ua20d': 'YiSyllableKep',
u'\u670f': 'CjkUnifiedIdeograph670f',
u'\u2811': 'BraillePatternDots15',
u'\ucf94': 'HangulSyllableKwael',
u'\ub096': 'HangulSyllableGgip',
u'\u7598': 'CjkUnifiedIdeograph7598',
u'\u369a': 'CjkUnifiedIdeograph369a',
u'\uda1d': 'UnicodeCharacter0xda1d',
u'\u9f1f': 'CjkUnifiedIdeograph9f1f',
u'\u4021': 'CjkUnifiedIdeograph4021',
u'\u0523': 'CyrillicSmallLetterEnWithMiddleHook',
u'\ue7a4': 'UnicodeCharacter0xe7a4',
u'\ua8a6': 'SaurashtraLetterPa',
u'\u6da8': 'CjkUnifiedIdeograph6da8',
u'\u2eaa': 'CjkRadicalBoltOfCloth',
u'\uf22d': 'UnicodeCharacter0xf22d',
u'\u13ac': 'CherokeeLetterGv',
u'\ub72f': 'HangulSyllableDdeud',
u'\u7831': 'CjkUnifiedIdeograph7831',
u'\u3d33': 'CjkUnifiedIdeograph3d33',
u'\uc0b6': 'HangulSyllableSalm',
u'\u85b8': 'CjkUnifiedIdeograph85b8',
u'\u46ba': 'CjkUnifiedIdeograph46ba',
u'\uea3d': 'UnicodeCharacter0xea3d',
u'\u0bbc': 'UnicodeCharacter0xbbc',
u'\uaf3f': 'HangulSyllableGgos',
u'\u9041': 'CjkUnifiedIdeograph9041',
u'\u5543': 'CjkUnifiedIdeograph5543',
u'\u1645': 'CanadianSyllabicsCarrierZa',
u'\uf8c6': 'UnicodeCharacter0xf8c6',
u'\ubdc8': 'HangulSyllableBwem',
u'\u7eca': 'CjkUnifiedIdeograph7eca',
u'\u23cc': 'DentistrySymbolLightVerticalAndBottomLeft',
u'\uc74f': 'HangulSyllableEus',
u'\u8851': 'CjkUnifiedIdeograph8851',
u'\u4d53': 'CjkUnifiedIdeograph4d53',
u'\u0e55': 'ThaiDigitFive',
u'\ud5d8': 'HangulSyllableHeom',
u'\u96da': 'CjkUnifiedIdeograph96da',
u'\u5bdc': 'CjkUnifiedIdeograph5bdc',
u'\uff5f': 'FullwidthLeftWhiteParenthesis',
u'\u1cde': 'UnicodeCharacter0x1cde',
u'\ua061': 'YiSyllableBbap',
u'\u6563': 'CjkUnifiedIdeograph6563',
u'\u2665': 'BlackHeartSuit',
u'\ucde8': 'HangulSyllableCwi',
u'\u8eea': 'CjkUnifiedIdeograph8eea',
u'\u73ec': 'CjkUnifiedIdeograph73ec',
u'\u34ee': 'CjkUnifiedIdeograph34ee',
u'\ud871': 'UnicodeCharacter0xd871',
u'\u9d73': 'CjkUnifiedIdeograph9d73',
u'\u5e75': 'CjkUnifiedIdeograph5e75',
u'\u0377': 'GreekSmallLetterPamphylianDigamma',
u'\ue5f8': 'UnicodeCharacter0xe5f8',
u'\ua6fa': 'UnicodeCharacter0xa6fa',
u'\u6bfc': 'CjkUnifiedIdeograph6bfc',
u'\u2cfe': 'CopticFullStop',
u'\u1200': 'EthiopicSyllableHa',
u'\u3683': 'CjkUnifiedIdeograph3683',
u'\ud102': 'HangulSyllableTyalm',
u'\uf585': 'UnicodeCharacter0xf585',
u'\u5b06': 'CjkUnifiedIdeograph5b06',
u'\u7f89': 'CjkUnifiedIdeograph7f89',
u'\u800c': 'CjkUnifiedIdeograph800c',
u'\ua48f': 'UnicodeCharacter0xa48f',
u'\u0a10': 'GurmukhiLetterAi',
u'\u2e93': 'CjkRadicalThread',
u'\uc912': 'HangulSyllableJuj',
u'\ued95': 'UnicodeCharacter0xed95',
u'\u7316': 'CjkUnifiedIdeograph7316',
u'\u9799': 'CjkUnifiedIdeograph9799',
u'\u199d': 'NewTaiLueLetterLowFa',
u'\ub81c': 'HangulSyllableRess',
u'\udc9f': 'UnicodeCharacter0xdc9f',
u'\u2220': 'Angle',
u'\u46a3': 'CjkUnifiedIdeograph46a3',
u'\ue122': 'UnicodeCharacter0xe122',
u'\u6b26': 'CjkUnifiedIdeograph6b26',
u'\u8fa9': 'CjkUnifiedIdeograph8fa9',
u'\u31ad': 'BopomofoLetterNgg',
u'\ud02c': 'HangulSyllableKwess',
u'\uf4af': 'UnicodeCharacter0xf4af',
u'\u5a30': 'CjkUnifiedIdeograph5a30',
u'\u7eb3': 'CjkUnifiedIdeograph7eb3',
u'\u8336': 'CjkUnifiedIdeograph8336',
u'\ua7b9': 'UnicodeCharacter0xa7b9',
u'\u053a': 'ArmenianCapitalLetterZhe',
u'\u29bd': 'UpArrowThroughCircle',
u'\uc83c': 'HangulSyllableJyeon',
u'\uecbf': 'UnicodeCharacter0xecbf',
u'\u7240': 'CjkUnifiedIdeograph7240',
u'\u96c3': 'CjkUnifiedIdeograph96c3',
u'\u18c7': 'UnicodeCharacter0x18c7',
u'\ubb46': 'HangulSyllableMubs',
u'\udfc9': 'UnicodeCharacter0xdfc9',
u'\u3d4a': 'CjkUnifiedIdeograph3d4a',
u'\u41cd': 'CjkUnifiedIdeograph41cd',
u'\ue04c': 'UnicodeCharacter0xe04c',
u'\u6a50': 'CjkUnifiedIdeograph6a50',
u'\u8ed3': 'CjkUnifiedIdeograph8ed3',
u'\u30d7': 'KatakanaLetterPu',
u'\ud356': 'HangulSyllablePyabs',
u'\uf7d9': 'UnicodeCharacter0xf7d9',
u'\u555a': 'CjkUnifiedIdeograph555a',
u'\u79dd': 'CjkUnifiedIdeograph79dd',
u'\u8260': 'CjkUnifiedIdeograph8260',
u'\ua6e3': 'UnicodeCharacter0xa6e3',
u'\u0464': 'CyrillicCapitalLetterIotifiedE',
u'\u28e7': 'BraillePatternDots123678',
u'\ucb66': 'HangulSyllableJjweogg',
u'\uefe9': 'UnicodeCharacter0xefe9',
u'\u4d6a': 'CjkUnifiedIdeograph4d6a',
u'\u91ed': 'CjkUnifiedIdeograph91ed',
u'\u1bf1': 'UnicodeCharacter0x1bf1',
u'\uba70': 'HangulSyllableMyeo',
u'\udef3': 'UnicodeCharacter0xdef3',
u'\u3c74': 'CjkUnifiedIdeograph3c74',
u'\u40f7': 'CjkUnifiedIdeograph40f7',
u'\ue376': 'UnicodeCharacter0xe376',
u'\u657a': 'CjkUnifiedIdeograph657a',
u'\u89fd': 'CjkUnifiedIdeograph89fd',
u'\ub301': 'HangulSyllableDaeg',
u'\ud280': 'HangulSyllableTwi',
u'\u3505': 'CjkUnifiedIdeograph3505',
u'\u5484': 'CjkUnifiedIdeograph5484',
u'\uf807': 'UnicodeCharacter0xf807',
u'\u620b': 'CjkUnifiedIdeograph620b',
u'\u9d8a': 'CjkUnifiedIdeograph9d8a',
u'\u078e': 'ThaanaLetterGaafu',
u'\uab11': 'UnicodeCharacter0xab11',
u'\uca90': 'HangulSyllableJjyeols',
u'\u2d15': 'GeorgianSmallLetterKhar',
u'\u4c94': 'CjkUnifiedIdeograph4c94',
u'\u9a1b': 'CjkUnifiedIdeograph9a1b',
u'\ub59a': 'HangulSyllableDdyaej',
u'\u1c1f': 'LepchaLetterVa',
u'\u3f9e': 'CjkUnifiedIdeograph3f9e',
u'\uc321': 'HangulSyllableSsaelt',
u'\ue2a0': 'UnicodeCharacter0xe2a0',
u'\u4525': 'CjkUnifiedIdeograph4525',
u'\u64a4': 'CjkUnifiedIdeograph64a4',
u'\ub22b': 'HangulSyllableNweolb',
u'\uadaa': 'HangulSyllableGwenh',
u'\u342f': 'CjkUnifiedIdeograph342f',
u'\u57ae': 'CjkUnifiedIdeograph57ae',
u'\ufb31': 'HebrewLetterBetWithDagesh',
u'\u7d35': 'CjkUnifiedIdeograph7d35',
u'\u9cb4': 'CjkUnifiedIdeograph9cb4',
u'\u06b8': 'ArabicLetterLamWithThreeDotsBelow',
u'\uaa3b': 'UnicodeCharacter0xaa3b',
u'\uc5ba': 'HangulSyllableEonh',
u'\u2c3f': 'GlagoliticSmallLetterMyslite',
u'\u4fbe': 'CjkUnifiedIdeograph4fbe',
u'\u9545': 'CjkUnifiedIdeograph9545',
u'\ub4c4': 'HangulSyllableDyun',
u'\u1f49': 'GreekCapitalLetterOmicronWithDasia',
u'\u3ec8': 'CjkUnifiedIdeograph3ec8',
u'\uc24b': 'HangulSyllableSweoc',
u'\ufdca': 'UnicodeCharacter0xfdca',
u'\u444f': 'CjkUnifiedIdeograph444f',
u'\u67ce': 'CjkUnifiedIdeograph67ce',
u'\u8d55': 'CjkUnifiedIdeograph8d55',
u'\uacd4': 'HangulSyllableGyem',
u'\u3759': 'CjkUnifiedIdeograph3759',
u'\u56d8': 'CjkUnifiedIdeograph56d8',
u'\ufa5b': 'CjkCompatibilityIdeographfa5b',
u'\u7c5f': 'CjkUnifiedIdeograph7c5f',
u'\u9fde': 'UnicodeCharacter0x9fde',
u'\u01e2': 'LatinCapitalLetterAeWithMacron',
u'\ua565': 'VaiSyllableZha',
u'\uc4e4': 'HangulSyllableSsyum',
u'\u2f69': 'KangxiRadicalWhite',
u'\u4ee8': 'CjkUnifiedIdeograph4ee8',
u'\u946f': 'CjkUnifiedIdeograph946f',
u'\ub7ee': 'HangulSyllableReogg',
u'\u1e73': 'LatinSmallLetterUWithDiaeresisBelow',
u'\u39f2': 'CjkUnifiedIdeograph39f2',
u'\udd75': 'UnicodeCharacter0xdd75',
u'\ufcf4': 'ArabicLigatureShaddaWithKasraMedialForm',
u'\u4779': 'CjkUnifiedIdeograph4779',
u'\u66f8': 'CjkUnifiedIdeograph66f8',
u'\u8c7f': 'CjkUnifiedIdeograph8c7f',
u'\uaffe': 'HangulSyllableGgwelp',
u'\uda06': 'UnicodeCharacter0xda06',
u'\ufe89': 'ArabicLetterYehWithHamzaAboveIsolatedForm',
u'\u9f08': 'CjkUnifiedIdeograph9f08',
u'\ua38b': 'YiSyllableRrup',
u'\u5c0a': 'CjkUnifiedIdeograph5c0a',
u'\u608d': 'CjkUnifiedIdeograph608d',
u'\u010c': 'LatinCapitalLetterCWithCaron',
u'\u258f': 'LeftOneEighthBlock',
u'\uf216': 'UnicodeCharacter0xf216',
u'\ub718': 'HangulSyllableDdyuls',
u'\udb9b': 'UnicodeCharacter0xdb9b',
u'\u741a': 'CjkUnifiedIdeograph741a',
u'\u989d': 'CjkUnifiedIdeograph989d',
u'\u391c': 'CjkUnifiedIdeograph391c',
u'\u5d9f': 'CjkUnifiedIdeograph5d9f',
u'\u02a1': 'LatinLetterGlottalStopWithStroke',
u'\uea26': 'UnicodeCharacter0xea26',
u'\uaf28': 'HangulSyllableGgyek',
u'\uf3ab': 'UnicodeCharacter0xf3ab',
u'\u6c2a': 'CjkUnifiedIdeograph6c2a',
u'\ub0ad': 'HangulSyllableNang',
u'\u512c': 'CjkUnifiedIdeograph512c',
u'\u75af': 'CjkUnifiedIdeograph75af',
u'\u162e': 'CanadianSyllabicsCarrierLhu',
u'\u3ab1': 'CjkUnifiedIdeograph3ab1',
u'\uc738': 'HangulSyllableYuk',
u'\uebbb': 'UnicodeCharacter0xebbb',
u'\u843a': 'CjkUnifiedIdeograph843a',
u'\ua8bd': 'SaurashtraVowelSignVocalicLl',
u'\u493c': 'CjkUnifiedIdeograph493c',
u'\u6dbf': 'CjkUnifiedIdeograph6dbf',
u'\u0e3e': 'UnicodeCharacter0xe3e',
u'\u52c1': 'CjkUnifiedIdeograph52c1',
u'\u17c3': 'KhmerVowelSignAi',
u'\uff48': 'FullwidthLatinSmallLetterH',
u'\ubc4a': 'HangulSyllableBaep',
u'\uc0cd': 'HangulSyllableSaenj',
u'\u614c': 'CjkUnifiedIdeograph614c',
u'\u85cf': 'CjkUnifiedIdeograph85cf',
u'\u264e': 'Libra',
u'\u4ad1': 'CjkUnifiedIdeograph4ad1',
u'\u0fd3': 'TibetanMarkInitialBrdaRnyingYigMgoMdunMa',
u'\ud45a': 'HangulSyllablePoep',
u'\uf8dd': 'UnicodeCharacter0xf8dd',
u'\u995c': 'CjkUnifiedIdeograph995c',
u'\ubddf': 'HangulSyllableBwilb',
u'\u5e5e': 'CjkUnifiedIdeograph5e5e',
u'\u62e1': 'CjkUnifiedIdeograph62e1',
u'\u0360': 'CombiningDoubleTilde',
u'\u27e3': 'WhiteConcavesidedDiamondWithRightwardsTick',
u'\ucc6a': 'HangulSyllableCyalm',
u'\ub16c': 'HangulSyllableNyem',
u'\ud5ef': 'HangulSyllableHelb',
u'\u766e': 'CjkUnifiedIdeograph766e',
u'\u9af1': 'CjkUnifiedIdeograph9af1',
u'\u3b70': 'CjkUnifiedIdeograph3b70',
u'\u5ff3': 'CjkUnifiedIdeograph5ff3',
u'\u1cf5': 'UnicodeCharacter0x1cf5',
u'\ue47a': 'UnicodeCharacter0xe47a',
u'\ua97c': 'UnicodeCharacter0xa97c',
u'\ucdff': 'HangulSyllableCwic',
u'\u6e7e': 'CjkUnifiedIdeograph6e7e',
u'\u5380': 'CjkUnifiedIdeograph5380',
u'\uf703': 'UnicodeCharacter0xf703',
u'\u1082': 'MyanmarConsonantSignShanMedialWa',
u'\ub405': 'HangulSyllableDwaelg',
u'\u7907': 'CjkUnifiedIdeograph7907',
u'\u3e09': 'CjkUnifiedIdeograph3e09',
u'\uc18c': 'HangulSyllableSo',
u'\u868e': 'CjkUnifiedIdeograph868e',
u'\u4b90': 'CjkUnifiedIdeograph4b90',
u'\uef13': 'UnicodeCharacter0xef13',
u'\u0892': 'UnicodeCharacter0x892',
u'\uac15': 'HangulSyllableGang',
u'\u9117': 'CjkUnifiedIdeograph9117',
u'\u5619': 'CjkUnifiedIdeograph5619',
u'\u1b1b': 'BalineseLetterJaJera',
u'\uf99c': 'CjkCompatibilityIdeographf99c',
u'\ube9e': 'HangulSyllableBbyanh',
u'\u63a0': 'CjkUnifiedIdeograph63a0',
u'\u20a2': 'CruzeiroSign',
u'\uc425': 'HangulSyllableSswaeng',
u'\u8927': 'CjkUnifiedIdeograph8927',
u'\u4e29': 'CjkUnifiedIdeograph4e29',
u'\u332b': 'SquarePaasento',
u'\ud6ae': 'HangulSyllableHyonh',
u'\u9bb0': 'CjkUnifiedIdeograph9bb0',
u'\u58b2': 'CjkUnifiedIdeograph58b2',
u'\ufc35': 'ArabicLigatureQafWithAlefMaksuraIsolatedForm',
u'\u1db4': 'ModifierLetterSmallEsh',
u'\ua137': 'YiSyllableDdix',
u'\u6639': 'CjkUnifiedIdeograph6639',
u'\u2b3b': 'LeftwardsTwoheadedArrowWithTail',
u'\ucebe': 'HangulSyllableKyabs',
u'\ub3c0': 'HangulSyllableDyek',
u'\u70c2': 'CjkUnifiedIdeograph70c2',
u'\u35c4': 'CjkUnifiedIdeograph35c4',
u'\ud947': 'UnicodeCharacter0xd947',
u'\u9e49': 'CjkUnifiedIdeograph9e49',
u'\u434b': 'CjkUnifiedIdeograph434b',
u'\ue6ce': 'UnicodeCharacter0xe6ce',
u'\uabd0': 'UnicodeCharacter0xabd0',
u'\u68d2': 'CjkUnifiedIdeograph68d2',
u'\u2dd4': 'EthiopicSyllableXyee',
u'\uf157': 'UnicodeCharacter0xf157',
u'\u12d6': 'EthiopicSyllablePharyngealO',
u'\ub659': 'HangulSyllableDdwaeb',
u'\u7b5b': 'CjkUnifiedIdeograph7b5b',
u'\u385d': 'CjkUnifiedIdeograph385d',
u'\uc3e0': 'HangulSyllableSsol',
u'\u80e2': 'CjkUnifiedIdeograph80e2',
u'\u45e4': 'CjkUnifiedIdeograph45e4',
u'\ue967': 'UnicodeCharacter0xe967',
u'\u0ae6': 'GujaratiDigitZero',
u'\uae69': 'HangulSyllableGgaeg',
u'\u936b': 'CjkUnifiedIdeograph936b',
u'\u506d': 'CjkUnifiedIdeograph506d',
u'\u156f': 'CanadianSyllabicsTth',
u'\ufbf0': 'ArabicLigatureYehWithHamzaAboveWithUIsolatedForm',
u'\ub8f2': 'HangulSyllableRulm',
u'\u7df4': 'CjkUnifiedIdeograph7df4',
u'\u22f6': 'ElementOfWithOverbar',
u'\uc679': 'HangulSyllableOeg',
u'\u8b7b': 'CjkUnifiedIdeograph8b7b',
u'\u487d': 'CjkUnifiedIdeograph487d',
u'\u0d7f': 'MalayalamLetterChilluK',
u'\u7581': 'CjkUnifiedIdeograph7581',
u'\u9204': 'CjkUnifiedIdeograph9204',
u'\ube87': 'HangulSyllableBbaelb',
u'\u1808': 'MongolianManchuComma',
u'\u248b': 'DigitFourFullStop',
u'\udb0a': 'UnicodeCharacter0xdb0a',
u'\ue78d': 'UnicodeCharacter0xe78d',
u'\u410e': 'CjkUnifiedIdeograph410e',
u'\u6d91': 'CjkUnifiedIdeograph6d91',
u'\u8a14': 'CjkUnifiedIdeograph8a14',
u'\ud697': 'HangulSyllableHoelb',
u'\u3018': 'LeftWhiteTortoiseShellBracket',
u'\u5c9b': 'CjkUnifiedIdeograph5c9b',
u'\uf31a': 'UnicodeCharacter0xf31a',
u'\u791e': 'CjkUnifiedIdeograph791e',
u'\u85a1': 'CjkUnifiedIdeograph85a1',
u'\u03a5': 'GreekCapitalLetterUpsilon',
u'\ua224': 'YiSyllableGgo',
u'\ucea7': 'HangulSyllableKaec',
u'\u2828': 'BraillePatternDots46',
u'\u74ab': 'CjkUnifiedIdeograph74ab',
u'\ueb2a': 'UnicodeCharacter0xeb2a',
u'\u912e': 'CjkUnifiedIdeograph912e',
u'\ubdb1': 'HangulSyllableBweong',
u'\u1f32': 'GreekSmallLetterIotaWithPsiliAndVaria',
u'\u3bb5': 'CjkUnifiedIdeograph3bb5',
u'\uda34': 'UnicodeCharacter0xda34',
u'\ue6b7': 'UnicodeCharacter0xe6b7',
u'\u4038': 'CjkUnifiedIdeograph4038',
u'\u6cbb': 'CjkUnifiedIdeograph6cbb',
u'\u893e': 'CjkUnifiedIdeograph893e',
u'\ud5c1': 'HangulSyllableHyaeng',
u'\u3742': 'CjkUnifiedIdeograph3742',
u'\u53c5': 'CjkUnifiedIdeograph53c5',
u'\uf244': 'UnicodeCharacter0xf244',
u'\u7848': 'CjkUnifiedIdeograph7848',
u'\u84cb': 'CjkUnifiedIdeograph84cb',
u'\u02cf': 'ModifierLetterLowAcuteAccent',
u'\ua14e': 'YiSyllableDdup',
u'\ucdd1': 'HangulSyllableCwenj',
u'\u2f52': 'KangxiRadicalClan',
u'\u4bd5': 'CjkUnifiedIdeograph4bd5',
u'\uea54': 'UnicodeCharacter0xea54',
u'\u9058': 'CjkUnifiedIdeograph9058',
u'\ubcdb': 'HangulSyllableByegs',
u'\u1e5c': 'LatinCapitalLetterRWithDotBelowAndMacron',
u'\u3adf': 'CjkUnifiedIdeograph3adf',
u'\ud95e': 'UnicodeCharacter0xd95e',
u'\ue5e1': 'UnicodeCharacter0xe5e1',
u'\u4762': 'CjkUnifiedIdeograph4762',
u'\u63e5': 'CjkUnifiedIdeograph63e5',
u'\u8868': 'CjkUnifiedIdeograph8868',
u'\ud4eb': 'HangulSyllablePyugs',
u'\u366c': 'CjkUnifiedIdeograph366c',
u'\u52ef': 'CjkUnifiedIdeograph52ef',
u'\uf16e': 'UnicodeCharacter0xf16e',
u'\u7f72': 'CjkUnifiedIdeograph7f72',
u'\u9bf5': 'CjkUnifiedIdeograph9bf5',
u'\u01f9': 'LatinSmallLetterNWithGrave',
u'\ua078': 'YiSyllableNbi',
u'\uccfb': 'HangulSyllableCyelh',
u'\u2e7c': 'UnicodeCharacter0x2e7c',
u'\u4aff': 'CjkUnifiedIdeograph4aff',
u'\ue97e': 'UnicodeCharacter0xe97e',
u'\u7003': 'CjkUnifiedIdeograph7003',
u'\u9782': 'CjkUnifiedIdeograph9782',
u'\u1d86': 'LatinSmallLetterMWithPalatalHook',
u'\ub909': 'HangulSyllableRweonj',
u'\ud888': 'UnicodeCharacter0xd888',
u'\u270d': 'WritingHand',
u'\u468c': 'CjkUnifiedIdeograph468c',
u'\ue20f': 'UnicodeCharacter0xe20f',
u'\u6813': 'CjkUnifiedIdeograph6813',
u'\u8f92': 'CjkUnifiedIdeograph8f92',
u'\u1617': 'CanadianSyllabicsCarrierJe',
u'\u3596': 'CjkUnifiedIdeograph3596',
u'\ud119': 'HangulSyllableTyaenj',
u'\uf098': 'UnicodeCharacter0xf098',
u'\u5f1d': 'CjkUnifiedIdeograph5f1d',
u'\u7e9c': 'CjkUnifiedIdeograph7e9c',
u'\u8023': 'CjkUnifiedIdeograph8023',
u'\ua7a2': 'UnicodeCharacter0xa7a2',
u'\u0e27': 'ThaiCharacterWoWaen',
u'\u2da6': 'EthiopicSyllableSso',
u'\uc929': 'HangulSyllableJweob',
u'\ue8a8': 'UnicodeCharacter0xe8a8',
u'\u772d': 'CjkUnifiedIdeograph772d',
u'\u96ac': 'CjkUnifiedIdeograph96ac',
u'\u1cb0': 'UnicodeCharacter0x1cb0',
u'\ub833': 'HangulSyllableRyeolh',
u'\udfb2': 'UnicodeCharacter0xdfb2',
u'\u2637': 'TrigramForEarth',
u'\u45b6': 'CjkUnifiedIdeograph45b6',
u'\ue139': 'UnicodeCharacter0xe139',
u'\u6f3d': 'CjkUnifiedIdeograph6f3d',
u'\u8ebc': 'CjkUnifiedIdeograph8ebc',
u'\u1541': 'CanadianSyllabicsSayisiYi',
u'\u34c0': 'CjkUnifiedIdeograph34c0',
u'\ud043': 'HangulSyllableKwilh',
u'\uf7c2': 'UnicodeCharacter0xf7c2',
u'\u5e47': 'CjkUnifiedIdeograph5e47',
u'\u7dc6': 'CjkUnifiedIdeograph7dc6',
u'\u874d': 'CjkUnifiedIdeograph874d',
u'\ua6cc': 'UnicodeCharacter0xa6cc',
u'\u0d51': 'UnicodeCharacter0xd51',
u'\u2cd0': 'CopticCapitalLetterLshapedHa',
u'\uc853': 'HangulSyllableJyeoh',
u'\uefd2': 'UnicodeCharacter0xefd2',
u'\u7657': 'CjkUnifiedIdeograph7657',
u'\u95d6': 'CjkUnifiedIdeograph95d6',
u'\u13da': 'CherokeeLetterDu',
u'\ubf5d': 'HangulSyllableBbwag',
u'\udedc': 'UnicodeCharacter0xdedc',
u'\u2561': 'BoxDrawingsVerticalSingleAndLeftDouble',
u'\u44e0': 'CjkUnifiedIdeograph44e0',
u'\ue063': 'UnicodeCharacter0xe063',
u'\u6e67': 'CjkUnifiedIdeograph6e67',
u'\u8de6': 'CjkUnifiedIdeograph8de6',
u'\u146b': 'CanadianSyllabicsKe',
u'\u0bea': 'TamilDigitFour',
u'\ud76d': 'HangulSyllableHyig',
u'\uf6ec': 'UnicodeCharacter0xf6ec',
u'\u5d71': 'CjkUnifiedIdeograph5d71',
u'\u7cf0': 'CjkUnifiedIdeograph7cf0',
u'\u8677': 'CjkUnifiedIdeograph8677',
u'\ua5f6': 'VaiSyllableDhhe',
u'\u0c7b': 'TeluguFractionDigitThreeForOddPowersOfFour',
u'\u23fa': 'UnicodeCharacter0x23fa',
u'\ucf7d': 'HangulSyllableKwalt',
u'\ueefc': 'UnicodeCharacter0xeefc',
u'\uf481': 'UnicodeCharacter0xf481',
u'\u9500': 'CjkUnifiedIdeograph9500',
u'\ub183': 'HangulSyllableNolb',
u'\u5602': 'CjkUnifiedIdeograph5602',
u'\u7285': 'CjkUnifiedIdeograph7285',
u'\u1304': 'EthiopicSyllableJee',
u'\u3f87': 'CjkUnifiedIdeograph3f87',
u'\uc00e': 'HangulSyllableBbwelm',
u'\uec91': 'UnicodeCharacter0xec91',
u'\u8d10': 'CjkUnifiedIdeograph8d10',
u'\ua993': 'UnicodeCharacter0xa993',
u'\u4e12': 'CjkUnifiedIdeograph4e12',
u'\u6a95': 'CjkUnifiedIdeograph6a95',
u'\u0b14': 'OriyaLetterAu',
u'\u5797': 'CjkUnifiedIdeograph5797',
u'\u1099': 'MyanmarShanDigitNine',
u'\uf81e': 'UnicodeCharacter0xf81e',
u'\ua520': 'VaiSyllableGee',
u'\uc1a3': 'HangulSyllableSoc',
u'\u6622': 'CjkUnifiedIdeograph6622',
u'\u82a5': 'CjkUnifiedIdeograph82a5',
u'\u2324': 'UpArrowheadBetweenTwoHorizontalBars',
u'\u4fa7': 'CjkUnifiedIdeograph4fa7',
u'\u08a9': 'UnicodeCharacter0x8a9',
u'\udd30': 'UnicodeCharacter0xdd30',
u'\uf9b3': 'CjkCompatibilityIdeographf9b3',
u'\u9e32': 'CjkUnifiedIdeograph9e32',
u'\ubab5': 'HangulSyllableMolt',
u'\u5b34': 'CjkUnifiedIdeograph5b34',
u'\u67b7': 'CjkUnifiedIdeograph67b7',
u'\u0436': 'CyrillicSmallLetterZhe',
u'\u20b9': 'UnicodeCharacter0x20b9',
u'\uf540': 'UnicodeCharacter0xf540',
u'\ub642': 'HangulSyllableDdwaj',
u'\ud2c5': 'HangulSyllableTeult',
u'\u7344': 'CjkUnifiedIdeograph7344',
u'\u9fc7': 'UnicodeCharacter0x9fc7',
u'\u3c46': 'CjkUnifiedIdeograph3c46',
u'\u58c9': 'CjkUnifiedIdeograph58c9',
u'\u05cb': 'UnicodeCharacter0x5cb',
u'\ued50': 'UnicodeCharacter0xed50',
u'\uae52': 'HangulSyllableGganh',
u'\ucad5': 'HangulSyllableJjot',
u'\u6b54': 'CjkUnifiedIdeograph6b54',
u'\ub7d7': 'HangulSyllableRyaed',
u'\u5456': 'CjkUnifiedIdeograph5456',
u'\u70d9': 'CjkUnifiedIdeograph70d9',
u'\u1158': 'HangulChoseongSsanghieuh',
u'\u3ddb': 'CjkUnifiedIdeograph3ddb',
u'\uc662': 'HangulSyllableWaenh',
u'\ue2e5': 'UnicodeCharacter0xe2e5',
u'\u8364': 'CjkUnifiedIdeograph8364',
u'\uafe7': 'HangulSyllableGgweos',
u'\u4c66': 'CjkUnifiedIdeograph4c66',
u'\u68e9': 'CjkUnifiedIdeograph68e9',
u'\u0968': 'DevanagariDigitTwo',
u'\u55eb': 'CjkUnifiedIdeograph55eb',
u'\u16ed': 'RunicCrossPunctuation',
u'\ufe72': 'ArabicDammatanIsolatedForm',
u'\ubb74': 'HangulSyllableMwel',
u'\uc7f7': 'HangulSyllableJyaes',
u'\u6476': 'CjkUnifiedIdeograph6476',
u'\u80f9': 'CjkUnifiedIdeograph80f9',
u'\u2178': 'SmallRomanNumeralNine',
u'\u4dfb': 'HexagramForLimitation',
u'\u0efd': 'UnicodeCharacter0xefd',
u'\u3401': 'CjkUnifiedIdeograph3401',
u'\ud384': 'HangulSyllablePeol',
u'\u9c86': 'CjkUnifiedIdeograph9c86',
u'\u5988': 'CjkUnifiedIdeograph5988',
u'\ue50b': 'UnicodeCharacter0xe50b',
u'\u1a8a': 'UnicodeCharacter0x1a8a',
u'\ua60d': 'VaiComma',
u'\u630f': 'CjkUnifiedIdeograph630f',
u'\u2c11': 'GlagoliticCapitalLetterOnu',
u'\ucb94': 'HangulSyllableJjwess',
u'\ub496': 'HangulSyllableDwelp',
u'\u7198': 'CjkUnifiedIdeograph7198',
u'\u329a': 'CircledIdeographMale',
u'\ude1d': 'UnicodeCharacter0xde1d',
u'\u9b1f': 'CjkUnifiedIdeograph9b1f',
u'\u4421': 'CjkUnifiedIdeograph4421',
u'\u0123': 'LatinSmallLetterGWithCedilla',
u'\ue3a4': 'UnicodeCharacter0xe3a4',
u'\uaca6': 'HangulSyllableGep',
u'\u69a8': 'CjkUnifiedIdeograph69a8',
u'\u2aaa': 'SmallerThan',
u'\uf62d': 'UnicodeCharacter0xf62d',
u'\u17ac': 'KhmerIndependentVowelRyy',
u'\ub32f': 'HangulSyllableDyas',
u'\u7c31': 'CjkUnifiedIdeograph7c31',
u'\u3933': 'CjkUnifiedIdeograph3933',
u'\uc4b6': 'HangulSyllableSswep',
u'\u81b8': 'CjkUnifiedIdeograph81b8',
u'\u42ba': 'CjkUnifiedIdeograph42ba',
u'\uee3d': 'UnicodeCharacter0xee3d',
u'\u0fbc': 'TibetanSubjoinedLetterFixedformRa',
u'\uab3f': 'UnicodeCharacter0xab3f',
u'\u9441': 'CjkUnifiedIdeograph9441',
u'\u5143': 'CjkUnifiedIdeograph5143',
u'\u1245': 'EthiopicSyllableQe',
u'\ufcc6': 'ArabicLigatureKafWithKhahInitialForm',
u'\ub9c8': 'HangulSyllableMa',
u'\u7aca': 'CjkUnifiedIdeograph7aca',
u'\u27cc': 'LongDivision',
u'\uc34f': 'HangulSyllableSsyaegs',
u'\u8c51': 'CjkUnifiedIdeograph8c51',
u'\u4953': 'CjkUnifiedIdeograph4953',
u'\u0a55': 'UnicodeCharacter0xa55',
u'\ud1d8': 'HangulSyllableTwae',
u'\u92da': 'CjkUnifiedIdeograph92da',
u'\u5fdc': 'CjkUnifiedIdeograph5fdc',
u'\ufb5f': 'ArabicLetterTtehehFinalForm',
u'\u18de': 'UnicodeCharacter0x18de',
u'\ua461': 'YiSyllableXiet',
u'\u6163': 'CjkUnifiedIdeograph6163',
u'\u2265': 'GreaterthanOrEqualTo',
u'\uc9e8': 'HangulSyllableJjals',
u'\u8aea': 'CjkUnifiedIdeograph8aea',
u'\u77ec': 'CjkUnifiedIdeograph77ec',
u'\u30ee': 'KatakanaLetterSmallWa',
u'\udc71': 'UnicodeCharacter0xdc71',
u'\u9973': 'CjkUnifiedIdeograph9973',
u'\u5a75': 'CjkUnifiedIdeograph5a75',
u'\u0777': 'ArabicLetterFarsiYehWithExtendedArabicindicDigitFourBelow',
u'\ue1f8': 'UnicodeCharacter0xe1f8',
u'\ua2fa': 'YiSyllableNzox',
u'\u6ffc': 'CjkUnifiedIdeograph6ffc',
u'\u28fe': 'BraillePatternDots2345678',
u'\u1600': 'CanadianSyllabicsCarrierKka',
u'\u3283': 'CircledIdeographFour',
u'\ud502': 'HangulSyllablePyup',
u'\uf185': 'UnicodeCharacter0xf185',
u'\u5f06': 'CjkUnifiedIdeograph5f06',
u'\u7b89': 'CjkUnifiedIdeograph7b89',
u'\u840c': 'CjkUnifiedIdeograph840c',
u'\ua08f': 'YiSyllableNbyrx',
u'\u0e10': 'ThaiCharacterThoThan',
u'\u2a93': 'LessthanAboveSlantedEqualAboveGreaterthanAboveSlantedEqual',
u'\ucd12': 'HangulSyllableColm',
u'\ue995': 'UnicodeCharacter0xe995',
u'\u7716': 'CjkUnifiedIdeograph7716',
u'\u9399': 'CjkUnifiedIdeograph9399',
u'\u1d9d': 'ModifierLetterSmallCWithCurl',
u'\ubc1c': 'HangulSyllableBal',
u'\ud89f': 'UnicodeCharacter0xd89f',
u'\u2620': 'SkullAndCrossbones',
u'\u42a3': 'CjkUnifiedIdeograph42a3',
u'\ue522': 'UnicodeCharacter0xe522',
u'\u6f26': 'CjkUnifiedIdeograph6f26',
u'\u8ba9': 'CjkUnifiedIdeograph8ba9',
u'\u35ad': 'CjkUnifiedIdeograph35ad',
u'\ud42c': 'HangulSyllablePwael',
u'\uf0af': 'UnicodeCharacter0xf0af',
u'\u5e30': 'CjkUnifiedIdeograph5e30',
u'\u7ab3': 'CjkUnifiedIdeograph7ab3',
u'\u8736': 'CjkUnifiedIdeograph8736',
u'\ua3b9': 'YiSyllableShep',
u'\u013a': 'LatinSmallLetterLWithAcute',
u'\u2dbd': 'EthiopicSyllableCche',
u'\ucc3c': 'HangulSyllableCass',
u'\ue8bf': 'UnicodeCharacter0xe8bf',
u'\u7640': 'CjkUnifiedIdeograph7640',
u'\u92c3': 'CjkUnifiedIdeograph92c3',
u'\u1cc7': 'UnicodeCharacter0x1cc7',
u'\ubf46': 'HangulSyllableBbonh',
u'\udbc9': 'UnicodeCharacter0xdbc9',
u'\u394a': 'CjkUnifiedIdeograph394a',
u'\u45cd': 'CjkUnifiedIdeograph45cd',
u'\ue44c': 'UnicodeCharacter0xe44c',
u'\u6e50': 'CjkUnifiedIdeograph6e50',
u'\u8ad3': 'CjkUnifiedIdeograph8ad3',
u'\u34d7': 'CjkUnifiedIdeograph34d7',
u'\ud756': 'HangulSyllableHeunh',
u'\uf3d9': 'UnicodeCharacter0xf3d9',
u'\u515a': 'CjkUnifiedIdeograph515a',
u'\u7ddd': 'CjkUnifiedIdeograph7ddd',
u'\u8660': 'CjkUnifiedIdeograph8660',
u'\ua2e3': 'YiSyllableZzu',
u'\u2ce7': 'CopticSymbolStauros',
u'\ucf66': 'HangulSyllableKobs',
u'\uebe9': 'UnicodeCharacter0xebe9',
u'\u496a': 'CjkUnifiedIdeograph496a',
u'\u95ed': 'CjkUnifiedIdeograph95ed',
u'\u1ff1': 'UnicodeCharacter0x1ff1',
u'\ube70': 'HangulSyllableBbam',
u'\udaf3': 'UnicodeCharacter0xdaf3',
u'\u3874': 'CjkUnifiedIdeograph3874',
u'\u44f7': 'CjkUnifiedIdeograph44f7',
u'\ue776': 'UnicodeCharacter0xe776',
u'\u617a': 'CjkUnifiedIdeograph617a',
u'\u8dfd': 'CjkUnifiedIdeograph8dfd',
u'\ub701': 'HangulSyllableDdwib',
u'\ud680': 'HangulSyllableHwaem',
u'\u3105': 'BopomofoLetterB',
u'\u5084': 'CjkUnifiedIdeograph5084',
u'\ufc07': 'ArabicLigatureBehWithKhahIsolatedForm',
u'\u660b': 'CjkUnifiedIdeograph660b',
u'\u998a': 'CjkUnifiedIdeograph998a',
u'\u038e': 'GreekCapitalLetterUpsilonWithTonos',
u'\uaf11': 'HangulSyllableGgyeg',
u'\uce90': 'HangulSyllableKae',
u'\u2915': 'RightwardsArrowWithTailWithDoubleVerticalStroke',
u'\u4894': 'CjkUnifiedIdeograph4894',
u'\u9e1b': 'CjkUnifiedIdeograph9e1b',
u'\ub19a': 'HangulSyllableNwanh',
u'\u181f': 'UnicodeCharacter0x181f',
u'\u3b9e': 'CjkUnifiedIdeograph3b9e',
u'\uc721': 'HangulSyllableYug',
u'\ue6a0': 'UnicodeCharacter0xe6a0',
u'\u4125': 'CjkUnifiedIdeograph4125',
u'\u60a4': 'CjkUnifiedIdeograph60a4',
u'\ub62b': 'HangulSyllableDdoh',
u'\ua9aa': 'UnicodeCharacter0xa9aa',
u'\u302f': 'HangulDoubleDotToneMark',
u'\u53ae': 'CjkUnifiedIdeograph53ae',
u'\uff31': 'FullwidthLatinCapitalLetterQ',
u'\u7935': 'CjkUnifiedIdeograph7935',
u'\u98b4': 'CjkUnifiedIdeograph98b4',
u'\u02b8': 'ModifierLetterSmallY',
u'\uae3b': 'HangulSyllableGilb',
u'\uc1ba': 'HangulSyllableSwabs',
u'\u283f': 'BraillePatternDots123456',
u'\u4bbe': 'CjkUnifiedIdeograph4bbe',
u'\u9145': 'CjkUnifiedIdeograph9145',
u'\ub0c4': 'HangulSyllableNaem',
u'\u1b49': 'BalineseLetterVeSasak',
u'\u3ac8': 'CjkUnifiedIdeograph3ac8',
u'\uc64b': 'HangulSyllableWalb',
u'\uf9ca': 'CjkCompatibilityIdeographf9ca',
u'\u404f': 'CjkUnifiedIdeograph404f',
u'\u63ce': 'CjkUnifiedIdeograph63ce',
u'\u8955': 'CjkUnifiedIdeograph8955',
u'\ua8d4': 'SaurashtraDigitFour',
u'\u3359': 'IdeographicTelegraphSymbolForHourOne',
u'\u52d8': 'CjkUnifiedIdeograph52d8',
u'\ufe5b': 'SmallLeftCurlyBracket',
u'\u785f': 'CjkUnifiedIdeograph785f',
u'\u9bde': 'CjkUnifiedIdeograph9bde',
u'\u05e2': 'HebrewLetterAyin',
u'\ua165': 'YiSyllableNdup',
u'\uc0e4': 'HangulSyllableSya',
u'\u2b69': 'UnicodeCharacter0x2b69',
u'\u4ae8': 'CjkUnifiedIdeograph4ae8',
u'\u906f': 'CjkUnifiedIdeograph906f',
u'\ub3ee': 'HangulSyllableDwalp',
u'\u1a73': 'UnicodeCharacter0x1a73',
u'\u3df2': 'CjkUnifiedIdeograph3df2',
u'\ud975': 'UnicodeCharacter0xd975',
u'\uf8f4': 'UnicodeCharacter0xf8f4',
u'\u4379': 'CjkUnifiedIdeograph4379',
u'\u62f8': 'CjkUnifiedIdeograph62f8',
u'\u887f': 'CjkUnifiedIdeograph887f',
u'\uabfe': 'UnicodeCharacter0xabfe',
u'\ude06': 'UnicodeCharacter0xde06',
u'\ufa89': 'CjkCompatibilityIdeographfa89',
u'\u9b08': 'CjkUnifiedIdeograph9b08',
u'\ua78b': 'LatinCapitalLetterSaltillo',
u'\u580a': 'CjkUnifiedIdeograph580a',
u'\u648d': 'CjkUnifiedIdeograph648d',
u'\u050c': 'CyrillicCapitalLetterKomiSje',
u'\u218f': 'UnicodeCharacter0x218f',
u'\uf616': 'UnicodeCharacter0xf616',
u'\ub318': 'HangulSyllableDaek',
u'\udf9b': 'UnicodeCharacter0xdf9b',
u'\u701a': 'CjkUnifiedIdeograph701a',
u'\u9c9d': 'CjkUnifiedIdeograph9c9d',
u'\u3d1c': 'CjkUnifiedIdeograph3d1c',
u'\u599f': 'CjkUnifiedIdeograph599f',
u'\u06a1': 'ArabicLetterDotlessFeh',
u'\uee26': 'UnicodeCharacter0xee26',
u'\uab28': 'UnicodeCharacter0xab28',
u'\uf7ab': 'UnicodeCharacter0xf7ab',
u'\u682a': 'CjkUnifiedIdeograph682a',
u'\ub4ad': 'HangulSyllableDwilg',
u'\u552c': 'CjkUnifiedIdeograph552c',
u'\u71af': 'CjkUnifiedIdeograph71af',
u'\u122e': 'EthiopicSyllableRo',
u'\u3eb1': 'CjkUnifiedIdeograph3eb1',
u'\uc338': 'HangulSyllableSsyal',
u'\uefbb': 'UnicodeCharacter0xefbb',
u'\u803a': 'CjkUnifiedIdeograph803a',
u'\uacbd': 'HangulSyllableGyeong',
u'\u4d3c': 'CjkUnifiedIdeograph4d3c',
u'\u69bf': 'CjkUnifiedIdeograph69bf',
u'\u0a3e': 'GurmukhiVowelSignAa',
u'\u56c1': 'CjkUnifiedIdeograph56c1',
u'\u13c3': 'CherokeeLetterNo',
u'\ufb48': 'HebrewLetterReshWithDagesh',
u'\ub84a': 'HangulSyllableRyelm',
u'\uc4cd': 'HangulSyllableSswing',
u'\u654c': 'CjkUnifiedIdeograph654c',
u'\u81cf': 'CjkUnifiedIdeograph81cf',
u'\u224e': 'GeometricallyEquivalentTo',
u'\u4ed1': 'CjkUnifiedIdeograph4ed1',
u'\u0bd3': 'UnicodeCharacter0xbd3',
u'\ud05a': 'HangulSyllableKyulm',
u'\ufcdd': 'ArabicLigatureYehWithMeemInitialForm',
u'\u9d5c': 'CjkUnifiedIdeograph9d5c',
u'\ub9df': 'HangulSyllableMac',
u'\u5a5e': 'CjkUnifiedIdeograph5a5e',
u'\u66e1': 'CjkUnifiedIdeograph66e1',
u'\u0760': 'ArabicLetterFehWithTwoDotsBelow',
u'\u23e3': 'BenzeneRingWithCircle',
u'\uc86a': 'HangulSyllableJyej',
u'\ub56c': 'HangulSyllableDdyan',
u'\ud1ef': 'HangulSyllableTwaec',
u'\u726e': 'CjkUnifiedIdeograph726e',
u'\u9ef1': 'CjkUnifiedIdeograph9ef1',
u'\u3f70': 'CjkUnifiedIdeograph3f70',
u'\u5bf3': 'CjkUnifiedIdeograph5bf3',
u'\u18f5': 'UnicodeCharacter0x18f5',
u'\ue07a': 'UnicodeCharacter0xe07a',
u'\uad7c': 'HangulSyllableGum',
u'\uc9ff': 'HangulSyllableJjaed',
u'\u6a7e': 'CjkUnifiedIdeograph6a7e',
u'\u5780': 'CjkUnifiedIdeograph5780',
u'\uf303': 'UnicodeCharacter0xf303',
u'\u1482': 'CanadianSyllabicsNaskapiKwaa',
u'\ub005': 'HangulSyllableGgweng',
u'\u7d07': 'CjkUnifiedIdeograph7d07',
u'\u3a09': 'CjkUnifiedIdeograph3a09',
u'\uc58c': 'HangulSyllableYam',
u'\u828e': 'CjkUnifiedIdeograph828e',
u'\u4f90': 'CjkUnifiedIdeograph4f90',
u'\ueb13': 'UnicodeCharacter0xeb13',
u'\u0c92': 'KannadaLetterO',
u'\ua815': 'SylotiNagriLetterTho',
u'\u9517': 'CjkUnifiedIdeograph9517',
u'\u5219': 'CjkUnifiedIdeograph5219',
u'\u1f1b': 'GreekCapitalLetterEpsilonWithDasiaAndVaria',
u'\ufd9c': 'ArabicLigatureYehWithMeemWithMeemFinalForm',
u'\uba9e': 'HangulSyllableMyebs',
u'\u67a0': 'CjkUnifiedIdeograph67a0',
u'\u24a2': 'ParenthesizedLatinSmallLetterG',
u'\uc025': 'HangulSyllableBbwinj',
u'\u8d27': 'CjkUnifiedIdeograph8d27',
u'\u4a29': 'CjkUnifiedIdeograph4a29',
u'\u372b': 'CjkUnifiedIdeograph372b',
u'\ud2ae': 'HangulSyllableTyubs',
u'\u9fb0': 'CjkUnifiedIdeograph9fb0',
u'\u5cb2': 'CjkUnifiedIdeograph5cb2',
u'\uf835': 'UnicodeCharacter0xf835',
u'\u19b4': 'NewTaiLueVowelSignUu',
u'\ua537': 'VaiSyllableLi',
u'\u6239': 'CjkUnifiedIdeograph6239',
u'\u2f3b': 'KangxiRadicalStep',
u'\ucabe': 'HangulSyllableJjogg',
u'\ub7c0': 'HangulSyllableRyals',
u'\u74c2': 'CjkUnifiedIdeograph74c2',
u'\u31c4': 'CjkStrokeSw',
u'\udd47': 'UnicodeCharacter0xdd47',
u'\u9a49': 'CjkUnifiedIdeograph9a49',
u'\u474b': 'CjkUnifiedIdeograph474b',
u'\u044d': 'CyrillicSmallLetterE',
u'\ue2ce': 'UnicodeCharacter0xe2ce',
u'\uafd0': 'HangulSyllableGguk',
u'\u6cd2': 'CjkUnifiedIdeograph6cd2',
u'\u29d4': 'TimesWithLeftHalfBlack',
u'\uf557': 'UnicodeCharacter0xf557',
u'\u16d6': 'RunicLetterEhwazEhE',
u'\ub259': 'HangulSyllableNwig',
u'\u7f5b': 'CjkUnifiedIdeograph7f5b',
u'\u3c5d': 'CjkUnifiedIdeograph3c5d',
u'\uc7e0': 'HangulSyllableJyak',
u'\u84e2': 'CjkUnifiedIdeograph84e2',
u'\u41e4': 'CjkUnifiedIdeograph41e4',
u'\ued67': 'UnicodeCharacter0xed67',
u'\u0ee6': 'UnicodeCharacter0xee6',
u'\uaa69': 'UnicodeCharacter0xaa69',
u'\u976b': 'CjkUnifiedIdeograph976b',
u'\u546d': 'CjkUnifiedIdeograph546d',
u'\u116f': 'HangulJungseongWeo',
u'\ufff0': 'UnicodeCharacter0xfff0',
u'\ubcf2': 'HangulSyllableByep',
u'\u79f4': 'CjkUnifiedIdeograph79f4',
u'\u26f6': 'UnicodeCharacter0x26f6',
u'\uc279': 'HangulSyllableSwilt',
u'\u8f7b': 'CjkUnifiedIdeograph8f7b',
u'\u4c7d': 'CjkUnifiedIdeograph4c7d',
u'\u097f': 'DevanagariLetterBba',
u'\u7981': 'CjkUnifiedIdeograph7981',
u'\u9e04': 'CjkUnifiedIdeograph9e04',
u'\ub287': 'HangulSyllableNyus',
u'\u1408': 'CanadianSyllabicsCarrierEe',
u'\u288b': 'BraillePatternDots1248',
u'\ud70a': 'HangulSyllableHwelp',
u'\ueb8d': 'UnicodeCharacter0xeb8d',
u'\u4d0e': 'CjkUnifiedIdeograph4d0e',
u'\u6191': 'CjkUnifiedIdeograph6191',
u'\u8614': 'CjkUnifiedIdeograph8614',
u'\uda97': 'UnicodeCharacter0xda97',
u'\u3c18': 'CjkUnifiedIdeograph3c18',
u'\u509b': 'CjkUnifiedIdeograph509b',
u'\uff1a': 'FullwidthColon',
u'\u751e': 'CjkUnifiedIdeograph751e',
u'\u89a1': 'CjkUnifiedIdeograph89a1',
u'\u0fa5': 'TibetanSubjoinedLetterPha',
u'\uae24': 'HangulSyllableGyim',
u'\uc2a7': 'HangulSyllableSeugs',
u'\u2428': 'UnicodeCharacter0x2428',
u'\u78ab': 'CjkUnifiedIdeograph78ab',
u'\ue72a': 'UnicodeCharacter0xe72a',
u'\u9d2e': 'CjkUnifiedIdeograph9d2e',
u'\ub1b1': 'HangulSyllableNwaeg',
u'\u1332': 'EthiopicSyllablePhi',
u'\u37b5': 'CjkUnifiedIdeograph37b5',
u'\ud634': 'HangulSyllableHyek',
u'\ueab7': 'UnicodeCharacter0xeab7',
u'\u4c38': 'CjkUnifiedIdeograph4c38',
u'\u60bb': 'CjkUnifiedIdeograph60bb',
u'\u853e': 'CjkUnifiedIdeograph853e',
u'\ud9c1': 'UnicodeCharacter0xd9c1',
u'\u3b42': 'CjkUnifiedIdeograph3b42',
u'\u5fc5': 'CjkUnifiedIdeograph5fc5',
u'\ufe44': 'PresentationFormForVerticalRightWhiteCornerBracket',
u'\u7448': 'CjkUnifiedIdeograph7448',
u'\u88cb': 'CjkUnifiedIdeograph88cb',
u'\u0ecf': 'UnicodeCharacter0xecf',
u'\uad4e': 'HangulSyllableGoep',
u'\uc1d1': 'HangulSyllableSwaelt',
u'\u2352': 'AplFunctionalSymbolDelStile',
u'\u47d5': 'CjkUnifiedIdeograph47d5',
u'\ue654': 'UnicodeCharacter0xe654',
u'\u9c58': 'CjkUnifiedIdeograph9c58',
u'\ub0db': 'HangulSyllableNyalb',
u'\u125c': 'EthiopicSyllableQhwee',
u'\u36df': 'CjkUnifiedIdeograph36df',
u'\ud55e': 'HangulSyllableHanh',
u'\ue9e1': 'UnicodeCharacter0xe9e1',
u'\u4b62': 'CjkUnifiedIdeograph4b62',
u'\u6fe5': 'CjkUnifiedIdeograph6fe5',
u'\u8468': 'CjkUnifiedIdeograph8468',
u'\ud8eb': 'UnicodeCharacter0xd8eb',
u'\u3a6c': 'CjkUnifiedIdeograph3a6c',
u'\u5eef': 'CjkUnifiedIdeograph5eef',
u'\ufd6e': 'ArabicLigatureDadWithHahWithAlefMaksuraFinalForm',
u'\u7372': 'CjkUnifiedIdeograph7372',
u'\u97f5': 'CjkUnifiedIdeograph97f5',
u'\u0df9': 'UnicodeCharacter0xdf9',
u'\uac78': 'HangulSyllableGeol',
u'\uc0fb': 'HangulSyllableSyac',
u'\u227c': 'PrecedesOrEqualTo',
u'\u46ff': 'CjkUnifiedIdeograph46ff',
u'\ue57e': 'UnicodeCharacter0xe57e',
u'\u7c03': 'CjkUnifiedIdeograph7c03',
u'\u9b82': 'CjkUnifiedIdeograph9b82',
u'\u1186': 'HangulJungseongYoyeo',
u'\ub509': 'HangulSyllableDyib',
u'\ud488': 'HangulSyllablePum',
u'\u2b0d': 'UpDownBlackArrow',
u'\u4a8c': 'CjkUnifiedIdeograph4a8c',
u'\uee0f': 'UnicodeCharacter0xee0f',
u'\u6413': 'CjkUnifiedIdeograph6413',
u'\u8392': 'CjkUnifiedIdeograph8392',
u'\u1a17': 'BugineseVowelSignI',
u'\u3996': 'CjkUnifiedIdeograph3996',
u'\udd19': 'UnicodeCharacter0xdd19',
u'\ufc98': 'ArabicLigatureYehWithHamzaAboveWithHahInitialForm',
u'\u531d': 'CjkUnifiedIdeograph531d',
u'\u729c': 'CjkUnifiedIdeograph729c',
u'\u8c23': 'CjkUnifiedIdeograph8c23',
u'\uaba2': 'UnicodeCharacter0xaba2',
u'\u0227': 'LatinSmallLetterAWithDotAbove',
u'\u21a6': 'RightwardsArrowFromBar',
u'\uc529': 'HangulSyllableSsig',
u'\ue4a8': 'UnicodeCharacter0xe4a8',
u'\u7b2d': 'CjkUnifiedIdeograph7b2d',
u'\u9aac': 'CjkUnifiedIdeograph9aac',
u'\u10b0': 'GeorgianCapitalLetterRae',
u'\ub433': 'HangulSyllableDoeh',
u'\ud3b2': 'HangulSyllablePep',
u'\u2a37': 'MultiplicationSignInDoubleCircle',
u'\u49b6': 'CjkUnifiedIdeograph49b6',
u'\ued39': 'UnicodeCharacter0xed39',
u'\u633d': 'CjkUnifiedIdeograph633d',
u'\u82bc': 'CjkUnifiedIdeograph82bc',
u'\u1941': 'UnicodeCharacter0x1941',
u'\u38c0': 'CjkUnifiedIdeograph38c0',
u'\udc43': 'UnicodeCharacter0xdc43',
u'\ufbc2': 'UnicodeCharacter0xfbc2',
u'\u5247': 'CjkUnifiedIdeograph5247',
u'\u71c6': 'CjkUnifiedIdeograph71c6',
u'\u8b4d': 'CjkUnifiedIdeograph8b4d',
u'\uaacc': 'UnicodeCharacter0xaacc',
u'\u0151': 'LatinSmallLetterOWithDoubleAcute',
u'\u20d0': 'CombiningLeftHarpoonAbove',
u'\uc453': 'HangulSyllableSsyolb',
u'\ue3d2': 'UnicodeCharacter0xe3d2',
u'\u7a57': 'CjkUnifiedIdeograph7a57',
u'\u99d6': 'CjkUnifiedIdeograph99d6',
u'\u1fda': 'GreekCapitalLetterIotaWithVaria',
u'\ub35d': 'HangulSyllableDeolg',
u'\ud2dc': 'HangulSyllableTyil',
u'\u2961': 'DownwardsHarpoonWithBarbLeftFromBar',
u'\u48e0': 'CjkUnifiedIdeograph48e0',
u'\uec63': 'UnicodeCharacter0xec63',
u'\u6267': 'CjkUnifiedIdeograph6267',
u'\u81e6': 'CjkUnifiedIdeograph81e6',
u'\u186b': 'MongolianLetterSibeFa',
u'\u07ea': 'NkoLetterJonaRa',
u'\udb6d': 'UnicodeCharacter0xdb6d',
u'\ufaec': 'UnicodeCharacter0xfaec',
u'\u5171': 'CjkUnifiedIdeograph5171',
u'\u70f0': 'CjkUnifiedIdeograph70f0',
u'\u8a77': 'CjkUnifiedIdeograph8a77',
u'\ua9f6': 'UnicodeCharacter0xa9f6',
u'\u2ffa': 'IdeographicDescriptionCharacterSurroundFromLowerLeft',
u'\uc37d': 'HangulSyllableSseong',
u'\ue2fc': 'UnicodeCharacter0xe2fc',
u'\uf881': 'UnicodeCharacter0xf881',
u'\u9900': 'CjkUnifiedIdeograph9900',
u'\ubd83': 'HangulSyllableBugs',
u'\u5a02': 'CjkUnifiedIdeograph5a02',
u'\u7e85': 'CjkUnifiedIdeograph7e85',
u'\u1f04': 'GreekSmallLetterAlphaWithPsiliAndOxia',
u'\u3387': 'SquareGb',
u'\ucc0e': 'HangulSyllableJjigg',
u'\ue091': 'UnicodeCharacter0xe091',
u'\u8110': 'CjkUnifiedIdeograph8110',
u'\ua593': 'VaiSyllableNoo',
u'\u4212': 'CjkUnifiedIdeograph4212',
u'\u6695': 'CjkUnifiedIdeograph6695',
u'\u0714': 'SyriacLetterGamalGarshuni',
u'\u5b97': 'CjkUnifiedIdeograph5b97',
u'\u1c99': 'UnicodeCharacter0x1c99',
u'\uf41e': 'UnicodeCharacter0xf41e',
u'\ua920': 'KayahLiLetterVa',
u'\ucda3': 'HangulSyllableCulh',
u'\u6a22': 'CjkUnifiedIdeograph6a22',
u'\u8ea5': 'CjkUnifiedIdeograph8ea5',
u'\u2f24': 'KangxiRadicalBig',
u'\u43a7': 'CjkUnifiedIdeograph43a7',
u'\u04a9': 'CyrillicSmallLetterAbkhasianHa',
u'\ud130': 'HangulSyllableTeo',
u'\uf5b3': 'UnicodeCharacter0xf5b3',
u'\u9232': 'CjkUnifiedIdeograph9232',
u'\ub6b5': 'HangulSyllableDdut',
u'\u5734': 'CjkUnifiedIdeograph5734',
u'\u6bb7': 'CjkUnifiedIdeograph6bb7',
u'\u0836': 'UnicodeCharacter0x836',
u'\u2cb9': 'CopticSmallLetterDialectpKapa',
u'\uf940': 'CjkCompatibilityIdeographf940',
u'\uba42': 'HangulSyllableMeolm',
u'\udec5': 'UnicodeCharacter0xdec5',
u'\u7f44': 'CjkUnifiedIdeograph7f44',
u'\u93c7': 'CjkUnifiedIdeograph93c7',
u'\u3046': 'HiraganaLetterU',
u'\u54c9': 'CjkUnifiedIdeograph54c9',
u'\u09cb': 'BengaliVowelSignO',
u'\ue150': 'UnicodeCharacter0xe150',
u'\ua252': 'YiSyllableHxuot',
u'\uc6d5': 'HangulSyllableWeolg',
u'\u6754': 'CjkUnifiedIdeograph6754',
u'\ubbd7': 'HangulSyllableMeuc',
u'\u5856': 'CjkUnifiedIdeograph5856',
u'\u7cd9': 'CjkUnifiedIdeograph7cd9',
u'\u1d58': 'ModifierLetterSmallU',
u'\u31db': 'CjkStrokePd',
u'\uca62': 'HangulSyllableJjeoj',
u'\ueee5': 'UnicodeCharacter0xeee5',
u'\u8f64': 'CjkUnifiedIdeograph8f64',
u'\ua3e7': 'YiSyllableJiep',
u'\u4066': 'CjkUnifiedIdeograph4066',
u'\u64e9': 'CjkUnifiedIdeograph64e9',
u'\u0568': 'ArmenianSmallLetterEt',
u'\u59eb': 'CjkUnifiedIdeograph59eb',
u'\u1aed': 'UnicodeCharacter0x1aed',
u'\uf272': 'UnicodeCharacter0xf272',
u'\ub774': 'HangulSyllableDdiss',
u'\ucbf7': 'HangulSyllableJjyid',
u'\u6876': 'CjkUnifiedIdeograph6876',
u'\u8cf9': 'CjkUnifiedIdeograph8cf9',
u'\u2d78': 'UnicodeCharacter0x2d78',
u'\u41fb': 'CjkUnifiedIdeograph41fb',
u'\u02fd': 'ModifierLetterShelf',
u'\u3801': 'CjkUnifiedIdeograph3801',
u'\udf84': 'UnicodeCharacter0xdf84',
u'\u9086': 'CjkUnifiedIdeograph9086',
u'\u5588': 'CjkUnifiedIdeograph5588',
u'\ue90b': 'UnicodeCharacter0xe90b',
u'\u168a': 'OghamLetterCeirt',
u'\uaa0d': 'ChamLetterChha',
u'\u6f0f': 'CjkUnifiedIdeograph6f0f',
u'\u2011': 'NonbreakingHyphen',
u'\uc794': 'HangulSyllableJan',
u'\ub896': 'HangulSyllableRwaegg',
u'\u7d98': 'CjkUnifiedIdeograph7d98',
u'\u3e9a': 'CjkUnifiedIdeograph3e9a',
u'\ud21d': 'HangulSyllableTyolt',
u'\u971f': 'CjkUnifiedIdeograph971f',
u'\u4821': 'CjkUnifiedIdeograph4821',
u'\u0d23': 'MalayalamLetterNna',
u'\uefa4': 'UnicodeCharacter0xefa4',
u'\ua0a6': 'YiSyllableHmup',
u'\u65a8': 'CjkUnifiedIdeograph65a8',
u'\u26aa': 'MediumWhiteCircle',
u'\ufa2d': 'CjkCompatibilityIdeographfa2d',
u'\u1bac': 'UnicodeCharacter0x1bac',
u'\ubf2f': 'HangulSyllableBbyelb',
u'\u7031': 'CjkUnifiedIdeograph7031',
u'\u3533': 'CjkUnifiedIdeograph3533',
u'\uc8b6': 'HangulSyllableJwaelp',
u'\u8db8': 'CjkUnifiedIdeograph8db8',
u'\u4eba': 'CjkUnifiedIdeograph4eba',
u'\ue23d': 'UnicodeCharacter0xe23d',
u'\u03bc': 'GreekSmallLetterMu',
u'\ua73f': 'LatinSmallLetterReversedCWithDot',
u'\u9841': 'CjkUnifiedIdeograph9841',
u'\u5d43': 'CjkUnifiedIdeograph5d43',
u'\u1e45': 'LatinSmallLetterNWithDotAbove',
u'\uf0c6': 'UnicodeCharacter0xf0c6',
u'\ub5c8': 'HangulSyllableDdels',
u'\u76ca': 'CjkUnifiedIdeograph76ca',
u'\u2bcc': 'UnicodeCharacter0x2bcc',
u'\ucf4f': 'HangulSyllableKyec',
u'\u8051': 'CjkUnifiedIdeograph8051',
u'\u4553': 'CjkUnifiedIdeograph4553',
u'\u0655': 'ArabicHamzaBelow',
u'\uddd8': 'UnicodeCharacter0xddd8',
u'\u9eda': 'CjkUnifiedIdeograph9eda',
u'\u53dc': 'CjkUnifiedIdeograph53dc',
u'\uf75f': 'UnicodeCharacter0xf75f',
u'\u14de': 'CanadianSyllabicsLwi',
u'\ua861': 'PhagspaLetterO',
u'\u6d63': 'CjkUnifiedIdeograph6d63',
u'\u2e65': 'UnicodeCharacter0x2e65',
u'\uc5e8': 'HangulSyllableEk',
u'\u86ea': 'CjkUnifiedIdeograph86ea',
u'\u7bec': 'CjkUnifiedIdeograph7bec',
u'\u3cee': 'CjkUnifiedIdeograph3cee',
u'\ud071': 'HangulSyllableKeunj',
u'\u9573': 'CjkUnifiedIdeograph9573',
u'\u5675': 'CjkUnifiedIdeograph5675',
u'\u0b77': 'UnicodeCharacter0xb77',
u'\uedf8': 'UnicodeCharacter0xedf8',
u'\uaefa': 'HangulSyllableGgyeonh',
u'\u63fc': 'CjkUnifiedIdeograph63fc',
u'\u24fe': 'DoubleCircledNumberTen',
u'\u1a00': 'BugineseLetterKa',
u'\u3e83': 'CjkUnifiedIdeograph3e83',
u'\ud902': 'UnicodeCharacter0xd902',
u'\ufd85': 'ArabicLigatureLamWithKhahWithMeemFinalForm',
u'\u5306': 'CjkUnifiedIdeograph5306',
u'\u7789': 'CjkUnifiedIdeograph7789',
u'\u880c': 'CjkUnifiedIdeograph880c',
u'\uac8f': 'HangulSyllableGegs',
u'\u0210': 'LatinCapitalLetterRWithDoubleGrave',
u'\u2693': 'Anchor',
u'\uc112': 'HangulSyllableSyaebs',
u'\ue595': 'UnicodeCharacter0xe595',
u'\u7b16': 'CjkUnifiedIdeograph7b16',
u'\u9f99': 'CjkUnifiedIdeograph9f99',
u'\u119d': 'HangulJungseongIaraea',
u'\ub01c': 'HangulSyllableGgwim',
u'\ud49f': 'HangulSyllablePweolb',
u'\u2a20': 'ZNotationSchemaPiping',
u'\u4ea3': 'CjkUnifiedIdeograph4ea3',
u'\ue922': 'UnicodeCharacter0xe922',
u'\u6326': 'CjkUnifiedIdeograph6326',
u'\u87a9': 'CjkUnifiedIdeograph87a9',
u'\u39ad': 'CjkUnifiedIdeograph39ad',
u'\ud82c': 'UnicodeCharacter0xd82c',
u'\ufcaf': 'ArabicLigatureSeenWithKhahInitialForm',
u'\u5230': 'CjkUnifiedIdeograph5230',
u'\u76b3': 'CjkUnifiedIdeograph76b3',
u'\u8b36': 'CjkUnifiedIdeograph8b36',
u'\uafb9': 'HangulSyllableGgug',
u'\u0d3a': 'UnicodeCharacter0xd3a',
u'\u21bd': 'LeftwardsHarpoonWithBarbDownwards',
u'\uc03c': 'HangulSyllableBbyu',
u'\ue4bf': 'UnicodeCharacter0xe4bf',
u'\u7a40': 'CjkUnifiedIdeograph7a40',
u'\u9ec3': 'CjkUnifiedIdeograph9ec3',
u'\u10c7': 'UnicodeCharacter0x10c7',
u'\ub346': 'HangulSyllableDyaelp',
u'\ud7c9': 'UnicodeCharacter0xd7c9',
u'\u354a': 'CjkUnifiedIdeograph354a',
u'\u49cd': 'CjkUnifiedIdeograph49cd',
u'\ue84c': 'UnicodeCharacter0xe84c',
u'\u6250': 'CjkUnifiedIdeograph6250',
u'\u86d3': 'CjkUnifiedIdeograph86d3',
u'\u38d7': 'CjkUnifiedIdeograph38d7',
u'\udb56': 'UnicodeCharacter0xdb56',
u'\uffd9': 'UnicodeCharacter0xffd9',
u'\u5d5a': 'CjkUnifiedIdeograph5d5a',
u'\u71dd': 'CjkUnifiedIdeograph71dd',
u'\u8a60': 'CjkUnifiedIdeograph8a60',
u'\uaee3': 'HangulSyllableGgelb',
u'\u0c64': 'UnicodeCharacter0xc64',
u'\u20e7': 'CombiningAnnuitySymbol',
u'\uc366': 'HangulSyllableSsyaep',
u'\ue7e9': 'UnicodeCharacter0xe7e9',
u'\u456a': 'CjkUnifiedIdeograph456a',
u'\u99ed': 'CjkUnifiedIdeograph99ed',
u'\u13f1': 'CherokeeLetterYi',
u'\ub270': 'HangulSyllableNwik',
u'\ud6f3': 'HangulSyllableHweos',
u'\u3474': 'CjkUnifiedIdeograph3474',
u'\u48f7': 'CjkUnifiedIdeograph48f7',
u'\ueb76': 'UnicodeCharacter0xeb76',
u'\u6d7a': 'CjkUnifiedIdeograph6d7a',
u'\u81fd': 'CjkUnifiedIdeograph81fd',
u'\ubb01': 'HangulSyllableMoenj',
u'\uda80': 'UnicodeCharacter0xda80',
u'\u3d05': 'CjkUnifiedIdeograph3d05',
u'\u5c84': 'CjkUnifiedIdeograph5c84',
u'\uf007': 'UnicodeCharacter0xf007',
u'\u6a0b': 'CjkUnifiedIdeograph6a0b',
u'\u958a': 'CjkUnifiedIdeograph958a',
u'\u0f8e': 'UnicodeCharacter0xf8e',
u'\ua311': 'YiSyllableSax',
u'\uc290': 'HangulSyllableSyul',
u'\u2515': 'BoxDrawingsUpLightAndRightHeavy',
u'\u4494': 'CjkUnifiedIdeograph4494',
u'\u921b': 'CjkUnifiedIdeograph921b',
u'\ubd9a': 'HangulSyllableBup',
u'\u141f': 'CanadianSyllabicsFinalAcute',
u'\u379e': 'CjkUnifiedIdeograph379e',
u'\ucb21': 'HangulSyllableJjoeb',
u'\ueaa0': 'UnicodeCharacter0xeaa0',
u'\u4d25': 'CjkUnifiedIdeograph4d25',
u'\u6ca4': 'CjkUnifiedIdeograph6ca4',
u'\uba2b': 'HangulSyllableMyaelh',
u'\ua5aa': 'VaiSyllableDu',
u'\u3c2f': 'CjkUnifiedIdeograph3c2f',
u'\u5fae': 'CjkUnifiedIdeograph5fae',
u'\uf331': 'UnicodeCharacter0xf331',
u'\u7535': 'CjkUnifiedIdeograph7535',
u'\u94b4': 'CjkUnifiedIdeograph94b4',
u'\u0eb8': 'LaoVowelSignU',
u'\ua23b': 'YiSyllableMgo',
u'\ucdba': 'HangulSyllableCweolm',
u'\u243f': 'UnicodeCharacter0x243f',
u'\u47be': 'CjkUnifiedIdeograph47be',
u'\u9d45': 'CjkUnifiedIdeograph9d45',
u'\ubcc4': 'HangulSyllableByeol',
u'\u1749': 'BuhidLetterPa',
u'\u36c8': 'CjkUnifiedIdeograph36c8',
u'\uca4b': 'HangulSyllableJjyaeh',
u'\uf5ca': 'UnicodeCharacter0xf5ca',
u'\u4c4f': 'CjkUnifiedIdeograph4c4f',
u'\u6fce': 'CjkUnifiedIdeograph6fce',
u'\u8555': 'CjkUnifiedIdeograph8555',
u'\ua4d4': 'UnicodeCharacter0xa4d4',
u'\u3f59': 'CjkUnifiedIdeograph3f59',
u'\u5ed8': 'CjkUnifiedIdeograph5ed8',
u'\uf25b': 'UnicodeCharacter0xf25b',
u'\u745f': 'CjkUnifiedIdeograph745f',
u'\u97de': 'CjkUnifiedIdeograph97de',
u'\u09e2': 'BengaliVowelSignVocalicL',
u'\uad65': 'HangulSyllableGyong',
u'\ucce4': 'HangulSyllableCyeoss',
u'\u2769': 'MediumRightParenthesisOrnament',
u'\u46e8': 'CjkUnifiedIdeograph46e8',
u'\u9c6f': 'CjkUnifiedIdeograph9c6f',
u'\ubfee': 'HangulSyllableBbweonh',
u'\u1673': 'CanadianSyllabicsNngo',
u'\u31f2': 'KatakanaLetterSmallSu',
u'\ud575': 'HangulSyllableHaeg',
u'\uf4f4': 'UnicodeCharacter0xf4f4',
u'\u4f79': 'CjkUnifiedIdeograph4f79',
u'\u6ef8': 'CjkUnifiedIdeograph6ef8',
u'\u847f': 'CjkUnifiedIdeograph847f',
u'\ua7fe': 'LatinEpigraphicLetterILonga',
u'\ud206': 'HangulSyllableToebs',
u'\uf689': 'UnicodeCharacter0xf689',
u'\u9708': 'CjkUnifiedIdeograph9708',
u'\uab8b': 'UnicodeCharacter0xab8b',
u'\u540a': 'CjkUnifiedIdeograph540a',
u'\u688d': 'CjkUnifiedIdeograph688d',
u'\u090c': 'DevanagariLetterVocalicL',
u'\u2d8f': 'EthiopicSyllableThoa',
u'\ufa16': 'CjkCompatibilityIdeographfa16',
u'\ubf18': 'HangulSyllableBbyeom',
u'\ud39b': 'HangulSyllablePegs',
u'\u7c1a': 'CjkUnifiedIdeograph7c1a',
u'\u909d': 'CjkUnifiedIdeograph909d',
u'\u311c': 'BopomofoLetterE',
u'\u559f': 'CjkUnifiedIdeograph559f',
u'\u0aa1': 'GujaratiLetterDda',
u'\ue226': 'UnicodeCharacter0xe226',
u'\ua728': 'LatinCapitalLetterTz',
u'\ufbab': 'ArabicLetterHehDoachashmeeFinalForm',
u'\u642a': 'CjkUnifiedIdeograph642a',
u'\ub8ad': 'HangulSyllableRwaet',
u'\u592c': 'CjkUnifiedIdeograph592c',
u'\u7daf': 'CjkUnifiedIdeograph7daf',
u'\u1e2e': 'LatinCapitalLetterIWithDiaeresisAndAcute',
u'\u32b1': 'CircledNumberThirtySix',
u'\ucf38': 'HangulSyllableKye',
u'\ue3bb': 'UnicodeCharacter0xe3bb',
u'\u8c3a': 'CjkUnifiedIdeograph8c3a',
u'\ua0bd': 'YiSyllableMot',
u'\u413c': 'CjkUnifiedIdeograph413c',
u'\u65bf': 'CjkUnifiedIdeograph65bf',
u'\u063e': 'ArabicLetterFarsiYehWithTwoDotsAbove',
u'\u5ac1': 'CjkUnifiedIdeograph5ac1',
u'\u1fc3': 'GreekSmallLetterEtaWithYpogegrammeni',
u'\uf748': 'UnicodeCharacter0xf748',
u'\ub44a': 'HangulSyllableDyoj',
u'\uc8cd': 'HangulSyllableJoelg',
u'\u694c': 'CjkUnifiedIdeograph694c',
u'\u8dcf': 'CjkUnifiedIdeograph8dcf',
u'\u2e4e': 'UnicodeCharacter0x2e4e',
u'\u42d1': 'CjkUnifiedIdeograph42d1',
u'\u07d3': 'NkoLetterBa',
u'\udc5a': 'UnicodeCharacter0xdc5a',
u'\uf0dd': 'UnicodeCharacter0xf0dd',
u'\u915c': 'CjkUnifiedIdeograph915c',
u'\ub5df': 'HangulSyllableDdyeod',
u'\u565e': 'CjkUnifiedIdeograph565e',
u'\u6ae1': 'CjkUnifiedIdeograph6ae1',
u'\u0b60': 'OriyaLetterVocalicRr',
u'\u2fe3': 'UnicodeCharacter0x2fe3',
u'\uc46a': 'HangulSyllableSsunh',
u'\ub96c': 'HangulSyllableRyuss',
u'\uddef': 'UnicodeCharacter0xddef',
u'\u7e6e': 'CjkUnifiedIdeograph7e6e',
u'\u92f1': 'CjkUnifiedIdeograph92f1',
u'\u3370': 'IdeographicTelegraphSymbolForHourTwentyfour',
u'\u57f3': 'CjkUnifiedIdeograph57f3',
u'\u14f5': 'CanadianSyllabicsSaa',
u'\uec7a': 'UnicodeCharacter0xec7a',
u'\ua17c': 'YiSyllableHnut',
u'\uc5ff': 'HangulSyllableYeos',
u'\u667e': 'CjkUnifiedIdeograph667e',
u'\u5b80': 'CjkUnifiedIdeograph5b80',
u'\uff03': 'FullwidthNumberSign',
u'\u1882': 'MongolianLetterAliGaliDamaru',
u'\ubc05': 'HangulSyllableMilt',
u'\u7107': 'CjkUnifiedIdeograph7107',
u'\u3609': 'CjkUnifiedIdeograph3609',
u'\uc98c': 'HangulSyllableJeun',
u'\u8e8e': 'CjkUnifiedIdeograph8e8e',
u'\u4390': 'CjkUnifiedIdeograph4390',
u'\ue713': 'UnicodeCharacter0xe713',
u'\x92': 'UnicodeCharacter0x92',
u'\ua415': 'YiSyllableQyp',
u'\u9917': 'CjkUnifiedIdeograph9917',
u'\u5e19': 'CjkUnifiedIdeograph5e19',
u'\u131b': 'EthiopicSyllableGgaa',
u'\uf19c': 'UnicodeCharacter0xf19c',
u'\ub69e': 'HangulSyllableDdugg',
u'\u6ba0': 'CjkUnifiedIdeograph6ba0',
u'\u28a2': 'BraillePatternDots268',
u'\ucc25': 'HangulSyllableJjit',
u'\u8127': 'CjkUnifiedIdeograph8127',
u'\u4629': 'CjkUnifiedIdeograph4629',
u'\u3b2b': 'CjkUnifiedIdeograph3b2b',
u'\udeae': 'UnicodeCharacter0xdeae',
u'\u93b0': 'CjkUnifiedIdeograph93b0',
u'\u50b2': 'CjkUnifiedIdeograph50b2',
u'\uf435': 'UnicodeCharacter0xf435',
u'\u15b4': 'CanadianSyllabicsBlackfootWe',
u'\ua937': 'RejangLetterBa',
u'\u6e39': 'CjkUnifiedIdeograph6e39',
u'\u233b': 'AplFunctionalSymbolQuadJot',
u'\uc6be': 'HangulSyllableUlp',
u'\ubbc0': 'HangulSyllableMeu',
u'\u78c2': 'CjkUnifiedIdeograph78c2',
u'\u3dc4': 'CjkUnifiedIdeograph3dc4',
u'\ud147': 'HangulSyllableTeoc',
u'\u9649': 'CjkUnifiedIdeograph9649',
u'\u4b4b': 'CjkUnifiedIdeograph4b4b',
u'\u084d': 'UnicodeCharacter0x84d',
u'\ueece': 'UnicodeCharacter0xeece',
u'\ua3d0': 'YiSyllableRop',
u'\u60d2': 'CjkUnifiedIdeograph60d2',
u'\u25d4': 'CircleWithUpperRightQuadrantBlack',
u'\uf957': 'CjkCompatibilityIdeographf957',
u'\u1ad6': 'UnicodeCharacter0x1ad6',
u'\ube59': 'HangulSyllableBing',
u'\u735b': 'CjkUnifiedIdeograph735b',
u'\u305d': 'HiraganaLetterSo',
u'\ucbe0': 'HangulSyllableJjeuls',
u'\u88e2': 'CjkUnifiedIdeograph88e2',
u'\u4de4': 'HexagramForTheFamily',
u'\ue167': 'UnicodeCharacter0xe167',
u'\u02e6': 'ModifierLetterHighToneBar',
u'\ua669': 'CyrillicSmallLetterMonocularO',
u'\u9b6b': 'CjkUnifiedIdeograph9b6b',
u'\u586d': 'CjkUnifiedIdeograph586d',
u'\u1d6f': 'LatinSmallLetterMWithMiddleTilde',
u'\uf3f0': 'UnicodeCharacter0xf3f0',
u'\ub0f2': 'HangulSyllableNyaenh',
u'\u75f4': 'CjkUnifiedIdeograph75f4',
u'\u2af6': 'TripleColonOperator',
u'\uce79': 'HangulSyllableKanj',
u'\u837b': 'CjkUnifiedIdeograph837b',
u'\u407d': 'CjkUnifiedIdeograph407d',
u'\u057f': 'ArmenianSmallLetterTiwn',
u'\u7d81': 'CjkUnifiedIdeograph7d81',
u'\u9a04': 'CjkUnifiedIdeograph9a04',
u'\ub687': 'HangulSyllableDdyod',
u'\u1008': 'MyanmarLetterJha',
u'\u2c8b': 'CopticSmallLetterSou',
u'\ud30a': 'HangulSyllableTip',
u'\uef8d': 'UnicodeCharacter0xef8d',
u'\u490e': 'CjkUnifiedIdeograph490e',
u'\u6591': 'CjkUnifiedIdeograph6591',
u'\u8214': 'CjkUnifiedIdeograph8214',
u'\ude97': 'UnicodeCharacter0xde97',
u'\u3818': 'CjkUnifiedIdeograph3818',
u'\u549b': 'CjkUnifiedIdeograph549b',
u'\ufb1a': 'UnicodeCharacter0xfb1a',
u'\u711e': 'CjkUnifiedIdeograph711e',
u'\u8da1': 'CjkUnifiedIdeograph8da1',
u'\u0ba5': 'UnicodeCharacter0xba5',
u'\uaa24': 'ChamLetterLa',
u'\uc6a7': 'HangulSyllableYos',
u'\u2028': 'LineSeparator',
u'\u7cab': 'CjkUnifiedIdeograph7cab',
u'\ue32a': 'UnicodeCharacter0xe32a',
u'\u992e': 'CjkUnifiedIdeograph992e',
u'\ub5b1': 'HangulSyllableDdeob',
u'\u1732': 'HanunooVowelSignI',
u'\u33b5': 'SquareNv',
u'\ud234': 'HangulSyllableTul',
u'\ueeb7': 'UnicodeCharacter0xeeb7',
u'\u4838': 'CjkUnifiedIdeograph4838',
u'\u64bb': 'CjkUnifiedIdeograph64bb',
u'\u813e': 'CjkUnifiedIdeograph813e',
u'\uddc1': 'UnicodeCharacter0xddc1',
u'\u3f42': 'CjkUnifiedIdeograph3f42',
u'\u5bc5': 'CjkUnifiedIdeograph5bc5',
u'\ufa44': 'CjkCompatibilityIdeographfa44',
u'\u7048': 'CjkUnifiedIdeograph7048',
u'\u8ccb': 'CjkUnifiedIdeograph8ccb',
u'\u0acf': 'UnicodeCharacter0xacf',
u'\ua94e': 'RejangVowelSignEa',
u'\uc5d1': 'HangulSyllableEg',
u'\u2752': 'UpperRightShadowedWhiteSquare',
u'\u43d5': 'CjkUnifiedIdeograph43d5',
u'\ue254': 'UnicodeCharacter0xe254',
u'\u9858': 'CjkUnifiedIdeograph9858',
u'\ub4db': 'HangulSyllableDyuh',
u'\u165c': 'CanadianSyllabicsCarrierTso',
u'\u32df': 'CircledKatakanaTa',
u'\ud15e': 'HangulSyllableTebs',
u'\uede1': 'UnicodeCharacter0xede1',
u'\u4f62': 'CjkUnifiedIdeograph4f62',
u'\u6be5': 'CjkUnifiedIdeograph6be5',
u'\u8068': 'CjkUnifiedIdeograph8068',
u'\udceb': 'UnicodeCharacter0xdceb',
u'\u3e6c': 'CjkUnifiedIdeograph3e6c',
u'\u5aef': 'CjkUnifiedIdeograph5aef',
u'\uf96e': 'CjkCompatibilityIdeographf96e',
u'\u7772': 'CjkUnifiedIdeograph7772',
u'\u93f5': 'CjkUnifiedIdeograph93f5',
u'\u09f9': 'BengaliCurrencyDenominatorSixteen',
u'\ua878': 'UnicodeCharacter0xa878',
u'\uc4fb': 'HangulSyllableSseulb',
u'\u267c': 'RecycledPaperSymbol',
u'\u42ff': 'CjkUnifiedIdeograph42ff',
u'\ue17e': 'UnicodeCharacter0xe17e',
u'\u7803': 'CjkUnifiedIdeograph7803',
u'\u9f82': 'CjkUnifiedIdeograph9f82',
u'\u1586': 'CanadianSyllabicsTlhe',
u'\ub109': 'HangulSyllableNeog',
u'\ud088': 'HangulSyllableKyi',
u'\u2f0d': 'KangxiRadicalCover',
u'\u4e8c': 'CjkUnifiedIdeograph4e8c',
u'\uea0f': 'UnicodeCharacter0xea0f',
u'\u6013': 'CjkUnifiedIdeograph6013',
u'\u8792': 'CjkUnifiedIdeograph8792',
u'\u1e17': 'LatinSmallLetterEWithMacronAndAcute',
u'\u3d96': 'CjkUnifiedIdeograph3d96',
u'\ud919': 'UnicodeCharacter0xd919',
u'\uf898': 'UnicodeCharacter0xf898',
u'\u571d': 'CjkUnifiedIdeograph571d',
u'\u769c': 'CjkUnifiedIdeograph769c',
u'\u8823': 'CjkUnifiedIdeograph8823',
u'\uafa2': 'HangulSyllableGgyonh',
u'\u0627': 'ArabicLetterAlef',
u'\u25a6': 'SquareWithOrthogonalCrosshatchFill',
u'\uc129': 'HangulSyllableSeolt',
u'\ue0a8': 'UnicodeCharacter0xe0a8',
u'\u7f2d': 'CjkUnifiedIdeograph7f2d',
u'\u9eac': 'CjkUnifiedIdeograph9eac',
u'\u14b0': 'CanadianSyllabicsMwii',
u'\ub033': 'HangulSyllableGgyulb',
u'\ud7b2': 'UnicodeCharacter0xd7b2',
u'\u2e37': 'UnicodeCharacter0x2e37',
u'\u4db6': 'UnicodeCharacter0x4db6',
u'\ue939': 'UnicodeCharacter0xe939',
u'\u673d': 'CjkUnifiedIdeograph673d',
u'\u86bc': 'CjkUnifiedIdeograph86bc',
u'\u1d41': 'ModifierLetterCapitalU',
u'\u3cc0': 'CjkUnifiedIdeograph3cc0',
u'\ud843': 'UnicodeCharacter0xd843',
u'\uffc2': 'HalfwidthHangulLetterA',
u'\u5647': 'CjkUnifiedIdeograph5647',
u'\u75c6': 'CjkUnifiedIdeograph75c6',
u'\u8f4d': 'CjkUnifiedIdeograph8f4d',
u'\uaecc': 'HangulSyllableGgeom',
u'\u0551': 'ArmenianCapitalLetterCo',
u'\u24d0': 'CircledLatinSmallLetterA',
u'\uc053': 'HangulSyllableBbyuc',
u'\ue7d2': 'UnicodeCharacter0xe7d2',
u'\u7e57': 'CjkUnifiedIdeograph7e57',
u'\u9dd6': 'CjkUnifiedIdeograph9dd6',
u'\u1bda': 'UnicodeCharacter0x1bda',
u'\ub75d': 'HangulSyllableDdyit',
u'\ud6dc': 'HangulSyllableHuk',
u'\u2d61': 'TifinaghLetterYaw',
u'\u4ce0': 'CjkUnifiedIdeograph4ce0',
u'\ue863': 'UnicodeCharacter0xe863',
u'\u6667': 'CjkUnifiedIdeograph6667',
u'\u85e6': 'CjkUnifiedIdeograph85e6',
u'\u1c6b': 'OlChikiLetterUd',
u'\u03ea': 'CopticCapitalLetterGangia',
u'\udf6d': 'UnicodeCharacter0xdf6d',
u'\ufeec': 'ArabicLetterHehMedialForm',
u'\u5571': 'CjkUnifiedIdeograph5571',
u'\u74f0': 'CjkUnifiedIdeograph74f0',
u'\u8e77': 'CjkUnifiedIdeograph8e77',
u'\uadf6': 'HangulSyllableGyup',
u'\u047b': 'CyrillicSmallLetterRoundOmega',
u'\u2bfa': 'UnicodeCharacter0x2bfa',
u'\uc77d': 'HangulSyllableIlg',
u'\ue6fc': 'UnicodeCharacter0xe6fc',
u'\ufc81': 'ArabicLigatureKafWithLamFinalForm',
u'\u9d00': 'CjkUnifiedIdeograph9d00',
u'\ub983': 'HangulSyllableReulh',
u'\u5e02': 'CjkUnifiedIdeograph5e02',
u'\u7a85': 'CjkUnifiedIdeograph7a85',
u'\u1b04': 'BalineseSignBisah',
u'\u3787': 'CjkUnifiedIdeograph3787',
u'\uc80e': 'HangulSyllableJeolp',
u'\ue491': 'UnicodeCharacter0xe491',
u'\u8510': 'CjkUnifiedIdeograph8510',
u'\ua193': 'YiSyllableNu',
u'\u4612': 'CjkUnifiedIdeograph4612',
u'\u6295': 'CjkUnifiedIdeograph6295',
u'\u0314': 'CombiningReversedCommaAbove',
u'\u5f97': 'CjkUnifiedIdeograph5f97',
u'\u1899': 'MongolianLetterTodoAliGaliZha',
u'\uf01e': 'UnicodeCharacter0xf01e',
u'\uad20': 'HangulSyllableGwael',
u'\uc9a3': 'HangulSyllableJeuh',
u'\u6e22': 'CjkUnifiedIdeograph6e22',
u'\u8aa5': 'CjkUnifiedIdeograph8aa5',
u'\u2b24': 'BlackLargeCircle',
u'\u47a7': 'CjkUnifiedIdeograph47a7',
u'\xa9': 'CopyrightSign',
u'\ud530': 'HangulSyllablePyim',
u'\uf1b3': 'UnicodeCharacter0xf1b3',
u'\u9632': 'CjkUnifiedIdeograph9632',
u'\ub2b5': 'HangulSyllableNyilg',
u'\u5334': 'CjkUnifiedIdeograph5334',
u'\u6fb7': 'CjkUnifiedIdeograph6fb7',
u'\u0c36': 'TeluguLetterSha',
u'\u28b9': 'BraillePatternDots14568',
u'\ufd40': 'UnicodeCharacter0xfd40',
u'\ube42': 'HangulSyllableByip',
u'\udac5': 'UnicodeCharacter0xdac5',
u'\u7b44': 'CjkUnifiedIdeograph7b44',
u'\u97c7': 'CjkUnifiedIdeograph97c7',
u'\u3446': 'CjkUnifiedIdeograph3446',
u'\u50c9': 'CjkUnifiedIdeograph50c9',
u'\u0dcb': 'UnicodeCharacter0xdcb',
u'\ue550': 'UnicodeCharacter0xe550',
u'\ua652': 'CyrillicCapitalLetterIotifiedYat',
u'\uc2d5': 'HangulSyllableSying',
u'\u6354': 'CjkUnifiedIdeograph6354',
u'\ubfd7': 'HangulSyllableBbulb',
u'\u5c56': 'CjkUnifiedIdeograph5c56',
u'\u78d9': 'CjkUnifiedIdeograph78d9',
u'\u1958': 'TaiLeLetterLa',
u'\u35db': 'CjkUnifiedIdeograph35db',
u'\uce62': 'HangulSyllableCilm',
u'\ueae5': 'UnicodeCharacter0xeae5',
u'\u8b64': 'CjkUnifiedIdeograph8b64',
u'\ua7e7': 'UnicodeCharacter0xa7e7',
u'\u4466': 'CjkUnifiedIdeograph4466',
u'\u60e9': 'CjkUnifiedIdeograph60e9',
u'\u0168': 'LatinCapitalLetterUWithTilde',
u'\u5deb': 'CjkUnifiedIdeograph5deb',
u'\u1eed': 'LatinSmallLetterUWithHornAndHookAbove',
u'\uf672': 'UnicodeCharacter0xf672',
u'\ub374': 'HangulSyllableDen',
u'\ucff7': 'HangulSyllableKuc',
u'\u6c76': 'CjkUnifiedIdeograph6c76',
u'\u88f9': 'CjkUnifiedIdeograph88f9',
u'\u2978': 'GreaterthanAboveRightwardsArrow',
u'\u45fb': 'CjkUnifiedIdeograph45fb',
u'\u06fd': 'ArabicSignSindhiAmpersand',
u'\u3c01': 'CjkUnifiedIdeograph3c01',
u'\udb84': 'UnicodeCharacter0xdb84',
u'\u9486': 'CjkUnifiedIdeograph9486',
u'\u5188': 'CjkUnifiedIdeograph5188',
u'\ued0b': 'UnicodeCharacter0xed0b',
u'\u128a': 'EthiopicSyllableXwi',
u'\uae0d': 'HangulSyllableGeung',
u'\u6b0f': 'CjkUnifiedIdeograph6b0f',
u'\u2411': 'SymbolForDeviceControlOne',
u'\uc394': 'HangulSyllableSsem',
u'\ubc96': 'HangulSyllableBeobs',
u'\u7998': 'CjkUnifiedIdeograph7998',
u'\u3a9a': 'CjkUnifiedIdeograph3a9a',
u'\ud61d': 'HangulSyllableHyeg',
u'\u931f': 'CjkUnifiedIdeograph931f',
u'\u4c21': 'CjkUnifiedIdeograph4c21',
u'\u0923': 'DevanagariLetterNna',
u'\ueba4': 'UnicodeCharacter0xeba4',
u'\ua4a6': 'YiRadicalGguo',
u'\u61a8': 'CjkUnifiedIdeograph61a8',
u'\u22aa': 'TripleVerticalBarRightTurnstile',
u'\ufe2d': 'UnicodeCharacter0xfe2d',
u'\u1fac': 'GreekCapitalLetterOmegaWithPsiliAndOxiaAndProsgegrammeni',
u'\ubb2f': 'HangulSyllableMyoc',
u'\u7431': 'CjkUnifiedIdeograph7431',
u'\u3133': 'HangulLetterKiyeoksios',
u'\uccb6': 'HangulSyllableCegg',
u'\u89b8': 'CjkUnifiedIdeograph89b8',
u'\u4aba': 'CjkUnifiedIdeograph4aba',
u'\ue63d': 'UnicodeCharacter0xe63d',
u'\u07bc': 'UnicodeCharacter0x7bc',
u'\ua33f': 'YiSyllableSsup',
u'\u9c41': 'CjkUnifiedIdeograph9c41',
u'\u5943': 'CjkUnifiedIdeograph5943',
u'\u1a45': 'UnicodeCharacter0x1a45',
u'\uf4c6': 'UnicodeCharacter0xf4c6',
u'\ub1c8': 'HangulSyllableNwaek',
u'\u72ca': 'CjkUnifiedIdeograph72ca',
u'\u2fcc': 'KangxiRadicalFrog',
u'\ucb4f': 'HangulSyllableJjud',
u'\u8451': 'CjkUnifiedIdeograph8451',
u'\u4153': 'CjkUnifiedIdeograph4153',
u'\u0255': 'LatinSmallLetterCWithCurl',
u'\ud9d8': 'UnicodeCharacter0xd9d8',
u'\u9ada': 'CjkUnifiedIdeograph9ada',
u'\u57dc': 'CjkUnifiedIdeograph57dc',
u'\uf35f': 'UnicodeCharacter0xf35f',
u'\u10de': 'GeorgianLetterPar',
u'\uac61': 'HangulSyllableGyaelt',
u'\u6963': 'CjkUnifiedIdeograph6963',
u'\u2a65': 'ZNotationRangeAntirestriction',
u'\uc1e8': 'HangulSyllableSoel',
u'\u82ea': 'CjkUnifiedIdeograph82ea',
u'\u7fec': 'CjkUnifiedIdeograph7fec',
u'\u38ee': 'CjkUnifiedIdeograph38ee',
u'\ud471': 'HangulSyllablePyong',
u'\u9173': 'CjkUnifiedIdeograph9173',
u'\u5275': 'CjkUnifiedIdeograph5275',
u'\u0f77': 'TibetanVowelSignVocalicRr',
u'\ue9f8': 'UnicodeCharacter0xe9f8',
u'\uaafa': 'UnicodeCharacter0xaafa',
u'\u67fc': 'CjkUnifiedIdeograph67fc',
u'\u20fe': 'UnicodeCharacter0x20fe',
u'\u1e00': 'LatinCapitalLetterAWithRingBelow',
u'\u3a83': 'CjkUnifiedIdeograph3a83',
u'\udd02': 'UnicodeCharacter0xdd02',
u'\uf985': 'CjkCompatibilityIdeographf985',
u'\u5706': 'CjkUnifiedIdeograph5706',
u'\u7389': 'CjkUnifiedIdeograph7389',
u'\u8c0c': 'CjkUnifiedIdeograph8c0c',
u'\ua88f': 'SaurashtraLetterO',
u'\u0610': 'ArabicSignSallallahouAlayheWassallam',
u'\u2293': 'SquareCap',
u'\uc512': 'HangulSyllableSsyinh',
u'\ue195': 'UnicodeCharacter0xe195',
u'\u7f16': 'CjkUnifiedIdeograph7f16',
u'\u9b99': 'CjkUnifiedIdeograph9b99',
u'\u159d': 'CanadianSyllabicsWoodscreeTho',
u'\ub41c': 'HangulSyllableDoen',
u'\ud09f': 'HangulSyllableKyic',
u'\u2e20': 'LeftVerticalBarWithQuill',
u'\u4aa3': 'CjkUnifiedIdeograph4aa3',
u'\ued22': 'UnicodeCharacter0xed22',
u'\u6726': 'CjkUnifiedIdeograph6726',
u'\u83a9': 'CjkUnifiedIdeograph83a9',
u'\u3dad': 'CjkUnifiedIdeograph3dad',
u'\udc2c': 'UnicodeCharacter0xdc2c',
u'\uf8af': 'UnicodeCharacter0xf8af',
u'\u5630': 'CjkUnifiedIdeograph5630',
u'\u72b3': 'CjkUnifiedIdeograph72b3',
u'\u8f36': 'CjkUnifiedIdeograph8f36',
u'\uabb9': 'UnicodeCharacter0xabb9',
u'\u093a': 'UnicodeCharacter0x93a',
u'\u25bd': 'WhiteDownpointingTriangle',
u'\uc43c': 'HangulSyllableSsoem',
u'\ue0bf': 'UnicodeCharacter0xe0bf',
u'\u7e40': 'CjkUnifiedIdeograph7e40',
u'\u9ac3': 'CjkUnifiedIdeograph9ac3',
u'\u14c7': 'CanadianSyllabicsNa',
u'\ub746': 'HangulSyllableDdyigg',
u'\ud3c9': 'HangulSyllablePyeong',
u'\u314a': 'HangulLetterChieuch',
u'\u4dcd': 'HexagramForGreatPossession',
u'\uec4c': 'UnicodeCharacter0xec4c',
u'\u6650': 'CjkUnifiedIdeograph6650',
u'\u82d3': 'CjkUnifiedIdeograph82d3',
u'\u3cd7': 'CjkUnifiedIdeograph3cd7',
u'\udf56': 'UnicodeCharacter0xdf56',
u'\ufbd9': 'ArabicLetterOeIsolatedForm',
u'\u595a': 'CjkUnifiedIdeograph595a',
u'\u75dd': 'CjkUnifiedIdeograph75dd',
u'\u8e60': 'CjkUnifiedIdeograph8e60',
u'\uaae3': 'UnicodeCharacter0xaae3',
u'\u0864': 'UnicodeCharacter0x864',
u'\u24e7': 'CircledLatinSmallLetterX',
u'\uc766': 'HangulSyllableYilp',
u'\ue3e9': 'UnicodeCharacter0xe3e9',
u'\u416a': 'CjkUnifiedIdeograph416a',
u'\u9ded': 'CjkUnifiedIdeograph9ded',
u'\u17f1': 'KhmerSymbolLekAttakMuoy',
u'\ub670': 'HangulSyllableDdoels',
u'\ud2f3': 'HangulSyllableTigs',
u'\u3074': 'HiraganaLetterPi',
u'\u4cf7': 'CjkUnifiedIdeograph4cf7',
u'\uef76': 'UnicodeCharacter0xef76',
u'\u697a': 'CjkUnifiedIdeograph697a',
u'\u85fd': 'CjkUnifiedIdeograph85fd',
u'\ubf01': 'HangulSyllableBbeng',
u'\ude80': 'UnicodeCharacter0xde80',
u'\u3905': 'CjkUnifiedIdeograph3905',
u'\u5884': 'CjkUnifiedIdeograph5884',
u'\uf407': 'UnicodeCharacter0xf407',
u'\u6e0b': 'CjkUnifiedIdeograph6e0b',
u'\u918a': 'CjkUnifiedIdeograph918a',
u'\u0b8e': 'TamilLetterE',
u'\ua711': 'ModifierLetterExtralowDottedLeftstemToneBar',
u'\uc690': 'HangulSyllableOek',
u'\u2115': 'DoublestruckCapitalN',
u'\u4094': 'CjkUnifiedIdeograph4094',
u'\u961b': 'CjkUnifiedIdeograph961b',
u'\ub99a': 'HangulSyllableRyilm',
u'\u101f': 'MyanmarLetterHa',
u'\u339e': 'SquareKm',
u'\ucf21': 'HangulSyllableKyeonj',
u'\ueea0': 'UnicodeCharacter0xeea0',
u'\u4925': 'CjkUnifiedIdeograph4925',
u'\u68a4': 'CjkUnifiedIdeograph68a4',
u'\ube2b': 'HangulSyllableByigs',
u'\ua1aa': 'YiSyllableHlep',
u'\u382f': 'CjkUnifiedIdeograph382f',
u'\u5bae': 'CjkUnifiedIdeograph5bae',
u'\uf731': 'UnicodeCharacter0xf731',
u'\u7135': 'CjkUnifiedIdeograph7135',
u'\u90b4': 'CjkUnifiedIdeograph90b4',
u'\u0ab8': 'GujaratiLetterSa',
u'\ua63b': 'UnicodeCharacter0xa63b',
u'\uc9ba': 'HangulSyllableJyij',
u'\u203f': 'Undertie',
u'\u43be': 'CjkUnifiedIdeograph43be',
u'\u9945': 'CjkUnifiedIdeograph9945',
u'\ub8c4': 'HangulSyllableRoess',
u'\u1349': 'EthiopicSyllableFu',
u'\u32c8': 'IdeographicTelegraphSymbolForSeptember',
u'\uce4b': 'HangulSyllableCyilh',
u'\uf1ca': 'UnicodeCharacter0xf1ca',
u'\u484f': 'CjkUnifiedIdeograph484f',
u'\u6bce': 'CjkUnifiedIdeograph6bce',
u'\u8155': 'CjkUnifiedIdeograph8155',
u'\ua0d4': 'YiSyllableFap',
u'\u3b59': 'CjkUnifiedIdeograph3b59',
u'\u5ad8': 'CjkUnifiedIdeograph5ad8',
u'\uf65b': 'UnicodeCharacter0xf65b',
u'\u705f': 'CjkUnifiedIdeograph705f',
u'\u93de': 'CjkUnifiedIdeograph93de',
u'\u0de2': 'UnicodeCharacter0xde2',
u'\ua965': 'UnicodeCharacter0xa965',
u'\uc8e4': 'HangulSyllableJyon',
u'\u2369': 'AplFunctionalSymbolGreaterthanDiaeresis',
u'\u42e8': 'CjkUnifiedIdeograph42e8',
u'\u986f': 'CjkUnifiedIdeograph986f',
u'\ubbee': 'HangulSyllableMyibs',
u'\u1273': 'EthiopicSyllableTaa',
u'\u35f2': 'CjkUnifiedIdeograph35f2',
u'\ud175': 'HangulSyllableTyeolt',
u'\uf0f4': 'UnicodeCharacter0xf0f4',
u'\u4b79': 'CjkUnifiedIdeograph4b79',
u'\u6af8': 'CjkUnifiedIdeograph6af8',
u'\u807f': 'CjkUnifiedIdeograph807f',
u'\ua3fe': 'YiSyllableQi',
u'\ud606': 'HangulSyllableHyeonh',
u'\uf289': 'UnicodeCharacter0xf289',
u'\u9308': 'CjkUnifiedIdeograph9308',
u'\uaf8b': 'HangulSyllableGgoelb',
u'\u500a': 'CjkUnifiedIdeograph500a',
u'\u6c8d': 'CjkUnifiedIdeograph6c8d',
u'\u0d0c': 'MalayalamLetterVocalicL',
u'\u298f': 'LeftSquareBracketWithTickInBottomCorner',
u'\ufe16': 'PresentationFormForVerticalQuestionMark',
u'\ubb18': 'HangulSyllableMyo',
u'\ud79b': 'HangulSyllableHis',
u'\u781a': 'CjkUnifiedIdeograph781a',
u'\u949d': 'CjkUnifiedIdeograph949d',
u'\u351c': 'CjkUnifiedIdeograph351c',
u'\u519f': 'CjkUnifiedIdeograph519f',
u'\u0ea1': 'LaoLetterMo',
u'\ue626': 'UnicodeCharacter0xe626',
u'\ua328': 'YiSyllableSyrx',
u'\uffab': 'HalfwidthHangulLetterRieulmieum',
u'\u602a': 'CjkUnifiedIdeograph602a',
u'\ubcad': 'HangulSyllableBelt',
u'\u5d2c': 'CjkUnifiedIdeograph5d2c',
u'\u79af': 'CjkUnifiedIdeograph79af',
u'\u1a2e': 'UnicodeCharacter0x1a2e',
u'\u36b1': 'CjkUnifiedIdeograph36b1',
u'\ucb38': 'HangulSyllableJjyols',
u'\ue7bb': 'UnicodeCharacter0xe7bb',
u'\u883a': 'CjkUnifiedIdeograph883a',
u'\ua4bd': 'YiRadicalCuop',
u'\u453c': 'CjkUnifiedIdeograph453c',
u'\u61bf': 'CjkUnifiedIdeograph61bf',
u'\u023e': 'LatinCapitalLetterTWithDiagonalStroke',
u'\u5ec1': 'CjkUnifiedIdeograph5ec1',
u'\u1bc3': 'UnicodeCharacter0x1bc3',
u'\uf348': 'UnicodeCharacter0xf348',
u'\ub04a': 'HangulSyllableGgeunh',
u'\ucccd': 'HangulSyllableCet',
u'\u6d4c': 'CjkUnifiedIdeograph6d4c',
u'\u89cf': 'CjkUnifiedIdeograph89cf',
u'\u2a4e': 'DoubleSquareIntersection',
u'\u46d1': 'CjkUnifiedIdeograph46d1',
u'\u03d3': 'GreekUpsilonWithAcuteAndHookSymbol',
u'\ud85a': 'UnicodeCharacter0xd85a',
u'\uf4dd': 'UnicodeCharacter0xf4dd',
u'\u955c': 'CjkUnifiedIdeograph955c',
u'\ub1df': 'HangulSyllableNoes',
u'\u525e': 'CjkUnifiedIdeograph525e',
u'\u6ee1': 'CjkUnifiedIdeograph6ee1',
u'\u0f60': 'TibetanLetterA',
u'\u2be3': 'UnicodeCharacter0x2be3',
u'\uc06a': 'HangulSyllableBbeubs',
u'\ubd6c': 'HangulSyllableByol',
u'\ud9ef': 'UnicodeCharacter0xd9ef',
u'\u7a6e': 'CjkUnifiedIdeograph7a6e',
u'\u96f1': 'CjkUnifiedIdeograph96f1',
u'\u3770': 'CjkUnifiedIdeograph3770',
u'\u53f3': 'CjkUnifiedIdeograph53f3',
u'\u10f5': 'GeorgianLetterHoe',
u'\ue87a': 'UnicodeCharacter0xe87a',
u'\ua57c': 'VaiSyllableGboo',
u'\uc1ff': 'HangulSyllableSyogs',
u'\u627e': 'CjkUnifiedIdeograph627e',
u'\u5f80': 'CjkUnifiedIdeograph5f80',
u'\ufb03': 'LatinSmallLigatureFfi',
u'\u1c82': 'UnicodeCharacter0x1c82',
u'\ub805': 'HangulSyllableReot',
u'\u7507': 'CjkUnifiedIdeograph7507',
u'\u3209': 'ParenthesizedHangulChieuch',
u'\ucd8c': 'HangulSyllableCyoss',
u'\u8a8e': 'CjkUnifiedIdeograph8a8e',
u'\u4790': 'CjkUnifiedIdeograph4790',
u'\ue313': 'UnicodeCharacter0xe313',
u'\u0492': 'CyrillicCapitalLetterGheWithStroke',
u'\ua015': 'YiSyllableWu',
u'\u9d17': 'CjkUnifiedIdeograph9d17',
u'\u5a19': 'CjkUnifiedIdeograph5a19',
u'\u171b': 'UnicodeCharacter0x171b',
u'\uf59c': 'UnicodeCharacter0xf59c',
u'\ub29e': 'HangulSyllableNeulp',
u'\u6fa0': 'CjkUnifiedIdeograph6fa0',
u'\u2ca2': 'CopticCapitalLetterRo',
u'\uc825': 'HangulSyllableJelg',
u'\u8527': 'CjkUnifiedIdeograph8527',
u'\u4229': 'CjkUnifiedIdeograph4229',
u'\u3f2b': 'CjkUnifiedIdeograph3f2b',
u'\udaae': 'UnicodeCharacter0xdaae',
u'\u97b0': 'CjkUnifiedIdeograph97b0',
u'\u54b2': 'CjkUnifiedIdeograph54b2',
u'\uf035': 'UnicodeCharacter0xf035',
u'\u11b4': 'HangulJongseongRieulthieuth',
u'\uad37': 'HangulSyllableGoegs',
u'\u6a39': 'CjkUnifiedIdeograph6a39',
u'\u273b': 'TeardropspokedAsterisk',
u'\uc2be': 'HangulSyllableSeup',
u'\ubfc0': 'HangulSyllableBbyom',
u'\u7cc2': 'CjkUnifiedIdeograph7cc2',
u'\u39c4': 'CjkUnifiedIdeograph39c4',
u'\ud547': 'HangulSyllablePilb',
u'\u9249': 'CjkUnifiedIdeograph9249',
u'\u4f4b': 'CjkUnifiedIdeograph4f4b',
u'\u0c4d': 'TeluguSignVirama',
u'\ueace': 'UnicodeCharacter0xeace',
u'\ua7d0': 'UnicodeCharacter0xa7d0',
u'\u64d2': 'CjkUnifiedIdeograph64d2',
u'\u21d4': 'LeftRightDoubleArrow',
u'\ufd57': 'ArabicLigatureTehWithMeemWithKhahInitialForm',
u'\u1ed6': 'LatinCapitalLetterOWithCircumflexAndTilde',
u'\uba59': 'HangulSyllableMenj',
u'\u775b': 'CjkUnifiedIdeograph775b',
u'\u345d': 'CjkUnifiedIdeograph345d',
u'\ucfe0': 'HangulSyllableKu',
u'\u8ce2': 'CjkUnifiedIdeograph8ce2',
u'\u49e4': 'CjkUnifiedIdeograph49e4',
u'\ue567': 'UnicodeCharacter0xe567',
u'\u06e6': 'ArabicSmallYeh',
u'\ua269': 'YiSyllableNgo',
u'\u9f6b': 'CjkUnifiedIdeograph9f6b',
u'\u5c6d': 'CjkUnifiedIdeograph5c6d',
u'\u196f': 'UnicodeCharacter0x196f',
u'\uf7f0': 'UnicodeCharacter0xf7f0',
u'\ub4f2': 'HangulSyllableDeuj',
u'\u71f4': 'CjkUnifiedIdeograph71f4',
u'\u2ef6': 'UnicodeCharacter0x2ef6',
u'\uca79': 'HangulSyllableJjeb',
u'\u877b': 'CjkUnifiedIdeograph877b',
u'\u447d': 'CjkUnifiedIdeograph447d',
u'\u017f': 'LatinSmallLetterLongS',
u'\u4181': 'CjkUnifiedIdeograph4181',
u'\ua604': 'VaiSyllableNgge',
u'\u8a87': 'CjkUnifiedIdeograph8a87',
u'\u2c08': 'GlagoliticCapitalLetterZemlja',
u'\u108b': 'MyanmarSignShanCouncilTone2',
u'\uef0a': 'UnicodeCharacter0xef0a',
u'\ud38d': 'HangulSyllablePeob',
u'\u750e': 'CjkUnifiedIdeograph750e',
u'\u5991': 'CjkUnifiedIdeograph5991',
u'\ube14': 'HangulSyllableBeul',
u'\ue297': 'UnicodeCharacter0xe297',
u'\u0418': 'CyrillicCapitalLetterI',
u'\u689b': 'CjkUnifiedIdeograph689b',
u'\uc71a': 'HangulSyllableWij',
u'\u4d1e': 'CjkUnifiedIdeograph4d1e',
u'\ub1a1': 'HangulSyllableNwalt',
u'\u37a5': 'CjkUnifiedIdeograph37a5',
u'\u9624': 'CjkUnifiedIdeograph9624',
u'\ufaa7': 'CjkCompatibilityIdeographfaa7',
u'\u1c28': 'LepchaVowelSignO',
u'\u40ab': 'CjkUnifiedIdeograph40ab',
u'\udf2a': 'UnicodeCharacter0xdf2a',
u'\ua52e': 'VaiSyllableKpi',
u'\u89b1': 'CjkUnifiedIdeograph89b1',
u'\u2b32': 'LeftArrowWithCircledPlus',
u'\u0fb5': 'TibetanSubjoinedLetterSsa',
u'\uee34': 'UnicodeCharacter0xee34',
u'\ud2b7': 'HangulSyllableTyuh',
u'\u7438': 'CjkUnifiedIdeograph7438',
u'\u58bb': 'CjkUnifiedIdeograph58bb',
u'\ubd3e': 'HangulSyllableBwaebs',
u'\ue1c1': 'UnicodeCharacter0xe1c1',
u'\u0342': 'CombiningGreekPerispomeni',
u'\u67c5': 'CjkUnifiedIdeograph67c5',
u'\uc644': 'HangulSyllableWan',
u'\u4c48': 'CjkUnifiedIdeograph4c48',
u'\ub0cb': 'HangulSyllableNaec',
u'\u36cf': 'CjkUnifiedIdeograph36cf',
u'\u954e': 'CjkUnifiedIdeograph954e',
u'\uf9d1': 'CjkCompatibilityIdeographf9d1',
u'\u1b52': 'BalineseDigitTwo',
u'\u7fd5': 'CjkUnifiedIdeograph7fd5',
u'\ude54': 'UnicodeCharacter0xde54',
u'\ua458': 'YiSyllableNyop',
u'\u88db': 'CjkUnifiedIdeograph88db',
u'\u2a5c': 'LogicalAndWithHorizontalDash',
u'\u0edf': 'UnicodeCharacter0xedf',
u'\ued5e': 'UnicodeCharacter0xed5e',
u'\ud1e1': 'HangulSyllableTwaelg',
u'\u7362': 'CjkUnifiedIdeograph7362',
u'\u57e5': 'CjkUnifiedIdeograph57e5',
u'\ubc68': 'HangulSyllableByae',
u'\ue0eb': 'UnicodeCharacter0xe0eb',
u'\u026c': 'LatinSmallLetterLWithBelt',
u'\u66ef': 'CjkUnifiedIdeograph66ef',
u'\uc56e': 'HangulSyllableAelp',
u'\u4b72': 'CjkUnifiedIdeograph4b72',
u'\uaff5': 'HangulSyllableGgwenj',
u'\u35f9': 'CjkUnifiedIdeograph35f9',
u'\u9478': 'CjkUnifiedIdeograph9478',
u'\uf8fb': 'UnicodeCharacter0xf8fb',
u'\u1a7c': 'UnicodeCharacter0x1a7c',
u'\u7eff': 'CjkUnifiedIdeograph7eff',
u'\udd7e': 'UnicodeCharacter0xdd7e',
u'\u4403': 'CjkUnifiedIdeograph4403',
u'\ua382': 'YiSyllableRro',
u'\u2986': 'RightWhiteParenthesis',
u'\u8d09': 'CjkUnifiedIdeograph8d09',
u'\uec88': 'UnicodeCharacter0xec88',
u'\u130d': 'EthiopicSyllableGe',
u'\u728c': 'CjkUnifiedIdeograph728c',
u'\ud60f': 'HangulSyllableHyeolh',
u'\u5c13': 'CjkUnifiedIdeograph5c13',
u'\ubb92': 'HangulSyllableMwilm',
u'\u2217': 'AsteriskOperator',
u'\u0196': 'LatinCapitalLetterIota',
u'\ue519': 'UnicodeCharacter0xe519',
u'\uc498': 'HangulSyllableSsweok',
u'\u6b1d': 'CjkUnifiedIdeograph6b1d',
u'\u4a9c': 'CjkUnifiedIdeograph4a9c',
u'\ub423': 'HangulSyllableDoelb',
u'\u93a2': 'CjkUnifiedIdeograph93a2',
u'\u3a27': 'CjkUnifiedIdeograph3a27',
u'\u19a6': 'NewTaiLueLetterHighKva',
u'\ufd29': 'ArabicLigatureSheenWithRehFinalForm',
u'\udca8': 'UnicodeCharacter0xdca8',
u'\u432d': 'CjkUnifiedIdeograph432d',
u'\ua2ac': 'YiSyllableZyrx',
u'\u28b0': 'BraillePatternDots568',
u'\u8c33': 'CjkUnifiedIdeograph8c33',
u'\uebb2': 'UnicodeCharacter0xebb2',
u'\u1237': 'EthiopicSyllableSwa',
u'\u71b6': 'CjkUnifiedIdeograph71b6',
u'\ud539': 'HangulSyllablePyit',
u'\u5b3d': 'CjkUnifiedIdeograph5b3d',
u'\ubabc': 'HangulSyllableMoss',
u'\u2141': 'TurnedSansserifCapitalG',
u'\xc0': 'LatinCapitalLetterAWithGrave',
u'\ue443': 'UnicodeCharacter0xe443',
u'\uc3c2': 'HangulSyllableSsyenh',
u'\u6a47': 'CjkUnifiedIdeograph6a47',
u'\u49c6': 'CjkUnifiedIdeograph49c6',
u'\ub34d': 'HangulSyllableDyaeng',
u'\u92cc': 'CjkUnifiedIdeograph92cc',
u'\u3951': 'CjkUnifiedIdeograph3951',
u'\u18d0': 'UnicodeCharacter0x18d0',
u'\ufc53': 'ArabicLigatureHehWithAlefMaksuraIsolatedForm',
u'\udbd2': 'UnicodeCharacter0xdbd2',
u'\u4257': 'CjkUnifiedIdeograph4257',
u'\ua1d6': 'YiSyllableLy',
u'\u27da': 'LeftAndRightDoubleTurnstile',
u'\u8b5d': 'CjkUnifiedIdeograph8b5d',
u'\ueadc': 'UnicodeCharacter0xeadc',
u'\u1161': 'HangulJungseongA',
u'\u70e0': 'CjkUnifiedIdeograph70e0',
u'\ud463': 'HangulSyllablePyod',
u'\u5a67': 'CjkUnifiedIdeograph5a67',
u'\ub9e6': 'HangulSyllableMaegg',
u'\u206b': 'ActivateSymmetricSwapping',
u'\u3fea': 'CjkUnifiedIdeograph3fea',
u'\ue36d': 'UnicodeCharacter0xe36d',
u'\uc2ec': 'HangulSyllableSim',
u'\u6971': 'CjkUnifiedIdeograph6971',
u'\u48f0': 'CjkUnifiedIdeograph48f0',
u'\ub277': 'HangulSyllableNyugs',
u'\u91f6': 'CjkUnifiedIdeograph91f6',
u'\u387b': 'CjkUnifiedIdeograph387b',
u'\u17fa': 'UnicodeCharacter0x17fa',
u'\ufb7d': 'ArabicLetterTchehMedialForm',
u'\udafc': 'UnicodeCharacter0xdafc',
u'\uc081': 'HangulSyllableBbyilt',
u'\ua100': 'YiSyllableDit',
u'\u8583': 'CjkUnifiedIdeograph8583',
u'\u6202': 'CjkUnifiedIdeograph6202',
u'\u4685': 'CjkUnifiedIdeograph4685',
u'\u2704': 'WhiteScissors',
u'\u0b87': 'TamilLetterI',
u'\uf40e': 'UnicodeCharacter0xf40e',
u'\ud891': 'UnicodeCharacter0xd891',
u'\ub910': 'HangulSyllableRweols',
u'\u9d93': 'CjkUnifiedIdeograph9d93',
u'\u7a12': 'CjkUnifiedIdeograph7a12',
u'\u5e95': 'CjkUnifiedIdeograph5e95',
u'\u3f14': 'CjkUnifiedIdeograph3f14',
u'\u6397': 'CjkUnifiedIdeograph6397',
u'\u2499': 'NumberEighteenFullStop',
u'\ucc1e': 'HangulSyllableJjibs',
u'\u9120': 'CjkUnifiedIdeograph9120',
u'\uf5a3': 'UnicodeCharacter0xf5a3',
u'\u5222': 'CjkUnifiedIdeograph5222',
u'\ub6a5': 'HangulSyllableDdulg',
u'\u1724': 'HanunooLetterGa',
u'\u7ba7': 'CjkUnifiedIdeograph7ba7',
u'\u3ca9': 'CjkUnifiedIdeograph3ca9',
u'\ue930': 'UnicodeCharacter0xe930',
u'\ucdb3': 'HangulSyllableCweogs',
u'\uaa32': 'ChamVowelSignUe',
u'\u8eb5': 'CjkUnifiedIdeograph8eb5',
u'\u6f34': 'CjkUnifiedIdeograph6f34',
u'\u53b7': 'CjkUnifiedIdeograph53b7',
u'\u3036': 'CircledPostalMark',
u'\u14b9': 'CanadianSyllabicsWestcreeMwaa',
u'\uc140': 'HangulSyllableSel',
u'\u8242': 'CjkUnifiedIdeograph8242',
u'\ue6c5': 'UnicodeCharacter0xe6c5',
u'\u4744': 'CjkUnifiedIdeograph4744',
u'\uabc7': 'UnicodeCharacter0xabc7',
u'\u0846': 'UnicodeCharacter0x846',
u'\u6cc9': 'CjkUnifiedIdeograph6cc9',
u'\u31cb': 'CjkStrokeHzzp',
u'\ud950': 'UnicodeCharacter0xd950',
u'\u9a52': 'CjkUnifiedIdeograph9a52',
u'\ufed5': 'ArabicLetterQafIsolatedForm',
u'\u5f54': 'CjkUnifiedIdeograph5f54',
u'\u83d7': 'CjkUnifiedIdeograph83d7',
u'\u6056': 'CjkUnifiedIdeograph6056',
u'\u44d9': 'CjkUnifiedIdeograph44d9',
u'\u2558': 'BoxDrawingsUpSingleAndRightDouble',
u'\u09db': 'UnicodeCharacter0x9db',
u'\uf262': 'UnicodeCharacter0xf262',
u'\ud6e5': 'HangulSyllableHweonj',
u'\ub764': 'HangulSyllableDdin',
u'\u9be7': 'CjkUnifiedIdeograph9be7',
u'\u7866': 'CjkUnifiedIdeograph7866',
u'\u5ce9': 'CjkUnifiedIdeograph5ce9',
u'\u3d68': 'CjkUnifiedIdeograph3d68',
u'\u61eb': 'CjkUnifiedIdeograph61eb',
u'\u22ed': 'DoesNotContainAsNormalSubgroupOrEqual',
u'\uca72': 'HangulSyllableJjelm',
u'\u8f74': 'CjkUnifiedIdeograph8f74',
u'\uf3f7': 'UnicodeCharacter0xf3f7',
u'\u5076': 'CjkUnifiedIdeograph5076',
u'\ub4f9': 'HangulSyllableDyig',
u'\u1578': 'CanadianSyllabicsNunavikHoo',
u'\u79fb': 'CjkUnifiedIdeograph79fb',
u'\u3afd': 'CjkUnifiedIdeograph3afd',
u'\ue784': 'UnicodeCharacter0xe784',
u'\ua886': 'SaurashtraLetterU',
u'\u6d88': 'CjkUnifiedIdeograph6d88',
u'\ud10b': 'HangulSyllableTyas',
u'\u2e8a': 'CjkRadicalDivination',
u'\u920d': 'CjkUnifiedIdeograph920d',
u'\u570f': 'CjkUnifiedIdeograph570f',
u'\u1811': 'MongolianDigitOne',
u'\uff94': 'HalfwidthKatakanaLetterYa',
u'\u8096': 'CjkUnifiedIdeograph8096',
u'\u4598': 'CjkUnifiedIdeograph4598',
u'\u069a': 'ArabicLetterSeenWithDotBelowAndDotAbove',
u'\uea1d': 'UnicodeCharacter0xea1d',
u'\uaf1f': 'HangulSyllableGgyelh',
u'\u7021': 'CjkUnifiedIdeograph7021',
u'\u3523': 'CjkUnifiedIdeograph3523',
u'\ud7a4': 'UnicodeCharacter0xd7a4',
u'\u98a6': 'CjkUnifiedIdeograph98a6',
u'\u5da8': 'CjkUnifiedIdeograph5da8',
u'\u1eaa': 'LatinCapitalLetterAWithCircumflexAndTilde',
u'\uc22d': 'HangulSyllableSung',
u'\u23ac': 'RightCurlyBracketMiddlePiece',
u'\u872f': 'CjkUnifiedIdeograph872f',
u'\u4831': 'CjkUnifiedIdeograph4831',
u'\u0d33': 'MalayalamLetterLla',
u'\uf0b6': 'UnicodeCharacter0xf0b6',
u'\ub5b8': 'HangulSyllableDdeok',
u'\u76ba': 'CjkUnifiedIdeograph76ba',
u'\uda3d': 'UnicodeCharacter0xda3d',
u'\u3bbc': 'CjkUnifiedIdeograph3bbc',
u'\u9f3f': 'CjkUnifiedIdeograph9f3f',
u'\ua041': 'YiSyllablePa',
u'\u6543': 'CjkUnifiedIdeograph6543',
u'\u2645': 'Uranus',
u'\uc8c6': 'HangulSyllableJoegg',
u'\u8dc8': 'CjkUnifiedIdeograph8dc8',
u'\u4eca': 'CjkUnifiedIdeograph4eca',
u'\u13cc': 'CherokeeLetterSa',
u'\uf74f': 'UnicodeCharacter0xf74f',
u'\ub851': 'HangulSyllableRyeb',
u'\u7d53': 'CjkUnifiedIdeograph7d53',
u'\u3e55': 'CjkUnifiedIdeograph3e55',
u'\ue5d8': 'UnicodeCharacter0xe5d8',
u'\ua6da': 'UnicodeCharacter0xa6da',
u'\u6bdc': 'CjkUnifiedIdeograph6bdc',
u'\ucf5f': 'HangulSyllableKolb',
u'\u2cde': 'CopticCapitalLetterOldNubianNgi',
u'\u9061': 'CjkUnifiedIdeograph9061',
u'\u5563': 'CjkUnifiedIdeograph5563',
u'\u1665': 'CanadianSyllabicsCarrierChi',
u'\ufde8': 'UnicodeCharacter0xfde8',
u'\ubeea': 'HangulSyllableBbeop',
u'\u43ec': 'CjkUnifiedIdeograph43ec',
u'\u04ee': 'CyrillicCapitalLetterUWithMacron',
u'\ue871': 'UnicodeCharacter0xe871',
u'\uad73': 'HangulSyllableGud',
u'\u6e75': 'CjkUnifiedIdeograph6e75',
u'\u3377': 'SquareDm',
u'\ud5f8': 'HangulSyllableHess',
u'\u96fa': 'CjkUnifiedIdeograph96fa',
u'\u5bfc': 'CjkUnifiedIdeograph5bfc',
u'\u1cfe': 'UnicodeCharacter0x1cfe',
u'\u2200': 'ForAll',
u'\u0683': 'ArabicLetterNyeh',
u'\ue102': 'UnicodeCharacter0xe102',
u'\uc585': 'HangulSyllableYalg',
u'\u6b06': 'CjkUnifiedIdeograph6b06',
u'\u4f89': 'CjkUnifiedIdeograph4f89',
u'\ub00c': 'HangulSyllableGgwi',
u'\u948f': 'CjkUnifiedIdeograph948f',
u'\u3a10': 'CjkUnifiedIdeograph3a10',
u'\u1e93': 'LatinSmallLetterZWithDotBelow',
u'\uf912': 'CjkCompatibilityIdeographf912',
u'\udd95': 'UnicodeCharacter0xdd95',
u'\u4316': 'CjkUnifiedIdeograph4316',
u'\ua799': 'UnicodeCharacter0xa799',
u'\u299d': 'MeasuredRightAngleWithDot',
u'\u881c': 'CjkUnifiedIdeograph881c',
u'\uec9f': 'UnicodeCharacter0xec9f',
u'\u1220': 'EthiopicSyllableSza',
u'\u76a3': 'CjkUnifiedIdeograph76a3',
u'\ud122': 'HangulSyllableTyaelp',
u'\u5b26': 'CjkUnifiedIdeograph5b26',
u'\ubfa9': 'HangulSyllableBboeng',
u'\u01ad': 'LatinSmallLetterTWithHook',
u'\ue02c': 'UnicodeCharacter0xe02c',
u'\uc4af': 'HangulSyllableSswes',
u'\u6a30': 'CjkUnifiedIdeograph6a30',
u'\u4eb3': 'CjkUnifiedIdeograph4eb3',
u'\ub336': 'HangulSyllableDyap',
u'\u97b9': 'CjkUnifiedIdeograph97b9',
u'\u353a': 'CjkUnifiedIdeograph353a',
u'\u19bd': 'NewTaiLueVowelSignOy',
u'\uf83c': 'UnicodeCharacter0xf83c',
u'\udcbf': 'UnicodeCharacter0xdcbf',
u'\u4240': 'CjkUnifiedIdeograph4240',
u'\ua6c3': 'UnicodeCharacter0xa6c3',
u'\u28c7': 'BraillePatternDots12378',
u'\u8b46': 'CjkUnifiedIdeograph8b46',
u'\uefc9': 'UnicodeCharacter0xefc9',
u'\u0d4a': 'MalayalamVowelSignO',
u'\u71cd': 'CjkUnifiedIdeograph71cd',
u'\ud04c': 'HangulSyllableKwik',
u'\u5a50': 'CjkUnifiedIdeograph5a50',
u'\ubed3': 'HangulSyllableBbeogs',
u'\xd7': 'MultiplicationSign',
u'\ue356': 'UnicodeCharacter0xe356',
u'\uc7d9': 'HangulSyllableJyab',
u'\u655a': 'CjkUnifiedIdeograph655a',
u'\u49dd': 'CjkUnifiedIdeograph49dd',
u'\ub260': 'HangulSyllableNwil',
u'\u96e3': 'CjkUnifiedIdeograph96e3',
u'\u3464': 'CjkUnifiedIdeograph3464',
u'\u18e7': 'UnicodeCharacter0x18e7',
u'\ufb66': 'ArabicLetterTtehIsolatedForm',
u'\udfe9': 'UnicodeCharacter0xdfe9',
u'\u7d6a': 'CjkUnifiedIdeograph7d6a',
u'\ua1ed': 'YiSyllableGop',
u'\u2bf1': 'UnicodeCharacter0x2bf1',
u'\u8a70': 'CjkUnifiedIdeograph8a70',
u'\ueef3': 'UnicodeCharacter0xeef3',
u'\u0c74': 'UnicodeCharacter0xc74',
u'\u70f7': 'CjkUnifiedIdeograph70f7',
u'\ud376': 'HangulSyllablePyaej',
u'\u557a': 'CjkUnifiedIdeograph557a',
u'\ub9fd': 'HangulSyllableMaet',
u'\u8301': 'CjkUnifiedIdeograph8301',
u'\ue280': 'UnicodeCharacter0xe280',
u'\u0505': 'CyrillicSmallLetterKomiZje',
u'\u6484': 'CjkUnifiedIdeograph6484',
u'\uc807': 'HangulSyllableJeod',
u'\u520b': 'CjkUnifiedIdeograph520b',
u'\uad8a': 'HangulSyllableGweogg',
u'\u378e': 'CjkUnifiedIdeograph378e',
u'\u9b11': 'CjkUnifiedIdeograph9b11',
u'\ufa90': 'CjkCompatibilityIdeographfa90',
u'\u1d15': 'LatinLetterSmallCapitalOu',
u'\u7c94': 'CjkUnifiedIdeograph7c94',
u'\uaa1b': 'ChamLetterPpa',
u'\u859a': 'CjkUnifiedIdeograph859a',
u'\u2c1f': 'GlagoliticCapitalLetterYeru',
u'\u0f9e': 'TibetanSubjoinedLetterNna',
u'\uf321': 'UnicodeCharacter0xf321',
u'\ud2a0': 'HangulSyllableTyun',
u'\u7525': 'CjkUnifiedIdeograph7525',
u'\u54a4': 'CjkUnifiedIdeograph54a4',
u'\u822b': 'CjkUnifiedIdeograph822b',
u'\u9daa': 'CjkUnifiedIdeograph9daa',
u'\u042f': 'CyrillicCapitalLetterYa',
u'\u67ae': 'CjkUnifiedIdeograph67ae',
u'\ucb31': 'HangulSyllableJjyonj',
u'\u4d35': 'CjkUnifiedIdeograph4d35',
u'\uacb4': 'HangulSyllableGyeols',
u'\u36b8': 'CjkUnifiedIdeograph36b8',
u'\u9a3b': 'CjkUnifiedIdeograph9a3b',
u'\uf5ba': 'UnicodeCharacter0xf5ba',
u'\u1c3f': 'LepchaPunctuationTshook',
u'\u7fbe': 'CjkUnifiedIdeograph7fbe',
u'\ua545': 'VaiSyllableGi',
u'\u84c4': 'CjkUnifiedIdeograph84c4',
u'\u2f49': 'KangxiRadicalMoon',
u'\u0ec8': 'LaoToneMaiEk',
u'\uf24b': 'UnicodeCharacter0xf24b',
u'\ucdca': 'HangulSyllableCweop',
u'\u744f': 'CjkUnifiedIdeograph744f',
u'\u57ce': 'CjkUnifiedIdeograph57ce',
u'\ubd55': 'HangulSyllableBoelt',
u'\u9cd4': 'CjkUnifiedIdeograph9cd4',
u'\u0759': 'ArabicLetterDalWithTwoDotsVerticallyBelowAndSmallTah',
u'\u66d8': 'CjkUnifiedIdeograph66d8',
u'\uca5b': 'HangulSyllableJjeolh',
u'\u4c5f': 'CjkUnifiedIdeograph4c5f',
u'\uafde': 'HangulSyllableGgweolm',
u'\u31e2': 'CjkStrokePg',
u'\u9565': 'CjkUnifiedIdeograph9565',
u'\uf4e4': 'UnicodeCharacter0xf4e4',
u'\u1f69': 'GreekCapitalLetterOmegaWithDasia',
u'\u7ee8': 'CjkUnifiedIdeograph7ee8',
u'\ua46f': 'YiSyllableXyrx',
u'\u87ee': 'CjkUnifiedIdeograph87ee',
u'\u2e73': 'UnicodeCharacter0x2e73',
u'\u09f2': 'BengaliRupeeMark',
u'\ued75': 'UnicodeCharacter0xed75',
u'\uccf4': 'HangulSyllableCyel',
u'\u7779': 'CjkUnifiedIdeograph7779',
u'\u56f8': 'CjkUnifiedIdeograph56f8',
u'\ubc7f': 'HangulSyllableByaec',
u'\u9ffe': 'UnicodeCharacter0x9ffe',
u'\uea06': 'UnicodeCharacter0xea06',
u'\uce89': 'HangulSyllableKang',
u'\uaf08': 'HangulSyllableGgyeoss',
u'\u938b': 'CjkUnifiedIdeograph938b',
u'\u6c0a': 'CjkUnifiedIdeograph6c0a',
u'\u508d': 'CjkUnifiedIdeograph508d',
u'\u310c': 'BopomofoLetterL',
u'\u158f': 'CanadianSyllabicsNgi',
u'\uc216': 'HangulSyllableSyop',
u'\u8718': 'CjkUnifiedIdeograph8718',
u'\ueb9b': 'UnicodeCharacter0xeb9b',
u'\u441a': 'CjkUnifiedIdeograph441a',
u'\ua89d': 'SaurashtraLetterTtha',
u'\u091c': 'DevanagariLetterJa',
u'\u6d9f': 'CjkUnifiedIdeograph6d9f',
u'\u32a1': 'CircledIdeographRest',
u'\uda26': 'UnicodeCharacter0xda26',
u'\u9f28': 'CjkUnifiedIdeograph9f28',
u'\uc3ab': 'HangulSyllableSsyeolb',
u'\u5c2a': 'CjkUnifiedIdeograph5c2a',
u'\u80ad': 'CjkUnifiedIdeograph80ad',
u'\u612c': 'CjkUnifiedIdeograph612c',
u'\u45af': 'CjkUnifiedIdeograph45af',
u'\u262e': 'PeaceSymbol',
u'\u0ab1': 'UnicodeCharacter0xab1',
u'\uf738': 'UnicodeCharacter0xf738',
u'\udbbb': 'UnicodeCharacter0xdbbb',
u'\ub43a': 'HangulSyllableDyonh',
u'\u98bd': 'CjkUnifiedIdeograph98bd',
u'\u793c': 'CjkUnifiedIdeograph793c',
u'\u5dbf': 'CjkUnifiedIdeograph5dbf',
u'\u3e3e': 'CjkUnifiedIdeograph3e3e',
u'\u62c1': 'CjkUnifiedIdeograph62c1',
u'\u27c3': 'OpenSubset',
u'\ucf48': 'HangulSyllableKyem',
u'\u8c4a': 'CjkUnifiedIdeograph8c4a',
u'\uf0cd': 'UnicodeCharacter0xf0cd',
u'\u514c': 'CjkUnifiedIdeograph514c',
u'\ub5cf': 'HangulSyllableDdes',
u'\u164e': 'CanadianSyllabicsCarrierSu',
u'\u7ad1': 'CjkUnifiedIdeograph7ad1',
u'\u3fd3': 'CjkUnifiedIdeograph3fd3',
u'\ue45a': 'UnicodeCharacter0xe45a',
u'\uc8dd': 'HangulSyllableJoet',
u'\ua95c': 'UnicodeCharacter0xa95c',
u'\u8ddf': 'CjkUnifiedIdeograph8ddf',
u'\u6e5e': 'CjkUnifiedIdeograph6e5e',
u'\u52e1': 'CjkUnifiedIdeograph52e1',
u'\u3360': 'IdeographicTelegraphSymbolForHourEight',
u'\u17e3': 'KhmerDigitThree',
u'\ufc6a': 'ArabicLigatureBehWithRehFinalForm',
u'\u816c': 'CjkUnifiedIdeograph816c',
u'\ue5ef': 'UnicodeCharacter0xe5ef',
u'\u466e': 'CjkUnifiedIdeograph466e',
u'\uaaf1': 'UnicodeCharacter0xaaf1',
u'\u0b70': 'OriyaIsshar',
u'\u6ff3': 'CjkUnifiedIdeograph6ff3',
u'\u2cf5': 'UnicodeCharacter0x2cf5',
u'\ud47a': 'HangulSyllablePugg',
u'\u997c': 'CjkUnifiedIdeograph997c',
u'\ufdff': 'UnicodeCharacter0xfdff',
u'\u5e7e': 'CjkUnifiedIdeograph5e7e',
u'\u6380': 'CjkUnifiedIdeograph6380',
u'\uc703': 'HangulSyllableWeh',
u'\u2082': 'SubscriptTwo',
u'\u8405': 'CjkUnifiedIdeograph8405',
u'\u4907': 'CjkUnifiedIdeograph4907',
u'\u0e09': 'ThaiCharacterChoChing',
u'\uf18c': 'UnicodeCharacter0xf18c',
u'\ub68e': 'HangulSyllableDdyolp',
u'\u7b90': 'CjkUnifiedIdeograph7b90',
u'\udf13': 'UnicodeCharacter0xdf13',
u'\u3892': 'CjkUnifiedIdeograph3892',
u'\u9c15': 'CjkUnifiedIdeograph9c15',
u'\ua117': 'YiSyllableDup',
u'\u6619': 'CjkUnifiedIdeograph6619',
u'\u2b1b': 'BlackLargeSquare',
u'\uc99c': 'HangulSyllableJeuss',
u'\u8e9e': 'CjkUnifiedIdeograph8e9e',
u'\u53a0': 'CjkUnifiedIdeograph53a0',
u'\u10a2': 'GeorgianCapitalLetterGan',
u'\uf425': 'UnicodeCharacter0xf425',
u'\ub927': 'HangulSyllableRwed',
u'\u7e29': 'CjkUnifiedIdeograph7e29',
u'\u032b': 'CombiningInvertedDoubleArchBelow',
u'\ue6ae': 'UnicodeCharacter0xe6ae',
u'\uabb0': 'UnicodeCharacter0xabb0',
u'\u68b2': 'CjkUnifiedIdeograph68b2',
u'\ucc35': 'HangulSyllableCalt',
u'\u2db4': 'EthiopicSyllableZzee',
u'\u9137': 'CjkUnifiedIdeograph9137',
u'\u5639': 'CjkUnifiedIdeograph5639',
u'\u1b3b': 'BalineseVowelSignRaRepaTedung',
u'\ufebe': 'ArabicLetterDadFinalForm',
u'\u83c0': 'CjkUnifiedIdeograph83c0',
u'\u40c2': 'CjkUnifiedIdeograph40c2',
u'\u05c4': 'HebrewMarkUpperDot',
u'\ue947': 'UnicodeCharacter0xe947',
u'\uae49': 'HangulSyllableGit',
u'\u734b': 'CjkUnifiedIdeograph734b',
u'\u304d': 'HiraganaLetterKi',
u'\ud6ce': 'HangulSyllableHulm',
u'\u9bd0': 'CjkUnifiedIdeograph9bd0',
u'\u58d2': 'CjkUnifiedIdeograph58d2',
u'\u1dd4': 'CombiningLatinSmallLetterAe',
u'\uc157': 'HangulSyllableSyeogs',
u'\u22d6': 'LessthanWithDot',
u'\u8659': 'CjkUnifiedIdeograph8659',
u'\u4b5b': 'CjkUnifiedIdeograph4b5b',
u'\u085d': 'UnicodeCharacter0x85d',
u'\uf3e0': 'UnicodeCharacter0xf3e0',
u'\ub0e2': 'HangulSyllableNyabs',
u'\u75e4': 'CjkUnifiedIdeograph75e4',
u'\ud967': 'UnicodeCharacter0xd967',
u'\u3ae6': 'CjkUnifiedIdeograph3ae6',
u'\u9e69': 'CjkUnifiedIdeograph9e69',
u'\ua36b': 'YiSyllableCho',
u'\u606d': 'CjkUnifiedIdeograph606d',
u'\u256f': 'BoxDrawingsLightArcUpAndLeft',
u'\ucbf0': 'HangulSyllableJjyi',
u'\u88f2': 'CjkUnifiedIdeograph88f2',
u'\u4df4': 'HexagramForDevelopment',
u'\u12f6': 'EthiopicSyllableDo',
u'\uf679': 'UnicodeCharacter0xf679',
u'\ubb7b': 'HangulSyllableMwelh',
u'\u787d': 'CjkUnifiedIdeograph787d',
u'\u3d7f': 'CjkUnifiedIdeograph3d7f',
u'\u4581': 'CjkUnifiedIdeograph4581',
u'\ua204': 'YiSyllableKuo',
u'\u8e87': 'CjkUnifiedIdeograph8e87',
u'\u2808': 'BraillePatternDots4',
u'\u148b': 'CanadianSyllabicsCi',
u'\ueb0a': 'UnicodeCharacter0xeb0a',
u'\ud78d': 'HangulSyllableHinj',
u'\u710e': 'CjkUnifiedIdeograph710e',
u'\u5d91': 'CjkUnifiedIdeograph5d91',
u'\uba14': 'HangulSyllableMyass',
u'\ue697': 'UnicodeCharacter0xe697',
u'\u6c9b': 'CjkUnifiedIdeograph6c9b',
u'\uc31a': 'HangulSyllableSsaenh',
u'\u491e': 'CjkUnifiedIdeograph491e',
u'\ub5a1': 'HangulSyllableDdeog',
u'\u33a5': 'SquareMCubed',
u'\u9224': 'CjkUnifiedIdeograph9224',
u'\ufea7': 'ArabicLetterKhahInitialForm',
u'\u1828': 'MongolianLetterNa',
u'\u44ab': 'CjkUnifiedIdeograph44ab',
u'\udb2a': 'UnicodeCharacter0xdb2a',
u'\ua12e': 'YiSyllableTe',
u'\u8db1': 'CjkUnifiedIdeograph8db1',
u'\u2f32': 'KangxiRadicalDry',
u'\u0bb5': 'TamilLetterVa',
u'\uea34': 'UnicodeCharacter0xea34',
u'\ud6b7': 'HangulSyllableHyolh',
u'\u7038': 'CjkUnifiedIdeograph7038',
u'\u5cbb': 'CjkUnifiedIdeograph5cbb',
u'\ub93e': 'HangulSyllableRwigg',
u'\ue5c1': 'UnicodeCharacter0xe5c1',
u'\u0742': 'SyriacRukkakha',
u'\u63c5': 'CjkUnifiedIdeograph63c5',
u'\uc244': 'HangulSyllableSweom',
u'\u4848': 'CjkUnifiedIdeograph4848',
u'\ub4cb': 'HangulSyllableDyulb',
u'\u32cf': 'LimitedLiabilitySign',
u'\u914e': 'CjkUnifiedIdeograph914e',
u'\ufdd1': 'UnicodeCharacter0xfdd1',
u'\u1f52': 'GreekSmallLetterUpsilonWithPsiliAndVaria',
u'\u7bd5': 'CjkUnifiedIdeograph7bd5',
u'\uda54': 'UnicodeCharacter0xda54',
u'\ua058': 'YiSyllableBbi',
u'\u8cdb': 'CjkUnifiedIdeograph8cdb',
u'\u2e5c': 'UnicodeCharacter0x2e5c',
u'\u0adf': 'UnicodeCharacter0xadf',
u'\ue95e': 'UnicodeCharacter0xe95e',
u'\ud5e1': 'HangulSyllableHeot',
u'\u7762': 'CjkUnifiedIdeograph7762',
u'\u53e5': 'CjkUnifiedIdeograph53e5',
u'\ub868': 'HangulSyllableRols',
u'\ue4eb': 'UnicodeCharacter0xe4eb',
u'\u066c': 'ArabicThousandsSeparator',
u'\u62ef': 'CjkUnifiedIdeograph62ef',
u'\uc16e': 'HangulSyllableSyeop',
u'\u4f72': 'CjkUnifiedIdeograph4f72',
u'\uabf5': 'UnicodeCharacter0xabf5',
u'\u31f9': 'KatakanaLetterSmallHo',
u'\u9078': 'CjkUnifiedIdeograph9078',
u'\ufcfb': 'ArabicLigatureSeenWithAlefMaksuraIsolatedForm',
u'\u1e7c': 'LatinCapitalLetterVWithTilde',
u'\u7aff': 'CjkUnifiedIdeograph7aff',
u'\ud97e': 'UnicodeCharacter0xd97e',
u'\u4003': 'CjkUnifiedIdeograph4003',
u'\ua782': 'LatinCapitalLetterInsularR',
u'\u2d86': 'EthiopicSyllableToa',
u'\u8909': 'CjkUnifiedIdeograph8909',
u'\ue888': 'UnicodeCharacter0xe888',
u'\u170d': 'UnicodeCharacter0x170d',
u'\u768c': 'CjkUnifiedIdeograph768c',
u'\ud20f': 'HangulSyllableToeh',
u'\u5813': 'CjkUnifiedIdeograph5813',
u'\ubf92': 'HangulSyllableBbwaep',
u'\u2617': 'BlackShogiPiece',
u'\u0596': 'HebrewAccentTipeha',
u'\ue119': 'UnicodeCharacter0xe119',
u'\uc098': 'HangulSyllableBbil',
u'\u6f1d': 'CjkUnifiedIdeograph6f1d',
u'\u4e9c': 'CjkUnifiedIdeograph4e9c',
u'\ub023': 'HangulSyllableGgwic',
u'\u97a2': 'CjkUnifiedIdeograph97a2',
u'\u3e27': 'CjkUnifiedIdeograph3e27',
u'\u1da6': 'ModifierLetterSmallCapitalI',
u'\uf929': 'CjkCompatibilityIdeographf929',
u'\ud8a8': 'UnicodeCharacter0xd8a8',
u'\u472d': 'CjkUnifiedIdeograph472d',
u'\ua6ac': 'UnicodeCharacter0xa6ac',
u'\u2cb0': 'CopticCapitalLetterOou',
u'\u8833': 'CjkUnifiedIdeograph8833',
u'\uefb2': 'UnicodeCharacter0xefb2',
u'\u1637': 'CanadianSyllabicsCarrierTlhee',
u'\u75b6': 'CjkUnifiedIdeograph75b6',
u'\ud139': 'HangulSyllableTeolg',
u'\u5f3d': 'CjkUnifiedIdeograph5f3d',
u'\ubebc': 'HangulSyllableBbyael',
u'\u2541': 'BoxDrawingsDownHeavyAndUpHorizontalLight',
u'\u04c0': 'CyrillicLetterPalochka',
u'\ue043': 'UnicodeCharacter0xe043',
u'\uc7c2': 'HangulSyllableJaej',
u'\u6e47': 'CjkUnifiedIdeograph6e47',
u'\u4dc6': 'HexagramForTheArmy',
u'\ub74d': 'HangulSyllableDdyilg',
u'\u96cc': 'CjkUnifiedIdeograph96cc',
u'\u3d51': 'CjkUnifiedIdeograph3d51',
u'\u1cd0': 'UnicodeCharacter0x1cd0',
u'\uf853': 'UnicodeCharacter0xf853',
u'\udfd2': 'UnicodeCharacter0xdfd2',
u'\u4657': 'CjkUnifiedIdeograph4657',
u'\ua5d6': 'VaiSyllableZho',
u'\u23da': 'EarthGround',
u'\u8f5d': 'CjkUnifiedIdeograph8f5d',
u'\ueedc': 'UnicodeCharacter0xeedc',
u'\u1561': 'CanadianSyllabicsNcreeThi',
u'\u74e0': 'CjkUnifiedIdeograph74e0',
u'\ud063': 'HangulSyllableKyus',
u'\u5e67': 'CjkUnifiedIdeograph5e67',
u'\ubde6': 'HangulSyllableBwibs',
u'\u246b': 'CircledNumberTwelve',
u'\u3bea': 'CjkUnifiedIdeograph3bea',
u'\ue76d': 'UnicodeCharacter0xe76d',
u'\uc6ec': 'HangulSyllableWen',
u'\u6d71': 'CjkUnifiedIdeograph6d71',
u'\u4cf0': 'CjkUnifiedIdeograph4cf0',
u'\ub677': 'HangulSyllableDdoes',
u'\u95f6': 'CjkUnifiedIdeograph95f6',
u'\u3c7b': 'CjkUnifiedIdeograph3c7b',
u'\u13fa': 'UnicodeCharacter0x13fa',
u'\uff7d': 'HalfwidthKatakanaLetterSu',
u'\udefc': 'UnicodeCharacter0xdefc',
u'\uc481': 'HangulSyllableSsweog',
u'\ua500': 'VaiSyllableEe',
u'\u8183': 'CjkUnifiedIdeograph8183',
u'\u6602': 'CjkUnifiedIdeograph6602',
u'\u4285': 'CjkUnifiedIdeograph4285',
u'\u2304': 'DownArrowhead',
u'\u0f87': 'TibetanSignYangRtags',
u'\uf00e': 'UnicodeCharacter0xf00e',
u'\udc91': 'UnicodeCharacter0xdc91',
u'\ubd10': 'HangulSyllableBwa',
u'\u9993': 'CjkUnifiedIdeograph9993',
u'\u7e12': 'CjkUnifiedIdeograph7e12',
u'\u5a95': 'CjkUnifiedIdeograph5a95',
u'\u3b14': 'CjkUnifiedIdeograph3b14',
u'\u6797': 'CjkUnifiedIdeograph6797',
u'\u2099': 'UnicodeCharacter0x2099',
u'\uc81e': 'HangulSyllableJegg',
u'\u9520': 'CjkUnifiedIdeograph9520',
u'\uf1a3': 'UnicodeCharacter0xf1a3',
u'\u5622': 'CjkUnifiedIdeograph5622',
u'\ub2a5': 'HangulSyllableNeung',
u'\u1324': 'EthiopicSyllableThee',
u'\u7fa7': 'CjkUnifiedIdeograph7fa7',
u'\u38a9': 'CjkUnifiedIdeograph38a9',
u'\ued30': 'UnicodeCharacter0xed30',
u'\uc9b3': 'HangulSyllableJyilh',
u'\uae32': 'HangulSyllableGigg',
u'\u8ab5': 'CjkUnifiedIdeograph8ab5',
u'\u6b34': 'CjkUnifiedIdeograph6b34',
u'\u57b7': 'CjkUnifiedIdeograph57b7',
u'\u3436': 'CjkUnifiedIdeograph3436',
u'\u10b9': 'GeorgianCapitalLetterChin',
u'\uc540': 'HangulSyllableSsik',
u'\u8642': 'CjkUnifiedIdeograph8642',
u'\ue2c5': 'UnicodeCharacter0xe2c5',
u'\u4344': 'CjkUnifiedIdeograph4344',
u'\uafc7': 'HangulSyllableGgulh',
u'\u0c46': 'TeluguVowelSignE',
u'\u68c9': 'CjkUnifiedIdeograph68c9',
u'\u35cb': 'CjkUnifiedIdeograph35cb',
u'\udd50': 'UnicodeCharacter0xdd50',
u'\u9e52': 'CjkUnifiedIdeograph9e52',
u'\ufad5': 'CjkCompatibilityIdeographfad5',
u'\u5b54': 'CjkUnifiedIdeograph5b54',
u'\u87d7': 'CjkUnifiedIdeograph87d7',
u'\u6456': 'CjkUnifiedIdeograph6456',
u'\u40d9': 'CjkUnifiedIdeograph40d9',
u'\u2158': 'VulgarFractionFourFifths',
u'\u0ddb': 'SinhalaVowelSignKombuDeka',
u'\uf662': 'UnicodeCharacter0xf662',
u'\ud2e5': 'HangulSyllableTyib',
u'\ub364': 'HangulSyllableDeom',
u'\u9fe7': 'UnicodeCharacter0x9fe7',
u'\u7c66': 'CjkUnifiedIdeograph7c66',
u'\u58e9': 'CjkUnifiedIdeograph58e9',
u'\u3968': 'CjkUnifiedIdeograph3968',
u'\u65eb': 'CjkUnifiedIdeograph65eb',
u'\u26ed': 'UnicodeCharacter0x26ed',
u'\uce72': 'HangulSyllableCip',
u'\u8b74': 'CjkUnifiedIdeograph8b74',
u'\uf7f7': 'UnicodeCharacter0xf7f7',
u'\u5476': 'CjkUnifiedIdeograph5476',
u'\ub0f9': 'HangulSyllableNyaelt',
u'\u1178': 'HangulJungseongYao',
u'\u7dfb': 'CjkUnifiedIdeograph7dfb',
u'\u3efd': 'CjkUnifiedIdeograph3efd',
u'\u0401': 'CyrillicCapitalLetterIo',
u'\ue384': 'UnicodeCharacter0xe384',
u'\uac86': 'HangulSyllableGeoj',
u'\u6988': 'CjkUnifiedIdeograph6988',
u'\ud50b': 'HangulSyllablePeud',
u'\u2a8a': 'GreaterthanAndNotApproximate',
u'\u960d': 'CjkUnifiedIdeograph960d',
u'\u530f': 'CjkUnifiedIdeograph530f',
u'\u1c11': 'LepchaLetterFa',
u'\ufb94': 'ArabicLetterGafInitialForm',
u'\u8496': 'CjkUnifiedIdeograph8496',
u'\u4198': 'CjkUnifiedIdeograph4198',
u'\u029a': 'LatinSmallLetterClosedOpenE',
u'\uee1d': 'UnicodeCharacter0xee1d',
u'\uab1f': 'UnicodeCharacter0xab1f',
u'\u7421': 'CjkUnifiedIdeograph7421',
u'\u3123': 'BopomofoLetterEn',
u'\ud3a4': 'HangulSyllablePels',
u'\u9ca6': 'CjkUnifiedIdeograph9ca6',
u'\u59a8': 'CjkUnifiedIdeograph59a8',
u'\u1aaa': 'UnicodeCharacter0x1aaa',
u'\uc62d': 'HangulSyllableOlg',
u'\u27ac': 'FronttiltedShadowedWhiteRightwardsArrow',
u'\u832f': 'CjkUnifiedIdeograph832f',
u'\u4c31': 'CjkUnifiedIdeograph4c31',
u'\u0933': 'DevanagariLetterLla',
u'\uf4b6': 'UnicodeCharacter0xf4b6',
u'\ub1b8': 'HangulSyllableNwael',
u'\u72ba': 'CjkUnifiedIdeograph72ba',
u'\ude3d': 'UnicodeCharacter0xde3d',
u'\u3fbc': 'CjkUnifiedIdeograph3fbc',
u'\u9b3f': 'CjkUnifiedIdeograph9b3f',
u'\ua441': 'YiSyllableNjup',
u'\u6143': 'CjkUnifiedIdeograph6143',
u'\u2245': 'ApproximatelyEqualTo',
u'\uccc6': 'HangulSyllableCebs',
u'\u89c8': 'CjkUnifiedIdeograph89c8',
u'\u4aca': 'CjkUnifiedIdeograph4aca',
u'\u17cc': 'KhmerSignRobat',
u'\uf34f': 'UnicodeCharacter0xf34f',
u'\ubc51': 'HangulSyllableByanj',
u'\u7953': 'CjkUnifiedIdeograph7953',
u'\u3a55': 'CjkUnifiedIdeograph3a55',
u'\ue1d8': 'UnicodeCharacter0xe1d8',
u'\ua2da': 'YiSyllableZza',
u'\u6fdc': 'CjkUnifiedIdeograph6fdc',
u'\ucb5f': 'HangulSyllableJjuc',
u'\u28de': 'BraillePatternDots234578',
u'\u9461': 'CjkUnifiedIdeograph9461',
u'\u5163': 'CjkUnifiedIdeograph5163',
u'\u1265': 'EthiopicSyllableBe',
u'\uf9e8': 'CjkCompatibilityIdeographf9e8',
u'\ubaea': 'HangulSyllableMwaelm',
u'\u47ec': 'CjkUnifiedIdeograph47ec',
u'\xee': 'LatinSmallLetterIWithCircumflex',
u'\uec71': 'UnicodeCharacter0xec71',
u'\ua973': 'UnicodeCharacter0xa973',
u'\u6a75': 'CjkUnifiedIdeograph6a75',
u'\u3777': 'CjkUnifiedIdeograph3777',
u'\ud1f8': 'HangulSyllableToen',
u'\u92fa': 'CjkUnifiedIdeograph92fa',
u'\u5ffc': 'CjkUnifiedIdeograph5ffc',
u'\u18fe': 'UnicodeCharacter0x18fe',
u'\u2600': 'BlackSunWithRays',
u'\u0283': 'LatinSmallLetterEsh',
u'\ue502': 'UnicodeCharacter0xe502',
u'\uc185': 'HangulSyllableSyeng',
u'\u6f06': 'CjkUnifiedIdeograph6f06',
u'\u4b89': 'CjkUnifiedIdeograph4b89',
u'\ub40c': 'HangulSyllableDwaem',
u'\u908f': 'CjkUnifiedIdeograph908f',
u'\u3e10': 'CjkUnifiedIdeograph3e10',
u'\u1a93': 'UnicodeCharacter0x1a93',
u'\ufd12': 'ArabicLigatureTahWithYehFinalForm',
u'\ud995': 'UnicodeCharacter0xd995',
u'\u4716': 'CjkUnifiedIdeograph4716',
u'\ua399': 'YiSyllableNro',
u'\u2d9d': 'UnicodeCharacter0x2d9d',
u'\u8c1c': 'CjkUnifiedIdeograph8c1c',
u'\ue89f': 'UnicodeCharacter0xe89f',
u'\u1620': 'CanadianSyllabicsCarrierJji',
u'\u72a3': 'CjkUnifiedIdeograph72a3',
u'\ud522': 'HangulSyllablePyigg',
u'\u5f26': 'CjkUnifiedIdeograph5f26',
u'\ubba9': 'HangulSyllableMyunj',
u'\u05ad': 'HebrewAccentDehi',
u'\ue42c': 'UnicodeCharacter0xe42c',
u'\uc0af': 'HangulSyllableSags',
u'\u6e30': 'CjkUnifiedIdeograph6e30',
u'\u4ab3': 'CjkUnifiedIdeograph4ab3',
u'\ub736': 'HangulSyllableDdeulp',
u'\u93b9': 'CjkUnifiedIdeograph93b9',
u'\u313a': 'HangulLetterRieulkiyeok',
u'\u1dbd': 'ModifierLetterSmallZWithCurl',
u'\ufc3c': 'ArabicLigatureKafWithMeemIsolatedForm',
u'\ud8bf': 'UnicodeCharacter0xd8bf',
u'\u4640': 'CjkUnifiedIdeograph4640',
u'\ua2c3': 'YiSyllableCep',
u'\u2cc7': 'CopticSmallLetterOldCopticEsh',
u'\u8f46': 'CjkUnifiedIdeograph8f46',
u'\uebc9': 'UnicodeCharacter0xebc9',
u'\u094a': 'DevanagariVowelSignShortO',
u'\u75cd': 'CjkUnifiedIdeograph75cd',
u'\ud44c': 'HangulSyllablePoels',
u'\u5e50': 'CjkUnifiedIdeograph5e50',
u'\ubad3': 'HangulSyllableMwalh',
u'\u04d7': 'CyrillicSmallLetterIeWithBreve',
u'\ue756': 'UnicodeCharacter0xe756',
u'\uc3d9': 'HangulSyllableSsog',
u'\u615a': 'CjkUnifiedIdeograph615a',
u'\u4ddd': 'HexagramForTheClingingFire',
u'\ub660': 'HangulSyllableDdwaek',
u'\u92e3': 'CjkUnifiedIdeograph92e3',
u'\u3064': 'HiraganaLetterTu',
u'\u1ce7': 'UnicodeCharacter0x1ce7',
u'\uff66': 'HalfwidthKatakanaLetterWo',
u'\udbe9': 'UnicodeCharacter0xdbe9',
u'\u796a': 'CjkUnifiedIdeograph796a',
u'\ua5ed': 'VaiSyllableKpen',
u'\u2ff1': 'IdeographicDescriptionCharacterAboveToBelow',
u'\u8e70': 'CjkUnifiedIdeograph8e70',
u'\ueaf3': 'UnicodeCharacter0xeaf3',
u'\u0874': 'UnicodeCharacter0x874',
u'\u74f7': 'CjkUnifiedIdeograph74f7',
u'\ud776': 'HangulSyllableHyilm',
u'\u517a': 'CjkUnifiedIdeograph517a',
u'\ubdfd': 'HangulSyllableByult',
u'\u8701': 'CjkUnifiedIdeograph8701',
u'\ue680': 'UnicodeCharacter0xe680',
u'\u0105': 'LatinSmallLetterAWithOgonek',
u'\u6084': 'CjkUnifiedIdeograph6084',
u'\ucc07': 'HangulSyllableJjyic',
u'\u560b': 'CjkUnifiedIdeograph560b',
u'\ua98a': 'UnicodeCharacter0xa98a',
u'\u338e': 'SquareMg',
u'\u9f11': 'CjkUnifiedIdeograph9f11',
u'\ufe90': 'ArabicLetterBehFinalForm',
u'\u1915': 'LimbuLetterYa',
u'\u7894': 'CjkUnifiedIdeograph7894',
u'\uae1b': 'HangulSyllableGyid',
u'\u819a': 'CjkUnifiedIdeograph819a',
u'\u281f': 'BraillePatternDots12345',
u'\u0b9e': 'TamilLetterNya',
u'\uf721': 'UnicodeCharacter0xf721',
u'\ud6a0': 'HangulSyllableHoess',
u'\u7125': 'CjkUnifiedIdeograph7125',
u'\u50a4': 'CjkUnifiedIdeograph50a4',
u'\u862b': 'CjkUnifiedIdeograph862b',
u'\u99aa': 'CjkUnifiedIdeograph99aa',
u'\u63ae': 'CjkUnifiedIdeograph63ae',
u'\ucf31': 'HangulSyllableKyeong',
u'\u4935': 'CjkUnifiedIdeograph4935',
u'\ua8b4': 'SaurashtraConsonantSignHaaru',
u'\u32b8': 'CircledNumberFortyThree',
u'\u9e3b': 'CjkUnifiedIdeograph9e3b',
u'\uf1ba': 'UnicodeCharacter0xf1ba',
u'\u183f': 'MongolianLetterZra',
u'\u7bbe': 'CjkUnifiedIdeograph7bbe',
u'\ua145': 'YiSyllableDdox',
u'\u80c4': 'CjkUnifiedIdeograph80c4',
u'\u2b49': 'TildeOperatorAboveLeftwardsArrow',
u'\u0ac8': 'GujaratiVowelSignAi',
u'\uf64b': 'UnicodeCharacter0xf64b',
u'\uc9ca': 'HangulSyllableJilm',
u'\u704f': 'CjkUnifiedIdeograph704f',
u'\u53ce': 'CjkUnifiedIdeograph53ce',
u'\ub955': 'HangulSyllableRwit',
u'\u98d4': 'CjkUnifiedIdeograph98d4',
u'\u0359': 'CombiningAsteriskBelow',
u'\u62d8': 'CjkUnifiedIdeograph62d8',
u'\uce5b': 'HangulSyllableCigs',
u'\u485f': 'CjkUnifiedIdeograph485f',
u'\uabde': 'UnicodeCharacter0xabde',
u'\u35e2': 'CjkUnifiedIdeograph35e2',
u'\u9165': 'CjkUnifiedIdeograph9165',
u'\uf0e4': 'UnicodeCharacter0xf0e4',
u'\u1b69': 'BalineseMusicalSymbolDaing',
u'\u7ae8': 'CjkUnifiedIdeograph7ae8',
u'\ua06f': 'YiSyllableBbup',
u'\u83ee': 'CjkUnifiedIdeograph83ee',
u'\u2a73': 'EqualsSignAboveTildeOperator',
u'\u0df2': 'SinhalaVowelSignDigaGaettapilla',
u'\ue975': 'UnicodeCharacter0xe975',
u'\uc8f4': 'HangulSyllableJyoss',
u'\u7379': 'CjkUnifiedIdeograph7379',
u'\u52f8': 'CjkUnifiedIdeograph52f8',
u'\ub87f': 'HangulSyllableRwad',
u'\u9bfe': 'CjkUnifiedIdeograph9bfe',
u'\uee06': 'UnicodeCharacter0xee06',
u'\uca89': 'HangulSyllableJjyeonj',
u'\uab08': 'UnicodeCharacter0xab08',
u'\u978b': 'CjkUnifiedIdeograph978b',
u'\u680a': 'CjkUnifiedIdeograph680a',
u'\u548d': 'CjkUnifiedIdeograph548d',
u'\u350c': 'CjkUnifiedIdeograph350c',
u'\u118f': 'HangulJungseongYueo',
u'\uc616': 'HangulSyllableYelp',
u'\u8318': 'CjkUnifiedIdeograph8318',
u'\uef9b': 'UnicodeCharacter0xef9b',
u'\u401a': 'CjkUnifiedIdeograph401a',
u'\uac9d': 'HangulSyllableGeb',
u'\u0d1c': 'MalayalamLetterJa',
u'\u699f': 'CjkUnifiedIdeograph699f',
u'\u36a1': 'CjkUnifiedIdeograph36a1',
u'\ude26': 'UnicodeCharacter0xde26',
u'\u9b28': 'CjkUnifiedIdeograph9b28',
u'\uc7ab': 'HangulSyllableJah',
u'\u582a': 'CjkUnifiedIdeograph582a',
u'\u84ad': 'CjkUnifiedIdeograph84ad',
u'\u652c': 'CjkUnifiedIdeograph652c',
u'\u41af': 'CjkUnifiedIdeograph41af',
u'\u222e': 'ContourIntegral',
u'\u0eb1': 'LaoVowelSignMaiKan',
u'\uf338': 'UnicodeCharacter0xf338',
u'\udfbb': 'UnicodeCharacter0xdfbb',
u'\ub03a': 'HangulSyllableGgyubs',
u'\u9cbd': 'CjkUnifiedIdeograph9cbd',
u'\u7d3c': 'CjkUnifiedIdeograph7d3c',
u'\u59bf': 'CjkUnifiedIdeograph59bf',
u'\u3a3e': 'CjkUnifiedIdeograph3a3e',
u'\u66c1': 'CjkUnifiedIdeograph66c1',
u'\u23c3': 'DentistrySymbolLightVerticalWithTriangle',
u'\ucb48': 'HangulSyllableJju',
u'\u884a': 'CjkUnifiedIdeograph884a',
u'\uf4cd': 'UnicodeCharacter0xf4cd',
u'\u554c': 'CjkUnifiedIdeograph554c',
u'\ub1cf': 'HangulSyllableNoegs',
u'\u124e': 'UnicodeCharacter0x124e',
u'\u7ed1': 'CjkUnifiedIdeograph7ed1',
u'\u3bd3': 'CjkUnifiedIdeograph3bd3',
u'\ue05a': 'UnicodeCharacter0xe05a',
u'\uccdd': 'HangulSyllableCyeolt',
u'\uad5c': 'HangulSyllableGyols',
u'\u89df': 'CjkUnifiedIdeograph89df',
u'\u6a5e': 'CjkUnifiedIdeograph6a5e',
u'\u56e1': 'CjkUnifiedIdeograph56e1',
u'\u3760': 'CjkUnifiedIdeograph3760',
u'\u13e3': 'CherokeeLetterTsa',
u'\uf86a': 'UnicodeCharacter0xf86a',
u'\u856c': 'CjkUnifiedIdeograph856c',
u'\ue1ef': 'UnicodeCharacter0xe1ef',
u'\u426e': 'CjkUnifiedIdeograph426e',
u'\uaef1': 'HangulSyllableGget',
u'\u0f70': 'UnicodeCharacter0xf70',
u'\u6bf3': 'CjkUnifiedIdeograph6bf3',
u'\u28f5': 'BraillePatternDots135678',
u'\ud07a': 'HangulSyllableKeulp',
u'\u9d7c': 'CjkUnifiedIdeograph9d7c',
u'\uf9ff': 'CjkCompatibilityIdeographf9ff',
u'\u5a7e': 'CjkUnifiedIdeograph5a7e',
u'\u6780': 'CjkUnifiedIdeograph6780',
u'\uc303': 'HangulSyllableSsalb',
u'\u2482': 'ParenthesizedNumberFifteen',
u'\u8005': 'CjkUnifiedIdeograph8005',
u'\u4d07': 'CjkUnifiedIdeograph4d07',
u'\u0a09': 'GurmukhiLetterU',
u'\uf58c': 'UnicodeCharacter0xf58c',
u'\ub28e': 'HangulSyllableNyup',
u'\u7f90': 'CjkUnifiedIdeograph7f90',
u'\udb13': 'UnicodeCharacter0xdb13',
u'\u3c92': 'CjkUnifiedIdeograph3c92',
u'\u9815': 'CjkUnifiedIdeograph9815',
u'\ua517': 'VaiSyllableShee',
u'\u6219': 'CjkUnifiedIdeograph6219',
u'\u2f1b': 'KangxiRadicalPrivate',
u'\ucd9c': 'HangulSyllableCul',
u'\u8a9e': 'CjkUnifiedIdeograph8a9e',
u'\u57a0': 'CjkUnifiedIdeograph57a0',
u'\u14a2': 'CanadianSyllabicsSayisiTh',
u'\uf025': 'UnicodeCharacter0xf025',
u'\ubd27': 'HangulSyllableBwac',
u'\u7a29': 'CjkUnifiedIdeograph7a29',
u'\u072b': 'SyriacLetterShin',
u'\ue2ae': 'UnicodeCharacter0xe2ae',
u'\uafb0': 'HangulSyllableGgyoss',
u'\u6cb2': 'CjkUnifiedIdeograph6cb2',
u'\uc835': 'HangulSyllableJet',
u'\u29b4': 'EmptySetWithLeftArrowAbove',
u'\u9537': 'CjkUnifiedIdeograph9537',
u'\u5239': 'CjkUnifiedIdeograph5239',
u'\u1f3b': 'GreekCapitalLetterIotaWithDasiaAndVaria',
u'\ufabe': 'CjkCompatibilityIdeographfabe',
u'\u87c0': 'CjkUnifiedIdeograph87c0',
u'\u44c2': 'CjkUnifiedIdeograph44c2',
u'\u01c4': 'LatinCapitalLetterDzWithCaron',
u'\ued47': 'UnicodeCharacter0xed47',
u'\uaa49': 'ChamLetterFinalR',
u'\u774b': 'CjkUnifiedIdeograph774b',
u'\u344d': 'CjkUnifiedIdeograph344d',
u'\ud2ce': 'HangulSyllableTeuj',
u'\u9fd0': 'UnicodeCharacter0x9fd0',
u'\u5cd2': 'CjkUnifiedIdeograph5cd2',
u'\u19d4': 'NewTaiLueDigitFour',
u'\uc557': 'HangulSyllableAs',
u'\u26d6': 'UnicodeCharacter0x26d6',
u'\u8259': 'CjkUnifiedIdeograph8259',
u'\u4f5b': 'CjkUnifiedIdeograph4f5b',
u'\u0c5d': 'UnicodeCharacter0xc5d',
u'\uf7e0': 'UnicodeCharacter0xf7e0',
u'\ub4e2': 'HangulSyllableDeunh',
u'\u71e4': 'CjkUnifiedIdeograph71e4',
u'\udd67': 'UnicodeCharacter0xdd67',
u'\u3ee6': 'CjkUnifiedIdeograph3ee6',
u'\u9a69': 'CjkUnifiedIdeograph9a69',
u'\ua76b': 'LatinSmallLetterEt',
u'\u646d': 'CjkUnifiedIdeograph646d',
u'\u216f': 'RomanNumeralOneThousand',
u'\ucff0': 'HangulSyllableKum',
u'\u8cf2': 'CjkUnifiedIdeograph8cf2',
u'\u49f4': 'CjkUnifiedIdeograph49f4',
u'\u16f6': 'UnicodeCharacter0x16f6',
u'\uf279': 'UnicodeCharacter0xf279',
u'\ubf7b': 'HangulSyllableBbwaegs',
u'\u7c7d': 'CjkUnifiedIdeograph7c7d',
u'\u397f': 'CjkUnifiedIdeograph397f',
u'\u4981': 'CjkUnifiedIdeograph4981',
u'\uae04': 'HangulSyllableGeuls',
u'\u8287': 'CjkUnifiedIdeograph8287',
u'\u2408': 'SymbolForBackspace',
u'\u188b': 'MongolianLetterAliGaliCa',
u'\ue70a': 'UnicodeCharacter0xe70a',
u'\udb8d': 'UnicodeCharacter0xdb8d',
u'\u7d0e': 'CjkUnifiedIdeograph7d0e',
u'\u5191': 'CjkUnifiedIdeograph5191',
u'\ub614': 'HangulSyllableDdon',
u'\uea97': 'UnicodeCharacter0xea97',
u'\u0c18': 'TeluguLetterGha',
u'\u609b': 'CjkUnifiedIdeograph609b',
u'\ucf1a': 'HangulSyllableKep',
u'\u451e': 'CjkUnifiedIdeograph451e',
u'\ub9a1': 'HangulSyllableRyib',
u'\u3fa5': 'CjkUnifiedIdeograph3fa5',
u'\u9e24': 'CjkUnifiedIdeograph9e24',
u'\uf2a7': 'UnicodeCharacter0xf2a7',
u'\u1428': 'CanadianSyllabicsFinalShortHorizontalStroke',
u'\u48ab': 'CjkUnifiedIdeograph48ab',
u'\ud72a': 'HangulSyllableHwibs',
u'\uad2e': 'HangulSyllableGwaej',
u'\u81b1': 'CjkUnifiedIdeograph81b1',
u'\u2332': 'ConicalTaper',
u'\u07b5': 'UnicodeCharacter0x7b5',
u'\ue634': 'UnicodeCharacter0xe634',
u'\udab7': 'UnicodeCharacter0xdab7',
u'\u7c38': 'CjkUnifiedIdeograph7c38',
u'\u50bb': 'CjkUnifiedIdeograph50bb',
u'\ub53e': 'HangulSyllableDdalp',
u'\ue9c1': 'UnicodeCharacter0xe9c1',
u'\u0b42': 'OriyaVowelSignUu',
u'\u6fc5': 'CjkUnifiedIdeograph6fc5',
u'\uce44': 'HangulSyllableCyil',
u'\u4448': 'CjkUnifiedIdeograph4448',
u'\ub8cb': 'HangulSyllableRoeh',
u'\u3ecf': 'CjkUnifiedIdeograph3ecf',
u'\u9d4e': 'CjkUnifiedIdeograph9d4e',
u'\uf1d1': 'UnicodeCharacter0xf1d1',
u'\u1352': 'EthiopicSyllablePi',
u'\u77d5': 'CjkUnifiedIdeograph77d5',
u'\ud654': 'HangulSyllableHwa',
u'\uac58': 'HangulSyllableGyaen',
u'\u80db': 'CjkUnifiedIdeograph80db',
u'\u225c': 'DeltaEqualTo',
u'\u06df': 'ArabicSmallHighRoundedZero',
u'\ue55e': 'UnicodeCharacter0xe55e',
u'\ud9e1': 'UnicodeCharacter0xd9e1',
u'\u7b62': 'CjkUnifiedIdeograph7b62',
u'\u5fe5': 'CjkUnifiedIdeograph5fe5',
u'\ub468': 'HangulSyllableDuk',
u'\ue8eb': 'UnicodeCharacter0xe8eb',
u'\u0a6c': 'GurmukhiDigitSix',
u'\u6eef': 'CjkUnifiedIdeograph6eef',
u'\ucd6e': 'HangulSyllableCoebs',
u'\u4372': 'CjkUnifiedIdeograph4372',
u'\ua7f5': 'UnicodeCharacter0xa7f5',
u'\u3df9': 'CjkUnifiedIdeograph3df9',
u'\u9c78': 'CjkUnifiedIdeograph9c78',
u'\uf0fb': 'UnicodeCharacter0xf0fb',
u'\u127c': 'EthiopicSyllableCee',
u'\u76ff': 'CjkUnifiedIdeograph76ff',
u'\ud57e': 'HangulSyllableHaelm',
u'\u4c03': 'CjkUnifiedIdeograph4c03',
u'\uab82': 'UnicodeCharacter0xab82',
u'\u2186': 'RomanNumeralFiftyEarlyForm',
u'\u8509': 'CjkUnifiedIdeograph8509',
u'\ue488': 'UnicodeCharacter0xe488',
u'\u1b0d': 'BalineseLetterLaLenga',
u'\u7a8c': 'CjkUnifiedIdeograph7a8c',
u'\ude0f': 'UnicodeCharacter0xde0f',
u'\u5413': 'CjkUnifiedIdeograph5413',
u'\ub392': 'HangulSyllableDyeonh',
u'\u2a17': 'IntegralWithLeftwardsArrowWithHook',
u'\u0996': 'BengaliLetterKha',
u'\ued19': 'UnicodeCharacter0xed19',
u'\ucc98': 'HangulSyllableCeo',
u'\u631d': 'CjkUnifiedIdeograph631d',
u'\u429c': 'CjkUnifiedIdeograph429c',
u'\ubc23': 'HangulSyllableBalh',
u'\u9ba2': 'CjkUnifiedIdeograph9ba2',
u'\u3227': 'ParenthesizedIdeographEight',
u'\u11a6': 'UnicodeCharacter0x11a6',
u'\uf529': 'UnicodeCharacter0xf529',
u'\ud4a8': 'HangulSyllablePweoss',
u'\u4b2d': 'CjkUnifiedIdeograph4b2d',
u'\uaaac': 'UnicodeCharacter0xaaac',
u'\u20b0': 'GermanPennySign',
u'\u8433': 'CjkUnifiedIdeograph8433',
u'\ue3b2': 'UnicodeCharacter0xe3b2',
u'\u1a37': 'UnicodeCharacter0x1a37',
u'\u79b6': 'CjkUnifiedIdeograph79b6',
u'\udd39': 'UnicodeCharacter0xdd39',
u'\u533d': 'CjkUnifiedIdeograph533d',
u'\ub2bc': 'HangulSyllableNyim',
u'\u2941': 'ClockwiseClosedCircleArrow',
u'\u08c0': 'UnicodeCharacter0x8c0',
u'\uec43': 'UnicodeCharacter0xec43',
u'\ucbc2': 'HangulSyllableJjyulm',
u'\u6247': 'CjkUnifiedIdeograph6247',
u'\u41c6': 'CjkUnifiedIdeograph41c6',
u'\ubb4d': 'HangulSyllableMut',
u'\u9acc': 'CjkUnifiedIdeograph9acc',
u'\u3151': 'HangulLetterYa',
u'\u10d0': 'GeorgianLetterAn',
u'\uf453': 'UnicodeCharacter0xf453',
u'\ud3d2': 'HangulSyllablePyegg',
u'\u4a57': 'CjkUnifiedIdeograph4a57',
u'\ua9d6': 'UnicodeCharacter0xa9d6',
u'\u2fda': 'UnicodeCharacter0x2fda',
u'\u835d': 'CjkUnifiedIdeograph835d',
u'\ue2dc': 'UnicodeCharacter0xe2dc',
u'\u1961': 'TaiLeLetterTsha',
u'\u78e0': 'CjkUnifiedIdeograph78e0',
u'\udc63': 'UnicodeCharacter0xdc63',
u'\u5267': 'CjkUnifiedIdeograph5267',
u'\ub1e6': 'HangulSyllableNoep',
u'\u286b': 'BraillePatternDots12467',
u'\u37ea': 'CjkUnifiedIdeograph37ea',
u'\ueb6d': 'UnicodeCharacter0xeb6d',
u'\ucaec': 'HangulSyllableJjwass',
u'\u6171': 'CjkUnifiedIdeograph6171',
u'\u40f0': 'CjkUnifiedIdeograph40f0',
u'\uba77': 'HangulSyllableMyeod',
u'\u99f6': 'CjkUnifiedIdeograph99f6',
u'\u307b': 'HiraganaLetterHo',
u'\u1ffa': 'GreekCapitalLetterOmegaWithVaria',
u'\uf37d': 'UnicodeCharacter0xf37d',
u'\ud2fc': 'HangulSyllableTils',
u'\uc881': 'HangulSyllableJob',
u'\ua900': 'KayahLiDigitZero',
u'\u8d83': 'CjkUnifiedIdeograph8d83',
u'\u6a02': 'CjkUnifiedIdeograph6a02',
u'\u4e85': 'CjkUnifiedIdeograph4e85',
u'\u2f04': 'KangxiRadicalSecond',
u'\u0387': 'GreekAnoTeleia',
u'\ufc0e': 'ArabicLigatureTehWithMeemIsolatedForm',
u'\ud091': 'HangulSyllableKyilg',
u'\ub110': 'HangulSyllableNeol',
u'\u9593': 'CjkUnifiedIdeograph9593',
u'\u7212': 'CjkUnifiedIdeograph7212',
u'\u5695': 'CjkUnifiedIdeograph5695',
u'\u3714': 'CjkUnifiedIdeograph3714',
u'\u6b97': 'CjkUnifiedIdeograph6b97',
u'\u2c99': 'CopticSmallLetterMi',
u'\uc41e': 'HangulSyllableSswaelp',
u'\u9920': 'CjkUnifiedIdeograph9920',
u'\ufda3': 'ArabicLigatureTehWithMeemWithYehFinalForm',
u'\u5a22': 'CjkUnifiedIdeograph5a22',
u'\ubea5': 'HangulSyllableBbyalt',
u'\u1f24': 'GreekSmallLetterEtaWithPsiliAndOxia',
u'\u73a7': 'CjkUnifiedIdeograph73a7',
u'\u34a9': 'CjkUnifiedIdeograph34a9',
u'\ue130': 'UnicodeCharacter0xe130',
u'\uc5b3': 'HangulSyllableYaeh',
u'\ua232': 'YiSyllableMgat',
u'\u86b5': 'CjkUnifiedIdeograph86b5',
u'\u6734': 'CjkUnifiedIdeograph6734',
u'\u5bb7': 'CjkUnifiedIdeograph5bb7',
u'\u3836': 'CjkUnifiedIdeograph3836',
u'\u1cb9': 'UnicodeCharacter0x1cb9',
u'\uc940': 'HangulSyllableJwels',
u'\u8a42': 'CjkUnifiedIdeograph8a42',
u'\ueec5': 'UnicodeCharacter0xeec5',
u'\u4f44': 'CjkUnifiedIdeograph4f44',
u'\ua3c7': 'YiSyllableRax',
u'\u64c9': 'CjkUnifiedIdeograph64c9',
u'\u39cb': 'CjkUnifiedIdeograph39cb',
u'\ud150': 'HangulSyllableTen',
u'\u9252': 'CjkUnifiedIdeograph9252',
u'\uf6d5': 'UnicodeCharacter0xf6d5',
u'\u5754': 'CjkUnifiedIdeograph5754',
u'\u8bd7': 'CjkUnifiedIdeograph8bd7',
u'\u6856': 'CjkUnifiedIdeograph6856',
u'\u4cd9': 'CjkUnifiedIdeograph4cd9',
u'\u2d58': 'TifinaghLetterAyerYagh',
u'\u01db': 'LatinCapitalLetterUWithDiaeresisAndGrave',
u'\ufa62': 'CjkCompatibilityIdeographfa62',
u'\udee5': 'UnicodeCharacter0xdee5',
u'\ubf64': 'HangulSyllableBbwal',
u'\u93e7': 'CjkUnifiedIdeograph93e7',
u'\u7066': 'CjkUnifiedIdeograph7066',
u'\u54e9': 'CjkUnifiedIdeograph54e9',
u'\u3568': 'CjkUnifiedIdeograph3568',
u'\u69eb': 'CjkUnifiedIdeograph69eb',
u'\u2aed': 'ReversedDoubleStrokeNotSign',
u'\uc272': 'HangulSyllableSwinh',
u'\u8774': 'CjkUnifiedIdeograph8774',
u'\ufbf7': 'ArabicLigatureYehWithHamzaAboveWithEFinalForm',
u'\u5876': 'CjkUnifiedIdeograph5876',
u'\ubcf9': 'HangulSyllableBonj',
u'\u1d78': 'ModifierLetterCyrillicEn',
u'\u71fb': 'CjkUnifiedIdeograph71fb',
u'\u32fd': 'CircledKatakanaWe',
u'\u0801': 'UnicodeCharacter0x801',
u'\uef84': 'UnicodeCharacter0xef84',
u'\ua086': 'YiSyllableNbux',
u'\u6588': 'CjkUnifiedIdeograph6588',
u'\ud90b': 'UnicodeCharacter0xd90b',
u'\u268a': 'MonogramForYang',
u'\u9a0d': 'CjkUnifiedIdeograph9a0d',
u'\u5f0f': 'CjkUnifiedIdeograph5f0f',
u'\u1011': 'MyanmarLetterTha',
u'\uf794': 'UnicodeCharacter0xf794',
u'\u8896': 'CjkUnifiedIdeograph8896',
u'\u4d98': 'CjkUnifiedIdeograph4d98',
u'\u0e9a': 'LaoLetterBo',
u'\ue21d': 'UnicodeCharacter0xe21d',
u'\ua71f': 'ModifierLetterLowInvertedExclamationMark',
u'\u7821': 'CjkUnifiedIdeograph7821',
u'\u3d23': 'CjkUnifiedIdeograph3d23',
u'\udfa4': 'UnicodeCharacter0xdfa4',
u'\u90a6': 'CjkUnifiedIdeograph90a6',
u'\u55a8': 'CjkUnifiedIdeograph55a8',
u'\u16aa': 'RunicLetterAcA',
u'\uca2d': 'HangulSyllableJjyat',
u'\u2bac': 'UnicodeCharacter0x2bac',
u'\u8f2f': 'CjkUnifiedIdeograph8f2f',
u'\u4031': 'CjkUnifiedIdeograph4031',
u'\u0533': 'ArmenianCapitalLetterGim',
u'\uf8b6': 'UnicodeCharacter0xf8b6',
u'\ubdb8': 'HangulSyllableBwe',
u'\u7eba': 'CjkUnifiedIdeograph7eba',
u'\ud23d': 'HangulSyllableTub',
u'\u33bc': 'SquareMuW',
u'\u973f': 'CjkUnifiedIdeograph973f',
u'\ua841': 'PhagspaLetterKha',
u'\u6d43': 'CjkUnifiedIdeograph6d43',
u'\u2e45': 'UnicodeCharacter0x2e45',
u'\uc0c6': 'HangulSyllableSap',
u'\u85c8': 'CjkUnifiedIdeograph85c8',
u'\u46ca': 'CjkUnifiedIdeograph46ca',
u'\u1bcc': 'UnicodeCharacter0x1bcc',
u'\uff4f': 'FullwidthLatinSmallLetterO',
u'\ub051': 'HangulSyllableGgeult',
u'\u7553': 'CjkUnifiedIdeograph7553',
u'\u3655': 'CjkUnifiedIdeograph3655',
u'\uedd8': 'UnicodeCharacter0xedd8',
u'\uaeda': 'HangulSyllableGgegg',
u'\u63dc': 'CjkUnifiedIdeograph63dc',
u'\uc75f': 'HangulSyllableYid',
u'\u24de': 'CircledLatinSmallLetterO',
u'\u9861': 'CjkUnifiedIdeograph9861',
u'\u5d63': 'CjkUnifiedIdeograph5d63',
u'\u1e65': 'LatinSmallLetterSWithAcuteAndDotAbove',
u'\uf5e8': 'UnicodeCharacter0xf5e8',
u'\ub6ea': 'HangulSyllableDdwej',
u'\u4bec': 'CjkUnifiedIdeograph4bec',
u'\u0cee': 'KannadaDigitEight',
u'\ue071': 'UnicodeCharacter0xe071',
u'\ua573': 'VaiSyllableHoo',
u'\u6675': 'CjkUnifiedIdeograph6675',
u'\u3b77': 'CjkUnifiedIdeograph3b77',
u'\uddf8': 'UnicodeCharacter0xddf8',
u'\u9efa': 'CjkUnifiedIdeograph9efa',
u'\u53fc': 'CjkUnifiedIdeograph53fc',
u'\u14fe': 'CanadianSyllabicsSwoo',
u'\u2a00': 'NaryCircledDotOperator',
u'\u0e83': 'UnicodeCharacter0xe83',
u'\ue902': 'UnicodeCharacter0xe902',
u'\ucd85': 'HangulSyllableCyolt',
u'\u6306': 'CjkUnifiedIdeograph6306',
u'\u4789': 'CjkUnifiedIdeograph4789',
u'\ub80c': 'HangulSyllableRen',
u'\u9c8f': 'CjkUnifiedIdeograph9c8f',
u'\u3210': 'ParenthesizedHangulTikeutA',
u'\u1693': 'OghamLetterEadhadh',
u'\uf112': 'UnicodeCharacter0xf112',
u'\ud595': 'HangulSyllableHyanj',
u'\u4b16': 'CjkUnifiedIdeograph4b16',
u'\uaf99': 'HangulSyllableGgoet',
u'\u219d': 'RightwardsWaveArrow',
u'\u801c': 'CjkUnifiedIdeograph801c',
u'\ue49f': 'UnicodeCharacter0xe49f',
u'\u1a20': 'UnicodeCharacter0x1a20',
u'\u7ea3': 'CjkUnifiedIdeograph7ea3',
u'\ud922': 'UnicodeCharacter0xd922',
u'\u5326': 'CjkUnifiedIdeograph5326',
u'\ub7a9': 'HangulSyllableRaeb',
u'\u09ad': 'BengaliLetterBha',
u'\ue82c': 'UnicodeCharacter0xe82c',
u'\uccaf': 'HangulSyllableCeoc',
u'\u6230': 'CjkUnifiedIdeograph6230',
u'\u46b3': 'CjkUnifiedIdeograph46b3',
u'\ubb36': 'HangulSyllableMugg',
u'\u9fb9': 'CjkUnifiedIdeograph9fb9',
u'\u3d3a': 'CjkUnifiedIdeograph3d3a',
u'\u11bd': 'HangulJongseongCieuc',
u'\uf03c': 'UnicodeCharacter0xf03c',
u'\ud4bf': 'HangulSyllablePwelh',
u'\u4a40': 'CjkUnifiedIdeograph4a40',
u'\uaec3': 'HangulSyllableGgeod',
u'\u20c7': 'UnicodeCharacter0x20c7',
u'\u8346': 'CjkUnifiedIdeograph8346',
u'\ue7c9': 'UnicodeCharacter0xe7c9',
u'\u054a': 'ArmenianCapitalLetterPeh',
u'\u79cd': 'CjkUnifiedIdeograph79cd',
u'\ud84c': 'UnicodeCharacter0xd84c',
u'\u5250': 'CjkUnifiedIdeograph5250',
u'\ub6d3': 'HangulSyllableDdweoh',
u'\u08d7': 'UnicodeCharacter0x8d7',
u'\ueb56': 'UnicodeCharacter0xeb56',
u'\ucfd9': 'HangulSyllableKyong',
u'\u6d5a': 'CjkUnifiedIdeograph6d5a',
u'\u41dd': 'CjkUnifiedIdeograph41dd',
u'\uba60': 'HangulSyllableMels',
u'\u9ee3': 'CjkUnifiedIdeograph9ee3',
u'\u3c64': 'CjkUnifiedIdeograph3c64',
u'\u10e7': 'GeorgianLetterQar',
u'\uf366': 'UnicodeCharacter0xf366',
u'\ud7e9': 'UnicodeCharacter0xd7e9',
u'\u756a': 'CjkUnifiedIdeograph756a',
u'\ua9ed': 'UnicodeCharacter0xa9ed',
u'\u23f1': 'UnicodeCharacter0x23f1',
u'\u8270': 'CjkUnifiedIdeograph8270',
u'\ue6f3': 'UnicodeCharacter0xe6f3',
u'\u0474': 'CyrillicCapitalLetterIzhitsa',
u'\u78f7': 'CjkUnifiedIdeograph78f7',
u'\udb76': 'UnicodeCharacter0xdb76',
u'\u5d7a': 'CjkUnifiedIdeograph5d7a',
u'\ub1fd': 'HangulSyllableNyong',
u'\u8b01': 'CjkUnifiedIdeograph8b01',
u'\uea80': 'UnicodeCharacter0xea80',
u'\u0d05': 'MalayalamLetterA',
u'\u6c84': 'CjkUnifiedIdeograph6c84',
u'\uc007': 'HangulSyllableBbwegs',
u'\u5a0b': 'CjkUnifiedIdeograph5a0b',
u'\ua58a': 'VaiSyllableZhoo',
u'\u3f8e': 'CjkUnifiedIdeograph3f8e',
u'\u9311': 'CjkUnifiedIdeograph9311',
u'\uf290': 'UnicodeCharacter0xf290',
u'\u1515': 'CanadianSyllabicsSha',
u'\u7494': 'CjkUnifiedIdeograph7494',
u'\ua21b': 'YiSyllableGgax',
u'\u8d9a': 'CjkUnifiedIdeograph8d9a',
u'\u241f': 'SymbolForUnitSeparator',
u'\u079e': 'ThaanaLetterSaadhu',
u'\ufb21': 'HebrewLetterWideAlef',
u'\udaa0': 'UnicodeCharacter0xdaa0',
u'\u7d25': 'CjkUnifiedIdeograph7d25',
u'\u5ca4': 'CjkUnifiedIdeograph5ca4',
u'\u8a2b': 'CjkUnifiedIdeograph8a2b',
u'\u95aa': 'CjkUnifiedIdeograph95aa',
u'\u0c2f': 'TeluguLetterYa',
u'\u6fae': 'CjkUnifiedIdeograph6fae',
u'\uc331': 'HangulSyllableSsyag',
u'\u4535': 'CjkUnifiedIdeograph4535',
u'\ua4b4': 'YiRadicalNzup',
u'\u3eb8': 'CjkUnifiedIdeograph3eb8',
u'\u923b': 'CjkUnifiedIdeograph923b',
u'\ufdba': 'ArabicLigatureLamWithJeemWithMeemInitialForm',
u'\u143f': 'CanadianSyllabicsWestcreePwii',
u'\u77be': 'CjkUnifiedIdeograph77be',
u'\uad45': 'HangulSyllableGoeb',
u'\u8cc4': 'CjkUnifiedIdeograph8cc4',
u'\u2749': 'BalloonspokedAsterisk',
u'\u06c8': 'ArabicLetterYu',
u'\ufa4b': 'CjkCompatibilityIdeographfa4b',
u'\uc5ca': 'HangulSyllableEoj',
u'\u7c4f': 'CjkUnifiedIdeograph7c4f',
u'\u5fce': 'CjkUnifiedIdeograph5fce',
u'\ub555': 'HangulSyllableDdaelg',
u'\u94d4': 'CjkUnifiedIdeograph94d4',
u'\u0f59': 'TibetanLetterTsa',
u'\u6ed8': 'CjkUnifiedIdeograph6ed8',
u'\uc25b': 'HangulSyllableSwelb',
u'\u445f': 'CjkUnifiedIdeograph445f',
u'\ua7de': 'UnicodeCharacter0xa7de',
u'\u39e2': 'CjkUnifiedIdeograph39e2',
u'\u9d65': 'CjkUnifiedIdeograph9d65',
u'\ufce4': 'ArabicLigatureTehWithHehMedialForm',
u'\u1769': 'TagbanwaLetterPa',
u'\u76e8': 'CjkUnifiedIdeograph76e8',
u'\uac6f': 'HangulSyllableGyaeh',
u'\u8fee': 'CjkUnifiedIdeograph8fee',
u'\u2673': 'RecyclingSymbolForType1Plastics',
u'\u01f2': 'LatinCapitalLetterDWithSmallLetterZ',
u'\ue575': 'UnicodeCharacter0xe575',
u'\uc4f4': 'HangulSyllableSseun',
u'\u7f79': 'CjkUnifiedIdeograph7f79',
u'\u5ef8': 'CjkUnifiedIdeograph5ef8',
u'\ub47f': 'HangulSyllableDweos',
u'\u97fe': 'CjkUnifiedIdeograph97fe',
u'\ue206': 'UnicodeCharacter0xe206',
u'\uc689': 'HangulSyllableOeb',
u'\ua708': 'ModifierLetterExtrahighDottedToneBar',
u'\u9b8b': 'CjkUnifiedIdeograph9b8b',
u'\u640a': 'CjkUnifiedIdeograph640a',
u'\u588d': 'CjkUnifiedIdeograph588d',
u'\u390c': 'CjkUnifiedIdeograph390c',
u'\u1d8f': 'LatinSmallLetterAWithRetroflexHook',
u'\uca16': 'HangulSyllableJjyagg',
u'\u8f18': 'CjkUnifiedIdeograph8f18',
u'\ue39b': 'UnicodeCharacter0xe39b',
u'\u4c1a': 'CjkUnifiedIdeograph4c1a',
u'\ua09d': 'YiSyllableHmuo',
u'\u011c': 'LatinCapitalLetterGWithCircumflex',
u'\u659f': 'CjkUnifiedIdeograph659f',
u'\u3aa1': 'CjkUnifiedIdeograph3aa1',
u'\ud226': 'HangulSyllableTyoj',
u'\u9728': 'CjkUnifiedIdeograph9728',
u'\ucbab': 'HangulSyllableJjwilh',
u'\u542a': 'CjkUnifiedIdeograph542a',
u'\u88ad': 'CjkUnifiedIdeograph88ad',
u'\u692c': 'CjkUnifiedIdeograph692c',
u'\u4daf': 'CjkUnifiedIdeograph4daf',
u'\u2e2e': 'ReversedQuestionMark',
u'\u02b1': 'ModifierLetterSmallHWithHook',
u'\uff38': 'FullwidthLatinCapitalLetterX',
u'\ud3bb': 'HangulSyllablePyeod',
u'\ubc3a': 'HangulSyllableBaelm',
u'\u90bd': 'CjkUnifiedIdeograph90bd',
u'\u713c': 'CjkUnifiedIdeograph713c',
u'\u55bf': 'CjkUnifiedIdeograph55bf',
u'\u363e': 'CjkUnifiedIdeograph363e',
u'\u6ac1': 'CjkUnifiedIdeograph6ac1',
u'\u2fc3': 'KangxiRadicalBird',
u'\uc748': 'HangulSyllableEuls',
u'\u844a': 'CjkUnifiedIdeograph844a',
u'\uf8cd': 'UnicodeCharacter0xf8cd',
u'\u594c': 'CjkUnifiedIdeograph594c',
u'\ubdcf': 'HangulSyllableBwec',
u'\u1e4e': 'LatinCapitalLetterOWithTildeAndDiaeresis',
u'\u72d1': 'CjkUnifiedIdeograph72d1',
u'\u37d3': 'CjkUnifiedIdeograph37d3',
u'\uec5a': 'UnicodeCharacter0xec5a',
u'\uc0dd': 'HangulSyllableSaeng',
u'\ua15c': 'YiSyllableNdox',
u'\u85df': 'CjkUnifiedIdeograph85df',
u'\u665e': 'CjkUnifiedIdeograph665e',
u'\u5ae1': 'CjkUnifiedIdeograph5ae1',
u'\u3b60': 'CjkUnifiedIdeograph3b60',
u'\u1fe3': 'GreekSmallLetterUpsilonWithDialytikaAndOxia',
u'\uf46a': 'UnicodeCharacter0xf46a',
u'\u896c': 'CjkUnifiedIdeograph896c',
u'\uedef': 'UnicodeCharacter0xedef',
u'\u4e6e': 'CjkUnifiedIdeograph4e6e',
u'\ua2f1': 'YiSyllableNziex',
u'\u0370': 'GreekCapitalLetterHeta',
u'\u67f3': 'CjkUnifiedIdeograph67f3',
u'\u24f5': 'DoubleCircledDigitOne',
u'\udc7a': 'UnicodeCharacter0xdc7a',
u'\u917c': 'CjkUnifiedIdeograph917c',
u'\uf5ff': 'UnicodeCharacter0xf5ff',
u'\u567e': 'CjkUnifiedIdeograph567e',
u'\u6b80': 'CjkUnifiedIdeograph6b80',
u'\ucf03': 'HangulSyllableKegs',
u'\u2882': 'BraillePatternDots28',
u'\u8c05': 'CjkUnifiedIdeograph8c05',
u'\u4107': 'CjkUnifiedIdeograph4107',
u'\u0609': 'ArabicindicPerMilleSign',
u'\uf98c': 'CjkCompatibilityIdeographf98c',
u'\ube8e': 'HangulSyllableBbaebs',
u'\u7390': 'CjkUnifiedIdeograph7390',
u'\ud713': 'HangulSyllableHwec',
u'\u3092': 'HiraganaLetterWo',
u'\u9415': 'CjkUnifiedIdeograph9415',
u'\ua917': 'KayahLiLetterMa',
u'\u6e19': 'CjkUnifiedIdeograph6e19',
u'\u231b': 'Hourglass',
u'\uc19c': 'HangulSyllableSom',
u'\u869e': 'CjkUnifiedIdeograph869e',
u'\u5ba0': 'CjkUnifiedIdeograph5ba0',
u'\u18a2': 'MongolianLetterManchuAliGaliSsa',
u'\ufc25': 'ArabicLigatureDadWithMeemIsolatedForm',
u'\ub127': 'HangulSyllableNegs',
u'\u7629': 'CjkUnifiedIdeograph7629',
u'\u0b2b': 'OriyaLetterPha',
u'\ueeae': 'UnicodeCharacter0xeeae',
u'\ua3b0': 'YiSyllableShuo',
u'\u60b2': 'CjkUnifiedIdeograph60b2',
u'\uc435': 'HangulSyllableSsoelg',
u'\u25b4': 'BlackUppointingSmallTriangle',
u'\u9937': 'CjkUnifiedIdeograph9937',
u'\u5e39': 'CjkUnifiedIdeograph5e39',
u'\u133b': 'EthiopicSyllableTsaa',
u'\uf6be': 'UnicodeCharacter0xf6be',
u'\u8bc0': 'CjkUnifiedIdeograph8bc0',
u'\u48c2': 'CjkUnifiedIdeograph48c2',
u'\u0dc4': 'SinhalaLetterHayanna',
u'\ue147': 'UnicodeCharacter0xe147',
u'\ua649': 'CyrillicSmallLetterDjerv',
u'\u7b4b': 'CjkUnifiedIdeograph7b4b',
u'\u384d': 'CjkUnifiedIdeograph384d',
u'\udece': 'UnicodeCharacter0xdece',
u'\u93d0': 'CjkUnifiedIdeograph93d0',
u'\u50d2': 'CjkUnifiedIdeograph50d2',
u'\u15d4': 'CanadianSyllabicsCarrierWi',
u'\uc957': 'HangulSyllableJwid',
u'\u2ad6': 'SupersetAboveSuperset',
u'\u8e59': 'CjkUnifiedIdeograph8e59',
u'\u435b': 'CjkUnifiedIdeograph435b',
u'\ufbe0': 'ArabicLetterKirghizOeIsolatedForm',
u'\ub8e2': 'HangulSyllableRyoj',
u'\u7de4': 'CjkUnifiedIdeograph7de4',
u'\ud167': 'HangulSyllableTeh',
u'\u32e6': 'CircledKatakanaNu',
u'\u9669': 'CjkUnifiedIdeograph9669',
u'\uab6b': 'UnicodeCharacter0xab6b',
u'\u686d': 'CjkUnifiedIdeograph686d',
u'\u2d6f': 'TifinaghModifierLetterLabializationMark',
u'\uc3f0': 'HangulSyllableSsok',
u'\u80f2': 'CjkUnifiedIdeograph80f2',
u'\u45f4': 'CjkUnifiedIdeograph45f4',
u'\u1af6': 'UnicodeCharacter0x1af6',
u'\ufe79': 'ArabicDammaMedialForm',
u'\ub37b': 'HangulSyllableDelb',
u'\u707d': 'CjkUnifiedIdeograph707d',
u'\u357f': 'CjkUnifiedIdeograph357f',
u'\u4d81': 'CjkUnifiedIdeograph4d81',
u'\uaa04': 'ChamLetterAi',
u'\u8687': 'CjkUnifiedIdeograph8687',
u'\u2008': 'PunctuationSpace',
u'\u1c8b': 'UnicodeCharacter0x1c8b',
u'\ue30a': 'UnicodeCharacter0xe30a',
u'\udf8d': 'UnicodeCharacter0xdf8d',
u'\u790e': 'CjkUnifiedIdeograph790e',
u'\u5591': 'CjkUnifiedIdeograph5591',
u'\ub214': 'HangulSyllableNum',
u'\uee97': 'UnicodeCharacter0xee97',
u'\u0818': 'UnicodeCharacter0x818',
u'\u649b': 'CjkUnifiedIdeograph649b',
u'\ucb1a': 'HangulSyllableJjoelm',
u'\u411e': 'CjkUnifiedIdeograph411e',
u'\ubda1': 'HangulSyllableBweonj',
u'\u3ba5': 'CjkUnifiedIdeograph3ba5',
u'\u9a24': 'CjkUnifiedIdeograph9a24',
u'\uf6a7': 'UnicodeCharacter0xf6a7',
u'\u1028': 'MyanmarLetterMonE',
u'\u4cab': 'CjkUnifiedIdeograph4cab',
u'\ud32a': 'HangulSyllablePaegg',
u'\ua92e': 'KayahLiSignCwi',
u'\u85b1': 'CjkUnifiedIdeograph85b1',
u'\u2732': 'OpenCentreAsterisk',
u'\u03b5': 'GreekSmallLetterEpsilon',
u'\ue234': 'UnicodeCharacter0xe234',
u'\udeb7': 'UnicodeCharacter0xdeb7',
u'\u7838': 'CjkUnifiedIdeograph7838',
u'\u54bb': 'CjkUnifiedIdeograph54bb',
u'\ub13e': 'HangulSyllableNep',
u'\uedc1': 'UnicodeCharacter0xedc1',
u'\u0f42': 'TibetanLetterGa',
u'\u6bc5': 'CjkUnifiedIdeograph6bc5',
u'\uca44': 'HangulSyllableJjyaess',
u'\u4048': 'CjkUnifiedIdeograph4048',
u'\ubccb': 'HangulSyllableByeolh',
u'\u3acf': 'CjkUnifiedIdeograph3acf',
u'\u994e': 'CjkUnifiedIdeograph994e',
u'\uf5d1': 'UnicodeCharacter0xf5d1',
u'\u1752': 'BuhidVowelSignI',
u'\u73d5': 'CjkUnifiedIdeograph73d5',
u'\ud254': 'HangulSyllableTweols',
u'\ua858': 'PhagspaLetterRa',
u'\u84db': 'CjkUnifiedIdeograph84db',
u'\u265c': 'BlackChessRook',
u'\u02df': 'ModifierLetterCrossAccent',
u'\ue15e': 'UnicodeCharacter0xe15e',
u'\udde1': 'UnicodeCharacter0xdde1',
u'\u7f62': 'CjkUnifiedIdeograph7f62',
u'\u5be5': 'CjkUnifiedIdeograph5be5',
u'\ub068': 'HangulSyllableGgyil',
u'\ueceb': 'UnicodeCharacter0xeceb',
u'\u0e6c': 'UnicodeCharacter0xe6c',
u'\u6aef': 'CjkUnifiedIdeograph6aef',
u'\uc96e': 'HangulSyllableJyugg',
u'\u4772': 'CjkUnifiedIdeograph4772',
u'\ua3f5': 'YiSyllableJur',
u'\u39f9': 'CjkUnifiedIdeograph39f9',
u'\u9878': 'CjkUnifiedIdeograph9878',
u'\uf4fb': 'UnicodeCharacter0xf4fb',
u'\u167c': 'UnicodeCharacter0x167c',
u'\u72ff': 'CjkUnifiedIdeograph72ff',
u'\ud17e': 'HangulSyllableTyeoj',
u'\u4803': 'CjkUnifiedIdeograph4803',
u'\uaf82': 'HangulSyllableGgoegg',
u'\u2586': 'LowerThreeQuartersBlock',
u'\u8109': 'CjkUnifiedIdeograph8109',
u'\ue088': 'UnicodeCharacter0xe088',
u'\u1f0d': 'GreekCapitalLetterAlphaWithDasiaAndOxia',
u'\u7e8c': 'CjkUnifiedIdeograph7e8c',
u'\uda0f': 'UnicodeCharacter0xda0f',
u'\u5013': 'CjkUnifiedIdeograph5013',
u'\ub792': 'HangulSyllableRaj',
u'\u2e17': 'DoubleObliqueHyphen',
u'\u0d96': 'SinhalaLetterAuyanna',
u'\ue919': 'UnicodeCharacter0xe919',
u'\uc898': 'HangulSyllableJwals',
u'\u671d': 'CjkUnifiedIdeograph671d',
u'\u469c': 'CjkUnifiedIdeograph469c',
u'\ub823': 'HangulSyllableReh',
u'\u9fa2': 'CjkUnifiedIdeograph9fa2',
u'\u3627': 'CjkUnifiedIdeograph3627',
u'\u15a6': 'CanadianSyllabicsLh',
u'\uf129': 'UnicodeCharacter0xf129',
u'\ud0a8': 'HangulSyllableKin',
u'\u4f2d': 'CjkUnifiedIdeograph4f2d',
u'\uaeac': 'HangulSyllableGgyaels',
u'\u24b0': 'ParenthesizedLatinSmallLetterU',
u'\u8033': 'CjkUnifiedIdeograph8033',
u'\ue7b2': 'UnicodeCharacter0xe7b2',
u'\u1e37': 'LatinSmallLetterLWithDotBelow',
u'\u7db6': 'CjkUnifiedIdeograph7db6',
u'\ud939': 'UnicodeCharacter0xd939',
u'\u573d': 'CjkUnifiedIdeograph573d',
u'\ub6bc': 'HangulSyllableDdweon',
u'\u2d41': 'TifinaghLetterBerberAcademyYah',
u'\u0cc0': 'KannadaVowelSignIi',
u'\ue843': 'UnicodeCharacter0xe843',
u'\ucfc2': 'HangulSyllableKoep',
u'\u6647': 'CjkUnifiedIdeograph6647',
u'\u45c6': 'CjkUnifiedIdeograph45c6',
u'\ubf4d': 'HangulSyllableBbolt',
u'\u9ecc': 'CjkUnifiedIdeograph9ecc',
u'\u3551': 'CjkUnifiedIdeograph3551',
u'\u14d0': 'CanadianSyllabicsN',
u'\uf053': 'UnicodeCharacter0xf053',
u'\ud7d2': 'UnicodeCharacter0xd7d2',
u'\u4e57': 'CjkUnifiedIdeograph4e57',
u'\uadd6': 'HangulSyllableGwij',
u'\u2bda': 'UnicodeCharacter0x2bda',
u'\u875d': 'CjkUnifiedIdeograph875d',
u'\ue6dc': 'UnicodeCharacter0xe6dc',
u'\u1d61': 'ModifierLetterSmallChi',
u'\u7ce0': 'CjkUnifiedIdeograph7ce0',
u'\ud863': 'UnicodeCharacter0xd863',
u'\u5667': 'CjkUnifiedIdeograph5667',
u'\ub5e6': 'HangulSyllableDdyeolp',
u'\u2c6b': 'LatinCapitalLetterZWithDescender',
u'\u33ea': 'IdeographicTelegraphSymbolForDayEleven',
u'\uef6d': 'UnicodeCharacter0xef6d',
u'\uceec': 'HangulSyllableKeol',
u'\u6571': 'CjkUnifiedIdeograph6571',
u'\u44f0': 'CjkUnifiedIdeograph44f0',
u'\ube77': 'HangulSyllableBbac',
u'\u9df6': 'CjkUnifiedIdeograph9df6',
u'\u347b': 'CjkUnifiedIdeograph347b',
u'\u1bfa': 'UnicodeCharacter0x1bfa',
u'\uf77d': 'UnicodeCharacter0xf77d',
u'\ud6fc': 'HangulSyllableHwe',
u'\ucc81': 'HangulSyllableCyaenj',
u'\uad00': 'HangulSyllableGwan',
u'\u8983': 'CjkUnifiedIdeograph8983',
u'\u6e02': 'CjkUnifiedIdeograph6e02',
u'\u4a85': 'CjkUnifiedIdeograph4a85',
u'\u2b04': 'LeftRightWhiteArrow',
u'\u0787': 'ThaanaLetterAlifu',
u'\uf80e': 'UnicodeCharacter0xf80e',
u'\ud491': 'HangulSyllablePut',
u'\ub510': 'HangulSyllableDyik',
u'\u9193': 'CjkUnifiedIdeograph9193',
u'\u7612': 'CjkUnifiedIdeograph7612',
u'\u5295': 'CjkUnifiedIdeograph5295',
u'\u3314': 'SquareKiro',
u'\u6f97': 'CjkUnifiedIdeograph6f97',
u'\u2899': 'BraillePatternDots1458',
u'\uc01e': 'HangulSyllableBbwep',
u'\u9d20': 'CjkUnifiedIdeograph9d20',
u'\uf9a3': 'CjkCompatibilityIdeographf9a3',
u'\u5e22': 'CjkUnifiedIdeograph5e22',
u'\ubaa5': 'HangulSyllableMyet',
u'\u1b24': 'BalineseLetterDa',
u'\u77a7': 'CjkUnifiedIdeograph77a7',
u'\u30a9': 'KatakanaLetterSmallO',
u'\ue530': 'UnicodeCharacter0xe530',
u'\uc1b3': 'HangulSyllableSwalb',
u'\ua632': 'UnicodeCharacter0xa632',
u'\u82b5': 'CjkUnifiedIdeograph82b5',
u'\u6334': 'CjkUnifiedIdeograph6334',
u'\u5fb7': 'CjkUnifiedIdeograph5fb7',
u'\u3c36': 'CjkUnifiedIdeograph3c36',
u'\u18b9': 'UnicodeCharacter0x18b9',
u'\ucd40': 'HangulSyllableCwae',
u'\u8e42': 'CjkUnifiedIdeograph8e42',
u'\ueac5': 'UnicodeCharacter0xeac5',
u'\u4b44': 'CjkUnifiedIdeograph4b44',
u'\ua7c7': 'UnicodeCharacter0xa7c7',
u'\u0446': 'CyrillicSmallLetterTse',
u'\u60c9': 'CjkUnifiedIdeograph60c9',
u'\u3dcb': 'CjkUnifiedIdeograph3dcb',
u'\ud550': 'HangulSyllablePiss',
u'\u9652': 'CjkUnifiedIdeograph9652',
u'\uf2d5': 'UnicodeCharacter0xf2d5',
u'\u5354': 'CjkUnifiedIdeograph5354',
u'\u8fd7': 'CjkUnifiedIdeograph8fd7',
u'\u6c56': 'CjkUnifiedIdeograph6c56',
u'\u48d9': 'CjkUnifiedIdeograph48d9',
u'\u2958': 'UpwardsHarpoonWithBarbLeftToBar',
u'\u05db': 'HebrewLetterKaf',
u'\ufe62': 'SmallPlusSign',
u'\udae5': 'UnicodeCharacter0xdae5',
u'\ubb64': 'HangulSyllableMweoss',
u'\u97e7': 'CjkUnifiedIdeograph97e7',
u'\u7466': 'CjkUnifiedIdeograph7466',
u'\u50e9': 'CjkUnifiedIdeograph50e9',
u'\u3168': 'HangulLetterNieunpansios',
u'\u6deb': 'CjkUnifiedIdeograph6deb',
u'\u2eed': 'CjkRadicalJsimplifiedTooth',
u'\uc672': 'HangulSyllableWaej',
u'\u8374': 'CjkUnifiedIdeograph8374',
u'\ufff7': 'UnicodeCharacter0xfff7',
u'\u5c76': 'CjkUnifiedIdeograph5c76',
u'\ub8f9': 'HangulSyllableRub',
u'\u1978': 'UnicodeCharacter0x1978',
u'\u75fb': 'CjkUnifiedIdeograph75fb',
u'\u36fd': 'CjkUnifiedIdeograph36fd',
u'\u0c01': 'TeluguSignCandrabindu',
u'\ueb84': 'UnicodeCharacter0xeb84',
u'\ua486': 'YiSyllableYur',
u'\u6188': 'CjkUnifiedIdeograph6188',
u'\udd0b': 'UnicodeCharacter0xdd0b',
u'\u228a': 'SubsetOfWithNotEqualTo',
u'\u9e0d': 'CjkUnifiedIdeograph9e0d',
u'\u5b0f': 'CjkUnifiedIdeograph5b0f',
u'\u1411': 'CanadianSyllabicsWestcreeWii',
u'\uf394': 'UnicodeCharacter0xf394',
u'\u8c96': 'CjkUnifiedIdeograph8c96',
u'\u4998': 'CjkUnifiedIdeograph4998',
u'\u0a9a': 'GujaratiLetterCa',
u'\ue61d': 'UnicodeCharacter0xe61d',
u'\ua31f': 'YiSyllableSux',
u'\u7c21': 'CjkUnifiedIdeograph7c21',
u'\u3923': 'CjkUnifiedIdeograph3923',
u'\udba4': 'UnicodeCharacter0xdba4',
u'\u94a6': 'CjkUnifiedIdeograph94a6',
u'\u51a8': 'CjkUnifiedIdeograph51a8',
u'\u12aa': 'EthiopicSyllableKi',
u'\uce2d': 'HangulSyllableCeult',
u'\u2fac': 'KangxiRadicalRain',
u'\u8b2f': 'CjkUnifiedIdeograph8b2f',
u'\u4431': 'CjkUnifiedIdeograph4431',
u'\u0133': 'LatinSmallLigatureIj',
u'\ufcb6': 'ArabicLigatureDadWithKhahInitialForm',
u'\ub9b8': 'HangulSyllableRils',
u'\u7aba': 'CjkUnifiedIdeograph7aba',
u'\ud63d': 'HangulSyllableHonj',
u'\u37bc': 'CjkUnifiedIdeograph37bc',
u'\u933f': 'CjkUnifiedIdeograph933f',
u'\uac41': 'HangulSyllableGyalg',
u'\u6943': 'CjkUnifiedIdeograph6943',
u'\u2a45': 'UnionWithLogicalOr',
u'\uc4c6': 'HangulSyllableSswilp',
u'\u81c8': 'CjkUnifiedIdeograph81c8',
u'\u42ca': 'CjkUnifiedIdeograph42ca',
u'\u1fcc': 'GreekCapitalLetterEtaWithProsgegrammeni',
u'\ufb4f': 'HebrewLigatureAlefLamed',
u'\ub451': 'HangulSyllableDug',
u'\u7153': 'CjkUnifiedIdeograph7153',
u'\u3255': 'CircledNumberTwentyFive',
u'\ue9d8': 'UnicodeCharacter0xe9d8',
u'\uaada': 'UnicodeCharacter0xaada',
u'\u67dc': 'CjkUnifiedIdeograph67dc',
u'\uc35f': 'HangulSyllableSsyaes',
u'\u20de': 'CombiningEnclosingSquare',
u'\u9c61': 'CjkUnifiedIdeograph9c61',
u'\u5963': 'CjkUnifiedIdeograph5963',
u'\u1a65': 'UnicodeCharacter0x1a65',
u'\uf1e8': 'UnicodeCharacter0xf1e8',
u'\ub2ea': 'HangulSyllableDanh',
u'\u4fec': 'CjkUnifiedIdeograph4fec',
u'\u08ee': 'UnicodeCharacter0x8ee',
u'\ue471': 'UnicodeCharacter0xe471',
u'\ua173': 'YiSyllableHnap',
u'\u6275': 'CjkUnifiedIdeograph6275',
u'\u3f77': 'CjkUnifiedIdeograph3f77',
u'\ud9f8': 'UnicodeCharacter0xd9f8',
u'\u9afa': 'CjkUnifiedIdeograph9afa',
u'\u57fc': 'CjkUnifiedIdeograph57fc',
u'\u10fe': 'UnicodeCharacter0x10fe',
u'\u2e00': 'RightAngleSubstitutionMarker',
u'\u0a83': 'GujaratiSignVisarga',
u'\ued02': 'UnicodeCharacter0xed02',
u'\uc985': 'HangulSyllableJyut',
u'\u6706': 'CjkUnifiedIdeograph6706',
u'\u4389': 'CjkUnifiedIdeograph4389',
u'\ubc0c': 'HangulSyllableMiss',
u'\u988f': 'CjkUnifiedIdeograph988f',
u'\u3610': 'CjkUnifiedIdeograph3610',
u'\u1293': 'EthiopicSyllableNaa',
u'\uf512': 'UnicodeCharacter0xf512',
u'\ud195': 'HangulSyllableTyeb',
u'\u4f16': 'CjkUnifiedIdeograph4f16',
u'\uab99': 'UnicodeCharacter0xab99',
u'\u259d': 'QuadrantUpperRight',
u'\u841c': 'CjkUnifiedIdeograph841c',
u'\ue09f': 'UnicodeCharacter0xe09f',
u'\u1e20': 'LatinCapitalLetterGWithMacron',
u'\u7aa3': 'CjkUnifiedIdeograph7aa3',
u'\udd22': 'UnicodeCharacter0xdd22',
u'\u5726': 'CjkUnifiedIdeograph5726',
u'\ub3a9': 'HangulSyllableDyeg',
u'\u0dad': 'SinhalaLetterAlpapraanaTayanna',
u'\uec2c': 'UnicodeCharacter0xec2c',
u'\uc8af': 'HangulSyllableJwaed',
u'\u6630': 'CjkUnifiedIdeograph6630',
u'\u42b3': 'CjkUnifiedIdeograph42b3',
u'\ubf36': 'HangulSyllableBbyebs',
u'\u9bb9': 'CjkUnifiedIdeograph9bb9',
u'\u393a': 'CjkUnifiedIdeograph393a',
u'\u15bd': 'CanadianSyllabicsBlackfootKi',
u'\uf43c': 'UnicodeCharacter0xf43c',
u'\ud0bf': 'HangulSyllableKih',
u'\u4e40': 'CjkUnifiedIdeograph4e40',
u'\uaac3': 'UnicodeCharacter0xaac3',
u'\u24c7': 'CircledLatinCapitalLetterR',
u'\u8746': 'CjkUnifiedIdeograph8746',
u'\ue3c9': 'UnicodeCharacter0xe3c9',
u'\u014a': 'LatinCapitalLetterEng',
u'\u7dcd': 'CjkUnifiedIdeograph7dcd',
u'\udc4c': 'UnicodeCharacter0xdc4c',
u'\u5650': 'CjkUnifiedIdeograph5650',
u'\ub2d3': 'HangulSyllableNilb',
u'\u0cd7': 'UnicodeCharacter0xcd7',
u'\uef56': 'UnicodeCharacter0xef56',
u'\ucbd9': 'HangulSyllableJjeunj',
u'\u695a': 'CjkUnifiedIdeograph695a',
u'\u45dd': 'CjkUnifiedIdeograph45dd',
u'\ube60': 'HangulSyllableBba',
u'\u9ae3': 'CjkUnifiedIdeograph9ae3',
u'\u3864': 'CjkUnifiedIdeograph3864',
u'\u14e7': 'CanadianSyllabicsWestcreeLwa',
u'\uf766': 'UnicodeCharacter0xf766',
u'\ud3e9': 'HangulSyllablePyet',
u'\u716a': 'CjkUnifiedIdeograph716a',
u'\uaded': 'HangulSyllableGyub',
u'\u27f1': 'DownwardsQuadrupleArrow',
u'\u8670': 'CjkUnifiedIdeograph8670',
u'\ue2f3': 'UnicodeCharacter0xe2f3',
u'\u7cf7': 'CjkUnifiedIdeograph7cf7',
u'\udf76': 'UnicodeCharacter0xdf76',
u'\u597a': 'CjkUnifiedIdeograph597a',
u'\ub5fd': 'HangulSyllableDdyelg',
u'\u8f01': 'CjkUnifiedIdeograph8f01',
u'\uee80': 'UnicodeCharacter0xee80',
u'\u0905': 'DevanagariLetterA',
u'\u6884': 'CjkUnifiedIdeograph6884',
u'\uc407': 'HangulSyllableSswas',
u'\u5e0b': 'CjkUnifiedIdeograph5e0b',
u'\ua18a': 'YiSyllableNot',
u'\u3b8e': 'CjkUnifiedIdeograph3b8e',
u'\u9711': 'CjkUnifiedIdeograph9711',
u'\uf690': 'UnicodeCharacter0xf690',
u'\u1115': 'HangulChoseongNieuntikeut',
u'\u7094': 'CjkUnifiedIdeograph7094',
u'\ua61b': 'VaiSymbolDoong',
u'\u899a': 'CjkUnifiedIdeograph899a',
u'\u201f': 'DoubleHighreversed9QuotationMark',
u'\u039e': 'GreekCapitalLetterXi',
u'\uff21': 'FullwidthLatinCapitalLetterA',
u'\udea0': 'UnicodeCharacter0xdea0',
u'\u7925': 'CjkUnifiedIdeograph7925',
u'\u58a4': 'CjkUnifiedIdeograph58a4',
u'\u8e2b': 'CjkUnifiedIdeograph8e2b',
u'\u91aa': 'CjkUnifiedIdeograph91aa',
u'\u082f': 'UnicodeCharacter0x82f',
u'\u6bae': 'CjkUnifiedIdeograph6bae',
u'\uc731': 'HangulSyllableYub',
u'\u4135': 'CjkUnifiedIdeograph4135',
u'\ua0b4': 'YiSyllableMiep',
u'\u3ab8': 'CjkUnifiedIdeograph3ab8',
u'\u963b': 'CjkUnifiedIdeograph963b',
u'\uf9ba': 'CjkCompatibilityIdeographf9ba',
u'\u103f': 'MyanmarLetterGreatSa',
u'\u73be': 'CjkUnifiedIdeograph73be',
u'\ua945': 'RejangLetterNyja',
u'\u88c4': 'CjkUnifiedIdeograph88c4',
u'\u2349': 'AplFunctionalSymbolCircleBackslash',
u'\u02c8': 'ModifierLetterVerticalLine',
u'\ufe4b': 'WavyOverline',
u'\uc1ca': 'HangulSyllableSwaenh',
u'\u784f': 'CjkUnifiedIdeograph784f',
u'\u5bce': 'CjkUnifiedIdeograph5bce',
u'\ub155': 'HangulSyllableNyeong',
u'\u90d4': 'CjkUnifiedIdeograph90d4',
u'\u0b59': 'UnicodeCharacter0xb59',
u'\u6ad8': 'CjkUnifiedIdeograph6ad8',
u'\uc65b': 'HangulSyllableWah',
u'\u405f': 'CjkUnifiedIdeograph405f',
u'\ua3de': 'YiSyllableRyrx',
u'\u3de2': 'CjkUnifiedIdeograph3de2',
u'\u9965': 'CjkUnifiedIdeograph9965',
u'\uf8e4': 'UnicodeCharacter0xf8e4',
u'\u1369': 'EthiopicDigitOne',
u'\u72e8': 'CjkUnifiedIdeograph72e8',
u'\ua86f': 'PhagspaLetterVoicedHa',
u'\u8bee': 'CjkUnifiedIdeograph8bee',
u'\u2273': 'GreaterthanOrEquivalentTo',
u'\u05f2': 'HebrewLigatureYiddishDoubleYod',
u'\ue175': 'UnicodeCharacter0xe175',
u'\uc0f4': 'HangulSyllableSyam',
u'\u7b79': 'CjkUnifiedIdeograph7b79',
u'\u5af8': 'CjkUnifiedIdeograph5af8',
u'\ub07f': 'HangulSyllableGgigs',
u'\u93fe': 'CjkUnifiedIdeograph93fe',
u'\ue606': 'UnicodeCharacter0xe606',
u'\uc289': 'HangulSyllableSyug',
u'\ua308': 'YiSyllableNzyr',
u'\u9f8b': 'CjkUnifiedIdeograph9f8b',
u'\u600a': 'CjkUnifiedIdeograph600a',
u'\u5c8d': 'CjkUnifiedIdeograph5c8d',
u'\u3d0c': 'CjkUnifiedIdeograph3d0c',
u'\u198f': 'NewTaiLueLetterHighTha',
u'\uce16': 'HangulSyllableCyubs',
u'\u8b18': 'CjkUnifiedIdeograph8b18',
u'\ue79b': 'UnicodeCharacter0xe79b',
u'\u481a': 'CjkUnifiedIdeograph481a',
u'\ua49d': 'YiRadicalYo',
u'\u051c': 'CyrillicCapitalLetterWe',
u'\u619f': 'CjkUnifiedIdeograph619f',
u'\u3ea1': 'CjkUnifiedIdeograph3ea1',
u'\ud626': 'HangulSyllableHyelm',
u'\u9328': 'CjkUnifiedIdeograph9328',
u'\ucfab': 'HangulSyllableKoegs',
u'\u502a': 'CjkUnifiedIdeograph502a',
u'\u8cad': 'CjkUnifiedIdeograph8cad',
u'\u6d2c': 'CjkUnifiedIdeograph6d2c',
u'\u49af': 'CjkUnifiedIdeograph49af',
u'\u2a2e': 'PlusSignInRightHalfCircle',
u'\u06b1': 'ArabicLetterNgoeh',
u'\ufb38': 'HebrewLetterTetWithDagesh',
u'\ud7bb': 'UnicodeCharacter0xd7bb',
u'\ub83a': 'HangulSyllableRyeoj',
u'\u94bd': 'CjkUnifiedIdeograph94bd',
u'\u753c': 'CjkUnifiedIdeograph753c',
u'\u51bf': 'CjkUnifiedIdeograph51bf',
u'\u323e': 'ParenthesizedIdeographResource',
u'\u6ec1': 'CjkUnifiedIdeograph6ec1',
u'\u2bc3': 'UnicodeCharacter0x2bc3',
u'\uc348': 'HangulSyllableSsyak',
u'\u804a': 'CjkUnifiedIdeograph804a',
u'\ufccd': 'ArabicLigatureLamWithHehInitialForm',
u'\u5d4c': 'CjkUnifiedIdeograph5d4c',
u'\ub9cf': 'HangulSyllableMad',
u'\u1a4e': 'UnicodeCharacter0x1a4e',
u'\u76d1': 'CjkUnifiedIdeograph76d1',
u'\u33d3': 'SquareLx',
u'\ue85a': 'UnicodeCharacter0xe85a',
u'\uc4dd': 'HangulSyllableSsyulg',
u'\ua55c': 'VaiSyllableDha',
u'\u81df': 'CjkUnifiedIdeograph81df',
u'\u625e': 'CjkUnifiedIdeograph625e',
u'\u5ee1': 'CjkUnifiedIdeograph5ee1',
u'\u3f60': 'CjkUnifiedIdeograph3f60',
u'\u1be3': 'UnicodeCharacter0x1be3',
u'\uf06a': 'UnicodeCharacter0xf06a',
u'\u8d6c': 'CjkUnifiedIdeograph8d6c',
u'\ue9ef': 'UnicodeCharacter0xe9ef',
u'\u4a6e': 'CjkUnifiedIdeograph4a6e',
u'\ua6f1': 'UnicodeCharacter0xa6f1',
u'\u0770': 'ArabicLetterSeenWithSmallArabicLetterTahAndTwoDots',
u'\u63f3': 'CjkUnifiedIdeograph63f3',
u'\u20f5': 'UnicodeCharacter0x20f5',
u'\ud87a': 'UnicodeCharacter0xd87a',
u'\u957c': 'CjkUnifiedIdeograph957c',
u'\uf1ff': 'UnicodeCharacter0xf1ff',
u'\u527e': 'CjkUnifiedIdeograph527e',
u'\u6f80': 'CjkUnifiedIdeograph6f80',
u'\ucb03': 'HangulSyllableJjwaelh',
u'\u2c82': 'CopticCapitalLetterVida',
u'\u8805': 'CjkUnifiedIdeograph8805',
u'\u4507': 'CjkUnifiedIdeograph4507',
u'\u0209': 'LatinSmallLetterIWithDoubleGrave',
u'\ufd8c': 'ArabicLigatureMeemWithJeemWithHahInitialForm',
u'\uba8e': 'HangulSyllableMyegg',
u'\u7790': 'CjkUnifiedIdeograph7790',
u'\ud313': 'HangulSyllablePad',
u'\u3492': 'CjkUnifiedIdeograph3492',
u'\u9015': 'CjkUnifiedIdeograph9015',
u'\uad17': 'HangulSyllableGwah',
u'\u6a19': 'CjkUnifiedIdeograph6a19',
u'\u271b': 'OpenCentreCross',
u'\uc59c': 'HangulSyllableYaen',
u'\u829e': 'CjkUnifiedIdeograph829e',
u'\u5fa0': 'CjkUnifiedIdeograph5fa0',
u'\u1ca2': 'UnicodeCharacter0x1ca2',
u'\uf825': 'UnicodeCharacter0xf825',
u'\ub527': 'HangulSyllableDis',
u'\u7229': 'CjkUnifiedIdeograph7229',
u'\u0f2b': 'TibetanDigitHalfTwo',
u'\ueaae': 'UnicodeCharacter0xeaae',
u'\ua7b0': 'UnicodeCharacter0xa7b0',
u'\u64b2': 'CjkUnifiedIdeograph64b2',
u'\uc035': 'HangulSyllableBbwing',
u'\u21b4': 'RightwardsArrowWithCornerDownwards',
u'\u9d37': 'CjkUnifiedIdeograph9d37',
u'\u5a39': 'CjkUnifiedIdeograph5a39',
u'\u173b': 'UnicodeCharacter0x173b',
u'\uf2be': 'UnicodeCharacter0xf2be',
u'\u8fc0': 'CjkUnifiedIdeograph8fc0',
u'\u4cc2': 'CjkUnifiedIdeograph4cc2',
u'\u09c4': 'BengaliVowelSignVocalicRr',
u'\ue547': 'UnicodeCharacter0xe547',
u'\ua249': 'YiSyllableHxip',
u'\u7f4b': 'CjkUnifiedIdeograph7f4b',
u'\u3c4d': 'CjkUnifiedIdeograph3c4d',
u'\udace': 'UnicodeCharacter0xdace',
u'\u97d0': 'CjkUnifiedIdeograph97d0',
u'\u54d2': 'CjkUnifiedIdeograph54d2',
u'\u11d4': 'HangulJongseongRieulpieuphieuh',
u'\ucd57': 'HangulSyllableCwaec',
u'\u2ed6': 'CjkRadicalMoundTwo',
u'\u8a59': 'CjkUnifiedIdeograph8a59',
u'\u475b': 'CjkUnifiedIdeograph475b',
u'\u045d': 'CyrillicSmallLetterIWithGrave',
u'\uffe0': 'FullwidthCentSign',
u'\ubce2': 'HangulSyllableByelm',
u'\u79e4': 'CjkUnifiedIdeograph79e4',
u'\ud567': 'HangulSyllableHalh',
u'\u36e6': 'CjkUnifiedIdeograph36e6',
u'\u9269': 'CjkUnifiedIdeograph9269',
u'\uaf6b': 'HangulSyllableGgwaed',
u'\u6c6d': 'CjkUnifiedIdeograph6c6d',
u'\u296f': 'DownwardsHarpoonWithBarbLeftBesideUpwardsHarpoonWithBarbRight',
u'\uc7f0': 'HangulSyllableJyaels',
u'\u84f2': 'CjkUnifiedIdeograph84f2',
u'\u41f4': 'CjkUnifiedIdeograph41f4',
u'\u1ef6': 'LatinCapitalLetterYWithHookAbove',
u'\ufa79': 'CjkCompatibilityIdeographfa79',
u'\ub77b': 'HangulSyllableDdih',
u'\u747d': 'CjkUnifiedIdeograph747d',
u'\u317f': 'HangulLetterPansios',
u'\u5181': 'CjkUnifiedIdeograph5181',
u'\ub604': 'HangulSyllableDdyem',
u'\u9a87': 'CjkUnifiedIdeograph9a87',
u'\u3c08': 'CjkUnifiedIdeograph3c08',
u'\x8b': 'UnicodeCharacter0x8b',
u'\uff0a': 'FullwidthAsterisk',
u'\uc38d': 'HangulSyllableSselg',
u'\u650e': 'CjkUnifiedIdeograph650e',
u'\u4991': 'CjkUnifiedIdeograph4991',
u'\uae14': 'HangulSyllableGyi',
u'\uf297': 'UnicodeCharacter0xf297',
u'\u1418': 'CanadianSyllabicsWestcreeWa',
u'\u789b': 'CjkUnifiedIdeograph789b',
u'\ud71a': 'HangulSyllableHwigg',
u'\u5d1e': 'CjkUnifiedIdeograph5d1e',
u'\ua1a1': 'YiSyllableHlap',
u'\u27a5': 'HeavyBlackCurvedDownwardsAndRightwardsArrow',
u'\u8624': 'CjkUnifiedIdeograph8624',
u'\ueaa7': 'UnicodeCharacter0xeaa7',
u'\u0c28': 'TeluguLetterNa',
u'\u50ab': 'CjkUnifiedIdeograph50ab',
u'\ucf2a': 'HangulSyllableKyeolp',
u'\ub52e': 'HangulSyllableDip',
u'\u99b1': 'CjkUnifiedIdeograph99b1',
u'\u3b32': 'CjkUnifiedIdeograph3b32',
u'\u1fb5': 'UnicodeCharacter0x1fb5',
u'\ufe34': 'PresentationFormForVerticalWavyLowLine',
u'\uc2b7': 'HangulSyllableSeus',
u'\u6438': 'CjkUnifiedIdeograph6438',
u'\u48bb': 'CjkUnifiedIdeograph48bb',
u'\uad3e': 'HangulSyllableGoelm',
u'\uf1c1': 'UnicodeCharacter0xf1c1',
u'\u1342': 'EthiopicSyllableTzi',
u'\u77c5': 'CjkUnifiedIdeograph77c5',
u'\ud644': 'HangulSyllableHols',
u'\u5c48': 'CjkUnifiedIdeograph5c48',
u'\ua0cb': 'YiSyllableMy',
u'\u26cf': 'UnicodeCharacter0x26cf',
u'\u854e': 'CjkUnifiedIdeograph854e',
u'\ue9d1': 'UnicodeCharacter0xe9d1',
u'\u0b52': 'UnicodeCharacter0xb52',
u'\u6fd5': 'CjkUnifiedIdeograph6fd5',
u'\uce54': 'HangulSyllableCyik',
u'\ub458': 'HangulSyllableDul',
u'\u98db': 'CjkUnifiedIdeograph98db',
u'\u3a5c': 'CjkUnifiedIdeograph3a5c',
u'\u1edf': 'LatinSmallLetterOWithHornAndHookAbove',
u'\ufd5e': 'ArabicLigatureSeenWithJeemWithAlefMaksuraFinalForm',
u'\uc1e1': 'HangulSyllableSoeg',
u'\u6362': 'CjkUnifiedIdeograph6362',
u'\u47e5': 'CjkUnifiedIdeograph47e5',
u'\uac68': 'HangulSyllableGyaess',
u'\uf0eb': 'UnicodeCharacter0xf0eb',
u'\u126c': 'EthiopicSyllableVee',
u'\u76ef': 'CjkUnifiedIdeograph76ef',
u'\ud56e': 'HangulSyllableHaj',
u'\u5b72': 'CjkUnifiedIdeograph5b72',
u'\ubff5': 'HangulSyllableBbweolt',
u'\u25f9': 'UpperRightTriangle',
u'\u8478': 'CjkUnifiedIdeograph8478',
u'\ue8fb': 'UnicodeCharacter0xe8fb',
u'\u0a7c': 'UnicodeCharacter0xa7c',
u'\u6eff': 'CjkUnifiedIdeograph6eff',
u'\ucd7e': 'HangulSyllableCyonh',
u'\u5403': 'CjkUnifiedIdeograph5403',
u'\ub382': 'HangulSyllableDebs',
u'\u3986': 'CjkUnifiedIdeograph3986',
u'\u9d09': 'CjkUnifiedIdeograph9d09',
u'\ufc88': 'ArabicLigatureMeemWithAlefFinalForm',
u'\u030d': 'CombiningVerticalLineAbove',
u'\u628c': 'CjkUnifiedIdeograph628c',
u'\uc60f': 'HangulSyllableYed',
u'\u4c13': 'CjkUnifiedIdeograph4c13',
u'\uab92': 'UnicodeCharacter0xab92',
u'\u3217': 'ParenthesizedHangulChieuchA',
u'\u1196': 'HangulJungseongEueu',
u'\uf519': 'UnicodeCharacter0xf519',
u'\ud498': 'HangulSyllablePweon',
u'\u7b1d': 'CjkUnifiedIdeograph7b1d',
u'\u5a9c': 'CjkUnifiedIdeograph5a9c',
u'\ua423': 'YiSyllableJjot',
u'\u83a2': 'CjkUnifiedIdeograph83a2',
u'\u2a27': 'PlusSignWithSubscriptTwo',
u'\u09a6': 'BengaliLetterDa',
u'\ued29': 'UnicodeCharacter0xed29',
u'\ucca8': 'HangulSyllableCeom',
u'\u532d': 'CjkUnifiedIdeograph532d',
u'\ub2ac': 'HangulSyllableNyi',
u'\u38b0': 'CjkUnifiedIdeograph38b0',
u'\u9c33': 'CjkUnifiedIdeograph9c33',
u'\ufbb2': 'UnicodeCharacter0xfbb2',
u'\u0237': 'LatinSmallLetterDotlessJ',
u'\u61b6': 'CjkUnifiedIdeograph61b6',
u'\uc539': 'HangulSyllableSsib',
u'\u4b3d': 'CjkUnifiedIdeograph4b3d',
u'\uaabc': 'UnicodeCharacter0xaabc',
u'\u3141': 'HangulLetterMieum',
u'\u10c0': 'GeorgianCapitalLetterHae',
u'\uf443': 'UnicodeCharacter0xf443',
u'\ud3c2': 'HangulSyllablePyeolp',
u'\u7a47': 'CjkUnifiedIdeograph7a47',
u'\u59c6': 'CjkUnifiedIdeograph59c6',
u'\ua34d': 'YiSyllableZhot',
u'\u82cc': 'CjkUnifiedIdeograph82cc',
u'\u2951': 'UpBarbLeftDownBarbLeftHarpoon',
u'\u08d0': 'UnicodeCharacter0x8d0',
u'\uec53': 'UnicodeCharacter0xec53',
u'\ucbd2': 'HangulSyllableJjyup',
u'\u5257': 'CjkUnifiedIdeograph5257',
u'\ub1d6': 'HangulSyllableNoelm',
u'\u37da': 'CjkUnifiedIdeograph37da',
u'\u9b5d': 'CjkUnifiedIdeograph9b5d',
u'\ufadc': 'UnicodeCharacter0xfadc',
u'\u0161': 'LatinSmallLetterSWithCaron',
u'\u60e0': 'CjkUnifiedIdeograph60e0',
u'\uc463': 'HangulSyllableSsyoh',
u'\u4a67': 'CjkUnifiedIdeograph4a67',
u'\ua9e6': 'UnicodeCharacter0xa9e6',
u'\u306b': 'HiraganaLetterNi',
u'\u2fea': 'UnicodeCharacter0x2fea',
u'\uf36d': 'UnicodeCharacter0xf36d',
u'\ud2ec': 'HangulSyllableTyik',
u'\u7971': 'CjkUnifiedIdeograph7971',
u'\u58f0': 'CjkUnifiedIdeograph58f0',
u'\ua277': 'YiSyllableHuo',
u'\u81f6': 'CjkUnifiedIdeograph81f6',
u'\u287b': 'BraillePatternDots124567',
u'\u07fa': 'NkoLajanyalan',
u'\ueb7d': 'UnicodeCharacter0xeb7d',
u'\ucafc': 'HangulSyllableJjwael',
u'\ud081': 'HangulSyllableKeung',
u'\ub100': 'HangulSyllableNyaess',
u'\u9583': 'CjkUnifiedIdeograph9583',
u'\u7202': 'CjkUnifiedIdeograph7202',
u'\u5685': 'CjkUnifiedIdeograph5685',
u'\u3704': 'CjkUnifiedIdeograph3704',
u'\u1b87': 'SundaneseLetterO',
u'\ue40e': 'UnicodeCharacter0xe40e',
u'\uc891': 'HangulSyllableJwanj',
u'\ua910': 'KayahLiLetterZa',
u'\u8d93': 'CjkUnifiedIdeograph8d93',
u'\u6a12': 'CjkUnifiedIdeograph6a12',
u'\u4e95': 'CjkUnifiedIdeograph4e95',
u'\u2f14': 'KangxiRadicalSpoon',
u'\u7397': 'CjkUnifiedIdeograph7397',
u'\u3499': 'CjkUnifiedIdeograph3499',
u'\udc1e': 'UnicodeCharacter0xdc1e',
u'\u8120': 'CjkUnifiedIdeograph8120',
u'\ue5a3': 'UnicodeCharacter0xe5a3',
u'\u4222': 'CjkUnifiedIdeograph4222',
u'\ua6a5': 'UnicodeCharacter0xa6a5',
u'\u0724': 'SyriacLetterFinalSemkath',
u'\u6ba7': 'CjkUnifiedIdeograph6ba7',
u'\u2ca9': 'CopticSmallLetterUa',
u'\uf930': 'CjkCompatibilityIdeographf930',
u'\uddb3': 'UnicodeCharacter0xddb3',
u'\uba32': 'HangulSyllableMyaej',
u'\u9eb5': 'CjkUnifiedIdeograph9eb5',
u'\u7f34': 'CjkUnifiedIdeograph7f34',
u'\u43b7': 'CjkUnifiedIdeograph43b7',
u'\u2036': 'ReversedDoublePrime',
u'\u04b9': 'CyrillicSmallLetterCheWithVerticalStroke',
u'\ud140': 'HangulSyllableTeom',
u'\u9242': 'CjkUnifiedIdeograph9242',
u'\uf6c5': 'UnicodeCharacter0xf6c5',
u'\u5744': 'CjkUnifiedIdeograph5744',
u'\ubbc7': 'HangulSyllableMeud',
u'\u1846': 'MongolianLetterTodoO',
u'\u7cc9': 'CjkUnifiedIdeograph7cc9',
u'\u21cb': 'LeftwardsHarpoonOverRightwardsHarpoon',
u'\uc950': 'HangulSyllableJwi',
u'\u8a52': 'CjkUnifiedIdeograph8a52',
u'\ueed5': 'UnicodeCharacter0xeed5',
u'\u4f54': 'CjkUnifiedIdeograph4f54',
u'\u93d7': 'CjkUnifiedIdeograph93d7',
u'\u7056': 'CjkUnifiedIdeograph7056',
u'\u54d9': 'CjkUnifiedIdeograph54d9',
u'\u3558': 'CjkUnifiedIdeograph3558',
u'\u19db': 'UnicodeCharacter0x19db',
u'\ue262': 'UnicodeCharacter0xe262',
u'\uc6e5': 'HangulSyllableWeot',
u'\ua764': 'LatinCapitalLetterThornWithStroke',
u'\u8be7': 'CjkUnifiedIdeograph8be7',
u'\u6866': 'CjkUnifiedIdeograph6866',
u'\u4ce9': 'CjkUnifiedIdeograph4ce9',
u'\u2d68': 'UnicodeCharacter0x2d68',
u'\u71eb': 'CjkUnifiedIdeograph71eb',
u'\u32ed': 'CircledKatakanaHo',
u'\uda72': 'UnicodeCharacter0xda72',
u'\u9f74': 'CjkUnifiedIdeograph9f74',
u'\ue3f7': 'UnicodeCharacter0xe3f7',
u'\u4076': 'CjkUnifiedIdeograph4076',
u'\ua4f9': 'UnicodeCharacter0xa4f9',
u'\u0578': 'ArmenianSmallLetterVo',
u'\u69fb': 'CjkUnifiedIdeograph69fb',
u'\u2afd': 'DoubleSolidusOperator',
u'\u1001': 'MyanmarLetterKha',
u'\uf784': 'UnicodeCharacter0xf784',
u'\ub886': 'HangulSyllableRwalp',
u'\u7d88': 'CjkUnifiedIdeograph7d88',
u'\uc10b': 'HangulSyllableSyaelb',
u'\u3e8a': 'CjkUnifiedIdeograph3e8a',
u'\u820d': 'CjkUnifiedIdeograph820d',
u'\u470f': 'CjkUnifiedIdeograph470f',
u'\u0811': 'UnicodeCharacter0x811',
u'\uef94': 'UnicodeCharacter0xef94',
u'\u9096': 'CjkUnifiedIdeograph9096',
u'\u5598': 'CjkUnifiedIdeograph5598',
u'\u169a': 'OghamLetterPeith',
u'\ufa1d': 'CjkCompatibilityIdeographfa1d',
u'\ubf1f': 'HangulSyllableBbyeoc',
u'\u6021': 'CjkUnifiedIdeograph6021',
u'\u2523': 'BoxDrawingsHeavyVerticalAndRight',
u'\uc7a4': 'HangulSyllableJass',
u'\u88a6': 'CjkUnifiedIdeograph88a6',
u'\u4da8': 'CjkUnifiedIdeograph4da8',
u'\u0eaa': 'LaoLetterSoSung',
u'\ud22d': 'HangulSyllableTug',
u'\u33ac': 'SquareGpa',
u'\u972f': 'CjkUnifiedIdeograph972f',
u'\u5831': 'CjkUnifiedIdeograph5831',
u'\u1d33': 'ModifierLetterCapitalG',
u'\ue0b6': 'UnicodeCharacter0xe0b6',
u'\ua5b8': 'VaiSyllableNu',
u'\u66ba': 'CjkUnifiedIdeograph66ba',
u'\uca3d': 'HangulSyllableJjyaelt',
u'\u2bbc': 'UnicodeCharacter0x2bbc',
u'\u8f3f': 'CjkUnifiedIdeograph8f3f',
u'\ub041': 'HangulSyllableGgyut',
u'\u7543': 'CjkUnifiedIdeograph7543',
u'\u3645': 'CjkUnifiedIdeograph3645',
u'\ud8c6': 'UnicodeCharacter0xd8c6',
u'\u9dc8': 'CjkUnifiedIdeograph9dc8',
u'\u5eca': 'CjkUnifiedIdeograph5eca',
u'\u03cc': 'GreekSmallLetterOmicronWithTonos',
u'\ue74f': 'UnicodeCharacter0xe74f',
u'\ua851': 'PhagspaLetterTsha',
u'\u6d53': 'CjkUnifiedIdeograph6d53',
u'\u2e55': 'UnicodeCharacter0x2e55',
u'\uf5d8': 'UnicodeCharacter0xf5d8',
u'\ub6da': 'HangulSyllableDdwenh',
u'\u7bdc': 'CjkUnifiedIdeograph7bdc',
u'\udf5f': 'UnicodeCharacter0xdf5f',
u'\u3cde': 'CjkUnifiedIdeograph3cde',
u'\u8061': 'CjkUnifiedIdeograph8061',
u'\u4563': 'CjkUnifiedIdeograph4563',
u'\u0665': 'ArabicindicDigitFive',
u'\uede8': 'UnicodeCharacter0xede8',
u'\uaeea': 'HangulSyllableGgebs',
u'\u53ec': 'CjkUnifiedIdeograph53ec',
u'\u14ee': 'CanadianSyllabicsSaai',
u'\uf871': 'UnicodeCharacter0xf871',
u'\ubd73': 'HangulSyllableByolh',
u'\u7e75': 'CjkUnifiedIdeograph7e75',
u'\u2377': 'AplFunctionalSymbolEpsilonUnderbar',
u'\uc5f8': 'HangulSyllableYeols',
u'\u86fa': 'CjkUnifiedIdeograph86fa',
u'\u4bfc': 'CjkUnifiedIdeograph4bfc',
u'\u0cfe': 'UnicodeCharacter0xcfe',
u'\u3200': 'ParenthesizedHangulKiyeok',
u'\u1683': 'OghamLetterFearn',
u'\uf102': 'UnicodeCharacter0xf102',
u'\ud585': 'HangulSyllableHaeb',
u'\u7b06': 'CjkUnifiedIdeograph7b06',
u'\u5f89': 'CjkUnifiedIdeograph5f89',
u'\ua00c': 'YiSyllableUox',
u'\u848f': 'CjkUnifiedIdeograph848f',
u'\u2a10': 'CirculationFunction',
u'\u0e93': 'UnicodeCharacter0xe93',
u'\ue912': 'UnicodeCharacter0xe912',
u'\ucd95': 'HangulSyllableCug',
u'\u5316': 'CjkUnifiedIdeograph5316',
u'\ub799': 'HangulSyllableRaeg',
u'\u399d': 'CjkUnifiedIdeograph399d',
u'\u981c': 'CjkUnifiedIdeograph981c',
u'\ufc9f': 'ArabicLigatureBehWithMeemInitialForm',
u'\u0220': 'LatinCapitalLetterNWithLongRightLeg',
u'\u66a3': 'CjkUnifiedIdeograph66a3',
u'\uc122': 'HangulSyllableSeonh',
u'\u4b26': 'CjkUnifiedIdeograph4b26',
u'\uafa9': 'HangulSyllableGgyolt',
u'\u11ad': 'HangulJongseongNieunhieuh',
u'\uf02c': 'UnicodeCharacter0xf02c',
u'\ud4af': 'HangulSyllablePweoh',
u'\u7a30': 'CjkUnifiedIdeograph7a30',
u'\u5eb3': 'CjkUnifiedIdeograph5eb3',
u'\ua336': 'YiSyllableSsox',
u'\u87b9': 'CjkUnifiedIdeograph87b9',
u'\u253a': 'BoxDrawingsLeftLightAndRightUpHeavy',
u'\u09bd': 'BengaliSignAvagraha',
u'\ue83c': 'UnicodeCharacter0xe83c',
u'\uccbf': 'HangulSyllableCelb',
u'\u5240': 'CjkUnifiedIdeograph5240',
u'\ub6c3': 'HangulSyllableDdweolb',
u'\u38c7': 'CjkUnifiedIdeograph38c7',
u'\u9b46': 'CjkUnifiedIdeograph9b46',
u'\uffc9': 'UnicodeCharacter0xffc9',
u'\u1d4a': 'ModifierLetterSmallSchwa',
u'\u61cd': 'CjkUnifiedIdeograph61cd',
u'\uc04c': 'HangulSyllableBbyum',
u'\u4a50': 'CjkUnifiedIdeograph4a50',
u'\uaed3': 'HangulSyllableGgeoc',
u'\u10d7': 'GeorgianLetterTan',
u'\uf356': 'UnicodeCharacter0xf356',
u'\ud7d9': 'UnicodeCharacter0xd7d9',
u'\u755a': 'CjkUnifiedIdeograph755a',
u'\u59dd': 'CjkUnifiedIdeograph59dd',
u'\ua260': 'YiSyllableNgat',
u'\u86e3': 'CjkUnifiedIdeograph86e3',
u'\u2464': 'CircledDigitFive',
u'\u08e7': 'UnicodeCharacter0x8e7',
u'\ueb66': 'UnicodeCharacter0xeb66',
u'\ucfe9': 'HangulSyllableKulg',
u'\u6d6a': 'CjkUnifiedIdeograph6d6a',
u'\ub1ed': 'HangulSyllableNyonj',
u'\u3bf1': 'CjkUnifiedIdeograph3bf1',
u'\u9a70': 'CjkUnifiedIdeograph9a70',
u'\ufef3': 'ArabicLetterYehInitialForm',
u'\u1c74': 'OlChikiLetterOtt',
u'\u60f7': 'CjkUnifiedIdeograph60f7',
u'\uc376': 'HangulSyllableSseolp',
u'\u457a': 'CjkUnifiedIdeograph457a',
u'\ua9fd': 'UnicodeCharacter0xa9fd',
u'\u9301': 'CjkUnifiedIdeograph9301',
u'\uf280': 'UnicodeCharacter0xf280',
u'\u1505': 'CanadianSyllabicsS',
u'\u7484': 'CjkUnifiedIdeograph7484',
u'\ud807': 'UnicodeCharacter0xd807',
u'\u420b': 'CjkUnifiedIdeograph420b',
u'\ubd8a': 'HangulSyllableBulm',
u'\u278e': 'DingbatNegativeCircledSansserifDigitFive',
u'\u8b11': 'CjkUnifiedIdeograph8b11',
u'\uea90': 'UnicodeCharacter0xea90',
u'\u0d15': 'MalayalamLetterKa',
u'\u6c94': 'CjkUnifiedIdeograph6c94',
u'\uba1b': 'HangulSyllableMyah',
u'\u959a': 'CjkUnifiedIdeograph959a',
u'\u3c1f': 'CjkUnifiedIdeograph3c1f',
u'\u1f9e': 'GreekCapitalLetterEtaWithPsiliAndPerispomeniAndProsgegrammeni',
u'\ue321': 'UnicodeCharacter0xe321',
u'\uc2a0': 'HangulSyllableSyuk',
u'\u6525': 'CjkUnifiedIdeograph6525',
u'\u44a4': 'CjkUnifiedIdeograph44a4',
u'\u922b': 'CjkUnifiedIdeograph922b',
u'\u8daa': 'CjkUnifiedIdeograph8daa',
u'\u142f': 'CanadianSyllabicsPe',
u'\u77ae': 'CjkUnifiedIdeograph77ae',
u'\udb31': 'UnicodeCharacter0xdb31',
u'\u5d35': 'CjkUnifiedIdeograph5d35',
u'\ubcb4': 'HangulSyllableBess',
u'\u26b8': 'BlackMoonLilith',
u'\u8a3b': 'CjkUnifiedIdeograph8a3b',
u'\ue5ba': 'UnicodeCharacter0xe5ba',
u'\u0c3f': 'TeluguVowelSignI',
u'\u6fbe': 'CjkUnifiedIdeograph6fbe',
u'\ub545': 'HangulSyllableDdang',
u'\u94c4': 'CjkUnifiedIdeograph94c4',
u'\u3f49': 'CjkUnifiedIdeograph3f49',
u'\u1ec8': 'LatinCapitalLetterIWithHookAbove',
u'\ue24b': 'UnicodeCharacter0xe24b',
u'\uddca': 'UnicodeCharacter0xddca',
u'\u644f': 'CjkUnifiedIdeograph644f',
u'\u47ce': 'CjkUnifiedIdeograph47ce',
u'\uad55': 'HangulSyllableGyonj',
u'\u8cd4': 'CjkUnifiedIdeograph8cd4',
u'\u1759': 'UnicodeCharacter0x1759',
u'\u76d8': 'CjkUnifiedIdeograph76d8',
u'\uda5b': 'UnicodeCharacter0xda5b',
u'\u5c5f': 'CjkUnifiedIdeograph5c5f',
u'\ubfde': 'HangulSyllableBbubs',
u'\u21e2': 'RightwardsDashedArrow',
u'\u8565': 'CjkUnifiedIdeograph8565',
u'\ue4e4': 'UnicodeCharacter0xe4e4',
u'\u0f69': 'TibetanLetterKssa',
u'\u6ee8': 'CjkUnifiedIdeograph6ee8',
u'\ub46f': 'HangulSyllableDweogs',
u'\u97ee': 'CjkUnifiedIdeograph97ee',
u'\u3e73': 'CjkUnifiedIdeograph3e73',
u'\u19f2': 'KhmerSymbolPiiRoc',
u'\ufd75': 'ArabicLigatureAinWithJeemWithMeemFinalForm',
u'\udcf4': 'UnicodeCharacter0xdcf4',
u'\u6779': 'CjkUnifiedIdeograph6779',
u'\u46f8': 'CjkUnifiedIdeograph46f8',
u'\uac7f': 'HangulSyllableGeolh',
u'\u8ffe': 'CjkUnifiedIdeograph8ffe',
u'\ufa06': 'CjkCompatibilityIdeographfa06',
u'\ude89': 'UnicodeCharacter0xde89',
u'\ubf08': 'HangulSyllableBbyeo',
u'\u838b': 'CjkUnifiedIdeograph838b',
u'\u7c0a': 'CjkUnifiedIdeograph7c0a',
u'\u408d': 'CjkUnifiedIdeograph408d',
u'\u210c': 'BlackletterCapitalH',
u'\u058f': 'UnicodeCharacter0x58f',
u'\ud216': 'HangulSyllableTyonh',
u'\u9718': 'CjkUnifiedIdeograph9718',
u'\ufb9b': 'ArabicLetterNgoehFinalForm',
u'\u541a': 'CjkUnifiedIdeograph541a',
u'\ub89d': 'HangulSyllableRwaelg',
u'\u191c': 'LimbuLetterHa',
u'\u7d9f': 'CjkUnifiedIdeograph7d9f',
u'\u22a1': 'SquaredDotOperator',
u'\uca26': 'HangulSyllableJjyabs',
u'\u8f28': 'CjkUnifiedIdeograph8f28',
u'\ud3ab': 'HangulSyllablePes',
u'\u4c2a': 'CjkUnifiedIdeograph4c2a',
u'\u90ad': 'CjkUnifiedIdeograph90ad',
u'\u712c': 'CjkUnifiedIdeograph712c',
u'\u55af': 'CjkUnifiedIdeograph55af',
u'\u362e': 'CjkUnifiedIdeograph362e',
u'\u1ab1': 'UnicodeCharacter0x1ab1',
u'\ue738': 'UnicodeCharacter0xe738',
u'\ucbbb': 'HangulSyllableJjyugs',
u'\ua43a': 'YiSyllableNjuo',
u'\u88bd': 'CjkUnifiedIdeograph88bd',
u'\u693c': 'CjkUnifiedIdeograph693c',
u'\u4dbf': 'UnicodeCharacter0x4dbf',
u'\u2e3e': 'UnicodeCharacter0x2e3e',
u'\u72c1': 'CjkUnifiedIdeograph72c1',
u'\u37c3': 'CjkUnifiedIdeograph37c3',
u'\udf48': 'UnicodeCharacter0xdf48',
u'\u9c4a': 'CjkUnifiedIdeograph9c4a',
u'\ue0cd': 'UnicodeCharacter0xe0cd',
u'\u414c': 'CjkUnifiedIdeograph414c',
u'\ua5cf': 'VaiSyllableLo',
u'\u064e': 'ArabicFatha',
u'\u6ad1': 'CjkUnifiedIdeograph6ad1',
u'\u2fd3': 'KangxiRadicalDragon',
u'\uf45a': 'UnicodeCharacter0xf45a',
u'\ud8dd': 'UnicodeCharacter0xd8dd',
u'\ub95c': 'HangulSyllableRyun',
u'\u9ddf': 'CjkUnifiedIdeograph9ddf',
u'\u7e5e': 'CjkUnifiedIdeograph7e5e',
u'\u42e1': 'CjkUnifiedIdeograph42e1',
u'\u2360': 'AplFunctionalSymbolQuadColon',
u'\u07e3': 'NkoLetterNa',
u'\uec6a': 'UnicodeCharacter0xec6a',
u'\u916c': 'CjkUnifiedIdeograph916c',
u'\uf5ef': 'UnicodeCharacter0xf5ef',
u'\u566e': 'CjkUnifiedIdeograph566e',
u'\ubaf1': 'HangulSyllableMwaeb',
u'\u1b70': 'BalineseMusicalSymbolCombiningKempulWithJegogan',
u'\u7ff3': 'CjkUnifiedIdeograph7ff3',
u'\u3cf5': 'CjkUnifiedIdeograph3cf5',
u'\uc47a': 'HangulSyllableSsuj',
u'\u897c': 'CjkUnifiedIdeograph897c',
u'\uedff': 'UnicodeCharacter0xedff',
u'\u4e7e': 'CjkUnifiedIdeograph4e7e',
u'\u7380': 'CjkUnifiedIdeograph7380',
u'\ud703': 'HangulSyllableHwed',
u'\u3082': 'HiraganaLetterMo',
u'\u9405': 'CjkUnifiedIdeograph9405',
u'\u5907': 'CjkUnifiedIdeograph5907',
u'\u1e09': 'LatinSmallLetterCWithCedillaAndAcute',
u'\ue18c': 'UnicodeCharacter0xe18c',
u'\ua68e': 'CyrillicCapitalLetterTswe',
u'\u6b90': 'CjkUnifiedIdeograph6b90',
u'\ucf13': 'HangulSyllableKes',
u'\u2892': 'BraillePatternDots258',
u'\u8c15': 'CjkUnifiedIdeograph8c15',
u'\ub117': 'HangulSyllableNeolh',
u'\u7619': 'CjkUnifiedIdeograph7619',
u'\u3b1b': 'CjkUnifiedIdeograph3b1b',
u'\ud99c': 'UnicodeCharacter0xd99c',
u'\u9e9e': 'CjkUnifiedIdeograph9e9e',
u'\u43a0': 'CjkUnifiedIdeograph43a0',
u'\xa2': 'CentSign',
u'\ue425': 'UnicodeCharacter0xe425',
u'\ua927': 'KayahLiVowelE',
u'\u6e29': 'CjkUnifiedIdeograph6e29',
u'\u132b': 'EthiopicSyllableChaa',
u'\uf6ae': 'UnicodeCharacter0xf6ae',
u'\ubbb0': 'HangulSyllableMyuls',
u'\u78b2': 'CjkUnifiedIdeograph78b2',
u'\udc35': 'UnicodeCharacter0xdc35',
u'\u3db4': 'CjkUnifiedIdeograph3db4',
u'\u8137': 'CjkUnifiedIdeograph8137',
u'\u4639': 'CjkUnifiedIdeograph4639',
u'\u0b3b': 'UnicodeCharacter0xb3b',
u'\ueebe': 'UnicodeCharacter0xeebe',
u'\u93c0': 'CjkUnifiedIdeograph93c0',
u'\u50c2': 'CjkUnifiedIdeograph50c2',
u'\u15c4': 'CanadianSyllabicsCarrierGhu',
u'\uf947': 'CjkCompatibilityIdeographf947',
u'\ube49': 'HangulSyllableBinj',
u'\u634b': 'CjkUnifiedIdeograph634b',
u'\u204d': 'BlackRightwardsBullet',
u'\uc6ce': 'HangulSyllableWeogg',
u'\u8bd0': 'CjkUnifiedIdeograph8bd0',
u'\u48d2': 'CjkUnifiedIdeograph48d2',
u'\u0dd4': 'SinhalaVowelSignKettiPaapilla',
u'\ud157': 'HangulSyllableTelb',
u'\u32d6': 'CircledKatakanaKi',
u'\u9659': 'CjkUnifiedIdeograph9659',
u'\u5b5b': 'CjkUnifiedIdeograph5b5b',
u'\u185d': 'MongolianLetterSibeE',
u'\ue3e0': 'UnicodeCharacter0xe3e0',
u'\ua0e2': 'YiSyllableVit',
u'\u65e4': 'CjkUnifiedIdeograph65e4',
u'\uc967': 'HangulSyllableJwic',
u'\u2ae6': 'LongDashFromLeftMemberOfDoubleVertical',
u'\u8e69': 'CjkUnifiedIdeograph8e69',
u'\ub36b': 'HangulSyllableDeoc',
u'\u706d': 'CjkUnifiedIdeograph706d',
u'\u356f': 'CjkUnifiedIdeograph356f',
u'\udbf0': 'UnicodeCharacter0xdbf0',
u'\u98f2': 'CjkUnifiedIdeograph98f2',
u'\u5df4': 'CjkUnifiedIdeograph5df4',
u'\u02f6': 'ModifierLetterMiddleDoubleAcuteAccent',
u'\ue679': 'UnicodeCharacter0xe679',
u'\uab7b': 'UnicodeCharacter0xab7b',
u'\u687d': 'CjkUnifiedIdeograph687d',
u'\u2d7f': 'UnicodeCharacter0x2d7f',
u'\u5581': 'CjkUnifiedIdeograph5581',
u'\ub204': 'HangulSyllableNu',
u'\u9e87': 'CjkUnifiedIdeograph9e87',
u'\u3808': 'CjkUnifiedIdeograph3808',
u'\u048b': 'CyrillicSmallLetterShortIWithTail',
u'\ufb0a': 'UnicodeCharacter0xfb0a',
u'\uc78d': 'HangulSyllableIt',
u'\u610e': 'CjkUnifiedIdeograph610e',
u'\u4d91': 'CjkUnifiedIdeograph4d91',
u'\uaa14': 'ChamLetterTha',
u'\uf697': 'UnicodeCharacter0xf697',
u'\u1018': 'MyanmarLetterBha',
u'\u7c9b': 'CjkUnifiedIdeograph7c9b',
u'\ud31a': 'HangulSyllablePalp',
u'\u591e': 'CjkUnifiedIdeograph591e',
u'\ua5a1': 'VaiSyllableGbu',
u'\u23a5': 'RightSquareBracketExtension',
u'\u8224': 'CjkUnifiedIdeograph8224',
u'\ueea7': 'UnicodeCharacter0xeea7',
u'\u0828': 'UnicodeCharacter0x828',
u'\u54ab': 'CjkUnifiedIdeograph54ab',
u'\ucb2a': 'HangulSyllableJjoep',
u'\ub12e': 'HangulSyllableNelm',
u'\u9db1': 'CjkUnifiedIdeograph9db1',
u'\u3f32': 'CjkUnifiedIdeograph3f32',
u'\u1bb5': 'SundaneseDigitFive',
u'\ufa34': 'CjkCompatibilityIdeographfa34',
u'\uc6b7': 'HangulSyllableUd',
u'\u6038': 'CjkUnifiedIdeograph6038',
u'\u4cbb': 'CjkUnifiedIdeograph4cbb',
u'\ua93e': 'RejangLetterLa',
u'\uf5c1': 'UnicodeCharacter0xf5c1',
u'\u1742': 'BuhidLetterU',
u'\u73c5': 'CjkUnifiedIdeograph73c5',
u'\ud244': 'HangulSyllableTuk',
u'\u5848': 'CjkUnifiedIdeograph5848',
u'\ua4cb': 'UnicodeCharacter0xa4cb',
u'\u22cf': 'CurlyLogicalAnd',
u'\u814e': 'CjkUnifiedIdeograph814e',
u'\uedd1': 'UnicodeCharacter0xedd1',
u'\u0f52': 'TibetanLetterDha',
u'\u6bd5': 'CjkUnifiedIdeograph6bd5',
u'\uca54': 'HangulSyllableJjeol',
u'\ub058': 'HangulSyllableGgeuss',
u'\u9cdb': 'CjkUnifiedIdeograph9cdb',
u'\u3e5c': 'CjkUnifiedIdeograph3e5c',
u'\u1adf': 'UnicodeCharacter0x1adf',
u'\uf95e': 'CjkCompatibilityIdeographf95e',
u'\uc5e1': 'HangulSyllableEb',
u'\u6762': 'CjkUnifiedIdeograph6762',
u'\u43e5': 'CjkUnifiedIdeograph43e5',
u'\ua868': 'PhagspaSubjoinedLetterYa',
u'\uf4eb': 'UnicodeCharacter0xf4eb',
u'\u166c': 'CanadianSyllabicsCarrierTtsa',
u'\u72ef': 'CjkUnifiedIdeograph72ef',
u'\ud16e': 'HangulSyllableTyeonh',
u'\u5f72': 'CjkUnifiedIdeograph5f72',
u'\ubbf5': 'HangulSyllableMyit',
u'\u21f9': 'LeftRightArrowWithVerticalStroke',
u'\u8078': 'CjkUnifiedIdeograph8078',
u'\uecfb': 'UnicodeCharacter0xecfb',
u'\u0e7c': 'UnicodeCharacter0xe7c',
u'\u6aff': 'CjkUnifiedIdeograph6aff',
u'\uc97e': 'HangulSyllableJyubs',
u'\u5003': 'CjkUnifiedIdeograph5003',
u'\ub782': 'HangulSyllableRanh',
u'\u3d86': 'CjkUnifiedIdeograph3d86',
u'\u9909': 'CjkUnifiedIdeograph9909',
u'\uf888': 'UnicodeCharacter0xf888',
u'\u070d': 'SyriacHarkleanAsteriscus',
u'\u668c': 'CjkUnifiedIdeograph668c',
u'\uc20f': 'HangulSyllableSyos',
u'\u4813': 'CjkUnifiedIdeograph4813',
u'\uaf92': 'HangulSyllableGgoebs',
u'\u3617': 'CjkUnifiedIdeograph3617',
u'\u1596': 'CanadianSyllabicsNng',
u'\uf119': 'UnicodeCharacter0xf119',
u'\ud098': 'HangulSyllableKyim',
u'\u7f1d': 'CjkUnifiedIdeograph7f1d',
u'\u5e9c': 'CjkUnifiedIdeograph5e9c',
u'\ua023': 'YiSyllableBuo',
u'\u87a2': 'CjkUnifiedIdeograph87a2',
u'\u2e27': 'RightSidewaysUBracket',
u'\u0da6': 'SinhalaLetterSanyakaJayanna',
u'\ue929': 'UnicodeCharacter0xe929',
u'\uc8a8': 'HangulSyllableJwae',
u'\u572d': 'CjkUnifiedIdeograph572d',
u'\ub6ac': 'HangulSyllableDdum',
u'\u3cb0': 'CjkUnifiedIdeograph3cb0',
u'\u9833': 'CjkUnifiedIdeograph9833',
u'\uffb2': 'HalfwidthHangulLetterPieup',
u'\u0637': 'ArabicLetterTah',
u'\u65b6': 'CjkUnifiedIdeograph65b6',
u'\uc139': 'HangulSyllableSeg',
u'\u4f3d': 'CjkUnifiedIdeograph4f3d',
u'\uaebc': 'HangulSyllableGgeo',
u'\u3541': 'CjkUnifiedIdeograph3541',
u'\u14c0': 'CanadianSyllabicsNe',
u'\uf043': 'UnicodeCharacter0xf043',
u'\ud7c2': 'UnicodeCharacter0xd7c2',
u'\u7e47': 'CjkUnifiedIdeograph7e47',
u'\u5dc6': 'CjkUnifiedIdeograph5dc6',
u'\ua74d': 'LatinSmallLetterOWithLoop',
u'\u86cc': 'CjkUnifiedIdeograph86cc',
u'\u2d51': 'TifinaghLetterTuaregYang',
u'\u0cd0': 'UnicodeCharacter0xcd0',
u'\ue853': 'UnicodeCharacter0xe853',
u'\ucfd2': 'HangulSyllableKyolp',
u'\u5657': 'CjkUnifiedIdeograph5657',
u'\ub5d6': 'HangulSyllableDdep',
u'\u33da': 'SquarePr',
u'\u9f5d': 'CjkUnifiedIdeograph9f5d',
u'\ufedc': 'ArabicLetterKafMedialForm',
u'\u0561': 'ArmenianSmallLetterAyb',
u'\u64e0': 'CjkUnifiedIdeograph64e0',
u'\uc063': 'HangulSyllableBbeulb',
u'\u4e67': 'CjkUnifiedIdeograph4e67',
u'\uade6': 'HangulSyllableGyulm',
u'\u346b': 'CjkUnifiedIdeograph346b',
u'\u2bea': 'UnicodeCharacter0x2bea',
u'\uf76d': 'UnicodeCharacter0xf76d',
u'\ud6ec': 'HangulSyllableHweols',
u'\u7d71': 'CjkUnifiedIdeograph7d71',
u'\u5cf0': 'CjkUnifiedIdeograph5cf0',
u'\ua677': 'UnicodeCharacter0xa677',
u'\u85f6': 'CjkUnifiedIdeograph85f6',
u'\u2c7b': 'LatinLetterSmallCapitalTurnedE',
u'\u03fa': 'GreekCapitalLetterSan',
u'\uef7d': 'UnicodeCharacter0xef7d',
u'\ucefc': 'HangulSyllableKeok',
u'\ud481': 'HangulSyllablePulg',
u'\ub500': 'HangulSyllableDyil',
u'\u9183': 'CjkUnifiedIdeograph9183',
u'\u7602': 'CjkUnifiedIdeograph7602',
u'\u5285': 'CjkUnifiedIdeograph5285',
u'\u3304': 'SquareIningu',
u'\u1f87': 'GreekSmallLetterAlphaWithDasiaAndPerispomeniAndYpogegrammeni',
u'\ue00e': 'UnicodeCharacter0xe00e',
u'\ucc91': 'HangulSyllableCyaeng',
u'\uad10': 'HangulSyllableGwass',
u'\u8993': 'CjkUnifiedIdeograph8993',
u'\u6e12': 'CjkUnifiedIdeograph6e12',
u'\u4a95': 'CjkUnifiedIdeograph4a95',
u'\u2b14': 'SquareWithUpperRightDiagonalHalfBlack',
u'\u7797': 'CjkUnifiedIdeograph7797',
u'\u3099': 'CombiningKatakanahiraganaVoicedSoundMark',
u'\ud81e': 'UnicodeCharacter0xd81e',
u'\u8520': 'CjkUnifiedIdeograph8520',
u'\ue1a3': 'UnicodeCharacter0xe1a3',
u'\u4622': 'CjkUnifiedIdeograph4622',
u'\ua2a5': 'YiSyllableZup',
u'\u0324': 'CombiningDiaeresisBelow',
u'\u6fa7': 'CjkUnifiedIdeograph6fa7',
u'\u28a9': 'BraillePatternDots1468',
u'\ufd30': 'ArabicLigatureSheenWithMeemInitialForm',
u'\ud9b3': 'UnicodeCharacter0xd9b3',
u'\ube32': 'HangulSyllableByilm',
u'\u9ab5': 'CjkUnifiedIdeograph9ab5',
u'\u7b34': 'CjkUnifiedIdeograph7b34',
u'\u47b7': 'CjkUnifiedIdeograph47b7',
u'\u2436': 'UnicodeCharacter0x2436',
u'\xb9': 'SuperscriptOne',
u'\ud540': 'HangulSyllablePin',
u'\u9642': 'CjkUnifiedIdeograph9642',
u'\uf2c5': 'UnicodeCharacter0xf2c5',
u'\u5344': 'CjkUnifiedIdeograph5344',
u'\ubfc7': 'HangulSyllableBbyoc',
u'\u1c46': 'LepchaDigitSix',
u'\u78c9': 'CjkUnifiedIdeograph78c9',
u'\u25cb': 'WhiteCircle',
u'\ucd50': 'HangulSyllableCwaem',
u'\u8e52': 'CjkUnifiedIdeograph8e52',
u'\uead5': 'UnicodeCharacter0xead5',
u'\u4b54': 'CjkUnifiedIdeograph4b54',
u'\u97d7': 'CjkUnifiedIdeograph97d7',
u'\u7456': 'CjkUnifiedIdeograph7456',
u'\u50d9': 'CjkUnifiedIdeograph50d9',
u'\u3158': 'HangulLetterWa',
u'\u1ddb': 'CombiningLatinLetterSmallCapitalG',
u'\ue662': 'UnicodeCharacter0xe662',
u'\uc2e5': 'HangulSyllableSilg',
u'\ua364': 'YiSyllableChap',
u'\u8fe7': 'CjkUnifiedIdeograph8fe7',
u'\u6c66': 'CjkUnifiedIdeograph6c66',
u'\u48e9': 'CjkUnifiedIdeograph48e9',
u'\u2968': 'RightwardsHarpoonWithBarbUpAboveLeftwardsHarpoonWithBarbUp',
u'\u75eb': 'CjkUnifiedIdeograph75eb',
u'\u36ed': 'CjkUnifiedIdeograph36ed',
u'\ude72': 'UnicodeCharacter0xde72',
u'\u9b74': 'CjkUnifiedIdeograph9b74',
u'\ue7f7': 'UnicodeCharacter0xe7f7',
u'\u4476': 'CjkUnifiedIdeograph4476',
u'\ua0f9': 'YiSyllableVur',
u'\u0178': 'LatinCapitalLetterYWithDiaeresis',
u'\u6dfb': 'CjkUnifiedIdeograph6dfb',
u'\u2efd': 'UnicodeCharacter0x2efd',
u'\u1401': 'CanadianSyllabicsE',
u'\uf384': 'UnicodeCharacter0xf384',
u'\ubc86': 'HangulSyllableBeogg',
u'\u7988': 'CjkUnifiedIdeograph7988',
u'\uc50b': 'HangulSyllableSseuh',
u'\u3a8a': 'CjkUnifiedIdeograph3a8a',
u'\u860d': 'CjkUnifiedIdeograph860d',
u'\u430f': 'CjkUnifiedIdeograph430f',
u'\u0c11': 'UnicodeCharacter0xc11',
u'\ueb94': 'UnicodeCharacter0xeb94',
u'\u9496': 'CjkUnifiedIdeograph9496',
u'\u5198': 'CjkUnifiedIdeograph5198',
u'\u129a': 'EthiopicSyllableNyi',
u'\ufe1d': 'UnicodeCharacter0xfe1d',
u'\ubb1f': 'HangulSyllableMyod',
u'\u6421': 'CjkUnifiedIdeograph6421',
u'\u2123': 'Versicle',
u'\uc3a4': 'HangulSyllableSsyeon',
u'\u8ca6': 'CjkUnifiedIdeograph8ca6',
u'\u49a8': 'CjkUnifiedIdeograph49a8',
u'\u0aaa': 'GujaratiLetterPa',
u'\ud62d': 'HangulSyllableHyeb',
u'\u37ac': 'CjkUnifiedIdeograph37ac',
u'\u932f': 'CjkUnifiedIdeograph932f',
u'\u5c31': 'CjkUnifiedIdeograph5c31',
u'\u1933': 'LimbuSmallLetterTa',
u'\ue4b6': 'UnicodeCharacter0xe4b6',
u'\ua1b8': 'YiSyllableLix',
u'\u62ba': 'CjkUnifiedIdeograph62ba',
u'\uce3d': 'HangulSyllableCyig',
u'\u2fbc': 'KangxiRadicalTall',
u'\u8b3f': 'CjkUnifiedIdeograph8b3f',
u'\ub441': 'HangulSyllableDyolt',
u'\u7143': 'CjkUnifiedIdeograph7143',
u'\u3245': 'UnicodeCharacter0x3245',
u'\udcc6': 'UnicodeCharacter0xdcc6',
u'\u99c8': 'CjkUnifiedIdeograph99c8',
u'\u5aca': 'CjkUnifiedIdeograph5aca',
u'\u07cc': 'NkoLetterI',
u'\ue34f': 'UnicodeCharacter0xe34f',
u'\uac51': 'HangulSyllableGyat',
u'\u6953': 'CjkUnifiedIdeograph6953',
u'\u2a55': 'TwoIntersectingLogicalAnd',
u'\uf1d8': 'UnicodeCharacter0xf1d8',
u'\ub2da': 'HangulSyllableNibs',
u'\u7fdc': 'CjkUnifiedIdeograph7fdc',
u'\udb5f': 'UnicodeCharacter0xdb5f',
u'\u38de': 'CjkUnifiedIdeograph38de',
u'\u8461': 'CjkUnifiedIdeograph8461',
u'\u4163': 'CjkUnifiedIdeograph4163',
u'\u0265': 'LatinSmallLetterTurnedH',
u'\ue9e8': 'UnicodeCharacter0xe9e8',
u'\uaaea': 'UnicodeCharacter0xaaea',
u'\u57ec': 'CjkUnifiedIdeograph57ec',
u'\u10ee': 'GeorgianLetterXan',
u'\ufc71': 'ArabicLigatureTehWithZainFinalForm',
u'\ub973': 'HangulSyllableRyuh',
u'\u7a75': 'CjkUnifiedIdeograph7a75',
u'\u2777': 'DingbatNegativeCircledDigitTwo',
u'\uc1f8': 'HangulSyllableSoek',
u'\u82fa': 'CjkUnifiedIdeograph82fa',
u'\u4ffc': 'CjkUnifiedIdeograph4ffc',
u'\u08fe': 'UnicodeCharacter0x8fe',
u'\u3600': 'CjkUnifiedIdeograph3600',
u'\u1283': 'EthiopicSyllableXaa',
u'\uf502': 'UnicodeCharacter0xf502',
u'\ud185': 'HangulSyllableTyeg',
u'\u7f06': 'CjkUnifiedIdeograph7f06',
u'\u5b89': 'CjkUnifiedIdeograph5b89',
u'\ua40c': 'YiSyllableQut',
u'\u808f': 'CjkUnifiedIdeograph808f',
u'\u2e10': 'ForkedParagraphos',
u'\u0a93': 'GujaratiLetterO',
u'\ued12': 'UnicodeCharacter0xed12',
u'\uc995': 'HangulSyllableJeult',
u'\u5716': 'CjkUnifiedIdeograph5716',
u'\ub399': 'HangulSyllableDyeolt',
u'\u3d9d': 'CjkUnifiedIdeograph3d9d',
u'\u9c1c': 'CjkUnifiedIdeograph9c1c',
u'\uf89f': 'UnicodeCharacter0xf89f',
u'\u0620': 'UnicodeCharacter0x620',
u'\u62a3': 'CjkUnifiedIdeograph62a3',
u'\uc522': 'HangulSyllableSsyij',
u'\u4f26': 'CjkUnifiedIdeograph4f26',
u'\uaba9': 'UnicodeCharacter0xaba9',
u'\u15ad': 'CanadianSyllabicsThcreeThaa',
u'\uf42c': 'UnicodeCharacter0xf42c',
u'\ud0af': 'HangulSyllableKilb',
u'\u7e30': 'CjkUnifiedIdeograph7e30',
u'\u5ab3': 'CjkUnifiedIdeograph5ab3',
u'\ua736': 'LatinCapitalLetterAu',
u'\u83b9': 'CjkUnifiedIdeograph83b9',
u'\u213a': 'RotatedCapitalQ',
u'\u0dbd': 'SinhalaLetterDantajaLayanna',
u'\uec3c': 'UnicodeCharacter0xec3c',
u'\uc8bf': 'HangulSyllableJwaec',
u'\u5640': 'CjkUnifiedIdeograph5640',
u'\ub2c3': 'HangulSyllableNyic',
u'\u3cc7': 'CjkUnifiedIdeograph3cc7',
u'\u9f46': 'CjkUnifiedIdeograph9f46',
u'\ufbc9': 'UnicodeCharacter0xfbc9',
u'\u194a': 'LimbuDigitFour',
u'\u65cd': 'CjkUnifiedIdeograph65cd',
u'\uc44c': 'HangulSyllableSsyon',
u'\u4e50': 'CjkUnifiedIdeograph4e50',
u'\uaad3': 'UnicodeCharacter0xaad3',
u'\u14d7': 'CanadianSyllabicsLo',
u'\uf756': 'UnicodeCharacter0xf756',
u'\ud3d9': 'HangulSyllablePyelg',
u'\u715a': 'CjkUnifiedIdeograph715a',
u'\u5ddd': 'CjkUnifiedIdeograph5ddd',
u'\ua660': 'UnicodeCharacter0xa660',
u'\u82e3': 'CjkUnifiedIdeograph82e3',
u'\u2064': 'InvisiblePlus',
u'\u0ce7': 'KannadaDigitOne',
u'\uef66': 'UnicodeCharacter0xef66',
u'\ucbe9': 'HangulSyllableJjeung',
u'\u696a': 'CjkUnifiedIdeograph696a',
u'\ub5ed': 'HangulSyllableDdyeong',
u'\u3ff1': 'CjkUnifiedIdeograph3ff1',
u'\u9e70': 'CjkUnifiedIdeograph9e70',
u'\ufaf3': 'UnicodeCharacter0xfaf3',
u'\u1874': 'MongolianLetterManchuKa',
u'\u64f7': 'CjkUnifiedIdeograph64f7',
u'\uc776': 'HangulSyllableIgg',
u'\u417a': 'CjkUnifiedIdeograph417a',
u'\uadfd': 'HangulSyllableGeunj',
u'\u9701': 'CjkUnifiedIdeograph9701',
u'\uf680': 'UnicodeCharacter0xf680',
u'\u1105': 'HangulChoseongRieul',
u'\u7084': 'CjkUnifiedIdeograph7084',
u'\udc07': 'UnicodeCharacter0xdc07',
u'\u460b': 'CjkUnifiedIdeograph460b',
u'\ub98a': 'HangulSyllableReuj',
u'\u238e': 'HysteresisSymbol',
u'\u8f11': 'CjkUnifiedIdeograph8f11',
u'\uee90': 'UnicodeCharacter0xee90',
u'\u0915': 'DevanagariLetterKa',
u'\u6894': 'CjkUnifiedIdeograph6894',
u'\ube1b': 'HangulSyllableBeulh',
u'\u919a': 'CjkUnifiedIdeograph919a',
u'\u381f': 'CjkUnifiedIdeograph381f',
u'\u1b9e': 'SundaneseLetterSa',
u'\ue721': 'UnicodeCharacter0xe721',
u'\uc6a0': 'HangulSyllableYols',
u'\u6125': 'CjkUnifiedIdeograph6125',
u'\u40a4': 'CjkUnifiedIdeograph40a4',
u'\u962b': 'CjkUnifiedIdeograph962b',
u'\u89aa': 'CjkUnifiedIdeograph89aa',
u'\u102f': 'MyanmarVowelSignU',
u'\u73ae': 'CjkUnifiedIdeograph73ae',
u'\udf31': 'UnicodeCharacter0xdf31',
u'\u5935': 'CjkUnifiedIdeograph5935',
u'\ub8b4': 'HangulSyllableRoen',
u'\u22b8': 'Multimap',
u'\u8e3b': 'CjkUnifiedIdeograph8e3b',
u'\ue1ba': 'UnicodeCharacter0xe1ba',
u'\u083f': 'UnicodeCharacter0x83f',
u'\u6bbe': 'CjkUnifiedIdeograph6bbe',
u'\ub145': 'HangulSyllableNyeonj',
u'\u90c4': 'CjkUnifiedIdeograph90c4',
u'\u3b49': 'CjkUnifiedIdeograph3b49',
u'\u1ac8': 'UnicodeCharacter0x1ac8',
u'\ue64b': 'UnicodeCharacter0xe64b',
u'\ud9ca': 'UnicodeCharacter0xd9ca',
u'\u604f': 'CjkUnifiedIdeograph604f',
u'\u43ce': 'CjkUnifiedIdeograph43ce',
u'\ua955': 'UnicodeCharacter0xa955',
u'\u88d4': 'CjkUnifiedIdeograph88d4',
u'\u1359': 'EthiopicSyllableMya',
u'\u72d8': 'CjkUnifiedIdeograph72d8',
u'\ude5b': 'UnicodeCharacter0xde5b',
u'\u585f': 'CjkUnifiedIdeograph585f',
u'\ubbde': 'HangulSyllableMyigg',
u'\u25e2': 'BlackLowerRightTriangle',
u'\u8165': 'CjkUnifiedIdeograph8165',
u'\ue0e4': 'UnicodeCharacter0xe0e4',
u'\u0b69': 'OriyaDigitThree',
u'\u6ae8': 'CjkUnifiedIdeograph6ae8',
u'\ub06f': 'HangulSyllableGgyilh',
u'\u93ee': 'CjkUnifiedIdeograph93ee',
u'\u3a73': 'CjkUnifiedIdeograph3a73',
u'\u1df2': 'UnicodeCharacter0x1df2',
u'\uf975': 'CjkCompatibilityIdeographf975',
u'\ud8f4': 'UnicodeCharacter0xd8f4',
u'\u6379': 'CjkUnifiedIdeograph6379',
u'\u42f8': 'CjkUnifiedIdeograph42f8',
u'\ua87f': 'UnicodeCharacter0xa87f',
u'\u8bfe': 'CjkUnifiedIdeograph8bfe',
u'\ufe06': 'VariationSelector7',
u'\uda89': 'UnicodeCharacter0xda89',
u'\ubb08': 'HangulSyllableMoels',
u'\u878b': 'CjkUnifiedIdeograph878b',
u'\u780a': 'CjkUnifiedIdeograph780a',
u'\u448d': 'CjkUnifiedIdeograph448d',
u'\u250c': 'BoxDrawingsLightDownAndRight',
u'\u018f': 'LatinCapitalLetterSchwa',
u'\ud616': 'HangulSyllableHyeoj',
u'\u9318': 'CjkUnifiedIdeograph9318',
u'\uff9b': 'HalfwidthKatakanaLetterRo',
u'\u501a': 'CjkUnifiedIdeograph501a',
u'\ubc9d': 'HangulSyllableBeot',
u'\u1d1c': 'LatinLetterSmallCapitalU',
u'\u799f': 'CjkUnifiedIdeograph799f',
u'\u26a1': 'HighVoltageSign',
u'\uce26': 'HangulSyllableCeunh',
u'\u8b28': 'CjkUnifiedIdeograph8b28',
u'\ud7ab': 'UnicodeCharacter0xd7ab',
u'\u482a': 'CjkUnifiedIdeograph482a',
u'\u94ad': 'CjkUnifiedIdeograph94ad',
u'\u752c': 'CjkUnifiedIdeograph752c',
u'\u51af': 'CjkUnifiedIdeograph51af',
u'\u322e': 'ParenthesizedIdeographMetal',
u'\u1eb1': 'LatinSmallLetterAWithBreveAndGrave',
u'\ue338': 'UnicodeCharacter0xe338',
u'\ucfbb': 'HangulSyllableKoes',
u'\ua03a': 'YiSyllablePi',
u'\u8cbd': 'CjkUnifiedIdeograph8cbd',
u'\u6d3c': 'CjkUnifiedIdeograph6d3c',
u'\u49bf': 'CjkUnifiedIdeograph49bf',
u'\u2a3e': 'ZNotationRelationalComposition',
u'\u76c1': 'CjkUnifiedIdeograph76c1',
u'\u33c3': 'SquareBq',
u'\udb48': 'UnicodeCharacter0xdb48',
u'\u984a': 'CjkUnifiedIdeograph984a',
u'\ue4cd': 'UnicodeCharacter0xe4cd',
u'\u454c': 'CjkUnifiedIdeograph454c',
u'\ua1cf': 'YiSyllableLux',
u'\u024e': 'LatinCapitalLetterYWithStroke',
u'\u6ed1': 'CjkUnifiedIdeograph6ed1',
u'\u2bd3': 'UnicodeCharacter0x2bd3',
u'\uf05a': 'UnicodeCharacter0xf05a',
u'\udcdd': 'UnicodeCharacter0xdcdd',
u'\ubd5c': 'HangulSyllableBoess',
u'\u99df': 'CjkUnifiedIdeograph99df',
u'\u7a5e': 'CjkUnifiedIdeograph7a5e',
u'\u46e1': 'CjkUnifiedIdeograph46e1',
u'\u2760': 'UnicodeCharacter0x2760',
u'\u03e3': 'CopticSmallLetterShei',
u'\ue86a': 'UnicodeCharacter0xe86a',
u'\u956c': 'CjkUnifiedIdeograph956c',
u'\uf1ef': 'UnicodeCharacter0xf1ef',
u'\u526e': 'CjkUnifiedIdeograph526e',
u'\ubef1': 'HangulSyllableBbenj',
u'\u1f70': 'GreekSmallLetterAlphaWithVaria',
u'\u7bf3': 'CjkUnifiedIdeograph7bf3',
u'\u38f5': 'CjkUnifiedIdeograph38f5',
u'\uc07a': 'HangulSyllableBbyinh',
u'\u8d7c': 'CjkUnifiedIdeograph8d7c',
u'\ue9ff': 'UnicodeCharacter0xe9ff',
u'\u4a7e': 'CjkUnifiedIdeograph4a7e',
u'\u7780': 'CjkUnifiedIdeograph7780',
u'\ud303': 'HangulSyllableTis',
u'\u3482': 'CjkUnifiedIdeograph3482',
u'\u9005': 'CjkUnifiedIdeograph9005',
u'\u5d07': 'CjkUnifiedIdeograph5d07',
u'\u1a09': 'BugineseLetterDa',
u'\ue58c': 'UnicodeCharacter0xe58c',
u'\ua28e': 'YiSyllableZix',
u'\u6f90': 'CjkUnifiedIdeograph6f90',
u'\ucb13': 'HangulSyllableJjoegs',
u'\u2c92': 'CopticCapitalLetterIauda',
u'\u8815': 'CjkUnifiedIdeograph8815',
u'\ub517': 'HangulSyllableDigs',
u'\u7219': 'CjkUnifiedIdeograph7219',
u'\u3f1b': 'CjkUnifiedIdeograph3f1b',
u'\udd9c': 'UnicodeCharacter0xdd9c',
u'\u9a9e': 'CjkUnifiedIdeograph9a9e',
u'\u47a0': 'CjkUnifiedIdeograph47a0',
u'\u04a2': 'CyrillicCapitalLetterEnWithDescender',
u'\ue025': 'UnicodeCharacter0xe025',
u'\uad27': 'HangulSyllableGwaelh',
u'\u6a29': 'CjkUnifiedIdeograph6a29',
u'\u172b': 'HanunooLetterMa',
u'\uf2ae': 'UnicodeCharacter0xf2ae',
u'\ubfb0': 'HangulSyllableBbyo',
u'\u7cb2': 'CjkUnifiedIdeograph7cb2',
u'\ud835': 'UnicodeCharacter0xd835',
u'\u39b4': 'CjkUnifiedIdeograph39b4',
u'\u8537': 'CjkUnifiedIdeograph8537',
u'\u4239': 'CjkUnifiedIdeograph4239',
u'\u0f3b': 'TibetanMarkGugRtagsGyas',
u'\ueabe': 'UnicodeCharacter0xeabe',
u'\u97c0': 'CjkUnifiedIdeograph97c0',
u'\u54c2': 'CjkUnifiedIdeograph54c2',
u'\u11c4': 'HangulJongseongKiyeoksioskiyeok',
u'\ufd47': 'UnicodeCharacter0xfd47',
u'\uba49': 'HangulSyllableMeob',
u'\u674b': 'CjkUnifiedIdeograph674b',
u'\u244d': 'UnicodeCharacter0x244d',
u'\uc2ce': 'HangulSyllableSyilp',
u'\u8fd0': 'CjkUnifiedIdeograph8fd0',
u'\u4cd2': 'CjkUnifiedIdeograph4cd2',
u'\u09d4': 'UnicodeCharacter0x9d4',
u'\ud557': 'HangulSyllablePih',
u'\u36d6': 'CjkUnifiedIdeograph36d6',
u'\u9259': 'CjkUnifiedIdeograph9259',
u'\u5f5b': 'CjkUnifiedIdeograph5f5b',
u'\u1c5d': 'OlChikiLetterAng',
u'\ue7e0': 'UnicodeCharacter0xe7e0',
u'\ua4e2': 'UnicodeCharacter0xa4e2',
u'\u61e4': 'CjkUnifiedIdeograph61e4',
u'\ucd67': 'HangulSyllableCoelb',
u'\u2ee6': 'CjkRadicalCsimplifiedBird',
u'\u8a69': 'CjkUnifiedIdeograph8a69',
u'\ub76b': 'HangulSyllableDdilb',
u'\u746d': 'CjkUnifiedIdeograph746d',
u'\u316f': 'HangulLetterMieumsios',
u'\udff0': 'UnicodeCharacter0xdff0',
u'\u9cf2': 'CjkUnifiedIdeograph9cf2',
u'\u59f4': 'CjkUnifiedIdeograph59f4',
u'\u06f6': 'ExtendedArabicindicDigitSix',
u'\ue279': 'UnicodeCharacter0xe279',
u'\uaf7b': 'HangulSyllableGgwaec',
u'\u6c7d': 'CjkUnifiedIdeograph6c7d',
u'\u297f': 'DownFishTail',
u'\u5981': 'CjkUnifiedIdeograph5981',
u'\ube04': 'HangulSyllableByuss',
u'\u9287': 'CjkUnifiedIdeograph9287',
u'\u3408': 'CjkUnifiedIdeograph3408',
u'\u088b': 'UnicodeCharacter0x88b',
u'\uf70a': 'UnicodeCharacter0xf70a',
u'\ucb8d': 'HangulSyllableJjwelt',
u'\u6d0e': 'CjkUnifiedIdeograph6d0e',
u'\u4191': 'CjkUnifiedIdeograph4191',
u'\ua614': 'VaiSymbolKeeng',
u'\ufa97': 'CjkCompatibilityIdeographfa97',
u'\u1c18': 'LepchaLetterTsha',
u'\u709b': 'CjkUnifiedIdeograph709b',
u'\udf1a': 'UnicodeCharacter0xdf1a',
u'\u551e': 'CjkUnifiedIdeograph551e',
u'\ua9a1': 'UnicodeCharacter0xa9a1',
u'\u2fa5': 'KangxiRadicalVillage',
u'\u8e24': 'CjkUnifiedIdeograph8e24',
u'\ue2a7': 'UnicodeCharacter0xe2a7',
u'\u0428': 'CyrillicCapitalLetterSha',
u'\u58ab': 'CjkUnifiedIdeograph58ab',
u'\uc72a': 'HangulSyllableYulm',
u'\ubd2e': 'HangulSyllableBwaegg',
u'\u91b1': 'CjkUnifiedIdeograph91b1',
u'\u3332': 'SquareHuaraddo',
u'\u17b5': 'KhmerVowelInherentAa',
u'\uf634': 'UnicodeCharacter0xf634',
u'\ucab7': 'HangulSyllableJjyec',
u'\u6c38': 'CjkUnifiedIdeograph6c38',
u'\u40bb': 'CjkUnifiedIdeograph40bb',
u'\ua53e': 'VaiSyllableZhi',
u'\uf9c1': 'CjkCompatibilityIdeographf9c1',
u'\u1b42': 'BalineseVowelSignPepet',
u'\u7fc5': 'CjkUnifiedIdeograph7fc5',
u'\ude44': 'UnicodeCharacter0xde44',
u'\u5448': 'CjkUnifiedIdeograph5448',
u'\ua8cb': 'UnicodeCharacter0xa8cb',
u'\u2ecf': 'CjkRadicalCity',
u'\u8d4e': 'CjkUnifiedIdeograph8d4e',
u'\ue1d1': 'UnicodeCharacter0xe1d1',
u'\u0352': 'CombiningFermata',
u'\u67d5': 'CjkUnifiedIdeograph67d5',
u'\uc654': 'HangulSyllableWass',
u'\ubc58': 'HangulSyllableByals',
u'\u90db': 'CjkUnifiedIdeograph90db',
u'\u325c': 'CircledNumberThirtyTwo',
u'\u16df': 'RunicLetterOthalanEthelO',
u'\uf55e': 'UnicodeCharacter0xf55e',
u'\uc9e1': 'HangulSyllableJjanj',
u'\u6b62': 'CjkUnifiedIdeograph6b62',
u'\u4fe5': 'CjkUnifiedIdeograph4fe5',
u'\ua468': 'YiSyllableXox',
u'\uf8eb': 'UnicodeCharacter0xf8eb',
u'\u1a6c': 'UnicodeCharacter0x1a6c',
u'\u7eef': 'CjkUnifiedIdeograph7eef',
u'\udd6e': 'UnicodeCharacter0xdd6e',
u'\u5372': 'CjkUnifiedIdeograph5372',
u'\ub7f5': 'HangulSyllableReolg',
u'\u2df9': 'CombiningCyrillicLetterMonographUk',
u'\u8c78': 'CjkUnifiedIdeograph8c78',
u'\ue0fb': 'UnicodeCharacter0xe0fb',
u'\u027c': 'LatinSmallLetterRWithLongLeg',
u'\u66ff': 'CjkUnifiedIdeograph66ff',
u'\uc57e': 'HangulSyllableYagg',
u'\u5c03': 'CjkUnifiedIdeograph5c03',
u'\ubb82': 'HangulSyllableMwej',
u'\u3186': 'HangulLetterYeorinhieuh',
u'\u9509': 'CjkUnifiedIdeograph9509',
u'\uf488': 'UnicodeCharacter0xf488',
u'\u0b0d': 'UnicodeCharacter0xb0d',
u'\u6a8c': 'CjkUnifiedIdeograph6a8c',
u'\uce0f': 'HangulSyllableCyulb',
u'\u4413': 'CjkUnifiedIdeograph4413',
u'\ua392': 'YiSyllableRryrx',
u'\u3a17': 'CjkUnifiedIdeograph3a17',
u'\u1996': 'NewTaiLueLetterHighMa',
u'\ufd19': 'ArabicLigatureSheenWithAlefMaksuraFinalForm',
u'\udc98': 'UnicodeCharacter0xdc98',
u'\u731d': 'CjkUnifiedIdeograph731d',
u'\u529c': 'CjkUnifiedIdeograph529c',
u'\uac23': 'HangulSyllableGaed',
u'\u8ba2': 'CjkUnifiedIdeograph8ba2',
u'\u2227': 'LogicalAnd',
u'\u01a6': 'LatinLetterYr',
u'\ue529': 'UnicodeCharacter0xe529',
u'\uc4a8': 'HangulSyllableSswels',
u'\u5b2d': 'CjkUnifiedIdeograph5b2d',
u'\ubaac': 'HangulSyllableMon',
u'\u30b0': 'KatakanaLetterGu',
u'\u9433': 'CjkUnifiedIdeograph9433',
u'\uf3b2': 'UnicodeCharacter0xf3b2',
u'\u0a37': 'UnicodeCharacter0xa37',
u'\u69b6': 'CjkUnifiedIdeograph69b6',
u'\ucd39': 'HangulSyllableCwang',
u'\u433d': 'CjkUnifiedIdeograph433d',
u'\ua2bc': 'YiSyllableCuop',
u'\u3941': 'CjkUnifiedIdeograph3941',
u'\u18c0': 'UnicodeCharacter0x18c0',
u'\ufc43': 'ArabicLigatureLamWithAlefMaksuraIsolatedForm',
u'\udbc2': 'UnicodeCharacter0xdbc2',
u'\u7247': 'CjkUnifiedIdeograph7247',
u'\u51c6': 'CjkUnifiedIdeograph51c6',
u'\uab4d': 'UnicodeCharacter0xab4d',
u'\u8acc': 'CjkUnifiedIdeograph8acc',
u'\u2151': 'UnicodeCharacter0x2151',
u'\xd0': 'LatinCapitalLetterEth',
u'\ue453': 'UnicodeCharacter0xe453',
u'\uc3d2': 'HangulSyllableSsyej',
u'\u5a57': 'CjkUnifiedIdeograph5a57',
u'\ub9d6': 'HangulSyllableMalp',
u'\u3fda': 'CjkUnifiedIdeograph3fda',
u'\u935d': 'CjkUnifiedIdeograph935d',
u'\uf2dc': 'UnicodeCharacter0xf2dc',
u'\u0961': 'DevanagariLetterVocalicLl',
u'\u68e0': 'CjkUnifiedIdeograph68e0',
u'\ucc63': 'HangulSyllableCyags',
u'\u4267': 'CjkUnifiedIdeograph4267',
u'\ua1e6': 'YiSyllableGuot',
u'\u386b': 'CjkUnifiedIdeograph386b',
u'\u27ea': 'MathematicalLeftDoubleAngleBracket',
u'\ufb6d': 'ArabicLetterVehMedialForm',
u'\udaec': 'UnicodeCharacter0xdaec',
u'\u7171': 'CjkUnifiedIdeograph7171',
u'\u50f0': 'CjkUnifiedIdeograph50f0',
u'\uaa77': 'UnicodeCharacter0xaa77',
u'\u89f6': 'CjkUnifiedIdeograph89f6',
u'\u207b': 'SuperscriptMinus',
u'\u0ffa': 'UnicodeCharacter0xffa',
u'\ue37d': 'UnicodeCharacter0xe37d',
u'\uc2fc': 'HangulSyllableSsan',
u'\ud881': 'UnicodeCharacter0xd881',
u'\ub900': 'HangulSyllableRuk',
u'\u9d83': 'CjkUnifiedIdeograph9d83',
u'\u7a02': 'CjkUnifiedIdeograph7a02',
u'\u5e85': 'CjkUnifiedIdeograph5e85',
u'\u3f04': 'CjkUnifiedIdeograph3f04',
u'\u1387': 'EthiopicSyllableBwe',
u'\uec0e': 'UnicodeCharacter0xec0e',
u'\uc091': 'HangulSyllableBbig',
u'\ua110': 'YiSyllableDop',
u'\u8593': 'CjkUnifiedIdeograph8593',
u'\u6212': 'CjkUnifiedIdeograph6212',
u'\u4695': 'CjkUnifiedIdeograph4695',
u'\u2714': 'HeavyCheckMark',
u'\u7b97': 'CjkUnifiedIdeograph7b97',
u'\u3c99': 'CjkUnifiedIdeograph3c99',
u'\ud41e': 'HangulSyllablePwaj',
u'\u8920': 'CjkUnifiedIdeograph8920',
u'\ueda3': 'UnicodeCharacter0xeda3',
u'\u4a22': 'CjkUnifiedIdeograph4a22',
u'\uaea5': 'HangulSyllableGgyaenj',
u'\u0f24': 'TibetanDigitFour',
u'\u63a7': 'CjkUnifiedIdeograph63a7',
u'\u24a9': 'ParenthesizedLatinSmallLetterN',
u'\uf130': 'UnicodeCharacter0xf130',
u'\ud5b3': 'HangulSyllableHyaed',
u'\ub232': 'HangulSyllableNweobs',
u'\u96b5': 'CjkUnifiedIdeograph96b5',
u'\u7734': 'CjkUnifiedIdeograph7734',
u'\u4bb7': 'CjkUnifiedIdeograph4bb7',
u'\u2836': 'BraillePatternDots2356',
u'\u0cb9': 'KannadaLetterHa',
u'\ud940': 'UnicodeCharacter0xd940',
u'\u9a42': 'CjkUnifiedIdeograph9a42',
u'\ufec5': 'ArabicLetterZahIsolatedForm',
u'\u5f44': 'CjkUnifiedIdeograph5f44',
u'\ub3c7': 'HangulSyllableDogs',
u'\u1046': 'MyanmarDigitSix',
u'\u74c9': 'CjkUnifiedIdeograph74c9',
u'\u29cb': 'TriangleWithUnderbar',
u'\uc150': 'HangulSyllableSek',
u'\u8252': 'CjkUnifiedIdeograph8252',
u'\ue6d5': 'UnicodeCharacter0xe6d5',
u'\u4754': 'CjkUnifiedIdeograph4754',
u'\u9bd7': 'CjkUnifiedIdeograph9bd7',
u'\u7856': 'CjkUnifiedIdeograph7856',
u'\u5cd9': 'CjkUnifiedIdeograph5cd9',
u'\u3d58': 'CjkUnifiedIdeograph3d58',
u'\u11db': 'HangulJongseongMieumrieul',
u'\uea62': 'UnicodeCharacter0xea62',
u'\ucee5': 'HangulSyllableKeog',
u'\uaf64': 'HangulSyllableGgwae',
u'\u83e7': 'CjkUnifiedIdeograph83e7',
u'\u6066': 'CjkUnifiedIdeograph6066',
u'\u44e9': 'CjkUnifiedIdeograph44e9',
u'\u2568': 'BoxDrawingsUpDoubleAndHorizontalSingle',
u'\u79eb': 'CjkUnifiedIdeograph79eb',
u'\u3aed': 'CjkUnifiedIdeograph3aed',
u'\ud272': 'HangulSyllableTwelp',
u'\u9774': 'CjkUnifiedIdeograph9774',
u'\uebf7': 'UnicodeCharacter0xebf7',
u'\u4876': 'CjkUnifiedIdeograph4876',
u'\uacf9': 'HangulSyllableGot',
u'\u0d78': 'UnicodeCharacter0xd78',
u'\u61fb': 'CjkUnifiedIdeograph61fb',
u'\u22fd': 'ContainsWithOverbar',
u'\u1801': 'MongolianEllipsis',
u'\uff84': 'HalfwidthKatakanaLetterTo',
u'\ub086': 'HangulSyllableGgilm',
u'\u7588': 'CjkUnifiedIdeograph7588',
u'\uc90b': 'HangulSyllableJulh',
u'\u368a': 'CjkUnifiedIdeograph368a',
u'\u8a0d': 'CjkUnifiedIdeograph8a0d',
u'\u4f0f': 'CjkUnifiedIdeograph4f0f',
u'\ue794': 'UnicodeCharacter0xe794',
u'\u9896': 'CjkUnifiedIdeograph9896',
u'\u5d98': 'CjkUnifiedIdeograph5d98',
u'\u1e9a': 'LatinSmallLetterAWithRightHalfRing',
u'\uf21d': 'UnicodeCharacter0xf21d',
u'\ub71f': 'HangulSyllableDdyus',
u'\u6821': 'CjkUnifiedIdeograph6821',
u'\u2d23': 'GeorgianSmallLetterWe',
u'\ucfa4': 'HangulSyllableKwaek',
u'\u80a6': 'CjkUnifiedIdeograph80a6',
u'\u45a8': 'CjkUnifiedIdeograph45a8',
u'\u06aa': 'ArabicLetterSwashKaf',
u'\uda2d': 'UnicodeCharacter0xda2d',
u'\u3bac': 'CjkUnifiedIdeograph3bac',
u'\u9f2f': 'CjkUnifiedIdeograph9f2f',
u'\u5031': 'CjkUnifiedIdeograph5031',
u'\u1533': 'CanadianSyllabicsYwii',
u'\ue8b6': 'UnicodeCharacter0xe8b6',
u'\uadb8': 'HangulSyllableGwess',
u'\u6eba': 'CjkUnifiedIdeograph6eba',
u'\uc23d': 'HangulSyllableSweolg',
u'\u23bc': 'HorizontalScanLine7',
u'\u873f': 'CjkUnifiedIdeograph873f',
u'\ub841': 'HangulSyllableRyeg',
u'\u7d43': 'CjkUnifiedIdeograph7d43',
u'\u3e45': 'CjkUnifiedIdeograph3e45',
u'\ud0c6': 'HangulSyllableTanh',
u'\u95c8': 'CjkUnifiedIdeograph95c8',
u'\u56ca': 'CjkUnifiedIdeograph56ca',
u'\u0bcc': 'TamilVowelSignAu',
u'\uef4f': 'UnicodeCharacter0xef4f',
u'\ua051': 'YiSyllablePyx',
u'\u6553': 'CjkUnifiedIdeograph6553',
u'\u2655': 'WhiteChessQueen',
u'\ufdd8': 'UnicodeCharacter0xfdd8',
u'\ubeda': 'HangulSyllableBbeolm',
u'\u73dc': 'CjkUnifiedIdeograph73dc',
u'\ud75f': 'HangulSyllableHeulh',
u'\u34de': 'CjkUnifiedIdeograph34de',
u'\u8861': 'CjkUnifiedIdeograph8861',
u'\u4d63': 'CjkUnifiedIdeograph4d63',
u'\u0e65': 'UnicodeCharacter0xe65',
u'\ue5e8': 'UnicodeCharacter0xe5e8',
u'\ua6ea': 'UnicodeCharacter0xa6ea',
u'\u5bec': 'CjkUnifiedIdeograph5bec',
u'\u1cee': 'UnicodeCharacter0x1cee',
u'\uf071': 'UnicodeCharacter0xf071',
u'\ub573': 'HangulSyllableDdyalb',
u'\u7675': 'CjkUnifiedIdeograph7675',
u'\u2b77': 'UnicodeCharacter0x2b77',
u'\ucdf8': 'HangulSyllableCwim',
u'\u8efa': 'CjkUnifiedIdeograph8efa',
u'\u43fc': 'CjkUnifiedIdeograph43fc',
u'\u04fe': 'CyrillicCapitalLetterHaWithStroke',
u'\u3a00': 'CjkUnifiedIdeograph3a00',
u'\u1e83': 'LatinSmallLetterWWithAcute',
u'\uf902': 'CjkCompatibilityIdeographf902',
u'\udd85': 'UnicodeCharacter0xdd85',
u'\u7306': 'CjkUnifiedIdeograph7306',
u'\u5789': 'CjkUnifiedIdeograph5789',
u'\ua80c': 'SylotiNagriLetterCo',
u'\u8c8f': 'CjkUnifiedIdeograph8c8f',
u'\u2210': 'NaryCoproduct',
u'\u0693': 'ArabicLetterRehWithRing',
u'\ue112': 'UnicodeCharacter0xe112',
u'\uc595': 'HangulSyllableYat',
u'\u5b16': 'CjkUnifiedIdeograph5b16',
u'\ubf99': 'HangulSyllableBboenj',
u'\u319d': 'IdeographicAnnotationHeavenMark',
u'\u901c': 'CjkUnifiedIdeograph901c',
u'\uf49f': 'UnicodeCharacter0xf49f',
u'\u0a20': 'GurmukhiLetterTtha',
u'\u6ea3': 'CjkUnifiedIdeograph6ea3',
u'\uc922': 'HangulSyllableJweolm',
u'\u4326': 'CjkUnifiedIdeograph4326',
u'\ua7a9': 'UnicodeCharacter0xa7a9',
u'\u19ad': 'UnicodeCharacter0x19ad',
u'\uf82c': 'UnicodeCharacter0xf82c',
u'\udcaf': 'UnicodeCharacter0xdcaf',
u'\u7230': 'CjkUnifiedIdeograph7230',
u'\u56b3': 'CjkUnifiedIdeograph56b3',
u'\uab36': 'UnicodeCharacter0xab36',
u'\u8fb9': 'CjkUnifiedIdeograph8fb9',
u'\u2d3a': 'TifinaghLetterYaddh',
u'\u01bd': 'LatinSmallLetterToneFive',
u'\ue03c': 'UnicodeCharacter0xe03c',
u'\uc4bf': 'HangulSyllableSswid',
u'\u5a40': 'CjkUnifiedIdeograph5a40',
u'\ubec3': 'HangulSyllableBbyaelh',
u'\u30c7': 'KatakanaLetterDe',
u'\u9346': 'CjkUnifiedIdeograph9346',
u'\uf7c9': 'UnicodeCharacter0xf7c9',
u'\u154a': 'CanadianSyllabicsWestcreeLo',
u'\u69cd': 'CjkUnifiedIdeograph69cd',
u'\uc84c': 'HangulSyllableJyeoss',
u'\u4250': 'CjkUnifiedIdeograph4250',
u'\ua6d3': 'UnicodeCharacter0xa6d3',
u'\u18d7': 'UnicodeCharacter0x18d7',
u'\ufb56': 'ArabicLetterPehIsolatedForm',
u'\udfd9': 'UnicodeCharacter0xdfd9',
u'\u7d5a': 'CjkUnifiedIdeograph7d5a',
u'\u51dd': 'CjkUnifiedIdeograph51dd',
u'\uaa60': 'UnicodeCharacter0xaa60',
u'\u8ee3': 'CjkUnifiedIdeograph8ee3',
u'\u2c64': 'LatinCapitalLetterRWithTail',
u'\xe7': 'LatinSmallLetterCWithCedilla',
u'\ue366': 'UnicodeCharacter0xe366',
u'\uc7e9': 'HangulSyllableJyaenj',
u'\u656a': 'CjkUnifiedIdeograph656a',
u'\ub9ed': 'HangulSyllableMaelg',
u'\u33f1': 'IdeographicTelegraphSymbolForDayEighteen',
u'\u9270': 'CjkUnifiedIdeograph9270',
u'\uf6f3': 'UnicodeCharacter0xf6f3',
u'\u1474': 'CanadianSyllabicsKwe',
u'\u68f7': 'CjkUnifiedIdeograph68f7',
u'\ucb76': 'HangulSyllableJjweobs',
u'\u4d7a': 'CjkUnifiedIdeograph4d7a',
u'\ua1fd': 'YiSyllableKie',
u'\u9b01': 'CjkUnifiedIdeograph9b01',
u'\ufa80': 'CjkCompatibilityIdeographfa80',
u'\u1d05': 'LatinLetterSmallCapitalD',
u'\u7c84': 'CjkUnifiedIdeograph7c84',
u'\ud007': 'HangulSyllableKweolb',
u'\u4a0b': 'CjkUnifiedIdeograph4a0b',
u'\ub58a': 'HangulSyllableDdyaenh',
u'\u2f8e': 'KangxiRadicalBlood',
u'\u8311': 'CjkUnifiedIdeograph8311',
u'\ue290': 'UnicodeCharacter0xe290',
u'\u0515': 'CyrillicSmallLetterLha',
u'\u6494': 'CjkUnifiedIdeograph6494',
u'\ub21b': 'HangulSyllableNuc',
u'\u9d9a': 'CjkUnifiedIdeograph9d9a',
u'\u341f': 'CjkUnifiedIdeograph341f',
u'\u179e': 'KhmerLetterSso',
u'\ueb21': 'UnicodeCharacter0xeb21',
u'\ucaa0': 'HangulSyllableJjye',
u'\u6d25': 'CjkUnifiedIdeograph6d25',
u'\u4ca4': 'CjkUnifiedIdeograph4ca4',
u'\u9a2b': 'CjkUnifiedIdeograph9a2b',
u'\u85aa': 'CjkUnifiedIdeograph85aa',
u'\u1c2f': 'LepchaConsonantSignL',
u'\u7fae': 'CjkUnifiedIdeograph7fae',
u'\ud331': 'HangulSyllablePaelg',
u'\u5535': 'CjkUnifiedIdeograph5535',
u'\ub4b4': 'HangulSyllableDwim',
u'\u2eb8': 'CjkRadicalEwe',
u'\u823b': 'CjkUnifiedIdeograph823b',
u'\uedba': 'UnicodeCharacter0xedba',
u'\u043f': 'CyrillicSmallLetterPe',
u'\u67be': 'CjkUnifiedIdeograph67be',
u'\ubd45': 'HangulSyllableBwaet',
u'\u9cc4': 'CjkUnifiedIdeograph9cc4',
u'\u3749': 'CjkUnifiedIdeograph3749',
u'\u16c8': 'RunicLetterPerthoPeorthP',
u'\uea4b': 'UnicodeCharacter0xea4b',
u'\ud5ca': 'HangulSyllableHeogg',
u'\u6c4f': 'CjkUnifiedIdeograph6c4f',
u'\u4fce': 'CjkUnifiedIdeograph4fce',
u'\ua555': 'VaiSyllableKpan',
u'\u84d4': 'CjkUnifiedIdeograph84d4',
u'\u1f59': 'GreekCapitalLetterUpsilonWithDasia',
u'\u7ed8': 'CjkUnifiedIdeograph7ed8',
u'\ud25b': 'HangulSyllableTweos',
u'\u545f': 'CjkUnifiedIdeograph545f',
u'\ub7de': 'HangulSyllableRyaelp',
u'\u29e2': 'ShuffleProduct',
u'\u8d65': 'CjkUnifiedIdeograph8d65',
u'\uece4': 'UnicodeCharacter0xece4',
u'\u0769': 'ArabicLetterNoonWithSmallV',
u'\u66e8': 'CjkUnifiedIdeograph66e8',
u'\ubc6f': 'HangulSyllableByaed',
u'\u9fee': 'UnicodeCharacter0x9fee',
u'\u3673': 'CjkUnifiedIdeograph3673',
u'\u11f2': 'HangulJongseongYesieungpansios',
u'\uf575': 'UnicodeCharacter0xf575',
u'\ud4f4': 'HangulSyllablePyuls',
u'\u6f79': 'CjkUnifiedIdeograph6f79',
u'\u4ef8': 'CjkUnifiedIdeograph4ef8',
u'\ua47f': 'YiSyllableYo',
u'\u87fe': 'CjkUnifiedIdeograph87fe',
u'\uf206': 'UnicodeCharacter0xf206',
u'\ud689': 'HangulSyllableHwaet',
u'\ub708': 'HangulSyllableDdwik',
u'\u8b8b': 'CjkUnifiedIdeograph8b8b',
u'\u740a': 'CjkUnifiedIdeograph740a',
u'\u488d': 'CjkUnifiedIdeograph488d',
u'\u290c': 'LeftwardsDoubleDashArrow',
u'\u0d8f': 'SinhalaLetterIluyanna',
u'\uda16': 'UnicodeCharacter0xda16',
u'\u9f18': 'CjkUnifiedIdeograph9f18',
u'\uf39b': 'UnicodeCharacter0xf39b',
u'\u5c1a': 'CjkUnifiedIdeograph5c1a',
u'\ub09d': 'HangulSyllableNanj',
u'\u111c': 'HangulChoseongMieumpieup',
u'\u759f': 'CjkUnifiedIdeograph759f',
u'\u2aa1': 'DoubleNestedLessthan',
u'\uc226': 'HangulSyllableSulp',
u'\u8728': 'CjkUnifiedIdeograph8728',
u'\udbab': 'UnicodeCharacter0xdbab',
u'\u442a': 'CjkUnifiedIdeograph442a',
u'\u98ad': 'CjkUnifiedIdeograph98ad',
u'\u792c': 'CjkUnifiedIdeograph792c',
u'\u5daf': 'CjkUnifiedIdeograph5daf',
u'\u3e2e': 'CjkUnifiedIdeograph3e2e',
u'\u12b1': 'UnicodeCharacter0x12b1',
u'\uef38': 'UnicodeCharacter0xef38',
u'\uc3bb': 'HangulSyllableSsyeoh',
u'\uac3a': 'HangulSyllableGyagg',
u'\u80bd': 'CjkUnifiedIdeograph80bd',
u'\u613c': 'CjkUnifiedIdeograph613c',
u'\u45bf': 'CjkUnifiedIdeograph45bf',
u'\u263e': 'LastQuarterMoon',
u'\u7ac1': 'CjkUnifiedIdeograph7ac1',
u'\u3fc3': 'CjkUnifiedIdeograph3fc3',
u'\ud748': 'HangulSyllableHyuss',
u'\u944a': 'CjkUnifiedIdeograph944a',
u'\ue8cd': 'UnicodeCharacter0xe8cd',
u'\u494c': 'CjkUnifiedIdeograph494c',
u'\uadcf': 'HangulSyllableGwilh',
u'\u0e4e': 'ThaiCharacterYamakkan',
u'\u62d1': 'CjkUnifiedIdeograph62d1',
u'\u27d3': 'LowerRightCornerWithDot',
u'\ufc5a': 'ArabicLigatureYehWithYehIsolatedForm',
u'\ud0dd': 'HangulSyllableTaeg',
u'\ub15c': 'HangulSyllableNye',
u'\u95df': 'CjkUnifiedIdeograph95df',
u'\u765e': 'CjkUnifiedIdeograph765e',
u'\u4ae1': 'CjkUnifiedIdeograph4ae1',
u'\u2b60': 'UnicodeCharacter0x2b60',
u'\u0fe3': 'UnicodeCharacter0xfe3',
u'\ue46a': 'UnicodeCharacter0xe46a',
u'\u996c': 'CjkUnifiedIdeograph996c',
u'\ufdef': 'UnicodeCharacter0xfdef',
u'\u5e6e': 'CjkUnifiedIdeograph5e6e',
u'\ub2f1': 'HangulSyllableDalt',
u'\u1370': 'EthiopicDigitEight',
u'\u77f3': 'CjkUnifiedIdeograph77f3',
u'\u34f5': 'CjkUnifiedIdeograph34f5',
u'\ucc7a': 'HangulSyllableCyap',
u'\u817c': 'CjkUnifiedIdeograph817c',
u'\ue5ff': 'UnicodeCharacter0xe5ff',
u'\u467e': 'CjkUnifiedIdeograph467e',
u'\u7b80': 'CjkUnifiedIdeograph7b80',
u'\udf03': 'UnicodeCharacter0xdf03',
u'\u3882': 'CjkUnifiedIdeograph3882',
u'\u9c05': 'CjkUnifiedIdeograph9c05',
u'\u5107': 'CjkUnifiedIdeograph5107',
u'\u1609': 'CanadianSyllabicsCarrierMo',
u'\ue98c': 'UnicodeCharacter0xe98c',
u'\uae8e': 'HangulSyllableGgyalm',
u'\u6390': 'CjkUnifiedIdeograph6390',
u'\uc713': 'HangulSyllableWilh',
u'\u2092': 'LatinSubscriptSmallLetterO',
u'\u8415': 'CjkUnifiedIdeograph8415',
u'\ub917': 'HangulSyllableRweos',
u'\u7e19': 'CjkUnifiedIdeograph7e19',
u'\u331b': 'SquareKuroone',
u'\ud19c': 'HangulSyllableTyek',
u'\u969e': 'CjkUnifiedIdeograph969e',
u'\u4ba0': 'CjkUnifiedIdeograph4ba0',
u'\u08a2': 'UnicodeCharacter0x8a2',
u'\uec25': 'UnicodeCharacter0xec25',
u'\ua127': 'YiSyllableTuo',
u'\u6629': 'CjkUnifiedIdeograph6629',
u'\u1b2b': 'BalineseLetterMa',
u'\ufeae': 'ArabicLetterRehFinalForm',
u'\ub3b0': 'HangulSyllableDyel',
u'\u70b2': 'CjkUnifiedIdeograph70b2',
u'\ud435': 'HangulSyllablePwaeb',
u'\u35b4': 'CjkUnifiedIdeograph35b4',
u'\u8937': 'CjkUnifiedIdeograph8937',
u'\u4e39': 'CjkUnifiedIdeograph4e39',
u'\u033b': 'CombiningSquareBelow',
u'\ue6be': 'UnicodeCharacter0xe6be',
u'\u9bc0': 'CjkUnifiedIdeograph9bc0',
u'\u58c2': 'CjkUnifiedIdeograph58c2',
u'\u1dc4': 'CombiningMacronacute',
u'\uf147': 'UnicodeCharacter0xf147',
u'\ub649': 'HangulSyllableDdwaeg',
u'\u6b4b': 'CjkUnifiedIdeograph6b4b',
u'\u284d': 'BraillePatternDots1347',
u'\ucece': 'HangulSyllableKyaenh',
u'\u83d0': 'CjkUnifiedIdeograph83d0',
u'\u40d2': 'CjkUnifiedIdeograph40d2',
u'\u05d4': 'HebrewLetterHe',
u'\ud957': 'UnicodeCharacter0xd957',
u'\u3ad6': 'CjkUnifiedIdeograph3ad6',
u'\u9e59': 'CjkUnifiedIdeograph9e59',
u'\u535b': 'CjkUnifiedIdeograph535b',
u'\u105d': 'MyanmarLetterMonBbe',
u'\uebe0': 'UnicodeCharacter0xebe0',
u'\ua8e2': 'UnicodeCharacter0xa8e2',
u'\u6de4': 'CjkUnifiedIdeograph6de4',
u'\uc167': 'HangulSyllableSyeos',
u'\u22e6': 'LessthanButNotEquivalentTo',
u'\u8669': 'CjkUnifiedIdeograph8669',
u'\ubb6b': 'HangulSyllableMweoh',
u'\u786d': 'CjkUnifiedIdeograph786d',
u'\u3d6f': 'CjkUnifiedIdeograph3d6f',
u'\ud3f0': 'HangulSyllablePon',
u'\u90f2': 'CjkUnifiedIdeograph90f2',
u'\u55f4': 'CjkUnifiedIdeograph55f4',
u'\u0af6': 'UnicodeCharacter0xaf6',
u'\uee79': 'UnicodeCharacter0xee79',
u'\ua37b': 'YiSyllableChyr',
u'\u607d': 'CjkUnifiedIdeograph607d',
u'\u257f': 'BoxDrawingsHeavyUpAndLightDown',
u'\u5d81': 'CjkUnifiedIdeograph5d81',
u'\uba04': 'HangulSyllableMyan',
u'\u9687': 'CjkUnifiedIdeograph9687',
u'\u3008': 'LeftAngleBracket',
u'\u0c8b': 'KannadaLetterVocalicR',
u'\uf30a': 'UnicodeCharacter0xf30a',
u'\ucf8d': 'HangulSyllableKwaeg',
u'\u690e': 'CjkUnifiedIdeograph690e',
u'\u4591': 'CjkUnifiedIdeograph4591',
u'\ua214': 'YiSyllableGgit',
u'\ufe97': 'ArabicLetterTehInitialForm',
u'\u1818': 'MongolianDigitEight',
u'\u749b': 'CjkUnifiedIdeograph749b',
u'\udb1a': 'UnicodeCharacter0xdb1a',
u'\u511e': 'CjkUnifiedIdeograph511e',
u'\uada1': 'HangulSyllableGweot',
u'\u2ba5': 'UnicodeCharacter0x2ba5',
u'\u8a24': 'CjkUnifiedIdeograph8a24',
u'\ue6a7': 'UnicodeCharacter0xe6a7',
u'\u5cab': 'CjkUnifiedIdeograph5cab',
u'\uc32a': 'HangulSyllableSsaej',
u'\ub92e': 'HangulSyllableRwelp',
u'\u95b1': 'CjkUnifiedIdeograph95b1',
u'\u3732': 'CjkUnifiedIdeograph3732',
u'\u13b5': 'CherokeeLetterLi',
u'\uf234': 'UnicodeCharacter0xf234',
u'\uceb7': 'HangulSyllableKyalb',
u'\u6838': 'CjkUnifiedIdeograph6838',
u'\u44bb': 'CjkUnifiedIdeograph44bb',
u'\ua13e': 'YiSyllableDdax',
u'\ufdc1': 'ArabicLigatureFehWithMeemWithYehFinalForm',
u'\u1f42': 'GreekSmallLetterOmicronWithPsiliAndVaria',
u'\u7bc5': 'CjkUnifiedIdeograph7bc5',
u'\uda44': 'UnicodeCharacter0xda44',
u'\u5048': 'CjkUnifiedIdeograph5048',
u'\uaccb': 'HangulSyllableGyed',
u'\u2acf': 'ClosedSubset',
u'\u894e': 'CjkUnifiedIdeograph894e',
u'\ue5d1': 'UnicodeCharacter0xe5d1',
u'\u0752': 'ArabicLetterBehWithThreeDotsPointingUpwardsBelow',
u'\u63d5': 'CjkUnifiedIdeograph63d5',
u'\uc254': 'HangulSyllableSwen',
u'\ub858': 'HangulSyllableRyek',
u'\u94db': 'CjkUnifiedIdeograph94db',
u'\u365c': 'CjkUnifiedIdeograph365c',
u'\u12df': 'EthiopicSyllableZwa',
u'\uf15e': 'UnicodeCharacter0xf15e',
u'\ucde1': 'HangulSyllableCweng',
u'\u6f62': 'CjkUnifiedIdeograph6f62',
u'\u4be5': 'CjkUnifiedIdeograph4be5',
u'\ua068': 'YiSyllableBbop',
u'\ufceb': 'ArabicLigatureKafWithLamMedialForm',
u'\u1e6c': 'LatinCapitalLetterTWithDotBelow',
u'\u7aef': 'CjkUnifiedIdeograph7aef',
u'\ud96e': 'UnicodeCharacter0xd96e',
u'\u5772': 'CjkUnifiedIdeograph5772',
u'\ub3f5': 'HangulSyllableDwang',
u'\u29f9': 'BigReverseSolidus',
u'\u8878': 'CjkUnifiedIdeograph8878',
u'\ue4fb': 'UnicodeCharacter0xe4fb',
u'\u067c': 'ArabicLetterTehWithRing',
u'\u62ff': 'CjkUnifiedIdeograph62ff',
u'\uc17e': 'HangulSyllableSyelp',
u'\u5803': 'CjkUnifiedIdeograph5803',
u'\ubf82': 'HangulSyllableBbwaelm',
u'\u3586': 'CjkUnifiedIdeograph3586',
u'\u9109': 'CjkUnifiedIdeograph9109',
u'\uf088': 'UnicodeCharacter0xf088',
u'\u0f0d': 'TibetanMarkShad',
u'\u6e8c': 'CjkUnifiedIdeograph6e8c',
u'\uca0f': 'HangulSyllableJjaec',
u'\u4013': 'CjkUnifiedIdeograph4013',
u'\ua792': 'UnicodeCharacter0xa792',
u'\u3e17': 'CjkUnifiedIdeograph3e17',
u'\u1d96': 'LatinSmallLetterIWithRetroflexHook',
u'\uf919': 'CjkCompatibilityIdeographf919',
u'\ud898': 'UnicodeCharacter0xd898',
u'\u771d': 'CjkUnifiedIdeograph771d',
u'\u569c': 'CjkUnifiedIdeograph569c',
u'\ua823': 'SylotiNagriVowelSignA',
u'\u8fa2': 'CjkUnifiedIdeograph8fa2',
u'\u2627': 'ChiRho',
u'\u05a6': 'HebrewAccentMerkhaKefula',
u'\ue129': 'UnicodeCharacter0xe129',
u'\uc0a8': 'HangulSyllableBbik',
u'\u5f2d': 'CjkUnifiedIdeograph5f2d',
u'\ubeac': 'HangulSyllableBbyass',
u'\u34b0': 'CjkUnifiedIdeograph34b0',
u'\u9033': 'CjkUnifiedIdeograph9033',
u'\uf7b2': 'UnicodeCharacter0xf7b2',
u'\u0e37': 'ThaiCharacterSaraUee',
u'\u6db6': 'CjkUnifiedIdeograph6db6',
u'\uc939': 'HangulSyllableJwenj',
u'\u473d': 'CjkUnifiedIdeograph473d',
u'\ua6bc': 'UnicodeCharacter0xa6bc',
u'\u3d41': 'CjkUnifiedIdeograph3d41',
u'\u1cc0': 'UnicodeCharacter0x1cc0',
u'\uf843': 'UnicodeCharacter0xf843',
u'\udfc2': 'UnicodeCharacter0xdfc2',
u'\u7647': 'CjkUnifiedIdeograph7647',
u'\u55c6': 'CjkUnifiedIdeograph55c6',
u'\uaf4d': 'HangulSyllableGgwanj',
u'\u8ecc': 'CjkUnifiedIdeograph8ecc',
u'\u2551': 'BoxDrawingsDoubleVertical',
u'\u04d0': 'CyrillicCapitalLetterAWithBreve',
u'\ue053': 'UnicodeCharacter0xe053',
u'\uc7d2': 'HangulSyllableJyalm',
u'\u5e57': 'CjkUnifiedIdeograph5e57',
u'\ubdd6': 'HangulSyllableBwigg',
u'\u3bda': 'CjkUnifiedIdeograph3bda',
u'\u975d': 'CjkUnifiedIdeograph975d',
u'\uf6dc': 'UnicodeCharacter0xf6dc',
u'\u0d61': 'MalayalamLetterVocalicLl',
u'\u6ce0': 'CjkUnifiedIdeograph6ce0',
u'\uc863': 'HangulSyllableJyelh',
u'\u4667': 'CjkUnifiedIdeograph4667',
u'\ua5e6': 'VaiSyllableWe',
u'\u3c6b': 'CjkUnifiedIdeograph3c6b',
u'\u23ea': 'UnicodeCharacter0x23ea',
u'\uff6d': 'HalfwidthKatakanaLetterSmallYu',
u'\udeec': 'UnicodeCharacter0xdeec',
u'\u7571': 'CjkUnifiedIdeograph7571',
u'\u54f0': 'CjkUnifiedIdeograph54f0',
u'\uae77': 'HangulSyllableGgaelh',
u'\u8df6': 'CjkUnifiedIdeograph8df6',
u'\u247b': 'ParenthesizedDigitEight',
u'\u0bfa': 'TamilNumberSign',
u'\ue77d': 'UnicodeCharacter0xe77d',
u'\uc6fc': 'HangulSyllableWess',
u'\udc81': 'UnicodeCharacter0xdc81',
u'\ubd00': 'HangulSyllableBols',
u'\u9983': 'CjkUnifiedIdeograph9983',
u'\u7e02': 'CjkUnifiedIdeograph7e02',
u'\u5a85': 'CjkUnifiedIdeograph5a85',
u'\u3b04': 'CjkUnifiedIdeograph3b04',
u'\u1787': 'KhmerLetterCo',
u'\ue80e': 'UnicodeCharacter0xe80e',
u'\uc491': 'HangulSyllableSsweob',
u'\ua510': 'VaiSyllableDhee',
u'\u8193': 'CjkUnifiedIdeograph8193',
u'\u6612': 'CjkUnifiedIdeograph6612',
u'\u4295': 'CjkUnifiedIdeograph4295',
u'\u2314': 'Sector',
u'\u7f97': 'CjkUnifiedIdeograph7f97',
u'\u3899': 'CjkUnifiedIdeograph3899',
u'\ud01e': 'HangulSyllableKwenh',
u'\u8d20': 'CjkUnifiedIdeograph8d20',
u'\ue9a3': 'UnicodeCharacter0xe9a3',
u'\u4e22': 'CjkUnifiedIdeograph4e22',
u'\uaaa5': 'UnicodeCharacter0xaaa5',
u'\u0b24': 'OriyaLetterTa',
u'\u67a7': 'CjkUnifiedIdeograph67a7',
u'\u20a9': 'WonSign',
u'\uf530': 'UnicodeCharacter0xf530',
u'\ud1b3': 'HangulSyllableTos',
u'\ub632': 'HangulSyllableDdwanh',
u'\u92b5': 'CjkUnifiedIdeograph92b5',
u'\u7334': 'CjkUnifiedIdeograph7334',
u'\u4fb7': 'CjkUnifiedIdeograph4fb7',
u'\u2c36': 'GlagoliticSmallLetterZhivete',
u'\u08b9': 'UnicodeCharacter0x8b9',
u'\udd40': 'UnicodeCharacter0xdd40',
u'\u9e42': 'CjkUnifiedIdeograph9e42',
u'\ufac5': 'CjkCompatibilityIdeographfac5',
u'\u5b44': 'CjkUnifiedIdeograph5b44',
u'\ub7c7': 'HangulSyllableRyas',
u'\u1446': 'CanadianSyllabicsPwaa',
u'\u70c9': 'CjkUnifiedIdeograph70c9',
u'\u2dcb': 'EthiopicSyllableKyaa',
u'\uc550': 'HangulSyllableAls',
u'\u8652': 'CjkUnifiedIdeograph8652',
u'\ue2d5': 'UnicodeCharacter0xe2d5',
u'\u4354': 'CjkUnifiedIdeograph4354',
u'\u9fd7': 'UnicodeCharacter0x9fd7',
u'\u7c56': 'CjkUnifiedIdeograph7c56',
u'\u58d9': 'CjkUnifiedIdeograph58d9',
u'\u3958': 'CjkUnifiedIdeograph3958',
u'\u15db': 'CanadianSyllabicsCarrierHwa',
u'\uee62': 'UnicodeCharacter0xee62',
u'\ucae5': 'HangulSyllableJjwalt',
u'\uab64': 'UnicodeCharacter0xab64',
u'\u87e7': 'CjkUnifiedIdeograph87e7',
u'\u6466': 'CjkUnifiedIdeograph6466',
u'\u40e9': 'CjkUnifiedIdeograph40e9',
u'\u2168': 'RomanNumeralNine',
u'\u7deb': 'CjkUnifiedIdeograph7deb',
u'\u3eed': 'CjkUnifiedIdeograph3eed',
u'\ud672': 'HangulSyllableHwaegg',
u'\u9374': 'CjkUnifiedIdeograph9374',
u'\ueff7': 'UnicodeCharacter0xeff7',
u'\u4c76': 'CjkUnifiedIdeograph4c76',
u'\ua8f9': 'UnicodeCharacter0xa8f9',
u'\u0978': 'UnicodeCharacter0x978',
u'\u65fb': 'CjkUnifiedIdeograph65fb',
u'\u26fd': 'UnicodeCharacter0x26fd',
u'\u1c01': 'LepchaLetterKla',
u'\ufb84': 'ArabicLetterDahalIsolatedForm',
u'\ub486': 'HangulSyllableDweop',
u'\u7188': 'CjkUnifiedIdeograph7188',
u'\ucd0b': 'HangulSyllableCogs',
u'\u328a': 'CircledIdeographMoon',
u'\u8e0d': 'CjkUnifiedIdeograph8e0d',
u'\u4b0f': 'CjkUnifiedIdeograph4b0f',
u'\u0411': 'CyrillicCapitalLetterBe',
u'\ue394': 'UnicodeCharacter0xe394',
u'\u9c96': 'CjkUnifiedIdeograph9c96',
u'\u5998': 'CjkUnifiedIdeograph5998',
u'\u1a9a': 'UnicodeCharacter0x1a9a',
u'\uf61d': 'UnicodeCharacter0xf61d',
u'\ub31f': 'HangulSyllableDyags',
u'\u6c21': 'CjkUnifiedIdeograph6c21',
u'\u2923': 'NorthWestArrowWithHook',
u'\ucba4': 'HangulSyllableJjwil',
u'\u84a6': 'CjkUnifiedIdeograph84a6',
u'\u41a8': 'CjkUnifiedIdeograph41a8',
u'\u02aa': 'LatinSmallLetterLsDigraph',
u'\ude2d': 'UnicodeCharacter0xde2d',
u'\u3fac': 'CjkUnifiedIdeograph3fac',
u'\u9b2f': 'CjkUnifiedIdeograph9b2f',
u'\u5431': 'CjkUnifiedIdeograph5431',
u'\u1133': 'HangulChoseongSiospieupkiyeok',
u'\uecb6': 'UnicodeCharacter0xecb6',
u'\ua9b8': 'UnicodeCharacter0xa9b8',
u'\u6aba': 'CjkUnifiedIdeograph6aba',
u'\uc63d': 'HangulSyllableOt',
u'\u27bc': 'WedgetailedRightwardsArrow',
u'\u833f': 'CjkUnifiedIdeograph833f',
u'\ubc41': 'HangulSyllableBaeb',
u'\u7943': 'CjkUnifiedIdeograph7943',
u'\u3a45': 'CjkUnifiedIdeograph3a45',
u'\ud4c6': 'HangulSyllablePwej',
u'\u91c8': 'CjkUnifiedIdeograph91c8',
u'\u52ca': 'CjkUnifiedIdeograph52ca',
u'\u0fcc': 'TibetanSymbolNorBuBzhiKhyil',
u'\ueb4f': 'UnicodeCharacter0xeb4f',
u'\ua451': 'YiSyllableNyiep',
u'\u6153': 'CjkUnifiedIdeograph6153',
u'\u2255': 'EqualsColon',
u'\uf9d8': 'CjkCompatibilityIdeographf9d8',
u'\ubada': 'HangulSyllableMwaj',
u'\u77dc': 'CjkUnifiedIdeograph77dc',
u'\ud35f': 'HangulSyllablePyah',
u'\u30de': 'KatakanaLetterMa',
u'\u8c61': 'CjkUnifiedIdeograph8c61',
u'\u4963': 'CjkUnifiedIdeograph4963',
u'\u0a65': 'UnicodeCharacter0xa65',
u'\ue1e8': 'UnicodeCharacter0xe1e8',
u'\ua2ea': 'YiSyllableZzyp',
u'\u5fec': 'CjkUnifiedIdeograph5fec',
u'\u18ee': 'UnicodeCharacter0x18ee',
u'\uf471': 'UnicodeCharacter0xf471',
u'\ub173': 'HangulSyllableNyec',
u'\u7275': 'CjkUnifiedIdeograph7275',
u'\u2f77': 'KangxiRadicalSilk',
u'\uc9f8': 'HangulSyllableJjae',
u'\u8afa': 'CjkUnifiedIdeograph8afa',
u'\u47fc': 'CjkUnifiedIdeograph47fc',
u'\xfe': 'LatinSmallLetterThorn',
u'\u3e00': 'CjkUnifiedIdeograph3e00',
u'\u1a83': 'UnicodeCharacter0x1a83',
u'\ufd02': 'ArabicLigatureJeemWithYehIsolatedForm',
u'\ud985': 'UnicodeCharacter0xd985',
u'\u7706': 'CjkUnifiedIdeograph7706',
u'\u5389': 'CjkUnifiedIdeograph5389',
u'\uac0c': 'HangulSyllableGals',
u'\u888f': 'CjkUnifiedIdeograph888f',
u'\u2610': 'BallotBox',
u'\u0293': 'LatinSmallLetterEzhWithCurl',
u'\ue512': 'UnicodeCharacter0xe512',
u'\uc195': 'HangulSyllableSolg',
u'\u5f16': 'CjkUnifiedIdeograph5f16',
u'\ubb99': 'HangulSyllableMwib',
u'\u359d': 'CjkUnifiedIdeograph359d',
u'\u941c': 'CjkUnifiedIdeograph941c',
u'\uf09f': 'UnicodeCharacter0xf09f',
u'\u0e20': 'ThaiCharacterPhoSamphao',
u'\u6aa3': 'CjkUnifiedIdeograph6aa3',
u'\ucd22': 'HangulSyllableCop',
u'\u4726': 'CjkUnifiedIdeograph4726',
u'\ua3a9': 'YiSyllableNryrx',
u'\u1dad': 'ModifierLetterSmallTurnedMWithLongLeg',
u'\ufc2c': 'ArabicLigatureGhainWithMeemIsolatedForm',
u'\ud8af': 'UnicodeCharacter0xd8af',
u'\u7630': 'CjkUnifiedIdeograph7630',
u'\u52b3': 'CjkUnifiedIdeograph52b3',
u'\uaf36': 'HangulSyllableGgolm',
u'\u8bb9': 'CjkUnifiedIdeograph8bb9',
u'\u293a': 'TopArcAnticlockwiseArrow',
u'\u05bd': 'HebrewPointMeteg',
u'\ue43c': 'UnicodeCharacter0xe43c',
u'\uc0bf': 'HangulSyllableSas',
u'\u5e40': 'CjkUnifiedIdeograph5e40',
u'\ubac3': 'HangulSyllableMoh',
u'\u34c7': 'CjkUnifiedIdeograph34c7',
u'\u9746': 'CjkUnifiedIdeograph9746',
u'\uf3c9': 'UnicodeCharacter0xf3c9',
u'\u114a': 'HangulChoseongIeungthieuth',
u'\u6dcd': 'CjkUnifiedIdeograph6dcd',
u'\ucc4c': 'HangulSyllableCael',
u'\u4650': 'CjkUnifiedIdeograph4650',
u'\ua2d3': 'YiSyllableZzip',
u'\u1cd7': 'UnicodeCharacter0x1cd7',
u'\uff56': 'FullwidthLatinSmallLetterV',
u'\udbd9': 'UnicodeCharacter0xdbd9',
u'\u795a': 'CjkUnifiedIdeograph795a',
u'\u55dd': 'CjkUnifiedIdeograph55dd',
u'\uae60': 'HangulSyllableGgass',
u'\u8ae3': 'CjkUnifiedIdeograph8ae3',
u'\u2864': 'BraillePatternDots367',
u'\u04e7': 'CyrillicSmallLetterOWithDiaeresis',
u'\ue766': 'UnicodeCharacter0xe766',
u'\uc3e9': 'HangulSyllableSsob',
u'\u616a': 'CjkUnifiedIdeograph616a',
u'\ubded': 'HangulSyllableBwit',
u'\u37f1': 'CjkUnifiedIdeograph37f1',
u'\u9670': 'CjkUnifiedIdeograph9670',
u'\uf2f3': 'UnicodeCharacter0xf2f3',
u'\u1074': 'MyanmarVowelSignKayahEe',
u'\u6cf7': 'CjkUnifiedIdeograph6cf7',
u'\ucf76': 'HangulSyllableKwanh',
u'\u497a': 'CjkUnifiedIdeograph497a',
u'\ua5fd': 'VaiSyllableZe',
u'\u9f01': 'CjkUnifiedIdeograph9f01',
u'\ufe80': 'ArabicLetterHamzaIsolatedForm',
u'\u1905': 'LimbuLetterNga',
u'\u7884': 'CjkUnifiedIdeograph7884',
u'\ud407': 'HangulSyllablePoh',
u'\u4e0b': 'CjkUnifiedIdeograph4e0b',
u'\ub18a': 'HangulSyllableNobs',
u'\u2b8e': 'UnicodeCharacter0x2b8e',
u'\u8711': 'CjkUnifiedIdeograph8711',
u'\ue690': 'UnicodeCharacter0xe690',
u'\u0115': 'LatinSmallLetterEWithBreve',
u'\u6094': 'CjkUnifiedIdeograph6094',
u'\ub61b': 'HangulSyllableDdolb',
u'\u999a': 'CjkUnifiedIdeograph999a',
u'\u301f': 'LowDoublePrimeQuotationMark',
u'\u139e': 'UnicodeCharacter0x139e',
u'\uef21': 'UnicodeCharacter0xef21',
u'\ucea0': 'HangulSyllableKaem',
u'\u6925': 'CjkUnifiedIdeograph6925',
u'\u48a4': 'CjkUnifiedIdeograph48a4',
u'\u9e2b': 'CjkUnifiedIdeograph9e2b',
u'\u81aa': 'CjkUnifiedIdeograph81aa',
u'\u182f': 'MongolianLetterLa',
u'\u7bae': 'CjkUnifiedIdeograph7bae',
u'\ud731': 'HangulSyllableHwit',
u'\u5135': 'CjkUnifiedIdeograph5135',
u'\ub0b4': 'HangulSyllableNae',
u'\u2ab8': 'SucceedsAboveAlmostEqualTo',
u'\u863b': 'CjkUnifiedIdeograph863b',
u'\ue9ba': 'UnicodeCharacter0xe9ba',
u'\u63be': 'CjkUnifiedIdeograph63be',
u'\ub945': 'HangulSyllableRwilg',
u'\u98c4': 'CjkUnifiedIdeograph98c4',
u'\u3349': 'SquareMiri',
u'\u12c8': 'EthiopicSyllableWa',
u'\uee4b': 'UnicodeCharacter0xee4b',
u'\ud1ca': 'HangulSyllableTwalp',
u'\u684f': 'CjkUnifiedIdeograph684f',
u'\u4bce': 'CjkUnifiedIdeograph4bce',
u'\ua155': 'YiSyllableNdiex',
u'\u80d4': 'CjkUnifiedIdeograph80d4',
u'\u1b59': 'BalineseDigitNine',
u'\u7ad8': 'CjkUnifiedIdeograph7ad8',
u'\ud65b': 'HangulSyllableHwad',
u'\u505f': 'CjkUnifiedIdeograph505f',
u'\ub3de': 'HangulSyllableDop',
u'\u2de2': 'CombiningCyrillicLetterGhe',
u'\u8965': 'CjkUnifiedIdeograph8965',
u'\ue8e4': 'UnicodeCharacter0xe8e4',
u'\u0369': 'CombiningLatinSmallLetterD',
u'\u62e8': 'CjkUnifiedIdeograph62e8',
u'\ub86f': 'HangulSyllableRos',
u'\u9bee': 'CjkUnifiedIdeograph9bee',
u'\u3273': 'CircledHangulPieupA',
u'\u15f2': 'CanadianSyllabicsCarrierGee',
u'\uf175': 'UnicodeCharacter0xf175',
u'\ud0f4': 'HangulSyllableTaek',
u'\u6b79': 'CjkUnifiedIdeograph6b79',
u'\u4af8': 'CjkUnifiedIdeograph4af8',
u'\ua07f': 'YiSyllableNba',
u'\u83fe': 'CjkUnifiedIdeograph83fe',
u'\uf606': 'UnicodeCharacter0xf606',
u'\ud289': 'HangulSyllableTwilg',
u'\ub308': 'HangulSyllableDael',
u'\u8f8b': 'CjkUnifiedIdeograph8f8b',
u'\u700a': 'CjkUnifiedIdeograph700a',
u'\u4c8d': 'CjkUnifiedIdeograph4c8d',
u'\u2d0c': 'GeorgianSmallLetterNar',
u'\u098f': 'BengaliLetterE',
u'\ude16': 'UnicodeCharacter0xde16',
u'\u9b18': 'CjkUnifiedIdeograph9b18',
u'\uf79b': 'UnicodeCharacter0xf79b',
u'\u581a': 'CjkUnifiedIdeograph581a',
u'\ub49d': 'HangulSyllableDweng',
u'\u151c': 'CanadianSyllabicsWestcreeShwii',
u'\u719f': 'CjkUnifiedIdeograph719f',
u'\u2ea1': 'CjkRadicalWaterOne',
u'\uc626': 'HangulSyllableOgg',
u'\u8328': 'CjkUnifiedIdeograph8328',
u'\udfab': 'UnicodeCharacter0xdfab',
u'\u402a': 'CjkUnifiedIdeograph402a',
u'\u9cad': 'CjkUnifiedIdeograph9cad',
u'\u7d2c': 'CjkUnifiedIdeograph7d2c',
u'\u59af': 'CjkUnifiedIdeograph59af',
u'\u3a2e': 'CjkUnifiedIdeograph3a2e',
u'\u16b1': 'RunicLetterRaidoRadReidR',
u'\ueb38': 'UnicodeCharacter0xeb38',
u'\uc7bb': 'HangulSyllableJaelh',
u'\ua83a': 'UnicodeCharacter0xa83a',
u'\u84bd': 'CjkUnifiedIdeograph84bd',
u'\u653c': 'CjkUnifiedIdeograph653c',
u'\u41bf': 'CjkUnifiedIdeograph41bf',
u'\u223e': 'InvertedLazyS',
u'\u7ec1': 'CjkUnifiedIdeograph7ec1',
u'\u3bc3': 'CjkUnifiedIdeograph3bc3',
u'\ud348': 'HangulSyllablePyan',
u'\u904a': 'CjkUnifiedIdeograph904a',
u'\ueccd': 'UnicodeCharacter0xeccd',
u'\u4d4c': 'CjkUnifiedIdeograph4d4c',
u'\ua9cf': 'UnicodeCharacter0xa9cf',
u'\u0a4e': 'UnicodeCharacter0xa4e',
u'\u66d1': 'CjkUnifiedIdeograph66d1',
u'\u23d3': 'MetricalShortOverLong',
u'\uf85a': 'UnicodeCharacter0xf85a',
u'\ud4dd': 'HangulSyllablePwib',
u'\ub55c': 'HangulSyllableDdaem',
u'\u91df': 'CjkUnifiedIdeograph91df',
u'\u725e': 'CjkUnifiedIdeograph725e',
u'\u4ee1': 'CjkUnifiedIdeograph4ee1',
u'\u2f60': 'KangxiRadicalMelon',
u'\u0be3': 'UnicodeCharacter0xbe3',
u'\ue06a': 'UnicodeCharacter0xe06a',
u'\u9d6c': 'CjkUnifiedIdeograph9d6c',
u'\uf9ef': 'CjkCompatibilityIdeographf9ef',
u'\u5a6e': 'CjkUnifiedIdeograph5a6e',
u'\ub6f1': 'HangulSyllableDdwig',
u'\u1770': 'TagbanwaLetterSa',
u'\u73f3': 'CjkUnifiedIdeograph73f3',
u'\u30f5': 'KatakanaLetterSmallKa',
u'\uc87a': 'HangulSyllableJolm',
u'\u857c': 'CjkUnifiedIdeograph857c',
u'\ue1ff': 'UnicodeCharacter0xe1ff',
u'\u427e': 'CjkUnifiedIdeograph427e',
u'\u7f80': 'CjkUnifiedIdeograph7f80',
u'\udb03': 'UnicodeCharacter0xdb03',
u'\u3c82': 'CjkUnifiedIdeograph3c82',
u'\u9805': 'CjkUnifiedIdeograph9805',
u'\u5507': 'CjkUnifiedIdeograph5507',
u'\u1209': 'EthiopicSyllableLu',
u'\ued8c': 'UnicodeCharacter0xed8c',
u'\uaa8e': 'UnicodeCharacter0xaa8e',
u'\u6790': 'CjkUnifiedIdeograph6790',
u'\uc313': 'HangulSyllableSsah',
u'\u2492': 'NumberElevenFullStop',
u'\u8015': 'CjkUnifiedIdeograph8015',
u'\ubd17': 'HangulSyllableBwad',
u'\u7a19': 'CjkUnifiedIdeograph7a19',
u'\u371b': 'CjkUnifiedIdeograph371b',
u'\ud59c': 'HangulSyllableHyals',
u'\u929e': 'CjkUnifiedIdeograph929e',
u'\u4fa0': 'CjkUnifiedIdeograph4fa0',
u'\u0ca2': 'KannadaLetterDdha',
u'\ue825': 'UnicodeCharacter0xe825',
u'\ua527': 'VaiSyllableHin',
u'\u6229': 'CjkUnifiedIdeograph6229',
u'\u1f2b': 'GreekCapitalLetterEtaWithDasiaAndVaria',
u'\ufaae': 'CjkCompatibilityIdeographfaae',
u'\ub7b0': 'HangulSyllableRaek',
u'\u74b2': 'CjkUnifiedIdeograph74b2',
u'\ud035': 'HangulSyllableKwig',
u'\u31b4': 'BopomofoFinalLetterP',
u'\u8d37': 'CjkUnifiedIdeograph8d37',
u'\u4a39': 'CjkUnifiedIdeograph4a39',
u'\u073b': 'SyriacHbasaBelow',
u'\ue2be': 'UnicodeCharacter0xe2be',
u'\u9fc0': 'UnicodeCharacter0x9fc0',
u'\u5cc2': 'CjkUnifiedIdeograph5cc2',
u'\u19c4': 'NewTaiLueLetterFinalM',
u'\uf547': 'UnicodeCharacter0xf547',
u'\ub249': 'HangulSyllableNwelt',
u'\u6f4b': 'CjkUnifiedIdeograph6f4b',
u'\u2c4d': 'GlagoliticSmallLetterChrivi',
u'\ucace': 'HangulSyllableJjobs',
u'\u87d0': 'CjkUnifiedIdeograph87d0',
u'\u44d2': 'CjkUnifiedIdeograph44d2',
u'\u01d4': 'LatinSmallLetterUWithCaron',
u'\udd57': 'UnicodeCharacter0xdd57',
u'\u3ed6': 'CjkUnifiedIdeograph3ed6',
u'\u9a59': 'CjkUnifiedIdeograph9a59',
u'\u575b': 'CjkUnifiedIdeograph575b',
u'\u145d': 'CanadianSyllabicsTwo',
u'\uefe0': 'UnicodeCharacter0xefe0',
u'\uace2': 'HangulSyllableGogg',
u'\u69e4': 'CjkUnifiedIdeograph69e4',
u'\uc567': 'HangulSyllableAed',
u'\u26e6': 'UnicodeCharacter0x26e6',
u'\u8269': 'CjkUnifiedIdeograph8269',
u'\ubf6b': 'HangulSyllableBbwalh',
u'\u7c6d': 'CjkUnifiedIdeograph7c6d',
u'\u396f': 'CjkUnifiedIdeograph396f',
u'\ud7f0': 'UnicodeCharacter0xd7f0',
u'\u94f2': 'CjkUnifiedIdeograph94f2',
u'\u51f4': 'CjkUnifiedIdeograph51f4',
u'\u0ef6': 'UnicodeCharacter0xef6',
u'\uea79': 'UnicodeCharacter0xea79',
u'\ua77b': 'LatinCapitalLetterInsularF',
u'\u647d': 'CjkUnifiedIdeograph647d',
u'\u217f': 'SmallRomanNumeralOneThousand',
u'\u2181': 'RomanNumeralFiveThousand',
u'\uc604': 'HangulSyllableYeok',
u'\uea87': 'UnicodeCharacter0xea87',
u'\u4c08': 'CjkUnifiedIdeograph4c08',
u'\u708b': 'CjkUnifiedIdeograph708b',
u'\u8f0a': 'CjkUnifiedIdeograph8f0a',
u'\ub38d': 'HangulSyllableDyeog',
u'\u150e': 'CanadianSyllabicsNaskapiSkwa',
u'\u3991': 'CjkUnifiedIdeograph3991',
u'\ude14': 'UnicodeCharacter0xde14',
u'\u8297': 'CjkUnifiedIdeograph8297',
u'\u6418': 'CjkUnifiedIdeograph6418',
u'\u089b': 'UnicodeCharacter0x89b',
u'\ua71a': 'ModifierLetterLowerRightCornerAngle',
u'\u2d1e': 'GeorgianSmallLetterXan',
u'\ud1a1': 'HangulSyllableTog',
u'\u57a5': 'CjkUnifiedIdeograph57a5',
u'\uf624': 'UnicodeCharacter0xf624',
u'\u9aa7': 'CjkUnifiedIdeograph9aa7',
u'\u7c28': 'CjkUnifiedIdeograph7c28',
u'\u20ab': 'DongSign',
u'\ubf2a': 'HangulSyllableBbyenh',
u'\uc52e': 'HangulSyllableSsinh',
u'\ue9b1': 'UnicodeCharacter0xe9b1',
u'\u4b32': 'CjkUnifiedIdeograph4b32',
u'\u6fb5': 'CjkUnifiedIdeograph6fb5',
u'\u8e34': 'CjkUnifiedIdeograph8e34',
u'\ub2b7': 'HangulSyllableNyilb',
u'\u1438': 'CanadianSyllabicsPa',
u'\u38bb': 'CjkUnifiedIdeograph38bb',
u'\udd3e': 'UnicodeCharacter0xdd3e',
u'\u81c1': 'CjkUnifiedIdeograph81c1',
u'\u6342': 'CjkUnifiedIdeograph6342',
u'\u07c5': 'NkoDigitFive',
u'\ua644': 'CyrillicCapitalLetterReversedDze',
u'\u2c48': 'GlagoliticSmallLetterHeru',
u'\ud0cb': 'HangulSyllableTalb',
u'\u56cf': 'CjkUnifiedIdeograph56cf',
u'\uf54e': 'UnicodeCharacter0xf54e',
u'\u99d1': 'CjkUnifiedIdeograph99d1',
u'\u7b52': 'CjkUnifiedIdeograph7b52',
u'\u1fd5': 'UnicodeCharacter0x1fd5',
u'\ube54': 'HangulSyllableBim',
u'\uc458': 'HangulSyllableSsyom',
u'\ue8db': 'UnicodeCharacter0xe8db',
u'\u4a5c': 'CjkUnifiedIdeograph4a5c',
u'\u6edf': 'CjkUnifiedIdeograph6edf',
u'\u8d5e': 'CjkUnifiedIdeograph8d5e',
u'\ub1e1': 'HangulSyllableNoeng',
u'\u1362': 'EthiopicFullStop',
u'\u37e5': 'CjkUnifiedIdeograph37e5',
u'\udc68': 'UnicodeCharacter0xdc68',
u'\u80eb': 'CjkUnifiedIdeograph80eb',
u'\u626c': 'CjkUnifiedIdeograph626c',
u'\u06ef': 'ArabicLetterRehWithInvertedV',
u'\ua56e': 'VaiSyllableMa',
u'\u2b72': 'UnicodeCharacter0x2b72',
u'\ucff5': 'HangulSyllableKung',
u'\u55f9': 'CjkUnifiedIdeograph55f9',
u'\uf478': 'UnicodeCharacter0xf478',
u'\u98fb': 'CjkUnifiedIdeograph98fb',
u'\u7a7c': 'CjkUnifiedIdeograph7a7c',
u'\u1eff': 'LatinSmallLetterYWithLoop',
u'\ubd7e': 'HangulSyllableByop',
u'\u2403': 'SymbolForEndOfText',
u'\uc382': 'HangulSyllableSseop',
u'\u4986': 'CjkUnifiedIdeograph4986',
u'\ued09': 'UnicodeCharacter0xed09',
u'\u8c88': 'CjkUnifiedIdeograph8c88',
u'\u730d': 'CjkUnifiedIdeograph730d',
u'\u128c': 'EthiopicSyllableXwee',
u'\ub60f': 'HangulSyllableDdyeh',
u'\u3c13': 'CjkUnifiedIdeograph3c13',
u'\udb92': 'UnicodeCharacter0xdb92',
u'\u4217': 'CjkUnifiedIdeograph4217',
u'\u6196': 'CjkUnifiedIdeograph6196',
u'\u8519': 'CjkUnifiedIdeograph8519',
u'\ua498': 'YiRadicalMi',
u'\u0b1d': 'OriyaLetterJha',
u'\u2a9c': 'DoublelineSlantedEqualToOrGreaterthan',
u'\ud423': 'HangulSyllablePwah',
u'\uf3a2': 'UnicodeCharacter0xf3a2',
u'\u5a27': 'CjkUnifiedIdeograph5a27',
u'\u79a6': 'CjkUnifiedIdeograph79a6',
u'\u9d29': 'CjkUnifiedIdeograph9d29',
u'\ubca8': 'HangulSyllableBel',
u'\u232d': 'Cylindricity',
u'\uc2ac': 'HangulSyllableSeul',
u'\u48b0': 'CjkUnifiedIdeograph48b0',
u'\uec33': 'UnicodeCharacter0xec33',
u'\u8bb2': 'CjkUnifiedIdeograph8bb2',
u'\u7237': 'CjkUnifiedIdeograph7237',
u'\u11b6': 'HangulJongseongRieulhieuh',
u'\ub539': 'HangulSyllableDdalg',
u'\u3b3d': 'CjkUnifiedIdeograph3b3d',
u'\udabc': 'UnicodeCharacter0xdabc',
u'\u4141': 'CjkUnifiedIdeograph4141',
u'\u60c0': 'CjkUnifiedIdeograph60c0',
u'\u8443': 'CjkUnifiedIdeograph8443',
u'\ua3c2': 'YiSyllableShy',
u'\u0a47': 'GurmukhiVowelSignEe',
u'\u29c6': 'SquaredAsterisk',
u'\ud34d': 'HangulSyllablePyalg',
u'\uf2cc': 'UnicodeCharacter0xf2cc',
u'\u5951': 'CjkUnifiedIdeograph5951',
u'\u78d0': 'CjkUnifiedIdeograph78d0',
u'\u9c53': 'CjkUnifiedIdeograph9c53',
u'\ubbd2': 'HangulSyllableMeubs',
u'\u2257': 'RingEqualTo',
u'\uc1d6': 'HangulSyllableSwaebs',
u'\u47da': 'CjkUnifiedIdeograph47da',
u'\ueb5d': 'UnicodeCharacter0xeb5d',
u'\u8adc': 'CjkUnifiedIdeograph8adc',
u'\u7161': 'CjkUnifiedIdeograph7161',
u'\u10e0': 'GeorgianLetterRae',
u'\ub463': 'HangulSyllableDus',
u'\u3a67': 'CjkUnifiedIdeograph3a67',
u'\ud9e6': 'UnicodeCharacter0xd9e6',
u'\u406b': 'CjkUnifiedIdeograph406b',
u'\u5fea': 'CjkUnifiedIdeograph5fea',
u'\u836d': 'CjkUnifiedIdeograph836d',
u'\ua2ec': 'YiSyllableZzyr',
u'\u0971': 'DevanagariSignHighSpacingDot',
u'\u28f0': 'BraillePatternDots5678',
u'\ud277': 'HangulSyllableTwes',
u'\uf1f6': 'UnicodeCharacter0xf1f6',
u'\u587b': 'CjkUnifiedIdeograph587b',
u'\u77fa': 'CjkUnifiedIdeograph77fa',
u'\u9b7d': 'CjkUnifiedIdeograph9b7d',
u'\ubafc': 'HangulSyllableMoe',
u'\ua081': 'YiSyllableNbot',
u'\uc100': 'HangulSyllableSyae',
u'\ue583': 'UnicodeCharacter0xe583',
u'\u0202': 'LatinCapitalLetterAWithInvertedBreve',
u'\u2685': 'DieFace6',
u'\u4704': 'CjkUnifiedIdeograph4704',
u'\u6b87': 'CjkUnifiedIdeograph6b87',
u'\u940e': 'CjkUnifiedIdeograph940e',
u'\ub891': 'HangulSyllableRwat',
u'\ud910': 'UnicodeCharacter0xd910',
u'\ufd93': 'ArabicLigatureHehWithMeemWithJeemInitialForm',
u'\u1a12': 'BugineseLetterLa',
u'\u3e95': 'CjkUnifiedIdeograph3e95',
u'\u5f14': 'CjkUnifiedIdeograph5f14',
u'\u0397': 'GreekCapitalLetterEta',
u'\u4499': 'CjkUnifiedIdeograph4499',
u'\uac1e': 'HangulSyllableGaegg',
u'\uf120': 'UnicodeCharacter0xf120',
u'\u95a3': 'CjkUnifiedIdeograph95a3',
u'\u3222': 'ParenthesizedIdeographThree',
u'\ud6a5': 'HangulSyllableHoet',
u'\u7724': 'CjkUnifiedIdeograph7724',
u'\u1ba7': 'SundaneseVowelSignPanolong',
u'\u5ca9': 'CjkUnifiedIdeograph5ca9',
u'\u8930': 'CjkUnifiedIdeograph8930',
u'\uadb3': 'HangulSyllableGwelh',
u'\uca32': 'HangulSyllableJjyaegg',
u'\ueeb5': 'UnicodeCharacter0xeeb5',
u'\u0f34': 'TibetanMarkBsdusRtags',
u'\u33b7': 'SquareMv',
u'\u5036': 'CjkUnifiedIdeograph5036',
u'\u74b9': 'CjkUnifiedIdeograph74b9',
u'\ua140': 'YiSyllableDdap',
u'\ue242': 'UnicodeCharacter0xe242',
u'\u86c5': 'CjkUnifiedIdeograph86c5',
u'\u2744': 'Snowflake',
u'\ucbc7': 'HangulSyllableJjyulh',
u'\u6846': 'CjkUnifiedIdeograph6846',
u'\u0cc9': 'UnicodeCharacter0xcc9',
u'\u51cb': 'CjkUnifiedIdeograph51cb',
u'\ub950': 'HangulSyllableRwiss',
u'\ufa52': 'CjkCompatibilityIdeographfa52',
u'\u9ed5': 'CjkUnifiedIdeograph9ed5',
u'\u3f54': 'CjkUnifiedIdeograph3f54',
u'\ue3d7': 'UnicodeCharacter0xe3d7',
u'\u24d9': 'CircledLatinSmallLetterJ',
u'\u4558': 'CjkUnifiedIdeograph4558',
u'\u69db': 'CjkUnifiedIdeograph69db',
u'\u9262': 'CjkUnifiedIdeograph9262',
u'\ub6e5': 'HangulSyllableDdweb',
u'\ud764': 'HangulSyllableHeuss',
u'\ufbe7': 'ArabicLetterEMedialForm',
u'\u1866': 'MongolianLetterSibePa',
u'\u3ce9': 'CjkUnifiedIdeograph3ce9',
u'\u5d68': 'CjkUnifiedIdeograph5d68',
u'\u01eb': 'LatinSmallLetterOWithOgonek',
u'\u42ed': 'CjkUnifiedIdeograph42ed',
u'\uaa72': 'UnicodeCharacter0xaa72',
u'\uef74': 'UnicodeCharacter0xef74',
u'\u93f7': 'CjkUnifiedIdeograph93f7',
u'\u3076': 'HiraganaLetterBu',
u'\ud4f9': 'HangulSyllablePyub',
u'\u7578': 'CjkUnifiedIdeograph7578',
u'\u19fb': 'KhmerSymbolDapmuoyRoc',
u'\u5afd': 'CjkUnifiedIdeograph5afd',
u'\u6001': 'CjkUnifiedIdeograph6001',
u'\u8784': 'CjkUnifiedIdeograph8784',
u'\uc886': 'HangulSyllableJoj',
u'\u0d88': 'SinhalaLetterAeeyanna',
u'\ub10b': 'HangulSyllableNeogs',
u'\u4e8a': 'CjkUnifiedIdeograph4e8a',
u'\uf20d': 'UnicodeCharacter0xf20d',
u'\u370f': 'CjkUnifiedIdeograph370f',
u'\u7811': 'CjkUnifiedIdeograph7811',
u'\u9f94': 'CjkUnifiedIdeograph9f94',
u'\ue096': 'UnicodeCharacter0xe096',
u'\u2598': 'QuadrantUpperLeft',
u'\u669a': 'CjkUnifiedIdeograph669a',
u'\u8a1d': 'CjkUnifiedIdeograph8a1d',
u'\ucf1f': 'HangulSyllableKyeogs',
u'\u1021': 'MyanmarLetterA',
u'\u5523': 'CjkUnifiedIdeograph5523',
u'\ub7a4': 'HangulSyllableRaels',
u'\uf8a6': 'UnicodeCharacter0xf8a6',
u'\u3da8': 'CjkUnifiedIdeograph3da8',
u'\u7eaa': 'CjkUnifiedIdeograph7eaa',
u'\ua22d': 'YiSyllableGgup',
u'\u43ac': 'CjkUnifiedIdeograph43ac',
u'\ue72f': 'UnicodeCharacter0xe72f',
u'\u2831': 'BraillePatternDots156',
u'\u6d33': 'CjkUnifiedIdeograph6d33',
u'\u90b6': 'CjkUnifiedIdeograph90b6',
u'\ud5b8': 'HangulSyllableHyaels',
u'\u16ba': 'RunicLetterHaglazH',
u'\uba3d': 'HangulSyllableMeonj',
u'\u5bbc': 'CjkUnifiedIdeograph5bbc',
u'\uff3f': 'FullwidthLowLine',
u'\uc041': 'HangulSyllableBbyunj',
u'\u0543': 'ArmenianCapitalLetterCheh',
u'\u4645': 'CjkUnifiedIdeograph4645',
u'\ua8c6': 'UnicodeCharacter0xa8c6',
u'\uedc8': 'UnicodeCharacter0xedc8',
u'\u2eca': 'CjkRadicalFoot',
u'\u73cc': 'CjkUnifiedIdeograph73cc',
u'\u974f': 'CjkUnifiedIdeograph974f',
u'\ud851': 'UnicodeCharacter0xd851',
u'\u1d53': 'ModifierLetterSmallOpenO',
u'\u5e55': 'CjkUnifiedIdeograph5e55',
u'\u85d8': 'CjkUnifiedIdeograph85d8',
u'\uc6da': 'HangulSyllableWeolp',
u'\u0bdc': 'UnicodeCharacter0xbdc',
u'\uaf5f': 'HangulSyllableGgwac',
u'\u4cde': 'CjkUnifiedIdeograph4cde',
u'\uf061': 'UnicodeCharacter0xf061',
u'\u3563': 'CjkUnifiedIdeograph3563',
u'\u7665': 'CjkUnifiedIdeograph7665',
u'\u9de8': 'CjkUnifiedIdeograph9de8',
u'\udeea': 'UnicodeCharacter0xdeea',
u'\u23ec': 'UnicodeCharacter0x23ec',
u'\u64ee': 'CjkUnifiedIdeograph64ee',
u'\u8871': 'CjkUnifiedIdeograph8871',
u'\ucd73': 'HangulSyllableCoec',
u'\u0e75': 'UnicodeCharacter0xe75',
u'\u5377': 'CjkUnifiedIdeograph5377',
u'\ub5f8': 'HangulSyllableDdyen',
u'\uf6fa': 'UnicodeCharacter0xf6fa',
u'\u3bfc': 'CjkUnifiedIdeograph3bfc',
u'\u7cfe': 'CjkUnifiedIdeograph7cfe',
u'\u4200': 'CjkUnifiedIdeograph4200',
u'\u6683': 'CjkUnifiedIdeograph6683',
u'\u8102': 'CjkUnifiedIdeograph8102',
u'\ua585': 'VaiSyllableDoo',
u'\u0b06': 'OriyaLetterAa',
u'\u2f89': 'KangxiRadicalStopping',
u'\ud00c': 'HangulSyllableKweom',
u'\uf48f': 'UnicodeCharacter0xf48f',
u'\u5a10': 'CjkUnifiedIdeograph5a10',
u'\u7e93': 'CjkUnifiedIdeograph7e93',
u'\u9912': 'CjkUnifiedIdeograph9912',
u'\ubd95': 'HangulSyllableBung',
u'\u2316': 'PositionIndicator',
u'\uc799': 'HangulSyllableJalg',
u'\u499d': 'CjkUnifiedIdeograph499d',
u'\ue81c': 'UnicodeCharacter0xe81c',
u'\u8c9f': 'CjkUnifiedIdeograph8c9f',
u'\u7220': 'CjkUnifiedIdeograph7220',
u'\u16a3': 'RunicLetterYr',
u'\ub122': 'HangulSyllableNeop',
u'\u3b26': 'CjkUnifiedIdeograph3b26',
u'\udfa9': 'UnicodeCharacter0xdfa9',
u'\u61ad': 'CjkUnifiedIdeograph61ad',
u'\u802c': 'CjkUnifiedIdeograph802c',
u'\ua4af': 'YiRadicalVur',
u'\u0a30': 'GurmukhiLetterRa',
u'\u2eb3': 'CjkRadicalNetThree',
u'\ud336': 'HangulSyllablePaelp',
u'\uf7b9': 'UnicodeCharacter0xf7b9',
u'\u553a': 'CjkUnifiedIdeograph553a',
u'\u79bd': 'CjkUnifiedIdeograph79bd',
u'\u983c': 'CjkUnifiedIdeograph983c',
u'\ubcbf': 'HangulSyllableByeogs',
u'\u2240': 'WreathProduct',
u'\uc6c3': 'HangulSyllableUs',
u'\u48c7': 'CjkUnifiedIdeograph48c7',
u'\ueb46': 'UnicodeCharacter0xeb46',
u'\u8fc9': 'CjkUnifiedIdeograph8fc9',
u'\u6d4a': 'CjkUnifiedIdeograph6d4a',
u'\u11cd': 'HangulJongseongRieulnieun',
u'\ub04c': 'HangulSyllableGgeul',
u'\u3a50': 'CjkUnifiedIdeograph3a50',
u'\uded3': 'UnicodeCharacter0xded3',
u'\u60d7': 'CjkUnifiedIdeograph60d7',
u'\u8356': 'CjkUnifiedIdeograph8356',
u'\ua7d9': 'UnicodeCharacter0xa7d9',
u'\u055a': 'ArmenianApostrophe',
u'\u29dd': 'TieOverInfinity',
u'\ud260': 'HangulSyllableTweok',
u'\uf6e3': 'UnicodeCharacter0xf6e3',
u'\u5464': 'CjkUnifiedIdeograph5464',
u'\u78e7': 'CjkUnifiedIdeograph78e7',
u'\u9b66': 'CjkUnifiedIdeograph9b66',
u'\ubfe9': 'HangulSyllableBbweog',
u'\u1d6a': 'GreekSubscriptSmallLetterChi',
u'\uc1ed': 'HangulSyllableSoelt',
u'\u4bf1': 'CjkUnifiedIdeograph4bf1',
u'\uea70': 'UnicodeCharacter0xea70',
u'\u8ef3': 'CjkUnifiedIdeograph8ef3',
u'\u6c74': 'CjkUnifiedIdeograph6c74',
u'\u10f7': 'GeorgianLetterYn',
u'\ub376': 'HangulSyllableDenh',
u'\u357a': 'CjkUnifiedIdeograph357a',
u'\ud9fd': 'UnicodeCharacter0xd9fd',
u'\ue301': 'UnicodeCharacter0xe301',
u'\u8280': 'CjkUnifiedIdeograph8280',
u'\u6505': 'CjkUnifiedIdeograph6505',
u'\u0484': 'CombiningCyrillicPalatalization',
u'\ua807': 'SylotiNagriLetterKo',
u'\u320b': 'ParenthesizedHangulThieuth',
u'\ucd8a': 'HangulSyllableCyobs',
u'\u578e': 'CjkUnifiedIdeograph578e',
u'\ufb11': 'UnicodeCharacter0xfb11',
u'\u9a90': 'CjkUnifiedIdeograph9a90',
u'\u7d15': 'CjkUnifiedIdeograph7d15',
u'\u1c94': 'UnicodeCharacter0x1c94',
u'\uca1b': 'HangulSyllableJjyad',
u'\ue59a': 'UnicodeCharacter0xe59a',
u'\u4c1f': 'CjkUnifiedIdeograph4c1f',
u'\u6f9e': 'CjkUnifiedIdeograph6f9e',
u'\u9321': 'CjkUnifiedIdeograph9321',
u'\ub2a0': 'HangulSyllableNeum',
u'\u1525': 'CanadianSyllabicsSh',
u'\u34a4': 'CjkUnifiedIdeograph34a4',
u'\ue22b': 'UnicodeCharacter0xe22b',
u'\ufdaa': 'ArabicLigatureSheenWithHahWithYehFinalForm',
u'\u642f': 'CjkUnifiedIdeograph642f',
u'\u07ae': 'ThaanaObofili',
u'\uab31': 'UnicodeCharacter0xab31',
u'\u2d35': 'TifinaghLetterBerberAcademyYaj',
u'\uccb4': 'HangulSyllableCe',
u'\u56b8': 'CjkUnifiedIdeograph56b8',
u'\ufa3b': 'CjkCompatibilityIdeographfa3b',
u'\u95ba': 'CjkUnifiedIdeograph95ba',
u'\u7c3f': 'CjkUnifiedIdeograph7c3f',
u'\u1fbe': 'GreekProsgegrammeni',
u'\uc545': 'HangulSyllableAg',
u'\ue4c4': 'UnicodeCharacter0xe4c4',
u'\u4f49': 'CjkUnifiedIdeograph4f49',
u'\u6ec8': 'CjkUnifiedIdeograph6ec8',
u'\u924b': 'CjkUnifiedIdeograph924b',
u'\uadca': 'HangulSyllableGwilm',
u'\u144f': 'CanadianSyllabicsTii',
u'\u37ce': 'CjkUnifiedIdeograph37ce',
u'\udd55': 'UnicodeCharacter0xdd55',
u'\ufcd4': 'ArabicLigatureNoonWithKhahInitialForm',
u'\u6759': 'CjkUnifiedIdeograph6759',
u'\u06d8': 'ArabicSmallHighMeemInitialForm',
u'\uaa5b': 'UnicodeCharacter0xaa5b',
u'\u2c5f': 'UnicodeCharacter0x2c5f',
u'\ucfde': 'HangulSyllableKyop',
u'\u51e2': 'CjkUnifiedIdeograph51e2',
u'\uf565': 'UnicodeCharacter0xf565',
u'\u94e4': 'CjkUnifiedIdeograph94e4',
u'\u7f69': 'CjkUnifiedIdeograph7f69',
u'\u1ee8': 'LatinCapitalLetterUWithHornAndAcute',
u'\uc46f': 'HangulSyllableSsulb',
u'\ue7ee': 'UnicodeCharacter0xe7ee',
u'\u4e73': 'CjkUnifiedIdeograph4e73',
u'\u69f2': 'CjkUnifiedIdeograph69f2',
u'\u8d75': 'CjkUnifiedIdeograph8d75',
u'\uacf4': 'HangulSyllableGoss',
u'\u1779': 'UnicodeCharacter0x1779',
u'\u36f8': 'CjkUnifiedIdeograph36f8',
u'\udc7f': 'UnicodeCharacter0xdc7f',
u'\ufffe': 'UnicodeCharacter0xfffe',
u'\u8a06': 'CjkUnifiedIdeograph8a06',
u'\uae89': 'HangulSyllableGgyanj',
u'\ucf08': 'HangulSyllableKel',
u'\uf38b': 'UnicodeCharacter0xf38b',
u'\u0c0a': 'TeluguLetterUu',
u'\u308d': 'HiraganaLetterRo',
u'\u510c': 'CjkUnifiedIdeograph510c',
u'\u758f': 'CjkUnifiedIdeograph758f',
u'\ua216': 'YiSyllableGgi',
u'\ue718': 'UnicodeCharacter0xe718',
u'\u8b9b': 'CjkUnifiedIdeograph8b9b',
u'\u241a': 'SymbolForSubstitute',
u'\uc89d': 'HangulSyllableJwab',
u'\u691c': 'CjkUnifiedIdeograph691c',
u'\u0d9f': 'SinhalaLetterSanyakaGayanna',
u'\u52a1': 'CjkUnifiedIdeograph52a1',
u'\uba26': 'HangulSyllableMyaelm',
u'\uff28': 'FullwidthLatinCapitalLetterH',
u'\ua3ab': 'YiSyllableShat',
u'\u3c2a': 'CjkUnifiedIdeograph3c2a',
u'\ue0ad': 'UnicodeCharacter0xe0ad',
u'\u012c': 'LatinCapitalLetterIWithBreve',
u'\u25af': 'WhiteVerticalRectangle',
u'\u462e': 'CjkUnifiedIdeograph462e',
u'\u6ab1': 'CjkUnifiedIdeograph6ab1',
u'\u9738': 'CjkUnifiedIdeograph9738',
u'\ubbbb': 'HangulSyllableMyuc',
u'\ud43a': 'HangulSyllablePwaej',
u'\uf8bd': 'UnicodeCharacter0xf8bd',
u'\u193c': 'UnicodeCharacter0x193c',
u'\u3dbf': 'CjkUnifiedIdeograph3dbf',
u'\u5e3e': 'CjkUnifiedIdeograph5e3e',
u'\u02c1': 'ModifierLetterReversedGlottalStop',
u'\u47c3': 'CjkUnifiedIdeograph47c3',
u'\uaf48': 'HangulSyllableGgwa',
u'\uec4a': 'UnicodeCharacter0xec4a',
u'\u90cd': 'CjkUnifiedIdeograph90cd',
u'\u314c': 'HangulLetterThieuth',
u'\ud5cf': 'HangulSyllableHeod',
u'\u764e': 'CjkUnifiedIdeograph764e',
u'\u1ad1': 'UnicodeCharacter0x1ad1',
u'\u5fd3': 'CjkUnifiedIdeograph5fd3',
u'\u845a': 'CjkUnifiedIdeograph845a',
u'\ua8dd': 'UnicodeCharacter0xa8dd',
u'\uc95c': 'HangulSyllableJwils',
u'\ueddf': 'UnicodeCharacter0xeddf',
u'\u0e5e': 'UnicodeCharacter0xe5e',
u'\u32e1': 'CircledKatakanaTu',
u'\u5360': 'CjkUnifiedIdeograph5360',
u'\u77e3': 'CjkUnifiedIdeograph77e3',
u'\u9c6a': 'CjkUnifiedIdeograph9c6a',
u'\ue16c': 'UnicodeCharacter0xe16c',
u'\u85ef': 'CjkUnifiedIdeograph85ef',
u'\u266e': 'MusicNaturalSign',
u'\ucaf1': 'HangulSyllableJjwat',
u'\u6b70': 'CjkUnifiedIdeograph6b70',
u'\u0ff3': 'UnicodeCharacter0xff3',
u'\u4cf5': 'CjkUnifiedIdeograph4cf5',
u'\ub47a': 'HangulSyllableDweolp',
u'\uf97c': 'CjkCompatibilityIdeographf97c',
u'\u9dff': 'CjkUnifiedIdeograph9dff',
u'\u3e7e': 'CjkUnifiedIdeograph3e7e',
u'\u0380': 'UnicodeCharacter0x380',
u'\ua703': 'ModifierLetterChineseToneYangShang',
u'\u4082': 'CjkUnifiedIdeograph4082',
u'\ue405': 'UnicodeCharacter0xe405',
u'\u2907': 'RightwardsDoubleArrowFromBar',
u'\u6e09': 'CjkUnifiedIdeograph6e09',
u'\u918c': 'CjkUnifiedIdeograph918c',
u'\ud68e': 'HangulSyllableHoegg',
u'\u1b90': 'SundaneseLetterZa',
u'\ubf13': 'HangulSyllableBbyeolb',
u'\u5892': 'CjkUnifiedIdeograph5892',
u'\ufc15': 'ArabicLigatureJeemWithHahIsolatedForm',
u'\uc117': 'HangulSyllableSyaec',
u'\u0619': 'ArabicSmallDamma',
u'\u4b1b': 'CjkUnifiedIdeograph4b1b',
u'\ua99c': 'UnicodeCharacter0xa99c',
u'\uee9e': 'UnicodeCharacter0xee9e',
u'\u33a0': 'SquareCmSquared',
u'\u70a2': 'CjkUnifiedIdeograph70a2',
u'\u9425': 'CjkUnifiedIdeograph9425',
u'\ud927': 'UnicodeCharacter0xd927',
u'\u1e29': 'LatinSmallLetterHWithCedilla',
u'\u632b': 'CjkUnifiedIdeograph632b',
u'\u86ae': 'CjkUnifiedIdeograph86ae',
u'\ucbb0': 'HangulSyllableJjwiss',
u'\u08b2': 'UnicodeCharacter0x8b2',
u'\uac35': 'HangulSyllableGaet',
u'\u4db4': 'CjkUnifiedIdeograph4db4',
u'\uf137': 'UnicodeCharacter0xf137',
u'\u3639': 'CjkUnifiedIdeograph3639',
u'\u7b3b': 'CjkUnifiedIdeograph7b3b',
u'\u9ebe': 'CjkUnifiedIdeograph9ebe',
u'\ue3c0': 'UnicodeCharacter0xe3c0',
u'\u20c2': 'UnicodeCharacter0x20c2',
u'\u65c4': 'CjkUnifiedIdeograph65c4',
u'\u8947': 'CjkUnifiedIdeograph8947',
u'\uce49': 'HangulSyllableCyilt',
u'\u134b': 'EthiopicSyllableFaa',
u'\u504d': 'CjkUnifiedIdeograph504d',
u'\ub6ce': 'HangulSyllableDdweoj',
u'\ufbd0': 'UnicodeCharacter0xfbd0',
u'\u38d2': 'CjkUnifiedIdeograph38d2',
u'\u7dd4': 'CjkUnifiedIdeograph7dd4',
u'\ua157': 'YiSyllableNdat',
u'\u42d6': 'CjkUnifiedIdeograph42d6',
u'\ue659': 'UnicodeCharacter0xe659',
u'\u2b5b': 'UnicodeCharacter0x2b5b',
u'\u685d': 'CjkUnifiedIdeograph685d',
u'\u93e0': 'CjkUnifiedIdeograph93e0',
u'\ud0e2': 'HangulSyllableTaenh',
u'\u15e4': 'CanadianSyllabicsCarrierTte',
u'\ub967': 'HangulSyllableRyulh',
u'\u5ae6': 'CjkUnifiedIdeograph5ae6',
u'\ufe69': 'SmallDollarSign',
u'\uc36b': 'HangulSyllableSseogs',
u'\u456f': 'CjkUnifiedIdeograph456f',
u'\uabf0': 'UnicodeCharacter0xabf0',
u'\ue8f2': 'UnicodeCharacter0xe8f2',
u'\u2df4': 'CombiningCyrillicLetterFita',
u'\u72f6': 'CjkUnifiedIdeograph72f6',
u'\u9679': 'CjkUnifiedIdeograph9679',
u'\udb7b': 'UnicodeCharacter0xdb7b',
u'\u187d': 'UnicodeCharacter0x187d',
u'\u5d7f': 'CjkUnifiedIdeograph5d7f',
u'\u2581': 'LowerOneEighthBlock',
u'\uc204': 'HangulSyllableSyol',
u'\uee87': 'UnicodeCharacter0xee87',
u'\u4808': 'CjkUnifiedIdeograph4808',
u'\u748b': 'CjkUnifiedIdeograph748b',
u'\u8b0a': 'CjkUnifiedIdeograph8b0a',
u'\ub78d': 'HangulSyllableRab',
u'\u110e': 'HangulChoseongChieuch',
u'\u3d91': 'CjkUnifiedIdeograph3d91',
u'\uda14': 'UnicodeCharacter0xda14',
u'\u8697': 'CjkUnifiedIdeograph8697',
u'\u6018': 'CjkUnifiedIdeograph6018',
u'\u0c9b': 'KannadaLetterCha',
u'\ua31a': 'YiSyllableSop',
u'\u291e': 'RightwardsArrowToBlackDiamond',
u'\ud5a1': 'HangulSyllableHyab',
u'\u53a5': 'CjkUnifiedIdeograph53a5',
u'\uf224': 'UnicodeCharacter0xf224',
u'\u9ea7': 'CjkUnifiedIdeograph9ea7',
u'\u7828': 'CjkUnifiedIdeograph7828',
u'\u24ab': 'ParenthesizedLatinSmallLetterP',
u'\ubb2a': 'HangulSyllableMyobs',
u'\uc12e': 'HangulSyllableSeobs',
u'\uedb1': 'UnicodeCharacter0xedb1',
u'\u4f32': 'CjkUnifiedIdeograph4f32',
u'\u6bb5': 'CjkUnifiedIdeograph6bb5',
u'\u8a34': 'CjkUnifiedIdeograph8a34',
u'\ub6b7': 'HangulSyllableDduh',
u'\u1038': 'MyanmarSignVisarga',
u'\u3cbb': 'CjkUnifiedIdeograph3cbb',
u'\ud93e': 'UnicodeCharacter0xd93e',
u'\u85c1': 'CjkUnifiedIdeograph85c1',
u'\u6742': 'CjkUnifiedIdeograph6742',
u'\u03c5': 'GreekSmallLetterUpsilon',
u'\ua244': 'YiSyllableMgurx',
u'\u2848': 'BraillePatternDots47',
u'\ud4cb': 'HangulSyllablePweh',
u'\u52cf': 'CjkUnifiedIdeograph52cf',
u'\uf14e': 'UnicodeCharacter0xf14e',
u'\u9dd1': 'CjkUnifiedIdeograph9dd1',
u'\u7f52': 'CjkUnifiedIdeograph7f52',
u'\u1bd5': 'UnicodeCharacter0x1bd5',
u'\uba54': 'HangulSyllableMe',
u'\uc058': 'HangulSyllableBbeu',
u'\uecdb': 'UnicodeCharacter0xecdb',
u'\u4e5c': 'CjkUnifiedIdeograph4e5c',
u'\u6adf': 'CjkUnifiedIdeograph6adf',
u'\u895e': 'CjkUnifiedIdeograph895e',
u'\ub5e1': 'HangulSyllableDdyeolg',
u'\u1762': 'TagbanwaLetterU',
u'\u33e5': 'IdeographicTelegraphSymbolForDaySix',
u'\ud868': 'UnicodeCharacter0xd868',
u'\u84eb': 'CjkUnifiedIdeograph84eb',
u'\u666c': 'CjkUnifiedIdeograph666c',
u'\u02ef': 'ModifierLetterLowDownArrowhead',
u'\ua16e': 'YiSyllableHnie',
u'\u2f72': 'KangxiRadicalGrain',
u'\ucbf5': 'HangulSyllableJjyinj',
u'\u51f9': 'CjkUnifiedIdeograph51f9',
u'\uf078': 'UnicodeCharacter0xf078',
u'\u9cfb': 'CjkUnifiedIdeograph9cfb',
u'\u7e7c': 'CjkUnifiedIdeograph7e7c',
u'\u1aff': 'UnicodeCharacter0x1aff',
u'\ub97e': 'HangulSyllableReulm',
u'\u2003': 'EmSpace',
u'\uc782': 'HangulSyllableIlp',
u'\u4d86': 'CjkUnifiedIdeograph4d86',
u'\ue909': 'UnicodeCharacter0xe909',
u'\u8888': 'CjkUnifiedIdeograph8888',
u'\u770d': 'CjkUnifiedIdeograph770d',
u'\u168c': 'OghamLetterGort',
u'\ub20f': 'HangulSyllableNulb',
u'\u3813': 'CjkUnifiedIdeograph3813',
u'\udf92': 'UnicodeCharacter0xdf92',
u'\u4617': 'CjkUnifiedIdeograph4617',
u'\u6596': 'CjkUnifiedIdeograph6596',
u'\u8119': 'CjkUnifiedIdeograph8119',
u'\ua098': 'YiSyllableHmat',
u'\u0f1d': 'TibetanSignRdelNagGcig',
u'\u2e9c': 'CjkRadicalSun',
u'\ud023': 'HangulSyllableKwelb',
u'\uf7a2': 'UnicodeCharacter0xf7a2',
u'\u5e27': 'CjkUnifiedIdeograph5e27',
u'\u7da6': 'CjkUnifiedIdeograph7da6',
u'\u9929': 'CjkUnifiedIdeograph9929',
u'\ub8a8': 'HangulSyllableRwaess',
u'\u272d': 'OutlinedBlackStar',
u'\uc6ac': 'HangulSyllableYok',
u'\u4cb0': 'CjkUnifiedIdeograph4cb0',
u'\ue833': 'UnicodeCharacter0xe833',
u'\u8fb2': 'CjkUnifiedIdeograph8fb2',
u'\u7637': 'CjkUnifiedIdeograph7637',
u'\u15b6': 'CanadianSyllabicsBlackfootWo',
u'\ub139': 'HangulSyllableNeng',
u'\u3f3d': 'CjkUnifiedIdeograph3f3d',
u'\udebc': 'UnicodeCharacter0xdebc',
u'\u4541': 'CjkUnifiedIdeograph4541',
u'\u64c0': 'CjkUnifiedIdeograph64c0',
u'\u8043': 'CjkUnifiedIdeograph8043',
u'\ua7c2': 'UnicodeCharacter0xa7c2',
u'\u0e47': 'ThaiCharacterMaitaikhu',
u'\u2dc6': 'EthiopicSyllableQyo',
u'\ud74d': 'HangulSyllableHyut',
u'\uf6cc': 'UnicodeCharacter0xf6cc',
u'\u5d51': 'CjkUnifiedIdeograph5d51',
u'\u7cd0': 'CjkUnifiedIdeograph7cd0',
u'\u9853': 'CjkUnifiedIdeograph9853',
u'\ubfd2': 'HangulSyllableBbunh',
u'\u2657': 'WhiteChessBishop',
u'\uc5d6': 'HangulSyllableEnh',
u'\u43da': 'CjkUnifiedIdeograph43da',
u'\uef5d': 'UnicodeCharacter0xef5d',
u'\u8edc': 'CjkUnifiedIdeograph8edc',
u'\u7561': 'CjkUnifiedIdeograph7561',
u'\u14e0': 'CanadianSyllabicsLwii',
u'\ub063': 'HangulSyllableGgyigs',
u'\u3e67': 'CjkUnifiedIdeograph3e67',
u'\udde6': 'UnicodeCharacter0xdde6',
u'\u446b': 'CjkUnifiedIdeograph446b',
u'\u5bea': 'CjkUnifiedIdeograph5bea',
u'\u876d': 'CjkUnifiedIdeograph876d',
u'\ua6ec': 'UnicodeCharacter0xa6ec',
u'\u0d71': 'MalayalamNumberOneHundred',
u'\u2cf0': 'UnicodeCharacter0x2cf0',
u'\ud677': 'HangulSyllableHwaed',
u'\uf5f6': 'UnicodeCharacter0xf5f6',
u'\u5c7b': 'CjkUnifiedIdeograph5c7b',
u'\u73fa': 'CjkUnifiedIdeograph73fa',
u'\u9f7d': 'CjkUnifiedIdeograph9f7d',
u'\ubefc': 'HangulSyllableBbem',
u'\ua481': 'YiSyllableYut',
u'\uc500': 'HangulSyllableSseum',
u'\ue183': 'UnicodeCharacter0xe183',
u'\u0602': 'ArabicFootnoteMarker',
u'\u2285': 'NotASupersetOf',
u'\u4304': 'CjkUnifiedIdeograph4304',
u'\u6f87': 'CjkUnifiedIdeograph6f87',
u'\u900e': 'CjkUnifiedIdeograph900e',
u'\ubc91': 'HangulSyllableBeolt',
u'\udd10': 'UnicodeCharacter0xdd10',
u'\uf993': 'CjkCompatibilityIdeographf993',
u'\u1e12': 'LatinCapitalLetterDWithCircumflexBelow',
u'\u3a95': 'CjkUnifiedIdeograph3a95',
u'\u5b14': 'CjkUnifiedIdeograph5b14',
u'\u0797': 'ThaanaLetterChaviyani',
u'\u4099': 'CjkUnifiedIdeograph4099',
u'\ua81e': 'SylotiNagriLetterRo',
u'\uf520': 'UnicodeCharacter0xf520',
u'\u91a3': 'CjkUnifiedIdeograph91a3',
u'\u3622': 'CjkUnifiedIdeograph3622',
u'\ud2a5': 'HangulSyllableTyulg',
u'\u7324': 'CjkUnifiedIdeograph7324',
u'\u1fa7': 'GreekSmallLetterOmegaWithDasiaAndPerispomeniAndYpogegrammeni',
u'\u58a9': 'CjkUnifiedIdeograph58a9',
u'\u8d30': 'CjkUnifiedIdeograph8d30',
u'\ua9b3': 'UnicodeCharacter0xa9b3',
u'\uce32': 'HangulSyllableCeubs',
u'\ueab5': 'UnicodeCharacter0xeab5',
u'\u0b34': 'UnicodeCharacter0xb34',
u'\u37b7': 'CjkUnifiedIdeograph37b7',
u'\u5436': 'CjkUnifiedIdeograph5436',
u'\u70b9': 'CjkUnifiedIdeograph70b9',
u'\ua540': 'VaiSyllableJi',
u'\ue642': 'UnicodeCharacter0xe642',
u'\u82c5': 'CjkUnifiedIdeograph82c5',
u'\u2344': 'AplFunctionalSymbolQuadGreaterthan',
u'\ucfc7': 'HangulSyllableKyogs',
u'\u6c46': 'CjkUnifiedIdeograph6c46',
u'\u08c9': 'UnicodeCharacter0x8c9',
u'\u55cb': 'CjkUnifiedIdeograph55cb',
u'\ubd50': 'HangulSyllableBoel',
u'\ufe52': 'SmallFullStop',
u'\u9ad5': 'CjkUnifiedIdeograph9ad5',
u'\u3b54': 'CjkUnifiedIdeograph3b54',
u'\ue7d7': 'UnicodeCharacter0xe7d7',
u'\u0456': 'CyrillicSmallLetterByelorussianukrainianI',
u'\u20d9': 'CombiningClockwiseRingOverlay',
u'\u4158': 'CjkUnifiedIdeograph4158',
u'\u6ddb': 'CjkUnifiedIdeograph6ddb',
u'\u9662': 'CjkUnifiedIdeograph9662',
u'\ub2e5': 'HangulSyllableDag',
u'\ud364': 'HangulSyllablePyaen',
u'\uffe7': 'UnicodeCharacter0xffe7',
u'\u1c66': 'OlChikiLetterIh',
u'\u38e9': 'CjkUnifiedIdeograph38e9',
u'\u5968': 'CjkUnifiedIdeograph5968',
u'\u05eb': 'UnicodeCharacter0x5eb',
u'\u46ed': 'CjkUnifiedIdeograph46ed',
u'\uae72': 'HangulSyllableGgaelm',
u'\ueb74': 'UnicodeCharacter0xeb74',
u'\u97f7': 'CjkUnifiedIdeograph97f7',
u'\u3476': 'CjkUnifiedIdeograph3476',
u'\ud0f9': 'HangulSyllableTyag',
u'\u7178': 'CjkUnifiedIdeograph7178',
u'\u1dfb': 'UnicodeCharacter0x1dfb',
u'\u5efd': 'CjkUnifiedIdeograph5efd',
u'\u6401': 'CjkUnifiedIdeograph6401',
u'\u8384': 'CjkUnifiedIdeograph8384',
u'\ucc86': 'HangulSyllableCyaelm',
u'\u0988': 'BengaliLetterIi',
u'\ub50b': 'HangulSyllableDyis',
u'\u4a8a': 'CjkUnifiedIdeograph4a8a',
u'\uf60d': 'UnicodeCharacter0xf60d',
u'\u330f': 'SquareGanma',
u'\u7c11': 'CjkUnifiedIdeograph7c11',
u'\u9b94': 'CjkUnifiedIdeograph9b94',
u'\ue496': 'UnicodeCharacter0xe496',
u'\u2198': 'SouthEastArrow',
u'\u629a': 'CjkUnifiedIdeograph629a',
u'\u8e1d': 'CjkUnifiedIdeograph8e1d',
u'\ucb1f': 'HangulSyllableJjoelh',
u'\u1421': 'CanadianSyllabicsFinalBottomHalfRing',
u'\u5123': 'CjkUnifiedIdeograph5123',
u'\ub3a4': 'HangulSyllableDyeok',
u'\ufca6': 'ArabicLigatureThehWithMeemInitialForm',
u'\u39a8': 'CjkUnifiedIdeograph39a8',
u'\u7aaa': 'CjkUnifiedIdeograph7aaa',
u'\ua62d': 'UnicodeCharacter0xa62d',
u'\u47ac': 'CjkUnifiedIdeograph47ac',
u'\ue32f': 'UnicodeCharacter0xe32f',
u'\u2c31': 'GlagoliticSmallLetterBuky',
u'\u6933': 'CjkUnifiedIdeograph6933',
u'\u94b6': 'CjkUnifiedIdeograph94b6',
u'\ud1b8': 'HangulSyllableTok',
u'\u12ba': 'EthiopicSyllableKxi',
u'\ube3d': 'HangulSyllableBying',
u'\u5fbc': 'CjkUnifiedIdeograph5fbc',
u'\ufb3f': 'UnicodeCharacter0xfb3f',
u'\uc441': 'HangulSyllableSsoeng',
u'\u0143': 'LatinCapitalLetterNWithAcute',
u'\u4245': 'CjkUnifiedIdeograph4245',
u'\uacc6': 'HangulSyllableGyegg',
u'\ue9c8': 'UnicodeCharacter0xe9c8',
u'\u2aca': 'SupersetOfAboveAlmostEqualTo',
u'\u77cc': 'CjkUnifiedIdeograph77cc',
u'\u934f': 'CjkUnifiedIdeograph934f',
u'\udc51': 'UnicodeCharacter0xdc51',
u'\u1953': 'TaiLeLetterTsa',
u'\u5a55': 'CjkUnifiedIdeograph5a55',
u'\u81d8': 'CjkUnifiedIdeograph81d8',
u'\uc2da': 'HangulSyllableSyip',
u'\u0fdc': 'UnicodeCharacter0xfdc',
u'\uab5f': 'UnicodeCharacter0xab5f',
u'\u48de': 'CjkUnifiedIdeograph48de',
u'\uf461': 'UnicodeCharacter0xf461',
u'\u3163': 'HangulLetterI',
u'\u7265': 'CjkUnifiedIdeograph7265',
u'\u99e8': 'CjkUnifiedIdeograph99e8',
u'\udaea': 'UnicodeCharacter0xdaea',
u'\u27ec': 'MathematicalLeftWhiteTortoiseShellBracket',
u'\u60ee': 'CjkUnifiedIdeograph60ee',
u'\u8c71': 'CjkUnifiedIdeograph8c71',
u'\uc973': 'HangulSyllableJyud',
u'\u0a75': 'GurmukhiSignYakash',
u'\u5777': 'CjkUnifiedIdeograph5777',
u'\ub1f8': 'HangulSyllableNyom',
u'\uf2fa': 'UnicodeCharacter0xf2fa',
u'\u3ffc': 'CjkUnifiedIdeograph3ffc',
u'\u78fe': 'CjkUnifiedIdeograph78fe',
u'\u4600': 'CjkUnifiedIdeograph4600',
u'\u6283': 'CjkUnifiedIdeograph6283',
u'\u8502': 'CjkUnifiedIdeograph8502',
u'\ua185': 'YiSyllableNa',
u'\u0f06': 'TibetanMarkCaretYigMgoPhurShadMa',
u'\u2b89': 'UnicodeCharacter0x2b89',
u'\ud40c': 'HangulSyllablePwan',
u'\uf08f': 'UnicodeCharacter0xf08f',
u'\u5e10': 'CjkUnifiedIdeograph5e10',
u'\u7a93': 'CjkUnifiedIdeograph7a93',
u'\u9d12': 'CjkUnifiedIdeograph9d12',
u'\ub995': 'HangulSyllableRyinj',
u'\u2716': 'HeavyMultiplicationX',
u'\uc399': 'HangulSyllableSseng',
u'\u4d9d': 'CjkUnifiedIdeograph4d9d',
u'\uec1c': 'UnicodeCharacter0xec1c',
u'\u889f': 'CjkUnifiedIdeograph889f',
u'\u7620': 'CjkUnifiedIdeograph7620',
u'\u12a3': 'EthiopicSyllableGlottalAa',
u'\ub522': 'HangulSyllableDilp',
u'\u3f26': 'CjkUnifiedIdeograph3f26',
u'\udba9': 'UnicodeCharacter0xdba9',
u'\u65ad': 'CjkUnifiedIdeograph65ad',
u'\u842c': 'CjkUnifiedIdeograph842c',
u'\ua0af': 'YiSyllableMix',
u'\u0e30': 'ThaiCharacterSaraA',
u'\u2ab3': 'PrecedesAboveEqualsSign',
u'\ud736': 'HangulSyllableHyugg',
u'\uf3b9': 'UnicodeCharacter0xf3b9',
u'\u513a': 'CjkUnifiedIdeograph513a',
u'\u7dbd': 'CjkUnifiedIdeograph7dbd',
u'\u9c3c': 'CjkUnifiedIdeograph9c3c',
u'\ub8bf': 'HangulSyllableRoelh',
u'\u2640': 'FemaleSign',
u'\uc2c3': 'HangulSyllableSyigs',
u'\u4cc7': 'CjkUnifiedIdeograph4cc7',
u'\uef46': 'UnicodeCharacter0xef46',
u'\u8bc9': 'CjkUnifiedIdeograph8bc9',
u'\u694a': 'CjkUnifiedIdeograph694a',
u'\u15cd': 'CanadianSyllabicsCarrierRee',
u'\ub44c': 'HangulSyllableDyok',
u'\u3e50': 'CjkUnifiedIdeograph3e50',
u'\udad3': 'UnicodeCharacter0xdad3',
u'\u64d7': 'CjkUnifiedIdeograph64d7',
u'\u8756': 'CjkUnifiedIdeograph8756',
u'\ua3d9': 'YiSyllableRur',
u'\u015a': 'LatinCapitalLetterSWithAcute',
u'\u2ddd': 'EthiopicSyllableGye',
u'\ud660': 'HangulSyllableHwals',
u'\uf2e3': 'UnicodeCharacter0xf2e3',
u'\u5064': 'CjkUnifiedIdeograph5064',
u'\u7ce7': 'CjkUnifiedIdeograph7ce7',
u'\u9f66': 'CjkUnifiedIdeograph9f66',
u'\ubbe9': 'HangulSyllableMyilt',
u'\u196a': 'TaiLeLetterUe',
u'\uc5ed': 'HangulSyllableYeog',
u'\u4ff1': 'CjkUnifiedIdeograph4ff1',
u'\uee70': 'UnicodeCharacter0xee70',
u'\u8af3': 'CjkUnifiedIdeograph8af3',
u'\u6874': 'CjkUnifiedIdeograph6874',
u'\u14f7': 'CanadianSyllabicsWestcreeSwe',
u'\ub776': 'HangulSyllableDdij',
u'\u317a': 'HangulLetterSioskiyeok',
u'\uddfd': 'UnicodeCharacter0xddfd',
u'\ue701': 'UnicodeCharacter0xe701',
u'\u8680': 'CjkUnifiedIdeograph8680',
u'\u6105': 'CjkUnifiedIdeograph6105',
u'\x84': 'UnicodeCharacter0x84',
u'\uac07': 'HangulSyllableGad',
u'\u360b': 'CjkUnifiedIdeograph360b',
u'\uc98a': 'HangulSyllableJeugg',
u'\u538e': 'CjkUnifiedIdeograph538e',
u'\uff11': 'FullwidthDigitOne',
u'\u9e90': 'CjkUnifiedIdeograph9e90',
u'\u7915': 'CjkUnifiedIdeograph7915',
u'\u1894': 'MongolianLetterAliGaliSsa',
u'\uce1b': 'HangulSyllableCyuc',
u'\ue19a': 'UnicodeCharacter0xe19a',
u'\u481f': 'CjkUnifiedIdeograph481f',
u'\u6b9e': 'CjkUnifiedIdeograph6b9e',
u'\u9721': 'CjkUnifiedIdeograph9721',
u'\ub6a0': 'HangulSyllableDdun',
u'\u1125': 'HangulChoseongPieupssangsios',
u'\u30a4': 'KatakanaLetterI',
u'\ue62b': 'UnicodeCharacter0xe62b',
u'\uf9aa': 'CjkCompatibilityIdeographf9aa',
u'\u602f': 'CjkUnifiedIdeograph602f',
u'\u03ae': 'GreekSmallLetterEtaWithTonos',
u'\uaf31': 'HangulSyllableGgonj',
u'\u2935': 'ArrowPointingRightwardsThenCurvingDownwards',
u'\uc8b4': 'HangulSyllableJwaels',
u'\u52b8': 'CjkUnifiedIdeograph52b8',
u'\ufe3b': 'PresentationFormForVerticalLeftBlackLenticularBracket',
u'\u91ba': 'CjkUnifiedIdeograph91ba',
u'\u783f': 'CjkUnifiedIdeograph783f',
u'\u1bbe': 'UnicodeCharacter0x1bbe',
u'\uc145': 'HangulSyllableSelt',
u'\ue0c4': 'UnicodeCharacter0xe0c4',
u'\u4b49': 'CjkUnifiedIdeograph4b49',
u'\u6ac8': 'CjkUnifiedIdeograph6ac8',
u'\u964b': 'CjkUnifiedIdeograph964b',
u'\ua9ca': 'UnicodeCharacter0xa9ca',
u'\u104f': 'MyanmarSymbolGenitive',
u'\u33ce': 'SquareKmCapital',
u'\ud955': 'UnicodeCharacter0xd955',
u'\uf8d4': 'UnicodeCharacter0xf8d4',
u'\u6359': 'CjkUnifiedIdeograph6359',
u'\u02d8': 'Breve',
u'\uae5b': 'HangulSyllableGgalh',
u'\u285f': 'BraillePatternDots123457',
u'\ucbde': 'HangulSyllableJjeulm',
u'\u55e2': 'CjkUnifiedIdeograph55e2',
u'\uf165': 'UnicodeCharacter0xf165',
u'\u90e4': 'CjkUnifiedIdeograph90e4',
u'\u7b69': 'CjkUnifiedIdeograph7b69',
u'\u1ae8': 'UnicodeCharacter0x1ae8',
u'\uc06f': 'HangulSyllableBbeuc',
u'\ue3ee': 'UnicodeCharacter0xe3ee',
u'\u4a73': 'CjkUnifiedIdeograph4a73',
u'\u6df2': 'CjkUnifiedIdeograph6df2',
u'\u8975': 'CjkUnifiedIdeograph8975',
u'\ua8f4': 'UnicodeCharacter0xa8f4',
u'\u1379': 'EthiopicNumberEighty',
u'\u32f8': 'CircledKatakanaRu',
u'\ud87f': 'UnicodeCharacter0xd87f',
u'\ufbfe': 'ArabicLetterFarsiYehInitialForm',
u'\u8e06': 'CjkUnifiedIdeograph8e06',
u'\uaa89': 'UnicodeCharacter0xaa89',
u'\ucb08': 'HangulSyllableJjwaess',
u'\uf78b': 'UnicodeCharacter0xf78b',
u'\u080a': 'UnicodeCharacter0x80a',
u'\u348d': 'CjkUnifiedIdeograph348d',
u'\u550c': 'CjkUnifiedIdeograph550c',
u'\u718f': 'CjkUnifiedIdeograph718f',
u'\ua616': 'VaiSymbolNii',
u'\ue318': 'UnicodeCharacter0xe318',
u'\u8f9b': 'CjkUnifiedIdeograph8f9b',
u'\u201a': 'SingleLow9QuotationMark',
u'\ucc9d': 'HangulSyllableCeonj',
u'\u6d1c': 'CjkUnifiedIdeograph6d1c',
u'\u099f': 'BengaliLetterTta',
u'\u56a1': 'CjkUnifiedIdeograph56a1',
u'\ube26': 'HangulSyllableBeup',
u'\ufb28': 'HebrewLetterWideTav',
u'\ua7ab': 'UnicodeCharacter0xa7ab',
u'\u382a': 'CjkUnifiedIdeograph382a',
u'\ue4ad': 'UnicodeCharacter0xe4ad',
u'\u052c': 'UnicodeCharacter0x52c',
u'\u21af': 'DownwardsZigzagArrow',
u'\u422e': 'CjkUnifiedIdeograph422e',
u'\u6eb1': 'CjkUnifiedIdeograph6eb1',
u'\u9338': 'CjkUnifiedIdeograph9338',
u'\ubfbb': 'HangulSyllableBbyolb',
u'\ud03a': 'HangulSyllableKwinh',
u'\ufcbd': 'ArabicLigatureGhainWithMeemInitialForm',
u'\u1d3c': 'ModifierLetterCapitalO',
u'\u39bf': 'CjkUnifiedIdeograph39bf',
u'\u5a3e': 'CjkUnifiedIdeograph5a3e',
u'\u06c1': 'ArabicLetterHehGoal',
u'\u43c3': 'CjkUnifiedIdeograph43c3',
u'\uab48': 'UnicodeCharacter0xab48',
u'\ue84a': 'UnicodeCharacter0xe84a',
u'\u94cd': 'CjkUnifiedIdeograph94cd',
u'\u354c': 'CjkUnifiedIdeograph354c',
u'\ud1cf': 'HangulSyllableTwas',
u'\u724e': 'CjkUnifiedIdeograph724e',
u'\u1ed1': 'LatinSmallLetterOWithCircumflexAndAcute',
u'\u5bd3': 'CjkUnifiedIdeograph5bd3',
u'\u805a': 'CjkUnifiedIdeograph805a',
u'\uacdd': 'HangulSyllableGyet',
u'\ucd5c': 'HangulSyllableCoe',
u'\ue9df': 'UnicodeCharacter0xe9df',
u'\u0a5e': 'GurmukhiLetterFa',
u'\u36e1': 'CjkUnifiedIdeograph36e1',
u'\u5760': 'CjkUnifiedIdeograph5760',
u'\u73e3': 'CjkUnifiedIdeograph73e3',
u'\u986a': 'CjkUnifiedIdeograph986a',
u'\ue56c': 'UnicodeCharacter0xe56c',
u'\u81ef': 'CjkUnifiedIdeograph81ef',
u'\u226e': 'NotLessthan',
u'\ucef1': 'HangulSyllableKeolt',
u'\u6f70': 'CjkUnifiedIdeograph6f70',
u'\u0bf3': 'TamilDaySign',
u'\u48f5': 'CjkUnifiedIdeograph48f5',
u'\ub07a': 'HangulSyllableGgyip',
u'\ufd7c': 'ArabicLigatureFehWithKhahWithMeemFinalForm',
u'\u99ff': 'CjkUnifiedIdeograph99ff',
u'\u3a7e': 'CjkUnifiedIdeograph3a7e',
u'\u0780': 'ThaanaLetterHaa',
u'\ua303': 'YiSyllableNzyt',
u'\u4482': 'CjkUnifiedIdeograph4482',
u'\ue005': 'UnicodeCharacter0xe005',
u'\u2d07': 'GeorgianSmallLetterTan',
u'\u6a09': 'CjkUnifiedIdeograph6a09',
u'\u958c': 'CjkUnifiedIdeograph958c',
u'\ud28e': 'HangulSyllableTwilp',
u'\u1f90': 'GreekSmallLetterEtaWithPsiliAndYpogegrammeni',
u'\ubb13': 'HangulSyllableMoec',
u'\u5c92': 'CjkUnifiedIdeograph5c92',
u'\uf815': 'UnicodeCharacter0xf815',
u'\uc517': 'HangulSyllableSsyilb',
u'\u0219': 'LatinSmallLetterSWithCommaBelow',
u'\u4f1b': 'CjkUnifiedIdeograph4f1b',
u'\uad9c': 'HangulSyllableGweoss',
u'\uea9e': 'UnicodeCharacter0xea9e',
u'\u37a0': 'CjkUnifiedIdeograph37a0',
u'\u74a2': 'CjkUnifiedIdeograph74a2',
u'\u9025': 'CjkUnifiedIdeograph9025',
u'\udd27': 'UnicodeCharacter0xdd27',
u'\u1a29': 'UnicodeCharacter0x1a29',
u'\u672b': 'CjkUnifiedIdeograph672b',
u'\u82ae': 'CjkUnifiedIdeograph82ae',
u'\ucfb0': 'HangulSyllableKoel',
u'\u0cb2': 'KannadaLetterLa',
u'\ua835': 'UnicodeCharacter0xa835',
u'\u49b4': 'CjkUnifiedIdeograph49b4',
u'\uf537': 'UnicodeCharacter0xf537',
u'\u3239': 'ParenthesizedIdeographRepresent',
u'\u7f3b': 'CjkUnifiedIdeograph7f3b',
u'\u9abe': 'CjkUnifiedIdeograph9abe',
u'\ue7c0': 'UnicodeCharacter0xe7c0',
u'\u24c2': 'CircledLatinCapitalLetterM',
u'\u61c4': 'CjkUnifiedIdeograph61c4',
u'\u8d47': 'CjkUnifiedIdeograph8d47',
u'\uca49': 'HangulSyllableJjyaet',
u'\u174b': 'BuhidLetterMa',
u'\u544d': 'CjkUnifiedIdeograph544d',
u'\ub2ce': 'HangulSyllableNinh',
u'\uffd0': 'UnicodeCharacter0xffd0',
u'\u3cd2': 'CjkUnifiedIdeograph3cd2',
u'\u79d4': 'CjkUnifiedIdeograph79d4',
u'\ua557': 'VaiSyllableGba',
u'\u46d6': 'CjkUnifiedIdeograph46d6',
u'\ue259': 'UnicodeCharacter0xe259',
u'\u2f5b': 'KangxiRadicalFang',
u'\u6c5d': 'CjkUnifiedIdeograph6c5d',
u'\u97e0': 'CjkUnifiedIdeograph97e0',
u'\ud4e2': 'HangulSyllablePwij',
u'\u11e4': 'HangulJongseongPieupphieuph',
u'\ubd67': 'HangulSyllableByogs',
u'\u5ee6': 'CjkUnifiedIdeograph5ee6',
u'\ufa69': 'CjkCompatibilityIdeographfa69',
u'\uc76b': 'HangulSyllableYis',
u'\u046d': 'CyrillicSmallLetterIotifiedBigYus',
u'\u416f': 'CjkUnifiedIdeograph416f',
u'\uaff0': 'HangulSyllableGgwe',
u'\uecf2': 'UnicodeCharacter0xecf2',
u'\u29f4': 'Ruledelayed',
u'\u76f6': 'CjkUnifiedIdeograph76f6',
u'\u9279': 'CjkUnifiedIdeograph9279',
u'\udf7b': 'UnicodeCharacter0xdf7b',
u'\u1c7d': 'OlChikiAhad',
u'\u597f': 'CjkUnifiedIdeograph597f',
u'\u2981': 'ZNotationSpot',
u'\uce04': 'HangulSyllableCyu',
u'\ue287': 'UnicodeCharacter0xe287',
u'\u4408': 'CjkUnifiedIdeograph4408',
u'\u788b': 'CjkUnifiedIdeograph788b',
u'\u870a': 'CjkUnifiedIdeograph870a',
u'\ubb8d': 'HangulSyllableMwinj',
u'\u1d0e': 'LatinLetterSmallCapitalReversedN',
u'\u3191': 'IdeographicAnnotationReverseMark',
u'\ud614': 'HangulSyllableHyeoss',
u'\u8a97': 'CjkUnifiedIdeograph8a97',
u'\u6c18': 'CjkUnifiedIdeograph6c18',
u'\x9b': 'UnicodeCharacter0x9b',
u'\uaf1a': 'HangulSyllableGgyelm',
u'\u251e': 'BoxDrawingsUpHeavyAndRightDownLight',
u'\ud9a1': 'UnicodeCharacter0xd9a1',
u'\u5fa5': 'CjkUnifiedIdeograph5fa5',
u'\ufe24': 'CombiningMacronLeftHalf',
u'\u92a7': 'CjkUnifiedIdeograph92a7',
u'\u7428': 'CjkUnifiedIdeograph7428',
u'\u28ab': 'BraillePatternDots12468',
u'\ub72a': 'HangulSyllableDdeugg',
u'\ucd2e': 'HangulSyllableCwalm',
u'\ue1b1': 'UnicodeCharacter0xe1b1',
u'\u4332': 'CjkUnifiedIdeograph4332',
u'\u67b5': 'CjkUnifiedIdeograph67b5',
u'\u8634': 'CjkUnifiedIdeograph8634',
u'\ubab7': 'HangulSyllableMolh',
u'\u1c38': 'UnicodeCharacter0x1c38',
u'\u30bb': 'KatakanaLetterSe',
u'\ud53e': 'HangulSyllablePigg',
u'\u89c1': 'CjkUnifiedIdeograph89c1',
u'\u6b42': 'CjkUnifiedIdeograph6b42',
u'\u0fc5': 'TibetanSymbolRdoRje',
u'\uae44': 'HangulSyllableGiss',
u'\u2448': 'OcrDash',
u'\ud8cb': 'UnicodeCharacter0xd8cb',
u'\u5ecf': 'CjkUnifiedIdeograph5ecf',
u'\ufd4e': 'UnicodeCharacter0xfd4e',
u'\u91d1': 'CjkUnifiedIdeograph91d1',
u'\u7352': 'CjkUnifiedIdeograph7352',
u'\u17d5': 'KhmerSignBariyoosan',
u'\ub654': 'HangulSyllableDdwaels',
u'\ucc58': 'HangulSyllableCaess',
u'\ue0db': 'UnicodeCharacter0xe0db',
u'\u425c': 'CjkUnifiedIdeograph425c',
u'\u66df': 'CjkUnifiedIdeograph66df',
u'\u855e': 'CjkUnifiedIdeograph855e',
u'\ub9e1': 'HangulSyllableMat',
u'\u1b62': 'BalineseMusicalSymbolDeng',
u'\u3fe5': 'CjkUnifiedIdeograph3fe5',
u'\ud468': 'HangulSyllablePyols',
u'\u88eb': 'CjkUnifiedIdeograph88eb',
u'\u6a6c': 'CjkUnifiedIdeograph6a6c',
u'\u0eef': 'UnicodeCharacter0xeef',
u'\uad6e': 'HangulSyllableGugg',
u'\u2372': 'AplFunctionalSymbolUpCaretTilde',
u'\uc7f5': 'HangulSyllableJyaeb',
u'\u5df9': 'CjkUnifiedIdeograph5df9',
u'\ufc78': 'ArabicLigatureThehWithMeemFinalForm',
u'\u90fb': 'CjkUnifiedIdeograph90fb',
u'\u727c': 'CjkUnifiedIdeograph727c',
u'\u16ff': 'UnicodeCharacter0x16ff',
u'\ub57e': 'HangulSyllableDdyaj',
u'\u2c03': 'GlagoliticCapitalLetterGlagoli',
u'\ucb82': 'HangulSyllableJjwegg',
u'\u4186': 'CjkUnifiedIdeograph4186',
u'\ue509': 'UnicodeCharacter0xe509',
u'\u8488': 'CjkUnifiedIdeograph8488',
u'\u7b0d': 'CjkUnifiedIdeograph7b0d',
u'\u1a8c': 'UnicodeCharacter0x1a8c',
u'\ube0f': 'HangulSyllableBeugs',
u'\u3413': 'CjkUnifiedIdeograph3413',
u'\ud392': 'HangulSyllablePeoj',
u'\u4a17': 'CjkUnifiedIdeograph4a17',
u'\u6996': 'CjkUnifiedIdeograph6996',
u'\u8d19': 'CjkUnifiedIdeograph8d19',
u'\uac98': 'HangulSyllableGels',
u'\u031d': 'CombiningUpTackBelow',
u'\u229c': 'CircledEquals',
u'\udc23': 'UnicodeCharacter0xdc23',
u'\ufba2': 'ArabicLetterRnoonInitialForm',
u'\u5227': 'CjkUnifiedIdeograph5227',
u'\u71a6': 'CjkUnifiedIdeograph71a6',
u'\u9529': 'CjkUnifiedIdeograph9529',
u'\ub4a8': 'HangulSyllableDwin',
u'\u2b2d': 'WhiteHorizontalEllipse',
u'\ucaac': 'HangulSyllableJjyels',
u'\u40b0': 'CjkUnifiedIdeograph40b0',
u'\ue433': 'UnicodeCharacter0xe433',
u'\u83b2': 'CjkUnifiedIdeograph83b2',
u'\u7a37': 'CjkUnifiedIdeograph7a37',
u'\u19b6': 'NewTaiLueVowelSignAe',
u'\ubd39': 'HangulSyllableBwaelt',
u'\u333d': 'SquarePointo',
u'\ud2bc': 'HangulSyllableTeun',
u'\u4941': 'CjkUnifiedIdeograph4941',
u'\u68c0': 'CjkUnifiedIdeograph68c0',
u'\u8c43': 'CjkUnifiedIdeograph8c43',
u'\uabc2': 'UnicodeCharacter0xabc2',
u'\u0247': 'LatinSmallLetterEWithStroke',
u'\u21c6': 'LeftwardsArrowOverRightwardsArrow',
u'\udb4d': 'UnicodeCharacter0xdb4d',
u'\ufacc': 'CjkCompatibilityIdeographfacc',
u'\u5151': 'CjkUnifiedIdeograph5151',
u'\u70d0': 'CjkUnifiedIdeograph70d0',
u'\u9453': 'CjkUnifiedIdeograph9453',
u'\ub3d2': 'HangulSyllableDolp',
u'\u2a57': 'SlopingLargeOr',
u'\uc9d6': 'HangulSyllableJij',
u'\u4fda': 'CjkUnifiedIdeograph4fda',
u'\ue35d': 'UnicodeCharacter0xe35d',
u'\u82dc': 'CjkUnifiedIdeograph82dc',
u'\u7961': 'CjkUnifiedIdeograph7961',
u'\u18e0': 'UnicodeCharacter0x18e0',
u'\ubc63': 'HangulSyllableByac',
u'\u3267': 'CircledHangulIeung',
u'\ud1e6': 'HangulSyllableTwaelp',
u'\u486b': 'CjkUnifiedIdeograph486b',
u'\u57ea': 'CjkUnifiedIdeograph57ea',
u'\u8b6d': 'CjkUnifiedIdeograph8b6d',
u'\uaaec': 'UnicodeCharacter0xaaec',
u'\u0171': 'LatinSmallLetterUWithDoubleAcute',
u'\u20f0': 'CombiningAsteriskAbove',
u'\uda77': 'UnicodeCharacter0xda77',
u'\uf9f6': 'CjkCompatibilityIdeographf9f6',
u'\u507b': 'CjkUnifiedIdeograph507b',
u'\u7ffa': 'CjkUnifiedIdeograph7ffa',
u'\u937d': 'CjkUnifiedIdeograph937d',
u'\ub2fc': 'HangulSyllableDak',
u'\ua881': 'SaurashtraSignVisarga',
u'\uc900': 'HangulSyllableJun',
u'\ued83': 'UnicodeCharacter0xed83',
u'\u0a02': 'GurmukhiSignBindi',
u'\u2e85': 'CjkRadicalPerson',
u'\u4f04': 'CjkUnifiedIdeograph4f04',
u'\u6387': 'CjkUnifiedIdeograph6387',
u'\u9c0e': 'CjkUnifiedIdeograph9c0e',
u'\ub091': 'HangulSyllableGging',
u'\ud110': 'HangulSyllableTyak',
u'\uf593': 'UnicodeCharacter0xf593',
u'\u1212': 'EthiopicSyllableHhi',
u'\u3695': 'CjkUnifiedIdeograph3695',
u'\u5714': 'CjkUnifiedIdeograph5714',
u'\u0b97': 'UnicodeCharacter0xb97',
u'\u4c99': 'CjkUnifiedIdeograph4c99',
u'\ua41e': 'YiSyllableJjie',
u'\uf920': 'CjkCompatibilityIdeographf920',
u'\u9da3': 'CjkUnifiedIdeograph9da3',
u'\u3a22': 'CjkUnifiedIdeograph3a22',
u'\udea5': 'UnicodeCharacter0xdea5',
u'\u7f24': 'CjkUnifiedIdeograph7f24',
u'\u13a7': 'CherokeeLetterKa',
u'\u54a9': 'CjkUnifiedIdeograph54a9',
u'\u8130': 'CjkUnifiedIdeograph8130',
u'\ua5b3': 'VaiSyllableYu',
u'\uc232': 'HangulSyllableSup',
u'\ue6b5': 'UnicodeCharacter0xe6b5',
u'\u0734': 'SyriacZqaphaBelow',
u'\u3bb7': 'CjkUnifiedIdeograph3bb7',
u'\u5836': 'CjkUnifiedIdeograph5836',
u'\u7cb9': 'CjkUnifiedIdeograph7cb9',
u'\ua940': 'RejangLetterWa',
u'\uea42': 'UnicodeCharacter0xea42',
u'\u8ec5': 'CjkUnifiedIdeograph8ec5',
u'\u2f44': 'KangxiRadicalAxe',
u'\uc3c7': 'HangulSyllableSsyelb',
u'\u6046': 'CjkUnifiedIdeograph6046',
u'\u04c9': 'CyrillicCapitalLetterEnWithTail',
u'\u59cb': 'CjkUnifiedIdeograph59cb',
u'\ub150': 'HangulSyllableNyeom',
u'\uf252': 'UnicodeCharacter0xf252',
u'\u96d5': 'CjkUnifiedIdeograph96d5',
u'\u3754': 'CjkUnifiedIdeograph3754',
u'\uebd7': 'UnicodeCharacter0xebd7',
u'\u0856': 'UnicodeCharacter0x856',
u'\u2cd9': 'CopticSmallLetterOldCopticDja',
u'\u4d58': 'CjkUnifiedIdeograph4d58',
u'\u61db': 'CjkUnifiedIdeograph61db',
u'\u9a62': 'CjkUnifiedIdeograph9a62',
u'\ubee5': 'HangulSyllableBbeong',
u'\udf64': 'UnicodeCharacter0xdf64',
u'\uf3e7': 'UnicodeCharacter0xf3e7',
u'\u1066': 'MyanmarLetterWesternPwoKarenPwa',
u'\u34e9': 'CjkUnifiedIdeograph34e9',
u'\u5568': 'CjkUnifiedIdeograph5568',
u'\u09eb': 'BengaliDigitFive',
u'\u4aed': 'CjkUnifiedIdeograph4aed',
u'\ua272': 'YiSyllableHax',
u'\ue774': 'UnicodeCharacter0xe774',
u'\u9bf7': 'CjkUnifiedIdeograph9bf7',
u'\u3876': 'CjkUnifiedIdeograph3876',
u'\udcf9': 'UnicodeCharacter0xdcf9',
u'\u7d78': 'CjkUnifiedIdeograph7d78',
u'\u11fb': 'UnicodeCharacter0x11fb',
u'\u52fd': 'CjkUnifiedIdeograph52fd',
u'\u6801': 'CjkUnifiedIdeograph6801',
u'\u8f84': 'CjkUnifiedIdeograph8f84',
u'\uc086': 'HangulSyllableBbyibs',
u'\u0588': 'UnicodeCharacter0x588',
u'\ub90b': 'HangulSyllableRweod',
u'\u468a': 'CjkUnifiedIdeograph468a',
u'\ufa0d': 'CjkCompatibilityIdeographfa0d',
u'\u3f0f': 'CjkUnifiedIdeograph3f0f',
u'\u7011': 'CjkUnifiedIdeograph7011',
u'\u9794': 'CjkUnifiedIdeograph9794',
u'\ue896': 'UnicodeCharacter0xe896',
u'\u2d98': 'UnicodeCharacter0x2d98',
u'\u6e9a': 'CjkUnifiedIdeograph6e9a',
u'\u821d': 'CjkUnifiedIdeograph821d',
u'\uc71f': 'HangulSyllableWih',
u'\u1821': 'MongolianLetterE',
u'\u5d23': 'CjkUnifiedIdeograph5d23',
u'\ubfa4': 'HangulSyllableBboem',
u'\uf0a6': 'UnicodeCharacter0xf0a6',
u'\u35a8': 'CjkUnifiedIdeograph35a8',
u'\u76aa': 'CjkUnifiedIdeograph76aa',
u'\uaa2d': 'ChamVowelSignU',
u'\u4bac': 'CjkUnifiedIdeograph4bac',
u'\uef2f': 'UnicodeCharacter0xef2f',
u'\u2031': 'PerTenThousandSign',
u'\u6533': 'CjkUnifiedIdeograph6533',
u'\u98b6': 'CjkUnifiedIdeograph98b6',
u'\uddb8': 'UnicodeCharacter0xddb8',
u'\u1eba': 'LatinCapitalLetterEWithHookAbove',
u'\ub23d': 'HangulSyllableNweg',
u'\u53bc': 'CjkUnifiedIdeograph53bc',
u'\uf73f': 'UnicodeCharacter0xf73f',
u'\uc841': 'HangulSyllableJyeolg',
u'\u0d43': 'MalayalamVowelSignVocalicR',
u'\u4e45': 'CjkUnifiedIdeograph4e45',
u'\ua0c6': 'YiSyllableMup',
u'\ue5c8': 'UnicodeCharacter0xe5c8',
u'\u26ca': 'UnicodeCharacter0x26ca',
u'\u7bcc': 'CjkUnifiedIdeograph7bcc',
u'\u9f4f': 'CjkUnifiedIdeograph9f4f',
u'\ud051': 'HangulSyllableKyug',
u'\u1553': 'CanadianSyllabicsFe',
u'\u5655': 'CjkUnifiedIdeograph5655',
u'\u8dd8': 'CjkUnifiedIdeograph8dd8',
u'\uceda': 'HangulSyllableKyaebs',
u'\u03dc': 'GreekLetterDigamma',
u'\ua75f': 'LatinSmallLetterVWithDiagonalStroke',
u'\u44de': 'CjkUnifiedIdeograph44de',
u'\uf861': 'UnicodeCharacter0xf861',
u'\u3d63': 'CjkUnifiedIdeograph3d63',
u'\u7e65': 'CjkUnifiedIdeograph7e65',
u'\u95e8': 'CjkUnifiedIdeograph95e8',
u'\ud6ea': 'HangulSyllableHweolm',
u'\u2bec': 'UnicodeCharacter0x2bec',
u'\u6cee': 'CjkUnifiedIdeograph6cee',
u'\u8071': 'CjkUnifiedIdeograph8071',
u'\uc573': 'HangulSyllableAes',
u'\u0675': 'ArabicLetterHighHamzaAlef',
u'\u5b77': 'CjkUnifiedIdeograph5b77',
u'\ubdf8': 'HangulSyllableByul',
u'\ufefa': 'ArabicLigatureLamWithAlefWithHamzaBelowFinalForm',
u'\u33fc': 'IdeographicTelegraphSymbolForDayTwentynine',
u'\u74fe': 'CjkUnifiedIdeograph74fe',
u'\u4a00': 'CjkUnifiedIdeograph4a00',
u'\u6e83': 'CjkUnifiedIdeograph6e83',
u'\u8902': 'CjkUnifiedIdeograph8902',
u'\uad85': 'HangulSyllableGut',
u'\u0306': 'CombiningBreve',
u'\u2789': 'DingbatCircledSansserifNumberTen',
u'\ud80c': 'UnicodeCharacter0xd80c',
u'\ufc8f': 'ArabicLigatureNoonWithYehFinalForm',
u'\u5210': 'CjkUnifiedIdeograph5210',
u'\u7693': 'CjkUnifiedIdeograph7693',
u'\u9112': 'CjkUnifiedIdeograph9112',
u'\ub595': 'HangulSyllableDdyaeb',
u'\u2b16': 'DiamondWithLeftHalfBlack',
u'\ucf99': 'HangulSyllableKwaelt',
u'\u419d': 'CjkUnifiedIdeograph419d',
u'\ue01c': 'UnicodeCharacter0xe01c',
u'\u849f': 'CjkUnifiedIdeograph849f',
u'\u7a20': 'CjkUnifiedIdeograph7a20',
u'\u1ea3': 'LatinSmallLetterAWithHookAbove',
u'\ub922': 'HangulSyllableRwegg',
u'\u3326': 'SquareDoru',
u'\ud7a9': 'UnicodeCharacter0xd7a9',
u'\u69ad': 'CjkUnifiedIdeograph69ad',
u'\u882c': 'CjkUnifiedIdeograph882c',
u'\uacaf': 'HangulSyllableGyeod',
u'\u0230': 'LatinCapitalLetterOWithDotAboveAndMacron',
u'\u26b3': 'Ceres',
u'\udb36': 'UnicodeCharacter0xdb36',
u'\uffb9': 'HalfwidthHangulLetterSsangcieuc',
u'\u5d3a': 'CjkUnifiedIdeograph5d3a',
u'\u71bd': 'CjkUnifiedIdeograph71bd',
u'\u903c': 'CjkUnifiedIdeograph903c',
u'\ub4bf': 'HangulSyllableDwih',
u'\u2a40': 'IntersectionWithDot',
u'\ucec3': 'HangulSyllableKyac',
u'\u40c7': 'CjkUnifiedIdeograph40c7',
u'\ue346': 'UnicodeCharacter0xe346',
u'\u87c9': 'CjkUnifiedIdeograph87c9',
u'\u654a': 'CjkUnifiedIdeograph654a',
u'\u19cd': 'UnicodeCharacter0x19cd',
u'\ub84c': 'HangulSyllableRyels',
u'\u3250': 'PartnershipSign',
u'\ud6d3': 'HangulSyllableHulh',
u'\u68d7': 'CjkUnifiedIdeograph68d7',
u'\u8b56': 'CjkUnifiedIdeograph8b56',
u'\uafd9': 'HangulSyllableGgweonj',
u'\u0d5a': 'UnicodeCharacter0xd5a',
u'\u21dd': 'RightwardsSquiggleArrow',
u'\uda60': 'UnicodeCharacter0xda60',
u'\ufee3': 'ArabicLetterMeemInitialForm',
u'\u5c64': 'CjkUnifiedIdeograph5c64',
u'\u70e7': 'CjkUnifiedIdeograph70e7',
u'\u9366': 'CjkUnifiedIdeograph9366',
u'\ub7e9': 'HangulSyllableRyaet',
u'\u156a': 'CanadianSyllabicsTh',
u'\uc9ed': 'HangulSyllableJjab',
u'\u43f1': 'CjkUnifiedIdeograph43f1',
u'\ue270': 'UnicodeCharacter0xe270',
u'\u86f3': 'CjkUnifiedIdeograph86f3',
u'\u6474': 'CjkUnifiedIdeograph6474',
u'\u18f7': 'UnicodeCharacter0x18f7',
u'\ubb76': 'HangulSyllableMwelm',
u'\u3d7a': 'CjkUnifiedIdeograph3d7a',
u'\ud1fd': 'HangulSyllableToelg',
u'\ueb01': 'UnicodeCharacter0xeb01',
u'\u8a80': 'CjkUnifiedIdeograph8a80',
u'\u6d05': 'CjkUnifiedIdeograph6d05',
u'\u0c84': 'UnicodeCharacter0xc84',
u'\ua007': 'YiSyllableIep',
u'\u3a0b': 'CjkUnifiedIdeograph3a0b',
u'\uc58a': 'HangulSyllableYalp',
u'\u5f8e': 'CjkUnifiedIdeograph5f8e',
u'\uf311': 'UnicodeCharacter0xf311',
u'\u9290': 'CjkUnifiedIdeograph9290',
u'\u7515': 'CjkUnifiedIdeograph7515',
u'\u1494': 'CanadianSyllabicsCwi',
u'\uc21b': 'HangulSyllableSugs',
u'\ued9a': 'UnicodeCharacter0xed9a',
u'\u441f': 'CjkUnifiedIdeograph441f',
u'\u679e': 'CjkUnifiedIdeograph679e',
u'\u9b21': 'CjkUnifiedIdeograph9b21',
u'\ubaa0': 'HangulSyllableMyess',
u'\u1d25': 'LatinLetterAin',
u'\u3ca4': 'CjkUnifiedIdeograph3ca4',
u'\uea2b': 'UnicodeCharacter0xea2b',
u'\uf5aa': 'UnicodeCharacter0xf5aa',
u'\u6c2f': 'CjkUnifiedIdeograph6c2f',
u'\u0fae': 'TibetanSubjoinedLetterZha',
u'\ua331': 'YiSyllableSsat',
u'\u2535': 'BoxDrawingsLeftHeavyAndRightUpLight',
u'\uc4b4': 'HangulSyllableSswek',
u'\u5eb8': 'CjkUnifiedIdeograph5eb8',
u'\uf23b': 'UnicodeCharacter0xf23b',
u'\u9dba': 'CjkUnifiedIdeograph9dba',
u'\u743f': 'CjkUnifiedIdeograph743f',
u'\u17be': 'KhmerVowelSignOe',
u'\ucd45': 'HangulSyllableCwaenj',
u'\uecc4': 'UnicodeCharacter0xecc4',
u'\u4749': 'CjkUnifiedIdeograph4749',
u'\u66c8': 'CjkUnifiedIdeograph66c8',
u'\u9a4b': 'CjkUnifiedIdeograph9a4b',
u'\ua5ca': 'VaiSyllableVo',
u'\u1c4f': 'LepchaLetterDda',
u'\u3fce': 'CjkUnifiedIdeograph3fce',
u'\ud555': 'HangulSyllablePit',
u'\uf4d4': 'UnicodeCharacter0xf4d4',
u'\u6f59': 'CjkUnifiedIdeograph6f59',
u'\u0ed8': 'LaoDigitEight',
u'\ua25b': 'YiSyllableHxe',
u'\u245f': 'UnicodeCharacter0x245f',
u'\uc7de': 'HangulSyllableJyaj',
u'\u59e2': 'CjkUnifiedIdeograph59e2',
u'\ufd65': 'ArabicLigatureSadWithHahWithHahInitialForm',
u'\u9ce4': 'CjkUnifiedIdeograph9ce4',
u'\u7769': 'CjkUnifiedIdeograph7769',
u'\u16e8': 'RunicLetterIcelandicyr',
u'\ucc6f': 'HangulSyllableCyalh',
u'\uefee': 'UnicodeCharacter0xefee',
u'\u4673': 'CjkUnifiedIdeograph4673',
u'\u61f2': 'CjkUnifiedIdeograph61f2',
u'\u8575': 'CjkUnifiedIdeograph8575',
u'\ua4f4': 'UnicodeCharacter0xa4f4',
u'\u1f79': 'GreekSmallLetterOmicronWithOxia',
u'\u3ef8': 'CjkUnifiedIdeograph3ef8',
u'\ud47f': 'HangulSyllablePud',
u'\uf7fe': 'UnicodeCharacter0xf7fe',
u'\u8206': 'CjkUnifiedIdeograph8206',
u'\ua689': 'CyrillicSmallLetterDzze',
u'\uc708': 'HangulSyllableWin',
u'\ufb8b': 'ArabicLetterJehFinalForm',
u'\u040a': 'CyrillicCapitalLetterNje',
u'\u388d': 'CjkUnifiedIdeograph388d',
u'\u590c': 'CjkUnifiedIdeograph590c',
u'\u7d8f': 'CjkUnifiedIdeograph7d8f',
u'\uaa16': 'ChamLetterDha',
u'\uef18': 'UnicodeCharacter0xef18',
u'\u839b': 'CjkUnifiedIdeograph839b',
u'\u2c1a': 'GlagoliticCapitalLetterPe',
u'\uc09d': 'HangulSyllableBbilt',
u'\u611c': 'CjkUnifiedIdeograph611c',
u'\u059f': 'HebrewAccentQarneyPara',
u'\u5aa1': 'CjkUnifiedIdeograph5aa1',
u'\ub226': 'HangulSyllableNweonh',
u'\uf728': 'UnicodeCharacter0xf728',
u'\uabab': 'UnicodeCharacter0xabab',
u'\u342a': 'CjkUnifiedIdeograph342a',
u'\ue8ad': 'UnicodeCharacter0xe8ad',
u'\u092c': 'DevanagariLetterBa',
u'\u2daf': 'UnicodeCharacter0x2daf',
u'\u4e2e': 'CjkUnifiedIdeograph4e2e',
u'\u62b1': 'CjkUnifiedIdeograph62b1',
u'\u9f38': 'CjkUnifiedIdeograph9f38',
u'\ub3bb': 'HangulSyllableDyes',
u'\udc3a': 'UnicodeCharacter0xdc3a',
u'\uf0bd': 'UnicodeCharacter0xf0bd',
u'\u113c': 'HangulChoseongChitueumsios',
u'\u35bf': 'CjkUnifiedIdeograph35bf',
u'\u563e': 'CjkUnifiedIdeograph563e',
u'\u0ac1': 'GujaratiVowelSignU',
u'\u4fc3': 'CjkUnifiedIdeograph4fc3',
u'\ua748': 'LatinCapitalLetterLWithHighStroke',
u'\ue44a': 'UnicodeCharacter0xe44a',
u'\u98cd': 'CjkUnifiedIdeograph98cd',
u'\u394c': 'CjkUnifiedIdeograph394c',
u'\uddcf': 'UnicodeCharacter0xddcf',
u'\u7e4e': 'CjkUnifiedIdeograph7e4e',
u'\u12d1': 'EthiopicSyllablePharyngealU',
u'\u57d3': 'CjkUnifiedIdeograph57d3',
u'\u8c5a': 'CjkUnifiedIdeograph8c5a',
u'\ua0dd': 'YiSyllableFur',
u'\uc15c': 'HangulSyllableSyeol',
u'\ue5df': 'UnicodeCharacter0xe5df',
u'\u065e': 'ArabicFathaWithTwoDots',
u'\u3ae1': 'CjkUnifiedIdeograph3ae1',
u'\u5b60': 'CjkUnifiedIdeograph5b60',
u'\u7fe3': 'CjkUnifiedIdeograph7fe3',
u'\u946a': 'CjkUnifiedIdeograph946a',
u'\ue96c': 'UnicodeCharacter0xe96c',
u'\u8def': 'CjkUnifiedIdeograph8def',
u'\u2e6e': 'UnicodeCharacter0x2e6e',
u'\uc2f1': 'HangulSyllableSing',
u'\u6370': 'CjkUnifiedIdeograph6370',
u'\u07f3': 'NkoCombiningDoubleDotAbove',
u'\u44f5': 'CjkUnifiedIdeograph44f5',
u'\ubc7a': 'HangulSyllableByaebs',
u'\uf17c': 'UnicodeCharacter0xf17c',
u'\u95ff': 'CjkUnifiedIdeograph95ff',
u'\u367e': 'CjkUnifiedIdeograph367e',
u'\u0b80': 'UnicodeCharacter0xb80',
u'\uaf03': 'HangulSyllableGgyeolh',
u'\u4882': 'CjkUnifiedIdeograph4882',
u'\uec05': 'UnicodeCharacter0xec05',
u'\u2107': 'EulerConstant',
u'\u6609': 'CjkUnifiedIdeograph6609',
u'\u998c': 'CjkUnifiedIdeograph998c',
u'\ude8e': 'UnicodeCharacter0xde8e',
u'\u1390': 'EthiopicTonalMarkYizet',
u'\ub713': 'HangulSyllableDdyud',
u'\u5092': 'CjkUnifiedIdeograph5092',
u'\uf415': 'UnicodeCharacter0xf415',
u'\uc917': 'HangulSyllableJuh',
u'\u0e19': 'ThaiCharacterNoNu',
u'\u431b': 'CjkUnifiedIdeograph431b',
u'\ua19c': 'YiSyllableHlie',
u'\ue69e': 'UnicodeCharacter0xe69e',
u'\u3ba0': 'CjkUnifiedIdeograph3ba0',
u'\u78a2': 'CjkUnifiedIdeograph78a2',
u'\u9c25': 'CjkUnifiedIdeograph9c25',
u'\ud127': 'HangulSyllableTyaes',
u'\u1629': 'CanadianSyllabicsCarrierDlo',
u'\u6b2b': 'CjkUnifiedIdeograph6b2b',
u'\u8eae': 'CjkUnifiedIdeograph8eae',
u'\uc3b0': 'HangulSyllableSsyeom',
u'\xb2': 'SuperscriptTwo',
u'\ua435': 'YiSyllableNjiet',
u'\u45b4': 'CjkUnifiedIdeograph45b4',
u'\uf937': 'CjkCompatibilityIdeographf937',
u'\u3e39': 'CjkUnifiedIdeograph3e39',
u'\u733b': 'CjkUnifiedIdeograph733b',
u'\u96be': 'CjkUnifiedIdeograph96be',
u'\uebc0': 'UnicodeCharacter0xebc0',
u'\u28c2': 'BraillePatternDots278',
u'\u6dc4': 'CjkUnifiedIdeograph6dc4',
u'\u8147': 'CjkUnifiedIdeograph8147',
u'\uc649': 'HangulSyllableWalg',
u'\u1b4b': 'BalineseLetterAsyuraSasak',
u'\u584d': 'CjkUnifiedIdeograph584d',
u'\ubece': 'HangulSyllableBbyaep',
u'\uf3d0': 'UnicodeCharacter0xf3d0',
u'\u30d2': 'KatakanaLetterHi',
u'\u75d4': 'CjkUnifiedIdeograph75d4',
u'\ua957': 'UnicodeCharacter0xa957',
u'\u4ad6': 'CjkUnifiedIdeograph4ad6',
u'\uee59': 'UnicodeCharacter0xee59',
u'\u235b': 'AplFunctionalSymbolJotUnderbar',
u'\u605d': 'CjkUnifiedIdeograph605d',
u'\u9be0': 'CjkUnifiedIdeograph9be0',
u'\ud8e2': 'UnicodeCharacter0xd8e2',
u'\u1de4': 'CombiningLatinSmallLetterS',
u'\ub167': 'HangulSyllableNyelb',
u'\u52e6': 'CjkUnifiedIdeograph52e6',
u'\uf669': 'UnicodeCharacter0xf669',
u'\ucb6b': 'HangulSyllableJjweod',
u'\u086d': 'UnicodeCharacter0x86d',
u'\u4d6f': 'CjkUnifiedIdeograph4d6f',
u'\ua3f0': 'YiSyllableJut',
u'\ue0f2': 'UnicodeCharacter0xe0f2',
u'\u25f4': 'WhiteCircleWithUpperLeftQuadrant',
u'\u7af6': 'CjkUnifiedIdeograph7af6',
u'\u9e79': 'CjkUnifiedIdeograph9e79',
u'\ud37b': 'HangulSyllablePyaeh',
u'\u107d': 'MyanmarLetterShanPha',
u'\u557f': 'CjkUnifiedIdeograph557f',
u'\u2d81': 'EthiopicSyllableMoa',
u'\uca04': 'HangulSyllableJjaels',
u'\ue687': 'UnicodeCharacter0xe687',
u'\u4008': 'CjkUnifiedIdeograph4008',
u'\u7c8b': 'CjkUnifiedIdeograph7c8b',
u'\u830a': 'CjkUnifiedIdeograph830a',
u'\ubf8d': 'HangulSyllableBbwaeng',
u'\u190e': 'LimbuLetterDha',
u'\u3591': 'CjkUnifiedIdeograph3591',
u'\ud214': 'HangulSyllableTyon',
u'\u8e97': 'CjkUnifiedIdeograph8e97',
u'\u6818': 'CjkUnifiedIdeograph6818',
u'\u049b': 'CyrillicSmallLetterKaWithDescender',
u'\uab1a': 'UnicodeCharacter0xab1a',
u'\u211e': 'PrescriptionTake',
u'\udda1': 'UnicodeCharacter0xdda1',
u'\u5ba5': 'CjkUnifiedIdeograph5ba5',
u'\ufa24': 'CjkCompatibilityIdeographfa24',
u'\u96a7': 'CjkUnifiedIdeograph96a7',
u'\u7028': 'CjkUnifiedIdeograph7028',
u'\u2cab': 'CopticSmallLetterFi',
u'\ub32a': 'HangulSyllableDyalp',
u'\uc92e': 'HangulSyllableJweoj',
u'\ue5b1': 'UnicodeCharacter0xe5b1',
u'\u4732': 'CjkUnifiedIdeograph4732',
u'\u63b5': 'CjkUnifiedIdeograph63b5',
u'\u8234': 'CjkUnifiedIdeograph8234',
u'\ubeb7': 'HangulSyllableBbyaegs',
u'\u1838': 'MongolianLetterWa',
u'\u34bb': 'CjkUnifiedIdeograph34bb',
u'\ud13e': 'HangulSyllableTeolp',
u'\u8dc1': 'CjkUnifiedIdeograph8dc1',
u'\u6f42': 'CjkUnifiedIdeograph6f42',
u'\u0bc5': 'UnicodeCharacter0xbc5',
u'\uaa44': 'ChamLetterFinalCh',
u'\u2048': 'QuestionExclamationMark',
u'\udccb': 'UnicodeCharacter0xdccb',
u'\u5acf': 'CjkUnifiedIdeograph5acf',
u'\uf94e': 'CjkCompatibilityIdeographf94e',
u'\u95d1': 'CjkUnifiedIdeograph95d1',
u'\u7752': 'CjkUnifiedIdeograph7752',
u'\u13d5': 'CherokeeLetterDe',
u'\ub254': 'HangulSyllableNwek',
u'\uc858': 'HangulSyllableJyen',
u'\ue4db': 'UnicodeCharacter0xe4db',
u'\u465c': 'CjkUnifiedIdeograph465c',
u'\u62df': 'CjkUnifiedIdeograph62df',
u'\u815e': 'CjkUnifiedIdeograph815e',
u'\ubde1': 'HangulSyllableBwilt',
u'\u1f62': 'GreekSmallLetterOmegaWithPsiliAndVaria',
u'\u3be5': 'CjkUnifiedIdeograph3be5',
u'\ud068': 'HangulSyllableKyuk',
u'\u8ceb': 'CjkUnifiedIdeograph8ceb',
u'\u6e6c': 'CjkUnifiedIdeograph6e6c',
u'\u0aef': 'GujaratiDigitNine',
u'\ua96e': 'UnicodeCharacter0xa96e',
u'\u2772': 'LightLeftTortoiseShellBracketOrnament',
u'\uc3f5': 'HangulSyllableSswag',
u'\u59f9': 'CjkUnifiedIdeograph59f9',
u'\uf878': 'UnicodeCharacter0xf878',
u'\u94fb': 'CjkUnifiedIdeograph94fb',
u'\u767c': 'CjkUnifiedIdeograph767c',
u'\u12ff': 'EthiopicSyllableDdwa',
u'\ub17e': 'HangulSyllableNonh',
u'\u2803': 'BraillePatternDots12',
u'\ucf82': 'HangulSyllableKwabs',
u'\u4586': 'CjkUnifiedIdeograph4586',
u'\ue109': 'UnicodeCharacter0xe109',
u'\u8088': 'CjkUnifiedIdeograph8088',
u'\u7f0d': 'CjkUnifiedIdeograph7f0d',
u'\u1e8c': 'LatinCapitalLetterXWithDiaeresis',
u'\uba0f': 'HangulSyllableMyalh',
u'\u3013': 'GetaMark',
u'\ud792': 'HangulSyllableHilm',
u'\u4e17': 'CjkUnifiedIdeograph4e17',
u'\u6d96': 'CjkUnifiedIdeograph6d96',
u'\u8919': 'CjkUnifiedIdeograph8919',
u'\ua898': 'SaurashtraLetterCha',
u'\u071d': 'SyriacLetterYudh',
u'\u269c': 'Fleurdelis',
u'\ud823': 'UnicodeCharacter0xd823',
u'\uffa2': 'HalfwidthHangulLetterSsangkiyeok',
u'\u5627': 'CjkUnifiedIdeograph5627',
u'\u75a6': 'CjkUnifiedIdeograph75a6',
u'\u9129': 'CjkUnifiedIdeograph9129',
u'\ub0a8': 'HangulSyllableNam',
u'\u2f2d': 'KangxiRadicalMountain',
u'\uceac': 'HangulSyllableKya',
u'\u44b0': 'CjkUnifiedIdeograph44b0',
u'\ue033': 'UnicodeCharacter0xe033',
u'\u87b2': 'CjkUnifiedIdeograph87b2',
u'\u7e37': 'CjkUnifiedIdeograph7e37',
u'\u1db6': 'ModifierLetterSmallUBar',
u'\ub939': 'HangulSyllableRwet',
u'\u373d': 'CjkUnifiedIdeograph373d',
u'\ud6bc': 'HangulSyllableHyoss',
u'\u4d41': 'CjkUnifiedIdeograph4d41',
u'\u6cc0': 'CjkUnifiedIdeograph6cc0',
u'\u8843': 'CjkUnifiedIdeograph8843',
u'\uafc2': 'HangulSyllableGgulm',
u'\u0647': 'ArabicLetterHeh',
u'\u25c6': 'BlackDiamond',
u'\udf4d': 'UnicodeCharacter0xdf4d',
u'\ufecc': 'ArabicLetterAinMedialForm',
u'\u5551': 'CjkUnifiedIdeograph5551',
u'\u74d0': 'CjkUnifiedIdeograph74d0',
u'\u9053': 'CjkUnifiedIdeograph9053',
u'\ub7d2': 'HangulSyllableRyaegg',
u'\u2e57': 'UnicodeCharacter0x2e57',
u'\ucdd6': 'HangulSyllableCwelm',
u'\u4bda': 'CjkUnifiedIdeograph4bda',
u'\ue75d': 'UnicodeCharacter0xe75d',
u'\u86dc': 'CjkUnifiedIdeograph86dc',
u'\u7d61': 'CjkUnifiedIdeograph7d61',
u'\u1ce0': 'UnicodeCharacter0x1ce0',
u'\ub863': 'HangulSyllableRod',
u'\u3667': 'CjkUnifiedIdeograph3667',
u'\ud5e6': 'HangulSyllableHegg',
u'\u4c6b': 'CjkUnifiedIdeograph4c6b',
u'\u53ea': 'CjkUnifiedIdeograph53ea',
u'\u8f6d': 'CjkUnifiedIdeograph8f6d',
u'\uaeec': 'HangulSyllableGgess',
u'\u0571': 'ArmenianSmallLetterJa',
u'\u24f0': 'NegativeCircledNumberSixteen',
u'\ude77': 'UnicodeCharacter0xde77',
u'\ufdf6': 'ArabicLigatureRasoulIsolatedForm',
u'\u547b': 'CjkUnifiedIdeograph547b',
u'\u7bfa': 'CjkUnifiedIdeograph7bfa',
u'\u977d': 'CjkUnifiedIdeograph977d',
u'\ub6fc': 'HangulSyllableDdwils',
u'\uac81': 'HangulSyllableGeob',
u'\ucd00': 'HangulSyllableCyess',
u'\ue983': 'UnicodeCharacter0xe983',
u'\u0e02': 'ThaiCharacterKhoKhai',
u'\u2a85': 'LessthanOrApproximate',
u'\u4b04': 'CjkUnifiedIdeograph4b04',
u'\u6787': 'CjkUnifiedIdeograph6787',
u'\u980e': 'CjkUnifiedIdeograph980e',
u'\ub491': 'HangulSyllableDwelg',
u'\ud510': 'HangulSyllablePeuls',
u'\uf193': 'UnicodeCharacter0xf193',
u'\u1612': 'CanadianSyllabicsCarrierYi',
u'\u3295': 'CircledIdeographSpecial',
u'\u5314': 'CjkUnifiedIdeograph5314',
u'\u0f97': 'TibetanSubjoinedLetterJa',
u'\u4899': 'CjkUnifiedIdeograph4899',
u'\ua01e': 'YiSyllableBat',
u'\ufd20': 'ArabicLigatureKhahWithYehFinalForm',
u'\u99a3': 'CjkUnifiedIdeograph99a3',
u'\u3e22': 'CjkUnifiedIdeograph3e22',
u'\udaa5': 'UnicodeCharacter0xdaa5',
u'\u7b24': 'CjkUnifiedIdeograph7b24',
u'\u17a7': 'KhmerIndependentVowelQu',
u'\u50a9': 'CjkUnifiedIdeograph50a9',
u'\u8530': 'CjkUnifiedIdeograph8530',
u'\ua1b3': 'YiSyllableHly',
u'\uc632': 'HangulSyllableOlp',
u'\ue2b5': 'UnicodeCharacter0xe2b5',
u'\u0334': 'CombiningTildeOverlay',
u'\u3fb7': 'CjkUnifiedIdeograph3fb7',
u'\u5c36': 'CjkUnifiedIdeograph5c36',
u'\u78b9': 'CjkUnifiedIdeograph78b9',
u'\uad40': 'HangulSyllableGoels',
u'\uee42': 'UnicodeCharacter0xee42',
u'\u8ac5': 'CjkUnifiedIdeograph8ac5',
u'\u2b44': 'RightwardsArrowThroughSuperset',
u'\uc7c7': 'HangulSyllableJaeh',
u'\u6446': 'CjkUnifiedIdeograph6446',
u'\xc9': 'LatinCapitalLetterEWithAcute',
u'\u5dcb': 'CjkUnifiedIdeograph5dcb',
u'\ub550': 'HangulSyllableDdaen',
u'\uf652': 'UnicodeCharacter0xf652',
u'\u92d5': 'CjkUnifiedIdeograph92d5',
u'\u3354': 'SquareRuuburu',
u'\uefd7': 'UnicodeCharacter0xefd7',
u'\u0c56': 'TeluguAiLengthMark',
u'\u28d9': 'BraillePatternDots14578',
u'\u4958': 'CjkUnifiedIdeograph4958',
u'\u65db': 'CjkUnifiedIdeograph65db',
u'\u9e62': 'CjkUnifiedIdeograph9e62',
u'\ubae5': 'HangulSyllableMwaenj',
u'\udb64': 'UnicodeCharacter0xdb64',
u'\uf7e7': 'UnicodeCharacter0xf7e7',
u'\u1466': 'CanadianSyllabicsT',
u'\u30e9': 'KatakanaLetterRa',
u'\u5168': 'CjkUnifiedIdeograph5168',
u'\u0deb': 'UnicodeCharacter0xdeb',
u'\u4eed': 'CjkUnifiedIdeograph4eed',
u'\ua672': 'CombiningCyrillicThousandMillionsSign',
u'\ue374': 'UnicodeCharacter0xe374',
u'\u9ff7': 'UnicodeCharacter0x9ff7',
u'\u3c76': 'CjkUnifiedIdeograph3c76',
u'\ud8f9': 'UnicodeCharacter0xd8f9',
u'\u7978': 'CjkUnifiedIdeograph7978',
u'\u15fb': 'CanadianSyllabicsCarrierKku',
u'\u56fd': 'CjkUnifiedIdeograph56fd',
u'\u6c01': 'CjkUnifiedIdeograph6c01',
u'\u8b84': 'CjkUnifiedIdeograph8b84',
u'\uc486': 'HangulSyllableSsweonh',
u'\u0188': 'LatinSmallLetterCWithHook',
u'\ubd0b': 'HangulSyllableBoc',
u'\u428a': 'CjkUnifiedIdeograph428a',
u'\ufe0d': 'VariationSelector14',
u'\u3b0f': 'CjkUnifiedIdeograph3b0f',
u'\u7411': 'CjkUnifiedIdeograph7411',
u'\u9394': 'CjkUnifiedIdeograph9394',
u'\uec96': 'UnicodeCharacter0xec96',
u'\u2998': 'RightBlackTortoiseShellBracket',
u'\u6a9a': 'CjkUnifiedIdeograph6a9a',
u'\u861d': 'CjkUnifiedIdeograph861d',
u'\uc31f': 'HangulSyllableSsaelb',
u'\u1c21': 'LepchaLetterSha',
u'\u5923': 'CjkUnifiedIdeograph5923',
u'\ubba4': 'HangulSyllableMyu',
u'\uf4a6': 'UnicodeCharacter0xf4a6',
u'\u31a8': 'BopomofoLetterIr',
u'\u72aa': 'CjkUnifiedIdeograph72aa',
u'\uae2d': 'HangulSyllableGyit',
u'\u4fac': 'CjkUnifiedIdeograph4fac',
u'\ueb2f': 'UnicodeCharacter0xeb2f',
u'\u2431': 'UnicodeCharacter0x2431',
u'\u6133': 'CjkUnifiedIdeograph6133',
u'\u9cb6': 'CjkUnifiedIdeograph9cb6',
u'\ud9b8': 'UnicodeCharacter0xd9b8',
u'\u1aba': 'UnicodeCharacter0x1aba',
u'\ub63d': 'HangulSyllableDdwab',
u'\u57bc': 'CjkUnifiedIdeograph57bc',
u'\uf33f': 'UnicodeCharacter0xf33f',
u'\ucc41': 'HangulSyllableCat',
u'\u0943': 'DevanagariVowelSignVocalicR',
u'\u4a45': 'CjkUnifiedIdeograph4a45',
u'\ua4c6': 'YiRadicalKe',
u'\ue1c8': 'UnicodeCharacter0xe1c8',
u'\u22ca': 'RightNormalFactorSemidirectProduct',
u'\u7fcc': 'CjkUnifiedIdeograph7fcc',
u'\u9b4f': 'CjkUnifiedIdeograph9b4f',
u'\ud451': 'HangulSyllablePoeb',
u'\u1153': 'HangulChoseongChieuchhieuh',
u'\u5255': 'CjkUnifiedIdeograph5255',
u'\u89d8': 'CjkUnifiedIdeograph89d8',
u'\ucada': 'HangulSyllableJjwagg',
u'\u07dc': 'NkoLetterGba',
u'\ua35f': 'YiSyllableZhyrx',
u'\u40de': 'CjkUnifiedIdeograph40de',
u'\ufc61': 'ArabicLigatureShaddaWithDammaIsolatedForm',
u'\u3963': 'CjkUnifiedIdeograph3963',
u'\u7a65': 'CjkUnifiedIdeograph7a65',
u'\u91e8': 'CjkUnifiedIdeograph91e8',
u'\ud2ea': 'HangulSyllableTyij',
u'\u2fec': 'UnicodeCharacter0x2fec',
u'\u68ee': 'CjkUnifiedIdeograph68ee',
u'\u8471': 'CjkUnifiedIdeograph8471',
u'\uc173': 'HangulSyllableSyegs',
u'\u0275': 'LatinSmallLetterBarredO',
u'\u5f77': 'CjkUnifiedIdeograph5f77',
u'\ub9f8': 'HangulSyllableMaess',
u'\ufafa': 'UnicodeCharacter0xfafa',
u'\u37fc': 'CjkUnifiedIdeograph37fc',
u'\u70fe': 'CjkUnifiedIdeograph70fe',
u'\u4e00': 'CjkUnifiedIdeograph4e00',
u'\u6a83': 'CjkUnifiedIdeograph6a83',
u'\u8d02': 'CjkUnifiedIdeograph8d02',
u'\ua985': 'UnicodeCharacter0xa985',
u'\u0706': 'SyriacColonSkewedLeft',
u'\u2389': 'CircledHorizontalBarWithNotch',
u'\udc0c': 'UnicodeCharacter0xdc0c',
u'\uf88f': 'UnicodeCharacter0xf88f',
u'\u5610': 'CjkUnifiedIdeograph5610',
u'\u7293': 'CjkUnifiedIdeograph7293',
u'\u9512': 'CjkUnifiedIdeograph9512',
u'\ub195': 'HangulSyllableNwag',
u'\u2f16': 'KangxiRadicalHidingEnclosure',
u'\ucb99': 'HangulSyllableJjwet',
u'\u459d': 'CjkUnifiedIdeograph459d',
u'\ue41c': 'UnicodeCharacter0xe41c',
u'\u809f': 'CjkUnifiedIdeograph809f',
u'\u7e20': 'CjkUnifiedIdeograph7e20',
u'\u1aa3': 'UnicodeCharacter0x1aa3',
u'\ubd22': 'HangulSyllableBwabs',
u'\u3726': 'CjkUnifiedIdeograph3726',
u'\ud3a9': 'HangulSyllablePeb',
u'\u6dad': 'CjkUnifiedIdeograph6dad',
u'\u8c2c': 'CjkUnifiedIdeograph8c2c',
u'\ua8af': 'SaurashtraLetterSha',
u'\u0630': 'ArabicLetterThal',
u'\u22b3': 'ContainsAsNormalSubgroup',
u'\udf36': 'UnicodeCharacter0xdf36',
u'\ufbb9': 'UnicodeCharacter0xfbb9',
u'\u593a': 'CjkUnifiedIdeograph593a',
u'\u75bd': 'CjkUnifiedIdeograph75bd',
u'\u943c': 'CjkUnifiedIdeograph943c',
u'\ub0bf': 'HangulSyllableNaelb',
u'\u2e40': 'UnicodeCharacter0x2e40',
u'\ucac3': 'HangulSyllableJjod',
u'\u44c7': 'CjkUnifiedIdeograph44c7',
u'\ue746': 'UnicodeCharacter0xe746',
u'\u83c9': 'CjkUnifiedIdeograph83c9',
u'\u614a': 'CjkUnifiedIdeograph614a',
u'\u1dcd': 'CombiningDoubleCircumflexAbove',
u'\ubc4c': 'HangulSyllableBya',
u'\u3650': 'CjkUnifiedIdeograph3650',
u'\ud2d3': 'HangulSyllableTeuh',
u'\u6cd7': 'CjkUnifiedIdeograph6cd7',
u'\u8f56': 'CjkUnifiedIdeograph8f56',
u'\uabd9': 'UnicodeCharacter0xabd9',
u'\u095a': 'DevanagariLetterGhha',
u'\u25dd': 'UpperRightQuadrantCircularArc',
u'\ude60': 'UnicodeCharacter0xde60',
u'\ufae3': 'UnicodeCharacter0xfae3',
u'\u5864': 'CjkUnifiedIdeograph5864',
u'\u74e7': 'CjkUnifiedIdeograph74e7',
u'\u9766': 'CjkUnifiedIdeograph9766',
u'\ub3e9': 'HangulSyllableDwalg',
u'\u116a': 'HangulJungseongWa',
u'\ucded': 'HangulSyllableCwinj',
u'\u47f1': 'CjkUnifiedIdeograph47f1',
u'\ue670': 'UnicodeCharacter0xe670',
u'\u82f3': 'CjkUnifiedIdeograph82f3',
u'\u6074': 'CjkUnifiedIdeograph6074',
u'\u1cf7': 'UnicodeCharacter0x1cf7',
u'\ubf76': 'HangulSyllableBbwap',
u'\u397a': 'CjkUnifiedIdeograph397a',
u'\ud5fd': 'HangulSyllableHet',
u'\uef01': 'UnicodeCharacter0xef01',
u'\u8e80': 'CjkUnifiedIdeograph8e80',
u'\u6905': 'CjkUnifiedIdeograph6905',
u'\u0884': 'UnicodeCharacter0x884',
u'\ua407': 'YiSyllableQuop',
u'\u3e0b': 'CjkUnifiedIdeograph3e0b',
u'\uc18a': 'HangulSyllableSyep',
u'\u5b8e': 'CjkUnifiedIdeograph5b8e',
u'\uf711': 'UnicodeCharacter0xf711',
u'\u9690': 'CjkUnifiedIdeograph9690',
u'\u7115': 'CjkUnifiedIdeograph7115',
u'\u1094': 'MyanmarShanDigitFour',
u'\uc61b': 'HangulSyllableYes',
u'\ue99a': 'UnicodeCharacter0xe99a',
u'\u401f': 'CjkUnifiedIdeograph401f',
u'\u639e': 'CjkUnifiedIdeograph639e',
u'\u9f21': 'CjkUnifiedIdeograph9f21',
u'\ubea0': 'HangulSyllableBbyal',
u'\u1925': 'LimbuVowelSignOo',
u'\u38a4': 'CjkUnifiedIdeograph38a4',
u'\uee2b': 'UnicodeCharacter0xee2b',
u'\uf1aa': 'UnicodeCharacter0xf1aa',
u'\u682f': 'CjkUnifiedIdeograph682f',
u'\u0bae': 'TamilLetterMa',
u'\ua731': 'LatinLetterSmallCapitalS',
u'\u2135': 'AlefSymbol',
u'\uc0b4': 'HangulSyllableSal',
u'\u5ab8': 'CjkUnifiedIdeograph5ab8',
u'\uf63b': 'UnicodeCharacter0xf63b',
u'\u99ba': 'CjkUnifiedIdeograph99ba',
u'\u703f': 'CjkUnifiedIdeograph703f',
u'\u13be': 'CherokeeLetterNa',
u'\uc945': 'HangulSyllableJweb',
u'\ue8c4': 'UnicodeCharacter0xe8c4',
u'\u4349': 'CjkUnifiedIdeograph4349',
u'\u62c8': 'CjkUnifiedIdeograph62c8',
u'\u9e4b': 'CjkUnifiedIdeograph9e4b',
u'\ua1ca': 'YiSyllableLop',
u'\u184f': 'MongolianLetterTodoMa',
u'\u3bce': 'CjkUnifiedIdeograph3bce',
u'\ud155': 'HangulSyllableTelg',
u'\uf0d4': 'UnicodeCharacter0xf0d4',
u'\u6b59': 'CjkUnifiedIdeograph6b59',
u'\u0ad8': 'UnicodeCharacter0xad8',
u'\ua65b': 'CyrillicSmallLetterBlendedYus',
u'\u205f': 'MediumMathematicalSpace',
u'\uc3de': 'HangulSyllableSsonh',
u'\u5de2': 'CjkUnifiedIdeograph5de2',
u'\uf965': 'CjkCompatibilityIdeographf965',
u'\u98e4': 'CjkUnifiedIdeograph98e4',
u'\u7369': 'CjkUnifiedIdeograph7369',
u'\u12e8': 'EthiopicSyllableYa',
u'\uc86f': 'HangulSyllableJyeh',
u'\uebee': 'UnicodeCharacter0xebee',
u'\u4273': 'CjkUnifiedIdeograph4273',
u'\u65f2': 'CjkUnifiedIdeograph65f2',
u'\u8175': 'CjkUnifiedIdeograph8175',
u'\ua0f4': 'YiSyllableVut',
u'\u1b79': 'BalineseMusicalSymbolLefthandOpenPung',
u'\u3af8': 'CjkUnifiedIdeograph3af8',
u'\ud07f': 'HangulSyllableKeus',
u'\uf3fe': 'UnicodeCharacter0xf3fe',
u'\u8606': 'CjkUnifiedIdeograph8606',
u'\ua289': 'YiSyllableWop',
u'\uc308': 'HangulSyllableSsam',
u'\uff8b': 'HalfwidthKatakanaLetterHi',
u'\u3c8d': 'CjkUnifiedIdeograph3c8d',
u'\u5d0c': 'CjkUnifiedIdeograph5d0c',
u'\u798f': 'CjkUnifiedIdeograph798f',
u'\uae16': 'HangulSyllableGyigg',
u'\ueb18': 'UnicodeCharacter0xeb18',
u'\u879b': 'CjkUnifiedIdeograph879b',
u'\u281a': 'BraillePatternDots245',
u'\uc49d': 'HangulSyllableSsweg',
u'\u651c': 'CjkUnifiedIdeograph651c',
u'\u019f': 'LatinCapitalLetterOWithMiddleTilde',
u'\u5ea1': 'CjkUnifiedIdeograph5ea1',
u'\ub626': 'HangulSyllableDdoj',
u'\uf328': 'UnicodeCharacter0xf328',
u'\uafab': 'HangulSyllableGgyolh',
u'\u302a': 'IdeographicLevelToneMark',
u'\uecad': 'UnicodeCharacter0xecad',
u'\u0d2c': 'MalayalamLetterBa',
u'\u29af': 'MeasuredAngleWithOpenArmEndingInArrowPointingLeftAndDown',
u'\u4a2e': 'CjkUnifiedIdeograph4a2e',
u'\u66b1': 'CjkUnifiedIdeograph66b1',
u'\u9b38': 'CjkUnifiedIdeograph9b38',
u'\ub7bb': 'HangulSyllableRyad',
u'\ud83a': 'UnicodeCharacter0xd83a',
u'\uf4bd': 'UnicodeCharacter0xf4bd',
u'\u153c': 'CanadianSyllabicsWestcreeYwaa',
u'\u31bf': 'UnicodeCharacter0x31bf',
u'\u523e': 'CjkUnifiedIdeograph523e',
u'\u0ec1': 'LaoVowelSignEi',
u'\u4bc3': 'CjkUnifiedIdeograph4bc3',
u'\ua348': 'YiSyllableZha',
u'\ue04a': 'UnicodeCharacter0xe04a',
u'\u9ccd': 'CjkUnifiedIdeograph9ccd',
u'\u3d4c': 'CjkUnifiedIdeograph3d4c',
u'\ud9cf': 'UnicodeCharacter0xd9cf',
u'\u7a4e': 'CjkUnifiedIdeograph7a4e',
u'\u16d1': 'RunicLetterD',
u'\u53d3': 'CjkUnifiedIdeograph53d3',
u'\u885a': 'CjkUnifiedIdeograph885a',
u'\ua4dd': 'UnicodeCharacter0xa4dd',
u'\uc55c': 'HangulSyllableAk',
u'\ue1df': 'UnicodeCharacter0xe1df',
u'\u025e': 'LatinSmallLetterClosedReversedOpenE',
u'\u3ee1': 'CjkUnifiedIdeograph3ee1',
u'\u5f60': 'CjkUnifiedIdeograph5f60',
u'\u7be3': 'CjkUnifiedIdeograph7be3',
u'\u906a': 'CjkUnifiedIdeograph906a',
u'\ued6c': 'UnicodeCharacter0xed6c',
u'\u89ef': 'CjkUnifiedIdeograph89ef',
u'\u2a6e': 'EqualsWithAsterisk',
u'\uc6f1': 'HangulSyllableWelg',
u'\u6770': 'CjkUnifiedIdeograph6770',
u'\u03f3': 'GreekLetterYot',
u'\u40f5': 'CjkUnifiedIdeograph40f5',
u'\ub87a': 'HangulSyllableRwagg',
u'\uf57c': 'UnicodeCharacter0xf57c',
u'\u91ff': 'CjkUnifiedIdeograph91ff',
u'\u327e': 'CircledHangulIeungU',
u'\u0f80': 'TibetanVowelSignReversedI',
u'\uab03': 'UnicodeCharacter0xab03',
u'\u4c82': 'CjkUnifiedIdeograph4c82',
u'\ue805': 'UnicodeCharacter0xe805',
u'\u2507': 'BoxDrawingsHeavyTripleDashVertical',
u'\u6209': 'CjkUnifiedIdeograph6209',
u'\u9d8c': 'CjkUnifiedIdeograph9d8c',
u'\uda8e': 'UnicodeCharacter0xda8e',
u'\u1790': 'KhmerLetterTha',
u'\ub313': 'HangulSyllableDaes',
u'\u5492': 'CjkUnifiedIdeograph5492',
u'\uf015': 'UnicodeCharacter0xf015',
u'\ucd17': 'HangulSyllableColh',
u'\u0a19': 'GurmukhiLetterNga',
u'\u471b': 'CjkUnifiedIdeograph471b',
u'\ua59c': 'VaiSyllableBhu',
u'\ue29e': 'UnicodeCharacter0xe29e',
u'\u3fa0': 'CjkUnifiedIdeograph3fa0',
u'\u7ca2': 'CjkUnifiedIdeograph7ca2',
u'\u9825': 'CjkUnifiedIdeograph9825',
u'\ud527': 'HangulSyllablePyid',
u'\u1229': 'EthiopicSyllableRu',
u'\u6f2b': 'CjkUnifiedIdeograph6f2b',
u'\u8aae': 'CjkUnifiedIdeograph8aae',
u'\uc7b0': 'HangulSyllableJaen',
u'\u04b2': 'CyrillicCapitalLetterHaWithDescender',
u'\ua035': 'YiSyllableByp',
u'\u41b4': 'CjkUnifiedIdeograph41b4',
u'\ufd37': 'ArabicLigatureSheenWithJeemMedialForm',
u'\u3a39': 'CjkUnifiedIdeograph3a39',
u'\u773b': 'CjkUnifiedIdeograph773b',
u'\u92be': 'CjkUnifiedIdeograph92be',
u'\uefc0': 'UnicodeCharacter0xefc0',
u'\u2cc2': 'CopticCapitalLetterCrossedShei',
u'\u69c4': 'CjkUnifiedIdeograph69c4',
u'\u8547': 'CjkUnifiedIdeograph8547',
u'\uc249': 'HangulSyllableSweong',
u'\u1f4b': 'GreekCapitalLetterOmicronWithDasiaAndVaria',
u'\u5c4d': 'CjkUnifiedIdeograph5c4d',
u'\ubace': 'HangulSyllableMwalm',
u'\uf7d0': 'UnicodeCharacter0xf7d0',
u'\u34d2': 'CjkUnifiedIdeograph34d2',
u'\u71d4': 'CjkUnifiedIdeograph71d4',
u'\uad57': 'HangulSyllableGyod',
u'\u4ed6': 'CjkUnifiedIdeograph4ed6',
u'\uea59': 'UnicodeCharacter0xea59',
u'\u275b': 'HeavySingleTurnedCommaQuotationMarkOrnament',
u'\u645d': 'CjkUnifiedIdeograph645d',
u'\u9fe0': 'UnicodeCharacter0x9fe0',
u'\udce2': 'UnicodeCharacter0xdce2',
u'\u19e4': 'KhmerSymbolBuonKoet',
u'\ub567': 'HangulSyllableDdaeh',
u'\u56e6': 'CjkUnifiedIdeograph56e6',
u'\uf269': 'UnicodeCharacter0xf269',
u'\ucf6b': 'HangulSyllableKoc',
u'\u0c6d': 'TeluguDigitSeven',
u'\u496f': 'CjkUnifiedIdeograph496f',
u'\ua7f0': 'UnicodeCharacter0xa7f0',
u'\ue4f2': 'UnicodeCharacter0xe4f2',
u'\u21f4': 'RightArrowWithSmallCircle',
u'\u7ef6': 'CjkUnifiedIdeograph7ef6',
u'\u9a79': 'CjkUnifiedIdeograph9a79',
u'\ud77b': 'HangulSyllableHyilh',
u'\u147d': 'CanadianSyllabicsWestcreeKwoo',
u'\u517f': 'CjkUnifiedIdeograph517f',
u'\u3181': 'HangulLetterYesieung',
u'\ud604': 'HangulSyllableHyeon',
u'\ufa87': 'CjkCompatibilityIdeographfa87',
u'\u5c08': 'CjkUnifiedIdeograph5c08',
u'\u608b': 'CjkUnifiedIdeograph608b',
u'\u9f0a': 'CjkUnifiedIdeograph9f0a',
u'\ua38d': 'YiSyllableRrur',
u'\u050e': 'CyrillicCapitalLetterKomiTje',
u'\u2991': 'LeftAngleBracketWithDot',
u'\uce14': 'HangulSyllableCyum',
u'\u9297': 'CjkUnifiedIdeograph9297',
u'\u7418': 'CjkUnifiedIdeograph7418',
u'\u189b': 'MongolianLetterManchuAliGaliNga',
u'\ub71a': 'HangulSyllableDdyulp',
u'\u3d1e': 'CjkUnifiedIdeograph3d1e',
u'\uc1a1': 'HangulSyllableSong',
u'\u47a5': 'CjkUnifiedIdeograph47a5',
u'\ue624': 'UnicodeCharacter0xe624',
u'\u8aa7': 'CjkUnifiedIdeograph8aa7',
u'\u6c28': 'CjkUnifiedIdeograph6c28',
u'\u30ab': 'KatakanaLetterKa',
u'\uaf2a': 'HangulSyllableGgyep',
u'\ud52e': 'HangulSyllablePyilp',
u'\uf9b1': 'CjkCompatibilityIdeographf9b1',
u'\u5b32': 'CjkUnifiedIdeograph5b32',
u'\u7fb5': 'CjkUnifiedIdeograph7fb5',
u'\u9e34': 'CjkUnifiedIdeograph9e34',
u'\ua2b7': 'YiSyllableCax',
u'\u0438': 'CyrillicSmallLetterI',
u'\u28bb': 'BraillePatternDots124568',
u'\ucd3e': 'HangulSyllableCwap',
u'\u91c1': 'CjkUnifiedIdeograph91c1',
u'\u7342': 'CjkUnifiedIdeograph7342',
u'\u17c5': 'KhmerVowelSignAu',
u'\ub644': 'HangulSyllableDdwak',
u'\u3c48': 'CjkUnifiedIdeograph3c48',
u'\uc0cb': 'HangulSyllableSaegs',
u'\u46cf': 'CjkUnifiedIdeograph46cf',
u'\ue54e': 'UnicodeCharacter0xe54e',
u'\u89d1': 'CjkUnifiedIdeograph89d1',
u'\u6b52': 'CjkUnifiedIdeograph6b52',
u'\u0fd5': 'UnicodeCharacter0xfd5',
u'\uae54': 'HangulSyllableGgal',
u'\ud458': 'HangulSyllablePoek',
u'\uf8db': 'UnicodeCharacter0xf8db',
u'\u5a5c': 'CjkUnifiedIdeograph5a5c',
u'\u7edf': 'CjkUnifiedIdeograph7edf',
u'\u9d5e': 'CjkUnifiedIdeograph9d5e',
u'\ua1e1': 'YiSyllableGiep',
u'\u0362': 'CombiningDoubleRightwardsArrowBelow',
u'\u27e5': 'WhiteSquareWithRightwardsTick',
u'\ucc68': 'HangulSyllableCyal',
u'\u90eb': 'CjkUnifiedIdeograph90eb',
u'\u726c': 'CjkUnifiedIdeograph726c',
u'\u16ef': 'RunicTvimadurSymbol',
u'\ub56e': 'HangulSyllableDdyanh',
u'\u3b72': 'CjkUnifiedIdeograph3b72',
u'\udff5': 'UnicodeCharacter0xdff5',
u'\u45f9': 'CjkUnifiedIdeograph45f9',
u'\ue478': 'UnicodeCharacter0xe478',
u'\u88fb': 'CjkUnifiedIdeograph88fb',
u'\u6a7c': 'CjkUnifiedIdeograph6a7c',
u'\u0eff': 'UnicodeCharacter0xeff',
u'\uad7e': 'HangulSyllableGubs',
u'\u3403': 'CjkUnifiedIdeograph3403',
u'\ud382': 'HangulSyllablePeonh',
u'\u5986': 'CjkUnifiedIdeograph5986',
u'\ufd09': 'ArabicLigatureSheenWithJeemIsolatedForm',
u'\u9c88': 'CjkUnifiedIdeograph9c88',
u'\u630d': 'CjkUnifiedIdeograph630d',
u'\u028c': 'LatinSmallLetterTurnedV',
u'\ua60f': 'VaiQuestionMark',
u'\u2c13': 'GlagoliticCapitalLetterRitsi',
u'\ucb92': 'HangulSyllableJjwebs',
u'\u5217': 'CjkUnifiedIdeograph5217',
u'\u7196': 'CjkUnifiedIdeograph7196',
u'\u9519': 'CjkUnifiedIdeograph9519',
u'\ub498': 'HangulSyllableDwem',
u'\u1b1d': 'BalineseLetterTaLatik',
u'\u3a9c': 'CjkUnifiedIdeograph3a9c',
u'\uc423': 'HangulSyllableSswaes',
u'\ue3a2': 'UnicodeCharacter0xe3a2',
u'\u4a27': 'CjkUnifiedIdeograph4a27',
u'\u69a6': 'CjkUnifiedIdeograph69a6',
u'\u8d29': 'CjkUnifiedIdeograph8d29',
u'\uaca8': 'HangulSyllableGyeo',
u'\u332d': 'SquareBaareru',
u'\ud2ac': 'HangulSyllableTyum',
u'\u58b0': 'CjkUnifiedIdeograph58b0',
u'\ufc33': 'ArabicLigatureQafWithHahIsolatedForm',
u'\u9bb2': 'CjkUnifiedIdeograph9bb2',
u'\u6237': 'CjkUnifiedIdeograph6237',
u'\u01b6': 'LatinSmallLetterZWithStroke',
u'\ua539': 'VaiSyllableDi',
u'\u2b3d': 'LeftwardsTwoheadedArrowWithTailWithDoubleVerticalStroke',
u'\ucabc': 'HangulSyllableJjo',
u'\u5141': 'CjkUnifiedIdeograph5141',
u'\u70c0': 'CjkUnifiedIdeograph70c0',
u'\u9443': 'CjkUnifiedIdeograph9443',
u'\ub3c2': 'HangulSyllableDyep',
u'\u1a47': 'UnicodeCharacter0x1a47',
u'\u39c6': 'CjkUnifiedIdeograph39c6',
u'\uc34d': 'HangulSyllableSsyaeg',
u'\ue2cc': 'UnicodeCharacter0xe2cc',
u'\u4951': 'CjkUnifiedIdeograph4951',
u'\u68d0': 'CjkUnifiedIdeograph68d0',
u'\u8c53': 'CjkUnifiedIdeograph8c53',
u'\uabd2': 'UnicodeCharacter0xabd2',
u'\u3257': 'CircledNumberTwentySeven',
u'\ud1d6': 'HangulSyllableTwap',
u'\u57da': 'CjkUnifiedIdeograph57da',
u'\ufb5d': 'ArabicLetterBehehMedialForm',
u'\u9adc': 'CjkUnifiedIdeograph9adc',
u'\u6161': 'CjkUnifiedIdeograph6161',
u'\xe0': 'LatinSmallLetterAWithGrave',
u'\ua463': 'YiSyllableXie',
u'\u2a67': 'IdenticalWithDotAbove',
u'\uc9e6': 'HangulSyllableJjalm',
u'\u506b': 'CjkUnifiedIdeograph506b',
u'\u4fea': 'CjkUnifiedIdeograph4fea',
u'\u936d': 'CjkUnifiedIdeograph936d',
u'\ub2ec': 'HangulSyllableDal',
u'\u1971': 'TaiLeLetterTone3',
u'\u38f0': 'CjkUnifiedIdeograph38f0',
u'\uc277': 'HangulSyllableSwilb',
u'\ue1f6': 'UnicodeCharacter0xe1f6',
u'\u487b': 'CjkUnifiedIdeograph487b',
u'\u67fa': 'CjkUnifiedIdeograph67fa',
u'\u8b7d': 'CjkUnifiedIdeograph8b7d',
u'\uaafc': 'UnicodeCharacter0xaafc',
u'\ub081': 'HangulSyllableGginj',
u'\ud100': 'HangulSyllableTyal',
u'\uf583': 'UnicodeCharacter0xf583',
u'\u1202': 'EthiopicSyllableHi',
u'\u3685': 'CjkUnifiedIdeograph3685',
u'\u5704': 'CjkUnifiedIdeograph5704',
u'\u7b87': 'CjkUnifiedIdeograph7b87',
u'\u840e': 'CjkUnifiedIdeograph840e',
u'\ua891': 'SaurashtraLetterAu',
u'\uc910': 'HangulSyllableJuss',
u'\ued93': 'UnicodeCharacter0xed93',
u'\u0a12': 'UnicodeCharacter0xa12',
u'\u2e95': 'CjkRadicalSnoutTwo',
u'\u4f14': 'CjkUnifiedIdeograph4f14',
u'\u1397': 'EthiopicTonalMarkHidet',
u'\u5499': 'CjkUnifiedIdeograph5499',
u'\ubc1e': 'HangulSyllableBalm',
u'\ue120': 'UnicodeCharacter0xe120',
u'\u85a3': 'CjkUnifiedIdeograph85a3',
u'\u2222': 'SphericalAngle',
u'\uc6a5': 'HangulSyllableYob',
u'\u6724': 'CjkUnifiedIdeograph6724',
u'\u0ba7': 'UnicodeCharacter0xba7',
u'\u4ca9': 'CjkUnifiedIdeograph4ca9',
u'\u9930': 'CjkUnifiedIdeograph9930',
u'\ubdb3': 'HangulSyllableBweoc',
u'\uda32': 'UnicodeCharacter0xda32',
u'\ufeb5': 'ArabicLetterSheenIsolatedForm',
u'\u1f34': 'GreekSmallLetterIotaWithPsiliAndOxia',
u'\u23b7': 'RadicalSymbolBottom',
u'\u4036': 'CjkUnifiedIdeograph4036',
u'\u64b9': 'CjkUnifiedIdeograph64b9',
u'\ub140': 'HangulSyllableNyeo',
u'\uf242': 'UnicodeCharacter0xf242',
u'\u96c5': 'CjkUnifiedIdeograph96c5',
u'\u3744': 'CjkUnifiedIdeograph3744',
u'\udbc7': 'UnicodeCharacter0xdbc7',
u'\u7846': 'CjkUnifiedIdeograph7846',
u'\u1cc9': 'UnicodeCharacter0x1cc9',
u'\u41cb': 'CjkUnifiedIdeograph41cb',
u'\ua950': 'RejangConsonantSignN',
u'\uea52': 'UnicodeCharacter0xea52',
u'\u8ed5': 'CjkUnifiedIdeograph8ed5',
u'\u2f54': 'KangxiRadicalWater',
u'\uf3d7': 'UnicodeCharacter0xf3d7',
u'\u1056': 'MyanmarVowelSignVocalicR',
u'\u34d9': 'CjkUnifiedIdeograph34d9',
u'\u5558': 'CjkUnifiedIdeograph5558',
u'\u79db': 'CjkUnifiedIdeograph79db',
u'\u8262': 'CjkUnifiedIdeograph8262',
u'\ua6e5': 'UnicodeCharacter0xa6e5',
u'\uc764': 'HangulSyllableYils',
u'\uebe7': 'UnicodeCharacter0xebe7',
u'\u0866': 'UnicodeCharacter0x866',
u'\u2ce9': 'CopticSymbolKhiRo',
u'\u4d68': 'CjkUnifiedIdeograph4d68',
u'\u11eb': 'HangulJongseongPansios',
u'\u52ed': 'CjkUnifiedIdeograph52ed',
u'\uba72': 'HangulSyllableMyeogg',
u'\uff74': 'HalfwidthKatakanaLetterE',
u'\u83f7': 'CjkUnifiedIdeograph83f7',
u'\u2076': 'SuperscriptSix',
u'\uc4f9': 'HangulSyllableSseulg',
u'\u6578': 'CjkUnifiedIdeograph6578',
u'\u09fb': 'UnicodeCharacter0x9fb',
u'\u4afd': 'CjkUnifiedIdeograph4afd',
u'\u7001': 'CjkUnifiedIdeograph7001',
u'\u9784': 'CjkUnifiedIdeograph9784',
u'\ud886': 'UnicodeCharacter0xd886',
u'\u1d88': 'LatinSmallLetterPWithPalatalHook',
u'\ua10b': 'YiSyllableDuox',
u'\u5e8a': 'CjkUnifiedIdeograph5e8a',
u'\ue20d': 'UnicodeCharacter0xe20d',
u'\u270f': 'Pencil',
u'\u6811': 'CjkUnifiedIdeograph6811',
u'\u8f94': 'CjkUnifiedIdeograph8f94',
u'\uf096': 'UnicodeCharacter0xf096',
u'\u3598': 'CjkUnifiedIdeograph3598',
u'\u769a': 'CjkUnifiedIdeograph769a',
u'\u9a1d': 'CjkUnifiedIdeograph9a1d',
u'\udf1f': 'UnicodeCharacter0xdf1f',
u'\u4523': 'CjkUnifiedIdeograph4523',
u'\ua7a4': 'UnicodeCharacter0xa7a4',
u'\ue8a6': 'UnicodeCharacter0xe8a6',
u'\u2da8': 'EthiopicSyllableCca',
u'\u6eaa': 'CjkUnifiedIdeograph6eaa',
u'\ub22d': 'HangulSyllableNweolt',
u'\u53ac': 'CjkUnifiedIdeograph53ac',
u'\uf72f': 'UnicodeCharacter0xf72f',
u'\u3831': 'CjkUnifiedIdeograph3831',
u'\u7d33': 'CjkUnifiedIdeograph7d33',
u'\u80b6': 'CjkUnifiedIdeograph80b6',
u'\uc5b8': 'HangulSyllableEon',
u'\u06ba': 'ArabicLetterNoonGhunna',
u'\uaa3d': 'UnicodeCharacter0xaa3d',
u'\u4bbc': 'CjkUnifiedIdeograph4bbc',
u'\uef3f': 'UnicodeCharacter0xef3f',
u'\ud041': 'HangulSyllableKwilt',
u'\u1543': 'CanadianSyllabicsRcreeRe',
u'\u5645': 'CjkUnifiedIdeograph5645',
u'\ub8c6': 'HangulSyllableRoej',
u'\ufdc8': 'UnicodeCharacter0xfdc8',
u'\u3eca': 'CjkUnifiedIdeograph3eca',
u'\u63cc': 'CjkUnifiedIdeograph63cc',
u'\u874f': 'CjkUnifiedIdeograph874f',
u'\uc851': 'HangulSyllableJyeot',
u'\u0d53': 'UnicodeCharacter0xd53',
u'\u4e55': 'CjkUnifiedIdeograph4e55',
u'\u95d8': 'CjkUnifiedIdeograph95d8',
u'\ud6da': 'HangulSyllableHuj',
u'\u1bdc': 'UnicodeCharacter0x1bdc',
u'\ubf5f': 'HangulSyllableBbwags',
u'\u5cde': 'CjkUnifiedIdeograph5cde',
u'\ue061': 'UnicodeCharacter0xe061',
u'\u2563': 'BoxDrawingsDoubleVerticalAndLeft',
u'\u6665': 'CjkUnifiedIdeograph6665',
u'\u8de8': 'CjkUnifiedIdeograph8de8',
u'\uceea': 'HangulSyllableKeonh',
u'\u33ec': 'IdeographicTelegraphSymbolForDayThirteen',
u'\u74ee': 'CjkUnifiedIdeograph74ee',
u'\u9871': 'CjkUnifiedIdeograph9871',
u'\udd73': 'UnicodeCharacter0xdd73',
u'\u1e75': 'LatinSmallLetterUWithTildeBelow',
u'\u4377': 'CjkUnifiedIdeograph4377',
u'\ua5f8': 'VaiSyllableRe',
u'\ue6fa': 'UnicodeCharacter0xe6fa',
u'\u2bfc': 'UnicodeCharacter0x2bfc',
u'\u6cfe': 'CjkUnifiedIdeograph6cfe',
u'\u5200': 'CjkUnifiedIdeograph5200',
u'\u7683': 'CjkUnifiedIdeograph7683',
u'\u9102': 'CjkUnifiedIdeograph9102',
u'\ub585': 'HangulSyllableDdyaeg',
u'\u1b06': 'BalineseLetterAkaraTedung',
u'\u3f89': 'CjkUnifiedIdeograph3f89',
u'\uc00c': 'HangulSyllableBbwel',
u'\ue48f': 'UnicodeCharacter0xe48f',
u'\u4a10': 'CjkUnifiedIdeograph4a10',
u'\u6e93': 'CjkUnifiedIdeograph6e93',
u'\u8912': 'CjkUnifiedIdeograph8912',
u'\uad95': 'HangulSyllableGweolt',
u'\u3316': 'SquareKiromeetoru',
u'\ud799': 'HangulSyllableHib',
u'\u599d': 'CjkUnifiedIdeograph599d',
u'\uf81c': 'UnicodeCharacter0xf81c',
u'\u9c9f': 'CjkUnifiedIdeograph9c9f',
u'\u6220': 'CjkUnifiedIdeograph6220',
u'\u06a3': 'ArabicLetterFehWithDotBelow',
u'\ua122': 'YiSyllableTax',
u'\u2b26': 'WhiteMediumDiamond',
u'\ucfa9': 'HangulSyllableKoeg',
u'\u71ad': 'CjkUnifiedIdeograph71ad',
u'\u902c': 'CjkUnifiedIdeograph902c',
u'\ub4af': 'HangulSyllableDwilb',
u'\u1a30': 'UnicodeCharacter0x1a30',
u'\u3eb3': 'CjkUnifiedIdeograph3eb3',
u'\uc336': 'HangulSyllableSsyanh',
u'\ue7b9': 'UnicodeCharacter0xe7b9',
u'\u453a': 'CjkUnifiedIdeograph453a',
u'\u69bd': 'CjkUnifiedIdeograph69bd',
u'\u883c': 'CjkUnifiedIdeograph883c',
u'\uacbf': 'HangulSyllableGyeoc',
u'\u3240': 'ParenthesizedIdeographFestival',
u'\ud6c3': 'HangulSyllableHyoh',
u'\u58c7': 'CjkUnifiedIdeograph58c7',
u'\ufb46': 'HebrewLetterTsadiWithDagesh',
u'\u9fc9': 'UnicodeCharacter0x9fc9',
u'\u7d4a': 'CjkUnifiedIdeograph7d4a',
u'\u01cd': 'LatinCapitalLetterAWithCaron',
u'\ua04c': 'YiSyllablePu',
u'\u2a50': 'ClosedUnionWithSerifsAndSmashProduct',
u'\uced3': 'HangulSyllableKyaelb',
u'\u70d7': 'CjkUnifiedIdeograph70d7',
u'\u9356': 'CjkUnifiedIdeograph9356',
u'\ub7d9': 'HangulSyllableRyaelg',
u'\u155a': 'CanadianSyllabicsFaa',
u'\u39dd': 'CjkUnifiedIdeograph39dd',
u'\uc260': 'HangulSyllableSwem',
u'\ue6e3': 'UnicodeCharacter0xe6e3',
u'\u4464': 'CjkUnifiedIdeograph4464',
u'\u68e7': 'CjkUnifiedIdeograph68e7',
u'\u8b66': 'CjkUnifiedIdeograph8b66',
u'\uafe9': 'HangulSyllableGgweong',
u'\u0d6a': 'MalayalamDigitFour',
u'\ud1ed': 'HangulSyllableTwaeng',
u'\u5bf1': 'CjkUnifiedIdeograph5bf1',
u'\ufa70': 'CjkCompatibilityIdeographfa70',
u'\u9ef3': 'CjkUnifiedIdeograph9ef3',
u'\u7c74': 'CjkUnifiedIdeograph7c74',
u'\xf7': 'DivisionSign',
u'\ua376': 'YiSyllableChyt',
u'\u257a': 'BoxDrawingsHeavyRight',
u'\uc9fd': 'HangulSyllableJjaenj',
u'\uf301': 'UnicodeCharacter0xf301',
u'\u9280': 'CjkUnifiedIdeograph9280',
u'\u7505': 'CjkUnifiedIdeograph7505',
u'\u1484': 'CanadianSyllabicsKw',
u'\ub807': 'HangulSyllableReoh',
u'\u220b': 'ContainsAsMember',
u'\udd8a': 'UnicodeCharacter0xdd8a',
u'\u478e': 'CjkUnifiedIdeograph478e',
u'\ueb11': 'UnicodeCharacter0xeb11',
u'\u8a90': 'CjkUnifiedIdeograph8a90',
u'\u6d15': 'CjkUnifiedIdeograph6d15',
u'\u0c94': 'KannadaLetterAu',
u'\uda1b': 'UnicodeCharacter0xda1b',
u'\uf59a': 'UnicodeCharacter0xf59a',
u'\u5c1f': 'CjkUnifiedIdeograph5c1f',
u'\u7f9e': 'CjkUnifiedIdeograph7f9e',
u'\u8321': 'CjkUnifiedIdeograph8321',
u'\ua2a0': 'YiSyllableZe',
u'\u0525': 'UnicodeCharacter0x525',
u'\u24a4': 'ParenthesizedLatinSmallLetterI',
u'\uf22b': 'UnicodeCharacter0xf22b',
u'\uedaa': 'UnicodeCharacter0xedaa',
u'\u742f': 'CjkUnifiedIdeograph742f',
u'\u17ae': 'KhmerIndependentVowelLyy',
u'\ubb31': 'HangulSyllableMyot',
u'\u3d35': 'CjkUnifiedIdeograph3d35',
u'\udcb4': 'UnicodeCharacter0xdcb4',
u'\u46b8': 'CjkUnifiedIdeograph46b8',
u'\uea3b': 'UnicodeCharacter0xea3b',
u'\u85ba': 'CjkUnifiedIdeograph85ba',
u'\u6c3f': 'CjkUnifiedIdeograph6c3f',
u'\u0fbe': 'TibetanKuRuKha',
u'\ud545': 'HangulSyllablePilg',
u'\uf4c4': 'UnicodeCharacter0xf4c4',
u'\u5f49': 'CjkUnifiedIdeograph5f49',
u'\u7ec8': 'CjkUnifiedIdeograph7ec8',
u'\u824b': 'CjkUnifiedIdeograph824b',
u'\ubdca': 'HangulSyllableBwebs',
u'\u044f': 'CyrillicSmallLetterYa',
u'\u27ce': 'UnicodeCharacter0x27ce',
u'\ucd55': 'HangulSyllableCwaeng',
u'\uecd4': 'UnicodeCharacter0xecd4',
u'\u7759': 'CjkUnifiedIdeograph7759',
u'\u16d8': 'RunicLetterLongbranchmadrM',
u'\uba5b': 'HangulSyllableMed',
u'\u3c5f': 'CjkUnifiedIdeograph3c5f',
u'\udfde': 'UnicodeCharacter0xdfde',
u'\u41e2': 'CjkUnifiedIdeograph41e2',
u'\ue565': 'UnicodeCharacter0xe565',
u'\u84e4': 'CjkUnifiedIdeograph84e4',
u'\u6f69': 'CjkUnifiedIdeograph6f69',
u'\u0ee8': 'UnicodeCharacter0xee8',
u'\ud46f': 'HangulSyllablePyos',
u'\uf7ee': 'UnicodeCharacter0xf7ee',
u'\u5e73': 'CjkUnifiedIdeograph5e73',
u'\u79f2': 'CjkUnifiedIdeograph79f2',
u'\u9d75': 'CjkUnifiedIdeograph9d75',
u'\ubcf4': 'HangulSyllableBo',
u'\u0779': 'ArabicLetterWawWithExtendedArabicindicDigitThreeAbove',
u'\u26f8': 'UnicodeCharacter0x26f8',
u'\ucc7f': 'HangulSyllableCyaegs',
u'\ueffe': 'UnicodeCharacter0xeffe',
u'\u9a06': 'CjkUnifiedIdeograph9a06',
u'\ube89': 'HangulSyllableBbaelt',
u'\udf08': 'UnicodeCharacter0xdf08',
u'\ue38b': 'UnicodeCharacter0xe38b',
u'\u1c0a': 'LepchaLetterTa',
u'\u208d': 'SubscriptLeftParenthesis',
u'\u410c': 'CjkUnifiedIdeograph410c',
u'\u658f': 'CjkUnifiedIdeograph658f',
u'\ub216': 'HangulSyllableNubs',
u'\uf718': 'UnicodeCharacter0xf718',
u'\u9b9b': 'CjkUnifiedIdeograph9b9b',
u'\u341a': 'CjkUnifiedIdeograph341a',
u'\ud89d': 'UnicodeCharacter0xd89d',
u'\u791c': 'CjkUnifiedIdeograph791c',
u'\u1d9f': 'ModifierLetterSmallReversedOpenE',
u'\u42a1': 'CjkUnifiedIdeograph42a1',
u'\uaa26': 'ChamLetterSsa',
u'\uef28': 'UnicodeCharacter0xef28',
u'\ub3ab': 'HangulSyllableDyegs',
u'\u2c2a': 'GlagoliticCapitalLetterFita',
u'\uf0ad': 'UnicodeCharacter0xf0ad',
u'\u112c': 'HangulChoseongKapyeounssangpieup',
u'\u35af': 'CjkUnifiedIdeograph35af',
u'\u562e': 'CjkUnifiedIdeograph562e',
u'\u7ab1': 'CjkUnifiedIdeograph7ab1',
u'\u8738': 'CjkUnifiedIdeograph8738',
u'\uabbb': 'UnicodeCharacter0xabbb',
u'\uc43a': 'HangulSyllableSsoelp',
u'\ue8bd': 'UnicodeCharacter0xe8bd',
u'\u093c': 'DevanagariSignNukta',
u'\u2dbf': 'UnicodeCharacter0x2dbf',
u'\u4e3e': 'CjkUnifiedIdeograph4e3e',
u'\u12c1': 'UnicodeCharacter0x12c1',
u'\u57c3': 'CjkUnifiedIdeograph57c3',
u'\ubf48': 'HangulSyllableBbol',
u'\ufc4a': 'ArabicLigatureMeemWithYehIsolatedForm',
u'\u80cd': 'CjkUnifiedIdeograph80cd',
u'\u214c': 'PerSign',
u'\uc5cf': 'HangulSyllableEoh',
u'\u664e': 'CjkUnifiedIdeograph664e',
u'\u0ad1': 'UnicodeCharacter0xad1',
u'\u4fd3': 'CjkUnifiedIdeograph4fd3',
u'\u945a': 'CjkUnifiedIdeograph945a',
u'\ub8dd': 'HangulSyllableRyob',
u'\ud95c': 'UnicodeCharacter0xd95c',
u'\ufddf': 'UnicodeCharacter0xfddf',
u'\u1e5e': 'LatinCapitalLetterRWithLineBelow',
u'\u22e1': 'DoesNotSucceedOrEqual',
u'\u4360': 'CjkUnifiedIdeograph4360',
u'\u67e3': 'CjkUnifiedIdeograph67e3',
u'\u8c6a': 'CjkUnifiedIdeograph8c6a',
u'\uf16c': 'UnicodeCharacter0xf16c',
u'\u95ef': 'CjkUnifiedIdeograph95ef',
u'\u366e': 'CjkUnifiedIdeograph366e',
u'\udaf1': 'UnicodeCharacter0xdaf1',
u'\u7b70': 'CjkUnifiedIdeograph7b70',
u'\u1ff3': 'GreekSmallLetterOmegaWithYpogegrammeni',
u'\u5cf5': 'CjkUnifiedIdeograph5cf5',
u'\ua47a': 'YiSyllableYuox',
u'\ue97c': 'UnicodeCharacter0xe97c',
u'\u8dff': 'CjkUnifiedIdeograph8dff',
u'\u2e7e': 'UnicodeCharacter0x2e7e',
u'\u1380': 'EthiopicSyllableSebatbeitMwa',
u'\ub703': 'HangulSyllableDdwis',
u'\u5082': 'CjkUnifiedIdeograph5082',
u'\uf405': 'UnicodeCharacter0xf405',
u'\u3907': 'CjkUnifiedIdeograph3907',
u'\u7e09': 'CjkUnifiedIdeograph7e09',
u'\u818c': 'CjkUnifiedIdeograph818c',
u'\uc68e': 'HangulSyllableOej',
u'\u0b90': 'TamilLetterAi',
u'\uaf13': 'HangulSyllableGgyegs',
u'\u4892': 'CjkUnifiedIdeograph4892',
u'\uec15': 'UnicodeCharacter0xec15',
u'\ud117': 'HangulSyllableTyaegs',
u'\u1619': 'CanadianSyllabicsCarrierJi',
u'\u5b1b': 'CjkUnifiedIdeograph5b1b',
u'\ub99c': 'HangulSyllableRyils',
u'\ufe9e': 'ArabicLetterJeemFinalForm',
u'\u23a0': 'RightParenthesisLowerHook',
u'\u60a2': 'CjkUnifiedIdeograph60a2',
u'\u8425': 'CjkUnifiedIdeograph8425',
u'\uc927': 'HangulSyllableJweolh',
u'\u0e29': 'ThaiCharacterSoRusi',
u'\u732b': 'CjkUnifiedIdeograph732b',
u'\u96ae': 'CjkUnifiedIdeograph96ae',
u'\udbb0': 'UnicodeCharacter0xdbb0',
u'\u18b2': 'UnicodeCharacter0x18b2',
u'\ubc35': 'HangulSyllableBaenj',
u'\u5db4': 'CjkUnifiedIdeograph5db4',
u'\ue137': 'UnicodeCharacter0xe137',
u'\u2639': 'WhiteFrowningFace',
u'\u6b3b': 'CjkUnifiedIdeograph6b3b',
u'\u8ebe': 'CjkUnifiedIdeograph8ebe',
u'\uf3c0': 'UnicodeCharacter0xf3c0',
u'\u30c2': 'KatakanaLetterDi',
u'\u75c4': 'CjkUnifiedIdeograph75c4',
u'\u9947': 'CjkUnifiedIdeograph9947',
u'\ude49': 'UnicodeCharacter0xde49',
u'\u034b': 'CombiningHomotheticAbove',
u'\u404d': 'CjkUnifiedIdeograph404d',
u'\ua6ce': 'UnicodeCharacter0xa6ce',
u'\uebd0': 'UnicodeCharacter0xebd0',
u'\u28d2': 'BraillePatternDots2578',
u'\u6dd4': 'CjkUnifiedIdeograph6dd4',
u'\ub157': 'HangulSyllableNyeoc',
u'\u52d6': 'CjkUnifiedIdeograph52d6',
u'\uf659': 'UnicodeCharacter0xf659',
u'\u3b5b': 'CjkUnifiedIdeograph3b5b',
u'\u785d': 'CjkUnifiedIdeograph785d',
u'\u83e0': 'CjkUnifiedIdeograph83e0',
u'\uc0e2': 'HangulSyllableSaep',
u'\u05e4': 'HebrewLetterPe',
u'\ua967': 'UnicodeCharacter0xa967',
u'\u4ae6': 'CjkUnifiedIdeograph4ae6',
u'\uee69': 'UnicodeCharacter0xee69',
u'\ud36b': 'HangulSyllablePyaelb',
u'\u106d': 'MyanmarSignWesternPwoKarenTone5',
u'\u556f': 'CjkUnifiedIdeograph556f',
u'\ubbf0': 'HangulSyllableMyiss',
u'\uf8f2': 'UnicodeCharacter0xf8f2',
u'\u3df4': 'CjkUnifiedIdeograph3df4',
u'\u62f6': 'CjkUnifiedIdeograph62f6',
u'\u8679': 'CjkUnifiedIdeograph8679',
u'\ucb7b': 'HangulSyllableJjweoc',
u'\u087d': 'UnicodeCharacter0x87d',
u'\u4d7f': 'CjkUnifiedIdeograph4d7f',
u'\u3581': 'CjkUnifiedIdeograph3581',
u'\ud204': 'HangulSyllableToem',
u'\ufe87': 'ArabicLetterAlefWithHamzaBelowIsolatedForm',
u'\u5808': 'CjkUnifiedIdeograph5808',
u'\u648b': 'CjkUnifiedIdeograph648b',
u'\u9b0a': 'CjkUnifiedIdeograph9b0a',
u'\ua78d': 'UnicodeCharacter0xa78d',
u'\u010e': 'LatinCapitalLetterDWithCaron',
u'\u2d91': 'EthiopicSyllablePhoa',
u'\uca14': 'HangulSyllableJjya',
u'\u9697': 'CjkUnifiedIdeograph9697',
u'\u7018': 'CjkUnifiedIdeograph7018',
u'\u1c9b': 'UnicodeCharacter0x1c9b',
u'\ub31a': 'HangulSyllableDaep',
u'\u391e': 'CjkUnifiedIdeograph391e',
u'\uc5a1': 'HangulSyllableYaelg',
u'\u43a5': 'CjkUnifiedIdeograph43a5',
u'\ue224': 'UnicodeCharacter0xe224',
u'\u8ea7': 'CjkUnifiedIdeograph8ea7',
u'\u6828': 'CjkUnifiedIdeograph6828',
u'\u34ab': 'CjkUnifiedIdeograph34ab',
u'\uab2a': 'UnicodeCharacter0xab2a',
u'\ud12e': 'HangulSyllableTyaep',
u'\ufdb1': 'ArabicLigatureMeemWithMeemWithYehFinalForm',
u'\u5f32': 'CjkUnifiedIdeograph5f32',
u'\u7bb5': 'CjkUnifiedIdeograph7bb5',
u'\u9a34': 'CjkUnifiedIdeograph9a34',
u'\ua6b7': 'UnicodeCharacter0xa6b7',
u'\u2cbb': 'CopticSmallLetterDialectpNi',
u'\uc93e': 'HangulSyllableJwelm',
u'\u95c1': 'CjkUnifiedIdeograph95c1',
u'\u7742': 'CjkUnifiedIdeograph7742',
u'\u13c5': 'CherokeeLetterNv',
u'\ub244': 'HangulSyllableNwel',
u'\u3848': 'CjkUnifiedIdeograph3848',
u'\uc4cb': 'HangulSyllableSswis',
u'\u42cf': 'CjkUnifiedIdeograph42cf',
u'\ue14e': 'UnicodeCharacter0xe14e',
u'\u8dd1': 'CjkUnifiedIdeograph8dd1',
u'\u6f52': 'CjkUnifiedIdeograph6f52',
u'\u0bd5': 'UnicodeCharacter0xbd5',
u'\uaa54': 'ChamDigitFour',
u'\ud058': 'HangulSyllableKyul',
u'\ufcdb': 'ArabicLigatureYehWithHahInitialForm',
u'\u5e5c': 'CjkUnifiedIdeograph5e5c',
u'\u7adf': 'CjkUnifiedIdeograph7adf',
u'\u995e': 'CjkUnifiedIdeograph995e',
u'\ua5e1': 'VaiSyllableE',
u'\u0762': 'ArabicLetterKehehWithDotAbove',
u'\u23e5': 'Flatness',
u'\uc868': 'HangulSyllableJyess',
u'\u94eb': 'CjkUnifiedIdeograph94eb',
u'\u766c': 'CjkUnifiedIdeograph766c',
u'\u12ef': 'EthiopicSyllableYoa',
u'\ub16e': 'HangulSyllableNyebs',
u'\u3f72': 'CjkUnifiedIdeograph3f72',
u'\udbf5': 'UnicodeCharacter0xdbf5',
u'\u41f9': 'CjkUnifiedIdeograph41f9',
u'\ue078': 'UnicodeCharacter0xe078',
u'\u8cfb': 'CjkUnifiedIdeograph8cfb',
u'\u6e7c': 'CjkUnifiedIdeograph6e7c',
u'\u0aff': 'UnicodeCharacter0xaff',
u'\ua97e': 'UnicodeCharacter0xa97e',
u'\u3003': 'DittoMark',
u'\ud782': 'HangulSyllableHyij',
u'\u5d86': 'CjkUnifiedIdeograph5d86',
u'\uf909': 'CjkCompatibilityIdeographf909',
u'\u9888': 'CjkUnifiedIdeograph9888',
u'\u670d': 'CjkUnifiedIdeograph670d',
u'\u068c': 'ArabicLetterDahal',
u'\ua20f': 'YiSyllableKux',
u'\u2813': 'BraillePatternDots125',
u'\ucf92': 'HangulSyllableKwaenh',
u'\u5617': 'CjkUnifiedIdeograph5617',
u'\u7596': 'CjkUnifiedIdeograph7596',
u'\u9119': 'CjkUnifiedIdeograph9119',
u'\ub098': 'HangulSyllableNa',
u'\u1f1d': 'GreekCapitalLetterEpsilonWithDasiaAndOxia',
u'\u3e9c': 'CjkUnifiedIdeograph3e9c',
u'\uc023': 'HangulSyllableBbwigs',
u'\ue7a2': 'UnicodeCharacter0xe7a2',
u'\u4e27': 'CjkUnifiedIdeograph4e27',
u'\u6da6': 'CjkUnifiedIdeograph6da6',
u'\u8929': 'CjkUnifiedIdeograph8929',
u'\ua8a8': 'SaurashtraLetterBa',
u'\u372d': 'CjkUnifiedIdeograph372d',
u'\ud6ac': 'HangulSyllableHyon',
u'\u5cb0': 'CjkUnifiedIdeograph5cb0',
u'\uf833': 'UnicodeCharacter0xf833',
u'\u9fb2': 'CjkUnifiedIdeograph9fb2',
u'\u6637': 'CjkUnifiedIdeograph6637',
u'\u05b6': 'HebrewPointSegol',
u'\ua139': 'YiSyllableDdip',
u'\u2f3d': 'KangxiRadicalHalberd',
u'\ucebc': 'HangulSyllableKyam',
u'\u5541': 'CjkUnifiedIdeograph5541',
u'\u74c0': 'CjkUnifiedIdeograph74c0',
u'\u9043': 'CjkUnifiedIdeograph9043',
u'\ub7c2': 'HangulSyllableRyalp',
u'\u1e47': 'LatinSmallLetterNWithDotBelow',
u'\u3dc6': 'CjkUnifiedIdeograph3dc6',
u'\uc74d': 'HangulSyllableEub',
u'\ue6cc': 'UnicodeCharacter0xe6cc',
u'\u4d51': 'CjkUnifiedIdeograph4d51',
u'\u6cd0': 'CjkUnifiedIdeograph6cd0',
u'\u8853': 'CjkUnifiedIdeograph8853',
u'\uafd2': 'HangulSyllableGgup',
u'\u3657': 'CjkUnifiedIdeograph3657',
u'\ud5d6': 'HangulSyllableHeolp',
u'\u53da': 'CjkUnifiedIdeograph53da',
u'\uff5d': 'FullwidthRightCurlyBracket',
u'\u9edc': 'CjkUnifiedIdeograph9edc',
u'\u6561': 'CjkUnifiedIdeograph6561',
u'\u04e0': 'CyrillicCapitalLetterAbkhasianDze',
u'\ua063': 'YiSyllableBbuo',
u'\u2e67': 'UnicodeCharacter0x2e67',
u'\ucde6': 'HangulSyllableCwep',
u'\u546b': 'CjkUnifiedIdeograph546b',
u'\u4bea': 'CjkUnifiedIdeograph4bea',
u'\u976d': 'CjkUnifiedIdeograph976d',
u'\ub6ec': 'HangulSyllableDdwek',
u'\u1d71': 'LatinSmallLetterPWithMiddleTilde',
u'\u3cf0': 'CjkUnifiedIdeograph3cf0',
u'\uc677': 'HangulSyllableWaeh',
u'\ue5f6': 'UnicodeCharacter0xe5f6',
u'\u4c7b': 'CjkUnifiedIdeograph4c7b',
u'\u63fa': 'CjkUnifiedIdeograph63fa',
u'\u8f7d': 'CjkUnifiedIdeograph8f7d',
u'\uaefc': 'HangulSyllableGgyeol',
u'\ub481': 'HangulSyllableDweong',
u'\ud500': 'HangulSyllablePyuk',
u'\uf183': 'UnicodeCharacter0xf183',
u'\u1602': 'CanadianSyllabicsCarrierNu',
u'\u3285': 'CircledIdeographSix',
u'\u5304': 'CjkUnifiedIdeograph5304',
u'\u7f87': 'CjkUnifiedIdeograph7f87',
u'\u800e': 'CjkUnifiedIdeograph800e',
u'\uac91': 'HangulSyllableGenj',
u'\ucd10': 'HangulSyllableCol',
u'\ue993': 'UnicodeCharacter0xe993',
u'\u0e12': 'ThaiCharacterThoPhuthao',
u'\u2a95': 'SlantedEqualToOrLessthan',
u'\u4b14': 'CjkUnifiedIdeograph4b14',
u'\u1797': 'KhmerLetterPho',
u'\u5099': 'CjkUnifiedIdeograph5099',
u'\ub81e': 'HangulSyllableRej',
u'\ue520': 'UnicodeCharacter0xe520',
u'\u81a3': 'CjkUnifiedIdeograph81a3',
u'\u2622': 'RadioactiveSign',
u'\uc2a5': 'HangulSyllableSeug',
u'\u6324': 'CjkUnifiedIdeograph6324',
u'\u0fa7': 'TibetanSubjoinedLetterBha',
u'\u48a9': 'CjkUnifiedIdeograph48a9',
u'\u9d30': 'CjkUnifiedIdeograph9d30',
u'\ub9b3': 'HangulSyllableRid',
u'\ude32': 'UnicodeCharacter0xde32',
u'\ufab5': 'CjkCompatibilityIdeographfab5',
u'\u1b34': 'BalineseSignRerekan',
u'\u27b7': 'HeavyBlackfeatheredSouthEastArrow',
u'\u4436': 'CjkUnifiedIdeograph4436',
u'\u60b9': 'CjkUnifiedIdeograph60b9',
u'\ub540': 'HangulSyllableDdam',
u'\uf642': 'UnicodeCharacter0xf642',
u'\u92c5': 'CjkUnifiedIdeograph92c5',
u'\u3344': 'SquareMairu',
u'\udfc7': 'UnicodeCharacter0xdfc7',
u'\u7c46': 'CjkUnifiedIdeograph7c46',
u'\u18c9': 'UnicodeCharacter0x18c9',
u'\u45cb': 'CjkUnifiedIdeograph45cb',
u'\uad50': 'HangulSyllableGyo',
u'\uee52': 'UnicodeCharacter0xee52',
u'\u8ad5': 'CjkUnifiedIdeograph8ad5',
u'\u2b54': 'WhiteRightpointingPentagon',
u'\uf7d7': 'UnicodeCharacter0xf7d7',
u'\u1456': 'CanadianSyllabicsTaa',
u'\u30d9': 'KatakanaLetterBe',
u'\u5158': 'CjkUnifiedIdeograph5158',
u'\u7ddb': 'CjkUnifiedIdeograph7ddb',
u'\u8662': 'CjkUnifiedIdeograph8662',
u'\ua2e5': 'YiSyllableZzurx',
u'\uc364': 'HangulSyllableSsyaek',
u'\uefe7': 'UnicodeCharacter0xefe7',
u'\u0c66': 'TeluguDigitZero',
u'\u28e9': 'BraillePatternDots14678',
u'\u4968': 'CjkUnifiedIdeograph4968',
u'\u15eb': 'CanadianSyllabicsCarrierPee',
u'\u56ed': 'CjkUnifiedIdeograph56ed',
u'\ube72': 'HangulSyllableBbabs',
u'\ufb74': 'ArabicLetterDyehInitialForm',
u'\u87f7': 'CjkUnifiedIdeograph87f7',
u'\u2476': 'ParenthesizedDigitThree',
u'\uc0f9': 'HangulSyllableSyang',
u'\u6178': 'CjkUnifiedIdeograph6178',
u'\u0dfb': 'UnicodeCharacter0xdfb',
u'\u4efd': 'CjkUnifiedIdeograph4efd',
u'\u7401': 'CjkUnifiedIdeograph7401',
u'\u9384': 'CjkUnifiedIdeograph9384',
u'\udc86': 'UnicodeCharacter0xdc86',
u'\u1988': 'NewTaiLueLetterHighTsa',
u'\ua50b': 'VaiSyllableGbee',
u'\u5a8a': 'CjkUnifiedIdeograph5a8a',
u'\ue60d': 'UnicodeCharacter0xe60d',
u'\u230f': 'TopLeftCrop',
u'\u6c11': 'CjkUnifiedIdeograph6c11',
u'\u8b94': 'CjkUnifiedIdeograph8b94',
u'\uf496': 'UnicodeCharacter0xf496',
u'\u3198': 'IdeographicAnnotationBottomMark',
u'\u729a': 'CjkUnifiedIdeograph729a',
u'\u9e1d': 'CjkUnifiedIdeograph9e1d',
u'\udb1f': 'UnicodeCharacter0xdb1f',
u'\u0421': 'CyrillicCapitalLetterEs',
u'\u4123': 'CjkUnifiedIdeograph4123',
u'\ua3a4': 'YiSyllableNrur',
u'\ueca6': 'UnicodeCharacter0xeca6',
u'\u29a8': 'MeasuredAngleWithOpenArmEndingInArrowPointingUpAndRight',
u'\u6aaa': 'CjkUnifiedIdeograph6aaa',
u'\ub62d': 'HangulSyllableDdwag',
u'\u57ac': 'CjkUnifiedIdeograph57ac',
u'\uf32f': 'UnicodeCharacter0xf32f',
u'\u3c31': 'CjkUnifiedIdeograph3c31',
u'\u7933': 'CjkUnifiedIdeograph7933',
u'\u84b6': 'CjkUnifiedIdeograph84b6',
u'\uc1b8': 'HangulSyllableSwam',
u'\u02ba': 'ModifierLetterDoublePrime',
u'\uae3d': 'HangulSyllableGilt',
u'\u4fbc': 'CjkUnifiedIdeograph4fbc',
u'\ueb3f': 'UnicodeCharacter0xeb3f',
u'\ud441': 'HangulSyllablePoeg',
u'\u1143': 'HangulChoseongIeungmieum',
u'\u5245': 'CjkUnifiedIdeograph5245',
u'\ubcc6': 'HangulSyllableByeolm',
u'\uf9c8': 'CjkCompatibilityIdeographf9c8',
u'\u3aca': 'CjkUnifiedIdeograph3aca',
u'\u67cc': 'CjkUnifiedIdeograph67cc',
u'\u834f': 'CjkUnifiedIdeograph834f',
u'\ucc51': 'HangulSyllableCaelt',
u'\u0953': 'DevanagariGraveAccent',
u'\u4a55': 'CjkUnifiedIdeograph4a55',
u'\u91d8': 'CjkUnifiedIdeograph91d8',
u'\ud2da': 'HangulSyllableTyinh',
u'\u1fdc': 'UnicodeCharacter0x1fdc',
u'\ubb5f': 'HangulSyllableMweolh',
u'\u58de': 'CjkUnifiedIdeograph58de',
u'\ue461': 'UnicodeCharacter0xe461',
u'\u2163': 'RomanNumeralFour',
u'\u6265': 'CjkUnifiedIdeograph6265',
u'\u89e8': 'CjkUnifiedIdeograph89e8',
u'\ucaea': 'HangulSyllableJjwabs',
u'\u37ec': 'CjkUnifiedIdeograph37ec',
u'\u70ee': 'CjkUnifiedIdeograph70ee',
u'\u9c71': 'CjkUnifiedIdeograph9c71',
u'\ud973': 'UnicodeCharacter0xd973',
u'\u1a75': 'UnicodeCharacter0x1a75',
u'\u4777': 'CjkUnifiedIdeograph4777',
u'\ua1f8': 'YiSyllableKit',
u'\ue2fa': 'UnicodeCharacter0xe2fa',
u'\u2ffc': 'UnicodeCharacter0x2ffc',
u'\u68fe': 'CjkUnifiedIdeograph68fe',
u'\u5600': 'CjkUnifiedIdeograph5600',
u'\u7283': 'CjkUnifiedIdeograph7283',
u'\u9502': 'CjkUnifiedIdeograph9502',
u'\ub185': 'HangulSyllableNolt',
u'\u1f06': 'GreekSmallLetterAlphaWithPsiliAndPerispomeni',
u'\u3b89': 'CjkUnifiedIdeograph3b89',
u'\uc40c': 'HangulSyllableSswak',
u'\ue08f': 'UnicodeCharacter0xe08f',
u'\u4e10': 'CjkUnifiedIdeograph4e10',
u'\u6a93': 'CjkUnifiedIdeograph6a93',
u'\u8d12': 'CjkUnifiedIdeograph8d12',
u'\ua995': 'UnicodeCharacter0xa995',
u'\u3716': 'CjkUnifiedIdeograph3716',
u'\ud399': 'HangulSyllablePeg',
u'\u5d9d': 'CjkUnifiedIdeograph5d9d',
u'\ufc1c': 'ArabicLigatureSeenWithJeemIsolatedForm',
u'\u989f': 'CjkUnifiedIdeograph989f',
u'\u6620': 'CjkUnifiedIdeograph6620',
u'\u02a3': 'LatinSmallLetterDzDigraph',
u'\ua522': 'VaiSyllableNee',
u'\u2f26': 'KangxiRadicalChild',
u'\ucba9': 'HangulSyllableJjwilt',
u'\u75ad': 'CjkUnifiedIdeograph75ad',
u'\u942c': 'CjkUnifiedIdeograph942c',
u'\ub0af': 'HangulSyllableNac',
u'\u1e30': 'LatinCapitalLetterKWithAcute',
u'\u3ab3': 'CjkUnifiedIdeograph3ab3',
u'\uc736': 'HangulSyllableYuj',
u'\ue3b9': 'UnicodeCharacter0xe3b9',
u'\u413a': 'CjkUnifiedIdeograph413a',
u'\u6dbd': 'CjkUnifiedIdeograph6dbd',
u'\u8c3c': 'CjkUnifiedIdeograph8c3c',
u'\ua8bf': 'SaurashtraVowelSignEe',
u'\u3640': 'CjkUnifiedIdeograph3640',
u'\ud2c3': 'HangulSyllableTeulb',
u'\u5cc7': 'CjkUnifiedIdeograph5cc7',
u'\uff46': 'FullwidthLatinSmallLetterF',
u'\u9bc9': 'CjkUnifiedIdeograph9bc9',
u'\u794a': 'CjkUnifiedIdeograph794a',
u'\u05cd': 'UnicodeCharacter0x5cd',
u'\ua44c': 'YiSyllableNyi',
u'\u2e50': 'UnicodeCharacter0x2e50',
u'\ucad3': 'HangulSyllableJjoc',
u'\u74d7': 'CjkUnifiedIdeograph74d7',
u'\u9756': 'CjkUnifiedIdeograph9756',
u'\ub3d9': 'HangulSyllableDong',
u'\u115a': 'UnicodeCharacter0x115a',
u'\u3ddd': 'CjkUnifiedIdeograph3ddd',
u'\uc660': 'HangulSyllableWaen',
u'\ue2e3': 'UnicodeCharacter0xe2e3',
u'\u4064': 'CjkUnifiedIdeograph4064',
u'\u6ce7': 'CjkUnifiedIdeograph6ce7',
u'\u8f66': 'CjkUnifiedIdeograph8f66',
u'\uabe9': 'UnicodeCharacter0xabe9',
u'\u096a': 'DevanagariDigitFour',
u'\ud5ed': 'HangulSyllableHelg',
u'\u5ff1': 'CjkUnifiedIdeograph5ff1',
u'\ufe70': 'ArabicFathatanIsolatedForm',
u'\u9af3': 'CjkUnifiedIdeograph9af3',
u'\u7874': 'CjkUnifiedIdeograph7874',
u'\u04f7': 'CyrillicSmallLetterGheWithDescender',
u'\ua776': 'LatinLetterSmallCapitalRum',
u'\u217a': 'SmallRomanNumeralEleven',
u'\ucdfd': 'HangulSyllableCwing',
u'\uf701': 'UnicodeCharacter0xf701',
u'\u9680': 'CjkUnifiedIdeograph9680',
u'\u7105': 'CjkUnifiedIdeograph7105',
u'\u1084': 'MyanmarVowelSignShanE',
u'\ubc07': 'HangulSyllableMilh',
u'\u260b': 'DescendingNode',
u'\ud98a': 'UnicodeCharacter0xd98a',
u'\u438e': 'CjkUnifiedIdeograph438e',
u'\uef11': 'UnicodeCharacter0xef11',
u'\u8e90': 'CjkUnifiedIdeograph8e90',
u'\u6915': 'CjkUnifiedIdeograph6915',
u'\u0894': 'UnicodeCharacter0x894',
u'\ude1b': 'UnicodeCharacter0xde1b',
u'\uf19a': 'UnicodeCharacter0xf19a',
u'\u581f': 'CjkUnifiedIdeograph581f',
u'\u7b9e': 'CjkUnifiedIdeograph7b9e',
u'\u8721': 'CjkUnifiedIdeograph8721',
u'\ua6a0': 'UnicodeCharacter0xa6a0',
u'\u0125': 'LatinSmallLetterHWithCircumflex',
u'\u20a4': 'LiraSign',
u'\uf62b': 'UnicodeCharacter0xf62b',
u'\ue9aa': 'UnicodeCharacter0xe9aa',
u'\u702f': 'CjkUnifiedIdeograph702f',
u'\u13ae': 'CherokeeLetterHe',
u'\ubf31': 'HangulSyllableBbyelt',
u'\u3935': 'CjkUnifiedIdeograph3935',
u'\ud8b4': 'UnicodeCharacter0xd8b4',
u'\u42b8': 'CjkUnifiedIdeograph42b8',
u'\uee3b': 'UnicodeCharacter0xee3b',
u'\u81ba': 'CjkUnifiedIdeograph81ba',
u'\u683f': 'CjkUnifiedIdeograph683f',
u'\u0bbe': 'TamilVowelSignAa',
u'\ud145': 'HangulSyllableTeong',
u'\uf0c4': 'UnicodeCharacter0xf0c4',
u'\u5b49': 'CjkUnifiedIdeograph5b49',
u'\u7ac8': 'CjkUnifiedIdeograph7ac8',
u'\u864b': 'CjkUnifiedIdeograph864b',
u'\ub9ca': 'HangulSyllableMagg',
u'\u23ce': 'ReturnSymbol',
u'\uc955': 'HangulSyllableJwinj',
u'\ue8d4': 'UnicodeCharacter0xe8d4',
u'\u7359': 'CjkUnifiedIdeograph7359',
u'\u12d8': 'EthiopicSyllableZa',
u'\ube5b': 'HangulSyllableBic',
u'\u385f': 'CjkUnifiedIdeograph385f',
u'\udbde': 'UnicodeCharacter0xdbde',
u'\u45e2': 'CjkUnifiedIdeograph45e2',
u'\ue165': 'UnicodeCharacter0xe165',
u'\u80e4': 'CjkUnifiedIdeograph80e4',
u'\u6b69': 'CjkUnifiedIdeograph6b69',
u'\u0ae8': 'GujaratiDigitTwo',
u'\ud06f': 'HangulSyllableKeugs',
u'\uf3ee': 'UnicodeCharacter0xf3ee',
u'\u5a73': 'CjkUnifiedIdeograph5a73',
u'\u7df2': 'CjkUnifiedIdeograph7df2',
u'\u9975': 'CjkUnifiedIdeograph9975',
u'\ub8f4': 'HangulSyllableRuls',
u'\u0379': 'UnicodeCharacter0x379',
u'\u22f8': 'ElementOfWithUnderbar',
u'\uc87f': 'HangulSyllableJolh',
u'\uebfe': 'UnicodeCharacter0xebfe',
u'\u9e06': 'CjkUnifiedIdeograph9e06',
u'\uba89': 'HangulSyllableMyeot',
u'\udb08': 'UnicodeCharacter0xdb08',
u'\ue78b': 'UnicodeCharacter0xe78b',
u'\u180a': 'MongolianNirugu',
u'\u248d': 'DigitSixFullStop',
u'\u450c': 'CjkUnifiedIdeograph450c',
u'\u618f': 'CjkUnifiedIdeograph618f',
u'\ub616': 'HangulSyllableDdonh',
u'\uf318': 'UnicodeCharacter0xf318',
u'\u9f9b': 'CjkUnifiedIdeograph9f9b',
u'\u301a': 'LeftWhiteSquareBracket',
u'\udc9d': 'UnicodeCharacter0xdc9d',
u'\u7d1c': 'CjkUnifiedIdeograph7d1c',
u'\u199f': 'NewTaiLueLetterLowLa',
u'\u46a1': 'CjkUnifiedIdeograph46a1',
u'\uae26': 'HangulSyllableGyibs',
u'\ueb28': 'UnicodeCharacter0xeb28',
u'\ub7ab': 'HangulSyllableRaes',
u'\u282a': 'BraillePatternDots246',
u'\uf4ad': 'UnicodeCharacter0xf4ad',
u'\u152c': 'CanadianSyllabicsYcreeYoo',
u'\u31af': 'BopomofoLetterAunn',
u'\u522e': 'CjkUnifiedIdeograph522e',
u'\u7eb1': 'CjkUnifiedIdeograph7eb1',
u'\u8338': 'CjkUnifiedIdeograph8338',
u'\uafbb': 'HangulSyllableGgugs',
u'\uc03a': 'HangulSyllableBbwip',
u'\uecbd': 'UnicodeCharacter0xecbd',
u'\u0d3c': 'UnicodeCharacter0xd3c',
u'\u29bf': 'CircledBullet',
u'\u4a3e': 'CjkUnifiedIdeograph4a3e',
u'\u16c1': 'RunicLetterIsazIsIssI',
u'\u53c3': 'CjkUnifiedIdeograph53c3',
u'\ubb48': 'HangulSyllableMuss',
u'\uf84a': 'UnicodeCharacter0xf84a',
u'\u84cd': 'CjkUnifiedIdeograph84cd',
u'\u254c': 'BoxDrawingsLightDoubleDashHorizontal',
u'\uc1cf': 'HangulSyllableSwaelb',
u'\u624e': 'CjkUnifiedIdeograph624e',
u'\u0ed1': 'LaoDigitOne',
u'\u4bd3': 'CjkUnifiedIdeograph4bd3',
u'\u905a': 'CjkUnifiedIdeograph905a',
u'\ubcdd': 'HangulSyllableByenj',
u'\udd5c': 'UnicodeCharacter0xdd5c',
u'\uf9df': 'CjkCompatibilityIdeographf9df',
u'\u1a5e': 'UnicodeCharacter0x1a5e',
u'\u26e1': 'UnicodeCharacter0x26e1',
u'\u4760': 'CjkUnifiedIdeograph4760',
u'\u63e3': 'CjkUnifiedIdeograph63e3',
u'\u886a': 'CjkUnifiedIdeograph886a',
u'\uf56c': 'UnicodeCharacter0xf56c',
u'\u91ef': 'CjkUnifiedIdeograph91ef',
u'\u326e': 'CircledHangulKiyeokA',
u'\udef1': 'UnicodeCharacter0xdef1',
u'\u7f70': 'CjkUnifiedIdeograph7f70',
u'\u1bf3': 'UnicodeCharacter0x1bf3',
u'\u58f5': 'CjkUnifiedIdeograph58f5',
u'\ua07a': 'YiSyllableNbiex',
u'\ued7c': 'UnicodeCharacter0xed7c',
u'\u89ff': 'CjkUnifiedIdeograph89ff',
u'\u2a7e': 'GreaterthanOrSlantedEqualTo',
u'\u1780': 'KhmerLetterKa',
u'\ub303': 'HangulSyllableDaegs',
u'\u5482': 'CjkUnifiedIdeograph5482',
u'\uf005': 'UnicodeCharacter0xf005',
u'\u3d07': 'CjkUnifiedIdeograph3d07',
u'\u7a09': 'CjkUnifiedIdeograph7a09',
u'\u858c': 'CjkUnifiedIdeograph858c',
u'\uc28e': 'HangulSyllableSyunh',
u'\u0f90': 'TibetanSubjoinedLetterKa',
u'\uab13': 'UnicodeCharacter0xab13',
u'\u4c92': 'CjkUnifiedIdeograph4c92',
u'\ue815': 'UnicodeCharacter0xe815',
u'\ud517': 'HangulSyllablePeus',
u'\u1219': 'EthiopicSyllableMu',
u'\u5f1b': 'CjkUnifiedIdeograph5f1b',
u'\ubd9c': 'HangulSyllableBweo',
u'\ufa9e': 'CjkCompatibilityIdeographfa9e',
u'\u27a0': 'HeavyDashedTriangleheadedRightwardsArrow',
u'\u64a2': 'CjkUnifiedIdeograph64a2',
u'\u8025': 'CjkUnifiedIdeograph8025',
u'\ucd27': 'HangulSyllableCwags',
u'\u0a29': 'UnicodeCharacter0xa29',
u'\u772b': 'CjkUnifiedIdeograph772b',
u'\u92ae': 'CjkUnifiedIdeograph92ae',
u'\udfb0': 'UnicodeCharacter0xdfb0',
u'\u1cb2': 'UnicodeCharacter0x1cb2',
u'\ub835': 'HangulSyllableRyeob',
u'\u59b4': 'CjkUnifiedIdeograph59b4',
u'\ue537': 'UnicodeCharacter0xe537',
u'\u2239': 'Excess',
u'\u6f3b': 'CjkUnifiedIdeograph6f3b',
u'\u8abe': 'CjkUnifiedIdeograph8abe',
u'\uf7c0': 'UnicodeCharacter0xf7c0',
u'\u34c2': 'CjkUnifiedIdeograph34c2',
u'\u71c4': 'CjkUnifiedIdeograph71c4',
u'\u9d47': 'CjkUnifiedIdeograph9d47',
u'\uda49': 'UnicodeCharacter0xda49',
u'\u074b': 'UnicodeCharacter0x74b',
u'\u444d': 'CjkUnifiedIdeograph444d',
u'\ua2ce': 'YiSyllableCyrx',
u'\uefd0': 'UnicodeCharacter0xefd0',
u'\u2cd2': 'CopticCapitalLetterOldCopticHei',
u'\u69d4': 'CjkUnifiedIdeograph69d4',
u'\ub557': 'HangulSyllableDdaelb',
u'\u56d6': 'CjkUnifiedIdeograph56d6',
u'\uf259': 'UnicodeCharacter0xf259',
u'\u3f5b': 'CjkUnifiedIdeograph3f5b',
u'\u7c5d': 'CjkUnifiedIdeograph7c5d',
u'\u87e0': 'CjkUnifiedIdeograph87e0',
u'\uc4e2': 'HangulSyllableSsyulp',
u'\u01e4': 'LatinCapitalLetterGWithStroke',
u'\uad67': 'HangulSyllableGyoc',
u'\u4ee6': 'CjkUnifiedIdeograph4ee6',
u'\uea69': 'UnicodeCharacter0xea69',
u'\ud76b': 'HangulSyllableHeuh',
u'\u146d': 'CanadianSyllabicsKi',
u'\u516f': 'CjkUnifiedIdeograph516f',
u'\ubff0': 'HangulSyllableBbweol',
u'\ufcf2': 'ArabicLigatureShaddaWithFathaMedialForm',
u'\u39f4': 'CjkUnifiedIdeograph39f4',
u'\u66f6': 'CjkUnifiedIdeograph66f6',
u'\u8279': 'CjkUnifiedIdeograph8279',
u'\ucf7b': 'HangulSyllableKwalb',
u'\u0c7d': 'TeluguFractionDigitTwoForEvenPowersOfFour',
u'\u497f': 'CjkUnifiedIdeograph497f',
u'\u3981': 'CjkUnifiedIdeograph3981',
u'\ude04': 'UnicodeCharacter0xde04',
u'\uf287': 'UnicodeCharacter0xf287',
u'\u5408': 'CjkUnifiedIdeograph5408',
u'\u688b': 'CjkUnifiedIdeograph688b',
u'\u970a': 'CjkUnifiedIdeograph970a',
u'\uab8d': 'UnicodeCharacter0xab8d',
u'\u0d0e': 'MalayalamLetterE',
u'\u2191': 'UpwardsArrow',
u'\uc614': 'HangulSyllableYels',
u'\u9a97': 'CjkUnifiedIdeograph9a97',
u'\u7c18': 'CjkUnifiedIdeograph7c18',
u'\u109b': 'UnicodeCharacter0x109b',
u'\ubf1a': 'HangulSyllableBbyeobs',
u'\u351e': 'CjkUnifiedIdeograph351e',
u'\uc9a1': 'HangulSyllableJeut',
u'\u4fa5': 'CjkUnifiedIdeograph4fa5',
u'\uee24': 'UnicodeCharacter0xee24',
u'\u82a7': 'CjkUnifiedIdeograph82a7',
u'\u6428': 'CjkUnifiedIdeograph6428',
u'\u38ab': 'CjkUnifiedIdeograph38ab',
u'\ua72a': 'LatinCapitalLetterTresillo',
u'\udd2e': 'UnicodeCharacter0xdd2e',
u'\uf1b1': 'UnicodeCharacter0xf1b1',
u'\u5332': 'CjkUnifiedIdeograph5332',
u'\u77b5': 'CjkUnifiedIdeograph77b5',
u'\u9634': 'CjkUnifiedIdeograph9634',
u'\uaab7': 'UnicodeCharacter0xaab7',
u'\u0c38': 'TeluguLetterSa',
u'\u20bb': 'UnicodeCharacter0x20bb',
u'\uc53e': 'HangulSyllableSsij',
u'\u99c1': 'CjkUnifiedIdeograph99c1',
u'\u7b42': 'CjkUnifiedIdeograph7b42',
u'\u1fc5': 'UnicodeCharacter0x1fc5',
u'\ube44': 'HangulSyllableBi',
u'\u3448': 'CjkUnifiedIdeograph3448',
u'\uc8cb': 'HangulSyllableJoed',
u'\u4ecf': 'CjkUnifiedIdeograph4ecf',
u'\ued4e': 'UnicodeCharacter0xed4e',
u'\u81d1': 'CjkUnifiedIdeograph81d1',
u'\u6352': 'CjkUnifiedIdeograph6352',
u'\u07d5': 'NkoLetterTa',
u'\ua654': 'CyrillicCapitalLetterReversedYu',
u'\udc58': 'UnicodeCharacter0xdc58',
u'\uf0db': 'UnicodeCharacter0xf0db',
u'\u525c': 'CjkUnifiedIdeograph525c',
u'\u76df': 'CjkUnifiedIdeograph76df',
u'\u955e': 'CjkUnifiedIdeograph955e',
u'\ua9e1': 'UnicodeCharacter0xa9e1',
u'\u0b62': 'OriyaVowelSignVocalicL',
u'\u2fe5': 'UnicodeCharacter0x2fe5',
u'\uc468': 'HangulSyllableSsun',
u'\u98eb': 'CjkUnifiedIdeograph98eb',
u'\u7a6c': 'CjkUnifiedIdeograph7a6c',
u'\u1eef': 'LatinSmallLetterUWithHornAndTilde',
u'\ubd6e': 'HangulSyllableByolm',
u'\u3372': 'SquareDa',
u'\ud7f5': 'UnicodeCharacter0xd7f5',
u'\u4df9': 'HexagramForTheJoyousLake',
u'\uec78': 'UnicodeCharacter0xec78',
u'\u80fb': 'CjkUnifiedIdeograph80fb',
u'\u627c': 'CjkUnifiedIdeograph627c',
u'\u06ff': 'ArabicLetterHehWithInvertedV',
u'\ua57e': 'VaiSyllableVoo',
u'\u3c03': 'CjkUnifiedIdeograph3c03',
u'\udb82': 'UnicodeCharacter0xdb82',
u'\u5186': 'CjkUnifiedIdeograph5186',
u'\uf509': 'UnicodeCharacter0xf509',
u'\u9488': 'CjkUnifiedIdeograph9488',
u'\u6b0d': 'CjkUnifiedIdeograph6b0d',
u'\u0a8c': 'GujaratiLetterVocalicL',
u'\uae0f': 'HangulSyllableGeuc',
u'\u2413': 'SymbolForDeviceControlThree',
u'\uc392': 'HangulSyllableSselp',
u'\u5a17': 'CjkUnifiedIdeograph5a17',
u'\u7996': 'CjkUnifiedIdeograph7996',
u'\u9d19': 'CjkUnifiedIdeograph9d19',
u'\ubc98': 'HangulSyllableBeoss',
u'\u131d': 'EthiopicSyllableGge',
u'\u329c': 'CircledIdeographSuitable',
u'\ucc23': 'HangulSyllableJjic',
u'\ueba2': 'UnicodeCharacter0xeba2',
u'\u4227': 'CjkUnifiedIdeograph4227',
u'\u61a6': 'CjkUnifiedIdeograph61a6',
u'\u8529': 'CjkUnifiedIdeograph8529',
u'\ua4a8': 'YiRadicalTu',
u'\u3b2d': 'CjkUnifiedIdeograph3b2d',
u'\udaac': 'UnicodeCharacter0xdaac',
u'\u50b0': 'CjkUnifiedIdeograph50b0',
u'\uf433': 'UnicodeCharacter0xf433',
u'\u93b2': 'CjkUnifiedIdeograph93b2',
u'\u6a37': 'CjkUnifiedIdeograph6a37',
u'\u09b6': 'BengaliLetterSha',
u'\uad39': 'HangulSyllableGoenj',
u'\u233d': 'AplFunctionalSymbolCircleStile',
u'\uc2bc': 'HangulSyllableSeuk',
u'\u5941': 'CjkUnifiedIdeograph5941',
u'\u78c0': 'CjkUnifiedIdeograph78c0',
u'\u9c43': 'CjkUnifiedIdeograph9c43',
u'\ubbc2': 'HangulSyllableMeugg',
u'\u1247': 'EthiopicSyllableQoa',
u'\u31c6': 'CjkStrokeHzg',
u'\ucb4d': 'HangulSyllableJjunj',
u'\ueacc': 'UnicodeCharacter0xeacc',
u'\u4151': 'CjkUnifiedIdeograph4151',
u'\u60d0': 'CjkUnifiedIdeograph60d0',
u'\u8453': 'CjkUnifiedIdeograph8453',
u'\ua3d2': 'YiSyllableRe',
u'\u3a57': 'CjkUnifiedIdeograph3a57',
u'\ud9d6': 'UnicodeCharacter0xd9d6',
u'\u5fda': 'CjkUnifiedIdeograph5fda',
u'\uf35d': 'UnicodeCharacter0xf35d',
u'\u92dc': 'CjkUnifiedIdeograph92dc',
u'\u6961': 'CjkUnifiedIdeograph6961',
u'\u08e0': 'UnicodeCharacter0x8e0',
u'\uac63': 'HangulSyllableGyaelh',
u'\u2267': 'GreaterthanOverEqualTo',
u'\uc1e6': 'HangulSyllableSoenh',
u'\u586b': 'CjkUnifiedIdeograph586b',
u'\u47ea': 'CjkUnifiedIdeograph47ea',
u'\u9b6d': 'CjkUnifiedIdeograph9b6d',
u'\ubaec': 'HangulSyllableMwaels',
u'\u1171': 'HangulJungseongWi',
u'\u30f0': 'KatakanaLetterWi',
u'\uca77': 'HangulSyllableJjelh',
u'\ue9f6': 'UnicodeCharacter0xe9f6',
u'\u407b': 'CjkUnifiedIdeograph407b',
u'\u6ffa': 'CjkUnifiedIdeograph6ffa',
u'\u837d': 'CjkUnifiedIdeograph837d',
u'\ua2fc': 'YiSyllableNzex',
u'\ub881': 'HangulSyllableRwalg',
u'\ud900': 'UnicodeCharacter0xd900',
u'\ufd83': 'ArabicLigatureLamWithJeemWithJeemInitialForm',
u'\u1a02': 'BugineseLetterNga',
u'\u3e85': 'CjkUnifiedIdeograph3e85',
u'\u5f04': 'CjkUnifiedIdeograph5f04',
u'\u7387': 'CjkUnifiedIdeograph7387',
u'\u8c0e': 'CjkUnifiedIdeograph8c0e',
u'\ua091': 'YiSyllableHmit',
u'\uc110': 'HangulSyllableSyaem',
u'\ue593': 'UnicodeCharacter0xe593',
u'\u0212': 'LatinCapitalLetterRWithInvertedBreve',
u'\u2695': 'StaffOfAesculapius',
u'\u4714': 'CjkUnifiedIdeograph4714',
u'\u1b97': 'SundaneseLetterVa',
u'\u5c99': 'CjkUnifiedIdeograph5c99',
u'\ub41e': 'HangulSyllableDoenh',
u'\ue920': 'UnicodeCharacter0xe920',
u'\u8da3': 'CjkUnifiedIdeograph8da3',
u'\u2a22': 'PlusSignWithSmallCircleAbove',
u'\ucea5': 'HangulSyllableKaeng',
u'\u6f24': 'CjkUnifiedIdeograph6f24',
u'\u03a7': 'GreekCapitalLetterChi',
u'\u44a9': 'CjkUnifiedIdeograph44a9',
u'\u9130': 'CjkUnifiedIdeograph9130',
u'\ub5b3': 'HangulSyllableDdeos',
u'\ud232': 'HangulSyllableTunh',
u'\uf6b5': 'UnicodeCharacter0xf6b5',
u'\u1734': 'HanunooSignPamudpod',
u'\u2bb7': 'UnicodeCharacter0x2bb7',
u'\u4836': 'CjkUnifiedIdeograph4836',
u'\u6cb9': 'CjkUnifiedIdeograph6cb9',
u'\ub940': 'HangulSyllableRwin',
u'\ufa42': 'CjkCompatibilityIdeographfa42',
u'\u9ec5': 'CjkUnifiedIdeograph9ec5',
u'\u3f44': 'CjkUnifiedIdeograph3f44',
u'\ud3c7': 'HangulSyllablePyeos',
u'\u7046': 'CjkUnifiedIdeograph7046',
u'\u14c9': 'CanadianSyllabicsNwe',
u'\u49cb': 'CjkUnifiedIdeograph49cb',
u'\ua150': 'YiSyllableDdur',
u'\ue252': 'UnicodeCharacter0xe252',
u'\u86d5': 'CjkUnifiedIdeograph86d5',
u'\u2754': 'UnicodeCharacter0x2754',
u'\ufbd7': 'ArabicLetterUIsolatedForm',
u'\u1856': 'MongolianLetterTodoWa',
u'\u3cd9': 'CjkUnifiedIdeograph3cd9',
u'\u5d58': 'CjkUnifiedIdeograph5d58',
u'\u71db': 'CjkUnifiedIdeograph71db',
u'\u8a62': 'CjkUnifiedIdeograph8a62',
u'\uaee5': 'HangulSyllableGgelt',
u'\ucf64': 'HangulSyllableKom',
u'\ue3e7': 'UnicodeCharacter0xe3e7',
u'\u24e9': 'CircledLatinSmallLetterZ',
u'\u4568': 'CjkUnifiedIdeograph4568',
u'\u19eb': 'KhmerSymbolDapmuoyKoet',
u'\u5aed': 'CjkUnifiedIdeograph5aed',
u'\ub272': 'HangulSyllableNwip',
u'\uf774': 'UnicodeCharacter0xf774',
u'\u8bf7': 'CjkUnifiedIdeograph8bf7',
u'\u2876': 'BraillePatternDots23567',
u'\uccf9': 'HangulSyllableCyelt',
u'\u6d78': 'CjkUnifiedIdeograph6d78',
u'\u01fb': 'LatinSmallLetterAWithRingAboveAndAcute',
u'\u42fd': 'CjkUnifiedIdeograph42fd',
u'\u7801': 'CjkUnifiedIdeograph7801',
u'\u9f84': 'CjkUnifiedIdeograph9f84',
u'\ud086': 'HangulSyllableKeup',
u'\u1588': 'CanadianSyllabicsTlho',
u'\ua90b': 'KayahLiLetterKha',
u'\u568a': 'CjkUnifiedIdeograph568a',
u'\uea0d': 'UnicodeCharacter0xea0d',
u'\u2f0f': 'KangxiRadicalTable',
u'\u6011': 'CjkUnifiedIdeograph6011',
u'\u8794': 'CjkUnifiedIdeograph8794',
u'\uf896': 'UnicodeCharacter0xf896',
u'\u3d98': 'CjkUnifiedIdeograph3d98',
u'\u7e9a': 'CjkUnifiedIdeograph7e9a',
u'\u921d': 'CjkUnifiedIdeograph921d',
u'\ud71f': 'HangulSyllableHwid',
u'\u0821': 'UnicodeCharacter0x821',
u'\u4d23': 'CjkUnifiedIdeograph4d23',
u'\uafa4': 'HangulSyllableGgyol',
u'\ue0a6': 'UnicodeCharacter0xe0a6',
u'\u25a8': 'SquareWithUpperRightToLowerLeftFill',
u'\u66aa': 'CjkUnifiedIdeograph66aa',
u'\uba2d': 'HangulSyllableMyaeb',
u'\u5bac': 'CjkUnifiedIdeograph5bac',
u'\uff2f': 'FullwidthLatinCapitalLetterO',
u'\u3031': 'VerticalKanaRepeatMark',
u'\u7533': 'CjkUnifiedIdeograph7533',
u'\u88b6': 'CjkUnifiedIdeograph88b6',
u'\ucdb8': 'HangulSyllableCweol',
u'\u0eba': 'UnicodeCharacter0xeba',
u'\ua23d': 'YiSyllableMgex',
u'\u43bc': 'CjkUnifiedIdeograph43bc',
u'\ue73f': 'UnicodeCharacter0xe73f',
u'\ud841': 'UnicodeCharacter0xd841',
u'\u1d43': 'ModifierLetterSmallA',
u'\u5e45': 'CjkUnifiedIdeograph5e45',
u'\ub0c6': 'HangulSyllableNaebs',
u'\uf5c8': 'UnicodeCharacter0xf5c8',
u'\u36ca': 'CjkUnifiedIdeograph36ca',
u'\u6bcc': 'CjkUnifiedIdeograph6bcc',
u'\u8f4f': 'CjkUnifiedIdeograph8f4f',
u'\uc051': 'HangulSyllableBbyung',
u'\u0553': 'ArmenianCapitalLetterPiwr',
u'\u4655': 'CjkUnifiedIdeograph4655',
u'\u9dd8': 'CjkUnifiedIdeograph9dd8',
u'\udeda': 'UnicodeCharacter0xdeda',
u'\u13dc': 'CherokeeLetterDla',
u'\ub75f': 'HangulSyllableDdyih',
u'\u54de': 'CjkUnifiedIdeograph54de',
u'\ue861': 'UnicodeCharacter0xe861',
u'\u2d63': 'TifinaghLetterYaz',
u'\u6e65': 'CjkUnifiedIdeograph6e65',
u'\u85e8': 'CjkUnifiedIdeograph85e8',
u'\uc6ea': 'HangulSyllableWegg',
u'\u3bec': 'CjkUnifiedIdeograph3bec',
u'\u7cee': 'CjkUnifiedIdeograph7cee',
u'\u9071': 'CjkUnifiedIdeograph9071',
u'\ud573': 'HangulSyllableHah',
u'\u1675': 'CanadianSyllabicsNnga',
u'\u4b77': 'CjkUnifiedIdeograph4b77',
u'\uadf8': 'HangulSyllableGeu',
u'\ueefa': 'UnicodeCharacter0xeefa',
u'\u23fc': 'UnicodeCharacter0x23fc',
u'\u64fe': 'CjkUnifiedIdeograph64fe',
u'\u5a00': 'CjkUnifiedIdeograph5a00',
u'\u7e83': 'CjkUnifiedIdeograph7e83',
u'\u9902': 'CjkUnifiedIdeograph9902',
u'\ubd85': 'HangulSyllableBunj',
u'\u1306': 'EthiopicSyllableJo',
u'\u3789': 'CjkUnifiedIdeograph3789',
u'\uc80c': 'HangulSyllableJeols',
u'\uec8f': 'UnicodeCharacter0xec8f',
u'\u4210': 'CjkUnifiedIdeograph4210',
u'\u6693': 'CjkUnifiedIdeograph6693',
u'\u8112': 'CjkUnifiedIdeograph8112',
u'\ua595': 'VaiSyllableU',
u'\u3b16': 'CjkUnifiedIdeograph3b16',
u'\udf99': 'UnicodeCharacter0xdf99',
u'\u519d': 'CjkUnifiedIdeograph519d',
u'\uf01c': 'UnicodeCharacter0xf01c',
u'\u949f': 'CjkUnifiedIdeograph949f',
u'\u6a20': 'CjkUnifiedIdeograph6a20',
u'\u0ea3': 'LaoLetterLoLing',
u'\ua922': 'KayahLiLetterA',
u'\u2326': 'EraseToTheRight',
u'\uc7a9': 'HangulSyllableJat',
u'\u79ad': 'CjkUnifiedIdeograph79ad',
u'\u982c': 'CjkUnifiedIdeograph982c',
u'\ubcaf': 'HangulSyllableBelh',
u'\u1230': 'EthiopicSyllableSa',
u'\u36b3': 'CjkUnifiedIdeograph36b3',
u'\ucb36': 'HangulSyllableJjyolm',
u'\uefb9': 'UnicodeCharacter0xefb9',
u'\u4d3a': 'CjkUnifiedIdeograph4d3a',
u'\u61bd': 'CjkUnifiedIdeograph61bd',
u'\u803c': 'CjkUnifiedIdeograph803c',
u'\ua4bf': 'YiRadicalHxop',
u'\u3a40': 'CjkUnifiedIdeograph3a40',
u'\udec3': 'UnicodeCharacter0xdec3',
u'\u50c7': 'CjkUnifiedIdeograph50c7',
u'\uf346': 'UnicodeCharacter0xf346',
u'\u97c9': 'CjkUnifiedIdeograph97c9',
u'\u754a': 'CjkUnifiedIdeograph754a',
u'\u09cd': 'BengaliSignVirama',
u'\ua84c': 'PhagspaLetterPa',
u'\u2250': 'ApproachesTheLimit',
u'\uc6d3': 'HangulSyllableWeod',
u'\u78d7': 'CjkUnifiedIdeograph78d7',
u'\u9b56': 'CjkUnifiedIdeograph9b56',
u'\ubfd9': 'HangulSyllableBbult',
u'\u1d5a': 'ModifierLetterSmallTurnedM',
u'\u31dd': 'CjkStrokeTn',
u'\uca60': 'HangulSyllableJjeoss',
u'\ueee3': 'UnicodeCharacter0xeee3',
u'\u4c64': 'CjkUnifiedIdeograph4c64',
u'\u60e7': 'CjkUnifiedIdeograph60e7',
u'\u8366': 'CjkUnifiedIdeograph8366',
u'\ua7e9': 'UnicodeCharacter0xa7e9',
u'\u056a': 'ArmenianSmallLetterZhe',
u'\ud9ed': 'UnicodeCharacter0xd9ed',
u'\u53f1': 'CjkUnifiedIdeograph53f1',
u'\uf270': 'UnicodeCharacter0xf270',
u'\u96f3': 'CjkUnifiedIdeograph96f3',
u'\u7474': 'CjkUnifiedIdeograph7474',
u'\u08f7': 'UnicodeCharacter0x8f7',
u'\uab76': 'UnicodeCharacter0xab76',
u'\u2d7a': 'UnicodeCharacter0x2d7a',
u'\uc1fd': 'HangulSyllableSyog',
u'\ufb01': 'LatinSmallLigatureFi',
u'\u9a80': 'CjkUnifiedIdeograph9a80',
u'\u7d05': 'CjkUnifiedIdeograph7d05',
u'\u1c84': 'UnicodeCharacter0x1c84',
u'\ub007': 'HangulSyllableGgwec',
u'\u2a0b': 'SummationWithIntegral',
u'\ud58a': 'HangulSyllableHaej',
u'\u4f8e': 'CjkUnifiedIdeograph4f8e',
u'\ue311': 'UnicodeCharacter0xe311',
u'\u8290': 'CjkUnifiedIdeograph8290',
u'\u6515': 'CjkUnifiedIdeograph6515',
u'\u0494': 'CyrillicCapitalLetterGheWithMiddleHook',
u'\ud21b': 'HangulSyllableTyolb',
u'\ufd9a': 'ArabicLigatureNoonWithMeemWithYehFinalForm',
u'\u541f': 'CjkUnifiedIdeograph541f',
u'\u779e': 'CjkUnifiedIdeograph779e',
u'\u8b21': 'CjkUnifiedIdeograph8b21',
u'\uaaa0': 'UnicodeCharacter0xaaa0',
u'\u0d25': 'MalayalamLetterTha',
u'\u2ca4': 'CopticCapitalLetterSima',
u'\ufa2b': 'CjkCompatibilityIdeographfa2b',
u'\ue5aa': 'UnicodeCharacter0xe5aa',
u'\u7c2f': 'CjkUnifiedIdeograph7c2f',
u'\u1fae': 'GreekCapitalLetterOmegaWithPsiliAndPerispomeniAndProsgegrammeni',
u'\ub331': 'HangulSyllableDyang',
u'\u3535': 'CjkUnifiedIdeograph3535',
u'\ud4b4': 'HangulSyllablePwen',
u'\u4eb8': 'CjkUnifiedIdeograph4eb8',
u'\ue23b': 'UnicodeCharacter0xe23b',
u'\u8dba': 'CjkUnifiedIdeograph8dba',
u'\u643f': 'CjkUnifiedIdeograph643f',
u'\u07be': 'UnicodeCharacter0x7be',
u'\udd45': 'UnicodeCharacter0xdd45',
u'\ufcc4': 'ArabicLigatureKafWithJeemInitialForm',
u'\u5749': 'CjkUnifiedIdeograph5749',
u'\u76c8': 'CjkUnifiedIdeograph76c8',
u'\u8a4b': 'CjkUnifiedIdeograph8a4b',
u'\ub5ca': 'HangulSyllableDdelp',
u'\u0c4f': 'UnicodeCharacter0xc4f',
u'\u2fce': 'KangxiRadicalDrum',
u'\uc555': 'HangulSyllableAb',
u'\ue4d4': 'UnicodeCharacter0xe4d4',
u'\u7f59': 'CjkUnifiedIdeograph7f59',
u'\u1ed8': 'LatinCapitalLetterOWithCircumflexAndDotBelow',
u'\ub25b': 'HangulSyllableNwigs',
u'\u345f': 'CjkUnifiedIdeograph345f',
u'\ud7de': 'UnicodeCharacter0xd7de',
u'\u49e2': 'CjkUnifiedIdeograph49e2',
u'\ued65': 'UnicodeCharacter0xed65',
u'\u8ce4': 'CjkUnifiedIdeograph8ce4',
u'\u6769': 'CjkUnifiedIdeograph6769',
u'\u06e8': 'ArabicSmallHighNoon',
u'\udc6f': 'UnicodeCharacter0xdc6f',
u'\uffee': 'HalfwidthWhiteCircle',
u'\u5673': 'CjkUnifiedIdeograph5673',
u'\u71f2': 'CjkUnifiedIdeograph71f2',
u'\u9575': 'CjkUnifiedIdeograph9575',
u'\ub4f4': 'HangulSyllableDeuk',
u'\u0f79': 'TibetanVowelSignVocalicLl',
u'\u2ef8': 'UnicodeCharacter0x2ef8',
u'\uc47f': 'HangulSyllableSsuh',
u'\ue7fe': 'UnicodeCharacter0xe7fe',
u'\u9206': 'CjkUnifiedIdeograph9206',
u'\ub689': 'HangulSyllableDdyolg',
u'\ud708': 'HangulSyllableHwels',
u'\ueb8b': 'UnicodeCharacter0xeb8b',
u'\u140a': 'CanadianSyllabicsA',
u'\u288d': 'BraillePatternDots1348',
u'\u490c': 'CjkUnifiedIdeograph490c',
u'\u6d8f': 'CjkUnifiedIdeograph6d8f',
u'\uba16': 'HangulSyllableMyaj',
u'\uff18': 'FullwidthDigitEight',
u'\u939b': 'CjkUnifiedIdeograph939b',
u'\u3c1a': 'CjkUnifiedIdeograph3c1a',
u'\ud09d': 'HangulSyllableKying',
u'\u711c': 'CjkUnifiedIdeograph711c',
u'\u159f': 'CanadianSyllabicsWoodscreeTh',
u'\u4aa1': 'CjkUnifiedIdeograph4aa1',
u'\ua226': 'YiSyllableGget',
u'\ue728': 'UnicodeCharacter0xe728',
u'\ubbab': 'HangulSyllableMyud',
u'\u242a': 'UnicodeCharacter0x242a',
u'\uf8ad': 'UnicodeCharacter0xf8ad',
u'\u192c': 'UnicodeCharacter0x192c',
u'\u3daf': 'CjkUnifiedIdeograph3daf',
u'\u5e2e': 'CjkUnifiedIdeograph5e2e',
u'\u72b1': 'CjkUnifiedIdeograph72b1',
u'\u8f38': 'CjkUnifiedIdeograph8f38',
u'\ua3bb': 'YiSyllableShux',
u'\ucc3a': 'HangulSyllableCabs',
u'\ue0bd': 'UnicodeCharacter0xe0bd',
u'\u013c': 'LatinSmallLetterLWithCedilla',
u'\u25bf': 'WhiteDownpointingSmallTriangle',
u'\u463e': 'CjkUnifiedIdeograph463e',
u'\u1ac1': 'UnicodeCharacter0x1ac1',
u'\u5fc3': 'CjkUnifiedIdeograph5fc3',
u'\ub748': 'HangulSyllableDdyin',
u'\uf44a': 'UnicodeCharacter0xf44a',
u'\u88cd': 'CjkUnifiedIdeograph88cd',
u'\u294c': 'UpBarbRightDownBarbLeftHarpoon',
u'\ucdcf': 'HangulSyllableCwegs',
u'\u6e4e': 'CjkUnifiedIdeograph6e4e',
u'\u02d1': 'ModifierLetterHalfTriangularColon',
u'\u47d3': 'CjkUnifiedIdeograph47d3',
u'\u9c5a': 'CjkUnifiedIdeograph9c5a',
u'\ub0dd': 'HangulSyllableNyalt',
u'\ud15c': 'HangulSyllableTem',
u'\uf5df': 'UnicodeCharacter0xf5df',
u'\u165e': 'CanadianSyllabicsCarrierTsee',
u'\u2ae1': 'PerpendicularWithS',
u'\u4b60': 'CjkUnifiedIdeograph4b60',
u'\u6fe3': 'CjkUnifiedIdeograph6fe3',
u'\u846a': 'CjkUnifiedIdeograph846a',
u'\uf96c': 'CjkCompatibilityIdeographf96c',
u'\u9def': 'CjkUnifiedIdeograph9def',
u'\u3e6e': 'CjkUnifiedIdeograph3e6e',
u'\ud2f1': 'HangulSyllableTig',
u'\u7370': 'CjkUnifiedIdeograph7370',
u'\u17f3': 'KhmerSymbolLekAttakBei',
u'\u54f5': 'CjkUnifiedIdeograph54f5',
u'\uac7a': 'HangulSyllableGeolm',
u'\ue17c': 'UnicodeCharacter0xe17c',
u'\u85ff': 'CjkUnifiedIdeograph85ff',
u'\u267e': 'PermanentPaperSign',
u'\u1b80': 'SundaneseSignPanyecek',
u'\ubf03': 'HangulSyllableBbec',
u'\u5882': 'CjkUnifiedIdeograph5882',
u'\ufc05': 'ArabicLigatureBehWithJeemIsolatedForm',
u'\u3107': 'BopomofoLetterM',
u'\u7609': 'CjkUnifiedIdeograph7609',
u'\u898c': 'CjkUnifiedIdeograph898c',
u'\uce8e': 'HangulSyllableKap',
u'\u0390': 'GreekSmallLetterIotaWithDialytikaAndTonos',
u'\ua713': 'ModifierLetterHighLeftstemToneBar',
u'\u4092': 'CjkUnifiedIdeograph4092',
u'\ue415': 'UnicodeCharacter0xe415',
u'\ud917': 'UnicodeCharacter0xd917',
u'\u1e19': 'LatinSmallLetterEWithCircumflexBelow',
u'\u531b': 'CjkUnifiedIdeograph531b',
u'\ub19c': 'HangulSyllableNwal',
u'\uf69e': 'UnicodeCharacter0xf69e',
u'\u2ba0': 'UnicodeCharacter0x2ba0',
u'\u68a2': 'CjkUnifiedIdeograph68a2',
u'\u8c25': 'CjkUnifiedIdeograph8c25',
u'\uc127': 'HangulSyllableSeolb',
u'\u0629': 'ArabicLetterTehMarbuta',
u'\u7b2b': 'CjkUnifiedIdeograph7b2b',
u'\u9eae': 'CjkUnifiedIdeograph9eae',
u'\ud3b0': 'HangulSyllablePek',
u'\u10b2': 'GeorgianCapitalLetterTar',
u'\ub435': 'HangulSyllableDyog',
u'\u55b4': 'CjkUnifiedIdeograph55b4',
u'\ue937': 'UnicodeCharacter0xe937',
u'\u2e39': 'UnicodeCharacter0x2e39',
u'\u633b': 'CjkUnifiedIdeograph633b',
u'\u86be': 'CjkUnifiedIdeograph86be',
u'\ufbc0': 'UnicodeCharacter0xfbc0',
u'\u38c2': 'CjkUnifiedIdeograph38c2',
u'\u7dc4': 'CjkUnifiedIdeograph7dc4',
u'\u9147': 'CjkUnifiedIdeograph9147',
u'\ud649': 'HangulSyllableHob',
u'\u0b4b': 'OriyaVowelSignO',
u'\u484d': 'CjkUnifiedIdeograph484d',
u'\uaece': 'HangulSyllableGgeobs',
u'\ue3d0': 'UnicodeCharacter0xe3d0',
u'\u20d2': 'CombiningLongVerticalLineOverlay',
u'\u65d4': 'CjkUnifiedIdeograph65d4',
u'\ub957': 'HangulSyllableRwih',
u'\u5ad6': 'CjkUnifiedIdeograph5ad6',
u'\ufe59': 'SmallLeftParenthesis',
u'\u335b': 'IdeographicTelegraphSymbolForHourThree',
u'\u705d': 'CjkUnifiedIdeograph705d',
u'\u8be0': 'CjkUnifiedIdeograph8be0',
u'\uc8e2': 'HangulSyllableJyogg',
u'\u0de4': 'UnicodeCharacter0xde4',
u'\ua167': 'YiSyllableNdur',
u'\u42e6': 'CjkUnifiedIdeograph42e6',
u'\ue669': 'UnicodeCharacter0xe669',
u'\udb6b': 'UnicodeCharacter0xdb6b',
u'\u186d': 'MongolianLetterSibeHaa',
u'\u5d6f': 'CjkUnifiedIdeograph5d6f',
u'\ub3f0': 'HangulSyllableDwam',
u'\uf0f2': 'UnicodeCharacter0xf0f2',
u'\u35f4': 'CjkUnifiedIdeograph35f4',
u'\u6af6': 'CjkUnifiedIdeograph6af6',
u'\u8e79': 'CjkUnifiedIdeograph8e79',
u'\uc37b': 'HangulSyllableSseos',
u'\u457f': 'CjkUnifiedIdeograph457f',
u'\u3d81': 'CjkUnifiedIdeograph3d81',
u'\uda04': 'UnicodeCharacter0xda04',
u'\uf687': 'UnicodeCharacter0xf687',
u'\u5008': 'CjkUnifiedIdeograph5008',
u'\u6c8b': 'CjkUnifiedIdeograph6c8b',
u'\u930a': 'CjkUnifiedIdeograph930a',
u'\uaf8d': 'HangulSyllableGgoelt',
u'\u090e': 'DevanagariLetterShortE',
u'\u2591': 'LightShade',
u'\uc214': 'HangulSyllableSyok',
u'\u9e97': 'CjkUnifiedIdeograph9e97',
u'\u7818': 'CjkUnifiedIdeograph7818',
u'\u149b': 'CanadianSyllabicsWestcreeCwoo',
u'\ubb1a': 'HangulSyllableMyogg',
u'\u311e': 'BopomofoLetterAi',
u'\ucda1': 'HangulSyllableCult',
u'\u4ba5': 'CjkUnifiedIdeograph4ba5',
u'\uea24': 'UnicodeCharacter0xea24',
u'\u86a7': 'CjkUnifiedIdeograph86a7',
u'\u6028': 'CjkUnifiedIdeograph6028',
u'\u3cab': 'CjkUnifiedIdeograph3cab',
u'\ua32a': 'YiSyllableSsit',
u'\ud92e': 'UnicodeCharacter0xd92e',
u'\uf5b1': 'UnicodeCharacter0xf5b1',
u'\u5732': 'CjkUnifiedIdeograph5732',
u'\u73b5': 'CjkUnifiedIdeograph73b5',
u'\u9234': 'CjkUnifiedIdeograph9234',
u'\uaeb7': 'HangulSyllableGgyaec',
u'\u0838': 'UnicodeCharacter0x838',
u'\u24bb': 'CircledLatinCapitalLetterF',
u'\uc13e': 'HangulSyllableSenh',
u'\u9dc1': 'CjkUnifiedIdeograph9dc1',
u'\u7f42': 'CjkUnifiedIdeograph7f42',
u'\u1bc5': 'UnicodeCharacter0x1bc5',
u'\uba44': 'HangulSyllableMeols',
u'\u3048': 'HiraganaLetterE',
u'\ucccb': 'HangulSyllableCec',
u'\u4acf': 'CjkUnifiedIdeograph4acf',
u'\ue94e': 'UnicodeCharacter0xe94e',
u'\u85d1': 'CjkUnifiedIdeograph85d1',
u'\u6752': 'CjkUnifiedIdeograph6752',
u'\u03d5': 'GreekPhiSymbol',
u'\ua254': 'YiSyllableHxuo',
u'\ud858': 'UnicodeCharacter0xd858',
u'\uf4db': 'UnicodeCharacter0xf4db',
u'\u565c': 'CjkUnifiedIdeograph565c',
u'\u72df': 'CjkUnifiedIdeograph72df',
u'\u915e': 'CjkUnifiedIdeograph915e',
u'\uade1': 'HangulSyllableGyunj',
u'\u0f62': 'TibetanLetterRa',
u'\u2be5': 'UnicodeCharacter0x2be5',
u'\uc068': 'HangulSyllableBbeum',
u'\u9ceb': 'CjkUnifiedIdeograph9ceb',
u'\u7e6c': 'CjkUnifiedIdeograph7e6c',
u'\u1aef': 'UnicodeCharacter0x1aef',
u'\ub96e': 'HangulSyllableRyuj',
u'\u3772': 'CjkUnifiedIdeograph3772',
u'\ud3f5': 'HangulSyllablePolg',
u'\u49f9': 'CjkUnifiedIdeograph49f9',
u'\ue878': 'UnicodeCharacter0xe878',
u'\u84fb': 'CjkUnifiedIdeograph84fb',
u'\u667c': 'CjkUnifiedIdeograph667c',
u'\u02ff': 'ModifierLetterLowLeftArrow',
u'\ua17e': 'YiSyllableNix',
u'\u3803': 'CjkUnifiedIdeograph3803',
u'\udf82': 'UnicodeCharacter0xdf82',
u'\u5586': 'CjkUnifiedIdeograph5586',
u'\uf109': 'UnicodeCharacter0xf109',
u'\u9088': 'CjkUnifiedIdeograph9088',
u'\u6f0d': 'CjkUnifiedIdeograph6f0d',
u'\u0e8c': 'UnicodeCharacter0xe8c',
u'\uaa0f': 'ChamLetterJha',
u'\u2013': 'EnDash',
u'\uc792': 'HangulSyllableJagg',
u'\u5e17': 'CjkUnifiedIdeograph5e17',
u'\u7d96': 'CjkUnifiedIdeograph7d96',
u'\u9919': 'CjkUnifiedIdeograph9919',
u'\ub898': 'HangulSyllableRwaen',
u'\u171d': 'UnicodeCharacter0x171d',
u'\u369c': 'CjkUnifiedIdeograph369c',
u'\uc823': 'HangulSyllableJed',
u'\uefa2': 'UnicodeCharacter0xefa2',
u'\u4627': 'CjkUnifiedIdeograph4627',
u'\u65a6': 'CjkUnifiedIdeograph65a6',
u'\u8129': 'CjkUnifiedIdeograph8129',
u'\ua0a8': 'YiSyllableHmur',
u'\u3f2d': 'CjkUnifiedIdeograph3f2d',
u'\udeac': 'UnicodeCharacter0xdeac',
u'\u54b0': 'CjkUnifiedIdeograph54b0',
u'\uf033': 'UnicodeCharacter0xf033',
u'\u97b2': 'CjkUnifiedIdeograph97b2',
u'\u6e37': 'CjkUnifiedIdeograph6e37',
u'\u0db6': 'SinhalaLetterAlpapraanaBayanna',
u'\ua939': 'RejangLetterCa',
u'\u273d': 'HeavyTeardropspokedAsterisk',
u'\uc6bc': 'HangulSyllableUls',
u'\u5d41': 'CjkUnifiedIdeograph5d41',
u'\u7cc0': 'CjkUnifiedIdeograph7cc0',
u'\u9843': 'CjkUnifiedIdeograph9843',
u'\ubfc2': 'HangulSyllableBbyobs',
u'\u1647': 'CanadianSyllabicsCarrierInitialZ',
u'\u35c6': 'CjkUnifiedIdeograph35c6',
u'\ucf4d': 'HangulSyllableKyeng',
u'\ueecc': 'UnicodeCharacter0xeecc',
u'\u4551': 'CjkUnifiedIdeograph4551',
u'\u64d0': 'CjkUnifiedIdeograph64d0',
u'\u8053': 'CjkUnifiedIdeograph8053',
u'\ua7d2': 'UnicodeCharacter0xa7d2',
u'\u3e57': 'CjkUnifiedIdeograph3e57',
u'\uddd6': 'UnicodeCharacter0xddd6',
u'\u5bda': 'CjkUnifiedIdeograph5bda',
u'\uf75d': 'UnicodeCharacter0xf75d',
u'\u96dc': 'CjkUnifiedIdeograph96dc',
u'\u6d61': 'CjkUnifiedIdeograph6d61',
u'\u0ce0': 'KannadaLetterVocalicRr',
u'\ua863': 'PhagspaLetterXa',
u'\u2667': 'WhiteClubSuit',
u'\uc5e6': 'HangulSyllableEj',
u'\u5c6b': 'CjkUnifiedIdeograph5c6b',
u'\u43ea': 'CjkUnifiedIdeograph43ea',
u'\u9f6d': 'CjkUnifiedIdeograph9f6d',
u'\ubeec': 'HangulSyllableBbe',
u'\u1571': 'CanadianSyllabicsTyi',
u'\u34f0': 'CjkUnifiedIdeograph34f0',
u'\uce77': 'HangulSyllableKags',
u'\uedf6': 'UnicodeCharacter0xedf6',
u'\u447b': 'CjkUnifiedIdeograph447b',
u'\u6bfa': 'CjkUnifiedIdeograph6bfa',
u'\u877d': 'CjkUnifiedIdeograph877d',
u'\ua6fc': 'UnicodeCharacter0xa6fc',
u'\ubc81': 'HangulSyllableByaet',
u'\udd00': 'UnicodeCharacter0xdd00',
u'\uf983': 'CjkCompatibilityIdeographf983',
u'\u1e02': 'LatinCapitalLetterBWithDotAbove',
u'\u3a85': 'CjkUnifiedIdeograph3a85',
u'\u5b04': 'CjkUnifiedIdeograph5b04',
u'\u7787': 'CjkUnifiedIdeograph7787',
u'\u880e': 'CjkUnifiedIdeograph880e',
u'\ua491': 'YiRadicalLi',
u'\uc510': 'HangulSyllableSsyin',
u'\ue193': 'UnicodeCharacter0xe193',
u'\u0612': 'ArabicSignRahmatullahAlayhe',
u'\u2295': 'CircledPlus',
u'\u4314': 'CjkUnifiedIdeograph4314',
u'\u1f97': 'GreekSmallLetterEtaWithDasiaAndPerispomeniAndYpogegrammeni',
u'\u5899': 'CjkUnifiedIdeograph5899',
u'\ub01e': 'HangulSyllableGgwibs',
u'\ued20': 'UnicodeCharacter0xed20',
u'\u89a3': 'CjkUnifiedIdeograph89a3',
u'\u2e22': 'TopLeftHalfBracket',
u'\ucaa5': 'HangulSyllableJjyenj',
u'\u6b24': 'CjkUnifiedIdeograph6b24',
u'\u07a7': 'ThaanaAabaafili',
u'\u40a9': 'CjkUnifiedIdeograph40a9',
u'\u9530': 'CjkUnifiedIdeograph9530',
u'\ub1b3': 'HangulSyllableNwaegs',
u'\ud632': 'HangulSyllableHyej',
u'\uf2b5': 'UnicodeCharacter0xf2b5',
u'\u1334': 'EthiopicSyllablePhee',
u'\u2fb7': 'KangxiRadicalEat',
u'\u4c36': 'CjkUnifiedIdeograph4c36',
u'\u68b9': 'CjkUnifiedIdeograph68b9',
u'\ubd40': 'HangulSyllableBwaess',
u'\ufe42': 'PresentationFormForVerticalRightCornerBracket',
u'\u9ac5': 'CjkUnifiedIdeograph9ac5',
u'\u3b44': 'CjkUnifiedIdeograph3b44',
u'\ud7c7': 'UnicodeCharacter0xd7c7',
u'\u7446': 'CjkUnifiedIdeograph7446',
u'\u10c9': 'UnicodeCharacter0x10c9',
u'\u4dcb': 'HexagramForStandstill',
u'\ua550': 'VaiSyllablePa',
u'\ue652': 'UnicodeCharacter0xe652',
u'\u82d5': 'CjkUnifiedIdeograph82d5',
u'\u2354': 'AplFunctionalSymbolQuadDel',
u'\uffd7': 'HalfwidthHangulLetterYu',
u'\u1c56': 'OlChikiDigitSix',
u'\u38d9': 'CjkUnifiedIdeograph38d9',
u'\u5958': 'CjkUnifiedIdeograph5958',
u'\u75db': 'CjkUnifiedIdeograph75db',
u'\u8e62': 'CjkUnifiedIdeograph8e62',
u'\uaae5': 'UnicodeCharacter0xaae5',
u'\ucb64': 'HangulSyllableJjweo',
u'\ue7e7': 'UnicodeCharacter0xe7e7',
u'\u0466': 'CyrillicCapitalLetterLittleYus',
u'\u20e9': 'CombiningWideBridgeAbove',
u'\u4168': 'CjkUnifiedIdeograph4168',
u'\u1deb': 'UnicodeCharacter0x1deb',
u'\u5eed': 'CjkUnifiedIdeograph5eed',
u'\ub672': 'HangulSyllableDdoelp',
u'\uf374': 'UnicodeCharacter0xf374',
u'\u8ff7': 'CjkUnifiedIdeograph8ff7',
u'\u2c76': 'LatinSmallLetterHalfH',
u'\uc8f9': 'HangulSyllableJyot',
u'\u6978': 'CjkUnifiedIdeograph6978',
u'\u05fb': 'UnicodeCharacter0x5fb',
u'\u46fd': 'CjkUnifiedIdeograph46fd',
u'\u7c01': 'CjkUnifiedIdeograph7c01',
u'\u9b84': 'CjkUnifiedIdeograph9b84',
u'\ud486': 'HangulSyllablePulp',
u'\u1188': 'HangulJungseongYoi',
u'\uad0b': 'HangulSyllableGwalh',
u'\u528a': 'CjkUnifiedIdeograph528a',
u'\uee0d': 'UnicodeCharacter0xee0d',
u'\u2b0f': 'RightwardsArrowWithTipUpwards',
u'\u6411': 'CjkUnifiedIdeograph6411',
u'\u8394': 'CjkUnifiedIdeograph8394',
u'\ufc96': 'ArabicLigatureYehWithYehFinalForm',
u'\u3998': 'CjkUnifiedIdeograph3998',
u'\u7a9a': 'CjkUnifiedIdeograph7a9a',
u'\u961d': 'CjkUnifiedIdeograph961d',
u'\ud31f': 'HangulSyllablePas',
u'\u0c21': 'TeluguLetterDda',
u'\u4923': 'CjkUnifiedIdeograph4923',
u'\uaba4': 'UnicodeCharacter0xaba4',
u'\ue4a6': 'UnicodeCharacter0xe4a6',
u'\u21a8': 'UpDownArrowWithBase',
u'\u62aa': 'CjkUnifiedIdeograph62aa',
u'\ube2d': 'HangulSyllableByinj',
u'\u5fac': 'CjkUnifiedIdeograph5fac',
u'\ufb2f': 'HebrewLetterAlefWithQamats',
u'\u3431': 'CjkUnifiedIdeograph3431',
u'\u7133': 'CjkUnifiedIdeograph7133',
u'\u8cb6': 'CjkUnifiedIdeograph8cb6',
u'\uc9b8': 'HangulSyllableJyiss',
u'\u0aba': 'UnicodeCharacter0xaba',
u'\ua63d': 'UnicodeCharacter0xa63d',
u'\u47bc': 'CjkUnifiedIdeograph47bc',
u'\ue33f': 'UnicodeCharacter0xe33f',
u'\udc41': 'UnicodeCharacter0xdc41',
u'\u1943': 'UnicodeCharacter0x1943',
u'\u5a45': 'CjkUnifiedIdeograph5a45',
u'\ub4c6': 'HangulSyllableDyunh',
u'\uf1c8': 'UnicodeCharacter0xf1c8',
u'\u32ca': 'IdeographicTelegraphSymbolForNovember',
u'\u6fcc': 'CjkUnifiedIdeograph6fcc',
u'\u8b4f': 'CjkUnifiedIdeograph8b4f',
u'\uc451': 'HangulSyllableSsyolg',
u'\u0153': 'LatinSmallLigatureOe',
u'\u4255': 'CjkUnifiedIdeograph4255',
u'\u99d8': 'CjkUnifiedIdeograph99d8',
u'\udada': 'UnicodeCharacter0xdada',
u'\u17dc': 'KhmerSignAvakrahasanya',
u'\ub35f': 'HangulSyllableDeolb',
u'\u50de': 'CjkUnifiedIdeograph50de',
u'\uec61': 'UnicodeCharacter0xec61',
u'\u2963': 'UpwardsHarpoonWithBarbLeftBesideUpwardsHarpoonWithBarbRight',
u'\u6a65': 'CjkUnifiedIdeograph6a65',
u'\u81e8': 'CjkUnifiedIdeograph81e8',
u'\uc2ea': 'HangulSyllableSilp',
u'\u3fec': 'CjkUnifiedIdeograph3fec',
u'\u78ee': 'CjkUnifiedIdeograph78ee',
u'\u9471': 'CjkUnifiedIdeograph9471',
u'\ud173': 'HangulSyllableTyeolb',
u'\u1275': 'EthiopicSyllableTe',
u'\u4f77': 'CjkUnifiedIdeograph4f77',
u'\ua9f8': 'UnicodeCharacter0xa9f8',
u'\ueafa': 'UnicodeCharacter0xeafa',
u'\u27fc': 'LongRightwardsArrowFromBar',
u'\u60fe': 'CjkUnifiedIdeograph60fe',
u'\u5e00': 'CjkUnifiedIdeograph5e00',
u'\u7a83': 'CjkUnifiedIdeograph7a83',
u'\u9d02': 'CjkUnifiedIdeograph9d02',
u'\ub985': 'HangulSyllableReub',
u'\u1706': 'TagalogLetterTa',
u'\u3389': 'SquareKcal',
u'\ucc0c': 'HangulSyllableJji',
u'\ue88f': 'UnicodeCharacter0xe88f',
u'\u4610': 'CjkUnifiedIdeograph4610',
u'\u6293': 'CjkUnifiedIdeograph6293',
u'\u8512': 'CjkUnifiedIdeograph8512',
u'\ua195': 'YiSyllableNurx',
u'\u3f16': 'CjkUnifiedIdeograph3f16',
u'\udb99': 'UnicodeCharacter0xdb99',
u'\u559d': 'CjkUnifiedIdeograph559d',
u'\uf41c': 'UnicodeCharacter0xf41c',
u'\u909f': 'CjkUnifiedIdeograph909f',
u'\u6e20': 'CjkUnifiedIdeograph6e20',
u'\u0aa3': 'GujaratiLetterNna',
u'\uad22': 'HangulSyllableGwaelm',
u'\u2726': 'BlackFourPointedStar',
u'\uc3a9': 'HangulSyllableSsyeolg',
u'\u7dad': 'CjkUnifiedIdeograph7dad',
u'\u9c2c': 'CjkUnifiedIdeograph9c2c',
u'\ub8af': 'HangulSyllableRwaeh',
u'\u1630': 'CanadianSyllabicsCarrierLhe',
u'\u32b3': 'CircledNumberThirtyEight',
u'\ucf36': 'HangulSyllableKyeop',
u'\uebb9': 'UnicodeCharacter0xebb9',
u'\u493a': 'CjkUnifiedIdeograph493a',
u'\u65bd': 'CjkUnifiedIdeograph65bd',
u'\u843c': 'CjkUnifiedIdeograph843c',
u'\ua0bf': 'YiSyllableMo',
u'\u3e40': 'CjkUnifiedIdeograph3e40',
u'\udac3': 'UnicodeCharacter0xdac3',
u'\u54c7': 'CjkUnifiedIdeograph54c7',
u'\uf746': 'UnicodeCharacter0xf746',
u'\u93c9': 'CjkUnifiedIdeograph93c9',
u'\u714a': 'CjkUnifiedIdeograph714a',
u'\u0dcd': 'UnicodeCharacter0xdcd',
u'\uac4c': 'HangulSyllableGyass',
u'\u2650': 'Sagittarius',
u'\uc2d3': 'HangulSyllableSyis',
u'\u7cd7': 'CjkUnifiedIdeograph7cd7',
u'\u9f56': 'CjkUnifiedIdeograph9f56',
u'\ubbd9': 'HangulSyllableMeut',
u'\u195a': 'TaiLeLetterPha',
u'\u35dd': 'CjkUnifiedIdeograph35dd',
u'\uce60': 'HangulSyllableCil',
u'\ueae3': 'UnicodeCharacter0xeae3',
u'\u4864': 'CjkUnifiedIdeograph4864',
u'\u64e7': 'CjkUnifiedIdeograph64e7',
u'\u8766': 'CjkUnifiedIdeograph8766',
u'\ua3e9': 'YiSyllableJuox',
u'\u016a': 'LatinCapitalLetterUWithMacron',
u'\udded': 'UnicodeCharacter0xdded',
u'\u57f1': 'CjkUnifiedIdeograph57f1',
u'\uf670': 'UnicodeCharacter0xf670',
u'\u92f3': 'CjkUnifiedIdeograph92f3',
u'\u7074': 'CjkUnifiedIdeograph7074',
u'\u0cf7': 'UnicodeCharacter0xcf7',
u'\uaf76': 'HangulSyllableGgwaebs',
u'\u297a': 'LeftwardsArrowThroughSubset',
u'\uc5fd': 'HangulSyllableYeob',
u'\uff01': 'FullwidthExclamationMark',
u'\u9e80': 'CjkUnifiedIdeograph9e80',
u'\u7905': 'CjkUnifiedIdeograph7905',
u'\u1884': 'MongolianLetterAliGaliInvertedUbadama',
u'\ub407': 'HangulSyllableDwaelb',
u'\u2e0b': 'RaisedSquare',
u'\ud18a': 'HangulSyllableTyenh',
u'\u4b8e': 'CjkUnifiedIdeograph4b8e',
u'\ue711': 'UnicodeCharacter0xe711',
u'\u8690': 'CjkUnifiedIdeograph8690',
u'\u6115': 'CjkUnifiedIdeograph6115',
u'\x94': 'UnicodeCharacter0x94',
u'\ud61b': 'HangulSyllableHyeoh',
u'\uf99a': 'CjkCompatibilityIdeographf99a',
u'\u501f': 'CjkUnifiedIdeograph501f',
u'\u739e': 'CjkUnifiedIdeograph739e',
u'\u8f21': 'CjkUnifiedIdeograph8f21',
u'\uaea0': 'HangulSyllableGgyae',
u'\u0925': 'DevanagariLetterTha',
u'\u28a4': 'BraillePatternDots368',
u'\ufe2b': 'UnicodeCharacter0xfe2b',
u'\ue1aa': 'UnicodeCharacter0xe1aa',
u'\u782f': 'CjkUnifiedIdeograph782f',
u'\u1bae': 'SundaneseLetterKha',
u'\ub731': 'HangulSyllableDdeulg',
u'\u3135': 'HangulLetterNieuncieuc',
u'\ud0b4': 'HangulSyllableKim',
u'\u4ab8': 'CjkUnifiedIdeograph4ab8',
u'\ue63b': 'UnicodeCharacter0xe63b',
u'\u89ba': 'CjkUnifiedIdeograph89ba',
u'\u603f': 'CjkUnifiedIdeograph603f',
u'\u03be': 'GreekSmallLetterXi',
u'\ud945': 'UnicodeCharacter0xd945',
u'\uf8c4': 'UnicodeCharacter0xf8c4',
u'\u5349': 'CjkUnifiedIdeograph5349',
u'\u72c8': 'CjkUnifiedIdeograph72c8',
u'\u8e4b': 'CjkUnifiedIdeograph8e4b',
u'\ub1ca': 'HangulSyllableNwaep',
u'\u084f': 'UnicodeCharacter0x84f',
u'\u2bce': 'UnicodeCharacter0x2bce',
u'\uc155': 'HangulSyllableSyeog',
u'\ue0d4': 'UnicodeCharacter0xe0d4',
u'\u7b59': 'CjkUnifiedIdeograph7b59',
u'\u1ad8': 'UnicodeCharacter0x1ad8',
u'\ub65b': 'HangulSyllableDdwaes',
u'\u305f': 'HiraganaLetterTa',
u'\ud3de': 'HangulSyllablePyelp',
u'\u4de2': 'HexagramForProgress',
u'\ue965': 'UnicodeCharacter0xe965',
u'\u88e4': 'CjkUnifiedIdeograph88e4',
u'\u6369': 'CjkUnifiedIdeograph6369',
u'\u02e8': 'ModifierLetterLowToneBar',
u'\ud86f': 'UnicodeCharacter0xd86f',
u'\ufbee': 'ArabicLigatureYehWithHamzaAboveWithWawIsolatedForm',
u'\u5273': 'CjkUnifiedIdeograph5273',
u'\u75f2': 'CjkUnifiedIdeograph75f2',
u'\u9175': 'CjkUnifiedIdeograph9175',
u'\ub0f4': 'HangulSyllableNyael',
u'\u0b79': 'UnicodeCharacter0xb79',
u'\u2af8': 'TripleNestedGreaterthan',
u'\uc07f': 'HangulSyllableBbyilb',
u'\ue3fe': 'UnicodeCharacter0xe3fe',
u'\u9606': 'CjkUnifiedIdeograph9606',
u'\ub289': 'HangulSyllableNyung',
u'\ud308': 'HangulSyllableTik',
u'\uef8b': 'UnicodeCharacter0xef8b',
u'\u100a': 'MyanmarLetterNnya',
u'\u2c8d': 'CopticSmallLetterZata',
u'\u4d0c': 'CjkUnifiedIdeograph4d0c',
u'\u698f': 'CjkUnifiedIdeograph698f',
u'\ube16': 'HangulSyllableBeulm',
u'\ufb18': 'UnicodeCharacter0xfb18',
u'\u979b': 'CjkUnifiedIdeograph979b',
u'\u381a': 'CjkUnifiedIdeograph381a',
u'\ud49d': 'HangulSyllablePweolg',
u'\u751c': 'CjkUnifiedIdeograph751c',
u'\u119f': 'HangulJungseongAraeaeo',
u'\u4ea1': 'CjkUnifiedIdeograph4ea1',
u'\ua626': 'VaiDigitSix',
u'\ue328': 'UnicodeCharacter0xe328',
u'\ubfab': 'HangulSyllableBboec',
u'\u202a': 'LefttorightEmbedding',
u'\ufcad': 'ArabicLigatureSeenWithJeemInitialForm',
u'\u1d2c': 'ModifierLetterCapitalA',
u'\u39af': 'CjkUnifiedIdeograph39af',
u'\u5a2e': 'CjkUnifiedIdeograph5a2e',
u'\u76b1': 'CjkUnifiedIdeograph76b1',
u'\u8b38': 'CjkUnifiedIdeograph8b38',
u'\ua7bb': 'UnicodeCharacter0xa7bb',
u'\uc83a': 'HangulSyllableJyeogg',
u'\ue4bd': 'UnicodeCharacter0xe4bd',
u'\u053c': 'ArmenianCapitalLetterLiwn',
u'\u21bf': 'UpwardsHarpoonWithBarbLeftwards',
u'\u423e': 'CjkUnifiedIdeograph423e',
u'\u1ec1': 'LatinSmallLetterEWithCircumflexAndGrave',
u'\u5bc3': 'CjkUnifiedIdeograph5bc3',
u'\ub348': 'HangulSyllableDyaem',
u'\uf04a': 'UnicodeCharacter0xf04a',
u'\u8ccd': 'CjkUnifiedIdeograph8ccd',
u'\u2d4c': 'TifinaghLetterTuaregYazh',
u'\uc9cf': 'HangulSyllableJilh',
u'\u6a4e': 'CjkUnifiedIdeograph6a4e',
u'\u06d1': 'ArabicLetterYehWithThreeDotsBelow',
u'\u43d3': 'CjkUnifiedIdeograph43d3',
u'\u985a': 'CjkUnifiedIdeograph985a',
u'\ub4dd': 'HangulSyllableDeug',
u'\ud55c': 'HangulSyllableHan',
u'\uf1df': 'UnicodeCharacter0xf1df',
u'\u125e': 'UnicodeCharacter0x125e',
u'\u2ee1': 'CjkRadicalHead',
u'\u4f60': 'CjkUnifiedIdeograph4f60',
u'\u6be3': 'CjkUnifiedIdeograph6be3',
u'\u806a': 'CjkUnifiedIdeograph806a',
u'\ufd6c': 'ArabicLigatureSheenWithMeemWithMeemFinalForm',
u'\u99ef': 'CjkUnifiedIdeograph99ef',
u'\u3a6e': 'CjkUnifiedIdeograph3a6e',
u'\ud6f1': 'HangulSyllableHweob',
u'\u7770': 'CjkUnifiedIdeograph7770',
u'\u13f3': 'CherokeeLetterYu',
u'\u50f5': 'CjkUnifiedIdeograph50f5',
u'\ua87a': 'UnicodeCharacter0xa87a',
u'\ue57c': 'UnicodeCharacter0xe57c',
u'\u81ff': 'CjkUnifiedIdeograph81ff',
u'\u227e': 'PrecedesOrEquivalentTo',
u'\u1f80': 'GreekSmallLetterAlphaWithPsiliAndYpogegrammeni',
u'\ubb03': 'HangulSyllableMoed',
u'\u5c82': 'CjkUnifiedIdeograph5c82',
u'\uf805': 'UnicodeCharacter0xf805',
u'\u3507': 'CjkUnifiedIdeograph3507',
u'\u7209': 'CjkUnifiedIdeograph7209',
u'\u8d8c': 'CjkUnifiedIdeograph8d8c',
u'\uca8e': 'HangulSyllableJjyeolm',
u'\u0790': 'ThaanaLetterSeenu',
u'\ua313': 'YiSyllableSap',
u'\u4492': 'CjkUnifiedIdeograph4492',
u'\ue015': 'UnicodeCharacter0xe015',
u'\udd17': 'UnicodeCharacter0xdd17',
u'\u1a19': 'BugineseVowelSignE',
u'\u571b': 'CjkUnifiedIdeograph571b',
u'\ub59c': 'HangulSyllableDdyaek',
u'\uf29e': 'UnicodeCharacter0xf29e',
u'\u2fa0': 'KangxiRadicalMorning',
u'\u6ca2': 'CjkUnifiedIdeograph6ca2',
u'\u8825': 'CjkUnifiedIdeograph8825',
u'\uc527': 'HangulSyllableSsyih',
u'\u0229': 'LatinSmallLetterEWithCedilla',
u'\u7f2b': 'CjkUnifiedIdeograph7f2b',
u'\u9aae': 'CjkUnifiedIdeograph9aae',
u'\ud7b0': 'UnicodeCharacter0xd7b0',
u'\u14b2': 'CanadianSyllabicsMwo',
u'\ub035': 'HangulSyllableGgyult',
u'\u51b4': 'CjkUnifiedIdeograph51b4',
u'\ued37': 'UnicodeCharacter0xed37',
u'\u2a39': 'PlusSignInTriangle',
u'\u673b': 'CjkUnifiedIdeograph673b',
u'\u82be': 'CjkUnifiedIdeograph82be',
u'\uffc0': 'UnicodeCharacter0xffc0',
u'\u3cc2': 'CjkUnifiedIdeograph3cc2',
u'\u79c4': 'CjkUnifiedIdeograph79c4',
u'\u9547': 'CjkUnifiedIdeograph9547',
u'\ud249': 'HangulSyllableTweog',
u'\u0f4b': 'TibetanLetterTtha',
u'\u4c4d': 'CjkUnifiedIdeograph4c4d',
u'\uaace': 'UnicodeCharacter0xaace',
u'\ue7d0': 'UnicodeCharacter0xe7d0',
u'\u24d2': 'CircledLatinSmallLetterC',
u'\u61d4': 'CjkUnifiedIdeograph61d4',
u'\ubd57': 'HangulSyllableBoelh',
u'\u5ed6': 'CjkUnifiedIdeograph5ed6',
u'\ufa59': 'CjkCompatibilityIdeographfa59',
u'\u375b': 'CjkUnifiedIdeograph375b',
u'\u745d': 'CjkUnifiedIdeograph745d',
u'\u8fe0': 'CjkUnifiedIdeograph8fe0',
u'\ucce2': 'HangulSyllableCyeobs',
u'\u09e4': 'UnicodeCharacter0x9e4',
u'\ua567': 'VaiSyllableJa',
u'\u46e6': 'CjkUnifiedIdeograph46e6',
u'\ue269': 'UnicodeCharacter0xe269',
u'\udf6b': 'UnicodeCharacter0xdf6b',
u'\u1c6d': 'OlChikiLetterUy',
u'\u596f': 'CjkUnifiedIdeograph596f',
u'\ub7f0': 'HangulSyllableReon',
u'\uf4f2': 'UnicodeCharacter0xf4f2',
u'\u31f4': 'KatakanaLetterSmallNu',
u'\u6ef6': 'CjkUnifiedIdeograph6ef6',
u'\u8a79': 'CjkUnifiedIdeograph8a79',
u'\uc77b': 'HangulSyllableId',
u'\u047d': 'CyrillicSmallLetterOmegaWithTitlo',
u'\u417f': 'CjkUnifiedIdeograph417f',
u'\u0181': 'LatinCapitalLetterBWithHook',
u'\ue604': 'UnicodeCharacter0xe604',
u'\uca87': 'HangulSyllableJjyeogs',
u'\u6c08': 'CjkUnifiedIdeograph6c08',
u'\u508b': 'CjkUnifiedIdeograph508b',
u'\uaf0a': 'HangulSyllableGgyeoj',
u'\u938d': 'CjkUnifiedIdeograph938d',
u'\u350e': 'CjkUnifiedIdeograph350e',
u'\u1991': 'NewTaiLueLetterLowTa',
u'\ufe14': 'PresentationFormForVerticalSemicolon',
u'\ua297': 'YiSyllableZap',
u'\u4418': 'CjkUnifiedIdeograph4418',
u'\u289b': 'BraillePatternDots12458',
u'\u871a': 'CjkUnifiedIdeograph871a',
u'\u0d1e': 'MalayalamLetterNya',
u'\uf1a1': 'UnicodeCharacter0xf1a1',
u'\u77a5': 'CjkUnifiedIdeograph77a5',
u'\ud624': 'HangulSyllableHyel',
u'\ubaa7': 'HangulSyllableMyeh',
u'\u5c28': 'CjkUnifiedIdeograph5c28',
u'\xab': 'LeftpointingDoubleAngleQuotationMark',
u'\u9f2a': 'CjkUnifiedIdeograph9f2a',
u'\ue52e': 'UnicodeCharacter0xe52e',
u'\uc9b1': 'HangulSyllableJyilt',
u'\u6b32': 'CjkUnifiedIdeograph6b32',
u'\u4fb5': 'CjkUnifiedIdeograph4fb5',
u'\uae34': 'HangulSyllableGin',
u'\u92b7': 'CjkUnifiedIdeograph92b7',
u'\u3438': 'CjkUnifiedIdeograph3438',
u'\u18bb': 'UnicodeCharacter0x18bb',
u'\ufd3e': 'OrnateLeftParenthesis',
u'\ua1c1': 'YiSyllableLa',
u'\u4342': 'CjkUnifiedIdeograph4342',
u'\u27c5': 'LeftSshapedBagDelimiter',
u'\u8644': 'CjkUnifiedIdeograph8644',
u'\u0c48': 'TeluguVowelSignAi',
u'\uf0cb': 'UnicodeCharacter0xf0cb',
u'\u76cf': 'CjkUnifiedIdeograph76cf',
u'\ud54e': 'HangulSyllablePibs',
u'\ub9d1': 'HangulSyllableMalg',
u'\u5b52': 'CjkUnifiedIdeograph5b52',
u'\u3fd5': 'CjkUnifiedIdeograph3fd5',
u'\u9e54': 'CjkUnifiedIdeograph9e54',
u'\ue458': 'UnicodeCharacter0xe458',
u'\uc8db': 'HangulSyllableJoec',
u'\u6a5c': 'CjkUnifiedIdeograph6a5c',
u'\u4edf': 'CjkUnifiedIdeograph4edf',
u'\uad5e': 'HangulSyllableGyolp',
u'\u91e1': 'CjkUnifiedIdeograph91e1',
u'\u3362': 'IdeographicTelegraphSymbolForHourTen',
u'\u17e5': 'KhmerDigitFive',
u'\ufc68': 'ArabicLigatureYehWithHamzaAboveWithAlefMaksuraFinalForm',
u'\ua0eb': 'YiSyllableVax',
u'\u426c': 'CjkUnifiedIdeograph426c',
u'\u26ef': 'UnicodeCharacter0x26ef',
u'\u856e': 'CjkUnifiedIdeograph856e',
u'\u0b72': 'UnicodeCharacter0xb72',
u'\ueff5': 'UnicodeCharacter0xeff5',
u'\u75f9': 'CjkUnifiedIdeograph75f9',
u'\ud478': 'HangulSyllablePu',
u'\ub8fb': 'HangulSyllableRus',
u'\u5a7c': 'CjkUnifiedIdeograph5a7c',
u'\u3eff': 'CjkUnifiedIdeograph3eff',
u'\u9d7e': 'CjkUnifiedIdeograph9d7e',
u'\u0403': 'CyrillicCapitalLetterGje',
u'\ue382': 'UnicodeCharacter0xe382',
u'\u6986': 'CjkUnifiedIdeograph6986',
u'\ucd09': 'HangulSyllableCog',
u'\uac88': 'HangulSyllableGeok',
u'\u530d': 'CjkUnifiedIdeograph530d',
u'\u328c': 'CircledIdeographWater',
u'\u960f': 'CjkUnifiedIdeograph960f',
u'\u1c13': 'LepchaLetterBa',
u'\ufb92': 'ArabicLetterGafIsolatedForm',
u'\u6217': 'CjkUnifiedIdeograph6217',
u'\u4196': 'CjkUnifiedIdeograph4196',
u'\ua519': 'VaiSyllableZhee',
u'\u8498': 'CjkUnifiedIdeograph8498',
u'\u2b1d': 'BlackVerySmallSquare',
u'\u0a9c': 'GujaratiLetterJa',
u'\uf423': 'UnicodeCharacter0xf423',
u'\ud3a2': 'HangulSyllablePelm',
u'\u7a27': 'CjkUnifiedIdeograph7a27',
u'\u59a6': 'CjkUnifiedIdeograph59a6',
u'\ubd29': 'HangulSyllableBwat',
u'\u9ca8': 'CjkUnifiedIdeograph9ca8',
u'\u032d': 'CombiningCircumflexAccentBelow',
u'\ue2ac': 'UnicodeCharacter0xe2ac',
u'\u68b0': 'CjkUnifiedIdeograph68b0',
u'\ucc33': 'HangulSyllableCalb',
u'\uabb2': 'UnicodeCharacter0xabb2',
u'\u5237': 'CjkUnifiedIdeograph5237',
u'\u31b6': 'BopomofoFinalLetterK',
u'\u9539': 'CjkUnifiedIdeograph9539',
u'\u1b3d': 'BalineseVowelSignLaLengaTedung',
u'\ufabc': 'CjkCompatibilityIdeographfabc',
u'\u6141': 'CjkUnifiedIdeograph6141',
u'\u40c0': 'CjkUnifiedIdeograph40c0',
u'\ua443': 'YiSyllableNjur',
u'\u83c2': 'CjkUnifiedIdeograph83c2',
u'\u2a47': 'IntersectionAboveUnion',
u'\u09c6': 'UnicodeCharacter0x9c6',
u'\uf34d': 'UnicodeCharacter0xf34d',
u'\ud2cc': 'HangulSyllableTeuss',
u'\u7951': 'CjkUnifiedIdeograph7951',
u'\u58d0': 'CjkUnifiedIdeograph58d0',
u'\ubc53': 'HangulSyllableByad',
u'\u9bd2': 'CjkUnifiedIdeograph9bd2',
u'\u0257': 'LatinSmallLetterDWithHook',
u'\ue1d6': 'UnicodeCharacter0xe1d6',
u'\u67da': 'CjkUnifiedIdeograph67da',
u'\ucb5d': 'HangulSyllableJjung',
u'\uaadc': 'UnicodeCharacter0xaadc',
u'\u5161': 'CjkUnifiedIdeograph5161',
u'\u30e0': 'KatakanaLetterMu',
u'\u9463': 'CjkUnifiedIdeograph9463',
u'\u1a67': 'UnicodeCharacter0x1a67',
u'\uf9e6': 'CjkCompatibilityIdeographf9e6',
u'\u606b': 'CjkUnifiedIdeograph606b',
u'\u7fea': 'CjkUnifiedIdeograph7fea',
u'\ua36d': 'YiSyllableChet',
u'\u82ec': 'CjkUnifiedIdeograph82ec',
u'\u2971': 'EqualsSignAboveRightwardsArrow',
u'\u08f0': 'UnicodeCharacter0x8f0',
u'\uf277': 'UnicodeCharacter0xf277',
u'\ud1f6': 'HangulSyllableToegg',
u'\u787b': 'CjkUnifiedIdeograph787b',
u'\u57fa': 'CjkUnifiedIdeograph57fa',
u'\ubb7d': 'HangulSyllableMweb',
u'\u9afc': 'CjkUnifiedIdeograph9afc',
u'\u8081': 'CjkUnifiedIdeograph8081',
u'\ue100': 'UnicodeCharacter0xe100',
u'\uc583': 'HangulSyllableYad',
u'\u2202': 'PartialDifferential',
u'\u0685': 'ArabicLetterHahWithThreeDotsAbove',
u'\u6704': 'CjkUnifiedIdeograph6704',
u'\u4b87': 'CjkUnifiedIdeograph4b87',
u'\ub40e': 'HangulSyllableDwaebs',
u'\u9891': 'CjkUnifiedIdeograph9891',
u'\uf910': 'CjkCompatibilityIdeographf910',
u'\udd93': 'UnicodeCharacter0xdd93',
u'\u3a12': 'CjkUnifiedIdeograph3a12',
u'\u1e95': 'LatinSmallLetterZWithLineBelow',
u'\u7f14': 'CjkUnifiedIdeograph7f14',
u'\u2397': 'PreviousPage',
u'\u6499': 'CjkUnifiedIdeograph6499',
u'\u8c1e': 'CjkUnifiedIdeograph8c1e',
u'\ud120': 'HangulSyllableTyaels',
u'\ub5a3': 'HangulSyllableDdeogs',
u'\u1222': 'EthiopicSyllableSzi',
u'\uf6a5': 'UnicodeCharacter0xf6a5',
u'\u5724': 'CjkUnifiedIdeograph5724',
u'\u3ba7': 'CjkUnifiedIdeograph3ba7',
u'\u7ca9': 'CjkUnifiedIdeograph7ca9',
u'\ua930': 'RejangLetterKa',
u'\u8db3': 'CjkUnifiedIdeograph8db3',
u'\uea32': 'UnicodeCharacter0xea32',
u'\uceb5': 'HangulSyllableKyalg',
u'\u2f34': 'KangxiRadicalDottedCliff',
u'\u13b7': 'CherokeeLetterLu',
u'\u7036': 'CjkUnifiedIdeograph7036',
u'\u54b9': 'CjkUnifiedIdeograph54b9',
u'\u8140': 'CjkUnifiedIdeograph8140',
u'\uc242': 'HangulSyllableSweolp',
u'\ua6c5': 'UnicodeCharacter0xa6c5',
u'\u0744': 'SyriacTwoVerticalDotsBelow',
u'\uebc7': 'UnicodeCharacter0xebc7',
u'\u4846': 'CjkUnifiedIdeograph4846',
u'\u2cc9': 'CopticSmallLetterAkhmimicKhei',
u'\u71cb': 'CjkUnifiedIdeograph71cb',
u'\u9950': 'CjkUnifiedIdeograph9950',
u'\uda52': 'UnicodeCharacter0xda52',
u'\ubed5': 'HangulSyllableBbeonj',
u'\u1f54': 'GreekSmallLetterUpsilonWithPsiliAndOxia',
u'\uc3d7': 'HangulSyllableSsyeh',
u'\u2056': 'ThreeDotPunctuation',
u'\u04d9': 'CyrillicSmallLetterSchwa',
u'\u6558': 'CjkUnifiedIdeograph6558',
u'\u49db': 'CjkUnifiedIdeograph49db',
u'\ub262': 'HangulSyllableNwilm',
u'\u96e5': 'CjkUnifiedIdeograph96e5',
u'\uf764': 'UnicodeCharacter0xf764',
u'\udbe7': 'UnicodeCharacter0xdbe7',
u'\u3866': 'CjkUnifiedIdeograph3866',
u'\u1ce9': 'UnicodeCharacter0x1ce9',
u'\u7d68': 'CjkUnifiedIdeograph7d68',
u'\u21eb': 'UpwardsWhiteArrowOnPedestal',
u'\u62ed': 'CjkUnifiedIdeograph62ed',
u'\u8a72': 'CjkUnifiedIdeograph8a72',
u'\ucf74': 'HangulSyllableKwan',
u'\ub3f7': 'HangulSyllableDwac',
u'\u1076': 'MyanmarLetterShanKha',
u'\uf4f9': 'UnicodeCharacter0xf4f9',
u'\u5578': 'CjkUnifiedIdeograph5578',
u'\u39fb': 'CjkUnifiedIdeograph39fb',
u'\u7afd': 'CjkUnifiedIdeograph7afd',
u'\u4001': 'CjkUnifiedIdeograph4001',
u'\ua784': 'LatinCapitalLetterInsularS',
u'\ue886': 'UnicodeCharacter0xe886',
u'\u2d88': 'EthiopicSyllableNoa',
u'\u910b': 'CjkUnifiedIdeograph910b',
u'\u6e8a': 'CjkUnifiedIdeograph6e8a',
u'\ud20d': 'HangulSyllableToet',
u'\u170f': 'TagalogLetterWa',
u'\u5811': 'CjkUnifiedIdeograph5811',
u'\ubf94': 'HangulSyllableBboe',
u'\uc096': 'HangulSyllableBbinh',
u'\u0598': 'HebrewAccentZarqa',
u'\u469a': 'CjkUnifiedIdeograph469a',
u'\uaa1d': 'ChamLetterBa',
u'\uef1f': 'UnicodeCharacter0xef1f',
u'\u3021': 'HangzhouNumeralOne',
u'\u7523': 'CjkUnifiedIdeograph7523',
u'\u97a4': 'CjkUnifiedIdeograph97a4',
u'\ud8a6': 'UnicodeCharacter0xd8a6',
u'\u1da8': 'ModifierLetterSmallJWithCrossedtail',
u'\u5eaa': 'CjkUnifiedIdeograph5eaa',
u'\u822d': 'CjkUnifiedIdeograph822d',
u'\u63ac': 'CjkUnifiedIdeograph63ac',
u'\uc72f': 'HangulSyllableYulh',
u'\u0831': 'UnicodeCharacter0x831',
u'\u4d33': 'CjkUnifiedIdeograph4d33',
u'\ub0b6': 'HangulSyllableNaegg',
u'\uf5b8': 'UnicodeCharacter0xf5b8',
u'\u36ba': 'CjkUnifiedIdeograph36ba',
u'\u9a3d': 'CjkUnifiedIdeograph9a3d',
u'\u7bbc': 'CjkUnifiedIdeograph7bbc',
u'\udf3f': 'UnicodeCharacter0xdf3f',
u'\ue041': 'UnicodeCharacter0xe041',
u'\u2543': 'BoxDrawingsLeftUpHeavyAndRightDownLight',
u'\u6645': 'CjkUnifiedIdeograph6645',
u'\u88c6': 'CjkUnifiedIdeograph88c6',
u'\ucdc8': 'HangulSyllableCweok',
u'\u0eca': 'LaoToneMaiTi',
u'\u53cc': 'CjkUnifiedIdeograph53cc',
u'\ub74f': 'HangulSyllableDdyilb',
u'\uf851': 'UnicodeCharacter0xf851',
u'\u3d53': 'CjkUnifiedIdeograph3d53',
u'\u7e55': 'CjkUnifiedIdeograph7e55',
u'\ua5d8': 'VaiSyllableJo',
u'\ue6da': 'UnicodeCharacter0xe6da',
u'\u2bdc': 'UnicodeCharacter0x2bdc',
u'\u8f5f': 'CjkUnifiedIdeograph8f5f',
u'\u6cde': 'CjkUnifiedIdeograph6cde',
u'\ud061': 'HangulSyllableKyub',
u'\u1563': 'CanadianSyllabicsNcreeThii',
u'\u5665': 'CjkUnifiedIdeograph5665',
u'\ubde8': 'HangulSyllableBwiss',
u'\ufeea': 'ArabicLetterHehFinalForm',
u'\u03ec': 'CopticCapitalLetterShima',
u'\u44ee': 'CjkUnifiedIdeograph44ee',
u'\ua871': 'PhagspaSubjoinedLetterRa',
u'\ued73': 'UnicodeCharacter0xed73',
u'\u2e75': 'UnicodeCharacter0x2e75',
u'\u7377': 'CjkUnifiedIdeograph7377',
u'\u95f8': 'CjkUnifiedIdeograph95f8',
u'\ud6fa': 'HangulSyllableHweop',
u'\u1bfc': 'UnicodeCharacter0x1bfc',
u'\u5cfe': 'CjkUnifiedIdeograph5cfe',
u'\u6200': 'CjkUnifiedIdeograph6200',
u'\u4683': 'CjkUnifiedIdeograph4683',
u'\ua102': 'YiSyllableDi',
u'\u8585': 'CjkUnifiedIdeograph8585',
u'\u2b06': 'UpwardsBlackArrow',
u'\u0f89': 'TibetanSignMchuCan',
u'\uf00c': 'UnicodeCharacter0xf00c',
u'\ud48f': 'HangulSyllablePuc',
u'\u7a10': 'CjkUnifiedIdeograph7a10',
u'\u5e93': 'CjkUnifiedIdeograph5e93',
u'\ub912': 'HangulSyllableRweolp',
u'\u9d95': 'CjkUnifiedIdeograph9d95',
u'\u0316': 'CombiningGraveAccentBelow',
u'\ue799': 'UnicodeCharacter0xe799',
u'\u699d': 'CjkUnifiedIdeograph699d',
u'\uc81c': 'HangulSyllableJe',
u'\uac9f': 'HangulSyllableGes',
u'\u5220': 'CjkUnifiedIdeograph5220',
u'\u36a3': 'CjkUnifiedIdeograph36a3',
u'\u9122': 'CjkUnifiedIdeograph9122',
u'\u1b26': 'BalineseLetterNa',
u'\uffa9': 'HalfwidthHangulLetterRieul',
u'\u41ad': 'CjkUnifiedIdeograph41ad',
u'\ua02c': 'YiSyllableBut',
u'\u84af': 'CjkUnifiedIdeograph84af',
u'\u2a30': 'MultiplicationSignWithDotAbove',
u'\u0eb3': 'LaoVowelSignAm',
u'\uf336': 'UnicodeCharacter0xf336',
u'\ud7b9': 'UnicodeCharacter0xd7b9',
u'\u753a': 'CjkUnifiedIdeograph753a',
u'\u59bd': 'CjkUnifiedIdeograph59bd',
u'\ub83c': 'HangulSyllableRyeok',
u'\u9cbf': 'CjkUnifiedIdeograph9cbf',
u'\u0240': 'LatinSmallLetterZWithSwashTail',
u'\ue6c3': 'UnicodeCharacter0xe6c3',
u'\u68c7': 'CjkUnifiedIdeograph68c7',
u'\ucb46': 'HangulSyllableJjyop',
u'\uafc9': 'HangulSyllableGgub',
u'\u4d4a': 'CjkUnifiedIdeograph4d4a',
u'\u31cd': 'CjkStrokeHzw',
u'\u904c': 'CjkUnifiedIdeograph904c',
u'\u1a50': 'UnicodeCharacter0x1a50',
u'\ufed3': 'ArabicLetterFehInitialForm',
u'\u40d7': 'CjkUnifiedIdeograph40d7',
u'\ua356': 'YiSyllableZhux',
u'\u87d9': 'CjkUnifiedIdeograph87d9',
u'\u255a': 'BoxDrawingsDoubleUpAndRight',
u'\u09dd': 'BengaliLetterRha',
u'\uf260': 'UnicodeCharacter0xf260',
u'\ud6e3': 'HangulSyllableHweogs',
u'\u7464': 'CjkUnifiedIdeograph7464',
u'\u58e7': 'CjkUnifiedIdeograph58e7',
u'\ubb66': 'HangulSyllableMweoj',
u'\u9fe9': 'UnicodeCharacter0x9fe9',
u'\u3d6a': 'CjkUnifiedIdeograph3d6a',
u'\ue1ed': 'UnicodeCharacter0xe1ed',
u'\u6bf1': 'CjkUnifiedIdeograph6bf1',
u'\uca70': 'HangulSyllableJjel',
u'\uaef3': 'HangulSyllableGgeh',
u'\u4c74': 'CjkUnifiedIdeograph4c74',
u'\u30f7': 'KatakanaLetterVa',
u'\u9376': 'CjkUnifiedIdeograph9376',
u'\u157a': 'CanadianSyllabicsNunavikHaa',
u'\uf9fd': 'CjkCompatibilityIdeographf9fd',
u'\uc301': 'HangulSyllableSsalg',
u'\ua280': 'YiSyllableWat',
u'\u4505': 'CjkUnifiedIdeograph4505',
u'\u2484': 'ParenthesizedNumberSeventeen',
u'\u8807': 'CjkUnifiedIdeograph8807',
u'\u120b': 'EthiopicSyllableLaa',
u'\ued8a': 'UnicodeCharacter0xed8a',
u'\u778e': 'CjkUnifiedIdeograph778e',
u'\udb11': 'UnicodeCharacter0xdb11',
u'\uba90': 'HangulSyllableMyen',
u'\u5d15': 'CjkUnifiedIdeograph5d15',
u'\u3c94': 'CjkUnifiedIdeograph3c94',
u'\uea1b': 'UnicodeCharacter0xea1b',
u'\uc59a': 'HangulSyllableYaegg',
u'\u6c1f': 'CjkUnifiedIdeograph6c1f',
u'\u4f9e': 'CjkUnifiedIdeograph4f9e',
u'\ub321': 'HangulSyllableDyanj',
u'\u92a0': 'CjkUnifiedIdeograph92a0',
u'\u3525': 'CjkUnifiedIdeograph3525',
u'\u14a4': 'CanadianSyllabicsMaai',
u'\uc22b': 'HangulSyllableSus',
u'\uddaa': 'UnicodeCharacter0xddaa',
u'\u442f': 'CjkUnifiedIdeograph442f',
u'\u27ae': 'HeavyUpperRightshadowedWhiteRightwardsArrow',
u'\u8b31': 'CjkUnifiedIdeograph8b31',
u'\u0d35': 'MalayalamLetterVa',
u'\uecb4': 'UnicodeCharacter0xecb4',
u'\u76b8': 'CjkUnifiedIdeograph76b8',
u'\uda3b': 'UnicodeCharacter0xda3b',
u'\ub5ba': 'HangulSyllableDdeop',
u'\u5c3f': 'CjkUnifiedIdeograph5c3f',
u'\u3fbe': 'CjkUnifiedIdeograph3fbe',
u'\ue545': 'UnicodeCharacter0xe545',
u'\uc4c4': 'HangulSyllableSswils',
u'\u6f49': 'CjkUnifiedIdeograph6f49',
u'\u4ec8': 'CjkUnifiedIdeograph4ec8',
u'\ub24b': 'HangulSyllableNwelh',
u'\u8dca': 'CjkUnifiedIdeograph8dca',
u'\u344f': 'CjkUnifiedIdeograph344f',
u'\u17ce': 'KhmerSignKakabat',
u'\ufd55': 'ArabicLigatureTehWithMeemWithJeemInitialForm',
u'\udcd4': 'UnicodeCharacter0xdcd4',
u'\u4759': 'CjkUnifiedIdeograph4759',
u'\u26d8': 'UnicodeCharacter0x26d8',
u'\u8a5b': 'CjkUnifiedIdeograph8a5b',
u'\u0c5f': 'UnicodeCharacter0xc5f',
u'\uefde': 'UnicodeCharacter0xefde',
u'\u71e2': 'CjkUnifiedIdeograph71e2',
u'\ud565': 'HangulSyllableHalt',
u'\ub4e4': 'HangulSyllableDeul',
u'\u5f69': 'CjkUnifiedIdeograph5f69',
u'\u3ee8': 'CjkUnifiedIdeograph3ee8',
u'\ue46f': 'UnicodeCharacter0xe46f',
u'\uc7ee': 'HangulSyllableJyaelm',
u'\u6e73': 'CjkUnifiedIdeograph6e73',
u'\u49f2': 'CjkUnifiedIdeograph49f2',
u'\uad75': 'HangulSyllableGulg',
u'\u8cf4': 'CjkUnifiedIdeograph8cf4',
u'\u3779': 'CjkUnifiedIdeograph3779',
u'\u16f8': 'UnicodeCharacter0x16f8',
u'\ufc7f': 'ArabicLigatureQafWithYehFinalForm',
u'\udffe': 'UnicodeCharacter0xdffe',
u'\uaa06': 'ChamLetterKa',
u'\u8e89': 'CjkUnifiedIdeograph8e89',
u'\uef08': 'UnicodeCharacter0xef08',
u'\ud38b': 'HangulSyllablePeolh',
u'\u2c0a': 'GlagoliticCapitalLetterInitialIzhe',
u'\u108d': 'MyanmarSignShanCouncilEmphaticTone',
u'\u710c': 'CjkUnifiedIdeograph710c',
u'\u558f': 'CjkUnifiedIdeograph558f',
u'\u8216': 'CjkUnifiedIdeograph8216',
u'\uc718': 'HangulSyllableWiss',
u'\uab9b': 'UnicodeCharacter0xab9b',
u'\u041a': 'CyrillicCapitalLetterKa',
u'\ue89d': 'UnicodeCharacter0xe89d',
u'\u491c': 'CjkUnifiedIdeograph491c',
u'\u2d9f': 'UnicodeCharacter0x2d9f',
u'\u72a1': 'CjkUnifiedIdeograph72a1',
u'\u9a26': 'CjkUnifiedIdeograph9a26',
u'\udf28': 'UnicodeCharacter0xdf28',
u'\u83ab': 'CjkUnifiedIdeograph83ab',
u'\u1c2a': 'LepchaVowelSignU',
u'\uc0ad': 'HangulSyllableSag',
u'\u212c': 'ScriptCapitalB',
u'\u05af': 'HebrewMarkMasoraCircle',
u'\u662e': 'CjkUnifiedIdeograph662e',
u'\u4ab1': 'CjkUnifiedIdeograph4ab1',
u'\ub738': 'HangulSyllableDdeum',
u'\u9bbb': 'CjkUnifiedIdeograph9bbb',
u'\uf43a': 'UnicodeCharacter0xf43a',
u'\ud8bd': 'UnicodeCharacter0xd8bd',
u'\u393c': 'CjkUnifiedIdeograph393c',
u'\u1dbf': 'ModifierLetterSmallTheta',
u'\u7e3e': 'CjkUnifiedIdeograph7e3e',
u'\u22c1': 'NaryLogicalOr',
u'\u67c3': 'CjkUnifiedIdeograph67c3',
u'\u8f48': 'CjkUnifiedIdeograph8f48',
u'\ucc4a': 'HangulSyllableCaenh',
u'\ub0cd': 'HangulSyllableNaet',
u'\u114c': 'HangulChoseongYesieung',
u'\uf5cf': 'UnicodeCharacter0xf5cf',
u'\u564e': 'CjkUnifiedIdeograph564e',
u'\u3ad1': 'CjkUnifiedIdeograph3ad1',
u'\u7fd3': 'CjkUnifiedIdeograph7fd3',
u'\ua45a': 'YiSyllableNyux',
u'\u88dd': 'CjkUnifiedIdeograph88dd',
u'\ue95c': 'UnicodeCharacter0xe95c',
u'\ucddf': 'HangulSyllableCwes',
u'\u2e5e': 'UnicodeCharacter0x2e5e',
u'\u12e1': 'EthiopicSyllableZhu',
u'\u7360': 'CjkUnifiedIdeograph7360',
u'\u57e3': 'CjkUnifiedIdeograph57e3',
u'\ubc6a': 'HangulSyllableByaegg',
u'\uc16c': 'HangulSyllableSyeok',
u'\ua5ef': 'VaiSyllableGbe',
u'\u066e': 'ArabicLetterDotlessBeh',
u'\ueaf1': 'UnicodeCharacter0xeaf1',
u'\u4b70': 'CjkUnifiedIdeograph4b70',
u'\u2ff3': 'IdeographicDescriptionCharacterAboveToMiddleAndBelow',
u'\u6cf5': 'CjkUnifiedIdeograph6cf5',
u'\u947a': 'CjkUnifiedIdeograph947a',
u'\ud97c': 'UnicodeCharacter0xd97c',
u'\ubdff': 'HangulSyllableByulh',
u'\u1e7e': 'LatinCapitalLetterVWithDotBelow',
u'\u2380': 'InsertionSymbol',
u'\u8703': 'CjkUnifiedIdeograph8703',
u'\u6082': 'CjkUnifiedIdeograph6082',
u'\uc405': 'HangulSyllableSswab',
u'\u0907': 'DevanagariLetterI',
u'\u4e09': 'CjkUnifiedIdeograph4e09',
u'\ub18c': 'HangulSyllableNoss',
u'\uf68e': 'UnicodeCharacter0xf68e',
u'\u3b90': 'CjkUnifiedIdeograph3b90',
u'\u9f13': 'CjkUnifiedIdeograph9f13',
u'\u7892': 'CjkUnifiedIdeograph7892',
u'\udc15': 'UnicodeCharacter0xdc15',
u'\ue117': 'UnicodeCharacter0xe117',
u'\u2619': 'ReversedRotatedFloralHeartBullet',
u'\u6b1b': 'CjkUnifiedIdeograph6b1b',
u'\u899c': 'CjkUnifiedIdeograph899c',
u'\uce9e': 'HangulSyllableKaelp',
u'\u13a0': 'CherokeeLetterA',
u'\u50a2': 'CjkUnifiedIdeograph50a2',
u'\ub425': 'HangulSyllableDoelt',
u'\uf927': 'CjkCompatibilityIdeographf927',
u'\u3e29': 'CjkUnifiedIdeograph3e29',
u'\u432b': 'CjkUnifiedIdeograph432b',
u'\ua6ae': 'UnicodeCharacter0xa6ae',
u'\uebb0': 'UnicodeCharacter0xebb0',
u'\u28b2': 'BraillePatternDots2568',
u'\u8c35': 'CjkUnifiedIdeograph8c35',
u'\u6db4': 'CjkUnifiedIdeograph6db4',
u'\ud137': 'HangulSyllableTeod',
u'\u1639': 'CanadianSyllabicsCarrierTlha',
u'\u5b3b': 'CjkUnifiedIdeograph5b3b',
u'\ubebe': 'HangulSyllableBbyaelm',
u'\uc3c0': 'HangulSyllableSsyen',
u'\xc2': 'LatinCapitalLetterAWithCircumflex',
u'\u45c4': 'CjkUnifiedIdeograph45c4',
u'\ua947': 'RejangVowelSignI',
u'\uee49': 'UnicodeCharacter0xee49',
u'\u334b': 'SquareMega',
u'\u704d': 'CjkUnifiedIdeograph704d',
u'\u96ce': 'CjkUnifiedIdeograph96ce',
u'\udbd0': 'UnicodeCharacter0xdbd0',
u'\u18d2': 'UnicodeCharacter0x18d2',
u'\u5dd4': 'CjkUnifiedIdeograph5dd4',
u'\u8157': 'CjkUnifiedIdeograph8157',
u'\u62d6': 'CjkUnifiedIdeograph62d6',
u'\uc659': 'HangulSyllableWat',
u'\u0b5b': 'UnicodeCharacter0xb5b',
u'\u485d': 'CjkUnifiedIdeograph485d',
u'\ub3e0': 'HangulSyllableDwa',
u'\uf0e2': 'UnicodeCharacter0xf0e2',
u'\u35e4': 'CjkUnifiedIdeograph35e4',
u'\u9967': 'CjkUnifiedIdeograph9967',
u'\u7ae6': 'CjkUnifiedIdeograph7ae6',
u'\ude69': 'UnicodeCharacter0xde69',
u'\ue36b': 'UnicodeCharacter0xe36b',
u'\u206d': 'ActivateArabicFormShaping',
u'\u656f': 'CjkUnifiedIdeograph656f',
u'\u8bf0': 'CjkUnifiedIdeograph8bf0',
u'\uc8f2': 'HangulSyllableJyobs',
u'\u0df4': 'SinhalaPunctuationKunddaliya',
u'\u52f6': 'CjkUnifiedIdeograph52f6',
u'\ub679': 'HangulSyllableDdoeng',
u'\ufb7b': 'ArabicLetterTchehFinalForm',
u'\u387d': 'CjkUnifiedIdeograph387d',
u'\u7d7f': 'CjkUnifiedIdeograph7d7f',
u'\u0581': 'ArmenianSmallLetterCo',
u'\ue204': 'UnicodeCharacter0xe204',
u'\uce87': 'HangulSyllableKas',
u'\u6808': 'CjkUnifiedIdeograph6808',
u'\u548b': 'CjkUnifiedIdeograph548b',
u'\uab0a': 'UnicodeCharacter0xab0a',
u'\u978d': 'CjkUnifiedIdeograph978d',
u'\u310e': 'BopomofoLetterK',
u'\u1d91': 'LatinSmallLetterDWithHookAndTail',
u'\ufa14': 'CjkCompatibilityIdeographfa14',
u'\ua697': 'CyrillicSmallLetterShwe',
u'\u4018': 'CjkUnifiedIdeograph4018',
u'\u2c9b': 'CopticSmallLetterNi',
u'\u831a': 'CjkUnifiedIdeograph831a',
u'\u091e': 'DevanagariLetterNya',
u'\uf5a1': 'UnicodeCharacter0xf5a1',
u'\u73a5': 'CjkUnifiedIdeograph73a5',
u'\ud224': 'HangulSyllableTyoss',
u'\ubea7': 'HangulSyllableBbyalh',
u'\u5828': 'CjkUnifiedIdeograph5828',
u'\u04ab': 'CyrillicSmallLetterEsWithDescender',
u'\u9b2a': 'CjkUnifiedIdeograph9b2a',
u'\ue12e': 'UnicodeCharacter0xe12e',
u'\ucdb1': 'HangulSyllableCweog',
u'\u6f32': 'CjkUnifiedIdeograph6f32',
u'\u4bb5': 'CjkUnifiedIdeograph4bb5',
u'\uaa34': 'ChamConsonantSignRa',
u'\u96b7': 'CjkUnifiedIdeograph96b7',
u'\u3038': 'HangzhouNumeralTen',
u'\u1cbb': 'UnicodeCharacter0x1cbb',
u'\uf93e': 'CjkCompatibilityIdeographf93e',
u'\ua5c1': 'VaiSyllablePo',
u'\u4742': 'CjkUnifiedIdeograph4742',
u'\u23c5': 'DentistrySymbolLightUpAndHorizontalWithTriangle',
u'\u8244': 'CjkUnifiedIdeograph8244',
u'\u0848': 'UnicodeCharacter0x848',
u'\uf4cb': 'UnicodeCharacter0xf4cb',
u'\u72cf': 'CjkUnifiedIdeograph72cf',
u'\ud14e': 'HangulSyllableTegg',
u'\ubdd1': 'HangulSyllableBwet',
u'\u5f52': 'CjkUnifiedIdeograph5f52',
u'\u3bd5': 'CjkUnifiedIdeograph3bd5',
u'\u9a54': 'CjkUnifiedIdeograph9a54',
u'\ue058': 'UnicodeCharacter0xe058',
u'\uccdb': 'HangulSyllableCyeolb',
u'\u6e5c': 'CjkUnifiedIdeograph6e5c',
u'\u4adf': 'CjkUnifiedIdeograph4adf',
u'\ua95e': 'UnicodeCharacter0xa95e',
u'\u95e1': 'CjkUnifiedIdeograph95e1',
u'\u3762': 'CjkUnifiedIdeograph3762',
u'\u13e5': 'CherokeeLetterTsi',
u'\uf868': 'UnicodeCharacter0xf868',
u'\ua4eb': 'UnicodeCharacter0xa4eb',
u'\u466c': 'CjkUnifiedIdeograph466c',
u'\u22ef': 'MidlineHorizontalEllipsis',
u'\u816e': 'CjkUnifiedIdeograph816e',
u'\u0f72': 'TibetanVowelSignI',
u'\uebf5': 'UnicodeCharacter0xebf5',
u'\u71f9': 'CjkUnifiedIdeograph71f9',
u'\ud078': 'HangulSyllableKeuls',
u'\ubcfb': 'HangulSyllableBod',
u'\u5e7c': 'CjkUnifiedIdeograph5e7c',
u'\u3aff': 'CjkUnifiedIdeograph3aff',
u'\u997e': 'CjkUnifiedIdeograph997e',
u'\ue782': 'UnicodeCharacter0xe782',
u'\u6d86': 'CjkUnifiedIdeograph6d86',
u'\uc909': 'HangulSyllableJult',
u'\ua888': 'SaurashtraLetterVocalicR',
u'\u570d': 'CjkUnifiedIdeograph570d',
u'\u368c': 'CjkUnifiedIdeograph368c',
u'\u920f': 'CjkUnifiedIdeograph920f',
u'\u1813': 'MongolianDigitThree',
u'\uff92': 'HalfwidthKatakanaLetterMe',
u'\u6617': 'CjkUnifiedIdeograph6617',
u'\u4596': 'CjkUnifiedIdeograph4596',
u'\ua119': 'YiSyllableDur',
u'\u8098': 'CjkUnifiedIdeograph8098',
u'\u2f1d': 'KangxiRadicalMouth',
u'\u0e9c': 'LaoLetterPhoSung',
u'\uf023': 'UnicodeCharacter0xf023',
u'\ud7a2': 'HangulSyllableHip',
u'\u7e27': 'CjkUnifiedIdeograph7e27',
u'\u5da6': 'CjkUnifiedIdeograph5da6',
u'\ub929': 'HangulSyllableRwelg',
u'\u98a8': 'CjkUnifiedIdeograph98a8',
u'\u072d': 'SyriacLetterPersianBheth',
u'\ue6ac': 'UnicodeCharacter0xe6ac',
u'\u6cb0': 'CjkUnifiedIdeograph6cb0',
u'\uc833': 'HangulSyllableJec',
u'\uafb2': 'HangulSyllableGgyoj',
u'\u5637': 'CjkUnifiedIdeograph5637',
u'\u35b6': 'CjkUnifiedIdeograph35b6',
u'\u9139': 'CjkUnifiedIdeograph9139',
u'\u1f3d': 'GreekCapitalLetterIotaWithDasiaAndOxia',
u'\ufebc': 'ArabicLetterSadMedialForm',
u'\u6541': 'CjkUnifiedIdeograph6541',
u'\u44c0': 'CjkUnifiedIdeograph44c0',
u'\ua043': 'YiSyllablePuox',
u'\u87c2': 'CjkUnifiedIdeograph87c2',
u'\u2e47': 'UnicodeCharacter0x2e47',
u'\u0dc6': 'SinhalaLetterFayanna',
u'\uf74d': 'UnicodeCharacter0xf74d',
u'\ud6cc': 'HangulSyllableHul',
u'\u7d51': 'CjkUnifiedIdeograph7d51',
u'\u5cd0': 'CjkUnifiedIdeograph5cd0',
u'\ub853': 'HangulSyllableRyes',
u'\u9fd2': 'UnicodeCharacter0x9fd2',
u'\u0657': 'ArabicInvertedDamma',
u'\ue5d6': 'UnicodeCharacter0xe5d6',
u'\u63da': 'CjkUnifiedIdeograph63da',
u'\ucf5d': 'HangulSyllableKolg',
u'\uaedc': 'HangulSyllableGgen',
u'\u5561': 'CjkUnifiedIdeograph5561',
u'\u34e0': 'CjkUnifiedIdeograph34e0',
u'\u9063': 'CjkUnifiedIdeograph9063',
u'\u1e67': 'LatinSmallLetterSWithCaronAndDotAbove',
u'\ufde6': 'UnicodeCharacter0xfde6',
u'\u646b': 'CjkUnifiedIdeograph646b',
u'\u7bea': 'CjkUnifiedIdeograph7bea',
u'\ua76d': 'LatinSmallLetterIs',
u'\u86ec': 'CjkUnifiedIdeograph86ec',
u'\u2d71': 'UnicodeCharacter0x2d71',
u'\u0cf0': 'UnicodeCharacter0xcf0',
u'\uf677': 'UnicodeCharacter0xf677',
u'\ud5f6': 'HangulSyllableHebs',
u'\u7c7b': 'CjkUnifiedIdeograph7c7b',
u'\u53fa': 'CjkUnifiedIdeograph53fa',
u'\ubf7d': 'HangulSyllableBbwaenj',
u'\u9efc': 'CjkUnifiedIdeograph9efc',
u'\u8481': 'CjkUnifiedIdeograph8481',
u'\ue500': 'UnicodeCharacter0xe500',
u'\uc183': 'HangulSyllableSyes',
u'\u2602': 'Umbrella',
u'\u0285': 'LatinSmallLetterSquatReversedEsh',
u'\u6304': 'CjkUnifiedIdeograph6304',
u'\u4f87': 'CjkUnifiedIdeograph4f87',
u'\ub00e': 'HangulSyllableGgwigg',
u'\u9c91': 'CjkUnifiedIdeograph9c91',
u'\ufd10': 'ArabicLigatureDadWithRehIsolatedForm',
u'\ud993': 'UnicodeCharacter0xd993',
u'\u3e12': 'CjkUnifiedIdeograph3e12',
u'\u1a95': 'UnicodeCharacter0x1a95',
u'\u7b14': 'CjkUnifiedIdeograph7b14',
u'\u2797': 'UnicodeCharacter0x2797',
u'\u6099': 'CjkUnifiedIdeograph6099',
u'\u881e': 'CjkUnifiedIdeograph881e',
u'\ud520': 'HangulSyllablePyi',
u'\ub1a3': 'HangulSyllableNwalh',
u'\u1622': 'CanadianSyllabicsCarrierLu',
u'\uf2a5': 'UnicodeCharacter0xf2a5',
u'\u5324': 'CjkUnifiedIdeograph5324',
u'\u3fa7': 'CjkUnifiedIdeograph3fa7',
u'\u78a9': 'CjkUnifiedIdeograph78a9',
u'\uad30': 'HangulSyllableGwaek',
u'\u89b3': 'CjkUnifiedIdeograph89b3',
u'\uee32': 'UnicodeCharacter0xee32',
u'\ucab5': 'HangulSyllableJjyeng',
u'\u2b34': 'LeftwardsTwoheadedArrowWithVerticalStroke',
u'\u17b7': 'KhmerVowelSignI',
u'\u7436': 'CjkUnifiedIdeograph7436',
u'\u50b9': 'CjkUnifiedIdeograph50b9',
u'\u8540': 'CjkUnifiedIdeograph8540',
u'\uc642': 'HangulSyllableWagg',
u'\ua2c5': 'YiSyllableCux',
u'\u0344': 'CombiningGreekDialytikaTonos',
u'\uefc7': 'UnicodeCharacter0xefc7',
u'\u4c46': 'CjkUnifiedIdeograph4c46',
u'\u28c9': 'BraillePatternDots1478',
u'\u75cb': 'CjkUnifiedIdeograph75cb',
u'\u9d50': 'CjkUnifiedIdeograph9d50',
u'\ude52': 'UnicodeCharacter0xde52',
u'\ubad5': 'HangulSyllableMwab',
u'\u1b54': 'BalineseDigitFour',
u'\uc7d7': 'HangulSyllableJyalh',
u'\u2456': 'UnicodeCharacter0x2456',
u'\xd9': 'LatinCapitalLetterUWithGrave',
u'\u6158': 'CjkUnifiedIdeograph6158',
u'\u4ddb': 'HexagramForGreatPreponderance',
u'\ub662': 'HangulSyllableDdwaep',
u'\u92e5': 'CjkUnifiedIdeograph92e5',
u'\uf364': 'UnicodeCharacter0xf364',
u'\udfe7': 'UnicodeCharacter0xdfe7',
u'\u3c66': 'CjkUnifiedIdeograph3c66',
u'\u18e9': 'UnicodeCharacter0x18e9',
u'\u7968': 'CjkUnifiedIdeograph7968',
u'\u25eb': 'WhiteSquareWithVerticalBisectingLine',
u'\u66ed': 'CjkUnifiedIdeograph66ed',
u'\u8e72': 'CjkUnifiedIdeograph8e72',
u'\ucb74': 'HangulSyllableJjweom',
u'\ub7f7': 'HangulSyllableReolb',
u'\u1476': 'CanadianSyllabicsKwi',
u'\uf0f9': 'UnicodeCharacter0xf0f9',
u'\u5178': 'CjkUnifiedIdeograph5178',
u'\u3dfb': 'CjkUnifiedIdeograph3dfb',
u'\u7efd': 'CjkUnifiedIdeograph7efd',
u'\u4401': 'CjkUnifiedIdeograph4401',
u'\ua384': 'YiSyllableRret',
u'\uec86': 'UnicodeCharacter0xec86',
u'\u2988': 'ZNotationRightImageBracket',
u'\u950b': 'CjkUnifiedIdeograph950b',
u'\u6a8a': 'CjkUnifiedIdeograph6a8a',
u'\ud60d': 'HangulSyllableHyeolt',
u'\u130f': 'EthiopicSyllableGoa',
u'\u5c11': 'CjkUnifiedIdeograph5c11',
u'\ubb94': 'HangulSyllableMwils',
u'\uc496': 'HangulSyllableSsweoj',
u'\u0198': 'LatinCapitalLetterKWithHook',
u'\u429a': 'CjkUnifiedIdeograph429a',
u'\uae1d': 'HangulSyllableGyilg',
u'\ueb1f': 'UnicodeCharacter0xeb1f',
u'\u3421': 'CjkUnifiedIdeograph3421',
u'\u7123': 'CjkUnifiedIdeograph7123',
u'\u93a4': 'CjkUnifiedIdeograph93a4',
u'\udca6': 'UnicodeCharacter0xdca6',
u'\u19a8': 'NewTaiLueLetterLowKva',
u'\u5aaa': 'CjkUnifiedIdeograph5aaa',
u'\u862d': 'CjkUnifiedIdeograph862d',
u'\u67ac': 'CjkUnifiedIdeograph67ac',
u'\uc32f': 'HangulSyllableSsaeh',
u'\u0c31': 'TeluguLetterRra',
u'\u4933': 'CjkUnifiedIdeograph4933',
u'\ub4b6': 'HangulSyllableDwibs',
u'\uf1b8': 'UnicodeCharacter0xf1b8',
u'\u32ba': 'CircledNumberFortyFive',
u'\u9e3d': 'CjkUnifiedIdeograph9e3d',
u'\u7fbc': 'CjkUnifiedIdeograph7fbc',
u'\udb3f': 'UnicodeCharacter0xdb3f',
u'\ue441': 'UnicodeCharacter0xe441',
u'\u2143': 'ReversedSansserifCapitalL',
u'\u6245': 'CjkUnifiedIdeograph6245',
u'\u8cc6': 'CjkUnifiedIdeograph8cc6',
u'\uc9c8': 'HangulSyllableJil',
u'\u0aca': 'UnicodeCharacter0xaca',
u'\u57cc': 'CjkUnifiedIdeograph57cc',
u'\ub34f': 'HangulSyllableDyaec',
u'\ufc51': 'ArabicLigatureHehWithJeemIsolatedForm',
u'\u3953': 'CjkUnifiedIdeograph3953',
u'\u7a55': 'CjkUnifiedIdeograph7a55',
u'\ua1d8': 'YiSyllableLyrx',
u'\ue2da': 'UnicodeCharacter0xe2da',
u'\u2fdc': 'UnicodeCharacter0x2fdc',
u'\u8b5f': 'CjkUnifiedIdeograph8b5f',
u'\u68de': 'CjkUnifiedIdeograph68de',
u'\ud461': 'HangulSyllablePyonj',
u'\u1163': 'HangulJungseongYa',
u'\u5265': 'CjkUnifiedIdeograph5265',
u'\ub9e8': 'HangulSyllableMaen',
u'\ufaea': 'UnicodeCharacter0xfaea',
u'\u07ec': 'NkoCombiningShortLowTone',
u'\u40ee': 'CjkUnifiedIdeograph40ee',
u'\uac71': 'HangulSyllableGeog',
u'\ue973': 'UnicodeCharacter0xe973',
u'\u2a75': 'TwoConsecutiveEqualsSigns',
u'\u7777': 'CjkUnifiedIdeograph7777',
u'\u91f8': 'CjkUnifiedIdeograph91f8',
u'\ud2fa': 'HangulSyllableTilm',
u'\u1ffc': 'GreekCapitalLetterOmegaWithProsgegrammeni',
u'\u58fe': 'CjkUnifiedIdeograph58fe',
u'\u6600': 'CjkUnifiedIdeograph6600',
u'\u4283': 'CjkUnifiedIdeograph4283',
u'\ua502': 'VaiSyllableHee',
u'\u8185': 'CjkUnifiedIdeograph8185',
u'\u2f06': 'KangxiRadicalTwo',
u'\u0b89': 'TamilLetterU',
u'\uf40c': 'UnicodeCharacter0xf40c',
u'\ud08f': 'HangulSyllableKyid',
u'\u7e10': 'CjkUnifiedIdeograph7e10',
u'\u5a93': 'CjkUnifiedIdeograph5a93',
u'\ubd12': 'HangulSyllableBwagg',
u'\u9995': 'CjkUnifiedIdeograph9995',
u'\u0716': 'SyriacLetterDotlessDalathRish',
u'\ue399': 'UnicodeCharacter0xe399',
u'\u6d9d': 'CjkUnifiedIdeograph6d9d',
u'\ucc1c': 'HangulSyllableJjim',
u'\ua89f': 'SaurashtraLetterDdha',
u'\u5620': 'CjkUnifiedIdeograph5620',
u'\u32a3': 'CircledIdeographCorrect',
u'\u9522': 'CjkUnifiedIdeograph9522',
u'\u1f26': 'GreekSmallLetterEtaWithPsiliAndPerispomeni',
u'\ufba9': 'ArabicLetterHehGoalMedialForm',
u'\u45ad': 'CjkUnifiedIdeograph45ad',
u'\ua42c': 'YiSyllableJjur',
u'\u80af': 'CjkUnifiedIdeograph80af',
u'\u2e30': 'RingPoint',
u'\u0ab3': 'GujaratiLetterLla',
u'\uf736': 'UnicodeCharacter0xf736',
u'\ud3b9': 'HangulSyllablePyeonj',
u'\u713a': 'CjkUnifiedIdeograph713a',
u'\u5dbd': 'CjkUnifiedIdeograph5dbd',
u'\ubc3c': 'HangulSyllableBaels',
u'\u98bf': 'CjkUnifiedIdeograph98bf',
u'\u0640': 'ArabicTatweel',
u'\ue2c3': 'UnicodeCharacter0xe2c3',
u'\u6cc7': 'CjkUnifiedIdeograph6cc7',
u'\ucf46': 'HangulSyllableKyelp',
u'\uabc9': 'UnicodeCharacter0xabc9',
u'\u494a': 'CjkUnifiedIdeograph494a',
u'\u35cd': 'CjkUnifiedIdeograph35cd',
u'\u944c': 'CjkUnifiedIdeograph944c',
u'\u1e50': 'LatinCapitalLetterOWithMacronAndGrave',
u'\ufad3': 'CjkCompatibilityIdeographfad3',
u'\u44d7': 'CjkUnifiedIdeograph44d7',
u'\ua756': 'LatinCapitalLetterQWithStrokeThroughDescender',
u'\u83d9': 'CjkUnifiedIdeograph83d9',
u'\u215a': 'VulgarFractionFiveSixths',
u'\u0ddd': 'SinhalaVowelSignKombuvaHaaDigaAelapilla',
u'\uf660': 'UnicodeCharacter0xf660',
u'\ud2e3': 'HangulSyllableTyilh',
u'\u7064': 'CjkUnifiedIdeograph7064',
u'\u5ce7': 'CjkUnifiedIdeograph5ce7',
u'\ubf66': 'HangulSyllableBbwalm',
u'\u9be9': 'CjkUnifiedIdeograph9be9',
u'\u396a': 'CjkUnifiedIdeograph396a',
u'\ue5ed': 'UnicodeCharacter0xe5ed',
u'\u6ff1': 'CjkUnifiedIdeograph6ff1',
u'\uce70': 'HangulSyllableCik',
u'\uaaf3': 'UnicodeCharacter0xaaf3',
u'\u4874': 'CjkUnifiedIdeograph4874',
u'\u34f7': 'CjkUnifiedIdeograph34f7',
u'\u9776': 'CjkUnifiedIdeograph9776',
u'\u117a': 'HangulJungseongEoo',
u'\ufdfd': 'ArabicLigatureBismillahArrahmanArraheem',
u'\uc701': 'HangulSyllableWet',
u'\ua680': 'CyrillicCapitalLetterDwe',
u'\u4105': 'CjkUnifiedIdeograph4105',
u'\u2084': 'SubscriptFour',
u'\u8c07': 'CjkUnifiedIdeograph8c07',
u'\u160b': 'CanadianSyllabicsCarrierMee',
u'\ue98a': 'UnicodeCharacter0xe98a',
u'\u738e': 'CjkUnifiedIdeograph738e',
u'\udf11': 'UnicodeCharacter0xdf11',
u'\ube90': 'HangulSyllableBbaess',
u'\u5915': 'CjkUnifiedIdeograph5915',
u'\u3894': 'CjkUnifiedIdeograph3894',
u'\uee1b': 'UnicodeCharacter0xee1b',
u'\uc19a': 'HangulSyllableSolp',
u'\u681f': 'CjkUnifiedIdeograph681f',
u'\u4b9e': 'CjkUnifiedIdeograph4b9e',
u'\ub721': 'HangulSyllableDdyung',
u'\u96a0': 'CjkUnifiedIdeograph96a0',
u'\u3125': 'BopomofoLetterEng',
u'\u10a4': 'GeorgianCapitalLetterEn',
u'\uc62b': 'HangulSyllableOd',
u'\ud9aa': 'UnicodeCharacter0xd9aa',
u'\u402f': 'CjkUnifiedIdeograph402f',
u'\u23ae': 'IntegralExtension',
u'\u8f31': 'CjkUnifiedIdeograph8f31',
u'\u0935': 'DevanagariLetterVa',
u'\ue8b4': 'UnicodeCharacter0xe8b4',
u'\u72b8': 'CjkUnifiedIdeograph72b8',
u'\ude3b': 'UnicodeCharacter0xde3b',
u'\ub1ba': 'HangulSyllableNwaelm',
u'\u583f': 'CjkUnifiedIdeograph583f',
u'\u3bbe': 'CjkUnifiedIdeograph3bbe',
u'\ue145': 'UnicodeCharacter0xe145',
u'\uc0c4': 'HangulSyllableSak',
u'\u6b49': 'CjkUnifiedIdeograph6b49',
u'\u4ac8': 'CjkUnifiedIdeograph4ac8',
u'\ub64b': 'HangulSyllableDdwaegs',
u'\u89ca': 'CjkUnifiedIdeograph89ca',
u'\u304f': 'HiraganaLetterKu',
u'\u13ce': 'CherokeeLetterSe',
u'\uf955': 'CjkCompatibilityIdeographf955',
u'\ud8d4': 'UnicodeCharacter0xd8d4',
u'\u4359': 'CjkUnifiedIdeograph4359',
u'\u22d8': 'VeryMuchLessthan',
u'\u8e5b': 'CjkUnifiedIdeograph8e5b',
u'\u085f': 'UnicodeCharacter0x85f',
u'\uebde': 'UnicodeCharacter0xebde',
u'\u75e2': 'CjkUnifiedIdeograph75e2',
u'\ud165': 'HangulSyllableTet',
u'\ub0e4': 'HangulSyllableNyass',
u'\u5b69': 'CjkUnifiedIdeograph5b69',
u'\u3ae8': 'CjkUnifiedIdeograph3ae8',
u'\ue06f': 'UnicodeCharacter0xe06f',
u'\uc3ee': 'HangulSyllableSsoj',
u'\u6a73': 'CjkUnifiedIdeograph6a73',
u'\u4df2': 'HexagramForTheArousingThunder',
u'\ua975': 'UnicodeCharacter0xa975',
u'\u88f4': 'CjkUnifiedIdeograph88f4',
u'\u3379': 'SquareDmCubed',
u'\u12f8': 'EthiopicSyllableDda',
u'\uf87f': 'UnicodeCharacter0xf87f',
u'\udbfe': 'UnicodeCharacter0xdbfe',
u'\uae06': 'HangulSyllableGeulp',
u'\u8a89': 'CjkUnifiedIdeograph8a89',
u'\ueb08': 'UnicodeCharacter0xeb08',
u'\ud78b': 'HangulSyllableHigs',
u'\u280a': 'BraillePatternDots24',
u'\u148d': 'CanadianSyllabicsCo',
u'\u750c': 'CjkUnifiedIdeograph750c',
u'\u518f': 'CjkUnifiedIdeograph518f',
u'\u8616': 'CjkUnifiedIdeograph8616',
u'\uc318': 'HangulSyllableSsaen',
u'\uaf9b': 'HangulSyllableGgoeh',
u'\uec9d': 'UnicodeCharacter0xec9d',
u'\u4d1c': 'CjkUnifiedIdeograph4d1c',
u'\u299f': 'AcuteAngle',
u'\u76a1': 'CjkUnifiedIdeograph76a1',
u'\u9e26': 'CjkUnifiedIdeograph9e26',
u'\udb28': 'UnicodeCharacter0xdb28',
u'\u87ab': 'CjkUnifiedIdeograph87ab',
u'\u182a': 'MongolianLetterBa',
u'\uc4ad': 'HangulSyllableSsweb',
u'\u252c': 'BoxDrawingsLightDownAndHorizontal',
u'\u01af': 'LatinCapitalLetterUWithHorn',
u'\u622e': 'CjkUnifiedIdeograph622e',
u'\u4eb1': 'CjkUnifiedIdeograph4eb1',
u'\ub338': 'HangulSyllableDyae',
u'\u9fbb': 'CjkUnifiedIdeograph9fbb',
u'\uf03a': 'UnicodeCharacter0xf03a',
u'\udcbd': 'UnicodeCharacter0xdcbd',
u'\u3d3c': 'CjkUnifiedIdeograph3d3c',
u'\u19bf': 'NewTaiLueVowelSignUey',
u'\u7a3e': 'CjkUnifiedIdeograph7a3e',
u'\u26c1': 'WhiteDraughtsKing',
u'\u63c3': 'CjkUnifiedIdeograph63c3',
u'\u8b48': 'CjkUnifiedIdeograph8b48',
u'\uc84a': 'HangulSyllableJyeobs',
u'\ub4cd': 'HangulSyllableDyult',
u'\u154c': 'CanadianSyllabicsRaa',
u'\uf1cf': 'UnicodeCharacter0xf1cf',
u'\u524e': 'CjkUnifiedIdeograph524e',
u'\u3ed1': 'CjkUnifiedIdeograph3ed1',
u'\u7bd3': 'CjkUnifiedIdeograph7bd3',
u'\ua05a': 'YiSyllableBbiet',
u'\u8cdd': 'CjkUnifiedIdeograph8cdd',
u'\ued5c': 'UnicodeCharacter0xed5c',
u'\uc9df': 'HangulSyllableJjags',
u'\u2a5e': 'LogicalAndWithDoubleOverbar',
u'\u16e1': 'RunicLetterIor',
u'\u7760': 'CjkUnifiedIdeograph7760',
u'\u53e3': 'CjkUnifiedIdeograph53e3',
u'\ub86a': 'HangulSyllableRolp',
u'\uc56c': 'HangulSyllableAels',
u'\ua1ef': 'YiSyllableGex',
u'\u026e': 'LatinSmallLetterLezh',
u'\ueef1': 'UnicodeCharacter0xeef1',
u'\u4f70': 'CjkUnifiedIdeograph4f70',
u'\u2bf3': 'UnicodeCharacter0x2bf3',
u'\u68f5': 'CjkUnifiedIdeograph68f5',
u'\u907a': 'CjkUnifiedIdeograph907a',
u'\udd7c': 'UnicodeCharacter0xdd7c',
u'\ub9ff': 'HangulSyllableMaeh',
u'\u1a7e': 'UnicodeCharacter0x1a7e',
u'\u2780': 'DingbatCircledSansserifDigitOne',
u'\u8303': 'CjkUnifiedIdeograph8303',
u'\u6482': 'CjkUnifiedIdeograph6482',
u'\uc005': 'HangulSyllableBbweg',
u'\u0d07': 'MalayalamLetterI',
u'\u4a09': 'CjkUnifiedIdeograph4a09',
u'\ub58c': 'HangulSyllableDdyael',
u'\uf28e': 'UnicodeCharacter0xf28e',
u'\u3f90': 'CjkUnifiedIdeograph3f90',
u'\u9b13': 'CjkUnifiedIdeograph9b13',
u'\u7c92': 'CjkUnifiedIdeograph7c92',
u'\ud815': 'UnicodeCharacter0xd815',
u'\ue517': 'UnicodeCharacter0xe517',
u'\u2219': 'BulletOperator',
u'\u6f1b': 'CjkUnifiedIdeograph6f1b',
u'\u8d9c': 'CjkUnifiedIdeograph8d9c',
u'\uca9e': 'HangulSyllableJjyeop',
u'\u17a0': 'KhmerLetterHa',
u'\u54a2': 'CjkUnifiedIdeograph54a2',
u'\ub025': 'HangulSyllableGgwit',
u'\ufd27': 'ArabicLigatureSheenWithKhahFinalForm',
u'\u3a29': 'CjkUnifiedIdeograph3a29',
u'\u472b': 'CjkUnifiedIdeograph472b',
u'\ua2ae': 'YiSyllableCit',
u'\uefb0': 'UnicodeCharacter0xefb0',
u'\u2cb2': 'CopticCapitalLetterDialectpAlef',
u'\u8835': 'CjkUnifiedIdeograph8835',
u'\u69b4': 'CjkUnifiedIdeograph69b4',
u'\ud537': 'HangulSyllablePyic',
u'\u1239': 'EthiopicSyllableShu',
u'\u5f3b': 'CjkUnifiedIdeograph5f3b',
u'\ubabe': 'HangulSyllableMoj',
u'\uc7c0': 'HangulSyllableJaess',
u'\u04c2': 'CyrillicSmallLetterZheWithBreve',
u'\u41c4': 'CjkUnifiedIdeograph41c4',
u'\uad47': 'HangulSyllableGoes',
u'\uea49': 'UnicodeCharacter0xea49',
u'\u374b': 'CjkUnifiedIdeograph374b',
u'\u744d': 'CjkUnifiedIdeograph744d',
u'\u92ce': 'CjkUnifiedIdeograph92ce',
u'\udfd0': 'UnicodeCharacter0xdfd0',
u'\u1cd2': 'UnicodeCharacter0x1cd2',
u'\u59d4': 'CjkUnifiedIdeograph59d4',
u'\u8557': 'CjkUnifiedIdeograph8557',
u'\u66d6': 'CjkUnifiedIdeograph66d6',
u'\uc259': 'HangulSyllableSwelg',
u'\u0f5b': 'TibetanLetterDza',
u'\u4c5d': 'CjkUnifiedIdeograph4c5d',
u'\ub7e0': 'HangulSyllableRyaem',
u'\uf4e2': 'UnicodeCharacter0xf4e2',
u'\u31e4': 'UnicodeCharacter0x31e4',
u'\u9d67': 'CjkUnifiedIdeograph9d67',
u'\u7ee6': 'CjkUnifiedIdeograph7ee6',
u'\uda69': 'UnicodeCharacter0xda69',
u'\ue76b': 'UnicodeCharacter0xe76b',
u'\u246d': 'CircledNumberFourteen',
u'\u616f': 'CjkUnifiedIdeograph616f',
u'\u8ff0': 'CjkUnifiedIdeograph8ff0',
u'\uccf2': 'HangulSyllableCyenh',
u'\u09f4': 'BengaliCurrencyNumeratorOne',
u'\u56f6': 'CjkUnifiedIdeograph56f6',
u'\ub279': 'HangulSyllableNyunj',
u'\uff7b': 'HalfwidthKatakanaLetterSa',
u'\u3c7d': 'CjkUnifiedIdeograph3c7d',
u'\u797f': 'CjkUnifiedIdeograph797f',
u'\u0981': 'BengaliSignCandrabindu',
u'\uee04': 'UnicodeCharacter0xee04',
u'\uc287': 'HangulSyllableSwih',
u'\u6408': 'CjkUnifiedIdeograph6408',
u'\u588b': 'CjkUnifiedIdeograph588b',
u'\ua70a': 'ModifierLetterMidDottedToneBar',
u'\u9b8d': 'CjkUnifiedIdeograph9b8d',
u'\u3d0e': 'CjkUnifiedIdeograph3d0e',
u'\u1191': 'HangulJungseongYuyeo',
u'\uf614': 'UnicodeCharacter0xf614',
u'\uaa97': 'UnicodeCharacter0xaa97',
u'\u4c18': 'CjkUnifiedIdeograph4c18',
u'\u209b': 'UnicodeCharacter0x209b',
u'\u8f1a': 'CjkUnifiedIdeograph8f1a',
u'\u051e': 'CyrillicCapitalLetterAleutKa',
u'\uf9a1': 'CjkCompatibilityIdeographf9a1',
u'\u7fa5': 'CjkUnifiedIdeograph7fa5',
u'\ude24': 'UnicodeCharacter0xde24',
u'\ub2a7': 'HangulSyllableNeuc',
u'\u5428': 'CjkUnifiedIdeograph5428',
u'\u08ab': 'UnicodeCharacter0x8ab',
u'\u972a': 'CjkUnifiedIdeograph972a',
u'\ued2e': 'UnicodeCharacter0xed2e',
u'\uc1b1': 'HangulSyllableSwalg',
u'\u6332': 'CjkUnifiedIdeograph6332',
u'\u47b5': 'CjkUnifiedIdeograph47b5',
u'\ua634': 'UnicodeCharacter0xa634',
u'\u9ab7': 'CjkUnifiedIdeograph9ab7',
u'\u3c38': 'CjkUnifiedIdeograph3c38',
u'\u10bb': 'GeorgianCapitalLetterJil',
u'\uf53e': 'UnicodeCharacter0xf53e',
u'\ua9c1': 'UnicodeCharacter0xa9c1',
u'\u4b42': 'CjkUnifiedIdeograph4b42',
u'\u2fc5': 'KangxiRadicalDeer',
u'\u8e44': 'CjkUnifiedIdeograph8e44',
u'\u0448': 'CyrillicSmallLetterSha',
u'\uf8cb': 'UnicodeCharacter0xf8cb',
u'\u7ecf': 'CjkUnifiedIdeograph7ecf',
u'\udd4e': 'UnicodeCharacter0xdd4e',
u'\ub1d1': 'HangulSyllableNoenj',
u'\u5352': 'CjkUnifiedIdeograph5352',
u'\u37d5': 'CjkUnifiedIdeograph37d5',
u'\u9654': 'CjkUnifiedIdeograph9654',
u'\uec58': 'UnicodeCharacter0xec58',
u'\uc0db': 'HangulSyllableSaes',
u'\u625c': 'CjkUnifiedIdeograph625c',
u'\u46df': 'CjkUnifiedIdeograph46df',
u'\ua55e': 'VaiSyllableLa',
u'\u99e1': 'CjkUnifiedIdeograph99e1',
u'\u3b62': 'CjkUnifiedIdeograph3b62',
u'\u1fe5': 'GreekSmallLetterRhoWithDasia',
u'\uf468': 'UnicodeCharacter0xf468',
u'\ua8eb': 'UnicodeCharacter0xa8eb',
u'\u4a6c': 'CjkUnifiedIdeograph4a6c',
u'\u2eef': 'CjkRadicalJsimplifiedDragon',
u'\u8d6e': 'CjkUnifiedIdeograph8d6e',
u'\u0372': 'GreekCapitalLetterArchaicSampi',
u'\ue7f5': 'UnicodeCharacter0xe7f5',
u'\u7df9': 'CjkUnifiedIdeograph7df9',
u'\udc78': 'UnicodeCharacter0xdc78',
u'\ub0fb': 'HangulSyllableNyaelh',
u'\u527c': 'CjkUnifiedIdeograph527c',
u'\u36ff': 'CjkUnifiedIdeograph36ff',
u'\u957e': 'CjkUnifiedIdeograph957e',
u'\u0c03': 'TeluguSignVisarga',
u'\ueb82': 'UnicodeCharacter0xeb82',
u'\u6186': 'CjkUnifiedIdeograph6186',
u'\uc509': 'HangulSyllableSseut',
u'\ua488': 'YiSyllableYyx',
u'\u5b0d': 'CjkUnifiedIdeograph5b0d',
u'\u3a8c': 'CjkUnifiedIdeograph3a8c',
u'\u9e0f': 'CjkUnifiedIdeograph9e0f',
u'\u1413': 'CanadianSyllabicsWestcreeWo',
u'\uf392': 'UnicodeCharacter0xf392',
u'\u6a17': 'CjkUnifiedIdeograph6a17',
u'\u4996': 'CjkUnifiedIdeograph4996',
u'\uad19': 'HangulSyllableGwaeg',
u'\u8c98': 'CjkUnifiedIdeograph8c98',
u'\u231d': 'TopRightCorner',
u'\u029c': 'LatinLetterSmallCapitalH',
u'\ufc23': 'ArabicLigatureDadWithHahIsolatedForm',
u'\udba2': 'UnicodeCharacter0xdba2',
u'\u7227': 'CjkUnifiedIdeograph7227',
u'\u51a6': 'CjkUnifiedIdeograph51a6',
u'\ub529': 'HangulSyllableDing',
u'\u94a8': 'CjkUnifiedIdeograph94a8',
u'\u0b2d': 'OriyaLetterBha',
u'\ueaac': 'UnicodeCharacter0xeaac',
u'\u60b0': 'CjkUnifiedIdeograph60b0',
u'\uc433': 'HangulSyllableSsoed',
u'\ua3b2': 'YiSyllableShot',
u'\u5a37': 'CjkUnifiedIdeograph5a37',
u'\u39b6': 'CjkUnifiedIdeograph39b6',
u'\u9d39': 'CjkUnifiedIdeograph9d39',
u'\u133d': 'EthiopicSyllableTse',
u'\uf2bc': 'UnicodeCharacter0xf2bc',
u'\u6941': 'CjkUnifiedIdeograph6941',
u'\u48c0': 'CjkUnifiedIdeograph48c0',
u'\uac43': 'HangulSyllableGyalb',
u'\u8bc2': 'CjkUnifiedIdeograph8bc2',
u'\u2247': 'NeitherApproximatelyNorActuallyEqualTo',
u'\u01c6': 'LatinSmallLetterDzWithCaron',
u'\ufb4d': 'HebrewLetterKafWithRafe',
u'\udacc': 'UnicodeCharacter0xdacc',
u'\u7151': 'CjkUnifiedIdeograph7151',
u'\u50d0': 'CjkUnifiedIdeograph50d0',
u'\ub453': 'HangulSyllableDugs',
u'\u93d2': 'CjkUnifiedIdeograph93d2',
u'\u0a57': 'UnicodeCharacter0xa57',
u'\ue9d6': 'UnicodeCharacter0xe9d6',
u'\u6fda': 'CjkUnifiedIdeograph6fda',
u'\uc35d': 'HangulSyllableSsyaeb',
u'\ua2dc': 'YiSyllableZzox',
u'\u5961': 'CjkUnifiedIdeograph5961',
u'\u38e0': 'CjkUnifiedIdeograph38e0',
u'\u9c63': 'CjkUnifiedIdeograph9c63',
u'\u1267': 'EthiopicSyllableBwa',
u'\uf1e6': 'UnicodeCharacter0xf1e6',
u'\u686b': 'CjkUnifiedIdeograph686b',
u'\u77ea': 'CjkUnifiedIdeograph77ea',
u'\uab6d': 'UnicodeCharacter0xab6d',
u'\u8aec': 'CjkUnifiedIdeograph8aec',
u'\u2171': 'SmallRomanNumeralTwo',
u'\xf0': 'LatinSmallLetterEth',
u'\ufa77': 'CjkCompatibilityIdeographfa77',
u'\ud9f6': 'UnicodeCharacter0xd9f6',
u'\u707b': 'CjkUnifiedIdeograph707b',
u'\u5ffa': 'CjkUnifiedIdeograph5ffa',
u'\ub37d': 'HangulSyllableDelt',
u'\u92fc': 'CjkUnifiedIdeograph92fc',
u'\u8881': 'CjkUnifiedIdeograph8881',
u'\ue900': 'UnicodeCharacter0xe900',
u'\ucd83': 'HangulSyllableCyolb',
u'\u2a02': 'NaryCircledTimesOperator',
u'\u0e85': 'UnicodeCharacter0xe85',
u'\u6f04': 'CjkUnifiedIdeograph6f04',
u'\u4387': 'CjkUnifiedIdeograph4387',
u'\ubc0e': 'HangulSyllableMij',
u'\u9091': 'CjkUnifiedIdeograph9091',
u'\uf110': 'UnicodeCharacter0xf110',
u'\ud593': 'HangulSyllableHyags',
u'\u3212': 'ParenthesizedHangulMieumA',
u'\u1695': 'OghamLetterEabhadh',
u'\u7714': 'CjkUnifiedIdeograph7714',
u'\u2b97': 'UnicodeCharacter0x2b97',
u'\u6c99': 'CjkUnifiedIdeograph6c99',
u'\u841e': 'CjkUnifiedIdeograph841e',
u'\ud920': 'UnicodeCharacter0xd920',
u'\ubda3': 'HangulSyllableBweod',
u'\u1a22': 'UnicodeCharacter0x1a22',
u'\ufea5': 'ArabicLetterKhahIsolatedForm',
u'\u5f24': 'CjkUnifiedIdeograph5f24',
u'\u33a7': 'SquareMOverS',
u'\u74a9': 'CjkUnifiedIdeograph74a9',
u'\ua130': 'YiSyllableTut',
u'\u85b3': 'CjkUnifiedIdeograph85b3',
u'\ue232': 'UnicodeCharacter0xe232',
u'\uc6b5': 'HangulSyllableUnj',
u'\u2734': 'EightPointedBlackStar',
u'\u1bb7': 'SundaneseDigitSeven',
u'\u7836': 'CjkUnifiedIdeograph7836',
u'\u5cb9': 'CjkUnifiedIdeograph5cb9',
u'\u8940': 'CjkUnifiedIdeograph8940',
u'\uca42': 'HangulSyllableJjyaebs',
u'\uaec5': 'HangulSyllableGgeolg',
u'\u0f44': 'TibetanLetterNga',
u'\ue3c7': 'UnicodeCharacter0xe3c7',
u'\u4046': 'CjkUnifiedIdeograph4046',
u'\u24c9': 'CircledLatinCapitalLetterT',
u'\u79cb': 'CjkUnifiedIdeograph79cb',
u'\u9150': 'CjkUnifiedIdeograph9150',
u'\ud252': 'HangulSyllableTweolm',
u'\ub6d5': 'HangulSyllableDdweg',
u'\u1754': 'UnicodeCharacter0x1754',
u'\ucbd7': 'HangulSyllableJjeugs',
u'\u2856': 'BraillePatternDots2357',
u'\u0cd9': 'UnicodeCharacter0xcd9',
u'\u6d58': 'CjkUnifiedIdeograph6d58',
u'\u41db': 'CjkUnifiedIdeograph41db',
u'\uba62': 'HangulSyllableMelp',
u'\u9ee5': 'CjkUnifiedIdeograph9ee5',
u'\uff64': 'HalfwidthIdeographicComma',
u'\ud3e7': 'HangulSyllablePyec',
u'\u3066': 'HiraganaLetterTe',
u'\u14e9': 'CanadianSyllabicsWestcreeLwaa',
u'\u7568': 'CjkUnifiedIdeograph7568',
u'\u29eb': 'BlackLozenge',
u'\u6aed': 'CjkUnifiedIdeograph6aed',
u'\u8272': 'CjkUnifiedIdeograph8272',
u'\uc774': 'HangulSyllableI',
u'\ubbf7': 'HangulSyllableMyih',
u'\u1876': 'MongolianLetterManchuFa',
u'\ufcf9': 'ArabicLigatureGhainWithAlefMaksuraIsolatedForm',
u'\u5d78': 'CjkUnifiedIdeograph5d78',
u'\u31fb': 'KatakanaLetterSmallRa',
u'\u72fd': 'CjkUnifiedIdeograph72fd',
u'\u4801': 'CjkUnifiedIdeograph4801',
u'\uaf84': 'HangulSyllableGgoen',
u'\ue086': 'UnicodeCharacter0xe086',
u'\u2588': 'FullBlock',
u'\u990b': 'CjkUnifiedIdeograph990b',
u'\u668a': 'CjkUnifiedIdeograph668a',
u'\uda0d': 'UnicodeCharacter0xda0d',
u'\u1f0f': 'GreekCapitalLetterAlphaWithDasiaAndPerispomeni',
u'\u5011': 'CjkUnifiedIdeograph5011',
u'\ub794': 'HangulSyllableRak',
u'\uc896': 'HangulSyllableJwalm',
u'\u0d98': 'UnicodeCharacter0xd98',
u'\u4e9a': 'CjkUnifiedIdeograph4e9a',
u'\ua21d': 'YiSyllableGgap',
u'\ue71f': 'UnicodeCharacter0xe71f',
u'\u3821': 'CjkUnifiedIdeograph3821',
u'\u7d23': 'CjkUnifiedIdeograph7d23',
u'\u9fa4': 'CjkUnifiedIdeograph9fa4',
u'\ud0a6': 'HangulSyllableKigg',
u'\u15a8': 'CanadianSyllabicsThcreeThi',
u'\u56aa': 'CjkUnifiedIdeograph56aa',
u'\u8a2d': 'CjkUnifiedIdeograph8a2d',
u'\u6bac': 'CjkUnifiedIdeograph6bac',
u'\ucf2f': 'HangulSyllableKyeos',
u'\u4533': 'CjkUnifiedIdeograph4533',
u'\ub8b6': 'HangulSyllableRoenh',
u'\ufdb8': 'ArabicLigatureNoonWithJeemWithHahInitialForm',
u'\u3eba': 'CjkUnifiedIdeograph3eba',
u'\u923d': 'CjkUnifiedIdeograph923d',
u'\u73bc': 'CjkUnifiedIdeograph73bc',
u'\ud73f': 'HangulSyllableHyulb',
u'\ue841': 'UnicodeCharacter0xe841',
u'\u2d43': 'TifinaghLetterYahh',
u'\u6e45': 'CjkUnifiedIdeograph6e45',
u'\u80c6': 'CjkUnifiedIdeograph80c6',
u'\uc5c8': 'HangulSyllableEoss',
u'\u06ca': 'ArabicLetterWawWithTwoDotsAbove',
u'\u5bcc': 'CjkUnifiedIdeograph5bcc',
u'\ubf4f': 'HangulSyllableBbolh',
u'\uf051': 'UnicodeCharacter0xf051',
u'\u3553': 'CjkUnifiedIdeograph3553',
u'\u7655': 'CjkUnifiedIdeograph7655',
u'\uadd8': 'HangulSyllableGwik',
u'\ueeda': 'UnicodeCharacter0xeeda',
u'\u23dc': 'TopParenthesis',
u'\u875f': 'CjkUnifiedIdeograph875f',
u'\u64de': 'CjkUnifiedIdeograph64de',
u'\ud861': 'UnicodeCharacter0xd861',
u'\u1d63': 'LatinSubscriptSmallLetterR',
u'\u5e65': 'CjkUnifiedIdeograph5e65',
u'\ub5e8': 'HangulSyllableDdyeom',
u'\uf6ea': 'UnicodeCharacter0xf6ea',
u'\u0bec': 'TamilDigitSix',
u'\u4cee': 'CjkUnifiedIdeograph4cee',
u'\ua071': 'YiSyllableBbur',
u'\ue573': 'UnicodeCharacter0xe573',
u'\u2675': 'RecyclingSymbolForType3Plastics',
u'\u7b77': 'CjkUnifiedIdeograph7b77',
u'\u9df8': 'CjkUnifiedIdeograph9df8',
u'\udefa': 'UnicodeCharacter0xdefa',
u'\u13fc': 'UnicodeCharacter0x13fc',
u'\u54fe': 'CjkUnifiedIdeograph54fe',
u'\u6a00': 'CjkUnifiedIdeograph6a00',
u'\u4e83': 'CjkUnifiedIdeograph4e83',
u'\ua902': 'KayahLiDigitTwo',
u'\u8d85': 'CjkUnifiedIdeograph8d85',
u'\u2306': 'Perspective',
u'\u0789': 'ThaanaLetterMeemu',
u'\uf80c': 'UnicodeCharacter0xf80c',
u'\udc8f': 'UnicodeCharacter0xdc8f',
u'\u7210': 'CjkUnifiedIdeograph7210',
u'\u5693': 'CjkUnifiedIdeograph5693',
u'\ub112': 'HangulSyllableNeolm',
u'\u9595': 'CjkUnifiedIdeograph9595',
u'\u0b16': 'OriyaLetterKha',
u'\uef99': 'UnicodeCharacter0xef99',
u'\u619d': 'CjkUnifiedIdeograph619d',
u'\uc01c': 'HangulSyllableBbwek',
u'\ua49f': 'YiRadicalHxuo',
u'\u5a20': 'CjkUnifiedIdeograph5a20',
u'\u3ea3': 'CjkUnifiedIdeograph3ea3',
u'\u9922': 'CjkUnifiedIdeograph9922',
u'\u1326': 'EthiopicSyllableTho',
u'\uf7a9': 'UnicodeCharacter0xf7a9',
u'\u49ad': 'CjkUnifiedIdeograph49ad',
u'\ua82c': 'UnicodeCharacter0xa82c',
u'\u8caf': 'CjkUnifiedIdeograph8caf',
u'\u2230': 'VolumeIntegral',
u'\u06b3': 'ArabicLetterGueh',
u'\ufb36': 'HebrewLetterZayinWithDagesh',
u'\udfb9': 'UnicodeCharacter0xdfb9',
u'\u7d3a': 'CjkUnifiedIdeograph7d3a',
u'\u51bd': 'CjkUnifiedIdeograph51bd',
u'\ub03c': 'HangulSyllableGgyuss',
u'\u94bf': 'CjkUnifiedIdeograph94bf',
u'\u0a40': 'GurmukhiVowelSignIi',
u'\ueec3': 'UnicodeCharacter0xeec3',
u'\u60c7': 'CjkUnifiedIdeograph60c7',
u'\uc346': 'HangulSyllableSsyaj',
u'\ua7c9': 'UnicodeCharacter0xa7c9',
u'\u454a': 'CjkUnifiedIdeograph454a',
u'\u39cd': 'CjkUnifiedIdeograph39cd',
u'\u984c': 'CjkUnifiedIdeograph984c',
u'\u1250': 'EthiopicSyllableQha',
u'\uf6d3': 'UnicodeCharacter0xf6d3',
u'\u48d7': 'CjkUnifiedIdeograph48d7',
u'\uab56': 'UnicodeCharacter0xab56',
u'\u8fd9': 'CjkUnifiedIdeograph8fd9',
u'\u2d5a': 'TifinaghLetterYass',
u'\u01dd': 'LatinSmallLetterTurnedE',
u'\ufa60': 'CjkCompatibilityIdeographfa60',
u'\udee3': 'UnicodeCharacter0xdee3',
u'\u7c64': 'CjkUnifiedIdeograph7c64',
u'\u50e7': 'CjkUnifiedIdeograph50e7',
u'\ub366': 'HangulSyllableDeobs',
u'\u97e9': 'CjkUnifiedIdeograph97e9',
u'\u356a': 'CjkUnifiedIdeograph356a',
u'\ue9ed': 'UnicodeCharacter0xe9ed',
u'\u63f1': 'CjkUnifiedIdeograph63f1',
u'\uc270': 'HangulSyllableSwin',
u'\ua6f3': 'UnicodeCharacter0xa6f3',
u'\u4474': 'CjkUnifiedIdeograph4474',
u'\u38f7': 'CjkUnifiedIdeograph38f7',
u'\u9b76': 'CjkUnifiedIdeograph9b76',
u'\u1d7a': 'LatinSmallLetterThWithStrikethrough',
u'\uf1fd': 'UnicodeCharacter0xf1fd',
u'\ucb01': 'HangulSyllableJjwaelt',
u'\uaa80': 'UnicodeCharacter0xaa80',
u'\u4d05': 'CjkUnifiedIdeograph4d05',
u'\u2c84': 'CopticCapitalLetterGamma',
u'\u8007': 'CjkUnifiedIdeograph8007',
u'\u1a0b': 'BugineseLetterNra',
u'\ue58a': 'UnicodeCharacter0xe58a',
u'\u7f8e': 'CjkUnifiedIdeograph7f8e',
u'\ud311': 'HangulSyllablePanj',
u'\ub290': 'HangulSyllableNeu',
u'\u5515': 'CjkUnifiedIdeograph5515',
u'\u3494': 'CjkUnifiedIdeograph3494',
u'\ue21b': 'UnicodeCharacter0xe21b',
u'\ucd9a': 'HangulSyllableCunh',
u'\u641f': 'CjkUnifiedIdeograph641f',
u'\u479e': 'CjkUnifiedIdeograph479e',
u'\ubb21': 'HangulSyllableMyolg',
u'\u9aa0': 'CjkUnifiedIdeograph9aa0',
u'\u3d25': 'CjkUnifiedIdeograph3d25',
u'\u1ca4': 'UnicodeCharacter0x1ca4',
u'\uca2b': 'HangulSyllableJjyac',
u'\ud5aa': 'HangulSyllableHyap',
u'\u4c2f': 'CjkUnifiedIdeograph4c2f',
u'\u2fae': 'KangxiRadicalWrong',
u'\u8331': 'CjkUnifiedIdeograph8331',
u'\u0535': 'ArmenianCapitalLetterEch',
u'\ue4b4': 'UnicodeCharacter0xe4b4',
u'\u7eb8': 'CjkUnifiedIdeograph7eb8',
u'\ud23b': 'HangulSyllableTulh',
u'\ubdba': 'HangulSyllableBwegg',
u'\u543f': 'CjkUnifiedIdeograph543f',
u'\u37be': 'CjkUnifiedIdeograph37be',
u'\ued45': 'UnicodeCharacter0xed45',
u'\uccc4': 'HangulSyllableCem',
u'\u6749': 'CjkUnifiedIdeograph6749',
u'\u46c8': 'CjkUnifiedIdeograph46c8',
u'\uba4b': 'HangulSyllableMeos',
u'\u85ca': 'CjkUnifiedIdeograph85ca',
u'\u3c4f': 'CjkUnifiedIdeograph3c4f',
u'\u1fce': 'GreekPsiliAndOxia',
u'\uf555': 'UnicodeCharacter0xf555',
u'\ud4d4': 'HangulSyllablePwil',
u'\u4f59': 'CjkUnifiedIdeograph4f59',
u'\u2ed8': 'CjkRadicalBlue',
u'\u825b': 'CjkUnifiedIdeograph825b',
u'\u045f': 'CyrillicSmallLetterDzhe',
u'\ue7de': 'UnicodeCharacter0xe7de',
u'\u79e2': 'CjkUnifiedIdeograph79e2',
u'\udd65': 'UnicodeCharacter0xdd65',
u'\ubce4': 'HangulSyllableByels',
u'\u5769': 'CjkUnifiedIdeograph5769',
u'\u36e8': 'CjkUnifiedIdeograph36e8',
u'\uec6f': 'UnicodeCharacter0xec6f',
u'\ucfee': 'HangulSyllableKulp',
u'\u6673': 'CjkUnifiedIdeograph6673',
u'\u41f2': 'CjkUnifiedIdeograph41f2',
u'\ua575': 'VaiSyllableWoon',
u'\u84f4': 'CjkUnifiedIdeograph84f4',
u'\u3f79': 'CjkUnifiedIdeograph3f79',
u'\u1ef8': 'LatinCapitalLetterYWithTilde',
u'\uf47f': 'UnicodeCharacter0xf47f',
u'\ud7fe': 'UnicodeCharacter0xd7fe',
u'\ua206': 'YiSyllableKot',
u'\u8689': 'CjkUnifiedIdeograph8689',
u'\ue708': 'UnicodeCharacter0xe708',
u'\udb8b': 'UnicodeCharacter0xdb8b',
u'\u240a': 'SymbolForLineFeed',
u'\u188d': 'MongolianLetterAliGaliTtha',
u'\u790c': 'CjkUnifiedIdeograph790c',
u'\u5d8f': 'CjkUnifiedIdeograph5d8f',
u'\u8a16': 'CjkUnifiedIdeograph8a16',
u'\ucf18': 'HangulSyllableKek',
u'\ua39b': 'YiSyllableNret',
u'\u0c1a': 'TeluguLetterCa',
u'\ue09d': 'UnicodeCharacter0xe09d',
u'\u411c': 'CjkUnifiedIdeograph411c',
u'\u259f': 'QuadrantUpperRightAndLowerLeftAndLowerRight',
u'\u7aa1': 'CjkUnifiedIdeograph7aa1',
u'\u9226': 'CjkUnifiedIdeograph9226',
u'\ud728': 'HangulSyllableHwim',
u'\u8bab': 'CjkUnifiedIdeograph8bab',
u'\u142a': 'CanadianSyllabicsFinalDownTack',
u'\uc8ad': 'HangulSyllableJwaenj',
u'\u292c': 'FallingDiagonalCrossingRisingDiagonal',
u'\u0daf': 'SinhalaLetterAlpapraanaDayanna',
u'\u6e2e': 'CjkUnifiedIdeograph6e2e',
u'\u42b1': 'CjkUnifiedIdeograph42b1',
u'\ubf38': 'HangulSyllableBbyess',
u'\u93bb': 'CjkUnifiedIdeograph93bb',
u'\ufc3a': 'ArabicLigatureKafWithKhahIsolatedForm',
u'\ud0bd': 'HangulSyllableKit',
u'\u313c': 'HangulLetterRieulpieup',
u'\u15bf': 'CanadianSyllabicsBlackfootKa',
u'\u763e': 'CjkUnifiedIdeograph763e',
u'\u2ac1': 'SubsetWithMultiplicationSignBelow',
u'\u6fc3': 'CjkUnifiedIdeograph6fc3',
u'\u8748': 'CjkUnifiedIdeograph8748',
u'\uc44a': 'HangulSyllableSsyogg',
u'\ub8cd': 'HangulSyllableRyog',
u'\u194c': 'LimbuDigitSix',
u'\ufdcf': 'UnicodeCharacter0xfdcf',
u'\u5e4e': 'CjkUnifiedIdeograph5e4e',
u'\u32d1': 'CircledKatakanaI',
u'\u77d3': 'CjkUnifiedIdeograph77d3',
u'\uac5a': 'HangulSyllableGyaenh',
u'\u80dd': 'CjkUnifiedIdeograph80dd',
u'\ue15c': 'UnicodeCharacter0xe15c',
u'\uc5df': 'HangulSyllableElh',
u'\u265e': 'BlackChessKnight',
u'\u1ae1': 'UnicodeCharacter0x1ae1',
u'\u7b60': 'CjkUnifiedIdeograph7b60',
u'\u5fe3': 'CjkUnifiedIdeograph5fe3',
u'\ub46a': 'HangulSyllableDup',
u'\uc96c': 'HangulSyllableJyu',
u'\uadef': 'HangulSyllableGyus',
u'\u0e6e': 'UnicodeCharacter0xe6e',
u'\ue2f1': 'UnicodeCharacter0xe2f1',
u'\u4370': 'CjkUnifiedIdeograph4370',
u'\u27f3': 'ClockwiseGappedCircleArrow',
u'\u64f5': 'CjkUnifiedIdeograph64f5',
u'\u9c7a': 'CjkUnifiedIdeograph9c7a',
u'\ud17c': 'HangulSyllableTyeoss',
u'\ub5ff': 'HangulSyllableDdyelb',
u'\u167e': 'UnicodeCharacter0x167e',
u'\u2b80': 'UnicodeCharacter0x2b80',
u'\u8f03': 'CjkUnifiedIdeograph8f03',
u'\u6882': 'CjkUnifiedIdeograph6882',
u'\ucc05': 'HangulSyllableJjying',
u'\u0107': 'LatinSmallLetterCWithAcute',
u'\u4609': 'CjkUnifiedIdeograph4609',
u'\ub98c': 'HangulSyllableReuk',
u'\ufe8e': 'ArabicLetterAlefFinalForm',
u'\u3390': 'SquareHz',
u'\u9713': 'CjkUnifiedIdeograph9713',
u'\u7092': 'CjkUnifiedIdeograph7092',
u'\ud415': 'HangulSyllablePwalt',
u'\ue917': 'UnicodeCharacter0xe917',
u'\u2e19': 'PalmBranch',
u'\u631b': 'CjkUnifiedIdeograph631b',
u'\u819c': 'CjkUnifiedIdeograph819c',
u'\uc69e': 'HangulSyllableYolm',
u'\u1ba0': 'SundaneseLetterHa',
u'\u58a2': 'CjkUnifiedIdeograph58a2',
u'\ubc25': 'HangulSyllableBab',
u'\uf127': 'UnicodeCharacter0xf127',
u'\u3629': 'CjkUnifiedIdeograph3629',
u'\u4b2b': 'CjkUnifiedIdeograph4b2b',
u'\uaeae': 'HangulSyllableGgyaelp',
u'\ue3b0': 'UnicodeCharacter0xe3b0',
u'\u20b2': 'GuaraniSign',
u'\u8435': 'CjkUnifiedIdeograph8435',
u'\u65b4': 'CjkUnifiedIdeograph65b4',
u'\ud937': 'UnicodeCharacter0xd937',
u'\u1e39': 'LatinSmallLetterLWithDotBelowAndMacron',
u'\u533b': 'CjkUnifiedIdeograph533b',
u'\ub6be': 'HangulSyllableDdweonh',
u'\ucbc0': 'HangulSyllableJjyul',
u'\u08c2': 'UnicodeCharacter0x8c2',
u'\u4dc4': 'HexagramForWaiting',
u'\ua147': 'YiSyllableDdop',
u'\ue649': 'UnicodeCharacter0xe649',
u'\u3b4b': 'CjkUnifiedIdeograph3b4b',
u'\u784d': 'CjkUnifiedIdeograph784d',
u'\u9ece': 'CjkUnifiedIdeograph9ece',
u'\ud3d0': 'HangulSyllablePye',
u'\u10d2': 'GeorgianLetterGan',
u'\u55d4': 'CjkUnifiedIdeograph55d4',
u'\u8957': 'CjkUnifiedIdeograph8957',
u'\u6ad6': 'CjkUnifiedIdeograph6ad6',
u'\uce59': 'HangulSyllableCig',
u'\u035b': 'CombiningZigzagAbove',
u'\u405d': 'CjkUnifiedIdeograph405d',
u'\ubbe0': 'HangulSyllableMyin',
u'\uf8e2': 'UnicodeCharacter0xf8e2',
u'\u3de4': 'CjkUnifiedIdeograph3de4',
u'\u9167': 'CjkUnifiedIdeograph9167',
u'\u72e6': 'CjkUnifiedIdeograph72e6',
u'\ud669': 'HangulSyllableHwang',
u'\ueb6b': 'UnicodeCharacter0xeb6b',
u'\u286d': 'BraillePatternDots13467',
u'\u6d6f': 'CjkUnifiedIdeograph6d6f',
u'\u83f0': 'CjkUnifiedIdeograph83f0',
u'\uc0f2': 'HangulSyllableSyalp',
u'\u05f4': 'HebrewPunctuationGershayim',
u'\u5af6': 'CjkUnifiedIdeograph5af6',
u'\ube79': 'HangulSyllableBbat',
u'\uf37b': 'UnicodeCharacter0xf37b',
u'\u307d': 'HiraganaLetterPo',
u'\u757f': 'CjkUnifiedIdeograph757f',
u'\u0d81': 'UnicodeCharacter0xd81',
u'\uea04': 'UnicodeCharacter0xea04',
u'\uc687': 'HangulSyllableOelh',
u'\u6008': 'CjkUnifiedIdeograph6008',
u'\u5c8b': 'CjkUnifiedIdeograph5c8b',
u'\ua30a': 'YiSyllableSix',
u'\u9f8d': 'CjkUnifiedIdeograph9f8d',
u'\u390e': 'CjkUnifiedIdeograph390e',
u'\u1591': 'CanadianSyllabicsNgo',
u'\uf214': 'UnicodeCharacter0xf214',
u'\uae97': 'HangulSyllableGgyas',
u'\u4818': 'CjkUnifiedIdeograph4818',
u'\u249b': 'NumberTwentyFullStop',
u'\u8b1a': 'CjkUnifiedIdeograph8b1a',
u'\u011e': 'LatinCapitalLetterGWithBreve',
u'\ufda1': 'ArabicLigatureTehWithKhahWithYehFinalForm',
u'\u7ba5': 'CjkUnifiedIdeograph7ba5',
u'\uda24': 'UnicodeCharacter0xda24',
u'\ub6a7': 'HangulSyllableDdulb',
u'\u5028': 'CjkUnifiedIdeograph5028',
u'\u0cab': 'KannadaLetterPha',
u'\u932a': 'CjkUnifiedIdeograph932a',
u'\ue92e': 'UnicodeCharacter0xe92e',
u'\uc5b1': 'HangulSyllableYaet',
u'\u6732': 'CjkUnifiedIdeograph6732',
u'\u43b5': 'CjkUnifiedIdeograph43b5',
u'\ua234': 'YiSyllableMga',
u'\u9eb7': 'CjkUnifiedIdeograph9eb7',
u'\u3838': 'CjkUnifiedIdeograph3838',
u'\u14bb': 'CanadianSyllabicsM',
u'\uf13e': 'UnicodeCharacter0xf13e',
u'\uadc1': 'HangulSyllableGwig',
u'\u4f42': 'CjkUnifiedIdeograph4f42',
u'\u2bc5': 'UnicodeCharacter0x2bc5',
u'\u8a44': 'CjkUnifiedIdeograph8a44',
u'\ufccb': 'ArabicLigatureLamWithKhahInitialForm',
u'\u7acf': 'CjkUnifiedIdeograph7acf',
u'\ud94e': 'UnicodeCharacter0xd94e',
u'\ub5d1': 'HangulSyllableDdeng',
u'\u5752': 'CjkUnifiedIdeograph5752',
u'\u33d5': 'SquareMil',
u'\u9254': 'CjkUnifiedIdeograph9254',
u'\ue858': 'UnicodeCharacter0xe858',
u'\uc4db': 'HangulSyllableSsyud',
u'\u665c': 'CjkUnifiedIdeograph665c',
u'\u42df': 'CjkUnifiedIdeograph42df',
u'\ua15e': 'YiSyllableNdop',
u'\u9de1': 'CjkUnifiedIdeograph9de1',
u'\u3f62': 'CjkUnifiedIdeograph3f62',
u'\u1be5': 'UnicodeCharacter0x1be5',
u'\uf068': 'UnicodeCharacter0xf068',
u'\uaceb': 'HangulSyllableGolb',
u'\u4e6c': 'CjkUnifiedIdeograph4e6c',
u'\u2aef': 'VerticalLineWithCircleAbove',
u'\u896e': 'CjkUnifiedIdeograph896e',
u'\u0772': 'ArabicLetterHahWithSmallArabicLetterTahAbove',
u'\ue3f5': 'UnicodeCharacter0xe3f5',
u'\u79f9': 'CjkUnifiedIdeograph79f9',
u'\ud878': 'UnicodeCharacter0xd878',
u'\ub4fb': 'HangulSyllableDyigs',
u'\u567c': 'CjkUnifiedIdeograph567c',
u'\u32ff': 'UnicodeCharacter0x32ff',
u'\u917e': 'CjkUnifiedIdeograph917e',
u'\u0803': 'UnicodeCharacter0x803',
u'\uef82': 'UnicodeCharacter0xef82',
u'\u6586': 'CjkUnifiedIdeograph6586',
u'\uc109': 'HangulSyllableSyaelg',
u'\ua088': 'YiSyllableNbup',
u'\u5f0d': 'CjkUnifiedIdeograph5f0d',
u'\u3e8c': 'CjkUnifiedIdeograph3e8c',
u'\u9a0f': 'CjkUnifiedIdeograph9a0f',
u'\u1013': 'MyanmarLetterDha',
u'\uf792': 'UnicodeCharacter0xf792',
u'\u6e17': 'CjkUnifiedIdeograph6e17',
u'\u4d96': 'CjkUnifiedIdeograph4d96',
u'\ua919': 'KayahLiLetterBa',
u'\u8898': 'CjkUnifiedIdeograph8898',
u'\u271d': 'LatinCross',
u'\u069c': 'ArabicLetterSeenWithThreeDotsBelowAndThreeDotsAbove',
u'\uf823': 'UnicodeCharacter0xf823',
u'\udfa2': 'UnicodeCharacter0xdfa2',
u'\u7627': 'CjkUnifiedIdeograph7627',
u'\u55a6': 'CjkUnifiedIdeograph55a6',
u'\ub129': 'HangulSyllableNenj',
u'\u90a8': 'CjkUnifiedIdeograph90a8',
u'\u0f2d': 'TibetanDigitHalfFour',
u'\ueeac': 'UnicodeCharacter0xeeac',
u'\u64b0': 'CjkUnifiedIdeograph64b0',
u'\uc033': 'HangulSyllableBbwis',
u'\ua7b2': 'UnicodeCharacter0xa7b2',
u'\u5e37': 'CjkUnifiedIdeograph5e37',
u'\u3db6': 'CjkUnifiedIdeograph3db6',
u'\u9939': 'CjkUnifiedIdeograph9939',
u'\u173d': 'UnicodeCharacter0x173d',
u'\uf6bc': 'UnicodeCharacter0xf6bc',
u'\u6d41': 'CjkUnifiedIdeograph6d41',
u'\u4cc0': 'CjkUnifiedIdeograph4cc0',
u'\ua843': 'PhagspaLetterNga',
u'\u8fc2': 'CjkUnifiedIdeograph8fc2',
u'\u2647': 'Pluto',
u'\u05c6': 'HebrewPunctuationNunHafukha',
u'\uff4d': 'FullwidthLatinSmallLetterM',
u'\udecc': 'UnicodeCharacter0xdecc',
u'\u7551': 'CjkUnifiedIdeograph7551',
u'\u54d0': 'CjkUnifiedIdeograph54d0',
u'\ub053': 'HangulSyllableGgeulh',
u'\u97d2': 'CjkUnifiedIdeograph97d2',
u'\u0e57': 'ThaiDigitSeven',
u'\uedd6': 'UnicodeCharacter0xedd6',
u'\u6bda': 'CjkUnifiedIdeograph6bda',
u'\uc75d': 'HangulSyllableYinj',
u'\ua6dc': 'UnicodeCharacter0xa6dc',
u'\u5d61': 'CjkUnifiedIdeograph5d61',
u'\u3ce0': 'CjkUnifiedIdeograph3ce0',
u'\u9863': 'CjkUnifiedIdeograph9863',
u'\u1667': 'CanadianSyllabicsCarrierTtsu',
u'\uf5e6': 'UnicodeCharacter0xf5e6',
u'\u6c6b': 'CjkUnifiedIdeograph6c6b',
u'\u73ea': 'CjkUnifiedIdeograph73ea',
u'\uaf6d': 'HangulSyllableGgwaelg',
u'\u8eec': 'CjkUnifiedIdeograph8eec',
u'\u2571': 'BoxDrawingsLightDiagonalUpperRightToLowerLeft',
u'\u04f0': 'CyrillicCapitalLetterUWithDiaeresis',
u'\ufe77': 'ArabicFathaMedialForm',
u'\uddf6': 'UnicodeCharacter0xddf6',
u'\u747b': 'CjkUnifiedIdeograph747b',
u'\u5bfa': 'CjkUnifiedIdeograph5bfa',
u'\ub77d': 'HangulSyllableRag',
u'\u96fc': 'CjkUnifiedIdeograph96fc',
u'\u8c81': 'CjkUnifiedIdeograph8c81',
u'\ued00': 'UnicodeCharacter0xed00',
u'\uc983': 'HangulSyllableJyuc',
u'\u2e02': 'LeftSubstitutionBracket',
u'\u0a85': 'GujaratiLetterA',
u'\u6b04': 'CjkUnifiedIdeograph6b04',
u'\u4787': 'CjkUnifiedIdeograph4787',
u'\ub80e': 'HangulSyllableRenh',
u'\u9491': 'CjkUnifiedIdeograph9491',
u'\uf510': 'UnicodeCharacter0xf510',
u'\ud193': 'HangulSyllableTyelh',
u'\u3612': 'CjkUnifiedIdeograph3612',
u'\u1295': 'EthiopicSyllableNe',
u'\u7314': 'CjkUnifiedIdeograph7314',
u'\u2f97': 'KangxiRadicalPig',
u'\u6899': 'CjkUnifiedIdeograph6899',
u'\u801e': 'CjkUnifiedIdeograph801e',
u'\udd20': 'UnicodeCharacter0xdd20',
u'\ub9a3': 'HangulSyllableRyis',
u'\u1e22': 'LatinCapitalLetterHWithDotAbove',
u'\ufaa5': 'CjkCompatibilityIdeographfaa5',
u'\u5b24': 'CjkUnifiedIdeograph5b24',
u'\u37a7': 'CjkUnifiedIdeograph37a7',
u'\u70a9': 'CjkUnifiedIdeograph70a9',
u'\ua530': 'VaiSyllableGbi',
u'\u81b3': 'CjkUnifiedIdeograph81b3',
u'\ue632': 'UnicodeCharacter0xe632',
u'\uc2b5': 'HangulSyllableSeub',
u'\u2334': 'Counterbore',
u'\u1fb7': 'GreekSmallLetterAlphaWithPerispomeniAndYpogegrammeni',
u'\u7c36': 'CjkUnifiedIdeograph7c36',
u'\u58b9': 'CjkUnifiedIdeograph58b9',
u'\u8d40': 'CjkUnifiedIdeograph8d40',
u'\uce42': 'HangulSyllableCyinh',
u'\uaac5': 'UnicodeCharacter0xaac5',
u'\u0b44': 'OriyaVowelSignVocalicRr',
u'\ue7c7': 'UnicodeCharacter0xe7c7',
u'\u4446': 'CjkUnifiedIdeograph4446',
u'\u20c9': 'UnicodeCharacter0x20c9',
u'\u7dcb': 'CjkUnifiedIdeograph7dcb',
u'\u9550': 'CjkUnifiedIdeograph9550',
u'\ud652': 'HangulSyllableHop',
u'\ub2d5': 'HangulSyllableNilt',
u'\u1354': 'EthiopicSyllablePee',
u'\ucfd7': 'HangulSyllableKyos',
u'\u2c56': 'GlagoliticSmallLetterYo',
u'\u08d9': 'UnicodeCharacter0x8d9',
u'\u6958': 'CjkUnifiedIdeograph6958',
u'\u45db': 'CjkUnifiedIdeograph45db',
u'\ube62': 'HangulSyllableBbagg',
u'\u9ae5': 'CjkUnifiedIdeograph9ae5',
u'\ufb64': 'ArabicLetterTehehInitialForm',
u'\ud7e7': 'UnicodeCharacter0xd7e7',
u'\u3466': 'CjkUnifiedIdeograph3466',
u'\u10e9': 'GeorgianLetterChin',
u'\u7168': 'CjkUnifiedIdeograph7168',
u'\u2deb': 'CombiningCyrillicLetterPe',
u'\u6eed': 'CjkUnifiedIdeograph6eed',
u'\u8672': 'CjkUnifiedIdeograph8672',
u'\uc374': 'HangulSyllableSseols',
u'\ubff7': 'HangulSyllableBbweolh',
u'\u1c76': 'OlChikiLetterOv',
u'\uf8f9': 'UnicodeCharacter0xf8f9',
u'\u5978': 'CjkUnifiedIdeograph5978',
u'\u35fb': 'CjkUnifiedIdeograph35fb',
u'\u76fd': 'CjkUnifiedIdeograph76fd',
u'\u4c01': 'CjkUnifiedIdeograph4c01',
u'\uab84': 'UnicodeCharacter0xab84',
u'\ue486': 'UnicodeCharacter0xe486',
u'\u2188': 'RomanNumeralOneHundredThousand',
u'\u9d0b': 'CjkUnifiedIdeograph9d0b',
u'\u628a': 'CjkUnifiedIdeograph628a',
u'\ude0d': 'UnicodeCharacter0xde0d',
u'\u1b0f': 'BalineseLetterEkara',
u'\u5411': 'CjkUnifiedIdeograph5411',
u'\ub394': 'HangulSyllableDyeol',
u'\ucc96': 'HangulSyllableCyaep',
u'\u0998': 'BengaliLetterGha',
u'\u4a9a': 'CjkUnifiedIdeograph4a9a',
u'\ua61d': 'VaiSymbolTong',
u'\ue31f': 'UnicodeCharacter0xe31f',
u'\u3c21': 'CjkUnifiedIdeograph3c21',
u'\u7923': 'CjkUnifiedIdeograph7923',
u'\u9ba4': 'CjkUnifiedIdeograph9ba4',
u'\ud4a6': 'HangulSyllablePweobs',
u'\u11a8': 'HangulJongseongKiyeok',
u'\u52aa': 'CjkUnifiedIdeograph52aa',
u'\u8e2d': 'CjkUnifiedIdeograph8e2d',
u'\u6fac': 'CjkUnifiedIdeograph6fac',
u'\ucb2f': 'HangulSyllableJjyogs',
u'\u0431': 'CyrillicSmallLetterBe',
u'\u4133': 'CjkUnifiedIdeograph4133',
u'\ubcb6': 'HangulSyllableBej',
u'\uf9b8': 'CjkCompatibilityIdeographf9b8',
u'\u3aba': 'CjkUnifiedIdeograph3aba',
u'\u963d': 'CjkUnifiedIdeograph963d',
u'\u77bc': 'CjkUnifiedIdeograph77bc',
u'\ud33f': 'HangulSyllablePaec',
u'\uec41': 'UnicodeCharacter0xec41',
u'\u2943': 'LeftwardsArrowAboveShortRightwardsArrow',
u'\u6a45': 'CjkUnifiedIdeograph6a45',
u'\u84c6': 'CjkUnifiedIdeograph84c6',
u'\uc1c8': 'HangulSyllableSwaen',
u'\u02ca': 'ModifierLetterAcuteAccent',
u'\u5fcc': 'CjkUnifiedIdeograph5fcc',
u'\ubb4f': 'HangulSyllableMuh',
u'\uf451': 'UnicodeCharacter0xf451',
u'\u3153': 'HangulLetterEo',
u'\u7255': 'CjkUnifiedIdeograph7255',
u'\ua9d8': 'UnicodeCharacter0xa9d8',
u'\ueada': 'UnicodeCharacter0xeada',
u'\u27dc': 'LeftMultimap',
u'\u835f': 'CjkUnifiedIdeograph835f',
u'\u60de': 'CjkUnifiedIdeograph60de',
u'\udc61': 'UnicodeCharacter0xdc61',
u'\u1963': 'TaiLeLetterA',
u'\u5a65': 'CjkUnifiedIdeograph5a65',
u'\ub1e8': 'HangulSyllableNyo',
u'\uf2ea': 'UnicodeCharacter0xf2ea',
u'\u0fec': 'UnicodeCharacter0xfec',
u'\u48ee': 'CjkUnifiedIdeograph48ee',
u'\ua471': 'YiSyllableYit',
u'\ue173': 'UnicodeCharacter0xe173',
u'\u2275': 'NeitherGreaterthanNorEquivalentTo',
u'\u7f77': 'CjkUnifiedIdeograph7f77',
u'\u99f8': 'CjkUnifiedIdeograph99f8',
u'\udafa': 'UnicodeCharacter0xdafa',
u'\u17fc': 'UnicodeCharacter0x17fc',
u'\u50fe': 'CjkUnifiedIdeograph50fe',
u'\u6e00': 'CjkUnifiedIdeograph6e00',
u'\u4a83': 'CjkUnifiedIdeograph4a83',
u'\uad02': 'HangulSyllableGwanh',
u'\u8985': 'CjkUnifiedIdeograph8985',
u'\u2706': 'TelephoneLocationSign',
u'\u0389': 'GreekCapitalLetterEtaWithTonos',
u'\ufc0c': 'ArabicLigatureTehWithHahIsolatedForm',
u'\ud88f': 'UnicodeCharacter0xd88f',
u'\u7610': 'CjkUnifiedIdeograph7610',
u'\u5293': 'CjkUnifiedIdeograph5293',
u'\ub512': 'HangulSyllableDyip',
u'\u9195': 'CjkUnifiedIdeograph9195',
u'\u0f16': 'TibetanLogotypeSignLhagRtags',
u'\ueb99': 'UnicodeCharacter0xeb99',
u'\u659d': 'CjkUnifiedIdeograph659d',
u'\uc41c': 'HangulSyllableSswaels',
u'\ua09f': 'YiSyllableHmot',
u'\u5e20': 'CjkUnifiedIdeograph5e20',
u'\u3aa3': 'CjkUnifiedIdeograph3aa3',
u'\u9d22': 'CjkUnifiedIdeograph9d22',
u'\u1726': 'HanunooLetterTa',
u'\uf3a9': 'UnicodeCharacter0xf3a9',
u'\u4dad': 'CjkUnifiedIdeograph4dad',
u'\uac2c': 'HangulSyllableGaem',
u'\u88af': 'CjkUnifiedIdeograph88af',
u'\u2630': 'TrigramForHeaven',
u'\u02b3': 'ModifierLetterSmallR',
u'\uff36': 'FullwidthLatinCapitalLetterV',
u'\udbb9': 'UnicodeCharacter0xdbb9',
u'\u793a': 'CjkUnifiedIdeograph793a',
u'\u55bd': 'CjkUnifiedIdeograph55bd',
u'\ub43c': 'HangulSyllableDyol',
u'\u90bf': 'CjkUnifiedIdeograph90bf',
u'\u0e40': 'ThaiCharacterSaraE',
u'\ueac3': 'UnicodeCharacter0xeac3',
u'\u64c7': 'CjkUnifiedIdeograph64c7',
u'\uc746': 'HangulSyllableEulm',
u'\ua3c9': 'YiSyllableRap',
u'\u414a': 'CjkUnifiedIdeograph414a',
u'\u3dcd': 'CjkUnifiedIdeograph3dcd',
u'\u9c4c': 'CjkUnifiedIdeograph9c4c',
u'\u1650': 'CanadianSyllabicsCarrierSe',
u'\uf2d3': 'UnicodeCharacter0xf2d3',
u'\u4cd7': 'CjkUnifiedIdeograph4cd7',
u'\uaf56': 'HangulSyllableGgwalp',
u'\u8bd9': 'CjkUnifiedIdeograph8bd9',
u'\u295a': 'LeftwardsHarpoonWithBarbUpFromBar',
u'\u05dd': 'HebrewLetterFinalMem',
u'\ufe60': 'SmallAmpersand',
u'\udae3': 'UnicodeCharacter0xdae3',
u'\u7864': 'CjkUnifiedIdeograph7864',
u'\u54e7': 'CjkUnifiedIdeograph54e7',
u'\ub766': 'HangulSyllableDdinh',
u'\u93e9': 'CjkUnifiedIdeograph93e9',
u'\u316a': 'HangulLetterRieultikeut',
u'\ueded': 'UnicodeCharacter0xeded',
u'\u67f1': 'CjkUnifiedIdeograph67f1',
u'\uc670': 'HangulSyllableWaess',
u'\ua2f3': 'YiSyllableNziep',
u'\u4074': 'CjkUnifiedIdeograph4074',
u'\u3cf7': 'CjkUnifiedIdeograph3cf7',
u'\u9f76': 'CjkUnifiedIdeograph9f76',
u'\u197a': 'UnicodeCharacter0x197a',
u'\uf5fd': 'UnicodeCharacter0xf5fd',
u'\ucf01': 'HangulSyllableKeg',
u'\uae80': 'HangulSyllableGgaek',
u'\u4905': 'CjkUnifiedIdeograph4905',
u'\u2884': 'BraillePatternDots38',
u'\u8407': 'CjkUnifiedIdeograph8407',
u'\u1e0b': 'LatinSmallLetterDWithDotAbove',
u'\ue18a': 'UnicodeCharacter0xe18a',
u'\u7b8e': 'CjkUnifiedIdeograph7b8e',
u'\ud711': 'HangulSyllableHweng',
u'\ub690': 'HangulSyllableDdyom',
u'\u5115': 'CjkUnifiedIdeograph5115',
u'\u3094': 'HiraganaLetterVu',
u'\ue61b': 'UnicodeCharacter0xe61b',
u'\uc99a': 'HangulSyllableJeubs',
u'\u601f': 'CjkUnifiedIdeograph601f',
u'\u439e': 'CjkUnifiedIdeograph439e',
u'\ubf21': 'HangulSyllableBbyeot',
u'\u9ea0': 'CjkUnifiedIdeograph9ea0',
u'\u3925': 'CjkUnifiedIdeograph3925',
u'\u18a4': 'MongolianLetterManchuAliGaliZha',
u'\uce2b': 'HangulSyllableCeulb',
u'\ud1aa': 'HangulSyllableTolm',
u'\u482f': 'CjkUnifiedIdeograph482f',
u'\u2bae': 'UnicodeCharacter0x2bae',
u'\u8731': 'CjkUnifiedIdeograph8731',
u'\u0135': 'LatinSmallLetterJWithCircumflex',
u'\ue0b4': 'UnicodeCharacter0xe0b4',
u'\u7ab8': 'CjkUnifiedIdeograph7ab8',
u'\ud63b': 'HangulSyllableHogs',
u'\ub9ba': 'HangulSyllableRilp',
u'\u503f': 'CjkUnifiedIdeograph503f',
u'\u33be': 'SquareKw',
u'\ue945': 'UnicodeCharacter0xe945',
u'\uc8c4': 'HangulSyllableJoe',
u'\u6349': 'CjkUnifiedIdeograph6349',
u'\u42c8': 'CjkUnifiedIdeograph42c8',
u'\ube4b': 'HangulSyllableBid',
u'\u81ca': 'CjkUnifiedIdeograph81ca',
u'\u384f': 'CjkUnifiedIdeograph384f',
u'\u1bce': 'UnicodeCharacter0x1bce',
u'\uf155': 'UnicodeCharacter0xf155',
u'\ud0d4': 'HangulSyllableTass',
u'\u4b59': 'CjkUnifiedIdeograph4b59',
u'\u2ad8': 'SupersetBesideAndJoinedByDashWithSubset',
u'\u865b': 'CjkUnifiedIdeograph865b',
u'\ue3de': 'UnicodeCharacter0xe3de',
u'\u7de2': 'CjkUnifiedIdeograph7de2',
u'\ud965': 'UnicodeCharacter0xd965',
u'\ub8e4': 'HangulSyllableRyok',
u'\u5369': 'CjkUnifiedIdeograph5369',
u'\u32e8': 'CircledKatakanaNo',
u'\ue86f': 'UnicodeCharacter0xe86f',
u'\ucbee': 'HangulSyllableJjeup',
u'\u6273': 'CjkUnifiedIdeograph6273',
u'\u45f2': 'CjkUnifiedIdeograph45f2',
u'\ua175': 'YiSyllableHnuo',
u'\u80f4': 'CjkUnifiedIdeograph80f4',
u'\u3b79': 'CjkUnifiedIdeograph3b79',
u'\u1af8': 'UnicodeCharacter0x1af8',
u'\uf07f': 'UnicodeCharacter0xf07f',
u'\ud3fe': 'HangulSyllablePobs',
u'\ua606': 'VaiSyllableGe',
u'\u8289': 'CjkUnifiedIdeograph8289',
u'\ue308': 'UnicodeCharacter0xe308',
u'\udf8b': 'UnicodeCharacter0xdf8b',
u'\u200a': 'HairSpace',
u'\u1c8d': 'UnicodeCharacter0x1c8d',
u'\u7d0c': 'CjkUnifiedIdeograph7d0c',
u'\u598f': 'CjkUnifiedIdeograph598f',
u'\u8e16': 'CjkUnifiedIdeograph8e16',
u'\ucb18': 'HangulSyllableJjoel',
u'\ua79b': 'UnicodeCharacter0xa79b',
u'\u081a': 'UnicodeCharacter0x81a',
u'\ue49d': 'UnicodeCharacter0xe49d',
u'\u451c': 'CjkUnifiedIdeograph451c',
u'\u219f': 'UpwardsTwoHeadedArrow',
u'\u7ea1': 'CjkUnifiedIdeograph7ea1',
u'\u9626': 'CjkUnifiedIdeograph9626',
u'\ud328': 'HangulSyllablePae',
u'\u8fab': 'CjkUnifiedIdeograph8fab',
u'\u102a': 'MyanmarLetterAu',
u'\uccad': 'HangulSyllableCeong',
u'\u2d2c': 'UnicodeCharacter0x2d2c',
u'\u09af': 'BengaliLetterYa',
u'\u6a2e': 'CjkUnifiedIdeograph6a2e',
u'\u46b1': 'CjkUnifiedIdeograph46b1',
u'\ubb38': 'HangulSyllableMun',
u'\u97bb': 'CjkUnifiedIdeograph97bb',
u'\uf83a': 'UnicodeCharacter0xf83a',
u'\ud4bd': 'HangulSyllablePwelt',
u'\u353c': 'CjkUnifiedIdeograph353c',
u'\u11bf': 'HangulJongseongKhieukh',
u'\u723e': 'CjkUnifiedIdeograph723e',
u'\u2ec1': 'CjkRadicalTiger',
u'\u6bc3': 'CjkUnifiedIdeograph6bc3',
u'\u8348': 'CjkUnifiedIdeograph8348',
u'\uc04a': 'HangulSyllableBbyulp',
u'\ubccd': 'HangulSyllableByeob',
u'\u1d4c': 'ModifierLetterSmallTurnedOpenE',
u'\uf9cf': 'CjkCompatibilityIdeographf9cf',
u'\u5a4e': 'CjkUnifiedIdeograph5a4e',
u'\u36d1': 'CjkUnifiedIdeograph36d1',
u'\u73d3': 'CjkUnifiedIdeograph73d3',
u'\ua85a': 'PhagspaLetterSha',
u'\u84dd': 'CjkUnifiedIdeograph84dd',
u'\ue55c': 'UnicodeCharacter0xe55c',
u'\uc1df': 'HangulSyllableSwaeh',
u'\u225e': 'MeasuredBy',
u'\u1ee1': 'LatinSmallLetterOWithHornAndTilde',
u'\u7f60': 'CjkUnifiedIdeograph7f60',
u'\u5be3': 'CjkUnifiedIdeograph5be3',
u'\ub06a': 'HangulSyllableGgyilm',
u'\ucd6c': 'HangulSyllableCoem',
u'\ua9ef': 'UnicodeCharacter0xa9ef',
u'\u0a6e': 'GurmukhiDigitEight',
u'\ue6f1': 'UnicodeCharacter0xe6f1',
u'\u4770': 'CjkUnifiedIdeograph4770',
u'\u23f3': 'UnicodeCharacter0x23f3',
u'\u60f5': 'CjkUnifiedIdeograph60f5',
u'\u987a': 'CjkUnifiedIdeograph987a',
u'\ud57c': 'HangulSyllableHael',
u'\ub1ff': 'HangulSyllableNyoc',
u'\u127e': 'EthiopicSyllableCo',
u'\u2f80': 'KangxiRadicalBrush',
u'\u8b03': 'CjkUnifiedIdeograph8b03',
u'\u6c82': 'CjkUnifiedIdeograph6c82',
u'\uc805': 'HangulSyllableJeonj',
u'\u0507': 'CyrillicSmallLetterKomiDzje',
u'\u4209': 'CjkUnifiedIdeograph4209',
u'\ubd8c': 'HangulSyllableBuls',
u'\ufa8e': 'CjkCompatibilityIdeographfa8e',
u'\u3790': 'CjkUnifiedIdeograph3790',
u'\u9313': 'CjkUnifiedIdeograph9313',
u'\u7492': 'CjkUnifiedIdeograph7492',
u'\ud015': 'HangulSyllableKweot',
u'\ued17': 'UnicodeCharacter0xed17',
u'\u2a19': 'IntegralWithIntersection',
u'\u671b': 'CjkUnifiedIdeograph671b',
u'\u859c': 'CjkUnifiedIdeograph859c',
u'\uc29e': 'HangulSyllableSyuj',
u'\u1fa0': 'GreekSmallLetterOmegaWithPsiliAndYpogegrammeni',
u'\u5ca2': 'CjkUnifiedIdeograph5ca2',
u'\ub825': 'HangulSyllableRyeog',
u'\uf527': 'UnicodeCharacter0xf527',
u'\u3229': 'ParenthesizedIdeographTen',
u'\u4f2b': 'CjkUnifiedIdeograph4f2b',
u'\uaaae': 'UnicodeCharacter0xaaae',
u'\ue7b0': 'UnicodeCharacter0xe7b0',
u'\u24b2': 'ParenthesizedLatinSmallLetterW',
u'\u8035': 'CjkUnifiedIdeograph8035',
u'\u61b4': 'CjkUnifiedIdeograph61b4',
u'\udd37': 'UnicodeCharacter0xdd37',
u'\u1a39': 'UnicodeCharacter0x1a39',
u'\u573b': 'CjkUnifiedIdeograph573b',
u'\ub2be': 'HangulSyllableNyibs',
u'\ucfc0': 'HangulSyllableKoek',
u'\u0cc2': 'KannadaVowelSignUu',
u'\u49c4': 'CjkUnifiedIdeograph49c4',
u'\ua547': 'VaiSyllableNi',
u'\ue249': 'UnicodeCharacter0xe249',
u'\u3f4b': 'CjkUnifiedIdeograph3f4b',
u'\u7c4d': 'CjkUnifiedIdeograph7c4d',
u'\u9ace': 'CjkUnifiedIdeograph9ace',
u'\ud7d0': 'UnicodeCharacter0xd7d0',
u'\u14d2': 'CanadianSyllabicsNh',
u'\u51d4': 'CjkUnifiedIdeograph51d4',
u'\u8d57': 'CjkUnifiedIdeograph8d57',
u'\u6ed6': 'CjkUnifiedIdeograph6ed6',
u'\uca59': 'HangulSyllableJjeolt',
u'\u075b': 'ArabicLetterRehWithStroke',
u'\u445d': 'CjkUnifiedIdeograph445d',
u'\ubfe0': 'HangulSyllableBbuss',
u'\ufce2': 'ArabicLigatureBehWithHehMedialForm',
u'\u39e4': 'CjkUnifiedIdeograph39e4',
u'\u9567': 'CjkUnifiedIdeograph9567',
u'\u76e6': 'CjkUnifiedIdeograph76e6',
u'\ud269': 'HangulSyllableTwenj',
u'\uef6b': 'UnicodeCharacter0xef6b',
u'\u2c6d': 'LatinCapitalLetterAlpha',
u'\u696f': 'CjkUnifiedIdeograph696f',
u'\u87f0': 'CjkUnifiedIdeograph87f0',
u'\uc4f2': 'HangulSyllableSseugg',
u'\u01f4': 'LatinCapitalLetterGWithAcute',
u'\u5ef6': 'CjkUnifiedIdeograph5ef6',
u'\uba79': 'HangulSyllableMyeolg',
u'\uf77b': 'UnicodeCharacter0xf77b',
u'\u347d': 'CjkUnifiedIdeograph347d',
u'\u717f': 'CjkUnifiedIdeograph717f',
u'\u1181': 'HangulJungseongOye',
u'\uf604': 'UnicodeCharacter0xf604',
u'\uda87': 'UnicodeCharacter0xda87',
u'\u7c08': 'CjkUnifiedIdeograph7c08',
u'\u408b': 'CjkUnifiedIdeograph408b',
u'\ubf0a': 'HangulSyllableBbyeogg',
u'\u838d': 'CjkUnifiedIdeograph838d',
u'\u250e': 'BoxDrawingsDownHeavyAndRightLight',
u'\u0991': 'UnicodeCharacter0x991',
u'\uee14': 'UnicodeCharacter0xee14',
u'\ub297': 'HangulSyllableNeud',
u'\u5418': 'CjkUnifiedIdeograph5418',
u'\u389b': 'CjkUnifiedIdeograph389b',
u'\u971a': 'CjkUnifiedIdeograph971a',
u'\u1d1e': 'LatinSmallLetterSidewaysDiaeresizedU',
u'\ue1a1': 'UnicodeCharacter0xe1a1',
u'\u67a5': 'CjkUnifiedIdeograph67a5',
u'\uc624': 'HangulSyllableO',
u'\uaaa7': 'UnicodeCharacter0xaaa7',
u'\u4c28': 'CjkUnifiedIdeograph4c28',
u'\u10ab': 'GeorgianCapitalLetterMan',
u'\u8f2a': 'CjkUnifiedIdeograph8f2a',
u'\uf52e': 'UnicodeCharacter0xf52e',
u'\ud9b1': 'UnicodeCharacter0xd9b1',
u'\u7b32': 'CjkUnifiedIdeograph7b32',
u'\u5fb5': 'CjkUnifiedIdeograph5fb5',
u'\ube34': 'HangulSyllableByils',
u'\u82b7': 'CjkUnifiedIdeograph82b7',
u'\u2438': 'UnicodeCharacter0x2438',
u'\u08bb': 'UnicodeCharacter0x8bb',
u'\ued3e': 'UnicodeCharacter0xed3e',
u'\ub1c1': 'HangulSyllableNwaeb',
u'\u5342': 'CjkUnifiedIdeograph5342',
u'\u37c5': 'CjkUnifiedIdeograph37c5',
u'\u9644': 'CjkUnifiedIdeograph9644',
u'\u1c48': 'LepchaDigitEight',
u'\ue0cb': 'UnicodeCharacter0xe0cb',
u'\u66cf': 'CjkUnifiedIdeograph66cf',
u'\uc54e': 'HangulSyllableAlm',
u'\ua9d1': 'UnicodeCharacter0xa9d1',
u'\u4b52': 'CjkUnifiedIdeograph4b52',
u'\u2fd5': 'KangxiRadicalFlute',
u'\u8e54': 'CjkUnifiedIdeograph8e54',
u'\uf458': 'UnicodeCharacter0xf458',
u'\ud8db': 'UnicodeCharacter0xd8db',
u'\u7a5c': 'CjkUnifiedIdeograph7a5c',
u'\u5edf': 'CjkUnifiedIdeograph5edf',
u'\ubd5e': 'HangulSyllableBoej',
u'\u81e1': 'CjkUnifiedIdeograph81e1',
u'\u2362': 'AplFunctionalSymbolDelDiaeresis',
u'\u07e5': 'NkoLetterWa',
u'\uec68': 'UnicodeCharacter0xec68',
u'\ub0eb': 'HangulSyllableNyah',
u'\u526c': 'CjkUnifiedIdeograph526c',
u'\u36ef': 'CjkUnifiedIdeograph36ef',
u'\u956e': 'CjkUnifiedIdeograph956e',
u'\u1b72': 'BalineseMusicalSymbolCombiningBende',
u'\ufff5': 'UnicodeCharacter0xfff5',
u'\u65f9': 'CjkUnifiedIdeograph65f9',
u'\uc478': 'HangulSyllableSsuss',
u'\ua8fb': 'UnicodeCharacter0xa8fb',
u'\u4a7c': 'CjkUnifiedIdeograph4a7c',
u'\u2eff': 'UnicodeCharacter0x2eff',
u'\u8d7e': 'CjkUnifiedIdeograph8d7e',
u'\u1403': 'CanadianSyllabicsI',
u'\uf382': 'UnicodeCharacter0xf382',
u'\u7986': 'CjkUnifiedIdeograph7986',
u'\udd09': 'UnicodeCharacter0xdd09',
u'\ubc88': 'HangulSyllableBeon',
u'\u430d': 'CjkUnifiedIdeograph430d',
u'\u228c': 'Multiset',
u'\u860f': 'CjkUnifiedIdeograph860f',
u'\u0c13': 'TeluguLetterOo',
u'\ueb92': 'UnicodeCharacter0xeb92',
u'\u7217': 'CjkUnifiedIdeograph7217',
u'\u5196': 'CjkUnifiedIdeograph5196',
u'\ub519': 'HangulSyllableDinj',
u'\u9498': 'CjkUnifiedIdeograph9498',
u'\u3b1d': 'CjkUnifiedIdeograph3b1d',
u'\u1a9c': 'UnicodeCharacter0x1a9c',
u'\ue423': 'UnicodeCharacter0xe423',
u'\uc3a2': 'HangulSyllableSsyeogg',
u'\u6a27': 'CjkUnifiedIdeograph6a27',
u'\u49a6': 'CjkUnifiedIdeograph49a6',
u'\uad29': 'HangulSyllableGwaeb',
u'\u8ca8': 'CjkUnifiedIdeograph8ca8',
u'\u132d': 'EthiopicSyllableChe',
u'\uf2ac': 'UnicodeCharacter0xf2ac',
u'\u78b0': 'CjkUnifiedIdeograph78b0',
u'\udc33': 'UnicodeCharacter0xdc33',
u'\ubbb2': 'HangulSyllableMyulp',
u'\u4237': 'CjkUnifiedIdeograph4237',
u'\u21b6': 'AnticlockwiseTopSemicircleArrow',
u'\u8539': 'CjkUnifiedIdeograph8539',
u'\u0b3d': 'OriyaSignAvagraha',
u'\ueabc': 'UnicodeCharacter0xeabc',
u'\u7141': 'CjkUnifiedIdeograph7141',
u'\u50c0': 'CjkUnifiedIdeograph50c0',
u'\ub443': 'HangulSyllableDyolh',
u'\u93c2': 'CjkUnifiedIdeograph93c2',
u'\u3a47': 'CjkUnifiedIdeograph3a47',
u'\u19c6': 'NewTaiLueLetterFinalD',
u'\ue34d': 'UnicodeCharacter0xe34d',
u'\uc2cc': 'HangulSyllableSyils',
u'\u6951': 'CjkUnifiedIdeograph6951',
u'\u48d0': 'CjkUnifiedIdeograph48d0',
u'\uac53': 'HangulSyllableGyah',
u'\u8bd2': 'CjkUnifiedIdeograph8bd2',
u'\u1257': 'UnicodeCharacter0x1257',
u'\uf1d6': 'UnicodeCharacter0xf1d6',
u'\u77da': 'CjkUnifiedIdeograph77da',
u'\udb5d': 'UnicodeCharacter0xdb5d',
u'\ubadc': 'HangulSyllableMwak',
u'\u4161': 'CjkUnifiedIdeograph4161',
u'\u20e0': 'CombiningEnclosingCircleBackslash',
u'\u8463': 'CjkUnifiedIdeograph8463',
u'\u0a67': 'GurmukhiDigitOne',
u'\ue9e6': 'UnicodeCharacter0xe9e6',
u'\u706b': 'CjkUnifiedIdeograph706b',
u'\u6fea': 'CjkUnifiedIdeograph6fea',
u'\ub36d': 'HangulSyllableDeot',
u'\u92ec': 'CjkUnifiedIdeograph92ec',
u'\u3971': 'CjkUnifiedIdeograph3971',
u'\u18f0': 'UnicodeCharacter0x18f0',
u'\ue277': 'UnicodeCharacter0xe277',
u'\uc1f6': 'HangulSyllableSoej',
u'\u687b': 'CjkUnifiedIdeograph687b',
u'\u47fa': 'CjkUnifiedIdeograph47fa',
u'\uab7d': 'UnicodeCharacter0xab7d',
u'\u8afc': 'CjkUnifiedIdeograph8afc',
u'\u9081': 'CjkUnifiedIdeograph9081',
u'\uf100': 'UnicodeCharacter0xf100',
u'\ud583': 'HangulSyllableHaelh',
u'\u3202': 'ParenthesizedHangulTikeut',
u'\u1685': 'OghamLetterNion',
u'\u7704': 'CjkUnifiedIdeograph7704',
u'\u5b87': 'CjkUnifiedIdeograph5b87',
u'\ua40e': 'YiSyllableQu',
u'\u8891': 'CjkUnifiedIdeograph8891',
u'\ue910': 'UnicodeCharacter0xe910',
u'\ucd93': 'HangulSyllableCyoh',
u'\u2a12': 'LineIntegrationWithRectangularPathAroundPole',
u'\u0e95': 'LaoLetterTo',
u'\u6f14': 'CjkUnifiedIdeograph6f14',
u'\u3397': 'SquareDl',
u'\u7499': 'CjkUnifiedIdeograph7499',
u'\u9c1e': 'CjkUnifiedIdeograph9c1e',
u'\uc120': 'HangulSyllableSeon',
u'\ua5a3': 'VaiSyllableVu',
u'\u0222': 'LatinCapitalLetterOu',
u'\ue6a5': 'UnicodeCharacter0xe6a5',
u'\u4724': 'CjkUnifiedIdeograph4724',
u'\u2ba7': 'UnicodeCharacter0x2ba7',
u'\u6ca9': 'CjkUnifiedIdeograph6ca9',
u'\ub930': 'HangulSyllableRwem',
u'\u9db3': 'CjkUnifiedIdeograph9db3',
u'\ufa32': 'CjkCompatibilityIdeographfa32',
u'\udeb5': 'UnicodeCharacter0xdeb5',
u'\u3f34': 'CjkUnifiedIdeograph3f34',
u'\u03b7': 'GreekSmallLetterEta',
u'\u6036': 'CjkUnifiedIdeograph6036',
u'\u44b9': 'CjkUnifiedIdeograph44b9',
u'\u9140': 'CjkUnifiedIdeograph9140',
u'\ud242': 'HangulSyllableTuj',
u'\ub6c5': 'HangulSyllableDdweolt',
u'\u1744': 'BuhidLetterGa',
u'\ufbc7': 'UnicodeCharacter0xfbc7',
u'\u5846': 'CjkUnifiedIdeograph5846',
u'\u3cc9': 'CjkUnifiedIdeograph3cc9',
u'\u61cb': 'CjkUnifiedIdeograph61cb',
u'\u8950': 'CjkUnifiedIdeograph8950',
u'\uca52': 'HangulSyllableJjeonh',
u'\uaed5': 'HangulSyllableGgeot',
u'\u0f54': 'TibetanLetterPa',
u'\ud3d7': 'HangulSyllablePyed',
u'\u3056': 'HiraganaLetterZa',
u'\u14d9': 'CanadianSyllabicsYcreeLoo',
u'\u7558': 'CjkUnifiedIdeograph7558',
u'\u59db': 'CjkUnifiedIdeograph59db',
u'\ua262': 'YiSyllableNga',
u'\u86e5': 'CjkUnifiedIdeograph86e5',
u'\ue764': 'UnicodeCharacter0xe764',
u'\ucbe7': 'HangulSyllableJjeus',
u'\u2866': 'BraillePatternDots2367',
u'\u0ce9': 'KannadaDigitThree',
u'\u6d68': 'CjkUnifiedIdeograph6d68',
u'\u31eb': 'UnicodeCharacter0x31eb',
u'\u72ed': 'CjkUnifiedIdeograph72ed',
u'\u9a72': 'CjkUnifiedIdeograph9a72',
u'\udf74': 'UnicodeCharacter0xdf74',
u'\ua3f7': 'YiSyllableJyx',
u'\ue4f9': 'UnicodeCharacter0xe4f9',
u'\u4578': 'CjkUnifiedIdeograph4578',
u'\u29fb': 'TriplePlus',
u'\u6afd': 'CjkUnifiedIdeograph6afd',
u'\u5001': 'CjkUnifiedIdeograph5001',
u'\ub784': 'HangulSyllableRal',
u'\uf886': 'UnicodeCharacter0xf886',
u'\u3d88': 'CjkUnifiedIdeograph3d88',
u'\u810b': 'CjkUnifiedIdeograph810b',
u'\u7e8a': 'CjkUnifiedIdeograph7e8a',
u'\uc20d': 'HangulSyllableSyob',
u'\u070f': 'SyriacAbbreviationMark',
u'\u4811': 'CjkUnifiedIdeograph4811',
u'\uaf94': 'HangulSyllableGgoess',
u'\ud096': 'HangulSyllableKyilp',
u'\u1598': 'CanadianSyllabicsSayisiShi',
u'\u569a': 'CjkUnifiedIdeograph569a',
u'\uba1d': 'HangulSyllableMyaeg',
u'\uff1f': 'FullwidthQuestionMark',
u'\u2021': 'DoubleDagger',
u'\u6523': 'CjkUnifiedIdeograph6523',
u'\u87a4': 'CjkUnifiedIdeograph87a4',
u'\uc8a6': 'HangulSyllableJwap',
u'\u0da8': 'SinhalaLetterMahaapraanaTtayanna',
u'\u4eaa': 'CjkUnifiedIdeograph4eaa',
u'\u922d': 'CjkUnifiedIdeograph922d',
u'\u73ac': 'CjkUnifiedIdeograph73ac',
u'\ud72f': 'HangulSyllableHwic',
u'\u1831': 'MongolianLetterSha',
u'\u5d33': 'CjkUnifiedIdeograph5d33',
u'\ua0b6': 'YiSyllableMax',
u'\ue5b8': 'UnicodeCharacter0xe5b8',
u'\u26ba': 'Semisextile',
u'\u8a3d': 'CjkUnifiedIdeograph8a3d',
u'\u6bbc': 'CjkUnifiedIdeograph6bbc',
u'\ucf3f': 'HangulSyllableKyed',
u'\uf041': 'UnicodeCharacter0xf041',
u'\u3543': 'CjkUnifiedIdeograph3543',
u'\u7645': 'CjkUnifiedIdeograph7645',
u'\u98c6': 'CjkUnifiedIdeograph98c6',
u'\uddc8': 'UnicodeCharacter0xddc8',
u'\u1eca': 'LatinCapitalLetterIWithDotBelow',
u'\u43cc': 'CjkUnifiedIdeograph43cc',
u'\ua74f': 'LatinSmallLetterOo',
u'\ue851': 'UnicodeCharacter0xe851',
u'\u2d53': 'TifinaghLetterYu',
u'\u6e55': 'CjkUnifiedIdeograph6e55',
u'\ub5d8': 'HangulSyllableDdyeo',
u'\uf6da': 'UnicodeCharacter0xf6da',
u'\u3bdc': 'CjkUnifiedIdeograph3bdc',
u'\u9f5f': 'CjkUnifiedIdeograph9f5f',
u'\u7cde': 'CjkUnifiedIdeograph7cde',
u'\uc061': 'HangulSyllableBbeulg',
u'\u0563': 'ArmenianSmallLetterGim',
u'\u4665': 'CjkUnifiedIdeograph4665',
u'\uade8': 'HangulSyllableGyuls',
u'\ueeea': 'UnicodeCharacter0xeeea',
u'\u13ec': 'CherokeeLetterWo',
u'\u54ee': 'CjkUnifiedIdeograph54ee',
u'\ub871': 'HangulSyllableRong',
u'\ufd73': 'ArabicLigatureTahWithMeemWithMeemInitialForm',
u'\u3e75': 'CjkUnifiedIdeograph3e75',
u'\u6377': 'CjkUnifiedIdeograph6377',
u'\u85f8': 'CjkUnifiedIdeograph85f8',
u'\uc6fa': 'HangulSyllableWebs',
u'\u0bfc': 'UnicodeCharacter0xbfc',
u'\u4cfe': 'CjkUnifiedIdeograph4cfe',
u'\u7200': 'CjkUnifiedIdeograph7200',
u'\u5683': 'CjkUnifiedIdeograph5683',
u'\ub102': 'HangulSyllableNyaej',
u'\u9585': 'CjkUnifiedIdeograph9585',
u'\u3b06': 'CjkUnifiedIdeograph3b06',
u'\u1f89': 'GreekCapitalLetterAlphaWithDasiaAndProsgegrammeni',
u'\ue00c': 'UnicodeCharacter0xe00c',
u'\uc48f': 'HangulSyllableSsweolh',
u'\u6a10': 'CjkUnifiedIdeograph6a10',
u'\u4e93': 'CjkUnifiedIdeograph4e93',
u'\ua912': 'KayahLiLetterTa',
u'\u8d95': 'CjkUnifiedIdeograph8d95',
u'\u1316': 'UnicodeCharacter0x1316',
u'\uf799': 'UnicodeCharacter0xf799',
u'\u799d': 'CjkUnifiedIdeograph799d',
u'\ud81c': 'UnicodeCharacter0xd81c',
u'\ubc9f': 'HangulSyllableBeoh',
u'\u4220': 'CjkUnifiedIdeograph4220',
u'\u26a3': 'DoubledMaleSign',
u'\u8122': 'CjkUnifiedIdeograph8122',
u'\u0b26': 'OriyaLetterDa',
u'\uefa9': 'UnicodeCharacter0xefa9',
u'\u51ad': 'CjkUnifiedIdeograph51ad',
u'\ub02c': 'HangulSyllableGgyun',
u'\u94af': 'CjkUnifiedIdeograph94af',
u'\u3a30': 'CjkUnifiedIdeograph3a30',
u'\u1eb3': 'LatinSmallLetterAWithBreveAndHookAbove',
u'\ue336': 'UnicodeCharacter0xe336',
u'\uc7b9': 'HangulSyllableJaelt',
u'\u653a': 'CjkUnifiedIdeograph653a',
u'\u49bd': 'CjkUnifiedIdeograph49bd',
u'\ua83c': 'UnicodeCharacter0xa83c',
u'\u8cbf': 'CjkUnifiedIdeograph8cbf',
u'\u1240': 'EthiopicSyllableQa',
u'\uf6c3': 'UnicodeCharacter0xf6c3',
u'\u78c7': 'CjkUnifiedIdeograph78c7',
u'\udb46': 'UnicodeCharacter0xdb46',
u'\ubfc9': 'HangulSyllableBbyot',
u'\u5d4a': 'CjkUnifiedIdeograph5d4a',
u'\u21cd': 'LeftwardsDoubleArrowWithStroke',
u'\u804c': 'CjkUnifiedIdeograph804c',
u'\u0a50': 'UnicodeCharacter0xa50',
u'\ueed3': 'UnicodeCharacter0xeed3',
u'\u50d7': 'CjkUnifiedIdeograph50d7',
u'\ub356': 'HangulSyllableDeogg',
u'\u97d9': 'CjkUnifiedIdeograph97d9',
u'\u355a': 'CjkUnifiedIdeograph355a',
u'\u19dd': 'UnicodeCharacter0x19dd',
u'\ue260': 'UnicodeCharacter0xe260',
u'\uc6e3': 'HangulSyllableWeoc',
u'\u6464': 'CjkUnifiedIdeograph6464',
u'\u48e7': 'CjkUnifiedIdeograph48e7',
u'\uab66': 'UnicodeCharacter0xab66',
u'\u8fe9': 'CjkUnifiedIdeograph8fe9',
u'\u2d6a': 'UnicodeCharacter0x2d6a',
u'\uf1ed': 'UnicodeCharacter0xf1ed',
u'\u7bf1': 'CjkUnifiedIdeograph7bf1',
u'\uda70': 'UnicodeCharacter0xda70',
u'\ubef3': 'HangulSyllableBbed',
u'\u5c74': 'CjkUnifiedIdeograph5c74',
u'\u20f7': 'UnicodeCharacter0x20f7',
u'\u8376': 'CjkUnifiedIdeograph8376',
u'\u057a': 'ArmenianSmallLetterPeh',
u'\ue9fd': 'UnicodeCharacter0xe9fd',
u'\ud301': 'HangulSyllableTib',
u'\ub280': 'HangulSyllableNyuls',
u'\u5505': 'CjkUnifiedIdeograph5505',
u'\u3484': 'CjkUnifiedIdeograph3484',
u'\u9807': 'CjkUnifiedIdeograph9807',
u'\u020b': 'LatinSmallLetterIWithInvertedBreve',
u'\ufd8a': 'ArabicLigatureMeemWithHahWithMeemInitialForm',
u'\u678e': 'CjkUnifiedIdeograph678e',
u'\ucb11': 'HangulSyllableJjoeg',
u'\uaa90': 'UnicodeCharacter0xaa90',
u'\u4d15': 'CjkUnifiedIdeograph4d15',
u'\u2c94': 'CopticCapitalLetterKapa',
u'\ufa1b': 'CjkCompatibilityIdeographfa1b',
u'\ud59a': 'HangulSyllableHyalm',
u'\u7c1f': 'CjkUnifiedIdeograph7c1f',
u'\u5f9e': 'CjkUnifiedIdeograph5f9e',
u'\ua321': 'YiSyllableSup',
u'\u82a0': 'CjkUnifiedIdeograph82a0',
u'\u2525': 'BoxDrawingsVerticalLightAndLeftHeavy',
u'\u04a4': 'CyrillicCapitalLigatureEnGhe',
u'\ud22b': 'HangulSyllableTyoh',
u'\ucdaa': 'HangulSyllableCuj',
u'\u542f': 'CjkUnifiedIdeograph542f',
u'\u37ae': 'CjkUnifiedIdeograph37ae',
u'\u9b31': 'CjkUnifiedIdeograph9b31',
u'\u1d35': 'ModifierLetterCapitalI',
u'\ufcb4': 'ArabicLigatureDadWithJeemInitialForm',
u'\u66b8': 'CjkUnifiedIdeograph66b8',
u'\uca3b': 'HangulSyllableJjyaelb',
u'\ua5ba': 'VaiSyllableO',
u'\u4c3f': 'CjkUnifiedIdeograph4c3f',
u'\u2fbe': 'KangxiRadicalFight',
u'\uf545': 'UnicodeCharacter0xf545',
u'\ud4c4': 'HangulSyllablePwess',
u'\u7f49': 'CjkUnifiedIdeograph7f49',
u'\u5ec8': 'CjkUnifiedIdeograph5ec8',
u'\ua24b': 'YiSyllableHxiex',
u'\u9dca': 'CjkUnifiedIdeograph9dca',
u'\u244f': 'UnicodeCharacter0x244f',
u'\u07ce': 'NkoLetterU',
u'\ued55': 'UnicodeCharacter0xed55',
u'\uccd4': 'HangulSyllableCyeon',
u'\u5759': 'CjkUnifiedIdeograph5759',
u'\u36d8': 'CjkUnifiedIdeograph36d8',
u'\u9a5b': 'CjkUnifiedIdeograph9a5b',
u'\u1c5f': 'OlChikiLetterLaa',
u'\uffde': 'UnicodeCharacter0xffde',
u'\u61e2': 'CjkUnifiedIdeograph61e2',
u'\uc565': 'HangulSyllableAenj',
u'\ua4e4': 'UnicodeCharacter0xa4e4',
u'\u4f69': 'CjkUnifiedIdeograph4f69',
u'\u2ee8': 'CjkRadicalSimplifiedWheat',
u'\uf46f': 'UnicodeCharacter0xf46f',
u'\ud7ee': 'UnicodeCharacter0xd7ee',
u'\u7e73': 'CjkUnifiedIdeograph7e73',
u'\u59f2': 'CjkUnifiedIdeograph59f2',
u'\ubd75': 'HangulSyllableByob',
u'\u9cf4': 'CjkUnifiedIdeograph9cf4',
u'\u2779': 'DingbatNegativeCircledDigitFour',
u'\u06f8': 'ExtendedArabicindicDigitEight',
u'\uec7f': 'UnicodeCharacter0xec7f',
u'\ucffe': 'HangulSyllableKweogg',
u'\uba06': 'HangulSyllableMyanh',
u'\u9e89': 'CjkUnifiedIdeograph9e89',
u'\uff08': 'FullwidthLeftParenthesis',
u'\uc38b': 'HangulSyllableSsed',
u'\u3c0a': 'CjkUnifiedIdeograph3c0a',
u'\x8d': 'UnicodeCharacter0x8d',
u'\u610c': 'CjkUnifiedIdeograph610c',
u'\u458f': 'CjkUnifiedIdeograph458f',
u'\u9216': 'CjkUnifiedIdeograph9216',
u'\ud718': 'HangulSyllableHwi',
u'\ubb9b': 'HangulSyllableMwis',
u'\u141a': 'CanadianSyllabicsWestcreeWaa',
u'\uf89d': 'UnicodeCharacter0xf89d',
u'\u591c': 'CjkUnifiedIdeograph591c',
u'\u3d9f': 'CjkUnifiedIdeograph3d9f',
u'\u62a1': 'CjkUnifiedIdeograph62a1',
u'\u8a26': 'CjkUnifiedIdeograph8a26',
u'\ucf28': 'HangulSyllableKyeols',
u'\u93ab': 'CjkUnifiedIdeograph93ab',
u'\u0c2a': 'TeluguLetterPa',
u'\ud0ad': 'HangulSyllableKilg',
u'\u312c': 'BopomofoLetterGn',
u'\u15af': 'CanadianSyllabicsAivilikB',
u'\u762e': 'CjkUnifiedIdeograph762e',
u'\u5ab1': 'CjkUnifiedIdeograph5ab1',
u'\ua738': 'LatinCapitalLetterAv',
u'\u8bbb': 'CjkUnifiedIdeograph8bbb',
u'\ue43a': 'UnicodeCharacter0xe43a',
u'\uc8bd': 'HangulSyllableJwaeng',
u'\u293c': 'TopArcClockwiseArrowWithMinus',
u'\u0dbf': 'UnicodeCharacter0xdbf',
u'\u6e3e': 'CjkUnifiedIdeograph6e3e',
u'\u32c1': 'IdeographicTelegraphSymbolForFebruary',
u'\u77c3': 'CjkUnifiedIdeograph77c3',
u'\u9f48': 'CjkUnifiedIdeograph9f48',
u'\udc4a': 'UnicodeCharacter0xdc4a',
u'\ua0cd': 'YiSyllableFit',
u'\u014c': 'LatinCapitalLetterOWithMacron',
u'\ue5cf': 'UnicodeCharacter0xe5cf',
u'\u464e': 'CjkUnifiedIdeograph464e',
u'\u2ad1': 'ClosedSubsetOrEqualTo',
u'\u6fd3': 'CjkUnifiedIdeograph6fd3',
u'\ub45a': 'HangulSyllableDulm',
u'\u98dd': 'CjkUnifiedIdeograph98dd',
u'\uf95c': 'CjkCompatibilityIdeographf95c',
u'\udddf': 'UnicodeCharacter0xdddf',
u'\u3e5e': 'CjkUnifiedIdeograph3e5e',
u'\u02e1': 'ModifierLetterSmallL',
u'\u6360': 'CjkUnifiedIdeograph6360',
u'\u47e3': 'CjkUnifiedIdeograph47e3',
u'\uac6a': 'HangulSyllableGyaej',
u'\ud16c': 'HangulSyllableTyeon',
u'\ub5ef': 'HangulSyllableDdyeoc',
u'\u166e': 'CanadianSyllabicsFullStop',
u'\ufaf1': 'UnicodeCharacter0xfaf1',
u'\u5b70': 'CjkUnifiedIdeograph5b70',
u'\u3ff3': 'CjkUnifiedIdeograph3ff3',
u'\u7cf5': 'CjkUnifiedIdeograph7cf5',
u'\u847a': 'CjkUnifiedIdeograph847a',
u'\uc97c': 'HangulSyllableJyum',
u'\uadff': 'HangulSyllableGeud',
u'\u0e7e': 'UnicodeCharacter0xe7e',
u'\u3380': 'SquarePaAmps',
u'\u9703': 'CjkUnifiedIdeograph9703',
u'\u7082': 'CjkUnifiedIdeograph7082',
u'\ud405': 'HangulSyllablePot',
u'\u1907': 'LimbuLetterCha',
u'\u5e09': 'CjkUnifiedIdeograph5e09',
u'\ua18c': 'YiSyllableNo',
u'\ue68e': 'UnicodeCharacter0xe68e',
u'\u2b90': 'UnicodeCharacter0x2b90',
u'\u8f13': 'CjkUnifiedIdeograph8f13',
u'\u6892': 'CjkUnifiedIdeograph6892',
u'\ucc15': 'HangulSyllableJjilg',
u'\uf117': 'UnicodeCharacter0xf117',
u'\u3619': 'CjkUnifiedIdeograph3619',
u'\u7b1b': 'CjkUnifiedIdeograph7b1b',
u'\u999c': 'CjkUnifiedIdeograph999c',
u'\ude9e': 'UnicodeCharacter0xde9e',
u'\u03a0': 'GreekCapitalLetterPi',
u'\u40a2': 'CjkUnifiedIdeograph40a2',
u'\ua425': 'YiSyllableJjo',
u'\ue927': 'UnicodeCharacter0xe927',
u'\u2e29': 'RightDoubleParenthesis',
u'\u532b': 'CjkUnifiedIdeograph532b',
u'\ub6ae': 'HangulSyllableDdubs',
u'\ufbb0': 'ArabicLetterYehBarreeWithHamzaAboveIsolatedForm',
u'\u38b2': 'CjkUnifiedIdeograph38b2',
u'\u9c35': 'CjkUnifiedIdeograph9c35',
u'\u7db4': 'CjkUnifiedIdeograph7db4',
u'\uc137': 'HangulSyllableSeoh',
u'\u0639': 'ArabicLetterAin',
u'\u4b3b': 'CjkUnifiedIdeograph4b3b',
u'\uaebe': 'HangulSyllableGgeogg',
u'\ud3c0': 'HangulSyllablePyeols',
u'\u10c2': 'GeorgianCapitalLetterHie',
u'\u55c4': 'CjkUnifiedIdeograph55c4',
u'\ub947': 'HangulSyllableRwilb',
u'\ufe49': 'DashedOverline',
u'\u234b': 'AplFunctionalSymbolDeltaStile',
u'\u604d': 'CjkUnifiedIdeograph604d',
u'\u86ce': 'CjkUnifiedIdeograph86ce',
u'\ucbd0': 'HangulSyllableJjyuk',
u'\u08d2': 'UnicodeCharacter0x8d2',
u'\u4dd4': 'HexagramForBitingThrough',
u'\u9157': 'CjkUnifiedIdeograph9157',
u'\u72d6': 'CjkUnifiedIdeograph72d6',
u'\ud659': 'HangulSyllableHwanj',
u'\u1b5b': 'BalinesePamada',
u'\u585d': 'CjkUnifiedIdeograph585d',
u'\ua3e0': 'YiSyllableJit',
u'\ue0e2': 'UnicodeCharacter0xe0e2',
u'\u25e4': 'BlackUpperLeftTriangle',
u'\u8967': 'CjkUnifiedIdeograph8967',
u'\u6ae6': 'CjkUnifiedIdeograph6ae6',
u'\uce69': 'HangulSyllableCib',
u'\uf36b': 'UnicodeCharacter0xf36b',
u'\u306d': 'HiraganaLetterNe',
u'\u756f': 'CjkUnifiedIdeograph756f',
u'\u9bf0': 'CjkUnifiedIdeograph9bf0',
u'\ud8f2': 'UnicodeCharacter0xd8f2',
u'\u1df4': 'UnicodeCharacter0x1df4',
u'\u42f6': 'CjkUnifiedIdeograph42f6',
u'\ua679': 'UnicodeCharacter0xa679',
u'\ueb7b': 'UnicodeCharacter0xeb7b',
u'\u287d': 'BraillePatternDots134567',
u'\u6d7f': 'CjkUnifiedIdeograph6d7f',
u'\u1581': 'CanadianSyllabicsQo',
u'\uf204': 'UnicodeCharacter0xf204',
u'\ude87': 'UnicodeCharacter0xde87',
u'\u7808': 'CjkUnifiedIdeograph7808',
u'\u448b': 'CjkUnifiedIdeograph448b',
u'\ubb0a': 'HangulSyllableMoelp',
u'\u878d': 'CjkUnifiedIdeograph878d',
u'\u210e': 'PlanckConstant',
u'\u0d91': 'SinhalaLetterEyanna',
u'\uea14': 'UnicodeCharacter0xea14',
u'\ub697': 'HangulSyllableDdyoc',
u'\u5018': 'CjkUnifiedIdeograph5018',
u'\u3c9b': 'CjkUnifiedIdeograph3c9b',
u'\u931a': 'CjkUnifiedIdeograph931a',
u'\u191e': 'UnicodeCharacter0x191e',
u'\ue5a1': 'UnicodeCharacter0xe5a1',
u'\u63a5': 'CjkUnifiedIdeograph63a5',
u'\uc224': 'HangulSyllableSuls',
u'\uaea7': 'HangulSyllableGgyaed',
u'\u4828': 'CjkUnifiedIdeograph4828',
u'\u14ab': 'CanadianSyllabicsMaa',
u'\u8b2a': 'CjkUnifiedIdeograph8b2a',
u'\uf12e': 'UnicodeCharacter0xf12e',
u'\uddb1': 'UnicodeCharacter0xddb1',
u'\u7f32': 'CjkUnifiedIdeograph7f32',
u'\u5bb5': 'CjkUnifiedIdeograph5bb5',
u'\uba34': 'HangulSyllableMyaek',
u'\u86b7': 'CjkUnifiedIdeograph86b7',
u'\u2038': 'Caret',
u'\u0cbb': 'UnicodeCharacter0xcbb',
u'\ue93e': 'UnicodeCharacter0xe93e',
u'\ub5c1': 'HangulSyllableDdenj',
u'\u5742': 'CjkUnifiedIdeograph5742',
u'\u33c5': 'SquareCd',
u'\u9244': 'CjkUnifiedIdeograph9244',
u'\u1848': 'MongolianLetterTodoOe',
u'\ue4cb': 'UnicodeCharacter0xe4cb',
u'\u62cf': 'CjkUnifiedIdeograph62cf',
u'\uc14e': 'HangulSyllableSej',
u'\uadd1': 'HangulSyllableGwib',
u'\u4f52': 'CjkUnifiedIdeograph4f52',
u'\u2bd5': 'UnicodeCharacter0x2bd5',
u'\u8a54': 'CjkUnifiedIdeograph8a54',
u'\uf058': 'UnicodeCharacter0xf058',
u'\udcdb': 'UnicodeCharacter0xdcdb',
u'\u7e5c': 'CjkUnifiedIdeograph7e5c',
u'\u5adf': 'CjkUnifiedIdeograph5adf',
u'\ub95e': 'HangulSyllableRyunh',
u'\u85e1': 'CjkUnifiedIdeograph85e1',
u'\u2762': 'HeavyExclamationMarkOrnament',
u'\u03e5': 'CopticSmallLetterFei',
u'\ue868': 'UnicodeCharacter0xe868',
u'\ub4eb': 'HangulSyllableDeulh',
u'\u566c': 'CjkUnifiedIdeograph566c',
u'\u32ef': 'CircledKatakanaMi',
u'\u916e': 'CjkUnifiedIdeograph916e',
u'\u1f72': 'GreekSmallLetterEpsilonWithVaria',
u'\ufbf5': 'ArabicLigatureYehWithHamzaAboveWithYuFinalForm',
u'\u61f9': 'CjkUnifiedIdeograph61f9',
u'\uc078': 'HangulSyllableBbyin',
u'\uacfb': 'HangulSyllableGoh',
u'\u4e7c': 'CjkUnifiedIdeograph4e7c',
u'\u2aff': 'NaryWhiteVerticalBar',
u'\u897e': 'CjkUnifiedIdeograph897e',
u'\u1003': 'MyanmarLetterGha',
u'\uf782': 'UnicodeCharacter0xf782',
u'\u7d86': 'CjkUnifiedIdeograph7d86',
u'\ud909': 'UnicodeCharacter0xd909',
u'\ub888': 'HangulSyllableRwam',
u'\u470d': 'CjkUnifiedIdeograph470d',
u'\u268c': 'DigramForGreaterYang',
u'\u820f': 'CjkUnifiedIdeograph820f',
u'\u0813': 'UnicodeCharacter0x813',
u'\uef92': 'UnicodeCharacter0xef92',
u'\u7617': 'CjkUnifiedIdeograph7617',
u'\u5596': 'CjkUnifiedIdeograph5596',
u'\ub119': 'HangulSyllableNeob',
u'\u9098': 'CjkUnifiedIdeograph9098',
u'\u3f1d': 'CjkUnifiedIdeograph3f1d',
u'\u1e9c': 'LatinSmallLetterLongSWithDiagonalStroke',
u'\ue023': 'UnicodeCharacter0xe023',
u'\uc7a2': 'HangulSyllableJabs',
u'\u6e27': 'CjkUnifiedIdeograph6e27',
u'\u4da6': 'CjkUnifiedIdeograph4da6',
u'\ua929': 'KayahLiVowelEe',
u'\u88a8': 'CjkUnifiedIdeograph88a8',
u'\u172d': 'HanunooLetterRa',
u'\uf6ac': 'UnicodeCharacter0xf6ac',
u'\u7cb0': 'CjkUnifiedIdeograph7cb0',
u'\ud833': 'UnicodeCharacter0xd833',
u'\ubfb2': 'HangulSyllableBbyogg',
u'\u4637': 'CjkUnifiedIdeograph4637',
u'\u25b6': 'BlackRightpointingTriangle',
u'\u8139': 'CjkUnifiedIdeograph8139',
u'\u0f3d': 'TibetanMarkAngKhangGyas',
u'\ueebc': 'UnicodeCharacter0xeebc',
u'\u7541': 'CjkUnifiedIdeograph7541',
u'\u54c0': 'CjkUnifiedIdeograph54c0',
u'\ub043': 'HangulSyllableGgyuh',
u'\u97c2': 'CjkUnifiedIdeograph97c2',
u'\u3e47': 'CjkUnifiedIdeograph3e47',
u'\u1dc6': 'CombiningMacrongrave',
u'\ue74d': 'UnicodeCharacter0xe74d',
u'\uc6cc': 'HangulSyllableWeo',
u'\u6d51': 'CjkUnifiedIdeograph6d51',
u'\u4cd0': 'CjkUnifiedIdeograph4cd0',
u'\ua853': 'PhagspaLetterWa',
u'\u8fd2': 'CjkUnifiedIdeograph8fd2',
u'\u1657': 'CanadianSyllabicsCarrierShee',
u'\uf5d6': 'UnicodeCharacter0xf5d6',
u'\u73da': 'CjkUnifiedIdeograph73da',
u'\udf5d': 'UnicodeCharacter0xdf5d',
u'\ubedc': 'HangulSyllableBbeols',
u'\u4561': 'CjkUnifiedIdeograph4561',
u'\u24e0': 'CircledLatinSmallLetterQ',
u'\u8063': 'CjkUnifiedIdeograph8063',
u'\u0e67': 'UnicodeCharacter0xe67',
u'\uede6': 'UnicodeCharacter0xede6',
u'\u746b': 'CjkUnifiedIdeograph746b',
u'\u6bea': 'CjkUnifiedIdeograph6bea',
u'\ub76d': 'HangulSyllableDdilt',
u'\u96ec': 'CjkUnifiedIdeograph96ec',
u'\u3d71': 'CjkUnifiedIdeograph3d71',
u'\u1cf0': 'UnicodeCharacter0x1cf0',
u'\ue677': 'UnicodeCharacter0xe677',
u'\uc5f6': 'HangulSyllableYeolm',
u'\u6c7b': 'CjkUnifiedIdeograph6c7b',
u'\u43fa': 'CjkUnifiedIdeograph43fa',
u'\uaf7d': 'HangulSyllableGgwaet',
u'\u8efc': 'CjkUnifiedIdeograph8efc',
u'\u9481': 'CjkUnifiedIdeograph9481',
u'\uf500': 'UnicodeCharacter0xf500',
u'\ud183': 'HangulSyllableTyeoh',
u'\u3602': 'CjkUnifiedIdeograph3602',
u'\u1285': 'EthiopicSyllableXe',
u'\u7304': 'CjkUnifiedIdeograph7304',
u'\u5f87': 'CjkUnifiedIdeograph5f87',
u'\ua00e': 'YiSyllableUop',
u'\u8c91': 'CjkUnifiedIdeograph8c91',
u'\ued10': 'UnicodeCharacter0xed10',
u'\uc993': 'HangulSyllableJeulb',
u'\u2e12': 'Hypodiastole',
u'\u0a95': 'GujaratiLetterKa',
u'\u6b14': 'CjkUnifiedIdeograph6b14',
u'\u3797': 'CjkUnifiedIdeograph3797',
u'\u7099': 'CjkUnifiedIdeograph7099',
u'\u981e': 'CjkUnifiedIdeograph981e',
u'\uc520': 'HangulSyllableSsyiss',
u'\ua1a3': 'YiSyllableHluo',
u'\u0622': 'ArabicLetterAlefWithMaddaAbove',
u'\ue2a5': 'UnicodeCharacter0xe2a5',
u'\u4324': 'CjkUnifiedIdeograph4324',
u'\u2fa7': 'KangxiRadicalLong',
u'\u68a9': 'CjkUnifiedIdeograph68a9',
u'\ubd30': 'HangulSyllableBwaen',
u'\u99b3': 'CjkUnifiedIdeograph99b3',
u'\ufe32': 'PresentationFormForVerticalEnDash',
u'\udab5': 'UnicodeCharacter0xdab5',
u'\u3b34': 'CjkUnifiedIdeograph3b34',
u'\u07b7': 'UnicodeCharacter0x7b7',
u'\u6436': 'CjkUnifiedIdeograph6436',
u'\u40b9': 'CjkUnifiedIdeograph40b9',
u'\u9540': 'CjkUnifiedIdeograph9540',
u'\ud642': 'HangulSyllableHolm',
u'\ub2c5': 'HangulSyllableNyit',
u'\u1344': 'EthiopicSyllableTzee',
u'\uffc7': 'HalfwidthHangulLetterE',
u'\u5c46': 'CjkUnifiedIdeograph5c46',
u'\u38c9': 'CjkUnifiedIdeograph38c9',
u'\u65cb': 'CjkUnifiedIdeograph65cb',
u'\u8d50': 'CjkUnifiedIdeograph8d50',
u'\uce52': 'HangulSyllableCyij',
u'\uaad5': 'UnicodeCharacter0xaad5',
u'\u0b54': 'UnicodeCharacter0xb54',
u'\ud7d7': 'UnicodeCharacter0xd7d7',
u'\u3456': 'CjkUnifiedIdeograph3456',
u'\u10d9': 'GeorgianLetterKan',
u'\u7158': 'CjkUnifiedIdeograph7158',
u'\u5ddb': 'CjkUnifiedIdeograph5ddb',
u'\ua662': 'CyrillicCapitalLetterSoftDe',
u'\u82e5': 'CjkUnifiedIdeograph82e5',
u'\ue364': 'UnicodeCharacter0xe364',
u'\ucfe7': 'HangulSyllableKud',
u'\u2c66': 'LatinSmallLetterTWithDiagonalStroke',
u'\u08e9': 'UnicodeCharacter0x8e9',
u'\u6968': 'CjkUnifiedIdeograph6968',
u'\u35eb': 'CjkUnifiedIdeograph35eb',
u'\u76ed': 'CjkUnifiedIdeograph76ed',
u'\u9e72': 'CjkUnifiedIdeograph9e72',
u'\udb74': 'UnicodeCharacter0xdb74',
u'\ua7f7': 'UnicodeCharacter0xa7f7',
u'\u0476': 'CyrillicCapitalLetterIzhitsaWithDoubleGraveAccent',
u'\ue0f9': 'UnicodeCharacter0xe0f9',
u'\u4178': 'CjkUnifiedIdeograph4178',
u'\u2dfb': 'CombiningCyrillicLetterYu',
u'\u6efd': 'CjkUnifiedIdeograph6efd',
u'\u5401': 'CjkUnifiedIdeograph5401',
u'\ub384': 'HangulSyllableDess',
u'\ufc86': 'ArabicLigatureLamWithAlefMaksuraFinalForm',
u'\u3988': 'CjkUnifiedIdeograph3988',
u'\u850b': 'CjkUnifiedIdeograph850b',
u'\u7a8a': 'CjkUnifiedIdeograph7a8a',
u'\uc60d': 'HangulSyllableYenj',
u'\u030f': 'CombiningDoubleGraveAccent',
u'\u4c11': 'CjkUnifiedIdeograph4c11',
u'\uab94': 'UnicodeCharacter0xab94',
u'\ud496': 'HangulSyllablePweogg',
u'\u1198': 'HangulJungseongIa',
u'\u529a': 'CjkUnifiedIdeograph529a',
u'\ube1d': 'HangulSyllableBeub',
u'\ufb1f': 'HebrewLigatureYiddishYodYodPatah',
u'\u2421': 'SymbolForDelete',
u'\u6123': 'CjkUnifiedIdeograph6123',
u'\u83a4': 'CjkUnifiedIdeograph83a4',
u'\ucca6': 'HangulSyllableCeolp',
u'\u09a8': 'BengaliLetterNa',
u'\u4aaa': 'CjkUnifiedIdeograph4aaa',
u'\u962d': 'CjkUnifiedIdeograph962d',
u'\u77ac': 'CjkUnifiedIdeograph77ac',
u'\ud32f': 'HangulSyllablePaed',
u'\u1c31': 'LepchaConsonantSignP',
u'\u5933': 'CjkUnifiedIdeograph5933',
u'\ua4b6': 'YiRadicalGot',
u'\ue1b8': 'UnicodeCharacter0xe1b8',
u'\u22ba': 'Intercalate',
u'\u8e3d': 'CjkUnifiedIdeograph8e3d',
u'\u6fbc': 'CjkUnifiedIdeograph6fbc',
u'\ucb3f': 'HangulSyllableJjyos',
u'\uf441': 'UnicodeCharacter0xf441',
u'\u3143': 'HangulLetterSsangpieup',
u'\u7245': 'CjkUnifiedIdeograph7245',
u'\u9cc6': 'CjkUnifiedIdeograph9cc6',
u'\ud9c8': 'UnicodeCharacter0xd9c8',
u'\u1aca': 'UnicodeCharacter0x1aca',
u'\u47cc': 'CjkUnifiedIdeograph47cc',
u'\ua34f': 'YiSyllableZho',
u'\uec51': 'UnicodeCharacter0xec51',
u'\u2953': 'RightwardsHarpoonWithBarbUpToBar',
u'\u6a55': 'CjkUnifiedIdeograph6a55',
u'\ub1d8': 'HangulSyllableNoels',
u'\uf2da': 'UnicodeCharacter0xf2da',
u'\u3fdc': 'CjkUnifiedIdeograph3fdc',
u'\u9b5f': 'CjkUnifiedIdeograph9b5f',
u'\u78de': 'CjkUnifiedIdeograph78de',
u'\uc461': 'HangulSyllableSsyot',
u'\u0163': 'LatinSmallLetterTWithCedilla',
u'\u4265': 'CjkUnifiedIdeograph4265',
u'\ua9e8': 'UnicodeCharacter0xa9e8',
u'\ueaea': 'UnicodeCharacter0xeaea',
u'\u17ec': 'UnicodeCharacter0x17ec',
u'\u50ee': 'CjkUnifiedIdeograph50ee',
u'\ubc71': 'HangulSyllableByaelg',
u'\uf973': 'CjkCompatibilityIdeographf973',
u'\u3a75': 'CjkUnifiedIdeograph3a75',
u'\u6777': 'CjkUnifiedIdeograph6777',
u'\u81f8': 'CjkUnifiedIdeograph81f8',
u'\uc2fa': 'HangulSyllableSsagg',
u'\u0ffc': 'UnicodeCharacter0xffc',
u'\u48fe': 'CjkUnifiedIdeograph48fe',
u'\u7600': 'CjkUnifiedIdeograph7600',
u'\u5283': 'CjkUnifiedIdeograph5283',
u'\ub502': 'HangulSyllableDyilm',
u'\u9185': 'CjkUnifiedIdeograph9185',
u'\u3f06': 'CjkUnifiedIdeograph3f06',
u'\u1b89': 'SundaneseLetterEu',
u'\ue40c': 'UnicodeCharacter0xe40c',
u'\uc08f': 'HangulSyllableBbyih',
u'\u6e10': 'CjkUnifiedIdeograph6e10',
u'\u4a93': 'CjkUnifiedIdeograph4a93',
u'\uad12': 'HangulSyllableGwaj',
u'\u8995': 'CjkUnifiedIdeograph8995',
u'\u1716': 'UnicodeCharacter0x1716',
u'\uf399': 'UnicodeCharacter0xf399',
u'\u7d9d': 'CjkUnifiedIdeograph7d9d',
u'\udc1c': 'UnicodeCharacter0xdc1c',
u'\ub89f': 'HangulSyllableRwaelb',
u'\u4620': 'CjkUnifiedIdeograph4620',
u'\u22a3': 'LeftTack',
u'\u8522': 'CjkUnifiedIdeograph8522',
u'\u0f26': 'TibetanDigitSix',
u'\ueba9': 'UnicodeCharacter0xeba9',
u'\u55ad': 'CjkUnifiedIdeograph55ad',
u'\ub42c': 'HangulSyllableDoess',
u'\u90af': 'CjkUnifiedIdeograph90af',
u'\u3e30': 'CjkUnifiedIdeograph3e30',
u'\u1ab3': 'UnicodeCharacter0x1ab3',
u'\ue736': 'UnicodeCharacter0xe736',
u'\uc3b9': 'HangulSyllableSsyeot',
u'\u613a': 'CjkUnifiedIdeograph613a',
u'\u4dbd': 'UnicodeCharacter0x4dbd',
u'\uac3c': 'HangulSyllableGyan',
u'\u88bf': 'CjkUnifiedIdeograph88bf',
u'\u1640': 'CanadianSyllabicsCarrierZu',
u'\uf2c3': 'UnicodeCharacter0xf2c3',
u'\u7cc7': 'CjkUnifiedIdeograph7cc7',
u'\udf46': 'UnicodeCharacter0xdf46',
u'\ubbc9': 'HangulSyllableMeulg',
u'\u594a': 'CjkUnifiedIdeograph594a',
u'\u25cd': 'CircleWithVerticalFill',
u'\u844c': 'CjkUnifiedIdeograph844c',
u'\u0e50': 'ThaiDigitZero',
u'\uead3': 'UnicodeCharacter0xead3',
u'\u54d7': 'CjkUnifiedIdeograph54d7',
u'\ub756': 'HangulSyllableDdyibs',
u'\u93d9': 'CjkUnifiedIdeograph93d9',
u'\u315a': 'HangulLetterOe',
u'\u1ddd': 'CombiningLatinSmallLetterL',
u'\ue660': 'UnicodeCharacter0xe660',
u'\uc2e3': 'HangulSyllableSid',
u'\u6064': 'CjkUnifiedIdeograph6064',
u'\u4ce7': 'CjkUnifiedIdeograph4ce7',
u'\uaf66': 'HangulSyllableGgwaegg',
u'\u8be9': 'CjkUnifiedIdeograph8be9',
u'\u296a': 'LeftwardsHarpoonWithBarbUpAboveLongDash',
u'\uf5ed': 'UnicodeCharacter0xf5ed',
u'\u7ff1': 'CjkUnifiedIdeograph7ff1',
u'\ude70': 'UnicodeCharacter0xde70',
u'\ubaf3': 'HangulSyllableMwaes',
u'\u5874': 'CjkUnifiedIdeograph5874',
u'\u24f7': 'DoubleCircledDigitThree',
u'\u8776': 'CjkUnifiedIdeograph8776',
u'\u017a': 'LatinSmallLetterZWithAcute',
u'\uedfd': 'UnicodeCharacter0xedfd',
u'\ud701': 'HangulSyllableHwenj',
u'\ub680': 'HangulSyllableDdyo',
u'\u5105': 'CjkUnifiedIdeograph5105',
u'\u3084': 'HiraganaLetterYa',
u'\u9c07': 'CjkUnifiedIdeograph9c07',
u'\u060b': 'AfghaniSign',
u'\uf98a': 'CjkCompatibilityIdeographf98a',
u'\u638e': 'CjkUnifiedIdeograph638e',
u'\ucf11': 'HangulSyllableKeb',
u'\uae90': 'HangulSyllableGgyals',
u'\u4915': 'CjkUnifiedIdeograph4915',
u'\u2894': 'BraillePatternDots358',
u'\ufe1b': 'UnicodeCharacter0xfe1b',
u'\ud19a': 'HangulSyllableTyej',
u'\u781f': 'CjkUnifiedIdeograph781f',
u'\u5b9e': 'CjkUnifiedIdeograph5b9e',
u'\ua721': 'ModifierLetterStressAndLowTone',
u'\u86a0': 'CjkUnifiedIdeograph86a0',
u'\u2125': 'OunceSign',
u'\xa4': 'CurrencySign',
u'\ud62b': 'HangulSyllableHyelh',
u'\uc9aa': 'HangulSyllableJyinh',
u'\u502f': 'CjkUnifiedIdeograph502f',
u'\u33ae': 'SquareRadOverS',
u'\u9f31': 'CjkUnifiedIdeograph9f31',
u'\u1935': 'LimbuSmallLetterPa',
u'\uf8b4': 'UnicodeCharacter0xf8b4',
u'\u62b8': 'CjkUnifiedIdeograph62b8',
u'\uce3b': 'HangulSyllableCeuh',
u'\ua1ba': 'YiSyllableLip',
u'\u483f': 'CjkUnifiedIdeograph483f',
u'\u2bbe': 'UnicodeCharacter0x2bbe',
u'\uf145': 'UnicodeCharacter0xf145',
u'\ud0c4': 'HangulSyllableTan',
u'\u7b49': 'CjkUnifiedIdeograph7b49',
u'\u5ac8': 'CjkUnifiedIdeograph5ac8',
u'\ua64b': 'CyrillicSmallLetterMonographUk',
u'\u99ca': 'CjkUnifiedIdeograph99ca',
u'\u204f': 'ReversedSemicolon',
u'\u03ce': 'GreekSmallLetterOmegaWithTonos',
u'\ue955': 'UnicodeCharacter0xe955',
u'\uc8d4': 'HangulSyllableJoem',
u'\u5359': 'CjkUnifiedIdeograph5359',
u'\u32d8': 'CircledKatakanaKe',
u'\u9e5b': 'CjkUnifiedIdeograph9e5b',
u'\u185f': 'MongolianLetterSibeIy',
u'\ufbde': 'ArabicLetterVeIsolatedForm',
u'\u65e2': 'CjkUnifiedIdeograph65e2',
u'\uc165': 'HangulSyllableSyeob',
u'\ua0e4': 'YiSyllableVi',
u'\u4b69': 'CjkUnifiedIdeograph4b69',
u'\u2ae8': 'ShortUpTackWithUnderbar',
u'\uf06f': 'UnicodeCharacter0xf06f',
u'\ud3ee': 'HangulSyllablePogg',
u'\u7a73': 'CjkUnifiedIdeograph7a73',
u'\u5df2': 'CjkUnifiedIdeograph5df2',
u'\ub975': 'HangulSyllableReug',
u'\u98f4': 'CjkUnifiedIdeograph98f4',
u'\u2379': 'AplFunctionalSymbolOmegaUnderbar',
u'\u02f8': 'ModifierLetterRaisedColon',
u'\ue87f': 'UnicodeCharacter0xe87f',
u'\ucbfe': 'HangulSyllableJjyilp',
u'\ube06': 'HangulSyllableByuj',
u'\u9a89': 'CjkUnifiedIdeograph9a89',
u'\ufb08': 'UnicodeCharacter0xfb08',
u'\uc78b': 'HangulSyllableIc',
u'\u380a': 'CjkUnifiedIdeograph380a',
u'\u048d': 'CyrillicSmallLetterSemisoftSign',
u'\u650c': 'CjkUnifiedIdeograph650c',
u'\u418f': 'CjkUnifiedIdeograph418f',
u'\u9616': 'CjkUnifiedIdeograph9616',
u'\ud318': 'HangulSyllablePals',
u'\ubf9b': 'HangulSyllableBboed',
u'\u101a': 'MyanmarLetterYa',
u'\ufc9d': 'ArabicLigatureBehWithHahInitialForm',
u'\u5d1c': 'CjkUnifiedIdeograph5d1c',
u'\u399f': 'CjkUnifiedIdeograph399f',
u'\u66a1': 'CjkUnifiedIdeograph66a1',
u'\u8e26': 'CjkUnifiedIdeograph8e26',
u'\ucb28': 'HangulSyllableJjoek',
u'\u97ab': 'CjkUnifiedIdeograph97ab',
u'\u082a': 'UnicodeCharacter0x82a',
u'\ud4ad': 'HangulSyllablePweot',
u'\u352c': 'CjkUnifiedIdeograph352c',
u'\u11af': 'HangulJongseongRieul',
u'\u722e': 'CjkUnifiedIdeograph722e',
u'\u5eb1': 'CjkUnifiedIdeograph5eb1',
u'\ua338': 'YiSyllableSsop',
u'\u8fbb': 'CjkUnifiedIdeograph8fbb',
u'\ue03a': 'UnicodeCharacter0xe03a',
u'\uccbd': 'HangulSyllableCelg',
u'\u2d3c': 'TifinaghLetterYaf',
u'\u09bf': 'BengaliVowelSignI',
u'\u6a3e': 'CjkUnifiedIdeograph6a3e',
u'\u36c1': 'CjkUnifiedIdeograph36c1',
u'\u73c3': 'CjkUnifiedIdeograph73c3',
u'\u9b48': 'CjkUnifiedIdeograph9b48',
u'\ud84a': 'UnicodeCharacter0xd84a',
u'\ua4cd': 'UnicodeCharacter0xa4cd',
u'\u054c': 'ArmenianCapitalLetterRa',
u'\ue1cf': 'UnicodeCharacter0xe1cf',
u'\u424e': 'CjkUnifiedIdeograph424e',
u'\u2ed1': 'CjkRadicalLongOne',
u'\u6bd3': 'CjkUnifiedIdeograph6bd3',
u'\ub05a': 'HangulSyllableGgeuj',
u'\u9cdd': 'CjkUnifiedIdeograph9cdd',
u'\ufd5c': 'ArabicLigatureSeenWithHahWithJeemInitialForm',
u'\ud9df': 'UnicodeCharacter0xd9df',
u'\u3a5e': 'CjkUnifiedIdeograph3a5e',
u'\u06e1': 'ArabicSmallHighDotlessHeadOfKhah',
u'\u6760': 'CjkUnifiedIdeograph6760',
u'\u43e3': 'CjkUnifiedIdeograph43e3',
u'\ua86a': 'PhagspaLetterTtha',
u'\ud56c': 'HangulSyllableHass',
u'\ub1ef': 'HangulSyllableNyod',
u'\u126e': 'EthiopicSyllableVo',
u'\ufef1': 'ArabicLetterYehIsolatedForm',
u'\u5f70': 'CjkUnifiedIdeograph5f70',
u'\u3bf3': 'CjkUnifiedIdeograph3bf3',
u'\u78f5': 'CjkUnifiedIdeograph78f5',
u'\u807a': 'CjkUnifiedIdeograph807a',
u'\ucd7c': 'HangulSyllableCyon',
u'\ua9ff': 'UnicodeCharacter0xa9ff',
u'\u0a7e': 'UnicodeCharacter0xa7e',
u'\u3780': 'CjkUnifiedIdeograph3780',
u'\u9303': 'CjkUnifiedIdeograph9303',
u'\u7482': 'CjkUnifiedIdeograph7482',
u'\ud005': 'HangulSyllableKweolg',
u'\u1d07': 'LatinLetterSmallCapitalE',
u'\u5a09': 'CjkUnifiedIdeograph5a09',
u'\ua58c': 'VaiSyllableJoo',
u'\ue28e': 'UnicodeCharacter0xe28e',
u'\u2f90': 'KangxiRadicalClothes',
u'\u8b13': 'CjkUnifiedIdeograph8b13',
u'\u6c92': 'CjkUnifiedIdeograph6c92',
u'\uc815': 'HangulSyllableJeong',
u'\uf517': 'UnicodeCharacter0xf517',
u'\u3219': 'ParenthesizedHangulThieuthA',
u'\u7f1b': 'CjkUnifiedIdeograph7f1b',
u'\u9d9c': 'CjkUnifiedIdeograph9d9c',
u'\uda9e': 'UnicodeCharacter0xda9e',
u'\u07a0': 'ThaanaLetterTo',
u'\u44a2': 'CjkUnifiedIdeograph44a2',
u'\ua025': 'YiSyllableBot',
u'\ued27': 'UnicodeCharacter0xed27',
u'\u2a29': 'MinusSignWithCommaAbove',
u'\u572b': 'CjkUnifiedIdeograph572b',
u'\ub2ae': 'HangulSyllableNyigg',
u'\uffb0': 'HalfwidthHangulLetterRieulhieuh',
u'\u3cb2': 'CjkUnifiedIdeograph3cb2',
u'\u9835': 'CjkUnifiedIdeograph9835',
u'\u79b4': 'CjkUnifiedIdeograph79b4',
u'\uc537': 'HangulSyllableSsilh',
u'\u0239': 'LatinSmallLetterQpDigraph',
u'\u4f3b': 'CjkUnifiedIdeograph4f3b',
u'\uaabe': 'UnicodeCharacter0xaabe',
u'\ud7c0': 'UnicodeCharacter0xd7c0',
u'\u14c2': 'CanadianSyllabicsNi',
u'\u51c4': 'CjkUnifiedIdeograph51c4',
u'\ubd47': 'HangulSyllableBwaeh',
u'\ufa49': 'CjkCompatibilityIdeographfa49',
u'\u274b': 'HeavyEightTeardropspokedPropellerAsterisk',
u'\u644d': 'CjkUnifiedIdeograph644d',
u'\u82ce': 'CjkUnifiedIdeograph82ce',
u'\ucfd0': 'HangulSyllableKyols',
u'\u0cd2': 'UnicodeCharacter0xcd2',
u'\u49d4': 'CjkUnifiedIdeograph49d4',
u'\u9557': 'CjkUnifiedIdeograph9557',
u'\u76d6': 'CjkUnifiedIdeograph76d6',
u'\ud259': 'HangulSyllableTweob',
u'\u1f5b': 'GreekCapitalLetterUpsilonWithDasiaAndVaria',
u'\u5c5d': 'CjkUnifiedIdeograph5c5d',
u'\ua7e0': 'UnicodeCharacter0xa7e0',
u'\ue4e2': 'UnicodeCharacter0xe4e2',
u'\u21e4': 'LeftwardsArrowToBar',
u'\u8d67': 'CjkUnifiedIdeograph8d67',
u'\u6ee6': 'CjkUnifiedIdeograph6ee6',
u'\uca69': 'HangulSyllableJjeg',
u'\uf76b': 'UnicodeCharacter0xf76b',
u'\u346d': 'CjkUnifiedIdeograph346d',
u'\u716f': 'CjkUnifiedIdeograph716f',
u'\u9ff0': 'UnicodeCharacter0x9ff0',
u'\udcf2': 'UnicodeCharacter0xdcf2',
u'\u19f4': 'KhmerSymbolBuonRoc',
u'\u46f6': 'CjkUnifiedIdeograph46f6',
u'\ua279': 'YiSyllableHot',
u'\uef7b': 'UnicodeCharacter0xef7b',
u'\u2c7d': 'ModifierLetterCapitalV',
u'\u697f': 'CjkUnifiedIdeograph697f',
u'\u1981': 'NewTaiLueLetterLowQa',
u'\ufe04': 'VariationSelector5',
u'\ud287': 'HangulSyllableTwid',
u'\u7408': 'CjkUnifiedIdeograph7408',
u'\u488b': 'CjkUnifiedIdeograph488b',
u'\ub70a': 'HangulSyllableDdwip',
u'\u8b8d': 'CjkUnifiedIdeograph8b8d',
u'\u2d0e': 'GeorgianSmallLetterPar',
u'\u0191': 'LatinCapitalLetterFWithHook',
u'\ue614': 'UnicodeCharacter0xe614',
u'\uba97': 'HangulSyllableMyelb',
u'\u5c18': 'CjkUnifiedIdeograph5c18',
u'\u309b': 'KatakanahiraganaVoicedSoundMark',
u'\u9f1a': 'CjkUnifiedIdeograph9f1a',
u'\u151e': 'CanadianSyllabicsWestcreeShwo',
u'\ue9a1': 'UnicodeCharacter0xe9a1',
u'\u6fa5': 'CjkUnifiedIdeograph6fa5',
u'\uce24': 'HangulSyllableCeun',
u'\ua2a7': 'YiSyllableZur',
u'\u4428': 'CjkUnifiedIdeograph4428',
u'\u18ab': 'UnicodeCharacter0x18ab',
u'\u872a': 'CjkUnifiedIdeograph872a',
u'\ufd2e': 'ArabicLigatureSheenWithHahInitialForm',
u'\ud1b1': 'HangulSyllableTob',
u'\u7332': 'CjkUnifiedIdeograph7332',
u'\u57b5': 'CjkUnifiedIdeograph57b5',
u'\ub634': 'HangulSyllableDdwal',
u'\u8ab7': 'CjkUnifiedIdeograph8ab7',
u'\u2c38': 'GlagoliticSmallLetterZemlja',
u'\xbb': 'RightpointingDoubleAngleQuotationMark',
u'\ue53e': 'UnicodeCharacter0xe53e',
u'\ub9c1': 'HangulSyllableRing',
u'\u5b42': 'CjkUnifiedIdeograph5b42',
u'\u3fc5': 'CjkUnifiedIdeograph3fc5',
u'\u9e44': 'CjkUnifiedIdeograph9e44',
u'\u1448': 'CanadianSyllabicsYcreePwaa',
u'\ue8cb': 'UnicodeCharacter0xe8cb',
u'\u6ecf': 'CjkUnifiedIdeograph6ecf',
u'\ucd4e': 'HangulSyllableCwaelp',
u'\ua1d1': 'YiSyllableLup',
u'\u4352': 'CjkUnifiedIdeograph4352',
u'\u27d5': 'LeftOuterJoin',
u'\u8654': 'CjkUnifiedIdeograph8654',
u'\ufc58': 'ArabicLigatureYehWithMeemIsolatedForm',
u'\ud0db': 'HangulSyllableTah',
u'\u725c': 'CjkUnifiedIdeograph725c',
u'\u56df': 'CjkUnifiedIdeograph56df',
u'\ub55e': 'HangulSyllableDdaebs',
u'\u89e1': 'CjkUnifiedIdeograph89e1',
u'\u2b62': 'UnicodeCharacter0x2b62',
u'\u0fe5': 'UnicodeCharacter0xfe5',
u'\ue468': 'UnicodeCharacter0xe468',
u'\ub8eb': 'HangulSyllableRugs',
u'\u5a6c': 'CjkUnifiedIdeograph5a6c',
u'\u3eef': 'CjkUnifiedIdeograph3eef',
u'\u9d6e': 'CjkUnifiedIdeograph9d6e',
u'\u1372': 'EthiopicNumberTen',
u'\uf7f5': 'UnicodeCharacter0xf7f5',
u'\u6df9': 'CjkUnifiedIdeograph6df9',
u'\ucc78': 'HangulSyllableCyak',
u'\ua0fb': 'YiSyllableVyx',
u'\u427c': 'CjkUnifiedIdeograph427c',
u'\u26ff': 'UnicodeCharacter0x26ff',
u'\u857e': 'CjkUnifiedIdeograph857e',
u'\u1c03': 'LepchaLetterGa',
u'\ufb82': 'ArabicLetterDdahalIsolatedForm',
u'\u7186': 'CjkUnifiedIdeograph7186',
u'\ud509': 'HangulSyllablePeunj',
u'\ub488': 'HangulSyllableDwe',
u'\u4b0d': 'CjkUnifiedIdeograph4b0d',
u'\u2a8c': 'GreaterthanAboveDoublelineEqualAboveLessthan',
u'\u8e0f': 'CjkUnifiedIdeograph8e0f',
u'\u0413': 'CyrillicCapitalLetterGhe',
u'\ue392': 'UnicodeCharacter0xe392',
u'\u7a17': 'CjkUnifiedIdeograph7a17',
u'\u5996': 'CjkUnifiedIdeograph5996',
u'\ubd19': 'HangulSyllableBwalg',
u'\u9c98': 'CjkUnifiedIdeograph9c98',
u'\u331d': 'SquareKoruna',
u'\u129c': 'EthiopicSyllableNyee',
u'\uec23': 'UnicodeCharacter0xec23',
u'\ucba2': 'HangulSyllableJjwinh',
u'\u6227': 'CjkUnifiedIdeograph6227',
u'\u41a6': 'CjkUnifiedIdeograph41a6',
u'\ua529': 'VaiSyllableWin',
u'\u84a8': 'CjkUnifiedIdeograph84a8',
u'\u1b2d': 'BalineseLetterRa',
u'\ufaac': 'CjkCompatibilityIdeographfaac',
u'\u70b0': 'CjkUnifiedIdeograph70b0',
u'\ud433': 'HangulSyllablePwaelh',
u'\ub3b2': 'HangulSyllableDyelm',
u'\u4a37': 'CjkUnifiedIdeograph4a37',
u'\u29b6': 'CircledVerticalBar',
u'\u8d39': 'CjkUnifiedIdeograph8d39',
u'\u033d': 'CombiningXAbove',
u'\ue2bc': 'UnicodeCharacter0xe2bc',
u'\u7941': 'CjkUnifiedIdeograph7941',
u'\u58c0': 'CjkUnifiedIdeograph58c0',
u'\ubc43': 'HangulSyllableBaes',
u'\u9bc2': 'CjkUnifiedIdeograph9bc2',
u'\u3247': 'UnicodeCharacter0x3247',
u'\u11c6': 'HangulJongseongNieuntikeut',
u'\ueb4d': 'UnicodeCharacter0xeb4d',
u'\ucacc': 'HangulSyllableJjom',
u'\u6151': 'CjkUnifiedIdeograph6151',
u'\u40d0': 'CjkUnifiedIdeograph40d0',
u'\ua453': 'YiSyllableNyuo',
u'\u83d2': 'CjkUnifiedIdeograph83d2',
u'\u1a57': 'UnicodeCharacter0x1a57',
u'\uf9d6': 'CjkCompatibilityIdeographf9d6',
u'\u7fda': 'CjkUnifiedIdeograph7fda',
u'\ud35d': 'HangulSyllablePyat',
u'\ub2dc': 'HangulSyllableNiss',
u'\u4961': 'CjkUnifiedIdeograph4961',
u'\u28e0': 'BraillePatternDots678',
u'\u8c63': 'CjkUnifiedIdeograph8c63',
u'\u0267': 'LatinSmallLetterHengWithHook',
u'\ue1e6': 'UnicodeCharacter0xe1e6',
u'\u786b': 'CjkUnifiedIdeograph786b',
u'\u67ea': 'CjkUnifiedIdeograph67ea',
u'\ubb6d': 'HangulSyllableMweg',
u'\u9aec': 'CjkUnifiedIdeograph9aec',
u'\u3171': 'HangulLetterKapyeounmieum',
u'\u10f0': 'GeorgianLetterHae',
u'\uea77': 'UnicodeCharacter0xea77',
u'\uc9f6': 'HangulSyllableJjap',
u'\u607b': 'CjkUnifiedIdeograph607b',
u'\u4ffa': 'CjkUnifiedIdeograph4ffa',
u'\ua37d': 'YiSyllableRra',
u'\u82fc': 'CjkUnifiedIdeograph82fc',
u'\u9881': 'CjkUnifiedIdeograph9881',
u'\uf900': 'CjkCompatibilityIdeographf900',
u'\udd83': 'UnicodeCharacter0xdd83',
u'\u3a02': 'CjkUnifiedIdeograph3a02',
u'\u1e85': 'LatinSmallLetterWWithDiaeresis',
u'\u7f04': 'CjkUnifiedIdeograph7f04',
u'\u5387': 'CjkUnifiedIdeograph5387',
u'\uac0e': 'HangulSyllableGalp',
u'\u8091': 'CjkUnifiedIdeograph8091',
u'\ue110': 'UnicodeCharacter0xe110',
u'\uc593': 'HangulSyllableYac',
u'\u2212': 'MinusSign',
u'\u0695': 'ArabicLetterRehWithSmallVBelow',
u'\u6714': 'CjkUnifiedIdeograph6714',
u'\u3b97': 'CjkUnifiedIdeograph3b97',
u'\u7c99': 'CjkUnifiedIdeograph7c99',
u'\u941e': 'CjkUnifiedIdeograph941e',
u'\uc920': 'HangulSyllableJweol',
u'\uada3': 'HangulSyllableGweoh',
u'\u0a22': 'GurmukhiLetterDdha',
u'\ueea5': 'UnicodeCharacter0xeea5',
u'\u4f24': 'CjkUnifiedIdeograph4f24',
u'\u23a7': 'LeftCurlyBracketUpperHook',
u'\u64a9': 'CjkUnifiedIdeograph64a9',
u'\ub130': 'HangulSyllableNels',
u'\u95b3': 'CjkUnifiedIdeograph95b3',
u'\uf232': 'UnicodeCharacter0xf232',
u'\ud6b5': 'HangulSyllableHyolt',
u'\u3734': 'CjkUnifiedIdeograph3734',
u'\u0bb7': 'TamilLetterSsa',
u'\u6836': 'CjkUnifiedIdeograph6836',
u'\u4cb9': 'CjkUnifiedIdeograph4cb9',
u'\u9940': 'CjkUnifiedIdeograph9940',
u'\uda42': 'UnicodeCharacter0xda42',
u'\ubec5': 'HangulSyllableBbyaeb',
u'\u1f44': 'GreekSmallLetterOmicronWithPsiliAndOxia',
u'\uf3c7': 'UnicodeCharacter0xf3c7',
u'\u5046': 'CjkUnifiedIdeograph5046',
u'\u34c9': 'CjkUnifiedIdeograph34c9',
u'\u69cb': 'CjkUnifiedIdeograph69cb',
u'\u8150': 'CjkUnifiedIdeograph8150',
u'\uc252': 'HangulSyllableSwegg',
u'\ua6d5': 'UnicodeCharacter0xa6d5',
u'\u0754': 'ArabicLetterBehWithTwoDotsBelowAndDotAbove',
u'\udbd7': 'UnicodeCharacter0xdbd7',
u'\u3856': 'CjkUnifiedIdeograph3856',
u'\u1cd9': 'UnicodeCharacter0x1cd9',
u'\u7d58': 'CjkUnifiedIdeograph7d58',
u'\u51db': 'CjkUnifiedIdeograph51db',
u'\uaa62': 'UnicodeCharacter0xaa62',
u'\u8ee5': 'CjkUnifiedIdeograph8ee5',
u'\uef64': 'UnicodeCharacter0xef64',
u'\uc3e7': 'HangulSyllableSsolh',
u'\u2066': 'UnicodeCharacter0x2066',
u'\u04e9': 'CyrillicSmallLetterBarredO',
u'\u6568': 'CjkUnifiedIdeograph6568',
u'\u39eb': 'CjkUnifiedIdeograph39eb',
u'\u7aed': 'CjkUnifiedIdeograph7aed',
u'\u9272': 'CjkUnifiedIdeograph9272',
u'\ud774': 'HangulSyllableHyil',
u'\uabf7': 'UnicodeCharacter0xabf7',
u'\u0876': 'UnicodeCharacter0x876',
u'\uecf9': 'UnicodeCharacter0xecf9',
u'\u4d78': 'CjkUnifiedIdeograph4d78',
u'\u21fb': 'RightwardsArrowWithDoubleVerticalStroke',
u'\u62fd': 'CjkUnifiedIdeograph62fd',
u'\u5801': 'CjkUnifiedIdeograph5801',
u'\ubf84': 'HangulSyllableBbwaels',
u'\uf086': 'UnicodeCharacter0xf086',
u'\u3588': 'CjkUnifiedIdeograph3588',
u'\u890b': 'CjkUnifiedIdeograph890b',
u'\u768a': 'CjkUnifiedIdeograph768a',
u'\uca0d': 'HangulSyllableJjaeng',
u'\u0f0f': 'TibetanMarkTshegShad',
u'\u4011': 'CjkUnifiedIdeograph4011',
u'\ua794': 'UnicodeCharacter0xa794',
u'\ud896': 'UnicodeCharacter0xd896',
u'\u1d98': 'LatinSmallLetterEshWithRetroflexHook',
u'\u5e9a': 'CjkUnifiedIdeograph5e9a',
u'\ub21d': 'HangulSyllableNut',
u'\uf71f': 'UnicodeCharacter0xf71f',
u'\u2821': 'BraillePatternDots16',
u'\u6d23': 'CjkUnifiedIdeograph6d23',
u'\u8fa4': 'CjkUnifiedIdeograph8fa4',
u'\uc0a6': 'HangulSyllableBbij',
u'\u05a8': 'HebrewAccentQadma',
u'\u46aa': 'CjkUnifiedIdeograph46aa',
u'\u9a2d': 'CjkUnifiedIdeograph9a2d',
u'\u7bac': 'CjkUnifiedIdeograph7bac',
u'\udf2f': 'UnicodeCharacter0xdf2f',
u'\u1031': 'MyanmarVowelSignE',
u'\u5533': 'CjkUnifiedIdeograph5533',
u'\ua8b6': 'SaurashtraVowelSignI',
u'\uedb8': 'UnicodeCharacter0xedb8',
u'\u2eba': 'CjkRadicalBrushOne',
u'\u823d': 'CjkUnifiedIdeograph823d',
u'\u63bc': 'CjkUnifiedIdeograph63bc',
u'\uc73f': 'HangulSyllableEugs',
u'\uf841': 'UnicodeCharacter0xf841',
u'\u3d43': 'CjkUnifiedIdeograph3d43',
u'\u7e45': 'CjkUnifiedIdeograph7e45',
u'\u90c6': 'CjkUnifiedIdeograph90c6',
u'\ud5c8': 'HangulSyllableHeo',
u'\u16ca': 'RunicLetterSowiloS',
u'\u4bcc': 'CjkUnifiedIdeograph4bcc',
u'\uaf4f': 'HangulSyllableGgwad',
u'\ue051': 'UnicodeCharacter0xe051',
u'\u2553': 'BoxDrawingsDownDoubleAndRightSingle',
u'\u6655': 'CjkUnifiedIdeograph6655',
u'\ubdd8': 'HangulSyllableBwin',
u'\ufeda': 'ArabicLetterKafFinalForm',
u'\u33dc': 'SquareSv',
u'\u975f': 'CjkUnifiedIdeograph975f',
u'\u74de': 'CjkUnifiedIdeograph74de',
u'\uc861': 'HangulSyllableJyelt',
u'\u0d63': 'MalayalamVowelSignVocalicLl',
u'\u4e65': 'CjkUnifiedIdeograph4e65',
u'\ua5e8': 'VaiSyllablePe',
u'\ue6ea': 'UnicodeCharacter0xe6ea',
u'\u1bec': 'UnicodeCharacter0x1bec',
u'\u5cee': 'CjkUnifiedIdeograph5cee',
u'\ub071': 'HangulSyllableGgyib',
u'\uf573': 'UnicodeCharacter0xf573',
u'\u3675': 'CjkUnifiedIdeograph3675',
u'\u6b77': 'CjkUnifiedIdeograph6b77',
u'\u8df8': 'CjkUnifiedIdeograph8df8',
u'\ucefa': 'HangulSyllableKeoj',
u'\u03fc': 'GreekRhoWithStrokeSymbol',
u'\u44fe': 'CjkUnifiedIdeograph44fe',
u'\u7a00': 'CjkUnifiedIdeograph7a00',
u'\u5e83': 'CjkUnifiedIdeograph5e83',
u'\ub902': 'HangulSyllableRup',
u'\u9d85': 'CjkUnifiedIdeograph9d85',
u'\u3306': 'SquareUon',
u'\u1789': 'KhmerLetterNyo',
u'\ue80c': 'UnicodeCharacter0xe80c',
u'\ucc8f': 'HangulSyllableCyaes',
u'\u6210': 'CjkUnifiedIdeograph6210',
u'\u4693': 'CjkUnifiedIdeograph4693',
u'\ua112': 'YiSyllableDe',
u'\u8595': 'CjkUnifiedIdeograph8595',
u'\u1b16': 'BalineseLetterGaGora',
u'\uff99': 'HalfwidthKatakanaLetterRu',
u'\u719d': 'CjkUnifiedIdeograph719d',
u'\ud01c': 'HangulSyllableKwen',
u'\ub49f': 'HangulSyllableDwec',
u'\u4a20': 'CjkUnifiedIdeograph4a20',
u'\u2ea3': 'CjkRadicalFire',
u'\u8922': 'CjkUnifiedIdeograph8922',
u'\u0326': 'CombiningCommaBelow',
u'\ue7a9': 'UnicodeCharacter0xe7a9',
u'\u59ad': 'CjkUnifiedIdeograph59ad',
u'\ub82c': 'HangulSyllableRyeol',
u'\u9caf': 'CjkUnifiedIdeograph9caf',
u'\u3230': 'ParenthesizedIdeographSun',
u'\u16b3': 'RunicLetterCen',
u'\ueb36': 'UnicodeCharacter0xeb36',
u'\ucfb9': 'HangulSyllableKoeb',
u'\u6d3a': 'CjkUnifiedIdeograph6d3a',
u'\u41bd': 'CjkUnifiedIdeograph41bd',
u'\ua03c': 'YiSyllablePiex',
u'\u84bf': 'CjkUnifiedIdeograph84bf',
u'\u1a40': 'UnicodeCharacter0x1a40',
u'\ufec3': 'ArabicLetterTahInitialForm',
u'\u70c7': 'CjkUnifiedIdeograph70c7',
u'\ud346': 'HangulSyllablePyagg',
u'\ub7c9': 'HangulSyllableRyang',
u'\u554a': 'CjkUnifiedIdeograph554a',
u'\u29cd': 'TriangleWithSerifsAtBottom',
u'\u884c': 'CjkUnifiedIdeograph884c',
u'\u0250': 'LatinSmallLetterTurnedA',
u'\ue6d3': 'UnicodeCharacter0xe6d3',
u'\u58d7': 'CjkUnifiedIdeograph58d7',
u'\ubb56': 'HangulSyllableMweonh',
u'\u9fd9': 'UnicodeCharacter0x9fd9',
u'\u3d5a': 'CjkUnifiedIdeograph3d5a',
u'\u11dd': 'HangulJongseongMieumsios',
u'\uea60': 'UnicodeCharacter0xea60',
u'\ucee3': 'HangulSyllableKyaeh',
u'\u6c64': 'CjkUnifiedIdeograph6c64',
u'\u40e7': 'CjkUnifiedIdeograph40e7',
u'\ua366': 'YiSyllableChuox',
u'\u87e9': 'CjkUnifiedIdeograph87e9',
u'\u256a': 'BoxDrawingsVerticalSingleAndHorizontalDouble',
u'\uf9ed': 'CjkCompatibilityIdeographf9ed',
u'\u73f1': 'CjkUnifiedIdeograph73f1',
u'\ud270': 'HangulSyllableTwels',
u'\ub6f3': 'HangulSyllableDdwigs',
u'\u5474': 'CjkUnifiedIdeograph5474',
u'\u28f7': 'BraillePatternDots1235678',
u'\u8b76': 'CjkUnifiedIdeograph8b76',
u'\u0d7a': 'MalayalamLetterChilluNn',
u'\ue1fd': 'UnicodeCharacter0xe1fd',
u'\udb01': 'UnicodeCharacter0xdb01',
u'\uba80': 'HangulSyllableMyeom',
u'\u5d05': 'CjkUnifiedIdeograph5d05',
u'\u3c84': 'CjkUnifiedIdeograph3c84',
u'\u9007': 'CjkUnifiedIdeograph9007',
u'\u0a0b': 'UnicodeCharacter0xa0b',
u'\uf58a': 'UnicodeCharacter0xf58a',
u'\u6f8e': 'CjkUnifiedIdeograph6f8e',
u'\uc311': 'HangulSyllableSsat',
u'\ua290': 'YiSyllableZip',
u'\u4515': 'CjkUnifiedIdeograph4515',
u'\u2494': 'NumberThirteenFullStop',
u'\uf21b': 'UnicodeCharacter0xf21b',
u'\udd9a': 'UnicodeCharacter0xdd9a',
u'\u741f': 'CjkUnifiedIdeograph741f',
u'\u579e': 'CjkUnifiedIdeograph579e',
u'\uab21': 'UnicodeCharacter0xab21',
u'\u8aa0': 'CjkUnifiedIdeograph8aa0',
u'\u2d25': 'GeorgianSmallLetterHoe',
u'\u0ca4': 'KannadaLetterTa',
u'\uda2b': 'UnicodeCharacter0xda2b',
u'\uc5aa': 'HangulSyllableYaebs',
u'\u5c2f': 'CjkUnifiedIdeograph5c2f',
u'\u3fae': 'CjkUnifiedIdeograph3fae',
u'\u9331': 'CjkUnifiedIdeograph9331',
u'\u1535': 'CanadianSyllabicsYwo',
u'\uf4b4': 'UnicodeCharacter0xf4b4',
u'\u6eb8': 'CjkUnifiedIdeograph6eb8',
u'\uc23b': 'HangulSyllableSweod',
u'\uadba': 'HangulSyllableGwej',
u'\u443f': 'CjkUnifiedIdeograph443f',
u'\u27be': 'OpenoutlinedRightwardsArrow',
u'\ufd45': 'UnicodeCharacter0xfd45',
u'\udcc4': 'UnicodeCharacter0xdcc4',
u'\u7749': 'CjkUnifiedIdeograph7749',
u'\u56c8': 'CjkUnifiedIdeograph56c8',
u'\uaa4b': 'ChamLetterFinalSs',
u'\u95ca': 'CjkUnifiedIdeograph95ca',
u'\u2c4f': 'GlagoliticSmallLetterYeru',
u'\u0fce': 'TibetanSignRdelNagRdelDkar',
u'\ue555': 'UnicodeCharacter0xe555',
u'\uc4d4': 'HangulSyllableSsyu',
u'\u5f59': 'CjkUnifiedIdeograph5f59',
u'\u3ed8': 'CjkUnifiedIdeograph3ed8',
u'\u925b': 'CjkUnifiedIdeograph925b',
u'\u145f': 'CanadianSyllabicsTwoo',
u'\uf7de': 'UnicodeCharacter0xf7de',
u'\u69e2': 'CjkUnifiedIdeograph69e2',
u'\ucd65': 'HangulSyllableCoelg',
u'\uace4': 'HangulSyllableGon',
u'\u4769': 'CjkUnifiedIdeograph4769',
u'\u26e8': 'UnicodeCharacter0x26e8',
u'\ufc6f': 'ArabicLigatureBehWithYehFinalForm',
u'\udfee': 'UnicodeCharacter0xdfee',
u'\u7673': 'CjkUnifiedIdeograph7673',
u'\u51f2': 'CjkUnifiedIdeograph51f2',
u'\ub575': 'HangulSyllableDdyalt',
u'\u94f4': 'CjkUnifiedIdeograph94f4',
u'\u2f79': 'KangxiRadicalNet',
u'\u0ef8': 'UnicodeCharacter0xef8',
u'\ue47f': 'UnicodeCharacter0xe47f',
u'\uc7fe': 'HangulSyllableJyaep',
u'\ub206': 'HangulSyllableNugg',
u'\u9689': 'CjkUnifiedIdeograph9689',
u'\uf708': 'UnicodeCharacter0xf708',
u'\ucb8b': 'HangulSyllableJjwelb',
u'\u340a': 'CjkUnifiedIdeograph340a',
u'\u088d': 'UnicodeCharacter0x88d',
u'\u690c': 'CjkUnifiedIdeograph690c',
u'\u4d8f': 'CjkUnifiedIdeograph4d8f',
u'\u9a16': 'CjkUnifiedIdeograph9a16',
u'\udf18': 'UnicodeCharacter0xdf18',
u'\ub39b': 'HangulSyllableDyeolh',
u'\u1c1a': 'LepchaLetterYa',
u'\uf09d': 'UnicodeCharacter0xf09d',
u'\u511c': 'CjkUnifiedIdeograph511c',
u'\u359f': 'CjkUnifiedIdeograph359f',
u'\u6aa1': 'CjkUnifiedIdeograph6aa1',
u'\u8226': 'CjkUnifiedIdeograph8226',
u'\uc728': 'HangulSyllableYul',
u'\u9bab': 'CjkUnifiedIdeograph9bab',
u'\u042a': 'CyrillicCapitalLetterHardSign',
u'\ud8ad': 'UnicodeCharacter0xd8ad',
u'\u392c': 'CjkUnifiedIdeograph392c',
u'\u1daf': 'ModifierLetterSmallNWithRetroflexHook',
u'\u7e2e': 'CjkUnifiedIdeograph7e2e',
u'\u52b1': 'CjkUnifiedIdeograph52b1',
u'\uaf38': 'HangulSyllableGgols',
u'\u83bb': 'CjkUnifiedIdeograph83bb',
u'\uec3a': 'UnicodeCharacter0xec3a',
u'\uc0bd': 'HangulSyllableSab',
u'\u213c': 'DoublestruckSmallPi',
u'\u05bf': 'HebrewPointRafe',
u'\u663e': 'CjkUnifiedIdeograph663e',
u'\u3ac1': 'CjkUnifiedIdeograph3ac1',
u'\u7fc3': 'CjkUnifiedIdeograph7fc3',
u'\u9748': 'CjkUnifiedIdeograph9748',
u'\ud44a': 'HangulSyllablePoelm',
u'\ua8cd': 'UnicodeCharacter0xa8cd',
u'\u094c': 'DevanagariVowelSignAu',
u'\uedcf': 'UnicodeCharacter0xedcf',
u'\u4e4e': 'CjkUnifiedIdeograph4e4e',
u'\u22d1': 'DoubleSuperset',
u'\u67d3': 'CjkUnifiedIdeograph67d3',
u'\ubc5a': 'HangulSyllableByalp',
u'\u90dd': 'CjkUnifiedIdeograph90dd',
u'\uf15c': 'UnicodeCharacter0xf15c',
u'\ud5df': 'HangulSyllableHeoc',
u'\u365e': 'CjkUnifiedIdeograph365e',
u'\u0ae1': 'GujaratiLetterVocalicLl',
u'\u6b60': 'CjkUnifiedIdeograph6b60',
u'\u4fe3': 'CjkUnifiedIdeograph4fe3',
u'\ua46a': 'YiSyllableXop',
u'\ud96c': 'UnicodeCharacter0xd96c',
u'\ubdef': 'HangulSyllableBwih',
u'\u1e6e': 'LatinCapitalLetterTWithLineBelow',
u'\uf2f1': 'UnicodeCharacter0xf2f1',
u'\u5370': 'CjkUnifiedIdeograph5370',
u'\u37f3': 'CjkUnifiedIdeograph37f3',
u'\u74f5': 'CjkUnifiedIdeograph74f5',
u'\u8c7a': 'CjkUnifiedIdeograph8c7a',
u'\uc17c': 'HangulSyllableSyels',
u'\ua5ff': 'VaiSyllableCe',
u'\u067e': 'ArabicLetterPeh',
u'\u3b80': 'CjkUnifiedIdeograph3b80',
u'\u9f03': 'CjkUnifiedIdeograph9f03',
u'\u7882': 'CjkUnifiedIdeograph7882',
u'\udc05': 'UnicodeCharacter0xdc05',
u'\u1107': 'HangulChoseongPieup',
u'\u5609': 'CjkUnifiedIdeograph5609',
u'\ua98c': 'UnicodeCharacter0xa98c',
u'\uee8e': 'UnicodeCharacter0xee8e',
u'\u2390': 'OpencircuitoutputLtypeSymbol',
u'\u8713': 'CjkUnifiedIdeograph8713',
u'\u6092': 'CjkUnifiedIdeograph6092',
u'\uc415': 'HangulSyllableSswaenj',
u'\uf917': 'CjkCompatibilityIdeographf917',
u'\u3e19': 'CjkUnifiedIdeograph3e19',
u'\u731b': 'CjkUnifiedIdeograph731b',
u'\u919c': 'CjkUnifiedIdeograph919c',
u'\ud69e': 'HangulSyllableHoebs',
u'\u0ba0': 'UnicodeCharacter0xba0',
u'\u48a2': 'CjkUnifiedIdeograph48a2',
u'\uac25': 'HangulSyllableGaelg',
u'\ue127': 'UnicodeCharacter0xe127',
u'\u2629': 'CrossOfJerusalem',
u'\u5b2b': 'CjkUnifiedIdeograph5b2b',
u'\ubeae': 'HangulSyllableBbyaj',
u'\uf3b0': 'UnicodeCharacter0xf3b0',
u'\u30b2': 'KatakanaLetterGe',
u'\u9435': 'CjkUnifiedIdeograph9435',
u'\u75b4': 'CjkUnifiedIdeograph75b4',
u'\uc937': 'HangulSyllableJwegs',
u'\u0e39': 'ThaiCharacterSaraUu',
u'\u433b': 'CjkUnifiedIdeograph433b',
u'\ua6be': 'UnicodeCharacter0xa6be',
u'\udbc0': 'UnicodeCharacter0xdbc0',
u'\u18c2': 'UnicodeCharacter0x18c2',
u'\u5dc4': 'CjkUnifiedIdeograph5dc4',
u'\ub147': 'HangulSyllableNyeod',
u'\uf649': 'UnicodeCharacter0xf649',
u'\u2b4b': 'LeftwardsArrowAboveReverseTildeOperator',
u'\u684d': 'CjkUnifiedIdeograph684d',
u'\u8ece': 'CjkUnifiedIdeograph8ece',
u'\uc3d0': 'HangulSyllableSsyess',
u'\xd2': 'LatinCapitalLetterOWithGrave',
u'\u45d4': 'CjkUnifiedIdeograph45d4',
u'\u9957': 'CjkUnifiedIdeograph9957',
u'\u7ad6': 'CjkUnifiedIdeograph7ad6',
u'\ude59': 'UnicodeCharacter0xde59',
u'\u135b': 'UnicodeCharacter0x135b',
u'\u505d': 'CjkUnifiedIdeograph505d',
u'\uabe0': 'UnicodeCharacter0xabe0',
u'\ue8e2': 'UnicodeCharacter0xe8e2',
u'\u2de4': 'CombiningCyrillicLetterZhe',
u'\u8167': 'CjkUnifiedIdeograph8167',
u'\u62e6': 'CjkUnifiedIdeograph62e6',
u'\uc669': 'HangulSyllableWaelt',
u'\ufb6b': 'ArabicLetterVehFinalForm',
u'\u386d': 'CjkUnifiedIdeograph386d',
u'\u7d6f': 'CjkUnifiedIdeograph7d6f',
u'\u93f0': 'CjkUnifiedIdeograph93f0',
u'\ud0f2': 'HangulSyllableTaej',
u'\u15f4': 'CanadianSyllabicsCarrierGa',
u'\u4af6': 'CjkUnifiedIdeograph4af6',
u'\uae79': 'HangulSyllableGgaeb',
u'\ue37b': 'UnicodeCharacter0xe37b',
u'\u207d': 'SuperscriptLeftParenthesis',
u'\u657f': 'CjkUnifiedIdeograph657f',
u'\u1d81': 'LatinSmallLetterDWithPalatalHook',
u'\ufa04': 'CjkCompatibilityIdeographfa04',
u'\ud687': 'HangulSyllableHwaec',
u'\u7008': 'CjkUnifiedIdeograph7008',
u'\u4c8b': 'CjkUnifiedIdeograph4c8b',
u'\ub30a': 'HangulSyllableDaelm',
u'\u8f8d': 'CjkUnifiedIdeograph8f8d',
u'\u290e': 'LeftwardsTripleDashArrow',
u'\u0591': 'HebrewAccentEtnahta',
u'\ue214': 'UnicodeCharacter0xe214',
u'\ube97': 'HangulSyllableBbaeh',
u'\u5818': 'CjkUnifiedIdeograph5818',
u'\u349b': 'CjkUnifiedIdeograph349b',
u'\u9b1a': 'CjkUnifiedIdeograph9b1a',
u'\u111e': 'HangulChoseongPieupkiyeok',
u'\ueda1': 'UnicodeCharacter0xeda1',
u'\u6ba5': 'CjkUnifiedIdeograph6ba5',
u'\uca24': 'HangulSyllableJjyam',
u'\ua6a7': 'UnicodeCharacter0xa6a7',
u'\u4028': 'CjkUnifiedIdeograph4028',
u'\u1cab': 'UnicodeCharacter0x1cab',
u'\u832a': 'CjkUnifiedIdeograph832a',
u'\uf92e': 'CjkCompatibilityIdeographf92e',
u'\ud5b1': 'HangulSyllableHyaenj',
u'\u7732': 'CjkUnifiedIdeograph7732',
u'\u53b5': 'CjkUnifiedIdeograph53b5',
u'\ub234': 'HangulSyllableNweoss',
u'\u8eb7': 'CjkUnifiedIdeograph8eb7',
u'\u2838': 'BraillePatternDots456',
u'\u04bb': 'CyrillicSmallLetterShha',
u'\ue13e': 'UnicodeCharacter0xe13e',
u'\ubdc1': 'HangulSyllableBwelg',
u'\u5f42': 'CjkUnifiedIdeograph5f42',
u'\u3bc5': 'CjkUnifiedIdeograph3bc5',
u'\u9a44': 'CjkUnifiedIdeograph9a44',
u'\u1048': 'MyanmarDigitEight',
u'\ueccb': 'UnicodeCharacter0xeccb',
u'\u6acf': 'CjkUnifiedIdeograph6acf',
u'\uc94e': 'HangulSyllableJwep',
u'\ua5d1': 'VaiSyllableDo',
u'\u4752': 'CjkUnifiedIdeograph4752',
u'\u23d5': 'MetricalTwoShortsOverLong',
u'\u8254': 'CjkUnifiedIdeograph8254',
u'\uf858': 'UnicodeCharacter0xf858',
u'\ud4db': 'HangulSyllablePwilh',
u'\u765c': 'CjkUnifiedIdeograph765c',
u'\u52df': 'CjkUnifiedIdeograph52df',
u'\ub15e': 'HangulSyllableNyegg',
u'\u8de1': 'CjkUnifiedIdeograph8de1',
u'\u2f62': 'KangxiRadicalSweet',
u'\u0be5': 'UnicodeCharacter0xbe5',
u'\ue068': 'UnicodeCharacter0xe068',
u'\ubceb': 'HangulSyllableByes',
u'\u5e6c': 'CjkUnifiedIdeograph5e6c',
u'\u3aef': 'CjkUnifiedIdeograph3aef',
u'\u996e': 'CjkUnifiedIdeograph996e',
u'\u1772': 'TagbanwaVowelSignI',
u'\uf3f5': 'UnicodeCharacter0xf3f5',
u'\u69f9': 'CjkUnifiedIdeograph69f9',
u'\uc878': 'HangulSyllableJol',
u'\ua4fb': 'UnicodeCharacter0xa4fb',
u'\u467c': 'CjkUnifiedIdeograph467c',
u'\u22ff': 'ZNotationBagMembership',
u'\u817e': 'CjkUnifiedIdeograph817e',
u'\u1803': 'MongolianFullStop',
u'\uff82': 'HalfwidthKatakanaLetterTu',
u'\u7586': 'CjkUnifiedIdeograph7586',
u'\ud109': 'HangulSyllableTyab',
u'\ub088': 'HangulSyllableGgils',
u'\u4f0d': 'CjkUnifiedIdeograph4f0d',
u'\u2e8c': 'CjkRadicalSmallOne',
u'\u8a0f': 'CjkUnifiedIdeograph8a0f',
u'\ue792': 'UnicodeCharacter0xe792',
u'\u7e17': 'CjkUnifiedIdeograph7e17',
u'\u5d96': 'CjkUnifiedIdeograph5d96',
u'\ub919': 'HangulSyllableRweong',
u'\u9898': 'CjkUnifiedIdeograph9898',
u'\u371d': 'CjkUnifiedIdeograph371d',
u'\u169c': 'OghamReversedFeatherMark',
u'\ue823': 'UnicodeCharacter0xe823',
u'\ucfa2': 'HangulSyllableKwaej',
u'\u6627': 'CjkUnifiedIdeograph6627',
u'\u45a6': 'CjkUnifiedIdeograph45a6',
u'\ua129': 'YiSyllableTot',
u'\u80a8': 'CjkUnifiedIdeograph80a8',
u'\u1f2d': 'GreekCapitalLetterEtaWithDasiaAndOxia',
u'\ufeac': 'ArabicLetterThalFinalForm',
u'\u74b0': 'CjkUnifiedIdeograph74b0',
u'\ud033': 'HangulSyllableKweh',
u'\ub7b2': 'HangulSyllableRaep',
u'\u4e37': 'CjkUnifiedIdeograph4e37',
u'\u2db6': 'EthiopicSyllableZzo',
u'\u8939': 'CjkUnifiedIdeograph8939',
u'\u073d': 'SyriacEsasaAbove',
u'\ue6bc': 'UnicodeCharacter0xe6bc',
u'\u7d41': 'CjkUnifiedIdeograph7d41',
u'\u5cc0': 'CjkUnifiedIdeograph5cc0',
u'\ub843': 'HangulSyllableRyegs',
u'\u9fc2': 'UnicodeCharacter0x9fc2',
u'\u3647': 'CjkUnifiedIdeograph3647',
u'\u15c6': 'CanadianSyllabicsCarrierGhe',
u'\uef4d': 'UnicodeCharacter0xef4d',
u'\ucecc': 'HangulSyllableKyaen',
u'\u6551': 'CjkUnifiedIdeograph6551',
u'\u44d0': 'CjkUnifiedIdeograph44d0',
u'\ua053': 'YiSyllablePyp',
u'\u87d2': 'CjkUnifiedIdeograph87d2',
u'\u1e57': 'LatinSmallLetterPWithDotAbove',
u'\ufdd6': 'UnicodeCharacter0xfdd6',
u'\u7bda': 'CjkUnifiedIdeograph7bda',
u'\ud75d': 'HangulSyllableHeult',
u'\ub6dc': 'HangulSyllableDdwel',
u'\u4d61': 'CjkUnifiedIdeograph4d61',
u'\u2ce0': 'CopticCapitalLetterOldNubianNyi',
u'\u8863': 'CjkUnifiedIdeograph8863',
u'\u0667': 'ArabicindicDigitSeven',
u'\ue5e6': 'UnicodeCharacter0xe5e6',
u'\u7c6b': 'CjkUnifiedIdeograph7c6b',
u'\u63ea': 'CjkUnifiedIdeograph63ea',
u'\ubf6d': 'HangulSyllableBbwab',
u'\u9eec': 'CjkUnifiedIdeograph9eec',
u'\u3571': 'CjkUnifiedIdeograph3571',
u'\u14f0': 'CanadianSyllabicsSii',
u'\uee77': 'UnicodeCharacter0xee77',
u'\ucdf6': 'HangulSyllableCwilp',
u'\u647b': 'CjkUnifiedIdeograph647b',
u'\u4bfa': 'CjkUnifiedIdeograph4bfa',
u'\ua77d': 'LatinCapitalLetterInsularG',
u'\u86fc': 'CjkUnifiedIdeograph86fc',
u'\u9c81': 'CjkUnifiedIdeograph9c81',
u'\ufd00': 'ArabicLigatureHahWithYehIsolatedForm',
u'\ud983': 'UnicodeCharacter0xd983',
u'\u3e02': 'CjkUnifiedIdeograph3e02',
u'\u1a85': 'UnicodeCharacter0x1a85',
u'\u7b04': 'CjkUnifiedIdeograph7b04',
u'\u5787': 'CjkUnifiedIdeograph5787',
u'\ua80e': 'SylotiNagriLetterJo',
u'\u8491': 'CjkUnifiedIdeograph8491',
u'\ue510': 'UnicodeCharacter0xe510',
u'\uc193': 'HangulSyllableSod',
u'\u2612': 'BallotBoxWithX',
u'\u0295': 'LatinLetterPharyngealVoicedFricative',
u'\u6314': 'CjkUnifiedIdeograph6314',
u'\u3f97': 'CjkUnifiedIdeograph3f97',
u'\u7899': 'CjkUnifiedIdeograph7899',
u'\u901e': 'CjkUnifiedIdeograph901e',
u'\ucd20': 'HangulSyllableCok',
u'\ua9a3': 'UnicodeCharacter0xa9a3',
u'\u0e22': 'ThaiCharacterYoYak',
u'\ueaa5': 'UnicodeCharacter0xeaa5',
u'\u4b24': 'CjkUnifiedIdeograph4b24',
u'\u27a7': 'SquatBlackRightwardsArrow',
u'\u60a9': 'CjkUnifiedIdeograph60a9',
u'\ub530': 'HangulSyllableDda',
u'\u91b3': 'CjkUnifiedIdeograph91b3',
u'\uf632': 'UnicodeCharacter0xf632',
u'\ud2b5': 'HangulSyllableTyut',
u'\u3334': 'SquareBussyeru',
u'\u0fb7': 'TibetanSubjoinedLetterHa',
u'\u6c36': 'CjkUnifiedIdeograph6c36',
u'\u48b9': 'CjkUnifiedIdeograph48b9',
u'\u9d40': 'CjkUnifiedIdeograph9d40',
u'\ude42': 'UnicodeCharacter0xde42',
u'\ubac5': 'HangulSyllableMwag',
u'\u1b44': 'BalineseAdegAdeg',
u'\uf7c7': 'UnicodeCharacter0xf7c7',
u'\u5446': 'CjkUnifiedIdeograph5446',
u'\u30c9': 'KatakanaLetterDo',
u'\u6dcb': 'CjkUnifiedIdeograph6dcb',
u'\u8550': 'CjkUnifiedIdeograph8550',
u'\uc652': 'HangulSyllableWabs',
u'\ua2d5': 'YiSyllableZziex',
u'\u0354': 'CombiningLeftArrowheadBelow',
u'\udfd7': 'UnicodeCharacter0xdfd7',
u'\u3c56': 'CjkUnifiedIdeograph3c56',
u'\u18d9': 'UnicodeCharacter0x18d9',
u'\u7958': 'CjkUnifiedIdeograph7958',
u'\u55db': 'CjkUnifiedIdeograph55db',
u'\uae62': 'HangulSyllableGgaj',
u'\u8ae5': 'CjkUnifiedIdeograph8ae5',
u'\ueb64': 'UnicodeCharacter0xeb64',
u'\uc7e7': 'HangulSyllableJyaegs',
u'\u2466': 'CircledDigitSeven',
u'\xe9': 'LatinSmallLetterEWithAcute',
u'\u6168': 'CjkUnifiedIdeograph6168',
u'\u3deb': 'CjkUnifiedIdeograph3deb',
u'\u7eed': 'CjkUnifiedIdeograph7eed',
u'\u9672': 'CjkUnifiedIdeograph9672',
u'\ud374': 'HangulSyllablePyaess',
u'\uaff7': 'HangulSyllableGgwed',
u'\u0c76': 'UnicodeCharacter0xc76',
u'\ue8f9': 'UnicodeCharacter0xe8f9',
u'\u4978': 'CjkUnifiedIdeograph4978',
u'\u25fb': 'WhiteMediumSquare',
u'\u66fd': 'CjkUnifiedIdeograph66fd',
u'\u5c01': 'CjkUnifiedIdeograph5c01',
u'\ubb84': 'HangulSyllableMwek',
u'\uf486': 'UnicodeCharacter0xf486',
u'\u3188': 'HangulLetterYoyae',
u'\u8d0b': 'CjkUnifiedIdeograph8d0b',
u'\u728a': 'CjkUnifiedIdeograph728a',
u'\uce0d': 'HangulSyllableCyulg',
u'\u0b0f': 'OriyaLetterE',
u'\u4411': 'CjkUnifiedIdeograph4411',
u'\ua394': 'YiSyllableNrat',
u'\udc96': 'UnicodeCharacter0xdc96',
u'\u1998': 'NewTaiLueLetterLowPha',
u'\u5a9a': 'CjkUnifiedIdeograph5a9a',
u'\ub61d': 'HangulSyllableDdolt',
u'\uf31f': 'UnicodeCharacter0xf31f',
u'\u2c21': 'GlagoliticCapitalLetterYati',
u'\u6923': 'CjkUnifiedIdeograph6923',
u'\u8ba4': 'CjkUnifiedIdeograph8ba4',
u'\uc4a6': 'HangulSyllableSswelm',
u'\u01a8': 'LatinSmallLetterToneTwo',
u'\u42aa': 'CjkUnifiedIdeograph42aa',
u'\u9e2d': 'CjkUnifiedIdeograph9e2d',
u'\u7fac': 'CjkUnifiedIdeograph7fac',
u'\udb2f': 'UnicodeCharacter0xdb2f',
u'\u1431': 'CanadianSyllabicsPi',
u'\u5133': 'CjkUnifiedIdeograph5133',
u'\uacb6': 'HangulSyllableGyeolp',
u'\ue9b8': 'UnicodeCharacter0xe9b8',
u'\u2aba': 'SucceedsAboveNotAlmostEqualTo',
u'\u863d': 'CjkUnifiedIdeograph863d',
u'\u67bc': 'CjkUnifiedIdeograph67bc',
u'\uc33f': 'HangulSyllableSsyalh',
u'\ufc41': 'ArabicLigatureLamWithKhahIsolatedForm',
u'\u3943': 'CjkUnifiedIdeograph3943',
u'\u7a45': 'CjkUnifiedIdeograph7a45',
u'\u94c6': 'CjkUnifiedIdeograph94c6',
u'\ud1c8': 'HangulSyllableTwals',
u'\u12ca': 'EthiopicSyllableWi',
u'\u4fcc': 'CjkUnifiedIdeograph4fcc',
u'\uab4f': 'UnicodeCharacter0xab4f',
u'\ue451': 'UnicodeCharacter0xe451',
u'\u2153': 'VulgarFractionOneThird',
u'\u6255': 'CjkUnifiedIdeograph6255',
u'\ub9d8': 'HangulSyllableMam',
u'\ufada': 'UnicodeCharacter0xfada',
u'\u37dc': 'CjkUnifiedIdeograph37dc',
u'\u935f': 'CjkUnifiedIdeograph935f',
u'\u70de': 'CjkUnifiedIdeograph70de',
u'\ucc61': 'HangulSyllableCyag',
u'\u0963': 'DevanagariVowelSignVocalicLl',
u'\u4a65': 'CjkUnifiedIdeograph4a65',
u'\ua1e8': 'YiSyllableGuo',
u'\ue2ea': 'UnicodeCharacter0xe2ea',
u'\u1fec': 'GreekCapitalLetterRhoWithDasia',
u'\u58ee': 'CjkUnifiedIdeograph58ee',
u'\ub471': 'HangulSyllableDweonj',
u'\uf173': 'UnicodeCharacter0xf173',
u'\u3275': 'CircledHangulIeungA',
u'\u6f77': 'CjkUnifiedIdeograph6f77',
u'\u89f8': 'CjkUnifiedIdeograph89f8',
u'\ucafa': 'HangulSyllableJjwaenh',
u'\u07fc': 'UnicodeCharacter0x7fc',
u'\u40fe': 'CjkUnifiedIdeograph40fe',
u'\u7e00': 'CjkUnifiedIdeograph7e00',
u'\u5a83': 'CjkUnifiedIdeograph5a83',
u'\ubd02': 'HangulSyllableBolp',
u'\u9985': 'CjkUnifiedIdeograph9985',
u'\u3706': 'CjkUnifiedIdeograph3706',
u'\u1389': 'EthiopicSyllableFwi',
u'\uec0c': 'UnicodeCharacter0xec0c',
u'\uc88f': 'HangulSyllableJwags',
u'\u6610': 'CjkUnifiedIdeograph6610',
u'\u4293': 'CjkUnifiedIdeograph4293',
u'\ua512': 'VaiSyllableLee',
u'\u8195': 'CjkUnifiedIdeograph8195',
u'\u1f16': 'UnicodeCharacter0x1f16',
u'\ufb99': 'ArabicLetterGuehMedialForm',
u'\u759d': 'CjkUnifiedIdeograph759d',
u'\ud41c': 'HangulSyllablePwass',
u'\ub09f': 'HangulSyllableNad',
u'\u4e20': 'CjkUnifiedIdeograph4e20',
u'\u2aa3': 'DoubleNestedLessthanWithUnderbar',
u'\u8d22': 'CjkUnifiedIdeograph8d22',
u'\u0726': 'SyriacLetterPe',
u'\ue3a9': 'UnicodeCharacter0xe3a9',
u'\u5dad': 'CjkUnifiedIdeograph5dad',
u'\ubc2c': 'HangulSyllableBak',
u'\u98af': 'CjkUnifiedIdeograph98af',
u'\u3630': 'CjkUnifiedIdeograph3630',
u'\u12b3': 'EthiopicSyllableKwaa',
u'\uef36': 'UnicodeCharacter0xef36',
u'\ucbb9': 'HangulSyllableJjyug',
u'\u693a': 'CjkUnifiedIdeograph693a',
u'\u45bd': 'CjkUnifiedIdeograph45bd',
u'\ua43c': 'YiSyllableNjox',
u'\u80bf': 'CjkUnifiedIdeograph80bf',
u'\u1e40': 'LatinCapitalLetterMWithDotAbove',
u'\ufac3': 'CjkCompatibilityIdeographfac3',
u'\u74c7': 'CjkUnifiedIdeograph74c7',
u'\ud746': 'HangulSyllableHyubs',
u'\ub3c9': 'HangulSyllableDonj',
u'\u514a': 'CjkUnifiedIdeograph514a',
u'\u2dcd': 'EthiopicSyllableKye',
u'\u8c4c': 'CjkUnifiedIdeograph8c4c',
u'\u0650': 'ArabicKasra',
u'\ue2d3': 'UnicodeCharacter0xe2d3',
u'\u5cd7': 'CjkUnifiedIdeograph5cd7',
u'\ubf56': 'HangulSyllableBboj',
u'\u9bd9': 'CjkUnifiedIdeograph9bd9',
u'\u395a': 'CjkUnifiedIdeograph395a',
u'\u15dd': 'CanadianSyllabicsCarrierTho',
u'\uee60': 'UnicodeCharacter0xee60',
u'\ucae3': 'HangulSyllableJjwalb',
u'\u6864': 'CjkUnifiedIdeograph6864',
u'\u44e7': 'CjkUnifiedIdeograph44e7',
u'\ua766': 'LatinCapitalLetterThornWithStrokeThroughDescender',
u'\u83e9': 'CjkUnifiedIdeograph83e9',
u'\u216a': 'RomanNumeralEleven',
u'\ufded': 'UnicodeCharacter0xfded',
u'\u77f1': 'CjkUnifiedIdeograph77f1',
u'\ud670': 'HangulSyllableHwae',
u'\ub2f3': 'HangulSyllableDalh',
u'\u5074': 'CjkUnifiedIdeograph5074',
u'\u2cf7': 'UnicodeCharacter0x2cf7',
u'\u8f76': 'CjkUnifiedIdeograph8f76',
u'\u097a': 'UnicodeCharacter0x97a',
u'\ue5fd': 'UnicodeCharacter0xe5fd',
u'\udf01': 'UnicodeCharacter0xdf01',
u'\ube80': 'HangulSyllableBbaen',
u'\u5905': 'CjkUnifiedIdeograph5905',
u'\u3884': 'CjkUnifiedIdeograph3884',
u'\u9407': 'CjkUnifiedIdeograph9407',
u'\u0e0b': 'ThaiCharacterSoSo',
u'\uf18a': 'UnicodeCharacter0xf18a',
u'\u6b8e': 'CjkUnifiedIdeograph6b8e',
u'\uc711': 'HangulSyllableWilt',
u'\ua690': 'CyrillicCapitalLetterTsse',
u'\u4115': 'CjkUnifiedIdeograph4115',
u'\u2094': 'LatinSubscriptSmallLetterSchwa',
u'\uf61b': 'UnicodeCharacter0xf61b',
u'\ud99a': 'UnicodeCharacter0xd99a',
u'\u701f': 'CjkUnifiedIdeograph701f',
u'\u539e': 'CjkUnifiedIdeograph539e',
u'\uaf21': 'HangulSyllableGgyeb',
u'\u8ea0': 'CjkUnifiedIdeograph8ea0',
u'\u2925': 'SouthEastArrowWithHook',
u'\u08a4': 'UnicodeCharacter0x8a4',
u'\ude2b': 'UnicodeCharacter0xde2b',
u'\uc1aa': 'HangulSyllableSwagg',
u'\u582f': 'CjkUnifiedIdeograph582f',
u'\u3bae': 'CjkUnifiedIdeograph3bae',
u'\u9731': 'CjkUnifiedIdeograph9731',
u'\u1135': 'HangulChoseongSiosieung',
u'\uf0b4': 'UnicodeCharacter0xf0b4',
u'\u6ab8': 'CjkUnifiedIdeograph6ab8',
u'\uc63b': 'HangulSyllableOc',
u'\ua9ba': 'UnicodeCharacter0xa9ba',
u'\u403f': 'CjkUnifiedIdeograph403f',
u'\u23be': 'DentistrySymbolLightVerticalAndTopRight',
u'\uf945': 'CjkCompatibilityIdeographf945',
u'\ud8c4': 'UnicodeCharacter0xd8c4',
u'\u7349': 'CjkUnifiedIdeograph7349',
u'\u52c8': 'CjkUnifiedIdeograph52c8',
u'\uae4b': 'HangulSyllableGih',
u'\u91ca': 'CjkUnifiedIdeograph91ca',
u'\u284f': 'BraillePatternDots12347',
u'\u0bce': 'UnicodeCharacter0xbce',
u'\ue155': 'UnicodeCharacter0xe155',
u'\uc0d4': 'HangulSyllableSaels',
u'\u5b59': 'CjkUnifiedIdeograph5b59',
u'\u3ad8': 'CjkUnifiedIdeograph3ad8',
u'\u965b': 'CjkUnifiedIdeograph965b',
u'\u105f': 'MyanmarConsonantSignMonMedialMa',
u'\uf3de': 'UnicodeCharacter0xf3de',
u'\u6de2': 'CjkUnifiedIdeograph6de2',
u'\uc965': 'HangulSyllableJwing',
u'\ua8e4': 'UnicodeCharacter0xa8e4',
u'\u4369': 'CjkUnifiedIdeograph4369',
u'\u22e8': 'PrecedesButNotEquivalentTo',
u'\uf86f': 'UnicodeCharacter0xf86f',
u'\udbee': 'UnicodeCharacter0xdbee',
u'\u7273': 'CjkUnifiedIdeograph7273',
u'\u55f2': 'CjkUnifiedIdeograph55f2',
u'\ub175': 'HangulSyllableNyet',
u'\u90f4': 'CjkUnifiedIdeograph90f4',
u'\u2b79': 'UnicodeCharacter0x2b79',
u'\u0af8': 'UnicodeCharacter0xaf8',
u'\ue07f': 'UnicodeCharacter0xe07f',
u'\uc3fe': 'HangulSyllableSswalm',
u'\ub606': 'HangulSyllableDdyebs',
u'\u9289': 'CjkUnifiedIdeograph9289',
u'\uf308': 'UnicodeCharacter0xf308',
u'\ucf8b': 'HangulSyllableKwah',
u'\u300a': 'LeftDoubleAngleBracket',
u'\u0c8d': 'UnicodeCharacter0xc8d',
u'\u6d0c': 'CjkUnifiedIdeograph6d0c',
u'\u498f': 'CjkUnifiedIdeograph498f',
u'\u9e16': 'CjkUnifiedIdeograph9e16',
u'\udb18': 'UnicodeCharacter0xdb18',
u'\ub79b': 'HangulSyllableRaegs',
u'\u181a': 'UnicodeCharacter0x181a',
u'\uf49d': 'UnicodeCharacter0xf49d',
u'\u551c': 'CjkUnifiedIdeograph551c',
u'\u319f': 'IdeographicAnnotationManMark',
u'\u6ea1': 'CjkUnifiedIdeograph6ea1',
u'\u8626': 'CjkUnifiedIdeograph8626',
u'\uc328': 'HangulSyllableSsaess',
u'\u9fab': 'CjkUnifiedIdeograph9fab',
u'\udcad': 'UnicodeCharacter0xdcad',
u'\u3d2c': 'CjkUnifiedIdeograph3d2c',
u'\u19af': 'UnicodeCharacter0x19af',
u'\u7a2e': 'CjkUnifiedIdeograph7a2e',
u'\u56b1': 'CjkUnifiedIdeograph56b1',
u'\uab38': 'UnicodeCharacter0xab38',
u'\u87bb': 'CjkUnifiedIdeograph87bb',
u'\ue83a': 'UnicodeCharacter0xe83a',
u'\uc4bd': 'HangulSyllableSswinj',
u'\u253c': 'BoxDrawingsLightVerticalAndHorizontal',
u'\u01bf': 'LatinLetterWynn',
u'\u623e': 'CjkUnifiedIdeograph623e',
u'\u3ec1': 'CjkUnifiedIdeograph3ec1',
u'\u7bc3': 'CjkUnifiedIdeograph7bc3',
u'\u9348': 'CjkUnifiedIdeograph9348',
u'\ud04a': 'HangulSyllableKwij',
u'\uaccd': 'HangulSyllableGyelg',
u'\u0d4c': 'MalayalamVowelSignAu',
u'\ue9cf': 'UnicodeCharacter0xe9cf',
u'\u4a4e': 'CjkUnifiedIdeograph4a4e',
u'\u26d1': 'UnicodeCharacter0x26d1',
u'\u63d3': 'CjkUnifiedIdeograph63d3',
u'\ub85a': 'HangulSyllableRyep',
u'\u94dd': 'CjkUnifiedIdeograph94dd',
u'\uf55c': 'UnicodeCharacter0xf55c',
u'\ud1df': 'HangulSyllableTwaed',
u'\u325e': 'CircledNumberThirtyFour',
u'\u0ee1': 'UnicodeCharacter0xee1',
u'\u6f60': 'CjkUnifiedIdeograph6f60',
u'\u4be3': 'CjkUnifiedIdeograph4be3',
u'\ua06a': 'YiSyllableBbe',
u'\udd6c': 'UnicodeCharacter0xdd6c',
u'\ub9ef': 'HangulSyllableMaelb',
u'\u1a6e': 'UnicodeCharacter0x1a6e',
u'\uf6f1': 'UnicodeCharacter0xf6f1',
u'\u5770': 'CjkUnifiedIdeograph5770',
u'\u33f3': 'IdeographicTelegraphSymbolForDayTwenty',
u'\u70f5': 'CjkUnifiedIdeograph70f5',
u'\u887a': 'CjkUnifiedIdeograph887a',
u'\uc57c': 'HangulSyllableYa',
u'\ua1ff': 'YiSyllableKat',
u'\u027e': 'LatinSmallLetterRWithFishhook',
u'\u3f80': 'CjkUnifiedIdeograph3f80',
u'\u9b03': 'CjkUnifiedIdeograph9b03',
u'\u7c82': 'CjkUnifiedIdeograph7c82',
u'\ud805': 'UnicodeCharacter0xd805',
u'\u1507': 'CanadianSyllabicsSw',
u'\u5209': 'CjkUnifiedIdeograph5209',
u'\uad8c': 'HangulSyllableGweon',
u'\uea8e': 'UnicodeCharacter0xea8e',
u'\u2790': 'DingbatNegativeCircledSansserifDigitSeven',
u'\u8313': 'CjkUnifiedIdeograph8313',
u'\u6492': 'CjkUnifiedIdeograph6492',
u'\uc015': 'HangulSyllableBbweb',
u'\ufd17': 'ArabicLigatureSeenWithAlefMaksuraFinalForm',
u'\u3a19': 'CjkUnifiedIdeograph3a19',
u'\u771b': 'CjkUnifiedIdeograph771b',
u'\u959c': 'CjkUnifiedIdeograph959c',
u'\ud29e': 'HangulSyllableTyugg',
u'\u0fa0': 'TibetanSubjoinedLetterTha',
u'\u4ca2': 'CjkUnifiedIdeograph4ca2',
u'\ua825': 'SylotiNagriVowelSignU',
u'\ue527': 'UnicodeCharacter0xe527',
u'\u2229': 'Intersection',
u'\u5f2b': 'CjkUnifiedIdeograph5f2b',
u'\ubaae': 'HangulSyllableMonh',
u'\uf7b0': 'UnicodeCharacter0xf7b0',
u'\u34b2': 'CjkUnifiedIdeograph34b2',
u'\u9035': 'CjkUnifiedIdeograph9035',
u'\u71b4': 'CjkUnifiedIdeograph71b4',
u'\ucd37': 'HangulSyllableCwas',
u'\u0a39': 'GurmukhiLetterHa',
u'\u473b': 'CjkUnifiedIdeograph473b',
u'\ua2be': 'YiSyllableCox',
u'\udfc0': 'UnicodeCharacter0xdfc0',
u'\u1cc2': 'UnicodeCharacter0x1cc2',
u'\u59c4': 'CjkUnifiedIdeograph59c4',
u'\ub547': 'HangulSyllableDdac',
u'\uf249': 'UnicodeCharacter0xf249',
u'\u2f4b': 'KangxiRadicalLack',
u'\u6c4d': 'CjkUnifiedIdeograph6c4d',
u'\u8ace': 'CjkUnifiedIdeograph8ace',
u'\uc7d0': 'HangulSyllableJyal',
u'\u04d2': 'CyrillicCapitalLetterAWithDiaeresis',
u'\u41d4': 'CjkUnifiedIdeograph41d4',
u'\u9d57': 'CjkUnifiedIdeograph9d57',
u'\u7ed6': 'CjkUnifiedIdeograph7ed6',
u'\uda59': 'UnicodeCharacter0xda59',
u'\u175b': 'UnicodeCharacter0x175b',
u'\u545d': 'CjkUnifiedIdeograph545d',
u'\uafe0': 'HangulSyllableGgweols',
u'\uece2': 'UnicodeCharacter0xece2',
u'\u29e4': 'EqualsSignAndSlantedParallelWithTildeAbove',
u'\u8567': 'CjkUnifiedIdeograph8567',
u'\u66e6': 'CjkUnifiedIdeograph66e6',
u'\uc269': 'HangulSyllableSwet',
u'\uff6b': 'HalfwidthKatakanaLetterSmallO',
u'\u3c6d': 'CjkUnifiedIdeograph3c6d',
u'\u796f': 'CjkUnifiedIdeograph796f',
u'\u97f0': 'CjkUnifiedIdeograph97f0',
u'\ud4f2': 'HangulSyllablePyulm',
u'\u11f4': 'HangulJongseongKapyeounphieuph',
u'\u4ef6': 'CjkUnifiedIdeograph4ef6',
u'\uaa79': 'UnicodeCharacter0xaa79',
u'\ue77b': 'UnicodeCharacter0xe77b',
u'\u247d': 'ParenthesizedNumberTen',
u'\u617f': 'CjkUnifiedIdeograph617f',
u'\ue181': 'UnicodeCharacter0xe181',
u'\u0604': 'UnicodeCharacter0x604',
u'\u2a87': 'LessthanAndSinglelineNotEqualTo',
u'\u8c08': 'CjkUnifiedIdeograph8c08',
u'\ub08b': 'HangulSyllableGgilh',
u'\u4f0a': 'CjkUnifiedIdeograph4f0a',
u'\u738d': 'CjkUnifiedIdeograph738d',
u'\ud50e': 'HangulSyllablePeulm',
u'\uf991': 'CjkCompatibilityIdeographf991',
u'\u1e14': 'LatinCapitalLetterEWithMacronAndGrave',
u'\u4297': 'CjkUnifiedIdeograph4297',
u'\ua418': 'YiSyllableJjit',
u'\uc89b': 'HangulSyllableJwalh',
u'\u671a': 'CjkUnifiedIdeograph671a',
u'\ued1e': 'UnicodeCharacter0xed1e',
u'\u11a1': 'HangulJungseongAraeai',
u'\u97a5': 'CjkUnifiedIdeograph97a5',
u'\u3624': 'CjkUnifiedIdeograph3624',
u'\u5aa7': 'CjkUnifiedIdeograph5aa7',
u'\ubc28': 'HangulSyllableBass',
u'\ue0ab': 'UnicodeCharacter0xe0ab',
u'\u7f2a': 'CjkUnifiedIdeograph7f2a',
u'\u052e': 'UnicodeCharacter0x52e',
u'\u29b1': 'EmptySetWithOverbar',
u'\u8b32': 'CjkUnifiedIdeograph8b32',
u'\uafb5': 'HangulSyllableGgyot',
u'\u4e34': 'CjkUnifiedIdeograph4e34',
u'\u72b7': 'CjkUnifiedIdeograph72b7',
u'\ud438': 'HangulSyllablePwaess',
u'\uf8bb': 'UnicodeCharacter0xf8bb',
u'\u1d3e': 'ModifierLetterCapitalP',
u'\u41c1': 'CjkUnifiedIdeograph41c1',
u'\ua342': 'YiSyllableSsy',
u'\uc7c5': 'HangulSyllableJaet',
u'\u6644': 'CjkUnifiedIdeograph6644',
u'\uec48': 'UnicodeCharacter0xec48',
u'\u10cb': 'UnicodeCharacter0x10cb',
u'\u96cf': 'CjkUnifiedIdeograph96cf',
u'\u354e': 'CjkUnifiedIdeograph354e',
u'\u59d1': 'CjkUnifiedIdeograph59d1',
u'\ubb52': 'HangulSyllableMweogg',
u'\udfd5': 'UnicodeCharacter0xdfd5',
u'\u7e54': 'CjkUnifiedIdeograph7e54',
u'\u0458': 'CyrillicSmallLetterJe',
u'\u28db': 'BraillePatternDots124578',
u'\u8a5c': 'CjkUnifiedIdeograph8a5c',
u'\uaedf': 'HangulSyllableGged',
u'\u4d5e': 'CjkUnifiedIdeograph4d5e',
u'\u71e1': 'CjkUnifiedIdeograph71e1',
u'\ud362': 'HangulSyllablePyaegg',
u'\uf7e5': 'UnicodeCharacter0xf7e5',
u'\u1c68': 'OlChikiLetterIr',
u'\u40eb': 'CjkUnifiedIdeograph40eb',
u'\ua26c': 'YiSyllableNge',
u'\uc6ef': 'HangulSyllableWed',
u'\u656e': 'CjkUnifiedIdeograph656e',
u'\ueb72': 'UnicodeCharacter0xeb72',
u'\u0ff5': 'UnicodeCharacter0xff5',
u'\u95f9': 'CjkUnifiedIdeograph95f9',
u'\u3478': 'CjkUnifiedIdeograph3478',
u'\u58fb': 'CjkUnifiedIdeograph58fb',
u'\uba7c': 'HangulSyllableMyeols',
u'\udeff': 'UnicodeCharacter0xdeff',
u'\u7d7e': 'CjkUnifiedIdeograph7d7e',
u'\ue403': 'UnicodeCharacter0xe403',
u'\u0382': 'UnicodeCharacter0x382',
u'\u8986': 'CjkUnifiedIdeograph8986',
u'\u2d09': 'GeorgianSmallLetterKan',
u'\u4c88': 'CjkUnifiedIdeograph4c88',
u'\ub30d': 'HangulSyllableDaelt',
u'\ud28c': 'HangulSyllableTwils',
u'\u760f': 'CjkUnifiedIdeograph760f',
u'\ufc13': 'ArabicLigatureThehWithAlefMaksuraIsolatedForm',
u'\u1b92': 'SundaneseLetterTa',
u'\u8217': 'CjkUnifiedIdeograph8217',
u'\ua196': 'YiSyllableNur',
u'\u4519': 'CjkUnifiedIdeograph4519',
u'\u6498': 'CjkUnifiedIdeograph6498',
u'\ucb1d': 'HangulSyllableJjoelt',
u'\uea9c': 'UnicodeCharacter0xea9c',
u'\u1423': 'CanadianSyllabicsFinalRightHalfRing',
u'\u33a2': 'SquareKmSquared',
u'\u9a27': 'CjkUnifiedIdeograph9a27',
u'\ub9a6': 'HangulSyllableRyij',
u'\u5d29': 'CjkUnifiedIdeograph5d29',
u'\u7ca8': 'CjkUnifiedIdeograph7ca8',
u'\ue32d': 'UnicodeCharacter0xe32d',
u'\u02ac': 'LatinLetterBilabialPercussive',
u'\u88b0': 'CjkUnifiedIdeograph88b0',
u'\u2c33': 'GlagoliticSmallLetterGlagoli',
u'\u4bb2': 'CjkUnifiedIdeograph4bb2',
u'\ub237': 'HangulSyllableNweoc',
u'\ud1b6': 'HangulSyllableToj',
u'\u7539': 'CjkUnifiedIdeograph7539',
u'\ufb3d': 'UnicodeCharacter0xfb3d',
u'\u1abc': 'UnicodeCharacter0x1abc',
u'\u8141': 'CjkUnifiedIdeograph8141',
u'\ua0c0': 'YiSyllableMop',
u'\u4443': 'CjkUnifiedIdeograph4443',
u'\u63c2': 'CjkUnifiedIdeograph63c2',
u'\uca47': 'HangulSyllableJjyaec',
u'\ue9c6': 'UnicodeCharacter0xe9c6',
u'\u134d': 'EthiopicSyllableFe',
u'\u32cc': 'SquareHg',
u'\u9951': 'CjkUnifiedIdeograph9951',
u'\ub8d0': 'HangulSyllableRyon',
u'\u5c53': 'CjkUnifiedIdeograph5c53',
u'\u7bd2': 'CjkUnifiedIdeograph7bd2',
u'\ue257': 'UnicodeCharacter0xe257',
u'\u01d6': 'LatinSmallLetterUWithDiaeresisAndMacron',
u'\u87da': 'CjkUnifiedIdeograph87da',
u'\u2b5d': 'UnicodeCharacter0x2b5d',
u'\u4adc': 'CjkUnifiedIdeograph4adc',
u'\ub161': 'HangulSyllableNyenj',
u'\ud0e0': 'HangulSyllableTaen',
u'\u7463': 'CjkUnifiedIdeograph7463',
u'\ufa67': 'CjkCompatibilityIdeographfa67',
u'\u19e6': 'KhmerSymbolPrammuoyKoet',
u'\u806b': 'CjkUnifiedIdeograph806b',
u'\u9fea': 'UnicodeCharacter0x9fea',
u'\u436d': 'CjkUnifiedIdeograph436d',
u'\u62ec': 'CjkUnifiedIdeograph62ec',
u'\uc971': 'HangulSyllableJyunj',
u'\ue8f0': 'UnicodeCharacter0xe8f0',
u'\u1277': 'EthiopicSyllableTwa',
u'\u31f6': 'KatakanaLetterSmallHi',
u'\u987b': 'CjkUnifiedIdeograph987b',
u'\ub7fa': 'HangulSyllableReolp',
u'\u5b7d': 'CjkUnifiedIdeograph5b7d',
u'\u7afc': 'CjkUnifiedIdeograph7afc',
u'\u6081': 'CjkUnifiedIdeograph6081',
u'\u0100': 'LatinCapitalLetterAWithMacron',
u'\u2583': 'LowerThreeEighthsBlock',
u'\uc202': 'HangulSyllableSyonh',
u'\ue685': 'UnicodeCharacter0xe685',
u'\u8704': 'CjkUnifiedIdeograph8704',
u'\uab87': 'UnicodeCharacter0xab87',
u'\u540e': 'CjkUnifiedIdeograph540e',
u'\u7891': 'CjkUnifiedIdeograph7891',
u'\u1910': 'LimbuLetterPa',
u'\u3d93': 'CjkUnifiedIdeograph3d93',
u'\uda12': 'UnicodeCharacter0xda12',
u'\ufe95': 'ArabicLetterTehIsolatedForm',
u'\u9f14': 'CjkUnifiedIdeograph9f14',
u'\uc397': 'HangulSyllableSses',
u'\u8499': 'CjkUnifiedIdeograph8499',
u'\u6c1e': 'CjkUnifiedIdeograph6c1e',
u'\u3120': 'BopomofoLetterAu',
u'\u55a3': 'CjkUnifiedIdeograph55a3',
u'\uf222': 'UnicodeCharacter0xf222',
u'\u16a5': 'RunicLetterW',
u'\ub724': 'HangulSyllableDdyuk',
u'\udba7': 'UnicodeCharacter0xdba7',
u'\u9ca9': 'CjkUnifiedIdeograph9ca9',
u'\u4930': 'CjkUnifiedIdeograph4930',
u'\u6db3': 'CjkUnifiedIdeograph6db3',
u'\u0a32': 'GurmukhiLetterLa',
u'\u2eb5': 'CjkRadicalMesh',
u'\ucf34': 'HangulSyllableKyeok',
u'\uf3b7': 'UnicodeCharacter0xf3b7',
u'\u9036': 'CjkUnifiedIdeograph9036',
u'\ub4b9': 'HangulSyllableDwing',
u'\u6140': 'CjkUnifiedIdeograph6140',
u'\u2242': 'MinusTilde',
u'\u46c5': 'CjkUnifiedIdeograph46c5',
u'\ue744': 'UnicodeCharacter0xe744',
u'\u0bc7': 'TamilVowelSignEe',
u'\ua846': 'PhagspaLetterJa',
u'\uccc9': 'HangulSyllableCeng',
u'\u91cb': 'CjkUnifiedIdeograph91cb',
u'\u7950': 'CjkUnifiedIdeograph7950',
u'\u3a52': 'CjkUnifiedIdeograph3a52',
u'\u5ed5': 'CjkUnifiedIdeograph5ed5',
u'\uff54': 'FullwidthLatinSmallLetterT',
u'\u23d7': 'MetricalTriseme',
u'\uc056': 'HangulSyllableBbyup',
u'\ue4d9': 'UnicodeCharacter0xe4d9',
u'\u8558': 'CjkUnifiedIdeograph8558',
u'\ua9db': 'UnicodeCharacter0xa9db',
u'\u5262': 'CjkUnifiedIdeograph5262',
u'\u76e5': 'CjkUnifiedIdeograph76e5',
u'\u1764': 'TagbanwaLetterGa',
u'\u3be7': 'CjkUnifiedIdeograph3be7',
u'\ud866': 'UnicodeCharacter0xd866',
u'\ufce9': 'ArabicLigatureSheenWithMeemMedialForm',
u'\u9d68': 'CjkUnifiedIdeograph9d68',
u'\uc1eb': 'HangulSyllableSoelb',
u'\u82ed': 'CjkUnifiedIdeograph82ed',
u'\u6a72': 'CjkUnifiedIdeograph6a72',
u'\u2f74': 'KangxiRadicalStand',
u'\u53f7': 'CjkUnifiedIdeograph53f7',
u'\uf076': 'UnicodeCharacter0xf076',
u'\u14f9': 'CanadianSyllabicsWestcreeSwi',
u'\ub578': 'HangulSyllableDdyam',
u'\ud9fb': 'UnicodeCharacter0xd9fb',
u'\u9afd': 'CjkUnifiedIdeograph9afd',
u'\ua001': 'YiSyllableIx',
u'\u4784': 'CjkUnifiedIdeograph4784',
u'\u0886': 'UnicodeCharacter0x886',
u'\ucd88': 'HangulSyllableCyom',
u'\u710b': 'CjkUnifiedIdeograph710b',
u'\u8e8a': 'CjkUnifiedIdeograph8e8a',
u'\u320d': 'ParenthesizedHangulHieuh',
u'\uf70f': 'UnicodeCharacter0xf70f',
u'\ub811': 'HangulSyllableRelg',
u'\u5f94': 'CjkUnifiedIdeograph5f94',
u'\u2096': 'UnicodeCharacter0x2096',
u'\ue598': 'UnicodeCharacter0xe598',
u'\ua69a': 'UnicodeCharacter0xa69a',
u'\u4a1d': 'CjkUnifiedIdeograph4a1d',
u'\u0f1f': 'TibetanSignRdelDkarRdelNag',
u'\ud021': 'HangulSyllableKwelg',
u'\u9523': 'CjkUnifiedIdeograph9523',
u'\u77a4': 'CjkUnifiedIdeograph77a4',
u'\u38a6': 'CjkUnifiedIdeograph38a6',
u'\ufda8': 'ArabicLigatureSeenWithKhahWithAlefMaksuraFinalForm',
u'\ubeaa': 'HangulSyllableBbyabs',
u'\u622d': 'CjkUnifiedIdeograph622d',
u'\u83ac': 'CjkUnifiedIdeograph83ac',
u'\u272f': 'PinwheelStar',
u'\ue831': 'UnicodeCharacter0xe831',
u'\uad33': 'HangulSyllableGwaeh',
u'\u50b6': 'CjkUnifiedIdeograph50b6',
u'\u15b8': 'CanadianSyllabicsBlackfootNe',
u'\ud6ba': 'HangulSyllableHyobs',
u'\u7a3d': 'CjkUnifiedIdeograph7a3d',
u'\u9bbc': 'CjkUnifiedIdeograph9bbc',
u'\u3f3f': 'CjkUnifiedIdeograph3f3f',
u'\uc543': 'HangulSyllableSsih',
u'\u8645': 'CjkUnifiedIdeograph8645',
u'\u68c6': 'CjkUnifiedIdeograph68c6',
u'\u2dc8': 'EthiopicSyllableKya',
u'\ueeca': 'UnicodeCharacter0xeeca',
u'\ub3cc': 'HangulSyllableDol',
u'\u574f': 'CjkUnifiedIdeograph574f',
u'\u1851': 'MongolianLetterTodoDa',
u'\udd53': 'UnicodeCharacter0xdd53',
u'\u9e55': 'CjkUnifiedIdeograph9e55',
u'\u45d8': 'CjkUnifiedIdeograph45d8',
u'\u06da': 'ArabicSmallHighJeem',
u'\ucbdc': 'HangulSyllableJjeul',
u'\u6f5f': 'CjkUnifiedIdeograph6f5f',
u'\u8cde': 'CjkUnifiedIdeograph8cde',
u'\u3061': 'HiraganaLetterTi',
u'\uf563': 'UnicodeCharacter0xf563',
u'\ub665': 'HangulSyllableDdoeg',
u'\u5de8': 'CjkUnifiedIdeograph5de8',
u'\u1eea': 'LatinCapitalLetterUWithHornAndGrave',
u'\ue3ec': 'UnicodeCharacter0xe3ec',
u'\ua4ee': 'UnicodeCharacter0xa4ee',
u'\u4871': 'CjkUnifiedIdeograph4871',
u'\u0d73': 'MalayalamFractionOneQuarter',
u'\uce75': 'HangulSyllableKag',
u'\u9377': 'CjkUnifiedIdeograph9377',
u'\u75f8': 'CjkUnifiedIdeograph75f8',
u'\u36fa': 'CjkUnifiedIdeograph36fa',
u'\ufbfc': 'ArabicLetterFarsiYehIsolatedForm',
u'\ubcfe': 'HangulSyllableBolm',
u'\u8200': 'CjkUnifiedIdeograph8200',
u'\ua683': 'CyrillicSmallLetterDzwe',
u'\u4102': 'CjkUnifiedIdeograph4102',
u'\u6585': 'CjkUnifiedIdeograph6585',
u'\ucb06': 'HangulSyllableJjwaebs',
u'\uef89': 'UnicodeCharacter0xef89',
u'\u100c': 'MyanmarLetterTtha',
u'\u348f': 'CjkUnifiedIdeograph348f',
u'\u9a10': 'CjkUnifiedIdeograph9a10',
u'\ube93': 'HangulSyllableBbaec',
u'\u5912': 'CjkUnifiedIdeograph5912',
u'\u7d95': 'CjkUnifiedIdeograph7d95',
u'\ue316': 'UnicodeCharacter0xe316',
u'\u0799': 'ThaanaLetterHhaa',
u'\u899d': 'CjkUnifiedIdeograph899d',
u'\u281c': 'BraillePatternDots345',
u'\u4c9f': 'CjkUnifiedIdeograph4c9f',
u'\ub220': 'HangulSyllableNweo',
u'\ud6a3': 'HangulSyllableHoec',
u'\u7122': 'CjkUnifiedIdeograph7122',
u'\ufb26': 'HebrewLetterWideFinalMem',
u'\u1fa9': 'GreekCapitalLetterOmegaWithDasiaAndProsgegrammeni',
u'\ua1ad': 'YiSyllableHlu',
u'\u402c': 'CjkUnifiedIdeograph402c',
u'\u64af': 'CjkUnifiedIdeograph64af',
u'\uca30': 'HangulSyllableJjyae',
u'\ueeb3': 'UnicodeCharacter0xeeb3',
u'\u1336': 'EthiopicSyllablePho',
u'\u37b9': 'CjkUnifiedIdeograph37b9',
u'\u953a': 'CjkUnifiedIdeograph953a',
u'\ub9bd': 'HangulSyllableRib',
u'\u583c': 'CjkUnifiedIdeograph583c',
u'\u7cbf': 'CjkUnifiedIdeograph7cbf',
u'\ue240': 'UnicodeCharacter0xe240',
u'\u06c3': 'ArabicLetterTehMarbutaGoal',
u'\u88c7': 'CjkUnifiedIdeograph88c7',
u'\u2b46': 'RightwardsQuadrupleArrow',
u'\u4fc9': 'CjkUnifiedIdeograph4fc9',
u'\uad4a': 'HangulSyllableGoej',
u'\ud1cd': 'HangulSyllableTwab',
u'\u704c': 'CjkUnifiedIdeograph704c',
u'\ufa50': 'CjkCompatibilityIdeographfa50',
u'\u1ed3': 'LatinSmallLetterOWithCircumflexAndGrave',
u'\ua0d7': 'YiSyllableFop',
u'\u4356': 'CjkUnifiedIdeograph4356',
u'\u67d9': 'CjkUnifiedIdeograph67d9',
u'\uc55a': 'HangulSyllableAj',
u'\ue9dd': 'UnicodeCharacter0xe9dd',
u'\u1260': 'EthiopicSyllableBa',
u'\u36e3': 'CjkUnifiedIdeograph36e3',
u'\u9464': 'CjkUnifiedIdeograph9464',
u'\ub8e7': 'HangulSyllableRyoh',
u'\u5b66': 'CjkUnifiedIdeograph5b66',
u'\u7fe9': 'CjkUnifiedIdeograph7fe9',
u'\udd6a': 'UnicodeCharacter0xdd6a',
u'\u01ed': 'LatinSmallLetterOWithOgonekAndMacron',
u'\u8bf1': 'CjkUnifiedIdeograph8bf1',
u'\u2a70': 'ApproximatelyEqualOrEqualTo',
u'\u4ef3': 'CjkUnifiedIdeograph4ef3',
u'\uac74': 'HangulSyllableGeon',
u'\ud0f7': 'HangulSyllableTaeh',
u'\u7376': 'CjkUnifiedIdeograph7376',
u'\uf57a': 'UnicodeCharacter0xf57a',
u'\u19fd': 'KhmerSymbolDapbeiRoc',
u'\u2301': 'ElectricArrow',
u'\u4280': 'CjkUnifiedIdeograph4280',
u'\ua505': 'VaiSyllablePee',
u'\uc484': 'HangulSyllableSsweon',
u'\u6807': 'CjkUnifiedIdeograph6807',
u'\uf20b': 'UnicodeCharacter0xf20b',
u'\u0d8a': 'SinhalaLetterIiyanna',
u'\u978e': 'CjkUnifiedIdeograph978e',
u'\u3b11': 'CjkUnifiedIdeograph3b11',
u'\u5a90': 'CjkUnifiedIdeograph5a90',
u'\ubd15': 'HangulSyllableBwanj',
u'\udc94': 'UnicodeCharacter0xdc94',
u'\u0a1b': 'GurmukhiLetterCha',
u'\u259a': 'QuadrantUpperLeftAndLowerRight',
u'\u8c1f': 'CjkUnifiedIdeograph8c1f',
u'\uaf9e': 'HangulSyllableGgyogg',
u'\u5321': 'CjkUnifiedIdeograph5321',
u'\u72a0': 'CjkUnifiedIdeograph72a0',
u'\ud525': 'HangulSyllablePyinj',
u'\uf4a4': 'UnicodeCharacter0xf4a4',
u'\u222b': 'Integral',
u'\u3daa': 'CjkUnifiedIdeograph3daa',
u'\ua42f': 'YiSyllableJjy',
u'\uc7ae': 'HangulSyllableJaegg',
u'\u6b31': 'CjkUnifiedIdeograph6b31',
u'\ued35': 'UnicodeCharacter0xed35',
u'\u0cb4': 'UnicodeCharacter0xcb4',
u'\u96b8': 'CjkUnifiedIdeograph96b8',
u'\u3a3b': 'CjkUnifiedIdeograph3a3b',
u'\u55ba': 'CjkUnifiedIdeograph55ba',
u'\ubc3f': 'HangulSyllableBaelh',
u'\udfbe': 'UnicodeCharacter0xdfbe',
u'\u0545': 'ArmenianCapitalLetterYi',
u'\u24c4': 'CircledLatinCapitalLetterO',
u'\u8f49': 'CjkUnifiedIdeograph8f49',
u'\uaec8': 'HangulSyllableGgeols',
u'\u524b': 'CjkUnifiedIdeograph524b',
u'\u6dca': 'CjkUnifiedIdeograph6dca',
u'\ud44f': 'HangulSyllablePoelh',
u'\uf7ce': 'UnicodeCharacter0xf7ce',
u'\u1d55': 'ModifierLetterSmallBottomHalfO',
u'\u3cd4': 'CjkUnifiedIdeograph3cd4',
u'\ua759': 'LatinSmallLetterQWithDiagonalStroke',
u'\uc6d8': 'HangulSyllableWeols',
u'\u6a5b': 'CjkUnifiedIdeograph6a5b',
u'\uec5f': 'UnicodeCharacter0xec5f',
u'\u0fde': 'UnicodeCharacter0xfde',
u'\u91e2': 'CjkUnifiedIdeograph91e2',
u'\u3565': 'CjkUnifiedIdeograph3565',
u'\u54e4': 'CjkUnifiedIdeograph54e4',
u'\ubf69': 'HangulSyllableBbwalt',
u'\udee8': 'UnicodeCharacter0xdee8',
u'\u046f': 'CyrillicSmallLetterKsi',
u'\u27ee': 'MathematicalLeftFlattenedParenthesis',
u'\u8e73': 'CjkUnifiedIdeograph8e73',
u'\ua9f2': 'UnicodeCharacter0xa9f2',
u'\u4d75': 'CjkUnifiedIdeograph4d75',
u'\u6cf4': 'CjkUnifiedIdeograph6cf4',
u'\ud779': 'HangulSyllableHyilt',
u'\uf6f8': 'UnicodeCharacter0xf6f8',
u'\u1c7f': 'OlChikiPunctuationDoubleMucaad',
u'\u3ffe': 'CjkUnifiedIdeograph3ffe',
u'\u4a06': 'CjkUnifiedIdeograph4a06',
u'\u6e89': 'CjkUnifiedIdeograph6e89',
u'\u0f08': 'TibetanMarkSbrulShad',
u'\u338b': 'SquareNf',
u'\ucc0a': 'HangulSyllableJjyip',
u'\uf08d': 'UnicodeCharacter0xf08d',
u'\u910c': 'CjkUnifiedIdeograph910c',
u'\ub58f': 'HangulSyllableDdyaelb',
u'\u6216': 'CjkUnifiedIdeograph6216',
u'\u2718': 'HeavyBallotX',
u'\u4b9b': 'CjkUnifiedIdeograph4b9b',
u'\ue41a': 'UnicodeCharacter0xe41a',
u'\u089d': 'UnicodeCharacter0x89d',
u'\ua91c': 'KayahLiLetterLa',
u'\ucd9f': 'HangulSyllableCulb',
u'\u92a1': 'CjkUnifiedIdeograph92a1',
u'\u7a26': 'CjkUnifiedIdeograph7a26',
u'\u3f28': 'CjkUnifiedIdeograph3f28',
u'\u63ab': 'CjkUnifiedIdeograph63ab',
u'\ufc2a': 'ArabicLigatureAinWithMeemIsolatedForm',
u'\u20ad': 'KipSign',
u'\uc12c': 'HangulSyllableSeom',
u'\ue5af': 'UnicodeCharacter0xe5af',
u'\u862e': 'CjkUnifiedIdeograph862e',
u'\uaab1': 'UnicodeCharacter0xaab1',
u'\u5738': 'CjkUnifiedIdeograph5738',
u'\u7bbb': 'CjkUnifiedIdeograph7bbb',
u'\u143a': 'CanadianSyllabicsPwe',
u'\u38bd': 'CjkUnifiedIdeograph38bd',
u'\ud93c': 'UnicodeCharacter0xd93c',
u'\ufdbf': 'ArabicLigatureHahWithJeemWithYehFinalForm',
u'\u9e3e': 'CjkUnifiedIdeograph9e3e',
u'\uc2c1': 'HangulSyllableSyig',
u'\u87c3': 'CjkUnifiedIdeograph87c3',
u'\u6f48': 'CjkUnifiedIdeograph6f48',
u'\u2c4a': 'GlagoliticSmallLetterPe',
u'\u50cd': 'CjkUnifiedIdeograph50cd',
u'\uf14c': 'UnicodeCharacter0xf14c',
u'\u15cf': 'CanadianSyllabicsCarrierRa',
u'\ub64e': 'HangulSyllableDdwaenh',
u'\udad1': 'UnicodeCharacter0xdad1',
u'\u9fd3': 'UnicodeCharacter0x9fd3',
u'\u445a': 'CjkUnifiedIdeograph445a',
u'\u68dd': 'CjkUnifiedIdeograph68dd',
u'\u095c': 'DevanagariLetterDddha',
u'\u2ddf': 'UnicodeCharacter0x2ddf',
u'\uce5e': 'HangulSyllableCinh',
u'\uf2e1': 'UnicodeCharacter0xf2e1',
u'\u9360': 'CjkUnifiedIdeograph9360',
u'\ub7e3': 'HangulSyllableRyaes',
u'\u5c6a': 'CjkUnifiedIdeograph5c6a',
u'\u216c': 'RomanNumeralFifty',
u'\u45ef': 'CjkUnifiedIdeograph45ef',
u'\ue66e': 'UnicodeCharacter0xe66e',
u'\u0af1': 'GujaratiRupeeSign',
u'\uab70': 'UnicodeCharacter0xab70',
u'\ucff3': 'HangulSyllableKus',
u'\u8cf5': 'CjkUnifiedIdeograph8cf5',
u'\u747a': 'CjkUnifiedIdeograph747a',
u'\u397c': 'CjkUnifiedIdeograph397c',
u'\u5dff': 'CjkUnifiedIdeograph5dff',
u'\ufe7e': 'ArabicSukunIsolatedForm',
u'\uc380': 'HangulSyllableSseok',
u'\u6703': 'CjkUnifiedIdeograph6703',
u'\u8082': 'CjkUnifiedIdeograph8082',
u'\u2405': 'SymbolForEnquiry',
u'\ue907': 'UnicodeCharacter0xe907',
u'\uae09': 'HangulSyllableGeub',
u'\u518c': 'CjkUnifiedIdeograph518c',
u'\u168e': 'OghamLetterStraif',
u'\udb90': 'UnicodeCharacter0xdb90',
u'\u7f13': 'CjkUnifiedIdeograph7f13',
u'\u9892': 'CjkUnifiedIdeograph9892',
u'\u3c15': 'CjkUnifiedIdeograph3c15',
u'\u0117': 'LatinSmallLetterEWithDotAbove',
u'\uc619': 'HangulSyllableYeb',
u'\u8b1b': 'CjkUnifiedIdeograph8b1b',
u'\u699c': 'CjkUnifiedIdeograph699c',
u'\u2e9e': 'CjkRadicalDeath',
u'\uf3a0': 'UnicodeCharacter0xf3a0',
u'\ub0a2': 'HangulSyllableNalm',
u'\u5425': 'CjkUnifiedIdeograph5425',
u'\u1927': 'LimbuVowelSignE',
u'\ude29': 'UnicodeCharacter0xde29',
u'\ua32b': 'YiSyllableSsix',
u'\u46ae': 'CjkUnifiedIdeograph46ae',
u'\u0bb0': 'TamilLetterRa',
u'\uc8b2': 'HangulSyllableJwaelm',
u'\u6c35': 'CjkUnifiedIdeograph6c35',
u'\u8db4': 'CjkUnifiedIdeograph8db4',
u'\u3137': 'HangulLetterTikeut',
u'\uf639': 'UnicodeCharacter0xf639',
u'\ubb3b': 'HangulSyllableMud',
u'\u5ebe': 'CjkUnifiedIdeograph5ebe',
u'\u23c0': 'DentistrySymbolLightVerticalWithCircle',
u'\ue0c2': 'UnicodeCharacter0xe0c2',
u'\ua5c4': 'VaiSyllableMbo',
u'\u4947': 'CjkUnifiedIdeograph4947',
u'\u0e49': 'ThaiCharacterMaiTho',
u'\ud34b': 'HangulSyllablePyad',
u'\u904d': 'CjkUnifiedIdeograph904d',
u'\u76ce': 'CjkUnifiedIdeograph76ce',
u'\u3bd0': 'CjkUnifiedIdeograph3bd0',
u'\uf8d2': 'UnicodeCharacter0xf8d2',
u'\ubdd4': 'HangulSyllableBwi',
u'\u6157': 'CjkUnifiedIdeograph6157',
u'\u82d6': 'CjkUnifiedIdeograph82d6',
u'\u2659': 'WhiteChessPawn',
u'\ueb5b': 'UnicodeCharacter0xeb5b',
u'\ua85d': 'PhagspaLetterA',
u'\u53e0': 'CjkUnifiedIdeograph53e0',
u'\u10e2': 'GeorgianLetterTar',
u'\ud5e4': 'HangulSyllableHe',
u'\u7967': 'CjkUnifiedIdeograph7967',
u'\u9ae6': 'CjkUnifiedIdeograph9ae6',
u'\u3e69': 'CjkUnifiedIdeograph3e69',
u'\u036b': 'CombiningLatinSmallLetterM',
u'\uc06d': 'HangulSyllableBbeung',
u'\u856f': 'CjkUnifiedIdeograph856f',
u'\u6bf0': 'CjkUnifiedIdeograph6bf0',
u'\u28f2': 'BraillePatternDots25678',
u'\uedf4': 'UnicodeCharacter0xedf4',
u'\ub2f6': 'HangulSyllableDabs',
u'\u5679': 'CjkUnifiedIdeograph5679',
u'\u1b7b': 'BalineseMusicalSymbolLefthandClosedPluk',
u'\ud87d': 'UnicodeCharacter0xd87d',
u'\u9d7f': 'CjkUnifiedIdeograph9d7f',
u'\ue581': 'UnicodeCharacter0xe581',
u'\u0204': 'LatinCapitalLetterEWithDoubleGrave',
u'\u2e87': 'CjkRadicalTable',
u'\u8808': 'CjkUnifiedIdeograph8808',
u'\ub48b': 'HangulSyllableDwegs',
u'\u4b0a': 'CjkUnifiedIdeograph4b0a',
u'\u778d': 'CjkUnifiedIdeograph778d',
u'\ud10e': 'HangulSyllableTyaj',
u'\ufd91': 'UnicodeCharacter0xfd91',
u'\u1a14': 'BugineseLetterSa',
u'\u4697': 'CjkUnifiedIdeograph4697',
u'\ua018': 'YiSyllableBi',
u'\ucc9b': 'HangulSyllableCeogs',
u'\u631a': 'CjkUnifiedIdeograph631a',
u'\ue91e': 'UnicodeCharacter0xe91e',
u'\u15a1': 'CanadianSyllabicsLhii',
u'\u93a5': 'CjkUnifiedIdeograph93a5',
u'\u3224': 'ParenthesizedIdeographFive',
u'\u5ea7': 'CjkUnifiedIdeograph5ea7',
u'\ub828': 'HangulSyllableRyeon',
u'\ue4ab': 'UnicodeCharacter0xe4ab',
u'\u7b2a': 'CjkUnifiedIdeograph7b2a',
u'\u012e': 'LatinCapitalLetterIWithOgonek',
u'\u2db1': 'EthiopicSyllableZzu',
u'\u8f32': 'CjkUnifiedIdeograph8f32',
u'\uabb5': 'UnicodeCharacter0xabb5',
u'\u4a34': 'CjkUnifiedIdeograph4a34',
u'\u76b7': 'CjkUnifiedIdeograph76b7',
u'\ud038': 'HangulSyllableKwin',
u'\ufcbb': 'ArabicLigatureAinWithMeemInitialForm',
u'\u193e': 'UnicodeCharacter0x193e',
u'\u45c1': 'CjkUnifiedIdeograph45c1',
u'\ua742': 'LatinCapitalLetterKWithDiagonalStroke',
u'\uc3c5': 'HangulSyllableSsyelg',
u'\u6244': 'CjkUnifiedIdeograph6244',
u'\ue848': 'UnicodeCharacter0xe848',
u'\u14cb': 'CanadianSyllabicsNwa',
u'\u92cf': 'CjkUnifiedIdeograph92cf',
u'\u314e': 'HangulLetterHieuh',
u'\u5dd1': 'CjkUnifiedIdeograph5dd1',
u'\ubf52': 'HangulSyllableBbobs',
u'\udbd5': 'UnicodeCharacter0xdbd5',
u'\u7a54': 'CjkUnifiedIdeograph7a54',
u'\u2cdb': 'CopticSmallLetterOldCopticShima',
u'\u8e5c': 'CjkUnifiedIdeograph8e5c',
u'\uaadf': 'UnicodeCharacter0xaadf',
u'\u495e': 'CjkUnifiedIdeograph495e',
u'\u75e1': 'CjkUnifiedIdeograph75e1',
u'\ud762': 'HangulSyllableHeubs',
u'\uf3e5': 'UnicodeCharacter0xf3e5',
u'\u1868': 'MongolianLetterSibeTa',
u'\u44eb': 'CjkUnifiedIdeograph44eb',
u'\ua66c': 'CyrillicCapitalLetterDoubleMonocularO',
u'\uc2ef': 'HangulSyllableSis',
u'\u616e': 'CjkUnifiedIdeograph616e',
u'\uef72': 'UnicodeCharacter0xef72',
u'\u0bf5': 'TamilYearSign',
u'\u91f9': 'CjkUnifiedIdeograph91f9',
u'\u3078': 'HiraganaLetterHe',
u'\u5cfb': 'CjkUnifiedIdeograph5cfb',
u'\ube7c': 'HangulSyllableBbae',
u'\udaff': 'UnicodeCharacter0xdaff',
u'\u797e': 'CjkUnifiedIdeograph797e',
u'\ue003': 'UnicodeCharacter0xe003',
u'\u0782': 'ThaanaLetterNoonu',
u'\u8d86': 'CjkUnifiedIdeograph8d86',
u'\u2909': 'UpwardsArrowWithHorizontalStroke',
u'\u4888': 'CjkUnifiedIdeograph4888',
u'\ub70d': 'HangulSyllableDdyug',
u'\ud68c': 'HangulSyllableHoe',
u'\u720f': 'CjkUnifiedIdeograph720f',
u'\uf813': 'UnicodeCharacter0xf813',
u'\u1f92': 'GreekSmallLetterEtaWithPsiliAndVariaAndYpogegrammeni',
u'\u8617': 'CjkUnifiedIdeograph8617',
u'\ua596': 'VaiSyllableUn',
u'\u4119': 'CjkUnifiedIdeograph4119',
u'\u6098': 'CjkUnifiedIdeograph6098',
u'\ucf1d': 'HangulSyllableKyeog',
u'\uee9c': 'UnicodeCharacter0xee9c',
u'\u1023': 'MyanmarLetterI',
u'\u37a2': 'CjkUnifiedIdeograph37a2',
u'\u9e27': 'CjkUnifiedIdeograph9e27',
u'\ubda6': 'HangulSyllableBweolm',
u'\u5929': 'CjkUnifiedIdeograph5929',
u'\u78a8': 'CjkUnifiedIdeograph78a8',
u'\ue72d': 'UnicodeCharacter0xe72d',
u'\u06ac': 'ArabicLetterKafWithDotAbove',
u'\u8cb0': 'CjkUnifiedIdeograph8cb0',
u'\u2833': 'BraillePatternDots1256',
u'\u4fb2': 'CjkUnifiedIdeograph4fb2',
u'\ub637': 'HangulSyllableDdwalb',
u'\ud5b6': 'HangulSyllableHyaelm',
u'\u7139': 'CjkUnifiedIdeograph7139',
u'\uff3d': 'FullwidthRightSquareBracket',
u'\u1ebc': 'LatinCapitalLetterEWithTilde',
u'\u8541': 'CjkUnifiedIdeograph8541',
u'\ua4c0': 'YiRadicalShat',
u'\u4043': 'CjkUnifiedIdeograph4043',
u'\u67c2': 'CjkUnifiedIdeograph67c2',
u'\uce47': 'HangulSyllableCyilb',
u'\uedc6': 'UnicodeCharacter0xedc6',
u'\u174d': 'BuhidLetterRa',
u'\u36cc': 'CjkUnifiedIdeograph36cc',
u'\u9d51': 'CjkUnifiedIdeograph9d51',
u'\ubcd0': 'HangulSyllableByeoss',
u'\u5853': 'CjkUnifiedIdeograph5853',
u'\u7fd2': 'CjkUnifiedIdeograph7fd2',
u'\ue657': 'UnicodeCharacter0xe657',
u'\u05d6': 'HebrewLetterZayin',
u'\u83da': 'CjkUnifiedIdeograph83da',
u'\u2f5d': 'KangxiRadicalDog',
u'\u4edc': 'CjkUnifiedIdeograph4edc',
u'\ub561': 'HangulSyllableDdaeng',
u'\ud4e0': 'HangulSyllablePwiss',
u'\u7063': 'CjkUnifiedIdeograph7063',
u'\ufe67': 'UnicodeCharacter0xfe67',
u'\u1de6': 'CombiningLatinSmallLetterZ',
u'\u846b': 'CjkUnifiedIdeograph846b',
u'\u9bea': 'CjkUnifiedIdeograph9bea',
u'\u476d': 'CjkUnifiedIdeograph476d',
u'\u66ec': 'CjkUnifiedIdeograph66ec',
u'\ucd71': 'HangulSyllableCoeng',
u'\uecf0': 'UnicodeCharacter0xecf0',
u'\u1677': 'UnicodeCharacter0x1677',
u'\u35f6': 'CjkUnifiedIdeograph35f6',
u'\u9c7b': 'CjkUnifiedIdeograph9c7b',
u'\ub3fa': 'HangulSyllableDwap',
u'\u5f7d': 'CjkUnifiedIdeograph5f7d',
u'\u7efc': 'CjkUnifiedIdeograph7efc',
u'\u6481': 'CjkUnifiedIdeograph6481',
u'\u0500': 'CyrillicCapitalLetterKomiDe',
u'\u2183': 'RomanNumeralReversedOneHundred',
u'\uc602': 'HangulSyllableYeoj',
u'\ue285': 'UnicodeCharacter0xe285',
u'\u8304': 'CjkUnifiedIdeograph8304',
u'\uaf87': 'HangulSyllableGgoed',
u'\u500e': 'CjkUnifiedIdeograph500e',
u'\u7c91': 'CjkUnifiedIdeograph7c91',
u'\u1d10': 'LatinLetterSmallCapitalOpenO',
u'\u3993': 'CjkUnifiedIdeograph3993',
u'\ude12': 'UnicodeCharacter0xde12',
u'\ufa95': 'CjkCompatibilityIdeographfa95',
u'\u9b14': 'CjkUnifiedIdeograph9b14',
u'\uc797': 'HangulSyllableJad',
u'\u8099': 'CjkUnifiedIdeograph8099',
u'\u681e': 'CjkUnifiedIdeograph681e',
u'\u3520': 'CjkUnifiedIdeograph3520',
u'\u51a3': 'CjkUnifiedIdeograph51a3',
u'\uf622': 'UnicodeCharacter0xf622',
u'\u12a5': 'EthiopicSyllableGlottalE',
u'\ub324': 'HangulSyllableDyal',
u'\udfa7': 'UnicodeCharacter0xdfa7',
u'\u98a9': 'CjkUnifiedIdeograph98a9',
u'\u4d30': 'CjkUnifiedIdeograph4d30',
u'\u69b3': 'CjkUnifiedIdeograph69b3',
u'\u0e32': 'ThaiCharacterSaraAa',
u'\u2ab5': 'PrecedesAboveNotEqualTo',
u'\ucb34': 'HangulSyllableJjyol',
u'\uf7b7': 'UnicodeCharacter0xf7b7',
u'\u9436': 'CjkUnifiedIdeograph9436',
u'\ub0b9': 'HangulSyllableNaenj',
u'\u6540': 'CjkUnifiedIdeograph6540',
u'\u2642': 'MaleSign',
u'\u42c5': 'CjkUnifiedIdeograph42c5',
u'\ue344': 'UnicodeCharacter0xe344',
u'\u0fc7': 'TibetanSymbolRdoRjeRgyaGram',
u'\uac46': 'HangulSyllableGyalp',
u'\uc8c9': 'HangulSyllableJoenj',
u'\u95cb': 'CjkUnifiedIdeograph95cb',
u'\u7d50': 'CjkUnifiedIdeograph7d50',
u'\u3e52': 'CjkUnifiedIdeograph3e52',
u'\u5ad5': 'CjkUnifiedIdeograph5ad5',
u'\ufb54': 'ArabicLetterBeehInitialForm',
u'\u27d7': 'FullOuterJoin',
u'\uc456': 'HangulSyllableSsyolp',
u'\ue0d9': 'UnicodeCharacter0xe0d9',
u'\u8158': 'CjkUnifiedIdeograph8158',
u'\uaddb': 'HangulSyllableGwih',
u'\u5662': 'CjkUnifiedIdeograph5662',
u'\u72e5': 'CjkUnifiedIdeograph72e5',
u'\u1364': 'EthiopicSemicolon',
u'\u3fe7': 'CjkUnifiedIdeograph3fe7',
u'\udc66': 'UnicodeCharacter0xdc66',
u'\uf8e9': 'UnicodeCharacter0xf8e9',
u'\u9968': 'CjkUnifiedIdeograph9968',
u'\uc5eb': 'HangulSyllableEh',
u'\u86ed': 'CjkUnifiedIdeograph86ed',
u'\u6e72': 'CjkUnifiedIdeograph6e72',
u'\u2b74': 'UnicodeCharacter0x2b74',
u'\u57f7': 'CjkUnifiedIdeograph57f7',
u'\uf476': 'UnicodeCharacter0xf476',
u'\u10f9': 'GeorgianLetterTurnedGan',
u'\ub178': 'HangulSyllableNo',
u'\uddfb': 'UnicodeCharacter0xddfb',
u'\u9efd': 'CjkUnifiedIdeograph9efd',
u'\ua401': 'YiSyllableQiex',
u'\u4384': 'CjkUnifiedIdeograph4384',
u'\u0c86': 'KannadaLetterAa',
u'\uc988': 'HangulSyllableJeu',
u'\u750b': 'CjkUnifiedIdeograph750b',
u'\u8a8a': 'CjkUnifiedIdeograph8a8a',
u'\u360d': 'CjkUnifiedIdeograph360d',
u'\uf30f': 'UnicodeCharacter0xf30f',
u'\ubc11': 'HangulSyllableMit',
u'\u5b94': 'CjkUnifiedIdeograph5b94',
u'\u2496': 'NumberFifteenFullStop',
u'\ue198': 'UnicodeCharacter0xe198',
u'\ua29a': 'YiSyllableZuop',
u'\u4e1d': 'CjkUnifiedIdeograph4e1d',
u'\u0b1f': 'OriyaLetterTta',
u'\ud421': 'HangulSyllablePwat',
u'\u9123': 'CjkUnifiedIdeograph9123',
u'\u73a4': 'CjkUnifiedIdeograph73a4',
u'\u3ca6': 'CjkUnifiedIdeograph3ca6',
u'\uf9a8': 'CjkCompatibilityIdeographf9a8',
u'\ubaaa': 'HangulSyllableMogg',
u'\u662d': 'CjkUnifiedIdeograph662d',
u'\u87ac': 'CjkUnifiedIdeograph87ac',
u'\u232f': 'Symmetry',
u'\uec31': 'UnicodeCharacter0xec31',
u'\ua933': 'RejangLetterTa',
u'\u54b6': 'CjkUnifiedIdeograph54b6',
u'\u11b8': 'HangulJongseongPieup',
u'\ud2ba': 'HangulSyllableTeugg',
u'\u7e3d': 'CjkUnifiedIdeograph7e3d',
u'\u9fbc': 'UnicodeCharacter0x9fbc',
u'\u3b3f': 'CjkUnifiedIdeograph3b3f',
u'\u0441': 'CyrillicSmallLetterEs',
u'\uc143': 'HangulSyllableSelb',
u'\u8245': 'CjkUnifiedIdeograph8245',
u'\u6cc6': 'CjkUnifiedIdeograph6cc6',
u'\u29c8': 'SquaredSquare',
u'\ueaca': 'UnicodeCharacter0xeaca',
u'\ub7cc': 'HangulSyllableRyak',
u'\u534f': 'CjkUnifiedIdeograph534f',
u'\u1c51': 'OlChikiDigitOne',
u'\ud953': 'UnicodeCharacter0xd953',
u'\u9a55': 'CjkUnifiedIdeograph9a55',
u'\u41d8': 'CjkUnifiedIdeograph41d8',
u'\u02da': 'RingAbove',
u'\ucfdc': 'HangulSyllableKyok',
u'\u6b5f': 'CjkUnifiedIdeograph6b5f',
u'\u88de': 'CjkUnifiedIdeograph88de',
u'\u3461': 'CjkUnifiedIdeograph3461',
u'\uf163': 'UnicodeCharacter0xf163',
u'\ub265': 'HangulSyllableNwilt',
u'\u59e8': 'CjkUnifiedIdeograph59e8',
u'\u1aea': 'UnicodeCharacter0x1aea',
u'\ue7ec': 'UnicodeCharacter0xe7ec',
u'\ua0ee': 'YiSyllableVot',
u'\u4c71': 'CjkUnifiedIdeograph4c71',
u'\u0973': 'UnicodeCharacter0x973',
u'\uca75': 'HangulSyllableJjelt',
u'\u9777': 'CjkUnifiedIdeograph9777',
u'\u71f8': 'CjkUnifiedIdeograph71f8',
u'\u32fa': 'CircledKatakanaRo',
u'\ufffc': 'ObjectReplacementCharacter',
u'\ub8fe': 'HangulSyllableRuj',
u'\u8600': 'CjkUnifiedIdeograph8600',
u'\ua283': 'YiSyllableWap',
u'\u4502': 'CjkUnifiedIdeograph4502',
u'\u6185': 'CjkUnifiedIdeograph6185',
u'\ucf06': 'HangulSyllableKenh',
u'\ueb89': 'UnicodeCharacter0xeb89',
u'\u140c': 'CanadianSyllabicsWe',
u'\u308f': 'HiraganaLetterWa',
u'\u9e10': 'CjkUnifiedIdeograph9e10',
u'\uba93': 'HangulSyllableMyed',
u'\u5d12': 'CjkUnifiedIdeograph5d12',
u'\u7995': 'CjkUnifiedIdeograph7995',
u'\ue716': 'UnicodeCharacter0xe716',
u'\u0399': 'GreekCapitalLetterIota',
u'\u8d9d': 'CjkUnifiedIdeograph8d9d',
u'\u2c1c': 'GlagoliticCapitalLetterTsi',
u'\u489f': 'CjkUnifiedIdeograph489f',
u'\ub620': 'HangulSyllableDdom',
u'\ud2a3': 'HangulSyllableTyud',
u'\u7522': 'CjkUnifiedIdeograph7522',
u'\uff26': 'FullwidthLatinCapitalLetterF',
u'\u1ba9': 'SundaneseVowelSignPaneuleung',
u'\ua5ad': 'VaiSyllableShu',
u'\u442c': 'CjkUnifiedIdeograph442c',
u'\u60af': 'CjkUnifiedIdeograph60af',
u'\uce30': 'HangulSyllableCeum',
u'\ueab3': 'UnicodeCharacter0xeab3',
u'\u1736': 'PhilippineDoublePunctuation',
u'\u33b9': 'SquareMvMega',
u'\u913a': 'CjkUnifiedIdeograph913a',
u'\ubdbd': 'HangulSyllableBwenj',
u'\u5c3c': 'CjkUnifiedIdeograph5c3c',
u'\u78bf': 'CjkUnifiedIdeograph78bf',
u'\ue640': 'UnicodeCharacter0xe640',
u'\u02c3': 'ModifierLetterRightArrowhead',
u'\u8cc7': 'CjkUnifiedIdeograph8cc7',
u'\u2f46': 'KangxiRadicalNot',
u'\u4bc9': 'CjkUnifiedIdeograph4bc9',
u'\ua94a': 'RejangVowelSignAi',
u'\ud5cd': 'HangulSyllableHeonj',
u'\u744c': 'CjkUnifiedIdeograph744c',
u'\ufe50': 'SmallComma',
u'\u1ad3': 'UnicodeCharacter0x1ad3',
u'\ua4d7': 'UnicodeCharacter0xa4d7',
u'\u4756': 'CjkUnifiedIdeograph4756',
u'\u63d9': 'CjkUnifiedIdeograph63d9',
u'\uc15a': 'HangulSyllableSyeonh',
u'\ueddd': 'UnicodeCharacter0xeddd',
u'\u1660': 'CanadianSyllabicsCarrierTsa',
u'\u32e3': 'CircledKatakanaTo',
u'\u9064': 'CjkUnifiedIdeograph9064',
u'\ubce7': 'HangulSyllableByelh',
u'\u5f66': 'CjkUnifiedIdeograph5f66',
u'\u7be9': 'CjkUnifiedIdeograph7be9',
u'\ud96a': 'UnicodeCharacter0xd96a',
u'\u05ed': 'UnicodeCharacter0x5ed',
u'\u8ff1': 'CjkUnifiedIdeograph8ff1',
u'\u2e70': 'UnicodeCharacter0x2e70',
u'\u4af3': 'CjkUnifiedIdeograph4af3',
u'\ua874': 'PhagspaSingleHeadMark',
u'\ud4f7': 'HangulSyllablePyulh',
u'\u7776': 'CjkUnifiedIdeograph7776',
u'\uf17a': 'UnicodeCharacter0xf17a',
u'\u1dfd': 'UnicodeCharacter0x1dfd',
u'\u2701': 'UpperBladeScissors',
u'\u4680': 'CjkUnifiedIdeograph4680',
u'\ua105': 'YiSyllableDie',
u'\uc084': 'HangulSyllableBbyim',
u'\u6c07': 'CjkUnifiedIdeograph6c07',
u'\uf60b': 'UnicodeCharacter0xf60b',
u'\u098a': 'BengaliLetterUu',
u'\u938e': 'CjkUnifiedIdeograph938e',
u'\u3f11': 'CjkUnifiedIdeograph3f11',
u'\u5e90': 'CjkUnifiedIdeograph5e90',
u'\ub915': 'HangulSyllableRweob',
u'\ud894': 'UnicodeCharacter0xd894',
u'\u0e1b': 'ThaiCharacterPoPla',
u'\u219a': 'LeftwardsArrowWithStroke',
u'\u881f': 'CjkUnifiedIdeograph881f',
u'\uab9e': 'UnicodeCharacter0xab9e',
u'\u5721': 'CjkUnifiedIdeograph5721',
u'\u76a0': 'CjkUnifiedIdeograph76a0',
u'\ud125': 'HangulSyllableTyaeb',
u'\uf0a4': 'UnicodeCharacter0xf0a4',
u'\u262b': 'FarsiSymbol',
u'\u39aa': 'CjkUnifiedIdeograph39aa',
u'\ua02f': 'YiSyllableBup',
u'\uc3ae': 'HangulSyllableSsyeolp',
u'\u6f31': 'CjkUnifiedIdeograph6f31',
u'\ue935': 'UnicodeCharacter0xe935',
u'\u08b4': 'UnicodeCharacter0x8b4',
u'\u92b8': 'CjkUnifiedIdeograph92b8',
u'\u3e3b': 'CjkUnifiedIdeograph3e3b',
u'\u51ba': 'CjkUnifiedIdeograph51ba',
u'\ub83f': 'HangulSyllableRyeoh',
u'\udbbe': 'UnicodeCharacter0xdbbe',
u'\u0145': 'LatinCapitalLetterNWithCedilla',
u'\u20c4': 'UnicodeCharacter0x20c4',
u'\u8b49': 'CjkUnifiedIdeograph8b49',
u'\uaac8': 'UnicodeCharacter0xaac8',
u'\u564b': 'CjkUnifiedIdeograph564b',
u'\u69ca': 'CjkUnifiedIdeograph69ca',
u'\ud04f': 'HangulSyllableKwih',
u'\uf3ce': 'UnicodeCharacter0xf3ce',
u'\u1955': 'TaiLeLetterYa',
u'\u38d4': 'CjkUnifiedIdeograph38d4',
u'\ua359': 'YiSyllableZhurx',
u'\uc2d8': 'HangulSyllableSyik',
u'\u6e5b': 'CjkUnifiedIdeograph6e5b',
u'\ue85f': 'UnicodeCharacter0xe85f',
u'\u0bde': 'UnicodeCharacter0xbde',
u'\u95e2': 'CjkUnifiedIdeograph95e2',
u'\u3165': 'HangulLetterSsangnieun',
u'\u50e4': 'CjkUnifiedIdeograph50e4',
u'\ubb69': 'HangulSyllableMweot',
u'\udae8': 'UnicodeCharacter0xdae8',
u'\u23ee': 'UnicodeCharacter0x23ee',
u'\u8a73': 'CjkUnifiedIdeograph8a73',
u'\uadf2': 'HangulSyllableGyuj',
u'\u4975': 'CjkUnifiedIdeograph4975',
u'\u68f4': 'CjkUnifiedIdeograph68f4',
u'\ud379': 'HangulSyllablePyaet',
u'\uf2f8': 'UnicodeCharacter0xf2f8',
u'\u187f': 'UnicodeCharacter0x187f',
u'\u3bfe': 'CjkUnifiedIdeograph3bfe',
u'\u4e06': 'CjkUnifiedIdeograph4e06',
u'\u6a89': 'CjkUnifiedIdeograph6a89',
u'\u0b08': 'OriyaLetterIi',
u'\u378b': 'CjkUnifiedIdeograph378b',
u'\uc80a': 'HangulSyllableJeolm',
u'\uf48d': 'UnicodeCharacter0xf48d',
u'\u950c': 'CjkUnifiedIdeograph950c',
u'\ub18f': 'HangulSyllableNoc',
u'\u6616': 'CjkUnifiedIdeograph6616',
u'\u2318': 'PlaceOfInterestSign',
u'\u4f9b': 'CjkUnifiedIdeograph4f9b',
u'\ue01a': 'UnicodeCharacter0xe01a',
u'\u0c9d': 'KannadaLetterJha',
u'\uad1c': 'HangulSyllableGwaen',
u'\uc99f': 'HangulSyllableJeuc',
u'\u96a1': 'CjkUnifiedIdeograph96a1',
u'\u7e26': 'CjkUnifiedIdeograph7e26',
u'\u3b28': 'CjkUnifiedIdeograph3b28',
u'\u67ab': 'CjkUnifiedIdeograph67ab',
u'\uf82a': 'UnicodeCharacter0xf82a',
u'\u24ad': 'ParenthesizedLatinSmallLetterR',
u'\uc52c': 'HangulSyllableSsin',
u'\ue1af': 'UnicodeCharacter0xe1af',
u'\u822e': 'CjkUnifiedIdeograph822e',
u'\uaeb1': 'HangulSyllableGgyaeb',
u'\u5338': 'CjkUnifiedIdeograph5338',
u'\u7fbb': 'CjkUnifiedIdeograph7fbb',
u'\u103a': 'MyanmarSignAsat',
u'\u3cbd': 'CjkUnifiedIdeograph3cbd',
u'\udd3c': 'UnicodeCharacter0xdd3c',
u'\uf9bf': 'CjkCompatibilityIdeographf9bf',
u'\u9a3e': 'CjkUnifiedIdeograph9a3e',
u'\uc6c1': 'HangulSyllableUb',
u'\u83c3': 'CjkUnifiedIdeograph83c3',
u'\u6b48': 'CjkUnifiedIdeograph6b48',
u'\u284a': 'BraillePatternDots247',
u'\u54cd': 'CjkUnifiedIdeograph54cd',
u'\uf54c': 'UnicodeCharacter0xf54c',
u'\u11cf': 'HangulJongseongRieultikeuthieuh',
u'\ub24e': 'HangulSyllableNwebs',
u'\uded1': 'UnicodeCharacter0xded1',
u'\u9bd3': 'CjkUnifiedIdeograph9bd3',
u'\u405a': 'CjkUnifiedIdeograph405a',
u'\u6cdd': 'CjkUnifiedIdeograph6cdd',
u'\u0d5c': 'UnicodeCharacter0xd5c',
u'\u29df': 'DoubleendedMultimap',
u'\uca5e': 'HangulSyllableJjeobs',
u'\uf6e1': 'UnicodeCharacter0xf6e1',
u'\u9760': 'CjkUnifiedIdeograph9760',
u'\ub3e3': 'HangulSyllableDwags',
u'\u586a': 'CjkUnifiedIdeograph586a',
u'\u256c': 'BoxDrawingsDoubleVerticalAndHorizontal',
u'\u41ef': 'CjkUnifiedIdeograph41ef',
u'\ue26e': 'UnicodeCharacter0xe26e',
u'\u0ef1': 'UnicodeCharacter0xef1',
u'\uaf70': 'HangulSyllableGgwaels',
u'\ucbf3': 'HangulSyllableJjyigs',
u'\u88f5': 'CjkUnifiedIdeograph88f5',
u'\u707a': 'CjkUnifiedIdeograph707a',
u'\u3d7c': 'CjkUnifiedIdeograph3d7c',
u'\u59ff': 'CjkUnifiedIdeograph59ff',
u'\ufa7e': 'CjkCompatibilityIdeographfa7e',
u'\uc780': 'HangulSyllableIls',
u'\u6303': 'CjkUnifiedIdeograph6303',
u'\u8482': 'CjkUnifiedIdeograph8482',
u'\u2005': 'FourperemSpace',
u'\ued07': 'UnicodeCharacter0xed07',
u'\uaa09': 'ChamLetterGha',
u'\u558c': 'CjkUnifiedIdeograph558c',
u'\u128e': 'UnicodeCharacter0x128e',
u'\udf90': 'UnicodeCharacter0xdf90',
u'\u7b13': 'CjkUnifiedIdeograph7b13',
u'\u9c92': 'CjkUnifiedIdeograph9c92',
u'\u3815': 'CjkUnifiedIdeograph3815',
u'\u0517': 'CyrillicSmallLetterRha',
u'\uc219': 'HangulSyllableSug',
u'\u8f1b': 'CjkUnifiedIdeograph8f1b',
u'\u6d9c': 'CjkUnifiedIdeograph6d9c',
u'\u2a9e': 'SimilarOrGreaterthan',
u'\uf7a0': 'UnicodeCharacter0xf7a0',
u'\ub4a2': 'HangulSyllableDwep',
u'\u5025': 'CjkUnifiedIdeograph5025',
u'\u1d27': 'GreekLetterSmallCapitalLamda',
u'\uda29': 'UnicodeCharacter0xda29',
u'\ua72b': 'LatinSmallLetterTresillo',
u'\u42ae': 'CjkUnifiedIdeograph42ae',
u'\u0fb0': 'TibetanSubjoinedLetterA',
u'\uccb2': 'HangulSyllableCeop',
u'\u6835': 'CjkUnifiedIdeograph6835',
u'\u89b4': 'CjkUnifiedIdeograph89b4',
u'\u3537': 'CjkUnifiedIdeograph3537',
u'\uf239': 'UnicodeCharacter0xf239',
u'\ubf3b': 'HangulSyllableBbyec',
u'\u5abe': 'CjkUnifiedIdeograph5abe',
u'\u27c0': 'ThreeDimensionalAngle',
u'\ue4c2': 'UnicodeCharacter0xe4c2',
u'\ua1c4': 'YiSyllableLuox',
u'\u4d47': 'CjkUnifiedIdeograph4d47',
u'\u0a49': 'UnicodeCharacter0xa49',
u'\ud74b': 'HangulSyllableHyuc',
u'\u944d': 'CjkUnifiedIdeograph944d',
u'\u72ce': 'CjkUnifiedIdeograph72ce',
u'\u3fd0': 'CjkUnifiedIdeograph3fd0',
u'\ufcd2': 'ArabicLigatureNoonWithJeemInitialForm',
u'\ub9d4': 'HangulSyllableMals',
u'\u6557': 'CjkUnifiedIdeograph6557',
u'\u86d6': 'CjkUnifiedIdeograph86d6',
u'\u2259': 'Estimates',
u'\uef5b': 'UnicodeCharacter0xef5b',
u'\uac5d': 'HangulSyllableGyaelg',
u'\u57e0': 'CjkUnifiedIdeograph57e0',
u'\u14e2': 'CanadianSyllabicsLwo',
u'\ud1e4': 'HangulSyllableTwaels',
u'\u7d67': 'CjkUnifiedIdeograph7d67',
u'\u9ee6': 'CjkUnifiedIdeograph9ee6',
u'\u3a69': 'CjkUnifiedIdeograph3a69',
u'\u076b': 'ArabicLetterRehWithTwoDotsVerticallyAbove',
u'\uc46d': 'HangulSyllableSsulg',
u'\u816f': 'CjkUnifiedIdeograph816f',
u'\u6ff0': 'CjkUnifiedIdeograph6ff0',
u'\u2cf2': 'UnicodeCharacter0x2cf2',
u'\ue9f4': 'UnicodeCharacter0xe9f4',
u'\ub6f6': 'HangulSyllableDdwinh',
u'\u5279': 'CjkUnifiedIdeograph5279',
u'\u1f7b': 'GreekSmallLetterUpsilonWithOxia',
u'\udc7d': 'UnicodeCharacter0xdc7d',
u'\u997f': 'CjkUnifiedIdeograph997f',
u'\ue981': 'UnicodeCharacter0xe981',
u'\u0e04': 'ThaiCharacterKhoKhwai',
u'\u2287': 'SupersetOfOrEqualTo',
u'\u8408': 'CjkUnifiedIdeograph8408',
u'\ub88b': 'HangulSyllableRwas',
u'\u470a': 'CjkUnifiedIdeograph470a',
u'\u7b8d': 'CjkUnifiedIdeograph7b8d',
u'\udd0e': 'UnicodeCharacter0xdd0e',
u'\uf191': 'UnicodeCharacter0xf191',
u'\u1614': 'CanadianSyllabicsCarrierJu',
u'\u4a97': 'CjkUnifiedIdeograph4a97',
u'\uac18': 'HangulSyllableGak',
u'\uc09b': 'HangulSyllableBbilb',
u'\u6f1a': 'CjkUnifiedIdeograph6f1a',
u'\ue51e': 'UnicodeCharacter0xe51e',
u'\u19a1': 'NewTaiLueLetterHighDa',
u'\u9fa5': 'CjkUnifiedIdeograph9fa5',
u'\u3e24': 'CjkUnifiedIdeograph3e24',
u'\u52a7': 'CjkUnifiedIdeograph52a7',
u'\ub428': 'HangulSyllableDoem',
u'\ue8ab': 'UnicodeCharacter0xe8ab',
u'\u772a': 'CjkUnifiedIdeograph772a',
u'\u0d2e': 'MalayalamLetterMa',
u'\u21b1': 'UpwardsArrowWithTipRightwards',
u'\u8332': 'CjkUnifiedIdeograph8332',
u'\ua7b5': 'UnicodeCharacter0xa7b5',
u'\u4634': 'CjkUnifiedIdeograph4634',
u'\u7ab7': 'CjkUnifiedIdeograph7ab7',
u'\udc38': 'UnicodeCharacter0xdc38',
u'\uf0bb': 'UnicodeCharacter0xf0bb',
u'\u153e': 'CanadianSyllabicsY',
u'\u49c1': 'CjkUnifiedIdeograph49c1',
u'\uab42': 'UnicodeCharacter0xab42',
u'\ucfc5': 'HangulSyllableKyog',
u'\u6e44': 'CjkUnifiedIdeograph6e44',
u'\ue448': 'UnicodeCharacter0xe448',
u'\u18cb': 'UnicodeCharacter0x18cb',
u'\u9ecf': 'CjkUnifiedIdeograph9ecf',
u'\u3d4e': 'CjkUnifiedIdeograph3d4e',
u'\u51d1': 'CjkUnifiedIdeograph51d1',
u'\ub352': 'HangulSyllableDyaep',
u'\ud7d5': 'UnicodeCharacter0xd7d5',
u'\u7654': 'CjkUnifiedIdeograph7654',
u'\u0c58': 'TeluguLetterTsa',
u'\u20db': 'CombiningThreeDotsAbove',
u'\u825c': 'CjkUnifiedIdeograph825c',
u'\ua6df': 'UnicodeCharacter0xa6df',
u'\u455e': 'CjkUnifiedIdeograph455e',
u'\u79e1': 'CjkUnifiedIdeograph79e1',
u'\udb62': 'UnicodeCharacter0xdb62',
u'\uffe5': 'FullwidthYenSign',
u'\u1468': 'CanadianSyllabicsTti',
u'\u48eb': 'CjkUnifiedIdeograph48eb',
u'\uaa6c': 'UnicodeCharacter0xaa6c',
u'\uceef': 'HangulSyllableKeolb',
u'\u6d6e': 'CjkUnifiedIdeograph6d6e',
u'\ue372': 'UnicodeCharacter0xe372',
u'\u07f5': 'NkoLowToneApostrophe',
u'\u9df9': 'CjkUnifiedIdeograph9df9',
u'\u3c78': 'CjkUnifiedIdeograph3c78',
u'\u50fb': 'CjkUnifiedIdeograph50fb',
u'\ub27c': 'HangulSyllableNyul',
u'\ud6ff': 'HangulSyllableHwegs',
u'\u757e': 'CjkUnifiedIdeograph757e',
u'\uec03': 'UnicodeCharacter0xec03',
u'\u0b82': 'TamilSignAnusvara',
u'\u8186': 'CjkUnifiedIdeograph8186',
u'\u2509': 'BoxDrawingsHeavyQuadrupleDashHorizontal',
u'\u4488': 'CjkUnifiedIdeograph4488',
u'\ubb0d': 'HangulSyllableMoeb',
u'\uda8c': 'UnicodeCharacter0xda8c',
u'\u7e0f': 'CjkUnifiedIdeograph7e0f',
u'\uf413': 'UnicodeCharacter0xf413',
u'\u1392': 'EthiopicTonalMarkRikrik',
u'\u8a17': 'CjkUnifiedIdeograph8a17',
u'\ua996': 'UnicodeCharacter0xa996',
u'\u4d19': 'CjkUnifiedIdeograph4d19',
u'\u6c98': 'CjkUnifiedIdeograph6c98',
u'\uc31d': 'HangulSyllableSsaelg',
u'\ue29c': 'UnicodeCharacter0xe29c',
u'\u1c23': 'LepchaLetterA',
u'\u3ba2': 'CjkUnifiedIdeograph3ba2',
u'\u9227': 'CjkUnifiedIdeograph9227',
u'\ub1a6': 'HangulSyllableNwabs',
u'\u5529': 'CjkUnifiedIdeograph5529',
u'\u74a8': 'CjkUnifiedIdeograph74a8',
u'\ueb2d': 'UnicodeCharacter0xeb2d',
u'\u0aac': 'GujaratiLetterBa',
u'\u80b0': 'CjkUnifiedIdeograph80b0',
u'\u2433': 'UnicodeCharacter0x2433',
u'\u43b2': 'CjkUnifiedIdeograph43b2',
u'\uba37': 'HangulSyllableMyaeh',
u'\ud9b6': 'UnicodeCharacter0xd9b6',
u'\u7d39': 'CjkUnifiedIdeograph7d39',
u'\uf33d': 'UnicodeCharacter0xf33d',
u'\u12bc': 'EthiopicSyllableKxee',
u'\u8941': 'CjkUnifiedIdeograph8941',
u'\ua8c0': 'SaurashtraVowelSignAi',
u'\u4c43': 'CjkUnifiedIdeograph4c43',
u'\u6bc2': 'CjkUnifiedIdeograph6bc2',
u'\uc247': 'HangulSyllableSweos',
u'\ue1c6': 'UnicodeCharacter0xe1c6',
u'\u1b4d': 'UnicodeCharacter0x1b4d',
u'\u3acc': 'CjkUnifiedIdeograph3acc',
u'\u9151': 'CjkUnifiedIdeograph9151',
u'\ub0d0': 'HangulSyllableNya',
u'\u5453': 'CjkUnifiedIdeograph5453',
u'\u73d2': 'CjkUnifiedIdeograph73d2',
u'\uea57': 'UnicodeCharacter0xea57',
u'\u09d6': 'UnicodeCharacter0x9d6',
u'\u8fda': 'CjkUnifiedIdeograph8fda',
u'\u235d': 'AplFunctionalSymbolUpShoeJot',
u'\u42dc': 'CjkUnifiedIdeograph42dc',
u'\ub961': 'HangulSyllableRyulg',
u'\ud8e0': 'UnicodeCharacter0xd8e0',
u'\u7c63': 'CjkUnifiedIdeograph7c63',
u'\uf267': 'UnicodeCharacter0xf267',
u'\u11e6': 'HangulJongseongKapyeounpieup',
u'\u886b': 'CjkUnifiedIdeograph886b',
u'\u97ea': 'CjkUnifiedIdeograph97ea',
u'\u4b6d': 'CjkUnifiedIdeograph4b6d',
u'\u6aec': 'CjkUnifiedIdeograph6aec',
u'\uc171': 'HangulSyllableSyeg',
u'\ue0f0': 'UnicodeCharacter0xe0f0',
u'\u1a77': 'UnicodeCharacter0x1a77',
u'\u39f6': 'CjkUnifiedIdeograph39f6',
u'\u907b': 'CjkUnifiedIdeograph907b',
u'\ubffa': 'HangulSyllableBbweobs',
u'\u537d': 'CjkUnifiedIdeograph537d',
u'\u72fc': 'CjkUnifiedIdeograph72fc',
u'\u6881': 'CjkUnifiedIdeograph6881',
u'\u0900': 'UnicodeCharacter0x900',
u'\u2d83': 'EthiopicSyllableSoa',
u'\uca02': 'HangulSyllableJjaelm',
u'\uee85': 'UnicodeCharacter0xee85',
u'\u8f04': 'CjkUnifiedIdeograph8f04',
u'\ua387': 'YiSyllableRrep',
u'\u5c0e': 'CjkUnifiedIdeograph5c0e',
u'\u7091': 'CjkUnifiedIdeograph7091',
u'\u1110': 'HangulChoseongThieuth',
u'\u3593': 'CjkUnifiedIdeograph3593',
u'\ud212': 'HangulSyllableTyogg',
u'\uf695': 'UnicodeCharacter0xf695',
u'\u9714': 'CjkUnifiedIdeograph9714',
u'\ucb97': 'HangulSyllableJjwec',
u'\u8c99': 'CjkUnifiedIdeograph8c99',
u'\u641e': 'CjkUnifiedIdeograph641e',
u'\u3920': 'CjkUnifiedIdeograph3920',
u'\u5da3': 'CjkUnifiedIdeograph5da3',
u'\ufa22': 'CjkCompatibilityIdeographfa22',
u'\u1ea5': 'LatinSmallLetterAWithCircumflexAndAcute',
u'\ubf24': 'HangulSyllableBbye',
u'\ud3a7': 'HangulSyllablePelh',
u'\u94a9': 'CjkUnifiedIdeograph94a9',
u'\u4130': 'CjkUnifiedIdeograph4130',
u'\u65b3': 'CjkUnifiedIdeograph65b3',
u'\u0232': 'LatinCapitalLetterYWithMacron',
u'\u26b5': 'Juno',
u'\uc734': 'HangulSyllableYuss',
u'\ufbb7': 'UnicodeCharacter0xfbb7',
u'\u9836': 'CjkUnifiedIdeograph9836',
u'\ubcb9': 'HangulSyllableBet',
u'\u6940': 'CjkUnifiedIdeograph6940',
u'\u2a42': 'UnionWithOverbar',
u'\u4ec5': 'CjkUnifiedIdeograph4ec5',
u'\uef44': 'UnicodeCharacter0xef44',
u'\u03c7': 'GreekSmallLetterChi',
u'\ua046': 'YiSyllablePot',
u'\uc4c9': 'HangulSyllableSswib',
u'\u99cb': 'CjkUnifiedIdeograph99cb',
u'\u7150': 'CjkUnifiedIdeograph7150',
u'\u3252': 'CircledNumberTwentyTwo',
u'\u56d5': 'CjkUnifiedIdeograph56d5',
u'\uf754': 'UnicodeCharacter0xf754',
u'\u2bd7': 'UnicodeCharacter0x2bd7',
u'\uc856': 'HangulSyllableJyegg',
u'\uecd9': 'UnicodeCharacter0xecd9',
u'\u8d58': 'CjkUnifiedIdeograph8d58',
u'\ua1db': 'YiSyllableGix',
u'\u5a62': 'CjkUnifiedIdeograph5a62',
u'\u7ee5': 'CjkUnifiedIdeograph7ee5',
u'\u1f64': 'GreekSmallLetterOmegaWithPsiliAndOxia',
u'\u33e7': 'IdeographicTelegraphSymbolForDayEight',
u'\ud066': 'HangulSyllableKyuj',
u'\uf4e9': 'UnicodeCharacter0xf4e9',
u'\u9568': 'CjkUnifiedIdeograph9568',
u'\uc9eb': 'HangulSyllableJjalh',
u'\u8aed': 'CjkUnifiedIdeograph8aed',
u'\u6272': 'CjkUnifiedIdeograph6272',
u'\u2774': 'MediumLeftCurlyBracketOrnament',
u'\u5bf7': 'CjkUnifiedIdeograph5bf7',
u'\uf876': 'UnicodeCharacter0xf876',
u'\u1cf9': 'UnicodeCharacter0x1cf9',
u'\ubd78': 'HangulSyllableByoss',
u'\ud1fb': 'HangulSyllableToed',
u'\u92fd': 'CjkUnifiedIdeograph92fd',
u'\ua801': 'SylotiNagriLetterI',
u'\u4f84': 'CjkUnifiedIdeograph4f84',
u'\x86': 'UnicodeCharacter0x86',
u'\uc588': 'HangulSyllableYals',
u'\u790b': 'CjkUnifiedIdeograph790b',
u'\u868a': 'CjkUnifiedIdeograph868a',
u'\u3a0d': 'CjkUnifiedIdeograph3a0d',
u'\uff0f': 'FullwidthSolidus',
u'\ub011': 'HangulSyllableGgwinj',
u'\u5794': 'CjkUnifiedIdeograph5794',
u'\u2896': 'BraillePatternDots2358',
u'\ued98': 'UnicodeCharacter0xed98',
u'\uae9a': 'HangulSyllableGgyaj',
u'\u421d': 'CjkUnifiedIdeograph421d',
u'\u071f': 'SyriacLetterKaph',
u'\ud821': 'UnicodeCharacter0xd821',
u'\u9d23': 'CjkUnifiedIdeograph9d23',
u'\u7fa4': 'CjkUnifiedIdeograph7fa4',
u'\u30a6': 'KatakanaLetterU',
u'\uf5a8': 'UnicodeCharacter0xf5a8',
u'\ub6aa': 'HangulSyllableDdulp',
u'\u6a2d': 'CjkUnifiedIdeograph6a2d',
u'\u8bac': 'CjkUnifiedIdeograph8bac',
u'\u2f2f': 'KangxiRadicalWork',
u'\ue031': 'UnicodeCharacter0xe031',
u'\ua533': 'VaiSyllableTi',
u'\u58b6': 'CjkUnifiedIdeograph58b6',
u'\u1db8': 'ModifierLetterSmallCapitalU',
u'\udeba': 'UnicodeCharacter0xdeba',
u'\u723d': 'CjkUnifiedIdeograph723d',
u'\u93bc': 'CjkUnifiedIdeograph93bc',
u'\u373f': 'CjkUnifiedIdeograph373f',
u'\u0841': 'UnicodeCharacter0x841',
u'\ucd43': 'HangulSyllableCwaegs',
u'\u8e45': 'CjkUnifiedIdeograph8e45',
u'\u60c6': 'CjkUnifiedIdeograph60c6',
u'\u25c8': 'WhiteDiamondContainingBlackSmallDiamond',
u'\ue6ca': 'UnicodeCharacter0xe6ca',
u'\ubbcc': 'HangulSyllableMeuls',
u'\u5f4f': 'CjkUnifiedIdeograph5f4f',
u'\u1051': 'MyanmarLetterSsa',
u'\ud553': 'HangulSyllablePic',
u'\u9655': 'CjkUnifiedIdeograph9655',
u'\u4dd8': 'HexagramForInnocence',
u'\u0eda': 'UnicodeCharacter0xeda',
u'\uc3dc': 'HangulSyllableSson',
u'\u675f': 'CjkUnifiedIdeograph675f',
u'\u84de': 'CjkUnifiedIdeograph84de',
u'\u3861': 'CjkUnifiedIdeograph3861',
u'\ufd63': 'ArabicLigatureSeenWithMeemWithMeemInitialForm',
u'\ube65': 'HangulSyllableBbanj',
u'\u55e8': 'CjkUnifiedIdeograph55e8',
u'\u16ea': 'RunicLetterX',
u'\uebec': 'UnicodeCharacter0xebec',
u'\uacee': 'HangulSyllableGolp',
u'\u4071': 'CjkUnifiedIdeograph4071',
u'\u0573': 'ArmenianSmallLetterCheh',
u'\uc675': 'HangulSyllableWaet',
u'\u9b77': 'CjkUnifiedIdeograph9b77',
u'\u7df8': 'CjkUnifiedIdeograph7df8',
u'\u3efa': 'CjkUnifiedIdeograph3efa',
u'\uf3fc': 'UnicodeCharacter0xf3fc',
u'\ub4fe': 'HangulSyllableDyinh',
u'\u8a00': 'CjkUnifiedIdeograph8a00',
u'\uae83': 'HangulSyllableGgaeh',
u'\u4902': 'CjkUnifiedIdeograph4902',
u'\u6d85': 'CjkUnifiedIdeograph6d85',
u'\uc306': 'HangulSyllableSsalp',
u'\ue789': 'UnicodeCharacter0xe789',
u'\u180c': 'MongolianFreeVariationSelectorTwo',
u'\u3c8f': 'CjkUnifiedIdeograph3c8f',
u'\u9210': 'CjkUnifiedIdeograph9210',
u'\ub693': 'HangulSyllableDdyos',
u'\u5112': 'CjkUnifiedIdeograph5112',
u'\u7595': 'CjkUnifiedIdeograph7595',
u'\ueb16': 'UnicodeCharacter0xeb16',
u'\u0f99': 'TibetanSubjoinedLetterNya',
u'\u819d': 'CjkUnifiedIdeograph819d',
u'\u201c': 'LeftDoubleQuotationMark',
u'\u449f': 'CjkUnifiedIdeograph449f',
u'\uba20': 'HangulSyllableMyaen',
u'\udea3': 'UnicodeCharacter0xdea3',
u'\u7922': 'CjkUnifiedIdeograph7922',
u'\uf326': 'UnicodeCharacter0xf326',
u'\u17a9': 'KhmerIndependentVowelQuu',
u'\ua9ad': 'UnicodeCharacter0xa9ad',
u'\u482c': 'CjkUnifiedIdeograph482c',
u'\u6caf': 'CjkUnifiedIdeograph6caf',
u'\uc230': 'HangulSyllableSuk',
u'\ue6b3': 'UnicodeCharacter0xe6b3',
u'\u1b36': 'BalineseVowelSignUlu',
u'\u3fb9': 'CjkUnifiedIdeograph3fb9',
u'\u9d3a': 'CjkUnifiedIdeograph9d3a',
u'\ub1bd': 'HangulSyllableNwaelt',
u'\u503c': 'CjkUnifiedIdeograph503c',
u'\u74bf': 'CjkUnifiedIdeograph74bf',
u'\uea40': 'UnicodeCharacter0xea40',
u'\u0ec3': 'LaoVowelSignAy',
u'\u80c7': 'CjkUnifiedIdeograph80c7',
u'\u2346': 'AplFunctionalSymbolRightwardsVane',
u'\u47c9': 'CjkUnifiedIdeograph47c9',
u'\ua54a': 'VaiSyllableAn',
u'\ud9cd': 'UnicodeCharacter0xd9cd',
u'\u784c': 'CjkUnifiedIdeograph784c',
u'\uf250': 'UnicodeCharacter0xf250',
u'\u16d3': 'RunicLetterShorttwigbjarkanB',
u'\ua8d7': 'SaurashtraDigitSeven',
u'\u4b56': 'CjkUnifiedIdeograph4b56',
u'\u6fd9': 'CjkUnifiedIdeograph6fd9',
u'\ucd5a': 'HangulSyllableCwaep',
u'\ue1dd': 'UnicodeCharacter0xe1dd',
u'\u1a60': 'UnicodeCharacter0x1a60',
u'\u3ee3': 'CjkUnifiedIdeograph3ee3',
u'\u9c64': 'CjkUnifiedIdeograph9c64',
u'\ub0e7': 'HangulSyllableNyac',
u'\u5366': 'CjkUnifiedIdeograph5366',
u'\u77e9': 'CjkUnifiedIdeograph77e9',
u'\ud56a': 'HangulSyllableHabs',
u'\u09ed': 'BengaliDigitSeven',
u'\u83f1': 'CjkUnifiedIdeograph83f1',
u'\u2270': 'NeitherLessthanNorEqualTo',
u'\u46f3': 'CjkUnifiedIdeograph46f3',
u'\ua474': 'YiSyllableYip',
u'\ud8f7': 'UnicodeCharacter0xd8f7',
u'\u7b76': 'CjkUnifiedIdeograph7b76',
u'\ufd7a': 'ArabicLigatureGhainWithMeemWithYehFinalForm',
u'\u11fd': 'UnicodeCharacter0x11fd',
u'\u2b01': 'NorthWestWhiteArrow',
u'\u4a80': 'CjkUnifiedIdeograph4a80',
u'\uad05': 'HangulSyllableGwalg',
u'\ucc84': 'HangulSyllableCyael',
u'\u6007': 'CjkUnifiedIdeograph6007',
u'\ufa0b': 'CjkCompatibilityIdeographfa0b',
u'\u058a': 'ArmenianHyphen',
u'\u9f8e': 'CjkUnifiedIdeograph9f8e',
u'\u3311': 'SquareGinii',
u'\u5290': 'CjkUnifiedIdeograph5290',
u'\ub515': 'HangulSyllableDig',
u'\ud494': 'HangulSyllablePweo',
u'\u021b': 'LatinSmallLetterTWithCommaBelow',
u'\u2d9a': 'UnicodeCharacter0x2d9a',
u'\u841f': 'CjkUnifiedIdeograph841f',
u'\ua79e': 'UnicodeCharacter0xa79e',
u'\u5b21': 'CjkUnifiedIdeograph5b21',
u'\u7aa0': 'CjkUnifiedIdeograph7aa0',
u'\udd25': 'UnicodeCharacter0xdd25',
u'\ufca4': 'ArabicLigatureTehWithMeemInitialForm',
u'\u2a2b': 'MinusSignWithFallingDots',
u'\u35aa': 'CjkUnifiedIdeograph35aa',
u'\uac2f': 'HangulSyllableGaes',
u'\ucfae': 'HangulSyllableKoenh',
u'\u6331': 'CjkUnifiedIdeograph6331',
u'\ue535': 'UnicodeCharacter0xe535',
u'\u04b4': 'CyrillicCapitalLigatureTeTse',
u'\u9eb8': 'CjkUnifiedIdeograph9eb8',
u'\u323b': 'ParenthesizedIdeographStudy',
u'\u5dba': 'CjkUnifiedIdeograph5dba',
u'\ub43f': 'HangulSyllableDyolb',
u'\ud7be': 'UnicodeCharacter0xd7be',
u'\u0d45': 'UnicodeCharacter0xd45',
u'\u2cc4': 'CopticCapitalLetterOldCopticShei',
u'\u8749': 'CjkUnifiedIdeograph8749',
u'\ua6c8': 'UnicodeCharacter0xa6c8',
u'\u5a4b': 'CjkUnifiedIdeograph5a4b',
u'\u65ca': 'CjkUnifiedIdeograph65ca',
u'\udc4f': 'UnicodeCharacter0xdc4f',
u'\uffce': 'HalfwidthHangulLetterWae',
u'\u1555': 'CanadianSyllabicsFi',
u'\u34d4': 'CjkUnifiedIdeograph34d4',
u'\uaf59': 'HangulSyllableGgwab',
u'\uced8': 'HangulSyllableKyaem',
u'\u625b': 'CjkUnifiedIdeograph625b',
u'\ue45f': 'UnicodeCharacter0xe45f',
u'\u07de': 'NkoLetterKa',
u'\u99e2': 'CjkUnifiedIdeograph99e2',
u'\u3d65': 'CjkUnifiedIdeograph3d65',
u'\u5ce4': 'CjkUnifiedIdeograph5ce4',
u'\ub769': 'HangulSyllableDdilg',
u'\ud6e8': 'HangulSyllableHweol',
u'\u0c6f': 'TeluguDigitNine',
u'\u2fee': 'UnicodeCharacter0x2fee',
u'\u8673': 'CjkUnifiedIdeograph8673',
u'\ua1f2': 'YiSyllableGut',
u'\u4575': 'CjkUnifiedIdeograph4575',
u'\u64f4': 'CjkUnifiedIdeograph64f4',
u'\udf79': 'UnicodeCharacter0xdf79',
u'\ufef8': 'ArabicLigatureLamWithAlefWithHamzaAboveFinalForm',
u'\u147f': 'CanadianSyllabicsWestcreeKwa',
u'\u37fe': 'CjkUnifiedIdeograph37fe',
u'\u4206': 'CjkUnifiedIdeograph4206',
u'\u6689': 'CjkUnifiedIdeograph6689',
u'\u0708': 'SyriacSupralinearColonSkewedLeft',
u'\u3b8b': 'CjkUnifiedIdeograph3b8b',
u'\uc40a': 'HangulSyllableSswaj',
u'\uf88d': 'UnicodeCharacter0xf88d',
u'\u990c': 'CjkUnifiedIdeograph990c',
u'\ubd8f': 'HangulSyllableBulh',
u'\u6a16': 'CjkUnifiedIdeograph6a16',
u'\u2f18': 'KangxiRadicalDivination',
u'\u439b': 'CjkUnifiedIdeograph439b',
u'\uec1a': 'UnicodeCharacter0xec1a',
u'\x9d': 'UnicodeCharacter0x9d',
u'\ua11c': 'YiSyllableTi',
u'\uc59f': 'HangulSyllableYaed',
u'\u9aa1': 'CjkUnifiedIdeograph9aa1',
u'\u7226': 'CjkUnifiedIdeograph7226',
u'\u3728': 'CjkUnifiedIdeograph3728',
u'\u6bab': 'CjkUnifiedIdeograph6bab',
u'\uf42a': 'UnicodeCharacter0xf42a',
u'\u28ad': 'BraillePatternDots13468',
u'\uc92c': 'HangulSyllableJweoss',
u'\uedaf': 'UnicodeCharacter0xedaf',
u'\u8e2e': 'CjkUnifiedIdeograph8e2e',
u'\ua2b1': 'YiSyllableCip',
u'\u5f38': 'CjkUnifiedIdeograph5f38',
u'\u73bb': 'CjkUnifiedIdeograph73bb',
u'\u1c3a': 'UnicodeCharacter0x1c3a',
u'\u30bd': 'KatakanaLetterSo',
u'\ud13c': 'HangulSyllableTeols',
u'\uf5bf': 'UnicodeCharacter0xf5bf',
u'\u963e': 'CjkUnifiedIdeograph963e',
u'\ucac1': 'HangulSyllableJjonj',
u'\u8fc3': 'CjkUnifiedIdeograph8fc3',
u'\u6748': 'CjkUnifiedIdeograph6748',
u'\u244a': 'OcrDoubleBackslash',
u'\u58cd': 'CjkUnifiedIdeograph58cd',
u'\uf94c': 'CjkCompatibilityIdeographf94c',
u'\u1dcf': 'CombiningZigzagBelow',
u'\ube4e': 'HangulSyllableBilm',
u'\ud2d1': 'HangulSyllableTeut',
u'\u97d3': 'CjkUnifiedIdeograph97d3',
u'\u4c5a': 'CjkUnifiedIdeograph4c5a',
u'\u60dd': 'CjkUnifiedIdeograph60dd',
u'\u015c': 'LatinCapitalLetterSWithCircumflex',
u'\u25df': 'LowerLeftQuadrantCircularArc',
u'\uc65e': 'HangulSyllableWaegg',
u'\ufae1': 'UnicodeCharacter0xfae1',
u'\u9b60': 'CjkUnifiedIdeograph9b60',
u'\ubfe3': 'HangulSyllableBbuc',
u'\u546a': 'CjkUnifiedIdeograph546a',
u'\u296c': 'RightwardsHarpoonWithBarbUpAboveLongDash',
u'\u4def': 'HexagramForTheWell',
u'\uee6e': 'UnicodeCharacter0xee6e',
u'\u02f1': 'ModifierLetterLowLeftArrowhead',
u'\ua370': 'YiSyllableChep',
u'\uc7f3': 'HangulSyllableJyaelh',
u'\u84f5': 'CjkUnifiedIdeograph84f5',
u'\u7c7a': 'CjkUnifiedIdeograph7c7a',
u'\u317c': 'HangulLetterSiostikeut',
u'\u55ff': 'CjkUnifiedIdeograph55ff',
u'\uf67e': 'UnicodeCharacter0xf67e',
u'\ucb80': 'HangulSyllableJjwe',
u'\u6f03': 'CjkUnifiedIdeograph6f03',
u'\u8882': 'CjkUnifiedIdeograph8882',
u'\u2c05': 'GlagoliticCapitalLetterYestu',
u'\ue107': 'UnicodeCharacter0xe107',
u'\ua609': 'VaiSyllableNe',
u'\u598c': 'CjkUnifiedIdeograph598c',
u'\u1e8e': 'LatinCapitalLetterYWithDotAbove',
u'\ud390': 'HangulSyllablePeoss',
u'\u7713': 'CjkUnifiedIdeograph7713',
u'\u9092': 'CjkUnifiedIdeograph9092',
u'\u3415': 'CjkUnifiedIdeograph3415',
u'\u0917': 'DevanagariLetterGa',
u'\uce19': 'HangulSyllableCyung',
u'\u831b': 'CjkUnifiedIdeograph831b',
u'\u619c': 'CjkUnifiedIdeograph619c',
u'\u269e': 'UnicodeCharacter0x269e',
u'\ufba0': 'ArabicLetterRnoonIsolatedForm',
u'\ub8a2': 'HangulSyllableRwaelp',
u'\u5c25': 'CjkUnifiedIdeograph5c25',
u'\u1127': 'HangulChoseongPieupcieuc',
u'\ud629': 'HangulSyllableHyelt',
u'\uab2b': 'UnicodeCharacter0xab2b',
u'\u4eae': 'CjkUnifiedIdeograph4eae',
u'\u03b0': 'GreekSmallLetterUpsilonWithDialytikaAndTonos',
u'\uc0b2': 'HangulSyllableSanh',
u'\u6435': 'CjkUnifiedIdeograph6435',
u'\u85b4': 'CjkUnifiedIdeograph85b4',
u'\u3937': 'CjkUnifiedIdeograph3937',
u'\ufe39': 'PresentationFormForVerticalLeftTortoiseShellBracket',
u'\ub33b': 'HangulSyllableDyaegs',
u'\u56be': 'CjkUnifiedIdeograph56be',
u'\u2bc0': 'UnicodeCharacter0x2bc0',
u'\ue8c2': 'UnicodeCharacter0xe8c2',
u'\uadc4': 'HangulSyllableGwin',
u'\u4147': 'CjkUnifiedIdeograph4147',
u'\u0649': 'ArabicLetterAlefMaksura',
u'\udb4b': 'UnicodeCharacter0xdb4b',
u'\u984d': 'CjkUnifiedIdeograph984d',
u'\u7ece': 'CjkUnifiedIdeograph7ece',
u'\u33d0': 'SquareLm',
u'\uf0d2': 'UnicodeCharacter0xf0d2',
u'\ub5d4': 'HangulSyllableDdek',
u'\u6957': 'CjkUnifiedIdeograph6957',
u'\u8ad6': 'CjkUnifiedIdeograph8ad6',
u'\u2e59': 'UnicodeCharacter0x2e59',
u'\ue35b': 'UnicodeCharacter0xe35b',
u'\ua05d': 'YiSyllableBbiep',
u'\u5be0': 'CjkUnifiedIdeograph5be0',
u'\u18e2': 'UnicodeCharacter0x18e2',
u'\udde4': 'UnicodeCharacter0xdde4',
u'\u7167': 'CjkUnifiedIdeograph7167',
u'\u92e6': 'CjkUnifiedIdeograph92e6',
u'\u3669': 'CjkUnifiedIdeograph3669',
u'\u0b6b': 'OriyaDigitFive',
u'\uc86d': 'HangulSyllableJyet',
u'\u8d6f': 'CjkUnifiedIdeograph8d6f',
u'\u63f0': 'CjkUnifiedIdeograph63f0',
u'\u20f2': 'UnicodeCharacter0x20f2',
u'\ue5f4': 'UnicodeCharacter0xe5f4',
u'\ubaf6': 'HangulSyllableMwaej',
u'\u5e79': 'CjkUnifiedIdeograph5e79',
u'\u137b': 'EthiopicNumberHundred',
u'\ud07d': 'HangulSyllableKeub',
u'\u957f': 'CjkUnifiedIdeograph957f',
u'\ued81': 'UnicodeCharacter0xed81',
u'\u0a04': 'UnicodeCharacter0xa04',
u'\u2687': 'WhiteCircleWithTwoDots',
u'\u8008': 'CjkUnifiedIdeograph8008',
u'\ubc8b': 'HangulSyllableBeod',
u'\u430a': 'CjkUnifiedIdeograph430a',
u'\u7f8d': 'CjkUnifiedIdeograph7f8d',
u'\ud90e': 'UnicodeCharacter0xd90e',
u'\uf591': 'UnicodeCharacter0xf591',
u'\u1214': 'EthiopicSyllableHhee',
u'\u4e97': 'CjkUnifiedIdeograph4e97',
u'\ua818': 'SylotiNagriLetterNo',
u'\uc49b': 'HangulSyllableSsweoh',
u'\u6b1a': 'CjkUnifiedIdeograph6b1a',
u'\ue11e': 'UnicodeCharacter0xe11e',
u'\u1da1': 'ModifierLetterSmallDotlessJWithStroke',
u'\u9ba5': 'CjkUnifiedIdeograph9ba5',
u'\u3a24': 'CjkUnifiedIdeograph3a24',
u'\u56a7': 'CjkUnifiedIdeograph56a7',
u'\ub028': 'HangulSyllableGgyu',
u'\uecab': 'UnicodeCharacter0xecab',
u'\u732a': 'CjkUnifiedIdeograph732a',
u'\u092e': 'DevanagariLetterMa',
u'\u25b1': 'WhiteParallelogram',
u'\u8732': 'CjkUnifiedIdeograph8732',
u'\ua3b5': 'YiSyllableShop',
u'\u4234': 'CjkUnifiedIdeograph4234',
u'\u7eb7': 'CjkUnifiedIdeograph7eb7',
u'\ud838': 'UnicodeCharacter0xd838',
u'\uf4bb': 'UnicodeCharacter0xf4bb',
u'\u113e': 'HangulChoseongCeongchieumsios',
u'\u4dc1': 'HexagramForTheReceptiveEarth',
u'\uaf42': 'HangulSyllableGgoj',
u'\ucbc5': 'HangulSyllableJjyult',
u'\u6a44': 'CjkUnifiedIdeograph6a44',
u'\ue048': 'UnicodeCharacter0xe048',
u'\u1ccb': 'UnicodeCharacter0x1ccb',
u'\u9acf': 'CjkUnifiedIdeograph9acf',
u'\u394e': 'CjkUnifiedIdeograph394e',
u'\u55d1': 'CjkUnifiedIdeograph55d1',
u'\ub752': 'HangulSyllableDdyilp',
u'\ud3d5': 'HangulSyllablePyenj',
u'\u7254': 'CjkUnifiedIdeograph7254',
u'\u0858': 'UnicodeCharacter0x858',
u'\u24db': 'CircledLatinSmallLetterL',
u'\u865c': 'CjkUnifiedIdeograph865c',
u'\ua2df': 'YiSyllableZzex',
u'\u415e': 'CjkUnifiedIdeograph415e',
u'\u7de1': 'CjkUnifiedIdeograph7de1',
u'\udf62': 'UnicodeCharacter0xdf62',
u'\ufbe5': 'ArabicLetterEFinalForm',
u'\u1068': 'MyanmarVowelSignWesternPwoKarenUe',
u'\u4ceb': 'CjkUnifiedIdeograph4ceb',
u'\uae6c': 'HangulSyllableGgaen',
u'\ucaef': 'HangulSyllableJjwac',
u'\u696e': 'CjkUnifiedIdeograph696e',
u'\ue772': 'UnicodeCharacter0xe772',
u'\u03f5': 'GreekLunateEpsilonSymbol',
u'\u99f9': 'CjkUnifiedIdeograph99f9',
u'\u3878': 'CjkUnifiedIdeograph3878',
u'\u54fb': 'CjkUnifiedIdeograph54fb',
u'\ub67c': 'HangulSyllableDdoek',
u'\ud2ff': 'HangulSyllableTilh',
u'\u717e': 'CjkUnifiedIdeograph717e',
u'\ue803': 'UnicodeCharacter0xe803',
u'\u0f82': 'TibetanSignNyiZlaNaaDa',
u'\u8586': 'CjkUnifiedIdeograph8586',
u'\u2109': 'DegreeFahrenheit',
u'\u4088': 'CjkUnifiedIdeograph4088',
u'\ubf0d': 'HangulSyllableBbyeonj',
u'\ude8c': 'UnicodeCharacter0xde8c',
u'\u7a0f': 'CjkUnifiedIdeograph7a0f',
u'\uf013': 'UnicodeCharacter0xf013',
u'\u1792': 'KhmerLetterTho',
u'\u8e17': 'CjkUnifiedIdeograph8e17',
u'\uad96': 'HangulSyllableGweolp',
u'\u4919': 'CjkUnifiedIdeograph4919',
u'\u6898': 'CjkUnifiedIdeograph6898',
u'\uc71d': 'HangulSyllableWit',
u'\ue69c': 'UnicodeCharacter0xe69c',
u'\u1823': 'MongolianLetterO',
u'\u3fa2': 'CjkUnifiedIdeograph3fa2',
u'\u9627': 'CjkUnifiedIdeograph9627',
u'\ub5a6': 'HangulSyllableDdeonh',
u'\u5129': 'CjkUnifiedIdeograph5129',
u'\u70a8': 'CjkUnifiedIdeograph70a8',
u'\uef2d': 'UnicodeCharacter0xef2d',
u'\u0eac': 'UnicodeCharacter0xeac',
u'\u84b0': 'CjkUnifiedIdeograph84b0',
u'\u2033': 'DoublePrime',
u'\u47b2': 'CjkUnifiedIdeograph47b2',
u'\ube37': 'HangulSyllableByilh',
u'\uddb6': 'UnicodeCharacter0xddb6',
u'\u7939': 'CjkUnifiedIdeograph7939',
u'\uf73d': 'UnicodeCharacter0xf73d',
u'\u16bc': 'RunicLetterLongbranchhagallH',
u'\u8d41': 'CjkUnifiedIdeograph8d41',
u'\uacc0': 'HangulSyllableGyeok',
u'\u4843': 'CjkUnifiedIdeograph4843',
u'\u6fc2': 'CjkUnifiedIdeograph6fc2',
u'\uc647': 'HangulSyllableWad',
u'\ue5c6': 'UnicodeCharacter0xe5c6',
u'\u1f4d': 'GreekCapitalLetterOmicronWithDasiaAndOxia',
u'\u3ecc': 'CjkUnifiedIdeograph3ecc',
u'\u9551': 'CjkUnifiedIdeograph9551',
u'\ub4d0': 'HangulSyllableDyum',
u'\u5053': 'CjkUnifiedIdeograph5053',
u'\u77d2': 'CjkUnifiedIdeograph77d2',
u'\uee57': 'UnicodeCharacter0xee57',
u'\u0dd6': 'SinhalaVowelSignDigaPaapilla',
u'\u8bda': 'CjkUnifiedIdeograph8bda',
u'\u275d': 'HeavyDoubleTurnedCommaQuotationMarkOrnament',
u'\u46dc': 'CjkUnifiedIdeograph46dc',
u'\ubd61': 'HangulSyllableBoet',
u'\udce0': 'UnicodeCharacter0xdce0',
u'\u7863': 'CjkUnifiedIdeograph7863',
u'\uf667': 'UnicodeCharacter0xf667',
u'\u15e6': 'CanadianSyllabicsCarrierTti',
u'\u8c6b': 'CjkUnifiedIdeograph8c6b',
u'\u93ea': 'CjkUnifiedIdeograph93ea',
u'\u4f6d': 'CjkUnifiedIdeograph4f6d',
u'\u6eec': 'CjkUnifiedIdeograph6eec',
u'\uc571': 'HangulSyllableAeb',
u'\ue4f0': 'UnicodeCharacter0xe4f0',
u'\u1e77': 'LatinSmallLetterUWithCircumflexBelow',
u'\u3df6': 'CjkUnifiedIdeograph3df6',
u'\u947b': 'CjkUnifiedIdeograph947b',
u'\ubbfa': 'HangulSyllableMigg',
u'\u577d': 'CjkUnifiedIdeograph577d',
u'\u76fc': 'CjkUnifiedIdeograph76fc',
u'\u6c81': 'CjkUnifiedIdeograph6c81',
u'\u0d00': 'UnicodeCharacter0xd00',
u'\u2983': 'LeftWhiteCurlyBracket',
u'\uce02': 'HangulSyllableCwip',
u'\uea85': 'UnicodeCharacter0xea85',
u'\u8b04': 'CjkUnifiedIdeograph8b04',
u'\ua787': 'LatinSmallLetterInsularT',
u'\u580e': 'CjkUnifiedIdeograph580e',
u'\u7491': 'CjkUnifiedIdeograph7491',
u'\u1510': 'CanadianSyllabicsShe',
u'\u3193': 'IdeographicAnnotationTwoMark',
u'\ud612': 'HangulSyllableHyeobs',
u'\uf295': 'UnicodeCharacter0xf295',
u'\u9314': 'CjkUnifiedIdeograph9314',
u'\ucf97': 'HangulSyllableKwaelb',
u'\u8899': 'CjkUnifiedIdeograph8899',
u'\u601e': 'CjkUnifiedIdeograph601e',
u'\u3d20': 'CjkUnifiedIdeograph3d20',
u'\u59a3': 'CjkUnifiedIdeograph59a3',
u'\ufe22': 'CombiningDoubleTildeLeftHalf',
u'\u1aa5': 'UnicodeCharacter0x1aa5',
u'\ubb24': 'HangulSyllableMyols',
u'\ud7a7': 'UnicodeCharacter0xd7a7',
u'\u90a9': 'CjkUnifiedIdeograph90a9',
u'\u4530': 'CjkUnifiedIdeograph4530',
u'\u61b3': 'CjkUnifiedIdeograph61b3',
u'\u0632': 'ArabicLetterZain',
u'\u22b5': 'ContainsAsNormalSubgroupOrEqualTo',
u'\uc334': 'HangulSyllableSsyan',
u'\uffb7': 'HalfwidthHangulLetterIeung',
u'\u9c36': 'CjkUnifiedIdeograph9c36',
u'\ub8b9': 'HangulSyllableRoelg',
u'\u6d40': 'CjkUnifiedIdeograph6d40',
u'\u2e42': 'UnicodeCharacter0x2e42',
u'\u4ac5': 'CjkUnifiedIdeograph4ac5',
u'\ueb44': 'UnicodeCharacter0xeb44',
u'\u07c7': 'NkoDigitSeven',
u'\ua446': 'YiSyllableNjy',
u'\uc0c9': 'HangulSyllableSaeg',
u'\u9dcb': 'CjkUnifiedIdeograph9dcb',
u'\u7550': 'CjkUnifiedIdeograph7550',
u'\u3652': 'CjkUnifiedIdeograph3652',
u'\u52d5': 'CjkUnifiedIdeograph52d5',
u'\uf354': 'UnicodeCharacter0xf354',
u'\u2fd7': 'UnicodeCharacter0x2fd7',
u'\ucc56': 'HangulSyllableCaebs',
u'\ue8d9': 'UnicodeCharacter0xe8d9',
u'\u8958': 'CjkUnifiedIdeograph8958',
u'\ua5db': 'VaiSyllableKo',
u'\u5e62': 'CjkUnifiedIdeograph5e62',
u'\u7ae5': 'CjkUnifiedIdeograph7ae5',
u'\u1b64': 'BalineseMusicalSymbolDang',
u'\u37e7': 'CjkUnifiedIdeograph37e7',
u'\ud466': 'HangulSyllablePyolm',
u'\uf0e9': 'UnicodeCharacter0xf0e9',
u'\u9168': 'CjkUnifiedIdeograph9168',
u'\ucdeb': 'HangulSyllableCwigs',
u'\u8eed': 'CjkUnifiedIdeograph8eed',
u'\u6672': 'CjkUnifiedIdeograph6672',
u'\u2374': 'AplFunctionalSymbolRho',
u'\u5ff7': 'CjkUnifiedIdeograph5ff7',
u'\ufc76': 'ArabicLigatureThehWithRehFinalForm',
u'\u18f9': 'UnicodeCharacter0x18f9',
u'\ub978': 'HangulSyllableReun',
u'\ud5fb': 'HangulSyllableHec',
u'\u96fd': 'CjkUnifiedIdeograph96fd',
u'\uac01': 'HangulSyllableGag',
u'\u4b84': 'CjkUnifiedIdeograph4b84',
u'\u0486': 'CombiningCyrillicPsiliPneumata',
u'\uc188': 'HangulSyllableSyek',
u'\u7d0b': 'CjkUnifiedIdeograph7d0b',
u'\u828a': 'CjkUnifiedIdeograph828a',
u'\u3e0d': 'CjkUnifiedIdeograph3e0d',
u'\ufb0f': 'UnicodeCharacter0xfb0f',
u'\ub411': 'HangulSyllableDwaeng',
u'\u5394': 'CjkUnifiedIdeograph5394',
u'\u2c96': 'CopticCapitalLetterLaula',
u'\ue998': 'UnicodeCharacter0xe998',
u'\uaa9a': 'UnicodeCharacter0xaa9a',
u'\u461d': 'CjkUnifiedIdeograph461d',
u'\u031f': 'CombiningPlusSignBelow',
u'\udc21': 'UnicodeCharacter0xdc21',
u'\u9923': 'CjkUnifiedIdeograph9923',
u'\u7ba4': 'CjkUnifiedIdeograph7ba4',
u'\u34a6': 'CjkUnifiedIdeograph34a6',
u'\uf1a8': 'UnicodeCharacter0xf1a8',
u'\ub2aa': 'HangulSyllableNeup',
u'\u6e2d': 'CjkUnifiedIdeograph6e2d',
u'\u8fac': 'CjkUnifiedIdeograph8fac',
u'\u2b2f': 'WhiteVerticalEllipse',
u'\ue431': 'UnicodeCharacter0xe431',
u'\ua133': 'YiSyllableTup',
u'\u5cb6': 'CjkUnifiedIdeograph5cb6',
u'\u19b8': 'NewTaiLueVowelSignOa',
u'\udaba': 'UnicodeCharacter0xdaba',
u'\u763d': 'CjkUnifiedIdeograph763d',
u'\u97bc': 'CjkUnifiedIdeograph97bc',
u'\u333f': 'SquareHon',
u'\u0c41': 'TeluguVowelSignU',
u'\uc943': 'HangulSyllableJwelh',
u'\u8a45': 'CjkUnifiedIdeograph8a45',
u'\u64c6': 'CjkUnifiedIdeograph64c6',
u'\u21c8': 'UpwardsPairedArrows',
u'\ue2ca': 'UnicodeCharacter0xe2ca',
u'\ubfcc': 'HangulSyllableBbu',
u'\u5b4f': 'CjkUnifiedIdeograph5b4f',
u'\u1451': 'CanadianSyllabicsToo',
u'\ud153': 'HangulSyllableTed',
u'\u9255': 'CjkUnifiedIdeograph9255',
u'\u49d8': 'CjkUnifiedIdeograph49d8',
u'\u0ada': 'UnicodeCharacter0xada',
u'\uc7dc': 'HangulSyllableJyass',
u'\u635f': 'CjkUnifiedIdeograph635f',
u'\u80de': 'CjkUnifiedIdeograph80de',
u'\u3c61': 'CjkUnifiedIdeograph3c61',
u'\uf963': 'CjkCompatibilityIdeographf963',
u'\uba65': 'HangulSyllableMeb',
u'\u51e8': 'CjkUnifiedIdeograph51e8',
u'\u12ea': 'EthiopicSyllableYi',
u'\uefec': 'UnicodeCharacter0xefec',
u'\ua8ee': 'UnicodeCharacter0xa8ee',
u'\u4471': 'CjkUnifiedIdeograph4471',
u'\u0173': 'LatinSmallLetterUWithOgonek',
u'\uc275': 'HangulSyllableSwilg',
u'\u9f77': 'CjkUnifiedIdeograph9f77',
u'\u79f8': 'CjkUnifiedIdeograph79f8',
u'\u3afa': 'CjkUnifiedIdeograph3afa',
u'\uf7fc': 'UnicodeCharacter0xf7fc',
u'\ub0fe': 'HangulSyllableNyaebs',
u'\u8e00': 'CjkUnifiedIdeograph8e00',
u'\uaa83': 'UnicodeCharacter0xaa83',
u'\u4d02': 'CjkUnifiedIdeograph4d02',
u'\u6985': 'CjkUnifiedIdeograph6985',
u'\uc706': 'HangulSyllableWigg',
u'\ue389': 'UnicodeCharacter0xe389',
u'\u1c0c': 'LepchaLetterDa',
u'\u388f': 'CjkUnifiedIdeograph388f',
u'\u9610': 'CjkUnifiedIdeograph9610',
u'\ub293': 'HangulSyllableNeugs',
u'\u5512': 'CjkUnifiedIdeograph5512',
u'\u7195': 'CjkUnifiedIdeograph7195',
u'\uef16': 'UnicodeCharacter0xef16',
u'\u0b99': 'TamilLetterNga',
u'\u859d': 'CjkUnifiedIdeograph859d',
u'\u241c': 'SymbolForFileSeparator',
u'\u409f': 'CjkUnifiedIdeograph409f',
u'\ube20': 'HangulSyllableBeuss',
u'\udaa3': 'UnicodeCharacter0xdaa3',
u'\u7d22': 'CjkUnifiedIdeograph7d22',
u'\uf726': 'UnicodeCharacter0xf726',
u'\u13a9': 'CherokeeLetterGi',
u'\uadad': 'HangulSyllableGwelg',
u'\u4c2c': 'CjkUnifiedIdeograph4c2c',
u'\u68af': 'CjkUnifiedIdeograph68af',
u'\uc630': 'HangulSyllableOls',
u'\ue2b3': 'UnicodeCharacter0xe2b3',
u'\u1f36': 'GreekSmallLetterIotaWithPsiliAndPerispomeni',
u'\u3bb9': 'CjkUnifiedIdeograph3bb9',
u'\u993a': 'CjkUnifiedIdeograph993a',
u'\ub5bd': 'HangulSyllableDdeg',
u'\u543c': 'CjkUnifiedIdeograph543c',
u'\u70bf': 'CjkUnifiedIdeograph70bf',
u'\uee40': 'UnicodeCharacter0xee40',
u'\u0ac3': 'GujaratiVowelSignVocalicR',
u'\u84c7': 'CjkUnifiedIdeograph84c7',
u'\u2746': 'HeavyChevronSnowflake',
u'\u43c9': 'CjkUnifiedIdeograph43c9',
u'\ua14a': 'YiSyllableDdep',
u'\uddcd': 'UnicodeCharacter0xddcd',
u'\u7c4c': 'CjkUnifiedIdeograph7c4c',
u'\uf650': 'UnicodeCharacter0xf650',
u'\u12d3': 'EthiopicSyllablePharyngealAa',
u'\uacd7': 'HangulSyllableGyes',
u'\u4f56': 'CjkUnifiedIdeograph4f56',
u'\u6bd9': 'CjkUnifiedIdeograph6bd9',
u'\uc95a': 'HangulSyllableJwilm',
u'\ue5dd': 'UnicodeCharacter0xe5dd',
u'\u1e60': 'LatinCapitalLetterSWithDotAbove',
u'\u3ae3': 'CjkUnifiedIdeograph3ae3',
u'\u9864': 'CjkUnifiedIdeograph9864',
u'\ub4e7': 'HangulSyllableDeulb',
u'\u5766': 'CjkUnifiedIdeograph5766',
u'\u73e9': 'CjkUnifiedIdeograph73e9',
u'\ud16a': 'HangulSyllableTyeogg',
u'\u0ded': 'UnicodeCharacter0xded',
u'\u87f1': 'CjkUnifiedIdeograph87f1',
u'\u2670': 'WestSyriacCross',
u'\u42f3': 'CjkUnifiedIdeograph42f3',
u'\ua074': 'YiSyllableBby',
u'\udcf7': 'UnicodeCharacter0xdcf7',
u'\u7f76': 'CjkUnifiedIdeograph7f76',
u'\uf97a': 'CjkCompatibilityIdeographf97a',
u'\u15fd': 'CanadianSyllabicsCarrierKke',
u'\u2f01': 'KangxiRadicalLine',
u'\u4e80': 'CjkUnifiedIdeograph4e80',
u'\ua905': 'KayahLiDigitFive',
u'\uc884': 'HangulSyllableJoss',
u'\u6407': 'CjkUnifiedIdeograph6407',
u'\ufe0b': 'VariationSelector12',
u'\u018a': 'LatinCapitalLetterDWithHook',
u'\u9b8e': 'CjkUnifiedIdeograph9b8e',
u'\u3711': 'CjkUnifiedIdeograph3711',
u'\u5690': 'CjkUnifiedIdeograph5690',
u'\ub115': 'HangulSyllableNeolt',
u'\ud094': 'HangulSyllableKyils',
u'\u061b': 'ArabicSemicolon',
u'\u299a': 'VerticalZigzagLine',
u'\u801f': 'CjkUnifiedIdeograph801f',
u'\ua39e': 'YiSyllableNrep',
u'\u5f21': 'CjkUnifiedIdeograph5f21',
u'\u7ea0': 'CjkUnifiedIdeograph7ea0',
u'\ud925': 'UnicodeCharacter0xd925',
u'\uf8a4': 'UnicodeCharacter0xf8a4',
u'\u2e2b': 'OneDotOverTwoDotsPunctuation',
u'\u31aa': 'BopomofoLetterInn',
u'\ua82f': 'UnicodeCharacter0xa82f',
u'\ucbae': 'HangulSyllableJjwibs',
u'\u6731': 'CjkUnifiedIdeograph6731',
u'\ue135': 'UnicodeCharacter0xe135',
u'\xb4': 'AcuteAccent',
u'\u9ab8': 'CjkUnifiedIdeograph9ab8',
u'\u363b': 'CjkUnifiedIdeograph363b',
u'\u59ba': 'CjkUnifiedIdeograph59ba',
u'\ub03f': 'HangulSyllableGgyuc',
u'\ud3be': 'HangulSyllablePyeolm',
u'\u0945': 'DevanagariVowelSignCandraE',
u'\u28c4': 'BraillePatternDots378',
u'\u8349': 'CjkUnifiedIdeograph8349',
u'\ua2c8': 'YiSyllableCurx',
u'\u5e4b': 'CjkUnifiedIdeograph5e4b',
u'\u61ca': 'CjkUnifiedIdeograph61ca',
u'\ud84f': 'UnicodeCharacter0xd84f',
u'\ufbce': 'UnicodeCharacter0xfbce',
u'\u1155': 'HangulChoseongCeongchieumchieuch',
u'\u30d4': 'KatakanaLetterPi',
u'\uab59': 'UnicodeCharacter0xab59',
u'\ucad8': 'HangulSyllableJjwa',
u'\u665b': 'CjkUnifiedIdeograph665b',
u'\ue05f': 'UnicodeCharacter0xe05f',
u'\u03de': 'GreekLetterKoppa',
u'\u9de2': 'CjkUnifiedIdeograph9de2',
u'\u3965': 'CjkUnifiedIdeograph3965',
u'\u58e4': 'CjkUnifiedIdeograph58e4',
u'\ub369': 'HangulSyllableDeong',
u'\ud2e8': 'HangulSyllableTyiss',
u'\u086f': 'UnicodeCharacter0x86f',
u'\u2bee': 'UnicodeCharacter0x2bee',
u'\u8273': 'CjkUnifiedIdeograph8273',
u'\ua5f2': 'VaiSyllableVe',
u'\u4175': 'CjkUnifiedIdeograph4175',
u'\u60f4': 'CjkUnifiedIdeograph60f4',
u'\udb79': 'UnicodeCharacter0xdb79',
u'\ufaf8': 'UnicodeCharacter0xfaf8',
u'\u107f': 'MyanmarLetterShanBa',
u'\u33fe': 'IdeographicTelegraphSymbolForDayThirtyone',
u'\u4606': 'CjkUnifiedIdeograph4606',
u'\u6289': 'CjkUnifiedIdeograph6289',
u'\u0308': 'CombiningDiaeresis',
u'\u3f8b': 'CjkUnifiedIdeograph3f8b',
u'\uc00a': 'HangulSyllableBbwenh',
u'\ufc8d': 'ArabicLigatureNoonWithNoonFinalForm',
u'\u9d0c': 'CjkUnifiedIdeograph9d0c',
u'\ub98f': 'HangulSyllableReuh',
u'\u6e16': 'CjkUnifiedIdeograph6e16',
u'\u2b18': 'DiamondWithTopHalfBlack',
u'\u479b': 'CjkUnifiedIdeograph479b',
u'\ue81a': 'UnicodeCharacter0xe81a',
u'\u049d': 'CyrillicSmallLetterKaWithVerticalStroke',
u'\ua51c': 'VaiSyllableNjee',
u'\uc19f': 'HangulSyllableSos',
u'\u9ea1': 'CjkUnifiedIdeograph9ea1',
u'\u7626': 'CjkUnifiedIdeograph7626',
u'\u3328': 'SquareNano',
u'\u6fab': 'CjkUnifiedIdeograph6fab',
u'\uf02a': 'UnicodeCharacter0xf02a',
u'\u2cad': 'CopticSmallLetterKhi',
u'\ucd2c': 'HangulSyllableCwal',
u'\ue9af': 'UnicodeCharacter0xe9af',
u'\u8a2e': 'CjkUnifiedIdeograph8a2e',
u'\ua6b1': 'UnicodeCharacter0xa6b1',
u'\u5b38': 'CjkUnifiedIdeograph5b38',
u'\u77bb': 'CjkUnifiedIdeograph77bb',
u'\u183a': 'MongolianLetterKa',
u'\u34bd': 'CjkUnifiedIdeograph34bd',
u'\ud53c': 'HangulSyllablePi',
u'\uf1bf': 'UnicodeCharacter0xf1bf',
u'\u923e': 'CjkUnifiedIdeograph923e',
u'\ucec1': 'HangulSyllableKyang',
u'\u8bc3': 'CjkUnifiedIdeograph8bc3',
u'\u6348': 'CjkUnifiedIdeograph6348',
u'\u204a': 'TironianSignEt',
u'\u5ccd': 'CjkUnifiedIdeograph5ccd',
u'\ufd4c': 'UnicodeCharacter0xfd4c',
u'\u19cf': 'UnicodeCharacter0x19cf',
u'\uba4e': 'HangulSyllableMeoj',
u'\ud6d1': 'HangulSyllableHult',
u'\u93d3': 'CjkUnifiedIdeograph93d3',
u'\u485a': 'CjkUnifiedIdeograph485a',
u'\u64dd': 'CjkUnifiedIdeograph64dd',
u'\u055c': 'ArmenianExclamationMark',
u'\u21df': 'DownwardsArrowWithDoubleStroke',
u'\uc25e': 'HangulSyllableSwelp',
u'\ufee1': 'ArabicLetterMeemIsolatedForm',
u'\u9f60': 'CjkUnifiedIdeograph9f60',
u'\ubbe3': 'HangulSyllableMyid',
u'\u506a': 'CjkUnifiedIdeograph506a',
u'\u2d6c': 'UnicodeCharacter0x2d6c',
u'\u49ef': 'CjkUnifiedIdeograph49ef',
u'\uea6e': 'UnicodeCharacter0xea6e',
u'\u06f1': 'ExtendedArabicindicDigitOne',
u'\ua770': 'ModifierLetterUs',
u'\uc3f3': 'HangulSyllableSsoh',
u'\u80f5': 'CjkUnifiedIdeograph80f5',
u'\u787a': 'CjkUnifiedIdeograph787a',
u'\u357c': 'CjkUnifiedIdeograph357c',
u'\u51ff': 'CjkUnifiedIdeograph51ff',
u'\uf27e': 'UnicodeCharacter0xf27e',
u'\ucf80': 'HangulSyllableKwam',
u'\u6b03': 'CjkUnifiedIdeograph6b03',
u'\u8c82': 'CjkUnifiedIdeograph8c82',
u'\u2805': 'BraillePatternDots13',
u'\ue507': 'UnicodeCharacter0xe507',
u'\ua209': 'YiSyllableKop',
u'\u5d8c': 'CjkUnifiedIdeograph5d8c',
u'\u1a8e': 'UnicodeCharacter0x1a8e',
u'\ud790': 'HangulSyllableHil',
u'\u7313': 'CjkUnifiedIdeograph7313',
u'\u9492': 'CjkUnifiedIdeograph9492',
u'\u3015': 'RightTortoiseShellBracket',
u'\u0d17': 'MalayalamLetterGa',
u'\uca19': 'HangulSyllableJjyanj',
u'\u871b': 'CjkUnifiedIdeograph871b',
u'\u659c': 'CjkUnifiedIdeograph659c',
u'\u229e': 'SquaredPlus',
u'\uffa0': 'HalfwidthHangulFiller',
u'\ubca2': 'HangulSyllableBegg',
u'\u5825': 'CjkUnifiedIdeograph5825',
u'\u1527': 'CanadianSyllabicsYaai',
u'\ud229': 'HangulSyllableTyot',
u'\uaf2b': 'HangulSyllableGgyeh',
u'\u4aae': 'CjkUnifiedIdeograph4aae',
u'\u07b0': 'ThaanaSukun',
u'\uc4b2': 'HangulSyllableSswej',
u'\u6035': 'CjkUnifiedIdeograph6035',
u'\u81b4': 'CjkUnifiedIdeograph81b4',
u'\u3d37': 'CjkUnifiedIdeograph3d37',
u'\ufa39': 'CjkCompatibilityIdeographfa39',
u'\ub73b': 'HangulSyllableDdeus',
u'\u52be': 'CjkUnifiedIdeograph52be',
u'\u2fc0': 'KangxiRadicalCauldron',
u'\uecc2': 'UnicodeCharacter0xecc2',
u'\ua9c4': 'UnicodeCharacter0xa9c4',
u'\u4547': 'CjkUnifiedIdeograph4547',
u'\u0249': 'LatinSmallLetterJWithStroke',
u'\udf4b': 'UnicodeCharacter0xdf4b',
u'\u9c4d': 'CjkUnifiedIdeograph9c4d',
u'\u7ace': 'CjkUnifiedIdeograph7ace',
u'\u37d0': 'CjkUnifiedIdeograph37d0',
u'\uf4d2': 'UnicodeCharacter0xf4d2',
u'\ub1d4': 'HangulSyllableNoel',
u'\u6d57': 'CjkUnifiedIdeograph6d57',
u'\u8ed6': 'CjkUnifiedIdeograph8ed6',
u'\u2a59': 'LogicalOrOverlappingLogicalAnd',
u'\ue75b': 'UnicodeCharacter0xe75b',
u'\ua45d': 'YiSyllableXit',
u'\u5fe0': 'CjkUnifiedIdeograph5fe0',
u'\u1ce2': 'UnicodeCharacter0x1ce2',
u'\ud9e4': 'UnicodeCharacter0xd9e4',
u'\u7567': 'CjkUnifiedIdeograph7567',
u'\u96e6': 'CjkUnifiedIdeograph96e6',
u'\u3269': 'CircledHangulChieuch',
u'\u0f6b': 'TibetanLetterKka',
u'\ucc6d': 'HangulSyllableCyalt',
u'\u896f': 'CjkUnifiedIdeograph896f',
u'\u67f0': 'CjkUnifiedIdeograph67f0',
u'\u24f2': 'NegativeCircledNumberEighteen',
u'\ue1f4': 'UnicodeCharacter0xe1f4',
u'\ubef6': 'HangulSyllableBbelm',
u'\u5a79': 'CjkUnifiedIdeograph5a79',
u'\u177b': 'UnicodeCharacter0x177b',
u'\ud47d': 'HangulSyllablePunj',
u'\u917f': 'CjkUnifiedIdeograph917f',
u'\uf181': 'UnicodeCharacter0xf181',
u'\u1604': 'CanadianSyllabicsCarrierNe',
u'\u3a87': 'CjkUnifiedIdeograph3a87',
u'\u9c08': 'CjkUnifiedIdeograph9c08',
u'\ua08b': 'YiSyllableNbyt',
u'\u5f0a': 'CjkUnifiedIdeograph5f0a',
u'\u638d': 'CjkUnifiedIdeograph638d',
u'\uc50e': 'HangulSyllableSsyigg',
u'\ue991': 'UnicodeCharacter0xe991',
u'\u0e14': 'ThaiCharacterDoDek',
u'\u5297': 'CjkUnifiedIdeograph5297',
u'\ub418': 'HangulSyllableDoe',
u'\ud89b': 'UnicodeCharacter0xd89b',
u'\u771a': 'CjkUnifiedIdeograph771a',
u'\ufd1e': 'ArabicLigatureJeemWithYehFinalForm',
u'\u01a1': 'LatinSmallLetterOWithHorn',
u'\u87a5': 'CjkUnifiedIdeograph87a5',
u'\u2624': 'Caduceus',
u'\u4aa7': 'CjkUnifiedIdeograph4aa7',
u'\uac28': 'HangulSyllableGaels',
u'\uf0ab': 'UnicodeCharacter0xf0ab',
u'\u6f2a': 'CjkUnifiedIdeograph6f2a',
u'\u152e': 'CanadianSyllabicsYaa',
u'\u39b1': 'CjkUnifiedIdeograph39b1',
u'\u9b32': 'CjkUnifiedIdeograph9b32',
u'\ubfb5': 'HangulSyllableBbyonj',
u'\u5e34': 'CjkUnifiedIdeograph5e34',
u'\u62b7': 'CjkUnifiedIdeograph62b7',
u'\uc438': 'HangulSyllableSsoels',
u'\ue8bb': 'UnicodeCharacter0xe8bb',
u'\u0d3e': 'MalayalamVowelSignAa',
u'\u51c1': 'CjkUnifiedIdeograph51c1',
u'\ub342': 'HangulSyllableDyaelm',
u'\ud7c5': 'UnicodeCharacter0xd7c5',
u'\u7644': 'CjkUnifiedIdeograph7644',
u'\ufc48': 'ArabicLigatureMeemWithMeemIsolatedForm',
u'\xcb': 'LatinCapitalLetterEWithDiaeresis',
u'\u86cf': 'CjkUnifiedIdeograph86cf',
u'\u254e': 'BoxDrawingsLightDoubleDashVertical',
u'\u49d1': 'CjkUnifiedIdeograph49d1',
u'\uab52': 'UnicodeCharacter0xab52',
u'\ucfd5': 'HangulSyllableKyob',
u'\u6e54': 'CjkUnifiedIdeograph6e54',
u'\u1458': 'CanadianSyllabicsWestcreeTwe',
u'\u38db': 'CjkUnifiedIdeograph38db',
u'\u9a5c': 'CjkUnifiedIdeograph9a5c',
u'\ubedf': 'HangulSyllableBbeolh',
u'\u5d5e': 'CjkUnifiedIdeograph5d5e',
u'\u61e1': 'CjkUnifiedIdeograph61e1',
u'\uc362': 'HangulSyllableSsyaej',
u'\ue7e5': 'UnicodeCharacter0xe7e5',
u'\u0c68': 'TeluguDigitTwo',
u'\u50eb': 'CjkUnifiedIdeograph50eb',
u'\ub26c': 'HangulSyllableNwiss',
u'\ud6ef': 'HangulSyllableHweolh',
u'\u756e': 'CjkUnifiedIdeograph756e',
u'\ufb72': 'ArabicLetterDyehIsolatedForm',
u'\u1ff5': 'UnicodeCharacter0x1ff5',
u'\u85f9': 'CjkUnifiedIdeograph85f9',
u'\u2478': 'ParenthesizedDigitFive',
u'\u48fb': 'CjkUnifiedIdeograph48fb',
u'\uaa7c': 'UnicodeCharacter0xaa7c',
u'\uceff': 'HangulSyllableKeoh',
u'\u6d7e': 'CjkUnifiedIdeograph6d7e',
u'\uf403': 'UnicodeCharacter0xf403',
u'\u1382': 'EthiopicSyllableMwee',
u'\u9986': 'CjkUnifiedIdeograph9986',
u'\u3d09': 'CjkUnifiedIdeograph3d09',
u'\u5c88': 'CjkUnifiedIdeograph5c88',
u'\ua30d': 'YiSyllableSiex',
u'\uc28c': 'HangulSyllableSyun',
u'\u660f': 'CjkUnifiedIdeograph660f',
u'\uec13': 'UnicodeCharacter0xec13',
u'\u0b92': 'TamilLetterO',
u'\u9217': 'CjkUnifiedIdeograph9217',
u'\ub196': 'HangulSyllableNwagg',
u'\u5519': 'CjkUnifiedIdeograph5519',
u'\u7498': 'CjkUnifiedIdeograph7498',
u'\udb1d': 'UnicodeCharacter0xdb1d',
u'\ufa9c': 'CjkCompatibilityIdeographfa9c',
u'\u0423': 'CyrillicCapitalLetterU',
u'\u23a2': 'LeftSquareBracketExtension',
u'\u8a27': 'CjkUnifiedIdeograph8a27',
u'\ua9a6': 'UnicodeCharacter0xa9a6',
u'\u4d29': 'CjkUnifiedIdeograph4d29',
u'\u6ca8': 'CjkUnifiedIdeograph6ca8',
u'\uf32d': 'UnicodeCharacter0xf32d',
u'\u12ac': 'EthiopicSyllableKee',
u'\u98b0': 'CjkUnifiedIdeograph98b0',
u'\u3c33': 'CjkUnifiedIdeograph3c33',
u'\u5bb2': 'CjkUnifiedIdeograph5bb2',
u'\ua237': 'YiSyllableMguo',
u'\uc1b6': 'HangulSyllableSwalp',
u'\u6539': 'CjkUnifiedIdeograph6539',
u'\ueb3d': 'UnicodeCharacter0xeb3d',
u'\u0abc': 'GujaratiSignNukta',
u'\u9141': 'CjkUnifiedIdeograph9141',
u'\ub0c0': 'HangulSyllableNaels',
u'\u5443': 'CjkUnifiedIdeograph5443',
u'\u73c2': 'CjkUnifiedIdeograph73c2',
u'\uda47': 'UnicodeCharacter0xda47',
u'\uf9c6': 'CjkCompatibilityIdeographf9c6',
u'\u034d': 'CombiningLeftRightArrowBelow',
u'\u22cc': 'RightSemidirectProduct',
u'\u8951': 'CjkUnifiedIdeograph8951',
u'\ua8d0': 'SaurashtraDigitZero',
u'\u4c53': 'CjkUnifiedIdeograph4c53',
u'\u6bd2': 'CjkUnifiedIdeograph6bd2',
u'\uf257': 'UnicodeCharacter0xf257',
u'\u11d6': 'HangulJongseongRieulssangsios',
u'\u97da': 'CjkUnifiedIdeograph97da',
u'\u3b5d': 'CjkUnifiedIdeograph3b5d',
u'\u5adc': 'CjkUnifiedIdeograph5adc',
u'\ua161': 'YiSyllableNdep',
u'\uc0e0': 'HangulSyllableSaek',
u'\u6463': 'CjkUnifiedIdeograph6463',
u'\uea67': 'UnicodeCharacter0xea67',
u'\u09e6': 'BengaliDigitZero',
u'\u906b': 'CjkUnifiedIdeograph906b',
u'\u8fea': 'CjkUnifiedIdeograph8fea',
u'\u536d': 'CjkUnifiedIdeograph536d',
u'\u72ec': 'CjkUnifiedIdeograph72ec',
u'\ud971': 'UnicodeCharacter0xd971',
u'\uf8f0': 'UnicodeCharacter0xf8f0',
u'\u0277': 'LatinSmallLetterClosedOmega',
u'\u21f6': 'ThreeRightwardsArrows',
u'\u887b': 'CjkUnifiedIdeograph887b',
u'\ua7fa': 'UnicodeCharacter0xa7fa',
u'\u4b7d': 'CjkUnifiedIdeograph4b7d',
u'\u6afc': 'CjkUnifiedIdeograph6afc',
u'\u7081': 'CjkUnifiedIdeograph7081',
u'\u1100': 'HangulChoseongKiyeok',
u'\u3583': 'CjkUnifiedIdeograph3583',
u'\ud202': 'HangulSyllableToelp',
u'\uf685': 'UnicodeCharacter0xf685',
u'\u9704': 'CjkUnifiedIdeograph9704',
u'\ubb87': 'HangulSyllableMweh',
u'\u440e': 'CjkUnifiedIdeograph440e',
u'\u6891': 'CjkUnifiedIdeograph6891',
u'\u0910': 'DevanagariLetterAi',
u'\u2d93': 'EthiopicSyllableGgwa',
u'\uca12': 'HangulSyllableJjaep',
u'\uee95': 'UnicodeCharacter0xee95',
u'\u8f14': 'CjkUnifiedIdeograph8f14',
u'\ud397': 'HangulSyllablePeoh',
u'\u9499': 'CjkUnifiedIdeograph9499',
u'\u7c1e': 'CjkUnifiedIdeograph7c1e',
u'\u2120': 'ServiceMark',
u'\u45a3': 'CjkUnifiedIdeograph45a3',
u'\ue222': 'UnicodeCharacter0xe222',
u'\u06a5': 'ArabicLetterFehWithThreeDotsBelow',
u'\ua724': 'LatinCapitalLetterEgyptologicalAin',
u'\ucba7': 'HangulSyllableJjwilb',
u'\u8ca9': 'CjkUnifiedIdeograph8ca9',
u'\u5930': 'CjkUnifiedIdeograph5930',
u'\u7db3': 'CjkUnifiedIdeograph7db3',
u'\u1a32': 'UnicodeCharacter0x1a32',
u'\u3eb5': 'CjkUnifiedIdeograph3eb5',
u'\udf34': 'UnicodeCharacter0xdf34',
u'\ue3b7': 'UnicodeCharacter0xe3b7',
u'\u8036': 'CjkUnifiedIdeograph8036',
u'\ua4b9': 'YiRadicalDu',
u'\u7140': 'CjkUnifiedIdeograph7140',
u'\u3242': 'ParenthesizedIdeographSelf',
u'\u56c5': 'CjkUnifiedIdeograph56c5',
u'\uf744': 'UnicodeCharacter0xf744',
u'\u1bc7': 'UnicodeCharacter0x1bc7',
u'\ub846': 'HangulSyllableRyenh',
u'\udcc9': 'UnicodeCharacter0xdcc9',
u'\u81cb': 'CjkUnifiedIdeograph81cb',
u'\u6950': 'CjkUnifiedIdeograph6950',
u'\u2a52': 'LogicalOrWithDotAbove',
u'\u4ed5': 'CjkUnifiedIdeograph4ed5',
u'\uef54': 'UnicodeCharacter0xef54',
u'\u33d7': 'SquarePh',
u'\ud056': 'HangulSyllableKyunh',
u'\uf4d9': 'UnicodeCharacter0xf4d9',
u'\u9558': 'CjkUnifiedIdeograph9558',
u'\ub9db': 'HangulSyllableMas',
u'\u4262': 'CjkUnifiedIdeograph4262',
u'\u66e5': 'CjkUnifiedIdeograph66e5',
u'\u0764': 'ArabicLetterKehehWithThreeDotsPointingUpwardsBelow',
u'\u2be7': 'UnicodeCharacter0x2be7',
u'\uc866': 'HangulSyllableJyebs',
u'\uece9': 'UnicodeCharacter0xece9',
u'\u8d68': 'CjkUnifiedIdeograph8d68',
u'\ud1eb': 'HangulSyllableTwaes',
u'\u92ed': 'CjkUnifiedIdeograph92ed',
u'\u7a72': 'CjkUnifiedIdeograph7a72',
u'\u3f74': 'CjkUnifiedIdeograph3f74',
u'\u43f7': 'CjkUnifiedIdeograph43f7',
u'\ue076': 'UnicodeCharacter0xe076',
u'\u04f9': 'CyrillicSmallLetterYeruWithDiaeresis',
u'\ua578': 'VaiSyllableBoo',
u'\uc9fb': 'HangulSyllableJjaegs',
u'\u8afd': 'CjkUnifiedIdeograph8afd',
u'\ub001': 'HangulSyllableGgweb',
u'\u5784': 'CjkUnifiedIdeograph5784',
u'\u1886': 'MongolianLetterAliGaliThreeBaluda',
u'\udd88': 'UnicodeCharacter0xdd88',
u'\u610b': 'CjkUnifiedIdeograph610b',
u'\u9e8a': 'CjkUnifiedIdeograph9e8a',
u'\u220d': 'SmallContainsAsMember',
u'\ue70f': 'UnicodeCharacter0xe70f',
u'\ua811': 'SylotiNagriLetterTtho',
u'\u4f94': 'CjkUnifiedIdeograph4f94',
u'\u3096': 'HiraganaLetterSmallKe',
u'\uf598': 'UnicodeCharacter0xf598',
u'\ub69a': 'HangulSyllableDdyop',
u'\u5a1d': 'CjkUnifiedIdeograph5a1d',
u'\u1f1f': 'UnicodeCharacter0x1f1f',
u'\uc021': 'HangulSyllableBbwig',
u'\u8523': 'CjkUnifiedIdeograph8523',
u'\u67a4': 'CjkUnifiedIdeograph67a4',
u'\u28a6': 'BraillePatternDots2368',
u'\ueda8': 'UnicodeCharacter0xeda8',
u'\uaeaa': 'HangulSyllableGgyaelm',
u'\u722d': 'CjkUnifiedIdeograph722d',
u'\u93ac': 'CjkUnifiedIdeograph93ac',
u'\u372f': 'CjkUnifiedIdeograph372f',
u'\uf831': 'UnicodeCharacter0xf831',
u'\ubd33': 'HangulSyllableBwaed',
u'\u40b6': 'CjkUnifiedIdeograph40b6',
u'\u05b8': 'HebrewPointQamats',
u'\uc6ba': 'HangulSyllableUlm',
u'\u6a3d': 'CjkUnifiedIdeograph6a3d',
u'\u8bbc': 'CjkUnifiedIdeograph8bbc',
u'\u2f3f': 'KangxiRadicalHand',
u'\u1041': 'MyanmarDigitOne',
u'\ud543': 'HangulSyllablePid',
u'\u9645': 'CjkUnifiedIdeograph9645',
u'\u78c6': 'CjkUnifiedIdeograph78c6',
u'\u3dc8': 'CjkUnifiedIdeograph3dc8',
u'\ufeca': 'ArabicLetterAinFinalForm',
u'\ua3cc': 'YiSyllableRuop',
u'\u474f': 'CjkUnifiedIdeograph474f',
u'\u0851': 'UnicodeCharacter0x851',
u'\ucd53': 'HangulSyllableCwaes',
u'\u8e55': 'CjkUnifiedIdeograph8e55',
u'\u55d8': 'CjkUnifiedIdeograph55d8',
u'\u16da': 'RunicLetterLaukazLaguLogrL',
u'\udbdc': 'UnicodeCharacter0xdbdc',
u'\u7f5f': 'CjkUnifiedIdeograph7f5f',
u'\u9cde': 'CjkUnifiedIdeograph9cde',
u'\u2061': 'FunctionApplication',
u'\ue563': 'UnicodeCharacter0xe563',
u'\ua665': 'CyrillicSmallLetterSoftEl',
u'\u4de8': 'HexagramForDecrease',
u'\u0eea': 'UnicodeCharacter0xeea',
u'\uf3ec': 'UnicodeCharacter0xf3ec',
u'\ub4ee': 'HangulSyllableDeubs',
u'\u5871': 'CjkUnifiedIdeograph5871',
u'\u1d73': 'LatinSmallLetterRWithFishhookAndMiddleTilde',
u'\ude75': 'UnicodeCharacter0xde75',
u'\u8377': 'CjkUnifiedIdeograph8377',
u'\u65f8': 'CjkUnifiedIdeograph65f8',
u'\u26fa': 'UnicodeCharacter0x26fa',
u'\uebfc': 'UnicodeCharacter0xebfc',
u'\uacfe': 'HangulSyllableGwagg',
u'\u9200': 'CjkUnifiedIdeograph9200',
u'\ub683': 'HangulSyllableDdyogs',
u'\u5102': 'CjkUnifiedIdeograph5102',
u'\u7585': 'CjkUnifiedIdeograph7585',
u'\udb06': 'UnicodeCharacter0xdb06',
u'\uff89': 'HalfwidthKatakanaLetterNo',
u'\u248f': 'DigitEightFullStop',
u'\u8a10': 'CjkUnifiedIdeograph8a10',
u'\uae93': 'HangulSyllableGgyalh',
u'\u4912': 'CjkUnifiedIdeograph4912',
u'\u6d95': 'CjkUnifiedIdeograph6d95',
u'\uf316': 'UnicodeCharacter0xf316',
u'\u1799': 'KhmerLetterYo',
u'\u999d': 'CjkUnifiedIdeograph999d',
u'\u381c': 'CjkUnifiedIdeograph381c',
u'\u5c9f': 'CjkUnifiedIdeograph5c9f',
u'\ua220': 'YiSyllableGguo',
u'\uc6a3': 'HangulSyllableYolh',
u'\u6122': 'CjkUnifiedIdeograph6122',
u'\ueb26': 'UnicodeCharacter0xeb26',
u'\u0fa9': 'TibetanSubjoinedLetterTsa',
u'\ub1ad': 'HangulSyllableNwat',
u'\u502c': 'CjkUnifiedIdeograph502c',
u'\u74af': 'CjkUnifiedIdeograph74af',
u'\uda30': 'UnicodeCharacter0xda30',
u'\ufeb3': 'ArabicLetterSeenInitialForm',
u'\u0336': 'CombiningLongStrokeOverlay',
u'\u27b9': 'HeavyBlackfeatheredNorthEastArrow',
u'\u853a': 'CjkUnifiedIdeograph853a',
u'\ua9bd': 'UnicodeCharacter0xa9bd',
u'\u483c': 'CjkUnifiedIdeograph483c',
u'\u6cbf': 'CjkUnifiedIdeograph6cbf',
u'\uf240': 'UnicodeCharacter0xf240',
u'\u16c3': 'RunicLetterJeranJ',
u'\u98c7': 'CjkUnifiedIdeograph98c7',
u'\u3b46': 'CjkUnifiedIdeograph3b46',
u'\u5fc9': 'CjkUnifiedIdeograph5fc9',
u'\ubd4a': 'HangulSyllableBoegg',
u'\uc1cd': 'HangulSyllableSwaelg',
u'\u604c': 'CjkUnifiedIdeograph604c',
u'\uea50': 'UnicodeCharacter0xea50',
u'\u0ed3': 'LaoDigitThree',
u'\ub0d7': 'HangulSyllableNyad',
u'\u5356': 'CjkUnifiedIdeograph5356',
u'\u77d9': 'CjkUnifiedIdeograph77d9',
u'\ud55a': 'HangulSyllableHagg',
u'\uf9dd': 'CjkCompatibilityIdeographf9dd',
u'\u0260': 'LatinSmallLetterGWithHook',
u'\u26e3': 'UnicodeCharacter0x26e3',
u'\u8464': 'CjkUnifiedIdeograph8464',
u'\ua8e7': 'UnicodeCharacter0xa8e7',
u'\u4b66': 'CjkUnifiedIdeograph4b66',
u'\u6fe9': 'CjkUnifiedIdeograph6fe9',
u'\ucd6a': 'HangulSyllableCoelp',
u'\u11ed': 'HangulJongseongIeungssangkiyeok',
u'\u9bf1': 'CjkUnifiedIdeograph9bf1',
u'\u3a70': 'CjkUnifiedIdeograph3a70',
u'\u5ef3': 'CjkUnifiedIdeograph5ef3',
u'\ubc74': 'HangulSyllableByaels',
u'\uc0f7': 'HangulSyllableSyas',
u'\u6376': 'CjkUnifiedIdeograph6376',
u'\ue57a': 'UnicodeCharacter0xe57a',
u'\u09fd': 'UnicodeCharacter0x9fd',
u'\u3301': 'SquareAruhua',
u'\u5280': 'CjkUnifiedIdeograph5280',
u'\ub505': 'HangulSyllableDyilt',
u'\ud484': 'HangulSyllablePuls',
u'\u7807': 'CjkUnifiedIdeograph7807',
u'\ue20b': 'UnicodeCharacter0xe20b',
u'\u1d8a': 'LatinSmallLetterSWithPalatalHook',
u'\u878e': 'CjkUnifiedIdeograph878e',
u'\u2b11': 'LeftwardsArrowWithTipUpwards',
u'\u4a90': 'CjkUnifiedIdeograph4a90',
u'\uad15': 'HangulSyllableGwat',
u'\ucc94': 'HangulSyllableCyaek',
u'\u1a1b': 'BugineseVowelSignAe',
u'\u359a': 'CjkUnifiedIdeograph359a',
u'\u9c1f': 'CjkUnifiedIdeograph9c1f',
u'\ubf9e': 'HangulSyllableBboelm',
u'\u4321': 'CjkUnifiedIdeograph4321',
u'\u62a0': 'CjkUnifiedIdeograph62a0',
u'\uc525': 'HangulSyllableSsyit',
u'\ue4a4': 'UnicodeCharacter0xe4a4',
u'\u322b': 'ParenthesizedIdeographFire',
u'\u2daa': 'EthiopicSyllableCci',
u'\ub42f': 'HangulSyllableDoec',
u'\ud7ae': 'UnicodeCharacter0xd7ae',
u'\u7b31': 'CjkUnifiedIdeograph7b31',
u'\ufd35': 'ArabicLigatureSeenWithHahMedialForm',
u'\u1cb4': 'UnicodeCharacter0x1cb4',
u'\u86b8': 'CjkUnifiedIdeograph86b8',
u'\u2a3b': 'MultiplicationSignInTriangle',
u'\u45ba': 'CjkUnifiedIdeograph45ba',
u'\uac3f': 'HangulSyllableGyad',
u'\ucfbe': 'HangulSyllableKoej',
u'\u1545': 'CanadianSyllabicsRaai',
u'\u34c4': 'CjkUnifiedIdeograph34c4',
u'\u9f49': 'CjkUnifiedIdeograph9f49',
u'\ubec8': 'HangulSyllableBbyaess',
u'\u424b': 'CjkUnifiedIdeograph424b',
u'\u7dca': 'CjkUnifiedIdeograph7dca',
u'\uc44f': 'HangulSyllableSsyod',
u'\ue7ce': 'UnicodeCharacter0xe7ce',
u'\u0d55': 'UnicodeCharacter0xd55',
u'\u2cd4': 'CopticCapitalLetterOldCopticHat',
u'\ub759': 'HangulSyllableDdying',
u'\ud6d8': 'HangulSyllableHuss',
u'\u7a5b': 'CjkUnifiedIdeograph7a5b',
u'\ufc5f': 'ArabicLigatureShaddaWithKasratanIsolatedForm',
u'\u1fde': 'GreekDasiaAndOxia',
u'\u81e2': 'CjkUnifiedIdeograph81e2',
u'\u2565': 'BoxDrawingsDownDoubleAndHorizontalSingle',
u'\u44e4': 'CjkUnifiedIdeograph44e4',
u'\uaf69': 'HangulSyllableGgwaenj',
u'\ucee8': 'HangulSyllableKeon',
u'\u146f': 'CanadianSyllabicsKo',
u'\u37ee': 'CjkUnifiedIdeograph37ee',
u'\u9e73': 'CjkUnifiedIdeograph9e73',
u'\ub9f2': 'HangulSyllableMaelp',
u'\u5d75': 'CjkUnifiedIdeograph5d75',
u'\u7cf4': 'CjkUnifiedIdeograph7cf4',
u'\uc779': 'HangulSyllableInj',
u'\ue6f8': 'UnicodeCharacter0xe6f8',
u'\u0c7f': 'TeluguSignTuumu',
u'\u2ffe': 'UnicodeCharacter0x2ffe',
u'\u5a06': 'CjkUnifiedIdeograph5a06',
u'\u7e89': 'CjkUnifiedIdeograph7e89',
u'\u1f08': 'GreekCapitalLetterAlphaWithPsili',
u'\u238b': 'BrokenCircleWithNorthwestArrow',
u'\udc0a': 'UnicodeCharacter0xdc0a',
u'\ue08d': 'UnicodeCharacter0xe08d',
u'\u810c': 'CjkUnifiedIdeograph810c',
u'\ua58f': 'VaiSyllableKoo',
u'\u7216': 'CjkUnifiedIdeograph7216',
u'\u3718': 'CjkUnifiedIdeograph3718',
u'\u5b9b': 'CjkUnifiedIdeograph5b9b',
u'\uf41a': 'UnicodeCharacter0xf41a',
u'\u189d': 'MongolianLetterManchuAliGaliJha',
u'\ub91c': 'HangulSyllableRweok',
u'\udd9f': 'UnicodeCharacter0xdd9f',
u'\u82a1': 'CjkUnifiedIdeograph82a1',
u'\u6a26': 'CjkUnifiedIdeograph6a26',
u'\u2f28': 'KangxiRadicalInch',
u'\u73ab': 'CjkUnifiedIdeograph73ab',
u'\uec2a': 'UnicodeCharacter0xec2a',
u'\u30ad': 'KatakanaLetterKi',
u'\ud12c': 'HangulSyllableTyaek',
u'\uf5af': 'UnicodeCharacter0xf5af',
u'\u962e': 'CjkUnifiedIdeograph962e',
u'\ubab1': 'HangulSyllableMolg',
u'\u4738': 'CjkUnifiedIdeograph4738',
u'\u6bbb': 'CjkUnifiedIdeograph6bbb',
u'\u043a': 'CyrillicSmallLetterKa',
u'\u28bd': 'BraillePatternDots134568',
u'\uc93c': 'HangulSyllableJwel',
u'\uedbf': 'UnicodeCharacter0xedbf',
u'\u8e3e': 'CjkUnifiedIdeograph8e3e',
u'\ud2c1': 'HangulSyllableTeulg',
u'\u97c3': 'CjkUnifiedIdeograph97c3',
u'\u7f48': 'CjkUnifiedIdeograph7f48',
u'\u3c4a': 'CjkUnifiedIdeograph3c4a',
u'\u40cd': 'CjkUnifiedIdeograph40cd',
u'\ue14c': 'UnicodeCharacter0xe14c',
u'\u05cf': 'UnicodeCharacter0x5cf',
u'\ua64e': 'CyrillicCapitalLetterNeutralYer',
u'\ucad1': 'HangulSyllableJjong',
u'\u8fd3': 'CjkUnifiedIdeograph8fd3',
u'\u545a': 'CjkUnifiedIdeograph545a',
u'\u78dd': 'CjkUnifiedIdeograph78dd',
u'\u195c': 'TaiLeLetterFa',
u'\u3ddf': 'CjkUnifiedIdeograph3ddf',
u'\ude5e': 'UnicodeCharacter0xde5e',
u'\ue2e1': 'UnicodeCharacter0xe2e1',
u'\u8360': 'CjkUnifiedIdeograph8360',
u'\ua7e3': 'UnicodeCharacter0xa7e3',
u'\u4c6a': 'CjkUnifiedIdeograph4c6a',
u'\u316c': 'HangulLetterRieulpansios',
u'\u55ef': 'CjkUnifiedIdeograph55ef',
u'\uf66e': 'UnicodeCharacter0xf66e',
u'\u1af1': 'UnicodeCharacter0x1af1',
u'\ubb70': 'HangulSyllableMwen',
u'\udff3': 'UnicodeCharacter0xdff3',
u'\u9cf5': 'CjkUnifiedIdeograph9cf5',
u'\u647a': 'CjkUnifiedIdeograph647a',
u'\u297c': 'LeftFishTail',
u'\u4dff': 'HexagramForBeforeCompletion',
u'\uee7e': 'UnicodeCharacter0xee7e',
u'\ud380': 'HangulSyllablePeon',
u'\u7703': 'CjkUnifiedIdeograph7703',
u'\u9082': 'CjkUnifiedIdeograph9082',
u'\u3405': 'CjkUnifiedIdeograph3405',
u'\uf907': 'CjkCompatibilityIdeographf907',
u'\ube09': 'HangulSyllableByut',
u'\u418c': 'CjkUnifiedIdeograph418c',
u'\u068e': 'ArabicLetterDul',
u'\ucb90': 'HangulSyllableJjwem',
u'\u6f13': 'CjkUnifiedIdeograph6f13',
u'\u8892': 'CjkUnifiedIdeograph8892',
u'\u2c15': 'GlagoliticCapitalLetterTvrido',
u'\u1117': 'HangulChoseongTikeutkiyeok',
u'\ud619': 'HangulSyllableHyeot',
u'\u9b1b': 'CjkUnifiedIdeograph9b1b',
u'\u799c': 'CjkUnifiedIdeograph799c',
u'\u3e9e': 'CjkUnifiedIdeograph3e9e',
u'\ue3a0': 'UnicodeCharacter0xe3a0',
u'\ua0a2': 'YiSyllableHmop',
u'\u4425': 'CjkUnifiedIdeograph4425',
u'\u0927': 'DevanagariLetterDha',
u'\uce29': 'HangulSyllableCeulg',
u'\ub32b': 'HangulSyllableDyalh',
u'\u56ae': 'CjkUnifiedIdeograph56ae',
u'\u1bb0': 'SundaneseDigitZero',
u'\ud8b2': 'UnicodeCharacter0xd8b2',
u'\u7c35': 'CjkUnifiedIdeograph7c35',
u'\u9db4': 'CjkUnifiedIdeograph9db4',
u'\u2137': 'GimelSymbol',
u'\ue639': 'UnicodeCharacter0xe639',
u'\uab3b': 'UnicodeCharacter0xab3b',
u'\u4ebe': 'CjkUnifiedIdeograph4ebe',
u'\u33c0': 'SquareKOhm',
u'\uf0c2': 'UnicodeCharacter0xf0c2',
u'\ub5c4': 'HangulSyllableDdel',
u'\u5947': 'CjkUnifiedIdeograph5947',
u'\u1e49': 'LatinSmallLetterNWithLineBelow',
u'\uc34b': 'HangulSyllableSsyah',
u'\u804d': 'CjkUnifiedIdeograph804d',
u'\u66ce': 'CjkUnifiedIdeograph66ce',
u'\u2bd0': 'UnicodeCharacter0x2bd0',
u'\ue8d2': 'UnicodeCharacter0xe8d2',
u'\uadd4': 'HangulSyllableGwiss',
u'\u7157': 'CjkUnifiedIdeograph7157',
u'\u92d6': 'CjkUnifiedIdeograph92d6',
u'\u3659': 'CjkUnifiedIdeograph3659',
u'\ufb5b': 'ArabicLetterBehehFinalForm',
u'\ub85d': 'HangulSyllableRog',
u'\u43e0': 'CjkUnifiedIdeograph43e0',
u'\xe2': 'LatinSmallLetterAWithCircumflex',
u'\uc5e4': 'HangulSyllableEss',
u'\u6967': 'CjkUnifiedIdeograph6967',
u'\u8ae6': 'CjkUnifiedIdeograph8ae6',
u'\u2e69': 'UnicodeCharacter0x2e69',
u'\u136b': 'EthiopicDigitThree',
u'\ud06d': 'HangulSyllableKeug',
u'\u956f': 'CjkUnifiedIdeograph956f',
u'\u7bf0': 'CjkUnifiedIdeograph7bf0',
u'\u38f2': 'CjkUnifiedIdeograph38f2',
u'\ufdf4': 'ArabicLigatureMohammadIsolatedForm',
u'\ua2f6': 'YiSyllableNza',
u'\u4679': 'CjkUnifiedIdeograph4679',
u'\u0b7b': 'UnicodeCharacter0xb7b',
u'\uc87d': 'HangulSyllableJolt',
u'\u8d7f': 'CjkUnifiedIdeograph8d7f',
u'\uf581': 'UnicodeCharacter0xf581',
u'\u1204': 'EthiopicSyllableHee',
u'\u3e87': 'CjkUnifiedIdeograph3e87',
u'\u9808': 'CjkUnifiedIdeograph9808',
u'\ua48b': 'YiSyllableYyrx',
u'\u5b0a': 'CjkUnifiedIdeograph5b0a',
u'\u678d': 'CjkUnifiedIdeograph678d',
u'\uc10e': 'HangulSyllableSyaelp',
u'\ued91': 'UnicodeCharacter0xed91',
u'\u0a14': 'GurmukhiLetterAu',
u'\u5697': 'CjkUnifiedIdeograph5697',
u'\ub018': 'HangulSyllableGgwils',
u'\udc9b': 'UnicodeCharacter0xdc9b',
u'\u731a': 'CjkUnifiedIdeograph731a',
u'\uf91e': 'CjkCompatibilityIdeographf91e',
u'\u05a1': 'HebrewAccentPazer',
u'\u83a5': 'CjkUnifiedIdeograph83a5',
u'\u2224': 'DoesNotDivide',
u'\u4ea7': 'CjkUnifiedIdeograph4ea7',
u'\ua828': 'SylotiNagriPoetryMark1',
u'\uf4ab': 'UnicodeCharacter0xf4ab',
u'\u6b2a': 'CjkUnifiedIdeograph6b2a',
u'\u112e': 'HangulChoseongSiosnieun',
u'\u3db1': 'CjkUnifiedIdeograph3db1',
u'\u9f32': 'CjkUnifiedIdeograph9f32',
u'\ubbb5': 'HangulSyllableMyub',
u'\u5a34': 'CjkUnifiedIdeograph5a34',
u'\u66b7': 'CjkUnifiedIdeograph66b7',
u'\uc038': 'HangulSyllableBbwik',
u'\uecbb': 'UnicodeCharacter0xecbb',
u'\u093e': 'DevanagariVowelSignAa',
u'\u55c1': 'CjkUnifiedIdeograph55c1',
u'\ub742': 'HangulSyllableDdeup',
u'\ud3c5': 'HangulSyllablePyeob',
u'\u7244': 'CjkUnifiedIdeograph7244',
u'\uf848': 'UnicodeCharacter0xf848',
u'\u04cb': 'CyrillicCapitalLetterKhakassianChe',
u'\u82cf': 'CjkUnifiedIdeograph82cf',
u'\u214e': 'TurnedSmallF',
u'\u4dd1': 'HexagramForWorkOnTheDecayed',
u'\uaf52': 'HangulSyllableGgwalm',
u'\ucbd5': 'HangulSyllableJjeug',
u'\u6a54': 'CjkUnifiedIdeograph6a54',
u'\u1058': 'MyanmarVowelSignVocalicL',
u'\u3cdb': 'CjkUnifiedIdeograph3cdb',
u'\u9e5c': 'CjkUnifiedIdeograph9e5c',
u'\ubadf': 'HangulSyllableMwah',
u'\u595e': 'CjkUnifiedIdeograph595e',
u'\u65e1': 'CjkUnifiedIdeograph65e1',
u'\uc762': 'HangulSyllableYilm',
u'\ue3e5': 'UnicodeCharacter0xe3e5',
u'\u0868': 'UnicodeCharacter0x868',
u'\u54eb': 'CjkUnifiedIdeograph54eb',
u'\ub66c': 'HangulSyllableDdoel',
u'\ud2ef': 'HangulSyllableTyih',
u'\u716e': 'CjkUnifiedIdeograph716e',
u'\uff72': 'HalfwidthKatakanaLetterI',
u'\u1bf5': 'UnicodeCharacter0x1bf5',
u'\u81f9': 'CjkUnifiedIdeograph81f9',
u'\u2078': 'SuperscriptEight',
u'\u4cfb': 'CjkUnifiedIdeograph4cfb',
u'\uae7c': 'HangulSyllableGgaess',
u'\ucaff': 'HangulSyllableJjwaelb',
u'\u697e': 'CjkUnifiedIdeograph697e',
u'\uf003': 'UnicodeCharacter0xf003',
u'\u1782': 'KhmerLetterKo',
u'\u9d86': 'CjkUnifiedIdeograph9d86',
u'\u3909': 'CjkUnifiedIdeograph3909',
u'\u5888': 'CjkUnifiedIdeograph5888',
u'\ua70d': 'ModifierLetterExtrahighDottedLeftstemToneBar',
u'\uc68c': 'HangulSyllableOess',
u'\u620f': 'CjkUnifiedIdeograph620f',
u'\ue813': 'UnicodeCharacter0xe813',
u'\u0f92': 'TibetanSubjoinedLetterGa',
u'\u9617': 'CjkUnifiedIdeograph9617',
u'\ub596': 'HangulSyllableDdyaebs',
u'\u5119': 'CjkUnifiedIdeograph5119',
u'\u7098': 'CjkUnifiedIdeograph7098',
u'\udf1d': 'UnicodeCharacter0xdf1d',
u'\ufe9c': 'ArabicLetterThehMedialForm',
u'\u27a2': 'ThreedToplightedRightwardsArrowhead',
u'\u8e27': 'CjkUnifiedIdeograph8e27',
u'\uada6': 'HangulSyllableGwegg',
u'\u4929': 'CjkUnifiedIdeograph4929',
u'\u68a8': 'CjkUnifiedIdeograph68a8',
u'\uf72d': 'UnicodeCharacter0xf72d',
u'\u16ac': 'RunicLetterLongbranchossO',
u'\u9cb0': 'CjkUnifiedIdeograph9cb0',
u'\u3833': 'CjkUnifiedIdeograph3833',
u'\u5fb2': 'CjkUnifiedIdeograph5fb2',
u'\ua637': 'UnicodeCharacter0xa637',
u'\uc5b6': 'HangulSyllableEogg',
u'\u6139': 'CjkUnifiedIdeograph6139',
u'\uef3d': 'UnicodeCharacter0xef3d',
u'\u0ebc': 'LaoSemivowelSignLo',
u'\u9541': 'CjkUnifiedIdeograph9541',
u'\ub4c0': 'HangulSyllableDyu',
u'\u5043': 'CjkUnifiedIdeograph5043',
u'\u77c2': 'CjkUnifiedIdeograph77c2',
u'\ude47': 'UnicodeCharacter0xde47',
u'\ufdc6': 'ArabicLigatureSeenWithKhahWithYehFinalForm',
u'\u074d': 'SyriacLetterSogdianZhain',
u'\u26cc': 'UnicodeCharacter0x26cc',
u'\u8d51': 'CjkUnifiedIdeograph8d51',
u'\uacd0': 'HangulSyllableGyels',
u'\u4853': 'CjkUnifiedIdeograph4853',
u'\u6fd2': 'CjkUnifiedIdeograph6fd2',
u'\uf657': 'UnicodeCharacter0xf657',
u'\u15d6': 'CanadianSyllabicsCarrierHwu',
u'\u93da': 'CjkUnifiedIdeograph93da',
u'\u3f5d': 'CjkUnifiedIdeograph3f5d',
u'\u5edc': 'CjkUnifiedIdeograph5edc',
u'\ua561': 'VaiSyllableNda',
u'\uc4e0': 'HangulSyllableSsyuls',
u'\u6063': 'CjkUnifiedIdeograph6063',
u'\uee67': 'UnicodeCharacter0xee67',
u'\u0de6': 'UnicodeCharacter0xde6',
u'\u946b': 'CjkUnifiedIdeograph946b',
u'\u8bea': 'CjkUnifiedIdeograph8bea',
u'\u576d': 'CjkUnifiedIdeograph576d',
u'\u76ec': 'CjkUnifiedIdeograph76ec',
u'\udd71': 'UnicodeCharacter0xdd71',
u'\ufcf0': 'ArabicLigatureYehWithMeemMedialForm',
u'\u0677': 'ArabicLetterUWithHamzaAbove',
u'\u25f6': 'WhiteCircleWithLowerRightQuadrant',
u'\u8c7b': 'CjkUnifiedIdeograph8c7b',
u'\ua3fa': 'YiSyllableJyrx',
u'\u4f7d': 'CjkUnifiedIdeograph4f7d',
u'\u6efc': 'CjkUnifiedIdeograph6efc',
u'\u7481': 'CjkUnifiedIdeograph7481',
u'\u1500': 'CanadianSyllabicsSwa',
u'\u3183': 'HangulLetterYesieungpansios',
u'\ud602': 'HangulSyllableHyeogg',
u'\uf285': 'UnicodeCharacter0xf285',
u'\u9304': 'CjkUnifiedIdeograph9304',
u'\ubf87': 'HangulSyllableBbwaelh',
u'\u400e': 'CjkUnifiedIdeograph400e',
u'\u6c91': 'CjkUnifiedIdeograph6c91',
u'\u0d10': 'MalayalamLetterAi',
u'\u2993': 'LeftArcLessthanBracket',
u'\uce12': 'HangulSyllableCyulp',
u'\uea95': 'UnicodeCharacter0xea95',
u'\u8b14': 'CjkUnifiedIdeograph8b14',
u'\ud797': 'HangulSyllableHilh',
u'\u9099': 'CjkUnifiedIdeograph9099',
u'\u781e': 'CjkUnifiedIdeograph781e',
u'\u2520': 'BoxDrawingsVerticalHeavyAndRightLight',
u'\u41a3': 'CjkUnifiedIdeograph41a3',
u'\ue622': 'UnicodeCharacter0xe622',
u'\u02a5': 'LatinSmallLetterDzDigraphWithCurl',
u'\ua324': 'YiSyllableSyt',
u'\ucfa7': 'HangulSyllableKwaeh',
u'\u88a9': 'CjkUnifiedIdeograph88a9',
u'\u5d30': 'CjkUnifiedIdeograph5d30',
u'\u79b3': 'CjkUnifiedIdeograph79b3',
u'\u1e32': 'LatinCapitalLetterKWithDotBelow',
u'\u3ab5': 'CjkUnifiedIdeograph3ab5',
u'\udb34': 'UnicodeCharacter0xdb34',
u'\ue7b7': 'UnicodeCharacter0xe7b7',
u'\u8436': 'CjkUnifiedIdeograph8436',
u'\ua0b9': 'YiSyllableMuot',
u'\u7540': 'CjkUnifiedIdeograph7540',
u'\u3642': 'CjkUnifiedIdeograph3642',
u'\u52c5': 'CjkUnifiedIdeograph52c5',
u'\uf344': 'UnicodeCharacter0xf344',
u'\u1fc7': 'GreekSmallLetterEtaWithPerispomeniAndYpogegrammeni',
u'\ubc46': 'HangulSyllableBaej',
u'\ud8c9': 'UnicodeCharacter0xd8c9',
u'\u85cb': 'CjkUnifiedIdeograph85cb',
u'\u6d50': 'CjkUnifiedIdeograph6d50',
u'\u2e52': 'UnicodeCharacter0x2e52',
u'\u4ad5': 'CjkUnifiedIdeograph4ad5',
u'\ueb54': 'UnicodeCharacter0xeb54',
u'\u37d7': 'CjkUnifiedIdeograph37d7',
u'\ud456': 'HangulSyllablePoej',
u'\uf0d9': 'UnicodeCharacter0xf0d9',
u'\u9158': 'CjkUnifiedIdeograph9158',
u'\ubddb': 'HangulSyllableBwid',
u'\u4662': 'CjkUnifiedIdeograph4662',
u'\u62e5': 'CjkUnifiedIdeograph62e5',
u'\u0364': 'CombiningLatinSmallLetterE',
u'\u2fe7': 'UnicodeCharacter0x2fe7',
u'\ucc66': 'HangulSyllableCyanh',
u'\ue8e9': 'UnicodeCharacter0xe8e9',
u'\u8968': 'CjkUnifiedIdeograph8968',
u'\ud5eb': 'HangulSyllableHed',
u'\u96ed': 'CjkUnifiedIdeograph96ed',
u'\u7e72': 'CjkUnifiedIdeograph7e72',
u'\u3b74': 'CjkUnifiedIdeograph3b74',
u'\u47f7': 'CjkUnifiedIdeograph47f7',
u'\ue476': 'UnicodeCharacter0xe476',
u'\xf9': 'LatinSmallLetterUWithGrave',
u'\ua178': 'YiSyllableHnop',
u'\ucdfb': 'HangulSyllableCwis',
u'\u8efd': 'CjkUnifiedIdeograph8efd',
u'\ub401': 'HangulSyllableDwaenj',
u'\u5384': 'CjkUnifiedIdeograph5384',
u'\u1c86': 'UnicodeCharacter0x1c86',
u'\ud988': 'UnicodeCharacter0xd988',
u'\u650b': 'CjkUnifiedIdeograph650b',
u'\u9a8a': 'CjkUnifiedIdeograph9a8a',
u'\u260d': 'Opposition',
u'\ue30f': 'UnicodeCharacter0xe30f',
u'\uac11': 'HangulSyllableGab',
u'\u4b94': 'CjkUnifiedIdeograph4b94',
u'\u3496': 'CjkUnifiedIdeograph3496',
u'\uf198': 'UnicodeCharacter0xf198',
u'\ub29a': 'HangulSyllableNeulm',
u'\u5e1d': 'CjkUnifiedIdeograph5e1d',
u'\u1b1f': 'BalineseLetterDaMurdaAlpaprana',
u'\uc421': 'HangulSyllableSswaeb',
u'\u8123': 'CjkUnifiedIdeograph8123',
u'\u63a4': 'CjkUnifiedIdeograph63a4',
u'\u2ca6': 'CopticCapitalLetterTau',
u'\ue9a8': 'UnicodeCharacter0xe9a8',
u'\uaaaa': 'UnicodeCharacter0xaaaa',
u'\u762d': 'CjkUnifiedIdeograph762d',
u'\u97ac': 'CjkUnifiedIdeograph97ac',
u'\u332f': 'SquarePikuru',
u'\ufc31': 'ArabicLigatureFehWithAlefMaksuraIsolatedForm',
u'\ub933': 'HangulSyllableRwes',
u'\u44b6': 'CjkUnifiedIdeograph44b6',
u'\u01b8': 'LatinCapitalLetterEzhReversed',
u'\uc2ba': 'HangulSyllableSeuj',
u'\u6e3d': 'CjkUnifiedIdeograph6e3d',
u'\u8fbc': 'CjkUnifiedIdeograph8fbc',
u'\u2b3f': 'WaveArrowPointingDirectlyLeft',
u'\u1441': 'CanadianSyllabicsWestcreePwo',
u'\ud143': 'HangulSyllableTeos',
u'\u9245': 'CjkUnifiedIdeograph9245',
u'\u7cc6': 'CjkUnifiedIdeograph7cc6',
u'\u39c8': 'CjkUnifiedIdeograph39c8',
u'\ufaca': 'CjkCompatibilityIdeographfaca',
u'\ua7cc': 'UnicodeCharacter0xa7cc',
u'\u434f': 'CjkUnifiedIdeograph434f',
u'\u0c51': 'UnicodeCharacter0xc51',
u'\uc953': 'HangulSyllableJwigs',
u'\u8a55': 'CjkUnifiedIdeograph8a55',
u'\u51d8': 'CjkUnifiedIdeograph51d8',
u'\u12da': 'EthiopicSyllableZi',
u'\udfdc': 'UnicodeCharacter0xdfdc',
u'\u7b5f': 'CjkUnifiedIdeograph7b5f',
u'\u98de': 'CjkUnifiedIdeograph98de',
u'\u2461': 'CircledDigitTwo',
u'\ue163': 'UnicodeCharacter0xe163',
u'\ua265': 'YiSyllableNguox',
u'\u49e8': 'CjkUnifiedIdeograph49e8',
u'\u0aea': 'GujaratiDigitFour',
u'\uf7ec': 'UnicodeCharacter0xf7ec',
u'\ub0ee': 'HangulSyllableNyaegg',
u'\u5c71': 'CjkUnifiedIdeograph5c71',
u'\u1973': 'TaiLeLetterTone5',
u'\uda75': 'UnicodeCharacter0xda75',
u'\u8777': 'CjkUnifiedIdeograph8777',
u'\u61f8': 'CjkUnifiedIdeograph61f8',
u'\u22fa': 'ContainsWithLongHorizontalStroke',
u'\ueffc': 'UnicodeCharacter0xeffc',
u'\ua8fe': 'UnicodeCharacter0xa8fe',
u'\u9600': 'CjkUnifiedIdeograph9600',
u'\ub283': 'HangulSyllableNyulh',
u'\u5502': 'CjkUnifiedIdeograph5502',
u'\u7185': 'CjkUnifiedIdeograph7185',
u'\udf06': 'UnicodeCharacter0xdf06',
u'\ufb89': 'ArabicLetterDdalFinalForm',
u'\u040c': 'CyrillicCapitalLetterKje',
u'\u208f': 'UnicodeCharacter0x208f',
u'\u8e10': 'CjkUnifiedIdeograph8e10',
u'\uaa93': 'UnicodeCharacter0xaa93',
u'\u4d12': 'CjkUnifiedIdeograph4d12',
u'\u6995': 'CjkUnifiedIdeograph6995',
u'\uf716': 'UnicodeCharacter0xf716',
u'\u1399': 'EthiopicTonalMarkKurt',
u'\u9d9d': 'CjkUnifiedIdeograph9d9d',
u'\u3c1c': 'CjkUnifiedIdeograph3c1c',
u'\u589f': 'CjkUnifiedIdeograph589f',
u'\ua620': 'VaiDigitZero',
u'\uc2a3': 'HangulSyllableSyuh',
u'\u6522': 'CjkUnifiedIdeograph6522',
u'\uef26': 'UnicodeCharacter0xef26',
u'\u0ba9': 'TamilLetterNnna',
u'\ub5ad': 'HangulSyllableDdeolt',
u'\u542c': 'CjkUnifiedIdeograph542c',
u'\u70af': 'CjkUnifiedIdeograph70af',
u'\ude30': 'UnicodeCharacter0xde30',
u'\ufab3': 'CjkCompatibilityIdeographfab3',
u'\u0736': 'SyriacRbasaAbove',
u'\u23b9': 'RightVerticalBoxLine',
u'\u813a': 'CjkUnifiedIdeograph813a',
u'\uadbd': 'HangulSyllableGwet',
u'\u4c3c': 'CjkUnifiedIdeograph4c3c',
u'\u68bf': 'CjkUnifiedIdeograph68bf',
u'\uf640': 'UnicodeCharacter0xf640',
u'\u12c3': 'EthiopicSyllableKxwaa',
u'\u9cc7': 'CjkUnifiedIdeograph9cc7',
u'\u3f46': 'CjkUnifiedIdeograph3f46',
u'\u5bc9': 'CjkUnifiedIdeograph5bc9',
u'\ub94a': 'HangulSyllableRwilp',
u'\uc5cd': 'HangulSyllableEot',
u'\u644c': 'CjkUnifiedIdeograph644c',
u'\uee50': 'UnicodeCharacter0xee50',
u'\u0ad3': 'UnicodeCharacter0xad3',
u'\ub4d7': 'HangulSyllableDyuc',
u'\u5756': 'CjkUnifiedIdeograph5756',
u'\u73d9': 'CjkUnifiedIdeograph73d9',
u'\ud15a': 'HangulSyllableTelp',
u'\ufddd': 'UnicodeCharacter0xfddd',
u'\u0660': 'ArabicindicDigitZero',
u'\u22e3': 'NotSquareOriginalOfOrEqualTo',
u'\u8064': 'CjkUnifiedIdeograph8064',
u'\uace7': 'HangulSyllableGod',
u'\u4f66': 'CjkUnifiedIdeograph4f66',
u'\u6be9': 'CjkUnifiedIdeograph6be9',
u'\uc96a': 'HangulSyllableJwip',
u'\u15ed': 'CanadianSyllabicsCarrierPa',
u'\u9ff1': 'UnicodeCharacter0x9ff1',
u'\u3e70': 'CjkUnifiedIdeograph3e70',
u'\u5af3': 'CjkUnifiedIdeograph5af3',
u'\ub874': 'HangulSyllableRok',
u'\uc4f7': 'HangulSyllableSseud',
u'\u6776': 'CjkUnifiedIdeograph6776',
u'\ue17a': 'UnicodeCharacter0xe17a',
u'\u0dfd': 'UnicodeCharacter0xdfd',
u'\u3701': 'CjkUnifiedIdeograph3701',
u'\u5680': 'CjkUnifiedIdeograph5680',
u'\ub105': 'HangulSyllableNyaet',
u'\ud084': 'HangulSyllableKeuk',
u'\u7c07': 'CjkUnifiedIdeograph7c07',
u'\ue60b': 'UnicodeCharacter0xe60b',
u'\u198a': 'NewTaiLueLetterHighYa',
u'\u838e': 'CjkUnifiedIdeograph838e',
u'\u2f11': 'KangxiRadicalKnife',
u'\u4e90': 'CjkUnifiedIdeograph4e90',
u'\ua915': 'KayahLiLetterPa',
u'\uc894': 'HangulSyllableJwal',
u'\u1e1b': 'LatinSmallLetterEWithTildeBelow',
u'\u319a': 'IdeographicAnnotationSecondMark',
u'\u981f': 'CjkUnifiedIdeograph981f',
u'\ubb9e': 'HangulSyllableMwij',
u'\u4721': 'CjkUnifiedIdeograph4721',
u'\u66a0': 'CjkUnifiedIdeograph66a0',
u'\uc125': 'HangulSyllableSeolg',
u'\ue0a4': 'UnicodeCharacter0xe0a4',
u'\u362b': 'CjkUnifiedIdeograph362b',
u'\u29aa': 'MeasuredAngleWithOpenArmEndingInArrowPointingDownAndRight',
u'\ub02f': 'HangulSyllableGgyud',
u'\ud3ae': 'HangulSyllablePej',
u'\u7f31': 'CjkUnifiedIdeograph7f31',
u'\uf935': 'CjkCompatibilityIdeographf935',
u'\u18b4': 'UnicodeCharacter0x18b4',
u'\u82b8': 'CjkUnifiedIdeograph82b8',
u'\u2e3b': 'UnicodeCharacter0x2e3b',
u'\u41ba': 'CjkUnifiedIdeograph41ba',
u'\ua83f': 'UnicodeCharacter0xa83f',
u'\ucbbe': 'HangulSyllableJjyunh',
u'\u1145': 'HangulChoseongIeungsios',
u'\u30c4': 'KatakanaLetterTu',
u'\u9b49': 'CjkUnifiedIdeograph9b49',
u'\ubac8': 'HangulSyllableMwan',
u'\u464b': 'CjkUnifiedIdeograph464b',
u'\u79ca': 'CjkUnifiedIdeograph79ca',
u'\uc04f': 'HangulSyllableBbyus',
u'\ue3ce': 'UnicodeCharacter0xe3ce',
u'\u0955': 'UnicodeCharacter0x955',
u'\u28d4': 'BraillePatternDots3578',
u'\ub359': 'HangulSyllableDeonj',
u'\ud2d8': 'HangulSyllableTyin',
u'\u7e5b': 'CjkUnifiedIdeograph7e5b',
u'\uf85f': 'UnicodeCharacter0xf85f',
u'\u1bde': 'UnicodeCharacter0x1bde',
u'\u85e2': 'CjkUnifiedIdeograph85e2',
u'\u2165': 'RomanNumeralSix',
u'\u40e4': 'CjkUnifiedIdeograph40e4',
u'\uab69': 'UnicodeCharacter0xab69',
u'\ucae8': 'HangulSyllableJjwam',
u'\u106f': 'MyanmarLetterEasternPwoKarenYwa',
u'\u33ee': 'IdeographicTelegraphSymbolForDayFifteen',
u'\u9a73': 'CjkUnifiedIdeograph9a73',
u'\ubdf2': 'HangulSyllableByugg',
u'\u5975': 'CjkUnifiedIdeograph5975',
u'\u78f4': 'CjkUnifiedIdeograph78f4',
u'\uc379': 'HangulSyllableSseob',
u'\ue2f8': 'UnicodeCharacter0xe2f8',
u'\u087f': 'UnicodeCharacter0x87f',
u'\u2bfe': 'UnicodeCharacter0x2bfe',
u'\u5e06': 'CjkUnifiedIdeograph5e06',
u'\u7a89': 'CjkUnifiedIdeograph7a89',
u'\u1b08': 'BalineseLetterIkaraTedung',
u'\u278b': 'DingbatNegativeCircledSansserifDigitTwo',
u'\ud80a': 'UnicodeCharacter0xd80a',
u'\ue48d': 'UnicodeCharacter0xe48d',
u'\u850c': 'CjkUnifiedIdeograph850c',
u'\ua18f': 'YiSyllableNe',
u'\u7616': 'CjkUnifiedIdeograph7616',
u'\u3318': 'SquareGuramu',
u'\u5f9b': 'CjkUnifiedIdeograph5f9b',
u'\uf01a': 'UnicodeCharacter0xf01a',
u'\u1c9d': 'UnicodeCharacter0x1c9d',
u'\ubd1c': 'HangulSyllableBwals',
u'\ud99f': 'UnicodeCharacter0xd99f',
u'\u86a1': 'CjkUnifiedIdeograph86a1',
u'\u6e26': 'CjkUnifiedIdeograph6e26',
u'\u2b28': 'WhiteMediumLozenge',
u'\u77ab': 'CjkUnifiedIdeograph77ab',
u'\ue82a': 'UnicodeCharacter0xe82a',
u'\u34ad': 'CjkUnifiedIdeograph34ad',
u'\ud52c': 'HangulSyllablePyils',
u'\uf1af': 'UnicodeCharacter0xf1af',
u'\u922e': 'CjkUnifiedIdeograph922e',
u'\ubeb1': 'HangulSyllableBbyat',
u'\u4338': 'CjkUnifiedIdeograph4338',
u'\u6fbb': 'CjkUnifiedIdeograph6fbb',
u'\u2cbd': 'CopticSmallLetterCryptogrammicNi',
u'\ucd3c': 'HangulSyllableCwak',
u'\ue9bf': 'UnicodeCharacter0xe9bf',
u'\u8a3e': 'CjkUnifiedIdeograph8a3e',
u'\ud6c1': 'HangulSyllableHyot',
u'\u93c3': 'CjkUnifiedIdeograph93c3',
u'\u7b48': 'CjkUnifiedIdeograph7b48',
u'\u384a': 'CjkUnifiedIdeograph384a',
u'\u44cd': 'CjkUnifiedIdeograph44cd',
u'\ue54c': 'UnicodeCharacter0xe54c',
u'\u01cf': 'LatinCapitalLetterIWithCaron',
u'\ua24e': 'YiSyllableHxat',
u'\uced1': 'HangulSyllableKyaelg',
u'\u8bd3': 'CjkUnifiedIdeograph8bd3',
u'\u505a': 'CjkUnifiedIdeograph505a',
u'\u7cdd': 'CjkUnifiedIdeograph7cdd',
u'\u1d5c': 'ModifierLetterSmallAin',
u'\u39df': 'CjkUnifiedIdeograph39df',
u'\uda5e': 'UnicodeCharacter0xda5e',
u'\ue6e1': 'UnicodeCharacter0xe6e1',
u'\u8760': 'CjkUnifiedIdeograph8760',
u'\ua3e3': 'YiSyllableJip',
u'\u486a': 'CjkUnifiedIdeograph486a',
u'\u356c': 'CjkUnifiedIdeograph356c',
u'\u51ef': 'CjkUnifiedIdeograph51ef',
u'\uf26e': 'UnicodeCharacter0xf26e',
u'\u1ef1': 'LatinSmallLetterUWithHornAndDotBelow',
u'\ubf70': 'HangulSyllableBbwass',
u'\udbf3': 'UnicodeCharacter0xdbf3',
u'\u98f5': 'CjkUnifiedIdeograph98f5',
u'\u607a': 'CjkUnifiedIdeograph607a',
u'\u2d7c': 'UnicodeCharacter0x2d7c',
u'\u49ff': 'CjkUnifiedIdeograph49ff',
u'\uea7e': 'UnicodeCharacter0xea7e',
u'\ud780': 'HangulSyllableHyiss',
u'\u7303': 'CjkUnifiedIdeograph7303',
u'\u9482': 'CjkUnifiedIdeograph9482',
u'\u3005': 'IdeographicIterationMark',
u'\ufd07': 'ArabicLigatureDadWithAlefMaksuraIsolatedForm',
u'\uba09': 'HangulSyllableMyalg',
u'\u458c': 'CjkUnifiedIdeograph458c',
u'\u028e': 'LatinSmallLetterTurnedY',
u'\ucf90': 'HangulSyllableKwaen',
u'\u6b13': 'CjkUnifiedIdeograph6b13',
u'\u8c92': 'CjkUnifiedIdeograph8c92',
u'\u2815': 'BraillePatternDots135',
u'\u1517': 'CanadianSyllabicsShwe',
u'\ud219': 'HangulSyllableTyolg',
u'\u9f1b': 'CjkUnifiedIdeograph9f1b',
u'\u7d9c': 'CjkUnifiedIdeograph7d9c',
u'\u3a9e': 'CjkUnifiedIdeograph3a9e',
u'\ue7a0': 'UnicodeCharacter0xe7a0',
u'\ua4a2': 'YiRadicalZup',
u'\u4025': 'CjkUnifiedIdeograph4025',
u'\u0d27': 'MalayalamLetterDha',
u'\uca29': 'HangulSyllableJjyang',
u'\ub72b': 'HangulSyllableDdeugs',
u'\u52ae': 'CjkUnifiedIdeograph52ae',
u'\u1fb0': 'GreekSmallLetterAlphaWithVrachy',
u'\udcb2': 'UnicodeCharacter0xdcb2',
u'\u7835': 'CjkUnifiedIdeograph7835',
u'\u99b4': 'CjkUnifiedIdeograph99b4',
u'\u2537': 'BoxDrawingsUpLightAndHorizontalHeavy',
u'\ue239': 'UnicodeCharacter0xe239',
u'\uaf3b': 'HangulSyllableGgolh',
u'\u4abe': 'CjkUnifiedIdeograph4abe',
u'\u37c0': 'CjkUnifiedIdeograph37c0',
u'\uf4c2': 'UnicodeCharacter0xf4c2',
u'\ub1c4': 'HangulSyllableNwaess',
u'\u5d47': 'CjkUnifiedIdeograph5d47',
u'\u1a49': 'UnicodeCharacter0x1a49',
u'\uc74b': 'HangulSyllableEulh',
u'\u844d': 'CjkUnifiedIdeograph844d',
u'\u62ce': 'CjkUnifiedIdeograph62ce',
u'\u2fd0': 'KangxiRadicalNose',
u'\uecd2': 'UnicodeCharacter0xecd2',
u'\ua9d4': 'UnicodeCharacter0xa9d4',
u'\u7557': 'CjkUnifiedIdeograph7557',
u'\u96d6': 'CjkUnifiedIdeograph96d6',
u'\u3259': 'CircledNumberTwentyNine',
u'\uff5b': 'FullwidthLeftCurlyBracket',
u'\ubc5d': 'HangulSyllableByab',
u'\u47e0': 'CjkUnifiedIdeograph47e0',
u'\u04e2': 'CyrillicCapitalLetterIWithMacron',
u'\uc1e4': 'HangulSyllableSoen',
u'\u6d67': 'CjkUnifiedIdeograph6d67',
u'\u8ee6': 'CjkUnifiedIdeograph8ee6',
u'\u2a69': 'TripleHorizontalBarWithTripleVerticalStroke',
u'\u176b': 'TagbanwaLetterMa',
u'\ud46d': 'HangulSyllablePyob',
u'\u916f': 'CjkUnifiedIdeograph916f',
u'\u7ff0': 'CjkUnifiedIdeograph7ff0',
u'\u3cf2': 'CjkUnifiedIdeograph3cf2',
u'\uf9f4': 'CjkCompatibilityIdeographf9f4',
u'\ua6f6': 'UnicodeCharacter0xa6f6',
u'\u4279': 'CjkUnifiedIdeograph4279',
u'\u0f7b': 'TibetanVowelSignEe',
u'\ucc7d': 'HangulSyllableCyaeg',
u'\u897f': 'CjkUnifiedIdeograph897f',
u'\uf981': 'CjkCompatibilityIdeographf981',
u'\u1e04': 'LatinCapitalLetterBWithDotBelow',
u'\u3287': 'CircledIdeographEight',
u'\u9408': 'CjkUnifiedIdeograph9408',
u'\ua88b': 'SaurashtraLetterVocalicLl',
u'\u570a': 'CjkUnifiedIdeograph570a',
u'\u6b8d': 'CjkUnifiedIdeograph6b8d',
u'\ucd0e': 'HangulSyllableConh',
u'\ue191': 'UnicodeCharacter0xe191',
u'\u0614': 'ArabicSignTakhallus',
u'\u5a97': 'CjkUnifiedIdeograph5a97',
u'\ubc18': 'HangulSyllableBan',
u'\ud09b': 'HangulSyllableKyis',
u'\u7f1a': 'CjkUnifiedIdeograph7f1a',
u'\uf51e': 'UnicodeCharacter0xf51e',
u'\u09a1': 'BengaliLetterDda',
u'\u8fa5': 'CjkUnifiedIdeograph8fa5',
u'\u2e24': 'BottomLeftHalfBracket',
u'\u42a7': 'CjkUnifiedIdeograph42a7',
u'\ua428': 'YiSyllableJjux',
u'\uf8ab': 'UnicodeCharacter0xf8ab',
u'\u672a': 'CjkUnifiedIdeograph672a',
u'\u1d2e': 'ModifierLetterCapitalB',
u'\u31b1': 'BopomofoLetterOm',
u'\u9332': 'CjkUnifiedIdeograph9332',
u'\ub7b5': 'HangulSyllableRyag',
u'\u5634': 'CjkUnifiedIdeograph5634',
u'\u6ab7': 'CjkUnifiedIdeograph6ab7',
u'\ucc38': 'HangulSyllableCam',
u'\ue0bb': 'UnicodeCharacter0xe0bb',
u'\u053e': 'ArmenianCapitalLetterCa',
u'\u59c1': 'CjkUnifiedIdeograph59c1',
u'\ubb42': 'HangulSyllableMulp',
u'\udfc5': 'UnicodeCharacter0xdfc5',
u'\u7e44': 'CjkUnifiedIdeograph7e44',
u'\uf448': 'UnicodeCharacter0xf448',
u'\u08cb': 'UnicodeCharacter0x8cb',
u'\u8ecf': 'CjkUnifiedIdeograph8ecf',
u'\u2d4e': 'TifinaghLetterYam',
u'\u41d1': 'CjkUnifiedIdeograph41d1',
u'\ua352': 'YiSyllableZhex',
u'\uc7d5': 'HangulSyllableJyalt',
u'\u6654': 'CjkUnifiedIdeograph6654',
u'\u1c58': 'OlChikiDigitEight',
u'\u30db': 'KatakanaLetterHo',
u'\u925c': 'CjkUnifiedIdeograph925c',
u'\ub6df': 'HangulSyllableDdwelb',
u'\u555e': 'CjkUnifiedIdeograph555e',
u'\u69e1': 'CjkUnifiedIdeograph69e1',
u'\ucb62': 'HangulSyllableJjup',
u'\uefe5': 'UnicodeCharacter0xefe5',
u'\u0468': 'CyrillicCapitalLetterIotifiedLittleYus',
u'\u58eb': 'CjkUnifiedIdeograph58eb',
u'\uba6c': 'HangulSyllableMek',
u'\udeef': 'UnicodeCharacter0xdeef',
u'\u7d6e': 'CjkUnifiedIdeograph7d6e',
u'\uf372': 'UnicodeCharacter0xf372',
u'\u17f5': 'KhmerSymbolLekAttakPram',
u'\u8df9': 'CjkUnifiedIdeograph8df9',
u'\u2c78': 'LatinSmallLetterEWithNotch',
u'\u40fb': 'CjkUnifiedIdeograph40fb',
u'\ua27c': 'YiSyllableHop',
u'\uc6ff': 'HangulSyllableWec',
u'\u657e': 'CjkUnifiedIdeograph657e',
u'\ufc03': 'ArabicLigatureYehWithHamzaAboveWithAlefMaksuraIsolatedForm',
u'\u1b82': 'SundaneseSignPangwisad',
u'\u9186': 'CjkUnifiedIdeograph9186',
u'\u3509': 'CjkUnifiedIdeograph3509',
u'\u5488': 'CjkUnifiedIdeograph5488',
u'\uab0d': 'UnicodeCharacter0xab0d',
u'\uca8c': 'HangulSyllableJjyeol',
u'\u6e0f': 'CjkUnifiedIdeograph6e0f',
u'\ue413': 'UnicodeCharacter0xe413',
u'\u0392': 'GreekCapitalLetterBeta',
u'\u9a17': 'CjkUnifiedIdeograph9a17',
u'\ub996': 'HangulSyllableRyinh',
u'\u5d19': 'CjkUnifiedIdeograph5d19',
u'\u7c98': 'CjkUnifiedIdeograph7c98',
u'\ud31d': 'HangulSyllablePab',
u'\uf29c': 'UnicodeCharacter0xf29c',
u'\u0c23': 'TeluguLetterNna',
u'\u2ba2': 'UnicodeCharacter0x2ba2',
u'\u8227': 'CjkUnifiedIdeograph8227',
u'\ua1a6': 'YiSyllableHlo',
u'\u4529': 'CjkUnifiedIdeograph4529',
u'\u64a8': 'CjkUnifiedIdeograph64a8',
u'\ufb2d': 'HebrewLetterShinWithDageshAndSinDot',
u'\u1aac': 'UnicodeCharacter0x1aac',
u'\u90b0': 'CjkUnifiedIdeograph90b0',
u'\u3433': 'CjkUnifiedIdeograph3433',
u'\u53b2': 'CjkUnifiedIdeograph53b2',
u'\uaa37': 'UnicodeCharacter0xaa37',
u'\uc9b6': 'HangulSyllableJyibs',
u'\u6d39': 'CjkUnifiedIdeograph6d39',
u'\ue33d': 'UnicodeCharacter0xe33d',
u'\u02bc': 'ModifierLetterApostrophe',
u'\u9941': 'CjkUnifiedIdeograph9941',
u'\ub8c0': 'HangulSyllableRoem',
u'\u5c43': 'CjkUnifiedIdeograph5c43',
u'\u7bc2': 'CjkUnifiedIdeograph7bc2',
u'\ud247': 'HangulSyllableTuh',
u'\uf1c6': 'UnicodeCharacter0xf1c6',
u'\u0b4d': 'OriyaSignVirama',
u'\u2acc': 'SupersetOfAboveNotEqualTo',
u'\u8151': 'CjkUnifiedIdeograph8151',
u'\ua0d0': 'YiSyllableFip',
u'\u4453': 'CjkUnifiedIdeograph4453',
u'\u63d2': 'CjkUnifiedIdeograph63d2',
u'\ufa57': 'CjkCompatibilityIdeographfa57',
u'\u19d6': 'NewTaiLueDigitSix',
u'\u9fda': 'UnicodeCharacter0x9fda',
u'\u335d': 'IdeographicTelegraphSymbolForHourFive',
u'\u52dc': 'CjkUnifiedIdeograph52dc',
u'\ua961': 'UnicodeCharacter0xa961',
u'\uc8e0': 'HangulSyllableJyo',
u'\u6c63': 'CjkUnifiedIdeograph6c63',
u'\ue267': 'UnicodeCharacter0xe267',
u'\u01e6': 'LatinCapitalLetterGWithCaron',
u'\u986b': 'CjkUnifiedIdeograph986b',
u'\u87ea': 'CjkUnifiedIdeograph87ea',
u'\u5b6d': 'CjkUnifiedIdeograph5b6d',
u'\u7aec': 'CjkUnifiedIdeograph7aec',
u'\ud171': 'HangulSyllableTyeolg',
u'\uf0f0': 'UnicodeCharacter0xf0f0',
u'\u0a77': 'UnicodeCharacter0xa77',
u'\u29f6': 'SolidusWithOverbar',
u'\u807b': 'CjkUnifiedIdeograph807b',
u'\uaffa': 'HangulSyllableGgwelm',
u'\u437d': 'CjkUnifiedIdeograph437d',
u'\u62fc': 'CjkUnifiedIdeograph62fc',
u'\u7881': 'CjkUnifiedIdeograph7881',
u'\u1900': 'LimbuVowelcarrierLetter',
u'\u3d83': 'CjkUnifiedIdeograph3d83',
u'\uda02': 'UnicodeCharacter0xda02',
u'\ufe85': 'ArabicLetterWawWithHamzaAboveIsolatedForm',
u'\u9f04': 'CjkUnifiedIdeograph9f04',
u'\ub387': 'HangulSyllableDec',
u'\u4c0e': 'CjkUnifiedIdeograph4c0e',
u'\u6091': 'CjkUnifiedIdeograph6091',
u'\u0110': 'LatinCapitalLetterDWithStroke',
u'\u2593': 'DarkShade',
u'\uc212': 'HangulSyllableSyoj',
u'\ue695': 'UnicodeCharacter0xe695',
u'\u8714': 'CjkUnifiedIdeograph8714',
u'\udb97': 'UnicodeCharacter0xdb97',
u'\u9c99': 'CjkUnifiedIdeograph9c99',
u'\u741e': 'CjkUnifiedIdeograph741e',
u'\u2920': 'RightwardsArrowFromBarToBlackDiamond',
u'\u4da3': 'CjkUnifiedIdeograph4da3',
u'\uea22': 'UnicodeCharacter0xea22',
u'\u0ea5': 'LaoLetterLoLoot',
u'\uaf24': 'HangulSyllableGgyess',
u'\uc3a7': 'HangulSyllableSsyeod',
u'\u84a9': 'CjkUnifiedIdeograph84a9',
u'\u5130': 'CjkUnifiedIdeograph5130',
u'\u75b3': 'CjkUnifiedIdeograph75b3',
u'\u1232': 'EthiopicSyllableSi',
u'\u36b5': 'CjkUnifiedIdeograph36b5',
u'\ud734': 'HangulSyllableHyu',
u'\uebb7': 'UnicodeCharacter0xebb7',
u'\u8836': 'CjkUnifiedIdeograph8836',
u'\uacb9': 'HangulSyllableGyeob',
u'\u7940': 'CjkUnifiedIdeograph7940',
u'\u3a42': 'CjkUnifiedIdeograph3a42',
u'\u5ec5': 'CjkUnifiedIdeograph5ec5',
u'\uff44': 'FullwidthLatinSmallLetterD',
u'\u13c7': 'CherokeeLetterQue',
u'\ub046': 'HangulSyllableGgeugg',
u'\ud4c9': 'HangulSyllablePwet',
u'\u89cb': 'CjkUnifiedIdeograph89cb',
u'\u6150': 'CjkUnifiedIdeograph6150',
u'\u2252': 'ApproximatelyEqualToOrTheImageOf',
u'\u46d5': 'CjkUnifiedIdeograph46d5',
u'\ue754': 'UnicodeCharacter0xe754',
u'\u3bd7': 'CjkUnifiedIdeograph3bd7',
u'\ud856': 'UnicodeCharacter0xd856',
u'\ufcd9': 'ArabicLigatureHehWithSuperscriptAlefInitialForm',
u'\u9d58': 'CjkUnifiedIdeograph9d58',
u'\ub1db': 'HangulSyllableNoelh',
u'\u4a62': 'CjkUnifiedIdeograph4a62',
u'\u6ee5': 'CjkUnifiedIdeograph6ee5',
u'\u0f64': 'TibetanLetterSha',
u'\u23e7': 'ElectricalIntersection',
u'\uc066': 'HangulSyllableBbeulp',
u'\ue4e9': 'UnicodeCharacter0xe4e9',
u'\u8568': 'CjkUnifiedIdeograph8568',
u'\ud9eb': 'UnicodeCharacter0xd9eb',
u'\u9aed': 'CjkUnifiedIdeograph9aed',
u'\u7272': 'CjkUnifiedIdeograph7272',
u'\u3774': 'CjkUnifiedIdeograph3774',
u'\u4bf7': 'CjkUnifiedIdeograph4bf7',
u'\ue876': 'UnicodeCharacter0xe876',
u'\u0cf9': 'UnicodeCharacter0xcf9',
u'\uad78': 'HangulSyllableGuls',
u'\uc1fb': 'HangulSyllableSoeh',
u'\u82fd': 'CjkUnifiedIdeograph82fd',
u'\ub801': 'HangulSyllableReong',
u'\u5f84': 'CjkUnifiedIdeograph5f84',
u'\u1086': 'MyanmarVowelSignShanFinalY',
u'\ud588': 'HangulSyllableHaess',
u'\u690b': 'CjkUnifiedIdeograph690b',
u'\u968a': 'CjkUnifiedIdeograph968a',
u'\u2a0d': 'FinitePartIntegral',
u'\uef0f': 'UnicodeCharacter0xef0f',
u'\ua011': 'YiSyllableO',
u'\u4794': 'CjkUnifiedIdeograph4794',
u'\u3896': 'CjkUnifiedIdeograph3896',
u'\ufd98': 'ArabicLigatureNoonWithJeemWithMeemInitialForm',
u'\ube9a': 'HangulSyllableBbyagg',
u'\u521d': 'CjkUnifiedIdeograph521d',
u'\u171f': 'UnicodeCharacter0x171f',
u'\uc821': 'HangulSyllableJenj',
u'\u8d23': 'CjkUnifiedIdeograph8d23',
u'\u6fa4': 'CjkUnifiedIdeograph6fa4',
u'\u20a6': 'NairaSign',
u'\ue5a8': 'UnicodeCharacter0xe5a8',
u'\ua6aa': 'UnicodeCharacter0xa6aa',
u'\u7a2d': 'CjkUnifiedIdeograph7a2d',
u'\u9bac': 'CjkUnifiedIdeograph9bac',
u'\u3f2f': 'CjkUnifiedIdeograph3f2f',
u'\uf031': 'UnicodeCharacter0xf031',
u'\ub533': 'HangulSyllableDdags',
u'\u48b6': 'CjkUnifiedIdeograph48b6',
u'\u0db8': 'SinhalaLetterMayanna',
u'\uceba': 'HangulSyllableKyalp',
u'\u623d': 'CjkUnifiedIdeograph623d',
u'\u83bc': 'CjkUnifiedIdeograph83bc',
u'\u273f': 'BlackFlorette',
u'\u1841': 'MongolianLetterZhi',
u'\udd43': 'UnicodeCharacter0xdd43',
u'\u9e45': 'CjkUnifiedIdeograph9e45',
u'\u70c6': 'CjkUnifiedIdeograph70c6',
u'\u35c8': 'CjkUnifiedIdeograph35c8',
u'\uf6ca': 'UnicodeCharacter0xf6ca',
u'\uabcc': 'UnicodeCharacter0xabcc',
u'\u4f4f': 'CjkUnifiedIdeograph4f4f',
u'\uc553': 'HangulSyllableAlh',
u'\u8655': 'CjkUnifiedIdeograph8655',
u'\u5dd8': 'CjkUnifiedIdeograph5dd8',
u'\u1eda': 'LatinCapitalLetterOWithHornAndAcute',
u'\ud3dc': 'HangulSyllablePyels',
u'\u775f': 'CjkUnifiedIdeograph775f',
u'\u94de': 'CjkUnifiedIdeograph94de',
u'\u2861': 'BraillePatternDots167',
u'\ued63': 'UnicodeCharacter0xed63',
u'\uae65': 'HangulSyllableGgat',
u'\u45e8': 'CjkUnifiedIdeograph45e8',
u'\u06ea': 'ArabicEmptyCentreLowStop',
u'\ufbec': 'ArabicLigatureYehWithHamzaAboveWithAeIsolatedForm',
u'\ubcee': 'HangulSyllableByej',
u'\u5071': 'CjkUnifiedIdeograph5071',
u'\u1573': 'CanadianSyllabicsTya',
u'\ud675': 'HangulSyllableHwaenj',
u'\u8b77': 'CjkUnifiedIdeograph8b77',
u'\u6df8': 'CjkUnifiedIdeograph6df8',
u'\u2efa': 'UnicodeCharacter0x2efa',
u'\ue3fc': 'UnicodeCharacter0xe3fc',
u'\ua4fe': 'UnicodeCharacter0xa4fe',
u'\u9a00': 'CjkUnifiedIdeograph9a00',
u'\ube83': 'HangulSyllableBbaed',
u'\u5902': 'CjkUnifiedIdeograph5902',
u'\u7d85': 'CjkUnifiedIdeograph7d85',
u'\ud306': 'HangulSyllableTij',
u'\uf789': 'UnicodeCharacter0xf789',
u'\u080c': 'UnicodeCharacter0x80c',
u'\u2c8f': 'CopticSmallLetterHate',
u'\u8210': 'CjkUnifiedIdeograph8210',
u'\ua693': 'CyrillicSmallLetterTche',
u'\u4112': 'CjkUnifiedIdeograph4112',
u'\u6595': 'CjkUnifiedIdeograph6595',
u'\ufb16': 'ArmenianSmallLigatureVewNow',
u'\u1f99': 'GreekCapitalLetterEtaWithDasiaAndProsgegrammeni',
u'\u919d': 'CjkUnifiedIdeograph919d',
u'\u301c': 'WaveDash',
u'\u549f': 'CjkUnifiedIdeograph549f',
u'\uaa20': 'ChamLetterMa',
u'\ucea3': 'HangulSyllableKaes',
u'\u6922': 'CjkUnifiedIdeograph6922',
u'\ue326': 'UnicodeCharacter0xe326',
u'\u07a9': 'ThaanaEebeefili',
u'\ub9ad': 'HangulSyllableRig',
u'\u582c': 'CjkUnifiedIdeograph582c',
u'\u7caf': 'CjkUnifiedIdeograph7caf',
u'\ud230': 'HangulSyllableTun',
u'\uf6b3': 'UnicodeCharacter0xf6b3',
u'\u0b36': 'OriyaLetterSha',
u'\u2fb9': 'KangxiRadicalFragrant',
u'\u8d3a': 'CjkUnifiedIdeograph8d3a',
u'\ua1bd': 'YiSyllableLie',
u'\u403c': 'CjkUnifiedIdeograph403c',
u'\u64bf': 'CjkUnifiedIdeograph64bf',
u'\ufa40': 'CjkCompatibilityIdeographfa40',
u'\u1ec3': 'LatinSmallLetterEWithCircumflexAndHookAbove',
u'\u90c7': 'CjkUnifiedIdeograph90c7',
u'\u3346': 'SquareMaruku',
u'\u57c9': 'CjkUnifiedIdeograph57c9',
u'\ub54a': 'HangulSyllableDdap',
u'\uc9cd': 'HangulSyllableJilt',
u'\u684c': 'CjkUnifiedIdeograph684c',
u'\ue250': 'UnicodeCharacter0xe250',
u'\u06d3': 'ArabicLetterYehBarreeWithHamzaAbove',
u'\ub8d7': 'HangulSyllableRyolb',
u'\u5b56': 'CjkUnifiedIdeograph5b56',
u'\u7fd9': 'CjkUnifiedIdeograph7fd9',
u'\udd5a': 'UnicodeCharacter0xdd5a',
u'\uf1dd': 'UnicodeCharacter0xf1dd',
u'\u0a60': 'UnicodeCharacter0xa60',
u'\u2ee3': 'CjkRadicalBone',
u'\u8c64': 'CjkUnifiedIdeograph8c64',
u'\ua0e7': 'YiSyllableViex',
u'\u4366': 'CjkUnifiedIdeograph4366',
u'\u67e9': 'CjkUnifiedIdeograph67e9',
u'\uc56a': 'HangulSyllableAelm',
u'\u19ed': 'KhmerSymbolDapbeiKoet',
u'\u93f1': 'CjkUnifiedIdeograph93f1',
u'\u3270': 'CircledHangulTikeutA',
u'\u56f3': 'CjkUnifiedIdeograph56f3',
u'\ub474': 'HangulSyllableDweol',
u'\uc8f7': 'HangulSyllableJyoc',
u'\u6b76': 'CjkUnifiedIdeograph6b76',
u'\ued7a': 'UnicodeCharacter0xed7a',
u'\u01fd': 'LatinSmallLetterAeWithAcute',
u'\u3b01': 'CjkUnifiedIdeograph3b01',
u'\u5a80': 'CjkUnifiedIdeograph5a80',
u'\ubd05': 'HangulSyllableBob',
u'\udc84': 'UnicodeCharacter0xdc84',
u'\u7007': 'CjkUnifiedIdeograph7007',
u'\uea0b': 'UnicodeCharacter0xea0b',
u'\u158a': 'CanadianSyllabicsWestcreeRe',
u'\u8f8e': 'CjkUnifiedIdeograph8f8e',
u'\u2311': 'SquareLozenge',
u'\u4290': 'CjkUnifiedIdeograph4290',
u'\ua515': 'VaiSyllableNdee',
u'\uc494': 'HangulSyllableSsweoss',
u'\u121b': 'EthiopicSyllableMaa',
u'\u3d9a': 'CjkUnifiedIdeograph3d9a',
u'\u941f': 'CjkUnifiedIdeograph941f',
u'\ub79e': 'HangulSyllableRaenh',
u'\u4b21': 'CjkUnifiedIdeograph4b21',
u'\u6aa0': 'CjkUnifiedIdeograph6aa0',
u'\ucd25': 'HangulSyllableCwag',
u'\ueca4': 'UnicodeCharacter0xeca4',
u'\u3a2b': 'CjkUnifiedIdeograph3a2b',
u'\u25aa': 'BlackSmallSquare',
u'\ubc2f': 'HangulSyllableBah',
u'\udfae': 'UnicodeCharacter0xdfae',
u'\u7331': 'CjkUnifiedIdeograph7331',
u'\uf535': 'UnicodeCharacter0xf535',
u'\u14b4': 'CanadianSyllabicsMwoo',
u'\u8eb8': 'CjkUnifiedIdeograph8eb8',
u'\u223b': 'Homothetic',
u'\u4dba': 'UnicodeCharacter0x4dba',
u'\ua43f': 'YiSyllableNjux',
u'\uc7be': 'HangulSyllableJaebs',
u'\u1d45': 'ModifierLetterSmallAlpha',
u'\u3cc4': 'CjkUnifiedIdeograph3cc4',
u'\u9749': 'CjkUnifiedIdeograph9749',
u'\ub6c8': 'HangulSyllableDdweom',
u'\u4a4b': 'CjkUnifiedIdeograph4a4b',
u'\u75ca': 'CjkUnifiedIdeograph75ca',
u'\ucc4f': 'HangulSyllableCaelb',
u'\uefce': 'UnicodeCharacter0xefce',
u'\u0555': 'ArmenianCapitalLetterOh',
u'\u24d4': 'CircledLatinSmallLetterE',
u'\ubf59': 'HangulSyllableBbot',
u'\uded8': 'UnicodeCharacter0xded8',
u'\u725b': 'CjkUnifiedIdeograph725b',
u'\uf45f': 'UnicodeCharacter0xf45f',
u'\u17de': 'UnicodeCharacter0x17de',
u'\u89e2': 'CjkUnifiedIdeograph89e2',
u'\u2d65': 'TifinaghLetterYazz',
u'\u4ce4': 'CjkUnifiedIdeograph4ce4',
u'\ua769': 'LatinSmallLetterVend',
u'\uc6e8': 'HangulSyllableWe',
u'\u1c6f': 'OlChikiLetterEp',
u'\u3fee': 'CjkUnifiedIdeograph3fee',
u'\u9673': 'CjkUnifiedIdeograph9673',
u'\ub1f2': 'HangulSyllableNyolm',
u'\u5575': 'CjkUnifiedIdeograph5575',
u'\u74f4': 'CjkUnifiedIdeograph74f4',
u'\ucf79': 'HangulSyllableKwalg',
u'\ueef8': 'UnicodeCharacter0xeef8',
u'\u047f': 'CyrillicSmallLetterOt',
u'\u27fe': 'LongRightwardsDoubleArrowFromBar',
u'\u5206': 'CjkUnifiedIdeograph5206',
u'\u7689': 'CjkUnifiedIdeograph7689',
u'\u1708': 'TagalogLetterNa',
u'\u2b8b': 'UnicodeCharacter0x2b8b',
u'\ud40a': 'HangulSyllablePwagg',
u'\ue88d': 'UnicodeCharacter0xe88d',
u'\u890c': 'CjkUnifiedIdeograph890c',
u'\uad8f': 'HangulSyllableGweod',
u'\u7a16': 'CjkUnifiedIdeograph7a16',
u'\u3f18': 'CjkUnifiedIdeograph3f18',
u'\u539b': 'CjkUnifiedIdeograph539b',
u'\ufc1a': 'ArabicLigatureKhahWithHahIsolatedForm',
u'\u109d': 'UnicodeCharacter0x109d',
u'\ub11c': 'HangulSyllableNeoss',
u'\ud59f': 'HangulSyllableHyalh',
u'\u8aa1': 'CjkUnifiedIdeograph8aa1',
u'\u6226': 'CjkUnifiedIdeograph6226',
u'\u2728': 'UnicodeCharacter0x2728',
u'\u7bab': 'CjkUnifiedIdeograph7bab',
u'\ue42a': 'UnicodeCharacter0xe42a',
u'\u38ad': 'CjkUnifiedIdeograph38ad',
u'\ud92c': 'UnicodeCharacter0xd92c',
u'\ufdaf': 'ArabicLigatureYehWithJeemWithYehFinalForm',
u'\u9e2e': 'CjkUnifiedIdeograph9e2e',
u'\ub2b1': 'HangulSyllableNyinj',
u'\u4f38': 'CjkUnifiedIdeograph4f38',
u'\u63bb': 'CjkUnifiedIdeograph63bb',
u'\u0c3a': 'UnicodeCharacter0xc3a',
u'\u20bd': 'UnicodeCharacter0x20bd',
u'\uc13c': 'HangulSyllableSen',
u'\ue5bf': 'UnicodeCharacter0xe5bf',
u'\u863e': 'CjkUnifiedIdeograph863e',
u'\udac1': 'UnicodeCharacter0xdac1',
u'\u9fc3': 'UnicodeCharacter0x9fc3',
u'\u7748': 'CjkUnifiedIdeograph7748',
u'\u344a': 'CjkUnifiedIdeograph344a',
u'\u48cd': 'CjkUnifiedIdeograph48cd',
u'\ue94c': 'UnicodeCharacter0xe94c',
u'\u0dcf': 'SinhalaVowelSignAelapilla',
u'\uae4e': 'HangulSyllableGgagg',
u'\uc2d1': 'HangulSyllableSyib',
u'\u87d3': 'CjkUnifiedIdeograph87d3',
u'\u5c5a': 'CjkUnifiedIdeograph5c5a',
u'\u70dd': 'CjkUnifiedIdeograph70dd',
u'\u115c': 'UnicodeCharacter0x115c',
u'\u35df': 'CjkUnifiedIdeograph35df',
u'\ud65e': 'HangulSyllableHwalm',
u'\ueae1': 'UnicodeCharacter0xeae1',
u'\u8b60': 'CjkUnifiedIdeograph8b60',
u'\uafe3': 'HangulSyllableGgweolh',
u'\u446a': 'CjkUnifiedIdeograph446a',
u'\u396c': 'CjkUnifiedIdeograph396c',
u'\u5def': 'CjkUnifiedIdeograph5def',
u'\ufe6e': 'UnicodeCharacter0xfe6e',
u'\u12f1': 'EthiopicSyllableDu',
u'\ub370': 'HangulSyllableDe',
u'\ud7f3': 'UnicodeCharacter0xd7f3',
u'\u94f5': 'CjkUnifiedIdeograph94f5',
u'\u6c7a': 'CjkUnifiedIdeograph6c7a',
u'\u217c': 'SmallRomanNumeralFifty',
u'\u45ff': 'CjkUnifiedIdeograph45ff',
u'\ue67e': 'UnicodeCharacter0xe67e',
u'\udb80': 'UnicodeCharacter0xdb80',
u'\u7f03': 'CjkUnifiedIdeograph7f03',
u'\u9882': 'CjkUnifiedIdeograph9882',
u'\u3c05': 'CjkUnifiedIdeograph3c05',
u'\uf107': 'UnicodeCharacter0xf107',
u'\ub609': 'HangulSyllableDdyeng',
u'\u498c': 'CjkUnifiedIdeograph498c',
u'\u0e8e': 'UnicodeCharacter0xe8e',
u'\uc390': 'HangulSyllableSsels',
u'\u6713': 'CjkUnifiedIdeograph6713',
u'\u8092': 'CjkUnifiedIdeograph8092',
u'\u2415': 'SymbolForNegativeAcknowledge',
u'\u1917': 'LimbuLetterLa',
u'\ude19': 'UnicodeCharacter0xde19',
u'\u931b': 'CjkUnifiedIdeograph931b',
u'\u719c': 'CjkUnifiedIdeograph719c',
u'\u369e': 'CjkUnifiedIdeograph369e',
u'\ueba0': 'UnicodeCharacter0xeba0',
u'\ua8a2': 'SaurashtraLetterTha',
u'\u4c25': 'CjkUnifiedIdeograph4c25',
u'\u0127': 'LatinSmallLetterHWithStroke',
u'\uc629': 'HangulSyllableOnj',
u'\ubb2b': 'HangulSyllableMyos',
u'\u5eae': 'CjkUnifiedIdeograph5eae',
u'\u13b0': 'CherokeeLetterHo',
u'\ud0b2': 'HangulSyllableKilp',
u'\u7435': 'CjkUnifiedIdeograph7435',
u'\u95b4': 'CjkUnifiedIdeograph95b4',
u'\u2937': 'ArrowPointingDownwardsThenCurvingRightwards',
u'\uee39': 'UnicodeCharacter0xee39',
u'\ua33b': 'YiSyllableSsep',
u'\u46be': 'CjkUnifiedIdeograph46be',
u'\u3bc0': 'CjkUnifiedIdeograph3bc0',
u'\uf8c2': 'UnicodeCharacter0xf8c2',
u'\ubdc4': 'HangulSyllableBwels',
u'\u5147': 'CjkUnifiedIdeograph5147',
u'\u1649': 'CanadianSyllabicsCarrierDzo',
u'\ucb4b': 'HangulSyllableJjugs',
u'\u884d': 'CjkUnifiedIdeograph884d',
u'\u6ece': 'CjkUnifiedIdeograph6ece',
u'\u23d0': 'VerticalLineExtension',
u'\ue0d2': 'UnicodeCharacter0xe0d2',
u'\ua5d4': 'VaiSyllableSho',
u'\u7957': 'CjkUnifiedIdeograph7957',
u'\u9ad6': 'CjkUnifiedIdeograph9ad6',
u'\u3e59': 'CjkUnifiedIdeograph3e59',
u'\uf35b': 'UnicodeCharacter0xf35b',
u'\ub05d': 'HangulSyllableGgeut',
u'\u4be0': 'CjkUnifiedIdeograph4be0',
u'\u08e2': 'UnicodeCharacter0x8e2',
u'\ucde4': 'HangulSyllableCwek',
u'\u6167': 'CjkUnifiedIdeograph6167',
u'\u82e6': 'CjkUnifiedIdeograph82e6',
u'\u2669': 'QuarterNote',
u'\u1b6b': 'BalineseMusicalSymbolCombiningTegeh',
u'\ud86d': 'UnicodeCharacter0xd86d',
u'\u9d6f': 'CjkUnifiedIdeograph9d6f',
u'\u73f0': 'CjkUnifiedIdeograph73f0',
u'\u30f2': 'KatakanaLetterWo',
u'\uf5f4': 'UnicodeCharacter0xf5f4',
u'\uaaf6': 'UnicodeCharacter0xaaf6',
u'\u4e79': 'CjkUnifiedIdeograph4e79',
u'\u037b': 'GreekSmallReversedLunateSigmaSymbol',
u'\uc07d': 'HangulSyllableBbyilg',
u'\u857f': 'CjkUnifiedIdeograph857f',
u'\ufd81': 'ArabicLigatureLamWithHahWithYehFinalForm',
u'\u1a04': 'BugineseLetterPa',
u'\u3687': 'CjkUnifiedIdeograph3687',
u'\u9008': 'CjkUnifiedIdeograph9008',
u'\uac8b': 'HangulSyllableGeoh',
u'\u530a': 'CjkUnifiedIdeograph530a',
u'\u6f8d': 'CjkUnifiedIdeograph6f8d',
u'\uc90e': 'HangulSyllableJubs',
u'\ue591': 'UnicodeCharacter0xe591',
u'\u0214': 'LatinCapitalLetterUWithDoubleGrave',
u'\u5e97': 'CjkUnifiedIdeograph5e97',
u'\ub818': 'HangulSyllableRem',
u'\ud49b': 'HangulSyllablePweod',
u'\u7b1a': 'CjkUnifiedIdeograph7b1a',
u'\uf11e': 'UnicodeCharacter0xf11e',
u'\u0da1': 'SinhalaLetterMahaapraanaCayanna',
u'\u8ba5': 'CjkUnifiedIdeograph8ba5',
u'\u2a24': 'PlusSignWithTildeAbove',
u'\u46a7': 'CjkUnifiedIdeograph46a7',
u'\ua028': 'YiSyllableBop',
u'\ufcab': 'ArabicLigatureKhahWithJeemInitialForm',
u'\u632a': 'CjkUnifiedIdeograph632a',
u'\u192e': 'UnicodeCharacter0x192e',
u'\u35b1': 'CjkUnifiedIdeograph35b1',
u'\u9732': 'CjkUnifiedIdeograph9732',
u'\ub3b5': 'HangulSyllableDyelt',
u'\u5234': 'CjkUnifiedIdeograph5234',
u'\u6eb7': 'CjkUnifiedIdeograph6eb7',
u'\uc838': 'HangulSyllableJyeo',
u'\ue4bb': 'UnicodeCharacter0xe4bb',
u'\u013e': 'LatinSmallLetterLWithCaron',
u'\u5dc1': 'CjkUnifiedIdeograph5dc1',
u'\ubf42': 'HangulSyllableBbogg',
u'\udbc5': 'UnicodeCharacter0xdbc5',
u'\u7a44': 'CjkUnifiedIdeograph7a44',
u'\uf048': 'UnicodeCharacter0xf048',
u'\u0ccb': 'KannadaVowelSignOo',
u'\u8acf': 'CjkUnifiedIdeograph8acf',
u'\u294e': 'LeftBarbUpRightBarbUpHarpoon',
u'\u45d1': 'CjkUnifiedIdeograph45d1',
u'\ua752': 'LatinCapitalLetterPWithFlourish',
u'\uc3d5': 'HangulSyllableSsyet',
u'\u6254': 'CjkUnifiedIdeograph6254',
u'\u1858': 'MongolianLetterTodoGaa',
u'\u34db': 'CjkUnifiedIdeograph34db',
u'\u965c': 'CjkUnifiedIdeograph965c',
u'\ub2df': 'HangulSyllableNic',
u'\u515e': 'CjkUnifiedIdeograph515e',
u'\u6de1': 'CjkUnifiedIdeograph6de1',
u'\ucf62': 'HangulSyllableKolp',
u'\uebe5': 'UnicodeCharacter0xebe5',
u'\u5ceb': 'CjkUnifiedIdeograph5ceb',
u'\ube6c': 'HangulSyllableBbals',
u'\udaef': 'UnicodeCharacter0xdaef',
u'\u796e': 'CjkUnifiedIdeograph796e',
u'\uf772': 'UnicodeCharacter0xf772',
u'\u13f5': 'UnicodeCharacter0x13f5',
u'\u89f9': 'CjkUnifiedIdeograph89f9',
u'\u2878': 'BraillePatternDots4567',
u'\u44fb': 'CjkUnifiedIdeograph44fb',
u'\ua67c': 'CombiningCyrillicKavyka',
u'\uc2ff': 'HangulSyllableSsad',
u'\u617e': 'CjkUnifiedIdeograph617e',
u'\uf803': 'UnicodeCharacter0xf803',
u'\u1f82': 'GreekSmallLetterAlphaWithPsiliAndVariaAndYpogegrammeni',
u'\u9586': 'CjkUnifiedIdeograph9586',
u'\u3109': 'BopomofoLetterD',
u'\u5088': 'CjkUnifiedIdeograph5088',
u'\uaf0d': 'HangulSyllableGgyeot',
u'\uce8c': 'HangulSyllableKak',
u'\u6a0f': 'CjkUnifiedIdeograph6a0f',
u'\ue013': 'UnicodeCharacter0xe013',
u'\u0792': 'ThaanaLetterZaviyani',
u'\u9e17': 'CjkUnifiedIdeograph9e17',
u'\ubd96': 'HangulSyllableBuj',
u'\u5919': 'CjkUnifiedIdeograph5919',
u'\u7898': 'CjkUnifiedIdeograph7898',
u'\ud71d': 'HangulSyllableHwinj',
u'\uf69c': 'UnicodeCharacter0xf69c',
u'\u0823': 'UnicodeCharacter0x823',
u'\u2fa2': 'KangxiRadicalCity',
u'\u8627': 'CjkUnifiedIdeograph8627',
u'\ua5a6': 'VaiSyllableDhu',
u'\u4129': 'CjkUnifiedIdeograph4129',
u'\u60a8': 'CjkUnifiedIdeograph60a8',
u'\uff2d': 'FullwidthLatinCapitalLetterM',
u'\u1eac': 'LatinCapitalLetterAWithCircumflexAndDotBelow',
u'\u94b0': 'CjkUnifiedIdeograph94b0',
u'\u3033': 'VerticalKanaRepeatMarkUpperHalf',
u'\u57b2': 'CjkUnifiedIdeograph57b2',
u'\uae37': 'HangulSyllableGid',
u'\ucdb6': 'HangulSyllableCweonh',
u'\u6939': 'CjkUnifiedIdeograph6939',
u'\ue73d': 'UnicodeCharacter0xe73d',
u'\u06bc': 'ArabicLetterNoonWithRing',
u'\u9d41': 'CjkUnifiedIdeograph9d41',
u'\ubcc0': 'HangulSyllableByeon',
u'\u5843': 'CjkUnifiedIdeograph5843',
u'\u7fc2': 'CjkUnifiedIdeograph7fc2',
u'\ud647': 'HangulSyllableHolh',
u'\uf5c6': 'UnicodeCharacter0xf5c6',
u'\u0f4d': 'TibetanLetterDdha',
u'\u2ecc': 'CjkRadicalSimplifiedWalk',
u'\u8551': 'CjkUnifiedIdeograph8551',
u'\ua4d0': 'UnicodeCharacter0xa4d0',
u'\u4053': 'CjkUnifiedIdeograph4053',
u'\u67d2': 'CjkUnifiedIdeograph67d2',
u'\ufe57': 'SmallExclamationMark',
u'\u1dd6': 'CombiningLatinSmallLetterAv',
u'\u9bda': 'CjkUnifiedIdeograph9bda',
u'\u375d': 'CjkUnifiedIdeograph375d',
u'\u56dc': 'CjkUnifiedIdeograph56dc',
u'\uad61': 'HangulSyllableGyob',
u'\ucce0': 'HangulSyllableCyeom',
u'\u6863': 'CjkUnifiedIdeograph6863',
u'\ue667': 'UnicodeCharacter0xe667',
u'\u05e6': 'HebrewLetterTsadi',
u'\u9c6b': 'CjkUnifiedIdeograph9c6b',
u'\u83ea': 'CjkUnifiedIdeograph83ea',
u'\u5f6d': 'CjkUnifiedIdeograph5f6d',
u'\u7eec': 'CjkUnifiedIdeograph7eec',
u'\ud571': 'HangulSyllableHat',
u'\uf4f0': 'UnicodeCharacter0xf4f0',
u'\u0e77': 'UnicodeCharacter0xe77',
u'\u2df6': 'CombiningCyrillicLetterA',
u'\u847b': 'CjkUnifiedIdeograph847b',
u'\uabfa': 'UnicodeCharacter0xabfa',
u'\u477d': 'CjkUnifiedIdeograph477d',
u'\u66fc': 'CjkUnifiedIdeograph66fc',
u'\u7c81': 'CjkUnifiedIdeograph7c81',
u'\u1d00': 'LatinLetterSmallCapitalA',
u'\u3983': 'CjkUnifiedIdeograph3983',
u'\ude02': 'UnicodeCharacter0xde02',
u'\ufa85': 'CjkCompatibilityIdeographfa85',
u'\u9b04': 'CjkUnifiedIdeograph9b04',
u'\ub787': 'HangulSyllableRalb',
u'\u480e': 'CjkUnifiedIdeograph480e',
u'\u6491': 'CjkUnifiedIdeograph6491',
u'\u0510': 'CyrillicCapitalLetterReversedZe',
u'\u2193': 'DownwardsArrow',
u'\uc612': 'HangulSyllableYelm',
u'\ue295': 'UnicodeCharacter0xe295',
u'\u8314': 'CjkUnifiedIdeograph8314',
u'\udf97': 'UnicodeCharacter0xdf97',
u'\u9899': 'CjkUnifiedIdeograph9899',
u'\u701e': 'CjkUnifiedIdeograph701e',
u'\u2d20': 'GeorgianSmallLetterHae',
u'\u49a3': 'CjkUnifiedIdeograph49a3',
u'\uee22': 'UnicodeCharacter0xee22',
u'\u0aa5': 'GujaratiLetterTha',
u'\uab24': 'UnicodeCharacter0xab24',
u'\uc7a7': 'HangulSyllableJac',
u'\u80a9': 'CjkUnifiedIdeograph80a9',
u'\u5530': 'CjkUnifiedIdeograph5530',
u'\u71b3': 'CjkUnifiedIdeograph71b3',
u'\u1632': 'CanadianSyllabicsCarrierLhi',
u'\u32b5': 'CircledNumberForty',
u'\ud334': 'HangulSyllablePaels',
u'\uefb7': 'UnicodeCharacter0xefb7',
u'\u8c36': 'CjkUnifiedIdeograph8c36',
u'\ua8b9': 'SaurashtraVowelSignUu',
u'\u7d40': 'CjkUnifiedIdeograph7d40',
u'\u3e42': 'CjkUnifiedIdeograph3e42',
u'\u5ac5': 'CjkUnifiedIdeograph5ac5',
u'\ufb44': 'HebrewLetterPeWithDagesh',
u'\u17c7': 'KhmerSignReahmuk',
u'\ub446': 'HangulSyllableDyobs',
u'\ud0c9': 'HangulSyllableTalg',
u'\u8dcb': 'CjkUnifiedIdeograph8dcb',
u'\u6550': 'CjkUnifiedIdeograph6550',
u'\u2652': 'Aquarius',
u'\u42d5': 'CjkUnifiedIdeograph42d5',
u'\ue354': 'UnicodeCharacter0xe354',
u'\u3fd7': 'CjkUnifiedIdeograph3fd7',
u'\udc56': 'UnicodeCharacter0xdc56',
u'\uf8d9': 'UnicodeCharacter0xf8d9',
u'\u9958': 'CjkUnifiedIdeograph9958',
u'\ub5db': 'HangulSyllableDdyeogs',
u'\u4e62': 'CjkUnifiedIdeograph4e62',
u'\u6ae5': 'CjkUnifiedIdeograph6ae5',
u'\u0b64': 'UnicodeCharacter0xb64',
u'\u27e7': 'MathematicalRightWhiteSquareBracket',
u'\uc466': 'HangulSyllableSsugg',
u'\ue0e9': 'UnicodeCharacter0xe0e9',
u'\u8168': 'CjkUnifiedIdeograph8168',
u'\uddeb': 'UnicodeCharacter0xddeb',
u'\u9eed': 'CjkUnifiedIdeograph9eed',
u'\u7672': 'CjkUnifiedIdeograph7672',
u'\u3374': 'SquareBar',
u'\u4ff7': 'CjkUnifiedIdeograph4ff7',
u'\uec76': 'UnicodeCharacter0xec76',
u'\u08f9': 'UnicodeCharacter0x8f9',
u'\ua978': 'UnicodeCharacter0xa978',
u'\uc5fb': 'HangulSyllableYeolh',
u'\u86fd': 'CjkUnifiedIdeograph86fd',
u'\ubc01': 'HangulSyllableMilg',
u'\u5b84': 'CjkUnifiedIdeograph5b84',
u'\u1486': 'CanadianSyllabicsSouthslaveyKih',
u'\ud188': 'HangulSyllableTyen',
u'\u6d0b': 'CjkUnifiedIdeograph6d0b',
u'\u928a': 'CjkUnifiedIdeograph928a',
u'\u2e0d': 'RightRaisedOmissionBracket',
u'\ueb0f': 'UnicodeCharacter0xeb0f',
u'\ua411': 'YiSyllableQur',
u'\u4394': 'CjkUnifiedIdeograph4394',
u'\u3c96': 'CjkUnifiedIdeograph3c96',
u'\uf998': 'CjkCompatibilityIdeographf998',
u'\uba9a': 'HangulSyllableMyelp',
u'\u561d': 'CjkUnifiedIdeograph561d',
u'\u131f': 'EthiopicSyllableGgwaa',
u'\ucc21': 'HangulSyllableJjing',
u'\u8923': 'CjkUnifiedIdeograph8923',
u'\u6ba4': 'CjkUnifiedIdeograph6ba4',
u'\u24a6': 'ParenthesizedLatinSmallLetterK',
u'\ue1a8': 'UnicodeCharacter0xe1a8',
u'\ua2aa': 'YiSyllableZy',
u'\u7e2d': 'CjkUnifiedIdeograph7e2d',
u'\u9fac': 'CjkUnifiedIdeograph9fac',
u'\u3b2f': 'CjkUnifiedIdeograph3b2f',
u'\uf431': 'UnicodeCharacter0xf431',
u'\ub133': 'HangulSyllableNelh',
u'\u4cb6': 'CjkUnifiedIdeograph4cb6',
u'\u09b8': 'BengaliLetterSa',
u'\ucaba': 'HangulSyllableJjyep',
u'\u663d': 'CjkUnifiedIdeograph663d',
u'\u87bc': 'CjkUnifiedIdeograph87bc',
u'\u233f': 'AplFunctionalSymbolSlashBar',
u'\u1c41': 'LepchaDigitOne',
u'\ud943': 'UnicodeCharacter0xd943',
u'\u9a45': 'CjkUnifiedIdeograph9a45',
u'\u74c6': 'CjkUnifiedIdeograph74c6',
u'\u31c8': 'CjkStrokeHzwg',
u'\uf2ca': 'UnicodeCharacter0xf2ca',
u'\uafcc': 'HangulSyllableGguss',
u'\u4b4f': 'CjkUnifiedIdeograph4b4f',
u'\u0451': 'CyrillicSmallLetterIo',
u'\uc153': 'HangulSyllableSeh',
u'\u8255': 'CjkUnifiedIdeograph8255',
u'\u59d8': 'CjkUnifiedIdeograph59d8',
u'\u1ada': 'UnicodeCharacter0x1ada',
u'\ud7dc': 'UnicodeCharacter0xd7dc',
u'\u735f': 'CjkUnifiedIdeograph735f',
u'\u90de': 'CjkUnifiedIdeograph90de',
u'\u2c61': 'LatinSmallLetterLWithDoubleBar',
u'\ue963': 'UnicodeCharacter0xe963',
u'\uaa65': 'UnicodeCharacter0xaa65',
u'\u41e8': 'CjkUnifiedIdeograph41e8',
u'\u02ea': 'ModifierLetterYinDepartingToneMark',
u'\uffec': 'HalfwidthDownwardsArrow',
u'\ub8ee': 'HangulSyllableRunh',
u'\u5471': 'CjkUnifiedIdeograph5471',
u'\u1173': 'HangulJungseongEu',
u'\ud275': 'HangulSyllableTweb',
u'\u8f77': 'CjkUnifiedIdeograph8f77',
u'\u69f8': 'CjkUnifiedIdeograph69f8',
u'\u2afa': 'DoublelineSlantedGreaterthanOrEqualTo',
u'\ue7fc': 'UnicodeCharacter0xe7fc',
u'\ua0fe': 'YiSyllableVyrx',
u'\u9e00': 'CjkUnifiedIdeograph9e00',
u'\uba83': 'HangulSyllableMyeos',
u'\u5d02': 'CjkUnifiedIdeograph5d02',
u'\u7985': 'CjkUnifiedIdeograph7985',
u'\ud706': 'HangulSyllableHwelm',
u'\uf389': 'UnicodeCharacter0xf389',
u'\u0c0c': 'TeluguLetterVocalicL',
u'\u288f': 'BraillePatternDots12348',
u'\u8610': 'CjkUnifiedIdeograph8610',
u'\ua293': 'YiSyllableZiep',
u'\u4512': 'CjkUnifiedIdeograph4512',
u'\u6195': 'CjkUnifiedIdeograph6195',
u'\uff16': 'FullwidthDigitSix',
u'\u1b99': 'SundaneseLetterMa',
u'\u959d': 'CjkUnifiedIdeograph959d',
u'\u341c': 'CjkUnifiedIdeograph341c',
u'\u509f': 'CjkUnifiedIdeograph509f',
u'\uae20': 'HangulSyllableGyils',
u'\ucaa3': 'HangulSyllableJjyegs',
u'\u6d22': 'CjkUnifiedIdeograph6d22',
u'\ue726': 'UnicodeCharacter0xe726',
u'\u03a9': 'GreekCapitalLetterOmega',
u'\ubdad': 'HangulSyllableBweob',
u'\u5c2c': 'CjkUnifiedIdeograph5c2c',
u'\u78af': 'CjkUnifiedIdeograph78af',
u'\ud630': 'HangulSyllableHyess',
u'\uf2b3': 'UnicodeCharacter0xf2b3',
u'\u0f36': 'TibetanMarkCaretDzudRtagsBzhiMigCan',
u'\u2bb9': 'UnicodeCharacter0x2bb9',
u'\u893a': 'CjkUnifiedIdeograph893a',
u'\ua5bd': 'VaiSyllableHo',
u'\u443c': 'CjkUnifiedIdeograph443c',
u'\u60bf': 'CjkUnifiedIdeograph60bf',
u'\ufe40': 'PresentationFormForVerticalRightAngleBracket',
u'\u1ac3': 'UnicodeCharacter0x1ac3',
u'\u94c7': 'CjkUnifiedIdeograph94c7',
u'\u3746': 'CjkUnifiedIdeograph3746',
u'\u53c9': 'CjkUnifiedIdeograph53c9',
u'\ub14a': 'HangulSyllableNyeolm',
u'\ucdcd': 'HangulSyllableCweg',
u'\u6c4c': 'CjkUnifiedIdeograph6c4c',
u'\ue650': 'UnicodeCharacter0xe650',
u'\u02d3': 'ModifierLetterCentredLeftHalfRing',
u'\ubcd7': 'HangulSyllableByeoh',
u'\u5f56': 'CjkUnifiedIdeograph5f56',
u'\u7bd9': 'CjkUnifiedIdeograph7bd9',
u'\ud95a': 'UnicodeCharacter0xd95a',
u'\uf5dd': 'UnicodeCharacter0xf5dd',
u'\u0e60': 'UnicodeCharacter0xe60',
u'\u2ae3': 'DoubleVerticalBarLeftTurnstile',
u'\u8864': 'CjkUnifiedIdeograph8864',
u'\ua4e7': 'UnicodeCharacter0xa4e7',
u'\u4766': 'CjkUnifiedIdeograph4766',
u'\u63e9': 'CjkUnifiedIdeograph63e9',
u'\uc16a': 'HangulSyllableSyeoj',
u'\u1ded': 'UnicodeCharacter0x1ded',
u'\u97f1': 'CjkUnifiedIdeograph97f1',
u'\u3670': 'CjkUnifiedIdeograph3670',
u'\u52f3': 'CjkUnifiedIdeograph52f3',
u'\ub074': 'HangulSyllableGgyiss',
u'\uccf7': 'HangulSyllableCyelb',
u'\u6f76': 'CjkUnifiedIdeograph6f76',
u'\ue97a': 'UnicodeCharacter0xe97a',
u'\u05fd': 'UnicodeCharacter0x5fd',
u'\u3f01': 'CjkUnifiedIdeograph3f01',
u'\u5e80': 'CjkUnifiedIdeograph5e80',
u'\ub905': 'HangulSyllableRweog',
u'\ud884': 'UnicodeCharacter0xd884',
u'\u7407': 'CjkUnifiedIdeograph7407',
u'\uee0b': 'UnicodeCharacter0xee0b',
u'\u118a': 'HangulJungseongUae',
u'\u8b8e': 'CjkUnifiedIdeograph8b8e',
u'\u2711': 'WhiteNib',
u'\u4690': 'CjkUnifiedIdeograph4690',
u'\ua115': 'YiSyllableDux',
u'\uc094': 'HangulSyllableBbin',
u'\u161b': 'CanadianSyllabicsCarrierJa',
u'\u399a': 'CjkUnifiedIdeograph399a',
u'\u901f': 'CjkUnifiedIdeograph901f',
u'\ub39e': 'HangulSyllableDyeobs',
u'\u4f21': 'CjkUnifiedIdeograph4f21',
u'\u6ea0': 'CjkUnifiedIdeograph6ea0',
u'\uc925': 'HangulSyllableJweolt',
u'\ue8a4': 'UnicodeCharacter0xe8a4',
u'\u3e2b': 'CjkUnifiedIdeograph3e2b',
u'\u21aa': 'RightwardsArrowWithHook',
u'\ub82f': 'HangulSyllableRyeolb',
u'\udbae': 'UnicodeCharacter0xdbae',
u'\u7731': 'CjkUnifiedIdeograph7731',
u'\uf135': 'UnicodeCharacter0xf135',
u'\u10b4': 'GeorgianCapitalLetterPhar',
u'\u8ab8': 'CjkUnifiedIdeograph8ab8',
u'\u263b': 'BlackSmilingFace',
u'\u49ba': 'CjkUnifiedIdeograph49ba',
u'\ua03f': 'YiSyllablePat',
u'\uc3be': 'HangulSyllableSsyegg',
u'\u1945': 'LimbuQuestionMark',
u'\u38c4': 'CjkUnifiedIdeograph38c4',
u'\u9349': 'CjkUnifiedIdeograph9349',
u'\ub2c8': 'HangulSyllableNi',
u'\u4e4b': 'CjkUnifiedIdeograph4e4b',
u'\u71ca': 'CjkUnifiedIdeograph71ca',
u'\uc84f': 'HangulSyllableJyeoc',
u'\uebce': 'UnicodeCharacter0xebce',
u'\u0155': 'LatinSmallLetterRWithAcute',
u'\u20d4': 'CombiningAnticlockwiseArrowAbove',
u'\ubb59': 'HangulSyllableMweolg',
u'\udad8': 'UnicodeCharacter0xdad8',
u'\u765b': 'CjkUnifiedIdeograph765b',
u'\uf05f': 'UnicodeCharacter0xf05f',
u'\u13de': 'CherokeeLetterTle',
u'\u8de2': 'CjkUnifiedIdeograph8de2',
u'\u2965': 'DownwardsHarpoonWithBarbLeftBesideDownwardsHarpoonWithBarbRight',
u'\u48e4': 'CjkUnifiedIdeograph48e4',
u'\ua369': 'YiSyllableChot',
u'\uc2e8': 'HangulSyllableSils',
u'\u186f': 'MongolianLetterSibeZa',
u'\u3bee': 'CjkUnifiedIdeograph3bee',
u'\u9273': 'CjkUnifiedIdeograph9273',
u'\ub5f2': 'HangulSyllableDdyeop',
u'\u5175': 'CjkUnifiedIdeograph5175',
u'\u70f4': 'CjkUnifiedIdeograph70f4',
u'\ucb79': 'HangulSyllableJjweong',
u'\ueaf8': 'UnicodeCharacter0xeaf8',
u'\u23fe': 'UnicodeCharacter0x23fe',
u'\u5606': 'CjkUnifiedIdeograph5606',
u'\u7289': 'CjkUnifiedIdeograph7289',
u'\u1308': 'EthiopicSyllableGa',
u'\u2f8b': 'KangxiRadicalGrass',
u'\ud00a': 'HangulSyllableKweolp',
u'\uec8d': 'UnicodeCharacter0xec8d',
u'\u8d0c': 'CjkUnifiedIdeograph8d0c',
u'\ua98f': 'UnicodeCharacter0xa98f',
u'\u7e16': 'CjkUnifiedIdeograph7e16',
u'\u3b18': 'CjkUnifiedIdeograph3b18',
u'\u579b': 'CjkUnifiedIdeograph579b',
u'\uf81a': 'UnicodeCharacter0xf81a',
u'\u149d': 'CanadianSyllabicsWestcreeCwa',
u'\ub51c': 'HangulSyllableDil',
u'\ud19f': 'HangulSyllableTyeh',
u'\u8ea1': 'CjkUnifiedIdeograph8ea1',
u'\u6626': 'CjkUnifiedIdeograph6626',
u'\u2328': 'Keyboard',
u'\u7fab': 'CjkUnifiedIdeograph7fab',
u'\ue02a': 'UnicodeCharacter0xe02a',
u'\u3cad': 'CjkUnifiedIdeograph3cad',
u'\udd2c': 'UnicodeCharacter0xdd2c',
u'\uf9af': 'CjkCompatibilityIdeographf9af',
u'\u9a2e': 'CjkUnifiedIdeograph9a2e',
u'\ub6b1': 'HangulSyllableDdung',
u'\u4b38': 'CjkUnifiedIdeograph4b38',
u'\u67bb': 'CjkUnifiedIdeograph67bb',
u'\u083a': 'UnicodeCharacter0x83a',
u'\u24bd': 'CircledLatinCapitalLetterH',
u'\uc53c': 'HangulSyllableSsiss',
u'\ue1bf': 'UnicodeCharacter0xe1bf',
u'\u823e': 'CjkUnifiedIdeograph823e',
u'\udec1': 'UnicodeCharacter0xdec1',
u'\u9bc3': 'CjkUnifiedIdeograph9bc3',
u'\u7348': 'CjkUnifiedIdeograph7348',
u'\u304a': 'HiraganaLetterO',
u'\u4ccd': 'CjkUnifiedIdeograph4ccd',
u'\ued4c': 'UnicodeCharacter0xed4c',
u'\u09cf': 'UnicodeCharacter0x9cf',
u'\uaa4e': 'UnicodeCharacter0xaa4e',
u'\uc6d1': 'HangulSyllableWeonj',
u'\u83d3': 'CjkUnifiedIdeograph83d3',
u'\u585a': 'CjkUnifiedIdeograph585a',
u'\u74dd': 'CjkUnifiedIdeograph74dd',
u'\u155c': 'CanadianSyllabicsWestcreeFwaa',
u'\u31df': 'CjkStrokeSwg',
u'\ud25e': 'HangulSyllableTweoj',
u'\ueee1': 'UnicodeCharacter0xeee1',
u'\u8f60': 'CjkUnifiedIdeograph8f60',
u'\uabe3': 'UnicodeCharacter0xabe3',
u'\u406a': 'CjkUnifiedIdeograph406a',
u'\u3d6c': 'CjkUnifiedIdeograph3d6c',
u'\u59ef': 'CjkUnifiedIdeograph59ef',
u'\ufa6e': 'UnicodeCharacter0xfa6e',
u'\u16f1': 'UnicodeCharacter0x16f1',
u'\ub770': 'HangulSyllableDdim',
u'\ud3f3': 'HangulSyllablePod',
u'\u90f5': 'CjkUnifiedIdeograph90f5',
u'\u687a': 'CjkUnifiedIdeograph687a',
u'\u257c': 'BoxDrawingsLightLeftAndHeavyRight',
u'\u41ff': 'CjkUnifiedIdeograph41ff',
u'\ue27e': 'UnicodeCharacter0xe27e',
u'\udf80': 'UnicodeCharacter0xdf80',
u'\u7b03': 'CjkUnifiedIdeograph7b03',
u'\u9c82': 'CjkUnifiedIdeograph9c82',
u'\u3805': 'CjkUnifiedIdeograph3805',
u'\uf507': 'UnicodeCharacter0xf507',
u'\ub209': 'HangulSyllableNunj',
u'\u4d8c': 'CjkUnifiedIdeograph4d8c',
u'\u0a8e': 'UnicodeCharacter0xa8e',
u'\uc790': 'HangulSyllableJa',
u'\u6313': 'CjkUnifiedIdeograph6313',
u'\u8492': 'CjkUnifiedIdeograph8492',
u'\u2015': 'HorizontalBar',
u'\u1d17': 'LatinSmallLetterBottomHalfO',
u'\uda19': 'UnicodeCharacter0xda19',
u'\u971b': 'CjkUnifiedIdeograph971b',
u'\u759c': 'CjkUnifiedIdeograph759c',
u'\u329e': 'CircledIdeographPrint',
u'\uefa0': 'UnicodeCharacter0xefa0',
u'\uaca2': 'HangulSyllableGej',
u'\u4825': 'CjkUnifiedIdeograph4825',
u'\u0527': 'UnicodeCharacter0x527',
u'\uc229': 'HangulSyllableSub',
u'\ubf2b': 'HangulSyllableBbyed',
u'\u5aae': 'CjkUnifiedIdeograph5aae',
u'\u17b0': 'KhmerIndependentVowelQai',
u'\ud4b2': 'HangulSyllablePwegg',
u'\u7035': 'CjkUnifiedIdeograph7035',
u'\u91b4': 'CjkUnifiedIdeograph91b4',
u'\u2d37': 'TifinaghLetterYad',
u'\uea39': 'UnicodeCharacter0xea39',
u'\ua73b': 'LatinSmallLetterAvWithHorizontalBar',
u'\u42be': 'CjkUnifiedIdeograph42be',
u'\u3fc0': 'CjkUnifiedIdeograph3fc0',
u'\ufcc2': 'ArabicLigatureQafWithHahInitialForm',
u'\ub9c4': 'HangulSyllableRik',
u'\u5547': 'CjkUnifiedIdeograph5547',
u'\u1249': 'UnicodeCharacter0x1249',
u'\ucf4b': 'HangulSyllableKyes',
u'\u8c4d': 'CjkUnifiedIdeograph8c4d',
u'\u6ace': 'CjkUnifiedIdeograph6ace',
u'\u27d0': 'WhiteDiamondWithCentredDot',
u'\ue4d2': 'UnicodeCharacter0xe4d2',
u'\ua1d4': 'YiSyllableLyt',
u'\u7d57': 'CjkUnifiedIdeograph7d57',
u'\u9ed6': 'CjkUnifiedIdeograph9ed6',
u'\u3a59': 'CjkUnifiedIdeograph3a59',
u'\uf75b': 'UnicodeCharacter0xf75b',
u'\ub45d': 'HangulSyllableDult',
u'\u4fe0': 'CjkUnifiedIdeograph4fe0',
u'\u0ce2': 'KannadaVowelSignVocalicL',
u'\uc9e4': 'HangulSyllableJjal',
u'\u6567': 'CjkUnifiedIdeograph6567',
u'\u86e6': 'CjkUnifiedIdeograph86e6',
u'\u2269': 'GreaterthanButNotEqualTo',
u'\u1f6b': 'GreekCapitalLetterOmegaWithDasiaAndVaria',
u'\udc6d': 'UnicodeCharacter0xdc6d',
u'\u996f': 'CjkUnifiedIdeograph996f',
u'\u77f0': 'CjkUnifiedIdeograph77f0',
u'\u34f2': 'CjkUnifiedIdeograph34f2',
u'\uf1f4': 'UnicodeCharacter0xf1f4',
u'\uaef6': 'HangulSyllableGgyeogg',
u'\u4a79': 'CjkUnifiedIdeograph4a79',
u'\u077b': 'ArabicLetterYehBarreeWithExtendedArabicindicDigitThreeAbove',
u'\uc47d': 'HangulSyllableSsut',
u'\u817f': 'CjkUnifiedIdeograph817f',
u'\uc181': 'HangulSyllableSyeb',
u'\u2604': 'Comet',
u'\u0a87': 'GujaratiLetterI',
u'\uac08': 'HangulSyllableGal',
u'\u908b': 'CjkUnifiedIdeograph908b',
u'\u6f0a': 'CjkUnifiedIdeograph6f0a',
u'\u538d': 'CjkUnifiedIdeograph538d',
u'\uf50e': 'UnicodeCharacter0xf50e',
u'\ud991': 'UnicodeCharacter0xd991',
u'\u3e14': 'CjkUnifiedIdeograph3e14',
u'\u6297': 'CjkUnifiedIdeograph6297',
u'\u8418': 'CjkUnifiedIdeograph8418',
u'\ue89b': 'UnicodeCharacter0xe89b',
u'\u471a': 'CjkUnifiedIdeograph471a',
u'\ucd1e': 'HangulSyllableCoj',
u'\u31a1': 'BopomofoLetterZi',
u'\ub7a5': 'HangulSyllableRaelt',
u'\u1624': 'CanadianSyllabicsCarrierLe',
u'\u7aa7': 'CjkUnifiedIdeograph7aa7',
u'\u9c28': 'CjkUnifiedIdeograph9c28',
u'\uc0ab': 'HangulSyllableBbih',
u'\u5f2a': 'CjkUnifiedIdeograph5f2a',
u'\u252e': 'BoxDrawingsRightHeavyAndLeftDownLight',
u'\u09b1': 'UnicodeCharacter0x9b1',
u'\uab32': 'UnicodeCharacter0xab32',
u'\u8fb5': 'CjkUnifiedIdeograph8fb5',
u'\u6e34': 'CjkUnifiedIdeograph6e34',
u'\u52b7': 'CjkUnifiedIdeograph52b7',
u'\uf438': 'UnicodeCharacter0xf438',
u'\ud8bb': 'UnicodeCharacter0xd8bb',
u'\u3d3e': 'CjkUnifiedIdeograph3d3e',
u'\u61c1': 'CjkUnifiedIdeograph61c1',
u'\u8342': 'CjkUnifiedIdeograph8342',
u'\ue7c5': 'UnicodeCharacter0xe7c5',
u'\u4644': 'CjkUnifiedIdeograph4644',
u'\ucc48': 'HangulSyllableCaen',
u'\u30cb': 'KatakanaLetterNi',
u'\ub6cf': 'HangulSyllableDdweoc',
u'\u154e': 'CanadianSyllabicsRwaa',
u'\u79d1': 'CjkUnifiedIdeograph79d1',
u'\u9b52': 'CjkUnifiedIdeograph9b52',
u'\uffd5': 'HalfwidthHangulLetterWe',
u'\u5e54': 'CjkUnifiedIdeograph5e54',
u'\u2458': 'UnicodeCharacter0x2458',
u'\u08db': 'UnicodeCharacter0x8db',
u'\uaa5c': 'ChamPunctuationSpiral',
u'\u8edf': 'CjkUnifiedIdeograph8edf',
u'\u6d5e': 'CjkUnifiedIdeograph6d5e',
u'\u51e1': 'CjkUnifiedIdeograph51e1',
u'\uf362': 'UnicodeCharacter0xf362',
u'\ud7e5': 'UnicodeCharacter0xd7e5',
u'\u3c68': 'CjkUnifiedIdeograph3c68',
u'\u60eb': 'CjkUnifiedIdeograph60eb',
u'\u826c': 'CjkUnifiedIdeograph826c',
u'\ue6ef': 'UnicodeCharacter0xe6ef',
u'\u456e': 'CjkUnifiedIdeograph456e',
u'\ucb72': 'HangulSyllableJjweolp',
u'\u2ff5': 'IdeographicDescriptionCharacterSurroundFromAbove',
u'\ub5f9': 'HangulSyllableDdyenj',
u'\u1478': 'CanadianSyllabicsKwii',
u'\u78fb': 'CjkUnifiedIdeograph78fb',
u'\u9a7c': 'CjkUnifiedIdeograph9a7c',
u'\ufeff': 'ZeroWidthNobreakSpace',
u'\u5d7e': 'CjkUnifiedIdeograph5d7e',
u'\uc403': 'HangulSyllableSswalh',
u'\u2382': 'DiscontinuousUnderlineSymbol',
u'\ua986': 'UnicodeCharacter0xa986',
u'\u0d09': 'MalayalamLetterU',
u'\u6c88': 'CjkUnifiedIdeograph6c88',
u'\u930d': 'CjkUnifiedIdeograph930d',
u'\uf28c': 'UnicodeCharacter0xf28c',
u'\u560f': 'CjkUnifiedIdeograph560f',
u'\udc13': 'UnicodeCharacter0xdc13',
u'\u3b92': 'CjkUnifiedIdeograph3b92',
u'\ua217': 'YiSyllableGgiex',
u'\u8196': 'CjkUnifiedIdeograph8196',
u'\u6519': 'CjkUnifiedIdeograph6519',
u'\u4498': 'CjkUnifiedIdeograph4498',
u'\ueb1d': 'UnicodeCharacter0xeb1d',
u'\uca9c': 'HangulSyllableJjyeok',
u'\u3423': 'CjkUnifiedIdeograph3423',
u'\u13a2': 'CherokeeLetterI',
u'\uba27': 'HangulSyllableMyaelb',
u'\u99a6': 'CjkUnifiedIdeograph99a6',
u'\u7d29': 'CjkUnifiedIdeograph7d29',
u'\u5ca8': 'CjkUnifiedIdeograph5ca8',
u'\uc32d': 'HangulSyllableSsaet',
u'\u22ac': 'DoesNotProve',
u'\ua8b0': 'SaurashtraLetterSsa',
u'\u0c33': 'TeluguLetterLla',
u'\u6bb2': 'CjkUnifiedIdeograph6bb2',
u'\u9237': 'CjkUnifiedIdeograph9237',
u'\uf1b6': 'UnicodeCharacter0xf1b6',
u'\u5539': 'CjkUnifiedIdeograph5539',
u'\udb3d': 'UnicodeCharacter0xdb3d',
u'\u3abc': 'CjkUnifiedIdeograph3abc',
u'\ua141': 'YiSyllableDduox',
u'\u80c0': 'CjkUnifiedIdeograph80c0',
u'\u6443': 'CjkUnifiedIdeograph6443',
u'\u43c2': 'CjkUnifiedIdeograph43c2',
u'\uea47': 'UnicodeCharacter0xea47',
u'\uc9c6': 'HangulSyllableJinh',
u'\u334d': 'SquareMeetoru',
u'\u12cc': 'EthiopicSyllableWee',
u'\ub951': 'HangulSyllableRwing',
u'\u98d0': 'CjkUnifiedIdeograph98d0',
u'\u7c53': 'CjkUnifiedIdeograph7c53',
u'\u5bd2': 'CjkUnifiedIdeograph5bd2',
u'\uc257': 'HangulSyllableSwed',
u'\u21d6': 'NorthWestDoubleArrow',
u'\ua7da': 'UnicodeCharacter0xa7da',
u'\u0b5d': 'OriyaLetterRha',
u'\u6adc': 'CjkUnifiedIdeograph6adc',
u'\u9161': 'CjkUnifiedIdeograph9161',
u'\uf0e0': 'UnicodeCharacter0xf0e0',
u'\u5463': 'CjkUnifiedIdeograph5463',
u'\uda67': 'UnicodeCharacter0xda67',
u'\u39e6': 'CjkUnifiedIdeograph39e6',
u'\ua06b': 'YiSyllableBbep',
u'\ubfea': 'HangulSyllableBbweogg',
u'\u636d': 'CjkUnifiedIdeograph636d',
u'\u42ec': 'CjkUnifiedIdeograph42ec',
u'\ue971': 'UnicodeCharacter0xe971',
u'\uc8f0': 'HangulSyllableJyom',
u'\u3277': 'CircledHangulChieuchA',
u'\u11f6': 'HangulJongseongHieuhrieul',
u'\ub87b': 'HangulSyllableRwags',
u'\u97fa': 'CjkUnifiedIdeograph97fa',
u'\u7b7d': 'CjkUnifiedIdeograph7b7d',
u'\u5afc': 'CjkUnifiedIdeograph5afc',
u'\u4081': 'CjkUnifiedIdeograph4081',
u'\u2100': 'AccountOf',
u'\u0583': 'ArmenianSmallLetterPiwr',
u'\ue202': 'UnicodeCharacter0xe202',
u'\uc685': 'HangulSyllableOelt',
u'\ua704': 'ModifierLetterChineseToneYinQu',
u'\u8b87': 'CjkUnifiedIdeograph8b87',
u'\u740e': 'CjkUnifiedIdeograph740e',
u'\u5891': 'CjkUnifiedIdeograph5891',
u'\u3910': 'CjkUnifiedIdeograph3910',
u'\u1d93': 'LatinSmallLetterOpenEWithRetroflexHook',
u'\ufa12': 'CjkCompatibilityIdeographfa12',
u'\ude95': 'UnicodeCharacter0xde95',
u'\ubf14': 'HangulSyllableBbyeols',
u'\ue397': 'UnicodeCharacter0xe397',
u'\ua499': 'YiRadicalHxit',
u'\u4c1e': 'CjkUnifiedIdeograph4c1e',
u'\u1120': 'HangulChoseongPieuptikeut',
u'\u75a3': 'CjkUnifiedIdeograph75a3',
u'\ud222': 'HangulSyllableTyobs',
u'\u36a5': 'CjkUnifiedIdeograph36a5',
u'\u9724': 'CjkUnifiedIdeograph9724',
u'\ufba7': 'ArabicLetterHehGoalFinalForm',
u'\ubca9': 'HangulSyllableBelg',
u'\u6930': 'CjkUnifiedIdeograph6930',
u'\u4db3': 'CjkUnifiedIdeograph4db3',
u'\u2a32': 'SemidirectProductWithBottomClosed',
u'\u0eb5': 'LaoVowelSignIi',
u'\uef34': 'UnicodeCharacter0xef34',
u'\ud3b7': 'HangulSyllablePyeogs',
u'\ub036': 'HangulSyllableGgyulp',
u'\u94b9': 'CjkUnifiedIdeograph94b9',
u'\u4140': 'CjkUnifiedIdeograph4140',
u'\u0242': 'LatinSmallLetterGlottalStop',
u'\u66c5': 'CjkUnifiedIdeograph66c5',
u'\uc744': 'HangulSyllableEul',
u'\u2bc7': 'UnicodeCharacter0x2bc7',
u'\u8846': 'CjkUnifiedIdeograph8846',
u'\uecc9': 'UnicodeCharacter0xecc9',
u'\ub1cb': 'HangulSyllableNwaeh',
u'\u5950': 'CjkUnifiedIdeograph5950',
u'\u1a52': 'UnicodeCharacter0x1a52',
u'\u7ed5': 'CjkUnifiedIdeograph7ed5',
u'\udf54': 'UnicodeCharacter0xdf54',
u'\u03d7': 'GreekKaiSymbol',
u'\ue056': 'UnicodeCharacter0xe056',
u'\uc4d9': 'HangulSyllableSsyunj',
u'\ua558': 'VaiSyllableFa',
u'\u89db': 'CjkUnifiedIdeograph89db',
u'\u7262': 'CjkUnifiedIdeograph7262',
u'\u56e5': 'CjkUnifiedIdeograph56e5',
u'\u3764': 'CjkUnifiedIdeograph3764',
u'\u1be7': 'UnicodeCharacter0x1be7',
u'\uf866': 'UnicodeCharacter0xf866',
u'\udce9': 'UnicodeCharacter0xdce9',
u'\ubd68': 'HangulSyllableByon',
u'\ue1eb': 'UnicodeCharacter0xe1eb',
u'\ua2ed': 'YiSyllableNzit',
u'\u4a72': 'CjkUnifiedIdeograph4a72',
u'\u0f74': 'TibetanVowelSignU',
u'\u73f7': 'CjkUnifiedIdeograph73f7',
u'\ud076': 'HangulSyllableKeulm',
u'\u34f9': 'CjkUnifiedIdeograph34f9',
u'\u9578': 'CjkUnifiedIdeograph9578',
u'\uf9fb': 'CjkCompatibilityIdeographf9fb',
u'\ubafd': 'HangulSyllableMoeg',
u'\u8001': 'CjkUnifiedIdeograph8001',
u'\u6784': 'CjkUnifiedIdeograph6784',
u'\u2886': 'BraillePatternDots238',
u'\ued88': 'UnicodeCharacter0xed88',
u'\u510b': 'CjkUnifiedIdeograph510b',
u'\uae8a': 'HangulSyllableGgyanh',
u'\u120d': 'EthiopicSyllableLe',
u'\ud70f': 'HangulSyllableHwes',
u'\u9811': 'CjkUnifiedIdeograph9811',
u'\u7f94': 'CjkUnifiedIdeograph7f94',
u'\x96': 'UnicodeCharacter0x96',
u'\uc598': 'HangulSyllableYae',
u'\u869a': 'CjkUnifiedIdeograph869a',
u'\u6a1d': 'CjkUnifiedIdeograph6a1d',
u'\u2f1f': 'KangxiRadicalEarth',
u'\uf021': 'UnicodeCharacter0xf021',
u'\ub523': 'HangulSyllableDilh',
u'\u57a4': 'CjkUnifiedIdeograph57a4',
u'\u18a6': 'MongolianLetterAliGaliHalfU',
u'\udda8': 'UnicodeCharacter0xdda8',
u'\u9eaa': 'CjkUnifiedIdeograph9eaa',
u'\u422d': 'CjkUnifiedIdeograph422d',
u'\ua3ac': 'YiSyllableShax',
u'\u072f': 'SyriacLetterPersianDhalath',
u'\uc831': 'HangulSyllableJeng',
u'\u8d33': 'CjkUnifiedIdeograph8d33',
u'\u70b6': 'CjkUnifiedIdeograph70b6',
u'\u35b8': 'CjkUnifiedIdeograph35b8',
u'\uf6ba': 'UnicodeCharacter0xf6ba',
u'\u5a3d': 'CjkUnifiedIdeograph5a3d',
u'\ubbbc': 'HangulSyllableMyuk',
u'\u1f3f': 'GreekCapitalLetterIotaWithDasiaAndPerispomeni',
u'\u2041': 'CaretInsertionPoint',
u'\ue543': 'UnicodeCharacter0xe543',
u'\ua645': 'CyrillicSmallLetterReversedDze',
u'\u48c6': 'CjkUnifiedIdeograph48c6',
u'\u0dc8': 'UnicodeCharacter0xdc8',
u'\uceca': 'HangulSyllableKyaegg',
u'\u93cc': 'CjkUnifiedIdeograph93cc',
u'\u774f': 'CjkUnifiedIdeograph774f',
u'\u3851': 'CjkUnifiedIdeograph3851',
u'\ufd53': 'ArabicLigatureTehWithHahWithMeemInitialForm',
u'\ube55': 'HangulSyllableBib',
u'\u65d8': 'CjkUnifiedIdeograph65d8',
u'\u26da': 'UnicodeCharacter0x26da',
u'\uebdc': 'UnicodeCharacter0xebdc',
u'\u4f5f': 'CjkUnifiedIdeograph4f5f',
u'\uacde': 'HangulSyllableGyep',
u'\u1061': 'MyanmarLetterSgawKarenSha',
u'\ud563': 'HangulSyllableHalb',
u'\u9665': 'CjkUnifiedIdeograph9665',
u'\u7de8': 'CjkUnifiedIdeograph7de8',
u'\u3eea': 'CjkUnifiedIdeograph3eea',
u'\uc3ec': 'HangulSyllableSsoss',
u'\u84ee': 'CjkUnifiedIdeograph84ee',
u'\u6871': 'CjkUnifiedIdeograph6871',
u'\u2d73': 'UnicodeCharacter0x2d73',
u'\uee75': 'UnicodeCharacter0xee75',
u'\ub377': 'HangulSyllableDed',
u'\u55f8': 'CjkUnifiedIdeograph55f8',
u'\u16fa': 'UnicodeCharacter0x16fa',
u'\udbfc': 'UnicodeCharacter0xdbfc',
u'\u9cfe': 'CjkUnifiedIdeograph9cfe',
u'\ua200': 'YiSyllableKax',
u'\u8683': 'CjkUnifiedIdeograph8683',
u'\u6102': 'CjkUnifiedIdeograph6102',
u'\u4585': 'CjkUnifiedIdeograph4585',
u'\ueb06': 'UnicodeCharacter0xeb06',
u'\ucf89': 'HangulSyllableKwat',
u'\u300c': 'LeftCornerBracket',
u'\u148f': 'CanadianSyllabicsYcreeCoo',
u'\uba10': 'HangulSyllableMyam',
u'\u9e93': 'CjkUnifiedIdeograph9e93',
u'\u7912': 'CjkUnifiedIdeograph7912',
u'\u5d95': 'CjkUnifiedIdeograph5d95',
u'\uc316': 'HangulSyllableSsaegg',
u'\u2799': 'HeavyRightwardsArrow',
u'\ua99d': 'UnicodeCharacter0xa99d',
u'\u081c': 'UnicodeCharacter0x81c',
u'\u6c9f': 'CjkUnifiedIdeograph6c9f',
u'\u9220': 'CjkUnifiedIdeograph9220',
u'\uf6a3': 'UnicodeCharacter0xf6a3',
u'\u5122': 'CjkUnifiedIdeograph5122',
u'\udb26': 'UnicodeCharacter0xdb26',
u'\u3fa9': 'CjkUnifiedIdeograph3fa9',
u'\u81ad': 'CjkUnifiedIdeograph81ad',
u'\u602c': 'CjkUnifiedIdeograph602c',
u'\u44af': 'CjkUnifiedIdeograph44af',
u'\uea30': 'UnicodeCharacter0xea30',
u'\uceb3': 'HangulSyllableKyad',
u'\u3336': 'SquareHekutaaru',
u'\u17b9': 'KhmerVowelSignY',
u'\ub53a': 'HangulSyllableDdalm',
u'\u99bd': 'CjkUnifiedIdeograph99bd',
u'\u783c': 'CjkUnifiedIdeograph783c',
u'\u5cbf': 'CjkUnifiedIdeograph5cbf',
u'\uc240': 'HangulSyllableSweols',
u'\u26c3': 'BlackDraughtsKing',
u'\ua8c7': 'UnicodeCharacter0xa8c7',
u'\u0b46': 'UnicodeCharacter0xb46',
u'\u6fc9': 'CjkUnifiedIdeograph6fc9',
u'\u8d4a': 'CjkUnifiedIdeograph8d4a',
u'\uf1cd': 'UnicodeCharacter0xf1cd',
u'\u504c': 'CjkUnifiedIdeograph504c',
u'\uda50': 'UnicodeCharacter0xda50',
u'\u3ed3': 'CjkUnifiedIdeograph3ed3',
u'\u80d7': 'CjkUnifiedIdeograph80d7',
u'\u6356': 'CjkUnifiedIdeograph6356',
u'\u47d9': 'CjkUnifiedIdeograph47d9',
u'\ue55a': 'UnicodeCharacter0xe55a',
u'\uc9dd': 'HangulSyllableJjag',
u'\u3260': 'CircledHangulKiyeok',
u'\u16e3': 'RunicLetterCalc',
u'\ub464': 'HangulSyllableDuss',
u'\u98e7': 'CjkUnifiedIdeograph98e7',
u'\u7b66': 'CjkUnifiedIdeograph7b66',
u'\u5fe9': 'CjkUnifiedIdeograph5fe9',
u'\ufd6a': 'ArabicLigatureSheenWithMeemWithKhahFinalForm',
u'\u21ed': 'UpwardsWhiteArrowOnPedestalWithVerticalBar',
u'\uabf1': 'UnicodeCharacter0xabf1',
u'\u0a70': 'GurmukhiTippi',
u'\u6ef3': 'CjkUnifiedIdeograph6ef3',
u'\u8c74': 'CjkUnifiedIdeograph8c74',
u'\uf0f7': 'UnicodeCharacter0xf0f7',
u'\u5376': 'CjkUnifiedIdeograph5376',
u'\ud57a': 'HangulSyllableHaenh',
u'\u39fd': 'CjkUnifiedIdeograph39fd',
u'\u0301': 'CombiningAcuteAccent',
u'\u6280': 'CjkUnifiedIdeograph6280',
u'\u8505': 'CjkUnifiedIdeograph8505',
u'\ue484': 'UnicodeCharacter0xe484',
u'\u4807': 'CjkUnifiedIdeograph4807',
u'\ud20b': 'HangulSyllableToec',
u'\u2d8a': 'EthiopicSyllableGlottalOa',
u'\ub78e': 'HangulSyllableRabs',
u'\u1b11': 'BalineseLetterOkara',
u'\u7a90': 'CjkUnifiedIdeograph7a90',
u'\u9d15': 'CjkUnifiedIdeograph9d15',
u'\ufc94': 'ArabicLigatureYehWithNoonFinalForm',
u'\u2a1b': 'IntegralWithOverbar',
u'\u059a': 'HebrewAccentYetiv',
u'\uac1f': 'HangulSyllableGaegs',
u'\u8f9e': 'CjkUnifiedIdeograph8f9e',
u'\u7321': 'CjkUnifiedIdeograph7321',
u'\u52a0': 'CjkUnifiedIdeograph52a0',
u'\uf525': 'UnicodeCharacter0xf525',
u'\ud4a4': 'HangulSyllablePweom',
u'\u022b': 'LatinSmallLetterOWithDiaeresisAndMacron',
u'\u1daa': 'ModifierLetterSmallLWithPalatalHook',
u'\u842f': 'CjkUnifiedIdeograph842f',
u'\ue7ae': 'UnicodeCharacter0xe7ae',
u'\u4b31': 'CjkUnifiedIdeograph4b31',
u'\ucd35': 'HangulSyllableCwab',
u'\u2cb4': 'CopticCapitalLetterOldCopticAin',
u'\ub6b8': 'HangulSyllableDdweo',
u'\u1a3b': 'UnicodeCharacter0x1a3b',
u'\u75ba': 'CjkUnifiedIdeograph75ba',
u'\u9c3f': 'CjkUnifiedIdeograph9c3f',
u'\uffbe': 'HalfwidthHangulLetterHieuh',
u'\u2545': 'BoxDrawingsLeftDownHeavyAndRightUpLight',
u'\u04c4': 'CyrillicSmallLetterKaWithHook',
u'\uaf49': 'HangulSyllableGgwag',
u'\u8ec8': 'CjkUnifiedIdeograph8ec8',
u'\u724b': 'CjkUnifiedIdeograph724b',
u'\u4dca': 'HexagramForPeace',
u'\uf44f': 'UnicodeCharacter0xf44f',
u'\ud7ce': 'UnicodeCharacter0xd7ce',
u'\u3d55': 'CjkUnifiedIdeograph3d55',
u'\u1cd4': 'UnicodeCharacter0x1cd4',
u'\u8759': 'CjkUnifiedIdeograph8759',
u'\ue6d8': 'UnicodeCharacter0xe6d8',
u'\u4a5b': 'CjkUnifiedIdeograph4a5b',
u'\ucc5f': 'HangulSyllableCaeh',
u'\u2fde': 'UnicodeCharacter0x2fde',
u'\ub1e2': 'HangulSyllableNoej',
u'\u1565': 'CanadianSyllabicsThoo',
u'\u74e4': 'CjkUnifiedIdeograph74e4',
u'\u9f69': 'CjkUnifiedIdeograph9f69',
u'\ufee8': 'ArabicLetterNoonMedialForm',
u'\u246f': 'CircledNumberSixteen',
u'\u07ee': 'NkoCombiningLongDescendingTone',
u'\uae73': 'HangulSyllableGgaelb',
u'\u89f2': 'CjkUnifiedIdeograph89f2',
u'\u6d75': 'CjkUnifiedIdeograph6d75',
u'\u4cf4': 'CjkUnifiedIdeograph4cf4',
u'\uf779': 'UnicodeCharacter0xf779',
u'\ud6f8': 'HangulSyllableHweok',
u'\u3c7f': 'CjkUnifiedIdeograph3c7f',
u'\u1ffe': 'GreekDasia',
u'\u6a06': 'CjkUnifiedIdeograph6a06',
u'\u4e89': 'CjkUnifiedIdeograph4e89',
u'\u2f08': 'KangxiRadicalMan',
u'\u138b': 'EthiopicSyllableFwe',
u'\uec0a': 'UnicodeCharacter0xec0a',
u'\ud08d': 'HangulSyllableKyinj',
u'\ub10c': 'HangulSyllableNeon',
u'\u958f': 'CjkUnifiedIdeograph958f',
u'\u4216': 'CjkUnifiedIdeograph4216',
u'\u0718': 'SyriacLetterWaw',
u'\u6b9b': 'CjkUnifiedIdeograph6b9b',
u'\uc41a': 'HangulSyllableSswaelm',
u'\u289d': 'BraillePatternDots13458',
u'\u891c': 'CjkUnifiedIdeograph891c',
u'\ued9f': 'UnicodeCharacter0xed9f',
u'\ub2a1': 'HangulSyllableNeub',
u'\u5a26': 'CjkUnifiedIdeograph5a26',
u'\u1f28': 'GreekCapitalLetterEtaWithPsili',
u'\u43ab': 'CjkUnifiedIdeograph43ab',
u'\udc2a': 'UnicodeCharacter0xdc2a',
u'\xad': 'SoftHyphen',
u'\ue12c': 'UnicodeCharacter0xe12c',
u'\uc5af': 'HangulSyllableYaec',
u'\ua62e': 'UnicodeCharacter0xa62e',
u'\u8ab1': 'CjkUnifiedIdeograph8ab1',
u'\u7738': 'CjkUnifiedIdeograph7738',
u'\u5bbb': 'CjkUnifiedIdeograph5bbb',
u'\u343a': 'CjkUnifiedIdeograph343a',
u'\u18bd': 'UnicodeCharacter0x18bd',
u'\uf93c': 'CjkCompatibilityIdeographf93c',
u'\uddbf': 'UnicodeCharacter0xddbf',
u'\ube3e': 'HangulSyllableByij',
u'\ue2c1': 'UnicodeCharacter0xe2c1',
u'\ua7c3': 'UnicodeCharacter0xa7c3',
u'\u4f48': 'CjkUnifiedIdeograph4f48',
u'\u0c4a': 'TeluguVowelSignO',
u'\u70cd': 'CjkUnifiedIdeograph70cd',
u'\ud14c': 'HangulSyllableTe',
u'\u35cf': 'CjkUnifiedIdeograph35cf',
u'\u964e': 'CjkUnifiedIdeograph964e',
u'\ufad1': 'CjkCompatibilityIdeographfad1',
u'\ubfd3': 'HangulSyllableBbud',
u'\u645a': 'CjkUnifiedIdeograph645a',
u'\u48dd': 'CjkUnifiedIdeograph48dd',
u'\u295c': 'UpwardsHarpoonWithBarbRightFromBar',
u'\u0ddf': 'SinhalaVowelSignGayanukitta',
u'\uee5e': 'UnicodeCharacter0xee5e',
u'\ud2e1': 'HangulSyllableTyilt',
u'\ub360': 'HangulSyllableDeols',
u'\u97e3': 'CjkUnifiedIdeograph97e3',
u'\u7c6a': 'CjkUnifiedIdeograph7c6a',
u'\u016c': 'LatinCapitalLetterUWithBreve',
u'\u65ef': 'CjkUnifiedIdeograph65ef',
u'\uc66e': 'HangulSyllableWaebs',
u'\u2af1': 'DownTackWithCircleBelow',
u'\u8b70': 'CjkUnifiedIdeograph8b70',
u'\ueff3': 'UnicodeCharacter0xeff3',
u'\uacf5': 'HangulSyllableGong',
u'\u547a': 'CjkUnifiedIdeograph547a',
u'\u197c': 'UnicodeCharacter0x197c',
u'\u7dff': 'CjkUnifiedIdeograph7dff',
u'\ude7e': 'UnicodeCharacter0xde7e',
u'\ue380': 'UnicodeCharacter0xe380',
u'\u4703': 'CjkUnifiedIdeograph4703',
u'\ua082': 'YiSyllableNbox',
u'\u0405': 'CyrillicCapitalLetterDze',
u'\uc907': 'HangulSyllableJulb',
u'\u8e09': 'CjkUnifiedIdeograph8e09',
u'\u718c': 'CjkUnifiedIdeograph718c',
u'\u368e': 'CjkUnifiedIdeograph368e',
u'\ufb90': 'ArabicLetterKehehInitialForm',
u'\u5f13': 'CjkUnifiedIdeograph5f13',
u'\ub892': 'HangulSyllableRwap',
u'\u1c15': 'LepchaLetterMa',
u'\u2117': 'SoundRecordingCopyright',
u'\ue619': 'UnicodeCharacter0xe619',
u'\uab1b': 'UnicodeCharacter0xab1b',
u'\u499c': 'CjkUnifiedIdeograph499c',
u'\u0e9e': 'LaoLetterPhoTam',
u'\ud3a0': 'HangulSyllablePel',
u'\u90a2': 'CjkUnifiedIdeograph90a2',
u'\u7425': 'CjkUnifiedIdeograph7425',
u'\u3927': 'CjkUnifiedIdeograph3927',
u'\ufe29': 'UnicodeCharacter0xfe29',
u'\u832b': 'CjkUnifiedIdeograph832b',
u'\u66ae': 'CjkUnifiedIdeograph66ae',
u'\u2bb0': 'UnicodeCharacter0x2bb0',
u'\ue8b2': 'UnicodeCharacter0xe8b2',
u'\u4c35': 'CjkUnifiedIdeograph4c35',
u'\uadb4': 'HangulSyllableGwem',
u'\u1137': 'HangulChoseongSioschieuch',
u'\ud639': 'HangulSyllableHog',
u'\u9b3b': 'CjkUnifiedIdeograph9b3b',
u'\u7ebe': 'CjkUnifiedIdeograph7ebe',
u'\u03c0': 'GreekSmallLetterPi',
u'\uc0c2': 'HangulSyllableSaj',
u'\u85c4': 'CjkUnifiedIdeograph85c4',
u'\u6947': 'CjkUnifiedIdeograph6947',
u'\u2e49': 'UnicodeCharacter0x2e49',
u'\uf34b': 'UnicodeCharacter0xf34b',
u'\ub04d': 'HangulSyllableGgeulg',
u'\u56ce': 'CjkUnifiedIdeograph56ce',
u'\u1bd0': 'UnicodeCharacter0x1bd0',
u'\ud8d2': 'UnicodeCharacter0xd8d2',
u'\u9dd4': 'CjkUnifiedIdeograph9dd4',
u'\u4157': 'CjkUnifiedIdeograph4157',
u'\ua2d6': 'YiSyllableZzie',
u'\u0659': 'ArabicZwarakay',
u'\ucb5b': 'HangulSyllableJjus',
u'\u885d': 'CjkUnifiedIdeograph885d',
u'\u73e0': 'CjkUnifiedIdeograph73e0',
u'\u30e2': 'KatakanaLetterMo',
u'\uf5e4': 'UnicodeCharacter0xf5e4',
u'\u5967': 'CjkUnifiedIdeograph5967',
u'\ubae6': 'HangulSyllableMwaenh',
u'\u1e69': 'LatinSmallLetterSWithDotBelowAndDotAbove',
u'\u236b': 'AplFunctionalSymbolDelTilde',
u'\ue06d': 'UnicodeCharacter0xe06d',
u'\ua56f': 'VaiSyllableNa',
u'\u4bf0': 'CjkUnifiedIdeograph4bf0',
u'\u08f2': 'UnicodeCharacter0x8f2',
u'\ucdf4': 'HangulSyllableCwils',
u'\u92f6': 'CjkUnifiedIdeograph92f6',
u'\u7679': 'CjkUnifiedIdeograph7679',
u'\u3b7b': 'CjkUnifiedIdeograph3b7b',
u'\uf87d': 'UnicodeCharacter0xf87d',
u'\ubd7f': 'HangulSyllableByoh',
u'\uc581': 'HangulSyllableYanj',
u'\u2204': 'ThereDoesNotExist',
u'\u0e87': 'LaoLetterNgo',
u'\ua808': 'SylotiNagriLetterKho',
u'\u948b': 'CjkUnifiedIdeograph948b',
u'\u6b0a': 'CjkUnifiedIdeograph6b0a',
u'\u578d': 'CjkUnifiedIdeograph578d',
u'\uf10e': 'UnicodeCharacter0xf10e',
u'\udd91': 'UnicodeCharacter0xdd91',
u'\u3a14': 'CjkUnifiedIdeograph3a14',
u'\u6697': 'CjkUnifiedIdeograph6697',
u'\u8018': 'CjkUnifiedIdeograph8018',
u'\uec9b': 'UnicodeCharacter0xec9b',
u'\u431a': 'CjkUnifiedIdeograph431a',
u'\uc91e': 'HangulSyllableJweonh',
u'\u35a1': 'CjkUnifiedIdeograph35a1',
u'\ub3a5': 'HangulSyllableDyeot',
u'\u1224': 'EthiopicSyllableSzee',
u'\u7ea7': 'CjkUnifiedIdeograph7ea7',
u'\u9828': 'CjkUnifiedIdeograph9828',
u'\uc4ab': 'HangulSyllableSswelh',
u'\u5b2a': 'CjkUnifiedIdeograph5b2a',
u'\u212e': 'EstimatedSymbol',
u'\u0db1': 'SinhalaLetterDantajaNayanna',
u'\uaf32': 'HangulSyllableGgonh',
u'\u8bb5': 'CjkUnifiedIdeograph8bb5',
u'\u6a34': 'CjkUnifiedIdeograph6a34',
u'\u56b7': 'CjkUnifiedIdeograph56b7',
u'\uf038': 'UnicodeCharacter0xf038',
u'\udcbb': 'UnicodeCharacter0xdcbb',
u'\u393e': 'CjkUnifiedIdeograph393e',
u'\u65c1': 'CjkUnifiedIdeograph65c1',
u'\u8742': 'CjkUnifiedIdeograph8742',
u'\ue3c5': 'UnicodeCharacter0xe3c5',
u'\u4244': 'CjkUnifiedIdeograph4244',
u'\uc848': 'HangulSyllableJyeom',
u'\u34cb': 'CjkUnifiedIdeograph34cb',
u'\ub2cf': 'HangulSyllableNid',
u'\u114e': 'HangulChoseongChitueumcieuc',
u'\u7dd1': 'CjkUnifiedIdeograph7dd1',
u'\u9f52': 'CjkUnifiedIdeograph9f52',
u'\ufbd5': 'ArabicLetterNgInitialForm',
u'\u5a54': 'CjkUnifiedIdeograph5a54',
u'\u2058': 'FourDotPunctuation',
u'\u0cdb': 'UnicodeCharacter0xcdb',
u'\uae5c': 'HangulSyllableGgam',
u'\u8adf': 'CjkUnifiedIdeograph8adf',
u'\u695e': 'CjkUnifiedIdeograph695e',
u'\u55e1': 'CjkUnifiedIdeograph55e1',
u'\uf762': 'UnicodeCharacter0xf762',
u'\ud3e5': 'HangulSyllablePyeng',
u'\u3868': 'CjkUnifiedIdeograph3868',
u'\u64eb': 'CjkUnifiedIdeograph64eb',
u'\u866c': 'CjkUnifiedIdeograph866c',
u'\ue2ef': 'UnicodeCharacter0xe2ef',
u'\u416e': 'CjkUnifiedIdeograph416e',
u'\ucf72': 'HangulSyllableKwagg',
u'\u2bf5': 'UnicodeCharacter0x2bf5',
u'\ub1f9': 'HangulSyllableNyob',
u'\u1078': 'MyanmarLetterShanCa',
u'\u7cfb': 'CjkUnifiedIdeograph7cfb',
u'\u9e7c': 'CjkUnifiedIdeograph9e7c',
u'\ufaff': 'UnicodeCharacter0xfaff',
u'\u597e': 'CjkUnifiedIdeograph597e',
u'\uc003': 'HangulSyllableBbweoh',
u'\u2782': 'DingbatCircledSansserifDigitThree',
u'\uad86': 'HangulSyllableGup',
u'\u0909': 'DevanagariLetterU',
u'\u6888': 'CjkUnifiedIdeograph6888',
u'\u970d': 'CjkUnifiedIdeograph970d',
u'\uf68c': 'UnicodeCharacter0xf68c',
u'\u520f': 'CjkUnifiedIdeograph520f',
u'\ud813': 'UnicodeCharacter0xd813',
u'\u3f92': 'CjkUnifiedIdeograph3f92',
u'\ua617': 'VaiSymbolBang',
u'\u8596': 'CjkUnifiedIdeograph8596',
u'\u6119': 'CjkUnifiedIdeograph6119',
u'\u4098': 'CjkUnifiedIdeograph4098',
u'\uef1d': 'UnicodeCharacter0xef1d',
u'\uce9c': 'HangulSyllableKaels',
u'\u3023': 'HangzhouNumeralThree',
u'\u17a2': 'KhmerLetterQa',
u'\ube27': 'HangulSyllableBeuh',
u'\u9da6': 'CjkUnifiedIdeograph9da6',
u'\u7929': 'CjkUnifiedIdeograph7929',
u'\u58a8': 'CjkUnifiedIdeograph58a8',
u'\uc72d': 'HangulSyllableYult',
u'\u26ac': 'MediumSmallWhiteCircle',
u'\uacb0': 'HangulSyllableGyeol',
u'\u0833': 'UnicodeCharacter0x833',
u'\u6fb2': 'CjkUnifiedIdeograph6fb2',
u'\u9637': 'CjkUnifiedIdeograph9637',
u'\uf5b6': 'UnicodeCharacter0xf5b6',
u'\u5139': 'CjkUnifiedIdeograph5139',
u'\udf3d': 'UnicodeCharacter0xdf3d',
u'\u3ebc': 'CjkUnifiedIdeograph3ebc',
u'\ua541': 'VaiSyllableNji',
u'\u84c0': 'CjkUnifiedIdeograph84c0',
u'\u6043': 'CjkUnifiedIdeograph6043',
u'\u47c2': 'CjkUnifiedIdeograph47c2',
u'\uee47': 'UnicodeCharacter0xee47',
u'\ucdc6': 'HangulSyllableCweoj',
u'\u374d': 'CjkUnifiedIdeograph374d',
u'\u16cc': 'RunicLetterShorttwigsolS',
u'\ubd51': 'HangulSyllableBoelg',
u'\u9cd0': 'CjkUnifiedIdeograph9cd0',
u'\u7853': 'CjkUnifiedIdeograph7853',
u'\u5fd2': 'CjkUnifiedIdeograph5fd2',
u'\uc657': 'HangulSyllableWac',
u'\u25d6': 'LeftHalfBlackCircle',
u'\ua3da': 'YiSyllableRyt',
u'\u0f5d': 'TibetanLetterWa',
u'\u6edc': 'CjkUnifiedIdeograph6edc',
u'\u9561': 'CjkUnifiedIdeograph9561',
u'\uf4e0': 'UnicodeCharacter0xf4e0',
u'\u5063': 'CjkUnifiedIdeograph5063',
u'\ude67': 'UnicodeCharacter0xde67',
u'\u3de6': 'CjkUnifiedIdeograph3de6',
u'\ua46b': 'YiSyllableXyt',
u'\ubbea': 'HangulSyllableMyilp',
u'\u676d': 'CjkUnifiedIdeograph676d',
u'\u46ec': 'CjkUnifiedIdeograph46ec',
u'\ued71': 'UnicodeCharacter0xed71',
u'\uccf0': 'HangulSyllableCyen',
u'\u3677': 'CjkUnifiedIdeograph3677',
u'\u15f6': 'CanadianSyllabicsCarrierKho',
u'\ubc7b': 'HangulSyllableByaes',
u'\u93fa': 'CjkUnifiedIdeograph93fa',
u'\u7f7d': 'CjkUnifiedIdeograph7f7d',
u'\u5efc': 'CjkUnifiedIdeograph5efc',
u'\u4481': 'CjkUnifiedIdeograph4481',
u'\u2500': 'BoxDrawingsLightHorizontal',
u'\u0183': 'LatinSmallLetterBWithTopbar',
u'\ue602': 'UnicodeCharacter0xe602',
u'\uc285': 'HangulSyllableSwit',
u'\ua304': 'YiSyllableNzyx',
u'\u8f87': 'CjkUnifiedIdeograph8f87',
u'\u700e': 'CjkUnifiedIdeograph700e',
u'\u5c91': 'CjkUnifiedIdeograph5c91',
u'\u3d10': 'CjkUnifiedIdeograph3d10',
u'\u1993': 'NewTaiLueLetterLowNa',
u'\ufe12': 'PresentationFormForVerticalIdeographicFullStop',
u'\uda95': 'UnicodeCharacter0xda95',
u'\ubb14': 'HangulSyllableMoek',
u'\ue797': 'UnicodeCharacter0xe797',
u'\ua099': 'YiSyllableHmax',
u'\u481e': 'CjkUnifiedIdeograph481e',
u'\u1520': 'CanadianSyllabicsWestcreeShwoo',
u'\u71a3': 'CjkUnifiedIdeograph71a3',
u'\ud622': 'HangulSyllableHyenh',
u'\u32a5': 'CircledIdeographCentre',
u'\u9324': 'CjkUnifiedIdeograph9324',
u'\uffa7': 'HalfwidthHangulLetterTikeut',
u'\ub8a9': 'HangulSyllableRwaeng',
u'\u6d30': 'CjkUnifiedIdeograph6d30',
u'\u49b3': 'CjkUnifiedIdeograph49b3',
u'\u2e32': 'UnicodeCharacter0x2e32',
u'\u0ab5': 'GujaratiLetterVa',
u'\ueb34': 'UnicodeCharacter0xeb34',
u'\ud7b7': 'UnicodeCharacter0xd7b7',
u'\ub436': 'HangulSyllableDyogg',
u'\u90b9': 'CjkUnifiedIdeograph90b9',
u'\u4540': 'CjkUnifiedIdeograph4540',
u'\u0642': 'ArabicLetterQaf',
u'\u62c5': 'CjkUnifiedIdeograph62c5',
u'\uc344': 'HangulSyllableSsyass',
u'\u2fc7': 'KangxiRadicalHemp',
u'\u8c46': 'CjkUnifiedIdeograph8c46',
u'\ue8c9': 'UnicodeCharacter0xe8c9',
u'\ub5cb': 'HangulSyllableDdelh',
u'\u5d50': 'CjkUnifiedIdeograph5d50',
u'\u1e52': 'LatinCapitalLetterOWithMacronAndAcute',
u'\u7ad5': 'CjkUnifiedIdeograph7ad5',
u'\udb54': 'UnicodeCharacter0xdb54',
u'\u07d7': 'NkoLetterCha',
u'\ue456': 'UnicodeCharacter0xe456',
u'\uc0d9': 'HangulSyllableSaeb',
u'\ua158': 'YiSyllableNdax',
u'\u8ddb': 'CjkUnifiedIdeograph8ddb',
u'\u7662': 'CjkUnifiedIdeograph7662',
u'\u52e5': 'CjkUnifiedIdeograph52e5',
u'\u3364': 'IdeographicTelegraphSymbolForHourTwelve',
u'\u1fe7': 'GreekSmallLetterUpsilonWithDialytikaAndPerispomeni',
u'\ufc66': 'ArabicLigatureYehWithHamzaAboveWithMeemFinalForm',
u'\ud8e9': 'UnicodeCharacter0xd8e9',
u'\ub968': 'HangulSyllableRyum',
u'\ue5eb': 'UnicodeCharacter0xe5eb',
u'\ua6ed': 'UnicodeCharacter0xa6ed',
u'\u4e72': 'CjkUnifiedIdeograph4e72',
u'\u0b74': 'UnicodeCharacter0xb74',
u'\u77f7': 'CjkUnifiedIdeograph77f7',
u'\ud476': 'HangulSyllablePyop',
u'\u30f9': 'KatakanaLetterVe',
u'\u9178': 'CjkUnifiedIdeograph9178',
u'\ufdfb': 'ArabicLigatureJallajalalouhou',
u'\ubefd': 'HangulSyllableBbeb',
u'\u8401': 'CjkUnifiedIdeograph8401',
u'\u6384': 'CjkUnifiedIdeograph6384',
u'\u2c86': 'CopticCapitalLetterDalda',
u'\ue988': 'UnicodeCharacter0xe988',
u'\u550b': 'CjkUnifiedIdeograph550b',
u'\uaa8a': 'UnicodeCharacter0xaa8a',
u'\u160d': 'CanadianSyllabicsCarrierMa',
u'\ud30f': 'HangulSyllablePags',
u'\u9c11': 'CjkUnifiedIdeograph9c11',
u'\u7b94': 'CjkUnifiedIdeograph7b94',
u'\u0496': 'CyrillicCapitalLetterZheWithDescender',
u'\uc198': 'HangulSyllableSols',
u'\u829a': 'CjkUnifiedIdeograph829a',
u'\u6e1d': 'CjkUnifiedIdeograph6e1d',
u'\u2b1f': 'BlackPentagon',
u'\uf421': 'UnicodeCharacter0xf421',
u'\ub123': 'HangulSyllableNeoh',
u'\u53a4': 'CjkUnifiedIdeograph53a4',
u'\u1ca6': 'UnicodeCharacter0x1ca6',
u'\ud9a8': 'UnicodeCharacter0xd9a8',
u'\u9aaa': 'CjkUnifiedIdeograph9aaa',
u'\u462d': 'CjkUnifiedIdeograph462d',
u'\ua7ac': 'UnicodeCharacter0xa7ac',
u'\u032f': 'CombiningInvertedBreveBelow',
u'\ucc31': 'HangulSyllableCalg',
u'\u8933': 'CjkUnifiedIdeograph8933',
u'\u74b6': 'CjkUnifiedIdeograph74b6',
u'\u31b8': 'UnicodeCharacter0x31b8',
u'\uf2ba': 'UnicodeCharacter0xf2ba',
u'\u5e3d': 'CjkUnifiedIdeograph5e3d',
u'\ubfbc': 'HangulSyllableBbyols',
u'\u1b3f': 'BalineseVowelSignTalingRepa',
u'\u2441': 'OcrChair',
u'\ue143': 'UnicodeCharacter0xe143',
u'\ua245': 'YiSyllableMgur',
u'\u4cc6': 'CjkUnifiedIdeograph4cc6',
u'\u09c8': 'BengaliVowelSignAi',
u'\ucaca': 'HangulSyllableJjolp',
u'\u97cc': 'CjkUnifiedIdeograph97cc',
u'\u734f': 'CjkUnifiedIdeograph734f',
u'\u3c51': 'CjkUnifiedIdeograph3c51',
u'\uf953': 'CjkCompatibilityIdeographf953',
u'\uba55': 'HangulSyllableMeg',
u'\u61d8': 'CjkUnifiedIdeograph61d8',
u'\u22da': 'LessthanEqualToOrGreaterthan',
u'\uefdc': 'UnicodeCharacter0xefdc',
u'\u4b5f': 'CjkUnifiedIdeograph4b5f',
u'\ua8de': 'UnicodeCharacter0xa8de',
u'\u1461': 'CanadianSyllabicsTwa',
u'\ud163': 'HangulSyllableTec',
u'\u9265': 'CjkUnifiedIdeograph9265',
u'\u79e8': 'CjkUnifiedIdeograph79e8',
u'\u3aea': 'CjkUnifiedIdeograph3aea',
u'\uc7ec': 'HangulSyllableJyael',
u'\u80ee': 'CjkUnifiedIdeograph80ee',
u'\u6c71': 'CjkUnifiedIdeograph6c71',
u'\u2973': 'LeftwardsArrowAboveTildeOperator',
u'\uea75': 'UnicodeCharacter0xea75',
u'\ub777': 'HangulSyllableDdic',
u'\u51f8': 'CjkUnifiedIdeograph51f8',
u'\u12fa': 'EthiopicSyllableDdi',
u'\udffc': 'UnicodeCharacter0xdffc',
u'\u98fe': 'CjkUnifiedIdeograph98fe',
u'\ua600': 'VaiSyllableJe',
u'\u8283': 'CjkUnifiedIdeograph8283',
u'\u6502': 'CjkUnifiedIdeograph6502',
u'\u4185': 'CjkUnifiedIdeograph4185',
u'\uef06': 'UnicodeCharacter0xef06',
u'\ucb89': 'HangulSyllableJjwelg',
u'\u340c': 'CjkUnifiedIdeograph340c',
u'\u108f': 'MyanmarSignRumaiPalaungTone5',
u'\ube10': 'HangulSyllableBeun',
u'\u9a93': 'CjkUnifiedIdeograph9a93',
u'\u7d12': 'CjkUnifiedIdeograph7d12',
u'\u5995': 'CjkUnifiedIdeograph5995',
u'\uc716': 'HangulSyllableWibs',
u'\u2399': 'PrintScreenSymbol',
u'\uad9d': 'HangulSyllableGweong',
u'\u0c1c': 'TeluguLetterJa',
u'\u689f': 'CjkUnifiedIdeograph689f',
u'\u9620': 'CjkUnifiedIdeograph9620',
u'\uf2a3': 'UnicodeCharacter0xf2a3',
u'\u5522': 'CjkUnifiedIdeograph5522',
u'\udf26': 'UnicodeCharacter0xdf26',
u'\u3ba9': 'CjkUnifiedIdeograph3ba9',
u'\u85ad': 'CjkUnifiedIdeograph85ad',
u'\u642c': 'CjkUnifiedIdeograph642c',
u'\u40af': 'CjkUnifiedIdeograph40af',
u'\uee30': 'UnicodeCharacter0xee30',
u'\ucab3': 'HangulSyllableJjyes',
u'\u3736': 'CjkUnifiedIdeograph3736',
u'\u13b9': 'CherokeeLetterMa',
u'\ub13a': 'HangulSyllableNej',
u'\u9dbd': 'CjkUnifiedIdeograph9dbd',
u'\u7c3c': 'CjkUnifiedIdeograph7c3c',
u'\u58bf': 'CjkUnifiedIdeograph58bf',
u'\uc640': 'HangulSyllableWa',
u'\u22c3': 'NaryUnion',
u'\uacc7': 'HangulSyllableGyegs',
u'\u0f46': 'TibetanLetterCha',
u'\u6bc9': 'CjkUnifiedIdeograph6bc9',
u'\u894a': 'CjkUnifiedIdeograph894a',
u'\uf5cd': 'UnicodeCharacter0xf5cd',
u'\u544c': 'CjkUnifiedIdeograph544c',
u'\ude50': 'UnicodeCharacter0xde50',
u'\u3ad3': 'CjkUnifiedIdeograph3ad3',
u'\u84d7': 'CjkUnifiedIdeograph84d7',
u'\u6756': 'CjkUnifiedIdeograph6756',
u'\u43d9': 'CjkUnifiedIdeograph43d9',
u'\ue15a': 'UnicodeCharacter0xe15a',
u'\ucddd': 'HangulSyllableCweb',
u'\u3660': 'CjkUnifiedIdeograph3660',
u'\u12e3': 'EthiopicSyllableZhaa',
u'\ub064': 'HangulSyllableGgyin',
u'\u9ce7': 'CjkUnifiedIdeograph9ce7',
u'\u7f66': 'CjkUnifiedIdeograph7f66',
u'\u5be9': 'CjkUnifiedIdeograph5be9',
u'\uf96a': 'CjkCompatibilityIdeographf96a',
u'\u25ed': 'UppointingTriangleWithLeftHalfBlack',
u'\uaff1': 'HangulSyllableGgweg',
u'\u0e70': 'UnicodeCharacter0xe70',
u'\u6af3': 'CjkUnifiedIdeograph6af3',
u'\u8874': 'CjkUnifiedIdeograph8874',
u'\uf4f7': 'UnicodeCharacter0xf4f7',
u'\u5776': 'CjkUnifiedIdeograph5776',
u'\ud17a': 'HangulSyllableTyeobs',
u'\u3dfd': 'CjkUnifiedIdeograph3dfd',
u'\u0701': 'SyriacSupralinearFullStop',
u'\u6680': 'CjkUnifiedIdeograph6680',
u'\u8105': 'CjkUnifiedIdeograph8105',
u'\ue084': 'UnicodeCharacter0xe084',
u'\u4c07': 'CjkUnifiedIdeograph4c07',
u'\ud60b': 'HangulSyllableHyeolb',
u'\u298a': 'ZNotationRightBindingBracket',
u'\ub38e': 'HangulSyllableDyeogg',
u'\u1f11': 'GreekSmallLetterEpsilonWithDasia',
u'\u7e90': 'CjkUnifiedIdeograph7e90',
u'\u9915': 'CjkUnifiedIdeograph9915',
u'\uf894': 'UnicodeCharacter0xf894',
u'\u2e1b': 'TildeWithRingAbove',
u'\u019a': 'LatinSmallLetterLWithBar',
u'\ua81f': 'SylotiNagriLetterLo',
u'\u8b9e': 'CjkUnifiedIdeograph8b9e',
u'\u7721': 'CjkUnifiedIdeograph7721',
u'\u56a0': 'CjkUnifiedIdeograph56a0',
u'\uf125': 'UnicodeCharacter0xf125',
u'\ud0a4': 'HangulSyllableKi',
u'\u062b': 'ArabicLetterTheh',
u'\u19aa': 'UnicodeCharacter0x19aa',
u'\u802f': 'CjkUnifiedIdeograph802f',
u'\ue3ae': 'UnicodeCharacter0xe3ae',
u'\u4f31': 'CjkUnifiedIdeograph4f31',
u'\uc935': 'HangulSyllableJweg',
u'\u28b4': 'BraillePatternDots3568',
u'\ub2b8': 'HangulSyllableNyils',
u'\u1e3b': 'LatinSmallLetterLWithLineBelow',
u'\u71ba': 'CjkUnifiedIdeograph71ba',
u'\u983f': 'CjkUnifiedIdeograph983f',
u'\ufbbe': 'UnicodeCharacter0xfbbe',
u'\u2145': 'DoublestruckItalicCapitalD',
u'\xc4': 'LatinCapitalLetterAWithDiaeresis',
u'\uab49': 'UnicodeCharacter0xab49',
u'\u8ac8': 'CjkUnifiedIdeograph8ac8',
u'\u764b': 'CjkUnifiedIdeograph764b',
u'\u49ca': 'CjkUnifiedIdeograph49ca',
u'\uf04f': 'UnicodeCharacter0xf04f',
u'\ud3ce': 'HangulSyllablePyeop',
u'\u3955': 'CjkUnifiedIdeograph3955',
u'\u18d4': 'UnicodeCharacter0x18d4',
u'\u8359': 'CjkUnifiedIdeograph8359',
u'\ue2d8': 'UnicodeCharacter0xe2d8',
u'\u4e5b': 'CjkUnifiedIdeograph4e5b',
u'\uc85f': 'HangulSyllableJyelb',
u'\u2bde': 'UnicodeCharacter0x2bde',
u'\ub5e2': 'HangulSyllableDdyeolm',
u'\u1165': 'HangulJungseongEo',
u'\u70e4': 'CjkUnifiedIdeograph70e4',
u'\u9b69': 'CjkUnifiedIdeograph9b69',
u'\ufae8': 'UnicodeCharacter0xfae8',
u'\u206f': 'NominalDigitShapes',
u'\u03ee': 'CopticCapitalLetterDei',
u'\uaa73': 'UnicodeCharacter0xaa73',
u'\u8df2': 'CjkUnifiedIdeograph8df2',
u'\u6975': 'CjkUnifiedIdeograph6975',
u'\u48f4': 'CjkUnifiedIdeograph48f4',
u'\uf379': 'UnicodeCharacter0xf379',
u'\ud2f8': 'HangulSyllableTil',
u'\u387f': 'CjkUnifiedIdeograph387f',
u'\u1bfe': 'UnicodeCharacter0x1bfe',
u'\u6e06': 'CjkUnifiedIdeograph6e06',
u'\u4a89': 'CjkUnifiedIdeograph4a89',
u'\u2b08': 'NorthEastBlackArrow',
u'\u178b': 'KhmerLetterTtha',
u'\ue80a': 'UnicodeCharacter0xe80a',
u'\ud48d': 'HangulSyllablePung',
u'\ub50c': 'HangulSyllableDyiss',
u'\u918f': 'CjkUnifiedIdeograph918f',
u'\u4616': 'CjkUnifiedIdeograph4616',
u'\u0318': 'CombiningLeftTackBelow',
u'\u6f9b': 'CjkUnifiedIdeograph6f9b',
u'\uc01a': 'HangulSyllableBbwej',
u'\u2c9d': 'CopticSmallLetterKsi',
u'\u8d1c': 'CjkUnifiedIdeograph8d1c',
u'\ue99f': 'UnicodeCharacter0xe99f',
u'\ub6a1': 'HangulSyllableDdunj',
u'\u5e26': 'CjkUnifiedIdeograph5e26',
u'\u1b28': 'BalineseLetterPaKapal',
u'\u47ab': 'CjkUnifiedIdeograph47ab',
u'\ud82a': 'UnicodeCharacter0xd82a',
u'\u04ad': 'CyrillicSmallLetterTeWithDescender',
u'\ue52c': 'UnicodeCharacter0xe52c',
u'\uc1af': 'HangulSyllableSwad',
u'\ua22e': 'YiSyllableGgurx',
u'\u8eb1': 'CjkUnifiedIdeograph8eb1',
u'\u7338': 'CjkUnifiedIdeograph7338',
u'\u5fbb': 'CjkUnifiedIdeograph5fbb',
u'\u303a': 'HangzhouNumeralThirty',
u'\u1cbd': 'UnicodeCharacter0x1cbd',
u'\ufd3c': 'ArabicLigatureAlefWithFathatanFinalForm',
u'\ud9bf': 'UnicodeCharacter0xd9bf',
u'\uba3e': 'HangulSyllableMeonh',
u'\ue6c1': 'UnicodeCharacter0xe6c1',
u'\ua3c3': 'YiSyllableShyp',
u'\u4b48': 'CjkUnifiedIdeograph4b48',
u'\u084a': 'UnicodeCharacter0x84a',
u'\u74cd': 'CjkUnifiedIdeograph74cd',
u'\ud54c': 'HangulSyllablePim',
u'\u31cf': 'CjkStrokeN',
u'\u924e': 'CjkUnifiedIdeograph924e',
u'\ufed1': 'ArabicLetterFehIsolatedForm',
u'\ubbd3': 'HangulSyllableMeus',
u'\u605a': 'CjkUnifiedIdeograph605a',
u'\u4cdd': 'CjkUnifiedIdeograph4cdd',
u'\u2d5c': 'TifinaghLetterYat',
u'\u09df': 'BengaliLetterYya',
u'\uea5e': 'UnicodeCharacter0xea5e',
u'\ud6e1': 'HangulSyllableHweog',
u'\ub760': 'HangulSyllableDdi',
u'\u93e3': 'CjkUnifiedIdeograph93e3',
u'\u786a': 'CjkUnifiedIdeograph786a',
u'\u056c': 'ArmenianSmallLetterLiwn',
u'\u61ef': 'CjkUnifiedIdeograph61ef',
u'\uc26e': 'HangulSyllableSwigg',
u'\u2ef1': 'CjkRadicalTurtle',
u'\u8f70': 'CjkUnifiedIdeograph8f70',
u'\uebf3': 'UnicodeCharacter0xebf3',
u'\ua8f5': 'UnicodeCharacter0xa8f5',
u'\u507a': 'CjkUnifiedIdeograph507a',
u'\u1d7c': 'LatinSmallLetterIotaWithStroke',
u'\u79ff': 'CjkUnifiedIdeograph79ff',
u'\uda7e': 'UnicodeCharacter0xda7e',
u'\ue780': 'UnicodeCharacter0xe780',
u'\u4303': 'CjkUnifiedIdeograph4303',
u'\ua482': 'YiSyllableYux',
u'\ucd07': 'HangulSyllableCyeh',
u'\u8a09': 'CjkUnifiedIdeograph8a09',
u'\u758c': 'CjkUnifiedIdeograph758c',
u'\u328e': 'CircledIdeographMetal',
u'\uff90': 'HalfwidthKatakanaLetterMi',
u'\u5b13': 'CjkUnifiedIdeograph5b13',
u'\ubc92': 'HangulSyllableBeolp',
u'\u1815': 'MongolianDigitFive',
u'\u2517': 'BoxDrawingsHeavyUpAndRight',
u'\ue219': 'UnicodeCharacter0xe219',
u'\uaf1b': 'HangulSyllableGgyelb',
u'\u4d9c': 'CjkUnifiedIdeograph4d9c',
u'\u0a9e': 'GujaratiLetterNya',
u'\ud7a0': 'HangulSyllableHik',
u'\u94a2': 'CjkUnifiedIdeograph94a2',
u'\u7025': 'CjkUnifiedIdeograph7025',
u'\u3d27': 'CjkUnifiedIdeograph3d27',
u'\ufa29': 'CjkCompatibilityIdeographfa29',
u'\u872b': 'CjkUnifiedIdeograph872b',
u'\u62ae': 'CjkUnifiedIdeograph62ae',
u'\u2fb0': 'KangxiRadicalLeather',
u'\uecb2': 'UnicodeCharacter0xecb2',
u'\u4835': 'CjkUnifiedIdeograph4835',
u'\ua9b4': 'UnicodeCharacter0xa9b4',
u'\u1537': 'CanadianSyllabicsYwoo',
u'\ud239': 'HangulSyllableTult',
u'\u9f3b': 'CjkUnifiedIdeograph9f3b',
u'\u7abe': 'CjkUnifiedIdeograph7abe',
u'\u07c0': 'NkoDigitZero',
u'\uc4c2': 'HangulSyllableSswilm',
u'\u81c4': 'CjkUnifiedIdeograph81c4',
u'\u6d47': 'CjkUnifiedIdeograph6d47',
u'\u2a49': 'IntersectionAboveBarAboveUnion',
u'\uf74b': 'UnicodeCharacter0xf74b',
u'\ub44d': 'HangulSyllableDyot',
u'\u52ce': 'CjkUnifiedIdeograph52ce',
u'\u1fd0': 'GreekSmallLetterIotaWithVrachy',
u'\udcd2': 'UnicodeCharacter0xdcd2',
u'\u99d4': 'CjkUnifiedIdeograph99d4',
u'\u4557': 'CjkUnifiedIdeograph4557',
u'\ua6d6': 'UnicodeCharacter0xa6d6',
u'\u0259': 'LatinSmallLetterSchwa',
u'\ucf5b': 'HangulSyllableKod',
u'\u8c5d': 'CjkUnifiedIdeograph8c5d',
u'\u77e0': 'CjkUnifiedIdeograph77e0',
u'\u34e2': 'CjkUnifiedIdeograph34e2',
u'\uf1e4': 'UnicodeCharacter0xf1e4',
u'\u5d67': 'CjkUnifiedIdeograph5d67',
u'\ubee6': 'HangulSyllableBbeoj',
u'\u1a69': 'UnicodeCharacter0x1a69',
u'\u276b': 'MediumFlattenedRightParenthesisOrnament',
u'\ue46d': 'UnicodeCharacter0xe46d',
u'\ua16f': 'YiSyllableHniep',
u'\u4ff0': 'CjkUnifiedIdeograph4ff0',
u'\u0cf2': 'KannadaSignUpadhmaniya',
u'\uc9f4': 'HangulSyllableJjak',
u'\u96f6': 'CjkUnifiedIdeograph96f6',
u'\u7279': 'CjkUnifiedIdeograph7279',
u'\u3f7b': 'CjkUnifiedIdeograph3f7b',
u'\ufc7d': 'ArabicLigatureFehWithYehFinalForm',
u'\ub97f': 'HangulSyllableReulb',
u'\uc981': 'HangulSyllableJyung',
u'\u2e04': 'LeftDottedSubstitutionBracket',
u'\u0287': 'LatinSmallLetterTurnedT',
u'\ua408': 'YiSyllableQot',
u'\u988b': 'CjkUnifiedIdeograph988b',
u'\u670a': 'CjkUnifiedIdeograph670a',
u'\u5b8d': 'CjkUnifiedIdeograph5b8d',
u'\ufd0e': 'ArabicLigatureSeenWithRehIsolatedForm',
u'\ud191': 'HangulSyllableTyelt',
u'\u3614': 'CjkUnifiedIdeograph3614',
u'\u6a97': 'CjkUnifiedIdeograph6a97',
u'\u8c18': 'CjkUnifiedIdeograph8c18',
u'\ue09b': 'UnicodeCharacter0xe09b',
u'\u4f1a': 'CjkUnifiedIdeograph4f1a',
u'\uc51e': 'HangulSyllableSsyibs',
u'\u39a1': 'CjkUnifiedIdeograph39a1',
u'\ubfa5': 'HangulSyllableBboeb',
u'\u1e24': 'LatinCapitalLetterHWithDotBelow',
u'\u72a7': 'CjkUnifiedIdeograph72a7',
u'\u9428': 'CjkUnifiedIdeograph9428',
u'\uc8ab': 'HangulSyllableJwaegs',
u'\u572a': 'CjkUnifiedIdeograph572a',
u'\u2d2e': 'UnicodeCharacter0x2d2e',
u'\u01b1': 'LatinCapitalLetterUpsilon',
u'\ua332': 'YiSyllableSsax',
u'\u87b5': 'CjkUnifiedIdeograph87b5',
u'\u6634': 'CjkUnifiedIdeograph6634',
u'\u5ab7': 'CjkUnifiedIdeograph5ab7',
u'\ufc38': 'ArabicLigatureKafWithJeemIsolatedForm',
u'\ud0bb': 'HangulSyllableKic',
u'\u353e': 'CjkUnifiedIdeograph353e',
u'\u69c1': 'CjkUnifiedIdeograph69c1',
u'\u8b42': 'CjkUnifiedIdeograph8b42',
u'\uefc5': 'UnicodeCharacter0xefc5',
u'\u4e44': 'CjkUnifiedIdeograph4e44',
u'\uc448': 'HangulSyllableSsyo',
u'\u38cb': 'CjkUnifiedIdeograph38cb',
u'\ubecf': 'HangulSyllableBbyaeh',
u'\u1d4e': 'ModifierLetterSmallTurnedI',
u'\u71d1': 'CjkUnifiedIdeograph71d1',
u'\u9352': 'CjkUnifiedIdeograph9352',
u'\uf7d5': 'UnicodeCharacter0xf7d5',
u'\u5654': 'CjkUnifiedIdeograph5654',
u'\u2c58': 'GlagoliticSmallLetterBigYus',
u'\xdb': 'LatinCapitalLetterUWithCircumflex',
u'\ua25c': 'YiSyllableHxep',
u'\u86df': 'CjkUnifiedIdeograph86df',
u'\u655e': 'CjkUnifiedIdeograph655e',
u'\u59e1': 'CjkUnifiedIdeograph59e1',
u'\ufb62': 'ArabicLetterTehehIsolatedForm',
u'\udfe5': 'UnicodeCharacter0xdfe5',
u'\u3468': 'CjkUnifiedIdeograph3468',
u'\u68eb': 'CjkUnifiedIdeograph68eb',
u'\u8a6c': 'CjkUnifiedIdeograph8a6c',
u'\ueeef': 'UnicodeCharacter0xeeef',
u'\u4d6e': 'CjkUnifiedIdeograph4d6e',
u'\uc372': 'HangulSyllableSseolm',
u'\u27f5': 'LongLeftwardsArrow',
u'\ubdf9': 'HangulSyllableByulg',
u'\u1c78': 'OlChikiMuTtuddag',
u'\u70fb': 'CjkUnifiedIdeograph70fb',
u'\u927c': 'CjkUnifiedIdeograph927c',
u'\uf6ff': 'UnicodeCharacter0xf6ff',
u'\u557e': 'CjkUnifiedIdeograph557e',
u'\ucc03': 'HangulSyllableJjyis',
u'\u2b82': 'UnicodeCharacter0x2b82',
u'\ua186': 'YiSyllableNap',
u'\u0509': 'CyrillicSmallLetterKomiLje',
u'\u6488': 'CjkUnifiedIdeograph6488',
u'\u9b0d': 'CjkUnifiedIdeograph9b0d',
u'\ufa8c': 'CjkCompatibilityIdeographfa8c',
u'\u5e0f': 'CjkUnifiedIdeograph5e0f',
u'\ud413': 'HangulSyllablePwalb',
u'\u3392': 'SquareMhz',
u'\uaa17': 'ChamLetterNue',
u'\u8996': 'CjkUnifiedIdeograph8996',
u'\u6d19': 'CjkUnifiedIdeograph6d19',
u'\u4c98': 'CjkUnifiedIdeograph4c98',
u'\ue31d': 'UnicodeCharacter0xe31d',
u'\uc29c': 'HangulSyllableSyuss',
u'\u3c23': 'CjkUnifiedIdeograph3c23',
u'\u1ba2': 'SundaneseConsonantSignPanyakra',
u'\ub227': 'HangulSyllableNweod',
u'\u91a6': 'CjkUnifiedIdeograph91a6',
u'\u7529': 'CjkUnifiedIdeograph7529',
u'\u54a8': 'CjkUnifiedIdeograph54a8',
u'\ucb2d': 'HangulSyllableJjyog',
u'\u2aac': 'SmallerThanOrEqualTo',
u'\ua0b0': 'YiSyllableMi',
u'\u0433': 'CyrillicSmallLetterGhe',
u'\u63b2': 'CjkUnifiedIdeograph63b2',
u'\u9a37': 'CjkUnifiedIdeograph9a37',
u'\uf9b6': 'CjkCompatibilityIdeographf9b6',
u'\u5d39': 'CjkUnifiedIdeograph5d39',
u'\ud33d': 'HangulSyllablePaeng',
u'\u32bc': 'CircledNumberFortySeven',
u'\ua941': 'RejangLetterHa',
u'\u88c0': 'CjkUnifiedIdeograph88c0',
u'\u6c43': 'CjkUnifiedIdeograph6c43',
u'\u4bc2': 'CjkUnifiedIdeograph4bc2',
u'\ue247': 'UnicodeCharacter0xe247',
u'\uc1c6': 'HangulSyllableSwaegg',
u'\u3b4d': 'CjkUnifiedIdeograph3b4d',
u'\u1acc': 'UnicodeCharacter0x1acc',
u'\ub151': 'HangulSyllableNyeob',
u'\u90d0': 'CjkUnifiedIdeograph90d0',
u'\u7453': 'CjkUnifiedIdeograph7453',
u'\u53d2': 'CjkUnifiedIdeograph53d2',
u'\uca57': 'HangulSyllableJjeolb',
u'\u29d6': 'WhiteHourglass',
u'\uafda': 'HangulSyllableGgweonh',
u'\u035d': 'CombiningDoubleBreve',
u'\u62dc': 'CjkUnifiedIdeograph62dc',
u'\u9961': 'CjkUnifiedIdeograph9961',
u'\uf8e0': 'UnicodeCharacter0xf8e0',
u'\u5c63': 'CjkUnifiedIdeograph5c63',
u'\ud267': 'HangulSyllableTwegs',
u'\u31e6': 'UnicodeCharacter0x31e6',
u'\ua86b': 'PhagspaLetterDda',
u'\ub7ea': 'HangulSyllableRyaep',
u'\u6b6d': 'CjkUnifiedIdeograph6b6d',
u'\u4aec': 'CjkUnifiedIdeograph4aec',
u'\ue171': 'UnicodeCharacter0xe171',
u'\uc0f0': 'HangulSyllableSyals',
u'\u3a77': 'CjkUnifiedIdeograph3a77',
u'\u19f6': 'KhmerSymbolPrammuoyRoc',
u'\ub07b': 'HangulSyllableGgyih',
u'\u9ffa': 'UnicodeCharacter0x9ffa',
u'\u737d': 'CjkUnifiedIdeograph737d',
u'\u52fc': 'CjkUnifiedIdeograph52fc',
u'\u4881': 'CjkUnifiedIdeograph4881',
u'\u2900': 'RightwardsTwoheadedArrowWithVerticalStroke',
u'\u0d83': 'SinhalaSignVisargaya',
u'\uea02': 'UnicodeCharacter0xea02',
u'\uce85': 'HangulSyllableKab',
u'\uaf04': 'HangulSyllableGgyeom',
u'\u8387': 'CjkUnifiedIdeograph8387',
u'\u7c0e': 'CjkUnifiedIdeograph7c0e',
u'\u5091': 'CjkUnifiedIdeograph5091',
u'\u3110': 'BopomofoLetterJ',
u'\u1593': 'CanadianSyllabicsNga',
u'\uf212': 'UnicodeCharacter0xf212',
u'\ud695': 'HangulSyllableHoelg',
u'\ub714': 'HangulSyllableDdyul',
u'\ueb97': 'UnicodeCharacter0xeb97',
u'\uac99': 'HangulSyllableGelt',
u'\u441e': 'CjkUnifiedIdeograph441e',
u'\u1920': 'LimbuVowelSignA',
u'\u7da3': 'CjkUnifiedIdeograph7da3',
u'\uda22': 'UnicodeCharacter0xda22',
u'\u3ea5': 'CjkUnifiedIdeograph3ea5',
u'\u9f24': 'CjkUnifiedIdeograph9f24',
u'\uf3a7': 'UnicodeCharacter0xf3a7',
u'\ub4a9': 'HangulSyllableDwinj',
u'\u6130': 'CjkUnifiedIdeograph6130',
u'\u45b3': 'CjkUnifiedIdeograph45b3',
u'\u2232': 'ClockwiseContourIntegral',
u'\u06b5': 'ArabicLetterLamWithSmallV',
u'\ue734': 'UnicodeCharacter0xe734',
u'\udbb7': 'UnicodeCharacter0xdbb7',
u'\ub836': 'HangulSyllableRyeobs',
u'\u9cb9': 'CjkUnifiedIdeograph9cb9',
u'\u4940': 'CjkUnifiedIdeograph4940',
u'\u0a42': 'GurmukhiVowelSignUu',
u'\u6ec5': 'CjkUnifiedIdeograph6ec5',
u'\ucf44': 'HangulSyllableKyels',
u'\u23c7': 'DentistrySymbolLightDownAndHorizontalWithWave',
u'\u8046': 'CjkUnifiedIdeograph8046',
u'\ue4c9': 'UnicodeCharacter0xe4c9',
u'\ub9cb': 'HangulSyllableMags',
u'\u5150': 'CjkUnifiedIdeograph5150',
u'\u1252': 'EthiopicSyllableQhi',
u'\u76d5': 'CjkUnifiedIdeograph76d5',
u'\ud754': 'HangulSyllableHeun',
u'\u0bd7': 'TamilAuLengthMark',
u'\ue856': 'UnicodeCharacter0xe856',
u'\uccd9': 'HangulSyllableCyeolg',
u'\uad58': 'HangulSyllableGyol',
u'\u81db': 'CjkUnifiedIdeograph81db',
u'\u7a62': 'CjkUnifiedIdeograph7a62',
u'\u5ee5': 'CjkUnifiedIdeograph5ee5',
u'\u3f64': 'CjkUnifiedIdeograph3f64',
u'\u13e7': 'CherokeeLetterTsu',
u'\uf066': 'UnicodeCharacter0xf066',
u'\ud4e9': 'HangulSyllablePyug',
u'\ub568': 'HangulSyllableDdya',
u'\ue9eb': 'UnicodeCharacter0xe9eb',
u'\uaaed': 'UnicodeCharacter0xaaed',
u'\u4272': 'CjkUnifiedIdeograph4272',
u'\u0774': 'ArabicLetterAlefWithExtendedArabicindicDigitThreeAbove',
u'\u7bf7': 'CjkUnifiedIdeograph7bf7',
u'\ud876': 'UnicodeCharacter0xd876',
u'\u3cf9': 'CjkUnifiedIdeograph3cf9',
u'\u9d78': 'CjkUnifiedIdeograph9d78',
u'\uf1fb': 'UnicodeCharacter0xf1fb',
u'\ub2fd': 'HangulSyllableDat',
u'\u8801': 'CjkUnifiedIdeograph8801',
u'\u6f84': 'CjkUnifiedIdeograph6f84',
u'\u2086': 'SubscriptSix',
u'\ue588': 'UnicodeCharacter0xe588',
u'\u590b': 'CjkUnifiedIdeograph590b',
u'\ua68a': 'CyrillicCapitalLetterTeWithMiddleHook',
u'\u1a0d': 'BugineseLetterJa',
u'\udf0f': 'UnicodeCharacter0xdf0f',
u'\u9011': 'CjkUnifiedIdeograph9011',
u'\u7794': 'CjkUnifiedIdeograph7794',
u'\u0896': 'UnicodeCharacter0x896',
u'\ucd98': 'HangulSyllableCun',
u'\u8e9a': 'CjkUnifiedIdeograph8e9a',
u'\u621d': 'CjkUnifiedIdeograph621d',
u'\u271f': 'OutlinedLatinCross',
u'\uf821': 'UnicodeCharacter0xf821',
u'\ubd23': 'HangulSyllableBwas',
u'\u5fa4': 'CjkUnifiedIdeograph5fa4',
u'\u10a6': 'GeorgianCapitalLetterZen',
u'\ud5a8': 'HangulSyllableHyak',
u'\u96aa': 'CjkUnifiedIdeograph96aa',
u'\u4a2d': 'CjkUnifiedIdeograph4a2d',
u'\uabac': 'UnicodeCharacter0xabac',
u'\u0f2f': 'TibetanDigitHalfSix',
u'\uc031': 'HangulSyllableBbwib',
u'\u8533': 'CjkUnifiedIdeograph8533',
u'\u78b6': 'CjkUnifiedIdeograph78b6',
u'\u3db8': 'CjkUnifiedIdeograph3db8',
u'\ufeba': 'ArabicLetterSadFinalForm',
u'\u523d': 'CjkUnifiedIdeograph523d',
u'\ub3bc': 'HangulSyllableDyess',
u'\u173f': 'UnicodeCharacter0x173f',
u'\u2841': 'BraillePatternDots17',
u'\ued43': 'UnicodeCharacter0xed43',
u'\uae45': 'HangulSyllableGing',
u'\u40c6': 'CjkUnifiedIdeograph40c6',
u'\u05c8': 'UnicodeCharacter0x5c8',
u'\uc6ca': 'HangulSyllableUp',
u'\u9bcc': 'CjkUnifiedIdeograph9bcc',
u'\u7f4f': 'CjkUnifiedIdeograph7f4f',
u'\u3051': 'HiraganaLetterKe',
u'\uf553': 'UnicodeCharacter0xf553',
u'\ub655': 'HangulSyllableDdwaelt',
u'\u6dd8': 'CjkUnifiedIdeograph6dd8',
u'\u2eda': 'CjkRadicalCsimplifiedLeaf',
u'\ue3dc': 'UnicodeCharacter0xe3dc',
u'\u475f': 'CjkUnifiedIdeograph475f',
u'\ua4de': 'UnicodeCharacter0xa4de',
u'\u1861': 'MongolianLetterSibeU',
u'\udd63': 'UnicodeCharacter0xdd63',
u'\u9e65': 'CjkUnifiedIdeograph9e65',
u'\u75e8': 'CjkUnifiedIdeograph75e8',
u'\u36ea': 'CjkUnifiedIdeograph36ea',
u'\ucbec': 'HangulSyllableJjeuk',
u'\u8cee': 'CjkUnifiedIdeograph8cee',
u'\u6071': 'CjkUnifiedIdeograph6071',
u'\u2573': 'BoxDrawingsLightDiagonalCross',
u'\ue675': 'UnicodeCharacter0xe675',
u'\ubb77': 'HangulSyllableMwelb',
u'\u5df8': 'CjkUnifiedIdeograph5df8',
u'\u1efa': 'LatinCapitalLetterMiddlewelshLl',
u'\ud3fc': 'HangulSyllablePom',
u'\u94fe': 'CjkUnifiedIdeograph94fe',
u'\uaa00': 'ChamLetterA',
u'\u8e83': 'CjkUnifiedIdeograph8e83',
u'\u6902': 'CjkUnifiedIdeograph6902',
u'\u4d85': 'CjkUnifiedIdeograph4d85',
u'\ue306': 'UnicodeCharacter0xe306',
u'\uc789': 'HangulSyllableIng',
u'\u380c': 'CjkUnifiedIdeograph380c',
u'\u1c8f': 'UnicodeCharacter0x1c8f',
u'\ub210': 'HangulSyllableNuls',
u'\u9693': 'CjkUnifiedIdeograph9693',
u'\u7112': 'CjkUnifiedIdeograph7112',
u'\u5595': 'CjkUnifiedIdeograph5595',
u'\ucb16': 'HangulSyllableJjoenh',
u'\u2f99': 'KangxiRadicalShell',
u'\ua19d': 'YiSyllableHliep',
u'\u649f': 'CjkUnifiedIdeograph649f',
u'\u9a20': 'CjkUnifiedIdeograph9a20',
u'\ufea3': 'ArabicLetterHahInitialForm',
u'\u5922': 'CjkUnifiedIdeograph5922',
u'\ud326': 'HangulSyllablePap',
u'\u37a9': 'CjkUnifiedIdeograph37a9',
u'\u89ad': 'CjkUnifiedIdeograph89ad',
u'\u682c': 'CjkUnifiedIdeograph682c',
u'\u4caf': 'CjkUnifiedIdeograph4caf',
u'\ue230': 'UnicodeCharacter0xe230',
u'\uc6b3': 'HangulSyllableUgs',
u'\u3b36': 'CjkUnifiedIdeograph3b36',
u'\u1fb9': 'GreekCapitalLetterAlphaWithMacron',
u'\ubd3a': 'HangulSyllableBwaelp',
u'\u91bd': 'CjkUnifiedIdeograph91bd',
u'\u703c': 'CjkUnifiedIdeograph703c',
u'\u54bf': 'CjkUnifiedIdeograph54bf',
u'\uca40': 'HangulSyllableJjyaem',
u'\u2ec3': 'CjkRadicalWestOne',
u'\ua0c7': 'YiSyllableMurx',
u'\u0346': 'CombiningBridgeAbove',
u'\u67c9': 'CjkUnifiedIdeograph67c9',
u'\u854a': 'CjkUnifiedIdeograph854a',
u'\uf9cd': 'CjkCompatibilityIdeographf9cd',
u'\u584c': 'CjkUnifiedIdeograph584c',
u'\ud250': 'HangulSyllableTweol',
u'\u36d3': 'CjkUnifiedIdeograph36d3',
u'\u88d7': 'CjkUnifiedIdeograph88d7',
u'\u6b56': 'CjkUnifiedIdeograph6b56',
u'\u4fd9': 'CjkUnifiedIdeograph4fd9',
u'\ued5a': 'UnicodeCharacter0xed5a',
u'\uc1dd': 'HangulSyllableSwaet',
u'\u3a60': 'CjkUnifiedIdeograph3a60',
u'\u1ee3': 'LatinSmallLetterOWithHornAndDotBelow',
u'\ubc64': 'HangulSyllableByak',
u'\u90e7': 'CjkUnifiedIdeograph90e7',
u'\u7366': 'CjkUnifiedIdeograph7366',
u'\u57e9': 'CjkUnifiedIdeograph57e9',
u'\uf56a': 'UnicodeCharacter0xf56a',
u'\u29ed': 'BlackCircleWithDownArrow',
u'\ua3f1': 'YiSyllableJux',
u'\u0270': 'LatinSmallLetterTurnedMWithLongLeg',
u'\u66f3': 'CjkUnifiedIdeograph66f3',
u'\u8474': 'CjkUnifiedIdeograph8474',
u'\uf8f7': 'UnicodeCharacter0xf8f7',
u'\u5b76': 'CjkUnifiedIdeograph5b76',
u'\udd7a': 'UnicodeCharacter0xdd7a',
u'\u31fd': 'KatakanaLetterSmallRu',
u'\u0b01': 'OriyaSignCandrabindu',
u'\u6a80': 'CjkUnifiedIdeograph6a80',
u'\u8d05': 'CjkUnifiedIdeograph8d05',
u'\uec84': 'UnicodeCharacter0xec84',
u'\u4007': 'CjkUnifiedIdeograph4007',
u'\uda0b': 'UnicodeCharacter0xda0b',
u'\u258a': 'LeftThreeQuartersBlock',
u'\ubf8e': 'HangulSyllableBbwaej',
u'\u1311': 'UnicodeCharacter0x1311',
u'\u7290': 'CjkUnifiedIdeograph7290',
u'\u9515': 'CjkUnifiedIdeograph9515',
u'\uf494': 'UnicodeCharacter0xf494',
u'\u221b': 'CubeRoot',
u'\u0d9a': 'SinhalaLetterAlpapraanaKayanna',
u'\ua41f': 'YiSyllableJjiep',
u'\u879e': 'CjkUnifiedIdeograph879e',
u'\u7b21': 'CjkUnifiedIdeograph7b21',
u'\u5aa0': 'CjkUnifiedIdeograph5aa0',
u'\ufd25': 'ArabicLigatureSheenWithJeemFinalForm',
u'\udca4': 'UnicodeCharacter0xdca4',
u'\u0a2b': 'GurmukhiLetterPha',
u'\u15aa': 'CanadianSyllabicsThcreeTho',
u'\u8c2f': 'CjkUnifiedIdeograph8c2f',
u'\uefae': 'UnicodeCharacter0xefae',
u'\u4331': 'CjkUnifiedIdeograph4331',
u'\uc535': 'HangulSyllableSsilt',
u'\u24b4': 'ParenthesizedLatinSmallLetterY',
u'\ubeb8': 'HangulSyllableBbyaen',
u'\u123b': 'EthiopicSyllableShaa',
u'\u7dba': 'CjkUnifiedIdeograph7dba',
u'\u943f': 'CjkUnifiedIdeograph943f',
u'\uf7be': 'UnicodeCharacter0xf7be',
u'\u2d45': 'TifinaghLetterYakh',
u'\u0cc4': 'KannadaVowelSignVocalicRr',
u'\ua749': 'LatinSmallLetterLWithHighStroke',
u'\u86c8': 'CjkUnifiedIdeograph86c8',
u'\u7a4b': 'CjkUnifiedIdeograph7a4b',
u'\u45ca': 'CjkUnifiedIdeograph45ca',
u'\ufc4f': 'ArabicLigatureNoonWithAlefMaksuraIsolatedForm',
u'\udfce': 'UnicodeCharacter0xdfce',
u'\u3555': 'CjkUnifiedIdeograph3555',
u'\u14d4': 'CanadianSyllabicsLaai',
u'\u8f59': 'CjkUnifiedIdeograph8f59',
u'\ueed8': 'UnicodeCharacter0xeed8',
u'\u425b': 'CjkUnifiedIdeograph425b',
u'\uc45f': 'HangulSyllableSsyoc',
u'\u27de': 'LongLeftTack',
u'\ub9e2': 'HangulSyllableMap',
u'\u1d65': 'LatinSubscriptSmallLetterV',
u'\u7ce4': 'CjkUnifiedIdeograph7ce4',
u'\u9769': 'CjkUnifiedIdeograph9769',
u'\uf6e8': 'UnicodeCharacter0xf6e8',
u'\u2c6f': 'LatinCapitalLetterTurnedA',
u'\u0fee': 'UnicodeCharacter0xfee',
u'\ua673': 'SlavonicAsterisk',
u'\u81f2': 'CjkUnifiedIdeograph81f2',
u'\u6575': 'CjkUnifiedIdeograph6575',
u'\u44f4': 'CjkUnifiedIdeograph44f4',
u'\uff79': 'HalfwidthKatakanaLetterKe',
u'\udef8': 'UnicodeCharacter0xdef8',
u'\u347f': 'CjkUnifiedIdeograph347f',
u'\u17fe': 'UnicodeCharacter0x17fe',
u'\u6206': 'CjkUnifiedIdeograph6206',
u'\u4689': 'CjkUnifiedIdeograph4689',
u'\u2708': 'Airplane',
u'\u1b8b': 'SundaneseLetterQa',
u'\ue40a': 'UnicodeCharacter0xe40a',
u'\ud88d': 'UnicodeCharacter0xd88d',
u'\ub90c': 'HangulSyllableRweol',
u'\u9d8f': 'CjkUnifiedIdeograph9d8f',
u'\u4a16': 'CjkUnifiedIdeograph4a16',
u'\u0f18': 'TibetanAstrologicalSignKhyudPa',
u'\u639b': 'CjkUnifiedIdeograph639b',
u'\ucc1a': 'HangulSyllableJjilp',
u'\u209d': 'UnicodeCharacter0x209d',
u'\u811c': 'CjkUnifiedIdeograph811c',
u'\ue59f': 'UnicodeCharacter0xe59f',
u'\ubaa1': 'HangulSyllableMyeng',
u'\u5226': 'CjkUnifiedIdeograph5226',
u'\u1728': 'HanunooLetterNa',
u'\u4bab': 'CjkUnifiedIdeograph4bab',
u'\ud42a': 'HangulSyllablePwaenh',
u'\u08ad': 'UnicodeCharacter0x8ad',
u'\ue92c': 'UnicodeCharacter0xe92c',
u'\ucdaf': 'HangulSyllableCuh',
u'\uae2e': 'HangulSyllableGyip',
u'\u82b1': 'CjkUnifiedIdeograph82b1',
u'\u7f38': 'CjkUnifiedIdeograph7f38',
u'\u53bb': 'CjkUnifiedIdeograph53bb',
u'\u3c3a': 'CjkUnifiedIdeograph3c3a',
u'\u10bd': 'GeorgianCapitalLetterChar',
u'\uf13c': 'UnicodeCharacter0xf13c',
u'\ud5bf': 'HangulSyllableHyaes',
u'\ub63e': 'HangulSyllableDdwabs',
u'\ueac1': 'UnicodeCharacter0xeac1',
u'\uafc3': 'HangulSyllableGgulb',
u'\u4748': 'CjkUnifiedIdeograph4748',
u'\u044a': 'CyrillicSmallLetterHardSign',
u'\u78cd': 'CjkUnifiedIdeograph78cd',
u'\ud94c': 'UnicodeCharacter0xd94c',
u'\u3dcf': 'CjkUnifiedIdeograph3dcf',
u'\u9e4e': 'CjkUnifiedIdeograph9e4e',
u'\uf2d1': 'UnicodeCharacter0xf2d1',
u'\ub7d3': 'HangulSyllableRyaegs',
u'\u6c5a': 'CjkUnifiedIdeograph6c5a',
u'\u40dd': 'CjkUnifiedIdeograph40dd',
u'\u215c': 'VulgarFractionThreeEighths',
u'\u05df': 'HebrewLetterFinalNun',
u'\ue65e': 'UnicodeCharacter0xe65e',
u'\udae1': 'UnicodeCharacter0xdae1',
u'\ubb60': 'HangulSyllableMweom',
u'\u9fe3': 'UnicodeCharacter0x9fe3',
u'\u746a': 'CjkUnifiedIdeograph746a',
u'\u096c': 'DevanagariDigitSix',
u'\u6def': 'CjkUnifiedIdeograph6def',
u'\uce6e': 'HangulSyllableCij',
u'\u22f1': 'DownRightDiagonalEllipsis',
u'\u8370': 'CjkUnifiedIdeograph8370',
u'\ue7f3': 'UnicodeCharacter0xe7f3',
u'\ua4f5': 'UnicodeCharacter0xa4f5',
u'\u5c7a': 'CjkUnifiedIdeograph5c7a',
u'\u117c': 'HangulJungseongEoeu',
u'\u75ff': 'CjkUnifiedIdeograph75ff',
u'\ud67e': 'HangulSyllableHwaelp',
u'\ueb80': 'UnicodeCharacter0xeb80',
u'\u4f03': 'CjkUnifiedIdeograph4f03',
u'\ua882': 'SaurashtraLetterA',
u'\u0c05': 'TeluguLetterA',
u'\uc107': 'HangulSyllableSyaed',
u'\u8609': 'CjkUnifiedIdeograph8609',
u'\u798c': 'CjkUnifiedIdeograph798c',
u'\u3e8e': 'CjkUnifiedIdeograph3e8e',
u'\uf390': 'UnicodeCharacter0xf390',
u'\u5713': 'CjkUnifiedIdeograph5713',
u'\ub092': 'HangulSyllableGgij',
u'\u1415': 'CanadianSyllabicsWestcreeWoo',
u'\u2917': 'RightwardsTwoheadedArrowWithTailWithVerticalStroke',
u'\uee19': 'UnicodeCharacter0xee19',
u'\ua31b': 'YiSyllableSex',
u'\u419c': 'CjkUnifiedIdeograph419c',
u'\u069e': 'ArabicLetterSadWithThreeDotsAbove',
u'\udba0': 'UnicodeCharacter0xdba0',
u'\u98a2': 'CjkUnifiedIdeograph98a2',
u'\u7c25': 'CjkUnifiedIdeograph7c25',
u'\u3127': 'BopomofoLetterI',
u'\uf629': 'UnicodeCharacter0xf629',
u'\u8b2b': 'CjkUnifiedIdeograph8b2b',
u'\u6eae': 'CjkUnifiedIdeograph6eae',
u'\u23b0': 'UpperLeftOrLowerRightCurlyBracketSection',
u'\ue0b2': 'UnicodeCharacter0xe0b2',
u'\u4435': 'CjkUnifiedIdeograph4435',
u'\ua5b4': 'VaiSyllableKu',
u'\u1937': 'LimbuSmallLetterRa',
u'\ude39': 'UnicodeCharacter0xde39',
u'\u933b': 'CjkUnifiedIdeograph933b',
u'\u76be': 'CjkUnifiedIdeograph76be',
u'\u0bc0': 'TamilVowelSignIi',
u'\uc8c2': 'HangulSyllableJwaep',
u'\u8dc4': 'CjkUnifiedIdeograph8dc4',
u'\u6147': 'CjkUnifiedIdeograph6147',
u'\u2649': 'Taurus',
u'\ufb4b': 'HebrewLetterVavWithHolam',
u'\ub84d': 'HangulSyllableRyelt',
u'\u5ece': 'CjkUnifiedIdeograph5ece',
u'\u13d0': 'CherokeeLetterSo',
u'\ud0d2': 'HangulSyllableTabs',
u'\u95d4': 'CjkUnifiedIdeograph95d4',
u'\u4957': 'CjkUnifiedIdeograph4957',
u'\uaad6': 'UnicodeCharacter0xaad6',
u'\u0e59': 'ThaiDigitNine',
u'\uc35b': 'HangulSyllableSsyaelh',
u'\u805d': 'CjkUnifiedIdeograph805d',
u'\u7be0': 'CjkUnifiedIdeograph7be0',
u'\u38e2': 'CjkUnifiedIdeograph38e2',
u'\ufde4': 'UnicodeCharacter0xfde4',
u'\u5167': 'CjkUnifiedIdeograph5167',
u'\ub2e6': 'HangulSyllableDagg',
u'\u1669': 'CanadianSyllabicsCarrierTtse',
u'\u2b6b': 'UnicodeCharacter0x2b6b',
u'\ue86d': 'UnicodeCharacter0xe86d',
u'\uad6f': 'HangulSyllableGugs',
u'\u43f0': 'CjkUnifiedIdeograph43f0',
u'\xf2': 'LatinSmallLetterOWithGrave',
u'\uc5f4': 'HangulSyllableYeol',
u'\u9af6': 'CjkUnifiedIdeograph9af6',
u'\u7e79': 'CjkUnifiedIdeograph7e79',
u'\u337b': 'SquareEraNameHeisei',
u'\uf07d': 'UnicodeCharacter0xf07d',
u'\ub57f': 'HangulSyllableDdyac',
u'\ucd81': 'HangulSyllableCyolg',
u'\u2a04': 'NaryUnionOperatorWithPlus',
u'\u0687': 'ArabicLetterTcheheh',
u'\ua008': 'YiSyllableAt',
u'\u9c8b': 'CjkUnifiedIdeograph9c8b',
u'\u630a': 'CjkUnifiedIdeograph630a',
u'\u5f8d': 'CjkUnifiedIdeograph5f8d',
u'\uf90e': 'CjkCompatibilityIdeographf90e',
u'\ud591': 'HangulSyllableHyag',
u'\u3214': 'ParenthesizedHangulSiosA',
u'\u6e97': 'CjkUnifiedIdeograph6e97',
u'\u8818': 'CjkUnifiedIdeograph8818',
u'\ue49b': 'UnicodeCharacter0xe49b',
u'\u4b1a': 'CjkUnifiedIdeograph4b1a',
u'\uc11e': 'HangulSyllableSeogg',
u'\u3da1': 'CjkUnifiedIdeograph3da1',
u'\ubba5': 'HangulSyllableMyug',
u'\u1a24': 'UnicodeCharacter0x1a24',
u'\u76a7': 'CjkUnifiedIdeograph76a7',
u'\u9028': 'CjkUnifiedIdeograph9028',
u'\uccab': 'HangulSyllableCeos',
u'\u532a': 'CjkUnifiedIdeograph532a',
u'\u292e': 'NorthEastArrowCrossingSouthEastArrow',
u'\u05b1': 'HebrewPointHatafSegol',
u'\ua732': 'LatinCapitalLetterAa',
u'\u83b5': 'CjkUnifiedIdeograph83b5',
u'\u6234': 'CjkUnifiedIdeograph6234',
u'\u5eb7': 'CjkUnifiedIdeograph5eb7',
u'\uf838': 'UnicodeCharacter0xf838',
u'\ud4bb': 'HangulSyllablePwelb',
u'\u313e': 'HangulLetterRieulthieuth',
u'\u6dc1': 'CjkUnifiedIdeograph6dc1',
u'\u8f42': 'CjkUnifiedIdeograph8f42',
u'\uebc5': 'UnicodeCharacter0xebc5',
u'\u4a44': 'CjkUnifiedIdeograph4a44',
u'\uc048': 'HangulSyllableBbyuls',
u'\u3ccb': 'CjkUnifiedIdeograph3ccb',
u'\ubacf': 'HangulSyllableMwalb',
u'\u194e': 'LimbuDigitEight',
u'\u75d1': 'CjkUnifiedIdeograph75d1',
u'\u9752': 'CjkUnifiedIdeograph9752',
u'\uf3d5': 'UnicodeCharacter0xf3d5',
u'\u5254': 'CjkUnifiedIdeograph5254',
u'\u2858': 'BraillePatternDots457',
u'\u04db': 'CyrillicSmallLetterSchwaWithDiaeresis',
u'\ua65c': 'CyrillicCapitalLetterIotifiedClosedLittleYus',
u'\u82df': 'CjkUnifiedIdeograph82df',
u'\u615e': 'CjkUnifiedIdeograph615e',
u'\u5de1': 'CjkUnifiedIdeograph5de1',
u'\uff62': 'HalfwidthLeftCornerBracket',
u'\udbe5': 'UnicodeCharacter0xdbe5',
u'\u3068': 'HiraganaLetterTo',
u'\u6ceb': 'CjkUnifiedIdeograph6ceb',
u'\u8e6c': 'CjkUnifiedIdeograph8e6c',
u'\ueaef': 'UnicodeCharacter0xeaef',
u'\u496e': 'CjkUnifiedIdeograph496e',
u'\uc772': 'HangulSyllableYip',
u'\u23f5': 'UnicodeCharacter0x23f5',
u'\ub9f9': 'HangulSyllableMaeng',
u'\u1878': 'UnicodeCharacter0x1878',
u'\u74fb': 'CjkUnifiedIdeograph74fb',
u'\u967c': 'CjkUnifiedIdeograph967c',
u'\uf2ff': 'UnicodeCharacter0xf2ff',
u'\u517e': 'CjkUnifiedIdeograph517e',
u'\uc803': 'HangulSyllableJeogs',
u'\u2f82': 'KangxiRadicalMinister',
u'\ua586': 'VaiSyllableNdoo',
u'\u0109': 'LatinSmallLetterCWithCircumflex',
u'\u6088': 'CjkUnifiedIdeograph6088',
u'\u9f0d': 'CjkUnifiedIdeograph9f0d',
u'\ufe8c': 'ArabicLetterYehWithHamzaAboveMedialForm',
u'\u5a0f': 'CjkUnifiedIdeograph5a0f',
u'\ud013': 'HangulSyllableKweoc',
u'\u3792': 'CjkUnifiedIdeograph3792',
u'\uae17': 'HangulSyllableGyigs',
u'\u8d96': 'CjkUnifiedIdeograph8d96',
u'\u6919': 'CjkUnifiedIdeograph6919',
u'\u4898': 'CjkUnifiedIdeograph4898',
u'\ue71d': 'UnicodeCharacter0xe71d',
u'\uc69c': 'HangulSyllableYol',
u'\u3823': 'CjkUnifiedIdeograph3823',
u'\u1fa2': 'GreekSmallLetterOmegaWithPsiliAndVariaAndYpogegrammeni',
u'\ub627': 'HangulSyllableDdoc',
u'\u95a6': 'CjkUnifiedIdeograph95a6',
u'\u7129': 'CjkUnifiedIdeograph7129',
u'\u50a8': 'CjkUnifiedIdeograph50a8',
u'\ucf2d': 'HangulSyllableKyeob',
u'\u2eac': 'CjkRadicalSpiritOne',
u'\ua4b0': 'YiRadicalShy',
u'\u67b2': 'CjkUnifiedIdeograph67b2',
u'\u9e37': 'CjkUnifiedIdeograph9e37',
u'\ufdb6': 'ArabicLigatureAinWithMeemWithYehFinalForm',
u'\u5939': 'CjkUnifiedIdeograph5939',
u'\ud73d': 'HangulSyllableHyulg',
u'\u36bc': 'CjkUnifiedIdeograph36bc',
u'\uad41': 'HangulSyllableGoelt',
u'\u8cc0': 'CjkUnifiedIdeograph8cc0',
u'\u6843': 'CjkUnifiedIdeograph6843',
u'\u4fc2': 'CjkUnifiedIdeograph4fc2',
u'\ue647': 'UnicodeCharacter0xe647',
u'\uc5c6': 'HangulSyllableEobs',
u'\u3f4d': 'CjkUnifiedIdeograph3f4d',
u'\u1ecc': 'LatinCapitalLetterOWithDotBelow',
u'\ub551': 'HangulSyllableDdaenj',
u'\u94d0': 'CjkUnifiedIdeograph94d0',
u'\u7053': 'CjkUnifiedIdeograph7053',
u'\u57d2': 'CjkUnifiedIdeograph57d2',
u'\uce57': 'HangulSyllableCyih',
u'\u2dd6': 'EthiopicSyllableXyo',
u'\uabda': 'UnicodeCharacter0xabda',
u'\u075d': 'ArabicLetterAinWithTwoDotsAbove',
u'\u66dc': 'CjkUnifiedIdeograph66dc',
u'\u9d61': 'CjkUnifiedIdeograph9d61',
u'\ufce0': 'ArabicLigatureYehWithHamzaAboveWithHehMedialForm',
u'\u5863': 'CjkUnifiedIdeograph5863',
u'\ud667': 'HangulSyllableHwas',
u'\u35e6': 'CjkUnifiedIdeograph35e6',
u'\uac6b': 'HangulSyllableGyaec',
u'\ub3ea': 'HangulSyllableDwalm',
u'\u6f6d': 'CjkUnifiedIdeograph6f6d',
u'\u4eec': 'CjkUnifiedIdeograph4eec',
u'\ue571': 'UnicodeCharacter0xe571',
u'\uc4f0': 'HangulSyllableSseu',
u'\u3e77': 'CjkUnifiedIdeograph3e77',
u'\u1df6': 'UnicodeCharacter0x1df6',
u'\ub47b': 'HangulSyllableDweolh',
u'\u9bfa': 'CjkUnifiedIdeograph9bfa',
u'\u777d': 'CjkUnifiedIdeograph777d',
u'\u56fc': 'CjkUnifiedIdeograph56fc',
u'\u4c81': 'CjkUnifiedIdeograph4c81',
u'\u2d00': 'GeorgianSmallLetterAn',
u'\u0983': 'BengaliSignVisarga',
u'\uee02': 'UnicodeCharacter0xee02',
u'\uca85': 'HangulSyllableJjyeog',
u'\uab04': 'UnicodeCharacter0xab04',
u'\u8787': 'CjkUnifiedIdeograph8787',
u'\u780e': 'CjkUnifiedIdeograph780e',
u'\u5491': 'CjkUnifiedIdeograph5491',
u'\u3510': 'CjkUnifiedIdeograph3510',
u'\u1193': 'HangulJungseongYuu',
u'\uf612': 'UnicodeCharacter0xf612',
u'\ud295': 'HangulSyllableTwing',
u'\ub314': 'HangulSyllableDaess',
u'\uef97': 'UnicodeCharacter0xef97',
u'\ua899': 'SaurashtraLetterJa',
u'\u401e': 'CjkUnifiedIdeograph401e',
u'\u1d20': 'LatinLetterSmallCapitalV',
u'\u79a3': 'CjkUnifiedIdeograph79a3',
u'\ude22': 'UnicodeCharacter0xde22',
u'\u3aa5': 'CjkUnifiedIdeograph3aa5',
u'\u9b24': 'CjkUnifiedIdeograph9b24',
u'\uf7a7': 'UnicodeCharacter0xf7a7',
u'\ub0a9': 'HangulSyllableNab',
u'\u6530': 'CjkUnifiedIdeograph6530',
u'\u41b3': 'CjkUnifiedIdeograph41b3',
u'\u2632': 'TrigramForFire',
u'\u02b5': 'ModifierLetterSmallTurnedRWithHook',
u'\ue334': 'UnicodeCharacter0xe334',
u'\udfb7': 'UnicodeCharacter0xdfb7',
u'\ubc36': 'HangulSyllableBaenh',
u'\u98b9': 'CjkUnifiedIdeograph98b9',
u'\u4d40': 'CjkUnifiedIdeograph4d40',
u'\u0e42': 'ThaiCharacterSaraO',
u'\u6ac5': 'CjkUnifiedIdeograph6ac5',
u'\ucb44': 'HangulSyllableJjyok',
u'\u27c7': 'OrWithDotInside',
u'\u8446': 'CjkUnifiedIdeograph8446',
u'\ue0c9': 'UnicodeCharacter0xe0c9',
u'\ubdcb': 'HangulSyllableBwes',
u'\u5550': 'CjkUnifiedIdeograph5550',
u'\u1652': 'CanadianSyllabicsCarrierSi',
u'\u72d5': 'CjkUnifiedIdeograph72d5',
u'\ud354': 'HangulSyllablePyam',
u'\u0fd7': 'UnicodeCharacter0xfd7',
u'\uec56': 'UnicodeCharacter0xec56',
u'\uc8d9': 'HangulSyllableJoeng',
u'\ua958': 'UnicodeCharacter0xa958',
u'\u85db': 'CjkUnifiedIdeograph85db',
u'\u7e62': 'CjkUnifiedIdeograph7e62',
u'\u5ae5': 'CjkUnifiedIdeograph5ae5',
u'\u3b64': 'CjkUnifiedIdeograph3b64',
u'\u17e7': 'KhmerDigitSeven',
u'\uf466': 'UnicodeCharacter0xf466',
u'\ud0e9': 'HangulSyllableTaelt',
u'\ub168': 'HangulSyllableNyels',
u'\uedeb': 'UnicodeCharacter0xedeb',
u'\uaeed': 'HangulSyllableGgeng',
u'\u4672': 'CjkUnifiedIdeograph4672',
u'\u0374': 'GreekNumeralSign',
u'\u7ff7': 'CjkUnifiedIdeograph7ff7',
u'\udc76': 'UnicodeCharacter0xdc76',
u'\u38f9': 'CjkUnifiedIdeograph38f9',
u'\u9978': 'CjkUnifiedIdeograph9978',
u'\uf5fb': 'UnicodeCharacter0xf5fb',
u'\ub6fd': 'HangulSyllableDdwilt',
u'\u8c01': 'CjkUnifiedIdeograph8c01',
u'\u6b84': 'CjkUnifiedIdeograph6b84',
u'\u2486': 'ParenthesizedNumberNineteen',
u'\ue188': 'UnicodeCharacter0xe188',
u'\u5d0b': 'CjkUnifiedIdeograph5d0b',
u'\ua28a': 'YiSyllableWex',
u'\u1e0d': 'LatinSmallLetterDWithDotBelow',
u'\udb0f': 'UnicodeCharacter0xdb0f',
u'\u9411': 'CjkUnifiedIdeograph9411',
u'\u7394': 'CjkUnifiedIdeograph7394',
u'\u0c96': 'KannadaLetterKha',
u'\uc998': 'HangulSyllableJeum',
u'\u8a9a': 'CjkUnifiedIdeograph8a9a',
u'\u661d': 'CjkUnifiedIdeograph661d',
u'\u231f': 'BottomRightCorner',
u'\ufc21': 'ArabicLigatureSadWithMeemIsolatedForm',
u'\ub923': 'HangulSyllableRwegs',
u'\u5ba4': 'CjkUnifiedIdeograph5ba4',
u'\u14a6': 'CanadianSyllabicsMii',
u'\ud1a8': 'HangulSyllableTol',
u'\u92aa': 'CjkUnifiedIdeograph92aa',
u'\u4e2d': 'CjkUnifiedIdeograph4e2d',
u'\uafac': 'HangulSyllableGgyom',
u'\u0b2f': 'OriyaLetterYa',
u'\uc431': 'HangulSyllableSsoenj',
u'\u8133': 'CjkUnifiedIdeograph8133',
u'\u7cb6': 'CjkUnifiedIdeograph7cb6',
u'\u39b8': 'CjkUnifiedIdeograph39b8',
u'\ufaba': 'CjkCompatibilityIdeographfaba',
u'\u563d': 'CjkUnifiedIdeograph563d',
u'\ub7bc': 'HangulSyllableRyal',
u'\u133f': 'EthiopicSyllableTswa',
u'\u2c41': 'GlagoliticSmallLetterOnu',
u'\ue943': 'UnicodeCharacter0xe943',
u'\uaa45': 'ChamLetterFinalT',
u'\u44c6': 'CjkUnifiedIdeograph44c6',
u'\u01c8': 'LatinCapitalLetterLWithSmallLetterJ',
u'\uc2ca': 'HangulSyllableSyilm',
u'\u9fcc': 'UnicodeCharacter0x9fcc',
u'\u7b4f': 'CjkUnifiedIdeograph7b4f',
u'\u3451': 'CjkUnifiedIdeograph3451',
u'\uf153': 'UnicodeCharacter0xf153',
u'\ub255': 'HangulSyllableNwet',
u'\u69d8': 'CjkUnifiedIdeograph69d8',
u'\u2ada': 'PitchforkWithTeeTop',
u'\ue7dc': 'UnicodeCharacter0xe7dc',
u'\u435f': 'CjkUnifiedIdeograph435f',
u'\ua0de': 'YiSyllableFyt',
u'\u1c61': 'OlChikiLetterAaj',
u'\ud963': 'UnicodeCharacter0xd963',
u'\u9a65': 'CjkUnifiedIdeograph9a65',
u'\u71e8': 'CjkUnifiedIdeograph71e8',
u'\u32ea': 'CircledKatakanaHi',
u'\ucfec': 'HangulSyllableKuls',
u'\u88ee': 'CjkUnifiedIdeograph88ee',
u'\u6471': 'CjkUnifiedIdeograph6471',
u'\u2173': 'SmallRomanNumeralFour',
u'\ue275': 'UnicodeCharacter0xe275',
u'\ubf77': 'HangulSyllableBbwah',
u'\u59f8': 'CjkUnifiedIdeograph59f8',
u'\u1afa': 'UnicodeCharacter0x1afa',
u'\ud7fc': 'UnicodeCharacter0xd7fc',
u'\u90fe': 'CjkUnifiedIdeograph90fe',
u'\uae00': 'HangulSyllableGeul',
u'\u8a83': 'CjkUnifiedIdeograph8a83',
u'\u6d02': 'CjkUnifiedIdeograph6d02',
u'\u4985': 'CjkUnifiedIdeograph4985',
u'\ue706': 'UnicodeCharacter0xe706',
u'\uc389': 'HangulSyllableSsenj',
u'\u3c0c': 'CjkUnifiedIdeograph3c0c',
u'\u188f': 'MongolianLetterAliGaliNna',
u'\ub610': 'HangulSyllableDdo',
u'\u9293': 'CjkUnifiedIdeograph9293',
u'\u7512': 'CjkUnifiedIdeograph7512',
u'\u5195': 'CjkUnifiedIdeograph5195',
u'\ucf16': 'HangulSyllableKej',
u'\u2b99': 'UnicodeCharacter0x2b99',
u'\ua59d': 'VaiSyllableBu',
u'\u041c': 'CyrillicCapitalLetterEm',
u'\u609f': 'CjkUnifiedIdeograph609f',
u'\u9e20': 'CjkUnifiedIdeograph9e20',
u'\ufaa3': 'CjkCompatibilityIdeographfaa3',
u'\u5d22': 'CjkUnifiedIdeograph5d22',
u'\ud726': 'HangulSyllableHwilp',
u'\u33a9': 'SquarePa',
u'\u8dad': 'CjkUnifiedIdeograph8dad',
u'\u6c2c': 'CjkUnifiedIdeograph6c2c',
u'\u48af': 'CjkUnifiedIdeograph48af',
u'\ue630': 'UnicodeCharacter0xe630',
u'\uc2b3': 'HangulSyllableSeulh',
u'\u3f36': 'CjkUnifiedIdeograph3f36',
u'\u1bb9': 'SundaneseDigitNine',
u'\ub93a': 'HangulSyllableRwep',
u'\u95bd': 'CjkUnifiedIdeograph95bd',
u'\u743c': 'CjkUnifiedIdeograph743c',
u'\u50bf': 'CjkUnifiedIdeograph50bf',
u'\uce40': 'HangulSyllableCyin',
u'\u2ac3': 'SubsetOfOrEqualToWithDotAbove',
u'\ua4c7': 'UnicodeCharacter0xa4c7',
u'\u0746': 'SyriacThreeDotsBelow',
u'\u63c9': 'CjkUnifiedIdeograph63c9',
u'\u814a': 'CjkUnifiedIdeograph814a',
u'\ufdcd': 'UnicodeCharacter0xfdcd',
u'\u5c4c': 'CjkUnifiedIdeograph5c4c',
u'\ud650': 'HangulSyllableHok',
u'\u32d3': 'CircledKatakanaE',
u'\u8cd7': 'CjkUnifiedIdeograph8cd7',
u'\u6f56': 'CjkUnifiedIdeograph6f56',
u'\u4bd9': 'CjkUnifiedIdeograph4bd9',
u'\ue95a': 'UnicodeCharacter0xe95a',
u'\uc5dd': 'HangulSyllableElt',
u'\u3e60': 'CjkUnifiedIdeograph3e60',
u'\u1ae3': 'UnicodeCharacter0x1ae3',
u'\ub864': 'HangulSyllableRol',
u'\u94e7': 'CjkUnifiedIdeograph94e7',
u'\u7766': 'CjkUnifiedIdeograph7766',
u'\u53e9': 'CjkUnifiedIdeograph53e9',
u'\uf16a': 'UnicodeCharacter0xf16a',
u'\u2ded': 'CombiningCyrillicLetterEs',
u'\ua7f1': 'UnicodeCharacter0xa7f1',
u'\u0670': 'ArabicLetterSuperscriptAlef',
u'\u62f3': 'CjkUnifiedIdeograph62f3',
u'\u8074': 'CjkUnifiedIdeograph8074',
u'\ufcf7': 'ArabicLigatureAinWithAlefMaksuraIsolatedForm',
u'\u5f76': 'CjkUnifiedIdeograph5f76',
u'\ud97a': 'UnicodeCharacter0xd97a',
u'\u35fd': 'CjkUnifiedIdeograph35fd',
u'\u0f01': 'TibetanMarkGterYigMgoTruncatedA',
u'\u6e80': 'CjkUnifiedIdeograph6e80',
u'\u8905': 'CjkUnifiedIdeograph8905',
u'\ue884': 'UnicodeCharacter0xe884',
u'\u4407': 'CjkUnifiedIdeograph4407',
u'\ude0b': 'UnicodeCharacter0xde0b',
u'\u218a': 'UnicodeCharacter0x218a',
u'\ubb8e': 'HangulSyllableMwinh',
u'\u1711': 'TagalogLetterHa',
u'\u7690': 'CjkUnifiedIdeograph7690',
u'\u9115': 'CjkUnifiedIdeograph9115',
u'\uf094': 'UnicodeCharacter0xf094',
u'\u261b': 'BlackRightPointingIndex',
u'\u099a': 'BengaliLetterCa',
u'\ua01f': 'YiSyllableBax',
u'\u839e': 'CjkUnifiedIdeograph839e',
u'\u7f21': 'CjkUnifiedIdeograph7f21',
u'\u5ea0': 'CjkUnifiedIdeograph5ea0',
u'\uf925': 'CjkCompatibilityIdeographf925',
u'\ud8a4': 'UnicodeCharacter0xd8a4',
u'\u0e2b': 'ThaiCharacterHoHip',
u'\u11aa': 'HangulJongseongKiyeoksios',
u'\u882f': 'CjkUnifiedIdeograph882f',
u'\uebae': 'UnicodeCharacter0xebae',
u'\u4731': 'CjkUnifiedIdeograph4731',
u'\uc135': 'HangulSyllableSeot',
u'\u20b4': 'HryvniaSign',
u'\ubab8': 'HangulSyllableMom',
u'\u163b': 'CanadianSyllabicsCarrierTlo',
u'\u79ba': 'CjkUnifiedIdeograph79ba',
u'\u903f': 'CjkUnifiedIdeograph903f',
u'\uf3be': 'UnicodeCharacter0xf3be',
u'\u2945': 'RightwardsArrowWithPlusBelow',
u'\u08c4': 'UnicodeCharacter0x8c4',
u'\ua349': 'YiSyllableZhap',
u'\u82c8': 'CjkUnifiedIdeograph82c8',
u'\u7e4b': 'CjkUnifiedIdeograph7e4b',
u'\u41ca': 'CjkUnifiedIdeograph41ca',
u'\uf84f': 'UnicodeCharacter0xf84f',
u'\udbce': 'UnicodeCharacter0xdbce',
u'\u3155': 'HangulLetterYeo',
u'\u10d4': 'GeorgianLetterEn',
u'\u8b59': 'CjkUnifiedIdeograph8b59',
u'\uead8': 'UnicodeCharacter0xead8',
u'\u465b': 'CjkUnifiedIdeograph465b',
u'\uc05f': 'HangulSyllableBbeud',
u'\u23de': 'TopCurlyBracket',
u'\ubde2': 'HangulSyllableBwilp',
u'\u1965': 'TaiLeLetterEe',
u'\u78e4': 'CjkUnifiedIdeograph78e4',
u'\u9369': 'CjkUnifiedIdeograph9369',
u'\uf2e8': 'UnicodeCharacter0xf2e8',
u'\u286f': 'BraillePatternDots123467',
u'\u0bee': 'TamilDigitEight',
u'\ua273': 'YiSyllableHa',
u'\u85f2': 'CjkUnifiedIdeograph85f2',
u'\u6175': 'CjkUnifiedIdeograph6175',
u'\u40f4': 'CjkUnifiedIdeograph40f4',
u'\ufb79': 'ArabicLetterNyehMedialForm',
u'\udaf8': 'UnicodeCharacter0xdaf8',
u'\u307f': 'HiraganaLetterMi',
u'\u13fe': 'UnicodeCharacter0x13fe',
u'\u6606': 'CjkUnifiedIdeograph6606',
u'\u4289': 'CjkUnifiedIdeograph4289',
u'\u2308': 'LeftCeiling',
u'\u1f8b': 'GreekCapitalLetterAlphaWithDasiaAndVariaAndProsgegrammeni',
u'\ue00a': 'UnicodeCharacter0xe00a',
u'\udc8d': 'UnicodeCharacter0xdc8d',
u'\ubd0c': 'HangulSyllableBok',
u'\u998f': 'CjkUnifiedIdeograph998f',
u'\u4e16': 'CjkUnifiedIdeograph4e16',
u'\u0b18': 'OriyaLetterGha',
u'\u679b': 'CjkUnifiedIdeograph679b',
u'\uc81a': 'HangulSyllableJeop',
u'\u249d': 'ParenthesizedLatinSmallLetterB',
u'\u851c': 'CjkUnifiedIdeograph851c',
u'\ue19f': 'UnicodeCharacter0xe19f',
u'\ubea1': 'HangulSyllableBbyalg',
u'\u5626': 'CjkUnifiedIdeograph5626',
u'\u1328': 'EthiopicSyllableCha',
u'\u4fab': 'CjkUnifiedIdeograph4fab',
u'\ud02a': 'HangulSyllableKwebs',
u'\u0cad': 'KannadaLetterBha',
u'\ued2c': 'UnicodeCharacter0xed2c',
u'\uc9af': 'HangulSyllableJyilb',
u'\uaa2e': 'ChamVowelSignOe',
u'\u86b1': 'CjkUnifiedIdeograph86b1',
u'\u7b38': 'CjkUnifiedIdeograph7b38',
u'\u57bb': 'CjkUnifiedIdeograph57bb',
u'\u383a': 'CjkUnifiedIdeograph383a',
u'\u14bd': 'CanadianSyllabicsMh',
u'\uf53c': 'UnicodeCharacter0xf53c',
u'\ud1bf': 'HangulSyllableTwags',
u'\ub23e': 'HangulSyllableNwegg',
u'\ueec1': 'UnicodeCharacter0xeec1',
u'\uabc3': 'UnicodeCharacter0xabc3',
u'\u4348': 'CjkUnifiedIdeograph4348',
u'\u7ccd': 'CjkUnifiedIdeograph7ccd',
u'\udd4c': 'UnicodeCharacter0xdd4c',
u'\u39cf': 'CjkUnifiedIdeograph39cf',
u'\u9a4e': 'CjkUnifiedIdeograph9a4e',
u'\uf6d1': 'UnicodeCharacter0xf6d1',
u'\ub3d3': 'HangulSyllableDolh',
u'\u685a': 'CjkUnifiedIdeograph685a',
u'\u44dd': 'CjkUnifiedIdeograph44dd',
u'\u255c': 'BoxDrawingsUpDoubleAndLeftSingle',
u'\u01df': 'LatinSmallLetterAWithDiaeresisAndMacron',
u'\ue25e': 'UnicodeCharacter0xe25e',
u'\udee1': 'UnicodeCharacter0xdee1',
u'\ubf60': 'HangulSyllableBbwan',
u'\u9be3': 'CjkUnifiedIdeograph9be3',
u'\u706a': 'CjkUnifiedIdeograph706a',
u'\u0d6c': 'MalayalamDigitSix',
u'\u69ef': 'CjkUnifiedIdeograph69ef',
u'\uca6e': 'HangulSyllableJjenh',
u'\u26f1': 'UnicodeCharacter0x26f1',
u'\u8770': 'CjkUnifiedIdeograph8770',
u'\ue3f3': 'UnicodeCharacter0xe3f3',
u'\ua0f5': 'YiSyllableVux',
u'\u587a': 'CjkUnifiedIdeograph587a',
u'\u157c': 'CanadianSyllabicsNunavutH',
u'\u71ff': 'CjkUnifiedIdeograph71ff',
u'\ud27e': 'HangulSyllableTwep',
u'\uef80': 'UnicodeCharacter0xef80',
u'\u4b03': 'CjkUnifiedIdeograph4b03',
u'\uac82': 'HangulSyllableGeobs',
u'\u0805': 'UnicodeCharacter0x805',
u'\uc507': 'HangulSyllableSseuc',
u'\u8209': 'CjkUnifiedIdeograph8209',
u'\u7d8c': 'CjkUnifiedIdeograph7d8c',
u'\u3a8e': 'CjkUnifiedIdeograph3a8e',
u'\uf790': 'UnicodeCharacter0xf790',
u'\u5313': 'CjkUnifiedIdeograph5313',
u'\ub492': 'HangulSyllableDwelm',
u'\u1015': 'MyanmarLetterPa',
u'\u2d17': 'GeorgianSmallLetterQar',
u'\uea19': 'UnicodeCharacter0xea19',
u'\ua71b': 'ModifierLetterRaisedUpArrow',
u'\u459c': 'CjkUnifiedIdeograph459c',
u'\u029e': 'LatinSmallLetterTurnedK',
u'\udfa0': 'UnicodeCharacter0xdfa0',
u'\u9ca2': 'CjkUnifiedIdeograph9ca2',
u'\u7825': 'CjkUnifiedIdeograph7825',
u'\u3527': 'CjkUnifiedIdeograph3527',
u'\uf229': 'UnicodeCharacter0xf229',
u'\u8f2b': 'CjkUnifiedIdeograph8f2b',
u'\u6aae': 'CjkUnifiedIdeograph6aae',
u'\u27b0': 'UnicodeCharacter0x27b0',
u'\ue4b2': 'UnicodeCharacter0xe4b2',
u'\u4035': 'CjkUnifiedIdeograph4035',
u'\ua1b4': 'YiSyllableHlyp',
u'\u1d37': 'ModifierLetterCapitalK',
u'\uda39': 'UnicodeCharacter0xda39',
u'\u973b': 'CjkUnifiedIdeograph973b',
u'\u72be': 'CjkUnifiedIdeograph72be',
u'\u0fc0': 'TibetanCantillationSignHeavyBeat',
u'\uccc2': 'HangulSyllableCelp',
u'\u89c4': 'CjkUnifiedIdeograph89c4',
u'\u6547': 'CjkUnifiedIdeograph6547',
u'\u2249': 'NotAlmostEqualTo',
u'\uff4b': 'FullwidthLatinSmallLetterK',
u'\ubc4d': 'HangulSyllableByag',
u'\u5ace': 'CjkUnifiedIdeograph5ace',
u'\u17d0': 'KhmerSignSamyokSannya',
u'\ud4d2': 'HangulSyllablePwinh',
u'\u91d4': 'CjkUnifiedIdeograph91d4',
u'\u4d57': 'CjkUnifiedIdeograph4d57',
u'\uaed6': 'HangulSyllableGgeop',
u'\u0a59': 'GurmukhiLetterKhha',
u'\uc75b': 'HangulSyllableYigs',
u'\u845d': 'CjkUnifiedIdeograph845d',
u'\u7fe0': 'CjkUnifiedIdeograph7fe0',
u'\u3ce2': 'CjkUnifiedIdeograph3ce2',
u'\uf9e4': 'CjkCompatibilityIdeographf9e4',
u'\u5567': 'CjkUnifiedIdeograph5567',
u'\ub6e6': 'HangulSyllableDdwebs',
u'\u1269': 'EthiopicSyllableVu',
u'\u2f6b': 'KangxiRadicalDish',
u'\uec6d': 'UnicodeCharacter0xec6d',
u'\ua96f': 'UnicodeCharacter0xa96f',
u'\u47f0': 'CjkUnifiedIdeograph47f0',
u'\u04f2': 'CyrillicCapitalLetterUWithDoubleAcute',
u'\uc1f4': 'HangulSyllableSoess',
u'\u9ef6': 'CjkUnifiedIdeograph9ef6',
u'\u7a79': 'CjkUnifiedIdeograph7a79',
u'\u377b': 'CjkUnifiedIdeograph377b',
u'\uf47d': 'UnicodeCharacter0xf47d',
u'\ub17f': 'HangulSyllableNod',
u'\ud181': 'HangulSyllableTyeot',
u'\u3604': 'CjkUnifiedIdeograph3604',
u'\u1a87': 'UnicodeCharacter0x1a87',
u'\ubc08': 'HangulSyllableMim',
u'\u808b': 'CjkUnifiedIdeograph808b',
u'\u7f0a': 'CjkUnifiedIdeograph7f0a',
u'\u438d': 'CjkUnifiedIdeograph438d',
u'\ue50e': 'UnicodeCharacter0xe50e',
u'\uc991': 'HangulSyllableJeulg',
u'\u2e14': 'DownwardsAncora',
u'\u7297': 'CjkUnifiedIdeograph7297',
u'\u9418': 'CjkUnifiedIdeograph9418',
u'\uf89b': 'UnicodeCharacter0xf89b',
u'\u571a': 'CjkUnifiedIdeograph571a',
u'\udd1e': 'UnicodeCharacter0xdd1e',
u'\u21a1': 'DownwardsTwoHeadedArrow',
u'\ua7a5': 'UnicodeCharacter0xa7a5',
u'\u0624': 'ArabicLetterWawWithHamzaAbove',
u'\u6aa7': 'CjkUnifiedIdeograph6aa7',
u'\u8c28': 'CjkUnifiedIdeograph8c28',
u'\ud0ab': 'HangulSyllableKid',
u'\u4f2a': 'CjkUnifiedIdeograph4f2a',
u'\u352e': 'CjkUnifiedIdeograph352e',
u'\u19b1': 'NewTaiLueVowelSignAa',
u'\ubb32': 'HangulSyllableMyop',
u'\u9fb5': 'CjkUnifiedIdeograph9fb5',
u'\u7e34': 'CjkUnifiedIdeograph7e34',
u'\u42b7': 'CjkUnifiedIdeograph42b7',
u'\ue438': 'UnicodeCharacter0xe438',
u'\uc8bb': 'HangulSyllableJwaes',
u'\u2d3e': 'TifinaghLetterTuaregYak',
u'\u71c1': 'CjkUnifiedIdeograph71c1',
u'\u9342': 'CjkUnifiedIdeograph9342',
u'\uf7c5': 'UnicodeCharacter0xf7c5',
u'\u5644': 'CjkUnifiedIdeograph5644',
u'\udc48': 'UnicodeCharacter0xdc48',
u'\u20cb': 'UnicodeCharacter0x20cb',
u'\ua6cf': 'UnicodeCharacter0xa6cf',
u'\u054e': 'ArmenianCapitalLetterVew',
u'\u69d1': 'CjkUnifiedIdeograph69d1',
u'\u8b52': 'CjkUnifiedIdeograph8b52',
u'\uefd5': 'UnicodeCharacter0xefd5',
u'\u4e54': 'CjkUnifiedIdeograph4e54',
u'\u3458': 'CjkUnifiedIdeograph3458',
u'\u18db': 'UnicodeCharacter0x18db',
u'\uba5c': 'HangulSyllableMel',
u'\u9edf': 'CjkUnifiedIdeograph9edf',
u'\u7d5e': 'CjkUnifiedIdeograph7d5e',
u'\u41e1': 'CjkUnifiedIdeograph41e1',
u'\ue362': 'UnicodeCharacter0xe362',
u'\uc7e5': 'HangulSyllableJyaeg',
u'\u2c68': 'LatinSmallLetterHWithDescender',
u'\u70eb': 'CjkUnifiedIdeograph70eb',
u'\u926c': 'CjkUnifiedIdeograph926c',
u'\uf6ef': 'UnicodeCharacter0xf6ef',
u'\u556e': 'CjkUnifiedIdeograph556e',
u'\udb72': 'UnicodeCharacter0xdb72',
u'\u3ff5': 'CjkUnifiedIdeograph3ff5',
u'\ua5f9': 'VaiSyllableDe',
u'\u0478': 'CyrillicCapitalLetterUk',
u'\u68fb': 'CjkUnifiedIdeograph68fb',
u'\u8a7c': 'CjkUnifiedIdeograph8a7c',
u'\ueeff': 'UnicodeCharacter0xeeff',
u'\u4d7e': 'CjkUnifiedIdeograph4d7e',
u'\ud403': 'HangulSyllablePoc',
u'\u3382': 'SquareMuA',
u'\ub986': 'HangulSyllableReubs',
u'\u1d09': 'LatinSmallLetterTurnedI',
u'\u7c88': 'CjkUnifiedIdeograph7c88',
u'\u830d': 'CjkUnifiedIdeograph830d',
u'\ue28c': 'UnicodeCharacter0xe28c',
u'\u460f': 'CjkUnifiedIdeograph460f',
u'\ucc13': 'HangulSyllableJjid',
u'\u2b92': 'UnicodeCharacter0x2b92',
u'\ub217': 'HangulSyllableNus',
u'\u9196': 'CjkUnifiedIdeograph9196',
u'\u7519': 'CjkUnifiedIdeograph7519',
u'\u5498': 'CjkUnifiedIdeograph5498',
u'\ufb1d': 'HebrewLetterYodWithHiriq',
u'\uda9c': 'UnicodeCharacter0xda9c',
u'\u2423': 'OpenBox',
u'\u03a2': 'UnicodeCharacter0x3a2',
u'\uaa27': 'ChamLetterSa',
u'\u89a6': 'CjkUnifiedIdeograph89a6',
u'\u6d29': 'CjkUnifiedIdeograph6d29',
u'\u4ca8': 'CjkUnifiedIdeograph4ca8',
u'\ud32d': 'HangulSyllablePaenj',
u'\u32ac': 'CircledIdeographSupervise',
u'\ub8b0': 'HangulSyllableRoe',
u'\u1c33': 'LepchaConsonantSignT',
u'\u7bb2': 'CjkUnifiedIdeograph7bb2',
u'\u8237': 'CjkUnifiedIdeograph8237',
u'\ue1b6': 'UnicodeCharacter0xe1b6',
u'\u4539': 'CjkUnifiedIdeograph4539',
u'\ucb3d': 'HangulSyllableJjyob',
u'\u2abc': 'DoubleSucceeds',
u'\ub141': 'HangulSyllableNyeog',
u'\u90c0': 'CjkUnifiedIdeograph90c0',
u'\u7443': 'CjkUnifiedIdeograph7443',
u'\u53c2': 'CjkUnifiedIdeograph53c2',
u'\ufa47': 'CjkCompatibilityIdeographfa47',
u'\ud9c6': 'UnicodeCharacter0xd9c6',
u'\u234d': 'AplFunctionalSymbolQuadDelta',
u'\u02cc': 'ModifierLetterLowVerticalLine',
u'\ua951': 'RejangConsonantSignR',
u'\u88d0': 'CjkUnifiedIdeograph88d0',
u'\u6c53': 'CjkUnifiedIdeograph6c53',
u'\u4bd2': 'CjkUnifiedIdeograph4bd2',
u'\ud257': 'HangulSyllableTweolh',
u'\u31d6': 'CjkStrokeHg',
u'\ub7da': 'HangulSyllableRyaelm',
u'\u1b5d': 'BalineseCarikPamungkah',
u'\u7adc': 'CjkUnifiedIdeograph7adc',
u'\u8161': 'CjkUnifiedIdeograph8161',
u'\ue0e0': 'UnicodeCharacter0xe0e0',
u'\u4463': 'CjkUnifiedIdeograph4463',
u'\uca67': 'HangulSyllableJjeoh',
u'\u29e6': 'GleichStark',
u'\ub06b': 'HangulSyllableGgyilb',
u'\uafea': 'HangulSyllableGgweoj',
u'\u736d': 'CjkUnifiedIdeograph736d',
u'\u52ec': 'CjkUnifiedIdeograph52ec',
u'\uf971': 'CjkCompatibilityIdeographf971',
u'\ud8f0': 'UnicodeCharacter0xd8f0',
u'\u2277': 'GreaterthanOrLessthan',
u'\u01f6': 'LatinCapitalLetterHwair',
u'\ua87b': 'UnicodeCharacter0xa87b',
u'\u87fa': 'CjkUnifiedIdeograph87fa',
u'\u6b7d': 'CjkUnifiedIdeograph6b7d',
u'\u4afc': 'CjkUnifiedIdeograph4afc',
u'\u5081': 'CjkUnifiedIdeograph5081',
u'\u3100': 'UnicodeCharacter0x3100',
u'\u1583': 'CanadianSyllabicsQa',
u'\uf202': 'UnicodeCharacter0xf202',
u'\ud685': 'HangulSyllableHwaeng',
u'\ub704': 'HangulSyllableDdwiss',
u'\u9b87': 'CjkUnifiedIdeograph9b87',
u'\u640e': 'CjkUnifiedIdeograph640e',
u'\u4891': 'CjkUnifiedIdeograph4891',
u'\u2910': 'RightwardsTwoheadedTripleDashArrow',
u'\u0d93': 'SinhalaLetterAiyanna',
u'\uea12': 'UnicodeCharacter0xea12',
u'\uce95': 'HangulSyllableKaenj',
u'\uaf14': 'HangulSyllableGgyen',
u'\uf397': 'UnicodeCharacter0xf397',
u'\ub499': 'HangulSyllableDweb',
u'\u5c1e': 'CjkUnifiedIdeograph5c1e',
u'\u0120': 'LatinCapitalLetterGWithDotAbove',
u'\u65a3': 'CjkUnifiedIdeograph65a3',
u'\uc222': 'HangulSyllableSulm',
u'\u26a5': 'MaleAndFemaleSign',
u'\u8724': 'CjkUnifiedIdeograph8724',
u'\ueba7': 'UnicodeCharacter0xeba7',
u'\uaca9': 'HangulSyllableGyeog',
u'\u7930': 'CjkUnifiedIdeograph7930',
u'\u5db3': 'CjkUnifiedIdeograph5db3',
u'\u3a32': 'CjkUnifiedIdeograph3a32',
u'\u1eb5': 'LatinSmallLetterAWithBreveAndTilde',
u'\uff34': 'FullwidthLatinCapitalLetterT',
u'\uc3b7': 'HangulSyllableSsyeoc',
u'\ua036': 'YiSyllableByrx',
u'\u84b9': 'CjkUnifiedIdeograph84b9',
u'\u5140': 'CjkUnifiedIdeograph5140',
u'\u1242': 'EthiopicSyllableQi',
u'\u76c5': 'CjkUnifiedIdeograph76c5',
u'\ud744': 'HangulSyllableHyum',
u'\u3bc7': 'CjkUnifiedIdeograph3bc7',
u'\u9846': 'CjkUnifiedIdeograph9846',
u'\ufcc9': 'ArabicLigatureLamWithJeemInitialForm',
u'\ua1cb': 'YiSyllableLex',
u'\u4950': 'CjkUnifiedIdeograph4950',
u'\u0a52': 'UnicodeCharacter0xa52',
u'\u6ed5': 'CjkUnifiedIdeograph6ed5',
u'\ucf54': 'HangulSyllableKo',
u'\u13d7': 'CherokeeLetterDi',
u'\uf056': 'UnicodeCharacter0xf056',
u'\ud4d9': 'HangulSyllablePwilt',
u'\ub558': 'HangulSyllableDdaels',
u'\u99db': 'CjkUnifiedIdeograph99db',
u'\u6262': 'CjkUnifiedIdeograph6262',
u'\u46e5': 'CjkUnifiedIdeograph46e5',
u'\u2764': 'HeavyBlackHeart',
u'\u0be7': 'TamilDigitOne',
u'\ue866': 'UnicodeCharacter0xe866',
u'\ucce9': 'HangulSyllableCyeot',
u'\uad68': 'HangulSyllableGyok',
u'\uf1eb': 'UnicodeCharacter0xf1eb',
u'\ub2ed': 'HangulSyllableDalg',
u'\u5a72': 'CjkUnifiedIdeograph5a72',
u'\u1f74': 'GreekSmallLetterEtaWithVaria',
u'\u63f7': 'CjkUnifiedIdeograph63f7',
u'\uc076': 'HangulSyllableBbyigg',
u'\u24f9': 'DoubleCircledDigitFive',
u'\u8578': 'CjkUnifiedIdeograph8578',
u'\ue9fb': 'UnicodeCharacter0xe9fb',
u'\uaafd': 'UnicodeCharacter0xaafd',
u'\u9001': 'CjkUnifiedIdeograph9001',
u'\u7784': 'CjkUnifiedIdeograph7784',
u'\u3886': 'CjkUnifiedIdeograph3886',
u'\ufd88': 'ArabicLigatureLamWithMeemWithHahInitialForm',
u'\u410b': 'CjkUnifiedIdeograph410b',
u'\ube8a': 'HangulSyllableBbaelp',
u'\u020d': 'LatinSmallLetterOWithDoubleGrave',
u'\uc70f': 'HangulSyllableWilb',
u'\u8811': 'CjkUnifiedIdeograph8811',
u'\u6f94': 'CjkUnifiedIdeograph6f94',
u'\u1096': 'MyanmarShanDigitSix',
u'\ud598': 'HangulSyllableHyal',
u'\u969a': 'CjkUnifiedIdeograph969a',
u'\u7a1d': 'CjkUnifiedIdeograph7a1d',
u'\u3f1f': 'CjkUnifiedIdeograph3f1f',
u'\ue021': 'UnicodeCharacter0xe021',
u'\ua523': 'VaiSyllableNyee',
u'\u47a4': 'CjkUnifiedIdeograph47a4',
u'\u08a6': 'UnicodeCharacter0x8a6',
u'\ucda8': 'HangulSyllableCuss',
u'\u8eaa': 'CjkUnifiedIdeograph8eaa',
u'\u522d': 'CjkUnifiedIdeograph522d',
u'\ub3ac': 'HangulSyllableDyen',
u'\u172f': 'HanunooLetterWa',
u'\ud831': 'UnicodeCharacter0xd831',
u'\u9d33': 'CjkUnifiedIdeograph9d33',
u'\u60b6': 'CjkUnifiedIdeograph60b6',
u'\u25b8': 'BlackRightpointingSmallTriangle',
u'\ue6ba': 'UnicodeCharacter0xe6ba',
u'\u4a3d': 'CjkUnifiedIdeograph4a3d',
u'\uabbc': 'UnicodeCharacter0xabbc',
u'\u0f3f': 'TibetanSignMarTshes',
u'\u3041': 'HiraganaLetterSmallA',
u'\uf543': 'UnicodeCharacter0xf543',
u'\ub645': 'HangulSyllableDdwat',
u'\u58c6': 'CjkUnifiedIdeograph58c6',
u'\u1dc8': 'CombiningGraveacutegrave',
u'\udeca': 'UnicodeCharacter0xdeca',
u'\u83cc': 'CjkUnifiedIdeograph83cc',
u'\u674f': 'CjkUnifiedIdeograph674f',
u'\u2851': 'BraillePatternDots157',
u'\ued53': 'UnicodeCharacter0xed53',
u'\uae55': 'HangulSyllableGgalg',
u'\u75d8': 'CjkUnifiedIdeograph75d8',
u'\u36da': 'CjkUnifiedIdeograph36da',
u'\ufbdc': 'ArabicLetterYuFinalForm',
u'\u5f5f': 'CjkUnifiedIdeograph5f5f',
u'\ubcde': 'HangulSyllableByenh',
u'\uc563': 'HangulSyllableAegs',
u'\u8665': 'CjkUnifiedIdeograph8665',
u'\u6de8': 'CjkUnifiedIdeograph6de8',
u'\u2eea': 'CjkRadicalCsimplifiedFrog',
u'\ud3ec': 'HangulSyllablePo',
u'\u94ee': 'CjkUnifiedIdeograph94ee',
u'\u7871': 'CjkUnifiedIdeograph7871',
u'\u3d73': 'CjkUnifiedIdeograph3d73',
u'\ufe75': 'UnicodeCharacter0xfe75',
u'\ua377': 'YiSyllableChyx',
u'\u45f8': 'CjkUnifiedIdeograph45f8',
u'\u06fa': 'ArabicLetterSheenWithDotBelow',
u'\ucbfc': 'HangulSyllableJjyils',
u'\u8cfe': 'CjkUnifiedIdeograph8cfe',
u'\ub200': 'HangulSyllableNyok',
u'\u9683': 'CjkUnifiedIdeograph9683',
u'\u7102': 'CjkUnifiedIdeograph7102',
u'\u5585': 'CjkUnifiedIdeograph5585',
u'\ufb06': 'LatinSmallLigatureSt',
u'\udf89': 'UnicodeCharacter0xdf89',
u'\u200c': 'ZeroWidthNonjoiner',
u'\u048f': 'CyrillicSmallLetterErWithTick',
u'\uaa10': 'ChamLetterNhue',
u'\u8e93': 'CjkUnifiedIdeograph8e93',
u'\u6912': 'CjkUnifiedIdeograph6912',
u'\u4d95': 'CjkUnifiedIdeograph4d95',
u'\ud316': 'HangulSyllablePalm',
u'\u3799': 'CjkUnifiedIdeograph3799',
u'\ub99d': 'HangulSyllableRyilt',
u'\u181c': 'UnicodeCharacter0x181c',
u'\u7c9f': 'CjkUnifiedIdeograph7c9f',
u'\u8220': 'CjkUnifiedIdeograph8220',
u'\ue6a3': 'UnicodeCharacter0xe6a3',
u'\u4122': 'CjkUnifiedIdeograph4122',
u'\ucb26': 'HangulSyllableJjoej',
u'\u2fa9': 'KangxiRadicalMound',
u'\u91ad': 'CjkUnifiedIdeograph91ad',
u'\u702c': 'CjkUnifiedIdeograph702c',
u'\u54af': 'CjkUnifiedIdeograph54af',
u'\ufa30': 'CjkCompatibilityIdeographfa30',
u'\udeb3': 'UnicodeCharacter0xdeb3',
u'\u2336': 'AplFunctionalSymbolIbeam',
u'\u07b9': 'UnicodeCharacter0x7b9',
u'\ua53a': 'VaiSyllableNdi',
u'\u89bd': 'CjkUnifiedIdeograph89bd',
u'\u683c': 'CjkUnifiedIdeograph683c',
u'\u4cbf': 'CjkUnifiedIdeograph4cbf',
u'\ud240': 'HangulSyllableTuss',
u'\u36c3': 'CjkUnifiedIdeograph36c3',
u'\ub8c7': 'HangulSyllableRoec',
u'\u1b46': 'BalineseLetterKhotSasak',
u'\u7fc9': 'CjkUnifiedIdeograph7fc9',
u'\u9d4a': 'CjkUnifiedIdeograph9d4a',
u'\ue1cd': 'UnicodeCharacter0xe1cd',
u'\u404c': 'CjkUnifiedIdeograph404c',
u'\uca50': 'HangulSyllableJjeon',
u'\u2ed3': 'CjkRadicalCsimplifiedLong',
u'\u90d7': 'CjkUnifiedIdeograph90d7',
u'\u7356': 'CjkUnifiedIdeograph7356',
u'\u57d9': 'CjkUnifiedIdeograph57d9',
u'\uf55a': 'UnicodeCharacter0xf55a',
u'\ud9dd': 'UnicodeCharacter0xd9dd',
u'\u2260': 'NotEqualTo',
u'\u06e3': 'ArabicSmallLowSeen',
u'\ua464': 'YiSyllableXiep',
u'\u88e7': 'CjkUnifiedIdeograph88e7',
u'\u6b66': 'CjkUnifiedIdeograph6b66',
u'\u4fe9': 'CjkUnifiedIdeograph4fe9',
u'\ued6a': 'UnicodeCharacter0xed6a',
u'\u31ed': 'UnicodeCharacter0x31ed',
u'\ubbf1': 'HangulSyllableMying',
u'\u1a70': 'UnicodeCharacter0x1a70',
u'\u7ef3': 'CjkUnifiedIdeograph7ef3',
u'\u9c74': 'CjkUnifiedIdeograph9c74',
u'\ue0f7': 'UnicodeCharacter0xe0f7',
u'\u4376': 'CjkUnifiedIdeograph4376',
u'\uc57a': 'HangulSyllableAep',
u'\u29fd': 'RightpointingCurvedAngleBracket',
u'\u1301': 'EthiopicSyllableJu',
u'\u7280': 'CjkUnifiedIdeograph7280',
u'\u9505': 'CjkUnifiedIdeograph9505',
u'\uf484': 'UnicodeCharacter0xf484',
u'\u5807': 'CjkUnifiedIdeograph5807',
u'\uc20b': 'HangulSyllableSyolh',
u'\u3d8a': 'CjkUnifiedIdeograph3d8a',
u'\ua78e': 'UnicodeCharacter0xa78e',
u'\u0b11': 'UnicodeCharacter0xb11',
u'\u6a90': 'CjkUnifiedIdeograph6a90',
u'\u8d15': 'CjkUnifiedIdeograph8d15',
u'\uec94': 'UnicodeCharacter0xec94',
u'\u3a1b': 'CjkUnifiedIdeograph3a1b',
u'\u159a': 'CanadianSyllabicsSayisiSha',
u'\ubc1f': 'HangulSyllableBalb',
u'\u9f9e': 'CjkUnifiedIdeograph9f9e',
u'\u6321': 'CjkUnifiedIdeograph6321',
u'\u42a0': 'CjkUnifiedIdeograph42a0',
u'\ue525': 'UnicodeCharacter0xe525',
u'\uc4a4': 'HangulSyllableSswel',
u'\u122b': 'EthiopicSyllableRaa',
u'\u0daa': 'SinhalaLetterMahaapraanaDdayanna',
u'\u942f': 'CjkUnifiedIdeograph942f',
u'\uf7ae': 'UnicodeCharacter0xf7ae',
u'\u5b31': 'CjkUnifiedIdeograph5b31',
u'\udd35': 'UnicodeCharacter0xdd35',
u'\u3cb4': 'CjkUnifiedIdeograph3cb4',
u'\ua6b8': 'UnicodeCharacter0xa6b8',
u'\u0a3b': 'UnicodeCharacter0xa3b',
u'\u65ba': 'CjkUnifiedIdeograph65ba',
u'\u8c3f': 'CjkUnifiedIdeograph8c3f',
u'\uefbe': 'UnicodeCharacter0xefbe',
u'\u3545': 'CjkUnifiedIdeograph3545',
u'\u14c4': 'CanadianSyllabicsNo',
u'\ubf49': 'HangulSyllableBbolg',
u'\u9ec8': 'CjkUnifiedIdeograph9ec8',
u'\u624b': 'CjkUnifiedIdeograph624b',
u'\u5dca': 'CjkUnifiedIdeograph5dca',
u'\ue44f': 'UnicodeCharacter0xe44f',
u'\uc7ce': 'HangulSyllableJyanh',
u'\u2d55': 'TifinaghLetterYarr',
u'\u0cd4': 'UnicodeCharacter0xcd4',
u'\u9759': 'CjkUnifiedIdeograph9759',
u'\uf6d8': 'UnicodeCharacter0xf6d8',
u'\u5a5b': 'CjkUnifiedIdeograph5a5b',
u'\udc5f': 'UnicodeCharacter0xdc5f',
u'\u3fde': 'CjkUnifiedIdeograph3fde',
u'\ua1e2': 'YiSyllableGat',
u'\u0565': 'ArmenianSmallLetterEch',
u'\u64e4': 'CjkUnifiedIdeograph64e4',
u'\u8f69': 'CjkUnifiedIdeograph8f69',
u'\ueee8': 'UnicodeCharacter0xeee8',
u'\u346f': 'CjkUnifiedIdeograph346f',
u'\u17ee': 'UnicodeCharacter0x17ee',
u'\ube73': 'HangulSyllableBbas',
u'\u99f2': 'CjkUnifiedIdeograph99f2',
u'\u7d75': 'CjkUnifiedIdeograph7d75',
u'\u5cf4': 'CjkUnifiedIdeograph5cf4',
u'\ue779': 'UnicodeCharacter0xe779',
u'\uc6f8': 'HangulSyllableWem',
u'\u2c7f': 'UnicodeCharacter0x2c7f',
u'\u0ffe': 'UnicodeCharacter0xffe',
u'\u7a06': 'CjkUnifiedIdeograph7a06',
u'\u5e89': 'CjkUnifiedIdeograph5e89',
u'\u3f08': 'CjkUnifiedIdeograph3f08',
u'\u038b': 'UnicodeCharacter0x38b',
u'\ufc0a': 'ArabicLigatureBehWithYehIsolatedForm',
u'\uc08d': 'HangulSyllableBbyit',
u'\ua10c': 'YiSyllableDuo',
u'\u858f': 'CjkUnifiedIdeograph858f',
u'\u5216': 'CjkUnifiedIdeograph5216',
u'\u1718': 'UnicodeCharacter0x1718',
u'\u7b9b': 'CjkUnifiedIdeograph7b9b',
u'\ud41a': 'HangulSyllablePwabs',
u'\u389d': 'CjkUnifiedIdeograph389d',
u'\u991c': 'CjkUnifiedIdeograph991c',
u'\ufd9f': 'ArabicLigatureTehWithJeemWithYehFinalForm',
u'\ua2a1': 'YiSyllableZep',
u'\u4a26': 'CjkUnifiedIdeograph4a26',
u'\u0f28': 'TibetanDigitEight',
u'\u53ab': 'CjkUnifiedIdeograph53ab',
u'\ucc2a': 'HangulSyllableCagg',
u'\u10ad': 'GeorgianCapitalLetterOn',
u'\uf12c': 'UnicodeCharacter0xf12c',
u'\ud5af': 'HangulSyllableHyaegs',
u'\ub62e': 'HangulSyllableDdwagg',
u'\u9ab1': 'CjkUnifiedIdeograph9ab1',
u'\u6738': 'CjkUnifiedIdeograph6738',
u'\u4bbb': 'CjkUnifiedIdeograph4bbb',
u'\u243a': 'UnicodeCharacter0x243a',
u'\u08bd': 'UnicodeCharacter0x8bd',
u'\ue93c': 'UnicodeCharacter0xe93c',
u'\ucdbf': 'HangulSyllableCweolh',
u'\uae3e': 'HangulSyllableGilp',
u'\uf2c1': 'UnicodeCharacter0xf2c1',
u'\ub7c3': 'HangulSyllableRyalh',
u'\u5f48': 'CjkUnifiedIdeograph5f48',
u'\u1c4a': 'UnicodeCharacter0x1c4a',
u'\u60cd': 'CjkUnifiedIdeograph60cd',
u'\uc14c': 'HangulSyllableSess',
u'\u25cf': 'BlackCircle',
u'\u864e': 'CjkUnifiedIdeograph864e',
u'\uead1': 'UnicodeCharacter0xead1',
u'\uafd3': 'HangulSyllableGguh',
u'\u745a': 'CjkUnifiedIdeograph745a',
u'\u58dd': 'CjkUnifiedIdeograph58dd',
u'\u395c': 'CjkUnifiedIdeograph395c',
u'\u1ddf': 'CombiningLatinLetterSmallCapitalM',
u'\ufe5e': 'SmallRightTortoiseShellBracket',
u'\uc2e1': 'HangulSyllableSinj',
u'\ua360': 'YiSyllableZhyr',
u'\u87e3': 'CjkUnifiedIdeograph87e3',
u'\u6c6a': 'CjkUnifiedIdeograph6c6a',
u'\u116c': 'HangulJungseongOe',
u'\u75ef': 'CjkUnifiedIdeograph75ef',
u'\ud66e': 'HangulSyllableHwap',
u'\u3af1': 'CjkUnifiedIdeograph3af1',
u'\u9b70': 'CjkUnifiedIdeograph9b70',
u'\ufff3': 'UnicodeCharacter0xfff3',
u'\ubcf5': 'HangulSyllableBog',
u'\u447a': 'CjkUnifiedIdeograph447a',
u'\u097c': 'DevanagariLetterJja',
u'\u6dff': 'CjkUnifiedIdeograph6dff',
u'\uce7e': 'HangulSyllableKalm',
u'\uf380': 'UnicodeCharacter0xf380',
u'\u5703': 'CjkUnifiedIdeograph5703',
u'\ub082': 'HangulSyllableGginh',
u'\u1405': 'CanadianSyllabicsO',
u'\ud907': 'UnicodeCharacter0xd907',
u'\u9e09': 'CjkUnifiedIdeograph9e09',
u'\u618c': 'CjkUnifiedIdeograph618c',
u'\u268e': 'DigramForLesserYang',
u'\ueb90': 'UnicodeCharacter0xeb90',
u'\u4f13': 'CjkUnifiedIdeograph4f13',
u'\ua892': 'SaurashtraLetterKa',
u'\u0c15': 'TeluguLetterKa',
u'\u3117': 'BopomofoLetterZ',
u'\uf619': 'UnicodeCharacter0xf619',
u'\ubb1b': 'HangulSyllableMyogs',
u'\u599c': 'CjkUnifiedIdeograph599c',
u'\u1e9e': 'LatinCapitalLetterSharpS',
u'\uc3a0': 'HangulSyllableSsyeo',
u'\u80a2': 'CjkUnifiedIdeograph80a2',
u'\u6425': 'CjkUnifiedIdeograph6425',
u'\u2927': 'NorthWestArrowAndNorthEastArrow',
u'\uee29': 'UnicodeCharacter0xee29',
u'\u932b': 'CjkUnifiedIdeograph932b',
u'\u76ae': 'CjkUnifiedIdeograph76ae',
u'\u3bb0': 'CjkUnifiedIdeograph3bb0',
u'\uf8b2': 'UnicodeCharacter0xf8b2',
u'\u5c35': 'CjkUnifiedIdeograph5c35',
u'\ubdb4': 'HangulSyllableBweok',
u'\u0137': 'LatinSmallLetterKWithCedilla',
u'\uc639': 'HangulSyllableOng',
u'\u8b3b': 'CjkUnifiedIdeograph8b3b',
u'\u6ebe': 'CjkUnifiedIdeograph6ebe',
u'\u13c0': 'CherokeeLetterNah',
u'\ud0c2': 'HangulSyllableTagg',
u'\u95c4': 'CjkUnifiedIdeograph95c4',
u'\u7947': 'CjkUnifiedIdeograph7947',
u'\u3e49': 'CjkUnifiedIdeograph3e49',
u'\ue34b': 'UnicodeCharacter0xe34b',
u'\ua04d': 'YiSyllablePup',
u'\u46ce': 'CjkUnifiedIdeograph46ce',
u'\u0bd0': 'TamilOm',
u'\uc8d2': 'HangulSyllableJoelp',
u'\u8dd4': 'CjkUnifiedIdeograph8dd4',
u'\u5157': 'CjkUnifiedIdeograph5157',
u'\ub2d6': 'HangulSyllableNilp',
u'\u1659': 'CanadianSyllabicsCarrierSha',
u'\udb5b': 'UnicodeCharacter0xdb5b',
u'\u985d': 'CjkUnifiedIdeograph985d',
u'\u63e0': 'CjkUnifiedIdeograph63e0',
u'\u20e2': 'CombiningEnclosingScreen',
u'\ue5e4': 'UnicodeCharacter0xe5e4',
u'\u4967': 'CjkUnifiedIdeograph4967',
u'\uaae6': 'UnicodeCharacter0xaae6',
u'\u0e69': 'UnicodeCharacter0xe69',
u'\u336b': 'IdeographicTelegraphSymbolForHourNineteen',
u'\uf06d': 'UnicodeCharacter0xf06d',
u'\ub56f': 'HangulSyllableDdyad',
u'\u5bf0': 'CjkUnifiedIdeograph5bf0',
u'\u18f2': 'UnicodeCharacter0x18f2',
u'\uddf4': 'UnicodeCharacter0xddf4',
u'\u82f6': 'CjkUnifiedIdeograph82f6',
u'\u6679': 'CjkUnifiedIdeograph6679',
u'\u2b7b': 'UnicodeCharacter0x2b7b',
u'\ue87d': 'UnicodeCharacter0xe87d',
u'\uad7f': 'HangulSyllableGus',
u'\ud581': 'HangulSyllableHaelt',
u'\u3204': 'ParenthesizedHangulMieum',
u'\u1e87': 'LatinSmallLetterWWithDotAbove',
u'\ub808': 'HangulSyllableRe',
u'\u848b': 'CjkUnifiedIdeograph848b',
u'\u7b0a': 'CjkUnifiedIdeograph7b0a',
u'\u478d': 'CjkUnifiedIdeograph478d',
u'\ue10e': 'UnicodeCharacter0xe10e',
u'\ucd91': 'HangulSyllableCyot',
u'\u2a14': 'LineIntegrationNotIncludingThePole',
u'\u7697': 'CjkUnifiedIdeograph7697',
u'\u9018': 'CjkUnifiedIdeograph9018',
u'\ufc9b': 'ArabicLigatureYehWithHamzaAboveWithHehInitialForm',
u'\u531a': 'CjkUnifiedIdeograph531a',
u'\ud91e': 'UnicodeCharacter0xd91e',
u'\u25a1': 'WhiteSquare',
u'\ua3a5': 'YiSyllableNryt',
u'\u0224': 'LatinCapitalLetterZWithHook',
u'\u6ea7': 'CjkUnifiedIdeograph6ea7',
u'\u8828': 'CjkUnifiedIdeograph8828',
u'\ud4ab': 'HangulSyllablePweoc',
u'\u4b2a': 'CjkUnifiedIdeograph4b2a',
u'\u312e': 'UnicodeCharacter0x312e',
u'\u1db1': 'ModifierLetterSmallBarredO',
u'\ubf32': 'HangulSyllableBbyelp',
u'\u9bb5': 'CjkUnifiedIdeograph9bb5',
u'\u7a34': 'CjkUnifiedIdeograph7a34',
u'\u46b7': 'CjkUnifiedIdeograph46b7',
u'\ue038': 'UnicodeCharacter0xe038',
u'\uccbb': 'HangulSyllableCed',
u'\u293e': 'LowerRightSemicircularClockwiseArrow',
u'\u75c1': 'CjkUnifiedIdeograph75c1',
u'\u9742': 'CjkUnifiedIdeograph9742',
u'\uf3c5': 'UnicodeCharacter0xf3c5',
u'\u5244': 'CjkUnifiedIdeograph5244',
u'\ud848': 'UnicodeCharacter0xd848',
u'\u24cb': 'CircledLatinCapitalLetterV',
u'\ua2cf': 'YiSyllableCyr',
u'\u014e': 'LatinCapitalLetterOWithBreve',
u'\u6dd1': 'CjkUnifiedIdeograph6dd1',
u'\u8f52': 'CjkUnifiedIdeograph8f52',
u'\uebd5': 'UnicodeCharacter0xebd5',
u'\u4a54': 'CjkUnifiedIdeograph4a54',
u'\u3058': 'HiraganaLetterZi',
u'\u1cdb': 'UnicodeCharacter0x1cdb',
u'\ube5c': 'HangulSyllableBik',
u'\u9adf': 'CjkUnifiedIdeograph9adf',
u'\u795e': 'CjkUnifiedIdeograph795e',
u'\u45e1': 'CjkUnifiedIdeograph45e1',
u'\ue762': 'UnicodeCharacter0xe762',
u'\uc3e5': 'HangulSyllableSsolt',
u'\u2868': 'BraillePatternDots467',
u'\u74eb': 'CjkUnifiedIdeograph74eb',
u'\u966c': 'CjkUnifiedIdeograph966c',
u'\uf2ef': 'UnicodeCharacter0xf2ef',
u'\u516e': 'CjkUnifiedIdeograph516e',
u'\udf72': 'UnicodeCharacter0xdf72',
u'\u3bf5': 'CjkUnifiedIdeograph3bf5',
u'\ua1f9': 'YiSyllableKix',
u'\u6cfb': 'CjkUnifiedIdeograph6cfb',
u'\u8e7c': 'CjkUnifiedIdeograph8e7c',
u'\ueaff': 'UnicodeCharacter0xeaff',
u'\u497e': 'CjkUnifiedIdeograph497e',
u'\ud003': 'HangulSyllableKweod',
u'\u3782': 'CjkUnifiedIdeograph3782',
u'\ubd86': 'HangulSyllableBunh',
u'\u1909': 'LimbuLetterJha',
u'\u7888': 'CjkUnifiedIdeograph7888',
u'\u870d': 'CjkUnifiedIdeograph870d',
u'\ue68c': 'UnicodeCharacter0xe68c',
u'\u420f': 'CjkUnifiedIdeograph420f',
u'\uc813': 'HangulSyllableJeos',
u'\u2f92': 'KangxiRadicalSee',
u'\ub617': 'HangulSyllableDdod',
u'\u9596': 'CjkUnifiedIdeograph9596',
u'\u7119': 'CjkUnifiedIdeograph7119',
u'\u5098': 'CjkUnifiedIdeograph5098',
u'\uff1d': 'FullwidthEqualsSign',
u'\ude9c': 'UnicodeCharacter0xde9c',
u'\u2023': 'TriangularBullet',
u'\u07a2': 'ThaanaLetterAinu',
u'\uae27': 'HangulSyllableGyis',
u'\u8da6': 'CjkUnifiedIdeograph8da6',
u'\u6929': 'CjkUnifiedIdeograph6929',
u'\u48a8': 'CjkUnifiedIdeograph48a8',
u'\ud72d': 'HangulSyllableHwing',
u'\u36ac': 'CjkUnifiedIdeograph36ac',
u'\ubcb0': 'HangulSyllableBem',
u'\u1833': 'MongolianLetterDa',
u'\u7fb2': 'CjkUnifiedIdeograph7fb2',
u'\u8637': 'CjkUnifiedIdeograph8637',
u'\ue5b6': 'UnicodeCharacter0xe5b6',
u'\u4139': 'CjkUnifiedIdeograph4139',
u'\ucf3d': 'HangulSyllableKyenj',
u'\u2ebc': 'CjkRadicalMeat',
u'\ub541': 'HangulSyllableDdab',
u'\u94c0': 'CjkUnifiedIdeograph94c0',
u'\u7043': 'CjkUnifiedIdeograph7043',
u'\u57c2': 'CjkUnifiedIdeograph57c2',
u'\ufe47': 'PresentationFormForVerticalLeftSquareBracket',
u'\uddc6': 'UnicodeCharacter0xddc6',
u'\u274d': 'ShadowedWhiteCircle',
u'\u06cc': 'ArabicLetterFarsiYeh',
u'\uad51': 'HangulSyllableGyog',
u'\u8cd0': 'CjkUnifiedIdeograph8cd0',
u'\u6853': 'CjkUnifiedIdeograph6853',
u'\u4fd2': 'CjkUnifiedIdeograph4fd2',
u'\ud657': 'HangulSyllableHwags',
u'\u35d6': 'CjkUnifiedIdeograph35d6',
u'\ub3da': 'HangulSyllableDoj',
u'\u1f5d': 'GreekCapitalLetterUpsilonWithDasiaAndOxia',
u'\u7edc': 'CjkUnifiedIdeograph7edc',
u'\u8561': 'CjkUnifiedIdeograph8561',
u'\ue4e0': 'UnicodeCharacter0xe4e0',
u'\u4063': 'CjkUnifiedIdeograph4063',
u'\uce67': 'HangulSyllableCilh',
u'\u2de6': 'CombiningCyrillicLetterKa',
u'\ub46b': 'HangulSyllableDuh',
u'\uabea': 'UnicodeCharacter0xabea',
u'\u776d': 'CjkUnifiedIdeograph776d',
u'\u56ec': 'CjkUnifiedIdeograph56ec',
u'\ufd71': 'ArabicLigatureTahWithMeemWithHahFinalForm',
u'\udcf0': 'UnicodeCharacter0xdcf0',
u'\u2677': 'RecyclingSymbolForType5Plastics',
u'\u05f6': 'UnicodeCharacter0x5f6',
u'\uac7b': 'HangulSyllableGeolb',
u'\u83fa': 'CjkUnifiedIdeograph83fa',
u'\u6f7d': 'CjkUnifiedIdeograph6f7d',
u'\u4efc': 'CjkUnifiedIdeograph4efc',
u'\u5481': 'CjkUnifiedIdeograph5481',
u'\u3500': 'CjkUnifiedIdeograph3500',
u'\u1183': 'HangulJungseongOu',
u'\uf602': 'UnicodeCharacter0xf602',
u'\ud285': 'HangulSyllableTwinj',
u'\ub304': 'HangulSyllableDaen',
u'\u9f87': 'CjkUnifiedIdeograph9f87',
u'\u600e': 'CjkUnifiedIdeograph600e',
u'\u4c91': 'CjkUnifiedIdeograph4c91',
u'\u2d10': 'GeorgianSmallLetterRae',
u'\u0993': 'BengaliLetterO',
u'\uee12': 'UnicodeCharacter0xee12',
u'\uca95': 'HangulSyllableJjyeob',
u'\uab14': 'UnicodeCharacter0xab14',
u'\uf797': 'UnicodeCharacter0xf797',
u'\ub099': 'HangulSyllableNag',
u'\u581e': 'CjkUnifiedIdeograph581e',
u'\u0520': 'CyrillicCapitalLetterElWithMiddleHook',
u'\u61a3': 'CjkUnifiedIdeograph61a3',
u'\uc622': 'HangulSyllableYep',
u'\u22a5': 'UpTack',
u'\u8324': 'CjkUnifiedIdeograph8324',
u'\uefa7': 'UnicodeCharacter0xefa7',
u'\ua8a9': 'SaurashtraLetterBha',
u'\u7d30': 'CjkUnifiedIdeograph7d30',
u'\u59b3': 'CjkUnifiedIdeograph59b3',
u'\u3e32': 'CjkUnifiedIdeograph3e32',
u'\u1ab5': 'UnicodeCharacter0x1ab5',
u'\ufb34': 'HebrewLetterHeWithMapiq',
u'\uc7b7': 'HangulSyllableJaelb',
u'\ua436': 'YiSyllableNjiex',
u'\u80b9': 'CjkUnifiedIdeograph80b9',
u'\u5540': 'CjkUnifiedIdeograph5540',
u'\u1642': 'CanadianSyllabicsCarrierZe',
u'\u72c5': 'CjkUnifiedIdeograph72c5',
u'\ud344': 'HangulSyllablePya',
u'\u3fc7': 'CjkUnifiedIdeograph3fc7',
u'\u9c46': 'CjkUnifiedIdeograph9c46',
u'\uf8c9': 'UnicodeCharacter0xf8c9',
u'\ua5cb': 'VaiSyllableTo',
u'\u4d50': 'CjkUnifiedIdeograph4d50',
u'\u0e52': 'ThaiDigitTwo',
u'\u6ad5': 'CjkUnifiedIdeograph6ad5',
u'\ucb54': 'HangulSyllableJjuls',
u'\u17d7': 'KhmerSignLekToo',
u'\uf456': 'UnicodeCharacter0xf456',
u'\ud0d9': 'HangulSyllableTat',
u'\ub158': 'HangulSyllableNyeok',
u'\u9ddb': 'CjkUnifiedIdeograph9ddb',
u'\u6662': 'CjkUnifiedIdeograph6662',
u'\u42e5': 'CjkUnifiedIdeograph42e5',
u'\u2364': 'AplFunctionalSymbolJotDiaeresis',
u'\u0fe7': 'UnicodeCharacter0xfe7',
u'\uec66': 'UnicodeCharacter0xec66',
u'\uc8e9': 'HangulSyllableJyolg',
u'\ua968': 'UnicodeCharacter0xa968',
u'\uf5eb': 'UnicodeCharacter0xf5eb',
u'\ub6ed': 'HangulSyllableDdwet',
u'\u5e72': 'CjkUnifiedIdeograph5e72',
u'\u1b74': 'BalineseMusicalSymbolRighthandOpenDug',
u'\u67f7': 'CjkUnifiedIdeograph67f7',
u'\uc476': 'HangulSyllableSsubs',
u'\u20f9': 'UnicodeCharacter0x20f9',
u'\u8178': 'CjkUnifiedIdeograph8178',
u'\uedfb': 'UnicodeCharacter0xedfb',
u'\uaefd': 'HangulSyllableGgyeolg',
u'\u9401': 'CjkUnifiedIdeograph9401',
u'\u7384': 'CjkUnifiedIdeograph7384',
u'\u3c86': 'CjkUnifiedIdeograph3c86',
u'\uf988': 'CjkCompatibilityIdeographf988',
u'\u450b': 'CjkUnifiedIdeograph450b',
u'\uba8a': 'HangulSyllableMyeop',
u'\u060d': 'ArabicDateSeparator',
u'\uc30f': 'HangulSyllableSsac',
u'\u8c11': 'CjkUnifiedIdeograph8c11',
u'\u6b94': 'CjkUnifiedIdeograph6b94',
u'\u1496': 'CanadianSyllabicsCwii',
u'\ud198': 'HangulSyllableTyess',
u'\u929a': 'CjkUnifiedIdeograph929a',
u'\u7e1d': 'CjkUnifiedIdeograph7e1d',
u'\u3b1f': 'CjkUnifiedIdeograph3b1f',
u'\ue421': 'UnicodeCharacter0xe421',
u'\ua123': 'YiSyllableTa',
u'\u43a4': 'CjkUnifiedIdeograph43a4',
u'\u0ca6': 'KannadaLetterDa',
u'\uc9a8': 'HangulSyllableJyin',
u'\u8aaa': 'CjkUnifiedIdeograph8aaa',
u'\u562d': 'CjkUnifiedIdeograph562d',
u'\ub7ac': 'HangulSyllableRaess',
u'\u132f': 'EthiopicSyllableChwa',
u'\udc31': 'UnicodeCharacter0xdc31',
u'\u9933': 'CjkUnifiedIdeograph9933',
u'\u64b6': 'CjkUnifiedIdeograph64b6',
u'\u21b8': 'NorthWestArrowToLongBar',
u'\ue2ba': 'UnicodeCharacter0xe2ba',
u'\u4e3d': 'CjkUnifiedIdeograph4e3d',
u'\uafbc': 'HangulSyllableGgun',
u'\u0b3f': 'OriyaVowelSignI',
u'\u3441': 'CjkUnifiedIdeograph3441',
u'\uf143': 'UnicodeCharacter0xf143',
u'\ub245': 'HangulSyllableNwelg',
u'\u5cc6': 'CjkUnifiedIdeograph5cc6',
u'\u19c8': 'NewTaiLueToneMark1',
u'\udaca': 'UnicodeCharacter0xdaca',
u'\u87cc': 'CjkUnifiedIdeograph87cc',
u'\u634f': 'CjkUnifiedIdeograph634f',
u'\u2c51': 'GlagoliticSmallLetterYati',
u'\ue953': 'UnicodeCharacter0xe953',
u'\uaa55': 'ChamDigitFive',
u'\u71d8': 'CjkUnifiedIdeograph71d8',
u'\u32da': 'CircledKatakanaSa',
u'\uffdc': 'HalfwidthHangulLetterI',
u'\u5b5f': 'CjkUnifiedIdeograph5b5f',
u'\ub8de': 'HangulSyllableRyobs',
u'\u0461': 'CyrillicSmallLetterOmega',
u'\uc163': 'HangulSyllableSyeolh',
u'\u8265': 'CjkUnifiedIdeograph8265',
u'\u69e8': 'CjkUnifiedIdeograph69e8',
u'\u2aea': 'DoubleDownTack',
u'\ud7ec': 'UnicodeCharacter0xd7ec',
u'\u90ee': 'CjkUnifiedIdeograph90ee',
u'\u7c71': 'CjkUnifiedIdeograph7c71',
u'\u3973': 'CjkUnifiedIdeograph3973',
u'\ufa75': 'CjkCompatibilityIdeographfa75',
u'\ua777': 'LatinSmallLetterTum',
u'\u41f8': 'CjkUnifiedIdeograph41f8',
u'\u02fa': 'ModifierLetterEndHighTone',
u'\ucffc': 'HangulSyllableKweo',
u'\u88fe': 'CjkUnifiedIdeograph88fe',
u'\ub600': 'HangulSyllableDdyels',
u'\u9283': 'CjkUnifiedIdeograph9283',
u'\u7502': 'CjkUnifiedIdeograph7502',
u'\u5185': 'CjkUnifiedIdeograph5185',
u'\uff06': 'FullwidthAmpersand',
u'\udb89': 'UnicodeCharacter0xdb89',
u'\u240c': 'SymbolForFormFeed',
u'\x8f': 'UnicodeCharacter0x8f',
u'\uae10': 'HangulSyllableGeuk',
u'\u8a93': 'CjkUnifiedIdeograph8a93',
u'\u6d12': 'CjkUnifiedIdeograph6d12',
u'\u4995': 'CjkUnifiedIdeograph4995',
u'\ud716': 'HangulSyllableHwep',
u'\u3399': 'SquareFm',
u'\ubd9d': 'HangulSyllableBweog',
u'\u1c1c': 'LepchaLetterLa',
u'\u789f': 'CjkUnifiedIdeograph789f',
u'\u8620': 'CjkUnifiedIdeograph8620',
u'\ue2a3': 'UnicodeCharacter0xe2a3',
u'\u4522': 'CjkUnifiedIdeograph4522',
u'\ucf26': 'HangulSyllableKyeolm',
u'\u2ba9': 'UnicodeCharacter0x2ba9',
u'\u95ad': 'CjkUnifiedIdeograph95ad',
u'\u742c': 'CjkUnifiedIdeograph742c',
u'\u50af': 'CjkUnifiedIdeograph50af',
u'\ufe30': 'PresentationFormForVerticalTwoDotLeader',
u'\udab3': 'UnicodeCharacter0xdab3',
u'\u2736': 'SixPointedBlackStar',
u'\u03b9': 'GreekSmallLetterIota',
u'\ua13a': 'YiSyllableDdiex',
u'\u8dbd': 'CjkUnifiedIdeograph8dbd',
u'\u6c3c': 'CjkUnifiedIdeograph6c3c',
u'\u48bf': 'CjkUnifiedIdeograph48bf',
u'\ud640': 'HangulSyllableHol',
u'\u32c3': 'IdeographicTelegraphSymbolForApril',
u'\ubcc7': 'HangulSyllableByeolb',
u'\u1f46': 'UnicodeCharacter0x1f46',
u'\u7bc9': 'CjkUnifiedIdeograph7bc9',
u'\u994a': 'CjkUnifiedIdeograph994a',
u'\ue5cd': 'UnicodeCharacter0xe5cd',
u'\u444c': 'CjkUnifiedIdeograph444c',
u'\uce50': 'HangulSyllableCyiss',
u'\u2ad3': 'SubsetAboveSuperset',
u'\u94d7': 'CjkUnifiedIdeograph94d7',
u'\u7756': 'CjkUnifiedIdeograph7756',
u'\u53d9': 'CjkUnifiedIdeograph53d9',
u'\uf15a': 'UnicodeCharacter0xf15a',
u'\udddd': 'UnicodeCharacter0xdddd',
u'\u2660': 'BlackSpadeSuit',
u'\u02e3': 'ModifierLetterSmallX',
u'\ua064': 'YiSyllableBbuop',
u'\u8ce7': 'CjkUnifiedIdeograph8ce7',
u'\u6f66': 'CjkUnifiedIdeograph6f66',
u'\u4be9': 'CjkUnifiedIdeograph4be9',
u'\ue96a': 'UnicodeCharacter0xe96a',
u'\u35ed': 'CjkUnifiedIdeograph35ed',
u'\ubff1': 'HangulSyllableBbweolg',
u'\u1e70': 'LatinCapitalLetterTWithCircumflexBelow',
u'\u7af3': 'CjkUnifiedIdeograph7af3',
u'\u9874': 'CjkUnifiedIdeograph9874',
u'\ue4f7': 'UnicodeCharacter0xe4f7',
u'\u4776': 'CjkUnifiedIdeograph4776',
u'\uc17a': 'HangulSyllableSyelm',
u'\u2dfd': 'CombiningCyrillicLetterLittleYus',
u'\u1701': 'TagalogLetterI',
u'\u7680': 'CjkUnifiedIdeograph7680',
u'\u9105': 'CjkUnifiedIdeograph9105',
u'\uf084': 'UnicodeCharacter0xf084',
u'\u5c07': 'CjkUnifiedIdeograph5c07',
u'\uc60b': 'HangulSyllableYegs',
u'\u398a': 'CjkUnifiedIdeograph398a',
u'\ua38e': 'YiSyllableRryt',
u'\u0f11': 'TibetanMarkRinChenSpungsShad',
u'\u6e90': 'CjkUnifiedIdeograph6e90',
u'\u8915': 'CjkUnifiedIdeograph8915',
u'\ue894': 'UnicodeCharacter0xe894',
u'\u3e1b': 'CjkUnifiedIdeograph3e1b',
u'\u119a': 'HangulJungseongIo',
u'\ub81f': 'HangulSyllableRec',
u'\u9b9e': 'CjkUnifiedIdeograph9b9e',
u'\u6721': 'CjkUnifiedIdeograph6721',
u'\u46a0': 'CjkUnifiedIdeograph46a0',
u'\ue125': 'UnicodeCharacter0xe125',
u'\uc0a4': 'HangulSyllableBbiss',
u'\u162b': 'CanadianSyllabicsCarrierDlee',
u'\u09aa': 'BengaliLetterPa',
u'\u902f': 'CjkUnifiedIdeograph902f',
u'\uf3ae': 'UnicodeCharacter0xf3ae',
u'\u5f31': 'CjkUnifiedIdeograph5f31',
u'\ud935': 'UnicodeCharacter0xd935',
u'\u38b4': 'CjkUnifiedIdeograph38b4',
u'\ua2b8': 'YiSyllableCa',
u'\u0e3b': 'UnicodeCharacter0xe3b',
u'\u61ba': 'CjkUnifiedIdeograph61ba',
u'\u883f': 'CjkUnifiedIdeograph883f',
u'\uebbe': 'UnicodeCharacter0xebbe',
u'\u3145': 'HangulLetterSios',
u'\u10c4': 'GeorgianCapitalLetterHar',
u'\ubb49': 'HangulSyllableMung',
u'\u9ac8': 'CjkUnifiedIdeograph9ac8',
u'\u664b': 'CjkUnifiedIdeograph664b',
u'\u59ca': 'CjkUnifiedIdeograph59ca',
u'\ue04f': 'UnicodeCharacter0xe04f',
u'\uc3ce': 'HangulSyllableSsyebs',
u'\u2955': 'DownwardsHarpoonWithBarbRightToBar',
u'\u08d4': 'UnicodeCharacter0x8d4',
u'\u9359': 'CjkUnifiedIdeograph9359',
u'\uf2d8': 'UnicodeCharacter0xf2d8',
u'\u5e5b': 'CjkUnifiedIdeograph5e5b',
u'\ud85f': 'UnicodeCharacter0xd85f',
u'\u3bde': 'CjkUnifiedIdeograph3bde',
u'\ua5e2': 'VaiSyllableEn',
u'\u0165': 'LatinSmallLetterTWithCaron',
u'\u60e4': 'CjkUnifiedIdeograph60e4',
u'\u8b69': 'CjkUnifiedIdeograph8b69',
u'\ueae8': 'UnicodeCharacter0xeae8',
u'\u306f': 'HiraganaLetterHa',
u'\u13ee': 'CherokeeLetterWv',
u'\uba73': 'HangulSyllableMyeogs',
u'\u9df2': 'CjkUnifiedIdeograph9df2',
u'\u7975': 'CjkUnifiedIdeograph7975',
u'\u58f4': 'CjkUnifiedIdeograph58f4',
u'\ue379': 'UnicodeCharacter0xe379',
u'\uc2f8': 'HangulSyllableSsa',
u'\u287f': 'BraillePatternDots1234567',
u'\u0bfe': 'UnicodeCharacter0xbfe',
u'\u7e06': 'CjkUnifiedIdeograph7e06',
u'\u5a89': 'CjkUnifiedIdeograph5a89',
u'\u3b08': 'CjkUnifiedIdeograph3b08',
u'\u078b': 'ThaanaLetterDhaalu',
u'\uf80a': 'UnicodeCharacter0xf80a',
u'\uc48d': 'HangulSyllableSsweolt',
u'\ua50c': 'VaiSyllableFee',
u'\u818f': 'CjkUnifiedIdeograph818f',
u'\u5616': 'CjkUnifiedIdeograph5616',
u'\u1318': 'EthiopicSyllableGga',
u'\u7f9b': 'CjkUnifiedIdeograph7f9b',
u'\ud01a': 'HangulSyllableKwegg',
u'\u3c9d': 'CjkUnifiedIdeograph3c9d',
u'\u9d1c': 'CjkUnifiedIdeograph9d1c',
u'\uf99f': 'CjkCompatibilityIdeographf99f',
u'\ua6a1': 'UnicodeCharacter0xa6a1',
u'\u4e26': 'CjkUnifiedIdeograph4e26',
u'\u0b28': 'OriyaLetterNa',
u'\u57ab': 'CjkUnifiedIdeograph57ab',
u'\uc82a': 'HangulSyllableJelp',
u'\u14ad': 'CanadianSyllabicsWestcreeMwe',
u'\uf52c': 'UnicodeCharacter0xf52c',
u'\ud1af': 'HangulSyllableTolh',
u'\ub22e': 'HangulSyllableNweolp',
u'\u9eb1': 'CjkUnifiedIdeograph9eb1',
u'\u6338': 'CjkUnifiedIdeograph6338',
u'\u4fbb': 'CjkUnifiedIdeograph4fbb',
u'\u203a': 'SingleRightpointingAngleQuotationMark',
u'\u0cbd': 'KannadaSignAvagraha',
u'\ued3c': 'UnicodeCharacter0xed3c',
u'\uc9bf': 'HangulSyllableJyih',
u'\uaa3e': 'UnicodeCharacter0xaa3e',
u'\uf6c1': 'UnicodeCharacter0xf6c1',
u'\ub3c3': 'HangulSyllableDyeh',
u'\u5b48': 'CjkUnifiedIdeograph5b48',
u'\u184a': 'MongolianLetterTodoAng',
u'\u64cd': 'CjkUnifiedIdeograph64cd',
u'\uc54c': 'HangulSyllableAl',
u'\u21cf': 'RightwardsDoubleArrowWithStroke',
u'\u824e': 'CjkUnifiedIdeograph824e',
u'\ueed1': 'UnicodeCharacter0xeed1',
u'\uabd3': 'UnicodeCharacter0xabd3',
u'\u705a': 'CjkUnifiedIdeograph705a',
u'\u5cdd': 'CjkUnifiedIdeograph5cdd',
u'\u3d5c': 'CjkUnifiedIdeograph3d5c',
u'\u19df': 'NewTaiLueSignLaev',
u'\ufa5e': 'CjkCompatibilityIdeographfa5e',
u'\uc6e1': 'HangulSyllableWeong',
u'\ua760': 'LatinCapitalLetterVy',
u'\u83e3': 'CjkUnifiedIdeograph83e3',
u'\u686a': 'CjkUnifiedIdeograph686a',
u'\u156c': 'CanadianSyllabicsTthi',
u'\u71ef': 'CjkUnifiedIdeograph71ef',
u'\ud26e': 'HangulSyllableTwelm',
u'\u3ef1': 'CjkUnifiedIdeograph3ef1',
u'\u9f70': 'CjkUnifiedIdeograph9f70',
u'\ufbf3': 'ArabicLigatureYehWithHamzaAboveWithOeFinalForm',
u'\ub8f5': 'HangulSyllableRult',
u'\u407a': 'CjkUnifiedIdeograph407a',
u'\u0d7c': 'MalayalamLetterChilluRr',
u'\u69ff': 'CjkUnifiedIdeograph69ff',
u'\uca7e': 'HangulSyllableJjej',
u'\uf780': 'UnicodeCharacter0xf780',
u'\u5303': 'CjkUnifiedIdeograph5303',
u'\ub482': 'HangulSyllableDweoj',
u'\u1005': 'MyanmarLetterCa',
u'\udd07': 'UnicodeCharacter0xdd07',
u'\u9a09': 'CjkUnifiedIdeograph9a09',
u'\u658c': 'CjkUnifiedIdeograph658c',
u'\u228e': 'MultisetUnion',
u'\uef90': 'UnicodeCharacter0xef90',
u'\u4b13': 'CjkUnifiedIdeograph4b13',
u'\uac92': 'HangulSyllableGenh',
u'\u0815': 'UnicodeCharacter0x815',
u'\u3517': 'CjkUnifiedIdeograph3517',
u'\uf219': 'UnicodeCharacter0xf219',
u'\ubf1b': 'HangulSyllableBbyeos',
u'\u5d9c': 'CjkUnifiedIdeograph5d9c',
u'\u1a9e': 'UnicodeCharacter0x1a9e',
u'\uc7a0': 'HangulSyllableJam',
u'\u84a2': 'CjkUnifiedIdeograph84a2',
u'\u6025': 'CjkUnifiedIdeograph6025',
u'\u2d27': 'UnicodeCharacter0x2d27',
u'\uea29': 'UnicodeCharacter0xea29',
u'\u972b': 'CjkUnifiedIdeograph972b',
u'\u72ae': 'CjkUnifiedIdeograph72ae',
u'\u3fb0': 'CjkUnifiedIdeograph3fb0',
u'\ufcb2': 'ArabicLigatureSadWithKhahInitialForm',
u'\u5835': 'CjkUnifiedIdeograph5835',
u'\ub9b4': 'HangulSyllableRil',
u'\u0537': 'ArmenianCapitalLetterEh',
u'\uc239': 'HangulSyllableSweonj',
u'\u8f3b': 'CjkUnifiedIdeograph8f3b',
u'\u6abe': 'CjkUnifiedIdeograph6abe',
u'\u17c0': 'KhmerVowelSignIe',
u'\ud4c2': 'HangulSyllablePwebs',
u'\u91c4': 'CjkUnifiedIdeograph91c4',
u'\u7d47': 'CjkUnifiedIdeograph7d47',
u'\u3a49': 'CjkUnifiedIdeograph3a49',
u'\ue74b': 'UnicodeCharacter0xe74b',
u'\ua44d': 'YiSyllableNyip',
u'\u42ce': 'CjkUnifiedIdeograph42ce',
u'\u0fd0': 'TibetanMarkBskaShogGiMgoRgyan',
u'\uccd2': 'HangulSyllableCyeogg',
u'\u89d4': 'CjkUnifiedIdeograph89d4',
u'\u5557': 'CjkUnifiedIdeograph5557',
u'\ub6d6': 'HangulSyllableDdwegg',
u'\u1259': 'UnicodeCharacter0x1259',
u'\udf5b': 'UnicodeCharacter0xdf5b',
u'\u9c5d': 'CjkUnifiedIdeograph9c5d',
u'\u67e0': 'CjkUnifiedIdeograph67e0',
u'\u24e2': 'CircledLatinSmallLetterS',
u'\ue1e4': 'UnicodeCharacter0xe1e4',
u'\u4d67': 'CjkUnifiedIdeograph4d67',
u'\uaee6': 'HangulSyllableGgelp',
u'\u0a69': 'GurmukhiDigitThree',
u'\u376b': 'CjkUnifiedIdeograph376b',
u'\uf46d': 'UnicodeCharacter0xf46d',
u'\ub16f': 'HangulSyllableNyes',
u'\u5ff0': 'CjkUnifiedIdeograph5ff0',
u'\u1cf2': 'UnicodeCharacter0x1cf2',
u'\ud9f4': 'UnicodeCharacter0xd9f4',
u'\u86f6': 'CjkUnifiedIdeograph86f6',
u'\u6279': 'CjkUnifiedIdeograph6279',
u'\u2f7b': 'KangxiRadicalFeather',
u'\uec7d': 'UnicodeCharacter0xec7d',
u'\ua97f': 'UnicodeCharacter0xa97f',
u'\ud981': 'UnicodeCharacter0xd981',
u'\u3e04': 'CjkUnifiedIdeograph3e04',
u'\u1287': 'EthiopicSyllableXoa',
u'\ub408': 'HangulSyllableDwaels',
u'\u888b': 'CjkUnifiedIdeograph888b',
u'\u770a': 'CjkUnifiedIdeograph770a',
u'\u4b8d': 'CjkUnifiedIdeograph4b8d',
u'\ued0e': 'UnicodeCharacter0xed0e',
u'\uc191': 'HangulSyllableSonj',
u'\u2614': 'UmbrellaWithRainDrops',
u'\u7a97': 'CjkUnifiedIdeograph7a97',
u'\u9c18': 'CjkUnifiedIdeograph9c18',
u'\uf09b': 'UnicodeCharacter0xf09b',
u'\u5f1a': 'CjkUnifiedIdeograph5f1a',
u'\ud51e': 'HangulSyllablePeup',
u'\u29a1': 'SphericalAngleOpeningUp',
u'\uafa5': 'HangulSyllableGgyolg',
u'\u0e24': 'ThaiCharacterRu',
u'\u62a7': 'CjkUnifiedIdeograph62a7',
u'\u8428': 'CjkUnifiedIdeograph8428',
u'\ud8ab': 'UnicodeCharacter0xd8ab',
u'\u472a': 'CjkUnifiedIdeograph472a',
u'\u3d2e': 'CjkUnifiedIdeograph3d2e',
u'\u11b1': 'HangulJongseongRieulmieum',
u'\ub332': 'HangulSyllableDyaj',
u'\u97b5': 'CjkUnifiedIdeograph97b5',
u'\u7634': 'CjkUnifiedIdeograph7634',
u'\u4ab7': 'CjkUnifiedIdeograph4ab7',
u'\uec38': 'UnicodeCharacter0xec38',
u'\uc0bb': 'HangulSyllableSalh',
u'\u253e': 'BoxDrawingsRightHeavyAndLeftVerticalLight',
u'\u79c1': 'CjkUnifiedIdeograph79c1',
u'\u9b42': 'CjkUnifiedIdeograph9b42',
u'\uffc5': 'HalfwidthHangulLetterYae',
u'\u5e44': 'CjkUnifiedIdeograph5e44',
u'\ud448': 'HangulSyllablePoel',
u'\u28cb': 'BraillePatternDots12478',
u'\uaecf': 'HangulSyllableGgeos',
u'\u0d4e': 'UnicodeCharacter0xd4e',
u'\u61d1': 'CjkUnifiedIdeograph61d1',
u'\u8352': 'CjkUnifiedIdeograph8352',
u'\ue7d5': 'UnicodeCharacter0xe7d5',
u'\u4654': 'CjkUnifiedIdeograph4654',
u'\u3c58': 'CjkUnifiedIdeograph3c58',
u'\u10db': 'GeorgianLetterMan',
u'\ub25c': 'HangulSyllableNwin',
u'\u96df': 'CjkUnifiedIdeograph96df',
u'\u755e': 'CjkUnifiedIdeograph755e',
u'\u49e1': 'CjkUnifiedIdeograph49e1',
u'\ueb62': 'UnicodeCharacter0xeb62',
u'\ucfe5': 'HangulSyllableKunj',
u'\u2468': 'CircledDigitNine',
u'\u78eb': 'CjkUnifiedIdeograph78eb',
u'\u9a6c': 'CjkUnifiedIdeograph9a6c',
u'\ufeef': 'ArabicLetterAlefMaksuraIsolatedForm',
u'\u5d6e': 'CjkUnifiedIdeograph5d6e',
u'\ud372': 'HangulSyllablePyaebs',
u'\u37f5': 'CjkUnifiedIdeograph37f5',
u'\uadf9': 'HangulSyllableGeug',
u'\u0c78': 'TeluguFractionDigitZeroForOddPowersOfFour',
u'\u60fb': 'CjkUnifiedIdeograph60fb',
u'\u827c': 'CjkUnifiedIdeograph827c',
u'\ue6ff': 'UnicodeCharacter0xe6ff',
u'\u457e': 'CjkUnifiedIdeograph457e',
u'\udc03': 'UnicodeCharacter0xdc03',
u'\u3b82': 'CjkUnifiedIdeograph3b82',
u'\ub186': 'HangulSyllableNolp',
u'\u1509': 'CanadianSyllabicsMoosecreeSk',
u'\u7488': 'CjkUnifiedIdeograph7488',
u'\u8b0d': 'CjkUnifiedIdeograph8b0d',
u'\uea8c': 'UnicodeCharacter0xea8c',
u'\u4e0f': 'CjkUnifiedIdeograph4e0f',
u'\uc413': 'HangulSyllableSswaegs',
u'\u2392': 'PassivepullupoutputSymbol',
u'\uba17': 'HangulSyllableMyac',
u'\u9996': 'CjkUnifiedIdeograph9996',
u'\u7d19': 'CjkUnifiedIdeograph7d19',
u'\u5c98': 'CjkUnifiedIdeograph5c98',
u'\uf31d': 'UnicodeCharacter0xf31d',
u'\ud29c': 'HangulSyllableTyu',
u'\u2c23': 'GlagoliticCapitalLetterYu',
u'\u0ba2': 'UnicodeCharacter0xba2',
u'\ua227': 'YiSyllableGgex',
u'\u81a6': 'CjkUnifiedIdeograph81a6',
u'\u6529': 'CjkUnifiedIdeograph6529',
u'\u44a8': 'CjkUnifiedIdeograph44a8',
u'\udb2d': 'UnicodeCharacter0xdb2d',
u'\u3aac': 'CjkUnifiedIdeograph3aac',
u'\ub0b0': 'HangulSyllableNak',
u'\u1433': 'CanadianSyllabicsPo',
u'\u73b2': 'CjkUnifiedIdeograph73b2',
u'\u8a37': 'CjkUnifiedIdeograph8a37',
u'\ue9b6': 'UnicodeCharacter0xe9b6',
u'\u4d39': 'CjkUnifiedIdeograph4d39',
u'\uc33d': 'HangulSyllableSsyalt',
u'\u22bc': 'Nand',
u'\ub941': 'HangulSyllableRwinj',
u'\u98c0': 'CjkUnifiedIdeograph98c0',
u'\u7c43': 'CjkUnifiedIdeograph7c43',
u'\u5bc2': 'CjkUnifiedIdeograph5bc2',
u'\uf247': 'UnicodeCharacter0xf247',
u'\ud1c6': 'HangulSyllableTwalm',
u'\u2b4d': 'UnicodeCharacter0x2b4d',
u'\u0acc': 'GujaratiVowelSignAu',
u'\ua151': 'YiSyllableNdit',
u'\u80d0': 'CjkUnifiedIdeograph80d0',
u'\u6453': 'CjkUnifiedIdeograph6453',
u'\u43d2': 'CjkUnifiedIdeograph43d2',
u'\uda57': 'UnicodeCharacter0xda57',
u'\u39d6': 'CjkUnifiedIdeograph39d6',
u'\ubfda': 'HangulSyllableBbulp',
u'\u135d': 'UnicodeCharacter0x135d',
u'\u72dc': 'CjkUnifiedIdeograph72dc',
u'\u8961': 'CjkUnifiedIdeograph8961',
u'\ue8e0': 'UnicodeCharacter0xe8e0',
u'\u4c63': 'CjkUnifiedIdeograph4c63',
u'\uc267': 'HangulSyllableSwec',
u'\u21e6': 'LeftwardsWhiteArrow',
u'\ub86b': 'HangulSyllableRolh',
u'\ua7ea': 'UnicodeCharacter0xa7ea',
u'\u7b6d': 'CjkUnifiedIdeograph7b6d',
u'\u5aec': 'CjkUnifiedIdeograph5aec',
u'\uf171': 'UnicodeCharacter0xf171',
u'\ud0f0': 'HangulSyllableTaess',
u'\u2a77': 'EqualsSignWithTwoDotsAboveAndTwoDotsBelow',
u'\u09f6': 'BengaliCurrencyNumeratorThree',
u'\ua07b': 'YiSyllableNbie',
u'\u8ffa': 'CjkUnifiedIdeograph8ffa',
u'\u637d': 'CjkUnifiedIdeograph637d',
u'\u42fc': 'CjkUnifiedIdeograph42fc',
u'\u5881': 'CjkUnifiedIdeograph5881',
u'\u3900': 'CjkUnifiedIdeograph3900',
u'\u1d83': 'LatinSmallLetterGWithPalatalHook',
u'\ufa02': 'CjkCompatibilityIdeographfa02',
u'\ude85': 'UnicodeCharacter0xde85',
u'\ubf04': 'HangulSyllableBbek',
u'\u9387': 'CjkUnifiedIdeograph9387',
u'\u6c0e': 'CjkUnifiedIdeograph6c0e',
u'\u4091': 'CjkUnifiedIdeograph4091',
u'\u2110': 'ScriptCapitalI',
u'\u0593': 'HebrewAccentShalshelet',
u'\ue212': 'UnicodeCharacter0xe212',
u'\uc695': 'HangulSyllableYog',
u'\ua714': 'ModifierLetterMidLeftstemToneBar',
u'\ufb97': 'ArabicLetterGuehFinalForm',
u'\ubc99': 'HangulSyllableBeong',
u'\u541e': 'CjkUnifiedIdeograph541e',
u'\u0920': 'DevanagariLetterTtha',
u'\u6da3': 'CjkUnifiedIdeograph6da3',
u'\uca22': 'HangulSyllableJjyalp',
u'\u2ea5': 'CjkRadicalPawTwo',
u'\u8f24': 'CjkUnifiedIdeograph8f24',
u'\ue3a7': 'UnicodeCharacter0xe3a7',
u'\ua4a9': 'YiRadicalOp',
u'\u7130': 'CjkUnifiedIdeograph7130',
u'\u55b3': 'CjkUnifiedIdeograph55b3',
u'\u3232': 'ParenthesizedIdeographHave',
u'\u16b5': 'RunicLetterG',
u'\uf734': 'UnicodeCharacter0xf734',
u'\ucbb7': 'HangulSyllableJjwih',
u'\ua836': 'UnicodeCharacter0xa836',
u'\u8cb9': 'CjkUnifiedIdeograph8cb9',
u'\u5940': 'CjkUnifiedIdeograph5940',
u'\u1a42': 'UnicodeCharacter0x1a42',
u'\u7ec5': 'CjkUnifiedIdeograph7ec5',
u'\udf44': 'UnicodeCharacter0xdf44',
u'\u33c7': 'SquareCo',
u'\u9046': 'CjkUnifiedIdeograph9046',
u'\uf4c9': 'UnicodeCharacter0xf4c9',
u'\ua9cb': 'UnicodeCharacter0xa9cb',
u'\u4150': 'CjkUnifiedIdeograph4150',
u'\u0252': 'LatinSmallLetterTurnedAlpha',
u'\u66d5': 'CjkUnifiedIdeograph66d5',
u'\uc754': 'HangulSyllableEuk',
u'\u1bd7': 'UnicodeCharacter0x1bd7',
u'\uf856': 'UnicodeCharacter0xf856',
u'\udcd9': 'UnicodeCharacter0xdcd9',
u'\ubd58': 'HangulSyllableBoem',
u'\u91db': 'CjkUnifiedIdeograph91db',
u'\u6a62': 'CjkUnifiedIdeograph6a62',
u'\u4ee5': 'CjkUnifiedIdeograph4ee5',
u'\u2f64': 'KangxiRadicalUse',
u'\u03e7': 'CopticSmallLetterKhei',
u'\ue066': 'UnicodeCharacter0xe066',
u'\uc4e9': 'HangulSyllableSsyung',
u'\ua568': 'VaiSyllableNja',
u'\uf9eb': 'CjkCompatibilityIdeographf9eb',
u'\ubaed': 'HangulSyllableMwaelt',
u'\u5272': 'CjkUnifiedIdeograph5272',
u'\u1774': 'UnicodeCharacter0x1774',
u'\u6bf7': 'CjkUnifiedIdeograph6bf7',
u'\uc876': 'HangulSyllableJonh',
u'\u2cf9': 'CopticOldNubianFullStop',
u'\u8d78': 'CjkUnifiedIdeograph8d78',
u'\ue1fb': 'UnicodeCharacter0xe1fb',
u'\ua2fd': 'YiSyllableNze',
u'\u9801': 'CjkUnifiedIdeograph9801',
u'\u7f84': 'CjkUnifiedIdeograph7f84',
u'\u3086': 'HiraganaLetterYu',
u'\uf588': 'UnicodeCharacter0xf588',
u'\u490b': 'CjkUnifiedIdeograph490b',
u'\ub68a': 'HangulSyllableDdyolm',
u'\u0a0d': 'UnicodeCharacter0xa0d',
u'\ucf0f': 'HangulSyllableKelh',
u'\u8011': 'CjkUnifiedIdeograph8011',
u'\u6794': 'CjkUnifiedIdeograph6794',
u'\u1896': 'MongolianLetterAliGaliZa',
u'\udd98': 'UnicodeCharacter0xdd98',
u'\u9e9a': 'CjkUnifiedIdeograph9e9a',
u'\u721d': 'CjkUnifiedIdeograph721d',
u'\u371f': 'CjkUnifiedIdeograph371f',
u'\ue821': 'UnicodeCharacter0xe821',
u'\uad23': 'HangulSyllableGwaelb',
u'\u4fa4': 'CjkUnifiedIdeograph4fa4',
u'\xa6': 'BrokenBar',
u'\uc5a8': 'HangulSyllableYaem',
u'\u86aa': 'CjkUnifiedIdeograph86aa',
u'\u5a2d': 'CjkUnifiedIdeograph5a2d',
u'\ubbac': 'HangulSyllableMyul',
u'\u1f2f': 'GreekCapitalLetterEtaWithDasiaAndPerispomeni',
u'\ud031': 'HangulSyllableKwet',
u'\u9533': 'CjkUnifiedIdeograph9533',
u'\u68b6': 'CjkUnifiedIdeograph68b6',
u'\u2db8': 'EthiopicSyllableCcha',
u'\ueeba': 'UnicodeCharacter0xeeba',
u'\u423d': 'CjkUnifiedIdeograph423d',
u'\ua3bc': 'YiSyllableShu',
u'\u073f': 'SyriacRwaha',
u'\u3841': 'CjkUnifiedIdeograph3841',
u'\ufd43': 'UnicodeCharacter0xfd43',
u'\ube45': 'HangulSyllableBig',
u'\u50c6': 'CjkUnifiedIdeograph50c6',
u'\u15c8': 'CanadianSyllabicsCarrierGhi',
u'\ud6ca': 'HangulSyllableHunh',
u'\u8bcc': 'CjkUnifiedIdeograph8bcc',
u'\u6f4f': 'CjkUnifiedIdeograph6f4f',
u'\u2051': 'TwoAsterisksAlignedVertically',
u'\ue553': 'UnicodeCharacter0xe553',
u'\ua655': 'CyrillicSmallLetterReversedYu',
u'\u7dd8': 'CjkUnifiedIdeograph7dd8',
u'\u3eda': 'CjkUnifiedIdeograph3eda',
u'\uf3dc': 'UnicodeCharacter0xf3dc',
u'\u575f': 'CjkUnifiedIdeograph575f',
u'\ub4de': 'HangulSyllableDeugg',
u'\u0861': 'UnicodeCharacter0x861',
u'\ucd63': 'HangulSyllableCoed',
u'\u8e65': 'CjkUnifiedIdeograph8e65',
u'\u65e8': 'CjkUnifiedIdeograph65e8',
u'\u26ea': 'UnicodeCharacter0x26ea',
u'\udbec': 'UnicodeCharacter0xdbec',
u'\u9cee': 'CjkUnifiedIdeograph9cee',
u'\u7071': 'CjkUnifiedIdeograph7071',
u'\u3573': 'CjkUnifiedIdeograph3573',
u'\uf675': 'UnicodeCharacter0xf675',
u'\uab77': 'UnicodeCharacter0xab77',
u'\u4df8': 'HexagramForTheGentleWind',
u'\u0efa': 'UnicodeCharacter0xefa',
u'\uc3fc': 'HangulSyllableSswal',
u'\u84fe': 'CjkUnifiedIdeograph84fe',
u'\uba00': 'HangulSyllableMya',
u'\u9e83': 'CjkUnifiedIdeograph9e83',
u'\u7902': 'CjkUnifiedIdeograph7902',
u'\u5d85': 'CjkUnifiedIdeograph5d85',
u'\uf306': 'UnicodeCharacter0xf306',
u'\ud789': 'HangulSyllableHig',
u'\u280c': 'BraillePatternDots34',
u'\u0c8f': 'KannadaLetterEe',
u'\ua210': 'YiSyllableKu',
u'\u8693': 'CjkUnifiedIdeograph8693',
u'\u6112': 'CjkUnifiedIdeograph6112',
u'\u4595': 'CjkUnifiedIdeograph4595',
u'\udb16': 'UnicodeCharacter0xdb16',
u'\u3f99': 'CjkUnifiedIdeograph3f99',
u'\ub19d': 'HangulSyllableNwalg',
u'\u101c': 'MyanmarLetterLa',
u'\u749f': 'CjkUnifiedIdeograph749f',
u'\u8a20': 'CjkUnifiedIdeograph8a20',
u'\ueea3': 'UnicodeCharacter0xeea3',
u'\u4922': 'CjkUnifiedIdeograph4922',
u'\uc326': 'HangulSyllableSsaebs',
u'\u27a9': 'RightshadedWhiteRightwardsArrow',
u'\u99ad': 'CjkUnifiedIdeograph99ad',
u'\u782c': 'CjkUnifiedIdeograph782c',
u'\u5caf': 'CjkUnifiedIdeograph5caf',
u'\uf230': 'UnicodeCharacter0xf230',
u'\ud6b3': 'HangulSyllableHyolb',
u'\u2b36': 'LeftwardsTwoheadedArrowFromBar',
u'\u0fb9': 'TibetanSubjoinedLetterKssa',
u'\uad3a': 'HangulSyllableGoenh',
u'\u81bd': 'CjkUnifiedIdeograph81bd',
u'\u603c': 'CjkUnifiedIdeograph603c',
u'\u44bf': 'CjkUnifiedIdeograph44bf',
u'\uda40': 'UnicodeCharacter0xda40',
u'\u3ec3': 'CjkUnifiedIdeograph3ec3',
u'\ub0c7': 'HangulSyllableNaes',
u'\u1346': 'EthiopicSyllableTzo',
u'\u77c9': 'CjkUnifiedIdeograph77c9',
u'\u954a': 'CjkUnifiedIdeograph954a',
u'\ue9cd': 'UnicodeCharacter0xe9cd',
u'\u484c': 'CjkUnifiedIdeograph484c',
u'\uc250': 'HangulSyllableSwe',
u'\u26d3': 'UnicodeCharacter0x26d3',
u'\u98d7': 'CjkUnifiedIdeograph98d7',
u'\u7b56': 'CjkUnifiedIdeograph7b56',
u'\u5fd9': 'CjkUnifiedIdeograph5fd9',
u'\ufd5a': 'ArabicLigatureHahWithMeemWithYehFinalForm',
u'\ud1dd': 'HangulSyllableTwaenj',
u'\u2a60': 'LogicalAndWithDoubleUnderbar',
u'\u0ee3': 'UnicodeCharacter0xee3',
u'\uac64': 'HangulSyllableGyaem',
u'\u80e7': 'CjkUnifiedIdeograph80e7',
u'\u6366': 'CjkUnifiedIdeograph6366',
u'\u47e9': 'CjkUnifiedIdeograph47e9',
u'\ue56a': 'UnicodeCharacter0xe56a',
u'\u39ed': 'CjkUnifiedIdeograph39ed',
u'\ub3f1': 'HangulSyllableDwab',
u'\u1270': 'EthiopicSyllableTa',
u'\u76f3': 'CjkUnifiedIdeograph76f3',
u'\u9474': 'CjkUnifiedIdeograph9474',
u'\ue8f7': 'UnicodeCharacter0xe8f7',
u'\u4b76': 'CjkUnifiedIdeograph4b76',
u'\ucd7a': 'HangulSyllableCyogg',
u'\u21fd': 'LeftwardsOpenheadedArrow',
u'\u1b01': 'BalineseSignUluCandra',
u'\u7a80': 'CjkUnifiedIdeograph7a80',
u'\u9d05': 'CjkUnifiedIdeograph9d05',
u'\ufc84': 'ArabicLigatureKafWithYehFinalForm',
u'\u5007': 'CjkUnifiedIdeograph5007',
u'\uca0b': 'HangulSyllableJjaes',
u'\u358a': 'CjkUnifiedIdeograph358a',
u'\uaf8e': 'HangulSyllableGgoelp',
u'\u0311': 'CombiningInvertedBreve',
u'\u6290': 'CjkUnifiedIdeograph6290',
u'\u8515': 'CjkUnifiedIdeograph8515',
u'\ue494': 'UnicodeCharacter0xe494',
u'\u321b': 'ParenthesizedHangulHieuhA',
u'\u1d9a': 'LatinSmallLetterEzhWithRetroflexHook',
u'\ub41f': 'HangulSyllableDoed',
u'\u979e': 'CjkUnifiedIdeograph979e',
u'\u6b21': 'CjkUnifiedIdeograph6b21',
u'\u4aa0': 'CjkUnifiedIdeograph4aa0',
u'\ued25': 'UnicodeCharacter0xed25',
u'\ucca4': 'HangulSyllableCeols',
u'\u1a2b': 'UnicodeCharacter0x1a2b',
u'\u05aa': 'HebrewAccentYerahBenYomo',
u'\u9c2f': 'CjkUnifiedIdeograph9c2f',
u'\uffae': 'HalfwidthHangulLetterRieulthieuth',
u'\u5331': 'CjkUnifiedIdeograph5331',
u'\ud535': 'HangulSyllablePying',
u'\u34b4': 'CjkUnifiedIdeograph34b4',
u'\uaeb8': 'HangulSyllableGgyaek',
u'\u023b': 'LatinCapitalLetterCWithStroke',
u'\u6dba': 'CjkUnifiedIdeograph6dba',
u'\u843f': 'CjkUnifiedIdeograph843f',
u'\ue7be': 'UnicodeCharacter0xe7be',
u'\u3d45': 'CjkUnifiedIdeograph3d45',
u'\u1cc4': 'UnicodeCharacter0x1cc4',
u'\ub749': 'HangulSyllableDdyinj',
u'\u96c8': 'CjkUnifiedIdeograph96c8',
u'\u6a4b': 'CjkUnifiedIdeograph6a4b',
u'\u55ca': 'CjkUnifiedIdeograph55ca',
u'\uec4f': 'UnicodeCharacter0xec4f',
u'\ucfce': 'HangulSyllableKyolm',
u'\u2555': 'BoxDrawingsDownSingleAndLeftDouble',
u'\u04d4': 'CyrillicCapitalLigatureAIe',
u'\u9f59': 'CjkUnifiedIdeograph9f59',
u'\ufed8': 'ArabicLetterQafMedialForm',
u'\u525b': 'CjkUnifiedIdeograph525b',
u'\ud45f': 'HangulSyllablePyogs',
u'\u37de': 'CjkUnifiedIdeograph37de',
u'\ua9e2': 'UnicodeCharacter0xa9e2',
u'\u0d65': 'UnicodeCharacter0xd65',
u'\u6ce4': 'CjkUnifiedIdeograph6ce4',
u'\u8769': 'CjkUnifiedIdeograph8769',
u'\ue6e8': 'UnicodeCharacter0xe6e8',
u'\u3c6f': 'CjkUnifiedIdeograph3c6f',
u'\u1fee': 'GreekDialytikaAndOxia',
u'\ub673': 'HangulSyllableDdoelh',
u'\u91f2': 'CjkUnifiedIdeograph91f2',
u'\u7575': 'CjkUnifiedIdeograph7575',
u'\u54f4': 'CjkUnifiedIdeograph54f4',
u'\uef79': 'UnicodeCharacter0xef79',
u'\ucef8': 'HangulSyllableKeoss',
u'\u247f': 'ParenthesizedNumberTwelve',
u'\u07fe': 'UnicodeCharacter0x7fe',
u'\u7206': 'CjkUnifiedIdeograph7206',
u'\u5689': 'CjkUnifiedIdeograph5689',
u'\u3708': 'CjkUnifiedIdeograph3708',
u'\u0b8b': 'UnicodeCharacter0xb8b',
u'\uf40a': 'UnicodeCharacter0xf40a',
u'\uc88d': 'HangulSyllableJwag',
u'\ua90c': 'KayahLiLetterGa',
u'\u8d8f': 'CjkUnifiedIdeograph8d8f',
u'\u5a16': 'CjkUnifiedIdeograph5a16',
u'\u1f18': 'GreekCapitalLetterEpsilonWithPsili',
u'\u739b': 'CjkUnifiedIdeograph739b',
u'\udc1a': 'UnicodeCharacter0xdc1a',
u'\u309d': 'HiraganaIterationMark',
u'\u911c': 'CjkUnifiedIdeograph911c',
u'\uf59f': 'UnicodeCharacter0xf59f',
u'\uaaa1': 'UnicodeCharacter0xaaa1',
u'\u4226': 'CjkUnifiedIdeograph4226',
u'\u0728': 'SyriacLetterSadhe',
u'\u5bab': 'CjkUnifiedIdeograph5bab',
u'\uc42a': 'HangulSyllableSswaep',
u'\u18ad': 'UnicodeCharacter0x18ad',
u'\uf92c': 'CjkCompatibilityIdeographf92c',
u'\uddaf': 'UnicodeCharacter0xddaf',
u'\ube2e': 'HangulSyllableByinh',
u'\u92b1': 'CjkUnifiedIdeograph92b1',
u'\u6f38': 'CjkUnifiedIdeograph6f38',
u'\u43bb': 'CjkUnifiedIdeograph43bb',
u'\u2c3a': 'GlagoliticSmallLetterInitialIzhe',
u'\xbd': 'VulgarFractionOneHalf',
u'\ue13c': 'UnicodeCharacter0xe13c',
u'\uc5bf': 'HangulSyllableEolb',
u'\ua63e': 'UnicodeCharacter0xa63e',
u'\ufac1': 'CjkCompatibilityIdeographfac1',
u'\ubfc3': 'HangulSyllableBbyos',
u'\u5748': 'CjkUnifiedIdeograph5748',
u'\u144a': 'CanadianSyllabicsWestcreeP',
u'\u68cd': 'CjkUnifiedIdeograph68cd',
u'\uc94c': 'HangulSyllableJwek',
u'\u2dcf': 'UnicodeCharacter0x2dcf',
u'\u8e4e': 'CjkUnifiedIdeograph8e4e',
u'\ue2d1': 'UnicodeCharacter0xe2d1',
u'\ua7d3': 'UnicodeCharacter0xa7d3',
u'\u7c5a': 'CjkUnifiedIdeograph7c5a',
u'\u50dd': 'CjkUnifiedIdeograph50dd',
u'\u315c': 'HangulLetterU',
u'\u15df': 'CanadianSyllabicsCarrierThee',
u'\uf65e': 'UnicodeCharacter0xf65e',
u'\ucae1': 'HangulSyllableJjwalg',
u'\uab60': 'UnicodeCharacter0xab60',
u'\u8fe3': 'CjkUnifiedIdeograph8fe3',
u'\u646a': 'CjkUnifiedIdeograph646a',
u'\u196c': 'TaiLeLetterAue',
u'\u7def': 'CjkUnifiedIdeograph7def',
u'\ude6e': 'UnicodeCharacter0xde6e',
u'\u32f1': 'CircledKatakanaMe',
u'\u9370': 'CjkUnifiedIdeograph9370',
u'\uf7f3': 'UnicodeCharacter0xf7f3',
u'\ub4f5': 'HangulSyllableDeut',
u'\u4c7a': 'CjkUnifiedIdeograph4c7a',
u'\u017c': 'LatinSmallLetterZWithDotAbove',
u'\u65ff': 'CjkUnifiedIdeograph65ff',
u'\uc67e': 'HangulSyllableOenh',
u'\ufb80': 'ArabicLetterTchehehInitialForm',
u'\u5f03': 'CjkUnifiedIdeograph5f03',
u'\ub882': 'HangulSyllableRwalm',
u'\u1c05': 'LepchaLetterNga',
u'\ud107': 'HangulSyllableTyalh',
u'\u9609': 'CjkUnifiedIdeograph9609',
u'\u698c': 'CjkUnifiedIdeograph698c',
u'\u2e8e': 'CjkRadicalLameOne',
u'\ue390': 'UnicodeCharacter0xe390',
u'\u4713': 'CjkUnifiedIdeograph4713',
u'\ua092': 'YiSyllableHmix',
u'\u0415': 'CyrillicCapitalLetterIe',
u'\u3917': 'CjkUnifiedIdeograph3917',
u'\ufe19': 'PresentationFormForVerticalHorizontalEllipsis',
u'\ub31b': 'HangulSyllableDaeh',
u'\u519c': 'CjkUnifiedIdeograph519c',
u'\u169e': 'UnicodeCharacter0x169e',
u'\ucba0': 'HangulSyllableJjwin',
u'\u88a2': 'CjkUnifiedIdeograph88a2',
u'\u6c25': 'CjkUnifiedIdeograph6c25',
u'\u2127': 'InvertedOhmSign',
u'\ue629': 'UnicodeCharacter0xe629',
u'\u9b2b': 'CjkUnifiedIdeograph9b2b',
u'\u7eae': 'CjkUnifiedIdeograph7eae',
u'\u33b0': 'SquarePs',
u'\uf0b2': 'UnicodeCharacter0xf0b2',
u'\u5435': 'CjkUnifiedIdeograph5435',
u'\ub5b4': 'HangulSyllableDdeoss',
u'\u0937': 'DevanagariLetterSsa',
u'\uce39': 'HangulSyllableCeut',
u'\u833b': 'CjkUnifiedIdeograph833b',
u'\u66be': 'CjkUnifiedIdeograph66be',
u'\u1bc0': 'UnicodeCharacter0x1bc0',
u'\ud8c2': 'UnicodeCharacter0xd8c2',
u'\u9dc4': 'CjkUnifiedIdeograph9dc4',
u'\u7147': 'CjkUnifiedIdeograph7147',
u'\u3649': 'CjkUnifiedIdeograph3649',
u'\ueb4b': 'UnicodeCharacter0xeb4b',
u'\ua84d': 'PhagspaLetterPha',
u'\u4ece': 'CjkUnifiedIdeograph4ece',
u'\u03d0': 'GreekBetaSymbol',
u'\uc0d2': 'HangulSyllableSaelm',
u'\u85d4': 'CjkUnifiedIdeograph85d4',
u'\u5957': 'CjkUnifiedIdeograph5957',
u'\ubad6': 'HangulSyllableMwabs',
u'\u1e59': 'LatinSmallLetterRWithDotAbove',
u'\ud35b': 'HangulSyllablePyac',
u'\u905d': 'CjkUnifiedIdeograph905d',
u'\u6be0': 'CjkUnifiedIdeograph6be0',
u'\u28e2': 'BraillePatternDots2678',
u'\uede4': 'UnicodeCharacter0xede4',
u'\u4167': 'CjkUnifiedIdeograph4167',
u'\ua2e6': 'YiSyllableZzur',
u'\u0669': 'ArabicindicDigitNine',
u'\u3b6b': 'CjkUnifiedIdeograph3b6b',
u'\uf86d': 'UnicodeCharacter0xf86d',
u'\ubd6f': 'HangulSyllableByolb',
u'\u53f0': 'CjkUnifiedIdeograph53f0',
u'\u10f2': 'GeorgianLetterHie',
u'\ud5f4': 'HangulSyllableHem',
u'\u8af6': 'CjkUnifiedIdeograph8af6',
u'\u6e79': 'CjkUnifiedIdeograph6e79',
u'\u237b': 'NotCheckMark',
u'\ue07d': 'UnicodeCharacter0xe07d',
u'\ua57f': 'VaiSyllableToo',
u'\udd81': 'UnicodeCharacter0xdd81',
u'\u3a04': 'CjkUnifiedIdeograph3a04',
u'\u1687': 'OghamLetterDair',
u'\ub008': 'HangulSyllableGgwek',
u'\u8c8b': 'CjkUnifiedIdeograph8c8b',
u'\u730a': 'CjkUnifiedIdeograph730a',
u'\u4f8d': 'CjkUnifiedIdeograph4f8d',
u'\ue90e': 'UnicodeCharacter0xe90e',
u'\uc591': 'HangulSyllableYang',
u'\u2214': 'DotPlus',
u'\u7e97': 'CjkUnifiedIdeograph7e97',
u'\u9818': 'CjkUnifiedIdeograph9818',
u'\uf49b': 'UnicodeCharacter0xf49b',
u'\u5b1a': 'CjkUnifiedIdeograph5b1a',
u'\ud11e': 'HangulSyllableTyaelm',
u'\u2da1': 'EthiopicSyllableSsu',
u'\uaba5': 'UnicodeCharacter0xaba5',
u'\u0a24': 'GurmukhiLetterTa',
u'\u66a7': 'CjkUnifiedIdeograph66a7',
u'\u8028': 'CjkUnifiedIdeograph8028',
u'\udcab': 'UnicodeCharacter0xdcab',
u'\u432a': 'CjkUnifiedIdeograph432a',
u'\u392e': 'CjkUnifiedIdeograph392e',
u'\u15b1': 'CanadianSyllabicsBlackfootI',
u'\ub732': 'HangulSyllableDdeulm',
u'\u93b5': 'CjkUnifiedIdeograph93b5',
u'\u7234': 'CjkUnifiedIdeograph7234',
u'\u4eb7': 'CjkUnifiedIdeograph4eb7',
u'\ue838': 'UnicodeCharacter0xe838',
u'\uc4bb': 'HangulSyllableSswigs',
u'\u213e': 'DoublestruckCapitalGamma',
u'\u7dc1': 'CjkUnifiedIdeograph7dc1',
u'\u9f42': 'CjkUnifiedIdeograph9f42',
u'\ufbc5': 'UnicodeCharacter0xfbc5',
u'\u5a44': 'CjkUnifiedIdeograph5a44',
u'\ud048': 'HangulSyllableKwiss',
u'\u2ccb': 'CopticSmallLetterDialectpHori',
u'\uaacf': 'UnicodeCharacter0xaacf',
u'\u094e': 'UnicodeCharacter0x94e',
u'\u65d1': 'CjkUnifiedIdeograph65d1',
u'\u8752': 'CjkUnifiedIdeograph8752',
u'\ue3d5': 'UnicodeCharacter0xe3d5',
u'\u4254': 'CjkUnifiedIdeograph4254',
u'\u3858': 'CjkUnifiedIdeograph3858',
u'\u14db': 'CanadianSyllabicsLaa',
u'\ub65c': 'HangulSyllableDdwaess',
u'\u92df': 'CjkUnifiedIdeograph92df',
u'\u715e': 'CjkUnifiedIdeograph715e',
u'\u4de1': 'HexagramForGreatPower',
u'\uef62': 'UnicodeCharacter0xef62',
u'\ucbe5': 'HangulSyllableJjeub',
u'\u2068': 'UnicodeCharacter0x2068',
u'\u7ceb': 'CjkUnifiedIdeograph7ceb',
u'\u9e6c': 'CjkUnifiedIdeograph9e6c',
u'\ufaef': 'UnicodeCharacter0xfaef',
u'\u596e': 'CjkUnifiedIdeograph596e',
u'\ud772': 'HangulSyllableHyinh',
u'\u33f5': 'IdeographicTelegraphSymbolForDayTwentytwo',
u'\ua9f9': 'UnicodeCharacter0xa9f9',
u'\u0878': 'UnicodeCharacter0x878',
u'\u64fb': 'CjkUnifiedIdeograph64fb',
u'\u867c': 'CjkUnifiedIdeograph867c',
u'\ue2ff': 'UnicodeCharacter0xe2ff',
u'\u417e': 'CjkUnifiedIdeograph417e',
u'\ud803': 'UnicodeCharacter0xd803',
u'\u3f82': 'CjkUnifiedIdeograph3f82',
u'\ub586': 'HangulSyllableDdyaegg',
u'\u1109': 'HangulChoseongSios',
u'\u7088': 'CjkUnifiedIdeograph7088',
u'\u8f0d': 'CjkUnifiedIdeograph8f0d',
u'\uee8c': 'UnicodeCharacter0xee8c',
u'\u4a0f': 'CjkUnifiedIdeograph4a0f',
u'\uc013': 'HangulSyllableBbwelh',
u'\u2792': 'DingbatNegativeCircledSansserifDigitNine',
u'\ube17': 'HangulSyllableBeulb',
u'\u9d96': 'CjkUnifiedIdeograph9d96',
u'\u7919': 'CjkUnifiedIdeograph7919',
u'\u5898': 'CjkUnifiedIdeograph5898',
u'\uf71d': 'UnicodeCharacter0xf71d',
u'\ud69c': 'HangulSyllableHoem',
u'\u2823': 'BraillePatternDots126',
u'\u0fa2': 'TibetanSubjoinedLetterDha',
u'\ua627': 'VaiDigitSeven',
u'\u85a6': 'CjkUnifiedIdeograph85a6',
u'\u6129': 'CjkUnifiedIdeograph6129',
u'\u40a8': 'CjkUnifiedIdeograph40a8',
u'\udf2d': 'UnicodeCharacter0xdf2d',
u'\u3eac': 'CjkUnifiedIdeograph3eac',
u'\ub4b0': 'HangulSyllableDwils',
u'\u1033': 'MyanmarVowelSignMonIi',
u'\u77b2': 'CjkUnifiedIdeograph77b2',
u'\u8e37': 'CjkUnifiedIdeograph8e37',
u'\uedb6': 'UnicodeCharacter0xedb6',
u'\u4939': 'CjkUnifiedIdeograph4939',
u'\uc73d': 'HangulSyllableEug',
u'\u26bc': 'Sesquiquadrate',
u'\ubd41': 'HangulSyllableBwaeng',
u'\u9cc0': 'CjkUnifiedIdeograph9cc0',
u'\u7843': 'CjkUnifiedIdeograph7843',
u'\u5fc2': 'CjkUnifiedIdeograph5fc2',
u'\uf647': 'UnicodeCharacter0xf647',
u'\ud5c6': 'HangulSyllableHyaep',
u'\u2f4d': 'KangxiRadicalDeath',
u'\u0ecc': 'LaoCancellationMark',
u'\ua551': 'VaiSyllableBha',
u'\u84d0': 'CjkUnifiedIdeograph84d0',
u'\u6053': 'CjkUnifiedIdeograph6053',
u'\u47d2': 'CjkUnifiedIdeograph47d2',
u'\ude57': 'UnicodeCharacter0xde57',
u'\u3dd6': 'CjkUnifiedIdeograph3dd6',
u'\ubbda': 'HangulSyllableMeup',
u'\u175d': 'UnicodeCharacter0x175d',
u'\u76dc': 'CjkUnifiedIdeograph76dc',
u'\u8d61': 'CjkUnifiedIdeograph8d61',
u'\uece0': 'UnicodeCharacter0xece0',
u'\u4863': 'CjkUnifiedIdeograph4863',
u'\uc667': 'HangulSyllableWaelb',
u'\u25e6': 'WhiteBullet',
u'\ubc6b': 'HangulSyllableByaegs',
u'\ua3ea': 'YiSyllableJuo',
u'\u7f6d': 'CjkUnifiedIdeograph7f6d',
u'\u5eec': 'CjkUnifiedIdeograph5eec',
u'\uf571': 'UnicodeCharacter0xf571',
u'\ud4f0': 'HangulSyllablePyul',
u'\u2e77': 'UnicodeCharacter0x2e77',
u'\u0df6': 'UnicodeCharacter0xdf6',
u'\ua47b': 'YiSyllableYuo',
u'\u8bfa': 'CjkUnifiedIdeograph8bfa',
u'\u677d': 'CjkUnifiedIdeograph677d',
u'\u46fc': 'CjkUnifiedIdeograph46fc',
u'\u5c81': 'CjkUnifiedIdeograph5c81',
u'\u3d00': 'CjkUnifiedIdeograph3d00',
u'\u1983': 'NewTaiLueLetterHighXa',
u'\ufe02': 'VariationSelector3',
u'\uda85': 'UnicodeCharacter0xda85',
u'\ubb04': 'HangulSyllableMoel',
u'\u9787': 'CjkUnifiedIdeograph9787',
u'\u680e': 'CjkUnifiedIdeograph680e',
u'\u4491': 'CjkUnifiedIdeograph4491',
u'\u2510': 'BoxDrawingsLightDownAndLeft',
u'\u0193': 'LatinCapitalLetterGWithHook',
u'\ue612': 'UnicodeCharacter0xe612',
u'\uc295': 'HangulSyllableSyult',
u'\ua314': 'YiSyllableSuox',
u'\uff97': 'HalfwidthKatakanaLetterRa',
u'\ub899': 'HangulSyllableRwaenj',
u'\u501e': 'CjkUnifiedIdeograph501e',
u'\u0d20': 'MalayalamLetterTtha',
u'\u69a3': 'CjkUnifiedIdeograph69a3',
u'\uce22': 'HangulSyllableCeugg',
u'\u2aa5': 'GreaterthanBesideLessthan',
u'\u8b24': 'CjkUnifiedIdeograph8b24',
u'\ue7a7': 'UnicodeCharacter0xe7a7',
u'\ua0a9': 'YiSyllableHmyx',
u'\u7530': 'CjkUnifiedIdeograph7530',
u'\u51b3': 'CjkUnifiedIdeograph51b3',
u'\u3632': 'CjkUnifiedIdeograph3632',
u'\u12b5': 'EthiopicSyllableKwe',
u'\uf334': 'UnicodeCharacter0xf334',
u'\ucfb7': 'HangulSyllableKoelh',
u'\uac36': 'HangulSyllableGaep',
u'\u88b9': 'CjkUnifiedIdeograph88b9',
u'\u5d40': 'CjkUnifiedIdeograph5d40',
u'\u1e42': 'LatinCapitalLetterMWithDotBelow',
u'\u7ac5': 'CjkUnifiedIdeograph7ac5',
u'\udb44': 'UnicodeCharacter0xdb44',
u'\u37c7': 'CjkUnifiedIdeograph37c7',
u'\u9446': 'CjkUnifiedIdeograph9446',
u'\uf0c9': 'UnicodeCharacter0xf0c9',
u'\uadcb': 'HangulSyllableGwilb',
u'\u4550': 'CjkUnifiedIdeograph4550',
u'\u0652': 'ArabicSukun',
u'\u62d5': 'CjkUnifiedIdeograph62d5',
u'\uc354': 'HangulSyllableSsyael',
u'\u1fd7': 'GreekSmallLetterIotaWithDialytikaAndPerispomeni',
u'\ufc56': 'ArabicLigatureYehWithHahIsolatedForm',
u'\ud8d9': 'UnicodeCharacter0xd8d9',
u'\ub958': 'HangulSyllableRyu',
u'\u95db': 'CjkUnifiedIdeograph95db',
u'\u6e62': 'CjkUnifiedIdeograph6e62',
u'\u4ae5': 'CjkUnifiedIdeograph4ae5',
u'\u2b64': 'UnicodeCharacter0x2b64',
u'\u07e7': 'NkoLetterNyaWoloso',
u'\ue466': 'UnicodeCharacter0xe466',
u'\uc0e9': 'HangulSyllableSyanj',
u'\ua168': 'YiSyllableHnit',
u'\ufdeb': 'UnicodeCharacter0xfdeb',
u'\ubeed': 'HangulSyllableBbeg',
u'\u5672': 'CjkUnifiedIdeograph5672',
u'\u1374': 'EthiopicNumberThirty',
u'\u6ff7': 'CjkUnifiedIdeograph6ff7',
u'\ucc76': 'HangulSyllableCyaj',
u'\u28f9': 'BraillePatternDots145678',
u'\u8978': 'CjkUnifiedIdeograph8978',
u'\ue5fb': 'UnicodeCharacter0xe5fb',
u'\ua6fd': 'UnicodeCharacter0xa6fd',
u'\u9c01': 'CjkUnifiedIdeograph9c01',
u'\u7b84': 'CjkUnifiedIdeograph7b84',
u'\u3486': 'CjkUnifiedIdeograph3486',
u'\uf188': 'UnicodeCharacter0xf188',
u'\u4d0b': 'CjkUnifiedIdeograph4d0b',
u'\ub28a': 'HangulSyllableNyuj',
u'\u0e0d': 'ThaiCharacterYoYing',
u'\ucb0f': 'HangulSyllableJjwaeh',
u'\u8411': 'CjkUnifiedIdeograph8411',
u'\u6394': 'CjkUnifiedIdeograph6394',
u'\u1c96': 'UnicodeCharacter0x1c96',
u'\ud998': 'UnicodeCharacter0xd998',
u'\u9a9a': 'CjkUnifiedIdeograph9a9a',
u'\u761d': 'CjkUnifiedIdeograph761d',
u'\u331f': 'SquareSaikuru',
u'\uec21': 'UnicodeCharacter0xec21',
u'\ua923': 'KayahLiLetterOe',
u'\u4ba4': 'CjkUnifiedIdeograph4ba4',
u'\u04a6': 'CyrillicCapitalLetterPeWithMiddleHook',
u'\uc1a8': 'HangulSyllableSwa',
u'\u82aa': 'CjkUnifiedIdeograph82aa',
u'\u5e2d': 'CjkUnifiedIdeograph5e2d',
u'\ubfac': 'HangulSyllableBboek',
u'\u1b2f': 'BalineseLetterWa',
u'\ud431': 'HangulSyllablePwaelt',
u'\u9133': 'CjkUnifiedIdeograph9133',
u'\u6cb6': 'CjkUnifiedIdeograph6cb6',
u'\u29b8': 'CircledReverseSolidus',
u'\ueaba': 'UnicodeCharacter0xeaba',
u'\u463d': 'CjkUnifiedIdeograph463d',
u'\ua7bc': 'UnicodeCharacter0xa7bc',
u'\u033f': 'CombiningDoubleOverline',
u'\u3c41': 'CjkUnifiedIdeograph3c41',
u'\uf943': 'CjkCompatibilityIdeographf943',
u'\uba45': 'HangulSyllableMeolt',
u'\u54c6': 'CjkUnifiedIdeograph54c6',
u'\u11c8': 'HangulJongseongNieunpansios',
u'\ud2ca': 'HangulSyllableTeubs',
u'\u8fcc': 'CjkUnifiedIdeograph8fcc',
u'\u6b4f': 'CjkUnifiedIdeograph6b4f',
u'\u2451': 'UnicodeCharacter0x2451',
u'\ue153': 'UnicodeCharacter0xe153',
u'\ua255': 'YiSyllableHxuop',
u'\u79d8': 'CjkUnifiedIdeograph79d8',
u'\u3ada': 'CjkUnifiedIdeograph3ada',
u'\uf7dc': 'UnicodeCharacter0xf7dc',
u'\u535f': 'CjkUnifiedIdeograph535f',
u'\ub0de': 'HangulSyllableNyalp',
u'\u0c61': 'TeluguLetterVocalicLl',
u'\uc963': 'HangulSyllableJwis',
u'\u8a65': 'CjkUnifiedIdeograph8a65',
u'\u61e8': 'CjkUnifiedIdeograph61e8',
u'\u22ea': 'NotNormalSubgroupOf',
u'\udfec': 'UnicodeCharacter0xdfec',
u'\u98ee': 'CjkUnifiedIdeograph98ee',
u'\u7471': 'CjkUnifiedIdeograph7471',
u'\u3173': 'HangulLetterPieuptikeut',
u'\uf275': 'UnicodeCharacter0xf275',
u'\uaf77': 'HangulSyllableGgwaes',
u'\u49f8': 'CjkUnifiedIdeograph49f8',
u'\u0afa': 'UnicodeCharacter0xafa',
u'\uc7fc': 'HangulSyllableJyaek',
u'\u80fe': 'CjkUnifiedIdeograph80fe',
u'\ube00': 'HangulSyllableByum',
u'\u9a83': 'CjkUnifiedIdeograph9a83',
u'\u7d02': 'CjkUnifiedIdeograph7d02',
u'\u5985': 'CjkUnifiedIdeograph5985',
u'\uf706': 'UnicodeCharacter0xf706',
u'\ud389': 'HangulSyllablePeolt',
u'\u2c0c': 'GlagoliticCapitalLetterDjervi',
u'\u088f': 'UnicodeCharacter0x88f',
u'\ua610': 'VaiSyllableNdoleFa',
u'\u8293': 'CjkUnifiedIdeograph8293',
u'\u6512': 'CjkUnifiedIdeograph6512',
u'\u4195': 'CjkUnifiedIdeograph4195',
u'\udf16': 'UnicodeCharacter0xdf16',
u'\u3b99': 'CjkUnifiedIdeograph3b99',
u'\ub59d': 'HangulSyllableDdyaet',
u'\u141c': 'CanadianSyllabicsAi',
u'\u709f': 'CjkUnifiedIdeograph709f',
u'\u8e20': 'CjkUnifiedIdeograph8e20',
u'\ueaa3': 'UnicodeCharacter0xeaa3',
u'\u4d22': 'CjkUnifiedIdeograph4d22',
u'\uc726': 'HangulSyllableYunh',
u'\u23a9': 'LeftCurlyBracketLowerHook',
u'\u9dad': 'CjkUnifiedIdeograph9dad',
u'\u7c2c': 'CjkUnifiedIdeograph7c2c',
u'\u58af': 'CjkUnifiedIdeograph58af',
u'\uf630': 'UnicodeCharacter0xf630',
u'\ud2b3': 'HangulSyllableTyuc',
u'\u2f36': 'KangxiRadicalTwoHands',
u'\u0bb9': 'TamilLetterHa',
u'\ua93a': 'RejangLetterJa',
u'\u85bd': 'CjkUnifiedIdeograph85bd',
u'\u643c': 'CjkUnifiedIdeograph643c',
u'\u40bf': 'CjkUnifiedIdeograph40bf',
u'\ude40': 'UnicodeCharacter0xde40',
u'\u3ac3': 'CjkUnifiedIdeograph3ac3',
u'\ub4c7': 'HangulSyllableDyud',
u'\u1746': 'BuhidLetterTa',
u'\u73c9': 'CjkUnifiedIdeograph73c9',
u'\u914a': 'CjkUnifiedIdeograph914a',
u'\uedcd': 'UnicodeCharacter0xedcd',
u'\u4c4c': 'CjkUnifiedIdeograph4c4c',
u'\uc650': 'HangulSyllableWam',
u'\u22d3': 'DoubleUnion',
u'\u9cd7': 'CjkUnifiedIdeograph9cd7',
u'\u7f56': 'CjkUnifiedIdeograph7f56',
u'\u5bd9': 'CjkUnifiedIdeograph5bd9',
u'\uf95a': 'CjkCompatibilityIdeographf95a',
u'\ud5dd': 'HangulSyllableHeong',
u'\u2e60': 'UnicodeCharacter0x2e60',
u'\u0ae3': 'GujaratiVowelSignVocalicLl',
u'\ua864': 'PhagspaLetterFa',
u'\u84e7': 'CjkUnifiedIdeograph84e7',
u'\u6766': 'CjkUnifiedIdeograph6766',
u'\u43e9': 'CjkUnifiedIdeograph43e9',
u'\ue16a': 'UnicodeCharacter0xe16a',
u'\u3ded': 'CjkUnifiedIdeograph3ded',
u'\ub7f1': 'HangulSyllableReonj',
u'\u1670': 'CanadianSyllabicsNgai',
u'\u72f3': 'CjkUnifiedIdeograph72f3',
u'\u9074': 'CjkUnifiedIdeograph9074',
u'\uecf7': 'UnicodeCharacter0xecf7',
u'\u4f76': 'CjkUnifiedIdeograph4f76',
u'\uc97a': 'HangulSyllableJyulp',
u'\u25fd': 'WhiteMediumSmallSquare',
u'\u1f01': 'GreekSmallLetterAlphaWithDasia',
u'\u7e80': 'CjkUnifiedIdeograph7e80',
u'\u9905': 'CjkUnifiedIdeograph9905',
u'\uf884': 'UnicodeCharacter0xf884',
u'\u5407': 'CjkUnifiedIdeograph5407',
u'\uce0b': 'HangulSyllableCyud',
u'\u318a': 'HangulLetterYuyeo',
u'\uab8e': 'UnicodeCharacter0xab8e',
u'\u0711': 'SyriacLetterSuperscriptAlaph',
u'\u6690': 'CjkUnifiedIdeograph6690',
u'\u8115': 'CjkUnifiedIdeograph8115',
u'\ue094': 'UnicodeCharacter0xe094',
u'\u361b': 'CjkUnifiedIdeograph361b',
u'\u199a': 'NewTaiLueLetterHighFa',
u'\ub01f': 'HangulSyllableGgwis',
u'\u939e': 'CjkUnifiedIdeograph939e',
u'\u6f21': 'CjkUnifiedIdeograph6f21',
u'\u4ea0': 'CjkUnifiedIdeograph4ea0',
u'\ue925': 'UnicodeCharacter0xe925',
u'\uc8a4': 'HangulSyllableJwak',
u'\u1e2b': 'LatinSmallLetterHWithBreveBelow',
u'\u01aa': 'LatinLetterReversedEshLoop',
u'\u982f': 'CjkUnifiedIdeograph982f',
u'\ufbae': 'ArabicLetterYehBarreeIsolatedForm',
u'\u5731': 'CjkUnifiedIdeograph5731',
u'\ud135': 'HangulSyllableTeonj',
u'\u30b4': 'KatakanaLetterGo',
u'\uaab8': 'UnicodeCharacter0xaab8',
u'\u063b': 'ArabicLetterKehehWithTwoDotsAbove',
u'\u69ba': 'CjkUnifiedIdeograph69ba',
u'\u803f': 'CjkUnifiedIdeograph803f',
u'\ue3be': 'UnicodeCharacter0xe3be',
u'\u3945': 'CjkUnifiedIdeograph3945',
u'\u18c4': 'UnicodeCharacter0x18c4',
u'\ub349': 'HangulSyllableDyaeb',
u'\u92c8': 'CjkUnifiedIdeograph92c8',
u'\u6e4b': 'CjkUnifiedIdeograph6e4b',
u'\u51ca': 'CjkUnifiedIdeograph51ca',
u'\ue84f': 'UnicodeCharacter0xe84f',
u'\ucbce': 'HangulSyllableJjyuj',
u'\u2155': 'VulgarFractionOneFifth',
u'\xd4': 'LatinCapitalLetterOWithCircumflex',
u'\u9b59': 'CjkUnifiedIdeograph9b59',
u'\ufad8': 'CjkCompatibilityIdeographfad8',
u'\u565b': 'CjkUnifiedIdeograph565b',
u'\ud05f': 'HangulSyllableKyulh',
u'\u33de': 'SquareVOverM',
u'\uade2': 'HangulSyllableGyunh',
u'\u0965': 'DevanagariDoubleDanda',
u'\u68e4': 'CjkUnifiedIdeograph68e4',
u'\u8369': 'CjkUnifiedIdeograph8369',
u'\ue2e8': 'UnicodeCharacter0xe2e8',
u'\u386f': 'CjkUnifiedIdeograph386f',
u'\u1bee': 'UnicodeCharacter0x1bee',
u'\ub273': 'HangulSyllableNwih',
u'\u95f2': 'CjkUnifiedIdeograph95f2',
u'\u7175': 'CjkUnifiedIdeograph7175',
u'\u50f4': 'CjkUnifiedIdeograph50f4',
u'\ueb79': 'UnicodeCharacter0xeb79',
u'\ucaf8': 'HangulSyllableJjwaen',
u'\u207f': 'SuperscriptLatinSmallLetterN',
u'\u03fe': 'GreekCapitalDottedLunateSigmaSymbol',
u'\u7606': 'CjkUnifiedIdeograph7606',
u'\u5289': 'CjkUnifiedIdeograph5289',
u'\u3308': 'SquareEekaa',
u'\u0f8b': 'TibetanSignGruMedRgyings',
u'\uf00a': 'UnicodeCharacter0xf00a',
u'\ucc8d': 'HangulSyllableCyaeb',
u'\uad0c': 'HangulSyllableGwam',
u'\u898f': 'CjkUnifiedIdeograph898f',
u'\u5e16': 'CjkUnifiedIdeograph5e16',
u'\u1b18': 'BalineseLetterCa',
u'\u779b': 'CjkUnifiedIdeograph779b',
u'\ud81a': 'UnicodeCharacter0xd81a',
u'\u349d': 'CjkUnifiedIdeograph349d',
u'\u951c': 'CjkUnifiedIdeograph951c',
u'\uf19f': 'UnicodeCharacter0xf19f',
u'\uaea1': 'HangulSyllableGgyaeg',
u'\u4626': 'CjkUnifiedIdeograph4626',
u'\u0328': 'CombiningOgonek',
u'\u5fab': 'CjkUnifiedIdeograph5fab',
u'\uc02a': 'HangulSyllableBbwilm',
u'\u1cad': 'UnicodeCharacter0x1cad',
u'\ufd2c': 'ArabicLigatureDadWithRehFinalForm',
u'\ud9af': 'UnicodeCharacter0xd9af',
u'\uba2e': 'HangulSyllableMyaebs',
u'\u96b1': 'CjkUnifiedIdeograph96b1',
u'\u6b38': 'CjkUnifiedIdeograph6b38',
u'\u47bb': 'CjkUnifiedIdeograph47bb',
u'\u283a': 'BraillePatternDots2456',
u'\u04bd': 'CyrillicSmallLetterAbkhasianChe',
u'\ue53c': 'UnicodeCharacter0xe53c',
u'\uc1bf': 'HangulSyllableSwac',
u'\ua23e': 'YiSyllableMge',
u'\ufec1': 'ArabicLetterTahIsolatedForm',
u'\ubbc3': 'HangulSyllableMeugs',
u'\u5348': 'CjkUnifiedIdeograph5348',
u'\u104a': 'MyanmarSignLittleSection',
u'\u6ccd': 'CjkUnifiedIdeograph6ccd',
u'\ucd4c': 'HangulSyllableCwaels',
u'\u29cf': 'LeftTriangleBesideVerticalBar',
u'\u8a4e': 'CjkUnifiedIdeograph8a4e',
u'\ue6d1': 'UnicodeCharacter0xe6d1',
u'\ua3d3': 'YiSyllableRep',
u'\u785a': 'CjkUnifiedIdeograph785a',
u'\u54dd': 'CjkUnifiedIdeograph54dd',
u'\u355c': 'CjkUnifiedIdeograph355c',
u'\u11df': 'HangulJongseongMieumpansios',
u'\uf25e': 'UnicodeCharacter0xf25e',
u'\ucee1': 'HangulSyllableKyaet',
u'\uaf60': 'HangulSyllableGgwak',
u'\u8be3': 'CjkUnifiedIdeograph8be3',
u'\u606a': 'CjkUnifiedIdeograph606a',
u'\u1d6c': 'LatinSmallLetterBWithMiddleTilde',
u'\u79ef': 'CjkUnifiedIdeograph79ef',
u'\uda6e': 'UnicodeCharacter0xda6e',
u'\u36f1': 'CjkUnifiedIdeograph36f1',
u'\u9770': 'CjkUnifiedIdeograph9770',
u'\uf3f3': 'UnicodeCharacter0xf3f3',
u'\ub0f5': 'HangulSyllableNyaelg',
u'\u487a': 'CjkUnifiedIdeograph487a',
u'\u057c': 'ArmenianSmallLetterRa',
u'\u61ff': 'CjkUnifiedIdeograph61ff',
u'\uc27e': 'HangulSyllableSwibs',
u'\uff80': 'HalfwidthKatakanaLetterTa',
u'\u5b03': 'CjkUnifiedIdeograph5b03',
u'\ubc82': 'HangulSyllableByaep',
u'\u1805': 'MongolianFourDots',
u'\ud507': 'HangulSyllablePeugs',
u'\u9209': 'CjkUnifiedIdeograph9209',
u'\u6d8c': 'CjkUnifiedIdeograph6d8c',
u'\u2a8e': 'GreaterthanAboveSimilarOrEqual',
u'\ue790': 'UnicodeCharacter0xe790',
u'\u4313': 'CjkUnifiedIdeograph4313',
u'\ua492': 'YiRadicalKit',
u'\u3d17': 'CjkUnifiedIdeograph3d17',
u'\ufa19': 'CjkCompatibilityIdeographfa19',
u'\ub71b': 'HangulSyllableDdyulh',
u'\u559c': 'CjkUnifiedIdeograph559c',
u'\u129e': 'EthiopicSyllableNyo',
u'\ucfa0': 'HangulSyllableKwaess',
u'\u8ca2': 'CjkUnifiedIdeograph8ca2',
u'\u6825': 'CjkUnifiedIdeograph6825',
u'\u2527': 'BoxDrawingsDownHeavyAndLeftUpLight',
u'\ue229': 'UnicodeCharacter0xe229',
u'\u9f2b': 'CjkUnifiedIdeograph9f2b',
u'\u7aae': 'CjkUnifiedIdeograph7aae',
u'\u37b0': 'CjkUnifiedIdeograph37b0',
u'\uf4b2': 'UnicodeCharacter0xf4b2',
u'\u5035': 'CjkUnifiedIdeograph5035',
u'\ub1b4': 'HangulSyllableNwaen',
u'\u0d37': 'MalayalamLetterSsa',
u'\uca39': 'HangulSyllableJjyaelg',
u'\u873b': 'CjkUnifiedIdeograph873b',
u'\u62be': 'CjkUnifiedIdeograph62be',
u'\u1fc0': 'GreekPerispomeni',
u'\udcc2': 'UnicodeCharacter0xdcc2',
u'\u99c4': 'CjkUnifiedIdeograph99c4',
u'\u7547': 'CjkUnifiedIdeograph7547',
u'\u3249': 'UnicodeCharacter0x3249',
u'\uef4b': 'UnicodeCharacter0xef4b',
u'\uac4d': 'HangulSyllableGyang',
u'\u4ace': 'CjkUnifiedIdeograph4ace',
u'\u07d0': 'NkoLetterO',
u'\uc4d2': 'HangulSyllableSswip',
u'\u81d4': 'CjkUnifiedIdeograph81d4',
u'\u5d57': 'CjkUnifiedIdeograph5d57',
u'\ubed6': 'HangulSyllableBbeonh',
u'\u1a59': 'UnicodeCharacter0x1a59',
u'\ud75b': 'HangulSyllableHeulb',
u'\u945d': 'CjkUnifiedIdeograph945d',
u'\u6fe0': 'CjkUnifiedIdeograph6fe0',
u'\u2ce2': 'CopticCapitalLetterOldNubianWau',
u'\ue9e4': 'UnicodeCharacter0xe9e4',
u'\u4567': 'CjkUnifiedIdeograph4567',
u'\ua6e6': 'UnicodeCharacter0xa6e6',
u'\u0269': 'LatinSmallLetterIota',
u'\u3f6b': 'CjkUnifiedIdeograph3f6b',
u'\ufc6d': 'ArabicLigatureBehWithNoonFinalForm',
u'\ub96f': 'HangulSyllableRyuc',
u'\u57f0': 'CjkUnifiedIdeograph57f0',
u'\u14f2': 'CanadianSyllabicsSoo',
u'\ud1f4': 'HangulSyllableToe',
u'\u8ef6': 'CjkUnifiedIdeograph8ef6',
u'\u6a79': 'CjkUnifiedIdeograph6a79',
u'\u277b': 'DingbatNegativeCircledDigitSix',
u'\ue47d': 'UnicodeCharacter0xe47d',
u'\ua17f': 'YiSyllableNi',
u'\ua181': 'YiSyllableNiex',
u'\u4604': 'CjkUnifiedIdeograph4604',
u'\u6a87': 'CjkUnifiedIdeograph6a87',
u'\ucc08': 'HangulSyllableJjyik',
u'\uf08b': 'UnicodeCharacter0xf08b',
u'\u0f0a': 'TibetanMarkBkaShogYigMgo',
u'\u338d': 'SquareMuG',
u'\u950e': 'CjkUnifiedIdeograph950e',
u'\ub991': 'HangulSyllableRyig',
u'\u5e14': 'CjkUnifiedIdeograph5e14',
u'\u0297': 'LatinLetterStretchedC',
u'\ue418': 'UnicodeCharacter0xe418',
u'\u889b': 'CjkUnifiedIdeograph889b',
u'\u271a': 'HeavyGreekCross',
u'\uad1e': 'HangulSyllableGwaenh',
u'\u51a1': 'CjkUnifiedIdeograph51a1',
u'\ud7a5': 'UnicodeCharacter0xd7a5',
u'\u7624': 'CjkUnifiedIdeograph7624',
u'\u1aa7': 'UnicodeCharacter0x1aa7',
u'\ufc28': 'ArabicLigatureZahWithMeemIsolatedForm',
u'\ua0ab': 'YiSyllableHmyp',
u'\u3f2a': 'CjkUnifiedIdeograph3f2a',
u'\u452e': 'CjkUnifiedIdeograph452e',
u'\u69b1': 'CjkUnifiedIdeograph69b1',
u'\ucb32': 'HangulSyllableJjyonh',
u'\uefb5': 'UnicodeCharacter0xefb5',
u'\u0e34': 'ThaiCharacterSaraI',
u'\u32b7': 'CircledNumberFortyTwo',
u'\u9438': 'CjkUnifiedIdeograph9438',
u'\ub8bb': 'HangulSyllableRoelb',
u'\u5d3e': 'CjkUnifiedIdeograph5d3e',
u'\u01c1': 'LatinLetterLateralClick',
u'\ue342': 'UnicodeCharacter0xe342',
u'\u87c5': 'CjkUnifiedIdeograph87c5',
u'\u2644': 'Saturn',
u'\uac48': 'HangulSyllableGyam',
u'\u50cb': 'CjkUnifiedIdeograph50cb',
u'\ud6cf': 'HangulSyllableHulb',
u'\u754e': 'CjkUnifiedIdeograph754e',
u'\u19d1': 'NewTaiLueDigitOne',
u'\ufb52': 'ArabicLetterBeehIsolatedForm',
u'\u9fd5': 'UnicodeCharacter0x9fd5',
u'\u3e54': 'CjkUnifiedIdeograph3e54',
u'\u4458': 'CjkUnifiedIdeograph4458',
u'\u68db': 'CjkUnifiedIdeograph68db',
u'\uca5c': 'HangulSyllableJjeom',
u'\ueedf': 'UnicodeCharacter0xeedf',
u'\u0d5e': 'UnicodeCharacter0xd5e',
u'\u31e1': 'CjkStrokeHzzzg',
u'\u9362': 'CjkUnifiedIdeograph9362',
u'\ub7e5': 'HangulSyllableRyaeng',
u'\u5c68': 'CjkUnifiedIdeograph5c68',
u'\xeb': 'LatinSmallLetterEWithDiaeresis',
u'\ue26c': 'UnicodeCharacter0xe26c',
u'\u86ef': 'CjkUnifiedIdeograph86ef',
u'\u256e': 'BoxDrawingsLightArcDownAndLeft',
u'\uab72': 'UnicodeCharacter0xab72',
u'\u4ff5': 'CjkUnifiedIdeograph4ff5',
u'\ud5f9': 'HangulSyllableHeng',
u'\u7478': 'CjkUnifiedIdeograph7478',
u'\u18fb': 'UnicodeCharacter0x18fb',
u'\ufa7c': 'CjkCompatibilityIdeographfa7c',
u'\u9eff': 'CjkUnifiedIdeograph9eff',
u'\u3d7e': 'CjkUnifiedIdeograph3d7e',
u'\ua403': 'YiSyllableQiep',
u'\u4382': 'CjkUnifiedIdeograph4382',
u'\uc986': 'HangulSyllableJyup',
u'\u6d09': 'CjkUnifiedIdeograph6d09',
u'\u0c88': 'KannadaLetterIi',
u'\uf30d': 'UnicodeCharacter0xf30d',
u'\u928c': 'CjkUnifiedIdeograph928c',
u'\u360f': 'CjkUnifiedIdeograph360f',
u'\ubc13': 'HangulSyllableMih',
u'\u5b92': 'CjkUnifiedIdeograph5b92',
u'\uc217': 'HangulSyllableSyoh',
u'\ue196': 'UnicodeCharacter0xe196',
u'\u0519': 'CyrillicSmallLetterYae',
u'\u2498': 'NumberSeventeenFullStop',
u'\u8b1d': 'CjkUnifiedIdeograph8b1d',
u'\uaa9c': 'UnicodeCharacter0xaa9c',
u'\u5423': 'CjkUnifiedIdeograph5423',
u'\u73a2': 'CjkUnifiedIdeograph73a2',
u'\uda27': 'UnicodeCharacter0xda27',
u'\uf9a6': 'CjkCompatibilityIdeographf9a6',
u'\u1d29': 'GreekLetterSmallCapitalRho',
u'\u3ca8': 'CjkUnifiedIdeograph3ca8',
u'\ua32d': 'YiSyllableSsip',
u'\u42ac': 'CjkUnifiedIdeograph42ac',
u'\uc8b0': 'HangulSyllableJwael',
u'\u6c33': 'CjkUnifiedIdeograph6c33',
u'\u0bb2': 'TamilLetterLa',
u'\uf237': 'UnicodeCharacter0xf237',
u'\u91b6': 'CjkUnifiedIdeograph91b6',
u'\u3539': 'CjkUnifiedIdeograph3539',
u'\ubb3d': 'HangulSyllableMulg',
u'\u5abc': 'CjkUnifiedIdeograph5abc',
u'\uc141': 'HangulSyllableSelg',
u'\ue0c0': 'UnicodeCharacter0xe0c0',
u'\u0443': 'CyrillicSmallLetterU',
u'\u23c2': 'DentistrySymbolLightUpAndHorizontalWithCircle',
u'\u8a47': 'CjkUnifiedIdeograph8a47',
u'\ua9c6': 'UnicodeCharacter0xa9c6',
u'\u534d': 'CjkUnifiedIdeograph534d',
u'\u72cc': 'CjkUnifiedIdeograph72cc',
u'\ud951': 'UnicodeCharacter0xd951',
u'\uf8d0': 'UnicodeCharacter0xf8d0',
u'\u1c53': 'OlChikiDigitThree',
u'\u3bd2': 'CjkUnifiedIdeograph3bd2',
u'\ua257': 'YiSyllableHxox',
u'\u41d6': 'CjkUnifiedIdeograph41d6',
u'\uc7da': 'HangulSyllableJyabs',
u'\u6b5d': 'CjkUnifiedIdeograph6b5d',
u'\u0adc': 'UnicodeCharacter0xadc',
u'\uf161': 'UnicodeCharacter0xf161',
u'\u90e0': 'CjkUnifiedIdeograph90e0',
u'\u3463': 'CjkUnifiedIdeograph3463',
u'\uba67': 'HangulSyllableMes',
u'\u59e6': 'CjkUnifiedIdeograph59e6',
u'\uc06b': 'HangulSyllableBbeus',
u'\udfea': 'UnicodeCharacter0xdfea',
u'\u036d': 'CombiningLatinSmallLetterT',
u'\u22ec': 'NotNormalSubgroupOfOrEqualTo',
u'\u8971': 'CjkUnifiedIdeograph8971',
u'\ua8f0': 'UnicodeCharacter0xa8f0',
u'\u5277': 'CjkUnifiedIdeograph5277',
u'\u71f6': 'CjkUnifiedIdeograph71f6',
u'\ud87b': 'UnicodeCharacter0xd87b',
u'\uf7fa': 'UnicodeCharacter0xf7fa',
u'\u1b7d': 'UnicodeCharacter0x1b7d',
u'\u3afc': 'CjkUnifiedIdeograph3afc',
u'\u2081': 'SubscriptOne',
u'\u4100': 'CjkUnifiedIdeograph4100',
u'\u6583': 'CjkUnifiedIdeograph6583',
u'\u8202': 'CjkUnifiedIdeograph8202',
u'\ua685': 'CyrillicSmallLetterZhwe',
u'\uc704': 'HangulSyllableWi',
u'\ueb87': 'UnicodeCharacter0xeb87',
u'\u140e': 'CanadianSyllabicsWi',
u'\u3891': 'CjkUnifiedIdeograph3891',
u'\u5910': 'CjkUnifiedIdeograph5910',
u'\u7d93': 'CjkUnifiedIdeograph7d93',
u'\u9a12': 'CjkUnifiedIdeograph9a12',
u'\ube95': 'HangulSyllableBbaet',
u'\udf14': 'UnicodeCharacter0xdf14',
u'\u8397': 'CjkUnifiedIdeograph8397',
u'\uc499': 'HangulSyllableSsweot',
u'\u2c1e': 'GlagoliticCapitalLetterSha',
u'\u7120': 'CjkUnifiedIdeograph7120',
u'\u15a3': 'CanadianSyllabicsLhoo',
u'\ub222': 'HangulSyllableNweogg',
u'\u56a5': 'CjkUnifiedIdeograph56a5',
u'\uf724': 'UnicodeCharacter0xf724',
u'\u9ba7': 'CjkUnifiedIdeograph9ba7',
u'\udca9': 'UnicodeCharacter0xdca9',
u'\u0930': 'DevanagariLetterRa',
u'\u2db3': 'EthiopicSyllableZzaa',
u'\u4a32': 'CjkUnifiedIdeograph4a32',
u'\u6eb5': 'CjkUnifiedIdeograph6eb5',
u'\u8f34': 'CjkUnifiedIdeograph8f34',
u'\ub3b7': 'HangulSyllableDyelh',
u'\ud036': 'HangulSyllableKwigg',
u'\uf4b9': 'UnicodeCharacter0xf4b9',
u'\u2140': 'DoublestruckNarySummation',
u'\u6242': 'CjkUnifiedIdeograph6242',
u'\u06c5': 'ArabicLetterKirghizOe',
u'\ua744': 'LatinCapitalLetterKWithStrokeAndDiagonalStroke',
u'\u4bc7': 'CjkUnifiedIdeograph4bc7',
u'\ue846': 'UnicodeCharacter0xe846',
u'\u8cc9': 'CjkUnifiedIdeograph8cc9',
u'\ud1cb': 'HangulSyllableTwalh',
u'\u3950': 'CjkUnifiedIdeograph3950',
u'\u7a52': 'CjkUnifiedIdeograph7a52',
u'\u1ed5': 'LatinSmallLetterOWithCircumflexAndHookAbove',
u'\ubf54': 'HangulSyllableBboss',
u'\u63d7': 'CjkUnifiedIdeograph63d7',
u'\u8056': 'CjkUnifiedIdeograph8056',
u'\ua4d9': 'UnicodeCharacter0xa4d9',
u'\uc558': 'HangulSyllableAss',
u'\ue9db': 'UnicodeCharacter0xe9db',
u'\u1262': 'EthiopicSyllableBi',
u'\u36e5': 'CjkUnifiedIdeograph36e5',
u'\u5764': 'CjkUnifiedIdeograph5764',
u'\u7be7': 'CjkUnifiedIdeograph7be7',
u'\u9866': 'CjkUnifiedIdeograph9866',
u'\ubce9': 'HangulSyllableByeb',
u'\udd68': 'UnicodeCharacter0xdd68',
u'\u81eb': 'CjkUnifiedIdeograph81eb',
u'\uc2ed': 'HangulSyllableSib',
u'\u2a72': 'PlusSignAboveEqualsSign',
u'\u6f74': 'CjkUnifiedIdeograph6f74',
u'\u13f7': 'UnicodeCharacter0x13f7',
u'\ub076': 'HangulSyllableGgyij',
u'\u54f9': 'CjkUnifiedIdeograph54f9',
u'\uf578': 'UnicodeCharacter0xf578',
u'\u99fb': 'CjkUnifiedIdeograph99fb',
u'\udafd': 'UnicodeCharacter0xdafd',
u'\ue001': 'UnicodeCharacter0xe001',
u'\u0784': 'ThaanaLetterBaa',
u'\u4886': 'CjkUnifiedIdeograph4886',
u'\u8d88': 'CjkUnifiedIdeograph8d88',
u'\u310b': 'BopomofoLetterN',
u'\uce8a': 'HangulSyllableKaj',
u'\u720d': 'CjkUnifiedIdeograph720d',
u'\ub70f': 'HangulSyllableDdyugs',
u'\uf811': 'UnicodeCharacter0xf811',
u'\u1f94': 'GreekSmallLetterEtaWithPsiliAndOxiaAndYpogegrammeni',
u'\u6096': 'CjkUnifiedIdeograph6096',
u'\ua598': 'VaiSyllableHun',
u'\ue69a': 'UnicodeCharacter0xe69a',
u'\u0a1d': 'GurmukhiLetterJha',
u'\u4f1f': 'CjkUnifiedIdeograph4f1f',
u'\u9021': 'CjkUnifiedIdeograph9021',
u'\ud523': 'HangulSyllablePyigs',
u'\u37a4': 'CjkUnifiedIdeograph37a4',
u'\u78a6': 'CjkUnifiedIdeograph78a6',
u'\ubda8': 'HangulSyllableBweols',
u'\ufeaa': 'ArabicLetterDalFinalForm',
u'\u222d': 'TripleIntegral',
u'\uc3ac': 'HangulSyllableSsyeols',
u'\u672f': 'CjkUnifiedIdeograph672f',
u'\ua831': 'UnicodeCharacter0xa831',
u'\ued33': 'UnicodeCharacter0xed33',
u'\u10b6': 'GeorgianCapitalLetterGhan',
u'\u55b8': 'CjkUnifiedIdeograph55b8',
u'\u96ba': 'CjkUnifiedIdeograph96ba',
u'\u3a3d': 'CjkUnifiedIdeograph3a3d',
u'\udbbc': 'UnicodeCharacter0xdbbc',
u'\u7f3f': 'CjkUnifiedIdeograph7f3f',
u'\u4041': 'CjkUnifiedIdeograph4041',
u'\u8543': 'CjkUnifiedIdeograph8543',
u'\uc645': 'HangulSyllableWanj',
u'\u28c6': 'BraillePatternDots2378',
u'\u6dc8': 'CjkUnifiedIdeograph6dc8',
u'\uaeca': 'HangulSyllableGgeolp',
u'\uf3cc': 'UnicodeCharacter0xf3cc',
u'\u174f': 'BuhidLetterWa',
u'\u5851': 'CjkUnifiedIdeograph5851',
u'\u9d53': 'CjkUnifiedIdeograph9d53',
u'\ude55': 'UnicodeCharacter0xde55',
u'\u05d8': 'HebrewLetterTet',
u'\u46da': 'CjkUnifiedIdeograph46da',
u'\u8bdc': 'CjkUnifiedIdeograph8bdc',
u'\u2f5f': 'KangxiRadicalJade',
u'\uccde': 'HangulSyllableCyeolp',
u'\u7061': 'CjkUnifiedIdeograph7061',
u'\ub563': 'HangulSyllableDdaec',
u'\uf665': 'UnicodeCharacter0xf665',
u'\u1de8': 'UnicodeCharacter0x1de8',
u'\u5eea': 'CjkUnifiedIdeograph5eea',
u'\ua3ec': 'YiSyllableJot',
u'\ue4ee': 'UnicodeCharacter0xe4ee',
u'\u0871': 'UnicodeCharacter0x871',
u'\u4d73': 'CjkUnifiedIdeograph4d73',
u'\u8e75': 'CjkUnifiedIdeograph8e75',
u'\ud377': 'HangulSyllablePyaec',
u'\u35f8': 'CjkUnifiedIdeograph35f8',
u'\u76fa': 'CjkUnifiedIdeograph76fa',
u'\ubbfc': 'HangulSyllableMin',
u'\ufcfe': 'ArabicLigatureSheenWithYehIsolatedForm',
u'\uc200': 'HangulSyllableSyon',
u'\ue683': 'UnicodeCharacter0xe683',
u'\u0102': 'LatinCapitalLetterAWithBreve',
u'\u2585': 'LowerFiveEighthsBlock',
u'\u8b06': 'CjkUnifiedIdeograph8b06',
u'\uaf89': 'HangulSyllableGgoelg',
u'\u500c': 'CjkUnifiedIdeograph500c',
u'\u748f': 'CjkUnifiedIdeograph748f',
u'\uda10': 'UnicodeCharacter0xda10',
u'\ufe93': 'ArabicLetterTehMarbutaIsolatedForm',
u'\u1912': 'LimbuLetterBa',
u'\u3d95': 'CjkUnifiedIdeograph3d95',
u'\ua316': 'YiSyllableSuop',
u'\u4799': 'CjkUnifiedIdeograph4799',
u'\uc99d': 'HangulSyllableJeung',
u'\u681c': 'CjkUnifiedIdeograph681c',
u'\u0c9f': 'KannadaLetterTta',
u'\uf220': 'UnicodeCharacter0xf220',
u'\u96a3': 'CjkUnifiedIdeograph96a3',
u'\u3122': 'BopomofoLetterAn',
u'\ubb26': 'HangulSyllableMyolp',
u'\u5fa9': 'CjkUnifiedIdeograph5fa9',
u'\ue1ad': 'UnicodeCharacter0xe1ad',
u'\u24af': 'ParenthesizedLatinSmallLetterT',
u'\u8a30': 'CjkUnifiedIdeograph8a30',
u'\uaeb3': 'HangulSyllableGgyaes',
u'\u5336': 'CjkUnifiedIdeograph5336',
u'\u77b9': 'CjkUnifiedIdeograph77b9',
u'\ud53a': 'HangulSyllablePyip',
u'\uf9bd': 'CjkCompatibilityIdeographf9bd',
u'\u183c': 'MongolianLetterTsa',
u'\u3cbf': 'CjkUnifiedIdeograph3cbf',
u'\ua240': 'YiSyllableMgut',
u'\u46c3': 'CjkUnifiedIdeograph46c3',
u'\uc8c7': 'HangulSyllableJoegs',
u'\u6b46': 'CjkUnifiedIdeograph6b46',
u'\u0fc9': 'TibetanSymbolNorBu',
u'\ued4a': 'UnicodeCharacter0xed4a',
u'\u91cd': 'CjkUnifiedIdeograph91cd',
u'\u304c': 'HiraganaLetterGa',
u'\uba50': 'HangulSyllableMeok',
u'\u5ed3': 'CjkUnifiedIdeograph5ed3',
u'\ue0d7': 'UnicodeCharacter0xe0d7',
u'\u0356': 'CombiningRightArrowheadAndUpArrowheadBelow',
u'\u27d9': 'LargeDownTack',
u'\u855a': 'CjkUnifiedIdeograph855a',
u'\ua9dd': 'UnicodeCharacter0xa9dd',
u'\u5260': 'CjkUnifiedIdeograph5260',
u'\u76e3': 'CjkUnifiedIdeograph76e3',
u'\ud464': 'HangulSyllablePyol',
u'\uf8e7': 'UnicodeCharacter0xf8e7',
u'\u1b66': 'BalineseMusicalSymbolDing',
u'\u3fe9': 'CjkUnifiedIdeograph3fe9',
u'\u9d6a': 'CjkUnifiedIdeograph9d6a',
u'\u41ed': 'CjkUnifiedIdeograph41ed',
u'\ucbf1': 'HangulSyllableJjyig',
u'\u6a70': 'CjkUnifiedIdeograph6a70',
u'\u0ef3': 'UnicodeCharacter0xef3',
u'\uec74': 'UnicodeCharacter0xec74',
u'\u90f7': 'CjkUnifiedIdeograph90f7',
u'\u3376': 'SquarePc',
u'\ub57a': 'HangulSyllableDdyabs',
u'\u59fd': 'CjkUnifiedIdeograph59fd',
u'\u6301': 'CjkUnifiedIdeograph6301',
u'\u0280': 'LatinLetterSmallCapitalR',
u'\ue505': 'UnicodeCharacter0xe505',
u'\u8484': 'CjkUnifiedIdeograph8484',
u'\u2807': 'BraillePatternDots123',
u'\ub20b': 'HangulSyllableNud',
u'\u4d8a': 'CjkUnifiedIdeograph4d8a',
u'\ud78e': 'HangulSyllableHinh',
u'\u7b11': 'CjkUnifiedIdeograph7b11',
u'\u1a90': 'UnicodeCharacter0x1a90',
u'\ufd15': 'ArabicLigatureGhainWithAlefMaksuraFinalForm',
u'\u9c94': 'CjkUnifiedIdeograph9c94',
u'\u4a1b': 'CjkUnifiedIdeograph4a1b',
u'\u659a': 'CjkUnifiedIdeograph659a',
u'\ucc1f': 'HangulSyllableJjis',
u'\uef9e': 'UnicodeCharacter0xef9e',
u'\u1321': 'EthiopicSyllableThu',
u'\u32a0': 'CircledIdeographItem',
u'\u9525': 'CjkUnifiedIdeograph9525',
u'\ub4a4': 'HangulSyllableDwi',
u'\u622b': 'CjkUnifiedIdeograph622b',
u'\u7daa': 'CjkUnifiedIdeograph7daa',
u'\ue42f': 'UnicodeCharacter0xe42f',
u'\u87ae': 'CjkUnifiedIdeograph87ae',
u'\u2b31': 'ThreeLeftwardsArrows',
u'\uad35': 'HangulSyllableGoeg',
u'\u4cb4': 'CjkUnifiedIdeograph4cb4',
u'\ud6b8': 'HangulSyllableHyom',
u'\u7a3b': 'CjkUnifiedIdeograph7a3b',
u'\u15ba': 'CanadianSyllabicsBlackfootNo',
u'\ufc3f': 'ArabicLigatureLamWithJeemIsolatedForm',
u'\u9fbe': 'UnicodeCharacter0x9fbe',
u'\u4545': 'CjkUnifiedIdeograph4545',
u'\u64c4': 'CjkUnifiedIdeograph64c4',
u'\ucf49': 'HangulSyllableKyeb',
u'\ueec8': 'UnicodeCharacter0xeec8',
u'\u124b': 'EthiopicSyllableQwaa',
u'\u2dca': 'EthiopicSyllableKyi',
u'\u944f': 'CjkUnifiedIdeograph944f',
u'\ub7ce': 'HangulSyllableRyap',
u'\u5d55': 'CjkUnifiedIdeograph5d55',
u'\u7cd4': 'CjkUnifiedIdeograph7cd4',
u'\ue759': 'UnicodeCharacter0xe759',
u'\u86d8': 'CjkUnifiedIdeograph86d8',
u'\u2a5b': 'LogicalOrWithMiddleStem',
u'\uac5f': 'HangulSyllableGyaelb',
u'\u4fde': 'CjkUnifiedIdeograph4fde',
u'\ud1e2': 'HangulSyllableTwaelm',
u'\u7565': 'CjkUnifiedIdeograph7565',
u'\u14e4': 'CanadianSyllabicsLwoo',
u'\uff69': 'HalfwidthKatakanaLetterSmallU',
u'\u9ee8': 'CjkUnifiedIdeograph9ee8',
u'\u446f': 'CjkUnifiedIdeograph446f',
u'\u67ee': 'CjkUnifiedIdeograph67ee',
u'\uce73': 'HangulSyllableCih',
u'\ue9f2': 'UnicodeCharacter0xe9f2',
u'\u0d75': 'MalayalamFractionThreeQuarters',
u'\u2cf4': 'UnicodeCharacter0x2cf4',
u'\u9779': 'CjkUnifiedIdeograph9779',
u'\ub6f8': 'HangulSyllableDdwil',
u'\u5c7f': 'CjkUnifiedIdeograph5c7f',
u'\u7ffe': 'CjkUnifiedIdeograph7ffe',
u'\u0a06': 'GurmukhiLetterAa',
u'\u2e89': 'CjkRadicalKnifeTwo',
u'\u4f08': 'CjkUnifiedIdeograph4f08',
u'\u738b': 'CjkUnifiedIdeograph738b',
u'\u8c0a': 'CjkUnifiedIdeograph8c0a',
u'\ub08d': 'HangulSyllableGgib',
u'\ud10c': 'HangulSyllableTyass',
u'\uf58f': 'UnicodeCharacter0xf58f',
u'\u2216': 'SetMinus',
u'\u6718': 'CjkUnifiedIdeograph6718',
u'\u0b9b': 'UnicodeCharacter0xb9b',
u'\ua41a': 'YiSyllableJji',
u'\u489d': 'CjkUnifiedIdeograph489d',
u'\ue91c': 'UnicodeCharacter0xe91c',
u'\u8d9f': 'CjkUnifiedIdeograph8d9f',
u'\ud2a1': 'HangulSyllableTyunj',
u'\u3a26': 'CjkUnifiedIdeograph3a26',
u'\u7f28': 'CjkUnifiedIdeograph7f28',
u'\u23ab': 'RightCurlyBracketUpperHook',
u'\ubc2a': 'HangulSyllableBaj',
u'\u60ad': 'CjkUnifiedIdeograph60ad',
u'\u812c': 'CjkUnifiedIdeograph812c',
u'\ua5af': 'VaiSyllableZhu',
u'\uc62e': 'HangulSyllableOlm',
u'\ueab1': 'UnicodeCharacter0xeab1',
u'\u1738': 'UnicodeCharacter0x1738',
u'\u3bbb': 'CjkUnifiedIdeograph3bbb',
u'\u543a': 'CjkUnifiedIdeograph543a',
u'\u78bd': 'CjkUnifiedIdeograph78bd',
u'\u993c': 'CjkUnifiedIdeograph993c',
u'\ubdbf': 'HangulSyllableBwed',
u'\ude3e': 'UnicodeCharacter0xde3e',
u'\u82c1': 'CjkUnifiedIdeograph82c1',
u'\uc7c3': 'HangulSyllableJaec',
u'\u2f48': 'KangxiRadicalSay',
u'\u6c4a': 'CjkUnifiedIdeograph6c4a',
u'\u10cd': 'UnicodeCharacter0x10cd',
u'\ub14c': 'HangulSyllableNyeols',
u'\u55cf': 'CjkUnifiedIdeograph55cf',
u'\uf64e': 'UnicodeCharacter0xf64e',
u'\u9ad1': 'CjkUnifiedIdeograph9ad1',
u'\udfd3': 'UnicodeCharacter0xdfd3',
u'\u045a': 'CyrillicSmallLetterNje',
u'\u28dd': 'BraillePatternDots134578',
u'\u495c': 'CjkUnifiedIdeograph495c',
u'\u6ddf': 'CjkUnifiedIdeograph6ddf',
u'\u8e5e': 'CjkUnifiedIdeograph8e5e',
u'\ub2e1': 'HangulSyllableNit',
u'\ud360': 'HangulSyllablePyae',
u'\uf7e3': 'UnicodeCharacter0xf7e3',
u'\u1c6a': 'OlChikiLetterUc',
u'\u616c': 'CjkUnifiedIdeograph616c',
u'\u05ef': 'UnicodeCharacter0x5ef',
u'\ua66e': 'CyrillicLetterMultiocularO',
u'\u4af1': 'CjkUnifiedIdeograph4af1',
u'\ueb70': 'UnicodeCharacter0xeb70',
u'\u8ff3': 'CjkUnifiedIdeograph8ff3',
u'\uccf5': 'HangulSyllableCyelg',
u'\u347a': 'CjkUnifiedIdeograph347a',
u'\u797c': 'CjkUnifiedIdeograph797c',
u'\u1dff': 'CombiningRightArrowheadAndDownArrowheadBelow',
u'\ube7e': 'HangulSyllableBbaegg',
u'\u8380': 'CjkUnifiedIdeograph8380',
u'\u2703': 'LowerBladeScissors',
u'\uc082': 'HangulSyllableBbyilp',
u'\u6405': 'CjkUnifiedIdeograph6405',
u'\ua907': 'KayahLiDigitSeven',
u'\uee09': 'UnicodeCharacter0xee09',
u'\u118c': 'HangulJungseongUye',
u'\u568e': 'CjkUnifiedIdeograph568e',
u'\u9b90': 'CjkUnifiedIdeograph9b90',
u'\u3f13': 'CjkUnifiedIdeograph3f13',
u'\ud892': 'UnicodeCharacter0xd892',
u'\u7c15': 'CjkUnifiedIdeograph7c15',
u'\u4117': 'CjkUnifiedIdeograph4117',
u'\u8619': 'CjkUnifiedIdeograph8619',
u'\ucb1b': 'HangulSyllableJjoelb',
u'\u299c': 'RightAngleVariantWithSquare',
u'\u6e9e': 'CjkUnifiedIdeograph6e9e',
u'\ub3a0': 'HangulSyllableDyeoss',
u'\uf0a2': 'UnicodeCharacter0xf0a2',
u'\u1425': 'CanadianSyllabicsFinalDoubleAcute',
u'\u5927': 'CjkUnifiedIdeograph5927',
u'\u9e29': 'CjkUnifiedIdeograph9e29',
u'\ue32b': 'UnicodeCharacter0xe32b',
u'\u06ae': 'ArabicLetterKafWithThreeDotsBelow',
u'\u4bb0': 'CjkUnifiedIdeograph4bb0',
u'\u88b2': 'CjkUnifiedIdeograph88b2',
u'\u2c35': 'GlagoliticSmallLetterYestu',
u'\ucdb4': 'HangulSyllableCweon',
u'\u7137': 'CjkUnifiedIdeograph7137',
u'\ub639': 'HangulSyllableDdwalt',
u'\ufb3b': 'HebrewLetterKafWithDagesh',
u'\u1ebe': 'LatinCapitalLetterEWithCircumflexAndAcute',
u'\u63c0': 'CjkUnifiedIdeograph63c0',
u'\ua0c2': 'YiSyllableMe',
u'\ue5c4': 'UnicodeCharacter0xe5c4',
u'\u0947': 'DevanagariVowelSignE',
u'\u4e49': 'CjkUnifiedIdeograph4e49',
u'\u934b': 'CjkUnifiedIdeograph934b',
u'\ud04d': 'HangulSyllableKwit',
u'\u36ce': 'CjkUnifiedIdeograph36ce',
u'\u7bd0': 'CjkUnifiedIdeograph7bd0',
u'\ub8d2': 'HangulSyllableRyonh',
u'\ufdd4': 'UnicodeCharacter0xfdd4',
u'\u2157': 'VulgarFractionThreeFifths',
u'\uc2d6': 'HangulSyllableSyij',
u'\u6659': 'CjkUnifiedIdeograph6659',
u'\uab5b': 'UnicodeCharacter0xab5b',
u'\ue85d': 'UnicodeCharacter0xe85d',
u'\u13e0': 'CherokeeLetterTlo',
u'\u50e2': 'CjkUnifiedIdeograph50e2',
u'\u95e4': 'CjkUnifiedIdeograph95e4',
u'\u3967': 'CjkUnifiedIdeograph3967',
u'\udae6': 'UnicodeCharacter0xdae6',
u'\u7e69': 'CjkUnifiedIdeograph7e69',
u'\u436b': 'CjkUnifiedIdeograph436b',
u'\u806d': 'CjkUnifiedIdeograph806d',
u'\uc56f': 'HangulSyllableAelh',
u'\u2bf0': 'UnicodeCharacter0x2bf0',
u'\u68f2': 'CjkUnifiedIdeograph68f2',
u'\uadf4': 'HangulSyllableGyuk',
u'\uf2f6': 'UnicodeCharacter0xf2f6',
u'\u1679': 'UnicodeCharacter0x1679',
u'\u5b7b': 'CjkUnifiedIdeograph5b7b',
u'\u987d': 'CjkUnifiedIdeograph987d',
u'\udd7f': 'UnicodeCharacter0xdd7f',
u'\ua581': 'VaiSyllableDhoo',
u'\u4204': 'CjkUnifiedIdeograph4204',
u'\u6e87': 'CjkUnifiedIdeograph6e87',
u'\uc808': 'HangulSyllableJeol',
u'\uf48b': 'UnicodeCharacter0xf48b',
u'\u0b0a': 'OriyaLetterUu',
u'\u378d': 'CjkUnifiedIdeograph378d',
u'\u910e': 'CjkUnifiedIdeograph910e',
u'\ubd91': 'HangulSyllableBub',
u'\u5a14': 'CjkUnifiedIdeograph5a14',
u'\u0697': 'ArabicLetterRehWithTwoDotsAbove',
u'\ue018': 'UnicodeCharacter0xe018',
u'\u8c9b': 'CjkUnifiedIdeograph8c9b',
u'\u231a': 'Watch',
u'\ua91e': 'KayahLiLetterTha',
u'\u55a1': 'CjkUnifiedIdeograph55a1',
u'\ud3a5': 'HangulSyllablePelt',
u'\u7224': 'CjkUnifiedIdeograph7224',
u'\u1ea7': 'LatinSmallLetterAWithCircumflexAndGrave',
u'\uf828': 'UnicodeCharacter0xf828',
u'\ua4ab': 'YiRadicalZot',
u'\u3b2a': 'CjkUnifiedIdeograph3b2a',
u'\u412e': 'CjkUnifiedIdeograph412e',
u'\u6db1': 'CjkUnifiedIdeograph6db1',
u'\ucf32': 'HangulSyllableKyeoj',
u'\uebb5': 'UnicodeCharacter0xebb5',
u'\u0a34': 'UnicodeCharacter0xa34',
u'\u36b7': 'CjkUnifiedIdeograph36b7',
u'\u9038': 'CjkUnifiedIdeograph9038',
u'\ubcbb': 'HangulSyllableBeh',
u'\u593e': 'CjkUnifiedIdeograph593e',
u'\u05c1': 'HebrewPointShinDot',
u'\ue742': 'UnicodeCharacter0xe742',
u'\u83c5': 'CjkUnifiedIdeograph83c5',
u'\u2244': 'NotAsymptoticallyEqualTo',
u'\ua848': 'PhagspaLetterTa',
u'\u54cb': 'CjkUnifiedIdeograph54cb',
u'\ud2cf': 'HangulSyllableTeuc',
u'\u714e': 'CjkUnifiedIdeograph714e',
u'\u1dd1': 'CombiningUrAbove',
u'\uff52': 'FullwidthLatinSmallLetterR',
u'\u9bd5': 'CjkUnifiedIdeograph9bd5',
u'\u3a54': 'CjkUnifiedIdeograph3a54',
u'\u4058': 'CjkUnifiedIdeograph4058',
u'\u6cdb': 'CjkUnifiedIdeograph6cdb',
u'\uce5c': 'HangulSyllableCin',
u'\ueadf': 'UnicodeCharacter0xeadf',
u'\u095e': 'DevanagariLetterFa',
u'\u35e1': 'CjkUnifiedIdeograph35e1',
u'\u9762': 'CjkUnifiedIdeograph9762',
u'\ub3e5': 'HangulSyllableDwanj',
u'\u5868': 'CjkUnifiedIdeograph5868',
u'\u04eb': 'CyrillicSmallLetterBarredOWithDiaeresis',
u'\ue66c': 'UnicodeCharacter0xe66c',
u'\u82ef': 'CjkUnifiedIdeograph82ef',
u'\u216e': 'RomanNumeralFiveHundred',
u'\uaf72': 'HangulSyllableGgwaelp',
u'\u4bf5': 'CjkUnifiedIdeograph4bf5',
u'\ud1f9': 'HangulSyllableToenj',
u'\u7078': 'CjkUnifiedIdeograph7078',
u'\u1cfb': 'UnicodeCharacter0x1cfb',
u'\ufe7c': 'ArabicShaddaIsolatedForm',
u'\u9aff': 'CjkUnifiedIdeograph9aff',
u'\u397e': 'CjkUnifiedIdeograph397e',
u'\ua003': 'YiSyllableIp',
u'\u4782': 'CjkUnifiedIdeograph4782',
u'\ucd86': 'HangulSyllableCyolp',
u'\u6909': 'CjkUnifiedIdeograph6909',
u'\u0888': 'UnicodeCharacter0x888',
u'\uf70d': 'UnicodeCharacter0xf70d',
u'\u968c': 'CjkUnifiedIdeograph968c',
u'\u320f': 'ParenthesizedHangulNieunA',
u'\ub813': 'HangulSyllableRelb',
u'\u5f92': 'CjkUnifiedIdeograph5f92',
u'\uc617': 'HangulSyllableYelh',
u'\ue596': 'UnicodeCharacter0xe596',
u'\u0119': 'LatinSmallLetterEWithOgonek',
u'\u2098': 'UnicodeCharacter0x2098',
u'\u8f1d': 'CjkUnifiedIdeograph8f1d',
u'\uae9c': 'HangulSyllableGgyak',
u'\u5023': 'CjkUnifiedIdeograph5023',
u'\u77a2': 'CjkUnifiedIdeograph77a2',
u'\ude27': 'UnicodeCharacter0xde27',
u'\ufda6': 'ArabicLigatureJeemWithHahWithAlefMaksuraFinalForm',
u'\u1929': 'LimbuSubjoinedLetterYa',
u'\u38a8': 'CjkUnifiedIdeograph38a8',
u'\ua72d': 'LatinSmallLetterCuatrillo',
u'\u46ac': 'CjkUnifiedIdeograph46ac',
u'\uccb0': 'HangulSyllableCeok',
u'\u6833': 'CjkUnifiedIdeograph6833',
u'\u0fb2': 'TibetanSubjoinedLetterRa',
u'\uf637': 'UnicodeCharacter0xf637',
u'\u95b6': 'CjkUnifiedIdeograph95b6',
u'\u3139': 'HangulLetterRieul',
u'\ubf3d': 'HangulSyllableBbyet',
u'\u5ebc': 'CjkUnifiedIdeograph5ebc',
u'\uc541': 'HangulSyllableSsit',
u'\ue4c0': 'UnicodeCharacter0xe4c0',
u'\u27c2': 'Perpendicular',
u'\u8e47': 'CjkUnifiedIdeograph8e47',
u'\uadc6': 'HangulSyllableGwinh',
u'\u574d': 'CjkUnifiedIdeograph574d',
u'\u76cc': 'CjkUnifiedIdeograph76cc',
u'\udd51': 'UnicodeCharacter0xdd51',
u'\ufcd0': 'ArabicLigatureMeemWithKhahInitialForm',
u'\u1853': 'MongolianLetterTodoJa',
u'\u3fd2': 'CjkUnifiedIdeograph3fd2',
u'\ua657': 'CyrillicSmallLetterIotifiedA',
u'\u45d6': 'CjkUnifiedIdeograph45d6',
u'\uc3da': 'HangulSyllableSsogg',
u'\u6f5d': 'CjkUnifiedIdeograph6f5d',
u'\u0edc': 'LaoHoNo',
u'\uf561': 'UnicodeCharacter0xf561',
u'\u94e0': 'CjkUnifiedIdeograph94e0',
u'\u3063': 'HiraganaLetterSmallTu',
u'\ube67': 'HangulSyllableBbad',
u'\u5de6': 'CjkUnifiedIdeograph5de6',
u'\uc46b': 'HangulSyllableSsud',
u'\udbea': 'UnicodeCharacter0xdbea',
u'\u076d': 'ArabicLetterSeenWithTwoDotsVerticallyAbove',
u'\u26ec': 'UnicodeCharacter0x26ec',
u'\u8d71': 'CjkUnifiedIdeograph8d71',
u'\uacf0': 'HangulSyllableGom',
u'\u5677': 'CjkUnifiedIdeograph5677',
u'\u75f6': 'CjkUnifiedIdeograph75f6',
u'\udc7b': 'UnicodeCharacter0xdc7b',
u'\uf3fa': 'UnicodeCharacter0xf3fa',
u'\u1f7d': 'GreekSmallLetterOmegaWithOxia',
u'\u3efc': 'CjkUnifiedIdeograph3efc',
u'\u2481': 'ParenthesizedNumberFourteen',
u'\u4500': 'CjkUnifiedIdeograph4500',
u'\u6183': 'CjkUnifiedIdeograph6183',
u'\u8602': 'CjkUnifiedIdeograph8602',
u'\ua285': 'YiSyllableWuo',
u'\uc304': 'HangulSyllableSsals',
u'\uef87': 'UnicodeCharacter0xef87',
u'\u100e': 'MyanmarLetterDdha',
u'\u3c91': 'CjkUnifiedIdeograph3c91',
u'\u5d10': 'CjkUnifiedIdeograph5d10',
u'\u7993': 'CjkUnifiedIdeograph7993',
u'\u9e12': 'CjkUnifiedIdeograph9e12',
u'\uba95': 'HangulSyllableMyelg',
u'\udb14': 'UnicodeCharacter0xdb14',
u'\u8797': 'CjkUnifiedIdeograph8797',
u'\uc099': 'HangulSyllableBbilg',
u'\u281e': 'BraillePatternDots2345',
u'\u7520': 'CjkUnifiedIdeograph7520',
u'\u11a3': 'UnicodeCharacter0x11a3',
u'\ub622': 'HangulSyllableDdobs',
u'\u52a5': 'CjkUnifiedIdeograph52a5',
u'\uf324': 'UnicodeCharacter0xf324',
u'\u9fa7': 'CjkUnifiedIdeograph9fa7',
u'\ud8a9': 'UnicodeCharacter0xd8a9',
u'\u0d30': 'MalayalamLetterRa',
u'\u29b3': 'EmptySetWithRightArrowAbove',
u'\u4e32': 'CjkUnifiedIdeograph4e32',
u'\u6ab5': 'CjkUnifiedIdeograph6ab5',
u'\u8b34': 'CjkUnifiedIdeograph8b34',
u'\ub7b7': 'HangulSyllableRyags',
u'\ud436': 'HangulSyllablePwaebs',
u'\uf0b9': 'UnicodeCharacter0xf0b9',
u'\u2540': 'BoxDrawingsUpHeavyAndDownHorizontalLight',
u'\u6642': 'CjkUnifiedIdeograph6642',
u'\u02c5': 'ModifierLetterDownArrowhead',
u'\ua344': 'YiSyllableSsyrx',
u'\u4fc7': 'CjkUnifiedIdeograph4fc7',
u'\uec46': 'UnicodeCharacter0xec46',
u'\u88c9': 'CjkUnifiedIdeograph88c9',
u'\ud5cb': 'HangulSyllableHeogs',
u'\u3d50': 'CjkUnifiedIdeograph3d50',
u'\u7e52': 'CjkUnifiedIdeograph7e52',
u'\u1ad5': 'UnicodeCharacter0x1ad5',
u'\ubb54': 'HangulSyllableMweon',
u'\u67d7': 'CjkUnifiedIdeograph67d7',
u'\u8456': 'CjkUnifiedIdeograph8456',
u'\ua0d9': 'YiSyllableFux',
u'\uc158': 'HangulSyllableSyeon',
u'\ueddb': 'UnicodeCharacter0xeddb',
u'\u1662': 'CanadianSyllabicsCarrierCho',
u'\u32e5': 'CircledKatakanaNi',
u'\u5364': 'CjkUnifiedIdeograph5364',
u'\u7fe7': 'CjkUnifiedIdeograph7fe7',
u'\u9c66': 'CjkUnifiedIdeograph9c66',
u'\ub8e9': 'HangulSyllableRug',
u'\ud968': 'UnicodeCharacter0xd968',
u'\u85eb': 'CjkUnifiedIdeograph85eb',
u'\uc6ed': 'HangulSyllableWenj',
u'\u2e72': 'UnicodeCharacter0x2e72',
u'\u6b74': 'CjkUnifiedIdeograph6b74',
u'\u17f7': 'KhmerSymbolLekAttakPrampii',
u'\ub476': 'HangulSyllableDweolm',
u'\u50f9': 'CjkUnifiedIdeograph50f9',
u'\uf178': 'UnicodeCharacter0xf178',
u'\u9dfb': 'CjkUnifiedIdeograph9dfb',
u'\udefd': 'UnicodeCharacter0xdefd',
u'\ue401': 'UnicodeCharacter0xe401',
u'\u0384': 'GreekTonos',
u'\u4c86': 'CjkUnifiedIdeograph4c86',
u'\u8988': 'CjkUnifiedIdeograph8988',
u'\u350b': 'CjkUnifiedIdeograph350b',
u'\uca8a': 'HangulSyllableJjyeonh',
u'\u760d': 'CjkUnifiedIdeograph760d',
u'\ub30f': 'HangulSyllableDaelh',
u'\ufc11': 'ArabicLigatureThehWithJeemIsolatedForm',
u'\u1b94': 'SundaneseLetterNa',
u'\u6496': 'CjkUnifiedIdeograph6496',
u'\ua198': 'YiSyllableHlix',
u'\ue29a': 'UnicodeCharacter0xe29a',
u'\u0e1d': 'ThaiCharacterFoFa',
u'\u4b1f': 'CjkUnifiedIdeograph4b1f',
u'\u9421': 'CjkUnifiedIdeograph9421',
u'\ud123': 'HangulSyllableTyaelh',
u'\u33a4': 'SquareCmCubed',
u'\u7ca6': 'CjkUnifiedIdeograph7ca6',
u'\ub9a8': 'HangulSyllableRyik',
u'\ufaaa': 'CjkCompatibilityIdeographfaaa',
u'\u262d': 'HammerAndSickle',
u'\uc7ac': 'HangulSyllableJae',
u'\u632f': 'CjkUnifiedIdeograph632f',
u'\uac31': 'HangulSyllableGaeng',
u'\ue933': 'UnicodeCharacter0xe933',
u'\u14b6': 'CanadianSyllabicsMwa',
u'\u51b8': 'CjkUnifiedIdeograph51b8',
u'\u92ba': 'CjkUnifiedIdeograph92ba',
u'\u3e3d': 'CjkUnifiedIdeograph3e3d',
u'\udfbc': 'UnicodeCharacter0xdfbc',
u'\u7b3f': 'CjkUnifiedIdeograph7b3f',
u'\u4441': 'CjkUnifiedIdeograph4441',
u'\u8143': 'CjkUnifiedIdeograph8143',
u'\uc245': 'HangulSyllableSweob',
u'\u2cc6': 'CopticCapitalLetterOldCopticEsh',
u'\u69c8': 'CjkUnifiedIdeograph69c8',
u'\uaaca': 'UnicodeCharacter0xaaca',
u'\uf7cc': 'UnicodeCharacter0xf7cc',
u'\u134f': 'EthiopicSyllableFwa',
u'\u5c51': 'CjkUnifiedIdeograph5c51',
u'\u9953': 'CjkUnifiedIdeograph9953',
u'\uda55': 'UnicodeCharacter0xda55',
u'\u01d8': 'LatinSmallLetterUWithDiaeresisAndAcute',
u'\u42da': 'CjkUnifiedIdeograph42da',
u'\u8fdc': 'CjkUnifiedIdeograph8fdc',
u'\u2b5f': 'UnicodeCharacter0x2b5f',
u'\uc8de': 'HangulSyllableJoep',
u'\u7461': 'CjkUnifiedIdeograph7461',
u'\ub163': 'HangulSyllableNyed',
u'\uf265': 'UnicodeCharacter0xf265',
u'\u19e8': 'KhmerSymbolPrambeiKoet',
u'\u5aea': 'CjkUnifiedIdeograph5aea',
u'\ua7ec': 'UnicodeCharacter0xa7ec',
u'\ue0ee': 'UnicodeCharacter0xe0ee',
u'\u0c71': 'UnicodeCharacter0xc71',
u'\u4973': 'CjkUnifiedIdeograph4973',
u'\u8a75': 'CjkUnifiedIdeograph8a75',
u'\ud777': 'HangulSyllableHyilb',
u'\u31f8': 'KatakanaLetterSmallHe',
u'\u72fa': 'CjkUnifiedIdeograph72fa',
u'\ubffc': 'HangulSyllableBbweoss',
u'\uf8fe': 'UnicodeCharacter0xf8fe',
u'\uc600': 'HangulSyllableYeoss',
u'\ue283': 'UnicodeCharacter0xe283',
u'\u0502': 'CyrillicCapitalLetterKomiDje',
u'\u2185': 'RomanNumeralSixLateForm',
u'\u8f06': 'CjkUnifiedIdeograph8f06',
u'\uab89': 'UnicodeCharacter0xab89',
u'\u540c': 'CjkUnifiedIdeograph540c',
u'\u708f': 'CjkUnifiedIdeograph708f',
u'\ude10': 'UnicodeCharacter0xde10',
u'\ufa93': 'CjkCompatibilityIdeographfa93',
u'\u1d12': 'LatinSmallLetterSidewaysOpenO',
u'\u3995': 'CjkUnifiedIdeograph3995',
u'\ua716': 'ModifierLetterExtralowLeftstemToneBar',
u'\u4399': 'CjkUnifiedIdeograph4399',
u'\ucd9d': 'HangulSyllableCulg',
u'\u6c1c': 'CjkUnifiedIdeograph6c1c',
u'\u089f': 'UnicodeCharacter0x89f',
u'\uf620': 'UnicodeCharacter0xf620',
u'\u92a3': 'CjkUnifiedIdeograph92a3',
u'\u3522': 'CjkUnifiedIdeograph3522',
u'\ubf26': 'HangulSyllableBbyegg',
u'\u5ba9': 'CjkUnifiedIdeograph5ba9',
u'\ue5ad': 'UnicodeCharacter0xe5ad',
u'\u042c': 'CyrillicCapitalLetterSoftSign',
u'\u20af': 'DrachmaSign',
u'\u8e30': 'CjkUnifiedIdeograph8e30',
u'\uaab3': 'UnicodeCharacter0xaab3',
u'\u5736': 'CjkUnifiedIdeograph5736',
u'\u73b9': 'CjkUnifiedIdeograph73b9',
u'\ud13a': 'HangulSyllableTeolm',
u'\ufdbd': 'ArabicLigatureNoonWithJeemWithHahFinalForm',
u'\u1c3c': 'LepchaPunctuationNyetThyoomTarol',
u'\u38bf': 'CjkUnifiedIdeograph38bf',
u'\ua640': 'CyrillicCapitalLetterZemlya',
u'\u42c3': 'CjkUnifiedIdeograph42c3',
u'\uccc7': 'HangulSyllableCes',
u'\u6f46': 'CjkUnifiedIdeograph6f46',
u'\u0bc9': 'UnicodeCharacter0xbc9',
u'\ue94a': 'UnicodeCharacter0xe94a',
u'\u95cd': 'CjkUnifiedIdeograph95cd',
u'\u344c': 'CjkUnifiedIdeograph344c',
u'\ube50': 'HangulSyllableBils',
u'\u5ad3': 'CjkUnifiedIdeograph5ad3',
u'\ue4d7': 'UnicodeCharacter0xe4d7',
u'\u0756': 'ArabicLetterBehWithSmallV',
u'\u23d9': 'MetricalPentaseme',
u'\u815a': 'CjkUnifiedIdeograph815a',
u'\uaddd': 'HangulSyllableGyug',
u'\u5660': 'CjkUnifiedIdeograph5660',
u'\u72e3': 'CjkUnifiedIdeograph72e3',
u'\ud064': 'HangulSyllableKyuss',
u'\ufce7': 'ArabicLigatureSeenWithMeemMedialForm',
u'\u1f66': 'GreekSmallLetterOmegaWithPsiliAndPerispomeni',
u'\u3be9': 'CjkUnifiedIdeograph3be9',
u'\u996a': 'CjkUnifiedIdeograph996a',
u'\u45ed': 'CjkUnifiedIdeograph45ed',
u'\ucff1': 'HangulSyllableKub',
u'\u6e70': 'CjkUnifiedIdeograph6e70',
u'\u0af3': 'UnicodeCharacter0xaf3',
u'\ue874': 'UnicodeCharacter0xe874',
u'\u94f7': 'CjkUnifiedIdeograph94f7',
u'\u3776': 'CjkUnifiedIdeograph3776',
u'\ub17a': 'HangulSyllableNogg',
u'\u5dfd': 'CjkUnifiedIdeograph5dfd',
u'\u6701': 'CjkUnifiedIdeograph6701',
u'\u0680': 'ArabicLetterBeheh',
u'\ue105': 'UnicodeCharacter0xe105',
u'\u8084': 'CjkUnifiedIdeograph8084',
u'\u2c07': 'GlagoliticCapitalLetterDzelo',
u'\ub60b': 'HangulSyllableDdyec',
u'\u498a': 'CjkUnifiedIdeograph498a',
u'\ud38e': 'HangulSyllablePeobs',
u'\u7f11': 'CjkUnifiedIdeograph7f11',
u'\u1e90': 'LatinCapitalLetterZWithCircumflex',
u'\uf915': 'CjkCompatibilityIdeographf915',
u'\u9894': 'CjkUnifiedIdeograph9894',
u'\u4e1b': 'CjkUnifiedIdeograph4e1b',
u'\u619a': 'CjkUnifiedIdeograph619a',
u'\uc81f': 'HangulSyllableJegs',
u'\ueb9e': 'UnicodeCharacter0xeb9e',
u'\u1721': 'HanunooLetterI',
u'\u36a0': 'CjkUnifiedIdeograph36a0',
u'\u9125': 'CjkUnifiedIdeograph9125',
u'\ub0a4': 'HangulSyllableNals',
u'\u662b': 'CjkUnifiedIdeograph662b',
u'\u79aa': 'CjkUnifiedIdeograph79aa',
u'\ue02f': 'UnicodeCharacter0xe02f',
u'\u83ae': 'CjkUnifiedIdeograph83ae',
u'\u2f31': 'KangxiRadicalTurban',
u'\ua935': 'RejangLetterNa',
u'\u48b4': 'CjkUnifiedIdeograph48b4',
u'\ud2b8': 'HangulSyllableTeu',
u'\u7e3b': 'CjkUnifiedIdeograph7e3b',
u'\u11ba': 'HangulJongseongSios',
u'\uf83f': 'UnicodeCharacter0xf83f',
u'\u9bbe': 'CjkUnifiedIdeograph9bbe',
u'\u4145': 'CjkUnifiedIdeograph4145',
u'\u60c4': 'CjkUnifiedIdeograph60c4',
u'\ucb49': 'HangulSyllableJjug',
u'\ueac8': 'UnicodeCharacter0xeac8',
u'\u164b': 'CanadianSyllabicsCarrierDzee',
u'\u29ca': 'TriangleWithDotAbove',
u'\u904f': 'CjkUnifiedIdeograph904f',
u'\ub3ce': 'HangulSyllableDolm',
u'\u5955': 'CjkUnifiedIdeograph5955',
u'\u78d4': 'CjkUnifiedIdeograph78d4',
u'\ue359': 'UnicodeCharacter0xe359',
u'\u82d8': 'CjkUnifiedIdeograph82d8',
u'\u2e5b': 'UnicodeCharacter0x2e5b',
u'\ua85f': 'PhagspaLetterU',
u'\u4bde': 'CjkUnifiedIdeograph4bde',
u'\ud5e2': 'HangulSyllableHeop',
u'\u7165': 'CjkUnifiedIdeograph7165',
u'\u10e4': 'GeorgianLetterPhar',
u'\ufb69': 'ArabicLetterTtehMedialForm',
u'\u9ae8': 'CjkUnifiedIdeograph9ae8',
u'\u406f': 'CjkUnifiedIdeograph406f',
u'\u63ee': 'CjkUnifiedIdeograph63ee',
u'\uca73': 'HangulSyllableJjelb',
u'\uedf2': 'UnicodeCharacter0xedf2',
u'\u0975': 'UnicodeCharacter0x975',
u'\u28f4': 'BraillePatternDots35678',
u'\u9379': 'CjkUnifiedIdeograph9379',
u'\ub2f8': 'HangulSyllableDass',
u'\u587f': 'CjkUnifiedIdeograph587f',
u'\u7bfe': 'CjkUnifiedIdeograph7bfe',
u'\u0e06': 'ThaiCharacterKhoRakhang',
u'\u2a89': 'LessthanAndNotApproximate',
u'\u4b08': 'CjkUnifiedIdeograph4b08',
u'\u778b': 'CjkUnifiedIdeograph778b',
u'\u880a': 'CjkUnifiedIdeograph880a',
u'\ub48d': 'HangulSyllableDwenj',
u'\ud50c': 'HangulSyllablePeul',
u'\uf18f': 'UnicodeCharacter0xf18f',
u'\u2616': 'WhiteShogiPiece',
u'\u6318': 'CjkUnifiedIdeograph6318',
u'\u0f9b': 'TibetanSubjoinedLetterTtha',
u'\ua01a': 'YiSyllableBiet',
u'\u4c9d': 'CjkUnifiedIdeograph4c9d',
u'\ued1c': 'UnicodeCharacter0xed1c',
u'\u899f': 'CjkUnifiedIdeograph899f',
u'\ud6a1': 'HangulSyllableHoeng',
u'\u3e26': 'CjkUnifiedIdeograph3e26',
u'\u7b28': 'CjkUnifiedIdeograph7b28',
u'\u27ab': 'BacktiltedShadowedWhiteRightwardsArrow',
u'\ub82a': 'HangulSyllableRyeonh',
u'\u64ad': 'CjkUnifiedIdeograph64ad',
u'\u852c': 'CjkUnifiedIdeograph852c',
u'\ua1af': 'YiSyllableHlurx',
u'\uc22e': 'HangulSyllableSuj',
u'\ueeb1': 'UnicodeCharacter0xeeb1',
u'\u1338': 'EthiopicSyllableTsa',
u'\u3fbb': 'CjkUnifiedIdeograph3fbb',
u'\u503a': 'CjkUnifiedIdeograph503a',
u'\u7cbd': 'CjkUnifiedIdeograph7cbd',
u'\u9d3c': 'CjkUnifiedIdeograph9d3c',
u'\ub9bf': 'HangulSyllableRis',
u'\uda3e': 'UnicodeCharacter0xda3e',
u'\u86c1': 'CjkUnifiedIdeograph86c1',
u'\uc3c3': 'HangulSyllableSsyed',
u'\u2b48': 'RightwardsArrowAboveReverseAlmostEqualTo',
u'\u684a': 'CjkUnifiedIdeograph684a',
u'\u14cd': 'CanadianSyllabicsNwaa',
u'\ub54c': 'HangulSyllableDdae',
u'\u51cf': 'CjkUnifiedIdeograph51cf',
u'\uf24e': 'UnicodeCharacter0xf24e',
u'\u9ed1': 'CjkUnifiedIdeograph9ed1',
u'\udbd3': 'UnicodeCharacter0xdbd3',
u'\u2cdd': 'CopticSmallLetterOldNubianShima',
u'\u4d5c': 'CjkUnifiedIdeograph4d5c',
u'\u69df': 'CjkUnifiedIdeograph69df',
u'\u8a5e': 'CjkUnifiedIdeograph8a5e',
u'\ub6e1': 'HangulSyllableDdwelt',
u'\ud760': 'HangulSyllableHeum',
u'\uf3e3': 'UnicodeCharacter0xf3e3',
u'\u186a': 'MongolianLetterSibeJa',
u'\u656c': 'CjkUnifiedIdeograph656c',
u'\u01ef': 'LatinSmallLetterEzhWithCaron',
u'\ua26e': 'YiSyllableHit',
u'\u4ef1': 'CjkUnifiedIdeograph4ef1',
u'\uef70': 'UnicodeCharacter0xef70',
u'\u8bf3': 'CjkUnifiedIdeograph8bf3',
u'\uc8f5': 'HangulSyllableJyong',
u'\u307a': 'HiraganaLetterPe',
u'\u7d7c': 'CjkUnifiedIdeograph7d7c',
u'\u19ff': 'KhmerSymbolDappramRoc',
u'\uba7e': 'HangulSyllableMyeolp',
u'\u8780': 'CjkUnifiedIdeograph8780',
u'\u2303': 'UpArrowhead',
u'\uc482': 'HangulSyllableSsweogg',
u'\u6005': 'CjkUnifiedIdeograph6005',
u'\uad07': 'HangulSyllableGwalb',
u'\uea09': 'UnicodeCharacter0xea09',
u'\u158c': 'CanadianSyllabicsWestcreeRo',
u'\u528e': 'CjkUnifiedIdeograph528e',
u'\u9f90': 'CjkUnifiedIdeograph9f90',
u'\u3b13': 'CjkUnifiedIdeograph3b13',
u'\udc92': 'UnicodeCharacter0xdc92',
u'\u7815': 'CjkUnifiedIdeograph7815',
u'\u4517': 'CjkUnifiedIdeograph4517',
u'\u8219': 'CjkUnifiedIdeograph8219',
u'\ucf1b': 'HangulSyllableKeh',
u'\u2d9c': 'UnicodeCharacter0x2d9c',
u'\u6a9e': 'CjkUnifiedIdeograph6a9e',
u'\ub7a0': 'HangulSyllableRael',
u'\uf4a2': 'UnicodeCharacter0xf4a2',
u'\u1025': 'MyanmarLetterU',
u'\u5d27': 'CjkUnifiedIdeograph5d27',
u'\u9a29': 'CjkUnifiedIdeograph9a29',
u'\ue72b': 'UnicodeCharacter0xe72b',
u'\u02ae': 'LatinSmallLetterTurnedHWithFishhook',
u'\u4fb0': 'CjkUnifiedIdeograph4fb0',
u'\u8cb2': 'CjkUnifiedIdeograph8cb2',
u'\u2835': 'BraillePatternDots1356',
u'\uc9b4': 'HangulSyllableJyim',
u'\u7537': 'CjkUnifiedIdeograph7537',
u'\ub239': 'HangulSyllableNweot',
u'\uff3b': 'FullwidthLeftSquareBracket',
u'\u1abe': 'UnicodeCharacter0x1abe',
u'\u67c0': 'CjkUnifiedIdeograph67c0',
u'\ua4c2': 'YiRadicalShop',
u'\ue1c4': 'UnicodeCharacter0xe1c4',
u'\u0d47': 'MalayalamVowelSignEe',
u'\u4a49': 'CjkUnifiedIdeograph4a49',
u'\u974b': 'CjkUnifiedIdeograph974b',
u'\ud44d': 'HangulSyllablePoelt',
u'\u32ce': 'SquareEv',
u'\u7fd0': 'CjkUnifiedIdeograph7fd0',
u'\ubcd2': 'HangulSyllableByeoj',
u'\uf9d4': 'CjkCompatibilityIdeographf9d4',
u'\u2557': 'BoxDrawingsDoubleDownAndLeft',
u'\uc6d6': 'HangulSyllableWeolm',
u'\u6259': 'CjkUnifiedIdeograph6259',
u'\uaf5b': 'HangulSyllableGgwas',
u'\uec5d': 'UnicodeCharacter0xec5d',
u'\u17e0': 'KhmerDigitZero',
u'\u54e2': 'CjkUnifiedIdeograph54e2',
u'\u91e4': 'CjkUnifiedIdeograph91e4',
u'\u3d67': 'CjkUnifiedIdeograph3d67',
u'\udee6': 'UnicodeCharacter0xdee6',
u'\u7a69': 'CjkUnifiedIdeograph7a69',
u'\u476b': 'CjkUnifiedIdeograph476b',
u'\u846d': 'CjkUnifiedIdeograph846d',
u'\uc16f': 'HangulSyllableSyeoh',
u'\u2ff0': 'IdeographicDescriptionCharacterLeftToRight',
u'\u6cf2': 'CjkUnifiedIdeograph6cf2',
u'\ua9f4': 'UnicodeCharacter0xa9f4',
u'\uf6f6': 'UnicodeCharacter0xf6f6',
u'\u1279': 'EthiopicSyllableCu',
u'\u5f7b': 'CjkUnifiedIdeograph5f7b',
u'\u9c7d': 'CjkUnifiedIdeograph9c7d',
u'\ud97f': 'UnicodeCharacter0xd97f',
u'\ua981': 'UnicodeCharacter0xa981',
u'\u4e04': 'CjkUnifiedIdeograph4e04',
u'\u6287': 'CjkUnifiedIdeograph6287',
u'\uc408': 'HangulSyllableSswass',
u'\uf88b': 'UnicodeCharacter0xf88b',
u'\u070a': 'SyriacContraction',
u'\u3b8d': 'CjkUnifiedIdeograph3b8d',
u'\u9d0e': 'CjkUnifiedIdeograph9d0e',
u'\ub191': 'HangulSyllableNot',
u'\u5614': 'CjkUnifiedIdeograph5614',
u'\u0a97': 'GujaratiLetterGa',
u'\uec18': 'UnicodeCharacter0xec18',
u'\u809b': 'CjkUnifiedIdeograph809b',
u'\u2f1a': 'KangxiRadicalCliff',
u'\ua51e': 'VaiSyllableKee',
u'\u59a1': 'CjkUnifiedIdeograph59a1',
u'\udfa5': 'UnicodeCharacter0xdfa5',
u'\u7e24': 'CjkUnifiedIdeograph7e24',
u'\u12a7': 'EthiopicSyllableGlottalWa',
u'\uf428': 'UnicodeCharacter0xf428',
u'\ua8ab': 'SaurashtraLetterYa',
u'\u372a': 'CjkUnifiedIdeograph372a',
u'\u4d2e': 'CjkUnifiedIdeograph4d2e',
u'\u61b1': 'CjkUnifiedIdeograph61b1',
u'\uc332': 'HangulSyllableSsyagg',
u'\ue7b5': 'UnicodeCharacter0xe7b5',
u'\u0634': 'ArabicLetterSheen',
u'\u3ab7': 'CjkUnifiedIdeograph3ab7',
u'\u9c38': 'CjkUnifiedIdeograph9c38',
u'\ub0bb': 'HangulSyllableNaed',
u'\u553e': 'CjkUnifiedIdeograph553e',
u'\u09c1': 'BengaliVowelSignU',
u'\ueb42': 'UnicodeCharacter0xeb42',
u'\u8fc5': 'CjkUnifiedIdeograph8fc5',
u'\u2e44': 'UnicodeCharacter0x2e44',
u'\ua448': 'YiSyllableNjyrx',
u'\u58cb': 'CjkUnifiedIdeograph58cb',
u'\udecf': 'UnicodeCharacter0xdecf',
u'\u7d4e': 'CjkUnifiedIdeograph7d4e',
u'\u11d1': 'HangulJongseongRieulmieumkiyeok',
u'\uf352': 'UnicodeCharacter0xf352',
u'\u97d5': 'CjkUnifiedIdeograph97d5',
u'\u3654': 'CjkUnifiedIdeograph3654',
u'\u4c58': 'CjkUnifiedIdeograph4c58',
u'\u60db': 'CjkUnifiedIdeograph60db',
u'\uc25c': 'HangulSyllableSwels',
u'\ue6df': 'UnicodeCharacter0xe6df',
u'\u055e': 'ArmenianQuestionMark',
u'\u39e1': 'CjkUnifiedIdeograph39e1',
u'\u9b62': 'CjkUnifiedIdeograph9b62',
u'\ubfe5': 'HangulSyllableBbut',
u'\u5468': 'CjkUnifiedIdeograph5468',
u'\u08eb': 'UnicodeCharacter0x8eb',
u'\uea6c': 'UnicodeCharacter0xea6c',
u'\u8eef': 'CjkUnifiedIdeograph8eef',
u'\u2d6e': 'UnicodeCharacter0x2d6e',
u'\ua372': 'YiSyllableChu',
u'\u47f5': 'CjkUnifiedIdeograph47f5',
u'\uddf9': 'UnicodeCharacter0xddf9',
u'\u7c78': 'CjkUnifiedIdeograph7c78',
u'\u10fb': 'GeorgianParagraphSeparator',
u'\uf27c': 'UnicodeCharacter0xf27c',
u'\u96ff': 'CjkUnifiedIdeograph96ff',
u'\u357e': 'CjkUnifiedIdeograph357e',
u'\uac03': 'HangulSyllableGags',
u'\u4b82': 'CjkUnifiedIdeograph4b82',
u'\uc186': 'HangulSyllableSyej',
u'\u6509': 'CjkUnifiedIdeograph6509',
u'\u0488': 'CombiningCyrillicHundredThousandsSign',
u'\ufb0d': 'UnicodeCharacter0xfb0d',
u'\u9a8c': 'CjkUnifiedIdeograph9a8c',
u'\u3e0f': 'CjkUnifiedIdeograph3e0f',
u'\ub413': 'HangulSyllableDwaec',
u'\u5392': 'CjkUnifiedIdeograph5392',
u'\uca17': 'HangulSyllableJjyags',
u'\ue996': 'UnicodeCharacter0xe996',
u'\u0d19': 'MalayalamLetterNga',
u'\u2c98': 'CopticCapitalLetterMi',
u'\u831d': 'CjkUnifiedIdeograph831d',
u'\ua29c': 'YiSyllableZox',
u'\u5c23': 'CjkUnifiedIdeograph5c23',
u'\u7ba2': 'CjkUnifiedIdeograph7ba2',
u'\ud227': 'HangulSyllableTyoc',
u'\uf1a6': 'UnicodeCharacter0xf1a6',
u'\u1529': 'CanadianSyllabicsYii',
u'\u34a8': 'CjkUnifiedIdeograph34a8',
u'\uab2d': 'UnicodeCharacter0xab2d',
u'\u4aac': 'CjkUnifiedIdeograph4aac',
u'\uc0b0': 'HangulSyllableSan',
u'\u6433': 'CjkUnifiedIdeograph6433',
u'\u03b2': 'GreekSmallLetterBeta',
u'\ufa37': 'CjkCompatibilityIdeographfa37',
u'\u99b6': 'CjkUnifiedIdeograph99b6',
u'\u3d39': 'CjkUnifiedIdeograph3d39',
u'\ub33d': 'HangulSyllableDyaenj',
u'\u52bc': 'CjkUnifiedIdeograph52bc',
u'\uc941': 'HangulSyllableJwelt',
u'\ue8c0': 'UnicodeCharacter0xe8c0',
u'\u0c43': 'TeluguVowelSignVocalicR',
u'\u2bc2': 'UnicodeCharacter0x2bc2',
u'\u8247': 'CjkUnifiedIdeograph8247',
u'\ua1c6': 'YiSyllableLuop',
u'\u5b4d': 'CjkUnifiedIdeograph5b4d',
u'\u7acc': 'CjkUnifiedIdeograph7acc',
u'\ud151': 'HangulSyllableTenj',
u'\uf0d0': 'UnicodeCharacter0xf0d0',
u'\u1453': 'CanadianSyllabicsCarrierDee',
u'\u33d2': 'SquareLog',
u'\uaa57': 'ChamDigitSeven',
u'\u49d6': 'CjkUnifiedIdeograph49d6',
u'\ucfda': 'HangulSyllableKyoj',
u'\u635d': 'CjkUnifiedIdeograph635d',
u'\u02dc': 'SmallTilde',
u'\uf961': 'CjkCompatibilityIdeographf961',
u'\u98e0': 'CjkUnifiedIdeograph98e0',
u'\u3c63': 'CjkUnifiedIdeograph3c63',
u'\ub267': 'HangulSyllableNwilh',
u'\u51e6': 'CjkUnifiedIdeograph51e6',
u'\uc86b': 'HangulSyllableJyec',
u'\ud7ea': 'UnicodeCharacter0xd7ea',
u'\u0b6d': 'OriyaDigitSeven',
u'\u2aec': 'DoubleStrokeNotSign',
u'\u8171': 'CjkUnifiedIdeograph8171',
u'\ua0f0': 'YiSyllableVo',
u'\u5a77': 'CjkUnifiedIdeograph5a77',
u'\u79f6': 'CjkUnifiedIdeograph79f6',
u'\ud07b': 'HangulSyllableKeulh',
u'\ufffa': 'InterlinearAnnotationSeparator',
u'\u137d': 'UnicodeCharacter0x137d',
u'\u32fc': 'CircledKatakanaWi',
u'\u2881': 'BraillePatternDots18',
u'\u4900': 'CjkUnifiedIdeograph4900',
u'\u6d83': 'CjkUnifiedIdeograph6d83',
u'\u8a02': 'CjkUnifiedIdeograph8a02',
u'\uae85': 'HangulSyllableGgyag',
u'\ucf04': 'HangulSyllableKen',
u'\ue387': 'UnicodeCharacter0xe387',
u'\u1c0e': 'LepchaLetterPa',
u'\u3091': 'HiraganaLetterWe',
u'\u5110': 'CjkUnifiedIdeograph5110',
u'\u7593': 'CjkUnifiedIdeograph7593',
u'\u9212': 'CjkUnifiedIdeograph9212',
u'\ub695': 'HangulSyllableDdyong',
u'\ud714': 'HangulSyllableHwek',
u'\u8b97': 'CjkUnifiedIdeograph8b97',
u'\ucc99': 'HangulSyllableCeog',
u'\u241e': 'SymbolForRecordSeparator',
u'\u7920': 'CjkUnifiedIdeograph7920',
u'\u1da3': 'ModifierLetterSmallTurnedH',
u'\uba22': 'HangulSyllableMyaenh',
u'\u5ea5': 'CjkUnifiedIdeograph5ea5',
u'\uff24': 'FullwidthLatinCapitalLetterD',
u'\u93a7': 'CjkUnifiedIdeograph93a7',
u'\ud4a9': 'HangulSyllablePweong',
u'\u0130': 'LatinCapitalLetterIWithDotAbove',
u'\u25b3': 'WhiteUppointingTriangle',
u'\u4232': 'CjkUnifiedIdeograph4232',
u'\u66b5': 'CjkUnifiedIdeograph66b5',
u'\u8734': 'CjkUnifiedIdeograph8734',
u'\ubbb7': 'HangulSyllableMyus',
u'\ud836': 'UnicodeCharacter0xd836',
u'\ufcb9': 'ArabicLigatureZahWithMeemInitialForm',
u'\u2940': 'AnticlockwiseClosedCircleArrow',
u'\u6a42': 'CjkUnifiedIdeograph6a42',
u'\u0ec5': 'UnicodeCharacter0xec5',
u'\uaf44': 'HangulSyllableGgok',
u'\u43c7': 'CjkUnifiedIdeograph43c7',
u'\ue046': 'UnicodeCharacter0xe046',
u'\u84c9': 'CjkUnifiedIdeograph84c9',
u'\ud9cb': 'UnicodeCharacter0xd9cb',
u'\u3150': 'HangulLetterAe',
u'\u7252': 'CjkUnifiedIdeograph7252',
u'\u16d5': 'RunicLetterOpenp',
u'\ub754': 'HangulSyllableDdyim',
u'\u6bd7': 'CjkUnifiedIdeograph6bd7',
u'\u8856': 'CjkUnifiedIdeograph8856',
u'\uacd9': 'HangulSyllableGyeng',
u'\ucd58': 'HangulSyllableCwaek',
u'\ue1db': 'UnicodeCharacter0xe1db',
u'\u1a62': 'UnicodeCharacter0x1a62',
u'\u3ee5': 'CjkUnifiedIdeograph3ee5',
u'\u5f64': 'CjkUnifiedIdeograph5f64',
u'\u73e7': 'CjkUnifiedIdeograph73e7',
u'\u9066': 'CjkUnifiedIdeograph9066',
u'\ub4e9': 'HangulSyllableDeult',
u'\ud568': 'HangulSyllableHam',
u'\u89eb': 'CjkUnifiedIdeograph89eb',
u'\ucaed': 'HangulSyllableJjwang',
u'\u2272': 'LessthanOrEquivalentTo',
u'\u6774': 'CjkUnifiedIdeograph6774',
u'\u1bf7': 'UnicodeCharacter0x1bf7',
u'\ub876': 'HangulSyllableRop',
u'\u5cf9': 'CjkUnifiedIdeograph5cf9',
u'\ufd78': 'ArabicLigatureAinWithMeemWithAlefMaksuraFinalForm',
u'\u91fb': 'CjkUnifiedIdeograph91fb',
u'\ud2fd': 'HangulSyllableTilt',
u'\ue801': 'UnicodeCharacter0xe801',
u'\u0f84': 'TibetanMarkHalanta',
u'\u4086': 'CjkUnifiedIdeograph4086',
u'\u8588': 'CjkUnifiedIdeograph8588',
u'\u390b': 'CjkUnifiedIdeograph390b',
u'\uc68a': 'HangulSyllableOebs',
u'\u7a0d': 'CjkUnifiedIdeograph7a0d',
u'\ubf0f': 'HangulSyllableBbyeod',
u'\uf011': 'UnicodeCharacter0xf011',
u'\u1794': 'KhmerLetterBa',
u'\u6896': 'CjkUnifiedIdeograph6896',
u'\uad98': 'HangulSyllableGweom',
u'\uee9a': 'UnicodeCharacter0xee9a',
u'\u021d': 'LatinSmallLetterYogh',
u'\u471f': 'CjkUnifiedIdeograph471f',
u'\u9821': 'CjkUnifiedIdeograph9821',
u'\udd23': 'UnicodeCharacter0xdd23',
u'\u3fa4': 'CjkUnifiedIdeograph3fa4',
u'\u70a6': 'CjkUnifiedIdeograph70a6',
u'\ub5a8': 'HangulSyllableDdeol',
u'\uf6aa': 'UnicodeCharacter0xf6aa',
u'\u2a2d': 'PlusSignInLeftHalfCircle',
u'\ucbac': 'HangulSyllableJjwim',
u'\u6f2f': 'CjkUnifiedIdeograph6f2f',
u'\ua031': 'YiSyllableBur',
u'\ue533': 'UnicodeCharacter0xe533',
u'\u18b6': 'UnicodeCharacter0x18b6',
u'\u5db8': 'CjkUnifiedIdeograph5db8',
u'\u9eba': 'CjkUnifiedIdeograph9eba',
u'\u323d': 'ParenthesizedIdeographEnterprise',
u'\ud3bc': 'HangulSyllablePyeol',
u'\u773f': 'CjkUnifiedIdeograph773f',
u'\u4841': 'CjkUnifiedIdeograph4841',
u'\u8d43': 'CjkUnifiedIdeograph8d43',
u'\uce45': 'HangulSyllableCyilg',
u'\u20c6': 'UnicodeCharacter0x20c6',
u'\u65c8': 'CjkUnifiedIdeograph65c8',
u'\ua6ca': 'UnicodeCharacter0xa6ca',
u'\ufbcc': 'UnicodeCharacter0xfbcc',
u'\u1f4f': 'UnicodeCharacter0x1f4f',
u'\u5051': 'CjkUnifiedIdeograph5051',
u'\u9553': 'CjkUnifiedIdeograph9553',
u'\ud655': 'HangulSyllableHwag',
u'\u0dd8': 'SinhalaVowelSignGaettapilla',
u'\u4eda': 'CjkUnifiedIdeograph4eda',
u'\u83dc': 'CjkUnifiedIdeograph83dc',
u'\u275f': 'UnicodeCharacter0x275f',
u'\uc4de': 'HangulSyllableSsyulm',
u'\u7861': 'CjkUnifiedIdeograph7861',
u'\ubd63': 'HangulSyllableBoeh',
u'\ufe65': 'SmallGreaterthanSign',
u'\u15e8': 'CanadianSyllabicsCarrierPu',
u'\u56ea': 'CjkUnifiedIdeograph56ea',
u'\uabec': 'UnicodeCharacter0xabec',
u'\uecee': 'UnicodeCharacter0xecee',
u'\u4573': 'CjkUnifiedIdeograph4573',
u'\u8675': 'CjkUnifiedIdeograph8675',
u'\udb77': 'UnicodeCharacter0xdb77',
u'\u3df8': 'CjkUnifiedIdeograph3df8',
u'\u7efa': 'CjkUnifiedIdeograph7efa',
u'\ub3fc': 'HangulSyllableDwae',
u'\uf4fe': 'UnicodeCharacter0xf4fe',
u'\uca00': 'HangulSyllableJjael',
u'\uee83': 'UnicodeCharacter0xee83',
u'\u0902': 'DevanagariSignAnusvara',
u'\u2d85': 'EthiopicSyllableBoa',
u'\u8306': 'CjkUnifiedIdeograph8306',
u'\ua789': 'ModifierLetterColon',
u'\u580c': 'CjkUnifiedIdeograph580c',
u'\u7c8f': 'CjkUnifiedIdeograph7c8f',
u'\ud210': 'HangulSyllableTyo',
u'\uf693': 'UnicodeCharacter0xf693',
u'\u1112': 'HangulChoseongHieuh',
u'\u3595': 'CjkUnifiedIdeograph3595',
u'\uab16': 'UnicodeCharacter0xab16',
u'\u4f99': 'CjkUnifiedIdeograph4f99',
u'\uc19d': 'HangulSyllableSob',
u'\u601c': 'CjkUnifiedIdeograph601c',
u'\u049f': 'CyrillicSmallLetterKaWithStroke',
u'\ufa20': 'CjkCompatibilityIdeographfa20',
u'\u9ea3': 'CjkUnifiedIdeograph9ea3',
u'\u3922': 'CjkUnifiedIdeograph3922',
u'\ub326': 'HangulSyllableDyalm',
u'\u57a9': 'CjkUnifiedIdeograph57a9',
u'\ue9ad': 'UnicodeCharacter0xe9ad',
u'\u082c': 'UnicodeCharacter0x82c',
u'\u2caf': 'CopticSmallLetterPsi',
u'\u8230': 'CjkUnifiedIdeograph8230',
u'\ua6b3': 'UnicodeCharacter0xa6b3',
u'\u5b36': 'CjkUnifiedIdeograph5b36',
u'\u7fb9': 'CjkUnifiedIdeograph7fb9',
u'\udd3a': 'UnicodeCharacter0xdd3a',
u'\uf1bd': 'UnicodeCharacter0xf1bd',
u'\u103c': 'MyanmarConsonantSignMedialRa',
u'\u34bf': 'CjkUnifiedIdeograph34bf',
u'\uaa40': 'ChamLetterFinalK',
u'\u4ec3': 'CjkUnifiedIdeograph4ec3',
u'\uc0c7': 'HangulSyllableSah',
u'\u6346': 'CjkUnifiedIdeograph6346',
u'\u07c9': 'NkoDigitNine',
u'\ue54a': 'UnicodeCharacter0xe54a',
u'\u99cd': 'CjkUnifiedIdeograph99cd',
u'\u384c': 'CjkUnifiedIdeograph384c',
u'\ub250': 'HangulSyllableNwess',
u'\u56d3': 'CjkUnifiedIdeograph56d3',
u'\ue8d7': 'UnicodeCharacter0xe8d7',
u'\u0b56': 'OriyaAiLengthMark',
u'\u2fd9': 'UnicodeCharacter0x2fd9',
u'\u8d5a': 'CjkUnifiedIdeograph8d5a',
u'\ua1dd': 'YiSyllableGip',
u'\u5a60': 'CjkUnifiedIdeograph5a60',
u'\u7ee3': 'CjkUnifiedIdeograph7ee3',
u'\udc64': 'UnicodeCharacter0xdc64',
u'\uf0e7': 'UnicodeCharacter0xf0e7',
u'\u1366': 'EthiopicPrefaceColon',
u'\u37e9': 'CjkUnifiedIdeograph37e9',
u'\u956a': 'CjkUnifiedIdeograph956a',
u'\u49ed': 'CjkUnifiedIdeograph49ed',
u'\uc3f1': 'HangulSyllableSsot',
u'\u6270': 'CjkUnifiedIdeograph6270',
u'\u06f3': 'ExtendedArabicindicDigitThree',
u'\ue474': 'UnicodeCharacter0xe474',
u'\u98f7': 'CjkUnifiedIdeograph98f7',
u'\u3b76': 'CjkUnifiedIdeograph3b76',
u'\ubd7a': 'HangulSyllableByoj',
u'\u51fd': 'CjkUnifiedIdeograph51fd',
u'\u6b01': 'CjkUnifiedIdeograph6b01',
u'\u0a80': 'UnicodeCharacter0xa80',
u'\ued05': 'UnicodeCharacter0xed05',
u'\u8c84': 'CjkUnifiedIdeograph8c84',
u'\u2007': 'FigureSpace',
u'\uba0b': 'HangulSyllableMyalb',
u'\u458a': 'CjkUnifiedIdeograph458a',
u'\udf8e': 'UnicodeCharacter0xdf8e',
u'\u7311': 'CjkUnifiedIdeograph7311',
u'\u1290': 'EthiopicSyllableNa',
u'\uf515': 'UnicodeCharacter0xf515',
u'\u9494': 'CjkUnifiedIdeograph9494',
u'\u421b': 'CjkUnifiedIdeograph421b',
u'\u6d9a': 'CjkUnifiedIdeograph6d9a',
u'\uc41f': 'HangulSyllableSswaelh',
u'\ue79e': 'UnicodeCharacter0xe79e',
u'\u1b21': 'BalineseLetterNaRambat',
u'\u3aa0': 'CjkUnifiedIdeograph3aa0',
u'\u9d25': 'CjkUnifiedIdeograph9d25',
u'\ubca4': 'HangulSyllableBen',
u'\u6a2b': 'CjkUnifiedIdeograph6a2b',
u'\u75aa': 'CjkUnifiedIdeograph75aa',
u'\uec2f': 'UnicodeCharacter0xec2f',
u'\u8fae': 'CjkUnifiedIdeograph8fae',
u'\u2331': 'DimensionOrigin',
u'\ua535': 'VaiSyllableDhi',
u'\u44b4': 'CjkUnifiedIdeograph44b4',
u'\udeb8': 'UnicodeCharacter0xdeb8',
u'\u723b': 'CjkUnifiedIdeograph723b',
u'\u1dba': 'ModifierLetterSmallTurnedV',
u'\uf43f': 'UnicodeCharacter0xf43f',
u'\u97be': 'CjkUnifiedIdeograph97be',
u'\u4d45': 'CjkUnifiedIdeograph4d45',
u'\u6cc4': 'CjkUnifiedIdeograph6cc4',
u'\uc749': 'HangulSyllableEult',
u'\ue6c8': 'UnicodeCharacter0xe6c8',
u'\u1a4b': 'UnicodeCharacter0x1a4b',
u'\u25ca': 'Lozenge',
u'\u9c4f': 'CjkUnifiedIdeograph9c4f',
u'\ubfce': 'HangulSyllableBbugg',
u'\u5555': 'CjkUnifiedIdeograph5555',
u'\u74d4': 'CjkUnifiedIdeograph74d4',
u'\uef59': 'UnicodeCharacter0xef59',
u'\u8ed8': 'CjkUnifiedIdeograph8ed8',
u'\u225b': 'StarEquals',
u'\ua45f': 'YiSyllableXi',
u'\u47de': 'CjkUnifiedIdeograph47de',
u'\ud9e2': 'UnicodeCharacter0xd9e2',
u'\u7d65': 'CjkUnifiedIdeograph7d65',
u'\u1ce4': 'UnicodeCharacter0x1ce4',
u'\uf769': 'UnicodeCharacter0xf769',
u'\u96e8': 'CjkUnifiedIdeograph96e8',
u'\u4c6f': 'CjkUnifiedIdeograph4c6f',
u'\u6fee': 'CjkUnifiedIdeograph6fee',
u'\uc673': 'HangulSyllableWaec',
u'\ue1f2': 'UnicodeCharacter0xe1f2',
u'\u0575': 'ArmenianSmallLetterYi',
u'\u24f4': 'NegativeCircledNumberTwenty',
u'\u9f79': 'CjkUnifiedIdeograph9f79',
u'\ubef8': 'HangulSyllableBbels',
u'\u547f': 'CjkUnifiedIdeograph547f',
u'\u77fe': 'CjkUnifiedIdeograph77fe',
u'\u0206': 'LatinCapitalLetterEWithInvertedBreve',
u'\u2689': 'BlackCircleWithTwoWhiteDots',
u'\u4708': 'CjkUnifiedIdeograph4708',
u'\u7b8b': 'CjkUnifiedIdeograph7b8b',
u'\u840a': 'CjkUnifiedIdeograph840a',
u'\ub88d': 'HangulSyllableRwang',
u'\ud90c': 'UnicodeCharacter0xd90c',
u'\ufd8f': 'ArabicLigatureMeemWithKhahWithMeemInitialForm',
u'\u2a16': 'QuaternionIntegralOperator',
u'\u6f18': 'CjkUnifiedIdeograph6f18',
u'\u039b': 'GreekCapitalLetterLamda',
u'\uac1a': 'HangulSyllableGap',
u'\u409d': 'CjkUnifiedIdeograph409d',
u'\ue11c': 'UnicodeCharacter0xe11c',
u'\u859f': 'CjkUnifiedIdeograph859f',
u'\udaa1': 'UnicodeCharacter0xdaa1',
u'\u3226': 'ParenthesizedIdeographSeven',
u'\u7728': 'CjkUnifiedIdeograph7728',
u'\u2bab': 'UnicodeCharacter0x2bab',
u'\ub42a': 'HangulSyllableDoebs',
u'\u68ad': 'CjkUnifiedIdeograph68ad',
u'\u892c': 'CjkUnifiedIdeograph892c',
u'\uadaf': 'HangulSyllableGwelb',
u'\uce2e': 'HangulSyllableCeulp',
u'\ue2b1': 'UnicodeCharacter0xe2b1',
u'\u1f38': 'GreekCapitalLetterIotaWithPsili',
u'\u33bb': 'SquareNw',
u'\u5c3a': 'CjkUnifiedIdeograph5c3a',
u'\u70bd': 'CjkUnifiedIdeograph70bd',
u'\u913c': 'CjkUnifiedIdeograph913c',
u'\ub5bf': 'HangulSyllableDdegs',
u'\ud63e': 'HangulSyllableHonh',
u'\u8ac1': 'CjkUnifiedIdeograph8ac1',
u'\ucfc3': 'HangulSyllableKoeh',
u'\u2748': 'HeavySparkle',
u'\u644a': 'CjkUnifiedIdeograph644a',
u'\u18cd': 'UnicodeCharacter0x18cd',
u'\ub94c': 'HangulSyllableRwim',
u'\u5dcf': 'CjkUnifiedIdeograph5dcf',
u'\ufe4e': 'CentrelineLowLine',
u'\u92d1': 'CjkUnifiedIdeograph92d1',
u'\ud7d3': 'UnicodeCharacter0xd7d3',
u'\u0c5a': 'UnicodeCharacter0xc5a',
u'\u20dd': 'CombiningEnclosingCircle',
u'\u415c': 'CjkUnifiedIdeograph415c',
u'\u65df': 'CjkUnifiedIdeograph65df',
u'\u865e': 'CjkUnifiedIdeograph865e',
u'\ubae1': 'HangulSyllableMwaeg',
u'\udb60': 'UnicodeCharacter0xdb60',
u'\uffe3': 'FullwidthMacron',
u'\u146a': 'CanadianSyllabicsTta',
u'\u696c': 'CjkUnifiedIdeograph696c',
u'\u0def': 'UnicodeCharacter0xdef',
u'\uae6e': 'HangulSyllableGgaenh',
u'\u42f1': 'CjkUnifiedIdeograph42f1',
u'\ue370': 'UnicodeCharacter0xe370',
u'\u87f3': 'CjkUnifiedIdeograph87f3',
u'\uc4f5': 'HangulSyllableSseunj',
u'\u3c7a': 'CjkUnifiedIdeograph3c7a',
u'\u717c': 'CjkUnifiedIdeograph717c',
u'\u15ff': 'CanadianSyllabicsCarrierKki',
u'\ub67e': 'HangulSyllableDdoep',
u'\u8b80': 'CjkUnifiedIdeograph8b80',
u'\u2f03': 'KangxiRadicalSlash',
u'\uc882': 'HangulSyllableJobs',
u'\u6c05': 'CjkUnifiedIdeograph6c05',
u'\ua107': 'YiSyllableDat',
u'\ue609': 'UnicodeCharacter0xe609',
u'\u198c': 'NewTaiLueLetterLowSa',
u'\u5e8e': 'CjkUnifiedIdeograph5e8e',
u'\u9390': 'CjkUnifiedIdeograph9390',
u'\u3713': 'CjkUnifiedIdeograph3713',
u'\ud092': 'HangulSyllableKyilm',
u'\u7415': 'CjkUnifiedIdeograph7415',
u'\u4917': 'CjkUnifiedIdeograph4917',
u'\u8e19': 'CjkUnifiedIdeograph8e19',
u'\uc31b': 'HangulSyllableSsaed',
u'\u219c': 'LeftwardsWaveArrow',
u'\u669e': 'CjkUnifiedIdeograph669e',
u'\ubba0': 'HangulSyllableMwik',
u'\uf8a2': 'UnicodeCharacter0xf8a2',
u'\u1c25': 'LepchaSubjoinedLetterRa',
u'\u5127': 'CjkUnifiedIdeograph5127',
u'\u9629': 'CjkUnifiedIdeograph9629',
u'\ueb2b': 'UnicodeCharacter0xeb2b',
u'\u0eae': 'LaoLetterHoTam',
u'\u43b0': 'CjkUnifiedIdeograph43b0',
u'\u80b2': 'CjkUnifiedIdeograph80b2',
u'\u2435': 'UnicodeCharacter0x2435',
u'\uc5b4': 'HangulSyllableEo',
u'\u7937': 'CjkUnifiedIdeograph7937',
u'\ube39': 'HangulSyllableByib',
u'\uf33b': 'UnicodeCharacter0xf33b',
u'\u16be': 'RunicLetterNaudizNydNaudN',
u'\u6bc0': 'CjkUnifiedIdeograph6bc0',
u'\ua8c2': 'SaurashtraVowelSignOo',
u'\uedc4': 'UnicodeCharacter0xedc4',
u'\u0147': 'LatinCapitalLetterNWithCaron',
u'\u4649': 'CjkUnifiedIdeograph4649',
u'\u9b4b': 'CjkUnifiedIdeograph9b4b',
u'\ud84d': 'UnicodeCharacter0xd84d',
u'\u3ece': 'CjkUnifiedIdeograph3ece',
u'\u73d0': 'CjkUnifiedIdeograph73d0',
u'\ub0d2': 'HangulSyllableNyagg',
u'\uf5d4': 'UnicodeCharacter0xf5d4',
u'\u2957': 'RightwardsHarpoonWithBarbDownToBar',
u'\ucad6': 'HangulSyllableJjop',
u'\u6e59': 'CjkUnifiedIdeograph6e59',
u'\ua35b': 'YiSyllableZhyt',
u'\ue05d': 'UnicodeCharacter0xe05d',
u'\u1be0': 'UnicodeCharacter0x1be0',
u'\u58e2': 'CjkUnifiedIdeograph58e2',
u'\u9de4': 'CjkUnifiedIdeograph9de4',
u'\u3167': 'HangulLetterNieunsios',
u'\ud2e6': 'HangulSyllableTyibs',
u'\u7669': 'CjkUnifiedIdeograph7669',
u'\u4b6b': 'CjkUnifiedIdeograph4b6b',
u'\u886d': 'CjkUnifiedIdeograph886d',
u'\ucd6f': 'HangulSyllableCoes',
u'\u23f0': 'UnicodeCharacter0x23f0',
u'\u60f2': 'CjkUnifiedIdeograph60f2',
u'\ua5f4': 'VaiSyllableThe',
u'\ufaf6': 'UnicodeCharacter0xfaf6',
u'\u1e79': 'LatinSmallLetterUWithTildeAndAcute',
u'\u537b': 'CjkUnifiedIdeograph537b',
u'\u907d': 'CjkUnifiedIdeograph907d',
u'\ud57f': 'HangulSyllableHaelb',
u'\uad81': 'HangulSyllableGung',
u'\u4a04': 'CjkUnifiedIdeograph4a04',
u'\u6687': 'CjkUnifiedIdeograph6687',
u'\uc008': 'HangulSyllableBbwen',
u'\ufc8b': 'ArabicLigatureNoonWithZainFinalForm',
u'\u030a': 'CombiningRingAbove',
u'\u3f8d': 'CjkUnifiedIdeograph3f8d',
u'\u990e': 'CjkUnifiedIdeograph990e',
u'\ub591': 'HangulSyllableDdyaelt',
u'\u5214': 'CjkUnifiedIdeograph5214',
u'\u0e97': 'LaoLetterThoTam',
u'\ue818': 'UnicodeCharacter0xe818',
u'\u849b': 'CjkUnifiedIdeograph849b',
u'\u2b1a': 'DottedSquare',
u'\ua11e': 'YiSyllableTiex',
u'\u5da1': 'CjkUnifiedIdeograph5da1',
u'\udba5': 'UnicodeCharacter0xdba5',
u'\u7a24': 'CjkUnifiedIdeograph7a24',
u'\u16a7': 'RunicLetterEth',
u'\uf028': 'UnicodeCharacter0xf028',
u'\uacab': 'HangulSyllableGyeogs',
u'\u332a': 'SquareHaitu',
u'\u492e': 'CjkUnifiedIdeograph492e',
u'\u65b1': 'CjkUnifiedIdeograph65b1',
u'\uc732': 'HangulSyllableYubs',
u'\ue3b5': 'UnicodeCharacter0xe3b5',
u'\u0234': 'LatinSmallLetterLWithCurl',
u'\u3eb7': 'CjkUnifiedIdeograph3eb7',
u'\u9838': 'CjkUnifiedIdeograph9838',
u'\ub4bb': 'HangulSyllableDwic',
u'\u513e': 'CjkUnifiedIdeograph513e',
u'\u0dc1': 'SinhalaLetterTaalujaSayanna',
u'\uef42': 'UnicodeCharacter0xef42',
u'\u8bc5': 'CjkUnifiedIdeograph8bc5',
u'\u2a44': 'IntersectionWithLogicalAnd',
u'\ua048': 'YiSyllablePo',
u'\u5ccb': 'CjkUnifiedIdeograph5ccb',
u'\udacf': 'UnicodeCharacter0xdacf',
u'\u794e': 'CjkUnifiedIdeograph794e',
u'\u15d1': 'CanadianSyllabicsCarrierWo',
u'\uf752': 'UnicodeCharacter0xf752',
u'\u93d5': 'CjkUnifiedIdeograph93d5',
u'\u3254': 'CircledNumberTwentyFour',
u'\u4858': 'CjkUnifiedIdeograph4858',
u'\u64db': 'CjkUnifiedIdeograph64db',
u'\uc65c': 'HangulSyllableWae',
u'\ue2df': 'UnicodeCharacter0xe2df',
u'\u015e': 'LatinCapitalLetterSWithCedilla',
u'\u3de1': 'CjkUnifiedIdeograph3de1',
u'\u9f62': 'CjkUnifiedIdeograph9f62',
u'\ubbe5': 'HangulSyllableMyilg',
u'\u5068': 'CjkUnifiedIdeograph5068',
u'\u0ceb': 'KannadaDigitFive',
u'\uee6c': 'UnicodeCharacter0xee6c',
u'\u8aef': 'CjkUnifiedIdeograph8aef',
u'\u296e': 'UpwardsHarpoonWithBarbLeftBesideDownwardsHarpoonWithBarbRight',
u'\ua772': 'LatinSmallLetterLum',
u'\u43f5': 'CjkUnifiedIdeograph43f5',
u'\ud9f9': 'UnicodeCharacter0xd9f9',
u'\u7878': 'CjkUnifiedIdeograph7878',
u'\u14fb': 'CanadianSyllabicsWestcreeSwii',
u'\uf67c': 'UnicodeCharacter0xf67c',
u'\u92ff': 'CjkUnifiedIdeograph92ff',
u'\u317e': 'HangulLetterSioscieuc',
u'\ua803': 'SylotiNagriLetterU',
u'\u4f82': 'CjkUnifiedIdeograph4f82',
u'\uc586': 'HangulSyllableYalm',
u'\u6109': 'CjkUnifiedIdeograph6109',
u'\x88': 'UnicodeCharacter0x88',
u'\uff0d': 'FullwidthHyphenminus',
u'\u9e8c': 'CjkUnifiedIdeograph9e8c',
u'\u3a0f': 'CjkUnifiedIdeograph3a0f',
u'\ub013': 'HangulSyllableGgwid',
u'\u5792': 'CjkUnifiedIdeograph5792',
u'\uce17': 'HangulSyllableCyus',
u'\ued96': 'UnicodeCharacter0xed96',
u'\u0919': 'DevanagariLetterNga',
u'\u2898': 'BraillePatternDots458',
u'\u871d': 'CjkUnifiedIdeograph871d',
u'\ua69c': 'UnicodeCharacter0xa69c',
u'\u5823': 'CjkUnifiedIdeograph5823',
u'\u7fa2': 'CjkUnifiedIdeograph7fa2',
u'\ud627': 'HangulSyllableHyelb',
u'\uf5a6': 'UnicodeCharacter0xf5a6',
u'\u1129': 'HangulChoseongPieupthieuth',
u'\u30a8': 'KatakanaLetterE',
u'\uaf2d': 'HangulSyllableGgog',
u'\u4eac': 'CjkUnifiedIdeograph4eac',
u'\uc4b0': 'HangulSyllableSswess',
u'\u6033': 'CjkUnifiedIdeograph6033',
u'\u07b2': 'UnicodeCharacter0x7b2',
u'\ufe37': 'PresentationFormForVerticalLeftCurlyBracket',
u'\u9db6': 'CjkUnifiedIdeograph9db6',
u'\u3939': 'CjkUnifiedIdeograph3939',
u'\ub73d': 'HangulSyllableDdeung',
u'\u56bc': 'CjkUnifiedIdeograph56bc',
u'\ucd41': 'HangulSyllableCwaeg',
u'\uecc0': 'UnicodeCharacter0xecc0',
u'\u0843': 'UnicodeCharacter0x843',
u'\u2fc2': 'KangxiRadicalFish',
u'\u8647': 'CjkUnifiedIdeograph8647',
u'\ua5c6': 'VaiSyllableMgbo',
u'\u5f4d': 'CjkUnifiedIdeograph5f4d',
u'\u7ecc': 'CjkUnifiedIdeograph7ecc',
u'\ud551': 'HangulSyllablePing',
u'\uf4d0': 'UnicodeCharacter0xf4d0',
u'\u1053': 'MyanmarLetterVocalicRr',
u'\u37d2': 'CjkUnifiedIdeograph37d2',
u'\uae57': 'HangulSyllableGgalb',
u'\u4dd6': 'HexagramForSplittingApart',
u'\ucbda': 'HangulSyllableJjeunh',
u'\u675d': 'CjkUnifiedIdeograph675d',
u'\u06dc': 'ArabicSmallHighSeen',
u'\ufd61': 'ArabicLigatureSeenWithMeemWithJeemInitialForm',
u'\u9ce0': 'CjkUnifiedIdeograph9ce0',
u'\u3863': 'CjkUnifiedIdeograph3863',
u'\ub667': 'HangulSyllableDdoegs',
u'\u55e6': 'CjkUnifiedIdeograph55e6',
u'\ucc6b': 'HangulSyllableCyalb',
u'\ud3ea': 'HangulSyllablePyep',
u'\u0f6d': 'UnicodeCharacter0xf6d',
u'\u2eec': 'CjkRadicalCsimplifiedEven',
u'\u8571': 'CjkUnifiedIdeograph8571',
u'\ua4f0': 'UnicodeCharacter0xa4f0',
u'\u5e77': 'CjkUnifiedIdeograph5e77',
u'\u7df6': 'CjkUnifiedIdeograph7df6',
u'\ud47b': 'HangulSyllablePugs',
u'\ufbfa': 'ArabicLigatureUighurKirghizYehWithHamzaAboveWithAlefMaksuraFinalForm',
u'\u177d': 'UnicodeCharacter0x177d',
u'\u36fc': 'CjkUnifiedIdeograph36fc',
u'\u2c81': 'CopticSmallLetterAlfa',
u'\u4d00': 'CjkUnifiedIdeograph4d00',
u'\u6983': 'CjkUnifiedIdeograph6983',
u'\u8e02': 'CjkUnifiedIdeograph8e02',
u'\uaa85': 'UnicodeCharacter0xaa85',
u'\ucb04': 'HangulSyllableJjwaem',
u'\ue787': 'UnicodeCharacter0xe787',
u'\u180e': 'MongolianVowelSeparator',
u'\u3491': 'CjkUnifiedIdeograph3491',
u'\u5510': 'CjkUnifiedIdeograph5510',
u'\u7193': 'CjkUnifiedIdeograph7193',
u'\u9612': 'CjkUnifiedIdeograph9612',
u'\ub295': 'HangulSyllableNeunj',
u'\ud314': 'HangulSyllablePal',
u'\u8f97': 'CjkUnifiedIdeograph8f97',
u'\uc899': 'HangulSyllableJwalt',
u'\u201e': 'DoubleLow9QuotationMark',
u'\u7d20': 'CjkUnifiedIdeograph7d20',
u'\u19a3': 'NewTaiLueLetterLowHa',
u'\ube22': 'HangulSyllableBeuj',
u'\u5aa5': 'CjkUnifiedIdeograph5aa5',
u'\ufb24': 'HebrewLetterWideKaf',
u'\u97a7': 'CjkUnifiedIdeograph97a7',
u'\ud0a9': 'HangulSyllableKinj',
u'\u0530': 'UnicodeCharacter0x530',
u'\u21b3': 'DownwardsArrowWithTipRightwards',
u'\u4632': 'CjkUnifiedIdeograph4632',
u'\u62b5': 'CjkUnifiedIdeograph62b5',
u'\u8334': 'CjkUnifiedIdeograph8334',
u'\ubfb7': 'HangulSyllableBbyod',
u'\udc36': 'UnicodeCharacter0xdc36',
u'\uf8b9': 'UnicodeCharacter0xf8b9',
u'\u2d40': 'TifinaghLetterYah',
u'\u6e42': 'CjkUnifiedIdeograph6e42',
u'\u0ac5': 'GujaratiVowelSignCandraE',
u'\uab44': 'UnicodeCharacter0xab44',
u'\u47c7': 'CjkUnifiedIdeograph47c7',
u'\ue446': 'UnicodeCharacter0xe446',
u'\u80c9': 'CjkUnifiedIdeograph80c9',
u'\uddcb': 'UnicodeCharacter0xddcb',
u'\u3550': 'CjkUnifiedIdeograph3550',
u'\u7652': 'CjkUnifiedIdeograph7652',
u'\u12d5': 'EthiopicSyllablePharyngealE',
u'\ub354': 'HangulSyllableDeo',
u'\u6fd7': 'CjkUnifiedIdeograph6fd7',
u'\u8c56': 'CjkUnifiedIdeograph8c56',
u'\ua8d9': 'SaurashtraDigitNine',
u'\uc958': 'HangulSyllableJwil',
u'\ue5db': 'UnicodeCharacter0xe5db',
u'\u1e62': 'LatinCapitalLetterSWithDotBelow',
u'\u3ae5': 'CjkUnifiedIdeograph3ae5',
u'\u5b64': 'CjkUnifiedIdeograph5b64',
u'\u77e7': 'CjkUnifiedIdeograph77e7',
u'\u9466': 'CjkUnifiedIdeograph9466',
u'\ub0e9': 'HangulSyllableNyat',
u'\ud168': 'HangulSyllableTyeo',
u'\u8deb': 'CjkUnifiedIdeograph8deb',
u'\uceed': 'HangulSyllableKeolg',
u'\u2672': 'UniversalRecyclingSymbol',
u'\u6374': 'CjkUnifiedIdeograph6374',
u'\u1ff7': 'GreekSmallLetterOmegaWithPerispomeniAndYpogegrammeni',
u'\ubc76': 'HangulSyllableByaelp',
u'\u58f9': 'CjkUnifiedIdeograph58f9',
u'\uf978': 'CjkCompatibilityIdeographf978',
u'\u95fb': 'CjkUnifiedIdeograph95fb',
u'\ud6fd': 'HangulSyllableHweg',
u'\uec01': 'UnicodeCharacter0xec01',
u'\u0b84': 'UnicodeCharacter0xb84',
u'\u4486': 'CjkUnifiedIdeograph4486',
u'\u8188': 'CjkUnifiedIdeograph8188',
u'\u3d0b': 'CjkUnifiedIdeograph3d0b',
u'\uc28a': 'HangulSyllableSyugg',
u'\u7e0d': 'CjkUnifiedIdeograph7e0d',
u'\ubb0f': 'HangulSyllableMoes',
u'\uf411': 'UnicodeCharacter0xf411',
u'\u1394': 'EthiopicTonalMarkDifat',
u'\u6c96': 'CjkUnifiedIdeograph6c96',
u'\ua998': 'UnicodeCharacter0xa998',
u'\uea9a': 'UnicodeCharacter0xea9a',
u'\u061d': 'UnicodeCharacter0x61d',
u'\u431f': 'CjkUnifiedIdeograph431f',
u'\u9c21': 'CjkUnifiedIdeograph9c21',
u'\ud923': 'UnicodeCharacter0xd923',
u'\u3ba4': 'CjkUnifiedIdeograph3ba4',
u'\u74a6': 'CjkUnifiedIdeograph74a6',
u'\ub1a8': 'HangulSyllableNwass',
u'\uf2aa': 'UnicodeCharacter0xf2aa',
u'\u2e2d': 'FiveDotMark',
u'\ucfac': 'HangulSyllableKoen',
u'\u6b2f': 'CjkUnifiedIdeograph6b2f',
u'\ua431': 'YiSyllableNjit',
u'\ue133': 'UnicodeCharacter0xe133',
u'\u1cb6': 'UnicodeCharacter0x1cb6',
u'\u59b8': 'CjkUnifiedIdeograph59b8',
u'\u9aba': 'CjkUnifiedIdeograph9aba',
u'\u363d': 'CjkUnifiedIdeograph363d',
u'\ud7bc': 'UnicodeCharacter0xd7bc',
u'\u733f': 'CjkUnifiedIdeograph733f',
u'\u4c41': 'CjkUnifiedIdeograph4c41',
u'\u8943': 'CjkUnifiedIdeograph8943',
u'\uca45': 'HangulSyllableJjyaeng',
u'\u24c6': 'CircledLatinCapitalLetterQ',
u'\u61c8': 'CjkUnifiedIdeograph61c8',
u'\ua2ca': 'YiSyllableCyt',
u'\uffcc': 'HalfwidthHangulLetterO',
u'\u1b4f': 'UnicodeCharacter0x1b4f',
u'\u5451': 'CjkUnifiedIdeograph5451',
u'\u9153': 'CjkUnifiedIdeograph9153',
u'\ud255': 'HangulSyllableTweolt',
u'\u09d8': 'UnicodeCharacter0x9d8',
u'\u4ada': 'CjkUnifiedIdeograph4ada',
u'\u87dc': 'CjkUnifiedIdeograph87dc',
u'\u235f': 'AplFunctionalSymbolCircleStar',
u'\uc0de': 'HangulSyllableSaej',
u'\u7c61': 'CjkUnifiedIdeograph7c61',
u'\ub963': 'HangulSyllableRyulb',
u'\ufa65': 'CjkCompatibilityIdeographfa65',
u'\u11e8': 'HangulJongseongSiostikeut',
u'\u52ea': 'CjkUnifiedIdeograph52ea',
u'\uafec': 'HangulSyllableGgweok',
u'\ue8ee': 'UnicodeCharacter0xe8ee',
u'\u0471': 'CyrillicSmallLetterPsi',
u'\u4173': 'CjkUnifiedIdeograph4173',
u'\u8275': 'CjkUnifiedIdeograph8275',
u'\udf77': 'UnicodeCharacter0xdf77',
u'\u39f8': 'CjkUnifiedIdeograph39f8',
u'\u7afa': 'CjkUnifiedIdeograph7afa',
u'\ub7fc': 'HangulSyllableReom',
u'\uf0fe': 'UnicodeCharacter0xf0fe',
u'\uce00': 'HangulSyllableCwik',
u'\uea83': 'UnicodeCharacter0xea83',
u'\u0d02': 'MalayalamSignAnusvara',
u'\u2985': 'LeftWhiteParenthesis',
u'\u8706': 'CjkUnifiedIdeograph8706',
u'\ua389': 'YiSyllableRrux',
u'\u5c0c': 'CjkUnifiedIdeograph5c0c',
u'\u788f': 'CjkUnifiedIdeograph788f',
u'\ud610': 'HangulSyllableHyeom',
u'\uf293': 'UnicodeCharacter0xf293',
u'\u1512': 'CanadianSyllabicsShii',
u'\u3195': 'IdeographicAnnotationFourMark',
u'\uaf16': 'HangulSyllableGgyenh',
u'\u4b99': 'CjkUnifiedIdeograph4b99',
u'\uc59d': 'HangulSyllableYaenj',
u'\u641c': 'CjkUnifiedIdeograph641c',
u'\x9f': 'UnicodeCharacter0x9f',
u'\ufe20': 'CombiningLigatureLeftHalf',
u'\u9aa3': 'CjkUnifiedIdeograph9aa3',
u'\u3d22': 'CjkUnifiedIdeograph3d22',
u'\ub726': 'HangulSyllableDdyup',
u'\u53a9': 'CjkUnifiedIdeograph53a9',
u'\uedad': 'UnicodeCharacter0xedad',
u'\u0c2c': 'TeluguLetterBa',
u'\u28af': 'BraillePatternDots123468',
u'\u8630': 'CjkUnifiedIdeograph8630',
u'\ua2b3': 'YiSyllableCiex',
u'\u5f36': 'CjkUnifiedIdeograph5f36',
u'\u7bb9': 'CjkUnifiedIdeograph7bb9',
u'\ud93a': 'UnicodeCharacter0xd93a',
u'\uf5bd': 'UnicodeCharacter0xf5bd',
u'\u143c': 'CanadianSyllabicsPwi',
u'\u30bf': 'KatakanaLetterTa',
u'\uae40': 'HangulSyllableGim',
u'\u4ac3': 'CjkUnifiedIdeograph4ac3',
u'\uc4c7': 'HangulSyllableSswilh',
u'\u6746': 'CjkUnifiedIdeograph6746',
u'\u03c9': 'GreekSmallLetterOmega',
u'\ue14a': 'UnicodeCharacter0xe14a',
u'\u9dcd': 'CjkUnifiedIdeograph9dcd',
u'\u3c4c': 'CjkUnifiedIdeograph3c4c',
u'\ub650': 'HangulSyllableDdwael',
u'\u52d3': 'CjkUnifiedIdeograph52d3',
u'\uecd7': 'UnicodeCharacter0xecd7',
u'\u0f56': 'TibetanLetterBa',
u'\u2bd9': 'UnicodeCharacter0x2bd9',
u'\u895a': 'CjkUnifiedIdeograph895a',
u'\ua5dd': 'VaiSyllableGo',
u'\u5e60': 'CjkUnifiedIdeograph5e60',
u'\u7ae3': 'CjkUnifiedIdeograph7ae3',
u'\ud864': 'UnicodeCharacter0xd864',
u'\uf4e7': 'UnicodeCharacter0xf4e7',
u'\u1766': 'TagbanwaLetterTa',
u'\u33e9': 'IdeographicTelegraphSymbolForDayTen',
u'\u916a': 'CjkUnifiedIdeograph916a',
u'\u4ded': 'HexagramForPushingUpward',
u'\uc7f1': 'HangulSyllableJyaelt',
u'\u6670': 'CjkUnifiedIdeograph6670',
u'\u02f3': 'ModifierLetterLowRing',
u'\ue074': 'UnicodeCharacter0xe074',
u'\u9cf7': 'CjkUnifiedIdeograph9cf7',
u'\u3f76': 'CjkUnifiedIdeograph3f76',
u'\ub97a': 'HangulSyllableReunh',
u'\u55fd': 'CjkUnifiedIdeograph55fd',
u'\u6f01': 'CjkUnifiedIdeograph6f01',
u'\u0e80': 'UnicodeCharacter0xe80',
u'\ue905': 'UnicodeCharacter0xe905',
u'\u8884': 'CjkUnifiedIdeograph8884',
u'\u2407': 'SymbolForBell',
u'\ube0b': 'HangulSyllableByuh',
u'\u418a': 'CjkUnifiedIdeograph418a',
u'\udb8e': 'UnicodeCharacter0xdb8e',
u'\u7711': 'CjkUnifiedIdeograph7711',
u'\u1690': 'OghamLetterAilm',
u'\uf115': 'UnicodeCharacter0xf115',
u'\u9094': 'CjkUnifiedIdeograph9094',
u'\u461b': 'CjkUnifiedIdeograph461b',
u'\u699a': 'CjkUnifiedIdeograph699a',
u'\uc01f': 'HangulSyllableBbweh',
u'\ue39e': 'UnicodeCharacter0xe39e',
u'\u1f21': 'GreekSmallLetterEtaWithDasia',
u'\u3ea0': 'CjkUnifiedIdeograph3ea0',
u'\u9925': 'CjkUnifiedIdeograph9925',
u'\ub8a4': 'HangulSyllableRwaem',
u'\u6e2b': 'CjkUnifiedIdeograph6e2b',
u'\u71aa': 'CjkUnifiedIdeograph71aa',
u'\ue82f': 'UnicodeCharacter0xe82f',
u'\u8bae': 'CjkUnifiedIdeograph8bae',
u'\u2731': 'HeavyAsterisk',
u'\ua135': 'YiSyllableTur',
u'\u40b4': 'CjkUnifiedIdeograph40b4',
u'\udab8': 'UnicodeCharacter0xdab8',
u'\u763b': 'CjkUnifiedIdeograph763b',
u'\u19ba': 'NewTaiLueVowelSignAy',
u'\uf03f': 'UnicodeCharacter0xf03f',
u'\u93be': 'CjkUnifiedIdeograph93be',
u'\u4945': 'CjkUnifiedIdeograph4945',
u'\u68c4': 'CjkUnifiedIdeograph68c4',
u'\uc349': 'HangulSyllableSsyat',
u'\ue2c8': 'UnicodeCharacter0xe2c8',
u'\u1e4b': 'LatinSmallLetterNWithCircumflexBelow',
u'\u21ca': 'DownwardsPairedArrows',
u'\u984f': 'CjkUnifiedIdeograph984f',
u'\ubbce': 'HangulSyllableMeulp',
u'\u5155': 'CjkUnifiedIdeograph5155',
u'\u70d4': 'CjkUnifiedIdeograph70d4',
u'\ueb59': 'UnicodeCharacter0xeb59',
u'\u8ad8': 'CjkUnifiedIdeograph8ad8',
u'\u265b': 'BlackChessQueen',
u'\ua05f': 'YiSyllableBbax',
u'\u43de': 'CjkUnifiedIdeograph43de',
u'\udde2': 'UnicodeCharacter0xdde2',
u'\u7965': 'CjkUnifiedIdeograph7965',
u'\u18e4': 'UnicodeCharacter0x18e4',
u'\uf369': 'UnicodeCharacter0xf369',
u'\u92e8': 'CjkUnifiedIdeograph92e8',
u'\u486f': 'CjkUnifiedIdeograph486f',
u'\u6bee': 'CjkUnifiedIdeograph6bee',
u'\uc273': 'HangulSyllableSwid',
u'\ue5f2': 'UnicodeCharacter0xe5f2',
u'\u0175': 'LatinSmallLetterWWithCircumflex',
u'\u20f4': 'UnicodeCharacter0x20f4',
u'\u9b79': 'CjkUnifiedIdeograph9b79',
u'\ubaf8': 'HangulSyllableMwaek',
u'\u507f': 'CjkUnifiedIdeograph507f',
u'\u73fe': 'CjkUnifiedIdeograph73fe',
u'\u0606': 'ArabicindicCubeRoot',
u'\u2289': 'NeitherASupersetOfNorEqualTo',
u'\u4308': 'CjkUnifiedIdeograph4308',
u'\u7f8b': 'CjkUnifiedIdeograph7f8b',
u'\u800a': 'CjkUnifiedIdeograph800a',
u'\ubc8d': 'HangulSyllableBeolg',
u'\udd0c': 'UnicodeCharacter0xdd0c',
u'\uf98f': 'CjkCompatibilityIdeographf98f',
u'\u2e16': 'DottedRightpointingAngle',
u'\u6b18': 'CjkUnifiedIdeograph6b18',
u'\u079b': 'ThaanaLetterThaalu',
u'\ua81a': 'SylotiNagriLetterPho',
u'\u449d': 'CjkUnifiedIdeograph449d',
u'\ue51c': 'UnicodeCharacter0xe51c',
u'\u819f': 'CjkUnifiedIdeograph819f',
u'\udea1': 'UnicodeCharacter0xdea1',
u'\u3626': 'CjkUnifiedIdeograph3626',
u'\u7328': 'CjkUnifiedIdeograph7328',
u'\u2fab': 'KangxiRadicalShortTailedBird',
u'\ub02a': 'HangulSyllableGgyugg',
u'\u6cad': 'CjkUnifiedIdeograph6cad',
u'\u8d2c': 'CjkUnifiedIdeograph8d2c',
u'\ua9af': 'UnicodeCharacter0xa9af',
u'\uca2e': 'HangulSyllableJjyap',
u'\ue6b1': 'UnicodeCharacter0xe6b1',
u'\u1b38': 'BalineseVowelSignSuku',
u'\u37bb': 'CjkUnifiedIdeograph37bb',
u'\u583a': 'CjkUnifiedIdeograph583a',
u'\u74bd': 'CjkUnifiedIdeograph74bd',
u'\u953c': 'CjkUnifiedIdeograph953c',
u'\ub1bf': 'HangulSyllableNwaelh',
u'\ud23e': 'HangulSyllableTubs',
u'\u8ec1': 'CjkUnifiedIdeograph8ec1',
u'\ucbc3': 'HangulSyllableJjyulb',
u'\u2348': 'AplFunctionalSymbolQuadRightwardsArrow',
u'\u604a': 'CjkUnifiedIdeograph604a',
u'\u1ccd': 'UnicodeCharacter0x1ccd',
u'\ubd4c': 'HangulSyllableBoen',
u'\u59cf': 'CjkUnifiedIdeograph59cf',
u'\ufa4e': 'CjkCompatibilityIdeographfa4e',
u'\u96d1': 'CjkUnifiedIdeograph96d1',
u'\ud3d3': 'HangulSyllablePyegs',
u'\u085a': 'UnicodeCharacter0x85a',
u'\u24dd': 'CircledLatinSmallLetterN',
u'\u455c': 'CjkUnifiedIdeograph455c',
u'\u61df': 'CjkUnifiedIdeograph61df',
u'\u825e': 'CjkUnifiedIdeograph825e',
u'\ubee1': 'HangulSyllableBbeob',
u'\udf60': 'UnicodeCharacter0xdf60',
u'\ufbe3': 'ArabicLetterKirghizYuFinalForm',
u'\u106a': 'MyanmarSignWesternPwoKarenTone2',
u'\u6d6c': 'CjkUnifiedIdeograph6d6c',
u'\u09ef': 'BengaliDigitNine',
u'\uaa6e': 'UnicodeCharacter0xaa6e',
u'\u46f1': 'CjkUnifiedIdeograph46f1',
u'\ue770': 'UnicodeCharacter0xe770',
u'\u83f3': 'CjkUnifiedIdeograph83f3',
u'\uc0f5': 'HangulSyllableSyab',
u'\u387a': 'CjkUnifiedIdeograph387a',
u'\u757c': 'CjkUnifiedIdeograph757c',
u'\u11ff': 'UnicodeCharacter0x11ff',
u'\ub27e': 'HangulSyllableNyulm',
u'\u8f80': 'CjkUnifiedIdeograph8f80',
u'\u2b03': 'SouthWestWhiteArrow',
u'\ucc82': 'HangulSyllableCyaenh',
u'\u6805': 'CjkUnifiedIdeograph6805',
u'\ua507': 'VaiSyllableBee',
u'\ue209': 'UnicodeCharacter0xe209',
u'\u1d8c': 'LatinSmallLetterVWithPalatalHook',
u'\u5a8e': 'CjkUnifiedIdeograph5a8e',
u'\u9790': 'CjkUnifiedIdeograph9790',
u'\u3313': 'SquareGirudaa',
u'\ud492': 'HangulSyllablePup',
u'\u7015': 'CjkUnifiedIdeograph7015',
u'\u4d17': 'CjkUnifiedIdeograph4d17',
u'\u8a19': 'CjkUnifiedIdeograph8a19',
u'\uc71b': 'HangulSyllableWic',
u'\u259c': 'QuadrantUpperLeftAndUpperRightAndLowerRight',
u'\u629e': 'CjkUnifiedIdeograph629e',
u'\ubfa0': 'HangulSyllableBboels',
u'\ufca2': 'ArabicLigatureTehWithHahInitialForm',
u'\u1825': 'MongolianLetterOe',
u'\u5527': 'CjkUnifiedIdeograph5527',
u'\u9229': 'CjkUnifiedIdeograph9229',
u'\uef2b': 'UnicodeCharacter0xef2b',
u'\u0aae': 'GujaratiLetterMa',
u'\u47b0': 'CjkUnifiedIdeograph47b0',
u'\u84b2': 'CjkUnifiedIdeograph84b2',
u'\u2035': 'ReversedPrime',
u'\uc1b4': 'HangulSyllableSwals',
u'\u7d37': 'CjkUnifiedIdeograph7d37',
u'\uba39': 'HangulSyllableMeog',
u'\uf73b': 'UnicodeCharacter0xf73b',
u'\u12be': 'EthiopicSyllableKxo',
u'\u6fc0': 'CjkUnifiedIdeograph6fc0',
u'\uacc2': 'HangulSyllableGyeop',
u'\ue9c4': 'UnicodeCharacter0xe9c4',
u'\u0547': 'ArmenianCapitalLetterSha',
u'\u4249': 'CjkUnifiedIdeograph4249',
u'\u9f4b': 'CjkUnifiedIdeograph9f4b',
u'\udc4d': 'UnicodeCharacter0xdc4d',
u'\u3ace': 'CjkUnifiedIdeograph3ace',
u'\u77d0': 'CjkUnifiedIdeograph77d0',
u'\ub4d2': 'HangulSyllableDyubs',
u'\uf1d4': 'UnicodeCharacter0xf1d4',
u'\u2d57': 'TifinaghLetterTuaregYagh',
u'\uced6': 'HangulSyllableKyaelp',
u'\u6a59': 'CjkUnifiedIdeograph6a59',
u'\ua75b': 'LatinSmallLetterRRotunda',
u'\ue45d': 'UnicodeCharacter0xe45d',
u'\u1fe0': 'GreekSmallLetterUpsilonWithVrachy',
u'\u5ce2': 'CjkUnifiedIdeograph5ce2',
u'\u99e4': 'CjkUnifiedIdeograph99e4',
u'\u3567': 'CjkUnifiedIdeograph3567',
u'\ud6e6': 'HangulSyllableHweonh',
u'\u7269': 'CjkUnifiedIdeograph7269',
u'\u4f6b': 'CjkUnifiedIdeograph4f6b',
u'\u8c6d': 'CjkUnifiedIdeograph8c6d',
u'\uc96f': 'HangulSyllableJyugs',
u'\u27f0': 'UpwardsQuadrupleArrow',
u'\u64f2': 'CjkUnifiedIdeograph64f2',
u'\ua1f4': 'YiSyllableGu',
u'\ufef6': 'ArabicLigatureLamWithAlefWithMaddaAboveFinalForm',
u'\u1a79': 'UnicodeCharacter0x1a79',
u'\u577b': 'CjkUnifiedIdeograph577b',
u'\u947d': 'CjkUnifiedIdeograph947d',
u'\ud17f': 'HangulSyllableTyeoc',
u'\ub181': 'HangulSyllableNolg',
u'\u5604': 'CjkUnifiedIdeograph5604',
u'\u7a87': 'CjkUnifiedIdeograph7a87',
u'\udc08': 'UnicodeCharacter0xdc08',
u'\ue08b': 'UnicodeCharacter0xe08b',
u'\u1f0a': 'GreekCapitalLetterAlphaWithPsiliAndVaria',
u'\u238d': 'MonostableSymbol',
u'\u850e': 'CjkUnifiedIdeograph850e',
u'\ua991': 'UnicodeCharacter0xa991',
u'\u4e14': 'CjkUnifiedIdeograph4e14',
u'\u1297': 'EthiopicSyllableNwa',
u'\uf418': 'UnicodeCharacter0xf418',
u'\u989b': 'CjkUnifiedIdeograph989b',
u'\u371a': 'CjkUnifiedIdeograph371a',
u'\ubd1e': 'HangulSyllableBwalp',
u'\u41a1': 'CjkUnifiedIdeograph41a1',
u'\uc7a5': 'HangulSyllableJang',
u'\u6624': 'CjkUnifiedIdeograph6624',
u'\u0aa7': 'GujaratiLetterDha',
u'\uec28': 'UnicodeCharacter0xec28',
u'\ub0ab': 'HangulSyllableNas',
u'\u2f2a': 'KangxiRadicalLame',
u'\u552e': 'CjkUnifiedIdeograph552e',
u'\u79b1': 'CjkUnifiedIdeograph79b1',
u'\udb32': 'UnicodeCharacter0xdb32',
u'\uffb5': 'HalfwidthHangulLetterSios',
u'\u1e34': 'LatinCapitalLetterKWithLineBelow',
u'\u22b7': 'ImageOf',
u'\u8438': 'CjkUnifiedIdeograph8438',
u'\ua8bb': 'SaurashtraVowelSignVocalicRr',
u'\u4d3e': 'CjkUnifiedIdeograph4d3e',
u'\u11c1': 'HangulJongseongPhieuph',
u'\uf342': 'UnicodeCharacter0xf342',
u'\u97c5': 'CjkUnifiedIdeograph97c5',
u'\u3644': 'CjkUnifiedIdeograph3644',
u'\ubc48': 'HangulSyllableBaek',
u'\u40cb': 'CjkUnifiedIdeograph40cb',
u'\uc6cf': 'HangulSyllableWeogs',
u'\u654e': 'CjkUnifiedIdeograph654e',
u'\u09d1': 'UnicodeCharacter0x9d1',
u'\ueb52': 'UnicodeCharacter0xeb52',
u'\u8fd5': 'CjkUnifiedIdeograph8fd5',
u'\u2e54': 'UnicodeCharacter0x2e54',
u'\u5458': 'CjkUnifiedIdeograph5458',
u'\u78db': 'CjkUnifiedIdeograph78db',
u'\uda5c': 'UnicodeCharacter0xda5c',
u'\ufedf': 'ArabicLetterLamInitialForm',
u'\u1d5e': 'ModifierLetterSmallGreekGamma',
u'\u21e1': 'UpwardsDashedArrow',
u'\u8362': 'CjkUnifiedIdeograph8362',
u'\ua7e5': 'UnicodeCharacter0xa7e5',
u'\u4c68': 'CjkUnifiedIdeograph4c68',
u'\u10eb': 'GeorgianLetterJil',
u'\uf26c': 'UnicodeCharacter0xf26c',
u'\u96ef': 'CjkUnifiedIdeograph96ef',
u'\u356e': 'CjkUnifiedIdeograph356e',
u'\ubb72': 'HangulSyllableMwenh',
u'\u5ff5': 'CjkUnifiedIdeograph5ff5',
u'\uc5f9': 'HangulSyllableYeolt',
u'\u6478': 'CjkUnifiedIdeograph6478',
u'\u08fb': 'UnicodeCharacter0x8fb',
u'\uea7c': 'UnicodeCharacter0xea7c',
u'\u8eff': 'CjkUnifiedIdeograph8eff',
u'\u2d7e': 'UnicodeCharacter0x2d7e',
u'\ub403': 'HangulSyllableDwaed',
u'\u5382': 'CjkUnifiedIdeograph5382',
u'\ud986': 'UnicodeCharacter0xd986',
u'\u7d09': 'CjkUnifiedIdeograph7d09',
u'\u1c88': 'UnicodeCharacter0x1c88',
u'\ue30d': 'UnicodeCharacter0xe30d',
u'\u828c': 'CjkUnifiedIdeograph828c',
u'\u260f': 'WhiteTelephone',
u'\uac13': 'HangulSyllableGas',
u'\u4b92': 'CjkUnifiedIdeograph4b92',
u'\ud217': 'HangulSyllableTyod',
u'\uf196': 'UnicodeCharacter0xf196',
u'\u1519': 'CanadianSyllabicsShwi',
u'\u3498': 'CjkUnifiedIdeograph3498',
u'\u9b1d': 'CjkUnifiedIdeograph9b1d',
u'\uba9c': 'HangulSyllableMyem',
u'\u4423': 'CjkUnifiedIdeograph4423',
u'\u63a2': 'CjkUnifiedIdeograph63a2',
u'\uca27': 'HangulSyllableJjyas',
u'\ue9a6': 'UnicodeCharacter0xe9a6',
u'\u0d29': 'UnicodeCharacter0xd29',
u'\u2ca8': 'CopticCapitalLetterUa',
u'\ub32d': 'HangulSyllableDyab',
u'\u52ac': 'CjkUnifiedIdeograph52ac',
u'\ud8b0': 'UnicodeCharacter0xd8b0',
u'\u7c33': 'CjkUnifiedIdeograph7c33',
u'\u1bb2': 'SundaneseDigitTwo',
u'\ue237': 'UnicodeCharacter0xe237',
u'\u81b6': 'CjkUnifiedIdeograph81b6',
u'\u2539': 'BoxDrawingsRightLightAndLeftUpHeavy',
u'\uab3d': 'UnicodeCharacter0xab3d',
u'\u4abc': 'CjkUnifiedIdeograph4abc',
u'\ud141': 'HangulSyllableTeob',
u'\uf0c0': 'UnicodeCharacter0xf0c0',
u'\u1443': 'CanadianSyllabicsWestcreePwoo',
u'\u33c2': 'SquareAm',
u'\u9a47': 'CjkUnifiedIdeograph9a47',
u'\ub9c6': 'HangulSyllableRip',
u'\u434d': 'CjkUnifiedIdeograph434d',
u'\u62cc': 'CjkUnifiedIdeograph62cc',
u'\uc951': 'HangulSyllableJwig',
u'\ue8d0': 'UnicodeCharacter0xe8d0',
u'\u0c53': 'UnicodeCharacter0xc53',
u'\u2bd2': 'UnicodeCharacter0x2bd2',
u'\ub257': 'HangulSyllableNweh',
u'\u51d6': 'CjkUnifiedIdeograph51d6',
u'\ud7da': 'UnicodeCharacter0xd7da',
u'\u7b5d': 'CjkUnifiedIdeograph7b5d',
u'\u1adc': 'UnicodeCharacter0x1adc',
u'\ue161': 'UnicodeCharacter0xe161',
u'\u80e0': 'CjkUnifiedIdeograph80e0',
u'\u2463': 'CircledDigitFour',
u'\uaa67': 'UnicodeCharacter0xaa67',
u'\u49e6': 'CjkUnifiedIdeograph49e6',
u'\ud06b': 'HangulSyllableKyuh',
u'\ucfea': 'HangulSyllableKulm',
u'\u136d': 'EthiopicDigitFive',
u'\u32ec': 'CircledKatakanaHe',
u'\u9971': 'CjkUnifiedIdeograph9971',
u'\ub8f0': 'HangulSyllableRul',
u'\u4277': 'CjkUnifiedIdeograph4277',
u'\u61f6': 'CjkUnifiedIdeograph61f6',
u'\uc87b': 'HangulSyllableJolb',
u'\ue7fa': 'UnicodeCharacter0xe7fa',
u'\u0b7d': 'UnicodeCharacter0xb7d',
u'\u2afc': 'LargeTripleVerticalBarOperator',
u'\u3081': 'HiraganaLetterMe',
u'\u5100': 'CjkUnifiedIdeograph5100',
u'\u7583': 'CjkUnifiedIdeograph7583',
u'\u9202': 'CjkUnifiedIdeograph9202',
u'\ub685': 'HangulSyllableDdyonj',
u'\ud704': 'HangulSyllableHwel',
u'\ufb87': 'ArabicLetterDulFinalForm',
u'\u040e': 'CyrillicCapitalLetterShortU',
u'\u2891': 'BraillePatternDots158',
u'\u4910': 'CjkUnifiedIdeograph4910',
u'\u6d93': 'CjkUnifiedIdeograph6d93',
u'\u8a12': 'CjkUnifiedIdeograph8a12',
u'\uae95': 'HangulSyllableGgyab',
u'\ucf14': 'HangulSyllableKess',
u'\u9397': 'CjkUnifiedIdeograph9397',
u'\ud499': 'HangulSyllablePweonj',
u'\u3c1e': 'CjkUnifiedIdeograph3c1e',
u'\u6120': 'CjkUnifiedIdeograph6120',
u'\u05a3': 'HebrewAccentMunah',
u'\ua222': 'YiSyllableGgot',
u'\u46a5': 'CjkUnifiedIdeograph46a5',
u'\ue724': 'UnicodeCharacter0xe724',
u'\u8ba7': 'CjkUnifiedIdeograph8ba7',
u'\ucca9': 'HangulSyllableCeob',
u'\u1930': 'LimbuSmallLetterKa',
u'\u3db3': 'CjkUnifiedIdeograph3db3',
u'\u5a32': 'CjkUnifiedIdeograph5a32',
u'\u7eb5': 'CjkUnifiedIdeograph7eb5',
u'\u9f34': 'CjkUnifiedIdeograph9f34',
u'\ua3b7': 'YiSyllableShex',
u'\uc036': 'HangulSyllableBbwij',
u'\ue4b9': 'UnicodeCharacter0xe4b9',
u'\u3140': 'HangulLetterRieulhieuh',
u'\u7242': 'CjkUnifiedIdeograph7242',
u'\u16c5': 'RunicLetterLongbrancharAe',
u'\ub744': 'HangulSyllableDdyi',
u'\u5bc7': 'CjkUnifiedIdeograph5bc7',
u'\uf846': 'UnicodeCharacter0xf846',
u'\u9cc9': 'CjkUnifiedIdeograph9cc9',
u'\uc1cb': 'HangulSyllableSwaed',
u'\u2950': 'LeftBarbDownRightBarbDownHarpoon',
u'\u6a52': 'CjkUnifiedIdeograph6a52',
u'\u0ed5': 'LaoDigitFive',
u'\uaf54': 'HangulSyllableGgwals',
u'\u73d7': 'CjkUnifiedIdeograph73d7',
u'\u9056': 'CjkUnifiedIdeograph9056',
u'\ub4d9': 'HangulSyllableDyut',
u'\ud558': 'HangulSyllableHa',
u'\uf9db': 'CjkCompatibilityIdeographf9db',
u'\u0262': 'LatinLetterSmallCapitalG',
u'\u26e5': 'UnicodeCharacter0x26e5',
u'\u4764': 'CjkUnifiedIdeograph4764',
u'\u6be7': 'CjkUnifiedIdeograph6be7',
u'\u8866': 'CjkUnifiedIdeograph8866',
u'\uace9': 'HangulSyllableGolg',
u'\ucd68': 'HangulSyllableCoels',
u'\u91eb': 'CjkUnifiedIdeograph91eb',
u'\ud2ed': 'HangulSyllableTyit',
u'\u3a72': 'CjkUnifiedIdeograph3a72',
u'\u7f74': 'CjkUnifiedIdeograph7f74',
u'\u03f7': 'GreekCapitalLetterSho',
u'\ua076': 'YiSyllableNbit',
u'\u44f9': 'CjkUnifiedIdeograph44f9',
u'\ue578': 'UnicodeCharacter0xe578',
u'\u89fb': 'CjkUnifiedIdeograph89fb',
u'\ucafd': 'HangulSyllableJjwaelg',
u'\uf001': 'UnicodeCharacter0xf001',
u'\u1784': 'KhmerLetterNgo',
u'\u5886': 'CjkUnifiedIdeograph5886',
u'\u9d88': 'CjkUnifiedIdeograph9d88',
u'\u210b': 'ScriptCapitalH',
u'\ude8a': 'UnicodeCharacter0xde8a',
u'\u620d': 'CjkUnifiedIdeograph620d',
u'\ua70f': 'ModifierLetterMidDottedLeftstemToneBar',
u'\ue811': 'UnicodeCharacter0xe811',
u'\u0f94': 'TibetanSubjoinedLetterNga',
u'\u7096': 'CjkUnifiedIdeograph7096',
u'\ub598': 'HangulSyllableDdyaess',
u'\uf69a': 'UnicodeCharacter0xf69a',
u'\u1a1d': 'UnicodeCharacter0x1a1d',
u'\u5f1f': 'CjkUnifiedIdeograph5f1f',
u'\u8021': 'CjkUnifiedIdeograph8021',
u'\uc523': 'HangulSyllableSsyic',
u'\u27a4': 'BlackRightwardsArrowhead',
u'\u68a6': 'CjkUnifiedIdeograph68a6',
u'\uada8': 'HangulSyllableGwen',
u'\ueeaa': 'UnicodeCharacter0xeeaa',
u'\u322d': 'ParenthesizedIdeographWood',
u'\ud3ac': 'HangulSyllablePess',
u'\u772f': 'CjkUnifiedIdeograph772f',
u'\ub831': 'HangulSyllableRyeolt',
u'\ufd33': 'ArabicLigatureTahWithMeemInitialForm',
u'\xb6': 'PilcrowSign',
u'\u45b8': 'CjkUnifiedIdeograph45b8',
u'\u86ba': 'CjkUnifiedIdeograph86ba',
u'\u2a3d': 'RighthandInteriorProduct',
u'\ucbbc': 'HangulSyllableJjyun',
u'\u6f3f': 'CjkUnifiedIdeograph6f3f',
u'\u5041': 'CjkUnifiedIdeograph5041',
u'\u9543': 'CjkUnifiedIdeograph9543',
u'\ud645': 'HangulSyllableHolt',
u'\u38c6': 'CjkUnifiedIdeograph38c6',
u'\u7dc8': 'CjkUnifiedIdeograph7dc8',
u'\ubeca': 'HangulSyllableBbyaej',
u'\ue3cc': 'UnicodeCharacter0xe3cc',
u'\u074f': 'SyriacLetterSogdianFe',
u'\u4851': 'CjkUnifiedIdeograph4851',
u'\u8d53': 'CjkUnifiedIdeograph8d53',
u'\uce55': 'HangulSyllableCyit',
u'\u15d8': 'CanadianSyllabicsCarrierHwe',
u'\u56da': 'CjkUnifiedIdeograph56da',
u'\u9bdc': 'CjkUnifiedIdeograph9bdc',
u'\u3f5f': 'CjkUnifiedIdeograph3f5f',
u'\udcde': 'UnicodeCharacter0xdcde',
u'\u6061': 'CjkUnifiedIdeograph6061',
u'\ua563': 'VaiSyllableSha',
u'\ue665': 'UnicodeCharacter0xe665',
u'\u0de8': 'UnicodeCharacter0xde8',
u'\u4eea': 'CjkUnifiedIdeograph4eea',
u'\ub3ec': 'HangulSyllableDwals',
u'\uf4ee': 'UnicodeCharacter0xf4ee',
u'\u1871': 'MongolianLetterSibeCha',
u'\u5d73': 'CjkUnifiedIdeograph5d73',
u'\u9e75': 'CjkUnifiedIdeograph9e75',
u'\uc377': 'HangulSyllableSseolh',
u'\u25f8': 'UpperLeftTriangle',
u'\u66fa': 'CjkUnifiedIdeograph66fa',
u'\uabfc': 'UnicodeCharacter0xabfc',
u'\uecfe': 'UnicodeCharacter0xecfe',
u'\ud200': 'HangulSyllableToels',
u'\uf683': 'UnicodeCharacter0xf683',
u'\u1102': 'HangulChoseongNieun',
u'\u3585': 'CjkUnifiedIdeograph3585',
u'\u9b06': 'CjkUnifiedIdeograph9b06',
u'\ubf89': 'HangulSyllableBbwaeb',
u'\u400c': 'CjkUnifiedIdeograph400c',
u'\u648f': 'CjkUnifiedIdeograph648f',
u'\uca10': 'HangulSyllableJjaek',
u'\uee93': 'UnicodeCharacter0xee93',
u'\u0912': 'DevanagariLetterShortO',
u'\u2d95': 'EthiopicSyllableGgwee',
u'\ub316': 'HangulSyllableDaej',
u'\u5799': 'CjkUnifiedIdeograph5799',
u'\ud99d': 'UnicodeCharacter0xd99d',
u'\u781c': 'CjkUnifiedIdeograph781c',
u'\u1c9f': 'UnicodeCharacter0x1c9f',
u'\ue220': 'UnicodeCharacter0xe220',
u'\u86a3': 'CjkUnifiedIdeograph86a3',
u'\u2122': 'TradeMarkSign',
u'\uab26': 'UnicodeCharacter0xab26',
u'\u4fa9': 'CjkUnifiedIdeograph4fa9',
u'\uf1ad': 'UnicodeCharacter0xf1ad',
u'\u102c': 'MyanmarVowelSignAa',
u'\u34af': 'CjkUnifiedIdeograph34af',
u'\u9a30': 'CjkUnifiedIdeograph9a30',
u'\ubeb3': 'HangulSyllableBbyah',
u'\u4336': 'CjkUnifiedIdeograph4336',
u'\u67b9': 'CjkUnifiedIdeograph67b9',
u'\uc53a': 'HangulSyllableSsibs',
u'\ue9bd': 'UnicodeCharacter0xe9bd',
u'\u083c': 'UnicodeCharacter0x83c',
u'\u2cbf': 'CopticSmallLetterOldCopticOou',
u'\ub240': 'HangulSyllableNwen',
u'\u56c3': 'CjkUnifiedIdeograph56c3',
u'\ud8c7': 'UnicodeCharacter0xd8c7',
u'\u7b46': 'CjkUnifiedIdeograph7b46',
u'\u1fc9': 'GreekCapitalLetterEpsilonWithOxia',
u'\ufd4a': 'UnicodeCharacter0xfd4a',
u'\u81cd': 'CjkUnifiedIdeograph81cd',
u'\u204c': 'BlackLeftwardsBullet',
u'\uaa50': 'ChamDigitZero',
u'\u4ed3': 'CjkUnifiedIdeograph4ed3',
u'\uf0d7': 'UnicodeCharacter0xf0d7',
u'\u1356': 'EthiopicSyllablePo',
u'\u37d9': 'CjkUnifiedIdeograph37d9',
u'\u955a': 'CjkUnifiedIdeograph955a',
u'\ub9dd': 'HangulSyllableMang',
u'\u4260': 'CjkUnifiedIdeograph4260',
u'\u66e3': 'CjkUnifiedIdeograph66e3',
u'\uc464': 'HangulSyllableSsu',
u'\ue8e7': 'UnicodeCharacter0xe8e7',
u'\u0b66': 'OriyaDigitZero',
u'\u2fe9': 'UnicodeCharacter0x2fe9',
u'\u8d6a': 'CjkUnifiedIdeograph8d6a',
u'\u51ed': 'CjkUnifiedIdeograph51ed',
u'\udbf1': 'UnicodeCharacter0xdbf1',
u'\u7a70': 'CjkUnifiedIdeograph7a70',
u'\u1ef3': 'LatinSmallLetterYWithGrave',
u'\ufc74': 'ArabicLigatureTehWithAlefMaksuraFinalForm',
u'\u80f7': 'CjkUnifiedIdeograph80f7',
u'\u2376': 'AplFunctionalSymbolAlphaUnderbar',
u'\ua57a': 'VaiSyllableKpoo',
u'\u49fd': 'CjkUnifiedIdeograph49fd',
u'\u7301': 'CjkUnifiedIdeograph7301',
u'\u1280': 'EthiopicSyllableXa',
u'\uf505': 'UnicodeCharacter0xf505',
u'\u9484': 'CjkUnifiedIdeograph9484',
u'\u3807': 'CjkUnifiedIdeograph3807',
u'\ua20b': 'YiSyllableKex',
u'\u5d8a': 'CjkUnifiedIdeograph5d8a',
u'\uc78e': 'HangulSyllableIp',
u'\u6b11': 'CjkUnifiedIdeograph6b11',
u'\u0a90': 'GujaratiLetterAi',
u'\ued15': 'UnicodeCharacter0xed15',
u'\u8c94': 'CjkUnifiedIdeograph8c94',
u'\u5a1b': 'CjkUnifiedIdeograph5a1b',
u'\u759a': 'CjkUnifiedIdeograph759a',
u'\udc1f': 'UnicodeCharacter0xdc1f',
u'\uff9e': 'HalfwidthKatakanaVoicedSoundMark',
u'\u0321': 'CombiningPalatalizedHookBelow',
u'\u22a0': 'SquaredTimes',
u'\u8525': 'CjkUnifiedIdeograph8525',
u'\ua4a4': 'YiRadicalDdur',
u'\u722b': 'CjkUnifiedIdeograph722b',
u'\u6daa': 'CjkUnifiedIdeograph6daa',
u'\uf42f': 'UnicodeCharacter0xf42f',
u'\u97ae': 'CjkUnifiedIdeograph97ae',
u'\u3b31': 'CjkUnifiedIdeograph3b31',
u'\ubd35': 'HangulSyllableBwaelg',
u'\u5cb4': 'CjkUnifiedIdeograph5cb4',
u'\uc6b8': 'HangulSyllableUl',
u'\u6a3b': 'CjkUnifiedIdeograph6a3b',
u'\u05ba': 'HebrewPointHolamHaserForVav',
u'\uec3f': 'UnicodeCharacter0xec3f',
u'\u8fbe': 'CjkUnifiedIdeograph8fbe',
u'\u5545': 'CjkUnifiedIdeograph5545',
u'\u74c4': 'CjkUnifiedIdeograph74c4',
u'\udf49': 'UnicodeCharacter0xdf49',
u'\ufec8': 'ArabicLetterZahMedialForm',
u'\u024b': 'LatinSmallLetterQWithHookTail',
u'\u3dca': 'CjkUnifiedIdeograph3dca',
u'\u844f': 'CjkUnifiedIdeograph844f',
u'\ua7ce': 'UnicodeCharacter0xa7ce',
u'\u4d55': 'CjkUnifiedIdeograph4d55',
u'\u6cd4': 'CjkUnifiedIdeograph6cd4',
u'\uf759': 'UnicodeCharacter0xf759',
u'\u96d8': 'CjkUnifiedIdeograph96d8',
u'\u3a5b': 'CjkUnifiedIdeograph3a5b',
u'\ubc5f': 'HangulSyllableByas',
u'\u5fde': 'CjkUnifiedIdeograph5fde',
u'\uc1e2': 'HangulSyllableSoegg',
u'\u6565': 'CjkUnifiedIdeograph6565',
u'\u04e4': 'CyrillicCapitalLetterIWithDiaeresis',
u'\uef69': 'UnicodeCharacter0xef69',
u'\u8ee8': 'CjkUnifiedIdeograph8ee8',
u'\u546f': 'CjkUnifiedIdeograph546f',
u'\u77ee': 'CjkUnifiedIdeograph77ee',
u'\ude73': 'UnicodeCharacter0xde73',
u'\uf9f2': 'CjkCompatibilityIdeographf9f2',
u'\u1d75': 'LatinSmallLetterTWithMiddleTilde',
u'\u3cf4': 'CjkUnifiedIdeograph3cf4',
u'\u8779': 'CjkUnifiedIdeograph8779',
u'\ua6f8': 'UnicodeCharacter0xa6f8',
u'\u4c7f': 'CjkUnifiedIdeograph4c7f',
u'\u6ffe': 'CjkUnifiedIdeograph6ffe',
u'\u1a06': 'BugineseLetterMa',
u'\u3e89': 'CjkUnifiedIdeograph3e89',
u'\u5f08': 'CjkUnifiedIdeograph5f08',
u'\u638b': 'CjkUnifiedIdeograph638b',
u'\u9c0a': 'CjkUnifiedIdeograph9c0a',
u'\ua08d': 'YiSyllableNby',
u'\uc10c': 'HangulSyllableSyaels',
u'\ue58f': 'UnicodeCharacter0xe58f',
u'\u3216': 'ParenthesizedHangulCieucA',
u'\u7718': 'CjkUnifiedIdeograph7718',
u'\u1b9b': 'SundaneseLetterRa',
u'\ub41a': 'HangulSyllableDoegg',
u'\u589d': 'CjkUnifiedIdeograph589d',
u'\uf91c': 'CjkCompatibilityIdeographf91c',
u'\u9d9f': 'CjkUnifiedIdeograph9d9f',
u'\uc2a1': 'HangulSyllableSyut',
u'\u2a26': 'PlusSignWithTildeBelow',
u'\u6f28': 'CjkUnifiedIdeograph6f28',
u'\u33ab': 'SquareMpa',
u'\uac2a': 'HangulSyllableGaelp',
u'\u70ad': 'CjkUnifiedIdeograph70ad',
u'\u912c': 'CjkUnifiedIdeograph912c',
u'\ub5af': 'HangulSyllableDdeolh',
u'\ud62e': 'HangulSyllableHyebs',
u'\ufab1': 'CjkCompatibilityIdeographfab1',
u'\u0738': 'SyriacDottedZlamaHorizontal',
u'\u2bbb': 'UnicodeCharacter0x2bbb',
u'\u443a': 'CjkUnifiedIdeograph443a',
u'\u68bd': 'CjkUnifiedIdeograph68bd',
u'\u893c': 'CjkUnifiedIdeograph893c',
u'\uadbf': 'HangulSyllableGweh',
u'\uce3e': 'HangulSyllableCyigg',
u'\u92c1': 'CjkUnifiedIdeograph92c1',
u'\ud7c3': 'UnicodeCharacter0xd7c3',
u'\u3f48': 'CjkUnifiedIdeograph3f48',
u'\u7c4a': 'CjkUnifiedIdeograph7c4a',
u'\xcd': 'LatinCapitalLetterIWithAcute',
u'\ua14c': 'YiSyllableDdux',
u'\u45cf': 'CjkUnifiedIdeograph45cf',
u'\ue64e': 'UnicodeCharacter0xe64e',
u'\u8ad1': 'CjkUnifiedIdeograph8ad1',
u'\ucfd3': 'HangulSyllableKyolh',
u'\u145a': 'CanadianSyllabicsWestcreeTwi',
u'\u38dd': 'CjkUnifiedIdeograph38dd',
u'\u595c': 'CjkUnifiedIdeograph595c',
u'\u7ddf': 'CjkUnifiedIdeograph7ddf',
u'\u9e5e': 'CjkUnifiedIdeograph9e5e',
u'\ua2e1': 'YiSyllableZzep',
u'\uc360': 'HangulSyllableSsyaess',
u'\ue7e3': 'UnicodeCharacter0xe7e3',
u'\u0c6a': 'TeluguDigitFour',
u'\u716c': 'CjkUnifiedIdeograph716c',
u'\u15ef': 'CanadianSyllabicsCarrierGu',
u'\ub66e': 'HangulSyllableDdoelm',
u'\u5af1': 'CjkUnifiedIdeograph5af1',
u'\ufb70': 'ArabicLetterPehehInitialForm',
u'\u9ff3': 'UnicodeCharacter0x9ff3',
u'\udcf5': 'UnicodeCharacter0xdcf5',
u'\u247a': 'ParenthesizedDigitSeven',
u'\u697c': 'CjkUnifiedIdeograph697c',
u'\u0dff': 'UnicodeCharacter0xdff',
u'\uae7e': 'HangulSyllableGgaej',
u'\u9380': 'CjkUnifiedIdeograph9380',
u'\u3703': 'CjkUnifiedIdeograph3703',
u'\ud082': 'HangulSyllableKeuj',
u'\u7405': 'CjkUnifiedIdeograph7405',
u'\ub907': 'HangulSyllableRweogs',
u'\ufe09': 'VariationSelector10',
u'\u018c': 'LatinSmallLetterDWithTopbar',
u'\u468e': 'CjkUnifiedIdeograph468e',
u'\u8b90': 'CjkUnifiedIdeograph8b90',
u'\u2f13': 'KangxiRadicalWrap',
u'\uc892': 'HangulSyllableJwanh',
u'\u6c15': 'CjkUnifiedIdeograph6c15',
u'\u5117': 'CjkUnifiedIdeograph5117',
u'\u9619': 'CjkUnifiedIdeograph9619',
u'\udb1b': 'UnicodeCharacter0xdb1b',
u'\u399c': 'CjkUnifiedIdeograph399c',
u'\u7e9e': 'CjkUnifiedIdeograph7e9e',
u'\ua3a0': 'YiSyllableNrux',
u'\ue0a2': 'UnicodeCharacter0xe0a2',
u'\u0425': 'CyrillicCapitalLetterHa',
u'\u4927': 'CjkUnifiedIdeograph4927',
u'\u8e29': 'CjkUnifiedIdeograph8e29',
u'\uf32b': 'UnicodeCharacter0xf32b',
u'\u16ae': 'RunicLetterO',
u'\u5bb0': 'CjkUnifiedIdeograph5bb0',
u'\u98b2': 'CjkUnifiedIdeograph98b2',
u'\u3c35': 'CjkUnifiedIdeograph3c35',
u'\uddb4': 'UnicodeCharacter0xddb4',
u'\u6137': 'CjkUnifiedIdeograph6137',
u'\ua639': 'UnicodeCharacter0xa639',
u'\ueb3b': 'UnicodeCharacter0xeb3b',
u'\u0ebe': 'UnicodeCharacter0xebe',
u'\u73c0': 'CjkUnifiedIdeograph73c0',
u'\ub0c2': 'HangulSyllableNaelp',
u'\uf5c4': 'UnicodeCharacter0xf5c4',
u'\u1947': 'LimbuDigitOne',
u'\u5e49': 'CjkUnifiedIdeograph5e49',
u'\u834b': 'CjkUnifiedIdeograph834b',
u'\uc04d': 'HangulSyllableBbyub',
u'\u26ce': 'UnicodeCharacter0x26ce',
u'\u6bd0': 'CjkUnifiedIdeograph6bd0',
u'\ua8d2': 'SaurashtraDigitTwo',
u'\uedd4': 'UnicodeCharacter0xedd4',
u'\u3157': 'HangulLetterO',
u'\ud2d6': 'HangulSyllableTyigg',
u'\u7659': 'CjkUnifiedIdeograph7659',
u'\ubb5b': 'HangulSyllableMweolb',
u'\uf85d': 'UnicodeCharacter0xf85d',
u'\u03e0': 'GreekLetterSampi',
u'\u40e2': 'CjkUnifiedIdeograph40e2',
u'\u85e4': 'CjkUnifiedIdeograph85e4',
u'\u2967': 'LeftwardsHarpoonWithBarbDownAboveRightwardsHarpoonWithBarbDown',
u'\ucae6': 'HangulSyllableJjwalp',
u'\u6e69': 'CjkUnifiedIdeograph6e69',
u'\u536b': 'CjkUnifiedIdeograph536b',
u'\u906d': 'CjkUnifiedIdeograph906d',
u'\ud56f': 'HangulSyllableHac',
u'\u3bf0': 'CjkUnifiedIdeograph3bf0',
u'\u78f2': 'CjkUnifiedIdeograph78f2',
u'\ubdf4': 'HangulSyllableByun',
u'\ue2f6': 'UnicodeCharacter0xe2f6',
u'\u0679': 'ArabicLetterTteh',
u'\u4b7b': 'CjkUnifiedIdeograph4b7b',
u'\u887d': 'CjkUnifiedIdeograph887d',
u'\ucd7f': 'HangulSyllableCyod',
u'\ub581': 'HangulSyllableDdyat',
u'\u5204': 'CjkUnifiedIdeograph5204',
u'\u7e87': 'CjkUnifiedIdeograph7e87',
u'\ud808': 'UnicodeCharacter0xd808',
u'\ue48b': 'UnicodeCharacter0xe48b',
u'\u1b0a': 'BalineseLetterUkaraTedung',
u'\u278d': 'DingbatNegativeCircledSansserifDigitFour',
u'\u810e': 'CjkUnifiedIdeograph810e',
u'\uad91': 'HangulSyllableGweolg',
u'\u4a14': 'CjkUnifiedIdeograph4a14',
u'\u1697': 'OghamLetterUilleann',
u'\uf018': 'UnicodeCharacter0xf018',
u'\u9c9b': 'CjkUnifiedIdeograph9c9b',
u'\u331a': 'SquareKuruzeiro',
u'\ub91e': 'HangulSyllableRweop',
u'\u45a1': 'CjkUnifiedIdeograph45a1',
u'\uc3a5': 'HangulSyllableSsyeonj',
u'\u6224': 'CjkUnifiedIdeograph6224',
u'\u0ea7': 'LaoLetterWo',
u'\ue828': 'UnicodeCharacter0xe828',
u'\ub4ab': 'HangulSyllableDwid',
u'\u2b2a': 'BlackSmallLozenge',
u'\u512e': 'CjkUnifiedIdeograph512e',
u'\u7db1': 'CjkUnifiedIdeograph7db1',
u'\udf32': 'UnicodeCharacter0xdf32',
u'\ufbb5': 'UnicodeCharacter0xfbb5',
u'\u1a34': 'UnicodeCharacter0x1a34',
u'\u26b7': 'Chiron',
u'\u8038': 'CjkUnifiedIdeograph8038',
u'\uacbb': 'HangulSyllableGyeos',
u'\u493e': 'CjkUnifiedIdeograph493e',
u'\u15c1': 'CanadianSyllabicsSayisiHi',
u'\uf742': 'UnicodeCharacter0xf742',
u'\u93c5': 'CjkUnifiedIdeograph93c5',
u'\u3244': 'UnicodeCharacter0x3244',
u'\ub848': 'HangulSyllableRyel',
u'\u44cb': 'CjkUnifiedIdeograph44cb',
u'\uc2cf': 'HangulSyllableSyilh',
u'\u614e': 'CjkUnifiedIdeograph614e',
u'\u0dd1': 'SinhalaVowelSignDigaAedapilla',
u'\uef52': 'UnicodeCharacter0xef52',
u'\u8bd5': 'CjkUnifiedIdeograph8bd5',
u'\u2a54': 'DoubleLogicalOr',
u'\u5058': 'CjkUnifiedIdeograph5058',
u'\u7cdb': 'CjkUnifiedIdeograph7cdb',
u'\ude5c': 'UnicodeCharacter0xde5c',
u'\ufadf': 'UnicodeCharacter0xfadf',
u'\u195e': 'TaiLeLetterHa',
u'\u25e1': 'LowerHalfCircle',
u'\u8762': 'CjkUnifiedIdeograph8762',
u'\ua3e5': 'YiSyllableJiex',
u'\u4868': 'CjkUnifiedIdeograph4868',
u'\u14eb': 'CanadianSyllabicsWestcreeL',
u'\uf66c': 'UnicodeCharacter0xf66c',
u'\u92ef': 'CjkUnifiedIdeograph92ef',
u'\u316e': 'HangulLetterMieumpieup',
u'\ubf72': 'HangulSyllableBbwaj',
u'\u5bf5': 'CjkUnifiedIdeograph5bf5',
u'\uc1f9': 'HangulSyllableSoet',
u'\u6078': 'CjkUnifiedIdeograph6078',
u'\u0cfb': 'UnicodeCharacter0xcfb',
u'\uee7c': 'UnicodeCharacter0xee7c',
u'\u8aff': 'CjkUnifiedIdeograph8aff',
u'\u297e': 'UpFishTail',
u'\ub003': 'HangulSyllableGgwes',
u'\u5782': 'CjkUnifiedIdeograph5782',
u'\udd86': 'UnicodeCharacter0xdd86',
u'\u7909': 'CjkUnifiedIdeograph7909',
u'\u1888': 'MongolianLetterAliGaliI',
u'\ue70d': 'UnicodeCharacter0xe70d',
u'\u868c': 'CjkUnifiedIdeograph868c',
u'\u220f': 'NaryProduct',
u'\ua813': 'SylotiNagriLetterDdho',
u'\u4f92': 'CjkUnifiedIdeograph4f92',
u'\ud617': 'HangulSyllableHyeoc',
u'\uf596': 'UnicodeCharacter0xf596',
u'\u1119': 'HangulChoseongSsangrieul',
u'\u3098': 'UnicodeCharacter0x3098',
u'\u9f1d': 'CjkUnifiedIdeograph9f1d',
u'\ube9c': 'HangulSyllableBbyan',
u'\u4023': 'CjkUnifiedIdeograph4023',
u'\u67a2': 'CjkUnifiedIdeograph67a2',
u'\uce27': 'HangulSyllableCeud',
u'\ueda6': 'UnicodeCharacter0xeda6',
u'\u0929': 'DevanagariLetterNnna',
u'\u28a8': 'BraillePatternDots468',
u'\ub72d': 'HangulSyllableDdeunj',
u'\u56ac': 'CjkUnifiedIdeograph56ac',
u'\udcb0': 'UnicodeCharacter0xdcb0',
u'\u7833': 'CjkUnifiedIdeograph7833',
u'\u1fb2': 'GreekSmallLetterAlphaWithVariaAndYpogegrammeni',
u'\ue637': 'UnicodeCharacter0xe637',
u'\u85b6': 'CjkUnifiedIdeograph85b6',
u'\u2139': 'InformationSource',
u'\uaf3d': 'HangulSyllableGgob',
u'\u4ebc': 'CjkUnifiedIdeograph4ebc',
u'\ud541': 'HangulSyllablePinj',
u'\uf4c0': 'UnicodeCharacter0xf4c0',
u'\u1043': 'MyanmarDigitThree',
u'\u37c2': 'CjkUnifiedIdeograph37c2',
u'\u9e47': 'CjkUnifiedIdeograph9e47',
u'\ubdc6': 'HangulSyllableBwelp',
u'\u474d': 'CjkUnifiedIdeograph474d',
u'\u66cc': 'CjkUnifiedIdeograph66cc',
u'\ucd51': 'HangulSyllableCwaeb',
u'\uecd0': 'UnicodeCharacter0xecd0',
u'\u0853': 'UnicodeCharacter0x853',
u'\u2fd2': 'KangxiRadicalTooth',
u'\ub657': 'HangulSyllableDdwaelh',
u'\u55d6': 'CjkUnifiedIdeograph55d6',
u'\ud3da': 'HangulSyllablePyelm',
u'\u7f5d': 'CjkUnifiedIdeograph7f5d',
u'\u1edc': 'LatinCapitalLetterOWithHornAndGrave',
u'\ue561': 'UnicodeCharacter0xe561',
u'\u84e0': 'CjkUnifiedIdeograph84e0',
u'\u2063': 'InvisibleSeparator',
u'\uae67': 'HangulSyllableGgah',
u'\u4de6': 'HexagramForObstruction',
u'\ud46b': 'HangulSyllablePyolh',
u'\ucbea': 'HangulSyllableJjeuj',
u'\u176d': 'UnicodeCharacter0x176d',
u'\u36ec': 'CjkUnifiedIdeograph36ec',
u'\u9d71': 'CjkUnifiedIdeograph9d71',
u'\ubcf0': 'HangulSyllableByek',
u'\u4677': 'CjkUnifiedIdeograph4677',
u'\u65f6': 'CjkUnifiedIdeograph65f6',
u'\ucc7b': 'HangulSyllableCyah',
u'\ue3fa': 'UnicodeCharacter0xe3fa',
u'\u0f7d': 'TibetanVowelSignOo',
u'\u2efc': 'UnicodeCharacter0x2efc',
u'\u3481': 'CjkUnifiedIdeograph3481',
u'\u5500': 'CjkUnifiedIdeograph5500',
u'\u7183': 'CjkUnifiedIdeograph7183',
u'\u9602': 'CjkUnifiedIdeograph9602',
u'\ub285': 'HangulSyllableNyub',
u'\ud304': 'HangulSyllableTiss',
u'\uff87': 'HalfwidthKatakanaLetterNu',
u'\u2c91': 'CopticSmallLetterThethe',
u'\u4d10': 'CjkUnifiedIdeograph4d10',
u'\u6993': 'CjkUnifiedIdeograph6993',
u'\u8e12': 'CjkUnifiedIdeograph8e12',
u'\uaa95': 'UnicodeCharacter0xaa95',
u'\ucb14': 'HangulSyllableJjoen',
u'\u9797': 'CjkUnifiedIdeograph9797',
u'\ud099': 'HangulSyllableKyib',
u'\u381e': 'CjkUnifiedIdeograph381e',
u'\u6520': 'CjkUnifiedIdeograph6520',
u'\u01a3': 'LatinSmallLetterOi',
u'\ua622': 'VaiDigitTwo',
u'\u42a5': 'CjkUnifiedIdeograph42a5',
u'\ue324': 'UnicodeCharacter0xe324',
u'\u8fa7': 'CjkUnifiedIdeograph8fa7',
u'\uc8a9': 'HangulSyllableJwaeg',
u'\u1d30': 'ModifierLetterCapitalD',
u'\u39b3': 'CjkUnifiedIdeograph39b3',
u'\u5e32': 'CjkUnifiedIdeograph5e32',
u'\u7ab5': 'CjkUnifiedIdeograph7ab5',
u'\u9b34': 'CjkUnifiedIdeograph9b34',
u'\ua7b7': 'UnicodeCharacter0xa7b7',
u'\uc436': 'HangulSyllableSsoelm',
u'\ue0b9': 'UnicodeCharacter0xe0b9',
u'\u3540': 'CjkUnifiedIdeograph3540',
u'\u7642': 'CjkUnifiedIdeograph7642',
u'\u12c5': 'EthiopicSyllableKxwe',
u'\ub344': 'HangulSyllableDyaels',
u'\u5fc7': 'CjkUnifiedIdeograph5fc7',
u'\ufc46': 'ArabicLigatureMeemWithHahIsolatedForm',
u'\u98c9': 'CjkUnifiedIdeograph98c9',
u'\uc5cb': 'HangulSyllableEoc',
u'\u2d50': 'TifinaghLetterTuaregYagn',
u'\u6e52': 'CjkUnifiedIdeograph6e52',
u'\u0ad5': 'UnicodeCharacter0xad5',
u'\uab54': 'UnicodeCharacter0xab54',
u'\u77d7': 'CjkUnifiedIdeograph77d7',
u'\u9456': 'CjkUnifiedIdeograph9456',
u'\ub0d9': 'HangulSyllableNyalg',
u'\ud158': 'HangulSyllableTels',
u'\ufddb': 'UnicodeCharacter0xfddb',
u'\u0662': 'ArabicindicDigitTwo',
u'\u22e5': 'SquareOriginalOfOrNotEqualTo',
u'\u4364': 'CjkUnifiedIdeograph4364',
u'\u6fe7': 'CjkUnifiedIdeograph6fe7',
u'\u8c66': 'CjkUnifiedIdeograph8c66',
u'\ua8e9': 'UnicodeCharacter0xa8e9',
u'\uc968': 'HangulSyllableJwik',
u'\u95eb': 'CjkUnifiedIdeograph95eb',
u'\ud6ed': 'HangulSyllableHweolt',
u'\u3e72': 'CjkUnifiedIdeograph3e72',
u'\u7b74': 'CjkUnifiedIdeograph7b74',
u'\u07f7': 'NkoSymbolGbakurunen',
u'\ua476': 'YiSyllableYiex',
u'\u40f9': 'CjkUnifiedIdeograph40f9',
u'\ue178': 'UnicodeCharacter0xe178',
u'\u8dfb': 'CjkUnifiedIdeograph8dfb',
u'\ucefd': 'HangulSyllableKeot',
u'\uf401': 'UnicodeCharacter0xf401',
u'\u1384': 'EthiopicSyllableSebatbeitBwa',
u'\u5c86': 'CjkUnifiedIdeograph5c86',
u'\u9988': 'CjkUnifiedIdeograph9988',
u'\u250b': 'BoxDrawingsHeavyQuadrupleDashVertical',
u'\uda8a': 'UnicodeCharacter0xda8a',
u'\u660d': 'CjkUnifiedIdeograph660d',
u'\ua30f': 'YiSyllableSiep',
u'\uec11': 'UnicodeCharacter0xec11',
u'\u0b94': 'TamilLetterAu',
u'\u7496': 'CjkUnifiedIdeograph7496',
u'\ub198': 'HangulSyllableNwan',
u'\uf29a': 'UnicodeCharacter0xf29a',
u'\u1e1d': 'LatinSmallLetterEWithCedillaAndBreve',
u'\u5b1f': 'CjkUnifiedIdeograph5b1f',
u'\u8421': 'CjkUnifiedIdeograph8421',
u'\uc123': 'HangulSyllableSeod',
u'\u23a4': 'RightSquareBracketUpperCorner',
u'\u6ca6': 'CjkUnifiedIdeograph6ca6',
u'\ua9a8': 'UnicodeCharacter0xa9a8',
u'\ueaaa': 'UnicodeCharacter0xeaaa',
u'\u362d': 'CjkUnifiedIdeograph362d',
u'\ud7ac': 'UnicodeCharacter0xd7ac',
u'\u732f': 'CjkUnifiedIdeograph732f',
u'\ubc31': 'HangulSyllableBaeg',
u'\uf933': 'CjkCompatibilityIdeographf933',
u'\u04b6': 'CyrillicCapitalLetterCheWithDescender',
u'\u41b8': 'CjkUnifiedIdeograph41b8',
u'\u82ba': 'CjkUnifiedIdeograph82ba',
u'\u2e3d': 'UnicodeCharacter0x2e3d',
u'\ucfbc': 'HangulSyllableKoess',
u'\u6b3f': 'CjkUnifiedIdeograph6b3f',
u'\u5441': 'CjkUnifiedIdeograph5441',
u'\u9143': 'CjkUnifiedIdeograph9143',
u'\ud245': 'HangulSyllableTut',
u'\u3cc6': 'CjkUnifiedIdeograph3cc6',
u'\u79c8': 'CjkUnifiedIdeograph79c8',
u'\ubaca': 'HangulSyllableMwanh',
u'\ue7cc': 'UnicodeCharacter0xe7cc',
u'\u034f': 'CombiningGraphemeJoiner',
u'\u4c51': 'CjkUnifiedIdeograph4c51',
u'\u8953': 'CjkUnifiedIdeograph8953',
u'\uca55': 'HangulSyllableJjeolg',
u'\u11d8': 'HangulJongseongRieulkhieukh',
u'\u52da': 'CjkUnifiedIdeograph52da',
u'\u9fdc': 'UnicodeCharacter0x9fdc',
u'\u3b5f': 'CjkUnifiedIdeograph3b5f',
u'\ud8de': 'UnicodeCharacter0xd8de',
u'\u6461': 'CjkUnifiedIdeograph6461',
u'\ua163': 'YiSyllableNdux',
u'\ue265': 'UnicodeCharacter0xe265',
u'\u09e8': 'BengaliDigitTwo',
u'\u4aea': 'CjkUnifiedIdeograph4aea',
u'\ub7ec': 'HangulSyllableReo',
u'\uf0ee': 'UnicodeCharacter0xf0ee',
u'\u1c71': 'OlChikiLetterEn',
u'\u5973': 'CjkUnifiedIdeograph5973',
u'\u9a75': 'CjkUnifiedIdeograph9a75',
u'\uc777': 'HangulSyllableIgs',
u'\u21f8': 'RightwardsArrowWithVerticalStroke',
u'\u62fa': 'CjkUnifiedIdeograph62fa',
u'\uaffc': 'HangulSyllableGgwels',
u'\ue8fe': 'UnicodeCharacter0xe8fe',
u'\ud600': 'HangulSyllableHyeo',
u'\uf283': 'UnicodeCharacter0xf283',
u'\u1502': 'CanadianSyllabicsSwaa',
u'\u3185': 'HangulLetterSsanghieuh',
u'\u9f06': 'CjkUnifiedIdeograph9f06',
u'\ubb89': 'HangulSyllableMwig',
u'\u440c': 'CjkUnifiedIdeograph440c',
u'\u608f': 'CjkUnifiedIdeograph608f',
u'\uce10': 'HangulSyllableCyuls',
u'\uea93': 'UnicodeCharacter0xea93',
u'\u0d12': 'MalayalamLetterO',
u'\u2995': 'DoubleLeftArcGreaterthanBracket',
u'\ub716': 'HangulSyllableDdyulm',
u'\u5399': 'CjkUnifiedIdeograph5399',
u'\udd9d': 'UnicodeCharacter0xdd9d',
u'\u7c1c': 'CjkUnifiedIdeograph7c1c',
u'\u189f': 'MongolianLetterManchuAliGaliDdha',
u'\ue620': 'UnicodeCharacter0xe620',
u'\u82a3': 'CjkUnifiedIdeograph82a3',
u'\u2522': 'BoxDrawingsUpLightAndRightDownHeavy',
u'\uaf26': 'HangulSyllableGgyej',
u'\u4ba9': 'CjkUnifiedIdeograph4ba9',
u'\uf5ad': 'UnicodeCharacter0xf5ad',
u'\u142c': 'CanadianSyllabicsIn',
u'\u30af': 'KatakanaLetterKu',
u'\u9e30': 'CjkUnifiedIdeograph9e30',
u'\ubab3': 'HangulSyllableMolb',
u'\u4736': 'CjkUnifiedIdeograph4736',
u'\u63b9': 'CjkUnifiedIdeograph63b9',
u'\uc13a': 'HangulSyllableSegg',
u'\uedbd': 'UnicodeCharacter0xedbd',
u'\u0c3c': 'UnicodeCharacter0xc3c',
u'\u28bf': 'BraillePatternDots1234568',
u'\ub640': 'HangulSyllableDdwass',
u'\u52c3': 'CjkUnifiedIdeograph52c3',
u'\udcc7': 'UnicodeCharacter0xdcc7',
u'\u7f46': 'CjkUnifiedIdeograph7f46',
u'\u1bc9': 'UnicodeCharacter0x1bc9',
u'\uf94a': 'CjkCompatibilityIdeographf94a',
u'\u85cd': 'CjkUnifiedIdeograph85cd',
u'\u244c': 'UnicodeCharacter0x244c',
u'\uae50': 'HangulSyllableGgan',
u'\u4ad3': 'CjkUnifiedIdeograph4ad3',
u'\uf4d7': 'UnicodeCharacter0xf4d7',
u'\u1756': 'UnicodeCharacter0x1756',
u'\u33d9': 'SquarePpm',
u'\u915a': 'CjkUnifiedIdeograph915a',
u'\ubddd': 'HangulSyllableBwilg',
u'\u4660': 'CjkUnifiedIdeograph4660',
u'\u62e3': 'CjkUnifiedIdeograph62e3',
u'\uc064': 'HangulSyllableBbeuls',
u'\uece7': 'UnicodeCharacter0xece7',
u'\u0f66': 'TibetanLetterSa',
u'\u2be9': 'UnicodeCharacter0x2be9',
u'\u896a': 'CjkUnifiedIdeograph896a',
u'\u55ed': 'CjkUnifiedIdeograph55ed',
u'\udff1': 'UnicodeCharacter0xdff1',
u'\u7e70': 'CjkUnifiedIdeograph7e70',
u'\u1af3': 'UnicodeCharacter0x1af3',
u'\uf874': 'UnicodeCharacter0xf874',
u'\u84f7': 'CjkUnifiedIdeograph84f7',
u'\u2776': 'DingbatNegativeCircledDigitOne',
u'\ua17a': 'YiSyllableHne',
u'\u4dfd': 'HexagramForSmallPreponderance',
u'\u7701': 'CjkUnifiedIdeograph7701',
u'\u1680': 'OghamSpaceMark',
u'\uf105': 'UnicodeCharacter0xf105',
u'\u9084': 'CjkUnifiedIdeograph9084',
u'\u3c07': 'CjkUnifiedIdeograph3c07',
u'\ua60b': 'VaiSyllableNg',
u'\u598a': 'CjkUnifiedIdeograph598a',
u'\uc38e': 'HangulSyllableSselm',
u'\u6f11': 'CjkUnifiedIdeograph6f11',
u'\u0e90': 'UnicodeCharacter0xe90',
u'\ue915': 'UnicodeCharacter0xe915',
u'\u8894': 'CjkUnifiedIdeograph8894',
u'\u5e1b': 'CjkUnifiedIdeograph5e1b',
u'\u719a': 'CjkUnifiedIdeograph719a',
u'\ud81f': 'UnicodeCharacter0xd81f',
u'\ufb9e': 'ArabicLetterNoonGhunnaIsolatedForm',
u'\u0721': 'SyriacLetterMim',
u'\u26a0': 'WarningSign',
u'\u8125': 'CjkUnifiedIdeograph8125',
u'\ua0a4': 'YiSyllableHmux',
u'\u762b': 'CjkUnifiedIdeograph762b',
u'\u69aa': 'CjkUnifiedIdeograph69aa',
u'\uf02f': 'UnicodeCharacter0xf02f',
u'\u93ae': 'CjkUnifiedIdeograph93ae',
u'\u3f31': 'CjkUnifiedIdeograph3f31',
u'\ub935': 'HangulSyllableRweng',
u'\u58b4': 'CjkUnifiedIdeograph58b4',
u'\uc2b8': 'HangulSyllableSeuss',
u'\u6e3b': 'CjkUnifiedIdeograph6e3b',
u'\u01ba': 'LatinSmallLetterEzhWithTail',
u'\ue83f': 'UnicodeCharacter0xe83f',
u'\u8bbe': 'CjkUnifiedIdeograph8bbe',
u'\u5145': 'CjkUnifiedIdeograph5145',
u'\u70c4': 'CjkUnifiedIdeograph70c4',
u'\udb49': 'UnicodeCharacter0xdb49',
u'\ufac8': 'CjkCompatibilityIdeographfac8',
u'\u064b': 'ArabicFathatan',
u'\u39ca': 'CjkUnifiedIdeograph39ca',
u'\u804f': 'CjkUnifiedIdeograph804f',
u'\ua3ce': 'YiSyllableRox',
u'\u4955': 'CjkUnifiedIdeograph4955',
u'\u68d4': 'CjkUnifiedIdeograph68d4',
u'\uf359': 'UnicodeCharacter0xf359',
u'\u92d8': 'CjkUnifiedIdeograph92d8',
u'\u3e5b': 'CjkUnifiedIdeograph3e5b',
u'\ub85f': 'HangulSyllableRogs',
u'\u5bde': 'CjkUnifiedIdeograph5bde',
u'\uc5e2': 'HangulSyllableEbs',
u'\u6165': 'CjkUnifiedIdeograph6165',
u'\xe4': 'LatinSmallLetterAWithDiaeresis',
u'\ueb69': 'UnicodeCharacter0xeb69',
u'\u8ae8': 'CjkUnifiedIdeograph8ae8',
u'\u506f': 'CjkUnifiedIdeograph506f',
u'\u73ee': 'CjkUnifiedIdeograph73ee',
u'\uda73': 'UnicodeCharacter0xda73',
u'\ufdf2': 'ArabicLigatureAllahIsolatedForm',
u'\u1975': 'UnicodeCharacter0x1975',
u'\u38f4': 'CjkUnifiedIdeograph38f4',
u'\u8379': 'CjkUnifiedIdeograph8379',
u'\ua2f8': 'YiSyllableNzuox',
u'\u487f': 'CjkUnifiedIdeograph487f',
u'\u6bfe': 'CjkUnifiedIdeograph6bfe',
u'\u1e06': 'LatinCapitalLetterBWithLineBelow',
u'\u3a89': 'CjkUnifiedIdeograph3a89',
u'\u5b08': 'CjkUnifiedIdeograph5b08',
u'\u678b': 'CjkUnifiedIdeograph678b',
u'\u980a': 'CjkUnifiedIdeograph980a',
u'\ua48d': 'UnicodeCharacter0xa48d',
u'\uc50c': 'HangulSyllableSsyi',
u'\ue18f': 'UnicodeCharacter0xe18f',
u'\u3616': 'CjkUnifiedIdeograph3616',
u'\u7318': 'CjkUnifiedIdeograph7318',
u'\u1f9b': 'GreekCapitalLetterEtaWithDasiaAndVariaAndProsgegrammeni',
u'\ub01a': 'HangulSyllableGgwilp',
u'\u5c9d': 'CjkUnifiedIdeograph5c9d',
u'\ufd1c': 'ArabicLigatureHahWithYehFinalForm',
u'\u999f': 'CjkUnifiedIdeograph999f',
u'\uc6a1': 'HangulSyllableYolt',
u'\u2e26': 'LeftSidewaysUBracket',
u'\u6b28': 'CjkUnifiedIdeograph6b28',
u'\u37ab': 'CjkUnifiedIdeograph37ab',
u'\ua82a': 'SylotiNagriPoetryMark3',
u'\u74ad': 'CjkUnifiedIdeograph74ad',
u'\u952c': 'CjkUnifiedIdeograph952c',
u'\ub1af': 'HangulSyllableNwah',
u'\ud22e': 'HangulSyllableTugg',
u'\ufeb1': 'ArabicLetterSeenIsolatedForm',
u'\u0338': 'CombiningLongSolidusOverlay',
u'\u2fbb': 'KangxiRadicalBone',
u'\u403a': 'CjkUnifiedIdeograph403a',
u'\u6cbd': 'CjkUnifiedIdeograph6cbd',
u'\u8d3c': 'CjkUnifiedIdeograph8d3c',
u'\ua9bf': 'UnicodeCharacter0xa9bf',
u'\uca3e': 'HangulSyllableJjyaelp',
u'\u96c1': 'CjkUnifiedIdeograph96c1',
u'\ud3c3': 'HangulSyllablePyeolh',
u'\u3b48': 'CjkUnifiedIdeograph3b48',
u'\u784a': 'CjkUnifiedIdeograph784a',
u'\u04cd': 'CyrillicCapitalLetterEmWithTail',
u'\ua54c': 'VaiSyllableHa',
u'\u41cf': 'CjkUnifiedIdeograph41cf',
u'\ue24e': 'UnicodeCharacter0xe24e',
u'\u8ed1': 'CjkUnifiedIdeograph8ed1',
u'\ucbd3': 'HangulSyllableJjyuh',
u'\u105a': 'MyanmarLetterMonNga',
u'\u3cdd': 'CjkUnifiedIdeograph3cdd',
u'\u5d5c': 'CjkUnifiedIdeograph5d5c',
u'\u79df': 'CjkUnifiedIdeograph79df',
u'\u9a5e': 'CjkUnifiedIdeograph9a5e',
u'\ua6e1': 'UnicodeCharacter0xa6e1',
u'\uc760': 'HangulSyllableYil',
u'\ue3e3': 'UnicodeCharacter0xe3e3',
u'\u086a': 'UnicodeCharacter0x86a',
u'\u756c': 'CjkUnifiedIdeograph756c',
u'\u11ef': 'HangulJongseongIeungkhieukh',
u'\ub26e': 'HangulSyllableNwij',
u'\u5ef1': 'CjkUnifiedIdeograph5ef1',
u'\uff70': 'HalfwidthKatakanahiraganaProlongedSoundMark',
u'\u9bf3': 'CjkUnifiedIdeograph9bf3',
u'\ud8f5': 'UnicodeCharacter0xd8f5',
u'\u207a': 'SuperscriptPlusSign',
u'\u6d7c': 'CjkUnifiedIdeograph6d7c',
u'\u09ff': 'UnicodeCharacter0x9ff',
u'\uaa7e': 'UnicodeCharacter0xaa7e',
u'\u9780': 'CjkUnifiedIdeograph9780',
u'\u3303': 'SquareAaru',
u'\ud482': 'HangulSyllablePulm',
u'\u7005': 'CjkUnifiedIdeograph7005',
u'\ubd07': 'HangulSyllableBos',
u'\ufa09': 'CjkCompatibilityIdeographfa09',
u'\u058c': 'UnicodeCharacter0x58c',
u'\u428e': 'CjkUnifiedIdeograph428e',
u'\u8f90': 'CjkUnifiedIdeograph8f90',
u'\u2b13': 'SquareWithBottomHalfBlack',
u'\ucc92': 'HangulSyllableCyaej',
u'\u6815': 'CjkUnifiedIdeograph6815',
u'\u5517': 'CjkUnifiedIdeograph5517',
u'\u9219': 'CjkUnifiedIdeograph9219',
u'\udf1b': 'UnicodeCharacter0xdf1b',
u'\u3d9c': 'CjkUnifiedIdeograph3d9c',
u'\u7a9e': 'CjkUnifiedIdeograph7a9e',
u'\ua7a0': 'UnicodeCharacter0xa7a0',
u'\ue4a2': 'UnicodeCharacter0xe4a2',
u'\u4d27': 'CjkUnifiedIdeograph4d27',
u'\u8a29': 'CjkUnifiedIdeograph8a29',
u'\uf72b': 'UnicodeCharacter0xf72b',
u'\u12ae': 'EthiopicSyllableKo',
u'\u5fb0': 'CjkUnifiedIdeograph5fb0',
u'\u9cb2': 'CjkUnifiedIdeograph9cb2',
u'\u3835': 'CjkUnifiedIdeograph3835',
u'\ud9b4': 'UnicodeCharacter0xd9b4',
u'\u6537': 'CjkUnifiedIdeograph6537',
u'\ua239': 'YiSyllableMgot',
u'\uef3b': 'UnicodeCharacter0xef3b',
u'\u0abe': 'GujaratiVowelSignAa',
u'\u77c0': 'CjkUnifiedIdeograph77c0',
u'\ub4c2': 'HangulSyllableDyugg',
u'\uf1c4': 'UnicodeCharacter0xf1c4',
u'\u1d47': 'ModifierLetterSmallB',
u'\u5a49': 'CjkUnifiedIdeograph5a49',
u'\u874b': 'CjkUnifiedIdeograph874b',
u'\uc44d': 'HangulSyllableSsyonj',
u'\u22ce': 'CurlyLogicalOr',
u'\u6fd0': 'CjkUnifiedIdeograph6fd0',
u'\uacd2': 'HangulSyllableGyelp',
u'\ue9d4': 'UnicodeCharacter0xe9d4',
u'\u3557': 'CjkUnifiedIdeograph3557',
u'\ud6d6': 'HangulSyllableHubs',
u'\u7259': 'CjkUnifiedIdeograph7259',
u'\ubf5b': 'HangulSyllableBboh',
u'\ufc5d': 'ArabicLigatureAlefMaksuraWithSuperscriptAlefIsolatedForm',
u'\u07e0': 'NkoLetterNaWoloso',
u'\u44e2': 'CjkUnifiedIdeograph44e2',
u'\u81e4': 'CjkUnifiedIdeograph81e4',
u'\u2d67': 'UnicodeCharacter0x2d67',
u'\ucee6': 'HangulSyllableKeogg',
u'\u6a69': 'CjkUnifiedIdeograph6a69',
u'\u576b': 'CjkUnifiedIdeograph576b',
u'\u946d': 'CjkUnifiedIdeograph946d',
u'\ud16f': 'HangulSyllableTyeod',
u'\u3ff0': 'CjkUnifiedIdeograph3ff0',
u'\u7cf2': 'CjkUnifiedIdeograph7cf2',
u'\ub9f4': 'HangulSyllableMaem',
u'\ue6f6': 'UnicodeCharacter0xe6f6',
u'\u0279': 'LatinSmallLetterTurnedR',
u'\u4f7b': 'CjkUnifiedIdeograph4f7b',
u'\u8c7d': 'CjkUnifiedIdeograph8c7d',
u'\uc97f': 'HangulSyllableJyus',
u'\ub981': 'HangulSyllableReult',
u'\u5e04': 'CjkUnifiedIdeograph5e04',
u'\u7287': 'CjkUnifiedIdeograph7287',
u'\ud408': 'HangulSyllablePwa',
u'\ue88b': 'UnicodeCharacter0xe88b',
u'\u170a': 'TagalogLetterBa',
u'\u2b8d': 'UnicodeCharacter0x2b8d',
u'\u8d0e': 'CjkUnifiedIdeograph8d0e',
u'\ua191': 'YiSyllableNut',
u'\u4614': 'CjkUnifiedIdeograph4614',
u'\u1a97': 'UnicodeCharacter0x1a97',
u'\ufc18': 'ArabicLigatureHahWithMeemIsolatedForm',
u'\u909b': 'CjkUnifiedIdeograph909b',
u'\u3f1a': 'CjkUnifiedIdeograph3f1a',
u'\ub51e': 'HangulSyllableDilm',
u'\u49a1': 'CjkUnifiedIdeograph49a1',
u'\ucfa5': 'HangulSyllableKwaet',
u'\u6e24': 'CjkUnifiedIdeograph6e24',
u'\u02a7': 'LatinSmallLetterTeshDigraph',
u'\ue428': 'UnicodeCharacter0xe428',
u'\ub8ab': 'HangulSyllableRwaec',
u'\u272a': 'CircledWhiteStar',
u'\u5d2e': 'CjkUnifiedIdeograph5d2e',
u'\u71b1': 'CjkUnifiedIdeograph71b1',
u'\ud332': 'HangulSyllablePaelm',
u'\uf7b5': 'UnicodeCharacter0xf7b5',
u'\u1634': 'CanadianSyllabicsCarrierTlhu',
u'\u2ab7': 'PrecedesAboveAlmostEqualTo',
u'\u8c38': 'CjkUnifiedIdeograph8c38',
u'\ua0bb': 'YiSyllableMuo',
u'\u453e': 'CjkUnifiedIdeograph453e',
u'\u19c1': 'NewTaiLueLetterFinalV',
u'\ufb42': 'UnicodeCharacter0xfb42',
u'\u9fc5': 'UnicodeCharacter0x9fc5',
u'\u3e44': 'CjkUnifiedIdeograph3e44',
u'\ub448': 'HangulSyllableDyoss',
u'\u48cb': 'CjkUnifiedIdeograph48cb',
u'\ucecf': 'HangulSyllableKyaed',
u'\u6d4e': 'CjkUnifiedIdeograph6d4e',
u'\u01d1': 'LatinCapitalLetterOWithCaron',
u'\ue352': 'UnicodeCharacter0xe352',
u'\u87d5': 'CjkUnifiedIdeograph87d5',
u'\u2654': 'WhiteChessKing',
u'\u5c58': 'CjkUnifiedIdeograph5c58',
u'\u70db': 'CjkUnifiedIdeograph70db',
u'\ud25c': 'HangulSyllableTweoss',
u'\uf6df': 'UnicodeCharacter0xf6df',
u'\u155e': 'CanadianSyllabicsThe',
u'\u29e1': 'IncreasesAs',
u'\u8b62': 'CjkUnifiedIdeograph8b62',
u'\uafe5': 'HangulSyllableGgweob',
u'\u4468': 'CjkUnifiedIdeograph4468',
u'\u18eb': 'UnicodeCharacter0x18eb',
u'\ufa6c': 'UnicodeCharacter0xfa6c',
u'\u9eef': 'CjkUnifiedIdeograph9eef',
u'\u3d6e': 'CjkUnifiedIdeograph3d6e',
u'\ub372': 'HangulSyllableDegg',
u'\u57f5': 'CjkUnifiedIdeograph57f5',
u'\ucdf9': 'HangulSyllableCwib',
u'\u6c78': 'CjkUnifiedIdeograph6c78',
u'\xfb': 'LatinSmallLetterUWithCircumflex',
u'\ue27c': 'UnicodeCharacter0xe27c',
u'\u86ff': 'CjkUnifiedIdeograph86ff',
u'\u257e': 'BoxDrawingsHeavyLeftAndLightRight',
u'\ubc03': 'HangulSyllableMilb',
u'\u5b82': 'CjkUnifiedIdeograph5b82',
u'\ud186': 'HangulSyllableTyegg',
u'\u7509': 'CjkUnifiedIdeograph7509',
u'\u1488': 'CanadianSyllabicsSouthslaveyKah',
u'\ueb0d': 'UnicodeCharacter0xeb0d',
u'\u8a8c': 'CjkUnifiedIdeograph8a8c',
u'\u2e0f': 'Paragraphos',
u'\ua413': 'YiSyllableQyx',
u'\u4392': 'CjkUnifiedIdeograph4392',
u'\uda17': 'UnicodeCharacter0xda17',
u'\uf996': 'CjkCompatibilityIdeographf996',
u'\u1d19': 'LatinLetterSmallCapitalReversedR',
u'\u3c98': 'CjkUnifiedIdeograph3c98',
u'\u931d': 'CjkUnifiedIdeograph931d',
u'\ub29c': 'HangulSyllableNeuls',
u'\u4c23': 'CjkUnifiedIdeograph4c23',
u'\u6ba2': 'CjkUnifiedIdeograph6ba2',
u'\uc227': 'HangulSyllableSulh',
u'\ue1a6': 'UnicodeCharacter0xe1a6',
u'\u0529': 'UnicodeCharacter0x529',
u'\u24a8': 'ParenthesizedLatinSmallLetterM',
u'\ubb2d': 'HangulSyllableMyong',
u'\u5aac': 'CjkUnifiedIdeograph5aac',
u'\ud0b0': 'HangulSyllableKils',
u'\u7433': 'CjkUnifiedIdeograph7433',
u'\u13b2': 'CherokeeLetterHv',
u'\uea37': 'UnicodeCharacter0xea37',
u'\u89b6': 'CjkUnifiedIdeograph89b6',
u'\u2d39': 'TifinaghLetterYadd',
u'\ua33d': 'YiSyllableSsux',
u'\u42bc': 'CjkUnifiedIdeograph42bc',
u'\ud941': 'UnicodeCharacter0xd941',
u'\uf8c0': 'UnicodeCharacter0xf8c0',
u'\u1c43': 'LepchaDigitThree',
u'\u3bc2': 'CjkUnifiedIdeograph3bc2',
u'\u9247': 'CjkUnifiedIdeograph9247',
u'\ub1c6': 'HangulSyllableNwaej',
u'\u4b4d': 'CjkUnifiedIdeograph4b4d',
u'\u6acc': 'CjkUnifiedIdeograph6acc',
u'\uc151': 'HangulSyllableSet',
u'\ue0d0': 'UnicodeCharacter0xe0d0',
u'\u0453': 'CyrillicSmallLetterGje',
u'\u23d2': 'MetricalLongOverShort',
u'\uba57': 'HangulSyllableMegs',
u'\u59d6': 'CjkUnifiedIdeograph59d6',
u'\udfda': 'UnicodeCharacter0xdfda',
u'\u735d': 'CjkUnifiedIdeograph735d',
u'\u12dc': 'EthiopicSyllableZee',
u'\ue961': 'UnicodeCharacter0xe961',
u'\u88e0': 'CjkUnifiedIdeograph88e0',
u'\u2c63': 'LatinCapitalLetterPWithStroke',
u'\ua267': 'YiSyllableNgot',
u'\u41e6': 'CjkUnifiedIdeograph41e6',
u'\ud86b': 'UnicodeCharacter0xd86b',
u'\uc7ea': 'HangulSyllableJyaenh',
u'\u1b6d': 'BalineseMusicalSymbolCombiningKempul',
u'\u3aec': 'CjkUnifiedIdeograph3aec',
u'\u9171': 'CjkUnifiedIdeograph9171',
u'\ub0f0': 'HangulSyllableNyaen',
u'\u4a77': 'CjkUnifiedIdeograph4a77',
u'\u69f6': 'CjkUnifiedIdeograph69f6',
u'\uc07b': 'HangulSyllableBbyid',
u'\ueffa': 'UnicodeCharacter0xeffa',
u'\u037d': 'GreekSmallReversedDottedLunateSigmaSymbol',
u'\u22fc': 'SmallContainsWithVerticalBarAtEndOfHorizontalStroke',
u'\u3881': 'CjkUnifiedIdeograph3881',
u'\u5900': 'CjkUnifiedIdeograph5900',
u'\u7d83': 'CjkUnifiedIdeograph7d83',
u'\u9a02': 'CjkUnifiedIdeograph9a02',
u'\ube85': 'HangulSyllableBbaelg',
u'\udf04': 'UnicodeCharacter0xdf04',
u'\uf387': 'UnicodeCharacter0xf387',
u'\u0c0e': 'TeluguLetterE',
u'\u2091': 'LatinSubscriptSmallLetterE',
u'\u4110': 'CjkUnifiedIdeograph4110',
u'\u6593': 'CjkUnifiedIdeograph6593',
u'\u8212': 'CjkUnifiedIdeograph8212',
u'\ua695': 'CyrillicSmallLetterHwe',
u'\uc714': 'HangulSyllableWim',
u'\u9b97': 'CjkUnifiedIdeograph9b97',
u'\udc99': 'UnicodeCharacter0xdc99',
u'\u341e': 'CjkUnifiedIdeograph341e',
u'\u6920': 'CjkUnifiedIdeograph6920',
u'\u0da3': 'SinhalaLetterMahaapraanaJayanna',
u'\uaa22': 'ChamLetterYa',
u'\u4ea5': 'CjkUnifiedIdeograph4ea5',
u'\uef24': 'UnicodeCharacter0xef24',
u'\u83a7': 'CjkUnifiedIdeograph83a7',
u'\uc4a9': 'HangulSyllableSswelt',
u'\u1130': 'HangulChoseongSiosrieul',
u'\u35b3': 'CjkUnifiedIdeograph35b3',
u'\u5232': 'CjkUnifiedIdeograph5232',
u'\u76b5': 'CjkUnifiedIdeograph76b5',
u'\u9734': 'CjkUnifiedIdeograph9734',
u'\uabb7': 'UnicodeCharacter0xabb7',
u'\uc836': 'HangulSyllableJep',
u'\uecb9': 'UnicodeCharacter0xecb9',
u'\u3940': 'CjkUnifiedIdeograph3940',
u'\u7a42': 'CjkUnifiedIdeograph7a42',
u'\u1ec5': 'LatinSmallLetterEWithCircumflexAndTilde',
u'\ubf44': 'HangulSyllableBbon',
u'\u53c7': 'CjkUnifiedIdeograph53c7',
u'\uf046': 'UnicodeCharacter0xf046',
u'\u94c9': 'CjkUnifiedIdeograph94c9',
u'\uc9cb': 'HangulSyllableJilb',
u'\u2150': 'UnicodeCharacter0x2150',
u'\u6252': 'CjkUnifiedIdeograph6252',
u'\u06d5': 'ArabicLetterAe',
u'\ua754': 'LatinCapitalLetterPWithSquirrelTail',
u'\u7bd7': 'CjkUnifiedIdeograph7bd7',
u'\u9856': 'CjkUnifiedIdeograph9856',
u'\ubcd9': 'HangulSyllableByeg',
u'\udd58': 'UnicodeCharacter0xdd58',
u'\uf1db': 'UnicodeCharacter0xf1db',
u'\u0a62': 'UnicodeCharacter0xa62',
u'\u2ee5': 'CjkRadicalCsimplifiedFish',
u'\u4f64': 'CjkUnifiedIdeograph4f64',
u'\u63e7': 'CjkUnifiedIdeograph63e7',
u'\u8066': 'CjkUnifiedIdeograph8066',
u'\ua4e9': 'UnicodeCharacter0xa4e9',
u'\uc568': 'HangulSyllableAel',
u'\u99eb': 'CjkUnifiedIdeograph99eb',
u'\udaed': 'UnicodeCharacter0xdaed',
u'\u3272': 'CircledHangulMieumA',
u'\u7774': 'CjkUnifiedIdeograph7774',
u'\u0bf7': 'TamilCreditSign',
u'\ua876': 'PhagspaMarkShad',
u'\u4cf9': 'CjkUnifiedIdeograph4cf9',
u'\ued78': 'UnicodeCharacter0xed78',
u'\u81fb': 'CjkUnifiedIdeograph81fb',
u'\uc2fd': 'HangulSyllableSsanj',
u'\uf801': 'UnicodeCharacter0xf801',
u'\u1f84': 'GreekSmallLetterAlphaWithPsiliAndOxiaAndYpogegrammeni',
u'\u5086': 'CjkUnifiedIdeograph5086',
u'\u9588': 'CjkUnifiedIdeograph9588',
u'\u290b': 'DownwardsTripleArrow',
u'\ud68a': 'HangulSyllableHwaep',
u'\u6a0d': 'CjkUnifiedIdeograph6a0d',
u'\uaf0f': 'HangulSyllableGgyeoh',
u'\ue011': 'UnicodeCharacter0xe011',
u'\u0794': 'ThaanaLetterYaa',
u'\u7896': 'CjkUnifiedIdeograph7896',
u'\ubd98': 'HangulSyllableBuk',
u'\ufe9a': 'ArabicLetterThehFinalForm',
u'\u121d': 'EthiopicSyllableMe',
u'\u571f': 'CjkUnifiedIdeograph571f',
u'\u8821': 'CjkUnifiedIdeograph8821',
u'\ucd23': 'HangulSyllableCoh',
u'\u2fa4': 'KangxiRadicalDistinguish',
u'\u60a6': 'CjkUnifiedIdeograph60a6',
u'\ua5a8': 'VaiSyllableLu',
u'\ue6aa': 'UnicodeCharacter0xe6aa',
u'\u3a2d': 'CjkUnifiedIdeograph3a2d',
u'\udbac': 'UnicodeCharacter0xdbac',
u'\u7f2f': 'CjkUnifiedIdeograph7f2f',
u'\ub031': 'HangulSyllableGgyulg',
u'\uf533': 'UnicodeCharacter0xf533',
u'\u08b6': 'UnicodeCharacter0x8b6',
u'\u4db8': 'UnicodeCharacter0x4db8',
u'\u8eba': 'CjkUnifiedIdeograph8eba',
u'\u223d': 'ReversedTilde',
u'\uc3bc': 'HangulSyllableSsye',
u'\u673f': 'CjkUnifiedIdeograph673f',
u'\u5841': 'CjkUnifiedIdeograph5841',
u'\u9d43': 'CjkUnifiedIdeograph9d43',
u'\ude45': 'UnicodeCharacter0xde45',
u'\u30c6': 'KatakanaLetterTe',
u'\u75c8': 'CjkUnifiedIdeograph75c8',
u'\ub6ca': 'HangulSyllableDdweobs',
u'\uebcc': 'UnicodeCharacter0xebcc',
u'\u0f4f': 'TibetanLetterTa',
u'\u4051': 'CjkUnifiedIdeograph4051',
u'\u8553': 'CjkUnifiedIdeograph8553',
u'\uc655': 'HangulSyllableWang',
u'\u1dd8': 'CombiningLatinSmallLetterInsularD',
u'\u5eda': 'CjkUnifiedIdeograph5eda',
u'\u93dc': 'CjkUnifiedIdeograph93dc',
u'\u375f': 'CjkUnifiedIdeograph375f',
u'\ud4de': 'HangulSyllablePwibs',
u'\u6861': 'CjkUnifiedIdeograph6861',
u'\uad63': 'HangulSyllableGyos',
u'\uee65': 'UnicodeCharacter0xee65',
u'\u05e8': 'HebrewLetterResh',
u'\u46ea': 'CjkUnifiedIdeograph46ea',
u'\ubbec': 'HangulSyllableMyim',
u'\ufcee': 'ArabicLigatureNoonWithMeemMedialForm',
u'\u1071': 'MyanmarVowelSignGebaKarenI',
u'\u5573': 'CjkUnifiedIdeograph5573',
u'\u9675': 'CjkUnifiedIdeograph9675',
u'\ucb77': 'HangulSyllableJjweos',
u'\u2df8': 'CombiningCyrillicLetterDjerv',
u'\u6efa': 'CjkUnifiedIdeograph6efa',
u'\ua3fc': 'YiSyllableQit',
u'\ue4fe': 'UnicodeCharacter0xe4fe',
u'\uda00': 'UnicodeCharacter0xda00',
u'\ufe83': 'ArabicLetterAlefWithHamzaAboveIsolatedForm',
u'\u1902': 'LimbuLetterKha',
u'\u3d85': 'CjkUnifiedIdeograph3d85',
u'\u9306': 'CjkUnifiedIdeograph9306',
u'\ub789': 'HangulSyllableRalt',
u'\u480c': 'CjkUnifiedIdeograph480c',
u'\u6c8f': 'CjkUnifiedIdeograph6c8f',
u'\uc210': 'HangulSyllableSyoss',
u'\ue693': 'UnicodeCharacter0xe693',
u'\u0112': 'LatinCapitalLetterEWithMacron',
u'\u2595': 'RightOneEighthBlock',
u'\ubb16': 'HangulSyllableMoep',
u'\u5f99': 'CjkUnifiedIdeograph5f99',
u'\ud19d': 'HangulSyllableTyet',
u'\u701c': 'CjkUnifiedIdeograph701c',
u'\u149f': 'CanadianSyllabicsWestcreeCwaa',
u'\uea20': 'UnicodeCharacter0xea20',
u'\u8ea3': 'CjkUnifiedIdeograph8ea3',
u'\u2922': 'NorthEastAndSouthWestArrow',
u'\ua326': 'YiSyllableSy',
u'\u47a9': 'CjkUnifiedIdeograph47a9',
u'\uf9ad': 'CjkCompatibilityIdeographf9ad',
u'\u182c': 'MongolianLetterQa',
u'\u3caf': 'CjkUnifiedIdeograph3caf',
u'\u9230': 'CjkUnifiedIdeograph9230',
u'\ub6b3': 'HangulSyllableDduc',
u'\u4b36': 'CjkUnifiedIdeograph4b36',
u'\u6fb9': 'CjkUnifiedIdeograph6fb9',
u'\ucd3a': 'HangulSyllableCwaj',
u'\ue1bd': 'UnicodeCharacter0xe1bd',
u'\u24bf': 'CircledLatinCapitalLetterJ',
u'\uba40': 'HangulSyllableMeol',
u'\u5ec3': 'CjkUnifiedIdeograph5ec3',
u'\ud0c7': 'HangulSyllableTad',
u'\u7346': 'CjkUnifiedIdeograph7346',
u'\u17c9': 'KhmerSignMuusikatoan',
u'\uf54a': 'UnicodeCharacter0xf54a',
u'\u89cd': 'CjkUnifiedIdeograph89cd',
u'\u284c': 'BraillePatternDots347',
u'\ua250': 'YiSyllableHxa',
u'\u46d3': 'CjkUnifiedIdeograph46d3',
u'\uf8d7': 'UnicodeCharacter0xf8d7',
u'\u1b56': 'BalineseDigitSix',
u'\u3fd9': 'CjkUnifiedIdeograph3fd9',
u'\u9d5a': 'CjkUnifiedIdeograph9d5a',
u'\ub1dd': 'HangulSyllableNoeb',
u'\u4a60': 'CjkUnifiedIdeograph4a60',
u'\u6ee3': 'CjkUnifiedIdeograph6ee3',
u'\ucc64': 'HangulSyllableCyan',
u'\ue0e7': 'UnicodeCharacter0xe0e7',
u'\u0366': 'CombiningLatinSmallLetterO',
u'\u27e9': 'MathematicalRightAngleBracket',
u'\u856a': 'CjkUnifiedIdeograph856a',
u'\u59ed': 'CjkUnifiedIdeograph59ed',
u'\ud3f1': 'HangulSyllablePonj',
u'\u7270': 'CjkUnifiedIdeograph7270',
u'\u16f3': 'UnicodeCharacter0x16f3',
u'\uf474': 'UnicodeCharacter0xf474',
u'\u88f7': 'CjkUnifiedIdeograph88f7',
u'\u2b76': 'UnicodeCharacter0x2b76',
u'\uad7a': 'HangulSyllableGulp',
u'\u41fd': 'CjkUnifiedIdeograph41fd',
u'\u7b01': 'CjkUnifiedIdeograph7b01',
u'\u1a80': 'UnicodeCharacter0x1a80',
u'\ufd05': 'ArabicLigatureSadWithAlefMaksuraIsolatedForm',
u'\u9c84': 'CjkUnifiedIdeograph9c84',
u'\u3007': 'IdeographicNumberZero',
u'\uaa0b': 'ChamLetterNga',
u'\u558a': 'CjkUnifiedIdeograph558a',
u'\ucf8e': 'HangulSyllableKwaegg',
u'\u6311': 'CjkUnifiedIdeograph6311',
u'\u0290': 'LatinSmallLetterZWithRetroflexHook',
u'\ue515': 'UnicodeCharacter0xe515',
u'\u8494': 'CjkUnifiedIdeograph8494',
u'\u521b': 'CjkUnifiedIdeograph521b',
u'\u7d9a': 'CjkUnifiedIdeograph7d9a',
u'\ud41f': 'HangulSyllablePwac',
u'\uf79e': 'UnicodeCharacter0xf79e',
u'\u0b21': 'OriyaLetterDda',
u'\u2aa0': 'SimilarAboveGreaterthanAboveEqualsSign',
u'\u8d25': 'CjkUnifiedIdeograph8d25',
u'\uaca4': 'HangulSyllableGek',
u'\u7a2b': 'CjkUnifiedIdeograph7a2b',
u'\u65aa': 'CjkUnifiedIdeograph65aa',
u'\ufc2f': 'ArabicLigatureFehWithKhahIsolatedForm',
u'\u9fae': 'CjkUnifiedIdeograph9fae',
u'\u3331': 'SquareBiru',
u'\ub535': 'HangulSyllableDdanj',
u'\u54b4': 'CjkUnifiedIdeograph54b4',
u'\uceb8': 'HangulSyllableKyals',
u'\u623b': 'CjkUnifiedIdeograph623b',
u'\u0dba': 'SinhalaLetterYayanna',
u'\ue43f': 'UnicodeCharacter0xe43f',
u'\u87be': 'CjkUnifiedIdeograph87be',
u'\u5d45': 'CjkUnifiedIdeograph5d45',
u'\u7cc4': 'CjkUnifiedIdeograph7cc4',
u'\ud749': 'HangulSyllableHyung',
u'\uf6c8': 'UnicodeCharacter0xf6c8',
u'\u0a4b': 'GurmukhiVowelSignOo',
u'\u35ca': 'CjkUnifiedIdeograph35ca',
u'\u8c4f': 'CjkUnifiedIdeograph8c4f',
u'\uafce': 'HangulSyllableGguj',
u'\u4555': 'CjkUnifiedIdeograph4555',
u'\u64d4': 'CjkUnifiedIdeograph64d4',
u'\uff59': 'FullwidthLatinSmallLetterY',
u'\u9ed8': 'CjkUnifiedIdeograph9ed8',
u'\u325b': 'CircledNumberThirtyOne',
u'\ub45f': 'HangulSyllableDulh',
u'\u57de': 'CjkUnifiedIdeograph57de',
u'\uc9e2': 'HangulSyllableJjanh',
u'\u6d65': 'CjkUnifiedIdeograph6d65',
u'\u0ce4': 'UnicodeCharacter0xce4',
u'\ue769': 'UnicodeCharacter0xe769',
u'\u86e8': 'CjkUnifiedIdeograph86e8',
u'\u5c6f': 'CjkUnifiedIdeograph5c6f',
u'\u7fee': 'CjkUnifiedIdeograph7fee',
u'\ud673': 'HangulSyllableHwaegs',
u'\uf1f2': 'UnicodeCharacter0xf1f2',
u'\u1575': 'CanadianSyllabicsNunavikHi',
u'\u34f4': 'CjkUnifiedIdeograph34f4',
u'\u8f79': 'CjkUnifiedIdeograph8f79',
u'\uaef8': 'HangulSyllableGgyeon',
u'\u447f': 'CjkUnifiedIdeograph447f',
u'\u67fe': 'CjkUnifiedIdeograph67fe',
u'\u1206': 'EthiopicSyllableHo',
u'\u3689': 'CjkUnifiedIdeograph3689',
u'\u5708': 'CjkUnifiedIdeograph5708',
u'\u6b8b': 'CjkUnifiedIdeograph6b8b',
u'\u940a': 'CjkUnifiedIdeograph940a',
u'\ua88d': 'SaurashtraLetterEe',
u'\uc90c': 'HangulSyllableJum',
u'\ued8f': 'UnicodeCharacter0xed8f',
u'\u3a16': 'CjkUnifiedIdeograph3a16',
u'\u7f18': 'CjkUnifiedIdeograph7f18',
u'\u139b': 'UnicodeCharacter0x139b',
u'\ubc1a': 'HangulSyllableBanh',
u'\u509d': 'CjkUnifiedIdeograph509d',
u'\uf11c': 'UnicodeCharacter0xf11c',
u'\u959f': 'CjkUnifiedIdeograph959f',
u'\ucaa1': 'HangulSyllableJjyeg',
u'\u2226': 'NotParallelTo',
u'\u6728': 'CjkUnifiedIdeograph6728',
u'\u3bab': 'CjkUnifiedIdeograph3bab',
u'\ua42a': 'YiSyllableJjup',
u'\u78ad': 'CjkUnifiedIdeograph78ad',
u'\u992c': 'CjkUnifiedIdeograph992c',
u'\ubdaf': 'HangulSyllableBweos',
u'\ude2e': 'UnicodeCharacter0xde2e',
u'\uf2b1': 'UnicodeCharacter0xf2b1',
u'\u0f38': 'TibetanMarkCheMgo',
u'\u23bb': 'HorizontalScanLine3',
u'\u4c3a': 'CjkUnifiedIdeograph4c3a',
u'\u60bd': 'CjkUnifiedIdeograph60bd',
u'\u813c': 'CjkUnifiedIdeograph813c',
u'\ua5bf': 'VaiSyllableWo',
u'\uc63e': 'HangulSyllableOp',
u'\u9ac1': 'CjkUnifiedIdeograph9ac1',
u'\udfc3': 'UnicodeCharacter0xdfc3',
u'\u3748': 'CjkUnifiedIdeograph3748',
u'\u744a': 'CjkUnifiedIdeograph744a',
u'\u08cd': 'UnicodeCharacter0x8cd',
u'\ua94c': 'RejangVowelSignAu',
u'\u4dcf': 'HexagramForEnthusiasm',
u'\uee4e': 'UnicodeCharacter0xee4e',
u'\u82d1': 'CjkUnifiedIdeograph82d1',
u'\uc7d3': 'HangulSyllableJyalb',
u'\u1c5a': 'OlChikiLetterLa',
u'\u30dd': 'KatakanaLetterPo',
u'\u515c': 'CjkUnifiedIdeograph515c',
u'\u75df': 'CjkUnifiedIdeograph75df',
u'\u965e': 'CjkUnifiedIdeograph965e',
u'\uaae1': 'UnicodeCharacter0xaae1',
u'\ucb60': 'HangulSyllableJjuk',
u'\uefe3': 'UnicodeCharacter0xefe3',
u'\u046a': 'CyrillicCapitalLetterBigYus',
u'\u796c': 'CjkUnifiedIdeograph796c',
u'\u1def': 'UnicodeCharacter0x1def',
u'\ube6e': 'HangulSyllableBbalp',
u'\u52f1': 'CjkUnifiedIdeograph52f1',
u'\uf370': 'UnicodeCharacter0xf370',
u'\u97f3': 'CjkUnifiedIdeograph97f3',
u'\ud4f5': 'HangulSyllablePyult',
u'\u2c7a': 'LatinSmallLetterOWithLowRingInside',
u'\u617c': 'CjkUnifiedIdeograph617c',
u'\u05ff': 'UnicodeCharacter0x5ff',
u'\ua67e': 'CyrillicKavyka',
u'\u9b80': 'CjkUnifiedIdeograph9b80',
u'\u3f03': 'CjkUnifiedIdeograph3f03',
u'\ud882': 'UnicodeCharacter0xd882',
u'\u7c05': 'CjkUnifiedIdeograph7c05',
u'\ub107': 'HangulSyllableNyaeh',
u'\uf609': 'UnicodeCharacter0xf609',
u'\u098c': 'BengaliLetterVocalicL',
u'\u4e8e': 'CjkUnifiedIdeograph4e8e',
u'\u8390': 'CjkUnifiedIdeograph8390',
u'\u2713': 'CheckMark',
u'\uc092': 'HangulSyllableBbigg',
u'\u6415': 'CjkUnifiedIdeograph6415',
u'\u5917': 'CjkUnifiedIdeograph5917',
u'\u9e19': 'CjkUnifiedIdeograph9e19',
u'\ud31b': 'HangulSyllablePalh',
u'\u319c': 'IdeographicAnnotationFourthMark',
u'\u769e': 'CjkUnifiedIdeograph769e',
u'\uaba0': 'UnicodeCharacter0xaba0',
u'\ue8a2': 'UnicodeCharacter0xe8a2',
u'\u0c25': 'TeluguLetterTha',
u'\u4127': 'CjkUnifiedIdeograph4127',
u'\u8629': 'CjkUnifiedIdeograph8629',
u'\ufb2b': 'HebrewLetterShinWithSinDot',
u'\u1eae': 'LatinCapitalLetterAWithBreveAndAcute',
u'\u53b0': 'CjkUnifiedIdeograph53b0',
u'\u90b2': 'CjkUnifiedIdeograph90b2',
u'\u3435': 'CjkUnifiedIdeograph3435',
u'\ud5b4': 'HangulSyllableHyael',
u'\u6937': 'CjkUnifiedIdeograph6937',
u'\uae39': 'HangulSyllableGilg',
u'\ue33b': 'UnicodeCharacter0xe33b',
u'\u06be': 'ArabicLetterHehDoachashmee',
u'\u7bc0': 'CjkUnifiedIdeograph7bc0',
u'\ub8c2': 'HangulSyllableRoebs',
u'\ufdc4': 'ArabicLigatureAinWithJeemWithMeemInitialForm',
u'\u1147': 'HangulChoseongSsangieung',
u'\u5649': 'CjkUnifiedIdeograph5649',
u'\u8b4b': 'CjkUnifiedIdeograph8b4b',
u'\uc84d': 'HangulSyllableJyeong',
u'\u2ece': 'CjkRadicalWalkTwo',
u'\u63d0': 'CjkUnifiedIdeograph63d0',
u'\ua0d2': 'YiSyllableFax',
u'\ue5d4': 'UnicodeCharacter0xe5d4',
u'\u3957': 'CjkUnifiedIdeograph3957',
u'\udad6': 'UnicodeCharacter0xdad6',
u'\u7e59': 'CjkUnifiedIdeograph7e59',
u'\ub35b': 'HangulSyllableDeod',
u'\uf05d': 'UnicodeCharacter0xf05d',
u'\u0be0': 'UnicodeCharacter0xbe0',
u'\u48e2': 'CjkUnifiedIdeograph48e2',
u'\u8de4': 'CjkUnifiedIdeograph8de4',
u'\u2167': 'RomanNumeralEight',
u'\uc2e6': 'HangulSyllableSilm',
u'\u6669': 'CjkUnifiedIdeograph6669',
u'\u5b6b': 'CjkUnifiedIdeograph5b6b',
u'\u986d': 'CjkUnifiedIdeograph986d',
u'\udd6f': 'UnicodeCharacter0xdd6f',
u'\u33f0': 'IdeographicTelegraphSymbolForDaySeventeen',
u'\u70f2': 'CjkUnifiedIdeograph70f2',
u'\ub5f4': 'HangulSyllableDdye',
u'\ueaf6': 'UnicodeCharacter0xeaf6',
u'\u0e79': 'UnicodeCharacter0xe79',
u'\u437b': 'CjkUnifiedIdeograph437b',
u'\u807d': 'CjkUnifiedIdeograph807d',
u'\uc57f': 'HangulSyllableYags',
u'\ubd81': 'HangulSyllableBug',
u'\u5a04': 'CjkUnifiedIdeograph5a04',
u'\u7687': 'CjkUnifiedIdeograph7687',
u'\ud008': 'HangulSyllableKweols',
u'\uec8b': 'UnicodeCharacter0xec8b',
u'\u130a': 'EthiopicSyllableGi',
u'\u2f8d': 'KangxiRadicalInsect',
u'\u890e': 'CjkUnifiedIdeograph890e',
u'\ua591': 'VaiSyllableGoo',
u'\u4214': 'CjkUnifiedIdeograph4214',
u'\u1e97': 'LatinSmallLetterTWithDiaeresis',
u'\uf818': 'UnicodeCharacter0xf818',
u'\u949b': 'CjkUnifiedIdeograph949b',
u'\u3b1a': 'CjkUnifiedIdeograph3b1a',
u'\ub11e': 'HangulSyllableNeoj',
u'\u4da1': 'CjkUnifiedIdeograph4da1',
u'\ucba5': 'HangulSyllableJjwilg',
u'\u6a24': 'CjkUnifiedIdeograph6a24',
u'\u06a7': 'ArabicLetterQafWithDotAbove',
u'\ue028': 'UnicodeCharacter0xe028',
u'\ubcab': 'HangulSyllableBelb',
u'\u232a': 'RightpointingAngleBracket',
u'\u592e': 'CjkUnifiedIdeograph592e',
u'\u75b1': 'CjkUnifiedIdeograph75b1',
u'\ud732': 'HangulSyllableHwip',
u'\uf3b5': 'UnicodeCharacter0xf3b5',
u'\u1234': 'EthiopicSyllableSee',
u'\u2eb7': 'CjkRadicalRam',
u'\u8838': 'CjkUnifiedIdeograph8838',
u'\ua4bb': 'YiRadicalLie',
u'\u413e': 'CjkUnifiedIdeograph413e',
u'\u1dc1': 'CombiningDottedAcuteAccent',
u'\uff42': 'FullwidthLatinSmallLetterB',
u'\u9bc5': 'CjkUnifiedIdeograph9bc5',
u'\u3a44': 'CjkUnifiedIdeograph3a44',
u'\ub048': 'HangulSyllableGgeun',
u'\u4ccb': 'CjkUnifiedIdeograph4ccb',
u'\ucacf': 'HangulSyllableJjos',
u'\u694e': 'CjkUnifiedIdeograph694e',
u'\u05d1': 'HebrewLetterBet',
u'\ue752': 'UnicodeCharacter0xe752',
u'\u83d5': 'CjkUnifiedIdeograph83d5',
u'\u2254': 'ColonEquals',
u'\u5858': 'CjkUnifiedIdeograph5858',
u'\u74db': 'CjkUnifiedIdeograph74db',
u'\ud65c': 'HangulSyllableHwal',
u'\uf2df': 'UnicodeCharacter0xf2df',
u'\u115e': 'UnicodeCharacter0x115e',
u'\u2de1': 'CombiningCyrillicLetterVe',
u'\u8f62': 'CjkUnifiedIdeograph8f62',
u'\uabe5': 'UnicodeCharacter0xabe5',
u'\u4068': 'CjkUnifiedIdeograph4068',
u'\u1ceb': 'UnicodeCharacter0x1ceb',
u'\ufe6c': 'UnicodeCharacter0xfe6c',
u'\u9aef': 'CjkUnifiedIdeograph9aef',
u'\u396e': 'CjkUnifiedIdeograph396e',
u'\ub772': 'HangulSyllableDdibs',
u'\u53f5': 'CjkUnifiedIdeograph53f5',
u'\uc9f9': 'HangulSyllableJjaeg',
u'\u6878': 'CjkUnifiedIdeograph6878',
u'\u04fb': 'CyrillicSmallLetterGheWithStrokeAndHook',
u'\ue67c': 'UnicodeCharacter0xe67c',
u'\u82ff': 'CjkUnifiedIdeograph82ff',
u'\u217e': 'SmallRomanNumeralFiveHundred',
u'\ub803': 'HangulSyllableReoc',
u'\u5f82': 'CjkUnifiedIdeograph5f82',
u'\ud586': 'HangulSyllableHaebs',
u'\u7109': 'CjkUnifiedIdeograph7109',
u'\u1088': 'MyanmarSignShanTone3',
u'\uef0d': 'UnicodeCharacter0xef0d',
u'\u8e8c': 'CjkUnifiedIdeograph8e8c',
u'\u2a0f': 'IntegralAverageWithSlash',
u'\ua013': 'YiSyllableEx',
u'\u4792': 'CjkUnifiedIdeograph4792',
u'\ude17': 'UnicodeCharacter0xde17',
u'\ufd96': 'ArabicLigatureNoonWithHahWithAlefMaksuraFinalForm',
u'\u1919': 'LimbuLetterSha',
u'\u3898': 'CjkUnifiedIdeograph3898',
u'\u971d': 'CjkUnifiedIdeograph971d',
u'\ub69c': 'HangulSyllableDdu',
u'\u4823': 'CjkUnifiedIdeograph4823',
u'\u6fa2': 'CjkUnifiedIdeograph6fa2',
u'\uc627': 'HangulSyllableOgs',
u'\ue5a6': 'UnicodeCharacter0xe5a6',
u'\u0129': 'LatinSmallLetterIWithTilde',
u'\u20a8': 'RupeeSign',
u'\ubf2d': 'HangulSyllableBbyelg',
u'\u5eac': 'CjkUnifiedIdeograph5eac',
u'\ud4b0': 'HangulSyllablePwe',
u'\u7033': 'CjkUnifiedIdeograph7033',
u'\u17b2': 'KhmerIndependentVowelQooTypeTwo',
u'\uee37': 'UnicodeCharacter0xee37',
u'\u8db6': 'CjkUnifiedIdeograph8db6',
u'\u2939': 'LeftsideArcAnticlockwiseArrow',
u'\ua73d': 'LatinSmallLetterAy',
u'\u46bc': 'CjkUnifiedIdeograph46bc',
u'\udd41': 'UnicodeCharacter0xdd41',
u'\ufcc0': 'ArabicLigatureFehWithKhahInitialForm',
u'\u1843': 'MongolianLetterTodoLongVowelSign',
u'\u3fc2': 'CjkUnifiedIdeograph3fc2',
u'\u9647': 'CjkUnifiedIdeograph9647',
u'\ub5c6': 'HangulSyllableDdelm',
u'\u4f4d': 'CjkUnifiedIdeograph4f4d',
u'\u6ecc': 'CjkUnifiedIdeograph6ecc',
u'\uc551': 'HangulSyllableAlt',
u'\ue4d0': 'UnicodeCharacter0xe4d0',
u'\u27d2': 'ElementOfOpeningUpwards',
u'\ube57': 'HangulSyllableBis',
u'\u5dd6': 'CjkUnifiedIdeograph5dd6',
u'\udbda': 'UnicodeCharacter0xdbda',
u'\u775d': 'CjkUnifiedIdeograph775d',
u'\u16dc': 'RunicLetterIngwaz',
u'\ued61': 'UnicodeCharacter0xed61',
u'\u8ce0': 'CjkUnifiedIdeograph8ce0',
u'\u2863': 'BraillePatternDots1267',
u'\ua667': 'CyrillicSmallLetterSoftEm',
u'\u45e6': 'CjkUnifiedIdeograph45e6',
u'\udc6b': 'UnicodeCharacter0xdc6b',
u'\uc3ea': 'HangulSyllableSsobs',
u'\u1f6d': 'GreekCapitalLetterOmegaWithDasiaAndOxia',
u'\u3eec': 'CjkUnifiedIdeograph3eec',
u'\u9571': 'CjkUnifiedIdeograph9571',
u'\ub4f0': 'HangulSyllableDeuss',
u'\u4e77': 'CjkUnifiedIdeograph4e77',
u'\u6df6': 'CjkUnifiedIdeograph6df6',
u'\uc47b': 'HangulSyllableSsuc',
u'\uebfa': 'UnicodeCharacter0xebfa',
u'\u077d': 'ArabicLetterSeenWithExtendedArabicindicDigitFourAbove',
u'\u26fc': 'UnicodeCharacter0x26fc',
u'\u3c81': 'CjkUnifiedIdeograph3c81',
u'\u5d00': 'CjkUnifiedIdeograph5d00',
u'\u7983': 'CjkUnifiedIdeograph7983',
u'\u9e02': 'CjkUnifiedIdeograph9e02',
u'\uba85': 'HangulSyllableMyeong',
u'\udb04': 'UnicodeCharacter0xdb04',
u'\uf787': 'UnicodeCharacter0xf787',
u'\u080e': 'UnicodeCharacter0x80e',
u'\u2491': 'NumberTenFullStop',
u'\u4510': 'CjkUnifiedIdeograph4510',
u'\u6193': 'CjkUnifiedIdeograph6193',
u'\u8612': 'CjkUnifiedIdeograph8612',
u'\ua295': 'YiSyllableZax',
u'\uc314': 'HangulSyllableSsae',
u'\u9f97': 'CjkUnifiedIdeograph9f97',
u'\ud899': 'UnicodeCharacter0xd899',
u'\u301e': 'DoublePrimeQuotationMark',
u'\u6d20': 'CjkUnifiedIdeograph6d20',
u'\u09a3': 'BengaliLetterNna',
u'\uae22': 'HangulSyllableGyilp',
u'\u4aa5': 'CjkUnifiedIdeograph4aa5',
u'\ueb24': 'UnicodeCharacter0xeb24',
u'\u87a7': 'CjkUnifiedIdeograph87a7',
u'\uc0a9': 'HangulSyllableBbit',
u'\u1530': 'CanadianSyllabicsWestcreeYwe',
u'\u31b3': 'BopomofoLetterInnn',
u'\u5632': 'CjkUnifiedIdeograph5632',
u'\u72b5': 'CjkUnifiedIdeograph72b5',
u'\u9334': 'CjkUnifiedIdeograph9334',
u'\uafb7': 'HangulSyllableGgyoh',
u'\ucc36': 'HangulSyllableCalp',
u'\ue8b9': 'UnicodeCharacter0xe8b9',
u'\u3d40': 'CjkUnifiedIdeograph3d40',
u'\u7e42': 'CjkUnifiedIdeograph7e42',
u'\u1ac5': 'UnicodeCharacter0x1ac5',
u'\ubb44': 'HangulSyllableMum',
u'\u57c7': 'CjkUnifiedIdeograph57c7',
u'\uf446': 'UnicodeCharacter0xf446',
u'\u90c9': 'CjkUnifiedIdeograph90c9',
u'\ucdcb': 'HangulSyllableCweoh',
u'\u2550': 'BoxDrawingsDoubleHorizontal',
u'\u6652': 'CjkUnifiedIdeograph6652',
u'\u02d5': 'ModifierLetterDownTack',
u'\ua354': 'YiSyllableZhep',
u'\u7fd7': 'CjkUnifiedIdeograph7fd7',
u'\u9c56': 'CjkUnifiedIdeograph9c56',
u'\ub8d9': 'HangulSyllableRyolt',
u'\ud958': 'UnicodeCharacter0xd958',
u'\uf5db': 'UnicodeCharacter0xf5db',
u'\u0e62': 'UnicodeCharacter0xe62',
u'\u2ae5': 'DoubleVerticalBarDoubleLeftTurnstile',
u'\u4b64': 'CjkUnifiedIdeograph4b64',
u'\u67e7': 'CjkUnifiedIdeograph67e7',
u'\u8466': 'CjkUnifiedIdeograph8466',
u'\ua0e9': 'YiSyllableViep',
u'\uc168': 'HangulSyllableSyeoss',
u'\u9deb': 'CjkUnifiedIdeograph9deb',
u'\udeed': 'UnicodeCharacter0xdeed',
u'\u3672': 'CjkUnifiedIdeograph3672',
u'\u7374': 'CjkUnifiedIdeograph7374',
u'\u0ff7': 'UnicodeCharacter0xff7',
u'\uac76': 'HangulSyllableGeonh',
u'\u48f9': 'CjkUnifiedIdeograph48f9',
u'\ue978': 'UnicodeCharacter0xe978',
u'\u85fb': 'CjkUnifiedIdeograph85fb',
u'\uc6fd': 'HangulSyllableWeng',
u'\ufc01': 'ArabicLigatureYehWithHamzaAboveWithHahIsolatedForm',
u'\u1b84': 'SundaneseLetterI',
u'\u5486': 'CjkUnifiedIdeograph5486',
u'\u9188': 'CjkUnifiedIdeograph9188',
u'\u2d0b': 'GeorgianSmallLetterMan',
u'\ud28a': 'HangulSyllableTwilm',
u'\u6e0d': 'CjkUnifiedIdeograph6e0d',
u'\uab0f': 'UnicodeCharacter0xab0f',
u'\ue411': 'UnicodeCharacter0xe411',
u'\u0394': 'GreekCapitalLetterDelta',
u'\u7c96': 'CjkUnifiedIdeograph7c96',
u'\ub998': 'HangulSyllableRyil',
u'\ufa9a': 'CjkCompatibilityIdeographfa9a',
u'\u161d': 'CanadianSyllabicsCarrierJjo',
u'\u531f': 'CjkUnifiedIdeograph531f',
u'\u8c21': 'CjkUnifiedIdeograph8c21',
u'\uc923': 'HangulSyllableJweolb',
u'\u2ba4': 'UnicodeCharacter0x2ba4',
u'\u64a6': 'CjkUnifiedIdeograph64a6',
u'\ua1a8': 'YiSyllableHlex',
u'\ue2aa': 'UnicodeCharacter0xe2aa',
u'\u3e2d': 'CjkUnifiedIdeograph3e2d',
u'\udfac': 'UnicodeCharacter0xdfac',
u'\u7b2f': 'CjkUnifiedIdeograph7b2f',
u'\ub431': 'HangulSyllableDoet',
u'\uf133': 'UnicodeCharacter0xf133',
u'\u0cb6': 'KannadaLetterSha',
u'\u49b8': 'CjkUnifiedIdeograph49b8',
u'\u8aba': 'CjkUnifiedIdeograph8aba',
u'\u263d': 'FirstQuarterMoon',
u'\uc7bc': 'HangulSyllableJaem',
u'\u633f': 'CjkUnifiedIdeograph633f',
u'\u5c41': 'CjkUnifiedIdeograph5c41',
u'\u9943': 'CjkUnifiedIdeograph9943',
u'\uda45': 'UnicodeCharacter0xda45',
u'\u34c6': 'CjkUnifiedIdeograph34c6',
u'\u71c8': 'CjkUnifiedIdeograph71c8',
u'\ub2ca': 'HangulSyllableNigg',
u'\uefcc': 'UnicodeCharacter0xefcc',
u'\u0b4f': 'UnicodeCharacter0xb4f',
u'\u4451': 'CjkUnifiedIdeograph4451',
u'\u8153': 'CjkUnifiedIdeograph8153',
u'\uc255': 'HangulSyllableSwenj',
u'\u19d8': 'NewTaiLueDigitEight',
u'\u5ada': 'CjkUnifiedIdeograph5ada',
u'\u97dc': 'CjkUnifiedIdeograph97dc',
u'\u335f': 'IdeographicTelegraphSymbolForHourSeven',
u'\ud0de': 'HangulSyllableTaegg',
u'\u6c61': 'CjkUnifiedIdeograph6c61',
u'\ua963': 'UnicodeCharacter0xa963',
u'\uea65': 'UnicodeCharacter0xea65',
u'\u01e8': 'LatinCapitalLetterKWithCaron',
u'\u42ea': 'CjkUnifiedIdeograph42ea',
u'\ubfec': 'HangulSyllableBbweon',
u'\uf8ee': 'UnicodeCharacter0xf8ee',
u'\u1471': 'CanadianSyllabicsYcreeKoo',
u'\u5173': 'CjkUnifiedIdeograph5173',
u'\u9275': 'CjkUnifiedIdeograph9275',
u'\ucf77': 'HangulSyllableKwad',
u'\u29f8': 'BigSolidus',
u'\u6afa': 'CjkUnifiedIdeograph6afa',
u'\ua7fc': 'LatinEpigraphicLetterReversedP',
u'\ue0fe': 'UnicodeCharacter0xe0fe',
u'\ude00': 'UnicodeCharacter0xde00',
u'\ufa83': 'CjkCompatibilityIdeographfa83',
u'\u1d02': 'LatinSmallLetterTurnedAe',
u'\u3985': 'CjkUnifiedIdeograph3985',
u'\u9706': 'CjkUnifiedIdeograph9706',
u'\ub389': 'HangulSyllableDet',
u'\u4c0c': 'CjkUnifiedIdeograph4c0c',
u'\u688f': 'CjkUnifiedIdeograph688f',
u'\uc610': 'HangulSyllableYel',
u'\ue293': 'UnicodeCharacter0xe293',
u'\u0512': 'CyrillicCapitalLetterElWithHook',
u'\u2195': 'UpDownArrow',
u'\ubf16': 'HangulSyllableBbyeolp',
u'\u5b99': 'CjkUnifiedIdeograph5b99',
u'\ud59d': 'HangulSyllableHyalt',
u'\u741c': 'CjkUnifiedIdeograph741c',
u'\u109f': 'MyanmarSymbolShanExclamation',
u'\uee20': 'UnicodeCharacter0xee20',
u'\u8aa3': 'CjkUnifiedIdeograph8aa3',
u'\u2d22': 'GeorgianSmallLetterHie',
u'\ua726': 'LatinCapitalLetterHeng',
u'\u43a9': 'CjkUnifiedIdeograph43a9',
u'\ufdad': 'ArabicLigatureLamWithMeemWithYehFinalForm',
u'\u1c2c': 'LepchaVowelSignE',
u'\u38af': 'CjkUnifiedIdeograph38af',
u'\u9630': 'CjkUnifiedIdeograph9630',
u'\ub2b3': 'HangulSyllableNyid',
u'\u4f36': 'CjkUnifiedIdeograph4f36',
u'\u6bb9': 'CjkUnifiedIdeograph6bb9',
u'\uc93a': 'HangulSyllableJwenh',
u'\ue5bd': 'UnicodeCharacter0xe5bd',
u'\u043c': 'CyrillicSmallLetterEm',
u'\u20bf': 'UnicodeCharacter0x20bf',
u'\ube40': 'HangulSyllableByik',
u'\u5ac3': 'CjkUnifiedIdeograph5ac3',
u'\ud4c7': 'HangulSyllablePwec',
u'\u7746': 'CjkUnifiedIdeograph7746',
u'\u13c9': 'CherokeeLetterQuo',
u'\uf14a': 'UnicodeCharacter0xf14a',
u'\u8dcd': 'CjkUnifiedIdeograph8dcd',
u'\u2c4c': 'GlagoliticSmallLetterTsi',
u'\ua650': 'CyrillicCapitalLetterYeruWithBackYer',
u'\u42d3': 'CjkUnifiedIdeograph42d3',
u'\ufcd7': 'ArabicLigatureHehWithJeemInitialForm',
u'\u1f56': 'GreekSmallLetterUpsilonWithPsiliAndPerispomeni',
u'\u3bd9': 'CjkUnifiedIdeograph3bd9',
u'\u995a': 'CjkUnifiedIdeograph995a',
u'\ub5dd': 'HangulSyllableDdyeonj',
u'\u4e60': 'CjkUnifiedIdeograph4e60',
u'\u6ae3': 'CjkUnifiedIdeograph6ae3',
u'\uc864': 'HangulSyllableJyem',
u'\ue4e7': 'UnicodeCharacter0xe4e7',
u'\u0766': 'ArabicLetterMeemWithDotBelow',
u'\u23e9': 'UnicodeCharacter0x23e9',
u'\u816a': 'CjkUnifiedIdeograph816a',
u'\u5ded': 'CjkUnifiedIdeograph5ded',
u'\ud7f1': 'UnicodeCharacter0xd7f1',
u'\u7670': 'CjkUnifiedIdeograph7670',
u'\u12f3': 'EthiopicSyllableDaa',
u'\uf074': 'UnicodeCharacter0xf074',
u'\u8cf7': 'CjkUnifiedIdeograph8cf7',
u'\u2f76': 'KangxiRadicalRice',
u'\ua97a': 'UnicodeCharacter0xa97a',
u'\u45fd': 'CjkUnifiedIdeograph45fd',
u'\u7f01': 'CjkUnifiedIdeograph7f01',
u'\u1e80': 'LatinCapitalLetterWWithGrave',
u'\uf905': 'CjkCompatibilityIdeographf905',
u'\u9884': 'CjkUnifiedIdeograph9884',
u'\u3407': 'CjkUnifiedIdeograph3407',
u'\uae0b': 'HangulSyllableGeus',
u'\u518a': 'CjkUnifiedIdeograph518a',
u'\ucb8e': 'HangulSyllableJjwelp',
u'\u6711': 'CjkUnifiedIdeograph6711',
u'\u0690': 'ArabicLetterDalWithFourDotsAbove',
u'\ue115': 'UnicodeCharacter0xe115',
u'\u8094': 'CjkUnifiedIdeograph8094',
u'\u561b': 'CjkUnifiedIdeograph561b',
u'\u799a': 'CjkUnifiedIdeograph799a',
u'\ud01f': 'HangulSyllableKwed',
u'\uf39e': 'UnicodeCharacter0xf39e',
u'\u0f21': 'TibetanDigitOne',
u'\u2ea0': 'CjkRadicalCivilian',
u'\u8925': 'CjkUnifiedIdeograph8925',
u'\ua8a4': 'SaurashtraLetterDha',
u'\u7e2b': 'CjkUnifiedIdeograph7e2b',
u'\u61aa': 'CjkUnifiedIdeograph61aa',
u'\uf82f': 'UnicodeCharacter0xf82f',
u'\u9bae': 'CjkUnifiedIdeograph9bae',
u'\u3731': 'CjkUnifiedIdeograph3731',
u'\ub135': 'HangulSyllableNeb',
u'\u50b4': 'CjkUnifiedIdeograph50b4',
u'\ucab8': 'HangulSyllableJjyek',
u'\u663b': 'CjkUnifiedIdeograph663b',
u'\u09ba': 'UnicodeCharacter0x9ba',
u'\ue03f': 'UnicodeCharacter0xe03f',
u'\u83be': 'CjkUnifiedIdeograph83be',
u'\u5945': 'CjkUnifiedIdeograph5945',
u'\u78c4': 'CjkUnifiedIdeograph78c4',
u'\ud349': 'HangulSyllablePyanj',
u'\uf2c8': 'UnicodeCharacter0xf2c8',
u'\u0e4b': 'ThaiCharacterMaiChattawa',
u'\u31ca': 'CjkStrokeHzt',
u'\u884f': 'CjkUnifiedIdeograph884f',
u'\uabce': 'UnicodeCharacter0xabce',
u'\u4155': 'CjkUnifiedIdeograph4155',
u'\u60d4': 'CjkUnifiedIdeograph60d4',
u'\ufb59': 'ArabicLetterPehMedialForm',
u'\u9ad8': 'CjkUnifiedIdeograph9ad8',
u'\u365b': 'CjkUnifiedIdeograph365b',
u'\ub05f': 'HangulSyllableGgeuh',
u'\u53de': 'CjkUnifiedIdeograph53de',
u'\ucde2': 'HangulSyllableCwej',
u'\u6965': 'CjkUnifiedIdeograph6965',
u'\u08e4': 'UnicodeCharacter0x8e4',
u'\ue369': 'UnicodeCharacter0xe369',
u'\u82e8': 'CjkUnifiedIdeograph82e8',
u'\u586f': 'CjkUnifiedIdeograph586f',
u'\u7bee': 'CjkUnifiedIdeograph7bee',
u'\ud273': 'HangulSyllableTwelh',
u'\uf5f2': 'UnicodeCharacter0xf5f2',
u'\u1175': 'HangulJungseongI',
u'\u30f4': 'KatakanaLetterVu',
u'\u8b79': 'CjkUnifiedIdeograph8b79',
u'\uaaf8': 'UnicodeCharacter0xaaf8',
u'\u407f': 'CjkUnifiedIdeograph407f',
u'\u63fe': 'CjkUnifiedIdeograph63fe',
u'\u1606': 'CanadianSyllabicsCarrierNi',
u'\u3289': 'CircledIdeographTen',
u'\u5308': 'CjkUnifiedIdeograph5308',
u'\u6f8b': 'CjkUnifiedIdeograph6f8b',
u'\u900a': 'CjkUnifiedIdeograph900a',
u'\uac8d': 'HangulSyllableGeg',
u'\ucd0c': 'HangulSyllableCon',
u'\ue98f': 'UnicodeCharacter0xe98f',
u'\u3e16': 'CjkUnifiedIdeograph3e16',
u'\u7b18': 'CjkUnifiedIdeograph7b18',
u'\u179b': 'KhmerLetterLo',
u'\ub81a': 'HangulSyllableRebs',
u'\u549d': 'CjkUnifiedIdeograph549d',
u'\uf51c': 'UnicodeCharacter0xf51c',
u'\u919f': 'CjkUnifiedIdeograph919f',
u'\ucea1': 'HangulSyllableKaeb',
u'\u2626': 'OrthodoxCross',
u'\u6328': 'CjkUnifiedIdeograph6328',
u'\u3fab': 'CjkUnifiedIdeograph3fab',
u'\ua02a': 'YiSyllableBe',
u'\u7cad': 'CjkUnifiedIdeograph7cad',
u'\u9d2c': 'CjkUnifiedIdeograph9d2c',
u'\ub9af': 'HangulSyllableRigs',
u'\uda2e': 'UnicodeCharacter0xda2e',
u'\uf6b1': 'UnicodeCharacter0xf6b1',
u'\u0b38': 'OriyaLetterSa',
u'\u27bb': 'HeavyTeardropshankedRightwardsArrow',
u'\u483a': 'CjkUnifiedIdeograph483a',
u'\u64bd': 'CjkUnifiedIdeograph64bd',
u'\u853c': 'CjkUnifiedIdeograph853c',
u'\ua1bf': 'YiSyllableLat',
u'\uc23e': 'HangulSyllableSweolm',
u'\u9ec1': 'CjkUnifiedIdeograph9ec1',
u'\udbc3': 'UnicodeCharacter0xdbc3',
u'\u3348': 'SquareMikuron',
u'\u704a': 'CjkUnifiedIdeograph704a',
u'\u0ccd': 'KannadaSignVirama',
u'\uad4c': 'HangulSyllableGoek',
u'\u49cf': 'CjkUnifiedIdeograph49cf',
u'\uea4e': 'UnicodeCharacter0xea4e',
u'\u86d1': 'CjkUnifiedIdeograph86d1',
u'\uc3d3': 'HangulSyllableSsyec',
u'\u185a': 'MongolianLetterTodoJia',
u'\u34dd': 'CjkUnifiedIdeograph34dd',
u'\u555c': 'CjkUnifiedIdeograph555c',
u'\u71df': 'CjkUnifiedIdeograph71df',
u'\u925e': 'CjkUnifiedIdeograph925e',
u'\uaee1': 'HangulSyllableGgelg',
u'\ucf60': 'HangulSyllableKols',
u'\uebe3': 'UnicodeCharacter0xebe3',
u'\u7d6c': 'CjkUnifiedIdeograph7d6c',
u'\u19ef': 'KhmerSymbolDappramKoet',
u'\uba6e': 'HangulSyllableMep',
u'\u56f1': 'CjkUnifiedIdeograph56f1',
u'\uf770': 'UnicodeCharacter0xf770',
u'\u93f3': 'CjkUnifiedIdeograph93f3',
u'\ud0f5': 'HangulSyllableTaet',
u'\u287a': 'BraillePatternDots24567',
u'\u657c': 'CjkUnifiedIdeograph657c',
u'\u01ff': 'LatinSmallLetterOWithStrokeAndAcute',
u'\ua27e': 'YiSyllableHe',
u'\u9f80': 'CjkUnifiedIdeograph9f80',
u'\u3b03': 'CjkUnifiedIdeograph3b03',
u'\udc82': 'UnicodeCharacter0xdc82',
u'\u7805': 'CjkUnifiedIdeograph7805',
u'\ub507': 'HangulSyllableDyilh',
u'\uf209': 'UnicodeCharacter0xf209',
u'\u0d8c': 'SinhalaLetterUuyanna',
u'\u4a8e': 'CjkUnifiedIdeograph4a8e',
u'\u8790': 'CjkUnifiedIdeograph8790',
u'\u2313': 'Segment',
u'\uc492': 'HangulSyllableSsweobs',
u'\u6015': 'CjkUnifiedIdeograph6015',
u'\u5d17': 'CjkUnifiedIdeograph5d17',
u'\u9a19': 'CjkUnifiedIdeograph9a19',
u'\ud71b': 'HangulSyllableHwigs',
u'\u359c': 'CjkUnifiedIdeograph359c',
u'\u729e': 'CjkUnifiedIdeograph729e',
u'\uafa0': 'HangulSyllableGgyon',
u'\ueca2': 'UnicodeCharacter0xeca2',
u'\u0825': 'UnicodeCharacter0x825',
u'\u4527': 'CjkUnifiedIdeograph4527',
u'\u8229': 'CjkUnifiedIdeograph8229',
u'\uff2b': 'FullwidthLatinCapitalLetterK',
u'\u1aae': 'UnicodeCharacter0x1aae',
u'\u57b0': 'CjkUnifiedIdeograph57b0',
u'\u94b2': 'CjkUnifiedIdeograph94b2',
u'\u3035': 'VerticalKanaRepeatMarkLowerHalf',
u'\ud1b4': 'HangulSyllableToss',
u'\u6d37': 'CjkUnifiedIdeograph6d37',
u'\uaa39': 'UnicodeCharacter0xaa39',
u'\ue73b': 'UnicodeCharacter0xe73b',
u'\u02be': 'ModifierLetterRightHalfRing',
u'\u7fc0': 'CjkUnifiedIdeograph7fc0',
u'\ubcc2': 'HangulSyllableByeonh',
u'\uf9c4': 'CjkCompatibilityIdeographf9c4',
u'\u1547': 'CanadianSyllabicsRii',
u'\u5249': 'CjkUnifiedIdeograph5249',
u'\u8f4b': 'CjkUnifiedIdeograph8f4b',
u'\ucc4d': 'HangulSyllableCaelg',
u'\u2ace': 'SquareRightOpenBoxOperator',
u'\u67d0': 'CjkUnifiedIdeograph67d0',
u'\ua4d2': 'UnicodeCharacter0xa4d2',
u'\ue1d4': 'UnicodeCharacter0xe1d4',
u'\u3d57': 'CjkUnifiedIdeograph3d57',
u'\uded6': 'UnicodeCharacter0xded6',
u'\u7a59': 'CjkUnifiedIdeograph7a59',
u'\ub75b': 'HangulSyllableDdyic',
u'\uf45d': 'UnicodeCharacter0xf45d',
u'\u0fe0': 'UnicodeCharacter0xfe0',
u'\u4ce2': 'CjkUnifiedIdeograph4ce2',
u'\u89e4': 'CjkUnifiedIdeograph89e4',
u'\u2567': 'BoxDrawingsUpSingleAndHorizontalDouble',
u'\uc6e6': 'HangulSyllableWeop',
u'\u6269': 'CjkUnifiedIdeograph6269',
u'\u5f6b': 'CjkUnifiedIdeograph5f6b',
u'\u9c6d': 'CjkUnifiedIdeograph9c6d',
u'\ud96f': 'UnicodeCharacter0xd96f',
u'\u37f0': 'CjkUnifiedIdeograph37f0',
u'\u74f2': 'CjkUnifiedIdeograph74f2',
u'\ub1f4': 'HangulSyllableNyols',
u'\ueef6': 'UnicodeCharacter0xeef6',
u'\u0a79': 'UnicodeCharacter0xa79',
u'\u477b': 'CjkUnifiedIdeograph477b',
u'\u847d': 'CjkUnifiedIdeograph847d',
u'\uc17f': 'HangulSyllableSyelh',
u'\u8181': 'CjkUnifiedIdeograph8181',
u'\u6604': 'CjkUnifiedIdeograph6604',
u'\u4a87': 'CjkUnifiedIdeograph4a87',
u'\uec08': 'UnicodeCharacter0xec08',
u'\ud08b': 'HangulSyllableKyigs',
u'\u2f0a': 'KangxiRadicalEnter',
u'\u138d': 'EthiopicSyllablePwi',
u'\ub50e': 'HangulSyllableDyij',
u'\u9991': 'CjkUnifiedIdeograph9991',
u'\u7e14': 'CjkUnifiedIdeograph7e14',
u'\u2297': 'CircledTimes',
u'\uc418': 'HangulSyllableSswael',
u'\ua89b': 'SaurashtraLetterNya',
u'\u071a': 'SyriacLetterHeth',
u'\u8d1e': 'CjkUnifiedIdeograph8d1e',
u'\u71a1': 'CjkUnifiedIdeograph71a1',
u'\uf7a5': 'UnicodeCharacter0xf7a5',
u'\u5624': 'CjkUnifiedIdeograph5624',
u'\u3aa7': 'CjkUnifiedIdeograph3aa7',
u'\udc28': 'UnicodeCharacter0xdc28',
u'\u80ab': 'CjkUnifiedIdeograph80ab',
u'\u1f2a': 'GreekCapitalLetterEtaWithPsiliAndVaria',
u'\u652e': 'CjkUnifiedIdeograph652e',
u'\u49b1': 'CjkUnifiedIdeograph49b1',
u'\ueb32': 'UnicodeCharacter0xeb32',
u'\ucfb5': 'HangulSyllableKoelt',
u'\u2e34': 'UnicodeCharacter0x2e34',
u'\u12b7': 'UnicodeCharacter0x12b7',
u'\ub438': 'HangulSyllableDyon',
u'\u98bb': 'CjkUnifiedIdeograph98bb',
u'\u7d3e': 'CjkUnifiedIdeograph7d3e',
u'\u21c1': 'RightwardsHarpoonWithBarbDownwards',
u'\uc342': 'HangulSyllableSsyabs',
u'\ua7c5': 'UnicodeCharacter0xa7c5',
u'\u0644': 'ArabicLetterLam',
u'\u8c48': 'CjkUnifiedIdeograph8c48',
u'\u70cb': 'CjkUnifiedIdeograph70cb',
u'\uf6cf': 'UnicodeCharacter0xf6cf',
u'\u554e': 'CjkUnifiedIdeograph554e',
u'\u39d1': 'CjkUnifiedIdeograph39d1',
u'\udb52': 'UnicodeCharacter0xdb52',
u'\ubfd5': 'HangulSyllableBbulg',
u'\u1e54': 'LatinCapitalLetterPWithAcute',
u'\u6458': 'CjkUnifiedIdeograph6458',
u'\u48db': 'CjkUnifiedIdeograph48db',
u'\uea5c': 'UnicodeCharacter0xea5c',
u'\ucedf': 'HangulSyllableKyaec',
u'\u2d5e': 'TifinaghLetterYach',
u'\u11e1': 'HangulJongseongMieumhieuh',
u'\ub362': 'HangulSyllableDeolp',
u'\u97e5': 'CjkUnifiedIdeograph97e5',
u'\u7c68': 'CjkUnifiedIdeograph7c68',
u'\u20eb': 'CombiningLongDoubleSolidusOverlay',
u'\uc26c': 'HangulSyllableSwi',
u'\ua6ef': 'UnicodeCharacter0xa6ef',
u'\u056e': 'ArmenianSmallLetterCa',
u'\u8b72': 'CjkUnifiedIdeograph8b72',
u'\u6ff5': 'CjkUnifiedIdeograph6ff5',
u'\uf5f9': 'UnicodeCharacter0xf5f9',
u'\u5478': 'CjkUnifiedIdeograph5478',
u'\u38fb': 'CjkUnifiedIdeograph38fb',
u'\uda7c': 'UnicodeCharacter0xda7c',
u'\ubeff': 'HangulSyllableBbes',
u'\u1d7e': 'LatinSmallCapitalLetterUWithStroke',
u'\u8403': 'CjkUnifiedIdeograph8403',
u'\u6382': 'CjkUnifiedIdeograph6382',
u'\ue986': 'UnicodeCharacter0xe986',
u'\u4d09': 'CjkUnifiedIdeograph4d09',
u'\u2c88': 'CopticCapitalLetterEie',
u'\ud30d': 'HangulSyllablePag',
u'\ub28c': 'HangulSyllableNyuk',
u'\u160f': 'CanadianSyllabicsCarrierYo',
u'\u9c13': 'CjkUnifiedIdeograph9c13',
u'\u7b92': 'CjkUnifiedIdeograph7b92',
u'\ue217': 'UnicodeCharacter0xe217',
u'\uc196': 'HangulSyllableSolm',
u'\u2519': 'BoxDrawingsUpLightAndLeftHeavy',
u'\u0498': 'CyrillicCapitalLetterZeWithDescender',
u'\uab1d': 'UnicodeCharacter0xab1d',
u'\u8a9c': 'CjkUnifiedIdeograph8a9c',
u'\u7423': 'CjkUnifiedIdeograph7423',
u'\u53a2': 'CjkUnifiedIdeograph53a2',
u'\ufa27': 'CjkCompatibilityIdeographfa27',
u'\ud9a6': 'UnicodeCharacter0xd9a6',
u'\u3d29': 'CjkUnifiedIdeograph3d29',
u'\u1ca8': 'UnicodeCharacter0x1ca8',
u'\u832d': 'CjkUnifiedIdeograph832d',
u'\u62ac': 'CjkUnifiedIdeograph62ac',
u'\ue8b0': 'UnicodeCharacter0xe8b0',
u'\u4c33': 'CjkUnifiedIdeograph4c33',
u'\u2bb2': 'UnicodeCharacter0x2bb2',
u'\ud237': 'HangulSyllableTulb',
u'\ub1b6': 'HangulSyllableNwaenh',
u'\u1539': 'CanadianSyllabicsYwa',
u'\u9b3d': 'CjkUnifiedIdeograph9b3d',
u'\u7abc': 'CjkUnifiedIdeograph7abc',
u'\ue141': 'UnicodeCharacter0xe141',
u'\uc0c0': 'HangulSyllableSass',
u'\u2443': 'OcrInvertedFork',
u'\u03c2': 'GreekSmallLetterFinalSigma',
u'\uaa47': 'ChamLetterFinalP',
u'\u89c6': 'CjkUnifiedIdeograph89c6',
u'\u734d': 'CjkUnifiedIdeograph734d',
u'\u52cc': 'CjkUnifiedIdeograph52cc',
u'\uf951': 'CjkCompatibilityIdeographf951',
u'\ud8d0': 'UnicodeCharacter0xd8d0',
u'\u3c53': 'CjkUnifiedIdeograph3c53',
u'\u1bd2': 'UnicodeCharacter0x1bd2',
u'\u8257': 'CjkUnifiedIdeograph8257',
u'\u61d6': 'CjkUnifiedIdeograph61d6',
u'\ue7da': 'UnicodeCharacter0xe7da',
u'\u4b5d': 'CjkUnifiedIdeograph4b5d',
u'\u2adc': 'Forking',
u'\ud161': 'HangulSyllableTeng',
u'\ub0e0': 'HangulSyllableNyam',
u'\u1463': 'CanadianSyllabicsTwaa',
u'\u9a67': 'CjkUnifiedIdeograph9a67',
u'\u79e6': 'CjkUnifiedIdeograph79e6',
u'\ue06b': 'UnicodeCharacter0xe06b',
u'\uffea': 'HalfwidthUpwardsArrow',
u'\u236d': 'AplFunctionalSymbolStileTilde',
u'\u02ec': 'ModifierLetterVoicing',
u'\ua971': 'UnicodeCharacter0xa971',
u'\u88f0': 'CjkUnifiedIdeograph88f0',
u'\u7277': 'CjkUnifiedIdeograph7277',
u'\u51f6': 'CjkUnifiedIdeograph51f6',
u'\uf87b': 'UnicodeCharacter0xf87b',
u'\ud7fa': 'UnicodeCharacter0xd7fa',
u'\u3b7d': 'CjkUnifiedIdeograph3b7d',
u'\u1afc': 'UnicodeCharacter0x1afc',
u'\x81': 'UnicodeCharacter0x81',
u'\u6100': 'CjkUnifiedIdeograph6100',
u'\u4583': 'CjkUnifiedIdeograph4583',
u'\ua202': 'YiSyllableKap',
u'\u8685': 'CjkUnifiedIdeograph8685',
u'\ue704': 'UnicodeCharacter0xe704',
u'\ucb87': 'HangulSyllableJjwed',
u'\u340e': 'CjkUnifiedIdeograph340e',
u'\u1891': 'MongolianLetterAliGaliDa',
u'\u7910': 'CjkUnifiedIdeograph7910',
u'\u5d93': 'CjkUnifiedIdeograph5d93',
u'\uba12': 'HangulSyllableMyabs',
u'\u9e95': 'CjkUnifiedIdeograph9e95',
u'\uff14': 'FullwidthDigitFour',
u'\ua397': 'YiSyllableNrap',
u'\ue499': 'UnicodeCharacter0xe499',
u'\u0c1e': 'TeluguLetterNya',
u'\u5120': 'CjkUnifiedIdeograph5120',
u'\u35a3': 'CjkUnifiedIdeograph35a3',
u'\u9222': 'CjkUnifiedIdeograph9222',
u'\u76a5': 'CjkUnifiedIdeograph76a5',
u'\ud724': 'HangulSyllableHwils',
u'\ubba7': 'HangulSyllableMyugs',
u'\ufca9': 'ArabicLigatureHahWithJeemInitialForm',
u'\u2930': 'RisingDiagonalCrossingSouthEastArrow',
u'\u0db3': 'SinhalaLetterSanyakaDayanna',
u'\u6a32': 'CjkUnifiedIdeograph6a32',
u'\u4eb5': 'CjkUnifiedIdeograph4eb5',
u'\uaf34': 'HangulSyllableGgol',
u'\u93b7': 'CjkUnifiedIdeograph93b7',
u'\uf036': 'UnicodeCharacter0xf036',
u'\ud4b9': 'HangulSyllablePwelg',
u'\u0140': 'LatinSmallLetterLWithMiddleDot',
u'\u4242': 'CjkUnifiedIdeograph4242',
u'\u26c5': 'UnicodeCharacter0x26c5',
u'\u8744': 'CjkUnifiedIdeograph8744',
u'\u6bc7': 'CjkUnifiedIdeograph6bc7',
u'\uc846': 'HangulSyllableJyeolp',
u'\uacc9': 'HangulSyllableGyenj',
u'\uf1cb': 'UnicodeCharacter0xf1cb',
u'\u1950': 'TaiLeLetterKa',
u'\u5a52': 'CjkUnifiedIdeograph5a52',
u'\u3ed5': 'CjkUnifiedIdeograph3ed5',
u'\u9f54': 'CjkUnifiedIdeograph9f54',
u'\u43d7': 'CjkUnifiedIdeograph43d7',
u'\ua056': 'YiSyllableBbit',
u'\u84d9': 'CjkUnifiedIdeograph84d9',
u'\ue558': 'UnicodeCharacter0xe558',
u'\uc9db': 'HangulSyllableJih',
u'\u3262': 'CircledHangulTikeut',
u'\u16e5': 'RunicLetterStan',
u'\u7764': 'CjkUnifiedIdeograph7764',
u'\u5be7': 'CjkUnifiedIdeograph5be7',
u'\ub866': 'HangulSyllableRolm',
u'\u9ce9': 'CjkUnifiedIdeograph9ce9',
u'\ufd68': 'ArabicLigatureSheenWithHahWithMeemInitialForm',
u'\ua1eb': 'YiSyllableGox',
u'\ue2ed': 'UnicodeCharacter0xe2ed',
u'\u0a72': 'GurmukhiIri',
u'\u4f74': 'CjkUnifiedIdeograph4f74',
u'\u33f7': 'IdeographicTelegraphSymbolForDayTwentyfour',
u'\u9076': 'CjkUnifiedIdeograph9076',
u'\u74f9': 'CjkUnifiedIdeograph74f9',
u'\ud578': 'HangulSyllableHaen',
u'\ub9fb': 'HangulSyllableMaec',
u'\ufafd': 'UnicodeCharacter0xfafd',
u'\uc001': 'HangulSyllableBbweot',
u'\u2784': 'DingbatCircledSansserifDigitFive',
u'\u6886': 'CjkUnifiedIdeograph6886',
u'\uad88': 'HangulSyllableGweo',
u'\u110b': 'HangulChoseongIeung',
u'\uee8a': 'UnicodeCharacter0xee8a',
u'\u520d': 'CjkUnifiedIdeograph520d',
u'\u970f': 'CjkUnifiedIdeograph970f',
u'\ud811': 'UnicodeCharacter0xd811',
u'\u3f94': 'CjkUnifiedIdeograph3f94',
u'\u4096': 'CjkUnifiedIdeograph4096',
u'\u8598': 'CjkUnifiedIdeograph8598',
u'\uc69a': 'HangulSyllableYonh',
u'\u2a1d': 'Join',
u'\u6f1f': 'CjkUnifiedIdeograph6f1f',
u'\ub021': 'HangulSyllableGgwing',
u'\uf523': 'UnicodeCharacter0xf523',
u'\u17a4': 'KhmerIndependentVowelQaa',
u'\u58a6': 'CjkUnifiedIdeograph58a6',
u'\u9da8': 'CjkUnifiedIdeograph9da8',
u'\udeaa': 'UnicodeCharacter0xdeaa',
u'\u022d': 'LatinSmallLetterOWithTildeAndMacron',
u'\ue3ac': 'UnicodeCharacter0xe3ac',
u'\u472f': 'CjkUnifiedIdeograph472f',
u'\u8831': 'CjkUnifiedIdeograph8831',
u'\ucd33': 'HangulSyllableCwalh',
u'\u30b6': 'KatakanaLetterZa',
u'\u75b8': 'CjkUnifiedIdeograph75b8',
u'\ub6ba': 'HangulSyllableDdweogg',
u'\u1a3d': 'UnicodeCharacter0x1a3d',
u'\ufbbc': 'UnicodeCharacter0xfbbc',
u'\u5f3f': 'CjkUnifiedIdeograph5f3f',
u'\u6041': 'CjkUnifiedIdeograph6041',
u'\ua543': 'VaiSyllableKi',
u'\ue645': 'UnicodeCharacter0xe645',
u'\u08c6': 'UnicodeCharacter0x8c6',
u'\u4dc8': 'HexagramForSmallTaming',
u'\u8eca': 'CjkUnifiedIdeograph8eca',
u'\ud3cc': 'HangulSyllablePyeok',
u'\u374f': 'CjkUnifiedIdeograph374f',
u'\u7851': 'CjkUnifiedIdeograph7851',
u'\ubd53': 'HangulSyllableBoelb',
u'\ufe55': 'SmallColon',
u'\u25d8': 'InverseBullet',
u'\u66da': 'CjkUnifiedIdeograph66da',
u'\uabdc': 'UnicodeCharacter0xabdc',
u'\u0f5f': 'TibetanLetterZa',
u'\uecde': 'UnicodeCharacter0xecde',
u'\u5061': 'CjkUnifiedIdeograph5061',
u'\u9563': 'CjkUnifiedIdeograph9563',
u'\ud665': 'HangulSyllableHwab',
u'\u3de8': 'CjkUnifiedIdeograph3de8',
u'\u7eea': 'CjkUnifiedIdeograph7eea',
u'\u83ec': 'CjkUnifiedIdeograph83ec',
u'\uc4ee': 'HangulSyllableSsyup',
u'\u2871': 'BraillePatternDots1567',
u'\u6d73': 'CjkUnifiedIdeograph6d73',
u'\uae75': 'HangulSyllableGgaelt',
u'\uf377': 'UnicodeCharacter0xf377',
u'\u15f8': 'CanadianSyllabicsCarrierKhee',
u'\u56fa': 'CjkUnifiedIdeograph56fa',
u'\u9bfc': 'CjkUnifiedIdeograph9bfc',
u'\udcfe': 'UnicodeCharacter0xdcfe',
u'\ue200': 'UnicodeCharacter0xe200',
u'\uc683': 'HangulSyllableOelb',
u'\u2102': 'DoublestruckCapitalC',
u'\u0585': 'ArmenianSmallLetterOh',
u'\uab06': 'UnicodeCharacter0xab06',
u'\u8f89': 'CjkUnifiedIdeograph8f89',
u'\u700c': 'CjkUnifiedIdeograph700c',
u'\u548f': 'CjkUnifiedIdeograph548f',
u'\ufa10': 'CjkCompatibilityIdeographfa10',
u'\ude93': 'UnicodeCharacter0xde93',
u'\u3912': 'CjkUnifiedIdeograph3912',
u'\u1d95': 'LatinSmallLetterSchwaWithRetroflexHook',
u'\u8316': 'CjkUnifiedIdeograph8316',
u'\u6799': 'CjkUnifiedIdeograph6799',
u'\ue99d': 'UnicodeCharacter0xe99d',
u'\u481c': 'CjkUnifiedIdeograph481c',
u'\u2c9f': 'CopticSmallLetterO',
u'\ud220': 'HangulSyllableTyom',
u'\ub6a3': 'HangulSyllableDdud',
u'\u1122': 'HangulChoseongPieupsioskiyeok',
u'\u9b26': 'CjkUnifiedIdeograph9b26',
u'\u7fa9': 'CjkUnifiedIdeograph7fa9',
u'\uc1ad': 'HangulSyllableSwanj',
u'\u202c': 'PopDirectionalFormatting',
u'\u04af': 'CyrillicSmallLetterStraightU',
u'\uaa30': 'ChamVowelSignAi',
u'\u8eb3': 'CjkUnifiedIdeograph8eb3',
u'\u7336': 'CjkUnifiedIdeograph7336',
u'\u57b9': 'CjkUnifiedIdeograph57b9',
u'\uf53a': 'UnicodeCharacter0xf53a',
u'\ud9bd': 'UnicodeCharacter0xd9bd',
u'\u383c': 'CjkUnifiedIdeograph383c',
u'\u1cbf': 'UnicodeCharacter0x1cbf',
u'\u8240': 'CjkUnifiedIdeograph8240',
u'\u66c3': 'CjkUnifiedIdeograph66c3',
u'\ue8c7': 'UnicodeCharacter0xe8c7',
u'\u4b46': 'CjkUnifiedIdeograph4b46',
u'\u2fc9': 'KangxiRadicalMillet',
u'\ucd4a': 'HangulSyllableCwaelm',
u'\ub1cd': 'HangulSyllableNoeg',
u'\u104c': 'MyanmarSymbolLocative',
u'\u9a50': 'CjkUnifiedIdeograph9a50',
u'\u7ed3': 'CjkUnifiedIdeograph7ed3',
u'\uc0d7': 'HangulSyllableSaelh',
u'\u2356': 'AplFunctionalSymbolDownwardsVane',
u'\u07d9': 'NkoLetterRa',
u'\ua55a': 'VaiSyllableTa',
u'\u89dd': 'CjkUnifiedIdeograph89dd',
u'\u7260': 'CjkUnifiedIdeograph7260',
u'\u56e3': 'CjkUnifiedIdeograph56e3',
u'\uf464': 'UnicodeCharacter0xf464',
u'\ud8e7': 'UnicodeCharacter0xd8e7',
u'\u3b66': 'CjkUnifiedIdeograph3b66',
u'\u1fe9': 'GreekCapitalLetterUpsilonWithMacron',
u'\ubd6a': 'HangulSyllableByonh',
u'\u61ed': 'CjkUnifiedIdeograph61ed',
u'\uebf1': 'UnicodeCharacter0xebf1',
u'\u4a70': 'CjkUnifiedIdeograph4a70',
u'\u2ef3': 'CjkRadicalCsimplifiedTurtle',
u'\ucc74': 'HangulSyllableCyass',
u'\ub0f7': 'HangulSyllableNyaelb',
u'\u1376': 'EthiopicNumberFifty',
u'\u957a': 'CjkUnifiedIdeograph957a',
u'\u79fd': 'CjkUnifiedIdeograph79fd',
u'\u4301': 'CjkUnifiedIdeograph4301',
u'\u2280': 'DoesNotPrecede',
u'\uc505': 'HangulSyllableSseung',
u'\ua484': 'YiSyllableYup',
u'\u0807': 'UnicodeCharacter0x807',
u'\u920b': 'CjkUnifiedIdeograph920b',
u'\u6d8a': 'CjkUnifiedIdeograph6d8a',
u'\uf78e': 'UnicodeCharacter0xf78e',
u'\u5b11': 'CjkUnifiedIdeograph5b11',
u'\u3a90': 'CjkUnifiedIdeograph3a90',
u'\udd15': 'UnicodeCharacter0xdd15',
u'\ubc94': 'HangulSyllableBeom',
u'\u6a1b': 'CjkUnifiedIdeograph6a1b',
u'\u459a': 'CjkUnifiedIdeograph459a',
u'\uec1f': 'UnicodeCharacter0xec1f',
u'\ucf9e': 'HangulSyllableKwaebs',
u'\u3321': 'SquareSiringu',
u'\u12a0': 'EthiopicSyllableGlottalA',
u'\ub525': 'HangulSyllableDib',
u'\u94a4': 'CjkUnifiedIdeograph94a4',
u'\u422b': 'CjkUnifiedIdeograph422b',
u'\u5daa': 'CjkUnifiedIdeograph5daa',
u'\uc42f': 'HangulSyllableSsoegs',
u'\ua7ae': 'UnicodeCharacter0xa7ae',
u'\u0b31': 'UnicodeCharacter0xb31',
u'\u8d35': 'CjkUnifiedIdeograph8d35',
u'\u6cb4': 'CjkUnifiedIdeograph6cb4',
u'\uf6b8': 'UnicodeCharacter0xf6b8',
u'\u5a3b': 'CjkUnifiedIdeograph5a3b',
u'\u35ba': 'CjkUnifiedIdeograph35ba',
u'\udc3f': 'UnicodeCharacter0xdc3f',
u'\ubfbe': 'HangulSyllableBbyolp',
u'\u6545': 'CjkUnifiedIdeograph6545',
u'\u44c4': 'CjkUnifiedIdeograph44c4',
u'\uef49': 'UnicodeCharacter0xef49',
u'\ucec8': 'HangulSyllableKyae',
u'\u324b': 'UnicodeCharacter0x324b',
u'\u0dca': 'SinhalaSignAllakuna',
u'\ub44f': 'HangulSyllableDyoh',
u'\u97ce': 'CjkUnifiedIdeograph97ce',
u'\u7d55': 'CjkUnifiedIdeograph7d55',
u'\u5cd4': 'CjkUnifiedIdeograph5cd4',
u'\uc759': 'HangulSyllableYig',
u'\ua6d8': 'UnicodeCharacter0xa6d8',
u'\u0a5b': 'GurmukhiLetterZa',
u'\u8c5f': 'CjkUnifiedIdeograph8c5f',
u'\u6fde': 'CjkUnifiedIdeograph6fde',
u'\uf1e2': 'UnicodeCharacter0xf1e2',
u'\u5565': 'CjkUnifiedIdeograph5565',
u'\u34e4': 'CjkUnifiedIdeograph34e4',
u'\udf69': 'UnicodeCharacter0xdf69',
u'\ubee8': 'HangulSyllableBbeok',
u'\u646f': 'CjkUnifiedIdeograph646f',
u'\u47ee': 'CjkUnifiedIdeograph47ee',
u'\uee73': 'UnicodeCharacter0xee73',
u'\uc9f2': 'HangulSyllableJjaj',
u'\u2d75': 'UnicodeCharacter0x2d75',
u'\u0cf4': 'UnicodeCharacter0xcf4',
u'\ub779': 'HangulSyllableDdit',
u'\u96f8': 'CjkUnifiedIdeograph96f8',
u'\u7c7f': 'CjkUnifiedIdeograph7c7f',
u'\u5ffe': 'CjkUnifiedIdeograph5ffe',
u'\u2a06': 'NarySquareUnionOperator',
u'\u0e89': 'UnicodeCharacter0xe89',
u'\u6f08': 'CjkUnifiedIdeograph6f08',
u'\u538b': 'CjkUnifiedIdeograph538b',
u'\uac0a': 'HangulSyllableGalm',
u'\u908d': 'CjkUnifiedIdeograph908d',
u'\uf10c': 'UnicodeCharacter0xf10c',
u'\ud58f': 'HangulSyllableHaeh',
u'\u0216': 'LatinCapitalLetterUWithInvertedBreve',
u'\u4718': 'CjkUnifiedIdeograph4718',
u'\u2b9b': 'UnicodeCharacter0x2b9b',
u'\u841a': 'CjkUnifiedIdeograph841a',
u'\u689d': 'CjkUnifiedIdeograph689d',
u'\uc91c': 'HangulSyllableJweon',
u'\uad9f': 'HangulSyllableGweoc',
u'\uf2a1': 'UnicodeCharacter0xf2a1',
u'\u1a26': 'UnicodeCharacter0x1a26',
u'\u5f28': 'CjkUnifiedIdeograph5f28',
u'\u03ab': 'GreekCapitalLetterUpsilonWithDialytika',
u'\u9c2a': 'CjkUnifiedIdeograph9c2a',
u'\u40ad': 'CjkUnifiedIdeograph40ad',
u'\ua12c': 'YiSyllableTop',
u'\u85af': 'CjkUnifiedIdeograph85af',
u'\ue62e': 'UnicodeCharacter0xe62e',
u'\ucab1': 'HangulSyllableJjyeb',
u'\u3738': 'CjkUnifiedIdeograph3738',
u'\u1bbb': 'UnicodeCharacter0x1bbb',
u'\u743a': 'CjkUnifiedIdeograph743a',
u'\u58bd': 'CjkUnifiedIdeograph58bd',
u'\ub93c': 'HangulSyllableRwi',
u'\u9dbf': 'CjkUnifiedIdeograph9dbf',
u'\ufe3e': 'PresentationFormForVerticalRightDoubleAngleBracket',
u'\ua2c1': 'YiSyllableCex',
u'\ue7c3': 'UnicodeCharacter0xe7c3',
u'\u0f48': 'UnicodeCharacter0xf48',
u'\u4c4a': 'CjkUnifiedIdeograph4c4a',
u'\u30cd': 'KatakanaLetterNe',
u'\u914c': 'CjkUnifiedIdeograph914c',
u'\u75cf': 'CjkUnifiedIdeograph75cf',
u'\ud64e': 'HangulSyllableHoj',
u'\ubad1': 'HangulSyllableMwalt',
u'\uffd3': 'HalfwidthHangulLetterU',
u'\u245a': 'UnicodeCharacter0x245a',
u'\u08dd': 'UnicodeCharacter0x8dd',
u'\u695c': 'CjkUnifiedIdeograph695c',
u'\u4ddf': 'HexagramForDuration',
u'\uae5e': 'HangulSyllableGgabs',
u'\u92e1': 'CjkUnifiedIdeograph92e1',
u'\uf360': 'UnicodeCharacter0xf360',
u'\ud7e3': 'UnicodeCharacter0xd7e3',
u'\u3c6a': 'CjkUnifiedIdeograph3c6a',
u'\u416c': 'CjkUnifiedIdeograph416c',
u'\u25ef': 'LargeCircle',
u'\u866e': 'CjkUnifiedIdeograph866e',
u'\u6af1': 'CjkUnifiedIdeograph6af1',
u'\ucb70': 'HangulSyllableJjweols',
u'\uaff3': 'HangulSyllableGgwegs',
u'\uecf5': 'UnicodeCharacter0xecf5',
u'\u147a': 'CanadianSyllabicsKwo',
u'\u597c': 'CjkUnifiedIdeograph597c',
u'\u3dff': 'CjkUnifiedIdeograph3dff',
u'\u9e7e': 'CjkUnifiedIdeograph9e7e',
u'\ua380': 'YiSyllableRrot',
u'\u0703': 'SyriacSupralinearColon',
u'\ue082': 'UnicodeCharacter0xe082',
u'\u4405': 'CjkUnifiedIdeograph4405',
u'\u8907': 'CjkUnifiedIdeograph8907',
u'\uce09': 'HangulSyllableCyunj',
u'\u318c': 'HangulLetterYui',
u'\u768e': 'CjkUnifiedIdeograph768e',
u'\ubb90': 'HangulSyllableMwil',
u'\u1f13': 'GreekSmallLetterEpsilonWithDasiaAndVaria',
u'\uf892': 'UnicodeCharacter0xf892',
u'\u5c15': 'CjkUnifiedIdeograph5c15',
u'\u6117': 'CjkUnifiedIdeograph6117',
u'\ua619': 'VaiSymbolTaa',
u'\ueb1b': 'UnicodeCharacter0xeb1b',
u'\u099c': 'BengaliLetterJa',
u'\u4e9e': 'CjkUnifiedIdeograph4e9e',
u'\u93a0': 'CjkUnifiedIdeograph93a0',
u'\ud0a2': 'HangulSyllableKyip',
u'\u3425': 'CjkUnifiedIdeograph3425',
u'\u7927': 'CjkUnifiedIdeograph7927',
u'\ube29': 'HangulSyllableByig',
u'\uc32b': 'HangulSyllableSsaec',
u'\u26ae': 'DivorceSymbol',
u'\u6bb0': 'CjkUnifiedIdeograph6bb0',
u'\ua8b2': 'SaurashtraLetterHa',
u'\u0c35': 'TeluguLetterVa',
u'\uedb4': 'UnicodeCharacter0xedb4',
u'\u5137': 'CjkUnifiedIdeograph5137',
u'\u9639': 'CjkUnifiedIdeograph9639',
u'\udb3b': 'UnicodeCharacter0xdb3b',
u'\u3ebe': 'CjkUnifiedIdeograph3ebe',
u'\u43c0': 'CjkUnifiedIdeograph43c0',
u'\u80c2': 'CjkUnifiedIdeograph80c2',
u'\uc5c4': 'HangulSyllableEom',
u'\u2947': 'RightwardsArrowThroughX',
u'\u6e49': 'CjkUnifiedIdeograph6e49',
u'\ub34b': 'HangulSyllableDyaes',
u'\uf04d': 'UnicodeCharacter0xf04d',
u'\u16ce': 'RunicLetterZ',
u'\u5bd0': 'CjkUnifiedIdeograph5bd0',
u'\u98d2': 'CjkUnifiedIdeograph98d2',
u'\uddd4': 'UnicodeCharacter0xddd4',
u'\u0157': 'LatinSmallLetterRWithCedilla',
u'\ue2d6': 'UnicodeCharacter0xe2d6',
u'\u4659': 'CjkUnifiedIdeograph4659',
u'\u8b5b': 'CjkUnifiedIdeograph8b5b',
u'\uc85d': 'HangulSyllableJyelg',
u'\u33e0': 'IdeographicTelegraphSymbolForDayOne',
u'\u70e2': 'CjkUnifiedIdeograph70e2',
u'\ub5e4': 'HangulSyllableDdyeols',
u'\u1967': 'TaiLeLetterU',
u'\ufae6': 'UnicodeCharacter0xfae6',
u'\u5e69': 'CjkUnifiedIdeograph5e69',
u'\u636b': 'CjkUnifiedIdeograph636b',
u'\ua06d': 'YiSyllableBbux',
u'\ue56f': 'UnicodeCharacter0xe56f',
u'\u0bf0': 'TamilNumberTen',
u'\u48f2': 'CjkUnifiedIdeograph48f2',
u'\u8df4': 'CjkUnifiedIdeograph8df4',
u'\ud2f6': 'HangulSyllableTinh',
u'\u3679': 'CjkUnifiedIdeograph3679',
u'\u7b7b': 'CjkUnifiedIdeograph7b7b',
u'\ub87d': 'HangulSyllableRwanj',
u'\ufd7f': 'ArabicLigatureQafWithMeemWithMeemFinalForm',
u'\u8581': 'CjkUnifiedIdeograph8581',
u'\u6204': 'CjkUnifiedIdeograph6204',
u'\u4e87': 'CjkUnifiedIdeograph4e87',
u'\ue808': 'UnicodeCharacter0xe808',
u'\ud48b': 'HangulSyllablePus',
u'\u2b0a': 'SouthEastBlackArrow',
u'\u178d': 'KhmerLetterTtho',
u'\ub10e': 'HangulSyllableNeonh',
u'\u9d91': 'CjkUnifiedIdeograph9d91',
u'\u7a14': 'CjkUnifiedIdeograph7a14',
u'\u2697': 'Alembic',
u'\uc018': 'HangulSyllableBbwess',
u'\uac9b': 'HangulSyllableGelh',
u'\u031a': 'CombiningLeftAngleAbove',
u'\u891e': 'CjkUnifiedIdeograph891e',
u'\u75a1': 'CjkUnifiedIdeograph75a1',
u'\uf3a5': 'UnicodeCharacter0xf3a5',
u'\u5224': 'CjkUnifiedIdeograph5224',
u'\u3ea7': 'CjkUnifiedIdeograph3ea7',
u'\ud828': 'UnicodeCharacter0xd828',
u'\u84ab': 'CjkUnifiedIdeograph84ab',
u'\u1b2a': 'BalineseLetterBaKembang',
u'\u612e': 'CjkUnifiedIdeograph612e',
u'\u4db1': 'CjkUnifiedIdeograph4db1',
u'\uef32': 'UnicodeCharacter0xef32',
u'\ucbb5': 'HangulSyllableJjwit',
u'\u2a34': 'MultiplicationSignInLeftHalfCircle',
u'\u16b7': 'RunicLetterGeboGyfuG',
u'\ub038': 'HangulSyllableGgyum',
u'\u9cbb': 'CjkUnifiedIdeograph9cbb',
u'\u793e': 'CjkUnifiedIdeograph793e',
u'\u25c1': 'WhiteLeftpointingTriangle',
u'\uc742': 'HangulSyllableEunh',
u'\ua3c5': 'YiSyllableShyr',
u'\u0244': 'LatinCapitalLetterUBar',
u'\u8848': 'CjkUnifiedIdeograph8848',
u'\u74cb': 'CjkUnifiedIdeograph74cb',
u'\uf2cf': 'UnicodeCharacter0xf2cf',
u'\u514e': 'CjkUnifiedIdeograph514e',
u'\u3dd1': 'CjkUnifiedIdeograph3dd1',
u'\udf52': 'UnicodeCharacter0xdf52',
u'\ubbd5': 'HangulSyllableMeung',
u'\u1a54': 'UnicodeCharacter0x1a54',
u'\u6058': 'CjkUnifiedIdeograph6058',
u'\u4cdb': 'CjkUnifiedIdeograph4cdb',
u'\uee5c': 'UnicodeCharacter0xee5c',
u'\ucadf': 'HangulSyllableJjwad',
u'\u295e': 'LeftwardsHarpoonWithBarbDownFromBar',
u'\u15e1': 'CanadianSyllabicsCarrierTha',
u'\ub762': 'HangulSyllableDdigg',
u'\u93e5': 'CjkUnifiedIdeograph93e5',
u'\u7868': 'CjkUnifiedIdeograph7868',
u'\u24eb': 'NegativeCircledNumberEleven',
u'\uc66c': 'HangulSyllableWaem',
u'\ua2ef': 'YiSyllableNzi',
u'\u016e': 'LatinCapitalLetterUWithRingAbove',
u'\u8f72': 'CjkUnifiedIdeograph8f72',
u'\u6bf5': 'CjkUnifiedIdeograph6bf5',
u'\uf1f9': 'UnicodeCharacter0xf1f9',
u'\u5078': 'CjkUnifiedIdeograph5078',
u'\u3cfb': 'CjkUnifiedIdeograph3cfb',
u'\ude7c': 'UnicodeCharacter0xde7c',
u'\ubaff': 'HangulSyllableMoegs',
u'\u197e': 'UnicodeCharacter0x197e',
u'\u8003': 'CjkUnifiedIdeograph8003',
u'\u6782': 'CjkUnifiedIdeograph6782',
u'\ued86': 'UnicodeCharacter0xed86',
u'\u4909': 'CjkUnifiedIdeograph4909',
u'\u2888': 'BraillePatternDots48',
u'\ud70d': 'HangulSyllableHweb',
u'\ub68c': 'HangulSyllableDdyols',
u'\u120f': 'EthiopicSyllableLwa',
u'\u9813': 'CjkUnifiedIdeograph9813',
u'\u7f92': 'CjkUnifiedIdeograph7f92',
u'\ue617': 'UnicodeCharacter0xe617',
u'\uc596': 'HangulSyllableYap',
u'\u2119': 'DoublestruckCapitalP',
u'\x98': 'UnicodeCharacter0x98',
u'\uaf1d': 'HangulSyllableGgyelt',
u'\u8e9c': 'CjkUnifiedIdeograph8e9c',
u'\u7023': 'CjkUnifiedIdeograph7023',
u'\u57a2': 'CjkUnifiedIdeograph57a2',
u'\ufe27': 'UnicodeCharacter0xfe27',
u'\udda6': 'UnicodeCharacter0xdda6',
u'\u3929': 'CjkUnifiedIdeograph3929',
u'\u18a8': 'MongolianLetterManchuAliGaliBha',
u'\u872d': 'CjkUnifiedIdeograph872d',
u'\u66ac': 'CjkUnifiedIdeograph66ac',
u'\uecb0': 'UnicodeCharacter0xecb0',
u'\u4833': 'CjkUnifiedIdeograph4833',
u'\u2fb2': 'KangxiRadicalLeek',
u'\ud637': 'HangulSyllableHyeh',
u'\ub5b6': 'HangulSyllableDdeoj',
u'\u1139': 'HangulChoseongSiosthieuth',
u'\u9f3d': 'CjkUnifiedIdeograph9f3d',
u'\u7ebc': 'CjkUnifiedIdeograph7ebc',
u'\ue541': 'UnicodeCharacter0xe541',
u'\uc4c0': 'HangulSyllableSswil',
u'\u2043': 'HyphenBullet',
u'\u07c2': 'NkoDigitTwo',
u'\uae47': 'HangulSyllableGic',
u'\u8dc6': 'CjkUnifiedIdeograph8dc6',
u'\u774d': 'CjkUnifiedIdeograph774d',
u'\u56cc': 'CjkUnifiedIdeograph56cc',
u'\ufd51': 'ArabicLigatureTehWithHahWithJeemFinalForm',
u'\udcd0': 'UnicodeCharacter0xdcd0',
u'\u3853': 'CjkUnifiedIdeograph3853',
u'\u1fd2': 'GreekSmallLetterIotaWithDialytikaAndVaria',
u'\u8657': 'CjkUnifiedIdeograph8657',
u'\u65d6': 'CjkUnifiedIdeograph65d6',
u'\ue3da': 'UnicodeCharacter0xe3da',
u'\u4f5d': 'CjkUnifiedIdeograph4f5d',
u'\u2edc': 'CjkRadicalCsimplifiedFly',
u'\ud561': 'HangulSyllableHalg',
u'\ub4e0': 'HangulSyllableDeun',
u'\u1063': 'MyanmarToneMarkSgawKarenHathi',
u'\u9e67': 'CjkUnifiedIdeograph9e67',
u'\u7de6': 'CjkUnifiedIdeograph7de6',
u'\ue46b': 'UnicodeCharacter0xe46b',
u'\ufbea': 'ArabicLigatureYehWithHamzaAboveWithAlefIsolatedForm',
u'\u276d': 'MediumRightpointingAngleBracketOrnament',
u'\u06ec': 'ArabicRoundedHighStopWithFilledCentre',
u'\uad71': 'HangulSyllableGunj',
u'\u8cf0': 'CjkUnifiedIdeograph8cf0',
u'\u7677': 'CjkUnifiedIdeograph7677',
u'\u55f6': 'CjkUnifiedIdeograph55f6',
u'\ufc7b': 'ArabicLigatureThehWithYehFinalForm',
u'\ud3fa': 'HangulSyllablePolp',
u'\u3f7d': 'CjkUnifiedIdeograph3f7d',
u'\u1efc': 'LatinCapitalLetterMiddlewelshV',
u'\u0481': 'CyrillicSmallLetterKoppa',
u'\u6500': 'CjkUnifiedIdeograph6500',
u'\u4183': 'CjkUnifiedIdeograph4183',
u'\ua602': 'VaiSyllableYe',
u'\u8285': 'CjkUnifiedIdeograph8285',
u'\ue304': 'UnicodeCharacter0xe304',
u'\ucf87': 'HangulSyllableKwac',
u'\u300e': 'LeftWhiteCornerBracket',
u'\u1c91': 'UnicodeCharacter0x1c91',
u'\u7d10': 'CjkUnifiedIdeograph7d10',
u'\u5993': 'CjkUnifiedIdeograph5993',
u'\ube12': 'HangulSyllableBeunh',
u'\u9a95': 'CjkUnifiedIdeograph9a95',
u'\ufb14': 'ArmenianSmallLigatureMenEch',
u'\ua797': 'UnicodeCharacter0xa797',
u'\ue099': 'UnicodeCharacter0xe099',
u'\u081e': 'UnicodeCharacter0x81e',
u'\u5520': 'CjkUnifiedIdeograph5520',
u'\u31a3': 'BopomofoLetterGu',
u'\u9622': 'CjkUnifiedIdeograph9622',
u'\u72a5': 'CjkUnifiedIdeograph72a5',
u'\ud324': 'HangulSyllablePak',
u'\ubfa7': 'HangulSyllableBboes',
u'\uf8a9': 'UnicodeCharacter0xf8a9',
u'\u2d30': 'TifinaghLetterYa',
u'\u09b3': 'UnicodeCharacter0x9b3',
u'\u6e32': 'CjkUnifiedIdeograph6e32',
u'\u4ab5': 'CjkUnifiedIdeograph4ab5',
u'\uab34': 'UnicodeCharacter0xab34',
u'\u97b7': 'CjkUnifiedIdeograph97b7',
u'\uf436': 'UnicodeCharacter0xf436',
u'\ud0b9': 'HangulSyllableKing',
u'\u0540': 'ArmenianCapitalLetterHo',
u'\u4642': 'CjkUnifiedIdeograph4642',
u'\u22c5': 'DotOperator',
u'\u8344': 'CjkUnifiedIdeograph8344',
u'\u6fc7': 'CjkUnifiedIdeograph6fc7',
u'\ucc46': 'HangulSyllableCaegg',
u'\ua8c9': 'UnicodeCharacter0xa8c9',
u'\uf5cb': 'UnicodeCharacter0xf5cb',
u'\u1d50': 'ModifierLetterSmallM',
u'\u5e52': 'CjkUnifiedIdeograph5e52',
u'\u3ad5': 'CjkUnifiedIdeograph3ad5',
u'\u9b54': 'CjkUnifiedIdeograph9b54',
u'\u47d7': 'CjkUnifiedIdeograph47d7',
u'\ua456': 'YiSyllableNyox',
u'\u80d9': 'CjkUnifiedIdeograph80d9',
u'\ue158': 'UnicodeCharacter0xe158',
u'\ucddb': 'HangulSyllableCwelh',
u'\u3662': 'CjkUnifiedIdeograph3662',
u'\u12e5': 'EthiopicSyllableZhe',
u'\u7364': 'CjkUnifiedIdeograph7364',
u'\u5fe7': 'CjkUnifiedIdeograph5fe7',
u'\ubc66': 'HangulSyllableByap',
u'\u98e9': 'CjkUnifiedIdeograph98e9',
u'\uf968': 'CjkCompatibilityIdeographf968',
u'\ua5eb': 'VaiSyllableMbe',
u'\ue6ed': 'UnicodeCharacter0xe6ed',
u'\u0e72': 'UnicodeCharacter0xe72',
u'\u4b74': 'CjkUnifiedIdeograph4b74',
u'\u37f7': 'CjkUnifiedIdeograph37f7',
u'\u9476': 'CjkUnifiedIdeograph9476',
u'\u70f9': 'CjkUnifiedIdeograph70f9',
u'\ud178': 'HangulSyllableTyeom',
u'\ubdfb': 'HangulSyllableByulb',
u'\ufefd': 'UnicodeCharacter0xfefd',
u'\uc401': 'HangulSyllableSswalt',
u'\u2384': 'CompositionSymbol',
u'\u6c86': 'CjkUnifiedIdeograph6c86',
u'\ua988': 'UnicodeCharacter0xa988',
u'\u150b': 'CanadianSyllabicsNaskapiSw',
u'\uea8a': 'UnicodeCharacter0xea8a',
u'\u560d': 'CjkUnifiedIdeograph560d',
u'\u930f': 'CjkUnifiedIdeograph930f',
u'\udc11': 'UnicodeCharacter0xdc11',
u'\u3b94': 'CjkUnifiedIdeograph3b94',
u'\u4496': 'CjkUnifiedIdeograph4496',
u'\u8198': 'CjkUnifiedIdeograph8198',
u'\uc29a': 'HangulSyllableSyubs',
u'\u2e1d': 'RightLowParaphraseBracket',
u'\u6b1f': 'CjkUnifiedIdeograph6b1f',
u'\ub421': 'HangulSyllableDoelg',
u'\uf123': 'UnicodeCharacter0xf123',
u'\u13a4': 'CherokeeLetterU',
u'\u5ca6': 'CjkUnifiedIdeograph5ca6',
u'\u99a8': 'CjkUnifiedIdeograph99a8',
u'\udaaa': 'UnicodeCharacter0xdaaa',
u'\u062d': 'ArabicLetterHah',
u'\ue7ac': 'UnicodeCharacter0xe7ac',
u'\u432f': 'CjkUnifiedIdeograph432f',
u'\u8c31': 'CjkUnifiedIdeograph8c31',
u'\uc933': 'HangulSyllableJweoh',
u'\u34b6': 'CjkUnifiedIdeograph34b6',
u'\u71b8': 'CjkUnifiedIdeograph71b8',
u'\ub2ba': 'HangulSyllableNyilp',
u'\u1e3d': 'LatinSmallLetterLWithCircumflexBelow',
u'\uffbc': 'HalfwidthHangulLetterThieuth',
u'\u5b3f': 'CjkUnifiedIdeograph5b3f',
u'\u6441': 'CjkUnifiedIdeograph6441',
u'\ua143': 'YiSyllableDduop',
u'\ue245': 'UnicodeCharacter0xe245',
u'\u0cc6': 'KannadaVowelSignE',
u'\u49c8': 'CjkUnifiedIdeograph49c8',
u'\u8aca': 'CjkUnifiedIdeograph8aca',
u'\ud7cc': 'UnicodeCharacter0xd7cc',
u'\u334f': 'SquareYaaru',
u'\u7c51': 'CjkUnifiedIdeograph7c51',
u'\ub953': 'HangulSyllableRwic',
u'\ufa55': 'CjkCompatibilityIdeographfa55',
u'\u21d8': 'SouthEastDoubleArrow',
u'\u62da': 'CjkUnifiedIdeograph62da',
u'\uafdc': 'HangulSyllableGgweol',
u'\u0b5f': 'OriyaLetterYya',
u'\ue8de': 'UnicodeCharacter0xe8de',
u'\u5461': 'CjkUnifiedIdeograph5461',
u'\u9163': 'CjkUnifiedIdeograph9163',
u'\ud265': 'HangulSyllableTweg',
u'\u39e8': 'CjkUnifiedIdeograph39e8',
u'\u7aea': 'CjkUnifiedIdeograph7aea',
u'\u87ec': 'CjkUnifiedIdeograph87ec',
u'\uc0ee': 'HangulSyllableSyalm',
u'\u2c71': 'LatinSmallLetterVWithRightHook',
u'\u6973': 'CjkUnifiedIdeograph6973',
u'\uaa75': 'UnicodeCharacter0xaa75',
u'\uf777': 'UnicodeCharacter0xf777',
u'\u11f8': 'HangulJongseongHieuhpieup',
u'\u52fa': 'CjkUnifiedIdeograph52fa',
u'\u9ffc': 'UnicodeCharacter0x9ffc',
u'\ud8fe': 'UnicodeCharacter0xd8fe',
u'\ue600': 'UnicodeCharacter0xe600',
u'\uc283': 'HangulSyllableSwic',
u'\u2502': 'BoxDrawingsLightVertical',
u'\u0185': 'LatinSmallLetterToneSix',
u'\uaf06': 'HangulSyllableGgyeobs',
u'\u8b89': 'CjkUnifiedIdeograph8b89',
u'\u740c': 'CjkUnifiedIdeograph740c',
u'\u508f': 'CjkUnifiedIdeograph508f',
u'\ufe10': 'PresentationFormForVerticalComma',
u'\uda93': 'UnicodeCharacter0xda93',
u'\u3d12': 'CjkUnifiedIdeograph3d12',
u'\u1995': 'NewTaiLueLetterHighPha',
u'\u8716': 'CjkUnifiedIdeograph8716',
u'\u6399': 'CjkUnifiedIdeograph6399',
u'\ued9d': 'UnicodeCharacter0xed9d',
u'\u4c1c': 'CjkUnifiedIdeograph4c1c',
u'\u289f': 'BraillePatternDots123458',
u'\ud620': 'HangulSyllableHyen',
u'\ub2a3': 'HangulSyllableNeus',
u'\u1522': 'CanadianSyllabicsWestcreeShwa',
u'\u9f26': 'CjkUnifiedIdeograph9f26',
u'\u7ba9': 'CjkUnifiedIdeograph7ba9',
u'\uc5ad': 'HangulSyllableYaeng',
u'\u242c': 'UnicodeCharacter0x242c',
u'\xaf': 'Macron',
u'\uae30': 'HangulSyllableGi',
u'\u8ab3': 'CjkUnifiedIdeograph8ab3',
u'\u7736': 'CjkUnifiedIdeograph7736',
u'\u53b9': 'CjkUnifiedIdeograph53b9',
u'\uf13a': 'UnicodeCharacter0xf13a',
u'\uddbd': 'UnicodeCharacter0xddbd',
u'\u3c3c': 'CjkUnifiedIdeograph3c3c',
u'\u18bf': 'UnicodeCharacter0x18bf',
u'\u8640': 'CjkUnifiedIdeograph8640',
u'\u62c3': 'CjkUnifiedIdeograph62c3',
u'\uecc7': 'UnicodeCharacter0xecc7',
u'\u4f46': 'CjkUnifiedIdeograph4f46',
u'\u2bc9': 'UnicodeCharacter0x2bc9',
u'\uc94a': 'HangulSyllableJwej',
u'\ub5cd': 'HangulSyllableDdeb',
u'\u144c': 'CanadianSyllabicsTe',
u'\u9e50': 'CjkUnifiedIdeograph9e50',
u'\u7ad3': 'CjkUnifiedIdeograph7ad3',
u'\uc4d7': 'HangulSyllableSsyugs',
u'\u2756': 'BlackDiamondMinusWhiteX',
u'\u03d9': 'GreekSmallLetterArchaicKoppa',
u'\ua15a': 'YiSyllableNdap',
u'\u8ddd': 'CjkUnifiedIdeograph8ddd',
u'\u7660': 'CjkUnifiedIdeograph7660',
u'\u52e3': 'CjkUnifiedIdeograph52e3',
u'\uf064': 'UnicodeCharacter0xf064',
u'\udce7': 'UnicodeCharacter0xdce7',
u'\u3f66': 'CjkUnifiedIdeograph3f66',
u'\u1be9': 'UnicodeCharacter0x1be9',
u'\ub96a': 'HangulSyllableRyubs',
u'\u65ed': 'CjkUnifiedIdeograph65ed',
u'\ueff1': 'UnicodeCharacter0xeff1',
u'\u4e70': 'CjkUnifiedIdeograph4e70',
u'\u2af3': 'ParallelWithTildeOperator',
u'\uc874': 'HangulSyllableJon',
u'\ub4f7': 'HangulSyllableDeuh',
u'\u1776': 'UnicodeCharacter0x1776',
u'\u917a': 'CjkUnifiedIdeograph917a',
u'\u7dfd': 'CjkUnifiedIdeograph7dfd',
u'\u4701': 'CjkUnifiedIdeograph4701',
u'\u2680': 'DieFace1',
u'\uc105': 'HangulSyllableSyaenj',
u'\ua084': 'YiSyllableNbop',
u'\u0c07': 'TeluguLetterI',
u'\u960b': 'CjkUnifiedIdeograph960b',
u'\u698a': 'CjkUnifiedIdeograph698a',
u'\uf38e': 'UnicodeCharacter0xf38e',
u'\u5f11': 'CjkUnifiedIdeograph5f11',
u'\u3e90': 'CjkUnifiedIdeograph3e90',
u'\ud915': 'UnicodeCharacter0xd915',
u'\ub894': 'HangulSyllableRwae',
u'\u6e1b': 'CjkUnifiedIdeograph6e1b',
u'\u419a': 'CjkUnifiedIdeograph419a',
u'\ue81f': 'UnicodeCharacter0xe81f',
u'\ucb9e': 'HangulSyllableJjwigg',
u'\u3721': 'CjkUnifiedIdeograph3721',
u'\u16a0': 'RunicLetterFehuFeohFeF',
u'\ub125': 'HangulSyllableNeg',
u'\u90a4': 'CjkUnifiedIdeograph90a4',
u'\u462b': 'CjkUnifiedIdeograph462b',
u'\u59aa': 'CjkUnifiedIdeograph59aa',
u'\uc02f': 'HangulSyllableBbwilh',
u'\ua3ae': 'YiSyllableShap',
u'\u0f31': 'TibetanDigitHalfEight',
u'\u8935': 'CjkUnifiedIdeograph8935',
u'\u68b4': 'CjkUnifiedIdeograph68b4',
u'\uf2b8': 'UnicodeCharacter0xf2b8',
u'\u5e3b': 'CjkUnifiedIdeograph5e3b',
u'\u31ba': 'UnicodeCharacter0x31ba',
u'\ud83f': 'UnicodeCharacter0xd83f',
u'\ubbbe': 'HangulSyllableMyup',
u'\u6145': 'CjkUnifiedIdeograph6145',
u'\u40c4': 'CjkUnifiedIdeograph40c4',
u'\ueb49': 'UnicodeCharacter0xeb49',
u'\ucac8': 'HangulSyllableJjols',
u'\u364b': 'CjkUnifiedIdeograph364b',
u'\u09ca': 'UnicodeCharacter0x9ca',
u'\ub04f': 'HangulSyllableGgeulb',
u'\u93ce': 'CjkUnifiedIdeograph93ce',
u'\u7955': 'CjkUnifiedIdeograph7955',
u'\u58d4': 'CjkUnifiedIdeograph58d4',
u'\uc359': 'HangulSyllableSsyaelt',
u'\ua2d8': 'YiSyllableZzat',
u'\u0e5b': 'ThaiCharacterKhomut',
u'\u885f': 'CjkUnifiedIdeograph885f',
u'\u6bde': 'CjkUnifiedIdeograph6bde',
u'\uf5e2': 'UnicodeCharacter0xf5e2',
u'\u5165': 'CjkUnifiedIdeograph5165',
u'\u30e4': 'KatakanaLetterYa',
u'\udb69': 'UnicodeCharacter0xdb69',
u'\ubae8': 'HangulSyllableMwael',
u'\u606f': 'CjkUnifiedIdeograph606f',
u'\u43ee': 'CjkUnifiedIdeograph43ee',
u'\uea73': 'UnicodeCharacter0xea73',
u'\ucdf2': 'HangulSyllableCwilm',
u'\u2975': 'RightwardsArrowAboveAlmostEqualTo',
u'\u08f4': 'UnicodeCharacter0x8f4',
u'\ub379': 'HangulSyllableDelg',
u'\u92f8': 'CjkUnifiedIdeograph92f8',
u'\u787f': 'CjkUnifiedIdeograph787f',
u'\u5bfe': 'CjkUnifiedIdeograph5bfe',
u'\u2e06': 'RaisedInterpolationMarker',
u'\u0a89': 'GujaratiLetterU',
u'\u6b08': 'CjkUnifiedIdeograph6b08',
u'\u578b': 'CjkUnifiedIdeograph578b',
u'\ua80a': 'SylotiNagriLetterGho',
u'\u948d': 'CjkUnifiedIdeograph948d',
u'\uf50c': 'UnicodeCharacter0xf50c',
u'\ud18f': 'HangulSyllableTyelb',
u'\u0616': 'ArabicSmallHighLigatureAlefWithLamWithYeh',
u'\u4318': 'CjkUnifiedIdeograph4318',
u'\u2f9b': 'KangxiRadicalRun',
u'\u801a': 'CjkUnifiedIdeograph801a',
u'\u6c9d': 'CjkUnifiedIdeograph6c9d',
u'\ucd1c': 'HangulSyllableCoss',
u'\ua99f': 'UnicodeCharacter0xa99f',
u'\uf6a1': 'UnicodeCharacter0xf6a1',
u'\u1e26': 'LatinCapitalLetterHWithDiaeresis',
u'\u5b28': 'CjkUnifiedIdeograph5b28',
u'\u07ab': 'ThaanaOoboofili',
u'\u982a': 'CjkUnifiedIdeograph982a',
u'\u44ad': 'CjkUnifiedIdeograph44ad',
u'\ua52c': 'VaiSyllableBi',
u'\u81af': 'CjkUnifiedIdeograph81af',
u'\ue22e': 'UnicodeCharacter0xe22e',
u'\uceb1': 'HangulSyllableKyanj',
u'\u3338': 'SquarePenihi',
u'\u1fbb': 'GreekCapitalLetterAlphaWithOxia',
u'\u703a': 'CjkUnifiedIdeograph703a',
u'\u5cbd': 'CjkUnifiedIdeograph5cbd',
u'\ubd3c': 'HangulSyllableBwaem',
u'\u99bf': 'CjkUnifiedIdeograph99bf',
u'\ufa3e': 'CjkCompatibilityIdeographfa3e',
u'\ua6c1': 'UnicodeCharacter0xa6c1',
u'\ue3c3': 'UnicodeCharacter0xe3c3',
u'\u0b48': 'OriyaVowelSignAi',
u'\u484a': 'CjkUnifiedIdeograph484a',
u'\u34cd': 'CjkUnifiedIdeograph34cd',
u'\u954c': 'CjkUnifiedIdeograph954c',
u'\u71cf': 'CjkUnifiedIdeograph71cf',
u'\ud24e': 'HangulSyllableTweonh',
u'\ubed1': 'HangulSyllableBbeog',
u'\ufbd3': 'ArabicLetterNgIsolatedForm',
u'\u205a': 'TwoDotPunctuation',
u'\u0cdd': 'UnicodeCharacter0xcdd',
u'\u6d5c': 'CjkUnifiedIdeograph6d5c',
u'\u49df': 'CjkUnifiedIdeograph49df',
u'\uaa5e': 'ChamPunctuationDoubleDanda',
u'\u96e1': 'CjkUnifiedIdeograph96e1',
u'\uf760': 'UnicodeCharacter0xf760',
u'\ud3e3': 'HangulSyllablePyes',
u'\u386a': 'CjkUnifiedIdeograph386a',
u'\u456c': 'CjkUnifiedIdeograph456c',
u'\u21ef': 'UpwardsWhiteDoubleArrowOnPedestal',
u'\u826e': 'CjkUnifiedIdeograph826e',
u'\u6ef1': 'CjkUnifiedIdeograph6ef1',
u'\ucf70': 'HangulSyllableKwa',
u'\uabf3': 'UnicodeCharacter0xabf3',
u'\ue8f5': 'UnicodeCharacter0xe8f5',
u'\u107a': 'MyanmarLetterShanNya',
u'\u5d7c': 'CjkUnifiedIdeograph5d7c',
u'\u39ff': 'CjkUnifiedIdeograph39ff',
u'\u9a7e': 'CjkUnifiedIdeograph9a7e',
u'\ua780': 'LatinCapitalLetterTurnedL',
u'\u0303': 'CombiningTilde',
u'\ue482': 'UnicodeCharacter0xe482',
u'\u4005': 'CjkUnifiedIdeograph4005',
u'\u8d07': 'CjkUnifiedIdeograph8d07',
u'\uca09': 'HangulSyllableJjaeb',
u'\u358c': 'CjkUnifiedIdeograph358c',
u'\u728e': 'CjkUnifiedIdeograph728e',
u'\ubf90': 'HangulSyllableBbwaek',
u'\u1b13': 'BalineseLetterKa',
u'\ufc92': 'ArabicLigatureYehWithZainFinalForm',
u'\u5815': 'CjkUnifiedIdeograph5815',
u'\u6517': 'CjkUnifiedIdeograph6517',
u'\ua219': 'YiSyllableGgiep',
u'\uef1b': 'UnicodeCharacter0xef1b',
u'\u0d9c': 'SinhalaLetterAlpapraanaGayanna',
u'\u4a9e': 'CjkUnifiedIdeograph4a9e',
u'\u97a0': 'CjkUnifiedIdeograph97a0',
u'\ud4a2': 'HangulSyllablePweolp',
u'\u3025': 'HangzhouNumeralFive',
u'\u7d27': 'CjkUnifiedIdeograph7d27',
u'\uba29': 'HangulSyllableMyaelt',
u'\uc72b': 'HangulSyllableYulb',
u'\u22ae': 'DoesNotForce',
u'\u6fb0': 'CjkUnifiedIdeograph6fb0',
u'\uacb2': 'HangulSyllableGyeolm',
u'\u0835': 'UnicodeCharacter0x835',
u'\ue9b4': 'UnicodeCharacter0xe9b4',
u'\u5537': 'CjkUnifiedIdeograph5537',
u'\u9239': 'CjkUnifiedIdeograph9239',
u'\udf3b': 'UnicodeCharacter0xdf3b',
u'\u3abe': 'CjkUnifiedIdeograph3abe',
u'\u47c0': 'CjkUnifiedIdeograph47c0',
u'\u84c2': 'CjkUnifiedIdeograph84c2',
u'\uc1c4': 'HangulSyllableSwae',
u'\u2d47': 'TifinaghLetterYaq',
u'\u6a49': 'CjkUnifiedIdeograph6a49',
u'\ub74b': 'HangulSyllableDdyid',
u'\uf44d': 'UnicodeCharacter0xf44d',
u'\u12ce': 'EthiopicSyllableWo',
u'\u5fd0': 'CjkUnifiedIdeograph5fd0',
u'\u9cd2': 'CjkUnifiedIdeograph9cd2',
u'\ud9d4': 'UnicodeCharacter0xd9d4',
u'\u0557': 'UnicodeCharacter0x557',
u'\ue6d6': 'UnicodeCharacter0xe6d6',
u'\u4259': 'CjkUnifiedIdeograph4259',
u'\u8f5b': 'CjkUnifiedIdeograph8f5b',
u'\ucc5d': 'HangulSyllableCaet',
u'\u37e0': 'CjkUnifiedIdeograph37e0',
u'\u74e2': 'CjkUnifiedIdeograph74e2',
u'\ub1e4': 'HangulSyllableNoek',
u'\u1d67': 'GreekSubscriptSmallLetterGamma',
u'\ufee6': 'ArabicLetterNoonFinalForm',
u'\u5a69': 'CjkUnifiedIdeograph5a69',
u'\u676b': 'CjkUnifiedIdeograph676b',
u'\ua46d': 'YiSyllableXy',
u'\ue16f': 'UnicodeCharacter0xe16f',
u'\u0ff0': 'UnicodeCharacter0xff0',
u'\u4cf2': 'CjkUnifiedIdeograph4cf2',
u'\u89f4': 'CjkUnifiedIdeograph89f4',
u'\ud6f6': 'HangulSyllableHweoj',
u'\u3279': 'CircledHangulThieuthA',
u'\u7f7b': 'CjkUnifiedIdeograph7f7b',
u'\ubc7d': 'HangulSyllableByaeng',
u'\uf97f': 'CjkCompatibilityIdeographf97f',
u'\u8981': 'CjkUnifiedIdeograph8981',
u'\u6e04': 'CjkUnifiedIdeograph6e04',
u'\u4287': 'CjkUnifiedIdeograph4287',
u'\ue408': 'UnicodeCharacter0xe408',
u'\ud88b': 'UnicodeCharacter0xd88b',
u'\u270a': 'UnicodeCharacter0x270a',
u'\u1b8d': 'SundaneseLetterNga',
u'\ubd0e': 'HangulSyllableBop',
u'\u9191': 'CjkUnifiedIdeograph9191',
u'\u7614': 'CjkUnifiedIdeograph7614',
u'\u2a97': 'SlantedEqualToOrLessthanWithDotInside',
u'\ucc18': 'HangulSyllableJjils',
u'\ua09b': 'YiSyllableHmap',
u'\u0f1a': 'TibetanSignRdelDkarGcig',
u'\u851e': 'CjkUnifiedIdeograph851e',
u'\u79a1': 'CjkUnifiedIdeograph79a1',
u'\uffa5': 'HalfwidthHangulLetterNieuncieuc',
u'\u5e24': 'CjkUnifiedIdeograph5e24',
u'\u32a7': 'CircledIdeographLeft',
u'\ud428': 'HangulSyllablePwaen',
u'\u88ab': 'CjkUnifiedIdeograph88ab',
u'\u172a': 'HanunooLetterBa',
u'\u6d2e': 'CjkUnifiedIdeograph6d2e',
u'\u41b1': 'CjkUnifiedIdeograph41b1',
u'\ue332': 'UnicodeCharacter0xe332',
u'\uc7b5': 'HangulSyllableJaelg',
u'\u2634': 'TrigramForWind',
u'\u1ab7': 'UnicodeCharacter0x1ab7',
u'\ubc38': 'HangulSyllableBael',
u'\u90bb': 'CjkUnifiedIdeograph90bb',
u'\u753e': 'CjkUnifiedIdeograph753e',
u'\u29c1': 'CircledGreaterthan',
u'\ucb42': 'HangulSyllableJjyoj',
u'\uafc5': 'HangulSyllableGgult',
u'\u0e44': 'ThaiCharacterSaraAiMaimalai',
u'\u8448': 'CjkUnifiedIdeograph8448',
u'\u78cb': 'CjkUnifiedIdeograph78cb',
u'\ufecf': 'ArabicLetterGhainInitialForm',
u'\u5d4e': 'CjkUnifiedIdeograph5d4e',
u'\u31d1': 'CjkStrokeS',
u'\ud352': 'HangulSyllablePyalp',
u'\ub7d5': 'HangulSyllableRyaenj',
u'\u1654': 'CanadianSyllabicsCarrierShu',
u'\u6c58': 'CjkUnifiedIdeograph6c58',
u'\u40db': 'CjkUnifiedIdeograph40db',
u'\ue25c': 'UnicodeCharacter0xe25c',
u'\uc6df': 'HangulSyllableWeos',
u'\u255e': 'BoxDrawingsVerticalSingleAndRightDouble',
u'\u19e1': 'KhmerSymbolMuoyKoet',
u'\ubb62': 'HangulSyllableMweobs',
u'\u9fe5': 'UnicodeCharacter0x9fe5',
u'\u7468': 'CjkUnifiedIdeograph7468',
u'\u28eb': 'BraillePatternDots124678',
u'\uca6c': 'HangulSyllableJjen',
u'\uaeef': 'HangulSyllableGgec',
u'\u0d6e': 'MalayalamDigitEight',
u'\u8372': 'CjkUnifiedIdeograph8372',
u'\u67f5': 'CjkUnifiedIdeograph67f5',
u'\ufdf9': 'ArabicLigatureSallaIsolatedForm',
u'\u5c78': 'CjkUnifiedIdeograph5c78',
u'\u30fb': 'KatakanaMiddleDot',
u'\ud27c': 'HangulSyllableTwek',
u'\ub6ff': 'HangulSyllableDdwilh',
u'\u157e': 'CanadianSyllabicsQaai',
u'\u8c03': 'CjkUnifiedIdeograph8c03',
u'\u6b82': 'CjkUnifiedIdeograph6b82',
u'\ue186': 'UnicodeCharacter0xe186',
u'\u4509': 'CjkUnifiedIdeograph4509',
u'\u2488': 'DigitOneFullStop',
u'\udb0d': 'UnicodeCharacter0xdb0d',
u'\uba8c': 'HangulSyllableMye',
u'\u1e0f': 'LatinSmallLetterDWithLineBelow',
u'\u9413': 'CjkUnifiedIdeograph9413',
u'\u7392': 'CjkUnifiedIdeograph7392',
u'\uea17': 'UnicodeCharacter0xea17',
u'\uc996': 'HangulSyllableJeulp',
u'\u2d19': 'GeorgianSmallLetterChin',
u'\u0c98': 'KannadaLetterGha',
u'\ua31d': 'YiSyllableSep',
u'\u829c': 'CjkUnifiedIdeograph829c',
u'\u7c23': 'CjkUnifiedIdeograph7c23',
u'\u5ba2': 'CjkUnifiedIdeograph5ba2',
u'\uf227': 'UnicodeCharacter0xf227',
u'\ud1a6': 'HangulSyllableTonh',
u'\u3529': 'CjkUnifiedIdeograph3529',
u'\u14a8': 'CanadianSyllabicsMoo',
u'\u8b2d': 'CjkUnifiedIdeograph8b2d',
u'\u6aac': 'CjkUnifiedIdeograph6aac',
u'\ue0b0': 'UnicodeCharacter0xe0b0',
u'\u4433': 'CjkUnifiedIdeograph4433',
u'\u23b2': 'SummationTop',
u'\uda37': 'UnicodeCharacter0xda37',
u'\ub9b6': 'HangulSyllableRilm',
u'\u1d39': 'ModifierLetterCapitalM',
u'\u933d': 'CjkUnifiedIdeograph933d',
u'\u72bc': 'CjkUnifiedIdeograph72bc',
u'\ue941': 'UnicodeCharacter0xe941',
u'\uc8c0': 'HangulSyllableJwaek',
u'\u2c43': 'GlagoliticSmallLetterRitsi',
u'\u0bc2': 'TamilVowelSignUu',
u'\ua247': 'YiSyllableHxix',
u'\u81c6': 'CjkUnifiedIdeograph81c6',
u'\u7b4d': 'CjkUnifiedIdeograph7b4d',
u'\u5acc': 'CjkUnifiedIdeograph5acc',
u'\uf151': 'UnicodeCharacter0xf151',
u'\ud0d0': 'HangulSyllableTam',
u'\u3453': 'CjkUnifiedIdeograph3453',
u'\u13d2': 'CherokeeLetterSv',
u'\u8a57': 'CjkUnifiedIdeograph8a57',
u'\u69d6': 'CjkUnifiedIdeograph69d6',
u'\uefda': 'UnicodeCharacter0xefda',
u'\u435d': 'CjkUnifiedIdeograph435d',
u'\u22dc': 'EqualToOrLessthan',
u'\ud961': 'UnicodeCharacter0xd961',
u'\ub8e0': 'HangulSyllableRyoss',
u'\u1c63': 'OlChikiLetterAaw',
u'\u9267': 'CjkUnifiedIdeograph9267',
u'\u71e6': 'CjkUnifiedIdeograph71e6',
u'\ue86b': 'UnicodeCharacter0xe86b',
u'\uf7ea': 'UnicodeCharacter0xf7ea',
u'\u2b6d': 'UnicodeCharacter0x2b6d',
u'\u0aec': 'GujaratiDigitSix',
u'\ua171': 'YiSyllableHnax',
u'\u80f0': 'CjkUnifiedIdeograph80f0',
u'\u7a77': 'CjkUnifiedIdeograph7a77',
u'\u59f6': 'CjkUnifiedIdeograph59f6',
u'\uf07b': 'UnicodeCharacter0xf07b',
u'\udffa': 'UnicodeCharacter0xdffa',
u'\u337d': 'SquareEraNameTaisyou',
u'\u12fc': 'EthiopicSyllableDdee',
u'\u0881': 'UnicodeCharacter0x881',
u'\u6900': 'CjkUnifiedIdeograph6900',
u'\u4d83': 'CjkUnifiedIdeograph4d83',
u'\uaa02': 'ChamLetterU',
u'\u8e85': 'CjkUnifiedIdeograph8e85',
u'\uef04': 'UnicodeCharacter0xef04',
u'\uc387': 'HangulSyllableSsegs',
u'\u3c0e': 'CjkUnifiedIdeograph3c0e',
u'\u1091': 'MyanmarShanDigitOne',
u'\u7110': 'CjkUnifiedIdeograph7110',
u'\u5593': 'CjkUnifiedIdeograph5593',
u'\ub212': 'HangulSyllableNulp',
u'\u9695': 'CjkUnifiedIdeograph9695',
u'\uf714': 'UnicodeCharacter0xf714',
u'\uab97': 'UnicodeCharacter0xab97',
u'\uec99': 'UnicodeCharacter0xec99',
u'\u041e': 'CyrillicCapitalLetterO',
u'\u5920': 'CjkUnifiedIdeograph5920',
u'\u3da3': 'CjkUnifiedIdeograph3da3',
u'\u9a22': 'CjkUnifiedIdeograph9a22',
u'\u7ea5': 'CjkUnifiedIdeograph7ea5',
u'\udf24': 'UnicodeCharacter0xdf24',
u'\ub3a7': 'HangulSyllableDyeoh',
u'\uf4a9': 'UnicodeCharacter0xf4a9',
u'\u2130': 'ScriptCapitalE',
u'\u05b3': 'HebrewPointHatafQamats',
u'\u6232': 'CjkUnifiedIdeograph6232',
u'\u46b5': 'CjkUnifiedIdeograph46b5',
u'\ua734': 'LatinCapitalLetterAo',
u'\u9bb7': 'CjkUnifiedIdeograph9bb7',
u'\uf836': 'UnicodeCharacter0xf836',
u'\udcb9': 'UnicodeCharacter0xdcb9',
u'\u0940': 'DevanagariVowelSignIi',
u'\u4a42': 'CjkUnifiedIdeograph4a42',
u'\u2ec5': 'CjkRadicalCsimplifiedSee',
u'\u8f44': 'CjkUnifiedIdeograph8f44',
u'\u63c7': 'CjkUnifiedIdeograph63c7',
u'\uc046': 'HangulSyllableBbyulm',
u'\ua4c9': 'UnicodeCharacter0xa4c9',
u'\uf9cb': 'CjkCompatibilityIdeographf9cb',
u'\u1150': 'HangulChoseongCeongchieumcieuc',
u'\u5252': 'CjkUnifiedIdeograph5252',
u'\u36d5': 'CjkUnifiedIdeograph36d5',
u'\u9754': 'CjkUnifiedIdeograph9754',
u'\u4bd7': 'CjkUnifiedIdeograph4bd7',
u'\ua856': 'PhagspaLetterSmallA',
u'\u8cd9': 'CjkUnifiedIdeograph8cd9',
u'\ued58': 'UnicodeCharacter0xed58',
u'\uc1db': 'HangulSyllableSwaec',
u'\u3a62': 'CjkUnifiedIdeograph3a62',
u'\u1ee5': 'LatinSmallLetterUWithDotBelow',
u'\u7f64': 'CjkUnifiedIdeograph7f64',
u'\u53e7': 'CjkUnifiedIdeograph53e7',
u'\ub066': 'HangulSyllableGgyinh',
u'\u94e9': 'CjkUnifiedIdeograph94e9',
u'\uf568': 'UnicodeCharacter0xf568',
u'\ua9eb': 'UnicodeCharacter0xa9eb',
u'\ueaed': 'UnicodeCharacter0xeaed',
u'\u0272': 'LatinSmallLetterNWithLeftHook',
u'\u4774': 'CjkUnifiedIdeograph4774',
u'\u3bf7': 'CjkUnifiedIdeograph3bf7',
u'\u9876': 'CjkUnifiedIdeograph9876',
u'\u7cf9': 'CjkUnifiedIdeograph7cf9',
u'\udd78': 'UnicodeCharacter0xdd78',
u'\ub1fb': 'HangulSyllableNyos',
u'\uf2fd': 'UnicodeCharacter0xf2fd',
u'\uc801': 'HangulSyllableJeog',
u'\u2f84': 'KangxiRadicalArrive',
u'\u6086': 'CjkUnifiedIdeograph6086',
u'\ua588': 'VaiSyllableShoo',
u'\u190b': 'LimbuLetterTa',
u'\ue68a': 'UnicodeCharacter0xe68a',
u'\u5a0d': 'CjkUnifiedIdeograph5a0d',
u'\u9f0f': 'CjkUnifiedIdeograph9f0f',
u'\ud011': 'HangulSyllableKweong',
u'\u3794': 'CjkUnifiedIdeograph3794',
u'\u4896': 'CjkUnifiedIdeograph4896',
u'\u8d98': 'CjkUnifiedIdeograph8d98',
u'\uce9a': 'HangulSyllableKaelm',
u'\u221d': 'ProportionalTo',
u'\u671f': 'CjkUnifiedIdeograph671f',
u'\ub821': 'HangulSyllableRet',
u'\ufd23': 'ArabicLigatureDadWithAlefMaksuraFinalForm',
u'\u1fa4': 'GreekSmallLetterOmegaWithPsiliAndOxiaAndYpogegrammeni',
u'\u50a6': 'CjkUnifiedIdeograph50a6',
u'\u95a8': 'CjkUnifiedIdeograph95a8',
u'\ud6aa': 'HangulSyllableHyogg',
u'\u0a2d': 'GurmukhiLetterBha',
u'\uebac': 'UnicodeCharacter0xebac',
u'\u4f2f': 'CjkUnifiedIdeograph4f2f',
u'\u8031': 'CjkUnifiedIdeograph8031',
u'\uc533': 'HangulSyllableSsilb',
u'\u38b6': 'CjkUnifiedIdeograph38b6',
u'\u7db8': 'CjkUnifiedIdeograph7db8',
u'\ubeba': 'HangulSyllableBbyaenh',
u'\u123d': 'EthiopicSyllableShe',
u'\uf3bc': 'UnicodeCharacter0xf3bc',
u'\u573f': 'CjkUnifiedIdeograph573f',
u'\u6841': 'CjkUnifiedIdeograph6841',
u'\uad43': 'HangulSyllableGoelh',
u'\uee45': 'UnicodeCharacter0xee45',
u'\xc6': 'LatinCapitalLetterAe',
u'\u45c8': 'CjkUnifiedIdeograph45c8',
u'\u86ca': 'CjkUnifiedIdeograph86ca',
u'\udbcc': 'UnicodeCharacter0xdbcc',
u'\u3f4f': 'CjkUnifiedIdeograph3f4f',
u'\u7051': 'CjkUnifiedIdeograph7051',
u'\ub553': 'HangulSyllableDdaed',
u'\uf655': 'UnicodeCharacter0xf655',
u'\u2dd8': 'EthiopicSyllableGya',
u'\u6eda': 'CjkUnifiedIdeograph6eda',
u'\ua3dc': 'YiSyllableRy',
u'\u075f': 'ArabicLetterAinWithTwoDotsVerticallyAbove',
u'\ue4de': 'UnicodeCharacter0xe4de',
u'\u5861': 'CjkUnifiedIdeograph5861',
u'\u9d63': 'CjkUnifiedIdeograph9d63',
u'\ude65': 'UnicodeCharacter0xde65',
u'\u35e8': 'CjkUnifiedIdeograph35e8',
u'\u76ea': 'CjkUnifiedIdeograph76ea',
u'\u8bec': 'CjkUnifiedIdeograph8bec',
u'\uccee': 'HangulSyllableCyegg',
u'\u2071': 'SuperscriptLatinSmallLetterI',
u'\u6573': 'CjkUnifiedIdeograph6573',
u'\ua675': 'UnicodeCharacter0xa675',
u'\ufb77': 'ArabicLetterNyehFinalForm',
u'\u1df8': 'UnicodeCharacter0x1df8',
u'\u5efa': 'CjkUnifiedIdeograph5efa',
u'\u93fc': 'CjkUnifiedIdeograph93fc',
u'\ud4fe': 'HangulSyllablePyuj',
u'\uea00': 'UnicodeCharacter0xea00',
u'\uce83': 'HangulSyllableKalh',
u'\u2902': 'LeftwardsDoubleArrowWithVerticalStroke',
u'\u0d85': 'SinhalaLetterAyanna',
u'\ua306': 'YiSyllableNzyp',
u'\u8789': 'CjkUnifiedIdeograph8789',
u'\u780c': 'CjkUnifiedIdeograph780c',
u'\u5c8f': 'CjkUnifiedIdeograph5c8f',
u'\uf210': 'UnicodeCharacter0xf210',
u'\ud693': 'HangulSyllableHoed',
u'\u3112': 'BopomofoLetterX',
u'\u1595': 'CanadianSyllabicsNg',
u'\u8b16': 'CjkUnifiedIdeograph8b16',
u'\u6f99': 'CjkUnifiedIdeograph6f99',
u'\ue19d': 'UnicodeCharacter0xe19d',
u'\u401c': 'CjkUnifiedIdeograph401c',
u'\u249f': 'ParenthesizedLatinSmallLetterD',
u'\uda20': 'UnicodeCharacter0xda20',
u'\ubea3': 'HangulSyllableBbyalb',
u'\u1922': 'LimbuVowelSignU',
u'\u9326': 'CjkUnifiedIdeograph9326',
u'\u77a9': 'CjkUnifiedIdeograph77a9',
u'\uc9ad': 'HangulSyllableJyilg',
u'\u282c': 'BraillePatternDots346',
u'\u0caf': 'KannadaLetterYa',
u'\ua230': 'YiSyllableMgiex',
u'\u86b3': 'CjkUnifiedIdeograph86b3',
u'\u7b36': 'CjkUnifiedIdeograph7b36',
u'\u5fb9': 'CjkUnifiedIdeograph5fb9',
u'\ufd3a': 'ArabicLigatureTahWithMeemMedialForm',
u'\ud1bd': 'HangulSyllableTwag',
u'\u303c': 'MasuMark',
u'\u14bf': 'CanadianSyllabicsSayisiM',
u'\u8a40': 'CjkUnifiedIdeograph8a40',
u'\u6ec3': 'CjkUnifiedIdeograph6ec3',
u'\ue0c7': 'UnicodeCharacter0xe0c7',
u'\u4346': 'CjkUnifiedIdeograph4346',
u'\u27c9': 'SupersetPrecedingSolidus',
u'\uc54a': 'HangulSyllableAnh',
u'\ub9cd': 'HangulSyllableManj',
u'\u184c': 'MongolianLetterTodoPa',
u'\u9250': 'CjkUnifiedIdeograph9250',
u'\u76d3': 'CjkUnifiedIdeograph76d3',
u'\uc8d7': 'HangulSyllableJoes',
u'\u2b56': 'UnicodeCharacter0x2b56',
u'\u0fd9': 'UnicodeCharacter0xfd9',
u'\uad5a': 'HangulSyllableGyolm',
u'\u81dd': 'CjkUnifiedIdeograph81dd',
u'\u7a60': 'CjkUnifiedIdeograph7a60',
u'\u5ee3': 'CjkUnifiedIdeograph5ee3',
u'\ufc64': 'ArabicLigatureYehWithHamzaAboveWithRehFinalForm',
u'\ud0e7': 'HangulSyllableTaelb',
u'\u3366': 'IdeographicTelegraphSymbolForHourFourteen',
u'\u17e9': 'KhmerDigitNine',
u'\ub56a': 'HangulSyllableDdyagg',
u'\u69ed': 'CjkUnifiedIdeograph69ed',
u'\ue3f1': 'UnicodeCharacter0xe3f1',
u'\u4270': 'CjkUnifiedIdeograph4270',
u'\u26f3': 'UnicodeCharacter0x26f3',
u'\uc474': 'HangulSyllableSsum',
u'\ub8f7': 'HangulSyllableRulh',
u'\u1b76': 'BalineseMusicalSymbolRighthandClosedTuk',
u'\u9d7a': 'CjkUnifiedIdeograph9d7a',
u'\u71fd': 'CjkUnifiedIdeograph71fd',
u'\u4b01': 'CjkUnifiedIdeograph4b01',
u'\u2a80': 'GreaterthanOrSlantedEqualToWithDotInside',
u'\ucd05': 'HangulSyllableCyet',
u'\uac84': 'HangulSyllableGeoss',
u'\u9a0b': 'CjkUnifiedIdeograph9a0b',
u'\u658a': 'CjkUnifiedIdeograph658a',
u'\uff8e': 'HalfwidthKatakanaLetterHo',
u'\u5311': 'CjkUnifiedIdeograph5311',
u'\u3290': 'CircledIdeographSun',
u'\ud515': 'HangulSyllablePeub',
u'\ub494': 'HangulSyllableDwels',
u'\u621b': 'CjkUnifiedIdeograph621b',
u'\u4d9a': 'CjkUnifiedIdeograph4d9a',
u'\ue41f': 'UnicodeCharacter0xe41f',
u'\uc79e': 'HangulSyllableJalp',
u'\u3b21': 'CjkUnifiedIdeograph3b21',
u'\u1aa0': 'UnicodeCharacter0x1aa0',
u'\ubd25': 'HangulSyllableBwang',
u'\u9ca4': 'CjkUnifiedIdeograph9ca4',
u'\u4a2b': 'CjkUnifiedIdeograph4a2b',
u'\u55aa': 'CjkUnifiedIdeograph55aa',
u'\ucc2f': 'HangulSyllableCad',
u'\uafae': 'HangulSyllableGgyobs',
u'\u0331': 'CombiningMacronBelow',
u'\u8535': 'CjkUnifiedIdeograph8535',
u'\u64b4': 'CjkUnifiedIdeograph64b4',
u'\ufeb8': 'ArabicLetterSheenMedialForm',
u'\u523b': 'CjkUnifiedIdeograph523b',
u'\u3dba': 'CjkUnifiedIdeograph3dba',
u'\ud43f': 'HangulSyllablePwaeh',
u'\ub7be': 'HangulSyllableRyalm',
u'\u6d45': 'CjkUnifiedIdeograph6d45',
u'\u4cc4': 'CjkUnifiedIdeograph4cc4',
u'\ue749': 'UnicodeCharacter0xe749',
u'\uc6c8': 'HangulSyllableUk',
u'\u3a4b': 'CjkUnifiedIdeograph3a4b',
u'\u05ca': 'UnicodeCharacter0x5ca',
u'\ubc4f': 'HangulSyllableByags',
u'\u9fce': 'UnicodeCharacter0x9fce',
u'\u7555': 'CjkUnifiedIdeograph7555',
u'\u54d4': 'CjkUnifiedIdeograph54d4',
u'\ucf59': 'HangulSyllableKonj',
u'\uaed8': 'HangulSyllableGge',
u'\u025b': 'LatinSmallLetterOpenE',
u'\u845f': 'CjkUnifiedIdeograph845f',
u'\u67de': 'CjkUnifiedIdeograph67de',
u'\uf9e2': 'CjkCompatibilityIdeographf9e2',
u'\u5d65': 'CjkUnifiedIdeograph5d65',
u'\u3ce4': 'CjkUnifiedIdeograph3ce4',
u'\ud769': 'HangulSyllableHeut',
u'\ub6e8': 'HangulSyllableDdwess',
u'\u6c6f': 'CjkUnifiedIdeograph6c6f',
u'\u4fee': 'CjkUnifiedIdeograph4fee',
u'\ue673': 'UnicodeCharacter0xe673',
u'\uc1f2': 'HangulSyllableSoebs',
u'\u2575': 'BoxDrawingsLightUp',
u'\u04f4': 'CyrillicCapitalLetterCheWithDiaeresis',
u'\ubf79': 'HangulSyllableBbwaeg',
u'\u9ef8': 'CjkUnifiedIdeograph9ef8',
u'\u747f': 'CjkUnifiedIdeograph747f',
u'\u57fe': 'CjkUnifiedIdeograph57fe',
u'\u2206': 'Increment',
u'\u0689': 'ArabicLetterDalWithRing',
u'\u6708': 'CjkUnifiedIdeograph6708',
u'\u5b8b': 'CjkUnifiedIdeograph5b8b',
u'\ua40a': 'YiSyllableQo',
u'\u988d': 'CjkUnifiedIdeograph988d',
u'\uf90c': 'CjkCompatibilityIdeographf90c',
u'\udd8f': 'UnicodeCharacter0xdd8f',
u'\u0a16': 'GurmukhiLetterKha',
u'\u4f18': 'CjkUnifiedIdeograph4f18',
u'\u239b': 'LeftParenthesisUpperHook',
u'\u8c1a': 'CjkUnifiedIdeograph8c1a',
u'\u609d': 'CjkUnifiedIdeograph609d',
u'\uc11c': 'HangulSyllableSeo',
u'\ua59f': 'VaiSyllableKpu',
u'\ufaa1': 'CjkCompatibilityIdeographfaa1',
u'\u1226': 'EthiopicSyllableSzo',
u'\u5728': 'CjkUnifiedIdeograph5728',
u'\u0bab': 'UnicodeCharacter0xbab',
u'\u942a': 'CjkUnifiedIdeograph942a',
u'\u48ad': 'CjkUnifiedIdeograph48ad',
u'\ua92c': 'KayahLiToneCalya',
u'\u8daf': 'CjkUnifiedIdeograph8daf',
u'\uee2e': 'UnicodeCharacter0xee2e',
u'\uc2b1': 'HangulSyllableSeult',
u'\u3f38': 'CjkUnifiedIdeograph3f38',
u'\u13bb': 'CherokeeLetterMi',
u'\u7c3a': 'CjkUnifiedIdeograph7c3a',
u'\u50bd': 'CjkUnifiedIdeograph50bd',
u'\ub13c': 'HangulSyllableNek',
u'\u95bf': 'CjkUnifiedIdeograph95bf',
u'\uf63e': 'UnicodeCharacter0xf63e',
u'\uaac1': 'UnicodeCharacter0xaac1',
u'\uefc3': 'UnicodeCharacter0xefc3',
u'\u0748': 'SyriacObliqueLineBelow',
u'\u444a': 'CjkUnifiedIdeograph444a',
u'\u38cd': 'CjkUnifiedIdeograph38cd',
u'\u994c': 'CjkUnifiedIdeograph994c',
u'\u7dcf': 'CjkUnifiedIdeograph7dcf',
u'\ude4e': 'UnicodeCharacter0xde4e',
u'\ub2d1': 'HangulSyllableNilg',
u'\uf7d3': 'UnicodeCharacter0xf7d3',
u'\u2c5a': 'GlagoliticSmallLetterFita',
u'\xdd': 'LatinCapitalLetterYWithAcute',
u'\u615c': 'CjkUnifiedIdeograph615c',
u'\u45df': 'CjkUnifiedIdeograph45df',
u'\ua65e': 'CyrillicCapitalLetterYn',
u'\u9ae1': 'CjkUnifiedIdeograph9ae1',
u'\ufb60': 'ArabicLetterTtehehInitialForm',
u'\udfe3': 'UnicodeCharacter0xdfe3',
u'\u346a': 'CjkUnifiedIdeograph346a',
u'\u496c': 'CjkUnifiedIdeograph496c',
u'\u2def': 'CombiningCyrillicLetterHa',
u'\u8e6e': 'CjkUnifiedIdeograph8e6e',
u'\u62f1': 'CjkUnifiedIdeograph62f1',
u'\uc370': 'HangulSyllableSseol',
u'\ua7f3': 'UnicodeCharacter0xa7f3',
u'\ue4f5': 'UnicodeCharacter0xe4f5',
u'\u1c7a': 'OlChikiMugaahlaaTtuddaag',
u'\u517c': 'CjkUnifiedIdeograph517c',
u'\u35ff': 'CjkUnifiedIdeograph35ff',
u'\u967e': 'CjkUnifiedIdeograph967e',
u'\uab80': 'UnicodeCharacter0xab80',
u'\u0f03': 'TibetanMarkGterYigMgoUmGterTshegMa',
u'\ue882': 'UnicodeCharacter0xe882',
u'\u4c05': 'CjkUnifiedIdeograph4c05',
u'\u8107': 'CjkUnifiedIdeograph8107',
u'\uc609': 'HangulSyllableYeg',
u'\u398c': 'CjkUnifiedIdeograph398c',
u'\u7e8e': 'CjkUnifiedIdeograph7e8e',
u'\ub390': 'HangulSyllableDyeon',
u'\u1713': 'TagalogVowelSignU',
u'\uf092': 'UnicodeCharacter0xf092',
u'\u5415': 'CjkUnifiedIdeograph5415',
u'\u6917': 'CjkUnifiedIdeograph6917',
u'\uae19': 'HangulSyllableGyinj',
u'\ue31b': 'UnicodeCharacter0xe31b',
u'\u019c': 'LatinCapitalLetterTurnedM',
u'\u469e': 'CjkUnifiedIdeograph469e',
u'\u9ba0': 'CjkUnifiedIdeograph9ba0',
u'\ud8a2': 'UnicodeCharacter0xd8a2',
u'\u3c25': 'CjkUnifiedIdeograph3c25',
u'\u7127': 'CjkUnifiedIdeograph7127',
u'\ub629': 'HangulSyllableDdot',
u'\ucb2b': 'HangulSyllableJjoeh',
u'\u2eae': 'CjkRadicalBamboo',
u'\u63b0': 'CjkUnifiedIdeograph63b0',
u'\ua0b2': 'YiSyllableMiex',
u'\u0435': 'CyrillicSmallLetterIe',
u'\ue5b4': 'UnicodeCharacter0xe5b4',
u'\u5937': 'CjkUnifiedIdeograph5937',
u'\u9e39': 'CjkUnifiedIdeograph9e39',
u'\ud33b': 'HangulSyllablePaes',
u'\u36be': 'CjkUnifiedIdeograph36be',
u'\u4bc0': 'CjkUnifiedIdeograph4bc0',
u'\u88c2': 'CjkUnifiedIdeograph88c2',
u'\ucdc4': 'HangulSyllableCweoss',
u'\u2147': 'DoublestruckItalicSmallE',
u'\u6649': 'CjkUnifiedIdeograph6649',
u'\ubb4b': 'HangulSyllableMuc',
u'\uf84d': 'UnicodeCharacter0xf84d',
u'\u1ece': 'LatinCapitalLetterOWithHookAbove',
u'\u53d0': 'CjkUnifiedIdeograph53d0',
u'\u90d2': 'CjkUnifiedIdeograph90d2',
u'\ud5d4': 'HangulSyllableHeols',
u'\u0957': 'UnicodeCharacter0x957',
u'\uead6': 'UnicodeCharacter0xead6',
u'\u4e59': 'CjkUnifiedIdeograph4e59',
u'\u835b': 'CjkUnifiedIdeograph835b',
u'\uc05d': 'HangulSyllableBbeunj',
u'\u3be0': 'CjkUnifiedIdeograph3be0',
u'\u78e2': 'CjkUnifiedIdeograph78e2',
u'\ubde4': 'HangulSyllableBwim',
u'\u1167': 'HangulJungseongYeo',
u'\uf2e6': 'UnicodeCharacter0xf2e6',
u'\u5669': 'CjkUnifiedIdeograph5669',
u'\u6b6b': 'CjkUnifiedIdeograph6b6b',
u'\ua86d': 'PhagspaLetterAlternateYa',
u'\ued6f': 'UnicodeCharacter0xed6f',
u'\u03f0': 'GreekKappaSymbol',
u'\u40f2': 'CjkUnifiedIdeograph40f2',
u'\u85f4': 'CjkUnifiedIdeograph85f4',
u'\udaf6': 'UnicodeCharacter0xdaf6',
u'\u3e79': 'CjkUnifiedIdeograph3e79',
u'\u737b': 'CjkUnifiedIdeograph737b',
u'\ub07d': 'HangulSyllableGgig',
u'\uf57f': 'UnicodeCharacter0xf57f',
u'\u8d81': 'CjkUnifiedIdeograph8d81',
u'\u6a04': 'CjkUnifiedIdeograph6a04',
u'\u4687': 'CjkUnifiedIdeograph4687',
u'\ue008': 'UnicodeCharacter0xe008',
u'\udc8b': 'UnicodeCharacter0xdc8b',
u'\u230a': 'LeftFloor',
u'\u1f8d': 'GreekCapitalLetterAlphaWithDasiaAndOxiaAndProsgegrammeni',
u'\ub90e': 'HangulSyllableRweolm',
u'\u9591': 'CjkUnifiedIdeograph9591',
u'\u7214': 'CjkUnifiedIdeograph7214',
u'\u2e97': 'CjkRadicalHeartTwo',
u'\uc818': 'HangulSyllableJeok',
u'\ua49b': 'YiRadicalBbut',
u'\u0b1a': 'OriyaLetterCa',
u'\u811e': 'CjkUnifiedIdeograph811e',
u'\u7da1': 'CjkUnifiedIdeograph7da1',
u'\ufba5': 'ArabicLetterHehWithYehAboveFinalForm',
u'\u5a24': 'CjkUnifiedIdeograph5a24',
u'\u36a7': 'CjkUnifiedIdeograph36a7',
u'\ud028': 'HangulSyllableKwem',
u'\u8cab': 'CjkUnifiedIdeograph8cab',
u'\u132a': 'EthiopicSyllableChi',
u'\u692e': 'CjkUnifiedIdeograph692e',
u'\u45b1': 'CjkUnifiedIdeograph45b1',
u'\ue732': 'UnicodeCharacter0xe732',
u'\uc3b5': 'HangulSyllableSsyeong',
u'\u2234': 'Therefore',
u'\u1eb7': 'LatinSmallLetterAWithBreveAndDotBelow',
u'\ub838': 'HangulSyllableRyeoss',
u'\u94bb': 'CjkUnifiedIdeograph94bb',
u'\u713e': 'CjkUnifiedIdeograph713e',
u'\u2dc1': 'EthiopicSyllableQyu',
u'\ucf42': 'HangulSyllableKyelm',
u'\uabc5': 'UnicodeCharacter0xabc5',
u'\u0a44': 'UnicodeCharacter0xa44',
u'\u8048': 'CjkUnifiedIdeograph8048',
u'\u7ccb': 'CjkUnifiedIdeograph7ccb',
u'\ufacf': 'CjkCompatibilityIdeographfacf',
u'\u594e': 'CjkUnifiedIdeograph594e',
u'\u35d1': 'CjkUnifiedIdeograph35d1',
u'\ud752': 'HangulSyllableHeugg',
u'\ub3d5': 'HangulSyllableDob',
u'\u1254': 'EthiopicSyllableQhee',
u'\u6858': 'CjkUnifiedIdeograph6858',
u'\u44db': 'CjkUnifiedIdeograph44db',
u'\ue65c': 'UnicodeCharacter0xe65c',
u'\uc2df': 'HangulSyllableSigs',
u'\u215e': 'VulgarFractionSevenEighths',
u'\u1de1': 'CombiningLatinLetterSmallCapitalN',
u'\ubf62': 'HangulSyllableBbwanh',
u'\u9be5': 'CjkUnifiedIdeograph9be5',
u'\u7068': 'CjkUnifiedIdeograph7068',
u'\u2ceb': 'UnicodeCharacter0x2ceb',
u'\uce6c': 'HangulSyllableCiss',
u'\uaaef': 'UnicodeCharacter0xaaef',
u'\u096e': 'DevanagariDigitEight',
u'\u8772': 'CjkUnifiedIdeograph8772',
u'\u63f5': 'CjkUnifiedIdeograph63f5',
u'\uf9f9': 'CjkCompatibilityIdeographf9f9',
u'\u5878': 'CjkUnifiedIdeograph5878',
u'\u34fb': 'CjkUnifiedIdeograph34fb',
u'\ud67c': 'HangulSyllableHwaels',
u'\ub2ff': 'HangulSyllableDah',
u'\u117e': 'HangulJungseongYeou',
u'\u8803': 'CjkUnifiedIdeograph8803',
u'\u6f82': 'CjkUnifiedIdeograph6f82',
u'\ue586': 'UnicodeCharacter0xe586',
u'\u4109': 'CjkUnifiedIdeograph4109',
u'\u2088': 'SubscriptEight',
u'\udf0d': 'UnicodeCharacter0xdf0d',
u'\ube8c': 'HangulSyllableBbaem',
u'\u1a0f': 'BugineseLetterNyca',
u'\u9013': 'CjkUnifiedIdeograph9013',
u'\u7792': 'CjkUnifiedIdeograph7792',
u'\uee17': 'UnicodeCharacter0xee17',
u'\ucd96': 'HangulSyllableCugg',
u'\u2919': 'LeftwardsArrowtail',
u'\u0898': 'UnicodeCharacter0x898',
u'\ua71d': 'ModifierLetterRaisedExclamationMark',
u'\u869c': 'CjkUnifiedIdeograph869c',
u'\u7823': 'CjkUnifiedIdeograph7823',
u'\u5fa2': 'CjkUnifiedIdeograph5fa2',
u'\uf627': 'UnicodeCharacter0xf627',
u'\ud5a6': 'HangulSyllableHyaj',
u'\u3129': 'BopomofoLetterIu',
u'\u10a8': 'GeorgianCapitalLetterIn',
u'\u8f2d': 'CjkUnifiedIdeograph8f2d',
u'\u6eac': 'CjkUnifiedIdeograph6eac',
u'\ue4b0': 'UnicodeCharacter0xe4b0',
u'\u4033': 'CjkUnifiedIdeograph4033',
u'\u27b2': 'CircledHeavyWhiteRightwardsArrow',
u'\ude37': 'UnicodeCharacter0xde37',
u'\ubdb6': 'HangulSyllableBweop',
u'\u1939': 'LimbuSignMukphreng',
u'\u973d': 'CjkUnifiedIdeograph973d',
u'\u76bc': 'CjkUnifiedIdeograph76bc',
u'\ued41': 'UnicodeCharacter0xed41',
u'\uccc0': 'HangulSyllableCels',
u'\u2843': 'BraillePatternDots127',
u'\u0fc2': 'TibetanCantillationSignCangTeu',
u'\ua647': 'CyrillicSmallLetterIota',
u'\u85c6': 'CjkUnifiedIdeograph85c6',
u'\u7f4d': 'CjkUnifiedIdeograph7f4d',
u'\u5ecc': 'CjkUnifiedIdeograph5ecc',
u'\uf551': 'UnicodeCharacter0xf551',
u'\ud4d0': 'HangulSyllablePwin',
u'\u3053': 'HiraganaLetterKo',
u'\u17d2': 'KhmerSignCoeng',
u'\u8e57': 'CjkUnifiedIdeograph8e57',
u'\u6dd6': 'CjkUnifiedIdeograph6dd6',
u'\uebda': 'UnicodeCharacter0xebda',
u'\u475d': 'CjkUnifiedIdeograph475d',
u'\u26dc': 'UnicodeCharacter0x26dc',
u'\udd61': 'UnicodeCharacter0xdd61',
u'\ubce0': 'HangulSyllableByel',
u'\u1863': 'MongolianLetterSibeKa',
u'\u9667': 'CjkUnifiedIdeograph9667',
u'\u75e6': 'CjkUnifiedIdeograph75e6',
u'\uec6b': 'UnicodeCharacter0xec6b',
u'\uf3ea': 'UnicodeCharacter0xf3ea',
u'\u2f6d': 'KangxiRadicalSpear',
u'\u0eec': 'UnicodeCharacter0xeec',
u'\ua571': 'VaiSyllableOo',
u'\u84f0': 'CjkUnifiedIdeograph84f0',
u'\u7e77': 'CjkUnifiedIdeograph7e77',
u'\u5df6': 'CjkUnifiedIdeograph5df6',
u'\uf47b': 'UnicodeCharacter0xf47b',
u'\udbfa': 'UnicodeCharacter0xdbfa',
u'\u377d': 'CjkUnifiedIdeograph377d',
u'\u16fc': 'UnicodeCharacter0x16fc',
u'\u0c81': 'UnicodeCharacter0xc81',
u'\u6d00': 'CjkUnifiedIdeograph6d00',
u'\u4983': 'CjkUnifiedIdeograph4983',
u'\uae02': 'HangulSyllableGeulm',
u'\u8a85': 'CjkUnifiedIdeograph8a85',
u'\ueb04': 'UnicodeCharacter0xeb04',
u'\uc787': 'HangulSyllableIs',
u'\u380e': 'CjkUnifiedIdeograph380e',
u'\u1491': 'CanadianSyllabicsCaa',
u'\u7510': 'CjkUnifiedIdeograph7510',
u'\u5193': 'CjkUnifiedIdeograph5193',
u'\ub612': 'HangulSyllableDdogg',
u'\u9295': 'CjkUnifiedIdeograph9295',
u'\uf314': 'UnicodeCharacter0xf314',
u'\uaf97': 'HangulSyllableGgoec',
u'\ue899': 'UnicodeCharacter0xe899',
u'\u5d20': 'CjkUnifiedIdeograph5d20',
u'\u39a3': 'CjkUnifiedIdeograph39a3',
u'\u9e22': 'CjkUnifiedIdeograph9e22',
u'\u7aa5': 'CjkUnifiedIdeograph7aa5',
u'\udb24': 'UnicodeCharacter0xdb24',
u'\ub7a7': 'HangulSyllableRaelh',
u'\uf0a9': 'UnicodeCharacter0xf0a9',
u'\u2530': 'BoxDrawingsDownHeavyAndHorizontalLight',
u'\u01b3': 'LatinCapitalLetterYWithHook',
u'\u6632': 'CjkUnifiedIdeograph6632',
u'\u42b5': 'CjkUnifiedIdeograph42b5',
u'\ua334': 'YiSyllableSsap',
u'\u9fb7': 'CjkUnifiedIdeograph9fb7',
u'\ufc36': 'ArabicLigatureQafWithYehIsolatedForm',
u'\ud8b9': 'UnicodeCharacter0xd8b9',
u'\u0d40': 'MalayalamVowelSignIi',
u'\u4e42': 'CjkUnifiedIdeograph4e42',
u'\u2ac5': 'SubsetOfAboveEqualsSign',
u'\u8b44': 'CjkUnifiedIdeograph8b44',
u'\u67c7': 'CjkUnifiedIdeograph67c7',
u'\uc446': 'HangulSyllableSsoep',
u'\ua0c9': 'YiSyllableMyt',
u'\ufdcb': 'UnicodeCharacter0xfdcb',
u'\u1550': 'CanadianSyllabicsR',
u'\u5652': 'CjkUnifiedIdeograph5652',
u'\u32d5': 'CircledKatakanaKa',
u'\u9354': 'CjkUnifiedIdeograph9354',
u'\u4fd7': 'CjkUnifiedIdeograph4fd7',
u'\uac56': 'HangulSyllableGyaegg',
u'\u88d9': 'CjkUnifiedIdeograph88d9',
u'\ue958': 'UnicodeCharacter0xe958',
u'\uc5db': 'HangulSyllableElb',
u'\u3e62': 'CjkUnifiedIdeograph3e62',
u'\u1ae5': 'UnicodeCharacter0x1ae5',
u'\u7b64': 'CjkUnifiedIdeograph7b64',
u'\u57e7': 'CjkUnifiedIdeograph57e7',
u'\ub466': 'HangulSyllableDuj',
u'\u90e9': 'CjkUnifiedIdeograph90e9',
u'\uf168': 'UnicodeCharacter0xf168',
u'\uadeb': 'HangulSyllableGyulh',
u'\ueeed': 'UnicodeCharacter0xeeed',
u'\u0672': 'ArabicLetterAlefWithWavyHamzaAbove',
u'\u4374': 'CjkUnifiedIdeograph4374',
u'\u3ff7': 'CjkUnifiedIdeograph3ff7',
u'\u9c76': 'CjkUnifiedIdeograph9c76',
u'\u78f9': 'CjkUnifiedIdeograph78f9',
u'\ud978': 'UnicodeCharacter0xd978',
u'\ub5fb': 'HangulSyllableDdyed',
u'\uf6fd': 'UnicodeCharacter0xf6fd',
u'\ucc01': 'HangulSyllableJjyib',
u'\u2b84': 'UnicodeCharacter0x2b84',
u'\u6486': 'CjkUnifiedIdeograph6486',
u'\ua188': 'YiSyllableNuo',
u'\u1d0b': 'LatinLetterSmallCapitalK',
u'\ue28a': 'UnicodeCharacter0xe28a',
u'\u5e0d': 'CjkUnifiedIdeograph5e0d',
u'\u9b0f': 'CjkUnifiedIdeograph9b0f',
u'\ud411': 'HangulSyllablePwalg',
u'\u3394': 'SquareThz',
u'\u4c96': 'CjkUnifiedIdeograph4c96',
u'\u8998': 'CjkUnifiedIdeograph8998',
u'\uca9a': 'HangulSyllableJjyeoj',
u'\u261d': 'WhiteUpPointingIndex',
u'\u631f': 'CjkUnifiedIdeograph631f',
u'\ubc21': 'HangulSyllableBalt',
u'\uf923': 'CjkCompatibilityIdeographf923',
u'\u1ba4': 'SundaneseVowelSignPanghulu',
u'\u54a6': 'CjkUnifiedIdeograph54a6',
u'\u91a8': 'CjkUnifiedIdeograph91a8',
u'\ud2aa': 'HangulSyllableTyulp',
u'\u0e2d': 'ThaiCharacterOAng',
u'\uefac': 'UnicodeCharacter0xefac',
u'\u4b2f': 'CjkUnifiedIdeograph4b2f',
u'\u8431': 'CjkUnifiedIdeograph8431',
u'\uc133': 'HangulSyllableSeoc',
u'\u3cb6': 'CjkUnifiedIdeograph3cb6',
u'\u79b8': 'CjkUnifiedIdeograph79b8',
u'\ubaba': 'HangulSyllableMobs',
u'\u163d': 'CanadianSyllabicsCarrierTlee',
u'\uf7bc': 'UnicodeCharacter0xf7bc',
u'\u533f': 'CjkUnifiedIdeograph533f',
u'\u6c41': 'CjkUnifiedIdeograph6c41',
u'\ua943': 'RejangLetterNgga',
u'\uea45': 'UnicodeCharacter0xea45',
u'\u04c6': 'CyrillicSmallLetterElWithTail',
u'\u41c8': 'CjkUnifiedIdeograph41c8',
u'\u82ca': 'CjkUnifiedIdeograph82ca',
u'\udfcc': 'UnicodeCharacter0xdfcc',
u'\u3b4f': 'CjkUnifiedIdeograph3b4f',
u'\u7451': 'CjkUnifiedIdeograph7451',
u'\ub153': 'HangulSyllableNyeos',
u'\uf255': 'UnicodeCharacter0xf255',
u'\u29d8': 'LeftWigglyFence',
u'\u6ada': 'CjkUnifiedIdeograph6ada',
u'\ua7dc': 'UnicodeCharacter0xa7dc',
u'\u035f': 'CombiningDoubleMacronBelow',
u'\ue0de': 'UnicodeCharacter0xe0de',
u'\u5c61': 'CjkUnifiedIdeograph5c61',
u'\u9963': 'CjkUnifiedIdeograph9963',
u'\uda65': 'UnicodeCharacter0xda65',
u'\u31e8': 'UnicodeCharacter0x31e8',
u'\u72ea': 'CjkUnifiedIdeograph72ea',
u'\u8fec': 'CjkUnifiedIdeograph8fec',
u'\uc8ee': 'HangulSyllableJyolp',
u'\u2471': 'CircledNumberEighteen',
u'\u6173': 'CjkUnifiedIdeograph6173',
u'\ua275': 'YiSyllableHuot',
u'\uff77': 'HalfwidthKatakanaLetterKi',
u'\u19f8': 'KhmerSymbolPrambeiRoc',
u'\u5afa': 'CjkUnifiedIdeograph5afa',
u'\u97fc': 'CjkUnifiedIdeograph97fc',
u'\ud0fe': 'HangulSyllableTyanh',
u'\uee00': 'UnicodeCharacter0xee00',
u'\uca83': 'HangulSyllableJjeh',
u'\u2d02': 'GeorgianSmallLetterGan',
u'\u0985': 'BengaliLetterA',
u'\ua706': 'ModifierLetterChineseToneYinRu',
u'\u8389': 'CjkUnifiedIdeograph8389',
u'\u7c0c': 'CjkUnifiedIdeograph7c0c',
u'\u588f': 'CjkUnifiedIdeograph588f',
u'\uf610': 'UnicodeCharacter0xf610',
u'\ud293': 'HangulSyllableTwis',
u'\u3512': 'CjkUnifiedIdeograph3512',
u'\u1195': 'HangulJungseongEuu',
u'\u8f16': 'CjkUnifiedIdeograph8f16',
u'\u6b99': 'CjkUnifiedIdeograph6b99',
u'\ue59d': 'UnicodeCharacter0xe59d',
u'\u441c': 'CjkUnifiedIdeograph441c',
u'\u209f': 'UnicodeCharacter0x209f',
u'\ude20': 'UnicodeCharacter0xde20',
u'\ubaa3': 'HangulSyllableMyec',
u'\u1d22': 'LatinLetterSmallCapitalZ',
u'\u9726': 'CjkUnifiedIdeograph9726',
u'\u73a9': 'CjkUnifiedIdeograph73a9',
u'\ucdad': 'HangulSyllableCut',
u'\u2c2c': 'GlagoliticCapitalLetterShtapic',
u'\u08af': 'UnicodeCharacter0x8af',
u'\ua630': 'UnicodeCharacter0xa630',
u'\u82b3': 'CjkUnifiedIdeograph82b3',
u'\u7f36': 'CjkUnifiedIdeograph7f36',
u'\u5bb9': 'CjkUnifiedIdeograph5bb9',
u'\uf93a': 'CjkCompatibilityIdeographf93a',
u'\ud5bd': 'HangulSyllableHyaeb',
u'\u343c': 'CjkUnifiedIdeograph343c',
u'\u10bf': 'GeorgianCapitalLetterJhan',
u'\u8e40': 'CjkUnifiedIdeograph8e40',
u'\u6ac3': 'CjkUnifiedIdeograph6ac3',
u'\ue4c7': 'UnicodeCharacter0xe4c7',
u'\u4746': 'CjkUnifiedIdeograph4746',
u'\u23c9': 'DentistrySymbolLightDownAndHorizontal',
u'\uc14a': 'HangulSyllableSebs',
u'\ubdcd': 'HangulSyllableBweng',
u'\u1c4c': 'UnicodeCharacter0x1c4c',
u'\u9650': 'CjkUnifiedIdeograph9650',
u'\u72d3': 'CjkUnifiedIdeograph72d3',
u'\uccd7': 'HangulSyllableCyeod',
u'\u2f56': 'KangxiRadicalClaw',
u'\u0bd9': 'UnicodeCharacter0xbd9',
u'\ua95a': 'UnicodeCharacter0xa95a',
u'\u85dd': 'CjkUnifiedIdeograph85dd',
u'\u7e60': 'CjkUnifiedIdeograph7e60',
u'\u5ae3': 'CjkUnifiedIdeograph5ae3',
u'\uf864': 'UnicodeCharacter0xf864',
u'\ud4e7': 'HangulSyllablePwih',
u'\u3766': 'CjkUnifiedIdeograph3766',
u'\u13e9': 'CherokeeLetterWa',
u'\ub16a': 'HangulSyllableNyelp',
u'\u6ded': 'CjkUnifiedIdeograph6ded',
u'\ue7f1': 'UnicodeCharacter0xe7f1',
u'\u4670': 'CjkUnifiedIdeograph4670',
u'\u22f3': 'ElementOfWithVerticalBarAtEndOfHorizontalStroke',
u'\uc074': 'HangulSyllableBbyi',
u'\ubcf7': 'HangulSyllableBogs',
u'\u1f76': 'GreekSmallLetterIotaWithVaria',
u'\u997a': 'CjkUnifiedIdeograph997a',
u'\u75fd': 'CjkUnifiedIdeograph75fd',
u'\u4f01': 'CjkUnifiedIdeograph4f01',
u'\u2e80': 'CjkRadicalRepeat',
u'\uc905': 'HangulSyllableJulg',
u'\ua884': 'SaurashtraLetterI',
u'\u0407': 'CyrillicCapitalLetterYi',
u'\u9e0b': 'CjkUnifiedIdeograph9e0b',
u'\u618a': 'CjkUnifiedIdeograph618a',
u'\ufb8e': 'ArabicLetterKehehIsolatedForm',
u'\u5711': 'CjkUnifiedIdeograph5711',
u'\u3690': 'CjkUnifiedIdeograph3690',
u'\ud115': 'HangulSyllableTyaeg',
u'\ub094': 'HangulSyllableGgik',
u'\u661b': 'CjkUnifiedIdeograph661b',
u'\u499a': 'CjkUnifiedIdeograph499a',
u'\ue01f': 'UnicodeCharacter0xe01f',
u'\uc39e': 'HangulSyllableSsep',
u'\u3f21': 'CjkUnifiedIdeograph3f21',
u'\u1ea0': 'LatinCapitalLetterAWithDotBelow',
u'\ub925': 'HangulSyllableRwenj',
u'\u98a4': 'CjkUnifiedIdeograph98a4',
u'\u4e2b': 'CjkUnifiedIdeograph4e2b',
u'\u51aa': 'CjkUnifiedIdeograph51aa',
u'\uc82f': 'HangulSyllableJes',
u'\uabae': 'UnicodeCharacter0xabae',
u'\u0731': 'SyriacPthahaBelow',
u'\u8135': 'CjkUnifiedIdeograph8135',
u'\u60b4': 'CjkUnifiedIdeograph60b4',
u'\ufab8': 'CjkCompatibilityIdeographfab8',
u'\u563b': 'CjkUnifiedIdeograph563b',
u'\u39ba': 'CjkUnifiedIdeograph39ba',
u'\ud03f': 'HangulSyllableKwilb',
u'\ub3be': 'HangulSyllableDyej',
u'\u6945': 'CjkUnifiedIdeograph6945',
u'\u48c4': 'CjkUnifiedIdeograph48c4',
u'\ue349': 'UnicodeCharacter0xe349',
u'\uc2c8': 'HangulSyllableSyil',
u'\u3e4b': 'CjkUnifiedIdeograph3e4b',
u'\u01ca': 'LatinCapitalLetterNj',
u'\ub84f': 'HangulSyllableRyelh',
u'\u9bce': 'CjkUnifiedIdeograph9bce',
u'\u7155': 'CjkUnifiedIdeograph7155',
u'\u50d4': 'CjkUnifiedIdeograph50d4',
u'\ucb59': 'HangulSyllableJjub',
u'\uaad8': 'UnicodeCharacter0xaad8',
u'\u065b': 'ArabicVowelSignInvertedSmallVAbove',
u'\u805f': 'CjkUnifiedIdeograph805f',
u'\u63de': 'CjkUnifiedIdeograph63de',
u'\ufde2': 'UnicodeCharacter0xfde2',
u'\u5965': 'CjkUnifiedIdeograph5965',
u'\u38e4': 'CjkUnifiedIdeograph38e4',
u'\ud369': 'HangulSyllablePyaelg',
u'\ub2e8': 'HangulSyllableDan',
u'\u686f': 'CjkUnifiedIdeograph686f',
u'\u4bee': 'CjkUnifiedIdeograph4bee',
u'\ue273': 'UnicodeCharacter0xe273',
u'\uc5f2': 'HangulSyllableYeonh',
u'\u2175': 'SmallRomanNumeralSix',
u'\xf4': 'LatinSmallLetterOWithCircumflex',
u'\ubb79': 'HangulSyllableMwelt',
u'\u9af8': 'CjkUnifiedIdeograph9af8',
u'\u707f': 'CjkUnifiedIdeograph707f',
u'\u53fe': 'CjkUnifiedIdeograph53fe',
u'\u2606': 'WhiteStar',
u'\u0289': 'LatinSmallLetterUBar',
u'\u6308': 'CjkUnifiedIdeograph6308',
u'\u5f8b': 'CjkUnifiedIdeograph5f8b',
u'\ua00a': 'YiSyllableA',
u'\u9c8d': 'CjkUnifiedIdeograph9c8d',
u'\ufd0c': 'ArabicLigatureSheenWithMeemIsolatedForm',
u'\ud98f': 'UnicodeCharacter0xd98f',
u'\u0e16': 'ThaiCharacterThoThung',
u'\u4b18': 'CjkUnifiedIdeograph4b18',
u'\u279b': 'DraftingPointRightwardsArrow',
u'\u881a': 'CjkUnifiedIdeograph881a',
u'\u649d': 'CjkUnifiedIdeograph649d',
u'\uc51c': 'HangulSyllableSsyim',
u'\ua19f': 'YiSyllableHlax',
u'\ufea1': 'ArabicLetterHahIsolatedForm',
u'\u1626': 'CanadianSyllabicsCarrierLi',
u'\u5328': 'CjkUnifiedIdeograph5328',
u'\u0fab': 'TibetanSubjoinedLetterDza',
u'\u902a': 'CjkUnifiedIdeograph902a',
u'\u4cad': 'CjkUnifiedIdeograph4cad',
u'\uad2c': 'HangulSyllableGwaess',
u'\u89af': 'CjkUnifiedIdeograph89af',
u'\uea2e': 'UnicodeCharacter0xea2e',
u'\uc6b1': 'HangulSyllableUg',
u'\u3b38': 'CjkUnifiedIdeograph3b38',
u'\u17bb': 'KhmerVowelSignU',
u'\u783a': 'CjkUnifiedIdeograph783a',
u'\u54bd': 'CjkUnifiedIdeograph54bd',
u'\ub53c': 'HangulSyllableDdals',
u'\u91bf': 'CjkUnifiedIdeograph91bf',
u'\uf23e': 'UnicodeCharacter0xf23e',
u'\uaec1': 'HangulSyllableGgeonj',
u'\uebc3': 'UnicodeCharacter0xebc3',
u'\u0348': 'CombiningDoubleVerticalLineBelow',
u'\u404a': 'CjkUnifiedIdeograph404a',
u'\u3ccd': 'CjkUnifiedIdeograph3ccd',
u'\u9d4c': 'CjkUnifiedIdeograph9d4c',
u'\u79cf': 'CjkUnifiedIdeograph79cf',
u'\uda4e': 'UnicodeCharacter0xda4e',
u'\ub6d1': 'HangulSyllableDdweot',
u'\uf3d3': 'UnicodeCharacter0xf3d3',
u'\u285a': 'BraillePatternDots2457',
u'\u04dd': 'CyrillicSmallLetterZheWithDiaeresis',
u'\u655c': 'CjkUnifiedIdeograph655c',
u'\u41df': 'CjkUnifiedIdeograph41df',
u'\ua25e': 'YiSyllableNgie',
u'\u9ee1': 'CjkUnifiedIdeograph9ee1',
u'\uff60': 'FullwidthRightWhiteParenthesis',
u'\udbe3': 'UnicodeCharacter0xdbe3',
u'\u306a': 'HiraganaLetterNa',
u'\u4d6c': 'CjkUnifiedIdeograph4d6c',
u'\u29ef': 'ErrorbarredBlackSquare',
u'\u8a6e': 'CjkUnifiedIdeograph8a6e',
u'\u66f1': 'CjkUnifiedIdeograph66f1',
u'\uc770': 'HangulSyllableYik',
u'\ua3f3': 'YiSyllableJup',
u'\ue0f5': 'UnicodeCharacter0xe0f5',
u'\u187a': 'UnicodeCharacter0x187a',
u'\u557c': 'CjkUnifiedIdeograph557c',
u'\u31ff': 'KatakanaLetterSmallRo',
u'\u927e': 'CjkUnifiedIdeograph927e',
u'\uaf80': 'HangulSyllableGgoe',
u'\u0b03': 'OriyaSignVisarga',
u'\uec82': 'UnicodeCharacter0xec82',
u'\u4805': 'CjkUnifiedIdeograph4805',
u'\u8507': 'CjkUnifiedIdeograph8507',
u'\uc209': 'HangulSyllableSyolt',
u'\u3d8c': 'CjkUnifiedIdeograph3d8c',
u'\u7a8e': 'CjkUnifiedIdeograph7a8e',
u'\ub790': 'HangulSyllableRass',
u'\u1313': 'EthiopicSyllableGwaa',
u'\uf492': 'UnicodeCharacter0xf492',
u'\u5015': 'CjkUnifiedIdeograph5015',
u'\u6d17': 'CjkUnifiedIdeograph6d17',
u'\uaa19': 'ChamLetterDda',
u'\ue71b': 'UnicodeCharacter0xe71b',
u'\u059c': 'HebrewAccentGeresh',
u'\u429e': 'CjkUnifiedIdeograph429e',
u'\u9fa0': 'CjkUnifiedIdeograph9fa0',
u'\udca2': 'UnicodeCharacter0xdca2',
u'\u3825': 'CjkUnifiedIdeograph3825',
u'\u7527': 'CjkUnifiedIdeograph7527',
u'\ub229': 'HangulSyllableNweolg',
u'\ucf2b': 'HangulSyllableKyeolh',
u'\u2aae': 'EqualsSignWithBumpyAbove',
u'\u67b0': 'CjkUnifiedIdeograph67b0',
u'\ua4b2': 'YiRadicalZa',
u'\ue1b4': 'UnicodeCharacter0xe1b4',
u'\u5d37': 'CjkUnifiedIdeograph5d37',
u'\u9a39': 'CjkUnifiedIdeograph9a39',
u'\ud73b': 'HangulSyllableHyud',
u'\u32be': 'CircledNumberFortyNine',
u'\u4fc0': 'CjkUnifiedIdeograph4fc0',
u'\u8cc2': 'CjkUnifiedIdeograph8cc2',
u'\uc9c4': 'HangulSyllableJin',
u'\u2547': 'BoxDrawingsDownLightAndUpHorizontalHeavy',
u'\u6249': 'CjkUnifiedIdeograph6249',
u'\ubf4b': 'HangulSyllableBbolb',
u'\ufc4d': 'ArabicLigatureNoonWithKhahIsolatedForm',
u'\u1ace': 'UnicodeCharacter0x1ace',
u'\u57d0': 'CjkUnifiedIdeograph57d0',
u'\u94d2': 'CjkUnifiedIdeograph94d2',
u'\ud1d4': 'HangulSyllableTwak',
u'\u0d57': 'MalayalamAuLengthMark',
u'\ueed6': 'UnicodeCharacter0xeed6',
u'\u4a59': 'CjkUnifiedIdeograph4a59',
u'\u875b': 'CjkUnifiedIdeograph875b',
u'\uc45d': 'HangulSyllableSsyong',
u'\u3fe0': 'CjkUnifiedIdeograph3fe0',
u'\u7ce2': 'CjkUnifiedIdeograph7ce2',
u'\ub9e4': 'HangulSyllableMae',
u'\u1567': 'CanadianSyllabicsThaa',
u'\uf6e6': 'UnicodeCharacter0xf6e6',
u'\u5269': 'CjkUnifiedIdeograph5269',
u'\u6f6b': 'CjkUnifiedIdeograph6f6b',
u'\uac6d': 'HangulSyllableGyaet',
u'\ue96f': 'UnicodeCharacter0xe96f',
u'\u07f0': 'NkoCombiningLongLowTone',
u'\u44f2': 'CjkUnifiedIdeograph44f2',
u'\u81f4': 'CjkUnifiedIdeograph81f4',
u'\udef6': 'UnicodeCharacter0xdef6',
u'\u3a79': 'CjkUnifiedIdeograph3a79',
u'\u777b': 'CjkUnifiedIdeograph777b',
u'\ub47d': 'HangulSyllableDweob',
u'\uf17f': 'UnicodeCharacter0xf17f',
u'\u9181': 'CjkUnifiedIdeograph9181',
u'\u7604': 'CjkUnifiedIdeograph7604',
u'\u5a87': 'CjkUnifiedIdeograph5a87',
u'\ufc08': 'ArabicLigatureBehWithMeemIsolatedForm',
u'\uc08b': 'HangulSyllableBbyic',
u'\u3f0a': 'CjkUnifiedIdeograph3f0a',
u'\u038d': 'UnicodeCharacter0x38d',
u'\ua50e': 'VaiSyllableTee',
u'\u8991': 'CjkUnifiedIdeograph8991',
u'\u6e14': 'CjkUnifiedIdeograph6e14',
u'\u3297': 'CircledIdeographCongratulation',
u'\ud418': 'HangulSyllablePwam',
u'\ub89b': 'HangulSyllableRwaed',
u'\u171a': 'UnicodeCharacter0x171a',
u'\u9d1e': 'CjkUnifiedIdeograph9d1e',
u'\u61a1': 'CjkUnifiedIdeograph61a1',
u'\ue7a5': 'UnicodeCharacter0xe7a5',
u'\u4624': 'CjkUnifiedIdeograph4624',
u'\u2aa7': 'GreaterthanClosedByCurve',
u'\ucc28': 'HangulSyllableCa',
u'\u90ab': 'CjkUnifiedIdeograph90ab',
u'\u0f2a': 'TibetanDigitHalfOne',
u'\u752e': 'CjkUnifiedIdeograph752e',
u'\u59b1': 'CjkUnifiedIdeograph59b1',
u'\ufb32': 'HebrewLetterGimelWithDagesh',
u'\udfb5': 'UnicodeCharacter0xdfb5',
u'\u3e34': 'CjkUnifiedIdeograph3e34',
u'\u02b7': 'ModifierLetterSmallW',
u'\ua438': 'YiSyllableNjiep',
u'\u88bb': 'CjkUnifiedIdeograph88bb',
u'\u6d3e': 'CjkUnifiedIdeograph6d3e',
u'\u31c1': 'CjkStrokeWg',
u'\ud342': 'HangulSyllablePaep',
u'\ub7c5': 'HangulSyllableRyab',
u'\u1644': 'CanadianSyllabicsCarrierZi',
u'\u9c48': 'CjkUnifiedIdeograph9c48',
u'\u60cb': 'CjkUnifiedIdeograph60cb',
u'\ue6cf': 'UnicodeCharacter0xe6cf',
u'\u454e': 'CjkUnifiedIdeograph454e',
u'\u29d1': 'BowtieWithLeftHalfBlack',
u'\ucb52': 'HangulSyllableJjulm',
u'\uafd5': 'HangulSyllableGgweog',
u'\u0e54': 'ThaiDigitFour',
u'\u7458': 'CjkUnifiedIdeograph7458',
u'\u58db': 'CjkUnifiedIdeograph58db',
u'\ufa5c': 'CjkCompatibilityIdeographfa5c',
u'\udedf': 'UnicodeCharacter0xdedf',
u'\u3d5e': 'CjkUnifiedIdeograph3d5e',
u'\u01e1': 'LatinSmallLetterAWithDotAboveAndMacron',
u'\ua362': 'YiSyllableChax',
u'\u87e5': 'CjkUnifiedIdeograph87e5',
u'\u6c68': 'CjkUnifiedIdeograph6c68',
u'\u30eb': 'KatakanaLetterRu',
u'\ud26c': 'HangulSyllableTwel',
u'\ub6ef': 'HangulSyllableDdweh',
u'\u156e': 'CanadianSyllabicsTtha',
u'\u9b72': 'CjkUnifiedIdeograph9b72',
u'\u7ff5': 'CjkUnifiedIdeograph7ff5',
u'\ue5f9': 'UnicodeCharacter0xe5f9',
u'\u4478': 'CjkUnifiedIdeograph4478',
u'\u28fb': 'BraillePatternDots1245678',
u'\uca7c': 'HangulSyllableJjess',
u'\uaeff': 'HangulSyllableGgyeolb',
u'\u0d7e': 'MalayalamLetterChilluLl',
u'\u9403': 'CjkUnifiedIdeograph9403',
u'\u7382': 'CjkUnifiedIdeograph7382',
u'\uf986': 'CjkCompatibilityIdeographf986',
u'\u5d09': 'CjkUnifiedIdeograph5d09',
u'\u3c88': 'CjkUnifiedIdeograph3c88',
u'\uc30d': 'HangulSyllableSsang',
u'\ua28c': 'YiSyllableWep',
u'\u060f': 'ArabicSignMisra',
u'\u8c13': 'CjkUnifiedIdeograph8c13',
u'\u6b92': 'CjkUnifiedIdeograph6b92',
u'\uf217': 'UnicodeCharacter0xf217',
u'\ud196': 'HangulSyllableTyebs',
u'\u3519': 'CjkUnifiedIdeograph3519',
u'\u1498': 'CanadianSyllabicsCwo',
u'\ubb1d': 'HangulSyllableMyonj',
u'\u9a9c': 'CjkUnifiedIdeograph9a9c',
u'\u6423': 'CjkUnifiedIdeograph6423',
u'\u43a2': 'CjkUnifiedIdeograph43a2',
u'\uea27': 'UnicodeCharacter0xea27',
u'\uc9a6': 'HangulSyllableJyigg',
u'\u2d29': 'UnicodeCharacter0x2d29',
u'\u0ca8': 'KannadaLetterNa',
u'\u932d': 'CjkUnifiedIdeograph932d',
u'\u72ac': 'CjkUnifiedIdeograph72ac',
u'\uf8b0': 'UnicodeCharacter0xf8b0',
u'\u5c33': 'CjkUnifiedIdeograph5c33',
u'\u3bb2': 'CjkUnifiedIdeograph3bb2',
u'\uc237': 'HangulSyllableSweogs',
u'\ua1b6': 'YiSyllableHlyr',
u'\u0539': 'ArmenianCapitalLetterTo',
u'\u8b3d': 'CjkUnifiedIdeograph8b3d',
u'\u6abc': 'CjkUnifiedIdeograph6abc',
u'\uf141': 'UnicodeCharacter0xf141',
u'\ud0c0': 'HangulSyllableTa',
u'\u3443': 'CjkUnifiedIdeograph3443',
u'\u13c2': 'CherokeeLetterNi',
u'\uba47': 'HangulSyllableMeolh',
u'\u99c6': 'CjkUnifiedIdeograph99c6',
u'\u634d': 'CjkUnifiedIdeograph634d',
u'\u42cc': 'CjkUnifiedIdeograph42cc',
u'\ue951': 'UnicodeCharacter0xe951',
u'\uc8d0': 'HangulSyllableJoels',
u'\u2c53': 'GlagoliticSmallLetterYu',
u'\u0bd2': 'UnicodeCharacter0xbd2',
u'\u9257': 'CjkUnifiedIdeograph9257',
u'\u71d6': 'CjkUnifiedIdeograph71d6',
u'\uf7da': 'UnicodeCharacter0xf7da',
u'\u5b5d': 'CjkUnifiedIdeograph5b5d',
u'\u3adc': 'CjkUnifiedIdeograph3adc',
u'\uc161': 'HangulSyllableSyeolt',
u'\ua0e0': 'YiSyllableFy',
u'\u0463': 'CyrillicSmallLetterYat',
u'\u8a67': 'CjkUnifiedIdeograph8a67',
u'\u69e6': 'CjkUnifiedIdeograph69e6',
u'\uf06b': 'UnicodeCharacter0xf06b',
u'\uefea': 'UnicodeCharacter0xefea',
u'\u336d': 'IdeographicTelegraphSymbolForHourTwentyone',
u'\u12ec': 'EthiopicSyllableYee',
u'\ub971': 'HangulSyllableRyut',
u'\u98f0': 'CjkUnifiedIdeograph98f0',
u'\u6277': 'CjkUnifiedIdeograph6277',
u'\u41f6': 'CjkUnifiedIdeograph41f6',
u'\ue87b': 'UnicodeCharacter0xe87b',
u'\uc7fa': 'HangulSyllableJyaej',
u'\u2b7d': 'UnicodeCharacter0x2b7d',
u'\u0afc': 'UnicodeCharacter0xafc',
u'\u1081': 'MyanmarLetterShanHa',
u'\u7100': 'CjkUnifiedIdeograph7100',
u'\u5583': 'CjkUnifiedIdeograph5583',
u'\ub202': 'HangulSyllableNyop',
u'\u9685': 'CjkUnifiedIdeograph9685',
u'\uf704': 'UnicodeCharacter0xf704',
u'\udb87': 'UnicodeCharacter0xdb87',
u'\u240e': 'SymbolForShiftOut',
u'\u0891': 'UnicodeCharacter0x891',
u'\u6910': 'CjkUnifiedIdeograph6910',
u'\u4d93': 'CjkUnifiedIdeograph4d93',
u'\uaa12': 'ChamLetterNhja',
u'\u8e95': 'CjkUnifiedIdeograph8e95',
u'\uef14': 'UnicodeCharacter0xef14',
u'\ub397': 'HangulSyllableDyeolb',
u'\uf499': 'UnicodeCharacter0xf499',
u'\u1c1e': 'LepchaLetterHla',
u'\u4120': 'CjkUnifiedIdeograph4120',
u'\u25a3': 'WhiteSquareContainingBlackSmallSquare',
u'\u8222': 'CjkUnifiedIdeograph8222',
u'\u66a5': 'CjkUnifiedIdeograph66a5',
u'\uc724': 'HangulSyllableYun',
u'\uaba7': 'UnicodeCharacter0xaba7',
u'\ueca9': 'UnicodeCharacter0xeca9',
u'\u3930': 'CjkUnifiedIdeograph3930',
u'\u1db3': 'ModifierLetterSmallSWithHook',
u'\u7a32': 'CjkUnifiedIdeograph7a32',
u'\u5eb5': 'CjkUnifiedIdeograph5eb5',
u'\ubf34': 'HangulSyllableBbyem',
u'\u83b7': 'CjkUnifiedIdeograph83b7',
u'\ue036': 'UnicodeCharacter0xe036',
u'\uc4b9': 'HangulSyllableSswig',
u'\u1140': 'HangulChoseongPansios',
u'\u5242': 'CjkUnifiedIdeograph5242',
u'\u36c5': 'CjkUnifiedIdeograph36c5',
u'\u9744': 'CjkUnifiedIdeograph9744',
u'\u7bc7': 'CjkUnifiedIdeograph7bc7',
u'\ud846': 'UnicodeCharacter0xd846',
u'\ubcc9': 'HangulSyllableByeolt',
u'\ue1cb': 'UnicodeCharacter0xe1cb',
u'\u0950': 'DevanagariOm',
u'\u4a52': 'CjkUnifiedIdeograph4a52',
u'\u2ed5': 'CjkRadicalMoundOne',
u'\u8f54': 'CjkUnifiedIdeograph8f54',
u'\u53d7': 'CjkUnifiedIdeograph53d7',
u'\ub056': 'HangulSyllableGgeubs',
u'\u94d9': 'CjkUnifiedIdeograph94d9',
u'\uf558': 'UnicodeCharacter0xf558',
u'\ud9db': 'UnicodeCharacter0xd9db',
u'\u2262': 'NotIdenticalTo',
u'\u06e5': 'ArabicSmallWaw',
u'\u6764': 'CjkUnifiedIdeograph6764',
u'\u4be7': 'CjkUnifiedIdeograph4be7',
u'\ua866': 'PhagspaLetterEe',
u'\u8ce9': 'CjkUnifiedIdeograph8ce9',
u'\ued68': 'UnicodeCharacter0xed68',
u'\ub1eb': 'HangulSyllableNyogs',
u'\uf2ed': 'UnicodeCharacter0xf2ed',
u'\u1a72': 'UnicodeCharacter0x1a72',
u'\u5f74': 'CjkUnifiedIdeograph5f74',
u'\u23f7': 'UnicodeCharacter0x23f7',
u'\u8076': 'CjkUnifiedIdeograph8076',
u'\u64f9': 'CjkUnifiedIdeograph64f9',
u'\uc578': 'HangulSyllableAek',
u'\ua9fb': 'UnicodeCharacter0xa9fb',
u'\ueafd': 'UnicodeCharacter0xeafd',
u'\ud001': 'HangulSyllableKweonj',
u'\u3784': 'CjkUnifiedIdeograph3784',
u'\u7886': 'CjkUnifiedIdeograph7886',
u'\ubd88': 'HangulSyllableBul',
u'\u010b': 'LatinSmallLetterCWithDotAbove',
u'\ufe8a': 'ArabicLetterYehWithHamzaAboveFinalForm',
u'\u420d': 'CjkUnifiedIdeograph420d',
u'\u870f': 'CjkUnifiedIdeograph870f',
u'\uc811': 'HangulSyllableJeob',
u'\u2f94': 'KangxiRadicalSpeech',
u'\u5096': 'CjkUnifiedIdeograph5096',
u'\u9598': 'CjkUnifiedIdeograph9598',
u'\ud69a': 'HangulSyllableHoelp',
u'\u3a1d': 'CjkUnifiedIdeograph3a1d',
u'\u7f1f': 'CjkUnifiedIdeograph7f1f',
u'\ua021': 'YiSyllableBap',
u'\ue523': 'UnicodeCharacter0xe523',
u'\u07a4': 'ThaanaLetterQaafu',
u'\u48a6': 'CjkUnifiedIdeograph48a6',
u'\u8da8': 'CjkUnifiedIdeograph8da8',
u'\uceaa': 'HangulSyllableKaep',
u'\u122d': 'EthiopicSyllableRe',
u'\uf3ac': 'UnicodeCharacter0xf3ac',
u'\u572f': 'CjkUnifiedIdeograph572f',
u'\u9831': 'CjkUnifiedIdeograph9831',
u'\udd33': 'UnicodeCharacter0xdd33',
u'\u20b6': 'UnicodeCharacter0x20b6',
u'\u65b8': 'CjkUnifiedIdeograph65b8',
u'\ua6ba': 'UnicodeCharacter0xa6ba',
u'\u0a3d': 'UnicodeCharacter0xa3d',
u'\uebbc': 'UnicodeCharacter0xebbc',
u'\u4f3f': 'CjkUnifiedIdeograph4f3f',
u'\u7041': 'CjkUnifiedIdeograph7041',
u'\ub543': 'HangulSyllableDdas',
u'\uf645': 'UnicodeCharacter0xf645',
u'\u18c6': 'UnicodeCharacter0x18c6',
u'\u5dc8': 'CjkUnifiedIdeograph5dc8',
u'\u9eca': 'CjkUnifiedIdeograph9eca',
u'\uc3cc': 'HangulSyllableSsyem',
u'\u274f': 'LowerRightDropshadowedWhiteSquare',
u'\u6851': 'CjkUnifiedIdeograph6851',
u'\uad53': 'HangulSyllableGyogs',
u'\uee55': 'UnicodeCharacter0xee55',
u'\u35d8': 'CjkUnifiedIdeograph35d8',
u'\u76da': 'CjkUnifiedIdeograph76da',
u'\ubbdc': 'HangulSyllableMyi',
u'\u1f5f': 'GreekCapitalLetterUpsilonWithDasiaAndPerispomeni',
u'\ufcde': 'ArabicLigatureYehWithHehInitialForm',
u'\u4061': 'CjkUnifiedIdeograph4061',
u'\u8563': 'CjkUnifiedIdeograph8563',
u'\uc665': 'HangulSyllableWaelg',
u'\u2de8': 'CombiningCyrillicLetterEm',
u'\u6eea': 'CjkUnifiedIdeograph6eea',
u'\u93ec': 'CjkUnifiedIdeograph93ec',
u'\ud4ee': 'HangulSyllablePyunh',
u'\u3871': 'CjkUnifiedIdeograph3871',
u'\u7d73': 'CjkUnifiedIdeograph7d73',
u'\ube75': 'HangulSyllableBbang',
u'\ue377': 'UnicodeCharacter0xe377',
u'\u05f8': 'UnicodeCharacter0x5f8',
u'\u46fa': 'CjkUnifiedIdeograph46fa',
u'\u8bfc': 'CjkUnifiedIdeograph8bfc',
u'\uccfe': 'HangulSyllableCyebs',
u'\uf200': 'UnicodeCharacter0xf200',
u'\ud683': 'HangulSyllableHwaes',
u'\u3102': 'UnicodeCharacter0x3102',
u'\u1585': 'CanadianSyllabicsQ',
u'\ubb06': 'HangulSyllableMoelm',
u'\u9f89': 'CjkUnifiedIdeograph9f89',
u'\u600c': 'CjkUnifiedIdeograph600c',
u'\u448f': 'CjkUnifiedIdeograph448f',
u'\uea10': 'UnicodeCharacter0xea10',
u'\uce93': 'HangulSyllableKaegs',
u'\u2912': 'UpwardsArrowToBar',
u'\u0d95': 'SinhalaLetterOoyanna',
u'\u9316': 'CjkUnifiedIdeograph9316',
u'\u7799': 'CjkUnifiedIdeograph7799',
u'\uf99d': 'CjkCompatibilityIdeographf99d',
u'\u581c': 'CjkUnifiedIdeograph581c',
u'\u3c9f': 'CjkUnifiedIdeograph3c9f',
u'\uc220': 'HangulSyllableSul',
u'\ua6a3': 'UnicodeCharacter0xa6a3',
u'\u0122': 'LatinCapitalLetterGWithCedilla',
u'\u8b26': 'CjkUnifiedIdeograph8b26',
u'\u6fa9': 'CjkUnifiedIdeograph6fa9',
u'\ud1ad': 'HangulSyllableTolt',
u'\u302c': 'IdeographicDepartingToneMark',
u'\u14af': 'CanadianSyllabicsWestcreeMwi',
u'\uba30': 'HangulSyllableMyaess',
u'\u9eb3': 'CjkUnifiedIdeograph9eb3',
u'\u6336': 'CjkUnifiedIdeograph6336',
u'\u47b9': 'CjkUnifiedIdeograph47b9',
u'\ue53a': 'UnicodeCharacter0xe53a',
u'\uc9bd': 'HangulSyllableJyit',
u'\u283c': 'BraillePatternDots3456',
u'\u0cbf': 'KannadaVowelSignI',
u'\u9240': 'CjkUnifiedIdeograph9240',
u'\u76c3': 'CjkUnifiedIdeograph76c3',
u'\uf8c7': 'UnicodeCharacter0xf8c7',
u'\u5b46': 'CjkUnifiedIdeograph5b46',
u'\u3fc9': 'CjkUnifiedIdeograph3fc9',
u'\udd4a': 'UnicodeCharacter0xdd4a',
u'\ua1cd': 'YiSyllableLep',
u'\u8a50': 'CjkUnifiedIdeograph8a50',
u'\u6ed3': 'CjkUnifiedIdeograph6ed3',
u'\ud0d7': 'HangulSyllableTac',
u'\u3356': 'SquareRentogen',
u'\u17d9': 'KhmerSignPhnaekMuan',
u'\ub55a': 'HangulSyllableDdaelp',
u'\u99dd': 'CjkUnifiedIdeograph99dd',
u'\u6260': 'CjkUnifiedIdeograph6260',
u'\u46e3': 'CjkUnifiedIdeograph46e3',
u'\ue464': 'UnicodeCharacter0xe464',
u'\uc8e7': 'HangulSyllableJyod',
u'\u2b66': 'UnicodeCharacter0x2b66',
u'\u0fe9': 'UnicodeCharacter0xfe9',
u'\uad6a': 'HangulSyllableGyop',
u'\u71ed': 'CjkUnifiedIdeograph71ed',
u'\ufbf1': 'ArabicLigatureYehWithHamzaAboveWithUFinalForm',
u'\u5a70': 'CjkUnifiedIdeograph5a70',
u'\u3ef3': 'CjkUnifiedIdeograph3ef3',
u'\udc74': 'UnicodeCharacter0xdc74',
u'\ua0f7': 'YiSyllableVup',
u'\u0376': 'GreekCapitalLetterPamphylianDigamma',
u'\u857a': 'CjkUnifiedIdeograph857a',
u'\u69fd': 'CjkUnifiedIdeograph69fd',
u'\u5301': 'CjkUnifiedIdeograph5301',
u'\u3280': 'CircledIdeographOne',
u'\ud505': 'HangulSyllablePeug',
u'\ub484': 'HangulSyllableDweok',
u'\u1807': 'MongolianSibeSyllableBoundaryMarker',
u'\u820b': 'CjkUnifiedIdeograph820b',
u'\u7d8a': 'CjkUnifiedIdeograph7d8a',
u'\ue78e': 'UnicodeCharacter0xe78e',
u'\u4b11': 'CjkUnifiedIdeograph4b11',
u'\u2a90': 'GreaterthanAboveSimilarAboveLessthan',
u'\ucd15': 'HangulSyllableColt',
u'\uac94': 'HangulSyllableGel',
u'\u7a1b': 'CjkUnifiedIdeograph7a1b',
u'\u559a': 'CjkUnifiedIdeograph559a',
u'\ufc1f': 'ArabicLigatureSeenWithMeemIsolatedForm',
u'\udf9e': 'UnicodeCharacter0xdf9e',
u'\u2321': 'BottomHalfIntegral',
u'\u02a0': 'LatinSmallLetterQWithHook',
u'\ua525': 'VaiSyllableIn',
u'\u84a4': 'CjkUnifiedIdeograph84a4',
u'\u522b': 'CjkUnifiedIdeograph522b',
u'\u4daa': 'CjkUnifiedIdeograph4daa',
u'\ud42f': 'HangulSyllablePwaelb',
u'\ub7ae': 'HangulSyllableRaej',
u'\u1b31': 'BalineseLetterSaSapa',
u'\u9d35': 'CjkUnifiedIdeograph9d35',
u'\u7cb4': 'CjkUnifiedIdeograph7cb4',
u'\ue6b8': 'UnicodeCharacter0xe6b8',
u'\u4a3b': 'CjkUnifiedIdeograph4a3b',
u'\u25ba': 'BlackRightpointingPointer',
u'\ucc3f': 'HangulSyllableCac',
u'\uafbe': 'HangulSyllableGgunh',
u'\u7545': 'CjkUnifiedIdeograph7545',
u'\u54c4': 'CjkUnifiedIdeograph54c4',
u'\uff49': 'FullwidthLatinSmallLetterI',
u'\udec8': 'UnicodeCharacter0xdec8',
u'\u224b': 'TripleTilde',
u'\u1dca': 'CombiningLatinSmallLetterRBelow',
u'\ua44f': 'YiSyllableNyiex',
u'\u87ce': 'CjkUnifiedIdeograph87ce',
u'\u6d55': 'CjkUnifiedIdeograph6d55',
u'\u4cd4': 'CjkUnifiedIdeograph4cd4',
u'\ud759': 'HangulSyllableHeulg',
u'\ub6d8': 'HangulSyllableDdwen',
u'\u1a5b': 'UnicodeCharacter0x1a5b',
u'\u9c5f': 'CjkUnifiedIdeograph9c5f',
u'\u7fde': 'CjkUnifiedIdeograph7fde',
u'\ue1e2': 'UnicodeCharacter0xe1e2',
u'\u4565': 'CjkUnifiedIdeograph4565',
u'\u24e4': 'CircledLatinSmallLetterU',
u'\ucf69': 'HangulSyllableKong',
u'\uaee8': 'HangulSyllableGgem',
u'\u746f': 'CjkUnifiedIdeograph746f',
u'\u57ee': 'CjkUnifiedIdeograph57ee',
u'\ufe73': 'ArabicTailFragment',
u'\ud9f2': 'UnicodeCharacter0xd9f2',
u'\u3d75': 'CjkUnifiedIdeograph3d75',
u'\u1cf4': 'UnicodeCharacter0x1cf4',
u'\ua779': 'LatinCapitalLetterInsularD',
u'\u86f8': 'CjkUnifiedIdeograph86f8',
u'\u6c7f': 'CjkUnifiedIdeograph6c7f',
u'\u4ffe': 'CjkUnifiedIdeograph4ffe',
u'\u3a06': 'CjkUnifiedIdeograph3a06',
u'\u1e89': 'LatinSmallLetterWWithDotBelow',
u'\u7f08': 'CjkUnifiedIdeograph7f08',
u'\u438b': 'CjkUnifiedIdeograph438b',
u'\ubc0a': 'HangulSyllableMibs',
u'\u808d': 'CjkUnifiedIdeograph808d',
u'\ue10c': 'UnicodeCharacter0xe10c',
u'\uc58f': 'HangulSyllableYas',
u'\u1216': 'EthiopicSyllableHho',
u'\u5718': 'CjkUnifiedIdeograph5718',
u'\u3b9b': 'CjkUnifiedIdeograph3b9b',
u'\u941a': 'CjkUnifiedIdeograph941a',
u'\u789d': 'CjkUnifiedIdeograph789d',
u'\ud91c': 'UnicodeCharacter0xd91c',
u'\ubd9f': 'HangulSyllableBweogs',
u'\ue2a1': 'UnicodeCharacter0xe2a1',
u'\u0a26': 'GurmukhiLetterDa',
u'\u4f28': 'CjkUnifiedIdeograph4f28',
u'\u13ab': 'CherokeeLetterGu',
u'\u8c2a': 'CjkUnifiedIdeograph8c2a',
u'\u50ad': 'CjkUnifiedIdeograph50ad',
u'\ub12c': 'HangulSyllableNel',
u'\u95af': 'CjkUnifiedIdeograph95af',
u'\uf62e': 'UnicodeCharacter0xf62e',
u'\udab1': 'UnicodeCharacter0xdab1',
u'\u2738': 'HeavyEightPointedRectilinearBlackStar',
u'\u0bbb': 'UnicodeCharacter0xbbb',
u'\u643a': 'CjkUnifiedIdeograph643a',
u'\u48bd': 'CjkUnifiedIdeograph48bd',
u'\ua93c': 'RejangLetterSa',
u'\u8dbf': 'CjkUnifiedIdeograph8dbf',
u'\uee3e': 'UnicodeCharacter0xee3e',
u'\ub2c1': 'HangulSyllableNying',
u'\uf7c3': 'UnicodeCharacter0xf7c3',
u'\u1f48': 'GreekCapitalLetterOmicronWithPsili',
u'\u5c4a': 'CjkUnifiedIdeograph5c4a',
u'\u20cd': 'UnicodeCharacter0x20cd',
u'\u814c': 'CjkUnifiedIdeograph814c',
u'\u65cf': 'CjkUnifiedIdeograph65cf',
u'\uc64e': 'HangulSyllableWalp',
u'\uaad1': 'UnicodeCharacter0xaad1',
u'\uefd3': 'UnicodeCharacter0xefd3',
u'\u345a': 'CjkUnifiedIdeograph345a',
u'\u18dd': 'UnicodeCharacter0x18dd',
u'\u795c': 'CjkUnifiedIdeograph795c',
u'\u5ddf': 'CjkUnifiedIdeograph5ddf',
u'\ube5e': 'HangulSyllableBip',
u'\u82e1': 'CjkUnifiedIdeograph82e1',
u'\ue360': 'UnicodeCharacter0xe360',
u'\uc7e3': 'HangulSyllableJyah',
u'\u2c6a': 'LatinSmallLetterKWithDescender',
u'\u516c': 'CjkUnifiedIdeograph516c',
u'\u35ef': 'CjkUnifiedIdeograph35ef',
u'\u966e': 'CjkUnifiedIdeograph966e',
u'\u7af1': 'CjkUnifiedIdeograph7af1',
u'\udb70': 'UnicodeCharacter0xdb70',
u'\ubff3': 'HangulSyllableBbweolb',
u'\ufcf5': 'ArabicLigatureTahWithAlefMaksuraIsolatedForm',
u'\u047a': 'CyrillicCapitalLetterRoundOmega',
u'\u497c': 'CjkUnifiedIdeograph497c',
u'\u2dff': 'CombiningCyrillicLetterIotifiedBigYus',
u'\u8e7e': 'CjkUnifiedIdeograph8e7e',
u'\ub380': 'HangulSyllableDem',
u'\u1703': 'TagalogLetterKa',
u'\uf082': 'UnicodeCharacter0xf082',
u'\u5405': 'CjkUnifiedIdeograph5405',
u'\u9907': 'CjkUnifiedIdeograph9907',
u'\ude09': 'UnicodeCharacter0xde09',
u'\u218c': 'UnicodeCharacter0x218c',
u'\u668e': 'CjkUnifiedIdeograph668e',
u'\uab90': 'UnicodeCharacter0xab90',
u'\u0f13': 'TibetanMarkCaretDzudRtagsMeLongCan',
u'\ue892': 'UnicodeCharacter0xe892',
u'\u4c15': 'CjkUnifiedIdeograph4c15',
u'\u7117': 'CjkUnifiedIdeograph7117',
u'\ub619': 'HangulSyllableDdolg',
u'\ufb1b': 'UnicodeCharacter0xfb1b',
u'\u199c': 'NewTaiLueLetterHighLa',
u'\u5e9e': 'CjkUnifiedIdeograph5e9e',
u'\u83a0': 'CjkUnifiedIdeograph83a0',
u'\uc0a2': 'HangulSyllableBbibs',
u'\u2425': 'SymbolForDeleteFormTwo',
u'\u6927': 'CjkUnifiedIdeograph6927',
u'\uae29': 'HangulSyllableGying',
u'\ud32b': 'HangulSyllablePaegs',
u'\u36ae': 'CjkUnifiedIdeograph36ae',
u'\u7bb0': 'CjkUnifiedIdeograph7bb0',
u'\ub8b2': 'HangulSyllableRoegg',
u'\u1c35': 'LepchaConsonantSignKang',
u'\ufdb4': 'ArabicLigatureQafWithMeemWithHahInitialForm',
u'\u4137': 'CjkUnifiedIdeograph4137',
u'\u8639': 'CjkUnifiedIdeograph8639',
u'\ucb3b': 'HangulSyllableJjyolh',
u'\u2ebe': 'CjkRadicalGrassOne',
u'\u53c0': 'CjkUnifiedIdeograph53c0',
u'\u90c2': 'CjkUnifiedIdeograph90c2',
u'\ud5c4': 'HangulSyllableHyaek',
u'\u3947': 'CjkUnifiedIdeograph3947',
u'\u7e49': 'CjkUnifiedIdeograph7e49',
u'\ua34b': 'YiSyllableZhuo',
u'\ue04d': 'UnicodeCharacter0xe04d',
u'\u06ce': 'ArabicLetterYehWithSmallV',
u'\u4bd0': 'CjkUnifiedIdeograph4bd0',
u'\u88d2': 'CjkUnifiedIdeograph88d2',
u'\ucdd4': 'HangulSyllableCwel',
u'\u1157': 'HangulChoseongKapyeounphieuph',
u'\uf2d6': 'UnicodeCharacter0xf2d6',
u'\u5659': 'CjkUnifiedIdeograph5659',
u'\u9b5b': 'CjkUnifiedIdeograph9b5b',
u'\ud85d': 'UnicodeCharacter0xd85d',
u'\u23e0': 'TopTortoiseShellBracket',
u'\u60e2': 'CjkUnifiedIdeograph60e2',
u'\ua5e4': 'VaiSyllableHe',
u'\u0967': 'DevanagariDigitOne',
u'\ueae6': 'UnicodeCharacter0xeae6',
u'\u4e69': 'CjkUnifiedIdeograph4e69',
u'\u736b': 'CjkUnifiedIdeograph736b',
u'\ub06d': 'HangulSyllableGgyilt',
u'\uf56f': 'UnicodeCharacter0xf56f',
u'\u1bf0': 'UnicodeCharacter0x1bf0',
u'\u58f2': 'CjkUnifiedIdeograph58f2',
u'\u9df4': 'CjkUnifiedIdeograph9df4',
u'\uc2f6': 'HangulSyllableSip',
u'\u2679': 'RecyclingSymbolForType7Plastics',
u'\u6b7b': 'CjkUnifiedIdeograph6b7b',
u'\ua87d': 'UnicodeCharacter0xa87d',
u'\ued7f': 'UnicodeCharacter0xed7f',
u'\u9581': 'CjkUnifiedIdeograph9581',
u'\u7204': 'CjkUnifiedIdeograph7204',
u'\u5e87': 'CjkUnifiedIdeograph5e87',
u'\uf808': 'UnicodeCharacter0xf808',
u'\uc48b': 'HangulSyllableSsweolb',
u'\u3b0a': 'CjkUnifiedIdeograph3b0a',
u'\u078d': 'ThaanaLetterLaamu',
u'\ua10e': 'YiSyllableDox',
u'\u8d91': 'CjkUnifiedIdeograph8d91',
u'\u6a14': 'CjkUnifiedIdeograph6a14',
u'\u3697': 'CjkUnifiedIdeograph3697',
u'\ud018': 'HangulSyllableKwe',
u'\ubc9b': 'HangulSyllableBeoc',
u'\u131a': 'EthiopicSyllableGgi',
u'\u991e': 'CjkUnifiedIdeograph991e',
u'\u65a1': 'CjkUnifiedIdeograph65a1',
u'\ue3a5': 'UnicodeCharacter0xe3a5',
u'\u4224': 'CjkUnifiedIdeograph4224',
u'\u2ea7': 'CjkRadicalCow',
u'\uc828': 'HangulSyllableJels',
u'\u94ab': 'CjkUnifiedIdeograph94ab',
u'\u0b2a': 'OriyaLetterPa',
u'\u712e': 'CjkUnifiedIdeograph712e',
u'\u5db1': 'CjkUnifiedIdeograph5db1',
u'\uff32': 'FullwidthLatinCapitalLetterR',
u'\udbb5': 'UnicodeCharacter0xdbb5',
u'\u3a34': 'CjkUnifiedIdeograph3a34',
u'\u06b7': 'ArabicLetterLamWithThreeDotsAbove',
u'\ua038': 'YiSyllablePit',
u'\u8cbb': 'CjkUnifiedIdeograph8cbb',
u'\u693e': 'CjkUnifiedIdeograph693e',
u'\u35c1': 'CjkUnifiedIdeograph35c1',
u'\ud742': 'HangulSyllableHyulp',
u'\ub3c5': 'HangulSyllableDog',
u'\u1244': 'EthiopicSyllableQee',
u'\u9848': 'CjkUnifiedIdeograph9848',
u'\u64cb': 'CjkUnifiedIdeograph64cb',
u'\ue2cf': 'UnicodeCharacter0xe2cf',
u'\u414e': 'CjkUnifiedIdeograph414e',
u'\u2dd1': 'EthiopicSyllableXyu',
u'\ucf52': 'HangulSyllableKyep',
u'\uabd5': 'UnicodeCharacter0xabd5',
u'\u0a54': 'UnicodeCharacter0xa54',
u'\u7058': 'CjkUnifiedIdeograph7058',
u'\u5cdb': 'CjkUnifiedIdeograph5cdb',
u'\ufe5c': 'SmallRightCurlyBracket',
u'\udadf': 'UnicodeCharacter0xdadf',
u'\u395e': 'CjkUnifiedIdeograph395e',
u'\u05e1': 'HebrewLetterSamekh',
u'\ua762': 'LatinCapitalLetterVisigothicZ',
u'\u83e5': 'CjkUnifiedIdeograph83e5',
u'\u6868': 'CjkUnifiedIdeograph6868',
u'\u34eb': 'CjkUnifiedIdeograph34eb',
u'\ud66c': 'HangulSyllableHwak',
u'\ub2ef': 'HangulSyllableDalb',
u'\u116e': 'HangulJungseongU',
u'\u9f72': 'CjkUnifiedIdeograph9f72',
u'\u7bf5': 'CjkUnifiedIdeograph7bf5',
u'\ue1f9': 'UnicodeCharacter0xe1f9',
u'\u4078': 'CjkUnifiedIdeograph4078',
u'\u2cfb': 'CopticOldNubianIndirectQuestionMark',
u'\uce7c': 'HangulSyllableKal',
u'\uaaff': 'UnicodeCharacter0xaaff',
u'\u097e': 'DevanagariLetterDdda',
u'\u9003': 'CjkUnifiedIdeograph9003',
u'\u7782': 'CjkUnifiedIdeograph7782',
u'\ufd86': 'ArabicLigatureLamWithKhahWithMeemInitialForm',
u'\u5909': 'CjkUnifiedIdeograph5909',
u'\u3888': 'CjkUnifiedIdeograph3888',
u'\uc70d': 'HangulSyllableWilg',
u'\ua68c': 'CyrillicCapitalLetterTwe',
u'\u020f': 'LatinSmallLetterOWithInvertedBreve',
u'\u8813': 'CjkUnifiedIdeograph8813',
u'\u6f92': 'CjkUnifiedIdeograph6f92',
u'\uf617': 'UnicodeCharacter0xf617',
u'\ud596': 'HangulSyllableHyanh',
u'\u3119': 'BopomofoLetterS',
u'\u1098': 'MyanmarShanDigitEight',
u'\ubf1d': 'HangulSyllableBbyeong',
u'\u9e9c': 'CjkUnifiedIdeograph9e9c',
u'\u6023': 'CjkUnifiedIdeograph6023',
u'\u47a2': 'CjkUnifiedIdeograph47a2',
u'\uee27': 'UnicodeCharacter0xee27',
u'\ucda6': 'HangulSyllableCubs',
u'\u2929': 'SouthEastArrowAndSouthWestArrow',
u'\u08a8': 'UnicodeCharacter0x8a8',
u'\u972d': 'CjkUnifiedIdeograph972d',
u'\u76ac': 'CjkUnifiedIdeograph76ac',
u'\ufcb0': 'ArabicLigatureSeenWithMeemInitialForm',
u'\u5833': 'CjkUnifiedIdeograph5833',
u'\u3fb2': 'CjkUnifiedIdeograph3fb2',
u'\uc637': 'HangulSyllableOs',
u'\ua5b6': 'VaiSyllableGu',
u'\u0139': 'LatinCapitalLetterLWithAcute',
u'\u8f3d': 'CjkUnifiedIdeograph8f3d',
u'\u6ebc': 'CjkUnifiedIdeograph6ebc',
u'\uf541': 'UnicodeCharacter0xf541',
u'\ud4c0': 'HangulSyllablePwem',
u'\u3043': 'HiraganaLetterSmallI',
u'\u17c2': 'KhmerVowelSignAe',
u'\ube47': 'HangulSyllableBigs',
u'\u9dc6': 'CjkUnifiedIdeograph9dc6',
u'\u674d': 'CjkUnifiedIdeograph674d',
u'\u46cc': 'CjkUnifiedIdeograph46cc',
u'\ued51': 'UnicodeCharacter0xed51',
u'\uccd0': 'HangulSyllableCyeo',
u'\u2853': 'BraillePatternDots1257',
u'\u0fd2': 'TibetanMarkNyisTsheg',
u'\u9657': 'CjkUnifiedIdeograph9657',
u'\u75d6': 'CjkUnifiedIdeograph75d6',
u'\uf3da': 'UnicodeCharacter0xf3da',
u'\u5f5d': 'CjkUnifiedIdeograph5f5d',
u'\u3edc': 'CjkUnifiedIdeograph3edc',
u'\uc561': 'HangulSyllableAeg',
u'\ua4e0': 'UnicodeCharacter0xa4e0',
u'\u8e67': 'CjkUnifiedIdeograph8e67',
u'\u6de6': 'CjkUnifiedIdeograph6de6',
u'\uf46b': 'UnicodeCharacter0xf46b',
u'\uebea': 'UnicodeCharacter0xebea',
u'\u376d': 'CjkUnifiedIdeograph376d',
u'\u16ec': 'RunicMultiplePunctuation',
u'\ubd71': 'HangulSyllableByolt',
u'\u9cf0': 'CjkUnifiedIdeograph9cf0',
u'\u6677': 'CjkUnifiedIdeograph6677',
u'\u45f6': 'CjkUnifiedIdeograph45f6',
u'\uec7b': 'UnicodeCharacter0xec7b',
u'\uc3fa': 'HangulSyllableSswanh',
u'\u2f7d': 'KangxiRadicalAnd',
u'\u0efc': 'UnicodeCharacter0xefc',
u'\u1481': 'CanadianSyllabicsWestcreeKwaa',
u'\u7500': 'CjkUnifiedIdeograph7500',
u'\u5183': 'CjkUnifiedIdeograph5183',
u'\ub602': 'HangulSyllableDdyelp',
u'\u9285': 'CjkUnifiedIdeograph9285',
u'\uf304': 'UnicodeCharacter0xf304',
u'\udf87': 'UnicodeCharacter0xdf87',
u'\u200e': 'LefttorightMark',
u'\u0c91': 'UnicodeCharacter0xc91',
u'\u6d10': 'CjkUnifiedIdeograph6d10',
u'\u4993': 'CjkUnifiedIdeograph4993',
u'\uae12': 'HangulSyllableGeup',
u'\u8a95': 'CjkUnifiedIdeograph8a95',
u'\ueb14': 'UnicodeCharacter0xeb14',
u'\ub797': 'HangulSyllableRah',
u'\uf099': 'UnicodeCharacter0xf099',
u'\u181e': 'UnicodeCharacter0x181e',
u'\u4520': 'CjkUnifiedIdeograph4520',
u'\u21a3': 'RightwardsArrowWithTail',
u'\u8622': 'CjkUnifiedIdeograph8622',
u'\u62a5': 'CjkUnifiedIdeograph62a5',
u'\uc324': 'HangulSyllableSsaem',
u'\uafa7': 'HangulSyllableGgyolb',
u'\ue8a9': 'UnicodeCharacter0xe8a9',
u'\u3d30': 'CjkUnifiedIdeograph3d30',
u'\u19b3': 'NewTaiLueVowelSignU',
u'\u7e32': 'CjkUnifiedIdeograph7e32',
u'\u5ab5': 'CjkUnifiedIdeograph5ab5',
u'\ubb34': 'HangulSyllableMu',
u'\u87b7': 'CjkUnifiedIdeograph87b7',
u'\ue436': 'UnicodeCharacter0xe436',
u'\uc0b9': 'HangulSyllableSalt',
u'\u1540': 'CanadianSyllabicsWestcreeY',
u'\u5642': 'CjkUnifiedIdeograph5642',
u'\u32c5': 'IdeographicTelegraphSymbolForJune',
u'\u9344': 'CjkUnifiedIdeograph9344',
u'\u7fc7': 'CjkUnifiedIdeograph7fc7',
u'\udc46': 'UnicodeCharacter0xdc46',
u'\ub8c9': 'HangulSyllableRoet',
u'\ue5cb': 'UnicodeCharacter0xe5cb',
u'\u0d50': 'UnicodeCharacter0xd50',
u'\u4e52': 'CjkUnifiedIdeograph4e52',
u'\u2ad5': 'SubsetAboveSubset',
u'\u8b54': 'CjkUnifiedIdeograph8b54',
u'\u57d7': 'CjkUnifiedIdeograph57d7',
u'\ub456': 'HangulSyllableDunh',
u'\u90d9': 'CjkUnifiedIdeograph90d9',
u'\uf158': 'UnicodeCharacter0xf158',
u'\udddb': 'UnicodeCharacter0xdddb',
u'\u2662': 'WhiteDiamondSuit',
u'\u02e5': 'ModifierLetterExtrahighToneBar',
u'\u6364': 'CjkUnifiedIdeograph6364',
u'\u4fe7': 'CjkUnifiedIdeograph4fe7',
u'\uac66': 'HangulSyllableGyaebs',
u'\u88e9': 'CjkUnifiedIdeograph88e9',
u'\ue968': 'UnicodeCharacter0xe968',
u'\ub5eb': 'HangulSyllableDdyeos',
u'\uf6ed': 'UnicodeCharacter0xf6ed',
u'\u1e72': 'LatinCapitalLetterUWithDiaeresisBelow',
u'\u5b74': 'CjkUnifiedIdeograph5b74',
u'\u27f7': 'LongLeftRightArrow',
u'\u8476': 'CjkUnifiedIdeograph8476',
u'\u60f9': 'CjkUnifiedIdeograph60f9',
u'\uc178': 'HangulSyllableSyel',
u'\uadfb': 'HangulSyllableGeugs',
u'\ueefd': 'UnicodeCharacter0xeefd',
u'\ud401': 'HangulSyllablePong',
u'\u3384': 'SquareKa',
u'\u7c86': 'CjkUnifiedIdeograph7c86',
u'\ub988': 'HangulSyllableReuss',
u'\u050b': 'CyrillicSmallLetterKomiNje',
u'\ufa8a': 'CjkCompatibilityIdeographfa8a',
u'\u460d': 'CjkUnifiedIdeograph460d',
u'\u830f': 'CjkUnifiedIdeograph830f',
u'\ucc11': 'HangulSyllableJjinj',
u'\u2b94': 'UnicodeCharacter0x2b94',
u'\u5496': 'CjkUnifiedIdeograph5496',
u'\u9198': 'CjkUnifiedIdeograph9198',
u'\ud29a': 'HangulSyllableTwip',
u'\u3e1d': 'CjkUnifiedIdeograph3e1d',
u'\u7b1f': 'CjkUnifiedIdeograph7b1f',
u'\ua421': 'YiSyllableJjuo',
u'\ue123': 'UnicodeCharacter0xe123',
u'\u03a4': 'GreekCapitalLetterTau',
u'\u4ca6': 'CjkUnifiedIdeograph4ca6',
u'\u89a8': 'CjkUnifiedIdeograph89a8',
u'\ucaaa': 'HangulSyllableJjyelm',
u'\u162d': 'CanadianSyllabicsCarrierDla',
u'\uf7ac': 'UnicodeCharacter0xf7ac',
u'\u532f': 'CjkUnifiedIdeograph532f',
u'\u9c31': 'CjkUnifiedIdeograph9c31',
u'\ud933': 'UnicodeCharacter0xd933',
u'\u24b6': 'CircledLatinCapitalLetterA',
u'\u61b8': 'CjkUnifiedIdeograph61b8',
u'\ua2ba': 'YiSyllableCuox',
u'\u0e3d': 'UnicodeCharacter0xe3d',
u'\uefbc': 'UnicodeCharacter0xefbc',
u'\u4b3f': 'CjkUnifiedIdeograph4b3f',
u'\u7441': 'CjkUnifiedIdeograph7441',
u'\ub143': 'HangulSyllableNyeogs',
u'\uf245': 'UnicodeCharacter0xf245',
u'\u1cc6': 'UnicodeCharacter0x1cc6',
u'\u59c8': 'CjkUnifiedIdeograph59c8',
u'\u9aca': 'CjkUnifiedIdeograph9aca',
u'\uc7cc': 'HangulSyllableJyan',
u'\u234f': 'AplFunctionalSymbolUpwardsVane',
u'\u6c51': 'CjkUnifiedIdeograph6c51',
u'\ua953': 'RejangVirama',
u'\uea55': 'UnicodeCharacter0xea55',
u'\u31d8': 'CjkStrokeSwz',
u'\u72da': 'CjkUnifiedIdeograph72da',
u'\ubfdc': 'HangulSyllableBbum',
u'\u1b5f': 'BalineseCarikPareren',
u'\uf8de': 'UnicodeCharacter0xf8de',
u'\u4461': 'CjkUnifiedIdeograph4461',
u'\u8163': 'CjkUnifiedIdeograph8163',
u'\uc265': 'HangulSyllableSweng',
u'\u29e8': 'DownpointingTriangleWithLeftHalfBlack',
u'\u6aea': 'CjkUnifiedIdeograph6aea',
u'\u97ec': 'CjkUnifiedIdeograph97ec',
u'\ud0ee': 'HangulSyllableTaebs',
u'\u3c71': 'CjkUnifiedIdeograph3c71',
u'\u7973': 'CjkUnifiedIdeograph7973',
u'\uba75': 'HangulSyllableMyeonj',
u'\ue777': 'UnicodeCharacter0xe777',
u'\u01f8': 'LatinCapitalLetterNWithGrave',
u'\u42fa': 'CjkUnifiedIdeograph42fa',
u'\u8ffc': 'CjkUnifiedIdeograph8ffc',
u'\uc8fe': 'HangulSyllableJugg',
u'\uf600': 'UnicodeCharacter0xf600',
u'\ud283': 'HangulSyllableTwigs',
u'\u3502': 'CjkUnifiedIdeograph3502',
u'\u1185': 'HangulJungseongYoyae',
u'\ubf06': 'HangulSyllableBbep',
u'\u9b89': 'CjkUnifiedIdeograph9b89',
u'\u640c': 'CjkUnifiedIdeograph640c',
u'\u408f': 'CjkUnifiedIdeograph408f',
u'\uee10': 'UnicodeCharacter0xee10',
u'\uca93': 'HangulSyllableJjyeolh',
u'\u2d12': 'GeorgianSmallLetterTar',
u'\u0995': 'BengaliLetterKa',
u'\u9716': 'CjkUnifiedIdeograph9716',
u'\u7399': 'CjkUnifiedIdeograph7399',
u'\ufd9d': 'ArabicLigatureYehWithMeemWithMeemInitialForm',
u'\u5c1c': 'CjkUnifiedIdeograph5c1c',
u'\u389f': 'CjkUnifiedIdeograph389f',
u'\uc620': 'HangulSyllableYek',
u'\ua2a3': 'YiSyllableZux',
u'\u0522': 'CyrillicCapitalLetterEnWithMiddleHook',
u'\u8f26': 'CjkUnifiedIdeograph8f26',
u'\u6ba9': 'CjkUnifiedIdeograph6ba9',
u'\ud5ad': 'HangulSyllableHyaeg',
u'\u342c': 'CjkUnifiedIdeograph342c',
u'\u10af': 'GeorgianCapitalLetterZhar',
u'\ube30': 'HangulSyllableByil',
u'\u9ab3': 'CjkUnifiedIdeograph9ab3',
u'\u6736': 'CjkUnifiedIdeograph6736',
u'\u43b9': 'CjkUnifiedIdeograph43b9',
u'\ue13a': 'UnicodeCharacter0xe13a',
u'\ucdbd': 'HangulSyllableCweolt',
u'\u2c3c': 'GlagoliticSmallLetterDjervi',
u'\u08bf': 'UnicodeCharacter0x8bf',
u'\u9640': 'CjkUnifiedIdeograph9640',
u'\u72c3': 'CjkUnifiedIdeograph72c3',
u'\ufcc7': 'ArabicLigatureKafWithLamInitialForm',
u'\u5f46': 'CjkUnifiedIdeograph5f46',
u'\u3bc9': 'CjkUnifiedIdeograph3bc9',
u'\ud94a': 'UnicodeCharacter0xd94a',
u'\ua5cd': 'VaiSyllableDho',
u'\u044c': 'CyrillicSmallLetterSoftSign',
u'\u8e50': 'CjkUnifiedIdeograph8e50',
u'\u6ad3': 'CjkUnifiedIdeograph6ad3',
u'\ud4d7': 'HangulSyllablePwilb',
u'\u3756': 'CjkUnifiedIdeograph3756',
u'\u13d9': 'CherokeeLetterDo',
u'\ub15a': 'HangulSyllableNyeop',
u'\u9ddd': 'CjkUnifiedIdeograph9ddd',
u'\u6660': 'CjkUnifiedIdeograph6660',
u'\u42e3': 'CjkUnifiedIdeograph42e3',
u'\ue064': 'UnicodeCharacter0xe064',
u'\ucce7': 'HangulSyllableCyeoc',
u'\u2f66': 'KangxiRadicalBoltOfCloth',
u'\u0be9': 'TamilDigitThree',
u'\ua96a': 'UnicodeCharacter0xa96a',
u'\u75ed': 'CjkUnifiedIdeograph75ed',
u'\ufff1': 'UnicodeCharacter0xfff1',
u'\u5e70': 'CjkUnifiedIdeograph5e70',
u'\u3af3': 'CjkUnifiedIdeograph3af3',
u'\ud874': 'UnicodeCharacter0xd874',
u'\ua4f7': 'UnicodeCharacter0xa4f7',
u'\u0776': 'ArabicLetterFarsiYehWithExtendedArabicindicDigitThreeAbove',
u'\u817a': 'CjkUnifiedIdeograph817a',
u'\u6dfd': 'CjkUnifiedIdeograph6dfd',
u'\u5701': 'CjkUnifiedIdeograph5701',
u'\u3680': 'CjkUnifiedIdeograph3680',
u'\ud105': 'HangulSyllableTyalt',
u'\ub084': 'HangulSyllableGgil',
u'\u1c07': 'LepchaLetterCha',
u'\u860b': 'CjkUnifiedIdeograph860b',
u'\u798a': 'CjkUnifiedIdeograph798a',
u'\ue38e': 'UnicodeCharacter0xe38e',
u'\u4f11': 'CjkUnifiedIdeograph4f11',
u'\u2e90': 'CjkRadicalLameThree',
u'\uc915': 'HangulSyllableJut',
u'\ua894': 'SaurashtraLetterGa',
u'\u7e1b': 'CjkUnifiedIdeograph7e1b',
u'\u519a': 'CjkUnifiedIdeograph519a',
u'\uf81f': 'UnicodeCharacter0xf81f',
u'\udb9e': 'UnicodeCharacter0xdb9e',
u'\u2721': 'StarOfDavid',
u'\u06a0': 'ArabicLetterAinWithThreeDotsAbove',
u'\ua125': 'YiSyllableTuot',
u'\u80a4': 'CjkUnifiedIdeograph80a4',
u'\u562b': 'CjkUnifiedIdeograph562b',
u'\u49aa': 'CjkUnifiedIdeograph49aa',
u'\ud02f': 'HangulSyllableKwec',
u'\ub3ae': 'HangulSyllableDyenh',
u'\u1f31': 'GreekSmallLetterIotaWithDasia',
u'\u9935': 'CjkUnifiedIdeograph9935',
u'\u78b4': 'CjkUnifiedIdeograph78b4',
u'\ue2b8': 'UnicodeCharacter0xe2b8',
u'\u4e3b': 'CjkUnifiedIdeograph4e3b',
u'\u21ba': 'AnticlockwiseOpenCircleArrow',
u'\uc83f': 'HangulSyllableJyeod',
u'\uabbe': 'UnicodeCharacter0xabbe',
u'\u7145': 'CjkUnifiedIdeograph7145',
u'\u50c4': 'CjkUnifiedIdeograph50c4',
u'\ufb49': 'HebrewLetterShinWithDagesh',
u'\udac8': 'UnicodeCharacter0xdac8',
u'\u264b': 'Cancer',
u'\u19ca': 'UnicodeCharacter0x19ca',
u'\ua04f': 'YiSyllablePur',
u'\u83ce': 'CjkUnifiedIdeograph83ce',
u'\u6955': 'CjkUnifiedIdeograph6955',
u'\u48d4': 'CjkUnifiedIdeograph48d4',
u'\ud359': 'HangulSyllablePyang',
u'\ub2d8': 'HangulSyllableNim',
u'\u1e5b': 'LatinSmallLetterRWithDotBelow',
u'\u985f': 'CjkUnifiedIdeograph985f',
u'\u7bde': 'CjkUnifiedIdeograph7bde',
u'\ue5e2': 'UnicodeCharacter0xe5e2',
u'\u4165': 'CjkUnifiedIdeograph4165',
u'\u20e4': 'CombiningEnclosingUpwardPointingTriangle',
u'\ucb69': 'HangulSyllableJjweonj',
u'\uaae8': 'UnicodeCharacter0xaae8',
u'\u706f': 'CjkUnifiedIdeograph706f',
u'\u53ee': 'CjkUnifiedIdeograph53ee',
u'\ufa73': 'CjkCompatibilityIdeographfa73',
u'\uddf2': 'UnicodeCharacter0xddf2',
u'\u3975': 'CjkUnifiedIdeograph3975',
u'\u18f4': 'UnicodeCharacter0x18f4',
u'\ua379': 'YiSyllableChyp',
u'\u82f8': 'CjkUnifiedIdeograph82f8',
u'\u687f': 'CjkUnifiedIdeograph687f',
u'\u4bfe': 'CjkUnifiedIdeograph4bfe',
u'\u3e06': 'CjkUnifiedIdeograph3e06',
u'\u1a89': 'UnicodeCharacter0x1a89',
u'\u7b08': 'CjkUnifiedIdeograph7b08',
u'\u478b': 'CjkUnifiedIdeograph478b',
u'\ub80a': 'HangulSyllableRegg',
u'\u848d': 'CjkUnifiedIdeograph848d',
u'\ue50c': 'UnicodeCharacter0xe50c',
u'\uc18f': 'HangulSyllableSogs',
u'\u1616': 'CanadianSyllabicsCarrierJo',
u'\u5318': 'CjkUnifiedIdeograph5318',
u'\u3f9b': 'CjkUnifiedIdeograph3f9b',
u'\u901a': 'CjkUnifiedIdeograph901a',
u'\u7c9d': 'CjkUnifiedIdeograph7c9d',
u'\udd1c': 'UnicodeCharacter0xdd1c',
u'\ub99f': 'HangulSyllableRyilh',
u'\ue6a1': 'UnicodeCharacter0xe6a1',
u'\u0e26': 'ThaiCharacterLu',
u'\u4b28': 'CjkUnifiedIdeograph4b28',
u'\u17ab': 'KhmerIndependentVowelRy',
u'\u882a': 'CjkUnifiedIdeograph882a',
u'\u54ad': 'CjkUnifiedIdeograph54ad',
u'\ub52c': 'HangulSyllableDik',
u'\u91af': 'CjkUnifiedIdeograph91af',
u'\uf22e': 'UnicodeCharacter0xf22e',
u'\udeb1': 'UnicodeCharacter0xdeb1',
u'\u2338': 'AplFunctionalSymbolQuadEqual',
u'\u0fbb': 'TibetanSubjoinedLetterFixedformYa',
u'\u603a': 'CjkUnifiedIdeograph603a',
u'\u4cbd': 'CjkUnifiedIdeograph4cbd',
u'\uad3c': 'HangulSyllableGoel',
u'\u89bf': 'CjkUnifiedIdeograph89bf',
u'\uea3e': 'UnicodeCharacter0xea3e',
u'\ub6c1': 'HangulSyllableDdweolg',
u'\uf3c3': 'UnicodeCharacter0xf3c3',
u'\u1b48': 'BalineseLetterEfSasak',
u'\u584a': 'CjkUnifiedIdeograph584a',
u'\u24cd': 'CircledLatinCapitalLetterX',
u'\u854c': 'CjkUnifiedIdeograph854c',
u'\u61cf': 'CjkUnifiedIdeograph61cf',
u'\uc24e': 'HangulSyllableSweop',
u'\uaed1': 'HangulSyllableGgeong',
u'\uebd3': 'UnicodeCharacter0xebd3',
u'\u305a': 'HiraganaLetterZu',
u'\u1cdd': 'UnicodeCharacter0x1cdd',
u'\u7d5c': 'CjkUnifiedIdeograph7d5c',
u'\u59df': 'CjkUnifiedIdeograph59df',
u'\uba5e': 'HangulSyllableMelm',
u'\u86e1': 'CjkUnifiedIdeograph86e1',
u'\ue760': 'UnicodeCharacter0xe760',
u'\uc3e3': 'HangulSyllableSsolb',
u'\u286a': 'BraillePatternDots2467',
u'\u556c': 'CjkUnifiedIdeograph556c',
u'\u31ef': 'UnicodeCharacter0x31ef',
u'\u926e': 'CjkUnifiedIdeograph926e',
u'\u7ef1': 'CjkUnifiedIdeograph7ef1',
u'\udf70': 'UnicodeCharacter0xdf70',
u'\ubbf3': 'HangulSyllableMyic',
u'\uf8f5': 'UnicodeCharacter0xf8f5',
u'\u4d7c': 'CjkUnifiedIdeograph4d7c',
u'\u29ff': 'Miny',
u'\u8a7e': 'CjkUnifiedIdeograph8a7e',
u'\ub780': 'HangulSyllableRan',
u'\u1303': 'EthiopicSyllableJaa',
u'\uf482': 'UnicodeCharacter0xf482',
u'\u5005': 'CjkUnifiedIdeograph5005',
u'\u9d07': 'CjkUnifiedIdeograph9d07',
u'\uda09': 'UnicodeCharacter0xda09',
u'\u258c': 'LeftHalfBlock',
u'\u628e': 'CjkUnifiedIdeograph628e',
u'\uaf90': 'HangulSyllableGgoem',
u'\u0b13': 'OriyaLetterO',
u'\uec92': 'UnicodeCharacter0xec92',
u'\u4815': 'CjkUnifiedIdeograph4815',
u'\u7517': 'CjkUnifiedIdeograph7517',
u'\ub219': 'HangulSyllableNung',
u'\uff1b': 'FullwidthSemicolon',
u'\u1d9c': 'ModifierLetterSmallC',
u'\u5a9e': 'CjkUnifiedIdeograph5a9e',
u'\u87a0': 'CjkUnifiedIdeograph87a0',
u'\uc4a2': 'HangulSyllableSswenh',
u'\u2025': 'TwoDotLeader',
u'\u6d27': 'CjkUnifiedIdeograph6d27',
u'\uaa29': 'ChamVowelSignAa',
u'\ud72b': 'HangulSyllableHwis',
u'\u32ae': 'CircledIdeographResource',
u'\u7fb0': 'CjkUnifiedIdeograph7fb0',
u'\ubcb2': 'HangulSyllableBebs',
u'\u1835': 'MongolianLetterJa',
u'\uf9b4': 'CjkCompatibilityIdeographf9b4',
u'\u4537': 'CjkUnifiedIdeograph4537',
u'\u8239': 'CjkUnifiedIdeograph8239',
u'\ucf3b': 'HangulSyllableKyegs',
u'\u2abe': 'SupersetWithDot',
u'\u57c0': 'CjkUnifiedIdeograph57c0',
u'\u94c2': 'CjkUnifiedIdeograph94c2',
u'\ud1c4': 'HangulSyllableTwal',
u'\u3d47': 'CjkUnifiedIdeograph3d47',
u'\u7a49': 'CjkUnifiedIdeograph7a49',
u'\ua74b': 'LatinSmallLetterOWithLongStrokeOverlay',
u'\ue44d': 'UnicodeCharacter0xe44d',
u'\u02ce': 'ModifierLetterLowGraveAccent',
u'\u4fd0': 'CjkUnifiedIdeograph4fd0',
u'\u8cd2': 'CjkUnifiedIdeograph8cd2',
u'\uc9d4': 'HangulSyllableJiss',
u'\u1557': 'CanadianSyllabicsFo',
u'\uf6d6': 'UnicodeCharacter0xf6d6',
u'\u5259': 'CjkUnifiedIdeograph5259',
u'\u9f5b': 'CjkUnifiedIdeograph9f5b',
u'\udc5d': 'UnicodeCharacter0xdc5d',
u'\u27e0': 'LozengeDividedByHorizontalRule',
u'\u64e2': 'CjkUnifiedIdeograph64e2',
u'\ua1e4': 'YiSyllableGa',
u'\u0d67': 'MalayalamDigitOne',
u'\ueee6': 'UnicodeCharacter0xeee6',
u'\u4a69': 'CjkUnifiedIdeograph4a69',
u'\u776b': 'CjkUnifiedIdeograph776b',
u'\ub46d': 'HangulSyllableDweog',
u'\uf16f': 'UnicodeCharacter0xf16f',
u'\u1ff0': 'UnicodeCharacter0x1ff0',
u'\u5cf2': 'CjkUnifiedIdeograph5cf2',
u'\u99f4': 'CjkUnifiedIdeograph99f4',
u'\uc6f6': 'HangulSyllableWelp',
u'\u2279': 'NeitherGreaterthanNorLessthan',
u'\u6f7b': 'CjkUnifiedIdeograph6f7b',
u'\uac7d': 'HangulSyllableGeolt',
u'\ue97f': 'UnicodeCharacter0xe97f',
u'\u9981': 'CjkUnifiedIdeograph9981',
u'\u7e04': 'CjkUnifiedIdeograph7e04',
u'\u5287': 'CjkUnifiedIdeograph5287',
u'\uf408': 'UnicodeCharacter0xf408',
u'\uc88b': 'HangulSyllableJoh',
u'\u370a': 'CjkUnifiedIdeograph370a',
u'\u0b8d': 'UnicodeCharacter0xb8d',
u'\uad0e': 'HangulSyllableGwabs',
u'\u8191': 'CjkUnifiedIdeograph8191',
u'\u6614': 'CjkUnifiedIdeograph6614',
u'\u3a97': 'CjkUnifiedIdeograph3a97',
u'\udc18': 'UnicodeCharacter0xdc18',
u'\ub09b': 'HangulSyllableNags',
u'\u1f1a': 'GreekCapitalLetterEpsilonWithPsiliAndVaria',
u'\u951e': 'CjkUnifiedIdeograph951e',
u'\u69a1': 'CjkUnifiedIdeograph69a1',
u'\uefa5': 'UnicodeCharacter0xefa5',
u'\u4e24': 'CjkUnifiedIdeograph4e24',
u'\u22a7': 'Models',
u'\uc428': 'HangulSyllableSswaek',
u'\u98ab': 'CjkUnifiedIdeograph98ab',
u'\u072a': 'SyriacLetterRish',
u'\u7d2e': 'CjkUnifiedIdeograph7d2e',
u'\u51b1': 'CjkUnifiedIdeograph51b1',
u'\uf332': 'UnicodeCharacter0xf332',
u'\ud7b5': 'UnicodeCharacter0xd7b5',
u'\u3634': 'CjkUnifiedIdeograph3634',
u'\u0ab7': 'GujaratiLetterSsa',
u'\uac38': 'HangulSyllableGya',
u'\u80bb': 'CjkUnifiedIdeograph80bb',
u'\u653e': 'CjkUnifiedIdeograph653e',
u'\u39c1': 'CjkUnifiedIdeograph39c1',
u'\udb42': 'UnicodeCharacter0xdb42',
u'\ubfc5': 'HangulSyllableBbyong',
u'\u1e44': 'LatinCapitalLetterNWithDotAbove',
u'\u9448': 'CjkUnifiedIdeograph9448',
u'\u68cb': 'CjkUnifiedIdeograph68cb',
u'\ueecf': 'UnicodeCharacter0xeecf',
u'\u4d4e': 'CjkUnifiedIdeograph4d4e',
u'\u21d1': 'UpwardsDoubleArrow',
u'\uc352': 'HangulSyllableSsyaenh',
u'\ua7d5': 'UnicodeCharacter0xa7d5',
u'\u0654': 'ArabicHamzaAbove',
u'\u7c58': 'CjkUnifiedIdeograph7c58',
u'\u50db': 'CjkUnifiedIdeograph50db',
u'\uf25c': 'UnicodeCharacter0xf25c',
u'\ud6df': 'HangulSyllableHuh',
u'\u355e': 'CjkUnifiedIdeograph355e',
u'\u09e1': 'BengaliLetterVocalicLl',
u'\uab62': 'UnicodeCharacter0xab62',
u'\u8fe5': 'CjkUnifiedIdeograph8fe5',
u'\u6468': 'CjkUnifiedIdeograph6468',
u'\u38eb': 'CjkUnifiedIdeograph38eb',
u'\uda6c': 'UnicodeCharacter0xda6c',
u'\ubeef': 'HangulSyllableBbegs',
u'\u1d6e': 'LatinSmallLetterFWithMiddleTilde',
u'\u9372': 'CjkUnifiedIdeograph9372',
u'\u77f5': 'CjkUnifiedIdeograph77f5',
u'\uedf9': 'UnicodeCharacter0xedf9',
u'\u4c78': 'CjkUnifiedIdeograph4c78',
u'\u20fb': 'UnicodeCharacter0x20fb',
u'\uc27c': 'HangulSyllableSwim',
u'\ua6ff': 'UnicodeCharacter0xa6ff',
u'\u057e': 'ArmenianSmallLetterVew',
u'\u9c03': 'CjkUnifiedIdeograph9c03',
u'\u7b82': 'CjkUnifiedIdeograph7b82',
u'\uf186': 'UnicodeCharacter0xf186',
u'\u5509': 'CjkUnifiedIdeograph5509',
u'\u3488': 'CjkUnifiedIdeograph3488',
u'\ucb0d': 'HangulSyllableJjwaet',
u'\uaa8c': 'UnicodeCharacter0xaa8c',
u'\u0e0f': 'ThaiCharacterToPatak',
u'\u8413': 'CjkUnifiedIdeograph8413',
u'\u6392': 'CjkUnifiedIdeograph6392',
u'\ufa17': 'CjkCompatibilityIdeographfa17',
u'\ud996': 'UnicodeCharacter0xd996',
u'\u3d19': 'CjkUnifiedIdeograph3d19',
u'\u1c98': 'UnicodeCharacter0x1c98',
u'\ub31d': 'HangulSyllableDyag',
u'\u929c': 'CjkUnifiedIdeograph929c',
u'\u6c23': 'CjkUnifiedIdeograph6c23',
u'\u4ba2': 'CjkUnifiedIdeograph4ba2',
u'\ue227': 'UnicodeCharacter0xe227',
u'\uc1a6': 'HangulSyllableSop',
u'\u2529': 'BoxDrawingsDownLightAndLeftUpHeavy',
u'\u04a8': 'CyrillicCapitalLetterAbkhasianHa',
u'\u9b2d': 'CjkUnifiedIdeograph9b2d',
u'\u7aac': 'CjkUnifiedIdeograph7aac',
u'\uf0b0': 'UnicodeCharacter0xf0b0',
u'\u5433': 'CjkUnifiedIdeograph5433',
u'\u33b2': 'SquareMuS',
u'\uca37': 'HangulSyllableJjyaed',
u'\ua9b6': 'UnicodeCharacter0xa9b6',
u'\u0d39': 'MalayalamLetterHa',
u'\u833d': 'CjkUnifiedIdeograph833d',
u'\u62bc': 'CjkUnifiedIdeograph62bc',
u'\uf941': 'CjkCompatibilityIdeographf941',
u'\ud8c0': 'UnicodeCharacter0xd8c0',
u'\u3c43': 'CjkUnifiedIdeograph3c43',
u'\u1bc2': 'UnicodeCharacter0x1bc2',
u'\ub247': 'HangulSyllableNwelb',
u'\u91c6': 'CjkUnifiedIdeograph91c6',
u'\u6b4d': 'CjkUnifiedIdeograph6b4d',
u'\u4acc': 'CjkUnifiedIdeograph4acc',
u'\ue151': 'UnicodeCharacter0xe151',
u'\uc0d0': 'HangulSyllableSael',
u'\u2453': 'UnicodeCharacter0x2453',
u'\u03d2': 'GreekUpsilonWithHookSymbol',
u'\u9a57': 'CjkUnifiedIdeograph9a57',
u'\u79d6': 'CjkUnifiedIdeograph79d6',
u'\uffda': 'HalfwidthHangulLetterEu',
u'\u535d': 'CjkUnifiedIdeograph535d',
u'\u32dc': 'CircledKatakanaSu',
u'\uc961': 'HangulSyllableJwib',
u'\ua8e0': 'UnicodeCharacter0xa8e0',
u'\u0c63': 'TeluguVowelSignVocalicLl',
u'\u8267': 'CjkUnifiedIdeograph8267',
u'\u61e6': 'CjkUnifiedIdeograph61e6',
u'\uf86b': 'UnicodeCharacter0xf86b',
u'\ue7ea': 'UnicodeCharacter0xe7ea',
u'\u3b6d': 'CjkUnifiedIdeograph3b6d',
u'\u1aec': 'UnicodeCharacter0x1aec',
u'\ub171': 'HangulSyllableNyeng',
u'\u90f0': 'CjkUnifiedIdeograph90f0',
u'\u6a77': 'CjkUnifiedIdeograph6a77',
u'\u49f6': 'CjkUnifiedIdeograph49f6',
u'\ue07b': 'UnicodeCharacter0xe07b',
u'\ucffa': 'HangulSyllableKup',
u'\u237d': 'ShoulderedOpenBox',
u'\u02fc': 'ModifierLetterEndLowTone',
u'\u1881': 'MongolianLetterAliGaliVisargaOne',
u'\u7900': 'CjkUnifiedIdeograph7900',
u'\u5d83': 'CjkUnifiedIdeograph5d83',
u'\uba02': 'HangulSyllableMyagg',
u'\u9e85': 'CjkUnifiedIdeograph9e85',
u'\uff04': 'FullwidthDollarSign',
u'\ud387': 'HangulSyllablePeolb',
u'\u2c0e': 'GlagoliticCapitalLetterLjudije',
u'\x91': 'UnicodeCharacter0x91',
u'\u6110': 'CjkUnifiedIdeograph6110',
u'\u4593': 'CjkUnifiedIdeograph4593',
u'\ua212': 'YiSyllableKurx',
u'\u8695': 'CjkUnifiedIdeograph8695',
u'\ue714': 'UnicodeCharacter0xe714',
u'\ubb97': 'HangulSyllableMwilh',
u'\ufc99': 'ArabicLigatureYehWithHamzaAboveWithKhahInitialForm',
u'\u141e': 'CanadianSyllabicsGlottalStop',
u'\u4920': 'CjkUnifiedIdeograph4920',
u'\u2da3': 'EthiopicSyllableSsaa',
u'\u8a22': 'CjkUnifiedIdeograph8a22',
u'\u6ea5': 'CjkUnifiedIdeograph6ea5',
u'\ucf24': 'HangulSyllableKyeol',
u'\ua3a7': 'YiSyllableNry',
u'\ue4a9': 'UnicodeCharacter0xe4a9',
u'\u3130': 'UnicodeCharacter0x3130',
u'\u15b3': 'CanadianSyllabicsBlackfootA',
u'\u7232': 'CjkUnifiedIdeograph7232',
u'\u56b5': 'CjkUnifiedIdeograph56b5',
u'\ub734': 'HangulSyllableDdeuls',
u'\u8bb7': 'CjkUnifiedIdeograph8bb7',
u'\ue836': 'UnicodeCharacter0xe836',
u'\uccb9': 'HangulSyllableCenj',
u'\u1940': 'LimbuSignLoo',
u'\u5a42': 'CjkUnifiedIdeograph5a42',
u'\u3ec5': 'CjkUnifiedIdeograph3ec5',
u'\u9f44': 'CjkUnifiedIdeograph9f44',
u'\u73c7': 'CjkUnifiedIdeograph73c7',
u'\ud046': 'HangulSyllableKwibs',
u'\ub4c9': 'HangulSyllableDyulg',
u'\ue9cb': 'UnicodeCharacter0xe9cb',
u'\u0150': 'LatinCapitalLetterOWithDoubleAcute',
u'\u4252': 'CjkUnifiedIdeograph4252',
u'\u26d5': 'UnicodeCharacter0x26d5',
u'\u8754': 'CjkUnifiedIdeograph8754',
u'\u5bd7': 'CjkUnifiedIdeograph5bd7',
u'\ub856': 'HangulSyllableRyej',
u'\u9cd9': 'CjkUnifiedIdeograph9cd9',
u'\ufd58': 'ArabicLigatureJeemWithMeemWithHahFinalForm',
u'\ud1db': 'HangulSyllableTwaegs',
u'\u2a62': 'LogicalOrWithDoubleOverbar',
u'\u0ee5': 'UnicodeCharacter0xee5',
u'\u6f64': 'CjkUnifiedIdeograph6f64',
u'\u43e7': 'CjkUnifiedIdeograph43e7',
u'\ua066': 'YiSyllableBbox',
u'\u84e9': 'CjkUnifiedIdeograph84e9',
u'\ue568': 'UnicodeCharacter0xe568',
u'\ub9eb': 'HangulSyllableMaed',
u'\ufaed': 'UnicodeCharacter0xfaed',
u'\u1272': 'EthiopicSyllableTi',
u'\u5774': 'CjkUnifiedIdeograph5774',
u'\u2bf7': 'UnicodeCharacter0x2bf7',
u'\u8876': 'CjkUnifiedIdeograph8876',
u'\u6cf9': 'CjkUnifiedIdeograph6cf9',
u'\ucd78': 'HangulSyllableCyo',
u'\ua1fb': 'YiSyllableKip',
u'\ue2fd': 'UnicodeCharacter0xe2fd',
u'\ud801': 'UnicodeCharacter0xd801',
u'\u3f84': 'CjkUnifiedIdeograph3f84',
u'\u7086': 'CjkUnifiedIdeograph7086',
u'\ub588': 'HangulSyllableDdyaen',
u'\u090b': 'DevanagariLetterVocalicR',
u'\uf68a': 'UnicodeCharacter0xf68a',
u'\u4a0d': 'CjkUnifiedIdeograph4a0d',
u'\u8f0f': 'CjkUnifiedIdeograph8f0f',
u'\uc011': 'HangulSyllableBbwelt',
u'\u2794': 'HeavyWideheadedRightwardsArrow',
u'\u5896': 'CjkUnifiedIdeograph5896',
u'\u9d98': 'CjkUnifiedIdeograph9d98',
u'\ude9a': 'UnicodeCharacter0xde9a',
u'\u321d': 'ParenthesizedKoreanCharacterOjeon',
u'\u771f': 'CjkUnifiedIdeograph771f',
u'\ua821': 'SylotiNagriLetterSo',
u'\ued23': 'UnicodeCharacter0xed23',
u'\u0fa4': 'TibetanSubjoinedLetterPa',
u'\u40a6': 'CjkUnifiedIdeograph40a6',
u'\u85a8': 'CjkUnifiedIdeograph85a8',
u'\uc6aa': 'HangulSyllableYoj',
u'\u1a2d': 'UnicodeCharacter0x1a2d',
u'\ufbac': 'ArabicLetterHehDoachashmeeInitialForm',
u'\u5f2f': 'CjkUnifiedIdeograph5f2f',
u'\u9031': 'CjkUnifiedIdeograph9031',
u'\ud533': 'HangulSyllablePyis',
u'\u28b6': 'BraillePatternDots23568',
u'\u6db8': 'CjkUnifiedIdeograph6db8',
u'\uaeba': 'HangulSyllableGgyaep',
u'\u023d': 'LatinCapitalLetterLWithBar',
u'\ue3bc': 'UnicodeCharacter0xe3bc',
u'\u473f': 'CjkUnifiedIdeograph473f',
u'\u7841': 'CjkUnifiedIdeograph7841',
u'\ubd43': 'HangulSyllableBwaec',
u'\ufe45': 'SesameDot',
u'\u10c6': 'UnicodeCharacter0x10c6',
u'\u55c8': 'CjkUnifiedIdeograph55c8',
u'\u96ca': 'CjkUnifiedIdeograph96ca',
u'\ucbcc': 'HangulSyllableJjyuss',
u'\u2f4f': 'KangxiRadicalDoNot',
u'\u6051': 'CjkUnifiedIdeograph6051',
u'\ua553': 'VaiSyllableMba',
u'\ue655': 'UnicodeCharacter0xe655',
u'\u3dd8': 'CjkUnifiedIdeograph3dd8',
u'\u7eda': 'CjkUnifiedIdeograph7eda',
u'\ub3dc': 'HangulSyllableDok',
u'\u175f': 'UnicodeCharacter0x175f',
u'\uf4de': 'UnicodeCharacter0xf4de',
u'\u4861': 'CjkUnifiedIdeograph4861',
u'\u8d63': 'CjkUnifiedIdeograph8d63',
u'\uce65': 'HangulSyllableCilt',
u'\u25e8': 'SquareWithRightHalfBlack',
u'\u66ea': 'CjkUnifiedIdeograph66ea',
u'\u9bec': 'CjkUnifiedIdeograph9bec',
u'\udcee': 'UnicodeCharacter0xdcee',
u'\u3071': 'HiraganaLetterPa',
u'\u7573': 'CjkUnifiedIdeograph7573',
u'\ub675': 'HangulSyllableDdoeb',
u'\ueb77': 'UnicodeCharacter0xeb77',
u'\u0df8': 'UnicodeCharacter0xdf8',
u'\u4efa': 'CjkUnifiedIdeograph4efa',
u'\u83fc': 'CjkUnifiedIdeograph83fc',
u'\uc4fe': 'HangulSyllableSseulp',
u'\ufa00': 'CjkCompatibilityIdeographfa00',
u'\ude83': 'UnicodeCharacter0xde83',
u'\u3902': 'CjkUnifiedIdeograph3902',
u'\u1d85': 'LatinSmallLetterLWithPalatalHook',
u'\ub306': 'HangulSyllableDaenh',
u'\u9789': 'CjkUnifiedIdeograph9789',
u'\u680c': 'CjkUnifiedIdeograph680c',
u'\u4c8f': 'CjkUnifiedIdeograph4c8f',
u'\ue210': 'UnicodeCharacter0xe210',
u'\uc693': 'HangulSyllableOeh',
u'\u2112': 'ScriptCapitalL',
u'\u0595': 'HebrewAccentZaqefGadol',
u'\u9b16': 'CjkUnifiedIdeograph9b16',
u'\u7f99': 'CjkUnifiedIdeograph7f99',
u'\uf19d': 'UnicodeCharacter0xf19d',
u'\u501c': 'CjkUnifiedIdeograph501c',
u'\u349f': 'CjkUnifiedIdeograph349f',
u'\uca20': 'HangulSyllableJjyals',
u'\uaea3': 'HangulSyllableGgyaegs',
u'\u0922': 'DevanagariLetterDdha',
u'\u8326': 'CjkUnifiedIdeograph8326',
u'\u67a9': 'CjkUnifiedIdeograph67a9',
u'\ud9ad': 'UnicodeCharacter0xd9ad',
u'\u382c': 'CjkUnifiedIdeograph382c',
u'\u1caf': 'UnicodeCharacter0x1caf',
u'\ub230': 'HangulSyllableNweom',
u'\u96b3': 'CjkUnifiedIdeograph96b3',
u'\u6b36': 'CjkUnifiedIdeograph6b36',
u'\u4fb9': 'CjkUnifiedIdeograph4fb9',
u'\ued3a': 'UnicodeCharacter0xed3a',
u'\uc1bd': 'HangulSyllableSwang',
u'\u203c': 'DoubleExclamationMark',
u'\u04bf': 'CyrillicSmallLetterAbkhasianCheWithDescender',
u'\u9a40': 'CjkUnifiedIdeograph9a40',
u'\u7ec3': 'CjkUnifiedIdeograph7ec3',
u'\uf0c7': 'UnicodeCharacter0xf0c7',
u'\u5346': 'CjkUnifiedIdeograph5346',
u'\u37c9': 'CjkUnifiedIdeograph37c9',
u'\ud54a': 'HangulSyllablePilp',
u'\ua9cd': 'UnicodeCharacter0xa9cd',
u'\u084c': 'UnicodeCharacter0x84c',
u'\u8250': 'CjkUnifiedIdeograph8250',
u'\u66d3': 'CjkUnifiedIdeograph66d3',
u'\ud8d7': 'UnicodeCharacter0xd8d7',
u'\u3b56': 'CjkUnifiedIdeograph3b56',
u'\u1fd9': 'GreekCapitalLetterIotaWithMacron',
u'\ubd5a': 'HangulSyllableBoebs',
u'\u91dd': 'CjkUnifiedIdeograph91dd',
u'\u6a60': 'CjkUnifiedIdeograph6a60',
u'\u4ee3': 'CjkUnifiedIdeograph4ee3',
u'\uec64': 'UnicodeCharacter0xec64',
u'\uc0e7': 'HangulSyllableSyags',
u'\u2366': 'AplFunctionalSymbolDownShoeStile',
u'\u07e9': 'NkoLetterJonaCha',
u'\ua56a': 'VaiSyllableKa',
u'\u79ed': 'CjkUnifiedIdeograph79ed',
u'\uf3f1': 'UnicodeCharacter0xf3f1',
u'\u5270': 'CjkUnifiedIdeograph5270',
u'\u36f3': 'CjkUnifiedIdeograph36f3',
u'\ud474': 'HangulSyllablePyok',
u'\ua8f7': 'UnicodeCharacter0xa8f7',
u'\u0b76': 'UnicodeCharacter0xb76',
u'\u8d7a': 'CjkUnifiedIdeograph8d7a',
u'\u61fd': 'CjkUnifiedIdeograph61fd',
u'\u5b01': 'CjkUnifiedIdeograph5b01',
u'\u3a80': 'CjkUnifiedIdeograph3a80',
u'\udd05': 'UnicodeCharacter0xdd05',
u'\ubc84': 'HangulSyllableBeo',
u'\u1007': 'MyanmarLetterJa',
u'\u8a0b': 'CjkUnifiedIdeograph8a0b',
u'\u758a': 'CjkUnifiedIdeograph758a',
u'\uef8e': 'UnicodeCharacter0xef8e',
u'\u4311': 'CjkUnifiedIdeograph4311',
u'\u2290': 'SquareOriginalOf',
u'\uc515': 'HangulSyllableSsyilg',
u'\ua494': 'YiRadicalCyp',
u'\u721b': 'CjkUnifiedIdeograph721b',
u'\u5d9a': 'CjkUnifiedIdeograph5d9a',
u'\uf41f': 'UnicodeCharacter0xf41f',
u'\ud79e': 'HangulSyllableHij',
u'\u2b21': 'WhiteHexagon',
u'\u0aa0': 'GujaratiLetterTtha',
u'\uad25': 'HangulSyllableGwaelt',
u'\u8ca4': 'CjkUnifiedIdeograph8ca4',
u'\u5a2b': 'CjkUnifiedIdeograph5a2b',
u'\u45aa': 'CjkUnifiedIdeograph45aa',
u'\udc2f': 'UnicodeCharacter0xdc2f',
u'\ubfae': 'HangulSyllableBboep',
u'\u1331': 'EthiopicSyllablePhu',
u'\u9535': 'CjkUnifiedIdeograph9535',
u'\u74b4': 'CjkUnifiedIdeograph74b4',
u'\ueeb8': 'UnicodeCharacter0xeeb8',
u'\u423b': 'CjkUnifiedIdeograph423b',
u'\u2dba': 'EthiopicSyllableCchi',
u'\uc43f': 'HangulSyllableSsoes',
u'\ua7be': 'UnicodeCharacter0xa7be',
u'\u7d45': 'CjkUnifiedIdeograph7d45',
u'\u5cc4': 'CjkUnifiedIdeograph5cc4',
u'\uf749': 'UnicodeCharacter0xf749',
u'\ud6c8': 'HangulSyllableHun',
u'\u2a4b': 'IntersectionBesideAndJoinedWithIntersection',
u'\u15ca': 'CanadianSyllabicsCarrierRu',
u'\uac4f': 'HangulSyllableGyac',
u'\u8fce': 'CjkUnifiedIdeograph8fce',
u'\u6555': 'CjkUnifiedIdeograph6555',
u'\u44d4': 'CjkUnifiedIdeograph44d4',
u'\udf59': 'UnicodeCharacter0xdf59',
u'\ubed8': 'HangulSyllableBbeol',
u'\u125b': 'EthiopicSyllableQhwaa',
u'\u945f': 'CjkUnifiedIdeograph945f',
u'\u77de': 'CjkUnifiedIdeograph77de',
u'\ue9e2': 'UnicodeCharacter0xe9e2',
u'\u4d65': 'CjkUnifiedIdeograph4d65',
u'\u2ce4': 'CopticSymbolKai',
u'\uc769': 'HangulSyllableYib',
u'\ua6e8': 'UnicodeCharacter0xa6e8',
u'\u7c6f': 'CjkUnifiedIdeograph7c6f',
u'\u5fee': 'CjkUnifiedIdeograph5fee',
u'\uf673': 'UnicodeCharacter0xf673',
u'\ud1f2': 'HangulSyllableTwaep',
u'\u3575': 'CjkUnifiedIdeograph3575',
u'\u14f4': 'CanadianSyllabicsSa',
u'\uaf79': 'HangulSyllableGgwaeng',
u'\u8ef8': 'CjkUnifiedIdeograph8ef8',
u'\u647f': 'CjkUnifiedIdeograph647f',
u'\u47fe': 'CjkUnifiedIdeograph47fe',
u'\u3206': 'ParenthesizedHangulSios',
u'\u1689': 'OghamLetterColl',
u'\u7708': 'CjkUnifiedIdeograph7708',
u'\u4b8b': 'CjkUnifiedIdeograph4b8b',
u'\ub40a': 'HangulSyllableDwaelp',
u'\u888d': 'CjkUnifiedIdeograph888d',
u'\ue90c': 'UnicodeCharacter0xe90c',
u'\ucd8f': 'HangulSyllableCyoc',
u'\u1a16': 'BugineseLetterHa',
u'\u5f18': 'CjkUnifiedIdeograph5f18',
u'\u339b': 'SquareMuM',
u'\u9c1a': 'CjkUnifiedIdeograph9c1a',
u'\u709d': 'CjkUnifiedIdeograph709d',
u'\ud11c': 'HangulSyllableTyael',
u'\ub59f': 'HangulSyllableDdyaeh',
u'\ueaa1': 'UnicodeCharacter0xeaa1',
u'\u0226': 'LatinCapitalLetterAWithDotAbove',
u'\u4728': 'CjkUnifiedIdeograph4728',
u'\u1bab': 'UnicodeCharacter0x1bab',
u'\u842a': 'CjkUnifiedIdeograph842a',
u'\u58ad': 'CjkUnifiedIdeograph58ad',
u'\ub92c': 'HangulSyllableRwels',
u'\u9daf': 'CjkUnifiedIdeograph9daf',
u'\ufe2e': 'UnicodeCharacter0xfe2e',
u'\ud2b1': 'HangulSyllableTyung',
u'\u2f38': 'KangxiRadicalBow',
u'\u03bb': 'GreekSmallLetterLamda',
u'\u6c3a': 'CjkUnifiedIdeograph6c3a',
u'\u40bd': 'CjkUnifiedIdeograph40bd',
u'\ua13c': 'YiSyllableDdiep',
u'\u85bf': 'CjkUnifiedIdeograph85bf',
u'\ue63e': 'UnicodeCharacter0xe63e',
u'\ubac1': 'HangulSyllableMot',
u'\uffc3': 'HalfwidthHangulLetterAe',
u'\u1748': 'BuhidLetterNa',
u'\u544a': 'CjkUnifiedIdeograph544a',
u'\u28cd': 'BraillePatternDots13478',
u'\u894c': 'CjkUnifiedIdeograph894c',
u'\u6dcf': 'CjkUnifiedIdeograph6dcf',
u'\uce4e': 'HangulSyllableCyibs',
u'\ua2d1': 'YiSyllableZzix',
u'\ue7d3': 'UnicodeCharacter0xe7d3',
u'\u3c5a': 'CjkUnifiedIdeograph3c5a',
u'\u10dd': 'GeorgianLetterOn',
u'\u715c': 'CjkUnifiedIdeograph715c',
u'\u55df': 'CjkUnifiedIdeograph55df',
u'\ub65e': 'HangulSyllableDdwaej',
u'\u8ae1': 'CjkUnifiedIdeograph8ae1',
u'\ueb60': 'UnicodeCharacter0xeb60',
u'\ucfe3': 'HangulSyllableKugs',
u'\u246a': 'CircledNumberEleven',
u'\u596c': 'CjkUnifiedIdeograph596c',
u'\u3def': 'CjkUnifiedIdeograph3def',
u'\u9e6e': 'CjkUnifiedIdeograph9e6e',
u'\u72f1': 'CjkUnifiedIdeograph72f1',
u'\ud370': 'HangulSyllablePyaem',
u'\ub7f3': 'HangulSyllableReod',
u'\uf4f5': 'UnicodeCharacter0xf4f5',
u'\u0c7a': 'TeluguFractionDigitTwoForOddPowersOfFour',
u'\u417c': 'CjkUnifiedIdeograph417c',
u'\u25ff': 'LowerRightTriangle',
u'\u867e': 'CjkUnifiedIdeograph867e',
u'\ubb80': 'HangulSyllableMwess',
u'\u1f03': 'GreekSmallLetterAlphaWithDasiaAndVaria',
u'\uf882': 'UnicodeCharacter0xf882',
u'\u5c05': 'CjkUnifiedIdeograph5c05',
u'\u9107': 'CjkUnifiedIdeograph9107',
u'\ud609': 'HangulSyllableHyeolg',
u'\u298c': 'RightSquareBracketWithUnderbar',
u'\u6e8e': 'CjkUnifiedIdeograph6e8e',
u'\ua390': 'YiSyllableRry',
u'\u0713': 'SyriacLetterGamal',
u'\ue092': 'UnicodeCharacter0xe092',
u'\u4415': 'CjkUnifiedIdeograph4415',
u'\u7917': 'CjkUnifiedIdeograph7917',
u'\ube19': 'HangulSyllableBeult',
u'\uf31b': 'UnicodeCharacter0xf31b',
u'\u119c': 'HangulJungseongIeu',
u'\u569e': 'CjkUnifiedIdeograph569e',
u'\u8ba0': 'CjkUnifiedIdeograph8ba0',
u'\uc8a2': 'HangulSyllableJwaj',
u'\u2c25': 'GlagoliticCapitalLetterSmallYusWithTail',
u'\u6127': 'CjkUnifiedIdeograph6127',
u'\ua629': 'VaiDigitNine',
u'\udb2b': 'UnicodeCharacter0xdb2b',
u'\u3eae': 'CjkUnifiedIdeograph3eae',
u'\u73b0': 'CjkUnifiedIdeograph73b0',
u'\ub0b2': 'HangulSyllableNap',
u'\u1435': 'CanadianSyllabicsYcreePoo',
u'\uf5b4': 'UnicodeCharacter0xf5b4',
u'\u4937': 'CjkUnifiedIdeograph4937',
u'\u8e39': 'CjkUnifiedIdeograph8e39',
u'\uc33b': 'HangulSyllableSsyalb',
u'\u26be': 'UnicodeCharacter0x26be',
u'\u5bc0': 'CjkUnifiedIdeograph5bc0',
u'\u98c2': 'CjkUnifiedIdeograph98c2',
u'\uddc4': 'UnicodeCharacter0xddc4',
u'\u3147': 'HangulLetterIeung',
u'\u7649': 'CjkUnifiedIdeograph7649',
u'\uab4b': 'UnicodeCharacter0xab4b',
u'\ue84d': 'UnicodeCharacter0xe84d',
u'\u0ece': 'UnicodeCharacter0xece',
u'\u43d0': 'CjkUnifiedIdeograph43d0',
u'\u80d2': 'CjkUnifiedIdeograph80d2',
u'\uc5d4': 'HangulSyllableEn',
u'\u1957': 'TaiLeLetterTha',
u'\ufad6': 'CjkCompatibilityIdeographfad6',
u'\u5e59': 'CjkUnifiedIdeograph5e59',
u'\u935b': 'CjkUnifiedIdeograph935b',
u'\ud05d': 'HangulSyllableKyult',
u'\u2be0': 'UnicodeCharacter0x2be0',
u'\u68e2': 'CjkUnifiedIdeograph68e2',
u'\uade4': 'HangulSyllableGyul',
u'\u0167': 'LatinSmallLetterTWithStroke',
u'\ue2e6': 'UnicodeCharacter0xe2e6',
u'\u4669': 'CjkUnifiedIdeograph4669',
u'\u7b6b': 'CjkUnifiedIdeograph7b6b',
u'\ub86d': 'HangulSyllableRob',
u'\ufd6f': 'ArabicLigatureDadWithKhahWithMeemFinalForm',
u'\u13f0': 'CherokeeLetterYe',
u'\u50f2': 'CjkUnifiedIdeograph50f2',
u'\u95f4': 'CjkUnifiedIdeograph95f4',
u'\ucaf6': 'HangulSyllableJjwaegg',
u'\u2e79': 'UnicodeCharacter0x2e79',
u'\u637b': 'CjkUnifiedIdeograph637b',
u'\ua07d': 'YiSyllableNbat',
u'\ue57f': 'UnicodeCharacter0xe57f',
u'\u9d81': 'CjkUnifiedIdeograph9d81',
u'\u7a04': 'CjkUnifiedIdeograph7a04',
u'\u5687': 'CjkUnifiedIdeograph5687',
u'\uf008': 'UnicodeCharacter0xf008',
u'\ucc8b': 'HangulSyllableCyaelh',
u'\u330a': 'SquareOomu',
u'\u0f8d': 'UnicodeCharacter0xf8d',
u'\ua90e': 'KayahLiLetterSa',
u'\u8591': 'CjkUnifiedIdeograph8591',
u'\u6214': 'CjkUnifiedIdeograph6214',
u'\u3e97': 'CjkUnifiedIdeograph3e97',
u'\ud818': 'UnicodeCharacter0xd818',
u'\ub49b': 'HangulSyllableDwes',
u'\u1b1a': 'BalineseLetterJa',
u'\u911e': 'CjkUnifiedIdeograph911e',
u'\u6da1': 'CjkUnifiedIdeograph6da1',
u'\ueba5': 'UnicodeCharacter0xeba5',
u'\u4a24': 'CjkUnifiedIdeograph4a24',
u'\u26a7': 'MaleWithStrokeAndMaleAndFemaleSign',
u'\uc028': 'HangulSyllableBbwil',
u'\u9cab': 'CjkUnifiedIdeograph9cab',
u'\u032a': 'CombiningBridgeBelow',
u'\u792e': 'CjkUnifiedIdeograph792e',
u'\u55b1': 'CjkUnifiedIdeograph55b1',
u'\uf732': 'UnicodeCharacter0xf732',
u'\ud3b5': 'HangulSyllablePyeog',
u'\u3234': 'ParenthesizedIdeographName',
u'\u0eb7': 'LaoVowelSignYy',
u'\ua838': 'UnicodeCharacter0xa838',
u'\u84bb': 'CjkUnifiedIdeograph84bb',
u'\u613e': 'CjkUnifiedIdeograph613e',
u'\u3dc1': 'CjkUnifiedIdeograph3dc1',
u'\udf42': 'UnicodeCharacter0xdf42',
u'\ubbc5': 'HangulSyllableMeunj',
u'\u1a44': 'UnicodeCharacter0x1a44',
u'\u9048': 'CjkUnifiedIdeograph9048',
u'\u6ccb': 'CjkUnifiedIdeograph6ccb',
u'\ueacf': 'UnicodeCharacter0xeacf',
u'\u494e': 'CjkUnifiedIdeograph494e',
u'\u25d1': 'CircleWithRightHalfBlack',
u'\uc752': 'HangulSyllableEuj',
u'\ua3d5': 'YiSyllableRux',
u'\u0254': 'LatinSmallLetterOpenO',
u'\u7858': 'CjkUnifiedIdeograph7858',
u'\u54db': 'CjkUnifiedIdeograph54db',
u'\uf65c': 'UnicodeCharacter0xf65c',
u'\ud2df': 'HangulSyllableTyilb',
u'\u315e': 'HangulLetterWe',
u'\u0de1': 'UnicodeCharacter0xde1',
u'\uaf62': 'HangulSyllableGgwap',
u'\u8be5': 'CjkUnifiedIdeograph8be5',
u'\u6068': 'CjkUnifiedIdeograph6068',
u'\u3ceb': 'CjkUnifiedIdeograph3ceb',
u'\ude6c': 'UnicodeCharacter0xde6c',
u'\ubaef': 'HangulSyllableMwaelh',
u'\u196e': 'UnicodeCharacter0x196e',
u'\u9772': 'CjkUnifiedIdeograph9772',
u'\u73f5': 'CjkUnifiedIdeograph73f5',
u'\ue9f9': 'UnicodeCharacter0xe9f9',
u'\u4878': 'CjkUnifiedIdeograph4878',
u'\u24fb': 'DoubleCircledDigitSeven',
u'\uc67c': 'HangulSyllableOen',
u'\ua2ff': 'YiSyllableNzu',
u'\u017e': 'LatinSmallLetterZWithCaron',
u'\u9803': 'CjkUnifiedIdeograph9803',
u'\u7f82': 'CjkUnifiedIdeograph7f82',
u'\uf586': 'UnicodeCharacter0xf586',
u'\u5109': 'CjkUnifiedIdeograph5109',
u'\u3088': 'HiraganaLetterYo',
u'\ucf0d': 'HangulSyllableKelt',
u'\uae8c': 'HangulSyllableGgyal',
u'\u0a0f': 'GurmukhiLetterEe',
u'\u8013': 'CjkUnifiedIdeograph8013',
u'\u6792': 'CjkUnifiedIdeograph6792',
u'\ufe17': 'PresentationFormForVerticalLeftWhiteLenticularBracket',
u'\udd96': 'UnicodeCharacter0xdd96',
u'\u3919': 'CjkUnifiedIdeograph3919',
u'\u1898': 'MongolianLetterTodoAliGaliTa',
u'\ub71d': 'HangulSyllableDdyub',
u'\u969c': 'CjkUnifiedIdeograph969c',
u'\u6823': 'CjkUnifiedIdeograph6823',
u'\u4fa2': 'CjkUnifiedIdeograph4fa2',
u'\ue627': 'UnicodeCharacter0xe627',
u'\uc5a6': 'HangulSyllableYaelp',
u'\u2129': 'TurnedGreekSmallLetterIota',
u'\xa8': 'Diaeresis',
u'\u9f2d': 'CjkUnifiedIdeograph9f2d',
u'\u7eac': 'CjkUnifiedIdeograph7eac',
u'\uf4b0': 'UnicodeCharacter0xf4b0',
u'\u5033': 'CjkUnifiedIdeograph5033',
u'\u37b2': 'CjkUnifiedIdeograph37b2',
u'\uce37': 'HangulSyllableCeuc',
u'\uadb6': 'HangulSyllableGwebs',
u'\u0939': 'DevanagariLetterHa',
u'\u873d': 'CjkUnifiedIdeograph873d',
u'\u66bc': 'CjkUnifiedIdeograph66bc',
u'\ufd41': 'UnicodeCharacter0xfd41',
u'\udcc0': 'UnicodeCharacter0xdcc0',
u'\u3843': 'CjkUnifiedIdeograph3843',
u'\u1fc2': 'GreekSmallLetterEtaWithVariaAndYpogegrammeni',
u'\ub647': 'HangulSyllableDdwah',
u'\u95c6': 'CjkUnifiedIdeograph95c6',
u'\u6f4d': 'CjkUnifiedIdeograph6f4d',
u'\u4ecc': 'CjkUnifiedIdeograph4ecc',
u'\ue551': 'UnicodeCharacter0xe551',
u'\uc4d0': 'HangulSyllableSswik',
u'\u2053': 'SwungDash',
u'\u07d2': 'NkoLetterN',
u'\u9e57': 'CjkUnifiedIdeograph9e57',
u'\u7dd6': 'CjkUnifiedIdeograph7dd6',
u'\ufbda': 'ArabicLetterOeFinalForm',
u'\u575d': 'CjkUnifiedIdeograph575d',
u'\u36dc': 'CjkUnifiedIdeograph36dc',
u'\ucd61': 'HangulSyllableCoenj',
u'\uace0': 'HangulSyllableGo',
u'\u0863': 'UnicodeCharacter0x863',
u'\u8667': 'CjkUnifiedIdeograph8667',
u'\u65e6': 'CjkUnifiedIdeograph65e6',
u'\ufc6b': 'ArabicLigatureBehWithZainFinalForm',
u'\ue3ea': 'UnicodeCharacter0xe3ea',
u'\u3f6d': 'CjkUnifiedIdeograph3f6d',
u'\u1eec': 'LatinCapitalLetterUWithHornAndHookAbove',
u'\ub571': 'HangulSyllableDdyalg',
u'\u94f0': 'CjkUnifiedIdeograph94f0',
u'\u6e77': 'CjkUnifiedIdeograph6e77',
u'\u4df6': 'HexagramForAbundance',
u'\ue47b': 'UnicodeCharacter0xe47b',
u'\ucbfa': 'HangulSyllableJjyilm',
u'\u277d': 'DingbatNegativeCircledDigitEight',
u'\u06fc': 'ArabicLetterGhainWithDotBelow',
u'\u1c81': 'UnicodeCharacter0x1c81',
u'\u7d00': 'CjkUnifiedIdeograph7d00',
u'\u5983': 'CjkUnifiedIdeograph5983',
u'\ube02': 'HangulSyllableByubs',
u'\u9a85': 'CjkUnifiedIdeograph9a85',
u'\ufb04': 'LatinSmallLigatureFfl',
u'\ud787': 'HangulSyllableHyih',
u'\u280e': 'BraillePatternDots234',
u'\u0491': 'CyrillicSmallLetterGheWithUpturn',
u'\u6510': 'CjkUnifiedIdeograph6510',
u'\u4193': 'CjkUnifiedIdeograph4193',
u'\ua612': 'VaiSyllableNdoleSoo',
u'\u8295': 'CjkUnifiedIdeograph8295',
u'\ue314': 'UnicodeCharacter0xe314',
u'\ubf97': 'HangulSyllableBboegs',
u'\uf899': 'UnicodeCharacter0xf899',
u'\u101e': 'MyanmarLetterSa',
u'\u4d20': 'CjkUnifiedIdeograph4d20',
u'\u29a3': 'ReversedAngle',
u'\u8e22': 'CjkUnifiedIdeograph8e22',
u'\u6aa5': 'CjkUnifiedIdeograph6aa5',
u'\ucb24': 'HangulSyllableJjoess',
u'\ua7a7': 'UnicodeCharacter0xa7a7',
u'\ue0a9': 'UnicodeCharacter0xe0a9',
u'\u3530': 'CjkUnifiedIdeograph3530',
u'\u11b3': 'HangulJongseongRieulsios',
u'\u7632': 'CjkUnifiedIdeograph7632',
u'\u52b5': 'CjkUnifiedIdeograph52b5',
u'\ub334': 'HangulSyllableDyak',
u'\u8fb7': 'CjkUnifiedIdeograph8fb7',
u'\uec36': 'UnicodeCharacter0xec36',
u'\uc8b9': 'HangulSyllableJwaeb',
u'\u1d40': 'ModifierLetterCapitalT',
u'\u5e42': 'CjkUnifiedIdeograph5e42',
u'\u3ac5': 'CjkUnifiedIdeograph3ac5',
u'\u9b44': 'CjkUnifiedIdeograph9b44',
u'\u77c7': 'CjkUnifiedIdeograph77c7',
u'\ud446': 'HangulSyllablePoenh',
u'\ub0c9': 'HangulSyllableNaeng',
u'\uedcb': 'UnicodeCharacter0xedcb',
u'\u0550': 'ArmenianCapitalLetterReh',
u'\u4652': 'CjkUnifiedIdeograph4652',
u'\u22d5': 'EqualAndParallelTo',
u'\u8354': 'CjkUnifiedIdeograph8354',
u'\u5fd7': 'CjkUnifiedIdeograph5fd7',
u'\ubc56': 'HangulSyllableByalm',
u'\u98d9': 'CjkUnifiedIdeograph98d9',
u'\uf958': 'CjkCompatibilityIdeographf958',
u'\ud5db': 'HangulSyllableHeos',
u'\u2e62': 'UnicodeCharacter0x2e62',
u'\u0ae5': 'UnicodeCharacter0xae5',
u'\u6b64': 'CjkUnifiedIdeograph6b64',
u'\u47e7': 'CjkUnifiedIdeograph47e7',
u'\ua466': 'YiSyllableXuo',
u'\u80e9': 'CjkUnifiedIdeograph80e9',
u'\ue168': 'UnicodeCharacter0xe168',
u'\ubdeb': 'HangulSyllableBwic',
u'\ufeed': 'ArabicLetterWawIsolatedForm',
u'\u1672': 'CanadianSyllabicsNngii',
u'\u5374': 'CjkUnifiedIdeograph5374',
u'\u2ff7': 'IdeographicDescriptionCharacterSurroundFromLeft',
u'\u8c76': 'CjkUnifiedIdeograph8c76',
u'\u68f9': 'CjkUnifiedIdeograph68f9',
u'\uc978': 'HangulSyllableJyuls',
u'\ua5fb': 'VaiSyllableSe',
u'\ue6fd': 'UnicodeCharacter0xe6fd',
u'\udc01': 'UnicodeCharacter0xdc01',
u'\u3b84': 'CjkUnifiedIdeograph3b84',
u'\u7486': 'CjkUnifiedIdeograph7486',
u'\ub188': 'HangulSyllableNom',
u'\u0d0b': 'MalayalamLetterVocalicR',
u'\uf28a': 'UnicodeCharacter0xf28a',
u'\u4e0d': 'CjkUnifiedIdeograph4e0d',
u'\u8b0f': 'CjkUnifiedIdeograph8b0f',
u'\uc411': 'HangulSyllableSswaeg',
u'\u2394': 'SoftwarefunctionSymbol',
u'\u5c96': 'CjkUnifiedIdeograph5c96',
u'\u9998': 'CjkUnifiedIdeograph9998',
u'\uda9a': 'UnicodeCharacter0xda9a',
u'\u361d': 'CjkUnifiedIdeograph361d',
u'\u731f': 'CjkUnifiedIdeograph731f',
u'\uac21': 'HangulSyllableGaenj',
u'\ue923': 'UnicodeCharacter0xe923',
u'\u0ba4': 'TamilLetterTa',
u'\u44a6': 'CjkUnifiedIdeograph44a6',
u'\u81a8': 'CjkUnifiedIdeograph81a8',
u'\uc2aa': 'HangulSyllableSeunh',
u'\u1e2d': 'LatinSmallLetterIWithTildeBelow',
u'\uffac': 'HalfwidthHangulLetterRieulpieup',
u'\u5b2f': 'CjkUnifiedIdeograph5b2f',
u'\u9431': 'CjkUnifiedIdeograph9431',
u'\ud133': 'HangulSyllableTeogs',
u'\u2cb6': 'CopticCapitalLetterCryptogrammicEie',
u'\u69b8': 'CjkUnifiedIdeograph69b8',
u'\uaaba': 'UnicodeCharacter0xaaba',
u'\u063d': 'ArabicLetterFarsiYehWithInvertedV',
u'\ue7bc': 'UnicodeCharacter0xe7bc',
u'\u433f': 'CjkUnifiedIdeograph433f',
u'\u7c41': 'CjkUnifiedIdeograph7c41',
u'\ub943': 'HangulSyllableRwid',
u'\ufa45': 'CjkCompatibilityIdeographfa45',
u'\u14c6': 'CanadianSyllabicsYcreeNoo',
u'\u51c8': 'CjkUnifiedIdeograph51c8',
u'\u92ca': 'CjkUnifiedIdeograph92ca',
u'\ucfcc': 'HangulSyllableKyol',
u'\u2b4f': 'UnicodeCharacter0x2b4f',
u'\u6451': 'CjkUnifiedIdeograph6451',
u'\ua153': 'YiSyllableNdi',
u'\ue255': 'UnicodeCharacter0xe255',
u'\u39d8': 'CjkUnifiedIdeograph39d8',
u'\u7ada': 'CjkUnifiedIdeograph7ada',
u'\ub7dc': 'HangulSyllableRyaels',
u'\u135f': 'EthiopicCombiningGeminationMark',
u'\uf0de': 'UnicodeCharacter0xf0de',
u'\u4c61': 'CjkUnifiedIdeograph4c61',
u'\u8963': 'CjkUnifiedIdeograph8963',
u'\uca65': 'HangulSyllableJjeot',
u'\u21e8': 'RightwardsWhiteArrow',
u'\u62ea': 'CjkUnifiedIdeograph62ea',
u'\u9fec': 'UnicodeCharacter0x9fec',
u'\ud8ee': 'UnicodeCharacter0xd8ee',
u'\u3471': 'CjkUnifiedIdeograph3471',
u'\u7173': 'CjkUnifiedIdeograph7173',
u'\ub275': 'HangulSyllableNyug',
u'\uef77': 'UnicodeCharacter0xef77',
u'\u09f8': 'BengaliCurrencyNumeratorOneLessThanTheDenominator',
u'\u4afa': 'CjkUnifiedIdeograph4afa',
u'\u87fc': 'CjkUnifiedIdeograph87fc',
u'\uc0fe': 'HangulSyllableSyap',
u'\ufe00': 'VariationSelector1',
u'\uda83': 'UnicodeCharacter0xda83',
u'\u3d02': 'CjkUnifiedIdeograph3d02',
u'\u1985': 'NewTaiLueLetterLowKa',
u'\ub706': 'HangulSyllableDdwij',
u'\u9389': 'CjkUnifiedIdeograph9389',
u'\u6c0c': 'CjkUnifiedIdeograph6c0c',
u'\u488f': 'CjkUnifiedIdeograph488f',
u'\ue610': 'UnicodeCharacter0xe610',
u'\uc293': 'HangulSyllableSyulb',
u'\u2512': 'BoxDrawingsDownHeavyAndLeftLight',
u'\u0195': 'LatinSmallLetterHv',
u'\u9f16': 'CjkUnifiedIdeograph9f16',
u'\u7b99': 'CjkUnifiedIdeograph7b99',
u'\uf59d': 'UnicodeCharacter0xf59d',
u'\u541c': 'CjkUnifiedIdeograph541c',
u'\u309f': 'HiraganaDigraphYori',
u'\uce20': 'HangulSyllableCeu',
u'\uaaa3': 'UnicodeCharacter0xaaa3',
u'\u0d22': 'MalayalamLetterDdha',
u'\u8726': 'CjkUnifiedIdeograph8726',
u'\u63a9': 'CjkUnifiedIdeograph63a9',
u'\uddad': 'UnicodeCharacter0xddad',
u'\u3c2c': 'CjkUnifiedIdeograph3c2c',
u'\u18af': 'UnicodeCharacter0x18af',
u'\ub630': 'HangulSyllableDdwan',
u'\u92b3': 'CjkUnifiedIdeograph92b3',
u'\u6f36': 'CjkUnifiedIdeograph6f36',
u'\u4bb9': 'CjkUnifiedIdeograph4bb9',
u'\ue93a': 'UnicodeCharacter0xe93a',
u'\uc5bd': 'HangulSyllableEolg',
u'\u243c': 'UnicodeCharacter0x243c',
u'\xbf': 'InvertedQuestionMark',
u'\u9e40': 'CjkUnifiedIdeograph9e40',
u'\u7ac3': 'CjkUnifiedIdeograph7ac3',
u'\uf4c7': 'UnicodeCharacter0xf4c7',
u'\u5746': 'CjkUnifiedIdeograph5746',
u'\u33c9': 'SquareGy',
u'\ud14a': 'HangulSyllableTeop',
u'\uadcd': 'HangulSyllableGwilt',
u'\u0c4c': 'TeluguVowelSignAu',
u'\u8650': 'CjkUnifiedIdeograph8650',
u'\u62d3': 'CjkUnifiedIdeograph62d3',
u'\udcd7': 'UnicodeCharacter0xdcd7',
u'\u3f56': 'CjkUnifiedIdeograph3f56',
u'\u1bd9': 'UnicodeCharacter0x1bd9',
u'\ub95a': 'HangulSyllableRyugg',
u'\u95dd': 'CjkUnifiedIdeograph95dd',
u'\u6e60': 'CjkUnifiedIdeograph6e60',
u'\u4ae3': 'CjkUnifiedIdeograph4ae3',
u'\ue864': 'UnicodeCharacter0xe864',
u'\uc4e7': 'HangulSyllableSsyus',
u'\u2766': 'FloralHeart',
u'\u03e9': 'CopticSmallLetterHori',
u'\ua16a': 'YiSyllableHni',
u'\u7ded': 'CjkUnifiedIdeograph7ded',
u'\uf7f1': 'UnicodeCharacter0xf7f1',
u'\u5670': 'CjkUnifiedIdeograph5670',
u'\u32f3': 'CircledKatakanaYa',
u'\ud074': 'HangulSyllableKeul',
u'\uacf7': 'HangulSyllableGoc',
u'\u0f76': 'TibetanVowelSignVocalicR',
u'\u897a': 'CjkUnifiedIdeograph897a',
u'\u65fd': 'CjkUnifiedIdeograph65fd',
u'\u5f01': 'CjkUnifiedIdeograph5f01',
u'\u3e80': 'CjkUnifiedIdeograph3e80',
u'\ud905': 'UnicodeCharacter0xd905',
u'\ub884': 'HangulSyllableRwals',
u'\u1407': 'CanadianSyllabicsYcreeOo',
u'\u8e0b': 'CjkUnifiedIdeograph8e0b',
u'\u718a': 'CjkUnifiedIdeograph718a',
u'\ueb8e': 'UnicodeCharacter0xeb8e',
u'\u4711': 'CjkUnifiedIdeograph4711',
u'\u2690': 'WhiteFlag',
u'\uc115': 'HangulSyllableSyaeng',
u'\ua094': 'YiSyllableHmip',
u'\u761b': 'CjkUnifiedIdeograph761b',
u'\u599a': 'CjkUnifiedIdeograph599a',
u'\uf01f': 'UnicodeCharacter0xf01f',
u'\ud39e': 'HangulSyllablePenh',
u'\u2f21': 'KangxiRadicalGo',
u'\u0ea0': 'UnicodeCharacter0xea0',
u'\ua925': 'KayahLiLetterOo',
u'\u88a4': 'CjkUnifiedIdeograph88a4',
u'\u5e2b': 'CjkUnifiedIdeograph5e2b',
u'\u41aa': 'CjkUnifiedIdeograph41aa',
u'\ud82f': 'UnicodeCharacter0xd82f',
u'\ubbae': 'HangulSyllableMyulm',
u'\u1731': 'HanunooLetterHa',
u'\u9135': 'CjkUnifiedIdeograph9135',
u'\u70b4': 'CjkUnifiedIdeograph70b4',
u'\ueab8': 'UnicodeCharacter0xeab8',
u'\u463b': 'CjkUnifiedIdeograph463b',
u'\u29ba': 'CircleDividedByHorizontalBarAndTopHalfDividedByVerticalBar',
u'\uc03f': 'HangulSyllableBbyugs',
u'\ua3be': 'YiSyllableShurx',
u'\u7945': 'CjkUnifiedIdeograph7945',
u'\u58c4': 'CjkUnifiedIdeograph58c4',
u'\uf349': 'UnicodeCharacter0xf349',
u'\ud2c8': 'HangulSyllableTeum',
u'\u2e4b': 'UnicodeCharacter0x2e4b',
u'\u11ca': 'HangulJongseongTikeutkiyeok',
u'\ua84f': 'PhagspaLetterMa',
u'\u8bce': 'CjkUnifiedIdeograph8bce',
u'\u6155': 'CjkUnifiedIdeograph6155',
u'\u40d4': 'CjkUnifiedIdeograph40d4',
u'\udb59': 'UnicodeCharacter0xdb59',
u'\ubad8': 'HangulSyllableMwass',
u'\u165b': 'CanadianSyllabicsCarrierTsu',
u'\u905f': 'CjkUnifiedIdeograph905f',
u'\u73de': 'CjkUnifiedIdeograph73de',
u'\uede2': 'UnicodeCharacter0xede2',
u'\u4965': 'CjkUnifiedIdeograph4965',
u'\u28e4': 'BraillePatternDots3678',
u'\uc369': 'HangulSyllableSseog',
u'\ua2e8': 'YiSyllableZzyx',
u'\u786f': 'CjkUnifiedIdeograph786f',
u'\u5bee': 'CjkUnifiedIdeograph5bee',
u'\uf273': 'UnicodeCharacter0xf273',
u'\ud5f2': 'HangulSyllableHelp',
u'\u3175': 'HangulLetterPieupsiostikeut',
u'\u10f4': 'GeorgianLetterHar',
u'\uab79': 'UnicodeCharacter0xab79',
u'\u8af8': 'CjkUnifiedIdeograph8af8',
u'\u607f': 'CjkUnifiedIdeograph607f',
u'\u43fe': 'CjkUnifiedIdeograph43fe',
u'\u3606': 'CjkUnifiedIdeograph3606',
u'\u1289': 'UnicodeCharacter0x1289',
u'\u7308': 'CjkUnifiedIdeograph7308',
u'\u4f8b': 'CjkUnifiedIdeograph4f8b',
u'\ub00a': 'HangulSyllableGgwep',
u'\u8c8d': 'CjkUnifiedIdeograph8c8d',
u'\ued0c': 'UnicodeCharacter0xed0c',
u'\uc98f': 'HangulSyllableJeud',
u'\u1e16': 'LatinCapitalLetterEWithMacronAndAcute',
u'\u5b18': 'CjkUnifiedIdeograph5b18',
u'\u379b': 'CjkUnifiedIdeograph379b',
u'\u981a': 'CjkUnifiedIdeograph981a',
u'\u749d': 'CjkUnifiedIdeograph749d',
u'\ud51c': 'HangulSyllablePeuk',
u'\ub19f': 'HangulSyllableNwalb',
u'\ueea1': 'UnicodeCharacter0xeea1',
u'\u0626': 'ArabicLetterYehWithHamzaAbove',
u'\u4328': 'CjkUnifiedIdeograph4328',
u'\u1fab': 'GreekCapitalLetterOmegaWithDasiaAndVariaAndProsgegrammeni',
u'\u802a': 'CjkUnifiedIdeograph802a',
u'\u5cad': 'CjkUnifiedIdeograph5cad',
u'\ubd2c': 'HangulSyllableBwae',
u'\u99af': 'CjkUnifiedIdeograph99af',
u'\ufa2e': 'UnicodeCharacter0xfa2e',
u'\ud6b1': 'HangulSyllableHyolg',
u'\u2b38': 'LeftwardsArrowWithDottedStem',
u'\u07bb': 'UnicodeCharacter0x7bb',
u'\u683a': 'CjkUnifiedIdeograph683a',
u'\u44bd': 'CjkUnifiedIdeograph44bd',
u'\ua53c': 'VaiSyllableShi',
u'\u81bf': 'CjkUnifiedIdeograph81bf',
u'\ue23e': 'UnicodeCharacter0xe23e',
u'\ubec1': 'HangulSyllableBbyaelt',
u'\ufbc3': 'UnicodeCharacter0xfbc3',
u'\u1348': 'EthiopicSyllableFa',
u'\u504a': 'CjkUnifiedIdeograph504a',
u'\u2ccd': 'CopticSmallLetterOldCopticHori',
u'\u8d4c': 'CjkUnifiedIdeograph8d4c',
u'\u69cf': 'CjkUnifiedIdeograph69cf',
u'\uca4e': 'HangulSyllableJjeogg',
u'\ua6d1': 'UnicodeCharacter0xa6d1',
u'\ue3d3': 'UnicodeCharacter0xe3d3',
u'\u385a': 'CjkUnifiedIdeograph385a',
u'\u14dd': 'CanadianSyllabicsWestcreeLwe',
u'\u755c': 'CjkUnifiedIdeograph755c',
u'\u51df': 'CjkUnifiedIdeograph51df',
u'\ub25e': 'HangulSyllableNwinh',
u'\u8ee1': 'CjkUnifiedIdeograph8ee1',
u'\uef60': 'UnicodeCharacter0xef60',
u'\ucbe3': 'HangulSyllableJjeulh',
u'\u206a': 'InhibitSymmetricSwapping',
u'\u5d6c': 'CjkUnifiedIdeograph5d6c',
u'\u39ef': 'CjkUnifiedIdeograph39ef',
u'\u9a6e': 'CjkUnifiedIdeograph9a6e',
u'\u76f1': 'CjkUnifiedIdeograph76f1',
u'\ud770': 'HangulSyllableHyin',
u'\ub3f3': 'HangulSyllableDwas',
u'\uf0f5': 'UnicodeCharacter0xf0f5',
u'\u087a': 'UnicodeCharacter0x87a',
u'\u457c': 'CjkUnifiedIdeograph457c',
u'\u21ff': 'LeftRightOpenheadedArrow',
u'\u827e': 'CjkUnifiedIdeograph827e',
u'\ubf80': 'HangulSyllableBbwael',
u'\u1b03': 'BalineseSignSurang',
u'\ufc82': 'ArabicLigatureKafWithMeemFinalForm',
u'\u5805': 'CjkUnifiedIdeograph5805',
u'\u9507': 'CjkUnifiedIdeograph9507',
u'\ud209': 'HangulSyllableToeng',
u'\u2d8c': 'EthiopicSyllableDoa',
u'\u6a8e': 'CjkUnifiedIdeograph6a8e',
u'\ua790': 'UnicodeCharacter0xa790',
u'\u0313': 'CombiningCommaAbove',
u'\ue492': 'UnicodeCharacter0xe492',
u'\u4015': 'CjkUnifiedIdeograph4015',
u'\u7d17': 'CjkUnifiedIdeograph7d17',
u'\uba19': 'HangulSyllableMyat',
u'\uf71b': 'UnicodeCharacter0xf71b',
u'\u159c': 'CanadianSyllabicsWoodscreeThi',
u'\u529e': 'CjkUnifiedIdeograph529e',
u'\u8fa0': 'CjkUnifiedIdeograph8fa0',
u'\ucca2': 'HangulSyllableCeolm',
u'\u2825': 'BraillePatternDots136',
u'\u6527': 'CjkUnifiedIdeograph6527',
u'\ua229': 'YiSyllableGgep',
u'\udf2b': 'UnicodeCharacter0xdf2b',
u'\u3aae': 'CjkUnifiedIdeograph3aae',
u'\u77b0': 'CjkUnifiedIdeograph77b0',
u'\ub4b2': 'HangulSyllableDwilp',
u'\u1035': 'MyanmarVowelSignEAbove',
u'\uf1b4': 'UnicodeCharacter0xf1b4',
u'\u4d37': 'CjkUnifiedIdeograph4d37',
u'\u8a39': 'CjkUnifiedIdeograph8a39',
u'\uc73b': 'HangulSyllableYuh',
u'\u22be': 'RightAngleWithArc',
u'\u5fc0': 'CjkUnifiedIdeograph5fc0',
u'\u9cc2': 'CjkUnifiedIdeograph9cc2',
u'\ud9c4': 'UnicodeCharacter0xd9c4',
u'\u3547': 'CjkUnifiedIdeograph3547',
u'\u7249': 'CjkUnifiedIdeograph7249',
u'\uaf4b': 'HangulSyllableGgwags',
u'\uec4d': 'UnicodeCharacter0xec4d',
u'\u0ace': 'UnicodeCharacter0xace',
u'\u47d0': 'CjkUnifiedIdeograph47d0',
u'\u84d2': 'CjkUnifiedIdeograph84d2',
u'\uc1d4': 'HangulSyllableSwaem',
u'\u1d57': 'ModifierLetterSmallT',
u'\ufed6': 'ArabicLetterQafFinalForm',
u'\u5a59': 'CjkUnifiedIdeograph5a59',
u'\u975b': 'CjkUnifiedIdeograph975b',
u'\ud45d': 'HangulSyllablePyog',
u'\u2fe0': 'UnicodeCharacter0x2fe0',
u'\u6ce2': 'CjkUnifiedIdeograph6ce2',
u'\ua9e4': 'UnicodeCharacter0xa9e4',
u'\u0567': 'ArmenianSmallLetterEh',
u'\ue6e6': 'UnicodeCharacter0xe6e6',
u'\u4269': 'CjkUnifiedIdeograph4269',
u'\u7f6b': 'CjkUnifiedIdeograph7f6b',
u'\ubc6d': 'HangulSyllableByaenj',
u'\uf96f': 'CjkCompatibilityIdeographf96f',
u'\u17f0': 'KhmerSymbolLekAttakSon',
u'\u54f2': 'CjkUnifiedIdeograph54f2',
u'\u91f4': 'CjkUnifiedIdeograph91f4',
u'\ucef6': 'HangulSyllableKeobs',
u'\u2a79': 'LessthanWithCircleInside',
u'\u677b': 'CjkUnifiedIdeograph677b',
u'\ua47d': 'YiSyllableYot',
u'\ue17f': 'UnicodeCharacter0xe17f'}

FromName = {
'ArabicLigatureSheenWithHahInitialForm': '\xef\xb4\xae',
'HangulSyllableDyinj': '\xeb\x93\xbd',
'CjkUnifiedIdeograph7f64': '\xe7\xbd\xa4',
'UnicodeCharacter0xe7f2': '\xee\x9f\xb2',
'CjkUnifiedIdeograph9628': '\xe9\x98\xa8',
'UnicodeCharacter0xe7f3': '\xee\x9f\xb3',
'LatinLetterSmallCapitalInvertedR': '\xca\x81',
'UnicodeCharacter0xe7f0': '\xee\x9f\xb0',
'HangulSyllableBbel': '\xeb\xbb\xb4',
'UnicodeCharacter0xe7f1': '\xee\x9f\xb1',
'HangulLetterPieuptikeut': '\xe3\x85\xb3',
'UnicodeCharacter0xf38c': '\xef\x8e\x8c',
'CjkUnifiedIdeograph48e1': '\xe4\xa3\xa1',
'HangulSyllableSyeonj': '\xec\x85\x99',
'BoxDrawingsLightVertical': '\xe2\x94\x82',
'SuperscriptZero': '\xe2\x81\xb0',
'BalineseMusicalSymbolDangGede': '\xe1\xad\xaa',
'HangulSyllableSyeong': '\xec\x85\xa9',
'HangulSyllableBbem': '\xeb\xbb\xbc',
'CjkUnifiedIdeograph86e3': '\xe8\x9b\xa3',
'CjkUnifiedIdeograph818f': '\xe8\x86\x8f',
'YiSyllableSur': '\xea\x8c\xa3',
'HangulSyllableJeop': '\xec\xa0\x9a',
'YiSyllableSup': '\xea\x8c\xa1',
'UnicodeCharacter0xe469': '\xee\x91\xa9',
'YiSyllableSut': '\xea\x8c\x9e',
'YiSyllableSux': '\xea\x8c\x9f',
'GimelSymbol': '\xe2\x84\xb7',
'KangxiRadicalCave': '\xe2\xbd\xb3',
'HangulSyllableMinh': '\xeb\xaf\xbe',
'CjkUnifiedIdeograph86e2': '\xe8\x9b\xa2',
'HangulSyllableMing': '\xeb\xb0\x8d',
'UnicodeCharacter0xf3ae': '\xef\x8e\xae',
'YiSyllableSuo': '\xea\x8c\x95',
'NkoCombiningLongLowTone': '\xdf\xb0',
'UnicodeCharacter0xe468': '\xee\x91\xa8',
'UnicodeCharacter0xebe8': '\xee\xaf\xa8',
'UnicodeCharacter0xebe9': '\xee\xaf\xa9',
'UnicodeCharacter0xf5f3': '\xef\x97\xb3',
'KangxiRadicalNose': '\xe2\xbf\x90',
'UnicodeCharacter0xebe6': '\xee\xaf\xa6',
'UnicodeCharacter0xe17f': '\xee\x85\xbf',
'ArabicLetterLamWithThreeDotsAbove': '\xda\xb7',
'UnicodeCharacter0xe17e': '\xee\x85\xbe',
'ArmenianSmallLigatureMenXeh': '\xef\xac\x97',
'CjkUnifiedIdeograph35b0': '\xe3\x96\xb0',
'CjkUnifiedIdeograph35b1': '\xe3\x96\xb1',
'CjkUnifiedIdeograph35b2': '\xe3\x96\xb2',
'CjkUnifiedIdeograph35b3': '\xe3\x96\xb3',
'CjkUnifiedIdeograph35b4': '\xe3\x96\xb4',
'CjkUnifiedIdeograph35b5': '\xe3\x96\xb5',
'TeluguLetterLla': '\xe0\xb0\xb3',
'CjkUnifiedIdeograph35b7': '\xe3\x96\xb7',
'CjkUnifiedIdeograph35b8': '\xe3\x96\xb8',
'CjkUnifiedIdeograph35b9': '\xe3\x96\xb9',
'UnicodeCharacter0xe542': '\xee\x95\x82',
'UnicodeCharacter0xe17b': '\xee\x85\xbb',
'CjkUnifiedIdeograph5962': '\xe5\xa5\xa2',
'CjkUnifiedIdeograph5963': '\xe5\xa5\xa3',
'CjkUnifiedIdeograph5960': '\xe5\xa5\xa0',
'CjkUnifiedIdeograph5961': '\xe5\xa5\xa1',
'CjkUnifiedIdeograph5966': '\xe5\xa5\xa6',
'CjkUnifiedIdeograph5967': '\xe5\xa5\xa7',
'CjkUnifiedIdeograph5964': '\xe5\xa5\xa4',
'CjkUnifiedIdeograph5965': '\xe5\xa5\xa5',
'UnicodeCharacter0xeefd': '\xee\xbb\xbd',
'UnicodeCharacter0xeefe': '\xee\xbb\xbe',
'CjkUnifiedIdeograph5968': '\xe5\xa5\xa8',
'CjkUnifiedIdeograph5969': '\xe5\xa5\xa9',
'UnicodeCharacter0xeefa': '\xee\xbb\xba',
'UnicodeCharacter0xeefb': '\xee\xbb\xbb',
'HebrewPointMeteg': '\xd6\xbd',
'HangulJongseongPansios': '\xe1\x87\xab',
'BalineseVowelSignUluSari': '\xe1\xac\xb7',
'UnicodeCharacter0xf7b6': '\xef\x9e\xb6',
'UnicodeCharacter0xf38f': '\xef\x8e\x8f',
'TibetanVowelSignAa': '\xe0\xbd\xb1',
'UnicodeCharacter0x1759': '\xe1\x9d\x99',
'CjkUnifiedIdeograph596b': '\xe5\xa5\xab',
'CjkUnifiedIdeograph596c': '\xe5\xa5\xac',
'CjkUnifiedIdeograph596a': '\xe5\xa5\xaa',
'CjkUnifiedIdeograph596f': '\xe5\xa5\xaf',
'UnicodeCharacter0xeef9': '\xee\xbb\xb9',
'CjkUnifiedIdeograph596d': '\xe5\xa5\xad',
'CjkUnifiedIdeograph596e': '\xe5\xa5\xae',
'UnicodeCharacter0xeef4': '\xee\xbb\xb4',
'DingbatNegativeCircledDigitThree': '\xe2\x9d\xb8',
'UnicodeCharacter0xe540': '\xee\x95\x80',
'UnicodeCharacter0xeef7': '\xee\xbb\xb7',
'UnicodeCharacter0xeef0': '\xee\xbb\xb0',
'UnicodeCharacter0xeef1': '\xee\xbb\xb1',
'SquareOriginalOfOrEqualTo': '\xe2\x8a\x92',
'UnicodeCharacter0xeef3': '\xee\xbb\xb3',
'CjkUnifiedIdeograph35ba': '\xe3\x96\xba',
'CjkUnifiedIdeograph35bb': '\xe3\x96\xbb',
'CjkUnifiedIdeograph35bc': '\xe3\x96\xbc',
'CjkUnifiedIdeograph35bd': '\xe3\x96\xbd',
'CjkUnifiedIdeograph35be': '\xe3\x96\xbe',
'CjkUnifiedIdeograph35bf': '\xe3\x96\xbf',
'CjkUnifiedIdeograph8f3e': '\xe8\xbc\xbe',
'CjkUnifiedIdeograph716c': '\xe7\x85\xac',
'YiSyllableNdat': '\xea\x85\x97',
'CjkUnifiedIdeograph8f3d': '\xe8\xbc\xbd',
'UnicodeCharacter0xe541': '\xee\x95\x81',
'YiSyllableNdax': '\xea\x85\x98',
'QuarterNote': '\xe2\x99\xa9',
'CjkStrokeSwg': '\xe3\x87\x9f',
'HangulSyllableSweonj': '\xec\x88\xb9',
'CjkStrokeSwz': '\xe3\x87\x98',
'UnicodeCharacter0x11a6': '\xe1\x86\xa6',
'CanadianSyllabicsThcreeThaa': '\xe1\x96\xad',
'HangulSyllableSweong': '\xec\x89\x89',
'UnicodeCharacter0xaad9': '\xea\xab\x99',
'CircledHangulSios': '\xe3\x89\xa6',
'CjkUnifiedIdeograph8f3c': '\xe8\xbc\xbc',
'UnicodeCharacter0xe546': '\xee\x95\x86',
'CopticSmallLetterOldCopticGangia': '\xe2\xb3\x97',
'HangulSyllableBbed': '\xeb\xbb\xb3',
'HangulSyllableKyilt': '\xed\x82\x95',
'HangulSyllableKyils': '\xed\x82\x94',
'BalineseVowelSignUlu': '\xe1\xac\xb6',
'HangulSyllableKyilp': '\xed\x82\x96',
'UnicodeCharacter0xd87c': '\xed\xa1\xbc',
'HangulSyllableKyilm': '\xed\x82\x92',
'HangulSyllableKyilh': '\xed\x82\x97',
'HangulSyllableKyilg': '\xed\x82\x91',
'ArabicLetterBeehIsolatedForm': '\xef\xad\x92',
'HangulSyllableKyilb': '\xed\x82\x93',
'UnicodeCharacter0xeac0': '\xee\xab\x80',
'UnicodeCharacter0xe547': '\xee\x95\x87',
'BottomParenthesis': '\xe2\x8f\x9d',
'HangulSyllableJeoh': '\xec\xa0\x9b',
'h': '\xce\xb7',
'ModifierLetterSmallW': '\xca\xb7',
'UnicodeCharacter0xe9f8': '\xee\xa7\xb8',
'UnicodeCharacter0xf7b1': '\xef\x9e\xb1',
'HangulSyllableCyaegg': '\xec\xb1\xbe',
'CjkUnifiedIdeograph8bcc': '\xe8\xaf\x8c',
'LatinCapitalLetterEWithCircumflexAndHookAbove': '\xe1\xbb\x82',
'ArabicSmallHighSeen': '\xdb\x9c',
'ArabicLigatureTehWithMeemInitialForm': '\xef\xb2\xa4',
'TagalogLetterMa': '\xe1\x9c\x8b',
'HangulSyllableHyaj': '\xed\x96\xa6',
'HangulSyllableHyah': '\xed\x96\xab',
'HangulSyllableHyan': '\xed\x96\x94',
'HangulSyllableHyam': '\xed\x96\xa0',
'HangulSyllableHyal': '\xed\x96\x98',
'HangulSyllableHyac': '\xed\x96\xa7',
'HangulSyllableHyab': '\xed\x96\xa1',
'HangulSyllableGwaelb': '\xea\xb4\xa3',
'HangulSyllableHyae': '\xed\x96\xac',
'CyrillicSmallLetterDjerv': '\xea\x99\x89',
'ChamLetterDa': '\xea\xa8\x95',
'SymbolForNull': '\xe2\x90\x80',
'HangulSyllableHyas': '\xed\x96\xa3',
'HangulSyllableHyap': '\xed\x96\xaa',
'UnicodeCharacter0xeac5': '\xee\xab\x85',
'NewTaiLueLetterHighSa': '\xe1\xa6\x89',
'CjkUnifiedIdeograph6bea': '\xe6\xaf\xaa',
'CjkUnifiedIdeograph86e8': '\xe8\x9b\xa8',
'HangulSyllableDdyuh': '\xeb\x9c\xa7',
'UnicodeCharacter0xe4ef': '\xee\x93\xaf',
'CjkRadicalEye': '\xe2\xba\xab',
'CjkUnifiedIdeograph632e': '\xe6\x8c\xae',
'CjkUnifiedIdeograph632d': '\xe6\x8c\xad',
'CjkUnifiedIdeograph951e': '\xe9\x94\x9e',
'CjkUnifiedIdeograph632f': '\xe6\x8c\xaf',
'CjkUnifiedIdeograph632a': '\xe6\x8c\xaa',
'CjkUnifiedIdeograph951b': '\xe9\x94\x9b',
'CjkUnifiedIdeograph632c': '\xe6\x8c\xac',
'CjkUnifiedIdeograph632b': '\xe6\x8c\xab',
'UnicodeCharacter0xeac4': '\xee\xab\x84',
'KangxiRadicalAnd': '\xe2\xbd\xbd',
'ArabicLigatureSeenWithRehFinalForm': '\xef\xb4\xaa',
'HangulSyllableJjwig': '\xec\xae\x9d',
'HangulSyllableJjwid': '\xec\xae\xa3',
'HangulSyllableJjwib': '\xec\xae\xad',
'HangulSyllableJjwic': '\xec\xae\xb3',
'HangulSyllableJjwin': '\xec\xae\xa0',
'HangulSyllableJeod': '\xec\xa0\x87',
'HangulSyllableJjwim': '\xec\xae\xac',
'HangulSyllableJjwij': '\xec\xae\xb2',
'HangulSyllableJjwik': '\xec\xae\xb4',
'HangulSyllableJjwih': '\xec\xae\xb7',
'YiRadicalShat': '\xea\x93\x80',
'SquareRightOpenBoxOperator': '\xe2\xab\x8e',
'CjkUnifiedIdeograph6098': '\xe6\x82\x98',
'HangulSyllableDyabs': '\xeb\x8c\xae',
'HangulSyllableJjwip': '\xec\xae\xb6',
'CjkUnifiedIdeograph6093': '\xe6\x82\x93',
'CjkUnifiedIdeograph6092': '\xe6\x82\x92',
'CjkUnifiedIdeograph6091': '\xe6\x82\x91',
'CjkUnifiedIdeograph6090': '\xe6\x82\x90',
'CjkUnifiedIdeograph6097': '\xe6\x82\x97',
'CjkUnifiedIdeograph6096': '\xe6\x82\x96',
'CjkUnifiedIdeograph6095': '\xe6\x82\x95',
'CjkUnifiedIdeograph6094': '\xe6\x82\x94',
'CjkUnifiedIdeograph6325': '\xe6\x8c\xa5',
'CjkUnifiedIdeograph6324': '\xe6\x8c\xa4',
'CjkUnifiedIdeograph6327': '\xe6\x8c\xa7',
'CjkUnifiedIdeograph6326': '\xe6\x8c\xa6',
'CjkUnifiedIdeograph6321': '\xe6\x8c\xa1',
'CjkUnifiedIdeograph6320': '\xe6\x8c\xa0',
'CjkUnifiedIdeograph6323': '\xe6\x8c\xa3',
'CjkUnifiedIdeograph6322': '\xe6\x8c\xa2',
'HangulSyllableGgye': '\xea\xbc\x90',
'DigramForLesserYin': '\xe2\x9a\x8d',
'CjkUnifiedIdeograph6329': '\xe6\x8c\xa9',
'CjkUnifiedIdeograph6328': '\xe6\x8c\xa8',
'CjkUnifiedIdeograph9519': '\xe9\x94\x99',
'CjkUnifiedIdeograph9518': '\xe9\x94\x98',
'HangulSyllableBbyil': '\xec\x81\xbc',
'UnicodeCharacter0xaad0': '\xea\xab\x90',
'HangulSyllableBbyin': '\xec\x81\xb8',
'HangulSyllableBbyih': '\xec\x82\x8f',
'HangulSyllableBbyij': '\xec\x82\x8a',
'HangulSyllableBbyik': '\xec\x82\x8c',
'HangulSyllableBbyid': '\xec\x81\xbb',
'HangulSyllableBbyig': '\xec\x81\xb5',
'CjkUnifiedIdeograph7f6e': '\xe7\xbd\xae',
'HangulSyllableBbyib': '\xec\x82\x85',
'HangulSyllableBbyic': '\xec\x82\x8b',
'GreekSmallLetterSho': '\xcf\xb8',
'ModifierLetterSmallBeta': '\xe1\xb5\x9d',
'UnicodeCharacter0xe7fe': '\xee\x9f\xbe',
'HangulSyllableJeob': '\xec\xa0\x91',
'CjkUnifiedIdeograph609c': '\xe6\x82\x9c',
'CjkUnifiedIdeograph609b': '\xe6\x82\x9b',
'CjkUnifiedIdeograph609a': '\xe6\x82\x9a',
'GreekCapitalLetterTau': '\xce\xa4',
'CjkUnifiedIdeograph609f': '\xe6\x82\x9f',
'CjkUnifiedIdeograph609e': '\xe6\x82\x9e',
'CjkUnifiedIdeograph609d': '\xe6\x82\x9d',
'YiSyllableHxiet': '\xea\x89\x8a',
'UnicodeCharacter0xeac9': '\xee\xab\x89',
'ArabicindicPerMilleSign': '\xd8\x89',
'TamilCreditSign': '\xe0\xaf\xb7',
'lambda': '\xce\xbb',
'UpwardsArrowWithTipRightwards': '\xe2\x86\xb1',
'YiSyllableHxiex': '\xea\x89\x8b',
'HangulSyllablePyac': '\xed\x8d\x9b',
'HangulSyllablePyab': '\xed\x8d\x95',
'HangulSyllablePyag': '\xed\x8d\x85',
'HangulSyllablePyae': '\xed\x8d\xa0',
'HangulSyllablePyad': '\xed\x8d\x8b',
'HangulSyllablePyak': '\xed\x8d\x9c',
'CjkUnifiedIdeograph6aef': '\xe6\xab\xaf',
'HangulSyllableMinj': '\xeb\xaf\xbd',
'UnicodeCharacter0xaad6': '\xea\xab\x96',
'HangulSyllablePyam': '\xed\x8d\x94',
'HangulSyllablePyal': '\xed\x8d\x8c',
'HangulSyllablePyas': '\xed\x8d\x97',
'UnicodeCharacter0xf3a2': '\xef\x8e\xa2',
'CjkUnifiedIdeograph84a2': '\xe8\x92\xa2',
'HangulSyllablePyat': '\xed\x8d\x9d',
'CjkUnifiedIdeograph42ac': '\xe4\x8a\xac',
'CjkUnifiedIdeograph42ab': '\xe4\x8a\xab',
'CjkUnifiedIdeograph42aa': '\xe4\x8a\xaa',
'CyrillicSmallLetterKomiTje': '\xd4\x8f',
'CjkUnifiedIdeograph42ae': '\xe4\x8a\xae',
'CjkUnifiedIdeograph42ad': '\xe4\x8a\xad',
'CjkUnifiedIdeograph705d': '\xe7\x81\x9d',
'CjkUnifiedIdeograph6a09': '\xe6\xa8\x89',
'CjkUnifiedIdeograph705f': '\xe7\x81\x9f',
'CjkUnifiedIdeograph705a': '\xe7\x81\x9a',
'CjkUnifiedIdeograph705b': '\xe7\x81\x9b',
'CjkUnifiedIdeograph705c': '\xe7\x81\x9c',
'CjkUnifiedIdeograph7bde': '\xe7\xaf\x9e',
'CjkUnifiedIdeograph7bdd': '\xe7\xaf\x9d',
'HangulSyllableByilt': '\xeb\xb8\xb5',
'CjkUnifiedIdeograph7bdf': '\xe7\xaf\x9f',
'CjkUnifiedIdeograph7bda': '\xe7\xaf\x9a',
'UnicodeCharacter0xdda5': '\xed\xb6\xa5',
'CjkUnifiedIdeograph7bdc': '\xe7\xaf\x9c',
'CjkUnifiedIdeograph7bdb': '\xe7\xaf\x9b',
'CjkUnifiedIdeograph374d': '\xe3\x9d\x8d',
'CjkUnifiedIdeograph374e': '\xe3\x9d\x8e',
'CjkUnifiedIdeograph374f': '\xe3\x9d\x8f',
'CjkUnifiedIdeograph374a': '\xe3\x9d\x8a',
'CjkUnifiedIdeograph374b': '\xe3\x9d\x8b',
'CjkUnifiedIdeograph374c': '\xe3\x9d\x8c',
'HangulSyllableDdyun': '\xeb\x9c\x90',
'UnicodeCharacter0xdda3': '\xed\xb6\xa3',
'UnicodeCharacter0xf3a5': '\xef\x8e\xa5',
'HebrewLetterWideAlef': '\xef\xac\xa1',
'HangulSyllableDyubs': '\xeb\x93\x92',
'CjkUnifiedIdeograph42a9': '\xe4\x8a\xa9',
'CjkUnifiedIdeograph42a8': '\xe4\x8a\xa8',
'CjkUnifiedIdeograph42a3': '\xe4\x8a\xa3',
'CjkUnifiedIdeograph42a2': '\xe4\x8a\xa2',
'CjkUnifiedIdeograph42a1': '\xe4\x8a\xa1',
'CanadianSyllabicsTaai': '\xe1\x91\x8d',
'SundaneseLetterDa': '\xe1\xae\x93',
'CjkUnifiedIdeograph42a6': '\xe4\x8a\xa6',
'CjkUnifiedIdeograph42a5': '\xe4\x8a\xa5',
'CjkUnifiedIdeograph42a4': '\xe4\x8a\xa4',
'UnicodeCharacter0xf8d9': '\xef\xa3\x99',
'BengaliDigitFive': '\xe0\xa7\xab',
'HangulSyllableMyunj': '\xeb\xae\xa9',
'CjkUnifiedIdeograph3744': '\xe3\x9d\x84',
'CjkUnifiedIdeograph3745': '\xe3\x9d\x85',
'CjkUnifiedIdeograph3746': '\xe3\x9d\x86',
'CjkUnifiedIdeograph3747': '\xe3\x9d\x87',
'CjkUnifiedIdeograph3740': '\xe3\x9d\x80',
'CjkUnifiedIdeograph3741': '\xe3\x9d\x81',
'CjkUnifiedIdeograph3742': '\xe3\x9d\x82',
'CjkUnifiedIdeograph3743': '\xe3\x9d\x83',
'HangulSyllableCang': '\xec\xb0\xbd',
'CjkUnifiedIdeograph86d8': '\xe8\x9b\x98',
'HangulSyllableBilh': '\xeb\xb9\x93',
'CjkUnifiedIdeograph3748': '\xe3\x9d\x88',
'CjkUnifiedIdeograph3749': '\xe3\x9d\x89',
'CjkUnifiedIdeograph86d9': '\xe8\x9b\x99',
'CjkUnifiedIdeograph7054': '\xe7\x81\x94',
'CjkUnifiedIdeograph7055': '\xe7\x81\x95',
'CjkUnifiedIdeograph7056': '\xe7\x81\x96',
'CjkUnifiedIdeograph7057': '\xe7\x81\x97',
'CjkUnifiedIdeograph7050': '\xe7\x81\x90',
'CjkUnifiedIdeograph7051': '\xe7\x81\x91',
'CjkUnifiedIdeograph7052': '\xe7\x81\x92',
'CjkUnifiedIdeograph7053': '\xe7\x81\x93',
'CjkUnifiedIdeograph7bd5': '\xe7\xaf\x95',
'CjkUnifiedIdeograph7bd4': '\xe7\xaf\x94',
'CjkUnifiedIdeograph7bd7': '\xe7\xaf\x97',
'CyrillicCapitalLetterByelorussianukrainianI': '\xd0\x86',
'CjkUnifiedIdeograph7058': '\xe7\x81\x98',
'CyrillicSmallLetterUWithDoubleAcute': '\xd3\xb3',
'CjkUnifiedIdeograph7bd3': '\xe7\xaf\x93',
'CjkUnifiedIdeograph7bd2': '\xe7\xaf\x92',
'LatinCapitalLetterCWithAcute': '\xc4\x86',
'UnicodeCharacter0x1c9c': '\xe1\xb2\x9c',
'UnicodeCharacter0x1c9b': '\xe1\xb2\x9b',
'HangulJungseongYuu': '\xe1\x86\x93',
'UnicodeCharacter0x1c9f': '\xe1\xb2\x9f',
'UnicodeCharacter0x1c9e': '\xe1\xb2\x9e',
'UnicodeCharacter0x1c9d': '\xe1\xb2\x9d',
'HangulJongseongKiyeoksios': '\xe1\x86\xaa',
'CjkUnifiedIdeograph651b': '\xe6\x94\x9b',
'HangulSyllableBbo': '\xeb\xbd\x80',
'HangulJungseongYui': '\xe1\x86\x94',
'BraillePatternDots48': '\xe2\xa2\x88',
'BraillePatternDots47': '\xe2\xa1\x88',
'BraillePatternDots46': '\xe2\xa0\xa8',
'BraillePatternDots45': '\xe2\xa0\x98',
'CjkUnifiedIdeograph62bc': '\xe6\x8a\xbc',
'CjkUnifiedIdeograph62bd': '\xe6\x8a\xbd',
'CjkUnifiedIdeograph62be': '\xe6\x8a\xbe',
'HangulJungseongYua': '\xe1\x86\x8e',
'UnicodeCharacter0xe9f4': '\xee\xa7\xb4',
'CjkUnifiedIdeograph651a': '\xe6\x94\x9a',
'UnicodeCharacter0xdf79': '\xed\xbd\xb9',
'UnicodeCharacter0xdf78': '\xed\xbd\xb8',
'UnicodeCharacter0xdf77': '\xed\xbd\xb7',
'CjkUnifiedIdeograph651f': '\xe6\x94\x9f',
'UnicodeCharacter0xdf75': '\xed\xbd\xb5',
'UnicodeCharacter0xdf74': '\xed\xbd\xb4',
'UnicodeCharacter0xdf73': '\xed\xbd\xb3',
'UnicodeCharacter0xdf72': '\xed\xbd\xb2',
'UnicodeCharacter0xdf71': '\xed\xbd\xb1',
'HangulSyllableSaess': '\xec\x83\x9c',
'ArabicLigatureFehWithKhahWithMeemInitialForm': '\xef\xb5\xbd',
'CjkUnifiedIdeograph651d': '\xe6\x94\x9d',
'UnicodeCharacter0xe5de': '\xee\x97\x9e',
'ChamPunctuationSpiral': '\xea\xa9\x9c',
'CjkUnifiedIdeograph5dd8': '\xe5\xb7\x98',
'TibetanSubjoinedLetterGha': '\xe0\xbe\x93',
'UnicodeCharacter0xe9f5': '\xee\xa7\xb5',
'CjkUnifiedIdeograph62b8': '\xe6\x8a\xb8',
'CjkUnifiedIdeograph62b9': '\xe6\x8a\xb9',
'CjkUnifiedIdeograph62b0': '\xe6\x8a\xb0',
'CjkUnifiedIdeograph62b1': '\xe6\x8a\xb1',
'CjkUnifiedIdeograph62b2': '\xe6\x8a\xb2',
'CjkUnifiedIdeograph62b3': '\xe6\x8a\xb3',
'CjkUnifiedIdeograph62b4': '\xe6\x8a\xb4',
'CjkUnifiedIdeograph62b5': '\xe6\x8a\xb5',
'CjkUnifiedIdeograph62b6': '\xe6\x8a\xb6',
'CjkUnifiedIdeograph62b7': '\xe6\x8a\xb7',
'UnicodeCharacter0x1c99': '\xe1\xb2\x99',
'UnicodeCharacter0x1c98': '\xe1\xb2\x98',
'UnicodeCharacter0x1c93': '\xe1\xb2\x93',
'UnicodeCharacter0x1c92': '\xe1\xb2\x92',
'UnicodeCharacter0x1c91': '\xe1\xb2\x91',
'UnicodeCharacter0x1c90': '\xe1\xb2\x90',
'UnicodeCharacter0x1c97': '\xe1\xb2\x97',
'UnicodeCharacter0x1c96': '\xe1\xb2\x96',
'UnicodeCharacter0x1c95': '\xe1\xb2\x95',
'UnicodeCharacter0x1c94': '\xe1\xb2\x94',
'HangulSyllableNyaec': '\xeb\x84\x83',
'HangulSyllableNyaeb': '\xeb\x83\xbd',
'HangulSyllableNyaed': '\xeb\x83\xb3',
'HangulSyllableNyaeg': '\xeb\x83\xad',
'CjkUnifiedIdeograph7e2e': '\xe7\xb8\xae',
'ArabicLigatureTahWithMeemWithHahFinalForm': '\xef\xb5\xb1',
'HangulSyllableNyaeh': '\xeb\x84\x87',
'HangulSyllableNyaek': '\xeb\x84\x84',
'HangulSyllableNyaej': '\xeb\x84\x82',
'HangulSyllableNyaem': '\xeb\x83\xbc',
'HangulSyllableNyael': '\xeb\x83\xb4',
'OghamLetterPeith': '\xe1\x9a\x9a',
'MongolianLetterSibeAng': '\xe1\xa1\xa2',
'HangulSyllableNyaep': '\xeb\x84\x86',
'HangulSyllableNyaes': '\xeb\x83\xbf',
'HangulSyllableNyaet': '\xeb\x84\x85',
'UnicodeCharacter0xf77e': '\xef\x9d\xbe',
'HangulSyllableCyels': '\xec\xb3\xb8',
'UnicodeCharacter0xdf7f': '\xed\xbd\xbf',
'UnicodeCharacter0xdf7e': '\xed\xbd\xbe',
'HangulSyllableCyelp': '\xec\xb3\xba',
'UnicodeCharacter0xdf7c': '\xed\xbd\xbc',
'UnicodeCharacter0xdf7b': '\xed\xbd\xbb',
'UnicodeCharacter0xdf7a': '\xed\xbd\xba',
'HangulSyllableCyelt': '\xec\xb3\xb9',
'CjkUnifiedIdeograph7e2f': '\xe7\xb8\xaf',
'LimbuSubjoinedLetterWa': '\xe1\xa4\xab',
'CjkUnifiedIdeograph7c0a': '\xe7\xb0\x8a',
'CjkUnifiedIdeograph7c0b': '\xe7\xb0\x8b',
'CjkUnifiedIdeograph7c0c': '\xe7\xb0\x8c',
'CjkUnifiedIdeograph7c0d': '\xe7\xb0\x8d',
'CjkUnifiedIdeograph7c0e': '\xe7\xb0\x8e',
'CjkUnifiedIdeograph7c0f': '\xe7\xb0\x8f',
'CombiningFourDotsAbove': '\xe2\x83\x9c',
'UnicodeCharacter0xf53d': '\xef\x94\xbd',
'CjkUnifiedIdeograph3f89': '\xe3\xbe\x89',
'CjkUnifiedIdeograph3f88': '\xe3\xbe\x88',
'HangulSyllableBbuh': '\xeb\xbf\xa7',
'UnicodeCharacter0xf565': '\xef\x95\xa5',
'HangulSyllableBbun': '\xeb\xbf\x90',
'KhmerSignViriam': '\xe1\x9f\x91',
'YiSyllableKo': '\xea\x88\x88',
'HangulSyllableBbum': '\xeb\xbf\x9c',
'LimbuLetterPha': '\xe1\xa4\x91',
'CjkUnifiedIdeograph3f80': '\xe3\xbe\x80',
'TifinaghLetterYakhh': '\xe2\xb4\xbf',
'CjkUnifiedIdeograph3f82': '\xe3\xbe\x82',
'CjkUnifiedIdeograph3f85': '\xe3\xbe\x85',
'CjkUnifiedIdeograph3f84': '\xe3\xbe\x84',
'CjkUnifiedIdeograph3f87': '\xe3\xbe\x87',
'CjkUnifiedIdeograph3f86': '\xe3\xbe\x86',
'MalayalamLetterBha': '\xe0\xb4\xad',
'HangulSyllableBbus': '\xeb\xbf\x9f',
'HangulSyllableBbup': '\xeb\xbf\xa6',
'YiRadicalLyr': '\xea\x92\x9a',
'HangulSyllableBbut': '\xeb\xbf\xa5',
'CjkUnifiedIdeograph7c00': '\xe7\xb0\x80',
'CjkUnifiedIdeograph7c01': '\xe7\xb0\x81',
'CjkUnifiedIdeograph7c02': '\xe7\xb0\x82',
'CjkUnifiedIdeograph7c03': '\xe7\xb0\x83',
'CjkUnifiedIdeograph7c04': '\xe7\xb0\x84',
'UpBarbRightDownBarbRightHarpoon': '\xe2\xa5\x8f',
'RightAngleBracketWithDot': '\xe2\xa6\x92',
'CjkUnifiedIdeograph7c07': '\xe7\xb0\x87',
'CjkUnifiedIdeograph7c08': '\xe7\xb0\x88',
'CjkUnifiedIdeograph7c09': '\xe7\xb0\x89',
'YiSyllableKop': '\xea\x88\x89',
'PartAlternationMark': '\xe3\x80\xbd',
'HalfwidthHangulLetterSsangcieuc': '\xef\xbe\xb9',
'UnicodeCharacter0xe8b2': '\xee\xa2\xb2',
'UnicodeCharacter0xed73': '\xee\xb5\xb3',
'CjkUnifiedIdeograph7e2a': '\xe7\xb8\xaa',
'HangulSyllableBeolh': '\xeb\xb2\x93',
'CjkUnifiedIdeograph58af': '\xe5\xa2\xaf',
'UnicodeCharacter0xfbbb': '\xef\xae\xbb',
'MonogramForYang': '\xe2\x9a\x8a',
'VaiSyllableDhhoo': '\xea\x96\x82',
'CjkUnifiedIdeograph3f8a': '\xe3\xbe\x8a',
'YiSyllableCot': '\xea\x8a\xbd',
'CjkUnifiedIdeograph3f8c': '\xe3\xbe\x8c',
'CjkUnifiedIdeograph3f8b': '\xe3\xbe\x8b',
'ArmenianSmallLetterYiwn': '\xd6\x82',
'CjkUnifiedIdeograph3f8d': '\xe3\xbe\x8d',
'CjkUnifiedIdeograph3f8f': '\xe3\xbe\x8f',
'TamilLetterUu': '\xe0\xae\x8a',
'GreaterthanAboveRightwardsArrow': '\xe2\xa5\xb8',
'GujaratiLetterTa': '\xe0\xaa\xa4',
'cwint': '\xe2\x88\xb1',
'increment': '\xe2\x88\x86',
'UnicodeCharacter0xf87e': '\xef\xa1\xbe',
'CjkUnifiedIdeograph7e2c': '\xe7\xb8\xac',
'UnicodeCharacter0xeeff': '\xee\xbb\xbf',
'BoxDrawingsLeftLightAndRightVerticalHeavy': '\xe2\x95\x8a',
'CopticCapitalLetterOou': '\xe2\xb2\xb0',
'HangulSyllableTong': '\xed\x86\xb5',
'UnicodeCharacter0xadb': '\xe0\xab\x9b',
'UnicodeCharacter0xadc': '\xe0\xab\x9c',
'UnicodeCharacter0xada': '\xe0\xab\x9a',
'UnicodeCharacter0xadf': '\xe0\xab\x9f',
'UnicodeCharacter0xadd': '\xe0\xab\x9d',
'UnicodeCharacter0xade': '\xe0\xab\x9e',
'YiSyllableLiep': '\xea\x86\xbe',
'HangulSyllableDwac': '\xeb\x8f\xb7',
'CjkCompatibilityIdeographfab8': '\xef\xaa\xb8',
'CjkUnifiedIdeograph6513': '\xe6\x94\x93',
'CjkCompatibilityIdeographfab3': '\xef\xaa\xb3',
'CjkCompatibilityIdeographfab2': '\xef\xaa\xb2',
'HangulSyllableDdyigs': '\xeb\x9d\x87',
'HangulSyllableSsiss': '\xec\x94\xbc',
'CjkCompatibilityIdeographfab7': '\xef\xaa\xb7',
'CjkUnifiedIdeograph6510': '\xe6\x94\x90',
'CjkCompatibilityIdeographfab5': '\xef\xaa\xb5',
'CjkCompatibilityIdeographfab4': '\xef\xaa\xb4',
'CjkUnifiedIdeograph6511': '\xe6\x94\x91',
'CjkUnifiedIdeograph6516': '\xe6\x94\x96',
'HangulSyllableCyaenh': '\xec\xb2\x82',
'HangulSyllableBbugg': '\xeb\xbf\x8e',
'CjkUnifiedIdeograph4bdf': '\xe4\xaf\x9f',
'LatinSmallLetterEzhWithTail': '\xc6\xba',
'HangulSyllableCyaeng': '\xec\xb2\x91',
'CjkUnifiedIdeograph6514': '\xe6\x94\x94',
'HangulSyllablePec': '\xed\x8e\xaf',
'MongolianLetterManchuAliGaliLha': '\xe1\xa2\xaa',
'UnicodeCharacter0x2da7': '\xe2\xb6\xa7',
'HangulSyllableBinh': '\xeb\xb9\x8a',
'CjkCompatibilityIdeographfabc': '\xef\xaa\xbc',
'CjkCompatibilityIdeographfabb': '\xef\xaa\xbb',
'CjkCompatibilityIdeographfaba': '\xef\xaa\xba',
'CjkCompatibilityIdeographfabf': '\xef\xaa\xbf',
'CjkCompatibilityIdeographfabe': '\xef\xaa\xbe',
'CjkCompatibilityIdeographfabd': '\xef\xaa\xbd',
'UnicodeCharacter0xad8': '\xe0\xab\x98',
'UnicodeCharacter0xad9': '\xe0\xab\x99',
'HangulJongseongPieuprieul': '\xe1\x87\xa3',
'UnicodeCharacter0xad2': '\xe0\xab\x92',
'UnicodeCharacter0xad3': '\xe0\xab\x93',
'UnicodeCharacter0xad1': '\xe0\xab\x91',
'UnicodeCharacter0xad6': '\xe0\xab\x96',
'UnicodeCharacter0xad7': '\xe0\xab\x97',
'UnicodeCharacter0xad4': '\xe0\xab\x94',
'UnicodeCharacter0xad5': '\xe0\xab\x95',
'CjkUnifiedIdeograph50da': '\xe5\x83\x9a',
'CjkUnifiedIdeograph50dc': '\xe5\x83\x9c',
'CjkUnifiedIdeograph50db': '\xe5\x83\x9b',
'CjkUnifiedIdeograph50de': '\xe5\x83\x9e',
'CjkUnifiedIdeograph50dd': '\xe5\x83\x9d',
'CjkUnifiedIdeograph50df': '\xe5\x83\x9f',
'TaiLeLetterPa': '\xe1\xa5\x99',
'CjkUnifiedIdeograph77f2': '\xe7\x9f\xb2',
'CjkUnifiedIdeograph77f3': '\xe7\x9f\xb3',
'CjkUnifiedIdeograph77f0': '\xe7\x9f\xb0',
'CjkUnifiedIdeograph77f1': '\xe7\x9f\xb1',
'CjkUnifiedIdeograph77f6': '\xe7\x9f\xb6',
'CjkUnifiedIdeograph77f7': '\xe7\x9f\xb7',
'BuhidLetterLa': '\xe1\x9d\x8e',
'CjkUnifiedIdeograph77f5': '\xe7\x9f\xb5',
'CjkUnifiedIdeograph77f8': '\xe7\x9f\xb8',
'CjkUnifiedIdeograph77f9': '\xe7\x9f\xb9',
'UnicodeCharacter0xe3a5': '\xee\x8e\xa5',
'HangulSyllableNunh': '\xeb\x88\x8a',
'UnicodeCharacter0x1fdc': '\xe1\xbf\x9c',
'CjkUnifiedIdeograph39db': '\xe3\xa7\x9b',
'CjkUnifiedIdeograph39dc': '\xe3\xa7\x9c',
'CjkUnifiedIdeograph39da': '\xe3\xa7\x9a',
'CjkUnifiedIdeograph39df': '\xe3\xa7\x9f',
'CjkUnifiedIdeograph39dd': '\xe3\xa7\x9d',
'CjkUnifiedIdeograph39de': '\xe3\xa7\x9e',
'ThaiCharacterSoSua': '\xe0\xb8\xaa',
'SinhalaVowelSignDigaPaapilla': '\xe0\xb7\x96',
'CjkUnifiedIdeograph6ebd': '\xe6\xba\xbd',
'UnicodeCharacter0xe0e9': '\xee\x83\xa9',
'HangulSyllableHwigs': '\xed\x9c\x9b',
'subv': '\xe1\xb5\xa5',
'CjkUnifiedIdeograph77fb': '\xe7\x9f\xbb',
'CjkUnifiedIdeograph77fc': '\xe7\x9f\xbc',
'CjkUnifiedIdeograph77fa': '\xe7\x9f\xba',
'CjkUnifiedIdeograph77ff': '\xe7\x9f\xbf',
'CjkUnifiedIdeograph77fd': '\xe7\x9f\xbd',
'CjkUnifiedIdeograph77fe': '\xe7\x9f\xbe',
'EthiopicSyllableTsu': '\xe1\x8c\xb9',
'CjkUnifiedIdeograph50d1': '\xe5\x83\x91',
'CjkUnifiedIdeograph50d0': '\xe5\x83\x90',
'EthiopicSyllableTsi': '\xe1\x8c\xba',
'CjkUnifiedIdeograph50d2': '\xe5\x83\x92',
'EthiopicSyllableTso': '\xe1\x8c\xbe',
'CjkUnifiedIdeograph50d4': '\xe5\x83\x94',
'CjkUnifiedIdeograph50d7': '\xe5\x83\x97',
'CjkUnifiedIdeograph50d6': '\xe5\x83\x96',
'CjkUnifiedIdeograph50d9': '\xe5\x83\x99',
'CjkUnifiedIdeograph50d8': '\xe5\x83\x98',
'EthiopicSyllableTsa': '\xe1\x8c\xb8',
'CanadianSyllabicsCarrierKkee': '\xe1\x97\xbe',
'EthiopicSyllableTse': '\xe1\x8c\xbd',
'GreekSmallLetterEtaWithDasiaAndPerispomeniAndYpogegrammeni': '\xe1\xbe\x97',
'HangulSyllableSyigs': '\xec\x8b\x83',
'SubsetOfOrEqualTo': '\xe2\x8a\x86',
'DottedSquare': '\xe2\xac\x9a',
'RightCurlyBracketMiddlePiece': '\xe2\x8e\xac',
'UnicodeCharacter0x1fd5': '\xe1\xbf\x95',
'UnicodeCharacter0x1fd4': '\xe1\xbf\x94',
'CombiningCyrillicLetterZhe': '\xe2\xb7\xa4',
'CjkUnifiedIdeograph39d9': '\xe3\xa7\x99',
'MultiplicationX': '\xe2\x9c\x95',
'CjkUnifiedIdeograph39d2': '\xe3\xa7\x92',
'CjkUnifiedIdeograph39d3': '\xe3\xa7\x93',
'CjkUnifiedIdeograph39d0': '\xe3\xa7\x90',
'CjkUnifiedIdeograph39d1': '\xe3\xa7\x91',
'CjkUnifiedIdeograph39d6': '\xe3\xa7\x96',
'CjkUnifiedIdeograph39d7': '\xe3\xa7\x97',
'CjkUnifiedIdeograph39d4': '\xe3\xa7\x94',
'CjkUnifiedIdeograph39d5': '\xe3\xa7\x95',
'ZNotationSchemaComposition': '\xe2\xa8\x9f',
'UnicodeCharacter0xa968': '\xea\xa5\xa8',
'UnicodeCharacter0xa969': '\xea\xa5\xa9',
'UnicodeCharacter0xa964': '\xea\xa5\xa4',
'UnicodeCharacter0xa965': '\xea\xa5\xa5',
'UnicodeCharacter0xa966': '\xea\xa5\xa6',
'UnicodeCharacter0xa967': '\xea\xa5\xa7',
'UnicodeCharacter0xa960': '\xea\xa5\xa0',
'UnicodeCharacter0xa961': '\xea\xa5\xa1',
'UnicodeCharacter0xa962': '\xea\xa5\xa2',
'UnicodeCharacter0xa963': '\xea\xa5\xa3',
'LeftRightDoubleArrowWithVerticalStroke': '\xe2\xa4\x84',
'UnicodeCharacter0xe098': '\xee\x82\x98',
'UnicodeCharacter0xe099': '\xee\x82\x99',
'CjkUnifiedIdeograph9585': '\xe9\x96\x85',
'UnicodeCharacter0xe094': '\xee\x82\x94',
'UnicodeCharacter0xe095': '\xee\x82\x95',
'UnicodeCharacter0xe096': '\xee\x82\x96',
'UnicodeCharacter0xe097': '\xee\x82\x97',
'UnicodeCharacter0xe090': '\xee\x82\x90',
'CjkUnifiedIdeograph735e': '\xe7\x8d\x9e',
'UnicodeCharacter0xe092': '\xee\x82\x92',
'UnicodeCharacter0xe093': '\xee\x82\x93',
'CjkUnifiedIdeograph5b2a': '\xe5\xac\xaa',
'CjkUnifiedIdeograph5b2c': '\xe5\xac\xac',
'CjkUnifiedIdeograph5b2b': '\xe5\xac\xab',
'CjkUnifiedIdeograph5b2e': '\xe5\xac\xae',
'CjkUnifiedIdeograph5b2d': '\xe5\xac\xad',
'CjkUnifiedIdeograph5b2f': '\xe5\xac\xaf',
'CjkUnifiedIdeograph9580': '\xe9\x96\x80',
'HangulSyllableEols': '\xec\x97\x80',
'HangulSyllableSsoegg': '\xec\x90\xae',
'SmallRomanNumeralEight': '\xe2\x85\xb7',
'CjkUnifiedIdeograph9581': '\xe9\x96\x81',
'CjkUnifiedIdeograph5a97': '\xe5\xaa\x97',
'CjkUnifiedIdeograph5a96': '\xe5\xaa\x96',
'CjkUnifiedIdeograph5a95': '\xe5\xaa\x95',
'CjkUnifiedIdeograph5a94': '\xe5\xaa\x94',
'CjkUnifiedIdeograph5a93': '\xe5\xaa\x93',
'CjkUnifiedIdeograph5a92': '\xe5\xaa\x92',
'CjkUnifiedIdeograph5a91': '\xe5\xaa\x91',
'CjkUnifiedIdeograph5a90': '\xe5\xaa\x90',
'=>': '\xe2\x87\x92',
'HangulSyllableEolb': '\xec\x96\xbf',
'UnicodeCharacter0xeef6': '\xee\xbb\xb6',
'CjkUnifiedIdeograph5a99': '\xe5\xaa\x99',
'CjkUnifiedIdeograph5a98': '\xe5\xaa\x98',
'CombiningCyrillicLetterLittleYus': '\xe2\xb7\xbd',
'BraillePatternDots467': '\xe2\xa1\xa8',
'CjkUnifiedIdeograph93a3': '\xe9\x8e\xa3',
'UnicodeCharacter0xe09e': '\xee\x82\x9e',
'BraillePatternDots468': '\xe2\xa2\xa8',
'GujaratiLetterBha': '\xe0\xaa\xad',
'UnicodeCharacter0xe09b': '\xee\x82\x9b',
'UnicodeCharacter0xe09c': '\xee\x82\x9c',
'UnicodeCharacter0xeef2': '\xee\xbb\xb2',
'UnicodeCharacter0xa96d': '\xea\xa5\xad',
'UnicodeCharacter0xa96e': '\xea\xa5\xae',
'UnicodeCharacter0xa96f': '\xea\xa5\xaf',
'UnicodeCharacter0xa96a': '\xea\xa5\xaa',
'ModifierLetterSmallScriptG': '\xe1\xb6\xa2',
'UnicodeCharacter0xa96c': '\xea\xa5\xac',
'TibetanSymbolPhurPa': '\xe0\xbf\x88',
'CjkUnifiedIdeograph5a9f': '\xe5\xaa\x9f',
'CjkUnifiedIdeograph5a9e': '\xe5\xaa\x9e',
'CjkUnifiedIdeograph5a9d': '\xe5\xaa\x9d',
'BengaliSignCandrabindu': '\xe0\xa6\x81',
'CjkUnifiedIdeograph5a9b': '\xe5\xaa\x9b',
'CjkUnifiedIdeograph5a9a': '\xe5\xaa\x9a',
'UnicodeCharacter0xf614': '\xef\x98\x94',
'UnicodeCharacter0xf619': '\xef\x98\x99',
'RightwardsTripleArrow': '\xe2\x87\x9b',
'HangulSyllableRwe': '\xeb\xa4\xa0',
'CjkUnifiedIdeograph5b21': '\xe5\xac\xa1',
'CjkUnifiedIdeograph5b20': '\xe5\xac\xa0',
'CjkUnifiedIdeograph5b23': '\xe5\xac\xa3',
'CjkUnifiedIdeograph5b22': '\xe5\xac\xa2',
'CjkUnifiedIdeograph5b25': '\xe5\xac\xa5',
'CjkUnifiedIdeograph5b24': '\xe5\xac\xa4',
'CjkUnifiedIdeograph5b27': '\xe5\xac\xa7',
'CjkUnifiedIdeograph5b26': '\xe5\xac\xa6',
'CjkUnifiedIdeograph5b29': '\xe5\xac\xa9',
'CjkUnifiedIdeograph5b28': '\xe5\xac\xa8',
'LatinSmallLetterNWithRetroflexHook': '\xc9\xb3',
'HangulSyllableBweoss': '\xeb\xb6\xb0',
'DigitFiveFullStop': '\xe2\x92\x8c',
'CjkUnifiedIdeograph8f6d': '\xe8\xbd\xad',
'LatinCapitalLetterOWithDotAbove': '\xc8\xae',
'CjkUnifiedIdeograph3eea': '\xe3\xbb\xaa',
'UnicodeCharacter0xed76': '\xee\xb5\xb6',
'CjkUnifiedIdeograph3eec': '\xe3\xbb\xac',
'CjkUnifiedIdeograph3eeb': '\xe3\xbb\xab',
'CjkUnifiedIdeograph3eee': '\xe3\xbb\xae',
'CjkUnifiedIdeograph3eed': '\xe3\xbb\xad',
'CjkUnifiedIdeograph3eef': '\xe3\xbb\xaf',
'CjkUnifiedIdeograph452a': '\xe4\x94\xaa',
'UnicodeCharacter0x2e38': '\xe2\xb8\xb8',
'UnicodeCharacter0x2e39': '\xe2\xb8\xb9',
'NaryIntersection': '\xe2\x8b\x82',
'UnicodeCharacter0x2e32': '\xe2\xb8\xb2',
'LepchaLetterDza': '\xe1\xb0\x99',
'UnicodeCharacter0x2e31': '\xe2\xb8\xb1',
'UnicodeCharacter0x2e36': '\xe2\xb8\xb6',
'UnicodeCharacter0x2e37': '\xe2\xb8\xb7',
'UnicodeCharacter0x2e34': '\xe2\xb8\xb4',
'UnicodeCharacter0x2e35': '\xe2\xb8\xb5',
'UnicodeCharacter0xf74b': '\xef\x9d\x8b',
'CjkUnifiedIdeograph3ab7': '\xe3\xaa\xb7',
'CjkUnifiedIdeograph3ab0': '\xe3\xaa\xb0',
'HangulSyllableDdang': '\xeb\x95\x85',
'CjkUnifiedIdeograph3ab1': '\xe3\xaa\xb1',
'CjkUnifiedIdeograph3ee9': '\xe3\xbb\xa9',
'CjkUnifiedIdeograph3ee8': '\xe3\xbb\xa8',
'LessthanOrSlantedEqualToWithDotAbove': '\xe2\xaa\x81',
'YiSyllableNdap': '\xea\x85\x9a',
'CjkUnifiedIdeograph3ee1': '\xe3\xbb\xa1',
'CjkUnifiedIdeograph3ee0': '\xe3\xbb\xa0',
'CjkUnifiedIdeograph3ee3': '\xe3\xbb\xa3',
'CjkUnifiedIdeograph3ee2': '\xe3\xbb\xa2',
'CjkUnifiedIdeograph3ee5': '\xe3\xbb\xa5',
'CjkUnifiedIdeograph3ee4': '\xe3\xbb\xa4',
'CjkUnifiedIdeograph3ee7': '\xe3\xbb\xa7',
'CjkUnifiedIdeograph3ee6': '\xe3\xbb\xa6',
'UnicodeCharacter0xd962': '\xed\xa5\xa2',
'CjkUnifiedIdeograph86a8': '\xe8\x9a\xa8',
'UnicodeCharacter0xe2e6': '\xee\x8b\xa6',
'CjkUnifiedIdeograph7d89': '\xe7\xb6\x89',
'CanadianSyllabicsCarrierTlhee': '\xe1\x98\xb7',
'DarkShade': '\xe2\x96\x93',
'GreekSmallLetterOmegaWithPsiliAndVariaAndYpogegrammeni': '\xe1\xbe\xa2',
'PresentationFormForVerticalLeftBlackLenticularBracket': '\xef\xb8\xbb',
'MyanmarShanDigitEight': '\xe1\x82\x98',
'UnicodeCharacter0xf218': '\xef\x88\x98',
'UnicodeCharacter0xd966': '\xed\xa5\xa6',
'HangulSyllableDdwing': '\xeb\x9c\x85',
'CjkRadicalSimplifiedWheat': '\xe2\xbb\xa8',
'UnicodeCharacter0x2e3b': '\xe2\xb8\xbb',
'UnicodeCharacter0x2e3c': '\xe2\xb8\xbc',
'UnicodeCharacter0xed95': '\xee\xb6\x95',
'UnicodeCharacter0x2e3a': '\xe2\xb8\xba',
'UnicodeCharacter0x2e3f': '\xe2\xb8\xbf',
'MyanmarLetterCha': '\xe1\x80\x86',
'UnicodeCharacter0x2e3d': '\xe2\xb8\xbd',
'UnicodeCharacter0x2e3e': '\xe2\xb8\xbe',
'CjkUnifiedIdeograph4176': '\xe4\x85\xb6',
'CjkUnifiedIdeograph4177': '\xe4\x85\xb7',
'CjkUnifiedIdeograph4174': '\xe4\x85\xb4',
'CjkUnifiedIdeograph4175': '\xe4\x85\xb5',
'CjkUnifiedIdeograph4172': '\xe4\x85\xb2',
'CjkUnifiedIdeograph4173': '\xe4\x85\xb3',
'CjkUnifiedIdeograph4170': '\xe4\x85\xb0',
'CjkUnifiedIdeograph4171': '\xe4\x85\xb1',
'VaiSyllableDi': '\xea\x94\xb9',
'HangulSyllableSsyegg': '\xec\x8e\xbe',
'DotMinus': '\xe2\x88\xb8',
'CjkUnifiedIdeograph4178': '\xe4\x85\xb8',
'CjkUnifiedIdeograph4179': '\xe4\x85\xb9',
'CjkUnifiedIdeograph5cbd': '\xe5\xb2\xbd',
'CjkUnifiedIdeograph5cbe': '\xe5\xb2\xbe',
'CjkUnifiedIdeograph5cbf': '\xe5\xb2\xbf',
'VaiSyllableDu': '\xea\x96\xaa',
'CjkUnifiedIdeograph5cba': '\xe5\xb2\xba',
'CjkUnifiedIdeograph5cbb': '\xe5\xb2\xbb',
'CjkUnifiedIdeograph5cbc': '\xe5\xb2\xbc',
'LatinCapitalLetterUWithDotBelow': '\xe1\xbb\xa4',
'HangulSyllableCeunh': '\xec\xb8\xa6',
'UnicodeCharacter0xdc96': '\xed\xb2\x96',
'LatinCapitalLetterFWithDotAbove': '\xe1\xb8\x9e',
'HangulSyllableSsyegs': '\xec\x8e\xbf',
'ThaanaLetterPaviyani': '\xde\x95',
'HangulSyllableGyiss': '\xea\xb8\xa8',
'CjkUnifiedIdeograph75c9': '\xe7\x97\x89',
'TamilVowelSignU': '\xe0\xaf\x81',
'CjkUnifiedIdeograph75c7': '\xe7\x97\x87',
'GujaratiDigitFour': '\xe0\xab\xaa',
'CjkUnifiedIdeograph75c5': '\xe7\x97\x85',
'CjkUnifiedIdeograph75c4': '\xe7\x97\x84',
'CjkUnifiedIdeograph75c3': '\xe7\x97\x83',
'CjkUnifiedIdeograph75c2': '\xe7\x97\x82',
'CjkUnifiedIdeograph75c1': '\xe7\x97\x81',
'CjkUnifiedIdeograph75c0': '\xe7\x97\x80',
'CjkUnifiedIdeograph958d': '\xe9\x96\x8d',
'HangulSyllableSweonh': '\xec\x88\xba',
'TamilVowelSignE': '\xe0\xaf\x86',
'UnicodeCharacter0xdc97': '\xed\xb2\x97',
'TamilVowelSignI': '\xe0\xae\xbf',
'TamilVowelSignO': '\xe0\xaf\x8a',
'UnicodeCharacter0xed1e': '\xee\xb4\x9e',
'CjkUnifiedIdeograph5cb4': '\xe5\xb2\xb4',
'CjkUnifiedIdeograph5cb5': '\xe5\xb2\xb5',
'CjkUnifiedIdeograph5cb6': '\xe5\xb2\xb6',
'CjkUnifiedIdeograph5cb7': '\xe5\xb2\xb7',
'CjkUnifiedIdeograph5cb0': '\xe5\xb2\xb0',
'CjkUnifiedIdeograph5cb1': '\xe5\xb2\xb1',
'CjkUnifiedIdeograph5cb2': '\xe5\xb2\xb2',
'CjkUnifiedIdeograph5cb3': '\xe5\xb2\xb3',
'CjkUnifiedIdeograph5cb8': '\xe5\xb2\xb8',
'CjkUnifiedIdeograph5cb9': '\xe5\xb2\xb9',
'CjkCompatibilityIdeographf9de': '\xef\xa7\x9e',
'CjkUnifiedIdeograph958a': '\xe9\x96\x8a',
'CjkUnifiedIdeograph417f': '\xe4\x85\xbf',
'CjkUnifiedIdeograph417d': '\xe4\x85\xbd',
'CjkUnifiedIdeograph417e': '\xe4\x85\xbe',
'CjkUnifiedIdeograph417b': '\xe4\x85\xbb',
'CjkUnifiedIdeograph417c': '\xe4\x85\xbc',
'GreekPsiliAndPerispomeni': '\xe1\xbf\x8f',
'CjkUnifiedIdeograph8859': '\xe8\xa1\x99',
'CjkUnifiedIdeograph7350': '\xe7\x8d\x90',
'CjkUnifiedIdeograph8858': '\xe8\xa1\x98',
'CanadianSyllabicsBlackfootKo': '\xe1\x96\xbe',
'TibetanSymbolPadmaGdan': '\xe0\xbf\x86',
'CanadianSyllabicsBlackfootKi': '\xe1\x96\xbd',
'UnicodeCharacter0xe0ea': '\xee\x83\xaa',
'CanadianSyllabicsBlackfootKe': '\xe1\x96\xbc',
'ModifierLetterPlusSign': '\xcb\x96',
'RejangLetterTa': '\xea\xa4\xb3',
'TeluguLetterYa': '\xe0\xb0\xaf',
'HangulSyllableSsyaenj': '\xec\x8d\x91',
'HangulSyllableSsyaenh': '\xec\x8d\x92',
'HangulSyllableSsyaeng': '\xec\x8d\xa1',
'CjkUnifiedIdeograph75cf': '\xe7\x97\x8f',
'CjkUnifiedIdeograph75ce': '\xe7\x97\x8e',
'CjkUnifiedIdeograph75cd': '\xe7\x97\x8d',
'CjkUnifiedIdeograph75cc': '\xe7\x97\x8c',
'CjkUnifiedIdeograph75cb': '\xe7\x97\x8b',
'CjkUnifiedIdeograph75ca': '\xe7\x97\x8a',
'HangulSyllableJjwaelb': '\xec\xab\xbf',
'HangulSyllableToegs': '\xed\x87\xb7',
'SmallerThan': '\xe2\xaa\xaa',
'LatinSmallLetterVend': '\xea\x9d\xa9',
'HangulSyllableYabs': '\xec\x96\x8e',
'HangulSyllableToegg': '\xed\x87\xb6',
'Saltire': '\xe2\x98\x93',
'HangulSyllableMweogg': '\xeb\xad\x92',
'AplFunctionalSymbolDelStile': '\xe2\x8d\x92',
'CjkUnifiedIdeograph9d2c': '\xe9\xb4\xac',
'UnicodeCharacter0xfdd1': '\xef\xb7\x91',
'HangulSyllableBwaenj': '\xeb\xb4\xb1',
'HangulSyllableBwaenh': '\xeb\xb4\xb2',
'CjkUnifiedIdeograph76c0': '\xe7\x9b\x80',
'CjkUnifiedIdeograph76c1': '\xe7\x9b\x81',
'CjkUnifiedIdeograph76c2': '\xe7\x9b\x82',
'CjkUnifiedIdeograph76c3': '\xe7\x9b\x83',
'CjkUnifiedIdeograph76c4': '\xe7\x9b\x84',
'CjkUnifiedIdeograph76c5': '\xe7\x9b\x85',
'LatinSmallLetterLWithCurl': '\xc8\xb4',
'CjkUnifiedIdeograph76c7': '\xe7\x9b\x87',
'CjkUnifiedIdeograph76c8': '\xe7\x9b\x88',
'CjkUnifiedIdeograph76c9': '\xe7\x9b\x89',
'CyrillicCapitalLetterKomiSje': '\xd4\x8c',
'LatinCapitalLetterEWithAcute': '\xc3\x89',
'CjkUnifiedIdeograph3abd': '\xe3\xaa\xbd',
'YiSyllableNuo': '\xea\x86\x88',
'CjkUnifiedIdeograph4520': '\xe4\x94\xa0',
'SyriacZqaphaAbove': '\xdc\xb3',
'CjkUnifiedIdeograph3abf': '\xe3\xaa\xbf',
'YiSyllableNux': '\xea\x86\x92',
'UnicodeCharacter0xf386': '\xef\x8e\x86',
'YiSyllableNur': '\xea\x86\x96',
'CyrillicCapitalLetterU': '\xd0\xa3',
'YiSyllableNup': '\xea\x86\x94',
'YiSyllableNut': '\xea\x86\x91',
'CjkUnifiedIdeograph3aba': '\xe3\xaa\xba',
'CjkUnifiedIdeograph76ca': '\xe7\x9b\x8a',
'CjkUnifiedIdeograph76cb': '\xe7\x9b\x8b',
'CjkUnifiedIdeograph76cc': '\xe7\x9b\x8c',
'CjkUnifiedIdeograph76cd': '\xe7\x9b\x8d',
'CjkUnifiedIdeograph3abb': '\xe3\xaa\xbb',
'CjkUnifiedIdeograph76cf': '\xe7\x9b\x8f',
'CjkUnifiedIdeograph6abf': '\xe6\xaa\xbf',
'CjkUnifiedIdeograph3abc': '\xe3\xaa\xbc',
'UnicodeCharacter0xd96c': '\xed\xa5\xac',
'UnicodeCharacter0xf389': '\xef\x8e\x89',
'UnicodeCharacter0xd96b': '\xed\xa5\xab',
'UnicodeCharacter0xef3a': '\xee\xbc\xba',
'ArabicLigatureThehWithZainFinalForm': '\xef\xb1\xb7',
'UnicodeCharacter0xd96a': '\xed\xa5\xaa',
'HangulSyllableJoenj': '\xec\xa3\x89',
'HangulSyllableJoenh': '\xec\xa3\x8a',
'HangulSyllableJoeng': '\xec\xa3\x99',
'CyrillicSmallLetterDze': '\xd1\x95',
'UnicodeCharacter0xdb91': '\xed\xae\x91',
'Between': '\xe2\x89\xac',
'GurmukhiLetterJha': '\xe0\xa8\x9d',
'UnicodeCharacter0xd96d': '\xed\xa5\xad',
'varbeta': '\xcf\x90',
'CjkUnifiedIdeograph3582': '\xe3\x96\x82',
'CjkUnifiedIdeograph3583': '\xe3\x96\x83',
'CjkUnifiedIdeograph3580': '\xe3\x96\x80',
'CjkUnifiedIdeograph3581': '\xe3\x96\x81',
'CjkUnifiedIdeograph3586': '\xe3\x96\x86',
'CjkUnifiedIdeograph3587': '\xe3\x96\x87',
'CjkUnifiedIdeograph3584': '\xe3\x96\x84',
'CjkUnifiedIdeograph3585': '\xe3\x96\x85',
'HangulSyllableMyog': '\xeb\xac\x99',
'CjkUnifiedIdeograph3588': '\xe3\x96\x88',
'CjkUnifiedIdeograph3589': '\xe3\x96\x89',
'HangulSyllableRoelp': '\xeb\xa2\xbe',
'HangulSyllableMyoc': '\xeb\xac\xaf',
'UnicodeCharacter0xdb90': '\xed\xae\x90',
'ModifierLetterLowVerticalLine': '\xcb\x8c',
'UnicodeCharacter0xee0f': '\xee\xb8\x8f',
'CjkUnifiedIdeograph469c': '\xe4\x9a\x9c',
'HangulSyllableMyot': '\xeb\xac\xb1',
'UnicodeCharacter0xee0e': '\xee\xb8\x8e',
'UnicodeCharacter0xee0b': '\xee\xb8\x8b',
'HangulSyllableMyos': '\xeb\xac\xab',
'HangulSyllableMyop': '\xeb\xac\xb2',
'UnicodeCharacter0xee0a': '\xee\xb8\x8a',
'CjkUnifiedIdeograph6abc': '\xe6\xaa\xbc',
'HangulSyllablePen': '\xed\x8e\x9c',
'CjkUnifiedIdeograph8bc0': '\xe8\xaf\x80',
'SquareWatto': '\xe3\x8d\x97',
'HangulSyllableAlm': '\xec\x95\x8e',
'UnicodeCharacter0xe3ba': '\xee\x8e\xba',
'UnicodeCharacter0xee08': '\xee\xb8\x88',
'UnicodeCharacter0xee09': '\xee\xb8\x89',
'UnicodeCharacter0xee06': '\xee\xb8\x86',
'CjkUnifiedIdeograph8bc3': '\xe8\xaf\x83',
'AplFunctionalSymbolQuadCircle': '\xe2\x8c\xbc',
'UnicodeCharacter0xee05': '\xee\xb8\x85',
'UnicodeCharacter0xee02': '\xee\xb8\x82',
'UnicodeCharacter0xee03': '\xee\xb8\x83',
'UnicodeCharacter0xee00': '\xee\xb8\x80',
'UnicodeCharacter0xee01': '\xee\xb8\x81',
'CjkUnifiedIdeograph358b': '\xe3\x96\x8b',
'CjkUnifiedIdeograph358c': '\xe3\x96\x8c',
'HangulChoseongSioskiyeok': '\xe1\x84\xad',
'CjkUnifiedIdeograph358f': '\xe3\x96\x8f',
'HangulSyllableJwaess': '\xec\xa2\xbc',
'CjkUnifiedIdeograph358d': '\xe3\x96\x8d',
'CjkUnifiedIdeograph358e': '\xe3\x96\x8e',
'CjkUnifiedIdeograph4918': '\xe4\xa4\x98',
'CjkUnifiedIdeograph4919': '\xe4\xa4\x99',
'CjkUnifiedIdeograph99ba': '\xe9\xa6\xba',
'CjkUnifiedIdeograph99bf': '\xe9\xa6\xbf',
'HangulSyllableSyaenj': '\xec\x84\x85',
'CjkUnifiedIdeograph99bd': '\xe9\xa6\xbd',
'CjkUnifiedIdeograph8bc2': '\xe8\xaf\x82',
'CjkUnifiedIdeograph4910': '\xe4\xa4\x90',
'CjkUnifiedIdeograph4911': '\xe4\xa4\x91',
'RightwardsHarpoonWithBarbDownBelowLongDash': '\xe2\xa5\xad',
'CjkUnifiedIdeograph4913': '\xe4\xa4\x93',
'CjkUnifiedIdeograph4914': '\xe4\xa4\x94',
'CjkUnifiedIdeograph4915': '\xe4\xa4\x95',
'CjkUnifiedIdeograph4916': '\xe4\xa4\x96',
'CjkUnifiedIdeograph4917': '\xe4\xa4\x97',
'CjkUnifiedIdeograph8cb3': '\xe8\xb2\xb3',
'UnicodeCharacter0xe36a': '\xee\x8d\xaa',
'CjkUnifiedIdeograph746e': '\xe7\x91\xae',
'CjkUnifiedIdeograph746d': '\xe7\x91\xad',
'CjkUnifiedIdeograph746f': '\xe7\x91\xaf',
'CjkUnifiedIdeograph746a': '\xe7\x91\xaa',
'CjkUnifiedIdeograph746c': '\xe7\x91\xac',
'CjkUnifiedIdeograph746b': '\xe7\x91\xab',
'CjkUnifiedIdeograph8b84': '\xe8\xae\x84',
'CjkUnifiedIdeograph8b85': '\xe8\xae\x85',
'CjkUnifiedIdeograph8b86': '\xe8\xae\x86',
'CjkUnifiedIdeograph8b87': '\xe8\xae\x87',
'CjkUnifiedIdeograph8b80': '\xe8\xae\x80',
'CanadianSyllabicsCarrierDlee': '\xe1\x98\xab',
'CjkUnifiedIdeograph8b82': '\xe8\xae\x82',
'CjkUnifiedIdeograph8b83': '\xe8\xae\x83',
'HangulSyllableHyaelt': '\xed\x96\xb9',
'CjkUnifiedIdeograph8aca': '\xe8\xab\x8a',
'HangulSyllableBbyigg': '\xec\x81\xb6',
'CjkUnifiedIdeograph8b88': '\xe8\xae\x88',
'CjkUnifiedIdeograph8b89': '\xe8\xae\x89',
'HangulSyllableHyaels': '\xed\x96\xb8',
'HangulSyllableGgwegs': '\xea\xbf\xb3',
'HangulSyllableHyaelm': '\xed\x96\xb6',
'HangulSyllableHyaelh': '\xed\x96\xbb',
'LaoLetterDo': '\xe0\xba\x94',
'CjkUnifiedIdeograph8a3d': '\xe8\xa8\xbd',
'CjkUnifiedIdeograph8a3e': '\xe8\xa8\xbe',
'CjkUnifiedIdeograph8a3b': '\xe8\xa8\xbb',
'Cedilla': '\xc2\xb8',
'HebrewLetterQof': '\xd7\xa7',
'CjkUnifiedIdeograph758e': '\xe7\x96\x8e',
'TibetanMarkTsaPhru': '\xe0\xbc\xb9',
'CjkUnifiedIdeograph7469': '\xe7\x91\xa9',
'HangulChoseongSioschieuch': '\xe1\x84\xb7',
'HangulSyllableSwaeg': '\xec\x87\x85',
'CjkUnifiedIdeograph7465': '\xe7\x91\xa5',
'CjkUnifiedIdeograph7464': '\xe7\x91\xa4',
'CjkUnifiedIdeograph7467': '\xe7\x91\xa7',
'CjkUnifiedIdeograph7466': '\xe7\x91\xa6',
'CjkUnifiedIdeograph7461': '\xe7\x91\xa1',
'CjkUnifiedIdeograph7460': '\xe7\x91\xa0',
'CjkUnifiedIdeograph7463': '\xe7\x91\xa3',
'CjkUnifiedIdeograph7462': '\xe7\x91\xa2',
'CjkUnifiedIdeograph99b2': '\xe9\xa6\xb2',
'CjkUnifiedIdeograph951f': '\xe9\x94\x9f',
'CjkUnifiedIdeograph99b0': '\xe9\xa6\xb0',
'CjkUnifiedIdeograph99b1': '\xe9\xa6\xb1',
'CjkUnifiedIdeograph99b6': '\xe9\xa6\xb6',
'CjkUnifiedIdeograph99b7': '\xe9\xa6\xb7',
'CjkUnifiedIdeograph99b4': '\xe9\xa6\xb4',
'CjkUnifiedIdeograph99b5': '\xe9\xa6\xb5',
'CjkUnifiedIdeograph491a': '\xe4\xa4\x9a',
'CjkUnifiedIdeograph491b': '\xe4\xa4\x9b',
'CjkUnifiedIdeograph491c': '\xe4\xa4\x9c',
'CjkUnifiedIdeograph491d': '\xe4\xa4\x9d',
'CjkUnifiedIdeograph491e': '\xe4\xa4\x9e',
'CjkUnifiedIdeograph491f': '\xe4\xa4\x9f',
'HangulSyllableSwaec': '\xec\x87\x9b',
'HangulSyllableNeolb': '\xeb\x84\x93',
'CjkUnifiedIdeograph8a38': '\xe8\xa8\xb8',
'CjkUnifiedIdeograph8a39': '\xe8\xa8\xb9',
'CjkUnifiedIdeograph8a36': '\xe8\xa8\xb6',
'CjkUnifiedIdeograph8a37': '\xe8\xa8\xb7',
'CjkUnifiedIdeograph8a34': '\xe8\xa8\xb4',
'CjkUnifiedIdeograph8a35': '\xe8\xa8\xb5',
'CjkUnifiedIdeograph8a32': '\xe8\xa8\xb2',
'CjkUnifiedIdeograph8a33': '\xe8\xa8\xb3',
'CjkUnifiedIdeograph8a30': '\xe8\xa8\xb0',
'CjkUnifiedIdeograph8a31': '\xe8\xa8\xb1',
'CjkUnifiedIdeograph8b8d': '\xe8\xae\x8d',
'CjkUnifiedIdeograph8b8e': '\xe8\xae\x8e',
'CjkUnifiedIdeograph8b8f': '\xe8\xae\x8f',
'HangulSyllableTwelm': '\xed\x89\xae',
'CjkUnifiedIdeograph8b8a': '\xe8\xae\x8a',
'RunicCrossPunctuation': '\xe1\x9b\xad',
'CjkUnifiedIdeograph8b8c': '\xe8\xae\x8c',
'HangulSyllableSelb': '\xec\x85\x83',
'UnicodeCharacter0xe478': '\xee\x91\xb8',
'CjkUnifiedIdeograph9d29': '\xe9\xb4\xa9',
'ArabicLigatureNoonWithRehFinalForm': '\xef\xb2\x8a',
'HangulSyllableSwaem': '\xec\x87\x94',
'HangulSyllableTwelh': '\xed\x89\xb3',
'subj': '\xe2\xb1\xbc',
'CanadianSyllabicsPoo': '\xe1\x90\xb4',
'UnicodeCharacter0xf74f': '\xef\x9d\x8f',
'UnicodeCharacter0xdf8e': '\xed\xbe\x8e',
'HangulSyllableSwaek': '\xec\x87\x9c',
'CjkUnifiedIdeograph883a': '\xe8\xa0\xba',
'HangulSyllableNeolt': '\xeb\x84\x95',
'UnicodeCharacter0xef3c': '\xee\xbc\xbc',
'IdeographicTelegraphSymbolForJuly': '\xe3\x8b\x86',
'HangulSyllablePwip': '\xed\x93\xa6',
'HangulSyllablePwis': '\xed\x93\x9f',
'HangulSyllablePwit': '\xed\x93\xa5',
'HangulSyllableHyulb': '\xed\x9c\xbf',
'UnicodeCharacter0x18fa': '\xe1\xa3\xba',
'HangulSyllableGeh': '\xea\xb2\xa7',
'UnicodeCharacter0x18fc': '\xe1\xa3\xbc',
'UnicodeCharacter0x18fb': '\xe1\xa3\xbb',
'UnicodeCharacter0x18fe': '\xe1\xa3\xbe',
'UnicodeCharacter0x18fd': '\xe1\xa3\xbd',
'UnicodeCharacter0x18ff': '\xe1\xa3\xbf',
'HangulSyllableNeolp': '\xeb\x84\x96',
'HangulSyllablePwic': '\xed\x93\xa3',
'HangulSyllablePwib': '\xed\x93\x9d',
'HangulSyllablePwid': '\xed\x93\x93',
'HangulSyllablePwig': '\xed\x93\x8d',
'HangulSyllableSwaes': '\xec\x87\x97',
'CyrillicSmallLetterTswe': '\xea\x9a\x8f',
'LatinSmallLetterLsDigraph': '\xca\xaa',
'CyrillicSmallLetterYeruWithDiaeresis': '\xd3\xb9',
'UnicodeCharacter0xf214': '\xef\x88\x94',
'HangulSyllableJjyulp': '\xec\xaf\x86',
'HangulSyllableTeonh': '\xed\x84\xb6',
'HangulSyllableTeonj': '\xed\x84\xb5',
'HangulSyllableJjyuls': '\xec\xaf\x84',
'HangulChoseongPieupthieuth': '\xe1\x84\xa9',
'HangulSyllableHyeng': '\xed\x98\xb1',
'HangulSyllableHyenj': '\xed\x98\xa1',
'HangulSyllableHyenh': '\xed\x98\xa2',
'CombiningLatinLetterSmallCapitalG': '\xe1\xb7\x9b',
'HangulSyllableJjyult': '\xec\xaf\x85',
'UnicodeCharacter0x18f1': '\xe1\xa3\xb1',
'UnicodeCharacter0x18f0': '\xe1\xa3\xb0',
'UnicodeCharacter0x18f3': '\xe1\xa3\xb3',
'UnicodeCharacter0x18f2': '\xe1\xa3\xb2',
'UnicodeCharacter0x18f5': '\xe1\xa3\xb5',
'UnicodeCharacter0x18f4': '\xe1\xa3\xb4',
'UnicodeCharacter0x18f7': '\xe1\xa3\xb7',
'UnicodeCharacter0x18f6': '\xe1\xa3\xb6',
'UnicodeCharacter0x18f9': '\xe1\xa3\xb9',
'UnicodeCharacter0x18f8': '\xe1\xa3\xb8',
'HangulSyllableDoeng': '\xeb\x90\xad',
'PhagspaLetterAlternateYa': '\xea\xa1\xad',
'HangulSyllableTweogs': '\xed\x89\x8b',
'YiSyllableNjox': '\xea\x90\xbc',
'MyanmarDigitFour': '\xe1\x81\x84',
'DoubleUpTack': '\xe2\xab\xab',
'CjkUnifiedIdeograph7647': '\xe7\x99\x87',
'UnicodeCharacter0xdebb': '\xed\xba\xbb',
'UnicodeCharacter0xf1be': '\xef\x86\xbe',
'HangulSyllableTweogg': '\xed\x89\x8a',
'CjkUnifiedIdeograph7646': '\xe7\x99\x86',
'MonostableSymbol': '\xe2\x8e\x8d',
'UnicodeCharacter0xdeba': '\xed\xba\xba',
'HangulSyllableCyaegs': '\xec\xb1\xbf',
'UnicodeCharacter0xe799': '\xee\x9e\x99',
'CjkUnifiedIdeograph34aa': '\xe3\x92\xaa',
'CjkUnifiedIdeograph34ab': '\xe3\x92\xab',
'CjkUnifiedIdeograph34ac': '\xe3\x92\xac',
'CjkUnifiedIdeograph34ad': '\xe3\x92\xad',
'CjkUnifiedIdeograph34ae': '\xe3\x92\xae',
'CjkUnifiedIdeograph34af': '\xe3\x92\xaf',
'HangulSyllableJjyabs': '\xec\xa8\xa6',
'CjkUnifiedIdeograph96d3': '\xe9\x9b\x93',
'CjkUnifiedIdeograph96d2': '\xe9\x9b\x92',
'CjkUnifiedIdeograph96d1': '\xe9\x9b\x91',
'CjkUnifiedIdeograph96d0': '\xe9\x9b\x90',
'CjkUnifiedIdeograph96d7': '\xe9\x9b\x97',
'CjkUnifiedIdeograph96d6': '\xe9\x9b\x96',
'CjkUnifiedIdeograph96d5': '\xe9\x9b\x95',
'CjkUnifiedIdeograph96d4': '\xe9\x9b\x94',
'CjkUnifiedIdeograph96d9': '\xe9\x9b\x99',
'CjkUnifiedIdeograph96d8': '\xe9\x9b\x98',
'YiSyllableMge': '\xea\x88\xbe',
'HangulSyllableCaeb': '\xec\xb1\x95',
'HangulSyllableSigg': '\xec\x8b\x9e',
'CjkUnifiedIdeograph34a0': '\xe3\x92\xa0',
'CjkUnifiedIdeograph34a1': '\xe3\x92\xa1',
'CjkUnifiedIdeograph34a2': '\xe3\x92\xa2',
'CjkUnifiedIdeograph34a3': '\xe3\x92\xa3',
'DevanagariLetterNna': '\xe0\xa4\xa3',
'CjkUnifiedIdeograph34a5': '\xe3\x92\xa5',
'CjkUnifiedIdeograph34a6': '\xe3\x92\xa6',
'CjkUnifiedIdeograph34a7': '\xe3\x92\xa7',
'CjkUnifiedIdeograph34a8': '\xe3\x92\xa8',
'CjkUnifiedIdeograph34a9': '\xe3\x92\xa9',
'TamilDigitNine': '\xe0\xaf\xaf',
'LatinSmallLetterSWithHook': '\xca\x82',
'HangulSyllableJjyulh': '\xec\xaf\x87',
'HangulSyllableRigs': '\xeb\xa6\xaf',
'HangulSyllableNilg': '\xeb\x8b\x91',
'HangulSyllableNilb': '\xeb\x8b\x93',
'MyanmarVowelSignWesternPwoKarenEu': '\xe1\x81\xa7',
'HangulSyllableMwaebs': '\xeb\xab\xb2',
'HangulSyllableNilm': '\xeb\x8b\x92',
'HangulSyllableNilh': '\xeb\x8b\x97',
'CjkUnifiedIdeograph96dc': '\xe9\x9b\x9c',
'CjkUnifiedIdeograph96db': '\xe9\x9b\x9b',
'CjkUnifiedIdeograph96da': '\xe9\x9b\x9a',
'HangulSyllableRigg': '\xeb\xa6\xae',
'CjkUnifiedIdeograph96df': '\xe9\x9b\x9f',
'CjkUnifiedIdeograph96de': '\xe9\x9b\x9e',
'CjkUnifiedIdeograph96dd': '\xe9\x9b\x9d',
'CjkUnifiedIdeograph76a1': '\xe7\x9a\xa1',
'UnicodeCharacter0xdf82': '\xed\xbe\x82',
'ArabicSmallKasra': '\xd8\x9a',
'CjkStrokeHzzzg': '\xe3\x87\xa1',
'CjkUnifiedIdeograph6ce4': '\xe6\xb3\xa4',
'CjkUnifiedIdeograph6ce5': '\xe6\xb3\xa5',
'CjkUnifiedIdeograph6ce6': '\xe6\xb3\xa6',
'CjkUnifiedIdeograph6ce7': '\xe6\xb3\xa7',
'CjkUnifiedIdeograph6ce0': '\xe6\xb3\xa0',
'CjkUnifiedIdeograph6ce1': '\xe6\xb3\xa1',
'CjkUnifiedIdeograph6ce2': '\xe6\xb3\xa2',
'LatinLetterWynn': '\xc6\xbf',
'HangulSyllableHyad': '\xed\x96\x97',
'EmphasisSymbol': '\xe2\x8e\x83',
'CjkUnifiedIdeograph6ce9': '\xe6\xb3\xa9',
'summation': '\xe2\x88\x91',
'IntersectionBesideAndJoinedWithIntersection': '\xe2\xa9\x8b',
'FourthRoot': '\xe2\x88\x9c',
'YiSyllableMga': '\xea\x88\xb4',
'DoublePrimeQuotationMark': '\xe3\x80\x9e',
'LeftwardsHarpoonWithBarbUpToBar': '\xe2\xa5\x92',
'LatinSmallLetterThornWithStroke': '\xea\x9d\xa5',
'ArabicLigatureNoonWithHahWithYehFinalForm': '\xef\xb6\xb3',
'UnicodeCharacter0xe45b': '\xee\x91\x9b',
'UnicodeCharacter0xe46a': '\xee\x91\xaa',
'UnicodeCharacter0x218f': '\xe2\x86\x8f',
'UnicodeCharacter0x218e': '\xe2\x86\x8e',
'UnicodeCharacter0x218d': '\xe2\x86\x8d',
'UnicodeCharacter0x218c': '\xe2\x86\x8c',
'UnicodeCharacter0x218b': '\xe2\x86\x8b',
'UnicodeCharacter0x218a': '\xe2\x86\x8a',
'KangxiRadicalKnife': '\xe2\xbc\x91',
'CjkUnifiedIdeograph6ced': '\xe6\xb3\xad',
'CjkUnifiedIdeograph6cee': '\xe6\xb3\xae',
'CjkUnifiedIdeograph6cef': '\xe6\xb3\xaf',
'CjkUnifiedIdeograph6cea': '\xe6\xb3\xaa',
'CjkUnifiedIdeograph6ceb': '\xe6\xb3\xab',
'CjkUnifiedIdeograph6cec': '\xe6\xb3\xac',
'HangulSyllableJyaegg': '\xec\x9f\xa6',
'HangulSyllableMenh': '\xeb\xa9\x9a',
'HangulSyllableMenj': '\xeb\xa9\x99',
'SquarePenihi': '\xe3\x8c\xb8',
'Mu': '\xce\x9c',
'PitchforkWithTeeTop': '\xe2\xab\x9a',
'LatinSmallLetterGWithPalatalHook': '\xe1\xb6\x83',
'LatinSmallLetterUWithAcute': '\xc3\xba',
'HangulSyllableJyaegs': '\xec\x9f\xa7',
'UnicodeCharacter0xe362': '\xee\x8d\xa2',
'UnicodeCharacter0x2189': '\xe2\x86\x89',
'UnicodeCharacter0xe363': '\xee\x8d\xa3',
'CircledNumberTwentyEight': '\xe3\x89\x98',
'UnicodeCharacter0xf212': '\xef\x88\x92',
'UnicodeCharacter0xe361': '\xee\x8d\xa1',
'UnicodeCharacter0xe46c': '\xee\x91\xac',
'RecyclingSymbolForType2Plastics': '\xe2\x99\xb4',
'UnicodeCharacter0xe366': '\xee\x8d\xa6',
'HangulSyllableHyat': '\xed\x96\xa9',
'HangulSyllableHeu': '\xed\x9d\x90',
'LatinCapitalLetterSWithCircumflex': '\xc5\x9c',
'UnicodeCharacter0xe367': '\xee\x8d\xa7',
'HangulSyllableHwaegs': '\xed\x99\xb3',
'CombiningDotBelow': '\xcc\xa3',
'HangulSyllableDdass': '\xeb\x95\x84',
'CjkUnifiedIdeograph826b': '\xe8\x89\xab',
'CjkUnifiedIdeograph826c': '\xe8\x89\xac',
'CjkUnifiedIdeograph826a': '\xe8\x89\xaa',
'CjkUnifiedIdeograph826f': '\xe8\x89\xaf',
'CjkUnifiedIdeograph826d': '\xe8\x89\xad',
'CjkUnifiedIdeograph826e': '\xe8\x89\xae',
'UnicodeCharacter0xe46b': '\xee\x91\xab',
'HangulSyllableHwaegg': '\xed\x99\xb2',
'QuadrupleIntegralOperator': '\xe2\xa8\x8c',
'CjkUnifiedIdeograph814d': '\xe8\x85\x8d',
'CjkUnifiedIdeograph4cbf': '\xe4\xb2\xbf',
'CjkUnifiedIdeograph4cbe': '\xe4\xb2\xbe',
'CjkUnifiedIdeograph4cbd': '\xe4\xb2\xbd',
'CjkUnifiedIdeograph4cbc': '\xe4\xb2\xbc',
'CjkUnifiedIdeograph4cbb': '\xe4\xb2\xbb',
'CjkUnifiedIdeograph4cba': '\xe4\xb2\xba',
'CjkUnifiedIdeograph6bae': '\xe6\xae\xae',
'CjkUnifiedIdeograph6bad': '\xe6\xae\xad',
'HangulSyllableMoes': '\xeb\xac\x8f',
'CjkUnifiedIdeograph6baf': '\xe6\xae\xaf',
'CjkUnifiedIdeograph6baa': '\xe6\xae\xaa',
'CjkUnifiedIdeograph6bac': '\xe6\xae\xac',
'CjkUnifiedIdeograph6bab': '\xe6\xae\xab',
'ForAll': '\xe2\x88\x80',
'YiSyllableShet': '\xea\x8e\xb6',
'YiSyllableShep': '\xea\x8e\xb9',
'HangulSyllableHej': '\xed\x97\xba',
'CjkUnifiedIdeograph947c': '\xe9\x91\xbc',
'CjkUnifiedIdeograph63e8': '\xe6\x8f\xa8',
'SmallRomanNumeralThree': '\xe2\x85\xb2',
'HangulSyllableMyilp': '\xeb\xaf\xaa',
'HangulSyllableMyils': '\xeb\xaf\xa8',
'HangulSyllableMyilm': '\xeb\xaf\xa6',
'CjkUnifiedIdeograph8268': '\xe8\x89\xa8',
'CjkUnifiedIdeograph8269': '\xe8\x89\xa9',
'HangulSyllableMyilh': '\xeb\xaf\xab',
'CjkUnifiedIdeograph8262': '\xe8\x89\xa2',
'CjkUnifiedIdeograph8263': '\xe8\x89\xa3',
'CjkUnifiedIdeograph8260': '\xe8\x89\xa0',
'CopticSmallLetterKapa': '\xe2\xb2\x95',
'CjkUnifiedIdeograph8266': '\xe8\x89\xa6',
'CjkUnifiedIdeograph8267': '\xe8\x89\xa7',
'CjkUnifiedIdeograph8264': '\xe8\x89\xa4',
'CjkUnifiedIdeograph8265': '\xe8\x89\xa5',
'CjkUnifiedIdeograph6ba5': '\xe6\xae\xa5',
'CjkUnifiedIdeograph6ba4': '\xe6\xae\xa4',
'CjkUnifiedIdeograph6ba7': '\xe6\xae\xa7',
'CjkUnifiedIdeograph6ba6': '\xe6\xae\xa6',
'CjkUnifiedIdeograph6ba1': '\xe6\xae\xa1',
'CjkUnifiedIdeograph6ba0': '\xe6\xae\xa0',
'CjkUnifiedIdeograph6ba3': '\xe6\xae\xa3',
'CjkUnifiedIdeograph6ba2': '\xe6\xae\xa2',
'HangulSyllableHec': '\xed\x97\xbb',
'CjkUnifiedIdeograph6ba9': '\xe6\xae\xa9',
'CjkUnifiedIdeograph6ba8': '\xe6\xae\xa8',
'ArabicLigatureMeemWithKhahWithYehFinalForm': '\xef\xb6\xb9',
'GeorgianCapitalLetterSan': '\xe1\x82\xb1',
'CjkUnifiedIdeograph4cb6': '\xe4\xb2\xb6',
'CjkUnifiedIdeograph4cb5': '\xe4\xb2\xb5',
'CjkUnifiedIdeograph4cb4': '\xe4\xb2\xb4',
'CjkUnifiedIdeograph4cb3': '\xe4\xb2\xb3',
'CjkUnifiedIdeograph4cb2': '\xe4\xb2\xb2',
'CjkUnifiedIdeograph4cb1': '\xe4\xb2\xb1',
'CjkUnifiedIdeograph4cb0': '\xe4\xb2\xb0',
'CjkUnifiedIdeograph951c': '\xe9\x94\x9c',
'CjkUnifiedIdeograph4cb9': '\xe4\xb2\xb9',
'CjkUnifiedIdeograph4cb8': '\xe4\xb2\xb8',
'HangulSyllableBeulb': '\xeb\xb8\x97',
'HangulSyllableKyelg': '\xec\xbd\x81',
'CjkUnifiedIdeograph951a': '\xe9\x94\x9a',
'UnicodeCharacter0xe46f': '\xee\x91\xaf',
'HangulSyllableKyelh': '\xec\xbd\x87',
'GeorgianCapitalLetterGan': '\xe1\x82\xa2',
'HangulSyllableKyelm': '\xec\xbd\x82',
'HangulSyllableSswanj': '\xec\x8f\xb9',
'HangulSyllableKyels': '\xec\xbd\x84',
'HangulSyllableKyelp': '\xec\xbd\x86',
'GetaMark': '\xe3\x80\x93',
'HangulSyllableKyelt': '\xec\xbd\x85',
'UnicodeCharacter0xe459': '\xee\x91\x99',
'UnicodeCharacter0xdeb9': '\xed\xba\xb9',
'HangulChoseongSiosnieun': '\xe1\x84\xae',
'TamilLetterTta': '\xe0\xae\x9f',
'UnicodeCharacter0xe456': '\xee\x91\x96',
'CjkUnifiedIdeograph5e9f': '\xe5\xba\x9f',
'HangulSyllableMoet': '\xeb\xac\x95',
'UnicodeCharacter0xe450': '\xee\x91\x90',
'CjkUnifiedIdeograph5e9d': '\xe5\xba\x9d',
'UnicodeCharacter0xe451': '\xee\x91\x91',
'CjkUnifiedIdeograph7a38': '\xe7\xa8\xb8',
'SundaneseConsonantSignPamingkal': '\xe1\xae\xa1',
'UnicodeCharacter0x10ff': '\xe1\x83\xbf',
'BengaliLetterKa': '\xe0\xa6\x95',
'UnicodeCharacter0xf51c': '\xef\x94\x9c',
'LatinCapitalLetterAWithBreveAndAcute': '\xe1\xba\xae',
'UnicodeCharacter0xdeb7': '\xed\xba\xb7',
'UpwardsPairedArrows': '\xe2\x87\x88',
'EthiopicSyllableFwa': '\xe1\x8d\x8f',
'CjkUnifiedIdeograph7a33': '\xe7\xa8\xb3',
'EthiopicSyllableFwe': '\xe1\x8e\x8b',
'CjkUnifiedIdeograph7a32': '\xe7\xa8\xb2',
'EthiopicSyllableFwi': '\xe1\x8e\x89',
'UnicodeCharacter0xd8d3': '\xed\xa3\x93',
'HangulSyllableKyenh': '\xec\xbc\xbe',
'CjkUnifiedIdeograph7a31': '\xe7\xa8\xb1',
'UnicodeCharacter0xdb9e': '\xed\xae\x9e',
'CjkUnifiedIdeograph64a7': '\xe6\x92\xa7',
'CjkUnifiedIdeograph64a6': '\xe6\x92\xa6',
'CjkUnifiedIdeograph64a5': '\xe6\x92\xa5',
'CjkUnifiedIdeograph64a4': '\xe6\x92\xa4',
'CjkUnifiedIdeograph64a3': '\xe6\x92\xa3',
'CjkUnifiedIdeograph56f8': '\xe5\x9b\xb8',
'CjkUnifiedIdeograph64a1': '\xe6\x92\xa1',
'CjkUnifiedIdeograph64a0': '\xe6\x92\xa0',
'CjkUnifiedIdeograph7a37': '\xe7\xa8\xb7',
'CjkUnifiedIdeograph94f8': '\xe9\x93\xb8',
'KhmerIndependentVowelQooTypeOne': '\xe1\x9e\xb1',
'CjkUnifiedIdeograph64a9': '\xe6\x92\xa9',
'CjkUnifiedIdeograph64a8': '\xe6\x92\xa8',
'HangulSyllableCwed': '\xec\xb7\x93',
'CjkUnifiedIdeograph875a': '\xe8\x9d\x9a',
'CjkUnifiedIdeograph7a35': '\xe7\xa8\xb5',
'HangulSyllableCweb': '\xec\xb7\x9d',
'HangulSyllableCwec': '\xec\xb7\xa3',
'HangulSyllableCwel': '\xec\xb7\x94',
'CjkUnifiedIdeograph875f': '\xe8\x9d\x9f',
'HangulSyllableCwen': '\xec\xb7\x90',
'CjkUnifiedIdeograph7a34': '\xe7\xa8\xb4',
'HangulSyllableCweh': '\xec\xb7\xa7',
'[2]': '\xe2\x82\x82',
'HangulSyllableCwej': '\xec\xb7\xa2',
'HangulSyllableCwek': '\xec\xb7\xa4',
'BengaliLetterKha': '\xe0\xa6\x96',
'GreekCapitalLetterIota': '\xce\x99',
'CjkUnifiedIdeograph875d': '\xe8\x9d\x9d',
'KayahLiDigitNine': '\xea\xa4\x89',
'ArabicLetterPehIsolatedForm': '\xef\xad\x96',
'UnicodeCharacter0xf7be': '\xef\x9e\xbe',
'CjkUnifiedIdeograph6099': '\xe6\x82\x99',
'YiSyllableJjuox': '\xea\x90\xa0',
'ArabicComma': '\xd8\x8c',
'HangulSyllableKyenj': '\xec\xbc\xbd',
'BoxDrawingsLightArcDownAndLeft': '\xe2\x95\xae',
'HangulSyllableJjwis': '\xec\xae\xaf',
'UnicodeCharacter0xe54a': '\xee\x95\x8a',
'OriyaLetterNya': '\xe0\xac\x9e',
'CjkUnifiedIdeograph64af': '\xe6\x92\xaf',
'CjkUnifiedIdeograph64ae': '\xe6\x92\xae',
'CjkUnifiedIdeograph64ad': '\xe6\x92\xad',
'CjkUnifiedIdeograph64ac': '\xe6\x92\xac',
'CjkUnifiedIdeograph64ab': '\xe6\x92\xab',
'CjkUnifiedIdeograph64aa': '\xe6\x92\xaa',
'LatinCapitalLetterVend': '\xea\x9d\xa8',
'InverseWhiteCircle': '\xe2\x97\x99',
'HangulSyllableTwaess': '\xed\x87\xac',
'oiiint': '\xe2\x88\xb0',
'HangulSyllableGyoss': '\xea\xb5\xa4',
'UnicodeCharacter0xe54f': '\xee\x95\x8f',
'ArabicVowelSignInvertedSmallVAbove': '\xd9\x9b',
'TamilNumberTen': '\xe0\xaf\xb0',
'LatinSmallLetterVWithDotBelow': '\xe1\xb9\xbf',
'HangulSyllableSyalm': '\xec\x83\xae',
'KangxiRadicalTiger': '\xe2\xbe\x8c',
'UnicodeCharacter0xf541': '\xef\x95\x81',
'HyphenationPoint': '\xe2\x80\xa7',
'CjkUnifiedIdeograph63a8': '\xe6\x8e\xa8',
'CjkUnifiedIdeograph63a9': '\xe6\x8e\xa9',
'HangulSyllableDuh': '\xeb\x91\xab',
'HangulSyllableDun': '\xeb\x91\x94',
'HangulSyllableDum': '\xeb\x91\xa0',
'CjkUnifiedIdeograph9516': '\xe9\x94\x96',
'SquareMil': '\xe3\x8f\x95',
'CjkUnifiedIdeograph63a1': '\xe6\x8e\xa1',
'CjkUnifiedIdeograph63a2': '\xe6\x8e\xa2',
'CjkUnifiedIdeograph5851': '\xe5\xa1\x91',
'CjkUnifiedIdeograph63a4': '\xe6\x8e\xa4',
'CjkUnifiedIdeograph63a5': '\xe6\x8e\xa5',
'CjkUnifiedIdeograph63a6': '\xe6\x8e\xa6',
'CjkUnifiedIdeograph63a7': '\xe6\x8e\xa7',
'LimbuDigitFive': '\xe1\xa5\x8b',
'CjkUnifiedIdeograph9514': '\xe9\x94\x94',
'HangulSyllableDus': '\xeb\x91\xa3',
'CjkRadicalSnoutOne': '\xe2\xba\x94',
'HangulSyllableGyulh': '\xea\xb7\xab',
'CjkUnifiedIdeograph9512': '\xe9\x94\x92',
'UnicodeCharacter0xbc4': '\xe0\xaf\x84',
'UnicodeCharacter0xbc5': '\xe0\xaf\x85',
'UnicodeCharacter0xe54d': '\xee\x95\x8d',
'CjkUnifiedIdeograph9511': '\xe9\x94\x91',
'UnicodeCharacter0xbc3': '\xe0\xaf\x83',
'CjkUnifiedIdeograph9510': '\xe9\x94\x90',
'UnicodeCharacter0xbc9': '\xe0\xaf\x89',
'CjkUnifiedIdeograph6d62': '\xe6\xb5\xa2',
'CjkUnifiedIdeograph6d63': '\xe6\xb5\xa3',
'CjkUnifiedIdeograph6d60': '\xe6\xb5\xa0',
'CjkUnifiedIdeograph6d61': '\xe6\xb5\xa1',
'CjkUnifiedIdeograph6d66': '\xe6\xb5\xa6',
'BalineseLetterCa': '\xe1\xac\x98',
'CjkUnifiedIdeograph6d64': '\xe6\xb5\xa4',
'CjkUnifiedIdeograph6d65': '\xe6\xb5\xa5',
'UnicodeCharacter0xd93a': '\xed\xa4\xba',
'CjkUnifiedIdeograph6d68': '\xe6\xb5\xa8',
'IdeographicTelegraphSymbolForDayThirtyone': '\xe3\x8f\xbe',
'UnicodeCharacter0xd93d': '\xed\xa4\xbd',
'UnicodeCharacter0xd93e': '\xed\xa4\xbe',
'LatinCapitalLetterEsh': '\xc6\xa9',
'CanadianSyllabicsQaai': '\xe1\x95\xbe',
'UnicodeCharacter0xe54e': '\xee\x95\x8e',
'HangulJongseongKapyeounpieup': '\xe1\x87\xa6',
'UnicodeCharacter0xf4b1': '\xef\x92\xb1',
'HangulSyllableBbyip': '\xec\x82\x8e',
'HangulSyllableGyulp': '\xea\xb7\xaa',
'CjkCompatibilityIdeographf943': '\xef\xa5\x83',
'CjkUnifiedIdeograph8143': '\xe8\x85\x83',
'UnicodeCharacter0xf613': '\xef\x98\x93',
'CjkUnifiedIdeograph63aa': '\xe6\x8e\xaa',
'CjkUnifiedIdeograph63ab': '\xe6\x8e\xab',
'CjkUnifiedIdeograph63ac': '\xe6\x8e\xac',
'CjkUnifiedIdeograph63ad': '\xe6\x8e\xad',
'CjkUnifiedIdeograph63ae': '\xe6\x8e\xae',
'CjkUnifiedIdeograph63af': '\xe6\x8e\xaf',
'CjkUnifiedIdeograph65ec': '\xe6\x97\xac',
'GeorgianCapitalLetterAn': '\xe1\x82\xa0',
'CjkUnifiedIdeograph6d6c': '\xe6\xb5\xac',
'CjkUnifiedIdeograph6d6a': '\xe6\xb5\xaa',
'CjkUnifiedIdeograph5877': '\xe5\xa1\xb7',
'ThaanaLetterAlifu': '\xde\x87',
'CjkUnifiedIdeograph6d6d': '\xe6\xb5\xad',
'CjkUnifiedIdeograph6d6e': '\xe6\xb5\xae',
'UnicodeCharacter0xd930': '\xed\xa4\xb0',
'UnicodeCharacter0xd931': '\xed\xa4\xb1',
'CyrillicSmallLetterGheWithMiddleHook': '\xd2\x95',
'LatinSmallLetterReversedE': '\xc9\x98',
'ArmenianSmallLetterRa': '\xd5\xbc',
'LatinSmallLetterReversedC': '\xe2\x86\x84',
'UnicodeCharacter0xd936': '\xed\xa4\xb6',
'CjkUnifiedIdeograph8146': '\xe8\x85\x86',
'UnicodeCharacter0xbce': '\xe0\xaf\x8e',
'UnicodeCharacter0xbcf': '\xe0\xaf\x8f',
'HangulSyllableAlb': '\xec\x95\x8f',
'UnicodeCharacter0xe545': '\xee\x95\x85',
'UnicodeCharacter0xf4b9': '\xef\x92\xb9',
'BugineseVowelSignAe': '\xe1\xa8\x9b',
'NewTaiLueDigitSeven': '\xe1\xa7\x97',
'UnicodeCharacter0xf0a1': '\xef\x82\xa1',
'ChamLetterNue': '\xea\xa8\x97',
'UnicodeCharacter0xdc9d': '\xed\xb2\x9d',
'CjkUnifiedIdeograph63ec': '\xe6\x8f\xac',
'KangxiRadicalFace': '\xe2\xbe\xaf',
'CjkUnifiedIdeograph5e22': '\xe5\xb8\xa2',
'CjkUnifiedIdeograph5e23': '\xe5\xb8\xa3',
'CjkUnifiedIdeograph5e20': '\xe5\xb8\xa0',
'CjkUnifiedIdeograph5723': '\xe5\x9c\xa3',
'CjkUnifiedIdeograph5e26': '\xe5\xb8\xa6',
'CjkUnifiedIdeograph5e27': '\xe5\xb8\xa7',
'CjkUnifiedIdeograph5e24': '\xe5\xb8\xa4',
'TaiLeLetterTsa': '\xe1\xa5\x93',
'HangulSyllableSsyeoc': '\xec\x8e\xb7',
'CjkUnifiedIdeograph5724': '\xe5\x9c\xa4',
'CjkUnifiedIdeograph5e28': '\xe5\xb8\xa8',
'CjkUnifiedIdeograph5e29': '\xe5\xb8\xa9',
'UnicodeCharacter0xe9ba': '\xee\xa6\xba',
'UnicodeCharacter0xdc9e': '\xed\xb2\x9e',
'UnicodeCharacter0xe9bc': '\xee\xa6\xbc',
'UnicodeCharacter0xf47c': '\xef\x91\xbc',
'HangulSyllableHyilg': '\xed\x9d\xb5',
'HangulSyllableHyilb': '\xed\x9d\xb7',
'CjkUnifiedIdeograph5c36': '\xe5\xb0\xb6',
'HangulSyllableHyilm': '\xed\x9d\xb6',
'HangulSyllableHyilh': '\xed\x9d\xbb',
'UnicodeCharacter0xf47a': '\xef\x91\xba',
'CyrillicSmallLetterGheWithUpturn': '\xd2\x91',
'CjkUnifiedIdeograph5728': '\xe5\x9c\xa8',
'UnicodeCharacter0xf4d8': '\xef\x93\x98',
'HangulSyllableHyilp': '\xed\x9d\xba',
'HangulSyllableGen': '\xea\xb2\x90',
'HangulSyllableHyils': '\xed\x9d\xb8',
'LatinCapitalLetterTWithCircumflexBelow': '\xe1\xb9\xb0',
'UnicodeCharacter0xf47f': '\xef\x91\xbf',
'CjkUnifiedIdeograph5e2b': '\xe5\xb8\xab',
'CjkUnifiedIdeograph5e2c': '\xe5\xb8\xac',
'CjkUnifiedIdeograph5e2a': '\xe5\xb8\xaa',
'KhmerLetterNno': '\xe1\x9e\x8e',
'UnicodeCharacter0xe9b9': '\xee\xa6\xb9',
'CjkUnifiedIdeograph5e2d': '\xe5\xb8\xad',
'CjkUnifiedIdeograph5e2e': '\xe5\xb8\xae',
'UnicodeCharacter0xe9b4': '\xee\xa6\xb4',
'UnicodeCharacter0xe9b5': '\xee\xa6\xb5',
'UnicodeCharacter0xe9b6': '\xee\xa6\xb6',
'UnicodeCharacter0xe9b7': '\xee\xa6\xb7',
'UnicodeCharacter0xe9b0': '\xee\xa6\xb0',
'UnicodeCharacter0xe9b1': '\xee\xa6\xb1',
'UnicodeCharacter0xe9b2': '\xee\xa6\xb2',
'UnicodeCharacter0xe9b3': '\xee\xa6\xb3',
'BalineseLetterUkaraTedung': '\xe1\xac\x8a',
'FractionSlash': '\xe2\x81\x84',
'UnicodeCharacter0xab38': '\xea\xac\xb8',
'HangulSyllableCyalm': '\xec\xb1\xaa',
'UnicodeCharacter0xab39': '\xea\xac\xb9',
'HangulSyllableCyalg': '\xec\xb1\xa9',
'HangulSyllableSsaelm': '\xec\x8c\x9e',
'HangulSyllableCyalb': '\xec\xb1\xab',
'CjkUnifiedIdeograph5c38': '\xe5\xb0\xb8',
'HangulSyllableCyals': '\xec\xb1\xac',
'HangulSyllableCyalp': '\xec\xb1\xae',
'UnicodeCharacter0xf42a': '\xef\x90\xaa',
'UnicodeCharacter0xf42b': '\xef\x90\xab',
'YiSyllableNyuo': '\xea\x91\x93',
'HiraganaLetterTu': '\xe3\x81\xa4',
'UnicodeCharacter0xf42e': '\xef\x90\xae',
'UnicodeCharacter0xf42f': '\xef\x90\xaf',
'HangulSyllableCyags': '\xec\xb1\xa3',
'TibetanLetterDdha': '\xe0\xbd\x8d',
'UnicodeCharacter0xe62e': '\xee\x98\xae',
'HiraganaLetterTa': '\xe3\x81\x9f',
'CjkUnifiedIdeograph4281': '\xe4\x8a\x81',
'CjkUnifiedIdeograph4282': '\xe4\x8a\x82',
'CjkUnifiedIdeograph4283': '\xe4\x8a\x83',
'HiraganaLetterTe': '\xe3\x81\xa6',
'CjkUnifiedIdeograph4285': '\xe4\x8a\x85',
'CjkUnifiedIdeograph4286': '\xe4\x8a\x86',
'CjkUnifiedIdeograph4287': '\xe4\x8a\x87',
'HiraganaLetterTi': '\xe3\x81\xa1',
'CjkUnifiedIdeograph4289': '\xe4\x8a\x89',
'UnicodeCharacter0xe68f': '\xee\x9a\x8f',
'YiSyllableNyup': '\xea\x91\x9c',
'MeasuredRightAngleWithDot': '\xe2\xa6\x9d',
'HiraganaLetterTo': '\xe3\x81\xa8',
'UnicodeCharacter0xe68b': '\xee\x9a\x8b',
'HangulSyllableBbwilm': '\xec\x80\xaa',
'UnicodeCharacter0xab33': '\xea\xac\xb3',
'HangulSyllableBbwilh': '\xec\x80\xaf',
'HangulSyllableBbwilg': '\xec\x80\xa9',
'UnicodeCharacter0xab34': '\xea\xac\xb4',
'HangulSyllableDdoenj': '\xeb\x99\xa9',
'HangulSyllableBbwilb': '\xec\x80\xab',
'UnicodeCharacter0xede7': '\xee\xb7\xa7',
'UnicodeCharacter0xab36': '\xea\xac\xb6',
'LatinSmallLetterLWithBar': '\xc6\x9a',
'UnicodeCharacter0xab37': '\xea\xac\xb7',
'CyrillicSmallLetterBroadOmega': '\xea\x99\x8d',
'UpwardsHarpoonWithBarbRightFromBar': '\xe2\xa5\x9c',
'HangulSyllableBbwilp': '\xec\x80\xae',
'CjkUnifiedIdeograph5885': '\xe5\xa2\x85',
'CjkUnifiedIdeograph428a': '\xe4\x8a\x8a',
'CjkUnifiedIdeograph428b': '\xe4\x8a\x8b',
'CjkUnifiedIdeograph428c': '\xe4\x8a\x8c',
'CjkUnifiedIdeograph428d': '\xe4\x8a\x8d',
'CjkUnifiedIdeograph428e': '\xe4\x8a\x8e',
'MongolianLetterAliGaliAh': '\xe1\xa2\x97',
'CjkUnifiedIdeograph5882': '\xe5\xa2\x82',
'UnicodeCharacter0xe685': '\xee\x9a\x85',
'UnicodeCharacter0xe684': '\xee\x9a\x84',
'UnicodeCharacter0xe687': '\xee\x9a\x87',
'YiSyllableHxiep': '\xea\x89\x8d',
'CjkUnifiedIdeograph5889': '\xe5\xa2\x89',
'CjkUnifiedIdeograph5888': '\xe5\xa2\x88',
'UnicodeCharacter0xe683': '\xee\x9a\x83',
'UnicodeCharacter0xe682': '\xee\x9a\x82',
'CanadianSyllabicsSwa': '\xe1\x94\x80',
'CjkUnifiedIdeograph7a3c': '\xe7\xa8\xbc',
'UnicodeCharacter0xf422': '\xef\x90\xa2',
'UnicodeCharacter0xf423': '\xef\x90\xa3',
'CanadianSyllabicsSwe': '\xe1\x93\xb6',
'LimbuLetterTa': '\xe1\xa4\x8b',
'UnicodeCharacter0xf426': '\xef\x90\xa6',
'UnicodeCharacter0xf427': '\xef\x90\xa7',
'CanadianSyllabicsSwi': '\xe1\x93\xb8',
'UnicodeCharacter0xf429': '\xef\x90\xa9',
'UnicodeCharacter0xe5d8': '\xee\x97\x98',
'CjkUnifiedIdeograph7a3a': '\xe7\xa8\xba',
'CanadianSyllabicsSwo': '\xe1\x93\xbc',
'CjkUnifiedIdeograph993c': '\xe9\xa4\xbc',
'CjkUnifiedIdeograph8752': '\xe8\x9d\x92',
'HangulSyllableByanj': '\xeb\xb1\x91',
'LatinCapitalLetterXWithDiaeresis': '\xe1\xba\x8c',
'CjkUnifiedIdeograph8753': '\xe8\x9d\x93',
'UnicodeCharacter0xfddf': '\xef\xb7\x9f',
'UnicodeCharacter0xe5ac': '\xee\x96\xac',
'CjkUnifiedIdeograph8750': '\xe8\x9d\x90',
'ArabicLigatureLamWithHahWithYehFinalForm': '\xef\xb6\x81',
'CjkUnifiedIdeograph6d2c': '\xe6\xb4\xac',
'CjkUnifiedIdeograph7517': '\xe7\x94\x97',
'CjkUnifiedIdeograph8756': '\xe8\x9d\x96',
'CjkUnifiedIdeograph7a3d': '\xe7\xa8\xbd',
'CjkUnifiedIdeograph6fd7': '\xe6\xbf\x97',
'CjkUnifiedIdeograph7418': '\xe7\x90\x98',
'CjkUnifiedIdeograph8701': '\xe8\x9c\x81',
'CjkUnifiedIdeograph8700': '\xe8\x9c\x80',
'CjkUnifiedIdeograph8703': '\xe8\x9c\x83',
'CjkUnifiedIdeograph8702': '\xe8\x9c\x82',
'CjkUnifiedIdeograph8705': '\xe8\x9c\x85',
'DevanagariSignVisarga': '\xe0\xa4\x83',
'CjkUnifiedIdeograph8707': '\xe8\x9c\x87',
'CjkUnifiedIdeograph8706': '\xe8\x9c\x86',
'CjkUnifiedIdeograph8709': '\xe8\x9c\x89',
'CjkUnifiedIdeograph8708': '\xe8\x9c\x88',
'CjkUnifiedIdeograph8755': '\xe8\x9d\x95',
'ArabicLigatureSheenWithHahFinalForm': '\xef\xb4\xa6',
'CjkUnifiedIdeograph53a1': '\xe5\x8e\xa1',
'CjkUnifiedIdeograph53a0': '\xe5\x8e\xa0',
'CjkUnifiedIdeograph53a3': '\xe5\x8e\xa3',
'CjkUnifiedIdeograph53a2': '\xe5\x8e\xa2',
'CjkUnifiedIdeograph53a5': '\xe5\x8e\xa5',
'CjkUnifiedIdeograph53a4': '\xe5\x8e\xa4',
'CjkUnifiedIdeograph53a7': '\xe5\x8e\xa7',
'CjkUnifiedIdeograph53a6': '\xe5\x8e\xa6',
'CjkUnifiedIdeograph53a9': '\xe5\x8e\xa9',
'CjkUnifiedIdeograph53a8': '\xe5\x8e\xa8',
'UnicodeCharacter0xdb9f': '\xed\xae\x9f',
'YiRadicalYit': '\xea\x92\xae',
'CircledIdeographFire': '\xe3\x8a\x8b',
'UnicodeCharacter0xf02a': '\xef\x80\xaa',
'GurmukhiLetterJa': '\xe0\xa8\x9c',
'CanadianSyllabicsCarrierHee': '\xe1\x90\xb6',
'HebrewAccentZaqefGadol': '\xd6\x95',
'UnicodeCharacter0xf8c0': '\xef\xa3\x80',
'HangulSyllablePyaj': '\xed\x8d\x9a',
'HebrewLetterFinalKafWithDagesh': '\xef\xac\xba',
'CjkUnifiedIdeograph5e64': '\xe5\xb9\xa4',
'CjkUnifiedIdeograph53aa': '\xe5\x8e\xaa',
'ThaiCharacterHoHip': '\xe0\xb8\xab',
'CjkUnifiedIdeograph53ac': '\xe5\x8e\xac',
'CjkUnifiedIdeograph53ab': '\xe5\x8e\xab',
'CjkUnifiedIdeograph53ae': '\xe5\x8e\xae',
'CjkUnifiedIdeograph53ad': '\xe5\x8e\xad',
'CjkUnifiedIdeograph53af': '\xe5\x8e\xaf',
'UnicodeCharacter0xefc0': '\xee\xbf\x80',
'PhagspaLetterDda': '\xea\xa1\xab',
'UnicodeCharacter0xf471': '\xef\x91\xb1',
'HangulSyllablePyan': '\xed\x8d\x88',
'CjkUnifiedIdeograph870a': '\xe8\x9c\x8a',
'CjkUnifiedIdeograph870c': '\xe8\x9c\x8c',
'CjkUnifiedIdeograph870b': '\xe8\x9c\x8b',
'CjkUnifiedIdeograph870e': '\xe8\x9c\x8e',
'CjkUnifiedIdeograph870d': '\xe8\x9c\x8d',
'CjkUnifiedIdeograph9f27': '\xe9\xbc\xa7',
'CjkUnifiedIdeograph870f': '\xe8\x9c\x8f',
'CombiningLongSolidusOverlay': '\xcc\xb8',
'UnicodeCharacter0xf477': '\xef\x91\xb7',
'CircledIdeographMoon': '\xe3\x8a\x8a',
'HangulSyllableCoss': '\xec\xb4\x9c',
'UnicodeCharacter0xf475': '\xef\x91\xb5',
'CjkUnifiedIdeograph658b': '\xe6\x96\x8b',
'SmallRightParenthesis': '\xef\xb9\x9a',
'HangulSyllablePyap': '\xed\x8d\x9e',
'UnicodeCharacter0xdb9c': '\xed\xae\x9c',
'CjkUnifiedIdeograph59b0': '\xe5\xa6\xb0',
'VaiSyllableVee': '\xea\x94\x8d',
'CyrillicSmallLetterKomiDzje': '\xd4\x87',
'CyrillicSmallLetterKoppa': '\xd2\x81',
'GreekCapitalLetterAlpha': '\xce\x91',
'CjkUnifiedIdeograph5098': '\xe5\x82\x98',
'UnicodeCharacter0xdb9b': '\xed\xae\x9b',
'TifinaghLetterYah': '\xe2\xb5\x80',
'CyrillicCapitalLetterQa': '\xd4\x9a',
'UnicodeCharacter0xe0ae': '\xee\x82\xae',
'CjkUnifiedIdeograph943e': '\xe9\x90\xbe',
'CjkUnifiedIdeograph470e': '\xe4\x9c\x8e',
'CjkUnifiedIdeograph470d': '\xe4\x9c\x8d',
'CjkUnifiedIdeograph470f': '\xe4\x9c\x8f',
'CjkUnifiedIdeograph470a': '\xe4\x9c\x8a',
'CjkUnifiedIdeograph943f': '\xe9\x90\xbf',
'CjkUnifiedIdeograph470c': '\xe4\x9c\x8c',
'CjkUnifiedIdeograph470b': '\xe4\x9c\x8b',
'CjkUnifiedIdeograph42af': '\xe4\x8a\xaf',
'TifinaghLetterYab': '\xe2\xb4\xb1',
'CjkUnifiedIdeograph6e59': '\xe6\xb9\x99',
'CjkUnifiedIdeograph5090': '\xe5\x82\x90',
'CjkUnifiedIdeograph65e3': '\xe6\x97\xa3',
'HangulSyllableSap': '\xec\x83\x86',
'UnicodeCharacter0xab3c': '\xea\xac\xbc',
'VaiSyllableWin': '\xea\x94\xa9',
'KayahLiVowelUe': '\xea\xa4\xa6',
'CjkUnifiedIdeograph705e': '\xe7\x81\x9e',
'GammaGamma': '\xe2\x84\xbe',
'CjkUnifiedIdeograph5096': '\xe5\x82\x96',
'CjkUnifiedIdeograph5097': '\xe5\x82\x97',
'HangulSyllableSab': '\xec\x82\xbd',
'HangulSyllableSac': '\xec\x83\x83',
'HangulSyllableSad': '\xec\x82\xb3',
'UnicodeCharacter0xab3f': '\xea\xac\xbf',
'HangulSyllableSag': '\xec\x82\xad',
'HangulSyllableSah': '\xec\x83\x87',
'HangulSyllableSaj': '\xec\x83\x82',
'TifinaghLetterYad': '\xe2\xb4\xb7',
'HangulSyllableSal': '\xec\x82\xb4',
'HangulSyllableSam': '\xec\x82\xbc',
'CjkUnifiedIdeograph7df7': '\xe7\xb7\xb7',
'UnicodeCharacter0xefc5': '\xee\xbf\x85',
'CjkUnifiedIdeograph572a': '\xe5\x9c\xaa',
'GeometricProportion': '\xe2\x88\xba',
'CjkUnifiedIdeograph4705': '\xe4\x9c\x85',
'CjkUnifiedIdeograph4704': '\xe4\x9c\x84',
'CjkUnifiedIdeograph4707': '\xe4\x9c\x87',
'CjkUnifiedIdeograph4706': '\xe4\x9c\x86',
'CjkUnifiedIdeograph4701': '\xe4\x9c\x81',
'CjkUnifiedIdeograph4700': '\xe4\x9c\x80',
'CjkUnifiedIdeograph4703': '\xe4\x9c\x83',
'CjkUnifiedIdeograph4702': '\xe4\x9c\x82',
'CjkUnifiedIdeograph572d': '\xe5\x9c\xad',
'CjkUnifiedIdeograph65e9': '\xe6\x97\xa9',
'CjkUnifiedIdeograph4709': '\xe4\x9c\x89',
'CjkUnifiedIdeograph4708': '\xe4\x9c\x88',
'HangulSyllableHinh': '\xed\x9e\x8e',
'CjkUnifiedIdeograph3df9': '\xe3\xb7\xb9',
'CjkUnifiedIdeograph3df8': '\xe3\xb7\xb8',
'HangulSyllableDdwibs': '\xeb\x9c\x82',
'UnicodeCharacter0xeb02': '\xee\xac\x82',
'CjkUnifiedIdeograph3959': '\xe3\xa5\x99',
'CjkUnifiedIdeograph3958': '\xe3\xa5\x98',
'UnicodeCharacter0xeb07': '\xee\xac\x87',
'UnicodeCharacter0xeb06': '\xee\xac\x86',
'CjkUnifiedIdeograph3955': '\xe3\xa5\x95',
'CjkUnifiedIdeograph3df0': '\xe3\xb7\xb0',
'CjkUnifiedIdeograph3957': '\xe3\xa5\x97',
'CjkUnifiedIdeograph3956': '\xe3\xa5\x96',
'CjkUnifiedIdeograph3951': '\xe3\xa5\x91',
'CjkUnifiedIdeograph3950': '\xe3\xa5\x90',
'CjkUnifiedIdeograph3953': '\xe3\xa5\x93',
'CjkUnifiedIdeograph3952': '\xe3\xa5\x92',
'HebrewLigatureYiddishDoubleVav': '\xd7\xb0',
'CjkUnifiedIdeograph4a32': '\xe4\xa8\xb2',
'CjkCompatibilityIdeographf959': '\xef\xa5\x99',
'ArabicLetterYehWithThreeDotsBelow': '\xdb\x91',
'RightwardsArrowWithPlusBelow': '\xe2\xa5\x85',
'CjkUnifiedIdeograph7245': '\xe7\x89\x85',
'CjkUnifiedIdeograph7244': '\xe7\x89\x84',
'CjkUnifiedIdeograph7247': '\xe7\x89\x87',
'CjkUnifiedIdeograph7246': '\xe7\x89\x86',
'CjkUnifiedIdeograph7241': '\xe7\x89\x81',
'CjkUnifiedIdeograph7240': '\xe7\x89\x80',
'RunicLetterOsO': '\xe1\x9a\xa9',
'CjkUnifiedIdeograph7242': '\xe7\x89\x82',
'CjkUnifiedIdeograph994d': '\xe9\xa5\x8d',
'CjkUnifiedIdeograph95d8': '\xe9\x97\x98',
'CjkUnifiedIdeograph994f': '\xe9\xa5\x8f',
'CjkUnifiedIdeograph4a31': '\xe4\xa8\xb1',
'UnicodeCharacter0xf5b1': '\xef\x96\xb1',
'CjkUnifiedIdeograph994a': '\xe9\xa5\x8a',
'CjkUnifiedIdeograph994b': '\xe9\xa5\x8b',
'CjkUnifiedIdeograph994c': '\xe9\xa5\x8c',
'HangulSyllableGgibs': '\xeb\x82\x8e',
'UnicodeCharacter0xf5b9': '\xef\x96\xb9',
'UnicodeCharacter0xabb9': '\xea\xae\xb9',
'CjkUnifiedIdeograph507d': '\xe5\x81\xbd',
'CjkUnifiedIdeograph507e': '\xe5\x81\xbe',
'CjkUnifiedIdeograph507f': '\xe5\x81\xbf',
'UnicodeCharacter0xe0ab': '\xee\x82\xab',
'CjkUnifiedIdeograph507a': '\xe5\x81\xba',
'CjkUnifiedIdeograph507b': '\xe5\x81\xbb',
'CjkUnifiedIdeograph507c': '\xe5\x81\xbc',
'ArabicLetterWaw': '\xd9\x88',
'UnicodeCharacter0xefc8': '\xee\xbf\x88',
'KangxiRadicalClan': '\xe2\xbd\x92',
'CjkUnifiedIdeograph724e': '\xe7\x89\x8e',
'OriyaDigitEight': '\xe0\xad\xae',
'CjkUnifiedIdeograph724f': '\xe7\x89\x8f',
'CjkUnifiedIdeograph724a': '\xe7\x89\x8a',
'UnicodeCharacter0xe549': '\xee\x95\x89',
'CjkUnifiedIdeograph724c': '\xe7\x89\x8c',
'CjkUnifiedIdeograph724b': '\xe7\x89\x8b',
'UnicodeCharacter0xeb0a': '\xee\xac\x8a',
'UnicodeCharacter0xeb0c': '\xee\xac\x8c',
'UnicodeCharacter0xeb0b': '\xee\xac\x8b',
'UnicodeCharacter0xeb0e': '\xee\xac\x8e',
'UnicodeCharacter0xeb0d': '\xee\xac\x8d',
'UnicodeCharacter0xeb0f': '\xee\xac\x8f',
'CjkUnifiedIdeograph395e': '\xe3\xa5\x9e',
'CjkUnifiedIdeograph395d': '\xe3\xa5\x9d',
'CruzeiroSign': '\xe2\x82\xa2',
'WhiteSmilingFace': '\xe2\x98\xba',
'CjkUnifiedIdeograph3dfe': '\xe3\xb7\xbe',
'CjkUnifiedIdeograph3dfd': '\xe3\xb7\xbd',
'CjkUnifiedIdeograph395c': '\xe3\xa5\x9c',
'CjkUnifiedIdeograph395b': '\xe3\xa5\x9b',
'CjkUnifiedIdeograph5074': '\xe5\x81\xb4',
'CjkUnifiedIdeograph5075': '\xe5\x81\xb5',
'CjkUnifiedIdeograph5076': '\xe5\x81\xb6',
'CjkUnifiedIdeograph5077': '\xe5\x81\xb7',
'CjkUnifiedIdeograph5070': '\xe5\x81\xb0',
'CjkUnifiedIdeograph5071': '\xe5\x81\xb1',
'CjkUnifiedIdeograph5072': '\xe5\x81\xb2',
'CjkUnifiedIdeograph5073': '\xe5\x81\xb3',
'SupersetOfAboveNotEqualTo': '\xe2\xab\x8c',
'HiraganaLetterSmallKa': '\xe3\x82\x95',
'CjkUnifiedIdeograph5078': '\xe5\x81\xb8',
'ArabicLetterRnoon': '\xda\xbb',
'HiraganaLetterSmallKe': '\xe3\x82\x96',
'CjkUnifiedIdeograph9944': '\xe9\xa5\x84',
'CjkUnifiedIdeograph9945': '\xe9\xa5\x85',
'CjkUnifiedIdeograph9946': '\xe9\xa5\x86',
'CjkUnifiedIdeograph9947': '\xe9\xa5\x87',
'CjkUnifiedIdeograph9940': '\xe9\xa5\x80',
'CjkUnifiedIdeograph9941': '\xe9\xa5\x81',
'CjkUnifiedIdeograph9942': '\xe9\xa5\x82',
'CjkUnifiedIdeograph9943': '\xe9\xa5\x83',
'ArabicSignMisra': '\xd8\x8f',
'ArabicLigatureYehWithKhahIsolatedForm': '\xef\xb1\x97',
'CjkUnifiedIdeograph80a0': '\xe8\x82\xa0',
'CjkUnifiedIdeograph9949': '\xe9\xa5\x89',
'CjkUnifiedIdeograph5ba4': '\xe5\xae\xa4',
'CjkUnifiedIdeograph5ba5': '\xe5\xae\xa5',
'CjkUnifiedIdeograph5ba6': '\xe5\xae\xa6',
'CjkUnifiedIdeograph5ba7': '\xe5\xae\xa7',
'CjkUnifiedIdeograph5ba0': '\xe5\xae\xa0',
'CjkUnifiedIdeograph5ba1': '\xe5\xae\xa1',
'CjkUnifiedIdeograph5ba2': '\xe5\xae\xa2',
'SinhalaLetterKantajaNaasikyaya': '\xe0\xb6\x9e',
'ArabicKasratanIsolatedForm': '\xef\xb9\xb4',
'CjkUnifiedIdeograph5ba8': '\xe5\xae\xa8',
'CjkUnifiedIdeograph5ba9': '\xe5\xae\xa9',
'UnicodeCharacter0xf1d5': '\xef\x87\x95',
'HangulSyllableKweolt': '\xed\x80\x89',
'CjkUnifiedIdeograph667c': '\xe6\x99\xbc',
'CjkUnifiedIdeograph667b': '\xe6\x99\xbb',
'CjkUnifiedIdeograph667a': '\xe6\x99\xba',
'CjkUnifiedIdeograph667f': '\xe6\x99\xbf',
'CjkUnifiedIdeograph667e': '\xe6\x99\xbe',
'CjkUnifiedIdeograph667d': '\xe6\x99\xbd',
'CjkUnifiedIdeograph59ba': '\xe5\xa6\xba',
'WhiteSpadeSuit': '\xe2\x99\xa4',
'MongolianLetterTodoBa': '\xe1\xa1\x8b',
'HangulSyllableKweolh': '\xed\x80\x8b',
'HangulSyllableTyes': '\xed\x86\x97',
'HangulSyllableTyep': '\xed\x86\x9e',
'HangulSyllableJjyeobs': '\xec\xaa\x96',
'HangulSyllableTyet': '\xed\x86\x9d',
'HangulSyllableTyek': '\xed\x86\x9c',
'HangulSyllableTyej': '\xed\x86\x9a',
'HangulSyllableTyeh': '\xed\x86\x9f',
'CjkUnifiedIdeograph7c22': '\xe7\xb0\xa2',
'HangulSyllableTyen': '\xed\x86\x88',
'EthiopicSyllableDwa': '\xe1\x8b\xb7',
'HangulSyllableTyel': '\xed\x86\x8c',
'HangulSyllableTyec': '\xed\x86\x9b',
'HangulSyllableGaenh': '\xea\xb0\xa2',
'HangulSyllableGaenj': '\xea\xb0\xa1',
'HangulSyllableTyeg': '\xed\x86\x85',
'HangulSyllableTyed': '\xed\x86\x8b',
'CjkUnifiedIdeograph6673': '\xe6\x99\xb3',
'CjkUnifiedIdeograph6672': '\xe6\x99\xb2',
'CjkUnifiedIdeograph6671': '\xe6\x99\xb1',
'CjkUnifiedIdeograph42a0': '\xe4\x8a\xa0',
'CjkUnifiedIdeograph6677': '\xe6\x99\xb7',
'BalineseMusicalSymbolRighthandClosedTak': '\xe1\xad\xb7',
'CjkUnifiedIdeograph6675': '\xe6\x99\xb5',
'CjkUnifiedIdeograph6674': '\xe6\x99\xb4',
'DoublelineSlantedEqualToOrLessthan': '\xe2\xaa\x9b',
'CjkUnifiedIdeograph42a7': '\xe4\x8a\xa7',
'CjkUnifiedIdeograph6679': '\xe6\x99\xb9',
'CjkUnifiedIdeograph6678': '\xe6\x99\xb8',
'TibetanLetterA': '\xe0\xbd\xa8',
'CjkUnifiedIdeograph5bad': '\xe5\xae\xad',
'CjkUnifiedIdeograph5bae': '\xe5\xae\xae',
'CjkUnifiedIdeograph5baf': '\xe5\xae\xaf',
'CjkUnifiedIdeograph5baa': '\xe5\xae\xaa',
'CjkUnifiedIdeograph5bab': '\xe5\xae\xab',
'CjkUnifiedIdeograph5bac': '\xe5\xae\xac',
'HangulSyllableHaegg': '\xed\x95\xb6',
'HangulSyllableBbuj': '\xeb\xbf\xa2',
'HangulSyllableKweolb': '\xed\x80\x87',
'CyrillicCapitalLetterSha': '\xd0\xa8',
'LongDashFromLeftMemberOfDoubleVertical': '\xe2\xab\xa6',
'UnicodeCharacter0xeec6': '\xee\xbb\x86',
'IdeographicVariationIndicator': '\xe3\x80\xbe',
'GreekLetterKoppa': '\xcf\x9e',
'CjkUnifiedIdeograph406f': '\xe4\x81\xaf',
'BenzeneRing': '\xe2\x8c\xac',
'HangulSyllableHaem': '\xed\x96\x84',
'CjkUnifiedIdeograph7c21': '\xe7\xb0\xa1',
'CjkUnifiedIdeograph5e6f': '\xe5\xb9\xaf',
'KannadaLetterGa': '\xe0\xb2\x97',
'CjkUnifiedIdeograph3e68': '\xe3\xb9\xa8',
'CjkUnifiedIdeograph3e69': '\xe3\xb9\xa9',
'CjkUnifiedIdeograph3e64': '\xe3\xb9\xa4',
'CjkUnifiedIdeograph3e65': '\xe3\xb9\xa5',
'CjkUnifiedIdeograph3e66': '\xe3\xb9\xa6',
'CjkUnifiedIdeograph3e67': '\xe3\xb9\xa7',
'CjkUnifiedIdeograph3e60': '\xe3\xb9\xa0',
'CjkUnifiedIdeograph3e61': '\xe3\xb9\xa1',
'CjkUnifiedIdeograph3e62': '\xe3\xb9\xa2',
'CjkUnifiedIdeograph3e63': '\xe3\xb9\xa3',
'EthiopicSyllableGoa': '\xe1\x8c\x8f',
'EthiopicSyllableNya': '\xe1\x8a\x98',
'UnicodeCharacter0xe948': '\xee\xa5\x88',
'EthiopicSyllableNye': '\xe1\x8a\x9d',
'CjkUnifiedIdeograph5e7d': '\xe5\xb9\xbd',
'EthiopicSyllableNyi': '\xe1\x8a\x9a',
'HangulSyllableHaeh': '\xed\x96\x8f',
'EthiopicSyllableNyo': '\xe1\x8a\x9e',
'HangulSyllableHiss': '\xed\x9e\x9c',
'GreekSmallLetterDigamma': '\xcf\x9d',
'HangulSyllableDanj': '\xeb\x8b\xa9',
'IdeographicTelegraphSymbolForDayThirteen': '\xe3\x8f\xac',
'EthiopicSyllableNyu': '\xe1\x8a\x99',
'ArabicLigatureLamWithMeemWithHahInitialForm': '\xef\xb6\x88',
'EmptySetWithLeftArrowAbove': '\xe2\xa6\xb4',
'HangulSyllableGgeolh': '\xea\xbb\x8b',
'GreekCapitalLetterEpsilonWithDasiaAndVaria': '\xe1\xbc\x9b',
'ArabicLigatureLamWithHahWithMeemInitialForm': '\xef\xb6\xb5',
'HangulSyllableBilb': '\xeb\xb9\x8f',
'ArabicLetterYehMedialForm': '\xef\xbb\xb4',
'CjkUnifiedIdeograph3e6d': '\xe3\xb9\xad',
'GurmukhiLetterTtha': '\xe0\xa8\xa0',
'CjkUnifiedIdeograph3e6f': '\xe3\xb9\xaf',
'CjkUnifiedIdeograph874d': '\xe8\x9d\x8d',
'CjkUnifiedIdeograph3e6a': '\xe3\xb9\xaa',
'CjkUnifiedIdeograph3e6b': '\xe3\xb9\xab',
'CjkUnifiedIdeograph3e6c': '\xe3\xb9\xac',
'LimbuSignSai': '\xe1\xa4\xbb',
'HangulChoseongKiyeok': '\xe1\x84\x80',
'HangulSyllableBilg': '\xeb\xb9\x8d',
'CjkRadicalCsimplifiedTurtle': '\xe2\xbb\xb3',
'YiSyllableQyx': '\xea\x90\x93',
'LatinCapitalLetterRWithAcute': '\xc5\x94',
'BoxDrawingsDownHeavyAndUpHorizontalLight': '\xe2\x95\x81',
'CjkUnifiedIdeograph7c24': '\xe7\xb0\xa4',
'YiSyllableQyp': '\xea\x90\x95',
'YiSyllableQyr': '\xea\x90\x97',
'YiSyllableQyt': '\xea\x90\x92',
'sqrt': '\xe2\x88\x9a',
'HangulSyllableTyilh': '\xed\x8b\xa3',
'CjkUnifiedIdeograph7b49': '\xe7\xad\x89',
'UnicodeCharacter0xdbee': '\xed\xaf\xae',
'CjkUnifiedIdeograph95db': '\xe9\x97\x9b',
'UnicodeCharacter0xdbed': '\xed\xaf\xad',
'HangulSyllableDdaep': '\xeb\x95\xa6',
'ReversedAngle': '\xe2\xa6\xa3',
'LatinSmallLetterTaillessPhi': '\xe2\xb1\xb7',
'UnicodeCharacter0xf045': '\xef\x81\x85',
'BottomTortoiseShellBracket': '\xe2\x8f\xa1',
'CjkUnifiedIdeograph7b48': '\xe7\xad\x88',
'HangulSyllableBbul': '\xeb\xbf\x94',
'HangulSyllablePyeols': '\xed\x8f\x80',
'HangulSyllableNwaess': '\xeb\x87\x84',
'HangulSyllableBilm': '\xeb\xb9\x8e',
'HangulSyllableJjip': '\xec\xb0\xa6',
'CjkUnifiedIdeograph6f6a': '\xe6\xbd\xaa',
'CjkUnifiedIdeograph6f6b': '\xe6\xbd\xab',
'CjkUnifiedIdeograph6f6c': '\xe6\xbd\xac',
'CjkUnifiedIdeograph6f6d': '\xe6\xbd\xad',
'CjkUnifiedIdeograph6f6e': '\xe6\xbd\xae',
'CjkUnifiedIdeograph6f6f': '\xe6\xbd\xaf',
'CanadianSyllabicsNgaa': '\xe1\x96\x94',
'HangulSyllableHaes': '\xed\x96\x87',
'HangulSyllableBilp': '\xeb\xb9\x92',
'CanadianSyllabicsNgai': '\xe1\x99\xb0',
'CanadianSyllabicsMwa': '\xe1\x92\xb6',
'UnicodeCharacter0xda03': '\xed\xa8\x83',
'HangulSyllablePyeolp': '\xed\x8f\x82',
'CanadianSyllabicsMwe': '\xe1\x92\xac',
'CjkUnifiedIdeograph7bd9': '\xe7\xaf\x99',
'CanadianSyllabicsMwi': '\xe1\x92\xae',
'CanadianSyllabicsMwo': '\xe1\x92\xb2',
'MyanmarDigitTwo': '\xe1\x81\x82',
'CjkUnifiedIdeograph7bd8': '\xe7\xaf\x98',
'HangulSyllablePwilg': '\xed\x93\x95',
'ArabicLigatureUighurKirghizYehWithHamzaAboveWithAlefMaksuraIsolatedForm': '\xef\xaf\xb9',
'HangulSyllablePwilb': '\xed\x93\x97',
'HangulSyllableGwib': '\xea\xb7\x91',
'HangulSyllablePwilm': '\xed\x93\x96',
'HangulSyllablePwilh': '\xed\x93\x9b',
'CjkUnifiedIdeograph7f45': '\xe7\xbd\x85',
'UnicodeCharacter0xe9f0': '\xee\xa7\xb0',
'HangulSyllablePwilt': '\xed\x93\x99',
'UnicodeCharacter0xebdc': '\xee\xaf\x9c',
'HangulSyllablePwils': '\xed\x93\x98',
'HangulSyllablePwilp': '\xed\x93\x9a',
'UnicodeCharacter0xe9f1': '\xee\xa7\xb1',
'HeavyDoubleTurnedCommaQuotationMarkOrnament': '\xe2\x9d\x9d',
'HangulSyllableByeh': '\xeb\xb3\xb3',
'HangulSyllableByek': '\xeb\xb3\xb0',
'MalayalamLetterLlla': '\xe0\xb4\xb4',
'HangulSyllableByem': '\xeb\xb3\xa8',
'HangulSyllableByel': '\xeb\xb3\xa0',
'HangulSyllableByeo': '\xeb\xb2\xbc',
'HangulSyllableDdyugg': '\xeb\x9c\x8e',
'CjkUnifiedIdeograph7bd1': '\xe7\xaf\x91',
'HangulSyllableByec': '\xeb\xb3\xaf',
'HangulSyllableDdyass': '\xeb\x95\xbc',
'HangulSyllableByed': '\xeb\xb3\x9f',
'HangulSyllableByeg': '\xeb\xb3\x99',
'HangulLetterNieunsios': '\xe3\x85\xa7',
'BoxDrawingsUpDoubleAndRightSingle': '\xe2\x95\x99',
'CjkUnifiedIdeograph6f61': '\xe6\xbd\xa1',
'CjkUnifiedIdeograph6f62': '\xe6\xbd\xa2',
'CjkUnifiedIdeograph6f63': '\xe6\xbd\xa3',
'CjkUnifiedIdeograph6f64': '\xe6\xbd\xa4',
'CjkUnifiedIdeograph6f65': '\xe6\xbd\xa5',
'CjkUnifiedIdeograph6f66': '\xe6\xbd\xa6',
'CjkUnifiedIdeograph6f67': '\xe6\xbd\xa7',
'CjkUnifiedIdeograph6f68': '\xe6\xbd\xa8',
'CjkUnifiedIdeograph6f69': '\xe6\xbd\xa9',
'HangulSyllableByes': '\xeb\xb3\xab',
'UnicodeCharacter0xe9f7': '\xee\xa7\xb7',
'HangulSyllableByet': '\xeb\xb3\xb1',
'CjkUnifiedIdeograph4f5c': '\xe4\xbd\x9c',
'CanadianSyllabicsCarrierSe': '\xe1\x99\x90',
'CjkUnifiedIdeograph4f5a': '\xe4\xbd\x9a',
'CanadianSyllabicsCarrierSa': '\xe1\x99\x93',
'CjkUnifiedIdeograph4f5e': '\xe4\xbd\x9e',
'CjkUnifiedIdeograph4f5d': '\xe4\xbd\x9d',
'CanadianSyllabicsCarrierSo': '\xe1\x99\x8f',
'CanadianSyllabicsCarrierSh': '\xe1\x99\x9a',
'CanadianSyllabicsCarrierSi': '\xe1\x99\x92',
'CjkUnifiedIdeograph98e2': '\xe9\xa3\xa2',
'CanadianSyllabicsCarrierSu': '\xe1\x99\x8e',
'CjkUnifiedIdeograph98e0': '\xe9\xa3\xa0',
'CjkUnifiedIdeograph98e1': '\xe9\xa3\xa1',
'HorizontalMaleWithStrokeSign': '\xe2\x9a\xa9',
'CjkUnifiedIdeograph98e7': '\xe9\xa3\xa7',
'CjkUnifiedIdeograph98e4': '\xe9\xa3\xa4',
'CjkUnifiedIdeograph98e5': '\xe9\xa3\xa5',
'CjkUnifiedIdeograph98e8': '\xe9\xa3\xa8',
'CjkUnifiedIdeograph98e9': '\xe9\xa3\xa9',
'Nu': '\xce\x9d',
'UnicodeCharacter0xeda8': '\xee\xb6\xa8',
'UnicodeCharacter0x558': '\xd5\x98',
'HangulSyllableReogs': '\xeb\x9f\xaf',
'CjkUnifiedIdeograph56cb': '\xe5\x9b\x8b',
'CjkUnifiedIdeograph56cc': '\xe5\x9b\x8c',
'UnicodeCharacter0x557': '\xd5\x97',
'CjkUnifiedIdeograph56ca': '\xe5\x9b\x8a',
'CjkUnifiedIdeograph56cf': '\xe5\x9b\x8f',
'HangulSyllablePyeolt': '\xed\x8f\x81',
'CjkUnifiedIdeograph56cd': '\xe5\x9b\x8d',
'CjkUnifiedIdeograph56ce': '\xe5\x9b\x8e',
'HangulSyllableReogg': '\xeb\x9f\xae',
'CanadianSyllabicsTlho': '\xe1\x96\x88',
'HangulSyllableTwass': '\xed\x87\x90',
'CanadianSyllabicsTlhi': '\xe1\x96\x87',
'CanadianSyllabicsTlhe': '\xe1\x96\x86',
'CanadianSyllabicsTlha': '\xe1\x96\x89',
'CjkUnifiedIdeograph98eb': '\xe9\xa3\xab',
'CjkUnifiedIdeograph98ec': '\xe9\xa3\xac',
'KatakanaLetterHe': '\xe3\x83\x98',
'CjkUnifiedIdeograph98ef': '\xe9\xa3\xaf',
'LatinSmallLetterZWithCircumflex': '\xe1\xba\x91',
'CjkUnifiedIdeograph98ed': '\xe9\xa3\xad',
'KatakanaLetterHa': '\xe3\x83\x8f',
'KatakanaLetterHo': '\xe3\x83\x9b',
'KatakanaLetterHi': '\xe3\x83\x92',
'CjkUnifiedIdeograph4f53': '\xe4\xbd\x93',
'CjkUnifiedIdeograph4f52': '\xe4\xbd\x92',
'CjkUnifiedIdeograph4f51': '\xe4\xbd\x91',
'CjkUnifiedIdeograph4f50': '\xe4\xbd\x90',
'MyanmarLetterYa': '\xe1\x80\x9a',
'TibetanLetterDzha': '\xe0\xbd\x9c',
'CjkUnifiedIdeograph4f55': '\xe4\xbd\x95',
'CjkUnifiedIdeograph4f54': '\xe4\xbd\x94',
'YiSyllableMex': '\xea\x83\x81',
'CjkUnifiedIdeograph4f59': '\xe4\xbd\x99',
'CjkUnifiedIdeograph4f58': '\xe4\xbd\x98',
'ArabicLetterLamWithThreeDotsBelow': '\xda\xb8',
'FullwidthDigitNine': '\xef\xbc\x99',
'CyrillicSmallLetterIotifiedE': '\xd1\xa5',
'CjkUnifiedIdeograph7b41': '\xe7\xad\x81',
'CjkUnifiedIdeograph7f44': '\xe7\xbd\x84',
'SylotiNagriLetterTto': '\xea\xa0\x90',
'UnicodeCharacter0xeda4': '\xee\xb6\xa4',
'HangulLetterSsangpieup': '\xe3\x85\x83',
'UnicodeCharacter0xf6a4': '\xef\x9a\xa4',
'HangulSyllableJyuss': '\xec\xa6\x80',
'ArabicLetterSeenMedialForm': '\xef\xba\xb4',
'CjkUnifiedIdeograph56c8': '\xe5\x9b\x88',
'CjkUnifiedIdeograph56c9': '\xe5\x9b\x89',
'HangulSyllableRoelg': '\xeb\xa2\xb9',
'CjkUnifiedIdeograph56c2': '\xe5\x9b\x82',
'CjkUnifiedIdeograph56c3': '\xe5\x9b\x83',
'CjkUnifiedIdeograph56c0': '\xe5\x9b\x80',
'CjkUnifiedIdeograph56c1': '\xe5\x9b\x81',
'CjkUnifiedIdeograph56c6': '\xe5\x9b\x86',
'CjkUnifiedIdeograph56c7': '\xe5\x9b\x87',
'CjkUnifiedIdeograph56c4': '\xe5\x9b\x84',
'CjkUnifiedIdeograph56c5': '\xe5\x9b\x85',
'UnicodeCharacter0xdbe1': '\xed\xaf\xa1',
'CjkUnifiedIdeograph7b40': '\xe7\xad\x80',
'SaurashtraSignVirama': '\xea\xa3\x84',
'HangulSyllableJyed': '\xec\xa1\x9b',
'HangulSyllableJyeg': '\xec\xa1\x95',
'UnicodeCharacter0xdbe0': '\xed\xaf\xa0',
'HangulSyllableJyeh': '\xec\xa1\xaf',
'HangulSyllableJyek': '\xec\xa1\xac',
'HangulSyllableJyej': '\xec\xa1\xaa',
'HangulSyllableJyem': '\xec\xa1\xa4',
'HangulSyllableJyel': '\xec\xa1\x9c',
'SylotiNagriSignAnusvara': '\xea\xa0\x8b',
'GeorgianLetterRae': '\xe1\x83\xa0',
'CjkUnifiedIdeograph6ebc': '\xe6\xba\xbc',
'CjkUnifiedIdeograph6ebb': '\xe6\xba\xbb',
'CjkUnifiedIdeograph6eba': '\xe6\xba\xba',
'CjkUnifiedIdeograph6ebf': '\xe6\xba\xbf',
'CjkUnifiedIdeograph6ebe': '\xe6\xba\xbe',
'CjkUnifiedIdeograph62ba': '\xe6\x8a\xba',
'ThaiDigitThree': '\xe0\xb9\x93',
'HangulJungseongYue': '\xe1\x86\x90',
'UnicodeCharacter0xe3a4': '\xee\x8e\xa4',
'MyanmarVowelSignShanEAbove': '\xe1\x82\x85',
'CjkUnifiedIdeograph62bf': '\xe6\x8a\xbf',
'CircledNumberThirty': '\xe3\x89\x9a',
'CjkUnifiedIdeograph4068': '\xe4\x81\xa8',
'UnicodeCharacter0xe3a3': '\xee\x8e\xa3',
'CjkUnifiedIdeograph6eb3': '\xe6\xba\xb3',
'CjkUnifiedIdeograph6eb2': '\xe6\xba\xb2',
'CjkUnifiedIdeograph6eb1': '\xe6\xba\xb1',
'CjkUnifiedIdeograph6eb0': '\xe6\xba\xb0',
'CjkUnifiedIdeograph6eb7': '\xe6\xba\xb7',
'CjkUnifiedIdeograph6eb6': '\xe6\xba\xb6',
'CjkUnifiedIdeograph6eb5': '\xe6\xba\xb5',
'CjkUnifiedIdeograph6eb4': '\xe6\xba\xb4',
'HangulSyllableCeun': '\xec\xb8\xa4',
'CjkUnifiedIdeograph6eb9': '\xe6\xba\xb9',
'CjkUnifiedIdeograph6eb8': '\xe6\xba\xb8',
'HangulSyllableSugs': '\xec\x88\x9b',
'UnicodeCharacter0xeeee': '\xee\xbb\xae',
'SlantedEqualToOrLessthan': '\xe2\xaa\x95',
'PhagspaLetterTtha': '\xea\xa1\xaa',
'HangulSyllableSugg': '\xec\x88\x9a',
'GreekSmallLetterAlphaWithDasiaAndVaria': '\xe1\xbc\x83',
'GlagoliticSmallLetterSha': '\xe2\xb1\x8e',
'CaretInsertionPoint': '\xe2\x81\x81',
'UnicodeCharacter0xdf76': '\xed\xbd\xb6',
'CjkUnifiedIdeograph92ae': '\xe9\x8a\xae',
'CjkUnifiedIdeograph7ed8': '\xe7\xbb\x98',
'LeftSquareBracketWithQuill': '\xe2\x81\x85',
'CjkUnifiedIdeograph7ed2': '\xe7\xbb\x92',
'CjkUnifiedIdeograph7ed3': '\xe7\xbb\x93',
'CjkUnifiedIdeograph7ed0': '\xe7\xbb\x90',
'CjkUnifiedIdeograph7ed1': '\xe7\xbb\x91',
'CjkUnifiedIdeograph7ed6': '\xe7\xbb\x96',
'CjkUnifiedIdeograph7ed7': '\xe7\xbb\x97',
'CjkUnifiedIdeograph7ed4': '\xe7\xbb\x94',
'CjkUnifiedIdeograph7ed5': '\xe7\xbb\x95',
'GeorgianCapitalLetterDon': '\xe1\x82\xa3',
'HangulSyllableByilh': '\xeb\xb8\xb7',
'HexagramForDeliverance': '\xe4\xb7\xa7',
'HangulSyllableByilm': '\xeb\xb8\xb2',
'HangulSyllableByilb': '\xeb\xb8\xb3',
'CjkUnifiedIdeograph854f': '\xe8\x95\x8f',
'CjkUnifiedIdeograph854e': '\xe8\x95\x8e',
'CjkUnifiedIdeograph854d': '\xe8\x95\x8d',
'CjkUnifiedIdeograph854c': '\xe8\x95\x8c',
'CjkUnifiedIdeograph854b': '\xe8\x95\x8b',
'CjkUnifiedIdeograph854a': '\xe8\x95\x8a',
'HangulSyllableBbwanh': '\xeb\xbd\xa2',
'BoxDrawingsHeavyQuadrupleDashHorizontal': '\xe2\x94\x89',
'HangulSyllableBbwanj': '\xeb\xbd\xa1',
'ArabicLigatureKhahWithMeemIsolatedForm': '\xef\xb0\x9b',
'SquarePaAmps': '\xe3\x8e\x80',
'DirectCurrentSymbolFormTwo': '\xe2\x8e\x93',
'HangulSyllableSsoebs': '\xec\x90\xbe',
'HangulSyllablePyeolm': '\xed\x8e\xbe',
'ModifierLetterRhoticHook': '\xcb\x9e',
'HangulSyllableSsuh': '\xec\x91\xbf',
'LatinCapitalLetterXWithDotAbove': '\xe1\xba\x8a',
'GreekSmallLetterOmegaWithDasiaAndPerispomeni': '\xe1\xbd\xa7',
'HangulSyllableSsuk': '\xec\x91\xbc',
'HangulSyllableSsul': '\xec\x91\xac',
'HangulSyllableSsum': '\xec\x91\xb4',
'CjkUnifiedIdeograph8549': '\xe8\x95\x89',
'CjkUnifiedIdeograph8548': '\xe8\x95\x88',
'CherokeeLetterNah': '\xe1\x8f\x80',
'CjkUnifiedIdeograph8546': '\xe8\x95\x86',
'CjkUnifiedIdeograph8545': '\xe8\x95\x85',
'CjkUnifiedIdeograph8544': '\xe8\x95\x84',
'CjkUnifiedIdeograph8543': '\xe8\x95\x83',
'CjkUnifiedIdeograph8542': '\xe8\x95\x82',
'CjkUnifiedIdeograph8541': '\xe8\x95\x81',
'CjkUnifiedIdeograph8540': '\xe8\x95\x80',
'HangulSyllableByaess': '\xeb\xb1\xbc',
'HangulSyllableKis': '\xed\x82\xb7',
'HangulSyllableMweobs': '\xeb\xad\xa2',
'CjkUnifiedIdeograph7edb': '\xe7\xbb\x9b',
'CjkUnifiedIdeograph7edc': '\xe7\xbb\x9c',
'CjkUnifiedIdeograph7eda': '\xe7\xbb\x9a',
'CjkUnifiedIdeograph7edf': '\xe7\xbb\x9f',
'CjkUnifiedIdeograph7edd': '\xe7\xbb\x9d',
'CjkUnifiedIdeograph7ede': '\xe7\xbb\x9e',
'ArmenianCapitalLetterZhe': '\xd4\xba',
'UnicodeCharacter0xe394': '\xee\x8e\x94',
'SubscriptEqualsSign': '\xe2\x82\x8c',
'UnicodeCharacter0xed03': '\xee\xb4\x83',
'UnicodeCharacter0xe6e3': '\xee\x9b\xa3',
'HangulSyllableDdeog': '\xeb\x96\xa1',
'HanunooLetterMa': '\xe1\x9c\xab',
'HangulSyllableNaess': '\xeb\x83\x88',
'CjkUnifiedIdeograph8099': '\xe8\x82\x99',
'CjkUnifiedIdeograph8098': '\xe8\x82\x98',
'KannadaSignVirama': '\xe0\xb3\x8d',
'UnicodeCharacter0xab8c': '\xea\xae\x8c',
'YiSyllableJiep': '\xea\x8f\xa7',
'CjkUnifiedIdeograph8091': '\xe8\x82\x91',
'CjkUnifiedIdeograph8090': '\xe8\x82\x90',
'CjkUnifiedIdeograph8093': '\xe8\x82\x93',
'CombiningGreekYpogegrammeni': '\xcd\x85',
'CjkUnifiedIdeograph8095': '\xe8\x82\x95',
'CjkUnifiedIdeograph8094': '\xe8\x82\x94',
'CjkUnifiedIdeograph8097': '\xe8\x82\x97',
'CjkUnifiedIdeograph8096': '\xe8\x82\x96',
'beta': '\xce\xb2',
'LatinCapitalLetterSharpS': '\xe1\xba\x9e',
'CjkUnifiedIdeograph8834': '\xe8\xa0\xb4',
'UnicodeCharacter0xda78': '\xed\xa9\xb8',
'CjkCompatibilityIdeographf945': '\xef\xa5\x85',
'UnicodeCharacter0xf286': '\xef\x8a\x86',
'ArabicLigatureSadWithMeemIsolatedForm': '\xef\xb0\xa1',
'SymbolForEndOfTransmission': '\xe2\x90\x84',
'GlagoliticSmallLetterIzhitsa': '\xe2\xb1\x9b',
'UnicodeCharacter0xdd58': '\xed\xb5\x98',
'HangulSyllableYalt': '\xec\x96\x89',
'SuperscriptMinus': '\xe2\x81\xbb',
'_a': '\xe2\x82\x90',
'UnicodeCharacter0xf5ed': '\xef\x97\xad',
'YiSyllableQyrx': '\xea\x90\x96',
'CjkUnifiedIdeograph6e93': '\xe6\xba\x93',
'UnicodeCharacter0xf5ef': '\xef\x97\xaf',
'HangulSyllableDyugg': '\xeb\x93\x82',
'FronttiltedShadowedWhiteRightwardsArrow': '\xe2\x9e\xac',
'ArabicTripleDotPunctuationMark': '\xd8\x9e',
'HangulSyllableDdeob': '\xeb\x96\xb1',
'KhmerSymbolPathamasat': '\xe1\xa7\xa0',
'CjkUnifiedIdeograph809a': '\xe8\x82\x9a',
'CjkUnifiedIdeograph809c': '\xe8\x82\x9c',
'CjkUnifiedIdeograph809b': '\xe8\x82\x9b',
'CjkUnifiedIdeograph809e': '\xe8\x82\x9e',
'CjkUnifiedIdeograph809d': '\xe8\x82\x9d',
'CjkUnifiedIdeograph809f': '\xe8\x82\x9f',
'CombiningCyrillicLetterDe': '\xe2\xb7\xa3',
'UnicodeCharacter0xe3a9': '\xee\x8e\xa9',
'UnicodeCharacter0xf5ec': '\xef\x97\xac',
'CjkUnifiedIdeograph7080': '\xe7\x82\x80',
'HangulSyllableDdeoc': '\xeb\x96\xb7',
'CjkUnifiedIdeograph9903': '\xe9\xa4\x83',
'HangulSyllableNweobs': '\xeb\x88\xb2',
'ModifierLetterHighDottedToneBar': '\xea\x9c\x89',
'UnicodeCharacter0xed44': '\xee\xb5\x84',
'YiSyllableJiet': '\xea\x8f\xa4',
'LatinCapitalLetterUWithMacron': '\xc5\xaa',
'HangulSyllableRwing': '\xeb\xa5\x91',
'HangulChoseongSiosrieul': '\xe1\x84\xb0',
'UnicodeCharacter0xf8ca': '\xef\xa3\x8a',
'BoxDrawingsUpSingleAndLeftDouble': '\xe2\x95\x9b',
'TibetanMarkGterTsheg': '\xe0\xbc\x94',
'UnicodeCharacter0xf8cf': '\xef\xa3\x8f',
'YiSyllableJi': '\xea\x8f\xa2',
'HangulSyllableDyass': '\xeb\x8c\xb0',
'ArabicLetterBeehFinalForm': '\xef\xad\x93',
'YiSyllableJo': '\xea\x8f\xae',
'TibetanSignRdelDkarGcig': '\xe0\xbc\x9a',
'HangulSyllableDwalm': '\xeb\x8f\xaa',
'SquareMairu': '\xe3\x8d\x84',
'YiSyllableJy': '\xea\x8f\xb8',
'UnicodeCharacter0xe94f': '\xee\xa5\x8f',
'LepchaLetterKha': '\xe1\xb0\x82',
'UnicodeCharacter0xe94d': '\xee\xa5\x8d',
'UnicodeCharacter0xe94e': '\xee\xa5\x8e',
'UnicodeCharacter0xe94b': '\xee\xa5\x8b',
'Scruple': '\xe2\x84\x88',
'TamilLetterLa': '\xe0\xae\xb2',
'HangulSyllableHye': '\xed\x98\x9c',
'HangulSyllableDyonj': '\xeb\x90\xb9',
'HangulSyllableTwit': '\xed\x8a\x99',
'UnicodeCharacter0xde65': '\xed\xb9\xa5',
'CjkUnifiedIdeograph7e1a': '\xe7\xb8\x9a',
'HangulSyllableCyelh': '\xec\xb3\xbb',
'CjkUnifiedIdeograph751e': '\xe7\x94\x9e',
'ArmenianCapitalLetterJheh': '\xd5\x8b',
'CjkUnifiedIdeograph7e1c': '\xe7\xb8\x9c',
'HangulSyllableGgyeh': '\xea\xbc\xab',
'CjkUnifiedIdeograph7e1b': '\xe7\xb8\x9b',
'HangulSyllableCyelm': '\xec\xb3\xb6',
'HangulSyllableSswaelh': '\xec\x90\x9f',
'UnicodeCharacter0xe949': '\xee\xa5\x89',
'UnicodeCharacter0xe22d': '\xee\x88\xad',
'CjkUnifiedIdeograph7e1e': '\xe7\xb8\x9e',
'UnicodeCharacter0xe944': '\xee\xa5\x84',
'HangulSyllableDdyalb': '\xeb\x95\xb3',
'UnicodeCharacter0xe942': '\xee\xa5\x82',
'UnicodeCharacter0xe943': '\xee\xa5\x83',
'UnicodeCharacter0xe940': '\xee\xa5\x80',
'CjkUnifiedIdeograph7e1d': '\xe7\xb8\x9d',
'CombiningXAbove': '\xcc\xbd',
'HangulSyllableBbyoss': '\xeb\xbf\x84',
'UnicodeCharacter0xebb6': '\xee\xae\xb6',
'CjkUnifiedIdeograph7e1f': '\xe7\xb8\x9f',
'EurocurrencySign': '\xe2\x82\xa0',
'GreekSmallLetterAlphaWithPsiliAndOxia': '\xe1\xbc\x84',
'HangulSyllableCyelg': '\xec\xb3\xb5',
'UnicodeCharacter0xdd51': '\xed\xb5\x91',
'UnicodeCharacter0xe39b': '\xee\x8e\x9b',
'YiSyllableHnax': '\xea\x85\xb1',
'UnicodeCharacter0xf838': '\xef\xa0\xb8',
'HangulSyllableCyunh': '\xec\xb8\x8a',
'CjkUnifiedIdeograph6c89': '\xe6\xb2\x89',
'HangulSyllableCyunj': '\xec\xb8\x89',
'CjkUnifiedIdeograph8d1e': '\xe8\xb4\x9e',
'LepchaLetterKa': '\xe1\xb0\x80',
'HangulSyllableCyung': '\xec\xb8\x99',
'CjkUnifiedIdeograph8d1f': '\xe8\xb4\x9f',
'CjkUnifiedIdeograph8d1a': '\xe8\xb4\x9a',
'YiSyllableHnat': '\xea\x85\xb0',
'CjkUnifiedIdeograph8d1c': '\xe8\xb4\x9c',
'CjkUnifiedIdeograph8d1b': '\xe8\xb4\x9b',
'CjkUnifiedIdeograph6c47': '\xe6\xb1\x87',
'CjkUnifiedIdeograph6c46': '\xe6\xb1\x86',
'CjkUnifiedIdeograph6c45': '\xe6\xb1\x85',
'CjkUnifiedIdeograph6c44': '\xe6\xb1\x84',
'CjkUnifiedIdeograph6c43': '\xe6\xb1\x83',
'ArabicLetterYu': '\xdb\x88',
'CjkUnifiedIdeograph6c41': '\xe6\xb1\x81',
'CjkUnifiedIdeograph6c40': '\xe6\xb1\x80',
'UnicodeCharacter0xdd50': '\xed\xb5\x90',
'CjkUnifiedIdeograph6c49': '\xe6\xb1\x89',
'SquareRad': '\xe3\x8e\xad',
'VaiSymbolTong': '\xea\x98\x9d',
'MyanmarLetterLla': '\xe1\x80\xa0',
'LatinSmallLetterOWithHornAndAcute': '\xe1\xbb\x9b',
'UnicodeCharacter0x2095': '\xe2\x82\x95',
'HangulSyllableTeut': '\xed\x8b\x91',
'ArmenianSmallLetterSeh': '\xd5\xbd',
'CjkUnifiedIdeograph8d18': '\xe8\xb4\x98',
'CjkUnifiedIdeograph8d15': '\xe8\xb4\x95',
'CjkUnifiedIdeograph8d14': '\xe8\xb4\x94',
'CjkUnifiedIdeograph8d17': '\xe8\xb4\x97',
'CjkUnifiedIdeograph8d16': '\xe8\xb4\x96',
'CjkUnifiedIdeograph8d11': '\xe8\xb4\x91',
'OriyaLetterMa': '\xe0\xac\xae',
'CjkUnifiedIdeograph8d13': '\xe8\xb4\x93',
'CjkUnifiedIdeograph8d12': '\xe8\xb4\x92',
'CjkUnifiedIdeograph874b': '\xe8\x9d\x8b',
'UnicodeCharacter0xe641': '\xee\x99\x81',
'CjkUnifiedIdeograph8eac': '\xe8\xba\xac',
'UnicodeCharacter0xe640': '\xee\x99\x80',
'MiddleDot': '\xc2\xb7',
'HexagramForTheArousingThunder': '\xe4\xb7\xb2',
'HangulSyllableSsoegs': '\xec\x90\xaf',
'HangulSyllableSwaed': '\xec\x87\x8b',
'UnicodeCharacter0xec56': '\xee\xb1\x96',
'HangulSyllableKwegs': '\xed\x80\x9b',
'DoubleCircledNumberTen': '\xe2\x93\xbe',
'HangulSyllableTeum': '\xed\x8b\x88',
'UnicodeCharacter0xf4b7': '\xef\x92\xb7',
'UnicodeCharacter0xe644': '\xee\x99\x84',
'HangulJongseongHieuh': '\xe1\x87\x82',
'IdeographicAnnotationMiddleMark': '\xe3\x86\x97',
'CjkUnifiedIdeograph6c4e': '\xe6\xb1\x8e',
'CjkUnifiedIdeograph6c4d': '\xe6\xb1\x8d',
'CjkUnifiedIdeograph6c4c': '\xe6\xb1\x8c',
'CjkUnifiedIdeograph6c4b': '\xe6\xb1\x8b',
'CjkUnifiedIdeograph6c4a': '\xe6\xb1\x8a',
'HangulSyllableTeuk': '\xed\x8b\x90',
'HangulSyllableTwij': '\xed\x8a\x96',
'CjkCompatibilityIdeographfab9': '\xef\xaa\xb9',
'CjkCompatibilityIdeographf9e5': '\xef\xa7\xa5',
'UnicodeCharacter0x1cd0': '\xe1\xb3\x90',
'UnicodeCharacter0x1cd1': '\xe1\xb3\x91',
'UnicodeCharacter0x1cd2': '\xe1\xb3\x92',
'UnicodeCharacter0x1cd3': '\xe1\xb3\x93',
'UnicodeCharacter0x1cd4': '\xe1\xb3\x94',
'UnicodeCharacter0x1cd5': '\xe1\xb3\x95',
'UnicodeCharacter0x1cd6': '\xe1\xb3\x96',
'UnicodeCharacter0x1cd7': '\xe1\xb3\x97',
'UnicodeCharacter0x1cd8': '\xe1\xb3\x98',
'UnicodeCharacter0x1cd9': '\xe1\xb3\x99',
'UnicodeCharacter0xe34f': '\xee\x8d\x8f',
'HangulSyllableTeuh': '\xed\x8b\x93',
'HangulSyllableNeuk': '\xeb\x8a\xa8',
'HangulSyllableTeug': '\xed\x8a\xb9',
'PlusSignWithDotBelow': '\xe2\xa8\xa5',
'CjkUnifiedIdeograph741d': '\xe7\x90\x9d',
'CopticCapitalLetterOldCopticHa': '\xe2\xb3\x8e',
'CjkUnifiedIdeograph3dc0': '\xe3\xb7\x80',
'MongolianLetterSibeIy': '\xe1\xa1\x9f',
'UnicodeCharacter0xf32b': '\xef\x8c\xab',
'HangulSyllableJjij': '\xec\xb0\xa2',
'HangulSyllableTeud': '\xed\x8a\xbf',
'HangulSyllableBbyec': '\xeb\xbc\xbb',
'UnicodeCharacter0xf32f': '\xef\x8c\xaf',
'UnicodeCharacter0xf32e': '\xef\x8c\xae',
'UnicodeCharacter0xf32d': '\xef\x8c\xad',
'HangulSyllableTeuc': '\xed\x8b\x8f',
'HangulSyllableTeub': '\xed\x8b\x89',
'HangulSyllableCweng': '\xec\xb7\xa1',
'HangulSyllableTwang': '\xed\x87\x91',
'HangulSyllableTwanh': '\xed\x87\x82',
'HangulSyllableTwanj': '\xed\x87\x81',
'GreekSmallLetterIotaWithMacron': '\xe1\xbf\x91',
'UnicodeCharacter0xeed8': '\xee\xbb\x98',
'CjkUnifiedIdeograph7e19': '\xe7\xb8\x99',
'HangulSyllableSwaess': '\xec\x87\x98',
'MyanmarLetterDa': '\xe1\x80\x92',
'HangulSyllablePwebs': '\xed\x93\x82',
'UnicodeCharacter0x1cda': '\xe1\xb3\x9a',
'UnicodeCharacter0x1cdb': '\xe1\xb3\x9b',
'UnicodeCharacter0x1cdc': '\xe1\xb3\x9c',
'UnicodeCharacter0x1cdd': '\xe1\xb3\x9d',
'UnicodeCharacter0x1cde': '\xe1\xb3\x9e',
'UnicodeCharacter0x1cdf': '\xe1\xb3\x9f',
'RightTortoiseShellBracket': '\xe3\x80\x95',
'CjkUnifiedIdeograph8e44': '\xe8\xb9\x84',
'UnicodeCharacter0xe34a': '\xee\x8d\x8a',
'DevanagariLetterSa': '\xe0\xa4\xb8',
'UnicodeCharacter0xe28e': '\xee\x8a\x8e',
'HangulSyllableDdiss': '\xeb\x9d\xb4',
'UnicodeCharacter0xf323': '\xef\x8c\xa3',
'CjkUnifiedIdeograph7e11': '\xe7\xb8\x91',
'UnicodeCharacter0xf321': '\xef\x8c\xa1',
'UnicodeCharacter0xf320': '\xef\x8c\xa0',
'UnicodeCharacter0xf327': '\xef\x8c\xa7',
'CjkUnifiedIdeograph8e41': '\xe8\xb9\x81',
'UnicodeCharacter0xf325': '\xef\x8c\xa5',
'CjkUnifiedIdeograph7e10': '\xe7\xb8\x90',
'UnicodeCharacter0xffe7': '\xef\xbf\xa7',
'UnicodeCharacter0xf329': '\xef\x8c\xa9',
'UnicodeCharacter0xde63': '\xed\xb9\xa3',
'SymbolForStartOfHeading': '\xe2\x90\x81',
'UnicodeCharacter0xe34b': '\xee\x8d\x8b',
'CjkUnifiedIdeograph40af': '\xe4\x82\xaf',
'CjkUnifiedIdeograph7e15': '\xe7\xb8\x95',
'CjkUnifiedIdeograph7e14': '\xe7\xb8\x94',
'CjkUnifiedIdeograph47f7': '\xe4\x9f\xb7',
'MongolianLetterManchuAliGaliCa': '\xe1\xa2\x9c',
'GlagoliticCapitalLetterSlovo': '\xe2\xb0\x94',
'CjkUnifiedIdeograph47f4': '\xe4\x9f\xb4',
'CjkUnifiedIdeograph47f3': '\xe4\x9f\xb3',
'CjkUnifiedIdeograph47f2': '\xe4\x9f\xb2',
'CjkUnifiedIdeograph47f1': '\xe4\x9f\xb1',
'CjkUnifiedIdeograph47f0': '\xe4\x9f\xb0',
'CjkUnifiedIdeograph7e16': '\xe7\xb8\x96',
'CjkUnifiedIdeograph47f9': '\xe4\x9f\xb9',
'CjkUnifiedIdeograph47f8': '\xe4\x9f\xb8',
'UnicodeCharacter0xe16a': '\xee\x85\xaa',
'UnicodeCharacter0xe16b': '\xee\x85\xab',
'UnicodeCharacter0xe16c': '\xee\x85\xac',
'UnicodeCharacter0xe16d': '\xee\x85\xad',
'UnicodeCharacter0xe16e': '\xee\x85\xae',
'UnicodeCharacter0xe16f': '\xee\x85\xaf',
'UnicodeCharacter0xf5e5': '\xef\x97\xa5',
'CjkUnifiedIdeograph783b': '\xe7\xa0\xbb',
'CjkUnifiedIdeograph783c': '\xe7\xa0\xbc',
'ArabicLigatureShaddaWithDammaIsolatedForm': '\xef\xb1\xa1',
'CjkUnifiedIdeograph783a': '\xe7\xa0\xba',
'CjkUnifiedIdeograph783f': '\xe7\xa0\xbf',
'UnicodeCharacter0xf5e6': '\xef\x97\xa6',
'CjkUnifiedIdeograph783d': '\xe7\xa0\xbd',
'CjkUnifiedIdeograph783e': '\xe7\xa0\xbe',
'UnicodeCharacter0xa67a': '\xea\x99\xba',
'UnicodeCharacter0xa67b': '\xea\x99\xbb',
'UnicodeCharacter0xf5e7': '\xef\x97\xa7',
'CjkUnifiedIdeograph6e9f': '\xe6\xba\x9f',
'YiSyllableDdux': '\xea\x85\x8c',
'UnicodeCharacter0xf5e1': '\xef\x97\xa1',
'CjkUnifiedIdeograph47ff': '\xe4\x9f\xbf',
'CjkUnifiedIdeograph47fe': '\xe4\x9f\xbe',
'CjkUnifiedIdeograph47fd': '\xe4\x9f\xbd',
'CjkUnifiedIdeograph47fc': '\xe4\x9f\xbc',
'CjkUnifiedIdeograph47fb': '\xe4\x9f\xbb',
'CjkUnifiedIdeograph47fa': '\xe4\x9f\xba',
'HangulSyllableBbuk': '\xeb\xbf\xa4',
'YiSyllableNdip': '\xea\x85\x94',
'HangulSyllableNog': '\xeb\x85\xb9',
'YiSyllableNdit': '\xea\x85\x91',
'HangulSyllableSwael': '\xec\x87\x8c',
'CanadianSyllabicsThcreeThii': '\xe1\x96\xa9',
'YiSyllableNdie': '\xea\x85\x96',
'UnicodeCharacter0xf8c3': '\xef\xa3\x83',
'UnicodeCharacter0xa674': '\xea\x99\xb4',
'UnicodeCharacter0xa675': '\xea\x99\xb5',
'UnicodeCharacter0xa676': '\xea\x99\xb6',
'TibetanSignMchuCan': '\xe0\xbe\x89',
'UnicodeCharacter0xa678': '\xea\x99\xb8',
'UnicodeCharacter0xa679': '\xea\x99\xb9',
'UnicodeCharacter0xea90': '\xee\xaa\x90',
'HeavyDashedTriangleheadedRightwardsArrow': '\xe2\x9e\xa0',
'HangulSyllableMabs': '\xeb\xa7\x9a',
'UnicodeCharacter0xf5e9': '\xef\x97\xa9',
'CanadianSyllabicsWestcreeSwoo': '\xe1\x93\xbf',
'CjkUnifiedIdeograph3f81': '\xe3\xbe\x81',
'CjkUnifiedIdeograph7838': '\xe7\xa0\xb8',
'CjkUnifiedIdeograph7839': '\xe7\xa0\xb9',
'UnicodeCharacter0xe164': '\xee\x85\xa4',
'UnicodeCharacter0xe165': '\xee\x85\xa5',
'GreekCapitalLetterAlphaWithDasia': '\xe1\xbc\x89',
'HangulSyllableBbuc': '\xeb\xbf\xa3',
'BraillePatternDots23478': '\xe2\xa3\x8e',
'CjkUnifiedIdeograph7833': '\xe7\xa0\xb3',
'CjkUnifiedIdeograph7830': '\xe7\xa0\xb0',
'CjkUnifiedIdeograph7831': '\xe7\xa0\xb1',
'CjkUnifiedIdeograph5cf8': '\xe5\xb3\xb8',
'CjkUnifiedIdeograph3f83': '\xe3\xbe\x83',
'CjkUnifiedIdeograph7834': '\xe7\xa0\xb4',
'CjkUnifiedIdeograph7835': '\xe7\xa0\xb5',
'CjkUnifiedIdeograph9935': '\xe9\xa4\xb5',
'IdeographicTelegraphSymbolForHourTwentytwo': '\xe3\x8d\xae',
'CjkUnifiedIdeograph9934': '\xe9\xa4\xb4',
'HangulSyllableDdobs': '\xeb\x98\xa2',
'HangulSyllableSwaej': '\xec\x87\x9a',
'HangulSyllableBbug': '\xeb\xbf\x8d',
'UnicodeCharacter0xde62': '\xed\xb9\xa2',
'CanadianSyllabicsChiSign': '\xe1\x99\xad',
'CjkUnifiedIdeograph9936': '\xe9\xa4\xb6',
'HangulSyllableBbud': '\xeb\xbf\x93',
'CjkUnifiedIdeograph9931': '\xe9\xa4\xb1',
'YiSyllableZzit': '\xea\x8b\x90',
'UnicodeCharacter0xf8c9': '\xef\xa3\x89',
'CjkUnifiedIdeograph9e62': '\xe9\xb9\xa2',
'CjkUnifiedIdeograph9e63': '\xe9\xb9\xa3',
'CjkUnifiedIdeograph9e60': '\xe9\xb9\xa0',
'CjkUnifiedIdeograph9e61': '\xe9\xb9\xa1',
'CjkUnifiedIdeograph9e66': '\xe9\xb9\xa6',
'CjkUnifiedIdeograph9933': '\xe9\xa4\xb3',
'CjkUnifiedIdeograph9e64': '\xe9\xb9\xa4',
'CjkUnifiedIdeograph9e65': '\xe9\xb9\xa5',
'CjkUnifiedIdeograph9e68': '\xe9\xb9\xa8',
'CjkUnifiedIdeograph5bfb': '\xe5\xaf\xbb',
'CjkUnifiedIdeograph5519': '\xe5\x94\x99',
'UnicodeCharacter0xece3': '\xee\xb3\xa3',
'GreekBetaSymbol': '\xcf\x90',
'UnicodeCharacter0xaa37': '\xea\xa8\xb7',
'CjkUnifiedIdeograph5518': '\xe5\x94\x98',
'HangulSyllableSwigs': '\xec\x89\xaf',
'HangulSyllableSyobs': '\xec\x88\x8e',
'HangulSyllableHyib': '\xed\x9d\xbd',
'HangulSyllableJyen': '\xec\xa1\x98',
'HangulSyllableHyid': '\xed\x9d\xb3',
'HangulSyllableHyik': '\xed\x9e\x84',
'HangulSyllableHyij': '\xed\x9e\x82',
'HangulSyllableHyih': '\xed\x9e\x87',
'HangulSyllableHyin': '\xed\x9d\xb0',
'HangulSyllableHyim': '\xed\x9d\xbc',
'HangulSyllableHyil': '\xed\x9d\xb4',
'HangulSyllableHyis': '\xed\x9d\xbf',
'OriyaDigitNine': '\xe0\xad\xaf',
'CjkUnifiedIdeograph5512': '\xe5\x94\x92',
'ChamLetterLa': '\xea\xa8\xa4',
'YiSyllableUo': '\xea\x80\x8d',
'HangulSyllableNoj': '\xeb\x86\x8e',
'UnicodeCharacter0xea94': '\xee\xaa\x94',
'Cancer': '\xe2\x99\x8b',
'CjkUnifiedIdeograph9e6b': '\xe9\xb9\xab',
'CjkUnifiedIdeograph9e6c': '\xe9\xb9\xac',
'CjkUnifiedIdeograph9e6a': '\xe9\xb9\xaa',
'CjkUnifiedIdeograph9e6f': '\xe9\xb9\xaf',
'CjkUnifiedIdeograph5515': '\xe5\x94\x95',
'CjkUnifiedIdeograph9e6d': '\xe9\xb9\xad',
'CjkUnifiedIdeograph9e6e': '\xe9\xb9\xae',
'CjkUnifiedIdeograph5514': '\xe5\x94\x94',
'CjkUnifiedIdeograph43eb': '\xe4\x8f\xab',
'CjkUnifiedIdeograph43ec': '\xe4\x8f\xac',
'UnicodeCharacter0xfd48': '\xef\xb5\x88',
'CjkUnifiedIdeograph43ea': '\xe4\x8f\xaa',
'CjkUnifiedIdeograph43ef': '\xe4\x8f\xaf',
'CjkUnifiedIdeograph43ed': '\xe4\x8f\xad',
'CjkUnifiedIdeograph43ee': '\xe4\x8f\xae',
'UnicodeCharacter0xfd42': '\xef\xb5\x82',
'CjkUnifiedIdeograph959f': '\xe9\x96\x9f',
'CjkUnifiedIdeograph959e': '\xe9\x96\x9e',
'RunicLetterLongbrancharAe': '\xe1\x9b\x85',
'CjkUnifiedIdeograph959c': '\xe9\x96\x9c',
'CjkUnifiedIdeograph959b': '\xe9\x96\x9b',
'CjkUnifiedIdeograph959a': '\xe9\x96\x9a',
'UnicodeCharacter0xfd45': '\xef\xb5\x85',
'CjkUnifiedIdeograph6013': '\xe6\x80\x93',
'CjkUnifiedIdeograph6012': '\xe6\x80\x92',
'CjkUnifiedIdeograph6011': '\xe6\x80\x91',
'CjkUnifiedIdeograph6010': '\xe6\x80\x90',
'CjkUnifiedIdeograph6017': '\xe6\x80\x97',
'SyriacHbasaAbove': '\xdc\xba',
'CjkUnifiedIdeograph6015': '\xe6\x80\x95',
'CjkUnifiedIdeograph5a0e': '\xe5\xa8\x8e',
'CjkUnifiedIdeograph79ae': '\xe7\xa6\xae',
'CjkUnifiedIdeograph79ad': '\xe7\xa6\xad',
'CjkUnifiedIdeograph6019': '\xe6\x80\x99',
'CjkUnifiedIdeograph6018': '\xe6\x80\x98',
'CjkUnifiedIdeograph79aa': '\xe7\xa6\xaa',
'HangulSyllableJjwac': '\xec\xab\xaf',
'CjkUnifiedIdeograph79ac': '\xe7\xa6\xac',
'EthiopicNumberTwenty': '\xe1\x8d\xb3',
'HangulSyllableBbyat': '\xeb\xba\xb1',
'CjkUnifiedIdeograph3acf': '\xe3\xab\x8f',
'CjkUnifiedIdeograph3ace': '\xe3\xab\x8e',
'LatinSmallLetterUWithDiaeresisAndGrave': '\xc7\x9c',
'CjkUnifiedIdeograph3acc': '\xe3\xab\x8c',
'CjkUnifiedIdeograph3acb': '\xe3\xab\x8b',
'CjkUnifiedIdeograph3aca': '\xe3\xab\x8a',
'CjkUnifiedIdeograph8ea1': '\xe8\xba\xa1',
'ArabicLetterNgoehMedialForm': '\xef\xae\x9d',
'HangulSyllableJjaegs': '\xec\xa7\xbb',
'HangulSyllableBbyad': '\xeb\xba\x9f',
'HangulSyllableBbyae': '\xeb\xba\xb4',
'CjkUnifiedIdeograph8ea3': '\xe8\xba\xa3',
'CjkUnifiedIdeograph6ad9': '\xe6\xab\x99',
'CjkUnifiedIdeograph6ad8': '\xe6\xab\x98',
'HangulSyllableBbyab': '\xeb\xba\xa9',
'GurmukhiDigitOne': '\xe0\xa9\xa7',
'CjkUnifiedIdeograph6ad5': '\xe6\xab\x95',
'CjkUnifiedIdeograph6ad4': '\xe6\xab\x94',
'CjkUnifiedIdeograph6ad7': '\xe6\xab\x97',
'CjkUnifiedIdeograph6ad6': '\xe6\xab\x96',
'CjkUnifiedIdeograph6ad1': '\xe6\xab\x91',
'CjkUnifiedIdeograph6ad0': '\xe6\xab\x90',
'CjkUnifiedIdeograph6ad3': '\xe6\xab\x93',
'CjkUnifiedIdeograph6ad2': '\xe6\xab\x92',
'CjkUnifiedIdeograph601c': '\xe6\x80\x9c',
'VerticalKanaRepeatWithVoicedSoundMarkUpperHalf': '\xe3\x80\xb4',
'CjkUnifiedIdeograph601a': '\xe6\x80\x9a',
'UnicodeCharacter0xe0f6': '\xee\x83\xb6',
'GreekSmallLetterIotaWithPsiliAndVaria': '\xe1\xbc\xb2',
'CjkUnifiedIdeograph601f': '\xe6\x80\x9f',
'CjkUnifiedIdeograph601e': '\xe6\x80\x9e',
'CjkUnifiedIdeograph601d': '\xe6\x80\x9d',
'CjkUnifiedIdeograph79a5': '\xe7\xa6\xa5',
'SinhalaVowelSignDigaKombuva': '\xe0\xb7\x9a',
'CjkUnifiedIdeograph79a7': '\xe7\xa6\xa7',
'CjkUnifiedIdeograph79a6': '\xe7\xa6\xa6',
'CjkUnifiedIdeograph79a1': '\xe7\xa6\xa1',
'CjkUnifiedIdeograph79a0': '\xe7\xa6\xa0',
'CjkUnifiedIdeograph79a3': '\xe7\xa6\xa3',
'CjkUnifiedIdeograph79a2': '\xe7\xa6\xa2',
'CjkUnifiedIdeograph43e2': '\xe4\x8f\xa2',
'CjkUnifiedIdeograph43e3': '\xe4\x8f\xa3',
'CjkUnifiedIdeograph43e0': '\xe4\x8f\xa0',
'CjkUnifiedIdeograph43e1': '\xe4\x8f\xa1',
'CjkUnifiedIdeograph43e6': '\xe4\x8f\xa6',
'CjkUnifiedIdeograph43e7': '\xe4\x8f\xa7',
'CjkUnifiedIdeograph43e4': '\xe4\x8f\xa4',
'CjkUnifiedIdeograph43e5': '\xe4\x8f\xa5',
'CjkUnifiedIdeograph9597': '\xe9\x96\x97',
'CjkUnifiedIdeograph9596': '\xe9\x96\x96',
'CjkUnifiedIdeograph43e8': '\xe4\x8f\xa8',
'CjkUnifiedIdeograph43e9': '\xe4\x8f\xa9',
'CjkUnifiedIdeograph9593': '\xe9\x96\x93',
'CjkUnifiedIdeograph9592': '\xe9\x96\x92',
'CjkUnifiedIdeograph9591': '\xe9\x96\x91',
'CjkUnifiedIdeograph9590': '\xe9\x96\x90',
'CjkUnifiedIdeograph6ade': '\xe6\xab\x9e',
'CjkUnifiedIdeograph6add': '\xe6\xab\x9d',
'CjkUnifiedIdeograph6adf': '\xe6\xab\x9f',
'CjkUnifiedIdeograph6ada': '\xe6\xab\x9a',
'CjkUnifiedIdeograph6adc': '\xe6\xab\x9c',
'CjkUnifiedIdeograph6adb': '\xe6\xab\x9b',
'CjkUnifiedIdeograph3ac7': '\xe3\xab\x87',
'CombiningTurnedCommaAbove': '\xcc\x92',
'CjkUnifiedIdeograph3ac5': '\xe3\xab\x85',
'CjkUnifiedIdeograph3ac4': '\xe3\xab\x84',
'CjkUnifiedIdeograph3ac3': '\xe3\xab\x83',
'CjkUnifiedIdeograph3ac2': '\xe3\xab\x82',
'CjkUnifiedIdeograph3ac1': '\xe3\xab\x81',
'CjkUnifiedIdeograph3ac0': '\xe3\xab\x80',
'HangulSyllableJyet': '\xec\xa1\xad',
'LaoLetterKhoTam': '\xe0\xba\x84',
'HangulSyllableJyelg': '\xec\xa1\x9d',
'UnicodeCharacter0xe6e5': '\xee\x9b\xa5',
'CjkUnifiedIdeograph3ac9': '\xe3\xab\x89',
'CjkUnifiedIdeograph3ac8': '\xe3\xab\x88',
'HangulSyllableJjye': '\xec\xaa\xa0',
'HangulSyllableDess': '\xeb\x8e\x84',
'HangulSyllableSwess': '\xec\x89\xa4',
'CircledAsteriskOperator': '\xe2\x8a\x9b',
'HangulSyllableNol': '\xeb\x86\x80',
'HangulSyllableJjyoss': '\xec\xad\x80',
'HangulSyllableJjya': '\xec\xa8\x94',
'CjkUnifiedIdeograph4514': '\xe4\x94\x94',
'CjkUnifiedIdeograph4515': '\xe4\x94\x95',
'CjkUnifiedIdeograph4516': '\xe4\x94\x96',
'CjkUnifiedIdeograph4517': '\xe4\x94\x97',
'CjkUnifiedIdeograph4510': '\xe4\x94\x90',
'CjkUnifiedIdeograph4511': '\xe4\x94\x91',
'KhmerDigitSeven': '\xe1\x9f\xa7',
'CjkUnifiedIdeograph4513': '\xe4\x94\x93',
'UnicodeCharacter0xde4e': '\xed\xb9\x8e',
'CjkUnifiedIdeograph3bca': '\xe3\xaf\x8a',
'CjkUnifiedIdeograph3bcb': '\xe3\xaf\x8b',
'CjkUnifiedIdeograph3bcc': '\xe3\xaf\x8c',
'CjkUnifiedIdeograph3bcd': '\xe3\xaf\x8d',
'CjkUnifiedIdeograph3bce': '\xe3\xaf\x8e',
'CjkUnifiedIdeograph3bcf': '\xe3\xaf\x8f',
'UnicodeCharacter0xde4b': '\xed\xb9\x8b',
'HangulSyllableYuss': '\xec\x9c\xb4',
'CjkUnifiedIdeograph74e3': '\xe7\x93\xa3',
'HangulSyllableCoeng': '\xec\xb5\xb1',
'CjkUnifiedIdeograph74e4': '\xe7\x93\xa4',
'WhiteConcavesidedDiamond': '\xe2\x9f\xa1',
'CjkUnifiedIdeograph74e5': '\xe7\x93\xa5',
'ArabicLigatureMeemWithYehIsolatedForm': '\xef\xb1\x8a',
'CjkUnifiedIdeograph74e6': '\xe7\x93\xa6',
'CjkUnifiedIdeograph74e7': '\xe7\x93\xa7',
'HangulLetterRieulyeorinhieuh': '\xe3\x85\xad',
'TripleVerticalBarWithHorizontalStroke': '\xe2\xab\xb5',
'HangulSyllableTeulg': '\xed\x8b\x81',
'SaurashtraLetterGa': '\xea\xa2\x94',
'MyanmarLetterShanDa': '\xe1\x81\xbb',
'HangulSyllableJjyu': '\xec\xae\xb8',
'HangulSyllableByenh': '\xeb\xb3\x9e',
'HangulSyllableByenj': '\xeb\xb3\x9d',
'HangulSyllableRwan': '\xeb\xa1\xbc',
'HangulSyllableByeng': '\xeb\xb3\xad',
'HangulSyllableKalp': '\xec\xba\x82',
'HangulSyllableKals': '\xec\xba\x80',
'CjkUnifiedIdeograph3bc8': '\xe3\xaf\x88',
'CjkUnifiedIdeograph3bc9': '\xe3\xaf\x89',
'CjkUnifiedIdeograph451f': '\xe4\x94\x9f',
'UnicodeCharacter0xde49': '\xed\xb9\x89',
'CjkUnifiedIdeograph451a': '\xe4\x94\x9a',
'CjkUnifiedIdeograph451b': '\xe4\x94\x9b',
'CjkUnifiedIdeograph451c': '\xe4\x94\x9c',
'CjkUnifiedIdeograph3bc0': '\xe3\xaf\x80',
'CjkUnifiedIdeograph3bc1': '\xe3\xaf\x81',
'CjkUnifiedIdeograph3bc2': '\xe3\xaf\x82',
'CjkUnifiedIdeograph3bc3': '\xe3\xaf\x83',
'CjkUnifiedIdeograph3bc4': '\xe3\xaf\x84',
'CjkUnifiedIdeograph3bc5': '\xe3\xaf\x85',
'CjkUnifiedIdeograph3bc6': '\xe3\xaf\x86',
'CjkUnifiedIdeograph3bc7': '\xe3\xaf\x87',
'ArabicLigatureSeenWithJeemIsolatedForm': '\xef\xb0\x9c',
'YiSyllableZzix': '\xea\x8b\x91',
'LatinCapitalLetterAeWithMacron': '\xc7\xa2',
'HangulSyllableRwam': '\xeb\xa2\x88',
'UnicodeCharacter0xf8a1': '\xef\xa2\xa1',
'WavyLowLine': '\xef\xb9\x8f',
'HangulSyllableJyaelg': '\xec\x9f\xad',
'HebrewPointTsere': '\xd6\xb5',
'UnicodeCharacter0xe7b3': '\xee\x9e\xb3',
'YiSyllableNjyp': '\xea\x91\x87',
'YiSyllableNjyt': '\xea\x91\x84',
'HangulSyllableNoec': '\xeb\x87\xa3',
'YiSyllableNjyx': '\xea\x91\x85',
'UnicodeCharacter0xf6cb': '\xef\x9b\x8b',
'UnicodeCharacter0xf4ae': '\xef\x92\xae',
'UnicodeCharacter0xf4ad': '\xef\x92\xad',
'UnicodeCharacter0xf4af': '\xef\x92\xaf',
'UnicodeCharacter0xf4aa': '\xef\x92\xaa',
'UnicodeCharacter0xf4ac': '\xef\x92\xac',
'UnicodeCharacter0xf4ab': '\xef\x92\xab',
'HangulSyllableNoem': '\xeb\x87\x9c',
'YiSyllableCox': '\xea\x8a\xbe',
'UnicodeCharacter0xf235': '\xef\x88\xb5',
'HangulSyllableRwak': '\xeb\xa2\x90',
'CjkUnifiedIdeograph92ee': '\xe9\x8b\xae',
'HangulSyllableHyaet': '\xed\x97\x85',
'HangulSyllableHwinh': '\xed\x9c\x9e',
'YiSyllableYyrx': '\xea\x92\x8b',
'CjkUnifiedIdeograph92ef': '\xe9\x8b\xaf',
'UnicodeCharacter0xf236': '\xef\x88\xb6',
'HangulSyllableHwinj': '\xed\x9c\x9d',
'UnicodeCharacter0xf231': '\xef\x88\xb1',
'CjkUnifiedIdeograph5a03': '\xe5\xa8\x83',
'ArabicLetterBehWithInvertedSmallVBelow': '\xdd\x95',
'Zeta': '\xce\x96',
'CjkUnifiedIdeograph92ea': '\xe9\x8b\xaa',
'HangulSyllableNyum': '\xeb\x8a\x84',
'CjkUnifiedIdeograph993e': '\xe9\xa4\xbe',
'UnicodeCharacter0xf4a7': '\xef\x92\xa7',
'HangulSyllableNyun': '\xeb\x89\xb8',
'HangulSyllableHwalb': '\xed\x99\x9f',
'CjkUnifiedIdeograph92eb': '\xe9\x8b\xab',
'HangulSyllableNyuk': '\xeb\x8a\x8c',
'CjkUnifiedIdeograph993d': '\xe9\xa4\xbd',
'CjkUnifiedIdeograph3f8e': '\xe3\xbe\x8e',
'HangulSyllableNyug': '\xeb\x89\xb5',
'HangulSyllableHwalm': '\xed\x99\x9e',
'HangulSyllableJyelb': '\xec\xa1\x9f',
'HangulSyllableByo': '\xeb\xb5\xa4',
'HangulSyllableNyuc': '\xeb\x8a\x8b',
'YiSyllableCop': '\xea\x8b\x80',
'YiSyllableBot': '\xea\x80\xa5',
'VaiSyllableRee': '\xea\x94\x93',
'CjkUnifiedIdeograph993f': '\xe9\xa4\xbf',
'YiSyllableBop': '\xea\x80\xa8',
'HangulSyllableHwals': '\xed\x99\xa0',
'HangulSyllableHwalp': '\xed\x99\xa2',
'CjkUnifiedIdeograph993a': '\xe9\xa4\xba',
'HangulSyllableJyelt': '\xec\xa1\xa1',
'YiSyllableBox': '\xea\x80\xa6',
'HangulSyllableNyup': '\xeb\x8a\x8e',
'HangulSyllableNyus': '\xeb\x8a\x87',
'HangulSyllableSseobs': '\xec\x8d\xba',
'CjkUnifiedIdeograph88b8': '\xe8\xa2\xb8',
'CjkUnifiedIdeograph88b9': '\xe8\xa2\xb9',
'CjkUnifiedIdeograph88b4': '\xe8\xa2\xb4',
'CjkUnifiedIdeograph88b5': '\xe8\xa2\xb5',
'CjkUnifiedIdeograph88b6': '\xe8\xa2\xb6',
'CjkUnifiedIdeograph88b7': '\xe8\xa2\xb7',
'CjkUnifiedIdeograph88b0': '\xe8\xa2\xb0',
'CjkUnifiedIdeograph88b1': '\xe8\xa2\xb1',
'LatinCapitalLetterOi': '\xc6\xa2',
'CjkUnifiedIdeograph88b3': '\xe8\xa2\xb3',
'HangulSyllablePyibs': '\xed\x94\xb2',
'LatinCapitalLetterOu': '\xc8\xa2',
'HangulSyllableTaess': '\xed\x83\xb0',
'UnicodeCharacter0x8cb': '\xe0\xa3\x8b',
'UnicodeCharacter0x8cc': '\xe0\xa3\x8c',
'CjkCompatibilityIdeographf94c': '\xef\xa5\x8c',
'UnicodeCharacter0x8ca': '\xe0\xa3\x8a',
'UnicodeCharacter0x8cf': '\xe0\xa3\x8f',
'UnicodeCharacter0xf1c2': '\xef\x87\x82',
'UnicodeCharacter0x8cd': '\xe0\xa3\x8d',
'UnicodeCharacter0x8ce': '\xe0\xa3\x8e',
'UnicodeCharacter0xaa3d': '\xea\xa8\xbd',
'UnicodeCharacter0xf615': '\xef\x98\x95',
'UnicodeCharacter0xaa3c': '\xea\xa8\xbc',
'HangulSyllableCubs': '\xec\xb6\xa6',
'UnicodeCharacter0xaa3b': '\xea\xa8\xbb',
'CjkUnifiedIdeograph6822': '\xe6\xa0\xa2',
'CjkUnifiedIdeograph6823': '\xe6\xa0\xa3',
'CjkUnifiedIdeograph6820': '\xe6\xa0\xa0',
'LatinCapitalLetterToneSix': '\xc6\x84',
'CjkUnifiedIdeograph6826': '\xe6\xa0\xa6',
'LeftThreeEighthsBlock': '\xe2\x96\x8d',
'CjkUnifiedIdeograph6824': '\xe6\xa0\xa4',
'CjkUnifiedIdeograph6825': '\xe6\xa0\xa5',
'CjkUnifiedIdeograph7e17': '\xe7\xb8\x97',
'HangulSyllableKweng': '\xed\x80\xad',
'CjkUnifiedIdeograph6828': '\xe6\xa0\xa8',
'CjkUnifiedIdeograph6829': '\xe6\xa0\xa9',
'HangulSyllableWit': '\xec\x9c\x9d',
'CjkCompatibilityIdeographf948': '\xef\xa5\x88',
'UnicodeCharacter0x8c8': '\xe0\xa3\x88',
'UnicodeCharacter0x8c9': '\xe0\xa3\x89',
'HangulSyllableWip': '\xec\x9c\x9e',
'CjkUnifiedIdeograph74ea': '\xe7\x93\xaa',
'UnicodeCharacter0x8c2': '\xe0\xa3\x82',
'UnicodeCharacter0x8c3': '\xe0\xa3\x83',
'UnicodeCharacter0x8c0': '\xe0\xa3\x80',
'UnicodeCharacter0x8c1': '\xe0\xa3\x81',
'UnicodeCharacter0x8c6': '\xe0\xa3\x86',
'UnicodeCharacter0x8c7': '\xe0\xa3\x87',
'UnicodeCharacter0x8c4': '\xe0\xa3\x84',
'UnicodeCharacter0x8c5': '\xe0\xa3\x85',
'HangulSyllableWid': '\xec\x9c\x8b',
'CjkUnifiedIdeograph74ec': '\xe7\x93\xac',
'HangulSyllableWib': '\xec\x9c\x95',
'HangulSyllableWic': '\xec\x9c\x9b',
'CjkUnifiedIdeograph88bd': '\xe8\xa2\xbd',
'CjkUnifiedIdeograph74ed': '\xe7\x93\xad',
'CjkUnifiedIdeograph88bf': '\xe8\xa2\xbf',
'UnicodeCharacter0x2daf': '\xe2\xb6\xaf',
'HangulSyllableWih': '\xec\x9c\x9f',
'CjkUnifiedIdeograph88ba': '\xe8\xa2\xba',
'CjkUnifiedIdeograph88bb': '\xe8\xa2\xbb',
'CjkUnifiedIdeograph88bc': '\xe8\xa2\xbc',
'CjkUnifiedIdeograph682b': '\xe6\xa0\xab',
'LatinSmallLetterYWithGrave': '\xe1\xbb\xb3',
'CjkUnifiedIdeograph682a': '\xe6\xa0\xaa',
'CjkUnifiedIdeograph682f': '\xe6\xa0\xaf',
'CjkUnifiedIdeograph74ef': '\xe7\x93\xaf',
'CjkUnifiedIdeograph682d': '\xe6\xa0\xad',
'CjkUnifiedIdeograph682e': '\xe6\xa0\xae',
'SyriacZqaphaBelow': '\xdc\xb4',
'YiSyllableHmop': '\xea\x82\xa2',
'YiSyllableHmot': '\xea\x82\x9f',
'GurmukhiDigitTwo': '\xe0\xa9\xa8',
'YiSyllableHmox': '\xea\x82\xa0',
'BengaliSignAvagraha': '\xe0\xa6\xbd',
'HangulSyllableSsuss': '\xec\x91\xb8',
'HangulLetterMieumsios': '\xe3\x85\xaf',
'CjkUnifiedIdeograph70fd': '\xe7\x83\xbd',
'GujaratiLetterLa': '\xe0\xaa\xb2',
'OlChikiPunctuationMucaad': '\xe1\xb1\xbe',
'CjkUnifiedIdeograph8d19': '\xe8\xb4\x99',
'UnicodeCharacter0x9cf': '\xe0\xa7\x8f',
'SixperemSpace': '\xe2\x80\x86',
'ArabicLigatureMeemWithHahInitialForm': '\xef\xb3\x8f',
'HangulSyllableGubs': '\xea\xb5\xbe',
'HexagramForOpposition': '\xe4\xb7\xa5',
'CjkUnifiedIdeograph7db9': '\xe7\xb6\xb9',
'CjkUnifiedIdeograph40ba': '\xe4\x82\xba',
'CjkUnifiedIdeograph40bb': '\xe4\x82\xbb',
'CjkUnifiedIdeograph40bc': '\xe4\x82\xbc',
'CjkUnifiedIdeograph40bd': '\xe4\x82\xbd',
'CjkUnifiedIdeograph40be': '\xe4\x82\xbe',
'CjkUnifiedIdeograph40bf': '\xe4\x82\xbf',
'ArabicLetterZahMedialForm': '\xef\xbb\x88',
'TibetanSymbolNorBu': '\xe0\xbf\x89',
'HangulSyllableKweonj': '\xed\x80\x81',
'CjkUnifiedIdeograph6e8e': '\xe6\xba\x8e',
'CjkUnifiedIdeograph6e8d': '\xe6\xba\x8d',
'GreekCapitalLetterOmegaWithPsiliAndVaria': '\xe1\xbd\xaa',
'CjkUnifiedIdeograph6e8f': '\xe6\xba\x8f',
'CjkUnifiedIdeograph6e8a': '\xe6\xba\x8a',
'CjkUnifiedIdeograph6e8c': '\xe6\xba\x8c',
'CjkUnifiedIdeograph6e8b': '\xe6\xba\x8b',
'SquareKpa': '\xe3\x8e\xaa',
'CjkUnifiedIdeograph7fa0': '\xe7\xbe\xa0',
'EthiopicSyllableRaa': '\xe1\x88\xab',
'UnicodeCharacter0x9c9': '\xe0\xa7\x89',
'UnicodeCharacter0x9c6': '\xe0\xa7\x86',
'UnicodeCharacter0x9c5': '\xe0\xa7\x85',
'CjkUnifiedIdeograph6e85': '\xe6\xba\x85',
'CjkUnifiedIdeograph6e84': '\xe6\xba\x84',
'CjkUnifiedIdeograph6e87': '\xe6\xba\x87',
'CjkUnifiedIdeograph6e86': '\xe6\xba\x86',
'CjkUnifiedIdeograph6e81': '\xe6\xba\x81',
'CjkUnifiedIdeograph6e80': '\xe6\xba\x80',
'CjkUnifiedIdeograph6e83': '\xe6\xba\x83',
'CjkUnifiedIdeograph6e82': '\xe6\xba\x82',
'KangxiRadicalGrain': '\xe2\xbd\xb2',
'CjkUnifiedIdeograph6e89': '\xe6\xba\x89',
'CjkUnifiedIdeograph6e88': '\xe6\xba\x88',
'CjkUnifiedIdeograph40b0': '\xe4\x82\xb0',
'CjkUnifiedIdeograph40b1': '\xe4\x82\xb1',
'CjkUnifiedIdeograph40b2': '\xe4\x82\xb2',
'CjkUnifiedIdeograph40b3': '\xe4\x82\xb3',
'CjkUnifiedIdeograph40b4': '\xe4\x82\xb4',
'CjkUnifiedIdeograph40b5': '\xe4\x82\xb5',
'CjkUnifiedIdeograph40b6': '\xe4\x82\xb6',
'CjkUnifiedIdeograph40b7': '\xe4\x82\xb7',
'CjkUnifiedIdeograph40b8': '\xe4\x82\xb8',
'CjkUnifiedIdeograph40b9': '\xe4\x82\xb9',
'UnicodeCharacter0xf113': '\xef\x84\x93',
'LatinCapitalLetterIWithDotAbove': '\xc4\xb0',
'ModifierLetterLowUpArrowhead': '\xcb\xb0',
'HangulSyllableTonh': '\xed\x86\xa6',
'TaiLeLetterHa': '\xe1\xa5\x9e',
'HangulSyllableTonj': '\xed\x86\xa5',
'CombiningShortSolidusOverlay': '\xcc\xb7',
'LowerOneQuarterBlock': '\xe2\x96\x82',
'CjkUnifiedIdeograph7e28': '\xe7\xb8\xa8',
'CjkUnifiedIdeograph7e29': '\xe7\xb8\xa9',
'CjkUnifiedIdeograph7e24': '\xe7\xb8\xa4',
'CjkUnifiedIdeograph4a9a': '\xe4\xaa\x9a',
'CjkUnifiedIdeograph4a9b': '\xe4\xaa\x9b',
'CjkUnifiedIdeograph4a9c': '\xe4\xaa\x9c',
'CjkUnifiedIdeograph4a9d': '\xe4\xaa\x9d',
'CjkUnifiedIdeograph4a9e': '\xe4\xaa\x9e',
'CjkUnifiedIdeograph4a9f': '\xe4\xaa\x9f',
'CjkUnifiedIdeograph7e23': '\xe7\xb8\xa3',
'HangulSyllableKyeong': '\xec\xbc\xb1',
'HangulSyllableKyeonh': '\xec\xbc\xa2',
'HangulSyllableKyeonj': '\xec\xbc\xa1',
'Om': '\xce\xa9',
'YiSyllableMgat': '\xea\x88\xb2',
'HryvniaSign': '\xe2\x82\xb4',
'YiSyllableMgap': '\xea\x88\xb5',
'ChamLetterSa': '\xea\xa8\xa7',
'UnicodeCharacter0x74b': '\xdd\x8b',
'UnicodeCharacter0x74c': '\xdd\x8c',
'HangulSyllableJyels': '\xec\xa1\xa0',
'EarthGround': '\xe2\x8f\x9a',
'HangulSyllableJaenh': '\xec\x9e\xb2',
'UnicodeCharacter0xf6f2': '\xef\x9b\xb2',
'CjkCompatibilityIdeographfab1': '\xef\xaa\xb1',
'ArabicLigatureSadWithHahIsolatedForm': '\xef\xb0\xa0',
'CjkCompatibilityIdeographfab0': '\xef\xaa\xb0',
'LatinSmallLetterAWithBreveAndAcute': '\xe1\xba\xaf',
'NewTaiLueLetterHighLa': '\xe1\xa6\x9c',
'CjkUnifiedIdeograph8e11': '\xe8\xb8\x91',
'HangulSyllableKeulp': '\xed\x81\xba',
'CjkCompatibilityIdeographfab6': '\xef\xaa\xb6',
'CjkUnifiedIdeograph4a98': '\xe4\xaa\x98',
'LatinSmallLetterLWithCedilla': '\xc4\xbc',
'SuperscriptFive': '\xe2\x81\xb5',
'CjkUnifiedIdeograph4a90': '\xe4\xaa\x90',
'CjkUnifiedIdeograph4a91': '\xe4\xaa\x91',
'TamilLetterMa': '\xe0\xae\xae',
'CjkUnifiedIdeograph4a93': '\xe4\xaa\x93',
'CjkUnifiedIdeograph4a94': '\xe4\xaa\x94',
'CjkUnifiedIdeograph4a95': '\xe4\xaa\x95',
'CjkUnifiedIdeograph4a96': '\xe4\xaa\x96',
'CjkUnifiedIdeograph4a97': '\xe4\xaa\x97',
'CjkUnifiedIdeograph4d42': '\xe4\xb5\x82',
'CjkUnifiedIdeograph4d43': '\xe4\xb5\x83',
'CjkUnifiedIdeograph4d40': '\xe4\xb5\x80',
'CjkUnifiedIdeograph4d41': '\xe4\xb5\x81',
'CjkUnifiedIdeograph4d46': '\xe4\xb5\x86',
'CjkUnifiedIdeograph4d47': '\xe4\xb5\x87',
'CjkUnifiedIdeograph4d44': '\xe4\xb5\x84',
'BoxDrawingsLeftLightAndRightUpHeavy': '\xe2\x94\xba',
'CjkUnifiedIdeograph5a17': '\xe5\xa8\x97',
'CjkUnifiedIdeograph5a16': '\xe5\xa8\x96',
'CjkUnifiedIdeograph4d48': '\xe4\xb5\x88',
'CjkUnifiedIdeograph4d49': '\xe4\xb5\x89',
'CjkUnifiedIdeograph5a13': '\xe5\xa8\x93',
'CjkUnifiedIdeograph5a12': '\xe5\xa8\x92',
'CjkUnifiedIdeograph5a11': '\xe5\xa8\x91',
'CjkUnifiedIdeograph5a10': '\xe5\xa8\x90',
'sampi': '\xcf\xa1',
'TamilNumberSign': '\xe0\xaf\xba',
'UnicodeCharacter0xe78': '\xe0\xb9\xb8',
'UnicodeCharacter0xe73': '\xe0\xb9\xb3',
'UnicodeCharacter0xe72': '\xe0\xb9\xb2',
'WhiteSmallStar': '\xe2\xad\x92',
'UnicodeCharacter0xe70': '\xe0\xb9\xb0',
'UnicodeCharacter0xe77': '\xe0\xb9\xb7',
'UnicodeCharacter0xe76': '\xe0\xb9\xb6',
'UnicodeCharacter0xe75': '\xe0\xb9\xb5',
'UnicodeCharacter0xe74': '\xe0\xb9\xb4',
'ArabicLetterSeenFinalForm': '\xef\xba\xb2',
'CjkUnifiedIdeograph92e4': '\xe9\x8b\xa4',
'CjkUnifiedIdeograph92e5': '\xe9\x8b\xa5',
'UnicodeCharacter0xf69c': '\xef\x9a\x9c',
'UnicodeCharacter0xf69b': '\xef\x9a\x9b',
'UnicodeCharacter0xf69a': '\xef\x9a\x9a',
'CjkUnifiedIdeograph92e6': '\xe9\x8b\xa6',
'UnicodeCharacter0xf69e': '\xef\x9a\x9e',
'UnicodeCharacter0xf69d': '\xef\x9a\x9d',
'UnicodeCharacter0xf121': '\xef\x84\xa1',
'UnicodeCharacter0xf120': '\xef\x84\xa0',
'UnicodeCharacter0xf123': '\xef\x84\xa3',
'CjkUnifiedIdeograph92e7': '\xe9\x8b\xa7',
'UnicodeCharacter0xf125': '\xef\x84\xa5',
'UnicodeCharacter0xf124': '\xef\x84\xa4',
'UnicodeCharacter0xf127': '\xef\x84\xa7',
'CjkUnifiedIdeograph615c': '\xe6\x85\x9c',
'CjkUnifiedIdeograph7dfc': '\xe7\xb7\xbc',
'BraillePatternDots4578': '\xe2\xa3\x98',
'CjkUnifiedIdeograph92e1': '\xe9\x8b\xa1',
'CircledTriangleDown': '\xe2\x8e\x8a',
'CjkUnifiedIdeograph8e14': '\xe8\xb8\x94',
'CjkUnifiedIdeograph92e2': '\xe9\x8b\xa2',
'UnicodeCharacter0xe7c': '\xe0\xb9\xbc',
'UnicodeCharacter0xe7b': '\xe0\xb9\xbb',
'UnicodeCharacter0xe7a': '\xe0\xb9\xba',
'CjkUnifiedIdeograph92e3': '\xe9\x8b\xa3',
'UnicodeCharacter0xe7f': '\xe0\xb9\xbf',
'UnicodeCharacter0xe7e': '\xe0\xb9\xbe',
'UnicodeCharacter0xe7d': '\xe0\xb9\xbd',
'CjkUnifiedIdeograph4d4b': '\xe4\xb5\x8b',
'CjkUnifiedIdeograph4d4c': '\xe4\xb5\x8c',
'CjkUnifiedIdeograph4d4a': '\xe4\xb5\x8a',
'CjkUnifiedIdeograph4d4f': '\xe4\xb5\x8f',
'CjkUnifiedIdeograph4d4d': '\xe4\xb5\x8d',
'CjkUnifiedIdeograph4d4e': '\xe4\xb5\x8e',
'CjkUnifiedIdeograph5a1f': '\xe5\xa8\x9f',
'CjkUnifiedIdeograph5a1e': '\xe5\xa8\x9e',
'CjkUnifiedIdeograph5a1d': '\xe5\xa8\x9d',
'NorthWestArrowWithHook': '\xe2\xa4\xa3',
'CjkUnifiedIdeograph5a1b': '\xe5\xa8\x9b',
'CjkUnifiedIdeograph5a1a': '\xe5\xa8\x9a',
'KangxiRadicalDownBox': '\xe2\xbc\x8c',
'UnicodeCharacter0xf12a': '\xef\x84\xaa',
'CjkUnifiedIdeograph8e15': '\xe8\xb8\x95',
'UnicodeCharacter0xf12b': '\xef\x84\xab',
'UnicodeCharacter0xf12e': '\xef\x84\xae',
'UnicodeCharacter0xe62': '\xe0\xb9\xa2',
'BalineseLetterEfSasak': '\xe1\xad\x88',
'UnicodeCharacter0xf12f': '\xef\x84\xaf',
'GlagoliticCapitalLetterSha': '\xe2\xb0\x9e',
'UnicodeCharacter0xf699': '\xef\x9a\x99',
'UnicodeCharacter0xf698': '\xef\x9a\x98',
'CjkUnifiedIdeograph7b7b': '\xe7\xad\xbb',
'HangulSyllableJjwaep': '\xec\xac\x8e',
'chi': '\xcf\x87',
'UnicodeCharacter0xf693': '\xef\x9a\x93',
'UnicodeCharacter0xf692': '\xef\x9a\x92',
'UnicodeCharacter0xf691': '\xef\x9a\x91',
'CjkUnifiedIdeograph9769': '\xe9\x9d\xa9',
'UnicodeCharacter0xf697': '\xef\x9a\x97',
'UnicodeCharacter0xf696': '\xef\x9a\x96',
'UnicodeCharacter0xf695': '\xef\x9a\x95',
'UnicodeCharacter0xf694': '\xef\x9a\x94',
'VaiSyllableNyoo': '\xea\x96\x94',
'ArabicSignSindhiPostpositionMen': '\xdb\xbe',
'HangulSyllableMwab': '\xeb\xab\x95',
'CjkUnifiedIdeograph850a': '\xe8\x94\x8a',
'BengaliCurrencyDenominatorSixteen': '\xe0\xa7\xb9',
'CombiningCyrillicPayerok': '\xea\x99\xbd',
'CjkUnifiedIdeograph3912': '\xe3\xa4\x92',
'HangulSyllableBbugs': '\xeb\xbf\x8f',
'CjkUnifiedIdeograph8e17': '\xe8\xb8\x97',
'CjkUnifiedIdeograph8978': '\xe8\xa5\xb8',
'CjkUnifiedIdeograph8979': '\xe8\xa5\xb9',
'HangulSyllableRyeogs': '\xeb\xa0\xa7',
'CjkUnifiedIdeograph8972': '\xe8\xa5\xb2',
'CjkUnifiedIdeograph8973': '\xe8\xa5\xb3',
'CjkUnifiedIdeograph8970': '\xe8\xa5\xb0',
'CjkUnifiedIdeograph8971': '\xe8\xa5\xb1',
'CjkUnifiedIdeograph8976': '\xe8\xa5\xb6',
'CjkUnifiedIdeograph8977': '\xe8\xa5\xb7',
'KatakanaLetterSmallWa': '\xe3\x83\xae',
'CjkUnifiedIdeograph8975': '\xe8\xa5\xb5',
'ArabicLigatureSheenWithMeemWithKhahInitialForm': '\xef\xb5\xab',
'CjkUnifiedIdeograph51f8': '\xe5\x87\xb8',
'SundaneseLetterQa': '\xe1\xae\x8b',
'UnicodeCharacter0xefbf': '\xee\xbe\xbf',
'CjkUnifiedIdeograph51f2': '\xe5\x87\xb2',
'CjkUnifiedIdeograph51f3': '\xe5\x87\xb3',
'CjkUnifiedIdeograph51f0': '\xe5\x87\xb0',
'CjkUnifiedIdeograph51f1': '\xe5\x87\xb1',
'GeorgianLetterShin': '\xe1\x83\xa8',
'CjkUnifiedIdeograph51f7': '\xe5\x87\xb7',
'CjkUnifiedIdeograph51f4': '\xe5\x87\xb4',
'CjkUnifiedIdeograph51f5': '\xe5\x87\xb5',
'IdeographicTelegraphSymbolForHourTwo': '\xe3\x8d\x9a',
'CjkUnifiedIdeograph4741': '\xe4\x9d\x81',
'HangulSyllablePwelh': '\xed\x92\xbf',
'HangulSyllablePwelm': '\xed\x92\xba',
'HangulSyllablePwelb': '\xed\x92\xbb',
'UnicodeCharacter0xefbc': '\xee\xbe\xbc',
'HangulSyllablePwelg': '\xed\x92\xb9',
'UnicodeCharacter0xefbb': '\xee\xbe\xbb',
'KannadaVowelSignUu': '\xe0\xb3\x82',
'HangulSyllablePwels': '\xed\x92\xbc',
'HangulSyllablePwelp': '\xed\x92\xbe',
'HangulSyllablePwelt': '\xed\x92\xbd',
'CjkUnifiedIdeograph3f34': '\xe3\xbc\xb4',
'UnicodeCharacter0xdb1f': '\xed\xac\x9f',
'UnicodeCharacter0xa7c4': '\xea\x9f\x84',
'CjkUnifiedIdeograph51fb': '\xe5\x87\xbb',
'CjkUnifiedIdeograph51fc': '\xe5\x87\xbc',
'CjkUnifiedIdeograph51fa': '\xe5\x87\xba',
'CjkUnifiedIdeograph51ff': '\xe5\x87\xbf',
'UnicodeCharacter0xa7c7': '\xea\x9f\x87',
'CjkUnifiedIdeograph51fd': '\xe5\x87\xbd',
'CjkUnifiedIdeograph51fe': '\xe5\x87\xbe',
'HangulSyllableHwa': '\xed\x99\x94',
'UnicodeCharacter0xf8c': '\xe0\xbe\x8c',
'UnicodeCharacter0xa7c6': '\xea\x9f\x86',
'UnicodeCharacter0xf8e': '\xe0\xbe\x8e',
'UnicodeCharacter0xf8d': '\xe0\xbe\x8d',
'UnicodeCharacter0xf8f': '\xe0\xbe\x8f',
'CjkUnifiedIdeograph897b': '\xe8\xa5\xbb',
'CjkUnifiedIdeograph897c': '\xe8\xa5\xbc',
'CjkUnifiedIdeograph897a': '\xe8\xa5\xba',
'HangulJongseongNieunpansios': '\xe1\x87\x88',
'ArabicLigatureHahWithJeemWithYehFinalForm': '\xef\xb6\xbf',
'CjkUnifiedIdeograph897d': '\xe8\xa5\xbd',
'CjkUnifiedIdeograph897e': '\xe8\xa5\xbe',
'SmallRomanNumeralTwo': '\xe2\x85\xb1',
'HangulSyllableHalp': '\xed\x95\xa6',
'CjkUnifiedIdeograph850e': '\xe8\x94\x8e',
'UnicodeCharacter0xa7c3': '\xea\x9f\x83',
'UnicodeCharacter0xe787': '\xee\x9e\x87',
'UnicodeCharacter0xebd9': '\xee\xaf\x99',
'UnicodeCharacter0xa7c2': '\xea\x9f\x82',
'NorthWestAndSouthEastArrow': '\xe2\xa4\xa1',
'HangulSyllableJjyegs': '\xec\xaa\xa3',
'WhiteSquareWithRoundedCorners': '\xe2\x96\xa2',
'PassivepulldownoutputSymbol': '\xe2\x8e\x91',
'UnicodeCharacter0xe57c': '\xee\x95\xbc',
'HangulSyllableSwat': '\xec\x87\x81',
'UnicodeCharacter0xe57a': '\xee\x95\xba',
'ArabicLigatureYehWithHamzaAboveWithMeemFinalForm': '\xef\xb1\xa6',
'UnicodeCharacter0xe57f': '\xee\x95\xbf',
'UnicodeCharacter0xe57e': '\xee\x95\xbe',
'UnicodeCharacter0xe57d': '\xee\x95\xbd',
'CjkUnifiedIdeograph850d': '\xe8\x94\x8d',
'LatinSmallLetterIWithMacron': '\xc4\xab',
'CjkUnifiedIdeograph8b52': '\xe8\xad\x92',
'HangulSyllableKebs': '\xec\xbc\x92',
'SaurashtraLetterCha': '\xea\xa2\x98',
'UnicodeCharacter0xa7c9': '\xea\x9f\x89',
'CjkUnifiedIdeograph3f39': '\xe3\xbc\xb9',
'ArabicLigatureYehWithHamzaAboveWithYuFinalForm': '\xef\xaf\xb5',
'CjkUnifiedIdeograph8358': '\xe8\x8d\x98',
'CjkUnifiedIdeograph8359': '\xe8\x8d\x99',
'CjkUnifiedIdeograph8356': '\xe8\x8d\x96',
'CjkUnifiedIdeograph8357': '\xe8\x8d\x97',
'CjkUnifiedIdeograph8354': '\xe8\x8d\x94',
'CjkUnifiedIdeograph8355': '\xe8\x8d\x95',
'CjkUnifiedIdeograph8352': '\xe8\x8d\x92',
'CjkUnifiedIdeograph8353': '\xe8\x8d\x93',
'CjkUnifiedIdeograph8350': '\xe8\x8d\x90',
'CjkUnifiedIdeograph8351': '\xe8\x8d\x91',
'UnicodeCharacter0xe579': '\xee\x95\xb9',
'UnicodeCharacter0xe578': '\xee\x95\xb8',
'UnicodeCharacter0xe781': '\xee\x9e\x81',
'TeluguLetterAu': '\xe0\xb0\x94',
'UnicodeCharacter0xe572': '\xee\x95\xb2',
'UnicodeCharacter0xe571': '\xee\x95\xb1',
'UnicodeCharacter0xe570': '\xee\x95\xb0',
'UnicodeCharacter0xe577': '\xee\x95\xb7',
'UnicodeCharacter0xe576': '\xee\x95\xb6',
'UnicodeCharacter0xe575': '\xee\x95\xb5',
'UnicodeCharacter0xe574': '\xee\x95\xb4',
'HalfwidthHangulLetterYe': '\xef\xbf\x8b',
'TeluguLetterAi': '\xe0\xb0\x90',
'HalfwidthHangulLetterYa': '\xef\xbf\x84',
'UnicodeCharacter0xe84c': '\xee\xa1\x8c',
'HangulJongseongRieulmieum': '\xe1\x86\xb1',
'TeluguLetterAa': '\xe0\xb0\x86',
'UnicodeCharacter0xe84b': '\xee\xa1\x8b',
'OlChikiLetterLo': '\xe1\xb1\xb3',
'GeometricallyEquivalentTo': '\xe2\x89\x8e',
'OlChikiLetterLi': '\xe1\xb1\xa4',
'CjkUnifiedIdeograph835f': '\xe8\x8d\x9f',
'CjkUnifiedIdeograph835d': '\xe8\x8d\x9d',
'OlChikiLetterLe': '\xe1\xb1\xae',
'CjkUnifiedIdeograph835b': '\xe8\x8d\x9b',
'CjkUnifiedIdeograph835c': '\xe8\x8d\x9c',
'OlChikiLetterLa': '\xe1\xb1\x9a',
'HangulSyllableSwad': '\xec\x86\xaf',
'UpwardsTwoheadedArrowFromSmallCircle': '\xe2\xa5\x89',
'HangulSyllableSwae': '\xec\x87\x84',
'CjkUnifiedIdeograph5629': '\xe5\x98\xa9',
'CjkUnifiedIdeograph5628': '\xe5\x98\xa8',
'UnicodeCharacter0xf7b5': '\xef\x9e\xb5',
'CjkUnifiedIdeograph5625': '\xe5\x98\xa5',
'LeftCornerBracket': '\xe3\x80\x8c',
'CjkUnifiedIdeograph5627': '\xe5\x98\xa7',
'CjkUnifiedIdeograph5626': '\xe5\x98\xa6',
'CjkUnifiedIdeograph5621': '\xe5\x98\xa1',
'CjkUnifiedIdeograph5620': '\xe5\x98\xa0',
'CjkUnifiedIdeograph5623': '\xe5\x98\xa3',
'CjkUnifiedIdeograph5622': '\xe5\x98\xa2',
'HangulSyllableHalh': '\xed\x95\xa7',
'CjkUnifiedIdeograph9759': '\xe9\x9d\x99',
'CjkUnifiedIdeograph9758': '\xe9\x9d\x98',
'CjkUnifiedIdeograph9755': '\xe9\x9d\x95',
'CjkUnifiedIdeograph9754': '\xe9\x9d\x94',
'CjkUnifiedIdeograph9757': '\xe9\x9d\x97',
'CjkUnifiedIdeograph9756': '\xe9\x9d\x96',
'CjkUnifiedIdeograph9751': '\xe9\x9d\x91',
'CjkUnifiedIdeograph9750': '\xe9\x9d\x90',
'CjkUnifiedIdeograph9753': '\xe9\x9d\x93',
'CjkUnifiedIdeograph9752': '\xe9\x9d\x92',
'MongolianLetterAliGaliZha': '\xe1\xa2\x95',
'HangulSyllableDdaelb': '\xeb\x95\x97',
'IdeographicTelegraphSymbolForOctober': '\xe3\x8b\x89',
'HangulSyllableNags': '\xeb\x82\x9b',
'HangulSyllableHalm': '\xed\x95\xa2',
'CjkUnifiedIdeograph4fbf': '\xe4\xbe\xbf',
'CjkUnifiedIdeograph4fbd': '\xe4\xbe\xbd',
'CjkUnifiedIdeograph4fbe': '\xe4\xbe\xbe',
'CjkUnifiedIdeograph4fbb': '\xe4\xbe\xbb',
'CjkUnifiedIdeograph4fbc': '\xe4\xbe\xbc',
'CjkUnifiedIdeograph4fba': '\xe4\xbe\xba',
'YiSyllableBix': '\xea\x80\x97',
'HangulSyllableHweng': '\xed\x9c\x91',
'ArabicLetterFehMedialForm': '\xef\xbb\x94',
'CjkUnifiedIdeograph7202': '\xe7\x88\x82',
'CjkUnifiedIdeograph975e': '\xe9\x9d\x9e',
'CjkUnifiedIdeograph7205': '\xe7\x88\x85',
'CjkUnifiedIdeograph975f': '\xe9\x9d\x9f',
'CjkUnifiedIdeograph975a': '\xe9\x9d\x9a',
'CjkUnifiedIdeograph975c': '\xe9\x9d\x9c',
'CjkUnifiedIdeograph975b': '\xe9\x9d\x9b',
'DigitThreeFullStop': '\xe2\x92\x8a',
'ArabicLigatureTahWithMeemIsolatedForm': '\xef\xb0\xa7',
'BoxDrawingsLightUpAndHeavyDown': '\xe2\x95\xbd',
'CjkUnifiedIdeograph562e': '\xe5\x98\xae',
'CjkUnifiedIdeograph562d': '\xe5\x98\xad',
'CjkUnifiedIdeograph562f': '\xe5\x98\xaf',
'CjkUnifiedIdeograph562a': '\xe5\x98\xaa',
'CjkUnifiedIdeograph562c': '\xe5\x98\xac',
'CjkUnifiedIdeograph562b': '\xe5\x98\xab',
'CjkUnifiedIdeograph4fb6': '\xe4\xbe\xb6',
'EthiopicSyllableSi': '\xe1\x88\xb2',
'CjkUnifiedIdeograph4fb4': '\xe4\xbe\xb4',
'CjkUnifiedIdeograph4fb5': '\xe4\xbe\xb5',
'CjkUnifiedIdeograph4fb2': '\xe4\xbe\xb2',
'CjkUnifiedIdeograph4fb3': '\xe4\xbe\xb3',
'CjkUnifiedIdeograph4fb0': '\xe4\xbe\xb0',
'EthiopicSyllableSo': '\xe1\x88\xb6',
'VaiSyllableLa': '\xea\x95\x9e',
'EthiopicSyllableSa': '\xe1\x88\xb0',
'CyrillicCapitalLetterGheWithDescender': '\xd3\xb6',
'VaiSyllableLe': '\xea\x97\xb7',
'EthiopicSyllableSe': '\xe1\x88\xb5',
'CjkUnifiedIdeograph4fb8': '\xe4\xbe\xb8',
'CjkUnifiedIdeograph4fb9': '\xe4\xbe\xb9',
'HangulSyllableKaebs': '\xec\xba\xa2',
'TeluguFractionDigitThreeForOddPowersOfFour': '\xe0\xb1\xbb',
'UnicodeCharacter0xda64': '\xed\xa9\xa4',
'VaiSyllableLu': '\xea\x96\xa8',
'EthiopicSyllableSu': '\xe1\x88\xb1',
'GeorgianSmallLetterWe': '\xe2\xb4\xa3',
'VaiSyllableVo': '\xea\x97\x8a',
'HeavyBlackfeatheredSouthEastArrow': '\xe2\x9e\xb7',
'DevanagariVowelSignE': '\xe0\xa5\x87',
'CjkUnifiedIdeograph77f4': '\xe7\x9f\xb4',
'ArabicLetterRehFinalForm': '\xef\xba\xae',
'UnicodeCharacter0xa7cd': '\xea\x9f\x8d',
'DevanagariVowelSignO': '\xe0\xa5\x8b',
'HangulSyllableIn': '\xec\x9d\xb8',
'DevanagariVowelSignI': '\xe0\xa4\xbf',
'DevanagariVowelSignU': '\xe0\xa5\x81',
'EthiopicSyllableKyaa': '\xe2\xb7\x8b',
'RightWhiteCurlyBracket': '\xe2\xa6\x84',
'HangulSyllableIh': '\xec\x9e\x8f',
'UnicodeCharacter0xf7b4': '\xef\x9e\xb4',
'ArabicLetterJehIsolatedForm': '\xef\xae\x8a',
'UnicodeCharacter0xe6d': '\xe0\xb9\xad',
'VaiSyllableVi': '\xea\x94\xb2',
'UnicodeCharacter0xa7cb': '\xea\x9f\x8b',
'CjkUnifiedIdeograph3508': '\xe3\x94\x88',
'CjkUnifiedIdeograph3509': '\xe3\x94\x89',
'ThaanaLetterShaviyani': '\xde\x81',
'CjkUnifiedIdeograph3502': '\xe3\x94\x82',
'CjkUnifiedIdeograph3503': '\xe3\x94\x83',
'CjkUnifiedIdeograph3500': '\xe3\x94\x80',
'CjkUnifiedIdeograph3501': '\xe3\x94\x81',
'CjkUnifiedIdeograph3506': '\xe3\x94\x86',
'CjkUnifiedIdeograph3507': '\xe3\x94\x87',
'CjkUnifiedIdeograph3504': '\xe3\x94\x84',
'CjkUnifiedIdeograph3505': '\xe3\x94\x85',
'SylotiNagriLetterDho': '\xea\xa0\x97',
'HangulSyllableIg': '\xec\x9d\xb5',
'ArabicLetterKirghizOeIsolatedForm': '\xef\xaf\xa0',
'HangulLetterSios': '\xe3\x85\x85',
'BengaliLetterNna': '\xe0\xa6\xa3',
'HangulSyllableIb': '\xec\x9e\x85',
'HangulSyllableBwegg': '\xeb\xb6\xba',
'b': '\xce\xb2',
'SucceedsUnderRelation': '\xe2\x8a\xb1',
'GreekCapitalLetterEpsilonWithOxia': '\xe1\xbf\x89',
'HangulSyllableNunj': '\xeb\x88\x89',
'CjkUnifiedIdeograph350b': '\xe3\x94\x8b',
'CjkUnifiedIdeograph350c': '\xe3\x94\x8c',
'LatinSmallLetterIWithRetroflexHook': '\xe1\xb6\x96',
'CjkUnifiedIdeograph350a': '\xe3\x94\x8a',
'CjkUnifiedIdeograph350f': '\xe3\x94\x8f',
'CjkUnifiedIdeograph350d': '\xe3\x94\x8d',
'CjkUnifiedIdeograph350e': '\xe3\x94\x8e',
'HangulSyllableHwigg': '\xed\x9c\x9a',
'TibetanSubjoinedLetterDza': '\xe0\xbe\xab',
'CanadianSyllabicsQoo': '\xe1\x96\x82',
'PhagspaLetterCa': '\xea\xa1\x84',
'HangulSyllableNung': '\xeb\x88\x99',
'UnicodeCharacter0xefb9': '\xee\xbe\xb9',
'UnicodeCharacter0xefb8': '\xee\xbe\xb8',
'KannadaLetterGha': '\xe0\xb2\x98',
'ModifierLetterSmallEth': '\xe1\xb6\x9e',
'CjkUnifiedIdeograph7570': '\xe7\x95\xb0',
'HangulSyllableKib': '\xed\x82\xb5',
'LatinSmallLetterWWithAcute': '\xe1\xba\x83',
'HangulSyllableIp': '\xec\x9e\x8e',
'HangulSyllableGgyobs': '\xea\xbe\xae',
'UnicodeCharacter0xefb2': '\xee\xbe\xb2',
'CjkUnifiedIdeograph499a': '\xe4\xa6\x9a',
'CjkUnifiedIdeograph499b': '\xe4\xa6\x9b',
'CjkUnifiedIdeograph499c': '\xe4\xa6\x9c',
'CjkUnifiedIdeograph499d': '\xe4\xa6\x9d',
'CjkUnifiedIdeograph499e': '\xe4\xa6\x9e',
'CjkUnifiedIdeograph499f': '\xe4\xa6\x9f',
'CjkUnifiedIdeograph8b04': '\xe8\xac\x84',
'CjkUnifiedIdeograph8b05': '\xe8\xac\x85',
'CjkUnifiedIdeograph8b06': '\xe8\xac\x86',
'CjkUnifiedIdeograph8b07': '\xe8\xac\x87',
'CjkUnifiedIdeograph8b00': '\xe8\xac\x80',
'CjkUnifiedIdeograph8b01': '\xe8\xac\x81',
'CjkUnifiedIdeograph8b02': '\xe8\xac\x82',
'CjkUnifiedIdeograph8b03': '\xe8\xac\x83',
'UnicodeCharacter0xe229': '\xee\x88\xa9',
'VaiSyllableThe': '\xea\x97\xb4',
'UnicodeCharacter0xeb45': '\xee\xad\x85',
'UnicodeCharacter0xeb44': '\xee\xad\x84',
'CjkUnifiedIdeograph3913': '\xe3\xa4\x93',
'CjkUnifiedIdeograph4990': '\xe4\xa6\x90',
'CjkUnifiedIdeograph4991': '\xe4\xa6\x91',
'CjkUnifiedIdeograph4992': '\xe4\xa6\x92',
'CjkUnifiedIdeograph4993': '\xe4\xa6\x93',
'CanadianSyllabicsLho': '\xe1\x96\xa2',
'CjkUnifiedIdeograph4995': '\xe4\xa6\x95',
'CjkUnifiedIdeograph4996': '\xe4\xa6\x96',
'CjkUnifiedIdeograph4997': '\xe4\xa6\x97',
'CjkUnifiedIdeograph4998': '\xe4\xa6\x98',
'GreekCapitalLetterEpsilonWithPsili': '\xe1\xbc\x98',
'CanadianSyllabicsLha': '\xe1\x96\xa4',
'GreekSmallLetterEtaWithPsiliAndVariaAndYpogegrammeni': '\xe1\xbe\x92',
'CjkUnifiedIdeograph8b0a': '\xe8\xac\x8a',
'CjkUnifiedIdeograph8b0b': '\xe8\xac\x8b',
'CjkUnifiedIdeograph8b0c': '\xe8\xac\x8c',
'PresentationFormForVerticalLeftCurlyBracket': '\xef\xb8\xb7',
'CjkUnifiedIdeograph3917': '\xe3\xa4\x97',
'SquareAnpea': '\xe3\x8c\x82',
'UnicodeCharacter0xeb42': '\xee\xad\x82',
'UnicodeCharacter0xdaaf': '\xed\xaa\xaf',
'CjkUnifiedIdeograph9b4f': '\xe9\xad\x8f',
'CjkUnifiedIdeograph9b4e': '\xe9\xad\x8e',
'CjkUnifiedIdeograph9b4d': '\xe9\xad\x8d',
'CjkUnifiedIdeograph9b4c': '\xe9\xad\x8c',
'CjkUnifiedIdeograph9b4b': '\xe9\xad\x8b',
'CjkUnifiedIdeograph9b4a': '\xe9\xad\x8a',
'UnicodeCharacter0xdaaa': '\xed\xaa\xaa',
'HangulSyllableJjyolh': '\xec\xac\xbb',
'HangulChoseongChitueumssangsios': '\xe1\x84\xbd',
'UnicodeCharacter0xa8ec': '\xea\xa3\xac',
'UnicodeCharacter0xdd61': '\xed\xb5\xa1',
'HangulSyllableByeogs': '\xeb\xb2\xbf',
'ArabicLetterTehWithRing': '\xd9\xbc',
'UnicodeCharacter0xdd67': '\xed\xb5\xa7',
'CjkUnifiedIdeograph433d': '\xe4\x8c\xbd',
'CjkUnifiedIdeograph433e': '\xe4\x8c\xbe',
'CjkUnifiedIdeograph433f': '\xe4\x8c\xbf',
'HangulSyllableHulg': '\xed\x9b\x8d',
'CjkUnifiedIdeograph433a': '\xe4\x8c\xba',
'CjkUnifiedIdeograph433b': '\xe4\x8c\xbb',
'CjkUnifiedIdeograph433c': '\xe4\x8c\xbc',
'HangulSyllableHulm': '\xed\x9b\x8e',
'DoesNotContainAsMember': '\xe2\x88\x8c',
'UnicodeCharacter0xdd65': '\xed\xb5\xa5',
'HangulSyllableHulh': '\xed\x9b\x93',
'LatinCapitalLetterVWithTilde': '\xe1\xb9\xbc',
'HangulSyllableHult': '\xed\x9b\x91',
'UnicodeCharacter0xf0d7': '\xef\x83\x97',
'CyrillicSmallLetterKomiZje': '\xd4\x85',
'HangulSyllableRaj': '\xeb\x9e\x92',
'KangxiRadicalGhost': '\xe2\xbf\x81',
'HangulSyllableHuls': '\xed\x9b\x90',
'CjkUnifiedIdeograph797f': '\xe7\xa5\xbf',
'CjkUnifiedIdeograph797e': '\xe7\xa5\xbe',
'CjkUnifiedIdeograph797d': '\xe7\xa5\xbd',
'CjkUnifiedIdeograph797c': '\xe7\xa5\xbc',
'LeftDottedSubstitutionBracket': '\xe2\xb8\x84',
'CjkUnifiedIdeograph797a': '\xe7\xa5\xba',
'UnicodeCharacter0xdd69': '\xed\xb5\xa9',
'HangulSyllableDyelm': '\xeb\x8e\xb2',
'EthiopicSyllableQyaa': '\xe2\xb7\x83',
'UnicodeCharacter0xf7b2': '\xef\x9e\xb2',
'HangulSyllableKwing': '\xed\x81\x89',
'ArabicLetterDahalFinalForm': '\xef\xae\x85',
'UnicodeCharacter0xf0d4': '\xef\x83\x94',
'CjkUnifiedIdeograph9b47': '\xe9\xad\x87',
'CjkUnifiedIdeograph9b46': '\xe9\xad\x86',
'CjkUnifiedIdeograph9b45': '\xe9\xad\x85',
'CjkUnifiedIdeograph9b44': '\xe9\xad\x84',
'CjkUnifiedIdeograph9b43': '\xe9\xad\x83',
'CjkUnifiedIdeograph9b42': '\xe9\xad\x82',
'CjkUnifiedIdeograph9b41': '\xe9\xad\x81',
'CjkUnifiedIdeograph9b40': '\xe9\xad\x80',
'UnicodeCharacter0xe7b8': '\xee\x9e\xb8',
'LatinSmallLetterDWithCedilla': '\xe1\xb8\x91',
'CjkUnifiedIdeograph9b49': '\xe9\xad\x89',
'CjkUnifiedIdeograph9b48': '\xe9\xad\x88',
'SubscriptSeven': '\xe2\x82\x87',
'HangulSyllableJjyolm': '\xec\xac\xb6',
'CjkUnifiedIdeograph7979': '\xe7\xa5\xb9',
'CjkUnifiedIdeograph7978': '\xe7\xa5\xb8',
'CjkUnifiedIdeograph7977': '\xe7\xa5\xb7',
'CjkUnifiedIdeograph7976': '\xe7\xa5\xb6',
'CjkUnifiedIdeograph7975': '\xe7\xa5\xb5',
'CjkUnifiedIdeograph7974': '\xe7\xa5\xb4',
'CjkUnifiedIdeograph7973': '\xe7\xa5\xb3',
'BuhidLetterYa': '\xe1\x9d\x8c',
'CjkUnifiedIdeograph7971': '\xe7\xa5\xb1',
'CjkUnifiedIdeograph7970': '\xe7\xa5\xb0',
'CjkUnifiedIdeograph4334': '\xe4\x8c\xb4',
'CjkUnifiedIdeograph4335': '\xe4\x8c\xb5',
'CjkUnifiedIdeograph4336': '\xe4\x8c\xb6',
'CjkUnifiedIdeograph4337': '\xe4\x8c\xb7',
'CjkUnifiedIdeograph4330': '\xe4\x8c\xb0',
'CjkUnifiedIdeograph4331': '\xe4\x8c\xb1',
'CjkUnifiedIdeograph4332': '\xe4\x8c\xb2',
'CjkUnifiedIdeograph4333': '\xe4\x8c\xb3',
'CjkUnifiedIdeograph50d5': '\xe5\x83\x95',
'CjkUnifiedIdeograph4338': '\xe4\x8c\xb8',
'CjkUnifiedIdeograph4339': '\xe4\x8c\xb9',
'TagbanwaLetterLa': '\xe1\x9d\xae',
'PresentationFormForVerticalLeftTortoiseShellBracket': '\xef\xb8\xb9',
'UnicodeCharacter0xdfaa': '\xed\xbe\xaa',
'UnicodeCharacter0xdfac': '\xed\xbe\xac',
'HangulSyllableRab': '\xeb\x9e\x8d',
'UnicodeCharacter0xdfae': '\xed\xbe\xae',
'UnicodeCharacter0xdfad': '\xed\xbe\xad',
'UnicodeCharacter0xdfaf': '\xed\xbe\xaf',
'HangulSyllableRam': '\xeb\x9e\x8c',
'HangulSyllableMit': '\xeb\xb0\x91',
'UnicodeCharacter0xe222': '\xee\x88\xa2',
'KoreanStandardSymbol': '\xe3\x89\xbf',
'FullBlock': '\xe2\x96\x88',
'ModifierLetterCapitalG': '\xe1\xb4\xb3',
'HangulJungseongWeo': '\xe1\x85\xaf',
'HangulSyllableRan': '\xeb\x9e\x80',
'UnicodeCharacter0xf0d3': '\xef\x83\x93',
'SyriacSupralinearColon': '\xdc\x83',
'HangulSyllableRah': '\xeb\x9e\x97',
'SquareOnsu': '\xe3\x8c\x89',
'SyriacEsasaBelow': '\xdc\xbe',
'HangulSyllableGweogs': '\xea\xb6\x8b',
'UnicodeCharacter0xe563': '\xee\x95\xa3',
'UnicodeCharacter0xe07c': '\xee\x81\xbc',
'UnicodeCharacter0xf0d0': '\xef\x83\x90',
'LatinCapitalLetterGWithDotAbove': '\xc4\xa0',
'UnicodeCharacter0xf5b4': '\xef\x96\xb4',
'UnicodeCharacter0xdfa9': '\xed\xbe\xa9',
'HangulSyllableMib': '\xeb\xb0\x89',
'UnicodeCharacter0xe220': '\xee\x88\xa0',
'GreekSmallLetterOmicronWithPsiliAndVaria': '\xe1\xbd\x82',
'UnicodeCharacter0xdfa1': '\xed\xbe\xa1',
'SinhalaLetterAyanna': '\xe0\xb6\x85',
'UnicodeCharacter0xdfa3': '\xed\xbe\xa3',
'UnicodeCharacter0xdfa2': '\xed\xbe\xa2',
'UnicodeCharacter0xdfa5': '\xed\xbe\xa5',
'ModifierLetterCapitalI': '\xe1\xb4\xb5',
'UnicodeCharacter0xdfa7': '\xed\xbe\xa7',
'UnicodeCharacter0xdfa6': '\xed\xbe\xa6',
'YiSyllableGut': '\xea\x87\xb2',
'logicalnot': '\xc2\xac',
'HangulSyllablePeuc': '\xed\x94\x9b',
'UnicodeCharacter0xeb4d': '\xee\xad\x8d',
'UnicodeCharacter0xe1e0': '\xee\x87\xa0',
'ModifierLetterCapitalH': '\xe1\xb4\xb4',
'HangulJongseongYeorinhieuh': '\xe1\x87\xb9',
'CjkUnifiedIdeograph68dd': '\xe6\xa3\x9d',
'CjkUnifiedIdeograph68de': '\xe6\xa3\x9e',
'CjkUnifiedIdeograph68df': '\xe6\xa3\x9f',
'UnicodeCharacter0xeb4f': '\xee\xad\x8f',
'CjkUnifiedIdeograph68da': '\xe6\xa3\x9a',
'CjkUnifiedIdeograph68db': '\xe6\xa3\x9b',
'CjkUnifiedIdeograph68dc': '\xe6\xa3\x9c',
'CjkUnifiedIdeograph6aa9': '\xe6\xaa\xa9',
'CjkUnifiedIdeograph391e': '\xe3\xa4\x9e',
'CjkUnifiedIdeograph6ff2': '\xe6\xbf\xb2',
'UnicodeCharacter0xe226': '\xee\x88\xa6',
'CjkUnifiedIdeograph620b': '\xe6\x88\x8b',
'CjkUnifiedIdeograph620c': '\xe6\x88\x8c',
'CjkUnifiedIdeograph925f': '\xe9\x89\x9f',
'LimbuDigitThree': '\xe1\xa5\x89',
'CjkUnifiedIdeograph620f': '\xe6\x88\x8f',
'CjkUnifiedIdeograph925a': '\xe9\x89\x9a',
'CjkUnifiedIdeograph620d': '\xe6\x88\x8d',
'CjkUnifiedIdeograph620e': '\xe6\x88\x8e',
'HangulSyllablePeud': '\xed\x94\x8b',
'HeavyLowerRightshadowedWhiteRightwardsArrow': '\xe2\x9e\xad',
'HangulSyllableSyigg': '\xec\x8b\x82',
'DevanagariLetterTta': '\xe0\xa4\x9f',
'UnicodeCharacter0xe225': '\xee\x88\xa5',
'CjkUnifiedIdeograph39d8': '\xe3\xa7\x98',
'CjkUnifiedIdeograph956f': '\xe9\x95\xaf',
'ModifierLetterCapitalJ': '\xe1\xb4\xb6',
'BraillePatternDots13578': '\xe2\xa3\x95',
'TibetanSymbolRdoRje': '\xe0\xbf\x85',
'BalineseMusicalSymbolLefthandOpenPing': '\xe1\xad\xbc',
'CjkUnifiedIdeograph6203': '\xe6\x88\x83',
'CjkUnifiedIdeograph6200': '\xe6\x88\x80',
'CjkUnifiedIdeograph6201': '\xe6\x88\x81',
'MalayalamLetterPha': '\xe0\xb4\xab',
'MongolianLetterTodoU': '\xe1\xa1\x87',
'CjkUnifiedIdeograph6204': '\xe6\x88\x84',
'CjkUnifiedIdeograph6205': '\xe6\x88\x85',
'SylotiNagriLetterJo': '\xea\xa0\x8e',
'CjkUnifiedIdeograph6208': '\xe6\x88\x88',
'CjkUnifiedIdeograph6209': '\xe6\x88\x89',
'CjkUnifiedIdeograph9258': '\xe9\x89\x98',
'CjkUnifiedIdeograph9259': '\xe9\x89\x99',
'CjkUnifiedIdeograph68d4': '\xe6\xa3\x94',
'CjkUnifiedIdeograph68d5': '\xe6\xa3\x95',
'CjkUnifiedIdeograph68d6': '\xe6\xa3\x96',
'CjkUnifiedIdeograph68d7': '\xe6\xa3\x97',
'CjkUnifiedIdeograph68d0': '\xe6\xa3\x90',
'ArabicLetterAinWithTwoDotsVerticallyAbove': '\xdd\x9f',
'CjkUnifiedIdeograph68d2': '\xe6\xa3\x92',
'CjkUnifiedIdeograph68d3': '\xe6\xa3\x93',
'MongolianLetterTodoI': '\xe1\xa1\x85',
'CjkUnifiedIdeograph68d8': '\xe6\xa3\x98',
'CjkUnifiedIdeograph68d9': '\xe6\xa3\x99',
'MongolianLetterTodoO': '\xe1\xa1\x86',
'UnicodeCharacter0xdcee': '\xed\xb3\xae',
'HangulSyllableBbwaelp': '\xeb\xbe\x86',
'CjkUnifiedIdeograph7c93': '\xe7\xb2\x93',
'UnicodeCharacter0xf7b0': '\xef\x9e\xb0',
'CjkUnifiedIdeograph7c92': '\xe7\xb2\x92',
'CjkUnifiedIdeograph7c91': '\xe7\xb2\x91',
'UnicodeCharacter0xf5b2': '\xef\x96\xb2',
'CjkUnifiedIdeograph7c90': '\xe7\xb2\x90',
'LatinSmallLetterNWithTilde': '\xc3\xb1',
'CjkUnifiedIdeograph6b42': '\xe6\xad\x82',
'IdeographicTelegraphSymbolForMarch': '\xe3\x8b\x82',
'HangulSyllableWilp': '\xec\x9c\x92',
'CjkUnifiedIdeograph7c96': '\xe7\xb2\x96',
'ModifierLetterCapitalO': '\xe1\xb4\xbc',
'UnicodeCharacter0xdb8e': '\xed\xae\x8e',
'CjkUnifiedIdeograph7c95': '\xe7\xb2\x95',
'CjkUnifiedIdeograph7c94': '\xe7\xb2\x94',
'HangulSyllableGgwec': '\xeb\x80\x87',
'CjkUnifiedIdeograph715a': '\xe7\x85\x9a',
'UnicodeCharacter0xa6cd': '\xea\x9b\x8d',
'CjkUnifiedIdeograph715c': '\xe7\x85\x9c',
'CjkUnifiedIdeograph715b': '\xe7\x85\x9b',
'CjkUnifiedIdeograph715e': '\xe7\x85\x9e',
'CjkUnifiedIdeograph715d': '\xe7\x85\x9d',
'HangulSyllableBoem': '\xeb\xb5\x98',
'CjkUnifiedIdeograph715f': '\xe7\x85\x9f',
'HangulSyllableBoec': '\xeb\xb5\x9f',
'HangulSyllableBoeb': '\xeb\xb5\x99',
'UnicodeCharacter0xdb8a': '\xed\xae\x8a',
'HangulSyllableBoeg': '\xeb\xb5\x89',
'UnicodeCharacter0xa6cf': '\xea\x9b\x8f',
'ModifierLetterCapitalN': '\xe1\xb4\xba',
'UnicodeCharacter0xe344': '\xee\x8d\x84',
'ArmenianHyphen': '\xd6\x8a',
'HangulSyllableTang': '\xed\x83\x95',
'HangulSyllableSsweolt': '\xec\x92\x8d',
'HangulSyllableBoes': '\xeb\xb5\x9b',
'LatinSmallLetterOWithCircumflexAndGrave': '\xe1\xbb\x93',
'HangulSyllableTanh': '\xed\x83\x86',
'HangulSyllableBoep': '\xeb\xb5\xa2',
'UnicodeCharacter0xa6ca': '\xea\x9b\x8a',
'RingInEqualTo': '\xe2\x89\x96',
'UnicodeCharacter0xa6cb': '\xea\x9b\x8b',
'UnicodeCharacter0xe1e9': '\xee\x87\xa9',
'CjkUnifiedIdeograph82ed': '\xe8\x8b\xad',
'invnot': '\xe2\x8c\x90',
'UnicodeCharacter0xe345': '\xee\x8d\x85',
'HangulSyllableHyaelg': '\xed\x96\xb5',
'HangulSyllableDdan': '\xeb\x94\xb4',
'UnicodeCharacter0xd8ef': '\xed\xa3\xaf',
'HangulSyllableYels': '\xec\x98\x94',
'HangulSyllableDdal': '\xeb\x94\xb8',
'HangulSyllableSsweolm': '\xec\x92\x8a',
'UnicodeCharacter0xd8ed': '\xed\xa3\xad',
'UnicodeCharacter0xdd5e': '\xed\xb5\x9e',
'GreekCapitalLetterOmegaWithPsiliAndOxiaAndProsgegrammeni': '\xe1\xbe\xac',
'UnicodeCharacter0xe1e8': '\xee\x87\xa8',
'UnicodeCharacter0xd8ec': '\xed\xa3\xac',
'EthiopicSyllableNu': '\xe1\x8a\x91',
'CjkUnifiedIdeograph7cda': '\xe7\xb3\x9a',
'CjkUnifiedIdeograph7151': '\xe7\x85\x91',
'CjkUnifiedIdeograph7150': '\xe7\x85\x90',
'EthiopicSyllableNi': '\xe1\x8a\x92',
'RightwardsTwoheadedArrowWithTail': '\xe2\xa4\x96',
'EthiopicSyllableNo': '\xe1\x8a\x96',
'CjkUnifiedIdeograph7154': '\xe7\x85\x94',
'CjkUnifiedIdeograph7157': '\xe7\x85\x97',
'CjkUnifiedIdeograph7156': '\xe7\x85\x96',
'CjkUnifiedIdeograph7159': '\xe7\x85\x99',
'CjkUnifiedIdeograph7158': '\xe7\x85\x98',
'EthiopicSyllableNa': '\xe1\x8a\x90',
'EthiopicSyllableNe': '\xe1\x8a\x95',
'HangulSyllableSsweolh': '\xec\x92\x8f',
'UnicodeCharacter0xe1e5': '\xee\x87\xa5',
'UnicodeCharacter0xe1e4': '\xee\x87\xa4',
'UnicodeCharacter0xe1e7': '\xee\x87\xa7',
'UnicodeCharacter0xe1e6': '\xee\x87\xa6',
'CjkUnifiedIdeograph78b9': '\xe7\xa2\xb9',
'CjkUnifiedIdeograph78b8': '\xe7\xa2\xb8',
'UnicodeCharacter0xe1e3': '\xee\x87\xa3',
'UnicodeCharacter0xe1e2': '\xee\x87\xa2',
'CjkUnifiedIdeograph78b5': '\xe7\xa2\xb5',
'CopticSmallLetterOldCopticEsh': '\xe2\xb3\x87',
'CjkUnifiedIdeograph78b7': '\xe7\xa2\xb7',
'CjkUnifiedIdeograph78b6': '\xe7\xa2\xb6',
'CjkUnifiedIdeograph78b1': '\xe7\xa2\xb1',
'CjkUnifiedIdeograph78b0': '\xe7\xa2\xb0',
'CjkUnifiedIdeograph78b3': '\xe7\xa2\xb3',
'CjkUnifiedIdeograph78b2': '\xe7\xa2\xb2',
'HangulSyllableBbwaelh': '\xeb\xbe\x87',
'HangulSyllableBbwaelm': '\xeb\xbe\x82',
'ParenthesizedHangulHieuhA': '\xe3\x88\x9b',
'UnicodeCharacter0xf8fa': '\xef\xa3\xba',
'HangulSyllableBbwaelb': '\xeb\xbe\x83',
'HangulSyllableBbwaelg': '\xeb\xbe\x81',
'HangulSyllableTwilb': '\xed\x8a\x8b',
'UnicodeCharacter0xd7ca': '\xed\x9f\x8a',
'BengaliLetterCa': '\xe0\xa6\x9a',
'UnicodeCharacter0xef29': '\xee\xbc\xa9',
'UnicodeCharacter0xef28': '\xee\xbc\xa8',
'UnicodeCharacter0xe340': '\xee\x8d\x80',
'UnicodeCharacter0xef26': '\xee\xbc\xa6',
'UnicodeCharacter0xef25': '\xee\xbc\xa5',
'UnicodeCharacter0xef24': '\xee\xbc\xa4',
'UnicodeCharacter0xef23': '\xee\xbc\xa3',
'UnicodeCharacter0xef22': '\xee\xbc\xa2',
'UnicodeCharacter0xef21': '\xee\xbc\xa1',
'UnicodeCharacter0xe091': '\xee\x82\x91',
'UnicodeCharacter0xf5b8': '\xef\x96\xb8',
'WhiteLargeSquare': '\xe2\xac\x9c',
'UnicodeCharacter0xd7ce': '\xed\x9f\x8e',
'HangulSyllableTwilg': '\xed\x8a\x89',
'HangulSyllableJjyaegg': '\xec\xa8\xb2',
'HangulSyllableGeolb': '\xea\xb1\xbb',
'LatinSubscriptSmallLetterA': '\xe2\x82\x90',
'ArabicLigatureUighurKirghizYehWithHamzaAboveWithAlefMaksuraFinalForm': '\xef\xaf\xba',
'BraillePatternDots45678': '\xe2\xa3\xb8',
'UnicodeCharacter0xf6d1': '\xef\x9b\x91',
'HangulSyllableRwalp': '\xeb\xa2\x86',
'ThaanaEebeefili': '\xde\xa9',
'HangulSyllableGeolm': '\xea\xb1\xba',
'UnicodeCharacter0xe1ee': '\xee\x87\xae',
'HangulSyllableGeolp': '\xea\xb1\xbe',
'ArabicLetterOe': '\xdb\x86',
'UnicodeCharacter0xe1ef': '\xee\x87\xaf',
'UnicodeCharacter0xe1ea': '\xee\x87\xaa',
'HangulSyllableGeolt': '\xea\xb1\xbd',
'UnicodeCharacter0xe1ec': '\xee\x87\xac',
'UnicodeCharacter0xe1eb': '\xee\x87\xab',
'CjkUnifiedIdeograph78be': '\xe7\xa2\xbe',
'CjkUnifiedIdeograph78bd': '\xe7\xa2\xbd',
'CjkUnifiedIdeograph78bf': '\xe7\xa2\xbf',
'CjkUnifiedIdeograph78ba': '\xe7\xa2\xba',
'HangulSyllableYelm': '\xec\x98\x92',
'CjkUnifiedIdeograph78bc': '\xe7\xa2\xbc',
'CjkUnifiedIdeograph78bb': '\xe7\xa2\xbb',
'UnicodeCharacter0xe342': '\xee\x8d\x82',
'CjkUnifiedIdeograph8327': '\xe8\x8c\xa7',
'MalayalamNumberOneThousand': '\xe0\xb5\xb2',
'InvisiblePlus': '\xe2\x81\xa4',
'CjkUnifiedIdeograph663f': '\xe6\x98\xbf',
'UnicodeCharacter0xef2f': '\xee\xbc\xaf',
'UnicodeCharacter0xef2e': '\xee\xbc\xae',
'HangulSyllableYelh': '\xec\x98\x97',
'UnicodeCharacter0xef2c': '\xee\xbc\xac',
'HangulSyllableDyibs': '\xeb\x94\x8a',
'UnicodeCharacter0xef2a': '\xee\xbc\xaa',
'HangulSyllableBbeul': '\xec\x81\xa0',
'UnicodeCharacter0xe343': '\xee\x8d\x83',
'HangulSyllableJinh': '\xec\xa7\x86',
'SyriacLetterTaw': '\xdc\xac',
'HangulSyllableJing': '\xec\xa7\x95',
'HangulSyllableKig': '\xed\x82\xa5',
'CjkUnifiedIdeograph663b': '\xe6\x98\xbb',
'CjkUnifiedIdeograph6a7a': '\xe6\xa9\xba',
'CjkUnifiedIdeograph6a7b': '\xe6\xa9\xbb',
'CjkUnifiedIdeograph6a7c': '\xe6\xa9\xbc',
'CjkUnifiedIdeograph6a7d': '\xe6\xa9\xbd',
'CjkUnifiedIdeograph6a7e': '\xe6\xa9\xbe',
'KangxiRadicalSprout': '\xe2\xbc\xac',
'HangulSyllableBbeum': '\xec\x81\xa8',
'CjkUnifiedIdeograph6787': '\xe6\x9e\x87',
'CjkUnifiedIdeograph6786': '\xe6\x9e\x86',
'CjkUnifiedIdeograph6785': '\xe6\x9e\x85',
'CjkUnifiedIdeograph6784': '\xe6\x9e\x84',
'BoxDrawingsHeavyLeftAndLightRight': '\xe2\x95\xbe',
'CjkUnifiedIdeograph6782': '\xe6\x9e\x82',
'CjkUnifiedIdeograph6781': '\xe6\x9e\x81',
'CjkUnifiedIdeograph6780': '\xe6\x9e\x80',
'CjkUnifiedIdeograph86ac': '\xe8\x9a\xac',
'CjkUnifiedIdeograph86ab': '\xe8\x9a\xab',
'CjkUnifiedIdeograph86aa': '\xe8\x9a\xaa',
'CjkUnifiedIdeograph86af': '\xe8\x9a\xaf',
'CjkUnifiedIdeograph6789': '\xe6\x9e\x89',
'CjkUnifiedIdeograph6788': '\xe6\x9e\x88',
'UnicodeCharacter0xdf9a': '\xed\xbe\x9a',
'UnicodeCharacter0xf7f3': '\xef\x9f\xb3',
'ArabicLetterRehWithStroke': '\xdd\x9b',
'CjkUnifiedIdeograph66ff': '\xe6\x9b\xbf',
'UnicodeCharacter0xf7f2': '\xef\x9f\xb2',
'HangulSyllableEolm': '\xec\x96\xbe',
'KangxiRadicalSound': '\xe2\xbe\xb3',
'ModifierLetterShortEqualsSign': '\xea\x9e\x8a',
'UnicodeCharacter0xf61f': '\xef\x98\x9f',
'CjkUnifiedIdeograph6a70': '\xe6\xa9\xb0',
'IdeographicTelegraphSymbolForHourZero': '\xe3\x8d\x98',
'CjkUnifiedIdeograph6a72': '\xe6\xa9\xb2',
'GeorgianSmallLetterLas': '\xe2\xb4\x8a',
'CjkUnifiedIdeograph6a74': '\xe6\xa9\xb4',
'CjkUnifiedIdeograph6a75': '\xe6\xa9\xb5',
'CjkUnifiedIdeograph6a76': '\xe6\xa9\xb6',
'CjkUnifiedIdeograph6a77': '\xe6\xa9\xb7',
'CjkUnifiedIdeograph6a78': '\xe6\xa9\xb8',
'CjkUnifiedIdeograph6a79': '\xe6\xa9\xb9',
'UnicodeCharacter0xf61d': '\xef\x98\x9d',
'UnicodeCharacter0xa6c6': '\xea\x9b\x86',
'UnicodeCharacter0xa6c7': '\xea\x9b\x87',
'UnicodeCharacter0xf7f5': '\xef\x9f\xb5',
'LimbuSmallLetterRa': '\xe1\xa4\xb7',
'UnicodeCharacter0xa6c0': '\xea\x9b\x80',
'UnicodeCharacter0xf7f4': '\xef\x9f\xb4',
'UnicodeCharacter0xa6c1': '\xea\x9b\x81',
'UnicodeCharacter0xa6c2': '\xea\x9b\x82',
'UnicodeCharacter0xa6c3': '\xea\x9b\x83',
'CjkUnifiedIdeograph678f': '\xe6\x9e\x8f',
'CjkUnifiedIdeograph678e': '\xe6\x9e\x8e',
'CjkUnifiedIdeograph678d': '\xe6\x9e\x8d',
'CjkUnifiedIdeograph678c': '\xe6\x9e\x8c',
'CjkUnifiedIdeograph678b': '\xe6\x9e\x8b',
'HebrewPunctuationGeresh': '\xd7\xb3',
'UnicodeCharacter0xd8e6': '\xed\xa3\xa6',
'CjkUnifiedIdeograph86a3': '\xe8\x9a\xa3',
'CjkUnifiedIdeograph86a2': '\xe8\x9a\xa2',
'CjkUnifiedIdeograph86a1': '\xe8\x9a\xa1',
'CjkUnifiedIdeograph86a0': '\xe8\x9a\xa0',
'CjkUnifiedIdeograph86a7': '\xe8\x9a\xa7',
'CjkUnifiedIdeograph7cd6': '\xe7\xb3\x96',
'CjkUnifiedIdeograph86a5': '\xe8\x9a\xa5',
'CjkUnifiedIdeograph86a4': '\xe8\x9a\xa4',
'Ps': '\xce\xa8',
'UnicodeCharacter0xd8e4': '\xed\xa3\xa4',
'UnicodeCharacter0xf7eb': '\xef\x9f\xab',
'MyanmarVowelSignShanE': '\xe1\x82\x84',
'HangulSyllableWaegs': '\xec\x99\x9f',
'UnicodeCharacter0xd8e1': '\xed\xa3\xa1',
'CjkUnifiedIdeograph89ad': '\xe8\xa6\xad',
'CjkUnifiedIdeograph89ae': '\xe8\xa6\xae',
'CjkUnifiedIdeograph89af': '\xe8\xa6\xaf',
'UnicodeCharacter0xd8e0': '\xed\xa3\xa0',
'differentiald': '\xe2\x85\x86',
'CjkUnifiedIdeograph89aa': '\xe8\xa6\xaa',
'CjkUnifiedIdeograph89ab': '\xe8\xa6\xab',
'CjkUnifiedIdeograph89ac': '\xe8\xa6\xac',
'Pallas': '\xe2\x9a\xb4',
'CircledIdeographRight': '\xe3\x8a\xa8',
'OghamLetterMuin': '\xe1\x9a\x8b',
'UnicodeCharacter0xfbba': '\xef\xae\xba',
'UnicodeCharacter0xe09d': '\xee\x82\x9d',
'HangulSyllableBbeuj': '\xec\x81\xae',
'differentialD': '\xe2\x85\x85',
'UnicodeCharacter0xe09f': '\xee\x82\x9f',
'ArmenianSmallLetterJa': '\xd5\xb1',
'CjkUnifiedIdeograph89a9': '\xe8\xa6\xa9',
'HangulJongseongChieuch': '\xe1\x86\xbe',
'GeorgianCapitalLetterIn': '\xe1\x82\xa8',
'CjkUnifiedIdeograph89a5': '\xe8\xa6\xa5',
'TeluguDigitSeven': '\xe0\xb1\xad',
'CjkUnifiedIdeograph89a7': '\xe8\xa6\xa7',
'CjkUnifiedIdeograph89a0': '\xe8\xa6\xa0',
'CjkUnifiedIdeograph89a1': '\xe8\xa6\xa1',
'CjkUnifiedIdeograph89a2': '\xe8\xa6\xa2',
'CjkUnifiedIdeograph7599': '\xe7\x96\x99',
'CjkUnifiedIdeograph7c9c': '\xe7\xb2\x9c',
'HangulSyllableBbeuk': '\xec\x81\xb0',
'LeftwardsArrowThroughSubset': '\xe2\xa5\xba',
'HangulSyllableTyeod': '\xed\x85\xaf',
'TamilDigitFive': '\xe0\xaf\xab',
'UnicodeCharacter0xdb16': '\xed\xac\x96',
'CjkUnifiedIdeograph7c9a': '\xe7\xb2\x9a',
'VaiSyllableMo': '\xea\x97\x9e',
'ArabicZwarakay': '\xd9\x99',
'CjkUnifiedIdeograph7c9f': '\xe7\xb2\x9f',
'CanadianSyllabicsPaai': '\xe1\x90\xb0',
'StarOfDavid': '\xe2\x9c\xa1',
'HangulSyllableJjwi': '\xec\xae\x9c',
'CjkUnifiedIdeograph7c9d': '\xe7\xb2\x9d',
'CjkUnifiedIdeograph8df6': '\xe8\xb7\xb6',
'PreviousPage': '\xe2\x8e\x97',
'ArabicLigatureSheenWithKhahInitialForm': '\xef\xb4\xaf',
'VaiSyllableMa': '\xea\x95\xae',
'UnicodeCharacter0xdb80': '\xed\xae\x80',
'ArabicLetterAlefFinalForm': '\xef\xba\x8e',
'CjkCompatibilityIdeographfa0a': '\xef\xa8\x8a',
'BalineseVowelSignSuku': '\xe1\xac\xb8',
'CjkUnifiedIdeograph8df2': '\xe8\xb7\xb2',
'HangulSyllableSsyogg': '\xec\x91\x8a',
'VaiSyllableMe': '\xea\x98\x88',
'HangulSyllableJjass': '\xec\xa7\xb0',
'HangulSyllableOebs': '\xec\x9a\x8a',
'HangulSyllableWelt': '\xec\x9b\xb5',
'CjkUnifiedIdeograph46a9': '\xe4\x9a\xa9',
'CjkUnifiedIdeograph46a8': '\xe4\x9a\xa8',
'CjkUnifiedIdeograph46a7': '\xe4\x9a\xa7',
'CjkUnifiedIdeograph46a6': '\xe4\x9a\xa6',
'CjkUnifiedIdeograph46a5': '\xe4\x9a\xa5',
'CjkUnifiedIdeograph46a4': '\xe4\x9a\xa4',
'CjkUnifiedIdeograph46a3': '\xe4\x9a\xa3',
'CjkUnifiedIdeograph46a2': '\xe4\x9a\xa2',
'CjkUnifiedIdeograph46a1': '\xe4\x9a\xa1',
'CjkUnifiedIdeograph46a0': '\xe4\x9a\xa0',
'CjkUnifiedIdeograph4b79': '\xe4\xad\xb9',
'UnicodeCharacter0xe7c3': '\xee\x9f\x83',
'UnicodeCharacter0xa96b': '\xea\xa5\xab',
'SupersetOfWithNotEqualTo': '\xe2\x8a\x8b',
'CjkUnifiedIdeograph6496': '\xe6\x92\x96',
'HangulSyllableJjeult': '\xec\xaf\xa1',
'HangulSyllableJol': '\xec\xa1\xb8',
'HangulSyllableJjeulp': '\xec\xaf\xa2',
'HangulSyllableJjeuls': '\xec\xaf\xa0',
'UnicodeCharacter0xf612': '\xef\x98\x92',
'HangulSyllableGwep': '\xea\xb6\xbe',
'HangulSyllableDyels': '\xeb\x8e\xb4',
'OriyaAuLengthMark': '\xe0\xad\x97',
'HangulSyllableGwes': '\xea\xb6\xb7',
'HangulSyllableGwet': '\xea\xb6\xbd',
'UnicodeCharacter0xf611': '\xef\x98\x91',
'HangulSyllableGweh': '\xea\xb6\xbf',
'HangulSyllableGwej': '\xea\xb6\xba',
'HangulSyllableGwek': '\xea\xb6\xbc',
'HangulSyllableGwel': '\xea\xb6\xac',
'HangulSyllableGwem': '\xea\xb6\xb4',
'HangulSyllableGwen': '\xea\xb6\xa8',
'HangulSyllableGweo': '\xea\xb6\x88',
'HangulSyllableJjeulm': '\xec\xaf\x9e',
'CjkUnifiedIdeograph5a9c': '\xe5\xaa\x9c',
'HangulSyllableGweb': '\xea\xb6\xb5',
'HangulSyllableGwec': '\xea\xb6\xbb',
'HangulSyllableGwed': '\xea\xb6\xab',
'YiSyllableHmut': '\xea\x82\xa3',
'CjkUnifiedIdeograph60c8': '\xe6\x83\x88',
'HangulSyllablePaelh': '\xed\x8c\xb7',
'UnicodeCharacter0xf1f9': '\xef\x87\xb9',
'HangulSyllablePaelm': '\xed\x8c\xb2',
'BopomofoLetterAinn': '\xe3\x86\xae',
'YiSyllableHmur': '\xea\x82\xa8',
'HangulSyllablePaelb': '\xed\x8c\xb3',
'HangulSyllablePaelg': '\xed\x8c\xb1',
'HangulSyllablePaelp': '\xed\x8c\xb6',
'CjkUnifiedIdeograph46af': '\xe4\x9a\xaf',
'CjkUnifiedIdeograph46ae': '\xe4\x9a\xae',
'CjkUnifiedIdeograph46ad': '\xe4\x9a\xad',
'CjkUnifiedIdeograph46ac': '\xe4\x9a\xac',
'CjkUnifiedIdeograph46ab': '\xe4\x9a\xab',
'LatinSmallLetterSharpS': '\xc3\x9f',
'ArabicLigatureAinWithAlefMaksuraFinalForm': '\xef\xb4\x93',
'UnicodeCharacter0xf618': '\xef\x98\x98',
'CjkUnifiedIdeograph580e': '\xe5\xa0\x8e',
'CjkUnifiedIdeograph580d': '\xe5\xa0\x8d',
'CjkUnifiedIdeograph883e': '\xe8\xa0\xbe',
'CjkUnifiedIdeograph580f': '\xe5\xa0\x8f',
'CjkUnifiedIdeograph580a': '\xe5\xa0\x8a',
'CjkUnifiedIdeograph883b': '\xe8\xa0\xbb',
'CjkUnifiedIdeograph580c': '\xe5\xa0\x8c',
'CjkUnifiedIdeograph580b': '\xe5\xa0\x8b',
'CjkUnifiedIdeograph52e8': '\xe5\x8b\xa8',
'CjkUnifiedIdeograph52e9': '\xe5\x8b\xa9',
'IdeographicDepartingToneMark': '\xe3\x80\xac',
'UnicodeCharacter0xe358': '\xee\x8d\x98',
'CjkUnifiedIdeograph52e0': '\xe5\x8b\xa0',
'CjkUnifiedIdeograph52e1': '\xe5\x8b\xa1',
'CjkUnifiedIdeograph52e2': '\xe5\x8b\xa2',
'CjkUnifiedIdeograph52e3': '\xe5\x8b\xa3',
'CjkUnifiedIdeograph52e4': '\xe5\x8b\xa4',
'CjkUnifiedIdeograph52e5': '\xe5\x8b\xa5',
'CjkUnifiedIdeograph52e6': '\xe5\x8b\xa6',
'CjkUnifiedIdeograph52e7': '\xe5\x8b\xa7',
'HangulSyllableJoec': '\xec\xa3\x9b',
'HangulSyllableJoeb': '\xec\xa3\x95',
'HangulSyllableJoeg': '\xec\xa3\x85',
'LatinCapitalLetterNWithLineBelow': '\xe1\xb9\x88',
'UnicodeCharacter0xf7aa': '\xef\x9e\xaa',
'HangulSyllableJoed': '\xec\xa3\x8b',
'CjkUnifiedIdeograph5c0f': '\xe5\xb0\x8f',
'HangulSyllableJoej': '\xec\xa3\x9a',
'CjkUnifiedIdeograph5c0d': '\xe5\xb0\x8d',
'CjkUnifiedIdeograph5c0e': '\xe5\xb0\x8e',
'CjkUnifiedIdeograph5c0b': '\xe5\xb0\x8b',
'CjkUnifiedIdeograph5c0c': '\xe5\xb0\x8c',
'HangulSyllableJoem': '\xec\xa3\x94',
'TamilAsAboveSign': '\xe0\xaf\xb8',
'CjkUnifiedIdeograph41a0': '\xe4\x86\xa0',
'CjkUnifiedIdeograph41a1': '\xe4\x86\xa1',
'CjkUnifiedIdeograph41a2': '\xe4\x86\xa2',
'CjkUnifiedIdeograph41a3': '\xe4\x86\xa3',
'CjkUnifiedIdeograph41a4': '\xe4\x86\xa4',
'CjkUnifiedIdeograph41a5': '\xe4\x86\xa5',
'CjkUnifiedIdeograph41a6': '\xe4\x86\xa6',
'CjkUnifiedIdeograph41a7': '\xe4\x86\xa7',
'CjkUnifiedIdeograph41a8': '\xe4\x86\xa8',
'CjkUnifiedIdeograph41a9': '\xe4\x86\xa9',
'LatinCapitalLetterUWithGrave': '\xc3\x99',
'UnicodeCharacter0xf71e': '\xef\x9c\x9e',
'UnicodeCharacter0xf71b': '\xef\x9c\x9b',
'UnicodeCharacter0xf71c': '\xef\x9c\x9c',
'UnicodeCharacter0xf71a': '\xef\x9c\x9a',
'HangulSyllableMwaegs': '\xeb\xab\xa3',
'GujaratiVowelCandraO': '\xe0\xaa\x91',
'UnicodeCharacter0xf6df': '\xef\x9b\x9f',
'RunicLetterCweorth': '\xe1\x9b\xa2',
'UnicodeCharacter0xe35c': '\xee\x8d\x9c',
'CjkUnifiedIdeograph52ea': '\xe5\x8b\xaa',
'CjkUnifiedIdeograph52eb': '\xe5\x8b\xab',
'CjkUnifiedIdeograph52ec': '\xe5\x8b\xac',
'CjkUnifiedIdeograph52ed': '\xe5\x8b\xad',
'CjkUnifiedIdeograph52ee': '\xe5\x8b\xae',
'GujaratiVowelCandraE': '\xe0\xaa\x8d',
'UnicodeCharacter0xe35d': '\xee\x8d\x9d',
'HangulChoseongChitueumssangcieuc': '\xe1\x85\x8f',
'CjkUnifiedIdeograph5808': '\xe5\xa0\x88',
'CjkUnifiedIdeograph8839': '\xe8\xa0\xb9',
'CjkUnifiedIdeograph8838': '\xe8\xa0\xb8',
'CjkUnifiedIdeograph5805': '\xe5\xa0\x85',
'CjkUnifiedIdeograph5804': '\xe5\xa0\x84',
'CjkUnifiedIdeograph5807': '\xe5\xa0\x87',
'CjkUnifiedIdeograph5806': '\xe5\xa0\x86',
'CjkUnifiedIdeograph5801': '\xe5\xa0\x81',
'CjkUnifiedIdeograph5800': '\xe5\xa0\x80',
'CjkUnifiedIdeograph5803': '\xe5\xa0\x83',
'CjkUnifiedIdeograph5802': '\xe5\xa0\x82',
'CjkUnifiedIdeograph41aa': '\xe4\x86\xaa',
'CjkUnifiedIdeograph41ab': '\xe4\x86\xab',
'CjkUnifiedIdeograph41ac': '\xe4\x86\xac',
'CjkUnifiedIdeograph41ad': '\xe4\x86\xad',
'CjkUnifiedIdeograph41ae': '\xe4\x86\xae',
'CjkUnifiedIdeograph41af': '\xe4\x86\xaf',
'UnicodeCharacter0xf719': '\xef\x9c\x99',
'UnicodeCharacter0xf716': '\xef\x9c\x96',
'UnicodeCharacter0xf717': '\xef\x9c\x97',
'UnicodeCharacter0xf714': '\xef\x9c\x94',
'UnicodeCharacter0xf715': '\xef\x9c\x95',
'UnicodeCharacter0xf712': '\xef\x9c\x92',
'HangulSyllableDdwagg': '\xeb\x98\xae',
'UnicodeCharacter0xf710': '\xef\x9c\x90',
'ChamLetterU': '\xea\xa8\x82',
'ChamLetterI': '\xea\xa8\x81',
'ChamLetterO': '\xea\xa8\x85',
'CjkUnifiedIdeograph5c08': '\xe5\xb0\x88',
'CjkUnifiedIdeograph5c09': '\xe5\xb0\x89',
'CjkUnifiedIdeograph5c06': '\xe5\xb0\x86',
'CjkUnifiedIdeograph5c07': '\xe5\xb0\x87',
'CjkUnifiedIdeograph5c04': '\xe5\xb0\x84',
'CjkUnifiedIdeograph5c05': '\xe5\xb0\x85',
'RunicLetterAesc': '\xe1\x9a\xab',
'CjkUnifiedIdeograph5c03': '\xe5\xb0\x83',
'CjkUnifiedIdeograph5c00': '\xe5\xb0\x80',
'CjkUnifiedIdeograph5c01': '\xe5\xb0\x81',
'CopticSmallLetterThethe': '\xe2\xb2\x91',
'CjkUnifiedIdeograph8789': '\xe8\x9e\x89',
'CjkUnifiedIdeograph8788': '\xe8\x9e\x88',
'CjkUnifiedIdeograph9d87': '\xe9\xb6\x87',
'CjkUnifiedIdeograph9d86': '\xe9\xb6\x86',
'CjkUnifiedIdeograph9d81': '\xe9\xb6\x81',
'CjkUnifiedIdeograph9d80': '\xe9\xb6\x80',
'CjkUnifiedIdeograph9d83': '\xe9\xb6\x83',
'CjkUnifiedIdeograph9743': '\xe9\x9d\x83',
'CjkUnifiedIdeograph8781': '\xe8\x9e\x81',
'CjkUnifiedIdeograph8780': '\xe8\x9e\x80',
'CjkUnifiedIdeograph8783': '\xe8\x9e\x83',
'CjkUnifiedIdeograph8782': '\xe8\x9e\x82',
'CjkUnifiedIdeograph8785': '\xe8\x9e\x85',
'CjkUnifiedIdeograph8784': '\xe8\x9e\x84',
'CjkUnifiedIdeograph8787': '\xe8\x9e\x87',
'CjkUnifiedIdeograph8786': '\xe8\x9e\x86',
'HangulSyllableHyonh': '\xed\x9a\xae',
'HangulSyllableToess': '\xed\x88\x88',
'HangulSyllableSweg': '\xec\x89\x91',
'HangulSyllableByeb': '\xeb\xb3\xa9',
'UnicodeCharacter0xdd18': '\xed\xb4\x98',
'HangulSyllableSsyeolb': '\xec\x8e\xab',
'OlChikiLetterEdd': '\xe1\xb1\xb0',
'UnicodeCharacter0xf7fe': '\xef\x9f\xbe',
'CjkUnifiedIdeograph9745': '\xe9\x9d\x85',
'HangulSyllableJass': '\xec\x9e\xa4',
'UnicodeCharacter0xd8d9': '\xed\xa3\x99',
'CjkUnifiedIdeograph9d8e': '\xe9\xb6\x8e',
'RightwardsArrowWithTipDownwards': '\xe2\xac\x8e',
'BraillePatternDots134567': '\xe2\xa1\xbd',
'CjkUnifiedIdeograph9d8a': '\xe9\xb6\x8a',
'CjkUnifiedIdeograph9d8c': '\xe9\xb6\x8c',
'CjkUnifiedIdeograph9d8b': '\xe9\xb6\x8b',
'CjkUnifiedIdeograph878a': '\xe8\x9e\x8a',
'HangulSyllablePyut': '\xed\x94\x81',
'CjkUnifiedIdeograph878c': '\xe8\x9e\x8c',
'CjkUnifiedIdeograph878b': '\xe8\x9e\x8b',
'BraillePatternDots134568': '\xe2\xa2\xbd',
'CjkUnifiedIdeograph878d': '\xe8\x9e\x8d',
'CjkUnifiedIdeograph878f': '\xe8\x9e\x8f',
'CjkUnifiedIdeograph93b0': '\xe9\x8e\xb0',
'ArabicLigatureTehWithKhahWithYehFinalForm': '\xef\xb6\xa1',
'HangulSyllableSyaet': '\xec\x84\x99',
'HangulSyllableSyaep': '\xec\x84\x9a',
'DoublestruckItalicSmallI': '\xe2\x85\x88',
'DoublestruckItalicSmallJ': '\xe2\x85\x89',
'HangulSyllableSyaes': '\xec\x84\x93',
'DoublestruckItalicSmallD': '\xe2\x85\x86',
'ModifierLetterSmallBarredO': '\xe1\xb6\xb1',
'UnicodeCharacter0xe92f': '\xee\xa4\xaf',
'ArabicLigatureKafWithLamFinalForm': '\xef\xb2\x81',
'CjkUnifiedIdeograph93b5': '\xe9\x8e\xb5',
'HangulSyllableSyaed': '\xec\x84\x87',
'HangulSyllableSyaeg': '\xec\x84\x81',
'HangulSyllableSyaeb': '\xec\x84\x91',
'HangulSyllableSyaec': '\xec\x84\x97',
'HangulSyllableSyael': '\xec\x84\x88',
'HangulSyllableSyaem': '\xec\x84\x90',
'HangulSyllableSyaen': '\xec\x84\x84',
'HangulSyllableHyong': '\xed\x9a\xbd',
'HangulSyllableSyaeh': '\xec\x84\x9b',
'HangulSyllableSweb': '\xec\x89\xa1',
'HangulSyllableSyaej': '\xec\x84\x96',
'HangulSyllableSyaek': '\xec\x84\x98',
'CjkUnifiedIdeograph4033': '\xe4\x80\xb3',
'CjkUnifiedIdeograph4032': '\xe4\x80\xb2',
'CjkUnifiedIdeograph4031': '\xe4\x80\xb1',
'CjkUnifiedIdeograph4030': '\xe4\x80\xb0',
'CyrillicCapitalLetterYi': '\xd0\x87',
'CjkUnifiedIdeograph4036': '\xe4\x80\xb6',
'BalineseDigitThree': '\xe1\xad\x93',
'CjkUnifiedIdeograph4034': '\xe4\x80\xb4',
'BottomArcAnticlockwiseArrow': '\xe2\xa4\xbb',
'CjkUnifiedIdeograph4039': '\xe4\x80\xb9',
'CjkUnifiedIdeograph4038': '\xe4\x80\xb8',
'CyrillicCapitalLetterYa': '\xd0\xaf',
'CjkUnifiedIdeograph3cd0': '\xe3\xb3\x90',
'CjkUnifiedIdeograph3cd1': '\xe3\xb3\x91',
'CjkUnifiedIdeograph3cd2': '\xe3\xb3\x92',
'CjkUnifiedIdeograph3cd3': '\xe3\xb3\x93',
'CjkUnifiedIdeograph3cd4': '\xe3\xb3\x94',
'CjkUnifiedIdeograph3cd5': '\xe3\xb3\x95',
'CjkUnifiedIdeograph3cd6': '\xe3\xb3\x96',
'RunicLetterEar': '\xe1\x9b\xa0',
'CjkUnifiedIdeograph478e': '\xe4\x9e\x8e',
'CjkUnifiedIdeograph3cd9': '\xe3\xb3\x99',
'CjkUnifiedIdeograph478f': '\xe4\x9e\x8f',
'CjkUnifiedIdeograph478a': '\xe4\x9e\x8a',
'CjkUnifiedIdeograph478c': '\xe4\x9e\x8c',
'CjkUnifiedIdeograph478b': '\xe4\x9e\x8b',
'UnicodeCharacter0xe43f': '\xee\x90\xbf',
'UnicodeCharacter0xe43d': '\xee\x90\xbd',
'UnicodeCharacter0xe43e': '\xee\x90\xbe',
'UnicodeCharacter0xe43b': '\xee\x90\xbb',
'UnicodeCharacter0xe43c': '\xee\x90\xbc',
'UnicodeCharacter0xe43a': '\xee\x90\xba',
'ArabicLigatureJeemWithYehFinalForm': '\xef\xb4\x9e',
'UnicodeCharacter0xe2cd': '\xee\x8b\x8d',
'HeavySparkle': '\xe2\x9d\x88',
'UnicodeCharacter0xe2cf': '\xee\x8b\x8f',
'UnicodeCharacter0xe2ca': '\xee\x8b\x8a',
'UnicodeCharacter0xe2cb': '\xee\x8b\x8b',
'UnicodeCharacter0xe2cc': '\xee\x8b\x8c',
'CjkUnifiedIdeograph4ea4': '\xe4\xba\xa4',
'CjkUnifiedIdeograph3cda': '\xe3\xb3\x9a',
'CjkUnifiedIdeograph4ea6': '\xe4\xba\xa6',
'CjkUnifiedIdeograph3cdc': '\xe3\xb3\x9c',
'CjkUnifiedIdeograph4ea0': '\xe4\xba\xa0',
'CjkUnifiedIdeograph4ea1': '\xe4\xba\xa1',
'CjkUnifiedIdeograph3cdf': '\xe3\xb3\x9f',
'CjkUnifiedIdeograph4ea3': '\xe4\xba\xa3',
'CjkUnifiedIdeograph4785': '\xe4\x9e\x85',
'CjkUnifiedIdeograph4784': '\xe4\x9e\x84',
'CjkUnifiedIdeograph4787': '\xe4\x9e\x87',
'CjkUnifiedIdeograph4786': '\xe4\x9e\x86',
'CjkUnifiedIdeograph4ea8': '\xe4\xba\xa8',
'CjkUnifiedIdeograph4ea9': '\xe4\xba\xa9',
'CjkUnifiedIdeograph4783': '\xe4\x9e\x83',
'CjkUnifiedIdeograph4782': '\xe4\x9e\x82',
'CjkUnifiedIdeograph403c': '\xe4\x80\xbc',
'CjkUnifiedIdeograph403b': '\xe4\x80\xbb',
'CjkUnifiedIdeograph403a': '\xe4\x80\xba',
'CjkUnifiedIdeograph403f': '\xe4\x80\xbf',
'CjkUnifiedIdeograph403e': '\xe4\x80\xbe',
'CjkUnifiedIdeograph403d': '\xe4\x80\xbd',
'HangulSyllableSyu': '\xec\x8a\x88',
'UnicodeCharacter0xe2c8': '\xee\x8b\x88',
'UnicodeCharacter0xe2c9': '\xee\x8b\x89',
'UnicodeCharacter0xe2c4': '\xee\x8b\x84',
'LatinCapitalLetterAWithCircumflexAndHookAbove': '\xe1\xba\xa8',
'UnicodeCharacter0xe2c6': '\xee\x8b\x86',
'UnicodeCharacter0xe2c7': '\xee\x8b\x87',
'UnicodeCharacter0xe2c0': '\xee\x8b\x80',
'UnicodeCharacter0xe2c1': '\xee\x8b\x81',
'UnicodeCharacter0xe2c2': '\xee\x8b\x82',
'UnicodeCharacter0xe2c3': '\xee\x8b\x83',
'CanadianSyllabicsWestcreeNwa': '\xe1\x93\x8c',
'CanadianSyllabicsWestcreeNwe': '\xe1\x93\x8a',
'CjkUnifiedIdeograph818c': '\xe8\x86\x8c',
'UnicodeCharacter0x2e33': '\xe2\xb8\xb3',
'UnicodeCharacter0xe436': '\xee\x90\xb6',
'UnicodeCharacter0xda00': '\xed\xa8\x80',
'UnicodeCharacter0xe434': '\xee\x90\xb4',
'UnicodeCharacter0xe435': '\xee\x90\xb5',
'UnicodeCharacter0xe432': '\xee\x90\xb2',
'UnicodeCharacter0xe433': '\xee\x90\xb3',
'UnicodeCharacter0xe430': '\xee\x90\xb0',
'LatinSmallLetterOWithTildeAndAcute': '\xe1\xb9\x8d',
'CjkUnifiedIdeograph9492': '\xe9\x92\x92',
'CjkUnifiedIdeograph9493': '\xe9\x92\x93',
'CjkUnifiedIdeograph9490': '\xe9\x92\x90',
'CjkUnifiedIdeograph9491': '\xe9\x92\x91',
'CjkUnifiedIdeograph9496': '\xe9\x92\x96',
'CjkUnifiedIdeograph9497': '\xe9\x92\x97',
'CjkUnifiedIdeograph9494': '\xe9\x92\x94',
'CjkUnifiedIdeograph9495': '\xe9\x92\x95',
'SymbolForVerticalTabulation': '\xe2\x90\x8b',
'CjkUnifiedIdeograph9498': '\xe9\x92\x98',
'CjkUnifiedIdeograph9499': '\xe9\x92\x99',
'UnicodeCharacter0xf7b9': '\xef\x9e\xb9',
'CjkUnifiedIdeograph92e0': '\xe9\x8b\xa0',
'HangulSyllableDyog': '\xeb\x90\xb5',
'UnicodeCharacter0xf4eb': '\xef\x93\xab',
'.': '\xe2\x8b\x85',
'CjkUnifiedIdeograph75f4': '\xe7\x97\xb4',
'TopArcClockwiseArrowWithMinus': '\xe2\xa4\xbc',
'HangulSyllableRup': '\xeb\xa4\x82',
'HangulSyllableRus': '\xeb\xa3\xbb',
'HangulSyllableRut': '\xeb\xa4\x81',
'HangulSyllableRwalm': '\xeb\xa2\x82',
'CjkUnifiedIdeograph5782': '\xe5\x9e\x82',
'CjkUnifiedIdeograph5783': '\xe5\x9e\x83',
'CjkUnifiedIdeograph5780': '\xe5\x9e\x80',
'CjkUnifiedIdeograph5781': '\xe5\x9e\x81',
'CjkUnifiedIdeograph5786': '\xe5\x9e\x86',
'CjkUnifiedIdeograph5787': '\xe5\x9e\x87',
'CjkUnifiedIdeograph5784': '\xe5\x9e\x84',
'CjkUnifiedIdeograph5785': '\xe5\x9e\x85',
'CjkUnifiedIdeograph5788': '\xe5\x9e\x88',
'CjkUnifiedIdeograph5789': '\xe5\x9e\x89',
'HangulSyllableRud': '\xeb\xa3\xaf',
'HangulSyllableRug': '\xeb\xa3\xa9',
'CjkUnifiedIdeograph9ea7': '\xe9\xba\xa7',
'CjkUnifiedIdeograph75f5': '\xe7\x97\xb5',
'CjkUnifiedIdeograph949b': '\xe9\x92\x9b',
'CjkUnifiedIdeograph758f': '\xe7\x96\x8f',
'UnicodeCharacter0xaa66': '\xea\xa9\xa6',
'MongolianLetterAliGaliNna': '\xe1\xa2\x8f',
'CjkUnifiedIdeograph949f': '\xe9\x92\x9f',
'UnicodeCharacter0xaa61': '\xea\xa9\xa1',
'CjkUnifiedIdeograph949d': '\xe9\x92\x9d',
'CjkUnifiedIdeograph949e': '\xe9\x92\x9e',
'UnicodeCharacter0x192c': '\xe1\xa4\xac',
'UnicodeCharacter0x192d': '\xe1\xa4\xad',
'UnicodeCharacter0x192e': '\xe1\xa4\xae',
'UnicodeCharacter0x192f': '\xe1\xa4\xaf',
'CjkUnifiedIdeograph578b': '\xe5\x9e\x8b',
'CjkUnifiedIdeograph578c': '\xe5\x9e\x8c',
'YiSyllableO': '\xea\x80\x91',
'SundaneseLetterSya': '\xe1\xae\xaf',
'CjkUnifiedIdeograph578f': '\xe5\x9e\x8f',
'CjkUnifiedIdeograph9ea1': '\xe9\xba\xa1',
'CjkUnifiedIdeograph578d': '\xe5\x9e\x8d',
'CjkUnifiedIdeograph578e': '\xe5\x9e\x8e',
'YiSyllableE': '\xea\x80\x94',
'CjkUnifiedIdeograph758b': '\xe7\x96\x8b',
'YiSyllableA': '\xea\x80\x8a',
'CjkUnifiedIdeograph758c': '\xe7\x96\x8c',
'HangulSyllableNyilg': '\xeb\x8a\xb5',
'HangulSyllableNyilb': '\xeb\x8a\xb7',
'CjkUnifiedIdeograph6987': '\xe6\xa6\x87',
'KhmerSymbolLekAttakPrampii': '\xe1\x9f\xb7',
'CjkUnifiedIdeograph93ba': '\xe9\x8e\xba',
'CjkUnifiedIdeograph758a': '\xe7\x96\x8a',
'HangulSyllableNyilh': '\xeb\x8a\xbb',
'CopticSmallLetterFei': '\xcf\xa5',
'CjkUnifiedIdeograph93be': '\xe9\x8e\xbe',
'ArabicLetterMeemWithDotAbove': '\xdd\xa5',
'MongolianLetterTodoJa': '\xe1\xa1\x93',
'CombiningCyrillicThousandMillionsSign': '\xea\x99\xb2',
'HangulSyllableGo': '\xea\xb3\xa0',
'AddressedToTheSubject': '\xe2\x84\x81',
'HangulSyllableSubs': '\xec\x88\xaa',
'UnicodeCharacter0xf0db': '\xef\x83\x9b',
'EthiopicSyllableDoa': '\xe2\xb6\x8c',
'UnicodeCharacter0xf4ee': '\xef\x93\xae',
'VerticalKanaRepeatMarkLowerHalf': '\xe3\x80\xb5',
'HangulSyllableKweonh': '\xed\x80\x82',
'UnicodeCharacter0xed2e': '\xee\xb4\xae',
'UnicodeCharacter0xed2d': '\xee\xb4\xad',
'LeftHalfBlackCircle': '\xe2\x97\x96',
'UnicodeCharacter0xed2f': '\xee\xb4\xaf',
'HangulSyllableGyeok': '\xea\xb3\x80',
'UnicodeCharacter0xed2c': '\xee\xb4\xac',
'UnicodeCharacter0xed2b': '\xee\xb4\xab',
'UnicodeCharacter0xeb5f': '\xee\xad\x9f',
'SaurashtraDigitOne': '\xea\xa3\x91',
'UnicodeCharacter0x31ef': '\xe3\x87\xaf',
'CjkUnifiedIdeograph8427': '\xe8\x90\xa7',
'UnicodeCharacter0xf0dc': '\xef\x83\x9c',
'CjkUnifiedIdeograph4fe1': '\xe4\xbf\xa1',
'UnicodeCharacter0x31ed': '\xe3\x87\xad',
'U': '\xce\xa5',
'UnicodeCharacter0x31ec': '\xe3\x87\xac',
'BalineseMusicalSymbolRighthandOpenDug': '\xe1\xad\xb4',
'HangulJungseongEuu': '\xe1\x86\x95',
'UnicodeCharacter0x31eb': '\xe3\x87\xab',
'UnicodeCharacter0xed29': '\xee\xb4\xa9',
'UnicodeCharacter0x31ea': '\xe3\x87\xaa',
'CjkUnifiedIdeograph3b13': '\xe3\xac\x93',
'UnicodeCharacter0xed25': '\xee\xb4\xa5',
'UnicodeCharacter0xed24': '\xee\xb4\xa4',
'UnicodeCharacter0xed27': '\xee\xb4\xa7',
'UnicodeCharacter0xed26': '\xee\xb4\xa6',
'UnicodeCharacter0xed21': '\xee\xb4\xa1',
'UnicodeCharacter0xed20': '\xee\xb4\xa0',
'UnicodeCharacter0xed23': '\xee\xb4\xa3',
'UnicodeCharacter0xed22': '\xee\xb4\xa2',
'^+': '\xe2\x81\xba',
'UnicodeCharacter0xeb5d': '\xee\xad\x9d',
'HangulSyllableGgwagg': '\xea\xbd\x8a',
'EthiopicSyllableGgu': '\xe1\x8c\x99',
'HangulSyllableHyegg': '\xed\x98\x9e',
'CjkUnifiedIdeograph4815': '\xe4\xa0\x95',
'EthiopicSyllableGgi': '\xe1\x8c\x9a',
'CjkUnifiedIdeograph4817': '\xe4\xa0\x97',
'CjkUnifiedIdeograph4816': '\xe4\xa0\x96',
'CjkUnifiedIdeograph4811': '\xe4\xa0\x91',
'CjkUnifiedIdeograph4810': '\xe4\xa0\x90',
'EthiopicSyllableZee': '\xe1\x8b\x9c',
'CjkUnifiedIdeograph4812': '\xe4\xa0\x92',
'CjkUnifiedIdeograph754b': '\xe7\x95\x8b',
'EthiopicSyllableGga': '\xe1\x8c\x98',
'CjkUnifiedIdeograph754a': '\xe7\x95\x8a',
'CjkUnifiedIdeograph4819': '\xe4\xa0\x99',
'EthiopicSyllableGge': '\xe1\x8c\x9d',
'CjkUnifiedIdeograph754d': '\xe7\x95\x8d',
'CjkUnifiedIdeograph754e': '\xe7\x95\x8e',
'CanadianSyllabicsNga': '\xe1\x96\x93',
'CanadianSyllabicsNgo': '\xe1\x96\x91',
'CanadianSyllabicsNgi': '\xe1\x96\x8f',
'UnicodeCharacter0xdb42': '\xed\xad\x82',
'UnicodeCharacter0xdb43': '\xed\xad\x83',
'UnicodeCharacter0xdb40': '\xed\xad\x80',
'UnicodeCharacter0xdb41': '\xed\xad\x81',
'UnicodeCharacter0xdb46': '\xed\xad\x86',
'UnicodeCharacter0xdb47': '\xed\xad\x87',
'UnicodeCharacter0xdb44': '\xed\xad\x84',
'UnicodeCharacter0xdb45': '\xed\xad\x85',
'UnicodeCharacter0xdb48': '\xed\xad\x88',
'UnicodeCharacter0xdb49': '\xed\xad\x89',
'HangulSyllableSyih': '\xec\x8b\x9b',
'CjkUnifiedIdeograph481e': '\xe4\xa0\x9e',
'CjkUnifiedIdeograph481d': '\xe4\xa0\x9d',
'CjkUnifiedIdeograph7548': '\xe7\x95\x88',
'CjkUnifiedIdeograph481f': '\xe4\xa0\x9f',
'CjkUnifiedIdeograph481a': '\xe4\xa0\x9a',
'CjkUnifiedIdeograph83d4': '\xe8\x8f\x94',
'CjkUnifiedIdeograph481c': '\xe4\xa0\x9c',
'CjkUnifiedIdeograph481b': '\xe4\xa0\x9b',
'CjkUnifiedIdeograph7542': '\xe7\x95\x82',
'CjkUnifiedIdeograph7543': '\xe7\x95\x83',
'CjkUnifiedIdeograph7540': '\xe7\x95\x80',
'CjkUnifiedIdeograph7541': '\xe7\x95\x81',
'GujaratiLetterNga': '\xe0\xaa\x99',
'CjkUnifiedIdeograph7547': '\xe7\x95\x87',
'CjkUnifiedIdeograph7544': '\xe7\x95\x84',
'CjkUnifiedIdeograph7545': '\xe7\x95\x85',
'HangulSyllableJult': '\xec\xa4\x89',
'HangulSyllableJuls': '\xec\xa4\x88',
'HangulSyllableJulp': '\xec\xa4\x8a',
'ArabicLigatureHahWithMeemInitialForm': '\xef\xb2\xaa',
'HangulSyllableGi': '\xea\xb8\xb0',
'HangulSyllableSyim': '\xec\x8b\x90',
'UnicodeCharacter0xdb4b': '\xed\xad\x8b',
'MalayalamVowelSignUu': '\xe0\xb5\x82',
'SymbolForFormFeed': '\xe2\x90\x8c',
'HangulSyllableGginj': '\xeb\x82\x81',
'UnicodeCharacter0xdb4f': '\xed\xad\x8f',
'UnicodeCharacter0xdb4d': '\xed\xad\x8d',
'UnicodeCharacter0xdb4e': '\xed\xad\x8e',
'HangulSyllableDdweos': '\xeb\x9b\x8b',
'HangulSyllableDdeunh': '\xeb\x9c\xae',
'CanadianSyllabicsFinalDoubleAcute': '\xe1\x90\xa5',
'GreekKaiSymbol': '\xcf\x97',
'CjkUnifiedIdeograph7584': '\xe7\x96\x84',
'HangulSyllableHubs': '\xed\x9b\x96',
'UnicodeCharacter0xde61': '\xed\xb9\xa1',
'CjkUnifiedIdeograph7585': '\xe7\x96\x85',
'HangulSyllableU': '\xec\x9a\xb0',
'CjkUnifiedIdeograph7582': '\xe7\x96\x82',
'OghamLetterSail': '\xe1\x9a\x84',
'CjkUnifiedIdeograph7583': '\xe7\x96\x83',
'ArabicLetterBeehMedialForm': '\xef\xad\x95',
'UnicodeCharacter0xf21f': '\xef\x88\x9f',
'HangulSyllableDwaeng': '\xeb\x90\x91',
'UnicodeCharacter0xeb5a': '\xee\xad\x9a',
'UnicodeCharacter0x2760': '\xe2\x9d\xa0',
'CjkUnifiedIdeograph7581': '\xe7\x96\x81',
'HangulSyllableDwaenh': '\xeb\x90\x82',
'HangulSyllableO': '\xec\x98\xa4',
'HangulSyllableDwaenj': '\xeb\x90\x81',
'CanadianSyllabicsMoo': '\xe1\x92\xa8',
'LimbuLetterGa': '\xe1\xa4\x83',
'YiSyllableJyrx': '\xea\x8f\xba',
'HangulSyllableBbyobs': '\xeb\xbf\x82',
'UnicodeCharacter0xea61': '\xee\xa9\xa1',
'CjkUnifiedIdeograph9a8d': '\xe9\xaa\x8d',
'HangulSyllableGing': '\xea\xb9\x85',
'LatinSmallLetterBWithStroke': '\xc6\x80',
'LatinSmallLetterAWithOgonek': '\xc4\x85',
'HangulJungseongUa': '\xe1\x86\x89',
'CyrillicCapitalLetterKhakassianChe': '\xd3\x8b',
'BalineseMusicalSymbolCombiningKempul': '\xe1\xad\xad',
'ArabicLetterHehWithYehAboveIsolatedForm': '\xef\xae\xa4',
'UnicodeCharacter0xdd96': '\xed\xb6\x96',
'UnicodeCharacter0x115a': '\xe1\x85\x9a',
'UnicodeCharacter0x115c': '\xe1\x85\x9c',
'UnicodeCharacter0x115b': '\xe1\x85\x9b',
'UnicodeCharacter0x115e': '\xe1\x85\x9e',
'UnicodeCharacter0x115d': '\xe1\x85\x9d',
'YiSyllableMgiex': '\xea\x88\xb0',
'VaiSyllableDa': '\xea\x95\xa0',
'UnicodeCharacter0xdd94': '\xed\xb6\x94',
'SaurashtraLetterOo': '\xea\xa2\x90',
'UnicodeCharacter0xdd93': '\xed\xb6\x93',
'UnicodeCharacter0xdd92': '\xed\xb6\x92',
'VerticalBarDoubleLeftTurnstile': '\xe2\xab\xa4',
'UnicodeCharacter0xdd91': '\xed\xb6\x91',
'VaiSyllableDe': '\xea\x97\xb9',
'LatinCapitalLetterIWithOgonek': '\xc4\xae',
'HangulSyllableDdweom': '\xeb\x9b\x88',
'HexagramForTheMarryingMaiden': '\xe4\xb7\xb5',
'UnicodeCharacter0xf1a3': '\xef\x86\xa3',
'LeftwardsHarpoonWithBarbDownAboveRightwardsHarpoonWithBarbDown': '\xe2\xa5\xa7',
'LeftWhiteCurlyBracket': '\xe2\xa6\x83',
'HangulSyllableHyeols': '\xed\x98\x8c',
'CjkUnifiedIdeograph7729': '\xe7\x9c\xa9',
'UnicodeCharacter0xda4a': '\xed\xa9\x8a',
'VaiSyllableDo': '\xea\x97\x91',
'CjkUnifiedIdeograph9baa': '\xe9\xae\xaa',
'HangulSyllableDdweok': '\xeb\x9b\x90',
'CjkUnifiedIdeograph8509': '\xe8\x94\x89',
'CjkUnifiedIdeograph7724': '\xe7\x9c\xa4',
'SemidirectProductWithBottomClosed': '\xe2\xa8\xb2',
'CjkUnifiedIdeograph9bac': '\xe9\xae\xac',
'HangulSyllableHeunj': '\xed\x9d\x95',
'HangulSyllableHyeolp': '\xed\x98\x8e',
'HangulSyllableHeunh': '\xed\x9d\x96',
'CjkUnifiedIdeograph9bad': '\xe9\xae\xad',
'CjkUnifiedIdeograph9bae': '\xe9\xae\xae',
'PhagspaLetterVoicedHa': '\xea\xa1\xaf',
'HangulSyllableJjaej': '\xec\xa8\x8e',
'CjkUnifiedIdeograph9baf': '\xe9\xae\xaf',
'YiSyllableNgie': '\xea\x89\x9e',
'LatinCapitalLetterAWithDotAbove': '\xc8\xa6',
'CjkUnifiedIdeograph8508': '\xe8\x94\x88',
'HangulSyllableOegs': '\xec\x99\xbb',
'TeluguVowelSignEe': '\xe0\xb1\x87',
'HangulSyllableCeung': '\xec\xb8\xb5',
'UnicodeCharacter0xfd4a': '\xef\xb5\x8a',
'HangulSyllableHeubs': '\xed\x9d\xa2',
'CyrillicCapitalLetterDe': '\xd0\x94',
'SyriacHbasaesasaDotted': '\xdc\xbc',
'EthiopicSyllableBwee': '\xe1\x8e\x86',
'HangulSyllableCeunj': '\xec\xb8\xa5',
'YiSyllableJux': '\xea\x8f\xb1',
'UnicodeCharacter0x31e6': '\xe3\x87\xa6',
'YiSyllableJut': '\xea\x8f\xb0',
'HangulSyllableJjos': '\xec\xab\x8f',
'YiSyllableJur': '\xea\x8f\xb5',
'UnicodeCharacter0x31e5': '\xe3\x87\xa5',
'YiSyllableJup': '\xea\x8f\xb3',
'YiSyllableJuo': '\xea\x8f\xaa',
'CircledDigitSeven': '\xe2\x91\xa6',
'Segment': '\xe2\x8c\x93',
'HangulSyllableRyeong': '\xeb\xa0\xb9',
'UnicodeCharacter0xf73d': '\xef\x9c\xbd',
'YiRadicalZa': '\xea\x92\xb2',
'BoxDrawingsDoubleDownAndRight': '\xe2\x95\x94',
'HangulSyllableRweonh': '\xeb\xa4\x8a',
'KhmerSymbolMuoyRoc': '\xe1\xa7\xb1',
'HangulSyllableDdweog': '\xeb\x9a\xb9',
'HangulSyllableRyeonh': '\xeb\xa0\xaa',
'HangulSyllableRweong': '\xeb\xa4\x99',
'CjkUnifiedIdeograph4396': '\xe4\x8e\x96',
'CanadianSyllabicsCarrierKk': '\xe1\x98\x81',
'CjkUnifiedIdeograph75c8': '\xe7\x97\x88',
'GreekCapitalLetterIotaWithDasiaAndVaria': '\xe1\xbc\xbb',
'UnicodeCharacter0xa970': '\xea\xa5\xb0',
'MediumLeftCurlyBracketOrnament': '\xe2\x9d\xb4',
'HangulSyllableTweolp': '\xed\x89\x96',
'LatinSmallLetterIWithOgonek': '\xc4\xaf',
'CjkUnifiedIdeograph75c6': '\xe7\x97\x86',
'HangulSyllableKaenj': '\xec\xba\x95',
'UnicodeCharacter0xd96e': '\xed\xa5\xae',
'UnicodeCharacter0xb5b': '\xe0\xad\x9b',
'UnicodeCharacter0xb5a': '\xe0\xad\x9a',
'YiSyllableNjuox': '\xea\x90\xb9',
'UnicodeCharacter0xb5e': '\xe0\xad\x9e',
'UnicodeCharacter0xef30': '\xee\xbc\xb0',
'HangulLetterHieuh': '\xe3\x85\x8e',
'CjkUnifiedIdeograph45e0': '\xe4\x97\xa0',
'ChamConsonantSignLa': '\xea\xa8\xb5',
'KangxiRadicalMelon': '\xe2\xbd\xa0',
'CopticSmallLetterSima': '\xe2\xb2\xa5',
'HangulSyllableHyaeb': '\xed\x96\xbd',
'ArabicLigatureAinWithYehIsolatedForm': '\xef\xb3\xb8',
'HangulSyllableGgyos': '\xea\xbe\xaf',
'YiSyllableJjuo': '\xea\x90\xa1',
'CjkUnifiedIdeograph6431': '\xe6\x90\xb1',
'RightParenthesisLowerHook': '\xe2\x8e\xa0',
'CjkUnifiedIdeograph6433': '\xe6\x90\xb3',
'CjkUnifiedIdeograph6432': '\xe6\x90\xb2',
'CjkUnifiedIdeograph6435': '\xe6\x90\xb5',
'CjkUnifiedIdeograph4398': '\xe4\x8e\x98',
'CjkUnifiedIdeograph6437': '\xe6\x90\xb7',
'CjkUnifiedIdeograph6436': '\xe6\x90\xb6',
'CjkUnifiedIdeograph6439': '\xe6\x90\xb9',
'CjkUnifiedIdeograph6438': '\xe6\x90\xb8',
'CjkUnifiedIdeograph45e7': '\xe4\x97\xa7',
'HangulSyllableGyegs': '\xea\xb3\x87',
'UnicodeCharacter0xda49': '\xed\xa9\x89',
'ArabicLetterDadFinalForm': '\xef\xba\xbe',
'HangulSyllableCwenj': '\xec\xb7\x91',
'EthiopicSyllableHhaa': '\xe1\x88\x93',
'HangulSyllableCwiss': '\xec\xb7\xbc',
'HangulSyllablePebs': '\xed\x8e\xaa',
'HangulSyllableHyaej': '\xed\x97\x82',
'UnicodeCharacter0xb52': '\xe0\xad\x92',
'UnicodeCharacter0xb53': '\xe0\xad\x93',
'UnicodeCharacter0xb50': '\xe0\xad\x90',
'UnicodeCharacter0xb51': '\xe0\xad\x91',
'RightFloor': '\xe2\x8c\x8b',
'UnicodeCharacter0xb54': '\xe0\xad\x94',
'UnicodeCharacter0xb55': '\xe0\xad\x95',
'UnicodeCharacter0xb58': '\xe0\xad\x98',
'UnicodeCharacter0xb59': '\xe0\xad\x99',
'OlChikiMugaahlaaTtuddaag': '\xe1\xb1\xba',
'SquareWithUpperLeftToLowerRightFill': '\xe2\x96\xa7',
'CjkUnifiedIdeograph643c': '\xe6\x90\xbc',
'CjkUnifiedIdeograph643b': '\xe6\x90\xbb',
'CjkUnifiedIdeograph643e': '\xe6\x90\xbe',
'CjkUnifiedIdeograph643d': '\xe6\x90\xbd',
'CjkUnifiedIdeograph643f': '\xe6\x90\xbf',
'CjkUnifiedIdeograph9a58': '\xe9\xa9\x98',
'l': '\xce\xbb',
'EthiopicSyllableCoa': '\xe2\xb6\x87',
'CjkUnifiedIdeograph9ba0': '\xe9\xae\xa0',
'VerticalFourDots': '\xe2\x81\x9e',
'IntegralWithDoubleStroke': '\xe2\xa8\x8e',
'UnicodeCharacter0xda40': '\xed\xa9\x80',
'CjkUnifiedIdeograph9ba2': '\xe9\xae\xa2',
'LatinLetterReversedEshLoop': '\xc6\xaa',
'CjkUnifiedIdeograph9ba3': '\xe9\xae\xa3',
'HangulSyllableYilt': '\xec\x9d\xa5',
'CjkUnifiedIdeograph9ba4': '\xe9\xae\xa4',
'CyrillicSmallLetterShwe': '\xea\x9a\x97',
'CjkUnifiedIdeograph9ba5': '\xe9\xae\xa5',
'ArmenianCapitalLetterLiwn': '\xd4\xbc',
'UnicodeCharacter0xda47': '\xed\xa9\x87',
'CjkUnifiedIdeograph9ba7': '\xe9\xae\xa7',
'UnicodeCharacter0xea6a': '\xee\xa9\xaa',
'HangulSyllableJyul': '\xec\xa5\xb4',
'CircledParallel': '\xe2\xa6\xb7',
'CircledNumberTwentyNine': '\xe3\x89\x99',
'HangulSyllableTugg': '\xed\x88\xae',
'CanadianSyllabicsWestcreePwa': '\xe1\x91\x85',
'CanadianSyllabicsWestcreePwe': '\xe1\x90\xbb',
'CanadianSyllabicsWestcreePwi': '\xe1\x90\xbd',
'TibetanCantillationSignCangTeu': '\xe0\xbf\x82',
'GeorgianLetterKhar': '\xe1\x83\xa5',
'HangulSyllableNeon': '\xeb\x84\x8c',
'UnicodeCharacter0xea6f': '\xee\xa9\xaf',
'GreekSmallLetterAlphaWithDasiaAndVariaAndYpogegrammeni': '\xe1\xbe\x83',
'HangulSyllableDdwid': '\xeb\x9b\xb7',
'UnicodeCharacter0xdd9e': '\xed\xb6\x9e',
'UnicodeCharacter0xdd9d': '\xed\xb6\x9d',
'UnicodeCharacter0xdd9c': '\xed\xb6\x9c',
'CjkUnifiedIdeograph9da8': '\xe9\xb6\xa8',
'UnicodeCharacter0xdd9b': '\xed\xb6\x9b',
'HangulSyllableDdwin': '\xeb\x9b\xb4',
'HangulSyllableDdwil': '\xeb\x9b\xb8',
'HangulSyllableBbyaegs': '\xeb\xba\xb7',
'CjkUnifiedIdeograph54c4': '\xe5\x93\x84',
'SquareMl': '\xe3\x8e\x96',
'SquareMm': '\xe3\x8e\x9c',
'SquareMg': '\xe3\x8e\x8e',
'CjkUnifiedIdeograph417a': '\xe4\x85\xba',
'SquareMb': '\xe3\x8e\x86',
'MongolianLetterAliGaliThreeBaluda': '\xe1\xa2\x86',
'SquareMa': '\xe3\x8e\x83',
'YiSyllableBuop': '\xea\x80\xa4',
'KangxiRadicalSlash': '\xe2\xbc\x83',
'CjkUnifiedIdeograph9da3': '\xe9\xb6\xa3',
'SquareMv': '\xe3\x8e\xb7',
'SquareMw': '\xe3\x8e\xbd',
'YiSyllableBuox': '\xea\x80\xa2',
'CjkUnifiedIdeograph8b3f': '\xe8\xac\xbf',
'SquareMs': '\xe3\x8e\xb3',
'CyrillicSmallLetterFita': '\xd1\xb3',
'HangulSyllableCulh': '\xec\xb6\xa3',
'CombiningVerticalLineAbove': '\xcc\x8d',
'YiSyllableRu': '\xea\x8f\x96',
'NewSheqelSign': '\xe2\x82\xaa',
'YiSyllableRy': '\xea\x8f\x9c',
'NaryLogicalAnd': '\xe2\x8b\x80',
'YiSyllableRa': '\xea\x8f\x88',
'KhmerLetterMo': '\xe1\x9e\x98',
'YiSyllableRe': '\xea\x8f\x92',
'UnicodeCharacter0xf07c': '\xef\x81\xbc',
'YiSyllableRo': '\xea\x8f\x8f',
'EthiopicSyllableCchee': '\xe2\xb6\xbc',
'HangulSyllableJyiss': '\xec\xa6\xb8',
'TibetanLetterTta': '\xe0\xbd\x8a',
'UnicodeCharacter0xe9c2': '\xee\xa7\x82',
'YiSyllableCuox': '\xea\x8a\xba',
'RightwardsHarpoonWithBarbUpAboveLeftwardsHarpoonWithBarbUp': '\xe2\xa5\xa8',
'YiSyllableCuop': '\xea\x8a\xbc',
'ArabicLetterSadWithTwoDotsBelow': '\xda\x9d',
'UnicodeCharacter0xe1ed': '\xee\x87\xad',
'MongolianLetterManchuZha': '\xe1\xa1\xb7',
'RunicLetterOthalanEthelO': '\xe1\x9b\x9f',
'HangulSyllableHeolb': '\xed\x97\x93',
'ArabicKasraMedialForm': '\xef\xb9\xbb',
'LatinSmallLetterTresillo': '\xea\x9c\xab',
'HangulSyllableByit': '\xeb\xb9\x81',
'HalfwidthHangulLetterPhieuph': '\xef\xbe\xbd',
'FullwidthLatinCapitalLetterL': '\xef\xbc\xac',
'HangulSyllableByip': '\xeb\xb9\x82',
'UnicodeCharacter0xf5be': '\xef\x96\xbe',
'CanadianSyllabicsBlackfootKa': '\xe1\x96\xbf',
'CjkUnifiedIdeograph91ef': '\xe9\x87\xaf',
'CjkUnifiedIdeograph91ee': '\xe9\x87\xae',
'CjkUnifiedIdeograph91ed': '\xe9\x87\xad',
'CjkUnifiedIdeograph91ec': '\xe9\x87\xac',
'CjkUnifiedIdeograph91eb': '\xe9\x87\xab',
'CjkUnifiedIdeograph91ea': '\xe9\x87\xaa',
'HangulSyllableByim': '\xeb\xb8\xb8',
'HangulSyllableMolb': '\xeb\xaa\xb3',
'GlagoliticSmallLetterYestu': '\xe2\xb0\xb5',
'HangulSyllableByil': '\xeb\xb8\xb0',
'HangulSyllableMolh': '\xeb\xaa\xb7',
'HangulSyllableMolm': '\xeb\xaa\xb2',
'ArabicLigatureSheenWithKhahFinalForm': '\xef\xb4\xa7',
'UnicodeCharacter0xf5bd': '\xef\x96\xbd',
'HangulSyllableMolp': '\xeb\xaa\xb6',
'HangulSyllableMols': '\xeb\xaa\xb4',
'HangulSyllableByin': '\xeb\xb8\xac',
'CjkUnifiedIdeograph7b38': '\xe7\xac\xb8',
'CjkUnifiedIdeograph7b39': '\xe7\xac\xb9',
'CjkUnifiedIdeograph7b34': '\xe7\xac\xb4',
'UnicodeCharacter0x2fd8': '\xe2\xbf\x98',
'CjkUnifiedIdeograph7b36': '\xe7\xac\xb6',
'CjkUnifiedIdeograph7b37': '\xe7\xac\xb7',
'CjkUnifiedIdeograph7b30': '\xe7\xac\xb0',
'CjkUnifiedIdeograph7b31': '\xe7\xac\xb1',
'CjkUnifiedIdeograph7b32': '\xe7\xac\xb2',
'UnicodeCharacter0x2fd9': '\xe2\xbf\x99',
'EthiopicSyllableTzoa': '\xe1\x8d\x87',
'UnicodeCharacter0xf1c3': '\xef\x87\x83',
'_rho': '\xe1\xb5\xa8',
'HangulSyllableByik': '\xeb\xb9\x80',
'CjkUnifiedIdeograph91e9': '\xe9\x87\xa9',
'CjkUnifiedIdeograph91e8': '\xe9\x87\xa8',
'CjkUnifiedIdeograph91e7': '\xe9\x87\xa7',
'CjkUnifiedIdeograph91e6': '\xe9\x87\xa6',
'CjkUnifiedIdeograph91e5': '\xe9\x87\xa5',
'CjkUnifiedIdeograph91e4': '\xe9\x87\xa4',
'CjkUnifiedIdeograph91e3': '\xe9\x87\xa3',
'CjkUnifiedIdeograph91e2': '\xe9\x87\xa2',
'CjkUnifiedIdeograph91e1': '\xe9\x87\xa1',
'CjkUnifiedIdeograph91e0': '\xe9\x87\xa0',
'HangulSyllableKuss': '\xec\xbf\xb4',
'CjkCompatibilityIdeographfa8d': '\xef\xaa\x8d',
'UnicodeCharacter0xf07e': '\xef\x81\xbe',
'HangulSyllableByig': '\xeb\xb8\xa9',
'CjkUnifiedIdeograph7b3d': '\xe7\xac\xbd',
'CjkUnifiedIdeograph7b3e': '\xe7\xac\xbe',
'KannadaLetterTa': '\xe0\xb2\xa4',
'HangulSyllableDdeugs': '\xeb\x9c\xab',
'CjkUnifiedIdeograph7b3a': '\xe7\xac\xba',
'CjkUnifiedIdeograph7b3b': '\xe7\xac\xbb',
'CjkUnifiedIdeograph7b3c': '\xe7\xac\xbc',
'HangulSyllablePyolb': '\xed\x91\xa7',
'CircledHorizontalBarWithNotch': '\xe2\x8e\x89',
'UnicodeCharacter0xf45b': '\xef\x91\x9b',
'CombiningTildeOverlay': '\xcc\xb4',
'HangulSyllableKwaels': '\xec\xbe\x98',
'CjkUnifiedIdeograph71f4': '\xe7\x87\xb4',
'CjkUnifiedIdeograph71f5': '\xe7\x87\xb5',
'CjkUnifiedIdeograph71f6': '\xe7\x87\xb6',
'CjkUnifiedIdeograph71f7': '\xe7\x87\xb7',
'CjkUnifiedIdeograph71f0': '\xe7\x87\xb0',
'CjkUnifiedIdeograph71f1': '\xe7\x87\xb1',
'CjkUnifiedIdeograph71f2': '\xe7\x87\xb2',
'CjkUnifiedIdeograph71f3': '\xe7\x87\xb3',
'CjkUnifiedIdeograph71f8': '\xe7\x87\xb8',
'CjkUnifiedIdeograph71f9': '\xe7\x87\xb9',
'HangulSyllablePyolg': '\xed\x91\xa5',
'CjkUnifiedIdeograph4c52': '\xe4\xb1\x92',
'CjkUnifiedIdeograph4c53': '\xe4\xb1\x93',
'CjkUnifiedIdeograph4c50': '\xe4\xb1\x90',
'CjkUnifiedIdeograph4c51': '\xe4\xb1\x91',
'CjkUnifiedIdeograph4c56': '\xe4\xb1\x96',
'CjkUnifiedIdeograph4c57': '\xe4\xb1\x97',
'CjkUnifiedIdeograph4c54': '\xe4\xb1\x94',
'CjkUnifiedIdeograph4c55': '\xe4\xb1\x95',
'TibetanMarkCaretDzudRtagsBzhiMigCan': '\xe0\xbc\xb6',
'CjkUnifiedIdeograph4c59': '\xe4\xb1\x99',
'CjkUnifiedIdeograph98a8': '\xe9\xa2\xa8',
'MyanmarLetterLa': '\xe1\x80\x9c',
'CjkUnifiedIdeograph87f9': '\xe8\x9f\xb9',
'CjkUnifiedIdeograph87f8': '\xe8\x9f\xb8',
'GreaterthanWithCircleInside': '\xe2\xa9\xba',
'UnicodeCharacter0xf07b': '\xef\x81\xbb',
'CjkUnifiedIdeograph98a7': '\xe9\xa2\xa7',
'CjkUnifiedIdeograph87f3': '\xe8\x9f\xb3',
'CjkUnifiedIdeograph87f2': '\xe8\x9f\xb2',
'CjkUnifiedIdeograph87f1': '\xe8\x9f\xb1',
'CjkUnifiedIdeograph87f0': '\xe8\x9f\xb0',
'OriginalOf': '\xe2\x8a\xb6',
'CjkUnifiedIdeograph87f6': '\xe8\x9f\xb6',
'CjkUnifiedIdeograph87f5': '\xe8\x9f\xb5',
'CjkUnifiedIdeograph693a': '\xe6\xa4\xba',
'DevanagariLetterKa': '\xe0\xa4\x95',
'CjkUnifiedIdeograph4c5c': '\xe4\xb1\x9c',
'CjkUnifiedIdeograph4c5a': '\xe4\xb1\x9a',
'CjkUnifiedIdeograph4c5f': '\xe4\xb1\x9f',
'UnicodeCharacter0xf5bc': '\xef\x96\xbc',
'CjkUnifiedIdeograph4c5d': '\xe4\xb1\x9d',
'CjkUnifiedIdeograph4c5e': '\xe4\xb1\x9e',
'ArabicDecimalSeparator': '\xd9\xab',
'CjkUnifiedIdeograph71fd': '\xe7\x87\xbd',
'EthiopicSyllableSebatbeitMwa': '\xe1\x8e\x80',
'CjkUnifiedIdeograph71ff': '\xe7\x87\xbf',
'UnicodeCharacter0xf07a': '\xef\x81\xba',
'CjkUnifiedIdeograph71fa': '\xe7\x87\xba',
'CjkUnifiedIdeograph71fb': '\xe7\x87\xbb',
'CjkUnifiedIdeograph71fc': '\xe7\x87\xbc',
'LatinSmallLetterLWithPalatalHook': '\xe1\xb6\x85',
'LepchaLetterCa': '\xe1\xb0\x86',
'DottedObelos': '\xe2\xb8\x93',
'CjkUnifiedIdeograph87fc': '\xe8\x9f\xbc',
'CjkUnifiedIdeograph87fb': '\xe8\x9f\xbb',
'CjkUnifiedIdeograph87fa': '\xe8\x9f\xba',
'CjkUnifiedIdeograph87ff': '\xe8\x9f\xbf',
'CjkUnifiedIdeograph87fe': '\xe8\x9f\xbe',
'CjkUnifiedIdeograph87fd': '\xe8\x9f\xbd',
'HeavyAsterisk': '\xe2\x9c\xb1',
'UnicodeCharacter0xe258': '\xee\x89\x98',
'HangulSyllableWanj': '\xec\x99\x85',
'UnicodeCharacter0xe256': '\xee\x89\x96',
'UnicodeCharacter0xe257': '\xee\x89\x97',
'UnicodeCharacter0xe254': '\xee\x89\x94',
'UnicodeCharacter0xe255': '\xee\x89\x95',
'UnicodeCharacter0xe252': '\xee\x89\x92',
'UnicodeCharacter0xe253': '\xee\x89\x93',
'UnicodeCharacter0xe250': '\xee\x89\x90',
'UnicodeCharacter0xe251': '\xee\x89\x91',
'EthiopicTonalMarkRikrik': '\xe1\x8e\x92',
'ArabicLetterDalWithInvertedSmallVBelow': '\xdd\x9a',
'HangulSyllableWanh': '\xec\x99\x86',
'SinhalaLetterSanyakaDdayanna': '\xe0\xb6\xac',
'ArrowPointingRightwardsThenCurvingUpwards': '\xe2\xa4\xb4',
'GreekCapitalLetterOmicronWithDasia': '\xe1\xbd\x89',
'VaiSyllableZhu': '\xea\x96\xaf',
'LatinSmallLetterYWithStroke': '\xc9\x8f',
'VaiSyllableZho': '\xea\x97\x96',
'VaiSyllableZhi': '\xea\x94\xbe',
'UnicodeCharacter0x1bfc': '\xe1\xaf\xbc',
'UnicodeCharacter0x1bfb': '\xe1\xaf\xbb',
'UnicodeCharacter0x1bfa': '\xe1\xaf\xba',
'UnicodeCharacter0x1bff': '\xe1\xaf\xbf',
'UnicodeCharacter0x1bfe': '\xe1\xaf\xbe',
'UnicodeCharacter0x1bfd': '\xe1\xaf\xbd',
'CombiningDotAboveRight': '\xcd\x98',
'BengaliIsshar': '\xe0\xa7\xba',
'HangulSyllableWang': '\xec\x99\x95',
'HangulSyllableGyeogg': '\xea\xb2\xaa',
'UnicodeCharacter0xf509': '\xef\x94\x89',
'UnicodeCharacter0xf508': '\xef\x94\x88',
'UnicodeCharacter0xe25f': '\xee\x89\x9f',
'UnicodeCharacter0xf506': '\xef\x94\x86',
'UnicodeCharacter0xe25d': '\xee\x89\x9d',
'UnicodeCharacter0xe25e': '\xee\x89\x9e',
'UnicodeCharacter0xe25b': '\xee\x89\x9b',
'UnicodeCharacter0xe25c': '\xee\x89\x9c',
'UnicodeCharacter0xf501': '\xef\x94\x81',
'UnicodeCharacter0xe25a': '\xee\x89\x9a',
'UnicodeCharacter0xe869': '\xee\xa1\xa9',
'UnicodeCharacter0x1bf9': '\xe1\xaf\xb9',
'UnicodeCharacter0x1bf8': '\xe1\xaf\xb8',
'UnicodeCharacter0x1bf3': '\xe1\xaf\xb3',
'UnicodeCharacter0x1bf2': '\xe1\xaf\xb2',
'UnicodeCharacter0x1bf1': '\xe1\xaf\xb1',
'UnicodeCharacter0x1bf0': '\xe1\xaf\xb0',
'UnicodeCharacter0x1bf7': '\xe1\xaf\xb7',
'UnicodeCharacter0x1bf6': '\xe1\xaf\xb6',
'UnicodeCharacter0x1bf5': '\xe1\xaf\xb5',
'UnicodeCharacter0x1bf4': '\xe1\xaf\xb4',
'HangulSyllableSsyabs': '\xec\x8d\x82',
'CanadianSyllabicsCarrierWee': '\xe1\x97\x93',
'UnicodeCharacter0xef35': '\xee\xbc\xb5',
'SmallSemicolon': '\xef\xb9\x94',
'UnicodeCharacter0xa11': '\xe0\xa8\x91',
'CjkUnifiedIdeograph8668': '\xe8\x99\xa8',
'UnicodeCharacter0xa12': '\xe0\xa8\x92',
'CjkUnifiedIdeograph8666': '\xe8\x99\xa6',
'CjkUnifiedIdeograph8667': '\xe8\x99\xa7',
'OriyaSignCandrabindu': '\xe0\xac\x81',
'CjkUnifiedIdeograph8665': '\xe8\x99\xa5',
'CjkUnifiedIdeograph8662': '\xe8\x99\xa2',
'CjkUnifiedIdeograph8663': '\xe8\x99\xa3',
'CjkUnifiedIdeograph8660': '\xe8\x99\xa0',
'CjkUnifiedIdeograph8661': '\xe8\x99\xa1',
'CjkUnifiedIdeograph9f9d': '\xe9\xbe\x9d',
'CjkUnifiedIdeograph9f9e': '\xe9\xbe\x9e',
'CjkUnifiedIdeograph9f9f': '\xe9\xbe\x9f',
'CjkUnifiedIdeograph9f9a': '\xe9\xbe\x9a',
'CjkUnifiedIdeograph9f9b': '\xe9\xbe\x9b',
'CjkUnifiedIdeograph9f9c': '\xe9\xbe\x9c',
'UnicodeCharacter0xe5d2': '\xee\x97\x92',
'SmallRomanNumeralSeven': '\xe2\x85\xb6',
'UnicodeCharacter0xe5d0': '\xee\x97\x90',
'UnicodeCharacter0xebb9': '\xee\xae\xb9',
'UnicodeCharacter0xe5d1': '\xee\x97\x91',
'CjkUnifiedIdeograph67ad': '\xe6\x9e\xad',
'CjkUnifiedIdeograph67ae': '\xe6\x9e\xae',
'CjkUnifiedIdeograph67af': '\xe6\x9e\xaf',
'CopticCapitalLetterOldNubianNyi': '\xe2\xb3\xa0',
'UnicodeCharacter0xebb7': '\xee\xae\xb7',
'CjkUnifiedIdeograph67aa': '\xe6\x9e\xaa',
'CjkUnifiedIdeograph67ab': '\xe6\x9e\xab',
'CjkUnifiedIdeograph67ac': '\xe6\x9e\xac',
'CjkUnifiedIdeograph9f94': '\xe9\xbe\x94',
'CjkUnifiedIdeograph9f95': '\xe9\xbe\x95',
'CjkUnifiedIdeograph9f96': '\xe9\xbe\x96',
'CjkUnifiedIdeograph9f97': '\xe9\xbe\x97',
'CjkUnifiedIdeograph9f90': '\xe9\xbe\x90',
'CjkUnifiedIdeograph76c6': '\xe7\x9b\x86',
'CjkUnifiedIdeograph9f92': '\xe9\xbe\x92',
'CjkUnifiedIdeograph9f93': '\xe9\xbe\x93',
'KhmerSymbolDappramKoet': '\xe1\xa7\xaf',
'UnicodeCharacter0xe5d5': '\xee\x97\x95',
'CjkUnifiedIdeograph9f98': '\xe9\xbe\x98',
'CjkUnifiedIdeograph9f99': '\xe9\xbe\x99',
'CopticFullStop': '\xe2\xb3\xbe',
'CjkUnifiedIdeograph866f': '\xe8\x99\xaf',
'CjkUnifiedIdeograph866d': '\xe8\x99\xad',
'CjkUnifiedIdeograph866e': '\xe8\x99\xae',
'CjkUnifiedIdeograph866b': '\xe8\x99\xab',
'CjkUnifiedIdeograph866c': '\xe8\x99\xac',
'PrecedesAboveSinglelineEqualsSign': '\xe2\xaa\xaf',
'HangulSyllableBang': '\xeb\xb0\xa9',
'LatinCapitalLetterIWithDotBelow': '\xe1\xbb\x8a',
'CjkUnifiedIdeograph67a9': '\xe6\x9e\xa9',
'CjkUnifiedIdeograph67a4': '\xe6\x9e\xa4',
'CjkUnifiedIdeograph67a5': '\xe6\x9e\xa5',
'CjkUnifiedIdeograph67a6': '\xe6\x9e\xa6',
'ReversedDoublePrimeQuotationMark': '\xe3\x80\x9d',
'CjkUnifiedIdeograph67a0': '\xe6\x9e\xa0',
'CjkUnifiedIdeograph67a1': '\xe6\x9e\xa1',
'CjkUnifiedIdeograph67a2': '\xe6\x9e\xa2',
'CjkUnifiedIdeograph67a3': '\xe6\x9e\xa3',
'YiSyllableRrax': '\xea\x8d\xbc',
'CyrillicSmallLetterReversedZe': '\xd4\x91',
'HangulSyllableBbilm': '\xec\x82\x9a',
'ElementOfOpeningDownwards': '\xe2\xab\x99',
'EthiopicSyllableChaa': '\xe1\x8c\xab',
'LeftwardsArrowThroughX': '\xe2\xac\xbe',
'MongolianLetterNa': '\xe1\xa0\xa8',
'CjkUnifiedIdeograph6939': '\xe6\xa4\xb9',
'CjkUnifiedIdeograph9aac': '\xe9\xaa\xac',
'CjkUnifiedIdeograph9aab': '\xe9\xaa\xab',
'CjkUnifiedIdeograph9aaa': '\xe9\xaa\xaa',
'LatinSmallLetterReversedRWithFishhook': '\xc9\xbf',
'CjkUnifiedIdeograph9aaf': '\xe9\xaa\xaf',
'CjkUnifiedIdeograph9aae': '\xe9\xaa\xae',
'CjkUnifiedIdeograph9aad': '\xe9\xaa\xad',
'HangulSyllableIj': '\xec\x9e\x8a',
'CanadianSyllabicsCarrierChee': '\xe1\x99\xa4',
'TrigramForHeaven': '\xe2\x98\xb0',
'MediumShade': '\xe2\x96\x92',
'ArabicLigatureNoonWithJeemWithHahInitialForm': '\xef\xb6\xb8',
'archaicsampi': '\xcd\xb3',
'CjkUnifiedIdeograph66ee': '\xe6\x9b\xae',
'CjkUnifiedIdeograph66ed': '\xe6\x9b\xad',
'CjkUnifiedIdeograph66ef': '\xe6\x9b\xaf',
'CjkUnifiedIdeograph66ea': '\xe6\x9b\xaa',
'CjkUnifiedIdeograph66ec': '\xe6\x9b\xac',
'CjkUnifiedIdeograph66eb': '\xe6\x9b\xab',
'CjkUnifiedIdeograph904e': '\xe9\x81\x8e',
'CjkUnifiedIdeograph904d': '\xe9\x81\x8d',
'CjkUnifiedIdeograph904f': '\xe9\x81\x8f',
'AplFunctionalSymbolIota': '\xe2\x8d\xb3',
'oo': '\xe2\x88\x9e',
'CjkUnifiedIdeograph904c': '\xe9\x81\x8c',
'CjkUnifiedIdeograph904b': '\xe9\x81\x8b',
'CjkUnifiedIdeograph9aa3': '\xe9\xaa\xa3',
'CjkUnifiedIdeograph9aa2': '\xe9\xaa\xa2',
'CjkUnifiedIdeograph9aa1': '\xe9\xaa\xa1',
'CjkUnifiedIdeograph9aa0': '\xe9\xaa\xa0',
'CjkUnifiedIdeograph9aa7': '\xe9\xaa\xa7',
'CjkUnifiedIdeograph9aa6': '\xe9\xaa\xa6',
'CjkUnifiedIdeograph9aa5': '\xe9\xaa\xa5',
'OghamLetterUilleann': '\xe1\x9a\x97',
'CjkUnifiedIdeograph9aa9': '\xe9\xaa\xa9',
'CjkUnifiedIdeograph9aa8': '\xe9\xaa\xa8',
'UnicodeCharacter0xea26': '\xee\xa8\xa6',
'ApproximatelyEqualTo': '\xe2\x89\x85',
'UnicodeCharacter0xa9ba': '\xea\xa6\xba',
'CjkUnifiedIdeograph4f10': '\xe4\xbc\x90',
'UnicodeCharacter0xa9bb': '\xea\xa6\xbb',
'ThaiCharacterAngkhankhu': '\xe0\xb9\x9a',
'CanadianSyllabicsLo': '\xe1\x93\x97',
'CjkUnifiedIdeograph9049': '\xe9\x81\x89',
'CjkUnifiedIdeograph9048': '\xe9\x81\x88',
'CanadianSyllabicsLh': '\xe1\x96\xa6',
'CanadianSyllabicsLi': '\xe1\x93\x95',
'CjkUnifiedIdeograph9045': '\xe9\x81\x85',
'CjkUnifiedIdeograph9044': '\xe9\x81\x84',
'CjkUnifiedIdeograph9047': '\xe9\x81\x87',
'CanadianSyllabicsLe': '\xe1\x93\x93',
'CjkUnifiedIdeograph9041': '\xe9\x81\x81',
'CjkUnifiedIdeograph9040': '\xe9\x81\x80',
'CjkUnifiedIdeograph9043': '\xe9\x81\x83',
'CanadianSyllabicsLa': '\xe1\x93\x9a',
'ElectricArrow': '\xe2\x8c\x81',
'KatakanaLetterTi': '\xe3\x83\x81',
'CjkUnifiedIdeograph66e9': '\xe6\x9b\xa9',
'CjkUnifiedIdeograph66e8': '\xe6\x9b\xa8',
'CjkUnifiedIdeograph66e5': '\xe6\x9b\xa5',
'CjkUnifiedIdeograph66e4': '\xe6\x9b\xa4',
'CjkUnifiedIdeograph66e7': '\xe6\x9b\xa7',
'CjkUnifiedIdeograph66e6': '\xe6\x9b\xa6',
'CjkUnifiedIdeograph66e1': '\xe6\x9b\xa1',
'CjkUnifiedIdeograph66e0': '\xe6\x9b\xa0',
'CjkUnifiedIdeograph66e3': '\xe6\x9b\xa3',
'CjkUnifiedIdeograph66e2': '\xe6\x9b\xa2',
'CjkUnifiedIdeograph7b58': '\xe7\xad\x98',
'CjkUnifiedIdeograph7328': '\xe7\x8c\xa8',
'CjkUnifiedIdeograph7329': '\xe7\x8c\xa9',
'SquareMansyon': '\xe3\x8d\x87',
'CjkUnifiedIdeograph7322': '\xe7\x8c\xa2',
'CjkUnifiedIdeograph70f5': '\xe7\x83\xb5',
'CjkUnifiedIdeograph7320': '\xe7\x8c\xa0',
'CjkUnifiedIdeograph7321': '\xe7\x8c\xa1',
'CjkUnifiedIdeograph7326': '\xe7\x8c\xa6',
'CjkUnifiedIdeograph7327': '\xe7\x8c\xa7',
'CjkUnifiedIdeograph7324': '\xe7\x8c\xa4',
'CjkUnifiedIdeograph7325': '\xe7\x8c\xa5',
'CjkUnifiedIdeograph98ae': '\xe9\xa2\xae',
'HangulSyllableJjwe': '\xec\xae\x80',
'UnicodeCharacter0xef37': '\xee\xbc\xb7',
'UnicodeCharacter0x177a': '\xe1\x9d\xba',
'UnicodeCharacter0x177c': '\xe1\x9d\xbc',
'UnicodeCharacter0x177b': '\xe1\x9d\xbb',
'UnicodeCharacter0x177e': '\xe1\x9d\xbe',
'UnicodeCharacter0x177d': '\xe1\x9d\xbd',
'UnicodeCharacter0x177f': '\xe1\x9d\xbf',
'CjkUnifiedIdeograph7b52': '\xe7\xad\x92',
'CjkUnifiedIdeograph8f19': '\xe8\xbc\x99',
'CjkUnifiedIdeograph8f18': '\xe8\xbc\x98',
'CjkUnifiedIdeograph4598': '\xe4\x96\x98',
'CjkUnifiedIdeograph4599': '\xe4\x96\x99',
'CjkUnifiedIdeograph7b53': '\xe7\xad\x93',
'CjkUnifiedIdeograph4594': '\xe4\x96\x94',
'CjkUnifiedIdeograph4595': '\xe4\x96\x95',
'CjkUnifiedIdeograph4596': '\xe4\x96\x96',
'CjkUnifiedIdeograph4597': '\xe4\x96\x97',
'ClosedSupersetOrEqualTo': '\xe2\xab\x92',
'CjkUnifiedIdeograph4591': '\xe4\x96\x91',
'CjkUnifiedIdeograph4592': '\xe4\x96\x92',
'CjkUnifiedIdeograph4593': '\xe4\x96\x93',
'HangulSyllableJwaes': '\xec\xa2\xbb',
'HangulSyllableHoh': '\xed\x99\x93',
'CjkUnifiedIdeograph91cf': '\xe9\x87\x8f',
'UnicodeCharacter0xe5dc': '\xee\x97\x9c',
'CjkUnifiedIdeograph732b': '\xe7\x8c\xab',
'CjkUnifiedIdeograph732c': '\xe7\x8c\xac',
'CjkUnifiedIdeograph732a': '\xe7\x8c\xaa',
'CjkUnifiedIdeograph732f': '\xe7\x8c\xaf',
'HebrewLetterAlefWithQamats': '\xef\xac\xaf',
'CjkUnifiedIdeograph732d': '\xe7\x8c\xad',
'CjkUnifiedIdeograph732e': '\xe7\x8c\xae',
'UnicodeCharacter0xe5da': '\xee\x97\x9a',
'HangulSyllableJweoss': '\xec\xa4\xac',
'CjkUnifiedIdeograph91cc': '\xe9\x87\x8c',
'KangxiRadicalHalfTreeTrunk': '\xe2\xbd\x99',
'CjkUnifiedIdeograph91cb': '\xe9\x87\x8b',
'HangulSingleDotToneMark': '\xe3\x80\xae',
'LatinSmallLetterClosedReversedOpenE': '\xc9\x9e',
'CjkUnifiedIdeograph76ce': '\xe7\x9b\x8e',
'YiSyllableLyx': '\xea\x87\x95',
'UnicodeCharacter0xe5dd': '\xee\x97\x9d',
'CjkUnifiedIdeograph459d': '\xe4\x96\x9d',
'CjkUnifiedIdeograph459e': '\xe4\x96\x9e',
'CjkUnifiedIdeograph459f': '\xe4\x96\x9f',
'CjkUnifiedIdeograph7d4d': '\xe7\xb5\x8d',
'YiSyllableLyt': '\xea\x87\x94',
'CjkUnifiedIdeograph459a': '\xe4\x96\x9a',
'CjkUnifiedIdeograph459b': '\xe4\x96\x9b',
'LatinSmallLetterLWithMiddleDot': '\xc5\x80',
'UnicodeCharacter0x1779': '\xe1\x9d\xb9',
'UnicodeCharacter0x1778': '\xe1\x9d\xb8',
'CjkRadicalGhost': '\xe2\xbb\xa4',
'UnicodeCharacter0x1771': '\xe1\x9d\xb1',
'UnicodeCharacter0x1775': '\xe1\x9d\xb5',
'UnicodeCharacter0x1774': '\xe1\x9d\xb4',
'UnicodeCharacter0x1777': '\xe1\x9d\xb7',
'UnicodeCharacter0x1776': '\xe1\x9d\xb6',
'EthiopicSyllableQhaa': '\xe1\x89\x93',
'GlagoliticSmallLetterSmallYus': '\xe2\xb1\x94',
'CjkUnifiedIdeograph653a': '\xe6\x94\xba',
'CjkUnifiedIdeograph653b': '\xe6\x94\xbb',
'CjkUnifiedIdeograph653c': '\xe6\x94\xbc',
'CjkUnifiedIdeograph653d': '\xe6\x94\xbd',
'CjkUnifiedIdeograph653e': '\xe6\x94\xbe',
'CjkUnifiedIdeograph653f': '\xe6\x94\xbf',
'HangulSyllableGgeunj': '\xeb\x81\x89',
'CombiningRightArrowheadBelow': '\xcd\x95',
'HangulSyllableGgeung': '\xeb\x81\x99',
'ArmenianCapitalLetterTiwn': '\xd5\x8f',
'HangulSyllableGgyu': '\xeb\x80\xa8',
'CjkUnifiedIdeograph5249': '\xe5\x89\x89',
'CjkUnifiedIdeograph5248': '\xe5\x89\x88',
'UnicodeCharacter0xe3d8': '\xee\x8f\x98',
'UnicodeCharacter0xe3d9': '\xee\x8f\x99',
'HangulSyllableColm': '\xec\xb4\x92',
'CjkUnifiedIdeograph5243': '\xe5\x89\x83',
'CjkUnifiedIdeograph5242': '\xe5\x89\x82',
'CjkUnifiedIdeograph5241': '\xe5\x89\x81',
'CjkUnifiedIdeograph5240': '\xe5\x89\x80',
'LatinCapitalLetterOWithTildeAndDiaeresis': '\xe1\xb9\x8e',
'CjkUnifiedIdeograph5246': '\xe5\x89\x86',
'CjkUnifiedIdeograph5245': '\xe5\x89\x85',
'CjkUnifiedIdeograph5244': '\xe5\x89\x84',
'CjkUnifiedIdeograph58ca': '\xe5\xa3\x8a',
'CjkUnifiedIdeograph58cb': '\xe5\xa3\x8b',
'CjkUnifiedIdeograph58cc': '\xe5\xa3\x8c',
'CjkUnifiedIdeograph58cd': '\xe5\xa3\x8d',
'CjkUnifiedIdeograph58ce': '\xe5\xa3\x8e',
'CjkUnifiedIdeograph58cf': '\xe5\xa3\x8f',
'ArabicLigatureNoonWithMeemWithYehFinalForm': '\xef\xb6\x9a',
'HangulSyllableCols': '\xec\xb4\x94',
'HangulSyllableGgyo': '\xea\xbe\x9c',
'HangulSyllableGgyi': '\xeb\x81\xa0',
'HangulSyllableColt': '\xec\xb4\x95',
'CjkUnifiedIdeograph6530': '\xe6\x94\xb0',
'CjkUnifiedIdeograph6531': '\xe6\x94\xb1',
'CjkUnifiedIdeograph6532': '\xe6\x94\xb2',
'CjkUnifiedIdeograph6533': '\xe6\x94\xb3',
'CjkUnifiedIdeograph6534': '\xe6\x94\xb4',
'CjkUnifiedIdeograph6535': '\xe6\x94\xb5',
'CjkUnifiedIdeograph6536': '\xe6\x94\xb6',
'CjkUnifiedIdeograph6537': '\xe6\x94\xb7',
'CjkUnifiedIdeograph6538': '\xe6\x94\xb8',
'CjkUnifiedIdeograph6539': '\xe6\x94\xb9',
'UnicodeCharacter0xd8db': '\xed\xa3\x9b',
'ArabicLigatureYehWithHamzaAboveWithYehFinalForm': '\xef\xb1\xa9',
'CombiningCyrillicPalatalization': '\xd2\x84',
'HangulSyllableBbyenh': '\xeb\xbc\xaa',
'CjkUnifiedIdeograph58c0': '\xe5\xa3\x80',
'CjkUnifiedIdeograph58c1': '\xe5\xa3\x81',
'CjkUnifiedIdeograph58c2': '\xe5\xa3\x82',
'CjkUnifiedIdeograph58c3': '\xe5\xa3\x83',
'CjkUnifiedIdeograph58c4': '\xe5\xa3\x84',
'CjkUnifiedIdeograph58c5': '\xe5\xa3\x85',
'CjkUnifiedIdeograph58c6': '\xe5\xa3\x86',
'CjkUnifiedIdeograph58c7': '\xe5\xa3\x87',
'CjkUnifiedIdeograph58c8': '\xe5\xa3\x88',
'CjkUnifiedIdeograph58c9': '\xe5\xa3\x89',
'UnicodeCharacter0xd8dc': '\xed\xa3\x9c',
'HangulSyllableJwaen': '\xec\xa2\xac',
'KipSign': '\xe2\x82\xad',
'CjkUnifiedIdeograph524c': '\xe5\x89\x8c',
'CjkUnifiedIdeograph524b': '\xe5\x89\x8b',
'CjkUnifiedIdeograph524a': '\xe5\x89\x8a',
'LatinCapitalLetterHWithStroke': '\xc4\xa6',
'UnicodeCharacter0xfaf5': '\xef\xab\xb5',
'CjkUnifiedIdeograph524f': '\xe5\x89\x8f',
'CjkUnifiedIdeograph524e': '\xe5\x89\x8e',
'CjkUnifiedIdeograph524d': '\xe5\x89\x8d',
'UnicodeCharacter0xabda': '\xea\xaf\x9a',
'SaurashtraDigitTwo': '\xea\xa3\x92',
'LatinLetterReversedGlottalStopWithStroke': '\xca\xa2',
'HangulSyllableNweog': '\xeb\x88\xa1',
'CjkUnifiedIdeograph9ad6': '\xe9\xab\x96',
'LatinCapitalLetterGWithCedilla': '\xc4\xa2',
'SquareHuaraddo': '\xe3\x8c\xb2',
'KannadaLetterBha': '\xe0\xb2\xad',
'UnicodeCharacter0xdf33': '\xed\xbc\xb3',
'CjkUnifiedIdeograph766f': '\xe7\x99\xaf',
'CjkUnifiedIdeograph766e': '\xe7\x99\xae',
'CjkUnifiedIdeograph766d': '\xe7\x99\xad',
'CjkUnifiedIdeograph766c': '\xe7\x99\xac',
'CjkUnifiedIdeograph766b': '\xe7\x99\xab',
'SouthEastArrowCrossingNorthEastArrow': '\xe2\xa4\xad',
'SouthEastWhiteArrow': '\xe2\xac\x82',
'UnicodeCharacter0xebfd': '\xee\xaf\xbd',
'HangulSyllableDdanj': '\xeb\x94\xb5',
'UnicodeCharacter0xef5d': '\xee\xbd\x9d',
'HangulSyllableWal': '\xec\x99\x88',
'HangulSyllableWam': '\xec\x99\x90',
'HangulSyllableWan': '\xec\x99\x84',
'HangulSyllableWah': '\xec\x99\x9b',
'UnicodeCharacter0xd8df': '\xed\xa3\x9f',
'HangulSyllableWaj': '\xec\x99\x96',
'HangulSyllableWak': '\xec\x99\x98',
'HangulSyllableWad': '\xec\x99\x87',
'HangulSyllableWae': '\xec\x99\x9c',
'EthiopicSyllableSee': '\xe1\x88\xb4',
'HangulSyllableWab': '\xec\x99\x91',
'HangulSyllableWac': '\xec\x99\x97',
'HangulSyllableHos': '\xed\x99\x8b',
'UnicodeCharacter0xf8fe': '\xef\xa3\xbe',
'UnicodeCharacter0xef5e': '\xee\xbd\x9e',
'UnicodeCharacter0xe852': '\xee\xa1\x92',
'HangulSyllableWat': '\xec\x99\x99',
'HangulSyllableWap': '\xec\x99\x9a',
'HangulSyllableWas': '\xec\x99\x93',
'HangulJungseongYeo': '\xe1\x85\xa7',
'HangulSyllableCeonj': '\xec\xb2\x9d',
'EqualToOrSucceeds': '\xe2\x8b\x9f',
'TibetanSubjoinedLetterTtha': '\xe0\xbe\x9b',
'CjkUnifiedIdeograph7667': '\xe7\x99\xa7',
'CjkUnifiedIdeograph7666': '\xe7\x99\xa6',
'CjkUnifiedIdeograph7665': '\xe7\x99\xa5',
'CjkUnifiedIdeograph7664': '\xe7\x99\xa4',
'CjkUnifiedIdeograph7663': '\xe7\x99\xa3',
'CjkUnifiedIdeograph7662': '\xe7\x99\xa2',
'CjkUnifiedIdeograph7661': '\xe7\x99\xa1',
'CjkUnifiedIdeograph7660': '\xe7\x99\xa0',
'LatinCapitalLigatureOe': '\xc5\x92',
'HangulSyllableBbeug': '\xec\x81\x99',
'HalfwidthHangulLetterKhieukh': '\xef\xbe\xbb',
'CjkUnifiedIdeograph3c53': '\xe3\xb1\x93',
'CjkUnifiedIdeograph3c52': '\xe3\xb1\x92',
'CjkUnifiedIdeograph3c51': '\xe3\xb1\x91',
'CjkUnifiedIdeograph3c50': '\xe3\xb1\x90',
'CjkUnifiedIdeograph3c57': '\xe3\xb1\x97',
'CjkUnifiedIdeograph3c56': '\xe3\xb1\x96',
'CjkUnifiedIdeograph3c55': '\xe3\xb1\x95',
'CjkUnifiedIdeograph3c54': '\xe3\xb1\x94',
'CjkUnifiedIdeograph6e0e': '\xe6\xb8\x8e',
'CjkUnifiedIdeograph6e0d': '\xe6\xb8\x8d',
'CjkUnifiedIdeograph3c59': '\xe3\xb1\x99',
'CjkUnifiedIdeograph3c58': '\xe3\xb1\x98',
'CjkUnifiedIdeograph6e0a': '\xe6\xb8\x8a',
'HangulSyllableRwabs': '\xeb\xa2\x8a',
'CjkUnifiedIdeograph6e0c': '\xe6\xb8\x8c',
'CjkUnifiedIdeograph6e0b': '\xe6\xb8\x8b',
'GreekSmallLetterUpsilonWithDialytikaAndVaria': '\xe1\xbf\xa2',
'CjkUnifiedIdeograph914e': '\xe9\x85\x8e',
'HangulSyllableBbeun': '\xec\x81\x9c',
'UnicodeCharacter0xdd87': '\xed\xb6\x87',
'HangulSyllableMyon': '\xeb\xac\x9c',
'ArabicLigatureTehWithKhahWithMeemInitialForm': '\xef\xb5\x94',
'UnicodeCharacter0xf73a': '\xef\x9c\xba',
'HangulSyllableGyeot': '\xea\xb3\x81',
'UnicodeCharacter0xab50': '\xea\xad\x90',
'KangxiRadicalDottedTent': '\xe2\xbd\xa8',
'HangulSyllableMyom': '\xeb\xac\xa8',
'SupersetAboveSuperset': '\xe2\xab\x96',
'CjkStrokeSzz': '\xe3\x87\x9e',
'HangulSyllableMyoj': '\xeb\xac\xae',
'KhmerSignAvakrahasanya': '\xe1\x9f\x9c',
'HangulSyllableMyok': '\xeb\xac\xb0',
'HangulSyllableCaeng': '\xec\xb1\x99',
'UnicodeCharacter0xf73b': '\xef\x9c\xbb',
'HangulSyllableMyoh': '\xeb\xac\xb3',
'GreekSmallLetterIotaWithVrachy': '\xe1\xbf\x90',
'HangulSyllableBbes': '\xeb\xbb\xbf',
'HangulSyllableBbep': '\xeb\xbc\x86',
'HangulSyllableTwaek': '\xed\x87\xb0',
'HangulSyllableBbet': '\xeb\xbc\x85',
'HangulSyllableBbeu': '\xec\x81\x98',
'CjkUnifiedIdeograph3c5c': '\xe3\xb1\x9c',
'CjkUnifiedIdeograph3c5b': '\xe3\xb1\x9b',
'CjkUnifiedIdeograph3c5a': '\xe3\xb1\x9a',
'DottedTranspositionMarker': '\xe2\xb8\x88',
'CjkUnifiedIdeograph3c5f': '\xe3\xb1\x9f',
'CjkUnifiedIdeograph3c5e': '\xe3\xb1\x9e',
'CjkUnifiedIdeograph3c5d': '\xe3\xb1\x9d',
'CjkUnifiedIdeograph6e05': '\xe6\xb8\x85',
'CjkUnifiedIdeograph6e04': '\xe6\xb8\x84',
'CjkUnifiedIdeograph6e07': '\xe6\xb8\x87',
'CjkUnifiedIdeograph6e06': '\xe6\xb8\x86',
'CjkUnifiedIdeograph6e01': '\xe6\xb8\x81',
'CjkUnifiedIdeograph6e00': '\xe6\xb8\x80',
'CjkUnifiedIdeograph6e03': '\xe6\xb8\x83',
'CjkUnifiedIdeograph6e02': '\xe6\xb8\x82',
'UnicodeCharacter0xdf20': '\xed\xbc\xa0',
'CjkUnifiedIdeograph4a1a': '\xe4\xa8\x9a',
'CjkUnifiedIdeograph4a1b': '\xe4\xa8\x9b',
'CjkUnifiedIdeograph4a1c': '\xe4\xa8\x9c',
'CjkUnifiedIdeograph4a1d': '\xe4\xa8\x9d',
'CjkUnifiedIdeograph4a1e': '\xe4\xa8\x9e',
'CjkUnifiedIdeograph4a1f': '\xe4\xa8\x9f',
'CjkUnifiedIdeograph7a26': '\xe7\xa8\xa6',
'ArabicLetterTah': '\xd8\xb7',
'HangulSyllableJjweobs': '\xec\xad\xb6',
'CjkUnifiedIdeograph91c5': '\xe9\x87\x85',
'LatinCapitalLetterEth': '\xc3\x90',
'HangulSyllableGyeolt': '\xea\xb2\xb5',
'ArabicLetterGhainIsolatedForm': '\xef\xbb\x8d',
'CjkUnifiedIdeograph7b5c': '\xe7\xad\x9c',
'VaiQuestionMark': '\xea\x98\x8f',
'KangxiRadicalRat': '\xe2\xbf\x8f',
'KangxiRadicalRap': '\xe2\xbd\x81',
'CjkUnifiedIdeograph91c6': '\xe9\x87\x86',
'HangulSyllableGyeos': '\xea\xb2\xbb',
'CjkUnifiedIdeograph7b5f': '\xe7\xad\x9f',
'OghamLetterUr': '\xe1\x9a\x92',
'nearrow': '\xe2\x86\x97',
'SmallTilde': '\xcb\x9c',
'ThaanaLetterBaa': '\xde\x84',
'CjkUnifiedIdeograph91c3': '\xe9\x87\x83',
'UnicodeCharacter0x4db8': '\xe4\xb6\xb8',
'UnicodeCharacter0x4db9': '\xe4\xb6\xb9',
'UnicodeCharacter0x4db6': '\xe4\xb6\xb6',
'UnicodeCharacter0x4db7': '\xe4\xb6\xb7',
'YiSyllableMgie': '\xea\x88\xb1',
'CjkUnifiedIdeograph91c2': '\xe9\x87\x82',
'HangulSyllableBbwagg': '\xeb\xbd\x9e',
'UnicodeCharacter0xf7ef': '\xef\x9f\xaf',
'TamilLetterEe': '\xe0\xae\x8f',
'UnicodeCharacter0xda10': '\xed\xa8\x90',
'HangulSyllableSogg': '\xec\x86\x8e',
'CjkUnifiedIdeograph4a10': '\xe4\xa8\x90',
'CjkUnifiedIdeograph4a11': '\xe4\xa8\x91',
'CjkUnifiedIdeograph4a12': '\xe4\xa8\x92',
'CjkUnifiedIdeograph4a13': '\xe4\xa8\x93',
'CjkUnifiedIdeograph4a14': '\xe4\xa8\x94',
'CjkUnifiedIdeograph4a15': '\xe4\xa8\x95',
'CjkUnifiedIdeograph4a16': '\xe4\xa8\x96',
'CjkUnifiedIdeograph4a17': '\xe4\xa8\x97',
'RunicLetterFehuFeohFeF': '\xe1\x9a\xa0',
'CjkUnifiedIdeograph4a19': '\xe4\xa8\x99',
'UnicodeCharacter0xee0d': '\xee\xb8\x8d',
'NewTaiLueLetterHighDa': '\xe1\xa6\xa1',
'GujaratiLetterDa': '\xe0\xaa\xa6',
'HangulSyllableGyeolg': '\xea\xb2\xb1',
'UnicodeCharacter0xab51': '\xea\xad\x91',
'HangulSyllableSsilh': '\xec\x94\xb7',
'UnicodeCharacter0x4dbf': '\xe4\xb6\xbf',
'UnicodeCharacter0x4dbd': '\xe4\xb6\xbd',
'UnicodeCharacter0x4dbe': '\xe4\xb6\xbe',
'UnicodeCharacter0x4dbb': '\xe4\xb6\xbb',
'UnicodeCharacter0x4dbc': '\xe4\xb6\xbc',
'UnicodeCharacter0x4dba': '\xe4\xb6\xba',
'UnicodeCharacter0xa8dc': '\xea\xa3\x9c',
'BraillePatternDots2458': '\xe2\xa2\x9a',
'UpDownArrowWithBase': '\xe2\x86\xa8',
'DoubleSquareIntersection': '\xe2\xa9\x8e',
'BraillePatternDots2456': '\xe2\xa0\xba',
'BraillePatternDots2457': '\xe2\xa1\x9a',
'HangulSyllableSsilp': '\xec\x94\xb6',
'HangulSyllableSsils': '\xec\x94\xb4',
'HangulSyllableGyeolb': '\xea\xb2\xb3',
'HangulSyllableJjeut': '\xec\xaf\xad',
'HangulSyllableHagg': '\xed\x95\x9a',
'HangulSyllableTyang': '\xed\x84\x8d',
'VaiSyllableHun': '\xea\x96\x98',
'HangulSyllableTyanj': '\xed\x83\xbd',
'HangulSyllableTyanh': '\xed\x83\xbe',
'HangulSyllableJjwagg': '\xec\xab\x9a',
'UnicodeCharacter0xdbcf': '\xed\xaf\x8f',
'UnicodeCharacter0xdbce': '\xed\xaf\x8e',
'UnicodeCharacter0xdbcd': '\xed\xaf\x8d',
'UnicodeCharacter0xdbcc': '\xed\xaf\x8c',
'LatinSmallLetterLzDigraph': '\xca\xab',
'HangulSyllableHags': '\xed\x95\x9b',
'GeorgianSmallLetterBan': '\xe2\xb4\x81',
'RightHalfBlackCircle': '\xe2\x97\x97',
'BalineseWindu': '\xe1\xad\x9c',
'CjkUnifiedIdeograph69dd': '\xe6\xa7\x9d',
'HalfwidthKatakanaLetterSu': '\xef\xbd\xbd',
'CjkUnifiedIdeograph9b59': '\xe9\xad\x99',
'ArabicLetterNoonWithTwoDotsBelow': '\xdd\xa7',
'UnicodeCharacter0xedc8': '\xee\xb7\x88',
'UnicodeCharacter0xedc9': '\xee\xb7\x89',
'UnicodeCharacter0xedc6': '\xee\xb7\x86',
'UnicodeCharacter0xedc7': '\xee\xb7\x87',
'UnicodeCharacter0xedc4': '\xee\xb7\x84',
'UnicodeCharacter0xedc5': '\xee\xb7\x85',
'UnicodeCharacter0xedc2': '\xee\xb7\x82',
'UnicodeCharacter0xedc3': '\xee\xb7\x83',
'UnicodeCharacter0xedc0': '\xee\xb7\x80',
'UnicodeCharacter0xedc1': '\xee\xb7\x81',
'UnicodeCharacter0xdbc7': '\xed\xaf\x87',
'UnicodeCharacter0xdbc6': '\xed\xaf\x86',
'UnicodeCharacter0xdbc5': '\xed\xaf\x85',
'HangulSyllableGeoss': '\xea\xb2\x84',
'UnicodeCharacter0xdbc3': '\xed\xaf\x83',
'UnicodeCharacter0xdbc2': '\xed\xaf\x82',
'UnicodeCharacter0xdbc1': '\xed\xaf\x81',
'ModifierLetterSmallNWithRetroflexHook': '\xe1\xb6\xaf',
'UnicodeCharacter0xdbc9': '\xed\xaf\x89',
'UnicodeCharacter0xdbc8': '\xed\xaf\x88',
'YiSyllableKux': '\xea\x88\x8f',
'YiSyllableLat': '\xea\x86\xbf',
'YiSyllableSsux': '\xea\x8c\xbd',
'HangulSyllableJjeud': '\xec\xaf\x9b',
'TripleVerticalBarDelimiter': '\xe2\xa6\x80',
'HangulSyllableSswagg': '\xec\x8f\xb6',
'UnicodeCharacter0xedcf': '\xee\xb7\x8f',
'UnicodeCharacter0xedcd': '\xee\xb7\x8d',
'UnicodeCharacter0xedce': '\xee\xb7\x8e',
'UnicodeCharacter0xedcb': '\xee\xb7\x8b',
'UnicodeCharacter0xedcc': '\xee\xb7\x8c',
'UnicodeCharacter0xedca': '\xee\xb7\x8a',
'CanadianSyllabicsWo': '\xe1\x90\x92',
'UnicodeCharacter0xabd2': '\xea\xaf\x92',
'CanadianSyllabicsWi': '\xe1\x90\x8e',
'YiSyllableKur': '\xea\x88\x93',
'HangulSyllableRyigg': '\xeb\xa6\x92',
'CanadianSyllabicsWe': '\xe1\x90\x8c',
'CanadianSyllabicsWa': '\xe1\x90\x97',
'SphericalAngleOpeningLeft': '\xe2\xa6\xa0',
'HangulSyllableDelb': '\xeb\x8d\xbb',
'YiSyllableKup': '\xea\x88\x91',
'LowerRightTriangle': '\xe2\x97\xbf',
'SundaneseVowelSignPanyuku': '\xe1\xae\xa5',
'HangulSyllableDelg': '\xeb\x8d\xb9',
'HangulSyllableDelh': '\xeb\x8d\xbf',
'YiSyllableRox': '\xea\x8f\x8e',
'HangulSyllableJjeuc': '\xec\xaf\xab',
'HangulSyllableDelm': '\xeb\x8d\xba',
'YiSyllableSsup': '\xea\x8c\xbf',
'CanadianSyllabicsNnga': '\xe1\x99\xb5',
'HangulSyllableJjeul': '\xec\xaf\x9c',
'HangulSyllableDels': '\xeb\x8d\xbc',
'HangulSyllableDelt': '\xeb\x8d\xbd',
'UnicodeCharacter0xabd8': '\xea\xaf\x98',
'CanadianSyllabicsNngi': '\xe1\x99\xb1',
'GlagoliticCapitalLetterChrivi': '\xe2\xb0\x9d',
'CjkUnifiedIdeograph846f': '\xe8\x91\xaf',
'YiSyllableKut': '\xea\x88\x8e',
'LatinSmallLetterTWithDotAbove': '\xe1\xb9\xab',
'CyrillicSmallLetterEnWithHook': '\xd3\x88',
'CanadianSyllabicsNngo': '\xe1\x99\xb3',
'CjkUnifiedIdeograph846c': '\xe8\x91\xac',
'HangulSyllableBwaelg': '\xeb\xb4\xb5',
'LatinSmallLetterKWithStroke': '\xea\x9d\x81',
'SundaneseLetterYa': '\xe1\xae\x9a',
'CjkUnifiedIdeograph933f': '\xe9\x8c\xbf',
'BlackRightwardsArrow': '\xe2\x9e\xa1',
'HangulSyllableJjeuj': '\xec\xaf\xaa',
'HangulSyllableBaeh': '\xeb\xb1\x8b',
'YiSyllableKuo': '\xea\x88\x84',
'Conjunction': '\xe2\x98\x8c',
'MongolianLetterManchuI': '\xe1\xa1\xb3',
'HangulSyllableCaen': '\xec\xb1\x88',
'HangulSyllableCael': '\xec\xb1\x8c',
'HangulSyllableCaem': '\xec\xb1\x94',
'CjkUnifiedIdeograph8468': '\xe8\x91\xa8',
'LatinSmallLetterTWithStroke': '\xc5\xa7',
'HangulSyllableCaeh': '\xec\xb1\x9f',
'CjkUnifiedIdeograph8464': '\xe8\x91\xa4',
'CjkUnifiedIdeograph8465': '\xe8\x91\xa5',
'CjkUnifiedIdeograph8466': '\xe8\x91\xa6',
'CjkUnifiedIdeograph8467': '\xe8\x91\xa7',
'CjkUnifiedIdeograph8460': '\xe8\x91\xa0',
'CjkUnifiedIdeograph8461': '\xe8\x91\xa1',
'CjkUnifiedIdeograph8462': '\xe8\x91\xa2',
'CjkUnifiedIdeograph8463': '\xe8\x91\xa3',
'HangulSyllableGanj': '\xea\xb0\x85',
'HangulSyllableGanh': '\xea\xb0\x86',
'UnicodeCharacter0xeb16': '\xee\xac\x96',
'HangulSyllableCaet': '\xec\xb1\x9d',
'HangulSyllableGang': '\xea\xb0\x95',
'HangulSyllableCaes': '\xec\xb1\x97',
'HangulSyllableCaep': '\xec\xb1\x9e',
'HangulSyllableSsyass': '\xec\x8d\x84',
'UnicodeCharacter0xec1f': '\xee\xb0\x9f',
'UnicodeCharacter0xec1e': '\xee\xb0\x9e',
'CyrillicSmallLigatureEnGhe': '\xd2\xa5',
'ArabicLetterKafIsolatedForm': '\xef\xbb\x99',
'UnicodeCharacter0xee07': '\xee\xb8\x87',
'YiSyllableYie': '\xea\x91\xb7',
'HangulChoseongSiostikeut': '\xe1\x84\xaf',
'UnicodeCharacter0xec1b': '\xee\xb0\x9b',
'UnicodeCharacter0xde55': '\xed\xb9\x95',
'GreekCapitalLetterUpsilon': '\xce\xa5',
'LatinSmallLetterEWithCircumflexAndHookAbove': '\xe1\xbb\x83',
'UnicodeCharacter0xec1a': '\xee\xb0\x9a',
'HangulSyllableDyaebs': '\xeb\x8d\x8a',
'HangulSyllableSaeng': '\xec\x83\x9d',
'KangxiRadicalSilk': '\xe2\xbd\xb7',
'SylotiNagriVowelSignOo': '\xea\xa0\xa7',
'HangulSyllableSaenj': '\xec\x83\x8d',
'UnicodeCharacter0x2b84': '\xe2\xae\x84',
'UnicodeCharacter0x2b85': '\xe2\xae\x85',
'ThaanaSukun': '\xde\xb0',
'UnicodeCharacter0x2b87': '\xe2\xae\x87',
'UnicodeCharacter0x2b80': '\xe2\xae\x80',
'UnicodeCharacter0x2b81': '\xe2\xae\x81',
'UnicodeCharacter0x2b82': '\xe2\xae\x82',
'ThaanaLetterHhaa': '\xde\x99',
'TeluguLetterIi': '\xe0\xb0\x88',
'UnicodeCharacter0x2b89': '\xe2\xae\x89',
'TripleIntegral': '\xe2\x88\xad',
'CjkUnifiedIdeograph358a': '\xe3\x96\x8a',
'UnicodeCharacter0xe9f2': '\xee\xa7\xb2',
'UnicodeCharacter0xeb53': '\xee\xad\x93',
'UnicodeCharacter0xe6b0': '\xee\x9a\xb0',
'KangxiRadicalRain': '\xe2\xbe\xac',
'product': '\xe2\x88\x8f',
'UnicodeCharacter0xed77': '\xee\xb5\xb7',
'HalfwidthHangulLetterAe': '\xef\xbf\x83',
'CjkUnifiedIdeograph8f8c': '\xe8\xbe\x8c',
'BraillePatternDots1235': '\xe2\xa0\x97',
'BraillePatternDots1234': '\xe2\xa0\x8f',
'BraillePatternDots1237': '\xe2\xa1\x87',
'BraillePatternDots1236': '\xe2\xa0\xa7',
'HangulSyllableSyegs': '\xec\x85\xb3',
'UnicodeCharacter0xde50': '\xed\xb9\x90',
'BraillePatternDots1238': '\xe2\xa2\x87',
'VulgarFractionThreeEighths': '\xe2\x85\x9c',
'YiSyllableHxuox': '\xea\x89\x93',
'YiSyllableHxuot': '\xea\x89\x92',
'NkoLetterJonaRa': '\xdf\xaa',
'YiSyllableHxuop': '\xea\x89\x95',
'GreekSmallLetterFinalSigma': '\xcf\x82',
'UnicodeCharacter0x2b8d': '\xe2\xae\x8d',
'UnicodeCharacter0x2b8e': '\xe2\xae\x8e',
'UnicodeCharacter0x2b8f': '\xe2\xae\x8f',
'CyrillicSmallLetterErWithTick': '\xd2\x8f',
'UnicodeCharacter0x2b8a': '\xe2\xae\x8a',
'UnicodeCharacter0x2b8b': '\xe2\xae\x8b',
'UnicodeCharacter0x2b8c': '\xe2\xae\x8c',
'UnicodeCharacter0xde51': '\xed\xb9\x91',
'CjkUnifiedIdeograph99bb': '\xe9\xa6\xbb',
'SundaneseLetterA': '\xe1\xae\x83',
'CjkCompatibilityIdeographf957': '\xef\xa5\x97',
'LatinSmallLetterZWithDotBelow': '\xe1\xba\x93',
'DescendingNode': '\xe2\x98\x8b',
'HexagramForGreatTaming': '\xe4\xb7\x99',
'YiSyllableNex': '\xea\x86\x8e',
'HangulSyllableKyabs': '\xec\xba\xbe',
'CjkUnifiedIdeograph5d27': '\xe5\xb4\xa7',
'ZNotationLeftImageBracket': '\xe2\xa6\x87',
'HangulSyllableSyaeng': '\xec\x84\x95',
'HangulSyllableRabs': '\xeb\x9e\x8e',
'UpwardsWhiteArrowOnPedestalWithVerticalBar': '\xe2\x87\xad',
'SundaneseLetterI': '\xe1\xae\x84',
'UnicodeCharacter0xe86e': '\xee\xa1\xae',
'UnicodeCharacter0xe86d': '\xee\xa1\xad',
'UnicodeCharacter0xe86f': '\xee\xa1\xaf',
'UnicodeCharacter0xe86a': '\xee\xa1\xaa',
'CjkUnifiedIdeograph4912': '\xe4\xa4\x92',
'UnicodeCharacter0xe86c': '\xee\xa1\xac',
'UnicodeCharacter0xe86b': '\xee\xa1\xab',
'CjkUnifiedIdeograph430a': '\xe4\x8c\x8a',
'HangulSyllableGgyilh': '\xeb\x81\xaf',
'HangulSyllableOe': '\xec\x99\xb8',
'HangulSyllableOb': '\xec\x98\xb5',
'HangulSyllableOc': '\xec\x98\xbb',
'HangulSyllableOn': '\xec\x98\xa8',
'HangulSyllableOl': '\xec\x98\xac',
'HangulSyllableOm': '\xec\x98\xb4',
'HangulSyllableOj': '\xec\x98\xba',
'HangulSyllableOk': '\xec\x98\xbc',
'HangulSyllableOh': '\xec\x98\xbf',
'LatinCapitalLetterKWithCaron': '\xc7\xa8',
'HangulSyllableOs': '\xec\x98\xb7',
'HangulSyllableOp': '\xec\x98\xbe',
'not': '\xc2\xac',
'KangxiRadicalDrum': '\xe2\xbf\x8e',
'VaiSyllableTu': '\xea\x96\xa4',
'HangulSyllableGgogg': '\xea\xbc\xae',
'CjkUnifiedIdeograph5d25': '\xe5\xb4\xa5',
'UnicodeCharacter0xe865': '\xee\xa1\xa5',
'UnicodeCharacter0xe864': '\xee\xa1\xa4',
'UnicodeCharacter0xe867': '\xee\xa1\xa7',
'UnicodeCharacter0xe866': '\xee\xa1\xa6',
'UnicodeCharacter0xe861': '\xee\xa1\xa1',
'UnicodeCharacter0xe860': '\xee\xa1\xa0',
'UnicodeCharacter0xe863': '\xee\xa1\xa3',
'UnicodeCharacter0xe862': '\xee\xa1\xa2',
'VaiSyllableTa': '\xea\x95\x9a',
'HangulSyllableGgogs': '\xea\xbc\xaf',
'VaiSyllableTe': '\xea\x97\xb3',
'ThaiCharacterRu': '\xe0\xb8\xa4',
'VaiSyllableTi': '\xea\x94\xb3',
'GreekReversedLunateEpsilonSymbol': '\xcf\xb6',
'UnicodeCharacter0xe648': '\xee\x99\x88',
'UnicodeCharacter0xf47b': '\xef\x91\xbb',
'VaiSyllableTo': '\xea\x97\x8b',
'LatinCapitalLetterUWithTilde': '\xc5\xa8',
'ArabicShaddaIsolatedForm': '\xef\xb9\xbc',
'UnicodeCharacter0xec17': '\xee\xb0\x97',
'UnicodeCharacter0xec16': '\xee\xb0\x96',
'GreekSmallLetterEtaWithPsili': '\xe1\xbc\xa0',
'UnicodeCharacter0xec15': '\xee\xb0\x95',
'CjkUnifiedIdeograph82c9': '\xe8\x8b\x89',
'CjkUnifiedIdeograph82c8': '\xe8\x8b\x88',
'UnicodeCharacter0xa6a8': '\xea\x9a\xa8',
'CjkUnifiedIdeograph3840': '\xe3\xa1\x80',
'CjkUnifiedIdeograph82c1': '\xe8\x8b\x81',
'CjkUnifiedIdeograph82c0': '\xe8\x8b\x80',
'CjkUnifiedIdeograph82c3': '\xe8\x8b\x83',
'CjkUnifiedIdeograph82c2': '\xe8\x8b\x82',
'CjkUnifiedIdeograph82c5': '\xe8\x8b\x85',
'CjkUnifiedIdeograph82c4': '\xe8\x8b\x84',
'CjkUnifiedIdeograph82c7': '\xe8\x8b\x87',
'CjkUnifiedIdeograph82c6': '\xe8\x8b\x86',
'HebrewPunctuationSofPasuq': '\xd7\x83',
'CjkUnifiedIdeograph6b65': '\xe6\xad\xa5',
'CjkUnifiedIdeograph6b66': '\xe6\xad\xa6',
'CjkUnifiedIdeograph6b67': '\xe6\xad\xa7',
'CjkUnifiedIdeograph6b60': '\xe6\xad\xa0',
'CjkUnifiedIdeograph6b61': '\xe6\xad\xa1',
'CjkUnifiedIdeograph6b62': '\xe6\xad\xa2',
'CjkUnifiedIdeograph6b63': '\xe6\xad\xa3',
'UnicodeCharacter0xf5e2': '\xef\x97\xa2',
'HangulSyllableRwin': '\xeb\xa5\x80',
'UnicodeCharacter0xec10': '\xee\xb0\x90',
'CjkUnifiedIdeograph6b68': '\xe6\xad\xa8',
'CjkUnifiedIdeograph6b69': '\xe6\xad\xa9',
'HangulSyllableNwa': '\xeb\x86\x94',
'GreekSmallLetterOmegaWithPsiliAndYpogegrammeni': '\xe1\xbe\xa0',
'HangulSyllableRwebs': '\xeb\xa4\xb2',
'BengaliSignVirama': '\xe0\xa7\x8d',
'HangulSyllableBweoh': '\xeb\xb6\xb7',
'HangulSyllableBweok': '\xeb\xb6\xb4',
'LatinSmallLetterWWithDotBelow': '\xe1\xba\x89',
'HangulSyllableBweom': '\xeb\xb6\xac',
'HangulSyllableBweol': '\xeb\xb6\xa4',
'HangulSyllableBweon': '\xeb\xb6\xa0',
'CjkUnifiedIdeograph82ca': '\xe8\x8b\x8a',
'UnicodeCharacter0xa6ac': '\xea\x9a\xac',
'CjkUnifiedIdeograph82cc': '\xe8\x8b\x8c',
'CjkUnifiedIdeograph82cb': '\xe8\x8b\x8b',
'CjkUnifiedIdeograph82ce': '\xe8\x8b\x8e',
'CjkUnifiedIdeograph82cd': '\xe8\x8b\x8d',
'UnicodeCharacter0xa6ad': '\xea\x9a\xad',
'CjkUnifiedIdeograph82cf': '\xe8\x8b\x8f',
'ArabicLetterYuFinalForm': '\xef\xaf\x9c',
'HangulSyllableBweop': '\xeb\xb6\xb6',
'HangulSyllableBweos': '\xeb\xb6\xaf',
'GreekSmallLetterIotaWithDasia': '\xe1\xbc\xb1',
'HangulSyllableBweot': '\xeb\xb6\xb5',
'CjkUnifiedIdeograph8b81': '\xe8\xae\x81',
'KhmerSymbolPramRoc': '\xe1\xa7\xb5',
'<<-': '\xe2\x86\x9e',
'CjkUnifiedIdeograph6b6d': '\xe6\xad\xad',
'CjkUnifiedIdeograph6b6e': '\xe6\xad\xae',
'CjkUnifiedIdeograph6b6f': '\xe6\xad\xaf',
'CircledIdeographEight': '\xe3\x8a\x87',
'CjkUnifiedIdeograph6b6a': '\xe6\xad\xaa',
'CjkUnifiedIdeograph6b6b': '\xe6\xad\xab',
'PlanckConstantOverTwoPi': '\xe2\x84\x8f',
'SquareLeftOpenBoxOperator': '\xe2\xab\x8d',
'LaoDigitNine': '\xe0\xbb\x99',
'CjkUnifiedIdeograph4b57': '\xe4\xad\x97',
'CjkUnifiedIdeograph4b56': '\xe4\xad\x96',
'CjkUnifiedIdeograph4b55': '\xe4\xad\x95',
'CjkUnifiedIdeograph4b54': '\xe4\xad\x94',
'CjkUnifiedIdeograph4b53': '\xe4\xad\x93',
'CjkUnifiedIdeograph4b52': '\xe4\xad\x92',
'CjkUnifiedIdeograph4b51': '\xe4\xad\x91',
'CjkUnifiedIdeograph4b50': '\xe4\xad\x90',
'HangulSyllableHyaelp': '\xed\x96\xba',
'CjkUnifiedIdeograph4b59': '\xe4\xad\x99',
'CjkUnifiedIdeograph4b58': '\xe4\xad\x98',
'UnicodeCharacter0xf47e': '\xef\x91\xbe',
'ArabicLigatureMeemWithKhahWithJeemInitialForm': '\xef\xb6\x8e',
'UnicodeCharacter0x1aea': '\xe1\xab\xaa',
'UnicodeCharacter0x1aec': '\xe1\xab\xac',
'UnicodeCharacter0x1aeb': '\xe1\xab\xab',
'UnicodeCharacter0x1aee': '\xe1\xab\xae',
'UnicodeCharacter0x1aed': '\xe1\xab\xad',
'UnicodeCharacter0x1aef': '\xe1\xab\xaf',
'UnicodeCharacter0xd804': '\xed\xa0\x84',
'UnicodeCharacter0xd805': '\xed\xa0\x85',
'UnicodeCharacter0xd806': '\xed\xa0\x86',
'UnicodeCharacter0xd807': '\xed\xa0\x87',
'BowtieWithRightHalfBlack': '\xe2\xa7\x92',
'CjkUnifiedIdeograph76dd': '\xe7\x9b\x9d',
'UnicodeCharacter0xd802': '\xed\xa0\x82',
'UnicodeCharacter0xd803': '\xed\xa0\x83',
'HangulSyllableNwe': '\xeb\x88\xbc',
'BengaliLetterTta': '\xe0\xa6\x9f',
'UnicodeCharacter0xd808': '\xed\xa0\x88',
'UnicodeCharacter0xd809': '\xed\xa0\x89',
'UnicodeCharacter0xf47d': '\xef\x91\xbd',
'ArmenianExclamationMark': '\xd5\x9c',
'CjkUnifiedIdeograph93d8': '\xe9\x8f\x98',
'CjkUnifiedIdeograph93d9': '\xe9\x8f\x99',
'CjkUnifiedIdeograph85cb': '\xe8\x97\x8b',
'SyriacLetterDalath': '\xdc\x95',
'CjkUnifiedIdeograph93d4': '\xe9\x8f\x94',
'TibetanMarkAngKhangGyon': '\xe0\xbc\xbc',
'CjkUnifiedIdeograph85cf': '\xe8\x97\x8f',
'Response': '\xe2\x84\x9f',
'CjkUnifiedIdeograph85cd': '\xe8\x97\x8d',
'CjkUnifiedIdeograph85ce': '\xe8\x97\x8e',
'KangxiRadicalProfound': '\xe2\xbd\x9e',
'UnicodeCharacter0x1ae8': '\xe1\xab\xa8',
'CjkUnifiedIdeograph5d18': '\xe5\xb4\x98',
'CjkUnifiedIdeograph5d19': '\xe5\xb4\x99',
'UnicodeCharacter0x1ae1': '\xe1\xab\xa1',
'UnicodeCharacter0x1ae0': '\xe1\xab\xa0',
'UnicodeCharacter0x1ae3': '\xe1\xab\xa3',
'UnicodeCharacter0x1ae2': '\xe1\xab\xa2',
'UnicodeCharacter0x1ae5': '\xe1\xab\xa5',
'UnicodeCharacter0x1ae4': '\xe1\xab\xa4',
'UnicodeCharacter0x1ae7': '\xe1\xab\xa7',
'UnicodeCharacter0x1ae6': '\xe1\xab\xa6',
'CjkUnifiedIdeograph4b5f': '\xe4\xad\x9f',
'CjkUnifiedIdeograph4b5e': '\xe4\xad\x9e',
'CjkUnifiedIdeograph4b5d': '\xe4\xad\x9d',
'CjkUnifiedIdeograph4b5c': '\xe4\xad\x9c',
'CjkUnifiedIdeograph4b5b': '\xe4\xad\x9b',
'CjkUnifiedIdeograph4b5a': '\xe4\xad\x9a',
'CjkUnifiedIdeograph85c8': '\xe8\x97\x88',
'CjkUnifiedIdeograph85c9': '\xe8\x97\x89',
'CyrillicSmallLetterDje': '\xd1\x92',
'CjkUnifiedIdeograph85c2': '\xe8\x97\x82',
'CjkUnifiedIdeograph85c3': '\xe8\x97\x83',
'CjkUnifiedIdeograph85c0': '\xe8\x97\x80',
'CjkUnifiedIdeograph85c1': '\xe8\x97\x81',
'CjkUnifiedIdeograph85c6': '\xe8\x97\x86',
'CjkUnifiedIdeograph85c7': '\xe8\x97\x87',
'CjkUnifiedIdeograph85c4': '\xe8\x97\x84',
'CjkUnifiedIdeograph85c5': '\xe8\x97\x85',
'HangulSyllableGyeon': '\xea\xb2\xac',
'UnicodeCharacter0xd80e': '\xed\xa0\x8e',
'HangulSyllableGyeol': '\xea\xb2\xb0',
'HangulSyllableGyeom': '\xea\xb2\xb8',
'HangulSyllableGyeoj': '\xea\xb2\xbe',
'MyanmarLetterSgawKarenSha': '\xe1\x81\xa1',
'HangulSyllableGyeoh': '\xea\xb3\x83',
'UnicodeCharacter0xd80c': '\xed\xa0\x8c',
'SquareThz': '\xe3\x8e\x94',
'HangulSyllableGyeog': '\xea\xb2\xa9',
'HangulSyllableGyeod': '\xea\xb2\xaf',
'CjkUnifiedIdeograph8a3a': '\xe8\xa8\xba',
'HangulSyllableGyeob': '\xea\xb2\xb9',
'HangulSyllableGyeoc': '\xea\xb2\xbf',
'MalayalamLetterMa': '\xe0\xb4\xae',
'HebrewLetterWideDalet': '\xef\xac\xa2',
'CjkUnifiedIdeograph90b9': '\xe9\x82\xb9',
'CjkUnifiedIdeograph90b8': '\xe9\x82\xb8',
'CjkUnifiedIdeograph45cf': '\xe4\x97\x8f',
'CjkUnifiedIdeograph90b6': '\xe9\x82\xb6',
'CjkUnifiedIdeograph45cd': '\xe4\x97\x8d',
'CjkUnifiedIdeograph45ce': '\xe4\x97\x8e',
'GurmukhiLetterBha': '\xe0\xa8\xad',
'CjkUnifiedIdeograph45cc': '\xe4\x97\x8c',
'CjkUnifiedIdeograph90b1': '\xe9\x82\xb1',
'CjkUnifiedIdeograph45ca': '\xe4\x97\x8a',
'UnicodeCharacter0xeaa9': '\xee\xaa\xa9',
'UnicodeCharacter0xeaa8': '\xee\xaa\xa8',
'LogicalOr': '\xe2\x88\xa8',
'ArabicindicDigitFour': '\xd9\xa4',
'UnicodeCharacter0xeaa1': '\xee\xaa\xa1',
'UnicodeCharacter0xeaa0': '\xee\xaa\xa0',
'UnicodeCharacter0xeaa3': '\xee\xaa\xa3',
'UnicodeCharacter0xeaa2': '\xee\xaa\xa2',
'UnicodeCharacter0xeaa5': '\xee\xaa\xa5',
'UnicodeCharacter0xeaa4': '\xee\xaa\xa4',
'UnicodeCharacter0xeaa7': '\xee\xaa\xa7',
'CjkUnifiedIdeograph7468': '\xe7\x91\xa8',
'RunicLetterMannazManM': '\xe1\x9b\x97',
'YiSyllableNrox': '\xea\x8e\x98',
'CjkCompatibilityIdeographf9fc': '\xef\xa7\xbc',
'UnicodeCharacter0xe04f': '\xee\x81\x8f',
'UnicodeCharacter0xe04e': '\xee\x81\x8e',
'UnicodeCharacter0xe04d': '\xee\x81\x8d',
'DownwardsHarpoonWithBarbRightwards': '\xe2\x87\x82',
'YiSyllableNrop': '\xea\x8e\x9a',
'UnicodeCharacter0xe04a': '\xee\x81\x8a',
'HangulSyllablePig': '\xed\x94\xbd',
'CombiningDoubleVerticalLineBelow': '\xcd\x88',
'KangxiRadicalWoman': '\xe2\xbc\xa5',
'HangulSyllableGgyag': '\xea\xba\x85',
'HangulSyllableGgyad': '\xea\xba\x8b',
'HangulSyllableGgyae': '\xea\xba\xa0',
'HangulSyllableGgyab': '\xea\xba\x95',
'HangulSyllableGgyac': '\xea\xba\x9b',
'KangxiRadicalMorning': '\xe2\xbe\xa0',
'HangulSyllableGgyan': '\xea\xba\x88',
'HangulSyllableGgyal': '\xea\xba\x8c',
'HangulSyllableGgyam': '\xea\xba\x94',
'HangulSyllableGgyaj': '\xea\xba\x9a',
'CopticOldNubianDirectQuestionMark': '\xe2\xb3\xba',
'HangulSyllableGgyah': '\xea\xba\x9f',
'UnicodeCharacter0xeaaf': '\xee\xaa\xaf',
'LatinSmallLetterDezhDigraph': '\xca\xa4',
'HangulSyllableSsyels': '\xec\x8f\x88',
'HangulSyllableGgyat': '\xea\xba\x9d',
'HangulSyllableGgyas': '\xea\xba\x97',
'TagbanwaLetterDa': '\xe1\x9d\xa7',
'CjkUnifiedIdeograph45c9': '\xe4\x97\x89',
'CjkUnifiedIdeograph45c6': '\xe4\x97\x86',
'CjkUnifiedIdeograph45c7': '\xe4\x97\x87',
'CjkUnifiedIdeograph45c4': '\xe4\x97\x84',
'CjkUnifiedIdeograph45c5': '\xe4\x97\x85',
'CjkUnifiedIdeograph45c2': '\xe4\x97\x82',
'CjkUnifiedIdeograph45c3': '\xe4\x97\x83',
'CjkUnifiedIdeograph45c0': '\xe4\x97\x80',
'CjkUnifiedIdeograph45c1': '\xe4\x97\x81',
'CjkUnifiedIdeograph99b3': '\xe9\xa6\xb3',
'HangulSyllableGals': '\xea\xb0\x8c',
'PhagspaLetterKa': '\xea\xa1\x80',
'NewTaiLueDigitThree': '\xe1\xa7\x93',
'UnicodeCharacter0xe049': '\xee\x81\x89',
'UnicodeCharacter0xe048': '\xee\x81\x88',
'UnicodeCharacter0xe047': '\xee\x81\x87',
'CyrillicSmallLetterEWithDiaeresis': '\xd3\xad',
'UnicodeCharacter0xe045': '\xee\x81\x85',
'HangulSyllableSsyelt': '\xec\x8f\x89',
'UnicodeCharacter0xe043': '\xee\x81\x83',
'GlagoliticCapitalLetterIzhitsa': '\xe2\xb0\xab',
'UnicodeCharacter0xe041': '\xee\x81\x81',
'UnicodeCharacter0xe040': '\xee\x81\x80',
'HangulSyllableCeol': '\xec\xb2\xa0',
'CjkUnifiedIdeograph69fc': '\xe6\xa7\xbc',
'CjkUnifiedIdeograph69fb': '\xe6\xa7\xbb',
'MaleWithStrokeSign': '\xe2\x9a\xa6',
'CjkUnifiedIdeograph69ff': '\xe6\xa7\xbf',
'CjkUnifiedIdeograph69fe': '\xe6\xa7\xbe',
'CjkUnifiedIdeograph69fd': '\xe6\xa7\xbd',
'UnicodeCharacter0xe9f6': '\xee\xa7\xb6',
'SmallVeeWithUnderbar': '\xe2\xa9\xa1',
'CjkUnifiedIdeograph99b8': '\xe9\xa6\xb8',
'UnicodeCharacter0xdfbd': '\xed\xbe\xbd',
'CanadianSyllabicsCarrierThu': '\xe1\x97\x9c',
'ArabicLetterHehInitialForm': '\xef\xbb\xab',
'UnicodeCharacter0xef52': '\xee\xbd\x92',
'HangulSyllableDyeobs': '\xeb\x8e\x9e',
'CjkUnifiedIdeograph546c': '\xe5\x91\xac',
'CjkUnifiedIdeograph546b': '\xe5\x91\xab',
'CjkUnifiedIdeograph546a': '\xe5\x91\xaa',
'CjkUnifiedIdeograph546f': '\xe5\x91\xaf',
'CjkUnifiedIdeograph546e': '\xe5\x91\xae',
'CjkUnifiedIdeograph546d': '\xe5\x91\xad',
'CanadianSyllabicsCarrierThe': '\xe1\x97\x9e',
'CanadianSyllabicsCarrierTha': '\xe1\x97\xa1',
'CanadianSyllabicsCarrierTho': '\xe1\x97\x9d',
'HeavyWideheadedRightwardsArrow': '\xe2\x9e\x94',
'UnicodeCharacter0xf59d': '\xef\x96\x9d',
'CanadianSyllabicsCarrierThi': '\xe1\x97\xa0',
'HangulSyllableWalb': '\xec\x99\x8b',
'UnicodeCharacter0xe917': '\xee\xa4\x97',
'HangulSyllableWalg': '\xec\x99\x89',
'HangulSyllableWalh': '\xec\x99\x8f',
'UnicodeCharacter0xdfbc': '\xed\xbe\xbc',
'HangulSyllableWalm': '\xec\x99\x8a',
'HangulSyllableRwip': '\xeb\xa5\x96',
'HangulSyllableWalp': '\xec\x99\x8e',
'CjkUnifiedIdeograph69f9': '\xe6\xa7\xb9',
'CjkUnifiedIdeograph69f8': '\xe6\xa7\xb8',
'HangulSyllableWalt': '\xec\x99\x8d',
'HangulSyllableJjyos': '\xec\xac\xbf',
'CjkUnifiedIdeograph69f3': '\xe6\xa7\xb3',
'CjkUnifiedIdeograph69f2': '\xe6\xa7\xb2',
'CopticFractionOneHalf': '\xe2\xb3\xbd',
'CjkUnifiedIdeograph69f0': '\xe6\xa7\xb0',
'CjkUnifiedIdeograph69f7': '\xe6\xa7\xb7',
'CjkUnifiedIdeograph69f6': '\xe6\xa7\xb6',
'CjkUnifiedIdeograph69f5': '\xe6\xa7\xb5',
'CjkUnifiedIdeograph69f4': '\xe6\xa7\xb4',
'GreekCapitalLetterAlphaWithDasiaAndVariaAndProsgegrammeni': '\xe1\xbe\x8b',
'UnicodeCharacter0xef50': '\xee\xbd\x90',
'CjkRadicalSecondTwo': '\xe2\xba\x83',
'LatinSmallLetterToneTwo': '\xc6\xa8',
'UnicodeCharacter0xf158': '\xef\x85\x98',
'UnionWithMinusSign': '\xe2\xa9\x81',
'HangulSyllableNoenj': '\xeb\x87\x91',
'UnicodeCharacter0xfd49': '\xef\xb5\x89',
'CjkUnifiedIdeograph5469': '\xe5\x91\xa9',
'CjkUnifiedIdeograph5468': '\xe5\x91\xa8',
'HangulSyllableJjyot': '\xec\xad\x85',
'CjkUnifiedIdeograph5463': '\xe5\x91\xa3',
'CjkUnifiedIdeograph5462': '\xe5\x91\xa2',
'CjkUnifiedIdeograph5461': '\xe5\x91\xa1',
'CjkUnifiedIdeograph5460': '\xe5\x91\xa0',
'CjkUnifiedIdeograph5467': '\xe5\x91\xa7',
'CjkUnifiedIdeograph5466': '\xe5\x91\xa6',
'CjkUnifiedIdeograph5465': '\xe5\x91\xa5',
'CjkUnifiedIdeograph5464': '\xe5\x91\xa4',
'NkoDigitTwo': '\xdf\x82',
'CjkCompatibilityIdeographfa3b': '\xef\xa8\xbb',
'CjkUnifiedIdeograph628b': '\xe6\x8a\x8b',
'CjkUnifiedIdeograph628c': '\xe6\x8a\x8c',
'GeorgianCapitalLetterPhar': '\xe1\x82\xb4',
'CjkUnifiedIdeograph628a': '\xe6\x8a\x8a',
'CjkUnifiedIdeograph628f': '\xe6\x8a\x8f',
'CjkUnifiedIdeograph628d': '\xe6\x8a\x8d',
'CjkUnifiedIdeograph628e': '\xe6\x8a\x8e',
'CjkRadicalHand': '\xe2\xba\x98',
'TamilAuLengthMark': '\xe0\xaf\x97',
'WordJoiner': '\xe2\x81\xa0',
'HangulSyllableCeom': '\xec\xb2\xa8',
'HangulSyllableYong': '\xec\x9a\xa9',
'HangulSyllableNoenh': '\xeb\x87\x92',
'EthiopicSyllableXyaa': '\xe2\xb7\x93',
'OlChikiDigitThree': '\xe1\xb1\x93',
'HangulSyllableYonh': '\xec\x9a\x9a',
'UnicodeCharacter0xdf28': '\xed\xbc\xa8',
'SylotiNagriLetterBo': '\xea\xa0\x9b',
'HangulSyllableCwigg': '\xec\xb7\xaa',
'HangulSyllableMagg': '\xeb\xa7\x8a',
'UnicodeCharacter0xed74': '\xee\xb5\xb4',
'HangulSyllablePwess': '\xed\x93\x84',
'UnicodeCharacter0xef57': '\xee\xbd\x97',
'ArabicLigatureSeenWithHahInitialForm': '\xef\xb2\xae',
'CjkUnifiedIdeograph9faa': '\xe9\xbe\xaa',
'HangulSyllableJeul': '\xec\xa6\x90',
'HangulSyllableJeun': '\xec\xa6\x8c',
'CjkUnifiedIdeograph8b8b': '\xe8\xae\x8b',
'TeluguLetterTta': '\xe0\xb0\x9f',
'HangulSyllableJeuj': '\xec\xa6\x9e',
'HangulSyllableJeud': '\xec\xa6\x8f',
'LessthanWithCircleInside': '\xe2\xa9\xb9',
'NewTaiLueLetterLowBa': '\xe1\xa6\xa5',
'DingbatNegativeCircledSansserifDigitThree': '\xe2\x9e\x8c',
'HangulSyllableJeuc': '\xec\xa6\x9f',
'HangulSyllableJeub': '\xec\xa6\x99',
'CombiningGreekPerispomeni': '\xcd\x82',
'CjkUnifiedIdeograph8583': '\xe8\x96\x83',
'CjkUnifiedIdeograph6288': '\xe6\x8a\x88',
'CjkUnifiedIdeograph6289': '\xe6\x8a\x89',
'CjkUnifiedIdeograph6282': '\xe6\x8a\x82',
'CjkUnifiedIdeograph6283': '\xe6\x8a\x83',
'CjkUnifiedIdeograph6280': '\xe6\x8a\x80',
'CjkUnifiedIdeograph6281': '\xe6\x8a\x81',
'CjkUnifiedIdeograph6286': '\xe6\x8a\x86',
'CjkUnifiedIdeograph6287': '\xe6\x8a\x87',
'CjkUnifiedIdeograph6284': '\xe6\x8a\x84',
'CjkRadicalCsimplifiedDragon': '\xe2\xbb\xb0',
'HangulSyllableJjwael': '\xec\xab\xbc',
'HangulSyllableJjwaem': '\xec\xac\x84',
'HangulSyllableJjwaen': '\xec\xab\xb8',
'CjkUnifiedIdeograph8580': '\xe8\x96\x80',
'HangulSyllableJjwaeh': '\xec\xac\x8f',
'ArabicLigatureShaddaWithFathaMedialForm': '\xef\xb3\xb2',
'HangulSyllableJjwaej': '\xec\xac\x8a',
'HangulSyllableJjwaek': '\xec\xac\x8c',
'HangulSyllableJjwaed': '\xec\xab\xbb',
'HangulSyllableMut': '\xeb\xad\x8d',
'CjkUnifiedIdeograph4e1c': '\xe4\xb8\x9c',
'HangulSyllableMus': '\xeb\xad\x87',
'HangulSyllableMup': '\xeb\xad\x8e',
'HangulSyllableJjwaec': '\xec\xac\x8b',
'HangulSyllableMun': '\xeb\xac\xb8',
'UpDownWhiteArrow': '\xe2\x87\xb3',
'HangulSyllableMul': '\xeb\xac\xbc',
'HangulSyllableMum': '\xeb\xad\x84',
'HangulSyllableMuj': '\xeb\xad\x8a',
'CjkUnifiedIdeograph8585': '\xe8\x96\x85',
'HangulSyllableMuh': '\xeb\xad\x8f',
'UnicodeCharacter0xef55': '\xee\xbd\x95',
'HangulSyllableJjwaet': '\xec\xac\x8d',
'HangulSyllableMug': '\xeb\xac\xb5',
'HangulSyllableMud': '\xeb\xac\xbb',
'HangulSyllableMub': '\xeb\xad\x85',
'HangulSyllableMuc': '\xeb\xad\x8b',
'HangulSyllableJjwaes': '\xec\xac\x87',
'UnicodeCharacter0xf077': '\xef\x81\xb7',
'HangulSyllableDdigg': '\xeb\x9d\xa2',
'YiSyllableYox': '\xea\x91\xbe',
'YiSyllableShux': '\xea\x8e\xbb',
'HangulSyllableNyebs': '\xeb\x85\xae',
'UnicodeCharacter0xe6e2': '\xee\x9b\xa2',
'LowerBladeScissors': '\xe2\x9c\x83',
'YiSyllableShut': '\xea\x8e\xba',
'YiSyllableYop': '\xea\x92\x80',
'DevanagariLetterRra': '\xe0\xa4\xb1',
'UnicodeCharacter0xe6e7': '\xee\x9b\xa7',
'YiSyllableShup': '\xea\x8e\xbd',
'YiSyllableYot': '\xea\x91\xbd',
'UnicodeCharacter0x81a': '\xe0\xa0\x9a',
'UnicodeCharacter0x81b': '\xe0\xa0\x9b',
'UnicodeCharacter0x81c': '\xe0\xa0\x9c',
'UnicodeCharacter0x81d': '\xe0\xa0\x9d',
'UnicodeCharacter0x81e': '\xe0\xa0\x9e',
'UnicodeCharacter0x81f': '\xe0\xa0\x9f',
'CjkUnifiedIdeograph55ff': '\xe5\x97\xbf',
'CjkUnifiedIdeograph55fd': '\xe5\x97\xbd',
'CjkUnifiedIdeograph55fe': '\xe5\x97\xbe',
'CjkUnifiedIdeograph55fb': '\xe5\x97\xbb',
'CjkUnifiedIdeograph55fc': '\xe5\x97\xbc',
'CjkUnifiedIdeograph55fa': '\xe5\x97\xba',
'CjkUnifiedIdeograph4411': '\xe4\x90\x91',
'CjkRadicalCliff': '\xe2\xba\x81',
'CjkUnifiedIdeograph4413': '\xe4\x90\x93',
'CjkUnifiedIdeograph4412': '\xe4\x90\x92',
'CjkUnifiedIdeograph4415': '\xe4\x90\x95',
'EthiopicNumberHundred': '\xe1\x8d\xbb',
'CjkUnifiedIdeograph4417': '\xe4\x90\x97',
'CjkUnifiedIdeograph4416': '\xe4\x90\x96',
'CjkUnifiedIdeograph4419': '\xe4\x90\x99',
'CjkUnifiedIdeograph4418': '\xe4\x90\x98',
'CjkUnifiedIdeograph382c': '\xe3\xa0\xac',
'CjkUnifiedIdeograph382b': '\xe3\xa0\xab',
'CjkUnifiedIdeograph382e': '\xe3\xa0\xae',
'CjkUnifiedIdeograph382d': '\xe3\xa0\xad',
'YiSyllableNze': '\xea\x8b\xbd',
'CjkUnifiedIdeograph382f': '\xe3\xa0\xaf',
'YiSyllableNzy': '\xea\x8c\x85',
'VaiSyllableNg': '\xea\x98\x8b',
'HebrewPointQamats': '\xd6\xb8',
'YiSyllableNzu': '\xea\x8b\xbf',
'UnicodeCharacter0x810': '\xe0\xa0\x90',
'UnicodeCharacter0x811': '\xe0\xa0\x91',
'UnicodeCharacter0x812': '\xe0\xa0\x92',
'UnicodeCharacter0x813': '\xe0\xa0\x93',
'UnicodeCharacter0x814': '\xe0\xa0\x94',
'UnicodeCharacter0x815': '\xe0\xa0\x95',
'UnicodeCharacter0x816': '\xe0\xa0\x96',
'UnicodeCharacter0x817': '\xe0\xa0\x97',
'UnicodeCharacter0x818': '\xe0\xa0\x98',
'CjkUnifiedIdeograph55f7': '\xe5\x97\xb7',
'CjkUnifiedIdeograph55f4': '\xe5\x97\xb4',
'CjkUnifiedIdeograph55f5': '\xe5\x97\xb5',
'CjkUnifiedIdeograph55f2': '\xe5\x97\xb2',
'CjkUnifiedIdeograph55f3': '\xe5\x97\xb3',
'CjkUnifiedIdeograph55f0': '\xe5\x97\xb0',
'CjkUnifiedIdeograph55f1': '\xe5\x97\xb1',
'EthiopicSyllableFa': '\xe1\x8d\x88',
'DevanagariLetterNga': '\xe0\xa4\x99',
'EthiopicSyllableFe': '\xe1\x8d\x8d',
'UnicodeCharacter0xe6eb': '\xee\x9b\xab',
'HangulSyllableMoegg': '\xeb\xab\xbe',
'EthiopicSyllableFi': '\xe1\x8d\x8a',
'UnicodeCharacter0xe6ea': '\xee\x9b\xaa',
'EthiopicSyllableFo': '\xe1\x8d\x8e',
'CjkUnifiedIdeograph4ce9': '\xe4\xb3\xa9',
'UnicodeCharacter0xe6ed': '\xee\x9b\xad',
'CjkUnifiedIdeograph84c6': '\xe8\x93\x86',
'UnicodeCharacter0x23f3': '\xe2\x8f\xb3',
'IdeographicTelegraphSymbolForDayEight': '\xe3\x8f\xa7',
'HangulSyllableCeum': '\xec\xb8\xb0',
'CjkUnifiedIdeograph84c5': '\xe8\x93\x85',
'CjkUnifiedIdeograph5ac8': '\xe5\xab\x88',
'CjkUnifiedIdeograph84c4': '\xe8\x93\x84',
'UnicodeCharacter0xe4cc': '\xee\x93\x8c',
'CjkUnifiedIdeograph441a': '\xe4\x90\x9a',
'CjkUnifiedIdeograph3828': '\xe3\xa0\xa8',
'CjkUnifiedIdeograph441c': '\xe4\x90\x9c',
'CjkUnifiedIdeograph441b': '\xe4\x90\x9b',
'CjkUnifiedIdeograph441e': '\xe4\x90\x9e',
'CjkUnifiedIdeograph441d': '\xe4\x90\x9d',
'CjkCompatibilityIdeographf9f1': '\xef\xa7\xb1',
'CjkUnifiedIdeograph441f': '\xe4\x90\x9f',
'CjkUnifiedIdeograph3821': '\xe3\xa0\xa1',
'CjkUnifiedIdeograph3820': '\xe3\xa0\xa0',
'CjkUnifiedIdeograph3823': '\xe3\xa0\xa3',
'SyriacSupralinearFullStop': '\xdc\x81',
'CjkUnifiedIdeograph3825': '\xe3\xa0\xa5',
'CjkUnifiedIdeograph3824': '\xe3\xa0\xa4',
'CjkUnifiedIdeograph3827': '\xe3\xa0\xa7',
'CjkUnifiedIdeograph3826': '\xe3\xa0\xa6',
'HangulSyllableCeub': '\xec\xb8\xb1',
'LimbuLetterCha': '\xe1\xa4\x87',
'LeftWhiteTortoiseShellBracket': '\xe3\x80\x98',
'MongolianManchuComma': '\xe1\xa0\x88',
'UnicodeCharacter0xdfb8': '\xed\xbe\xb8',
'HangulSyllableBbeugs': '\xec\x81\x9b',
'CjkCompatibilityIdeographfa36': '\xef\xa8\xb6',
'CjkUnifiedIdeograph6c15': '\xe6\xb0\x95',
'CanadianSyllabicsSouthslaveyKah': '\xe1\x92\x88',
'CjkCompatibilityIdeographfa37': '\xef\xa8\xb7',
'SymbolForHorizontalTabulation': '\xe2\x90\x89',
'HangulSyllableCeh': '\xec\xb3\x8f',
'CjkUnifiedIdeograph6c17': '\xe6\xb0\x97',
'CjkUnifiedIdeograph5ac3': '\xe5\xab\x83',
'HangulSyllableCeuc': '\xec\xb8\xb7',
'CjkUnifiedIdeograph5ac2': '\xe5\xab\x82',
'UnicodeCharacter0xf071': '\xef\x81\xb1',
'WhiteLeftpointingPointer': '\xe2\x97\x85',
'CjkUnifiedIdeograph5ac6': '\xe5\xab\x86',
'UnicodeCharacter0xf248': '\xef\x89\x88',
'HangulSyllableCaess': '\xec\xb1\x98',
'CjkCompatibilityIdeographfa39': '\xef\xa8\xb9',
'CopticSmallLetterOldNubianShima': '\xe2\xb3\x9d',
'UnicodeCharacter0xf070': '\xef\x81\xb0',
'BalineseLetterIkaraTedung': '\xe1\xac\x88',
'BalineseLetterRaRepaTedung': '\xe1\xac\x8c',
'HangulSyllableTeuss': '\xed\x8b\x8c',
'GurmukhiVowelSignEe': '\xe0\xa9\x87',
'CjkUnifiedIdeograph7f5c': '\xe7\xbd\x9c',
'HangulSyllableGalb': '\xea\xb0\x8b',
'HangulLetterRieulkiyeok': '\xe3\x84\xba',
'HangulSyllableGgunj': '\xea\xbe\xbd',
'HangulSyllableGgunh': '\xea\xbe\xbe',
'HangulSyllableGgung': '\xea\xbf\x8d',
'CjkUnifiedIdeograph907f': '\xe9\x81\xbf',
'SixteenPointedAsterisk': '\xe2\x9c\xba',
'HangulSyllablePyaebs': '\xed\x8d\xb2',
'CjkUnifiedIdeograph9261': '\xe9\x89\xa1',
'CjkUnifiedIdeograph44cf': '\xe4\x93\x8f',
'CjkUnifiedIdeograph72cd': '\xe7\x8b\x8d',
'CjkUnifiedIdeograph72ce': '\xe7\x8b\x8e',
'CjkUnifiedIdeograph72cf': '\xe7\x8b\x8f',
'HangulSyllableUbs': '\xec\x9b\x82',
'CjkUnifiedIdeograph44ce': '\xe4\x93\x8e',
'CjkUnifiedIdeograph72cb': '\xe7\x8b\x8b',
'CjkUnifiedIdeograph72cc': '\xe7\x8b\x8c',
'UnicodeCharacter0xaadb': '\xea\xab\x9b',
'CjkUnifiedIdeograph3d9a': '\xe3\xb6\x9a',
'CjkUnifiedIdeograph3d9b': '\xe3\xb6\x9b',
'CjkUnifiedIdeograph3d9c': '\xe3\xb6\x9c',
'CjkUnifiedIdeograph3d9d': '\xe3\xb6\x9d',
'CjkUnifiedIdeograph3d9e': '\xe3\xb6\x9e',
'CjkUnifiedIdeograph3d9f': '\xe3\xb6\x9f',
'LaoLetterYo': '\xe0\xba\xa2',
'UnicodeCharacter0x1cae': '\xe1\xb2\xae',
'HangulSyllablePwih': '\xed\x93\xa7',
'HangulSyllableCek': '\xec\xb3\x8c',
'ParenthesizedHangulThieuth': '\xe3\x88\x8b',
'HangulSyllablePwij': '\xed\x93\xa2',
'CjkUnifiedIdeograph6709': '\xe6\x9c\x89',
'CjkUnifiedIdeograph6708': '\xe6\x9c\x88',
'CjkUnifiedIdeograph6707': '\xe6\x9c\x87',
'CjkUnifiedIdeograph6706': '\xe6\x9c\x86',
'CjkUnifiedIdeograph6705': '\xe6\x9c\x85',
'CjkUnifiedIdeograph6704': '\xe6\x9c\x84',
'CjkUnifiedIdeograph6703': '\xe6\x9c\x83',
'CjkUnifiedIdeograph6702': '\xe6\x9c\x82',
'CjkUnifiedIdeograph6701': '\xe6\x9c\x81',
'CjkUnifiedIdeograph6700': '\xe6\x9c\x80',
'CjkUnifiedIdeograph3d98': '\xe3\xb6\x98',
'CjkUnifiedIdeograph3d99': '\xe3\xb6\x99',
'HangulSyllableCeg': '\xec\xb2\xb5',
'MeasuredAngleWithOpenArmEndingInArrowPointingRightAndDown': '\xe2\xa6\xae',
'CjkUnifiedIdeograph3d90': '\xe3\xb6\x90',
'CjkUnifiedIdeograph3d91': '\xe3\xb6\x91',
'CjkUnifiedIdeograph3d92': '\xe3\xb6\x92',
'CjkUnifiedIdeograph3d93': '\xe3\xb6\x93',
'CjkUnifiedIdeograph3d94': '\xe3\xb6\x94',
'CjkUnifiedIdeograph3d95': '\xe3\xb6\x95',
'CjkUnifiedIdeograph3d96': '\xe3\xb6\x96',
'CjkUnifiedIdeograph3d97': '\xe3\xb6\x97',
'CjkUnifiedIdeograph72c8': '\xe7\x8b\x88',
'CjkUnifiedIdeograph72c9': '\xe7\x8b\x89',
'CjkUnifiedIdeograph72c4': '\xe7\x8b\x84',
'CjkUnifiedIdeograph72c5': '\xe7\x8b\x85',
'CjkUnifiedIdeograph72c6': '\xe7\x8b\x86',
'CjkUnifiedIdeograph72c7': '\xe7\x8b\x87',
'CjkUnifiedIdeograph72c0': '\xe7\x8b\x80',
'CjkUnifiedIdeograph72c1': '\xe7\x8b\x81',
'CjkUnifiedIdeograph72c2': '\xe7\x8b\x82',
'CjkUnifiedIdeograph72c3': '\xe7\x8b\x83',
'CjkUnifiedIdeograph670f': '\xe6\x9c\x8f',
'CjkUnifiedIdeograph670e': '\xe6\x9c\x8e',
'CjkUnifiedIdeograph670d': '\xe6\x9c\x8d',
'GreekCapitalLetterIotaWithOxia': '\xe1\xbf\x9b',
'CjkUnifiedIdeograph670b': '\xe6\x9c\x8b',
'RunicArlaugSymbol': '\xe1\x9b\xae',
'KayahLiDigitOne': '\xea\xa4\x81',
'CyrillicCapitalLetterOmega': '\xd1\xa0',
'EthiopicSyllablePho': '\xe1\x8c\xb6',
'BalineseLetterSa': '\xe1\xac\xb2',
'EqualsSignAboveLeftwardsArrow': '\xe2\xad\x80',
'HangulSyllableGgulm': '\xea\xbf\x82',
'EthiopicSyllablePhe': '\xe1\x8c\xb5',
'EthiopicSyllablePha': '\xe1\x8c\xb0',
'CjkUnifiedIdeograph3a42': '\xe3\xa9\x82',
'CjkUnifiedIdeograph3a43': '\xe3\xa9\x83',
'CjkUnifiedIdeograph3a40': '\xe3\xa9\x80',
'CjkUnifiedIdeograph3a41': '\xe3\xa9\x81',
'CjkUnifiedIdeograph3a46': '\xe3\xa9\x86',
'UnicodeCharacter0xea4': '\xe0\xba\xa4',
'CjkUnifiedIdeograph3a44': '\xe3\xa9\x84',
'CjkUnifiedIdeograph3a45': '\xe3\xa9\x85',
'UnicodeCharacter0xea9': '\xe0\xba\xa9',
'UnicodeCharacter0xea8': '\xe0\xba\xa8',
'CjkUnifiedIdeograph3a48': '\xe3\xa9\x88',
'EthiopicSyllablePhu': '\xe1\x8c\xb1',
'SinhalaVowelSignKombuvaHaaGayanukitta': '\xe0\xb7\x9e',
'CjkRadicalJsimplifiedTooth': '\xe2\xbb\xad',
'UnicodeCharacter0xaadc': '\xea\xab\x9c',
'HangulSyllableSoegs': '\xec\x87\xa3',
'MyanmarConsonantSignMonMedialMa': '\xe1\x81\x9f',
'MyanmarVowelSignAi': '\xe1\x80\xb2',
'UnicodeCharacter0xf7fb': '\xef\x9f\xbb',
'HangulSyllableCes': '\xec\xb3\x87',
'MyanmarVowelSignAa': '\xe1\x80\xac',
'CjkUnifiedIdeograph3a4b': '\xe3\xa9\x8b',
'CjkUnifiedIdeograph3a4c': '\xe3\xa9\x8c',
'UnicodeCharacter0xeac': '\xe0\xba\xac',
'CjkUnifiedIdeograph3a4a': '\xe3\xa9\x8a',
'CjkUnifiedIdeograph3a4f': '\xe3\xa9\x8f',
'CjkUnifiedIdeograph3a4d': '\xe3\xa9\x8d',
'CjkUnifiedIdeograph3a4e': '\xe3\xa9\x8e',
'KayahLiDigitZero': '\xea\xa4\x80',
'UnicodeCharacter0xde5f': '\xed\xb9\x9f',
'CjkUnifiedIdeograph4e13': '\xe4\xb8\x93',
'ArabicHamzaAbove': '\xd9\x94',
'HangulSyllableGgeul': '\xeb\x81\x8c',
'UnicodeCharacter0xe72d': '\xee\x9c\xad',
'BlackRightpointingPentagon': '\xe2\xad\x93',
'BraillePatternDots124567': '\xe2\xa1\xbb',
'BraillePatternDots124568': '\xe2\xa2\xbb',
'HangulSyllableJyoj': '\xec\xa3\xb6',
'HangulSyllableGobs': '\xea\xb3\xb2',
'ModifierLetterSmallDelta': '\xe1\xb5\x9f',
'HangulSyllableGgeum': '\xeb\x81\x94',
'UnicodeCharacter0x2b99': '\xe2\xae\x99',
'HalfwidthHangulLetterWe': '\xef\xbf\x95',
'EthiopicTonalMarkShortRikrik': '\xe1\x8e\x93',
'HangulSyllableDeunh': '\xeb\x93\xa2',
'SinhalaLetterUyanna': '\xe0\xb6\x8b',
'HangulSyllableMags': '\xeb\xa7\x8b',
'HangulSyllableDeung': '\xeb\x93\xb1',
'CyrillicCapitalLigatureTeTse': '\xd2\xb4',
'KannadaLetterVocalicRr': '\xe0\xb3\xa0',
'UnicodeCharacter0xd83b': '\xed\xa0\xbb',
'ArabicLigatureSeenWithMeemWithHahInitialForm': '\xef\xb5\xa0',
'CjkUnifiedIdeograph7f49': '\xe7\xbd\x89',
'CjkUnifiedIdeograph7f48': '\xe7\xbd\x88',
'HangulSyllablePweolb': '\xed\x92\x9f',
'CjkUnifiedIdeograph73c9': '\xe7\x8f\x89',
'UnicodeCharacter0xdc8': '\xe0\xb7\x88',
'HangulSyllableGgeuk': '\xeb\x81\x9c',
'CjkUnifiedIdeograph73c5': '\xe7\x8f\x85',
'CjkUnifiedIdeograph73c4': '\xe7\x8f\x84',
'CjkUnifiedIdeograph73c7': '\xe7\x8f\x87',
'CjkUnifiedIdeograph73c6': '\xe7\x8f\x86',
'CjkUnifiedIdeograph73c1': '\xe7\x8f\x81',
'CjkUnifiedIdeograph73c0': '\xe7\x8f\x80',
'LatinSmallLetterNWithLineBelow': '\xe1\xb9\x89',
'CjkUnifiedIdeograph73c2': '\xe7\x8f\x82',
'HangulSyllablePweolp': '\xed\x92\xa2',
'HangulSyllableCyegs': '\xec\xb3\xaf',
'HangulSyllableCoels': '\xec\xb5\xa8',
'HangulSyllableTeugs': '\xed\x8a\xbb',
'HangulSyllableRils': '\xeb\xa6\xb8',
'HangulSyllableRilp': '\xeb\xa6\xba',
'HangulSyllableRilt': '\xeb\xa6\xb9',
'LatinSmallLetterAWithBreveAndTilde': '\xe1\xba\xb5',
'HangulSyllableKobs': '\xec\xbd\xa6',
'HangulSyllableRilb': '\xeb\xa6\xb7',
'MediumRightParenthesisOrnament': '\xe2\x9d\xa9',
'KangxiRadicalFragrant': '\xe2\xbe\xb9',
'HangulSyllableRilg': '\xeb\xa6\xb5',
'ArabicLigatureSallaUsedAsKoranicStopSignIsolatedForm': '\xef\xb7\xb0',
'HangulSyllableDoenh': '\xeb\x90\x9e',
'HangulSyllableRilh': '\xeb\xa6\xbb',
'HangulSyllableRilm': '\xeb\xa6\xb6',
'HangulSyllableDoenj': '\xeb\x90\x9d',
'UnicodeCharacter0xe72e': '\xee\x9c\xae',
'HangulSyllableBbwig': '\xec\x80\xa1',
'UnicodeCharacter0xe046': '\xee\x81\x86',
'HebrewPointHolamHaserForVav': '\xd6\xba',
'MongolianLetterManchuAliGaliDdha': '\xe1\xa2\x9f',
'CjkUnifiedIdeograph73ce': '\xe7\x8f\x8e',
'CjkUnifiedIdeograph73cd': '\xe7\x8f\x8d',
'CjkUnifiedIdeograph7f4c': '\xe7\xbd\x8c',
'CjkUnifiedIdeograph73cf': '\xe7\x8f\x8f',
'CjkUnifiedIdeograph73ca': '\xe7\x8f\x8a',
'CjkUnifiedIdeograph7f4d': '\xe7\xbd\x8d',
'CjkUnifiedIdeograph73cc': '\xe7\x8f\x8c',
'CjkUnifiedIdeograph73cb': '\xe7\x8f\x8b',
'CjkUnifiedIdeograph5c8f': '\xe5\xb2\x8f',
'CjkUnifiedIdeograph5c8d': '\xe5\xb2\x8d',
'CjkUnifiedIdeograph5c8e': '\xe5\xb2\x8e',
'CjkUnifiedIdeograph5c8b': '\xe5\xb2\x8b',
'CjkUnifiedIdeograph5c8c': '\xe5\xb2\x8c',
'CjkUnifiedIdeograph5c8a': '\xe5\xb2\x8a',
'CjkUnifiedIdeograph5f54': '\xe5\xbd\x94',
'DentistrySymbolLightVerticalAndTopLeft': '\xe2\x8f\x8b',
'CjkUnifiedIdeograph5f56': '\xe5\xbd\x96',
'CjkUnifiedIdeograph5f57': '\xe5\xbd\x97',
'CjkUnifiedIdeograph5f50': '\xe5\xbd\x90',
'CjkUnifiedIdeograph5f51': '\xe5\xbd\x91',
'CjkUnifiedIdeograph5f52': '\xe5\xbd\x92',
'CjkUnifiedIdeograph5f53': '\xe5\xbd\x93',
'UnicodeCharacter0xe6ef': '\xee\x9b\xaf',
'UnicodeCharacter0xde5a': '\xed\xb9\x9a',
'HangulSyllableGgeug': '\xeb\x81\x85',
'HangulSyllableKeobs': '\xec\xbb\xb6',
'GreekSmallLetterXi': '\xce\xbe',
'CjkUnifiedIdeograph8d98': '\xe8\xb6\x98',
'CombiningLigatureRightHalf': '\xef\xb8\xa1',
'HangulSyllableBbwih': '\xec\x80\xbb',
'BalineseVowelSignTalingTedung': '\xe1\xad\x80',
'CjkUnifiedIdeograph9531': '\xe9\x94\xb1',
'GreekSmallLetterEtaWithPsiliAndOxia': '\xe1\xbc\xa4',
'HangulSyllableGgeud': '\xeb\x81\x8b',
'UnicodeCharacter0xec64': '\xee\xb1\xa4',
'YiSyllableHie': '\xea\x89\xb0',
'HangulSyllableJjonj': '\xec\xab\x81',
'HangulSyllablePyals': '\xed\x8d\x90',
'HangulSyllableMwibs': '\xeb\xae\x9a',
'CjkUnifiedIdeograph5c86': '\xe5\xb2\x86',
'CjkUnifiedIdeograph5c87': '\xe5\xb2\x87',
'CjkUnifiedIdeograph5c84': '\xe5\xb2\x84',
'CjkUnifiedIdeograph5c85': '\xe5\xb2\x85',
'CjkUnifiedIdeograph5c82': '\xe5\xb2\x82',
'CjkUnifiedIdeograph5c83': '\xe5\xb2\x83',
'CjkUnifiedIdeograph5c80': '\xe5\xb2\x80',
'CjkUnifiedIdeograph5c81': '\xe5\xb2\x81',
'CjkUnifiedIdeograph5f5d': '\xe5\xbd\x9d',
'CjkUnifiedIdeograph5f5e': '\xe5\xbd\x9e',
'MongolianLetterAliGaliTtha': '\xe1\xa2\x8d',
'CjkUnifiedIdeograph5f5a': '\xe5\xbd\x9a',
'CjkUnifiedIdeograph5c88': '\xe5\xb2\x88',
'UnicodeCharacter0x2d2a': '\xe2\xb4\xaa',
'UnicodeCharacter0x1ca3': '\xe1\xb2\xa3',
'CjkCompatibilityIdeographf955': '\xef\xa5\x95',
'UnicodeCharacter0x1ca2': '\xe1\xb2\xa2',
'HangulSyllableGgyugs': '\xeb\x80\xab',
'UnicodeCharacter0x1ca1': '\xe1\xb2\xa1',
'RunicLetterShorttwigyr': '\xe1\x9b\xa7',
'UnicodeCharacter0x1ca0': '\xe1\xb2\xa0',
'CjkUnifiedIdeograph9533': '\xe9\x94\xb3',
'UnicodeCharacter0x1ca7': '\xe1\xb2\xa7',
'CjkStrokeBxg': '\xe3\x87\x83',
'LatinSmallLetterBarredO': '\xc9\xb5',
'UnicodeCharacter0x1ca6': '\xe1\xb2\xa6',
'UnicodeCharacter0x1ca5': '\xe1\xb2\xa5',
'UnicodeCharacter0x1ca4': '\xe1\xb2\xa4',
'CjkRadicalMeat': '\xe2\xba\xbc',
'ArabicLetterHamzaIsolatedForm': '\xef\xba\x80',
'HangulSyllableGgeuc': '\xeb\x81\x9b',
'UnicodeCharacter0xf472': '\xef\x91\xb2',
'HangulChoseongPhieuph': '\xe1\x84\x91',
'LatinSmallLetterVWithPalatalHook': '\xe1\xb6\x8c',
'CjkUnifiedIdeograph9535': '\xe9\x94\xb5',
'CjkRadicalLongOne': '\xe2\xbb\x91',
'CjkUnifiedIdeograph9d0e': '\xe9\xb4\x8e',
'CjkUnifiedIdeograph9d0d': '\xe9\xb4\x8d',
'SquareWithVerticalFill': '\xe2\x96\xa5',
'CjkUnifiedIdeograph9d0f': '\xe9\xb4\x8f',
'CjkUnifiedIdeograph9d0a': '\xe9\xb4\x8a',
'CjkUnifiedIdeograph9d0c': '\xe9\xb4\x8c',
'NkoLetterKa': '\xdf\x9e',
'HorizontalScanLine1': '\xe2\x8e\xba',
'HorizontalScanLine3': '\xe2\x8e\xbb',
'OriyaLetterRra': '\xe0\xad\x9c',
'HorizontalScanLine7': '\xe2\x8e\xbc',
'HorizontalScanLine9': '\xe2\x8e\xbd',
'HangulSyllableTyaelm': '\xed\x84\x9e',
'UnicodeCharacter0xf470': '\xef\x91\xb0',
'HiraganaLetterDa': '\xe3\x81\xa0',
'HiraganaLetterDe': '\xe3\x81\xa7',
'CjkUnifiedIdeograph9d08': '\xe9\xb4\x88',
'HangulSyllableNeonh': '\xeb\x84\x8e',
'TaiLeLetterKha': '\xe1\xa5\xa0',
'HiraganaLetterDi': '\xe3\x81\xa2',
'CjkUnifiedIdeograph9d04': '\xe9\xb4\x84',
'CjkUnifiedIdeograph9d07': '\xe9\xb4\x87',
'CjkUnifiedIdeograph9d06': '\xe9\xb4\x86',
'CjkUnifiedIdeograph9d01': '\xe9\xb4\x81',
'CjkUnifiedIdeograph9d00': '\xe9\xb4\x80',
'HiraganaLetterDo': '\xe3\x81\xa9',
'CjkUnifiedIdeograph9d02': '\xe9\xb4\x82',
'BalineseLetterSaSapa': '\xe1\xac\xb1',
'HangulSyllableTyaels': '\xed\x84\xa0',
'HiraganaLetterDu': '\xe3\x81\xa5',
'CjkUnifiedIdeograph6727': '\xe6\x9c\xa7',
'HangulSyllableGgyoss': '\xea\xbe\xb0',
'HangulSyllableKyolp': '\xec\xbf\x92',
'HangulSyllableHom': '\xed\x99\x88',
'UnicodeCharacter0xe533': '\xee\x94\xb3',
'HangulSyllableJjeulg': '\xec\xaf\x9d',
'HangulSyllableKyols': '\xec\xbf\x90',
'CjkUnifiedIdeograph816e': '\xe8\x85\xae',
'CjkUnifiedIdeograph816d': '\xe8\x85\xad',
'CjkUnifiedIdeograph816f': '\xe8\x85\xaf',
'CjkUnifiedIdeograph816a': '\xe8\x85\xaa',
'CjkUnifiedIdeograph816c': '\xe8\x85\xac',
'CjkUnifiedIdeograph816b': '\xe8\x85\xab',
'HangulSyllableTubs': '\xed\x88\xbe',
'UnicodeCharacter0xf2f0': '\xef\x8b\xb0',
'HangulSyllablePweols': '\xed\x92\xa0',
'HangulSyllableKyolt': '\xec\xbf\x91',
'HangulSyllableGgal': '\xea\xb9\x94',
'CopticSmallLetterShima': '\xcf\xad',
'HangulSyllableGgam': '\xea\xb9\x9c',
'UnicodeCharacter0xf2f6': '\xef\x8b\xb6',
'EthiopicSyllableDdwa': '\xe1\x8b\xbf',
'HangulSyllableGgan': '\xea\xb9\x90',
'UnicodeCharacter0xf2f7': '\xef\x8b\xb7',
'KayahLiLetterOo': '\xea\xa4\xa5',
'UnicodeCharacter0xf2f4': '\xef\x8b\xb4',
'KayahLiVowelEe': '\xea\xa4\xa9',
'KayahLiLetterOe': '\xea\xa4\xa3',
'HangulSyllableGgah': '\xea\xb9\xa7',
'UnicodeCharacter0xf2f5': '\xef\x8b\xb5',
'CjkUnifiedIdeograph8165': '\xe8\x85\xa5',
'CjkUnifiedIdeograph8164': '\xe8\x85\xa4',
'CjkUnifiedIdeograph8167': '\xe8\x85\xa7',
'CjkUnifiedIdeograph8166': '\xe8\x85\xa6',
'CjkUnifiedIdeograph8161': '\xe8\x85\xa1',
'CjkUnifiedIdeograph8160': '\xe8\x85\xa0',
'CopticCapitalLetterTau': '\xe2\xb2\xa6',
'CjkUnifiedIdeograph8162': '\xe8\x85\xa2',
'BalineseVowelSignTedung': '\xe1\xac\xb5',
'HangulSyllableGgak': '\xea\xb9\xa4',
'CjkUnifiedIdeograph8169': '\xe8\x85\xa9',
'CjkUnifiedIdeograph8168': '\xe8\x85\xa8',
'TibetanSubjoinedLetterYa': '\xe0\xbe\xb1',
'OpenCentreAsterisk': '\xe2\x9c\xb2',
'HangulSyllableDdweogg': '\xeb\x9a\xba',
'Homothetic': '\xe2\x88\xbb',
'HangulSyllableGyunh': '\xea\xb7\xa2',
'HangulSyllableGyunj': '\xea\xb7\xa1',
'HangulSyllableGyung': '\xea\xb7\xb1',
'SmallHyphenminus': '\xef\xb9\xa3',
'ArabicLigatureMeemWithHahWithMeemInitialForm': '\xef\xb6\x8a',
'KayahLiLetterHa': '\xea\xa4\x9f',
'DoubleCircledDigitFive': '\xe2\x93\xb9',
'CjkUnifiedIdeograph5708': '\xe5\x9c\x88',
'CjkUnifiedIdeograph5709': '\xe5\x9c\x89',
'CjkUnifiedIdeograph5702': '\xe5\x9c\x82',
'CjkUnifiedIdeograph5703': '\xe5\x9c\x83',
'CjkUnifiedIdeograph5700': '\xe5\x9c\x80',
'CjkUnifiedIdeograph5701': '\xe5\x9c\x81',
'CjkUnifiedIdeograph5706': '\xe5\x9c\x86',
'CjkUnifiedIdeograph5707': '\xe5\x9c\x87',
'CjkUnifiedIdeograph5704': '\xe5\x9c\x84',
'CjkUnifiedIdeograph5705': '\xe5\x9c\x85',
'HangulSyllableCwing': '\xec\xb7\xbd',
'CjkUnifiedIdeograph9418': '\xe9\x90\x98',
'CjkUnifiedIdeograph9419': '\xe9\x90\x99',
'UnicodeCharacter0xde16': '\xed\xb8\x96',
'CjkUnifiedIdeograph9412': '\xe9\x90\x92',
'CjkUnifiedIdeograph9413': '\xe9\x90\x93',
'NumberFourteenFullStop': '\xe2\x92\x95',
'CjkUnifiedIdeograph9411': '\xe9\x90\x91',
'CjkUnifiedIdeograph9416': '\xe9\x90\x96',
'CjkUnifiedIdeograph9417': '\xe9\x90\x97',
'CjkUnifiedIdeograph9414': '\xe9\x90\x94',
'CjkUnifiedIdeograph9415': '\xe9\x90\x95',
'EthiopicSyllableZzaa': '\xe2\xb6\xb3',
'HangulSyllableJjeulb': '\xec\xaf\x9f',
'HangulSyllableKoelm': '\xec\xbe\xb2',
'SouthWestArrow': '\xe2\x86\x99',
'CjkUnifiedIdeograph59d8': '\xe5\xa7\x98',
'CjkUnifiedIdeograph59d9': '\xe5\xa7\x99',
'CjkUnifiedIdeograph8bb4': '\xe8\xae\xb4',
'CjkUnifiedIdeograph8bb5': '\xe8\xae\xb5',
'CjkUnifiedIdeograph8bb2': '\xe8\xae\xb2',
'CjkUnifiedIdeograph8bb3': '\xe8\xae\xb3',
'CjkUnifiedIdeograph8bb0': '\xe8\xae\xb0',
'CjkUnifiedIdeograph8bb1': '\xe8\xae\xb1',
'CjkUnifiedIdeograph59d0': '\xe5\xa7\x90',
'CjkUnifiedIdeograph59d1': '\xe5\xa7\x91',
'CjkUnifiedIdeograph59d2': '\xe5\xa7\x92',
'CjkUnifiedIdeograph59d3': '\xe5\xa7\x93',
'CjkUnifiedIdeograph59d4': '\xe5\xa7\x94',
'NotAsymptoticallyEqualTo': '\xe2\x89\x84',
'CjkUnifiedIdeograph59d6': '\xe5\xa7\x96',
'CjkUnifiedIdeograph59d7': '\xe5\xa7\x97',
'HangulSyllableNwass': '\xeb\x86\xa8',
'UnicodeCharacter0xf399': '\xef\x8e\x99',
'MalayalamDigitSix': '\xe0\xb5\xac',
'KhmerSymbolPrampiiRoc': '\xe1\xa7\xb7',
'CjkUnifiedIdeograph941c': '\xe9\x90\x9c',
'CjkUnifiedIdeograph941a': '\xe9\x90\x9a',
'CjkUnifiedIdeograph941f': '\xe9\x90\x9f',
'CjkUnifiedIdeograph941d': '\xe9\x90\x9d',
'CjkUnifiedIdeograph941e': '\xe9\x90\x9e',
'LeftwardsArrowWithStroke': '\xe2\x86\x9a',
'CjkUnifiedIdeograph5fcc': '\xe5\xbf\x8c',
'CjkUnifiedIdeograph570b': '\xe5\x9c\x8b',
'CjkUnifiedIdeograph570c': '\xe5\x9c\x8c',
'HangulSyllableMulh': '\xeb\xad\x83',
'DevanagariLetterCha': '\xe0\xa4\x9b',
'CjkUnifiedIdeograph570f': '\xe5\x9c\x8f',
'CjkUnifiedIdeograph570d': '\xe5\x9c\x8d',
'CjkUnifiedIdeograph570e': '\xe5\x9c\x8e',
'CjkUnifiedIdeograph8bbf': '\xe8\xae\xbf',
'UnicodeCharacter0xab53': '\xea\xad\x93',
'CjkUnifiedIdeograph8bbd': '\xe8\xae\xbd',
'CjkUnifiedIdeograph8bbe': '\xe8\xae\xbe',
'CjkUnifiedIdeograph8bbb': '\xe8\xae\xbb',
'CjkUnifiedIdeograph8bbc': '\xe8\xae\xbc',
'UnicodeCharacter0xab54': '\xea\xad\x94',
'CjkUnifiedIdeograph8bba': '\xe8\xae\xba',
'CjkUnifiedIdeograph59da': '\xe5\xa7\x9a',
'CjkUnifiedIdeograph59db': '\xe5\xa7\x9b',
'CjkUnifiedIdeograph59dc': '\xe5\xa7\x9c',
'CjkUnifiedIdeograph59dd': '\xe5\xa7\x9d',
'CjkUnifiedIdeograph59de': '\xe5\xa7\x9e',
'CjkUnifiedIdeograph59df': '\xe5\xa7\x9f',
'HangulSyllableBeogs': '\xeb\xb2\x87',
'HangulSyllableTyuk': '\xed\x8a\xb4',
'HangulSyllableTyuj': '\xed\x8a\xb2',
'HangulSyllableDalg': '\xeb\x8b\xad',
'CjkUnifiedIdeograph9e4a': '\xe9\xb9\x8a',
'HangulSyllableTyun': '\xed\x8a\xa0',
'HangulSyllableDalb': '\xeb\x8b\xaf',
'CjkUnifiedIdeograph771c': '\xe7\x9c\x9c',
'HangulSyllableTyuc': '\xed\x8a\xb3',
'HangulSyllableDalm': '\xeb\x8b\xae',
'HangulSyllableDalh': '\xeb\x8b\xb3',
'HangulSyllableTyud': '\xed\x8a\xa3',
'HangulSyllableDalt': '\xeb\x8b\xb1',
'LatinSmallLetterEzhWithRetroflexHook': '\xe1\xb6\x9a',
'HangulSyllableDalp': '\xeb\x8b\xb2',
'HangulSyllableDals': '\xeb\x8b\xb0',
'ParenthesizedHangulKhieukhA': '\xe3\x88\x98',
'CjkUnifiedIdeograph34a4': '\xe3\x92\xa4',
'CjkUnifiedIdeograph80fb': '\xe8\x83\xbb',
'IdeographicTelegraphSymbolForDayTwentyfive': '\xe3\x8f\xb8',
'CjkUnifiedIdeograph80fd': '\xe8\x83\xbd',
'CjkUnifiedIdeograph80fe': '\xe8\x83\xbe',
'Countersink': '\xe2\x8c\xb5',
'HangulSyllableTyut': '\xed\x8a\xb5',
'fourthroot': '\xe2\x88\x9c',
'YiSyllableHlux': '\xea\x86\xac',
'IdeographicTelegraphSymbolForHourSix': '\xe3\x8d\x9e',
'HangulSyllableWeonh': '\xec\x9b\x92',
'HangulSyllableWeong': '\xec\x9b\xa1',
'YiSyllableHlur': '\xea\x86\xb0',
'YiSyllableHlup': '\xea\x86\xae',
'HangulSyllableSass': '\xec\x83\x80',
'YiSyllableHlut': '\xea\x86\xab',
'UnicodeCharacter0xf2fb': '\xef\x8b\xbb',
'UnicodeCharacter0xe3b6': '\xee\x8e\xb6',
'YiSyllableHluo': '\xea\x86\xa3',
'UnicodeCharacter0xe98c': '\xee\xa6\x8c',
'HangulSyllableNeug': '\xeb\x8a\x91',
'RejangLetterNya': '\xea\xa4\xbb',
'CjkUnifiedIdeograph80f8': '\xe8\x83\xb8',
'CjkUnifiedIdeograph80f9': '\xe8\x83\xb9',
'HangulSyllableGgwaeng': '\xea\xbd\xb9',
'UnicodeCharacter0xf2fa': '\xef\x8b\xba',
'SoundRecordingCopyright': '\xe2\x84\x97',
'UnicodeCharacter0xec81': '\xee\xb2\x81',
'CjkUnifiedIdeograph80f0': '\xe8\x83\xb0',
'CjkUnifiedIdeograph80f1': '\xe8\x83\xb1',
'CjkUnifiedIdeograph80f2': '\xe8\x83\xb2',
'CjkUnifiedIdeograph80f3': '\xe8\x83\xb3',
'CjkUnifiedIdeograph80f4': '\xe8\x83\xb4',
'CjkUnifiedIdeograph6236': '\xe6\x88\xb6',
'CjkUnifiedIdeograph80f6': '\xe8\x83\xb6',
'CjkUnifiedIdeograph80f7': '\xe8\x83\xb7',
'CyrillicCapitalLetterYeruWithBackYer': '\xea\x99\x90',
'UnicodeCharacter0xe98d': '\xee\xa6\x8d',
'HangulSyllableJjeulh': '\xec\xaf\xa3',
'UnicodeCharacter0xf2fe': '\xef\x8b\xbe',
'HangulSyllableGwaebs': '\xea\xb4\xaa',
'HangulSyllablePunj': '\xed\x91\xbd',
'UnicodeCharacter0xf1f6': '\xef\x87\xb6',
'HangulSyllableTyeolh': '\xed\x85\xb7',
'HangulSyllableJjwelb': '\xec\xae\x8b',
'UnicodeCharacter0xec87': '\xee\xb2\x87',
'TibetanLetterKssa': '\xe0\xbd\xa9',
'HalfwidthHangulLetterMieum': '\xef\xbe\xb1',
'UnicodeCharacter0xe28a': '\xee\x8a\x8a',
'UnicodeCharacter0xe28c': '\xee\x8a\x8c',
'OghamLetterDair': '\xe1\x9a\x87',
'TeluguLetterVocalicR': '\xe0\xb0\x8b',
'HangulSyllableTyeolb': '\xed\x85\xb3',
'ArabicLigatureDadWithYehIsolatedForm': '\xef\xb4\x88',
'HangulSyllableNeuh': '\xeb\x8a\xab',
'CanadianSyllabicsNoo': '\xe1\x93\x85',
'TeluguLetterVocalicL': '\xe0\xb0\x8c',
'SundaneseConsonantSignPanyakra': '\xe1\xae\xa2',
'SylotiNagriLetterDdho': '\xea\xa0\x93',
'UnicodeCharacter0x864': '\xe0\xa1\xa4',
'HebrewPunctuationNunHafukha': '\xd7\x86',
'CjkUnifiedIdeograph8e69': '\xe8\xb9\xa9',
'CjkUnifiedIdeograph8e68': '\xe8\xb9\xa8',
'CjkUnifiedIdeograph4449': '\xe4\x91\x89',
'ThaanaLetterKaafu': '\xde\x86',
'CjkUnifiedIdeograph489e': '\xe4\xa2\x9e',
'CjkUnifiedIdeograph489d': '\xe4\xa2\x9d',
'CjkUnifiedIdeograph8e63': '\xe8\xb9\xa3',
'CjkUnifiedIdeograph489f': '\xe4\xa2\x9f',
'CjkUnifiedIdeograph489a': '\xe4\xa2\x9a',
'CjkUnifiedIdeograph8e64': '\xe8\xb9\xa4',
'CjkUnifiedIdeograph489c': '\xe4\xa2\x9c',
'CjkUnifiedIdeograph489b': '\xe4\xa2\x9b',
'CjkUnifiedIdeograph9fa0': '\xe9\xbe\xa0',
'CjkRadicalMother': '\xe2\xba\x9f',
'Tau': '\xce\xa4',
'LeftParenthesisLowerHook': '\xe2\x8e\x9d',
'CjkUnifiedIdeograph7149': '\xe7\x85\x89',
'SquareHekutaaru': '\xe3\x8c\xb6',
'ElementOfWithTwoHorizontalStrokes': '\xe2\x8b\xb9',
'UnicodeCharacter0xef5b': '\xee\xbd\x9b',
'UnicodeCharacter0xef5c': '\xee\xbd\x9c',
'UnicodeCharacter0xf827': '\xef\xa0\xa7',
'EqualsColon': '\xe2\x89\x95',
'UnicodeCharacter0xef5a': '\xee\xbd\x9a',
'CjkUnifiedIdeograph4899': '\xe4\xa2\x99',
'CjkUnifiedIdeograph4898': '\xe4\xa2\x98',
'BengaliLetterTtha': '\xe0\xa6\xa0',
'CjkUnifiedIdeograph4895': '\xe4\xa2\x95',
'CjkUnifiedIdeograph4894': '\xe4\xa2\x94',
'CjkUnifiedIdeograph4897': '\xe4\xa2\x97',
'CjkUnifiedIdeograph4896': '\xe4\xa2\x96',
'CjkUnifiedIdeograph4891': '\xe4\xa2\x91',
'CjkUnifiedIdeograph4890': '\xe4\xa2\x90',
'CjkUnifiedIdeograph4893': '\xe4\xa2\x93',
'CjkUnifiedIdeograph4892': '\xe4\xa2\x92',
'HangulSyllableTyeolp': '\xed\x85\xb6',
'UnicodeCharacter0xf820': '\xef\xa0\xa0',
'CjkCompatibilityIdeographf9ae': '\xef\xa6\xae',
'TaiLeLetterTone3': '\xe1\xa5\xb1',
'UnionBesideAndJoinedWithUnion': '\xe2\xa9\x8a',
'TaiLeLetterTone5': '\xe1\xa5\xb3',
'TaiLeLetterTone4': '\xe1\xa5\xb2',
'ErrorbarredWhiteCircle': '\xe2\xa7\xb2',
'CjkUnifiedIdeograph9c6d': '\xe9\xb1\xad',
'CjkUnifiedIdeograph9c6e': '\xe9\xb1\xae',
'CjkUnifiedIdeograph9c6f': '\xe9\xb1\xaf',
'UnicodeCharacter0xa7e0': '\xea\x9f\xa0',
'CjkUnifiedIdeograph9c6a': '\xe9\xb1\xaa',
'CjkUnifiedIdeograph9c6b': '\xe9\xb1\xab',
'CjkUnifiedIdeograph9c6c': '\xe9\xb1\xac',
'HangulSyllableGgweogg': '\xea\xbf\x96',
'HangulSyllableBanj': '\xeb\xb0\x99',
'UnicodeCharacter0xe1f2': '\xee\x87\xb2',
'CjkUnifiedIdeograph5e4f': '\xe5\xb9\x8f',
'UnicodeCharacter0xe1f3': '\xee\x87\xb3',
'UnicodeCharacter0xf821': '\xef\xa0\xa1',
'HangulSyllableJin': '\xec\xa7\x84',
'CjkUnifiedIdeograph427a': '\xe4\x89\xba',
'CjkUnifiedIdeograph427c': '\xe4\x89\xbc',
'CjkUnifiedIdeograph427b': '\xe4\x89\xbb',
'CjkUnifiedIdeograph427e': '\xe4\x89\xbe',
'CjkUnifiedIdeograph427d': '\xe4\x89\xbd',
'UnicodeCharacter0xf219': '\xef\x88\x99',
'CjkUnifiedIdeograph427f': '\xe4\x89\xbf',
'HangulSyllableNoelp': '\xeb\x87\x9a',
'HangulSyllableByup': '\xeb\xb8\x8a',
'HangulSyllableByus': '\xeb\xb8\x83',
'HangulSyllableNoels': '\xeb\x87\x98',
'MongolianLetterAliGaliTa': '\xe1\xa2\x90',
'HangulSyllableNilt': '\xeb\x8b\x95',
'UnicodeCharacter0xf211': '\xef\x88\x91',
'UnicodeCharacter0xf210': '\xef\x88\x90',
'HangulSyllableNoelh': '\xeb\x87\x9b',
'CjkUnifiedIdeograph78c2': '\xe7\xa3\x82',
'HangulSyllableByuk': '\xeb\xb8\x88',
'HangulSyllableByuj': '\xeb\xb8\x86',
'HangulSyllableByum': '\xeb\xb8\x80',
'LatinSmallLetterAlphaWithRetroflexHook': '\xe1\xb6\x90',
'HangulSyllableByun': '\xeb\xb7\xb4',
'ActivateSymmetricSwapping': '\xe2\x81\xab',
'HangzhouNumeralOne': '\xe3\x80\xa1',
'HangulSyllableByub': '\xeb\xb8\x81',
'HangulSyllableByud': '\xeb\xb7\xb7',
'HangulSyllableByug': '\xeb\xb7\xb1',
'HangulSyllableNoelg': '\xeb\x87\x95',
'CjkUnifiedIdeograph72a6': '\xe7\x8a\xa6',
'HangulSyllableKwac': '\xec\xbe\x87',
'HangulSyllableNilp': '\xeb\x8b\x96',
'CjkUnifiedIdeograph78c6': '\xe7\xa3\x86',
'CanadianSyllabicsWestcreeRwaa': '\xe1\x95\x8f',
'GlagoliticCapitalLetterIotatedBigYus': '\xe2\xb0\xa9',
'CjkUnifiedIdeograph9c65': '\xe9\xb1\xa5',
'CjkUnifiedIdeograph9c66': '\xe9\xb1\xa6',
'CjkUnifiedIdeograph72a5': '\xe7\x8a\xa5',
'CjkUnifiedIdeograph9c60': '\xe9\xb1\xa0',
'CjkUnifiedIdeograph6430': '\xe6\x90\xb0',
'CjkUnifiedIdeograph9c62': '\xe9\xb1\xa2',
'CjkUnifiedIdeograph9c63': '\xe9\xb1\xa3',
'UnicodeCharacter0xe03a': '\xee\x80\xba',
'VaiSyllableNdoo': '\xea\x96\x86',
'CjkUnifiedIdeograph9c68': '\xe9\xb1\xa8',
'CjkUnifiedIdeograph9c69': '\xe9\xb1\xa9',
'CjkUnifiedIdeograph5de6': '\xe5\xb7\xa6',
'UnicodeCharacter0xf39b': '\xef\x8e\x9b',
'UnicodeCharacter0xfd44': '\xef\xb5\x84',
'HangulSyllableHyegs': '\xed\x98\x9f',
'YiSyllableShy': '\xea\x8f\x82',
'CjkUnifiedIdeograph5e4a': '\xe5\xb9\x8a',
'CjkUnifiedIdeograph4271': '\xe4\x89\xb1',
'CjkUnifiedIdeograph4270': '\xe4\x89\xb0',
'CjkUnifiedIdeograph4273': '\xe4\x89\xb3',
'CjkUnifiedIdeograph4272': '\xe4\x89\xb2',
'CjkUnifiedIdeograph4275': '\xe4\x89\xb5',
'CjkUnifiedIdeograph4274': '\xe4\x89\xb4',
'CjkUnifiedIdeograph4277': '\xe4\x89\xb7',
'CjkUnifiedIdeograph4276': '\xe4\x89\xb6',
'CjkUnifiedIdeograph4279': '\xe4\x89\xb9',
'CjkUnifiedIdeograph4278': '\xe4\x89\xb8',
'UnicodeCharacter0xf21e': '\xef\x88\x9e',
'UnicodeCharacter0xf21d': '\xef\x88\x9d',
'UnicodeCharacter0xf21c': '\xef\x88\x9c',
'UnicodeCharacter0xf21b': '\xef\x88\x9b',
'UnicodeCharacter0xf21a': '\xef\x88\x9a',
'HangulSyllableNwaelt': '\xeb\x86\xbd',
'GurmukhiLetterGa': '\xe0\xa8\x97',
'UnicodeCharacter0xefda': '\xee\xbf\x9a',
'UnicodeCharacter0xefdc': '\xee\xbf\x9c',
'UnicodeCharacter0xefdb': '\xee\xbf\x9b',
'UnicodeCharacter0xefde': '\xee\xbf\x9e',
'ChamLetterPha': '\xea\xa8\x9c',
'CyrillicCapitalLetterNje': '\xd0\x8a',
'CyrillicCapitalLetterDzhe': '\xd0\x8f',
'HangulSyllableKwap': '\xec\xbe\x8a',
'SmallEqualsSign': '\xef\xb9\xa6',
'CjkUnifiedIdeograph72a9': '\xe7\x8a\xa9',
'RomanNumeralSix': '\xe2\x85\xa5',
'GeorgianSmallLetterGan': '\xe2\xb4\x82',
'YiSyllableMup': '\xea\x83\x86',
'YiSyllableMur': '\xea\x83\x88',
'YiSyllableMut': '\xea\x83\x83',
'YiSyllableMux': '\xea\x83\x84',
'HangulSyllablePom': '\xed\x8f\xbc',
'YiSyllableShu': '\xea\x8e\xbc',
'HebrewPointRafe': '\xd6\xbf',
'CombiningAsteriskAbove': '\xe2\x83\xb0',
'HangulChoseongMieum': '\xe1\x84\x86',
'YiSyllableMuo': '\xea\x82\xbb',
'UpDownBlackArrow': '\xe2\xac\x8d',
'YiSyllableNgax': '\xea\x89\xa1',
'AnticlockwiseContourIntegral': '\xe2\x88\xb3',
'YiSyllableNgap': '\xea\x89\xa3',
'CjkUnifiedIdeograph6434': '\xe6\x90\xb4',
'LessthanOrSlantedEqualToWithDotAboveRight': '\xe2\xaa\x83',
'NotSign': '\xc2\xac',
'UnicodeCharacter0xefd9': '\xee\xbf\x99',
'UnicodeCharacter0xefd8': '\xee\xbf\x98',
'UnicodeCharacter0xef59': '\xee\xbd\x99',
'UnicodeCharacter0xefd1': '\xee\xbf\x91',
'UnicodeCharacter0xefd0': '\xee\xbf\x90',
'UnicodeCharacter0xefd3': '\xee\xbf\x93',
'UnicodeCharacter0xefd2': '\xee\xbf\x92',
'UnicodeCharacter0xefd5': '\xee\xbf\x95',
'UnicodeCharacter0xefd4': '\xee\xbf\x94',
'UnicodeCharacter0xefd7': '\xee\xbf\x97',
'UnicodeCharacter0xefd6': '\xee\xbf\x96',
'UnicodeCharacter0x86c': '\xe0\xa1\xac',
'UnicodeCharacter0xf877': '\xef\xa1\xb7',
'HangulSyllableJjwelm': '\xec\xae\x8a',
'VaiSyllableNggen': '\xea\x98\x85',
'ArabicLigatureFehWithKhahWithMeemFinalForm': '\xef\xb5\xbc',
'VaiSymbolFeeng': '\xea\x98\x93',
'HangulSyllableJjwes': '\xec\xae\x93',
'VaiSyllableNggee': '\xea\x94\x9f',
'MyanmarConsonantSignMedialRa': '\xe1\x80\xbc',
'OlChikiDigitTwo': '\xe1\xb1\x92',
'UnicodeCharacter0xef51': '\xee\xbd\x91',
'HangulSyllablePyaen': '\xed\x8d\xa4',
'UnicodeCharacter0xef56': '\xee\xbd\x96',
'HangulSyllableDdwal': '\xeb\x98\xb4',
'CjkUnifiedIdeograph6ce3': '\xe6\xb3\xa3',
'HangulSyllablePyaem': '\xed\x8d\xb0',
'LatinSmallLetterBWithHook': '\xc9\x93',
'CjkUnifiedIdeograph935a': '\xe9\x8d\x9a',
'CjkUnifiedIdeograph935c': '\xe9\x8d\x9c',
'CjkUnifiedIdeograph935b': '\xe9\x8d\x9b',
'CjkUnifiedIdeograph935e': '\xe9\x8d\x9e',
'CjkUnifiedIdeograph935d': '\xe9\x8d\x9d',
'HangulSyllableJjwep': '\xec\xae\x9a',
'CjkUnifiedIdeograph935f': '\xe9\x8d\x9f',
'CjkUnifiedIdeograph612f': '\xe6\x84\xaf',
'CjkUnifiedIdeograph612e': '\xe6\x84\xae',
'CjkUnifiedIdeograph612d': '\xe6\x84\xad',
'CjkUnifiedIdeograph612c': '\xe6\x84\xac',
'CjkUnifiedIdeograph612b': '\xe6\x84\xab',
'CjkUnifiedIdeograph612a': '\xe6\x84\xaa',
'HangulSyllablePyaej': '\xed\x8d\xb6',
'CjkUnifiedIdeograph7cff': '\xe7\xb3\xbf',
'CjkUnifiedIdeograph7cfd': '\xe7\xb3\xbd',
'CjkUnifiedIdeograph7cfe': '\xe7\xb3\xbe',
'CjkUnifiedIdeograph7cfb': '\xe7\xb3\xbb',
'CjkUnifiedIdeograph7cfc': '\xe7\xb3\xbc',
'CjkUnifiedIdeograph7cfa': '\xe7\xb3\xba',
'CjkUnifiedIdeograph6ce8': '\xe6\xb3\xa8',
'LatinCapitalLetterAWithDiaeresisAndMacron': '\xc7\x9e',
'HangulSyllablePyaeh': '\xed\x8d\xbb',
'HangulSyllablePyaeg': '\xed\x8d\xa1',
'UnicodeCharacter0x37f': '\xcd\xbf',
'UnicodeCharacter0xd8ca': '\xed\xa3\x8a',
'EthiopicSyllableQhwee': '\xe1\x89\x9c',
'UnicodeCharacter0xd8cc': '\xed\xa3\x8c',
'UnicodeCharacter0xd8cb': '\xed\xa3\x8b',
'UnicodeCharacter0xd8ce': '\xed\xa3\x8e',
'UnicodeCharacter0xd8cd': '\xed\xa3\x8d',
'BalineseLetterDaMurdaAlpaprana': '\xe1\xac\x9f',
'CjkUnifiedIdeograph9351': '\xe9\x8d\x91',
'CjkUnifiedIdeograph9350': '\xe9\x8d\x90',
'CjkUnifiedIdeograph9353': '\xe9\x8d\x93',
'CjkUnifiedIdeograph9352': '\xe9\x8d\x92',
'CjkUnifiedIdeograph9355': '\xe9\x8d\x95',
'CjkUnifiedIdeograph9354': '\xe9\x8d\x94',
'CjkUnifiedIdeograph6129': '\xe6\x84\xa9',
'CyrillicCapitalLetterGheWithStroke': '\xd2\x92',
'CjkUnifiedIdeograph6127': '\xe6\x84\xa7',
'CjkUnifiedIdeograph6126': '\xe6\x84\xa6',
'CjkUnifiedIdeograph6125': '\xe6\x84\xa5',
'CjkUnifiedIdeograph6124': '\xe6\x84\xa4',
'CjkUnifiedIdeograph6123': '\xe6\x84\xa3',
'CjkUnifiedIdeograph6122': '\xe6\x84\xa2',
'CjkUnifiedIdeograph6121': '\xe6\x84\xa1',
'CjkUnifiedIdeograph6120': '\xe6\x84\xa0',
'UnicodeCharacter0xe281': '\xee\x8a\x81',
'DingbatNegativeCircledDigitEight': '\xe2\x9d\xbd',
'LatinCapitalLetterInsularR': '\xea\x9e\x82',
'LatinCapitalLetterInsularS': '\xea\x9e\x84',
'CjkUnifiedIdeograph8a19': '\xe8\xa8\x99',
'LatinCapitalLetterInsularT': '\xea\x9e\x86',
'UnicodeCharacter0x379': '\xcd\xb9',
'UnicodeCharacter0x378': '\xcd\xb8',
'UnicodeCharacter0xd8c3': '\xed\xa3\x83',
'UnicodeCharacter0xd8c2': '\xed\xa3\x82',
'UnicodeCharacter0xd8c5': '\xed\xa3\x85',
'UnicodeCharacter0xd8c4': '\xed\xa3\x84',
'UnicodeCharacter0xd8c7': '\xed\xa3\x87',
'UnicodeCharacter0xd8c6': '\xed\xa3\x86',
'CjkUnifiedIdeograph7cf6': '\xe7\xb3\xb6',
'CjkUnifiedIdeograph7cf7': '\xe7\xb3\xb7',
'CjkUnifiedIdeograph7cf4': '\xe7\xb3\xb4',
'CjkUnifiedIdeograph7cf5': '\xe7\xb3\xb5',
'CjkUnifiedIdeograph7cf2': '\xe7\xb3\xb2',
'CjkUnifiedIdeograph7cf3': '\xe7\xb3\xb3',
'CjkUnifiedIdeograph7cf0': '\xe7\xb3\xb0',
'KangxiRadicalDoor': '\xe2\xbc\xbe',
'DingbatCircledSansserifDigitFour': '\xe2\x9e\x83',
'GreekSmallLetterAlphaWithPsili': '\xe1\xbc\x80',
'CjkUnifiedIdeograph7cf9': '\xe7\xb3\xb9',
'HangulSyllableDdwah': '\xeb\x99\x87',
'CjkUnifiedIdeograph8d5b': '\xe8\xb5\x9b',
'HangulSyllableCwaelh': '\xec\xb5\x8f',
'ThaanaOaboafili': '\xde\xaf',
'TibetanLetterMa': '\xe0\xbd\x98',
'HangulSyllableCwaelb': '\xec\xb5\x8b',
'HangulSyllableJjwet': '\xec\xae\x99',
'CjkUnifiedIdeograph8d5d': '\xe8\xb5\x9d',
'HangulSyllableGweong': '\xea\xb6\x9d',
'UnicodeCharacter0xe130': '\xee\x84\xb0',
'HexagramForModesty': '\xe4\xb7\x8e',
'UnicodeCharacter0xe137': '\xee\x84\xb7',
'InteriorProduct': '\xe2\xa8\xbc',
'HangulSyllableCwaelt': '\xec\xb5\x8d',
'HangulSyllableCwaels': '\xec\xb5\x8c',
'HangulSyllableGweonj': '\xea\xb6\x8d',
'HangulSyllableTyaenh': '\xed\x84\x9a',
'DoubleSubset': '\xe2\x8b\x90',
'HangulSyllableTyeog': '\xed\x85\xa9',
'HangulSyllableTyugs': '\xed\x8a\x9f',
'CjkUnifiedIdeograph729f': '\xe7\x8a\x9f',
'HangulSyllablePyaes': '\xed\x8d\xb3',
'CjkUnifiedIdeograph729d': '\xe7\x8a\x9d',
'CjkUnifiedIdeograph729e': '\xe7\x8a\x9e',
'CjkUnifiedIdeograph729b': '\xe7\x8a\x9b',
'CjkUnifiedIdeograph729c': '\xe7\x8a\x9c',
'UnicodeCharacter0xf48a': '\xef\x92\x8a',
'CjkUnifiedIdeograph729a': '\xe7\x8a\x9a',
'CjkUnifiedIdeograph9eca': '\xe9\xbb\x8a',
'CjkUnifiedIdeograph9ecc': '\xe9\xbb\x8c',
'CjkUnifiedIdeograph9ecb': '\xe9\xbb\x8b',
'CjkUnifiedIdeograph9ece': '\xe9\xbb\x8e',
'CjkUnifiedIdeograph9ecd': '\xe9\xbb\x8d',
'GlagoliticSmallLetterFritu': '\xe2\xb1\x87',
'PrecedesAboveNotAlmostEqualTo': '\xe2\xaa\xb9',
'CjkUnifiedIdeograph398f': '\xe3\xa6\x8f',
'CjkUnifiedIdeograph398d': '\xe3\xa6\x8d',
'CjkUnifiedIdeograph398e': '\xe3\xa6\x8e',
'CjkUnifiedIdeograph398b': '\xe3\xa6\x8b',
'CjkUnifiedIdeograph398c': '\xe3\xa6\x8c',
'CjkUnifiedIdeograph398a': '\xe3\xa6\x8a',
'CjkUnifiedIdeograph5d2a': '\xe5\xb4\xaa',
'UnicodeCharacter0xe138': '\xee\x84\xb8',
'UnicodeCharacter0xdf1a': '\xed\xbc\x9a',
'HangulSyllableTyeon': '\xed\x85\xac',
'HiraganaLetterSmallTu': '\xe3\x81\xa3',
'CjkUnifiedIdeograph9ec1': '\xe9\xbb\x81',
'CjkUnifiedIdeograph9ec0': '\xe9\xbb\x80',
'CjkUnifiedIdeograph9ec3': '\xe9\xbb\x83',
'OlChikiLetterAng': '\xe1\xb1\x9d',
'CjkUnifiedIdeograph9ec5': '\xe9\xbb\x85',
'CjkUnifiedIdeograph9ec4': '\xe9\xbb\x84',
'CjkUnifiedIdeograph3988': '\xe3\xa6\x88',
'CjkUnifiedIdeograph3989': '\xe3\xa6\x89',
'CjkUnifiedIdeograph3986': '\xe3\xa6\x86',
'CjkUnifiedIdeograph3987': '\xe3\xa6\x87',
'CjkUnifiedIdeograph3984': '\xe3\xa6\x84',
'CjkUnifiedIdeograph3985': '\xe3\xa6\x85',
'CjkUnifiedIdeograph3982': '\xe3\xa6\x82',
'CjkUnifiedIdeograph3983': '\xe3\xa6\x83',
'CjkUnifiedIdeograph3980': '\xe3\xa6\x80',
'CjkUnifiedIdeograph3981': '\xe3\xa6\x81',
'StaffOfHermes': '\xe2\x9a\x9a',
'CjkUnifiedIdeograph865d': '\xe8\x99\x9d',
'UnicodeCharacter0x991': '\xe0\xa6\x91',
'ArabicLetterPehehInitialForm': '\xef\xad\xb0',
'CjkUnifiedIdeograph7298': '\xe7\x8a\x98',
'UnicodeCharacter0x992': '\xe0\xa6\x92',
'CjkUnifiedIdeograph7296': '\xe7\x8a\x96',
'CjkUnifiedIdeograph7297': '\xe7\x8a\x97',
'CjkUnifiedIdeograph7294': '\xe7\x8a\x94',
'CjkUnifiedIdeograph7295': '\xe7\x8a\x95',
'CjkUnifiedIdeograph7292': '\xe7\x8a\x92',
'CjkUnifiedIdeograph7293': '\xe7\x8a\x93',
'CjkUnifiedIdeograph7290': '\xe7\x8a\x90',
'CjkUnifiedIdeograph7291': '\xe7\x8a\x91',
'HangulSyllableGaess': '\xea\xb0\xb0',
'LatinSmallLetterOpenEWithRetroflexHook': '\xe1\xb6\x93',
'UnicodeCharacter0xed60': '\xee\xb5\xa0',
'UnicodeCharacter0xf69f': '\xef\x9a\x9f',
'HebrewPunctuationMaqaf': '\xd6\xbe',
'SquareKaratto': '\xe3\x8c\x8c',
'OriyaLetterCha': '\xe0\xac\x9b',
'ModifierLetterLowInvertedExclamationMark': '\xea\x9c\x9f',
'CjkUnifiedIdeograph3678': '\xe3\x99\xb8',
'CjkUnifiedIdeograph3679': '\xe3\x99\xb9',
'UnicodeCharacter0xda6a': '\xed\xa9\xaa',
'CjkUnifiedIdeograph9bca': '\xe9\xaf\x8a',
'CjkUnifiedIdeograph9bcf': '\xe9\xaf\x8f',
'HangulSyllableMeng': '\xeb\xa9\xa9',
'CjkUnifiedIdeograph9bcd': '\xe9\xaf\x8d',
'IdeographicTelegraphSymbolForHourSeven': '\xe3\x8d\x9f',
'CjkUnifiedIdeograph3670': '\xe3\x99\xb0',
'CjkUnifiedIdeograph3671': '\xe3\x99\xb1',
'CjkUnifiedIdeograph3672': '\xe3\x99\xb2',
'CjkUnifiedIdeograph3673': '\xe3\x99\xb3',
'MaleSign': '\xe2\x99\x82',
'CjkUnifiedIdeograph3675': '\xe3\x99\xb5',
'CjkUnifiedIdeograph3676': '\xe3\x99\xb6',
'HangzhouNumeralSeven': '\xe3\x80\xa7',
'CjkUnifiedIdeograph7748': '\xe7\x9d\x88',
'CjkUnifiedIdeograph7749': '\xe7\x9d\x89',
'HangulSyllableWeb': '\xec\x9b\xb9',
'MongolianLetterAliGaliHalfU': '\xe1\xa2\xa6',
'CjkUnifiedIdeograph7741': '\xe7\x9d\x81',
'CjkUnifiedIdeograph7742': '\xe7\x9d\x82',
'CjkUnifiedIdeograph7743': '\xe7\x9d\x83',
'CjkUnifiedIdeograph7744': '\xe7\x9d\x84',
'CjkUnifiedIdeograph7745': '\xe7\x9d\x85',
'CjkUnifiedIdeograph7746': '\xe7\x9d\x86',
'CjkUnifiedIdeograph7747': '\xe7\x9d\x87',
'UnicodeCharacter0xebab': '\xee\xae\xab',
'LatinSmallLetterWWithDotAbove': '\xe1\xba\x87',
'UnicodeCharacter0xebac': '\xee\xae\xac',
'UnicodeCharacter0xdf1e': '\xed\xbc\x9e',
'HangulSyllableJjit': '\xec\xb0\xa5',
'HangulSyllableJjis': '\xec\xb0\x9f',
'HangzhouNumeralSix': '\xe3\x80\xa6',
'HangulSyllableJjin': '\xec\xb0\x90',
'UnicodeCharacter0xf1f1': '\xef\x87\xb1',
'HangulSyllableWec': '\xec\x9b\xbf',
'HangulSyllableJjim': '\xec\xb0\x9c',
'HangulSyllableDdwac': '\xeb\x99\x83',
'HangulSyllableJjik': '\xec\xb0\xa4',
'HangulSyllableJjih': '\xec\xb0\xa7',
'HangulSyllableJjig': '\xec\xb0\x8d',
'HangulSyllableJjid': '\xec\xb0\x93',
'HangulSyllableJjib': '\xec\xb0\x9d',
'HangulSyllableJjic': '\xec\xb0\xa3',
'HangulSyllableGeun': '\xea\xb7\xbc',
'HangulSyllableGeul': '\xea\xb8\x80',
'HangulSyllableGeum': '\xea\xb8\x88',
'HangulSyllableGeuj': '\xea\xb8\x8e',
'CyrillicSmallLetterIWithDiaeresis': '\xd3\xa5',
'HangulSyllableGeuh': '\xea\xb8\x93',
'HangulSyllableGaeng': '\xea\xb0\xb1',
'CjkUnifiedIdeograph774a': '\xe7\x9d\x8a',
'CjkUnifiedIdeograph774b': '\xe7\x9d\x8b',
'CjkUnifiedIdeograph774c': '\xe7\x9d\x8c',
'CjkUnifiedIdeograph774d': '\xe7\x9d\x8d',
'CjkUnifiedIdeograph774e': '\xe7\x9d\x8e',
'CjkUnifiedIdeograph774f': '\xe7\x9d\x8f',
'CjkUnifiedIdeograph9bc2': '\xe9\xaf\x82',
'CjkUnifiedIdeograph9bc3': '\xe9\xaf\x83',
'HangulJongseongMieumkiyeok': '\xe1\x87\x9a',
'CjkUnifiedIdeograph9bc1': '\xe9\xaf\x81',
'CjkUnifiedIdeograph9bc6': '\xe9\xaf\x86',
'CjkUnifiedIdeograph9bc7': '\xe9\xaf\x87',
'CjkUnifiedIdeograph9bc4': '\xe9\xaf\x84',
'CjkUnifiedIdeograph9bc5': '\xe9\xaf\x85',
'UnicodeCharacter0xea0b': '\xee\xa8\x8b',
'CjkUnifiedIdeograph367a': '\xe3\x99\xba',
'CjkUnifiedIdeograph367b': '\xe3\x99\xbb',
'CjkUnifiedIdeograph367c': '\xe3\x99\xbc',
'CjkUnifiedIdeograph367d': '\xe3\x99\xbd',
'CjkUnifiedIdeograph367e': '\xe3\x99\xbe',
'CjkUnifiedIdeograph367f': '\xe3\x99\xbf',
'UnicodeCharacter0xea0e': '\xee\xa8\x8e',
'BoxDrawingsDownDoubleAndHorizontalSingle': '\xe2\x95\xa5',
'YiSyllableZy': '\xea\x8a\xaa',
'HangulSyllableJolb': '\xec\xa1\xbb',
'CjkUnifiedIdeograph54a8': '\xe5\x92\xa8',
'CjkUnifiedIdeograph54a9': '\xe5\x92\xa9',
'CjkUnifiedIdeograph54a6': '\xe5\x92\xa6',
'CjkUnifiedIdeograph54a7': '\xe5\x92\xa7',
'CjkUnifiedIdeograph54a4': '\xe5\x92\xa4',
'CjkUnifiedIdeograph54a5': '\xe5\x92\xa5',
'CjkUnifiedIdeograph54a2': '\xe5\x92\xa2',
'CjkUnifiedIdeograph54a3': '\xe5\x92\xa3',
'CjkUnifiedIdeograph54a0': '\xe5\x92\xa0',
'CjkUnifiedIdeograph54a1': '\xe5\x92\xa1',
'YiSyllableZi': '\xea\x8a\x8f',
'HangulSyllableColg': '\xec\xb4\x91',
'RightwardsArrowAboveReverseAlmostEqualTo': '\xe2\xad\x88',
'YiSyllableZo': '\xea\x8a\x9d',
'YiSyllableZa': '\xea\x8a\x96',
'SinhalaLetterIluuyanna': '\xe0\xb6\x90',
'YiSyllableZe': '\xea\x8a\xa0',
'UnicodeCharacter0xffc': '\xe0\xbf\xbc',
'UnicodeCharacter0xffb': '\xe0\xbf\xbb',
'UnicodeCharacter0xffa': '\xe0\xbf\xba',
'CjkUnifiedIdeograph9e4c': '\xe9\xb9\x8c',
'UnicodeCharacter0xfff': '\xe0\xbf\xbf',
'UnicodeCharacter0xffe': '\xe0\xbf\xbe',
'UnicodeCharacter0xffd': '\xe0\xbf\xbd',
'YiSyllableNie': '\xea\x86\x82',
'ArmenianSmallLetterFeh': '\xd6\x86',
'UnicodeCharacter0xa9d2': '\xea\xa7\x92',
'UnicodeCharacter0xa9d3': '\xea\xa7\x93',
'UnicodeCharacter0xa9d0': '\xea\xa7\x90',
'UnicodeCharacter0xa9d1': '\xea\xa7\x91',
'UnicodeCharacter0xa9d6': '\xea\xa7\x96',
'UnicodeCharacter0xa9d7': '\xea\xa7\x97',
'UnicodeCharacter0xa9d4': '\xea\xa7\x94',
'UnicodeCharacter0xa9d5': '\xea\xa7\x95',
'CyrillicSmallLetterHa': '\xd1\x85',
'CjkUnifiedIdeograph691d': '\xe6\xa4\x9d',
'CjkUnifiedIdeograph691e': '\xe6\xa4\x9e',
'CjkUnifiedIdeograph691b': '\xe6\xa4\x9b',
'CjkUnifiedIdeograph691c': '\xe6\xa4\x9c',
'CjkUnifiedIdeograph691a': '\xe6\xa4\x9a',
'SinhalaLetterMuurdhajaLayanna': '\xe0\xb7\x85',
'MongolianDigitSeven': '\xe1\xa0\x97',
'N': '\xce\x9d',
'ArabicLigatureMeemWithMeemWithYehFinalForm': '\xef\xb6\xb1',
'CjkUnifiedIdeograph54af': '\xe5\x92\xaf',
'CjkUnifiedIdeograph54ad': '\xe5\x92\xad',
'CjkUnifiedIdeograph54ae': '\xe5\x92\xae',
'CjkUnifiedIdeograph54ab': '\xe5\x92\xab',
'CjkUnifiedIdeograph54ac': '\xe5\x92\xac',
'LatinSmallLetterLWithStroke': '\xc5\x82',
'CjkUnifiedIdeograph54aa': '\xe5\x92\xaa',
'UnicodeCharacter0xa9db': '\xea\xa7\x9b',
'UnicodeCharacter0xa9dc': '\xea\xa7\x9c',
'KangxiRadicalHorse': '\xe2\xbe\xba',
'UnicodeCharacter0xa9df': '\xea\xa7\x9f',
'CjkUnifiedIdeograph6918': '\xe6\xa4\x98',
'CjkUnifiedIdeograph6919': '\xe6\xa4\x99',
'CjkUnifiedIdeograph6916': '\xe6\xa4\x96',
'CjkUnifiedIdeograph6917': '\xe6\xa4\x97',
'CjkUnifiedIdeograph6914': '\xe6\xa4\x94',
'CjkUnifiedIdeograph6915': '\xe6\xa4\x95',
'CjkUnifiedIdeograph6912': '\xe6\xa4\x92',
'CjkUnifiedIdeograph6913': '\xe6\xa4\x93',
'CjkUnifiedIdeograph6910': '\xe6\xa4\x90',
'CjkUnifiedIdeograph6911': '\xe6\xa4\x91',
'UnicodeCharacter0xff3': '\xe0\xbf\xb3',
'UnicodeCharacter0xff2': '\xe0\xbf\xb2',
'UnicodeCharacter0xff1': '\xe0\xbf\xb1',
'UnicodeCharacter0xff0': '\xe0\xbf\xb0',
'UnicodeCharacter0xff7': '\xe0\xbf\xb7',
'UnicodeCharacter0xff6': '\xe0\xbf\xb6',
'UnicodeCharacter0xff5': '\xe0\xbf\xb5',
'UnicodeCharacter0xff4': '\xe0\xbf\xb4',
'UnicodeCharacter0xff9': '\xe0\xbf\xb9',
'UnicodeCharacter0xff8': '\xe0\xbf\xb8',
'YiSyllableFap': '\xea\x83\x94',
'CjkUnifiedIdeograph3b67': '\xe3\xad\xa7',
'CjkUnifiedIdeograph3b66': '\xe3\xad\xa6',
'CjkUnifiedIdeograph3b65': '\xe3\xad\xa5',
'CjkUnifiedIdeograph3b64': '\xe3\xad\xa4',
'CjkUnifiedIdeograph3b63': '\xe3\xad\xa3',
'CjkUnifiedIdeograph3b62': '\xe3\xad\xa2',
'CjkUnifiedIdeograph3b61': '\xe3\xad\xa1',
'CjkUnifiedIdeograph3b60': '\xe3\xad\xa0',
'YiSyllableNdep': '\xea\x85\xa1',
'KayahLiVowelU': '\xea\xa4\xa8',
'CjkUnifiedIdeograph3b69': '\xe3\xad\xa9',
'CjkUnifiedIdeograph3b68': '\xe3\xad\xa8',
'GreekKoronis': '\xe1\xbe\xbd',
'HangulSyllableBogs': '\xeb\xb3\xb7',
'UnicodeCharacter0xf53b': '\xef\x94\xbb',
'OunceSign': '\xe2\x84\xa5',
'ArabicLigatureZahWithMeemIsolatedForm': '\xef\xb0\xa8',
'UnicodeCharacter0xf53c': '\xef\x94\xbc',
'ArabicLigatureSeenWithMeemMedialForm': '\xef\xb3\xa7',
'CjkUnifiedIdeograph960d': '\xe9\x98\x8d',
'HangulSyllableHal': '\xed\x95\xa0',
'CjkUnifiedIdeograph8d58': '\xe8\xb5\x98',
'CjkUnifiedIdeograph8650': '\xe8\x99\x90',
'GeorgianLetterGan': '\xe1\x83\x92',
'UpwardsHarpoonWithBarbLeftBesideDownwardsHarpoonWithBarbRight': '\xe2\xa5\xae',
'KangxiRadicalSun': '\xe2\xbd\x87',
'UnicodeCharacter0xf85e': '\xef\xa1\x9e',
'BraillePatternDots2348': '\xe2\xa2\x8e',
'UnicodeCharacter0xe54b': '\xee\x95\x8b',
'GeorgianSmallLetterPar': '\xe2\xb4\x8e',
'BraillePatternDots2345': '\xe2\xa0\x9e',
'BraillePatternDots2346': '\xe2\xa0\xae',
'BraillePatternDots2347': '\xe2\xa1\x8e',
'CjkUnifiedIdeograph6f91': '\xe6\xbe\x91',
'CjkUnifiedIdeograph3b6f': '\xe3\xad\xaf',
'CjkUnifiedIdeograph3b6e': '\xe3\xad\xae',
'CjkUnifiedIdeograph3b6d': '\xe3\xad\xad',
'CjkUnifiedIdeograph3b6c': '\xe3\xad\xac',
'CjkUnifiedIdeograph3b6b': '\xe3\xad\xab',
'CjkUnifiedIdeograph3b6a': '\xe3\xad\xaa',
'CjkUnifiedIdeograph6f96': '\xe6\xbe\x96',
'CjkUnifiedIdeograph6f99': '\xe6\xbe\x99',
'CjkUnifiedIdeograph6f98': '\xe6\xbe\x98',
'UnicodeCharacter0xe13c': '\xee\x84\xbc',
'digamma': '\xcf\x9d',
'HangulSyllableDdyonh': '\xeb\x9a\x86',
'YiSyllableGex': '\xea\x87\xaf',
'YiSyllableGet': '\xea\x87\xae',
'HangulSyllableBbeols': '\xeb\xbb\x9c',
'UnicodeCharacter0xe13a': '\xee\x84\xba',
'YiSyllableGep': '\xea\x87\xb1',
'FloralHeart': '\xe2\x9d\xa6',
'KannadaLetterLa': '\xe0\xb2\xb2',
'koppa': '\xcf\x9f',
'KhmerSymbolPrambeiRoc': '\xe1\xa7\xb8',
'HangulSyllableBbeolh': '\xeb\xbb\x9f',
'HangulSyllableBbeolg': '\xeb\xbb\x99',
'HangulSyllableDyalb': '\xeb\x8c\xa7',
'UnicodeCharacter0xe13e': '\xee\x84\xbe',
'CjkUnifiedIdeograph9df9': '\xe9\xb7\xb9',
'CjkUnifiedIdeograph9df8': '\xe9\xb7\xb8',
'CjkUnifiedIdeograph9df7': '\xe9\xb7\xb7',
'CjkUnifiedIdeograph9df6': '\xe9\xb7\xb6',
'CjkUnifiedIdeograph9df5': '\xe9\xb7\xb5',
'CjkUnifiedIdeograph9df4': '\xe9\xb7\xb4',
'CjkUnifiedIdeograph9df3': '\xe9\xb7\xb3',
'CjkUnifiedIdeograph9df2': '\xe9\xb7\xb2',
'CjkUnifiedIdeograph9df1': '\xe9\xb7\xb1',
'GreaterthanEqualToOrLessthan': '\xe2\x8b\x9b',
'DentistrySymbolLightVerticalAndBottomRight': '\xe2\x8e\xbf',
'HangulSyllableBags': '\xeb\xb0\x97',
'ArabicLigatureNoonWithJeemInitialForm': '\xef\xb3\x92',
'UnicodeCharacter0xe7b2': '\xee\x9e\xb2',
'CjkUnifiedIdeograph7e18': '\xe7\xb8\x98',
'HangulSyllableJjwej': '\xec\xae\x96',
'UnicodeCharacter0xe7b0': '\xee\x9e\xb0',
'SupersetOf': '\xe2\x8a\x83',
'UnicodeCharacter0xe7b1': '\xee\x9e\xb1',
'CjkCompatibilityIdeographf9f9': '\xef\xa7\xb9',
'MyanmarLetterTa': '\xe1\x80\x90',
'DevanagariLetterCa': '\xe0\xa4\x9a',
'UnicodeCharacter0xe7b6': '\xee\x9e\xb6',
'CjkUnifiedIdeograph7f26': '\xe7\xbc\xa6',
'KayahLiLetterSa': '\xea\xa4\x8e',
'UnicodeCharacter0xe7b4': '\xee\x9e\xb4',
'HangulSyllableTyeogg': '\xed\x85\xaa',
'UnicodeCharacter0xdddb': '\xed\xb7\x9b',
'UnicodeCharacter0xeb6a': '\xee\xad\xaa',
'KangxiRadicalVillage': '\xe2\xbe\xa5',
'CjkCompatibilityIdeographf9f8': '\xef\xa7\xb8',
'CjkUnifiedIdeograph9dff': '\xe9\xb7\xbf',
'CjkUnifiedIdeograph9dfe': '\xe9\xb7\xbe',
'CjkUnifiedIdeograph9dfd': '\xe9\xb7\xbd',
'CjkUnifiedIdeograph9dfc': '\xe9\xb7\xbc',
'CjkUnifiedIdeograph9dfb': '\xe9\xb7\xbb',
'CjkUnifiedIdeograph9dfa': '\xe9\xb7\xba',
'UnicodeCharacter0xe7b9': '\xee\x9e\xb9',
'HangulSyllableJjweh': '\xec\xae\x9b',
'UnicodeCharacter0xa62d': '\xea\x98\xad',
'FullwidthTilde': '\xef\xbd\x9e',
'HangulSyllableTen': '\xed\x85\x90',
'CjkCompatibilityIdeographfa5b': '\xef\xa9\x9b',
'HangulSyllableTyeogs': '\xed\x85\xab',
'CjkCompatibilityIdeographfa5a': '\xef\xa9\x9a',
'CjkCompatibilityIdeographfa5f': '\xef\xa9\x9f',
'StarAndCrescent': '\xe2\x98\xaa',
'CjkCompatibilityIdeographfa5d': '\xef\xa9\x9d',
'CjkCompatibilityIdeographfa5e': '\xef\xa9\x9e',
'HangulSyllableTem': '\xed\x85\x9c',
'UnicodeCharacter0xeba2': '\xee\xae\xa2',
'HangulSyllableJjwen': '\xec\xae\x84',
'UnicodeCharacter0xeba3': '\xee\xae\xa3',
'CjkUnifiedIdeograph4e3f': '\xe4\xb8\xbf',
'CjkUnifiedIdeograph4e3d': '\xe4\xb8\xbd',
'CjkUnifiedIdeograph4e3e': '\xe4\xb8\xbe',
'CjkUnifiedIdeograph4e3b': '\xe4\xb8\xbb',
'CjkUnifiedIdeograph4e3c': '\xe4\xb8\xbc',
'CjkUnifiedIdeograph4e3a': '\xe4\xb8\xba',
'CjkUnifiedIdeograph4f84': '\xe4\xbe\x84',
'CjkUnifiedIdeograph4f85': '\xe4\xbe\x85',
'CjkUnifiedIdeograph4f86': '\xe4\xbe\x86',
'CjkUnifiedIdeograph4f87': '\xe4\xbe\x87',
'CjkUnifiedIdeograph4f80': '\xe4\xbe\x80',
'CjkUnifiedIdeograph4f81': '\xe4\xbe\x81',
'CjkUnifiedIdeograph4f82': '\xe4\xbe\x82',
'CjkUnifiedIdeograph4f83': '\xe4\xbe\x83',
'UnicodeCharacter0xf05e': '\xef\x81\x9e',
'UnicodeCharacter0xe4aa': '\xee\x92\xaa',
'UnicodeCharacter0xe4ab': '\xee\x92\xab',
'UnicodeCharacter0xe4ac': '\xee\x92\xac',
'CjkUnifiedIdeograph4f88': '\xe4\xbe\x88',
'CjkUnifiedIdeograph4f89': '\xe4\xbe\x89',
'UnicodeCharacter0xe4af': '\xee\x92\xaf',
'UnicodeCharacter0xf05b': '\xef\x81\x9b',
'HangulSyllableJjweo': '\xec\xad\xa4',
'CjkCompatibilityIdeographfa53': '\xef\xa9\x93',
'CjkCompatibilityIdeographfa50': '\xef\xa9\x90',
'CjkCompatibilityIdeographfa51': '\xef\xa9\x91',
'CjkCompatibilityIdeographfa56': '\xef\xa9\x96',
'UnicodeCharacter0xeba4': '\xee\xae\xa4',
'CjkCompatibilityIdeographfa54': '\xef\xa9\x94',
'CjkCompatibilityIdeographfa55': '\xef\xa9\x95',
'CjkUnifiedIdeograph9603': '\xe9\x98\x83',
'CjkCompatibilityIdeographfa58': '\xef\xa9\x98',
'VaiSyllableKpan': '\xea\x95\x95',
'UnicodeCharacter0xeada': '\xee\xab\x9a',
'BugineseLetterVa': '\xe1\xa8\x93',
'YiRadicalOp': '\xea\x92\xa9',
'HebrewLetterGimel': '\xd7\x92',
'UnicodeCharacter0xe620': '\xee\x98\xa0',
'CjkUnifiedIdeograph4f8d': '\xe4\xbe\x8d',
'CjkUnifiedIdeograph4f8e': '\xe4\xbe\x8e',
'CjkUnifiedIdeograph4f8f': '\xe4\xbe\x8f',
'UnicodeCharacter0xeba9': '\xee\xae\xa9',
'UnicodeCharacter0xf059': '\xef\x81\x99',
'CjkUnifiedIdeograph4f8a': '\xe4\xbe\x8a',
'CjkUnifiedIdeograph4f8b': '\xe4\xbe\x8b',
'CjkUnifiedIdeograph4f8c': '\xe4\xbe\x8c',
'UnicodeCharacter0xe4a0': '\xee\x92\xa0',
'UnicodeCharacter0xe4a1': '\xee\x92\xa1',
'UnicodeCharacter0xe4a2': '\xee\x92\xa2',
'UnicodeCharacter0xe4a3': '\xee\x92\xa3',
'UnicodeCharacter0xe4a4': '\xee\x92\xa4',
'UnicodeCharacter0xe4a5': '\xee\x92\xa5',
'UnicodeCharacter0xe4a6': '\xee\x92\xa6',
'CjkUnifiedIdeograph8baa': '\xe8\xae\xaa',
'CjkUnifiedIdeograph4e38': '\xe4\xb8\xb8',
'CjkUnifiedIdeograph4e39': '\xe4\xb8\xb9',
'CjkUnifiedIdeograph4e36': '\xe4\xb8\xb6',
'CjkUnifiedIdeograph4e37': '\xe4\xb8\xb7',
'CjkUnifiedIdeograph4e34': '\xe4\xb8\xb4',
'CjkUnifiedIdeograph4e35': '\xe4\xb8\xb5',
'CjkUnifiedIdeograph4e32': '\xe4\xb8\xb2',
'CjkUnifiedIdeograph4e33': '\xe4\xb8\xb3',
'LatinCapitalLetterLWithCedilla': '\xc4\xbb',
'CjkUnifiedIdeograph4e31': '\xe4\xb8\xb1',
'HangulSyllableJjwae': '\xec\xab\xb4',
'CjkUnifiedIdeograph9f1d': '\xe9\xbc\x9d',
'CjkUnifiedIdeograph9f1e': '\xe9\xbc\x9e',
'CjkUnifiedIdeograph9f1f': '\xe9\xbc\x9f',
'CjkUnifiedIdeograph9f1a': '\xe9\xbc\x9a',
'HebrewAccentTelishaQetana': '\xd6\xa9',
'CjkUnifiedIdeograph9f1c': '\xe9\xbc\x9c',
'alpha': '\xce\xb1',
'UnicodeCharacter0xf1ae': '\xef\x86\xae',
'UnicodeCharacter0xa92': '\xe0\xaa\x92',
'HangulSyllableJjolm': '\xec\xab\x86',
'BraillePatternDots234568': '\xe2\xa2\xbe',
'TibetanVowelSignVocalicL': '\xe0\xbd\xb8',
'CjkUnifiedIdeograph57f9': '\xe5\x9f\xb9',
'CjkUnifiedIdeograph57f4': '\xe5\x9f\xb4',
'CjkUnifiedIdeograph57f5': '\xe5\x9f\xb5',
'CjkUnifiedIdeograph57f6': '\xe5\x9f\xb6',
'CjkUnifiedIdeograph57f7': '\xe5\x9f\xb7',
'CjkUnifiedIdeograph57f0': '\xe5\x9f\xb0',
'CjkUnifiedIdeograph57f1': '\xe5\x9f\xb1',
'BraillePatternDots234567': '\xe2\xa1\xbe',
'CjkUnifiedIdeograph57f3': '\xe5\x9f\xb3',
'UnicodeCharacter0xf587': '\xef\x96\x87',
'ArrowPointingDownwardsThenCurvingRightwards': '\xe2\xa4\xb7',
'UnicodeCharacter0xf585': '\xef\x96\x85',
'HangulSyllableMwilh': '\xeb\xae\x97',
'EthiopicSyllableBoa': '\xe2\xb6\x85',
'UnicodeCharacter0xf582': '\xef\x96\x82',
'HangulSyllableMwilm': '\xeb\xae\x92',
'UnicodeCharacter0xf580': '\xef\x96\x80',
'KannadaLetterDdha': '\xe0\xb2\xa2',
'HangulSyllableMwilb': '\xeb\xae\x93',
'TibetanVowelSignVocalicR': '\xe0\xbd\xb6',
'HangulSyllableMwilg': '\xeb\xae\x91',
'HangulSyllableJjolp': '\xec\xab\x8a',
'HangulSyllableJjols': '\xec\xab\x88',
'UnicodeCharacter0xf588': '\xef\x96\x88',
'HangulSyllableNaenh': '\xeb\x82\xba',
'UnicodeCharacter0xe7bb': '\xee\x9e\xbb',
'SquareRuuburu': '\xe3\x8d\x94',
'HangulSyllableCwilb': '\xec\xb7\xb3',
'YiSyllableShex': '\xea\x8e\xb7',
'HangulSyllableCwilg': '\xec\xb7\xb1',
'UnicodeCharacter0xe7bc': '\xee\x9e\xbc',
'CjkUnifiedIdeograph6f9d': '\xe6\xbe\x9d',
'HangulSyllableCwilh': '\xec\xb7\xb7',
'HangulSyllableCwilm': '\xec\xb7\xb2',
'HangulSyllableNaeng': '\xeb\x83\x89',
'KangxiRadicalHorn': '\xe2\xbe\x93',
'HangulSyllableCwilp': '\xec\xb7\xb6',
'HangulSyllableCwils': '\xec\xb7\xb4',
'CjkUnifiedIdeograph7e12': '\xe7\xb8\x92',
'CjkUnifiedIdeograph9f18': '\xe9\xbc\x98',
'CjkUnifiedIdeograph9f19': '\xe9\xbc\x99',
'CjkUnifiedIdeograph9f14': '\xe9\xbc\x94',
'CjkUnifiedIdeograph9f15': '\xe9\xbc\x95',
'CjkUnifiedIdeograph98aa': '\xe9\xa2\xaa',
'CjkUnifiedIdeograph9f17': '\xe9\xbc\x97',
'CjkUnifiedIdeograph9f10': '\xe9\xbc\x90',
'CjkUnifiedIdeograph9f11': '\xe9\xbc\x91',
'CjkUnifiedIdeograph9f12': '\xe9\xbc\x92',
'SquarePikuru': '\xe3\x8c\xaf',
'UnicodeCharacter0xf58f': '\xef\x96\x8f',
'RightwardsHarpoonWithBarbDownwards': '\xe2\x87\x81',
'YiSyllableRryt': '\xea\x8e\x8e',
'UnicodeCharacter0xf58c': '\xef\x96\x8c',
'YiSyllableRryr': '\xea\x8e\x93',
'UnicodeCharacter0xf58a': '\xef\x96\x8a',
'YiSyllableRryp': '\xea\x8e\x91',
'UnicodeCharacter0xe7be': '\xee\x9e\xbe',
'YiSyllableRryx': '\xea\x8e\x8f',
'HangulSyllableCwij': '\xec\xb7\xbe',
'subr': '\xe1\xb5\xa3',
'CjkUnifiedIdeograph57fd': '\xe5\x9f\xbd',
'CjkUnifiedIdeograph57fe': '\xe5\x9f\xbe',
'CjkUnifiedIdeograph57ff': '\xe5\x9f\xbf',
'CjkRadicalThread': '\xe2\xba\x93',
'CjkUnifiedIdeograph57fa': '\xe5\x9f\xba',
'CjkUnifiedIdeograph57fb': '\xe5\x9f\xbb',
'ThaiCharacterPhoPhung': '\xe0\xb8\x9c',
'HangulSyllablePalh': '\xed\x8c\x9b',
'HangulSyllableDdyeos': '\xeb\x97\xab',
'HangulSyllableSoels': '\xec\x87\xac',
'HangulSyllablePalb': '\xed\x8c\x97',
'UnicodeCharacter0x1f16': '\xe1\xbc\x96',
'UnicodeCharacter0x1f17': '\xe1\xbc\x97',
'LessthanWithQuestionMarkAbove': '\xe2\xa9\xbb',
'CjkUnifiedIdeograph8aa4': '\xe8\xaa\xa4',
'CjkUnifiedIdeograph8aa5': '\xe8\xaa\xa5',
'LeftWhiteParenthesis': '\xe2\xa6\x85',
'CjkUnifiedIdeograph8aa7': '\xe8\xaa\xa7',
'CjkUnifiedIdeograph8aa0': '\xe8\xaa\xa0',
'CjkUnifiedIdeograph8aa1': '\xe8\xaa\xa1',
'CjkUnifiedIdeograph8aa2': '\xe8\xaa\xa2',
'CjkUnifiedIdeograph8aa3': '\xe8\xaa\xa3',
'CjkUnifiedIdeograph466b': '\xe4\x99\xab',
'CjkUnifiedIdeograph466c': '\xe4\x99\xac',
'CjkUnifiedIdeograph466a': '\xe4\x99\xaa',
'CjkUnifiedIdeograph466f': '\xe4\x99\xaf',
'CjkUnifiedIdeograph8aa9': '\xe8\xaa\xa9',
'CjkUnifiedIdeograph466d': '\xe4\x99\xad',
'CjkUnifiedIdeograph466e': '\xe4\x99\xae',
'HangulSyllablePyags': '\xed\x8d\x87',
'HangulSyllableGyagg': '\xea\xb0\xba',
'UnicodeCharacter0xe77e': '\xee\x9d\xbe',
'UnicodeCharacter0xe77d': '\xee\x9d\xbd',
'UnicodeCharacter0xe77f': '\xee\x9d\xbf',
'UnicodeCharacter0xe77a': '\xee\x9d\xba',
'TifinaghLetterYagh': '\xe2\xb5\x96',
'UnicodeCharacter0xe77c': '\xee\x9d\xbc',
'UnicodeCharacter0xe77b': '\xee\x9d\xbb',
'UnicodeCharacter0xe71e': '\xee\x9c\x9e',
'HangulSyllableDdyeok': '\xeb\x97\xb0',
'HangulSyllablePyagg': '\xed\x8d\x86',
'HangulSyllableMyilt': '\xeb\xaf\xa9',
'HangulSyllableJjyaels': '\xec\xa8\xbc',
'CjkUnifiedIdeograph8aad': '\xe8\xaa\xad',
'CjkUnifiedIdeograph8aae': '\xe8\xaa\xae',
'CjkUnifiedIdeograph4668': '\xe4\x99\xa8',
'CjkUnifiedIdeograph4669': '\xe4\x99\xa9',
'CjkUnifiedIdeograph8aaa': '\xe8\xaa\xaa',
'CjkUnifiedIdeograph8aab': '\xe8\xaa\xab',
'CjkUnifiedIdeograph8aac': '\xe8\xaa\xac',
'CjkUnifiedIdeograph4662': '\xe4\x99\xa2',
'CjkUnifiedIdeograph4663': '\xe4\x99\xa3',
'CjkUnifiedIdeograph4660': '\xe4\x99\xa0',
'CjkUnifiedIdeograph4661': '\xe4\x99\xa1',
'CjkUnifiedIdeograph4666': '\xe4\x99\xa6',
'CjkUnifiedIdeograph4667': '\xe4\x99\xa7',
'CjkUnifiedIdeograph4664': '\xe4\x99\xa4',
'CjkUnifiedIdeograph4665': '\xe4\x99\xa5',
'UnicodeCharacter0xf1ac': '\xef\x86\xac',
'MongolianLetterDa': '\xe1\xa0\xb3',
'HangulSyllablePwabs': '\xed\x90\x9a',
'UnicodeCharacter0x1f1f': '\xe1\xbc\x9f',
'DoubleLeftArcGreaterthanBracket': '\xe2\xa6\x95',
'UnicodeCharacter0x1f1e': '\xe1\xbc\x9e',
'UnicodeCharacter0xf6bd': '\xef\x9a\xbd',
'HangulSyllableDdyeoc': '\xeb\x97\xaf',
'CjkCompatibilityIdeographfa35': '\xef\xa8\xb5',
'UnicodeCharacter0xf859': '\xef\xa1\x99',
'UnicodeCharacter0xf858': '\xef\xa1\x98',
'UnicodeCharacter0xe779': '\xee\x9d\xb9',
'UnicodeCharacter0xe778': '\xee\x9d\xb8',
'UnicodeCharacter0xe775': '\xee\x9d\xb5',
'UnicodeCharacter0xe774': '\xee\x9d\xb4',
'UnicodeCharacter0xe777': '\xee\x9d\xb7',
'UnicodeCharacter0xe776': '\xee\x9d\xb6',
'UnicodeCharacter0xe771': '\xee\x9d\xb1',
'UnicodeCharacter0xe770': '\xee\x9d\xb0',
'UnicodeCharacter0xe773': '\xee\x9d\xb3',
'UnicodeCharacter0xe772': '\xee\x9d\xb2',
'YiSyllableGo': '\xea\x87\xac',
'ArmenianSmallLetterEch': '\xd5\xa5',
'HotBeverage': '\xe2\x98\x95',
'YiSyllableGi': '\xea\x87\x9c',
'SquareEekaa': '\xe3\x8c\x88',
'CombiningLeftArrowAbove': '\xe2\x83\x96',
'CjkUnifiedIdeograph5fdf': '\xe5\xbf\x9f',
'CjkUnifiedIdeograph5fde': '\xe5\xbf\x9e',
'CjkUnifiedIdeograph5fdd': '\xe5\xbf\x9d',
'CjkUnifiedIdeograph5fdc': '\xe5\xbf\x9c',
'CjkUnifiedIdeograph5fdb': '\xe5\xbf\x9b',
'CjkUnifiedIdeograph5fda': '\xe5\xbf\x9a',
'CjkUnifiedIdeograph8f93': '\xe8\xbe\x93',
'TeluguLetterDdha': '\xe0\xb0\xa2',
'CjkUnifiedIdeograph8f91': '\xe8\xbe\x91',
'CjkUnifiedIdeograph8f90': '\xe8\xbe\x90',
'CjkUnifiedIdeograph8f97': '\xe8\xbe\x97',
'CjkUnifiedIdeograph8f96': '\xe8\xbe\x96',
'CjkUnifiedIdeograph8f95': '\xe8\xbe\x95',
'CjkUnifiedIdeograph8f94': '\xe8\xbe\x94',
'CjkUnifiedIdeograph962a': '\xe9\x98\xaa',
'YiSyllableZhet': '\xea\x8d\x91',
'CjkUnifiedIdeograph8f99': '\xe8\xbe\x99',
'CjkUnifiedIdeograph8f98': '\xe8\xbe\x98',
'CjkUnifiedIdeograph962e': '\xe9\x98\xae',
'CjkUnifiedIdeograph962d': '\xe9\x98\xad',
'CjkUnifiedIdeograph962f': '\xe9\x98\xaf',
'TeluguDigitThree': '\xe0\xb1\xa9',
'BopomofoFinalLetterT': '\xe3\x86\xb5',
'BopomofoFinalLetterP': '\xe3\x86\xb4',
'CjkUnifiedIdeograph513a': '\xe5\x84\xba',
'UppointingTriangleWithLeftHalfBlack': '\xe2\x97\xad',
'CjkUnifiedIdeograph513c': '\xe5\x84\xbc',
'CjkUnifiedIdeograph513b': '\xe5\x84\xbb',
'CjkUnifiedIdeograph513e': '\xe5\x84\xbe',
'CjkUnifiedIdeograph513d': '\xe5\x84\xbd',
'UnicodeCharacter0xf87a': '\xef\xa1\xba',
'CjkUnifiedIdeograph513f': '\xe5\x84\xbf',
'HangulSyllableJjeong': '\xec\xa9\xa1',
'UnicodeCharacter0xf670': '\xef\x99\xb0',
'HangulSyllableDyeolt': '\xeb\x8e\x99',
'HangulSyllableMyilb': '\xeb\xaf\xa7',
'BopomofoFinalLetterK': '\xe3\x86\xb6',
'HangulSyllableJjeonh': '\xec\xa9\x92',
'Nor': '\xe2\x8a\xbd',
'BopomofoFinalLetterH': '\xe3\x86\xb7',
'CjkUnifiedIdeograph8f9c': '\xe8\xbe\x9c',
'UnicodeCharacter0x180f': '\xe1\xa0\x8f',
'CjkUnifiedIdeograph8f9a': '\xe8\xbe\x9a',
'CjkUnifiedIdeograph8f9f': '\xe8\xbe\x9f',
'CjkUnifiedIdeograph8f9e': '\xe8\xbe\x9e',
'CjkUnifiedIdeograph8f9d': '\xe8\xbe\x9d',
'CjkUnifiedIdeograph9621': '\xe9\x98\xa1',
'CjkUnifiedIdeograph9620': '\xe9\x98\xa0',
'CjkUnifiedIdeograph9623': '\xe9\x98\xa3',
'CjkUnifiedIdeograph9622': '\xe9\x98\xa2',
'CjkUnifiedIdeograph9625': '\xe9\x98\xa5',
'CjkUnifiedIdeograph9624': '\xe9\x98\xa4',
'CjkUnifiedIdeograph9627': '\xe9\x98\xa7',
'CjkUnifiedIdeograph9626': '\xe9\x98\xa6',
'UnicodeCharacter0xf673': '\xef\x99\xb3',
'RejangVirama': '\xea\xa5\x93',
'CjkUnifiedIdeograph5fd9': '\xe5\xbf\x99',
'CjkUnifiedIdeograph5fd8': '\xe5\xbf\x98',
'CjkUnifiedIdeograph5fd7': '\xe5\xbf\x97',
'CjkUnifiedIdeograph5fd6': '\xe5\xbf\x96',
'CjkUnifiedIdeograph5fd5': '\xe5\xbf\x95',
'CjkUnifiedIdeograph5fd4': '\xe5\xbf\x94',
'CjkUnifiedIdeograph5fd3': '\xe5\xbf\x93',
'CjkUnifiedIdeograph5fd2': '\xe5\xbf\x92',
'HexagramForBeforeCompletion': '\xe4\xb7\xbf',
'CjkUnifiedIdeograph5fd0': '\xe5\xbf\x90',
'CjkUnifiedIdeograph5f76': '\xe5\xbd\xb6',
'UnicodeCharacter0xdde0': '\xed\xb7\xa0',
'UnicodeCharacter0xf369': '\xef\x8d\xa9',
'GreekSmallLetterIotaWithPsili': '\xe1\xbc\xb0',
'UnicodeCharacter0xed78': '\xee\xb5\xb8',
'UnicodeCharacter0xdde6': '\xed\xb7\xa6',
'CjkUnifiedIdeograph5139': '\xe5\x84\xb9',
'CjkUnifiedIdeograph5138': '\xe5\x84\xb8',
'HangulSyllableJyaeng': '\xec\x9f\xb9',
'Uranus': '\xe2\x99\x85',
'UnicodeCharacter0xf878': '\xef\xa1\xb8',
'CjkUnifiedIdeograph5131': '\xe5\x84\xb1',
'CjkUnifiedIdeograph5130': '\xe5\x84\xb0',
'CjkUnifiedIdeograph5133': '\xe5\x84\xb3',
'CjkUnifiedIdeograph5132': '\xe5\x84\xb2',
'ParenthesizedHangulChieuch': '\xe3\x88\x89',
'CjkUnifiedIdeograph5134': '\xe5\x84\xb4',
'CjkUnifiedIdeograph5137': '\xe5\x84\xb7',
'CjkUnifiedIdeograph5136': '\xe5\x84\xb6',
'TibetanDigitHalfNine': '\xe0\xbc\xb2',
'LatinCapitalLetterZWithCaron': '\xc5\xbd',
'TamilDigitEight': '\xe0\xaf\xae',
'UnicodeCharacter0xf7ec': '\xef\x9f\xac',
'PresentationFormForVerticalIdeographicFullStop': '\xef\xb8\x92',
'CjkUnifiedIdeograph347f': '\xe3\x91\xbf',
'CjkUnifiedIdeograph4cb7': '\xe4\xb2\xb7',
'CjkUnifiedIdeograph347d': '\xe3\x91\xbd',
'CjkUnifiedIdeograph347e': '\xe3\x91\xbe',
'CjkUnifiedIdeograph347b': '\xe3\x91\xbb',
'CjkUnifiedIdeograph347c': '\xe3\x91\xbc',
'CjkUnifiedIdeograph347a': '\xe3\x91\xba',
'HangulSyllableJyaenh': '\xec\x9f\xaa',
'ArmenianSmallLetterSha': '\xd5\xb7',
'UnicodeCharacter0xfbc8': '\xef\xaf\x88',
'MongolianLetterKha': '\xe1\xa0\xbb',
'HangulSyllableJyaenj': '\xec\x9f\xa9',
'UnicodeCharacter0xe55c': '\xee\x95\x9c',
'LatinSmallLetterAa': '\xea\x9c\xb3',
'LatinSmallLetterAe': '\xc3\xa6',
'LatinSmallLetterAo': '\xea\x9c\xb5',
'varsigma': '\xcf\x82',
'HangulSyllableGgaej': '\xea\xb9\xbe',
'LatinSmallLetterAv': '\xea\x9c\xb9',
'LatinSmallLetterAu': '\xea\x9c\xb7',
'UnicodeCharacter0xe55b': '\xee\x95\x9b',
'LatinSmallLetterAy': '\xea\x9c\xbd',
'GreekCapitalLetterSigma': '\xce\xa3',
'CjkUnifiedIdeograph3476': '\xe3\x91\xb6',
'CjkUnifiedIdeograph3477': '\xe3\x91\xb7',
'CjkUnifiedIdeograph3474': '\xe3\x91\xb4',
'CjkUnifiedIdeograph3475': '\xe3\x91\xb5',
'CjkUnifiedIdeograph3472': '\xe3\x91\xb2',
'CjkUnifiedIdeograph3473': '\xe3\x91\xb3',
'CjkUnifiedIdeograph3470': '\xe3\x91\xb0',
'CjkUnifiedIdeograph3471': '\xe3\x91\xb1',
'CjkUnifiedIdeograph3478': '\xe3\x91\xb8',
'CjkUnifiedIdeograph3479': '\xe3\x91\xb9',
'BlackCircleWithWhiteDotRight': '\xe2\x9a\x88',
'UnicodeCharacter0xf679': '\xef\x99\xb9',
'HangulSyllableKyelb': '\xec\xbd\x83',
'CjkUnifiedIdeograph97bd': '\xe9\x9e\xbd',
'CjkUnifiedIdeograph97be': '\xe9\x9e\xbe',
'CjkUnifiedIdeograph97bf': '\xe9\x9e\xbf',
'CjkUnifiedIdeograph97ba': '\xe9\x9e\xba',
'CjkUnifiedIdeograph97bb': '\xe9\x9e\xbb',
'CjkUnifiedIdeograph97bc': '\xe9\x9e\xbc',
'CanadianSyllabicsWoo': '\xe1\x90\x94',
'AplFunctionalSymbolQuadEqual': '\xe2\x8c\xb8',
'HangulSyllableNyals': '\xeb\x83\x9c',
'DoesNotForce': '\xe2\x8a\xae',
'RomanNumeralFour': '\xe2\x85\xa3',
'UnicodeCharacter0xf678': '\xef\x99\xb8',
'HangulSyllableCut': '\xec\xb6\xad',
'HangulSyllableCup': '\xec\xb6\xae',
'HangulSyllableCus': '\xec\xb6\xa7',
'HangulSyllableCul': '\xec\xb6\x9c',
'HangulSyllableCum': '\xec\xb6\xa4',
'HangulSyllableCun': '\xec\xb6\x98',
'UnicodeCharacter0xdc62': '\xed\xb1\xa2',
'HangulSyllableCuh': '\xec\xb6\xaf',
'DevanagariVowelSignUu': '\xe0\xa5\x82',
'HangulSyllableCuj': '\xec\xb6\xaa',
'HangulSyllableCuk': '\xec\xb6\xac',
'HangulSyllableCud': '\xec\xb6\x9b',
'CjkUnifiedIdeograph8c3a': '\xe8\xb0\xba',
'CjkUnifiedIdeograph8c3b': '\xe8\xb0\xbb',
'CjkUnifiedIdeograph8c3c': '\xe8\xb0\xbc',
'CjkUnifiedIdeograph8c3d': '\xe8\xb0\xbd',
'CjkUnifiedIdeograph8c3e': '\xe8\xb0\xbe',
'CjkUnifiedIdeograph8c3f': '\xe8\xb0\xbf',
'HangulSyllableCuc': '\xec\xb6\xab',
'HangulSyllableRyelg': '\xeb\xa1\x89',
'UnicodeCharacter0xf147': '\xef\x85\x87',
'HangulSyllableRyelb': '\xeb\xa1\x8b',
'ArabicLigatureUighurKirghizYehWithHamzaAboveWithAlefMaksuraInitialForm': '\xef\xaf\xbb',
'LowerLeftSemicircularAnticlockwiseArrow': '\xe2\xa4\xbf',
'HangulSyllableRyelm': '\xeb\xa1\x8a',
'UnicodeCharacter0xe627': '\xee\x98\xa7',
'HangulSyllableRyelh': '\xeb\xa1\x8f',
'HangulSyllableRyelt': '\xeb\xa1\x8d',
'CjkUnifiedIdeograph97b8': '\xe9\x9e\xb8',
'CjkUnifiedIdeograph97b9': '\xe9\x9e\xb9',
'HangulSyllableRyelp': '\xeb\xa1\x8e',
'CjkUnifiedIdeograph97b4': '\xe9\x9e\xb4',
'CjkUnifiedIdeograph97b5': '\xe9\x9e\xb5',
'CjkUnifiedIdeograph97b6': '\xe9\x9e\xb6',
'CjkUnifiedIdeograph97b7': '\xe9\x9e\xb7',
'CjkUnifiedIdeograph97b0': '\xe9\x9e\xb0',
'CjkUnifiedIdeograph97b1': '\xe9\x9e\xb1',
'CjkUnifiedIdeograph97b2': '\xe9\x9e\xb2',
'CjkUnifiedIdeograph97b3': '\xe9\x9e\xb3',
'CjkUnifiedIdeograph8c38': '\xe8\xb0\xb8',
'CjkUnifiedIdeograph8c39': '\xe8\xb0\xb9',
'UnicodeCharacter0xdc6c': '\xed\xb1\xac',
'YiSyllableNjip': '\xea\x90\xb4',
'UnicodeCharacter0xdc6e': '\xed\xb1\xae',
'UnicodeCharacter0xdc6d': '\xed\xb1\xad',
'NonbreakingHyphen': '\xe2\x80\x91',
'CjkUnifiedIdeograph8c30': '\xe8\xb0\xb0',
'CjkUnifiedIdeograph8c31': '\xe8\xb0\xb1',
'CjkUnifiedIdeograph8c32': '\xe8\xb0\xb2',
'CjkUnifiedIdeograph8c33': '\xe8\xb0\xb3',
'CjkUnifiedIdeograph8c34': '\xe8\xb0\xb4',
'CjkUnifiedIdeograph8c35': '\xe8\xb0\xb5',
'CjkUnifiedIdeograph8c36': '\xe8\xb0\xb6',
'CjkUnifiedIdeograph8c37': '\xe8\xb0\xb7',
'BoxDrawingsLeftLightAndRightDownHeavy': '\xe2\x94\xb2',
'YiSyllableNjie': '\xea\x90\xb7',
'UnicodeCharacter0xddeb': '\xed\xb7\xab',
'PhagspaLetterGga': '\xea\xa1\xa5',
'CjkUnifiedIdeograph81e8': '\xe8\x87\xa8',
'CjkUnifiedIdeograph81e9': '\xe8\x87\xa9',
'CjkCompatibilityIdeographf952': '\xef\xa5\x92',
'CjkUnifiedIdeograph788c': '\xe7\xa2\x8c',
'CjkUnifiedIdeograph788b': '\xe7\xa2\x8b',
'CjkUnifiedIdeograph788a': '\xe7\xa2\x8a',
'CjkUnifiedIdeograph81e3': '\xe8\x87\xa3',
'CjkUnifiedIdeograph81e4': '\xe8\x87\xa4',
'CjkUnifiedIdeograph788f': '\xe7\xa2\x8f',
'CjkUnifiedIdeograph788e': '\xe7\xa2\x8e',
'CjkUnifiedIdeograph788d': '\xe7\xa2\x8d',
'LowerHalfBlock': '\xe2\x96\x84',
'CjkUnifiedIdeograph595c': '\xe5\xa5\x9c',
'CjkUnifiedIdeograph595b': '\xe5\xa5\x9b',
'CjkUnifiedIdeograph595a': '\xe5\xa5\x9a',
'CjkUnifiedIdeograph595f': '\xe5\xa5\x9f',
'CjkUnifiedIdeograph595e': '\xe5\xa5\x9e',
'CjkUnifiedIdeograph595d': '\xe5\xa5\x9d',
'UnicodeCharacter0xdb28': '\xed\xac\xa8',
'KangxiRadicalFrog': '\xe2\xbf\x8c',
'CjkUnifiedIdeograph35ea': '\xe3\x97\xaa',
'CjkUnifiedIdeograph35ec': '\xe3\x97\xac',
'CjkUnifiedIdeograph35eb': '\xe3\x97\xab',
'CjkUnifiedIdeograph35ee': '\xe3\x97\xae',
'CjkUnifiedIdeograph35ed': '\xe3\x97\xad',
'CjkUnifiedIdeograph35ef': '\xe3\x97\xaf',
'LatinSmallLetterIWithGrave': '\xc3\xac',
'LatinCapitalLetterRWithDotAbove': '\xe1\xb9\x98',
'HangulSyllableGwebs': '\xea\xb6\xb6',
'BoxDrawingsDownHeavyAndRightUpLight': '\xe2\x94\x9f',
'CjkUnifiedIdeograph5959': '\xe5\xa5\x99',
'BoxDrawingsUpLightAndDownHorizontalHeavy': '\xe2\x95\x88',
'CjkUnifiedIdeograph5953': '\xe5\xa5\x93',
'CjkUnifiedIdeograph5952': '\xe5\xa5\x92',
'HangulJongseongRieulpieuphieuh': '\xe1\x87\x94',
'CjkUnifiedIdeograph5950': '\xe5\xa5\x90',
'CjkUnifiedIdeograph5957': '\xe5\xa5\x97',
'CjkUnifiedIdeograph5956': '\xe5\xa5\x96',
'CjkUnifiedIdeograph5955': '\xe5\xa5\x95',
'CjkUnifiedIdeograph5954': '\xe5\xa5\x94',
'CjkUnifiedIdeograph8341': '\xe8\x8d\x81',
'CjkUnifiedIdeograph7889': '\xe7\xa2\x89',
'CjkUnifiedIdeograph7888': '\xe7\xa2\x88',
'CjkUnifiedIdeograph7883': '\xe7\xa2\x83',
'CjkUnifiedIdeograph7882': '\xe7\xa2\x82',
'CjkUnifiedIdeograph7881': '\xe7\xa2\x81',
'CjkUnifiedIdeograph7880': '\xe7\xa2\x80',
'CjkUnifiedIdeograph7887': '\xe7\xa2\x87',
'CjkUnifiedIdeograph7886': '\xe7\xa2\x86',
'CjkUnifiedIdeograph7885': '\xe7\xa2\x85',
'ShortLeftTack': '\xe2\xab\x9e',
'UnicodeCharacter0xe626': '\xee\x98\xa6',
'HangulSyllableCeu': '\xec\xb8\xa0',
'CjkUnifiedIdeograph35e9': '\xe3\x97\xa9',
'CjkUnifiedIdeograph35e8': '\xe3\x97\xa8',
'UnicodeCharacter0xdafc': '\xed\xab\xbc',
'BraillePatternDots12568': '\xe2\xa2\xb3',
'CjkUnifiedIdeograph35e1': '\xe3\x97\xa1',
'CjkUnifiedIdeograph35e0': '\xe3\x97\xa0',
'CjkUnifiedIdeograph35e3': '\xe3\x97\xa3',
'CjkUnifiedIdeograph35e2': '\xe3\x97\xa2',
'CjkUnifiedIdeograph35e5': '\xe3\x97\xa5',
'CjkUnifiedIdeograph35e4': '\xe3\x97\xa4',
'CjkUnifiedIdeograph35e7': '\xe3\x97\xa7',
'CjkUnifiedIdeograph35e6': '\xe3\x97\xa6',
'UnicodeCharacter0xeec7': '\xee\xbb\x87',
'HangulSyllableSeolm': '\xec\x84\xa6',
'UnicodeCharacter0xeec5': '\xee\xbb\x85',
'UnicodeCharacter0xeec4': '\xee\xbb\x84',
'UnicodeCharacter0xeec3': '\xee\xbb\x83',
'UnicodeCharacter0xeec2': '\xee\xbb\x82',
'UnicodeCharacter0xeec1': '\xee\xbb\x81',
'UnicodeCharacter0xeec0': '\xee\xbb\x80',
'ApproximatelyEqualToOrTheImageOf': '\xe2\x89\x92',
'UnicodeCharacter0xeec9': '\xee\xbb\x89',
'UnicodeCharacter0xeec8': '\xee\xbb\x88',
'VaiSyllableGen': '\xea\x98\x87',
'UnicodeCharacter0xdafb': '\xed\xab\xbb',
'HangulSyllableSeolb': '\xec\x84\xa7',
'VaiSyllableGee': '\xea\x94\xa0',
'MultisetMultiplication': '\xe2\x8a\x8d',
'LatinCapitalLetterYWithHookAbove': '\xe1\xbb\xb6',
'CjkUnifiedIdeograph8342': '\xe8\x8d\x82',
'UnicodeCharacter0xdafa': '\xed\xab\xba',
'HangulSyllableCyael': '\xec\xb2\x84',
'HangulSyllableCyaem': '\xec\xb2\x8c',
'HangulJungseongUye': '\xe1\x86\x8c',
'UnicodeCharacter0xfdec': '\xef\xb7\xac',
'HangulSyllableSeolg': '\xec\x84\xa5',
'CircledIdeographFinancial': '\xe3\x8a\x96',
'DownwardsTwoHeadedArrow': '\xe2\x86\xa1',
'CjkUnifiedIdeograph89b2': '\xe8\xa6\xb2',
'ArabicLigatureKhahWithJeemIsolatedForm': '\xef\xb0\x99',
'VaiSyllableBhee': '\xea\x94\x86',
'HangulSyllableCyaeh': '\xec\xb2\x97',
'ChamDigitFour': '\xea\xa9\x94',
'UnicodeCharacter0xeecf': '\xee\xbb\x8f',
'UnicodeCharacter0xeece': '\xee\xbb\x8e',
'HangulSyllableBbyeogg': '\xeb\xbc\x8a',
'UnicodeCharacter0xeecc': '\xee\xbb\x8c',
'HangulSyllableCyaej': '\xec\xb2\x92',
'UnicodeCharacter0xeeca': '\xee\xbb\x8a',
'UnicodeCharacter0xfdeb': '\xef\xb7\xab',
'HebrewLigatureYiddishDoubleYod': '\xd7\xb2',
'LatinCapitalLetterLWithDoubleBar': '\xe2\xb1\xa0',
'BoxDrawingsUpHeavyAndRightDownLight': '\xe2\x94\x9e',
'Integral': '\xe2\x88\xab',
'UnicodeCharacter0xe0a7': '\xee\x82\xa7',
'MongolianFreeVariationSelectorOne': '\xe1\xa0\x8b',
'HangulSyllableSwibs': '\xec\x89\xbe',
'SquareMaikuro': '\xe3\x8d\x83',
'UnicodeCharacter0xe0a1': '\xee\x82\xa1',
'UnicodeCharacter0xe0a2': '\xee\x82\xa2',
'HangulSyllableJwaegg': '\xec\xa2\xaa',
'HangulSyllableJjoels': '\xec\xac\x9c',
'CjkUnifiedIdeograph3450': '\xe3\x91\x90',
'HangulSyllableJjoelp': '\xec\xac\x9e',
'HangulSyllableJjoelt': '\xec\xac\x9d',
'HangulSyllableJwaegs': '\xec\xa2\xab',
'HangulSyllableJjoelh': '\xec\xac\x9f',
'HangulSyllableJjoelm': '\xec\xac\x9a',
'HangulSyllableJjoelb': '\xec\xac\x9b',
'HangulSyllableJjoelg': '\xec\xac\x99',
'UnicodeCharacter0xe5db': '\xee\x97\x9b',
'HangulSyllableDdyil': '\xeb\x9d\x8c',
'HangulSyllableDdyim': '\xeb\x9d\x94',
'HangulSyllableDdyin': '\xeb\x9d\x88',
'HangulSyllableDdyih': '\xeb\x9d\x9f',
'UnicodeCharacter0xdc04': '\xed\xb0\x84',
'HangulSyllableDdyij': '\xeb\x9d\x9a',
'HangulSyllableDdyik': '\xeb\x9d\x9c',
'HangulSyllableDdyid': '\xeb\x9d\x8b',
'UnicodeCharacter0xe0a8': '\xee\x82\xa8',
'HangulSyllableDdyig': '\xeb\x9d\x85',
'HangulSyllableDdyib': '\xeb\x9d\x95',
'HangulSyllableDdyic': '\xeb\x9d\x9b',
'HangulSyllableDdyit': '\xeb\x9d\x9d',
'DevanagariLetterJha': '\xe0\xa4\x9d',
'HangulSyllableDdyip': '\xeb\x9d\x9e',
'CjkUnifiedIdeograph7059': '\xe7\x81\x99',
'CjkUnifiedIdeograph8f4d': '\xe8\xbd\x8d',
'CjkUnifiedIdeograph8f4e': '\xe8\xbd\x8e',
'ExtendedArabicindicDigitFive': '\xdb\xb5',
'CjkUnifiedIdeograph9014': '\xe9\x80\x94',
'UnicodeCharacter0xf6be': '\xef\x9a\xbe',
'UnicodeCharacter0xfdef': '\xef\xb7\xaf',
'CombiningSeagullBelow': '\xcc\xbc',
'MyanmarVowelSignShanFinalY': '\xe1\x82\x86',
'CjkUnifiedIdeograph9012': '\xe9\x80\x92',
'YiRadicalBbut': '\xea\x92\x9b',
'CjkUnifiedIdeograph9013': '\xe9\x80\x93',
'CjkUnifiedIdeograph8f4b': '\xe8\xbd\x8b',
'CjkUnifiedIdeograph8f4c': '\xe8\xbd\x8c',
'ApproximatelyButNotActuallyEqualTo': '\xe2\x89\x86',
'!<<->>': '\xe2\x87\x8e',
'CyrillicSmallLetterLittleYus': '\xd1\xa7',
'SundaneseLetterAe': '\xe1\xae\x86',
'UnicodeCharacter0xc5f': '\xe0\xb1\x9f',
'UnicodeCharacter0xc5e': '\xe0\xb1\x9e',
'UnicodeCharacter0xc5d': '\xe0\xb1\x9d',
'UnicodeCharacter0xc5c': '\xe0\xb1\x9c',
'UnicodeCharacter0xc5b': '\xe0\xb1\x9b',
'UnicodeCharacter0xc5a': '\xe0\xb1\x9a',
'HangulSyllableKwaebs': '\xec\xbe\x9e',
'YiSyllableJjyt': '\xea\x90\xad',
'HangulSyllableRaeng': '\xeb\x9e\xad',
'CjkUnifiedIdeograph7aa5': '\xe7\xaa\xa5',
'CjkUnifiedIdeograph7aa4': '\xe7\xaa\xa4',
'CjkUnifiedIdeograph7aa7': '\xe7\xaa\xa7',
'CjkUnifiedIdeograph7aa6': '\xe7\xaa\xa6',
'CjkUnifiedIdeograph7aa1': '\xe7\xaa\xa1',
'CjkUnifiedIdeograph7aa0': '\xe7\xaa\xa0',
'CjkUnifiedIdeograph7aa3': '\xe7\xaa\xa3',
'CjkUnifiedIdeograph7aa2': '\xe7\xaa\xa2',
'HalfwidthKatakanaLetterRe': '\xef\xbe\x9a',
'CjkUnifiedIdeograph7aa9': '\xe7\xaa\xa9',
'CjkUnifiedIdeograph7aa8': '\xe7\xaa\xa8',
'CjkUnifiedIdeograph6378': '\xe6\x8d\xb8',
'CjkUnifiedIdeograph6379': '\xe6\x8d\xb9',
'CjkUnifiedIdeograph6376': '\xe6\x8d\xb6',
'CjkUnifiedIdeograph6377': '\xe6\x8d\xb7',
'CjkUnifiedIdeograph6374': '\xe6\x8d\xb4',
'CjkUnifiedIdeograph6375': '\xe6\x8d\xb5',
'CjkUnifiedIdeograph6372': '\xe6\x8d\xb2',
'CjkUnifiedIdeograph6373': '\xe6\x8d\xb3',
'CjkUnifiedIdeograph6370': '\xe6\x8d\xb0',
'CjkUnifiedIdeograph6371': '\xe6\x8d\xb1',
'CircleWithAllButUpperLeftQuadrantBlack': '\xe2\x97\x95',
'CjkUnifiedIdeograph9ad4': '\xe9\xab\x94',
'CjkUnifiedIdeograph9ad5': '\xe9\xab\x95',
'UnicodeCharacter0xc57': '\xe0\xb1\x97',
'UnicodeCharacter0xc54': '\xe0\xb1\x94',
'UnicodeCharacter0xc53': '\xe0\xb1\x93',
'UnicodeCharacter0xc52': '\xe0\xb1\x92',
'UnicodeCharacter0xc51': '\xe0\xb1\x91',
'UnicodeCharacter0xc50': '\xe0\xb1\x90',
'ArabicLetterTchehInitialForm': '\xef\xad\xbc',
'CjkUnifiedIdeograph9ad7': '\xe9\xab\x97',
'CyrillicCapitalLetterElWithHook': '\xd4\x92',
'CjkUnifiedIdeograph9ad0': '\xe9\xab\x90',
'CjkUnifiedIdeograph637d': '\xe6\x8d\xbd',
'CjkUnifiedIdeograph637e': '\xe6\x8d\xbe',
'CjkUnifiedIdeograph637b': '\xe6\x8d\xbb',
'CjkUnifiedIdeograph637c': '\xe6\x8d\xbc',
'CjkUnifiedIdeograph637a': '\xe6\x8d\xba',
'CjkUnifiedIdeograph7aae': '\xe7\xaa\xae',
'CjkUnifiedIdeograph7aad': '\xe7\xaa\xad',
'HexagramForTheCauldron': '\xe4\xb7\xb1',
'CjkUnifiedIdeograph7aaa': '\xe7\xaa\xaa',
'CjkUnifiedIdeograph9ad2': '\xe9\xab\x92',
'CjkUnifiedIdeograph7aac': '\xe7\xaa\xac',
'CjkUnifiedIdeograph7aab': '\xe7\xaa\xab',
'KayahLiSignShya': '\xea\xa4\xaf',
'CjkUnifiedIdeograph9ad3': '\xe9\xab\x93',
'CjkUnifiedIdeograph42d4': '\xe4\x8b\x94',
'CjkUnifiedIdeograph42d5': '\xe4\x8b\x95',
'CjkUnifiedIdeograph42d6': '\xe4\x8b\x96',
'CjkUnifiedIdeograph42d7': '\xe4\x8b\x97',
'CjkUnifiedIdeograph42d0': '\xe4\x8b\x90',
'CjkUnifiedIdeograph42d1': '\xe4\x8b\x91',
'CjkUnifiedIdeograph42d2': '\xe4\x8b\x92',
'CjkUnifiedIdeograph42d3': '\xe4\x8b\x93',
'CjkUnifiedIdeograph42d8': '\xe4\x8b\x98',
'CjkUnifiedIdeograph42d9': '\xe4\x8b\x99',
'WhiteParallelogram': '\xe2\x96\xb1',
'UnicodeCharacter0xf730': '\xef\x9c\xb0',
'HangulSyllableBanh': '\xeb\xb0\x9a',
'UnicodeCharacter0xe0aa': '\xee\x82\xaa',
'HangulSyllableBbyalt': '\xeb\xba\xa5',
'HangulSyllableBbyalp': '\xeb\xba\xa6',
'HangulSyllableBbyals': '\xeb\xba\xa4',
'HangulSyllableByogs': '\xeb\xb5\xa7',
'SymbolForCancel': '\xe2\x90\x98',
'CjkUnifiedIdeograph7ba7': '\xe7\xae\xa7',
'CjkUnifiedIdeograph7ba4': '\xe7\xae\xa4',
'CjkUnifiedIdeograph7ba5': '\xe7\xae\xa5',
'CjkUnifiedIdeograph7ba2': '\xe7\xae\xa2',
'CjkUnifiedIdeograph7ba3': '\xe7\xae\xa3',
'CjkUnifiedIdeograph7ba0': '\xe7\xae\xa0',
'CjkUnifiedIdeograph7ba1': '\xe7\xae\xa1',
'HangulSyllableBbyalm': '\xeb\xba\xa2',
'CjkRadicalSmallTwo': '\xe2\xba\x8d',
'HangulSyllableBbyalh': '\xeb\xba\xa7',
'CjkUnifiedIdeograph7ba8': '\xe7\xae\xa8',
'CjkUnifiedIdeograph7ba9': '\xe7\xae\xa9',
'GeorgianLetterJil': '\xe1\x83\xab',
'HangulSyllableJyegg': '\xec\xa1\x96',
'CjkUnifiedIdeograph42dd': '\xe4\x8b\x9d',
'CjkUnifiedIdeograph42de': '\xe4\x8b\x9e',
'CjkUnifiedIdeograph42df': '\xe4\x8b\x9f',
'CjkUnifiedIdeograph42da': '\xe4\x8b\x9a',
'CjkUnifiedIdeograph42db': '\xe4\x8b\x9b',
'CjkUnifiedIdeograph42dc': '\xe4\x8b\x9c',
'HangulSyllableSoet': '\xec\x87\xb9',
'HangulSyllableTwebs': '\xed\x89\xb6',
'HangulSyllableJyegs': '\xec\xa1\x97',
'ArabicLigatureLamWithHahWithMeemFinalForm': '\xef\xb6\x80',
'BlackSmallStar': '\xe2\xad\x91',
'CjkUnifiedIdeograph7313': '\xe7\x8c\x93',
'CjkUnifiedIdeograph7bad': '\xe7\xae\xad',
'CjkUnifiedIdeograph7bae': '\xe7\xae\xae',
'CjkUnifiedIdeograph7bab': '\xe7\xae\xab',
'ThaiCharacterThoThong': '\xe0\xb8\x98',
'UnicodeCharacter0xdf40': '\xed\xbd\x80',
'CjkUnifiedIdeograph7312': '\xe7\x8c\x92',
'HangulSyllableMyigg': '\xeb\xaf\x9e',
'HangulSyllableNug': '\xeb\x88\x85',
'UnicodeCharacter0xdf48': '\xed\xbd\x88',
'UnicodeCharacter0xdf49': '\xed\xbd\x89',
'NkoLetterJonaJa': '\xdf\xa8',
'HangulSyllableJjweom': '\xec\xad\xb4',
'UnicodeCharacter0xf82a': '\xef\xa0\xaa',
'HangulSyllableJjweoj': '\xec\xad\xba',
'EthiopicSyllableYee': '\xe1\x8b\xac',
'HangulSyllableJjweok': '\xec\xad\xbc',
'CjkUnifiedIdeograph62e1': '\xe6\x8b\xa1',
'CjkUnifiedIdeograph62e0': '\xe6\x8b\xa0',
'CjkUnifiedIdeograph62e3': '\xe6\x8b\xa3',
'CjkUnifiedIdeograph62e2': '\xe6\x8b\xa2',
'CjkUnifiedIdeograph62e5': '\xe6\x8b\xa5',
'CjkUnifiedIdeograph62e4': '\xe6\x8b\xa4',
'CjkUnifiedIdeograph62e7': '\xe6\x8b\xa7',
'CjkUnifiedIdeograph62e6': '\xe6\x8b\xa6',
'CjkUnifiedIdeograph62e9': '\xe6\x8b\xa9',
'CjkUnifiedIdeograph62e8': '\xe6\x8b\xa8',
'UnicodeCharacter0xe718': '\xee\x9c\x98',
'HangulSyllableNub': '\xeb\x88\x95',
'UnicodeCharacter0xdc08': '\xed\xb0\x88',
'MongolianDigitEight': '\xe1\xa0\x98',
'YiSyllableZyx': '\xea\x8a\xa9',
'YiSyllableMox': '\xea\x82\xbe',
'UnicodeCharacter0xf82b': '\xef\xa0\xab',
'HangulSyllableNul': '\xeb\x88\x8c',
'YiSyllableZyr': '\xea\x8a\xad',
'HangulSyllableCwep': '\xec\xb7\xa6',
'YiSyllableZyp': '\xea\x8a\xab',
'ArabicLetterTteh': '\xd9\xb9',
'CjkUnifiedIdeograph89bc': '\xe8\xa6\xbc',
'YiSyllableZyt': '\xea\x8a\xa8',
'CjkUnifiedIdeograph64a2': '\xe6\x92\xa2',
'HangulSyllableNun': '\xeb\x88\x88',
'CombiningCyrillicKavyka': '\xea\x99\xbc',
'UnicodeCharacter0xdc06': '\xed\xb0\x86',
'HangulSyllableGu': '\xea\xb5\xac',
'UnicodeCharacter0xdc05': '\xed\xb0\x85',
'HangulSyllableNuh': '\xeb\x88\x9f',
'CjkUnifiedIdeograph3f5b': '\xe3\xbd\x9b',
'CjkUnifiedIdeograph3f5c': '\xe3\xbd\x9c',
'UnicodeCharacter0xa7ea': '\xea\x9f\xaa',
'CjkUnifiedIdeograph3f5a': '\xe3\xbd\x9a',
'CjkUnifiedIdeograph3f5f': '\xe3\xbd\x9f',
'UnicodeCharacter0xa7ef': '\xea\x9f\xaf',
'CjkUnifiedIdeograph3f5d': '\xe3\xbd\x9d',
'CjkUnifiedIdeograph3f5e': '\xe3\xbd\x9e',
'CjkUnifiedIdeograph89bf': '\xe8\xa6\xbf',
'HangulSyllableGa': '\xea\xb0\x80',
'UnicodeCharacter0xdc01': '\xed\xb0\x81',
'ArabicLetterUighurKazakhKirghizAlefMaksuraInitialForm': '\xef\xaf\xa8',
'HangulSyllableJib': '\xec\xa7\x91',
'CjkUnifiedIdeograph62ea': '\xe6\x8b\xaa',
'HangulSyllableJim': '\xec\xa7\x90',
'IdeographicAnnotationFourthMark': '\xe3\x86\x9c',
'CjkUnifiedIdeograph62eb': '\xe6\x8b\xab',
'CjkUnifiedIdeograph62ee': '\xe6\x8b\xae',
'CjkUnifiedIdeograph62ed': '\xe6\x8b\xad',
'CjkUnifiedIdeograph62ef': '\xe6\x8b\xaf',
'CjkUnifiedIdeograph8c5d': '\xe8\xb1\x9d',
'CjkUnifiedIdeograph3f52': '\xe3\xbd\x92',
'CjkUnifiedIdeograph3f53': '\xe3\xbd\x93',
'CjkUnifiedIdeograph3f50': '\xe3\xbd\x90',
'CjkUnifiedIdeograph3f51': '\xe3\xbd\x91',
'CjkUnifiedIdeograph3f56': '\xe3\xbd\x96',
'EthiopicSyllableCwa': '\xe1\x89\xbf',
'CjkUnifiedIdeograph3f54': '\xe3\xbd\x94',
'CjkUnifiedIdeograph3f55': '\xe3\xbd\x95',
'CjkUnifiedIdeograph3f58': '\xe3\xbd\x98',
'CjkUnifiedIdeograph3f59': '\xe3\xbd\x99',
'HangulSyllableMwanh': '\xeb\xab\x8a',
'YiSyllableCyr': '\xea\x8b\x8f',
'HangulSyllableCweg': '\xec\xb7\x8d',
'UnicodeCharacter0xdc09': '\xed\xb0\x89',
'GeorgianLetterIn': '\xe1\x83\x98',
'YiSyllableCyp': '\xea\x8b\x8d',
'HangulSyllableNenj': '\xeb\x84\xa9',
'CjkUnifiedIdeograph9adf': '\xe9\xab\x9f',
'HangulSyllableGgelp': '\xea\xbb\xa6',
'HangulSyllableGgels': '\xea\xbb\xa4',
'CanadianSyllabicsWestcreeShwa': '\xe1\x94\xa2',
'HangulSyllableGgelt': '\xea\xbb\xa5',
'CjkUnifiedIdeograph9ada': '\xe9\xab\x9a',
'HangulSyllableRyanh': '\xeb\x9e\xba',
'HangulSyllableRyanj': '\xeb\x9e\xb9',
'HangulSyllableGgelb': '\xea\xbb\xa3',
'CjkUnifiedIdeograph9adb': '\xe9\xab\x9b',
'HangulSyllableGgelg': '\xea\xbb\xa1',
'HangulSyllableCwem': '\xec\xb7\x9c',
'HangulSyllableGgelh': '\xea\xbb\xa7',
'HangulSyllableGgelm': '\xea\xbb\xa2',
'HangulSyllableGgalp': '\xea\xb9\x9a',
'UnicodeCharacter0xf1a4': '\xef\x86\xa4',
'CjkUnifiedIdeograph8f44': '\xe8\xbd\x84',
'ModifierLetterSmallTurnedMWithLongLeg': '\xe1\xb6\xad',
'HangulSyllableCweo': '\xec\xb6\xb0',
'HangulSyllableGgals': '\xea\xb9\x98',
'CjkCompatibilityIdeographf998': '\xef\xa6\x98',
'CjkUnifiedIdeograph8f45': '\xe8\xbd\x85',
'MyanmarShanDigitFour': '\xe1\x82\x94',
'CjkCompatibilityIdeographf997': '\xef\xa6\x97',
'CjkCompatibilityIdeographf994': '\xef\xa6\x94',
'CjkCompatibilityIdeographf995': '\xef\xa6\x95',
'CjkCompatibilityIdeographf992': '\xef\xa6\x92',
'CjkUnifiedIdeograph8f46': '\xe8\xbd\x86',
'CjkCompatibilityIdeographf990': '\xef\xa6\x90',
'CjkCompatibilityIdeographf991': '\xef\xa6\x91',
'CjkUnifiedIdeograph7c79': '\xe7\xb1\xb9',
'CjkUnifiedIdeograph7c78': '\xe7\xb1\xb8',
'CjkUnifiedIdeograph8f47': '\xe8\xbd\x87',
'CjkUnifiedIdeograph7604': '\xe7\x98\x84',
'CjkUnifiedIdeograph7c71': '\xe7\xb1\xb1',
'CjkUnifiedIdeograph7c70': '\xe7\xb1\xb0',
'CjkUnifiedIdeograph7c73': '\xe7\xb1\xb3',
'CjkUnifiedIdeograph7c72': '\xe7\xb1\xb2',
'CjkUnifiedIdeograph7c75': '\xe7\xb1\xb5',
'LaoDigitEight': '\xe0\xbb\x98',
'CjkUnifiedIdeograph7c77': '\xe7\xb1\xb7',
'CjkUnifiedIdeograph7c76': '\xe7\xb1\xb6',
'CjkUnifiedIdeograph7606': '\xe7\x98\x86',
'CjkUnifiedIdeograph8f42': '\xe8\xbd\x82',
'CjkUnifiedIdeograph7601': '\xe7\x98\x81',
'CjkUnifiedIdeograph8f43': '\xe8\xbd\x83',
'VaiSyllableGbi': '\xea\x94\xb0',
'SaurashtraVowelSignVocalicL': '\xea\xa2\xbc',
'CjkUnifiedIdeograph7602': '\xe7\x98\x82',
'UnicodeCharacter0xe711': '\xee\x9c\x91',
'YiSyllableHlie': '\xea\x86\x9c',
'HangulSyllableDdogs': '\xeb\x98\x93',
'CjkUnifiedIdeograph7c7a': '\xe7\xb1\xba',
'CjkUnifiedIdeograph8f48': '\xe8\xbd\x88',
'CjkUnifiedIdeograph6749': '\xe6\x9d\x89',
'CjkUnifiedIdeograph7c7b': '\xe7\xb1\xbb',
'CjkUnifiedIdeograph7c7e': '\xe7\xb1\xbe',
'CjkUnifiedIdeograph7c7d': '\xe7\xb1\xbd',
'CjkUnifiedIdeograph7c7f': '\xe7\xb1\xbf',
'HangulSyllableDdogg': '\xeb\x98\x92',
'CjkCompatibilityIdeographf99f': '\xef\xa6\x9f',
'CjkUnifiedIdeograph9dbf': '\xe9\xb6\xbf',
'UnicodeCharacter0xf1a7': '\xef\x86\xa7',
'CjkCompatibilityIdeographf99e': '\xef\xa6\x9e',
'CjkCompatibilityIdeographf99b': '\xef\xa6\x9b',
'CjkCompatibilityIdeographf99c': '\xef\xa6\x9c',
'KangxiRadicalDeer': '\xe2\xbf\x85',
'BoxDrawingsDoubleVerticalAndHorizontal': '\xe2\x95\xac',
'CjkRadicalChoke': '\xe2\xba\x9b',
'KhmerVowelSignYa': '\xe1\x9e\xbf',
'KhmerVowelSignYy': '\xe1\x9e\xba',
'CjkUnifiedIdeograph77ec': '\xe7\x9f\xac',
'CjkUnifiedIdeograph77eb': '\xe7\x9f\xab',
'CjkUnifiedIdeograph77ea': '\xe7\x9f\xaa',
'CjkUnifiedIdeograph77ef': '\xe7\x9f\xaf',
'CjkUnifiedIdeograph77ee': '\xe7\x9f\xae',
'CjkUnifiedIdeograph77ed': '\xe7\x9f\xad',
'UnicodeCharacter0xeaf2': '\xee\xab\xb2',
'LatinCapitalLetterEWithMacronAndGrave': '\xe1\xb8\x94',
'UnicodeCharacter0xde34': '\xed\xb8\xb4',
'UnicodeCharacter0xd888': '\xed\xa2\x88',
'UnicodeCharacter0xac6': '\xe0\xab\x86',
'UnicodeCharacter0xf1a1': '\xef\x86\xa1',
'HangulSyllablePwaeng': '\xed\x90\xb9',
'HangulSyllablePwaenh': '\xed\x90\xaa',
'UnicodeCharacter0xd885': '\xed\xa2\x85',
'HangulSyllablePwaenj': '\xed\x90\xa9',
'UnicodeCharacter0xd887': '\xed\xa2\x87',
'UnicodeCharacter0xd880': '\xed\xa2\x80',
'UnicodeCharacter0xd881': '\xed\xa2\x81',
'UnicodeCharacter0xd882': '\xed\xa2\x82',
'UnicodeCharacter0xd883': '\xed\xa2\x83',
'UnicodeCharacter0xf67e': '\xef\x99\xbe',
'CjkUnifiedIdeograph77e9': '\xe7\x9f\xa9',
'CjkUnifiedIdeograph77e8': '\xe7\x9f\xa8',
'CjkStrokeHpwg': '\xe3\x87\x8c',
'CjkUnifiedIdeograph77e3': '\xe7\x9f\xa3',
'CjkUnifiedIdeograph77e2': '\xe7\x9f\xa2',
'LatinCapitalLetterZWithHook': '\xc8\xa4',
'CjkUnifiedIdeograph77e0': '\xe7\x9f\xa0',
'SyriacContraction': '\xdc\x8a',
'CjkUnifiedIdeograph77e6': '\xe7\x9f\xa6',
'CjkUnifiedIdeograph77e5': '\xe7\x9f\xa5',
'CjkUnifiedIdeograph77e4': '\xe7\x9f\xa4',
'CjkUnifiedIdeograph86d4': '\xe8\x9b\x94',
'UnicodeCharacter0xf1a2': '\xef\x86\xa2',
'UnicodeCharacter0xde32': '\xed\xb8\xb2',
'UnicodeCharacter0xf733': '\xef\x9c\xb3',
'HeavyLeftpointingAngleBracketOrnament': '\xe2\x9d\xb0',
'HangulSyllableGgoess': '\xea\xbe\x94',
'UnicodeCharacter0xf67d': '\xef\x99\xbd',
'HangulSyllableHwagg': '\xed\x99\x96',
'UnicodeCharacter0xaca': '\xe0\xab\x8a',
'UnicodeCharacter0xacf': '\xe0\xab\x8f',
'UnicodeCharacter0xace': '\xe0\xab\x8e',
'UnicodeCharacter0xd88d': '\xed\xa2\x8d',
'UnicodeCharacter0xd88e': '\xed\xa2\x8e',
'UnicodeCharacter0xd88f': '\xed\xa2\x8f',
'HangulSyllableToss': '\xed\x86\xb4',
'UnicodeCharacter0xd88a': '\xed\xa2\x8a',
'UnicodeCharacter0xd88b': '\xed\xa2\x8b',
'UnicodeCharacter0xd88c': '\xed\xa2\x8c',
'HangulJongseongPieuphieuh': '\xe1\x87\xa5',
'CjkUnifiedIdeograph86d5': '\xe8\x9b\x95',
'UnicodeCharacter0xe714': '\xee\x9c\x94',
'CjkUnifiedIdeograph9446': '\xe9\x91\x86',
'MetricalLongOverShort': '\xe2\x8f\x92',
'ArabicLetterDdalFinalForm': '\xef\xae\x89',
'UnicodeCharacter0xf62b': '\xef\x98\xab',
'UnicodeCharacter0xf62c': '\xef\x98\xac',
'UnicodeCharacter0xf62a': '\xef\x98\xaa',
'UnicodeCharacter0xf62f': '\xef\x98\xaf',
'UnicodeCharacter0xf62d': '\xef\x98\xad',
'BengaliLetterRra': '\xe0\xa7\x9c',
'HangulSyllableHyet': '\xed\x98\xb5',
'VaiDigitOne': '\xea\x98\xa1',
'YiRadicalCyp': '\xea\x92\x94',
'UnicodeCharacter0xf1f2': '\xef\x87\xb2',
'CjkUnifiedIdeograph5d96': '\xe5\xb6\x96',
'CjkUnifiedIdeograph5d97': '\xe5\xb6\x97',
'CjkUnifiedIdeograph5d94': '\xe5\xb6\x94',
'CanadianSyllabicsCarrierJja': '\xe1\x98\xa1',
'TibetanSignYangRtags': '\xe0\xbe\x87',
'CjkUnifiedIdeograph5d93': '\xe5\xb6\x93',
'CjkUnifiedIdeograph5d90': '\xe5\xb6\x90',
'CanadianSyllabicsCarrierJje': '\xe1\x98\x9e',
'CjkUnifiedIdeograph9a3e': '\xe9\xa8\xbe',
'CjkUnifiedIdeograph9a3d': '\xe9\xa8\xbd',
'HangulSyllableGgyit': '\xeb\x81\xb9',
'CanadianSyllabicsCarrierJji': '\xe1\x98\xa0',
'CjkUnifiedIdeograph9a3a': '\xe9\xa8\xba',
'CanadianSyllabicsCarrierJjo': '\xe1\x98\x9d',
'CjkUnifiedIdeograph5d98': '\xe5\xb6\x98',
'CjkUnifiedIdeograph5d99': '\xe5\xb6\x99',
'HangulSyllableGgyin': '\xeb\x81\xa4',
'HangulSyllableGgyil': '\xeb\x81\xa8',
'HangulSyllableGgyim': '\xeb\x81\xb0',
'HangulSyllableGgyij': '\xeb\x81\xb6',
'HangulSyllableGgyik': '\xeb\x81\xb8',
'HangulSyllableGgyih': '\xeb\x81\xbb',
'CanadianSyllabicsCarrierJju': '\xe1\x98\x9c',
'CjkUnifiedIdeograph7f4e': '\xe7\xbd\x8e',
'HangulSyllableGgyid': '\xeb\x81\xa7',
'RadioactiveSign': '\xe2\x98\xa2',
'HangulSyllableGgyib': '\xeb\x81\xb1',
'HangulSyllableGgyic': '\xeb\x81\xb7',
'HangulSyllableRwals': '\xeb\xa2\x84',
'UnicodeCharacter0xf628': '\xef\x98\xa8',
'UnicodeCharacter0xf629': '\xef\x98\xa9',
'UnicodeCharacter0xf622': '\xef\x98\xa2',
'UnicodeCharacter0xf623': '\xef\x98\xa3',
'UnicodeCharacter0xf620': '\xef\x98\xa0',
'UnicodeCharacter0xf621': '\xef\x98\xa1',
'UnicodeCharacter0xf626': '\xef\x98\xa6',
'UnicodeCharacter0xebb2': '\xee\xae\xb2',
'UnicodeCharacter0xf624': '\xef\x98\xa4',
'UnicodeCharacter0xf625': '\xef\x98\xa5',
'HangulJungseongYoya': '\xe1\x86\x84',
'HangulSyllableRwegs': '\xeb\xa4\xa3',
'CanadianSyllabicsWestcreeYwii': '\xe1\x94\xb4',
'CjkUnifiedIdeograph9540': '\xe9\x95\x80',
'UnicodeCharacter0xf684': '\xef\x9a\x84',
'CjkUnifiedIdeograph5d9f': '\xe5\xb6\x9f',
'PhagspaLetterSa': '\xea\xa1\x9b',
'CjkUnifiedIdeograph5d9d': '\xe5\xb6\x9d',
'CjkUnifiedIdeograph5d9e': '\xe5\xb6\x9e',
'CjkUnifiedIdeograph5d9b': '\xe5\xb6\x9b',
'CjkUnifiedIdeograph5d9c': '\xe5\xb6\x9c',
'CjkUnifiedIdeograph5d9a': '\xe5\xb6\x9a',
'CjkUnifiedIdeograph9a35': '\xe9\xa8\xb5',
'LatinSmallLetterEWithDiaeresis': '\xc3\xab',
'CjkUnifiedIdeograph9725': '\xe9\x9c\xa5',
'CjkUnifiedIdeograph9a36': '\xe9\xa8\xb6',
'CjkUnifiedIdeograph9a31': '\xe9\xa8\xb1',
'CjkUnifiedIdeograph9a30': '\xe9\xa8\xb0',
'ModifierLetterShelf': '\xcb\xbd',
'CjkUnifiedIdeograph9a32': '\xe9\xa8\xb2',
'NewTaiLueVowelSignUy': '\xe1\xa6\xbc',
'UnicodeCharacter0xd9d0': '\xed\xa7\x90',
'UnicodeCharacter0xd9d3': '\xed\xa7\x93',
'Flower': '\xe2\x9a\x98',
'HangulSyllableNweonh': '\xeb\x88\xa6',
'UnicodeCharacter0xd9d4': '\xed\xa7\x94',
'HangulSyllableNweonj': '\xeb\x88\xa5',
'UnicodeCharacter0xd9d6': '\xed\xa7\x96',
'UnicodeCharacter0xd9d9': '\xed\xa7\x99',
'UnicodeCharacter0xd9d8': '\xed\xa7\x98',
'HangulSyllableNweong': '\xeb\x88\xb5',
'NewTaiLueVowelSignUu': '\xe1\xa6\xb4',
'HangulSyllableJwat': '\xec\xa2\xa5',
'ArmenianCapitalLetterJa': '\xd5\x81',
'NewTaiLueVowelSignUe': '\xe1\xa6\xb9',
'UnicodeCharacter0xf7e2': '\xef\x9f\xa2',
'CjkUnifiedIdeograph86d3': '\xe8\x9b\x93',
'UnicodeCharacter0xe3af': '\xee\x8e\xaf',
'GeorgianLetterCil': '\xe1\x83\xac',
'UnicodeCharacter0xe3ae': '\xee\x8e\xae',
'UnicodeCharacter0x2e6e': '\xe2\xb9\xae',
'UnicodeCharacter0x2e6d': '\xe2\xb9\xad',
'UnicodeCharacter0x2e6f': '\xe2\xb9\xaf',
'UnicodeCharacter0x2e6a': '\xe2\xb9\xaa',
'UnicodeCharacter0x2e6c': '\xe2\xb9\xac',
'UnicodeCharacter0x2e6b': '\xe2\xb9\xab',
'UnicodeCharacter0xf7e3': '\xef\x9f\xa3',
'UnicodeCharacter0xe3ab': '\xee\x8e\xab',
'CanadianSyllabicsPwo': '\xe1\x91\x80',
'UnicodeCharacter0xe3aa': '\xee\x8e\xaa',
'CanadianSyllabicsPwi': '\xe1\x90\xbc',
'LatinLetterSmallCapitalAe': '\xe1\xb4\x81',
'CanadianSyllabicsPwe': '\xe1\x90\xba',
'CanadianSyllabicsPwa': '\xe1\x91\x84',
'UnicodeCharacter0xd9da': '\xed\xa7\x9a',
'TibetanMarkGterYigMgoTruncatedA': '\xe0\xbc\x81',
'UnicodeCharacter0xd9dc': '\xed\xa7\x9c',
'HangulSyllableNwels': '\xeb\x89\x88',
'UnicodeCharacter0xd9de': '\xed\xa7\x9e',
'UnicodeCharacter0xaae8': '\xea\xab\xa8',
'UnicodeCharacter0xd9df': '\xed\xa7\x9f',
'HangulSyllableNwelt': '\xeb\x89\x89',
'CjkUnifiedIdeograph81c1': '\xe8\x87\x81',
'UnicodeCharacter0xf7e0': '\xef\x9f\xa0',
'HangulJongseongRieulthieuth': '\xe1\x86\xb4',
'UnicodeCharacter0x2e65': '\xe2\xb9\xa5',
'UnicodeCharacter0x2e64': '\xe2\xb9\xa4',
'UnicodeCharacter0x2e67': '\xe2\xb9\xa7',
'UnicodeCharacter0x2e66': '\xe2\xb9\xa6',
'UnicodeCharacter0x2e61': '\xe2\xb9\xa1',
'UnicodeCharacter0x2e60': '\xe2\xb9\xa0',
'UnicodeCharacter0x2e63': '\xe2\xb9\xa3',
'UnicodeCharacter0x2e62': '\xe2\xb9\xa2',
'ZeroWidthNonjoiner': '\xe2\x80\x8c',
'CjkUnifiedIdeograph36d2': '\xe3\x9b\x92',
'UnicodeCharacter0x2e69': '\xe2\xb9\xa9',
'UnicodeCharacter0x2e68': '\xe2\xb9\xa8',
'CyrillicCapitalLetterBroadOmega': '\xea\x99\x8c',
'UpwardsArrowWithDoubleStroke': '\xe2\x87\x9e',
'HangulSyllableSsoelb': '\xec\x90\xb7',
'UnicodeCharacter0xf53f': '\xef\x94\xbf',
'UnicodeCharacter0xf7e1': '\xef\x9f\xa1',
'DoubleUnion': '\xe2\x8b\x93',
'CjkUnifiedIdeograph9541': '\xe9\x95\x81',
'CyrillicSmallLetterKhakassianChe': '\xd3\x8c',
'CjkUnifiedIdeograph593f': '\xe5\xa4\xbf',
'FullwidthRightWhiteParenthesis': '\xef\xbd\xa0',
'UnicodeCharacter0xebb4': '\xee\xae\xb4',
'HangulSyllableBbwej': '\xec\x80\x9a',
'HangulJongseongYesieungpansios': '\xe1\x87\xb2',
'UnicodeCharacter0xe555': '\xee\x95\x95',
'UnicodeCharacter0xf876': '\xef\xa1\xb6',
'LessthanAboveLeftwardsArrow': '\xe2\xa5\xb6',
'UnicodeCharacter0xdcb1': '\xed\xb2\xb1',
'CjkUnifiedIdeograph414f': '\xe4\x85\x8f',
'CjkUnifiedIdeograph414e': '\xe4\x85\x8e',
'CjkUnifiedIdeograph414d': '\xe4\x85\x8d',
'CjkUnifiedIdeograph414c': '\xe4\x85\x8c',
'CjkUnifiedIdeograph414b': '\xe4\x85\x8b',
'CjkUnifiedIdeograph414a': '\xe4\x85\x8a',
'SinhalaLetterAlpapraanaDdayanna': '\xe0\xb6\xa9',
'KannadaLetterTha': '\xe0\xb2\xa5',
'UnicodeCharacter0xf7e7': '\xef\x9f\xa7',
'CjkUnifiedIdeograph912b': '\xe9\x84\xab',
'CjkUnifiedIdeograph912c': '\xe9\x84\xac',
'CjkUnifiedIdeograph912a': '\xe9\x84\xaa',
'CjkUnifiedIdeograph912f': '\xe9\x84\xaf',
'TeluguLetterVa': '\xe0\xb0\xb5',
'CjkUnifiedIdeograph912d': '\xe9\x84\xad',
'CjkUnifiedIdeograph912e': '\xe9\x84\xae',
'VaiSyllableYee': '\xea\x94\x9d',
'AplFunctionalSymbolQuadDownwardsArrow': '\xe2\x8d\x97',
'HangulSyllableBbyeonh': '\xeb\xbc\x8e',
'HebrewLetterYod': '\xd7\x99',
'SymbolForCarriageReturn': '\xe2\x90\x8d',
'HangulSyllableHaj': '\xed\x95\xae',
'LatinSmallLetterHWithDotAbove': '\xe1\xb8\xa3',
'UnicodeCharacter0xf1da': '\xef\x87\x9a',
'GreekNumeralSign': '\xcd\xb4',
'UnicodeCharacter0xf53e': '\xef\x94\xbe',
'HangulSyllableGwelb': '\xea\xb6\xaf',
'UnicodeCharacter0xe557': '\xee\x95\x97',
'VaiSyllableZhe': '\xea\x97\xbe',
'CjkUnifiedIdeograph9128': '\xe9\x84\xa8',
'CjkUnifiedIdeograph9129': '\xe9\x84\xa9',
'HangulSyllableBbulh': '\xeb\xbf\x9b',
'CjkUnifiedIdeograph9122': '\xe9\x84\xa2',
'CjkUnifiedIdeograph9123': '\xe9\x84\xa3',
'CjkUnifiedIdeograph9120': '\xe9\x84\xa0',
'CjkUnifiedIdeograph9121': '\xe9\x84\xa1',
'CjkUnifiedIdeograph9126': '\xe9\x84\xa6',
'CjkUnifiedIdeograph9127': '\xe9\x84\xa7',
'CjkUnifiedIdeograph9124': '\xe9\x84\xa4',
'CjkUnifiedIdeograph9125': '\xe9\x84\xa5',
'UnicodeCharacter0xf7e5': '\xef\x9f\xa5',
'UnicodeCharacter0xf440': '\xef\x91\x80',
'CjkUnifiedIdeograph4149': '\xe4\x85\x89',
'CjkUnifiedIdeograph4148': '\xe4\x85\x88',
'CyrillicSmallLetterZheWithBreve': '\xd3\x82',
'CjkUnifiedIdeograph4146': '\xe4\x85\x86',
'CjkUnifiedIdeograph4145': '\xe4\x85\x85',
'CjkUnifiedIdeograph4144': '\xe4\x85\x84',
'CjkUnifiedIdeograph4143': '\xe4\x85\x83',
'CjkUnifiedIdeograph4142': '\xe4\x85\x82',
'CjkUnifiedIdeograph4141': '\xe4\x85\x81',
'CjkUnifiedIdeograph4140': '\xe4\x85\x80',
'HangulSyllableDuk': '\xeb\x91\xa8',
'ArabicLetterAlefWaslaIsolatedForm': '\xef\xad\x90',
'IdeographicTelegraphSymbolForApril': '\xe3\x8b\x83',
'UnicodeCharacter0x89a': '\xe0\xa2\x9a',
'UnicodeCharacter0x89b': '\xe0\xa2\x9b',
'UnicodeCharacter0x89c': '\xe0\xa2\x9c',
'UnicodeCharacter0x89d': '\xe0\xa2\x9d',
'UnicodeCharacter0x89e': '\xe0\xa2\x9e',
'UnicodeCharacter0x89f': '\xe0\xa2\x9f',
'HangulSyllableJyuh': '\xec\xa6\x87',
'HangulSyllableNalm': '\xeb\x82\xa2',
'HangulSyllableNalh': '\xeb\x82\xa7',
'UnicodeCharacter0xde3e': '\xed\xb8\xbe',
'CjkUnifiedIdeograph9085': '\xe9\x82\x85',
'UnicodeCharacter0xf736': '\xef\x9c\xb6',
'RejangConsonantSignN': '\xea\xa5\x90',
'HangulSyllableNalb': '\xeb\x82\xa3',
'UnicodeCharacter0xde3f': '\xed\xb8\xbf',
'HangulSyllableGeung': '\xea\xb8\x8d',
'ThaiCharacterFongman': '\xe0\xb9\x8f',
'YiSyllableNru': '\xea\x8e\xa1',
'CjkUnifiedIdeograph978f': '\xe9\x9e\x8f',
'CjkUnifiedIdeograph978d': '\xe9\x9e\x8d',
'CjkUnifiedIdeograph9086': '\xe9\x82\x86',
'CyrillicCapitalLetterLittleYus': '\xd1\xa6',
'CjkUnifiedIdeograph978c': '\xe9\x9e\x8c',
'CjkUnifiedIdeograph58d4': '\xe5\xa3\x94',
'CjkUnifiedIdeograph4499': '\xe4\x92\x99',
'CjkUnifiedIdeograph4498': '\xe4\x92\x98',
'YiSyllableNra': '\xea\x8e\x96',
'CanadianSyllabicsWestcreeCwii': '\xe1\x92\x97',
'UnicodeCharacter0xde3b': '\xed\xb8\xbb',
'YiSyllableNre': '\xea\x8e\x9d',
'CjkUnifiedIdeograph9088': '\xe9\x82\x88',
'CjkUnifiedIdeograph4491': '\xe4\x92\x91',
'CjkUnifiedIdeograph4490': '\xe4\x92\x90',
'CjkUnifiedIdeograph4493': '\xe4\x92\x93',
'CjkUnifiedIdeograph4492': '\xe4\x92\x92',
'CjkUnifiedIdeograph4495': '\xe4\x92\x95',
'CjkUnifiedIdeograph4494': '\xe4\x92\x94',
'CjkUnifiedIdeograph4497': '\xe4\x92\x97',
'CjkUnifiedIdeograph4496': '\xe4\x92\x96',
'VaiSyllableAn': '\xea\x95\x8a',
'HangulSyllableDyaes': '\xeb\x8d\x8b',
'CjkUnifiedIdeograph63a3': '\xe6\x8e\xa3',
'CjkUnifiedIdeograph7c86': '\xe7\xb2\x86',
'UnicodeCharacter0xe3a6': '\xee\x8e\xa6',
'HangulSyllableDug': '\xeb\x91\x91',
'UnicodeCharacter0xf53a': '\xef\x94\xba',
'SmallPlusSign': '\xef\xb9\xa2',
'HangulSyllableJyuj': '\xec\xa6\x82',
'UnicodeCharacter0x898': '\xe0\xa2\x98',
'UnicodeCharacter0x899': '\xe0\xa2\x99',
'ArabicLetterYehWithHamzaAboveFinalForm': '\xef\xba\x8a',
'HangulSyllableDyaep': '\xeb\x8d\x92',
'HangulSyllableSsyebs': '\xec\x8f\x8e',
'UnicodeCharacter0x890': '\xe0\xa2\x90',
'UnicodeCharacter0x891': '\xe0\xa2\x91',
'UnicodeCharacter0x892': '\xe0\xa2\x92',
'UnicodeCharacter0x893': '\xe0\xa2\x93',
'UnicodeCharacter0x894': '\xe0\xa2\x94',
'UnicodeCharacter0x895': '\xe0\xa2\x95',
'UnicodeCharacter0x896': '\xe0\xa2\x96',
'UnicodeCharacter0x897': '\xe0\xa2\x97',
'EthiopicSyllableChu': '\xe1\x8c\xa9',
'CjkUnifiedIdeograph76d8': '\xe7\x9b\x98',
'UnicodeCharacter0xe6ec': '\xee\x9b\xac',
'UnicodeCharacter0xe3a1': '\xee\x8e\xa1',
'CjkUnifiedIdeograph449a': '\xe4\x92\x9a',
'CjkUnifiedIdeograph76d0': '\xe7\x9b\x90',
'CjkUnifiedIdeograph449c': '\xe4\x92\x9c',
'CjkUnifiedIdeograph449b': '\xe4\x92\x9b',
'CjkUnifiedIdeograph449e': '\xe4\x92\x9e',
'CjkUnifiedIdeograph449d': '\xe4\x92\x9d',
'CjkUnifiedIdeograph76d7': '\xe7\x9b\x97',
'CjkUnifiedIdeograph449f': '\xe4\x92\x9f',
'EthiopicSyllableChe': '\xe1\x8c\xad',
'CjkRadicalEatOne': '\xe2\xbb\x9d',
'EthiopicSyllableCha': '\xe1\x8c\xa8',
'TagalogLetterNga': '\xe1\x9c\x85',
'CjkUnifiedIdeograph9788': '\xe9\x9e\x88',
'CjkUnifiedIdeograph9789': '\xe9\x9e\x89',
'CjkUnifiedIdeograph9786': '\xe9\x9e\x86',
'CjkUnifiedIdeograph9787': '\xe9\x9e\x87',
'EthiopicSyllableCho': '\xe1\x8c\xae',
'CjkUnifiedIdeograph9785': '\xe9\x9e\x85',
'EthiopicSyllableChi': '\xe1\x8c\xaa',
'CjkUnifiedIdeograph9783': '\xe9\x9e\x83',
'CjkUnifiedIdeograph9780': '\xe9\x9e\x80',
'CjkUnifiedIdeograph9781': '\xe9\x9e\x81',
'HalfwidthHangulLetterSsangpieup': '\xef\xbe\xb3',
'HangulChoseongCieuc': '\xe1\x84\x8c',
'CanadianSyllabicsSouthslaveyKih': '\xe1\x92\x86',
'CjkUnifiedIdeograph43af': '\xe4\x8e\xaf',
'UnicodeCharacter0xdaf3': '\xed\xab\xb3',
'HangulSyllableJjwek': '\xec\xae\x98',
'CjkUnifiedIdeograph955a': '\xe9\x95\x9a',
'CjkUnifiedIdeograph5368': '\xe5\x8d\xa8',
'CjkUnifiedIdeograph5369': '\xe5\x8d\xa9',
'LatinSmallLetterFWithHook': '\xc6\x92',
'CjkUnifiedIdeograph5360': '\xe5\x8d\xa0',
'CjkUnifiedIdeograph5361': '\xe5\x8d\xa1',
'CjkUnifiedIdeograph5362': '\xe5\x8d\xa2',
'CjkUnifiedIdeograph5363': '\xe5\x8d\xa3',
'CjkUnifiedIdeograph5364': '\xe5\x8d\xa4',
'CjkUnifiedIdeograph5365': '\xe5\x8d\xa5',
'CjkUnifiedIdeograph5366': '\xe5\x8d\xa6',
'CjkUnifiedIdeograph5367': '\xe5\x8d\xa7',
'CjkUnifiedIdeograph9878': '\xe9\xa1\xb8',
'CjkUnifiedIdeograph9879': '\xe9\xa1\xb9',
'CyrillicCapitalLetterHaWithDescender': '\xd2\xb2',
'MongolianEllipsis': '\xe1\xa0\x81',
'HangulSyllableJjwel': '\xec\xae\x88',
'UnicodeCharacter0xf0d8': '\xef\x83\x98',
'UnicodeCharacter0xf0d9': '\xef\x83\x99',
'CjkUnifiedIdeograph9870': '\xe9\xa1\xb0',
'HangulJongseongMieumssangsios': '\xe1\x87\x9e',
'CjkUnifiedIdeograph9872': '\xe9\xa1\xb2',
'CjkUnifiedIdeograph955d': '\xe9\x95\x9d',
'CjkUnifiedIdeograph9874': '\xe9\xa1\xb4',
'CjkUnifiedIdeograph8185': '\xe8\x86\x85',
'LatinCapitalLetterEWithDiaeresis': '\xc3\x8b',
'CjkUnifiedIdeograph9877': '\xe9\xa1\xb7',
'HangulSyllableYaet': '\xec\x96\xb1',
'UnicodeCharacter0xd9ab': '\xed\xa6\xab',
'HangulSyllableYaep': '\xec\x96\xb2',
'HangulSyllableJyun': '\xec\xa5\xb0',
'HangulSyllableYaes': '\xec\x96\xab',
'HangulSyllableKoelt': '\xec\xbe\xb5',
'CjkUnifiedIdeograph91d9': '\xe9\x87\x99',
'HangulSyllableYaed': '\xec\x96\x9f',
'UnicodeCharacter0xdaf1': '\xed\xab\xb1',
'HangulSyllableYaeg': '\xec\x96\x99',
'CjkUnifiedIdeograph8181': '\xe8\x86\x81',
'HangulSyllableYaeb': '\xec\x96\xa9',
'HangulSyllableYaec': '\xec\x96\xaf',
'HangulSyllableYael': '\xec\x96\xa0',
'HangulSyllableYaem': '\xec\x96\xa8',
'HangulSyllableYaen': '\xec\x96\x9c',
'UnicodeCharacter0xaae7': '\xea\xab\xa7',
'FinitePartIntegral': '\xe2\xa8\x8d',
'HangulSyllableYaej': '\xec\x96\xae',
'HangulSyllableYaek': '\xec\x96\xb0',
'UnicodeCharacter0xd9ac': '\xed\xa6\xac',
'SquareUon': '\xe3\x8c\x86',
'HangulSyllableKwabs': '\xec\xbe\x82',
'CjkUnifiedIdeograph987a': '\xe9\xa1\xba',
'CjkUnifiedIdeograph987b': '\xe9\xa1\xbb',
'CjkUnifiedIdeograph987c': '\xe9\xa1\xbc',
'CjkUnifiedIdeograph987d': '\xe9\xa1\xbd',
'CjkUnifiedIdeograph987e': '\xe9\xa1\xbe',
'CjkUnifiedIdeograph987f': '\xe9\xa1\xbf',
'HangulSyllableDyaen': '\xeb\x8c\xbc',
'HangulSyllableSoes': '\xec\x87\xb3',
'CombiningDoubleTildeLeftHalf': '\xef\xb8\xa2',
'EightSpokedAsterisk': '\xe2\x9c\xb3',
'CjkUnifiedIdeograph536a': '\xe5\x8d\xaa',
'CjkUnifiedIdeograph536b': '\xe5\x8d\xab',
'CjkUnifiedIdeograph536c': '\xe5\x8d\xac',
'CjkUnifiedIdeograph536d': '\xe5\x8d\xad',
'CjkUnifiedIdeograph536e': '\xe5\x8d\xae',
'CjkUnifiedIdeograph536f': '\xe5\x8d\xaf',
'squared': '\xc2\xb2',
'ParenthesizedDigitSeven': '\xe2\x91\xba',
'WhiteCircleWithLowerLeftQuadrant': '\xe2\x97\xb5',
'CjkUnifiedIdeograph496a': '\xe4\xa5\xaa',
'UnicodeCharacter0xde9': '\xe0\xb7\xa9',
'CjkUnifiedIdeograph496c': '\xe4\xa5\xac',
'CjkUnifiedIdeograph496b': '\xe4\xa5\xab',
'CjkUnifiedIdeograph496e': '\xe4\xa5\xae',
'CjkUnifiedIdeograph496d': '\xe4\xa5\xad',
'CjkUnifiedIdeograph496f': '\xe4\xa5\xaf',
'LeftwardsDoubleArrowWithStroke': '\xe2\x87\x8d',
'UnicodeCharacter0xde1': '\xe0\xb7\xa1',
'UnicodeCharacter0xde2': '\xe0\xb7\xa2',
'UnicodeCharacter0xde3': '\xe0\xb7\xa3',
'UnicodeCharacter0xde4': '\xe0\xb7\xa4',
'UnicodeCharacter0xde5': '\xe0\xb7\xa5',
'UnicodeCharacter0xde6': '\xe0\xb7\xa6',
'UnicodeCharacter0xde7': '\xe0\xb7\xa7',
'KannadaLetterU': '\xe0\xb2\x89',
'HangulSyllableDaenh': '\xeb\x8c\x86',
'HangulSyllableHyoss': '\xed\x9a\xbc',
'GreekSmallLetterUpsilonWithDialytika': '\xcf\x8b',
'KannadaLetterE': '\xe0\xb2\x8e',
'KannadaLetterA': '\xe0\xb2\x85',
'KannadaLetterO': '\xe0\xb2\x92',
'HeavyBlackCurvedUpwardsAndRightwardsArrow': '\xe2\x9e\xa6',
'KannadaLetterI': '\xe0\xb2\x87',
'ProportionalTo': '\xe2\x88\x9d',
'CjkUnifiedIdeograph4960': '\xe4\xa5\xa0',
'CjkUnifiedIdeograph4963': '\xe4\xa5\xa3',
'CjkUnifiedIdeograph4962': '\xe4\xa5\xa2',
'CjkUnifiedIdeograph4965': '\xe4\xa5\xa5',
'CjkUnifiedIdeograph4964': '\xe4\xa5\xa4',
'CjkUnifiedIdeograph4967': '\xe4\xa5\xa7',
'CjkUnifiedIdeograph4966': '\xe4\xa5\xa6',
'CjkUnifiedIdeograph4969': '\xe4\xa5\xa9',
'UnicodeCharacter0xdea': '\xe0\xb7\xaa',
'UnicodeCharacter0xdeb': '\xe0\xb7\xab',
'CanadianSyllabicsLwa': '\xe1\x93\xa6',
'UnicodeCharacter0xded': '\xe0\xb7\xad',
'UnicodeCharacter0xdee': '\xe0\xb7\xae',
'UnicodeCharacter0xdef': '\xe0\xb7\xaf',
'CanadianSyllabicsLwe': '\xe1\x93\x9c',
'BottomSquareBracket': '\xe2\x8e\xb5',
'Kappa': '\xce\x9a',
':-(': '\xe2\x8c\xa2',
'UnicodeCharacter0xd93b': '\xed\xa4\xbb',
'UnicodeCharacter0xdc90': '\xed\xb2\x90',
'CjkUnifiedIdeograph6d69': '\xe6\xb5\xa9',
'MalayalamLetterPa': '\xe0\xb4\xaa',
'UnicodeCharacter0xdc91': '\xed\xb2\x91',
'CyrillicSmallLetterMonocularO': '\xea\x99\xa9',
'EthiopicSyllableQwaa': '\xe1\x89\x8b',
'UnicodeCharacter0xdf4d': '\xed\xbd\x8d',
's': '\xcf\x83',
'ArabicLetterTehIsolatedForm': '\xef\xba\x95',
'CjkUnifiedIdeograph5819': '\xe5\xa0\x99',
'UnicodeCharacter0xd93f': '\xed\xa4\xbf',
'ArabicLetterBehWithThreeDotsPointingUpwardsBelow': '\xdd\x92',
'CopticSymbolMiRo': '\xe2\xb3\xa5',
'HangulSyllableIlt': '\xec\x9e\x81',
'CjkUnifiedIdeograph6832': '\xe6\xa0\xb2',
'CjkUnifiedIdeograph7437': '\xe7\x90\xb7',
'CjkUnifiedIdeograph7434': '\xe7\x90\xb4',
'CjkUnifiedIdeograph7435': '\xe7\x90\xb5',
'CjkUnifiedIdeograph7432': '\xe7\x90\xb2',
'CjkUnifiedIdeograph689e': '\xe6\xa2\x9e',
'CjkUnifiedIdeograph7430': '\xe7\x90\xb0',
'LatinCapitalLetterLWithLineBelow': '\xe1\xb8\xba',
'CjkUnifiedIdeograph99a3': '\xe9\xa6\xa3',
'CjkUnifiedIdeograph99a2': '\xe9\xa6\xa2',
'UnicodeCharacter0x18ab': '\xe1\xa2\xab',
'UnicodeCharacter0x18ac': '\xe1\xa2\xac',
'UnicodeCharacter0x18ad': '\xe1\xa2\xad',
'UnicodeCharacter0x18ae': '\xe1\xa2\xae',
'UnicodeCharacter0x18af': '\xe1\xa2\xaf',
'CjkUnifiedIdeograph7439': '\xe7\x90\xb9',
'CjkUnifiedIdeograph8824': '\xe8\xa0\xa4',
'EthiopicSyllableKee': '\xe1\x8a\xac',
'ThaanaLetterGnaviyani': '\xde\x8f',
'CjkUnifiedIdeograph8825': '\xe8\xa0\xa5',
'HangulSyllableBbyel': '\xeb\xbc\xac',
'HangulSyllableDyenj': '\xeb\x8e\xad',
'CjkUnifiedIdeograph8826': '\xe8\xa0\xa6',
'HangulSyllableDyenh': '\xeb\x8e\xae',
'MyanmarVowelSignIi': '\xe1\x80\xae',
'CjkUnifiedIdeograph3d10': '\xe3\xb4\x90',
'CjkUnifiedIdeograph3d11': '\xe3\xb4\x91',
'CjkUnifiedIdeograph3d12': '\xe3\xb4\x92',
'CjkUnifiedIdeograph3d13': '\xe3\xb4\x93',
'CjkUnifiedIdeograph3d14': '\xe3\xb4\x94',
'CjkUnifiedIdeograph3d15': '\xe3\xb4\x95',
'CjkUnifiedIdeograph3d16': '\xe3\xb4\x96',
'CjkUnifiedIdeograph3d17': '\xe3\xb4\x97',
'CjkUnifiedIdeograph3d18': '\xe3\xb4\x98',
'CjkUnifiedIdeograph3d19': '\xe3\xb4\x99',
'CjkUnifiedIdeograph6f94': '\xe6\xbe\x94',
'HangulSyllableBbyeo': '\xeb\xbc\x88',
'HangulSyllableJjweon': '\xec\xad\xa8',
'CjkUnifiedIdeograph8a09': '\xe8\xa8\x89',
'CjkUnifiedIdeograph8821': '\xe8\xa0\xa1',
'CjkUnifiedIdeograph9553': '\xe9\x95\x93',
'CjkUnifiedIdeograph8822': '\xe8\xa0\xa2',
'CjkUnifiedIdeograph43a7': '\xe4\x8e\xa7',
'CjkUnifiedIdeograph8823': '\xe8\xa0\xa3',
'CjkUnifiedIdeograph43a4': '\xe4\x8e\xa4',
'VerticalBarWithHorizontalStroke': '\xe2\x9f\x8a',
'UnicodeCharacter0xdaf9': '\xed\xab\xb9',
'CjkUnifiedIdeograph43a5': '\xe4\x8e\xa5',
'BoxDrawingsUpHeavyAndRightLight': '\xe2\x94\x96',
'CjkUnifiedIdeograph9557': '\xe9\x95\x97',
'DoesNotPrecede': '\xe2\x8a\x80',
'BuhidLetterRa': '\xe1\x9d\x8d',
'CjkUnifiedIdeograph9556': '\xe9\x95\x96',
'CjkUnifiedIdeograph743f': '\xe7\x90\xbf',
'CjkUnifiedIdeograph743d': '\xe7\x90\xbd',
'CjkUnifiedIdeograph743e': '\xe7\x90\xbe',
'CjkUnifiedIdeograph743b': '\xe7\x90\xbb',
'CjkUnifiedIdeograph743c': '\xe7\x90\xbc',
'CjkUnifiedIdeograph743a': '\xe7\x90\xba',
'CjkUnifiedIdeograph99ac': '\xe9\xa6\xac',
'CjkUnifiedIdeograph99ab': '\xe9\xa6\xab',
'CjkUnifiedIdeograph91d2': '\xe9\x87\x92',
'CjkUnifiedIdeograph4683': '\xe4\x9a\x83',
'CjkUnifiedIdeograph99af': '\xe9\xa6\xaf',
'CjkUnifiedIdeograph99ae': '\xe9\xa6\xae',
'BoxDrawingsLightUp': '\xe2\x95\xb5',
'HangulSyllableMyalt': '\xeb\xa8\x8d',
'HangulSyllableMyalp': '\xeb\xa8\x8e',
'ArabicLetterHahWithHamzaAbove': '\xda\x81',
'ModifierLetterLeftArrowhead': '\xcb\x82',
'CjkUnifiedIdeograph3d1a': '\xe3\xb4\x9a',
'KangxiRadicalMountain': '\xe2\xbc\xad',
'CjkUnifiedIdeograph3d1c': '\xe3\xb4\x9c',
'CjkUnifiedIdeograph3d1d': '\xe3\xb4\x9d',
'CjkUnifiedIdeograph3d1e': '\xe3\xb4\x9e',
'CjkUnifiedIdeograph3d1f': '\xe3\xb4\x9f',
'HangulSyllableMyalb': '\xeb\xa8\x8b',
'HangulSyllableMyalm': '\xeb\xa8\x8a',
'CjkUnifiedIdeograph40a8': '\xe4\x82\xa8',
'CanadianSyllabicsSayisiJi': '\xe1\x98\x9a',
'UnicodeCharacter0xf706': '\xef\x9c\x86',
'HangulSyllableJjaegg': '\xec\xa7\xba',
'CjkUnifiedIdeograph65af': '\xe6\x96\xaf',
'LatinCapitalLetterGWithMacron': '\xe1\xb8\xa0',
'OrnateLeftParenthesis': '\xef\xb4\xbe',
'UnicodeCharacter0xf700': '\xef\x9c\x80',
'ArabicLigatureQafWithMeemWithMeemFinalForm': '\xef\xb5\xbf',
'CjkUnifiedIdeograph96a8': '\xe9\x9a\xa8',
'CjkUnifiedIdeograph65ad': '\xe6\x96\xad',
'CjkUnifiedIdeograph605f': '\xe6\x81\x9f',
'CjkUnifiedIdeograph96a4': '\xe9\x9a\xa4',
'CjkUnifiedIdeograph96a5': '\xe9\x9a\xa5',
'CjkUnifiedIdeograph96a6': '\xe9\x9a\xa6',
'CjkUnifiedIdeograph96a7': '\xe9\x9a\xa7',
'CjkUnifiedIdeograph96a0': '\xe9\x9a\xa0',
'CanadianSyllabicsSayisiJu': '\xe1\x98\x95',
'CjkUnifiedIdeograph96a2': '\xe9\x9a\xa2',
'CjkUnifiedIdeograph96a3': '\xe9\x9a\xa3',
'UnicodeCharacter0xe6df': '\xee\x9b\x9f',
'HangulSyllableYaenj': '\xec\x96\x9d',
'HangulSyllableBbyes': '\xeb\xbc\xb7',
'CjkUnifiedIdeograph60e2': '\xe6\x83\xa2',
'YiSyllableFix': '\xea\x83\x8e',
'UnicodeCharacter0xf4f5': '\xef\x93\xb5',
'LatinSmallLetterYWithDotAbove': '\xe1\xba\x8f',
'HangulSyllableYaeng': '\xec\x96\xad',
'UnicodeCharacter0xf4f6': '\xef\x93\xb6',
'CjkUnifiedIdeograph605b': '\xe6\x81\x9b',
'ModifierLetterSmallTWithPalatalHook': '\xe1\xb6\xb5',
'CjkUnifiedIdeograph6d6b': '\xe6\xb5\xab',
'CircledNumberThirtyTwo': '\xe3\x89\x9c',
'CjkUnifiedIdeograph96ad': '\xe9\x9a\xad',
'CjkRadicalKnifeTwo': '\xe2\xba\x89',
'CjkUnifiedIdeograph96af': '\xe9\x9a\xaf',
'YiSyllableKat': '\xea\x87\xbf',
'CjkUnifiedIdeograph96aa': '\xe9\x9a\xaa',
'CjkUnifiedIdeograph7d94': '\xe7\xb6\x94',
'CjkUnifiedIdeograph96ac': '\xe9\x9a\xac',
'CjkUnifiedIdeograph6541': '\xe6\x95\x81',
'YiSyllableSsat': '\xea\x8c\xb1',
'CjkUnifiedIdeograph6d6f': '\xe6\xb5\xaf',
'CjkRadicalSecondThree': '\xe2\xba\x84',
'HangulSyllableDugg': '\xeb\x91\x92',
'HangulSyllableByeolg': '\xeb\xb3\x85',
'HangulChoseongHieuh': '\xe1\x84\x92',
'HangulSyllableByeolb': '\xeb\xb3\x87',
'HangulSyllableByeolm': '\xeb\xb3\x86',
'HangulSyllableByeolh': '\xeb\xb3\x8b',
'HangulSyllableTying': '\xed\x8b\xa9',
'HangulSyllableByeolt': '\xeb\xb3\x89',
'HangulSyllableByeolp': '\xeb\xb3\x8a',
'HangulSyllableDugs': '\xeb\x91\x93',
'HangulSyllableByeols': '\xeb\xb3\x88',
'HangulSyllableTugs': '\xed\x88\xaf',
'LatinSmallLetterReversedOpenEWithHook': '\xc9\x9d',
'HangulSyllableDdeut': '\xeb\x9d\x81',
'HangulSyllableCenh': '\xec\xb2\xba',
'UnicodeCharacter0xdc9a': '\xed\xb2\x9a',
'UnicodeCharacter0xd934': '\xed\xa4\xb4',
'UnicodeCharacter0xfb18': '\xef\xac\x98',
'HangulSyllableCeng': '\xec\xb3\x89',
'UnicodeCharacter0xdc9b': '\xed\xb2\x9b',
'UnicodeCharacter0xd935': '\xed\xa4\xb5',
'UnicodeCharacter0xf875': '\xef\xa1\xb5',
'UnicodeCharacter0xdc9c': '\xed\xb2\x9c',
'HangulSyllableDdeup': '\xeb\x9d\x82',
'UnicodeCharacter0xd937': '\xed\xa4\xb7',
'CjkUnifiedIdeograph6896': '\xe6\xa2\x96',
'GreekSmallLetterEpsilonWithPsiliAndVaria': '\xe1\xbc\x92',
'UnicodeCharacter0xdc9f': '\xed\xb2\x9f',
'ArabicLigatureKafWithMeemWithYehFinalForm': '\xef\xb6\xb7',
'HangulSyllableJebs': '\xec\xa0\xae',
'UnicodeCharacter0xe0fb': '\xee\x83\xbb',
'GeorgianLetterYn': '\xe1\x83\xb7',
'RejangLetterJa': '\xea\xa4\xba',
'CyrillicCapitalLetterSoftSign': '\xd0\xac',
'CjkUnifiedIdeograph34f9': '\xe3\x93\xb9',
'CjkUnifiedIdeograph34f8': '\xe3\x93\xb8',
'YiSyllableHax': '\xea\x89\xb2',
'CjkUnifiedIdeograph581c': '\xe5\xa0\x9c',
'CjkUnifiedIdeograph34f1': '\xe3\x93\xb1',
'CyrillicSmallLetterTshe': '\xd1\x9b',
'CjkUnifiedIdeograph34f3': '\xe3\x93\xb3',
'CjkUnifiedIdeograph34f2': '\xe3\x93\xb2',
'CjkUnifiedIdeograph34f5': '\xe3\x93\xb5',
'CjkUnifiedIdeograph34f4': '\xe3\x93\xb4',
'CjkUnifiedIdeograph34f7': '\xe3\x93\xb7',
'CjkUnifiedIdeograph34f6': '\xe3\x93\xb6',
'HangulSyllableJjyaeb': '\xec\xa9\x81',
'HangulSyllableJjyaec': '\xec\xa9\x87',
'HangulSyllableRyilh': '\xeb\xa6\x9f',
'ModifierLetterSmallMWithHook': '\xe1\xb6\xac',
'HangulSyllableJjyaeg': '\xec\xa8\xb1',
'HangulSyllableJjyaed': '\xec\xa8\xb7',
'HangulSyllableRyilm': '\xeb\xa6\x9a',
'HangulSyllableRyilb': '\xeb\xa6\x9b',
'HangulSyllableJjyaek': '\xec\xa9\x88',
'HangulSyllableJjyaeh': '\xec\xa9\x8b',
'HangulSyllableJjyaen': '\xec\xa8\xb4',
'HangulSyllableRyilg': '\xeb\xa6\x99',
'HangulSyllableJjyael': '\xec\xa8\xb8',
'CjkUnifiedIdeograph882a': '\xe8\xa0\xaa',
'HangulSyllableJjyaes': '\xec\xa9\x83',
'HangulSyllableJjyaep': '\xec\xa9\x8a',
'HangulSyllableYan': '\xec\x96\x80',
'CjkUnifiedIdeograph882b': '\xe8\xa0\xab',
'HangulSyllableJjyaet': '\xec\xa9\x89',
'BengaliLetterCha': '\xe0\xa6\x9b',
'HangulSyllableRyils': '\xeb\xa6\x9c',
'HangulSyllableRyilp': '\xeb\xa6\x9e',
'CjkUnifiedIdeograph882c': '\xe8\xa0\xac',
'CjkUnifiedIdeograph79e3': '\xe7\xa7\xa3',
'HangulSyllableRyilt': '\xeb\xa6\x9d',
'HangulSyllableSibs': '\xec\x8b\xae',
'KangxiRadicalMan': '\xe2\xbc\x88',
'DentistrySymbolLightUpAndHorizontal': '\xe2\x8f\x8a',
'CjkUnifiedIdeograph34fa': '\xe3\x93\xba',
'CjkUnifiedIdeograph34fc': '\xe3\x93\xbc',
'CjkUnifiedIdeograph34fb': '\xe3\x93\xbb',
'CjkUnifiedIdeograph34fe': '\xe3\x93\xbe',
'CjkUnifiedIdeograph34fd': '\xe3\x93\xbd',
'CjkUnifiedIdeograph34ff': '\xe3\x93\xbf',
'YiSyllableMuox': '\xea\x82\xba',
'CjkUnifiedIdeograph9d91': '\xe9\xb6\x91',
'CjkUnifiedIdeograph9c08': '\xe9\xb0\x88',
'UnicodeCharacter0xf0c5': '\xef\x83\x85',
'ArabicLetterMeemFinalForm': '\xef\xbb\xa2',
'CjkUnifiedIdeograph79e6': '\xe7\xa7\xa6',
'LimbuVowelSignOo': '\xe1\xa4\xa5',
'HangulSyllablePyen': '\xed\x8f\x94',
'CjkUnifiedIdeograph6bdf': '\xe6\xaf\x9f',
'CjkUnifiedIdeograph6bdd': '\xe6\xaf\x9d',
'CjkUnifiedIdeograph6bde': '\xe6\xaf\x9e',
'CjkUnifiedIdeograph6bdb': '\xe6\xaf\x9b',
'CjkUnifiedIdeograph6bdc': '\xe6\xaf\x9c',
'CjkUnifiedIdeograph6bda': '\xe6\xaf\x9a',
'DownwardsDoubleArrow': '\xe2\x87\x93',
'HangulSyllablePyel': '\xed\x8f\x98',
'CjkUnifiedIdeograph9c09': '\xe9\xb0\x89',
'HangulSyllableDdeum': '\xeb\x9c\xb8',
'SquareNano': '\xe3\x8c\xa8',
'CjkUnifiedIdeograph7d93': '\xe7\xb6\x93',
'HangulSyllableRoeng': '\xeb\xa3\x85',
'HangulSyllableBwils': '\xeb\xb7\xa0',
'LatinCapitalLetterRWithDotBelow': '\xe1\xb9\x9a',
'HangulSyllableDdeuh': '\xeb\x9d\x83',
'HangulSyllablePyeh': '\xed\x8f\xab',
'ModifierLetterLowDottedToneBar': '\xea\x9c\x8b',
'HangulSyllableGginh': '\xeb\x82\x82',
'HangulSyllableDdeuj': '\xeb\x9c\xbe',
'RighttoleftMark': '\xe2\x80\x8f',
'HangulSyllableDdeuk': '\xeb\x9d\x80',
'HangulSyllableMyenj': '\xeb\xaa\x91',
'UnicodeCharacter0xeea8': '\xee\xba\xa8',
'HangulSyllableSeob': '\xec\x84\xad',
'HangulSyllableSeoc': '\xec\x84\xb3',
'HangulSyllablePyeb': '\xed\x8f\xa1',
'UnicodeCharacter0xab7d': '\xea\xad\xbd',
'HangulSyllableSwass': '\xec\x86\xbc',
'CjkUnifiedIdeograph6bd8': '\xe6\xaf\x98',
'CjkUnifiedIdeograph6bd9': '\xe6\xaf\x99',
'CjkUnifiedIdeograph6bd6': '\xe6\xaf\x96',
'CjkUnifiedIdeograph6bd7': '\xe6\xaf\x97',
'CjkUnifiedIdeograph6bd4': '\xe6\xaf\x94',
'CjkUnifiedIdeograph6bd5': '\xe6\xaf\x95',
'CjkUnifiedIdeograph6bd2': '\xe6\xaf\x92',
'CjkUnifiedIdeograph6bd3': '\xe6\xaf\x93',
'CjkUnifiedIdeograph6bd0': '\xe6\xaf\x90',
'CjkUnifiedIdeograph6bd1': '\xe6\xaf\x91',
'UnicodeCharacter0xeea6': '\xee\xba\xa6',
'UnicodeCharacter0xf4fb': '\xef\x93\xbb',
'UnicodeCharacter0xf5ee': '\xef\x97\xae',
'ThaanaLetterZaa': '\xde\x9c',
'HangulSyllableSeoh': '\xec\x84\xb7',
'UnicodeCharacter0xf4fc': '\xef\x93\xbc',
'SubscriptEight': '\xe2\x82\x88',
'CjkUnifiedIdeograph5e21': '\xe5\xb8\xa1',
'HangulSyllableBbaen': '\xeb\xba\x80',
'HangulSyllableGonj': '\xea\xb3\xa5',
'CjkUnifiedIdeograph584a': '\xe5\xa1\x8a',
'CjkUnifiedIdeograph9b21': '\xe9\xac\xa1',
'EthiopicSyllableDdoa': '\xe2\xb6\x8d',
'MalayalamLetterChilluL': '\xe0\xb5\xbd',
'MalayalamLetterChilluN': '\xe0\xb5\xbb',
'CjkUnifiedIdeograph5e25': '\xe5\xb8\xa5',
'CjkUnifiedIdeograph5c7f': '\xe5\xb1\xbf',
'CjkUnifiedIdeograph8253': '\xe8\x89\x93',
'CjkUnifiedIdeograph8252': '\xe8\x89\x92',
'CjkUnifiedIdeograph8251': '\xe8\x89\x91',
'CjkUnifiedIdeograph8250': '\xe8\x89\x90',
'ThaanaLetterNaa': '\xde\xb1',
'CjkUnifiedIdeograph5c7e': '\xe5\xb1\xbe',
'CjkUnifiedIdeograph8255': '\xe8\x89\x95',
'CjkUnifiedIdeograph8254': '\xe8\x89\x94',
'HangulSyllableMyeng': '\xeb\xaa\xa1',
'CjkUnifiedIdeograph8259': '\xe8\x89\x99',
'CjkUnifiedIdeograph8258': '\xe8\x89\x98',
'UnicodeCharacter0xe9bf': '\xee\xa6\xbf',
'HangulSyllableYeolp': '\xec\x97\xba',
'HangulSyllablePyes': '\xed\x8f\xa3',
'HangulSyllableGgyibs': '\xeb\x81\xb2',
'ArabicLigatureSeenWithJeemWithAlefMaksuraFinalForm': '\xef\xb5\x9e',
'HangulSyllablePoess': '\xed\x91\x94',
'CjkUnifiedIdeograph65a9': '\xe6\x96\xa9',
'CjkRadicalCsimplifiedSpeech': '\xe2\xbb\x88',
'HangulSyllableSsweob': '\xec\x92\x91',
'HangulSyllableCyigs': '\xec\xb8\xbf',
'HangulSyllablePyep': '\xed\x8f\xaa',
'CjkUnifiedIdeograph825c': '\xe8\x89\x9c',
'CjkUnifiedIdeograph825b': '\xe8\x89\x9b',
'CjkUnifiedIdeograph825a': '\xe8\x89\x9a',
'Join': '\xe2\xa8\x9d',
'CjkUnifiedIdeograph825f': '\xe8\x89\x9f',
'CjkUnifiedIdeograph825e': '\xe8\x89\x9e',
'CjkUnifiedIdeograph825d': '\xe8\x89\x9d',
'UnicodeCharacter0xfb1c': '\xef\xac\x9c',
'HalfwidthRightCornerBracket': '\xef\xbd\xa3',
'UnicodeCharacter0xf70e': '\xef\x9c\x8e',
'CjkUnifiedIdeograph9b22': '\xe9\xac\xa2',
'UnicodeCharacter0xf70d': '\xef\x9c\x8d',
'BraillePatternDots1378': '\xe2\xa3\x85',
'UnicodeCharacter0xf70f': '\xef\x9c\x8f',
'UnicodeCharacter0xe6ee': '\xee\x9b\xae',
'HalfwidthKatakanaLetterWa': '\xef\xbe\x9c',
'UnicodeCharacter0xf70a': '\xef\x9c\x8a',
'UnicodeCharacter0xdfc7': '\xed\xbf\x87',
'CjkUnifiedIdeograph584e': '\xe5\xa1\x8e',
'GeorgianLetterQar': '\xe1\x83\xa7',
'UnicodeCharacter0xf70c': '\xef\x9c\x8c',
'UnicodeCharacter0xf70b': '\xef\x9c\x8b',
'HangulSyllableHyubs': '\xed\x9d\x86',
'HangulJongseongSsangsios': '\xe1\x86\xbb',
'CjkUnifiedIdeograph59fe': '\xe5\xa7\xbe',
'BengaliSignVisarga': '\xe0\xa6\x83',
'UnicodeCharacter0xe2d4': '\xee\x8b\x94',
'UnicodeCharacter0x1a7c': '\xe1\xa9\xbc',
'UnicodeCharacter0x1a7b': '\xe1\xa9\xbb',
'UnicodeCharacter0x1a7a': '\xe1\xa9\xba',
'UnicodeCharacter0x1a7f': '\xe1\xa9\xbf',
'UnicodeCharacter0x1a7e': '\xe1\xa9\xbe',
'UnicodeCharacter0x1a7d': '\xe1\xa9\xbd',
'GurmukhiLetterPha': '\xe0\xa8\xab',
'HangulSyllableHyilt': '\xed\x9d\xb9',
'WhiteFrowningFace': '\xe2\x98\xb9',
'SouthWestArrowWithHook': '\xe2\xa4\xa6',
'PesoSign': '\xe2\x82\xb1',
'ThreeLeftwardsArrows': '\xe2\xac\xb1',
'YiSyllableDdiep': '\xea\x84\xbc',
'UnicodeCharacter0x1a73': '\xe1\xa9\xb3',
'UnicodeCharacter0x1a72': '\xe1\xa9\xb2',
'SquareKcal': '\xe3\x8e\x89',
'UnicodeCharacter0x1a70': '\xe1\xa9\xb0',
'UnicodeCharacter0x1a77': '\xe1\xa9\xb7',
'UnicodeCharacter0x1a76': '\xe1\xa9\xb6',
'UnicodeCharacter0x1a75': '\xe1\xa9\xb5',
'UnicodeCharacter0x1a74': '\xe1\xa9\xb4',
'SylotiNagriPoetryMark1': '\xea\xa0\xa8',
'UnicodeCharacter0x1a79': '\xe1\xa9\xb9',
'UnicodeCharacter0x1a78': '\xe1\xa9\xb8',
'ArabicLetterPehehMedialForm': '\xef\xad\xb1',
'ArmenianSmallLetterPeh': '\xd5\xba',
'HangulSyllableKyags': '\xec\xba\xaf',
'BengaliVowelSignVocalicRr': '\xe0\xa7\x84',
'UnicodeCharacter0xab78': '\xea\xad\xb8',
'YiSyllableDdiex': '\xea\x84\xba',
'GreekCapitalLetterAlphaWithPsiliAndOxia': '\xe1\xbc\x8c',
'UnicodeCharacter0xde8c': '\xed\xba\x8c',
'UnicodeCharacter0xab79': '\xea\xad\xb9',
'UnicodeCharacter0xde8b': '\xed\xba\x8b',
'LogicalAndWithUnderbar': '\xe2\xa9\x9f',
'UnicodeCharacter0xde8e': '\xed\xba\x8e',
'LatinSmallLetterUWithRetroflexHook': '\xe1\xb6\x99',
'UnicodeCharacter0xde8d': '\xed\xba\x8d',
'UnicodeCharacter0xfde0': '\xef\xb7\xa0',
'HalfwidthHangulLetterYo': '\xef\xbf\x92',
'UnicodeCharacter0xed86': '\xee\xb6\x86',
'UnicodeCharacter0xebda': '\xee\xaf\x9a',
'OriyaLetterDda': '\xe0\xac\xa1',
'AplFunctionalSymbolDeltaStile': '\xe2\x8d\x8b',
'UnicodeCharacter0xab71': '\xea\xad\xb1',
'CjkUnifiedIdeograph63fa': '\xe6\x8f\xba',
'CjkUnifiedIdeograph63fc': '\xe6\x8f\xbc',
'CjkUnifiedIdeograph63fb': '\xe6\x8f\xbb',
'CjkUnifiedIdeograph63fe': '\xe6\x8f\xbe',
'CjkUnifiedIdeograph63fd': '\xe6\x8f\xbd',
'CjkUnifiedIdeograph63ff': '\xe6\x8f\xbf',
'UnicodeCharacter0xf8bd': '\xef\xa2\xbd',
'CjkUnifiedIdeograph5e2f': '\xe5\xb8\xaf',
'DiamondOperator': '\xe2\x8b\x84',
'CjkUnifiedIdeograph6a81': '\xe6\xaa\x81',
'CjkUnifiedIdeograph6a80': '\xe6\xaa\x80',
'CjkUnifiedIdeograph6a83': '\xe6\xaa\x83',
'CjkUnifiedIdeograph6a82': '\xe6\xaa\x82',
'CjkUnifiedIdeograph6a85': '\xe6\xaa\x85',
'CjkUnifiedIdeograph6a84': '\xe6\xaa\x84',
'CjkUnifiedIdeograph6a87': '\xe6\xaa\x87',
'CjkUnifiedIdeograph6a86': '\xe6\xaa\x86',
'CjkUnifiedIdeograph6a89': '\xe6\xaa\x89',
'CjkUnifiedIdeograph6a88': '\xe6\xaa\x88',
'HangulSyllableGyabs': '\xea\xb1\x8a',
'HangulSyllableRweop': '\xeb\xa4\x9e',
'ArabicLetterSheenInitialForm': '\xef\xba\xb7',
'CjkUnifiedIdeograph6d5c': '\xe6\xb5\x9c',
'CjkUnifiedIdeograph6d5b': '\xe6\xb5\x9b',
'CjkUnifiedIdeograph6d5a': '\xe6\xb5\x9a',
'CjkUnifiedIdeograph7a2c': '\xe7\xa8\xac',
'MyanmarLetterJha': '\xe1\x80\x88',
'CjkUnifiedIdeograph6d5f': '\xe6\xb5\x9f',
'CjkUnifiedIdeograph6d5e': '\xe6\xb5\x9e',
'CjkUnifiedIdeograph6d5d': '\xe6\xb5\x9d',
'CjkUnifiedIdeograph63f1': '\xe6\x8f\xb1',
'CjkUnifiedIdeograph63f0': '\xe6\x8f\xb0',
'CjkUnifiedIdeograph63f3': '\xe6\x8f\xb3',
'BraillePatternDots278': '\xe2\xa3\x82',
'CjkUnifiedIdeograph63f5': '\xe6\x8f\xb5',
'CjkUnifiedIdeograph63f4': '\xe6\x8f\xb4',
'CjkUnifiedIdeograph63f7': '\xe6\x8f\xb7',
'CjkUnifiedIdeograph63f6': '\xe6\x8f\xb6',
'CjkUnifiedIdeograph63f9': '\xe6\x8f\xb9',
'CjkUnifiedIdeograph63f8': '\xe6\x8f\xb8',
'DevanagariSignHighSpacingDot': '\xe0\xa5\xb1',
'NotAlmostEqualTo': '\xe2\x89\x89',
'HangulSyllableCyeoss': '\xec\xb3\xa4',
'CjkUnifiedIdeograph8795': '\xe8\x9e\x95',
'UnicodeCharacter0xfde5': '\xef\xb7\xa5',
'UnicodeCharacter0xed83': '\xee\xb6\x83',
'HangulSyllableGyugs': '\xea\xb7\x9f',
'CjkUnifiedIdeograph7a28': '\xe7\xa8\xa8',
'CjkUnifiedIdeograph7a29': '\xe7\xa8\xa9',
'CjkUnifiedIdeograph6d59': '\xe6\xb5\x99',
'CjkUnifiedIdeograph6d58': '\xe6\xb5\x98',
'CopticCapitalLetterOldCopticHei': '\xe2\xb3\x92',
'CjkUnifiedIdeograph6d53': '\xe6\xb5\x93',
'CjkUnifiedIdeograph6d52': '\xe6\xb5\x92',
'CjkUnifiedIdeograph6d51': '\xe6\xb5\x91',
'CjkUnifiedIdeograph6d50': '\xe6\xb5\x90',
'CjkUnifiedIdeograph6d57': '\xe6\xb5\x97',
'CjkUnifiedIdeograph6d56': '\xe6\xb5\x96',
'CjkUnifiedIdeograph6d55': '\xe6\xb5\x95',
'CjkUnifiedIdeograph6d54': '\xe6\xb5\x94',
'SquareGpa': '\xe3\x8e\xac',
'CjkUnifiedIdeograph6a8c': '\xe6\xaa\x8c',
'CjkUnifiedIdeograph6a8b': '\xe6\xaa\x8b',
'CjkUnifiedIdeograph6a8e': '\xe6\xaa\x8e',
'CjkUnifiedIdeograph6a8d': '\xe6\xaa\x8d',
'CjkUnifiedIdeograph6a8f': '\xe6\xaa\x8f',
'CjkUnifiedIdeograph650f': '\xe6\x94\x8f',
'CjkUnifiedIdeograph70a7': '\xe7\x82\xa7',
'HangulSyllableDdweolm': '\xeb\x9b\x82',
'TibetanDigitNine': '\xe0\xbc\xa9',
'CjkUnifiedIdeograph80db': '\xe8\x83\x9b',
'CircledKatakanaKu': '\xe3\x8b\x97',
'HangulSyllableNyaelh': '\xeb\x83\xbb',
'HangulSyllableWeolp': '\xec\x9b\x9a',
'HangulSyllableNyaelg': '\xeb\x83\xb5',
'HangulSyllableSeogg': '\xec\x84\x9e',
'HangulSyllableNyaelb': '\xeb\x83\xb7',
'HangulSyllableRot': '\xeb\xa1\xb5',
'UnicodeCharacter0x2bca': '\xe2\xaf\x8a',
'HangulSyllablePaeb': '\xed\x8c\xb9',
'UnicodeCharacter0x2bcc': '\xe2\xaf\x8c',
'CircledKatakanaKe': '\xe3\x8b\x98',
'UnicodeCharacter0x2bce': '\xe2\xaf\x8e',
'UnicodeCharacter0x2bcd': '\xe2\xaf\x8d',
'CircledKatakanaKa': '\xe3\x8b\x95',
'CjkUnifiedIdeograph5e1c': '\xe5\xb8\x9c',
'CircledKatakanaKo': '\xe3\x8b\x99',
'CjkUnifiedIdeograph5e1a': '\xe5\xb8\x9a',
'HangulSyllableDdweolh': '\xeb\x9b\x87',
'CjkUnifiedIdeograph5e1f': '\xe5\xb8\x9f',
'CjkUnifiedIdeograph5e1e': '\xe5\xb8\x9e',
'CircledKatakanaKi': '\xe3\x8b\x96',
'HangulSyllableBbyelp': '\xeb\xbc\xb2',
'HangulSyllableBbyels': '\xeb\xbc\xb0',
'HangulSyllableGgyaelb': '\xea\xba\xab',
'CjkUnifiedIdeograph80dd': '\xe8\x83\x9d',
'HangulSyllableGgyaelg': '\xea\xba\xa9',
'HangulSyllableJjeugs': '\xec\xaf\x97',
'HangulSyllableGgyaelh': '\xea\xba\xaf',
'UnicodeCharacter0xd98b': '\xed\xa6\x8b',
'HangulSyllableGgyaelm': '\xea\xba\xaa',
'UnicodeCharacter0xcfb': '\xe0\xb3\xbb',
'UnicodeCharacter0xcfc': '\xe0\xb3\xbc',
'HangulSyllableGgyaels': '\xea\xba\xac',
'UnicodeCharacter0xcfa': '\xe0\xb3\xba',
'UnicodeCharacter0xcff': '\xe0\xb3\xbf',
'HangulSyllableGgyaelt': '\xea\xba\xad',
'UnicodeCharacter0xcfd': '\xe0\xb3\xbd',
'UnicodeCharacter0xcfe': '\xe0\xb3\xbe',
'HangulSyllableBbyelh': '\xeb\xbc\xb3',
'HangulSyllableBbyelm': '\xeb\xbc\xae',
'UnicodeCharacter0xede6': '\xee\xb7\xa6',
'OriyaVowelSignVocalicLl': '\xe0\xad\xa3',
'MongolianDigitZero': '\xe1\xa0\x90',
'UnicodeCharacter0x2bc1': '\xe2\xaf\x81',
'UnicodeCharacter0x2bc0': '\xe2\xaf\x80',
'UnicodeCharacter0x2bc3': '\xe2\xaf\x83',
'UnicodeCharacter0x2bc2': '\xe2\xaf\x82',
'UnicodeCharacter0x2bc5': '\xe2\xaf\x85',
'UnicodeCharacter0x2bc4': '\xe2\xaf\x84',
'UnicodeCharacter0x2bc7': '\xe2\xaf\x87',
'UnicodeCharacter0x2bc6': '\xe2\xaf\x86',
'UnicodeCharacter0x2bc9': '\xe2\xaf\x89',
'UnicodeCharacter0x2bc8': '\xe2\xaf\x88',
'CjkUnifiedIdeograph5e11': '\xe5\xb8\x91',
'CjkUnifiedIdeograph5e10': '\xe5\xb8\x90',
'CjkUnifiedIdeograph5e17': '\xe5\xb8\x97',
'CjkUnifiedIdeograph5e16': '\xe5\xb8\x96',
'CjkUnifiedIdeograph5e15': '\xe5\xb8\x95',
'MalayalamLetterCha': '\xe0\xb4\x9b',
'UnicodeCharacter0xede2': '\xee\xb7\xa2',
'ThaanaLetterThaa': '\xde\x8c',
'ShortUpTackWithUnderbar': '\xe2\xab\xa8',
'UnicodeCharacter0xcf3': '\xe0\xb3\xb3',
'UnicodeCharacter0xcf0': '\xe0\xb3\xb0',
'UnicodeCharacter0xcf6': '\xe0\xb3\xb6',
'UnicodeCharacter0xcf7': '\xe0\xb3\xb7',
'UnicodeCharacter0xcf4': '\xe0\xb3\xb4',
'UnicodeCharacter0xcf5': '\xe0\xb3\xb5',
'UnicodeCharacter0xcf8': '\xe0\xb3\xb8',
'UnicodeCharacter0xcf9': '\xe0\xb3\xb9',
'HangulSyllableTwe': '\xed\x89\xa4',
'CanadianSyllabicsCwi': '\xe1\x92\x94',
'HangulSyllableMegg': '\xeb\xa9\x96',
'HangulSyllableTwa': '\xed\x86\xbc',
'LatinSmallLetterSWithDotAbove': '\xe1\xb9\xa1',
'ArabicLetterBehehMedialForm': '\xef\xad\x9d',
'HangulSyllableTwi': '\xed\x8a\x80',
'UnicodeCharacter0xde83': '\xed\xba\x83',
'UnicodeCharacter0xf297': '\xef\x8a\x97',
'UnicodeCharacter0xf296': '\xef\x8a\x96',
'UnicodeCharacter0xf295': '\xef\x8a\x95',
'UnicodeCharacter0xde82': '\xed\xba\x82',
'UnicodeCharacter0xf293': '\xef\x8a\x93',
'HangulSyllableJong': '\xec\xa2\x85',
'UnicodeCharacter0xf291': '\xef\x8a\x91',
'UnicodeCharacter0xf290': '\xef\x8a\x90',
'HangulSyllableJonj': '\xec\xa1\xb5',
'UnicodeCharacter0xde85': '\xed\xba\x85',
'LatinCapitalLetterIWithDiaeresis': '\xc3\x8f',
'UnicodeCharacter0xf299': '\xef\x8a\x99',
'HangulSyllableRebs': '\xeb\xa0\x9a',
'UnicodeCharacter0x1df9': '\xe1\xb7\xb9',
'UnicodeCharacter0x1df8': '\xe1\xb7\xb8',
'CombiningLatinSmallLetterCCedilla': '\xe1\xb7\x97',
'UnicodeCharacter0x1df5': '\xe1\xb7\xb5',
'UpwardsWhiteArrowFromBar': '\xe2\x87\xaa',
'UnicodeCharacter0x1df7': '\xe1\xb7\xb7',
'UnicodeCharacter0x1df6': '\xe1\xb7\xb6',
'UnicodeCharacter0x1df1': '\xe1\xb7\xb1',
'UnicodeCharacter0x1df0': '\xe1\xb7\xb0',
'UnicodeCharacter0x1df3': '\xe1\xb7\xb3',
'UnicodeCharacter0x1df2': '\xe1\xb7\xb2',
'CjkUnifiedIdeograph7019': '\xe7\x80\x99',
'CjkUnifiedIdeograph37bb': '\xe3\x9e\xbb',
'CjkUnifiedIdeograph37bc': '\xe3\x9e\xbc',
'CjkUnifiedIdeograph37ba': '\xe3\x9e\xba',
'CjkUnifiedIdeograph37bf': '\xe3\x9e\xbf',
'CjkUnifiedIdeograph37bd': '\xe3\x9e\xbd',
'CjkUnifiedIdeograph37be': '\xe3\x9e\xbe',
'CanadianSyllabicsLwii': '\xe1\x93\xa0',
'UnicodeCharacter0x1dfd': '\xe1\xb7\xbd',
'UnicodeCharacter0x1dfa': '\xe1\xb7\xba',
'LatinSmallLetterVWithTilde': '\xe1\xb9\xbd',
'UnicodeCharacter0x1dfc': '\xe1\xb7\xbc',
'UnicodeCharacter0x1dfb': '\xe1\xb7\xbb',
'UnicodeCharacter0xf29f': '\xef\x8a\x9f',
'BraillePatternDots235678': '\xe2\xa3\xb6',
'UnicodeCharacter0xf29d': '\xef\x8a\x9d',
'UnicodeCharacter0xf29c': '\xef\x8a\x9c',
'UnicodeCharacter0xf29b': '\xef\x8a\x9b',
'UnicodeCharacter0xf29a': '\xef\x8a\x9a',
'KangxiRadicalFang': '\xe2\xbd\x9b',
'CjkUnifiedIdeograph37b2': '\xe3\x9e\xb2',
'CjkUnifiedIdeograph37b3': '\xe3\x9e\xb3',
'CjkUnifiedIdeograph37b0': '\xe3\x9e\xb0',
'CjkUnifiedIdeograph37b1': '\xe3\x9e\xb1',
'CjkUnifiedIdeograph37b6': '\xe3\x9e\xb6',
'CjkUnifiedIdeograph37b7': '\xe3\x9e\xb7',
'CjkUnifiedIdeograph37b4': '\xe3\x9e\xb4',
'MongolianLetterManchuAliGaliTta': '\xe1\xa2\x9e',
'HangulSyllableSsaej': '\xec\x8c\xaa',
'HangulSyllableSsaek': '\xec\x8c\xac',
'CjkUnifiedIdeograph37b8': '\xe3\x9e\xb8',
'CjkUnifiedIdeograph37b9': '\xe3\x9e\xb9',
'HangulSyllableSsaen': '\xec\x8c\x98',
'HangulSyllableSsael': '\xec\x8c\x9c',
'HangulSyllableSsaem': '\xec\x8c\xa4',
'HangulSyllableDdyob': '\xeb\x9a\x91',
'HangulSyllableSsaep': '\xec\x8c\xae',
'Gemini': '\xe2\x99\x8a',
'HangulSyllableSsaet': '\xec\x8c\xad',
'ArabicSmallHighJeem': '\xdb\x9a',
'HangulSyllableByonh': '\xeb\xb5\xaa',
'BopomofoLetterNgg': '\xe3\x86\xad',
'UnicodeCharacter0xe674': '\xee\x99\xb4',
'UnicodeCharacter0xe675': '\xee\x99\xb5',
'UnicodeCharacter0xe676': '\xee\x99\xb6',
'UnicodeCharacter0xe677': '\xee\x99\xb7',
'UnicodeCharacter0xe670': '\xee\x99\xb0',
'UnicodeCharacter0xe671': '\xee\x99\xb1',
'UnicodeCharacter0xe672': '\xee\x99\xb2',
'UnicodeCharacter0xe673': '\xee\x99\xb3',
'UnicodeCharacter0xe678': '\xee\x99\xb8',
'UnicodeCharacter0xe679': '\xee\x99\xb9',
'TibetanDigitHalfZero': '\xe0\xbc\xb3',
'UnicodeCharacter0xf42c': '\xef\x90\xac',
'ChamLetterPpa': '\xea\xa8\x9b',
'HangulSyllableBbanj': '\xeb\xb9\xa5',
'ArabicLetterPehMedialForm': '\xef\xad\x99',
'UnicodeCharacter0xf42d': '\xef\x90\xad',
'HangulSyllableBbang': '\xeb\xb9\xb5',
'CjkUnifiedIdeograph38dc': '\xe3\xa3\x9c',
'CjkUnifiedIdeograph38db': '\xe3\xa3\x9b',
'CjkUnifiedIdeograph38da': '\xe3\xa3\x9a',
'CombiningWideBridgeAbove': '\xe2\x83\xa9',
'CjkUnifiedIdeograph60e7': '\xe6\x83\xa7',
'CjkUnifiedIdeograph38df': '\xe3\xa3\x9f',
'CjkUnifiedIdeograph38de': '\xe3\xa3\x9e',
'CjkUnifiedIdeograph38dd': '\xe3\xa3\x9d',
'CjkUnifiedIdeograph550a': '\xe5\x94\x8a',
'CjkUnifiedIdeograph550b': '\xe5\x94\x8b',
'CjkUnifiedIdeograph550c': '\xe5\x94\x8c',
'CjkUnifiedIdeograph550d': '\xe5\x94\x8d',
'CjkUnifiedIdeograph550e': '\xe5\x94\x8e',
'CjkUnifiedIdeograph550f': '\xe5\x94\x8f',
'CjkUnifiedIdeograph8bb6': '\xe8\xae\xb6',
'CjkUnifiedIdeograph3fd5': '\xe3\xbf\x95',
'CjkUnifiedIdeograph3fd4': '\xe3\xbf\x94',
'CjkUnifiedIdeograph3fd7': '\xe3\xbf\x97',
'CjkUnifiedIdeograph3fd6': '\xe3\xbf\x96',
'CjkUnifiedIdeograph3fd1': '\xe3\xbf\x91',
'CjkUnifiedIdeograph3fd0': '\xe3\xbf\x90',
'CjkUnifiedIdeograph3fd3': '\xe3\xbf\x93',
'CjkUnifiedIdeograph3fd2': '\xe3\xbf\x92',
'HangulSyllableDaebs': '\xeb\x8c\x92',
'CjkUnifiedIdeograph3fd9': '\xe3\xbf\x99',
'CjkUnifiedIdeograph3fd8': '\xe3\xbf\x98',
'UnicodeCharacter0xaaee': '\xea\xab\xae',
'HangulSyllableHweg': '\xed\x9b\xbd',
'Upsilon': '\xce\xa5',
'HangulSyllableHwec': '\xed\x9c\x93',
'HangulSyllableHweb': '\xed\x9c\x8d',
'HangulSyllableHwem': '\xed\x9c\x8c',
'HangulSyllableHwel': '\xed\x9c\x84',
'HangulSyllableHweo': '\xed\x9b\xa0',
'HangulSyllablePeobs': '\xed\x8e\x8e',
'CjkUnifiedIdeograph8bb7': '\xe8\xae\xb7',
'HangulSyllableHwek': '\xed\x9c\x94',
'HangulSyllableHwej': '\xed\x9c\x92',
'UnicodeCharacter0xe67d': '\xee\x99\xbd',
'HangulSyllableHwet': '\xed\x9c\x95',
'UnicodeCharacter0xe67f': '\xee\x99\xbf',
'BoxDrawingsLightDownAndLeft': '\xe2\x94\x90',
'HangulSyllableHwes': '\xed\x9c\x8f',
'HangulSyllableDdyos': '\xeb\x9a\x93',
'SquareCOverKg': '\xe3\x8f\x86',
'CjkUnifiedIdeograph3fde': '\xe3\xbf\x9e',
'HiraganaLetterWi': '\xe3\x82\x90',
'CjkUnifiedIdeograph3fdf': '\xe3\xbf\x9f',
'CjkUnifiedIdeograph3fda': '\xe3\xbf\x9a',
'EthiopicSyllableZhwa': '\xe1\x8b\xa7',
'CjkUnifiedIdeograph3fdc': '\xe3\xbf\x9c',
'HiraganaLetterWo': '\xe3\x82\x92',
'HiraganaLetterWa': '\xe3\x82\x8f',
'HiraganaLetterWe': '\xe3\x82\x91',
'UnicodeCharacter0xe5b4': '\xee\x96\xb4',
'CjkUnifiedIdeograph38d3': '\xe3\xa3\x93',
'CjkUnifiedIdeograph5509': '\xe5\x94\x89',
'CjkUnifiedIdeograph38d1': '\xe3\xa3\x91',
'CjkUnifiedIdeograph38d0': '\xe3\xa3\x90',
'CjkUnifiedIdeograph38d7': '\xe3\xa3\x97',
'CjkUnifiedIdeograph38d6': '\xe3\xa3\x96',
'CjkUnifiedIdeograph38d5': '\xe3\xa3\x95',
'CjkUnifiedIdeograph38d4': '\xe3\xa3\x94',
'CjkUnifiedIdeograph5500': '\xe5\x94\x80',
'CjkUnifiedIdeograph4284': '\xe4\x8a\x84',
'CjkUnifiedIdeograph38d9': '\xe3\xa3\x99',
'CjkUnifiedIdeograph38d8': '\xe3\xa3\x98',
'CjkUnifiedIdeograph5504': '\xe5\x94\x84',
'CjkUnifiedIdeograph5505': '\xe5\x94\x85',
'CjkUnifiedIdeograph5506': '\xe5\x94\x86',
'CjkUnifiedIdeograph5507': '\xe5\x94\x87',
'DownwardsHarpoonWithBarbLeftBesideDownwardsHarpoonWithBarbRight': '\xe2\xa5\xa5',
'CjkUnifiedIdeograph588c': '\xe5\xa2\x8c',
'CjkUnifiedIdeograph588b': '\xe5\xa2\x8b',
'HalfwidthHangulLetterNieunhieuh': '\xef\xbe\xa6',
'CjkUnifiedIdeograph4288': '\xe4\x8a\x88',
'DoubleHighreversed9QuotationMark': '\xe2\x80\x9f',
'HebrewPointHatafSegol': '\xd6\xb1',
'UnicodeCharacter0xe68d': '\xee\x9a\x8d',
'CentreLineSymbol': '\xe2\x84\x84',
'CanadianSyllabicsCarrierH': '\xe1\x91\x8b',
'CanadianSyllabicsCarrierI': '\xe1\x90\x89',
'RingEqualTo': '\xe2\x89\x97',
'CanadianSyllabicsCarrierP': '\xe1\x97\xae',
'UnicodeCharacter0xe68c': '\xee\x9a\x8c',
'CanadianSyllabicsCarrierZ': '\xe1\x99\x86',
'UnicodeCharacter0xf799': '\xef\x9e\x99',
'ArabicLigatureYehWithHamzaAboveWithNoonFinalForm': '\xef\xb1\xa7',
'YiSyllableQot': '\xea\x90\x88',
'GujaratiLetterCha': '\xe0\xaa\x9b',
'UnicodeCharacter0xeedb': '\xee\xbb\x9b',
'TibetanSubjoinedLetterNa': '\xe0\xbe\xa3',
'CjkUnifiedIdeograph879a': '\xe8\x9e\x9a',
'epsilon': '\xcf\xb5',
'OghamLetterRuis': '\xe1\x9a\x8f',
'ArabicLetterYehWithSmallV': '\xdb\x8e',
'ArabicLigatureHehWithMeemWithJeemInitialForm': '\xef\xb6\x93',
'HangulSyllableGgwaels': '\xea\xbd\xb0',
'UnicodeCharacter0xeedc': '\xee\xbb\x9c',
'CjkUnifiedIdeograph80d3': '\xe8\x83\x93',
'ParenthesizedIdeographTwo': '\xe3\x88\xa1',
'LineIntegrationNotIncludingThePole': '\xe2\xa8\x94',
'HangulSyllableGgwaelp': '\xea\xbd\xb2',
'GurmukhiLetterOo': '\xe0\xa8\x93',
'UnicodeCharacter0xd981': '\xed\xa6\x81',
'LatinSmallLetterNWithDotAbove': '\xe1\xb9\x85',
'ArabicLetterTtehMedialForm': '\xef\xad\xa9',
'HangulSyllableJjels': '\xec\xa9\xb4',
'UnicodeCharacter0xd980': '\xed\xa6\x80',
'CjkUnifiedIdeograph721f': '\xe7\x88\x9f',
'CjkUnifiedIdeograph721d': '\xe7\x88\x9d',
'CjkUnifiedIdeograph721e': '\xe7\x88\x9e',
'CjkUnifiedIdeograph721b': '\xe7\x88\x9b',
'CjkUnifiedIdeograph721c': '\xe7\x88\x9c',
'TelephoneLocationSign': '\xe2\x9c\x86',
'HangulSyllableCels': '\xec\xb3\x80',
'HangulSyllableGgwaelt': '\xea\xbd\xb1',
'CjkUnifiedIdeograph4798': '\xe4\x9e\x98',
'CjkUnifiedIdeograph390f': '\xe3\xa4\x8f',
'UnicodeCharacter0xeded': '\xee\xb7\xad',
'CjkUnifiedIdeograph390d': '\xe3\xa4\x8d',
'CjkUnifiedIdeograph390e': '\xe3\xa4\x8e',
'CjkUnifiedIdeograph390b': '\xe3\xa4\x8b',
'CjkUnifiedIdeograph390c': '\xe3\xa4\x8c',
'CjkUnifiedIdeograph390a': '\xe3\xa4\x8a',
'UnicodeCharacter0x193c': '\xe1\xa4\xbc',
'UnicodeCharacter0xedef': '\xee\xb7\xaf',
'ArabicLigatureSheenWithMeemMedialForm': '\xef\xb3\xa9',
'HangulSyllableBbwen': '\xec\x80\x88',
'CjkUnifiedIdeograph7ac0': '\xe7\xab\x80',
'CjkUnifiedIdeograph4758': '\xe4\x9d\x98',
'CjkUnifiedIdeograph4759': '\xe4\x9d\x99',
'CjkUnifiedIdeograph4756': '\xe4\x9d\x96',
'CjkUnifiedIdeograph4757': '\xe4\x9d\x97',
'CjkUnifiedIdeograph4754': '\xe4\x9d\x94',
'CjkUnifiedIdeograph4755': '\xe4\x9d\x95',
'CjkUnifiedIdeograph4752': '\xe4\x9d\x92',
'CjkUnifiedIdeograph4753': '\xe4\x9d\x93',
'WhiteChessQueen': '\xe2\x99\x95',
'CjkUnifiedIdeograph4751': '\xe4\x9d\x91',
'CjkUnifiedIdeograph4792': '\xe4\x9e\x92',
'CjkUnifiedIdeograph7ac5': '\xe7\xab\x85',
'CjkUnifiedIdeograph4793': '\xe4\x9e\x93',
'HangulSyllableSsweoh': '\xec\x92\x9b',
'UnicodeCharacter0xedec': '\xee\xb7\xac',
'CjkUnifiedIdeograph3906': '\xe3\xa4\x86',
'CjkUnifiedIdeograph3907': '\xe3\xa4\x87',
'CjkUnifiedIdeograph3904': '\xe3\xa4\x84',
'CjkUnifiedIdeograph3905': '\xe3\xa4\x85',
'CjkUnifiedIdeograph3902': '\xe3\xa4\x82',
'CjkUnifiedIdeograph3903': '\xe3\xa4\x83',
'CjkUnifiedIdeograph3900': '\xe3\xa4\x80',
'CjkUnifiedIdeograph3901': '\xe3\xa4\x81',
'CjkUnifiedIdeograph3908': '\xe3\xa4\x88',
'CjkUnifiedIdeograph3909': '\xe3\xa4\x89',
'CjkUnifiedIdeograph60e5': '\xe6\x83\xa5',
'CjkUnifiedIdeograph7217': '\xe7\x88\x97',
'CjkUnifiedIdeograph7214': '\xe7\x88\x94',
'CjkUnifiedIdeograph7215': '\xe7\x88\x95',
'CjkUnifiedIdeograph7212': '\xe7\x88\x92',
'CjkUnifiedIdeograph7213': '\xe7\x88\x93',
'CjkUnifiedIdeograph7210': '\xe7\x88\x90',
'CjkUnifiedIdeograph7211': '\xe7\x88\x91',
'HangulSyllableJjelp': '\xec\xa9\xb6',
'UnicodeCharacter0xf7bf': '\xef\x9e\xbf',
'SylotiNagriLetterE': '\xea\xa0\x84',
'CjkUnifiedIdeograph7218': '\xe7\x88\x98',
'CjkUnifiedIdeograph7219': '\xe7\x88\x99',
'HangulSyllableBbwilt': '\xec\x80\xad',
'HangulSyllableBbwils': '\xec\x80\xac',
'CyrillicCapitalLetterTe': '\xd0\xa2',
'KangxiRadicalMillet': '\xe2\xbf\x89',
'UnicodeCharacter0xeb79': '\xee\xad\xb9',
'Arc': '\xe2\x8c\x92',
'CombiningEnclosingCircle': '\xe2\x83\x9d',
'CjkUnifiedIdeograph475f': '\xe4\x9d\x9f',
'UnicodeCharacter0xeb71': '\xee\xad\xb1',
'CjkUnifiedIdeograph475d': '\xe4\x9d\x9d',
'CjkUnifiedIdeograph475e': '\xe4\x9d\x9e',
'CjkUnifiedIdeograph475b': '\xe4\x9d\x9b',
'CjkUnifiedIdeograph475c': '\xe4\x9d\x9c',
'UnicodeCharacter0xeb76': '\xee\xad\xb6',
'CjkUnifiedIdeograph475a': '\xe4\x9d\x9a',
'UnicodeCharacter0xdd71': '\xed\xb5\xb1',
'UnicodeCharacter0xdd70': '\xed\xb5\xb0',
'UnicodeCharacter0xdd73': '\xed\xb5\xb3',
'UnicodeCharacter0xdd72': '\xed\xb5\xb2',
'UnicodeCharacter0xdd75': '\xed\xb5\xb5',
'UnicodeCharacter0xdd74': '\xed\xb5\xb4',
'UnicodeCharacter0xdd77': '\xed\xb5\xb7',
'CjkUnifiedIdeograph5884': '\xe5\xa2\x84',
'UnicodeCharacter0xdd79': '\xed\xb5\xb9',
'UnicodeCharacter0xdd78': '\xed\xb5\xb8',
'UnicodeCharacter0xea88': '\xee\xaa\x88',
'UnicodeCharacter0xea89': '\xee\xaa\x89',
'CjkUnifiedIdeograph5887': '\xe5\xa2\x87',
'CjkUnifiedIdeograph5886': '\xe5\xa2\x86',
'CjkUnifiedIdeograph5881': '\xe5\xa2\x81',
'CjkUnifiedIdeograph5880': '\xe5\xa2\x80',
'UnicodeCharacter0xa79e': '\xea\x9e\x9e',
'CjkUnifiedIdeograph428f': '\xe4\x8a\x8f',
'HangulSyllableJjap': '\xec\xa7\xb6',
'HangulSyllableTwah': '\xed\x87\x97',
'UnicodeCharacter0xf79a': '\xef\x9e\x9a',
'HangulSyllableJjag': '\xec\xa7\x9d',
'HangulSyllableJjad': '\xec\xa7\xa3',
'CanadianSyllabicsShwii': '\xe1\x94\x9b',
'HangulSyllableJjab': '\xec\xa7\xad',
'HangulSyllableJjac': '\xec\xa7\xb3',
'HangulSyllableBbweo': '\xeb\xbf\xa8',
'HangulSyllableJjan': '\xec\xa7\xa0',
'UnicodeCharacter0xf6ef': '\xef\x9b\xaf',
'LeftSquareBracketWithUnderbar': '\xe2\xa6\x8b',
'HangulSyllableJjam': '\xec\xa7\xac',
'HangulSyllableJjaj': '\xec\xa7\xb2',
'HangulSyllableJjak': '\xec\xa7\xb4',
'HangulSyllableJjah': '\xec\xa7\xb7',
'UnicodeCharacter0xa79d': '\xea\x9e\x9d',
'InvertedUndertie': '\xe2\x81\x94',
'LatinCapitalLetterCWithDotAbove': '\xc4\x8a',
'UnicodeCharacter0xe681': '\xee\x9a\x81',
'MinusSign': '\xe2\x88\x92',
'HeavyMultiplicationX': '\xe2\x9c\x96',
'UnicodeCharacter0xe680': '\xee\x9a\x80',
'CyrillicCapitalLetterAleutKa': '\xd4\x9e',
'UnicodeCharacter0xea8c': '\xee\xaa\x8c',
'UnicodeCharacter0xdd7c': '\xed\xb5\xbc',
'CjkUnifiedIdeograph8bb8': '\xe8\xae\xb8',
'UnicodeCharacter0xdd7e': '\xed\xb5\xbe',
'UnicodeCharacter0xdd7d': '\xed\xb5\xbd',
'UnicodeCharacter0xea8d': '\xee\xaa\x8d',
'UnicodeCharacter0xdd7f': '\xed\xb5\xbf',
'UnicodeCharacter0xa79c': '\xea\x9e\x9c',
'UnicodeCharacter0xf420': '\xef\x90\xa0',
'HangulSyllableJjyiss': '\xec\xb0\x84',
'LatinSmallLetterSidewaysTurnedM': '\xe1\xb4\x9f',
'UnicodeCharacter0xf421': '\xef\x90\xa1',
'UnicodeCharacter0xf6e7': '\xef\x9b\xa7',
'UnicodeCharacter0xf6e6': '\xef\x9b\xa6',
'UnicodeCharacter0xf6e5': '\xef\x9b\xa5',
'UnicodeCharacter0xf6e4': '\xef\x9b\xa4',
'UnicodeCharacter0xf6e3': '\xef\x9b\xa3',
'UnicodeCharacter0xf6e2': '\xef\x9b\xa2',
'UnicodeCharacter0xf6e1': '\xef\x9b\xa1',
'UnicodeCharacter0xf6e0': '\xef\x9b\xa0',
'HangulSyllablePyobs': '\xed\x91\xae',
'UnicodeCharacter0xe2b2': '\xee\x8a\xb2',
'UnicodeCharacter0xf424': '\xef\x90\xa4',
'UnicodeCharacter0xa79b': '\xea\x9e\x9b',
'HangulSyllableRok': '\xeb\xa1\xb4',
'UnicodeCharacter0xf425': '\xef\x90\xa5',
'CjkUnifiedIdeograph3e9e': '\xe3\xba\x9e',
'CjkUnifiedIdeograph3e9d': '\xe3\xba\x9d',
'CjkUnifiedIdeograph3e9f': '\xe3\xba\x9f',
'LozengeDividedByHorizontalRule': '\xe2\x9f\xa0',
'CjkUnifiedIdeograph3e9c': '\xe3\xba\x9c',
'CjkUnifiedIdeograph3e9b': '\xe3\xba\x9b',
'UnicodeCharacter0xd7b1': '\xed\x9e\xb1',
'UnicodeCharacter0xd7b0': '\xed\x9e\xb0',
'HangulSyllableDdibs': '\xeb\x9d\xb2',
'UnicodeCharacter0xd7b2': '\xed\x9e\xb2',
'UnicodeCharacter0xd7b5': '\xed\x9e\xb5',
'CjkUnifiedIdeograph8e01': '\xe8\xb8\x81',
'UnicodeCharacter0xd7b7': '\xed\x9e\xb7',
'UnicodeCharacter0xd7b6': '\xed\x9e\xb6',
'CjkUnifiedIdeograph699f': '\xe6\xa6\x9f',
'UnicodeCharacter0xdb94': '\xed\xae\x94',
'CjkUnifiedIdeograph699d': '\xe6\xa6\x9d',
'CjkUnifiedIdeograph699e': '\xe6\xa6\x9e',
'CjkUnifiedIdeograph699b': '\xe6\xa6\x9b',
'CjkUnifiedIdeograph699c': '\xe6\xa6\x9c',
'UnicodeCharacter0xdb93': '\xed\xae\x93',
'CjkUnifiedIdeograph699a': '\xe6\xa6\x9a',
'CjkUnifiedIdeograph8e07': '\xe8\xb8\x87',
'UnicodeCharacter0xa79a': '\xea\x9e\x9a',
'HebrewLetterTavWithDagesh': '\xef\xad\x8a',
'UnicodeCharacter0xdb99': '\xed\xae\x99',
'UnicodeCharacter0xdb98': '\xed\xae\x98',
'CjkUnifiedIdeograph8e06': '\xe8\xb8\x86',
'CjkUnifiedIdeograph60b8': '\xe6\x82\xb8',
'CjkUnifiedIdeograph60b9': '\xe6\x82\xb9',
'CjkUnifiedIdeograph60b2': '\xe6\x82\xb2',
'CjkUnifiedIdeograph60b3': '\xe6\x82\xb3',
'CircledHangulPhieuphA': '\xe3\x89\xba',
'CjkUnifiedIdeograph60b1': '\xe6\x82\xb1',
'CjkUnifiedIdeograph5849': '\xe5\xa1\x89',
'CjkUnifiedIdeograph60b7': '\xe6\x82\xb7',
'CjkUnifiedIdeograph60b4': '\xe6\x82\xb4',
'CjkUnifiedIdeograph60b5': '\xe6\x82\xb5',
'CjkUnifiedIdeograph6628': '\xe6\x98\xa8',
'CjkUnifiedIdeograph6629': '\xe6\x98\xa9',
'CjkUnifiedIdeograph6624': '\xe6\x98\xa4',
'CjkUnifiedIdeograph6625': '\xe6\x98\xa5',
'CjkUnifiedIdeograph6626': '\xe6\x98\xa6',
'CjkUnifiedIdeograph6627': '\xe6\x98\xa7',
'CjkUnifiedIdeograph6620': '\xe6\x98\xa0',
'CjkUnifiedIdeograph6621': '\xe6\x98\xa1',
'CjkUnifiedIdeograph6622': '\xe6\x98\xa2',
'CjkUnifiedIdeograph6623': '\xe6\x98\xa3',
'MyanmarLetterGreatSa': '\xe1\x80\xbf',
'MyanmarVowelSignU': '\xe1\x80\xaf',
'CjkUnifiedIdeograph6994': '\xe6\xa6\x94',
'SquareMOverS': '\xe3\x8e\xa7',
'CjkUnifiedIdeograph6992': '\xe6\xa6\x92',
'CjkUnifiedIdeograph6993': '\xe6\xa6\x93',
'CjkUnifiedIdeograph6990': '\xe6\xa6\x90',
'CjkUnifiedIdeograph6991': '\xe6\xa6\x91',
'CjkUnifiedIdeograph5848': '\xe5\xa1\x88',
'CjkUnifiedIdeograph6998': '\xe6\xa6\x98',
'CjkUnifiedIdeograph6999': '\xe6\xa6\x99',
'CjkUnifiedIdeograph3e95': '\xe3\xba\x95',
'CjkUnifiedIdeograph3e94': '\xe3\xba\x94',
'CjkUnifiedIdeograph3e97': '\xe3\xba\x97',
'CjkUnifiedIdeograph3e96': '\xe3\xba\x96',
'CjkUnifiedIdeograph3e91': '\xe3\xba\x91',
'CjkUnifiedIdeograph3e90': '\xe3\xba\x90',
'CjkUnifiedIdeograph3e93': '\xe3\xba\x93',
'CjkUnifiedIdeograph3e92': '\xe3\xba\x92',
'UnicodeCharacter0xd7ba': '\xed\x9e\xba',
'UnicodeCharacter0xd7bc': '\xed\x9e\xbc',
'UnicodeCharacter0xd7bb': '\xed\x9e\xbb',
'CjkUnifiedIdeograph3e99': '\xe3\xba\x99',
'CjkUnifiedIdeograph3e98': '\xe3\xba\x98',
'UnicodeCharacter0xd7bf': '\xed\x9e\xbf',
'YiSyllableFit': '\xea\x83\x8d',
'SubsetOfAboveAlmostEqualTo': '\xe2\xab\x89',
'HangulSyllableBbyaelm': '\xeb\xba\xbe',
'YiSyllableFip': '\xea\x83\x90',
'HangulSyllableJjyeogs': '\xec\xaa\x87',
'CjkUnifiedIdeograph662d': '\xe6\x98\xad',
'UnicodeCharacter0x9fd': '\xe0\xa7\xbd',
'CjkUnifiedIdeograph662f': '\xe6\x98\xaf',
'LatinCapitalLetterOWithOgonek': '\xc7\xaa',
'CjkUnifiedIdeograph662b': '\xe6\x98\xab',
'CjkUnifiedIdeograph662c': '\xe6\x98\xac',
'HangulSyllableJjyeogg': '\xec\xaa\x86',
'CjkUnifiedIdeograph6e58': '\xe6\xb9\x98',
'HangulSyllableBbyaelb': '\xeb\xba\xbf',
'CjkUnifiedIdeograph60bb': '\xe6\x82\xbb',
'CjkUnifiedIdeograph60bc': '\xe6\x82\xbc',
'CjkUnifiedIdeograph60ba': '\xe6\x82\xba',
'CjkUnifiedIdeograph60bf': '\xe6\x82\xbf',
'MongolianLetterTodoGa': '\xe1\xa1\x8e',
'CjkUnifiedIdeograph60bd': '\xe6\x82\xbd',
'CjkUnifiedIdeograph60be': '\xe6\x82\xbe',
'CanadianSyllabicsYcreeMoo': '\xe1\x92\xa9',
'ReversedSansserifCapitalL': '\xe2\x85\x83',
'HangulSyllableDdyaebs': '\xeb\x96\x96',
'HangulSyllableByang': '\xeb\xb1\xa1',
'LatinCapitalLetterOo': '\xea\x9d\x8e',
'LeftwardsSquiggleArrow': '\xe2\x87\x9c',
'UnicodeCharacter0xda7e': '\xed\xa9\xbe',
'CjkUnifiedIdeograph9468': '\xe9\x91\xa8',
'HangulSyllableDdyubs': '\xeb\x9c\x9e',
'HangulSyllableNoess': '\xeb\x87\xa0',
'CjkUnifiedIdeograph9466': '\xe9\x91\xa6',
'ArabicLetterSadIsolatedForm': '\xef\xba\xb9',
'CyrillicSmallLetterElWithTail': '\xd3\x86',
'HangulSyllableCyass': '\xec\xb1\xb4',
'GlagoliticCapitalLetterYo': '\xe2\xb0\xa6',
'CjkUnifiedIdeograph9462': '\xe9\x91\xa2',
'UnicodeCharacter0xf593': '\xef\x96\x93',
'HangulSyllableBwit': '\xeb\xb7\xad',
'GlagoliticCapitalLetterYu': '\xe2\xb0\xa3',
'HangulSyllableBwis': '\xeb\xb7\xa7',
'KannadaLetterDa': '\xe0\xb2\xa6',
'CjkUnifiedIdeograph9460': '\xe9\x91\xa0',
'CjkUnifiedIdeograph95ab': '\xe9\x96\xab',
'LeftArcLessthanBracket': '\xe2\xa6\x93',
'SquareKeesu': '\xe3\x8c\x9c',
'HangulSyllableMunh': '\xeb\xac\xba',
'HangulSyllableNess': '\xeb\x84\xb8',
'CjkUnifiedIdeograph7a23': '\xe7\xa8\xa3',
'LatinSmallLetterOWithDiaeresisAndMacron': '\xc8\xab',
'LatinCapitalLetterToneTwo': '\xc6\xa7',
'DigitSevenFullStop': '\xe2\x92\x8e',
'HangulSyllableRoes': '\xeb\xa3\x83',
'SundaneseDigitSeven': '\xe1\xae\xb7',
'HangulSyllableRoep': '\xeb\xa3\x8a',
'RunicLetterCen': '\xe1\x9a\xb3',
'HangulSyllableRoet': '\xeb\xa3\x89',
'HangulSyllableRoek': '\xeb\xa3\x88',
'HangulSyllableRoej': '\xeb\xa3\x86',
'UnicodeCharacter0x588': '\xd6\x88',
'HangulSyllableRoeh': '\xeb\xa3\x8b',
'LepchaSignNukta': '\xe1\xb0\xb7',
'HangulSyllableRoen': '\xeb\xa2\xb4',
'HangulSyllableRoem': '\xeb\xa3\x80',
'KayahLiDigitTwo': '\xea\xa4\x82',
'CjkUnifiedIdeograph98fc': '\xe9\xa3\xbc',
'CjkUnifiedIdeograph98fb': '\xe9\xa3\xbb',
'ArabicLetterDyeh': '\xda\x84',
'ArabicLetterBehWithDotBelowAndThreeDotsAbove': '\xdd\x91',
'HangulSyllableRoeg': '\xeb\xa2\xb1',
'CjkUnifiedIdeograph98ff': '\xe9\xa3\xbf',
'CjkUnifiedIdeograph98fe': '\xe9\xa3\xbe',
'CjkUnifiedIdeograph8704': '\xe8\x9c\x84',
'CjkUnifiedIdeograph6f19': '\xe6\xbc\x99',
'CjkUnifiedIdeograph6354': '\xe6\x8d\x94',
'CjkUnifiedIdeograph7d5b': '\xe7\xb5\x9b',
'CjkUnifiedIdeograph7d5c': '\xe7\xb5\x9c',
'CjkUnifiedIdeograph7d5d': '\xe7\xb5\x9d',
'CjkUnifiedIdeograph7d5e': '\xe7\xb5\x9e',
'CjkUnifiedIdeograph7d5f': '\xe7\xb5\x9f',
'HangulSyllableRwels': '\xeb\xa4\xac',
'CjkUnifiedIdeograph6f11': '\xe6\xbc\x91',
'CjkUnifiedIdeograph6f10': '\xe6\xbc\x90',
'CjkUnifiedIdeograph6f13': '\xe6\xbc\x93',
'CjkUnifiedIdeograph6f12': '\xe6\xbc\x92',
'CjkUnifiedIdeograph6f15': '\xe6\xbc\x95',
'CjkUnifiedIdeograph6f14': '\xe6\xbc\x94',
'CjkUnifiedIdeograph6f17': '\xe6\xbc\x97',
'CjkUnifiedIdeograph6f16': '\xe6\xbc\x96',
'FullwidthAsterisk': '\xef\xbc\x8a',
'UnicodeCharacter0x2fe9': '\xe2\xbf\xa9',
'UnicodeCharacter0x2fe8': '\xe2\xbf\xa8',
'HangulSyllableGgweolb': '\xea\xbf\x9f',
'HangulSyllableRwelb': '\xeb\xa4\xab',
'UnicodeCharacter0x2fe3': '\xe2\xbf\xa3',
'UnicodeCharacter0x2fe2': '\xe2\xbf\xa2',
'UnicodeCharacter0x2fe1': '\xe2\xbf\xa1',
'UnicodeCharacter0x2fe0': '\xe2\xbf\xa0',
'UnicodeCharacter0x2fe7': '\xe2\xbf\xa7',
'UnicodeCharacter0x2fe6': '\xe2\xbf\xa6',
'UnicodeCharacter0x2fe5': '\xe2\xbf\xa5',
'UnicodeCharacter0x2fe4': '\xe2\xbf\xa4',
'HangulSyllableSsaelb': '\xec\x8c\x9f',
'CjkUnifiedIdeograph98f9': '\xe9\xa3\xb9',
'CjkUnifiedIdeograph98f8': '\xe9\xa3\xb8',
'HangulSyllableSsaelg': '\xec\x8c\x9d',
'UnicodeCharacter0x58b': '\xd6\x8b',
'UnicodeCharacter0x58c': '\xd6\x8c',
'CjkUnifiedIdeograph98f1': '\xe9\xa3\xb1',
'CjkUnifiedIdeograph98f0': '\xe9\xa3\xb0',
'UnicodeCharacter0x58f': '\xd6\x8f',
'CjkUnifiedIdeograph98f6': '\xe9\xa3\xb6',
'UnicodeCharacter0x58d': '\xd6\x8d',
'UnicodeCharacter0x58e': '\xd6\x8e',
'HangulSyllableSsaels': '\xec\x8c\xa0',
'TibetanSubjoinedLetterTsa': '\xe0\xbe\xa9',
'HangulSyllableSsaelp': '\xec\x8c\xa2',
'HangulSyllableYeolm': '\xec\x97\xb6',
'CanadianSyllabicsFinalDownTack': '\xe1\x90\xaa',
'HangulSyllableSsaelt': '\xec\x8c\xa1',
'ArabicLigatureSheenWithRehIsolatedForm': '\xef\xb4\x8d',
'HangulSyllableYeols': '\xec\x97\xb8',
'BigReverseSolidus': '\xe2\xa7\xb9',
'HangulSyllablePying': '\xed\x94\xb5',
'UnicodeCharacter0x2fec': '\xe2\xbf\xac',
'UnicodeCharacter0x2feb': '\xe2\xbf\xab',
'UnicodeCharacter0x2fea': '\xe2\xbf\xaa',
'ArabicLigatureHehWithMeemInitialForm': '\xef\xb3\x98',
'UnicodeCharacter0x2fef': '\xe2\xbf\xaf',
'UnicodeCharacter0x2fee': '\xe2\xbf\xae',
'UnicodeCharacter0x2fed': '\xe2\xbf\xad',
'CjkUnifiedIdeograph7d50': '\xe7\xb5\x90',
'CjkUnifiedIdeograph7d51': '\xe7\xb5\x91',
'CjkUnifiedIdeograph7d52': '\xe7\xb5\x92',
'CjkUnifiedIdeograph7d53': '\xe7\xb5\x93',
'CjkUnifiedIdeograph7d54': '\xe7\xb5\x94',
'MyanmarShanDigitTwo': '\xe1\x82\x92',
'CjkUnifiedIdeograph7d56': '\xe7\xb5\x96',
'CjkUnifiedIdeograph7d57': '\xe7\xb5\x97',
'CjkUnifiedIdeograph6f1a': '\xe6\xbc\x9a',
'CjkUnifiedIdeograph7d59': '\xe7\xb5\x99',
'CjkUnifiedIdeograph6f1c': '\xe6\xbc\x9c',
'CjkUnifiedIdeograph6f1b': '\xe6\xbc\x9b',
'CjkUnifiedIdeograph6f1e': '\xe6\xbc\x9e',
'CjkUnifiedIdeograph6f1d': '\xe6\xbc\x9d',
'CjkUnifiedIdeograph6f1f': '\xe6\xbc\x9f',
'CanadianSyllabicsLaai': '\xe1\x93\x94',
'CjkUnifiedIdeograph4f08': '\xe4\xbc\x88',
'CjkUnifiedIdeograph4f09': '\xe4\xbc\x89',
'UnicodeCharacter0xead7': '\xee\xab\x97',
'CjkUnifiedIdeograph4f04': '\xe4\xbc\x84',
'CjkUnifiedIdeograph4f05': '\xe4\xbc\x85',
'CjkUnifiedIdeograph4f06': '\xe4\xbc\x86',
'CjkUnifiedIdeograph4f07': '\xe4\xbc\x87',
'CjkUnifiedIdeograph4f00': '\xe4\xbc\x80',
'CjkUnifiedIdeograph4f01': '\xe4\xbc\x81',
'CjkUnifiedIdeograph4f02': '\xe4\xbc\x82',
'CjkUnifiedIdeograph4f03': '\xe4\xbc\x83',
'UnicodeCharacter0xe42b': '\xee\x90\xab',
'EthiopicSyllableXya': '\xe2\xb7\x90',
'Multimap': '\xe2\x8a\xb8',
'EthiopicSyllableXye': '\xe2\xb7\x95',
'HangulSyllableTyonj': '\xed\x88\x95',
'BugineseLetterNa': '\xe1\xa8\x8a',
'EthiopicSyllableXyi': '\xe2\xb7\x92',
'YiRadicalGa': '\xea\x92\xa1',
'UnicodeCharacter0xd9a6': '\xed\xa6\xa6',
'EthiopicSyllableXyo': '\xe2\xb7\x96',
'VaiSyllableCoo': '\xea\x96\x8b',
'EthiopicSyllableXyu': '\xe2\xb7\x91',
'ArabicLetterGuehFinalForm': '\xef\xae\x97',
'LatinSmallLetterRWithDotBelowAndMacron': '\xe1\xb9\x9d',
'UnicodeCharacter0xead5': '\xee\xab\x95',
'Increment': '\xe2\x88\x86',
'CjkCompatibilityIdeographf90d': '\xef\xa4\x8d',
'CjkUnifiedIdeograph5b3e': '\xe5\xac\xbe',
'QuadrantUpperRightAndLowerLeft': '\xe2\x96\x9e',
'HangulSyllableNeung': '\xeb\x8a\xa5',
'LatinCapitalLetterAWithBreve': '\xc4\x82',
'CjkUnifiedIdeograph4f0d': '\xe4\xbc\x8d',
'LatinSmallLetterUWithCircumflexBelow': '\xe1\xb9\xb7',
'CjkUnifiedIdeograph4f0f': '\xe4\xbc\x8f',
'EmDash': '\xe2\x80\x94',
'CjkUnifiedIdeograph4f0a': '\xe4\xbc\x8a',
'CjkUnifiedIdeograph4f0b': '\xe4\xbc\x8b',
'CjkUnifiedIdeograph4f0c': '\xe4\xbc\x8c',
'UnicodeCharacter0xead2': '\xee\xab\x92',
'CjkCompatibilityIdeographf90c': '\xef\xa4\x8c',
'CjkUnifiedIdeograph5b3b': '\xe5\xac\xbb',
'DegreeFahrenheit': '\xe2\x84\x89',
'CjkCompatibilityIdeographf90b': '\xef\xa4\x8b',
'CjkUnifiedIdeograph8408': '\xe8\x90\x88',
'UnicodeCharacter0xe2bf': '\xee\x8a\xbf',
'CjkUnifiedIdeograph8409': '\xe8\x90\x89',
'CjkUnifiedIdeograph7e53': '\xe7\xb9\x93',
'UnicodeCharacter0xe2bd': '\xee\x8a\xbd',
'LatinSmallLetterTurnedOe': '\xe1\xb4\x94',
'UnicodeCharacter0xead3': '\xee\xab\x93',
'HangulSyllableHeuc': '\xed\x9d\xa7',
'ThaiDigitNine': '\xe0\xb9\x99',
'HangulSyllableHeug': '\xed\x9d\x91',
'GreekSmallLetterAlphaWithPsiliAndPerispomeni': '\xe1\xbc\x86',
'CjkUnifiedIdeograph5a4a': '\xe5\xa9\x8a',
'HangulSyllableHeuk': '\xed\x9d\xa8',
'YiSyllableJjuop': '\xea\x90\xa2',
'MalayalamVowelSignVocalicR': '\xe0\xb5\x83',
'HangulSyllableHeuh': '\xed\x9d\xab',
'HangulSyllableHeun': '\xed\x9d\x94',
'HangulSyllableHeum': '\xed\x9d\xa0',
'HangulSyllableHeul': '\xed\x9d\x98',
'HangulSyllableHeus': '\xed\x9d\xa3',
'HangulSyllableByess': '\xeb\xb3\xac',
'CjkUnifiedIdeograph5a4c': '\xe5\xa9\x8c',
'MalayalamVowelSignVocalicL': '\xe0\xb5\xa2',
'HangulSyllableHeut': '\xed\x9d\xa9',
'KhmerSymbolPiiKoet': '\xe1\xa7\xa2',
'UnicodeCharacter0xead0': '\xee\xab\x90',
'BlackDownpointingSmallTriangle': '\xe2\x96\xbe',
'HanunooLetterNa': '\xe1\x9c\xa8',
'<->': '\xe2\x86\x94',
'UnicodeCharacter0xd9a2': '\xed\xa6\xa2',
'UnicodeCharacter0xd49': '\xe0\xb5\x89',
'UnicodeCharacter0xd45': '\xe0\xb5\x85',
'IdeographicAnnotationBottomMark': '\xe3\x86\x98',
'CjkUnifiedIdeograph8407': '\xe8\x90\x87',
'SundaneseDigitOne': '\xe1\xae\xb1',
'CjkUnifiedIdeograph5b3a': '\xe5\xac\xba',
'UnicodeCharacter0xed8e': '\xee\xb6\x8e',
'UnicodeCharacter0xf7ba': '\xef\x9e\xba',
'TibetanDigitHalfTwo': '\xe0\xbc\xab',
'UnicodeCharacter0xa9bd': '\xea\xa6\xbd',
'ArabicLigatureSeenWithMeemIsolatedForm': '\xef\xb0\x9f',
'GreekLetterSmallCapitalPsi': '\xe1\xb4\xaa',
'TagalogSignVirama': '\xe1\x9c\x94',
'UnicodeCharacter0xed8d': '\xee\xb6\x8d',
'UnicodeCharacter0xaa72': '\xea\xa9\xb2',
'HangulSyllableJjuh': '\xec\xad\xa3',
'LessthanButNotEquivalentTo': '\xe2\x8b\xa6',
'KangxiRadicalWater': '\xe2\xbd\x94',
'UnicodeCharacter0xaa70': '\xea\xa9\xb0',
'UnicodeCharacter0xd4f': '\xe0\xb5\x8f',
'UnicodeCharacter0xd4e': '\xe0\xb5\x8e',
'UnicodeCharacter0xd9a1': '\xed\xa6\xa1',
'UnicodeCharacter0xed8c': '\xee\xb6\x8c',
'HangulSyllableJjul': '\xec\xad\x90',
'HangulSyllableJjum': '\xec\xad\x98',
'UnicodeCharacter0xe9e9': '\xee\xa7\xa9',
'HangulSyllableDaegs': '\xeb\x8c\x83',
'CjkUnifiedIdeograph804b': '\xe8\x81\x8b',
'CjkUnifiedIdeograph804c': '\xe8\x81\x8c',
'HangulChoseongPieupsiostikeut': '\xe1\x84\xa3',
'CjkUnifiedIdeograph804f': '\xe8\x81\x8f',
'UnicodeCharacter0xd7a9': '\xed\x9e\xa9',
'CjkUnifiedIdeograph4ad9': '\xe4\xab\x99',
'CjkUnifiedIdeograph4ad8': '\xe4\xab\x98',
'CjkUnifiedIdeograph4ad7': '\xe4\xab\x97',
'CanadianSyllabicsNaskapiCwaa': '\xe1\x92\xa0',
'CjkUnifiedIdeograph4ad5': '\xe4\xab\x95',
'CjkUnifiedIdeograph4ad4': '\xe4\xab\x94',
'CjkUnifiedIdeograph4ad3': '\xe4\xab\x93',
'CjkUnifiedIdeograph4ad2': '\xe4\xab\x92',
'CjkUnifiedIdeograph4ad1': '\xe4\xab\x91',
'CjkUnifiedIdeograph4ad0': '\xe4\xab\x90',
'BalineseLetterAkaraTedung': '\xe1\xac\x86',
'HangulSyllableJjug': '\xec\xad\x89',
'CjkUnifiedIdeograph8598': '\xe8\x96\x98',
'CjkUnifiedIdeograph8599': '\xe8\x96\x99',
'HangulSyllableYiss': '\xec\x9d\xac',
'CjkUnifiedIdeograph8590': '\xe8\x96\x90',
'CjkUnifiedIdeograph8591': '\xe8\x96\x91',
'CjkUnifiedIdeograph8592': '\xe8\x96\x92',
'CjkUnifiedIdeograph8593': '\xe8\x96\x93',
'CjkUnifiedIdeograph8594': '\xe8\x96\x94',
'CjkUnifiedIdeograph8595': '\xe8\x96\x95',
'CjkUnifiedIdeograph8596': '\xe8\x96\x96',
'OghamLetterEadhadh': '\xe1\x9a\x93',
'CjkUnifiedIdeograph8042': '\xe8\x81\x82',
'CjkUnifiedIdeograph8043': '\xe8\x81\x83',
'MalayalamDigitFour': '\xe0\xb5\xaa',
'BottomRightCorner': '\xe2\x8c\x9f',
'CjkUnifiedIdeograph8046': '\xe8\x81\x86',
'CjkUnifiedIdeograph8047': '\xe8\x81\x87',
'CjkUnifiedIdeograph8044': '\xe8\x81\x84',
'CjkUnifiedIdeograph8045': '\xe8\x81\x85',
'CjkUnifiedIdeograph4adf': '\xe4\xab\x9f',
'CjkUnifiedIdeograph4ade': '\xe4\xab\x9e',
'CjkUnifiedIdeograph4add': '\xe4\xab\x9d',
'CjkUnifiedIdeograph4adc': '\xe4\xab\x9c',
'CjkUnifiedIdeograph4adb': '\xe4\xab\x9b',
'CjkUnifiedIdeograph4ada': '\xe4\xab\x9a',
'SaurashtraSignVisarga': '\xea\xa2\x81',
'CyrillicCapitalLetterTeWithMiddleHook': '\xea\x9a\x8a',
'CjkUnifiedIdeograph859a': '\xe8\x96\x9a',
'CjkUnifiedIdeograph859b': '\xe8\x96\x9b',
'CjkUnifiedIdeograph859c': '\xe8\x96\x9c',
'CjkUnifiedIdeograph859d': '\xe8\x96\x9d',
'CjkUnifiedIdeograph859e': '\xe8\x96\x9e',
'CjkUnifiedIdeograph859f': '\xe8\x96\x9f',
'HangulSyllableOlp': '\xec\x98\xb2',
'HangulSyllableOlm': '\xec\x98\xae',
'ChamConsonantSignWa': '\xea\xa8\xb6',
'HangulSyllableOlh': '\xec\x98\xb3',
'HangulSyllableOlg': '\xec\x98\xad',
'ArabicLigatureAinWithJeemWithMeemFinalForm': '\xef\xb5\xb5',
'HangulSyllableOlb': '\xec\x98\xaf',
'downarrow': '\xe2\x86\x93',
'YiSyllableOt': '\xea\x80\x8f',
'HangulSyllableDaegg': '\xeb\x8c\x82',
'YiSyllableOp': '\xea\x80\x92',
'HangulSyllableNwilt': '\xeb\x89\xa5',
'UnicodeCharacter0xe99c': '\xee\xa6\x9c',
'UnicodeCharacter0xe99b': '\xee\xa6\x9b',
'HangulSyllableNwilp': '\xeb\x89\xa6',
'UnicodeCharacter0xe99d': '\xee\xa6\x9d',
'YiSyllableOx': '\xea\x80\x90',
'HangulSyllableNwilm': '\xeb\x89\xa2',
'HangulSyllableNwilh': '\xeb\x89\xa7',
'UnicodeCharacter0xe7a9': '\xee\x9e\xa9',
'CanadianSyllabicsCarrierZee': '\xe1\x99\x83',
'HangulSyllableNwilg': '\xeb\x89\xa1',
'HangulSyllableNwilb': '\xeb\x89\xa3',
'CjkUnifiedIdeograph5ab6': '\xe5\xaa\xb6',
'CjkUnifiedIdeograph5ab7': '\xe5\xaa\xb7',
'CjkUnifiedIdeograph5ab4': '\xe5\xaa\xb4',
'CjkUnifiedIdeograph5ab5': '\xe5\xaa\xb5',
'CjkUnifiedIdeograph5ab2': '\xe5\xaa\xb2',
'CjkUnifiedIdeograph5ab3': '\xe5\xaa\xb3',
'CjkUnifiedIdeograph5ab0': '\xe5\xaa\xb0',
'LessthanAndNotApproximate': '\xe2\xaa\x89',
'YiSyllableLie': '\xea\x86\xbd',
'CjkUnifiedIdeograph5ab8': '\xe5\xaa\xb8',
'CjkUnifiedIdeograph5ab9': '\xe5\xaa\xb9',
'KhmerSignRobat': '\xe1\x9f\x8c',
'CyrillicSmallLetterElWithMiddleHook': '\xd4\xa1',
'YiSyllableLip': '\xea\x86\xba',
'YiSyllableLit': '\xea\x86\xb7',
'RunicLetterShorttwigsolS': '\xe1\x9b\x8c',
'OriyaLetterDdha': '\xe0\xac\xa2',
'ModifierLetterLowerRightCornerAngle': '\xea\x9c\x9a',
'CircledIdeographSupervise': '\xe3\x8a\xac',
'UnicodeCharacter0xe998': '\xee\xa6\x98',
'HebrewLetterShinWithDageshAndShinDot': '\xef\xac\xac',
'UnicodeCharacter0xe991': '\xee\xa6\x91',
'UnicodeCharacter0xe990': '\xee\xa6\x90',
'UnicodeCharacter0xe993': '\xee\xa6\x93',
'UnicodeCharacter0xe992': '\xee\xa6\x92',
'UnicodeCharacter0xe995': '\xee\xa6\x95',
'UnicodeCharacter0xe994': '\xee\xa6\x94',
'UnicodeCharacter0xe997': '\xee\xa6\x97',
'UnicodeCharacter0xe996': '\xee\xa6\x96',
'CherokeeLetterWe': '\xe1\x8f\xaa',
'CherokeeLetterWa': '\xe1\x8f\xa9',
'CherokeeLetterWo': '\xe1\x8f\xac',
'CherokeeLetterWi': '\xe1\x8f\xab',
'CherokeeLetterWu': '\xe1\x8f\xad',
'CjkUnifiedIdeograph5abd': '\xe5\xaa\xbd',
'CherokeeLetterWv': '\xe1\x8f\xae',
'CjkUnifiedIdeograph5abb': '\xe5\xaa\xbb',
'CjkUnifiedIdeograph5abc': '\xe5\xaa\xbc',
'CjkUnifiedIdeograph5aba': '\xe5\xaa\xba',
'CjkUnifiedIdeograph37d0': '\xe3\x9f\x90',
'HangulSyllableGgid': '\xeb\x82\x83',
'HangulSyllableGgig': '\xeb\x81\xbd',
'HangulSyllableGgib': '\xeb\x82\x8d',
'HangulSyllableGgic': '\xeb\x82\x93',
'UnicodeCharacter0xa959': '\xea\xa5\x99',
'HangulSyllableGgim': '\xeb\x82\x8c',
'HangulSyllableGgin': '\xeb\x82\x80',
'HangulSyllableGgih': '\xeb\x82\x97',
'HangulSyllableBuk': '\xeb\xb6\x98',
'HangulSyllableGgij': '\xeb\x82\x92',
'HangulSyllableGgik': '\xeb\x82\x94',
'HangulSyllableGgit': '\xeb\x82\x95',
'UnicodeCharacter0xda7b': '\xed\xa9\xbb',
'HangulSyllableBuj': '\xeb\xb6\x96',
'HangulSyllableGgip': '\xeb\x82\x96',
'HangulSyllableGgis': '\xeb\x82\x8f',
'ModifierLetterSmallJWithCrossedtail': '\xe1\xb6\xa8',
'HangulSyllableBum': '\xeb\xb6\x90',
'HangulSyllableBul': '\xeb\xb6\x88',
'LatinSmallLetterIs': '\xea\x9d\xad',
'HangulSyllableBun': '\xeb\xb6\x84',
'CjkCompatibilityIdeographf906': '\xef\xa4\x86',
'OriyaLetterNa': '\xe0\xac\xa8',
'UnicodeCharacter0xe453': '\xee\x91\x93',
'UnicodeCharacter0xf279': '\xef\x89\xb9',
'G': '\xce\x93',
'NewTaiLueLetterLowNga': '\xe1\xa6\x87',
'KannadaLetterYa': '\xe0\xb2\xaf',
'HangulSyllableBud': '\xeb\xb6\x87',
'CjkUnifiedIdeograph686f': '\xe6\xa1\xaf',
'HangulSyllableRelm': '\xeb\xa0\x92',
'UnicodeCharacter0xe7c4': '\xee\x9f\x84',
'HangulSyllableNoeng': '\xeb\x87\xa1',
'YiSyllableVur': '\xea\x83\xb9',
'CjkCompatibilityIdeographf909': '\xef\xa4\x89',
'HexagramForGreatPreponderance': '\xe4\xb7\x9b',
'YiSyllableVup': '\xea\x83\xb7',
'CjkCompatibilityIdeographf908': '\xef\xa4\x88',
'CjkUnifiedIdeograph7bc4': '\xe7\xaf\x84',
'YiSyllableHxax': '\xea\x89\x8f',
'HangulSyllableBbeoss': '\xeb\xbb\xa4',
'CjkUnifiedIdeograph92aa': '\xe9\x8a\xaa',
'CjkUnifiedIdeograph92ab': '\xe9\x8a\xab',
'CjkUnifiedIdeograph9c0e': '\xe9\xb0\x8e',
'CjkUnifiedIdeograph8d46': '\xe8\xb5\x86',
'CjkUnifiedIdeograph8d47': '\xe8\xb5\x87',
'CjkUnifiedIdeograph8d44': '\xe8\xb5\x84',
'CjkUnifiedIdeograph8d45': '\xe8\xb5\x85',
'CjkUnifiedIdeograph8d42': '\xe8\xb5\x82',
'GreekCapitalLetterIotaWithTonos': '\xce\x8a',
'CjkUnifiedIdeograph8d40': '\xe8\xb5\x80',
'CjkUnifiedIdeograph8d41': '\xe8\xb5\x81',
'OriyaDigitThree': '\xe0\xad\xa9',
'HangulSyllableRelb': '\xeb\xa0\x93',
'CjkUnifiedIdeograph5a47': '\xe5\xa9\x87',
'KangxiRadicalDry': '\xe2\xbc\xb2',
'MyanmarLetterGa': '\xe1\x80\x82',
'CjkUnifiedIdeograph8d49': '\xe8\xb5\x89',
'CjkUnifiedIdeograph5a48': '\xe5\xa9\x88',
'CjkUnifiedIdeograph92af': '\xe9\x8a\xaf',
'MetricalTwoShortsJoined': '\xe2\x8f\x96',
'YiSyllableWat': '\xea\x8a\x80',
'YiSyllableVux': '\xea\x83\xb5',
'CjkUnifiedIdeograph4a6b': '\xe4\xa9\xab',
'HangulSyllableGgabs': '\xea\xb9\x9e',
'IdeographicTelegraphLineFeedSeparatorSymbol': '\xe3\x80\xb7',
'BengaliLetterJha': '\xe0\xa6\x9d',
'FullwidthSemicolon': '\xef\xbc\x9b',
'YiSyllableHxat': '\xea\x89\x8e',
'HangulSyllableSsaess': '\xec\x8c\xa8',
'LepchaLetterVa': '\xe1\xb0\x9f',
'UnicodeCharacter0xe3bf': '\xee\x8e\xbf',
'tripleintegral': '\xe2\x88\xad',
'CjkUnifiedIdeograph8d4f': '\xe8\xb5\x8f',
'CjkUnifiedIdeograph5ef6': '\xe5\xbb\xb6',
'CjkUnifiedIdeograph8d4d': '\xe8\xb5\x8d',
'CjkUnifiedIdeograph8d4e': '\xe8\xb5\x8e',
'CjkUnifiedIdeograph8d4b': '\xe8\xb5\x8b',
'CjkUnifiedIdeograph8d4c': '\xe8\xb5\x8c',
'CjkUnifiedIdeograph8d4a': '\xe8\xb5\x8a',
'HangulSyllableRelt': '\xeb\xa0\x95',
'GreekSmallLetterPi': '\xcf\x80',
'CjkUnifiedIdeograph9b2e': '\xe9\xac\xae',
'CjkUnifiedIdeograph5ef2': '\xe5\xbb\xb2',
'ArabicLigatureSeenWithHehMedialForm': '\xef\xb3\xa8',
'HangulSyllableRels': '\xeb\xa0\x94',
'HangulSyllableRelp': '\xeb\xa0\x96',
'FullwidthLeftSquareBracket': '\xef\xbc\xbb',
'LatinSmallLetterAWithDoubleGrave': '\xc8\x81',
'BrokenBar': '\xc2\xa6',
'CjkUnifiedIdeograph37df': '\xe3\x9f\x9f',
'ArabicLetterAinWithThreeDotsAbove': '\xda\xa0',
'HangulSyllableEss': '\xec\x97\xa4',
'FullwidthCommercialAt': '\xef\xbc\xa0',
'HangulSyllableBbyanj': '\xeb\xba\x9d',
'SquareWithLeftHalfBlack': '\xe2\x97\xa7',
'UnicodeCharacter0xf318': '\xef\x8c\x98',
'EthiopicSyllableGgwee': '\xe2\xb6\x95',
'GlagoliticSmallLetterZemlja': '\xe2\xb0\xb8',
'UnicodeCharacter0xf313': '\xef\x8c\x93',
'UnicodeCharacter0xf310': '\xef\x8c\x90',
'HangulSyllableBbyanh': '\xeb\xba\x9e',
'UnicodeCharacter0xf316': '\xef\x8c\x96',
'UnicodeCharacter0xf317': '\xef\x8c\x97',
'ArabicLetterSheen': '\xd8\xb4',
'UnicodeCharacter0xf315': '\xef\x8c\x95',
'UnicodeCharacter0xfff6': '\xef\xbf\xb6',
'UnicodeCharacter0xfff7': '\xef\xbf\xb7',
'UnicodeCharacter0xfff4': '\xef\xbf\xb4',
'LatinSmallLetterSWithCaronAndDotAbove': '\xe1\xb9\xa7',
'UnicodeCharacter0xfff2': '\xef\xbf\xb2',
'UnicodeCharacter0xf743': '\xef\x9d\x83',
'UnicodeCharacter0xfff0': '\xef\xbf\xb0',
'ArabicLigatureYehWithHahInitialForm': '\xef\xb3\x9b',
'UnicodeCharacter0xfff8': '\xef\xbf\xb8',
'HangulSyllableKwip': '\xed\x81\x8e',
'HangulSyllableKwis': '\xed\x81\x87',
'HangulSyllableKwit': '\xed\x81\x8d',
'LatinCapitalLetterAWithBreveAndDotBelow': '\xe1\xba\xb6',
'KayahLiDigitEight': '\xea\xa4\x88',
'HangulSyllableJyeonj': '\xec\xa0\xbd',
'HangulSyllableJyeonh': '\xec\xa0\xbe',
'HangulSyllableKwic': '\xed\x81\x8b',
'HangulSyllableKwid': '\xed\x80\xbb',
'HangulSyllableKwig': '\xed\x80\xb5',
'HangulSyllableKwih': '\xed\x81\x8f',
'CombiningNotTildeAbove': '\xcd\x8a',
'HangulSyllableKwik': '\xed\x81\x8c',
'HangulSyllableKwil': '\xed\x80\xbc',
'HangulSyllableJyeong': '\xec\xa1\x8d',
'HangulSyllableKwin': '\xed\x80\xb8',
'UnicodeCharacter0xffff': '\xef\xbf\xbf',
'UnicodeCharacter0xfffe': '\xef\xbf\xbe',
'HangulSyllableSyunh': '\xec\x8a\x8e',
'HangulSyllableHoess': '\xed\x9a\xa0',
'HangulSyllableSyunj': '\xec\x8a\x8d',
'CjkUnifiedIdeograph7b59': '\xe7\xad\x99',
'HangulSyllableSyung': '\xec\x8a\x9d',
'YiSyllableShurx': '\xea\x8e\xbe',
'UnicodeCharacter0xf31b': '\xef\x8c\x9b',
'UnicodeCharacter0xf31c': '\xef\x8c\x9c',
'UnicodeCharacter0xf31a': '\xef\x8c\x9a',
'UnicodeCharacter0xf31f': '\xef\x8c\x9f',
'HangulSyllableOls': '\xec\x98\xb0',
'UnicodeCharacter0xf31d': '\xef\x8c\x9d',
'UnicodeCharacter0xf31e': '\xef\x8c\x9e',
'GlagoliticSmallLetterDobro': '\xe2\xb0\xb4',
'ChamLetterKa': '\xea\xa8\x86',
'CjkUnifiedIdeograph9c48': '\xe9\xb1\x88',
'GreekSmallLetterArchaicSampi': '\xcd\xb3',
'CyrillicCapitalLetterZemlya': '\xea\x99\x80',
'HangulJungseongUae': '\xe1\x86\x8a',
'CjkUnifiedIdeograph9e58': '\xe9\xb9\x98',
'HangulSyllableHeng': '\xed\x97\xb9',
'CjkUnifiedIdeograph9e53': '\xe9\xb9\x93',
'CjkUnifiedIdeograph9b2c': '\xe9\xac\xac',
'CjkUnifiedIdeograph9e51': '\xe9\xb9\x91',
'CjkUnifiedIdeograph9e50': '\xe9\xb9\x90',
'CjkUnifiedIdeograph9e57': '\xe9\xb9\x97',
'CjkUnifiedIdeograph9e56': '\xe9\xb9\x96',
'CjkUnifiedIdeograph9e55': '\xe9\xb9\x95',
'CjkUnifiedIdeograph9e54': '\xe9\xb9\x94',
'CyrillicCapitalLetterZhe': '\xd0\x96',
'CjkUnifiedIdeograph8f10': '\xe8\xbc\x90',
'CjkUnifiedIdeograph9c45': '\xe9\xb1\x85',
'HangulSyllableNweop': '\xeb\x88\xba',
'HangulSyllableNweos': '\xeb\x88\xb3',
'GreaterthanClosedByCurveAboveSlantedEqual': '\xe2\xaa\xa9',
'NewTaiLueLetterHighTa': '\xe1\xa6\x8e',
'CjkUnifiedIdeograph7803': '\xe7\xa0\x83',
'CjkUnifiedIdeograph7802': '\xe7\xa0\x82',
'CjkUnifiedIdeograph7801': '\xe7\xa0\x81',
'TagalogLetterHa': '\xe1\x9c\x91',
'CjkUnifiedIdeograph7807': '\xe7\xa0\x87',
'CjkUnifiedIdeograph7806': '\xe7\xa0\x86',
'CjkUnifiedIdeograph7805': '\xe7\xa0\x85',
'CjkUnifiedIdeograph7804': '\xe7\xa0\x84',
'HangulSyllableNweom': '\xeb\x88\xb0',
'HangulSyllableNweol': '\xeb\x88\xa8',
'CjkUnifiedIdeograph7809': '\xe7\xa0\x89',
'CjkUnifiedIdeograph7808': '\xe7\xa0\x88',
'HangulSyllableNweoh': '\xeb\x88\xbb',
'HangulSyllableNweok': '\xeb\x88\xb8',
'CjkUnifiedIdeograph8f17': '\xe8\xbc\x97',
'HangulSyllableByalp': '\xeb\xb1\x9a',
'CjkUnifiedIdeograph9e5c': '\xe9\xb9\x9c',
'CjkUnifiedIdeograph9e5b': '\xe9\xb9\x9b',
'CjkUnifiedIdeograph9e5a': '\xe9\xb9\x9a',
'CjkUnifiedIdeograph9e5f': '\xe9\xb9\x9f',
'CjkUnifiedIdeograph9e5e': '\xe9\xb9\x9e',
'CjkUnifiedIdeograph9e5d': '\xe9\xb9\x9d',
'UnicodeCharacter0xdab6': '\xed\xaa\xb6',
'ArabicLigatureShaddaWithKasratanIsolatedForm': '\xef\xb1\x9f',
'HangulSyllableMyed': '\xeb\xaa\x93',
'ArmenianSmallLetterNow': '\xd5\xb6',
'CanadianSyllabicsFullStop': '\xe1\x99\xae',
'CjkUnifiedIdeograph8f16': '\xe8\xbc\x96',
'CjkUnifiedIdeograph780c': '\xe7\xa0\x8c',
'CjkUnifiedIdeograph780b': '\xe7\xa0\x8b',
'CjkUnifiedIdeograph780a': '\xe7\xa0\x8a',
'CjkUnifiedIdeograph780f': '\xe7\xa0\x8f',
'CjkUnifiedIdeograph780e': '\xe7\xa0\x8e',
'CjkUnifiedIdeograph780d': '\xe7\xa0\x8d',
'ArabicLetterAlefWithHamzaBelow': '\xd8\xa5',
'UnicodeCharacter0xe3a7': '\xee\x8e\xa7',
'KayahLiLetterCa': '\xea\xa4\xa1',
'NewTaiLueVowelSignUey': '\xe1\xa6\xbf',
'CjkUnifiedIdeograph602d': '\xe6\x80\xad',
'CjkUnifiedIdeograph40fc': '\xe4\x83\xbc',
'UnicodeCharacter0xa793': '\xea\x9e\x93',
'HangulSyllableDdyae': '\xeb\x96\x84',
'HangulSyllableDdyag': '\xeb\x95\xa9',
'HangulSyllableDdyab': '\xeb\x95\xb9',
'HangulSyllableDdyac': '\xeb\x95\xbf',
'HangulSyllableDdyal': '\xeb\x95\xb0',
'HangulSyllableDdyam': '\xeb\x95\xb8',
'HangulSyllableDdyan': '\xeb\x95\xac',
'HangulSyllableDdyah': '\xeb\x96\x83',
'HangulSyllableDdyaj': '\xeb\x95\xbe',
'CjkUnifiedIdeograph5efc': '\xe5\xbb\xbc',
'HangulSyllableDdyat': '\xeb\x96\x81',
'UnicodeCharacter0xd8eb': '\xed\xa3\xab',
'HangulSyllableSas': '\xec\x82\xbf',
'HangulSyllableDdyap': '\xeb\x96\x82',
'HangulSyllableDdyas': '\xeb\x95\xbb',
'CjkUnifiedIdeograph604d': '\xe6\x81\x8d',
'CjkUnifiedIdeograph604e': '\xe6\x81\x8e',
'CjkUnifiedIdeograph604f': '\xe6\x81\x8f',
'CjkUnifiedIdeograph909e': '\xe9\x82\x9e',
'CjkUnifiedIdeograph909b': '\xe9\x82\x9b',
'CjkUnifiedIdeograph604a': '\xe6\x81\x8a',
'CjkUnifiedIdeograph604b': '\xe6\x81\x8b',
'CjkUnifiedIdeograph604c': '\xe6\x81\x8c',
'CjkUnifiedIdeograph36e2': '\xe3\x9b\xa2',
'CjkUnifiedIdeograph36e3': '\xe3\x9b\xa3',
'CjkUnifiedIdeograph36e0': '\xe3\x9b\xa0',
'CjkUnifiedIdeograph36e1': '\xe3\x9b\xa1',
'CjkUnifiedIdeograph36e6': '\xe3\x9b\xa6',
'CjkUnifiedIdeograph36e7': '\xe3\x9b\xa7',
'CjkUnifiedIdeograph36e4': '\xe3\x9b\xa4',
'CjkUnifiedIdeograph36e5': '\xe3\x9b\xa5',
'CjkUnifiedIdeograph36e8': '\xe3\x9b\xa8',
'CjkUnifiedIdeograph36e9': '\xe3\x9b\xa9',
'UnicodeCharacter0xeed9': '\xee\xbb\x99',
'HangulSyllableYing': '\xec\x9d\xad',
'HangulSyllableYinh': '\xec\x9d\x9e',
'HangulSyllableYinj': '\xec\x9d\x9d',
'UnicodeCharacter0xa791': '\xea\x9e\x91',
'CjkUnifiedIdeograph84c8': '\xe8\x93\x88',
'UnicodeCharacter0xed18': '\xee\xb4\x98',
'CjkUnifiedIdeograph8edb': '\xe8\xbb\x9b',
'CjkUnifiedIdeograph6048': '\xe6\x81\x88',
'GreekCapitalLetterAlphaWithPsili': '\xe1\xbc\x88',
'CjkUnifiedIdeograph9098': '\xe9\x82\x98',
'CjkUnifiedIdeograph9099': '\xe9\x82\x99',
'CjkUnifiedIdeograph6044': '\xe6\x81\x84',
'LowAsterisk': '\xe2\x81\x8e',
'CjkUnifiedIdeograph6046': '\xe6\x81\x86',
'CjkUnifiedIdeograph6047': '\xe6\x81\x87',
'CjkUnifiedIdeograph6040': '\xe6\x81\x80',
'CjkUnifiedIdeograph6041': '\xe6\x81\x81',
'CjkUnifiedIdeograph6042': '\xe6\x81\x82',
'CjkUnifiedIdeograph6043': '\xe6\x81\x83',
'CjkUnifiedIdeograph92a0': '\xe9\x8a\xa0',
'UnicodeCharacter0xdb5b': '\xed\xad\x9b',
'SlantedEqualToOrGreaterthanWithDotInside': '\xe2\xaa\x98',
'CjkUnifiedIdeograph92a1': '\xe9\x8a\xa1',
'UnicodeCharacter0xa790': '\xea\x9e\x90',
'HangulSyllableWi': '\xec\x9c\x84',
'CjkUnifiedIdeograph92a2': '\xe9\x8a\xa2',
'HangulSyllableRwanj': '\xeb\xa1\xbd',
'CjkUnifiedIdeograph92a3': '\xe9\x8a\xa3',
'TamilSignAnusvara': '\xe0\xae\x82',
'UnicodeCharacter0xdab5': '\xed\xaa\xb5',
'ArabicLetterRrehIsolatedForm': '\xef\xae\x8c',
'CjkUnifiedIdeograph92a4': '\xe9\x8a\xa4',
'CjkUnifiedIdeograph92a5': '\xe9\x8a\xa5',
'CjkUnifiedIdeograph36eb': '\xe3\x9b\xab',
'GreekSmallLetterOmegaWithDasiaAndVariaAndYpogegrammeni': '\xe1\xbe\xa3',
'CjkUnifiedIdeograph6aa4': '\xe6\xaa\xa4',
'CjkUnifiedIdeograph36ea': '\xe3\x9b\xaa',
'CjkUnifiedIdeograph36ef': '\xe3\x9b\xaf',
'CjkUnifiedIdeograph6aa3': '\xe6\xaa\xa3',
'CjkUnifiedIdeograph36ed': '\xe3\x9b\xad',
'CjkUnifiedIdeograph36ee': '\xe3\x9b\xae',
'GreekCapitalLetterPi': '\xce\xa0',
'HebrewLetterVav': '\xd7\x95',
'CjkUnifiedIdeograph6aa8': '\xe6\xaa\xa8',
'GeorgianCapitalLetterPar': '\xe1\x82\xae',
'CjkUnifiedIdeograph79d6': '\xe7\xa7\x96',
'CjkUnifiedIdeograph79d7': '\xe7\xa7\x97',
'CjkUnifiedIdeograph43f9': '\xe4\x8f\xb9',
'CjkUnifiedIdeograph43f8': '\xe4\x8f\xb8',
'CjkUnifiedIdeograph79d2': '\xe7\xa7\x92',
'CjkUnifiedIdeograph79d3': '\xe7\xa7\x93',
'CjkUnifiedIdeograph79d0': '\xe7\xa7\x90',
'CjkUnifiedIdeograph79d1': '\xe7\xa7\x91',
'UnicodeCharacter0x2073': '\xe2\x81\xb3',
'UnicodeCharacter0x2072': '\xe2\x81\xb2',
'CjkUnifiedIdeograph43f1': '\xe4\x8f\xb1',
'CjkUnifiedIdeograph43f0': '\xe4\x8f\xb0',
'CjkUnifiedIdeograph43f7': '\xe4\x8f\xb7',
'CjkUnifiedIdeograph43f6': '\xe4\x8f\xb6',
'CjkUnifiedIdeograph43f5': '\xe4\x8f\xb5',
'CjkUnifiedIdeograph43f4': '\xe4\x8f\xb4',
'HangulSyllableBoeng': '\xeb\xb5\x9d',
'GlagoliticSmallLetterI': '\xe2\xb0\xbb',
'MyanmarLetterShanGa': '\xe1\x81\xb7',
'HangulSyllableCeuss': '\xec\xb8\xb4',
'UnicodeCharacter0xed17': '\xee\xb4\x97',
'HangulSyllableBoenj': '\xeb\xb5\x8d',
'HangulSyllableBoenh': '\xeb\xb5\x8e',
'SupersetAboveSubset': '\xe2\xab\x94',
'HangulSyllableMalm': '\xeb\xa7\x92',
'HangulSyllableSak': '\xec\x83\x84',
'HangulSyllableMalh': '\xeb\xa7\x97',
'CjkUnifiedIdeograph739c': '\xe7\x8e\x9c',
'CjkUnifiedIdeograph739b': '\xe7\x8e\x9b',
'CjkUnifiedIdeograph739a': '\xe7\x8e\x9a',
'DashedTriangleheadedRightwardsArrow': '\xe2\x9e\x9f',
'archaickoppa': '\xcf\x99',
'CjkUnifiedIdeograph739f': '\xe7\x8e\x9f',
'CjkUnifiedIdeograph739e': '\xe7\x8e\x9e',
'CjkUnifiedIdeograph739d': '\xe7\x8e\x9d',
'HangulSyllableSan': '\xec\x82\xb0',
'HangulSyllableDul': '\xeb\x91\x98',
'HangulSyllableMalt': '\xeb\xa7\x95',
'HangulSyllableMals': '\xeb\xa7\x94',
'HangulSyllableMalp': '\xeb\xa7\x96',
'HangulSyllableJabs': '\xec\x9e\xa2',
'HangulChoseongSsangkiyeok': '\xe1\x84\x81',
'HangulSyllableJwaj': '\xec\xa2\xa2',
'UnicodeCharacter0xe3f6': '\xee\x8f\xb6',
'CjkUnifiedIdeograph79df': '\xe7\xa7\x9f',
'CjkUnifiedIdeograph79dd': '\xe7\xa7\x9d',
'CjkUnifiedIdeograph79de': '\xe7\xa7\x9e',
'CjkUnifiedIdeograph79db': '\xe7\xa7\x9b',
'CjkUnifiedIdeograph79dc': '\xe7\xa7\x9c',
'CjkUnifiedIdeograph79da': '\xe7\xa7\x9a',
'CjkUnifiedIdeograph43fc': '\xe4\x8f\xbc',
'CjkUnifiedIdeograph43fb': '\xe4\x8f\xbb',
'CjkUnifiedIdeograph43fa': '\xe4\x8f\xba',
'CjkUnifiedIdeograph954c': '\xe9\x95\x8c',
'CjkUnifiedIdeograph954d': '\xe9\x95\x8d',
'CjkUnifiedIdeograph43ff': '\xe4\x8f\xbf',
'IdeographicComma': '\xe3\x80\x81',
'CjkUnifiedIdeograph43fd': '\xe4\x8f\xbd',
'HangulSyllableJjwegg': '\xec\xae\x82',
'HangulSyllableNalt': '\xeb\x82\xa5',
'SundaneseDigitZero': '\xe1\xae\xb0',
'HangulSyllableJjaebs': '\xec\xa8\x8a',
'CombiningShortVerticalLineOverlay': '\xe2\x83\x93',
'CjkUnifiedIdeograph7399': '\xe7\x8e\x99',
'CjkUnifiedIdeograph7398': '\xe7\x8e\x98',
'HangulSyllableJjwegs': '\xec\xae\x83',
'VaiSyllableHen': '\xea\x97\xa5',
'CjkUnifiedIdeograph7393': '\xe7\x8e\x93',
'BengaliCurrencyNumeratorTwo': '\xe0\xa7\xb5',
'CjkUnifiedIdeograph7391': '\xe7\x8e\x91',
'CjkUnifiedIdeograph7390': '\xe7\x8e\x90',
'CjkUnifiedIdeograph7397': '\xe7\x8e\x97',
'CjkUnifiedIdeograph7396': '\xe7\x8e\x96',
'CjkUnifiedIdeograph7395': '\xe7\x8e\x95',
'CjkUnifiedIdeograph7394': '\xe7\x8e\x94',
'CjkUnifiedIdeograph7025': '\xe7\x80\xa5',
'CjkUnifiedIdeograph7024': '\xe7\x80\xa4',
'CjkUnifiedIdeograph7027': '\xe7\x80\xa7',
'CjkUnifiedIdeograph7026': '\xe7\x80\xa6',
'CjkUnifiedIdeograph7021': '\xe7\x80\xa1',
'CjkUnifiedIdeograph7020': '\xe7\x80\xa0',
'CjkUnifiedIdeograph7023': '\xe7\x80\xa3',
'CjkUnifiedIdeograph7022': '\xe7\x80\xa2',
'UnicodeCharacter0xe3f4': '\xee\x8f\xb4',
'CjkUnifiedIdeograph7029': '\xe7\x80\xa9',
'CjkUnifiedIdeograph7028': '\xe7\x80\xa8',
'ThaiCharacterKhoRakhang': '\xe0\xb8\x86',
'HangulSyllableMwid': '\xeb\xae\x8f',
'HangulSyllableJyeog': '\xec\xa0\xb9',
'HangulSyllableJyeod': '\xec\xa0\xbf',
'HangulSyllableJyeoc': '\xec\xa1\x8f',
'HangulSyllableJyeob': '\xec\xa1\x89',
'HangulSyllableHing': '\xed\x9e\x9d',
'HangulSyllableJyeon': '\xec\xa0\xbc',
'HangulSyllableJyeom': '\xec\xa1\x88',
'HangulSyllableJyeol': '\xec\xa1\x80',
'HangulSyllableJyeok': '\xec\xa1\x90',
'HangulSyllableJyeoj': '\xec\xa1\x8e',
'ArabicSmallLowMeem': '\xdb\xad',
'CjkUnifiedIdeograph3735': '\xe3\x9c\xb5',
'CjkUnifiedIdeograph3734': '\xe3\x9c\xb4',
'CjkUnifiedIdeograph3bd3': '\xe3\xaf\x93',
'CjkUnifiedIdeograph3bd2': '\xe3\xaf\x92',
'CjkUnifiedIdeograph3bd5': '\xe3\xaf\x95',
'CjkUnifiedIdeograph3bd4': '\xe3\xaf\x94',
'CjkUnifiedIdeograph3bd7': '\xe3\xaf\x97',
'CjkUnifiedIdeograph3bd6': '\xe3\xaf\x96',
'GreekCapitalLetterUpsilonWithVrachy': '\xe1\xbf\xa8',
'CjkUnifiedIdeograph3bd8': '\xe3\xaf\x98',
'CjkUnifiedIdeograph3f77': '\xe3\xbd\xb7',
'CjkUnifiedIdeograph3739': '\xe3\x9c\xb9',
'CjkUnifiedIdeograph3738': '\xe3\x9c\xb8',
'BalineseLetterPaKapal': '\xe1\xac\xa8',
'UnicodeCharacter0xf0ee': '\xef\x83\xae',
'CombiningAcuteAccent': '\xcc\x81',
'UnicodeCharacter0xa798': '\xea\x9e\x98',
'HangulSyllableJwass': '\xec\xa2\xa0',
'CjkUnifiedIdeograph702e': '\xe7\x80\xae',
'CjkUnifiedIdeograph702d': '\xe7\x80\xad',
'LatinSmallLetterOWithDotBelow': '\xe1\xbb\x8d',
'CjkUnifiedIdeograph702a': '\xe7\x80\xaa',
'HangulSyllableBbeunh': '\xec\x81\x9e',
'CjkUnifiedIdeograph702c': '\xe7\x80\xac',
'CjkUnifiedIdeograph702b': '\xe7\x80\xab',
'CanadianSyllabicsWestcreeMwaa': '\xe1\x92\xb9',
'CjkUnifiedIdeograph373e': '\xe3\x9c\xbe',
'CjkUnifiedIdeograph373d': '\xe3\x9c\xbd',
'CjkUnifiedIdeograph3bdc': '\xe3\xaf\x9c',
'CjkUnifiedIdeograph373f': '\xe3\x9c\xbf',
'CjkUnifiedIdeograph373a': '\xe3\x9c\xba',
'CjkUnifiedIdeograph3bdd': '\xe3\xaf\x9d',
'CjkUnifiedIdeograph373c': '\xe3\x9c\xbc',
'CjkUnifiedIdeograph373b': '\xe3\x9c\xbb',
'HangulSyllableHinj': '\xed\x9e\x8d',
'HangulSyllablePoc': '\xed\x90\x83',
'HangulSyllablePob': '\xed\x8f\xbd',
'HangulSyllablePoe': '\xed\x91\x80',
'HangulSyllablePod': '\xed\x8f\xb3',
'HangulSyllablePog': '\xed\x8f\xad',
'AplFunctionalSymbolUpShoeJot': '\xe2\x8d\x9d',
'CjkUnifiedIdeograph9a81': '\xe9\xaa\x81',
'GeorgianLetterElifi': '\xe1\x83\xb8',
'HangulSyllableMwic': '\xeb\xae\x9f',
'HangulSyllablePop': '\xed\x90\x86',
'HangulSyllablePos': '\xed\x8f\xbf',
'HangulSyllablePot': '\xed\x90\x85',
'UnicodeCharacter0xeb01': '\xee\xac\x81',
'YiSyllableZap': '\xea\x8a\x97',
'HangulSyllableEs': '\xec\x97\xa3',
'LaoLetterHoTam': '\xe0\xba\xae',
'YiSyllableZat': '\xea\x8a\x94',
'CjkUnifiedIdeograph9fb8': '\xe9\xbe\xb8',
'NotEqualTo': '\xe2\x89\xa0',
'TibetanSignLciRtags': '\xe0\xbe\x86',
'RunicBelgthorSymbol': '\xe1\x9b\xb0',
'UnicodeCharacter0xeb03': '\xee\xac\x83',
'UnicodeCharacter0xe80f': '\xee\xa0\x8f',
'CjkUnifiedIdeograph584d': '\xe5\xa1\x8d',
'Aktieselskab': '\xe2\x85\x8d',
'ArabicSignSanah': '\xd8\x81',
'UnicodeCharacter0xf0ef': '\xef\x83\xaf',
'UnicodeCharacter0xeb04': '\xee\xac\x84',
'UnicodeCharacter0xe3f9': '\xee\x8f\xb9',
'BraillePatternDots38': '\xe2\xa2\x84',
'UnicodeCharacter0xe8e9': '\xee\xa3\xa9',
'BraillePatternDots36': '\xe2\xa0\xa4',
'BraillePatternDots37': '\xe2\xa1\x84',
'BraillePatternDots34': '\xe2\xa0\x8c',
'BraillePatternDots35': '\xe2\xa0\x94',
'Taurus': '\xe2\x99\x89',
'CjkUnifiedIdeograph8827': '\xe8\xa0\xa7',
'CjkUnifiedIdeograph3df1': '\xe3\xb7\xb1',
'UnicodeCharacter0xe615': '\xee\x98\x95',
'CjkUnifiedIdeograph3954': '\xe3\xa5\x94',
'CjkUnifiedIdeograph681c': '\xe6\xa0\x9c',
'CjkUnifiedIdeograph681b': '\xe6\xa0\x9b',
'CjkUnifiedIdeograph681a': '\xe6\xa0\x9a',
'UnicodeCharacter0xf0ea': '\xef\x83\xaa',
'CjkUnifiedIdeograph3df3': '\xe3\xb7\xb3',
'CjkUnifiedIdeograph681e': '\xe6\xa0\x9e',
'CjkUnifiedIdeograph681d': '\xe6\xa0\x9d',
'CjkUnifiedIdeograph3df2': '\xe3\xb7\xb2',
'UnicodeCharacter0xee6f': '\xee\xb9\xaf',
'CjkUnifiedIdeograph3df5': '\xe3\xb7\xb5',
'HangulSyllableJeubs': '\xec\xa6\x9a',
'CjkUnifiedIdeograph8820': '\xe8\xa0\xa0',
'CjkUnifiedIdeograph3df4': '\xe3\xb7\xb4',
'CjkUnifiedIdeograph3df7': '\xe3\xb7\xb7',
'CjkUnifiedIdeograph3df6': '\xe3\xb7\xb6',
'GeorgianLetterAn': '\xe1\x83\x90',
'GreekUpsilonWithHookSymbol': '\xcf\x92',
'CjkUnifiedIdeograph6819': '\xe6\xa0\x99',
'CjkUnifiedIdeograph6818': '\xe6\xa0\x98',
'CjkUnifiedIdeograph6813': '\xe6\xa0\x93',
'CjkUnifiedIdeograph6812': '\xe6\xa0\x92',
'CjkUnifiedIdeograph6811': '\xe6\xa0\x91',
'CjkUnifiedIdeograph6810': '\xe6\xa0\x90',
'CjkUnifiedIdeograph6817': '\xe6\xa0\x97',
'CjkUnifiedIdeograph6816': '\xe6\xa0\x96',
'CjkUnifiedIdeograph6815': '\xe6\xa0\x95',
'CjkUnifiedIdeograph6814': '\xe6\xa0\x94',
'UnicodeCharacter0xe618': '\xee\x98\x98',
'HangulSyllableGaegg': '\xea\xb0\x9e',
'CjkUnifiedIdeograph7249': '\xe7\x89\x89',
'UnicodeCharacter0xe619': '\xee\x98\x99',
'CyrillicCapitalLetterLje': '\xd0\x89',
'HangulSyllableByalg': '\xeb\xb1\x95',
'CjkUnifiedIdeograph7248': '\xe7\x89\x88',
'UnicodeCharacter0x8d3': '\xe0\xa3\x93',
'UnicodeCharacter0x8d2': '\xe0\xa3\x92',
'UnicodeCharacter0x8d1': '\xe0\xa3\x91',
'UnicodeCharacter0x8d0': '\xe0\xa3\x90',
'UnicodeCharacter0x8d7': '\xe0\xa3\x97',
'UnicodeCharacter0x8d6': '\xe0\xa3\x96',
'UnicodeCharacter0x8d5': '\xe0\xa3\x95',
'UnicodeCharacter0x8d4': '\xe0\xa3\x94',
'BoxDrawingsHeavyVertical': '\xe2\x94\x83',
'UnicodeCharacter0x8d9': '\xe0\xa3\x99',
'UnicodeCharacter0x8d8': '\xe0\xa3\x98',
'CjkUnifiedIdeograph3c88': '\xe3\xb2\x88',
'CjkUnifiedIdeograph3c89': '\xe3\xb2\x89',
'BraillePatternDots1457': '\xe2\xa1\x99',
'BraillePatternDots1456': '\xe2\xa0\xb9',
'UnicodeCharacter0xf0eb': '\xef\x83\xab',
'HangulSyllableMolg': '\xeb\xaa\xb1',
'ArabicLigatureDadWithMeemIsolatedForm': '\xef\xb0\xa5',
'SymbolForDeleteFormTwo': '\xe2\x90\xa5',
'HalfwidthHangulLetterRieulkiyeok': '\xef\xbe\xaa',
'HangulLetterThieuth': '\xe3\x85\x8c',
'CjkCompatibilityIdeographf916': '\xef\xa4\x96',
'CjkCompatibilityIdeographf917': '\xef\xa4\x97',
'CjkCompatibilityIdeographf914': '\xef\xa4\x94',
'CjkCompatibilityIdeographf915': '\xef\xa4\x95',
'CjkCompatibilityIdeographf912': '\xef\xa4\x92',
'CjkUnifiedIdeograph7243': '\xe7\x89\x83',
'CjkCompatibilityIdeographf910': '\xef\xa4\x90',
'CjkCompatibilityIdeographf911': '\xef\xa4\x91',
'SquareCorporation': '\xe3\x8d\xbf',
'Projective': '\xe2\x8c\x85',
'CjkUnifiedIdeograph61ea': '\xe6\x87\xaa',
'CjkUnifiedIdeograph61ef': '\xe6\x87\xaf',
'VaiSyllableZi': '\xea\x94\xbd',
'CjkUnifiedIdeograph61ed': '\xe6\x87\xad',
'CjkUnifiedIdeograph61ee': '\xe6\x87\xae',
'UnicodeCharacter0xf5b5': '\xef\x96\xb5',
'IdeographicTelegraphSymbolForJanuary': '\xe3\x8b\x80',
'CjkUnifiedIdeograph994e': '\xe9\xa5\x8e',
'VaiSyllableZo': '\xea\x97\x95',
'UnicodeCharacter0xdb56': '\xed\xad\x96',
'UnicodeCharacter0xf5b7': '\xef\x96\xb7',
'CjkUnifiedIdeograph6c97': '\xe6\xb2\x97',
'UnicodeCharacter0x8dc': '\xe0\xa3\x9c',
'UnicodeCharacter0x8db': '\xe0\xa3\x9b',
'UnicodeCharacter0x8da': '\xe0\xa3\x9a',
'UnicodeCharacter0xf5b6': '\xef\x96\xb6',
'HangulSyllableTaenj': '\xed\x83\xa1',
'UnicodeCharacter0x8df': '\xe0\xa3\x9f',
'UnicodeCharacter0x8de': '\xe0\xa3\x9e',
'CjkUnifiedIdeograph3c8c': '\xe3\xb2\x8c',
'HangulSyllableTaeng': '\xed\x83\xb1',
'UnicodeCharacter0xed1b': '\xee\xb4\x9b',
'UnicodeCharacter0xf5b0': '\xef\x96\xb0',
'CjkCompatibilityIdeographf91f': '\xef\xa4\x9f',
'HangulSyllableKyanh': '\xec\xba\xb2',
'CjkUnifiedIdeograph61e8': '\xe6\x87\xa8',
'CjkUnifiedIdeograph61e9': '\xe6\x87\xa9',
'CjkCompatibilityIdeographf91b': '\xef\xa4\x9b',
'UnicodeCharacter0xf5b3': '\xef\x96\xb3',
'CjkCompatibilityIdeographf91a': '\xef\xa4\x9a',
'CjkUnifiedIdeograph61e2': '\xe6\x87\xa2',
'CjkUnifiedIdeograph61e3': '\xe6\x87\xa3',
'CjkUnifiedIdeograph61e0': '\xe6\x87\xa0',
'LatinSmallLetterNWithPalatalHook': '\xe1\xb6\x87',
'CjkUnifiedIdeograph61e6': '\xe6\x87\xa6',
'CjkUnifiedIdeograph61e7': '\xe6\x87\xa7',
'CjkUnifiedIdeograph61e4': '\xe6\x87\xa4',
'CombiningCyrillicPsiliPneumata': '\xd2\x86',
'HangulSyllableBweolh': '\xeb\xb6\xab',
'HangulSyllableSyeonh': '\xec\x85\x9a',
'UnicodeCharacter0xf8f1': '\xef\xa3\xb1',
'UnicodeCharacter0xf8f2': '\xef\xa3\xb2',
'HangulSyllableMaegs': '\xeb\xa7\xa7',
'UnicodeCharacter0xf8f3': '\xef\xa3\xb3',
'UnicodeCharacter0xe921': '\xee\xa4\xa1',
'HangulSyllableMaegg': '\xeb\xa7\xa6',
'LepchaLetterKla': '\xe1\xb0\x81',
'TamilLetterPa': '\xe0\xae\xaa',
'TagalogLetterSa': '\xe1\x9c\x90',
'HangulSyllableSyilb': '\xec\x8b\x8b',
'CjkUnifiedIdeograph40e1': '\xe4\x83\xa1',
'CjkUnifiedIdeograph40e0': '\xe4\x83\xa0',
'CjkUnifiedIdeograph40e3': '\xe4\x83\xa3',
'CjkUnifiedIdeograph40e2': '\xe4\x83\xa2',
'CjkUnifiedIdeograph40e5': '\xe4\x83\xa5',
'CjkUnifiedIdeograph40e4': '\xe4\x83\xa4',
'CjkUnifiedIdeograph40e7': '\xe4\x83\xa7',
'CjkUnifiedIdeograph40e6': '\xe4\x83\xa6',
'LaoCancellationMark': '\xe0\xbb\x8c',
'W': '\xce\xa9',
'HangulSyllableSyilt': '\xec\x8b\x8d',
'HangulSyllableSyils': '\xec\x8b\x8c',
'CjkUnifiedIdeograph83e7': '\xe8\x8f\xa7',
'ArabicLigatureMeemWithAlefMaksuraIsolatedForm': '\xef\xb1\x89',
'HangulSyllableGgyeng': '\xea\xbc\xa5',
'NkoLetterDagbasinna': '\xdf\x91',
'CyrillicCapitalLetterShortI': '\xd0\x99',
'CyrillicSmallLetterLje': '\xd1\x99',
'CyrillicCapitalLetterShortU': '\xd0\x8e',
'UnicodeCharacter0xe61e': '\xee\x98\x9e',
'CanadianSyllabicsShwo': '\xe1\x94\x9d',
'CanadianSyllabicsShwa': '\xe1\x94\xa1',
'CanadianSyllabicsShwe': '\xe1\x94\x97',
'HangulSyllableDdoelb': '\xeb\x99\xaf',
'UnicodeCharacter0xf6ab': '\xef\x9a\xab',
'YiSyllableZziep': '\xea\x8b\x97',
'LatinSmallLetterAWithDiaeresis': '\xc3\xa4',
'BraillePatternDots3578': '\xe2\xa3\x94',
'HangulSyllableHoej': '\xed\x9a\xa2',
'CjkUnifiedIdeograph40ea': '\xe4\x83\xaa',
'CjkUnifiedIdeograph40ec': '\xe4\x83\xac',
'CjkUnifiedIdeograph40eb': '\xe4\x83\xab',
'CjkUnifiedIdeograph40ee': '\xe4\x83\xae',
'CjkUnifiedIdeograph40ed': '\xe4\x83\xad',
'CjkUnifiedIdeograph40ef': '\xe4\x83\xaf',
'UnicodeCharacter0x1fc5': '\xe1\xbf\x85',
'MyanmarSignShanTone2': '\xe1\x82\x87',
'MyanmarSignShanTone3': '\xe1\x82\x88',
'MyanmarSignShanTone6': '\xe1\x82\x8a',
'MyanmarSignShanTone5': '\xe1\x82\x89',
'SurfaceIntegral': '\xe2\x88\xaf',
'LatinCapitalLetterGWithBreve': '\xc4\x9e',
'TagbanwaLetterTa': '\xe1\x9d\xa6',
'MyanmarShanDigitOne': '\xe1\x82\x91',
'CjkRadicalRain': '\xe2\xbb\x97',
'CjkUnifiedIdeograph4d7b': '\xe4\xb5\xbb',
'CjkUnifiedIdeograph4d7a': '\xe4\xb5\xba',
'UnicodeCharacter0xaada': '\xea\xab\x9a',
'UnicodeCharacter0xaadf': '\xea\xab\x9f',
'CjkUnifiedIdeograph4d7f': '\xe4\xb5\xbf',
'CjkUnifiedIdeograph4d7e': '\xe4\xb5\xbe',
'CjkUnifiedIdeograph4d7d': '\xe4\xb5\xbd',
'CjkUnifiedIdeograph50ab': '\xe5\x82\xab',
'CjkUnifiedIdeograph50ac': '\xe5\x82\xac',
'CjkUnifiedIdeograph50aa': '\xe5\x82\xaa',
'CjkUnifiedIdeograph50af': '\xe5\x82\xaf',
'UnicodeCharacter0xf1c6': '\xef\x87\x86',
'CjkUnifiedIdeograph50ad': '\xe5\x82\xad',
'CjkUnifiedIdeograph50ae': '\xe5\x82\xae',
'UnicodeCharacter0xf15a': '\xef\x85\x9a',
'UnicodeCharacter0xf15b': '\xef\x85\x9b',
'UnicodeCharacter0xf15c': '\xef\x85\x9c',
'UnicodeCharacter0xf15d': '\xef\x85\x9d',
'UnicodeCharacter0xf15e': '\xef\x85\x9e',
'UnicodeCharacter0xf15f': '\xef\x85\x9f',
'HangulSyllableJjyeos': '\xec\xaa\x97',
'HangulSyllableGgyenj': '\xea\xbc\x95',
'ArabicLetterDul': '\xda\x8e',
'CanadianSyllabicsWestcreeYwaa': '\xe1\x94\xbc',
'UnicodeCharacter0xf6af': '\xef\x9a\xaf',
'ArabicLigatureBehWithHehMedialForm': '\xef\xb3\xa2',
'CjkUnifiedIdeograph50a2': '\xe5\x82\xa2',
'CjkUnifiedIdeograph50a3': '\xe5\x82\xa3',
'CjkUnifiedIdeograph50a0': '\xe5\x82\xa0',
'CjkUnifiedIdeograph50a1': '\xe5\x82\xa1',
'CjkUnifiedIdeograph50a6': '\xe5\x82\xa6',
'CjkUnifiedIdeograph50a7': '\xe5\x82\xa7',
'CjkUnifiedIdeograph50a4': '\xe5\x82\xa4',
'CjkUnifiedIdeograph50a5': '\xe5\x82\xa5',
'UnicodeCharacter0xf150': '\xef\x85\x90',
'UnicodeCharacter0xf151': '\xef\x85\x91',
'CjkUnifiedIdeograph50a8': '\xe5\x82\xa8',
'CjkUnifiedIdeograph50a9': '\xe5\x82\xa9',
'UnicodeCharacter0xf154': '\xef\x85\x94',
'UnicodeCharacter0xf155': '\xef\x85\x95',
'TibetanCantillationSignLightBeat': '\xe0\xbf\x81',
'KhmerVowelSignAi': '\xe1\x9f\x83',
'CjkUnifiedIdeograph4d79': '\xe4\xb5\xb9',
'KhmerVowelSignAu': '\xe1\x9f\x85',
'HangulSyllableNang': '\xeb\x82\xad',
'UnicodeCharacter0xe0bb': '\xee\x82\xbb',
'CjkUnifiedIdeograph4d73': '\xe4\xb5\xb3',
'CjkUnifiedIdeograph4d72': '\xe4\xb5\xb2',
'CjkUnifiedIdeograph4d71': '\xe4\xb5\xb1',
'CjkUnifiedIdeograph4d70': '\xe4\xb5\xb0',
'KangxiRadicalEarth': '\xe2\xbc\x9f',
'CjkUnifiedIdeograph4d76': '\xe4\xb5\xb6',
'CjkUnifiedIdeograph4d75': '\xe4\xb5\xb5',
'CjkUnifiedIdeograph4d74': '\xe4\xb5\xb4',
'CjkUnifiedIdeograph5b72': '\xe5\xad\xb2',
'CjkUnifiedIdeograph5b73': '\xe5\xad\xb3',
'CjkUnifiedIdeograph5b70': '\xe5\xad\xb0',
'CjkUnifiedIdeograph5b71': '\xe5\xad\xb1',
'CjkUnifiedIdeograph5b76': '\xe5\xad\xb6',
'CjkUnifiedIdeograph5b77': '\xe5\xad\xb7',
'CjkUnifiedIdeograph5b74': '\xe5\xad\xb4',
'CjkUnifiedIdeograph5b75': '\xe5\xad\xb5',
'HangulSyllablePweok': '\xed\x92\xac',
'HangulSyllablePweoj': '\xed\x92\xaa',
'CjkUnifiedIdeograph5b78': '\xe5\xad\xb8',
'CjkUnifiedIdeograph5b79': '\xe5\xad\xb9',
'HebrewLetterShinWithSinDot': '\xef\xac\xab',
'HangulSyllablePweon': '\xed\x92\x98',
'CjkUnifiedIdeograph90c9': '\xe9\x83\x89',
'HangulSyllablePweol': '\xed\x92\x9c',
'HangulSyllablePweos': '\xed\x92\xa7',
'CjkUnifiedIdeograph3dfa': '\xe3\xb7\xba',
'HangulSyllablePweop': '\xed\x92\xae',
'HangulSyllableDdeogg': '\xeb\x96\xa2',
'HangulSyllablePweot': '\xed\x92\xad',
'CjkUnifiedIdeograph3dfc': '\xe3\xb7\xbc',
'CjkUnifiedIdeograph51ec': '\xe5\x87\xac',
'CjkUnifiedIdeograph51eb': '\xe5\x87\xab',
'CjkUnifiedIdeograph51ea': '\xe5\x87\xaa',
'CjkUnifiedIdeograph3dfb': '\xe3\xb7\xbb',
'UnicodeCharacter0xf164': '\xef\x85\xa4',
'CjkUnifiedIdeograph51ef': '\xe5\x87\xaf',
'CjkUnifiedIdeograph51ee': '\xe5\x87\xae',
'CjkUnifiedIdeograph51ed': '\xe5\x87\xad',
'CjkUnifiedIdeograph395a': '\xe3\xa5\x9a',
'CjkUnifiedIdeograph894c': '\xe8\xa5\x8c',
'CjkUnifiedIdeograph894b': '\xe8\xa5\x8b',
'CjkUnifiedIdeograph894a': '\xe8\xa5\x8a',
'CjkUnifiedIdeograph894f': '\xe8\xa5\x8f',
'CjkUnifiedIdeograph894e': '\xe8\xa5\x8e',
'CjkUnifiedIdeograph894d': '\xe8\xa5\x8d',
'CjkUnifiedIdeograph3dff': '\xe3\xb7\xbf',
'GeorgianCapitalLetterHoe': '\xe1\x83\x85',
'YiSyllableBbur': '\xea\x81\xb1',
'CjkRadicalBrushOne': '\xe2\xba\xba',
'CircledHangulTikeutA': '\xe3\x89\xb0',
'IdeographicTelegraphSymbolForHourFive': '\xe3\x8d\x9d',
'CjkUnifiedIdeograph5b7b': '\xe5\xad\xbb',
'CjkUnifiedIdeograph5b7c': '\xe5\xad\xbc',
'CjkUnifiedIdeograph5b7a': '\xe5\xad\xba',
'CjkUnifiedIdeograph5b7f': '\xe5\xad\xbf',
'CombiningHomotheticAbove': '\xcd\x8b',
'CjkUnifiedIdeograph5b7d': '\xe5\xad\xbd',
'CjkUnifiedIdeograph5b7e': '\xe5\xad\xbe',
'SyriacPthahaAbove': '\xdc\xb0',
'LatinSmallLetterEWithMacron': '\xc4\x93',
'CjkUnifiedIdeograph8943': '\xe8\xa5\x83',
'CjkUnifiedIdeograph8942': '\xe8\xa5\x82',
'CjkUnifiedIdeograph8941': '\xe8\xa5\x81',
'HexagramForPushingUpward': '\xe4\xb7\xad',
'CjkUnifiedIdeograph8947': '\xe8\xa5\x87',
'CjkUnifiedIdeograph8946': '\xe8\xa5\x86',
'CjkUnifiedIdeograph8945': '\xe8\xa5\x85',
'CjkUnifiedIdeograph8944': '\xe8\xa5\x84',
'HangulSyllableSwel': '\xec\x89\x98',
'HangulSyllableSwem': '\xec\x89\xa0',
'CjkUnifiedIdeograph8949': '\xe8\xa5\x89',
'CjkUnifiedIdeograph8948': '\xe8\xa5\x88',
'HangulSyllableSweh': '\xec\x89\xab',
'HangulSyllableTyaen': '\xed\x84\x98',
'ArabicLetterNoonWithSmallTah': '\xdd\xa8',
'HangulSyllableSwek': '\xec\x89\xa8',
'CjkUnifiedIdeograph51e3': '\xe5\x87\xa3',
'CjkUnifiedIdeograph51e2': '\xe5\x87\xa2',
'CjkUnifiedIdeograph51e1': '\xe5\x87\xa1',
'CjkUnifiedIdeograph51e0': '\xe5\x87\xa0',
'CjkUnifiedIdeograph51e7': '\xe5\x87\xa7',
'CjkUnifiedIdeograph51e6': '\xe5\x87\xa6',
'CanadianSyllabicsWestcreeMwi': '\xe1\x92\xaf',
'CjkUnifiedIdeograph51e4': '\xe5\x87\xa4',
'CanadianSyllabicsWestcreeMwe': '\xe1\x92\xad',
'CjkUnifiedIdeograph51e8': '\xe5\x87\xa8',
'CanadianSyllabicsWestcreeMwa': '\xe1\x92\xb7',
'SylotiNagriLetterRo': '\xea\xa0\x9e',
'YiSyllableBbux': '\xea\x81\xad',
'MalayalamLetterLla': '\xe0\xb4\xb3',
'HangulSyllableKeugg': '\xed\x81\xae',
'LatinSmallLetterEWithDotAbove': '\xc4\x97',
'YiRadicalJjut': '\xea\x92\xaa',
'CjkUnifiedIdeograph5079': '\xe5\x81\xb9',
'CjkUnifiedIdeograph933c': '\xe9\x8c\xbc',
'GreekSmallLetterArchaicKoppa': '\xcf\x99',
'CjkUnifiedIdeograph933a': '\xe9\x8c\xba',
'CjkUnifiedIdeograph75fa': '\xe7\x97\xba',
'CjkUnifiedIdeograph75fb': '\xe7\x97\xbb',
'CjkUnifiedIdeograph75fc': '\xe7\x97\xbc',
'CjkUnifiedIdeograph75fd': '\xe7\x97\xbd',
'CjkUnifiedIdeograph75fe': '\xe7\x97\xbe',
'CjkUnifiedIdeograph75ff': '\xe7\x97\xbf',
'HangulSyllableMyugs': '\xeb\xae\xa7',
'CjkUnifiedIdeograph6fbd': '\xe6\xbe\xbd',
'CjkUnifiedIdeograph6fbe': '\xe6\xbe\xbe',
'CjkUnifiedIdeograph6fbf': '\xe6\xbe\xbf',
'CombiningCandrabindu': '\xcc\x90',
'CjkUnifiedIdeograph8744': '\xe8\x9d\x84',
'CjkUnifiedIdeograph6fba': '\xe6\xbe\xba',
'CjkUnifiedIdeograph6fbb': '\xe6\xbe\xbb',
'CjkUnifiedIdeograph610d': '\xe6\x84\x8d',
'HangulSyllableRwilh': '\xeb\xa5\x8b',
'ExclamationQuestionMark': '\xe2\x81\x89',
'YiSyllableHmyrx': '\xea\x82\xac',
'BoxDrawingsLeftHeavyAndRightDownLight': '\xe2\x94\xad',
'UnicodeCharacter0xf5ba': '\xef\x96\xba',
'HangulSyllableRwilb': '\xeb\xa5\x87',
'CjkUnifiedIdeograph933d': '\xe9\x8c\xbd',
'HangulSyllableRwilg': '\xeb\xa5\x85',
'HangulSyllableMen': '\xeb\xa9\x98',
'OlChikiLetterOv': '\xe1\xb1\xb6',
'HangulSyllableMel': '\xeb\xa9\x9c',
'HangulSyllableMem': '\xeb\xa9\xa4',
'HangulSyllableMej': '\xeb\xa9\xaa',
'HangulSyllableMek': '\xeb\xa9\xac',
'ParenthesizedDigitOne': '\xe2\x91\xb4',
'UnicodeCharacter0xeff2': '\xee\xbf\xb2',
'HangulSyllableMeg': '\xeb\xa9\x95',
'CjkUnifiedIdeograph8747': '\xe8\x9d\x87',
'UnicodeCharacter0xf5bb': '\xef\x96\xbb',
'HangulSyllableMeb': '\xeb\xa9\xa5',
'CjkUnifiedIdeograph6b82': '\xe6\xae\x82',
'HangulSyllableJyoss': '\xec\xa3\xb4',
'===>': '\xe2\x87\x9b',
'OlChikiLetterOb': '\xe1\xb1\xb5',
'diffD': '\xe2\x85\x85',
'HangulSyllableJaess': '\xec\x9f\x80',
'ChamDigitSix': '\xea\xa9\x96',
'HangulSyllableMeu': '\xeb\xaf\x80',
'HangulSyllableMes': '\xeb\xa9\xa7',
'HangulSyllableNoe': '\xeb\x87\x8c',
'OlChikiLetterOh': '\xe1\xb1\xb7',
'KatakanaLetterSmallRe': '\xe3\x87\xbe',
'KatakanaLetterSmallRa': '\xe3\x87\xbb',
'division': '\xc3\xb7',
'CjkUnifiedIdeograph849d': '\xe8\x92\x9d',
'CjkUnifiedIdeograph9948': '\xe9\xa5\x88',
'KatakanaLetterSmallRo': '\xe3\x87\xbf',
'KatakanaLetterSmallRi': '\xe3\x87\xbc',
'CjkUnifiedIdeograph75f0': '\xe7\x97\xb0',
'KatakanaLetterSmallRu': '\xe3\x87\xbd',
'CjkUnifiedIdeograph75f2': '\xe7\x97\xb2',
'ArmenianCapitalLetterRa': '\xd5\x8c',
'CjkUnifiedIdeograph6fb8': '\xe6\xbe\xb8',
'CjkUnifiedIdeograph6fb9': '\xe6\xbe\xb9',
'CjkUnifiedIdeograph75f6': '\xe7\x97\xb6',
'CjkUnifiedIdeograph75f7': '\xe7\x97\xb7',
'CjkUnifiedIdeograph6fb4': '\xe6\xbe\xb4',
'CjkUnifiedIdeograph6fb5': '\xe6\xbe\xb5',
'CjkUnifiedIdeograph6fb6': '\xe6\xbe\xb6',
'CircledVerticalBar': '\xe2\xa6\xb6',
'CjkUnifiedIdeograph6fb0': '\xe6\xbe\xb0',
'CjkUnifiedIdeograph6fb1': '\xe6\xbe\xb1',
'CombiningBrevemacron': '\xe1\xb7\x8b',
'CjkUnifiedIdeograph6fb3': '\xe6\xbe\xb3',
'CjkUnifiedIdeograph7dd3': '\xe7\xb7\x93',
'CjkUnifiedIdeograph7dd2': '\xe7\xb7\x92',
'CjkUnifiedIdeograph7dd1': '\xe7\xb7\x91',
'SquareInti': '\xe3\x8c\x85',
'CjkUnifiedIdeograph7dd7': '\xe7\xb7\x97',
'CjkUnifiedIdeograph7dd6': '\xe7\xb7\x96',
'CjkUnifiedIdeograph7dd5': '\xe7\xb7\x95',
'CjkUnifiedIdeograph7dd4': '\xe7\xb7\x94',
'CjkUnifiedIdeograph4faf': '\xe4\xbe\xaf',
'CjkUnifiedIdeograph4fae': '\xe4\xbe\xae',
'CjkUnifiedIdeograph4fad': '\xe4\xbe\xad',
'CjkUnifiedIdeograph4fac': '\xe4\xbe\xac',
'CjkUnifiedIdeograph4fab': '\xe4\xbe\xab',
'CjkUnifiedIdeograph4faa': '\xe4\xbe\xaa',
'UnicodeCharacter0xf7b8': '\xef\x9e\xb8',
'CjkUnifiedIdeograph567f': '\xe5\x99\xbf',
'CjkUnifiedIdeograph5ce4': '\xe5\xb3\xa4',
'CjkUnifiedIdeograph567d': '\xe5\x99\xbd',
'CjkUnifiedIdeograph567e': '\xe5\x99\xbe',
'CjkUnifiedIdeograph567b': '\xe5\x99\xbb',
'CjkUnifiedIdeograph567c': '\xe5\x99\xbc',
'CjkUnifiedIdeograph5ce3': '\xe5\xb3\xa3',
'CjkUnifiedIdeograph567a': '\xe5\x99\xba',
'CjkUnifiedIdeograph5ce9': '\xe5\xb3\xa9',
'CjkUnifiedIdeograph5ce8': '\xe5\xb3\xa8',
'EthiopicSyllableVa': '\xe1\x89\xa8',
'GlagoliticCapitalLetterYeru': '\xe2\xb0\x9f',
'CanadianSyllabicsWestcreeCwaa': '\xe1\x92\x9f',
'EthiopicSyllableVe': '\xe1\x89\xad',
'HangulSyllableKaegs': '\xec\xba\x93',
'EthiopicSyllableVi': '\xe1\x89\xaa',
'CjkUnifiedIdeograph5ba3': '\xe5\xae\xa3',
'EthiopicSyllableVo': '\xe1\x89\xae',
'YiSyllableHmuop': '\xea\x82\x9e',
'EthiopicSyllableVu': '\xe1\x89\xa9',
'DiscontinuousUnderlineSymbol': '\xe2\x8e\x82',
'HangulSyllableKaegg': '\xec\xba\x92',
'GlagoliticCapitalLetterYeri': '\xe2\xb0\xa0',
'CjkUnifiedIdeograph5676': '\xe5\x99\xb6',
'CjkUnifiedIdeograph5677': '\xe5\x99\xb7',
'CjkUnifiedIdeograph5674': '\xe5\x99\xb4',
'CjkUnifiedIdeograph5675': '\xe5\x99\xb5',
'CjkUnifiedIdeograph5672': '\xe5\x99\xb2',
'CjkUnifiedIdeograph5673': '\xe5\x99\xb3',
'CjkUnifiedIdeograph5670': '\xe5\x99\xb0',
'CjkUnifiedIdeograph5671': '\xe5\x99\xb1',
'CjkUnifiedIdeograph7c58': '\xe7\xb1\x98',
'CjkUnifiedIdeograph5678': '\xe5\x99\xb8',
'CjkUnifiedIdeograph5679': '\xe5\x99\xb9',
'CjkUnifiedIdeograph7ddc': '\xe7\xb7\x9c',
'CjkUnifiedIdeograph7ddb': '\xe7\xb7\x9b',
'CjkUnifiedIdeograph7dda': '\xe7\xb7\x9a',
'CjkUnifiedIdeograph9705': '\xe9\x9c\x85',
'CjkUnifiedIdeograph9702': '\xe9\x9c\x82',
'CjkUnifiedIdeograph7ddf': '\xe7\xb7\x9f',
'CjkUnifiedIdeograph4fa9': '\xe4\xbe\xa9',
'CjkUnifiedIdeograph4fa8': '\xe4\xbe\xa8',
'CjkUnifiedIdeograph4fa7': '\xe4\xbe\xa7',
'CjkUnifiedIdeograph4fa6': '\xe4\xbe\xa6',
'CjkUnifiedIdeograph4fa5': '\xe4\xbe\xa5',
'CjkUnifiedIdeograph4fa4': '\xe4\xbe\xa4',
'CjkUnifiedIdeograph4fa3': '\xe4\xbe\xa3',
'CjkUnifiedIdeograph4fa2': '\xe4\xbe\xa2',
'GreekSmallLetterEpsilonWithVaria': '\xe1\xbd\xb2',
'CjkUnifiedIdeograph4fa0': '\xe4\xbe\xa0',
'UnicodeCharacter0xab7e': '\xea\xad\xbe',
'CjkUnifiedIdeograph7c54': '\xe7\xb1\x94',
'AcuteAngle': '\xe2\xa6\x9f',
'HangulSyllableByaebs': '\xeb\xb1\xba',
'HangulSyllablePwegs': '\xed\x92\xb3',
'CanadianSyllabicsNaskapiWoo': '\xe1\x90\x96',
'HangulSyllableDdwenj': '\xeb\x9b\x99',
'CrossOfJerusalem': '\xe2\x98\xa9',
'ArabicLetterReh': '\xd8\xb1',
'CjkUnifiedIdeograph353c': '\xe3\x94\xbc',
'CyrillicSmallLetterRha': '\xd4\x97',
'CjkUnifiedIdeograph353a': '\xe3\x94\xba',
'HangulSyllableNyuss': '\xeb\x8a\x88',
'CjkUnifiedIdeograph353f': '\xe3\x94\xbf',
'CjkUnifiedIdeograph353e': '\xe3\x94\xbe',
'CjkUnifiedIdeograph353d': '\xe3\x94\xbd',
'ArabicLetterBehInitialForm': '\xef\xba\x91',
'LatinSmallLetterNWithMiddleTilde': '\xe1\xb5\xb0',
'CjkUnifiedIdeograph9333': '\xe9\x8c\xb3',
'VaiSyllableNdoleKa': '\xea\x98\x91',
'CjkUnifiedIdeograph9332': '\xe9\x8c\xb2',
'HangulSyllableBwebs': '\xeb\xb7\x8a',
'CjkUnifiedIdeograph9331': '\xe9\x8c\xb1',
'GurmukhiVowelSignUu': '\xe0\xa9\x82',
'CjkUnifiedIdeograph6102': '\xe6\x84\x82',
'CjkUnifiedIdeograph9337': '\xe9\x8c\xb7',
'HangulSyllableGgweoss': '\xea\xbf\xa8',
'YiSyllableZzex': '\xea\x8b\x9f',
'CjkUnifiedIdeograph9336': '\xe9\x8c\xb6',
'UnicodeCharacter0xf533': '\xef\x94\xb3',
'HangulSyllableJjyelb': '\xec\xaa\xab',
'CjkUnifiedIdeograph9335': '\xe9\x8c\xb5',
'CjkUnifiedIdeograph8edf': '\xe8\xbb\x9f',
'UnicodeCharacter0xf169': '\xef\x85\xa9',
'CjkUnifiedIdeograph9334': '\xe9\x8c\xb4',
'GreekCapitalLetterHeta': '\xcd\xb0',
'ThaiCharacterNikhahit': '\xe0\xb9\x8d',
'HangulSyllableDweonj': '\xeb\x91\xb1',
'CjkUnifiedIdeograph9f34': '\xe9\xbc\xb4',
'HangulSyllableKyass': '\xec\xbb\x80',
'TibetanDigitFive': '\xe0\xbc\xa5',
'LatinSmallLetterAWithCaron': '\xc7\x8e',
'CjkUnifiedIdeograph3533': '\xe3\x94\xb3',
'CjkUnifiedIdeograph3532': '\xe3\x94\xb2',
'CjkUnifiedIdeograph3531': '\xe3\x94\xb1',
'CjkUnifiedIdeograph3530': '\xe3\x94\xb0',
'CjkUnifiedIdeograph3537': '\xe3\x94\xb7',
'CjkUnifiedIdeograph3536': '\xe3\x94\xb6',
'CjkUnifiedIdeograph3535': '\xe3\x94\xb5',
'CjkUnifiedIdeograph3534': '\xe3\x94\xb4',
'CjkUnifiedIdeograph3539': '\xe3\x94\xb9',
'CjkUnifiedIdeograph3538': '\xe3\x94\xb8',
'UnicodeCharacter0xe17d': '\xee\x85\xbd',
'CjkUnifiedIdeograph9f35': '\xe9\xbc\xb5',
'HangulSyllableJjanh': '\xec\xa7\xa2',
'HangulSyllableJjanj': '\xec\xa7\xa1',
'HangulSyllableCweong': '\xec\xb7\x85',
'HangulSyllableJjang': '\xec\xa7\xb1',
'HangulSyllableSanh': '\xec\x82\xb2',
'UnicodeCharacter0xab7c': '\xea\xad\xbc',
'Forces': '\xe2\x8a\xa9',
'GreekCapitalLetterEtaWithPsiliAndVariaAndProsgegrammeni': '\xe1\xbe\x9a',
'HangulSyllableHyic': '\xed\x9e\x83',
'ArabicLigatureFehWithJeemIsolatedForm': '\xef\xb0\xad',
'LatinLetterDentalClick': '\xc7\x80',
'ArabicLigatureKafWithYehIsolatedForm': '\xef\xb0\xbe',
'CanadianSyllabicsLoo': '\xe1\x93\x98',
'UnicodeCharacter0xee51': '\xee\xb9\x91',
'HangulSyllableRu': '\xeb\xa3\xa8',
'UnicodeCharacter0xee53': '\xee\xb9\x93',
'UnicodeCharacter0xee52': '\xee\xb9\x92',
'UnicodeCharacter0xee55': '\xee\xb9\x95',
'UnicodeCharacter0xee54': '\xee\xb9\x94',
'UnicodeCharacter0xee57': '\xee\xb9\x97',
'GreekSmallLetterEtaWithPsiliAndVaria': '\xe1\xbc\xa2',
'UnicodeCharacter0xee59': '\xee\xb9\x99',
'HangulSyllableTyeo': '\xed\x85\xa8',
'CombiningShortStrokeOverlay': '\xcc\xb5',
'CjkUnifiedIdeograph9f33': '\xe9\xbc\xb3',
'HangulSyllableJyilb': '\xec\xa6\xaf',
'HangulSyllableJelt': '\xec\xa0\xa9',
'HangulSyllableJwiss': '\xec\xa5\xa4',
'UnicodeCharacter0xaac9': '\xea\xab\x89',
'UnicodeCharacter0xaac8': '\xea\xab\x88',
'HangulSyllableCigg': '\xec\xb9\x9a',
'UnicodeCharacter0xaac7': '\xea\xab\x87',
'ThaiCharacterMaitaikhu': '\xe0\xb9\x87',
'UnicodeCharacter0xaac6': '\xea\xab\x86',
'HangulSyllableCigs': '\xec\xb9\x9b',
'UnicodeCharacter0xe51d': '\xee\x94\x9d',
'UnicodeCharacter0xaac5': '\xea\xab\x85',
'UnicodeCharacter0xee5a': '\xee\xb9\x9a',
'BuginesePallawa': '\xe1\xa8\x9e',
'UnicodeCharacter0xee5c': '\xee\xb9\x9c',
'UnicodeCharacter0xaac4': '\xea\xab\x84',
'UnicodeCharacter0xee5e': '\xee\xb9\x9e',
'UnicodeCharacter0xee5d': '\xee\xb9\x9d',
'UnicodeCharacter0xee5f': '\xee\xb9\x9f',
'UnicodeCharacter0xaac3': '\xea\xab\x83',
'UnicodeCharacter0xaac2': '\xea\xab\x82',
'UnicodeCharacter0xaac1': '\xea\xab\x81',
'CjkUnifiedIdeograph9f31': '\xe9\xbc\xb1',
'UnicodeCharacter0xaac0': '\xea\xab\x80',
'DevanagariVowelSignShortO': '\xe0\xa5\x8a',
'VariationSelector8': '\xef\xb8\x87',
'VariationSelector9': '\xef\xb8\x88',
'DevanagariVowelSignShortE': '\xe0\xa5\x86',
'VariationSelector7': '\xef\xb8\x86',
'VariationSelector4': '\xef\xb8\x83',
'VariationSelector5': '\xef\xb8\x84',
'VariationSelector2': '\xef\xb8\x81',
'VariationSelector3': '\xef\xb8\x82',
'CircledHangulPieup': '\xe3\x89\xa5',
'VariationSelector1': '\xef\xb8\x80',
'CjkUnifiedIdeograph6670': '\xe6\x99\xb0',
'CjkUnifiedIdeograph6676': '\xe6\x99\xb6',
'HangulSyllableTweolm': '\xed\x89\x92',
'HangulSyllableTweolh': '\xed\x89\x97',
'UnicodeCharacter0xd8a2': '\xed\xa2\xa2',
'LeftwardsArrowWithDottedStem': '\xe2\xac\xb8',
'HangulSyllableTweolg': '\xed\x89\x91',
'UnicodeCharacter0xeffa': '\xee\xbf\xba',
'HangulSyllableTweolb': '\xed\x89\x93',
'AngleWithUnderbar': '\xe2\xa6\xa4',
'UnicodeCharacter0x1311': '\xe1\x8c\x91',
'UnicodeCharacter0xd8a0': '\xed\xa2\xa0',
'UnicodeCharacter0x1317': '\xe1\x8c\x97',
'UnicodeCharacter0x1316': '\xe1\x8c\x96',
'CjkUnifiedIdeograph8a89': '\xe8\xaa\x89',
'TagbanwaLetterKa': '\xe1\x9d\xa3',
'CjkUnifiedIdeograph8a87': '\xe8\xaa\x87',
'CjkUnifiedIdeograph8a86': '\xe8\xaa\x86',
'CjkUnifiedIdeograph8a85': '\xe8\xaa\x85',
'CjkUnifiedIdeograph8a84': '\xe8\xaa\x84',
'CjkUnifiedIdeograph8a83': '\xe8\xaa\x83',
'CjkUnifiedIdeograph8a82': '\xe8\xaa\x82',
'GreaterthanAboveLessthanAboveDoublelineEqual': '\xe2\xaa\x92',
'CjkUnifiedIdeograph8a80': '\xe8\xaa\x80',
'HangulSyllableBbyibs': '\xec\x82\x86',
'UnicodeCharacter0xeffe': '\xee\xbf\xbe',
'CjkUnifiedIdeograph5ce6': '\xe5\xb3\xa6',
'UnicodeCharacter0xd8a4': '\xed\xa2\xa4',
'UnicodeCharacter0xf5af': '\xef\x96\xaf',
'LatinCapitalLetterOWithCircumflexAndDotBelow': '\xe1\xbb\x98',
'ArmenianSmallLetterCheh': '\xd5\xb3',
'CjkUnifiedIdeograph8a8f': '\xe8\xaa\x8f',
'CjkUnifiedIdeograph8a8e': '\xe8\xaa\x8e',
'CjkUnifiedIdeograph8a8d': '\xe8\xaa\x8d',
'CjkUnifiedIdeograph8a8c': '\xe8\xaa\x8c',
'CjkUnifiedIdeograph8a8b': '\xe8\xaa\x8b',
'CjkUnifiedIdeograph8a8a': '\xe8\xaa\x8a',
'LongRightwardsDoubleArrow': '\xe2\x9f\xb9',
'[3]': '\xe2\x82\x83',
'CjkRadicalWalkOne': '\xe2\xbb\x8d',
'CombiningLongStrokeOverlay': '\xcc\xb6',
'HangulSyllableSswelb': '\xec\x92\xa7',
'SoftHyphen': '\xc2\xad',
'HangulSyllableBoelp': '\xeb\xb5\x96',
'BugineseLetterMpa': '\xe1\xa8\x87',
'YiSyllableKie': '\xea\x87\xbd',
'YiSyllableSsie': '\xea\x8c\xaf',
'BallotX': '\xe2\x9c\x97',
'YiSyllableKit': '\xea\x87\xb8',
'ArabicLigatureTehWithRehFinalForm': '\xef\xb1\xb0',
'YiSyllableKip': '\xea\x87\xbb',
'HangulLetterSsangieung': '\xe3\x86\x80',
'UnicodeCharacter0xd7be': '\xed\x9e\xbe',
'YiSyllableSsit': '\xea\x8c\xaa',
'YiSyllableKix': '\xea\x87\xb9',
'ParenthesizedIdeographSociety': '\xe3\x88\xb3',
'HangulSyllableGweolp': '\xea\xb6\x96',
'ArabicLigatureNoonWithJeemWithMeemFinalForm': '\xef\xb6\x97',
'HangulSyllableSswelp': '\xec\x92\xaa',
'CyrillicSmallLetterDzhe': '\xd1\x9f',
'CjkUnifiedIdeograph77ca': '\xe7\x9f\x8a',
'HangulSyllableTass': '\xed\x83\x94',
'CjkUnifiedIdeograph4b34': '\xe4\xac\xb4',
'HangulSyllableGgyess': '\xea\xbc\xa4',
'CjkUnifiedIdeograph77cc': '\xe7\x9f\x8c',
'CjkUnifiedIdeograph4d0f': '\xe4\xb4\x8f',
'CjkUnifiedIdeograph77cb': '\xe7\x9f\x8b',
'CjkUnifiedIdeograph882d': '\xe8\xa0\xad',
'CjkUnifiedIdeograph77ce': '\xe7\x9f\x8e',
'GreekCapitalLetterEtaWithPsiliAndPerispomeniAndProsgegrammeni': '\xe1\xbe\x9e',
'UnicodeCharacter0xfbbc': '\xef\xae\xbc',
'UnicodeCharacter0xaace': '\xea\xab\x8e',
'CombiningZigzagAbove': '\xcd\x9b',
'UnicodeCharacter0xaacd': '\xea\xab\x8d',
'ArabicLigatureYehWithHamzaAboveWithWawFinalForm': '\xef\xaf\xaf',
'HangulSyllableSaelp': '\xec\x83\x96',
'UnicodeCharacter0xaacc': '\xea\xab\x8c',
'CjkUnifiedIdeograph77cf': '\xe7\x9f\x8f',
'LatinCapitalLetterDWithLineBelow': '\xe1\xb8\x8e',
'HangulSyllableJeoc': '\xec\xa0\x97',
'UnicodeCharacter0xdff1': '\xed\xbf\xb1',
'UnicodeCharacter0xdff2': '\xed\xbf\xb2',
'UnicodeCharacter0xdff3': '\xed\xbf\xb3',
'UnicodeCharacter0xdff4': '\xed\xbf\xb4',
'HangulSyllableBwed': '\xeb\xb6\xbf',
'CjkUnifiedIdeograph882e': '\xe8\xa0\xae',
'UnicodeCharacter0xdff7': '\xed\xbf\xb7',
'UnicodeCharacter0xdff8': '\xed\xbf\xb8',
'UnicodeCharacter0xdff9': '\xed\xbf\xb9',
'HangulSyllableRwaj': '\xeb\xa2\x8e',
'RejangLetterBa': '\xea\xa4\xb7',
'CyrillicSmallLetterYeruWithBackYer': '\xea\x99\x91',
'UnicodeCharacter0xed81': '\xee\xb6\x81',
'HangulSyllableGweolb': '\xea\xb6\x93',
'UnicodeCharacter0xe17a': '\xee\x85\xba',
'UnicodeCharacter0xdffa': '\xed\xbf\xba',
'UnicodeCharacter0xdffb': '\xed\xbf\xbb',
'HangulSyllableBoe': '\xeb\xb5\x88',
'UnicodeCharacter0xdffd': '\xed\xbf\xbd',
'UnicodeCharacter0xdffe': '\xed\xbf\xbe',
'UnicodeCharacter0xdfff': '\xed\xbf\xbf',
'HangulSyllablePyip': '\xed\x94\xba',
'ContainsWithVerticalBarAtEndOfHorizontalStroke': '\xe2\x8b\xbb',
'KangxiRadicalShoot': '\xe2\xbc\xb7',
'CjkUnifiedIdeograph95b0': '\xe9\x96\xb0',
'UnicodeCharacter0xf16d': '\xef\x85\xad',
'VaiSyllableFoo': '\xea\x95\xbd',
'HangulSyllableBulm': '\xeb\xb6\x8a',
'HangulSyllableYulg': '\xec\x9c\xa9',
'CombiningGraveAccentBelow': '\xcc\x96',
'HangulSyllableYulb': '\xec\x9c\xab',
'HangulSyllableGeot': '\xea\xb2\x89',
'HangulSyllableYulm': '\xec\x9c\xaa',
'HangulSyllableYulh': '\xec\x9c\xaf',
'HangulSyllableYult': '\xec\x9c\xad',
'HangulSyllableYuls': '\xec\x9c\xac',
'HangulSyllableYulp': '\xec\x9c\xae',
'Pisces': '\xe2\x99\x93',
'HangulSyllableJyalh': '\xec\x9f\x97',
'ThaanaObofili': '\xde\xae',
'HangulSyllableJyalm': '\xec\x9f\x92',
'HangulSyllableJyalb': '\xec\x9f\x93',
'EthiopicSyllableQhwaa': '\xe1\x89\x9b',
'HangulSyllableJyalg': '\xec\x9f\x91',
'UnicodeCharacter0xe3be': '\xee\x8e\xbe',
'HangulSyllableGwabs': '\xea\xb4\x8e',
'HangulSyllableJyals': '\xec\x9f\x94',
'HangulSyllableJyalp': '\xec\x9f\x96',
'HangulSyllableJyalt': '\xec\x9f\x95',
'CjkUnifiedIdeograph6cb5': '\xe6\xb2\xb5',
'CjkUnifiedIdeograph6cb4': '\xe6\xb2\xb4',
'CjkUnifiedIdeograph6cb7': '\xe6\xb2\xb7',
'CjkUnifiedIdeograph6cb6': '\xe6\xb2\xb6',
'CanadianSyllabicsSayisiM': '\xe1\x92\xbf',
'CjkUnifiedIdeograph6cb0': '\xe6\xb2\xb0',
'CjkUnifiedIdeograph6cb3': '\xe6\xb2\xb3',
'EthiopicSyllableMu': '\xe1\x88\x99',
'HangzhouNumeralEight': '\xe3\x80\xa8',
'HangulSyllableMoelt': '\xeb\xac\x89',
'CjkUnifiedIdeograph6cb9': '\xe6\xb2\xb9',
'CjkUnifiedIdeograph6cb8': '\xe6\xb2\xb8',
'HangulSyllableMoelp': '\xeb\xac\x8a',
'CjkUnifiedIdeograph6239': '\xe6\x88\xb9',
'EthiopicSyllableMa': '\xe1\x88\x98',
'ThaiCharacterThoNangmontho': '\xe0\xb8\x91',
'CjkUnifiedIdeograph6233': '\xe6\x88\xb3',
'CjkUnifiedIdeograph6232': '\xe6\x88\xb2',
'CjkUnifiedIdeograph6231': '\xe6\x88\xb1',
'EthiopicSyllableMi': '\xe1\x88\x9a',
'CjkUnifiedIdeograph6237': '\xe6\x88\xb7',
'EthiopicSyllableMo': '\xe1\x88\x9e',
'CjkUnifiedIdeograph6235': '\xe6\x88\xb5',
'CjkUnifiedIdeograph6234': '\xe6\x88\xb4',
'HangulSyllableGoeng': '\xea\xb5\x89',
'HangulSyllablePwel': '\xed\x92\xb8',
'DentistrySymbolLightVerticalWithCircle': '\xe2\x8f\x80',
'KangxiRadicalRoof': '\xe2\xbc\xa7',
'HangulSyllableBwen': '\xeb\xb6\xbc',
'CanadianSyllabicsKwoo': '\xe1\x91\xbc',
'BalinesePamada': '\xe1\xad\x9b',
'HangulSyllableGoenh': '\xea\xb4\xba',
'UnicodeCharacter0xeb9b': '\xee\xae\x9b',
'CombiningGreekKoronis': '\xcd\x83',
'UnicodeCharacter0xeb9c': '\xee\xae\x9c',
'HangulSyllableJyilp': '\xec\xa6\xb2',
'RightLowParaphraseBracket': '\xe2\xb8\x9d',
'UnicodeCharacter0xeb9a': '\xee\xae\x9a',
'LatinSmallLetterEgyptologicalAlef': '\xea\x9c\xa3',
'CjkUnifiedIdeograph623c': '\xe6\x88\xbc',
'BalineseVowelSignTaling': '\xe1\xac\xbe',
'CjkUnifiedIdeograph623a': '\xe6\x88\xba',
'OrthodoxCross': '\xe2\x98\xa6',
'CjkUnifiedIdeograph623f': '\xe6\x88\xbf',
'CjkUnifiedIdeograph623e': '\xe6\x88\xbe',
'CjkUnifiedIdeograph623d': '\xe6\x88\xbd',
'CjkUnifiedIdeograph6cbe': '\xe6\xb2\xbe',
'CjkUnifiedIdeograph6cbd': '\xe6\xb2\xbd',
'TibetanSignNyiZlaNaaDa': '\xe0\xbe\x82',
'CjkUnifiedIdeograph6cbf': '\xe6\xb2\xbf',
'KangxiRadicalOneself': '\xe2\xbc\xb0',
'UnicodeCharacter0xeb9d': '\xee\xae\x9d',
'CjkUnifiedIdeograph6cbc': '\xe6\xb2\xbc',
'CjkUnifiedIdeograph6cbb': '\xe6\xb2\xbb',
'UnicodeCharacter0xeb9e': '\xee\xae\x9e',
'CjkUnifiedIdeograph4b3e': '\xe4\xac\xbe',
'HangulSyllableGgwilp': '\xeb\x80\x9a',
'HangulSyllableGgwils': '\xeb\x80\x98',
'CjkUnifiedIdeograph77c0': '\xe7\x9f\x80',
'HangulSyllableSwaeng': '\xec\x87\x99',
'HangulSyllableGgwilg': '\xeb\x80\x95',
'CyrillicCapitalLetterPeWithMiddleHook': '\xd2\xa6',
'HangulSyllableGgwilb': '\xeb\x80\x97',
'HangulSyllableGgwilm': '\xeb\x80\x96',
'CjkUnifiedIdeograph77c5': '\xe7\x9f\x85',
'HangulSyllableGgwilh': '\xeb\x80\x9b',
'CjkUnifiedIdeograph77c7': '\xe7\x9f\x87',
'CjkUnifiedIdeograph77c6': '\xe7\x9f\x86',
'HangulSyllableTilp': '\xed\x8b\xbe',
'UnicodeCharacter0xef1f': '\xee\xbc\x9f',
'UnicodeCharacter0xef1d': '\xee\xbc\x9d',
'UnicodeCharacter0xef1e': '\xee\xbc\x9e',
'UnicodeCharacter0xef1b': '\xee\xbc\x9b',
'UnicodeCharacter0xef1c': '\xee\xbc\x9c',
'IntersectionWithLogicalAnd': '\xe2\xa9\x84',
'HangulSyllableKyuk': '\xed\x81\xa8',
'ChamVowelSignAu': '\xea\xa8\xb1',
'HangulSyllableDweoh': '\xeb\x92\x87',
'Nabla': '\xe2\x88\x87',
'CjkUnifiedIdeograph8937': '\xe8\xa4\xb7',
'ModuloTwoSum': '\xe2\xa8\x8a',
'ChamVowelSignAa': '\xea\xa8\xa9',
'LatinSmallLetterTWithPalatalHook': '\xc6\xab',
'MongolianFreeVariationSelectorTwo': '\xe1\xa0\x8c',
'ChamVowelSignAi': '\xea\xa8\xb0',
'HangulSyllableKyud': '\xed\x81\x97',
'UnicodeCharacter0xe792': '\xee\x9e\x92',
'UnicodeCharacter0xef16': '\xee\xbc\x96',
'UnicodeCharacter0xef17': '\xee\xbc\x97',
'UnicodeCharacter0xef14': '\xee\xbc\x94',
'UnicodeCharacter0xef15': '\xee\xbc\x95',
'UnicodeCharacter0xef12': '\xee\xbc\x92',
'HangulSyllableHyiss': '\xed\x9e\x80',
'UnicodeCharacter0xef10': '\xee\xbc\x90',
'UnicodeCharacter0xef11': '\xee\xbc\x91',
'CjkUnifiedIdeograph8931': '\xe8\xa4\xb1',
'SaurashtraDigitNine': '\xea\xa3\x99',
'HangulSyllableKyuc': '\xed\x81\xa7',
'CjkUnifiedIdeograph758d': '\xe7\x96\x8d',
'HangulSyllableBwep': '\xeb\xb7\x92',
'HangulSyllableYonj': '\xec\x9a\x99',
'HangulSyllableJeuss': '\xec\xa6\x9c',
'HangulSyllableTyulm': '\xed\x8a\xa6',
'LaoToneMaiTi': '\xe0\xbb\x8a',
'ChamLetterYa': '\xea\xa8\xa2',
'CjkUnifiedIdeograph8938': '\xe8\xa4\xb8',
'CjkUnifiedIdeograph64da': '\xe6\x93\x9a',
'CjkUnifiedIdeograph64db': '\xe6\x93\x9b',
'CjkUnifiedIdeograph64dc': '\xe6\x93\x9c',
'CjkUnifiedIdeograph64dd': '\xe6\x93\x9d',
'CjkUnifiedIdeograph64de': '\xe6\x93\x9e',
'ArabicLetterAinWithTwoDotsAbove': '\xdd\x9d',
'UnicodeCharacter0xf30c': '\xef\x8c\x8c',
'HangulSyllableTwaelb': '\xed\x87\xa3',
'HangulSyllableGyolg': '\xea\xb5\x99',
'GurmukhiDigitThree': '\xe0\xa9\xa9',
'HangulSyllableGyolb': '\xea\xb5\x9b',
'HangulSyllableGyolm': '\xea\xb5\x9a',
'HangulSyllableKyeok': '\xec\xbc\xb4',
'UnicodeCharacter0xa6db': '\xea\x9b\x9b',
'KatakanaLetterSmallSi': '\xe3\x87\xb1',
'HangulSyllableTwaelm': '\xed\x87\xa2',
'HangulSyllableGyolt': '\xea\xb5\x9d',
'HangulSyllableTwaelp': '\xed\x87\xa6',
'CjkUnifiedIdeograph51b5': '\xe5\x86\xb5',
'HangulSyllableGyolp': '\xea\xb5\x9e',
'HangulSyllableGyols': '\xea\xb5\x9c',
'HangulSyllableDweot': '\xeb\x92\x85',
'HangulSyllableYit': '\xec\x9d\xb1',
'HangulJongseongRieulssangsios': '\xe1\x87\x96',
'ArabicLetterSeenWithSmallArabicLetterTahAndTwoDots': '\xdd\xb0',
'CjkUnifiedIdeograph64d9': '\xe6\x93\x99',
'UnicodeCharacter0x1b4c': '\xe1\xad\x8c',
'UnicodeCharacter0x1b4e': '\xe1\xad\x8e',
'UnicodeCharacter0x1b4d': '\xe1\xad\x8d',
'UnicodeCharacter0x1b4f': '\xe1\xad\x8f',
'CjkUnifiedIdeograph64d0': '\xe6\x93\x90',
'CjkUnifiedIdeograph64d1': '\xe6\x93\x91',
'CjkUnifiedIdeograph64d2': '\xe6\x93\x92',
'CjkUnifiedIdeograph64d3': '\xe6\x93\x93',
'CjkUnifiedIdeograph64d4': '\xe6\x93\x94',
'CjkUnifiedIdeograph64d5': '\xe6\x93\x95',
'CjkUnifiedIdeograph64d6': '\xe6\x93\x96',
'UnicodeCharacter0xabb': '\xe0\xaa\xbb',
'UnicodeCharacter0xeb99': '\xee\xae\x99',
'HangulSyllableTyulb': '\xed\x8a\xa7',
'notelem': '\xe2\x88\x89',
'UnicodeCharacter0xe919': '\xee\xa4\x99',
'BengaliLetterLa': '\xe0\xa6\xb2',
'UnicodeCharacter0xeb92': '\xee\xae\x92',
'UnicodeCharacter0xe440': '\xee\x91\x80',
'CjkUnifiedIdeograph9ee2': '\xe9\xbb\xa2',
'UnicodeCharacter0xeb90': '\xee\xae\x90',
'NkoLetterE': '\xdf\x8d',
'YiSyllableFax': '\xea\x83\x92',
'NkoLetterA': '\xdf\x8a',
'NkoLetterN': '\xdf\x92',
'NkoLetterO': '\xdf\x90',
'NkoLetterI': '\xdf\x8c',
'GreekCapitalLetterOmega': '\xce\xa9',
'NkoLetterU': '\xdf\x8e',
'UnicodeCharacter0xeb94': '\xee\xae\x94',
'HangulSyllableWaeh': '\xec\x99\xb7',
'UnicodeCharacter0xeb95': '\xee\xae\x95',
'HangulSyllableTilh': '\xed\x8b\xbf',
'KangxiRadicalRice': '\xe2\xbd\xb6',
'HangulSyllableTyulp': '\xed\x8a\xaa',
'UnicodeCharacter0xf357': '\xef\x8d\x97',
'LepchaDigitSix': '\xe1\xb1\x86',
'CjkUnifiedIdeograph6a08': '\xe6\xa8\x88',
'CjkUnifiedIdeograph86d6': '\xe8\x9b\x96',
'CjkUnifiedIdeograph86d7': '\xe8\x9b\x97',
'CjkUnifiedIdeograph86d0': '\xe8\x9b\x90',
'CjkUnifiedIdeograph86d1': '\xe8\x9b\x91',
'CjkUnifiedIdeograph86d2': '\xe8\x9b\x92',
'DivisionSlash': '\xe2\x88\x95',
'CjkUnifiedIdeograph6a01': '\xe6\xa8\x81',
'CjkUnifiedIdeograph6a00': '\xe6\xa8\x80',
'CjkUnifiedIdeograph6a03': '\xe6\xa8\x83',
'CjkUnifiedIdeograph6a02': '\xe6\xa8\x82',
'CjkUnifiedIdeograph6a05': '\xe6\xa8\x85',
'CjkUnifiedIdeograph6a04': '\xe6\xa8\x84',
'CjkUnifiedIdeograph6a07': '\xe6\xa8\x87',
'CjkUnifiedIdeograph6a06': '\xe6\xa8\x86',
'HangulSyllableKyeol': '\xec\xbc\xa4',
'HangulSyllableBot': '\xeb\xb4\x8d',
'SquareGhz': '\xe3\x8e\x93',
'HangulSyllableBbi': '\xec\x82\x90',
'CjkUnifiedIdeograph63e2': '\xe6\x8f\xa2',
'HangulSyllableBbe': '\xeb\xbb\xac',
'ThaiDigitSix': '\xe0\xb9\x96',
'HangulSyllableBba': '\xeb\xb9\xa0',
'CyrillicCapitalLetterKoppa': '\xd2\x80',
'LatinSmallLetterUpsilonWithStroke': '\xe1\xb5\xbf',
'HangulSyllableBbu': '\xeb\xbf\x8c',
'HangulSyllableKyeom': '\xec\xbc\xac',
'HangulSyllableBos': '\xeb\xb4\x87',
'CjkUnifiedIdeograph86dd': '\xe8\x9b\x9d',
'CjkUnifiedIdeograph86de': '\xe8\x9b\x9e',
'CjkUnifiedIdeograph86df': '\xe8\x9b\x9f',
'CjkUnifiedIdeograph86da': '\xe8\x9b\x9a',
'CjkUnifiedIdeograph86db': '\xe8\x9b\x9b',
'CjkUnifiedIdeograph86dc': '\xe8\x9b\x9c',
'CjkUnifiedIdeograph6a0a': '\xe6\xa8\x8a',
'CjkUnifiedIdeograph6a0c': '\xe6\xa8\x8c',
'CjkUnifiedIdeograph6a0b': '\xe6\xa8\x8b',
'CjkUnifiedIdeograph6a0e': '\xe6\xa8\x8e',
'CjkUnifiedIdeograph6a0d': '\xe6\xa8\x8d',
'CjkUnifiedIdeograph6a0f': '\xe6\xa8\x8f',
'HangulSyllablePyis': '\xed\x94\xb3',
'HangulSyllableJjwigs': '\xec\xae\x9f',
'BraillePatternDots1234678': '\xe2\xa3\xaf',
'CircledKatakanaSo': '\xe3\x8b\x9e',
'CircledKatakanaSi': '\xe3\x8b\x9b',
'CanadianSyllabicsSayisiYi': '\xe1\x95\x81',
'CircledKatakanaSe': '\xe3\x8b\x9d',
'KayahLiLetterDa': '\xea\xa4\x98',
'CircledKatakanaSa': '\xe3\x8b\x9a',
'ExtendedArabicindicDigitZero': '\xdb\xb0',
'HangulSyllableKyeoc': '\xec\xbc\xb3',
'CircledKatakanaSu': '\xe3\x8b\x9c',
'[5]': '\xe2\x82\x85',
'NewTaiLueDigitEight': '\xe1\xa7\x98',
'ArabicLigatureLamWithAlefIsolatedForm': '\xef\xbb\xbb',
'HangulSyllableNalg': '\xeb\x82\xa1',
'UnicodeCharacter0xd969': '\xed\xa5\xa9',
'CjkUnifiedIdeograph83f3': '\xe8\x8f\xb3',
'ArabicLigatureLamWithAlefWithHamzaAboveFinalForm': '\xef\xbb\xb8',
'UnicodeCharacter0xd963': '\xed\xa5\xa3',
'HangulSyllableBop': '\xeb\xb4\x8e',
'UnicodeCharacter0xd961': '\xed\xa5\xa1',
'UnicodeCharacter0xd960': '\xed\xa5\xa0',
'UnicodeCharacter0xd967': '\xed\xa5\xa7',
'CjkUnifiedIdeograph7768': '\xe7\x9d\xa8',
'UnicodeCharacter0xd965': '\xed\xa5\xa5',
'UnicodeCharacter0xd964': '\xed\xa5\xa4',
'ArabicLetterBehMedialForm': '\xef\xba\x92',
'UnicodeCharacter0xd8ee': '\xed\xa3\xae',
'LeftAndRightDoubleTurnstile': '\xe2\x9f\x9a',
'HangulSyllableEulm': '\xec\x9d\x86',
'HangulSyllableTom': '\xed\x86\xb0',
'HangulSyllableTol': '\xed\x86\xa8',
'HangulSyllableTon': '\xed\x86\xa4',
'HangulSyllableToh': '\xed\x86\xbb',
'HangulSyllableTok': '\xed\x86\xb8',
'HangulSyllableToj': '\xed\x86\xb6',
'HangulSyllableToe': '\xed\x87\xb4',
'HangulSyllableTod': '\xed\x86\xa7',
'HangulSyllableTog': '\xed\x86\xa1',
'UnicodeCharacter0xd96f': '\xed\xa5\xaf',
'HangulSyllableToc': '\xed\x86\xb7',
'HangulSyllableTob': '\xed\x86\xb1',
'HangulSyllableTilg': '\xed\x8b\xb9',
'YiSyllableHlex': '\xea\x86\xa8',
'\\/': '\xe2\x88\xa8',
'HangulSyllableTot': '\xed\x86\xb9',
'YiSyllableHlep': '\xea\x86\xaa',
'HangulSyllableTop': '\xed\x86\xba',
'HangulSyllableTos': '\xed\x86\xb3',
'HangulSyllableDdweolb': '\xeb\x9b\x83',
'CjkUnifiedIdeograph52ba': '\xe5\x8a\xba',
'UnicodeCharacter0xe36c': '\xee\x8d\xac',
'CjkUnifiedIdeograph52bc': '\xe5\x8a\xbc',
'CjkUnifiedIdeograph52bb': '\xe5\x8a\xbb',
'CjkUnifiedIdeograph52be': '\xe5\x8a\xbe',
'CjkUnifiedIdeograph52bd': '\xe5\x8a\xbd',
'CanadianSyllabicsThe': '\xe1\x95\x9e',
'CjkUnifiedIdeograph52bf': '\xe5\x8a\xbf',
'CanadianSyllabicsThi': '\xe1\x95\xa0',
'CanadianSyllabicsTho': '\xe1\x95\xa4',
'HangulSyllableTwels': '\xed\x89\xb0',
'HangulSyllableTwelp': '\xed\x89\xb2',
'UnicodeCharacter0xa87b': '\xea\xa1\xbb',
'UnicodeCharacter0xa87c': '\xea\xa1\xbc',
'UnicodeCharacter0xa87a': '\xea\xa1\xba',
'UnicodeCharacter0xa87f': '\xea\xa1\xbf',
'UnicodeCharacter0xa87d': '\xea\xa1\xbd',
'UnicodeCharacter0xa87e': '\xea\xa1\xbe',
'HangulSyllableTwelg': '\xed\x89\xad',
'HangulSyllableTwelb': '\xed\x89\xaf',
'YiSyllableNzop': '\xea\x8b\xbb',
'CjkUnifiedIdeograph5e93': '\xe5\xba\x93',
'CjkUnifiedIdeograph5e92': '\xe5\xba\x92',
'CjkUnifiedIdeograph5e91': '\xe5\xba\x91',
'CjkUnifiedIdeograph5e90': '\xe5\xba\x90',
'CjkUnifiedIdeograph5e97': '\xe5\xba\x97',
'CjkUnifiedIdeograph5e96': '\xe5\xba\x96',
'CjkUnifiedIdeograph5e95': '\xe5\xba\x95',
'CjkUnifiedIdeograph5e94': '\xe5\xba\x94',
'UnicodeCharacter0xdebc': '\xed\xba\xbc',
'CanadianSyllabicsLwaa': '\xe1\x93\xa8',
'CjkUnifiedIdeograph5e99': '\xe5\xba\x99',
'CjkUnifiedIdeograph5e98': '\xe5\xba\x98',
'UnicodeCharacter0xdebf': '\xed\xba\xbf',
'UnicodeCharacter0xdebe': '\xed\xba\xbe',
'UnicodeCharacter0xdebd': '\xed\xba\xbd',
'HangulSyllableKyeod': '\xec\xbc\xa3',
'HangulSyllableJjyulb': '\xec\xaf\x83',
'HangulSyllableJjyulg': '\xec\xaf\x81',
'CjkUnifiedIdeograph41fa': '\xe4\x87\xba',
'CjkUnifiedIdeograph4be6': '\xe4\xaf\xa6',
'CjkUnifiedIdeograph41fc': '\xe4\x87\xbc',
'CjkUnifiedIdeograph41fb': '\xe4\x87\xbb',
'CjkUnifiedIdeograph41fe': '\xe4\x87\xbe',
'CjkUnifiedIdeograph41fd': '\xe4\x87\xbd',
'CjkUnifiedIdeograph41ff': '\xe4\x87\xbf',
'CjkUnifiedIdeograph9a1c': '\xe9\xa8\x9c',
'UnicodeCharacter0xd8ea': '\xed\xa3\xaa',
'CjkUnifiedIdeograph9a1b': '\xe9\xa8\x9b',
'SquareGuramuton': '\xe3\x8c\x99',
'ArabicLigatureSeenWithJeemMedialForm': '\xef\xb4\xb4',
'UnicodeCharacter0xa878': '\xea\xa1\xb8',
'UnicodeCharacter0xa879': '\xea\xa1\xb9',
'CjkUnifiedIdeograph5aae': '\xe5\xaa\xae',
'SaurashtraLetterLa': '\xea\xa2\xad',
'CjkUnifiedIdeograph52b1': '\xe5\x8a\xb1',
'CjkUnifiedIdeograph52b0': '\xe5\x8a\xb0',
'CjkUnifiedIdeograph52b3': '\xe5\x8a\xb3',
'CjkUnifiedIdeograph52b2': '\xe5\x8a\xb2',
'CjkUnifiedIdeograph52b5': '\xe5\x8a\xb5',
'CjkUnifiedIdeograph52b4': '\xe5\x8a\xb4',
'CjkUnifiedIdeograph52b7': '\xe5\x8a\xb7',
'CjkUnifiedIdeograph52b6': '\xe5\x8a\xb6',
'CjkUnifiedIdeograph52b9': '\xe5\x8a\xb9',
'CjkUnifiedIdeograph52b8': '\xe5\x8a\xb8',
'UnicodeCharacter0xe368': '\xee\x8d\xa8',
'UnicodeCharacter0xe369': '\xee\x8d\xa9',
'TibetanMarkGugRtagsGyas': '\xe0\xbc\xbb',
'CjkUnifiedIdeograph41f9': '\xe4\x87\xb9',
'CjkUnifiedIdeograph41f8': '\xe4\x87\xb8',
'inter': '\xe2\x88\xa9',
'CjkUnifiedIdeograph41f1': '\xe4\x87\xb1',
'CjkUnifiedIdeograph41f0': '\xe4\x87\xb0',
'CjkUnifiedIdeograph41f3': '\xe4\x87\xb3',
'CjkUnifiedIdeograph41f2': '\xe4\x87\xb2',
'CjkUnifiedIdeograph41f5': '\xe4\x87\xb5',
'CjkUnifiedIdeograph41f4': '\xe4\x87\xb4',
'CjkUnifiedIdeograph41f7': '\xe4\x87\xb7',
'CjkUnifiedIdeograph41f6': '\xe4\x87\xb6',
'CjkUnifiedIdeograph5e9c': '\xe5\xba\x9c',
'CjkUnifiedIdeograph5e9b': '\xe5\xba\x9b',
'CjkUnifiedIdeograph5e9a': '\xe5\xba\x9a',
'UnicodeCharacter0xdeb8': '\xed\xba\xb8',
'Precedes': '\xe2\x89\xba',
'CjkUnifiedIdeograph5e9e': '\xe5\xba\x9e',
'CjkUnifiedIdeograph39a0': '\xe3\xa6\xa0',
'UnicodeCharacter0xdeb3': '\xed\xba\xb3',
'UnicodeCharacter0xdeb2': '\xed\xba\xb2',
'UnicodeCharacter0xdeb1': '\xed\xba\xb1',
'UnicodeCharacter0xdeb0': '\xed\xba\xb0',
'HangulSyllableSsyobs': '\xec\x91\x9a',
'UnicodeCharacter0xdeb6': '\xed\xba\xb6',
'UnicodeCharacter0xdeb5': '\xed\xba\xb5',
'UnicodeCharacter0xdeb4': '\xed\xba\xb4',
'CjkUnifiedIdeograph875b': '\xe8\x9d\x9b',
'CjkUnifiedIdeograph558a': '\xe5\x96\x8a',
'CjkUnifiedIdeograph558b': '\xe5\x96\x8b',
'CjkUnifiedIdeograph558c': '\xe5\x96\x8c',
'CjkUnifiedIdeograph558d': '\xe5\x96\x8d',
'CjkUnifiedIdeograph558e': '\xe5\x96\x8e',
'CjkUnifiedIdeograph558f': '\xe5\x96\x8f',
'CjkUnifiedIdeograph875e': '\xe8\x9d\x9e',
'HangulSyllableGyaeng': '\xea\xb1\xa9',
'CjkUnifiedIdeograph4138': '\xe4\x84\xb8',
'CjkUnifiedIdeograph5856': '\xe5\xa1\x96',
'CjkUnifiedIdeograph5857': '\xe5\xa1\x97',
'CjkUnifiedIdeograph5854': '\xe5\xa1\x94',
'CjkUnifiedIdeograph5855': '\xe5\xa1\x95',
'CjkUnifiedIdeograph5852': '\xe5\xa1\x92',
'CjkUnifiedIdeograph5853': '\xe5\xa1\x93',
'CjkUnifiedIdeograph5850': '\xe5\xa1\x90',
'GreekCapitalLetterOmicronWithOxia': '\xe1\xbf\xb9',
'KangxiRadicalEnclosure': '\xe2\xbc\x9e',
'GlagoliticSmallLetterDzelo': '\xe2\xb0\xb7',
'CjkUnifiedIdeograph5858': '\xe5\xa1\x98',
'CjkUnifiedIdeograph5859': '\xe5\xa1\x99',
'CjkUnifiedIdeograph65ef': '\xe6\x97\xaf',
'UnicodeCharacter0xea24': '\xee\xa8\xa4',
'CjkUnifiedIdeograph65ed': '\xe6\x97\xad',
'CjkUnifiedIdeograph65ee': '\xe6\x97\xae',
'CjkUnifiedIdeograph65eb': '\xe6\x97\xab',
'KannadaVowelSignVocalicL': '\xe0\xb3\xa2',
'CjkUnifiedIdeograph65ea': '\xe6\x97\xaa',
'UnicodeCharacter0xd854': '\xed\xa1\x94',
'HangulSyllableGgweb': '\xeb\x80\x81',
'UnicodeCharacter0xa6d8': '\xea\x9b\x98',
'CjkUnifiedIdeograph5c37': '\xe5\xb0\xb7',
'CjkUnifiedIdeograph4132': '\xe4\x84\xb2',
'CjkUnifiedIdeograph5c35': '\xe5\xb0\xb5',
'CjkUnifiedIdeograph5c34': '\xe5\xb0\xb4',
'CjkUnifiedIdeograph5c33': '\xe5\xb0\xb3',
'CjkUnifiedIdeograph5c32': '\xe5\xb0\xb2',
'CjkUnifiedIdeograph5c31': '\xe5\xb0\xb1',
'CjkUnifiedIdeograph5c30': '\xe5\xb0\xb0',
'HangulSyllableSsyuc': '\xec\x93\xab',
'HangulSyllableGgwed': '\xea\xbf\xb7',
'KannadaVowelSignVocalicR': '\xe0\xb3\x83',
'HalfwidthHangulLetterPieup': '\xef\xbe\xb2',
'CjkUnifiedIdeograph5c39': '\xe5\xb0\xb9',
'LatinCapitalLetterUWithOgonek': '\xc5\xb2',
'CjkUnifiedIdeograph585f': '\xe5\xa1\x9f',
'UnicodeCharacter0xf72b': '\xef\x9c\xab',
'CjkUnifiedIdeograph585d': '\xe5\xa1\x9d',
'CjkUnifiedIdeograph585e': '\xe5\xa1\x9e',
'CjkUnifiedIdeograph585b': '\xe5\xa1\x9b',
'CjkUnifiedIdeograph585c': '\xe5\xa1\x9c',
'ArabicLetterJeemInitialForm': '\xef\xba\x9f',
'CjkUnifiedIdeograph585a': '\xe5\xa1\x9a',
'CanadianSyllabicsCarrierGha': '\xe1\x97\x89',
'HangulSyllableGgweg': '\xea\xbf\xb1',
'HangulSyllableGgoss': '\xea\xbd\x80',
'CjkUnifiedIdeograph5580': '\xe5\x96\x80',
'CjkUnifiedIdeograph5581': '\xe5\x96\x81',
'CjkUnifiedIdeograph5582': '\xe5\x96\x82',
'CjkUnifiedIdeograph5583': '\xe5\x96\x83',
'CjkUnifiedIdeograph5584': '\xe5\x96\x84',
'CjkUnifiedIdeograph5585': '\xe5\x96\x85',
'CjkUnifiedIdeograph5586': '\xe5\x96\x86',
'CjkUnifiedIdeograph5587': '\xe5\x96\x87',
'CjkUnifiedIdeograph5588': '\xe5\x96\x88',
'CjkUnifiedIdeograph5589': '\xe5\x96\x89',
'CjkUnifiedIdeograph8758': '\xe8\x9d\x98',
'CjkUnifiedIdeograph8759': '\xe8\x9d\x99',
'KangxiRadicalHalberd': '\xe2\xbc\xbd',
'LatinCapitalLetterAWithHookAbove': '\xe1\xba\xa2',
'HangulSyllableSyabs': '\xec\x83\xb6',
'UnicodeCharacter0xf473': '\xef\x91\xb3',
'CjkUnifiedIdeograph5c3f': '\xe5\xb0\xbf',
'CjkUnifiedIdeograph5c3e': '\xe5\xb0\xbe',
'CjkUnifiedIdeograph5c3d': '\xe5\xb0\xbd',
'CjkUnifiedIdeograph5c3c': '\xe5\xb0\xbc',
'CjkUnifiedIdeograph5c3b': '\xe5\xb0\xbb',
'CjkUnifiedIdeograph5c3a': '\xe5\xb0\xba',
'UnicodeCharacter0xf474': '\xef\x91\xb4',
'UnicodeCharacter0xea28': '\xee\xa8\xa8',
'UnicodeCharacter0xf479': '\xef\x91\xb9',
'UnicodeCharacter0xf478': '\xef\x91\xb8',
'CjkUnifiedIdeograph7dcd': '\xe7\xb7\x8d',
'UnicodeCharacter0xea29': '\xee\xa8\xa9',
'CjkUnifiedIdeograph65e6': '\xe6\x97\xa6',
'CjkUnifiedIdeograph65e7': '\xe6\x97\xa7',
'CjkUnifiedIdeograph65e4': '\xe6\x97\xa4',
'CjkUnifiedIdeograph65e5': '\xe6\x97\xa5',
'CjkUnifiedIdeograph65e2': '\xe6\x97\xa2',
'GreekCapitalLetterUpsilonWithTonos': '\xce\x8e',
'CjkUnifiedIdeograph65e0': '\xe6\x97\xa0',
'CjkUnifiedIdeograph65e1': '\xe6\x97\xa1',
'VaiSyllableHa': '\xea\x95\x8c',
'HangulSyllableGguss': '\xea\xbf\x8c',
'MongolianLetterAliGaliDa': '\xe1\xa2\x91',
'CjkUnifiedIdeograph65e8': '\xe6\x97\xa8',
'SquareOomu': '\xe3\x8c\x8a',
'UnicodeCharacter0xf322': '\xef\x8c\xa2',
'CyrillicSmallLetterSchwaWithDiaeresis': '\xd3\x9b',
'UnicodeCharacter0xe1dc': '\xee\x87\x9c',
'CjkUnifiedIdeograph3cca': '\xe3\xb3\x8a',
'CjkUnifiedIdeograph3ccc': '\xe3\xb3\x8c',
'CjkUnifiedIdeograph3ccb': '\xe3\xb3\x8b',
'CjkUnifiedIdeograph3cce': '\xe3\xb3\x8e',
'CjkUnifiedIdeograph3ccd': '\xe3\xb3\x8d',
'CjkUnifiedIdeograph3ccf': '\xe3\xb3\x8f',
'CjkUnifiedIdeograph53d2': '\xe5\x8f\x92',
'CjkUnifiedIdeograph53d3': '\xe5\x8f\x93',
'CjkUnifiedIdeograph53d0': '\xe5\x8f\x90',
'CjkUnifiedIdeograph53d1': '\xe5\x8f\x91',
'CjkUnifiedIdeograph53d6': '\xe5\x8f\x96',
'CjkUnifiedIdeograph53d7': '\xe5\x8f\x97',
'CjkUnifiedIdeograph53d4': '\xe5\x8f\x94',
'CjkUnifiedIdeograph53d5': '\xe5\x8f\x95',
'CjkUnifiedIdeograph406d': '\xe4\x81\xad',
'CjkUnifiedIdeograph406e': '\xe4\x81\xae',
'CjkUnifiedIdeograph53d8': '\xe5\x8f\x98',
'CjkUnifiedIdeograph53d9': '\xe5\x8f\x99',
'CjkUnifiedIdeograph406a': '\xe4\x81\xaa',
'CjkUnifiedIdeograph406b': '\xe4\x81\xab',
'CjkUnifiedIdeograph406c': '\xe4\x81\xac',
'EthiopicSyllableFaa': '\xe1\x8d\x8b',
'UnicodeCharacter0xf0e0': '\xef\x83\xa0',
'LatinSmallLetterOWithHornAndDotBelow': '\xe1\xbb\xa3',
'CyrillicCapitalLetterIWithDiaeresis': '\xd3\xa4',
'HebrewAccentGershayim': '\xd6\x9e',
'CjkUnifiedIdeograph3cc9': '\xe3\xb3\x89',
'CjkUnifiedIdeograph3cc8': '\xe3\xb3\x88',
'dotminus': '\xe2\x88\xb8',
'HangulSyllableDdaes': '\xeb\x95\x9f',
'HangulSyllableDdaet': '\xeb\x95\xa5',
'UnicodeCharacter0xe89b': '\xee\xa2\x9b',
'CjkUnifiedIdeograph3cc1': '\xe3\xb3\x81',
'CjkUnifiedIdeograph3cc0': '\xe3\xb3\x80',
'CjkUnifiedIdeograph3cc3': '\xe3\xb3\x83',
'CjkUnifiedIdeograph3cc2': '\xe3\xb3\x82',
'CjkUnifiedIdeograph3cc5': '\xe3\xb3\x85',
'CjkUnifiedIdeograph3cc4': '\xe3\xb3\x84',
'CjkUnifiedIdeograph3cc7': '\xe3\xb3\x87',
'CjkUnifiedIdeograph3cc6': '\xe3\xb3\x86',
'GlagoliticCapitalLetterInitialIzhe': '\xe2\xb0\x8a',
'HangulSyllableDdaeb': '\xeb\x95\x9d',
'HangulSyllableDdaec': '\xeb\x95\xa3',
'HangulSyllableDdaed': '\xeb\x95\x93',
'HangulSyllableDdaeg': '\xeb\x95\x8d',
'HangulSyllableDdaeh': '\xeb\x95\xa7',
'HangulSyllableDdaej': '\xeb\x95\xa2',
'HangulSyllableDdaek': '\xeb\x95\xa4',
'HangulSyllableDdael': '\xeb\x95\x94',
'HangulSyllableDdaem': '\xeb\x95\x9c',
'HangulSyllableDdaen': '\xeb\x95\x90',
'CjkUnifiedIdeograph776b': '\xe7\x9d\xab',
'DivisionSign': '\xc3\xb7',
'CjkUnifiedIdeograph53db': '\xe5\x8f\x9b',
'CjkUnifiedIdeograph53dc': '\xe5\x8f\x9c',
'CjkUnifiedIdeograph53da': '\xe5\x8f\x9a',
'CjkUnifiedIdeograph53df': '\xe5\x8f\x9f',
'CjkUnifiedIdeograph4069': '\xe4\x81\xa9',
'CjkUnifiedIdeograph53dd': '\xe5\x8f\x9d',
'CjkUnifiedIdeograph53de': '\xe5\x8f\x9e',
'CjkUnifiedIdeograph4064': '\xe4\x81\xa4',
'CjkUnifiedIdeograph4065': '\xe4\x81\xa5',
'CjkUnifiedIdeograph4066': '\xe4\x81\xa6',
'CjkUnifiedIdeograph4067': '\xe4\x81\xa7',
'CjkUnifiedIdeograph4060': '\xe4\x81\xa0',
'CjkUnifiedIdeograph4061': '\xe4\x81\xa1',
'CjkUnifiedIdeograph4062': '\xe4\x81\xa2',
'CjkUnifiedIdeograph4063': '\xe4\x81\xa3',
'UnicodeCharacter0xf646': '\xef\x99\x86',
'CjkUnifiedIdeograph9680': '\xe9\x9a\x80',
'CjkUnifiedIdeograph3dcb': '\xe3\xb7\x8b',
'CjkUnifiedIdeograph3dcc': '\xe3\xb7\x8c',
'ArabicLigatureTehWithKhahInitialForm': '\xef\xb2\xa3',
'CjkUnifiedIdeograph3dca': '\xe3\xb7\x8a',
'CjkUnifiedIdeograph3dcf': '\xe3\xb7\x8f',
'ArabicLetterGafInitialForm': '\xef\xae\x94',
'CjkUnifiedIdeograph3dcd': '\xe3\xb7\x8d',
'CjkUnifiedIdeograph3dce': '\xe3\xb7\x8e',
'UnicodeCharacter0xab8e': '\xea\xae\x8e',
'UnicodeCharacter0xab8d': '\xea\xae\x8d',
'HangulSyllableJyebs': '\xec\xa1\xa6',
'UnicodeCharacter0xab8f': '\xea\xae\x8f',
'UnicodeCharacter0xab8a': '\xea\xae\x8a',
'DownwardsHarpoonWithBarbLeftwards': '\xe2\x87\x83',
'UnicodeCharacter0xab8b': '\xea\xae\x8b',
'KhmerSignBantoc': '\xe1\x9f\x8b',
'BalineseLetterUkara': '\xe1\xac\x89',
'CjkUnifiedIdeograph60d6': '\xe6\x83\x96',
'UnicodeCharacter0xe2f7': '\xee\x8b\xb7',
'UnicodeCharacter0xe2f6': '\xee\x8b\xb6',
'UnicodeCharacter0xe2f5': '\xee\x8b\xb5',
'UnicodeCharacter0xe2f4': '\xee\x8b\xb4',
'UnicodeCharacter0xe2f3': '\xee\x8b\xb3',
'UnicodeCharacter0xe2f2': '\xee\x8b\xb2',
'HangulSyllableSsyenh': '\xec\x8f\x82',
'UnicodeCharacter0xe2f0': '\xee\x8b\xb0',
'ArabicLigatureSheenWithJeemInitialForm': '\xef\xb4\xad',
'UnicodeCharacter0xe2f9': '\xee\x8b\xb9',
'UnicodeCharacter0xe2f8': '\xee\x8b\xb8',
'HangulSyllableDyonh': '\xeb\x90\xba',
'CircledDigitSix': '\xe2\x91\xa5',
'HangulSyllableDyong': '\xeb\x91\x89',
'HangulSyllableSyilg': '\xec\x8b\x89',
'UnicodeCharacter0xab85': '\xea\xae\x85',
'UnicodeCharacter0xab84': '\xea\xae\x84',
'UnicodeCharacter0xab87': '\xea\xae\x87',
'UnicodeCharacter0xab86': '\xea\xae\x86',
'UnicodeCharacter0xab81': '\xea\xae\x81',
'UnicodeCharacter0xab80': '\xea\xae\x80',
'UnicodeCharacter0xab83': '\xea\xae\x83',
'UnicodeCharacter0xab82': '\xea\xae\x82',
'KannadaDigitEight': '\xe0\xb3\xae',
'CjkCompatibilityIdeographfa5c': '\xef\xa9\x9c',
'RunicLetterEng': '\xe1\x9a\xb6',
'UnicodeCharacter0xab89': '\xea\xae\x89',
'UnicodeCharacter0xab88': '\xea\xae\x88',
'HangulSyllableTeup': '\xed\x8b\x92',
'HangulSyllableTeun': '\xed\x8a\xbc',
'CjkUnifiedIdeograph3dc8': '\xe3\xb7\x88',
'CjkUnifiedIdeograph3dc9': '\xe3\xb7\x89',
'HangulSyllableBwelm': '\xeb\xb7\x82',
'HangulSyllableTeuj': '\xed\x8b\x8e',
'UnicodeCharacter0xd852': '\xed\xa1\x92',
'CjkUnifiedIdeograph6fc6': '\xe6\xbf\x86',
'CjkUnifiedIdeograph3dc2': '\xe3\xb7\x82',
'CjkUnifiedIdeograph3dc3': '\xe3\xb7\x83',
'CareOf': '\xe2\x84\x85',
'CjkUnifiedIdeograph3dc1': '\xe3\xb7\x81',
'CjkUnifiedIdeograph3dc6': '\xe3\xb7\x86',
'CjkUnifiedIdeograph3dc7': '\xe3\xb7\x87',
'CjkUnifiedIdeograph3dc4': '\xe3\xb7\x84',
'CjkUnifiedIdeograph3dc5': '\xe3\xb7\x85',
'HangulSyllableCos': '\xec\xb4\x9b',
'HangulSyllableSsyenj': '\xec\x8f\x81',
'UnicodeCharacter0xe1f8': '\xee\x87\xb8',
'UnicodeCharacter0xea2b': '\xee\xa8\xab',
'UnicodeCharacter0xf037': '\xef\x80\xb7',
'CjkCompatibilityIdeographf9d6': '\xef\xa7\x96',
'UnicodeCharacter0xea2c': '\xee\xa8\xac',
'CjkUnifiedIdeograph6fc2': '\xe6\xbf\x82',
'UnicodeCharacter0xf5e4': '\xef\x97\xa4',
'UnicodeCharacter0xe2ff': '\xee\x8b\xbf',
'UnicodeCharacter0xe2fe': '\xee\x8b\xbe',
'UnicodeCharacter0xe2fd': '\xee\x8b\xbd',
'UnicodeCharacter0xe2fc': '\xee\x8b\xbc',
'UnicodeCharacter0xe2fb': '\xee\x8b\xbb',
'UnicodeCharacter0xe2fa': '\xee\x8b\xba',
'UnicodeCharacter0xf5e3': '\xef\x97\xa3',
'UnicodeCharacter0xdcbe': '\xed\xb2\xbe',
'UnicodeCharacter0xf5e8': '\xef\x97\xa8',
'LatinCapitalLetterTWithDotBelow': '\xe1\xb9\xac',
'CjkUnifiedIdeograph5bfe': '\xe5\xaf\xbe',
'CjkUnifiedIdeograph5bfd': '\xe5\xaf\xbd',
'CjkUnifiedIdeograph9937': '\xe9\xa4\xb7',
'CjkUnifiedIdeograph5bff': '\xe5\xaf\xbf',
'CjkUnifiedIdeograph5bfa': '\xe5\xaf\xba',
'CjkUnifiedIdeograph9930': '\xe9\xa4\xb0',
'CjkUnifiedIdeograph5bfc': '\xe5\xaf\xbc',
'BalineseMusicalSymbolCombiningTegeh': '\xe1\xad\xab',
'CjkUnifiedIdeograph5cea': '\xe5\xb3\xaa',
'CjkUnifiedIdeograph9939': '\xe9\xa4\xb9',
'CjkUnifiedIdeograph9938': '\xe9\xa4\xb8',
'CombiningMacronLeftHalf': '\xef\xb8\xa4',
'HangulSyllableDdwej': '\xeb\x9b\xaa',
'CanadianSyllabicsShwaa': '\xe1\x94\xa3',
'ModifierLetterCapitalM': '\xe1\xb4\xb9',
'HangulSyllableDdwek': '\xeb\x9b\xac',
'CjkUnifiedIdeograph7dc2': '\xe7\xb7\x82',
'HangulSyllableWeoss': '\xec\x9b\xa0',
'CircledIdeographStock': '\xe3\x8a\x91',
'HangulSyllableDdweh': '\xeb\x9b\xaf',
'LatinSmallLetterEzhReversed': '\xc6\xb9',
'CjkUnifiedIdeograph9a13': '\xe9\xa8\x93',
'CombiningLeftArrowheadBelow': '\xcd\x94',
'CjkUnifiedIdeograph8b7f': '\xe8\xad\xbf',
'TripleVerticalBarRightTurnstile': '\xe2\x8a\xaa',
'SylotiNagriLetterSo': '\xea\xa0\xa1',
'HangulSyllableJjyi': '\xec\xaf\xb0',
'HangulSyllableDdwen': '\xeb\x9b\x98',
'CjkUnifiedIdeograph7dc1': '\xe7\xb7\x81',
'HangulSyllableBbeh': '\xeb\xbc\x87',
'CjkUnifiedIdeograph5005': '\xe5\x80\x85',
'CjkUnifiedIdeograph5004': '\xe5\x80\x84',
'CjkUnifiedIdeograph5007': '\xe5\x80\x87',
'CjkUnifiedIdeograph5006': '\xe5\x80\x86',
'CjkUnifiedIdeograph5001': '\xe5\x80\x81',
'CjkUnifiedIdeograph5000': '\xe5\x80\x80',
'CjkUnifiedIdeograph5003': '\xe5\x80\x83',
'CjkUnifiedIdeograph5002': '\xe5\x80\x82',
'CjkUnifiedIdeograph74e8': '\xe7\x93\xa8',
'CjkUnifiedIdeograph74e9': '\xe7\x93\xa9',
'BlackSmallSquare': '\xe2\x96\xaa',
'CjkUnifiedIdeograph5009': '\xe5\x80\x89',
'CjkUnifiedIdeograph5008': '\xe5\x80\x88',
'HangulSyllableNoeg': '\xeb\x87\x8d',
'UnicodeCharacter0xe353': '\xee\x8d\x93',
'HangulSyllableNoed': '\xeb\x87\x93',
'KangxiRadicalShortTailedBird': '\xe2\xbe\xab',
'HangulSyllableNoeb': '\xeb\x87\x9d',
'HangulSyllableByu': '\xeb\xb7\xb0',
'HangulSyllableNoen': '\xeb\x87\x90',
'CjkUnifiedIdeograph8832': '\xe8\xa0\xb2',
'HangulSyllableNoel': '\xeb\x87\x94',
'HangulSyllableNoek': '\xeb\x87\xa4',
'HangulSyllableNoej': '\xeb\x87\xa2',
'CjkUnifiedIdeograph5cf5': '\xe5\xb3\xb5',
'CjkUnifiedIdeograph5bf5': '\xe5\xaf\xb5',
'CjkUnifiedIdeograph5bf4': '\xe5\xaf\xb4',
'CjkUnifiedIdeograph5bf7': '\xe5\xaf\xb7',
'CjkUnifiedIdeograph5bf6': '\xe5\xaf\xb6',
'CjkUnifiedIdeograph5bf1': '\xe5\xaf\xb1',
'CjkUnifiedIdeograph5bf0': '\xe5\xaf\xb0',
'CjkUnifiedIdeograph5bf3': '\xe5\xaf\xb3',
'CjkUnifiedIdeograph5bf2': '\xe5\xaf\xb2',
'HangulSyllableBye': '\xeb\xb3\x98',
'UnicodeCharacter0xaa3f': '\xea\xa8\xbf',
'UnicodeCharacter0xaa3e': '\xea\xa8\xbe',
'CjkUnifiedIdeograph7dc9': '\xe7\xb7\x89',
'CjkUnifiedIdeograph5bf9': '\xe5\xaf\xb9',
'CjkUnifiedIdeograph5bf8': '\xe5\xaf\xb8',
'UnicodeCharacter0xaa3a': '\xea\xa8\xba',
'HangulSyllableSwaegg': '\xec\x87\x86',
'CjkUnifiedIdeograph500e': '\xe5\x80\x8e',
'CjkUnifiedIdeograph500d': '\xe5\x80\x8d',
'CjkUnifiedIdeograph74eb': '\xe7\x93\xab',
'CjkUnifiedIdeograph500f': '\xe5\x80\x8f',
'CjkUnifiedIdeograph500a': '\xe5\x80\x8a',
'CjkUnifiedIdeograph74ee': '\xe7\x93\xae',
'CjkUnifiedIdeograph500c': '\xe5\x80\x8c',
'CjkUnifiedIdeograph500b': '\xe5\x80\x8b',
'UnicodeCharacter0xf01f': '\xef\x80\x9f',
'CjkUnifiedIdeograph39ae': '\xe3\xa6\xae',
'UnicodeCharacter0xf161': '\xef\x85\xa1',
'HangulSyllableCwaenh': '\xec\xb5\x86',
'HangulSyllableCwaenj': '\xec\xb5\x85',
'YiSyllableXiep': '\xea\x91\xa4',
'UnicodeCharacter0xf6a0': '\xef\x9a\xa0',
'CjkUnifiedIdeograph8fbb': '\xe8\xbe\xbb',
'CjkUnifiedIdeograph8fbc': '\xe8\xbe\xbc',
'CjkUnifiedIdeograph8fba': '\xe8\xbe\xba',
'CjkUnifiedIdeograph8fbf': '\xe8\xbe\xbf',
'CjkUnifiedIdeograph8fbd': '\xe8\xbe\xbd',
'CjkUnifiedIdeograph8fbe': '\xe8\xbe\xbe',
'CjkUnifiedIdeograph642b': '\xe6\x90\xab',
'UnicodeCharacter0xdcbd': '\xed\xb2\xbd',
'YiSyllableXiet': '\xea\x91\xa1',
'CjkUnifiedIdeograph484f': '\xe4\xa1\x8f',
'CjkUnifiedIdeograph5fa6': '\xe5\xbe\xa6',
'CjkUnifiedIdeograph484d': '\xe4\xa1\x8d',
'CjkUnifiedIdeograph484e': '\xe4\xa1\x8e',
'CjkUnifiedIdeograph484b': '\xe4\xa1\x8b',
'CjkUnifiedIdeograph484c': '\xe4\xa1\x8c',
'CjkUnifiedIdeograph484a': '\xe4\xa1\x8a',
'HangulSyllableMwan': '\xeb\xab\x88',
'UnicodeCharacter0x1cb6': '\xe1\xb2\xb6',
'HangulSyllableMwam': '\xeb\xab\x94',
'HangulSyllableMwaj': '\xeb\xab\x9a',
'GeorgianCapitalLetterBan': '\xe1\x82\xa1',
'SyriacLetterHeth': '\xdc\x9a',
'HangulSyllableSwaegs': '\xec\x87\x87',
'HangulSyllableMwag': '\xeb\xab\x85',
'HangulSyllableMwad': '\xeb\xab\x8b',
'HangulSyllableMwae': '\xeb\xab\xa0',
'SymbolForBackspace': '\xe2\x90\x88',
'HangulSyllableMwac': '\xeb\xab\x9b',
'UnicodeCharacter0xf27d': '\xef\x89\xbd',
'CjkUnifiedIdeograph413d': '\xe4\x84\xbd',
'UnicodeCharacter0xdb1e': '\xed\xac\x9e',
'UnicodeCharacter0xdb1d': '\xed\xac\x9d',
'HangulSyllableMwat': '\xeb\xab\x9d',
'CjkUnifiedIdeograph413e': '\xe4\x84\xbe',
'SmallRomanNumeralOneThousand': '\xe2\x85\xbf',
'HangulSyllableMwas': '\xeb\xab\x97',
'HangulSyllableMwap': '\xeb\xab\x9e',
'UnicodeCharacter0xdb1b': '\xed\xac\x9b',
'HangulSyllableDolg': '\xeb\x8f\x8d',
'ArabicLigatureTehWithMeemMedialForm': '\xef\xb3\xa3',
'GreekSmallLetterIota': '\xce\xb9',
'CjkUnifiedIdeograph4848': '\xe4\xa1\x88',
'CjkUnifiedIdeograph4849': '\xe4\xa1\x89',
'CjkUnifiedIdeograph4846': '\xe4\xa1\x86',
'CjkUnifiedIdeograph4847': '\xe4\xa1\x87',
'CjkUnifiedIdeograph4844': '\xe4\xa1\x84',
'CjkUnifiedIdeograph4845': '\xe4\xa1\x85',
'CjkUnifiedIdeograph4842': '\xe4\xa1\x82',
'CjkUnifiedIdeograph4843': '\xe4\xa1\x83',
'CjkUnifiedIdeograph4840': '\xe4\xa1\x80',
'CjkUnifiedIdeograph4841': '\xe4\xa1\x81',
'CjkUnifiedIdeograph8fb2': '\xe8\xbe\xb2',
'MongolianLetterTodoOe': '\xe1\xa1\x88',
'CjkUnifiedIdeograph8fb0': '\xe8\xbe\xb0',
'CjkUnifiedIdeograph8fb1': '\xe8\xbe\xb1',
'CjkUnifiedIdeograph8fb6': '\xe8\xbe\xb6',
'CjkUnifiedIdeograph8fb7': '\xe8\xbe\xb7',
'HangulChoseongSsangtikeut': '\xe1\x84\x84',
'CjkUnifiedIdeograph8fb5': '\xe8\xbe\xb5',
'UnicodeCharacter0xe356': '\xee\x8d\x96',
'UnicodeCharacter0xf6a5': '\xef\x9a\xa5',
'CjkUnifiedIdeograph8fb8': '\xe8\xbe\xb8',
'CjkUnifiedIdeograph8fb9': '\xe8\xbe\xb9',
'UnicodeCharacter0xdb19': '\xed\xac\x99',
'UnicodeCharacter0xdb18': '\xed\xac\x98',
'CanadianSyllabicsWestcreeWii': '\xe1\x90\x91',
'UnicodeCharacter0xdb15': '\xed\xac\x95',
'UnicodeCharacter0xdb14': '\xed\xac\x94',
'UnicodeCharacter0xdb17': '\xed\xac\x97',
'HangulSyllableByen': '\xeb\xb3\x9c',
'KhmerDigitFive': '\xe1\x9f\xa5',
'UnicodeCharacter0xdb10': '\xed\xac\x90',
'UnicodeCharacter0xdb13': '\xed\xac\x93',
'UnicodeCharacter0xdb12': '\xed\xac\x92',
'CjkUnifiedIdeograph6fcf': '\xe6\xbf\x8f',
'CjkUnifiedIdeograph7573': '\xe7\x95\xb3',
'CjkUnifiedIdeograph7572': '\xe7\x95\xb2',
'CjkUnifiedIdeograph7571': '\xe7\x95\xb1',
'KangxiRadicalSee': '\xe2\xbe\x92',
'CjkUnifiedIdeograph7577': '\xe7\x95\xb7',
'CjkUnifiedIdeograph7576': '\xe7\x95\xb6',
'CjkUnifiedIdeograph7575': '\xe7\x95\xb5',
'CjkUnifiedIdeograph7574': '\xe7\x95\xb4',
'CjkUnifiedIdeograph7579': '\xe7\x95\xb9',
'CjkUnifiedIdeograph7578': '\xe7\x95\xb8',
'GujaratiLetterTta': '\xe0\xaa\x9f',
'WhiteChessKing': '\xe2\x99\x94',
'CanadianSyllabicsTwa': '\xe1\x91\xa1',
'KannadaLetterNna': '\xe0\xb2\xa3',
'CanadianSyllabicsSaa': '\xe1\x93\xb5',
'CanadianSyllabicsTwe': '\xe1\x91\x97',
'UnicodeCharacter0x275f': '\xe2\x9d\x9f',
'CopticSymbolPiRo': '\xe2\xb3\xa6',
'CanadianSyllabicsTwi': '\xe1\x91\x99',
'CanadianSyllabicsTwo': '\xe1\x91\x9d',
'SaurashtraDigitSix': '\xea\xa3\x96',
'YiSyllableGuo': '\xea\x87\xa8',
'CjkUnifiedIdeograph6f60': '\xe6\xbd\xa0',
'CjkUnifiedIdeograph548b': '\xe5\x92\x8b',
'CjkUnifiedIdeograph3e19': '\xe3\xb8\x99',
'CjkUnifiedIdeograph3e18': '\xe3\xb8\x98',
'UnicodeCharacter0xf7c9': '\xef\x9f\x89',
'CjkUnifiedIdeograph3e15': '\xe3\xb8\x95',
'CjkUnifiedIdeograph3e14': '\xe3\xb8\x94',
'CjkUnifiedIdeograph3e17': '\xe3\xb8\x97',
'CjkUnifiedIdeograph3e16': '\xe3\xb8\x96',
'CjkUnifiedIdeograph3e11': '\xe3\xb8\x91',
'CjkUnifiedIdeograph3e10': '\xe3\xb8\x90',
'CjkUnifiedIdeograph3e13': '\xe3\xb8\x93',
'CjkUnifiedIdeograph3e12': '\xe3\xb8\x92',
'HangulSyllableDdyugs': '\xeb\x9c\x8f',
'PresentationFormForVerticalIdeographicComma': '\xef\xb8\x91',
'YiSyllableGux': '\xea\x87\xb3',
'UnicodeCharacter0xe0f5': '\xee\x83\xb5',
'CjkUnifiedIdeograph548f': '\xe5\x92\x8f',
'YiSyllableGur': '\xea\x87\xb7',
'YiSyllableGup': '\xea\x87\xb5',
'HangulSyllableDogs': '\xeb\x8f\x87',
'LeftAngleBracketWithDot': '\xe2\xa6\x91',
'UnicodeCharacter0xd85d': '\xed\xa1\x9d',
'GurmukhiSignNukta': '\xe0\xa8\xbc',
'UnicodeCharacter0x2757': '\xe2\x9d\x97',
'UnicodeCharacter0x2754': '\xe2\x9d\x94',
'UnicodeCharacter0x2755': '\xe2\x9d\x95',
'GreekCapitalLetterOmegaWithPsiliAndProsgegrammeni': '\xe1\xbe\xa8',
'UnicodeCharacter0x2753': '\xe2\x9d\x93',
'EthiopicDigitFive': '\xe1\x8d\xad',
'CjkUnifiedIdeograph757c': '\xe7\x95\xbc',
'CjkUnifiedIdeograph757b': '\xe7\x95\xbb',
'CjkUnifiedIdeograph757a': '\xe7\x95\xba',
'HangulSyllableDogg': '\xeb\x8f\x86',
'CjkUnifiedIdeograph757f': '\xe7\x95\xbf',
'CjkUnifiedIdeograph757e': '\xe7\x95\xbe',
'CjkUnifiedIdeograph757d': '\xe7\x95\xbd',
'HangulSyllableBbobs': '\xeb\xbd\x92',
'LatinLetterVoicedLaryngealSpirant': '\xe1\xb4\xa4',
'ArabicLigatureLamWithKhahWithMeemInitialForm': '\xef\xb6\x86',
'BalineseMusicalSymbolDaing': '\xe1\xad\xa9',
'LatinCapitalLetterJWithCircumflex': '\xc4\xb4',
'CjkUnifiedIdeograph3e1e': '\xe3\xb8\x9e',
'CjkUnifiedIdeograph3e1d': '\xe3\xb8\x9d',
'CjkUnifiedIdeograph3e1f': '\xe3\xb8\x9f',
'CjkUnifiedIdeograph3e1a': '\xe3\xb8\x9a',
'CjkUnifiedIdeograph3e1c': '\xe3\xb8\x9c',
'CjkUnifiedIdeograph3e1b': '\xe3\xb8\x9b',
'HangulSyllableSyilh': '\xec\x8b\x8f',
'VaiSymbolTing': '\xea\x98\x95',
'CjkUnifiedIdeograph4f5b': '\xe4\xbd\x9b',
'UnicodeCharacter0x19ab': '\xe1\xa6\xab',
'HangulSyllableBbass': '\xeb\xb9\xb4',
'ArabicLetterVeh': '\xda\xa4',
'CjkUnifiedIdeograph4f5f': '\xe4\xbd\x9f',
'HangulSyllableJwalh': '\xec\xa2\x9b',
'YiSyllableMiex': '\xea\x82\xb2',
'ApproximatelyEqualOrEqualTo': '\xe2\xa9\xb0',
'HangulSyllableJwalm': '\xec\xa2\x96',
'UnicodeCharacter0xdccf': '\xed\xb3\x8f',
'YiSyllableMiep': '\xea\x82\xb4',
'HangulSyllableJwalb': '\xec\xa2\x97',
'UnicodeCharacter0xdcce': '\xed\xb3\x8e',
'UnicodeCharacter0xdccb': '\xed\xb3\x8b',
'UnicodeCharacter0xdccc': '\xed\xb3\x8c',
'TibetanSubjoinedLetterFixedformRa': '\xe0\xbe\xbc',
'UpwardsQuadrupleArrow': '\xe2\x9f\xb0',
'HangulSyllableRaess': '\xeb\x9e\xac',
'HangulSyllableJwalp': '\xec\xa2\x9a',
'UnicodeCharacter0xa9f8': '\xea\xa7\xb8',
'HangulSyllableJwals': '\xec\xa2\x98',
'HangulSyllableJwalt': '\xec\xa2\x99',
'UnicodeCharacter0xf4e0': '\xef\x93\xa0',
'SquareRoot': '\xe2\x88\x9a',
'CyrillicCapitalLetterReversedYu': '\xea\x99\x94',
'LatinSmallLetterNWithGrave': '\xc7\xb9',
'HangulSyllableYang': '\xec\x96\x91',
'UnicodeCharacter0xa9f5': '\xea\xa7\xb5',
'MyanmarConsonantSignMedialYa': '\xe1\x80\xbb',
'CjkUnifiedIdeograph7586': '\xe7\x96\x86',
'CjkUnifiedIdeograph697c': '\xe6\xa5\xbc',
'HangulSyllablePwigs': '\xed\x93\x8f',
'UnicodeCharacter0xdde2': '\xed\xb7\xa2',
'Symmetry': '\xe2\x8c\xaf',
'CjkUnifiedIdeograph98e3': '\xe9\xa3\xa3',
'UnicodeCharacter0xa9f0': '\xea\xa7\xb0',
'HangulSyllableKwaegg': '\xec\xbe\x8e',
'UnicodeCharacter0xa9f1': '\xea\xa7\xb1',
'HangulSyllablePwigg': '\xed\x93\x8e',
'ArabicLigatureMeemWithJeemIsolatedForm': '\xef\xb1\x85',
'UnicodeCharacter0xa9f2': '\xea\xa7\xb2',
'UnicodeCharacter0xdcc8': '\xed\xb3\x88',
'UnicodeCharacter0xdcc9': '\xed\xb3\x89',
'UnicodeCharacter0xdcc6': '\xed\xb3\x86',
'CjkUnifiedIdeograph98e6': '\xe9\xa3\xa6',
'UnicodeCharacter0xdcc4': '\xed\xb3\x84',
'UnicodeCharacter0xdcc5': '\xed\xb3\x85',
'UnicodeCharacter0xdcc2': '\xed\xb3\x82',
'UnicodeCharacter0xdcc3': '\xed\xb3\x83',
'UnicodeCharacter0xdcc0': '\xed\xb3\x80',
'UnicodeCharacter0xdcc1': '\xed\xb3\x81',
'LatinCapitalLetterEWithGrave': '\xc3\x88',
'HangulJongseongHieuhnieun': '\xe1\x87\xb5',
'HangulSyllableJjebs': '\xec\xa9\xba',
'UnicodeCharacter0xe532': '\xee\x94\xb2',
'WonSign': '\xe2\x82\xa9',
'CanadianSyllabicsNunavikH': '\xe1\x95\xbb',
'HanunooLetterA': '\xe1\x9c\xa0',
'HangulSyllableWaeng': '\xec\x99\xb1',
'HangulSyllableWaenh': '\xec\x99\xa2',
'HangulSyllableWaenj': '\xec\x99\xa1',
'DoubleLowLine': '\xe2\x80\x97',
'HanunooLetterI': '\xe1\x9c\xa1',
'HanunooLetterU': '\xe1\x9c\xa2',
'CjkUnifiedIdeograph6427': '\xe6\x90\xa7',
'HangulSyllableWils': '\xec\x9c\x90',
'KatakanaLetterMu': '\xe3\x83\xa0',
'UnicodeCharacter0x318f': '\xe3\x86\x8f',
'GurmukhiLetterPa': '\xe0\xa8\xaa',
'WarningSign': '\xe2\x9a\xa0',
'HangulSyllableKwaeh': '\xec\xbe\xa7',
'LatinCapitalLetterAWithInvertedBreve': '\xc8\x82',
'HangulSyllableWilt': '\xec\x9c\x91',
'KatakanaLetterMe': '\xe3\x83\xa1',
'HangulSyllableKwiss': '\xed\x81\x88',
'KatakanaLetterMa': '\xe3\x83\x9e',
'HangulSyllableReobs': '\xeb\x9f\xbe',
'HangulSyllableRyeonj': '\xeb\xa0\xa9',
'KatakanaLetterMo': '\xe3\x83\xa2',
'ModifierLetterSmallOpenO': '\xe1\xb5\x93',
'KatakanaLetterMi': '\xe3\x83\x9f',
'UnicodeCharacter0xd9bb': '\xed\xa6\xbb',
'HangulSyllableHyaeg': '\xed\x96\xad',
'LatinCapitalLetterEWithCedilla': '\xc8\xa8',
'HangulSyllablePegs': '\xed\x8e\x9b',
'HangulSyllableHyaed': '\xed\x96\xb3',
'HangulSyllableHyaec': '\xed\x97\x83',
'HangulSyllableJae': '\xec\x9e\xac',
'CjkUnifiedIdeograph5d26': '\xe5\xb4\xa6',
'CjkUnifiedIdeograph9f3f': '\xe9\xbc\xbf',
'HangulSyllableHyaem': '\xed\x96\xbc',
'HangulSyllableHyael': '\xed\x96\xb4',
'HangulSyllableHyaek': '\xed\x97\x84',
'MongolianFourDots': '\xe1\xa0\x85',
'HangulSyllableHyaeh': '\xed\x97\x87',
'UnicodeCharacter0xe3a2': '\xee\x8e\xa2',
'CjkUnifiedIdeograph5d24': '\xe5\xb4\xa4',
'HangulSyllableHyaes': '\xed\x96\xbf',
'HangulSyllablePegg': '\xed\x8e\x9a',
'HangulSyllableHyaep': '\xed\x97\x86',
'CjkUnifiedIdeograph5d23': '\xe5\xb4\xa3',
'CjkUnifiedIdeograph5d22': '\xe5\xb4\xa2',
'CjkUnifiedIdeograph5d21': '\xe5\xb4\xa1',
'UnicodeCharacter0xec6b': '\xee\xb1\xab',
'LatinSmallLetterTWithDotBelow': '\xe1\xb9\xad',
'CjkUnifiedIdeograph5d20': '\xe5\xb4\xa0',
'HalfwidthHangulLetterRieulpieup': '\xef\xbe\xac',
'LatinCapitalLetterCWithHook': '\xc6\x87',
'UnicodeCharacter0xe14e': '\xee\x85\x8e',
'CjkUnifiedIdeograph6979': '\xe6\xa5\xb9',
'AplFunctionalSymbolCircleStar': '\xe2\x8d\x9f',
'UnicodeCharacter0xe198': '\xee\x86\x98',
'CjkUnifiedIdeograph5661': '\xe5\x99\xa1',
'CjkUnifiedIdeograph5660': '\xe5\x99\xa0',
'HangulSyllableDdaebs': '\xeb\x95\x9e',
'HangulSyllableKwaet': '\xec\xbe\xa5',
'RunicLetterKaunK': '\xe1\x9a\xb4',
'VaiSymbolKeeng': '\xea\x98\x94',
'HangulSyllableByak': '\xeb\xb1\xa4',
'EthiopicSemicolon': '\xe1\x8d\xa4',
'YiSyllableJjo': '\xea\x90\xa5',
'HangulSyllableNyud': '\xeb\x89\xbb',
'YiSyllableJji': '\xea\x90\x9a',
'HangulSyllableKwaes': '\xec\xbe\x9f',
'UnicodeCharacter0xa9fd': '\xea\xa7\xbd',
'YiSyllableJju': '\xea\x90\xa9',
'CjkUnifiedIdeograph9ef5': '\xe9\xbb\xb5',
'HangulSyllableJjeng': '\xec\xa9\xbd',
'CjkUnifiedIdeograph6971': '\xe6\xa5\xb1',
'HangulSyllableJjenh': '\xec\xa9\xae',
'LeftwardsArrowWithTail': '\xe2\x86\xa2',
'UnicodeCharacter0xa9ff': '\xea\xa7\xbf',
'YiSyllableJjy': '\xea\x90\xaf',
'UnicodeCharacter0xb29': '\xe0\xac\xa9',
'CjkUnifiedIdeograph6973': '\xe6\xa5\xb3',
'GreekCapitalLetterXi': '\xce\x9e',
'TildeOperator': '\xe2\x88\xbc',
'CjkUnifiedIdeograph8510': '\xe8\x94\x90',
'CjkUnifiedIdeograph8511': '\xe8\x94\x91',
'CjkUnifiedIdeograph8512': '\xe8\x94\x92',
'CjkUnifiedIdeograph8513': '\xe8\x94\x93',
'CjkUnifiedIdeograph8514': '\xe8\x94\x94',
'CjkUnifiedIdeograph8515': '\xe8\x94\x95',
'CjkUnifiedIdeograph8516': '\xe8\x94\x96',
'CjkUnifiedIdeograph8517': '\xe8\x94\x97',
'CjkUnifiedIdeograph8518': '\xe8\x94\x98',
'CjkUnifiedIdeograph8519': '\xe8\x94\x99',
'UnicodeCharacter0xa9fc': '\xea\xa7\xbc',
'CjkUnifiedIdeograph9f3c': '\xe9\xbc\xbc',
'CjkUnifiedIdeograph98ee': '\xe9\xa3\xae',
'VaiSyllableNye': '\xea\x98\x8a',
'HangulSyllableNonj': '\xeb\x85\xbd',
'HangulSyllableYalp': '\xec\x96\x8a',
'LeftWhiteLenticularBracket': '\xe3\x80\x96',
'RightwardsDoubleArrowWithStroke': '\xe2\x87\x8f',
'UnicodeCharacter0xec6c': '\xee\xb1\xac',
'YiSyllableNit': '\xea\x85\xbd',
'FullwidthDigitOne': '\xef\xbc\x91',
'CjkUnifiedIdeograph851a': '\xe8\x94\x9a',
'CjkUnifiedIdeograph851b': '\xe8\x94\x9b',
'CjkUnifiedIdeograph851c': '\xe8\x94\x9c',
'CjkUnifiedIdeograph851d': '\xe8\x94\x9d',
'CjkUnifiedIdeograph851e': '\xe8\x94\x9e',
'CjkUnifiedIdeograph851f': '\xe8\x94\x9f',
'TibetanMarkCaretYigMgoPhurShadMa': '\xe0\xbc\x86',
'CjkUnifiedIdeograph9ef8': '\xe9\xbb\xb8',
'CjkUnifiedIdeograph8315': '\xe8\x8c\x95',
'VaiSyllableNyo': '\xea\x97\xa0',
'YiSyllableDduox': '\xea\x85\x81',
'HangulSyllableByigs': '\xeb\xb8\xab',
'YiSyllableDduop': '\xea\x85\x83',
'ArabicLigatureJallajalalouhou': '\xef\xb7\xbb',
'HangulSyllablePwec': '\xed\x93\x87',
'KangxiRadicalMeat': '\xe2\xbe\x81',
'CombiningEnclosingKeycap': '\xe2\x83\xa3',
'CjkUnifiedIdeograph9ef9': '\xe9\xbb\xb9',
'WhiteSquareWithLowerRightQuadrant': '\xe2\x97\xb2',
'CjkUnifiedIdeograph7300': '\xe7\x8c\x80',
'GeorgianParagraphSeparator': '\xe1\x83\xbb',
'UnicodeCharacter0xe95d': '\xee\xa5\x9d',
'UnicodeCharacter0xfdcf': '\xef\xb7\x8f',
'CjkUnifiedIdeograph4f57': '\xe4\xbd\x97',
'HangulSyllableSsweogs': '\xec\x92\x83',
'UnicodeCharacter0xfdce': '\xef\xb7\x8e',
'CjkUnifiedIdeograph4f56': '\xe4\xbd\x96',
'UnicodeCharacter0xfdcb': '\xef\xb7\x8b',
'UnicodeCharacter0xe0f1': '\xee\x83\xb1',
'CyrillicSmallLetterIotifiedA': '\xea\x99\x97',
'MongolianLetterAng': '\xe1\xa0\xa9',
'YiSyllableLap': '\xea\x87\x82',
'CjkUnifiedIdeograph971b': '\xe9\x9c\x9b',
'LatinSmallLetterUWithBreve': '\xc5\xad',
'KangxiRadicalStep': '\xe2\xbc\xbb',
'YiSyllableLax': '\xea\x87\x80',
'UnicodeCharacter0xd8e3': '\xed\xa3\xa3',
'BoxDrawingsLeftUpHeavyAndRightDownLight': '\xe2\x95\x83',
'ArabicLigatureSadWithHahWithYehFinalForm': '\xef\xb6\xa9',
'HangulSyllableBbeunj': '\xec\x81\x9d',
'CjkUnifiedIdeograph3b48': '\xe3\xad\x88',
'HangulSyllableBbeung': '\xec\x81\xad',
'CjkUnifiedIdeograph971f': '\xe9\x9c\x9f',
'UnicodeCharacter0xd8e2': '\xed\xa3\xa2',
'UpperBladeScissors': '\xe2\x9c\x81',
'UnicodeCharacter0xf271': '\xef\x89\xb1',
'HangulSyllableSyass': '\xec\x83\xb8',
'CjkUnifiedIdeograph70b6': '\xe7\x82\xb6',
'UnicodeCharacter0xf584': '\xef\x96\x84',
'HangulSyllablePweb': '\xed\x93\x81',
'House': '\xe2\x8c\x82',
'GreekCapitalLetterArchaicSampi': '\xcd\xb2',
'ArabicLetterDyehIsolatedForm': '\xef\xad\xb2',
'MyanmarLetterO': '\xe1\x80\xa9',
'CircledIdeographExcellent': '\xe3\x8a\x9d',
'KayahLiDigitSix': '\xea\xa4\x86',
'CopticSmallLetterOldCopticHori': '\xe2\xb3\x8d',
'ArabicLigatureBehWithKhahInitialForm': '\xef\xb2\x9e',
'UnicodeCharacter0xf270': '\xef\x89\xb0',
'dotplus': '\xe2\x88\x94',
'TripleColonOperator': '\xe2\xab\xb6',
'UnicodeCharacter0xf8d8': '\xef\xa3\x98',
'OriyaLetterVa': '\xe0\xac\xb5',
'LatinCapitalLetterMWithDotBelow': '\xe1\xb9\x82',
'HangulSyllableJjyok': '\xec\xad\x84',
'HangulSyllableJjyoh': '\xec\xad\x87',
'HangulSyllableJjyon': '\xec\xac\xb0',
'HangulSyllableJjyol': '\xec\xac\xb4',
'HangulSyllableJjyom': '\xec\xac\xbc',
'HangulSyllableJjyob': '\xec\xac\xbd',
'HangulSyllableJjyoc': '\xec\xad\x83',
'UnicodeCharacter0xf273': '\xef\x89\xb3',
'UnicodeCharacter0xf222': '\xef\x88\xa2',
'HangulSyllableJjyog': '\xec\xac\xad',
'HangulSyllableJjyod': '\xec\xac\xb3',
'CjkUnifiedIdeograph84cc': '\xe8\x93\x8c',
'CjkUnifiedIdeograph84cb': '\xe8\x93\x8b',
'CjkUnifiedIdeograph84ca': '\xe8\x93\x8a',
'UnicodeCharacter0xe912': '\xee\xa4\x92',
'CyrillicCapitalLetterIWithMacron': '\xd3\xa2',
'CjkUnifiedIdeograph84cf': '\xe8\x93\x8f',
'CjkUnifiedIdeograph84ce': '\xe8\x93\x8e',
'CjkUnifiedIdeograph84cd': '\xe8\x93\x8d',
'HangulSyllableBes': '\xeb\xb2\xb3',
'CjkUnifiedIdeograph6c1a': '\xe6\xb0\x9a',
'CjkUnifiedIdeograph6c1b': '\xe6\xb0\x9b',
'CjkUnifiedIdeograph6c1c': '\xe6\xb0\x9c',
'CjkUnifiedIdeograph6c1d': '\xe6\xb0\x9d',
'CjkUnifiedIdeograph6c1e': '\xe6\xb0\x9e',
'CjkUnifiedIdeograph6c1f': '\xe6\xb0\x9f',
'ModifierLetterLowGraveAccent': '\xcb\x8e',
'BalineseMusicalSymbolCombiningBende': '\xe1\xad\xb2',
'BoxDrawingsHeavyUpAndRight': '\xe2\x94\x97',
'UnicodeCharacter0xda82': '\xed\xaa\x82',
'CjkUnifiedIdeograph6c18': '\xe6\xb0\x98',
'CjkUnifiedIdeograph6c19': '\xe6\xb0\x99',
'CjkUnifiedIdeograph84c1': '\xe8\x93\x81',
'CjkUnifiedIdeograph84c0': '\xe8\x93\x80',
'CjkUnifiedIdeograph84c7': '\xe8\x93\x87',
'KhmerLetterPo': '\xe1\x9e\x96',
'CombiningXBelow': '\xcd\x93',
'CjkUnifiedIdeograph5d2f': '\xe5\xb4\xaf',
'CjkUnifiedIdeograph6c10': '\xe6\xb0\x90',
'CjkUnifiedIdeograph6c11': '\xe6\xb0\x91',
'CjkUnifiedIdeograph6c12': '\xe6\xb0\x92',
'CjkUnifiedIdeograph6c13': '\xe6\xb0\x93',
'CjkUnifiedIdeograph6c14': '\xe6\xb0\x94',
'CjkUnifiedIdeograph5d2e': '\xe5\xb4\xae',
'CjkUnifiedIdeograph6c16': '\xe6\xb0\x96',
'DottedRightpointingAngle': '\xe2\xb8\x96',
'YiSyllableWo': '\xea\x8a\x88',
'CjkUnifiedIdeograph5d2d': '\xe5\xb4\xad',
'HangulSyllableEubs': '\xec\x9d\x8e',
'CjkUnifiedIdeograph5d2c': '\xe5\xb4\xac',
'YiSyllableWe': '\xea\x8a\x8b',
'CjkUnifiedIdeograph92f6': '\xe9\x8b\xb6',
'YiSyllableWa': '\xea\x8a\x82',
'CjkUnifiedIdeograph5d2b': '\xe5\xb4\xab',
'UnicodeCharacter0x1cac': '\xe1\xb2\xac',
'UnicodeCharacter0x1cab': '\xe1\xb2\xab',
'UnicodeCharacter0x1caa': '\xe1\xb2\xaa',
'CjkUnifiedIdeograph44cd': '\xe4\x93\x8d',
'CjkUnifiedIdeograph44cc': '\xe4\x93\x8c',
'UnicodeCharacter0x1caf': '\xe1\xb2\xaf',
'CjkUnifiedIdeograph44ca': '\xe4\x93\x8a',
'UnicodeCharacter0x1cad': '\xe1\xb2\xad',
'HangulSyllableCed': '\xec\xb2\xbb',
'HangulSyllableAnh': '\xec\x95\x8a',
'HangulSyllableJyec': '\xec\xa1\xab',
'HangulSyllableCeb': '\xec\xb3\x85',
'HangulSyllableCec': '\xec\xb3\x8b',
'HangulSyllableAng': '\xec\x95\x99',
'HangulSyllableJyeb': '\xec\xa1\xa5',
'ShuffleProduct': '\xe2\xa7\xa2',
'YiSyllableZhut': '\xea\x8d\x95',
'CjkUnifiedIdeograph4c6c': '\xe4\xb1\xac',
'CjkUnifiedIdeograph4c6b': '\xe4\xb1\xab',
'CjkUnifiedIdeograph4c6a': '\xe4\xb1\xaa',
'HangulSyllableCep': '\xec\xb3\x8e',
'CjkUnifiedIdeograph4c6f': '\xe4\xb1\xaf',
'CjkUnifiedIdeograph4c6e': '\xe4\xb1\xae',
'CjkUnifiedIdeograph4c6d': '\xe4\xb1\xad',
'CjkUnifiedIdeograph87cd': '\xe8\x9f\x8d',
'GreekSmallLetterAlphaWithDasiaAndPerispomeni': '\xe1\xbc\x87',
'CjkUnifiedIdeograph87cf': '\xe8\x9f\x8f',
'UnicodeCharacter0xf277': '\xef\x89\xb7',
'CjkUnifiedIdeograph87ca': '\xe8\x9f\x8a',
'CjkUnifiedIdeograph87cb': '\xe8\x9f\x8b',
'CjkUnifiedIdeograph87cc': '\xe8\x9f\x8c',
'LepchaLetterNa': '\xe1\xb0\x8d',
'YiSyllableZhup': '\xea\x8d\x98',
'CjkUnifiedIdeograph8fc0': '\xe8\xbf\x80',
'SinhalaLetterAlpapraanaJayanna': '\xe0\xb6\xa2',
'HangulSyllableTyiss': '\xed\x8b\xa8',
'UnicodeCharacter0x139d': '\xe1\x8e\x9d',
'UnicodeCharacter0xf276': '\xef\x89\xb6',
'CjkUnifiedIdeograph4c69': '\xe4\xb1\xa9',
'CjkUnifiedIdeograph4c68': '\xe4\xb1\xa8',
'CjkCompatibilityIdeographfa8b': '\xef\xaa\x8b',
'CjkUnifiedIdeograph4c63': '\xe4\xb1\xa3',
'CjkUnifiedIdeograph4c62': '\xe4\xb1\xa2',
'CjkUnifiedIdeograph4c61': '\xe4\xb1\xa1',
'CjkUnifiedIdeograph4c60': '\xe4\xb1\xa0',
'CjkUnifiedIdeograph4c67': '\xe4\xb1\xa7',
'CjkUnifiedIdeograph4c66': '\xe4\xb1\xa6',
'CjkUnifiedIdeograph4c65': '\xe4\xb1\xa5',
'CjkUnifiedIdeograph4c64': '\xe4\xb1\xa4',
'CjkUnifiedIdeograph44c7': '\xe4\x93\x87',
'CjkUnifiedIdeograph44c6': '\xe4\x93\x86',
'CjkUnifiedIdeograph44c5': '\xe4\x93\x85',
'CjkUnifiedIdeograph44c4': '\xe4\x93\x84',
'CjkUnifiedIdeograph44c3': '\xe4\x93\x83',
'CjkUnifiedIdeograph44c2': '\xe4\x93\x82',
'CjkUnifiedIdeograph44c1': '\xe4\x93\x81',
'CjkUnifiedIdeograph44c0': '\xe4\x93\x80',
'UnicodeCharacter0x1ca9': '\xe1\xb2\xa9',
'UnicodeCharacter0x1ca8': '\xe1\xb2\xa8',
'CjkUnifiedIdeograph44c9': '\xe4\x93\x89',
'CjkUnifiedIdeograph44c8': '\xe4\x93\x88',
'HangulSyllableJyep': '\xec\xa1\xae',
'DottedFence': '\xe2\xa6\x99',
'UnicodeCharacter0xede9': '\xee\xb7\xa9',
'HangulSyllableMoss': '\xeb\xaa\xbc',
'HangulSyllableGgap': '\xea\xb9\xa6',
'HangulSyllableJyes': '\xec\xa1\xa7',
'HangulSyllableGgas': '\xea\xb9\x9f',
'CjkUnifiedIdeograph87c4': '\xe8\x9f\x84',
'CjkUnifiedIdeograph87c5': '\xe8\x9f\x85',
'CjkUnifiedIdeograph87c6': '\xe8\x9f\x86',
'CjkUnifiedIdeograph87c7': '\xe8\x9f\x87',
'CjkUnifiedIdeograph87c0': '\xe8\x9f\x80',
'CjkUnifiedIdeograph87c1': '\xe8\x9f\x81',
'EthiopicDigitSix': '\xe1\x8d\xae',
'CjkUnifiedIdeograph87c3': '\xe8\x9f\x83',
'HangulSyllableGgad': '\xea\xb9\x93',
'HangulSyllableGgae': '\xea\xb9\xa8',
'HangulSyllableGgag': '\xea\xb9\x8d',
'CjkUnifiedIdeograph87c8': '\xe8\x9f\x88',
'CjkUnifiedIdeograph87c9': '\xe8\x9f\x89',
'HangulSyllableGgab': '\xea\xb9\x9d',
'HangulSyllableGgac': '\xea\xb9\xa3',
'UnicodeCharacter0xf392': '\xef\x8e\x92',
'UnicodeCharacter0xf393': '\xef\x8e\x93',
'LatinSmallLetterEng': '\xc5\x8b',
'UnicodeCharacter0xf391': '\xef\x8e\x91',
'UnicodeCharacter0xf396': '\xef\x8e\x96',
'UnicodeCharacter0xf397': '\xef\x8e\x97',
'UnicodeCharacter0xf394': '\xef\x8e\x94',
'UnicodeCharacter0xf395': '\xef\x8e\x95',
'CjkUnifiedIdeograph47ca': '\xe4\x9f\x8a',
'CjkUnifiedIdeograph47cb': '\xe4\x9f\x8b',
'CjkUnifiedIdeograph47cc': '\xe4\x9f\x8c',
'CjkUnifiedIdeograph47cd': '\xe4\x9f\x8d',
'CjkUnifiedIdeograph47ce': '\xe4\x9f\x8e',
'CjkUnifiedIdeograph47cf': '\xe4\x9f\x8f',
'UnicodeCharacter0xe14a': '\xee\x85\x8a',
'MongolianLetterTodoAng': '\xe1\xa1\x8a',
'YiSyllableLyp': '\xea\x87\x97',
'HangulSyllableTyeolm': '\xed\x85\xb2',
'YiSyllableTuox': '\xea\x84\xa6',
'RecycledPaperSymbol': '\xe2\x99\xbc',
'CjkUnifiedIdeograph75e1': '\xe7\x97\xa1',
'YiSyllableTuot': '\xea\x84\xa5',
'HangulSyllableTyeolg': '\xed\x85\xb1',
'HangulSyllableKilt': '\xed\x82\xb1',
'UnicodeCharacter0xe28b': '\xee\x8a\x8b',
'YiSyllableTuop': '\xea\x84\xa8',
'UnicodeCharacter0xe28d': '\xee\x8a\x8d',
'UnicodeCharacter0xe28f': '\xee\x8a\x8f',
'UnicodeCharacter0xde4f': '\xed\xb9\x8f',
'ModifierLetterBeginLowTone': '\xcb\xbb',
'ArabicLigatureSheenWithHahWithMeemInitialForm': '\xef\xb5\xa8',
'Caduceus': '\xe2\x98\xa4',
'HangulSyllableTyeolt': '\xed\x85\xb5',
'SaurashtraLetterA': '\xea\xa2\x82',
'CjkUnifiedIdeograph83f4': '\xe8\x8f\xb4',
'HangulSyllableKwah': '\xec\xbe\x8b',
'HangulSyllableKwaj': '\xec\xbe\x86',
'HangulSyllableKwak': '\xec\xbe\x88',
'HangulSyllableKwal': '\xec\xbd\xb8',
'HangulSyllableKwam': '\xec\xbe\x80',
'HangulSyllableKwan': '\xec\xbd\xb4',
'YiSyllableSho': '\xea\x8e\xb4',
'YiSyllableSha': '\xea\x8e\xad',
'HangulSyllableKwab': '\xec\xbe\x81',
'HangulSyllableBbyeoss': '\xeb\xbc\x9c',
'CanadianSyllabicsYcreePwaa': '\xe1\x91\x88',
'YiSyllableShe': '\xea\x8e\xb8',
'LessthanAboveSlantedEqualAboveGreaterthanAboveSlantedEqual': '\xe2\xaa\x93',
'HangulSyllableKwag': '\xec\xbd\xb1',
'CjkUnifiedIdeograph47c8': '\xe4\x9f\x88',
'CjkUnifiedIdeograph47c9': '\xe4\x9f\x89',
'Fleurdelis': '\xe2\x9a\x9c',
'UnicodeCharacter0xf39f': '\xef\x8e\x9f',
'UnicodeCharacter0xf39d': '\xef\x8e\x9d',
'UnicodeCharacter0xf39e': '\xef\x8e\x9e',
'CjkUnifiedIdeograph47c0': '\xe4\x9f\x80',
'CjkUnifiedIdeograph47c1': '\xe4\x9f\x81',
'CjkUnifiedIdeograph47c2': '\xe4\x9f\x82',
'CjkUnifiedIdeograph47c3': '\xe4\x9f\x83',
'CjkUnifiedIdeograph47c4': '\xe4\x9f\x84',
'CjkUnifiedIdeograph47c5': '\xe4\x9f\x85',
'CjkUnifiedIdeograph47c6': '\xe4\x9f\x86',
'CjkUnifiedIdeograph47c7': '\xe4\x9f\x87',
'SimilarOrGreaterthan': '\xe2\xaa\x9e',
'MongolianLetterSibeDa': '\xe1\xa1\xa9',
'UnicodeCharacter0xe289': '\xee\x8a\x89',
'UnicodeCharacter0xe288': '\xee\x8a\x88',
'HangulSyllableWed': '\xec\x9b\xaf',
'UnicodeCharacter0xe280': '\xee\x8a\x80',
'UnicodeCharacter0xe283': '\xee\x8a\x83',
'UnicodeCharacter0xe282': '\xee\x8a\x82',
'UnicodeCharacter0xe285': '\xee\x8a\x85',
'UnicodeCharacter0xe284': '\xee\x8a\x84',
'UnicodeCharacter0xe287': '\xee\x8a\x87',
'UnicodeCharacter0xe286': '\xee\x8a\x86',
'UnicodeCharacter0xe133': '\xee\x84\xb3',
'CjkUnifiedIdeograph75e2': '\xe7\x97\xa2',
'HangulSyllableTyeoc': '\xed\x85\xbf',
'HangulSyllableTyeob': '\xed\x85\xb9',
'UnicodeCharacter0x9fe8': '\xe9\xbf\xa8',
'LogicalOrWithHorizontalDash': '\xe2\xa9\x9d',
'CjkUnifiedIdeograph79ab': '\xe7\xa6\xab',
'UnicodeCharacter0xe134': '\xee\x84\xb4',
'SquareAOverM': '\xe3\x8f\x9f',
'HangulSyllableTyeok': '\xed\x86\x80',
'HangulSyllableTyeoj': '\xed\x85\xbe',
'HangulSyllableTyeom': '\xed\x85\xb8',
'HangulSyllableTyeol': '\xed\x85\xb0',
'HangulSyllableSyogs': '\xec\x87\xbf',
'UnicodeCharacter0xf536': '\xef\x94\xb6',
'CjkUnifiedIdeograph865f': '\xe8\x99\x9f',
'CjkUnifiedIdeograph865e': '\xe8\x99\x9e',
'RunicLetterYr': '\xe1\x9a\xa3',
'CjkUnifiedIdeograph865c': '\xe8\x99\x9c',
'CjkUnifiedIdeograph865b': '\xe8\x99\x9b',
'CjkUnifiedIdeograph865a': '\xe8\x99\x9a',
'UnicodeCharacter0xf531': '\xef\x94\xb1',
'HangulSyllableSyogg': '\xec\x87\xbe',
'UnicodeCharacter0xf538': '\xef\x94\xb8',
'UnicodeCharacter0xf539': '\xef\x94\xb9',
'UnicodeCharacter0x1be2': '\xe1\xaf\xa2',
'UnicodeCharacter0x1be3': '\xe1\xaf\xa3',
'UnicodeCharacter0x1be0': '\xe1\xaf\xa0',
'UnicodeCharacter0x1be1': '\xe1\xaf\xa1',
'UnicodeCharacter0x1be6': '\xe1\xaf\xa6',
'UnicodeCharacter0x1be7': '\xe1\xaf\xa7',
'UnicodeCharacter0x1be4': '\xe1\xaf\xa4',
'UnicodeCharacter0x1be5': '\xe1\xaf\xa5',
'RightCeiling': '\xe2\x8c\x89',
'UnicodeCharacter0x1be8': '\xe1\xaf\xa8',
'UnicodeCharacter0x1be9': '\xe1\xaf\xa9',
'SyriacHarkleanObelus': '\xdc\x8b',
'HangulSyllableKelb': '\xec\xbc\x8b',
'HangulSyllableNwegs': '\xeb\x88\xbf',
'CjkUnifiedIdeograph8657': '\xe8\x99\x97',
'CjkUnifiedIdeograph8656': '\xe8\x99\x96',
'CjkUnifiedIdeograph8655': '\xe8\x99\x95',
'CjkUnifiedIdeograph8654': '\xe8\x99\x94',
'CjkUnifiedIdeograph8653': '\xe8\x99\x93',
'CjkUnifiedIdeograph8652': '\xe8\x99\x92',
'CjkUnifiedIdeograph8651': '\xe8\x99\x91',
'BoxDrawingsRightLightAndLeftUpHeavy': '\xe2\x94\xb9',
'CjkUnifiedIdeograph8659': '\xe8\x99\x99',
'CjkUnifiedIdeograph8658': '\xe8\x99\x98',
'UnicodeCharacter0xd7f6': '\xed\x9f\xb6',
'UnicodeCharacter0xe13b': '\xee\x84\xbb',
'KayahLiVowelE': '\xea\xa4\xa7',
'UnicodeCharacter0xe13f': '\xee\x84\xbf',
'HangulSyllableGgaj': '\xea\xb9\xa2',
'UnicodeCharacter0xe13d': '\xee\x84\xbd',
'KayahLiVowelO': '\xea\xa4\xaa',
'ArabicLigatureThehWithMeemFinalForm': '\xef\xb1\xb8',
'UnicodeCharacter0xa62c': '\xea\x98\xac',
'UnicodeCharacter0xdad9': '\xed\xab\x99',
'AllEqualTo': '\xe2\x89\x8c',
'UnicodeCharacter0xa62f': '\xea\x98\xaf',
'UnicodeCharacter0xa62e': '\xea\x98\xae',
'ThaiCharacterThoThan': '\xe0\xb8\x90',
'UnicodeCharacter0xd7f1': '\xed\x9f\xb1',
'UnicodeCharacter0x1beb': '\xe1\xaf\xab',
'UnicodeCharacter0x1bec': '\xe1\xaf\xac',
'UnicodeCharacter0xe91c': '\xee\xa4\x9c',
'UnicodeCharacter0x1bea': '\xe1\xaf\xaa',
'UnicodeCharacter0x1bef': '\xe1\xaf\xaf',
'UnicodeCharacter0xeba7': '\xee\xae\xa7',
'UnicodeCharacter0x1bed': '\xe1\xaf\xad',
'UnicodeCharacter0x1bee': '\xe1\xaf\xae',
'UnicodeCharacter0xeba8': '\xee\xae\xa8',
'VaiSyllableMbee': '\xea\x94\x88',
'NewTaiLueLetterHighPha': '\xe1\xa6\x95',
'DevanagariAbbreviationSign': '\xe0\xa5\xb0',
'TifinaghLetterYahh': '\xe2\xb5\x83',
'HangulSyllableRun': '\xeb\xa3\xac',
'UnicodeCharacter0xdad1': '\xed\xab\x91',
'UnicodeCharacter0xdad0': '\xed\xab\x90',
'KangxiRadicalColor': '\xe2\xbe\x8a',
'HangulSyllableBbyenj': '\xeb\xbc\xa9',
'HangulSyllableBbeonj': '\xeb\xbb\x95',
'YiSyllableKurx': '\xea\x88\x92',
'UnicodeCharacter0xe35a': '\xee\x8d\x9a',
'UnicodeCharacter0xdad2': '\xed\xab\x92',
'GreekCapitalLetterOmegaWithDasiaAndVaria': '\xe1\xbd\xab',
'CanadianSyllabicsOo': '\xe1\x90\x86',
'CanadianSyllabicsOn': '\xe1\x90\xad',
'UnicodeCharacter0xea5b': '\xee\xa9\x9b',
'UnicodeCharacter0xdad4': '\xed\xab\x94',
'UnicodeCharacter0xdde9': '\xed\xb7\xa9',
'UnicodeCharacter0xdde8': '\xed\xb7\xa8',
'CjkUnifiedIdeograph8b4b': '\xe8\xad\x8b',
'YiSyllableQiex': '\xea\x90\x81',
'ModifierLetterChineseToneYangQu': '\xea\x9c\x85',
'UnicodeCharacter0xdde3': '\xed\xb7\xa3',
'YiSyllableQiet': '\xea\x90\x80',
'UnicodeCharacter0xdde1': '\xed\xb7\xa1',
'UnicodeCharacter0xd886': '\xed\xa2\x86',
'UnicodeCharacter0xdde7': '\xed\xb7\xa7',
'YiSyllableQiep': '\xea\x90\x83',
'UnicodeCharacter0xdde5': '\xed\xb7\xa5',
'UnicodeCharacter0xdde4': '\xed\xb7\xa4',
'UnicodeCharacter0xda16': '\xed\xa8\x96',
'PhagspaLetterTha': '\xea\xa1\x89',
'TibetanSignLceTsaCan': '\xe0\xbe\x88',
'UnicodeCharacter0xa987': '\xea\xa6\x87',
'UnicodeCharacter0xda14': '\xed\xa8\x94',
'HangulSyllableJyeop': '\xec\xa1\x92',
'ArabicTailFragment': '\xef\xb9\xb3',
'YiSyllableQop': '\xea\x90\x8b',
'NewTaiLueToneMark1': '\xe1\xa7\x88',
'NewTaiLueToneMark2': '\xe1\xa7\x89',
'UnicodeCharacter0xda12': '\xed\xa8\x92',
'HebrewPointDageshOrMapiq': '\xd6\xbc',
'UnicodeCharacter0xfd91': '\xef\xb6\x91',
'UnicodeCharacter0xfd90': '\xef\xb6\x90',
'UnicodeCharacter0xda13': '\xed\xa8\x93',
'UnicodeCharacter0xe95c': '\xee\xa5\x9c',
':-)': '\xe2\x8c\xa3',
'Hypodiastole': '\xe2\xb8\x92',
'UnicodeCharacter0xddec': '\xed\xb7\xac',
'UnicodeCharacter0xdd2e': '\xed\xb4\xae',
'UnicodeCharacter0xddea': '\xed\xb7\xaa',
'CjkUnifiedIdeograph9bd0': '\xe9\xaf\x90',
'UnicodeCharacter0xddef': '\xed\xb7\xaf',
'UnicodeCharacter0xddee': '\xed\xb7\xae',
'UnicodeCharacter0xdded': '\xed\xb7\xad',
'FullwidthHyphenminus': '\xef\xbc\x8d',
'UnicodeCharacter0x9fe0': '\xe9\xbf\xa0',
'CjkUnifiedIdeograph731c': '\xe7\x8c\x9c',
'CjkUnifiedIdeograph731b': '\xe7\x8c\x9b',
'CjkUnifiedIdeograph731a': '\xe7\x8c\x9a',
'BraillePatternDots2345678': '\xe2\xa3\xbe',
'CjkUnifiedIdeograph731f': '\xe7\x8c\x9f',
'CjkUnifiedIdeograph731e': '\xe7\x8c\x9e',
'CjkUnifiedIdeograph731d': '\xe7\x8c\x9d',
'HangulSyllableJweolt': '\xec\xa4\xa5',
'UnicodeCharacter0xd92d': '\xed\xa4\xad',
'HangulSyllableJweols': '\xec\xa4\xa4',
'HangulSyllableJweolp': '\xec\xa4\xa6',
'UnicodeCharacter0xe0a4': '\xee\x82\xa4',
'UnicodeCharacter0xe0a5': '\xee\x82\xa5',
'UnicodeCharacter0xe0a6': '\xee\x82\xa6',
'HangulSyllableJweolm': '\xec\xa4\xa2',
'UnicodeCharacter0xe0a0': '\xee\x82\xa0',
'UnicodeCharacter0xda18': '\xed\xa8\x98',
'HangulSyllableJweolh': '\xec\xa4\xa7',
'UnicodeCharacter0xe0a3': '\xee\x82\xa3',
'UnicodeCharacter0xe35e': '\xee\x8d\x9e',
'HangulSyllableJweolg': '\xec\xa4\xa1',
'CjkUnifiedIdeograph9bd8': '\xe9\xaf\x98',
'HangulSyllableJweolb': '\xec\xa4\xa3',
'UnicodeCharacter0xe0a9': '\xee\x82\xa9',
'CombiningRightHalfRingAbove': '\xcd\x97',
'CjkUnifiedIdeograph456e': '\xe4\x95\xae',
'CjkUnifiedIdeograph456d': '\xe4\x95\xad',
'CjkUnifiedIdeograph8f4f': '\xe8\xbd\x8f',
'CjkUnifiedIdeograph456f': '\xe4\x95\xaf',
'CjkUnifiedIdeograph456a': '\xe4\x95\xaa',
'CjkUnifiedIdeograph8f4a': '\xe8\xbd\x8a',
'CjkUnifiedIdeograph456c': '\xe4\x95\xac',
'CjkUnifiedIdeograph456b': '\xe4\x95\xab',
'UnicodeCharacter0xede4': '\xee\xb7\xa4',
'UnicodeCharacter0xe074': '\xee\x81\xb4',
'CjkUnifiedIdeograph9018': '\xe9\x80\x98',
'CjkUnifiedIdeograph9019': '\xe9\x80\x99',
'CjkUnifiedIdeograph3af8': '\xe3\xab\xb8',
'CjkUnifiedIdeograph3af9': '\xe3\xab\xb9',
'UnicodeCharacter0xe075': '\xee\x81\xb5',
'CjkUnifiedIdeograph9ad8': '\xe9\xab\x98',
'CopticSmallLetterPsi': '\xe2\xb2\xaf',
'CjkUnifiedIdeograph3af0': '\xe3\xab\xb0',
'CjkUnifiedIdeograph3af1': '\xe3\xab\xb1',
'CjkUnifiedIdeograph3af2': '\xe3\xab\xb2',
'CjkUnifiedIdeograph3af3': '\xe3\xab\xb3',
'CjkUnifiedIdeograph3af4': '\xe3\xab\xb4',
'CjkUnifiedIdeograph3af5': '\xe3\xab\xb5',
'CjkUnifiedIdeograph3af6': '\xe3\xab\xb6',
'CjkUnifiedIdeograph3af7': '\xe3\xab\xb7',
'UnicodeCharacter0xe0ad': '\xee\x82\xad',
'HangulSyllableNut': '\xeb\x88\x9d',
'UnicodeCharacter0xe0af': '\xee\x82\xaf',
'HangulSyllableNup': '\xeb\x88\x9e',
'HangulSyllableNus': '\xeb\x88\x97',
'UnicodeCharacter0xe0ac': '\xee\x82\xac',
'HangulSyllableJjweot': '\xec\xad\xbd',
'CjkUnifiedIdeograph7927': '\xe7\xa4\xa7',
'ArabicLigatureThehWithYehIsolatedForm': '\xef\xb0\x94',
'HangulSyllableJjweos': '\xec\xad\xb7',
'HangulSyllableJjweop': '\xec\xad\xbe',
'CjkUnifiedIdeograph9dd4': '\xe9\xb7\x94',
'MyanmarSignShanCouncilTone2': '\xe1\x82\x8b',
'MyanmarSignShanCouncilTone3': '\xe1\x82\x8c',
'CjkUnifiedIdeograph7311': '\xe7\x8c\x91',
'CjkUnifiedIdeograph7310': '\xe7\x8c\x90',
'CjkUnifiedIdeograph7317': '\xe7\x8c\x97',
'CjkUnifiedIdeograph7316': '\xe7\x8c\x96',
'CjkUnifiedIdeograph7315': '\xe7\x8c\x95',
'CjkUnifiedIdeograph7314': '\xe7\x8c\x94',
'HangulSyllableNum': '\xeb\x88\x94',
'ArabicLetterRehWithSmallVBelow': '\xda\x95',
'CjkUnifiedIdeograph7319': '\xe7\x8c\x99',
'CjkUnifiedIdeograph7318': '\xe7\x8c\x98',
'HangulSyllableJjweob': '\xec\xad\xb5',
'MongolianLetterSa': '\xe1\xa0\xb0',
'CjkUnifiedIdeograph3a04': '\xe3\xa8\x84',
'HangulSyllableNuj': '\xeb\x88\x9a',
'UnicodeCharacter0xdd2a': '\xed\xb4\xaa',
'HangulSyllableTyul': '\xed\x8a\xa4',
'UnicodeCharacter0xf764': '\xef\x9d\xa4',
'CjkUnifiedIdeograph9add': '\xe9\xab\x9d',
'CjkUnifiedIdeograph3afa': '\xe3\xab\xba',
'CjkUnifiedIdeograph3afb': '\xe3\xab\xbb',
'CjkUnifiedIdeograph3afc': '\xe3\xab\xbc',
'CjkUnifiedIdeograph3afd': '\xe3\xab\xbd',
'CjkUnifiedIdeograph3afe': '\xe3\xab\xbe',
'CjkUnifiedIdeograph3aff': '\xe3\xab\xbf',
'CjkUnifiedIdeograph9adc': '\xe9\xab\x9c',
'CjkUnifiedIdeograph4565': '\xe4\x95\xa5',
'CjkUnifiedIdeograph4564': '\xe4\x95\xa4',
'CjkUnifiedIdeograph4567': '\xe4\x95\xa7',
'CjkUnifiedIdeograph4566': '\xe4\x95\xa6',
'CjkUnifiedIdeograph4561': '\xe4\x95\xa1',
'CjkUnifiedIdeograph4560': '\xe4\x95\xa0',
'CjkUnifiedIdeograph4563': '\xe4\x95\xa3',
'CjkUnifiedIdeograph4562': '\xe4\x95\xa2',
'CjkUnifiedIdeograph4569': '\xe4\x95\xa9',
'CjkUnifiedIdeograph4568': '\xe4\x95\xa8',
'YiSyllableChuo': '\xea\x8d\xa7',
'HangulSyllableHwags': '\xed\x99\x97',
'UnicodeCharacter0xde39': '\xed\xb8\xb9',
'CjkUnifiedIdeograph58da': '\xe5\xa3\x9a',
'UnicodeCharacter0xde35': '\xed\xb8\xb5',
'CjkUnifiedIdeograph58dc': '\xe5\xa3\x9c',
'CjkUnifiedIdeograph58db': '\xe5\xa3\x9b',
'CjkUnifiedIdeograph58de': '\xe5\xa3\x9e',
'CjkUnifiedIdeograph58dd': '\xe5\xa3\x9d',
'OriyaAiLengthMark': '\xe0\xad\x96',
'CjkUnifiedIdeograph58df': '\xe5\xa3\x9f',
'YiSyllableNyuox': '\xea\x91\x92',
'YiSyllableChux': '\xea\x8d\xb1',
'YiSyllableNyuop': '\xea\x91\x94',
'HebrewAccentShalshelet': '\xd6\x93',
'YiSyllableChur': '\xea\x8d\xb5',
'YiSyllableChup': '\xea\x8d\xb3',
'CjkRadicalMesh': '\xe2\xba\xb5',
'LatinSmallLetterTz': '\xea\x9c\xa9',
'HangulSyllableMaebs': '\xeb\xa7\xb6',
'UnicodeCharacter0xec5f': '\xee\xb1\x9f',
'CjkUnifiedIdeograph521d': '\xe5\x88\x9d',
'CjkUnifiedIdeograph521e': '\xe5\x88\x9e',
'CjkUnifiedIdeograph521f': '\xe5\x88\x9f',
'UnicodeCharacter0xe3ad': '\xee\x8e\xad',
'UnicodeCharacter0xe3ac': '\xee\x8e\xac',
'CjkUnifiedIdeograph521a': '\xe5\x88\x9a',
'CjkUnifiedIdeograph521b': '\xe5\x88\x9b',
'CjkUnifiedIdeograph521c': '\xe5\x88\x9c',
'StaffOfAesculapius': '\xe2\x9a\x95',
'ArabicSubscriptAlef': '\xd9\x96',
'UnicodeCharacter0xec5d': '\xee\xb1\x9d',
'UnicodeCharacter0xdadc': '\xed\xab\x9c',
'HangulSyllablePwa': '\xed\x90\x88',
'UnicodeCharacter0xede5': '\xee\xb7\xa5',
'CjkUnifiedIdeograph9b1c': '\xe9\xac\x9c',
'HangulSyllablePwe': '\xed\x92\xb0',
'UnicodeCharacter0xee44': '\xee\xb9\x84',
'HangulSyllablePwi': '\xed\x93\x8c',
'UnicodeCharacter0xdade': '\xed\xab\x9e',
'CanadianSyllabicsWestcreeMwii': '\xe1\x92\xb1',
'KatakanaLetterSi': '\xe3\x82\xb7',
'UnicodeCharacter0xdadd': '\xed\xab\x9d',
'CjkUnifiedIdeograph58d9': '\xe5\xa3\x99',
'CjkUnifiedIdeograph58d8': '\xe5\xa3\x98',
'YiSyllableVie': '\xea\x83\xa8',
'GlagoliticCapitalLetterTrokutastiA': '\xe2\xb0\xad',
'CjkUnifiedIdeograph58d1': '\xe5\xa3\x91',
'CjkUnifiedIdeograph58d0': '\xe5\xa3\x90',
'CjkUnifiedIdeograph58d3': '\xe5\xa3\x93',
'CjkUnifiedIdeograph58d2': '\xe5\xa3\x92',
'CjkUnifiedIdeograph58d5': '\xe5\xa3\x95',
'GlagoliticCapitalLetterYati': '\xe2\xb0\xa1',
'CjkUnifiedIdeograph58d7': '\xe5\xa3\x97',
'CjkUnifiedIdeograph58d6': '\xe5\xa3\x96',
'CjkUnifiedIdeograph5214': '\xe5\x88\x94',
'CjkUnifiedIdeograph5215': '\xe5\x88\x95',
'CjkUnifiedIdeograph5216': '\xe5\x88\x96',
'CjkUnifiedIdeograph5217': '\xe5\x88\x97',
'CjkUnifiedIdeograph5210': '\xe5\x88\x90',
'CjkUnifiedIdeograph5211': '\xe5\x88\x91',
'CjkUnifiedIdeograph5212': '\xe5\x88\x92',
'CjkUnifiedIdeograph5213': '\xe5\x88\x93',
'GujaratiDigitSeven': '\xe0\xab\xad',
'CjkUnifiedIdeograph5218': '\xe5\x88\x98',
'CjkUnifiedIdeograph5219': '\xe5\x88\x99',
'TifinaghLetterAyerYagh': '\xe2\xb5\x98',
'UnicodeCharacter0xe3a8': '\xee\x8e\xa8',
'HangulSyllableDyaec': '\xeb\x8d\x8f',
'HangulSyllableDyaeb': '\xeb\x8d\x89',
'RightwardsArrowWithStroke': '\xe2\x86\x9b',
'HangulSyllableDyaeg': '\xeb\x8c\xb9',
'HalfwidthIdeographicComma': '\xef\xbd\xa4',
'HangulSyllableDyaed': '\xeb\x8c\xbf',
'HangulSyllableDyaek': '\xeb\x8d\x90',
'HangulSyllableDyaej': '\xeb\x8d\x8e',
'HangulSyllableDyaeh': '\xeb\x8d\x93',
'KatakanaLetterSa': '\xe3\x82\xb5',
'HangulSyllableDyaem': '\xeb\x8d\x88',
'HangulSyllableDyael': '\xeb\x8d\x80',
'ArabicLigatureKafWithKhahInitialForm': '\xef\xb3\x86',
'CjkUnifiedIdeograph88ee': '\xe8\xa3\xae',
'CjkUnifiedIdeograph88ed': '\xe8\xa3\xad',
'CjkUnifiedIdeograph88ef': '\xe8\xa3\xaf',
'CjkUnifiedIdeograph88ea': '\xe8\xa3\xaa',
'CanadianSyllabicsYcreeSoo': '\xe1\x93\xb3',
'CjkUnifiedIdeograph88ec': '\xe8\xa3\xac',
'CjkUnifiedIdeograph88eb': '\xe8\xa3\xab',
'CjkUnifiedIdeograph689c': '\xe6\xa2\x9c',
'CjkUnifiedIdeograph689b': '\xe6\xa2\x9b',
'CjkUnifiedIdeograph689a': '\xe6\xa2\x9a',
'CjkUnifiedIdeograph7ed9': '\xe7\xbb\x99',
'UnicodeCharacter0xdc94': '\xed\xb2\x94',
'CjkUnifiedIdeograph689f': '\xe6\xa2\x9f',
'LatinSmallLetterTWithCommaBelow': '\xc8\x9b',
'CjkUnifiedIdeograph689d': '\xe6\xa2\x9d',
'UnicodeCharacter0xdc98': '\xed\xb2\x98',
'UnicodeCharacter0xdc99': '\xed\xb2\x99',
'HangulSyllableSwaelg': '\xec\x87\x8d',
'DotPlus': '\xe2\x88\x94',
'CjkUnifiedIdeograph6549': '\xe6\x95\x89',
'CjkUnifiedIdeograph6548': '\xe6\x95\x88',
'HangulSyllableTyinh': '\xed\x8b\x9a',
'UnicodeCharacter0xf4f7': '\xef\x93\xb7',
'UnicodeCharacter0xf4f0': '\xef\x93\xb0',
'UnicodeCharacter0xf4f1': '\xef\x93\xb1',
'UnicodeCharacter0xf4f2': '\xef\x93\xb2',
'HangulSyllableByils': '\xeb\xb8\xb4',
'SquareMaruku': '\xe3\x8d\x86',
'CjkUnifiedIdeograph6540': '\xe6\x95\x80',
'CjkUnifiedIdeograph6543': '\xe6\x95\x83',
'CjkUnifiedIdeograph6542': '\xe6\x95\x82',
'CjkUnifiedIdeograph6545': '\xe6\x95\x85',
'CjkUnifiedIdeograph6544': '\xe6\x95\x84',
'CjkUnifiedIdeograph6547': '\xe6\x95\x87',
'CjkUnifiedIdeograph6546': '\xe6\x95\x86',
'CjkUnifiedIdeograph6893': '\xe6\xa2\x93',
'CjkUnifiedIdeograph6892': '\xe6\xa2\x92',
'CjkUnifiedIdeograph6891': '\xe6\xa2\x91',
'CjkUnifiedIdeograph6890': '\xe6\xa2\x90',
'CjkUnifiedIdeograph6897': '\xe6\xa2\x97',
'GurmukhiLetterSha': '\xe0\xa8\xb6',
'CjkUnifiedIdeograph6895': '\xe6\xa2\x95',
'OghamSpaceMark': '\xe1\x9a\x80',
'CjkUnifiedIdeograph6899': '\xe6\xa2\x99',
'CjkUnifiedIdeograph6898': '\xe6\xa2\x98',
'UnicodeCharacter0xe07d': '\xee\x81\xbd',
'YiRadicalSsi': '\xea\x92\x95',
'HangulSyllableDdeud': '\xeb\x9c\xaf',
'HangulSyllableDdeug': '\xeb\x9c\xa9',
'CjkUnifiedIdeograph88e9': '\xe8\xa3\xa9',
'CjkUnifiedIdeograph88e8': '\xe8\xa3\xa8',
'HangulSyllableDdeub': '\xeb\x9c\xb9',
'HangulSyllableDdeuc': '\xeb\x9c\xbf',
'CjkUnifiedIdeograph88e5': '\xe8\xa3\xa5',
'CjkUnifiedIdeograph88e4': '\xe8\xa3\xa4',
'CjkUnifiedIdeograph88e7': '\xe8\xa3\xa7',
'CjkUnifiedIdeograph88e6': '\xe8\xa3\xa6',
'CjkUnifiedIdeograph88e1': '\xe8\xa3\xa1',
'CjkUnifiedIdeograph88e0': '\xe8\xa3\xa0',
'CjkUnifiedIdeograph88e3': '\xe8\xa3\xa3',
'CjkUnifiedIdeograph88e2': '\xe8\xa3\xa2',
'UnicodeCharacter0xf4fd': '\xef\x93\xbd',
'UnicodeCharacter0xf4fe': '\xef\x93\xbe',
'UnicodeCharacter0xf4ff': '\xef\x93\xbf',
'UnicodeCharacter0xf4fa': '\xef\x93\xba',
'CjkUnifiedIdeograph9d19': '\xe9\xb4\x99',
'LefttorightEmbedding': '\xe2\x80\xaa',
'CjkUnifiedIdeograph654a': '\xe6\x95\x8a',
'CjkUnifiedIdeograph654c': '\xe6\x95\x8c',
'CjkUnifiedIdeograph654b': '\xe6\x95\x8b',
'CjkUnifiedIdeograph654e': '\xe6\x95\x8e',
'CjkUnifiedIdeograph654d': '\xe6\x95\x8d',
'CjkUnifiedIdeograph654f': '\xe6\x95\x8f',
'SyriacLetterWaw': '\xdc\x98',
'RunicLetterAlgizEolhx': '\xe1\x9b\x89',
'GreekSmallLetterAlphaWithMacron': '\xe1\xbe\xb1',
'FacsimileSign': '\xe2\x84\xbb',
'BengaliDigitFour': '\xe0\xa7\xaa',
'UnicodeCharacter0xaad4': '\xea\xab\x94',
'VaiSyllableMgboo': '\xea\x95\xbb',
'CopticCapitalLetterLaula': '\xe2\xb2\x96',
'BopomofoLetterUnn': '\xe3\x86\xab',
'UnicodeCharacter0xec58': '\xee\xb1\x98',
'CjkUnifiedIdeograph3c08': '\xe3\xb0\x88',
'CjkUnifiedIdeograph3c09': '\xe3\xb0\x89',
'LatinSmallLetterLWithHighStroke': '\xea\x9d\x89',
'CjkUnifiedIdeograph3c04': '\xe3\xb0\x84',
'CjkUnifiedIdeograph3c05': '\xe3\xb0\x85',
'CjkUnifiedIdeograph3c06': '\xe3\xb0\x86',
'CjkUnifiedIdeograph3c07': '\xe3\xb0\x87',
'CjkUnifiedIdeograph3c00': '\xe3\xb0\x80',
'CjkUnifiedIdeograph3c01': '\xe3\xb0\x81',
'CjkUnifiedIdeograph3c02': '\xe3\xb0\x82',
'CjkUnifiedIdeograph3c03': '\xe3\xb0\x83',
'UnicodeCharacter0xda1f': '\xed\xa8\x9f',
'HangulSyllableRyul': '\xeb\xa5\xa0',
'CjkUnifiedIdeograph9bdf': '\xe9\xaf\x9f',
'CjkUnifiedIdeograph9bde': '\xe9\xaf\x9e',
'HangulSyllableByilg': '\xeb\xb8\xb1',
'CjkUnifiedIdeograph9bdd': '\xe9\xaf\x9d',
'ModifierLetterSmallSchwa': '\xe1\xb5\x8a',
'UnicodeCharacter0xda1b': '\xed\xa8\x9b',
'MyanmarLetterTtha': '\xe1\x80\x8c',
'UnicodeCharacter0xda1c': '\xed\xa8\x9c',
'HangulSyllableAeb': '\xec\x95\xb1',
'HangulSyllableAec': '\xec\x95\xb7',
'CjkUnifiedIdeograph3c0d': '\xe3\xb0\x8d',
'CjkUnifiedIdeograph3c0e': '\xe3\xb0\x8e',
'CjkUnifiedIdeograph3c0f': '\xe3\xb0\x8f',
'CjkUnifiedIdeograph3c0a': '\xe3\xb0\x8a',
'CjkUnifiedIdeograph3c0b': '\xe3\xb0\x8b',
'CjkUnifiedIdeograph3c0c': '\xe3\xb0\x8c',
'UnicodeCharacter0xe9c5': '\xee\xa7\x85',
'HangulSyllableJyeoss': '\xec\xa1\x8c',
'BoxDrawingsLightTripleDashVertical': '\xe2\x94\x86',
'HangulSyllableKod': '\xec\xbd\x9b',
'UnicodeCharacter0xef6f': '\xee\xbd\xaf',
'ErrorbarredWhiteDiamond': '\xe2\xa7\xb0',
'YiSyllableZix': '\xea\x8a\x8e',
'HangulSyllableAed': '\xec\x95\xa7',
'LatinSmallLetterHWithDescender': '\xe2\xb1\xa8',
'YiSyllableZip': '\xea\x8a\x90',
'UnicodeCharacter0xe3a0': '\xee\x8e\xa0',
'YiSyllableZit': '\xea\x8a\x8d',
'ArabicLetterKhahInitialForm': '\xef\xba\xa7',
'GujaratiSignAvagraha': '\xe0\xaa\xbd',
'RecyclingSymbolForType4Plastics': '\xe2\x99\xb6',
'YiSyllableZie': '\xea\x8a\x92',
'CanadianSyllabicsCarrierKha': '\xe1\x97\xba',
'CanadianSyllabicsCarrierKhe': '\xe1\x97\xb7',
'MongolianLetterAliGaliA': '\xe1\xa2\x87',
'CanadianSyllabicsCarrierKhi': '\xe1\x97\xb9',
'TamilLetterHa': '\xe0\xae\xb9',
'LeftSemidirectProduct': '\xe2\x8b\x8b',
'th': '\xce\xb8',
'MongolianLetterAliGaliI': '\xe1\xa2\x88',
'CanadianSyllabicsCarrierKho': '\xe1\x97\xb6',
'CanadianSyllabicsCarrierKhu': '\xe1\x97\xb5',
'ChamLetterVa': '\xea\xa8\xa5',
'HangulSyllableAem': '\xec\x95\xb0',
'LeftRightOpenheadedArrow': '\xe2\x87\xbf',
'HangulSyllableKyess': '\xec\xbd\x8c',
'GreekCapitalLetterEtaWithDasiaAndPerispomeniAndProsgegrammeni': '\xe1\xbe\x9f',
'HangulSyllableAes': '\xec\x95\xb3',
'CjkUnifiedIdeograph7e5e': '\xe7\xb9\x9e',
'CjkUnifiedIdeograph7e5d': '\xe7\xb9\x9d',
'CjkUnifiedIdeograph7e5f': '\xe7\xb9\x9f',
'CjkUnifiedIdeograph7e5a': '\xe7\xb9\x9a',
'CjkUnifiedIdeograph7e5c': '\xe7\xb9\x9c',
'CjkUnifiedIdeograph7e5b': '\xe7\xb9\x9b',
'CjkUnifiedIdeograph6e56': '\xe6\xb9\x96',
'CjkUnifiedIdeograph6e57': '\xe6\xb9\x97',
'CjkUnifiedIdeograph6e54': '\xe6\xb9\x94',
'CjkUnifiedIdeograph6e55': '\xe6\xb9\x95',
'CjkUnifiedIdeograph6e52': '\xe6\xb9\x92',
'CjkUnifiedIdeograph6e53': '\xe6\xb9\x93',
'CjkUnifiedIdeograph6e50': '\xe6\xb9\x90',
'CjkUnifiedIdeograph6e51': '\xe6\xb9\x91',
'GujaratiLetterGa': '\xe0\xaa\x97',
'UnicodeCharacter0x9fb': '\xe0\xa7\xbb',
'UnicodeCharacter0x9fc': '\xe0\xa7\xbc',
'PlusSignWithTildeAbove': '\xe2\xa8\xa4',
'UnicodeCharacter0x9fe': '\xe0\xa7\xbe',
'UnicodeCharacter0x9ff': '\xe0\xa7\xbf',
'CanadianSyllabicsShoo': '\xe1\x94\x94',
'IdeographicDescriptionCharacterSurroundFromLeft': '\xe2\xbf\xb7',
'YiSyllableDdie': '\xea\x84\xbb',
'MyanmarLetterMonE': '\xe1\x80\xa8',
'HangulSyllableAet': '\xec\x95\xb9',
'SaurashtraVowelSignE': '\xea\xa2\xbe',
'SaurashtraVowelSignI': '\xea\xa2\xb6',
'SaurashtraVowelSignO': '\xea\xa3\x81',
'SmallRomanNumeralTwelve': '\xe2\x85\xbb',
'SaurashtraVowelSignU': '\xea\xa2\xb8',
'ParenthesizedIdeographLabor': '\xe3\x88\xb8',
'TibetanVowelSignOo': '\xe0\xbd\xbd',
'CjkUnifiedIdeograph6e5f': '\xe6\xb9\x9f',
'CjkUnifiedIdeograph6e5d': '\xe6\xb9\x9d',
'CjkUnifiedIdeograph6e5e': '\xe6\xb9\x9e',
'CjkUnifiedIdeograph6e5b': '\xe6\xb9\x9b',
'CjkUnifiedIdeograph6e5c': '\xe6\xb9\x9c',
'CjkUnifiedIdeograph6e5a': '\xe6\xb9\x9a',
'HangulSyllableMeulb': '\xeb\xaf\x8b',
'KangxiRadicalSnout': '\xe2\xbc\xb9',
'HangulSyllableMeulm': '\xeb\xaf\x8a',
'UnicodeCharacter0xfe1c': '\xef\xb8\x9c',
'CjkUnifiedIdeograph7e59': '\xe7\xb9\x99',
'CjkUnifiedIdeograph7e58': '\xe7\xb9\x98',
'SaurashtraVowelSignUu': '\xea\xa2\xb9',
'WritingHand': '\xe2\x9c\x8d',
'CjkUnifiedIdeograph7e54': '\xe7\xb9\x94',
'CjkUnifiedIdeograph7e57': '\xe7\xb9\x97',
'CjkUnifiedIdeograph7e56': '\xe7\xb9\x96',
'CjkUnifiedIdeograph7e51': '\xe7\xb9\x91',
'CjkUnifiedIdeograph7e50': '\xe7\xb9\x90',
'TwoConsecutiveEqualsSigns': '\xe2\xa9\xb5',
'CjkUnifiedIdeograph7e52': '\xe7\xb9\x92',
'CjkUnifiedIdeograph4a69': '\xe4\xa9\xa9',
'CjkUnifiedIdeograph4a68': '\xe4\xa9\xa8',
'CjkUnifiedIdeograph5a4b': '\xe5\xa9\x8b',
'CircledIdeographSecret': '\xe3\x8a\x99',
'CjkUnifiedIdeograph5a4d': '\xe5\xa9\x8d',
'CjkUnifiedIdeograph5a4e': '\xe5\xa9\x8e',
'CjkUnifiedIdeograph5a4f': '\xe5\xa9\x8f',
'HangulSyllableGgyonj': '\xea\xbe\xa1',
'CjkUnifiedIdeograph4a61': '\xe4\xa9\xa1',
'CjkUnifiedIdeograph4a60': '\xe4\xa9\xa0',
'CjkUnifiedIdeograph4a63': '\xe4\xa9\xa3',
'CjkUnifiedIdeograph4a62': '\xe4\xa9\xa2',
'CjkUnifiedIdeograph4a65': '\xe4\xa9\xa5',
'CjkUnifiedIdeograph4a64': '\xe4\xa9\xa4',
'CjkUnifiedIdeograph4a67': '\xe4\xa9\xa7',
'CjkUnifiedIdeograph4a66': '\xe4\xa9\xa6',
'HangulSyllableJoj': '\xec\xa2\x86',
'FullwidthLatinCapitalLetterJ': '\xef\xbc\xaa',
'HangulSyllableNuss': '\xeb\x88\x98',
'HangulSyllableSsun': '\xec\x91\xa8',
'HangulSyllableKyubs': '\xed\x81\xa2',
'EthiopicSyllableNyee': '\xe1\x8a\x9c',
'CjkUnifiedIdeograph69c9': '\xe6\xa7\x89',
'CanadianSyllabicsRa': '\xe1\x95\x8b',
'CjkCompatibilityIdeographf91c': '\xef\xa4\x9c',
'CanadianSyllabicsRe': '\xe1\x95\x82',
'ChamLetterDha': '\xea\xa8\x96',
'CanadianSyllabicsRi': '\xe1\x95\x86',
'HangulSyllableTyup': '\xed\x8a\xb6',
'HangulSyllableSsub': '\xec\x91\xb5',
'NkoLajanyalan': '\xdf\xba',
'CanadianSyllabicsRo': '\xe1\x95\x88',
'HangulSyllableSsuc': '\xec\x91\xbb',
'UnicodeCharacter0xf7af': '\xef\x9e\xaf',
'HangulSyllableSsud': '\xec\x91\xab',
'UnicodeCharacter0xfd4d': '\xef\xb5\x8d',
'CjkUnifiedIdeograph5a40': '\xe5\xa9\x80',
'CjkUnifiedIdeograph5a41': '\xe5\xa9\x81',
'CjkUnifiedIdeograph5a42': '\xe5\xa9\x82',
'CjkUnifiedIdeograph5a43': '\xe5\xa9\x83',
'CjkUnifiedIdeograph5a44': '\xe5\xa9\x84',
'CjkUnifiedIdeograph5a45': '\xe5\xa9\x85',
'CjkUnifiedIdeograph5a46': '\xe5\xa9\x86',
'KhmerVowelSignIi': '\xe1\x9e\xb8',
'CjkUnifiedIdeograph4a6a': '\xe4\xa9\xaa',
'CjkUnifiedIdeograph5a49': '\xe5\xa9\x89',
'CjkUnifiedIdeograph4a6c': '\xe4\xa9\xac',
'KhmerVowelSignIe': '\xe1\x9f\x80',
'CjkUnifiedIdeograph4a6e': '\xe4\xa9\xae',
'CjkUnifiedIdeograph4a6d': '\xe4\xa9\xad',
'CjkUnifiedIdeograph4a6f': '\xe4\xa9\xaf',
'HexagramForYouthfulFolly': '\xe4\xb7\x83',
'CjkUnifiedIdeograph60ef': '\xe6\x83\xaf',
'HangulSyllableHyang': '\xed\x96\xa5',
'CjkUnifiedIdeograph6fc8': '\xe6\xbf\x88',
'ArabicLigatureTehWithAlefMaksuraFinalForm': '\xef\xb1\xb4',
'UpTackWithCircleAbove': '\xe2\x9f\x9f',
'CjkUnifiedIdeograph92fc': '\xe9\x8b\xbc',
'YiSyllableVip': '\xea\x83\xa5',
'HangulSyllableRuss': '\xeb\xa3\xbc',
'HangulSyllableNyel': '\xeb\x85\xa4',
'UnicodeCharacter0xede1': '\xee\xb7\xa1',
'HangulSyllableSsus': '\xec\x91\xb7',
'NorthWestArrow': '\xe2\x86\x96',
'imj': '\xe2\x85\x89',
'HangulSyllableRwiss': '\xeb\xa5\x90',
'HangulSyllableHyanh': '\xed\x96\x96',
'HangulSyllableJjoss': '\xec\xab\x90',
'CjkCompatibilityIdeographfa7b': '\xef\xa9\xbb',
'BoxDrawingsDownHeavyAndLeftLight': '\xe2\x94\x92',
'CjkCompatibilityIdeographfa7c': '\xef\xa9\xbc',
'CjkUnifiedIdeograph6fc5': '\xe6\xbf\x85',
'UnicodeCharacter0xf690': '\xef\x9a\x90',
'ThaanaUbufili': '\xde\xaa',
'LatinSmallLetterEWithCircumflex': '\xc3\xaa',
'HangulSyllableJod': '\xec\xa1\xb7',
'CjkCompatibilityIdeographfa7f': '\xef\xa9\xbf',
'CjkUnifiedIdeograph8fcd': '\xe8\xbf\x8d',
'HangulSyllableDweop': '\xeb\x92\x86',
'HangulSyllableBweolt': '\xeb\xb6\xa9',
'HangulSyllableBweols': '\xeb\xb6\xa8',
'HangulSyllableBweolp': '\xeb\xb6\xaa',
'UnicodeCharacter0xe69f': '\xee\x9a\x9f',
'HangulSyllableTuk': '\xed\x89\x84',
'HangulSyllableBweolg': '\xeb\xb6\xa5',
'ModifierLetterChineseToneYinPing': '\xea\x9c\x80',
'BraillePatternDots5678': '\xe2\xa3\xb0',
'ccwint': '\xe2\x88\xb2',
'HangulSyllableBweolm': '\xeb\xb6\xa6',
'UnicodeCharacter0xf33b': '\xef\x8c\xbb',
'UnicodeCharacter0xaa39': '\xea\xa8\xb9',
'CjkUnifiedIdeograph8495': '\xe8\x92\x95',
'CjkUnifiedIdeograph8494': '\xe8\x92\x94',
'CjkUnifiedIdeograph8497': '\xe8\x92\x97',
'CjkUnifiedIdeograph8496': '\xe8\x92\x96',
'CjkUnifiedIdeograph8491': '\xe8\x92\x91',
'CjkUnifiedIdeograph8490': '\xe8\x92\x90',
'CjkUnifiedIdeograph8493': '\xe8\x92\x93',
'CjkUnifiedIdeograph8492': '\xe8\x92\x92',
'CjkUnifiedIdeograph8499': '\xe8\x92\x99',
'CjkUnifiedIdeograph8498': '\xe8\x92\x98',
'HangulSyllableBbyagg': '\xeb\xba\x9a',
'CjkUnifiedIdeograph8fcf': '\xe8\xbf\x8f',
'UnicodeCharacter0xf33c': '\xef\x8c\xbc',
'UnicodeCharacter0xaa38': '\xea\xa8\xb8',
'CjkRadicalSheep': '\xe2\xba\xb6',
'HangulSyllableJwelm': '\xec\xa4\xbe',
'HangulSyllableGaelh': '\xea\xb0\xab',
'HangulSyllableJwelh': '\xec\xa5\x83',
'CjkUnifiedIdeograph8ed9': '\xe8\xbb\x99',
'CjkUnifiedIdeograph8ed8': '\xe8\xbb\x98',
'CjkUnifiedIdeograph8ed7': '\xe8\xbb\x97',
'CjkUnifiedIdeograph8ed6': '\xe8\xbb\x96',
'CjkUnifiedIdeograph8ed5': '\xe8\xbb\x95',
'EthiopicSyllableSsee': '\xe2\xb6\xa4',
'CjkUnifiedIdeograph8ed3': '\xe8\xbb\x93',
'CjkUnifiedIdeograph8ed2': '\xe8\xbb\x92',
'CjkUnifiedIdeograph8ed1': '\xe8\xbb\x91',
'CjkUnifiedIdeograph8ed0': '\xe8\xbb\x90',
'CjkUnifiedIdeograph99dd': '\xe9\xa7\x9d',
'UnicodeCharacter0xc84': '\xe0\xb2\x84',
'UnicodeCharacter0xc80': '\xe0\xb2\x80',
'UnicodeCharacter0xc81': '\xe0\xb2\x81',
'HangulSyllableJjyigg': '\xec\xaf\xb2',
'CjkUnifiedIdeograph9046': '\xe9\x81\x86',
'VaiSyllableBhe': '\xea\x97\xa9',
'CjkUnifiedIdeograph9190': '\xe9\x86\x90',
'VaiSyllableBha': '\xea\x95\x91',
'UnicodeCharacter0x83': '\xc2\x83',
'VaiSyllableBho': '\xea\x97\x82',
'VaiSyllableBhi': '\xea\x94\xab',
'PresentationFormForVerticalEmDash': '\xef\xb8\xb1',
'CjkUnifiedIdeograph9196': '\xe9\x86\x96',
'CjkUnifiedIdeograph849e': '\xe8\x92\x9e',
'CjkUnifiedIdeograph75ee': '\xe7\x97\xae',
'CanadianSyllabicsWestcreeLwe': '\xe1\x93\x9d',
'CjkUnifiedIdeograph849a': '\xe8\x92\x9a',
'HangulSyllableKeugs': '\xed\x81\xaf',
'CjkUnifiedIdeograph849c': '\xe8\x92\x9c',
'CanadianSyllabicsWestcreeLwa': '\xe1\x93\xa7',
'KangxiRadicalTripod': '\xe2\xbf\x8d',
'CjkUnifiedIdeograph7863': '\xe7\xa1\xa3',
'CanadianSyllabicsWestcreeLwi': '\xe1\x93\x9f',
'SundaneseLetterVa': '\xe1\xae\x97',
'ArabicLigatureYehWithJeemWithYehFinalForm': '\xef\xb6\xaf',
'UnicodeCharacter0x84': '\xc2\x84',
'YiSyllableNjo': '\xea\x90\xbd',
'UnicodeCharacter0xc8d': '\xe0\xb2\x8d',
'GreekCapitalLetterEta': '\xce\x97',
'ArmenianCapitalLetterZa': '\xd4\xb6',
'UnicodeCharacter0xf7ad': '\xef\x9e\xad',
'ThaiCharacterYamakkan': '\xe0\xb9\x8e',
'HangulSyllableSulh': '\xec\x88\xa7',
'LatinCapitalLetterHWithDotBelow': '\xe1\xb8\xa4',
'CjkUnifiedIdeograph8ede': '\xe8\xbb\x9e',
'CjkUnifiedIdeograph8edd': '\xe8\xbb\x9d',
'CjkUnifiedIdeograph8edc': '\xe8\xbb\x9c',
'CjkUnifiedIdeograph4e5b': '\xe4\xb9\x9b',
'CjkUnifiedIdeograph8eda': '\xe8\xbb\x9a',
'UnicodeCharacter0x89': '\xc2\x89',
'PerMilleSign': '\xe2\x80\xb0',
'VaiSyllableLee': '\xea\x94\x92',
'HangulSyllableSulm': '\xec\x88\xa2',
'HangulSyllableRo': '\xeb\xa1\x9c',
'CjkRadicalPawOne': '\xe2\xba\xa4',
'HangulSyllableRe': '\xeb\xa0\x88',
'HangulSyllableRa': '\xeb\x9d\xbc',
'HangulSyllableSswelh': '\xec\x92\xab',
'HangulSyllableSswelm': '\xec\x92\xa6',
'SaurashtraLetterNna': '\xea\xa2\xa0',
'DoubleVerticalBarDoubleLeftTurnstile': '\xe2\xab\xa5',
'ArabicLigatureFehWithMeemIsolatedForm': '\xef\xb0\xb0',
'CjkUnifiedIdeograph75ef': '\xe7\x97\xaf',
'HangulSyllableSswelg': '\xec\x92\xa5',
'CjkUnifiedIdeograph8db0': '\xe8\xb6\xb0',
'CjkUnifiedIdeograph8db1': '\xe8\xb6\xb1',
'CjkUnifiedIdeograph8db2': '\xe8\xb6\xb2',
'CjkUnifiedIdeograph8db3': '\xe8\xb6\xb3',
'CjkUnifiedIdeograph8db4': '\xe8\xb6\xb4',
'CjkUnifiedIdeograph8db5': '\xe8\xb6\xb5',
'CjkUnifiedIdeograph8db6': '\xe8\xb6\xb6',
'CjkUnifiedIdeograph8db7': '\xe8\xb6\xb7',
'CjkUnifiedIdeograph8db8': '\xe8\xb6\xb8',
'CjkUnifiedIdeograph8db9': '\xe8\xb6\xb9',
'HangulSyllableSswels': '\xec\x92\xa8',
'GreekSmallLetterOmicronWithVaria': '\xe1\xbd\xb8',
'HangulSyllableSswelt': '\xec\x92\xa9',
'HangulSyllableSult': '\xec\x88\xa5',
'CjkUnifiedIdeograph8092': '\xe8\x82\x92',
'MathematicalLeftWhiteTortoiseShellBracket': '\xe2\x9f\xac',
'HangulChoseongIeungthieuth': '\xe1\x85\x8a',
'UnicodeCharacter0xf534': '\xef\x94\xb4',
'CjkUnifiedIdeograph8dba': '\xe8\xb6\xba',
'CjkUnifiedIdeograph8dbb': '\xe8\xb6\xbb',
'CjkUnifiedIdeograph8dbc': '\xe8\xb6\xbc',
'CjkUnifiedIdeograph8dbd': '\xe8\xb6\xbd',
'CjkUnifiedIdeograph8dbe': '\xe8\xb6\xbe',
'CjkUnifiedIdeograph8dbf': '\xe8\xb6\xbf',
'HangulSyllableKyut': '\xed\x81\xa9',
'HangulSyllableKyus': '\xed\x81\xa3',
'HangulSyllableKyup': '\xed\x81\xaa',
'HangulSyllableKyun': '\xed\x81\x94',
'HangulSyllableTyegg': '\xed\x86\x86',
'HangulSyllableKyul': '\xed\x81\x98',
'HangulSyllableKyum': '\xed\x81\xa0',
'HangulSyllableKyuj': '\xed\x81\xa6',
'CjkUnifiedIdeograph9810': '\xe9\xa0\x90',
'HangulSyllableKyuh': '\xed\x81\xab',
'HangulSyllableKyug': '\xed\x81\x91',
'BengaliDigitThree': '\xe0\xa7\xa9',
'HangulSyllableKyub': '\xed\x81\xa1',
'YiSyllableSy': '\xea\x8c\xa6',
'UnicodeCharacter0xa6de': '\xea\x9b\x9e',
'UnicodeCharacter0xa6dd': '\xea\x9b\x9d',
'UnicodeCharacter0xa6df': '\xea\x9b\x9f',
'UnicodeCharacter0xa6da': '\xea\x9b\x9a',
'UnicodeCharacter0xa6dc': '\xea\x9b\x9c',
'SwungDash': '\xe2\x81\x93',
'UnicodeCharacter0xf7ae': '\xef\x9e\xae',
'ModifierLetterMidLeftstemToneBar': '\xea\x9c\x94',
'HangulSyllableTyulg': '\xed\x8a\xa5',
'HangulSyllableYis': '\xec\x9d\xab',
'HangulSyllableYip': '\xec\x9d\xb2',
'HangulSyllableYin': '\xec\x9d\x9c',
'HangulSyllableYil': '\xec\x9d\xa0',
'HangulSyllableYim': '\xec\x9d\xa8',
'HangulSyllableYij': '\xec\x9d\xae',
'HangulSyllableYik': '\xec\x9d\xb0',
'HangulSyllableYih': '\xec\x9d\xb3',
'HangulSyllableTyult': '\xed\x8a\xa9',
'HangulSyllableYig': '\xec\x9d\x99',
'HangulSyllableYid': '\xec\x9d\x9f',
'HangulSyllableYib': '\xec\x9d\xa9',
'HangulSyllableYic': '\xec\x9d\xaf',
'HangulSyllableTyuls': '\xed\x8a\xa8',
'UnicodeCharacter0xe894': '\xee\xa2\x94',
'UnicodeCharacter0xe895': '\xee\xa2\x95',
'UnicodeCharacter0xe896': '\xee\xa2\x96',
'UnicodeCharacter0xe897': '\xee\xa2\x97',
'UnicodeCharacter0xe890': '\xee\xa2\x90',
'UnicodeCharacter0xe891': '\xee\xa2\x91',
'UnicodeCharacter0xe892': '\xee\xa2\x92',
'CjkUnifiedIdeograph7b08': '\xe7\xac\x88',
'UnicodeCharacter0xe898': '\xee\xa2\x98',
'UnicodeCharacter0xe899': '\xee\xa2\x99',
'HangulSyllableSags': '\xec\x82\xaf',
'NewTaiLueLetterHighXva': '\xe1\xa6\xa7',
'HangulSyllableGgolg': '\xea\xbc\xb5',
'ArabicLigatureLamWithJeemWithJeemInitialForm': '\xef\xb6\x83',
'UnicodeCharacter0xa6d5': '\xea\x9b\x95',
'UnicodeCharacter0xa6d4': '\xea\x9b\x94',
'UnicodeCharacter0xa6d7': '\xea\x9b\x97',
'UnicodeCharacter0xa6d6': '\xea\x9b\x96',
'UnicodeCharacter0xa6d1': '\xea\x9b\x91',
'UnicodeCharacter0xa6d0': '\xea\x9b\x90',
'UnicodeCharacter0xa6d3': '\xea\x9b\x93',
'UnicodeCharacter0xa6d2': '\xea\x9b\x92',
'UnicodeCharacter0xa6d9': '\xea\x9b\x99',
'YiSyllableSi': '\xea\x8c\x8b',
'CjkCompatibilityIdeographfa78': '\xef\xa9\xb8',
'YiSyllableNba': '\xea\x81\xbf',
'HangulSyllableRags': '\xeb\x9d\xbf',
'YiSyllableSo': '\xea\x8c\x99',
'YiSyllableNbi': '\xea\x81\xb8',
'InterlinearAnnotationTerminator': '\xef\xbf\xbb',
'YiSyllableNbo': '\xea\x82\x83',
'UnicodeCharacter0xe89d': '\xee\xa2\x9d',
'UnicodeCharacter0xe89e': '\xee\xa2\x9e',
'UnicodeCharacter0xe89f': '\xee\xa2\x9f',
'CjkUnifiedIdeograph7b07': '\xe7\xac\x87',
'YiSyllableNbu': '\xea\x82\x87',
'UnicodeCharacter0xe89c': '\xee\xa2\x9c',
'YiSyllableNby': '\xea\x82\x8d',
'SinhalaLetterEeyanna': '\xe0\xb6\x92',
'HangulSyllableRagg': '\xeb\x9d\xbe',
'LatinSmallLetterIWithStroke': '\xc9\xa8',
'HangulSyllableDdwes': '\xeb\x9b\xa7',
'HangulSyllableDdwep': '\xeb\x9b\xae',
'HangulSyllableKwe': '\xed\x80\x98',
'HangulSyllableDdwet': '\xeb\x9b\xad',
'HangulSyllableBeng': '\xeb\xb2\xb5',
'EthiopicSyllableTaa': '\xe1\x89\xb3',
'HangulSyllableDdweb': '\xeb\x9b\xa5',
'HangulSyllableDdwec': '\xeb\x9b\xab',
'HangulSyllableDdweg': '\xeb\x9b\x95',
'HangulSyllableDdwed': '\xeb\x9b\x9b',
'UnicodeCharacter0xd858': '\xed\xa1\x98',
'CjkUnifiedIdeograph8b7e': '\xe8\xad\xbe',
'CjkUnifiedIdeograph8b7d': '\xe8\xad\xbd',
'HangulSyllableNo': '\xeb\x85\xb8',
'CjkUnifiedIdeograph7c59': '\xe7\xb1\x99',
'CjkUnifiedIdeograph8b7a': '\xe8\xad\xba',
'HangulSyllableDdweo': '\xeb\x9a\xb8',
'CjkUnifiedIdeograph8b7c': '\xe8\xad\xbc',
'CjkUnifiedIdeograph8b7b': '\xe8\xad\xbb',
'YiSyllableZhux': '\xea\x8d\x96',
'HangulSyllableGyelt': '\xea\xb3\x91',
'HangulSyllableGyels': '\xea\xb3\x90',
'DoublestruckCapitalZ': '\xe2\x84\xa4',
'HangulSyllableGyelp': '\xea\xb3\x92',
'DoesNotDivideWithReversedNegationSlash': '\xe2\xab\xae',
'DoublestruckCapitalR': '\xe2\x84\x9d',
'DoublestruckCapitalQ': '\xe2\x84\x9a',
'DoublestruckCapitalP': '\xe2\x84\x99',
'HangulSyllableGyelg': '\xea\xb3\x8d',
'DoublestruckCapitalN': '\xe2\x84\x95',
'HangulSyllableGyelb': '\xea\xb3\x8f',
'DoublestruckCapitalH': '\xe2\x84\x8d',
'HangulSyllableGyelm': '\xea\xb3\x8e',
'DoublestruckCapitalC': '\xe2\x84\x82',
'CyrillicSmallLetterDwe': '\xea\x9a\x81',
'HangulSyllableGyelh': '\xea\xb3\x93',
'CjkUnifiedIdeograph8b79': '\xe8\xad\xb9',
'CjkUnifiedIdeograph8b78': '\xe8\xad\xb8',
'HangulSyllableKwalp': '\xec\xbd\xbe',
'CjkUnifiedIdeograph8b75': '\xe8\xad\xb5',
'CjkUnifiedIdeograph8b74': '\xe8\xad\xb4',
'CjkUnifiedIdeograph8b77': '\xe8\xad\xb7',
'CjkUnifiedIdeograph8b76': '\xe8\xad\xb6',
'CjkUnifiedIdeograph8b71': '\xe8\xad\xb1',
'CjkUnifiedIdeograph8b70': '\xe8\xad\xb0',
'CjkUnifiedIdeograph8b73': '\xe8\xad\xb3',
'CjkUnifiedIdeograph8b72': '\xe8\xad\xb2',
'UnicodeCharacter0xda4c': '\xed\xa9\x8c',
'SinhalaVowelSignDigaGayanukitta': '\xe0\xb7\xb3',
'ReferenceMark': '\xe2\x80\xbb',
'HangulSyllableToelb': '\xed\x87\xbf',
'CjkUnifiedIdeograph9225': '\xe9\x88\xa5',
'HangulSyllableGgwenh': '\xea\xbf\xb6',
'BlackParallelogram': '\xe2\x96\xb0',
'HangulSyllableBenj': '\xeb\xb2\xa5',
'CjkUnifiedIdeograph9224': '\xe9\x88\xa4',
'CjkUnifiedIdeograph9227': '\xe9\x88\xa7',
'RightFishTail': '\xe2\xa5\xbd',
'ModifierLetterSmallLWithRetroflexHook': '\xe1\xb6\xa9',
'NewTaiLueLetterLowPha': '\xe1\xa6\x98',
'ModifierLetterSmallOpenE': '\xe1\xb5\x8b',
'ConicalTaper': '\xe2\x8c\xb2',
'UnicodeCharacter0x1ab0': '\xe1\xaa\xb0',
'UnicodeCharacter0x1ab1': '\xe1\xaa\xb1',
'UnicodeCharacter0x1ab2': '\xe1\xaa\xb2',
'UnicodeCharacter0x1ab3': '\xe1\xaa\xb3',
'UnicodeCharacter0x1ab4': '\xe1\xaa\xb4',
'UnicodeCharacter0x1ab5': '\xe1\xaa\xb5',
'UnicodeCharacter0x1ab6': '\xe1\xaa\xb6',
'UnicodeCharacter0x1ab7': '\xe1\xaa\xb7',
'UnicodeCharacter0x1ab8': '\xe1\xaa\xb8',
'UnicodeCharacter0x1ab9': '\xe1\xaa\xb9',
'LeftArrowWithSmallCircle': '\xe2\xac\xb0',
'CjkUnifiedIdeograph5d29': '\xe5\xb4\xa9',
'CjkUnifiedIdeograph5d28': '\xe5\xb4\xa8',
'CjkUnifiedIdeograph4da2': '\xe4\xb6\xa2',
'CjkUnifiedIdeograph7c55': '\xe7\xb1\x95',
'HangulSyllableToelh': '\xed\x88\x83',
'SucceedsOrEqualTo': '\xe2\x89\xbd',
'CjkUnifiedIdeograph4e58': '\xe4\xb9\x98',
'BalineseDigitNine': '\xe1\xad\x99',
'TifinaghLetterTuaregYakh': '\xe2\xb5\x86',
'CjkUnifiedIdeograph9229': '\xe9\x88\xa9',
'ArabicLetterNgoeh': '\xda\xb1',
'UnicodeCharacter0xfdcd': '\xef\xb7\x8d',
'UnicodeCharacter0xfdcc': '\xef\xb7\x8c',
'HangulSyllableToelt': '\xed\x88\x81',
'UnicodeCharacter0xfdca': '\xef\xb7\x8a',
'CjkUnifiedIdeograph4da5': '\xe4\xb6\xa5',
'HangulSyllableKwalb': '\xec\xbd\xbb',
'CircledIdeographItem': '\xe3\x8a\xa0',
'HangulSyllableDyeob': '\xeb\x8e\x9d',
'LatinCapitalLetterMWithDotAbove': '\xe1\xb9\x80',
'BuhidLetterBa': '\xe1\x9d\x8a',
'UnicodeCharacter0xe69b': '\xee\x9a\x9b',
'YiSyllableZhuo': '\xea\x8d\x8b',
'HangulSyllableToelp': '\xed\x88\x82',
'TildeOperatorAboveLeftwardsArrow': '\xe2\xad\x89',
'HangulSyllableMwal': '\xeb\xab\x8c',
'CombiningDottedGraveAccent': '\xe1\xb7\x80',
'CjkUnifiedIdeograph4da4': '\xe4\xb6\xa4',
'PhagspaLetterLa': '\xea\xa1\x99',
'UnicodeCharacter0x1aba': '\xe1\xaa\xba',
'UnicodeCharacter0x1abb': '\xe1\xaa\xbb',
'UnicodeCharacter0x1abc': '\xe1\xaa\xbc',
'UnicodeCharacter0x1abd': '\xe1\xaa\xbd',
'UnicodeCharacter0x1abe': '\xe1\xaa\xbe',
'UnicodeCharacter0x1abf': '\xe1\xaa\xbf',
'HangulSyllableDwagg': '\xeb\x8f\xa2',
'UnicodeCharacter0x139c': '\xe1\x8e\x9c',
'UnicodeCharacter0x139b': '\xe1\x8e\x9b',
'LatinSmallLetterEWithOgonek': '\xc4\x99',
'UnicodeCharacter0x139f': '\xe1\x8e\x9f',
'UnicodeCharacter0x139e': '\xe1\x8e\x9e',
'IdeographicTelegraphSymbolForHourNine': '\xe3\x8d\xa1',
'HangulSyllableKwalm': '\xec\xbd\xba',
'CjkUnifiedIdeograph530c': '\xe5\x8c\x8c',
'YiSyllableNguox': '\xea\x89\xa5',
'UnicodeCharacter0xf079': '\xef\x81\xb9',
'UnicodeCharacter0xfdc9': '\xef\xb7\x89',
'UnicodeCharacter0xfdc8': '\xef\xb7\x88',
'ParenthesizedIdeographCall': '\xe3\x88\xba',
'UnicodeCharacter0xf078': '\xef\x81\xb8',
'YiSyllableNguot': '\xea\x89\xa4',
'CjkUnifiedIdeograph4345': '\xe4\x8d\x85',
'CjkUnifiedIdeograph4344': '\xe4\x8d\x84',
'CjkUnifiedIdeograph4347': '\xe4\x8d\x87',
'CjkUnifiedIdeograph4346': '\xe4\x8d\x86',
'CjkUnifiedIdeograph4341': '\xe4\x8d\x81',
'CjkUnifiedIdeograph4340': '\xe4\x8d\x80',
'CjkUnifiedIdeograph4343': '\xe4\x8d\x83',
'CjkUnifiedIdeograph4342': '\xe4\x8d\x82',
'HangulSyllableWass': '\xec\x99\x94',
'CjkUnifiedIdeograph4349': '\xe4\x8d\x89',
'CjkUnifiedIdeograph4348': '\xe4\x8d\x88',
'CjkUnifiedIdeograph9b18': '\xe9\xac\x98',
'CjkUnifiedIdeograph9289': '\xe9\x8a\x89',
'CjkUnifiedIdeograph9b10': '\xe9\xac\x90',
'CjkUnifiedIdeograph9b11': '\xe9\xac\x91',
'CjkUnifiedIdeograph9b12': '\xe9\xac\x92',
'CjkUnifiedIdeograph9b13': '\xe9\xac\x93',
'CjkUnifiedIdeograph9b14': '\xe9\xac\x94',
'CjkUnifiedIdeograph9b15': '\xe9\xac\x95',
'CjkUnifiedIdeograph9b16': '\xe9\xac\x96',
'CjkUnifiedIdeograph9b17': '\xe9\xac\x97',
'UnicodeCharacter0xa986': '\xea\xa6\x86',
'CjkUnifiedIdeograph68ce': '\xe6\xa3\x8e',
'UnicodeCharacter0xa984': '\xea\xa6\x84',
'UnicodeCharacter0xa985': '\xea\xa6\x85',
'UnicodeCharacter0xa982': '\xea\xa6\x82',
'UnicodeCharacter0xa983': '\xea\xa6\x83',
'UnicodeCharacter0xa980': '\xea\xa6\x80',
'UnicodeCharacter0xa981': '\xea\xa6\x81',
'GreekCapitalLetterRho': '\xce\xa1',
'UnicodeCharacter0xf8da': '\xef\xa3\x9a',
'UnicodeCharacter0xa988': '\xea\xa6\x88',
'UnicodeCharacter0xa989': '\xea\xa6\x89',
'CjkUnifiedIdeograph7920': '\xe7\xa4\xa0',
'BraillePatternDots23567': '\xe2\xa1\xb6',
'CjkUnifiedIdeograph7922': '\xe7\xa4\xa2',
'CjkUnifiedIdeograph7923': '\xe7\xa4\xa3',
'CjkUnifiedIdeograph7924': '\xe7\xa4\xa4',
'CjkUnifiedIdeograph7925': '\xe7\xa4\xa5',
'CjkUnifiedIdeograph7926': '\xe7\xa4\xa6',
'LaoVowelSignMaiKon': '\xe0\xba\xbb',
'LatinSmallLetterAlpha': '\xc9\x91',
'CjkUnifiedIdeograph7929': '\xe7\xa4\xa9',
'HangulSyllableBbweh': '\xec\x80\x9f',
'BraillePatternDots23568': '\xe2\xa2\xb6',
'UnicodeCharacter0xe079': '\xee\x81\xb9',
'HangulSyllableMwah': '\xeb\xab\x9f',
'UnicodeCharacter0xdada': '\xed\xab\x9a',
'CjkUnifiedIdeograph9b1a': '\xe9\xac\x9a',
'CjkUnifiedIdeograph9b1b': '\xe9\xac\x9b',
'CjkUnifiedIdeograph68cb': '\xe6\xa3\x8b',
'CjkUnifiedIdeograph9b1d': '\xe9\xac\x9d',
'CjkUnifiedIdeograph9b1e': '\xe9\xac\x9e',
'CjkUnifiedIdeograph9b1f': '\xe9\xac\x9f',
'UnicodeCharacter0xaba3': '\xea\xae\xa3',
'CjkUnifiedIdeograph434e': '\xe4\x8d\x8e',
'CjkUnifiedIdeograph434d': '\xe4\x8d\x8d',
'CjkUnifiedIdeograph434f': '\xe4\x8d\x8f',
'CjkUnifiedIdeograph434a': '\xe4\x8d\x8a',
'HangulSyllableBbwel': '\xec\x80\x8c',
'CjkUnifiedIdeograph434c': '\xe4\x8d\x8c',
'CjkUnifiedIdeograph434b': '\xe4\x8d\x8b',
'ArabicLigatureSeenWithYehFinalForm': '\xef\xb4\x98',
'HangulSyllableBbwem': '\xec\x80\x94',
'UnicodeCharacter0xe07f': '\xee\x81\xbf',
'CjkUnifiedIdeograph792a': '\xe7\xa4\xaa',
'CjkUnifiedIdeograph792b': '\xe7\xa4\xab',
'CjkUnifiedIdeograph792c': '\xe7\xa4\xac',
'CjkUnifiedIdeograph792d': '\xe7\xa4\xad',
'CjkUnifiedIdeograph792e': '\xe7\xa4\xae',
'CjkUnifiedIdeograph792f': '\xe7\xa4\xaf',
'UnicodeCharacter0xe07a': '\xee\x81\xba',
'HangzhouNumeralFour': '\xe3\x80\xa4',
'n': '\xce\xbd',
'UnicodeCharacter0xa98f': '\xea\xa6\x8f',
'UnicodeCharacter0xa98d': '\xea\xa6\x8d',
'UnicodeCharacter0xa98e': '\xea\xa6\x8e',
'UnicodeCharacter0xa98b': '\xea\xa6\x8b',
'UnicodeCharacter0xa98c': '\xea\xa6\x8c',
'UnicodeCharacter0xa98a': '\xea\xa6\x8a',
'CjkUnifiedIdeograph9ccd': '\xe9\xb3\x8d',
'YiSyllableChyrx': '\xea\x8d\xba',
'CjkUnifiedIdeograph494a': '\xe4\xa5\x8a',
'CjkUnifiedIdeograph922d': '\xe9\x88\xad',
'WhiteMediumDiamond': '\xe2\xac\xa6',
'RecyclingSymbolForType6Plastics': '\xe2\x99\xb8',
'CjkUnifiedIdeograph69c4': '\xe6\xa7\x84',
'CjkUnifiedIdeograph69c5': '\xe6\xa7\x85',
'CjkUnifiedIdeograph69c6': '\xe6\xa7\x86',
'CjkUnifiedIdeograph69c7': '\xe6\xa7\x87',
'CjkUnifiedIdeograph69c0': '\xe6\xa7\x80',
'CjkUnifiedIdeograph69c1': '\xe6\xa7\x81',
'CjkUnifiedIdeograph69c2': '\xe6\xa7\x82',
'CjkUnifiedIdeograph69c3': '\xe6\xa7\x83',
'CjkUnifiedIdeograph919c': '\xe9\x86\x9c',
'CjkUnifiedIdeograph919b': '\xe9\x86\x9b',
'CjkUnifiedIdeograph919a': '\xe9\x86\x9a',
'CjkUnifiedIdeograph69c8': '\xe6\xa7\x88',
'GreekSmallLetterOmegaWithDasiaAndPerispomeniAndYpogegrammeni': '\xe1\xbe\xa7',
'CjkUnifiedIdeograph919e': '\xe9\x86\x9e',
'CjkUnifiedIdeograph919d': '\xe9\x86\x9d',
'SubsetWithPlusSignBelow': '\xe2\xaa\xbf',
'RightwardsTwoheadedArrowFromBar': '\xe2\xa4\x85',
'CjkUnifiedIdeograph5434': '\xe5\x90\xb4',
'CjkUnifiedIdeograph5435': '\xe5\x90\xb5',
'CjkUnifiedIdeograph5436': '\xe5\x90\xb6',
'CjkUnifiedIdeograph5437': '\xe5\x90\xb7',
'CjkUnifiedIdeograph5430': '\xe5\x90\xb0',
'CjkUnifiedIdeograph5431': '\xe5\x90\xb1',
'CjkUnifiedIdeograph5432': '\xe5\x90\xb2',
'CjkUnifiedIdeograph5433': '\xe5\x90\xb3',
'HangulSyllableEunj': '\xec\x9d\x81',
'CjkUnifiedIdeograph5438': '\xe5\x90\xb8',
'CjkUnifiedIdeograph5439': '\xe5\x90\xb9',
'CjkUnifiedIdeograph69cd': '\xe6\xa7\x8d',
'CjkUnifiedIdeograph69ce': '\xe6\xa7\x8e',
'CjkUnifiedIdeograph69cf': '\xe6\xa7\x8f',
'CjkUnifiedIdeograph9198': '\xe9\x86\x98',
'CjkUnifiedIdeograph69ca': '\xe6\xa7\x8a',
'CjkUnifiedIdeograph69cb': '\xe6\xa7\x8b',
'CjkUnifiedIdeograph69cc': '\xe6\xa7\x8c',
'CjkUnifiedIdeograph9193': '\xe9\x86\x93',
'CjkUnifiedIdeograph9192': '\xe9\x86\x92',
'CjkUnifiedIdeograph7009': '\xe7\x80\x89',
'LepchaLetterPla': '\xe1\xb0\x8f',
'CjkUnifiedIdeograph9197': '\xe9\x86\x97',
'LowerLeftTriangle': '\xe2\x97\xba',
'CjkUnifiedIdeograph9195': '\xe9\x86\x95',
'CjkUnifiedIdeograph9194': '\xe9\x86\x94',
'UnicodeCharacter0xf27a': '\xef\x89\xba',
'CjkUnifiedIdeograph543d': '\xe5\x90\xbd',
'CjkUnifiedIdeograph543e': '\xe5\x90\xbe',
'CjkUnifiedIdeograph543f': '\xe5\x90\xbf',
'CjkUnifiedIdeograph543a': '\xe5\x90\xba',
'CjkUnifiedIdeograph543b': '\xe5\x90\xbb',
'CjkUnifiedIdeograph543c': '\xe5\x90\xbc',
'ArabicFathaMedialForm': '\xef\xb9\xb7',
'PresentationFormForVerticalComma': '\xef\xb8\x90',
'NkoCombiningLongDescendingTone': '\xdf\xae',
'ArabicLetterTehMarbutaFinalForm': '\xef\xba\x94',
'UnicodeCharacter0xf73e': '\xef\x9c\xbe',
'mult': '\xc3\x97',
'HangulSyllableMwelh': '\xeb\xad\xbb',
'HangulSyllableJjugs': '\xec\xad\x8b',
'CjkStrokeHzg': '\xe3\x87\x86',
'HangulJongseongNieuntikeut': '\xe1\x87\x86',
'CjkUnifiedIdeograph91dd': '\xe9\x87\x9d',
'CjkStrokeHzz': '\xe3\x87\x85',
'CjkUnifiedIdeograph9226': '\xe9\x88\xa6',
'CjkUnifiedIdeograph9221': '\xe9\x88\xa1',
'CjkUnifiedIdeograph9220': '\xe9\x88\xa0',
'CjkUnifiedIdeograph9223': '\xe9\x88\xa3',
'CjkUnifiedIdeograph9222': '\xe9\x88\xa2',
'HangulSyllableJjugg': '\xec\xad\x8a',
'ThaiCharacterPhinthu': '\xe0\xb8\xba',
'CjkStrokeHzt': '\xe3\x87\x8a',
'CjkUnifiedIdeograph9228': '\xe9\x88\xa8',
'HangulSyllableDyeon': '\xeb\x8e\x90',
'CjkStrokeHzw': '\xe3\x87\x8d',
'AplFunctionalSymbolRho': '\xe2\x8d\xb4',
'DevanagariLetterDha': '\xe0\xa4\xa7',
'CjkUnifiedIdeograph868a': '\xe8\x9a\x8a',
'HangulSyllableNyij': '\xeb\x8b\x82',
'UnicodeCharacter0xf27b': '\xef\x89\xbb',
'CjkUnifiedIdeograph868b': '\xe8\x9a\x8b',
'LogicalAnd': '\xe2\x88\xa7',
'CombiningCyrillicLetterEste': '\xe2\xb7\xb5',
'ModifierLetterRightArrowhead': '\xcb\x83',
'CjkUnifiedIdeograph68cd': '\xe6\xa3\x8d',
'CjkUnifiedIdeograph68cf': '\xe6\xa3\x8f',
'CjkUnifiedIdeograph68ca': '\xe6\xa3\x8a',
'CjkUnifiedIdeograph91db': '\xe9\x87\x9b',
'CjkUnifiedIdeograph68cc': '\xe6\xa3\x8c',
'GeorgianSmallLetterIn': '\xe2\xb4\x88',
'CjkUnifiedIdeograph9ccc': '\xe9\xb3\x8c',
'CjkUnifiedIdeograph868d': '\xe8\x9a\x8d',
'CjkUnifiedIdeograph9cca': '\xe9\xb3\x8a',
'CjkUnifiedIdeograph9ccf': '\xe9\xb3\x8f',
'CjkUnifiedIdeograph9cce': '\xe9\xb3\x8e',
'CjkUnifiedIdeograph868e': '\xe8\x9a\x8e',
'CjkUnifiedIdeograph922e': '\xe9\x88\xae',
'CjkUnifiedIdeograph68c5': '\xe6\xa3\x85',
'ArabicLetterFarsiYeh': '\xdb\x8c',
'CjkUnifiedIdeograph922a': '\xe9\x88\xaa',
'CjkUnifiedIdeograph868f': '\xe8\x9a\x8f',
'CjkUnifiedIdeograph922c': '\xe9\x88\xac',
'CjkUnifiedIdeograph922b': '\xe9\x88\xab',
'UnicodeCharacter0xf27e': '\xef\x89\xbe',
'ArabicLigatureTehWithHahIsolatedForm': '\xef\xb0\x8c',
'CjkUnifiedIdeograph57d1': '\xe5\x9f\x91',
'CjkUnifiedIdeograph9cc9': '\xe9\xb3\x89',
'LatinCapitalLetterCWithCedillaAndAcute': '\xe1\xb8\x88',
'CjkUnifiedIdeograph9cc8': '\xe9\xb3\x88',
'HangulSyllableCyaelg': '\xec\xb2\x85',
'SyriacTwoVerticalDotsAbove': '\xdd\x83',
'HangulSyllableNaelh': '\xeb\x83\x83',
'UnicodeCharacter0xde48': '\xed\xb9\x88',
'EthiopicSyllableKxee': '\xe1\x8a\xbc',
'CjkUnifiedIdeograph68c4': '\xe6\xa3\x84',
'CjkUnifiedIdeograph68c7': '\xe6\xa3\x87',
'CjkUnifiedIdeograph68c6': '\xe6\xa3\x86',
'CjkUnifiedIdeograph68c1': '\xe6\xa3\x81',
'CjkUnifiedIdeograph68c0': '\xe6\xa3\x80',
'CjkUnifiedIdeograph68c3': '\xe6\xa3\x83',
'CjkUnifiedIdeograph68c2': '\xe6\xa3\x82',
'CjkUnifiedIdeograph9cc3': '\xe9\xb3\x83',
'CjkUnifiedIdeograph9cc2': '\xe9\xb3\x82',
'CjkUnifiedIdeograph9cc1': '\xe9\xb3\x81',
'CjkUnifiedIdeograph9cc0': '\xe9\xb3\x80',
'CjkUnifiedIdeograph68c9': '\xe6\xa3\x89',
'CjkUnifiedIdeograph68c8': '\xe6\xa3\x88',
'CjkUnifiedIdeograph9cc5': '\xe9\xb3\x85',
'CjkUnifiedIdeograph9cc4': '\xe9\xb3\x84',
'ModifierLetterSmallPhi': '\xe1\xb6\xb2',
'DoublestruckCapitalGamma': '\xe2\x84\xbe',
'MetricalShortOverLong': '\xe2\x8f\x93',
'HangulSyllableNaelm': '\xeb\x82\xbe',
'HangulSyllableMwelt': '\xeb\xad\xb9',
'CjkUnifiedIdeograph55e7': '\xe5\x97\xa7',
'CjkUnifiedIdeograph55e6': '\xe5\x97\xa6',
'CjkUnifiedIdeograph55e5': '\xe5\x97\xa5',
'CjkUnifiedIdeograph55e4': '\xe5\x97\xa4',
'CjkUnifiedIdeograph55e3': '\xe5\x97\xa3',
'CjkUnifiedIdeograph55e2': '\xe5\x97\xa2',
'CjkUnifiedIdeograph55e1': '\xe5\x97\xa1',
'CjkUnifiedIdeograph55e0': '\xe5\x97\xa0',
'CjkUnifiedIdeograph9cc7': '\xe9\xb3\x87',
'CjkUnifiedIdeograph7108': '\xe7\x84\x88',
'SinhalaLetterMayanna': '\xe0\xb6\xb8',
'LatinSmallLetterSidewaysDiaeresizedU': '\xe1\xb4\x9e',
'CjkUnifiedIdeograph55e9': '\xe5\x97\xa9',
'CjkUnifiedIdeograph55e8': '\xe5\x97\xa8',
'HangulSyllableJjyalp': '\xec\xa8\xa2',
'RunicLetterPerthoPeorthP': '\xe1\x9b\x88',
'HangulSyllablePyaegg': '\xed\x8d\xa2',
'UnicodeCharacter0xf27f': '\xef\x89\xbf',
'UnicodeCharacter0xef9f': '\xee\xbe\x9f',
'UnicodeCharacter0xef9d': '\xee\xbe\x9d',
'UnicodeCharacter0xef9e': '\xee\xbe\x9e',
'UnicodeCharacter0xef9b': '\xee\xbe\x9b',
'UnicodeCharacter0xef9c': '\xee\xbe\x9c',
'ArmenianFullStop': '\xd6\x89',
'UnicodeCharacter0xef9a': '\xee\xbe\x9a',
'UnicodeCharacter0x829': '\xe0\xa0\xa9',
'UnicodeCharacter0x828': '\xe0\xa0\xa8',
'TibetanSubjoinedLetterDda': '\xe0\xbe\x9c',
'HangulSyllableE': '\xec\x97\x90',
'ArabicLetterDalWithFourDotsAbove': '\xda\x90',
'UnicodeCharacter0x821': '\xe0\xa0\xa1',
'UnicodeCharacter0x820': '\xe0\xa0\xa0',
'UnicodeCharacter0x823': '\xe0\xa0\xa3',
'UnicodeCharacter0x822': '\xe0\xa0\xa2',
'UnicodeCharacter0x825': '\xe0\xa0\xa5',
'UnicodeCharacter0x824': '\xe0\xa0\xa4',
'UnicodeCharacter0x827': '\xe0\xa0\xa7',
'UnicodeCharacter0x826': '\xe0\xa0\xa6',
'CjkUnifiedIdeograph5e73': '\xe5\xb9\xb3',
'CanadianSyllabicsCarrierTtee': '\xe1\x97\xa5',
'CjkUnifiedIdeograph710b': '\xe7\x84\x8b',
'CjkUnifiedIdeograph55ef': '\xe5\x97\xaf',
'CjkUnifiedIdeograph55ee': '\xe5\x97\xae',
'CjkUnifiedIdeograph55ed': '\xe5\x97\xad',
'CjkUnifiedIdeograph55ec': '\xe5\x97\xac',
'CjkUnifiedIdeograph55eb': '\xe5\x97\xab',
'CjkUnifiedIdeograph55ea': '\xe5\x97\xaa',
'CjkUnifiedIdeograph710e': '\xe7\x84\x8e',
'HangulSyllableDdwaelm': '\xeb\x99\x92',
'HangulSyllablePoeg': '\xed\x91\x81',
'GeorgianCapitalLetterLas': '\xe1\x82\xaa',
'CjkRadicalTable': '\xe2\xba\x87',
'_e': '\xe2\x82\x91',
'ThaiCharacterSaraUu': '\xe0\xb8\xb9',
'UnicodeCharacter0x82a': '\xe0\xa0\xaa',
'CjkCompatibilityIdeographf9ce': '\xef\xa7\x8e',
'UnicodeCharacter0x82c': '\xe0\xa0\xac',
'UnicodeCharacter0x82b': '\xe0\xa0\xab',
'UnicodeCharacter0x82e': '\xe0\xa0\xae',
'UnicodeCharacter0x82d': '\xe0\xa0\xad',
'CjkCompatibilityIdeographf9cb': '\xef\xa7\x8b',
'UnicodeCharacter0x82f': '\xe0\xa0\xaf',
'UnicodeCharacter0xeb47': '\xee\xad\x87',
'UnicodeCharacter0xde47': '\xed\xb9\x87',
'UnicodeCharacter0xef98': '\xee\xbe\x98',
'NkoLetterSa': '\xdf\x9b',
'UnicodeCharacter0xef96': '\xee\xbe\x96',
'UnicodeCharacter0xef97': '\xee\xbe\x97',
'UnicodeCharacter0xef94': '\xee\xbe\x94',
'UnicodeCharacter0xef95': '\xee\xbe\x95',
'CjkUnifiedIdeograph5e75': '\xe5\xb9\xb5',
'UnicodeCharacter0xef93': '\xee\xbe\x93',
'UnicodeCharacter0xef90': '\xee\xbe\x90',
'UnicodeCharacter0xef91': '\xee\xbe\x91',
'CjkUnifiedIdeograph7102': '\xe7\x84\x82',
'EthiopicSyllableHhe': '\xe1\x88\x95',
'HangulSyllableJjoe': '\xec\xac\x90',
'EthiopicSyllableHha': '\xe1\x88\x90',
'EthiopicSyllableHho': '\xe1\x88\x96',
'CjkUnifiedIdeograph7100': '\xe7\x84\x80',
'EthiopicSyllableHhi': '\xe1\x88\x92',
'HangulSyllableCwibs': '\xec\xb7\xba',
'EthiopicSyllableHhu': '\xe1\x88\x91',
'HangulSyllableDdwaelg': '\xeb\x99\x91',
'HangulSyllableNyib': '\xeb\x8a\xbd',
'CjkUnifiedIdeograph7106': '\xe7\x84\x86',
'FullwidthDigitFour': '\xef\xbc\x94',
'CjkUnifiedIdeograph7107': '\xe7\x84\x87',
'UnicodeCharacter0xabae': '\xea\xae\xae',
'RisingDiagonalCrossingFallingDiagonal': '\xe2\xa4\xab',
'CjkUnifiedIdeograph7104': '\xe7\x84\x84',
'UnicodeCharacter0x20f5': '\xe2\x83\xb5',
'CjkUnifiedIdeograph7105': '\xe7\x84\x85',
'UnicodeCharacter0xabac': '\xea\xae\xac',
'TurnedSmallF': '\xe2\x85\x8e',
'CjkUnifiedIdeograph5e77': '\xe5\xb9\xb7',
'UnicodeCharacter0xe7d8': '\xee\x9f\x98',
'CjkUnifiedIdeograph8680': '\xe8\x9a\x80',
'CjkUnifiedIdeograph7109': '\xe7\x84\x89',
'UnicodeCharacter0x20f6': '\xe2\x83\xb6',
'CjkUnifiedIdeograph8682': '\xe8\x9a\x82',
'UnicodeCharacter0xde40': '\xed\xb9\x80',
'UnicodeCharacter0xf28b': '\xef\x8a\x8b',
'CjkUnifiedIdeograph57de': '\xe5\x9f\x9e',
'HangulSyllablePoec': '\xed\x91\x97',
'CjkUnifiedIdeograph5e76': '\xe5\xb9\xb6',
'CjkUnifiedIdeograph8684': '\xe8\x9a\x84',
'MongolianLetterAliGaliPha': '\xe1\xa2\x93',
'CjkUnifiedIdeograph8685': '\xe8\x9a\x85',
'UnicodeCharacter0xea8a': '\xee\xaa\x8a',
'PhagspaLetterI': '\xea\xa1\x9e',
'ZNotationSchemaProjection': '\xe2\xa8\xa1',
'PhagspaLetterO': '\xea\xa1\xa1',
'CjkUnifiedIdeograph888b': '\xe8\xa2\x8b',
'CjkUnifiedIdeograph8687': '\xe8\x9a\x87',
'PhagspaLetterE': '\xea\xa1\xa0',
'YiSyllableJu': '\xea\x8f\xb2',
'HangulSyllablePoeb': '\xed\x91\x91',
'BengaliLetterDa': '\xe0\xa6\xa6',
'UnicodeCharacter0xe94a': '\xee\xa5\x8a',
'YiSyllableDex': '\xea\x84\x91',
'PhagspaLetterU': '\xea\xa1\x9f',
'ArmenianSmallLetterOh': '\xd6\x85',
'UnicodeCharacter0xde42': '\xed\xb9\x82',
'HangulSyllableBbub': '\xeb\xbf\x9d',
'ArabicLigatureSeenWithMeemWithMeemInitialForm': '\xef\xb5\xa3',
'HangulSyllablePoem': '\xed\x91\x90',
'SaurashtraDoubleDanda': '\xea\xa3\x8f',
'UnicodeCharacter0xe24d': '\xee\x89\x8d',
'CjkUnifiedIdeograph3a7c': '\xe3\xa9\xbc',
'CjkUnifiedIdeograph3a7b': '\xe3\xa9\xbb',
'CjkUnifiedIdeograph3a7a': '\xe3\xa9\xba',
'UnicodeCharacter0xede': '\xe0\xbb\x9e',
'UnicodeCharacter0xedb': '\xe0\xbb\x9b',
'CjkUnifiedIdeograph3a7f': '\xe3\xa9\xbf',
'CjkUnifiedIdeograph3a7e': '\xe3\xa9\xbe',
'CjkUnifiedIdeograph3a7d': '\xe3\xa9\xbd',
'CjkUnifiedIdeograph9987': '\xe9\xa6\x87',
'YiSyllableRyrx': '\xea\x8f\x9e',
'KangxiRadicalEvening': '\xe2\xbc\xa3',
'HangulSyllablePoel': '\xed\x91\x88',
'CjkUnifiedIdeograph748f': '\xe7\x92\x8f',
'CjkUnifiedIdeograph748e': '\xe7\x92\x8e',
'CjkUnifiedIdeograph748d': '\xe7\x92\x8d',
'LatinSmallLetterEWithBreve': '\xc4\x95',
'CjkUnifiedIdeograph748b': '\xe7\x92\x8b',
'CjkUnifiedIdeograph748a': '\xe7\x92\x8a',
'InterlockedFemaleAndMaleSign': '\xe2\x9a\xa4',
'UnicodeCharacter0xe24b': '\xee\x89\x8b',
'CjkUnifiedIdeograph3a79': '\xe3\xa9\xb9',
'CjkUnifiedIdeograph3a78': '\xe3\xa9\xb8',
'CjkUnifiedIdeograph3a73': '\xe3\xa9\xb3',
'CjkUnifiedIdeograph3a72': '\xe3\xa9\xb2',
'CjkUnifiedIdeograph3a71': '\xe3\xa9\xb1',
'CjkUnifiedIdeograph3a70': '\xe3\xa9\xb0',
'CjkUnifiedIdeograph3a77': '\xe3\xa9\xb7',
'CjkUnifiedIdeograph3a76': '\xe3\xa9\xb6',
'CjkUnifiedIdeograph3a75': '\xe3\xa9\xb5',
'CjkUnifiedIdeograph3a74': '\xe3\xa9\xb4',
'UnicodeCharacter0xd7fa': '\xed\x9f\xba',
'HangulSyllableJyeo': '\xec\xa0\xb8',
'UnicodeCharacter0xecab': '\xee\xb2\xab',
'CjkUnifiedIdeograph7487': '\xe7\x92\x87',
'CjkUnifiedIdeograph7486': '\xe7\x92\x86',
'CjkUnifiedIdeograph7485': '\xe7\x92\x85',
'BopomofoLetterOu': '\xe3\x84\xa1',
'CjkUnifiedIdeograph7483': '\xe7\x92\x83',
'CjkUnifiedIdeograph7482': '\xe7\x92\x82',
'LatinSmallLetterDotlessJWithStrokeAndHook': '\xca\x84',
'CjkUnifiedIdeograph7480': '\xe7\x92\x80',
'HangulSyllableSeus': '\xec\x8a\xb7',
'HangulSyllableSeup': '\xec\x8a\xbe',
'CjkUnifiedIdeograph7489': '\xe7\x92\x89',
'CjkUnifiedIdeograph7488': '\xe7\x92\x88',
'BraillePatternDots1345678': '\xe2\xa3\xbd',
'HangulSyllableSeuk': '\xec\x8a\xbc',
'HangulSyllableSeuh': '\xec\x8a\xbf',
'UnicodeCharacter0xdfe6': '\xed\xbf\xa6',
'CjkUnifiedIdeograph8f8a': '\xe8\xbe\x8a',
'UnicodeCharacter0xdb9d': '\xed\xae\x9d',
'HangulSyllableSeul': '\xec\x8a\xac',
'HangulSyllableSeum': '\xec\x8a\xb4',
'HangulSyllableSeub': '\xec\x8a\xb5',
'BopomofoLetterOo': '\xe3\x86\xa6',
'BopomofoLetterOm': '\xe3\x86\xb1',
'HangulSyllableSeug': '\xec\x8a\xa5',
'HangulSyllableSeud': '\xec\x8a\xab',
'CjkUnifiedIdeograph888c': '\xe8\xa2\x8c',
'HangulSyllableKoet': '\xec\xbf\x81',
'HangulSyllableKoep': '\xec\xbf\x82',
'ModifierLetterChineseToneYangPing': '\xea\x9c\x81',
'LatinSubscriptSmallLetterU': '\xe1\xb5\xa4',
'HangulSyllableKoes': '\xec\xbe\xbb',
'HangulSyllableTilt': '\xed\x8b\xbd',
'HangulSyllableCwagg': '\xec\xb4\xa6',
'HangulSyllableTwabs': '\xed\x87\x8e',
'HangulSyllableTils': '\xed\x8b\xbc',
'HangulSyllableKoed': '\xec\xbe\xaf',
'HangulSyllableTilm': '\xed\x8b\xba',
'SquareApaato': '\xe3\x8c\x80',
'CjkUnifiedIdeograph89f9': '\xe8\xa7\xb9',
'CjkUnifiedIdeograph89f8': '\xe8\xa7\xb8',
'HangulSyllableKoeb': '\xec\xbe\xb9',
'HangulSyllableWeok': '\xec\x9b\xa4',
'CjkUnifiedIdeograph89f5': '\xe8\xa7\xb5',
'CjkUnifiedIdeograph89f4': '\xe8\xa7\xb4',
'CjkUnifiedIdeograph89f7': '\xe8\xa7\xb7',
'KangxiRadicalGo': '\xe2\xbc\xa1',
'CjkUnifiedIdeograph89f1': '\xe8\xa7\xb1',
'CjkUnifiedIdeograph89f0': '\xe8\xa7\xb0',
'CjkUnifiedIdeograph89f3': '\xe8\xa7\xb3',
'CjkUnifiedIdeograph89f2': '\xe8\xa7\xb2',
'CjkUnifiedIdeograph46da': '\xe4\x9b\x9a',
'CjkUnifiedIdeograph46db': '\xe4\x9b\x9b',
'CjkUnifiedIdeograph46dc': '\xe4\x9b\x9c',
'CjkUnifiedIdeograph46dd': '\xe4\x9b\x9d',
'CjkUnifiedIdeograph46de': '\xe4\x9b\x9e',
'CjkUnifiedIdeograph46df': '\xe4\x9b\x9f',
'TibetanLetterSha': '\xe0\xbd\xa4',
'UnicodeCharacter0xdd24': '\xed\xb4\xa4',
'UnicodeCharacter0xe7d0': '\xee\x9f\x90',
'CjkUnifiedIdeograph73ff': '\xe7\x8f\xbf',
'CjkUnifiedIdeograph7f9c': '\xe7\xbe\x9c',
'BoxDrawingsLightUpAndLeft': '\xe2\x94\x98',
'CjkUnifiedIdeograph73fe': '\xe7\x8f\xbe',
'CjkUnifiedIdeograph73fb': '\xe7\x8f\xbb',
'CjkUnifiedIdeograph73fc': '\xe7\x8f\xbc',
'CjkUnifiedIdeograph7f9d': '\xe7\xbe\x9d',
'CjkUnifiedIdeograph73fa': '\xe7\x8f\xba',
'OlChikiMuTtuddag': '\xe1\xb1\xb8',
'GreekCapitalLetterEtaWithPsiliAndProsgegrammeni': '\xe1\xbe\x98',
'CjkCompatibilityIdeographf9c9': '\xef\xa7\x89',
'UnicodeCharacter0x2e4b': '\xe2\xb9\x8b',
'CjkUnifiedIdeograph6fcb': '\xe6\xbf\x8b',
'HangulSyllablePoek': '\xed\x91\x98',
'LatinSmallLetterDbDigraph': '\xc8\xb8',
'LepchaVowelSignOo': '\xe1\xb0\xa9',
'CjkUnifiedIdeograph89fe': '\xe8\xa7\xbe',
'CjkUnifiedIdeograph89fd': '\xe8\xa7\xbd',
'CjkUnifiedIdeograph89ff': '\xe8\xa7\xbf',
'CjkUnifiedIdeograph89fa': '\xe8\xa7\xba',
'UnicodeCharacter0xe7d1': '\xee\x9f\x91',
'CjkUnifiedIdeograph89fc': '\xe8\xa7\xbc',
'CjkUnifiedIdeograph89fb': '\xe8\xa7\xbb',
'HangulSyllableJjoh': '\xec\xab\x97',
'CjkCompatibilityIdeographf9c4': '\xef\xa7\x84',
'HangulSyllableKaj': '\xec\xba\x8a',
'HangulSyllableKak': '\xec\xba\x8c',
'SinhalaVowelSignDigaIspilla': '\xe0\xb7\x93',
'YiSyllableNzuox': '\xea\x8b\xb8',
'HangulSyllableKan': '\xec\xb9\xb8',
'CjkCompatibilityIdeographf9c5': '\xef\xa7\x85',
'HangulSyllableKab': '\xec\xba\x85',
'HangulSyllableKac': '\xec\xba\x8b',
'HangulSyllableKad': '\xec\xb9\xbb',
'HangulSyllableKae': '\xec\xba\x90',
'HangulSyllableKag': '\xec\xb9\xb5',
'CjkUnifiedIdeograph73f6': '\xe7\x8f\xb6',
'CjkUnifiedIdeograph73f7': '\xe7\x8f\xb7',
'CjkUnifiedIdeograph73f4': '\xe7\x8f\xb4',
'CjkUnifiedIdeograph73f5': '\xe7\x8f\xb5',
'CjkUnifiedIdeograph73f2': '\xe7\x8f\xb2',
'CjkUnifiedIdeograph73f3': '\xe7\x8f\xb3',
'CjkUnifiedIdeograph73f0': '\xe7\x8f\xb0',
'CjkUnifiedIdeograph73f1': '\xe7\x8f\xb1',
'HangulSyllableGgyenh': '\xea\xbc\x96',
'CjkUnifiedIdeograph7f98': '\xe7\xbe\x98',
'CjkUnifiedIdeograph7f99': '\xe7\xbe\x99',
'UnicodeCharacter0xe75e': '\xee\x9d\x9e',
'CjkUnifiedIdeograph73f8': '\xe7\x8f\xb8',
'CjkUnifiedIdeograph73f9': '\xe7\x8f\xb9',
'CjkUnifiedIdeograph46d0': '\xe4\x9b\x90',
'CjkUnifiedIdeograph46d1': '\xe4\x9b\x91',
'CjkUnifiedIdeograph46d2': '\xe4\x9b\x92',
'CjkUnifiedIdeograph46d3': '\xe4\x9b\x93',
'CjkUnifiedIdeograph46d4': '\xe4\x9b\x94',
'CjkUnifiedIdeograph46d5': '\xe4\x9b\x95',
'CjkUnifiedIdeograph46d6': '\xe4\x9b\x96',
'CjkUnifiedIdeograph46d7': '\xe4\x9b\x97',
'CjkUnifiedIdeograph46d8': '\xe4\x9b\x98',
'CjkUnifiedIdeograph46d9': '\xe4\x9b\x99',
'CjkCompatibilityIdeographf9c3': '\xef\xa7\x83',
'UnicodeCharacter0xf28d': '\xef\x8a\x8d',
'PrecedesAboveAlmostEqualTo': '\xe2\xaa\xb7',
'BlackSmallLozenge': '\xe2\xac\xaa',
'HangulSyllableDdyop': '\xeb\x9a\x9a',
'UnicodeCharacter0xe75a': '\xee\x9d\x9a',
'UnicodeCharacter0xdd23': '\xed\xb4\xa3',
'UnicodeCharacter0xe7d3': '\xee\x9f\x93',
'CjkUnifiedIdeograph8f8f': '\xe8\xbe\x8f',
'CjkUnifiedIdeograph8ac9': '\xe8\xab\x89',
'LatinSmallLetterHWithCedilla': '\xe1\xb8\xa9',
'HangulSyllableKeogg': '\xec\xbb\xa6',
'CjkUnifiedIdeograph7ec4': '\xe7\xbb\x84',
'HangulSyllableKeogs': '\xec\xbb\xa7',
'IntersectionAboveBarAboveUnion': '\xe2\xa9\x89',
'UnicodeCharacter0xdd20': '\xed\xb4\xa0',
'UnicodeCharacter0xe7d4': '\xee\x9f\x94',
'backepsilon': '\xcf\xb6',
'CyrillicCapitalLetterZeWithDiaeresis': '\xd3\x9e',
'UnicodeCharacter0xe244': '\xee\x89\x84',
'UnicodeCharacter0xe3f0': '\xee\x8f\xb0',
'CjkUnifiedIdeograph802d': '\xe8\x80\xad',
'YiSyllableCurx': '\xea\x8b\x88',
'UnicodeCharacter0xe7d5': '\xee\x9f\x95',
'CjkUnifiedIdeograph9f37': '\xe9\xbc\xb7',
'HangulSyllableHweonj': '\xed\x9b\xa5',
'CjkUnifiedIdeograph72e8': '\xe7\x8b\xa8',
'HangulSyllableHweonh': '\xed\x9b\xa6',
'UnicodeCharacter0xe240': '\xee\x89\x80',
'ParenthesizedIdeographSix': '\xe3\x88\xa5',
'CjkUnifiedIdeograph72e9': '\xe7\x8b\xa9',
'HangulSyllableDyelh': '\xeb\x8e\xb7',
'UnicodeCharacter0xe242': '\xee\x89\x82',
'HangulSyllableMyilg': '\xeb\xaf\xa5',
'CjkUnifiedIdeograph802a': '\xe8\x80\xaa',
'HangulSyllableBonj': '\xeb\xb3\xb9',
'OriyaDigitFive': '\xe0\xad\xab',
'CjkUnifiedIdeograph9f30': '\xe9\xbc\xb0',
'CjkUnifiedIdeograph710c': '\xe7\x84\x8c',
'HangulSyllablePwaegg': '\xed\x90\xa6',
'ArabicLetterQafWithDotAbove': '\xda\xa7',
'HangulSyllableBonh': '\xeb\xb3\xba',
'HangulSyllableKoss': '\xec\xbd\xa8',
'CjkUnifiedIdeograph72e2': '\xe7\x8b\xa2',
'CjkUnifiedIdeograph710a': '\xe7\x84\x8a',
'LatinSmallLetterWWithGrave': '\xe1\xba\x81',
'CjkUnifiedIdeograph710f': '\xe7\x84\x8f',
'UnicodeCharacter0xe259': '\xee\x89\x99',
'UnicodeCharacter0xe1b1': '\xee\x86\xb1',
'UnicodeCharacter0xe7d7': '\xee\x9f\x97',
'HangulSyllableDyelg': '\xeb\x8e\xb1',
'CjkUnifiedIdeograph710d': '\xe7\x84\x8d',
'UnicodeCharacter0xe1b3': '\xee\x86\xb3',
'UnicodeCharacter0xf28e': '\xef\x8a\x8e',
'ArmenianSmallLetterDa': '\xd5\xa4',
'UnicodeCharacter0xf3fa': '\xef\x8f\xba',
'UnicodeCharacter0xdc26': '\xed\xb0\xa6',
'CjkUnifiedIdeograph72e5': '\xe7\x8b\xa5',
'DoubleCircledDigitEight': '\xe2\x93\xbc',
'GreekSmallLetterOmegaWithDasiaAndVaria': '\xe1\xbd\xa3',
'LatinCapitalLetterGWithStroke': '\xc7\xa4',
'HangulSyllableBbwiss': '\xec\x80\xb4',
'UnicodeCharacter0xe1b8': '\xee\x86\xb8',
'CjkUnifiedIdeograph888a': '\xe8\xa2\x8a',
'HiraganaLetterGu': '\xe3\x81\x90',
'CjkUnifiedIdeograph8ea0': '\xe8\xba\xa0',
'UnicodeCharacter0xf741': '\xef\x9d\x81',
'LatinSmallLetterRWithFishhook': '\xc9\xbe',
'CjkUnifiedIdeograph886b': '\xe8\xa1\xab',
'CjkUnifiedIdeograph886c': '\xe8\xa1\xac',
'CjkUnifiedIdeograph886d': '\xe8\xa1\xad',
'CjkUnifiedIdeograph886e': '\xe8\xa1\xae',
'CjkUnifiedIdeograph886f': '\xe8\xa1\xaf',
'HiraganaLetterGo': '\xe3\x81\x94',
'UnicodeCharacter0xf749': '\xef\x9d\x89',
'HiraganaLetterGa': '\xe3\x81\x8c',
'UnicodeCharacter0xe8dd': '\xee\xa3\x9d',
'HiraganaLetterGe': '\xe3\x81\x92',
'CopticSmallLetterHori': '\xcf\xa9',
'HangulSyllableTwaem': '\xed\x87\xa8',
'CjkUnifiedIdeograph5d39': '\xe5\xb4\xb9',
'ArabicLigatureLamWithHahInitialForm': '\xef\xb3\x8a',
'HangulSyllableTwael': '\xed\x87\xa0',
'HangulSyllableGyon': '\xea\xb5\x94',
'ArabicLigatureLamWithJeemWithMeemFinalForm': '\xef\xb6\xbc',
'HangulSyllableDyelt': '\xeb\x8e\xb5',
'HangulSyllableTwaen': '\xed\x87\x9c',
'CjkUnifiedIdeograph3a0a': '\xe3\xa8\x8a',
'HangulSyllableTwaeh': '\xed\x87\xb3',
'CjkUnifiedIdeograph8860': '\xe8\xa1\xa0',
'CjkUnifiedIdeograph8861': '\xe8\xa1\xa1',
'CjkUnifiedIdeograph8862': '\xe8\xa1\xa2',
'CjkUnifiedIdeograph8863': '\xe8\xa1\xa3',
'CanadianSyllabicsCarrierKhee': '\xe1\x97\xb8',
'CjkUnifiedIdeograph6b18': '\xe6\xac\x98',
'CjkUnifiedIdeograph8866': '\xe8\xa1\xa6',
'CjkUnifiedIdeograph6ca9': '\xe6\xb2\xa9',
'CjkUnifiedIdeograph8868': '\xe8\xa1\xa8',
'CjkUnifiedIdeograph8869': '\xe8\xa1\xa9',
'Hyphen': '\xe2\x80\x90',
'TwelvePointedBlackStar': '\xe2\x9c\xb9',
'HebrewAccentTelishaGedola': '\xd6\xa0',
'HangulSyllableCeug': '\xec\xb8\xa1',
'HangulSyllableCeud': '\xec\xb8\xa7',
'CjkCompatibilityIdeographf9cd': '\xef\xa7\x8d',
'RightwardsPairedArrows': '\xe2\x87\x89',
'HangulSyllableCeus': '\xec\xb8\xb3',
'HangulSyllableCeup': '\xec\xb8\xba',
'TrigramForWater': '\xe2\x98\xb5',
'MyanmarLetterKa': '\xe1\x80\x80',
'HangulSyllableCeut': '\xec\xb8\xb9',
'UnicodeCharacter0xe757': '\xee\x9d\x97',
'UnicodeCharacter0xf152': '\xef\x85\x92',
'HangulSyllableNeoj': '\xeb\x84\x9e',
'UnicodeCharacter0xe756': '\xee\x9d\x96',
'HangulSyllableDyaet': '\xeb\x8d\x91',
'CjkUnifiedIdeograph5b19': '\xe5\xac\x99',
'UnicodeCharacter0xe755': '\xee\x9d\x95',
'CjkCompatibilityIdeographf9ca': '\xef\xa7\x8a',
'UnicodeCharacter0xe754': '\xee\x9d\x94',
'HangulSyllableTwaec': '\xed\x87\xaf',
'CjkUnifiedIdeograph8ce8': '\xe8\xb3\xa8',
'CjkUnifiedIdeograph8ce9': '\xe8\xb3\xa9',
'CjkUnifiedIdeograph8ce6': '\xe8\xb3\xa6',
'CjkUnifiedIdeograph8ce7': '\xe8\xb3\xa7',
'CjkUnifiedIdeograph8ce4': '\xe8\xb3\xa4',
'CjkUnifiedIdeograph8ce5': '\xe8\xb3\xa5',
'CjkUnifiedIdeograph8ce2': '\xe8\xb3\xa2',
'CjkUnifiedIdeograph8ce3': '\xe8\xb3\xa3',
'CjkUnifiedIdeograph8ce0': '\xe8\xb3\xa0',
'CjkUnifiedIdeograph8ce1': '\xe8\xb3\xa1',
'CjkUnifiedIdeograph828d': '\xe8\x8a\x8d',
'CjkUnifiedIdeograph828e': '\xe8\x8a\x8e',
'CjkUnifiedIdeograph828f': '\xe8\x8a\x8f',
'CjkUnifiedIdeograph9d55': '\xe9\xb5\x95',
'CjkUnifiedIdeograph9d52': '\xe9\xb5\x92',
'CjkUnifiedIdeograph828a': '\xe8\x8a\x8a',
'CjkUnifiedIdeograph828b': '\xe8\x8a\x8b',
'CjkUnifiedIdeograph828c': '\xe8\x8a\x8c',
'UnicodeCharacter0xf28f': '\xef\x8a\x8f',
'HangulSyllableKulb': '\xec\xbf\xab',
'UnicodeCharacter0xe750': '\xee\x9d\x90',
'CjkUnifiedIdeograph9d58': '\xe9\xb5\x98',
'CjkUnifiedIdeograph9d59': '\xe9\xb5\x99',
'SaurashtraLetterDda': '\xea\xa2\x9e',
'UnicodeCharacter0xf122': '\xef\x84\xa2',
'UnicodeCharacter0xda41': '\xed\xa9\x81',
'CjkUnifiedIdeograph6ca7': '\xe6\xb2\xa7',
'NewTaiLueVowelSignAay': '\xe1\xa6\xbb',
'UnicodeCharacter0xeca2': '\xee\xb2\xa2',
'UnicodeCharacter0xeca1': '\xee\xb2\xa1',
'HebrewLetterMemWithDagesh': '\xef\xac\xbe',
'CjkUnifiedIdeograph8cef': '\xe8\xb3\xaf',
'UnicodeCharacter0xea27': '\xee\xa8\xa7',
'CjkUnifiedIdeograph8ced': '\xe8\xb3\xad',
'CjkUnifiedIdeograph8cee': '\xe8\xb3\xae',
'CjkUnifiedIdeograph8ceb': '\xe8\xb3\xab',
'CjkUnifiedIdeograph8cec': '\xe8\xb3\xac',
'CjkUnifiedIdeograph8cea': '\xe8\xb3\xaa',
'UnicodeCharacter0xe759': '\xee\x9d\x99',
'HangulSyllableDdyeolb': '\xeb\x97\xa3',
'HangulSyllableTwaet': '\xed\x87\xb1',
'GurmukhiSignUdaat': '\xe0\xa9\x91',
'CjkUnifiedIdeograph99d9': '\xe9\xa7\x99',
'CjkUnifiedIdeograph7fe9': '\xe7\xbf\xa9',
'CanadianSyllabicsGlottalStop': '\xe1\x90\x9e',
'RightVerticalBarWithQuill': '\xe2\xb8\xa1',
'vartheta': '\xcf\x91',
'NkoLetterHa': '\xdf\xa4',
'UnicodeCharacter0xea20': '\xee\xa8\xa0',
'CjkCompatibilityIdeographfa00': '\xef\xa8\x80',
'HangulSyllableGyop': '\xea\xb5\xaa',
'LatinSmallLetterSWithAcute': '\xc5\x9b',
'CjkUnifiedIdeograph8285': '\xe8\x8a\x85',
'CjkUnifiedIdeograph8286': '\xe8\x8a\x86',
'CjkUnifiedIdeograph8287': '\xe8\x8a\x87',
'CjkUnifiedIdeograph8280': '\xe8\x8a\x80',
'CjkUnifiedIdeograph8281': '\xe8\x8a\x81',
'CjkUnifiedIdeograph8282': '\xe8\x8a\x82',
'CjkUnifiedIdeograph8283': '\xe8\x8a\x83',
'ArabicLigatureFehWithYehFinalForm': '\xef\xb1\xbd',
'HangulSyllableGyos': '\xea\xb5\xa3',
'CjkUnifiedIdeograph8288': '\xe8\x8a\x88',
'CjkUnifiedIdeograph8289': '\xe8\x8a\x89',
'EthiopicTonalMarkDifat': '\xe1\x8e\x94',
'HangulSyllableSyulg': '\xec\x8a\x91',
'CjkUnifiedIdeograph942c': '\xe9\x90\xac',
'CjkUnifiedIdeograph942b': '\xe9\x90\xab',
'CjkUnifiedIdeograph942a': '\xe9\x90\xaa',
'UnicodeCharacter0xe462': '\xee\x91\xa2',
'UnicodeCharacter0xe465': '\xee\x91\xa5',
'CjkUnifiedIdeograph942f': '\xe9\x90\xaf',
'CjkUnifiedIdeograph942e': '\xe9\x90\xae',
'CjkUnifiedIdeograph942d': '\xe9\x90\xad',
'UnicodeCharacter0x197c': '\xe1\xa5\xbc',
'UnicodeCharacter0x197b': '\xe1\xa5\xbb',
'UnicodeCharacter0x197a': '\xe1\xa5\xba',
'UnicodeCharacter0xab0f': '\xea\xac\x8f',
'UnicodeCharacter0xab0a': '\xea\xac\x8a',
'UnicodeCharacter0x197f': '\xe1\xa5\xbf',
'UnicodeCharacter0x197e': '\xe1\xa5\xbe',
'UnicodeCharacter0x197d': '\xe1\xa5\xbd',
'CjkUnifiedIdeograph4ef5': '\xe4\xbb\xb5',
'CjkUnifiedIdeograph4ef4': '\xe4\xbb\xb4',
'CjkUnifiedIdeograph4ef7': '\xe4\xbb\xb7',
'CjkUnifiedIdeograph4ef6': '\xe4\xbb\xb6',
'CjkUnifiedIdeograph4ef1': '\xe4\xbb\xb1',
'CjkUnifiedIdeograph4ef0': '\xe4\xbb\xb0',
'CjkUnifiedIdeograph4ef3': '\xe4\xbb\xb3',
'CjkUnifiedIdeograph4ef2': '\xe4\xbb\xb2',
'CjkUnifiedIdeograph6c48': '\xe6\xb1\x88',
'CjkUnifiedIdeograph4ef9': '\xe4\xbb\xb9',
'UnicodeCharacter0x18c4': '\xe1\xa3\x84',
'CjkUnifiedIdeograph59ca': '\xe5\xa7\x8a',
'CjkUnifiedIdeograph59cc': '\xe5\xa7\x8c',
'CjkUnifiedIdeograph59cb': '\xe5\xa7\x8b',
'CjkUnifiedIdeograph59ce': '\xe5\xa7\x8e',
'CjkUnifiedIdeograph59cd': '\xe5\xa7\x8d',
'CjkUnifiedIdeograph59cf': '\xe5\xa7\x8f',
'CjkUnifiedIdeograph58fd': '\xe5\xa3\xbd',
'BoxDrawingsLeftHeavyAndRightVerticalLight': '\xe2\x94\xbd',
'GlagoliticCapitalLetterSpideryHa': '\xe2\xb0\xa2',
'RightVerticalBoxLine': '\xe2\x8e\xb9',
'UnicodeCharacter0xef45': '\xee\xbd\x85',
'UnicodeCharacter0xda45': '\xed\xa9\x85',
'HangulSyllableNyigs': '\xeb\x8a\xaf',
'CjkUnifiedIdeograph8138': '\xe8\x84\xb8',
'ParenthesizedDigitNine': '\xe2\x91\xbc',
'CjkUnifiedIdeograph8136': '\xe8\x84\xb6',
'CjkUnifiedIdeograph8137': '\xe8\x84\xb7',
'CjkUnifiedIdeograph8134': '\xe8\x84\xb4',
'CjkUnifiedIdeograph8135': '\xe8\x84\xb5',
'CjkUnifiedIdeograph8132': '\xe8\x84\xb2',
'CjkUnifiedIdeograph8133': '\xe8\x84\xb3',
'CjkUnifiedIdeograph8130': '\xe8\x84\xb0',
'CjkUnifiedIdeograph8131': '\xe8\x84\xb1',
'CjkUnifiedIdeograph4efe': '\xe4\xbb\xbe',
'CjkUnifiedIdeograph4efd': '\xe4\xbb\xbd',
'CjkUnifiedIdeograph5731': '\xe5\x9c\xb1',
'CjkUnifiedIdeograph4eff': '\xe4\xbb\xbf',
'CjkUnifiedIdeograph4efa': '\xe4\xbb\xba',
'CyrillicSmallLetterAbkhasianChe': '\xd2\xbd',
'CjkUnifiedIdeograph4efc': '\xe4\xbb\xbc',
'CjkUnifiedIdeograph4efb': '\xe4\xbb\xbb',
'CjkUnifiedIdeograph5739': '\xe5\x9c\xb9',
'CjkUnifiedIdeograph5738': '\xe5\x9c\xb8',
'UnicodeCharacter0xef9': '\xe0\xbb\xb9',
'UnicodeCharacter0xda44': '\xed\xa9\x84',
'TrigramForLake': '\xe2\x98\xb1',
'CjkUnifiedIdeograph9422': '\xe9\x90\xa2',
'UnicodeCharacter0x1979': '\xe1\xa5\xb9',
'UnicodeCharacter0x1978': '\xe1\xa5\xb8',
'CjkUnifiedIdeograph9427': '\xe9\x90\xa7',
'CjkUnifiedIdeograph9426': '\xe9\x90\xa6',
'CjkUnifiedIdeograph9425': '\xe9\x90\xa5',
'CyrillicCapitalLetterRoundOmega': '\xd1\xba',
'UnicodeCharacter0xab05': '\xea\xac\x85',
'UnicodeCharacter0xab04': '\xea\xac\x84',
'CjkUnifiedIdeograph9429': '\xe9\x90\xa9',
'CjkUnifiedIdeograph9428': '\xe9\x90\xa8',
'UnicodeCharacter0x1977': '\xe1\xa5\xb7',
'UnicodeCharacter0x1976': '\xe1\xa5\xb6',
'UnicodeCharacter0x1975': '\xe1\xa5\xb5',
'UnicodeCharacter0xab02': '\xea\xac\x82',
'CjkUnifiedIdeograph8ba9': '\xe8\xae\xa9',
'BengaliLetterYa': '\xe0\xa6\xaf',
'CjkUnifiedIdeograph813f': '\xe8\x84\xbf',
'CjkUnifiedIdeograph8ba6': '\xe8\xae\xa6',
'CjkUnifiedIdeograph813d': '\xe8\x84\xbd',
'CjkUnifiedIdeograph813e': '\xe8\x84\xbe',
'CjkUnifiedIdeograph813b': '\xe8\x84\xbb',
'CjkUnifiedIdeograph813c': '\xe8\x84\xbc',
'CjkUnifiedIdeograph8ba1': '\xe8\xae\xa1',
'CjkUnifiedIdeograph813a': '\xe8\x84\xba',
'CjkUnifiedIdeograph59c1': '\xe5\xa7\x81',
'CjkUnifiedIdeograph59c0': '\xe5\xa7\x80',
'CjkUnifiedIdeograph59c3': '\xe5\xa7\x83',
'ArmenianSmallLetterPiwr': '\xd6\x83',
'CjkUnifiedIdeograph59c5': '\xe5\xa7\x85',
'CjkUnifiedIdeograph59c4': '\xe5\xa7\x84',
'CjkUnifiedIdeograph59c7': '\xe5\xa7\x87',
'CjkUnifiedIdeograph59c6': '\xe5\xa7\x86',
'CjkUnifiedIdeograph59c9': '\xe5\xa7\x89',
'CjkUnifiedIdeograph59c8': '\xe5\xa7\x88',
'LaoEllipsis': '\xe0\xba\xaf',
'HangulSyllableGgwaess': '\xea\xbd\xb8',
'SquarePpm': '\xe3\x8f\x99',
'CjkUnifiedIdeograph5089': '\xe5\x82\x89',
'CjkUnifiedIdeograph5088': '\xe5\x82\x88',
'TibetanDigitFour': '\xe0\xbc\xa4',
'CjkUnifiedIdeograph5085': '\xe5\x82\x85',
'BlackLozenge': '\xe2\xa7\xab',
'CjkUnifiedIdeograph5087': '\xe5\x82\x87',
'CjkUnifiedIdeograph5086': '\xe5\x82\x86',
'CjkUnifiedIdeograph5081': '\xe5\x82\x81',
'CjkUnifiedIdeograph5080': '\xe5\x82\x80',
'CjkUnifiedIdeograph5083': '\xe5\x82\x83',
'CjkUnifiedIdeograph5082': '\xe5\x82\x82',
'UnicodeCharacter0xf572': '\xef\x95\xb2',
'UnicodeCharacter0xee84': '\xee\xba\x84',
'HangulSyllableBbyils': '\xec\x82\x80',
'ChamLetterChha': '\xea\xa8\x8d',
'HangulSyllableKogg': '\xec\xbd\x96',
'ShadowedWhiteCircle': '\xe2\x9d\x8d',
'UnicodeCharacter0xe014': '\xee\x80\x94',
'GeorgianCapitalLetterWe': '\xe1\x83\x83',
'EthiopicSyllablePhaa': '\xe1\x8c\xb3',
'UnicodeCharacter0xf83e': '\xef\xa0\xbe',
'UnicodeCharacter0xf1e9': '\xef\x87\xa9',
'CjkUnifiedIdeograph9f3d': '\xe9\xbc\xbd',
'UnicodeCharacter0xea07': '\xee\xa8\x87',
'SyriacObliqueLineBelow': '\xdd\x88',
'CjkUnifiedIdeograph9f3e': '\xe9\xbc\xbe',
'UnicodeCharacter0xf574': '\xef\x95\xb4',
'CjkUnifiedIdeograph9f3b': '\xe9\xbc\xbb',
'UnicodeCharacter0xf575': '\xef\x95\xb5',
'CjkUnifiedIdeograph8021': '\xe8\x80\xa1',
'UnicodeCharacter0xf83d': '\xef\xa0\xbd',
'CjkCompatibilityIdeographf98f': '\xef\xa6\x8f',
'MalayalamLetterE': '\xe0\xb4\x8e',
'YiRadicalJjie': '\xea\x92\xb7',
'HangulSyllableGgols': '\xea\xbc\xb8',
'MalayalamLetterA': '\xe0\xb4\x85',
'MalayalamLetterO': '\xe0\xb4\x92',
'CjkUnifiedIdeograph9f3a': '\xe9\xbc\xba',
'LogicalAndWithMiddleStem': '\xe2\xa9\x9a',
'GurmukhiLetterKha': '\xe0\xa8\x96',
'MalayalamLetterI': '\xe0\xb4\x87',
'CjkUnifiedIdeograph8d10': '\xe8\xb4\x90',
'MalayalamLetterU': '\xe0\xb4\x89',
'CjkUnifiedIdeograph98f2': '\xe9\xa3\xb2',
'HangulSyllableMyanj': '\xeb\xa8\x85',
'HangulJongseongNieuncieuc': '\xe1\x86\xac',
'CjkUnifiedIdeograph508d': '\xe5\x82\x8d',
'CjkUnifiedIdeograph508f': '\xe5\x82\x8f',
'CjkUnifiedIdeograph508a': '\xe5\x82\x8a',
'CjkUnifiedIdeograph508c': '\xe5\x82\x8c',
'CjkUnifiedIdeograph508b': '\xe5\x82\x8b',
'HebrewLigatureAlefLamed': '\xef\xad\x8f',
'HangulSyllableMyanh': '\xeb\xa8\x86',
'CjkCompatibilityIdeographf9e2': '\xef\xa7\xa2',
'UnicodeCharacter0xf1e4': '\xef\x87\xa4',
'UnicodeCharacter0xf1e5': '\xef\x87\xa5',
'Ph': '\xce\xa6',
'UnicodeCharacter0xdf29': '\xed\xbc\xa9',
'ch': '\xcf\x87',
'HangulSyllableBbyilb': '\xec\x81\xbf',
'CjkUnifiedIdeograph80a1': '\xe8\x82\xa1',
'KannadaLetterDda': '\xe0\xb2\xa1',
'CjkUnifiedIdeograph80a3': '\xe8\x82\xa3',
'CjkUnifiedIdeograph80a2': '\xe8\x82\xa2',
'CjkUnifiedIdeograph80a5': '\xe8\x82\xa5',
'OlChikiLetterIny': '\xe1\xb1\xa7',
'CjkUnifiedIdeograph80a7': '\xe8\x82\xa7',
'CjkUnifiedIdeograph80a6': '\xe8\x82\xa6',
'CjkUnifiedIdeograph80a9': '\xe8\x82\xa9',
'CjkUnifiedIdeograph80a8': '\xe8\x82\xa8',
'UnicodeCharacter0xa4c8': '\xea\x93\x88',
'UnicodeCharacter0xa4c9': '\xea\x93\x89',
'HangulSyllableMwij': '\xeb\xae\x9e',
'HangulSyllableMwik': '\xeb\xae\xa0',
'HangulSyllableMwih': '\xeb\xae\xa3',
'HangulSyllableMwit': '\xeb\xae\xa1',
'Pi': '\xce\xa0',
'HangulSyllableMwis': '\xeb\xae\x9b',
'HangulSyllableMwip': '\xeb\xae\xa2',
'HangulSyllableCwa': '\xec\xb4\xa4',
'HangulSyllableKat': '\xec\xba\x8d',
'ArabicLigatureYehWithYehFinalForm': '\xef\xb2\x96',
'CjkUnifiedIdeograph98f7': '\xe9\xa3\xb7',
'YiSyllableFyt': '\xea\x83\x9e',
'UnicodeCharacter0xed58': '\xee\xb5\x98',
'UnicodeCharacter0xed59': '\xee\xb5\x99',
'YiSyllableFyp': '\xea\x83\xa1',
'HangulSyllableBolh': '\xeb\xb4\x83',
'UnicodeCharacter0xed55': '\xee\xb5\x95',
'UnicodeCharacter0xed56': '\xee\xb5\x96',
'HangulSyllableMyess': '\xeb\xaa\xa0',
'UnicodeCharacter0xed50': '\xee\xb5\x90',
'UnicodeCharacter0xed51': '\xee\xb5\x91',
'YiSyllableFyx': '\xea\x83\x9f',
'SlopingLargeAnd': '\xe2\xa9\x98',
'GlagoliticSmallLetterGlagoli': '\xe2\xb0\xb3',
'CjkUnifiedIdeograph5ffb': '\xe5\xbf\xbb',
'FullwidthNotSign': '\xef\xbf\xa2',
'RightwardsArrowWithHook': '\xe2\x86\xaa',
'CjkUnifiedIdeograph9cf9': '\xe9\xb3\xb9',
'MongolianLetterTodoWa': '\xe1\xa1\x96',
'CircledHangulHieuh': '\xe3\x89\xad',
'CjkUnifiedIdeograph80aa': '\xe8\x82\xaa',
'UnicodeCharacter0xa4cc': '\xea\x93\x8c',
'CjkUnifiedIdeograph80ac': '\xe8\x82\xac',
'CjkUnifiedIdeograph80ab': '\xe8\x82\xab',
'RightwardsArrowAboveTildeOperator': '\xe2\xa5\xb4',
'CjkUnifiedIdeograph80ad': '\xe8\x82\xad',
'CanadianSyllabicsWestcreeWaa': '\xe1\x90\x9a',
'CjkUnifiedIdeograph80af': '\xe8\x82\xaf',
'ArabicLetterGhainWithDotBelow': '\xdb\xbc',
'CjkUnifiedIdeograph515c': '\xe5\x85\x9c',
'CanadianSyllabicsCarrierDli': '\xe1\x98\xac',
'CanadianSyllabicsCarrierDlo': '\xe1\x98\xa9',
'ArabicLetterVehIsolatedForm': '\xef\xad\xaa',
'UnicodeCharacter0xed5d': '\xee\xb5\x9d',
'UnicodeCharacter0xed5e': '\xee\xb5\x9e',
'UnicodeCharacter0xed5f': '\xee\xb5\x9f',
'CanadianSyllabicsCarrierDla': '\xe1\x98\xad',
'CjkUnifiedIdeograph98f5': '\xe9\xa3\xb5',
'UnicodeCharacter0xed5a': '\xee\xb5\x9a',
'UnicodeCharacter0xed5b': '\xee\xb5\x9b',
'CanadianSyllabicsCarrierDle': '\xe1\x98\xaa',
'BraillePatternDots156': '\xe2\xa0\xb1',
'BraillePatternDots157': '\xe2\xa1\x91',
'LaoLetterO': '\xe0\xba\xad',
'CjkUnifiedIdeograph8367': '\xe8\x8d\xa7',
'LatinSmallLetterZWithPalatalHook': '\xe1\xb6\x8e',
'HangulSyllableCinj': '\xec\xb9\x9d',
'BraillePatternDots158': '\xe2\xa2\x91',
'CanadianSyllabicsCarrierDlu': '\xe1\x98\xa8',
'CanadianSyllabicsSii': '\xe1\x93\xb0',
'SinhalaLetterIruyanna': '\xe0\xb6\x8d',
'CjkUnifiedIdeograph8619': '\xe8\x98\x99',
'CjkUnifiedIdeograph83a2': '\xe8\x8e\xa2',
'CjkUnifiedIdeograph83a3': '\xe8\x8e\xa3',
'CjkUnifiedIdeograph83a0': '\xe8\x8e\xa0',
'CjkUnifiedIdeograph83a1': '\xe8\x8e\xa1',
'CjkUnifiedIdeograph83a6': '\xe8\x8e\xa6',
'CjkUnifiedIdeograph83a7': '\xe8\x8e\xa7',
'CjkUnifiedIdeograph83a4': '\xe8\x8e\xa4',
'CjkUnifiedIdeograph83a5': '\xe8\x8e\xa5',
'LeftTack': '\xe2\x8a\xa3',
'CjkUnifiedIdeograph83a8': '\xe8\x8e\xa8',
'CjkUnifiedIdeograph83a9': '\xe8\x8e\xa9',
'UnicodeCharacter0xf8b8': '\xef\xa2\xb8',
'UnicodeCharacter0xf8b9': '\xef\xa2\xb9',
'HangulSyllableGgwabs': '\xea\xbd\x9a',
'YiSyllableNjot': '\xea\x90\xbb',
'CjkUnifiedIdeograph6c4f': '\xe6\xb1\x8f',
'HangulSyllableGgyalt': '\xea\xba\x91',
'HangulSyllableGgyals': '\xea\xba\x90',
'HangulSyllableGgyalp': '\xea\xba\x92',
'UnicodeCharacter0xf22f': '\xef\x88\xaf',
'VaiSyllableKpee': '\xea\x94\x89',
'HangulSyllableGgyalm': '\xea\xba\x8e',
'UnicodeCharacter0xf22e': '\xef\x88\xae',
'UnicodeCharacter0xf22b': '\xef\x88\xab',
'UnicodeCharacter0xf22c': '\xef\x88\xac',
'HangulSyllableGgyalh': '\xea\xba\x93',
'HangulSyllableGgyalg': '\xea\xba\x8d',
'HangulSyllableGgyalb': '\xea\xba\x8f',
'CjkUnifiedIdeograph83ab': '\xe8\x8e\xab',
'CjkUnifiedIdeograph83ac': '\xe8\x8e\xac',
'MultiplicationSignInRightHalfCircle': '\xe2\xa8\xb5',
'CjkUnifiedIdeograph83aa': '\xe8\x8e\xaa',
'CjkUnifiedIdeograph83af': '\xe8\x8e\xaf',
'UnicodeCharacter0xf8ba': '\xef\xa2\xba',
'CjkUnifiedIdeograph83ad': '\xe8\x8e\xad',
'CjkUnifiedIdeograph83ae': '\xe8\x8e\xae',
'HangulSyllableSsanj': '\xec\x8b\xbd',
'RejangLetterCa': '\xea\xa4\xb9',
'HangulSyllableJjyang': '\xec\xa8\xa9',
'UnicodeCharacter0xf57d': '\xef\x95\xbd',
'HangulSyllableBbegg': '\xeb\xbb\xae',
'HangulSyllableCwalm': '\xec\xb4\xae',
'UnicodeCharacter0xf57e': '\xef\x95\xbe',
'YiSyllableHxa': '\xea\x89\x90',
'UnicodeCharacter0xf226': '\xef\x88\xa6',
'UnicodeCharacter0xf227': '\xef\x88\xa7',
'UnicodeCharacter0xf224': '\xef\x88\xa4',
'UnicodeCharacter0xf225': '\xef\x88\xa5',
'CyrillicSmallLetterPe': '\xd0\xbf',
'UnicodeCharacter0xf223': '\xef\x88\xa3',
'HangulSyllableDdung': '\xeb\x9a\xb1',
'UnicodeCharacter0xf221': '\xef\x88\xa1',
'HangulSyllableDdunh': '\xeb\x9a\xa2',
'HangulSyllableDdunj': '\xeb\x9a\xa1',
'UnicodeCharacter0xf228': '\xef\x88\xa8',
'UnicodeCharacter0xf229': '\xef\x88\xa9',
'DoublelineEqualToOrGreaterthan': '\xe2\xaa\x9a',
'CjkUnifiedIdeograph6240': '\xe6\x89\x80',
'ArabicLetterNoonGhunnaFinalForm': '\xef\xae\x9f',
'CjkUnifiedIdeograph6241': '\xe6\x89\x81',
'BraillePatternDots12456': '\xe2\xa0\xbb',
'BraillePatternDots12457': '\xe2\xa1\x9b',
'UnicodeCharacter0xf725': '\xef\x9c\xa5',
'VaiSyllableDhoo': '\xea\x96\x81',
'BraillePatternDots12458': '\xe2\xa2\x9b',
'UnicodeCharacter0xabdf': '\xea\xaf\x9f',
'HangulSyllableCwi': '\xec\xb7\xa8',
'UnicodeCharacter0xf81d': '\xef\xa0\x9d',
'CjkCompatibilityIdeographf9e0': '\xef\xa7\xa0',
'HangulSyllableCwalg': '\xec\xb4\xad',
'HangulSyllableDdyuls': '\xeb\x9c\x98',
'HangulSyllablePyaeng': '\xed\x8d\xb5',
'UnicodeCharacter0xf724': '\xef\x9c\xa4',
'LeftwardsArrowToBlackDiamond': '\xe2\xa4\x9d',
'CjkUnifiedIdeograph92d7': '\xe9\x8b\x97',
'CjkUnifiedIdeograph92d6': '\xe9\x8b\x96',
'CjkUnifiedIdeograph92d5': '\xe9\x8b\x95',
'CjkUnifiedIdeograph92d4': '\xe9\x8b\x94',
'CjkUnifiedIdeograph92d3': '\xe9\x8b\x93',
'CjkUnifiedIdeograph92d2': '\xe9\x8b\x92',
'CjkUnifiedIdeograph92d1': '\xe9\x8b\x91',
'CjkUnifiedIdeograph92d0': '\xe9\x8b\x90',
'CjkUnifiedIdeograph6166': '\xe6\x85\xa6',
'CjkUnifiedIdeograph92d9': '\xe9\x8b\x99',
'CjkUnifiedIdeograph92d8': '\xe9\x8b\x98',
'HangulSyllableRying': '\xeb\xa6\xa5',
'UnicodeCharacter0xf723': '\xef\x9c\xa3',
'_7': '\xe2\x82\x87',
'DingbatNegativeCircledSansserifDigitFour': '\xe2\x9e\x8d',
'OghamLetterCeirt': '\xe1\x9a\x8a',
'UnicodeCharacter0xeb4e': '\xee\xad\x8e',
'HangulSyllableRyinh': '\xeb\xa6\x96',
'CjkUnifiedIdeograph92df': '\xe9\x8b\x9f',
'CjkUnifiedIdeograph92de': '\xe9\x8b\x9e',
'LimbuLetterBa': '\xe1\xa4\x92',
'BraillePatternDots24567': '\xe2\xa1\xba',
'CjkUnifiedIdeograph92db': '\xe9\x8b\x9b',
'CjkUnifiedIdeograph92da': '\xe9\x8b\x9a',
'_6': '\xe2\x82\x86',
'BraillePatternDots24568': '\xe2\xa2\xba',
'KangxiRadicalBlue': '\xe2\xbe\xad',
'HangulSyllableDdyult': '\xeb\x9c\x99',
'PhagspaLetterA': '\xea\xa1\x9d',
'_5': '\xe2\x82\x85',
'UnicodeCharacter0x3104': '\xe3\x84\x84',
'CanadianSyllabicsCarrierNu': '\xe1\x98\x82',
'UnicodeCharacter0x3100': '\xe3\x84\x80',
'UnicodeCharacter0x3101': '\xe3\x84\x81',
'UnicodeCharacter0x3102': '\xe3\x84\x82',
'UnicodeCharacter0x3103': '\xe3\x84\x83',
'CanadianSyllabicsCarrierNo': '\xe1\x98\x83',
'UnicodeCharacter0xefc1': '\xee\xbf\x81',
'UnicodeCharacter0xefc2': '\xee\xbf\x82',
'UnicodeCharacter0xefc3': '\xee\xbf\x83',
'UnicodeCharacter0xefc4': '\xee\xbf\x84',
'YiRadicalWo': '\xea\x92\xb8',
'CanadianSyllabicsCarrierNi': '\xe1\x98\x86',
'UnicodeCharacter0xefc7': '\xee\xbf\x87',
'CanadianSyllabicsCarrierNg': '\xe1\x93\x91',
'UnicodeCharacter0xefc9': '\xee\xbf\x89',
'CanadianSyllabicsCarrierNe': '\xe1\x98\x84',
'CanadianSyllabicsCarrierNa': '\xe1\x98\x87',
'CjkUnifiedIdeograph5156': '\xe5\x85\x96',
'_4': '\xe2\x82\x84',
'YiSyllableLox': '\xea\x87\x88',
'CjkCompatibilityIdeographf9e1': '\xef\xa7\xa1',
'DigramForGreaterYang': '\xe2\x9a\x8c',
'volumeintegral': '\xe2\x88\xb0',
'YiSyllableDep': '\xea\x84\x93',
'WhiteDownpointingSmallTriangle': '\xe2\x96\xbf',
'YiSyllableLot': '\xea\x87\x87',
'UnicodeCharacter0xefca': '\xee\xbf\x8a',
'LimbuLetterJha': '\xe1\xa4\x89',
'UnicodeCharacter0xefcc': '\xee\xbf\x8c',
'UnicodeCharacter0xefcd': '\xee\xbf\x8d',
'UnicodeCharacter0xefce': '\xee\xbf\x8e',
'UnicodeCharacter0xefcf': '\xee\xbf\x8f',
'HangulSyllableDeg': '\xeb\x8d\xb1',
'UnicodeCharacter0xde59': '\xed\xb9\x99',
'UnicodeCharacter0xf32a': '\xef\x8c\xaa',
'UnicodeCharacter0xeb4a': '\xee\xad\x8a',
'CyrillicCapitalLetterDzwe': '\xea\x9a\x82',
'GreekCapitalLetterEpsilonWithDasia': '\xe1\xbc\x99',
'_2': '\xe2\x82\x82',
'BraillePatternDots13468': '\xe2\xa2\xad',
'CjkUnifiedIdeograph9643': '\xe9\x99\x83',
'HangulSyllableJaek': '\xec\x9f\x84',
'HangulSyllableJaej': '\xec\x9f\x82',
'HangulSyllableJaem': '\xec\x9e\xbc',
'HangulSyllableJael': '\xec\x9e\xb4',
'CjkUnifiedIdeograph9642': '\xe9\x99\x82',
'HangulSyllableJaec': '\xec\x9f\x83',
'HangulSyllableJaeb': '\xec\x9e\xbd',
'ParenthesizedIdeographStock': '\xe3\x88\xb1',
'HangulSyllableJaed': '\xec\x9e\xb3',
'HangulSyllableJaeg': '\xec\x9e\xad',
'BraillePatternDots13467': '\xe2\xa1\xad',
'YiSyllableDe': '\xea\x84\x92',
'UnicodeCharacter0xffef': '\xef\xbf\xaf',
'HangulSyllableJaep': '\xec\x9f\x86',
'HangulSyllableJaes': '\xec\x9e\xbf',
'HangulSyllableJaet': '\xec\x9f\x85',
'HiraganaLetterSmallYa': '\xe3\x82\x83',
'SyriacLetterTeth': '\xdc\x9b',
'HangulSyllablePeoj': '\xed\x8e\x92',
'HangulSyllablePeom': '\xed\x8e\x8c',
'HangulSyllablePeol': '\xed\x8e\x84',
'HangulSyllablePeon': '\xed\x8e\x80',
'YiSyllableQurx': '\xea\x90\x90',
'HangulSyllablePeoc': '\xed\x8e\x93',
'HangulSyllablePeob': '\xed\x8e\x8d',
'HangulSyllablePeod': '\xed\x8e\x83',
'HangulSyllablePeog': '\xed\x8d\xbd',
'HiraganaLetterSmallYo': '\xe3\x82\x87',
'CjkUnifiedIdeograph646b': '\xe6\x91\xab',
'CjkUnifiedIdeograph646c': '\xe6\x91\xac',
'CjkUnifiedIdeograph646a': '\xe6\x91\xaa',
'CjkUnifiedIdeograph646f': '\xe6\x91\xaf',
'HiraganaLetterSmallYu': '\xe3\x82\x85',
'CjkUnifiedIdeograph646d': '\xe6\x91\xad',
'CjkUnifiedIdeograph646e': '\xe6\x91\xae',
'HangulSyllablePeop': '\xed\x8e\x96',
'HangulSyllablePeos': '\xed\x8e\x8f',
'_0': '\xe2\x82\x80',
'CjkUnifiedIdeograph9649': '\xe9\x99\x89',
'UnicodeCharacter0xd8d0': '\xed\xa3\x90',
'UnicodeCharacter0xd8d1': '\xed\xa3\x91',
'UnicodeCharacter0xd8d2': '\xed\xa3\x92',
'CjkUnifiedIdeograph9648': '\xe9\x99\x88',
'UnicodeCharacter0xd8d4': '\xed\xa3\x94',
'UnicodeCharacter0xd8d5': '\xed\xa3\x95',
'CjkUnifiedIdeograph7ce9': '\xe7\xb3\xa9',
'CjkUnifiedIdeograph7ce8': '\xe7\xb3\xa8',
'CjkUnifiedIdeograph7ce7': '\xe7\xb3\xa7',
'CjkUnifiedIdeograph7ce6': '\xe7\xb3\xa6',
'CjkUnifiedIdeograph7ce5': '\xe7\xb3\xa5',
'BoxDrawingsLightDiagonalUpperLeftToLowerRight': '\xe2\x95\xb2',
'CjkUnifiedIdeograph7ce3': '\xe7\xb3\xa3',
'CjkUnifiedIdeograph7ce2': '\xe7\xb3\xa2',
'CjkUnifiedIdeograph7ce1': '\xe7\xb3\xa1',
'CjkUnifiedIdeograph7ce0': '\xe7\xb3\xa0',
'CjkRadicalCsimplifiedGate': '\xe2\xbb\x94',
'ArmenianSmallLetterKen': '\xd5\xaf',
'ArmenianSmallLetterKeh': '\xd6\x84',
'CjkUnifiedIdeograph6462': '\xe6\x91\xa2',
'CjkUnifiedIdeograph6463': '\xe6\x91\xa3',
'CjkUnifiedIdeograph6460': '\xe6\x91\xa0',
'CjkUnifiedIdeograph6461': '\xe6\x91\xa1',
'MalayalamLetterRra': '\xe0\xb4\xb1',
'ModifierLetterRightHalfRing': '\xca\xbe',
'CjkUnifiedIdeograph6464': '\xe6\x91\xa4',
'CjkUnifiedIdeograph6465': '\xe6\x91\xa5',
'CjkUnifiedIdeograph6468': '\xe6\x91\xa8',
'CjkUnifiedIdeograph6469': '\xe6\x91\xa9',
'ArabicLetterHighHamzaWaw': '\xd9\xb6',
'UnicodeCharacter0xdf6b': '\xed\xbd\xab',
'CyrillicCapitalLetterOt': '\xd1\xbe',
'MalayalamLetterNna': '\xe0\xb4\xa3',
'HangulSyllableNyenj': '\xeb\x85\xa1',
'CopticSmallLetterZata': '\xe2\xb2\x8d',
'HangulSyllableEulb': '\xec\x9d\x87',
'HangulSyllableEulg': '\xec\x9d\x85',
'HangulSyllableEulh': '\xec\x9d\x8b',
'HangulSyllableRyogg': '\xeb\xa3\x8e',
'HangulSyllableEuls': '\xec\x9d\x88',
'UnicodeCharacter0xd8da': '\xed\xa3\x9a',
'KangxiRadicalRiver': '\xe2\xbc\xae',
'HangulSyllableEulp': '\xec\x9d\x8a',
'UnicodeCharacter0xd8dd': '\xed\xa3\x9d',
'UnicodeCharacter0xd8de': '\xed\xa3\x9e',
'LogicalAndWithDotAbove': '\xe2\xa9\x91',
'HangulSyllableEult': '\xec\x9d\x89',
'DoublestruckSmallGamma': '\xe2\x84\xbd',
'CjkUnifiedIdeograph7cee': '\xe7\xb3\xae',
'CjkUnifiedIdeograph7ced': '\xe7\xb3\xad',
'CjkUnifiedIdeograph7cec': '\xe7\xb3\xac',
'CjkUnifiedIdeograph7ceb': '\xe7\xb3\xab',
'CjkUnifiedIdeograph7cea': '\xe7\xb3\xaa',
'UnicodeCharacter0xe84e': '\xee\xa1\x8e',
'UnicodeCharacter0xedf': '\xe0\xbb\x9f',
'CjkUnifiedIdeograph773a': '\xe7\x9c\xba',
'CjkUnifiedIdeograph773c': '\xe7\x9c\xbc',
'CjkUnifiedIdeograph773b': '\xe7\x9c\xbb',
'CjkUnifiedIdeograph773e': '\xe7\x9c\xbe',
'CjkUnifiedIdeograph773d': '\xe7\x9c\xbd',
'CjkUnifiedIdeograph773f': '\xe7\x9c\xbf',
'CjkUnifiedIdeograph360a': '\xe3\x98\x8a',
'UnicodeCharacter0xdd8a': '\xed\xb6\x8a',
'CjkUnifiedIdeograph360c': '\xe3\x98\x8c',
'CjkUnifiedIdeograph360b': '\xe3\x98\x8b',
'CjkUnifiedIdeograph360e': '\xe3\x98\x8e',
'CjkUnifiedIdeograph360d': '\xe3\x98\x8d',
'CjkUnifiedIdeograph9205': '\xe9\x88\x85',
'CjkUnifiedIdeograph360f': '\xe3\x98\x8f',
'UnicodeCharacter0xeda': '\xe0\xbb\x9a',
'HangulSyllableBwaelp': '\xeb\xb4\xba',
'HangulSyllableBbaeng': '\xeb\xba\x91',
'HangulSyllableBbaenj': '\xeb\xba\x81',
'HangulSyllableBbaenh': '\xeb\xba\x82',
'UnicodeCharacter0xf810': '\xef\xa0\x90',
'HangulSyllableGwaem': '\xea\xb4\xa8',
'CjkUnifiedIdeograph3601': '\xe3\x98\x81',
'CjkUnifiedIdeograph3600': '\xe3\x98\x80',
'CjkUnifiedIdeograph3603': '\xe3\x98\x83',
'CjkUnifiedIdeograph3602': '\xe3\x98\x82',
'CjkUnifiedIdeograph3605': '\xe3\x98\x85',
'CjkUnifiedIdeograph3604': '\xe3\x98\x84',
'CjkUnifiedIdeograph3607': '\xe3\x98\x87',
'CjkUnifiedIdeograph3606': '\xe3\x98\x86',
'CjkUnifiedIdeograph3609': '\xe3\x98\x89',
'CjkUnifiedIdeograph3608': '\xe3\x98\x88',
'UnicodeCharacter0xec2e': '\xee\xb0\xae',
'UnicodeCharacter0xea59': '\xee\xa9\x99',
'UnicodeCharacter0xea58': '\xee\xa9\x98',
'CjkUnifiedIdeograph7731': '\xe7\x9c\xb1',
'CjkUnifiedIdeograph7730': '\xe7\x9c\xb0',
'CjkUnifiedIdeograph7733': '\xe7\x9c\xb3',
'CjkUnifiedIdeograph7732': '\xe7\x9c\xb2',
'EthiopicSyllableBwi': '\xe1\x8e\x85',
'TibetanLetterHa': '\xe0\xbd\xa7',
'CjkUnifiedIdeograph7737': '\xe7\x9c\xb7',
'CjkUnifiedIdeograph67b0': '\xe6\x9e\xb0',
'EthiopicSyllableBwe': '\xe1\x8e\x87',
'CjkUnifiedIdeograph7738': '\xe7\x9c\xb8',
'HangulSyllableRom': '\xeb\xa1\xac',
'HangulSyllableRol': '\xeb\xa1\xa4',
'EthiopicSyllableBwa': '\xe1\x89\xa7',
'HangulSyllableRoj': '\xeb\xa1\xb2',
'HangulSyllableRoh': '\xeb\xa1\xb7',
'UnicodeCharacter0xe4b8': '\xee\x92\xb8',
'YiSyllableRuop': '\xea\x8f\x8c',
'HangulSyllableGwaet': '\xea\xb4\xb1',
'YiSyllableRuox': '\xea\x8f\x8a',
'_(': '\xe2\x82\x8d',
'CyrillicSmallLigatureTeTse': '\xd2\xb5',
'YiSyllableNuox': '\xea\x86\x87',
'YiSyllableZiex': '\xea\x8a\x91',
'CjkUnifiedIdeograph54bf': '\xe5\x92\xbf',
'CjkUnifiedIdeograph54be': '\xe5\x92\xbe',
'CjkUnifiedIdeograph54bd': '\xe5\x92\xbd',
'CjkUnifiedIdeograph54bc': '\xe5\x92\xbc',
'CjkUnifiedIdeograph54bb': '\xe5\x92\xbb',
'CjkUnifiedIdeograph54ba': '\xe5\x92\xba',
'CjkUnifiedIdeograph9db1': '\xe9\xb6\xb1',
'HangulSyllableKwinj': '\xed\x80\xb9',
'HangulSyllableMess': '\xeb\xa9\xa8',
'CherokeeLetterGu': '\xe1\x8e\xab',
'VaiSyllableNde': '\xea\x97\xba',
'UnicodeCharacter0xda58': '\xed\xa9\x98',
'CherokeeLetterGv': '\xe1\x8e\xac',
'VaiSyllableNda': '\xea\x95\xa1',
'KangxiRadicalBow': '\xe2\xbc\xb8',
'UnicodeCharacter0xda52': '\xed\xa9\x92',
'UnicodeCharacter0xda53': '\xed\xa9\x93',
'UnicodeCharacter0xda50': '\xed\xa9\x90',
'VaiSyllableNdo': '\xea\x97\x92',
'UnicodeCharacter0xda56': '\xed\xa9\x96',
'VaiSyllableNdi': '\xea\x94\xba',
'UnicodeCharacter0xda54': '\xed\xa9\x94',
'UnicodeCharacter0xda55': '\xed\xa9\x95',
'CherokeeLetterGe': '\xe1\x8e\xa8',
'VaiSyllableNdu': '\xea\x96\xab',
'CherokeeLetterGa': '\xe1\x8e\xa6',
'HangulSyllableCelh': '\xec\xb3\x83',
'UnicodeCharacter0xa9ae': '\xea\xa6\xae',
'CjkUnifiedIdeograph748c': '\xe7\x92\x8c',
'CherokeeLetterGo': '\xe1\x8e\xaa',
'UnicodeCharacter0xa9af': '\xea\xa6\xaf',
'CherokeeLetterGi': '\xe1\x8e\xa9',
'EthiopicDigitOne': '\xe1\x8d\xa9',
'UnicodeCharacter0xa9ab': '\xea\xa6\xab',
'CjkUnifiedIdeograph54b7': '\xe5\x92\xb7',
'CjkUnifiedIdeograph54b6': '\xe5\x92\xb6',
'CjkUnifiedIdeograph54b5': '\xe5\x92\xb5',
'CjkUnifiedIdeograph54b4': '\xe5\x92\xb4',
'CjkUnifiedIdeograph54b3': '\xe5\x92\xb3',
'CjkUnifiedIdeograph54b2': '\xe5\x92\xb2',
'CjkUnifiedIdeograph54b1': '\xe5\x92\xb1',
'CjkUnifiedIdeograph54b0': '\xe5\x92\xb0',
'HangulSyllableCelm': '\xec\xb2\xbe',
'LeftwardsHarpoonWithBarbUpAboveRightwardsHarpoonWithBarbUp': '\xe2\xa5\xa6',
'CjkUnifiedIdeograph54b9': '\xe5\x92\xb9',
'CjkUnifiedIdeograph54b8': '\xe5\x92\xb8',
'HangulSyllableJjyeolg': '\xec\xaa\x8d',
'UnicodeCharacter0xdd99': '\xed\xb6\x99',
'HangulSyllableJjyeolb': '\xec\xaa\x8f',
'HangulSyllableJjyeolm': '\xec\xaa\x8e',
'HangulSyllableJjyeolh': '\xec\xaa\x93',
'UnicodeCharacter0xec2a': '\xee\xb0\xaa',
'CjkUnifiedIdeograph8362': '\xe8\x8d\xa2',
'UnicodeCharacter0xa9a8': '\xea\xa6\xa8',
'UnicodeCharacter0xa9a5': '\xea\xa6\xa5',
'UnicodeCharacter0xa9a4': '\xea\xa6\xa4',
'UnicodeCharacter0xa9a7': '\xea\xa6\xa7',
'UnicodeCharacter0xa9a6': '\xea\xa6\xa6',
'QuadrantUpperLeftAndLowerLeftAndLowerRight': '\xe2\x96\x99',
'CjkUnifiedIdeograph5f38': '\xe5\xbc\xb8',
'UnicodeCharacter0xa9a3': '\xea\xa6\xa3',
'UnicodeCharacter0xa9a2': '\xea\xa6\xa2',
'HebrewAccentMahapakh': '\xd6\xa4',
'TildeOperatorWithDotAbove': '\xe2\xa9\xaa',
'UnicodeCharacter0xda5b': '\xed\xa9\x9b',
'UnicodeCharacter0xda5c': '\xed\xa9\x9c',
'UnicodeCharacter0xda5a': '\xed\xa9\x9a',
'UnicodeCharacter0xda5f': '\xed\xa9\x9f',
'UnicodeCharacter0xda5d': '\xed\xa9\x9d',
'UnicodeCharacter0xda5e': '\xed\xa9\x9e',
'TibetanLetterNya': '\xe0\xbd\x89',
'CjkUnifiedIdeograph70ed': '\xe7\x83\xad',
'CjkUnifiedIdeograph5f32': '\xe5\xbc\xb2',
'CjkUnifiedIdeograph6c9b': '\xe6\xb2\x9b',
'CjkUnifiedIdeograph6c9c': '\xe6\xb2\x9c',
'CjkUnifiedIdeograph6c9d': '\xe6\xb2\x9d',
'CjkUnifiedIdeograph6c9e': '\xe6\xb2\x9e',
'CanadianSyllabicsYcreeToo': '\xe1\x91\x92',
'CjkUnifiedIdeograph70ec': '\xe7\x83\xac',
'CjkUnifiedIdeograph7b4e': '\xe7\xad\x8e',
'CjkUnifiedIdeograph7b4d': '\xe7\xad\x8d',
'CjkUnifiedIdeograph7b4f': '\xe7\xad\x8f',
'CjkUnifiedIdeograph7b4a': '\xe7\xad\x8a',
'CjkUnifiedIdeograph7b4c': '\xe7\xad\x8c',
'CjkUnifiedIdeograph7b4b': '\xe7\xad\x8b',
'CjkUnifiedIdeograph6927': '\xe6\xa4\xa7',
'CjkUnifiedIdeograph6926': '\xe6\xa4\xa6',
'CjkUnifiedIdeograph6925': '\xe6\xa4\xa5',
'CjkUnifiedIdeograph6924': '\xe6\xa4\xa4',
'CjkUnifiedIdeograph6923': '\xe6\xa4\xa3',
'CjkUnifiedIdeograph6922': '\xe6\xa4\xa2',
'CjkUnifiedIdeograph6921': '\xe6\xa4\xa1',
'CjkUnifiedIdeograph6920': '\xe6\xa4\xa0',
'UnicodeCharacter0xa8ed': '\xea\xa3\xad',
'UnicodeCharacter0xa8ee': '\xea\xa3\xae',
'UnicodeCharacter0xa8ef': '\xea\xa3\xaf',
'UnicodeCharacter0xa8ea': '\xea\xa3\xaa',
'CjkUnifiedIdeograph6929': '\xe6\xa4\xa9',
'CjkUnifiedIdeograph6928': '\xe6\xa4\xa8',
'KannadaLetterIi': '\xe0\xb2\x88',
'HangulSyllableCweolg': '\xec\xb6\xb9',
'UnicodeCharacter0xd9fb': '\xed\xa7\xbb',
'UnicodeCharacter0xf326': '\xef\x8c\xa6',
'CjkUnifiedIdeograph6c90': '\xe6\xb2\x90',
'CjkUnifiedIdeograph6c91': '\xe6\xb2\x91',
'CjkUnifiedIdeograph6c92': '\xe6\xb2\x92',
'CjkUnifiedIdeograph6c93': '\xe6\xb2\x93',
'CjkUnifiedIdeograph6c94': '\xe6\xb2\x94',
'CjkUnifiedIdeograph6c95': '\xe6\xb2\x95',
'CjkUnifiedIdeograph6c96': '\xe6\xb2\x96',
'SquareHz': '\xe3\x8e\x90',
'CjkUnifiedIdeograph6c98': '\xe6\xb2\x98',
'CjkUnifiedIdeograph6c99': '\xe6\xb2\x99',
'CjkUnifiedIdeograph7b47': '\xe7\xad\x87',
'CjkUnifiedIdeograph7b46': '\xe7\xad\x86',
'CjkUnifiedIdeograph70e8': '\xe7\x83\xa8',
'SquareHp': '\xe3\x8f\x8b',
'TeardropbarbedRightwardsArrow': '\xe2\x9e\xba',
'CjkUnifiedIdeograph7b42': '\xe7\xad\x82',
'iint': '\xe2\x88\xac',
'ArabicLetterOeFinalForm': '\xef\xaf\x9a',
'SquareHg': '\xe3\x8b\x8c',
'SquareHa': '\xe3\x8f\x8a',
'UnicodeCharacter0xf328': '\xef\x8c\xa8',
'ArabicLetterLamFinalForm': '\xef\xbb\x9e',
'LatinCapitalLetterOWithCircumflex': '\xc3\x94',
'YiRadicalKit': '\xea\x92\x92',
'CjkUnifiedIdeograph692f': '\xe6\xa4\xaf',
'IdeographicAnnotationSecondMark': '\xe3\x86\x9a',
'KhmerLetterHa': '\xe1\x9e\xa0',
'CjkUnifiedIdeograph692c': '\xe6\xa4\xac',
'CjkUnifiedIdeograph692b': '\xe6\xa4\xab',
'CjkUnifiedIdeograph692a': '\xe6\xa4\xaa',
'UnicodeCharacter0xa8e4': '\xea\xa3\xa4',
'UnicodeCharacter0xa8e5': '\xea\xa3\xa5',
'LatinSmallLetterEWithTildeBelow': '\xe1\xb8\x9b',
'UnicodeCharacter0xa8e7': '\xea\xa3\xa7',
'UnicodeCharacter0xa8e0': '\xea\xa3\xa0',
'UnicodeCharacter0xa8e1': '\xea\xa3\xa1',
'UnicodeCharacter0xa8e2': '\xea\xa3\xa2',
'UnicodeCharacter0xa8e3': '\xea\xa3\xa3',
'CjkUnifiedIdeograph71ae': '\xe7\x86\xae',
'CjkUnifiedIdeograph71ad': '\xe7\x86\xad',
'CjkUnifiedIdeograph71af': '\xe7\x86\xaf',
'CjkUnifiedIdeograph71aa': '\xe7\x86\xaa',
'CjkUnifiedIdeograph71ac': '\xe7\x86\xac',
'CjkUnifiedIdeograph71ab': '\xe7\x86\xab',
'HangulChoseongRieulhieuh': '\xe1\x84\x9a',
'UnicodeCharacter0xea2d': '\xee\xa8\xad',
'HangulSyllablePilb': '\xed\x95\x87',
'HangulSyllablePilg': '\xed\x95\x85',
'HangulSyllablePilh': '\xed\x95\x8b',
'RejangLetterNyja': '\xea\xa5\x85',
'HangulSyllablePilm': '\xed\x95\x86',
'UnicodeCharacter0xdc7d': '\xed\xb1\xbd',
'HangulSyllablePilp': '\xed\x95\x8a',
'LepchaLetterFa': '\xe1\xb0\x91',
'HangulSyllablePils': '\xed\x95\x88',
'HangulSyllablePilt': '\xed\x95\x89',
'LatinSmallLetterOWithInvertedBreve': '\xc8\x8f',
'YiSyllablePuop': '\xea\x81\x85',
'YiSyllablePuox': '\xea\x81\x83',
'CjkUnifiedIdeograph71a5': '\xe7\x86\xa5',
'CjkUnifiedIdeograph71a4': '\xe7\x86\xa4',
'GreekSmallLetterIotaWithDasiaAndOxia': '\xe1\xbc\xb5',
'CjkUnifiedIdeograph71a6': '\xe7\x86\xa6',
'CjkUnifiedIdeograph71a1': '\xe7\x86\xa1',
'CjkUnifiedIdeograph71a0': '\xe7\x86\xa0',
'CjkUnifiedIdeograph71a3': '\xe7\x86\xa3',
'LepchaLetterTta': '\xe1\xb1\x8d',
'CanadianSyllabicsYii': '\xe1\x94\xa9',
'CjkUnifiedIdeograph71a9': '\xe7\x86\xa9',
'CjkUnifiedIdeograph71a8': '\xe7\x86\xa8',
'PhagspaLetterDza': '\xea\xa1\x92',
'diffd': '\xe2\x85\x86',
'CjkUnifiedIdeograph7481': '\xe7\x92\x81',
'CjkUnifiedIdeograph47f6': '\xe4\x9f\xb6',
'CjkUnifiedIdeograph47f5': '\xe4\x9f\xb5',
'ArabicLigatureYehWithJeemInitialForm': '\xef\xb3\x9a',
'HangulSyllableJjoes': '\xec\xac\xa3',
'UnicodeCharacter0xe20a': '\xee\x88\x8a',
'YiRadicalLi': '\xea\x92\x91',
'UnicodeCharacter0xe20b': '\xee\x88\x8b',
'UnicodeCharacter0xe20e': '\xee\x88\x8e',
'CjkUnifiedIdeograph76e1': '\xe7\x9b\xa1',
'UnicodeCharacter0xe20f': '\xee\x88\x8f',
'HangulSyllableTyaegs': '\xed\x84\x97',
'CircledHangulRieul': '\xe3\x89\xa3',
'UnicodeCharacter0xf12d': '\xef\x84\xad',
'HangulSyllableSeut': '\xec\x8a\xbd',
'UnicodeCharacter0xd800': '\xed\xa0\x80',
'RunicLetterBerkananBeorcBjarkanB': '\xe1\x9b\x92',
'CombiningLatinSmallLetterX': '\xcd\xaf',
'HangulSyllableSeuj': '\xec\x8a\xba',
'YiSyllableByt': '\xea\x80\xb2',
'UnicodeCharacter0xecec': '\xee\xb3\xac',
'UnicodeCharacter0xeceb': '\xee\xb3\xab',
'UnicodeCharacter0xecea': '\xee\xb3\xaa',
'DingbatNegativeCircledSansserifDigitNine': '\xe2\x9e\x92',
'OneDotOverTwoDotsPunctuation': '\xe2\xb8\xab',
'UnicodeCharacter0xecee': '\xee\xb3\xae',
'GreekSmallLetterEtaWithVariaAndYpogegrammeni': '\xe1\xbf\x82',
'CjkUnifiedIdeograph5f3b': '\xe5\xbc\xbb',
'SlavonicAsterisk': '\xea\x99\xb3',
'UnicodeCharacter0xd801': '\xed\xa0\x81',
'CjkUnifiedIdeograph5f3c': '\xe5\xbc\xbc',
'CjkUnifiedIdeograph9fb2': '\xe9\xbe\xb2',
'HangulSyllableSeun': '\xec\x8a\xa8',
'UnicodeCharacter0xf1df': '\xef\x87\x9f',
'HangulSyllableCyong': '\xec\xb6\x8d',
'CjkUnifiedIdeograph5f3a': '\xe5\xbc\xba',
'HangulSyllableCyonj': '\xec\xb5\xbd',
'HangulSyllableCyonh': '\xec\xb5\xbe',
'UnicodeCharacter0xe201': '\xee\x88\x81',
'HangulSyllableJjoeb': '\xec\xac\xa1',
'UnicodeCharacter0xe203': '\xee\x88\x83',
'UnicodeCharacter0xe202': '\xee\x88\x82',
'UnicodeCharacter0xe205': '\xee\x88\x85',
'CombiningLatinSmallLetterS': '\xe1\xb7\xa4',
'UnicodeCharacter0xe207': '\xee\x88\x87',
'HangulSyllableSeuc': '\xec\x8a\xbb',
'UnicodeCharacter0xe209': '\xee\x88\x89',
'UnicodeCharacter0xe208': '\xee\x88\x88',
'CjkUnifiedIdeograph5f3e': '\xe5\xbc\xbe',
'HangulJungseongU': '\xe1\x85\xae',
'HangulSyllableJjoeg': '\xec\xac\x91',
'DevanagariLetterLa': '\xe0\xa4\xb2',
'HangulJungseongE': '\xe1\x85\xa6',
'UnicodeCharacter0xece8': '\xee\xb3\xa8',
'ArabicLigatureBehWithKhahWithYehFinalForm': '\xef\xb6\x9e',
'HangulJungseongA': '\xe1\x85\xa1',
'HangulJungseongO': '\xe1\x85\xa9',
'UnicodeCharacter0xece2': '\xee\xb3\xa2',
'UnicodeCharacter0xece1': '\xee\xb3\xa1',
'UnicodeCharacter0xece0': '\xee\xb3\xa0',
'UnicodeCharacter0xece7': '\xee\xb3\xa7',
'UnicodeCharacter0xece6': '\xee\xb3\xa6',
'HangulJungseongI': '\xe1\x85\xb5',
'UnicodeCharacter0xece4': '\xee\xb3\xa4',
'CjkCompatibilityIdeographfa29': '\xef\xa8\xa9',
'CjkCompatibilityIdeographfa28': '\xef\xa8\xa8',
'UnicodeCharacter0xd98': '\xe0\xb6\x98',
'UnicodeCharacter0xd99': '\xe0\xb6\x99',
'CjkCompatibilityIdeographfa23': '\xef\xa8\xa3',
'CjkCompatibilityIdeographfa22': '\xef\xa8\xa2',
'CircledHeavyWhiteRightwardsArrow': '\xe2\x9e\xb2',
'UnicodeCharacter0xd97': '\xe0\xb6\x97',
'CjkCompatibilityIdeographfa27': '\xef\xa8\xa7',
'MyanmarLetterShanKha': '\xe1\x81\xb6',
'CjkCompatibilityIdeographfa25': '\xef\xa8\xa5',
'CjkCompatibilityIdeographfa24': '\xef\xa8\xa4',
'LeftwardsArrowFromBarToBlackDiamond': '\xe2\xa4\x9f',
'HangulSyllableHoebs': '\xed\x9a\x9e',
'UnicodeCharacter0x9fe7': '\xe9\xbf\xa7',
'UnicodeCharacter0xa4a': '\xe0\xa9\x8a',
'UnicodeCharacter0xa4f': '\xe0\xa9\x8f',
'UnicodeCharacter0x9fe4': '\xe9\xbf\xa4',
'UnicodeCharacter0xa4e': '\xe0\xa9\x8e',
'HangulSyllableKwenj': '\xed\x80\x9d',
'UnicodeCharacter0xf836': '\xef\xa0\xb6',
'LatinSmallLetterKWithAcute': '\xe1\xb8\xb1',
'UnicodeCharacter0x9fe2': '\xe9\xbf\xa2',
'HeavyOpenCentreCross': '\xe2\x9c\x9c',
'SylotiNagriLetterCho': '\xea\xa0\x8d',
'CjkUnifiedIdeograph4e07': '\xe4\xb8\x87',
'CjkUnifiedIdeograph4e06': '\xe4\xb8\x86',
'CjkUnifiedIdeograph4e05': '\xe4\xb8\x85',
'CjkUnifiedIdeograph4e04': '\xe4\xb8\x84',
'CjkUnifiedIdeograph4e03': '\xe4\xb8\x83',
'CjkUnifiedIdeograph4e02': '\xe4\xb8\x82',
'CjkUnifiedIdeograph4e01': '\xe4\xb8\x81',
'CjkUnifiedIdeograph4e00': '\xe4\xb8\x80',
'NumberNineteenFullStop': '\xe2\x92\x9a',
'CjkUnifiedIdeograph4e09': '\xe4\xb8\x89',
'CjkUnifiedIdeograph4e08': '\xe4\xb8\x88',
'WhiteSquareWithUpperLeftQuadrant': '\xe2\x97\xb0',
'UnicodeCharacter0xa49': '\xe0\xa9\x89',
'BraillePatternDots568': '\xe2\xa2\xb0',
'BraillePatternDots567': '\xe2\xa1\xb0',
'UnicodeCharacter0xa43': '\xe0\xa9\x83',
'CopticSmallLetterOldCopticHat': '\xe2\xb3\x95',
'UnicodeCharacter0xa46': '\xe0\xa9\x86',
'UnicodeCharacter0xa44': '\xe0\xa9\x84',
'UnicodeCharacter0xa45': '\xe0\xa9\x85',
'HangulSyllableKam': '\xec\xba\x84',
'DingbatNegativeCircledNumberTen': '\xe2\x9d\xbf',
'CjkUnifiedIdeograph954f': '\xe9\x95\x8f',
'CjkCompatibilityIdeographfa2b': '\xef\xa8\xab',
'CjkCompatibilityIdeographfa2a': '\xef\xa8\xaa',
'CjkCompatibilityIdeographfa2d': '\xef\xa8\xad',
'CjkUnifiedIdeograph4e0f': '\xe4\xb8\x8f',
'CjkUnifiedIdeograph4e0e': '\xe4\xb8\x8e',
'CjkUnifiedIdeograph4e0d': '\xe4\xb8\x8d',
'CjkUnifiedIdeograph4e0c': '\xe4\xb8\x8c',
'CjkUnifiedIdeograph4e0b': '\xe4\xb8\x8b',
'CjkUnifiedIdeograph4e0a': '\xe4\xb8\x8a',
'UnicodeCharacter0xe5a9': '\xee\x96\xa9',
'UnicodeCharacter0xee82': '\xee\xba\x82',
'HangulSyllableKoeg': '\xec\xbe\xa9',
'CjkUnifiedIdeograph78f7': '\xe7\xa3\xb7',
'HangulSyllableCweolh': '\xec\xb6\xbf',
'HangulSyllableHweogs': '\xed\x9b\xa3',
'YiSyllableJjie': '\xea\x90\x9e',
'EthiopicSyllableXwee': '\xe1\x8a\x8c',
'YiSyllableNdix': '\xea\x85\x92',
'HangulSyllableKoec': '\xec\xbe\xbf',
'CanadianSyllabicsNaskapiSw': '\xe1\x94\x8b',
'UnicodeCharacter0xf00a': '\xef\x80\x8a',
'UnicodeCharacter0x1f4e': '\xe1\xbd\x8e',
'HangulSyllableKoel': '\xec\xbe\xb0',
'UnicodeCharacter0x1f4f': '\xe1\xbd\x8f',
'YiSyllableJjip': '\xea\x90\x9b',
'HangulSyllablePyabs': '\xed\x8d\x96',
'HangulSyllableKoen': '\xec\xbe\xac',
'YiSyllableJjix': '\xea\x90\x99',
'SouthWestDoubleArrow': '\xe2\x87\x99',
'CjkUnifiedIdeograph49df': '\xe4\xa7\x9f',
'CjkUnifiedIdeograph49de': '\xe4\xa7\x9e',
'CjkUnifiedIdeograph49dd': '\xe4\xa7\x9d',
'CjkUnifiedIdeograph49dc': '\xe4\xa7\x9c',
'HangulLetterPansios': '\xe3\x85\xbf',
'CjkUnifiedIdeograph49da': '\xe4\xa7\x9a',
'HangulSyllableWenh': '\xec\x9b\xae',
'HangulSyllableWeng': '\xec\x9b\xbd',
'HangulSyllableKoeh': '\xec\xbf\x83',
'sum': '\xe2\x88\x91',
'UnicodeCharacter0xdd95': '\xed\xb6\x95',
'UnicodeCharacter0xe4fe': '\xee\x93\xbe',
'CjkUnifiedIdeograph67f9': '\xe6\x9f\xb9',
'CjkUnifiedIdeograph6305': '\xe6\x8c\x85',
'intersect': '\xe2\x88\xa9',
'CjkUnifiedIdeograph67f5': '\xe6\x9f\xb5',
'CjkUnifiedIdeograph67f4': '\xe6\x9f\xb4',
'CjkUnifiedIdeograph67f7': '\xe6\x9f\xb7',
'CjkUnifiedIdeograph67f6': '\xe6\x9f\xb6',
'CjkUnifiedIdeograph67f1': '\xe6\x9f\xb1',
'CjkUnifiedIdeograph67f0': '\xe6\x9f\xb0',
'CjkUnifiedIdeograph67f3': '\xe6\x9f\xb3',
'CjkUnifiedIdeograph67f2': '\xe6\x9f\xb2',
'HangulSyllableDwilb': '\xeb\x92\xaf',
'CjkUnifiedIdeograph8a26': '\xe8\xa8\xa6',
'UnicodeCharacter0x1f47': '\xe1\xbd\x87',
'UnicodeCharacter0x1f46': '\xe1\xbd\x86',
'CjkUnifiedIdeograph97cf': '\xe9\x9f\x8f',
'HangulSyllableDwilg': '\xeb\x92\xad',
'HangulSyllableSsyop': '\xec\x91\xa2',
'HangulSyllableDwilh': '\xeb\x92\xb3',
'HangulSyllableSsyos': '\xec\x91\x9b',
'HangulSyllableSsyot': '\xec\x91\xa1',
'CjkUnifiedIdeograph9058': '\xe9\x81\x98',
'LatinSmallLetterLWithDoubleBar': '\xe2\xb1\xa1',
'HangulSyllableSsyoh': '\xec\x91\xa3',
'HangulSyllableDwils': '\xeb\x92\xb0',
'HangulSyllableDwilp': '\xeb\x92\xb2',
'HangulSyllableSsyok': '\xec\x91\xa0',
'HangulSyllableSsyol': '\xec\x91\x90',
'HangulSyllableSsyom': '\xec\x91\x98',
'HangulSyllableDwilt': '\xeb\x92\xb1',
'HangulSyllableSsyob': '\xec\x91\x99',
'HangulSyllableSsyoc': '\xec\x91\x9f',
'HangulSyllableSsyod': '\xec\x91\x8f',
'HangulSyllableSsyog': '\xec\x91\x89',
'LatinSmallLetterOpenO': '\xc9\x94',
'CjkUnifiedIdeograph67fe': '\xe6\x9f\xbe',
'CjkUnifiedIdeograph67fd': '\xe6\x9f\xbd',
'HangulSyllableTyegs': '\xed\x86\x87',
'CjkUnifiedIdeograph67ff': '\xe6\x9f\xbf',
'CjkUnifiedIdeograph67fa': '\xe6\x9f\xba',
'LatinSmallLetterOpenE': '\xc9\x9b',
'CjkUnifiedIdeograph67fc': '\xe6\x9f\xbc',
'BalineseLetterOkaraTedung': '\xe1\xac\x92',
'CjkUnifiedIdeograph49d7': '\xe4\xa7\x97',
'CjkUnifiedIdeograph49d6': '\xe4\xa7\x96',
'CjkUnifiedIdeograph49d5': '\xe4\xa7\x95',
'CjkUnifiedIdeograph49d4': '\xe4\xa7\x94',
'CjkUnifiedIdeograph49d3': '\xe4\xa7\x93',
'LeftwardsHarpoonWithBarbUpAboveLongDash': '\xe2\xa5\xaa',
'CjkUnifiedIdeograph49d1': '\xe4\xa7\x91',
'CjkUnifiedIdeograph49d0': '\xe4\xa7\x90',
'CjkUnifiedIdeograph9050': '\xe9\x81\x90',
'CjkUnifiedIdeograph49d9': '\xe4\xa7\x99',
'CjkUnifiedIdeograph49d8': '\xe4\xa7\x98',
'CjkUnifiedIdeograph4653': '\xe4\x99\x93',
'CjkUnifiedIdeograph4652': '\xe4\x99\x92',
'CjkUnifiedIdeograph4651': '\xe4\x99\x91',
'CjkUnifiedIdeograph4650': '\xe4\x99\x90',
'CjkUnifiedIdeograph4657': '\xe4\x99\x97',
'CjkUnifiedIdeograph4656': '\xe4\x99\x96',
'CjkUnifiedIdeograph4655': '\xe4\x99\x95',
'CjkUnifiedIdeograph4654': '\xe4\x99\x94',
'HangulSyllableByaelb': '\xeb\xb1\xb3',
'CjkUnifiedIdeograph8f0d': '\xe8\xbc\x8d',
'CjkUnifiedIdeograph4659': '\xe4\x99\x99',
'CjkUnifiedIdeograph4658': '\xe4\x99\x98',
'DoublestruckCapitalPi': '\xe2\x84\xbf',
'CjkUnifiedIdeograph9057': '\xe9\x81\x97',
'CjkUnifiedIdeograph9054': '\xe9\x81\x94',
'HangulSyllableByaels': '\xeb\xb1\xb4',
'HangulSyllableByaelp': '\xeb\xb1\xb6',
'OneDotLeader': '\xe2\x80\xa4',
'YiSyllableDurx': '\xea\x84\x98',
'CjkUnifiedIdeograph7f9b': '\xe7\xbe\x9b',
'CjkUnifiedIdeograph54ff': '\xe5\x93\xbf',
'CjkUnifiedIdeograph48ba': '\xe4\xa2\xba',
'CjkUnifiedIdeograph48bb': '\xe4\xa2\xbb',
'CjkUnifiedIdeograph48bc': '\xe4\xa2\xbc',
'CjkUnifiedIdeograph48bd': '\xe4\xa2\xbd',
'CjkUnifiedIdeograph48be': '\xe4\xa2\xbe',
'CjkUnifiedIdeograph48bf': '\xe4\xa2\xbf',
'UnicodeCharacter0xe784': '\xee\x9e\x84',
'PerTenThousandSign': '\xe2\x80\xb1',
'UnicodeCharacter0xe786': '\xee\x9e\x86',
'CjkUnifiedIdeograph7f9a': '\xe7\xbe\x9a',
'UnicodeCharacter0xe780': '\xee\x9e\x80',
'MongolianLetterKa': '\xe1\xa0\xba',
'UnicodeCharacter0xe782': '\xee\x9e\x82',
'UnicodeCharacter0xe783': '\xee\x9e\x83',
'CjkUnifiedIdeograph7f9f': '\xe7\xbe\x9f',
'ArabicLigatureTehWithJeemWithYehFinalForm': '\xef\xb6\x9f',
'UnicodeCharacter0xe788': '\xee\x9e\x88',
'UnicodeCharacter0xe789': '\xee\x9e\x89',
'HangulSyllableGwinj': '\xea\xb7\x85',
'CanadianSyllabicsCarrierTtsi': '\xe1\x99\xab',
'CjkUnifiedIdeograph4619': '\xe4\x98\x99',
'ArabicLetterAinInitialForm': '\xef\xbb\x8b',
'CjkUnifiedIdeograph7f9e': '\xe7\xbe\x9e',
'HangulSyllableGwing': '\xea\xb7\x95',
'CjkUnifiedIdeograph465c': '\xe4\x99\x9c',
'CjkUnifiedIdeograph465b': '\xe4\x99\x9b',
'CjkUnifiedIdeograph465a': '\xe4\x99\x9a',
'CjkUnifiedIdeograph465f': '\xe4\x99\x9f',
'CjkUnifiedIdeograph465e': '\xe4\x99\x9e',
'LatinSmallLetterGWithMacron': '\xe1\xb8\xa1',
'CjkUnifiedIdeograph66d0': '\xe6\x9b\x90',
'UnicodeCharacter0xe78d': '\xee\x9e\x8d',
'UnicodeCharacter0xe78e': '\xee\x9e\x8e',
'UnicodeCharacter0xe78f': '\xee\x9e\x8f',
'CjkUnifiedIdeograph66d1': '\xe6\x9b\x91',
'UnicodeCharacter0xe78a': '\xee\x9e\x8a',
'UnicodeCharacter0xe78b': '\xee\x9e\x8b',
'UnicodeCharacter0xe78c': '\xee\x9e\x8c',
'CjkUnifiedIdeograph66d6': '\xe6\x9b\x96',
'UnicodeCharacter0xa677': '\xea\x99\xb7',
'CjkUnifiedIdeograph66d7': '\xe6\x9b\x97',
'CjkUnifiedIdeograph48b8': '\xe4\xa2\xb8',
'CjkUnifiedIdeograph48b9': '\xe4\xa2\xb9',
'CjkUnifiedIdeograph66d4': '\xe6\x9b\x94',
'CjkUnifiedIdeograph48b0': '\xe4\xa2\xb0',
'CjkUnifiedIdeograph48b1': '\xe4\xa2\xb1',
'CjkUnifiedIdeograph48b2': '\xe4\xa2\xb2',
'CjkUnifiedIdeograph48b3': '\xe4\xa2\xb3',
'CjkUnifiedIdeograph48b4': '\xe4\xa2\xb4',
'CjkUnifiedIdeograph48b5': '\xe4\xa2\xb5',
'CjkUnifiedIdeograph48b6': '\xe4\xa2\xb6',
'CjkUnifiedIdeograph48b7': '\xe4\xa2\xb7',
'GreekCapitalLetterEtaWithDasiaAndProsgegrammeni': '\xe1\xbe\x99',
'CyrillicThousandsSign': '\xd2\x82',
'HangulSyllablePaet': '\xed\x8d\x81',
'HangulSyllableJjyong': '\xec\xad\x81',
'ArabicLetterKehehIsolatedForm': '\xef\xae\x8e',
'HangulSyllableJjyonh': '\xec\xac\xb2',
'HangulSyllableJjyonj': '\xec\xac\xb1',
'CjkRadicalBrushTwo': '\xe2\xba\xbb',
'UnicodeCharacter0xe160': '\xee\x85\xa0',
'VaiSyllableO': '\xea\x96\xba',
'VaiSyllableI': '\xea\x94\xa4',
'UnicodeCharacter0xe161': '\xee\x85\xa1',
'CjkUnifiedIdeograph529d': '\xe5\x8a\x9d',
'CjkUnifiedIdeograph529e': '\xe5\x8a\x9e',
'CjkUnifiedIdeograph529f': '\xe5\x8a\x9f',
'CjkUnifiedIdeograph529a': '\xe5\x8a\x9a',
'CjkUnifiedIdeograph529b': '\xe5\x8a\x9b',
'CjkUnifiedIdeograph529c': '\xe5\x8a\x9c',
'UnicodeCharacter0xf846': '\xef\xa1\x86',
'HangulSyllableKap': '\xec\xba\x8e',
'UnicodeCharacter0xf844': '\xef\xa1\x84',
'HexagramForInfluence': '\xe4\xb7\x9e',
'UnicodeCharacter0xf842': '\xef\xa1\x82',
'BalineseDigitFive': '\xe1\xad\x95',
'LatinCapitalLetterNWithTilde': '\xc3\x91',
'UnicodeCharacter0xf841': '\xef\xa1\x81',
'VaiSyllableU': '\xea\x96\x95',
'UnicodeCharacter0xf848': '\xef\xa1\x88',
'HangulLetterSioscieuc': '\xe3\x85\xbe',
'ArabicReversedDamma': '\xd9\x9d',
'HangulSyllableKas': '\xec\xba\x87',
'SymbolForEnquiry': '\xe2\x90\x85',
'UnicodeCharacter0xe166': '\xee\x85\xa6',
'UnicodeCharacter0xf72d': '\xef\x9c\xad',
'HangulSyllableNim': '\xeb\x8b\x98',
'UnicodeCharacter0xe167': '\xee\x85\xa7',
'HangulSyllableNil': '\xeb\x8b\x90',
'ThaanaLetterGaafu': '\xde\x8e',
'CjkUnifiedIdeograph7832': '\xe7\xa0\xb2',
'UnicodeCharacter0xf740': '\xef\x9d\x80',
'HangulSyllableJub': '\xec\xa4\x8d',
'UnicodeCharacter0xe169': '\xee\x85\xa9',
'UnicodeCharacter0xf51a': '\xef\x94\x9a',
'UnicodeCharacter0xf84f': '\xef\xa1\x8f',
'HangulSyllableKah': '\xec\xba\x8f',
'UnicodeCharacter0xf84d': '\xef\xa1\x8d',
'UnicodeCharacter0xf84e': '\xef\xa1\x8e',
'UnicodeCharacter0xf84b': '\xef\xa1\x8b',
'CjkUnifiedIdeograph8c1d': '\xe8\xb0\x9d',
'HangulSyllableJjwess': '\xec\xae\x94',
'CjkUnifiedIdeograph7836': '\xe7\xa0\xb6',
'UnicodeCharacter0xe7dc': '\xee\x9f\x9c',
'FullwidthReverseSolidus': '\xef\xbc\xbc',
'CjkUnifiedIdeograph7837': '\xe7\xa0\xb7',
'HangulSyllableBboe': '\xeb\xbe\x94',
'WhiteDraughtsMan': '\xe2\x9b\x80',
'CjkUnifiedIdeograph5298': '\xe5\x8a\x98',
'CjkUnifiedIdeograph5299': '\xe5\x8a\x99',
'CjkUnifiedIdeograph5294': '\xe5\x8a\x94',
'CjkUnifiedIdeograph5295': '\xe5\x8a\x95',
'CjkUnifiedIdeograph5296': '\xe5\x8a\x96',
'CjkUnifiedIdeograph5297': '\xe5\x8a\x97',
'CjkUnifiedIdeograph5290': '\xe5\x8a\x90',
'CjkUnifiedIdeograph5291': '\xe5\x8a\x91',
'CjkUnifiedIdeograph5292': '\xe5\x8a\x92',
'CjkUnifiedIdeograph5293': '\xe5\x8a\x93',
'DevanagariLetterFa': '\xe0\xa5\x9e',
'CircledNumberFortyThree': '\xe3\x8a\xb8',
'HangulSyllableJum': '\xec\xa4\x8c',
'HangulSyllableRyeot': '\xeb\xa0\xbd',
'GreekCapitalLetterEpsilonWithPsiliAndOxia': '\xe1\xbc\x9c',
'HangulSyllableCwab': '\xec\xb4\xb5',
'HangulSyllableRyeop': '\xeb\xa0\xbe',
'HangulSyllableRyeon': '\xeb\xa0\xa8',
'HangulSyllableRyeom': '\xeb\xa0\xb4',
'HangulSyllableRyeol': '\xeb\xa0\xac',
'HangulSyllableRyeok': '\xeb\xa0\xbc',
'HangulSyllableRyeoj': '\xeb\xa0\xba',
'UnicodeCharacter0xe33b': '\xee\x8c\xbb',
'HangulSyllableRyeoh': '\xeb\xa0\xbf',
'HangulSyllableRyeog': '\xeb\xa0\xa5',
'HangulSyllableRyeod': '\xeb\xa0\xab',
'HangulSyllableRyeoc': '\xeb\xa0\xbb',
'HangulSyllableRyeob': '\xeb\xa0\xb5',
'HangulSyllablePael': '\xed\x8c\xb0',
'CjkUnifiedIdeograph3449': '\xe3\x91\x89',
'CjkUnifiedIdeograph3448': '\xe3\x91\x88',
'CjkUnifiedIdeograph3447': '\xe3\x91\x87',
'CjkUnifiedIdeograph3446': '\xe3\x91\x86',
'CjkUnifiedIdeograph3445': '\xe3\x91\x85',
'CjkUnifiedIdeograph3444': '\xe3\x91\x84',
'CjkUnifiedIdeograph3443': '\xe3\x91\x83',
'CjkUnifiedIdeograph3442': '\xe3\x91\x82',
'CjkUnifiedIdeograph3441': '\xe3\x91\x81',
'CjkUnifiedIdeograph3440': '\xe3\x91\x80',
'HangulSyllableCwah': '\xec\xb4\xbf',
'YiSyllableCup': '\xea\x8b\x87',
'UnicodeCharacter0xdc1a': '\xed\xb0\x9a',
'UnicodeCharacter0xdc1b': '\xed\xb0\x9b',
'UnicodeCharacter0xdc1c': '\xed\xb0\x9c',
'UnicodeCharacter0xdc1d': '\xed\xb0\x9d',
'HangulSyllableCwaj': '\xec\xb4\xba',
'UnicodeCharacter0xdc1f': '\xed\xb0\x9f',
'CjkUnifiedIdeograph7f92': '\xe7\xbe\x92',
'LatinSmallLetterTWithCircumflexBelow': '\xe1\xb9\xb1',
'CjkUnifiedIdeograph64e0': '\xe6\x93\xa0',
'HangulSyllableSyeols': '\xec\x85\xa0',
'CjkUnifiedIdeograph7f93': '\xe7\xbe\x93',
'HangulSyllableCogs': '\xec\xb4\x8b',
'CjkUnifiedIdeograph651c': '\xe6\x94\x9c',
'CjkUnifiedIdeograph7f90': '\xe7\xbe\x90',
'HangulSyllableCwam': '\xec\xb4\xb4',
'CjkUnifiedIdeograph7f91': '\xe7\xbe\x91',
'HangulSyllableGgweobs': '\xea\xbf\xa6',
'HangulSyllableCwan': '\xec\xb4\xa8',
'ArabicLigatureYehWithKhahInitialForm': '\xef\xb3\x9c',
'CjkUnifiedIdeograph7f96': '\xe7\xbe\x96',
'HyphenWithDiaeresis': '\xe2\xb8\x9a',
'HangulSyllableCogg': '\xec\xb4\x8a',
'CjkUnifiedIdeograph7f97': '\xe7\xbe\x97',
'UnicodeCharacter0xdc18': '\xed\xb0\x98',
'UnicodeCharacter0xdc19': '\xed\xb0\x99',
'UnicodeCharacter0xdc40': '\xed\xb1\x80',
'CjkUnifiedIdeograph7f94': '\xe7\xbe\x94',
'UnicodeCharacter0xdc10': '\xed\xb0\x90',
'UnicodeCharacter0xdc11': '\xed\xb0\x91',
'UnicodeCharacter0xdc12': '\xed\xb0\x92',
'CjkUnifiedIdeograph7f95': '\xe7\xbe\x95',
'UnicodeCharacter0xdc14': '\xed\xb0\x94',
'UnicodeCharacter0xdc15': '\xed\xb0\x95',
'UnicodeCharacter0xdc16': '\xed\xb0\x96',
'UnicodeCharacter0xdc17': '\xed\xb0\x97',
'HangulSyllableJibs': '\xec\xa7\x92',
'CjkUnifiedIdeograph66dc': '\xe6\x9b\x9c',
'CjkUnifiedIdeograph344f': '\xe3\x91\x8f',
'CjkUnifiedIdeograph344e': '\xe3\x91\x8e',
'CjkUnifiedIdeograph344d': '\xe3\x91\x8d',
'CjkUnifiedIdeograph344c': '\xe3\x91\x8c',
'CjkUnifiedIdeograph344b': '\xe3\x91\x8b',
'CjkUnifiedIdeograph344a': '\xe3\x91\x8a',
'CjkUnifiedIdeograph7630': '\xe7\x98\xb0',
'GreekCapitalLetterOmegaWithDasiaAndPerispomeniAndProsgegrammeni': '\xe1\xbe\xaf',
'BoxDrawingsDoubleDownAndHorizontal': '\xe2\x95\xa6',
'CjkUnifiedIdeograph66da': '\xe6\x9b\x9a',
'CjkUnifiedIdeograph7634': '\xe7\x98\xb4',
'CjkUnifiedIdeograph7635': '\xe7\x98\xb5',
'CjkUnifiedIdeograph7636': '\xe7\x98\xb6',
'CjkUnifiedIdeograph7637': '\xe7\x98\xb7',
'CjkUnifiedIdeograph7638': '\xe7\x98\xb8',
'MyanmarDigitFive': '\xe1\x81\x85',
'CjkUnifiedIdeograph81ba': '\xe8\x86\xba',
'CjkUnifiedIdeograph81bc': '\xe8\x86\xbc',
'CjkUnifiedIdeograph81bb': '\xe8\x86\xbb',
'CjkUnifiedIdeograph81be': '\xe8\x86\xbe',
'CjkUnifiedIdeograph66dd': '\xe6\x9b\x9d',
'CjkUnifiedIdeograph81bf': '\xe8\x86\xbf',
'DoubleWavyOverline': '\xef\xb9\x8c',
'AplFunctionalSymbolQuadSlash': '\xe2\x8d\x81',
'CjkUnifiedIdeograph8c41': '\xe8\xb1\x81',
'CjkUnifiedIdeograph8c40': '\xe8\xb1\x80',
'CjkUnifiedIdeograph8c43': '\xe8\xb1\x83',
'CjkUnifiedIdeograph8c42': '\xe8\xb1\x82',
'CjkUnifiedIdeograph8c45': '\xe8\xb1\x85',
'CjkUnifiedIdeograph8c44': '\xe8\xb1\x84',
'CjkUnifiedIdeograph8c47': '\xe8\xb1\x87',
'CjkUnifiedIdeograph8c46': '\xe8\xb1\x86',
'CjkUnifiedIdeograph8c49': '\xe8\xb1\x89',
'CjkUnifiedIdeograph8c48': '\xe8\xb1\x88',
'GreekSmallLetterOmicronWithPsiliAndOxia': '\xe1\xbd\x84',
'CjkCompatibilityIdeographfa0e': '\xef\xa8\x8e',
'SymbolForEndOfText': '\xe2\x90\x83',
'HangulJongseongMieumrieul': '\xe1\x87\x9b',
'CjkUnifiedIdeograph81b1': '\xe8\x86\xb1',
'CjkUnifiedIdeograph81b0': '\xe8\x86\xb0',
'CjkUnifiedIdeograph81b3': '\xe8\x86\xb3',
'CjkUnifiedIdeograph81b2': '\xe8\x86\xb2',
'CjkUnifiedIdeograph81b5': '\xe8\x86\xb5',
'GreekSmallLetterAlphaWithPsiliAndVariaAndYpogegrammeni': '\xe1\xbe\x82',
'CjkUnifiedIdeograph81b7': '\xe8\x86\xb7',
'CjkUnifiedIdeograph81b6': '\xe8\x86\xb6',
'CjkUnifiedIdeograph81b9': '\xe8\x86\xb9',
'CjkUnifiedIdeograph81b8': '\xe8\x86\xb8',
'CjkUnifiedIdeograph763a': '\xe7\x98\xba',
'CjkUnifiedIdeograph763b': '\xe7\x98\xbb',
'CjkUnifiedIdeograph763c': '\xe7\x98\xbc',
'CjkUnifiedIdeograph763d': '\xe7\x98\xbd',
'CjkUnifiedIdeograph763e': '\xe7\x98\xbe',
'CjkUnifiedIdeograph763f': '\xe7\x98\xbf',
'CjkUnifiedIdeograph905c': '\xe9\x81\x9c',
'CjkUnifiedIdeograph8f02': '\xe8\xbc\x82',
'CjkUnifiedIdeograph86eb': '\xe8\x9b\xab',
'HangulSyllableDyuk': '\xeb\x93\x98',
'HangulSyllableSsuls': '\xec\x91\xb0',
'CjkUnifiedIdeograph905a': '\xe9\x81\x9a',
'HangulSyllableSsult': '\xec\x91\xb1',
'HangulSyllableDyuj': '\xeb\x93\x96',
'CjkUnifiedIdeograph8f04': '\xe8\xbc\x84',
'CjkUnifiedIdeograph8f05': '\xe8\xbc\x85',
'CjkUnifiedIdeograph8c4a': '\xe8\xb1\x8a',
'CjkUnifiedIdeograph8c4c': '\xe8\xb1\x8c',
'CjkUnifiedIdeograph6bf7': '\xe6\xaf\xb7',
'CjkUnifiedIdeograph8c4e': '\xe8\xb1\x8e',
'CjkUnifiedIdeograph8c4d': '\xe8\xb1\x8d',
'HangulSyllableSsulg': '\xec\x91\xad',
'CjkUnifiedIdeograph8c4f': '\xe8\xb1\x8f',
'HangulSyllableSsulh': '\xec\x91\xb3',
'CjkUnifiedIdeograph905e': '\xe9\x81\x9e',
'HangulSyllableSsulm': '\xec\x91\xae',
'ArabicMaddahAbove': '\xd9\x93',
'HangulSyllableDyun': '\xeb\x93\x84',
'CjkUnifiedIdeograph5904': '\xe5\xa4\x84',
'CjkUnifiedIdeograph5905': '\xe5\xa4\x85',
'CjkUnifiedIdeograph5906': '\xe5\xa4\x86',
'CjkUnifiedIdeograph5907': '\xe5\xa4\x87',
'CjkUnifiedIdeograph5900': '\xe5\xa4\x80',
'CjkUnifiedIdeograph5901': '\xe5\xa4\x81',
'CjkUnifiedIdeograph5902': '\xe5\xa4\x82',
'CjkUnifiedIdeograph5903': '\xe5\xa4\x83',
'HangulSyllableDyul': '\xeb\x93\x88',
'CjkUnifiedIdeograph5908': '\xe5\xa4\x88',
'NewTaiLueLetterHighYa': '\xe1\xa6\x8a',
'ChamLetterFinalCh': '\xea\xa9\x84',
'LatinSmallLetterSaltillo': '\xea\x9e\x8c',
'HexagramForBreakthrough': '\xe4\xb7\xaa',
'HangulSyllableDot': '\xeb\x8f\x9d',
'HangulSyllableDyub': '\xeb\x93\x91',
'EqualsSignAndSlantedParallelWithTildeAbove': '\xe2\xa7\xa4',
'UnicodeCharacter0xe1d2': '\xee\x87\x92',
'CjkUnifiedIdeograph9cfd': '\xe9\xb3\xbd',
'IdeographicDescriptionCharacterLeftToRight': '\xe2\xbf\xb0',
'HangulSyllableDyug': '\xeb\x93\x81',
'HangulSyllableDdyeobs': '\xeb\x97\xaa',
'CjkUnifiedIdeograph94ef': '\xe9\x93\xaf',
'AllAroundprofile': '\xe2\x8c\xae',
'CjkUnifiedIdeograph94ed': '\xe9\x93\xad',
'CjkUnifiedIdeograph94ee': '\xe9\x93\xae',
'CjkUnifiedIdeograph94eb': '\xe9\x93\xab',
'CjkUnifiedIdeograph94ec': '\xe9\x93\xac',
'CjkUnifiedIdeograph94ea': '\xe9\x93\xaa',
'HangulSyllableDyud': '\xeb\x93\x87',
'HangulSyllableDdwabs': '\xeb\x98\xbe',
'YiSyllableIet': '\xea\x80\x84',
'YiSyllableIep': '\xea\x80\x87',
'LatinSmallLetterAWithCircumflexAndAcute': '\xe1\xba\xa5',
'YiSyllableIex': '\xea\x80\x85',
'CjkUnifiedIdeograph590d': '\xe5\xa4\x8d',
'CjkUnifiedIdeograph590e': '\xe5\xa4\x8e',
'CjkUnifiedIdeograph590f': '\xe5\xa4\x8f',
'CjkUnifiedIdeograph590a': '\xe5\xa4\x8a',
'CjkUnifiedIdeograph590b': '\xe5\xa4\x8b',
'CjkUnifiedIdeograph590c': '\xe5\xa4\x8c',
'UnicodeCharacter0xfb08': '\xef\xac\x88',
'UnicodeCharacter0xfb09': '\xef\xac\x89',
'UnicodeCharacter0xe02f': '\xee\x80\xaf',
'CjkUnifiedIdeograph94e6': '\xe9\x93\xa6',
'CjkUnifiedIdeograph94e7': '\xe9\x93\xa7',
'LeftLowParaphraseBracket': '\xe2\xb8\x9c',
'CjkUnifiedIdeograph94e5': '\xe9\x93\xa5',
'CjkUnifiedIdeograph94e2': '\xe9\x93\xa2',
'CjkUnifiedIdeograph94e3': '\xe9\x93\xa3',
'CjkUnifiedIdeograph94e0': '\xe9\x93\xa0',
'EthiopicDigitFour': '\xe1\x8d\xac',
'HangulSyllableGweot': '\xea\xb6\xa1',
'TrigramForFire': '\xe2\x98\xb2',
'HangulSyllableGweop': '\xea\xb6\xa2',
'CjkUnifiedIdeograph94e8': '\xe9\x93\xa8',
'CjkUnifiedIdeograph94e9': '\xe9\x93\xa9',
'HangulSyllableGweol': '\xea\xb6\x90',
'HangulSyllableGweom': '\xea\xb6\x98',
'HangulSyllableGweon': '\xea\xb6\x8c',
'HangulSyllableGweoh': '\xea\xb6\xa3',
'HangulSyllableGweoj': '\xea\xb6\x9e',
'HangulSyllableGweok': '\xea\xb6\xa0',
'HangulSyllableGweod': '\xea\xb6\x8f',
'CopticSmallLetterAlfa': '\xe2\xb2\x81',
'HangulSyllableGweog': '\xea\xb6\x89',
'HangulSyllablePung': '\xed\x92\x8d',
'HangulSyllableGweob': '\xea\xb6\x99',
'HangulSyllableGweoc': '\xea\xb6\x9f',
'CjkUnifiedIdeograph9dbc': '\xe9\xb6\xbc',
'HangulSyllableMyang': '\xeb\xa8\x95',
'OutlinedBlackStar': '\xe2\x9c\xad',
'!<<-': '\xe2\x87\x8d',
'HangulSyllableTyung': '\xed\x8a\xb1',
'HangulSyllableDop': '\xeb\x8f\x9e',
'UnicodeCharacter0xedfa': '\xee\xb7\xba',
'UnicodeCharacter0xedfc': '\xee\xb7\xbc',
'UnicodeCharacter0xedfb': '\xee\xb7\xbb',
'UnicodeCharacter0xedfe': '\xee\xb7\xbe',
'UnicodeCharacter0xdc4e': '\xed\xb1\x8e',
'UnicodeCharacter0xedff': '\xee\xb7\xbf',
'HangulSyllableHyig': '\xed\x9d\xad',
'YiSyllableZax': '\xea\x8a\x95',
'UnicodeCharacter0xf51d': '\xef\x94\x9d',
'CjkUnifiedIdeograph9dbb': '\xe9\xb6\xbb',
'CjkUnifiedIdeograph866a': '\xe8\x99\xaa',
'CyrillicCapitalLetterGhe': '\xd0\x93',
'CjkUnifiedIdeograph58f3': '\xe5\xa3\xb3',
'CopticCapitalLetterFei': '\xcf\xa4',
'CjkUnifiedIdeograph8c16': '\xe8\xb0\x96',
'HangulSyllableRwenh': '\xeb\xa4\xa6',
'UnicodeCharacter0xedf1': '\xee\xb7\xb1',
'UnicodeCharacter0xedf0': '\xee\xb7\xb0',
'UnicodeCharacter0xedf3': '\xee\xb7\xb3',
'YiSyllableHxip': '\xea\x89\x89',
'UnicodeCharacter0xedf5': '\xee\xb7\xb5',
'UnicodeCharacter0xedf4': '\xee\xb7\xb4',
'UnicodeCharacter0xedf7': '\xee\xb7\xb7',
'HexagramForHoldingTogether': '\xe4\xb7\x87',
'UnicodeCharacter0xedf9': '\xee\xb7\xb9',
'UnicodeCharacter0xedf8': '\xee\xb7\xb8',
'YiSyllableHxix': '\xea\x89\x87',
'HangulSyllableSswaess': '\xec\x90\xa4',
'YiSyllableHxie': '\xea\x89\x8c',
'UnicodeCharacter0xe650': '\xee\x99\x90',
'GreekDasia': '\xe1\xbf\xbe',
'CjkUnifiedIdeograph9cfb': '\xe9\xb3\xbb',
'CjkUnifiedIdeograph8c19': '\xe8\xb0\x99',
'HangulSyllableSobs': '\xec\x86\x9e',
'HangulSyllableDom': '\xeb\x8f\x94',
'ArabicLetterZain': '\xd8\xb2',
'HangulSyllableBbenj': '\xeb\xbb\xb1',
'HangulSyllableHyip': '\xed\x9e\x86',
'UnicodeCharacter0xe651': '\xee\x99\x91',
'CircledPostalMark': '\xe3\x80\xb6',
'HangulSyllableBbwabs': '\xeb\xbd\xae',
'CyrillicCapitalLetterDoubleMonocularO': '\xea\x99\xac',
'HangulSyllableGaej': '\xea\xb0\xb2',
'HangulSyllableGaek': '\xea\xb0\xb4',
'HangulSyllableGaeh': '\xea\xb0\xb7',
'HangulSyllableGaen': '\xea\xb0\xa0',
'HangulSyllableGael': '\xea\xb0\xa4',
'CjkUnifiedIdeograph8d61': '\xe8\xb5\xa1',
'HangulSyllableGaeb': '\xea\xb0\xad',
'HangulSyllableGaec': '\xea\xb0\xb3',
'HangulSyllableHyit': '\xed\x9e\x85',
'HangulSyllableGaeg': '\xea\xb0\x9d',
'HangulSyllableGaed': '\xea\xb0\xa3',
'CjkUnifiedIdeograph597f': '\xe5\xa5\xbf',
'HangulSyllableByam': '\xeb\xb1\x9c',
'CjkUnifiedIdeograph7ad8': '\xe7\xab\x98',
'CjkUnifiedIdeograph7ad9': '\xe7\xab\x99',
'CjkUnifiedIdeograph7ad6': '\xe7\xab\x96',
'CjkUnifiedIdeograph7ad7': '\xe7\xab\x97',
'CjkUnifiedIdeograph7ad4': '\xe7\xab\x94',
'CjkUnifiedIdeograph7ad5': '\xe7\xab\x95',
'CjkUnifiedIdeograph7ad2': '\xe7\xab\x92',
'CjkUnifiedIdeograph7ad3': '\xe7\xab\x93',
'CjkUnifiedIdeograph7ad0': '\xe7\xab\x90',
'CjkUnifiedIdeograph7ad1': '\xe7\xab\x91',
'CjkUnifiedIdeograph634f': '\xe6\x8d\x8f',
'KannadaDigitOne': '\xe0\xb3\xa7',
'CjkUnifiedIdeograph634d': '\xe6\x8d\x8d',
'CjkUnifiedIdeograph634c': '\xe6\x8d\x8c',
'CjkUnifiedIdeograph634b': '\xe6\x8d\x8b',
'CjkUnifiedIdeograph634a': '\xe6\x8d\x8a',
'Epsilon': '\xce\x95',
'SmallPercentSign': '\xef\xb9\xaa',
'HangulSyllableJjyess': '\xec\xaa\xb4',
'NorthEastWhiteArrow': '\xe2\xac\x80',
'UnicodeCharacter0xdf43': '\xed\xbd\x83',
'CjkUnifiedIdeograph7adf': '\xe7\xab\x9f',
'CjkUnifiedIdeograph8235': '\xe8\x88\xb5',
'CjkUnifiedIdeograph7add': '\xe7\xab\x9d',
'CjkUnifiedIdeograph7ade': '\xe7\xab\x9e',
'WheelOfDharma': '\xe2\x98\xb8',
'CjkUnifiedIdeograph7adc': '\xe7\xab\x9c',
'CjkUnifiedIdeograph7ada': '\xe7\xab\x9a',
'UnicodeCharacter0xe3eb': '\xee\x8f\xab',
'UnicodeCharacter0xe3ea': '\xee\x8f\xaa',
'EthiopicFullStop': '\xe1\x8d\xa2',
'CjkUnifiedIdeograph35cc': '\xe3\x97\x8c',
'InvertedInterrobang': '\xe2\xb8\x98',
'UnicodeCharacter0xf084': '\xef\x82\x84',
'DownwardsArrowWithDoubleStroke': '\xe2\x87\x9f',
'RejangVowelSignEa': '\xea\xa5\x8e',
'LatinCapitalLetterKWithStrokeAndDiagonalStroke': '\xea\x9d\x84',
'CopticCapitalLetterOldCopticShei': '\xe2\xb3\x84',
'CjkUnifiedIdeograph7383': '\xe7\x8e\x83',
'UnicodeCharacter0xe3ef': '\xee\x8f\xaf',
'CjkUnifiedIdeograph6347': '\xe6\x8d\x87',
'CjkUnifiedIdeograph6346': '\xe6\x8d\x86',
'CjkUnifiedIdeograph6345': '\xe6\x8d\x85',
'CjkUnifiedIdeograph6344': '\xe6\x8d\x84',
'CjkUnifiedIdeograph6343': '\xe6\x8d\x83',
'MathematicalLeftDoubleAngleBracket': '\xe2\x9f\xaa',
'CjkUnifiedIdeograph6341': '\xe6\x8d\x81',
'ArmenianCapitalLetterYiwn': '\xd5\x92',
'LatinSmallLetterBWithLineBelow': '\xe1\xb8\x87',
'CjkUnifiedIdeograph6349': '\xe6\x8d\x89',
'CjkUnifiedIdeograph6348': '\xe6\x8d\x88',
'LatinLetterPharyngealVoicedFricative': '\xca\x95',
'HangulSyllableHweong': '\xed\x9b\xb5',
'UnicodeCharacter0xc04': '\xe0\xb0\x84',
'HangulSyllableGgoenj': '\xea\xbe\x85',
'HangulSyllableGgoenh': '\xea\xbe\x86',
'UnicodeCharacter0xc00': '\xe0\xb0\x80',
'CjkUnifiedIdeograph8418': '\xe8\x90\x98',
'UnicodeCharacter0xf3f6': '\xef\x8f\xb6',
'CjkUnifiedIdeograph8415': '\xe8\x90\x95',
'LessthanButNotEqualTo': '\xe2\x89\xa8',
'CjkUnifiedIdeograph8417': '\xe8\x90\x97',
'CjkUnifiedIdeograph8416': '\xe8\x90\x96',
'CjkUnifiedIdeograph8411': '\xe8\x90\x91',
'CjkUnifiedIdeograph8410': '\xe8\x90\x90',
'CjkUnifiedIdeograph8413': '\xe8\x90\x93',
'CjkUnifiedIdeograph8412': '\xe8\x90\x92',
'HangulSyllablePyom': '\xed\x91\xac',
'UnicodeCharacter0x2b5f': '\xe2\xad\x9f',
'UnicodeCharacter0x2b5e': '\xe2\xad\x9e',
'UnicodeCharacter0x2b5d': '\xe2\xad\x9d',
'UnicodeCharacter0x2b5c': '\xe2\xad\x9c',
'UnicodeCharacter0x2b5b': '\xe2\xad\x9b',
'UnicodeCharacter0x2b5a': '\xe2\xad\x9a',
'HangulSyllablePyoj': '\xed\x91\xb2',
'CjkUnifiedIdeograph66d5': '\xe6\x9b\x95',
'HangulSyllablePyod': '\xed\x91\xa3',
'HangulSyllablePyog': '\xed\x91\x9d',
'HangulSyllableGgu': '\xea\xbe\xb8',
'HangulSyllablePyob': '\xed\x91\xad',
'CjkUnifiedIdeograph7bbf': '\xe7\xae\xbf',
'CjkUnifiedIdeograph7bbe': '\xe7\xae\xbe',
'CjkUnifiedIdeograph7bbd': '\xe7\xae\xbd',
'CjkUnifiedIdeograph7bbc': '\xe7\xae\xbc',
'UnicodeCharacter0x27bf': '\xe2\x9e\xbf',
'CjkUnifiedIdeograph7bba': '\xe7\xae\xba',
'CjkUnifiedIdeograph6a39': '\xe6\xa8\xb9',
'UnicodeCharacter0xc0d': '\xe0\xb0\x8d',
'BraillePatternDots134678': '\xe2\xa3\xad',
'CjkUnifiedIdeograph841d': '\xe8\x90\x9d',
'CjkUnifiedIdeograph841f': '\xe8\x90\x9f',
'CjkUnifiedIdeograph841a': '\xe8\x90\x9a',
'CjkUnifiedIdeograph841c': '\xe8\x90\x9c',
'CjkUnifiedIdeograph841b': '\xe8\x90\x9b',
'IntegralWithIntersection': '\xe2\xa8\x99',
'HangulSyllableBaess': '\xeb\xb1\x84',
'CjkUnifiedIdeograph7bb9': '\xe7\xae\xb9',
'CjkUnifiedIdeograph7bb8': '\xe7\xae\xb8',
'CjkUnifiedIdeograph7bb7': '\xe7\xae\xb7',
'MyanmarSignWesternPwoKarenTone1': '\xe1\x81\xa9',
'MyanmarSignWesternPwoKarenTone2': '\xe1\x81\xaa',
'UnicodeCharacter0x27b0': '\xe2\x9e\xb0',
'MyanmarSignWesternPwoKarenTone4': '\xe1\x81\xac',
'MyanmarSignWesternPwoKarenTone5': '\xe1\x81\xad',
'CjkUnifiedIdeograph7bb1': '\xe7\xae\xb1',
'CjkUnifiedIdeograph7bb0': '\xe7\xae\xb0',
'UnicodeCharacter0x2b57': '\xe2\xad\x97',
'UnicodeCharacter0x2b56': '\xe2\xad\x96',
'UnicodeCharacter0x2b55': '\xe2\xad\x95',
'HangulSyllableDyulh': '\xeb\x93\x8f',
'CjkUnifiedIdeograph86e9': '\xe8\x9b\xa9',
'CjkUnifiedIdeograph6a33': '\xe6\xa8\xb3',
'UnicodeCharacter0x2b59': '\xe2\xad\x99',
'UnicodeCharacter0x2b58': '\xe2\xad\x98',
'UpwardsWhiteArrowOnPedestal': '\xe2\x87\xab',
'EthiopicSyllableSsi': '\xe2\xb6\xa2',
'VaiSyllableYe': '\xea\x98\x82',
'CombiningBridgeAbove': '\xcd\x86',
'MalayalamFractionOneHalf': '\xe0\xb5\xb4',
'EthiopicSyllableSso': '\xe2\xb6\xa6',
'CjkUnifiedIdeograph3f6c': '\xe3\xbd\xac',
'CjkUnifiedIdeograph3f6b': '\xe3\xbd\xab',
'CjkUnifiedIdeograph3f6a': '\xe3\xbd\xaa',
'UnicodeCharacter0xa7fa': '\xea\x9f\xba',
'EthiopicSyllableSse': '\xe2\xb6\xa5',
'CjkUnifiedIdeograph3f6e': '\xe3\xbd\xae',
'CjkUnifiedIdeograph3f6d': '\xe3\xbd\xad',
'YiSyllableHmyx': '\xea\x82\xa9',
'VaiSyllableYu': '\xea\x96\xb3',
'YiSyllableHmyr': '\xea\x82\xad',
'RunicMultiplePunctuation': '\xe1\x9b\xac',
'YiSyllableHmyp': '\xea\x82\xab',
'EthiopicSyllableSsu': '\xe2\xb6\xa1',
'HangulSyllableSyult': '\xec\x8a\x95',
'HangulSyllableMwass': '\xeb\xab\x98',
'SaurashtraDigitThree': '\xea\xa3\x93',
'UnicodeCharacter0xfd43': '\xef\xb5\x83',
'UnicodeCharacter0xf156': '\xef\x85\x96',
'HangulSyllableJjwat': '\xec\xab\xb1',
'CjkUnifiedIdeograph959d': '\xe9\x96\x9d',
'HalfwidthKatakanaLetterNi': '\xef\xbe\x86',
'UnicodeCharacter0xfd46': '\xef\xb5\x86',
'HangulSyllableJjwas': '\xec\xab\xab',
'CjkUnifiedIdeograph3f69': '\xe3\xbd\xa9',
'CjkUnifiedIdeograph3f68': '\xe3\xbd\xa8',
'HangulSyllableJjwap': '\xec\xab\xb2',
'CjkUnifiedIdeograph3f63': '\xe3\xbd\xa3',
'CjkUnifiedIdeograph3f62': '\xe3\xbd\xa2',
'CjkUnifiedIdeograph3f61': '\xe3\xbd\xa1',
'CjkUnifiedIdeograph3f60': '\xe3\xbd\xa0',
'CjkUnifiedIdeograph3f67': '\xe3\xbd\xa7',
'CjkUnifiedIdeograph3f66': '\xe3\xbd\xa6',
'CjkUnifiedIdeograph3f65': '\xe3\xbd\xa5',
'CjkUnifiedIdeograph3f64': '\xe3\xbd\xa4',
'BraillePatternDots1245': '\xe2\xa0\x9b',
'BraillePatternDots1246': '\xe2\xa0\xab',
'BraillePatternDots1247': '\xe2\xa1\x8b',
'WhiteDiamondWithCentredDot': '\xe2\x9f\x90',
'UnicodeCharacter0xde92': '\xed\xba\x92',
'HalfwidthKatakanaLetterNo': '\xef\xbe\x89',
'BraillePatternDots1248': '\xe2\xa2\x8b',
'HangulSyllableJjwal': '\xec\xab\xa0',
'SquareDaasu': '\xe3\x8c\xa4',
'LatinCapitalLetterEWithTilde': '\xe1\xba\xbc',
'UnicodeCharacter0xe3e3': '\xee\x8f\xa3',
'HangulSyllableJjwam': '\xec\xab\xa8',
'TeluguLetterNa': '\xe0\xb0\xa8',
'UnicodeCharacter0xe3e2': '\xee\x8f\xa2',
'VaiComma': '\xea\x98\x8d',
'UnicodeCharacter0xe3e1': '\xee\x8f\xa1',
'CjkUnifiedIdeograph6016': '\xe6\x80\x96',
'OriyaLetterBha': '\xe0\xac\xad',
'HangulSyllableJjwah': '\xec\xab\xb3',
'LatinSmallLetterAWithAcute': '\xc3\xa1',
'CjkRadicalBone': '\xe2\xbb\xa3',
'UnicodeCharacter0xe3e7': '\xee\x8f\xa7',
'CjkUnifiedIdeograph6014': '\xe6\x80\x94',
'HangulSyllableDoc': '\xeb\x8f\x9b',
'UnicodeCharacter0xe3e6': '\xee\x8f\xa6',
'BlackDiamondSuit': '\xe2\x99\xa6',
'UnicodeCharacter0xe3e5': '\xee\x8f\xa5',
'HangulSyllableJjwag': '\xec\xab\x99',
'HangulSyllableCoegg': '\xec\xb5\x9e',
'HangulSyllableJjwad': '\xec\xab\x9f',
'SquareKmCubed': '\xe3\x8e\xa6',
'CjkUnifiedIdeograph6b1e': '\xe6\xac\x9e',
'CjkUnifiedIdeograph6b1d': '\xe6\xac\x9d',
'BlackCircleWithTwoWhiteDots': '\xe2\x9a\x89',
'DoubleLogicalOr': '\xe2\xa9\x94',
'CjkUnifiedIdeograph6b1c': '\xe6\xac\x9c',
'CjkStrokeHzzp': '\xe3\x87\x8b',
'HangulSyllableJjwab': '\xec\xab\xa9',
'HangulSyllableDob': '\xeb\x8f\x95',
'ParenthesizedIdeographEight': '\xe3\x88\xa7',
'CjkStrokeHzzz': '\xe3\x87\x8e',
'ModifierLetterExtralowLeftstemToneBar': '\xea\x9c\x96',
'UnicodeCharacter0xe818': '\xee\xa0\x98',
'CircledIdeographStudy': '\xe3\x8a\xab',
'HangulSyllableBwaess': '\xeb\xb5\x80',
'UnicodeCharacter0xe814': '\xee\xa0\x94',
'UnicodeCharacter0xe815': '\xee\xa0\x95',
'UnicodeCharacter0xe816': '\xee\xa0\x96',
'LatinSmallLetterAWithDotAboveAndMacron': '\xc7\xa1',
'UnicodeCharacter0xe810': '\xee\xa0\x90',
'HangulSyllableHeolt': '\xed\x97\x95',
'ClockwiseGappedCircleArrow': '\xe2\x9f\xb3',
'UnicodeCharacter0xe813': '\xee\xa0\x93',
'HangulSyllableCulp': '\xec\xb6\xa2',
'HangulSyllableCuls': '\xec\xb6\xa0',
'GreekSmallLetterAlphaWithDasiaAndOxiaAndYpogegrammeni': '\xe1\xbe\x85',
'HangulSyllableCult': '\xec\xb6\xa1',
'HangulSyllableKwags': '\xec\xbd\xb3',
'UnicodeCharacter0xf3fb': '\xef\x8f\xbb',
'TibetanLetterRra': '\xe0\xbd\xac',
'CjkUnifiedIdeograph3acd': '\xe3\xab\x8d',
'CjkUnifiedIdeograph7e05': '\xe7\xb8\x85',
'HangulSyllableCulg': '\xec\xb6\x9d',
'HangulSyllableBbyap': '\xeb\xba\xb2',
'HangulSyllableCulm': '\xec\xb6\x9e',
'DevanagariDigitSeven': '\xe0\xa5\xad',
'UnicodeCharacter0xe81d': '\xee\xa0\x9d',
'UnicodeCharacter0xe81e': '\xee\xa0\x9e',
'UnicodeCharacter0xe81f': '\xee\xa0\x9f',
'SquareMuW': '\xe3\x8e\xbc',
'UnicodeCharacter0xe81a': '\xee\xa0\x9a',
'UnicodeCharacter0xe81b': '\xee\xa0\x9b',
'UnicodeCharacter0xe81c': '\xee\xa0\x9c',
'CjkUnifiedIdeograph6b15': '\xe6\xac\x95',
'CjkUnifiedIdeograph6b14': '\xe6\xac\x94',
'CjkUnifiedIdeograph6b17': '\xe6\xac\x97',
'CjkUnifiedIdeograph6b16': '\xe6\xac\x96',
'CjkUnifiedIdeograph6b11': '\xe6\xac\x91',
'CjkUnifiedIdeograph6b10': '\xe6\xac\x90',
'CjkUnifiedIdeograph6b13': '\xe6\xac\x93',
'CjkUnifiedIdeograph6b12': '\xe6\xac\x92',
'HangulSyllableJo': '\xec\xa1\xb0',
'CjkUnifiedIdeograph6b19': '\xe6\xac\x99',
'LongLeftRightDoubleArrow': '\xe2\x9f\xba',
'HangulSyllableBbigs': '\xec\x82\x93',
'HangulSyllableHeolh': '\xed\x97\x97',
'HangulSyllableTyeobs': '\xed\x85\xba',
'HangulSyllableBbyolb': '\xeb\xbe\xbb',
'BraillePatternDots348': '\xe2\xa2\x8c',
'BraillePatternDots347': '\xe2\xa1\x8c',
'CjkUnifiedIdeograph4b0a': '\xe4\xac\x8a',
'CjkUnifiedIdeograph4b0b': '\xe4\xac\x8b',
'CjkUnifiedIdeograph4b0c': '\xe4\xac\x8c',
'CjkUnifiedIdeograph4b0d': '\xe4\xac\x8d',
'CjkUnifiedIdeograph4b0e': '\xe4\xac\x8e',
'CjkUnifiedIdeograph4b0f': '\xe4\xac\x8f',
'UnicodeCharacter0xf648': '\xef\x99\x88',
'BraillePatternDots34578': '\xe2\xa3\x9c',
'LatinCapitalLetterSWithCaronAndDotAbove': '\xe1\xb9\xa6',
'CjkUnifiedIdeograph82f9': '\xe8\x8b\xb9',
'CjkUnifiedIdeograph82f2': '\xe8\x8b\xb2',
'CjkUnifiedIdeograph82f3': '\xe8\x8b\xb3',
'CjkUnifiedIdeograph82f0': '\xe8\x8b\xb0',
'CjkUnifiedIdeograph82f1': '\xe8\x8b\xb1',
'CjkUnifiedIdeograph82f6': '\xe8\x8b\xb6',
'MalayalamLetterHa': '\xe0\xb4\xb9',
'CjkUnifiedIdeograph82f4': '\xe8\x8b\xb4',
'CjkUnifiedIdeograph5978': '\xe5\xa5\xb8',
'HangulSyllableBbyag': '\xeb\xba\x99',
'SquareTon': '\xe3\x8c\xa7',
'UnicodeCharacter0xf649': '\xef\x99\x89',
'CjkUnifiedIdeograph4b08': '\xe4\xac\x88',
'CjkUnifiedIdeograph4b09': '\xe4\xac\x89',
'NewTaiLueDigitNine': '\xe1\xa7\x99',
'CjkUnifiedIdeograph7148': '\xe7\x85\x88',
'CjkUnifiedIdeograph4b00': '\xe4\xac\x80',
'CjkUnifiedIdeograph4b01': '\xe4\xac\x81',
'CjkUnifiedIdeograph4b02': '\xe4\xac\x82',
'CjkUnifiedIdeograph4b03': '\xe4\xac\x83',
'CjkUnifiedIdeograph4b04': '\xe4\xac\x84',
'CjkUnifiedIdeograph4b05': '\xe4\xac\x85',
'CjkUnifiedIdeograph4b06': '\xe4\xac\x86',
'CjkUnifiedIdeograph4b07': '\xe4\xac\x87',
'HangulSyllableBbyal': '\xeb\xba\xa0',
'LowerSevenEighthsBlock': '\xe2\x96\x87',
'HangulSyllableBbyam': '\xeb\xba\xa8',
'CjkUnifiedIdeograph7d24': '\xe7\xb4\xa4',
'HangulSyllableBbyan': '\xeb\xba\x9c',
'CyrillicSmallLetterBashkirKa': '\xd2\xa1',
'HangulSyllableYat': '\xec\x96\x95',
'UnicodeCharacter0xaaf6': '\xea\xab\xb6',
'HangulSyllableYas': '\xec\x96\x8f',
'HangulSyllableYap': '\xec\x96\x96',
'SquareMuL': '\xe3\x8e\x95',
'LaoLetterLoLoot': '\xe0\xba\xa5',
'HangulSyllableYag': '\xec\x95\xbd',
'HangulSyllableYad': '\xec\x96\x83',
'HangulSyllableYae': '\xec\x96\x98',
'HangulSyllableYab': '\xec\x96\x8d',
'HangulSyllableBbyaj': '\xeb\xba\xae',
'CjkUnifiedIdeograph82fb': '\xe8\x8b\xbb',
'CjkUnifiedIdeograph82fc': '\xe8\x8b\xbc',
'HangulSyllableYal': '\xec\x96\x84',
'CjkUnifiedIdeograph82fa': '\xe8\x8b\xba',
'CjkUnifiedIdeograph82ff': '\xe8\x8b\xbf',
'IdeographicHalfFillSpace': '\xe3\x80\xbf',
'CjkUnifiedIdeograph82fd': '\xe8\x8b\xbd',
'Therefore': '\xe2\x88\xb4',
'HangulSyllableRwaelh': '\xeb\xa2\xa3',
'GreekSmallLetterEpsilonWithTonos': '\xce\xad',
'CjkUnifiedIdeograph601b': '\xe6\x80\x9b',
'UnicodeCharacter0xdb0a': '\xed\xac\x8a',
'UnicodeCharacter0xe0f7': '\xee\x83\xb7',
'CjkUnifiedIdeograph79a9': '\xe7\xa6\xa9',
'CjkUnifiedIdeograph79a8': '\xe7\xa6\xa8',
'UnicodeCharacter0xe0f3': '\xee\x83\xb3',
'BengaliLetterDha': '\xe0\xa6\xa7',
'UnicodeCharacter0xe0f2': '\xee\x83\xb2',
'HangulSyllableJwigs': '\xec\xa5\x93',
'HangulSyllableBbyunj': '\xec\x81\x81',
'HangulSyllableBbyunh': '\xec\x81\x82',
'CurvedStemParagraphSignOrnament': '\xe2\x9d\xa1',
'CjkUnifiedIdeograph79a4': '\xe7\xa6\xa4',
'CjkUnifiedIdeograph39bd': '\xe3\xa6\xbd',
'CjkUnifiedIdeograph39be': '\xe3\xa6\xbe',
'CjkUnifiedIdeograph39bf': '\xe3\xa6\xbf',
'HangulSyllableBbyung': '\xec\x81\x91',
'CjkUnifiedIdeograph39ba': '\xe3\xa6\xba',
'CjkUnifiedIdeograph39bb': '\xe3\xa6\xbb',
'CjkUnifiedIdeograph39bc': '\xe3\xa6\xbc',
'HangulSyllableSswaels': '\xec\x90\x9c',
'HangulSyllableSswaelp': '\xec\x90\x9e',
'HangulSyllableAnj': '\xec\x95\x89',
'HangulSyllableSelg': '\xec\x85\x81',
'HangulSyllableSswaelt': '\xec\x90\x9d',
'HangulSyllableCyak': '\xec\xb1\xb8',
'CjkUnifiedIdeograph7061': '\xe7\x81\xa1',
'UnicodeCharacter0xe0f8': '\xee\x83\xb8',
'HangulSyllableSswaelb': '\xec\x90\x9b',
'HangulSyllableSswaelg': '\xec\x90\x99',
'KangxiRadicalFight': '\xe2\xbe\xbe',
'HangulSyllableRwaelg': '\xeb\xa2\x9d',
'CopticSmallLetterOldCopticShei': '\xe2\xb3\x85',
'HangulSyllableSswaelm': '\xec\x90\x9a',
'UnicodeCharacter0xd7e1': '\xed\x9f\xa1',
'KangxiRadicalAgain': '\xe2\xbc\x9c',
'CjkUnifiedIdeograph39b8': '\xe3\xa6\xb8',
'CjkUnifiedIdeograph39b9': '\xe3\xa6\xb9',
'CjkUnifiedIdeograph39b4': '\xe3\xa6\xb4',
'CjkUnifiedIdeograph39b5': '\xe3\xa6\xb5',
'CjkUnifiedIdeograph39b6': '\xe3\xa6\xb6',
'CjkUnifiedIdeograph39b7': '\xe3\xa6\xb7',
'CjkUnifiedIdeograph39b0': '\xe3\xa6\xb0',
'CjkUnifiedIdeograph39b1': '\xe3\xa6\xb1',
'CjkUnifiedIdeograph39b2': '\xe3\xa6\xb2',
'CjkUnifiedIdeograph39b3': '\xe3\xa6\xb3',
'UnicodeCharacter0xe9ff': '\xee\xa7\xbf',
'CjkUnifiedIdeograph7142': '\xe7\x85\x82',
'UnicodeCharacter0xf520': '\xef\x94\xa0',
'YiSyllableSsyx': '\xea\x8d\x81',
'YiSyllableGgie': '\xea\x88\x98',
'CjkUnifiedIdeograph9599': '\xe9\x96\x99',
'LatinSmallLetterUWithHornAndAcute': '\xe1\xbb\xa9',
'YiSyllableSsyp': '\xea\x8d\x83',
'CjkUnifiedIdeograph9598': '\xe9\x96\x98',
'LatinCapitalLetterMWithHook': '\xe2\xb1\xae',
'CjkUnifiedIdeograph899d': '\xe8\xa6\x9d',
'CjkUnifiedIdeograph899e': '\xe8\xa6\x9e',
'CjkUnifiedIdeograph899f': '\xe8\xa6\x9f',
'UnicodeCharacter0xf676': '\xef\x99\xb6',
'UnicodeCharacter0xf671': '\xef\x99\xb1',
'CjkUnifiedIdeograph899a': '\xe8\xa6\x9a',
'CjkUnifiedIdeograph899b': '\xe8\xa6\x9b',
'CjkUnifiedIdeograph899c': '\xe8\xa6\x9c',
'VerticalLineWithMiddleDot': '\xe2\x8d\xbf',
'UnicodeCharacter0xf272': '\xef\x89\xb2',
'UnicodeCharacter0xf523': '\xef\x94\xa3',
'LatinLetterSmallCapitalEzh': '\xe1\xb4\xa3',
'CjkUnifiedIdeograph36ca': '\xe3\x9b\x8a',
'CjkUnifiedIdeograph9b9a': '\xe9\xae\x9a',
'CjkUnifiedIdeograph9b9b': '\xe9\xae\x9b',
'CjkUnifiedIdeograph9594': '\xe9\x96\x94',
'CjkUnifiedIdeograph9b9d': '\xe9\xae\x9d',
'CjkUnifiedIdeograph9b9e': '\xe9\xae\x9e',
'TamilLetterU': '\xe0\xae\x89',
'HiraganaLetterGi': '\xe3\x81\x8e',
'HangulSyllableRwaelt': '\xeb\xa2\xa1',
'UnicodeCharacter0xd9cb': '\xed\xa7\x8b',
'IdeographicTelegraphSymbolForHourTen': '\xe3\x8d\xa2',
'UnicodeCharacter0xd9cd': '\xed\xa7\x8d',
'UnicodeCharacter0xd9ce': '\xed\xa7\x8e',
'UnicodeCharacter0xd9cf': '\xed\xa7\x8f',
'CjkUnifiedIdeograph9a66': '\xe9\xa9\xa6',
'CjkUnifiedIdeograph9a67': '\xe9\xa9\xa7',
'TamilLetterA': '\xe0\xae\x85',
'LeftshadedWhiteRightwardsArrow': '\xe2\x9e\xaa',
'CjkUnifiedIdeograph9a62': '\xe9\xa9\xa2',
'CjkUnifiedIdeograph9a63': '\xe9\xa9\xa3',
'TamilLetterE': '\xe0\xae\x8e',
'CjkUnifiedIdeograph9a61': '\xe9\xa9\xa1',
'HangulSyllableDyaeng': '\xeb\x8d\x8d',
'TamilLetterI': '\xe0\xae\x87',
'UnicodeCharacter0xfd4e': '\xef\xb5\x8e',
'TamilLetterO': '\xe0\xae\x92',
'CjkUnifiedIdeograph9a68': '\xe9\xa9\xa8',
'CjkUnifiedIdeograph9a69': '\xe9\xa9\xa9',
'CjkUnifiedIdeograph8994': '\xe8\xa6\x94',
'CjkUnifiedIdeograph8995': '\xe8\xa6\x95',
'CjkUnifiedIdeograph8996': '\xe8\xa6\x96',
'CjkUnifiedIdeograph8997': '\xe8\xa6\x97',
'CjkUnifiedIdeograph8990': '\xe8\xa6\x90',
'CjkUnifiedIdeograph8991': '\xe8\xa6\x91',
'CjkUnifiedIdeograph8992': '\xe8\xa6\x92',
'CjkUnifiedIdeograph8993': '\xe8\xa6\x93',
'UnicodeCharacter0xf746': '\xef\x9d\x86',
'CjkUnifiedIdeograph8998': '\xe8\xa6\x98',
'CjkUnifiedIdeograph8999': '\xe8\xa6\x99',
'CjkCompatibilityIdeographf9b1': '\xef\xa6\xb1',
'CjkUnifiedIdeograph4316': '\xe4\x8c\x96',
'UnicodeCharacter0xf748': '\xef\x9d\x88',
'HangulSyllableDec': '\xeb\x8e\x87',
'BoxDrawingsLeftHeavyAndRightUpLight': '\xe2\x94\xb5',
'BlackUpperRightTriangle': '\xe2\x97\xa5',
'CjkUnifiedIdeograph9a6d': '\xe9\xa9\xad',
'CjkUnifiedIdeograph9a6e': '\xe9\xa9\xae',
'CjkUnifiedIdeograph9a6b': '\xe9\xa9\xab',
'CjkUnifiedIdeograph9a6c': '\xe9\xa9\xac',
'ArmenianSmallLetterGhad': '\xd5\xb2',
'UnicodeCharacter0xe941': '\xee\xa5\x81',
'CjkUnifiedIdeograph9b90': '\xe9\xae\x90',
'CjkUnifiedIdeograph9b91': '\xe9\xae\x91',
'CjkUnifiedIdeograph9b92': '\xe9\xae\x92',
'CjkUnifiedIdeograph9b93': '\xe9\xae\x93',
'CjkUnifiedIdeograph9b94': '\xe9\xae\x94',
'CjkUnifiedIdeograph9b95': '\xe9\xae\x95',
'CjkUnifiedIdeograph9b96': '\xe9\xae\x96',
'CjkUnifiedIdeograph9b97': '\xe9\xae\x97',
'CjkUnifiedIdeograph9b98': '\xe9\xae\x98',
'CjkUnifiedIdeograph9b99': '\xe9\xae\x99',
'UnicodeCharacter0xd9c2': '\xed\xa7\x82',
'UnicodeCharacter0xd9c3': '\xed\xa7\x83',
'UnicodeCharacter0xd9c4': '\xed\xa7\x84',
'UnicodeCharacter0xd9c5': '\xed\xa7\x85',
'UnicodeCharacter0xd9c6': '\xed\xa7\x86',
'UnicodeCharacter0xd9c7': '\xed\xa7\x87',
'UnicodeCharacter0xdcba': '\xed\xb2\xba',
'ReversedRotatedFloralHeartBullet': '\xe2\x98\x99',
'ArabicLigatureDadWithRehFinalForm': '\xef\xb4\xac',
'LatinCapitalLetterEgyptologicalAin': '\xea\x9c\xa4',
'VaiDigitFive': '\xea\x98\xa5',
'LatinCapitalLetterVisigothicZ': '\xea\x9d\xa2',
'CjkUnifiedIdeograph3ac6': '\xe3\xab\x86',
'CyrillicSmallLetterGheWithStrokeAndHook': '\xd3\xbb',
'UnicodeCharacter0xf275': '\xef\x89\xb5',
'HangulSyllableCyac': '\xec\xb1\xb7',
'KatakanaLetterSmallYa': '\xe3\x83\xa3',
'KatakanaLetterSmallYo': '\xe3\x83\xa7',
'KatakanaLetterSmallYu': '\xe3\x83\xa5',
'UnicodeCharacter0xd974': '\xed\xa5\xb4',
'PhagspaSingleHeadMark': '\xea\xa1\xb4',
'ThaiDigitSeven': '\xe0\xb9\x97',
'CjkUnifiedIdeograph952d': '\xe9\x94\xad',
'UnicodeCharacter0xed72': '\xee\xb5\xb2',
'UnicodeCharacter0xd976': '\xed\xa5\xb6',
'HangulSyllableRoegg': '\xeb\xa2\xb2',
'HangulSyllableBbweonh': '\xeb\xbf\xae',
'CanadianSyllabicsCarrierTse': '\xe1\x99\x9d',
'HangulSyllableBbweonj': '\xeb\xbf\xad',
'CanadianSyllabicsCarrierTsa': '\xe1\x99\xa0',
'CanadianSyllabicsCarrierTso': '\xe1\x99\x9c',
'AplFunctionalSymbolEpsilonUnderbar': '\xe2\x8d\xb7',
'CyrillicSmallLetterKomiDje': '\xd4\x83',
'CanadianSyllabicsCarrierTsi': '\xe1\x99\x9f',
'CjkRadicalHorn': '\xe2\xbb\x87',
'MongolianLetterManchuFa': '\xe1\xa1\xb6',
'UnicodeCharacter0xd972': '\xed\xa5\xb2',
'HangulSyllableYess': '\xec\x98\x9c',
'HangulSyllableGgwigs': '\xeb\x80\x8f',
'HangulSyllableDyeogg': '\xeb\x8e\x8e',
'UnicodeCharacter0x1a5c': '\xe1\xa9\x9c',
'CircledIdeographMetal': '\xe3\x8a\x8e',
'CjkUnifiedIdeograph5dab': '\xe5\xb6\xab',
'CjkUnifiedIdeograph5dac': '\xe5\xb6\xac',
'Libra': '\xe2\x99\x8e',
'UnicodeCharacter0xf402': '\xef\x90\x82',
'UnicodeCharacter0x1a5f': '\xe1\xa9\x9f',
'GeorgianSmallLetterAn': '\xe2\xb4\x80',
'CjkUnifiedIdeograph444b': '\xe4\x91\x8b',
'CjkUnifiedIdeograph444c': '\xe4\x91\x8c',
'CjkUnifiedIdeograph9111': '\xe9\x84\x91',
'CjkUnifiedIdeograph444a': '\xe4\x91\x8a',
'CjkUnifiedIdeograph444f': '\xe4\x91\x8f',
'CjkUnifiedIdeograph9116': '\xe9\x84\x96',
'CjkUnifiedIdeograph444d': '\xe4\x91\x8d',
'CjkUnifiedIdeograph444e': '\xe4\x91\x8e',
'CjkUnifiedIdeograph4190': '\xe4\x86\x90',
'CjkUnifiedIdeograph4191': '\xe4\x86\x91',
'CjkUnifiedIdeograph4192': '\xe4\x86\x92',
'CjkUnifiedIdeograph4193': '\xe4\x86\x93',
'CjkUnifiedIdeograph4194': '\xe4\x86\x94',
'CjkUnifiedIdeograph4195': '\xe4\x86\x95',
'GreekCapitalLetterOmegaWithPsili': '\xe1\xbd\xa8',
'CjkUnifiedIdeograph4197': '\xe4\x86\x97',
'UnicodeCharacter0xe59e': '\xee\x96\x9e',
'UnicodeCharacter0xe59d': '\xee\x96\x9d',
'UnicodeCharacter0xe59f': '\xee\x96\x9f',
'UnicodeCharacter0xe59a': '\xee\x96\x9a',
'UnicodeCharacter0xed71': '\xee\xb5\xb1',
'UnicodeCharacter0xe59c': '\xee\x96\x9c',
'UnicodeCharacter0xdc71': '\xed\xb1\xb1',
'LatinSmallCapitalLetterIWithStroke': '\xe1\xb5\xbb',
'CyrillicSmallLetterMonographUk': '\xea\x99\x8b',
'CjkUnifiedIdeograph650b': '\xe6\x94\x8b',
'UnicodeCharacter0xf6ad': '\xef\x9a\xad',
'IdeographicTelegraphSymbolForHourTwentyone': '\xe3\x8d\xad',
'HangulSyllableCeuk': '\xec\xb8\xb8',
'CjkUnifiedIdeograph4be0': '\xe4\xaf\xa0',
'HangulSyllableCeuh': '\xec\xb8\xbb',
'CjkUnifiedIdeograph849b': '\xe8\x92\x9b',
'HangulSyllableCyad': '\xec\xb1\xa7',
'P': '\xce\xa0',
'CyrillicCapitalLetterSoftDe': '\xea\x99\xa2',
'UnicodeCharacter0xa6ff': '\xea\x9b\xbf',
'CjkUnifiedIdeograph8864': '\xe8\xa1\xa4',
'CjkUnifiedIdeograph8865': '\xe8\xa1\xa5',
'UnicodeCharacter0xe595': '\xee\x96\x95',
'UnicodeCharacter0xe594': '\xee\x96\x94',
'CircledNumberTwentySix': '\xe3\x89\x96',
'ThaiCharacterFoFan': '\xe0\xb8\x9f',
'RejangLetterRa': '\xea\xa4\xbd',
'HangulSyllableCeul': '\xec\xb8\xa8',
'TeluguLetterSa': '\xe0\xb0\xb8',
'UnicodeCharacter0xe592': '\xee\x96\x92',
'UnicodeCharacter0xa7d3': '\xea\x9f\x93',
'CjkUnifiedIdeograph8867': '\xe8\xa1\xa7',
'CyrillicCapitalLetterKaWithStroke': '\xd2\x9e',
'HangulSyllableCyae': '\xec\xb1\xbc',
'VaiSymbolNii': '\xea\x98\x96',
'CjkUnifiedIdeograph4442': '\xe4\x91\x82',
'CjkUnifiedIdeograph4443': '\xe4\x91\x83',
'CjkUnifiedIdeograph4440': '\xe4\x91\x80',
'CjkUnifiedIdeograph4441': '\xe4\x91\x81',
'CjkUnifiedIdeograph4446': '\xe4\x91\x86',
'CjkUnifiedIdeograph4447': '\xe4\x91\x87',
'CjkUnifiedIdeograph4444': '\xe4\x91\x84',
'CjkUnifiedIdeograph4445': '\xe4\x91\x85',
'CjkUnifiedIdeograph419a': '\xe4\x86\x9a',
'CjkUnifiedIdeograph4448': '\xe4\x91\x88',
'CanadianSyllabicsWestcreeTwa': '\xe1\x91\xa2',
'CjkUnifiedIdeograph419d': '\xe4\x86\x9d',
'CjkUnifiedIdeograph419e': '\xe4\x86\x9e',
'CjkUnifiedIdeograph419f': '\xe4\x86\x9f',
'UnicodeCharacter0xd80d': '\xed\xa0\x8d',
'CjkUnifiedIdeograph3878': '\xe3\xa1\xb8',
'CjkUnifiedIdeograph3879': '\xe3\xa1\xb9',
'UnicodeCharacter0xec48': '\xee\xb1\x88',
'UnicodeCharacter0xec49': '\xee\xb1\x89',
'CjkUnifiedIdeograph3872': '\xe3\xa1\xb2',
'CjkUnifiedIdeograph3873': '\xe3\xa1\xb3',
'CanadianSyllabicsWestcreeLwo': '\xe1\x93\xa3',
'CjkUnifiedIdeograph3871': '\xe3\xa1\xb1',
'CjkUnifiedIdeograph3876': '\xe3\xa1\xb6',
'CjkUnifiedIdeograph3877': '\xe3\xa1\xb7',
'CjkUnifiedIdeograph3874': '\xe3\xa1\xb4',
'CjkUnifiedIdeograph3875': '\xe3\xa1\xb5',
'BlackShogiPiece': '\xe2\x98\x97',
'LowerRightCornerWithDot': '\xe2\x9f\x93',
'CjkUnifiedIdeograph7189': '\xe7\x86\x89',
'CjkUnifiedIdeograph7182': '\xe7\x86\x82',
'CjkUnifiedIdeograph7183': '\xe7\x86\x83',
'CjkUnifiedIdeograph7180': '\xe7\x86\x80',
'CjkUnifiedIdeograph7181': '\xe7\x86\x81',
'CjkUnifiedIdeograph7186': '\xe7\x86\x86',
'CjkUnifiedIdeograph7187': '\xe7\x86\x87',
'CjkUnifiedIdeograph7184': '\xe7\x86\x84',
'CjkUnifiedIdeograph7185': '\xe7\x86\x85',
'LatinCapitalLetterSWithCommaBelow': '\xc8\x98',
'UnicodeCharacter0xf76a': '\xef\x9d\xaa',
'CjkUnifiedIdeograph980c': '\xe9\xa0\x8c',
'CjkUnifiedIdeograph980b': '\xe9\xa0\x8b',
'CjkUnifiedIdeograph980e': '\xe9\xa0\x8e',
'CjkUnifiedIdeograph980d': '\xe9\xa0\x8d',
'CjkUnifiedIdeograph980f': '\xe9\xa0\x8f',
'UnicodeCharacter0xed75': '\xee\xb5\xb5',
'CjkUnifiedIdeograph531a': '\xe5\x8c\x9a',
'CjkUnifiedIdeograph531c': '\xe5\x8c\x9c',
'CjkUnifiedIdeograph531b': '\xe5\x8c\x9b',
'CjkUnifiedIdeograph531e': '\xe5\x8c\x9e',
'CjkUnifiedIdeograph531d': '\xe5\x8c\x9d',
'CjkUnifiedIdeograph531f': '\xe5\x8c\x9f',
'UnicodeCharacter0xd80f': '\xed\xa0\x8f',
'CircledIdeographNight': '\xe3\x8a\xb0',
'degree': '\xc2\xb0',
'CjkUnifiedIdeograph718b': '\xe7\x86\x8b',
'CjkUnifiedIdeograph718c': '\xe7\x86\x8c',
'CjkUnifiedIdeograph718a': '\xe7\x86\x8a',
'CjkUnifiedIdeograph718f': '\xe7\x86\x8f',
'CjkUnifiedIdeograph718d': '\xe7\x86\x8d',
'CjkUnifiedIdeograph718e': '\xe7\x86\x8e',
'LogicalOrWithDotAbove': '\xe2\xa9\x92',
'CjkUnifiedIdeograph7703': '\xe7\x9c\x83',
'HangulSyllableMyud': '\xeb\xae\xab',
'CjkUnifiedIdeograph387b': '\xe3\xa1\xbb',
'SundaneseLetterLa': '\xe1\xae\x9c',
'UnicodeCharacter0xec4f': '\xee\xb1\x8f',
'CjkUnifiedIdeograph387a': '\xe3\xa1\xba',
'CjkUnifiedIdeograph387f': '\xe3\xa1\xbf',
'UnicodeCharacter0xec4a': '\xee\xb1\x8a',
'CjkUnifiedIdeograph387d': '\xe3\xa1\xbd',
'CjkUnifiedIdeograph387e': '\xe3\xa1\xbe',
'CjkUnifiedIdeograph5311': '\xe5\x8c\x91',
'CjkUnifiedIdeograph5310': '\xe5\x8c\x90',
'CjkUnifiedIdeograph5313': '\xe5\x8c\x93',
'CjkUnifiedIdeograph5312': '\xe5\x8c\x92',
'CjkUnifiedIdeograph5315': '\xe5\x8c\x95',
'CjkUnifiedIdeograph5314': '\xe5\x8c\x94',
'CjkUnifiedIdeograph5317': '\xe5\x8c\x97',
'CjkUnifiedIdeograph5316': '\xe5\x8c\x96',
'CjkUnifiedIdeograph5319': '\xe5\x8c\x99',
'CjkUnifiedIdeograph5318': '\xe5\x8c\x98',
'UnicodeCharacter0xa6fa': '\xea\x9b\xba',
'HangulSyllableGeuss': '\xea\xb8\x8c',
'CjkUnifiedIdeograph9801': '\xe9\xa0\x81',
'CjkUnifiedIdeograph9800': '\xe9\xa0\x80',
'CjkUnifiedIdeograph9803': '\xe9\xa0\x83',
'CjkUnifiedIdeograph9802': '\xe9\xa0\x82',
'CjkUnifiedIdeograph9805': '\xe9\xa0\x85',
'CjkUnifiedIdeograph9804': '\xe9\xa0\x84',
'CjkUnifiedIdeograph9807': '\xe9\xa0\x87',
'CjkUnifiedIdeograph9806': '\xe9\xa0\x86',
'CjkUnifiedIdeograph9809': '\xe9\xa0\x89',
'CjkUnifiedIdeograph9808': '\xe9\xa0\x88',
'CjkUnifiedIdeograph953d': '\xe9\x94\xbd',
'UnicodeCharacter0xa7d8': '\xea\x9f\x98',
'UnicodeCharacter0xdfa': '\xe0\xb7\xba',
'CopticOldNubianVerseDivider': '\xe2\xb3\xbc',
'UnicodeCharacter0xdfc': '\xe0\xb7\xbc',
'UnicodeCharacter0xdfb': '\xe0\xb7\xbb',
'UnicodeCharacter0xdfe': '\xe0\xb7\xbe',
'UnicodeCharacter0xdfd': '\xe0\xb7\xbd',
'UnicodeCharacter0xdff': '\xe0\xb7\xbf',
'CjkUnifiedIdeograph4518': '\xe4\x94\x98',
'UnicodeCharacter0xeeef': '\xee\xbb\xaf',
'CjkUnifiedIdeograph4519': '\xe4\x94\x99',
'CjkUnifiedIdeograph675a': '\xe6\x9d\x9a',
'CjkUnifiedIdeograph675b': '\xe6\x9d\x9b',
'CjkUnifiedIdeograph675c': '\xe6\x9d\x9c',
'CjkUnifiedIdeograph675d': '\xe6\x9d\x9d',
'CjkUnifiedIdeograph675e': '\xe6\x9d\x9e',
'CjkUnifiedIdeograph675f': '\xe6\x9d\x9f',
'UnicodeCharacter0xd80a': '\xed\xa0\x8a',
'UnicodeCharacter0xa7d9': '\xea\x9f\x99',
'CanadianSyllabicsYcreeLoo': '\xe1\x93\x99',
'HangulSyllableJwek': '\xec\xa5\x8c',
'HangulSyllableJwej': '\xec\xa5\x8a',
'HangulSyllableJweh': '\xec\xa5\x8f',
'HangulSyllableJweo': '\xec\xa4\x98',
'SkullAndCrossbones': '\xe2\x98\xa0',
'HangulSyllableJwem': '\xec\xa5\x84',
'HangulSyllableBbyigs': '\xec\x81\xb7',
'HangulSyllableMyup': '\xeb\xae\xbe',
'FirstQuarterMoon': '\xe2\x98\xbd',
'HangulSyllableMyus': '\xeb\xae\xb7',
'HangulSyllableMyut': '\xeb\xae\xbd',
'CombiningFermata': '\xcd\x92',
'HangulSyllableJwed': '\xec\xa4\xbb',
'HangulSyllableMyuh': '\xeb\xae\xbf',
'CjkUnifiedIdeograph8014': '\xe8\x80\x94',
'HangulSyllableMyuj': '\xeb\xae\xba',
'HangulSyllableMyuk': '\xeb\xae\xbc',
'HangulSyllableMyul': '\xeb\xae\xac',
'HangulSyllableMyum': '\xeb\xae\xb4',
'HangulSyllableMyun': '\xeb\xae\xa8',
'HangulSyllableJwes': '\xec\xa5\x87',
'HangulSyllableMyub': '\xeb\xae\xb5',
'HangulSyllableMyuc': '\xeb\xae\xbb',
'MalayalamLetterSa': '\xe0\xb4\xb8',
'UnicodeCharacter0xe65b': '\xee\x99\x9b',
'HangulSyllableMyug': '\xeb\xae\xa5',
'CjkUnifiedIdeograph6750': '\xe6\x9d\x90',
'CjkUnifiedIdeograph6751': '\xe6\x9d\x91',
'CjkUnifiedIdeograph6752': '\xe6\x9d\x92',
'CjkUnifiedIdeograph6753': '\xe6\x9d\x93',
'CjkUnifiedIdeograph6754': '\xe6\x9d\x94',
'CjkUnifiedIdeograph6755': '\xe6\x9d\x95',
'CjkUnifiedIdeograph6756': '\xe6\x9d\x96',
'CjkUnifiedIdeograph6757': '\xe6\x9d\x97',
'CjkUnifiedIdeograph6758': '\xe6\x9d\x98',
'GeorgianSmallLetterSan': '\xe2\xb4\x91',
'UnicodeCharacter0xf7ab': '\xef\x9e\xab',
'UnicodeCharacter0xdf1': '\xe0\xb7\xb1',
'UnicodeCharacter0xdf0': '\xe0\xb7\xb0',
'UnicodeCharacter0xdf5': '\xe0\xb7\xb5',
'UnicodeCharacter0xdf7': '\xe0\xb7\xb7',
'UnicodeCharacter0xdf6': '\xe0\xb7\xb6',
'UnicodeCharacter0xdf9': '\xe0\xb7\xb9',
'UnicodeCharacter0xdf8': '\xe0\xb7\xb8',
'UnicodeCharacter0xe65c': '\xee\x99\x9c',
'HangulSyllableCoenh': '\xec\xb5\xa2',
'UnicodeCharacter0xee34': '\xee\xb8\xb4',
'LatinSmallLetterAWithRightHalfRing': '\xe1\xba\x9a',
'UnicodeCharacter0xec3a': '\xee\xb0\xba',
'LimbuSignKemphreng': '\xe1\xa4\xba',
'HangulJongseongSiosrieul': '\xe1\x87\xa9',
'HangulSyllableGgum': '\xea\xbf\x88',
'CjkUnifiedIdeograph8019': '\xe8\x80\x99',
'HangulSyllableRwaen': '\xeb\xa2\x98',
'UnicodeCharacter0xf61b': '\xef\x98\x9b',
'CjkUnifiedIdeograph3d69': '\xe3\xb5\xa9',
'CjkUnifiedIdeograph3d68': '\xe3\xb5\xa8',
'CjkUnifiedIdeograph99df': '\xe9\xa7\x9f',
'CjkUnifiedIdeograph99da': '\xe9\xa7\x9a',
'CjkUnifiedIdeograph99db': '\xe9\xa7\x9b',
'CjkUnifiedIdeograph99dc': '\xe9\xa7\x9c',
'CjkUnifiedIdeograph3d61': '\xe3\xb5\xa1',
'CjkUnifiedIdeograph3d60': '\xe3\xb5\xa0',
'CjkUnifiedIdeograph3d63': '\xe3\xb5\xa3',
'CjkUnifiedIdeograph3d62': '\xe3\xb5\xa2',
'CjkUnifiedIdeograph3d65': '\xe3\xb5\xa5',
'CjkUnifiedIdeograph3d64': '\xe3\xb5\xa4',
'CjkUnifiedIdeograph3d67': '\xe3\xb5\xa7',
'CjkUnifiedIdeograph3d66': '\xe3\xb5\xa6',
'CjkUnifiedIdeograph623b': '\xe6\x88\xbb',
'HangulSyllableGoec': '\xea\xb5\x8b',
'HangulSyllableGoed': '\xea\xb4\xbb',
'HangulSyllableGoeg': '\xea\xb4\xb5',
'WhiteSquareContainingBlackSmallSquare': '\xe2\x96\xa3',
'CjkUnifiedIdeograph740f': '\xe7\x90\x8f',
'CjkUnifiedIdeograph740e': '\xe7\x90\x8e',
'CjkUnifiedIdeograph740d': '\xe7\x90\x8d',
'CjkUnifiedIdeograph740c': '\xe7\x90\x8c',
'CjkUnifiedIdeograph740b': '\xe7\x90\x8b',
'CjkUnifiedIdeograph740a': '\xe7\x90\x8a',
'HangulSyllableGoep': '\xea\xb5\x8e',
'HangulSyllableGoes': '\xea\xb5\x87',
'HangulSyllableGoet': '\xea\xb5\x8d',
'UnicodeCharacter0xe65f': '\xee\x99\x9f',
'CircledNumberFortyNine': '\xe3\x8a\xbe',
'CjkUnifiedIdeograph72de': '\xe7\x8b\x9e',
'CjkUnifiedIdeograph72dd': '\xe7\x8b\x9d',
'CjkUnifiedIdeograph72df': '\xe7\x8b\x9f',
'CjkUnifiedIdeograph72da': '\xe7\x8b\x9a',
'CjkUnifiedIdeograph72dc': '\xe7\x8b\x9c',
'CjkUnifiedIdeograph72db': '\xe7\x8b\x9b',
'CjkUnifiedIdeograph99d4': '\xe9\xa7\x94',
'CjkUnifiedIdeograph99d5': '\xe9\xa7\x95',
'CjkUnifiedIdeograph99d6': '\xe9\xa7\x96',
'CjkUnifiedIdeograph99d7': '\xe9\xa7\x97',
'CjkUnifiedIdeograph99d0': '\xe9\xa7\x90',
'CjkUnifiedIdeograph99d1': '\xe9\xa7\x91',
'CjkUnifiedIdeograph99d2': '\xe9\xa7\x92',
'CjkUnifiedIdeograph99d3': '\xe9\xa7\x93',
'CjkUnifiedIdeograph3d6a': '\xe3\xb5\xaa',
'CjkUnifiedIdeograph3d6c': '\xe3\xb5\xac',
'CjkUnifiedIdeograph3d6b': '\xe3\xb5\xab',
'CjkUnifiedIdeograph3d6e': '\xe3\xb5\xae',
'CjkUnifiedIdeograph3d6d': '\xe3\xb5\xad',
'HangulSyllableSswen': '\xec\x92\xa0',
'CjkUnifiedIdeograph3d6f': '\xe3\xb5\xaf',
'HangulSyllableSswep': '\xec\x92\xb6',
'CjkUnifiedIdeograph5da6': '\xe5\xb6\xa6',
'HangulSyllableSswes': '\xec\x92\xaf',
'HexagramForInnocence': '\xe4\xb7\x98',
'CjkUnifiedIdeograph5da7': '\xe5\xb6\xa7',
'CyrillicCapitalLetterSemisoftSign': '\xd2\x8c',
'ArabicFullStop': '\xdb\x94',
'SmallRomanNumeralFifty': '\xe2\x85\xbc',
'CjkUnifiedIdeograph9d53': '\xe9\xb5\x93',
'BopomofoLetterGn': '\xe3\x84\xac',
'CjkUnifiedIdeograph5da5': '\xe5\xb6\xa5',
'CjkUnifiedIdeograph72d9': '\xe7\x8b\x99',
'CjkUnifiedIdeograph72d8': '\xe7\x8b\x98',
'LeftOneEighthBlock': '\xe2\x96\x8f',
'CjkUnifiedIdeograph72d5': '\xe7\x8b\x95',
'CjkUnifiedIdeograph5da2': '\xe5\xb6\xa2',
'CjkUnifiedIdeograph72d7': '\xe7\x8b\x97',
'CjkUnifiedIdeograph72d6': '\xe7\x8b\x96',
'CjkUnifiedIdeograph72d1': '\xe7\x8b\x91',
'CjkUnifiedIdeograph72d0': '\xe7\x8b\x90',
'CjkUnifiedIdeograph72d3': '\xe7\x8b\x93',
'CjkUnifiedIdeograph5da3': '\xe5\xb6\xa3',
'UnicodeCharacter0xe65d': '\xee\x99\x9d',
'CjkUnifiedIdeograph5da0': '\xe5\xb6\xa0',
'SphericalAngleOpeningUp': '\xe2\xa6\xa1',
'CjkUnifiedIdeograph7408': '\xe7\x90\x88',
'CjkUnifiedIdeograph7407': '\xe7\x90\x87',
'CjkUnifiedIdeograph7406': '\xe7\x90\x86',
'CjkUnifiedIdeograph7405': '\xe7\x90\x85',
'BopomofoLetterGu': '\xe3\x86\xa3',
'CjkUnifiedIdeograph7403': '\xe7\x90\x83',
'CjkUnifiedIdeograph7402': '\xe7\x90\x82',
'CjkUnifiedIdeograph7401': '\xe7\x90\x81',
'CjkUnifiedIdeograph7400': '\xe7\x90\x80',
'GeorgianLetterZen': '\xe1\x83\x96',
'ArabicLigatureSheenWithHahMedialForm': '\xef\xb4\xb8',
'UnicodeCharacter0xe65e': '\xee\x99\x9e',
'HangulSyllableDilb': '\xeb\x94\x9f',
'CanadianSyllabicsPii': '\xe1\x90\xb2',
'HangulSyllableKip': '\xed\x82\xbe',
'HangulSyllableDdyelb': '\xeb\x97\xbf',
'HangulSyllableKit': '\xed\x82\xbd',
'HangulSyllableJjoeng': '\xec\xac\xa5',
'HangulSyllableDdyelg': '\xeb\x97\xbd',
'CjkUnifiedIdeograph7f1b': '\xe7\xbc\x9b',
'CjkUnifiedIdeograph7f1c': '\xe7\xbc\x9c',
'UnicodeCharacter0xe7aa': '\xee\x9e\xaa',
'CjkUnifiedIdeograph7f1a': '\xe7\xbc\x9a',
'CjkUnifiedIdeograph7f1f': '\xe7\xbc\x9f',
'CjkUnifiedIdeograph8117': '\xe8\x84\x97',
'CjkUnifiedIdeograph7f1d': '\xe7\xbc\x9d',
'CjkUnifiedIdeograph7f1e': '\xe7\xbc\x9e',
'CjkUnifiedIdeograph8bed': '\xe8\xaf\xad',
'HangulSyllableDdyels': '\xeb\x98\x80',
'HangulSyllableKic': '\xed\x82\xbb',
'HangulSyllableKid': '\xed\x82\xab',
'HangulSyllableDdyelt': '\xeb\x98\x81',
'HangulSyllableKoegg': '\xec\xbe\xaa',
'HangulSyllableKih': '\xed\x82\xbf',
'CjkUnifiedIdeograph7678': '\xe7\x99\xb8',
'HangulSyllableKij': '\xed\x82\xba',
'HangulSyllableKik': '\xed\x82\xbc',
'HangulSyllableKil': '\xed\x82\xac',
'HangulSyllableKim': '\xed\x82\xb4',
'HangulSyllableKin': '\xed\x82\xa8',
'CjkUnifiedIdeograph7679': '\xe7\x99\xb9',
'GreaterthanButNotEqualTo': '\xe2\x89\xa9',
'SinhalaLetterDantajaLayanna': '\xe0\xb6\xbd',
'UnicodeCharacter0xed52': '\xee\xb5\x92',
'CjkUnifiedIdeograph9b3a': '\xe9\xac\xba',
'BraillePatternDots24678': '\xe2\xa3\xaa',
'HangulSyllableDilt': '\xeb\x94\xa1',
'HangulSyllableJeulp': '\xec\xa6\x96',
'BoxDrawingsDownSingleAndHorizontalDouble': '\xe2\x95\xa4',
'LatinSmallLetterLWithDotBelow': '\xe1\xb8\xb7',
'UnicodeCharacter0xf2f8': '\xef\x8b\xb8',
'HangulSyllableTigg': '\xed\x8b\xb2',
'CompositionSymbol': '\xe2\x8e\x84',
'HangulSyllableDilp': '\xeb\x94\xa2',
'HangulSyllableGwak': '\xea\xb4\x94',
'YiSyllableShyrx': '\xea\x8f\x84',
'CjkUnifiedIdeograph7f18': '\xe7\xbc\x98',
'CjkUnifiedIdeograph7f19': '\xe7\xbc\x99',
'HangulSyllableTyik': '\xed\x8b\xac',
'UnicodeCharacter0xfe6d': '\xef\xb9\xad',
'HangulSyllableKalt': '\xec\xba\x81',
'RightAngleWithArc': '\xe2\x8a\xbe',
'CjkUnifiedIdeograph7f13': '\xe7\xbc\x93',
'CjkUnifiedIdeograph7f10': '\xe7\xbc\x90',
'CjkUnifiedIdeograph7f11': '\xe7\xbc\x91',
'CjkUnifiedIdeograph7f16': '\xe7\xbc\x96',
'CjkUnifiedIdeograph7f17': '\xe7\xbc\x97',
'CjkUnifiedIdeograph7f14': '\xe7\xbc\x94',
'CjkUnifiedIdeograph7f15': '\xe7\xbc\x95',
'UnicodeCharacter0xf2f9': '\xef\x8b\xb9',
'CjkUnifiedIdeograph7bd6': '\xe7\xaf\x96',
'CjkUnifiedIdeograph5f2e': '\xe5\xbc\xae',
'CjkUnifiedIdeograph5f2d': '\xe5\xbc\xad',
'CjkUnifiedIdeograph5f2f': '\xe5\xbc\xaf',
'CjkUnifiedIdeograph5f2a': '\xe5\xbc\xaa',
'CjkUnifiedIdeograph5f2c': '\xe5\xbc\xac',
'CjkUnifiedIdeograph5f2b': '\xe5\xbc\xab',
'CjkUnifiedIdeograph9718': '\xe9\x9c\x98',
'ArabicLetterKhahFinalForm': '\xef\xba\xa6',
'CjkUnifiedIdeograph451d': '\xe4\x94\x9d',
'CjkUnifiedIdeograph451e': '\xe4\x94\x9e',
'LatinSmallLetterQWithDiagonalStroke': '\xea\x9d\x99',
'HalfwidthKatakanaLetterO': '\xef\xbd\xb5',
'HangulSyllableBbyeobs': '\xeb\xbc\x9a',
'CjkUnifiedIdeograph7bd0': '\xe7\xaf\x90',
'ParenthesizedIdeographWood': '\xe3\x88\xad',
'HangulSyllableTeubs': '\xed\x8b\x8a',
'CopticSmallLetterNi': '\xe2\xb2\x9b',
'UnicodeCharacter0xde45': '\xed\xb9\x85',
'CjkUnifiedIdeograph5f29': '\xe5\xbc\xa9',
'SyriacLetterPersianDhalath': '\xdc\xaf',
'HangulSyllableMeobs': '\xeb\xa9\x8a',
'UnicodeCharacter0xde44': '\xed\xb9\x84',
'CjkUnifiedIdeograph5f25': '\xe5\xbc\xa5',
'CjkUnifiedIdeograph5f24': '\xe5\xbc\xa4',
'KangxiRadicalGate': '\xe2\xbe\xa8',
'CjkUnifiedIdeograph5f26': '\xe5\xbc\xa6',
'CjkUnifiedIdeograph5f21': '\xe5\xbc\xa1',
'CjkUnifiedIdeograph5f20': '\xe5\xbc\xa0',
'CjkUnifiedIdeograph5f23': '\xe5\xbc\xa3',
'CjkUnifiedIdeograph5f22': '\xe5\xbc\xa2',
'HangulSyllablePaess': '\xed\x8c\xbc',
'CyrillicCapitalLetterZheWithBreve': '\xd3\x81',
'g': '\xce\xb3',
'UnicodeCharacter0xde41': '\xed\xb9\x81',
'LatinCapitalLetterWWithHook': '\xe2\xb1\xb2',
'TripleVerticalBarBinaryRelation': '\xe2\xab\xb4',
'ModifierLetterSmallGreekPhi': '\xe1\xb5\xa0',
'UnicodeCharacter0xdea5': '\xed\xba\xa5',
'HangulSyllableKalb': '\xec\xb9\xbf',
'ArabicLigatureKafWithMeemIsolatedForm': '\xef\xb0\xbc',
'BlackLargeSquare': '\xe2\xac\x9b',
'SyriacLetterYudh': '\xdc\x9d',
'YiSyllableNro': '\xea\x8e\x99',
'HangulSyllableSwalp': '\xec\x86\xb6',
'HangulSyllableSwals': '\xec\x86\xb4',
'HangulSyllableTyij': '\xed\x8b\xaa',
'HangulSyllableSwalt': '\xec\x86\xb5',
'-/+': '\xe2\x88\x93',
'HangulSyllableSwalh': '\xec\x86\xb7',
'HangulJongseongKiyeok': '\xe1\x86\xa8',
'HangulChoseongPhieuphpieup': '\xe1\x85\x96',
'HangulSyllableSwalm': '\xec\x86\xb2',
'HangulSyllableCegs': '\xec\xb2\xb7',
'UnicodeCharacter0xf2f2': '\xef\x8b\xb2',
'HangulSyllableSwalb': '\xec\x86\xb3',
'UnicodeCharacter0xe5af': '\xee\x96\xaf',
'HangulSyllableSwalg': '\xec\x86\xb1',
'ArabicindicDigitThree': '\xd9\xa3',
'RunicLetterJeranJ': '\xe1\x9b\x83',
'CjkUnifiedIdeograph6d3c': '\xe6\xb4\xbc',
'UnicodeCharacter0xda01': '\xed\xa8\x81',
'UnicodeCharacter0xf2f3': '\xef\x8b\xb3',
'ArmenianSmallLetterYi': '\xd5\xb5',
'HangulSyllableSsyugs': '\xec\x93\x97',
'CyrillicSmallLetterStraightU': '\xd2\xaf',
'HangulSyllableBbweogs': '\xeb\xbf\xab',
'HangulSyllableSsyugg': '\xec\x93\x96',
'CjkUnifiedIdeograph6894': '\xe6\xa2\x94',
'UnicodeCharacter0xe317': '\xee\x8c\x97',
'HangulSyllableJjwaeng': '\xec\xac\x89',
'YiSyllableNjyr': '\xea\x91\x89',
'UnicodeCharacter0xe443': '\xee\x91\x83',
'CjkUnifiedIdeograph3c47': '\xe3\xb1\x87',
'CjkUnifiedIdeograph52a6': '\xe5\x8a\xa6',
'UnicodeCharacter0xf2f1': '\xef\x8b\xb1',
'CjkUnifiedIdeograph767d': '\xe7\x99\xbd',
'UnicodeCharacter0xe314': '\xee\x8c\x94',
'CjkUnifiedIdeograph767e': '\xe7\x99\xbe',
'CjkUnifiedIdeograph52a0': '\xe5\x8a\xa0',
'UnicodeCharacter0xe64b': '\xee\x99\x8b',
'UnicodeCharacter0xeca6': '\xee\xb2\xa6',
'UnicodeCharacter0xdc6f': '\xed\xb1\xaf',
'UnicodeCharacter0xe312': '\xee\x8c\x92',
'HangulSyllableDyilb': '\xeb\x94\x83',
'UnicodeCharacter0xe311': '\xee\x8c\x91',
'HangulSyllableDyilg': '\xeb\x94\x81',
'HangulSyllableDyilh': '\xeb\x94\x87',
'CjkUnifiedIdeograph52a3': '\xe5\x8a\xa3',
'HangulSyllableDyilm': '\xeb\x94\x82',
'CjkUnifiedIdeograph767a': '\xe7\x99\xba',
'HangulSyllableDyilp': '\xeb\x94\x86',
'SquareGinii': '\xe3\x8c\x91',
'CjkUnifiedIdeograph63d4': '\xe6\x8f\x94',
'HangulSyllableDyilt': '\xeb\x94\x85',
'HangulSyllableNeobs': '\xeb\x84\x9a',
'CjkUnifiedIdeograph820d': '\xe8\x88\x8d',
'LepchaLetterBla': '\xe1\xb0\x94',
'CjkUnifiedIdeograph820f': '\xe8\x88\x8f',
'ArabicSmallHighThreeDots': '\xdb\x9b',
'CjkUnifiedIdeograph820a': '\xe8\x88\x8a',
'CjkUnifiedIdeograph820b': '\xe8\x88\x8b',
'CjkUnifiedIdeograph63d2': '\xe6\x8f\x92',
'^8': '\xe2\x81\xb8',
'SetMinus': '\xe2\x88\x96',
'CjkUnifiedIdeograph9e7a': '\xe9\xb9\xba',
'^=': '\xe2\x81\xbc',
'UnicodeCharacter0xf1c5': '\xef\x87\x85',
'^0': '\xe2\x81\xb0',
'CjkUnifiedIdeograph52a8': '\xe5\x8a\xa8',
'^2': '\xc2\xb2',
'^3': '\xc2\xb3',
'^4': '\xe2\x81\xb4',
'CjkUnifiedIdeograph9d5d': '\xe9\xb5\x9d',
'^6': '\xe2\x81\xb6',
'CjkUnifiedIdeograph52a9': '\xe5\x8a\xa9',
'^(': '\xe2\x81\xbd',
'^)': '\xe2\x81\xbe',
'GreekCapitalLetterOmegaWithPsiliAndPerispomeni': '\xe1\xbd\xae',
'CjkUnifiedIdeograph9d5e': '\xe9\xb5\x9e',
'^-': '\xe2\x81\xbb',
'CjkUnifiedIdeograph9d5b': '\xe9\xb5\x9b',
'YiSyllableNyux': '\xea\x91\x9a',
'UnicodeCharacter0xe318': '\xee\x8c\x98',
'CjkUnifiedIdeograph9d5c': '\xe9\xb5\x9c',
'HalfwidthHangulLetterEo': '\xef\xbf\x86',
'ArabicLigatureSeenWithMeemWithHahFinalForm': '\xef\xb5\x9f',
'CjkUnifiedIdeograph8208': '\xe8\x88\x88',
'CjkUnifiedIdeograph8209': '\xe8\x88\x89',
'CjkUnifiedIdeograph8204': '\xe8\x88\x84',
'TifinaghLetterTuaregYah': '\xe2\xb5\x82',
'TifinaghLetterTuaregYak': '\xe2\xb4\xbe',
'CjkUnifiedIdeograph8207': '\xe8\x88\x87',
'CjkUnifiedIdeograph8200': '\xe8\x88\x80',
'CjkUnifiedIdeograph8201': '\xe8\x88\x81',
'CjkUnifiedIdeograph8202': '\xe8\x88\x82',
'CjkUnifiedIdeograph8203': '\xe8\x88\x83',
'TifinaghLetterTuaregYaq': '\xe2\xb5\x88',
'HangulSyllableTeol': '\xed\x84\xb8',
'HexagramForTheGentleWind': '\xe4\xb7\xb8',
'RightWigglyFence': '\xe2\xa7\x99',
'CjkUnifiedIdeograph63d8': '\xe6\x8f\x98',
'HalfwidthKatakanaLetterA': '\xef\xbd\xb1',
'GreekSmallLetterEtaWithDasiaAndOxiaAndYpogegrammeni': '\xe1\xbe\x95',
'UnicodeCharacter0xf7d3': '\xef\x9f\x93',
'^i': '\xe2\x81\xb1',
'^n': '\xe2\x81\xbf',
'CjkUnifiedIdeograph41e5': '\xe4\x87\xa5',
'ThreedToplightedRightwardsArrowhead': '\xe2\x9e\xa2',
'HangulSyllableMyibs': '\xeb\xaf\xae',
'UnicodeCharacter0xe461': '\xee\x91\xa1',
'HangulSyllableBeobs': '\xeb\xb2\x96',
'UnicodeCharacter0xe460': '\xee\x91\xa0',
'UnicodeCharacter0xf7d2': '\xef\x9f\x92',
'UnicodeCharacter0xe463': '\xee\x91\xa3',
'UnicodeCharacter0xf847': '\xef\xa1\x87',
'ThaiCharacterSaraIi': '\xe0\xb8\xb5',
'UnicodeCharacter0xe445': '\xee\x91\x85',
'UnicodeCharacter0xe464': '\xee\x91\xa4',
'HangulSyllableMaeb': '\xeb\xa7\xb5',
'UnicodeCharacter0xe467': '\xee\x91\xa7',
'UnicodeCharacter0xe568': '\xee\x95\xa8',
'UnicodeCharacter0xe466': '\xee\x91\xa6',
'HangulSyllableNegg': '\xeb\x84\xa6',
'AplFunctionalSymbolRightwardsVane': '\xe2\x8d\x86',
'UnicodeCharacter0xab0e': '\xea\xac\x8e',
'YiSyllableNyut': '\xea\x91\x99',
'UnicodeCharacter0xfbcf': '\xef\xaf\x8f',
'UnicodeCharacter0xfbca': '\xef\xaf\x8a',
'UnicodeCharacter0xfbcc': '\xef\xaf\x8c',
'UnicodeCharacter0xab0d': '\xea\xac\x8d',
'HangulSyllableNegs': '\xeb\x84\xa7',
'ArabicLigatureTehWithKhahIsolatedForm': '\xef\xb0\x8d',
'UnicodeCharacter0xef5f': '\xee\xbd\x9f',
'HangulJongseongNieun': '\xe1\x86\xab',
'MalayalamNumberTen': '\xe0\xb5\xb0',
'BiohazardSign': '\xe2\x98\xa3',
'KayahLiLetterBa': '\xea\xa4\x99',
'UnicodeCharacter0xdec4': '\xed\xbb\x84',
'CjkUnifiedIdeograph8ae4': '\xe8\xab\xa4',
'UnicodeCharacter0xab0c': '\xea\xac\x8c',
'UnicodeCharacter0xab0b': '\xea\xac\x8b',
'UnicodeCharacter0xfbc5': '\xef\xaf\x85',
'CjkUnifiedIdeograph573c': '\xe5\x9c\xbc',
'UnicodeCharacter0xfbc7': '\xef\xaf\x87',
'UnicodeCharacter0xfbc6': '\xef\xaf\x86',
'UnicodeCharacter0xfbc1': '\xef\xaf\x81',
'ModifierLetterCyrillicEn': '\xe1\xb5\xb8',
'HangulSyllableGgwaeh': '\xea\xbd\xbf',
'CjkUnifiedIdeograph573b': '\xe5\x9c\xbb',
'SquaredFourDotPunctuation': '\xe2\xb8\xac',
'CjkUnifiedIdeograph7a49': '\xe7\xa9\x89',
'UnicodeCharacter0xfbc9': '\xef\xaf\x89',
'CjkUnifiedIdeograph573a': '\xe5\x9c\xba',
'UnicodeCharacter0xdec1': '\xed\xbb\x81',
'CjkUnifiedIdeograph7a46': '\xe7\xa9\x86',
'HangulSyllableGgwigg': '\xeb\x80\x8e',
'RightwardsWaveArrow': '\xe2\x86\x9d',
'CjkUnifiedIdeograph7a47': '\xe7\xa9\x87',
'HangulSyllableDdyeon': '\xeb\x97\x9c',
'UnicodeCharacter0xdec3': '\xed\xbb\x83',
'CjkUnifiedIdeograph7a44': '\xe7\xa9\x84',
'CjkUnifiedIdeograph573f': '\xe5\x9c\xbf',
'BlackMediumLozenge': '\xe2\xac\xa7',
'LatinLetterSmallCapitalJ': '\xe1\xb4\x8a',
'LatinLetterSmallCapitalI': '\xc9\xaa',
'LatinLetterSmallCapitalH': '\xca\x9c',
'RomanNumeralTwo': '\xe2\x85\xa1',
'LatinLetterSmallCapitalN': '\xc9\xb4',
'LatinLetterSmallCapitalM': '\xe1\xb4\x8d',
'LatinLetterSmallCapitalL': '\xca\x9f',
'LatinLetterSmallCapitalC': '\xe1\xb4\x84',
'LatinLetterSmallCapitalB': '\xca\x99',
'LatinLetterSmallCapitalA': '\xe1\xb4\x80',
'CjkUnifiedIdeograph573d': '\xe5\x9c\xbd',
'LatinLetterSmallCapitalG': '\xc9\xa2',
'LatinLetterSmallCapitalF': '\xea\x9c\xb0',
'LatinLetterSmallCapitalE': '\xe1\xb4\x87',
'LatinLetterSmallCapitalD': '\xe1\xb4\x85',
'UnicodeCharacter0xd86e': '\xed\xa1\xae',
'LatinLetterSmallCapitalZ': '\xe1\xb4\xa2',
'LatinLetterSmallCapitalY': '\xca\x8f',
'HangulSyllableCyeonj': '\xec\xb3\x95',
'LatinLetterSmallCapitalS': '\xea\x9c\xb1',
'LatinLetterSmallCapitalR': '\xca\x80',
'LatinLetterSmallCapitalP': '\xe1\xb4\x98',
'LatinLetterSmallCapitalW': '\xe1\xb4\xa1',
'LatinLetterSmallCapitalV': '\xe1\xb4\xa0',
'LatinLetterSmallCapitalU': '\xe1\xb4\x9c',
'LatinLetterSmallCapitalT': '\xe1\xb4\x9b',
'HangulSyllableBim': '\xeb\xb9\x94',
'HangulSyllableMis': '\xeb\xb0\x8b',
'UnicodeCharacter0xe564': '\xee\x95\xa4',
'HangulSyllableBin': '\xeb\xb9\x88',
'HangulSyllableBih': '\xeb\xb9\x9f',
'HangulSyllableBik': '\xeb\xb9\x9c',
'HangulSyllableBij': '\xeb\xb9\x9a',
'HangulSyllableBid': '\xeb\xb9\x8b',
'HangulSyllableBig': '\xeb\xb9\x85',
'HangulSyllableCel': '\xec\xb2\xbc',
'HangulSyllableBic': '\xeb\xb9\x9b',
'CjkUnifiedIdeograph4ef8': '\xe4\xbb\xb8',
'UnicodeCharacter0x1a48': '\xe1\xa9\x88',
'UnicodeCharacter0x1a49': '\xe1\xa9\x89',
'HangulJongseongRieulkapyeounpieup': '\xe1\x87\x95',
'CjkUnifiedIdeograph5840': '\xe5\xa1\x80',
'UnicodeCharacter0x1a42': '\xe1\xa9\x82',
'UnicodeCharacter0x1a43': '\xe1\xa9\x83',
'UnicodeCharacter0x1a40': '\xe1\xa9\x80',
'UnicodeCharacter0x1a41': '\xe1\xa9\x81',
'UnicodeCharacter0x1a46': '\xe1\xa9\x86',
'UnicodeCharacter0x1a47': '\xe1\xa9\x87',
'UnicodeCharacter0x1a44': '\xe1\xa9\x84',
'UnicodeCharacter0x1a45': '\xe1\xa9\x85',
'UnicodeCharacter0xf7d8': '\xef\x9f\x98',
'HangulSyllableNyigg': '\xeb\x8a\xae',
'ShortUpTackAboveShortDownTack': '\xe2\xab\xa9',
'CyrillicSmallLetterIotifiedBigYus': '\xd1\xad',
'GreekSmallLetterAlphaWithPsiliAndYpogegrammeni': '\xe1\xbe\x80',
'HangulSyllableBwags': '\xeb\xb4\x93',
'ArabicLigatureYehWithMeemWithMeemFinalForm': '\xef\xb6\x9c',
'HangulSyllableBwagg': '\xeb\xb4\x92',
'UnicodeCharacter0xe566': '\xee\x95\xa6',
'VulgarFractionOneSixth': '\xe2\x85\x99',
'UnicodeCharacter0x1a4b': '\xe1\xa9\x8b',
'UnicodeCharacter0x1a4c': '\xe1\xa9\x8c',
'UnicodeCharacter0x1a4a': '\xe1\xa9\x8a',
'NorthEastDoubleArrow': '\xe2\x87\x97',
'UnicodeCharacter0x1a4d': '\xe1\xa9\x8d',
'UnicodeCharacter0x1a4e': '\xe1\xa9\x8e',
'ArabicLetterBeeh': '\xd9\xbb',
'HangulSyllableGgwaem': '\xea\xbd\xb4',
'UnicodeCharacter0xf5ea': '\xef\x97\xaa',
'HangulSyllableBbyeoh': '\xeb\xbc\xa3',
'UnicodeCharacter0xeca9': '\xee\xb2\xa9',
'HangulSyllableBbyeoj': '\xeb\xbc\x9e',
'HangulSyllableBbyeok': '\xeb\xbc\xa0',
'HangulSyllableBbyeol': '\xeb\xbc\x90',
'HangulSyllableBbyeom': '\xeb\xbc\x98',
'HangulSyllableBbyeon': '\xeb\xbc\x8c',
'HangulSyllableMyulp': '\xeb\xae\xb2',
'HangulSyllableBbyeob': '\xeb\xbc\x99',
'HangulSyllableBbyeoc': '\xeb\xbc\x9f',
'HangulSyllableBbyeod': '\xeb\xbc\x8f',
'HangulSyllableBbyeog': '\xeb\xbc\x89',
'cTh': '\xcf\xb4',
'HangulSyllableKeonj': '\xec\xbb\xa9',
'CjkUnifiedIdeograph8e3b': '\xe8\xb8\xbb',
'CjkUnifiedIdeograph8e3c': '\xe8\xb8\xbc',
'UnicodeCharacter0xe560': '\xee\x95\xa0',
'CjkUnifiedIdeograph8e3a': '\xe8\xb8\xba',
'CjkUnifiedIdeograph8e3f': '\xe8\xb8\xbf',
'EthiopicSyllableShoa': '\xe2\xb6\x84',
'CjkUnifiedIdeograph8e3e': '\xe8\xb8\xbe',
'CjkUnifiedIdeograph6d04': '\xe6\xb4\x84',
'CjkUnifiedIdeograph6d05': '\xe6\xb4\x85',
'CjkUnifiedIdeograph6d06': '\xe6\xb4\x86',
'CjkUnifiedIdeograph6d07': '\xe6\xb4\x87',
'CjkUnifiedIdeograph6d00': '\xe6\xb4\x80',
'CjkUnifiedIdeograph6d01': '\xe6\xb4\x81',
'CjkUnifiedIdeograph6d02': '\xe6\xb4\x82',
'CjkUnifiedIdeograph6d03': '\xe6\xb4\x83',
'CjkUnifiedIdeograph7a59': '\xe7\xa9\x99',
'CjkUnifiedIdeograph7a58': '\xe7\xa9\x98',
'HangulSyllableNyaeng': '\xeb\x84\x81',
'CjkUnifiedIdeograph6d08': '\xe6\xb4\x88',
'CjkUnifiedIdeograph6d09': '\xe6\xb4\x89',
'UnicodeCharacter0xe561': '\xee\x95\xa1',
'SquareMol': '\xe3\x8f\x96',
'UnicodeCharacter0xf4a5': '\xef\x92\xa5',
'CjkUnifiedIdeograph8e38': '\xe8\xb8\xb8',
'CjkUnifiedIdeograph8e39': '\xe8\xb8\xb9',
'HangulSyllableSeung': '\xec\x8a\xb9',
'HangulSyllableNyul': '\xeb\x89\xbc',
'CjkUnifiedIdeograph8e32': '\xe8\xb8\xb2',
'CjkUnifiedIdeograph8e33': '\xe8\xb8\xb3',
'CjkUnifiedIdeograph8e30': '\xe8\xb8\xb0',
'CjkUnifiedIdeograph8139': '\xe8\x84\xb9',
'CjkUnifiedIdeograph8e36': '\xe8\xb8\xb6',
'CjkUnifiedIdeograph8e37': '\xe8\xb8\xb7',
'CjkUnifiedIdeograph8e34': '\xe8\xb8\xb4',
'CjkUnifiedIdeograph8e35': '\xe8\xb8\xb5',
'UnicodeCharacter0xe649': '\xee\x99\x89',
'LatinSmallLetterHengWithHook': '\xc9\xa7',
'HangulSyllableJyelm': '\xec\xa1\x9e',
'UnicodeCharacter0xe562': '\xee\x95\xa2',
'CjkUnifiedIdeograph8baf': '\xe8\xae\xaf',
'UnicodeCharacter0xf4a1': '\xef\x92\xa1',
'ArabicLetterAinWithThreeDotsPointingDownwardsAbove': '\xdd\x9e',
'CjkUnifiedIdeograph8bae': '\xe8\xae\xae',
'GreekCapitalLetterUpsilonWithDasiaAndPerispomeni': '\xe1\xbd\x9f',
'HangulSyllableNyuh': '\xeb\x8a\x8f',
'HangulSyllableGyags': '\xea\xb0\xbb',
'CjkUnifiedIdeograph8bad': '\xe8\xae\xad',
'HangulSyllableJyelh': '\xec\xa1\xa3',
'CircledKatakanaMa': '\xe3\x8b\xae',
'CjkUnifiedIdeograph8bac': '\xe8\xae\xac',
'HangulSyllableNyuj': '\xeb\x8a\x8a',
'CjkUnifiedIdeograph5e7c': '\xe5\xb9\xbc',
'CjkUnifiedIdeograph8bab': '\xe8\xae\xab',
'CjkUnifiedIdeograph6d0d': '\xe6\xb4\x8d',
'CjkUnifiedIdeograph6d0e': '\xe6\xb4\x8e',
'CjkUnifiedIdeograph6d0f': '\xe6\xb4\x8f',
'ModifierLetterSmallZWithRetroflexHook': '\xe1\xb6\xbc',
'CjkUnifiedIdeograph7a5e': '\xe7\xa9\x9e',
'CjkUnifiedIdeograph6d0a': '\xe6\xb4\x8a',
'CjkUnifiedIdeograph6d0b': '\xe6\xb4\x8b',
'CjkUnifiedIdeograph6d0c': '\xe6\xb4\x8c',
'CjkUnifiedIdeograph5e7e': '\xe5\xb9\xbe',
'BalineseMusicalSymbolLefthandClosedPluk': '\xe1\xad\xbb',
'CjkUnifiedIdeograph949c': '\xe9\x92\x9c',
'CircledKatakanaMe': '\xe3\x8b\xb1',
'CjkUnifiedIdeograph5e44': '\xe5\xb9\x84',
'CjkUnifiedIdeograph5733': '\xe5\x9c\xb3',
'CjkUnifiedIdeograph5e46': '\xe5\xb9\x86',
'CjkUnifiedIdeograph5e47': '\xe5\xb9\x87',
'CjkUnifiedIdeograph5e40': '\xe5\xb9\x80',
'CjkUnifiedIdeograph5e41': '\xe5\xb9\x81',
'CjkUnifiedIdeograph5e42': '\xe5\xb9\x82',
'CjkUnifiedIdeograph5732': '\xe5\x9c\xb2',
'CjkUnifiedIdeograph37ac': '\xe3\x9e\xac',
'CjkUnifiedIdeograph37ab': '\xe3\x9e\xab',
'CjkUnifiedIdeograph37aa': '\xe3\x9e\xaa',
'CjkUnifiedIdeograph5e7f': '\xe5\xb9\xbf',
'CjkUnifiedIdeograph5e48': '\xe5\xb9\x88',
'CjkUnifiedIdeograph37af': '\xe3\x9e\xaf',
'CjkUnifiedIdeograph37ae': '\xe3\x9e\xae',
'CjkUnifiedIdeograph37ad': '\xe3\x9e\xad',
'LeftwardsArrowThroughLessthan': '\xe2\xa5\xb7',
'CjkUnifiedIdeograph5730': '\xe5\x9c\xb0',
'HangulSyllableHwalh': '\xed\x99\xa3',
'LaoSemivowelSignLo': '\xe0\xba\xbc',
'CjkUnifiedIdeograph5737': '\xe5\x9c\xb7',
'ModifierLetterColon': '\xea\x9e\x89',
'KhmerIndependentVowelRyy': '\xe1\x9e\xac',
'CjkUnifiedIdeograph5736': '\xe5\x9c\xb6',
'EthiopicSyllableZhu': '\xe1\x8b\xa1',
'CjkUnifiedIdeograph5735': '\xe5\x9c\xb5',
'UnicodeCharacter0xe9df': '\xee\xa7\x9f',
'HangulSyllableDwaet': '\xeb\x90\x95',
'UnicodeCharacter0xe9dd': '\xee\xa7\x9d',
'CjkUnifiedIdeograph5734': '\xe5\x9c\xb4',
'UnicodeCharacter0xe9db': '\xee\xa7\x9b',
'HangulSyllableDwaep': '\xeb\x90\x96',
'HangulSyllableDwaes': '\xeb\x90\x8f',
'UnicodeCharacter0xe9da': '\xee\xa7\x9a',
'UnicodeCharacter0x2bd8': '\xe2\xaf\x98',
'UnicodeCharacter0x2bd9': '\xe2\xaf\x99',
'HebrewAccentMunah': '\xd6\xa3',
'EthiopicSyllableZha': '\xe1\x8b\xa0',
'CjkUnifiedIdeograph95b8': '\xe9\x96\xb8',
'CjkUnifiedIdeograph95b9': '\xe9\x96\xb9',
'UnicodeCharacter0x2bd0': '\xe2\xaf\x90',
'UnicodeCharacter0x2bd1': '\xe2\xaf\x91',
'UnicodeCharacter0x2bd2': '\xe2\xaf\x92',
'UnicodeCharacter0x2bd3': '\xe2\xaf\x93',
'UnicodeCharacter0x2bd4': '\xe2\xaf\x94',
'UnicodeCharacter0x2bd5': '\xe2\xaf\x95',
'UnicodeCharacter0x2bd6': '\xe2\xaf\x96',
'UnicodeCharacter0x2bd7': '\xe2\xaf\x97',
'UnicodeCharacter0xee67': '\xee\xb9\xa7',
'HangulSyllableBweb': '\xeb\xb7\x89',
'CjkUnifiedIdeograph923b': '\xe9\x88\xbb',
'HangulSyllableTal': '\xed\x83\x88',
'CjkUnifiedIdeograph5e4d': '\xe5\xb9\x8d',
'CjkUnifiedIdeograph41ed': '\xe4\x87\xad',
'CjkUnifiedIdeograph37a9': '\xe3\x9e\xa9',
'CjkUnifiedIdeograph37a8': '\xe3\x9e\xa8',
'KannadaLetterVocalicLl': '\xe0\xb3\xa1',
'CjkUnifiedIdeograph5e4b': '\xe5\xb9\x8b',
'CjkUnifiedIdeograph5e4c': '\xe5\xb9\x8c',
'CjkUnifiedIdeograph37a3': '\xe3\x9e\xa3',
'CjkUnifiedIdeograph37a2': '\xe3\x9e\xa2',
'CjkUnifiedIdeograph37a1': '\xe3\x9e\xa1',
'CjkUnifiedIdeograph37a0': '\xe3\x9e\xa0',
'CjkUnifiedIdeograph37a7': '\xe3\x9e\xa7',
'CjkUnifiedIdeograph37a6': '\xe3\x9e\xa6',
'CjkUnifiedIdeograph37a5': '\xe3\x9e\xa5',
'CjkUnifiedIdeograph37a4': '\xe3\x9e\xa4',
'CjkUnifiedIdeograph9423': '\xe9\x90\xa3',
'ArabicLetterNoonIsolatedForm': '\xef\xbb\xa5',
'CjkUnifiedIdeograph95bf': '\xe9\x96\xbf',
'UnicodeCharacter0x2bda': '\xe2\xaf\x9a',
'UnicodeCharacter0x2bdb': '\xe2\xaf\x9b',
'UnicodeCharacter0x2bdc': '\xe2\xaf\x9c',
'UnicodeCharacter0x2bdd': '\xe2\xaf\x9d',
'UnicodeCharacter0x2bde': '\xe2\xaf\x9e',
'UnicodeCharacter0x2bdf': '\xe2\xaf\x9f',
'CjkUnifiedIdeograph95ba': '\xe9\x96\xba',
'HangulSyllableBwek': '\xeb\xb7\x90',
'CjkUnifiedIdeograph9420': '\xe9\x90\xa0',
'HangulSyllableJyelp': '\xec\xa1\xa2',
'UnicodeCharacter0xe9d8': '\xee\xa7\x98',
'HangulSyllableBwej': '\xeb\xb7\x8e',
'UnicodeCharacter0xe9d6': '\xee\xa7\x96',
'UnicodeCharacter0xab09': '\xea\xac\x89',
'UnicodeCharacter0xe9d4': '\xee\xa7\x94',
'UnicodeCharacter0xe9d5': '\xee\xa7\x95',
'UnicodeCharacter0xe9d2': '\xee\xa7\x92',
'LeftBlackTortoiseShellBracket': '\xe2\xa6\x97',
'UnicodeCharacter0xe9d0': '\xee\xa7\x90',
'UnicodeCharacter0xab08': '\xea\xac\x88',
'CjkUnifiedIdeograph4222': '\xe4\x88\xa2',
'CjkUnifiedIdeograph4223': '\xe4\x88\xa3',
'CjkUnifiedIdeograph4220': '\xe4\x88\xa0',
'CjkUnifiedIdeograph4221': '\xe4\x88\xa1',
'CjkUnifiedIdeograph4226': '\xe4\x88\xa6',
'CjkUnifiedIdeograph4227': '\xe4\x88\xa7',
'CjkUnifiedIdeograph4224': '\xe4\x88\xa4',
'CjkUnifiedIdeograph4225': '\xe4\x88\xa5',
'FrenchFrancSign': '\xe2\x82\xa3',
'HangulSyllableBweo': '\xeb\xb6\x9c',
'CjkUnifiedIdeograph4228': '\xe4\x88\xa8',
'CjkUnifiedIdeograph4229': '\xe4\x88\xa9',
'HangulSyllableAelb': '\xec\x95\xab',
'UnicodeCharacter0xdaad': '\xed\xaa\xad',
'CjkUnifiedIdeograph95ef': '\xe9\x97\xaf',
'UnicodeCharacter0xe8ba': '\xee\xa2\xba',
'UnicodeCharacter0xdf70': '\xed\xbd\xb0',
'UnicodeCharacter0xe8bb': '\xee\xa2\xbb',
'UnicodeCharacter0xe8be': '\xee\xa2\xbe',
'HangulSyllableBwem': '\xeb\xb7\x88',
'UnicodeCharacter0xe8bf': '\xee\xa2\xbf',
'HangulSyllableBwel': '\xeb\xb7\x80',
'HangulSyllableAels': '\xec\x95\xac',
'LimbuDigitOne': '\xe1\xa5\x87',
'HangulSyllableAelp': '\xec\x95\xae',
'CjkUnifiedIdeograph6318': '\xe6\x8c\x98',
'UnicodeCharacter0xab06': '\xea\xac\x86',
'UnicodeCharacter0xab01': '\xea\xac\x81',
'HiraganaLetterZu': '\xe3\x81\x9a',
'UnicodeCharacter0xe62d': '\xee\x98\xad',
'UnicodeCharacter0xe62c': '\xee\x98\xac',
'UnicodeCharacter0xe62b': '\xee\x98\xab',
'UnicodeCharacter0xe62a': '\xee\x98\xaa',
'UnicodeCharacter0xab00': '\xea\xac\x80',
'HiraganaLetterZo': '\xe3\x81\x9e',
'CjkUnifiedIdeograph9c1d': '\xe9\xb0\x9d',
'HammerAndPick': '\xe2\x9a\x92',
'FallingDiagonalCrossingNorthEastArrow': '\xe2\xa4\xaf',
'UnicodeCharacter0xab03': '\xea\xac\x83',
'HiraganaLetterZi': '\xe3\x81\x98',
'CjkUnifiedIdeograph9c1b': '\xe9\xb0\x9b',
'CjkUnifiedIdeograph3faf': '\xe3\xbe\xaf',
'CjkUnifiedIdeograph3fad': '\xe3\xbe\xad',
'CjkUnifiedIdeograph3fae': '\xe3\xbe\xae',
'CjkUnifiedIdeograph3fab': '\xe3\xbe\xab',
'CjkUnifiedIdeograph3fac': '\xe3\xbe\xac',
'HiraganaLetterZa': '\xe3\x81\x96',
'CjkUnifiedIdeograph3faa': '\xe3\xbe\xaa',
'UnicodeCharacter0xe8b1': '\xee\xa2\xb1',
'UnicodeCharacter0xe8b0': '\xee\xa2\xb0',
'LatinCapitalLetterTWithRetroflexHook': '\xc6\xae',
'HangulSyllableReot': '\xeb\xa0\x85',
'HangulSyllableReos': '\xeb\x9f\xbf',
'UnicodeCharacter0xe8b4': '\xee\xa2\xb4',
'HangulSyllableJoss': '\xec\xa2\x84',
'HangulSyllableReop': '\xeb\xa0\x86',
'UnicodeCharacter0xe8b9': '\xee\xa2\xb9',
'UnicodeCharacter0xe8b8': '\xee\xa2\xb8',
'ArabicLetterAlef': '\xd8\xa7',
'LimbuLetterJa': '\xe1\xa4\x88',
'VaiSyllableNjoo': '\xea\x96\x8d',
'CjkUnifiedIdeograph422b': '\xe4\x88\xab',
'CjkUnifiedIdeograph422c': '\xe4\x88\xac',
'BlackCircle': '\xe2\x97\x8f',
'CjkUnifiedIdeograph422a': '\xe4\x88\xaa',
'CjkUnifiedIdeograph422f': '\xe4\x88\xaf',
'CjkUnifiedIdeograph88b2': '\xe8\xa2\xb2',
'CjkUnifiedIdeograph422d': '\xe4\x88\xad',
'CjkUnifiedIdeograph422e': '\xe4\x88\xae',
'HangulSyllableReon': '\xeb\x9f\xb0',
'HangulSyllableReom': '\xeb\x9f\xbc',
'HangulSyllableReol': '\xeb\x9f\xb4',
'HangulSyllableReok': '\xeb\xa0\x84',
'HangulSyllableReoj': '\xeb\xa0\x82',
'CjkUnifiedIdeograph923c': '\xe9\x88\xbc',
'CjkUnifiedIdeograph9c15': '\xe9\xb0\x95',
'CjkUnifiedIdeograph8745': '\xe8\x9d\x85',
'CjkUnifiedIdeograph9c17': '\xe9\xb0\x97',
'CjkUnifiedIdeograph9c16': '\xe9\xb0\x96',
'CjkUnifiedIdeograph9c11': '\xe9\xb0\x91',
'CjkUnifiedIdeograph9c10': '\xe9\xb0\x90',
'CjkUnifiedIdeograph3fa8': '\xe3\xbe\xa8',
'CjkUnifiedIdeograph3fa9': '\xe3\xbe\xa9',
'CjkUnifiedIdeograph3fa6': '\xe3\xbe\xa6',
'CjkUnifiedIdeograph3fa7': '\xe3\xbe\xa7',
'CjkUnifiedIdeograph3fa4': '\xe3\xbe\xa4',
'CjkUnifiedIdeograph3fa5': '\xe3\xbe\xa5',
'CjkUnifiedIdeograph3fa2': '\xe3\xbe\xa2',
'CjkUnifiedIdeograph3fa3': '\xe3\xbe\xa3',
'CjkUnifiedIdeograph3fa0': '\xe3\xbe\xa0',
'CjkUnifiedIdeograph3fa1': '\xe3\xbe\xa1',
'CjkUnifiedIdeograph8ba7': '\xe8\xae\xa7',
'CjkUnifiedIdeograph8746': '\xe8\x9d\x86',
'HangulSyllableDdyanj': '\xeb\x95\xad',
'UnicodeCharacter0xe629': '\xee\x98\xa9',
'HangulSyllableDdyanh': '\xeb\x95\xae',
'HangulSyllableDdyang': '\xeb\x95\xbd',
'CjkUnifiedIdeograph8741': '\xe8\x9d\x81',
'UnicodeCharacter0xe625': '\xee\x98\xa5',
'UnicodeCharacter0xe624': '\xee\x98\xa4',
'UnicodeCharacter0xe623': '\xee\x98\xa3',
'CjkUnifiedIdeograph8ba5': '\xe8\xae\xa5',
'UnicodeCharacter0xe621': '\xee\x98\xa1',
'CjkUnifiedIdeograph8740': '\xe8\x9d\x80',
'TwoDotPunctuation': '\xe2\x81\x9a',
'CjkRadicalJsimplifiedTurtle': '\xe2\xbb\xb2',
'CjkUnifiedIdeograph6e1a': '\xe6\xb8\x9a',
'CjkUnifiedIdeograph8743': '\xe8\x9d\x83',
'CjkUnifiedIdeograph5e70': '\xe5\xb9\xb0',
'Sun': '\xe2\x98\x89',
'CjkUnifiedIdeograph8742': '\xe8\x9d\x82',
'TibetanSubjoinedLetterCa': '\xe0\xbe\x95',
'UnicodeCharacter0x2be5': '\xe2\xaf\xa5',
'CjkUnifiedIdeograph8ba2': '\xe8\xae\xa2',
'SaurashtraLetterPa': '\xea\xa2\xa6',
'BlackRightPointingIndex': '\xe2\x98\x9b',
'CjkUnifiedIdeograph5e72': '\xe5\xb9\xb2',
'CjkUnifiedIdeograph938f': '\xe9\x8e\x8f',
'IntegralWithLeftwardsArrowWithHook': '\xe2\xa8\x97',
'CjkUnifiedIdeograph8ba0': '\xe8\xae\xa0',
'HangulSyllableJjweol': '\xec\xad\xac',
'CjkUnifiedIdeograph5e74': '\xe5\xb9\xb4',
'HangulSyllableTeot': '\xed\x85\x89',
'CjkCompatibilityIdeographf94b': '\xef\xa5\x8b',
'CjkUnifiedIdeograph38a8': '\xe3\xa2\xa8',
'CjkUnifiedIdeograph38a9': '\xe3\xa2\xa9',
'TeluguSignVisarga': '\xe0\xb0\x83',
'CjkUnifiedIdeograph38a4': '\xe3\xa2\xa4',
'CjkUnifiedIdeograph38a5': '\xe3\xa2\xa5',
'CjkUnifiedIdeograph38a6': '\xe3\xa2\xa6',
'CjkUnifiedIdeograph38a7': '\xe3\xa2\xa7',
'CjkUnifiedIdeograph38a0': '\xe3\xa2\xa0',
'CjkUnifiedIdeograph38a1': '\xe3\xa2\xa1',
'CjkUnifiedIdeograph38a2': '\xe3\xa2\xa2',
'CjkUnifiedIdeograph38a3': '\xe3\xa2\xa3',
'CjkUnifiedIdeograph5571': '\xe5\x95\xb1',
'CjkUnifiedIdeograph5570': '\xe5\x95\xb0',
'CjkUnifiedIdeograph5573': '\xe5\x95\xb3',
'CjkUnifiedIdeograph5572': '\xe5\x95\xb2',
'CjkUnifiedIdeograph5575': '\xe5\x95\xb5',
'CjkUnifiedIdeograph5574': '\xe5\x95\xb4',
'CjkUnifiedIdeograph5577': '\xe5\x95\xb7',
'CjkUnifiedIdeograph5576': '\xe5\x95\xb6',
'CjkUnifiedIdeograph5579': '\xe5\x95\xb9',
'CjkUnifiedIdeograph5578': '\xe5\x95\xb8',
'CjkCompatibilityIdeographf94f': '\xef\xa5\x8f',
'GreekSmallLetterOmicron': '\xce\xbf',
'UnicodeCharacter0xf464': '\xef\x91\xa4',
'LatinSmallLetterIWithHookAbove': '\xe1\xbb\x89',
'ArmenianCapitalLetterNow': '\xd5\x86',
'UnicodeCharacter0xf465': '\xef\x91\xa5',
'UnicodeCharacter0xf467': '\xef\x91\xa7',
'UnicodeCharacter0xeb3d': '\xee\xac\xbd',
'CanadianSyllabicsNaskapiKwaa': '\xe1\x92\x82',
'UnicodeCharacter0xf461': '\xef\x91\xa1',
'CjkUnifiedIdeograph557a': '\xe5\x95\xba',
'CjkUnifiedIdeograph557c': '\xe5\x95\xbc',
'CjkUnifiedIdeograph557b': '\xe5\x95\xbb',
'CjkUnifiedIdeograph557e': '\xe5\x95\xbe',
'CjkUnifiedIdeograph557d': '\xe5\x95\xbd',
'CjkUnifiedIdeograph557f': '\xe5\x95\xbf',
'KhmerSymbolLekAttakSon': '\xe1\x9f\xb0',
'VaiSyllableGi': '\xea\x95\x85',
'HangulSyllableTaebs': '\xed\x83\xae',
'CjkUnifiedIdeograph38ad': '\xe3\xa2\xad',
'CjkUnifiedIdeograph38ae': '\xe3\xa2\xae',
'CjkUnifiedIdeograph38af': '\xe3\xa2\xaf',
'CjkUnifiedIdeograph38aa': '\xe3\xa2\xaa',
'CjkUnifiedIdeograph38ab': '\xe3\xa2\xab',
'CjkUnifiedIdeograph38ac': '\xe3\xa2\xac',
'ArabicLigatureDadWithHahIsolatedForm': '\xef\xb0\xa3',
'UnicodeCharacter0xf468': '\xef\x91\xa8',
'UnicodeCharacter0xf469': '\xef\x91\xa9',
'CjkUnifiedIdeograph6170': '\xe6\x85\xb0',
'CjkUnifiedIdeograph6171': '\xe6\x85\xb1',
'GreekSmallLetterAlphaWithDasiaAndYpogegrammeni': '\xe1\xbe\x81',
'CjkUnifiedIdeograph6173': '\xe6\x85\xb3',
'CjkUnifiedIdeograph6174': '\xe6\x85\xb4',
'CjkUnifiedIdeograph6175': '\xe6\x85\xb5',
'CjkUnifiedIdeograph6176': '\xe6\x85\xb6',
'GeorgianSmallLetterZhar': '\xe2\xb4\x8f',
'CjkUnifiedIdeograph6178': '\xe6\x85\xb8',
'CjkUnifiedIdeograph6179': '\xe6\x85\xb9',
'CjkUnifiedIdeograph9308': '\xe9\x8c\x88',
'CjkUnifiedIdeograph9309': '\xe9\x8c\x89',
'CyrillicCapitalLetterWe': '\xd4\x9c',
'YiSyllableNyiex': '\xea\x91\x8f',
'CjkUnifiedIdeograph53ed': '\xe5\x8f\xad',
'YiSyllableNyiet': '\xea\x91\x8e',
'YiSyllableNyiep': '\xea\x91\x91',
'CjkUnifiedIdeograph476f': '\xe4\x9d\xaf',
'CjkUnifiedIdeograph476e': '\xe4\x9d\xae',
'CjkUnifiedIdeograph476d': '\xe4\x9d\xad',
'CjkUnifiedIdeograph476c': '\xe4\x9d\xac',
'CjkUnifiedIdeograph476b': '\xe4\x9d\xab',
'CjkUnifiedIdeograph476a': '\xe4\x9d\xaa',
'CjkUnifiedIdeograph5084': '\xe5\x82\x84',
'HangulSyllableTeos': '\xed\x85\x83',
'CombiningDoubleVerticalStrokeOverlay': '\xe2\x83\xa6',
'DevanagariDoubleDanda': '\xe0\xa5\xa5',
'CjkUnifiedIdeograph6827': '\xe6\xa0\xa7',
'CjkUnifiedIdeograph930b': '\xe9\x8c\x8b',
'CjkUnifiedIdeograph617a': '\xe6\x85\xba',
'CjkUnifiedIdeograph617b': '\xe6\x85\xbb',
'CjkUnifiedIdeograph617c': '\xe6\x85\xbc',
'CjkUnifiedIdeograph617d': '\xe6\x85\xbd',
'CjkUnifiedIdeograph617e': '\xe6\x85\xbe',
'CjkUnifiedIdeograph617f': '\xe6\x85\xbf',
'CjkUnifiedIdeograph930e': '\xe9\x8c\x8e',
'CjkUnifiedIdeograph53eb': '\xe5\x8f\xab',
'ModifierLetterDotVerticalBar': '\xea\x9c\x97',
'TibetanMarkNyisShad': '\xe0\xbc\x8e',
'CjkUnifiedIdeograph4767': '\xe4\x9d\xa7',
'CjkUnifiedIdeograph4766': '\xe4\x9d\xa6',
'CjkUnifiedIdeograph4765': '\xe4\x9d\xa5',
'CjkUnifiedIdeograph4764': '\xe4\x9d\xa4',
'CjkUnifiedIdeograph4763': '\xe4\x9d\xa3',
'CjkUnifiedIdeograph4762': '\xe4\x9d\xa2',
'CjkUnifiedIdeograph4761': '\xe4\x9d\xa1',
'CjkUnifiedIdeograph4760': '\xe4\x9d\xa0',
'EthiopicSyllableXaa': '\xe1\x8a\x83',
'HangulSyllableSot': '\xec\x86\xa5',
'HangulSyllableHyo': '\xed\x9a\xa8',
'HangulSyllableSos': '\xec\x86\x9f',
'CjkUnifiedIdeograph4769': '\xe4\x9d\xa9',
'CjkUnifiedIdeograph4768': '\xe4\x9d\xa8',
'HangulSyllableSon': '\xec\x86\x90',
'HangulSyllableSol': '\xec\x86\x94',
'HangulSyllableSom': '\xec\x86\x9c',
'HangulSyllableSoj': '\xec\x86\xa2',
'HangulSyllableSok': '\xec\x86\xa4',
'HangulSyllableSoh': '\xec\x86\xa7',
'UnicodeCharacter0xde01': '\xed\xb8\x81',
'SucceedsOrEquivalentTo': '\xe2\x89\xbf',
'HangulSyllableSod': '\xec\x86\x93',
'HangulSyllableSoe': '\xec\x87\xa0',
'HangulSyllableSob': '\xec\x86\x9d',
'HangulSyllableSoc': '\xec\x86\xa3',
'MusicSharpSign': '\xe2\x99\xaf',
'CjkUnifiedIdeograph874e': '\xe8\x9d\x8e',
'ArabicLetterHehDoachashmeeInitialForm': '\xef\xae\xac',
'ExtendedArabicindicDigitSix': '\xdb\xb6',
'CjkUnifiedIdeograph7229': '\xe7\x88\xa9',
'CjkUnifiedIdeograph7228': '\xe7\x88\xa8',
'CjkUnifiedIdeograph7227': '\xe7\x88\xa7',
'CjkUnifiedIdeograph7226': '\xe7\x88\xa6',
'CjkUnifiedIdeograph7225': '\xe7\x88\xa5',
'CjkUnifiedIdeograph7224': '\xe7\x88\xa4',
'CjkUnifiedIdeograph7223': '\xe7\x88\xa3',
'CjkUnifiedIdeograph7222': '\xe7\x88\xa2',
'CjkUnifiedIdeograph7221': '\xe7\x88\xa1',
'CjkUnifiedIdeograph631e': '\xe6\x8c\x9e',
'HangulSyllableMip': '\xeb\xb0\x92',
'UnicodeCharacter0xe893': '\xee\xa2\x93',
'CjkUnifiedIdeograph874c': '\xe8\x9d\x8c',
'CjkUnifiedIdeograph368a': '\xe3\x9a\x8a',
'UnicodeCharacter0xdd0a': '\xed\xb4\x8a',
'CjkUnifiedIdeograph368c': '\xe3\x9a\x8c',
'CjkUnifiedIdeograph368b': '\xe3\x9a\x8b',
'CjkUnifiedIdeograph368e': '\xe3\x9a\x8e',
'CjkUnifiedIdeograph368d': '\xe3\x9a\x8d',
'UnicodeCharacter0xdd0f': '\xed\xb4\x8f',
'CjkUnifiedIdeograph368f': '\xe3\x9a\x8f',
'CombiningPalatalizedHookBelow': '\xcc\xa1',
'CjkUnifiedIdeograph9ef3': '\xe9\xbb\xb3',
'CjkUnifiedIdeograph9ef0': '\xe9\xbb\xb0',
'CjkUnifiedIdeograph9ef1': '\xe9\xbb\xb1',
'CjkUnifiedIdeograph9ef6': '\xe9\xbb\xb6',
'CjkUnifiedIdeograph9ef7': '\xe9\xbb\xb7',
'CjkUnifiedIdeograph3939': '\xe3\xa4\xb9',
'CjkUnifiedIdeograph3938': '\xe3\xa4\xb8',
'CjkUnifiedIdeograph3937': '\xe3\xa4\xb7',
'CjkUnifiedIdeograph3936': '\xe3\xa4\xb6',
'CjkUnifiedIdeograph3935': '\xe3\xa4\xb5',
'CjkUnifiedIdeograph3934': '\xe3\xa4\xb4',
'CjkUnifiedIdeograph3933': '\xe3\xa4\xb3',
'CjkUnifiedIdeograph3932': '\xe3\xa4\xb2',
'CjkUnifiedIdeograph3931': '\xe3\xa4\xb1',
'CjkUnifiedIdeograph3930': '\xe3\xa4\xb0',
'CjkCompatibilityIdeographf942': '\xef\xa5\x82',
'HangulSyllableGgyugg': '\xeb\x80\xaa',
'KannadaLetterSha': '\xe0\xb2\xb6',
'CjkCompatibilityIdeographf944': '\xef\xa5\x84',
'CjkCompatibilityIdeographf947': '\xef\xa5\x87',
'CjkUnifiedIdeograph722f': '\xe7\x88\xaf',
'CjkUnifiedIdeograph722e': '\xe7\x88\xae',
'CjkUnifiedIdeograph722d': '\xe7\x88\xad',
'CjkUnifiedIdeograph722c': '\xe7\x88\xac',
'TibetanLetterPa': '\xe0\xbd\x94',
'CjkUnifiedIdeograph722a': '\xe7\x88\xaa',
'CjkUnifiedIdeograph9efb': '\xe9\xbb\xbb',
'CjkUnifiedIdeograph9efc': '\xe9\xbb\xbc',
'OlChikiDigitSeven': '\xe1\xb1\x97',
'CjkUnifiedIdeograph95e0': '\xe9\x97\xa0',
'CjkUnifiedIdeograph9eff': '\xe9\xbb\xbf',
'CjkUnifiedIdeograph9efd': '\xe9\xbb\xbd',
'CjkUnifiedIdeograph9efe': '\xe9\xbb\xbe',
'CjkUnifiedIdeograph393f': '\xe3\xa4\xbf',
'CjkUnifiedIdeograph393e': '\xe3\xa4\xbe',
'CjkUnifiedIdeograph393d': '\xe3\xa4\xbd',
'CjkUnifiedIdeograph393c': '\xe3\xa4\xbc',
'CjkUnifiedIdeograph393b': '\xe3\xa4\xbb',
'CjkUnifiedIdeograph393a': '\xe3\xa4\xba',
'CjkUnifiedIdeograph95e6': '\xe9\x97\xa6',
'CjkUnifiedIdeograph3689': '\xe3\x9a\x89',
'CjkUnifiedIdeograph3688': '\xe3\x9a\x88',
'HangulSyllableWig': '\xec\x9c\x85',
'HangulJungseongOye': '\xe1\x86\x81',
'UnicodeCharacter0xedbd': '\xee\xb6\xbd',
'CjkUnifiedIdeograph3681': '\xe3\x9a\x81',
'CjkUnifiedIdeograph3680': '\xe3\x9a\x80',
'CjkUnifiedIdeograph3683': '\xe3\x9a\x83',
'CjkUnifiedIdeograph3682': '\xe3\x9a\x82',
'CjkUnifiedIdeograph3685': '\xe3\x9a\x85',
'CjkUnifiedIdeograph3684': '\xe3\x9a\x84',
'CjkUnifiedIdeograph3687': '\xe3\x9a\x87',
'CjkUnifiedIdeograph3686': '\xe3\x9a\x86',
'HangulSyllableSsweobs': '\xec\x92\x92',
'UnicodeCharacter0xde00': '\xed\xb8\x80',
'HebrewLetterVavWithDagesh': '\xef\xac\xb5',
'HangulSyllablePwil': '\xed\x93\x94',
'HangulSyllableWil': '\xec\x9c\x8c',
'UnicodeCharacter0xd7eb': '\xed\x9f\xab',
'UnicodeCharacter0xd7ec': '\xed\x9f\xac',
'UnicodeCharacter0xd7ed': '\xed\x9f\xad',
'UnicodeCharacter0xd7ee': '\xed\x9f\xae',
'UnicodeCharacter0xd7ef': '\xed\x9f\xaf',
'ParenthesizedDigitFive': '\xe2\x91\xb8',
'CjkUnifiedIdeograph5eb6': '\xe5\xba\xb6',
'HangulSyllableWin': '\xec\x9c\x88',
'UnicodeCharacter0xf6f6': '\xef\x9b\xb6',
'UnicodeCharacter0xf6f7': '\xef\x9b\xb7',
'UnicodeCharacter0xf6f4': '\xef\x9b\xb4',
'UnicodeCharacter0xf6f5': '\xef\x9b\xb5',
'Umbrella': '\xe2\x98\x82',
'CjkUnifiedIdeograph5eb4': '\xe5\xba\xb4',
'UnicodeCharacter0xf6f0': '\xef\x9b\xb0',
'UnicodeCharacter0xf6f1': '\xef\x9b\xb1',
'CjkUnifiedIdeograph5eb5': '\xe5\xba\xb5',
'UnicodeCharacter0xf0a9': '\xef\x82\xa9',
'UnicodeCharacter0xf6f8': '\xef\x9b\xb8',
'UnicodeCharacter0xf6f9': '\xef\x9b\xb9',
'CjkUnifiedIdeograph60ac': '\xe6\x82\xac',
'CjkUnifiedIdeograph60ab': '\xe6\x82\xab',
'CjkUnifiedIdeograph60aa': '\xe6\x82\xaa',
'MongolianLetterTodoDa': '\xe1\xa1\x91',
'CjkUnifiedIdeograph60af': '\xe6\x82\xaf',
'CjkUnifiedIdeograph60ae': '\xe6\x82\xae',
'CjkUnifiedIdeograph60ad': '\xe6\x82\xad',
'HangulSyllableWik': '\xec\x9c\x9c',
'CjkUnifiedIdeograph5eb8': '\xe5\xba\xb8',
'HangulSyllableCyig': '\xec\xb8\xbd',
'ModifierLetterCapitalAe': '\xe1\xb4\xad',
'GeorgianSmallLetterHie': '\xe2\xb4\xa2',
'UnicodeCharacter0xd7e2': '\xed\x9f\xa2',
'CjkUnifiedIdeograph5eb9': '\xe5\xba\xb9',
'UnicodeCharacter0xd7e4': '\xed\x9f\xa4',
'HangulSyllableBbob': '\xeb\xbd\x91',
'UnicodeCharacter0xd7e6': '\xed\x9f\xa6',
'TripleNestedGreaterthan': '\xe2\xab\xb8',
'UnicodeCharacter0xd7e8': '\xed\x9f\xa8',
'UnicodeCharacter0xd7e9': '\xed\x9f\xa9',
'LepchaLetterTsha': '\xe1\xb0\x98',
'CjkUnifiedIdeograph60a3': '\xe6\x82\xa3',
'CjkUnifiedIdeograph60a2': '\xe6\x82\xa2',
'CjkUnifiedIdeograph60a1': '\xe6\x82\xa1',
'CjkUnifiedIdeograph60a0': '\xe6\x82\xa0',
'CjkUnifiedIdeograph60a7': '\xe6\x82\xa7',
'CjkUnifiedIdeograph60a6': '\xe6\x82\xa6',
'CjkUnifiedIdeograph60a5': '\xe6\x82\xa5',
'CjkUnifiedIdeograph60a4': '\xe6\x82\xa4',
'HangulSyllablePalp': '\xed\x8c\x9a',
'CjkUnifiedIdeograph60a9': '\xe6\x82\xa9',
'CjkUnifiedIdeograph60a8': '\xe6\x82\xa8',
'HangulSyllableHyeolm': '\xed\x98\x8a',
'HangulSyllableKyogg': '\xec\xbf\x86',
'HangulLetterRieulpansios': '\xe3\x85\xac',
'CjkRadicalWestOne': '\xe2\xbb\x83',
'UnicodeCharacter0xf6fe': '\xef\x9b\xbe',
'UnicodeCharacter0xf6fb': '\xef\x9b\xbb',
'HangulSyllableCyil': '\xec\xb9\x84',
'HangulSyllableHyeolt': '\xed\x98\x8d',
'UnicodeCharacter0xf6fa': '\xef\x9b\xba',
'HangulSyllableCyim': '\xec\xb9\x8c',
'KannadaLetterAa': '\xe0\xb2\x86',
'CjkRadicalLameThree': '\xe2\xba\x90',
'KannadaLetterAi': '\xe0\xb2\x90',
'KannadaLetterAu': '\xe0\xb2\x94',
'A': '\xce\x91',
'CjkUnifiedIdeograph3b3a': '\xe3\xac\xba',
'CjkUnifiedIdeograph3b3b': '\xe3\xac\xbb',
'CjkUnifiedIdeograph3b3c': '\xe3\xac\xbc',
'CjkUnifiedIdeograph3b3d': '\xe3\xac\xbd',
'CjkUnifiedIdeograph3b3e': '\xe3\xac\xbe',
'CjkUnifiedIdeograph3b3f': '\xe3\xac\xbf',
'HangulSyllableDols': '\xeb\x8f\x90',
'HangulSyllableDolp': '\xeb\x8f\x92',
'CjkUnifiedIdeograph764f': '\xe7\x99\x8f',
'HangulSyllableDolt': '\xeb\x8f\x91',
'HebrewLetterShinWithDageshAndSinDot': '\xef\xac\xad',
'subbeta': '\xe1\xb5\xa6',
'HangulSyllableDolb': '\xeb\x8f\x8f',
'HangulSyllableGgyass': '\xea\xba\x98',
'KangxiRadicalMouth': '\xe2\xbc\x9d',
'HangulSyllableDolh': '\xeb\x8f\x93',
'HangulSyllableDolm': '\xeb\x8f\x8e',
'CjkUnifiedIdeograph3b38': '\xe3\xac\xb8',
'CjkUnifiedIdeograph3b39': '\xe3\xac\xb9',
'CjkUnifiedIdeograph3b30': '\xe3\xac\xb0',
'CjkUnifiedIdeograph3b31': '\xe3\xac\xb1',
'CjkUnifiedIdeograph3b32': '\xe3\xac\xb2',
'CjkUnifiedIdeograph3b33': '\xe3\xac\xb3',
'CjkUnifiedIdeograph3b34': '\xe3\xac\xb4',
'CjkUnifiedIdeograph3b35': '\xe3\xac\xb5',
'CjkUnifiedIdeograph3b36': '\xe3\xac\xb6',
'CjkUnifiedIdeograph3b37': '\xe3\xac\xb7',
'UnicodeCharacter0xe38c': '\xee\x8e\x8c',
'UnicodeCharacter0xfcd': '\xe0\xbf\x8d',
'YiSyllableVyx': '\xea\x83\xbb',
'UnicodeCharacter0xf2fc': '\xef\x8b\xbc',
'MyanmarLetterShanA': '\xe1\x80\xa2',
'UnicodeCharacter0xe38d': '\xee\x8e\x8d',
'CjkUnifiedIdeograph9b39': '\xe9\xac\xb9',
'CyrillicSmallLetterEn': '\xd0\xbd',
'TamilRupeeSign': '\xe0\xaf\xb9',
'CyrillicSmallLetterEl': '\xd0\xbb',
'CyrillicSmallLetterEm': '\xd0\xbc',
'BlackSmallDiamond': '\xe2\xac\xa9',
'CyrillicSmallLetterEf': '\xd1\x84',
'UnicodeCharacter0xe838': '\xee\xa0\xb8',
'CombiningEnclosingDiamond': '\xe2\x83\x9f',
'UnicodeCharacter0xa7dd': '\xea\x9f\x9d',
'CyrillicSmallLetterEr': '\xd1\x80',
'CyrillicSmallLetterEs': '\xd1\x81',
'HangulSyllableDwam': '\xeb\x8f\xb0',
'CjkUnifiedIdeograph9dca': '\xe9\xb7\x8a',
'CjkUnifiedIdeograph9dcb': '\xe9\xb7\x8b',
'CjkUnifiedIdeograph9dcc': '\xe9\xb7\x8c',
'CjkUnifiedIdeograph9dcd': '\xe9\xb7\x8d',
'CjkUnifiedIdeograph9dce': '\xe9\xb7\x8e',
'CjkUnifiedIdeograph9dcf': '\xe9\xb7\x8f',
'HangulSyllableDwaj': '\xeb\x8f\xb6',
'HangulSyllableDwae': '\xeb\x8f\xbc',
'HangulSyllableDwad': '\xeb\x8f\xa7',
'HangulSyllableDwag': '\xeb\x8f\xa1',
'ModifierLetterAcuteAccent': '\xcb\x8a',
'HangulSyllableDwab': '\xeb\x8f\xb1',
'CjkUnifiedIdeograph508e': '\xe5\x82\x8e',
'LatinSmallLetterPWithAcute': '\xe1\xb9\x95',
'HangulSyllableDwap': '\xeb\x8f\xba',
'HangulSyllableDwas': '\xeb\x8f\xb3',
'UnicodeCharacter0x2ffc': '\xe2\xbf\xbc',
'LatinSmallLetterBWithDotBelow': '\xe1\xb8\x85',
'BengaliAuLengthMark': '\xe0\xa7\x97',
'UnicodeCharacter0x2ffd': '\xe2\xbf\xbd',
'UnicodeCharacter0x2ffe': '\xe2\xbf\xbe',
'NairaSign': '\xe2\x82\xa6',
'LatinCapitalLetterUWithDoubleAcute': '\xc5\xb0',
'UnicodeCharacter0xf2ff': '\xef\x8b\xbf',
'UnicodeCharacter0xdf6c': '\xed\xbd\xac',
'CanadianSyllabicsWestcreeTwoo': '\xe1\x91\xa0',
'GurmukhiLetterGhha': '\xe0\xa9\x9a',
'UnicodeCharacter0xde02': '\xed\xb8\x82',
'ModifierLetterCentredRightHalfRing': '\xcb\x92',
'UnicodeCharacter0x9ca': '\xe0\xa7\x8a',
'UnicodeCharacter0xdf6e': '\xed\xbd\xae',
'HangulSyllableDdyaegg': '\xeb\x96\x86',
'DingbatNegativeCircledSansserifDigitSix': '\xe2\x9e\x8f',
'UnicodeCharacter0xdf6f': '\xed\xbd\xaf',
'HangulJungseongFiller': '\xe1\x85\xa0',
'CanadianSyllabicsYaa': '\xe1\x94\xae',
'cbeta': '\xcf\x90',
'CjkUnifiedIdeograph9dc0': '\xe9\xb7\x80',
'CjkUnifiedIdeograph9dc1': '\xe9\xb7\x81',
'CjkUnifiedIdeograph9dc2': '\xe9\xb7\x82',
'CjkUnifiedIdeograph9dc3': '\xe9\xb7\x83',
'CjkUnifiedIdeograph5f3d': '\xe5\xbc\xbd',
'CjkUnifiedIdeograph9dc5': '\xe9\xb7\x85',
'CjkUnifiedIdeograph9dc6': '\xe9\xb7\x86',
'CjkUnifiedIdeograph9dc7': '\xe9\xb7\x87',
'CjkUnifiedIdeograph9dc8': '\xe9\xb7\x88',
'CjkUnifiedIdeograph9dc9': '\xe9\xb7\x89',
'HangulSyllablePyuss': '\xed\x93\xbc',
'UnicodeCharacter0xe9e7': '\xee\xa7\xa7',
'BottomLeftCrop': '\xe2\x8c\x8d',
'UnicodeCharacter0xe9e6': '\xee\xa7\xa6',
'HangulSyllableMaek': '\xeb\xa7\xbc',
'HangulSyllableByos': '\xeb\xb5\xb7',
'HangulSyllableByop': '\xeb\xb5\xbe',
'UnicodeCharacter0xf2fd': '\xef\x8b\xbd',
'HangulSyllableByot': '\xeb\xb5\xbd',
'HangulSyllableByok': '\xeb\xb5\xbc',
'HangulSyllableByoj': '\xeb\xb5\xba',
'HangulSyllableByoh': '\xeb\xb5\xbf',
'CjkUnifiedIdeograph7049': '\xe7\x81\x89',
'HangulSyllableByom': '\xeb\xb5\xb4',
'HangulSyllableByol': '\xeb\xb5\xac',
'HangulSyllableByoc': '\xeb\xb5\xbb',
'HangulSyllableByob': '\xeb\xb5\xb5',
'HangulSyllableByog': '\xeb\xb5\xa5',
'HangulSyllableByod': '\xeb\xb5\xab',
'CjkUnifiedIdeograph98c4': '\xe9\xa3\x84',
'DownwardsArrowLeftwardsOfUpwardsArrow': '\xe2\x87\xb5',
'CjkUnifiedIdeograph98c6': '\xe9\xa3\x86',
'CjkUnifiedIdeograph98c7': '\xe9\xa3\x87',
'CjkUnifiedIdeograph98c0': '\xe9\xa3\x80',
'CjkUnifiedIdeograph98c1': '\xe9\xa3\x81',
'CjkUnifiedIdeograph98c2': '\xe9\xa3\x82',
'CjkUnifiedIdeograph98c3': '\xe9\xa3\x83',
'CjkUnifiedIdeograph98c8': '\xe9\xa3\x88',
'CjkUnifiedIdeograph98c9': '\xe9\xa3\x89',
'UnicodeCharacter0xa7da': '\xea\x9f\x9a',
'UnicodeCharacter0x605': '\xd8\x85',
'UnicodeCharacter0x604': '\xd8\x84',
'CjkUnifiedIdeograph7d29': '\xe7\xb4\xa9',
'CjkUnifiedIdeograph7d28': '\xe7\xb4\xa8',
'CjkUnifiedIdeograph7d21': '\xe7\xb4\xa1',
'CjkUnifiedIdeograph7d20': '\xe7\xb4\xa0',
'CjkUnifiedIdeograph7d23': '\xe7\xb4\xa3',
'CjkUnifiedIdeograph7d22': '\xe7\xb4\xa2',
'CjkUnifiedIdeograph7d25': '\xe7\xb4\xa5',
'HebrewAccentZarqa': '\xd6\x98',
'CjkUnifiedIdeograph7d27': '\xe7\xb4\xa7',
'CjkUnifiedIdeograph7d26': '\xe7\xb4\xa6',
'HangulSyllableBalh': '\xeb\xb0\xa3',
'HangulSyllableMaess': '\xeb\xa7\xb8',
'HangulSyllableBalm': '\xeb\xb0\x9e',
'CjkUnifiedIdeograph56ed': '\xe5\x9b\xad',
'CjkUnifiedIdeograph56ee': '\xe5\x9b\xae',
'CjkUnifiedIdeograph56ef': '\xe5\x9b\xaf',
'CjkUnifiedIdeograph56ea': '\xe5\x9b\xaa',
'CjkUnifiedIdeograph56eb': '\xe5\x9b\xab',
'MongolianLetterSibeTa': '\xe1\xa1\xa8',
'KatakanaLetterVu': '\xe3\x83\xb4',
'SmashProduct': '\xe2\xa8\xb3',
'CjkUnifiedIdeograph98cd': '\xe9\xa3\x8d',
'KatakanaLetterVe': '\xe3\x83\xb9',
'HighVoltageSign': '\xe2\x9a\xa1',
'KatakanaLetterVa': '\xe3\x83\xb7',
'CjkUnifiedIdeograph98cb': '\xe9\xa3\x8b',
'CjkUnifiedIdeograph98cc': '\xe9\xa3\x8c',
'CyrillicSmallLetterDzelo': '\xea\x99\x83',
'KatakanaLetterVo': '\xe3\x83\xba',
'UnicodeCharacter0xe517': '\xee\x94\x97',
'KatakanaLetterVi': '\xe3\x83\xb8',
'CjkUnifiedIdeograph56e8': '\xe5\x9b\xa8',
'CjkUnifiedIdeograph56e9': '\xe5\x9b\xa9',
'LatinCapitalLetterOWithHornAndHookAbove': '\xe1\xbb\x9e',
'DevanagariLetterDa': '\xe0\xa4\xa6',
'CjkUnifiedIdeograph56e6': '\xe5\x9b\xa6',
'CjkUnifiedIdeograph56e7': '\xe5\x9b\xa7',
'CjkUnifiedIdeograph56e0': '\xe5\x9b\xa0',
'CjkUnifiedIdeograph56e1': '\xe5\x9b\xa1',
'CjkUnifiedIdeograph56e2': '\xe5\x9b\xa2',
'CjkUnifiedIdeograph56e3': '\xe5\x9b\xa3',
'CjkUnifiedIdeograph64d7': '\xe6\x93\x97',
'UnicodeCharacter0xedb3': '\xee\xb6\xb3',
'CjkUnifiedIdeograph7d2a': '\xe7\xb4\xaa',
'CjkUnifiedIdeograph5ebf': '\xe5\xba\xbf',
'CjkUnifiedIdeograph7d2c': '\xe7\xb4\xac',
'CjkUnifiedIdeograph7d2b': '\xe7\xb4\xab',
'LeftMultimap': '\xe2\x9f\x9c',
'CjkUnifiedIdeograph7d2d': '\xe7\xb4\xad',
'UnicodeCharacter0xe5bf': '\xee\x96\xbf',
'CjkUnifiedIdeograph7d2f': '\xe7\xb4\xaf',
'UnicodeCharacter0xabbf': '\xea\xae\xbf',
'UnicodeCharacter0xabbe': '\xea\xae\xbe',
'UnicodeCharacter0xabbd': '\xea\xae\xbd',
'UnicodeCharacter0xabbc': '\xea\xae\xbc',
'UnicodeCharacter0xabbb': '\xea\xae\xbb',
'CjkUnifiedIdeograph4e89': '\xe4\xba\x89',
'CjkUnifiedIdeograph4e88': '\xe4\xba\x88',
'CjkUnifiedIdeograph4e87': '\xe4\xba\x87',
'CjkUnifiedIdeograph4e86': '\xe4\xba\x86',
'CjkUnifiedIdeograph4e85': '\xe4\xba\x85',
'MyanmarSymbolShanOne': '\xe1\x82\x9e',
'SquareLozenge': '\xe2\x8c\x91',
'CjkUnifiedIdeograph4e82': '\xe4\xba\x82',
'CjkUnifiedIdeograph4e81': '\xe4\xba\x81',
'CjkUnifiedIdeograph4e80': '\xe4\xba\x80',
'UnicodeCharacter0xdbd1': '\xed\xaf\x91',
'HangulSyllableKyogs': '\xec\xbf\x87',
'HangulSyllableMwib': '\xeb\xae\x99',
'HexagramForTheArmy': '\xe4\xb7\x86',
'CjkUnifiedIdeograph80a4': '\xe8\x82\xa4',
'BraillePatternDots57': '\xe2\xa1\x90',
'_x': '\xe2\x82\x93',
'CjkUnifiedIdeograph57ae': '\xe5\x9e\xae',
'CjkUnifiedIdeograph57ad': '\xe5\x9e\xad',
'_u': '\xe1\xb5\xa4',
'CjkUnifiedIdeograph57af': '\xe5\x9e\xaf',
'CjkUnifiedIdeograph57aa': '\xe5\x9e\xaa',
'_r': '\xe1\xb5\xa3',
'CjkUnifiedIdeograph57ac': '\xe5\x9e\xac',
'CjkUnifiedIdeograph57ab': '\xe5\x9e\xab',
'_o': '\xe2\x82\x92',
'HangulSyllableMwin': '\xeb\xae\x8c',
'UnicodeCharacter0xe4d9': '\xee\x93\x99',
'UnicodeCharacter0xe4d8': '\xee\x93\x98',
'UnicodeCharacter0xf028': '\xef\x80\xa8',
'UnicodeCharacter0xf029': '\xef\x80\xa9',
'SubscriptNine': '\xe2\x82\x89',
'UnicodeCharacter0xe4d3': '\xee\x93\x93',
'UnicodeCharacter0xe4d2': '\xee\x93\x92',
'UnicodeCharacter0xe4d1': '\xee\x93\x91',
'UnicodeCharacter0xe4d0': '\xee\x93\x90',
'UnicodeCharacter0xe4d7': '\xee\x93\x97',
'HangulSyllableMwil': '\xeb\xae\x90',
'UnicodeCharacter0xe4d5': '\xee\x93\x95',
'UnicodeCharacter0xe4d4': '\xee\x93\x94',
'HangulSyllableJjess': '\xec\xa9\xbc',
'HangulSyllableMwim': '\xeb\xae\x98',
'UnicodeCharacter0xdf61': '\xed\xbd\xa1',
'HangulSyllablePeu': '\xed\x94\x84',
'UnicodeCharacter0xabb7': '\xea\xae\xb7',
'UnicodeCharacter0xabb6': '\xea\xae\xb6',
'UnicodeCharacter0xabb5': '\xea\xae\xb5',
'UnicodeCharacter0xabb4': '\xea\xae\xb4',
'UnicodeCharacter0xabb3': '\xea\xae\xb3',
'UnicodeCharacter0xabb2': '\xea\xae\xb2',
'UnicodeCharacter0xabb1': '\xea\xae\xb1',
'UnicodeCharacter0xabb0': '\xea\xae\xb0',
'CjkUnifiedIdeograph4e8f': '\xe4\xba\x8f',
'CjkUnifiedIdeograph4e8e': '\xe4\xba\x8e',
'CjkUnifiedIdeograph4e8d': '\xe4\xba\x8d',
'CjkUnifiedIdeograph4e8c': '\xe4\xba\x8c',
'CjkUnifiedIdeograph4e8b': '\xe4\xba\x8b',
'CjkUnifiedIdeograph4e8a': '\xe4\xba\x8a',
'UnicodeCharacter0xabb8': '\xea\xae\xb8',
'_=': '\xe2\x82\x8c',
'UnicodeCharacter0xd11': '\xe0\xb4\x91',
'_9': '\xe2\x82\x89',
'_8': '\xe2\x82\x88',
'UnicodeCharacter0xe4dc': '\xee\x93\x9c',
'UnicodeCharacter0xe4db': '\xee\x93\x9b',
'UnicodeCharacter0xe4da': '\xee\x93\x9a',
'HangulSyllablePep': '\xed\x8e\xb2',
'_3': '\xe2\x82\x83',
'UnicodeCharacter0xe4df': '\xee\x93\x9f',
'UnicodeCharacter0xe4de': '\xee\x93\x9e',
'UnicodeCharacter0xe4dd': '\xee\x93\x9d',
'_-': '\xe2\x82\x8b',
'CanadianSyllabicsNcreeThii': '\xe1\x95\xa3',
'CjkUnifiedIdeograph57a8': '\xe5\x9e\xa8',
'_)': '\xe2\x82\x8e',
'UnicodeCharacter0xdf69': '\xed\xbd\xa9',
'CjkUnifiedIdeograph57a5': '\xe5\x9e\xa5',
'CjkUnifiedIdeograph57a4': '\xe5\x9e\xa4',
'CjkUnifiedIdeograph57a7': '\xe5\x9e\xa7',
'CjkUnifiedIdeograph57a6': '\xe5\x9e\xa6',
'BugineseLetterSa': '\xe1\xa8\x94',
'CjkUnifiedIdeograph57a0': '\xe5\x9e\xa0',
'CjkUnifiedIdeograph57a3': '\xe5\x9e\xa3',
'CjkUnifiedIdeograph57a2': '\xe5\x9e\xa2',
'UnicodeCharacter0xe9ef': '\xee\xa7\xaf',
'UnicodeCharacter0x2d97': '\xe2\xb6\x97',
'UnicodeCharacter0x2d99': '\xe2\xb6\x99',
'UnicodeCharacter0x2d98': '\xe2\xb6\x98',
'LeftSquareBracketWithTickInTopCorner': '\xe2\xa6\x8d',
'MalayalamLetterJha': '\xe0\xb4\x9d',
'BottomLeftCorner': '\xe2\x8c\x9e',
'TibetanLetterZha': '\xe0\xbd\x9e',
'UnicodeCharacter0xe9eb': '\xee\xa7\xab',
'HangulSyllableHod': '\xed\x98\xbf',
'HangulSyllableDdyilm': '\xeb\x9d\x8e',
'HangulSyllablePeg': '\xed\x8e\x99',
'HangulSyllableDdyilh': '\xeb\x9d\x93',
'HangulSyllableDdyilg': '\xeb\x9d\x8d',
'HangulSyllableDdyilb': '\xeb\x9d\x8f',
'HangulSyllablePed': '\xed\x8e\x9f',
'CjkUnifiedIdeograph7eb8': '\xe7\xba\xb8',
'CjkUnifiedIdeograph7eb9': '\xe7\xba\xb9',
'YiSyllableDdurx': '\xea\x85\x8f',
'HangulSyllableSeu': '\xec\x8a\xa4',
'CjkUnifiedIdeograph7eb4': '\xe7\xba\xb4',
'DrachmaSign': '\xe2\x82\xaf',
'CjkUnifiedIdeograph7eb6': '\xe7\xba\xb6',
'CjkUnifiedIdeograph7eb7': '\xe7\xba\xb7',
'CjkUnifiedIdeograph7eb0': '\xe7\xba\xb0',
'CjkUnifiedIdeograph7eb1': '\xe7\xba\xb1',
'CjkUnifiedIdeograph7eb2': '\xe7\xba\xb2',
'CjkUnifiedIdeograph7eb3': '\xe7\xba\xb3',
'CombiningRightArrowheadAbove': '\xcd\x90',
'HangulSyllableHoj': '\xed\x99\x8e',
'CombiningUpTackBelow': '\xcc\x9d',
'ChamConsonantSignRa': '\xea\xa8\xb4',
'UnicodeCharacter0x2d9a': '\xe2\xb6\x9a',
'UnicodeCharacter0x2d9c': '\xe2\xb6\x9c',
'UnicodeCharacter0x2d9b': '\xe2\xb6\x9b',
'UnicodeCharacter0x2d9e': '\xe2\xb6\x9e',
'UnicodeCharacter0x2d9d': '\xe2\xb6\x9d',
'UnicodeCharacter0x2d9f': '\xe2\xb6\x9f',
'HangulSyllableOg': '\xec\x98\xa5',
'YiSyllableJjurx': '\xea\x90\xab',
'HangulSyllableHon': '\xed\x98\xbc',
'HangulSyllableKyob': '\xec\xbf\x95',
'HangulSyllableSel': '\xec\x85\x80',
'UnicodeCharacter0xeb91': '\xee\xae\x91',
'CjkUnifiedIdeograph9f69': '\xe9\xbd\xa9',
'CyrillicSmallLetterHaWithHook': '\xd3\xbd',
'HangulSyllableHop': '\xed\x99\x92',
'CjkUnifiedIdeograph7ebd': '\xe7\xba\xbd',
'CjkUnifiedIdeograph7ebe': '\xe7\xba\xbe',
'CjkUnifiedIdeograph7ebf': '\xe7\xba\xbf',
'AplFunctionalSymbolCircleDiaeresis': '\xe2\x8d\xa5',
'CjkUnifiedIdeograph9f61': '\xe9\xbd\xa1',
'HanunooLetterKa': '\xe1\x9c\xa3',
'CjkUnifiedIdeograph7ebb': '\xe7\xba\xbb',
'CjkUnifiedIdeograph7ebc': '\xe7\xba\xbc',
'CanadianSyllabicsKaai': '\xe1\x91\xac',
'LeftDoubleAngleBracket': '\xe3\x80\x8a',
'HangulSyllablePeh': '\xed\x8e\xb3',
'CopticSmallLetterVida': '\xe2\xb2\x83',
'UnicodeCharacter0xf1b1': '\xef\x86\xb1',
'UnicodeCharacter0xf1b0': '\xef\x86\xb0',
'UnicodeCharacter0xf1b3': '\xef\x86\xb3',
'UnicodeCharacter0xf1b2': '\xef\x86\xb2',
'UnicodeCharacter0xe708': '\xee\x9c\x88',
'UnicodeCharacter0xe709': '\xee\x9c\x89',
'UnicodeCharacter0xf1b7': '\xef\x86\xb7',
'UnicodeCharacter0xf1b6': '\xef\x86\xb6',
'UnicodeCharacter0xe704': '\xee\x9c\x84',
'UnicodeCharacter0xe705': '\xee\x9c\x85',
'UnicodeCharacter0xe706': '\xee\x9c\x86',
'UnicodeCharacter0xe707': '\xee\x9c\x87',
'UnicodeCharacter0xe700': '\xee\x9c\x80',
'UnicodeCharacter0xe701': '\xee\x9c\x81',
'UnicodeCharacter0xe702': '\xee\x9c\x82',
'ArabicLetterSad': '\xd8\xb5',
'CjkUnifiedIdeograph5aa9': '\xe5\xaa\xa9',
'CjkUnifiedIdeograph5aa8': '\xe5\xaa\xa8',
'CjkUnifiedIdeograph5aa7': '\xe5\xaa\xa7',
'CjkUnifiedIdeograph5aa6': '\xe5\xaa\xa6',
'CjkUnifiedIdeograph5aa5': '\xe5\xaa\xa5',
'CjkUnifiedIdeograph5aa4': '\xe5\xaa\xa4',
'CjkUnifiedIdeograph5aa3': '\xe5\xaa\xa3',
'CjkUnifiedIdeograph5aa2': '\xe5\xaa\xa2',
'CjkUnifiedIdeograph5aa1': '\xe5\xaa\xa1',
'CjkUnifiedIdeograph5aa0': '\xe5\xaa\xa0',
'CjkUnifiedIdeograph8af5': '\xe8\xab\xb5',
'CjkUnifiedIdeograph8af4': '\xe8\xab\xb4',
'CjkUnifiedIdeograph8af7': '\xe8\xab\xb7',
'MongolianLetterManchuAliGaliDha': '\xe1\xa2\xa1',
'CjkUnifiedIdeograph8af1': '\xe8\xab\xb1',
'CjkUnifiedIdeograph8af0': '\xe8\xab\xb0',
'CjkUnifiedIdeograph8af3': '\xe8\xab\xb3',
'CjkUnifiedIdeograph8af2': '\xe8\xab\xb2',
'HangulSyllableNem': '\xeb\x84\xb4',
'HangulSyllableNel': '\xeb\x84\xac',
'HangulSyllableNeo': '\xeb\x84\x88',
'HangulSyllableNen': '\xeb\x84\xa8',
'CjkUnifiedIdeograph8af9': '\xe8\xab\xb9',
'CjkUnifiedIdeograph8af8': '\xe8\xab\xb8',
'HangulSyllableNek': '\xeb\x84\xbc',
'HangulSyllableNej': '\xeb\x84\xba',
'HangulSyllableNeu': '\xeb\x8a\x90',
'HangulSyllableNet': '\xeb\x84\xbd',
'CjkUnifiedIdeograph8079': '\xe8\x81\xb9',
'CjkUnifiedIdeograph8078': '\xe8\x81\xb8',
'CherokeeLetterTi': '\xe1\x8f\x98',
'HangulSyllableNes': '\xeb\x84\xb7',
'CjkUnifiedIdeograph8073': '\xe8\x81\xb3',
'CherokeeLetterTe': '\xe1\x8f\x96',
'CjkUnifiedIdeograph8071': '\xe8\x81\xb1',
'CjkUnifiedIdeograph8070': '\xe8\x81\xb0',
'CjkUnifiedIdeograph8077': '\xe8\x81\xb7',
'CherokeeLetterTa': '\xe1\x8f\x94',
'CjkUnifiedIdeograph8075': '\xe8\x81\xb5',
'CjkUnifiedIdeograph8074': '\xe8\x81\xb4',
'CjkUnifiedIdeograph62c0': '\xe6\x8b\x80',
'CjkUnifiedIdeograph5aaf': '\xe5\xaa\xaf',
'EthiopicSyllableKxwee': '\xe1\x8b\x84',
'CjkUnifiedIdeograph5aad': '\xe5\xaa\xad',
'CjkUnifiedIdeograph5aac': '\xe5\xaa\xac',
'CjkUnifiedIdeograph5aab': '\xe5\xaa\xab',
'CjkUnifiedIdeograph5aaa': '\xe5\xaa\xaa',
'UnicodeCharacter0xf1ba': '\xef\x86\xba',
'UnicodeCharacter0xed54': '\xee\xb5\x94',
'UnicodeCharacter0xf1bc': '\xef\x86\xbc',
'HangulSyllableByibs': '\xeb\xb8\xba',
'KhmerLetterSha': '\xe1\x9e\x9d',
'UnicodeCharacter0xf1bd': '\xef\x86\xbd',
'UnicodeCharacter0xf1bf': '\xef\x86\xbf',
'UnicodeCharacter0xe70d': '\xee\x9c\x8d',
'MongolianLetterTodoDza': '\xe1\xa1\x9c',
'UnicodeCharacter0xe70f': '\xee\x9c\x8f',
'CjkUnifiedIdeograph62c4': '\xe6\x8b\x84',
'CjkUnifiedIdeograph891e': '\xe8\xa4\x9e',
'UnicodeCharacter0xe70a': '\xee\x9c\x8a',
'UnicodeCharacter0xe70b': '\xee\x9c\x8b',
'UnicodeCharacter0xe70c': '\xee\x9c\x8c',
'HangulSyllablePwaep': '\xed\x90\xbe',
'HangulSyllablePwaes': '\xed\x90\xb7',
'UnicodeCharacter0xed57': '\xee\xb5\x97',
'HangulSyllablePwaet': '\xed\x90\xbd',
'CjkUnifiedIdeograph807c': '\xe8\x81\xbc',
'CjkUnifiedIdeograph807b': '\xe8\x81\xbb',
'CjkUnifiedIdeograph807a': '\xe8\x81\xba',
'CjkUnifiedIdeograph807f': '\xe8\x81\xbf',
'CjkUnifiedIdeograph807e': '\xe8\x81\xbe',
'CjkUnifiedIdeograph807d': '\xe8\x81\xbd',
'CjkUnifiedIdeograph8afe': '\xe8\xab\xbe',
'CjkUnifiedIdeograph8afd': '\xe8\xab\xbd',
'HangulSyllablePwaec': '\xed\x90\xbb',
'CjkUnifiedIdeograph8aff': '\xe8\xab\xbf',
'CjkUnifiedIdeograph8afa': '\xe8\xab\xba',
'HangulSyllableKyog': '\xec\xbf\x85',
'CjkUnifiedIdeograph8afc': '\xe8\xab\xbc',
'CjkUnifiedIdeograph8afb': '\xe8\xab\xbb',
'HangulSyllablePwaeh': '\xed\x90\xbf',
'CjkUnifiedIdeograph891f': '\xe8\xa4\x9f',
'ThaiCharacterKhoKhuat': '\xe0\xb8\x83',
'HangulSyllablePwaem': '\xed\x90\xb4',
'HangulSyllablePwael': '\xed\x90\xac',
'HangulSyllablePwaen': '\xed\x90\xa8',
'CjkUnifiedIdeograph5168': '\xe5\x85\xa8',
'CjkUnifiedIdeograph5169': '\xe5\x85\xa9',
'NaryCircledPlusOperator': '\xe2\xa8\x81',
'ParenthesizedIdeographName': '\xe3\x88\xb4',
'LatinSmallLetterHWithCircumflex': '\xc4\xa5',
'ThaiCharacterKhomut': '\xe0\xb9\x9b',
'CjkUnifiedIdeograph5161': '\xe5\x85\xa1',
'CjkUnifiedIdeograph5166': '\xe5\x85\xa6',
'CjkUnifiedIdeograph5167': '\xe5\x85\xa7',
'CjkUnifiedIdeograph5164': '\xe5\x85\xa4',
'CjkUnifiedIdeograph5165': '\xe5\x85\xa5',
'HangulSyllableSwelm': '\xec\x89\x9a',
'ArabicLetterLam': '\xd9\x84',
'CjkUnifiedIdeograph9678': '\xe9\x99\xb8',
'CjkUnifiedIdeograph8be4': '\xe8\xaf\xa4',
'HangulSyllableSwelh': '\xec\x89\x9f',
'LatinSmallLetterDz': '\xc7\xb3',
'CjkUnifiedIdeograph9672': '\xe9\x99\xb2',
'CjkUnifiedIdeograph9673': '\xe9\x99\xb3',
'CjkUnifiedIdeograph9670': '\xe9\x99\xb0',
'MediumBlackCircle': '\xe2\x9a\xab',
'CjkUnifiedIdeograph9676': '\xe9\x99\xb6',
'CjkUnifiedIdeograph9677': '\xe9\x99\xb7',
'CjkUnifiedIdeograph9674': '\xe9\x99\xb4',
'CjkUnifiedIdeograph9675': '\xe9\x99\xb5',
'HangulSyllableGgeoss': '\xea\xbb\x90',
'HangulSyllableSsoec': '\xec\x91\x83',
'HangulSyllableTab': '\xed\x83\x91',
'CjkRadicalLameFour': '\xe2\xba\x91',
'CjkUnifiedIdeograph5faa': '\xe5\xbe\xaa',
'CjkUnifiedIdeograph5fab': '\xe5\xbe\xab',
'GreekSmallLetterIotaWithDialytikaAndTonos': '\xce\x90',
'CjkUnifiedIdeograph5fad': '\xe5\xbe\xad',
'CjkUnifiedIdeograph5fae': '\xe5\xbe\xae',
'CjkUnifiedIdeograph5faf': '\xe5\xbe\xaf',
'UnicodeCharacter0xf5ac': '\xef\x96\xac',
'GujaratiSignNukta': '\xe0\xaa\xbc',
'LeftRightDoubleArrow': '\xe2\x87\x94',
'HangulSyllableDigg': '\xeb\x94\x96',
'CjkUnifiedIdeograph9c03': '\xe9\xb0\x83',
'CjkUnifiedIdeograph967b': '\xe9\x99\xbb',
'CjkUnifiedIdeograph967c': '\xe9\x99\xbc',
'CjkUnifiedIdeograph967a': '\xe9\x99\xba',
'CjkUnifiedIdeograph967f': '\xe9\x99\xbf',
'CjkUnifiedIdeograph9c00': '\xe9\xb0\x80',
'LatinLetterAlveolarClick': '\xc7\x82',
'CjkUnifiedIdeograph967e': '\xe9\x99\xbe',
'CjkUnifiedIdeograph9c01': '\xe9\xb0\x81',
'CjkUnifiedIdeograph516b': '\xe5\x85\xab',
'CjkUnifiedIdeograph516c': '\xe5\x85\xac',
'UnicodeCharacter0xe839': '\xee\xa0\xb9',
'CjkUnifiedIdeograph516a': '\xe5\x85\xaa',
'CjkUnifiedIdeograph516f': '\xe5\x85\xaf',
'CjkUnifiedIdeograph516d': '\xe5\x85\xad',
'CjkUnifiedIdeograph516e': '\xe5\x85\xae',
'CjkUnifiedIdeograph5fa0': '\xe5\xbe\xa0',
'CjkUnifiedIdeograph5fa1': '\xe5\xbe\xa1',
'CjkUnifiedIdeograph5fa2': '\xe5\xbe\xa2',
'CjkUnifiedIdeograph5fa3': '\xe5\xbe\xa3',
'CjkUnifiedIdeograph5fa4': '\xe5\xbe\xa4',
'CjkUnifiedIdeograph5fa5': '\xe5\xbe\xa5',
'HangulJongseongRieulphieuph': '\xe1\x86\xb5',
'CjkUnifiedIdeograph5fa7': '\xe5\xbe\xa7',
'CjkUnifiedIdeograph5fa8': '\xe5\xbe\xa8',
'CjkUnifiedIdeograph5fa9': '\xe5\xbe\xa9',
'YiSyllableDdup': '\xea\x85\x8e',
'HangulSyllableKass': '\xec\xba\x88',
'CjkUnifiedIdeograph556f': '\xe5\x95\xaf',
'UnicodeCharacter0xed79': '\xee\xb5\xb9',
'UnicodeCharacter0xa4ca': '\xea\x93\x8a',
'HangulSyllableAm': '\xec\x95\x94',
'CjkUnifiedIdeograph91ad': '\xe9\x86\xad',
'CjkUnifiedIdeograph80ae': '\xe8\x82\xae',
'CjkUnifiedIdeograph3ff1': '\xe3\xbf\xb1',
'UnicodeCharacter0xfdff': '\xef\xb7\xbf',
'UnicodeCharacter0xab96': '\xea\xae\x96',
'CjkUnifiedIdeograph556e': '\xe5\x95\xae',
'HangulLetterAraeae': '\xe3\x86\x8e',
'UnicodeCharacter0xa4cd': '\xea\x93\x8d',
'UnicodeCharacter0xa6c5': '\xea\x9b\x85',
'CjkUnifiedIdeograph556b': '\xe5\x95\xab',
'UnicodeCharacter0xa4ce': '\xea\x93\x8e',
'ModifierLetterHalfTriangularColon': '\xcb\x91',
'UnicodeCharacter0xeb97': '\xee\xae\x97',
'CjkUnifiedIdeograph3ff6': '\xe3\xbf\xb6',
'HangulSyllableSwip': '\xec\x8a\x86',
'CjkUnifiedIdeograph556a': '\xe5\x95\xaa',
'UnicodeCharacter0xde09': '\xed\xb8\x89',
'CjkUnifiedIdeograph8d7f': '\xe8\xb5\xbf',
'CjkUnifiedIdeograph8d7e': '\xe8\xb5\xbe',
'CjkUnifiedIdeograph8d7d': '\xe8\xb5\xbd',
'CjkUnifiedIdeograph8d7c': '\xe8\xb5\xbc',
'CjkUnifiedIdeograph8d7b': '\xe8\xb5\xbb',
'CjkUnifiedIdeograph8d7a': '\xe8\xb5\xba',
'UnicodeCharacter0xe513': '\xee\x94\x93',
'VerticalEllipsis': '\xe2\x8b\xae',
'HangulSyllableMyaegg': '\xeb\xa8\x9e',
'HangulSyllableAs': '\xec\x95\x97',
'HangulSyllablePyunj': '\xed\x93\xad',
'YiSyllableDduo': '\xea\x85\x82',
'HangulSyllablePyunh': '\xed\x93\xae',
'UnicodeCharacter0xe653': '\xee\x99\x93',
'UnicodeCharacter0xab99': '\xea\xae\x99',
'HangulSyllablePyung': '\xed\x93\xbd',
'UnicodeCharacter0x13f6': '\xe1\x8f\xb6',
'RightTranspositionBracket': '\xe2\xb8\x8a',
'GreekCapitalThetaSymbol': '\xcf\xb4',
'YiSyllableChex': '\xea\x8d\xae',
'UnicodeCharacter0xe656': '\xee\x99\x96',
'YiSyllableChet': '\xea\x8d\xad',
'YiSyllableChep': '\xea\x8d\xb0',
'UnicodeCharacter0xe657': '\xee\x99\x97',
'CjkUnifiedIdeograph4dab': '\xe4\xb6\xab',
'CjkUnifiedIdeograph8d79': '\xe8\xb5\xb9',
'CjkUnifiedIdeograph8d78': '\xe8\xb5\xb8',
'CjkUnifiedIdeograph8d77': '\xe8\xb5\xb7',
'CjkUnifiedIdeograph4dae': '\xe4\xb6\xae',
'CjkUnifiedIdeograph8d75': '\xe8\xb5\xb5',
'CjkUnifiedIdeograph8d74': '\xe8\xb5\xb4',
'CjkUnifiedIdeograph8d73': '\xe8\xb5\xb3',
'CjkUnifiedIdeograph8d72': '\xe8\xb5\xb2',
'CjkUnifiedIdeograph8d71': '\xe8\xb5\xb1',
'IdeographicTelegraphSymbolForDayTwentynine': '\xe3\x8f\xbc',
'CanadianSyllabicsWestcreeSwii': '\xe1\x93\xbb',
'Iota': '\xce\x99',
'UnicodeCharacter0xed5c': '\xee\xb5\x9c',
'XInARectangleBox': '\xe2\x8c\xa7',
'YiSyllableSsi': '\xea\x8c\xac',
'Pencil': '\xe2\x9c\x8f',
'LeftwardsArrowWithVerticalStroke': '\xe2\x87\xb7',
'YiSyllableSso': '\xea\x8c\xb7',
'VaiSymbolDoong': '\xea\x98\x9b',
'YiSyllableSsa': '\xea\x8c\xb3',
'YiSyllableSse': '\xea\x8c\xba',
'CjkUnifiedIdeograph3db0': '\xe3\xb6\xb0',
'YiSyllableSsy': '\xea\x8d\x82',
'UnicodeCharacter0xf34d': '\xef\x8d\x8d',
'UnicodeCharacter0xf34f': '\xef\x8d\x8f',
'CircledNumberThirtyFour': '\xe3\x89\x9e',
'UnicodeCharacter0x13f8': '\xe1\x8f\xb8',
'UnicodeCharacter0xf34c': '\xef\x8d\x8c',
'UnicodeCharacter0xf34b': '\xef\x8d\x8b',
'CjkUnifiedIdeograph97ee': '\xe9\x9f\xae',
'CjkUnifiedIdeograph97ed': '\xe9\x9f\xad',
'ThaiCharacterWoWaen': '\xe0\xb8\xa7',
'CjkUnifiedIdeograph97ea': '\xe9\x9f\xaa',
'CjkUnifiedIdeograph97ec': '\xe9\x9f\xac',
'CjkUnifiedIdeograph97eb': '\xe9\x9f\xab',
'HangulSyllableMilg': '\xeb\xb0\x81',
'HangulSyllableJjyilm': '\xec\xaf\xba',
'BalineseLetterJaJera': '\xe1\xac\x9b',
'HangulSyllableMilb': '\xeb\xb0\x83',
'UnicodeCharacter0xf5f8': '\xef\x97\xb8',
'HangulSyllableMilm': '\xeb\xb0\x82',
'UnicodeCharacter0xe44c': '\xee\x91\x8c',
'HangulSyllableMilh': '\xeb\xb0\x87',
'HangulSyllableSsyeobs': '\xec\x8e\xb2',
'HangulSyllableMilt': '\xeb\xb0\x85',
'HangulSyllableMils': '\xeb\xb0\x84',
'HangulSyllableMilp': '\xeb\xb0\x86',
'HangulSyllableJjwaegs': '\xec\xab\xb7',
'CjkUnifiedIdeograph875c': '\xe8\x9d\x9c',
'Alembic': '\xe2\x9a\x97',
'HangulSyllableJjyilh': '\xec\xaf\xbf',
'UnicodeCharacter0xf345': '\xef\x8d\x85',
'UnicodeCharacter0xf344': '\xef\x8d\x84',
'UnicodeCharacter0xf347': '\xef\x8d\x87',
'UnicodeCharacter0xf346': '\xef\x8d\x86',
'PlaceOfInterestSign': '\xe2\x8c\x98',
'CjkUnifiedIdeograph97e8': '\xe9\x9f\xa8',
'UnicodeCharacter0xf343': '\xef\x8d\x83',
'UnicodeCharacter0xf342': '\xef\x8d\x82',
'CjkUnifiedIdeograph97e5': '\xe9\x9f\xa5',
'CjkUnifiedIdeograph97e4': '\xe9\x9f\xa4',
'LatinSmallLetterEWithAcute': '\xc3\xa9',
'CjkUnifiedIdeograph97e6': '\xe9\x9f\xa6',
'CjkUnifiedIdeograph97e1': '\xe9\x9f\xa1',
'CjkUnifiedIdeograph97e0': '\xe9\x9f\xa0',
'CjkUnifiedIdeograph97e3': '\xe9\x9f\xa3',
'CjkUnifiedIdeograph97e2': '\xe9\x9f\xa2',
'HangulSyllableRyegg': '\xeb\xa1\x82',
'YiSyllableZhyrx': '\xea\x8d\x9f',
'GlagoliticSmallLetterShtapic': '\xe2\xb1\x9c',
'CjkUnifiedIdeograph7e25': '\xe7\xb8\xa5',
'HangulSyllableRoel': '\xeb\xa2\xb8',
'DevanagariLetterYa': '\xe0\xa4\xaf',
'CjkUnifiedIdeograph7e26': '\xe7\xb8\xa6',
'SaurashtraLetterDdha': '\xea\xa2\x9f',
'CjkUnifiedIdeograph9e0d': '\xe9\xb8\x8d',
'CjkUnifiedIdeograph9e0e': '\xe9\xb8\x8e',
'CjkUnifiedIdeograph9e0f': '\xe9\xb8\x8f',
'CjkUnifiedIdeograph7e27': '\xe7\xb8\xa7',
'CjkUnifiedIdeograph9e0a': '\xe9\xb8\x8a',
'CjkUnifiedIdeograph9e0b': '\xe9\xb8\x8b',
'CjkUnifiedIdeograph9e0c': '\xe9\xb8\x8c',
'HangulSyllableTwid': '\xed\x8a\x87',
'CjkUnifiedIdeograph7e20': '\xe7\xb8\xa0',
'inf': '\xe2\x88\x9e',
'CjkUnifiedIdeograph7e21': '\xe7\xb8\xa1',
'CjkUnifiedIdeograph5988': '\xe5\xa6\x88',
'CjkUnifiedIdeograph5989': '\xe5\xa6\x89',
'KhmerLetterDa': '\xe1\x9e\x8a',
'GreekSmallLetterRho': '\xcf\x81',
'CjkUnifiedIdeograph5984': '\xe5\xa6\x84',
'SmallerThanOrEqualTo': '\xe2\xaa\xac',
'CjkUnifiedIdeograph5986': '\xe5\xa6\x86',
'CjkUnifiedIdeograph5987': '\xe5\xa6\x87',
'CjkUnifiedIdeograph5980': '\xe5\xa6\x80',
'CjkUnifiedIdeograph5981': '\xe5\xa6\x81',
'CjkUnifiedIdeograph5982': '\xe5\xa6\x82',
'CjkUnifiedIdeograph5983': '\xe5\xa6\x83',
'HangulLetterPieupthieuth': '\xe3\x85\xb7',
'ArabicLigatureSeenWithJeemInitialForm': '\xef\xb2\xad',
'UnicodeCharacter0xf76d': '\xef\x9d\xad',
'UnicodeCharacter0xf35d': '\xef\x8d\x9d',
'CjkUnifiedIdeograph8450': '\xe8\x91\x90',
'CjkUnifiedIdeograph785d': '\xe7\xa1\x9d',
'CjkUnifiedIdeograph785e': '\xe7\xa1\x9e',
'CjkUnifiedIdeograph785f': '\xe7\xa1\x9f',
'CjkUnifiedIdeograph62cd': '\xe6\x8b\x8d',
'CjkUnifiedIdeograph785a': '\xe7\xa1\x9a',
'CjkUnifiedIdeograph785b': '\xe7\xa1\x9b',
'CjkUnifiedIdeograph785c': '\xe7\xa1\x9c',
'HangulSyllableMyuss': '\xeb\xae\xb8',
'KangxiRadicalJar': '\xe2\xbd\xb8',
'CjkUnifiedIdeograph38b8': '\xe3\xa2\xb8',
'CjkUnifiedIdeograph598d': '\xe5\xa6\x8d',
'CjkUnifiedIdeograph598e': '\xe5\xa6\x8e',
'CjkUnifiedIdeograph598f': '\xe5\xa6\x8f',
'CjkUnifiedIdeograph598a': '\xe5\xa6\x8a',
'CjkUnifiedIdeograph598b': '\xe5\xa6\x8b',
'CjkUnifiedIdeograph598c': '\xe5\xa6\x8c',
'CjkUnifiedIdeograph9e04': '\xe9\xb8\x84',
'CjkUnifiedIdeograph9e05': '\xe9\xb8\x85',
'CjkUnifiedIdeograph9e06': '\xe9\xb8\x86',
'CjkUnifiedIdeograph9e07': '\xe9\xb8\x87',
'CjkUnifiedIdeograph9e00': '\xe9\xb8\x80',
'CjkUnifiedIdeograph9e01': '\xe9\xb8\x81',
'CjkUnifiedIdeograph9e02': '\xe9\xb8\x82',
'CjkUnifiedIdeograph9e03': '\xe9\xb8\x83',
'SyriacSupralinearColonSkewedLeft': '\xdc\x88',
'CjkUnifiedIdeograph8456': '\xe8\x91\x96',
'CjkUnifiedIdeograph9e08': '\xe9\xb8\x88',
'CjkUnifiedIdeograph9e09': '\xe9\xb8\x89',
'UnicodeCharacter0xf8b4': '\xef\xa2\xb4',
'HangulSyllableSweolb': '\xec\x88\xbf',
'CjkUnifiedIdeograph7858': '\xe7\xa1\x98',
'CjkUnifiedIdeograph7859': '\xe7\xa1\x99',
'HangulSyllableSweolg': '\xec\x88\xbd',
'UnicodeCharacter0xf8b5': '\xef\xa2\xb5',
'CjkUnifiedIdeograph7854': '\xe7\xa1\x94',
'CjkUnifiedIdeograph7855': '\xe7\xa1\x95',
'BraillePatternDots125678': '\xe2\xa3\xb3',
'CjkUnifiedIdeograph7857': '\xe7\xa1\x97',
'CjkUnifiedIdeograph7850': '\xe7\xa1\x90',
'CjkUnifiedIdeograph7851': '\xe7\xa1\x91',
'CjkUnifiedIdeograph7852': '\xe7\xa1\x92',
'CjkUnifiedIdeograph7853': '\xe7\xa1\x93',
'HangulSyllableSweolp': '\xec\x89\x82',
'HangulSyllableSweols': '\xec\x89\x80',
'UnicodeCharacter0xf8b7': '\xef\xa2\xb7',
'HangulSyllableSweolt': '\xec\x89\x81',
'LaoDigitSeven': '\xe0\xbb\x97',
'UnicodeCharacter0xf8b0': '\xef\xa2\xb0',
'UnicodeCharacter0xf8b1': '\xef\xa2\xb1',
'CjkUnifiedIdeograph4da1': '\xe4\xb6\xa1',
'HangulSyllableCyet': '\xec\xb4\x85',
'EnDash': '\xe2\x80\x93',
'HangulSyllableCyes': '\xec\xb3\xbf',
'HangulSyllableCyep': '\xec\xb4\x86',
'GreekSmallLetterIotaWithVaria': '\xe1\xbd\xb6',
'HangulSyllableKyinj': '\xed\x82\x8d',
'UnicodeCharacter0xf7de': '\xef\x9f\x9e',
'UnicodeCharacter0x13fd': '\xe1\x8f\xbd',
'NorthEastArrowCrossingSouthEastArrow': '\xe2\xa4\xae',
'HiraganaLetterYo': '\xe3\x82\x88',
'YiSyllableSsiex': '\xea\x8c\xae',
'CjkUnifiedIdeograph36f9': '\xe3\x9b\xb9',
'CjkUnifiedIdeograph36f8': '\xe3\x9b\xb8',
'HangulSyllableCyeb': '\xec\xb3\xbd',
'CjkUnifiedIdeograph38b7': '\xe3\xa2\xb7',
'CjkUnifiedIdeograph36f3': '\xe3\x9b\xb3',
'CjkUnifiedIdeograph36f2': '\xe3\x9b\xb2',
'CjkUnifiedIdeograph36f1': '\xe3\x9b\xb1',
'CjkUnifiedIdeograph36f0': '\xe3\x9b\xb0',
'CjkUnifiedIdeograph36f7': '\xe3\x9b\xb7',
'CjkUnifiedIdeograph36f6': '\xe3\x9b\xb6',
'CjkUnifiedIdeograph36f5': '\xe3\x9b\xb5',
'CjkUnifiedIdeograph36f4': '\xe3\x9b\xb4',
'YiSyllableVep': '\xea\x83\xb3',
'CombiningRingOverlay': '\xe2\x83\x98',
'HangulSyllableSsoet': '\xec\x91\x85',
'YiSyllableVex': '\xea\x83\xb2',
'UnicodeCharacter0x13fa': '\xe1\x8f\xba',
'HangulSyllableBbeugg': '\xec\x81\x9a',
'CjkUnifiedIdeograph3dbe': '\xe3\xb6\xbe',
'CjkUnifiedIdeograph4da8': '\xe4\xb6\xa8',
'CjkUnifiedIdeograph9c0f': '\xe9\xb0\x8f',
'HangulJongseongKiyeokrieul': '\xe1\x87\x83',
'CjkUnifiedIdeograph9c0d': '\xe9\xb0\x8d',
'CjkUnifiedIdeograph36fc': '\xe3\x9b\xbc',
'CjkUnifiedIdeograph36fb': '\xe3\x9b\xbb',
'CopticCapitalLetterOldNubianWau': '\xe2\xb3\xa2',
'HiraganaLetterYu': '\xe3\x82\x86',
'CjkUnifiedIdeograph36ff': '\xe3\x9b\xbf',
'CjkUnifiedIdeograph36fe': '\xe3\x9b\xbe',
'CjkUnifiedIdeograph36fd': '\xe3\x9b\xbd',
'LatinCapitalLetterUWithHornAndGrave': '\xe1\xbb\xaa',
'GlagoliticSmallLetterShta': '\xe2\xb1\x8b',
'YiSyllableMgax': '\xea\x88\xb3',
'CjkUnifiedIdeograph5564': '\xe5\x95\xa4',
'NewTaiLueLetterHighQa': '\xe1\xa6\x80',
'CircledNumberTwenty': '\xe2\x91\xb3',
'HangulSyllableRoed': '\xeb\xa2\xb7',
'LatinSmallLetterAWithCircumflexAndTilde': '\xe1\xba\xab',
'HangulSyllablePwin': '\xed\x93\x90',
'CjkUnifiedIdeograph3fff': '\xe3\xbf\xbf',
'CjkUnifiedIdeograph3ffe': '\xe3\xbf\xbe',
'TagalogLetterKa': '\xe1\x9c\x83',
'ThaanaLetterYaa': '\xde\x94',
'HangulChoseongIeungcieuc': '\xe1\x85\x88',
'UnicodeCharacter0xab9c': '\xea\xae\x9c',
'HangulSyllableRyalb': '\xeb\x9e\xbf',
'UnicodeCharacter0xe56d': '\xee\x95\xad',
'CircledIdeographLeft': '\xe3\x8a\xa7',
'CjkUnifiedIdeograph43cd': '\xe4\x8f\x8d',
'CjkUnifiedIdeograph43ce': '\xe4\x8f\x8e',
'CjkUnifiedIdeograph43cf': '\xe4\x8f\x8f',
'CjkUnifiedIdeograph953b': '\xe9\x94\xbb',
'CjkUnifiedIdeograph953e': '\xe9\x94\xbe',
'CjkUnifiedIdeograph43ca': '\xe4\x8f\x8a',
'CjkUnifiedIdeograph43cb': '\xe4\x8f\x8b',
'CjkUnifiedIdeograph43cc': '\xe4\x8f\x8c',
'CjkUnifiedIdeograph7a8f': '\xe7\xaa\x8f',
'HangulSyllableRyalg': '\xeb\x9e\xbd',
'HangulSyllableJwaelp': '\xec\xa2\xb6',
'UnicodeCharacter0xe56e': '\xee\x95\xae',
'HangulSyllableJwaels': '\xec\xa2\xb4',
'HangulSyllableJwaelt': '\xec\xa2\xb5',
'CjkUnifiedIdeograph6ab9': '\xe6\xaa\xb9',
'CjkUnifiedIdeograph6ab8': '\xe6\xaa\xb8',
'CjkUnifiedIdeograph6ab7': '\xe6\xaa\xb7',
'CjkUnifiedIdeograph6ab6': '\xe6\xaa\xb6',
'CjkUnifiedIdeograph6ab5': '\xe6\xaa\xb5',
'CjkUnifiedIdeograph6ab4': '\xe6\xaa\xb4',
'CjkUnifiedIdeograph6ab3': '\xe6\xaa\xb3',
'CjkUnifiedIdeograph6ab2': '\xe6\xaa\xb2',
'CjkUnifiedIdeograph6ab1': '\xe6\xaa\xb1',
'CjkUnifiedIdeograph6ab0': '\xe6\xaa\xb0',
'UnicodeCharacter0xf7d1': '\xef\x9f\x91',
'HangulSyllableJwaelb': '\xec\xa2\xb3',
'HangulSyllableJwaelg': '\xec\xa2\xb1',
'HangulSyllableJwaelh': '\xec\xa2\xb7',
'HangulChoseongRieulnieun': '\xe1\x84\x98',
'CjkUnifiedIdeograph43c4': '\xe4\x8f\x84',
'CjkUnifiedIdeograph43c5': '\xe4\x8f\x85',
'CjkUnifiedIdeograph43c6': '\xe4\x8f\x86',
'CjkUnifiedIdeograph43c7': '\xe4\x8f\x87',
'CjkUnifiedIdeograph43c0': '\xe4\x8f\x80',
'CjkUnifiedIdeograph43c1': '\xe4\x8f\x81',
'CjkUnifiedIdeograph43c2': '\xe4\x8f\x82',
'CjkUnifiedIdeograph43c3': '\xe4\x8f\x83',
'CjkUnifiedIdeograph9539': '\xe9\x94\xb9',
'CjkUnifiedIdeograph9538': '\xe9\x94\xb8',
'UnicodeCharacter0xf22a': '\xef\x88\xaa',
'CjkUnifiedIdeograph43c8': '\xe4\x8f\x88',
'CjkUnifiedIdeograph43c9': '\xe4\x8f\x89',
'CjkUnifiedIdeograph7ef8': '\xe7\xbb\xb8',
'HangulLetterPhieuph': '\xe3\x85\x8d',
'CjkUnifiedIdeograph6abe': '\xe6\xaa\xbe',
'CjkUnifiedIdeograph6abd': '\xe6\xaa\xbd',
'CyrillicSmallLetterChe': '\xd1\x87',
'CjkUnifiedIdeograph5032': '\xe5\x80\xb2',
'CjkUnifiedIdeograph6aba': '\xe6\xaa\xba',
'CjkUnifiedIdeograph42ce': '\xe4\x8b\x8e',
'CjkUnifiedIdeograph42cd': '\xe4\x8b\x8d',
'NewTaiLueDigitFive': '\xe1\xa7\x95',
'CjkUnifiedIdeograph42ca': '\xe4\x8b\x8a',
'CjkUnifiedIdeograph42cc': '\xe4\x8b\x8c',
'CjkUnifiedIdeograph42cb': '\xe4\x8b\x8b',
'UnicodeCharacter0xf8bf': '\xef\xa2\xbf',
'BlackHexagon': '\xe2\xac\xa2',
'UnicodeCharacter0xeae0': '\xee\xab\xa0',
'HangulSyllableSsang': '\xec\x8c\x8d',
'UnicodeCharacter0xe56a': '\xee\x95\xaa',
'HalfwidthKatakanaLetterNa': '\xef\xbe\x85',
'CjkUnifiedIdeograph961a': '\xe9\x98\x9a',
'CjkUnifiedIdeograph9908': '\xe9\xa4\x88',
'UnicodeCharacter0xf8bb': '\xef\xa2\xbb',
'CjkUnifiedIdeograph4a99': '\xe4\xaa\x99',
'CjkUnifiedIdeograph707f': '\xe7\x81\xbf',
'CjkUnifiedIdeograph707d': '\xe7\x81\xbd',
'CjkUnifiedIdeograph707e': '\xe7\x81\xbe',
'CjkUnifiedIdeograph707b': '\xe7\x81\xbb',
'CjkUnifiedIdeograph659c': '\xe6\x96\x9c',
'UnicodeCharacter0xeae3': '\xee\xab\xa3',
'DevanagariLetterDdda': '\xe0\xa5\xbe',
'SundaneseLetterFa': '\xe1\xae\x96',
'SquareWithRightHalfBlack': '\xe2\x97\xa8',
'CjkUnifiedIdeograph376f': '\xe3\x9d\xaf',
'CopticCapitalLetterAkhmimicKhei': '\xe2\xb3\x88',
'CjkUnifiedIdeograph376d': '\xe3\x9d\xad',
'CjkUnifiedIdeograph376e': '\xe3\x9d\xae',
'CjkUnifiedIdeograph376b': '\xe3\x9d\xab',
'CjkUnifiedIdeograph376c': '\xe3\x9d\xac',
'CjkUnifiedIdeograph376a': '\xe3\x9d\xaa',
'HangulSyllableSsanh': '\xec\x8b\xbe',
'UnicodeCharacter0xf7d0': '\xef\x9f\x90',
'HangulSyllableDdyo': '\xeb\x9a\x80',
'HalfwidthKatakanaLetterYu': '\xef\xbe\x95',
'TamilLetterSsa': '\xe0\xae\xb7',
'CjkUnifiedIdeograph42c5': '\xe4\x8b\x85',
'CjkUnifiedIdeograph42c4': '\xe4\x8b\x84',
'CjkUnifiedIdeograph42c7': '\xe4\x8b\x87',
'CjkUnifiedIdeograph42c6': '\xe4\x8b\x86',
'CjkUnifiedIdeograph42c1': '\xe4\x8b\x81',
'CjkUnifiedIdeograph42c0': '\xe4\x8b\x80',
'CjkUnifiedIdeograph42c3': '\xe4\x8b\x83',
'RightwardsHarpoonWithBarbUpAboveRightwardsHarpoonWithBarbDown': '\xe2\xa5\xa4',
'CjkUnifiedIdeograph42c9': '\xe4\x8b\x89',
'CjkUnifiedIdeograph42c8': '\xe4\x8b\x88',
'VaiSyllableMgbi': '\xea\x94\xaf',
'TagalogLetterPa': '\xe1\x9c\x89',
'HangulSyllableRyegs': '\xeb\xa1\x83',
'CjkUnifiedIdeograph3768': '\xe3\x9d\xa8',
'CjkUnifiedIdeograph3769': '\xe3\x9d\xa9',
'CjkUnifiedIdeograph3766': '\xe3\x9d\xa6',
'MyanmarLetterShanBa': '\xe1\x81\xbf',
'CjkUnifiedIdeograph3764': '\xe3\x9d\xa4',
'CjkUnifiedIdeograph3765': '\xe3\x9d\xa5',
'CjkUnifiedIdeograph3762': '\xe3\x9d\xa2',
'CjkUnifiedIdeograph3763': '\xe3\x9d\xa3',
'CjkUnifiedIdeograph3760': '\xe3\x9d\xa0',
'CjkUnifiedIdeograph3761': '\xe3\x9d\xa1',
'CjkUnifiedIdeograph7e2b': '\xe7\xb8\xab',
'CjkUnifiedIdeograph7078': '\xe7\x81\xb8',
'CjkUnifiedIdeograph7079': '\xe7\x81\xb9',
'CjkUnifiedIdeograph5cda': '\xe5\xb3\x9a',
'CjkUnifiedIdeograph7077': '\xe7\x81\xb7',
'CjkUnifiedIdeograph7074': '\xe7\x81\xb4',
'CjkUnifiedIdeograph7075': '\xe7\x81\xb5',
'CjkUnifiedIdeograph7072': '\xe7\x81\xb2',
'CjkUnifiedIdeograph7073': '\xe7\x81\xb3',
'CjkUnifiedIdeograph7070': '\xe7\x81\xb0',
'CjkUnifiedIdeograph7071': '\xe7\x81\xb1',
'YiSyllableHmap': '\xea\x82\x9b',
'YiSyllableHmat': '\xea\x82\x98',
'YiSyllableHmax': '\xea\x82\x99',
'HangulSyllableByeong': '\xeb\xb3\x91',
'HangulChoseongPieupssangsios': '\xe1\x84\xa5',
'HalfwidthKatakanaLetterNe': '\xef\xbe\x88',
'UnicodeCharacter0xdf19': '\xed\xbc\x99',
'UnicodeCharacter0xdf18': '\xed\xbc\x98',
'CjkUnifiedIdeograph5a19': '\xe5\xa8\x99',
'UnicodeCharacter0xdf11': '\xed\xbc\x91',
'UnicodeCharacter0xdf10': '\xed\xbc\x90',
'UnicodeCharacter0xdf13': '\xed\xbc\x93',
'CjkUnifiedIdeograph4d45': '\xe4\xb5\x85',
'UnicodeCharacter0xdf15': '\xed\xbc\x95',
'UnicodeCharacter0xdf14': '\xed\xbc\x94',
'UnicodeCharacter0xdf17': '\xed\xbc\x97',
'ChamLetterBba': '\xea\xa8\xa1',
'UnicodeCharacter0xf180': '\xef\x86\x80',
'CjkUnifiedIdeograph5be0': '\xe5\xaf\xa0',
'HangulSyllableDdoess': '\xeb\x99\xb8',
'ArabicLetterEMedialForm': '\xef\xaf\xa7',
'CjkUnifiedIdeograph5a15': '\xe5\xa8\x95',
'UnicodeCharacter0xe071': '\xee\x81\xb1',
'HangulSyllableKyen': '\xec\xbc\xbc',
'HangulSyllableKyeo': '\xec\xbc\x9c',
'HangulSyllableKyel': '\xec\xbd\x80',
'CjkUnifiedIdeograph5a14': '\xe5\xa8\x94',
'HangulSyllableKyej': '\xec\xbd\x8e',
'HangulSyllableKyek': '\xec\xbd\x90',
'HangulSyllableKyeh': '\xec\xbd\x93',
'UnicodeCharacter0xf794': '\xef\x9e\x94',
'HangulSyllableCweobs': '\xec\xb7\x82',
'HangulSyllableKyed': '\xec\xbc\xbf',
'HangulSyllableKyeb': '\xec\xbd\x89',
'HangulSyllableKyec': '\xec\xbd\x8f',
'UnicodeCharacter0xdaa7': '\xed\xaa\xa7',
'CombiningLeftHarpoonAbove': '\xe2\x83\x90',
'HangulSyllableKyet': '\xec\xbd\x91',
'HangulSyllableKyes': '\xec\xbd\x8b',
'HangulSyllableKyep': '\xec\xbd\x92',
'BraillePatternDots67': '\xe2\xa1\xa0',
'HangulSyllableBbyaenh': '\xeb\xba\xba',
'HangulSyllableBbyaenj': '\xeb\xba\xb9',
'HangulSyllableBbyaeng': '\xeb\xbb\x89',
'LatinSmallLetterRWithLongLeg': '\xc9\xbc',
'HangulSyllableNyos': '\xeb\x87\xbb',
'UnicodeCharacter0xe79': '\xe0\xb9\xb9',
'HangulSyllableNyop': '\xeb\x88\x82',
'BoxDrawingsUpHeavyAndLeftLight': '\xe2\x94\x9a',
'HangulSyllableNyon': '\xeb\x87\xac',
'HangulSyllableNyom': '\xeb\x87\xb8',
'HangulSyllableNyol': '\xeb\x87\xb0',
'HangulSyllableNyok': '\xeb\x88\x80',
'HangulSyllableNyoj': '\xeb\x87\xbe',
'HangulSyllableNyoh': '\xeb\x88\x83',
'HangulSyllableNyog': '\xeb\x87\xa9',
'UnicodeCharacter0xdf1c': '\xed\xbc\x9c',
'HangulSyllableNyod': '\xeb\x87\xaf',
'HangulSyllableNyoc': '\xeb\x87\xbf',
'LatinSmallLetterGWithStroke': '\xc7\xa5',
'UnicodeCharacter0xdf1f': '\xed\xbc\x9f',
'UnicodeCharacter0xed45': '\xee\xb5\x85',
'LatinSmallLetterEWithCedilla': '\xc8\xa9',
'CentrelineOverline': '\xef\xb9\x8a',
'CjkUnifiedIdeograph6b9e': '\xe6\xae\x9e',
'CjkUnifiedIdeograph6b9d': '\xe6\xae\x9d',
'UnicodeCharacter0xfaeb': '\xef\xab\xab',
'CjkUnifiedIdeograph6b9f': '\xe6\xae\x9f',
'CjkUnifiedIdeograph6b9a': '\xe6\xae\x9a',
'UnicodeCharacter0xe71': '\xe0\xb9\xb1',
'CjkUnifiedIdeograph6b9c': '\xe6\xae\x9c',
'CjkUnifiedIdeograph6b9b': '\xe6\xae\x9b',
'CjkUnifiedIdeograph6844': '\xe6\xa1\x84',
'CjkUnifiedIdeograph6845': '\xe6\xa1\x85',
'CjkUnifiedIdeograph6846': '\xe6\xa1\x86',
'CjkUnifiedIdeograph6847': '\xe6\xa1\x87',
'CjkUnifiedIdeograph6840': '\xe6\xa1\x80',
'CjkUnifiedIdeograph6841': '\xe6\xa1\x81',
'CjkUnifiedIdeograph6842': '\xe6\xa1\x82',
'CjkUnifiedIdeograph6843': '\xe6\xa1\x83',
'HangulSyllableBbwe': '\xec\x80\x84',
'HebrewLetterWideResh': '\xef\xac\xa7',
'HangulSyllableSyilm': '\xec\x8b\x8a',
'CjkUnifiedIdeograph6848': '\xe6\xa1\x88',
'CjkUnifiedIdeograph6849': '\xe6\xa1\x89',
'HangulSyllablePyilb': '\xed\x94\xab',
'UnicodeCharacter0xebeb': '\xee\xaf\xab',
'HangulSyllablePyilt': '\xed\x94\xad',
'HangulSyllableBogg': '\xeb\xb3\xb6',
'KangxiRadicalHeart': '\xe2\xbc\xbc',
'CjkUnifiedIdeograph61cf': '\xe6\x87\x8f',
'UnicodeCharacter0xdfa4': '\xed\xbe\xa4',
'HangulSyllablePyils': '\xed\x94\xac',
'UnicodeCharacter0xfae8': '\xef\xab\xa8',
'UnicodeCharacter0xfae9': '\xef\xab\xa9',
'CjkUnifiedIdeograph7c28': '\xe7\xb0\xa8',
'CjkUnifiedIdeograph7c29': '\xe7\xb0\xa9',
'CjkUnifiedIdeograph6b99': '\xe6\xae\x99',
'CjkUnifiedIdeograph6b98': '\xe6\xae\x98',
'ArabicSmallHighMeemInitialForm': '\xdb\x98',
'CjkUnifiedIdeograph6b95': '\xe6\xae\x95',
'CjkUnifiedIdeograph6b94': '\xe6\xae\x94',
'CjkUnifiedIdeograph6b97': '\xe6\xae\x97',
'CjkUnifiedIdeograph6b96': '\xe6\xae\x96',
'CjkUnifiedIdeograph6b91': '\xe6\xae\x91',
'CjkUnifiedIdeograph6b90': '\xe6\xae\x90',
'CjkUnifiedIdeograph6b93': '\xe6\xae\x93',
'CjkUnifiedIdeograph6b92': '\xe6\xae\x92',
'HangulSyllableTyad': '\xed\x83\xbf',
'ArabicLigatureGhainWithMeemWithMeemFinalForm': '\xef\xb5\xb9',
'UnicodeCharacter0xebec': '\xee\xaf\xac',
'SmallRomanNumeralNine': '\xe2\x85\xb8',
'HangulSyllableBo': '\xeb\xb3\xb4',
'SyriacLetterHe': '\xdc\x97',
'TimesWithRightHalfBlack': '\xe2\xa7\x95',
'HangulSyllableBe': '\xeb\xb2\xa0',
'CyrillicSmallLetterHardSign': '\xd1\x8a',
'HangulSyllableBa': '\xeb\xb0\x94',
'CjkUnifiedIdeograph684d': '\xe6\xa1\x8d',
'CjkUnifiedIdeograph684e': '\xe6\xa1\x8e',
'CjkUnifiedIdeograph684f': '\xe6\xa1\x8f',
'ArabicLigatureSalamIsolatedForm': '\xef\xb7\xb5',
'CjkUnifiedIdeograph684a': '\xe6\xa1\x8a',
'CjkUnifiedIdeograph684b': '\xe6\xa1\x8b',
'CjkUnifiedIdeograph684c': '\xe6\xa1\x8c',
'HangulSyllableBu': '\xeb\xb6\x80',
'CjkUnifiedIdeograph6149': '\xe6\x85\x89',
'HalfwidthDownwardsArrow': '\xef\xbf\xac',
'ArabicLigatureYehWithZainFinalForm': '\xef\xb2\x92',
'HangulSyllableTyam': '\xed\x84\x88',
'HangulSyllableGol': '\xea\xb3\xa8',
'HangulSyllableNi': '\xeb\x8b\x88',
'UnicodeCharacter0xdaa1': '\xed\xaa\xa1',
'CjkUnifiedIdeograph7e8f': '\xe7\xba\x8f',
'CjkUnifiedIdeograph7e8d': '\xe7\xba\x8d',
'CjkUnifiedIdeograph7e8e': '\xe7\xba\x8e',
'CjkUnifiedIdeograph7e8b': '\xe7\xba\x8b',
'CjkUnifiedIdeograph7e8c': '\xe7\xba\x8c',
'CjkUnifiedIdeograph7e8a': '\xe7\xba\x8a',
'UnicodeCharacter0xafa': '\xe0\xab\xba',
'UnicodeCharacter0xafb': '\xe0\xab\xbb',
'UnicodeCharacter0xafc': '\xe0\xab\xbc',
'UnicodeCharacter0xafd': '\xe0\xab\xbd',
'UnicodeCharacter0xafe': '\xe0\xab\xbe',
'UnicodeCharacter0xaff': '\xe0\xab\xbf',
'UnicodeCharacter0xa7bb': '\xea\x9e\xbb',
'HangulSyllableGom': '\xea\xb3\xb0',
'UnicodeCharacter0xa7bc': '\xea\x9e\xbc',
'CjkUnifiedIdeograph61f3': '\xe6\x87\xb3',
'CjkUnifiedIdeograph61f2': '\xe6\x87\xb2',
'CjkUnifiedIdeograph61f1': '\xe6\x87\xb1',
'CjkUnifiedIdeograph61f0': '\xe6\x87\xb0',
'CjkUnifiedIdeograph61f7': '\xe6\x87\xb7',
'CjkUnifiedIdeograph61f6': '\xe6\x87\xb6',
'CjkUnifiedIdeograph61f5': '\xe6\x87\xb5',
'CjkUnifiedIdeograph61f4': '\xe6\x87\xb4',
'CjkUnifiedIdeograph61f9': '\xe6\x87\xb9',
'CjkUnifiedIdeograph61f8': '\xe6\x87\xb8',
'HangulSyllableGgyeogg': '\xea\xbb\xb6',
'CjkCompatibilityIdeographfad1': '\xef\xab\x91',
'HangulSyllableNa': '\xeb\x82\x98',
'CjkCompatibilityIdeographfad3': '\xef\xab\x93',
'KangxiRadicalUse': '\xe2\xbd\xa4',
'CjkCompatibilityIdeographfad5': '\xef\xab\x95',
'CjkCompatibilityIdeographfad4': '\xef\xab\x94',
'CjkCompatibilityIdeographfad7': '\xef\xab\x97',
'CjkCompatibilityIdeographfad6': '\xef\xab\x96',
'CjkCompatibilityIdeographfad9': '\xef\xab\x99',
'RightAngleWithDownwardsZigzagArrow': '\xe2\x8d\xbc',
'UnicodeCharacter0xaf0': '\xe0\xab\xb0',
'UnicodeCharacter0xaf2': '\xe0\xab\xb2',
'UnicodeCharacter0xaf3': '\xe0\xab\xb3',
'UnicodeCharacter0xaf4': '\xe0\xab\xb4',
'UnicodeCharacter0xaf5': '\xe0\xab\xb5',
'UnicodeCharacter0xaf6': '\xe0\xab\xb6',
'UnicodeCharacter0xaf7': '\xe0\xab\xb7',
'UnicodeCharacter0xaf8': '\xe0\xab\xb8',
'UnicodeCharacter0xaf9': '\xe0\xab\xb9',
'UnicodeCharacter0xf126': '\xef\x84\xa6',
'ModifierLetterExtrahighLeftstemToneBar': '\xea\x9c\x92',
'HangulSyllableRwalb': '\xeb\xa2\x83',
'CjkUnifiedIdeograph7e88': '\xe7\xba\x88',
'CjkUnifiedIdeograph7e89': '\xe7\xba\x89',
'CjkUnifiedIdeograph7e86': '\xe7\xba\x86',
'CjkUnifiedIdeograph7e87': '\xe7\xba\x87',
'CjkUnifiedIdeograph7e84': '\xe7\xba\x84',
'CjkUnifiedIdeograph7e85': '\xe7\xba\x85',
'CjkUnifiedIdeograph7e82': '\xe7\xba\x82',
'CjkUnifiedIdeograph7e83': '\xe7\xba\x83',
'CjkUnifiedIdeograph7e80': '\xe7\xba\x80',
'CjkUnifiedIdeograph7e81': '\xe7\xba\x81',
'CjkUnifiedIdeograph8f08': '\xe8\xbc\x88',
'HangulSyllablePuss': '\xed\x92\x8c',
'CjkUnifiedIdeograph61fc': '\xe6\x87\xbc',
'CjkUnifiedIdeograph61fb': '\xe6\x87\xbb',
'CjkUnifiedIdeograph61fa': '\xe6\x87\xba',
'CjkUnifiedIdeograph61ff': '\xe6\x87\xbf',
'CjkUnifiedIdeograph61fe': '\xe6\x87\xbe',
'CjkUnifiedIdeograph61fd': '\xe6\x87\xbd',
'CjkUnifiedIdeograph8f09': '\xe8\xbc\x89',
'CjkUnifiedIdeograph50bc': '\xe5\x82\xbc',
'CjkUnifiedIdeograph50bb': '\xe5\x82\xbb',
'CjkUnifiedIdeograph50ba': '\xe5\x82\xba',
'HangulSyllableTwib': '\xed\x8a\x91',
'CjkUnifiedIdeograph50bf': '\xe5\x82\xbf',
'CjkUnifiedIdeograph50be': '\xe5\x82\xbe',
'CjkUnifiedIdeograph50bd': '\xe5\x82\xbd',
'UnicodeCharacter0xebef': '\xee\xaf\xaf',
'HangulSyllableDdoet': '\xeb\x99\xbd',
'RunicLetterAcA': '\xe1\x9a\xaa',
'UnicodeCharacter0xf6d9': '\xef\x9b\x99',
'CjkUnifiedIdeograph4b80': '\xe4\xae\x80',
'CjkUnifiedIdeograph4b81': '\xe4\xae\x81',
'CjkUnifiedIdeograph4b82': '\xe4\xae\x82',
'ParenthesizedIdeographReach': '\xe3\x89\x83',
'UnicodeCharacter0x2dc7': '\xe2\xb7\x87',
'CjkUnifiedIdeograph4b85': '\xe4\xae\x85',
'CjkUnifiedIdeograph4b86': '\xe4\xae\x86',
'CjkUnifiedIdeograph4b87': '\xe4\xae\x87',
'CjkUnifiedIdeograph4b88': '\xe4\xae\x88',
'UnicodeCharacter0x1fff': '\xe1\xbf\xbf',
'BlackClubSuit': '\xe2\x99\xa3',
'YiSyllableZhep': '\xea\x8d\x94',
'True': '\xe2\x8a\xa8',
'CjkCompatibilityIdeographfaa8': '\xef\xaa\xa8',
'HangulSyllableHweoc': '\xed\x9b\xb7',
'CjkCompatibilityIdeographfaa9': '\xef\xaa\xa9',
'CjkCompatibilityIdeographfaa6': '\xef\xaa\xa6',
'CjkUnifiedIdeograph50b9': '\xe5\x82\xb9',
'CjkUnifiedIdeograph50b8': '\xe5\x82\xb8',
'CircledNumberThirtyNine': '\xe3\x8a\xb4',
'CjkCompatibilityIdeographfaa7': '\xef\xaa\xa7',
'CjkUnifiedIdeograph50b3': '\xe5\x82\xb3',
'CjkUnifiedIdeograph50b2': '\xe5\x82\xb2',
'CjkUnifiedIdeograph50b1': '\xe5\x82\xb1',
'CjkUnifiedIdeograph50b0': '\xe5\x82\xb0',
'CjkUnifiedIdeograph50b7': '\xe5\x82\xb7',
'CjkUnifiedIdeograph50b6': '\xe5\x82\xb6',
'CjkUnifiedIdeograph50b5': '\xe5\x82\xb5',
'CjkUnifiedIdeograph50b4': '\xe5\x82\xb4',
'HangulSyllableTult': '\xed\x88\xb9',
'UnicodeCharacter0xd8e7': '\xed\xa3\xa7',
'HangulSyllableHweob': '\xed\x9b\xb1',
'TamilLetterSa': '\xe0\xae\xb8',
'CjkCompatibilityIdeographfaa3': '\xef\xaa\xa3',
'CjkUnifiedIdeograph4b8a': '\xe4\xae\x8a',
'CjkUnifiedIdeograph4b8b': '\xe4\xae\x8b',
'CjkUnifiedIdeograph4b8c': '\xe4\xae\x8c',
'CjkUnifiedIdeograph4b8d': '\xe4\xae\x8d',
'UnicodeCharacter0x2dcf': '\xe2\xb7\x8f',
'CjkUnifiedIdeograph4b8f': '\xe4\xae\x8f',
'UnicodeCharacter0x1ff5': '\xe1\xbf\xb5',
'UnicodeCharacter0x1ff1': '\xe1\xbf\xb1',
'UnicodeCharacter0x1ff0': '\xe1\xbf\xb0',
'HangulSyllableDdwels': '\xeb\x9b\xa0',
'UnicodeCharacter0xe98': '\xe0\xba\x98',
'HangulSyllableDdwelp': '\xeb\x9b\xa2',
'HangulSyllableDdwelt': '\xeb\x9b\xa1',
'UnicodeCharacter0xe91': '\xe0\xba\x91',
'UnicodeCharacter0xe90': '\xe0\xba\x90',
'UnicodeCharacter0xe93': '\xe0\xba\x93',
'UnicodeCharacter0xe92': '\xe0\xba\x92',
'CjkUnifiedIdeograph891a': '\xe8\xa4\x9a',
'CjkUnifiedIdeograph891b': '\xe8\xa4\x9b',
'CjkUnifiedIdeograph891c': '\xe8\xa4\x9c',
'HangulSyllableDdwelb': '\xeb\x9b\x9f',
'UnicodeCharacter0xefe4': '\xee\xbf\xa4',
'HangulSyllableDdwelg': '\xeb\x9b\x9d',
'EthiopicSyllableThee': '\xe1\x8c\xa4',
'UnicodeCharacter0xeee0': '\xee\xbb\xa0',
'HangulSyllableDdwelh': '\xeb\x9b\xa3',
'HangulSyllableDdwelm': '\xeb\x9b\x9e',
'UnicodeCharacter0xf183': '\xef\x86\x83',
'UnicodeCharacter0xf182': '\xef\x86\x82',
'UnicodeCharacter0xf181': '\xef\x86\x81',
'CjkUnifiedIdeograph73fd': '\xe7\x8f\xbd',
'HangulSyllableEong': '\xec\x97\x89',
'UnicodeCharacter0xf186': '\xef\x86\x86',
'UnicodeCharacter0xf185': '\xef\x86\x85',
'UnicodeCharacter0xf184': '\xef\x86\x84',
'CjkUnifiedIdeograph5b4c': '\xe5\xad\x8c',
'CjkUnifiedIdeograph5a1c': '\xe5\xa8\x9c',
'CjkUnifiedIdeograph5b4a': '\xe5\xad\x8a',
'HangulSyllableEonh': '\xec\x96\xba',
'CjkUnifiedIdeograph5b4f': '\xe5\xad\x8f',
'CjkUnifiedIdeograph5b4e': '\xe5\xad\x8e',
'CjkUnifiedIdeograph5b4d': '\xe5\xad\x8d',
'CjkUnifiedIdeograph4d24': '\xe4\xb4\xa4',
'CjkUnifiedIdeograph4d25': '\xe4\xb4\xa5',
'CjkUnifiedIdeograph4d26': '\xe4\xb4\xa6',
'CjkUnifiedIdeograph4d27': '\xe4\xb4\xa7',
'CjkUnifiedIdeograph4d20': '\xe4\xb4\xa0',
'CjkUnifiedIdeograph4d21': '\xe4\xb4\xa1',
'CjkUnifiedIdeograph4d22': '\xe4\xb4\xa2',
'CjkUnifiedIdeograph4d23': '\xe4\xb4\xa3',
'CjkUnifiedIdeograph4d28': '\xe4\xb4\xa8',
'CjkUnifiedIdeograph4d29': '\xe4\xb4\xa9',
'ArabicLigatureLamWithAlefFinalForm': '\xef\xbb\xbc',
'LatinSmallLetterJWithCrossedtail': '\xca\x9d',
'UnicodeCharacter0xf12c': '\xef\x84\xac',
'ContainsAsNormalSubgroupOrEqualTo': '\xe2\x8a\xb5',
'CjkUnifiedIdeograph8919': '\xe8\xa4\x99',
'LatinSmallLetterMWithMiddleTilde': '\xe1\xb5\xaf',
'CjkUnifiedIdeograph8914': '\xe8\xa4\x94',
'CjkUnifiedIdeograph8915': '\xe8\xa4\x95',
'CjkUnifiedIdeograph8916': '\xe8\xa4\x96',
'CjkUnifiedIdeograph8917': '\xe8\xa4\x97',
'PlusSignWithTildeBelow': '\xe2\xa8\xa6',
'CjkUnifiedIdeograph8911': '\xe8\xa4\x91',
'CjkUnifiedIdeograph8912': '\xe8\xa4\x92',
'CjkUnifiedIdeograph8913': '\xe8\xa4\x93',
'CjkUnifiedIdeograph4d2d': '\xe4\xb4\xad',
'CjkUnifiedIdeograph4d2e': '\xe4\xb4\xae',
'CjkUnifiedIdeograph4d2f': '\xe4\xb4\xaf',
'UnicodeCharacter0xaaaf': '\xea\xaa\xaf',
'UnicodeCharacter0xaaaa': '\xea\xaa\xaa',
'CjkUnifiedIdeograph4d2a': '\xe4\xb4\xaa',
'CjkUnifiedIdeograph4d2b': '\xe4\xb4\xab',
'CjkUnifiedIdeograph4d2c': '\xe4\xb4\xac',
'HangulSyllableHyulp': '\xed\x9d\x82',
'UnicodeCharacter0xd7f0': '\xed\x9f\xb0',
'HangulSyllableHyuls': '\xed\x9d\x80',
'HangulSyllableHyult': '\xed\x9d\x81',
'HangulSyllableHyulh': '\xed\x9d\x83',
'UnicodeCharacter0xf18b': '\xef\x86\x8b',
'MyanmarLetterDha': '\xe1\x80\x93',
'CjkUnifiedIdeograph5b48': '\xe5\xad\x88',
'HangulSyllableHyulm': '\xed\x9c\xbe',
'UnicodeCharacter0xf18e': '\xef\x86\x8e',
'UnicodeCharacter0xf18d': '\xef\x86\x8d',
'CombiningCedilla': '\xcc\xa7',
'CjkUnifiedIdeograph5b42': '\xe5\xad\x82',
'CjkUnifiedIdeograph5b41': '\xe5\xad\x81',
'CjkUnifiedIdeograph5b40': '\xe5\xad\x80',
'CjkUnifiedIdeograph5b47': '\xe5\xad\x87',
'CjkUnifiedIdeograph5b46': '\xe5\xad\x86',
'CjkUnifiedIdeograph5b45': '\xe5\xad\x85',
'TagbanwaLetterSa': '\xe1\x9d\xb0',
'CjkUnifiedIdeograph3ecc': '\xe3\xbb\x8c',
'CjkUnifiedIdeograph3ecb': '\xe3\xbb\x8b',
'CjkUnifiedIdeograph3eca': '\xe3\xbb\x8a',
'HangulSyllableHyaelb': '\xed\x96\xb7',
'CjkUnifiedIdeograph3ecf': '\xe3\xbb\x8f',
'CjkUnifiedIdeograph3ece': '\xe3\xbb\x8e',
'CjkUnifiedIdeograph3ecd': '\xe3\xbb\x8d',
'GreekPiSymbol': '\xcf\x96',
'KangxiRadicalWine': '\xe2\xbe\xa3',
'KangxiRadicalWind': '\xe2\xbe\xb5',
'UnicodeCharacter0xeee4': '\xee\xbb\xa4',
'CjkUnifiedIdeograph723d': '\xe7\x88\xbd',
'AplFunctionalSymbolQuadDownCaret': '\xe2\x8d\x8c',
'HangulSyllableHweod': '\xed\x9b\xa7',
'HangulSyllableJwess': '\xec\xa5\x88',
'ArabicLigatureThalWithSuperscriptAlefIsolatedForm': '\xef\xb1\x9b',
'VaiSyllableBee': '\xea\x94\x87',
'CjkUnifiedIdeograph443e': '\xe4\x90\xbe',
'UnicodeCharacter0xeee7': '\xee\xbb\xa7',
'UnicodeCharacter0xebed': '\xee\xaf\xad',
'HangulSyllablePyeod': '\xed\x8e\xbb',
'HangulSyllablePyeog': '\xed\x8e\xb5',
'HangulSyllablePyeoc': '\xed\x8f\x8b',
'CjkUnifiedIdeograph443d': '\xe4\x90\xbd',
'HangulSyllablePyeom': '\xed\x8f\x84',
'HangulSyllablePyeol': '\xed\x8e\xbc',
'HangulSyllablePyeon': '\xed\x8e\xb8',
'UnicodeCharacter0xeee6': '\xee\xbb\xa6',
'HangulSyllablePyeoh': '\xed\x8f\x8f',
'HangulSyllableDdagg': '\xeb\x94\xb2',
'HangulSyllablePyeoj': '\xed\x8f\x8a',
'KannadaVowelSignOo': '\xe0\xb3\x8b',
'CjkUnifiedIdeograph3ec2': '\xe3\xbb\x82',
'CjkUnifiedIdeograph3ec1': '\xe3\xbb\x81',
'CjkUnifiedIdeograph3ec0': '\xe3\xbb\x80',
'CjkUnifiedIdeograph3ec7': '\xe3\xbb\x87',
'CjkUnifiedIdeograph3ec6': '\xe3\xbb\x86',
'CjkUnifiedIdeograph3ec5': '\xe3\xbb\x85',
'CjkUnifiedIdeograph3ec4': '\xe3\xbb\x84',
'CjkUnifiedIdeograph3ec9': '\xe3\xbb\x89',
'CjkUnifiedIdeograph3ec8': '\xe3\xbb\x88',
'ArabicLigatureShaddaWithDammaMedialForm': '\xef\xb3\xb3',
'UnicodeCharacter0xa7b6': '\xea\x9e\xb6',
'LatinCapitalLetterTWithCommaBelow': '\xc8\x9a',
'CyrillicCapitalLetterTswe': '\xea\x9a\x8e',
'UnicodeCharacter0xeee9': '\xee\xbb\xa9',
'UnicodeCharacter0xa7b4': '\xea\x9e\xb4',
'YiSyllableGgax': '\xea\x88\x9b',
'UnicodeCharacter0xa7b5': '\xea\x9e\xb5',
'YiSyllableGgap': '\xea\x88\x9d',
'HangulSyllableTwilh': '\xed\x8a\x8f',
'YiSyllableGgat': '\xea\x88\x9a',
'UnicodeCharacter0xa7b3': '\xea\x9e\xb3',
'CjkUnifiedIdeograph4110': '\xe4\x84\x90',
'CjkUnifiedIdeograph4111': '\xe4\x84\x91',
'CjkUnifiedIdeograph4112': '\xe4\x84\x92',
'CjkUnifiedIdeograph4113': '\xe4\x84\x93',
'BalineseVowelSignPepetTedung': '\xe1\xad\x83',
'CjkUnifiedIdeograph4115': '\xe4\x84\x95',
'CjkUnifiedIdeograph4116': '\xe4\x84\x96',
'CjkUnifiedIdeograph4850': '\xe4\xa1\x90',
'CjkUnifiedIdeograph4118': '\xe4\x84\x98',
'CjkUnifiedIdeograph4119': '\xe4\x84\x99',
'VaiSyllableFu': '\xea\x96\xa2',
'UnicodeCharacter0xa7b1': '\xea\x9e\xb1',
'UnicodeCharacter0x17ea': '\xe1\x9f\xaa',
'CjkUnifiedIdeograph392a': '\xe3\xa4\xaa',
'VaiSyllableFo': '\xea\x97\x89',
'HangulSyllableGgyaeg': '\xea\xba\xa1',
'GreekKappaSymbol': '\xcf\xb0',
'AplFunctionalSymbolAlpha': '\xe2\x8d\xba',
'HangulSyllableByis': '\xeb\xb8\xbb',
'VaiSyllableFe': '\xea\x97\xb1',
'HangulSyllableSswegs': '\xec\x92\x9f',
'VaiSyllableFa': '\xea\x95\x98',
'CjkUnifiedIdeograph4854': '\xe4\xa1\x94',
'CjkUnifiedIdeograph75a9': '\xe7\x96\xa9',
'CjkUnifiedIdeograph75a8': '\xe7\x96\xa8',
'CjkUnifiedIdeograph4857': '\xe4\xa1\x97',
'HangulSyllableSsyilp': '\xec\x94\x9a',
'CjkUnifiedIdeograph75a1': '\xe7\x96\xa1',
'CjkUnifiedIdeograph75a0': '\xe7\x96\xa0',
'CjkUnifiedIdeograph75a3': '\xe7\x96\xa3',
'UnicodeCharacter0x17ed': '\xe1\x9f\xad',
'CjkUnifiedIdeograph75a5': '\xe7\x96\xa5',
'CjkUnifiedIdeograph75a4': '\xe7\x96\xa4',
'CjkUnifiedIdeograph75a7': '\xe7\x96\xa7',
'CjkUnifiedIdeograph75a6': '\xe7\x96\xa6',
'HangulSyllableGgyael': '\xea\xba\xa8',
'GreekCapitalLetterEpsilon': '\xce\x95',
'UnicodeCharacter0xa7b8': '\xea\x9e\xb8',
'HangulSyllableGgyaem': '\xea\xba\xb0',
'UnicodeCharacter0xa7b9': '\xea\x9e\xb9',
'CyrillicCapitalLetterKomiTje': '\xd4\x8e',
'ArabicLetterTtehFinalForm': '\xef\xad\xa7',
'Rho': '\xce\xa1',
'WhiteCircleWithDotRight': '\xe2\x9a\x86',
'CjkUnifiedIdeograph411b': '\xe4\x84\x9b',
'CjkUnifiedIdeograph411c': '\xe4\x84\x9c',
'TibetanLetterTsha': '\xe0\xbd\x9a',
'CjkUnifiedIdeograph411e': '\xe4\x84\x9e',
'CjkUnifiedIdeograph411f': '\xe4\x84\x9f',
'CjkUnifiedIdeograph92dc': '\xe9\x8b\x9c',
'CjkUnifiedIdeograph8fc5': '\xe8\xbf\x85',
'HangulSyllablePwaelb': '\xed\x90\xaf',
'CjkUnifiedIdeograph8fc7': '\xe8\xbf\x87',
'HangulSyllableHet': '\xed\x97\xbd',
'CjkUnifiedIdeograph8fc6': '\xe8\xbf\x86',
'CjkUnifiedIdeograph7da8': '\xe7\xb6\xa8',
'CjkUnifiedIdeograph7da9': '\xe7\xb6\xa9',
'UnicodeCharacter0xf7a8': '\xef\x9e\xa8',
'UnicodeCharacter0xf7a9': '\xef\x9e\xa9',
'CjkUnifiedIdeograph75aa': '\xe7\x96\xaa',
'CjkUnifiedIdeograph7da5': '\xe7\xb6\xa5',
'CjkUnifiedIdeograph75ac': '\xe7\x96\xac',
'CjkUnifiedIdeograph75ab': '\xe7\x96\xab',
'CjkUnifiedIdeograph75ae': '\xe7\x96\xae',
'CjkUnifiedIdeograph75ad': '\xe7\x96\xad',
'CjkUnifiedIdeograph7da2': '\xe7\xb6\xa2',
'CjkUnifiedIdeograph75af': '\xe7\x96\xaf',
'HangulSyllableSsyeoss': '\xec\x8e\xb4',
'ArmenianSmallLetterGim': '\xd5\xa3',
'CjkUnifiedIdeograph9739': '\xe9\x9c\xb9',
'CjkUnifiedIdeograph9738': '\xe9\x9c\xb8',
'CjkUnifiedIdeograph9737': '\xe9\x9c\xb7',
'CjkUnifiedIdeograph9736': '\xe9\x9c\xb6',
'CjkUnifiedIdeograph9735': '\xe9\x9c\xb5',
'CjkUnifiedIdeograph750a': '\xe7\x94\x8a',
'CjkUnifiedIdeograph9733': '\xe9\x9c\xb3',
'CjkUnifiedIdeograph9732': '\xe9\x9c\xb2',
'CjkUnifiedIdeograph9731': '\xe9\x9c\xb1',
'CjkUnifiedIdeograph9730': '\xe9\x9c\xb0',
'UnicodeCharacter0xf0aa': '\xef\x82\xaa',
'UnicodeCharacter0xf0ac': '\xef\x82\xac',
'CyrillicCapitalLetterHwe': '\xea\x9a\x94',
'UnicodeCharacter0xf0ae': '\xef\x82\xae',
'UnicodeCharacter0xf0ad': '\xef\x82\xad',
'UnicodeCharacter0xf0af': '\xef\x82\xaf',
'CjkUnifiedIdeograph988a': '\xe9\xa2\x8a',
'CjkUnifiedIdeograph988c': '\xe9\xa2\x8c',
'CjkUnifiedIdeograph988b': '\xe9\xa2\x8b',
'CjkUnifiedIdeograph988e': '\xe9\xa2\x8e',
'HexagramForTheCreativeHeaven': '\xe4\xb7\x80',
'VulgarFractionOneFifth': '\xe2\x85\x95',
'CjkUnifiedIdeograph988f': '\xe9\xa2\x8f',
'ModifierLetterDoubleApostrophe': '\xcb\xae',
'HangulSyllableKwagg': '\xec\xbd\xb2',
'CjkUnifiedIdeograph539a': '\xe5\x8e\x9a',
'ArabicLigatureTehWithNoonFinalForm': '\xef\xb1\xb3',
'CjkUnifiedIdeograph539c': '\xe5\x8e\x9c',
'CjkUnifiedIdeograph539b': '\xe5\x8e\x9b',
'CjkUnifiedIdeograph539e': '\xe5\x8e\x9e',
'CjkUnifiedIdeograph539d': '\xe5\x8e\x9d',
'HangulSyllableGeug': '\xea\xb7\xb9',
'HangulChoseongSiosmieum': '\xe1\x84\xb1',
'CjkUnifiedIdeograph76a8': '\xe7\x9a\xa8',
'CjkUnifiedIdeograph76a9': '\xe7\x9a\xa9',
'CjkUnifiedIdeograph5649': '\xe5\x99\x89',
'CjkUnifiedIdeograph5648': '\xe5\x99\x88',
'CjkUnifiedIdeograph5647': '\xe5\x99\x87',
'CjkUnifiedIdeograph5646': '\xe5\x99\x86',
'CjkUnifiedIdeograph5645': '\xe5\x99\x85',
'LatinSmallLetterRWithCedilla': '\xc5\x97',
'CjkUnifiedIdeograph5643': '\xe5\x99\x83',
'CopticCapitalLetterOldCopticOou': '\xe2\xb2\xbe',
'CjkUnifiedIdeograph5641': '\xe5\x99\x81',
'CjkUnifiedIdeograph5640': '\xe5\x99\x80',
'CjkUnifiedIdeograph9889': '\xe9\xa2\x89',
'CjkUnifiedIdeograph9888': '\xe9\xa2\x88',
'UnicodeCharacter0xf0a3': '\xef\x82\xa3',
'UnicodeCharacter0xf0a2': '\xef\x82\xa2',
'UnicodeCharacter0xf0a5': '\xef\x82\xa5',
'UnicodeCharacter0xf0a4': '\xef\x82\xa4',
'UnicodeCharacter0xf0a7': '\xef\x82\xa7',
'UnicodeCharacter0xf0a6': '\xef\x82\xa6',
'CjkUnifiedIdeograph9881': '\xe9\xa2\x81',
'CjkUnifiedIdeograph9880': '\xe9\xa2\x80',
'CjkUnifiedIdeograph9883': '\xe9\xa2\x83',
'CjkUnifiedIdeograph9882': '\xe9\xa2\x82',
'CjkUnifiedIdeograph9885': '\xe9\xa2\x85',
'CjkUnifiedIdeograph9884': '\xe9\xa2\x84',
'CjkUnifiedIdeograph9887': '\xe9\xa2\x87',
'CjkUnifiedIdeograph9886': '\xe9\xa2\x86',
'UpBarbLeftDownBarbLeftHarpoon': '\xe2\xa5\x91',
'UnicodeCharacter0xf8ee': '\xef\xa3\xae',
'ExtendedArabicindicDigitNine': '\xdb\xb9',
'CjkUnifiedIdeograph973e': '\xe9\x9c\xbe',
'CjkUnifiedIdeograph973d': '\xe9\x9c\xbd',
'CjkUnifiedIdeograph973c': '\xe9\x9c\xbc',
'CjkUnifiedIdeograph973b': '\xe9\x9c\xbb',
'CjkUnifiedIdeograph973a': '\xe9\x9c\xba',
'UnicodeCharacter0xf7d9': '\xef\x9f\x99',
'ArabicLigatureTehWithYehFinalForm': '\xef\xb1\xb5',
'CjkUnifiedIdeograph76ab': '\xe7\x9a\xab',
'CjkUnifiedIdeograph564f': '\xe5\x99\x8f',
'CjkUnifiedIdeograph564e': '\xe5\x99\x8e',
'CjkUnifiedIdeograph564d': '\xe5\x99\x8d',
'CjkUnifiedIdeograph564c': '\xe5\x99\x8c',
'CjkUnifiedIdeograph564b': '\xe5\x99\x8b',
'CjkUnifiedIdeograph564a': '\xe5\x99\x8a',
'CjkUnifiedIdeograph76ae': '\xe7\x9a\xae',
'CjkUnifiedIdeograph5399': '\xe5\x8e\x99',
'CjkUnifiedIdeograph5398': '\xe5\x8e\x98',
'HangulSyllableYo': '\xec\x9a\x94',
'HangulSyllableYi': '\xec\x9d\x98',
'CjkUnifiedIdeograph5391': '\xe5\x8e\x91',
'CjkUnifiedIdeograph5390': '\xe5\x8e\x90',
'CjkUnifiedIdeograph5393': '\xe5\x8e\x93',
'CjkUnifiedIdeograph5392': '\xe5\x8e\x92',
'CjkUnifiedIdeograph5395': '\xe5\x8e\x95',
'CjkUnifiedIdeograph5394': '\xe5\x8e\x94',
'LeftRightArrowWithStroke': '\xe2\x86\xae',
'CjkUnifiedIdeograph5396': '\xe5\x8e\x96',
'UnicodeCharacter0xdb05': '\xed\xac\x85',
'HangulSyllableYek': '\xec\x98\xa0',
'HangulSyllableGgilt': '\xeb\x82\x89',
'KhmerSignSamyokSannya': '\xe1\x9f\x90',
'OriyaLetterVocalicLl': '\xe0\xad\xa1',
'HangulSyllableCibs': '\xec\xb9\xaa',
'CjkUnifiedIdeograph8974': '\xe8\xa5\xb4',
'UnicodeCharacter0xf08c': '\xef\x82\x8c',
'HangulSyllableYen': '\xec\x98\x8c',
'HangulSyllableBwilm': '\xeb\xb7\x9e',
'HangulSyllableYeo': '\xec\x97\xac',
'RightpointingAngleBracket': '\xe2\x8c\xaa',
'CjkUnifiedIdeograph3568': '\xe3\x95\xa8',
'CjkUnifiedIdeograph3569': '\xe3\x95\xa9',
'HangulSyllableByij': '\xeb\xb8\xbe',
'CjkUnifiedIdeograph3564': '\xe3\x95\xa4',
'CjkUnifiedIdeograph3565': '\xe3\x95\xa5',
'CjkUnifiedIdeograph3566': '\xe3\x95\xa6',
'CjkUnifiedIdeograph3567': '\xe3\x95\xa7',
'CjkUnifiedIdeograph3560': '\xe3\x95\xa0',
'CjkUnifiedIdeograph3561': '\xe3\x95\xa1',
'CjkUnifiedIdeograph3562': '\xe3\x95\xa2',
'CjkUnifiedIdeograph3563': '\xe3\x95\xa3',
'ThaiCharacterBoBaimai': '\xe0\xb8\x9a',
'HangulSyllableYeb': '\xec\x98\x99',
'CjkUnifiedIdeograph51f9': '\xe5\x87\xb9',
'HangulSyllableYec': '\xec\x98\x9f',
'LatinSmallLetterDWithHook': '\xc9\x97',
'UnicodeCharacter0xee2b': '\xee\xb8\xab',
'UnicodeCharacter0xee2c': '\xee\xb8\xac',
'UnicodeCharacter0xee2d': '\xee\xb8\xad',
'UnicodeCharacter0xee2e': '\xee\xb8\xae',
'UnicodeCharacter0xee2f': '\xee\xb8\xaf',
'HangulJongseongRieulmieumsios': '\xe1\x87\x92',
'YiSyllableDup': '\xea\x84\x97',
'LatinCapitalLetterNWithLeftHook': '\xc6\x9d',
'LongLeftRightArrow': '\xe2\x9f\xb7',
'squareroot': '\xe2\x88\x9a',
'YiSyllableDux': '\xea\x84\x95',
'UnicodeCharacter0xe143': '\xee\x85\x83',
'GreekCapitalLetterOmicronWithVaria': '\xe1\xbf\xb8',
'YiSyllableDuo': '\xea\x84\x8c',
'VaiSyllableBoo': '\xea\x95\xb8',
'UnicodeCharacter0xee28': '\xee\xb8\xa8',
'HangulSyllableHel': '\xed\x97\xac',
'UnicodeCharacter0xe6e4': '\xee\x9b\xa4',
'UnicodeCharacter0xee20': '\xee\xb8\xa0',
'CjkUnifiedIdeograph51f6': '\xe5\x87\xb6',
'ThaanaLetterKhaa': '\xde\x9a',
'YiSyllableNzi': '\xea\x8b\xaf',
'UnicodeCharacter0xee24': '\xee\xb8\xa4',
'UnicodeCharacter0xee25': '\xee\xb8\xa5',
'UnicodeCharacter0xee26': '\xee\xb8\xa6',
'ThaiCharacterSaraU': '\xe0\xb8\xb8',
'ThaiCharacterSaraI': '\xe0\xb8\xb4',
'ThaiCharacterSaraO': '\xe0\xb9\x82',
'CjkUnifiedIdeograph356d': '\xe3\x95\xad',
'CjkUnifiedIdeograph356e': '\xe3\x95\xae',
'CjkUnifiedIdeograph356f': '\xe3\x95\xaf',
'ThaiCharacterSaraA': '\xe0\xb8\xb0',
'CjkUnifiedIdeograph356a': '\xe3\x95\xaa',
'CjkUnifiedIdeograph356b': '\xe3\x95\xab',
'ThaiCharacterSaraE': '\xe0\xb9\x80',
'ArabicLigatureNoonWithKhahInitialForm': '\xef\xb3\x94',
'CjkUnifiedIdeograph8fce': '\xe8\xbf\x8e',
'BuhidLetterWa': '\xe1\x9d\x8f',
'UnicodeCharacter0xefc6': '\xee\xbf\x86',
'CjkUnifiedIdeograph7507': '\xe7\x94\x87',
'CyrillicCapitalLetterIotifiedYat': '\xea\x99\x92',
'CjkUnifiedIdeograph4932': '\xe4\xa4\xb2',
'CjkUnifiedIdeograph4933': '\xe4\xa4\xb3',
'CjkUnifiedIdeograph4930': '\xe4\xa4\xb0',
'CjkUnifiedIdeograph4931': '\xe4\xa4\xb1',
'CjkUnifiedIdeograph4936': '\xe4\xa4\xb6',
'CjkUnifiedIdeograph4937': '\xe4\xa4\xb7',
'CjkUnifiedIdeograph4934': '\xe4\xa4\xb4',
'CjkUnifiedIdeograph4935': '\xe4\xa4\xb5',
'CjkUnifiedIdeograph8fca': '\xe8\xbf\x8a',
'CjkUnifiedIdeograph4938': '\xe4\xa4\xb8',
'CjkUnifiedIdeograph4939': '\xe4\xa4\xb9',
'HangulSyllableJyilg': '\xec\xa6\xad',
'ModifierLetterSmallEzh': '\xe1\xb6\xbe',
'CjkUnifiedIdeograph8fcc': '\xe8\xbf\x8c',
'HangulSyllableNyugg': '\xeb\x89\xb6',
'CjkUnifiedIdeograph8fcb': '\xe8\xbf\x8b',
'SymbolForFileSeparator': '\xe2\x90\x9c',
'HangulSyllableHyaenj': '\xed\x96\xb1',
'HangulSyllableHyaenh': '\xed\x96\xb2',
'HangulSyllableBwilb': '\xeb\xb7\x9f',
'HangulSyllableYet': '\xec\x98\xa1',
'CjkUnifiedIdeograph8a5a': '\xe8\xa9\x9a',
'CjkUnifiedIdeograph8a5b': '\xe8\xa9\x9b',
'CjkUnifiedIdeograph8a5c': '\xe8\xa9\x9c',
'CjkUnifiedIdeograph8a5d': '\xe8\xa9\x9d',
'CjkUnifiedIdeograph8a5e': '\xe8\xa9\x9e',
'CjkUnifiedIdeograph8a5f': '\xe8\xa9\x9f',
'CjkUnifiedIdeograph3929': '\xe3\xa4\xa9',
'CjkUnifiedIdeograph493b': '\xe4\xa4\xbb',
'CjkUnifiedIdeograph493c': '\xe4\xa4\xbc',
'CjkUnifiedIdeograph493a': '\xe4\xa4\xba',
'CjkUnifiedIdeograph493f': '\xe4\xa4\xbf',
'UnicodeCharacter0xeb58': '\xee\xad\x98',
'CjkUnifiedIdeograph493d': '\xe4\xa4\xbd',
'CjkUnifiedIdeograph493e': '\xe4\xa4\xbe',
'MalayalamLetterChilluNn': '\xe0\xb5\xba',
'UnicodeCharacter0xeb59': '\xee\xad\x99',
'UnicodeCharacter0xeb56': '\xee\xad\x96',
'AplFunctionalSymbolJotUnderbar': '\xe2\x8d\x9b',
'UnicodeCharacter0xeb57': '\xee\xad\x97',
'HangulSyllableJyilm': '\xec\xa6\xae',
'CjkUnifiedIdeograph6b87': '\xe6\xae\x87',
'UnicodeCharacter0xeb54': '\xee\xad\x94',
'CjkUnifiedIdeograph8a58': '\xe8\xa9\x98',
'CircledDigitThree': '\xe2\x91\xa2',
'UnicodeCharacter0xeb55': '\xee\xad\x95',
'CjkUnifiedIdeograph8a50': '\xe8\xa9\x90',
'CjkUnifiedIdeograph8a51': '\xe8\xa9\x91',
'CjkUnifiedIdeograph8a52': '\xe8\xa9\x92',
'CjkUnifiedIdeograph8a53': '\xe8\xa9\x93',
'LatinSmallLetterSWithCedilla': '\xc5\x9f',
'CjkUnifiedIdeograph8a55': '\xe8\xa9\x95',
'CjkUnifiedIdeograph8a56': '\xe8\xa9\x96',
'CjkUnifiedIdeograph8a57': '\xe8\xa9\x97',
'UnicodeCharacter0xeb88': '\xee\xae\x88',
'CombiningClockwiseArrowAbove': '\xe2\x83\x95',
'UnicodeCharacter0xeb50': '\xee\xad\x90',
'UnicodeCharacter0xe851': '\xee\xa1\x91',
'UnicodeCharacter0xeb51': '\xee\xad\x91',
'UpwardsHarpoonWithBarbLeftFromBar': '\xe2\xa5\xa0',
'UnicodeCharacter0xe8e': '\xe0\xba\x8e',
'UnicodeCharacter0xdd17': '\xed\xb4\x97',
'UnicodeCharacter0xdd16': '\xed\xb4\x96',
'CanadianSyllabicsPaa': '\xe1\x90\xb9',
'LaoVowelSignU': '\xe0\xba\xb8',
'ArabicLigatureDadWithKhahInitialForm': '\xef\xb2\xb6',
'UnicodeCharacter0xdd14': '\xed\xb4\x94',
'ArabicPlaceOfSajdah': '\xdb\xa9',
'UnicodeCharacter0xdd13': '\xed\xb4\x93',
'HangulChoseongSsanghieuh': '\xe1\x85\x98',
'HangulSyllableDdweonj': '\xeb\x9a\xbd',
'UnicodeCharacter0xdd12': '\xed\xb4\x92',
'HangulSyllableTyeb': '\xed\x86\x95',
'BoxDrawingsLightDiagonalUpperRightToLowerLeft': '\xe2\x95\xb1',
'UnicodeCharacter0xdd11': '\xed\xb4\x91',
'UnicodeCharacter0xdd10': '\xed\xb4\x90',
'ArabicLetterBehWithThreeDotsPointingUpwardsBelowAndTwoDotsAbove': '\xdd\x93',
'CjkUnifiedIdeograph6b5b': '\xe6\xad\x9b',
'UnicodeCharacter0x18dc': '\xe1\xa3\x9c',
'UnicodeCharacter0x18db': '\xe1\xa3\x9b',
'UnicodeCharacter0x18da': '\xe1\xa3\x9a',
'ZNotationTypeColon': '\xe2\xa6\x82',
'UnicodeCharacter0x18df': '\xe1\xa3\x9f',
'UnicodeCharacter0x18de': '\xe1\xa3\x9e',
'UnicodeCharacter0x18dd': '\xe1\xa3\x9d',
'HangulSyllableDoelt': '\xeb\x90\xa5',
'YiSyllableNret': '\xea\x8e\x9b',
'HangulSyllableDoels': '\xeb\x90\xa4',
'HangulSyllableDoelp': '\xeb\x90\xa6',
'ModifierLetterVoicing': '\xcb\xac',
'SyriacLetterDotlessDalathRish': '\xdc\x96',
'HangulSyllableBwilg': '\xeb\xb7\x9d',
'HangulSyllableTeolg': '\xed\x84\xb9',
'HangulSyllableDoelg': '\xeb\x90\xa1',
'ModifierLetterChineseToneYinShang': '\xea\x9c\x82',
'ArabicFathatanIsolatedForm': '\xef\xb9\xb0',
'HangulSyllableDoelb': '\xeb\x90\xa3',
'UnicodeCharacter0xdd19': '\xed\xb4\x99',
'SaurashtraLetterHa': '\xea\xa2\xb2',
'HangulSyllableTeols': '\xed\x84\xbc',
'HangulSyllableTeolp': '\xed\x84\xbe',
'YiSyllableNza': '\xea\x8b\xb6',
'HangulSyllableDoelh': '\xeb\x90\xa7',
'BoxDrawingsHeavyTripleDashHorizontal': '\xe2\x94\x85',
'HangulSyllableSyaelt': '\xec\x84\x8d',
'UnicodeCharacter0x18d9': '\xe1\xa3\x99',
'UnicodeCharacter0x18d8': '\xe1\xa3\x98',
'HangulSyllableSyaelp': '\xec\x84\x8e',
'HangulSyllableSyaels': '\xec\x84\x8c',
'UnicodeCharacter0x18d3': '\xe1\xa3\x93',
'UnicodeCharacter0x18d2': '\xe1\xa3\x92',
'UnicodeCharacter0x18d1': '\xe1\xa3\x91',
'UnicodeCharacter0x18d0': '\xe1\xa3\x90',
'UnicodeCharacter0x18d7': '\xe1\xa3\x97',
'UnicodeCharacter0x18d6': '\xe1\xa3\x96',
'UnicodeCharacter0x18d5': '\xe1\xa3\x95',
'UnicodeCharacter0x18d4': '\xe1\xa3\x94',
'ParenthesizedKoreanCharacterOHu': '\xe3\x88\x9e',
'HangulSyllableSyaelg': '\xec\x84\x89',
'ArabicLetterRnoonFinalForm': '\xef\xae\xa1',
'HangulSyllableSyaelb': '\xec\x84\x8b',
'GlagoliticCapitalLetterKako': '\xe2\xb0\x8d',
'UnicodeCharacter0xe141': '\xee\x85\x81',
'GreekSmallLetterOmegaWithPsiliAndVaria': '\xe1\xbd\xa2',
'HangulSyllableBbenh': '\xeb\xbb\xb2',
'UnicodeCharacter0xefcb': '\xee\xbf\x8b',
'HangulSyllableNyoss': '\xeb\x87\xbc',
'CjkUnifiedIdeograph34cb': '\xe3\x93\x8b',
'CjkUnifiedIdeograph34cc': '\xe3\x93\x8c',
'CjkUnifiedIdeograph34ca': '\xe3\x93\x8a',
'CjkUnifiedIdeograph34cf': '\xe3\x93\x8f',
'CjkUnifiedIdeograph34cd': '\xe3\x93\x8d',
'CjkUnifiedIdeograph34ce': '\xe3\x93\x8e',
'CjkUnifiedIdeograph96f9': '\xe9\x9b\xb9',
'CjkUnifiedIdeograph96f8': '\xe9\x9b\xb8',
'RotatedHeavyBlackHeartBullet': '\xe2\x9d\xa5',
'CjkUnifiedIdeograph96f5': '\xe9\x9b\xb5',
'CjkUnifiedIdeograph96f4': '\xe9\x9b\xb4',
'CjkUnifiedIdeograph96f7': '\xe9\x9b\xb7',
'CjkUnifiedIdeograph96f6': '\xe9\x9b\xb6',
'CjkUnifiedIdeograph96f1': '\xe9\x9b\xb1',
'CjkUnifiedIdeograph96f0': '\xe9\x9b\xb0',
'UnicodeCharacter0x321f': '\xe3\x88\x9f',
'CjkUnifiedIdeograph96f2': '\xe9\x9b\xb2',
'un': '\xe2\x88\xaa',
'ArabicLigatureYehWithHamzaAboveWithZainFinalForm': '\xef\xb1\xa5',
'CopticSmallLetterFi': '\xe2\xb2\xab',
'ModifierLetterDotSlash': '\xea\x9c\x98',
'CanadianSyllabicsBiblecreeY': '\xe1\x94\xbf',
'oint': '\xe2\x88\xae',
'KhmerDigitNine': '\xe1\x9f\xa9',
'IdeographicTelegraphSymbolForJune': '\xe3\x8b\x85',
'CjkUnifiedIdeograph34c8': '\xe3\x93\x88',
'CjkUnifiedIdeograph34c9': '\xe3\x93\x89',
'CjkUnifiedIdeograph34c2': '\xe3\x93\x82',
'CjkUnifiedIdeograph34c3': '\xe3\x93\x83',
'CjkUnifiedIdeograph34c0': '\xe3\x93\x80',
'CjkUnifiedIdeograph34c1': '\xe3\x93\x81',
'CjkUnifiedIdeograph34c6': '\xe3\x93\x86',
'CjkUnifiedIdeograph34c7': '\xe3\x93\x87',
'NotCheckMark': '\xe2\x8d\xbb',
'CjkUnifiedIdeograph34c5': '\xe3\x93\x85',
'UnicodeCharacter0xdb0f': '\xed\xac\x8f',
'ArabicLetterNoon': '\xd9\x86',
'RomanNumeralReversedOneHundred': '\xe2\x86\x83',
'GlagoliticCapitalLetterDobro': '\xe2\xb0\x84',
'ArabicLigatureJeemWithMeemInitialForm': '\xef\xb2\xa8',
'UnicodeCharacter0xdb0d': '\xed\xac\x8d',
'HangulSyllableNinh': '\xeb\x8b\x8e',
'HangulSyllableNinj': '\xeb\x8b\x8d',
'FullwidthPlusSign': '\xef\xbc\x8b',
'CjkUnifiedIdeograph897f': '\xe8\xa5\xbf',
'HangulSyllableNing': '\xeb\x8b\x9d',
'HangulSyllableGgwat': '\xea\xbd\xa1',
'BlackfeatheredSouthEastArrow': '\xe2\x9e\xb4',
'CjkUnifiedIdeograph96fe': '\xe9\x9b\xbe',
'CjkUnifiedIdeograph96fd': '\xe9\x9b\xbd',
'CjkUnifiedIdeograph96ff': '\xe9\x9b\xbf',
'CjkUnifiedIdeograph96fa': '\xe9\x9b\xba',
'CjkUnifiedIdeograph96fc': '\xe9\x9b\xbc',
'CjkUnifiedIdeograph96fb': '\xe9\x9b\xbb',
'CombiningAsteriskBelow': '\xcd\x99',
'HangulSyllableCwelg': '\xec\xb7\x95',
'UnicodeCharacter0xebe4': '\xee\xaf\xa4',
'CjkUnifiedIdeograph626d': '\xe6\x89\xad',
'CjkUnifiedIdeograph626e': '\xe6\x89\xae',
'CjkUnifiedIdeograph626f': '\xe6\x89\xaf',
'CjkUnifiedIdeograph626a': '\xe6\x89\xaa',
'CjkUnifiedIdeograph626b': '\xe6\x89\xab',
'CjkUnifiedIdeograph626c': '\xe6\x89\xac',
'HangulSyllableGgwan': '\xea\xbd\x8c',
'UnicodeCharacter0x9fbc': '\xe9\xbe\xbc',
'UnicodeCharacter0xebe5': '\xee\xaf\xa5',
'UnicodeCharacter0x9fbf': '\xe9\xbe\xbf',
'UnicodeCharacter0x9fbe': '\xe9\xbe\xbe',
'UnicodeCharacter0x9fbd': '\xe9\xbe\xbd',
'HangulSyllableMelh': '\xeb\xa9\xa3',
'HangulSyllableMelm': '\xeb\xa9\x9e',
'HangulSyllableJaeh': '\xec\x9f\x87',
'HangulSyllableMelb': '\xeb\xa9\x9f',
'HangulSyllableMelg': '\xeb\xa9\x9d',
'ArabicLetterBehWithTwoDotsBelowAndDotAbove': '\xdd\x94',
'HangulSyllableMels': '\xeb\xa9\xa0',
'UnicodeCharacter0xebe2': '\xee\xaf\xa2',
'HangulSyllableMelp': '\xeb\xa9\xa2',
'TopRightCorner': '\xe2\x8c\x9d',
'SylotiNagriLetterLo': '\xea\xa0\x9f',
'HangulSyllableJegg': '\xec\xa0\x9e',
'HangulSyllableNag': '\xeb\x82\x99',
'HangulSyllableRyigs': '\xeb\xa6\x93',
'HangulSyllableJaen': '\xec\x9e\xb0',
'SquarePensu': '\xe3\x8c\xba',
'CjkUnifiedIdeograph6264': '\xe6\x89\xa4',
'CjkUnifiedIdeograph6265': '\xe6\x89\xa5',
'CjkUnifiedIdeograph6266': '\xe6\x89\xa6',
'HangulLetterWae': '\xe3\x85\x99',
'CjkUnifiedIdeograph6260': '\xe6\x89\xa0',
'CjkUnifiedIdeograph6261': '\xe6\x89\xa1',
'CjkUnifiedIdeograph6262': '\xe6\x89\xa2',
'BoxDrawingsUpHeavyAndDownHorizontalLight': '\xe2\x95\x80',
'HangulSyllableGgwac': '\xea\xbd\x9f',
'CjkUnifiedIdeograph6268': '\xe6\x89\xa8',
'CjkUnifiedIdeograph6269': '\xe6\x89\xa9',
'HangulSyllableSsogg': '\xec\x8f\x9a',
'HangulSyllableKyuls': '\xed\x81\x9c',
'HangulSyllablePyaelg': '\xed\x8d\xa9',
'HangulSyllablePyaelb': '\xed\x8d\xab',
'HangulSyllablePyaelm': '\xed\x8d\xaa',
'HangulSyllablePyaelh': '\xed\x8d\xaf',
'TurnedCapitalF': '\xe2\x84\xb2',
'CjkUnifiedIdeograph7efd': '\xe7\xbb\xbd',
'HangulSyllablePyaelt': '\xed\x8d\xad',
'GreekRhoWithStrokeSymbol': '\xcf\xbc',
'HangulSyllablePyaelp': '\xed\x8d\xae',
'HangulSyllableNah': '\xeb\x82\xb3',
'HangulSyllablePyaels': '\xed\x8d\xac',
'WedgetailedRightwardsArrow': '\xe2\x9e\xbc',
'CjkUnifiedIdeograph681f': '\xe6\xa0\x9f',
'RegisteredSign': '\xc2\xae',
'UnicodeCharacter0xf826': '\xef\xa0\xa6',
'HangulSyllablePin': '\xed\x95\x80',
'ArabicLigatureTehWithMeemIsolatedForm': '\xef\xb0\x8e',
'HangulSyllableKyult': '\xed\x81\x9d',
'BlackVerticalRectangle': '\xe2\x96\xae',
'TaiLeLetterQa': '\xe1\xa5\x9f',
'YiSyllableYyr': '\xea\x92\x8c',
'ArabicLetterBehehInitialForm': '\xef\xad\x9c',
'YiSyllableYyt': '\xea\x92\x87',
'ArabicLetterWawIsolatedForm': '\xef\xbb\xad',
'YiSyllableYyx': '\xea\x92\x88',
'GreekTonos': '\xce\x84',
'crho': '\xcf\xb1',
'HangulSyllableDdaenj': '\xeb\x95\x91',
'I': '\xce\x99',
'HangulSyllableGwags': '\xea\xb3\xbf',
'HangulSyllableBwilt': '\xeb\xb7\xa1',
'TwoIntersectingLogicalAnd': '\xe2\xa9\x95',
'UnicodeCharacter0xe85a': '\xee\xa1\x9a',
'PlanckConstant': '\xe2\x84\x8e',
'HangulSyllableGwagg': '\xea\xb3\xbe',
'UnicodeCharacter0xf252': '\xef\x89\x92',
'UnicodeCharacter0xdd1f': '\xed\xb4\x9f',
'BoxDrawingsLightLeftAndHeavyRight': '\xe2\x95\xbc',
'UnicodeCharacter0xe85d': '\xee\xa1\x9d',
'HangulSyllableJjult': '\xec\xad\x95',
'HangulSyllableJjuls': '\xec\xad\x94',
'UnicodeCharacter0xdd1e': '\xed\xb4\x9e',
'HangulSyllableJjulp': '\xec\xad\x96',
'HangulSyllableJjulm': '\xec\xad\x92',
'UnicodeCharacter0xdd1d': '\xed\xb4\x9d',
'HangulSyllableJjulh': '\xec\xad\x97',
'HangulSyllableJjulg': '\xec\xad\x91',
'UnicodeCharacter0xdd1c': '\xed\xb4\x9c',
'UnicodeCharacter0x1bcb': '\xe1\xaf\x8b',
'HangulSyllableJjulb': '\xec\xad\x93',
'UnicodeCharacter0xdd1b': '\xed\xb4\x9b',
'HangulSyllableNeoc': '\xeb\x84\x9f',
'HangulSyllableNeob': '\xeb\x84\x99',
'HangulSyllableKyeng': '\xec\xbd\x8d',
'LatinSmallLetterYWithLoop': '\xe1\xbb\xbf',
'HangulSyllableNeog': '\xeb\x84\x89',
'ThaiCharacterSaraAm': '\xe0\xb8\xb3',
'UnicodeCharacter0xf8e4': '\xef\xa3\xa4',
'SquareFoot': '\xe2\x8f\x8d',
'HangulSyllableNeok': '\xeb\x84\xa0',
'ThaiCharacterSaraAa': '\xe0\xb8\xb2',
'HangulSyllableNeoh': '\xeb\x84\xa3',
'ThaiCharacterSaraAe': '\xe0\xb9\x81',
'HangulSyllableNeom': '\xeb\x84\x98',
'HangulSyllableNeol': '\xeb\x84\x90',
'CjkUnifiedIdeograph4bbb': '\xe4\xae\xbb',
'CjkUnifiedIdeograph4bbc': '\xe4\xae\xbc',
'CjkUnifiedIdeograph4bba': '\xe4\xae\xba',
'CjkUnifiedIdeograph4bbf': '\xe4\xae\xbf',
'CjkUnifiedIdeograph4bbd': '\xe4\xae\xbd',
'CjkUnifiedIdeograph4bbe': '\xe4\xae\xbe',
'ArabicSmallHighMadda': '\xdb\xa4',
'HangulSyllableYoss': '\xec\x9a\xa8',
'LatinCapitalLetterYWithMacron': '\xc8\xb2',
'Eta': '\xce\x97',
'MongolianLetterTsa': '\xe1\xa0\xbc',
'UnicodeCharacter0x1bcd': '\xe1\xaf\x8d',
'CjkUnifiedIdeograph78d8': '\xe7\xa3\x98',
'CjkUnifiedIdeograph78d7': '\xe7\xa3\x97',
'CjkUnifiedIdeograph78d6': '\xe7\xa3\x96',
'CjkUnifiedIdeograph78d5': '\xe7\xa3\x95',
'CjkUnifiedIdeograph78d4': '\xe7\xa3\x94',
'CjkUnifiedIdeograph78d3': '\xe7\xa3\x93',
'CjkUnifiedIdeograph78d2': '\xe7\xa3\x92',
'CjkUnifiedIdeograph78d1': '\xe7\xa3\x91',
'CjkUnifiedIdeograph78d0': '\xe7\xa3\x90',
'CjkUnifiedIdeograph4bb2': '\xe4\xae\xb2',
'CjkUnifiedIdeograph4bb3': '\xe4\xae\xb3',
'CjkUnifiedIdeograph4bb0': '\xe4\xae\xb0',
'CjkUnifiedIdeograph4bb1': '\xe4\xae\xb1',
'CjkUnifiedIdeograph4bb6': '\xe4\xae\xb6',
'CjkUnifiedIdeograph4bb7': '\xe4\xae\xb7',
'CjkUnifiedIdeograph4bb4': '\xe4\xae\xb4',
'CjkUnifiedIdeograph4bb5': '\xe4\xae\xb5',
'CjkUnifiedIdeograph4bb8': '\xe4\xae\xb8',
'CjkUnifiedIdeograph4bb9': '\xe4\xae\xb9',
'HangulSyllableSyuls': '\xec\x8a\x94',
'YiSyllableZzep': '\xea\x8b\xa1',
'UnicodeCharacter0x1bce': '\xe1\xaf\x8e',
'KayahLiLetterZa': '\xea\xa4\x90',
'OcrFork': '\xe2\x91\x82',
'BengaliLetterIi': '\xe0\xa6\x88',
'HangulSyllableDdwan': '\xeb\x98\xb0',
'GreekCapitalLetterZeta': '\xce\x96',
'GreekLetterSmallCapitalGamma': '\xe1\xb4\xa6',
'CjkUnifiedIdeograph78df': '\xe7\xa3\x9f',
'CjkUnifiedIdeograph78de': '\xe7\xa3\x9e',
'CjkUnifiedIdeograph78dd': '\xe7\xa3\x9d',
'CjkUnifiedIdeograph78dc': '\xe7\xa3\x9c',
'CjkUnifiedIdeograph78db': '\xe7\xa3\x9b',
'CjkUnifiedIdeograph78da': '\xe7\xa3\x9a',
'SyriacRwaha': '\xdc\xbf',
'EthiopicSyllableFya': '\xe1\x8d\x9a',
'HangulSyllableDdwak': '\xeb\x99\x84',
'CjkUnifiedIdeograph64c1': '\xe6\x93\x81',
'CjkUnifiedIdeograph64c0': '\xe6\x93\x80',
'CjkUnifiedIdeograph64c3': '\xe6\x93\x83',
'CjkUnifiedIdeograph64c2': '\xe6\x93\x82',
'CjkUnifiedIdeograph64c5': '\xe6\x93\x85',
'CjkUnifiedIdeograph64c4': '\xe6\x93\x84',
'CjkUnifiedIdeograph64c7': '\xe6\x93\x87',
'CjkUnifiedIdeograph64c6': '\xe6\x93\x86',
'CjkUnifiedIdeograph64c9': '\xe6\x93\x89',
'CjkUnifiedIdeograph64c8': '\xe6\x93\x88',
'HangulSyllableBan': '\xeb\xb0\x98',
'CjkUnifiedIdeograph9cff': '\xe9\xb3\xbf',
'HangulSyllableBak': '\xeb\xb0\xac',
'HangulSyllableBaj': '\xeb\xb0\xaa',
'HangulSyllableBat': '\xeb\xb0\xad',
'HalfwidthHangulLetterYu': '\xef\xbf\x97',
'HangulSyllableBap': '\xeb\xb0\xae',
'HangulSyllableBas': '\xeb\xb0\xa7',
'HangulJongseongHieuhpieup': '\xe1\x87\xb8',
'NewTaiLueLetterLowTsa': '\xe1\xa6\x8b',
'CyrillicCapitalLetterShha': '\xd2\xba',
'HangulSyllableDdwigs': '\xeb\x9b\xb3',
'HangulSyllableDdwab': '\xeb\x98\xbd',
'NaryProduct': '\xe2\x88\x8f',
'OlChikiPhaarkaa': '\xe1\xb1\xbc',
'CyrillicCapitalLetterIotifiedClosedLittleYus': '\xea\x99\x9c',
'CjkUnifiedIdeograph8fc1': '\xe8\xbf\x81',
'CjkUnifiedIdeograph8757': '\xe8\x9d\x97',
'LatinCapitalLetterUWithCaron': '\xc7\x93',
'HangulSyllableBwilh': '\xeb\xb7\xa3',
'HangulSyllableTyolb': '\xed\x88\x9b',
'ModifierLetterPrime': '\xca\xb9',
'CjkUnifiedIdeograph64ca': '\xe6\x93\x8a',
'HangulSyllablePil': '\xed\x95\x84',
'CjkUnifiedIdeograph64cc': '\xe6\x93\x8c',
'CjkUnifiedIdeograph64cb': '\xe6\x93\x8b',
'CjkUnifiedIdeograph64ce': '\xe6\x93\x8e',
'CjkUnifiedIdeograph64cd': '\xe6\x93\x8d',
'CjkUnifiedIdeograph64cf': '\xe6\x93\x8f',
'EthiopicDigitNine': '\xe1\x8d\xb1',
'UnicodeCharacter0xd8d6': '\xed\xa3\x96',
'UnicodeCharacter0xd8d7': '\xed\xa3\x97',
'HangulSyllableByag': '\xeb\xb1\x8d',
'UnicodeCharacter0xd8d8': '\xed\xa3\x98',
'HangulSyllableNebs': '\xeb\x84\xb6',
'HangulSyllableGilp': '\xea\xb8\xbe',
'HangulSyllableGils': '\xea\xb8\xbc',
'CjkUnifiedIdeograph7ce4': '\xe7\xb3\xa4',
'HangulSyllableByac': '\xeb\xb1\xa3',
'HangulSyllableNweolh': '\xeb\x88\xaf',
'HangulSyllableGilt': '\xea\xb8\xbd',
'CjkUnifiedIdeograph63c2': '\xe6\x8f\x82',
'CjkUnifiedIdeograph63c3': '\xe6\x8f\x83',
'CjkUnifiedIdeograph63c0': '\xe6\x8f\x80',
'CjkUnifiedIdeograph63c1': '\xe6\x8f\x81',
'CjkUnifiedIdeograph63c6': '\xe6\x8f\x86',
'CjkUnifiedIdeograph63c7': '\xe6\x8f\x87',
'CjkUnifiedIdeograph63c4': '\xe6\x8f\x84',
'CjkUnifiedIdeograph63c5': '\xe6\x8f\x85',
'HangulSyllableTyeols': '\xed\x85\xb4',
'CjkUnifiedIdeograph63c8': '\xe6\x8f\x88',
'CjkUnifiedIdeograph63c9': '\xe6\x8f\x89',
'UnicodeCharacter0xbe2': '\xe0\xaf\xa2',
'UnicodeCharacter0xbe3': '\xe0\xaf\xa3',
'UnicodeCharacter0xbe0': '\xe0\xaf\xa0',
'UnicodeCharacter0xbe1': '\xe0\xaf\xa1',
'UnicodeCharacter0xd95f': '\xed\xa5\x9f',
'UnicodeCharacter0xbe4': '\xe0\xaf\xa4',
'UnicodeCharacter0xbe5': '\xe0\xaf\xa5',
'VaiSyllableKpu': '\xea\x96\x9f',
'UnicodeCharacter0xd7d8': '\xed\x9f\x98',
'ArabicSmallHighMeemIsolatedForm': '\xdb\xa2',
'UnicodeCharacter0xd7d7': '\xed\x9f\x97',
'VaiSyllableKpo': '\xea\x97\x85',
'CjkUnifiedIdeograph6d88': '\xe6\xb6\x88',
'CjkUnifiedIdeograph6d89': '\xe6\xb6\x89',
'YiSyllableNzurx': '\xea\x8c\x81',
'UnicodeCharacter0xd9f2': '\xed\xa7\xb2',
'CjkUnifiedIdeograph6d84': '\xe6\xb6\x84',
'CjkUnifiedIdeograph6d85': '\xe6\xb6\x85',
'CjkUnifiedIdeograph6d86': '\xe6\xb6\x86',
'CjkUnifiedIdeograph6d87': '\xe6\xb6\x87',
'CjkUnifiedIdeograph6d80': '\xe6\xb6\x80',
'CjkUnifiedIdeograph6d81': '\xe6\xb6\x81',
'HangulLetterYesieung': '\xe3\x86\x81',
'CjkUnifiedIdeograph6d83': '\xe6\xb6\x83',
'CjkUnifiedIdeograph63cb': '\xe6\x8f\x8b',
'CjkUnifiedIdeograph63cc': '\xe6\x8f\x8c',
'CjkUnifiedIdeograph63ca': '\xe6\x8f\x8a',
'CjkUnifiedIdeograph63cf': '\xe6\x8f\x8f',
'CjkUnifiedIdeograph63cd': '\xe6\x8f\x8d',
'CjkUnifiedIdeograph63ce': '\xe6\x8f\x8e',
'VerticalMaleWithStrokeSign': '\xe2\x9a\xa8',
'UnicodeCharacter0xd7d2': '\xed\x9f\x92',
'UpArrowhead': '\xe2\x8c\x83',
'HangulSyllableCoenj': '\xec\xb5\xa1',
'HangulSyllableJjweoh': '\xec\xad\xbf',
'ArabicLetterWawFinalForm': '\xef\xbb\xae',
'CjkUnifiedIdeograph7efe': '\xe7\xbb\xbe',
'GreekSmallLetterOmega': '\xcf\x89',
'GreekSmallLetterPhi': '\xcf\x86',
'LatinSmallLetterNPrecededByApostrophe': '\xc5\x89',
'UnicodeCharacter0xe8f8': '\xee\xa3\xb8',
'CjkUnifiedIdeograph6d8d': '\xe6\xb6\x8d',
'CjkUnifiedIdeograph6d8e': '\xe6\xb6\x8e',
'CjkUnifiedIdeograph6d8f': '\xe6\xb6\x8f',
'CjkUnifiedIdeograph6d8a': '\xe6\xb6\x8a',
'CjkUnifiedIdeograph6d8b': '\xe6\xb6\x8b',
'CjkUnifiedIdeograph6d8c': '\xe6\xb6\x8c',
'UnicodeCharacter0xd952': '\xed\xa5\x92',
'HangulSyllableCwels': '\xec\xb7\x98',
'UnicodeCharacter0xd950': '\xed\xa5\x90',
'UnicodeCharacter0xd951': '\xed\xa5\x91',
'UnicodeCharacter0xd956': '\xed\xa5\x96',
'UnicodeCharacter0xd957': '\xed\xa5\x97',
'UnicodeCharacter0xd954': '\xed\xa5\x94',
'UnicodeCharacter0xd955': '\xed\xa5\x95',
'CjkRadicalSimplifiedYellow': '\xe2\xbb\xa9',
'UnicodeCharacter0xd958': '\xed\xa5\x98',
'UnicodeCharacter0xd959': '\xed\xa5\x99',
'CjkUnifiedIdeograph835e': '\xe8\x8d\x9e',
'ArabicLetterRehIsolatedForm': '\xef\xba\xad',
'HangulSyllableBbwed': '\xec\x80\x8b',
'CircledNumberFortyFive': '\xe3\x8a\xba',
'CjkUnifiedIdeograph6466': '\xe6\x91\xa6',
'CjkUnifiedIdeograph835a': '\xe8\x8d\x9a',
'CjkUnifiedIdeograph6467': '\xe6\x91\xa7',
'CircleWithTwoHorizontalStrokesToTheRight': '\xe2\xa7\x83',
'HangulSyllableHyinj': '\xed\x9d\xb1',
'HangulSyllableHyinh': '\xed\x9d\xb2',
'HangulSyllableHying': '\xed\x9e\x81',
'EthiopicNumberSeventy': '\xe1\x8d\xb8',
'LeftwardsArrowAboveTildeOperator': '\xe2\xa5\xb3',
'CircledKatakanaTe': '\xe3\x8b\xa2',
'CircledKatakanaTa': '\xe3\x8b\x9f',
'CircledKatakanaTo': '\xe3\x8b\xa3',
'ThaanaLetterVaavu': '\xde\x88',
'UnicodeCharacter0xdd27': '\xed\xb4\xa7',
'CircledKatakanaTi': '\xe3\x8b\xa0',
'CircledKatakanaTu': '\xe3\x8b\xa1',
'HangulSyllableCyang': '\xec\xb1\xb5',
'EthiopicSyllableMee': '\xe1\x88\x9c',
'HangulSyllableCyanj': '\xec\xb1\xa5',
'CanadianSyllabicsSayisiTh': '\xe1\x92\xa2',
'cph': '\xcf\x86',
'cpi': '\xcf\x96',
'OlChikiLetterLu': '\xe1\xb1\xa9',
'HangulSyllableBbyegg': '\xeb\xbc\xa6',
'HangulSyllableNiss': '\xeb\x8b\x9c',
'UnicodeCharacter0xfa2e': '\xef\xa8\xae',
'ArabicLigatureLamWithYehIsolatedForm': '\xef\xb1\x84',
'HangulSyllableBbwek': '\xec\x80\x9c',
'HangulSyllableTyolh': '\xed\x88\x9f',
'HangulSyllableBbyegs': '\xeb\xbc\xa7',
'UnicodeCharacter0xf822': '\xef\xa0\xa2',
'HiraganaLetterRu': '\xe3\x82\x8b',
'UnicodeCharacter0xe339': '\xee\x8c\xb9',
'UnicodeCharacter0xe338': '\xee\x8c\xb8',
'MicroSign': '\xc2\xb5',
'UnicodeCharacter0xe335': '\xee\x8c\xb5',
'UnicodeCharacter0xe334': '\xee\x8c\xb4',
'ReversedForkedParagraphos': '\xe2\xb8\x91',
'UnicodeCharacter0xe336': '\xee\x8c\xb6',
'UnicodeCharacter0xe331': '\xee\x8c\xb1',
'UnicodeCharacter0xe330': '\xee\x8c\xb0',
'UnicodeCharacter0xe333': '\xee\x8c\xb3',
'UnicodeCharacter0xe332': '\xee\x8c\xb2',
'CjkUnifiedIdeograph889a': '\xe8\xa2\x9a',
'UnicodeCharacter0xa79f': '\xea\x9e\x9f',
'HiraganaLetterRe': '\xe3\x82\x8c',
'CjkUnifiedIdeograph889b': '\xe8\xa2\x9b',
'CjkUnifiedIdeograph889e': '\xe8\xa2\x9e',
'CjkUnifiedIdeograph889d': '\xe8\xa2\x9d',
'HiraganaLetterRa': '\xe3\x82\x89',
'CjkUnifiedIdeograph889f': '\xe8\xa2\x9f',
'HiraganaLetterRo': '\xe3\x82\x8d',
'CjkUnifiedIdeograph586f': '\xe5\xa1\xaf',
'CjkUnifiedIdeograph586e': '\xe5\xa1\xae',
'CjkUnifiedIdeograph586d': '\xe5\xa1\xad',
'CjkUnifiedIdeograph586c': '\xe5\xa1\xac',
'CjkUnifiedIdeograph586b': '\xe5\xa1\xab',
'HiraganaLetterRi': '\xe3\x82\x8a',
'HangulSyllableBbwinj': '\xec\x80\xa5',
'CjkUnifiedIdeograph5b14': '\xe5\xac\x94',
'CjkUnifiedIdeograph9c9e': '\xe9\xb2\x9e',
'CjkUnifiedIdeograph9c9d': '\xe9\xb2\x9d',
'CjkUnifiedIdeograph9c9f': '\xe9\xb2\x9f',
'CjkUnifiedIdeograph9c9a': '\xe9\xb2\x9a',
'CanadianSyllabicsYcreeOo': '\xe1\x90\x87',
'CjkUnifiedIdeograph9c9c': '\xe9\xb2\x9c',
'CjkUnifiedIdeograph9c9b': '\xe9\xb2\x9b',
'UnicodeCharacter0xdea2': '\xed\xba\xa2',
'UnicodeCharacter0xdea3': '\xed\xba\xa3',
'UnicodeCharacter0xdea0': '\xed\xba\xa0',
'UnicodeCharacter0xdea1': '\xed\xba\xa1',
'AplFunctionalSymbolQuadJot': '\xe2\x8c\xbb',
'UnicodeCharacter0xdea7': '\xed\xba\xa7',
'UnicodeCharacter0xdea4': '\xed\xba\xa4',
'CjkUnifiedIdeograph5624': '\xe5\x98\xa4',
'UnicodeCharacter0xdea8': '\xed\xba\xa8',
'UnicodeCharacter0xdea9': '\xed\xba\xa9',
'LatinCapitalLetterZWithDotBelow': '\xe1\xba\x92',
'CjkUnifiedIdeograph8891': '\xe8\xa2\x91',
'CjkUnifiedIdeograph8890': '\xe8\xa2\x90',
'CjkUnifiedIdeograph8893': '\xe8\xa2\x93',
'CjkUnifiedIdeograph8892': '\xe8\xa2\x92',
'CjkUnifiedIdeograph8895': '\xe8\xa2\x95',
'CjkUnifiedIdeograph8894': '\xe8\xa2\x94',
'CjkUnifiedIdeograph5869': '\xe5\xa1\xa9',
'CjkUnifiedIdeograph5868': '\xe5\xa1\xa8',
'CjkUnifiedIdeograph5867': '\xe5\xa1\xa7',
'CjkUnifiedIdeograph5866': '\xe5\xa1\xa6',
'CjkUnifiedIdeograph5865': '\xe5\xa1\xa5',
'ArabicLetterGafWithRing': '\xda\xb0',
'CjkUnifiedIdeograph5863': '\xe5\xa1\xa3',
'CjkUnifiedIdeograph5862': '\xe5\xa1\xa2',
'CjkUnifiedIdeograph5861': '\xe5\xa1\xa1',
'Pitchfork': '\xe2\x8b\x94',
'HangulSyllableByalb': '\xeb\xb1\x97',
'CanadianSyllabicsTthe': '\xe1\x95\xab',
'UnicodeCharacter0xf448': '\xef\x91\x88',
'LimbuLetterRa': '\xe1\xa4\x96',
'HangulSyllableTweoc': '\xed\x89\x9f',
'CanadianSyllabicsTtha': '\xe1\x95\xae',
'Vesta': '\xe2\x9a\xb6',
'UnicodeCharacter0xf32c': '\xef\x8c\xac',
'UnicodeCharacter0xe33e': '\xee\x8c\xbe',
'HangulSyllableDeonh': '\xeb\x8d\x9a',
'HangulSyllableByalh': '\xeb\xb1\x9b',
'CanadianSyllabicsTtho': '\xe1\x95\xad',
'HangulSyllableRegs': '\xeb\xa0\x8b',
'CanadianSyllabicsTthi': '\xe1\x95\xac',
'UnicodeCharacter0xe33c': '\xee\x8c\xbc',
'HangulSyllableByalm': '\xeb\xb1\x96',
'UnicodeCharacter0xdeab': '\xed\xba\xab',
'BugineseVowelSignO': '\xe1\xa8\x9a',
'UnicodeCharacter0xdeaa': '\xed\xba\xaa',
'UnicodeCharacter0xdeaf': '\xed\xba\xaf',
'UnicodeCharacter0xd815': '\xed\xa0\x95',
'UnicodeCharacter0xdead': '\xed\xba\xad',
'BugineseVowelSignI': '\xe1\xa8\x97',
'UnicodeCharacter0xa82e': '\xea\xa0\xae',
'CjkUnifiedIdeograph5b18': '\xe5\xac\x98',
'CjkUnifiedIdeograph8f1d': '\xe8\xbc\x9d',
'BugineseVowelSignE': '\xe1\xa8\x99',
'UnicodeCharacter0xa82c': '\xea\xa0\xac',
'ArabicDamma': '\xd9\x8f',
'CjkUnifiedIdeograph9c99': '\xe9\xb2\x99',
'CjkUnifiedIdeograph9c98': '\xe9\xb2\x98',
'CjkUnifiedIdeograph8fc2': '\xe8\xbf\x82',
'HangulSyllableRyael': '\xeb\x9f\x98',
'PrintScreenSymbol': '\xe2\x8e\x99',
'IdeographicTelegraphSymbolForDaySixteen': '\xe3\x8f\xaf',
'CjkUnifiedIdeograph9c97': '\xe9\xb2\x97',
'CyrillicCapitalLetterIotifiedE': '\xd1\xa4',
'CjkUnifiedIdeograph9c91': '\xe9\xb2\x91',
'CjkUnifiedIdeograph9c90': '\xe9\xb2\x90',
'CjkUnifiedIdeograph9c93': '\xe9\xb2\x93',
'CjkUnifiedIdeograph9c92': '\xe9\xb2\x92',
'ArabicLigatureSadWithHahWithHahInitialForm': '\xef\xb5\xa5',
'CjkUnifiedIdeograph8769': '\xe8\x9d\xa9',
'CjkUnifiedIdeograph8768': '\xe8\x9d\xa8',
'HangulSyllableDoel': '\xeb\x90\xa0',
'CjkUnifiedIdeograph8763': '\xe8\x9d\xa3',
'CjkUnifiedIdeograph8762': '\xe8\x9d\xa2',
'CjkUnifiedIdeograph8761': '\xe8\x9d\xa1',
'CjkUnifiedIdeograph8760': '\xe8\x9d\xa0',
'CjkUnifiedIdeograph8767': '\xe8\x9d\xa7',
'CjkUnifiedIdeograph8766': '\xe8\x9d\xa6',
'CjkUnifiedIdeograph8765': '\xe8\x9d\xa5',
'CjkUnifiedIdeograph8764': '\xe8\x9d\xa4',
'HangulSyllableRyaek': '\xeb\x9f\xa8',
'HangulSyllableHyak': '\xed\x96\xa8',
'BoxDrawingsDoubleHorizontal': '\xe2\x95\x90',
'HangulSyllableRyaej': '\xeb\x9f\xa6',
'HangulSyllableGgeun': '\xeb\x81\x88',
'CjkUnifiedIdeograph65f9': '\xe6\x97\xb9',
'CjkUnifiedIdeograph65f8': '\xe6\x97\xb8',
'CjkUnifiedIdeograph65f7': '\xe6\x97\xb7',
'CjkUnifiedIdeograph65f6': '\xe6\x97\xb6',
'CjkUnifiedIdeograph65f5': '\xe6\x97\xb5',
'GurmukhiLetterHa': '\xe0\xa8\xb9',
'CjkUnifiedIdeograph65f3': '\xe6\x97\xb3',
'CjkUnifiedIdeograph65f2': '\xe6\x97\xb2',
'CjkUnifiedIdeograph65f1': '\xe6\x97\xb1',
'CjkUnifiedIdeograph65f0': '\xe6\x97\xb0',
'UnicodeCharacter0xf7a6': '\xef\x9e\xa6',
'ArabicLetterGuehIsolatedForm': '\xef\xae\x96',
'CjkUnifiedIdeograph93c8': '\xe9\x8f\x88',
'HangulSyllableGgeuj': '\xeb\x81\x9a',
'LatinCapitalLetterSmallQWithHookTail': '\xc9\x8a',
'HangulSyllableRyaes': '\xeb\x9f\xa3',
'UnicodeCharacter0xfa6e': '\xef\xa9\xae',
'UnicodeCharacter0xfa6d': '\xef\xa9\xad',
'UnicodeCharacter0xfa6f': '\xef\xa9\xaf',
'GreekCapitalLetterUpsilonWithDasia': '\xe1\xbd\x99',
'UnicodeCharacter0xfa6b': '\xef\xa9\xab',
'SinhalaLetterTaalujaNaasikyaya': '\xe0\xb6\xa4',
'CjkUnifiedIdeograph7cef': '\xe7\xb3\xaf',
'CjkUnifiedIdeograph876c': '\xe8\x9d\xac',
'CjkUnifiedIdeograph876b': '\xe8\x9d\xab',
'CjkUnifiedIdeograph876a': '\xe8\x9d\xaa',
'WhiteLeftpointingSmallTriangle': '\xe2\x97\x83',
'CjkUnifiedIdeograph876e': '\xe8\x9d\xae',
'CjkUnifiedIdeograph876d': '\xe8\x9d\xad',
'HangulLetterYuyeo': '\xe3\x86\x8a',
'HangulSyllableGgeub': '\xeb\x81\x95',
'UnicodeCharacter0xf78d': '\xef\x9e\x8d',
'CjkUnifiedIdeograph93c6': '\xe9\x8f\x86',
'UnicodeCharacter0xf289': '\xef\x8a\x89',
'UnicodeCharacter0xdd25': '\xed\xb4\xa5',
'CjkUnifiedIdeograph65ff': '\xe6\x97\xbf',
'CjkUnifiedIdeograph65fe': '\xe6\x97\xbe',
'CjkUnifiedIdeograph65fd': '\xe6\x97\xbd',
'CjkUnifiedIdeograph65fc': '\xe6\x97\xbc',
'RightwardsWhiteArrowFromWall': '\xe2\x87\xb0',
'CjkUnifiedIdeograph65fa': '\xe6\x97\xba',
'CherokeeLetterTlu': '\xe1\x8f\xa1',
'CherokeeLetterTlv': '\xe1\x8f\xa2',
'CjkUnifiedIdeograph5d69': '\xe5\xb5\xa9',
'CherokeeLetterTla': '\xe1\x8f\x9d',
'HangulJongseongSioskiyeok': '\xe1\x87\xa7',
'UnicodeCharacter0xe2e8': '\xee\x8b\xa8',
'UnicodeCharacter0xd7df': '\xed\x9f\x9f',
'CherokeeLetterTli': '\xe1\x8f\x9f',
'UnicodeCharacter0xe2e7': '\xee\x8b\xa7',
'UnicodeCharacter0xe2e4': '\xee\x8b\xa4',
'UnicodeCharacter0xe2e5': '\xee\x8b\xa5',
'UnicodeCharacter0xe2e2': '\xee\x8b\xa2',
'HangulSyllableDoec': '\xeb\x90\xaf',
'CherokeeLetterTlo': '\xe1\x8f\xa0',
'UnicodeCharacter0xe2e1': '\xee\x8b\xa1',
'HangulSyllableHwenh': '\xed\x9c\x82',
'UnicodeCharacter0xd7dd': '\xed\x9f\x9d',
'HangulSyllableHwenj': '\xed\x9c\x81',
'UnicodeCharacter0xda1e': '\xed\xa8\x9e',
'CjkUnifiedIdeograph4095': '\xe4\x82\x95',
'CjkUnifiedIdeograph4094': '\xe4\x82\x94',
'CjkUnifiedIdeograph4097': '\xe4\x82\x97',
'CjkUnifiedIdeograph4096': '\xe4\x82\x96',
'CjkUnifiedIdeograph4091': '\xe4\x82\x91',
'CjkUnifiedIdeograph4090': '\xe4\x82\x90',
'CjkUnifiedIdeograph4093': '\xe4\x82\x93',
'CjkUnifiedIdeograph4092': '\xe4\x82\x92',
'CjkUnifiedIdeograph938b': '\xe9\x8e\x8b',
'CjkUnifiedIdeograph938c': '\xe9\x8e\x8c',
'CjkUnifiedIdeograph938a': '\xe9\x8e\x8a',
'KangxiRadicalField': '\xe2\xbd\xa5',
'CjkUnifiedIdeograph4098': '\xe4\x82\x98',
'CjkUnifiedIdeograph938d': '\xe9\x8e\x8d',
'CjkUnifiedIdeograph938e': '\xe9\x8e\x8e',
'LatinCapitalLetterLWithDotBelow': '\xe1\xb8\xb6',
'UnicodeCharacter0xf7a7': '\xef\x9e\xa7',
'UnicodeCharacter0xe2ef': '\xee\x8b\xaf',
'UnicodeCharacter0xe2ed': '\xee\x8b\xad',
'UnicodeCharacter0xe2ee': '\xee\x8b\xae',
'HangulSyllableRweoss': '\xeb\xa4\x98',
'UnicodeCharacter0xe2ec': '\xee\x8b\xac',
'UnicodeCharacter0xe2ea': '\xee\x8b\xaa',
'HangulSyllableNagg': '\xeb\x82\x9a',
'KangxiRadicalArrive': '\xe2\xbe\x84',
'CjkUnifiedIdeograph409e': '\xe4\x82\x9e',
'CjkUnifiedIdeograph409d': '\xe4\x82\x9d',
'CjkUnifiedIdeograph9388': '\xe9\x8e\x88',
'CjkUnifiedIdeograph409f': '\xe4\x82\x9f',
'CjkUnifiedIdeograph409a': '\xe4\x82\x9a',
'CjkUnifiedIdeograph5b1b': '\xe5\xac\x9b',
'CjkUnifiedIdeograph409c': '\xe4\x82\x9c',
'CjkUnifiedIdeograph409b': '\xe4\x82\x9b',
'CjkUnifiedIdeograph9382': '\xe9\x8e\x82',
'CjkUnifiedIdeograph9383': '\xe9\x8e\x83',
'CjkUnifiedIdeograph9380': '\xe9\x8e\x80',
'CjkUnifiedIdeograph5b1c': '\xe5\xac\x9c',
'CjkUnifiedIdeograph9386': '\xe9\x8e\x86',
'TibetanSubjoinedLetterNya': '\xe0\xbe\x99',
'CjkUnifiedIdeograph9384': '\xe9\x8e\x84',
'CjkUnifiedIdeograph9385': '\xe9\x8e\x85',
'HangulSyllableHilh': '\xed\x9e\x97',
'TibetanSubjoinedLetterKa': '\xe0\xbe\x90',
'HangulSyllableHilm': '\xed\x9e\x92',
'HangulSyllableDoed': '\xeb\x90\x9f',
'HangulSyllableHilb': '\xed\x9e\x93',
'CjkUnifiedIdeograph9cfc': '\xe9\xb3\xbc',
'UnicodeCharacter0x1af1': '\xe1\xab\xb1',
'HangulSyllableOeb': '\xec\x9a\x89',
'HangulSyllableTug': '\xed\x88\xad',
'UnicodeCharacter0xeb28': '\xee\xac\xa8',
'HangulSyllableCwaess': '\xec\xb5\x94',
'YiSyllableNjurx': '\xea\x91\x82',
'HangulSyllableRolm': '\xeb\xa1\xa6',
'UnicodeCharacter0xeb23': '\xee\xac\xa3',
'UnicodeCharacter0xeb22': '\xee\xac\xa2',
'UnicodeCharacter0xeb21': '\xee\xac\xa1',
'YiSyllableUox': '\xea\x80\x8c',
'HangulSyllableDdwils': '\xeb\x9b\xbc',
'UnicodeCharacter0xeb26': '\xee\xac\xa6',
'UnicodeCharacter0xeb25': '\xee\xac\xa5',
'HangulSyllableDdwilp': '\xeb\x9b\xbe',
'HangulChoseongCieucieung': '\xe1\x85\x8d',
'HangulSyllableDdwilm': '\xeb\x9b\xba',
'ModifierLetterChineseToneYangRu': '\xea\x9c\x87',
'EthiopicSyllableZhe': '\xe1\x8b\xa5',
'HangulSyllableDdwilh': '\xeb\x9b\xbf',
'HangulSyllableDdwilg': '\xeb\x9b\xb9',
'HangulSyllableRolh': '\xeb\xa1\xab',
'UnicodeCharacter0xedeb': '\xee\xb7\xab',
'HangulSyllableDdwilb': '\xeb\x9b\xbb',
'HangulSyllableOec': '\xec\x9a\x8f',
'HangulSyllableJjying': '\xec\xb0\x85',
'CjkUnifiedIdeograph996f': '\xe9\xa5\xaf',
'CjkUnifiedIdeograph996d': '\xe9\xa5\xad',
'CjkUnifiedIdeograph996e': '\xe9\xa5\xae',
'CjkUnifiedIdeograph996b': '\xe9\xa5\xab',
'CjkUnifiedIdeograph996c': '\xe9\xa5\xac',
'CjkUnifiedIdeograph90ca': '\xe9\x83\x8a',
'CjkUnifiedIdeograph8ba8': '\xe8\xae\xa8',
'CjkUnifiedIdeograph975d': '\xe9\x9d\x9d',
'CjkUnifiedIdeograph505f': '\xe5\x81\x9f',
'GreekSmallLetterEpsilonWithPsiliAndOxia': '\xe1\xbc\x94',
'CjkUnifiedIdeograph505d': '\xe5\x81\x9d',
'CjkUnifiedIdeograph505e': '\xe5\x81\x9e',
'CjkUnifiedIdeograph505b': '\xe5\x81\x9b',
'CjkUnifiedIdeograph505c': '\xe5\x81\x9c',
'CjkUnifiedIdeograph505a': '\xe5\x81\x9a',
'UnicodeCharacter0x94f': '\xe0\xa5\x8f',
'UnicodeCharacter0x94e': '\xe0\xa5\x8e',
'UnicodeCharacter0xf60c': '\xef\x98\x8c',
'FullwidthQuotationMark': '\xef\xbc\x82',
'UnicodeCharacter0xfff1': '\xef\xbf\xb1',
'UnicodeCharacter0xeb2c': '\xee\xac\xac',
'UnicodeCharacter0xeb2b': '\xee\xac\xab',
'UnicodeCharacter0xeb2a': '\xee\xac\xaa',
'UnicodeCharacter0xf7a4': '\xef\x9e\xa4',
'UnicodeCharacter0xeb2f': '\xee\xac\xaf',
'UnicodeCharacter0xeb2e': '\xee\xac\xae',
'UnicodeCharacter0xeb2d': '\xee\xac\xad',
'CjkUnifiedIdeograph74b9': '\xe7\x92\xb9',
'CjkUnifiedIdeograph74b8': '\xe7\x92\xb8',
'HangulSyllableGgaelp': '\xea\xb9\xb6',
'CjkUnifiedIdeograph5058': '\xe5\x81\x98',
'CjkUnifiedIdeograph5059': '\xe5\x81\x99',
'CjkUnifiedIdeograph5056': '\xe5\x81\x96',
'CjkUnifiedIdeograph5057': '\xe5\x81\x97',
'CjkUnifiedIdeograph5054': '\xe5\x81\x94',
'CjkUnifiedIdeograph5055': '\xe5\x81\x95',
'CjkUnifiedIdeograph5052': '\xe5\x81\x92',
'CjkUnifiedIdeograph5053': '\xe5\x81\x93',
'CjkUnifiedIdeograph5050': '\xe5\x81\x90',
'CjkUnifiedIdeograph5051': '\xe5\x81\x91',
'HangulSyllableRyulb': '\xeb\xa5\xa3',
'HangulSyllableGgaelb': '\xea\xb9\xb3',
'HangulSyllableGgaelg': '\xea\xb9\xb1',
'CjkUnifiedIdeograph9968': '\xe9\xa5\xa8',
'CjkUnifiedIdeograph9969': '\xe9\xa5\xa9',
'CjkUnifiedIdeograph9966': '\xe9\xa5\xa6',
'CjkUnifiedIdeograph9967': '\xe9\xa5\xa7',
'CjkUnifiedIdeograph9964': '\xe9\xa5\xa4',
'CjkUnifiedIdeograph9965': '\xe9\xa5\xa5',
'CjkUnifiedIdeograph9962': '\xe9\xa5\xa2',
'CjkUnifiedIdeograph9963': '\xe9\xa5\xa3',
'CjkUnifiedIdeograph9960': '\xe9\xa5\xa0',
'CjkUnifiedIdeograph9961': '\xe9\xa5\xa1',
'CjkUnifiedIdeograph5bc8': '\xe5\xaf\x88',
'CjkUnifiedIdeograph5bc9': '\xe5\xaf\x89',
'CjkUnifiedIdeograph5bc6': '\xe5\xaf\x86',
'CjkUnifiedIdeograph5bc7': '\xe5\xaf\x87',
'CjkUnifiedIdeograph5bc4': '\xe5\xaf\x84',
'CjkUnifiedIdeograph5bc5': '\xe5\xaf\x85',
'CjkUnifiedIdeograph5bc2': '\xe5\xaf\x82',
'Nand': '\xe2\x8a\xbc',
'CjkUnifiedIdeograph5bc0': '\xe5\xaf\x80',
'CjkUnifiedIdeograph5bc1': '\xe5\xaf\x81',
'HangulSyllableOeg': '\xec\x99\xb9',
'UnicodeCharacter0xed88': '\xee\xb6\x88',
'CjkUnifiedIdeograph665e': '\xe6\x99\x9e',
'IdeographicTelegraphSymbolForDayEighteen': '\xe3\x8f\xb1',
'UnicodeCharacter0xed85': '\xee\xb6\x85',
'CjkUnifiedIdeograph665f': '\xe6\x99\x9f',
'CjkUnifiedIdeograph665a': '\xe6\x99\x9a',
'UnicodeCharacter0xed82': '\xee\xb6\x82',
'CjkUnifiedIdeograph665c': '\xe6\x99\x9c',
'CjkUnifiedIdeograph665b': '\xe6\x99\x9b',
'UnicodeCharacter0xea5e': '\xee\xa9\x9e',
'UnicodeCharacter0xea5d': '\xee\xa9\x9d',
'HangulSyllableCeoj': '\xec\xb2\xae',
'HalfwidthHangulLetterNieun': '\xef\xbe\xa4',
'UnicodeCharacter0xdd8b': '\xed\xb6\x8b',
'GeorgianSmallLetterHar': '\xe2\xb4\xa4',
'GreekCapitalLetterEtaWithDasiaAndOxiaAndProsgegrammeni': '\xe1\xbe\x9d',
'UnicodeCharacter0xdd8c': '\xed\xb6\x8c',
'GeorgianSmallLetterHae': '\xe2\xb4\xa0',
'HangulSyllableRols': '\xeb\xa1\xa8',
'UnicodeCharacter0xdd8e': '\xed\xb6\x8e',
'VaiSyllableLi': '\xea\x94\xb7',
'CjkUnifiedIdeograph6659': '\xe6\x99\x99',
'BalineseLetterTaTawa': '\xe1\xac\xa3',
'CjkUnifiedIdeograph4fb7': '\xe4\xbe\xb7',
'CjkUnifiedIdeograph6655': '\xe6\x99\x95',
'CjkUnifiedIdeograph6654': '\xe6\x99\x94',
'CjkUnifiedIdeograph6657': '\xe6\x99\x97',
'CjkUnifiedIdeograph6656': '\xe6\x99\x96',
'DingbatNegativeCircledSansserifNumberTen': '\xe2\x9e\x93',
'CjkUnifiedIdeograph6650': '\xe6\x99\x90',
'CjkUnifiedIdeograph6653': '\xe6\x99\x93',
'CjkUnifiedIdeograph6652': '\xe6\x99\x92',
'CjkUnifiedIdeograph5bcf': '\xe5\xaf\x8f',
'CjkUnifiedIdeograph5bcd': '\xe5\xaf\x8d',
'CjkUnifiedIdeograph5bce': '\xe5\xaf\x8e',
'CjkUnifiedIdeograph5bcb': '\xe5\xaf\x8b',
'CjkUnifiedIdeograph5bcc': '\xe5\xaf\x8c',
'CjkUnifiedIdeograph5bca': '\xe5\xaf\x8a',
'ArabicLigatureLamWithMeemInitialForm': '\xef\xb3\x8c',
'VaiSyllableLo': '\xea\x97\x8f',
'CjkUnifiedIdeograph4fb1': '\xe4\xbe\xb1',
'ChamPunctuationDoubleDanda': '\xea\xa9\x9e',
'CjkUnifiedIdeograph3c76': '\xe3\xb1\xb6',
'UnicodeCharacter0xf7a5': '\xef\x9e\xa5',
'GreaterthanOrSlantedEqualTo': '\xe2\xa9\xbe',
'CjkUnifiedIdeograph4877': '\xe4\xa1\xb7',
'CjkUnifiedIdeograph4876': '\xe4\xa1\xb6',
'CjkUnifiedIdeograph4875': '\xe4\xa1\xb5',
'CjkUnifiedIdeograph4874': '\xe4\xa1\xb4',
'CjkUnifiedIdeograph4873': '\xe4\xa1\xb3',
'CjkUnifiedIdeograph4872': '\xe4\xa1\xb2',
'CjkUnifiedIdeograph4871': '\xe4\xa1\xb1',
'CjkUnifiedIdeograph4870': '\xe4\xa1\xb0',
'CjkUnifiedIdeograph8fa3': '\xe8\xbe\xa3',
'CjkUnifiedIdeograph8fa2': '\xe8\xbe\xa2',
'CjkUnifiedIdeograph8fa1': '\xe8\xbe\xa1',
'CjkUnifiedIdeograph8fa0': '\xe8\xbe\xa0',
'CjkUnifiedIdeograph8fa7': '\xe8\xbe\xa7',
'CjkUnifiedIdeograph8fa6': '\xe8\xbe\xa6',
'CjkUnifiedIdeograph4879': '\xe4\xa1\xb9',
'CjkUnifiedIdeograph4878': '\xe4\xa1\xb8',
'HangulSyllableBbogs': '\xeb\xbd\x83',
'UnicodeCharacter0xeeec': '\xee\xbb\xac',
'HangulSyllableWa': '\xec\x99\x80',
'ArabicLigatureAinWithYehFinalForm': '\xef\xb4\x94',
'YiSyllableBurx': '\xea\x80\xb0',
'UnicodeCharacter0x1afa': '\xe1\xab\xba',
'GeorgianSmallLetterKan': '\xe2\xb4\x89',
'KhmerSymbolLekAttakBei': '\xe1\x9f\xb3',
'CjkUnifiedIdeograph752d': '\xe7\x94\xad',
'CjkUnifiedIdeograph752e': '\xe7\x94\xae',
'CjkUnifiedIdeograph752f': '\xe7\x94\xaf',
'MyanmarSignLittleSection': '\xe1\x81\x8a',
'CjkUnifiedIdeograph752a': '\xe7\x94\xaa',
'CjkUnifiedIdeograph752b': '\xe7\x94\xab',
'CjkUnifiedIdeograph752c': '\xe7\x94\xac',
'CjkUnifiedIdeograph3e4f': '\xe3\xb9\x8f',
'CjkUnifiedIdeograph3e4d': '\xe3\xb9\x8d',
'CjkUnifiedIdeograph3e4e': '\xe3\xb9\x8e',
'CjkUnifiedIdeograph3e4b': '\xe3\xb9\x8b',
'CjkUnifiedIdeograph3e4c': '\xe3\xb9\x8c',
'UnicodeCharacter0xeeeb': '\xee\xbb\xab',
'CjkUnifiedIdeograph3e4a': '\xe3\xb9\x8a',
'DegreeSign': '\xc2\xb0',
'LatinSmallLetterThWithStrikethrough': '\xe1\xb5\xba',
'HangulSyllableMab': '\xeb\xa7\x99',
'HangulSyllableDdyulp': '\xeb\x9c\x9a',
'YiRadicalShop': '\xea\x93\x82',
'CjkUnifiedIdeograph9530': '\xe9\x94\xb0',
'CjkUnifiedIdeograph487f': '\xe4\xa1\xbf',
'CjkUnifiedIdeograph487e': '\xe4\xa1\xbe',
'CjkUnifiedIdeograph487d': '\xe4\xa1\xbd',
'CjkUnifiedIdeograph487c': '\xe4\xa1\xbc',
'CjkUnifiedIdeograph487b': '\xe4\xa1\xbb',
'CjkUnifiedIdeograph487a': '\xe4\xa1\xba',
'CjkUnifiedIdeograph8fac': '\xe8\xbe\xac',
'CjkUnifiedIdeograph8fab': '\xe8\xbe\xab',
'CjkUnifiedIdeograph8faa': '\xe8\xbe\xaa',
'CjkUnifiedIdeograph8faf': '\xe8\xbe\xaf',
'CjkUnifiedIdeograph8fae': '\xe8\xbe\xae',
'CjkUnifiedIdeograph8fad': '\xe8\xbe\xad',
'CjkUnifiedIdeograph3e46': '\xe3\xb9\x86',
'CjkUnifiedIdeograph3e47': '\xe3\xb9\x87',
'CjkUnifiedIdeograph3e44': '\xe3\xb9\x84',
'CjkUnifiedIdeograph3e45': '\xe3\xb9\x85',
'CjkUnifiedIdeograph3e42': '\xe3\xb9\x82',
'CjkUnifiedIdeograph3e43': '\xe3\xb9\x83',
'CjkUnifiedIdeograph3e40': '\xe3\xb9\x80',
'CjkUnifiedIdeograph3e41': '\xe3\xb9\x81',
'UnicodeCharacter0xf7b7': '\xef\x9e\xb7',
'LatinLetterSmallCapitalReversedR': '\xe1\xb4\x99',
'CjkUnifiedIdeograph3e48': '\xe3\xb9\x88',
'CjkUnifiedIdeograph3e49': '\xe3\xb9\x89',
'InverseBullet': '\xe2\x97\x98',
'UnicodeCharacter0xeeed': '\xee\xbb\xad',
'HangulSyllableSseg': '\xec\x8e\x85',
'CjkUnifiedIdeograph7528': '\xe7\x94\xa8',
'CjkUnifiedIdeograph7529': '\xe7\x94\xa9',
'w': '\xcf\x89',
'LatinLetterSmallCapitalReversedN': '\xe1\xb4\x8e',
'CjkUnifiedIdeograph7524': '\xe7\x94\xa4',
'CjkUnifiedIdeograph7525': '\xe7\x94\xa5',
'CjkUnifiedIdeograph7526': '\xe7\x94\xa6',
'CjkUnifiedIdeograph7527': '\xe7\x94\xa7',
'CjkUnifiedIdeograph7520': '\xe7\x94\xa0',
'CjkUnifiedIdeograph7521': '\xe7\x94\xa1',
'CjkUnifiedIdeograph63f2': '\xe6\x8f\xb2',
'CjkUnifiedIdeograph7523': '\xe7\x94\xa3',
'UnicodeCharacter0xf8ed': '\xef\xa3\xad',
'SundaneseDigitNine': '\xe1\xae\xb9',
'HangulSyllableGyegg': '\xea\xb3\x86',
'SquareYuan': '\xe3\x8d\x90',
'MalayalamLetterEe': '\xe0\xb4\x8f',
'UnicodeCharacter0x243c': '\xe2\x90\xbc',
'UnicodeCharacter0x243b': '\xe2\x90\xbb',
'UnicodeCharacter0x243a': '\xe2\x90\xba',
'UnicodeCharacter0x243f': '\xe2\x90\xbf',
'UnicodeCharacter0x243e': '\xe2\x90\xbe',
'UnicodeCharacter0x243d': '\xe2\x90\xbd',
'HangulSyllableSsoeh': '\xec\x91\x87',
'HangulSyllableSsoej': '\xec\x91\x82',
'HangulSyllableSsoek': '\xec\x91\x84',
'HangulSyllableSsoel': '\xec\x90\xb4',
'HangulSyllableNyeogg': '\xeb\x85\x82',
'HangulSyllableSsoen': '\xec\x90\xb0',
'CjkUnifiedIdeograph6f4b': '\xe6\xbd\x8b',
'CjkUnifiedIdeograph6f4c': '\xe6\xbd\x8c',
'HangulSyllableSsoeb': '\xec\x90\xbd',
'CjkUnifiedIdeograph6f4a': '\xe6\xbd\x8a',
'CjkUnifiedIdeograph6f4f': '\xe6\xbd\x8f',
'UnicodeCharacter0xea06': '\xee\xa8\x86',
'CjkUnifiedIdeograph6f4d': '\xe6\xbd\x8d',
'CjkUnifiedIdeograph6f4e': '\xe6\xbd\x8e',
'CircleWithSmallCircleToTheRight': '\xe2\xa7\x82',
'HangulSyllableDdyulb': '\xeb\x9c\x97',
'HangulSyllableSsoep': '\xec\x91\x86',
'NkoLetterNyaWoloso': '\xdf\xa7',
'HangulSyllableSsoes': '\xec\x90\xbf',
'LatinCapitalLetterUBar': '\xc9\x84',
'UnicodeCharacter0x2433': '\xe2\x90\xb3',
'UnicodeCharacter0x2432': '\xe2\x90\xb2',
'UnicodeCharacter0x2431': '\xe2\x90\xb1',
'UnicodeCharacter0x2430': '\xe2\x90\xb0',
'UnicodeCharacter0x2437': '\xe2\x90\xb7',
'UnicodeCharacter0x2436': '\xe2\x90\xb6',
'UnicodeCharacter0x2435': '\xe2\x90\xb5',
'UnicodeCharacter0x2434': '\xe2\x90\xb4',
'UnicodeCharacter0x2439': '\xe2\x90\xb9',
'UnicodeCharacter0x2438': '\xe2\x90\xb8',
'UnicodeCharacter0xdd82': '\xed\xb6\x82',
'KannadaSignAvagraha': '\xe0\xb2\xbd',
'HangulSyllableRos': '\xeb\xa1\xaf',
'HangulSyllableMag': '\xeb\xa7\x89',
'HangulSyllableJyab': '\xec\x9f\x99',
'UnicodeCharacter0xf8ea': '\xef\xa3\xaa',
'UnicodeCharacter0xdd85': '\xed\xb6\x85',
'UnicodeCharacter0xe767': '\xee\x9d\xa7',
'UnicodeCharacter0xd89e': '\xed\xa2\x9e',
'UnicodeCharacter0xdd86': '\xed\xb6\x86',
'HangulSyllableBbyols': '\xeb\xbe\xbc',
'GreekSmallLetterOmegaWithPerispomeniAndYpogegrammeni': '\xe1\xbf\xb7',
'HangulSyllableRop': '\xeb\xa1\xb6',
'HangulSyllableBbyolt': '\xeb\xbe\xbd',
'UnicodeCharacter0xdd88': '\xed\xb6\x88',
'CjkUnifiedIdeograph6f48': '\xe6\xbd\x88',
'CjkUnifiedIdeograph6f49': '\xe6\xbd\x89',
'KhmerSignCamnucPiiKuuh': '\xe1\x9f\x96',
'HangulSyllableBbyolm': '\xeb\xbe\xba',
'UnicodeCharacter0xdd89': '\xed\xb6\x89',
'CjkUnifiedIdeograph6f42': '\xe6\xbd\x82',
'CjkUnifiedIdeograph6f43': '\xe6\xbd\x83',
'SquareEraNameMeizi': '\xe3\x8d\xbe',
'CjkUnifiedIdeograph6f41': '\xe6\xbd\x81',
'CjkUnifiedIdeograph6f46': '\xe6\xbd\x86',
'CjkUnifiedIdeograph6f47': '\xe6\xbd\x87',
'CjkUnifiedIdeograph6f44': '\xe6\xbd\x84',
'CjkUnifiedIdeograph6f45': '\xe6\xbd\x85',
'CjkUnifiedIdeograph3b0e': '\xe3\xac\x8e',
'CjkUnifiedIdeograph4f7e': '\xe4\xbd\xbe',
'CjkUnifiedIdeograph4f7d': '\xe4\xbd\xbd',
'CjkUnifiedIdeograph4f7f': '\xe4\xbd\xbf',
'CjkUnifiedIdeograph4f7a': '\xe4\xbd\xba',
'CjkUnifiedIdeograph4f7c': '\xe4\xbd\xbc',
'CjkUnifiedIdeograph4f7b': '\xe4\xbd\xbb',
'HangulSyllableMae': '\xeb\xa7\xa4',
'CanadianSyllabicsNngoo': '\xe1\x99\xb4',
'UnicodeCharacter0xf8ec': '\xef\xa3\xac',
'UnicodeCharacter0xe765': '\xee\x9d\xa5',
'SaurashtraLetterEe': '\xea\xa2\x8d',
'HangulSyllableRog': '\xeb\xa1\x9d',
'LatinCapitalLetterHWithCedilla': '\xe1\xb8\xa8',
'HangulSyllableKilp': '\xed\x82\xb2',
'HangulSyllableKils': '\xed\x82\xb0',
'GurmukhiLetterUu': '\xe0\xa8\x8a',
'HangulSyllableRoe': '\xeb\xa2\xb0',
'HangulSyllableKilg': '\xed\x82\xad',
'HangulSyllableRod': '\xeb\xa1\xa3',
'ArmenianCapitalLetterPiwr': '\xd5\x93',
'CyrillicCapitalLetterDzze': '\xea\x9a\x88',
'IdeographicTelegraphSymbolForHourEleven': '\xe3\x8d\xa3',
'HangulSyllableKilb': '\xed\x82\xaf',
'HangulSyllableKilm': '\xed\x82\xae',
'CjkUnifiedIdeograph7735': '\xe7\x9c\xb5',
'HangulSyllableKilh': '\xed\x82\xb3',
'CjkUnifiedIdeograph7734': '\xe7\x9c\xb4',
'HangulSyllableRonj': '\xeb\xa1\xa1',
'HangulSyllableRonh': '\xeb\xa1\xa2',
'KatakanaLetterNo': '\xe3\x83\x8e',
'CjkUnifiedIdeograph9053': '\xe9\x81\x93',
'KatakanaLetterNi': '\xe3\x83\x8b',
'UnicodeCharacter0xf0f8': '\xef\x83\xb8',
'KatakanaLetterNe': '\xe3\x83\x8d',
'YiSyllableMgu': '\xea\x89\x82',
'CjkUnifiedIdeograph7736': '\xe7\x9c\xb6',
'CanadianSyllabicsCarrierShee': '\xe1\x99\x97',
'KatakanaLetterNa': '\xe3\x83\x8a',
'YiSyllableMgo': '\xea\x88\xbb',
'CjkUnifiedIdeograph7739': '\xe7\x9c\xb9',
'CjkUnifiedIdeograph4f79': '\xe4\xbd\xb9',
'CjkUnifiedIdeograph4f78': '\xe4\xbd\xb8',
'HangulSyllableRon': '\xeb\xa1\xa0',
'CjkUnifiedIdeograph4f75': '\xe4\xbd\xb5',
'CjkUnifiedIdeograph4f74': '\xe4\xbd\xb4',
'CjkUnifiedIdeograph4f77': '\xe4\xbd\xb7',
'CjkUnifiedIdeograph4f76': '\xe4\xbd\xb6',
'CjkUnifiedIdeograph4f71': '\xe4\xbd\xb1',
'CjkUnifiedIdeograph4f70': '\xe4\xbd\xb0',
'CjkUnifiedIdeograph4f73': '\xe4\xbd\xb3',
'CjkUnifiedIdeograph4f72': '\xe4\xbd\xb2',
'ArabicFivePointedStar': '\xd9\xad',
'HangulSyllableDwid': '\xeb\x92\xab',
'HangulSyllableDwig': '\xeb\x92\xa5',
'UnicodeCharacter0xf0f9': '\xef\x83\xb9',
'HangulSyllableDwic': '\xeb\x92\xbb',
'HangulSyllableDwib': '\xeb\x92\xb5',
'HangulSyllableDwim': '\xeb\x92\xb4',
'HangulSyllableDwil': '\xeb\x92\xac',
'HangulSyllableDwin': '\xeb\x92\xa8',
'HangulSyllableDwih': '\xeb\x92\xbf',
'HangulSyllableDwik': '\xeb\x92\xbc',
'HangulSyllableDwij': '\xeb\x92\xba',
'HangulSyllableMyeobs': '\xeb\xaa\x82',
'HangulSyllableDwit': '\xeb\x92\xbd',
'BraillePatternDots2578': '\xe2\xa3\x92',
'HangulSyllableDwip': '\xeb\x92\xbe',
'HangulSyllableDwis': '\xeb\x92\xb7',
'UnicodeCharacter0xe760': '\xee\x9d\xa0',
'HangulSyllableRaenh': '\xeb\x9e\x9e',
'HangulSyllableRaenj': '\xeb\x9e\x9d',
'QuaternionIntegralOperator': '\xe2\xa8\x96',
'CjkRadicalCivilian': '\xe2\xba\xa0',
'UnicodeCharacter0xe761': '\xee\x9d\xa1',
'OlChikiLetterEp': '\xe1\xb1\xaf',
'TeluguVowelSignOo': '\xe0\xb1\x8b',
'HangulSyllableBbunj': '\xeb\xbf\x91',
'HangulSyllableWigs': '\xec\x9c\x87',
'HangulSyllableYaenh': '\xec\x96\x9e',
'HangulSyllableWigg': '\xec\x9c\x86',
'YiSyllableJox': '\xea\x8f\xad',
'YiSyllableJot': '\xea\x8f\xac',
'BalineseVowelSignSukuIlut': '\xe1\xac\xb9',
'YiSyllableJop': '\xea\x8f\xaf',
'HangulSyllableRult': '\xeb\xa3\xb5',
'Prime': '\xe2\x80\xb2',
'UnicodeCharacter0xd918': '\xed\xa4\x98',
'HangulSyllableEugg': '\xec\x9c\xbe',
'HangulSyllableRuls': '\xeb\xa3\xb4',
'HangulSyllableEugs': '\xec\x9c\xbf',
'CanadianSyllabicsWestcreeKwo': '\xe1\x91\xbb',
'CanadianSyllabicsWestcreeKwi': '\xe1\x91\xb7',
'HangulSyllableRulp': '\xeb\xa3\xb6',
'CanadianSyllabicsWestcreeKwe': '\xe1\x91\xb5',
'FourDotPunctuation': '\xe2\x81\x98',
'HebrewLetterAlef': '\xd7\x90',
'CanadianSyllabicsWestcreeKwa': '\xe1\x91\xbf',
'SinhalaLetterIyanna': '\xe0\xb6\x89',
'HangulSyllableBbwelg': '\xec\x80\x8d',
'CjkRadicalGrassOne': '\xe2\xba\xbe',
'HangulSyllableBbwelh': '\xec\x80\x93',
'HangulSyllableJyok': '\xec\xa3\xb8',
'HangulSyllableBbwelm': '\xec\x80\x8e',
'HangulSyllableBbwels': '\xec\x80\x90',
'GreekSmallLetterEtaWithDasiaAndVaria': '\xe1\xbc\xa3',
'HangulSyllableRyo': '\xeb\xa3\x8c',
'YiSyllableJit': '\xea\x8f\xa0',
'HangulSyllableBbwelt': '\xec\x80\x91',
'UnicodeCharacter0xf146': '\xef\x85\x86',
'HangulSyllableSyeob': '\xec\x85\xa5',
'HangulSyllableSsulp': '\xec\x91\xb2',
'HangulSyllableGegg': '\xea\xb2\x8e',
'CjkUnifiedIdeograph6dbf': '\xe6\xb6\xbf',
'CjkUnifiedIdeograph6dbd': '\xe6\xb6\xbd',
'CjkUnifiedIdeograph6dbe': '\xe6\xb6\xbe',
'CjkUnifiedIdeograph6dbb': '\xe6\xb6\xbb',
'CjkUnifiedIdeograph6dbc': '\xe6\xb6\xbc',
'CjkUnifiedIdeograph6dba': '\xe6\xb6\xba',
'VaiSyllableNo': '\xea\x97\x9f',
'HangulSyllableRulg': '\xeb\xa3\xb1',
'CjkUnifiedIdeograph856a': '\xe8\x95\xaa',
'HangulSyllableRyolh': '\xeb\xa3\x9b',
'CjkUnifiedIdeograph856c': '\xe8\x95\xac',
'CjkUnifiedIdeograph856b': '\xe8\x95\xab',
'CjkUnifiedIdeograph856e': '\xe8\x95\xae',
'CjkUnifiedIdeograph856d': '\xe8\x95\xad',
'UnicodeCharacter0xea9e': '\xee\xaa\x9e',
'CjkUnifiedIdeograph856f': '\xe8\x95\xaf',
'LeftTortoiseShellBracket': '\xe3\x80\x94',
'SinhalaLetterMahaapraanaDdayanna': '\xe0\xb6\xaa',
'HangulSyllableRyi': '\xeb\xa6\x90',
'HangulSyllableKwinh': '\xed\x80\xba',
'backsigma': '\xcf\xb9',
'CjkUnifiedIdeograph8569': '\xe8\x95\xa9',
'CjkUnifiedIdeograph8568': '\xe8\x95\xa8',
'ArabicLetterNoonFinalForm': '\xef\xbb\xa6',
'UnicodeCharacter0xeab5': '\xee\xaa\xb5',
'CjkUnifiedIdeograph8561': '\xe8\x95\xa1',
'CjkUnifiedIdeograph8560': '\xe8\x95\xa0',
'CjkUnifiedIdeograph8563': '\xe8\x95\xa3',
'CjkUnifiedIdeograph8562': '\xe8\x95\xa2',
'CjkUnifiedIdeograph8565': '\xe8\x95\xa5',
'CjkUnifiedIdeograph8564': '\xe8\x95\xa4',
'CjkUnifiedIdeograph8567': '\xe8\x95\xa7',
'CjkUnifiedIdeograph8566': '\xe8\x95\xa6',
'UnicodeCharacter0xda59': '\xed\xa9\x99',
'UnicodeCharacter0xed3a': '\xee\xb4\xba',
'HangulSyllableSyeol': '\xec\x85\x9c',
'CjkUnifiedIdeograph6db8': '\xe6\xb6\xb8',
'CjkUnifiedIdeograph6db9': '\xe6\xb6\xb9',
'CjkUnifiedIdeograph6db6': '\xe6\xb6\xb6',
'CjkUnifiedIdeograph6db7': '\xe6\xb6\xb7',
'CjkUnifiedIdeograph6db4': '\xe6\xb6\xb4',
'CjkUnifiedIdeograph6db5': '\xe6\xb6\xb5',
'CjkUnifiedIdeograph6db2': '\xe6\xb6\xb2',
'CjkUnifiedIdeograph6db3': '\xe6\xb6\xb3',
'CjkUnifiedIdeograph6db0': '\xe6\xb6\xb0',
'CjkUnifiedIdeograph6db1': '\xe6\xb6\xb1',
'HebrewAccentDarga': '\xd6\xa7',
'HangulSyllableSyeos': '\xec\x85\xa7',
'YiSyllableKet': '\xea\x88\x8a',
'UnicodeCharacter0xeab3': '\xee\xaa\xb3',
'NkoLetterJonaCha': '\xdf\xa9',
'CjkUnifiedIdeograph9051': '\xe9\x81\x91',
'UnicodeCharacter0xf0f2': '\xef\x83\xb2',
'Sextile': '\xe2\x9a\xb9',
'GreekSmallLetterDelta': '\xce\xb4',
'HangulSyllableReunh': '\xeb\xa5\xba',
'UnicodeCharacter0xda51': '\xed\xa9\x91',
'HangulSyllableSyeot': '\xec\x85\xad',
'MultisetUnion': '\xe2\x8a\x8e',
'HangulSyllableReunj': '\xeb\xa5\xb9',
'UnicodeCharacter0xda57': '\xed\xa9\x97',
'HangulSyllableRyolt': '\xeb\xa3\x99',
'u': '\xcf\x85',
'HangulSyllableHwegg': '\xed\x9b\xbe',
'CjkUnifiedIdeograph4aa0': '\xe4\xaa\xa0',
'CjkUnifiedIdeograph4aa1': '\xe4\xaa\xa1',
'CjkUnifiedIdeograph4aa2': '\xe4\xaa\xa2',
'CjkUnifiedIdeograph4aa3': '\xe4\xaa\xa3',
'CjkUnifiedIdeograph4aa4': '\xe4\xaa\xa4',
'CjkUnifiedIdeograph4aa5': '\xe4\xaa\xa5',
'CombiningLatinSmallLetterAo': '\xe1\xb7\x95',
'CjkUnifiedIdeograph4aa7': '\xe4\xaa\xa7',
'CjkUnifiedIdeograph4aa8': '\xe4\xaa\xa8',
'CjkUnifiedIdeograph4aa9': '\xe4\xaa\xa9',
'CjkUnifiedIdeograph5cc0': '\xe5\xb3\x80',
'CombiningLatinSmallLetterAe': '\xe1\xb7\x94',
'UnicodeCharacter0x61c': '\xd8\x9c',
'HangulSyllableRyols': '\xeb\xa3\x98',
'CombiningLatinSmallLetterAv': '\xe1\xb7\x96',
'HangulJungseongAe': '\xe1\x85\xa2',
'HangulJungseongAo': '\xe1\x85\xb6',
'HangulSyllableBbwenj': '\xec\x80\x89',
'UnicodeCharacter0x61d': '\xd8\x9d',
'HangulJungseongAu': '\xe1\x85\xb7',
'UnicodeCharacter0xa9ad': '\xea\xa6\xad',
'HangulSyllableReung': '\xeb\xa6\x89',
'AplFunctionalSymbolQuoteUnderbar': '\xe2\x8d\x98',
'CombiningLatinSmallLetterEth': '\xe1\xb7\x99',
'CombiningCyrillicLetterBe': '\xe2\xb7\xa0',
'YiSyllableJjyp': '\xea\x90\xb0',
'UnicodeCharacter0xe5f1': '\xee\x97\xb1',
'UnicodeCharacter0xa9aa': '\xea\xa6\xaa',
'SquareKairi': '\xe3\x8c\x8b',
'UnicodeCharacter0xf0f6': '\xef\x83\xb6',
'YiSyllableJjyx': '\xea\x90\xae',
'CjkUnifiedIdeograph4aaa': '\xe4\xaa\xaa',
'CjkUnifiedIdeograph4aab': '\xe4\xaa\xab',
'CjkUnifiedIdeograph4aac': '\xe4\xaa\xac',
'CjkUnifiedIdeograph4aad': '\xe4\xaa\xad',
'CjkUnifiedIdeograph4aae': '\xe4\xaa\xae',
'CjkUnifiedIdeograph4aaf': '\xe4\xaa\xaf',
'ClosedSubsetOrEqualTo': '\xe2\xab\x91',
'CjkUnifiedIdeograph724d': '\xe7\x89\x8d',
'ArabicLetterQafFinalForm': '\xef\xbb\x96',
'HangulSyllableRogg': '\xeb\xa1\x9e',
'HangulSyllableJjyeolt': '\xec\xaa\x91',
'CjkUnifiedIdeograph84fd': '\xe8\x93\xbd',
'SoftwarefunctionSymbol': '\xe2\x8e\x94',
'CjkUnifiedIdeograph84ff': '\xe8\x93\xbf',
'CjkUnifiedIdeograph84fa': '\xe8\x93\xba',
'CjkUnifiedIdeograph84fb': '\xe8\x93\xbb',
'CjkUnifiedIdeograph84fc': '\xe8\x93\xbc',
'CjkUnifiedIdeograph6391': '\xe6\x8e\x91',
'HangulSyllableJjyeolp': '\xec\xaa\x92',
'HangulSyllableRya': '\xeb\x9e\xb4',
'YiSyllableHe': '\xea\x89\xbe',
'UnicodeCharacter0xf281': '\xef\x8a\x81',
'HangulSyllableJjyeols': '\xec\xaa\x90',
'LatinCapitalLetterOWithOgonekAndMacron': '\xc7\xac',
'YiSyllableHo': '\xea\x89\xbb',
'ArabicLigatureLamWithHahWithAlefMaksuraFinalForm': '\xef\xb6\x82',
'CircledNumberTwentyThree': '\xe3\x89\x93',
'HangulSyllableJyop': '\xec\xa3\xba',
'HangulSyllableBoej': '\xeb\xb5\x9e',
'UnicodeCharacter0xe96a': '\xee\xa5\xaa',
'UnicodeCharacter0xe96b': '\xee\xa5\xab',
'SquareEraNameSyouwa': '\xe3\x8d\xbc',
'UnicodeCharacter0xe96d': '\xee\xa5\xad',
'UnicodeCharacter0xe96e': '\xee\xa5\xae',
'UnicodeCharacter0xe96f': '\xee\xa5\xaf',
'TeluguLetterU': '\xe0\xb0\x89',
'UnicodeCharacter0xe5ce': '\xee\x97\x8e',
'TeluguLetterO': '\xe0\xb0\x92',
'CjkUnifiedIdeograph7d3e': '\xe7\xb4\xbe',
'TeluguLetterI': '\xe0\xb0\x87',
'CjkUnifiedIdeograph84f9': '\xe8\x93\xb9',
'TeluguLetterE': '\xe0\xb0\x8e',
'CjkUnifiedIdeograph84f5': '\xe8\x93\xb5',
'ReversedPrime': '\xe2\x80\xb5',
'CjkUnifiedIdeograph84f7': '\xe8\x93\xb7',
'TeluguLetterA': '\xe0\xb0\x85',
'CjkUnifiedIdeograph6a9d': '\xe6\xaa\x9d',
'CjkUnifiedIdeograph84f2': '\xe8\x93\xb2',
'CjkUnifiedIdeograph5db3': '\xe5\xb6\xb3',
'UnicodeCharacter0xe968': '\xee\xa5\xa8',
'UnicodeCharacter0xe969': '\xee\xa5\xa9',
'UnicodeCharacter0xe5ca': '\xee\x97\x8a',
'LatinCapitalLetterUWithCircumflexBelow': '\xe1\xb9\xb6',
'CjkUnifiedIdeograph8532': '\xe8\x94\xb2',
'UnicodeCharacter0xe960': '\xee\xa5\xa0',
'UnicodeCharacter0xe961': '\xee\xa5\xa1',
'GreekSmallLetterUpsilonWithMacron': '\xe1\xbf\xa1',
'CjkUnifiedIdeograph76bd': '\xe7\x9a\xbd',
'UnicodeCharacter0xe964': '\xee\xa5\xa4',
'YiSyllableRurx': '\xea\x8f\x98',
'UnicodeCharacter0xd90e': '\xed\xa4\x8e',
'UnicodeCharacter0xe967': '\xee\xa5\xa7',
'BlackChessBishop': '\xe2\x99\x9d',
'CjkUnifiedIdeograph6f36': '\xe6\xbc\xb6',
'CjkUnifiedIdeograph5db2': '\xe5\xb6\xb2',
'ArabicLigatureLamWithAlefMaksuraFinalForm': '\xef\xb2\x86',
'HangulSyllableCyeogs': '\xec\xb3\x93',
'GlagoliticSmallLetterYo': '\xe2\xb1\x96',
'EmptySetWithOverbar': '\xe2\xa6\xb1',
'HangulLetterPieupkiyeok': '\xe3\x85\xb2',
'LepchaLetterMa': '\xe1\xb0\x95',
'HangulSyllableRyaenh': '\xeb\x9f\x96',
'MongolianLetterAliGaliDagalga': '\xe1\xa2\xa9',
'HangulSyllableRyaenj': '\xeb\x9f\x95',
'HangulSyllableRyaeng': '\xeb\x9f\xa5',
'GlagoliticSmallLetterYu': '\xe2\xb1\x93',
'CjkUnifiedIdeograph850f': '\xe8\x94\x8f',
'CjkUnifiedIdeograph6c61': '\xe6\xb1\xa1',
'CjkUnifiedIdeograph6c60': '\xe6\xb1\xa0',
'CjkUnifiedIdeograph6c63': '\xe6\xb1\xa3',
'CjkUnifiedIdeograph6c62': '\xe6\xb1\xa2',
'CjkUnifiedIdeograph6c65': '\xe6\xb1\xa5',
'CjkUnifiedIdeograph6c64': '\xe6\xb1\xa4',
'CjkUnifiedIdeograph6c67': '\xe6\xb1\xa7',
'CjkUnifiedIdeograph6c66': '\xe6\xb1\xa6',
'CjkUnifiedIdeograph6c69': '\xe6\xb1\xa9',
'CjkUnifiedIdeograph6c68': '\xe6\xb1\xa8',
'CjkUnifiedIdeograph7b94': '\xe7\xae\x94',
'CjkUnifiedIdeograph7b95': '\xe7\xae\x95',
'CjkUnifiedIdeograph7b92': '\xe7\xae\x92',
'CjkUnifiedIdeograph7b93': '\xe7\xae\x93',
'CjkUnifiedIdeograph7b90': '\xe7\xae\x90',
'CjkUnifiedIdeograph7b91': '\xe7\xae\x91',
'CjkUnifiedIdeograph6dda': '\xe6\xb7\x9a',
'UnicodeCharacter0xf282': '\xef\x8a\x82',
'CjkUnifiedIdeograph9955': '\xe9\xa5\x95',
'MongolianLetterSibeHaa': '\xe1\xa1\xad',
'CjkUnifiedIdeograph6397': '\xe6\x8e\x97',
'OriyaLetterKa': '\xe0\xac\x95',
'CjkUnifiedIdeograph6dde': '\xe6\xb7\x9e',
'EthiopicSyllableTzee': '\xe1\x8d\x84',
'CjkUnifiedIdeograph6ddf': '\xe6\xb7\x9f',
'CopticSmallLetterOldCopticDja': '\xe2\xb3\x99',
'ArabicLigatureQafWithMeemInitialForm': '\xef\xb3\x83',
'ArabicLigatureGhainWithMeemWithAlefMaksuraFinalForm': '\xef\xb5\xbb',
'UnicodeCharacter0xa9a9': '\xea\xa6\xa9',
'UnicodeCharacter0xf852': '\xef\xa1\x92',
'HangulSyllablePalt': '\xed\x8c\x99',
'ArabicLetterZainFinalForm': '\xef\xba\xb0',
'BoxDrawingsLightVerticalAndRight': '\xe2\x94\x9c',
'LatinSmallLetterLj': '\xc7\x89',
'CjkUnifiedIdeograph6c6a': '\xe6\xb1\xaa',
'CjkUnifiedIdeograph6c6c': '\xe6\xb1\xac',
'CjkUnifiedIdeograph6c6b': '\xe6\xb1\xab',
'CjkUnifiedIdeograph6c6e': '\xe6\xb1\xae',
'LatinSmallLetterAWithBreveAndHookAbove': '\xe1\xba\xb3',
'CjkUnifiedIdeograph6c6f': '\xe6\xb1\xaf',
'CjkUnifiedIdeograph7b9f': '\xe7\xae\x9f',
'CjkUnifiedIdeograph7b9d': '\xe7\xae\x9d',
'CjkUnifiedIdeograph7b9e': '\xe7\xae\x9e',
'CjkUnifiedIdeograph7b9b': '\xe7\xae\x9b',
'CjkUnifiedIdeograph7b9c': '\xe7\xae\x9c',
'UnicodeCharacter0xe7ba': '\xee\x9e\xba',
'CjkUnifiedIdeograph7b9a': '\xe7\xae\x9a',
'CjkUnifiedIdeograph4c34': '\xe4\xb0\xb4',
'CjkUnifiedIdeograph4c35': '\xe4\xb0\xb5',
'UnicodeCharacter0x1cb8': '\xe1\xb2\xb8',
'CjkUnifiedIdeograph4c37': '\xe4\xb0\xb7',
'CjkUnifiedIdeograph4c30': '\xe4\xb0\xb0',
'CjkUnifiedIdeograph4c31': '\xe4\xb0\xb1',
'CjkUnifiedIdeograph4c32': '\xe4\xb0\xb2',
'CjkUnifiedIdeograph4c33': '\xe4\xb0\xb3',
'UnicodeCharacter0x1cb2': '\xe1\xb2\xb2',
'UnicodeCharacter0x1cb3': '\xe1\xb2\xb3',
'DevanagariSignAnusvara': '\xe0\xa4\x82',
'UnicodeCharacter0x1cb1': '\xe1\xb2\xb1',
'CjkUnifiedIdeograph4c38': '\xe4\xb0\xb8',
'CjkUnifiedIdeograph4c39': '\xe4\xb0\xb9',
'UnicodeCharacter0x1cb4': '\xe1\xb2\xb4',
'UnicodeCharacter0x1cb5': '\xe1\xb2\xb5',
'UnicodeCharacter0xe5f3': '\xee\x97\xb3',
'SaurashtraLetterMa': '\xea\xa2\xaa',
'KhmerSymbolDapbuonRoc': '\xe1\xa7\xbe',
'UnicodeCharacter0xe03e': '\xee\x80\xbe',
'CjkUnifiedIdeograph889c': '\xe8\xa2\x9c',
'HangulSyllableHwang': '\xed\x99\xa9',
'YiRadicalHxit': '\xea\x92\x99',
'MongolianLetterSibeKa': '\xe1\xa1\xa3',
'HangulSyllableTwals': '\xed\x87\x88',
'HangulSyllableTwalp': '\xed\x87\x8a',
'HangulSyllableTwalt': '\xed\x87\x89',
'HangulSyllableTwalh': '\xed\x87\x8b',
'ArabicLigatureLamWithAlefWithMaddaAboveIsolatedForm': '\xef\xbb\xb5',
'HangulSyllableTwalm': '\xed\x87\x86',
'HangulSyllableTwalb': '\xed\x87\x87',
'SucceedsAboveEqualsSign': '\xe2\xaa\xb4',
'HangulSyllableTwalg': '\xed\x87\x85',
'MyanmarLetterBa': '\xe1\x80\x97',
'KangxiRadicalDog': '\xe2\xbd\x9d',
'UnicodeCharacter0xeabb': '\xee\xaa\xbb',
'HangulSyllableBbwaenh': '\xeb\xbd\xbe',
'ArabicLigatureHahWithJeemIsolatedForm': '\xef\xb0\x97',
'CjkUnifiedIdeograph4c3d': '\xe4\xb0\xbd',
'CjkUnifiedIdeograph4c3e': '\xe4\xb0\xbe',
'DevanagariLetterQa': '\xe0\xa5\x98',
'VaiSyllableShoo': '\xea\x96\x88',
'KannadaLetterRra': '\xe0\xb2\xb1',
'CjkUnifiedIdeograph4c3b': '\xe4\xb0\xbb',
'CjkUnifiedIdeograph4c3c': '\xe4\xb0\xbc',
'UnicodeCharacter0x1cbb': '\xe1\xb2\xbb',
'UnicodeCharacter0x1cbc': '\xe1\xb2\xbc',
'UnicodeCharacter0x1cba': '\xe1\xb2\xba',
'UnicodeCharacter0x1cbf': '\xe1\xb2\xbf',
'UnicodeCharacter0x1cbd': '\xe1\xb2\xbd',
'UnicodeCharacter0x1cbe': '\xe1\xb2\xbe',
'HangulSyllableWenj': '\xec\x9b\xad',
'HangulSyllableBoeh': '\xeb\xb5\xa3',
'HangulSyllableCyegg': '\xec\xb3\xae',
'VaiSyllableCu': '\xea\x96\xb0',
'HangulSyllableDwels': '\xeb\x92\x94',
'UnicodeCharacter0xffc9': '\xef\xbf\x89',
'UnicodeCharacter0xffc8': '\xef\xbf\x88',
'CjkUnifiedIdeograph9d84': '\xe9\xb6\x84',
'DentistrySymbolLightDownAndHorizontalWithWave': '\xe2\x8f\x87',
'UnicodeCharacter0xffc1': '\xef\xbf\x81',
'CjkUnifiedIdeograph5ccc': '\xe5\xb3\x8c',
'CjkUnifiedIdeograph711d': '\xe7\x84\x9d',
'integral': '\xe2\x88\xab',
'CjkUnifiedIdeograph47d1': '\xe4\x9f\x91',
'CjkUnifiedIdeograph47d0': '\xe4\x9f\x90',
'CjkUnifiedIdeograph47d3': '\xe4\x9f\x93',
'CjkUnifiedIdeograph47d2': '\xe4\x9f\x92',
'CjkUnifiedIdeograph47d5': '\xe4\x9f\x95',
'CjkUnifiedIdeograph47d4': '\xe4\x9f\x94',
'CjkUnifiedIdeograph47d7': '\xe4\x9f\x97',
'CjkUnifiedIdeograph47d6': '\xe4\x9f\x96',
'CjkUnifiedIdeograph47d9': '\xe4\x9f\x99',
'CjkUnifiedIdeograph47d8': '\xe4\x9f\x98',
'YiSyllableMap': '\xea\x82\xb8',
'ParenthesizedHangulMieumA': '\xe3\x88\x92',
'[4]': '\xe2\x82\x84',
'UnicodeCharacter0xe10b': '\xee\x84\x8b',
'CjkUnifiedIdeograph5db8': '\xe5\xb6\xb8',
'UnicodeCharacter0xe10a': '\xee\x84\x8a',
'UnicodeCharacter0xe10f': '\xee\x84\x8f',
'UnicodeCharacter0xe10d': '\xee\x84\x8d',
'UnicodeCharacter0xe10e': '\xee\x84\x8e',
'LeftwardsTwoheadedArrowWithTailWithDoubleVerticalStroke': '\xe2\xac\xbd',
'HangulSyllablePoenh': '\xed\x91\x86',
'CjkUnifiedIdeograph8b08': '\xe8\xac\x88',
'CjkUnifiedIdeograph8b09': '\xe8\xac\x89',
'CjkUnifiedIdeograph47da': '\xe4\x9f\x9a',
'CjkUnifiedIdeograph47dc': '\xe4\x9f\x9c',
'CjkUnifiedIdeograph47db': '\xe4\x9f\x9b',
'CjkUnifiedIdeograph47de': '\xe4\x9f\x9e',
'CjkUnifiedIdeograph47dd': '\xe4\x9f\x9d',
'CjkUnifiedIdeograph47df': '\xe4\x9f\x9f',
'LowerRightShadowedWhiteSquare': '\xe2\x9d\x91',
'LatinSmallLetterEsh': '\xca\x83',
'LeftwardsArrowToBar': '\xe2\x87\xa4',
'ArmenianSmallLetterLiwn': '\xd5\xac',
'CircledHangulIeung': '\xe3\x89\xa7',
'PhagspaLetterTsa': '\xea\xa1\x90',
'HangulSyllableBbwess': '\xec\x80\x98',
'CopticCapitalLetterCryptogrammicEie': '\xe2\xb2\xb6',
'UnicodeCharacter0xf284': '\xef\x8a\x84',
'CjkUnifiedIdeograph6c9a': '\xe6\xb2\x9a',
'CjkUnifiedIdeograph70ef': '\xe7\x83\xaf',
'BraillePatternDots23458': '\xe2\xa2\x9e',
'UnicodeCharacter0xe109': '\xee\x84\x89',
'CanadianSyllabicsWestcreeSwaa': '\xe1\x94\x83',
'UnicodeCharacter0xe102': '\xee\x84\x82',
'UnicodeCharacter0xe103': '\xee\x84\x83',
'UnicodeCharacter0xe100': '\xee\x84\x80',
'UnicodeCharacter0xe101': '\xee\x84\x81',
'BraillePatternDots23456': '\xe2\xa0\xbe',
'BraillePatternDots23457': '\xe2\xa1\x9e',
'UnicodeCharacter0xe104': '\xee\x84\x84',
'CjkUnifiedIdeograph70ea': '\xe7\x83\xaa',
'CjkUnifiedIdeograph6c9f': '\xe6\xb2\x9f',
'ThaiCurrencySymbolBaht': '\xe0\xb8\xbf',
'BalineseSignUluRicem': '\xe1\xac\x80',
'ArabicLetterLamWithSmallV': '\xda\xb5',
'HangulSyllableHyag': '\xed\x96\x91',
'UnicodeCharacter0xddfb': '\xed\xb7\xbb',
'UnicodeCharacter0xddfc': '\xed\xb7\xbc',
'CombiningTripleUnderdot': '\xe2\x83\xa8',
'UnicodeCharacter0xddfa': '\xed\xb7\xba',
'CjkUnifiedIdeograph9e88': '\xe9\xba\x88',
'CjkUnifiedIdeograph9e89': '\xe9\xba\x89',
'UnicodeCharacter0xddfd': '\xed\xb7\xbd',
'UnicodeCharacter0xddfe': '\xed\xb7\xbe',
'CjkUnifiedIdeograph9e84': '\xe9\xba\x84',
'CjkUnifiedIdeograph9e85': '\xe9\xba\x85',
'CjkUnifiedIdeograph9e86': '\xe9\xba\x86',
'CjkUnifiedIdeograph9e87': '\xe9\xba\x87',
'CjkUnifiedIdeograph9e80': '\xe9\xba\x80',
'CjkUnifiedIdeograph76b1': '\xe7\x9a\xb1',
'CjkUnifiedIdeograph9e82': '\xe9\xba\x82',
'CjkUnifiedIdeograph9e83': '\xe9\xba\x83',
'HangulSyllableDwigs': '\xeb\x92\xa7',
'GreekCapitalLetterMu': '\xce\x9c',
'CjkUnifiedIdeograph76b0': '\xe7\x9a\xb0',
'HangulSyllablePyeonh': '\xed\x8e\xba',
'UnicodeCharacter0xe5c2': '\xee\x97\x82',
'HangulSyllableKwals': '\xec\xbd\xbc',
'HangulSyllableHyod': '\xed\x9a\xaf',
'HangulSyllableHyog': '\xed\x9a\xa9',
'HangulSyllablePyeonj': '\xed\x8e\xb9',
'HangulSyllableHyoc': '\xed\x9a\xbf',
'GreekVaria': '\xe1\xbf\xaf',
'HangulSyllableHyom': '\xed\x9a\xb8',
'HangulSyllableHyol': '\xed\x9a\xb0',
'HangulSyllableHyon': '\xed\x9a\xac',
'GlagoliticSmallLetterUku': '\xe2\xb1\x86',
'HangulSyllableHyok': '\xed\x9b\x80',
'HangulSyllableHyoj': '\xed\x9a\xbe',
'ChamLetterNa': '\xea\xa8\x98',
'HangulSyllableHyot': '\xed\x9b\x81',
'ThaanaLetterMeemu': '\xde\x89',
'HangulSyllableHyop': '\xed\x9b\x82',
'HangulSyllableHyos': '\xed\x9a\xbb',
'RightwardsArrowAboveShortLeftwardsArrow': '\xe2\xa5\x82',
'CjkUnifiedIdeograph8503': '\xe8\x94\x83',
'HangulSyllablePyeong': '\xed\x8f\x89',
'CjkUnifiedIdeograph6dd9': '\xe6\xb7\x99',
'HangulSyllableSsonh': '\xec\x8f\x9e',
'UnicodeCharacter0xddf2': '\xed\xb7\xb2',
'UnicodeCharacter0xddf3': '\xed\xb7\xb3',
'UnicodeCharacter0xddf0': '\xed\xb7\xb0',
'UnicodeCharacter0xddf1': '\xed\xb7\xb1',
'UnicodeCharacter0xddf6': '\xed\xb7\xb6',
'CjkUnifiedIdeograph7d38': '\xe7\xb4\xb8',
'UnicodeCharacter0xddf4': '\xed\xb7\xb4',
'CjkUnifiedIdeograph57f8': '\xe5\x9f\xb8',
'CjkUnifiedIdeograph9e8d': '\xe9\xba\x8d',
'CombiningRightTackBelow': '\xcc\x99',
'CjkUnifiedIdeograph9e8f': '\xe9\xba\x8f',
'HangulSyllableSsonj': '\xec\x8f\x9d',
'HangulSyllableTeop': '\xed\x85\x8a',
'CyrillicCapitalLetterElWithMiddleHook': '\xd4\xa0',
'CjkUnifiedIdeograph9e8b': '\xe9\xba\x8b',
'CjkUnifiedIdeograph8506': '\xe8\x94\x86',
'CombiningDownTackBelow': '\xcc\x9e',
'HangulSyllableBoen': '\xeb\xb5\x8c',
'KayahLiLetterKha': '\xea\xa4\x8b',
'MongolianLetterPa': '\xe1\xa0\xab',
'CjkUnifiedIdeograph6038': '\xe6\x80\xb8',
'CjkUnifiedIdeograph9029': '\xe9\x80\xa9',
'CjkUnifiedIdeograph6dd1': '\xe6\xb7\x91',
'CjkUnifiedIdeograph6035': '\xe6\x80\xb5',
'CjkUnifiedIdeograph6034': '\xe6\x80\xb4',
'CjkUnifiedIdeograph6037': '\xe6\x80\xb7',
'CjkUnifiedIdeograph6036': '\xe6\x80\xb6',
'CjkUnifiedIdeograph6031': '\xe6\x80\xb1',
'CjkUnifiedIdeograph6030': '\xe6\x80\xb0',
'CjkUnifiedIdeograph6033': '\xe6\x80\xb3',
'CjkUnifiedIdeograph6032': '\xe6\x80\xb2',
'RunicLetterLongbranchhagallH': '\xe1\x9a\xbc',
'CjkUnifiedIdeograph3aac': '\xe3\xaa\xac',
'CjkUnifiedIdeograph3aab': '\xe3\xaa\xab',
'CjkUnifiedIdeograph3aae': '\xe3\xaa\xae',
'CjkUnifiedIdeograph3aad': '\xe3\xaa\xad',
'UnicodeCharacter0xecc5': '\xee\xb3\x85',
'CjkUnifiedIdeograph3aaf': '\xe3\xaa\xaf',
'ContainsWithLongHorizontalStroke': '\xe2\x8b\xba',
'LatinCapitalLetterOWithDotAboveAndMacron': '\xc8\xb0',
'ChamLetterDda': '\xea\xa8\x99',
'CjkUnifiedIdeograph468d': '\xe4\x9a\x8d',
'CjkUnifiedIdeograph468e': '\xe4\x9a\x8e',
'CjkUnifiedIdeograph468f': '\xe4\x9a\x8f',
'CjkUnifiedIdeograph468a': '\xe4\x9a\x8a',
'CjkUnifiedIdeograph468b': '\xe4\x9a\x8b',
'CjkUnifiedIdeograph468c': '\xe4\x9a\x8c',
'CjkUnifiedIdeograph66cf': '\xe6\x9b\x8f',
'CjkUnifiedIdeograph66ce': '\xe6\x9b\x8e',
'CjkUnifiedIdeograph66cd': '\xe6\x9b\x8d',
'CjkUnifiedIdeograph66cc': '\xe6\x9b\x8c',
'CjkUnifiedIdeograph66cb': '\xe6\x9b\x8b',
'CjkUnifiedIdeograph66ca': '\xe6\x9b\x8a',
'CherokeeLetterHna': '\xe1\x8e\xbf',
'SundaneseDigitEight': '\xe1\xae\xb8',
'ArabicLigatureSallaIsolatedForm': '\xef\xb7\xb9',
'LeftHalfBlock': '\xe2\x96\x8c',
'UnicodeCharacter0xe0d8': '\xee\x83\x98',
'GeorgianLetterVin': '\xe1\x83\x95',
'CjkUnifiedIdeograph603d': '\xe6\x80\xbd',
'CjkUnifiedIdeograph902e': '\xe9\x80\xae',
'CjkUnifiedIdeograph603f': '\xe6\x80\xbf',
'CjkUnifiedIdeograph603a': '\xe6\x80\xba',
'CanadianSyllabicsLhi': '\xe1\x96\xa0',
'CjkUnifiedIdeograph603c': '\xe6\x80\xbc',
'CjkUnifiedIdeograph603b': '\xe6\x80\xbb',
'CanadianSyllabicsCarrierTsee': '\xe1\x99\x9e',
'AplFunctionalSymbolLeftwardsVane': '\xe2\x8d\x85',
'CjkUnifiedIdeograph4994': '\xe4\xa6\x94',
'YiRadicalTat': '\xea\x92\xa0',
'GreekSmallLetterEtaWithDasiaAndYpogegrammeni': '\xe1\xbe\x91',
'UnicodeCharacter0xe084': '\xee\x82\x84',
'NorthEastBlackArrow': '\xe2\xac\x88',
'CjkUnifiedIdeograph4684': '\xe4\x9a\x84',
'CjkUnifiedIdeograph4685': '\xe4\x9a\x85',
'CjkUnifiedIdeograph4686': '\xe4\x9a\x86',
'CjkUnifiedIdeograph4687': '\xe4\x9a\x87',
'CjkUnifiedIdeograph4680': '\xe4\x9a\x80',
'CjkUnifiedIdeograph4681': '\xe4\x9a\x81',
'CjkUnifiedIdeograph4682': '\xe4\x9a\x82',
'TelephoneRecorder': '\xe2\x8c\x95',
'CjkUnifiedIdeograph66c7': '\xe6\x9b\x87',
'CjkUnifiedIdeograph66c6': '\xe6\x9b\x86',
'CjkUnifiedIdeograph66c5': '\xe6\x9b\x85',
'CjkUnifiedIdeograph66c4': '\xe6\x9b\x84',
'CjkUnifiedIdeograph4688': '\xe4\x9a\x88',
'CjkUnifiedIdeograph4689': '\xe4\x9a\x89',
'CjkUnifiedIdeograph66c1': '\xe6\x9b\x81',
'CjkUnifiedIdeograph66c0': '\xe6\x9b\x80',
'CjkUnifiedIdeograph3aa1': '\xe3\xaa\xa1',
'CjkUnifiedIdeograph3aa0': '\xe3\xaa\xa0',
'CjkUnifiedIdeograph3aa3': '\xe3\xaa\xa3',
'CjkUnifiedIdeograph3aa2': '\xe3\xaa\xa2',
'CjkUnifiedIdeograph3aa5': '\xe3\xaa\xa5',
'CjkUnifiedIdeograph3aa4': '\xe3\xaa\xa4',
'CjkUnifiedIdeograph3aa7': '\xe3\xaa\xa7',
'CjkUnifiedIdeograph3aa6': '\xe3\xaa\xa6',
'CjkUnifiedIdeograph3aa9': '\xe3\xaa\xa9',
'CjkUnifiedIdeograph3aa8': '\xe3\xaa\xa8',
'HangulSyllableKyoss': '\xec\xbf\x98',
'CjkUnifiedIdeograph8b0f': '\xe8\xac\x8f',
'CjkUnifiedIdeograph70e4': '\xe7\x83\xa4',
'CjkUnifiedIdeograph7348': '\xe7\x8d\x88',
'CjkUnifiedIdeograph7349': '\xe7\x8d\x89',
'CjkUnifiedIdeograph70e5': '\xe7\x83\xa5',
'CjkUnifiedIdeograph7344': '\xe7\x8d\x84',
'CjkUnifiedIdeograph7345': '\xe7\x8d\x85',
'CjkUnifiedIdeograph7346': '\xe7\x8d\x86',
'CjkUnifiedIdeograph7347': '\xe7\x8d\x87',
'CjkUnifiedIdeograph7340': '\xe7\x8d\x80',
'CjkUnifiedIdeograph70e6': '\xe7\x83\xa6',
'ParenthesizedIdeographAlliance': '\xe3\x88\xbf',
'CjkUnifiedIdeograph7343': '\xe7\x8d\x83',
'CjkUnifiedIdeograph3bab': '\xe3\xae\xab',
'CjkUnifiedIdeograph3bac': '\xe3\xae\xac',
'UnicodeCharacter0xde6e': '\xed\xb9\xae',
'CjkUnifiedIdeograph3baa': '\xe3\xae\xaa',
'CjkUnifiedIdeograph3baf': '\xe3\xae\xaf',
'UnicodeCharacter0xde6b': '\xed\xb9\xab',
'CjkUnifiedIdeograph3bad': '\xe3\xae\xad',
'CjkUnifiedIdeograph3bae': '\xe3\xae\xae',
'ViewdataSquare': '\xe2\x8c\x97',
'CjkUnifiedIdeograph70e0': '\xe7\x83\xa0',
'LatinCapitalLetterHWithBreveBelow': '\xe1\xb8\xaa',
'CjkUnifiedIdeograph70e1': '\xe7\x83\xa1',
'CanadianSyllabicsCarrierChi': '\xe1\x99\xa5',
'HangulSyllableCoelt': '\xec\xb5\xa9',
'CjkUnifiedIdeograph70e2': '\xe7\x83\xa2',
'CanadianSyllabicsCarrierCho': '\xe1\x99\xa2',
'CjkUnifiedIdeograph4539': '\xe4\x94\xb9',
'CanadianSyllabicsCarrierCha': '\xe1\x99\xa6',
'CjkUnifiedIdeograph4537': '\xe4\x94\xb7',
'CjkUnifiedIdeograph4534': '\xe4\x94\xb4',
'CjkUnifiedIdeograph4535': '\xe4\x94\xb5',
'CanadianSyllabicsCarrierChe': '\xe1\x99\xa3',
'CjkUnifiedIdeograph4533': '\xe4\x94\xb3',
'CjkUnifiedIdeograph4530': '\xe4\x94\xb0',
'CjkUnifiedIdeograph4531': '\xe4\x94\xb1',
'CjkUnifiedIdeograph7b45': '\xe7\xad\x85',
'HangulSyllableCoelg': '\xec\xb5\xa5',
'CjkUnifiedIdeograph7b44': '\xe7\xad\x84',
'HangulSyllableCoelm': '\xec\xb5\xa6',
'HangulSyllableKweolp': '\xed\x80\x8a',
'CanadianSyllabicsCarrierChu': '\xe1\x99\xa1',
'HangulSyllableCoelh': '\xec\xb5\xab',
'ArabicSmallHighLigatureQafWithLamWithAlefMaksura': '\xdb\x97',
'MyanmarLetterShanZa': '\xe1\x81\xb9',
'CjkUnifiedIdeograph3ba0': '\xe3\xae\xa0',
'CjkUnifiedIdeograph3ba1': '\xe3\xae\xa1',
'CjkUnifiedIdeograph3ba6': '\xe3\xae\xa6',
'ThaiCharacterHoNokhuk': '\xe0\xb8\xae',
'CjkUnifiedIdeograph3ba4': '\xe3\xae\xa4',
'CjkUnifiedIdeograph3ba5': '\xe3\xae\xa5',
'CherokeeLetterV': '\xe1\x8e\xa5',
'CjkUnifiedIdeograph3ba8': '\xe3\xae\xa8',
'CjkUnifiedIdeograph3ba9': '\xe3\xae\xa9',
'CherokeeLetterS': '\xe1\x8f\x8d',
'UnicodeCharacter0xde69': '\xed\xb9\xa9',
'CjkUnifiedIdeograph70e9': '\xe7\x83\xa9',
'CherokeeLetterO': '\xe1\x8e\xa3',
'HangulSyllableByelg': '\xeb\xb3\xa1',
'CjkUnifiedIdeograph95fc': '\xe9\x97\xbc',
'CjkUnifiedIdeograph7b43': '\xe7\xad\x83',
'CherokeeLetterI': '\xe1\x8e\xa2',
'UnicodeCharacter0xe3b9': '\xee\x8e\xb9',
'CjkUnifiedIdeograph734d': '\xe7\x8d\x8d',
'CjkUnifiedIdeograph734e': '\xe7\x8d\x8e',
'CherokeeLetterE': '\xe1\x8e\xa1',
'HangulSyllableByelm': '\xeb\xb3\xa2',
'UnicodeCharacter0xe3b2': '\xee\x8e\xb2',
'CjkUnifiedIdeograph734a': '\xe7\x8d\x8a',
'CherokeeLetterA': '\xe1\x8e\xa0',
'CjkUnifiedIdeograph734c': '\xe7\x8d\x8c',
'HangulSyllableKang': '\xec\xba\x89',
'UnicodeCharacter0xf8e6': '\xef\xa3\xa6',
'CjkRadicalEwe': '\xe2\xba\xb8',
'HangulSyllableKanj': '\xec\xb9\xb9',
'HangulSyllableKanh': '\xec\xb9\xba',
'UnicodeCharacter0xf03c': '\xef\x80\xbc',
'UnicodeCharacter0xf03b': '\xef\x80\xbb',
'CjkUnifiedIdeograph453f': '\xe4\x94\xbf',
'LeftsideArcAnticlockwiseArrow': '\xe2\xa4\xb9',
'CjkUnifiedIdeograph453e': '\xe4\x94\xbe',
'CjkUnifiedIdeograph453b': '\xe4\x94\xbb',
'CjkUnifiedIdeograph453c': '\xe4\x94\xbc',
'LatinSmallLetterOWithHornAndTilde': '\xe1\xbb\xa1',
'UnicodeCharacter0xdf91': '\xed\xbe\x91',
'UnicodeCharacter0xdf90': '\xed\xbe\x90',
'UnicodeCharacter0xdf93': '\xed\xbe\x93',
'UnicodeCharacter0xdf92': '\xed\xbe\x92',
'UnicodeCharacter0xdf95': '\xed\xbe\x95',
'UnicodeCharacter0xdf94': '\xed\xbe\x94',
'UnicodeCharacter0xdf97': '\xed\xbe\x97',
'CyrillicSmallLetterDzwe': '\xea\x9a\x83',
'UnicodeCharacter0xdf99': '\xed\xbe\x99',
'UnicodeCharacter0xdf98': '\xed\xbe\x98',
'UnicodeCharacter0xfe53': '\xef\xb9\x93',
'HangulSyllableByeogg': '\xeb\xb2\xbe',
'UnicodeCharacter0xf03f': '\xef\x80\xbf',
'UnicodeCharacter0xdaae': '\xed\xaa\xae',
'UnicodeCharacter0xf03e': '\xef\x80\xbe',
'UnicodeCharacter0xdaab': '\xed\xaa\xab',
'UnicodeCharacter0xf03d': '\xef\x80\xbd',
'CombiningLeftHalfRingBelow': '\xcc\x9c',
'UnicodeCharacter0xdaac': '\xed\xaa\xac',
'CanadianSyllabicsYcreeNoo': '\xe1\x93\x86',
'CjkUnifiedIdeograph5342': '\xe5\x8d\x82',
'UnicodeCharacter0xe044': '\xee\x81\x84',
'CjkUnifiedIdeograph659a': '\xe6\x96\x9a',
'CjkUnifiedIdeograph659f': '\xe6\x96\x9f',
'CjkUnifiedIdeograph659d': '\xe6\x96\x9d',
'CjkUnifiedIdeograph5343': '\xe5\x8d\x83',
'ArabicLigatureSeenWithMeemWithMeemFinalForm': '\xef\xb5\xa2',
'CjkUnifiedIdeograph5341': '\xe5\x8d\x81',
'HangulSyllableGgwit': '\xeb\x80\xa5',
'ArabicVowelSignSmallVAbove': '\xd9\x9a',
'CjkUnifiedIdeograph5346': '\xe5\x8d\x86',
'UnicodeCharacter0xf4ce': '\xef\x93\x8e',
'UnicodeCharacter0xf4cd': '\xef\x93\x8d',
'UnicodeCharacter0xf4cc': '\xef\x93\x8c',
'PhagspaDoubleHeadMark': '\xea\xa1\xb5',
'UnicodeCharacter0xf4ca': '\xef\x93\x8a',
'CjkUnifiedIdeograph359c': '\xe3\x96\x9c',
'CjkUnifiedIdeograph359b': '\xe3\x96\x9b',
'HangulSyllableNyass': '\xeb\x83\xa4',
'UnicodeCharacter0xdf9c': '\xed\xbe\x9c',
'UnicodeCharacter0xdf9b': '\xed\xbe\x9b',
'UnicodeCharacter0xdf9e': '\xed\xbe\x9e',
'UnicodeCharacter0xdf9d': '\xed\xbe\x9d',
'UnicodeCharacter0xdf9f': '\xed\xbe\x9f',
'HangulSyllablePup': '\xed\x92\x92',
'LatinSmallLetterWWithDiaeresis': '\xe1\xba\x85',
'UnicodeCharacter0xe76b': '\xee\x9d\xab',
'UnicodeCharacter0xf0f3': '\xef\x83\xb3',
'UnicodeCharacter0xf4c9': '\xef\x93\x89',
'LeftCeiling': '\xe2\x8c\x88',
'UnicodeCharacter0xf4c7': '\xef\x93\x87',
'UnicodeCharacter0xf4c6': '\xef\x93\x86',
'UnicodeCharacter0xf4c5': '\xef\x93\x85',
'UnicodeCharacter0xf4c4': '\xef\x93\x84',
'UnicodeCharacter0xf4c3': '\xef\x93\x83',
'UnicodeCharacter0xf4c2': '\xef\x93\x82',
'UnicodeCharacter0xf4c1': '\xef\x93\x81',
'UnicodeCharacter0xf0f5': '\xef\x83\xb5',
'CjkUnifiedIdeograph6592': '\xe6\x96\x92',
'CjkUnifiedIdeograph6593': '\xe6\x96\x93',
'CjkUnifiedIdeograph6590': '\xe6\x96\x90',
'CjkUnifiedIdeograph6591': '\xe6\x96\x91',
'IdeographicTelegraphSymbolForDayTwentytwo': '\xe3\x8f\xb5',
'CjkUnifiedIdeograph6597': '\xe6\x96\x97',
'CjkUnifiedIdeograph6594': '\xe6\x96\x94',
'CjkUnifiedIdeograph6595': '\xe6\x96\x95',
'HangulSyllablePeoss': '\xed\x8e\x90',
'UnicodeCharacter0xdfc2': '\xed\xbf\x82',
'CjkUnifiedIdeograph6598': '\xe6\x96\x98',
'CjkUnifiedIdeograph6599': '\xe6\x96\x99',
'HangulSyllableBbod': '\xeb\xbd\x87',
'UnicodeCharacter0x8aa': '\xe0\xa2\xaa',
'UnicodeCharacter0x8ab': '\xe0\xa2\xab',
'UnicodeCharacter0x8ac': '\xe0\xa2\xac',
'UnicodeCharacter0x8ad': '\xe0\xa2\xad',
'UnicodeCharacter0x8ae': '\xe0\xa2\xae',
'UnicodeCharacter0x8af': '\xe0\xa2\xaf',
'HangulSyllableBboc': '\xeb\xbd\x97',
'HangulSyllableBbol': '\xeb\xbd\x88',
'HangulSyllableBbom': '\xeb\xbd\x90',
'HangulSyllableBbon': '\xeb\xbd\x84',
'HangulSyllableBboh': '\xeb\xbd\x9b',
'WhiteHexagon': '\xe2\xac\xa1',
'HangulSyllableBboj': '\xeb\xbd\x96',
'HangulSyllableBbok': '\xeb\xbd\x98',
'HangulSyllableBbot': '\xeb\xbd\x99',
'CjkCompatibilityIdeographf96f': '\xef\xa5\xaf',
'CjkCompatibilityIdeographf96e': '\xef\xa5\xae',
'CjkCompatibilityIdeographf96d': '\xef\xa5\xad',
'HangulSyllableBbop': '\xeb\xbd\x9a',
'CjkCompatibilityIdeographf96b': '\xef\xa5\xab',
'CjkCompatibilityIdeographf96a': '\xef\xa5\xaa',
'HangulSyllableBbos': '\xeb\xbd\x93',
'OghamLetterOnn': '\xe1\x9a\x91',
'CjkUnifiedIdeograph692d': '\xe6\xa4\xad',
'BalineseVowelSignRaRepaTedung': '\xe1\xac\xbb',
'UnicodeCharacter0xa8e8': '\xea\xa3\xa8',
'TironianSignEt': '\xe2\x81\x8a',
'UnicodeCharacter0xa8e9': '\xea\xa3\xa9',
'CjkCompatibilityIdeographf967': '\xef\xa5\xa7',
'HangulSyllableJep': '\xec\xa0\xb6',
'HangulSyllableJes': '\xec\xa0\xaf',
'CjkCompatibilityIdeographf964': '\xef\xa5\xa4',
'HangulSyllableJeu': '\xec\xa6\x88',
'HangulSyllableJet': '\xec\xa0\xb5',
'CjkCompatibilityIdeographf961': '\xef\xa5\xa1',
'CjkCompatibilityIdeographf960': '\xef\xa5\xa0',
'HexagramForRetreat': '\xe4\xb7\xa0',
'HangulSyllableSyelh': '\xec\x85\xbf',
'CircledIdeographHigh': '\xe3\x8a\xa4',
'CjkCompatibilityIdeographf969': '\xef\xa5\xa9',
'CjkCompatibilityIdeographf968': '\xef\xa5\xa8',
'UnicodeCharacter0x8a0': '\xe0\xa2\xa0',
'NkoDigitZero': '\xdf\x80',
'UnicodeCharacter0x8a2': '\xe0\xa2\xa2',
'UnicodeCharacter0x8a3': '\xe0\xa2\xa3',
'UnicodeCharacter0x8a4': '\xe0\xa2\xa4',
'UnicodeCharacter0x8a5': '\xe0\xa2\xa5',
'UnicodeCharacter0x8a6': '\xe0\xa2\xa6',
'UnicodeCharacter0x8a7': '\xe0\xa2\xa7',
'UnicodeCharacter0x8a8': '\xe0\xa2\xa8',
'UnicodeCharacter0x8a9': '\xe0\xa2\xa9',
'HangulSyllableJek': '\xec\xa0\xb4',
'HangulSyllableJej': '\xec\xa0\xb2',
'HangulSyllableJem': '\xec\xa0\xac',
'UnicodeCharacter0xa8e6': '\xea\xa3\xa6',
'CjkUnifiedIdeograph97f9': '\xe9\x9f\xb9',
'HangulSyllableJen': '\xec\xa0\xa0',
'YiSyllableHmix': '\xea\x82\x92',
'YiSyllableCep': '\xea\x8b\x83',
'HangulSyllableHulb': '\xed\x9b\x8f',
'LatinCapitalLetterWWithAcute': '\xe1\xba\x82',
'UnicodeCharacter0xecc6': '\xee\xb3\x86',
'YiSyllableHmip': '\xea\x82\x94',
'YiSyllableCex': '\xea\x8b\x81',
'YiSyllableHmit': '\xea\x82\x91',
'UnicodeCharacter0xf8e9': '\xef\xa3\xa9',
'HangulSyllableGgaeb': '\xea\xb9\xb9',
'CjkUnifiedIdeograph7b2e': '\xe7\xac\xae',
'CjkUnifiedIdeograph7339': '\xe7\x8c\xb9',
'YiSyllableHmie': '\xea\x82\x96',
'MyanmarVowelSignMonIi': '\xe1\x80\xb3',
'CjkUnifiedIdeograph7a0b': '\xe7\xa8\x8b',
'CjkUnifiedIdeograph4ac6': '\xe4\xab\x86',
'GujaratiLetterJa': '\xe0\xaa\x9c',
'UnicodeCharacter0xf4f8': '\xef\x93\xb8',
'CjkUnifiedIdeograph7a0c': '\xe7\xa8\x8c',
'LepchaDigitOne': '\xe1\xb1\x81',
'UnicodeCharacter0xf4f9': '\xef\x93\xb9',
'ArabicLigatureKafWithHahIsolatedForm': '\xef\xb0\xb9',
'HangulSyllableTweonj': '\xed\x89\x8d',
'HangulSyllableEoss': '\xec\x97\x88',
'CjkUnifiedIdeograph6e6f': '\xe6\xb9\xaf',
'CjkUnifiedIdeograph6e6e': '\xe6\xb9\xae',
'CjkUnifiedIdeograph6e6d': '\xe6\xb9\xad',
'CjkUnifiedIdeograph6e6c': '\xe6\xb9\xac',
'CjkUnifiedIdeograph6e6b': '\xe6\xb9\xab',
'CjkUnifiedIdeograph6e6a': '\xe6\xb9\xaa',
'CjkUnifiedIdeograph534b': '\xe5\x8d\x8b',
'CjkCompatibilityIdeographf9bb': '\xef\xa6\xbb',
'CjkUnifiedIdeograph3594': '\xe3\x96\x94',
'HangulSyllableWeolt': '\xec\x9b\x99',
'CjkUnifiedIdeograph8fb3': '\xe8\xbe\xb3',
'HangulSyllableDyot': '\xeb\x91\x8d',
'CjkUnifiedIdeograph534a': '\xe5\x8d\x8a',
'IdeographicTelegraphSymbolForFebruary': '\xe3\x8b\x81',
'CjkUnifiedIdeograph3591': '\xe3\x96\x91',
'UnicodeCharacter0xf543': '\xef\x95\x83',
'CjkUnifiedIdeograph534d': '\xe5\x8d\x8d',
'CjkUnifiedIdeograph615e': '\xe6\x85\x9e',
'CjkUnifiedIdeograph6e68': '\xe6\xb9\xa8',
'CjkUnifiedIdeograph6e67': '\xe6\xb9\xa7',
'CjkUnifiedIdeograph6e66': '\xe6\xb9\xa6',
'CjkUnifiedIdeograph6e65': '\xe6\xb9\xa5',
'HangulChoseongKhieukh': '\xe1\x84\x8f',
'CjkUnifiedIdeograph6e63': '\xe6\xb9\xa3',
'CjkUnifiedIdeograph6e62': '\xe6\xb9\xa2',
'CjkUnifiedIdeograph6e61': '\xe6\xb9\xa1',
'CjkUnifiedIdeograph6e60': '\xe6\xb9\xa0',
'HangulSyllableDyoj': '\xeb\x91\x8a',
'HangulSyllableSyelb': '\xec\x85\xbb',
'UnicodeCharacter0xf0fd': '\xef\x83\xbd',
'UnicodeCharacter0xf324': '\xef\x8c\xa4',
'HangulSyllableAk': '\xec\x95\x9c',
'UnicodeCharacter0xf0fe': '\xef\x83\xbe',
'UnicodeCharacter0xe6da': '\xee\x9b\x9a',
'ArabicLigatureYehWithHamzaAboveWithEFinalForm': '\xef\xaf\xb7',
'SmallRomanNumeralEleven': '\xe2\x85\xba',
'MalayalamSignVisarga': '\xe0\xb4\x83',
'UnicodeCharacter0xe6dc': '\xee\x9b\x9c',
'UnicodeCharacter0xe6db': '\xee\x9b\x9b',
'UnicodeCharacter0x9e5': '\xe0\xa7\xa5',
'UnicodeCharacter0x9e4': '\xe0\xa7\xa4',
'KayahLiLetterNga': '\xea\xa4\x8d',
'UnicodeCharacter0xe6dd': '\xee\x9b\x9d',
'HangzhouNumeralThree': '\xe3\x80\xa3',
'CjkUnifiedIdeograph7e07': '\xe7\xb8\x87',
'MyanmarSymbolAforementioned': '\xe1\x81\x8e',
'NkoSymbolOoDennen': '\xdf\xb6',
'CjkUnifiedIdeograph7e02': '\xe7\xb8\x82',
'CjkUnifiedIdeograph7e03': '\xe7\xb8\x83',
'CjkUnifiedIdeograph7e00': '\xe7\xb8\x80',
'CjkUnifiedIdeograph7e01': '\xe7\xb8\x81',
'CjkUnifiedIdeograph7a0d': '\xe7\xa8\x8d',
'HangulSyllableBbwegs': '\xec\x80\x87',
'OghamLetterOr': '\xe1\x9a\x96',
'CjkUnifiedIdeograph7e09': '\xe7\xb8\x89',
'NotASubsetOf': '\xe2\x8a\x84',
'UnicodeCharacter0xf033': '\xef\x80\xb3',
'UnicodeCharacter0xdec': '\xe0\xb7\xac',
'ClockwiseTopSemicircleArrow': '\xe2\x86\xb7',
'Ogonek': '\xcb\x9b',
'UnicodeCharacter0xf031': '\xef\x80\xb1',
'CjkUnifiedIdeograph7e0f': '\xe7\xb8\x8f',
'UnicodeCharacter0xdaa6': '\xed\xaa\xa6',
'CjkUnifiedIdeograph7e0d': '\xe7\xb8\x8d',
'CjkUnifiedIdeograph7e0e': '\xe7\xb8\x8e',
'BalineseLetterOkara': '\xe1\xac\x91',
'CjkUnifiedIdeograph7e0c': '\xe7\xb8\x8c',
'CjkUnifiedIdeograph7e0a': '\xe7\xb8\x8a',
'CjkUnifiedIdeograph56fe': '\xe5\x9b\xbe',
'NewTaiLueLetterHighBa': '\xe1\xa6\xa2',
'UnicodeCharacter0xdaa4': '\xed\xaa\xa4',
'UnicodeCharacter0xf036': '\xef\x80\xb6',
'TamilLetterKa': '\xe0\xae\x95',
'UnicodeCharacter0xdaa5': '\xed\xaa\xa5',
'UnicodeCharacter0xdaa2': '\xed\xaa\xa2',
'GreekSmallLetterBeta': '\xce\xb2',
'CjkUnifiedIdeograph56ff': '\xe5\x9b\xbf',
'UnicodeCharacter0xdaa3': '\xed\xaa\xa3',
'CjkUnifiedIdeograph5a39': '\xe5\xa8\xb9',
'CjkUnifiedIdeograph5a38': '\xe5\xa8\xb8',
'UnicodeCharacter0xdaa0': '\xed\xaa\xa0',
'TeluguFractionDigitTwoForEvenPowersOfFour': '\xe0\xb1\xbd',
'CjkUnifiedIdeograph5a31': '\xe5\xa8\xb1',
'CjkUnifiedIdeograph5a30': '\xe5\xa8\xb0',
'CjkUnifiedIdeograph5a33': '\xe5\xa8\xb3',
'CjkUnifiedIdeograph5a32': '\xe5\xa8\xb2',
'CircledHangulChieuchA': '\xe3\x89\xb7',
'CjkUnifiedIdeograph5a34': '\xe5\xa8\xb4',
'CjkUnifiedIdeograph5a37': '\xe5\xa8\xb7',
'CjkUnifiedIdeograph5a36': '\xe5\xa8\xb6',
'CjkCompatibilityIdeographf935': '\xef\xa4\xb5',
'HangulSyllableDdaess': '\xeb\x95\xa0',
'HangulSyllableRweolp': '\xeb\xa4\x92',
'EqualsSignAboveRightwardsArrow': '\xe2\xa5\xb1',
'LatinSmallLetterEWithCircumflexAndTilde': '\xe1\xbb\x85',
'CombiningCyrillicPokrytie': '\xd2\x87',
'UnicodeCharacter0xf109': '\xef\x84\x89',
'UnicodeCharacter0xf108': '\xef\x84\x88',
'UnicodeCharacter0xdaa8': '\xed\xaa\xa8',
'UnicodeCharacter0xf103': '\xef\x84\x83',
'UnicodeCharacter0xf102': '\xef\x84\x82',
'UnicodeCharacter0xf101': '\xef\x84\x81',
'CjkUnifiedIdeograph969c': '\xe9\x9a\x9c',
'UnicodeCharacter0xf107': '\xef\x84\x87',
'UnicodeCharacter0xf106': '\xef\x84\x86',
'UnicodeCharacter0xf105': '\xef\x84\x85',
'UnicodeCharacter0xf104': '\xef\x84\x84',
'UnicodeCharacter0xeb7c': '\xee\xad\xbc',
'CjkUnifiedIdeograph8400': '\xe8\x90\x80',
'GreekLetterSmallCapitalLamda': '\xe1\xb4\xa7',
'HangulSyllableRuk': '\xeb\xa4\x80',
'ArabicLetterTahMedialForm': '\xef\xbb\x84',
'CjkUnifiedIdeograph5a3a': '\xe5\xa8\xba',
'CjkUnifiedIdeograph5a3c': '\xe5\xa8\xbc',
'CjkUnifiedIdeograph5a3b': '\xe5\xa8\xbb',
'CjkUnifiedIdeograph5a3e': '\xe5\xa8\xbe',
'CjkUnifiedIdeograph5a3d': '\xe5\xa8\xbd',
'CjkUnifiedIdeograph5a3f': '\xe5\xa8\xbf',
'Sector': '\xe2\x8c\x94',
'HangulSyllableKyugs': '\xed\x81\x93',
'UnicodeCharacter0xf10c': '\xef\x84\x8c',
'UnicodeCharacter0xf10b': '\xef\x84\x8b',
'UnicodeCharacter0xf10a': '\xef\x84\x8a',
'UnicodeCharacter0xf10f': '\xef\x84\x8f',
'UnicodeCharacter0xf10e': '\xef\x84\x8e',
'UnicodeCharacter0xf10d': '\xef\x84\x8d',
'UnicodeCharacter0xabe1': '\xea\xaf\xa1',
'HangulSyllableKyugg': '\xed\x81\x92',
'HangulSyllableDdyils': '\xeb\x9d\x90',
'UnicodeCharacter0xd7e0': '\xed\x9f\xa0',
'UnicodeCharacter0xabe5': '\xea\xaf\xa5',
'CjkUnifiedIdeograph639d': '\xe6\x8e\x9d',
'CjkUnifiedIdeograph7972': '\xe7\xa5\xb2',
'ArabicLigatureKafWithAlefMaksuraFinalForm': '\xef\xb2\x83',
'UnicodeCharacter0xeb7f': '\xee\xad\xbf',
'HangulSyllableBbwat': '\xeb\xbd\xb5',
'CjkUnifiedIdeograph5dbb': '\xe5\xb6\xbb',
'HangulSyllableKwaelg': '\xec\xbe\x95',
'RejangVowelSignU': '\xea\xa5\x88',
'HangulSyllableKwaelb': '\xec\xbe\x97',
'ArmenianCapitalLetterXeh': '\xd4\xbd',
'HexagramForContemplation': '\xe4\xb7\x93',
'ArmenianCapitalLetterYi': '\xd5\x85',
'HangulSyllablePyegs': '\xed\x8f\x93',
'HangulSyllableKwaelh': '\xec\xbe\x9b',
'CjkCompatibilityIdeographf913': '\xef\xa4\x93',
'CircledNumberThirtySix': '\xe3\x8a\xb1',
'RejangVowelSignE': '\xea\xa5\x89',
'ArabicLetterHahWithTwoDotsAbove': '\xdd\x97',
'HangulSyllableKwaelp': '\xec\xbe\x9a',
'RejangVowelSignI': '\xea\xa5\x87',
'SundaneseLetterSa': '\xe1\xae\x9e',
'HangulSyllablePyegg': '\xed\x8f\x92',
'SaurashtraLetterBha': '\xea\xa2\xa9',
'HangulSyllableMwiss': '\xeb\xae\x9c',
'UnicodeCharacter0xf243': '\xef\x89\x83',
'HiraganaVoicedIterationMark': '\xe3\x82\x9e',
'TifinaghLetterTuaregYang': '\xe2\xb5\x91',
'HangulSyllableBbyek': '\xeb\xbc\xbc',
'HangulSyllableDdeobs': '\xeb\x96\xb2',
'NotNormalSubgroupOf': '\xe2\x8b\xaa',
'UnicodeCharacter0x2e9a': '\xe2\xba\x9a',
'ArabicLigatureSheenWithMeemIsolatedForm': '\xef\xb4\x8c',
'UnicodeCharacter0xe1db': '\xee\x87\x9b',
'UnicodeCharacter0xda42': '\xed\xa9\x82',
'CjkUnifiedIdeograph71a7': '\xe7\x86\xa7',
'CjkUnifiedIdeograph8724': '\xe8\x9c\xa4',
'ArabicLetterHehDoachashmee': '\xda\xbe',
'EqualsSignAboveTildeOperator': '\xe2\xa9\xb3',
'LatinSmallLetterMiddlewelshV': '\xe1\xbb\xbd',
'EraseToTheLeft': '\xe2\x8c\xab',
'GlagoliticSmallLetterTsi': '\xe2\xb1\x8c',
'CjkUnifiedIdeograph6fe9': '\xe6\xbf\xa9',
'CjkUnifiedIdeograph6fe8': '\xe6\xbf\xa8',
'CjkUnifiedIdeograph6fe5': '\xe6\xbf\xa5',
'CjkUnifiedIdeograph6fe4': '\xe6\xbf\xa4',
'CjkUnifiedIdeograph6fe7': '\xe6\xbf\xa7',
'CjkUnifiedIdeograph6fe6': '\xe6\xbf\xa6',
'CjkUnifiedIdeograph6fe1': '\xe6\xbf\xa1',
'CjkUnifiedIdeograph6fe0': '\xe6\xbf\xa0',
'CjkUnifiedIdeograph6fe3': '\xe6\xbf\xa3',
'CjkUnifiedIdeograph6fe2': '\xe6\xbf\xa2',
'UnicodeCharacter0xd893': '\xed\xa2\x93',
'UnicodeCharacter0xf76e': '\xef\x9d\xae',
'EthiopicSyllableCee': '\xe1\x89\xbc',
'UnicodeCharacter0xec3e': '\xee\xb0\xbe',
'UnicodeCharacter0xf540': '\xef\x95\x80',
'HangulSyllableKelp': '\xec\xbc\x8e',
'HangulSyllableKels': '\xec\xbc\x8c',
'HangulSyllableSsyih': '\xec\x94\xa7',
'HangulSyllableKelt': '\xec\xbc\x8d',
'UnicodeCharacter0xd892': '\xed\xa2\x92',
'UpwardsDoubleArrow': '\xe2\x87\x91',
'CjkUnifiedIdeograph8339': '\xe8\x8c\xb9',
'HangulSyllableKelm': '\xec\xbc\x8a',
'CjkUnifiedIdeograph8330': '\xe8\x8c\xb0',
'CjkUnifiedIdeograph8331': '\xe8\x8c\xb1',
'CjkUnifiedIdeograph8332': '\xe8\x8c\xb2',
'CjkUnifiedIdeograph8333': '\xe8\x8c\xb3',
'CjkUnifiedIdeograph8334': '\xe8\x8c\xb4',
'CjkUnifiedIdeograph8335': '\xe8\x8c\xb5',
'CjkUnifiedIdeograph8336': '\xe8\x8c\xb6',
'CjkUnifiedIdeograph8337': '\xe8\x8c\xb7',
'InformationSource': '\xe2\x84\xb9',
'UnicodeCharacter0xdfab': '\xed\xbe\xab',
'TeluguLetterCa': '\xe0\xb0\x9a',
'CjkUnifiedIdeograph6fee': '\xe6\xbf\xae',
'CjkUnifiedIdeograph6fed': '\xe6\xbf\xad',
'UnicodeCharacter0xd7e7': '\xed\x9f\xa7',
'CjkUnifiedIdeograph6fef': '\xe6\xbf\xaf',
'CjkUnifiedIdeograph6fea': '\xe6\xbf\xaa',
'CjkUnifiedIdeograph6fec': '\xe6\xbf\xac',
'CjkUnifiedIdeograph6feb': '\xe6\xbf\xab',
'CjkUnifiedIdeograph8de9': '\xe8\xb7\xa9',
'CjkUnifiedIdeograph8de8': '\xe8\xb7\xa8',
'CjkUnifiedIdeograph8de1': '\xe8\xb7\xa1',
'CjkUnifiedIdeograph833a': '\xe8\x8c\xba',
'CjkUnifiedIdeograph833b': '\xe8\x8c\xbb',
'BlackHorizontalEllipse': '\xe2\xac\xac',
'CjkUnifiedIdeograph833d': '\xe8\x8c\xbd',
'CjkUnifiedIdeograph833e': '\xe8\x8c\xbe',
'CjkUnifiedIdeograph833f': '\xe8\x8c\xbf',
'CjkUnifiedIdeograph8de6': '\xe8\xb7\xa6',
'SquareWithDiagonalCrosshatchFill': '\xe2\x96\xa9',
'DingbatNegativeCircledDigitSeven': '\xe2\x9d\xbc',
'LimbuSignMukphreng': '\xe1\xa4\xb9',
'UnicodeCharacter0xec3f': '\xee\xb0\xbf',
'UpperRightDropshadowedWhiteSquare': '\xe2\x9d\x90',
'ArchaicKoppa': '\xcf\x98',
'CjkUnifiedIdeograph4909': '\xe4\xa4\x89',
'ArabicLetterYuIsolatedForm': '\xef\xaf\x9b',
'FourperemSpace': '\xe2\x80\x85',
'CjkStrokeHzwg': '\xe3\x87\x88',
'AplFunctionalSymbolCircleJot': '\xe2\x8c\xbe',
'CjkUnifiedIdeograph9d8d': '\xe9\xb6\x8d',
'LatinSmallLetterEshWithPalatalHook': '\xe1\xb6\x8b',
'CjkCompatibilityIdeographf9bf': '\xef\xa6\xbf',
'OutlinedGreekCross': '\xe2\x9c\x99',
'HangulSyllableHwelb': '\xed\x9c\x87',
'RightDoubleAngleBracket': '\xe3\x80\x8b',
'HangulSyllableHwelh': '\xed\x9c\x8b',
'HangulSyllableHwelm': '\xed\x9c\x86',
'YiSyllableNox': '\xea\x86\x8b',
'CjkUnifiedIdeograph4fda': '\xe4\xbf\x9a',
'CjkUnifiedIdeograph4fdb': '\xe4\xbf\x9b',
'CjkUnifiedIdeograph4fdc': '\xe4\xbf\x9c',
'BoxDrawingsUpLightAndLeftDownHeavy': '\xe2\x94\xaa',
'CjkUnifiedIdeograph4fde': '\xe4\xbf\x9e',
'CjkUnifiedIdeograph4fdf': '\xe4\xbf\x9f',
'YiSyllableNop': '\xea\x86\x8d',
'YiSyllableNot': '\xea\x86\x8a',
'HangulSyllableNyogg': '\xeb\x87\xaa',
'CjkCompatibilityIdeographf918': '\xef\xa4\x98',
'HangulSyllableJeung': '\xec\xa6\x9d',
'HangulSyllableJeunh': '\xec\xa6\x8e',
'HangulSyllableJeunj': '\xec\xa6\x8d',
'CjkUnifiedIdeograph9aff': '\xe9\xab\xbf',
'CjkUnifiedIdeograph4906': '\xe4\xa4\x86',
'FullwidthLatinSmallLetterW': '\xef\xbd\x97',
'HangulSyllableBwilp': '\xeb\xb7\xa2',
'CjkUnifiedIdeograph9d8f': '\xe9\xb6\x8f',
'LatinSmallLetterFWithPalatalHook': '\xe1\xb6\x82',
'HangulSyllableJjeb': '\xec\xa9\xb9',
'YiSyllableGguo': '\xea\x88\xa0',
'UnicodeCharacter0xe90f': '\xee\xa4\x8f',
'ArabicLetterGafIsolatedForm': '\xef\xae\x92',
'CjkCompatibilityIdeographf919': '\xef\xa4\x99',
'CjkUnifiedIdeograph4fd0': '\xe4\xbf\x90',
'CjkUnifiedIdeograph4fd1': '\xe4\xbf\x91',
'CjkUnifiedIdeograph4fd2': '\xe4\xbf\x92',
'CjkUnifiedIdeograph4fd3': '\xe4\xbf\x93',
'CjkUnifiedIdeograph4fd4': '\xe4\xbf\x94',
'CjkUnifiedIdeograph4fd5': '\xe4\xbf\x95',
'CjkUnifiedIdeograph4fd6': '\xe4\xbf\x96',
'CjkUnifiedIdeograph4fd7': '\xe4\xbf\x97',
'CjkUnifiedIdeograph4fd8': '\xe4\xbf\x98',
'CjkUnifiedIdeograph4fd9': '\xe4\xbf\x99',
'HangulSyllableSsweoj': '\xec\x92\x96',
'HangulSyllableSsweok': '\xec\x92\x98',
'HangulSyllableSsweol': '\xec\x92\x88',
'HangulSyllableSsweom': '\xec\x92\x90',
'VaiSyllableNi': '\xea\x95\x87',
'HangulSyllableSsweop': '\xec\x92\x9a',
'UnicodeCharacter0xec3b': '\xee\xb0\xbb',
'VaiSyllableNu': '\xea\x96\xb8',
'HangulSyllableSsweos': '\xec\x92\x93',
'HangulSyllableSsweot': '\xec\x92\x99',
'HangulSyllableKaels': '\xec\xba\x9c',
'HangulSyllableKaelp': '\xec\xba\x9e',
'HangulSyllableKaelt': '\xec\xba\x9d',
'HangulSyllableMyeh': '\xeb\xaa\xa7',
'HangulSyllableMyej': '\xeb\xaa\xa2',
'HangulSyllableGigg': '\xea\xb8\xb2',
'HangulSyllableMyel': '\xeb\xaa\x94',
'HangulSyllableMyem': '\xeb\xaa\x9c',
'HangulSyllableMyen': '\xeb\xaa\x90',
'RotatedCapitalQ': '\xe2\x84\xba',
'cb': '\xcf\x90',
'HangulSyllableMyeb': '\xeb\xaa\x9d',
'HangulSyllableMyec': '\xeb\xaa\xa3',
'MalayalamLetterCa': '\xe0\xb4\x9a',
'CjkUnifiedIdeograph3834': '\xe3\xa0\xb4',
'ce': '\xce\xb5',
'HangulSyllableMyeg': '\xeb\xaa\x8d',
'CjkUnifiedIdeograph3835': '\xe3\xa0\xb5',
'HangulSyllableGigs': '\xea\xb8\xb3',
'HangulSyllableMyep': '\xeb\xaa\xa6',
'HangulSyllableCaelg': '\xec\xb1\x8d',
'HangulJongseongTikeut': '\xe1\x86\xae',
'HangulSyllableMyes': '\xeb\xaa\x9f',
'HangulSyllableMyet': '\xeb\xaa\xa5',
'UnicodeCharacter0xe20d': '\xee\x88\x8d',
'TieOverInfinity': '\xe2\xa7\x9d',
'LeftSubstitutionBracket': '\xe2\xb8\x82',
'YiSyllablePat': '\xea\x80\xbf',
'HangulSyllableJyeobs': '\xec\xa1\x8a',
'YiSyllablePap': '\xea\x81\x82',
'>->': '\xe2\x86\xa3',
'UnicodeCharacter0x31b8': '\xe3\x86\xb8',
'UnicodeCharacter0x31b9': '\xe3\x86\xb9',
'YiSyllablePax': '\xea\x81\x80',
'c+': '\xe2\x8a\x95',
'c*': '\xe2\x8a\x97',
'HangulSyllableCaelh': '\xec\xb1\x93',
'KannadaVowelSignU': '\xe0\xb3\x81',
'c.': '\xe2\x8a\x99',
'c-': '\xe2\x8a\x96',
'CjkUnifiedIdeograph8b96': '\xe8\xae\x96',
'UnicodeCharacter0xd866': '\xed\xa1\xa6',
'UnicodeCharacter0xd867': '\xed\xa1\xa7',
'UnicodeCharacter0xd864': '\xed\xa1\xa4',
'UppointingTriangleWithRightHalfBlack': '\xe2\x97\xae',
'ThaanaEybeyfili': '\xde\xad',
'CjkUnifiedIdeograph3838': '\xe3\xa0\xb8',
'UnicodeCharacter0xd860': '\xed\xa1\xa0',
'UnicodeCharacter0xd861': '\xed\xa1\xa1',
'KannadaVowelSignI': '\xe0\xb2\xbf',
'CjkUnifiedIdeograph8b93': '\xe8\xae\x93',
'CjkUnifiedIdeograph55a4': '\xe5\x96\xa4',
'StarOperator': '\xe2\x8b\x86',
'KannadaVowelSignO': '\xe0\xb3\x8a',
'UnicodeCharacter0xd869': '\xed\xa1\xa9',
'UnicodeCharacter0x31bb': '\xe3\x86\xbb',
'UnicodeCharacter0x31bc': '\xe3\x86\xbc',
'UnicodeCharacter0x31ba': '\xe3\x86\xba',
'UnicodeCharacter0x31bf': '\xe3\x86\xbf',
'UnicodeCharacter0x31bd': '\xe3\x86\xbd',
'UnicodeCharacter0x31be': '\xe3\x86\xbe',
'HangulSyllableNyunh': '\xeb\x89\xba',
'HangulSyllableNyunj': '\xeb\x89\xb9',
'CjkUnifiedIdeograph8b90': '\xe8\xae\x90',
'TibetanSubjoinedLetterCha': '\xe0\xbe\x96',
'NkoDigitThree': '\xdf\x83',
'CjkUnifiedIdeograph8b26': '\xe8\xac\xa6',
'DevanagariDanda': '\xe0\xa5\xa4',
'CjkUnifiedIdeograph8b24': '\xe8\xac\xa4',
'CjkUnifiedIdeograph8b25': '\xe8\xac\xa5',
'CjkUnifiedIdeograph8b22': '\xe8\xac\xa2',
'CjkUnifiedIdeograph8b23': '\xe8\xac\xa3',
'CjkUnifiedIdeograph8b20': '\xe8\xac\xa0',
'CjkUnifiedIdeograph8b21': '\xe8\xac\xa1',
'UnicodeCharacter0xe642': '\xee\x99\x82',
'ThaanaLetterSeenu': '\xde\x90',
'CjkUnifiedIdeograph8b28': '\xe8\xac\xa8',
'CjkUnifiedIdeograph8b29': '\xe8\xac\xa9',
'CjkUnifiedIdeograph85e8': '\xe8\x97\xa8',
'CjkUnifiedIdeograph85e9': '\xe8\x97\xa9',
'CjkUnifiedIdeograph85e4': '\xe8\x97\xa4',
'CjkUnifiedIdeograph85e5': '\xe8\x97\xa5',
'CjkUnifiedIdeograph85e6': '\xe8\x97\xa6',
'CjkUnifiedIdeograph85e7': '\xe8\x97\xa7',
'CjkUnifiedIdeograph85e0': '\xe8\x97\xa0',
'CjkUnifiedIdeograph85e1': '\xe8\x97\xa1',
'LaoLetterNo': '\xe0\xba\x99',
'CjkUnifiedIdeograph85e3': '\xe8\x97\xa3',
'DownwardsWhiteArrow': '\xe2\x87\xa9',
'UnicodeCharacter0xe7ed': '\xee\x9f\xad',
'[6]': '\xe2\x82\x86',
'HangulSyllableBbibs': '\xec\x82\xa2',
'CyrillicSmallLetterTse': '\xd1\x86',
'UnicodeCharacter0xe7ec': '\xee\x9f\xac',
'CjkUnifiedIdeograph85ed': '\xe8\x97\xad',
'CjkUnifiedIdeograph85ee': '\xe8\x97\xae',
'CjkUnifiedIdeograph85ef': '\xe8\x97\xaf',
'ArabicLigatureQafWithAlefMaksuraIsolatedForm': '\xef\xb0\xb5',
'CjkUnifiedIdeograph85ea': '\xe8\x97\xaa',
'CjkUnifiedIdeograph85eb': '\xe8\x97\xab',
'CjkUnifiedIdeograph85ec': '\xe8\x97\xac',
'CjkUnifiedIdeograph8b2f': '\xe8\xac\xaf',
'CjkUnifiedIdeograph8b2d': '\xe8\xac\xad',
'CjkUnifiedIdeograph8b2e': '\xe8\xac\xae',
'CjkUnifiedIdeograph8b2b': '\xe8\xac\xab',
'CjkUnifiedIdeograph8b2c': '\xe8\xac\xac',
'CjkUnifiedIdeograph8b2a': '\xe8\xac\xaa',
'HangulSyllableDdyelh': '\xeb\x98\x83',
'ArabicLigatureSeenWithHahIsolatedForm': '\xef\xb0\x9d',
'CjkUnifiedIdeograph7ea3': '\xe7\xba\xa3',
'GreekSmallLetterOmegaWithPsiliAndOxiaAndYpogegrammeni': '\xe1\xbe\xa4',
'LessthanOrApproximate': '\xe2\xaa\x85',
'UnicodeCharacter0xe7eb': '\xee\x9f\xab',
'UnicodeCharacter0xdfa8': '\xed\xbe\xa8',
'HangulSyllableHyaen': '\xed\x96\xb0',
'UnicodeCharacter0xe569': '\xee\x95\xa9',
'ArabicLetterTtehehMedialForm': '\xef\xad\xa1',
'CjkUnifiedIdeograph9b6a': '\xe9\xad\xaa',
'HangulSyllableKya': '\xec\xba\xac',
'CjkUnifiedIdeograph9b6c': '\xe9\xad\xac',
'CjkUnifiedIdeograph9b6b': '\xe9\xad\xab',
'CjkUnifiedIdeograph9b6e': '\xe9\xad\xae',
'CjkUnifiedIdeograph9b6d': '\xe9\xad\xad',
'UnicodeCharacter0xdacf': '\xed\xab\x8f',
'CjkUnifiedIdeograph9b6f': '\xe9\xad\xaf',
'HangulSyllableKyi': '\xed\x82\x88',
'HangulSyllableKyo': '\xec\xbf\x84',
'UnicodeCharacter0xecdc': '\xee\xb3\x9c',
'CjkUnifiedIdeograph7453': '\xe7\x91\x93',
'UnicodeCharacter0xab6a': '\xea\xad\xaa',
'HangulSyllableSswinh': '\xec\x92\xbe',
'HangulSyllableSswing': '\xec\x93\x8d',
'ParenthesizedIdeographFestival': '\xe3\x89\x80',
'UnicodeCharacter0xeced': '\xee\xb3\xad',
'ArabicLigatureYehWithHamzaAboveWithEInitialForm': '\xef\xaf\xb8',
'UnicodeCharacter0x20fd': '\xe2\x83\xbd',
'UnicodeCharacter0x20fe': '\xe2\x83\xbe',
'CjkUnifiedIdeograph431d': '\xe4\x8c\x9d',
'CjkUnifiedIdeograph431e': '\xe4\x8c\x9e',
'CjkUnifiedIdeograph431b': '\xe4\x8c\x9b',
'UnicodeCharacter0x20fa': '\xe2\x83\xba',
'UnicodeCharacter0x20fb': '\xe2\x83\xbb',
'CjkUnifiedIdeograph431a': '\xe4\x8c\x9a',
'ArabicLetterTehehMedialForm': '\xef\xad\xa5',
'YiSyllableNrux': '\xea\x8e\xa0',
'CjkUnifiedIdeograph795a': '\xe7\xa5\x9a',
'CjkUnifiedIdeograph795c': '\xe7\xa5\x9c',
'CjkUnifiedIdeograph795b': '\xe7\xa5\x9b',
'CjkUnifiedIdeograph795e': '\xe7\xa5\x9e',
'CjkUnifiedIdeograph795d': '\xe7\xa5\x9d',
'CjkUnifiedIdeograph795f': '\xe7\xa5\x9f',
'UnicodeCharacter0xf6ee': '\xef\x9b\xae',
'CjkUnifiedIdeograph9b61': '\xe9\xad\xa1',
'CjkUnifiedIdeograph9b60': '\xe9\xad\xa0',
'LatinSmallLetterQWithHookTail': '\xc9\x8b',
'CjkUnifiedIdeograph9b62': '\xe9\xad\xa2',
'CjkUnifiedIdeograph9b65': '\xe9\xad\xa5',
'CjkUnifiedIdeograph9b64': '\xe9\xad\xa4',
'CjkUnifiedIdeograph9b67': '\xe9\xad\xa7',
'CjkUnifiedIdeograph9b66': '\xe9\xad\xa6',
'CjkUnifiedIdeograph9b69': '\xe9\xad\xa9',
'CjkUnifiedIdeograph9b68': '\xe9\xad\xa8',
'LaoDigitZero': '\xe0\xbb\x90',
'GujaratiVowelSignVocalicRr': '\xe0\xab\x84',
'CjkUnifiedIdeograph7958': '\xe7\xa5\x98',
'UnicodeCharacter0xec03': '\xee\xb0\x83',
'TagbanwaLetterNa': '\xe1\x9d\xa8',
'CjkUnifiedIdeograph7951': '\xe7\xa5\x91',
'CjkUnifiedIdeograph7950': '\xe7\xa5\x90',
'CjkUnifiedIdeograph7953': '\xe7\xa5\x93',
'CjkUnifiedIdeograph7952': '\xe7\xa5\x92',
'CjkUnifiedIdeograph7955': '\xe7\xa5\x95',
'CjkUnifiedIdeograph7954': '\xe7\xa5\x94',
'CjkUnifiedIdeograph7957': '\xe7\xa5\x97',
'CjkUnifiedIdeograph7956': '\xe7\xa5\x96',
'HangulSyllableHyuss': '\xed\x9d\x88',
'UnicodeCharacter0x20f8': '\xe2\x83\xb8',
'UnicodeCharacter0x20f9': '\xe2\x83\xb9',
'CjkUnifiedIdeograph4318': '\xe4\x8c\x98',
'CjkUnifiedIdeograph4319': '\xe4\x8c\x99',
'UnicodeCharacter0x20f4': '\xe2\x83\xb4',
'CjkUnifiedIdeograph4317': '\xe4\x8c\x97',
'CjkUnifiedIdeograph4314': '\xe4\x8c\x94',
'UnicodeCharacter0x20f7': '\xe2\x83\xb7',
'CjkUnifiedIdeograph4312': '\xe4\x8c\x92',
'CjkUnifiedIdeograph4313': '\xe4\x8c\x93',
'CjkUnifiedIdeograph4310': '\xe4\x8c\x90',
'UnicodeCharacter0x20f3': '\xe2\x83\xb3',
'HangulSyllableReugs': '\xeb\xa5\xb7',
'HangulChoseongIeungpansios': '\xe1\x85\x86',
'HangulSyllableYugs': '\xec\x9c\xa3',
'HangulSyllableDdwaebs': '\xeb\x99\x9a',
'HangulSyllableYugg': '\xec\x9c\xa2',
'UnicodeCharacter0xe71b': '\xee\x9c\x9b',
'GreekSmallLetterEtaWithVaria': '\xe1\xbd\xb4',
'UnicodeCharacter0xf1af': '\xef\x86\xaf',
'SinhalaVowelSignAelapilla': '\xe0\xb7\x8f',
'SinhalaLetterAlpapraanaTayanna': '\xe0\xb6\xad',
'GeorgianLetterChin': '\xe1\x83\xa9',
'UnicodeCharacter0xf1aa': '\xef\x86\xaa',
'ArmenianCapitalLetterDa': '\xd4\xb4',
'VaiSymbolJong': '\xea\x98\x9f',
'HangulSyllableKoej': '\xec\xbe\xbe',
'ArabicLetterKehehWithDotAbove': '\xdd\xa2',
'UnicodeCharacter0xe71d': '\xee\x9c\x9d',
'HangulSyllableHyeom': '\xed\x98\x90',
'CanadianSyllabicsFoo': '\xe1\x95\x98',
'HangulSyllableHyeol': '\xed\x98\x88',
'CjkUnifiedIdeograph9cf7': '\xe9\xb3\xb7',
'CjkUnifiedIdeograph8a23': '\xe8\xa8\xa3',
'DownwardsHarpoonWithBarbLeftBesideUpwardsHarpoonWithBarbRight': '\xe2\xa5\xaf',
'CjkUnifiedIdeograph8a22': '\xe8\xa8\xa2',
'MongolianLetterManchuKa': '\xe1\xa1\xb4',
'HangulSyllableBbweoss': '\xeb\xbf\xbc',
'HangulSyllableYaess': '\xec\x96\xac',
'CircledLatinSmallLetterI': '\xe2\x93\x98',
'UnicodeCharacter0xdc0a': '\xed\xb0\x8a',
'UnicodeCharacter0xe909': '\xee\xa4\x89',
'HangulSyllableHyeok': '\xed\x98\x98',
'UnicodeCharacter0xe200': '\xee\x88\x80',
'ArabicSignTakhallus': '\xd8\x94',
'CircledLatinSmallLetterK': '\xe2\x93\x9a',
'CjkUnifiedIdeograph8a29': '\xe8\xa8\xa9',
'CircledLatinSmallLetterJ': '\xe2\x93\x99',
'CjkUnifiedIdeograph8a28': '\xe8\xa8\xa8',
'CircledLatinSmallLetterM': '\xe2\x93\x9c',
'ThaiCharacterMaiyamok': '\xe0\xb9\x86',
'LogicalOrOverlappingLogicalAnd': '\xe2\xa9\x99',
'CjkUnifiedIdeograph383c': '\xe3\xa0\xbc',
'KangxiRadicalDottedCliff': '\xe2\xbc\xb4',
'CjkUnifiedIdeograph806c': '\xe8\x81\xac',
'EqualToOrPrecedes': '\xe2\x8b\x9e',
'CjkUnifiedIdeograph68ff': '\xe6\xa3\xbf',
'CjkUnifiedIdeograph9f76': '\xe9\xbd\xb6',
'CjkUnifiedIdeograph68fd': '\xe6\xa3\xbd',
'CjkUnifiedIdeograph383a': '\xe3\xa0\xba',
'CjkUnifiedIdeograph68fb': '\xe6\xa3\xbb',
'CjkUnifiedIdeograph68fc': '\xe6\xa3\xbc',
'CjkUnifiedIdeograph68fa': '\xe6\xa3\xba',
'Virgo': '\xe2\x99\x8d',
'CircledLatinSmallLetterA': '\xe2\x93\x90',
'CjkUnifiedIdeograph9f74': '\xe9\xbd\xb4',
'CjkUnifiedIdeograph927f': '\xe9\x89\xbf',
'RightwardsTwoheadedArrowWithTailWithDoubleVerticalStroke': '\xe2\xa4\x98',
'CjkUnifiedIdeograph927e': '\xe9\x89\xbe',
'CjkUnifiedIdeograph927b': '\xe9\x89\xbb',
'CjkUnifiedIdeograph55aa': '\xe5\x96\xaa',
'CjkUnifiedIdeograph927a': '\xe9\x89\xba',
'UnicodeCharacter0xdfc3': '\xed\xbf\x83',
'YiSyllableKi': '\xea\x87\xba',
'UnicodeCharacter0xdfc1': '\xed\xbf\x81',
'UnicodeCharacter0xdfc0': '\xed\xbf\x80',
'LatinSmallLetterTWithCaron': '\xc5\xa5',
'CjkUnifiedIdeograph9afb': '\xe9\xab\xbb',
'UnicodeCharacter0xdfc5': '\xed\xbf\x85',
'UnicodeCharacter0xdfc4': '\xed\xbf\x84',
'ArabicLetterQafInitialForm': '\xef\xbb\x97',
'UnicodeCharacter0xdfc9': '\xed\xbf\x89',
'UnicodeCharacter0xdfc8': '\xed\xbf\x88',
'UnicodeCharacter0xe819': '\xee\xa0\x99',
'ArabicLetterUIsolatedForm': '\xef\xaf\x97',
'CircledLatinSmallLetterD': '\xe2\x93\x93',
'HangulSyllableGyibs': '\xea\xb8\xa6',
'CircledLatinSmallLetterG': '\xe2\x93\x96',
'CircledLatinSmallLetterF': '\xe2\x93\x95',
'PhagspaSubjoinedLetterWa': '\xea\xa1\xa7',
'CjkUnifiedIdeograph9278': '\xe9\x89\xb8',
'CjkUnifiedIdeograph9279': '\xe9\x89\xb9',
'CjkUnifiedIdeograph9276': '\xe9\x89\xb6',
'CjkUnifiedIdeograph9277': '\xe9\x89\xb7',
'CjkUnifiedIdeograph9274': '\xe9\x89\xb4',
'CjkUnifiedIdeograph9275': '\xe9\x89\xb5',
'CjkUnifiedIdeograph9272': '\xe9\x89\xb2',
'CjkUnifiedIdeograph9273': '\xe9\x89\xb3',
'CjkUnifiedIdeograph9270': '\xe9\x89\xb0',
'CjkUnifiedIdeograph9271': '\xe9\x89\xb1',
'HangulSyllableJeon': '\xec\xa0\x84',
'HangulSyllableJeom': '\xec\xa0\x90',
'HangulSyllableJeol': '\xec\xa0\x88',
'HangulSyllableJeok': '\xec\xa0\x98',
'HangulSyllableJeoj': '\xec\xa0\x96',
'CjkUnifiedIdeograph68f8': '\xe6\xa3\xb8',
'CjkUnifiedIdeograph68f9': '\xe6\xa3\xb9',
'CjkUnifiedIdeograph68f6': '\xe6\xa3\xb6',
'NewTaiLueLetterLowLa': '\xe1\xa6\x9f',
'CjkUnifiedIdeograph68f4': '\xe6\xa3\xb4',
'CjkUnifiedIdeograph68f5': '\xe6\xa3\xb5',
'CjkUnifiedIdeograph68f2': '\xe6\xa3\xb2',
'CjkUnifiedIdeograph68f3': '\xe6\xa3\xb3',
'CjkUnifiedIdeograph68f0': '\xe6\xa3\xb0',
'CjkUnifiedIdeograph68f1': '\xe6\xa3\xb1',
'CjkUnifiedIdeograph925d': '\xe9\x89\x9d',
'CjkUnifiedIdeograph925e': '\xe9\x89\x9e',
'CyrillicSmallLetterTeWithMiddleHook': '\xea\x9a\x8b',
'HangulSyllableHyeop': '\xed\x98\x9a',
'UnicodeCharacter0xdfcc': '\xed\xbf\x8c',
'UnicodeCharacter0xdfcb': '\xed\xbf\x8b',
'UnicodeCharacter0xdfca': '\xed\xbf\x8a',
'CjkUnifiedIdeograph620a': '\xe6\x88\x8a',
'UnicodeCharacter0xdfcf': '\xed\xbf\x8f',
'UnicodeCharacter0xdfce': '\xed\xbf\x8e',
'UnicodeCharacter0xdfcd': '\xed\xbf\x8d',
'ArabicLigatureQafWithMeemIsolatedForm': '\xef\xb0\xb4',
'UnicodeCharacter0xec0b': '\xee\xb0\x8b',
'HangulSyllableBwah': '\xeb\xb4\xab',
'CjkUnifiedIdeograph925b': '\xe9\x89\x9b',
'UnicodeCharacter0xec0d': '\xee\xb0\x8d',
'CjkUnifiedIdeograph925c': '\xe9\x89\x9c',
'UnicodeCharacter0xe719': '\xee\x9c\x99',
'CjkCompatibilityIdeographf9d2': '\xef\xa7\x92',
'YiSyllableYuox': '\xea\x91\xba',
'UnicodeCharacter0xec0f': '\xee\xb0\x8f',
'UnicodeCharacter0xf1a8': '\xef\x86\xa8',
'YiSyllableYuop': '\xea\x91\xbc',
'YiSyllableYuot': '\xea\x91\xb9',
'UnicodeCharacter0x87f': '\xe0\xa1\xbf',
'CjkUnifiedIdeograph9cf5': '\xe9\xb3\xb5',
'UnicodeCharacter0x87d': '\xe0\xa1\xbd',
'UnicodeCharacter0x87e': '\xe0\xa1\xbe',
'GeorgianLetterNar': '\xe1\x83\x9c',
'UnicodeCharacter0x87c': '\xe0\xa1\xbc',
'CjkUnifiedIdeograph9cf2': '\xe9\xb3\xb2',
'UnicodeCharacter0x87a': '\xe0\xa1\xba',
'CjkUnifiedIdeograph713c': '\xe7\x84\xbc',
'CjkUnifiedIdeograph713b': '\xe7\x84\xbb',
'CjkUnifiedIdeograph713a': '\xe7\x84\xba',
'CjkUnifiedIdeograph9cf8': '\xe9\xb3\xb8',
'CjkUnifiedIdeograph713f': '\xe7\x84\xbf',
'CjkUnifiedIdeograph713e': '\xe7\x84\xbe',
'CjkUnifiedIdeograph713d': '\xe7\x84\xbd',
'UnicodeCharacter0xe713': '\xee\x9c\x93',
'ArabicLetterTchehIsolatedForm': '\xef\xad\xba',
'HangulSyllableTalg': '\xed\x83\x89',
'HangulSyllableTalb': '\xed\x83\x8b',
'UnicodeCharacter0xe712': '\xee\x9c\x92',
'HangulSyllableTalm': '\xed\x83\x8a',
'UnicodeCharacter0xec9e': '\xee\xb2\x9e',
'UnicodeCharacter0xec9d': '\xee\xb2\x9d',
'HangulSyllableJyagg': '\xec\x9f\x8a',
'CanadianSyllabicsNaskapiYwaa': '\xe1\x94\xbd',
'NkoLetterPa': '\xdf\x94',
'UnicodeCharacter0xe710': '\xee\x9c\x90',
'ArabicLetterGafMedialForm': '\xef\xae\x95',
'UnicodeCharacter0xe717': '\xee\x9c\x97',
'HangulSyllableNogs': '\xeb\x85\xbb',
'CjkUnifiedIdeograph84d4': '\xe8\x93\x94',
'UnicodeCharacter0xe716': '\xee\x9c\x96',
'UnicodeCharacter0xe4ee': '\xee\x93\xae',
'CjkUnifiedIdeograph84d5': '\xe8\x93\x95',
'UnicodeCharacter0xe715': '\xee\x9c\x95',
'LatinSmallLetterLum': '\xea\x9d\xb2',
'GeorgianLetterZhar': '\xe1\x83\x9f',
'KangxiRadicalCow': '\xe2\xbd\x9c',
'CjkUnifiedIdeograph84d0': '\xe8\x93\x90',
'UnicodeCharacter0xec99': '\xee\xb2\x99',
'UnicodeCharacter0xec98': '\xee\xb2\x98',
'EthiopicSyllableHu': '\xe1\x88\x81',
'UnicodeCharacter0xec96': '\xee\xb2\x96',
'UnicodeCharacter0xec95': '\xee\xb2\x95',
'HangulSyllableBbyeb': '\xeb\xbc\xb5',
'UnicodeCharacter0xec93': '\xee\xb2\x93',
'CjkUnifiedIdeograph745b': '\xe7\x91\x9b',
'UnicodeCharacter0xec91': '\xee\xb2\x91',
'HangulSyllableDweng': '\xeb\x92\x9d',
'UnicodeCharacter0x876': '\xe0\xa1\xb6',
'UnicodeCharacter0x877': '\xe0\xa1\xb7',
'EthiopicSyllableHo': '\xe1\x88\x86',
'UnicodeCharacter0x875': '\xe0\xa1\xb5',
'EthiopicSyllableHi': '\xe1\x88\x82',
'UnicodeCharacter0x873': '\xe0\xa1\xb3',
'UnicodeCharacter0x870': '\xe0\xa1\xb0',
'UnicodeCharacter0x871': '\xe0\xa1\xb1',
'EthiopicSyllableHe': '\xe1\x88\x85',
'CjkUnifiedIdeograph7132': '\xe7\x84\xb2',
'CjkUnifiedIdeograph7131': '\xe7\x84\xb1',
'CjkUnifiedIdeograph7130': '\xe7\x84\xb0',
'EthiopicSyllableHa': '\xe1\x88\x80',
'CjkUnifiedIdeograph7136': '\xe7\x84\xb6',
'UnicodeCharacter0x878': '\xe0\xa1\xb8',
'UnicodeCharacter0x879': '\xe0\xa1\xb9',
'HangulSyllableMweolg': '\xeb\xad\x99',
'HangulSyllableMweolb': '\xeb\xad\x9b',
'BoxDrawingsUpHeavyAndHorizontalLight': '\xe2\x94\xb8',
'HangulSyllableMweolm': '\xeb\xad\x9a',
'UnicodeCharacter0xdef1': '\xed\xbb\xb1',
'HangulSyllableBbwaenj': '\xeb\xbd\xbd',
'HangulLetterSiospieup': '\xe3\x85\xbd',
'HangulSyllableMweolt': '\xeb\xad\x9d',
'HangulSyllableMweols': '\xeb\xad\x9c',
'CjkUnifiedIdeograph84d8': '\xe8\x93\x98',
'HangulSyllableMweolp': '\xeb\xad\x9e',
'SquarePaasento': '\xe3\x8c\xab',
'CjkUnifiedIdeograph84d9': '\xe8\x93\x99',
'UnicodeCharacter0xef49': '\xee\xbd\x89',
'UnicodeCharacter0xef48': '\xee\xbd\x88',
'InhibitArabicFormShaping': '\xe2\x81\xac',
'YiSyllableTuo': '\xea\x84\xa7',
'UnicodeCharacter0xef41': '\xee\xbd\x81',
'UnicodeCharacter0xef40': '\xee\xbd\x80',
'UnicodeCharacter0xef43': '\xee\xbd\x83',
'HebrewAccentGeresh': '\xd6\x9c',
'Watch': '\xe2\x8c\x9a',
'UnicodeCharacter0xef44': '\xee\xbd\x84',
'UnicodeCharacter0xef47': '\xee\xbd\x87',
'UnicodeCharacter0xef46': '\xee\xbd\x86',
'SylotiNagriLetterCo': '\xea\xa0\x8c',
'BengaliLetterAi': '\xe0\xa6\x90',
'HebrewAccentIluy': '\xd6\xac',
'CjkCompatibilityIdeographfa21': '\xef\xa8\xa1',
'BengaliLetterAa': '\xe0\xa6\x86',
'HangulSyllableSseulg': '\xec\x93\xb9',
'UnicodeCharacter0xec44': '\xee\xb1\x84',
'CjkCompatibilityIdeographfa20': '\xef\xa8\xa0',
'HangulSyllableSseulb': '\xec\x93\xbb',
'UnicodeCharacter0xe4ea': '\xee\x93\xaa',
'HangulSyllableSseulm': '\xec\x93\xba',
'CanadianSyllabicsCarrierJjee': '\xe1\x98\x9f',
'TibetanSubjoinedLetterZha': '\xe0\xbe\xae',
'HangulSyllableSseulh': '\xec\x93\xbf',
'ScriptSmallO': '\xe2\x84\xb4',
'ScriptSmallL': '\xe2\x84\x93',
'HangulSyllableSseult': '\xec\x93\xbd',
'HangulSyllableSseuls': '\xec\x93\xbc',
'TurnedNotSign': '\xe2\x8c\x99',
'HangulSyllableSseulp': '\xec\x93\xbe',
'HangulSyllableGeobs': '\xea\xb2\x82',
'ScriptSmallG': '\xe2\x84\x8a',
'ScriptSmallE': '\xe2\x84\xaf',
'ThinSpace': '\xe2\x80\x89',
'SlantedEqualToOrLessthanWithDotInside': '\xe2\xaa\x97',
'ArabicLigatureSadWithHahInitialForm': '\xef\xb2\xb1',
'UnicodeCharacter0xe4eb': '\xee\x93\xab',
'ArabicLetterZahIsolatedForm': '\xef\xbb\x85',
'HangulSyllableNyolb': '\xeb\x87\xb3',
'CjkUnifiedIdeograph9056': '\xe9\x81\x96',
'UnicodeCharacter0xf75f': '\xef\x9d\x9f',
'KhmerSignKoomuut': '\xe1\x9f\x9a',
'UnicodeCharacter0xef4a': '\xee\xbd\x8a',
'UnicodeCharacter0xef4c': '\xee\xbd\x8c',
'YiRadicalPut': '\xea\x92\x9e',
'UnicodeCharacter0xef4e': '\xee\xbd\x8e',
'UnicodeCharacter0xef4d': '\xee\xbd\x8d',
'UnicodeCharacter0xef4f': '\xee\xbd\x8f',
'CjkUnifiedIdeograph6202': '\xe6\x88\x82',
'HangulSyllableJjwibs': '\xec\xae\xae',
'UnicodeCharacter0xe4ec': '\xee\x93\xac',
'SquareSiringu': '\xe3\x8c\xa1',
'CjkUnifiedIdeograph9256': '\xe9\x89\x96',
'CjkUnifiedIdeograph86ce': '\xe8\x9b\x8e',
'CjkUnifiedIdeograph86cd': '\xe8\x9b\x8d',
'GreekSmallLetterUpsilonWithPsiliAndVaria': '\xe1\xbd\x92',
'CjkUnifiedIdeograph86cf': '\xe8\x9b\x8f',
'CjkUnifiedIdeograph86ca': '\xe8\x9b\x8a',
'CjkUnifiedIdeograph86cc': '\xe8\x9b\x8c',
'CjkUnifiedIdeograph86cb': '\xe8\x9b\x8b',
'GreekCapitalLetterEtaWithPsiliAndOxiaAndProsgegrammeni': '\xe1\xbe\x9c',
'CjkUnifiedIdeograph6206': '\xe6\x88\x86',
'IdeographicTelegraphSymbolForDayTwentyseven': '\xe3\x8f\xba',
'CjkUnifiedIdeograph6a5f': '\xe6\xa9\x9f',
'CjkUnifiedIdeograph6a5d': '\xe6\xa9\x9d',
'CjkUnifiedIdeograph6207': '\xe6\x88\x87',
'DigramForGreaterYin': '\xe2\x9a\x8f',
'CjkUnifiedIdeograph8a2a': '\xe8\xa8\xaa',
'CjkUnifiedIdeograph9252': '\xe9\x89\x92',
'ImageOf': '\xe2\x8a\xb7',
'HangulSyllableUlg': '\xec\x9a\xb9',
'CjkUnifiedIdeograph9253': '\xe9\x89\x93',
'HangulSyllableUlb': '\xec\x9a\xbb',
'HangulSyllableHyaess': '\xed\x97\x80',
'CjkUnifiedIdeograph8a2b': '\xe8\xa8\xab',
'UnicodeCharacter0xdbc': '\xe0\xb6\xbc',
'HangulSyllableUlt': '\xec\x9a\xbd',
'HangulSyllableUlp': '\xec\x9a\xbe',
'HangulSyllableUls': '\xec\x9a\xbc',
'CjkUnifiedIdeograph86c5': '\xe8\x9b\x85',
'CjkUnifiedIdeograph86c4': '\xe8\x9b\x84',
'CjkUnifiedIdeograph6a58': '\xe6\xa9\x98',
'CjkUnifiedIdeograph6a59': '\xe6\xa9\x99',
'CjkUnifiedIdeograph86c1': '\xe8\x9b\x81',
'CjkUnifiedIdeograph86c0': '\xe8\x9b\x80',
'CjkUnifiedIdeograph86c3': '\xe8\x9b\x83',
'CjkUnifiedIdeograph86c2': '\xe8\x9b\x82',
'CjkUnifiedIdeograph6a52': '\xe6\xa9\x92',
'CjkUnifiedIdeograph6a53': '\xe6\xa9\x93',
'CjkUnifiedIdeograph6a50': '\xe6\xa9\x90',
'CjkUnifiedIdeograph6a51': '\xe6\xa9\x91',
'CjkUnifiedIdeograph6a56': '\xe6\xa9\x96',
'CjkUnifiedIdeograph6a57': '\xe6\xa9\x97',
'CjkUnifiedIdeograph6a54': '\xe6\xa9\x94',
'CjkUnifiedIdeograph6a55': '\xe6\xa9\x95',
'CjkUnifiedIdeograph9f7b': '\xe9\xbd\xbb',
'CjkUnifiedIdeograph9f7c': '\xe9\xbd\xbc',
'IdeographicDescriptionCharacterSurroundFromLowerLeft': '\xe2\xbf\xba',
'LimbuSmallLetterPa': '\xe1\xa4\xb5',
'LaoDigitThree': '\xe0\xbb\x93',
'HangulSyllableHoep': '\xed\x9a\xa6',
'HangulSyllableHoes': '\xed\x9a\x9f',
'GreekSmallLetterEtaWithPsiliAndPerispomeniAndYpogegrammeni': '\xe1\xbe\x96',
'CjkUnifiedIdeograph9f7f': '\xe9\xbd\xbf',
'CjkUnifiedIdeograph931e': '\xe9\x8c\x9e',
'ElementOfWithOverbar': '\xe2\x8b\xb6',
'HangulSyllableSsegg': '\xec\x8e\x86',
'CjkUnifiedIdeograph9f7d': '\xe9\xbd\xbd',
'CyrillicSmallLetterTeWithDescender': '\xd2\xad',
'CjkRadicalSmallOne': '\xe2\xba\x8c',
'CjkUnifiedIdeograph9f7e': '\xe9\xbd\xbe',
'MongolianLetterTodoE': '\xe1\xa1\x84',
'CjkUnifiedIdeograph3a24': '\xe3\xa8\xa4',
'CjkUnifiedIdeograph3a25': '\xe3\xa8\xa5',
'CjkUnifiedIdeograph3a26': '\xe3\xa8\xa6',
'CjkUnifiedIdeograph3a27': '\xe3\xa8\xa7',
'CjkUnifiedIdeograph3a20': '\xe3\xa8\xa0',
'CjkUnifiedIdeograph3a21': '\xe3\xa8\xa1',
'CjkUnifiedIdeograph3a22': '\xe3\xa8\xa2',
'CjkUnifiedIdeograph3a23': '\xe3\xa8\xa3',
'CjkUnifiedIdeograph73ef': '\xe7\x8f\xaf',
'CjkUnifiedIdeograph73ee': '\xe7\x8f\xae',
'CjkUnifiedIdeograph73ed': '\xe7\x8f\xad',
'CjkUnifiedIdeograph3a28': '\xe3\xa8\xa8',
'CjkUnifiedIdeograph3a29': '\xe3\xa8\xa9',
'CjkUnifiedIdeograph73ea': '\xe7\x8f\xaa',
'LeftSidewaysUBracket': '\xe2\xb8\xa6',
'BalineseLetterYa': '\xe1\xac\xac',
'CjkCompatibilityIdeographf91e': '\xef\xa4\x9e',
'CjkUnifiedIdeograph89e8': '\xe8\xa7\xa8',
'UnicodeCharacter0xf2af': '\xef\x8a\xaf',
'LatinSmallLetterTWithMiddleTilde': '\xe1\xb5\xb5',
'CjkCompatibilityIdeographfa1f': '\xef\xa8\x9f',
'UnicodeCharacter0xf2ac': '\xef\x8a\xac',
'DingbatNegativeCircledSansserifDigitFive': '\xe2\x9e\x8e',
'ArmenianSmallLetterHo': '\xd5\xb0',
'HangulSyllableHeonj': '\xed\x97\x8d',
'GeorgianCapitalLetterOn': '\xe1\x82\xad',
'UnicodeCharacter0xf2ab': '\xef\x8a\xab',
'CjkUnifiedIdeograph3a2d': '\xe3\xa8\xad',
'CjkUnifiedIdeograph3a2e': '\xe3\xa8\xae',
'CjkUnifiedIdeograph3a2f': '\xe3\xa8\xaf',
'CjkUnifiedIdeograph89c5': '\xe8\xa7\x85',
'CjkUnifiedIdeograph89c2': '\xe8\xa7\x82',
'CjkUnifiedIdeograph3a2a': '\xe3\xa8\xaa',
'CjkUnifiedIdeograph3a2b': '\xe3\xa8\xab',
'CjkUnifiedIdeograph3a2c': '\xe3\xa8\xac',
'CjkUnifiedIdeograph73e7': '\xe7\x8f\xa7',
'CjkUnifiedIdeograph73e6': '\xe7\x8f\xa6',
'CjkUnifiedIdeograph73e5': '\xe7\x8f\xa5',
'CjkUnifiedIdeograph73e4': '\xe7\x8f\xa4',
'CjkUnifiedIdeograph73e3': '\xe7\x8f\xa3',
'CjkUnifiedIdeograph73e2': '\xe7\x8f\xa2',
'CjkUnifiedIdeograph73e1': '\xe7\x8f\xa1',
'CjkUnifiedIdeograph73e0': '\xe7\x8f\xa0',
'CopticCapitalLetterShima': '\xcf\xac',
'HangulSyllableKonh': '\xec\xbd\x9a',
'HangulSyllableJoes': '\xec\xa3\x97',
'LepchaConsonantSignN': '\xe1\xb0\xb0',
'LepchaConsonantSignM': '\xe1\xb0\xae',
'LepchaConsonantSignL': '\xe1\xb0\xaf',
'LepchaConsonantSignK': '\xe1\xb0\xad',
'LepchaConsonantSignT': '\xe1\xb0\xb3',
'LepchaConsonantSignR': '\xe1\xb0\xb2',
'LepchaConsonantSignP': '\xe1\xb0\xb1',
'CjkCompatibilityIdeographfa1d': '\xef\xa8\x9d',
'NewTaiLueDigitFour': '\xe1\xa7\x94',
'NaryUnionOperatorWithPlus': '\xe2\xa8\x84',
'ParenthesizedIdeographOne': '\xe3\x88\xa0',
'CjkUnifiedIdeograph46c9': '\xe4\x9b\x89',
'CjkUnifiedIdeograph46c8': '\xe4\x9b\x88',
'HangulSyllableKonj': '\xec\xbd\x99',
'UnicodeCharacter0xd90a': '\xed\xa4\x8a',
'UnicodeCharacter0xe07b': '\xee\x81\xbb',
'CjkUnifiedIdeograph46c1': '\xe4\x9b\x81',
'CjkUnifiedIdeograph46c0': '\xe4\x9b\x80',
'CjkUnifiedIdeograph46c3': '\xe4\x9b\x83',
'CjkUnifiedIdeograph46c2': '\xe4\x9b\x82',
'CjkUnifiedIdeograph46c5': '\xe4\x9b\x85',
'CjkUnifiedIdeograph46c4': '\xe4\x9b\x84',
'CjkUnifiedIdeograph46c7': '\xe4\x9b\x87',
'CjkUnifiedIdeograph46c6': '\xe4\x9b\x86',
'GreekSmallLetterEtaWithPerispomeni': '\xe1\xbf\x86',
'HangulSyllableJyij': '\xec\xa6\xba',
'VaiSyllableZee': '\xea\x94\x98',
'OcrDoubleBackslash': '\xe2\x91\x8a',
'UnicodeCharacter0xf703': '\xef\x9c\x83',
'CjkUnifiedIdeograph84da': '\xe8\x93\x9a',
'HangulSyllablePaeng': '\xed\x8c\xbd',
'CjkUnifiedIdeograph6c0b': '\xe6\xb0\x8b',
'HangulSyllablePaenj': '\xed\x8c\xad',
'HangulSyllablePaenh': '\xed\x8c\xae',
'HangulSyllableYaelh': '\xec\x96\xa7',
'UpwardsArrowToBar': '\xe2\xa4\x92',
'CanadianSyllabicsNaskapiScwa': '\xe1\x94\x8f',
'CjkUnifiedIdeograph9294': '\xe9\x8a\x94',
'ArabicLetterTahInitialForm': '\xef\xbb\x83',
'CjkUnifiedIdeograph46ca': '\xe4\x9b\x8a',
'CjkUnifiedIdeograph46cc': '\xe4\x9b\x8c',
'CjkUnifiedIdeograph46cb': '\xe4\x9b\x8b',
'CjkUnifiedIdeograph46ce': '\xe4\x9b\x8e',
'CjkUnifiedIdeograph46cd': '\xe4\x9b\x8d',
'CjkUnifiedIdeograph46cf': '\xe4\x9b\x8f',
'CjkUnifiedIdeograph881a': '\xe8\xa0\x9a',
'CjkUnifiedIdeograph881c': '\xe8\xa0\x9c',
'CjkUnifiedIdeograph881b': '\xe8\xa0\x9b',
'CjkUnifiedIdeograph881e': '\xe8\xa0\x9e',
'CjkUnifiedIdeograph881d': '\xe8\xa0\x9d',
'CjkUnifiedIdeograph881f': '\xe8\xa0\x9f',
'HangulSyllableDwaess': '\xeb\x90\x90',
'CjkCompatibilityIdeographfa2c': '\xef\xa8\xac',
'CjkCompatibilityIdeographfa1a': '\xef\xa8\x9a',
'FullwidthLatinSmallLetterH': '\xef\xbd\x88',
'CjkUnifiedIdeograph6c7c': '\xe6\xb1\xbc',
'ckappa': '\xcf\xb0',
'ArabicLetterSadInitialForm': '\xef\xba\xbb',
'CjkUnifiedIdeograph5c6a': '\xe5\xb1\xaa',
'CjkUnifiedIdeograph5c6b': '\xe5\xb1\xab',
'CjkUnifiedIdeograph5c6c': '\xe5\xb1\xac',
'CjkUnifiedIdeograph5c6d': '\xe5\xb1\xad',
'CjkUnifiedIdeograph5c6e': '\xe5\xb1\xae',
'CjkUnifiedIdeograph5c6f': '\xe5\xb1\xaf',
'ArabicLetterDalWithRing': '\xda\x89',
'CjkUnifiedIdeograph41c9': '\xe4\x87\x89',
'BraillePatternDots8': '\xe2\xa2\x80',
'BraillePatternDots6': '\xe2\xa0\xa0',
'CjkUnifiedIdeograph41c3': '\xe4\x87\x83',
'BraillePatternDots4': '\xe2\xa0\x88',
'CjkUnifiedIdeograph41c1': '\xe4\x87\x81',
'BraillePatternDots2': '\xe2\xa0\x82',
'BraillePatternDots3': '\xe2\xa0\x84',
'CjkUnifiedIdeograph41c4': '\xe4\x87\x84',
'BraillePatternDots1': '\xe2\xa0\x81',
'HangulSyllableJeolp': '\xec\xa0\x8e',
'ArabicLetterWawWithHamzaAboveFinalForm': '\xef\xba\x86',
'ArabicLigatureSheenWithMeemWithKhahFinalForm': '\xef\xb5\xaa',
'CjkUnifiedIdeograph8819': '\xe8\xa0\x99',
'CjkUnifiedIdeograph8818': '\xe8\xa0\x98',
'CjkUnifiedIdeograph8811': '\xe8\xa0\x91',
'CjkUnifiedIdeograph8810': '\xe8\xa0\x90',
'CjkUnifiedIdeograph8813': '\xe8\xa0\x93',
'ArabicLetterAin': '\xd8\xb9',
'CjkUnifiedIdeograph8815': '\xe8\xa0\x95',
'CjkUnifiedIdeograph82b7': '\xe8\x8a\xb7',
'CjkUnifiedIdeograph8817': '\xe8\xa0\x97',
'CjkUnifiedIdeograph8816': '\xe8\xa0\x96',
'UnicodeCharacter0xf738': '\xef\x9c\xb8',
'UnicodeCharacter0xf739': '\xef\x9c\xb9',
'Chi': '\xce\xa7',
'HangulSyllableJjyugg': '\xec\xae\xba',
'CjkUnifiedIdeograph41cb': '\xe4\x87\x8b',
'CjkUnifiedIdeograph41cc': '\xe4\x87\x8c',
'UnicodeCharacter0xf732': '\xef\x9c\xb2',
'CjkUnifiedIdeograph41ca': '\xe4\x87\x8a',
'CjkUnifiedIdeograph41cf': '\xe4\x87\x8f',
'UnicodeCharacter0xf735': '\xef\x9c\xb5',
'CjkUnifiedIdeograph41cd': '\xe4\x87\x8d',
'CjkUnifiedIdeograph41ce': '\xe4\x87\x8e',
'CjkUnifiedIdeograph5c68': '\xe5\xb1\xa8',
'CjkUnifiedIdeograph5c69': '\xe5\xb1\xa9',
'GujaratiDigitZero': '\xe0\xab\xa6',
'CjkUnifiedIdeograph5c60': '\xe5\xb1\xa0',
'CjkUnifiedIdeograph5c61': '\xe5\xb1\xa1',
'CjkUnifiedIdeograph5c62': '\xe5\xb1\xa2',
'CjkUnifiedIdeograph5c63': '\xe5\xb1\xa3',
'CjkUnifiedIdeograph5c64': '\xe5\xb1\xa4',
'CjkUnifiedIdeograph5c65': '\xe5\xb1\xa5',
'CjkUnifiedIdeograph5c66': '\xe5\xb1\xa6',
'CjkUnifiedIdeograph5c67': '\xe5\xb1\xa7',
'CjkUnifiedIdeograph8cff': '\xe8\xb3\xbf',
'CjkUnifiedIdeograph8cfe': '\xe8\xb3\xbe',
'CjkUnifiedIdeograph8cfd': '\xe8\xb3\xbd',
'CjkUnifiedIdeograph8cfc': '\xe8\xb3\xbc',
'CjkUnifiedIdeograph8cfb': '\xe8\xb3\xbb',
'CjkUnifiedIdeograph8cfa': '\xe8\xb3\xba',
'UpperLeftOrLowerRightCurlyBracketSection': '\xe2\x8e\xb0',
'HangulSyllableDdyelp': '\xeb\x98\x82',
'UnicodeCharacter0xf476': '\xef\x91\xb6',
'UnicodeCharacter0xe5ae': '\xee\x96\xae',
'HangulSyllableDdabs': '\xeb\x95\x82',
'CjkUnifiedIdeograph9d69': '\xe9\xb5\xa9',
'CjkUnifiedIdeograph9d68': '\xe9\xb5\xa8',
'CjkUnifiedIdeograph9d67': '\xe9\xb5\xa7',
'RightwardsArrowAboveReverseTildeOperator': '\xe2\xad\x8c',
'CjkUnifiedIdeograph9d65': '\xe9\xb5\xa5',
'CjkUnifiedIdeograph9d64': '\xe9\xb5\xa4',
'CjkUnifiedIdeograph9d63': '\xe9\xb5\xa3',
'CjkUnifiedIdeograph9d62': '\xe9\xb5\xa2',
'CjkUnifiedIdeograph9d61': '\xe9\xb5\xa1',
'CjkUnifiedIdeograph9d60': '\xe9\xb5\xa0',
'LatinSmallLetterOWithDotAboveAndMacron': '\xc8\xb1',
'YiSyllableSit': '\xea\x8c\x89',
'CjkUnifiedIdeograph71fe': '\xe7\x87\xbe',
'KhmerSymbolDapmuoyKoet': '\xe1\xa7\xab',
'EthiopicSyllableTswa': '\xe1\x8c\xbf',
'NkoLetterMa': '\xdf\xa1',
'UnicodeCharacter0xe5ad': '\xee\x96\xad',
'ArabicLetterYehBarreeWithHamzaAbove': '\xdb\x93',
'GeorgianCapitalLetterJil': '\xe1\x82\xbb',
'HangulSyllableBoek': '\xeb\xb5\xa0',
'GreaterthanOrEquivalentTo': '\xe2\x89\xb3',
'HangulLetterEu': '\xe3\x85\xa1',
'CjkUnifiedIdeograph9d6f': '\xe9\xb5\xaf',
'CjkUnifiedIdeograph9d6e': '\xe9\xb5\xae',
'CjkUnifiedIdeograph9d6d': '\xe9\xb5\xad',
'CjkUnifiedIdeograph9d6c': '\xe9\xb5\xac',
'CjkUnifiedIdeograph9d6b': '\xe9\xb5\xab',
'CjkUnifiedIdeograph9d6a': '\xe9\xb5\xaa',
'CjkUnifiedIdeograph8cf7': '\xe8\xb3\xb7',
'CjkUnifiedIdeograph6714': '\xe6\x9c\x94',
'CjkUnifiedIdeograph8cf5': '\xe8\xb3\xb5',
'CjkUnifiedIdeograph8cf4': '\xe8\xb3\xb4',
'CjkUnifiedIdeograph8cf3': '\xe8\xb3\xb3',
'CjkUnifiedIdeograph8cf2': '\xe8\xb3\xb2',
'CjkUnifiedIdeograph8cf1': '\xe8\xb3\xb1',
'CjkUnifiedIdeograph8cf0': '\xe8\xb3\xb0',
'UnicodeCharacter0xe32c': '\xee\x8c\xac',
'HangulLetterEo': '\xe3\x85\x93',
'CjkUnifiedIdeograph8cf9': '\xe8\xb3\xb9',
'CjkUnifiedIdeograph8cf8': '\xe8\xb3\xb8',
'ThaiCharacterMoMa': '\xe0\xb8\xa1',
'HangulSyllableHaess': '\xed\x96\x88',
'HangulSyllableJjyaelg': '\xec\xa8\xb9',
'HangulSyllableBoel': '\xeb\xb5\x90',
'HangulSyllableSyags': '\xec\x83\xa7',
'MalayalamNumberOneHundred': '\xe0\xb5\xb1',
'UnicodeCharacter0xf44d': '\xef\x91\x8d',
'HangulSyllableSyagg': '\xec\x83\xa6',
'HangulSyllableSswaeh': '\xec\x90\xab',
'ArabicLigatureTahWithHahIsolatedForm': '\xef\xb0\xa6',
'HebrewLetterResh': '\xd7\xa8',
'CjkUnifiedIdeograph8d6a': '\xe8\xb5\xaa',
'CjkUnifiedIdeograph7b28': '\xe7\xac\xa8',
'CjkUnifiedIdeograph4019': '\xe4\x80\x99',
'CjkUnifiedIdeograph4018': '\xe4\x80\x98',
'HangulSyllableTenh': '\xed\x85\x92',
'CjkUnifiedIdeograph4015': '\xe4\x80\x95',
'CjkUnifiedIdeograph4014': '\xe4\x80\x94',
'CjkUnifiedIdeograph4017': '\xe4\x80\x97',
'CjkUnifiedIdeograph4016': '\xe4\x80\x96',
'CjkUnifiedIdeograph4011': '\xe4\x80\x91',
'CjkUnifiedIdeograph4010': '\xe4\x80\x90',
'CjkUnifiedIdeograph4013': '\xe4\x80\x93',
'CjkUnifiedIdeograph4012': '\xe4\x80\x92',
'CopticSmallLetterOldNubianWau': '\xe2\xb3\xa3',
'CjkUnifiedIdeograph3cf8': '\xe3\xb3\xb8',
'CjkUnifiedIdeograph3cf9': '\xe3\xb3\xb9',
'RunicLetterHaglazH': '\xe1\x9a\xba',
'SquareBiru': '\xe3\x8c\xb1',
'CjkUnifiedIdeograph4ecf': '\xe4\xbb\x8f',
'CjkUnifiedIdeograph3cf3': '\xe3\xb3\xb3',
'CjkUnifiedIdeograph4ecd': '\xe4\xbb\x8d',
'CjkUnifiedIdeograph3cf1': '\xe3\xb3\xb1',
'CjkUnifiedIdeograph3cf6': '\xe3\xb3\xb6',
'CjkUnifiedIdeograph3cf7': '\xe3\xb3\xb7',
'CjkUnifiedIdeograph3cf4': '\xe3\xb3\xb4',
'CjkUnifiedIdeograph3cf5': '\xe3\xb3\xb5',
'HangulSyllableHis': '\xed\x9e\x9b',
'CjkUnifiedIdeograph810f': '\xe8\x84\x8f',
'CjkUnifiedIdeograph810e': '\xe8\x84\x8e',
'CjkUnifiedIdeograph810d': '\xe8\x84\x8d',
'CjkUnifiedIdeograph810c': '\xe8\x84\x8c',
'CjkUnifiedIdeograph810b': '\xe8\x84\x8b',
'CjkUnifiedIdeograph810a': '\xe8\x84\x8a',
'HangulSyllableHit': '\xed\x9e\xa1',
'UnicodeCharacter0xe41a': '\xee\x90\x9a',
'UnicodeCharacter0xe41b': '\xee\x90\x9b',
'MediumVerticalBar': '\xe2\x9d\x99',
'UnicodeCharacter0xe41d': '\xee\x90\x9d',
'UnicodeCharacter0xe41e': '\xee\x90\x9e',
'UnicodeCharacter0xe41f': '\xee\x90\x9f',
'UnicodeCharacter0xf2a2': '\xef\x8a\xa2',
'HangulSyllableHic': '\xed\x9e\x9f',
'HangulSyllableHib': '\xed\x9e\x99',
'HangulSyllableHyeolg': '\xed\x98\x89',
'HangulSyllableHig': '\xed\x9e\x89',
'UnicodeCharacter0xf2a1': '\xef\x8a\xa1',
'HangulSyllableHid': '\xed\x9e\x8f',
'HangulSyllableHik': '\xed\x9e\xa0',
'HangulSyllableBass': '\xeb\xb0\xa8',
'HangulSyllableHih': '\xed\x9e\xa3',
'HangulSyllableHin': '\xed\x9e\x8c',
'HangulSyllableHim': '\xed\x9e\x98',
'HangulSyllableHil': '\xed\x9e\x90',
'HangulSyllableSswalm': '\xec\x8f\xbe',
'HangulSyllablePoed': '\xed\x91\x87',
'HangulSyllableMwaeh': '\xeb\xab\xbb',
'HangulSyllableGgyilt': '\xeb\x81\xad',
'HangulSyllableGgyils': '\xeb\x81\xac',
'HangulSyllableSswalh': '\xec\x90\x83',
'CjkUnifiedIdeograph4ec8': '\xe4\xbb\x88',
'CjkUnifiedIdeograph4ec9': '\xe4\xbb\x89',
'CjkUnifiedIdeograph3cfb': '\xe3\xb3\xbb',
'CjkUnifiedIdeograph3cfc': '\xe3\xb3\xbc',
'CjkUnifiedIdeograph4ec4': '\xe4\xbb\x84',
'CjkUnifiedIdeograph3cfa': '\xe3\xb3\xba',
'BoxDrawingsDownHeavyAndHorizontalLight': '\xe2\x94\xb0',
'CjkUnifiedIdeograph4ec3': '\xe4\xbb\x83',
'LepchaDigitThree': '\xe1\xb1\x83',
'CjkUnifiedIdeograph3cfe': '\xe3\xb3\xbe',
'HangulSyllableGgyilg': '\xeb\x81\xa9',
'HangulSyllableTanj': '\xed\x83\x85',
'IdeographicTelegraphSymbolForDayTwelve': '\xe3\x8f\xab',
'HangulSyllableGgyilb': '\xeb\x81\xab',
'KangxiRadicalSlice': '\xe2\xbd\x9a',
'CjkUnifiedIdeograph401e': '\xe4\x80\x9e',
'CjkUnifiedIdeograph401d': '\xe4\x80\x9d',
'HangulSyllableGgyilm': '\xeb\x81\xaa',
'CjkUnifiedIdeograph401f': '\xe4\x80\x9f',
'TibetanSubjoinedLetterSa': '\xe0\xbe\xb6',
'HangulSyllableSswalp': '\xec\x90\x82',
'CjkUnifiedIdeograph401c': '\xe4\x80\x9c',
'CjkUnifiedIdeograph401b': '\xe4\x80\x9b',
'UnicodeCharacter0xe5ab': '\xee\x96\xab',
'CjkUnifiedIdeograph8107': '\xe8\x84\x87',
'CjkUnifiedIdeograph8106': '\xe8\x84\x86',
'CjkUnifiedIdeograph8105': '\xe8\x84\x85',
'CjkUnifiedIdeograph8104': '\xe8\x84\x84',
'CjkUnifiedIdeograph8103': '\xe8\x84\x83',
'CjkUnifiedIdeograph8102': '\xe8\x84\x82',
'CjkUnifiedIdeograph8101': '\xe8\x84\x81',
'CjkUnifiedIdeograph8100': '\xe8\x84\x80',
'UnicodeCharacter0xe410': '\xee\x90\x90',
'UnicodeCharacter0xde79': '\xed\xb9\xb9',
'UnicodeCharacter0xe412': '\xee\x90\x92',
'HangulSyllableBoet': '\xeb\xb5\xa1',
'SquareBoruto': '\xe3\x8c\xbe',
'UnicodeCharacter0xe415': '\xee\x90\x95',
'CjkUnifiedIdeograph8109': '\xe8\x84\x89',
'CjkUnifiedIdeograph8108': '\xe8\x84\x88',
'CjkUnifiedIdeograph9478': '\xe9\x91\xb8',
'CjkUnifiedIdeograph9479': '\xe9\x91\xb9',
'CjkUnifiedIdeograph7a03': '\xe7\xa8\x83',
'CjkUnifiedIdeograph9474': '\xe9\x91\xb4',
'CjkUnifiedIdeograph9475': '\xe9\x91\xb5',
'CjkUnifiedIdeograph9476': '\xe9\x91\xb6',
'LepchaPunctuationTshookCerwa': '\xe1\xb0\xbe',
'CjkUnifiedIdeograph9470': '\xe9\x91\xb0',
'CjkUnifiedIdeograph9471': '\xe9\x91\xb1',
'CjkUnifiedIdeograph9472': '\xe9\x91\xb2',
'CjkUnifiedIdeograph9473': '\xe9\x91\xb3',
'UnicodeCharacter0x1942': '\xe1\xa5\x82',
'UnicodeCharacter0x1943': '\xe1\xa5\x83',
'UnicodeCharacter0x1941': '\xe1\xa5\x81',
'KhmerLetterTha': '\xe1\x9e\x90',
'KhmerLetterTho': '\xe1\x9e\x92',
'HangulSyllableMwaek': '\xeb\xab\xb8',
'GujaratiLetterVocalicR': '\xe0\xaa\x8b',
'UnicodeCharacter0xdba1': '\xed\xae\xa1',
'CjkUnifiedIdeograph5768': '\xe5\x9d\xa8',
'CjkUnifiedIdeograph5769': '\xe5\x9d\xa9',
'HangulSyllableRwi': '\xeb\xa4\xbc',
'CjkUnifiedIdeograph7b22': '\xe7\xac\xa2',
'CjkUnifiedIdeograph5764': '\xe5\x9d\xa4',
'CjkUnifiedIdeograph5765': '\xe5\x9d\xa5',
'GujaratiLetterVocalicL': '\xe0\xaa\x8c',
'GreekLetterSmallCapitalPi': '\xe1\xb4\xa8',
'CjkUnifiedIdeograph5760': '\xe5\x9d\xa0',
'CjkUnifiedIdeograph5761': '\xe5\x9d\xa1',
'CjkUnifiedIdeograph5762': '\xe5\x9d\xa2',
'CjkUnifiedIdeograph5763': '\xe5\x9d\xa3',
'UnicodeCharacter0xeb40': '\xee\xad\x80',
'GeometricallyEqualTo': '\xe2\x89\x91',
'TifinaghLetterYey': '\xe2\xb4\xbb',
'HangulSyllableMigg': '\xeb\xaf\xba',
'HebrewPunctuationPaseq': '\xd7\x80',
'HangulSyllableHelb': '\xed\x97\xaf',
'UnicodeCharacter0xdba0': '\xed\xae\xa0',
'LimbuLetterYan': '\xe1\xa4\x8a',
'UnicodeCharacter0xaa88': '\xea\xaa\x88',
'UnicodeCharacter0xaa89': '\xea\xaa\x89',
'CjkUnifiedIdeograph947d': '\xe9\x91\xbd',
'CjkUnifiedIdeograph947e': '\xe9\x91\xbe',
'CjkUnifiedIdeograph947f': '\xe9\x91\xbf',
'UnicodeCharacter0xaa85': '\xea\xaa\x85',
'UnicodeCharacter0xaa82': '\xea\xaa\x82',
'CjkUnifiedIdeograph947a': '\xe9\x91\xba',
'UnionWithOverbar': '\xe2\xa9\x82',
'CjkUnifiedIdeograph49db': '\xe4\xa7\x9b',
'CjkUnifiedIdeograph7a06': '\xe7\xa8\x86',
'HangulSyllableSoenj': '\xec\x87\xa5',
'CjkUnifiedIdeograph576d': '\xe5\x9d\xad',
'CjkUnifiedIdeograph576e': '\xe5\x9d\xae',
'CjkUnifiedIdeograph576f': '\xe5\x9d\xaf',
'HangulSyllableNyibs': '\xeb\x8a\xbe',
'CjkUnifiedIdeograph576a': '\xe5\x9d\xaa',
'CjkUnifiedIdeograph576b': '\xe5\x9d\xab',
'CjkUnifiedIdeograph576c': '\xe5\x9d\xac',
'LowerRightDropshadowedWhiteSquare': '\xe2\x9d\x8f',
'YiSyllableDuox': '\xea\x84\x8b',
'GeorgianLetterJhan': '\xe1\x83\xaf',
'CjkUnifiedIdeograph7a07': '\xe7\xa8\x87',
'EthiopicNumberEighty': '\xe1\x8d\xb9',
'MongolianLetterTodoTa': '\xe1\xa1\x90',
'ModifierLetterMiddleDoubleGraveAccent': '\xcb\xb5',
'HangulSyllableMeoc': '\xeb\xa9\x8f',
'HangulSyllableCeugs': '\xec\xb8\xa3',
'IdeographicTelegraphSymbolForHourSixteen': '\xe3\x8d\xa8',
'HangulSyllableMeog': '\xeb\xa8\xb9',
'SphericalAngle': '\xe2\x88\xa2',
'UnicodeCharacter0xed0f': '\xee\xb4\x8f',
'UnicodeCharacter0xed0e': '\xee\xb4\x8e',
'UnicodeCharacter0xed0d': '\xee\xb4\x8d',
'UnicodeCharacter0xed0c': '\xee\xb4\x8c',
'HangulSyllableMeod': '\xeb\xa8\xbf',
'UnicodeCharacter0xed0a': '\xee\xb4\x8a',
'NkoDigitFour': '\xdf\x84',
'UnicodeCharacter0xa4fe': '\xea\x93\xbe',
'UnicodeCharacter0xa4fd': '\xea\x93\xbd',
'UnicodeCharacter0xa4ff': '\xea\x93\xbf',
'UnicodeCharacter0xa4fa': '\xea\x93\xba',
'UnicodeCharacter0xa4fc': '\xea\x93\xbc',
'UnicodeCharacter0xa4fb': '\xea\x93\xbb',
'KangxiRadicalShell': '\xe2\xbe\x99',
'HangulSyllableBweoj': '\xeb\xb6\xb2',
'UnicodeCharacter0xf6a1': '\xef\x9a\xa1',
'SquareIningu': '\xe3\x8c\x84',
'HalfwidthHangulLetterSsangtikeut': '\xef\xbe\xa8',
'CjkUnifiedIdeograph67f8': '\xe6\x9f\xb8',
'HangulSyllableGyals': '\xea\xb1\x84',
'HangulSyllableGyalp': '\xea\xb1\x86',
'HangulSyllableGyalt': '\xea\xb1\x85',
'dd': '\xe2\x85\x86',
'HangulSyllableGyalb': '\xea\xb1\x83',
'CjkUnifiedIdeograph9bab': '\xe9\xae\xab',
'HangulSyllableGyalg': '\xea\xb1\x81',
'HangulSyllableMeol': '\xeb\xa9\x80',
'UnicodeCharacter0xeb43': '\xee\xad\x83',
'HangulSyllableGyalh': '\xea\xb1\x87',
'HangulSyllableGyalm': '\xea\xb1\x82',
'CjkUnifiedIdeograph87ba': '\xe8\x9e\xba',
'CombiningMacronRightHalf': '\xef\xb8\xa5',
'HangulSyllableMeos': '\xeb\xa9\x8b',
'ArabicLetterSwashKaf': '\xda\xaa',
'HangulSyllableNong': '\xeb\x86\x8d',
'UnicodeCharacter0xed05': '\xee\xb4\x85',
'UnicodeCharacter0xed04': '\xee\xb4\x84',
'UnicodeCharacter0xa4f9': '\xea\x93\xb9',
'UnicodeCharacter0xa4f8': '\xea\x93\xb8',
'UnicodeCharacter0xed01': '\xee\xb4\x81',
'UnicodeCharacter0xed00': '\xee\xb4\x80',
'UnicodeCharacter0xa4f5': '\xea\x93\xb5',
'UnicodeCharacter0xa4f4': '\xea\x93\xb4',
'UnicodeCharacter0xa4f7': '\xea\x93\xb7',
'UnicodeCharacter0xa4f6': '\xea\x93\xb6',
'UnicodeCharacter0xa4f1': '\xea\x93\xb1',
'UnicodeCharacter0xa4f0': '\xea\x93\xb0',
'UnicodeCharacter0xa4f3': '\xea\x93\xb3',
'UnicodeCharacter0xa4f2': '\xea\x93\xb2',
'HangulSyllableGgwaej': '\xea\xbd\xba',
'HangulSyllableGgwaek': '\xea\xbd\xbc',
'SquarePw': '\xe3\x8e\xba',
'SquarePv': '\xe3\x8e\xb4',
'HangulSyllableGgwaen': '\xea\xbd\xa8',
'SuperscriptEight': '\xe2\x81\xb8',
'SquarePs': '\xe3\x8e\xb0',
'EthiopicSyllableGee': '\xe1\x8c\x8c',
'LatinCapitalLetterUWithCircumflex': '\xc3\x9b',
'HangulSyllableGgwaec': '\xea\xbd\xbb',
'HangulSyllableGgwaeg': '\xea\xbd\xa5',
'HangulSyllableGgwaed': '\xea\xbd\xab',
'HangulSyllableGgaenj': '\xea\xb9\xad',
'TibetanCantillationSignSbubChal': '\xe0\xbf\x83',
'SquarePf': '\xe3\x8e\x8a',
'MathematicalRightWhiteTortoiseShellBracket': '\xe2\x9f\xad',
'SquarePc': '\xe3\x8d\xb6',
'SquarePm': '\xe3\x8f\x98',
'CjkUnifiedIdeograph83bb': '\xe8\x8e\xbb',
'CjkUnifiedIdeograph83ba': '\xe8\x8e\xba',
'SquarePh': '\xe3\x8f\x97',
'CjkUnifiedIdeograph83be': '\xe8\x8e\xbe',
'CjkUnifiedIdeograph83bd': '\xe8\x8e\xbd',
'CjkUnifiedIdeograph8e89': '\xe8\xba\x89',
'CjkUnifiedIdeograph8e88': '\xe8\xba\x88',
'HangulSyllableJjegs': '\xec\xa9\xab',
'CjkUnifiedIdeograph8e83': '\xe8\xba\x83',
'CjkUnifiedIdeograph8e82': '\xe8\xba\x82',
'LeftCurlyBracketUpperHook': '\xe2\x8e\xa7',
'CjkUnifiedIdeograph8e80': '\xe8\xba\x80',
'CjkUnifiedIdeograph8e87': '\xe8\xba\x87',
'CjkUnifiedIdeograph7153': '\xe7\x85\x93',
'CjkUnifiedIdeograph8e85': '\xe8\xba\x85',
'CjkUnifiedIdeograph8e84': '\xe8\xba\x84',
'CjkUnifiedIdeograph7152': '\xe7\x85\x92',
'UnicodeCharacter0xdb68': '\xed\xad\xa8',
'CyrillicSmallLetterUk': '\xd1\xb9',
'HangulSyllableNyaebs': '\xeb\x83\xbe',
'UnicodeCharacter0x139a': '\xe1\x8e\x9a',
'CjkUnifiedIdeograph7155': '\xe7\x85\x95',
'UnicodeCharacter0xdb66': '\xed\xad\xa6',
'UnicodeCharacter0xdb67': '\xed\xad\xa7',
'UnicodeCharacter0xdb60': '\xed\xad\xa0',
'UnicodeCharacter0xdb61': '\xed\xad\xa1',
'HangulLetterSsangtikeut': '\xe3\x84\xb8',
'UnicodeCharacter0xdb63': '\xed\xad\xa3',
'HangulSyllableJunh': '\xec\xa4\x82',
'UnicodeCharacter0xeb74': '\xee\xad\xb4',
'CjkUnifiedIdeograph83b9': '\xe8\x8e\xb9',
'CjkUnifiedIdeograph83b8': '\xe8\x8e\xb8',
'CjkUnifiedIdeograph83b3': '\xe8\x8e\xb3',
'CjkUnifiedIdeograph83b2': '\xe8\x8e\xb2',
'CjkUnifiedIdeograph5e68': '\xe5\xb9\xa8',
'CjkUnifiedIdeograph83b0': '\xe8\x8e\xb0',
'CjkUnifiedIdeograph83b7': '\xe8\x8e\xb7',
'CjkUnifiedIdeograph83b6': '\xe8\x8e\xb6',
'CjkUnifiedIdeograph83b5': '\xe8\x8e\xb5',
'CjkUnifiedIdeograph83b4': '\xe8\x8e\xb4',
'NumberFifteenFullStop': '\xe2\x92\x96',
'CjkUnifiedIdeograph88fc': '\xe8\xa3\xbc',
'UnicodeCharacter0xa7b7': '\xea\x9e\xb7',
'LatinCapitalLetterUWithTildeBelow': '\xe1\xb9\xb4',
'CombiningRightArrowBelow': '\xe2\x83\xaf',
'CjkUnifiedIdeograph5fb8': '\xe5\xbe\xb8',
'CjkUnifiedIdeograph5e69': '\xe5\xb9\xa9',
'UnicodeCharacter0xdb6e': '\xed\xad\xae',
'HangulSyllableDdul': '\xeb\x9a\xa4',
'LatinCapitalLetterAWithDotBelow': '\xe1\xba\xa0',
'HangulSyllableSsyoj': '\xec\x91\x9e',
'UnicodeCharacter0xdb6b': '\xed\xad\xab',
'UnicodeCharacter0xdb6c': '\xed\xad\xac',
'HangulSyllableJjubs': '\xec\xad\x9a',
'HangulSyllableJjeug': '\xec\xaf\x95',
'YiSyllableBuo': '\xea\x80\xa3',
'CjkUnifiedIdeograph8e8c': '\xe8\xba\x8c',
'CjkUnifiedIdeograph8e8b': '\xe8\xba\x8b',
'CjkUnifiedIdeograph8e8a': '\xe8\xba\x8a',
'CjkUnifiedIdeograph8e8f': '\xe8\xba\x8f',
'OriyaVowelSignVocalicRr': '\xe0\xad\x84',
'CopticSmallLetterOldCopticShima': '\xe2\xb3\x9b',
'HangulSyllableDwaels': '\xeb\x90\x88',
'GlagoliticCapitalLetterShta': '\xe2\xb0\x9b',
'FullwidthLatinSmallLetterE': '\xef\xbd\x85',
'HangulSyllableSsyon': '\xec\x91\x8c',
'HangulSyllableDwaelt': '\xeb\x90\x89',
'KangxiRadicalTongue': '\xe2\xbe\x86',
'UnicodeCharacter0xee50': '\xee\xb9\x90',
'LatinSubscriptSmallLetterI': '\xe1\xb5\xa2',
'LatinSubscriptSmallLetterJ': '\xe2\xb1\xbc',
'HangulSyllableDwaelb': '\xeb\x90\x87',
'UnicodeCharacter0x2700': '\xe2\x9c\x80',
'LatinSubscriptSmallLetterV': '\xe1\xb5\xa5',
'UnicodeCharacter0x2705': '\xe2\x9c\x85',
'HangulSyllableDwaelg': '\xeb\x90\x85',
'LatinSubscriptSmallLetterR': '\xe1\xb5\xa3',
'EthiopicSyllableZhee': '\xe1\x8b\xa4',
'UnicodeCharacter0xe1e1': '\xee\x87\xa1',
'HangulSyllableDwaelh': '\xeb\x90\x8b',
'LatinSubscriptSmallLetterX': '\xe2\x82\x93',
'HangulSyllableBbyilm': '\xec\x81\xbe',
'HangulSyllableDwaelm': '\xeb\x90\x86',
'HangulSyllableBweob': '\xeb\xb6\xad',
'UpRightDiagonalEllipsis': '\xe2\x8b\xb0',
'BopomofoLetterAunn': '\xe3\x86\xaf',
'TeluguLetterNna': '\xe0\xb0\xa3',
'CjkUnifiedIdeograph78b4': '\xe7\xa2\xb4',
'HangulSyllablePeogs': '\xed\x8d\xbf',
'HangulSyllableBbwaels': '\xeb\xbe\x84',
'UnicodeCharacter0x270a': '\xe2\x9c\x8a',
'UnicodeCharacter0x270b': '\xe2\x9c\x8b',
'HangulSyllableHolg': '\xed\x99\x81',
'LatinCapitalLetterUpsilon': '\xc6\xb1',
'HangulSyllableBbwaelt': '\xeb\xbe\x85',
'HangulSyllableJjeub': '\xec\xaf\xa5',
'UnicodeCharacter0xebe7': '\xee\xaf\xa7',
'HangulSyllableDyigg': '\xeb\x93\xba',
'UnicodeCharacter0xf59a': '\xef\x96\x9a',
'FullwidthSolidus': '\xef\xbc\x8f',
'CjkUnifiedIdeograph5e65': '\xe5\xb9\xa5',
'CjkRadicalPawTwo': '\xe2\xba\xa5',
'UnicodeCharacter0xe12e': '\xee\x84\xae',
'CjkUnifiedIdeograph4c00': '\xe4\xb0\x80',
'UnicodeCharacter0xe12f': '\xee\x84\xaf',
'UnicodeCharacter0xf27c': '\xef\x89\xbc',
'BraillePatternDots12468': '\xe2\xa2\xab',
'YiSyllableBux': '\xea\x80\xad',
'UnicodeCharacter0xf59b': '\xef\x96\x9b',
'BraillePatternDots12467': '\xe2\xa1\xab',
'CyrillicSmallLetterZheWithDiaeresis': '\xd3\x9d',
'UnicodeCharacter0xdcf1': '\xed\xb3\xb1',
'HangulSyllableJoet': '\xec\xa3\x9d',
'UnicodeCharacter0xdcf3': '\xed\xb3\xb3',
'UnicodeCharacter0xdcf2': '\xed\xb3\xb2',
'UnicodeCharacter0xdcf5': '\xed\xb3\xb5',
'ParenthesizedIdeographTen': '\xe3\x88\xa9',
'UnicodeCharacter0xdcf7': '\xed\xb3\xb7',
'UnicodeCharacter0xdcf6': '\xed\xb3\xb6',
'UnicodeCharacter0xdcf9': '\xed\xb3\xb9',
'UnicodeCharacter0xdcf8': '\xed\xb3\xb8',
'HangulSyllableJjyuj': '\xec\xaf\x8e',
'YiRadicalCip': '\xea\x92\xbe',
'UnicodeCharacter0xdb7c': '\xed\xad\xbc',
'CanadianSyllabicsCarrierRee': '\xe1\x97\x8d',
'CjkUnifiedIdeograph67fb': '\xe6\x9f\xbb',
'ArabicLigatureNoonWithMeemWithAlefMaksuraFinalForm': '\xef\xb6\x9b',
'UnicodeCharacter0xec34': '\xee\xb0\xb4',
'UnicodeCharacter0xdd34': '\xed\xb4\xb4',
'HangulSyllableJjeum': '\xec\xaf\xa4',
'YiSyllableMot': '\xea\x82\xbd',
'YiSyllableMop': '\xea\x83\x80',
'CjkUnifiedIdeograph49d2': '\xe4\xa7\x92',
'YiSyllableXo': '\xea\x91\xa9',
'YiSyllableDdax': '\xea\x84\xbe',
'HangulSyllableJwelp': '\xec\xa5\x82',
'YiSyllableDdap': '\xea\x85\x80',
'YiSyllableDdat': '\xea\x84\xbd',
'UnicodeCharacter0xdcfa': '\xed\xb3\xba',
'UnicodeCharacter0xf59e': '\xef\x96\x9e',
'UnicodeCharacter0xdcfc': '\xed\xb3\xbc',
'UnicodeCharacter0xdcfb': '\xed\xb3\xbb',
'UnicodeCharacter0xdcfe': '\xed\xb3\xbe',
'CircledNumberFortyEight': '\xe3\x8a\xbd',
'**7': '\xe2\x81\xb7',
'UnicodeCharacter0xdcff': '\xed\xb3\xbf',
'Y': '\xce\xa8',
'HangulJongseongTikeutkiyeok': '\xe1\x87\x8a',
'CopticCapitalLetterSima': '\xe2\xb2\xa4',
'ArabicLetterVehFinalForm': '\xef\xad\xab',
'KayahLiLetterPa': '\xea\xa4\x95',
'CjkUnifiedIdeograph6cd6': '\xe6\xb3\x96',
'HangulSyllableAegg': '\xec\x95\xa2',
'Miny': '\xe2\xa7\xbf',
'HangulSyllableCeonh': '\xec\xb2\x9e',
'CjkUnifiedIdeograph6759': '\xe6\x9d\x99',
'HangulSyllableCeong': '\xec\xb2\xad',
'TibetanSubjoinedLetterFixedformWa': '\xe0\xbe\xba',
'TibetanMarkClosingYigMgoSgabMa': '\xe0\xbc\x85',
'CjkRadicalCsimplifiedFish': '\xe2\xbb\xa5',
'TibetanDigitSeven': '\xe0\xbc\xa7',
'HangulSyllableByaelh': '\xeb\xb1\xb7',
'UnicodeCharacter0xdd30': '\xed\xb4\xb0',
'UnicodeCharacter0xef20': '\xee\xbc\xa0',
'CyrillicCapitalLetterBe': '\xd0\x91',
'CjkUnifiedIdeograph618a': '\xe6\x86\x8a',
'CjkUnifiedIdeograph618c': '\xe6\x86\x8c',
'CjkUnifiedIdeograph618b': '\xe6\x86\x8b',
'CjkUnifiedIdeograph618e': '\xe6\x86\x8e',
'CjkUnifiedIdeograph618d': '\xe6\x86\x8d',
'CjkUnifiedIdeograph618f': '\xe6\x86\x8f',
'CjkUnifiedIdeograph955c': '\xe9\x95\x9c',
'HangulSyllableByaelm': '\xeb\xb1\xb2',
'UnicodeCharacter0xe298': '\xee\x8a\x98',
'TibetanLetterKka': '\xe0\xbd\xab',
'HangulSyllableRyobs': '\xeb\xa3\x9e',
'UnicodeCharacter0xe299': '\xee\x8a\x99',
'HangulSyllableBbebs': '\xeb\xbb\xbe',
'KangxiRadicalTable': '\xe2\xbc\x8f',
'RightwardsArrowWithCornerDownwards': '\xe2\x86\xb4',
'HangulSyllableDdej': '\xeb\x97\x92',
'HangulSyllableDdek': '\xeb\x97\x94',
'HangulSyllableDdeh': '\xeb\x97\x97',
'HangulSyllableDden': '\xeb\x97\x80',
'HangulSyllableDdeo': '\xeb\x96\xa0',
'HangulSyllableDdel': '\xeb\x97\x84',
'HangulSyllableDdem': '\xeb\x97\x8c',
'HangulSyllableDdeb': '\xeb\x97\x8d',
'HangulSyllableDdec': '\xeb\x97\x93',
'HangulSyllableRweolm': '\xeb\xa4\x8e',
'BugineseLetterCa': '\xe1\xa8\x8c',
'HangulSyllableDdeg': '\xeb\x96\xbd',
'HangulSyllableDded': '\xeb\x97\x83',
'HangulSyllableByaelg': '\xeb\xb1\xb1',
'ActivateArabicFormShaping': '\xe2\x81\xad',
'HangulSyllableRweolt': '\xeb\xa4\x91',
'UnicodeCharacter0xec32': '\xee\xb0\xb2',
'HangulSyllableRweols': '\xeb\xa4\x90',
'OghamLetterLuis': '\xe1\x9a\x82',
'HangulSyllableSsenh': '\xec\x8e\x8a',
'HangulSyllableSseng': '\xec\x8e\x99',
'HangulSyllableDdes': '\xeb\x97\x8f',
'HangulSyllableDdep': '\xeb\x97\x96',
'LatinCapitalLetterAWithDoubleGrave': '\xc8\x80',
'HangulSyllableDdet': '\xeb\x97\x95',
'HangulSyllableDdeu': '\xeb\x9c\xa8',
'CanadianSyllabicsCarrierMe': '\xe1\x98\x8a',
'CanadianSyllabicsCarrierMa': '\xe1\x98\x8d',
'CanadianSyllabicsCarrierMo': '\xe1\x98\x89',
'CanadianSyllabicsCarrierMi': '\xe1\x98\x8c',
'CjkUnifiedIdeograph6181': '\xe6\x86\x81',
'CjkUnifiedIdeograph6180': '\xe6\x86\x80',
'CjkUnifiedIdeograph6183': '\xe6\x86\x83',
'CanadianSyllabicsCarrierMu': '\xe1\x98\x88',
'CjkUnifiedIdeograph6185': '\xe6\x86\x85',
'CjkUnifiedIdeograph6184': '\xe6\x86\x84',
'CjkUnifiedIdeograph6187': '\xe6\x86\x87',
'CjkUnifiedIdeograph6186': '\xe6\x86\x86',
'CjkUnifiedIdeograph6189': '\xe6\x86\x89',
'CjkUnifiedIdeograph6188': '\xe6\x86\x88',
'UnicodeCharacter0xb7a': '\xe0\xad\xba',
'UnicodeCharacter0xb7b': '\xe0\xad\xbb',
'UnicodeCharacter0xb7c': '\xe0\xad\xbc',
'UnicodeCharacter0xb7d': '\xe0\xad\xbd',
'UnicodeCharacter0xb7e': '\xe0\xad\xbe',
'UnicodeCharacter0xb7f': '\xe0\xad\xbf',
'CjkUnifiedIdeograph639f': '\xe6\x8e\x9f',
'UnicodeCharacter0xf49d': '\xef\x92\x9d',
'HangulSyllableDdwiss': '\xeb\x9c\x84',
'Divides': '\xe2\x88\xa3',
'HexagramForStandstill': '\xe4\xb7\x8b',
'HangulSyllableJjeoss': '\xec\xa9\xa0',
'HebrewLetterZayin': '\xd7\x96',
'ParenthesizedIdeographFive': '\xe3\x88\xa4',
'Leo': '\xe2\x99\x8c',
'HangulJungseongYuyeo': '\xe1\x86\x91',
'UnicodeCharacter0xe7e7': '\xee\x9f\xa7',
'CjkUnifiedIdeograph6459': '\xe6\x91\x99',
'CjkUnifiedIdeograph6458': '\xe6\x91\x98',
'DevanagariDigitZero': '\xe0\xa5\xa6',
'OriyaLetterTha': '\xe0\xac\xa5',
'HangulSyllableGeolh': '\xea\xb1\xbf',
'RomanNumeralThree': '\xe2\x85\xa2',
'CjkUnifiedIdeograph6452': '\xe6\x91\x92',
'CjkUnifiedIdeograph6451': '\xe6\x91\x91',
'CjkUnifiedIdeograph6450': '\xe6\x91\x90',
'TibetanLetterGa': '\xe0\xbd\x82',
'CjkUnifiedIdeograph6456': '\xe6\x91\x96',
'CjkUnifiedIdeograph6455': '\xe6\x91\x95',
'CjkUnifiedIdeograph6454': '\xe6\x91\x94',
'CjkUnifiedIdeograph955f': '\xe9\x95\x9f',
'HangulSyllableDulp': '\xeb\x91\x9e',
'LatinCapitalLetterLWithAcute': '\xc4\xb9',
'UnicodeCharacter0xe7ee': '\xee\x9f\xae',
'UnicodeCharacter0xe129': '\xee\x84\xa9',
'UnicodeCharacter0xb78': '\xe0\xad\xb8',
'UnicodeCharacter0xb79': '\xe0\xad\xb9',
'HangulSyllableDuls': '\xeb\x91\x9c',
'ModifierLetterRaisedUpArrow': '\xea\x9c\x9b',
'UnicodeCharacter0xb72': '\xe0\xad\xb2',
'UnicodeCharacter0xb73': '\xe0\xad\xb3',
'UnicodeCharacter0xb74': '\xe0\xad\xb4',
'UnicodeCharacter0xb75': '\xe0\xad\xb5',
'UnicodeCharacter0xb76': '\xe0\xad\xb6',
'UnicodeCharacter0xb77': '\xe0\xad\xb7',
'HangulSyllableJyoc': '\xec\xa3\xb7',
'HangulSyllableJyob': '\xec\xa3\xb1',
'HangulSyllableJyog': '\xec\xa3\xa1',
'HangulSyllableJyod': '\xec\xa3\xa7',
'CjkUnifiedIdeograph645c': '\xe6\x91\x9c',
'CjkUnifiedIdeograph645b': '\xe6\x91\x9b',
'CjkUnifiedIdeograph645a': '\xe6\x91\x9a',
'HangulSyllableJyoh': '\xec\xa3\xbb',
'CjkUnifiedIdeograph645f': '\xe6\x91\x9f',
'CjkUnifiedIdeograph645e': '\xe6\x91\x9e',
'CjkUnifiedIdeograph645d': '\xe6\x91\x9d',
'HangulSyllableJyos': '\xec\xa3\xb3',
'HangulSyllableNwabs': '\xeb\x86\xa6',
'HangulSyllableJyot': '\xec\xa3\xb9',
'HangulSyllableJyigg': '\xec\xa6\xa6',
'HangulSyllableJjyaegs': '\xec\xa8\xb3',
'CanadianSyllabicsTi': '\xe1\x91\x8e',
'HangulSyllableSyut': '\xec\x8a\xa1',
'CherokeeLetterDo': '\xe1\x8f\x99',
'CherokeeLetterDi': '\xe1\x8f\x97',
'HangulSyllableSyup': '\xec\x8a\xa2',
'CherokeeLetterDe': '\xe1\x8f\x95',
'UnicodeCharacter0xe7e4': '\xee\x9f\xa4',
'CherokeeLetterDa': '\xe1\x8f\x93',
'HangulSyllableSyug': '\xec\x8a\x89',
'HangulSyllableSyud': '\xec\x8a\x8f',
'HangulSyllableSyub': '\xec\x8a\x99',
'HangulSyllableSyuc': '\xec\x8a\x9f',
'HangulSyllableSyun': '\xec\x8a\x8c',
'CherokeeLetterDu': '\xe1\x8f\x9a',
'CherokeeLetterDv': '\xe1\x8f\x9b',
'HangulSyllableSyum': '\xec\x8a\x98',
'HangulSyllableSyuj': '\xec\x8a\x9e',
'HangulSyllableSyuk': '\xec\x8a\xa0',
'HangulSyllableSyuh': '\xec\x8a\xa3',
'CjkCompatibilityIdeographfa14': '\xef\xa8\x94',
'UnicodeCharacter0xdf32': '\xed\xbc\xb2',
'UnicodeCharacter0xda02': '\xed\xa8\x82',
'UnicodeCharacter0xe785': '\xee\x9e\x85',
'BoxDrawingsLightDownAndHorizontal': '\xe2\x94\xac',
'UnicodeCharacter0xe7e3': '\xee\x9f\xa3',
'UnicodeCharacter0xd81b': '\xed\xa0\x9b',
'HangulJungseongIu': '\xe1\x86\x9b',
'CjkUnifiedIdeograph9bed': '\xe9\xaf\xad',
'CjkUnifiedIdeograph9bee': '\xe9\xaf\xae',
'CjkUnifiedIdeograph9bef': '\xe9\xaf\xaf',
'YiSyllableGguop': '\xea\x88\xa1',
'HangulJungseongIo': '\xe1\x86\x9a',
'CjkUnifiedIdeograph9bea': '\xe9\xaf\xaa',
'CjkUnifiedIdeograph9beb': '\xe9\xaf\xab',
'CjkUnifiedIdeograph9bec': '\xe9\xaf\xac',
'DevanagariLetterLla': '\xe0\xa4\xb3',
'HangulJungseongIa': '\xe1\x86\x98',
'YiSyllableGguox': '\xea\x88\x9f',
'ThaiCharacterDoChada': '\xe0\xb8\x8e',
'YiSyllableChurx': '\xea\x8d\xb4',
'LatinSmallLetterUWithDoubleGrave': '\xc8\x95',
'CjkUnifiedIdeograph6bbc': '\xe6\xae\xbc',
'HangulSyllableGgwil': '\xeb\x80\x94',
'UnicodeCharacter0xe7e2': '\xee\x9f\xa2',
'ArabicLigatureTehWithZainFinalForm': '\xef\xb1\xb1',
'LepchaPunctuationTarol': '\xe1\xb0\xbb',
'HangulSyllableDulh': '\xeb\x91\x9f',
'UnicodeCharacter0xf528': '\xef\x94\xa8',
'CjkCompatibilityIdeographfa12': '\xef\xa8\x92',
'UnicodeCharacter0xd91c': '\xed\xa4\x9c',
'CjkUnifiedIdeograph9be4': '\xe9\xaf\xa4',
'CjkUnifiedIdeograph9be5': '\xe9\xaf\xa5',
'CjkUnifiedIdeograph9be6': '\xe9\xaf\xa6',
'CjkUnifiedIdeograph9be7': '\xe9\xaf\xa7',
'CjkUnifiedIdeograph9be0': '\xe9\xaf\xa0',
'CjkUnifiedIdeograph9be1': '\xe9\xaf\xa1',
'CjkUnifiedIdeograph9be2': '\xe9\xaf\xa2',
'CjkUnifiedIdeograph9be3': '\xe9\xaf\xa3',
'GreekCapitalLetterOmegaWithProsgegrammeni': '\xe1\xbf\xbc',
'HangulSyllableRyuc': '\xeb\xa5\xaf',
'HangulSyllableGeos': '\xea\xb2\x83',
'CjkUnifiedIdeograph9be8': '\xe9\xaf\xa8',
'CjkUnifiedIdeograph9be9': '\xe9\xaf\xa9',
'HangulSyllableRyug': '\xeb\xa5\x99',
'HangulSyllableGeoh': '\xea\xb2\x8b',
'HangulSyllableGeoj': '\xea\xb2\x86',
'HangulSyllableGeok': '\xea\xb2\x88',
'HangulSyllableGeol': '\xea\xb1\xb8',
'HangulSyllableGeom': '\xea\xb2\x80',
'HangulSyllableGeon': '\xea\xb1\xb4',
'UnicodeCharacter0xe29c': '\xee\x8a\x9c',
'HangulSyllableRyup': '\xeb\xa5\xb2',
'HangulSyllableGeob': '\xea\xb2\x81',
'CjkUnifiedIdeograph73be': '\xe7\x8e\xbe',
'HangulSyllableGeod': '\xea\xb1\xb7',
'HangulSyllableRyut': '\xeb\xa5\xb1',
'HangulSyllableGeog': '\xea\xb1\xb1',
'YiSyllablePo': '\xea\x81\x88',
'SquareKg': '\xe3\x8e\x8f',
'SquareKa': '\xe3\x8e\x84',
'SquareKb': '\xe3\x8e\x85',
'YiSyllablePi': '\xea\x80\xba',
'SquareKl': '\xe3\x8e\x98',
'SquareKm': '\xe3\x8e\x9e',
'HangulSyllableJwels': '\xec\xa5\x80',
'HangulSyllableSik': '\xec\x8b\xb4',
'SquareKk': '\xe3\x8f\x8d',
'SquareKt': '\xe3\x8f\x8f',
'SquareKv': '\xe3\x8e\xb8',
'SquareKw': '\xe3\x8e\xbe',
'YiSyllablePy': '\xea\x81\x92',
'YiSyllablePu': '\xea\x81\x8c',
'HangulSyllableMeh': '\xeb\xa9\xaf',
'HangulSyllableDdils': '\xeb\x9d\xac',
'HangulSyllableDdilp': '\xeb\x9d\xae',
'HangulSyllableGgyebs': '\xea\xbc\xa2',
'HangulSyllableDdilt': '\xeb\x9d\xad',
'UnicodeCharacter0xebe3': '\xee\xaf\xa3',
'YiRadicalNyop': '\xea\x92\xa7',
'OrnateRightParenthesis': '\xef\xb4\xbf',
'HangulSyllableDdilb': '\xeb\x9d\xab',
'ArabicLigatureAinWithMeemWithMeemFinalForm': '\xef\xb5\xb6',
'HangulSyllableDdilg': '\xeb\x9d\xa9',
'CjkCompatibilityIdeographfa11': '\xef\xa8\x91',
'UnicodeCharacter0xdf35': '\xed\xbc\xb5',
'HangulSyllableHyeon': '\xed\x98\x84',
'HangulSyllableDdilh': '\xeb\x9d\xaf',
'HangulSyllableKen': '\xec\xbc\x84',
'HangulSyllableDdilm': '\xeb\x9d\xaa',
'ArabicLetterFarsiYehWithInvertedV': '\xd8\xbd',
'SaurashtraLetterJha': '\xea\xa2\x9a',
'ArabicLetterRnoonInitialForm': '\xef\xae\xa2',
'HangulSyllableJuss': '\xec\xa4\x90',
'KangxiRadicalSteam': '\xe2\xbd\x93',
'UnicodeCharacter0xec36': '\xee\xb0\xb6',
'RunicLetterTiwazTirTyrT': '\xe1\x9b\x8f',
'HangulSyllableKek': '\xec\xbc\x98',
'HebrewLetterAlternativeAyin': '\xef\xac\xa0',
'HangulSyllableCok': '\xec\xb4\xa0',
'HangulSyllableKet': '\xec\xbc\x99',
'UnicodeCharacter0xef2d': '\xee\xbc\xad',
'TibetanMarkHalanta': '\xe0\xbe\x84',
'UnicodeCharacter0xe04c': '\xee\x81\x8c',
'CombiningDoubleCircumflexAbove': '\xe1\xb7\x8d',
'MyanmarLetterMonNga': '\xe1\x81\x9a',
'HangulSyllableKep': '\xec\xbc\x9a',
'TeluguLetterDza': '\xe0\xb1\x99',
'CjkUnifiedIdeograph7b16': '\xe7\xac\x96',
'CjkUnifiedIdeograph7b17': '\xe7\xac\x97',
'CjkUnifiedIdeograph7b14': '\xe7\xac\x94',
'CjkUnifiedIdeograph7b15': '\xe7\xac\x95',
'CjkUnifiedIdeograph7b12': '\xe7\xac\x92',
'CjkUnifiedIdeograph7b13': '\xe7\xac\x93',
'CjkUnifiedIdeograph7b10': '\xe7\xac\x90',
'CjkUnifiedIdeograph7b11': '\xe7\xac\x91',
'CjkStrokeWg': '\xe3\x87\x81',
'Macron': '\xc2\xaf',
'ArabicLigatureDadWithAlefMaksuraIsolatedForm': '\xef\xb4\x87',
'CjkUnifiedIdeograph7b18': '\xe7\xac\x98',
'CjkUnifiedIdeograph7b19': '\xe7\xac\x99',
'UnicodeCharacter0xf116': '\xef\x84\x96',
'HangulSyllableKwegg': '\xed\x80\x9a',
'HangulSyllableMonj': '\xeb\xaa\xad',
'HangulSyllableMonh': '\xeb\xaa\xae',
'HangulSyllableJinj': '\xec\xa7\x85',
'HangulSyllableMong': '\xeb\xaa\xbd',
'UnicodeCharacter0xea39': '\xee\xa8\xb9',
'HangulSyllableGgolb': '\xea\xbc\xb7',
'UnicodeCharacter0xdf39': '\xed\xbc\xb9',
'UnicodeCharacter0xe5a5': '\xee\x96\xa5',
'UnicodeCharacter0xda74': '\xed\xa9\xb4',
'UnicodeCharacter0xebe0': '\xee\xaf\xa0',
'CircleWithUpperHalfBlack': '\xe2\x97\x93',
'ArrowPointingDownwardsThenCurvingLeftwards': '\xe2\xa4\xb6',
'MongolianLetterAliGaliBaluda': '\xe1\xa2\x85',
'UnicodeCharacter0xe64e': '\xee\x99\x8e',
'CjkUnifiedIdeograph68fe': '\xe6\xa3\xbe',
'HeavySouthEastArrow': '\xe2\x9e\x98',
'UnicodeCharacter0xe002': '\xee\x80\x82',
'UnicodeCharacter0xea38': '\xee\xa8\xb8',
'CjkUnifiedIdeograph7b1f': '\xe7\xac\x9f',
'CjkUnifiedIdeograph7b1d': '\xe7\xac\x9d',
'OriyaLetterSa': '\xe0\xac\xb8',
'CjkUnifiedIdeograph7b1b': '\xe7\xac\x9b',
'CjkUnifiedIdeograph7b1c': '\xe7\xac\x9c',
'CjkUnifiedIdeograph7b1a': '\xe7\xac\x9a',
'UnicodeCharacter0xda75': '\xed\xa9\xb5',
'ArabicLigatureSeenWithHahWithJeemInitialForm': '\xef\xb5\x9c',
'HangulSyllableRas': '\xeb\x9e\x8f',
'HangulSyllableCot': '\xec\xb4\xa1',
'CjkUnifiedIdeograph74bc': '\xe7\x92\xbc',
'Counterbore': '\xe2\x8c\xb4',
'KannadaLetterVa': '\xe0\xb2\xb5',
'UnicodeCharacter0xe5a7': '\xee\x96\xa7',
'LongRightwardsArrow': '\xe2\x9f\xb6',
'HangulSyllableGgwin': '\xeb\x80\x90',
'p': '\xcf\x80',
'UnicodeCharacter0xe7e8': '\xee\x9f\xa8',
'HangulSyllableCop': '\xec\xb4\xa2',
'UnicodeCharacter0xda76': '\xed\xa9\xb6',
'UnicodeCharacter0xde5b': '\xed\xb9\x9b',
'EthiopicSyllableWaa': '\xe1\x8b\x8b',
'CjkUnifiedIdeograph87d5': '\xe8\x9f\x95',
'CjkUnifiedIdeograph87d4': '\xe8\x9f\x94',
'BengaliVowelSignAi': '\xe0\xa7\x88',
'CjkUnifiedIdeograph87d6': '\xe8\x9f\x96',
'CjkUnifiedIdeograph87d1': '\xe8\x9f\x91',
'CjkUnifiedIdeograph6a7f': '\xe6\xa9\xbf',
'CjkUnifiedIdeograph71d8': '\xe7\x87\x98',
'CjkUnifiedIdeograph71d9': '\xe7\x87\x99',
'CyrillicCapitalLetterKaWithDescender': '\xd2\x9a',
'CjkUnifiedIdeograph71d7': '\xe7\x87\x97',
'BengaliVowelSignAa': '\xe0\xa6\xbe',
'CjkUnifiedIdeograph71d5': '\xe7\x87\x95',
'CjkUnifiedIdeograph71d2': '\xe7\x87\x92',
'CjkUnifiedIdeograph71d3': '\xe7\x87\x93',
'CjkUnifiedIdeograph71d0': '\xe7\x87\x90',
'LatinSmallLetterBWithPalatalHook': '\xe1\xb6\x80',
'CjkUnifiedIdeograph44f8': '\xe4\x93\xb8',
'CjkUnifiedIdeograph44f9': '\xe4\x93\xb9',
'CjkUnifiedIdeograph44f0': '\xe4\x93\xb0',
'CjkUnifiedIdeograph44f1': '\xe4\x93\xb1',
'CjkUnifiedIdeograph44f2': '\xe4\x93\xb2',
'CjkUnifiedIdeograph44f3': '\xe4\x93\xb3',
'CjkUnifiedIdeograph44f4': '\xe4\x93\xb4',
'CjkUnifiedIdeograph44f5': '\xe4\x93\xb5',
'CjkUnifiedIdeograph44f6': '\xe4\x93\xb6',
'CjkUnifiedIdeograph44f7': '\xe4\x93\xb7',
'HangulSyllableHals': '\xed\x95\xa4',
'GreekCapitalLetterOmicronWithPsili': '\xe1\xbd\x88',
'HebrewLetterBetWithDagesh': '\xef\xac\xb1',
'HangzhouNumeralTwenty': '\xe3\x80\xb9',
'ArmenianSmallLetterCo': '\xd6\x81',
'MyanmarLetterJa': '\xe1\x80\x87',
'UnicodeCharacter0xe7e5': '\xee\x9f\xa5',
'GeorgianLetterDon': '\xe1\x83\x93',
'UnicodeCharacter0xda70': '\xed\xa9\xb0',
'CjkUnifiedIdeograph74bd': '\xe7\x92\xbd',
'UnicodeCharacter0xf014': '\xef\x80\x94',
'CjkUnifiedIdeograph7f53': '\xe7\xbd\x93',
'SaurashtraLetterVocalicRr': '\xea\xa2\x89',
'DevanagariLetterIi': '\xe0\xa4\x88',
'MyanmarLetterEasternPwoKarenYwa': '\xe1\x81\xaf',
'UnicodeCharacter0xe7e1': '\xee\x9f\xa1',
'UnicodeCharacter0xe5a0': '\xee\x96\xa0',
'CjkUnifiedIdeograph44fa': '\xe4\x93\xba',
'CjkUnifiedIdeograph44fb': '\xe4\x93\xbb',
'CjkUnifiedIdeograph44fc': '\xe4\x93\xbc',
'CjkUnifiedIdeograph44fd': '\xe4\x93\xbd',
'CjkUnifiedIdeograph44fe': '\xe4\x93\xbe',
'CjkUnifiedIdeograph44ff': '\xe4\x93\xbf',
'CjkUnifiedIdeograph8a3f': '\xe8\xa8\xbf',
'CjkUnifiedIdeograph87de': '\xe8\x9f\x9e',
'CombiningCommaAbove': '\xcc\x93',
'CjkUnifiedIdeograph87df': '\xe8\x9f\x9f',
'CjkUnifiedIdeograph87da': '\xe8\x9f\x9a',
'UnicodeCharacter0xdaa9': '\xed\xaa\xa9',
'CjkUnifiedIdeograph87dc': '\xe8\x9f\x9c',
'CjkUnifiedIdeograph87db': '\xe8\x9f\x9b',
'CjkUnifiedIdeograph71df': '\xe7\x87\x9f',
'CjkUnifiedIdeograph6783': '\xe6\x9e\x83',
'CjkUnifiedIdeograph71dd': '\xe7\x87\x9d',
'CjkUnifiedIdeograph71de': '\xe7\x87\x9e',
'HexagramForInnerTruth': '\xe4\xb7\xbc',
'CjkUnifiedIdeograph71dc': '\xe7\x87\x9c',
'CjkUnifiedIdeograph71da': '\xe7\x87\x9a',
'UnicodeCharacter0xe5a3': '\xee\x96\xa3',
'UnicodeCharacter0xf6a9': '\xef\x9a\xa9',
'EthiopicSyllableVee': '\xe1\x89\xac',
'HangulSyllableNwaebs': '\xeb\x87\x82',
'UnicodeCharacter0xda72': '\xed\xa9\xb2',
'HangulSyllableJjyaelb': '\xec\xa8\xbb',
'UnicodeCharacter0xe7e9': '\xee\x9f\xa9',
'YiRadicalHxuo': '\xea\x92\x9f',
'CjkUnifiedIdeograph7f59': '\xe7\xbd\x99',
'UnicodeCharacter0xe004': '\xee\x80\x84',
'UnicodeCharacter0xe18b': '\xee\x86\x8b',
'UnicodeCharacter0xe18c': '\xee\x86\x8c',
'UnicodeCharacter0xe18a': '\xee\x86\x8a',
'UnicodeCharacter0xe18f': '\xee\x86\x8f',
'UnicodeCharacter0xe18d': '\xee\x86\x8d',
'UnicodeCharacter0xe18e': '\xee\x86\x8e',
'MyanmarLetterEasternPwoKarenNna': '\xe1\x81\xae',
'UnicodeCharacter0xe271': '\xee\x89\xb1',
'UnicodeCharacter0xe272': '\xee\x89\xb2',
'UnicodeCharacter0xe273': '\xee\x89\xb3',
'UnicodeCharacter0xe274': '\xee\x89\xb4',
'UnicodeCharacter0xda73': '\xed\xa9\xb3',
'UnicodeCharacter0xe276': '\xee\x89\xb6',
'UnicodeCharacter0xe277': '\xee\x89\xb7',
'UnicodeCharacter0xa69b': '\xea\x9a\x9b',
'UnicodeCharacter0xa69c': '\xea\x9a\x9c',
'LeftSquareBracketExtension': '\xe2\x8e\xa2',
'UnicodeCharacter0xa69f': '\xea\x9a\x9f',
'CjkUnifiedIdeograph86ae': '\xe8\x9a\xae',
'UnicodeCharacter0xa69d': '\xea\x9a\x9d',
'UnicodeCharacter0xa69e': '\xea\x9a\x9e',
'HangulSyllableRyiss': '\xeb\xa6\xa4',
'CjkUnifiedIdeograph86ad': '\xe8\x9a\xad',
'UnicodeCharacter0xe58b': '\xee\x96\x8b',
'BoxDrawingsDoubleVertical': '\xe2\x95\x91',
'UnicodeCharacter0xebf6': '\xee\xaf\xb6',
'HangulChoseongSsangpieup': '\xe1\x84\x88',
'UnicodeCharacter0xe83a': '\xee\xa0\xba',
'LatinSmallLetterOWithTildeAndDiaeresis': '\xe1\xb9\x8f',
'UnicodeCharacter0xecf2': '\xee\xb3\xb2',
'UnicodeCharacter0xecf3': '\xee\xb3\xb3',
'UnicodeCharacter0xecf0': '\xee\xb3\xb0',
'UnicodeCharacter0xe4e5': '\xee\x93\xa5',
'UnicodeCharacter0xecf6': '\xee\xb3\xb6',
'UnicodeCharacter0xecf7': '\xee\xb3\xb7',
'UnicodeCharacter0xecf4': '\xee\xb3\xb4',
'UnicodeCharacter0xecf5': '\xee\xb3\xb5',
'UnicodeCharacter0xebf3': '\xee\xaf\xb3',
'UnicodeCharacter0xecf8': '\xee\xb3\xb8',
'UnicodeCharacter0xecf9': '\xee\xb3\xb9',
'MeasuredAngleWithOpenArmEndingInArrowPointingLeftAndUp': '\xe2\xa6\xad',
'UnicodeCharacter0xa698': '\xea\x9a\x98',
'UnicodeCharacter0xa699': '\xea\x9a\x99',
'UnicodeCharacter0xe182': '\xee\x86\x82',
'UnicodeCharacter0xe183': '\xee\x86\x83',
'LatinSmallLetterEWithDoubleGrave': '\xc8\x85',
'UnicodeCharacter0xe181': '\xee\x86\x81',
'UnicodeCharacter0xe186': '\xee\x86\x86',
'UnicodeCharacter0xe187': '\xee\x86\x87',
'UnicodeCharacter0xe184': '\xee\x86\x84',
'UnicodeCharacter0xe185': '\xee\x86\x85',
'UnicodeCharacter0xf561': '\xef\x95\xa1',
'AplFunctionalSymbolUpTackJot': '\xe2\x8d\x95',
'UnicodeCharacter0xe188': '\xee\x86\x88',
'YiSyllableHnuox': '\xea\x85\xb4',
'UnicodeCharacter0xe27d': '\xee\x89\xbd',
'UnicodeCharacter0xe27e': '\xee\x89\xbe',
'UnicodeCharacter0xe27f': '\xee\x89\xbf',
'UnicodeCharacter0xf566': '\xef\x95\xa6',
'UnicodeCharacter0xecfb': '\xee\xb3\xbb',
'UnicodeCharacter0xecfc': '\xee\xb3\xbc',
'HangulSyllableSsyalm': '\xec\x8c\xba',
'UnicodeCharacter0xecfa': '\xee\xb3\xba',
'UnicodeCharacter0xecff': '\xee\xb3\xbf',
'CjkUnifiedIdeograph8a3c': '\xe8\xa8\xbc',
'UnicodeCharacter0xecfd': '\xee\xb3\xbd',
'HangulSyllableSsyalh': '\xec\x8c\xbf',
'HangulSyllableSsyalg': '\xec\x8c\xb9',
'HangulSyllableSsyalb': '\xec\x8c\xbb',
'LatinSmallLetterAWithCircumflex': '\xc3\xa2',
'CyrillicSmallLetterIzhitsa': '\xd1\xb5',
'NorthWestArrowToLongBar': '\xe2\x86\xb8',
'SubsetOfWithNotEqualTo': '\xe2\x8a\x8a',
'HangulSyllableSsyalt': '\xec\x8c\xbd',
'KangxiRadicalScholar': '\xe2\xbc\xa0',
'HangulLetterKiyeok': '\xe3\x84\xb1',
'HangulSyllableSsyalp': '\xec\x8c\xbe',
'CjkUnifiedIdeograph8608': '\xe8\x98\x88',
'CjkUnifiedIdeograph8609': '\xe8\x98\x89',
'UnicodeCharacter0xa31': '\xe0\xa8\xb1',
'ThaiCharacterLakkhangyao': '\xe0\xb9\x85',
'UnicodeCharacter0xa37': '\xe0\xa8\xb7',
'NewTaiLueVowelSignO': '\xe1\xa6\xb7',
'UnicodeCharacter0xa34': '\xe0\xa8\xb4',
'CjkUnifiedIdeograph8600': '\xe8\x98\x80',
'CjkUnifiedIdeograph8601': '\xe8\x98\x81',
'CjkUnifiedIdeograph8602': '\xe8\x98\x82',
'CjkUnifiedIdeograph8603': '\xe8\x98\x83',
'GurmukhiDigitFive': '\xe0\xa9\xab',
'CjkUnifiedIdeograph8605': '\xe8\x98\x85',
'CjkUnifiedIdeograph8606': '\xe8\x98\x86',
'CjkUnifiedIdeograph8607': '\xe8\x98\x87',
'LatinCapitalLetterYWithAcute': '\xc3\x9d',
'CjkUnifiedIdeograph6a73': '\xe6\xa9\xb3',
'NewTaiLueVowelSignU': '\xe1\xa6\xb3',
'CjkUnifiedIdeograph67cf': '\xe6\x9f\x8f',
'HangulSyllableTyebs': '\xed\x86\x96',
'CjkUnifiedIdeograph67cd': '\xe6\x9f\x8d',
'CjkUnifiedIdeograph67ce': '\xe6\x9f\x8e',
'CjkUnifiedIdeograph67cb': '\xe6\x9f\x8b',
'CopticCapitalLetterOldNubianNgi': '\xe2\xb3\x9e',
'CjkUnifiedIdeograph67ca': '\xe6\x9f\x8a',
'UnicodeCharacter0xe4e1': '\xee\x93\xa1',
'HangulSyllableJjobs': '\xec\xab\x8e',
'UnicodeCharacter0x2d6a': '\xe2\xb5\xaa',
'UnicodeCharacter0x2d6b': '\xe2\xb5\xab',
'UnicodeCharacter0x2d6c': '\xe2\xb5\xac',
'UnicodeCharacter0x2d6d': '\xe2\xb5\xad',
'UnicodeCharacter0x2d6e': '\xe2\xb5\xae',
'HangulSyllableRyaegs': '\xeb\x9f\x93',
'KannadaDigitSix': '\xe0\xb3\xac',
'UnicodeCharacter0xda79': '\xed\xa9\xb9',
'UnicodeCharacter0xe4e2': '\xee\x93\xa2',
'HangulSyllableNwebs': '\xeb\x89\x8e',
'UnicodeCharacter0xa3b': '\xe0\xa8\xbb',
'UnicodeCharacter0xa3a': '\xe0\xa8\xba',
'UnicodeCharacter0xa3d': '\xe0\xa8\xbd',
'CjkUnifiedIdeograph860a': '\xe8\x98\x8a',
'CjkUnifiedIdeograph860b': '\xe8\x98\x8b',
'CjkUnifiedIdeograph860c': '\xe8\x98\x8c',
'CjkUnifiedIdeograph860d': '\xe8\x98\x8d',
'CjkUnifiedIdeograph860e': '\xe8\x98\x8e',
'CjkUnifiedIdeograph860f': '\xe8\x98\x8f',
'CjkUnifiedIdeograph95f8': '\xe9\x97\xb8',
'ArabicLetterZainIsolatedForm': '\xef\xba\xaf',
'UnicodeCharacter0x2d66': '\xe2\xb5\xa6',
'UnicodeCharacter0x2d67': '\xe2\xb5\xa7',
'UnicodeCharacter0x2d68': '\xe2\xb5\xa8',
'UnicodeCharacter0x2d69': '\xe2\xb5\xa9',
'UnicodeCharacter0xf598': '\xef\x96\x98',
'CjkUnifiedIdeograph67c6': '\xe6\x9f\x86',
'CjkUnifiedIdeograph67c7': '\xe6\x9f\x87',
'CjkUnifiedIdeograph67c4': '\xe6\x9f\x84',
'CjkUnifiedIdeograph67c5': '\xe6\x9f\x85',
'CjkUnifiedIdeograph67c2': '\xe6\x9f\x82',
'CjkUnifiedIdeograph67c3': '\xe6\x9f\x83',
'CjkUnifiedIdeograph67c0': '\xe6\x9f\x80',
'CjkUnifiedIdeograph67c1': '\xe6\x9f\x81',
'HangulSyllableDwigg': '\xeb\x92\xa6',
'ArabicLigatureSeenWithMeemInitialForm': '\xef\xb2\xb0',
'CjkUnifiedIdeograph67c8': '\xe6\x9f\x88',
'CjkUnifiedIdeograph67c9': '\xe6\x9f\x89',
'EthiopicSyllablePharyngealO': '\xe1\x8b\x96',
'EthiopicSyllablePharyngealI': '\xe1\x8b\x92',
'HangulSyllableJjoess': '\xec\xac\xa4',
'EthiopicSyllablePharyngealE': '\xe1\x8b\x95',
'BalineseMusicalSymbolDung': '\xe1\xad\xa3',
'NewTaiLueDigitZero': '\xe1\xa7\x90',
'EthiopicSyllableChoa': '\xe2\xb6\x90',
'VaiSyllableA': '\xea\x95\x89',
'CjkUnifiedIdeograph9ace': '\xe9\xab\x8e',
'CjkUnifiedIdeograph9acd': '\xe9\xab\x8d',
'EthiopicSyllablePharyngealU': '\xe1\x8b\x91',
'CjkUnifiedIdeograph9acf': '\xe9\xab\x8f',
'CjkUnifiedIdeograph9aca': '\xe9\xab\x8a',
'CjkUnifiedIdeograph9acc': '\xe9\xab\x8c',
'CjkUnifiedIdeograph9acb': '\xe9\xab\x8b',
'CombiningAnticlockwiseArrowAbove': '\xe2\x83\x94',
'HangulSyllableTyelg': '\xed\x86\x8d',
'UnicodeCharacter0xe3d5': '\xee\x8f\x95',
'HangulSyllableTyaebs': '\xed\x84\xa6',
'CjkUnifiedIdeograph460d': '\xe4\x98\x8d',
'CjkUnifiedIdeograph460e': '\xe4\x98\x8e',
'CjkUnifiedIdeograph460f': '\xe4\x98\x8f',
'CjkUnifiedIdeograph460a': '\xe4\x98\x8a',
'CjkUnifiedIdeograph460b': '\xe4\x98\x8b',
'CjkUnifiedIdeograph460c': '\xe4\x98\x8c',
'HangulSyllableGyoj': '\xea\xb5\xa6',
'UnicodeCharacter0xf845': '\xef\xa1\x85',
'SylotiNagriLetterJho': '\xea\xa0\x8f',
'UnicodeCharacter0xf843': '\xef\xa1\x83',
'LatinSmallLetterHWithCaron': '\xc8\x9f',
'LatinLetterSmallCapitalGWithHook': '\xca\x9b',
'SylotiNagriVowelSignU': '\xea\xa0\xa5',
'CjkUnifiedIdeograph9ac9': '\xe9\xab\x89',
'CjkUnifiedIdeograph9ac8': '\xe9\xab\x88',
'CjkUnifiedIdeograph9ac5': '\xe9\xab\x85',
'CjkUnifiedIdeograph9ac4': '\xe9\xab\x84',
'CjkUnifiedIdeograph9ac7': '\xe9\xab\x87',
'CjkUnifiedIdeograph9ac6': '\xe9\xab\x86',
'IdeographicTelegraphSymbolForDaySeventeen': '\xe3\x8f\xb0',
'CjkUnifiedIdeograph9ac0': '\xe9\xab\x80',
'CjkUnifiedIdeograph9ac3': '\xe9\xab\x83',
'CjkUnifiedIdeograph9ac2': '\xe9\xab\x82',
'SylotiNagriVowelSignE': '\xea\xa0\xa6',
'SylotiNagriVowelSignA': '\xea\xa0\xa3',
'LatinSmallLetterLongS': '\xc5\xbf',
'SylotiNagriVowelSignI': '\xea\xa0\xa4',
'HangulSyllableCyug': '\xec\xb8\x85',
'HangulSyllableCyud': '\xec\xb8\x8b',
'HangulSyllableCyub': '\xec\xb8\x95',
'HangulSyllableCyuc': '\xec\xb8\x9b',
'HangulSyllableCyun': '\xec\xb8\x88',
'CjkUnifiedIdeograph9314': '\xe9\x8c\x94',
'HangulSyllableCyum': '\xec\xb8\x94',
'HangulSyllableCyuj': '\xec\xb8\x9a',
'HangulSyllableCyuk': '\xec\xb8\x9c',
'HangulSyllableCyuh': '\xec\xb8\x9f',
'HangulSyllableCyut': '\xec\xb8\x9d',
'CjkUnifiedIdeograph4608': '\xe4\x98\x88',
'CjkUnifiedIdeograph4609': '\xe4\x98\x89',
'HangulSyllableCyup': '\xec\xb8\x9e',
'CjkUnifiedIdeograph4604': '\xe4\x98\x84',
'CjkUnifiedIdeograph4605': '\xe4\x98\x85',
'CjkUnifiedIdeograph4606': '\xe4\x98\x86',
'CjkUnifiedIdeograph4607': '\xe4\x98\x87',
'CjkUnifiedIdeograph4600': '\xe4\x98\x80',
'CjkUnifiedIdeograph4601': '\xe4\x98\x81',
'CjkUnifiedIdeograph4602': '\xe4\x98\x82',
'CjkUnifiedIdeograph4603': '\xe4\x98\x83',
'CjkUnifiedIdeograph8f35': '\xe8\xbc\xb5',
'CjkUnifiedIdeograph8f34': '\xe8\xbc\xb4',
'CjkUnifiedIdeograph8f37': '\xe8\xbc\xb7',
'CjkUnifiedIdeograph8f36': '\xe8\xbc\xb6',
'CjkUnifiedIdeograph8f31': '\xe8\xbc\xb1',
'CjkUnifiedIdeograph8f30': '\xe8\xbc\xb0',
'CjkUnifiedIdeograph8f33': '\xe8\xbc\xb3',
'CjkUnifiedIdeograph8f32': '\xe8\xbc\xb2',
'CjkUnifiedIdeograph9317': '\xe9\x8c\x97',
'HangulSyllableMyo': '\xeb\xac\x98',
'CjkUnifiedIdeograph8f39': '\xe8\xbc\xb9',
'CjkUnifiedIdeograph8f38': '\xe8\xbc\xb8',
'UnicodeCharacter0x175c': '\xe1\x9d\x9c',
'UnicodeCharacter0x175b': '\xe1\x9d\x9b',
'UnicodeCharacter0x175a': '\xe1\x9d\x9a',
'UnicodeCharacter0x175f': '\xe1\x9d\x9f',
'UnicodeCharacter0x175e': '\xe1\x9d\x9e',
'UnicodeCharacter0x175d': '\xe1\x9d\x9d',
'et': '\xce\xb7',
'ShortRightwardsArrowAboveLeftwardsArrow': '\xe2\xa5\x84',
'DevanagariLetterYya': '\xe0\xa5\x9f',
'HangulSyllableRyag': '\xeb\x9e\xb5',
'UnicodeCharacter0xdf7d': '\xed\xbd\xbd',
'UnicodeCharacter0xe228': '\xee\x88\xa8',
'CjkUnifiedIdeograph86a9': '\xe8\x9a\xa9',
'CircledKoreanCharacterChamko': '\xe3\x89\xbc',
'CjkUnifiedIdeograph48e0': '\xe4\xa3\xa0',
'CjkUnifiedIdeograph48e3': '\xe4\xa3\xa3',
'CjkUnifiedIdeograph48e2': '\xe4\xa3\xa2',
'CjkUnifiedIdeograph48e5': '\xe4\xa3\xa5',
'CjkUnifiedIdeograph48e4': '\xe4\xa3\xa4',
'CjkUnifiedIdeograph48e7': '\xe4\xa3\xa7',
'CjkUnifiedIdeograph48e6': '\xe4\xa3\xa6',
'CjkUnifiedIdeograph48e9': '\xe4\xa3\xa9',
'CjkUnifiedIdeograph48e8': '\xe4\xa3\xa8',
'CjkUnifiedIdeograph5f63': '\xe5\xbd\xa3',
'RightwardsDoubleArrow': '\xe2\x87\x92',
'HangulSyllableGgolt': '\xea\xbc\xb9',
'UnicodeCharacter0x1757': '\xe1\x9d\x97',
'UnicodeCharacter0x1756': '\xe1\x9d\x96',
'UnicodeCharacter0x1755': '\xe1\x9d\x95',
'UnicodeCharacter0x1754': '\xe1\x9d\x94',
'BalineseLetterEkara': '\xe1\xac\x8f',
'UnicodeCharacter0x1758': '\xe1\x9d\x98',
'UnicodeCharacter0xf372': '\xef\x8d\xb2',
'UnicodeCharacter0x11a5': '\xe1\x86\xa5',
'UnicodeCharacter0x11a4': '\xe1\x86\xa4',
'UnicodeCharacter0x11a7': '\xe1\x86\xa7',
'UpwardsWhiteArrow': '\xe2\x87\xa7',
'CjkUnifiedIdeograph8f3a': '\xe8\xbc\xba',
'UnicodeCharacter0x11a3': '\xe1\x86\xa3',
'CjkUnifiedIdeograph8f3b': '\xe8\xbc\xbb',
'HebrewAccentPazer': '\xd6\xa1',
'CjkUnifiedIdeograph5f66': '\xe5\xbd\xa6',
'UnicodeCharacter0xddd6': '\xed\xb7\x96',
'UnicodeCharacter0xf375': '\xef\x8d\xb5',
'CjkUnifiedIdeograph48ea': '\xe4\xa3\xaa',
'CjkUnifiedIdeograph48ec': '\xe4\xa3\xac',
'CjkUnifiedIdeograph48eb': '\xe4\xa3\xab',
'CjkUnifiedIdeograph48ee': '\xe4\xa3\xae',
'CjkUnifiedIdeograph48ed': '\xe4\xa3\xad',
'RunicTvimadurSymbol': '\xe1\x9b\xaf',
'CjkUnifiedIdeograph48ef': '\xe4\xa3\xaf',
'KangxiRadicalAxe': '\xe2\xbd\x84',
'ArabicSukunMedialForm': '\xef\xb9\xbf',
'CjkUnifiedIdeograph86a6': '\xe8\x9a\xa6',
'UnicodeCharacter0xf378': '\xef\x8d\xb8',
'UnicodeCharacter0xddd2': '\xed\xb7\x92',
'UnicodeCharacter0xe56f': '\xee\x95\xaf',
'UnicodeCharacter0xf379': '\xef\x8d\xb9',
'UnicodeCharacter0xf138': '\xef\x84\xb8',
'HangulSyllableJjwels': '\xec\xae\x8c',
'HangulSyllableRyaess': '\xeb\x9f\xa4',
'HangulSyllableGgweogs': '\xea\xbf\x97',
'CjkUnifiedIdeograph3498': '\xe3\x92\x98',
'CjkUnifiedIdeograph3499': '\xe3\x92\x99',
'HangulSyllableHweobs': '\xed\x9b\xb2',
'CjkUnifiedIdeograph3490': '\xe3\x92\x90',
'CjkUnifiedIdeograph3491': '\xe3\x92\x91',
'CjkUnifiedIdeograph3492': '\xe3\x92\x92',
'CjkUnifiedIdeograph3493': '\xe3\x92\x93',
'CjkUnifiedIdeograph3494': '\xe3\x92\x94',
'CjkUnifiedIdeograph3495': '\xe3\x92\x95',
'CjkUnifiedIdeograph3496': '\xe3\x92\x96',
'CjkUnifiedIdeograph3497': '\xe3\x92\x97',
'CjkCompatibilityIdeographf90f': '\xef\xa4\x8f',
'CjkUnifiedIdeograph5265': '\xe5\x89\xa5',
'CjkUnifiedIdeograph5264': '\xe5\x89\xa4',
'CjkUnifiedIdeograph5267': '\xe5\x89\xa7',
'CjkUnifiedIdeograph5266': '\xe5\x89\xa6',
'CjkUnifiedIdeograph5261': '\xe5\x89\xa1',
'CjkUnifiedIdeograph5260': '\xe5\x89\xa0',
'CjkUnifiedIdeograph5263': '\xe5\x89\xa3',
'CjkUnifiedIdeograph5262': '\xe5\x89\xa2',
'CjkUnifiedIdeograph58ab': '\xe5\xa2\xab',
'CjkUnifiedIdeograph58ac': '\xe5\xa2\xac',
'CjkUnifiedIdeograph58aa': '\xe5\xa2\xaa',
'CjkUnifiedIdeograph5269': '\xe5\x89\xa9',
'CjkUnifiedIdeograph5268': '\xe5\x89\xa8',
'CjkUnifiedIdeograph58ad': '\xe5\xa2\xad',
'CjkUnifiedIdeograph58ae': '\xe5\xa2\xae',
'CjkUnifiedIdeograph6518': '\xe6\x94\x98',
'CjkUnifiedIdeograph6519': '\xe6\x94\x99',
'CjkUnifiedIdeograph939c': '\xe9\x8e\x9c',
'CjkUnifiedIdeograph6512': '\xe6\x94\x92',
'CjkUnifiedIdeograph349a': '\xe3\x92\x9a',
'CjkUnifiedIdeograph349b': '\xe3\x92\x9b',
'CjkUnifiedIdeograph349c': '\xe3\x92\x9c',
'CjkUnifiedIdeograph349d': '\xe3\x92\x9d',
'CjkUnifiedIdeograph349e': '\xe3\x92\x9e',
'CjkUnifiedIdeograph349f': '\xe3\x92\x9f',
'CjkUnifiedIdeograph6515': '\xe6\x94\x95',
'YiSyllableBax': '\xea\x80\x9f',
'LatinSmallLetterOWithDiaeresis': '\xc3\xb6',
'LatinCapitalLetterPWithSquirrelTail': '\xea\x9d\x94',
'HangulSyllableRyam': '\xeb\x9f\x84',
'YiSyllableBap': '\xea\x80\xa1',
'YiSyllableBat': '\xea\x80\x9e',
'UnicodeCharacter0xf84c': '\xef\xa1\x8c',
'CjkUnifiedIdeograph526e': '\xe5\x89\xae',
'CjkUnifiedIdeograph526d': '\xe5\x89\xad',
'CjkUnifiedIdeograph58a8': '\xe5\xa2\xa8',
'CjkUnifiedIdeograph526f': '\xe5\x89\xaf',
'CjkUnifiedIdeograph526a': '\xe5\x89\xaa',
'CjkUnifiedIdeograph526c': '\xe5\x89\xac',
'CjkUnifiedIdeograph526b': '\xe5\x89\xab',
'CjkUnifiedIdeograph58a2': '\xe5\xa2\xa2',
'CjkUnifiedIdeograph58a3': '\xe5\xa2\xa3',
'CjkUnifiedIdeograph58a0': '\xe5\xa2\xa0',
'CjkUnifiedIdeograph58a1': '\xe5\xa2\xa1',
'CjkUnifiedIdeograph58a6': '\xe5\xa2\xa6',
'CjkUnifiedIdeograph58a7': '\xe5\xa2\xa7',
'CjkUnifiedIdeograph58a4': '\xe5\xa2\xa4',
'CjkUnifiedIdeograph58a5': '\xe5\xa2\xa5',
'HexagramForTheWell': '\xe4\xb7\xaf',
'ArabicLigatureGhainWithMeemInitialForm': '\xef\xb2\xbd',
'TibetanSubjoinedLetterTsha': '\xe0\xbe\xaa',
'UnicodeCharacter0xe75b': '\xee\x9d\x9b',
'GreekSmallLetterEta': '\xce\xb7',
'CjkUnifiedIdeograph9001': '\xe9\x80\x81',
'CjkUnifiedIdeograph9000': '\xe9\x80\x80',
'CjkUnifiedIdeograph9003': '\xe9\x80\x83',
'HangulSyllableJjwaelh': '\xec\xac\x83',
'Phi': '\xce\xa6',
'HangulSyllableJjwelh': '\xec\xae\x8f',
'HangulSyllableJjwaelm': '\xec\xab\xbe',
'IdeographicAnnotationOneMark': '\xe3\x86\x92',
'CjkUnifiedIdeograph764a': '\xe7\x99\x8a',
'UnicodeCharacter0xe0be': '\xee\x82\xbe',
'CjkUnifiedIdeograph764c': '\xe7\x99\x8c',
'CjkUnifiedIdeograph764b': '\xe7\x99\x8b',
'CjkUnifiedIdeograph764e': '\xe7\x99\x8e',
'CjkUnifiedIdeograph764d': '\xe7\x99\x8d',
'HangulSyllableJjwaelg': '\xec\xab\xbd',
'EqualsSignAndSlantedParallel': '\xe2\xa7\xa3',
'HangulSyllableRyan': '\xeb\x9e\xb8',
'CjkUnifiedIdeograph7d3a': '\xe7\xb4\xba',
'HangulSyllableMye': '\xeb\xaa\x8c',
'HangulSyllableJjwaelp': '\xec\xac\x82',
'HangulSyllableJjwaels': '\xec\xac\x80',
'UnicodeCharacter0xe0bf': '\xee\x82\xbf',
'HangulSyllableJjwaelt': '\xec\xac\x81',
'MyanmarVowelSignShanAa': '\xe1\x82\x83',
'LatinSmallLetterIWithDiaeresisAndAcute': '\xe1\xb8\xaf',
'UnicodeCharacter0xa9dd': '\xea\xa7\x9d',
'CombiningMacronBelow': '\xcc\xb1',
'KannadaSignVisarga': '\xe0\xb2\x83',
'HangulSyllableBwas': '\xeb\xb4\xa3',
'ArmenianSmallLigatureMenEch': '\xef\xac\x94',
'UnicodeCharacter0xfe2f': '\xef\xb8\xaf',
'UnicodeCharacter0xa8eb': '\xea\xa3\xab',
'HangulSyllableMyaelh': '\xeb\xa8\xab',
'HangulSyllableHoen': '\xed\x9a\x90',
'CjkUnifiedIdeograph9552': '\xe9\x95\x92',
'IdeographicTelegraphSymbolForDayThirty': '\xe3\x8f\xbd',
'CjkUnifiedIdeograph7649': '\xe7\x99\x89',
'CjkUnifiedIdeograph7648': '\xe7\x99\x88',
'YiSyllableNjop': '\xea\x90\xbe',
'CjkUnifiedIdeograph9aef': '\xe9\xab\xaf',
'CjkUnifiedIdeograph7641': '\xe7\x99\x81',
'CjkUnifiedIdeograph7640': '\xe7\x99\x80',
'CjkUnifiedIdeograph7643': '\xe7\x99\x83',
'CjkUnifiedIdeograph7642': '\xe7\x99\x82',
'CjkUnifiedIdeograph7645': '\xe7\x99\x85',
'CjkUnifiedIdeograph7644': '\xe7\x99\x84',
'LatinSmallLetterIWithDoubleGrave': '\xc8\x89',
'KhmerSymbolLekAttakBuon': '\xe1\x9f\xb4',
'HangulSyllableCyibs': '\xec\xb9\x8e',
'CjkUnifiedIdeograph9551': '\xe9\x95\x91',
'CjkUnifiedIdeograph94ff': '\xe9\x93\xbf',
'CjkUnifiedIdeograph94fe': '\xe9\x93\xbe',
'CjkUnifiedIdeograph94fd': '\xe9\x93\xbd',
'CjkUnifiedIdeograph94fc': '\xe9\x93\xbc',
'CjkUnifiedIdeograph94fb': '\xe9\x93\xbb',
'CjkUnifiedIdeograph94fa': '\xe9\x93\xba',
'CjkUnifiedIdeograph9aeb': '\xe9\xab\xab',
'BlackLowerRightTriangle': '\xe2\x97\xa2',
'UnicodeCharacter0xda19': '\xed\xa8\x99',
'TopLeftCrop': '\xe2\x8c\x8f',
'Z': '\xce\x96',
'CjkUnifiedIdeograph3c79': '\xe3\xb1\xb9',
'CjkUnifiedIdeograph3c78': '\xe3\xb1\xb8',
'CjkUnifiedIdeograph9550': '\xe9\x95\x90',
'CjkUnifiedIdeograph3c75': '\xe3\xb1\xb5',
'CjkUnifiedIdeograph3c74': '\xe3\xb1\xb4',
'CjkUnifiedIdeograph3c77': '\xe3\xb1\xb7',
'MetricalTwoShortsOverLong': '\xe2\x8f\x95',
'CjkUnifiedIdeograph3c71': '\xe3\xb1\xb1',
'CjkUnifiedIdeograph3c70': '\xe3\xb1\xb0',
'CjkUnifiedIdeograph3c73': '\xe3\xb1\xb3',
'CjkUnifiedIdeograph3c72': '\xe3\xb1\xb2',
'HangulSyllableHoek': '\xed\x9a\xa4',
'UnicodeCharacter0xed6c': '\xee\xb5\xac',
'HangulSyllableRyeos': '\xeb\xa0\xb7',
'HangulSyllableDuc': '\xeb\x91\xa7',
'CjkUnifiedIdeograph94f9': '\xe9\x93\xb9',
'HangulChoseongChieuch': '\xe1\x84\x8e',
'CjkUnifiedIdeograph94f7': '\xe9\x93\xb7',
'CjkUnifiedIdeograph94f6': '\xe9\x93\xb6',
'CjkUnifiedIdeograph94f5': '\xe9\x93\xb5',
'CjkUnifiedIdeograph94f4': '\xe9\x93\xb4',
'CjkUnifiedIdeograph94f3': '\xe9\x93\xb3',
'CjkUnifiedIdeograph94f2': '\xe9\x93\xb2',
'CjkUnifiedIdeograph94f1': '\xe9\x93\xb1',
'CjkUnifiedIdeograph94f0': '\xe9\x93\xb0',
'HangulSyllableSyeogs': '\xec\x85\x97',
'LatinSmallLetterGWithBreve': '\xc4\x9f',
'HangulSyllableDub': '\xeb\x91\xa1',
'ArabicLigatureBehWithHahInitialForm': '\xef\xb2\x9d',
'HysteresisSymbol': '\xe2\x8e\x8e',
'LatinCapitalLetterEt': '\xea\x9d\xaa',
'CjkUnifiedIdeograph3c7e': '\xe3\xb1\xbe',
'CjkUnifiedIdeograph3c7d': '\xe3\xb1\xbd',
'HangulSyllableYeobs': '\xec\x97\xbe',
'CjkUnifiedIdeograph3c7f': '\xe3\xb1\xbf',
'CjkUnifiedIdeograph3c7a': '\xe3\xb1\xba',
'CjkUnifiedIdeograph3c7c': '\xe3\xb1\xbc',
'CjkUnifiedIdeograph3c7b': '\xe3\xb1\xbb',
'CjkUnifiedIdeograph9555': '\xe9\x95\x95',
'CjkUnifiedIdeograph707c': '\xe7\x81\xbc',
'UnicodeCharacter0xed6e': '\xee\xb5\xae',
'UnicodeCharacter0xf4be': '\xef\x92\xbe',
'UnicodeCharacter0xd917': '\xed\xa4\x97',
'CjkUnifiedIdeograph4a3b': '\xe4\xa8\xbb',
'CjkUnifiedIdeograph4a3c': '\xe4\xa8\xbc',
'CjkUnifiedIdeograph4a3a': '\xe4\xa8\xba',
'CjkUnifiedIdeograph4a3f': '\xe4\xa8\xbf',
'CjkUnifiedIdeograph4a3d': '\xe4\xa8\xbd',
'CjkUnifiedIdeograph4a3e': '\xe4\xa8\xbe',
'HeavyTriangleheadedRightwardsArrow': '\xe2\x9e\x9e',
'LeftOuterJoin': '\xe2\x9f\x95',
'CjkUnifiedIdeograph89a8': '\xe8\xa6\xa8',
'HebrewPointHatafPatah': '\xd6\xb2',
'UnicodeCharacter0xf502': '\xef\x94\x82',
'CjkUnifiedIdeograph9554': '\xe9\x95\x94',
'ArabicLetterDyehInitialForm': '\xef\xad\xb4',
'YiSyllableMgox': '\xea\x88\xba',
'ParenthesizedHangulPieupA': '\xe3\x88\x93',
'YiSyllableMgop': '\xea\x88\xbc',
'YiSyllableMgot': '\xea\x88\xb9',
'CjkUnifiedIdeograph89a4': '\xe8\xa6\xa4',
'UnicodeCharacter0xf37c': '\xef\x8d\xbc',
'UnicodeCharacter0xe0b0': '\xee\x82\xb0',
'HangulSyllableTweonh': '\xed\x89\x8e',
'UnicodeCharacter0xe646': '\xee\x99\x86',
'CjkUnifiedIdeograph89a6': '\xe8\xa6\xa6',
'UnicodeCharacter0xf37e': '\xef\x8d\xbe',
'HangulSyllableSoel': '\xec\x87\xa8',
'HangulSyllableSoem': '\xec\x87\xb0',
'HangulSyllableSoen': '\xec\x87\xa4',
'IdeographicTelegraphSymbolForDayThree': '\xe3\x8f\xa2',
'HangulSyllableSoeh': '\xec\x87\xbb',
'KangxiRadicalClaw': '\xe2\xbd\x96',
'HangulSyllableSoek': '\xec\x87\xb8',
'HangulSyllableSoed': '\xec\x87\xa7',
'KhmerSignReahmuk': '\xe1\x9f\x87',
'HangulSyllableSoeg': '\xec\x87\xa1',
'HangulSyllableSoeb': '\xec\x87\xb1',
'HangulSyllableSoec': '\xec\x87\xb7',
'HangulSyllableBunh': '\xeb\xb6\x86',
'CjkUnifiedIdeograph4a38': '\xe4\xa8\xb8',
'CjkUnifiedIdeograph4a39': '\xe4\xa8\xb9',
'TamilLetterCa': '\xe0\xae\x9a',
'CjkUnifiedIdeograph4a33': '\xe4\xa8\xb3',
'CjkUnifiedIdeograph4a30': '\xe4\xa8\xb0',
'NaryCircledDotOperator': '\xe2\xa8\x80',
'CjkUnifiedIdeograph4a36': '\xe4\xa8\xb6',
'CjkUnifiedIdeograph4a37': '\xe4\xa8\xb7',
'CjkUnifiedIdeograph4a34': '\xe4\xa8\xb4',
'CjkUnifiedIdeograph4a35': '\xe4\xa8\xb5',
'HangulSyllableGulg': '\xea\xb5\xb5',
'HangulSyllableSyulp': '\xec\x8a\x96',
'GujaratiLetterBa': '\xe0\xaa\xac',
'HangulSyllableGulb': '\xea\xb5\xb7',
'HangulSyllableGulm': '\xea\xb5\xb6',
'HangulSyllableEolh': '\xec\x97\x83',
'HangulSyllableKweols': '\xed\x80\x88',
'LaoSemivowelSignNyo': '\xe0\xba\xbd',
'HangulSyllableGulh': '\xea\xb5\xbb',
'LatinCapitalLetterUWithRingAbove': '\xc5\xae',
'HangulSyllableGult': '\xea\xb5\xb9',
'HangulSyllableDdyeogg': '\xeb\x97\x9a',
'HangulSyllableKweolm': '\xed\x80\x86',
'HangulSyllableGulp': '\xea\xb5\xba',
'HangulSyllableGuls': '\xea\xb5\xb8',
'SubscriptLeftParenthesis': '\xe2\x82\x8d',
'PermanentPaperSign': '\xe2\x99\xbe',
'GeorgianLetterXan': '\xe1\x83\xae',
'HangulSyllableRyas': '\xeb\x9f\x87',
'HangulSyllableJyulg': '\xec\xa5\xb5',
'HangulSyllableKweolg': '\xed\x80\x85',
'HangulSyllableHaen': '\xed\x95\xb8',
'HangulSyllableHaelb': '\xed\x95\xbf',
'HangulSyllableHael': '\xed\x95\xbc',
'HangulSyllableHaek': '\xed\x96\x8c',
'HangulSyllableHaej': '\xed\x96\x8a',
'DoubleStrokeNotSign': '\xe2\xab\xac',
'HangulSyllableHaelg': '\xed\x95\xbd',
'HangulSyllableHaeg': '\xed\x95\xb5',
'HangulSyllableHaed': '\xed\x95\xbb',
'HangulSyllableHaec': '\xed\x96\x8b',
'HangulSyllableHaelm': '\xed\x95\xbe',
'HangulSyllableHaelp': '\xed\x96\x82',
'UnicodeCharacter0xdbec': '\xed\xaf\xac',
'HangulSyllableHaels': '\xed\x96\x80',
'HangulSyllableHaelt': '\xed\x96\x81',
'EthiopicSyllableJaa': '\xe1\x8c\x83',
'TeluguLetterPha': '\xe0\xb0\xab',
'UnicodeCharacter0xdbef': '\xed\xaf\xaf',
'HangulSyllableHaet': '\xed\x96\x8d',
'HangulSyllableDud': '\xeb\x91\x97',
'HangulSyllableHaep': '\xed\x96\x8e',
'RightpointingDoubleAngleQuotationMark': '\xc2\xbb',
'CjkUnifiedIdeograph3ed6': '\xe3\xbb\x96',
'HangulSyllableHigg': '\xed\x9e\x8a',
'HangulSyllableHoec': '\xed\x9a\xa3',
'UnicodeCharacter0xeda9': '\xee\xb6\xa9',
'PlusSignInLeftHalfCircle': '\xe2\xa8\xad',
'UnicodeCharacter0xeda0': '\xee\xb6\xa0',
'UnicodeCharacter0xeda1': '\xee\xb6\xa1',
'UnicodeCharacter0xeda2': '\xee\xb6\xa2',
'UnicodeCharacter0xeda3': '\xee\xb6\xa3',
'DevanagariLetterVocalicRr': '\xe0\xa5\xa0',
'UnicodeCharacter0xeda5': '\xee\xb6\xa5',
'UnicodeCharacter0xeda6': '\xee\xb6\xa6',
'UnicodeCharacter0xeda7': '\xee\xb6\xa7',
'UnicodeCharacter0xaaf7': '\xea\xab\xb7',
'CjkUnifiedIdeograph66a9': '\xe6\x9a\xa9',
'UnicodeCharacter0xdbe3': '\xed\xaf\xa3',
'UnicodeCharacter0xdbe2': '\xed\xaf\xa2',
'UnicodeCharacter0xdbe5': '\xed\xaf\xa5',
'UnicodeCharacter0xdbe4': '\xed\xaf\xa4',
'UnicodeCharacter0xdbe7': '\xed\xaf\xa7',
'UnicodeCharacter0xdbe6': '\xed\xaf\xa6',
'UnicodeCharacter0xdbe9': '\xed\xaf\xa9',
'UnicodeCharacter0xdbe8': '\xed\xaf\xa8',
'ClosedIntersectionWithSerifs': '\xe2\xa9\x8d',
'HangulSyllableHoeb': '\xed\x9a\x9d',
'HangulSyllableWegs': '\xec\x9b\xab',
'UnicodeCharacter0xdf3e': '\xed\xbc\xbe',
'VaiSyllableHe': '\xea\x97\xa4',
'HangulSyllableTwaeng': '\xed\x87\xad',
'UnicodeCharacter0xe89a': '\xee\xa2\x9a',
'HangulLetterYui': '\xe3\x86\x8c',
'UnicodeCharacter0xeac8': '\xee\xab\x88',
'ArabicLigatureTehWithMeemWithHahInitialForm': '\xef\xb5\x96',
'UnicodeCharacter0xedaa': '\xee\xb6\xaa',
'UnicodeCharacter0xedab': '\xee\xb6\xab',
'UnicodeCharacter0xedac': '\xee\xb6\xac',
'UnicodeCharacter0xedad': '\xee\xb6\xad',
'UnicodeCharacter0xedae': '\xee\xb6\xae',
'UnicodeCharacter0xedaf': '\xee\xb6\xaf',
'UnicodeCharacter0xdc1e': '\xed\xb0\x9e',
'CjkUnifiedIdeograph9cf0': '\xe9\xb3\xb0',
'SmallLeftParenthesis': '\xef\xb9\x99',
'TwoLogicalAndOperator': '\xe2\xa8\x87',
'CjkCompatibilityIdeographf999': '\xef\xa6\x99',
'CjkUnifiedIdeograph844f': '\xe8\x91\x8f',
'CjkUnifiedIdeograph844d': '\xe8\x91\x8d',
'CjkUnifiedIdeograph844e': '\xe8\x91\x8e',
'CjkUnifiedIdeograph844b': '\xe8\x91\x8b',
'CjkUnifiedIdeograph844c': '\xe8\x91\x8c',
'CjkUnifiedIdeograph844a': '\xe8\x91\x8a',
'EthiopicSyllableYoa': '\xe1\x8b\xaf',
'LatinCapitalLetterEWithDotAbove': '\xc4\x96',
'GlagoliticCapitalLetterBuky': '\xe2\xb0\x81',
'HangulSyllableYigg': '\xec\x9d\x9a',
'GreekSmallLetterHeta': '\xcd\xb1',
'RightwardsArrowThroughX': '\xe2\xa5\x87',
'HangulSyllableDebs': '\xeb\x8e\x82',
'UnicodeCharacter0xc3a': '\xe0\xb0\xba',
'UnicodeCharacter0xc3c': '\xe0\xb0\xbc',
'UnicodeCharacter0xc3b': '\xe0\xb0\xbb',
'BraillePatternDots14678': '\xe2\xa3\xa9',
'CjkUnifiedIdeograph8eaa': '\xe8\xba\xaa',
'UnicodeCharacter0x2097': '\xe2\x82\x97',
'UnicodeCharacter0x2096': '\xe2\x82\x96',
'CjkUnifiedIdeograph8ead': '\xe8\xba\xad',
'CjkUnifiedIdeograph8eae': '\xe8\xba\xae',
'CjkUnifiedIdeograph8eaf': '\xe8\xba\xaf',
'TamilLetterSha': '\xe0\xae\xb6',
'UnicodeCharacter0x2099': '\xe2\x82\x99',
'UnicodeCharacter0x2098': '\xe2\x82\x98',
'HalfwidthKatakanaLetterTu': '\xef\xbe\x82',
'HalfwidthKatakanaLetterTi': '\xef\xbe\x81',
'TriangleheadedRightwardsArrow': '\xe2\x9e\x9d',
'HalfwidthKatakanaLetterTo': '\xef\xbe\x84',
'HangulSyllableCagg': '\xec\xb0\xaa',
'HalfwidthKatakanaLetterTa': '\xef\xbe\x80',
'HangulSyllableTwiss': '\xed\x8a\x94',
'HalfwidthKatakanaLetterTe': '\xef\xbe\x83',
'NewTaiLueLetterHighVa': '\xe1\xa6\x9b',
'HorizontalBlackHexagon': '\xe2\xac\xa3',
'UnicodeCharacter0xc34': '\xe0\xb0\xb4',
'LimbuLetterKha': '\xe1\xa4\x82',
'CjkUnifiedIdeograph8446': '\xe8\x91\x86',
'CjkUnifiedIdeograph8447': '\xe8\x91\x87',
'CjkUnifiedIdeograph8444': '\xe8\x91\x84',
'CjkUnifiedIdeograph8445': '\xe8\x91\x85',
'CjkUnifiedIdeograph8442': '\xe8\x91\x82',
'CjkUnifiedIdeograph8443': '\xe8\x91\x83',
'CjkUnifiedIdeograph8440': '\xe8\x91\x80',
'CjkUnifiedIdeograph8441': '\xe8\x91\x81',
'CopticCapitalLetterIauda': '\xe2\xb2\x92',
'UnicodeCharacter0xd819': '\xed\xa0\x99',
'CjkUnifiedIdeograph8448': '\xe8\x91\x88',
'CjkUnifiedIdeograph8449': '\xe8\x91\x89',
'CjkUnifiedIdeograph9ae4': '\xe9\xab\xa4',
'UnicodeCharacter0xe56c': '\xee\x95\xac',
'UnicodeCharacter0x209e': '\xe2\x82\x9e',
'UnicodeCharacter0x209d': '\xe2\x82\x9d',
'CjkUnifiedIdeograph8ea2': '\xe8\xba\xa2',
'UnicodeCharacter0x209f': '\xe2\x82\x9f',
'UnicodeCharacter0x209a': '\xe2\x82\x9a',
'CjkUnifiedIdeograph8ea5': '\xe8\xba\xa5',
'UnicodeCharacter0x209c': '\xe2\x82\x9c',
'UnicodeCharacter0x209b': '\xe2\x82\x9b',
'CjkUnifiedIdeograph8ea8': '\xe8\xba\xa8',
'CjkUnifiedIdeograph8ea9': '\xe8\xba\xa9',
'UnicodeCharacter0xd818': '\xed\xa0\x98',
'CjkUnifiedIdeograph6aaa': '\xe6\xaa\xaa',
'Psi': '\xce\xa8',
'WhiteRightpointingPentagon': '\xe2\xad\x94',
'CjkUnifiedIdeograph7865': '\xe7\xa1\xa5',
'ParenthesizedIdeographFinancial': '\xe3\x88\xb6',
'HangulSyllableSsyogs': '\xec\x91\x8b',
'BoxDrawingsDownHeavyAndLeftUpLight': '\xe2\x94\xa7',
'LatinCapitalLetterTurnedV': '\xc9\x85',
'UnicodeCharacter0x2b68': '\xe2\xad\xa8',
'UnicodeCharacter0x2b69': '\xe2\xad\xa9',
'UnicodeCharacter0x2b66': '\xe2\xad\xa6',
'UnicodeCharacter0x2b67': '\xe2\xad\xa7',
'UnicodeCharacter0x2b64': '\xe2\xad\xa4',
'UnicodeCharacter0x2b65': '\xe2\xad\xa5',
'UnicodeCharacter0x2b62': '\xe2\xad\xa2',
'UnicodeCharacter0x2b63': '\xe2\xad\xa3',
'UnicodeCharacter0x2b60': '\xe2\xad\xa0',
'UnicodeCharacter0x2b61': '\xe2\xad\xa1',
'LatinCapitalLetterTurnedA': '\xe2\xb1\xaf',
'HangulSyllableBbyas': '\xeb\xba\xab',
'LatinCapitalLetterTurnedM': '\xc6\x9c',
'LatinCapitalLetterTurnedL': '\xea\x9e\x80',
'HangulSyllableKwenh': '\xed\x80\x9e',
'CjkUnifiedIdeograph74b5': '\xe7\x92\xb5',
'TeluguLetterKa': '\xe0\xb0\x95',
'ArabicLetterDahal': '\xda\x8c',
'UnicodeCharacter0xfbd2': '\xef\xaf\x92',
'HangulSyllableBbyabs': '\xeb\xba\xaa',
'HangulSyllableDdyogg': '\xeb\x9a\x82',
'CanadianSyllabicsMaai': '\xe1\x92\xa4',
'HalfwidthKatakanaVoicedSoundMark': '\xef\xbe\x9e',
'CyrillicCapitalLetterKomiDe': '\xd4\x80',
'UnicodeCharacter0xe51b': '\xee\x94\x9b',
'NorthWestWhiteArrow': '\xe2\xac\x81',
'UnicodeCharacter0xde0a': '\xed\xb8\x8a',
'HangulSyllableKeult': '\xed\x81\xb9',
'CjkUnifiedIdeograph4a5a': '\xe4\xa9\x9a',
'HangulSyllableBeuc': '\xeb\xb8\xa3',
'HangulSyllableJaenj': '\xec\x9e\xb1',
'CjkUnifiedIdeograph6d4a': '\xe6\xb5\x8a',
'NegativeCircledNumberTwenty': '\xe2\x93\xb4',
'HangulSyllableJaeng': '\xec\x9f\x81',
'CjkUnifiedIdeograph7364': '\xe7\x8d\xa4',
'HangulSyllableCyeong': '\xec\xb3\xa5',
'HangulSyllableEolg': '\xec\x96\xbd',
'HangulSyllableKeulg': '\xed\x81\xb5',
'CjkUnifiedIdeograph7365': '\xe7\x8d\xa5',
'LatinSmallLetterCuatrilloWithComma': '\xea\x9c\xaf',
'UnicodeCharacter0x2b6f': '\xe2\xad\xaf',
'CjkUnifiedIdeograph7362': '\xe7\x8d\xa2',
'UnicodeCharacter0x2b6d': '\xe2\xad\xad',
'UnicodeCharacter0x2b6e': '\xe2\xad\xae',
'UnicodeCharacter0x2b6b': '\xe2\xad\xab',
'UnicodeCharacter0x2b6c': '\xe2\xad\xac',
'subchi': '\xe1\xb5\xaa',
'UnicodeCharacter0x2b6a': '\xe2\xad\xaa',
'UnicodeCharacter0xdc13': '\xed\xb0\x93',
'HangulSyllableDup': '\xeb\x91\xaa',
'CjkUnifiedIdeograph7360': '\xe7\x8d\xa0',
'VaiSyllableHo': '\xea\x96\xbd',
'ParenthesizedIdeographRest': '\xe3\x89\x81',
'HangulSyllableCaek': '\xec\xb1\x9c',
'UnicodeCharacter0xde0f': '\xed\xb8\x8f',
'CjkUnifiedIdeograph9e3c': '\xe9\xb8\xbc',
'HangulSyllableBeuh': '\xeb\xb8\xa7',
'DentistrySymbolLightDownAndHorizontal': '\xe2\x8f\x89',
'CjkUnifiedIdeograph3f35': '\xe3\xbc\xb5',
'CjkUnifiedIdeograph3f36': '\xe3\xbc\xb6',
'CjkUnifiedIdeograph3f37': '\xe3\xbc\xb7',
'CjkUnifiedIdeograph3f30': '\xe3\xbc\xb0',
'CjkUnifiedIdeograph3f31': '\xe3\xbc\xb1',
'CjkUnifiedIdeograph3f32': '\xe3\xbc\xb2',
'CjkUnifiedIdeograph3f33': '\xe3\xbc\xb3',
'UnicodeCharacter0x1a99': '\xe1\xaa\x99',
'CjkUnifiedIdeograph3f38': '\xe3\xbc\xb8',
'CyrillicCapitalLetterOmegaWithTitlo': '\xd1\xbc',
'HangulSyllablePyiss': '\xed\x94\xb4',
'CjkUnifiedIdeograph5de7': '\xe5\xb7\xa7',
'UnicodeCharacter0xe84f': '\xee\xa1\x8f',
'HangulSyllableRalh': '\xeb\x9e\x8b',
'UnicodeCharacter0xe84d': '\xee\xa1\x8d',
'UnicodeCharacter0xe5fd': '\xee\x97\xbd',
'CjkUnifiedIdeograph5de4': '\xe5\xb7\xa4',
'UnicodeCharacter0xe84a': '\xee\xa1\x8a',
'HangulSyllableRalm': '\xeb\x9e\x86',
'HangulSyllableRalb': '\xeb\x9e\x87',
'HangulSyllableRalg': '\xeb\x9e\x85',
'UnicodeCharacter0xda7c': '\xed\xa9\xbc',
'OlChikiDigitFour': '\xe1\xb1\x94',
'YiSyllableNga': '\xea\x89\xa2',
'YiSyllableNge': '\xea\x89\xac',
'GreekOxia': '\xe1\xbf\xbd',
'HangulSyllableNels': '\xeb\x84\xb0',
'HangulSyllableRals': '\xeb\x9e\x88',
'HangulSyllableRalp': '\xeb\x9e\x8a',
'CjkUnifiedIdeograph7369': '\xe7\x8d\xa9',
'SquareLog': '\xe3\x8f\x92',
'CjkUnifiedIdeograph5de8': '\xe5\xb7\xa8',
'HangulSyllableRalt': '\xeb\x9e\x89',
'YiSyllableNgo': '\xea\x89\xa9',
'CjkUnifiedIdeograph3f3d': '\xe3\xbc\xbd',
'CjkUnifiedIdeograph3f3e': '\xe3\xbc\xbe',
'CjkUnifiedIdeograph3f3f': '\xe3\xbc\xbf',
'CjkUnifiedIdeograph5de9': '\xe5\xb7\xa9',
'UnicodeCharacter0xa7ca': '\xea\x9f\x8a',
'CjkUnifiedIdeograph3f3a': '\xe3\xbc\xba',
'CjkUnifiedIdeograph3f3b': '\xe3\xbc\xbb',
'CjkUnifiedIdeograph3f3c': '\xe3\xbc\xbc',
'HangulSyllableId': '\xec\x9d\xbb',
'VaiSyllableVe': '\xea\x97\xb2',
'CjkUnifiedIdeograph6453': '\xe6\x91\x93',
'VaiSyllableVa': '\xea\x95\x99',
'HangulSyllableIc': '\xec\x9e\x8b',
'HangulSyllableNelt': '\xeb\x84\xb1',
'HangulSyllableIt': '\xec\x9e\x8d',
'VaiSyllableVu': '\xea\x96\xa3',
'HebrewAccentQarneyPara': '\xd6\x9f',
'CircledHangulKhieukhA': '\xe3\x89\xb8',
'HangulSyllableIs': '\xec\x9e\x87',
'ThaiCharacterLu': '\xe0\xb8\xa6',
'UnicodeCharacter0xe847': '\xee\xa1\x87',
'UnicodeCharacter0xe846': '\xee\xa1\x86',
'UnicodeCharacter0xe845': '\xee\xa1\x85',
'UnicodeCharacter0xe844': '\xee\xa1\x84',
'UnicodeCharacter0xe843': '\xee\xa1\x83',
'UnicodeCharacter0xe842': '\xee\xa1\x82',
'UnicodeCharacter0xe841': '\xee\xa1\x81',
'UnicodeCharacter0xe840': '\xee\xa1\x80',
'OptionKey': '\xe2\x8c\xa5',
'HangulSyllableDut': '\xeb\x91\xa9',
'UnicodeCharacter0xe849': '\xee\xa1\x89',
'UnicodeCharacter0xe848': '\xee\xa1\x88',
'CjkUnifiedIdeograph4553': '\xe4\x95\x93',
'CjkUnifiedIdeograph7631': '\xe7\x98\xb1',
'YiSyllablePie': '\xea\x80\xbd',
'CjkUnifiedIdeograph7632': '\xe7\x98\xb2',
'UnicodeCharacter0xdeea': '\xed\xbb\xaa',
'CjkUnifiedIdeograph7633': '\xe7\x98\xb3',
'YiSyllablePix': '\xea\x80\xb9',
'YiSyllablePit': '\xea\x80\xb8',
'YiSyllablePip': '\xea\x80\xbb',
'CjkUnifiedIdeograph7c99': '\xe7\xb2\x99',
'CjkUnifiedIdeograph7c98': '\xe7\xb2\x98',
'CjkUnifiedIdeograph6b48': '\xe6\xad\x88',
'CjkUnifiedIdeograph6b49': '\xe6\xad\x89',
'CjkUnifiedIdeograph6b46': '\xe6\xad\x86',
'CjkUnifiedIdeograph6b47': '\xe6\xad\x87',
'CjkUnifiedIdeograph6b44': '\xe6\xad\x84',
'CjkUnifiedIdeograph6b45': '\xe6\xad\x85',
'CjkUnifiedIdeograph3db8': '\xe3\xb6\xb8',
'CjkUnifiedIdeograph6b43': '\xe6\xad\x83',
'CjkUnifiedIdeograph6b40': '\xe6\xad\x80',
'CjkUnifiedIdeograph6b41': '\xe6\xad\x81',
'CjkUnifiedIdeograph82ec': '\xe8\x8b\xac',
'CjkUnifiedIdeograph82eb': '\xe8\x8b\xab',
'CjkUnifiedIdeograph82ea': '\xe8\x8b\xaa',
'CjkUnifiedIdeograph82ef': '\xe8\x8b\xaf',
'CjkUnifiedIdeograph82ee': '\xe8\x8b\xae',
'CjkUnifiedIdeograph7639': '\xe7\x98\xb9',
'UnicodeCharacter0xfdde': '\xef\xb7\x9e',
'SmallLeftTortoiseShellBracket': '\xef\xb9\x9d',
'BalineseLetterNya': '\xe1\xac\x9c',
'GreekCapitalLetterEtaWithPsiliAndOxia': '\xe1\xbc\xac',
'HangulSyllableDweobs': '\xeb\x91\xbe',
'CjkUnifiedIdeograph8f50': '\xe8\xbd\x90',
'TopLeftCorner': '\xe2\x8c\x9c',
'UnicodeCharacter0xe359': '\xee\x8d\x99',
'CjkUnifiedIdeograph82e3': '\xe8\x8b\xa3',
'CjkUnifiedIdeograph82e2': '\xe8\x8b\xa2',
'CjkUnifiedIdeograph82e1': '\xe8\x8b\xa1',
'CjkUnifiedIdeograph82e0': '\xe8\x8b\xa0',
'CjkUnifiedIdeograph82e7': '\xe8\x8b\xa7',
'CjkUnifiedIdeograph82e6': '\xe8\x8b\xa6',
'CjkUnifiedIdeograph82e5': '\xe8\x8b\xa5',
'CjkUnifiedIdeograph82e4': '\xe8\x8b\xa4',
'ModifierLetterSmallZWithCurl': '\xe1\xb6\xbd',
'CjkUnifiedIdeograph82e9': '\xe8\x8b\xa9',
'CjkUnifiedIdeograph82e8': '\xe8\x8b\xa8',
'UnicodeCharacter0xa6c8': '\xea\x9b\x88',
'UnicodeCharacter0xa6c9': '\xea\x9b\x89',
'CjkUnifiedIdeograph6b4f': '\xe6\xad\x8f',
'CjkUnifiedIdeograph7c9b': '\xe7\xb2\x9b',
'CjkUnifiedIdeograph6b4d': '\xe6\xad\x8d',
'CjkUnifiedIdeograph6b4e': '\xe6\xad\x8e',
'CjkUnifiedIdeograph6b4b': '\xe6\xad\x8b',
'CjkUnifiedIdeograph6b4c': '\xe6\xad\x8c',
'CjkUnifiedIdeograph7c9e': '\xe7\xb2\x9e',
'CjkUnifiedIdeograph6b4a': '\xe6\xad\x8a',
'CjkUnifiedIdeograph4b71': '\xe4\xad\xb1',
'CjkUnifiedIdeograph4b70': '\xe4\xad\xb0',
'CjkUnifiedIdeograph4b73': '\xe4\xad\xb3',
'CjkUnifiedIdeograph4b72': '\xe4\xad\xb2',
'CjkUnifiedIdeograph4b75': '\xe4\xad\xb5',
'CjkUnifiedIdeograph4b74': '\xe4\xad\xb4',
'CjkUnifiedIdeograph4b77': '\xe4\xad\xb7',
'CjkUnifiedIdeograph4b76': '\xe4\xad\xb6',
'KannadaSignJihvamuliya': '\xe0\xb3\xb1',
'CjkUnifiedIdeograph4b78': '\xe4\xad\xb8',
'CjkUnifiedIdeograph3782': '\xe3\x9e\x82',
'CjkUnifiedIdeograph736e': '\xe7\x8d\xae',
'KangxiRadicalFly': '\xe2\xbe\xb6',
'CjkUnifiedIdeograph5d7a': '\xe5\xb5\xba',
'CyrillicCapitalLetterIeWithGrave': '\xd0\x80',
'CjkUnifiedIdeograph5d7c': '\xe5\xb5\xbc',
'CjkUnifiedIdeograph5d7d': '\xe5\xb5\xbd',
'CjkUnifiedIdeograph5d7e': '\xe5\xb5\xbe',
'CjkUnifiedIdeograph5d7f': '\xe5\xb5\xbf',
'CjkUnifiedIdeograph736c': '\xe7\x8d\xac',
'UnicodeCharacter0xde07': '\xed\xb8\x87',
'SquareMmSquared': '\xe3\x8e\x9f',
'HangulSyllableJyu': '\xec\xa5\xac',
'CjkUnifiedIdeograph736a': '\xe7\x8d\xaa',
'TibetanMarkAngKhangGyas': '\xe0\xbc\xbd',
'RecyclingSymbolForGenericMaterials': '\xe2\x99\xba',
'CjkUnifiedIdeograph93b8': '\xe9\x8e\xb8',
'CjkUnifiedIdeograph93b9': '\xe9\x8e\xb9',
'HangulSyllableHyonj': '\xed\x9a\xad',
'UnicodeCharacter0xde08': '\xed\xb8\x88',
'MalayalamSignAvagraha': '\xe0\xb4\xbd',
'CjkUnifiedIdeograph93b1': '\xe9\x8e\xb1',
'CjkUnifiedIdeograph93b2': '\xe9\x8e\xb2',
'CjkUnifiedIdeograph93b3': '\xe9\x8e\xb3',
'CjkUnifiedIdeograph93b4': '\xe9\x8e\xb4',
'BoxDrawingsDoubleUpAndHorizontal': '\xe2\x95\xa9',
'CjkUnifiedIdeograph93b6': '\xe9\x8e\xb6',
'CjkUnifiedIdeograph93b7': '\xe9\x8e\xb7',
'CjkUnifiedIdeograph5d78': '\xe5\xb5\xb8',
'CjkUnifiedIdeograph5d79': '\xe5\xb5\xb9',
'HangulSyllableSwaebs': '\xec\x87\x96',
'CjkUnifiedIdeograph5d70': '\xe5\xb5\xb0',
'CjkUnifiedIdeograph5d71': '\xe5\xb5\xb1',
'CjkUnifiedIdeograph5d72': '\xe5\xb5\xb2',
'CjkUnifiedIdeograph5d73': '\xe5\xb5\xb3',
'CjkUnifiedIdeograph5d74': '\xe5\xb5\xb4',
'CjkUnifiedIdeograph5d75': '\xe5\xb5\xb5',
'CjkUnifiedIdeograph5d76': '\xe5\xb5\xb6',
'CjkUnifiedIdeograph5d77': '\xe5\xb5\xb7',
'CjkUnifiedIdeograph4b7a': '\xe4\xad\xba',
'CjkUnifiedIdeograph4b7c': '\xe4\xad\xbc',
'CjkUnifiedIdeograph4b7b': '\xe4\xad\xbb',
'CjkUnifiedIdeograph4b7e': '\xe4\xad\xbe',
'CjkUnifiedIdeograph4b7d': '\xe4\xad\xbd',
'CjkUnifiedIdeograph4b7f': '\xe4\xad\xbf',
'CjkUnifiedIdeograph786c': '\xe7\xa1\xac',
'HangulSyllableByaegg': '\xeb\xb1\xaa',
'MalayalamLetterKa': '\xe0\xb4\x95',
'LatinCapitalLetterZWithStroke': '\xc6\xb5',
'HangulSyllableGuss': '\xea\xb6\x80',
'CjkUnifiedIdeograph786e': '\xe7\xa1\xae',
'CjkUnifiedIdeograph93bb': '\xe9\x8e\xbb',
'CjkUnifiedIdeograph93bc': '\xe9\x8e\xbc',
'CjkUnifiedIdeograph93bd': '\xe9\x8e\xbd',
'DoublestruckNarySummation': '\xe2\x85\x80',
'CjkUnifiedIdeograph93bf': '\xe9\x8e\xbf',
'HangulSyllableCying': '\xec\xb9\x91',
'HangulSyllableIl': '\xec\x9d\xbc',
'CjkUnifiedIdeograph439f': '\xe4\x8e\x9f',
'^9': '\xe2\x81\xb9',
'CjkUnifiedIdeograph439d': '\xe4\x8e\x9d',
'CjkUnifiedIdeograph439e': '\xe4\x8e\x9e',
'CjkUnifiedIdeograph439b': '\xe4\x8e\x9b',
'CjkUnifiedIdeograph439c': '\xe4\x8e\x9c',
'CjkUnifiedIdeograph439a': '\xe4\x8e\x9a',
'CjkUnifiedIdeograph45ea': '\xe4\x97\xaa',
'CjkUnifiedIdeograph45eb': '\xe4\x97\xab',
'CjkUnifiedIdeograph45ec': '\xe4\x97\xac',
'CjkUnifiedIdeograph45ed': '\xe4\x97\xad',
'CjkUnifiedIdeograph45ee': '\xe4\x97\xae',
'CjkUnifiedIdeograph45ef': '\xe4\x97\xaf',
'HangulSyllableBbyac': '\xeb\xba\xaf',
'CjkUnifiedIdeograph7099': '\xe7\x82\x99',
'HangulSyllableIm': '\xec\x9e\x84',
'HangulSyllableCweoss': '\xec\xb7\x84',
'UnicodeCharacter0xe02a': '\xee\x80\xaa',
'UnicodeCharacter0xe02c': '\xee\x80\xac',
'UnicodeCharacter0xe02b': '\xee\x80\xab',
'UnicodeCharacter0xe02e': '\xee\x80\xae',
'UnicodeCharacter0xe02d': '\xee\x80\xad',
'CombiningCyrillicLetterYat': '\xe2\xb7\xba',
'UnicodeCharacter0xeacc': '\xee\xab\x8c',
'UnicodeCharacter0xeacb': '\xee\xab\x8b',
'UnicodeCharacter0xeaca': '\xee\xab\x8a',
'UnicodeCharacter0xeacf': '\xee\xab\x8f',
'UnicodeCharacter0xeace': '\xee\xab\x8e',
'UnicodeCharacter0xeacd': '\xee\xab\x8d',
'CjkUnifiedIdeograph9a5f': '\xe9\xa9\x9f',
'CjkUnifiedIdeograph9a5e': '\xe9\xa9\x9e',
'CjkUnifiedIdeograph9a5d': '\xe9\xa9\x9d',
'CjkUnifiedIdeograph9a5c': '\xe9\xa9\x9c',
'CjkUnifiedIdeograph9a5b': '\xe9\xa9\x9b',
'CjkUnifiedIdeograph9a5a': '\xe9\xa9\x9a',
'HangulSyllableGgyoh': '\xea\xbe\xb7',
'GreekSmallLetterOmegaWithPsiliAndPerispomeni': '\xe1\xbd\xa6',
'HangulSyllableGgyoj': '\xea\xbe\xb2',
'HangulSyllableGgyok': '\xea\xbe\xb4',
'HangulSyllableGgyol': '\xea\xbe\xa4',
'HangulSyllableGgyom': '\xea\xbe\xac',
'HangulSyllableGgyon': '\xea\xbe\xa0',
'HangulSyllableGgyob': '\xea\xbe\xad',
'HangulSyllableGgyoc': '\xea\xbe\xb3',
'HangulSyllableGgyod': '\xea\xbe\xa3',
'HangulSyllableGgyog': '\xea\xbe\x9d',
'CjkUnifiedIdeograph45e8': '\xe4\x97\xa8',
'CjkUnifiedIdeograph45e9': '\xe4\x97\xa9',
'CjkUnifiedIdeograph4394': '\xe4\x8e\x94',
'CjkUnifiedIdeograph4395': '\xe4\x8e\x95',
'CjkUnifiedIdeograph4392': '\xe4\x8e\x92',
'CjkUnifiedIdeograph4393': '\xe4\x8e\x93',
'CjkUnifiedIdeograph4390': '\xe4\x8e\x90',
'CjkUnifiedIdeograph4391': '\xe4\x8e\x91',
'CjkUnifiedIdeograph3db2': '\xe3\xb6\xb2',
'CjkUnifiedIdeograph45e1': '\xe4\x97\xa1',
'CjkUnifiedIdeograph45e2': '\xe4\x97\xa2',
'CjkUnifiedIdeograph45e3': '\xe4\x97\xa3',
'CjkUnifiedIdeograph45e4': '\xe4\x97\xa4',
'CjkUnifiedIdeograph45e5': '\xe4\x97\xa5',
'CjkUnifiedIdeograph45e6': '\xe4\x97\xa6',
'CjkUnifiedIdeograph4399': '\xe4\x8e\x99',
'TaiLeLetterYa': '\xe1\xa5\x95',
'HangulSyllableBoed': '\xeb\xb5\x8f',
'UnicodeCharacter0xeac1': '\xee\xab\x81',
'HangulSyllableCwenh': '\xec\xb7\x92',
'UnicodeCharacter0xeac7': '\xee\xab\x87',
'UnicodeCharacter0xeac6': '\xee\xab\x86',
'CjkUnifiedIdeograph9a59': '\xe9\xa9\x99',
'CircledNumberTwentyFour': '\xe3\x89\x94',
'CjkUnifiedIdeograph9a57': '\xe9\xa9\x97',
'CjkUnifiedIdeograph9a56': '\xe9\xa9\x96',
'CjkUnifiedIdeograph9a55': '\xe9\xa9\x95',
'CjkUnifiedIdeograph9a54': '\xe9\xa9\x94',
'CjkUnifiedIdeograph9a53': '\xe9\xa9\x93',
'CjkUnifiedIdeograph9a52': '\xe9\xa9\x92',
'CjkUnifiedIdeograph9a51': '\xe9\xa9\x91',
'CjkUnifiedIdeograph9a50': '\xe9\xa9\x90',
'UnicodeCharacter0xe029': '\xee\x80\xa9',
'UnicodeCharacter0xe028': '\xee\x80\xa8',
'CanadianSyllabicsWestcreeYwoo': '\xe1\x94\xb8',
'BuhidLetterGa': '\xe1\x9d\x84',
'UnicodeCharacter0xe021': '\xee\x80\xa1',
'UnicodeCharacter0xe020': '\xee\x80\xa0',
'UnicodeCharacter0xe023': '\xee\x80\xa3',
'UnicodeCharacter0xe022': '\xee\x80\xa2',
'UnicodeCharacter0xe025': '\xee\x80\xa5',
'UnicodeCharacter0xe024': '\xee\x80\xa4',
'UnicodeCharacter0xe027': '\xee\x80\xa7',
'UnicodeCharacter0xe026': '\xee\x80\xa6',
'YiSyllableHnap': '\xea\x85\xb3',
'HangulSyllableRoebs': '\xeb\xa3\x82',
'GeorgianLetterChar': '\xe1\x83\xad',
'CjkUnifiedIdeograph91a0': '\xe9\x86\xa0',
'CjkUnifiedIdeograph544e': '\xe5\x91\x8e',
'CjkUnifiedIdeograph544d': '\xe5\x91\x8d',
'CjkUnifiedIdeograph544f': '\xe5\x91\x8f',
'CjkUnifiedIdeograph544a': '\xe5\x91\x8a',
'CjkUnifiedIdeograph544c': '\xe5\x91\x8c',
'CjkUnifiedIdeograph544b': '\xe5\x91\x8b',
'HangulSyllableJyonj': '\xec\xa3\xa5',
'CjkUnifiedIdeograph81b4': '\xe8\x86\xb4',
'HangulSyllableJyonh': '\xec\xa3\xa6',
'CanadianSyllabicsNunavutH': '\xe1\x95\xbc',
'HangulSyllableBbyah': '\xeb\xba\xb3',
'CjkCompatibilityIdeographf99d': '\xef\xa6\x9d',
'HangulSyllableSwak': '\xec\x87\x80',
'HangulSyllableJyong': '\xec\xa3\xb5',
'HangulSyllablePyeot': '\xed\x8f\x8d',
'CjkUnifiedIdeograph5445': '\xe5\x91\x85',
'CjkUnifiedIdeograph5444': '\xe5\x91\x84',
'CjkUnifiedIdeograph5447': '\xe5\x91\x87',
'CjkUnifiedIdeograph5446': '\xe5\x91\x86',
'CjkUnifiedIdeograph5441': '\xe5\x91\x81',
'CjkUnifiedIdeograph5440': '\xe5\x91\x80',
'CjkUnifiedIdeograph5443': '\xe5\x91\x83',
'CjkUnifiedIdeograph5442': '\xe5\x91\x82',
'CjkUnifiedIdeograph970d': '\xe9\x9c\x8d',
'CjkUnifiedIdeograph5449': '\xe5\x91\x89',
'CjkUnifiedIdeograph5448': '\xe5\x91\x88',
'HangulSyllableIk': '\xec\x9e\x8c',
'CjkUnifiedIdeograph7a0a': '\xe7\xa8\x8a',
'CjkUnifiedIdeograph7a0f': '\xe7\xa8\x8f',
'HangulSyllablePaels': '\xed\x8c\xb4',
'GreekSmallLetterAlphaWithOxia': '\xe1\xbd\xb1',
'CjkUnifiedIdeograph970e': '\xe9\x9c\x8e',
'CjkUnifiedIdeograph455d': '\xe4\x95\x9d',
'HeavyVerticalBar': '\xe2\x9d\x9a',
'CjkUnifiedIdeograph7a0e': '\xe7\xa8\x8e',
'ArabicLetterYehBarreeIsolatedForm': '\xef\xae\xae',
'LatinCapitalLetterUWithTildeAndAcute': '\xe1\xb9\xb8',
'SinhalaLetterMahaapraanaTtayanna': '\xe0\xb6\xa8',
'CjkUnifiedIdeograph46aa': '\xe4\x9a\xaa',
'SquareWithLowerLeftDiagonalHalfBlack': '\xe2\xac\x95',
'HangulSyllableSsyad': '\xec\x8c\xb7',
'HangulSyllableBbyak': '\xeb\xba\xb0',
'CjkUnifiedIdeograph9148': '\xe9\x85\x88',
'CjkUnifiedIdeograph9149': '\xe9\x85\x89',
'KatakanahiraganaDoubleHyphen': '\xe3\x82\xa0',
'CjkUnifiedIdeograph9145': '\xe9\x85\x85',
'CjkUnifiedIdeograph9146': '\xe9\x85\x86',
'CjkUnifiedIdeograph9147': '\xe9\x85\x87',
'CjkUnifiedIdeograph9140': '\xe9\x85\x80',
'CjkUnifiedIdeograph9141': '\xe9\x85\x81',
'CjkUnifiedIdeograph9142': '\xe9\x85\x82',
'CjkUnifiedIdeograph9143': '\xe9\x85\x83',
'CjkUnifiedIdeograph69d5': '\xe6\xa7\x95',
'CjkUnifiedIdeograph69d4': '\xe6\xa7\x94',
'CjkUnifiedIdeograph69d7': '\xe6\xa7\x97',
'CjkUnifiedIdeograph69d6': '\xe6\xa7\x96',
'CjkUnifiedIdeograph69d1': '\xe6\xa7\x91',
'CjkUnifiedIdeograph69d0': '\xe6\xa7\x90',
'CjkUnifiedIdeograph69d3': '\xe6\xa7\x93',
'CjkUnifiedIdeograph69d2': '\xe6\xa7\x92',
'UnicodeCharacter0xf6ff': '\xef\x9b\xbf',
'CjkUnifiedIdeograph69d9': '\xe6\xa7\x99',
'CjkUnifiedIdeograph69d8': '\xe6\xa7\x98',
'TeluguLetterPa': '\xe0\xb0\xaa',
'NewTaiLueLetterLowDa': '\xe1\xa6\xa4',
'HangulSyllableKess': '\xec\xbc\x94',
'CjkUnifiedIdeograph883f': '\xe8\xa0\xbf',
'HangulSyllableBbeud': '\xec\x81\x9f',
'CjkUnifiedIdeograph883d': '\xe8\xa0\xbd',
'HangulSyllableBbeub': '\xec\x81\xa9',
'HangulSyllableBbeuc': '\xec\x81\xaf',
'CjkUnifiedIdeograph914d': '\xe9\x85\x8d',
'CjkUnifiedIdeograph883c': '\xe8\xa0\xbc',
'CjkUnifiedIdeograph914f': '\xe9\x85\x8f',
'HangulSyllableGwaelt': '\xea\xb4\xa5',
'HangulSyllableBbeuh': '\xec\x81\xb3',
'CjkUnifiedIdeograph914a': '\xe9\x85\x8a',
'CjkUnifiedIdeograph914b': '\xe9\x85\x8b',
'CjkUnifiedIdeograph914c': '\xe9\x85\x8c',
'HangulSyllableBbeut': '\xec\x81\xb1',
'CjkUnifiedIdeograph6acd': '\xe6\xab\x8d',
'HangulSyllableBbeup': '\xec\x81\xb2',
'LatinSmallLetterUWithMacron': '\xc5\xab',
'HangulSyllableBbeus': '\xec\x81\xab',
'SaurashtraLetterRa': '\xea\xa2\xac',
'HangulSyllableMwe': '\xeb\xad\xac',
'HangulSyllableMwa': '\xeb\xab\x84',
'HangulSyllableGwass': '\xea\xb4\x90',
'HangulSyllableMwi': '\xeb\xae\x88',
'CjkUnifiedIdeograph6aca': '\xe6\xab\x8a',
'CjkUnifiedIdeograph69de': '\xe6\xa7\x9e',
'CombiningDoubleBreve': '\xcd\x9d',
'CjkUnifiedIdeograph69df': '\xe6\xa7\x9f',
'CjkUnifiedIdeograph69da': '\xe6\xa7\x9a',
'CjkUnifiedIdeograph6acb': '\xe6\xab\x8b',
'CjkUnifiedIdeograph69dc': '\xe6\xa7\x9c',
'CjkUnifiedIdeograph69db': '\xe6\xa7\x9b',
'HangulSyllableJjwags': '\xec\xab\x9b',
'CjkUnifiedIdeograph6acc': '\xe6\xab\x8c',
'UnicodeCharacter0xf093': '\xef\x82\x93',
'YiSyllableYip': '\xea\x91\xb4',
'YiSyllableYit': '\xea\x91\xb1',
'HangulSyllableMwig': '\xeb\xae\x89',
'YiSyllableYix': '\xea\x91\xb2',
'LimbuLetterDha': '\xe1\xa4\x8e',
'CjkUnifiedIdeograph384c': '\xe3\xa1\x8c',
'CjkUnifiedIdeograph384b': '\xe3\xa1\x8b',
'CjkUnifiedIdeograph384a': '\xe3\xa1\x8a',
'UnicodeCharacter0xec1d': '\xee\xb0\x9d',
'OghamLetterNgeadal': '\xe1\x9a\x8d',
'CjkUnifiedIdeograph384f': '\xe3\xa1\x8f',
'CjkUnifiedIdeograph384e': '\xe3\xa1\x8e',
'CjkUnifiedIdeograph384d': '\xe3\xa1\x8d',
'ModifierLetterMidDottedLeftstemToneBar': '\xea\x9c\x8f',
'UnicodeCharacter0xe351': '\xee\x8d\x91',
'LatinSmallLetterOWithHornAndGrave': '\xe1\xbb\x9d',
'HangulSyllableSwet': '\xec\x89\xa9',
'CjkUnifiedIdeograph4479': '\xe4\x91\xb9',
'CjkUnifiedIdeograph4478': '\xe4\x91\xb8',
'HalfwidthHangulLetterThieuth': '\xef\xbe\xbc',
'CjkUnifiedIdeograph4473': '\xe4\x91\xb3',
'CjkUnifiedIdeograph4472': '\xe4\x91\xb2',
'CjkUnifiedIdeograph4471': '\xe4\x91\xb1',
'CjkUnifiedIdeograph4470': '\xe4\x91\xb0',
'CjkUnifiedIdeograph4477': '\xe4\x91\xb7',
'CjkUnifiedIdeograph4476': '\xe4\x91\xb6',
'CjkUnifiedIdeograph4475': '\xe4\x91\xb5',
'CjkUnifiedIdeograph4474': '\xe4\x91\xb4',
'CjkUnifiedIdeograph8c4b': '\xe8\xb1\x8b',
'UnicodeCharacter0xe355': '\xee\x8d\x95',
'ArabicLigatureLamWithKhahInitialForm': '\xef\xb3\x8b',
'UnicodeCharacter0xe354': '\xee\x8d\x94',
'UnicodeCharacter0xf6fc': '\xef\x9b\xbc',
'CjkUnifiedIdeograph3843': '\xe3\xa1\x83',
'CjkUnifiedIdeograph3842': '\xe3\xa1\x82',
'CjkUnifiedIdeograph3841': '\xe3\xa1\x81',
'CanadianSyllabicsAn': '\xe1\x90\xae',
'CjkUnifiedIdeograph3847': '\xe3\xa1\x87',
'CjkUnifiedIdeograph3846': '\xe3\xa1\x86',
'CjkUnifiedIdeograph3845': '\xe3\xa1\x85',
'CjkUnifiedIdeograph3844': '\xe3\xa1\x84',
'CjkUnifiedIdeograph3849': '\xe3\xa1\x89',
'CjkUnifiedIdeograph3848': '\xe3\xa1\x88',
'HangulSyllableKyoj': '\xec\xbf\x9a',
'UnicodeCharacter0xec19': '\xee\xb0\x99',
'UnicodeCharacter0xec18': '\xee\xb0\x98',
'fa': '\xe2\x88\x80',
'HangulSyllableKyok': '\xec\xbf\x9c',
'VaiSyllableKoo': '\xea\x96\x8f',
'HangulSyllableKyol': '\xec\xbf\x8c',
'HangulSyllableCyeod': '\xec\xb3\x97',
'HangulSyllableKyom': '\xec\xbf\x94',
'UnicodeCharacter0xe647': '\xee\x99\x87',
'HangulSyllableSsyelh': '\xec\x8f\x8b',
'HangulSyllableKyon': '\xec\xbf\x88',
'HangulSyllableSsyelm': '\xec\x8f\x86',
'HangulSyllableSsyelb': '\xec\x8f\x87',
'HangulSyllableSsyelg': '\xec\x8f\x85',
'HangulSyllableDdugg': '\xeb\x9a\x9e',
'HangulSyllableJoek': '\xec\xa3\x9c',
'CjkUnifiedIdeograph447c': '\xe4\x91\xbc',
'CjkUnifiedIdeograph447b': '\xe4\x91\xbb',
'CjkUnifiedIdeograph447a': '\xe4\x91\xba',
'HangulSyllableSsyelp': '\xec\x8f\x8a',
'HangulSyllableCags': '\xec\xb0\xab',
'CjkUnifiedIdeograph447f': '\xe4\x91\xbf',
'CjkUnifiedIdeograph447e': '\xe4\x91\xbe',
'CjkUnifiedIdeograph447d': '\xe4\x91\xbd',
'HangulSyllableJoeh': '\xec\xa3\x9f',
'LatinSmallLetterUBar': '\xca\x89',
'UnicodeCharacter0xfb0b': '\xef\xac\x8b',
'HangulSyllableJoen': '\xec\xa3\x88',
'UnicodeCharacter0xfb0c': '\xef\xac\x8c',
'CjkUnifiedIdeograph657b': '\xe6\x95\xbb',
'UnicodeCharacter0xfb0d': '\xef\xac\x8d',
'CjkUnifiedIdeograph5208': '\xe5\x88\x88',
'RightwardsArrow': '\xe2\x86\x92',
'UnicodeCharacter0xfb0e': '\xef\xac\x8e',
'YiSyllableZzur': '\xea\x8b\xa6',
'YiSyllableZzup': '\xea\x8b\xa4',
'UnicodeCharacter0xfb0f': '\xef\xac\x8f',
'^5': '\xe2\x81\xb5',
'YiSyllableZzux': '\xea\x8b\xa2',
'CircledHangulSiosA': '\xe3\x89\xb4',
'HangulSyllableKyuss': '\xed\x81\xa4',
'LatinSmallLetterKWithStrokeAndDiagonalStroke': '\xea\x9d\x85',
'mu': '\xce\xbc',
'HangulSyllablePoenj': '\xed\x91\x85',
'HangulSyllableJoep': '\xec\xa3\x9e',
'UnicodeCharacter0xd889': '\xed\xa2\x89',
'ArabicLetterAe': '\xdb\x95',
'HangulSyllablePoeng': '\xed\x91\x95',
'V': '\xce\x9d',
'CjkUnifiedIdeograph6e34': '\xe6\xb8\xb4',
'CjkUnifiedIdeograph91a5': '\xe9\x86\xa5',
'ModifierLetterGeorgianNar': '\xe1\x83\xbc',
'CjkUnifiedIdeograph5909': '\xe5\xa4\x89',
'UnicodeCharacter0xf4ea': '\xef\x93\xaa',
'UnicodeCharacter0xf71d': '\xef\x9c\x9d',
'HangulSyllableGgulb': '\xea\xbf\x83',
'^7': '\xe2\x81\xb7',
'HangulSyllableGgulg': '\xea\xbf\x81',
'HangulSyllableGgulh': '\xea\xbf\x87',
'SaurashtraLetterPha': '\xea\xa2\xa7',
'HangulSyllableGgulp': '\xea\xbf\x86',
'HangulSyllableGguls': '\xea\xbf\x84',
'HangulSyllableGgult': '\xea\xbf\x85',
'CjkUnifiedIdeograph6ac5': '\xe6\xab\x85',
'HangulSyllableTwagg': '\xed\x86\xbe',
'EthiopicSyllableQhe': '\xe1\x89\x95',
'HangulSyllableGgong': '\xea\xbd\x81',
'CircledIdeographFour': '\xe3\x8a\x83',
'EthiopicSyllableQha': '\xe1\x89\x90',
'UnicodeCharacter0xdc9': '\xe0\xb7\x89',
'EthiopicSyllableQho': '\xe1\x89\x96',
'UnicodeCharacter0xdc7': '\xe0\xb7\x87',
'EthiopicSyllableQhi': '\xe1\x89\x92',
'ArabicLigatureThehWithJeemIsolatedForm': '\xef\xb0\x91',
'EthiopicSyllableQhu': '\xe1\x89\x91',
'NkoCombiningLongHighTone': '\xdf\xaf',
'ArabicSmallHighUprightRectangularZero': '\xdb\xa0',
'CjkUnifiedIdeograph7a02': '\xe7\xa8\x82',
'CjkUnifiedIdeograph6d74': '\xe6\xb5\xb4',
'BopomofoLetterJi': '\xe3\x86\xa2',
'CjkUnifiedIdeograph7a00': '\xe7\xa8\x80',
'CjkUnifiedIdeograph6729': '\xe6\x9c\xa9',
'CjkUnifiedIdeograph6728': '\xe6\x9c\xa8',
'CjkUnifiedIdeograph7a01': '\xe7\xa8\x81',
'ArabicLigatureYehWithHamzaAboveWithMeemMedialForm': '\xef\xb3\x9f',
'UnicodeCharacter0xe35b': '\xee\x8d\x9b',
'CjkUnifiedIdeograph6721': '\xe6\x9c\xa1',
'CjkUnifiedIdeograph6720': '\xe6\x9c\xa0',
'CjkUnifiedIdeograph6723': '\xe6\x9c\xa3',
'CjkUnifiedIdeograph6722': '\xe6\x9c\xa2',
'CjkUnifiedIdeograph6725': '\xe6\x9c\xa5',
'CjkUnifiedIdeograph6724': '\xe6\x9c\xa4',
'MongolianLetterTodoTsa': '\xe1\xa1\x94',
'CjkUnifiedIdeograph6726': '\xe6\x9c\xa6',
'UnicodeCharacter0xe0de': '\xee\x83\x9e',
'HangulSyllableJoegs': '\xec\xa3\x87',
'CjkUnifiedIdeograph7a04': '\xe7\xa8\x84',
'CjkUnifiedIdeograph8beb': '\xe8\xaf\xab',
'CjkUnifiedIdeograph7a05': '\xe7\xa8\x85',
'UnicodeCharacter0xe35f': '\xee\x8d\x9f',
'CjkUnifiedIdeograph52ef': '\xe5\x8b\xaf',
'HangulSyllableJoegg': '\xec\xa3\x86',
'UnicodeCharacter0xdcd': '\xe0\xb7\x8d',
'UnicodeCharacter0xdce': '\xe0\xb7\x8e',
'UnicodeCharacter0xdcb': '\xe0\xb7\x8b',
'UnicodeCharacter0xdcc': '\xe0\xb7\x8c',
'YiRadicalZot': '\xea\x92\xab',
'CongruentWithDotAbove': '\xe2\xa9\xad',
'CjkUnifiedIdeograph672a': '\xe6\x9c\xaa',
'CjkUnifiedIdeograph672c': '\xe6\x9c\xac',
'CjkUnifiedIdeograph672b': '\xe6\x9c\xab',
'CjkUnifiedIdeograph672e': '\xe6\x9c\xae',
'CjkUnifiedIdeograph672d': '\xe6\x9c\xad',
'GreekSmallLetterIotaWithPsiliAndOxia': '\xe1\xbc\xb4',
'SouthWestArrowAndNorthWestArrow': '\xe2\xa4\xaa',
'MalayalamLetterVa': '\xe0\xb4\xb5',
'CjkUnifiedIdeograph5809': '\xe5\xa0\x89',
'YiSyllableNbix': '\xea\x81\xb7',
'UnicodeCharacter0x3098': '\xe3\x82\x98',
'UnicodeCharacter0x3097': '\xe3\x82\x97',
'YiSyllableNbip': '\xea\x81\xb9',
'YiSyllableNbit': '\xea\x81\xb6',
'CjkUnifiedIdeograph8837': '\xe8\xa0\xb7',
'ModifierLetterExtrahighDottedToneBar': '\xea\x9c\x88',
'ArabicLigatureTehWithJeemWithAlefMaksuraFinalForm': '\xef\xb6\xa0',
'CjkUnifiedIdeograph8836': '\xe8\xa0\xb6',
'UnicodeCharacter0xec7': '\xe0\xbb\x87',
'UnicodeCharacter0xec5': '\xe0\xbb\x85',
'CjkUnifiedIdeograph8835': '\xe8\xa0\xb5',
'YiSyllableNbie': '\xea\x81\xbb',
'SquareKuruzeiro': '\xe3\x8c\x9a',
'CjkUnifiedIdeograph72a7': '\xe7\x8a\xa7',
'CjkUnifiedIdeograph72a4': '\xe7\x8a\xa4',
'SouthEastBlackArrow': '\xe2\xac\x8a',
'CjkUnifiedIdeograph72a2': '\xe7\x8a\xa2',
'CjkUnifiedIdeograph72a3': '\xe7\x8a\xa3',
'CjkUnifiedIdeograph72a0': '\xe7\x8a\xa0',
'CjkUnifiedIdeograph72a1': '\xe7\x8a\xa1',
'CjkUnifiedIdeograph8833': '\xe8\xa0\xb3',
'ArabicLetterKehehWithTwoDotsAbove': '\xd8\xbb',
'CjkUnifiedIdeograph72a8': '\xe7\x8a\xa8',
'LatinSmallLetterRamsHorn': '\xc9\xa4',
'HangulSyllableKelh': '\xec\xbc\x8f',
'CjkUnifiedIdeograph8831': '\xe8\xa0\xb1',
'Delta': '\xce\x94',
'CjkUnifiedIdeograph8830': '\xe8\xa0\xb0',
'HangulSyllableDyugs': '\xeb\x93\x83',
'HangulSyllableBbeolt': '\xeb\xbb\x9d',
'VaiSyllablePu': '\xea\x96\x9b',
'HangulSyllableSyaegs': '\xec\x84\x83',
'UnicodeCharacter0xecf': '\xe0\xbb\x8f',
'UnicodeCharacter0xece': '\xe0\xbb\x8e',
'HangulSyllableSyaegg': '\xec\x84\x82',
'CircledHangulRieulA': '\xe3\x89\xb1',
'SaurashtraVowelSignVocalicLl': '\xea\xa2\xbd',
'LatinCapitalLetterZWithAcute': '\xc5\xb9',
'HangulSyllableKeb': '\xec\xbc\x91',
'CjkUnifiedIdeograph7139': '\xe7\x84\xb9',
'PartialDifferential': '\xe2\x88\x82',
'UnicodeCharacter0xf718': '\xef\x9c\x98',
'UnicodeCharacter0xe352': '\xee\x8d\x92',
'UnicodeCharacter0xf569': '\xef\x95\xa9',
'CjkUnifiedIdeograph72af': '\xe7\x8a\xaf',
'HangulSyllableHyeoj': '\xed\x98\x96',
'CjkUnifiedIdeograph72ad': '\xe7\x8a\xad',
'CjkUnifiedIdeograph72ae': '\xe7\x8a\xae',
'CjkUnifiedIdeograph72ab': '\xe7\x8a\xab',
'CjkUnifiedIdeograph72ac': '\xe7\x8a\xac',
'UnicodeCharacter0xf659': '\xef\x99\x99',
'CjkUnifiedIdeograph72aa': '\xe7\x8a\xaa',
'UnicodeCharacter0xa7b0': '\xea\x9e\xb0',
'HangulSyllableSilt': '\xec\x8b\xa9',
'HangulSyllableSilp': '\xec\x8b\xaa',
'HangulSyllableSils': '\xec\x8b\xa8',
'UnicodeCharacter0xf4e4': '\xef\x93\xa4',
'HangulSyllableSilm': '\xec\x8b\xa6',
'UnicodeCharacter0xf658': '\xef\x99\x98',
'HangulSyllableSilh': '\xec\x8b\xab',
'CopticSmallLetterMi': '\xe2\xb2\x99',
'HangulSyllableSilg': '\xec\x8b\xa5',
'UnicodeCharacter0xf711': '\xef\x9c\x91',
'HangulSyllableSilb': '\xec\x8b\xa7',
'HangulSyllablePweonj': '\xed\x92\x99',
'HangulSyllablePweonh': '\xed\x92\x9a',
'HangulSyllablePweong': '\xed\x92\xa9',
'CjkUnifiedIdeograph7f23': '\xe7\xbc\xa3',
'CjkUnifiedIdeograph7f22': '\xe7\xbc\xa2',
'CjkUnifiedIdeograph7f21': '\xe7\xbc\xa1',
'CjkUnifiedIdeograph7f20': '\xe7\xbc\xa0',
'CjkUnifiedIdeograph7f27': '\xe7\xbc\xa7',
'MyanmarSignDotBelow': '\xe1\x80\xb7',
'CjkUnifiedIdeograph7f25': '\xe7\xbc\xa5',
'CjkUnifiedIdeograph7f24': '\xe7\xbc\xa4',
'CjkUnifiedIdeograph6578': '\xe6\x95\xb8',
'CjkUnifiedIdeograph7f29': '\xe7\xbc\xa9',
'CjkUnifiedIdeograph7f28': '\xe7\xbc\xa8',
'varkappa': '\xcf\xb0',
'CjkUnifiedIdeograph6579': '\xe6\x95\xb9',
'HangulSyllableBwan': '\xeb\xb4\x94',
'WhiteDiamondContainingBlackSmallDiamond': '\xe2\x97\x88',
'HangulSyllableBwam': '\xeb\xb4\xa0',
'BraillePatternDots123578': '\xe2\xa3\x97',
'UnicodeCharacter0xf656': '\xef\x99\x96',
'LatinSmallLetterEgyptologicalAin': '\xea\x9c\xa5',
'HangulSyllableRing': '\xeb\xa7\x81',
'rho': '\xcf\x81',
'HangulSyllableDdwags': '\xeb\x98\xaf',
'UnicodeCharacter0xd909': '\xed\xa4\x89',
'HangulSyllableRinh': '\xeb\xa6\xb2',
'HangulSyllableRinj': '\xeb\xa6\xb1',
'HangulSyllableMyelt': '\xeb\xaa\x99',
'CjkUnifiedIdeograph7f2c': '\xe7\xbc\xac',
'CjkUnifiedIdeograph7f2b': '\xe7\xbc\xab',
'CjkUnifiedIdeograph7f2a': '\xe7\xbc\xaa',
'CjkUnifiedIdeograph6577': '\xe6\x95\xb7',
'UnicodeCharacter0xe7bf': '\xee\x9e\xbf',
'CjkUnifiedIdeograph7f2f': '\xe7\xbc\xaf',
'CjkUnifiedIdeograph7f2e': '\xe7\xbc\xae',
'CjkUnifiedIdeograph7f2d': '\xe7\xbc\xad',
'HangulSyllableDdyeop': '\xeb\x97\xb2',
'CjkUnifiedIdeograph5c02': '\xe5\xb0\x82',
'HangulSyllableGgyaenh': '\xea\xba\xa6',
'HangulSyllableDdyeot': '\xeb\x97\xb1',
'UnicodeCharacter0xe0dc': '\xee\x83\x9c',
'UnicodeCharacter0xf655': '\xef\x99\x95',
'HangulSyllableDdyeoh': '\xeb\x97\xb3',
'HangulSyllableBwag': '\xeb\xb4\x91',
'HangulSyllableDdyeoj': '\xeb\x97\xae',
'CjkUnifiedIdeograph817a': '\xe8\x85\xba',
'HangulSyllableDdyeol': '\xeb\x97\xa0',
'HangulSyllableDdyeom': '\xeb\x97\xa8',
'CanadianSyllabicsMwii': '\xe1\x92\xb0',
'UnicodeCharacter0xd904': '\xed\xa4\x84',
'HangulSyllableDdyeob': '\xeb\x97\xa9',
'ChamLetterE': '\xea\xa8\x83',
'HangulSyllableDdyeod': '\xeb\x97\x9f',
'UnicodeCharacter0xd907': '\xed\xa4\x87',
'HangulSyllableCoes': '\xec\xb5\xaf',
'HangulSyllableDdyeog': '\xeb\x97\x99',
'UnicodeCharacter0xd906': '\xed\xa4\x86',
'HangulSyllableSsyubs': '\xec\x93\xa6',
'HangulSyllableBbyebs': '\xeb\xbc\xb6',
'CjkUnifiedIdeograph817e': '\xe8\x85\xbe',
'UnicodeCharacter0xf654': '\xef\x99\x94',
'HangulSyllableJyogs': '\xec\xa3\xa3',
'BengaliSignAnusvara': '\xe0\xa6\x82',
'CanadianSyllabicsWoodscreeThe': '\xe1\x96\x9b',
'CjkUnifiedIdeograph5f77': '\xe5\xbd\xb7',
'CjkUnifiedIdeograph5f74': '\xe5\xbd\xb4',
'CjkUnifiedIdeograph5f75': '\xe5\xbd\xb5',
'CanadianSyllabicsWoodscreeTha': '\xe1\x96\x9e',
'CjkUnifiedIdeograph5f73': '\xe5\xbd\xb3',
'CjkUnifiedIdeograph5f70': '\xe5\xbd\xb0',
'CjkUnifiedIdeograph5f71': '\xe5\xbd\xb1',
'CanadianSyllabicsWoodscreeTho': '\xe1\x96\x9d',
'LatinCapitalLetterDWithStroke': '\xc4\x90',
'CanadianSyllabicsWoodscreeThi': '\xe1\x96\x9c',
'CjkUnifiedIdeograph5f78': '\xe5\xbd\xb8',
'CjkUnifiedIdeograph5f79': '\xe5\xbd\xb9',
'UnicodeCharacter0xa9a1': '\xea\xa6\xa1',
'ArabicLetterTahFinalForm': '\xef\xbb\x82',
'HangulSyllableBbig': '\xec\x82\x91',
'TurnedGreekSmallLetterIota': '\xe2\x84\xa9',
'HangulSyllableBbid': '\xec\x82\x97',
'UnicodeCharacter0xa9a0': '\xea\xa6\xa0',
'CjkUnifiedIdeograph5f7f': '\xe5\xbd\xbf',
'CjkUnifiedIdeograph5f7d': '\xe5\xbd\xbd',
'CjkUnifiedIdeograph5f7e': '\xe5\xbd\xbe',
'CjkUnifiedIdeograph5f7b': '\xe5\xbd\xbb',
'CjkUnifiedIdeograph5f7c': '\xe5\xbd\xbc',
'CjkUnifiedIdeograph5f7a': '\xe5\xbd\xba',
'HangulSyllableBbib': '\xec\x82\xa1',
'CjkUnifiedIdeograph94e4': '\xe9\x93\xa4',
'UnicodeCharacter0xe9a1': '\xee\xa6\xa1',
'HangulSyllableBbic': '\xec\x82\xa7',
'HangulSyllableBoelt': '\xeb\xb5\x95',
'YiSyllableHop': '\xea\x89\xbc',
'CjkUnifiedIdeograph9330': '\xe9\x8c\xb0',
'CopticSmallLetterDialectpNi': '\xe2\xb2\xbb',
'YiSyllableHot': '\xea\x89\xb9',
'HangulSyllableMyelh': '\xeb\xaa\x9b',
'UnicodeCharacter0xf18c': '\xef\x86\x8c',
'YiSyllableHox': '\xea\x89\xba',
'HangulSyllableCej': '\xec\xb3\x8a',
'CjkUnifiedIdeograph9d85': '\xe9\xb6\x85',
'HangulSyllableBwat': '\xeb\xb4\xa9',
'CircledHangulCieuc': '\xe3\x89\xa8',
'CjkUnifiedIdeograph88f1': '\xe8\xa3\xb1',
'HangulSyllableMyelm': '\xeb\xaa\x96',
'CjkUnifiedIdeograph94e1': '\xe9\x93\xa1',
'UnicodeCharacter0xe9a0': '\xee\xa6\xa0',
'UnicodeCharacter0xf514': '\xef\x94\x94',
'HangulSyllableBbyeogs': '\xeb\xbc\x8b',
'CjkUnifiedIdeograph88f4': '\xe8\xa3\xb4',
'HangulSyllableBoels': '\xeb\xb5\x94',
'ArabicLigatureSadWithYehFinalForm': '\xef\xb4\xa2',
'UnicodeCharacter0xf650': '\xef\x99\x90',
'HangulSyllableJyogg': '\xec\xa3\xa2',
'LimbuVowelSignAu': '\xe1\xa4\xa6',
'UnicodeCharacter0xe9a3': '\xee\xa6\xa3',
'CjkUnifiedIdeograph9d82': '\xe9\xb6\x82',
'CjkUnifiedIdeograph5a18': '\xe5\xa8\x98',
'HangulJongseongRieultikeuthieuh': '\xe1\x87\x8f',
'UnicodeCharacter0xf515': '\xef\x94\x95',
'HangulSyllableSsed': '\xec\x8e\x8b',
'HangulSyllableDyad': '\xeb\x8c\xa3',
'LimbuVowelSignAi': '\xe1\xa4\xa4',
'SuperscriptEqualsSign': '\xe2\x81\xbc',
'HangulSyllableGweos': '\xea\xb6\x9b',
'HangulSyllableBbit': '\xec\x82\xa9',
'divisionsign': '\xc3\xb7',
'DownwardsHarpoonWithBarbRightFromBar': '\xe2\xa5\x9d',
'HangulSyllableSswaej': '\xec\x90\xa6',
'CjkUnifiedIdeograph8be2': '\xe8\xaf\xa2',
'UnicodeCharacter0xe9a2': '\xee\xa6\xa2',
'CjkUnifiedIdeograph9d89': '\xe9\xb6\x89',
'PhagspaLetterNya': '\xea\xa1\x87',
'CjkUnifiedIdeograph9d88': '\xe9\xb6\x88',
'UnicodeCharacter0xe57b': '\xee\x95\xbb',
'CjkUnifiedIdeograph7691': '\xe7\x9a\x91',
'HangulSyllableDyac': '\xeb\x8c\xb3',
'CjkUnifiedIdeograph8be0': '\xe8\xaf\xa0',
'NkoLetterEe': '\xdf\x8b',
'HangulSyllableSswaen': '\xec\x90\x94',
'WhiteShogiPiece': '\xe2\x98\x96',
'LatinSmallLetterUWithDotBelow': '\xe1\xbb\xa5',
'CjkUnifiedIdeograph8be6': '\xe8\xaf\xa6',
'OriyaLetterKha': '\xe0\xac\x96',
'LowDoublePrimeQuotationMark': '\xe3\x80\x9f',
'HangulSyllableSswael': '\xec\x90\x98',
'CjkUnifiedIdeograph7516': '\xe7\x94\x96',
'UnicodeCharacter0xf4f3': '\xef\x93\xb3',
'HangulSyllableDyab': '\xeb\x8c\xad',
'YiSyllableLurx': '\xea\x87\x92',
'CjkUnifiedIdeograph9567': '\xe9\x95\xa7',
'HiraganaLetterBe': '\xe3\x81\xb9',
'CjkUnifiedIdeograph8be8': '\xe8\xaf\xa8',
'HiraganaLetterBa': '\xe3\x81\xb0',
'HiraganaLetterBo': '\xe3\x81\xbc',
'IdenticalTo': '\xe2\x89\xa1',
'UnicodeCharacter0xf518': '\xef\x94\x98',
'HiraganaLetterBi': '\xe3\x81\xb3',
'HiraganaLetterBu': '\xe3\x81\xb6',
'SuperscriptNine': '\xe2\x81\xb9',
'HangulSyllableManj': '\xeb\xa7\x8d',
'CjkUnifiedIdeograph5193': '\xe5\x86\x93',
'ArabicKasra': '\xd9\x90',
'CyrillicSmallLetterIWithGrave': '\xd1\x9d',
'HangulSyllableTyaec': '\xed\x84\xab',
'UpBarbRightDownBarbLeftHarpoon': '\xe2\xa5\x8c',
'KhmerVowelInherentAa': '\xe1\x9e\xb5',
'ArabicLetterBehehIsolatedForm': '\xef\xad\x9a',
'HangulSyllableBbaelm': '\xeb\xba\x86',
'UnicodeCharacter0xf7e6': '\xef\x9f\xa6',
'ChamVowelSignIi': '\xea\xa8\xab',
'KhmerVowelInherentAq': '\xe1\x9e\xb4',
'CjkUnifiedIdeograph818e': '\xe8\x86\x8e',
'CjkUnifiedIdeograph818d': '\xe8\x86\x8d',
'SquareGuramu': '\xe3\x8c\x98',
'CjkUnifiedIdeograph818b': '\xe8\x86\x8b',
'CjkUnifiedIdeograph818a': '\xe8\x86\x8a',
'HangulSyllableHak': '\xed\x95\xb0',
'LeftSquareBracketWithTickInBottomCorner': '\xe2\xa6\x8f',
'HangulSyllableSwe': '\xec\x89\x90',
'HangulSyllableHan': '\xed\x95\x9c',
'HangulSyllableHam': '\xed\x95\xa8',
'HangulSyllableSwa': '\xec\x86\xa8',
'HangulSyllableHac': '\xed\x95\xaf',
'RejangConsonantSignH': '\xea\xa5\x92',
'UnicodeCharacter0xe357': '\xee\x8d\x97',
'HangulSyllableHag': '\xed\x95\x99',
'HangulSyllableHae': '\xed\x95\xb4',
'MalayalamLetterTtha': '\xe0\xb4\xa0',
'UnicodeCharacter0xf415': '\xef\x90\x95',
'RejangConsonantSignR': '\xea\xa5\x91',
'BoxDrawingsDownLightAndHorizontalHeavy': '\xe2\x94\xaf',
'BengaliLetterRa': '\xe0\xa6\xb0',
'HangulSyllableHas': '\xed\x95\xab',
'HangulSyllableHap': '\xed\x95\xb2',
'HangulSyllableHat': '\xed\x95\xb1',
'HebrewAccentAtnahHafukh': '\xd6\xa2',
'SyriacLetterSogdianZhain': '\xdd\x8d',
'CjkUnifiedIdeograph8189': '\xe8\x86\x89',
'CjkUnifiedIdeograph8188': '\xe8\x86\x88',
'CjkUnifiedIdeograph8187': '\xe8\x86\x87',
'CjkUnifiedIdeograph8186': '\xe8\x86\x86',
'LatinCapitalLetterPWithAcute': '\xe1\xb9\x94',
'CjkUnifiedIdeograph8184': '\xe8\x86\x84',
'CjkUnifiedIdeograph8183': '\xe8\x86\x83',
'CjkUnifiedIdeograph8182': '\xe8\x86\x82',
'LogicalAndWithDoubleUnderbar': '\xe2\xa9\xa0',
'CjkUnifiedIdeograph8180': '\xe8\x86\x80',
'UnicodeCharacter0xf414': '\xef\x90\x94',
'DevanagariLetterBha': '\xe0\xa4\xad',
'LatinSmallLetterSWithAcuteAndDotAbove': '\xe1\xb9\xa5',
'HangulChoseongIeungphieuph': '\xe1\x85\x8b',
'HangulSyllableHwaenh': '\xed\x99\xb6',
'CjkUnifiedIdeograph79fd': '\xe7\xa7\xbd',
'HangulSyllableHwaenj': '\xed\x99\xb5',
'CjkUnifiedIdeograph9564': '\xe9\x95\xa4',
'CjkUnifiedIdeograph7ca2': '\xe7\xb2\xa2',
'ChamVowelSignOe': '\xea\xa8\xae',
'UnicodeCharacter0xe00a': '\xee\x80\x8a',
'HangulSyllableHwaeng': '\xed\x9a\x85',
'CjkUnifiedIdeograph59f2': '\xe5\xa7\xb2',
'CjkUnifiedIdeograph59f3': '\xe5\xa7\xb3',
'CjkUnifiedIdeograph59f0': '\xe5\xa7\xb0',
'CjkUnifiedIdeograph59f1': '\xe5\xa7\xb1',
'CjkUnifiedIdeograph59f6': '\xe5\xa7\xb6',
'CjkUnifiedIdeograph59f7': '\xe5\xa7\xb7',
'CjkUnifiedIdeograph59f4': '\xe5\xa7\xb4',
'CjkUnifiedIdeograph59f5': '\xe5\xa7\xb5',
'CjkUnifiedIdeograph59f8': '\xe5\xa7\xb8',
'CjkUnifiedIdeograph59f9': '\xe5\xa7\xb9',
'CjkUnifiedIdeograph88fb': '\xe8\xa3\xbb',
'CyrillicSmallLetterBinocularO': '\xea\x99\xab',
'CjkUnifiedIdeograph7ca1': '\xe7\xb2\xa1',
'VaiSyllablePa': '\xea\x95\x90',
'ArabicLigatureYehWithAlefMaksuraIsolatedForm': '\xef\xb1\x99',
'HangulSyllableTut': '\xed\x89\x85',
'CjkUnifiedIdeograph5e7b': '\xe5\xb9\xbb',
'HangulSyllableSeot': '\xec\x84\xb5',
'UnicodeCharacter0x1a1d': '\xe1\xa8\x9d',
'HangulSyllableDyak': '\xeb\x8c\xb4',
'HangulSyllableSeop': '\xec\x84\xb6',
'UnicodeCharacter0x1a1c': '\xe1\xa8\x9c',
'HangulSyllableSeos': '\xec\x84\xaf',
'HangulSyllableJjyo': '\xec\xac\xac',
'UnicodeCharacter0xf411': '\xef\x90\x91',
'CjkUnifiedIdeograph8bd0': '\xe8\xaf\x90',
'CjkUnifiedIdeograph8bd1': '\xe8\xaf\x91',
'CjkUnifiedIdeograph8bd2': '\xe8\xaf\x92',
'CjkUnifiedIdeograph8bd3': '\xe8\xaf\x93',
'CjkUnifiedIdeograph8bd4': '\xe8\xaf\x94',
'CjkUnifiedIdeograph8bd5': '\xe8\xaf\x95',
'CjkUnifiedIdeograph8bd6': '\xe8\xaf\x96',
'CjkUnifiedIdeograph8bd7': '\xe8\xaf\x97',
'CjkUnifiedIdeograph8bd8': '\xe8\xaf\x98',
'CjkUnifiedIdeograph8bd9': '\xe8\xaf\x99',
'UnicodeCharacter0xab7f': '\xea\xad\xbf',
'HangulSyllableGging': '\xeb\x82\x91',
'UnicodeCharacter0xab7a': '\xea\xad\xba',
'UnicodeCharacter0xab7b': '\xea\xad\xbb',
'UnicodeCharacter0xa7e6': '\xea\x9f\xa6',
'HangulSyllableDyaj': '\xeb\x8c\xb2',
'UnicodeCharacter0xd994': '\xed\xa6\x94',
'CanadianSyllabicsPwii': '\xe1\x90\xbe',
'UnicodeCharacter0xf410': '\xef\x90\x90',
'CjkUnifiedIdeograph878e': '\xe8\x9e\x8e',
'HangulSyllableCyu': '\xec\xb8\x84',
'CjkUnifiedIdeograph59fb': '\xe5\xa7\xbb',
'CjkUnifiedIdeograph59fc': '\xe5\xa7\xbc',
'CjkUnifiedIdeograph59fa': '\xe5\xa7\xba',
'CjkUnifiedIdeograph59ff': '\xe5\xa7\xbf',
'CjkUnifiedIdeograph59fd': '\xe5\xa7\xbd',
'CombiningKatakanahiraganaVoicedSoundMark': '\xe3\x82\x99',
'HangulSyllableWeo': '\xec\x9b\x8c',
'UnicodeCharacter0xe3d1': '\xee\x8f\x91',
'UnicodeCharacter0xab44': '\xea\xad\x84',
'UnicodeCharacter0xd995': '\xed\xa6\x95',
'CjkUnifiedIdeograph8bda': '\xe8\xaf\x9a',
'CjkUnifiedIdeograph8bdb': '\xe8\xaf\x9b',
'CjkUnifiedIdeograph8bdc': '\xe8\xaf\x9c',
'CjkUnifiedIdeograph8bdd': '\xe8\xaf\x9d',
'CjkUnifiedIdeograph8bde': '\xe8\xaf\x9e',
'CjkUnifiedIdeograph8bdf': '\xe8\xaf\x9f',
'UnicodeCharacter0xdc65': '\xed\xb1\xa5',
'UnicodeCharacter0xab74': '\xea\xad\xb4',
'UnicodeCharacter0xab75': '\xea\xad\xb5',
'UnicodeCharacter0xab76': '\xea\xad\xb6',
'UnicodeCharacter0xab77': '\xea\xad\xb7',
'UnicodeCharacter0xab70': '\xea\xad\xb0',
'BalineseLetterNaRambat': '\xe1\xac\xa1',
'UnicodeCharacter0xab72': '\xea\xad\xb2',
'UnicodeCharacter0xab73': '\xea\xad\xb3',
'RomanNumeralFiftyThousand': '\xe2\x86\x87',
'ReversedNotSign': '\xe2\x8c\x90',
'UnicodeCharacter0xa7e4': '\xea\x9f\xa4',
'HangulSyllableDyah': '\xeb\x8c\xb7',
'UnicodeCharacter0xd992': '\xed\xa6\x92',
'NewTaiLueLetterFinalK': '\xe1\xa7\x85',
'CjkUnifiedIdeograph80dc': '\xe8\x83\x9c',
'HangulSyllableWeols': '\xec\x9b\x98',
'CjkUnifiedIdeograph80da': '\xe8\x83\x9a',
'CjkUnifiedIdeograph80df': '\xe8\x83\x9f',
'NewTaiLueLetterFinalN': '\xe1\xa7\x83',
'NewTaiLueLetterFinalM': '\xe1\xa7\x84',
'CjkUnifiedIdeograph80de': '\xe8\x83\x9e',
'NewTaiLueLetterFinalB': '\xe1\xa7\x87',
'HangulSyllableCyo': '\xec\xb5\xb8',
'EthiopicSyllableDee': '\xe1\x8b\xb4',
'HangulSyllableCyeop': '\xec\xb3\xaa',
'NewTaiLueLetterFinalD': '\xe1\xa7\x86',
'HangulSyllableWeolb': '\xec\x9b\x97',
'HangulSyllableWeolg': '\xec\x9b\x95',
'HangulSyllableCya': '\xec\xb1\xa0',
'BalineseLetterLa': '\xe1\xac\xae',
'HangulSyllableWeolh': '\xec\x9b\x9b',
'HangulSyllableWeolm': '\xec\x9b\x96',
'NewTaiLueLetterFinalV': '\xe1\xa7\x81',
'UnicodeCharacter0xe9a4': '\xee\xa6\xa4',
'CjkUnifiedIdeograph429e': '\xe4\x8a\x9e',
'UnicodeCharacter0xe9a6': '\xee\xa6\xa6',
'HangulSyllableCye': '\xec\xb3\xac',
'ArabicLetterHehMedialForm': '\xef\xbb\xac',
'CjkUnifiedIdeograph3c11': '\xe3\xb0\x91',
'HangulJongseongHieuhrieul': '\xe1\x87\xb6',
'UnicodeCharacter0xe9a8': '\xee\xa6\xa8',
'VaiSyllableSee': '\xea\x94\x96',
'UnicodeCharacter0xd853': '\xed\xa1\x93',
'HangulSyllableGgwaelb': '\xea\xbd\xaf',
'UnicodeCharacter0xe0bc': '\xee\x82\xbc',
'HangulSyllableGgwaelg': '\xea\xbd\xad',
'UnicodeCharacter0xf7dc': '\xef\x9f\x9c',
'HangulSyllableByuss': '\xeb\xb8\x84',
'HangulSyllableGgwaelh': '\xea\xbd\xb3',
'HangulSyllableGgwaelm': '\xea\xbd\xae',
'CjkUnifiedIdeograph80d2': '\xe8\x83\x92',
'FullOuterJoin': '\xe2\x9f\x97',
'CjkUnifiedIdeograph80d0': '\xe8\x83\x90',
'CjkUnifiedIdeograph80d1': '\xe8\x83\x91',
'CjkUnifiedIdeograph80d6': '\xe8\x83\x96',
'CjkUnifiedIdeograph80d7': '\xe8\x83\x97',
'CjkUnifiedIdeograph80d4': '\xe8\x83\x94',
'CjkUnifiedIdeograph80d5': '\xe8\x83\x95',
'UnicodeCharacter0xd991': '\xed\xa6\x91',
'CjkUnifiedIdeograph80d8': '\xe8\x83\x98',
'CjkUnifiedIdeograph80d9': '\xe8\x83\x99',
'UnicodeCharacter0xd989': '\xed\xa6\x89',
'UnicodeCharacter0xd988': '\xed\xa6\x88',
'HangulSyllableCyeob': '\xec\xb3\xa1',
'HangulSyllableCyeoc': '\xec\xb3\xa7',
'DoublestruckItalicSmallE': '\xe2\x85\x87',
'SymbolForUnitSeparator': '\xe2\x90\x9f',
'HangulSyllableCyeog': '\xec\xb3\x91',
'BraillePatternDots127': '\xe2\xa1\x83',
'BraillePatternDots126': '\xe2\xa0\xa3',
'BraillePatternDots125': '\xe2\xa0\x93',
'BraillePatternDots124': '\xe2\xa0\x8b',
'BraillePatternDots123': '\xe2\xa0\x87',
'DigitEightFullStop': '\xe2\x92\x8f',
'CjkUnifiedIdeograph8e05': '\xe8\xb8\x85',
'CjkUnifiedIdeograph8e04': '\xe8\xb8\x84',
'CjkUnifiedIdeograph8e09': '\xe8\xb8\x89',
'CjkUnifiedIdeograph8e08': '\xe8\xb8\x88',
'EthiopicSyllableZoa': '\xe2\xb6\x8b',
'BraillePatternDots128': '\xe2\xa2\x83',
'HangulSyllableRyab': '\xeb\x9f\x85',
'UnicodeCharacter0x124e': '\xe1\x89\x8e',
'CombiningRingBelow': '\xcc\xa5',
'UnicodeCharacter0x124f': '\xe1\x89\x8f',
'SquareVOverM': '\xe3\x8f\x9e',
'ArabicSmallHighLigatureSadWithLamWithAlefMaksura': '\xdb\x96',
'HangulSyllableCyeoh': '\xec\xb3\xab',
'UnicodeCharacter0xee56': '\xee\xb9\x96',
'CjkUnifiedIdeograph95af': '\xe9\x96\xaf',
'CjkUnifiedIdeograph95ae': '\xe9\x96\xae',
'CjkUnifiedIdeograph95ad': '\xe9\x96\xad',
'CjkUnifiedIdeograph95ac': '\xe9\x96\xac',
'RightSquareBracketWithTickInBottomCorner': '\xe2\xa6\x8e',
'CjkUnifiedIdeograph95aa': '\xe9\x96\xaa',
'HangulSyllableBwip': '\xeb\xb7\xae',
'CjkUnifiedIdeograph6aa7': '\xe6\xaa\xa7',
'HangulSyllablePweoss': '\xed\x92\xa8',
'HangulSyllableBwig': '\xeb\xb7\x95',
'HangulSyllableBwid': '\xeb\xb7\x9b',
'HangulSyllableBwic': '\xeb\xb7\xab',
'HangulSyllableBwib': '\xeb\xb7\xa5',
'HangulSyllableBwin': '\xeb\xb7\x98',
'HangulSyllableBwim': '\xeb\xb7\xa4',
'HangulSyllableBwil': '\xeb\xb7\x9c',
'HangulSyllableBwik': '\xeb\xb7\xac',
'HangulSyllableBwij': '\xeb\xb7\xaa',
'HangulSyllableBwih': '\xeb\xb7\xaf',
'SubscriptZero': '\xe2\x82\x80',
'UnicodeCharacter0x1249': '\xe1\x89\x89',
'UnicodeCharacter0xde98': '\xed\xba\x98',
'CjkUnifiedIdeograph8e0c': '\xe8\xb8\x8c',
'CjkUnifiedIdeograph8e0b': '\xe8\xb8\x8b',
'CjkUnifiedIdeograph8e0a': '\xe8\xb8\x8a',
'UnicodeCharacter0xde99': '\xed\xba\x99',
'CjkUnifiedIdeograph8e0f': '\xe8\xb8\x8f',
'CjkUnifiedIdeograph8e0e': '\xe8\xb8\x8e',
'CjkUnifiedIdeograph8e0d': '\xe8\xb8\x8d',
'IdeographicTelegraphSymbolForDayNine': '\xe3\x8f\xa8',
'UnicodeCharacter0xf7db': '\xef\x9f\x9b',
'LeftDoubleQuotationMark': '\xe2\x80\x9c',
'UnicodeCharacter0xdf2c': '\xed\xbc\xac',
'EightPointedRectilinearBlackStar': '\xe2\x9c\xb7',
'CjkUnifiedIdeograph95a7': '\xe9\x96\xa7',
'CjkUnifiedIdeograph95a6': '\xe9\x96\xa6',
'CjkUnifiedIdeograph95a5': '\xe9\x96\xa5',
'CjkUnifiedIdeograph95a4': '\xe9\x96\xa4',
'CjkUnifiedIdeograph95a3': '\xe9\x96\xa3',
'CjkUnifiedIdeograph95a2': '\xe9\x96\xa2',
'EthiopicSyllableSebatbeitFwa': '\xe1\x8e\x88',
'CjkUnifiedIdeograph95a0': '\xe9\x96\xa0',
'UnicodeCharacter0xde90': '\xed\xba\x90',
'CjkUnifiedIdeograph95a9': '\xe9\x96\xa9',
'CjkUnifiedIdeograph95a8': '\xe9\x96\xa8',
'UnicodeCharacter0xde91': '\xed\xba\x91',
'CjkUnifiedIdeograph9c4f': '\xe9\xb1\x8f',
'UnicodeCharacter0xde96': '\xed\xba\x96',
'CjkUnifiedIdeograph9c4d': '\xe9\xb1\x8d',
'CjkUnifiedIdeograph9c4e': '\xe9\xb1\x8e',
'CjkUnifiedIdeograph9c4b': '\xe9\xb1\x8b',
'CjkUnifiedIdeograph9c4c': '\xe9\xb1\x8c',
'KannadaLengthMark': '\xe0\xb3\x95',
'CjkUnifiedIdeograph9c4a': '\xe9\xb1\x8a',
'CjkUnifiedIdeograph37d4': '\xe3\x9f\x94',
'CjkUnifiedIdeograph37d5': '\xe3\x9f\x95',
'CjkUnifiedIdeograph37d6': '\xe3\x9f\x96',
'CjkUnifiedIdeograph37d7': '\xe3\x9f\x97',
'VerticalLineWithCircleAbove': '\xe2\xab\xaf',
'CjkUnifiedIdeograph37d1': '\xe3\x9f\x91',
'CjkUnifiedIdeograph37d2': '\xe3\x9f\x92',
'CjkUnifiedIdeograph37d3': '\xe3\x9f\x93',
'CjkUnifiedIdeograph4037': '\xe4\x80\xb7',
'TeluguVowelSignVocalicRr': '\xe0\xb1\x84',
'CjkUnifiedIdeograph37d8': '\xe3\x9f\x98',
'CjkUnifiedIdeograph37d9': '\xe3\x9f\x99',
'HangulSyllableByags': '\xeb\xb1\x8f',
'UnicodeCharacter0xf278': '\xef\x89\xb8',
'MongolianLetterAliGaliZa': '\xe1\xa2\x96',
'YiSyllableWax': '\xea\x8a\x81',
'CjkUnifiedIdeograph4035': '\xe4\x80\xb5',
'HangulSyllableRelh': '\xeb\xa0\x97',
'CjkUnifiedIdeograph421c': '\xe4\x88\x9c',
'CjkUnifiedIdeograph421b': '\xe4\x88\x9b',
'CjkUnifiedIdeograph421a': '\xe4\x88\x9a',
'CjkUnifiedIdeograph92ac': '\xe9\x8a\xac',
'CjkUnifiedIdeograph92ad': '\xe9\x8a\xad',
'CjkUnifiedIdeograph421f': '\xe4\x88\x9f',
'CjkUnifiedIdeograph421e': '\xe4\x88\x9e',
'CjkUnifiedIdeograph421d': '\xe4\x88\x9d',
'MidlineHorizontalEllipsis': '\xe2\x8b\xaf',
'ReversedDoublePrime': '\xe2\x80\xb6',
'LimbuLetterMa': '\xe1\xa4\x94',
'CjkUnifiedIdeograph7008': '\xe7\x80\x88',
'SouthEastArrowAndSouthWestArrow': '\xe2\xa4\xa9',
'PlusSignInRightHalfCircle': '\xe2\xa8\xae',
'BengaliDigitZero': '\xe0\xa7\xa6',
'TibetanMarkPaluta': '\xe0\xbe\x85',
'DieFace6': '\xe2\x9a\x85',
'DieFace5': '\xe2\x9a\x84',
'DieFace4': '\xe2\x9a\x83',
'DieFace3': '\xe2\x9a\x82',
'DieFace2': '\xe2\x9a\x81',
'DieFace1': '\xe2\x9a\x80',
'CjkUnifiedIdeograph37dd': '\xe3\x9f\x9d',
'CjkUnifiedIdeograph37de': '\xe3\x9f\x9e',
'LineSeparator': '\xe2\x80\xa8',
'CjkUnifiedIdeograph37da': '\xe3\x9f\x9a',
'CjkUnifiedIdeograph37db': '\xe3\x9f\x9b',
'CjkUnifiedIdeograph37dc': '\xe3\x9f\x9c',
'CanadianSyllabicsLwoo': '\xe1\x93\xa4',
'ArabicLigatureNoonWithHehMedialForm': '\xef\xb3\xaf',
'UnicodeCharacter0xdede': '\xed\xbb\x9e',
'GermanPennySign': '\xe2\x82\xb0',
'LatinSmallLetterTWithDiaeresis': '\xe1\xba\x97',
'CjkUnifiedIdeograph9c49': '\xe9\xb1\x89',
'CjkUnifiedIdeograph9c46': '\xe9\xb1\x86',
'CjkUnifiedIdeograph9c47': '\xe9\xb1\x87',
'CjkUnifiedIdeograph9c44': '\xe9\xb1\x84',
'BlackDiamondWithDownArrow': '\xe2\xa7\xaa',
'CjkUnifiedIdeograph9c42': '\xe9\xb1\x82',
'CjkUnifiedIdeograph9c43': '\xe9\xb1\x83',
'CjkUnifiedIdeograph9c40': '\xe9\xb1\x80',
'CjkUnifiedIdeograph9c41': '\xe9\xb1\x81',
'CjkUnifiedIdeograph4ead': '\xe4\xba\xad',
'CjkUnifiedIdeograph4eae': '\xe4\xba\xae',
'TildeWithDotBelow': '\xe2\xb8\x9f',
'CjkUnifiedIdeograph4eaf': '\xe4\xba\xaf',
'CjkUnifiedIdeograph92a8': '\xe9\x8a\xa8',
'CjkUnifiedIdeograph92a9': '\xe9\x8a\xa9',
'CjkUnifiedIdeograph4219': '\xe4\x88\x99',
'CjkUnifiedIdeograph4218': '\xe4\x88\x98',
'CjkUnifiedIdeograph7005': '\xe7\x80\x85',
'ArabicLetterFehWithThreeDotsPointingUpwardsBelow': '\xdd\xa1',
'TamilSignVirama': '\xe0\xaf\x8d',
'CjkUnifiedIdeograph4213': '\xe4\x88\x93',
'CjkUnifiedIdeograph4212': '\xe4\x88\x92',
'CjkUnifiedIdeograph4211': '\xe4\x88\x91',
'CjkUnifiedIdeograph4210': '\xe4\x88\x90',
'CjkUnifiedIdeograph4217': '\xe4\x88\x97',
'CjkUnifiedIdeograph4216': '\xe4\x88\x96',
'CjkUnifiedIdeograph4215': '\xe4\x88\x95',
'CjkUnifiedIdeograph4214': '\xe4\x88\x94',
'CircledIdeographAttention': '\xe3\x8a\x9f',
'ModifierLetterCapitalReversedN': '\xe1\xb4\xbb',
'HangulSyllableBwegs': '\xeb\xb6\xbb',
'CjkUnifiedIdeograph3cd7': '\xe3\xb3\x97',
'GurmukhiLetterEe': '\xe0\xa8\x8f',
'ModifierLetterCapitalReversedE': '\xe1\xb4\xb2',
'YiSyllableTax': '\xea\x84\xa2',
'CyrillicCapitalLetterYu': '\xd0\xae',
'CjkUnifiedIdeograph58a9': '\xe5\xa2\xa9',
'TibetanLetterFixedformRa': '\xe0\xbd\xaa',
'CjkUnifiedIdeograph478d': '\xe4\x9e\x8d',
'YiSyllableTap': '\xea\x84\xa4',
'YiSyllableTat': '\xea\x84\xa1',
'ArabicindicFourthRoot': '\xd8\x87',
'CjkUnifiedIdeograph3fb7': '\xe3\xbe\xb7',
'CjkUnifiedIdeograph3fb6': '\xe3\xbe\xb6',
'CjkUnifiedIdeograph3fb5': '\xe3\xbe\xb5',
'CjkUnifiedIdeograph3fb4': '\xe3\xbe\xb4',
'CjkUnifiedIdeograph3fb3': '\xe3\xbe\xb3',
'CjkUnifiedIdeograph3fb2': '\xe3\xbe\xb2',
'CjkUnifiedIdeograph3fb1': '\xe3\xbe\xb1',
'CjkUnifiedIdeograph3fb0': '\xe3\xbe\xb0',
'CjkUnifiedIdeograph3fb9': '\xe3\xbe\xb9',
'CjkUnifiedIdeograph3fb8': '\xe3\xbe\xb8',
'UnicodeCharacter0xe616': '\xee\x98\x96',
'UnicodeCharacter0xe617': '\xee\x98\x97',
'UnicodeCharacter0xe614': '\xee\x98\x94',
'YiSyllableDdip': '\xea\x84\xb9',
'UnicodeCharacter0xe612': '\xee\x98\x92',
'SquareMvMega': '\xe3\x8e\xb9',
'UnicodeCharacter0xe610': '\xee\x98\x90',
'YiSyllableDdit': '\xea\x84\xb6',
'UnicodeCharacter0xe8e0': '\xee\xa3\xa0',
'UnicodeCharacter0xe8e1': '\xee\xa3\xa1',
'UnicodeCharacter0xe8e2': '\xee\xa3\xa2',
'YiSyllableDdix': '\xea\x84\xb7',
'UnicodeCharacter0xe8e4': '\xee\xa3\xa4',
'ModifierLetterSmallCWithCurl': '\xe1\xb6\x9d',
'UnicodeCharacter0xe2f1': '\xee\x8b\xb1',
'UnicodeCharacter0xab4d': '\xea\xad\x8d',
'GujaratiDigitThree': '\xe0\xab\xa9',
'ArabicLigatureBismillahArrahmanArraheem': '\xef\xb7\xbd',
'UnicodeCharacter0xab4f': '\xea\xad\x8f',
'TibetanSubjoinedLetterTha': '\xe0\xbe\xa0',
'UnicodeCharacter0xee48': '\xee\xb9\x88',
'CombiningGravemacron': '\xe1\xb7\x85',
'UnicodeCharacter0xecd4': '\xee\xb3\x94',
'UnicodeCharacter0xe61f': '\xee\x98\x9f',
'UnicodeCharacter0xe9aa': '\xee\xa6\xaa',
'UnicodeCharacter0xe61d': '\xee\x98\x9d',
'HangulSyllableHul': '\xed\x9b\x8c',
'UnicodeCharacter0xe61b': '\xee\x98\x9b',
'UnicodeCharacter0xe61c': '\xee\x98\x9c',
'UnicodeCharacter0xe61a': '\xee\x98\x9a',
'UnicodeCharacter0xe8ea': '\xee\xa3\xaa',
'UnicodeCharacter0xe8eb': '\xee\xa3\xab',
'UnicodeCharacter0xe8ec': '\xee\xa3\xac',
'UnicodeCharacter0xe8ed': '\xee\xa3\xad',
'UnicodeCharacter0xe8ee': '\xee\xa3\xae',
'UnicodeCharacter0xe8ef': '\xee\xa3\xaf',
'HangulSyllableCanj': '\xec\xb0\xad',
'CjkUnifiedIdeograph3fbf': '\xe3\xbe\xbf',
'CjkUnifiedIdeograph3fbe': '\xe3\xbe\xbe',
'CjkUnifiedIdeograph3fbd': '\xe3\xbe\xbd',
'CjkUnifiedIdeograph3fbc': '\xe3\xbe\xbc',
'CjkUnifiedIdeograph3fbb': '\xe3\xbe\xbb',
'CjkUnifiedIdeograph3fba': '\xe3\xbe\xba',
'CjkUnifiedIdeograph4297': '\xe4\x8a\x97',
'UnicodeCharacter0xe9ad': '\xee\xa6\xad',
'ArabicLetterGafFinalForm': '\xef\xae\x93',
'HebrewPointHatafQamats': '\xd6\xb3',
'UnicodeCharacter0xf7dd': '\xef\x9f\x9d',
'UnicodeCharacter0xe9af': '\xee\xa6\xaf',
'RightwardsDoubleArrowFromBar': '\xe2\xa4\x87',
'HangulSyllableBboelh': '\xeb\xbe\xa3',
'CjkUnifiedIdeograph610a': '\xe6\x84\x8a',
'CjkUnifiedIdeograph933b': '\xe9\x8c\xbb',
'CjkUnifiedIdeograph610c': '\xe6\x84\x8c',
'CjkUnifiedIdeograph610b': '\xe6\x84\x8b',
'CjkUnifiedIdeograph610e': '\xe6\x84\x8e',
'ModifierLetterReversedGlottalStop': '\xcb\x81',
'CjkUnifiedIdeograph933e': '\xe9\x8c\xbe',
'CjkUnifiedIdeograph610f': '\xe6\x84\x8f',
'UnicodeCharacter0xd8ac': '\xed\xa2\xac',
'UnicodeCharacter0xd8ab': '\xed\xa2\xab',
'UnicodeCharacter0xd8aa': '\xed\xa2\xaa',
'UnicodeCharacter0xeff0': '\xee\xbf\xb0',
'UnicodeCharacter0xeff7': '\xee\xbf\xb7',
'HangulSyllableBboelm': '\xeb\xbe\x9e',
'UnicodeCharacter0xd8ae': '\xed\xa2\xae',
'UnicodeCharacter0xd8ad': '\xed\xa2\xad',
'UnicodeCharacter0xeff9': '\xee\xbf\xb9',
'HangulSyllableGaem': '\xea\xb0\xac',
'CjkUnifiedIdeograph8126': '\xe8\x84\xa6',
'CircledIdeographTen': '\xe3\x8a\x89',
'ArabicLigatureSeenWithRehIsolatedForm': '\xef\xb4\x8e',
'CjkUnifiedIdeograph6109': '\xe6\x84\x89',
'CjkUnifiedIdeograph6108': '\xe6\x84\x88',
'CjkUnifiedIdeograph9339': '\xe9\x8c\xb9',
'CjkUnifiedIdeograph9338': '\xe9\x8c\xb8',
'HangulSyllablePab': '\xed\x8c\x9d',
'CjkUnifiedIdeograph6101': '\xe6\x84\x81',
'CjkUnifiedIdeograph6100': '\xe6\x84\x80',
'CjkUnifiedIdeograph6103': '\xe6\x84\x83',
'CanadianSyllabicsCarrierEe': '\xe1\x90\x88',
'CjkUnifiedIdeograph6105': '\xe6\x84\x85',
'CjkUnifiedIdeograph6104': '\xe6\x84\x84',
'CjkUnifiedIdeograph6107': '\xe6\x84\x87',
'CjkUnifiedIdeograph6106': '\xe6\x84\x86',
'HangulSyllableJweogg': '\xec\xa4\x9a',
'UnicodeCharacter0xaaa9': '\xea\xaa\xa9',
'UnicodeCharacter0xe2ce': '\xee\x8b\x8e',
'HangulSyllablePag': '\xed\x8c\x8d',
'CjkUnifiedIdeograph8e7f': '\xe8\xb9\xbf',
'HangulSyllablePae': '\xed\x8c\xa8',
'UnicodeCharacter0xd997': '\xed\xa6\x97',
'VulgarFractionTwoFifths': '\xe2\x85\x96',
'KangxiRadicalSecond': '\xe2\xbc\x84',
'HangulSyllableMwils': '\xeb\xae\x94',
'UnicodeCharacter0xd8a3': '\xed\xa2\xa3',
'HangulSyllablePak': '\xed\x8c\xa4',
'UnicodeCharacter0xd8a1': '\xed\xa2\xa1',
'CjkUnifiedIdeograph8e7e': '\xe8\xb9\xbe',
'UnicodeCharacter0xd8a7': '\xed\xa2\xa7',
'UnicodeCharacter0xd8a6': '\xed\xa2\xa6',
'UnicodeCharacter0xd8a5': '\xed\xa2\xa5',
'HangulSyllablePaj': '\xed\x8c\xa2',
'CjkUnifiedIdeograph8e7b': '\xe8\xb9\xbb',
'UnicodeCharacter0xd8a9': '\xed\xa2\xa9',
'UnicodeCharacter0xd8a8': '\xed\xa2\xa8',
'HangulSyllableRegg': '\xeb\xa0\x8a',
'SaurashtraLetterUu': '\xea\xa2\x87',
'CjkUnifiedIdeograph8e7c': '\xe8\xb9\xbc',
'HangulSyllableSswang': '\xec\x90\x89',
'HalfwidthHangulLetterRieulphieuph': '\xef\xbe\xaf',
'UnicodeCharacter0xd990': '\xed\xa6\x90',
'CjkUnifiedIdeograph4ea5': '\xe4\xba\xa5',
'HangulSyllableBal': '\xeb\xb0\x9c',
'HangulSyllableGweolt': '\xea\xb6\x95',
'CjkUnifiedIdeograph8e7a': '\xe8\xb9\xba',
'AndWithDot': '\xe2\x9f\x91',
'HangulSyllableGweols': '\xea\xb6\x94',
'HangulSyllablePan': '\xed\x8c\x90',
'HangulSyllableGweolm': '\xea\xb6\x92',
'CjkUnifiedIdeograph4ea7': '\xe4\xba\xa7',
'HangulSyllableGweolh': '\xea\xb6\x97',
'HangulSyllableGgyubs': '\xeb\x80\xba',
'HangulSyllableGaes': '\xea\xb0\xaf',
'CjkUnifiedIdeograph3cdd': '\xe3\xb3\x9d',
'HangulSyllableGweolg': '\xea\xb6\x91',
'HangulSyllablePal': '\xed\x8c\x94',
'HangulSyllableGaep': '\xea\xb0\xb6',
'CjkUnifiedIdeograph3cde': '\xe3\xb3\x9e',
'BraillePatternDots12368': '\xe2\xa2\xa7',
'UnicodeCharacter0xdf24': '\xed\xbc\xa4',
'_j': '\xe2\xb1\xbc',
'SubscriptThree': '\xe2\x82\x83',
'HanunooLetterSa': '\xe1\x9c\xb0',
'ArabicLetterYehBarreeFinalForm': '\xef\xae\xaf',
'CjkUnifiedIdeograph79d4': '\xe7\xa7\x94',
'HangulSyllableSyo': '\xec\x87\xbc',
'BraillePatternDots12367': '\xe2\xa1\xa7',
'LatinSmallLetterPWithPalatalHook': '\xe1\xb6\x88',
'UnicodeCharacter0xe7df': '\xee\x9f\x9f',
'UnicodeCharacter0xdf27': '\xed\xbc\xa7',
'HangulSyllableGaet': '\xea\xb0\xb5',
'HangulSyllableHoels': '\xed\x9a\x98',
'HangulSyllableSya': '\xec\x83\xa4',
'CjkUnifiedIdeograph9eec': '\xe9\xbb\xac',
'CjkUnifiedIdeograph9eeb': '\xe9\xbb\xab',
'CjkUnifiedIdeograph9eea': '\xe9\xbb\xaa',
'UnicodeCharacter0xd999': '\xed\xa6\x99',
'UnicodeCharacter0xeb9f': '\xee\xae\x9f',
'MalteseCross': '\xe2\x9c\xa0',
'CjkUnifiedIdeograph9eee': '\xe9\xbb\xae',
'CjkUnifiedIdeograph9eed': '\xe9\xbb\xad',
'GujaratiSignAnusvara': '\xe0\xaa\x82',
'CjkUnifiedIdeograph4781': '\xe4\x9e\x81',
'UnicodeCharacter0xe3d2': '\xee\x8f\x92',
'UnicodeCharacter0xdf23': '\xed\xbc\xa3',
'CjkUnifiedIdeograph634e': '\xe6\x8d\x8e',
'CjkUnifiedIdeograph4780': '\xe4\x9e\x80',
'UnicodeCharacter0xde9b': '\xed\xba\x9b',
'CyrillicSmallLetterGje': '\xd1\x93',
'HangulSyllableHud': '\xed\x9b\x8b',
'UnicodeCharacter0xde9c': '\xed\xba\x9c',
'HangulSyllableSswanh': '\xec\x8f\xba',
'GreekCapitalLetterKappa': '\xce\x9a',
'UnicodeCharacter0xe6fa': '\xee\x9b\xba',
'CjkUnifiedIdeograph9ee9': '\xe9\xbb\xa9',
'CjkUnifiedIdeograph9ee8': '\xe9\xbb\xa8',
'InterlinearAnnotationSeparator': '\xef\xbf\xba',
'CjkUnifiedIdeograph9ee3': '\xe9\xbb\xa3',
'ArabicSmallHighLigatureAlefWithLamWithYeh': '\xd8\x96',
'CjkUnifiedIdeograph9ee1': '\xe9\xbb\xa1',
'CjkUnifiedIdeograph9ee0': '\xe9\xbb\xa0',
'CjkUnifiedIdeograph9ee7': '\xe9\xbb\xa7',
'CjkUnifiedIdeograph9ee6': '\xe9\xbb\xa6',
'CjkUnifiedIdeograph9ee5': '\xe9\xbb\xa5',
'CjkUnifiedIdeograph9ee4': '\xe9\xbb\xa4',
'CjkUnifiedIdeograph9421': '\xe9\x90\xa1',
'CyrillicCapitalLetterJe': '\xd0\x88',
'ModifierLetterSmallBottomHalfO': '\xe1\xb5\x95',
'Stigma': '\xcf\x9a',
'UnicodeCharacter0xde9e': '\xed\xba\x9e',
'CjkUnifiedIdeograph7762': '\xe7\x9d\xa2',
'CjkUnifiedIdeograph7763': '\xe7\x9d\xa3',
'CjkUnifiedIdeograph7760': '\xe7\x9d\xa0',
'CjkUnifiedIdeograph7761': '\xe7\x9d\xa1',
'CjkUnifiedIdeograph7766': '\xe7\x9d\xa6',
'CjkUnifiedIdeograph7767': '\xe7\x9d\xa7',
'CjkUnifiedIdeograph7764': '\xe7\x9d\xa4',
'CjkUnifiedIdeograph7765': '\xe7\x9d\xa5',
'CjkUnifiedIdeograph60dd': '\xe6\x83\x9d',
'CjkUnifiedIdeograph60de': '\xe6\x83\x9e',
'CjkUnifiedIdeograph60df': '\xe6\x83\x9f',
'CjkUnifiedIdeograph7769': '\xe7\x9d\xa9',
'HangulSyllableCem': '\xec\xb3\x84',
'CjkUnifiedIdeograph60da': '\xe6\x83\x9a',
'CjkUnifiedIdeograph60db': '\xe6\x83\x9b',
'CjkUnifiedIdeograph60dc': '\xe6\x83\x9c',
'UnicodeCharacter0xf191': '\xef\x86\x91',
'HangulSyllableTyaed': '\xed\x84\x9b',
'HangulSyllableGgwep': '\xeb\x80\x8a',
'UnicodeCharacter0xd7de': '\xed\x9f\x9e',
'HangulSyllableGgwes': '\xeb\x80\x83',
'HangulSyllableGgwet': '\xeb\x80\x89',
'UnicodeCharacter0xe698': '\xee\x9a\x98',
'HeavyGreekCross': '\xe2\x9c\x9a',
'CjkUnifiedIdeograph5ec3': '\xe5\xbb\x83',
'UnicodeCharacter0xda8d': '\xed\xaa\x8d',
'HangulSyllableCen': '\xec\xb2\xb8',
'UnicodeCharacter0xda8f': '\xed\xaa\x8f',
'UnicodeCharacter0xda8a': '\xed\xaa\x8a',
'UnicodeCharacter0xda8c': '\xed\xaa\x8c',
'UnicodeCharacter0xda8b': '\xed\xaa\x8b',
'CjkUnifiedIdeograph3652': '\xe3\x99\x92',
'CjkUnifiedIdeograph3653': '\xe3\x99\x93',
'CjkUnifiedIdeograph3650': '\xe3\x99\x90',
'CjkUnifiedIdeograph3651': '\xe3\x99\x91',
'CjkUnifiedIdeograph3656': '\xe3\x99\x96',
'CjkUnifiedIdeograph3657': '\xe3\x99\x97',
'CjkUnifiedIdeograph3654': '\xe3\x99\x94',
'CjkUnifiedIdeograph3655': '\xe3\x99\x95',
'HangulSyllableGgweh': '\xeb\x80\x8b',
'CjkUnifiedIdeograph3658': '\xe3\x99\x98',
'CjkUnifiedIdeograph3659': '\xe3\x99\x99',
'HangulSyllableGgwel': '\xea\xbf\xb8',
'HangulSyllableGgwem': '\xeb\x80\x80',
'HangulSyllableGgwen': '\xea\xbf\xb4',
'HangulSyllableGgweo': '\xea\xbf\x94',
'UnicodeCharacter0xe695': '\xee\x9a\x95',
'HangulSyllableCeo': '\xec\xb2\x98',
'MongolianLetterTodoAliGaliZha': '\xe1\xa2\x99',
'LatinLetterTwoWithStroke': '\xc6\xbb',
'CjkUnifiedIdeograph776c': '\xe7\x9d\xac',
'CjkUnifiedIdeograph776a': '\xe7\x9d\xaa',
'CjkUnifiedIdeograph60d8': '\xe6\x83\x98',
'CjkUnifiedIdeograph60d9': '\xe6\x83\x99',
'CjkUnifiedIdeograph776d': '\xe7\x9d\xad',
'CjkUnifiedIdeograph776e': '\xe7\x9d\xae',
'CjkUnifiedIdeograph60d4': '\xe6\x83\x94',
'CjkUnifiedIdeograph60d5': '\xe6\x83\x95',
'CircledHangulTikeut': '\xe3\x89\xa2',
'CjkUnifiedIdeograph60d7': '\xe6\x83\x97',
'CjkUnifiedIdeograph60d0': '\xe6\x83\x90',
'CjkUnifiedIdeograph60d1': '\xe6\x83\x91',
'CjkUnifiedIdeograph60d2': '\xe6\x83\x92',
'CjkUnifiedIdeograph60d3': '\xe6\x83\x93',
'CjkUnifiedIdeograph365b': '\xe3\x99\x9b',
'CjkUnifiedIdeograph365c': '\xe3\x99\x9c',
'UnicodeCharacter0xea2f': '\xee\xa8\xaf',
'CjkUnifiedIdeograph365a': '\xe3\x99\x9a',
'CjkUnifiedIdeograph365f': '\xe3\x99\x9f',
'UnicodeCharacter0xea2a': '\xee\xa8\xaa',
'CjkUnifiedIdeograph365d': '\xe3\x99\x9d',
'CjkUnifiedIdeograph365e': '\xe3\x99\x9e',
'UnicodeCharacter0xf417': '\xef\x90\x97',
'HangulSyllableCwaeng': '\xec\xb5\x95',
'UnicodeCharacter0xf416': '\xef\x90\x96',
'UnicodeCharacter0xda89': '\xed\xaa\x89',
'UnicodeCharacter0xda88': '\xed\xaa\x88',
'UnicodeCharacter0xe2c5': '\xee\x8b\x85',
'UnicodeCharacter0xda85': '\xed\xaa\x85',
'UnicodeCharacter0xda84': '\xed\xaa\x84',
'UnicodeCharacter0xda87': '\xed\xaa\x87',
'UnicodeCharacter0xda86': '\xed\xaa\x86',
'UnicodeCharacter0xda81': '\xed\xaa\x81',
'CjkUnifiedIdeograph6e43': '\xe6\xb9\x83',
'UnicodeCharacter0xda83': '\xed\xaa\x83',
'YiSyllableTie': '\xea\x84\x9f',
'CjkUnifiedIdeograph8e76': '\xe8\xb9\xb6',
'UnicodeCharacter0xfadc': '\xef\xab\x9c',
'HangulSyllableTwal': '\xed\x87\x84',
'CjkUnifiedIdeograph873b': '\xe8\x9c\xbb',
'CjkUnifiedIdeograph80ed': '\xe8\x83\xad',
'CjkUnifiedIdeograph9424': '\xe9\x90\xa4',
'CjkUnifiedIdeograph873c': '\xe8\x9c\xbc',
'UnicodeCharacter0xd99d': '\xed\xa6\x9d',
'HangulSyllableNwiss': '\xeb\x89\xac',
'CjkUnifiedIdeograph6e44': '\xe6\xb9\x84',
'UnicodeCharacter0xf65e': '\xef\x99\x9e',
'CjkUnifiedIdeograph8e75': '\xe8\xb9\xb5',
'SquarePeezi': '\xe3\x8c\xbb',
'HangulSyllableJjiss': '\xec\xb0\xa0',
'CjkUnifiedIdeograph8e72': '\xe8\xb9\xb2',
'ArabicLigatureSheenWithHahWithMeemFinalForm': '\xef\xb5\xa7',
'YiSyllableXi': '\xea\x91\x9f',
'CyrillicSmallLetterJe': '\xd1\x98',
'CjkUnifiedIdeograph8e70': '\xe8\xb9\xb0',
'CjkUnifiedIdeograph6688': '\xe6\x9a\x88',
'CjkUnifiedIdeograph6689': '\xe6\x9a\x89',
'CjkUnifiedIdeograph6686': '\xe6\x9a\x86',
'CjkUnifiedIdeograph6687': '\xe6\x9a\x87',
'CjkUnifiedIdeograph6684': '\xe6\x9a\x84',
'CjkUnifiedIdeograph6685': '\xe6\x9a\x85',
'CjkUnifiedIdeograph6682': '\xe6\x9a\x82',
'CjkUnifiedIdeograph6683': '\xe6\x9a\x83',
'CjkUnifiedIdeograph6680': '\xe6\x9a\x80',
'CjkUnifiedIdeograph6681': '\xe6\x9a\x81',
'CircledNumberThirtyFive': '\xe3\x89\x9f',
'UnicodeCharacter0xa9f9': '\xea\xa7\xb9',
'GreekSmallLetterChi': '\xcf\x87',
'UnicodeCharacter0xa9f4': '\xea\xa7\xb4',
'CjkUnifiedIdeograph697a': '\xe6\xa5\xba',
'CjkUnifiedIdeograph697b': '\xe6\xa5\xbb',
'NumberElevenFullStop': '\xe2\x92\x92',
'CjkUnifiedIdeograph697d': '\xe6\xa5\xbd',
'CjkUnifiedIdeograph697e': '\xe6\xa5\xbe',
'CjkUnifiedIdeograph697f': '\xe6\xa5\xbf',
'UnicodeCharacter0xa9f3': '\xea\xa7\xb3',
'HangulSyllableTwan': '\xed\x87\x80',
'Undertie': '\xe2\x80\xbf',
'UnicodeCharacter0xe438': '\xee\x90\xb8',
'HalfwidthHangulLetterPieupsios': '\xef\xbe\xb4',
'CjkUnifiedIdeograph7adb': '\xe7\xab\x9b',
'UnicodeCharacter0xe439': '\xee\x90\xb9',
'RunicLetterKauna': '\xe1\x9a\xb2',
'CjkUnifiedIdeograph74a4': '\xe7\x92\xa4',
'UnicodeCharacter0xe437': '\xee\x90\xb7',
'EthiopicSyllableSha': '\xe1\x88\xb8',
'CjkUnifiedIdeograph5ec9': '\xe5\xbb\x89',
'EthiopicNumberTenThousand': '\xe1\x8d\xbc',
'CjkUnifiedIdeograph6978': '\xe6\xa5\xb8',
'CherokeeLetterLe': '\xe1\x8e\xb4',
'VaiSyllableNyu': '\xea\x96\xb9',
'CherokeeLetterLa': '\xe1\x8e\xb3',
'CjkUnifiedIdeograph6970': '\xe6\xa5\xb0',
'DownwardsHarpoonWithBarbLeftToBar': '\xe2\xa5\x99',
'CjkUnifiedIdeograph6972': '\xe6\xa5\xb2',
'CherokeeLetterLo': '\xe1\x8e\xb6',
'CjkUnifiedIdeograph6974': '\xe6\xa5\xb4',
'CherokeeLetterLi': '\xe1\x8e\xb5',
'CjkUnifiedIdeograph6976': '\xe6\xa5\xb6',
'CjkUnifiedIdeograph6977': '\xe6\xa5\xb7',
'CherokeeLetterLu': '\xe1\x8e\xb7',
'CherokeeLetterLv': '\xe1\x8e\xb8',
'VaiSyllableNya': '\xea\x95\xb0',
'CjkUnifiedIdeograph668f': '\xe6\x9a\x8f',
'CjkUnifiedIdeograph668d': '\xe6\x9a\x8d',
'CjkUnifiedIdeograph668e': '\xe6\x9a\x8e',
'CjkUnifiedIdeograph668b': '\xe6\x9a\x8b',
'CjkUnifiedIdeograph668c': '\xe6\x9a\x8c',
'VaiSyllableNyi': '\xea\x95\x88',
'CjkUnifiedIdeograph668a': '\xe6\x9a\x8a',
'CjkUnifiedIdeograph3b41': '\xe3\xad\x81',
'CjkUnifiedIdeograph3b40': '\xe3\xad\x80',
'CjkUnifiedIdeograph3b43': '\xe3\xad\x83',
'CjkUnifiedIdeograph3b42': '\xe3\xad\x82',
'CjkUnifiedIdeograph3b45': '\xe3\xad\x85',
'CjkUnifiedIdeograph3b44': '\xe3\xad\x84',
'CjkUnifiedIdeograph3b47': '\xe3\xad\x87',
'CjkUnifiedIdeograph3b46': '\xe3\xad\x86',
'CjkUnifiedIdeograph3b49': '\xe3\xad\x89',
'CjkUnifiedIdeograph38fa': '\xe3\xa3\xba',
'UnicodeCharacter0xaa6f': '\xea\xa9\xaf',
'CjkUnifiedIdeograph70b5': '\xe7\x82\xb5',
'CjkUnifiedIdeograph70b4': '\xe7\x82\xb4',
'CjkUnifiedIdeograph70b7': '\xe7\x82\xb7',
'VulgarFractionOneQuarter': '\xc2\xbc',
'CjkUnifiedIdeograph70b1': '\xe7\x82\xb1',
'CjkUnifiedIdeograph70b0': '\xe7\x82\xb0',
'CjkUnifiedIdeograph70b3': '\xe7\x82\xb3',
'CjkUnifiedIdeograph70b2': '\xe7\x82\xb2',
'HangulSyllablePyeobs': '\xed\x8f\x86',
'CjkUnifiedIdeograph70b9': '\xe7\x82\xb9',
'CjkUnifiedIdeograph70b8': '\xe7\x82\xb8',
'UnicodeCharacter0xaa6a': '\xea\xa9\xaa',
'CjkUnifiedIdeograph687a': '\xe6\xa1\xba',
'HangulSyllablePyoh': '\xed\x91\xb7',
'UnicodeCharacter0xaa6b': '\xea\xa9\xab',
'CyrillicCapitalLetterAbkhasianDze': '\xd3\xa0',
'YiSyllableZhox': '\xea\x8d\x8e',
'UnicodeCharacter0xf51e': '\xef\x94\x9e',
'UnicodeCharacter0xe573': '\xee\x95\xb3',
'UnicodeCharacter0xe5cc': '\xee\x97\x8c',
'HangulSyllablePeolh': '\xed\x8e\x8b',
'ArabicLigatureAinWithMeemWithYehFinalForm': '\xef\xb6\xb6',
'CjkUnifiedIdeograph70be': '\xe7\x82\xbe',
'CjkUnifiedIdeograph70bd': '\xe7\x82\xbd',
'CjkUnifiedIdeograph70bf': '\xe7\x82\xbf',
'CjkUnifiedIdeograph70ba': '\xe7\x82\xba',
'CjkUnifiedIdeograph70bc': '\xe7\x82\xbc',
'CjkUnifiedIdeograph70bb': '\xe7\x82\xbb',
'UnicodeCharacter0xeafa': '\xee\xab\xba',
'KannadaLetterNa': '\xe0\xb2\xa8',
'ArabicLigatureLamWithAlefWithHamzaBelowIsolatedForm': '\xef\xbb\xb9',
'CjkUnifiedIdeograph3b4a': '\xe3\xad\x8a',
'CjkUnifiedIdeograph3b4c': '\xe3\xad\x8c',
'CjkUnifiedIdeograph3b4b': '\xe3\xad\x8b',
'CjkUnifiedIdeograph3b4e': '\xe3\xad\x8e',
'CjkUnifiedIdeograph3b4d': '\xe3\xad\x8d',
'UnicodeCharacter0xdfa0': '\xed\xbe\xa0',
'CjkUnifiedIdeograph3b4f': '\xe3\xad\x8f',
'OriyaDigitZero': '\xe0\xad\xa6',
'SquareCm': '\xe3\x8e\x9d',
'HangulSyllableBbeong': '\xeb\xbb\xa5',
'SquareCo': '\xe3\x8f\x87',
'UnicodeCharacter0xeafb': '\xee\xab\xbb',
'SquareCd': '\xe3\x8f\x85',
'DentistrySymbolLightDownAndHorizontalWithCircle': '\xe2\x8f\x81',
'HangulSyllableBbeonh': '\xeb\xbb\x96',
'SquareCc': '\xe3\x8f\x84',
'KangxiRadicalWrong': '\xe2\xbe\xae',
'LaoLetterNgo': '\xe0\xba\x87',
'UnicodeCharacter0x65f': '\xd9\x9f',
'HexagramForTreading': '\xe4\xb7\x89',
'UnicodeCharacter0xeafc': '\xee\xab\xbc',
'HangulSyllableJjwa': '\xec\xab\x98',
'CjkUnifiedIdeograph8735': '\xe8\x9c\xb5',
'CjkUnifiedIdeograph9dd1': '\xe9\xb7\x91',
'CjkUnifiedIdeograph9dd0': '\xe9\xb7\x90',
'CjkUnifiedIdeograph9dd3': '\xe9\xb7\x93',
'CjkUnifiedIdeograph9dd2': '\xe9\xb7\x92',
'CjkUnifiedIdeograph9dd5': '\xe9\xb7\x95',
'CjkUnifiedIdeograph8736': '\xe8\x9c\xb6',
'CjkUnifiedIdeograph9dd7': '\xe9\xb7\x97',
'CjkUnifiedIdeograph9dd6': '\xe9\xb7\x96',
'UnicodeCharacter0xf149': '\xef\x85\x89',
'HangulSyllableRaels': '\xeb\x9e\xa4',
'CjkUnifiedIdeograph5521': '\xe5\x94\xa1',
'HangulSyllableBbeoc': '\xeb\xbb\xa7',
'CjkUnifiedIdeograph8730': '\xe8\x9c\xb0',
'DoublestruckSmallPi': '\xe2\x84\xbc',
'CjkUnifiedIdeograph8731': '\xe8\x9c\xb1',
'KhmerLetterTtha': '\xe1\x9e\x8b',
'DevanagariLetterAa': '\xe0\xa4\x86',
'KatakanaLetterSo': '\xe3\x82\xbd',
'CjkUnifiedIdeograph8732': '\xe8\x9c\xb2',
'DevanagariLetterAi': '\xe0\xa4\x90',
'CjkUnifiedIdeograph8733': '\xe8\x9c\xb3',
'HangulSyllableRyak': '\xeb\x9f\x8c',
'KatakanaLetterSe': '\xe3\x82\xbb',
'KhmerLetterTtho': '\xe1\x9e\x8d',
'MyanmarLetterRa': '\xe1\x80\x9b',
'ChamLetterNhue': '\xea\xa8\x90',
'DevanagariLetterAu': '\xe0\xa4\x94',
'MasculineOrdinalIndicator': '\xc2\xba',
'KatakanaLetterSu': '\xe3\x82\xb9',
'CjkUnifiedIdeograph6342': '\xe6\x8d\x82',
'UnicodeCharacter0xeafe': '\xee\xab\xbe',
'CjkUnifiedIdeograph9701': '\xe9\x9c\x81',
'HangulSyllableSyalt': '\xec\x83\xb1',
'LatinCapitalLetterUWithHornAndTilde': '\xe1\xbb\xae',
'HangulSyllableSyals': '\xec\x83\xb0',
'CjkUnifiedIdeograph6340': '\xe6\x8d\x80',
'ArabicLigatureThehWithAlefMaksuraFinalForm': '\xef\xb1\xba',
'UnicodeCharacter0xf41e': '\xef\x90\x9e',
'CjkUnifiedIdeograph9dda': '\xe9\xb7\x9a',
'CjkUnifiedIdeograph9ddc': '\xe9\xb7\x9c',
'CjkUnifiedIdeograph9ddb': '\xe9\xb7\x9b',
'CjkUnifiedIdeograph9dde': '\xe9\xb7\x9e',
'MalayalamVowelSignVocalicLl': '\xe0\xb5\xa3',
'UnicodeCharacter0xeaff': '\xee\xab\xbf',
'CjkUnifiedIdeograph9ddf': '\xe9\xb7\x9f',
'LogicalOrWithMiddleStem': '\xe2\xa9\x9b',
'HangulSyllableJjweogs': '\xec\xad\xa7',
'PalmBranch': '\xe2\xb8\x99',
'ArabicLigatureSadWithMeemWithMeemFinalForm': '\xef\xb5\xa6',
'VaiSyllableNggoo': '\xea\x96\x90',
'HangulSyllableCeoss': '\xec\xb2\xac',
'UnicodeCharacter0xf41d': '\xef\x90\x9d',
'CjkCompatibilityIdeographfa7a': '\xef\xa9\xba',
'HangulSyllableTwac': '\xed\x87\x93',
'HangulSyllableSyalh': '\xec\x83\xb3',
'CjkCompatibilityIdeographfa7d': '\xef\xa9\xbd',
'CjkCompatibilityIdeographfa7e': '\xef\xa9\xbe',
'BoxDrawingsUpLightAndRightDownHeavy': '\xe2\x94\xa2',
'UnicodeCharacter0xdc6a': '\xed\xb1\xaa',
'TibetanLetterBha': '\xe0\xbd\x97',
'ArabicDateSeparator': '\xd8\x8d',
'HangulSyllableCwac': '\xec\xb4\xbb',
'UnicodeCharacter0x1257': '\xe1\x89\x97',
'UnicodeCharacter0x82': '\xc2\x82',
'BengaliLetterYya': '\xe0\xa7\x9f',
'UnicodeCharacter0x80': '\xc2\x80',
'UnicodeCharacter0x81': '\xc2\x81',
'UnicodeCharacter0x86': '\xc2\x86',
'UnicodeCharacter0x87': '\xc2\x87',
'TibetanSymbolNorBuNyisKhyil': '\xe0\xbf\x8a',
'UnicodeCharacter0x85': '\xc2\x85',
'UnicodeCharacter0xd99f': '\xed\xa6\x9f',
'CjkUnifiedIdeograph4e5a': '\xe4\xb9\x9a',
'UnicodeCharacter0x88': '\xc2\x88',
'CjkUnifiedIdeograph4e5c': '\xe4\xb9\x9c',
'CjkUnifiedIdeograph4e5d': '\xe4\xb9\x9d',
'CjkUnifiedIdeograph4e5e': '\xe4\xb9\x9e',
'CjkUnifiedIdeograph4e5f': '\xe4\xb9\x9f',
'RightpointingCurvedAngleBracket': '\xe2\xa7\xbd',
'HangulSyllableSulp': '\xec\x88\xa6',
'HangulSyllableSuls': '\xec\x88\xa4',
'YiSyllableMgex': '\xea\x88\xbd',
'BugineseLetterPa': '\xe1\xa8\x84',
'CjkUnifiedIdeograph7e64': '\xe7\xb9\xa4',
'CjkUnifiedIdeograph7e65': '\xe7\xb9\xa5',
'ThaiCharacterThoPhuthao': '\xe0\xb8\x92',
'CyrillicCapitalLetterClosedLittleYus': '\xea\x99\x98',
'HangulSyllableRyaj': '\xeb\x9f\x8a',
'HangulSyllableKyeobs': '\xec\xbc\xae',
'CjkUnifiedIdeograph6870': '\xe6\xa1\xb0',
'CjkUnifiedIdeograph72e4': '\xe7\x8b\xa4',
'UnicodeCharacter0xf41a': '\xef\x90\x9a',
'CanadianSyllabicsCarrierLee': '\xe1\x98\xa5',
'CjkCompatibilityIdeographfa79': '\xef\xa9\xb9',
'R': '\xce\xa1',
'UnicodeCharacter0xd29': '\xe0\xb4\xa9',
'UnicodeCharacter0xd85f': '\xed\xa1\x9f',
'CombiningCyrillicLetterGhe': '\xe2\xb7\xa2',
'CjkCompatibilityIdeographfa70': '\xef\xa9\xb0',
'CjkCompatibilityIdeographfa71': '\xef\xa9\xb1',
'CjkCompatibilityIdeographfa72': '\xef\xa9\xb2',
'CjkCompatibilityIdeographfa73': '\xef\xa9\xb3',
'CjkCompatibilityIdeographfa74': '\xef\xa9\xb4',
'HangulSyllableRuc': '\xeb\xa3\xbf',
'CjkCompatibilityIdeographfa76': '\xef\xa9\xb6',
'CjkCompatibilityIdeographfa77': '\xef\xa9\xb7',
'YiSyllableMgep': '\xea\x88\xbf',
'BengaliVowelSignIi': '\xe0\xa7\x80',
'DoubleVerticalBarDoubleRightTurnstile': '\xe2\x8a\xab',
'CyrillicCapitalLetterEmWithTail': '\xd3\x8d',
'UnicodeCharacter0x8b': '\xc2\x8b',
'CjkUnifiedIdeograph4e59': '\xe4\xb9\x99',
'UnicodeCharacter0xf075': '\xef\x81\xb5',
'UnicodeCharacter0x8a': '\xc2\x8a',
'UnicodeCharacter0x8f': '\xc2\x8f',
'UnicodeCharacter0xf072': '\xef\x81\xb2',
'UnicodeCharacter0x8d': '\xc2\x8d',
'UnicodeCharacter0x8e': '\xc2\x8e',
'CjkUnifiedIdeograph4e50': '\xe4\xb9\x90',
'CjkUnifiedIdeograph4e51': '\xe4\xb9\x91',
'CjkUnifiedIdeograph4e52': '\xe4\xb9\x92',
'CjkUnifiedIdeograph4e53': '\xe4\xb9\x93',
'CjkUnifiedIdeograph4e54': '\xe4\xb9\x94',
'CjkUnifiedIdeograph4e55': '\xe4\xb9\x95',
'CjkUnifiedIdeograph4e56': '\xe4\xb9\x96',
'CjkUnifiedIdeograph4e57': '\xe4\xb9\x97',
'HangulSyllableBbweb': '\xec\x80\x95',
'HangulSyllableBbwec': '\xec\x80\x9b',
'UnicodeCharacter0xd99e': '\xed\xa6\x9e',
'HangulSyllableBbweg': '\xec\x80\x85',
'UnicodeCharacter0xaba8': '\xea\xae\xa8',
'UnicodeCharacter0xaba9': '\xea\xae\xa9',
'UnicodeCharacter0xaba6': '\xea\xae\xa6',
'UnicodeCharacter0xaba7': '\xea\xae\xa7',
'UnicodeCharacter0xaba4': '\xea\xae\xa4',
'UnicodeCharacter0xaba5': '\xea\xae\xa5',
'UnicodeCharacter0xaba2': '\xea\xae\xa2',
'HexagramForSplittingApart': '\xe4\xb7\x96',
'UnicodeCharacter0xaba0': '\xea\xae\xa0',
'UnicodeCharacter0xaba1': '\xea\xae\xa1',
'HangulSyllableBbwes': '\xec\x80\x97',
'HangulSyllableBbwep': '\xec\x80\x9e',
'CjkUnifiedIdeograph9708': '\xe9\x9c\x88',
'HangulSyllableBbwet': '\xec\x80\x9d',
'CjkUnifiedIdeograph8414': '\xe8\x90\x94',
'HangulSyllableObs': '\xec\x98\xb6',
'CjkUnifiedIdeograph4a59': '\xe4\xa9\x99',
'UnicodeCharacter0xd99b': '\xed\xa6\x9b',
'HangulSyllableNaels': '\xeb\x83\x80',
'HangulSyllableNaelp': '\xeb\x83\x82',
'HangulSyllableGwa': '\xea\xb3\xbc',
'HangulSyllableNaelt': '\xeb\x83\x81',
'HangulSyllableGgoeng': '\xea\xbe\x95',
'ParenthesizedNumberSeventeen': '\xe2\x92\x84',
'CjkUnifiedIdeograph57d6': '\xe5\x9f\x96',
'CjkUnifiedIdeograph57d7': '\xe5\x9f\x97',
'CjkUnifiedIdeograph57d4': '\xe5\x9f\x94',
'CjkUnifiedIdeograph57d5': '\xe5\x9f\x95',
'CjkUnifiedIdeograph57d2': '\xe5\x9f\x92',
'CjkUnifiedIdeograph57d3': '\xe5\x9f\x93',
'CjkUnifiedIdeograph57d0': '\xe5\x9f\x90',
'CjkUnifiedIdeograph4a55': '\xe4\xa9\x95',
'CjkUnifiedIdeograph9a65': '\xe9\xa9\xa5',
'HangulSyllableGolp': '\xea\xb3\xae',
'LepchaLetterFla': '\xe1\xb0\x92',
'CjkUnifiedIdeograph57d8': '\xe5\x9f\x98',
'CjkUnifiedIdeograph57d9': '\xe5\x9f\x99',
'BengaliLetterU': '\xe0\xa6\x89',
'KayahLiLetterA': '\xea\xa4\xa2',
'HangulSyllablePyol': '\xed\x91\xa4',
'KayahLiLetterI': '\xea\xa4\xa4',
'CjkUnifiedIdeograph4a51': '\xe4\xa9\x91',
'BengaliLetterE': '\xe0\xa6\x8f',
'HangulSyllablePyon': '\xed\x91\xa0',
'BengaliLetterA': '\xe0\xa6\x85',
'UnicodeCharacter0xabaf': '\xea\xae\xaf',
'UnicodeCharacter0xabad': '\xea\xae\xad',
'BengaliLetterO': '\xe0\xa6\x93',
'UnicodeCharacter0xabab': '\xea\xae\xab',
'BengaliLetterI': '\xe0\xa6\x87',
'UnicodeCharacter0xabaa': '\xea\xae\xaa',
'CjkUnifiedIdeograph57df': '\xe5\x9f\x9f',
'CjkUnifiedIdeograph8681': '\xe8\x9a\x81',
'CjkUnifiedIdeograph57dd': '\xe5\x9f\x9d',
'HebrewAccentMerkhaKefula': '\xd6\xa6',
'CjkUnifiedIdeograph57db': '\xe5\x9f\x9b',
'CjkUnifiedIdeograph57dc': '\xe5\x9f\x9c',
'CjkUnifiedIdeograph8686': '\xe8\x9a\x86',
'CjkUnifiedIdeograph57da': '\xe5\x9f\x9a',
'CjkUnifiedIdeograph8688': '\xe8\x9a\x88',
'CjkUnifiedIdeograph8689': '\xe8\x9a\x89',
'GujaratiLetterVocalicRr': '\xe0\xab\xa0',
'TamilYearSign': '\xe0\xaf\xb5',
'HangulJongseongYesieung': '\xe1\x87\xb0',
'UnicodeCharacter0xaa60': '\xea\xa9\xa0',
'UnicodeCharacter0xd99a': '\xed\xa6\x9a',
'GreekCapitalLetterIotaWithDasia': '\xe1\xbc\xb9',
'UnicodeCharacter0xf4a6': '\xef\x92\xa6',
'CopticSmallLetterKsi': '\xe2\xb2\x9d',
'UnicodeCharacter0xaa62': '\xea\xa9\xa2',
'ArabicLigatureAinWithMeemIsolatedForm': '\xef\xb0\xaa',
'UnicodeCharacter0xaa63': '\xea\xa9\xa3',
'LatinSmallLetterUpsilon': '\xca\x8a',
'CjkUnifiedIdeograph6b88': '\xe6\xae\x88',
'CjkUnifiedIdeograph7fef': '\xe7\xbf\xaf',
'UnicodeCharacter0xe75f': '\xee\x9d\x9f',
'CjkUnifiedIdeograph7fed': '\xe7\xbf\xad',
'CjkUnifiedIdeograph6b89': '\xe6\xae\x89',
'CjkUnifiedIdeograph7feb': '\xe7\xbf\xab',
'CjkUnifiedIdeograph7fec': '\xe7\xbf\xac',
'CjkUnifiedIdeograph8ac8': '\xe8\xab\x88',
'CjkUnifiedIdeograph7fea': '\xe7\xbf\xaa',
'CjkUnifiedIdeograph8ac6': '\xe8\xab\x86',
'CjkUnifiedIdeograph8ac7': '\xe8\xab\x87',
'CjkUnifiedIdeograph8ac4': '\xe8\xab\x84',
'CjkUnifiedIdeograph8ac5': '\xe8\xab\x85',
'CjkUnifiedIdeograph8ac2': '\xe8\xab\x82',
'CjkUnifiedIdeograph8ac3': '\xe8\xab\x83',
'CjkUnifiedIdeograph8ac0': '\xe8\xab\x80',
'CjkUnifiedIdeograph8ac1': '\xe8\xab\x81',
'DentistrySymbolLightVerticalWithTriangle': '\xe2\x8f\x83',
'CjkUnifiedIdeograph802e': '\xe8\x80\xae',
'CjkUnifiedIdeograph802f': '\xe8\x80\xaf',
'WhiteSmallLozenge': '\xe2\xac\xab',
'CjkUnifiedIdeograph9f32': '\xe9\xbc\xb2',
'BalineseMusicalSymbolDeung': '\xe1\xad\xa8',
'CjkUnifiedIdeograph802b': '\xe8\x80\xab',
'CjkUnifiedIdeograph802c': '\xe8\x80\xac',
'UnicodeCharacter0xaa68': '\xea\xa9\xa8',
'UnicodeCharacter0xf1eb': '\xef\x87\xab',
'HangulSyllableSyek': '\xec\x86\x88',
'UnicodeCharacter0xf1ed': '\xef\x87\xad',
'UnicodeCharacter0xf1ee': '\xef\x87\xae',
'CjkUnifiedIdeograph9f38': '\xe9\xbc\xb8',
'CjkUnifiedIdeograph9f39': '\xe9\xbc\xb9',
'CjkUnifiedIdeograph49a8': '\xe4\xa6\xa8',
'CjkUnifiedIdeograph49a9': '\xe4\xa6\xa9',
'LatinCapitalLetterRWithCaron': '\xc5\x98',
'CjkUnifiedIdeograph49a0': '\xe4\xa6\xa0',
'CjkUnifiedIdeograph49a1': '\xe4\xa6\xa1',
'CjkUnifiedIdeograph49a2': '\xe4\xa6\xa2',
'CjkUnifiedIdeograph49a3': '\xe4\xa6\xa3',
'CjkUnifiedIdeograph49a4': '\xe4\xa6\xa4',
'CjkUnifiedIdeograph49a5': '\xe4\xa6\xa5',
'CjkUnifiedIdeograph49a6': '\xe4\xa6\xa6',
'CjkUnifiedIdeograph49a7': '\xe4\xa6\xa7',
'CjkUnifiedIdeograph7fe6': '\xe7\xbf\xa6',
'CjkUnifiedIdeograph7fe7': '\xe7\xbf\xa7',
'CjkUnifiedIdeograph7fe4': '\xe7\xbf\xa4',
'CjkUnifiedIdeograph7fe5': '\xe7\xbf\xa5',
'CjkUnifiedIdeograph7fe2': '\xe7\xbf\xa2',
'CjkUnifiedIdeograph7fe3': '\xe7\xbf\xa3',
'CjkUnifiedIdeograph7fe0': '\xe7\xbf\xa0',
'CjkUnifiedIdeograph7fe1': '\xe7\xbf\xa1',
'CjkUnifiedIdeograph8acf': '\xe8\xab\x8f',
'CjkUnifiedIdeograph6b86': '\xe6\xae\x86',
'CjkUnifiedIdeograph8acd': '\xe8\xab\x8d',
'CjkUnifiedIdeograph8ace': '\xe8\xab\x8e',
'CjkUnifiedIdeograph8acb': '\xe8\xab\x8b',
'CjkUnifiedIdeograph8acc': '\xe8\xab\x8c',
'CjkUnifiedIdeograph7fe8': '\xe7\xbf\xa8',
'CjkRadicalDog': '\xe2\xba\xa8',
'CjkUnifiedIdeograph578a': '\xe5\x9e\x8a',
'UnionAboveIntersection': '\xe2\xa9\x86',
'TamilDigitFour': '\xe0\xaf\xaa',
'HangulSyllablePyot': '\xed\x91\xb5',
'YiSyllableI': '\xea\x80\x82',
'HangulSyllableNyils': '\xeb\x8a\xb8',
'HangulSyllableNyilp': '\xeb\x8a\xba',
'UnicodeCharacter0xdb4c': '\xed\xad\x8c',
'CjkUnifiedIdeograph49aa': '\xe4\xa6\xaa',
'CjkUnifiedIdeograph49ab': '\xe4\xa6\xab',
'CjkUnifiedIdeograph49ac': '\xe4\xa6\xac',
'CjkUnifiedIdeograph49ad': '\xe4\xa6\xad',
'CjkUnifiedIdeograph49ae': '\xe4\xa6\xae',
'CjkUnifiedIdeograph49af': '\xe4\xa6\xaf',
'CjkUnifiedIdeograph7bbb': '\xe7\xae\xbb',
'CjkUnifiedIdeograph8024': '\xe8\x80\xa4',
'CjkUnifiedIdeograph8025': '\xe8\x80\xa5',
'CjkUnifiedIdeograph8026': '\xe8\x80\xa6',
'CjkUnifiedIdeograph8027': '\xe8\x80\xa7',
'CjkUnifiedIdeograph8020': '\xe8\x80\xa0',
'LatinCapitalLetterHWithDiaeresis': '\xe1\xb8\xa6',
'CjkUnifiedIdeograph8022': '\xe8\x80\xa2',
'CjkUnifiedIdeograph8023': '\xe8\x80\xa3',
'UnicodeCharacter0xf1e0': '\xef\x87\xa0',
'KatakanaVoicedIterationMark': '\xe3\x83\xbe',
'UnicodeCharacter0xf1e2': '\xef\x87\xa2',
'UnicodeCharacter0xf1e3': '\xef\x87\xa3',
'CjkUnifiedIdeograph8028': '\xe8\x80\xa8',
'CjkUnifiedIdeograph8029': '\xe8\x80\xa9',
'UnicodeCharacter0xf1e6': '\xef\x87\xa6',
'UnicodeCharacter0xf1e7': '\xef\x87\xa7',
'CjkUnifiedIdeograph964c': '\xe9\x99\x8c',
'CjkUnifiedIdeograph964b': '\xe9\x99\x8b',
'CjkUnifiedIdeograph964a': '\xe9\x99\x8a',
'CjkUnifiedIdeograph964f': '\xe9\x99\x8f',
'CjkUnifiedIdeograph964e': '\xe9\x99\x8e',
'CjkUnifiedIdeograph964d': '\xe9\x99\x8d',
'CjkUnifiedIdeograph5ffa': '\xe5\xbf\xba',
'CjkUnifiedIdeograph5ffc': '\xe5\xbf\xbc',
'ArabicindicDigitTwo': '\xd9\xa2',
'CjkUnifiedIdeograph5ffe': '\xe5\xbf\xbe',
'CjkUnifiedIdeograph5ffd': '\xe5\xbf\xbd',
'CjkUnifiedIdeograph5fff': '\xe5\xbf\xbf',
'MongolianDigitOne': '\xe1\xa0\x91',
'CjkUnifiedIdeograph515b': '\xe5\x85\x9b',
'CjkUnifiedIdeograph515a': '\xe5\x85\x9a',
'CjkUnifiedIdeograph515f': '\xe5\x85\x9f',
'CjkUnifiedIdeograph515e': '\xe5\x85\x9e',
'CjkUnifiedIdeograph515d': '\xe5\x85\x9d',
'YiSyllableEx': '\xea\x80\x93',
'MalayalamDigitThree': '\xe0\xb5\xa9',
'UnicodeCharacter0xdb4a': '\xed\xad\x8a',
'CircledMinus': '\xe2\x8a\x96',
'UnicodeCharacter0xf81a': '\xef\xa0\x9a',
'CombiningUsAbove': '\xe1\xb7\x92',
'UnicodeCharacter0xf81c': '\xef\xa0\x9c',
'UnicodeCharacter0xf81b': '\xef\xa0\x9b',
'UnicodeCharacter0xf81e': '\xef\xa0\x9e',
'Perpendicular': '\xe2\x9f\x82',
'UnicodeCharacter0xf81f': '\xef\xa0\x9f',
'CjkUnifiedIdeograph841e': '\xe8\x90\x9e',
'ModifierLetterMiddleGraveAccent': '\xcb\xb4',
'CircledNumberTwentySeven': '\xe3\x89\x97',
'CjkUnifiedIdeograph5153': '\xe5\x85\x93',
'CjkUnifiedIdeograph5152': '\xe5\x85\x92',
'CjkUnifiedIdeograph5151': '\xe5\x85\x91',
'CjkUnifiedIdeograph5150': '\xe5\x85\x90',
'CjkUnifiedIdeograph5157': '\xe5\x85\x97',
'ExtendedArabicindicDigitTwo': '\xdb\xb2',
'CjkUnifiedIdeograph5155': '\xe5\x85\x95',
'CjkUnifiedIdeograph5154': '\xe5\x85\x94',
'CjkUnifiedIdeograph5159': '\xe5\x85\x99',
'CjkUnifiedIdeograph5158': '\xe5\x85\x98',
'YiSyllableLop': '\xea\x87\x8a',
'CjkUnifiedIdeograph5ff9': '\xe5\xbf\xb9',
'CjkUnifiedIdeograph5ff8': '\xe5\xbf\xb8',
'CjkUnifiedIdeograph9641': '\xe9\x99\x81',
'CjkUnifiedIdeograph9640': '\xe9\x99\x80',
'CjkUnifiedIdeograph9647': '\xe9\x99\x87',
'CjkUnifiedIdeograph9646': '\xe9\x99\x86',
'CjkUnifiedIdeograph9645': '\xe9\x99\x85',
'CjkUnifiedIdeograph9644': '\xe9\x99\x84',
'CjkUnifiedIdeograph5ff1': '\xe5\xbf\xb1',
'CjkUnifiedIdeograph5ff0': '\xe5\xbf\xb0',
'CjkUnifiedIdeograph5ff3': '\xe5\xbf\xb3',
'CjkUnifiedIdeograph5ff2': '\xe5\xbf\xb2',
'CjkUnifiedIdeograph5ff5': '\xe5\xbf\xb5',
'CjkUnifiedIdeograph5ff4': '\xe5\xbf\xb4',
'CjkUnifiedIdeograph5ff7': '\xe5\xbf\xb7',
'CjkUnifiedIdeograph5ff6': '\xe5\xbf\xb6',
'HangulSyllableNyilm': '\xeb\x8a\xb6',
'EthiopicSyllableChwa': '\xe1\x8c\xaf',
'CjkUnifiedIdeograph91f0': '\xe9\x87\xb0',
'UnicodeCharacter0xf819': '\xef\xa0\x99',
'UnicodeCharacter0xf818': '\xef\xa0\x98',
'ArabicLigatureYehWithHamzaAboveWithUFinalForm': '\xef\xaf\xb1',
'HangulSyllableKweok': '\xed\x80\x94',
'UnicodeCharacter0xf811': '\xef\xa0\x91',
'UnicodeCharacter0x125f': '\xe1\x89\x9f',
'UnicodeCharacter0xf813': '\xef\xa0\x93',
'UnicodeCharacter0xf3bf': '\xef\x8e\xbf',
'UnicodeCharacter0xf815': '\xef\xa0\x95',
'UnicodeCharacter0xf814': '\xef\xa0\x94',
'UnicodeCharacter0xf817': '\xef\xa0\x97',
'UnicodeCharacter0xf816': '\xef\xa0\x96',
'UnicodeCharacter0x125e': '\xe1\x89\x9e',
'CjkUnifiedIdeograph91f1': '\xe9\x87\xb1',
'CjkUnifiedIdeograph5f5f': '\xe5\xbd\x9f',
'HangulSyllableGgwe': '\xea\xbf\xb0',
'MillSign': '\xe2\x82\xa5',
'HangulSyllableCobs': '\xec\xb4\x9a',
'HangulSyllableGgwa': '\xea\xbd\x88',
'HangulSyllableCeubs': '\xec\xb8\xb2',
'HangulSyllableGgwi': '\xeb\x80\x8c',
'CjkUnifiedIdeograph341a': '\xe3\x90\x9a',
'CjkUnifiedIdeograph341b': '\xe3\x90\x9b',
'CjkUnifiedIdeograph341c': '\xe3\x90\x9c',
'CjkUnifiedIdeograph341d': '\xe3\x90\x9d',
'CjkUnifiedIdeograph341e': '\xe3\x90\x9e',
'CjkUnifiedIdeograph341f': '\xe3\x90\x9f',
'OriyaLetterDa': '\xe0\xac\xa6',
'ArabicFootnoteMarker': '\xd8\x82',
'HangulSyllableTyaem': '\xed\x84\xa4',
'UnicodeCharacter0xe6be': '\xee\x9a\xbe',
'YiSyllableByr': '\xea\x80\xb7',
'YiSyllableByp': '\xea\x80\xb5',
'BengaliLetterDdha': '\xe0\xa6\xa2',
'YiSyllableByx': '\xea\x80\xb3',
'HangulSyllableKweon': '\xed\x80\x80',
'UnicodeCharacter0xf269': '\xef\x89\xa9',
'HangulSyllableTyaegg': '\xed\x84\x96',
'HangulSyllableSoenh': '\xec\x87\xa6',
'CjkUnifiedIdeograph5a53': '\xe5\xa9\x93',
'<=>': '\xe2\x87\x94',
'CjkUnifiedIdeograph5a52': '\xe5\xa9\x92',
'UnicodeCharacter0xdad5': '\xed\xab\x95',
'GurmukhiDigitSeven': '\xe0\xa9\xad',
'ArabicLigatureDadWithYehFinalForm': '\xef\xb4\xa4',
'CjkUnifiedIdeograph4d84': '\xe4\xb6\x84',
'UnicodeCharacter0xde8a': '\xed\xba\x8a',
'CjkUnifiedIdeograph3410': '\xe3\x90\x90',
'CjkUnifiedIdeograph3411': '\xe3\x90\x91',
'CjkUnifiedIdeograph3412': '\xe3\x90\x92',
'CjkUnifiedIdeograph3413': '\xe3\x90\x93',
'CjkUnifiedIdeograph3414': '\xe3\x90\x94',
'CjkUnifiedIdeograph3415': '\xe3\x90\x95',
'CjkUnifiedIdeograph3416': '\xe3\x90\x96',
'CjkUnifiedIdeograph3417': '\xe3\x90\x97',
'CjkUnifiedIdeograph3418': '\xe3\x90\x98',
'CjkUnifiedIdeograph3419': '\xe3\x90\x99',
'CjkUnifiedIdeograph7bb5': '\xe7\xae\xb5',
'CjkUnifiedIdeograph4d83': '\xe4\xb6\x83',
'MyanmarSignWesternPwoKarenTone3': '\xe1\x81\xab',
'CjkUnifiedIdeograph5a55': '\xe5\xa9\x95',
'UnicodeCharacter0xfadf': '\xef\xab\x9f',
'CjkUnifiedIdeograph5f5b': '\xe5\xbd\x9b',
'RightwardsHarpoonWithBarbDownToBar': '\xe2\xa5\x97',
'KatakanahiraganaVoicedSoundMark': '\xe3\x82\x9b',
'CjkUnifiedIdeograph5a54': '\xe5\xa9\x94',
'CjkUnifiedIdeograph7bb2': '\xe7\xae\xb2',
'CanadianSyllabicsWii': '\xe1\x90\x90',
'IdeographicTelegraphSymbolForDayTwo': '\xe3\x8f\xa1',
'HangulSyllableMuss': '\xeb\xad\x88',
'HangulSyllableTaelp': '\xed\x83\xaa',
'HangulSyllableDyuls': '\xeb\x93\x8c',
'ArabicLigatureAinWithJeemIsolatedForm': '\xef\xb0\xa9',
'UnicodeCharacter0xfadb': '\xef\xab\x9b',
'CjkUnifiedIdeograph8c1b': '\xe8\xb0\x9b',
'CjkUnifiedIdeograph8c1c': '\xe8\xb0\x9c',
'HangulSyllableJuc': '\xec\xa4\x93',
'CjkUnifiedIdeograph8c1a': '\xe8\xb0\x9a',
'CjkUnifiedIdeograph8c1f': '\xe8\xb0\x9f',
'HangulSyllableJud': '\xec\xa4\x83',
'CjkUnifiedIdeograph5f5c': '\xe5\xbd\x9c',
'CjkUnifiedIdeograph8c1e': '\xe8\xb0\x9e',
'HangulSyllableJuh': '\xec\xa4\x97',
'HangulSyllableJuk': '\xec\xa4\x94',
'IdeographicTelegraphSymbolForHourNineteen': '\xe3\x8d\xab',
'YiSyllableCuo': '\xea\x8a\xbb',
'HangulSyllableJul': '\xec\xa4\x84',
'HangulSyllableJun': '\xec\xa4\x80',
'YiSyllableCur': '\xea\x8b\x89',
'HangulSyllableJus': '\xec\xa4\x8f',
'DevanagariLetterZa': '\xe0\xa5\x9b',
'HangulSyllableJut': '\xec\xa4\x95',
'YiSyllableCut': '\xea\x8b\x84',
'UnicodeCharacter0xdc43': '\xed\xb1\x83',
'UnicodeCharacter0xdc42': '\xed\xb1\x82',
'UnicodeCharacter0xdc41': '\xed\xb1\x81',
'YiSyllableCux': '\xea\x8b\x85',
'SyriacLetterPe': '\xdc\xa6',
'UnicodeCharacter0xdc46': '\xed\xb1\x86',
'UnicodeCharacter0xdc45': '\xed\xb1\x85',
'UnicodeCharacter0xdc44': '\xed\xb1\x84',
'HangulSyllableBwiss': '\xeb\xb7\xa8',
'HangulSyllableDdweolg': '\xeb\x9b\x81',
'HangulSyllableTyael': '\xed\x84\x9c',
'HangulSyllableTweot': '\xed\x89\xa1',
'DevanagariVowelSignCandraE': '\xe0\xa5\x85',
'HangulSyllableRyebs': '\xeb\xa1\x92',
'DevanagariVowelSignCandraO': '\xe0\xa5\x89',
'CjkUnifiedIdeograph91f8': '\xe9\x87\xb8',
'HangulSyllableDyulg': '\xeb\x93\x89',
'PresentationFormForVerticalLeftParenthesis': '\xef\xb8\xb5',
'HangulSyllableWaess': '\xec\x99\xb0',
'UnicodeCharacter0xdc4c': '\xed\xb1\x8c',
'UnicodeCharacter0xdc4b': '\xed\xb1\x8b',
'UnicodeCharacter0xdc4a': '\xed\xb1\x8a',
'LatinSmallLigatureOe': '\xc5\x93',
'UnicodeCharacter0xdc4f': '\xed\xb1\x8f',
'HangulSyllableDdebs': '\xeb\x97\x8e',
'UnicodeCharacter0xdc4d': '\xed\xb1\x8d',
'CjkUnifiedIdeograph8c12': '\xe8\xb0\x92',
'CjkUnifiedIdeograph8c13': '\xe8\xb0\x93',
'CjkUnifiedIdeograph8c10': '\xe8\xb0\x90',
'CjkUnifiedIdeograph8c11': '\xe8\xb0\x91',
'LepchaLetterPa': '\xe1\xb0\x8e',
'CjkUnifiedIdeograph8c17': '\xe8\xb0\x97',
'CjkUnifiedIdeograph8c14': '\xe8\xb0\x94',
'CjkUnifiedIdeograph8c15': '\xe8\xb0\x95',
'CjkUnifiedIdeograph8c18': '\xe8\xb0\x98',
'RunicLetterIsazIsIssI': '\xe1\x9b\x81',
'CjkUnifiedIdeograph91f9': '\xe9\x87\xb9',
'CjkUnifiedIdeograph597e': '\xe5\xa5\xbe',
'CjkUnifiedIdeograph597d': '\xe5\xa5\xbd',
'CanadianSyllabicsWestcreeFwaa': '\xe1\x95\x9c',
'CjkUnifiedIdeograph597a': '\xe5\xa5\xba',
'HangulSyllableTaelb': '\xed\x83\xa7',
'CjkUnifiedIdeograph597c': '\xe5\xa5\xbc',
'CjkUnifiedIdeograph597b': '\xe5\xa5\xbb',
'UnicodeCharacter0xe9ab': '\xee\xa6\xab',
'BoxDrawingsHeavyDoubleDashVertical': '\xe2\x95\x8f',
'CjkUnifiedIdeograph35cb': '\xe3\x97\x8b',
'CjkUnifiedIdeograph35ca': '\xe3\x97\x8a',
'CjkUnifiedIdeograph35cf': '\xe3\x97\x8f',
'CjkUnifiedIdeograph35ce': '\xe3\x97\x8e',
'CjkUnifiedIdeograph35cd': '\xe3\x97\x8d',
'natE': '\xe2\x85\x87',
'HebrewLetterWideHe': '\xef\xac\xa3',
'HangulSyllableJjyags': '\xec\xa8\x97',
'UnicodeCharacter0xf58e': '\xef\x96\x8e',
'UnicodeCharacter0xe9ae': '\xee\xa6\xae',
'HangulSyllableTaelh': '\xed\x83\xab',
'VaiSyllableYa': '\xea\x95\xa9',
'EthiopicSyllableSsa': '\xe2\xb6\xa0',
'ArabicLetterDadIsolatedForm': '\xef\xba\xbd',
'CjkUnifiedIdeograph35c3': '\xe3\x97\x83',
'CjkUnifiedIdeograph35c2': '\xe3\x97\x82',
'CjkUnifiedIdeograph35c1': '\xe3\x97\x81',
'CjkUnifiedIdeograph35c0': '\xe3\x97\x80',
'CjkUnifiedIdeograph35c7': '\xe3\x97\x87',
'CjkUnifiedIdeograph35c6': '\xe3\x97\x86',
'CjkUnifiedIdeograph35c5': '\xe3\x97\x85',
'CjkUnifiedIdeograph35c4': '\xe3\x97\x84',
'ArabicLigatureDadWithKhahWithMeemInitialForm': '\xef\xb5\xb0',
'CjkUnifiedIdeograph35c9': '\xe3\x97\x89',
'CjkUnifiedIdeograph35c8': '\xe3\x97\x88',
'HangulSyllableHeols': '\xed\x97\x94',
'HangulSyllableHeolp': '\xed\x97\x96',
'CjkUnifiedIdeograph3f6f': '\xe3\xbd\xaf',
'CjkUnifiedIdeograph5975': '\xe5\xa5\xb5',
'CjkUnifiedIdeograph5974': '\xe5\xa5\xb4',
'CjkUnifiedIdeograph5977': '\xe5\xa5\xb7',
'CjkUnifiedIdeograph5976': '\xe5\xa5\xb6',
'CjkUnifiedIdeograph5971': '\xe5\xa5\xb1',
'CjkUnifiedIdeograph5970': '\xe5\xa5\xb0',
'CjkUnifiedIdeograph5973': '\xe5\xa5\xb3',
'CjkUnifiedIdeograph5972': '\xe5\xa5\xb2',
'HangulSyllableGgeog': '\xea\xba\xbd',
'HangulSyllableHeolg': '\xed\x97\x91',
'VaiSyllableYi': '\xea\x95\x82',
'CjkUnifiedIdeograph5979': '\xe5\xa5\xb9',
'VeryMuchGreaterthan': '\xe2\x8b\x99',
'UnicodeCharacter0xf4d5': '\xef\x93\x95',
'ParenthesizedIdeographSeven': '\xe3\x88\xa6',
'LatinSmallLigatureLongST': '\xef\xac\x85',
'UnicodeCharacter0xe6bc': '\xee\x9a\xbc',
'UnicodeCharacter0xed2a': '\xee\xb4\xaa',
'ring': '\xe2\x88\x98',
'BalineseSignRerekan': '\xe1\xac\xb4',
'UnicodeCharacter0xe8e3': '\xee\xa3\xa3',
'CjkUnifiedIdeograph36cd': '\xe3\x9b\x8d',
'CjkUnifiedIdeograph36ce': '\xe3\x9b\x8e',
'CjkUnifiedIdeograph36cf': '\xe3\x9b\x8f',
'UnicodeCharacter0xeee2': '\xee\xbb\xa2',
'UnicodeCharacter0xeee5': '\xee\xbb\xa5',
'CyrillicSmallLetterStraightUWithStroke': '\xd2\xb1',
'CjkUnifiedIdeograph36cb': '\xe3\x9b\x8b',
'CjkUnifiedIdeograph36cc': '\xe3\x9b\x8c',
'HangulSyllableSwing': '\xec\x8a\x81',
'HangulSyllableSsyus': '\xec\x93\xa7',
'HangulSyllableSsyup': '\xec\x93\xae',
'CjkUnifiedIdeograph5a5a': '\xe5\xa9\x9a',
'HangulSyllableSsyut': '\xec\x93\xad',
'HangulSyllableSsyuj': '\xec\x93\xaa',
'HangulSyllableSsyuk': '\xec\x93\xac',
'HangulSyllableSsyuh': '\xec\x93\xaf',
'HangulSyllableDdyuk': '\xeb\x9c\xa4',
'HangulSyllableSsyun': '\xec\x93\x98',
'HangulSyllableSsyul': '\xec\x93\x9c',
'HangulSyllableSsyum': '\xec\x93\xa4',
'HangulSyllableSsyub': '\xec\x93\xa5',
'HangulSyllableDdyul': '\xeb\x9c\x94',
'SinhalaLetterMahaapraanaGayanna': '\xe0\xb6\x9d',
'HangulSyllableSsyug': '\xec\x93\x95',
'HangulSyllableSsyud': '\xec\x93\x9b',
'CjkUnifiedIdeograph5a5f': '\xe5\xa9\x9f',
'TagalogLetterNa': '\xe1\x9c\x88',
'CjkUnifiedIdeograph97a9': '\xe9\x9e\xa9',
'CjkUnifiedIdeograph5a5d': '\xe5\xa9\x9d',
'LatinSmallLetterYWithHook': '\xc6\xb4',
'MyanmarSignRumaiPalaungTone5': '\xe1\x82\x8f',
'SinhalaLetterAlpapraanaDayanna': '\xe0\xb6\xaf',
'HangulSyllableDdyud': '\xeb\x9c\x93',
'HangulSyllableKyalp': '\xec\xba\xba',
'MyanmarLetterNna': '\xe1\x80\x8f',
'CjkUnifiedIdeograph36c4': '\xe3\x9b\x84',
'CjkUnifiedIdeograph36c5': '\xe3\x9b\x85',
'CjkUnifiedIdeograph36c6': '\xe3\x9b\x86',
'CjkUnifiedIdeograph36c7': '\xe3\x9b\x87',
'CjkUnifiedIdeograph36c0': '\xe3\x9b\x80',
'CjkUnifiedIdeograph36c1': '\xe3\x9b\x81',
'CjkUnifiedIdeograph36c2': '\xe3\x9b\x82',
'CjkUnifiedIdeograph36c3': '\xe3\x9b\x83',
'SinhalaLetterAmbaBayanna': '\xe0\xb6\xb9',
'CjkUnifiedIdeograph36c8': '\xe3\x9b\x88',
'CjkUnifiedIdeograph36c9': '\xe3\x9b\x89',
'ColonEquals': '\xe2\x89\x94',
'CjkUnifiedIdeograph9e67': '\xe9\xb9\xa7',
'HangulSyllableSseogs': '\xec\x8d\xab',
'CyrillicSmallLetterZeWithDescender': '\xd2\x99',
'swarrow': '\xe2\x86\x99',
'HangulSyllableRwaess': '\xeb\xa2\xa8',
'HangulSyllableJjoenh': '\xec\xac\x96',
'HangulSyllableJjoenj': '\xec\xac\x95',
'TibetanDigitTwo': '\xe0\xbc\xa2',
'HangulSyllableDdyup': '\xeb\x9c\xa6',
'ReturnSymbol': '\xe2\x8f\x8e',
'HangulSyllableTyinj': '\xed\x8b\x99',
'CjkUnifiedIdeograph727e': '\xe7\x89\xbe',
'ArabicLetterDotlessFeh': '\xda\xa1',
'MyanmarShanDigitSix': '\xe1\x82\x96',
'LatinSmallLetterAWithStroke': '\xe2\xb1\xa5',
'UnicodeCharacter0xdad6': '\xed\xab\x96',
'HangulSyllableNwep': '\xeb\x89\x96',
'DevanagariLetterJja': '\xe0\xa5\xbc',
'HexagramForWorkOnTheDecayed': '\xe4\xb7\x91',
'CjkCompatibilityIdeographf93d': '\xef\xa4\xbd',
'UnicodeCharacter0xa6a3': '\xea\x9a\xa3',
'YinYang': '\xe2\x98\xaf',
'UnicodeCharacter0xeee1': '\xee\xbb\xa1',
'ArabicLetterEIsolatedForm': '\xef\xaf\xa4',
'CjkUnifiedIdeograph95f3': '\xe9\x97\xb3',
'YiSyllableNyop': '\xea\x91\x98',
'ParenthesizedHangulTikeut': '\xe3\x88\x82',
'UnicodeCharacter0xed28': '\xee\xb4\xa8',
'UnicodeCharacter0xecfe': '\xee\xb3\xbe',
'UnicodeCharacter0xa6a0': '\xea\x9a\xa0',
'UnicodeCharacter0xeb1c': '\xee\xac\x9c',
'HangulSyllableGoek': '\xea\xb5\x8c',
'CjkUnifiedIdeograph7daf': '\xe7\xb6\xaf',
'HotSprings': '\xe2\x99\xa8',
'YiSyllableNyot': '\xea\x91\x95',
'CombiningGraveAccent': '\xcc\x80',
'UnicodeCharacter0xeb1f': '\xee\xac\x9f',
'RightwardsHarpoonWithBarbUpAboveLongDash': '\xe2\xa5\xac',
'CircledIdeographSun': '\xe3\x8a\x90',
'SundaneseLetterCa': '\xe1\xae\x8e',
'UnicodeCharacter0xeb1e': '\xee\xac\x9e',
'CjkUnifiedIdeograph3dee': '\xe3\xb7\xae',
'UnicodeCharacter0x93a': '\xe0\xa4\xba',
'UnicodeCharacter0x26e9': '\xe2\x9b\xa9',
'UnicodeCharacter0x26e8': '\xe2\x9b\xa8',
'HangulSyllableJweobs': '\xec\xa4\xaa',
'UnicodeCharacter0x26e3': '\xe2\x9b\xa3',
'UnicodeCharacter0x26e2': '\xe2\x9b\xa2',
'UnicodeCharacter0x26e1': '\xe2\x9b\xa1',
'UnicodeCharacter0x26e0': '\xe2\x9b\xa0',
'UnicodeCharacter0x26e7': '\xe2\x9b\xa7',
'UnicodeCharacter0x26e6': '\xe2\x9b\xa6',
'UnicodeCharacter0x26e5': '\xe2\x9b\xa5',
'UnicodeCharacter0x26e4': '\xe2\x9b\xa4',
'TamilLetterLla': '\xe0\xae\xb3',
'CjkUnifiedIdeograph6319': '\xe6\x8c\x99',
'LatinCapitalLetterYWithStroke': '\xc9\x8e',
'CjkUnifiedIdeograph6310': '\xe6\x8c\x90',
'CjkUnifiedIdeograph6311': '\xe6\x8c\x91',
'CjkUnifiedIdeograph6312': '\xe6\x8c\x92',
'CjkUnifiedIdeograph6313': '\xe6\x8c\x93',
'CjkUnifiedIdeograph6314': '\xe6\x8c\x94',
'CjkUnifiedIdeograph6315': '\xe6\x8c\x95',
'CjkUnifiedIdeograph6316': '\xe6\x8c\x96',
'CjkUnifiedIdeograph6317': '\xe6\x8c\x97',
'CyrillicCapitalLetterEs': '\xd0\xa1',
'UpwardsBlackArrow': '\xe2\xac\x86',
'UnicodeCharacter0xdc92': '\xed\xb2\x92',
'MalayalamLetterTta': '\xe0\xb4\x9f',
'ArabicLigatureYehWithMeemFinalForm': '\xef\xb2\x93',
'LatinSmallLetterOWithMacronAndAcute': '\xe1\xb9\x93',
'HangulSyllableMyobs': '\xeb\xac\xaa',
'HangulSyllableMaeng': '\xeb\xa7\xb9',
'VaiSyllableHoo': '\xea\x95\xb3',
'VaiSyllableHon': '\xea\x96\xbe',
'HangulSyllableRit': '\xeb\xa7\x85',
'CjkUnifiedIdeograph631a': '\xe6\x8c\x9a',
'CjkUnifiedIdeograph631b': '\xe6\x8c\x9b',
'CjkUnifiedIdeograph631c': '\xe6\x8c\x9c',
'CjkUnifiedIdeograph631d': '\xe6\x8c\x9d',
'LatinSmallLetterEWithDotBelow': '\xe1\xba\xb9',
'CjkUnifiedIdeograph631f': '\xe6\x8c\x9f',
'MongolianLetterAliGaliNga': '\xe1\xa2\x8a',
'UnicodeCharacter0xa7f9': '\xea\x9f\xb9',
'UnicodeCharacter0x26ec': '\xe2\x9b\xac',
'UnicodeCharacter0x26eb': '\xe2\x9b\xab',
'UnicodeCharacter0x26ea': '\xe2\x9b\xaa',
'UnicodeCharacter0x26ef': '\xe2\x9b\xaf',
'UnicodeCharacter0x26ee': '\xe2\x9b\xae',
'UnicodeCharacter0x26ed': '\xe2\x9b\xad',
'CjkUnifiedIdeograph5eb2': '\xe5\xba\xb2',
'CjkUnifiedIdeograph5eb3': '\xe5\xba\xb3',
'CjkUnifiedIdeograph5eb0': '\xe5\xba\xb0',
'CjkUnifiedIdeograph5eb1': '\xe5\xba\xb1',
'ImageOfOrApproximatelyEqualTo': '\xe2\x89\x93',
'CjkUnifiedIdeograph5eb7': '\xe5\xba\xb7',
'CjkUnifiedIdeograph42f8': '\xe4\x8b\xb8',
'CjkUnifiedIdeograph42f9': '\xe4\x8b\xb9',
'CjkUnifiedIdeograph42f6': '\xe4\x8b\xb6',
'CjkUnifiedIdeograph42f7': '\xe4\x8b\xb7',
'CjkUnifiedIdeograph42f4': '\xe4\x8b\xb4',
'CjkUnifiedIdeograph42f5': '\xe4\x8b\xb5',
'CjkUnifiedIdeograph42f2': '\xe4\x8b\xb2',
'CjkUnifiedIdeograph42f3': '\xe4\x8b\xb3',
'CjkUnifiedIdeograph42f0': '\xe4\x8b\xb0',
'CjkUnifiedIdeograph42f1': '\xe4\x8b\xb1',
'UnicodeCharacter0xa7f2': '\xea\x9f\xb2',
'UnicodeCharacter0xa7f3': '\xea\x9f\xb3',
'HangulSyllableNwils': '\xeb\x89\xa4',
'WhiteCircleWithDownArrow': '\xe2\xa7\xac',
'HangulSyllableGgiss': '\xeb\x82\x90',
'UnicodeCharacter0xa7f0': '\xea\x9f\xb0',
'CjkUnifiedIdeograph3757': '\xe3\x9d\x97',
'CjkUnifiedIdeograph3756': '\xe3\x9d\x96',
'CjkUnifiedIdeograph3755': '\xe3\x9d\x95',
'CjkUnifiedIdeograph3754': '\xe3\x9d\x94',
'CjkUnifiedIdeograph3753': '\xe3\x9d\x93',
'CjkUnifiedIdeograph3752': '\xe3\x9d\x92',
'CjkUnifiedIdeograph3751': '\xe3\x9d\x91',
'CjkUnifiedIdeograph3750': '\xe3\x9d\x90',
'UnicodeCharacter0xe97a': '\xee\xa5\xba',
'UnicodeCharacter0xa7f6': '\xea\x9f\xb6',
'CjkUnifiedIdeograph3759': '\xe3\x9d\x99',
'CjkUnifiedIdeograph3758': '\xe3\x9d\x98',
'CjkUnifiedIdeograph7047': '\xe7\x81\x87',
'CjkUnifiedIdeograph7046': '\xe7\x81\x86',
'CjkUnifiedIdeograph7045': '\xe7\x81\x85',
'CjkUnifiedIdeograph7044': '\xe7\x81\x84',
'CjkUnifiedIdeograph7043': '\xe7\x81\x83',
'CjkUnifiedIdeograph7042': '\xe7\x81\x82',
'CjkUnifiedIdeograph7041': '\xe7\x81\x81',
'CjkUnifiedIdeograph7040': '\xe7\x81\x80',
'MyanmarVowelSignMonO': '\xe1\x80\xb4',
'UnicodeCharacter0xa7f5': '\xea\x9f\xb5',
'CircledNumberFortySix': '\xe3\x8a\xbb',
'CjkUnifiedIdeograph7048': '\xe7\x81\x88',
'HangulSyllableDobs': '\xeb\x8f\x96',
'CjkRadicalOld': '\xe2\xba\xb9',
'HangulSyllableByobs': '\xeb\xb5\xb6',
'UnicodeCharacter0xf5ca': '\xef\x97\x8a',
'HangulSyllableRulm': '\xeb\xa3\xb2',
'CjkUnifiedIdeograph5ebb': '\xe5\xba\xbb',
'CjkUnifiedIdeograph5ebc': '\xe5\xba\xbc',
'CjkUnifiedIdeograph5eba': '\xe5\xba\xba',
'BraillePatternDots58': '\xe2\xa2\x90',
'CjkUnifiedIdeograph5ebd': '\xe5\xba\xbd',
'CjkUnifiedIdeograph5ebe': '\xe5\xba\xbe',
'CjkUnifiedIdeograph42ff': '\xe4\x8b\xbf',
'UnicodeCharacter0xdc93': '\xed\xb2\x93',
'CjkUnifiedIdeograph42fd': '\xe4\x8b\xbd',
'CjkUnifiedIdeograph42fe': '\xe4\x8b\xbe',
'CjkUnifiedIdeograph42fb': '\xe4\x8b\xbb',
'CjkUnifiedIdeograph42fc': '\xe4\x8b\xbc',
'CjkUnifiedIdeograph42fa': '\xe4\x8b\xba',
'UnicodeCharacter0xdf60': '\xed\xbd\xa0',
'CjkUnifiedIdeograph704f': '\xe7\x81\x8f',
'CjkUnifiedIdeograph704e': '\xe7\x81\x8e',
'CjkUnifiedIdeograph704d': '\xe7\x81\x8d',
'LatinSmallLetterHWithLineBelow': '\xe1\xba\x96',
'CjkUnifiedIdeograph704b': '\xe7\x81\x8b',
'CjkUnifiedIdeograph704a': '\xe7\x81\x8a',
'CjkUnifiedIdeograph4814': '\xe4\xa0\x94',
'UnicodeCharacter0xdf68': '\xed\xbd\xa8',
'GlagoliticSmallLetterPokoji': '\xe2\xb1\x82',
'CjkUnifiedIdeograph83dc': '\xe8\x8f\x9c',
'GurmukhiSignYakash': '\xe0\xa9\xb5',
'CjkUnifiedIdeograph375f': '\xe3\x9d\x9f',
'CjkUnifiedIdeograph375e': '\xe3\x9d\x9e',
'CjkUnifiedIdeograph375d': '\xe3\x9d\x9d',
'CjkUnifiedIdeograph375c': '\xe3\x9d\x9c',
'CjkUnifiedIdeograph375b': '\xe3\x9d\x9b',
'CjkUnifiedIdeograph375a': '\xe3\x9d\x9a',
'HangulSyllablePeo': '\xed\x8d\xbc',
'CjkUnifiedIdeograph83de': '\xe8\x8f\x9e',
'HangulSyllablePem': '\xed\x8e\xa8',
'HangulSyllablePel': '\xed\x8e\xa0',
'HangulSyllablePek': '\xed\x8e\xb0',
'HangulSyllableRis': '\xeb\xa6\xbf',
'CjkUnifiedIdeograph83dd': '\xe8\x8f\x9d',
'UnicodeCharacter0xeb12': '\xee\xac\x92',
'CjkUnifiedIdeograph62c9': '\xe6\x8b\x89',
'CjkUnifiedIdeograph62c8': '\xe6\x8b\x88',
'CjkUnifiedIdeograph95f9': '\xe9\x97\xb9',
'CjkUnifiedIdeograph4813': '\xe4\xa0\x93',
'ArabicLigatureKafWithMeemMedialForm': '\xef\xb3\xac',
'UnicodeCharacter0xeb13': '\xee\xac\x93',
'CjkUnifiedIdeograph62c3': '\xe6\x8b\x83',
'CjkUnifiedIdeograph62c2': '\xe6\x8b\x82',
'CjkUnifiedIdeograph62c1': '\xe6\x8b\x81',
'EthiopicSyllableGgo': '\xe1\x8c\x9e',
'CjkUnifiedIdeograph62c7': '\xe6\x8b\x87',
'CjkUnifiedIdeograph62c6': '\xe6\x8b\x86',
'CjkUnifiedIdeograph62c5': '\xe6\x8b\x85',
'GreekSmallLetterEtaWithPerispomeniAndYpogegrammeni': '\xe1\xbf\x87',
'UnicodeCharacter0xeb11': '\xee\xac\x91',
'UnicodeCharacter0x1c88': '\xe1\xb2\x88',
'UnicodeCharacter0x1c89': '\xe1\xb2\x89',
'HangulSyllableMung': '\xeb\xad\x89',
'CjkUnifiedIdeograph754c': '\xe7\x95\x8c',
'UnicodeCharacter0x1c84': '\xe1\xb2\x84',
'UnicodeCharacter0x1c85': '\xe1\xb2\x85',
'UnicodeCharacter0x1c86': '\xe1\xb2\x86',
'UnicodeCharacter0x1c87': '\xe1\xb2\x87',
'UnicodeCharacter0x1c80': '\xe1\xb2\x80',
'UnicodeCharacter0x1c81': '\xe1\xb2\x81',
'UnicodeCharacter0x1c82': '\xe1\xb2\x82',
'UnicodeCharacter0x1c83': '\xe1\xb2\x83',
'HangulSyllableAd': '\xec\x95\x8b',
'HangulSyllableAe': '\xec\x95\xa0',
'HangulSyllableAg': '\xec\x95\x85',
'UnicodeCharacter0xeb14': '\xee\xac\x94',
'HangulSyllableAb': '\xec\x95\x95',
'HangulSyllableAc': '\xec\x95\x9b',
'HangulSyllableAl': '\xec\x95\x8c',
'CjkUnifiedIdeograph754f': '\xe7\x95\x8f',
'HangulSyllableAn': '\xec\x95\x88',
'UnicodeCharacter0xeb15': '\xee\xac\x95',
'HangulSyllableAh': '\xec\x95\x9f',
'HangulSyllableAj': '\xec\x95\x9a',
'CjkUnifiedIdeograph4818': '\xe4\xa0\x98',
'HangulSyllableAt': '\xec\x95\x9d',
'CjkUnifiedIdeograph3968': '\xe3\xa5\xa8',
'HangulSyllableAp': '\xec\x95\x9e',
'CjkUnifiedIdeograph3de5': '\xe3\xb7\xa5',
'WhiteHourglass': '\xe2\xa7\x96',
'HangulSyllableGgwags': '\xea\xbd\x8b',
'UnicodeCharacter0xeb18': '\xee\xac\x98',
'LatinCapitalLetterRWithInvertedBreve': '\xc8\x92',
'UnicodeCharacter0xeb19': '\xee\xac\x99',
'UnicodeCharacter0x1c8d': '\xe1\xb2\x8d',
'UnicodeCharacter0x1c8e': '\xe1\xb2\x8e',
'UnicodeCharacter0x1c8f': '\xe1\xb2\x8f',
'SmallRomanNumeralTen': '\xe2\x85\xb9',
'UnicodeCharacter0x1c8a': '\xe1\xb2\x8a',
'UnicodeCharacter0x1c8b': '\xe1\xb2\x8b',
'UnicodeCharacter0x1c8c': '\xe1\xb2\x8c',
'UnicodeCharacter0xe8f4': '\xee\xa3\xb4',
'CjkUnifiedIdeograph62cc': '\xe6\x8b\x8c',
'CjkUnifiedIdeograph62cb': '\xe6\x8b\x8b',
'CjkUnifiedIdeograph62ca': '\xe6\x8b\x8a',
'CjkUnifiedIdeograph62cf': '\xe6\x8b\x8f',
'CjkUnifiedIdeograph62ce': '\xe6\x8b\x8e',
'CjkRadicalDivination': '\xe2\xba\x8a',
'CjkUnifiedIdeograph7ae9': '\xe7\xab\xa9',
'GreaterthanOrApproximate': '\xe2\xaa\x86',
'GeorgianLetterOn': '\xe1\x83\x9d',
'CjkUnifiedIdeograph7273': '\xe7\x89\xb3',
'CjkUnifiedIdeograph7c13': '\xe7\xb0\x93',
'CjkUnifiedIdeograph7c12': '\xe7\xb0\x92',
'CjkUnifiedIdeograph7c11': '\xe7\xb0\x91',
'CjkUnifiedIdeograph7c10': '\xe7\xb0\x90',
'CjkUnifiedIdeograph7c17': '\xe7\xb0\x97',
'CjkUnifiedIdeograph7c16': '\xe7\xb0\x96',
'CjkUnifiedIdeograph7c15': '\xe7\xb0\x95',
'CjkUnifiedIdeograph7c14': '\xe7\xb0\x94',
'HangulSyllableGgebs': '\xea\xbb\xaa',
'CjkUnifiedIdeograph7c19': '\xe7\xb0\x99',
'CjkUnifiedIdeograph7277': '\xe7\x89\xb7',
'HangulSyllableRyalm': '\xeb\x9e\xbe',
'HangulSyllableRyals': '\xeb\x9f\x80',
'HangulSyllableRyalp': '\xeb\x9f\x82',
'HangulSyllableRin': '\xeb\xa6\xb0',
'CjkUnifiedIdeograph61c8': '\xe6\x87\x88',
'CjkUnifiedIdeograph61c9': '\xe6\x87\x89',
'HangulSyllableRyalt': '\xeb\x9f\x81',
'CjkUnifiedIdeograph61c4': '\xe6\x87\x84',
'CjkUnifiedIdeograph61c5': '\xe6\x87\x85',
'CjkUnifiedIdeograph61c6': '\xe6\x87\x86',
'CjkUnifiedIdeograph61c7': '\xe6\x87\x87',
'CjkUnifiedIdeograph61c0': '\xe6\x87\x80',
'CjkUnifiedIdeograph61c1': '\xe6\x87\x81',
'CjkUnifiedIdeograph61c2': '\xe6\x87\x82',
'CjkUnifiedIdeograph61c3': '\xe6\x87\x83',
'CyrillicSmallLetterKaWithVerticalStroke': '\xd2\x9d',
'CjkCompatibilityIdeographfaaf': '\xef\xaa\xaf',
'CjkCompatibilityIdeographfaad': '\xef\xaa\xad',
'CjkCompatibilityIdeographfaae': '\xef\xaa\xae',
'CjkCompatibilityIdeographfaab': '\xef\xaa\xab',
'CjkCompatibilityIdeographfaac': '\xef\xaa\xac',
'CanadianSyllabicsNunavikHoo': '\xe1\x95\xb8',
'CjkCompatibilityIdeographfaaa': '\xef\xaa\xaa',
'HangulSyllableHyulg': '\xed\x9c\xbd',
'ThaanaAbafili': '\xde\xa6',
'UnicodeCharacter0xf5fd': '\xef\x97\xbd',
'HangulSyllableDyogs': '\xeb\x90\xb7',
'CjkUnifiedIdeograph61cd': '\xe6\x87\x8d',
'CjkUnifiedIdeograph61ce': '\xe6\x87\x8e',
'CyrillicCapitalLetterBarredOWithDiaeresis': '\xd3\xaa',
'CjkUnifiedIdeograph61ca': '\xe6\x87\x8a',
'CjkUnifiedIdeograph61cb': '\xe6\x87\x8b',
'CjkUnifiedIdeograph61cc': '\xe6\x87\x8c',
'CjkUnifiedIdeograph7c1c': '\xe7\xb0\x9c',
'CjkUnifiedIdeograph7c1b': '\xe7\xb0\x9b',
'CjkUnifiedIdeograph7c1a': '\xe7\xb0\x9a',
'EthiopicSyllableLaa': '\xe1\x88\x8b',
'CjkUnifiedIdeograph7c1f': '\xe7\xb0\x9f',
'CjkUnifiedIdeograph7c1e': '\xe7\xb0\x9e',
'CjkUnifiedIdeograph7c1d': '\xe7\xb0\x9d',
'HangulSyllableJeolh': '\xec\xa0\x8f',
'IdeographicAnnotationFourMark': '\xe3\x86\x95',
'UnicodeCharacter0xf5c3': '\xef\x97\x83',
'OghamLetterFearn': '\xe1\x9a\x83',
'ArabicLetterYehInitialForm': '\xef\xbb\xb3',
'HangulSyllableDdoes': '\xeb\x99\xb7',
'HangulSyllableDdoep': '\xeb\x99\xbe',
'HangulSyllableMaes': '\xeb\xa7\xb7',
'ArabicLetterFehWithDotMovedBelow': '\xda\xa2',
'HangulSyllableDdoej': '\xeb\x99\xba',
'HangulSyllableDdoek': '\xeb\x99\xbc',
'HangulSyllableDdoeh': '\xeb\x99\xbf',
'HangulSyllableDdoen': '\xeb\x99\xa8',
'LatinCapitalLetterVy': '\xea\x9d\xa0',
'HangulSyllableDdoel': '\xeb\x99\xac',
'HangulSyllableDdoem': '\xeb\x99\xb4',
'HangulSyllableDdoeb': '\xeb\x99\xb5',
'HangulSyllableDdoec': '\xeb\x99\xbb',
'CjkCompatibilityIdeographfaa4': '\xef\xaa\xa4',
'CjkCompatibilityIdeographfaa5': '\xef\xaa\xa5',
'CjkCompatibilityIdeographfaa2': '\xef\xaa\xa2',
'HangulSyllableDdoeg': '\xeb\x99\xa5',
'HangulSyllableDdoed': '\xeb\x99\xab',
'CjkCompatibilityIdeographfaa1': '\xef\xaa\xa1',
'CjkUnifiedIdeograph83d1': '\xe8\x8f\x91',
'UnicodeCharacter0xdc95': '\xed\xb2\x95',
'CjkUnifiedIdeograph83d0': '\xe8\x8f\x90',
'EthiopicSyllableGlottalAa': '\xe1\x8a\xa3',
'CopticSymbolKai': '\xe2\xb3\xa4',
'CjkUnifiedIdeograph7549': '\xe7\x95\x89',
'CjkUnifiedIdeograph83d5': '\xe8\x8f\x95',
'UnicodeCharacter0xf8e0': '\xef\xa3\xa0',
'HangulSyllablePet': '\xed\x8e\xb1',
'CjkUnifiedIdeograph83d7': '\xe8\x8f\x97',
'HangulSyllablePwaelt': '\xed\x90\xb1',
'HexagramForTheFamily': '\xe4\xb7\xa4',
'HangulSyllablePwaels': '\xed\x90\xb0',
'HangulSyllablePwaelp': '\xed\x90\xb2',
'ArabicLetterTchehehIsolatedForm': '\xef\xad\xbe',
'CjkUnifiedIdeograph83d9': '\xe8\x8f\x99',
'HangulSyllablePwaelm': '\xed\x90\xae',
'HangulSyllablePwaelh': '\xed\x90\xb3',
'CjkUnifiedIdeograph83d8': '\xe8\x8f\x98',
'HangulSyllablePwaelg': '\xed\x90\xad',
'LatinSmallLetterUWithHornAndDotBelow': '\xe1\xbb\xb1',
'UnicodeCharacter0xae5': '\xe0\xab\xa5',
'UnicodeCharacter0xae4': '\xe0\xab\xa4',
'KhmerIndependentVowelLy': '\xe1\x9e\xad',
'HangulSyllablePes': '\xed\x8e\xab',
'HangulSyllableJulm': '\xec\xa4\x86',
'CjkUnifiedIdeograph7546': '\xe7\x95\x86',
'CombiningBreve': '\xcc\x86',
'UnicodeCharacter0xf8e8': '\xef\xa3\xa8',
'CjkUnifiedIdeograph6b1f': '\xe6\xac\x9f',
'HangulSyllableJulh': '\xec\xa4\x8b',
'TamilLetterVa': '\xe0\xae\xb5',
'CjkUnifiedIdeograph6b1a': '\xe6\xac\x9a',
'CjkUnifiedIdeograph91ff': '\xe9\x87\xbf',
'HangulSyllableTess': '\xed\x85\xa0',
'HebrewLetterNun': '\xd7\xa0',
'CjkUnifiedIdeograph6b1b': '\xe6\xac\x9b',
'KangxiRadicalTurban': '\xe2\xbc\xb1',
'ThaiCharacterMaiEk': '\xe0\xb9\x88',
'GujaratiLetterUu': '\xe0\xaa\x8a',
'NorthWestArrowAndNorthEastArrow': '\xe2\xa4\xa7',
'YiSyllableNrex': '\xea\x8e\x9c',
'HangulSyllableNwaes': '\xeb\x87\x83',
'ArabicLetterTchehWithDotAbove': '\xda\xbf',
'UnicodeCharacter0xdd59': '\xed\xb5\x99',
'YiSyllableNrep': '\xea\x8e\x9e',
'UnicodeCharacter0xa95a': '\xea\xa5\x9a',
'UnicodeCharacter0xa95c': '\xea\xa5\x9c',
'UnicodeCharacter0xa95b': '\xea\xa5\x9b',
'UnicodeCharacter0xa95e': '\xea\xa5\x9e',
'UnicodeCharacter0xa95d': '\xea\xa5\x9d',
'ArabicLetterSeenIsolatedForm': '\xef\xba\xb1',
'PresentationFormForVerticalRightSquareBracket': '\xef\xb9\x88',
'CjkUnifiedIdeograph905b': '\xe9\x81\x9b',
'HebrewAccentMerkha': '\xd6\xa5',
'ArabicLigatureBehWithMeemIsolatedForm': '\xef\xb0\x88',
'HangulSyllableNwaem': '\xeb\x87\x80',
'HangulSyllableRyih': '\xeb\xa6\xab',
'UnicodeCharacter0xdd53': '\xed\xb5\x93',
'CjkUnifiedIdeograph39e9': '\xe3\xa7\xa9',
'CjkUnifiedIdeograph39e8': '\xe3\xa7\xa8',
'CjkUnifiedIdeograph91fe': '\xe9\x87\xbe',
'UnicodeCharacter0xdd52': '\xed\xb5\x92',
'CjkUnifiedIdeograph39e5': '\xe3\xa7\xa5',
'CjkUnifiedIdeograph39e4': '\xe3\xa7\xa4',
'CjkUnifiedIdeograph39e7': '\xe3\xa7\xa7',
'CjkUnifiedIdeograph39e6': '\xe3\xa7\xa6',
'CjkUnifiedIdeograph39e1': '\xe3\xa7\xa1',
'CjkUnifiedIdeograph39e0': '\xe3\xa7\xa0',
'CjkUnifiedIdeograph39e3': '\xe3\xa7\xa3',
'CjkUnifiedIdeograph39e2': '\xe3\xa7\xa2',
'EthiopicSyllableNywa': '\xe1\x8a\x9f',
'HangulSyllableKyulp': '\xed\x81\x9e',
'UnicodeCharacter0xa955': '\xea\xa5\x95',
'UnicodeCharacter0xa954': '\xea\xa5\x94',
'UnicodeCharacter0xa957': '\xea\xa5\x97',
'UnicodeCharacter0xa956': '\xea\xa5\x96',
'HyphenBullet': '\xe2\x81\x83',
'UnicodeCharacter0xa958': '\xea\xa5\x98',
'PhagspaLetterQa': '\xea\xa1\xa2',
'HangulSyllableKyulb': '\xed\x81\x9b',
'ChamLetterSsa': '\xea\xa8\xa6',
'HangulSyllableKyulg': '\xed\x81\x99',
'UnicodeCharacter0xdd55': '\xed\xb5\x95',
'HangulSyllableKyulh': '\xed\x81\x9f',
'HangulSyllableKyulm': '\xed\x81\x9a',
'UnicodeCharacter0xe817': '\xee\xa0\x97',
'KhmerSignCoeng': '\xe1\x9f\x92',
'TeluguFractionDigitThreeForEvenPowersOfFour': '\xe0\xb1\xbe',
'CjkUnifiedIdeograph39ee': '\xe3\xa7\xae',
'CjkUnifiedIdeograph39ed': '\xe3\xa7\xad',
'CjkUnifiedIdeograph39ef': '\xe3\xa7\xaf',
'CjkUnifiedIdeograph39ea': '\xe3\xa7\xaa',
'CjkUnifiedIdeograph39ec': '\xe3\xa7\xac',
'CjkUnifiedIdeograph39eb': '\xe3\xa7\xab',
'ParenthesizedIdeographStudy': '\xe3\x88\xbb',
'UnicodeCharacter0xe812': '\xee\xa0\x92',
'CjkCompatibilityIdeographf9c8': '\xef\xa7\x88',
'CjkRadicalCow': '\xe2\xba\xa7',
'HangulSyllableJwibs': '\xec\xa5\xa2',
'CyrillicCapitalLetterZhwe': '\xea\x9a\x84',
'UnicodeCharacter0xd9f8': '\xed\xa7\xb8',
'HangulSyllableNweolm': '\xeb\x88\xaa',
'HangulSyllableNweolb': '\xeb\x88\xab',
'BraillePatternDots478': '\xe2\xa3\x88',
'UnicodeCharacter0xd9f1': '\xed\xa7\xb1',
'ArmenianCapitalLetterTo': '\xd4\xb9',
'UnicodeCharacter0xf70': '\xe0\xbd\xb0',
'HangulSyllableNweolg': '\xeb\x88\xa9',
'UnicodeCharacter0xd9f5': '\xed\xa7\xb5',
'UnicodeCharacter0xd9f4': '\xed\xa7\xb4',
'UnicodeCharacter0xe8b': '\xe0\xba\x8b',
'UnicodeCharacter0xe8c': '\xe0\xba\x8c',
'HangulSyllableNweols': '\xeb\x88\xac',
'HangulSyllableNweolp': '\xeb\x88\xae',
'HangulSyllableNweolt': '\xeb\x88\xad',
'UnicodeCharacter0xf604': '\xef\x98\x84',
'UnicodeCharacter0xf605': '\xef\x98\x85',
'UnicodeCharacter0xf606': '\xef\x98\x86',
'UnicodeCharacter0xf607': '\xef\x98\x87',
'UnicodeCharacter0xf600': '\xef\x98\x80',
'UnicodeCharacter0xf601': '\xef\x98\x81',
'UnicodeCharacter0xf602': '\xef\x98\x82',
'UnicodeCharacter0xf603': '\xef\x98\x83',
'HangulSyllableDyogg': '\xeb\x90\xb6',
'UnicodeCharacter0xe8f': '\xe0\xba\x8f',
'BlackHeartSuit': '\xe2\x99\xa5',
'UnicodeCharacter0xeefc': '\xee\xbb\xbc',
'UnicodeCharacter0xf609': '\xef\x98\x89',
'LatinSmallLetterAWithBreve': '\xc4\x83',
'CjkUnifiedIdeograph5b15': '\xe5\xac\x95',
'CjkUnifiedIdeograph5b16': '\xe5\xac\x96',
'CjkUnifiedIdeograph5b17': '\xe5\xac\x97',
'CjkUnifiedIdeograph5b10': '\xe5\xac\x90',
'CjkUnifiedIdeograph5b11': '\xe5\xac\x91',
'CjkUnifiedIdeograph5b12': '\xe5\xac\x92',
'CjkUnifiedIdeograph5b13': '\xe5\xac\x93',
'UnicodeCharacter0x2e4f': '\xe2\xb9\x8f',
'UnicodeCharacter0x2e4e': '\xe2\xb9\x8e',
'UnicodeCharacter0x2e4d': '\xe2\xb9\x8d',
'UnicodeCharacter0x2e4c': '\xe2\xb9\x8c',
'WhiteSquareWithUpperRightQuadrant': '\xe2\x97\xb3',
'UnicodeCharacter0x2e4a': '\xe2\xb9\x8a',
'UpBarbLeftDownBarbRightHarpoon': '\xe2\xa5\x8d',
'HangulSyllableGwaess': '\xea\xb4\xac',
'CircledHangulKhieukh': '\xe3\x89\xaa',
'UnicodeCharacter0xaabb': '\xea\xaa\xbb',
'GeorgianCapitalLetterTan': '\xe1\x82\xa7',
'GeorgianCapitalLetterTar': '\xe1\x82\xb2',
'UnicodeCharacter0xd9fc': '\xed\xa7\xbc',
'UnicodeCharacter0xaabd': '\xea\xaa\xbd',
'UnicodeCharacter0xd9fa': '\xed\xa7\xba',
'MongolianVowelSeparator': '\xe1\xa0\x8e',
'UnicodeCharacter0xd9ff': '\xed\xa7\xbf',
'UnicodeCharacter0xd9fe': '\xed\xa7\xbe',
'UnicodeCharacter0xaabe': '\xea\xaa\xbe',
'CjkUnifiedIdeograph5b1d': '\xe5\xac\x9d',
'CjkUnifiedIdeograph5b1e': '\xe5\xac\x9e',
'CjkUnifiedIdeograph5b1f': '\xe5\xac\x9f',
'CjkUnifiedIdeograph5b1a': '\xe5\xac\x9a',
'UnicodeCharacter0x2e49': '\xe2\xb9\x89',
'UnicodeCharacter0x2e48': '\xe2\xb9\x88',
'UnicodeCharacter0x2e47': '\xe2\xb9\x87',
'UnicodeCharacter0x2e46': '\xe2\xb9\x86',
'UnicodeCharacter0x2e45': '\xe2\xb9\x85',
'UnicodeCharacter0x2e44': '\xe2\xb9\x84',
'UnicodeCharacter0x2e43': '\xe2\xb9\x83',
'UnicodeCharacter0x2e42': '\xe2\xb9\x82',
'UnicodeCharacter0x2e41': '\xe2\xb9\x81',
'UnicodeCharacter0x2e40': '\xe2\xb9\x80',
'UnicodeCharacter0xf60d': '\xef\x98\x8d',
'UnicodeCharacter0xf60e': '\xef\x98\x8e',
'UnicodeCharacter0xf60f': '\xef\x98\x8f',
'HangulSyllableCulb': '\xec\xb6\x9f',
'ArabicTatweel': '\xd9\x80',
'UnicodeCharacter0xf60b': '\xef\x98\x8b',
'ThaiCharacterOAng': '\xe0\xb8\xad',
'HangulSyllableCet': '\xec\xb3\x8d',
'BoxDrawingsDownLightAndLeftUpHeavy': '\xe2\x94\xa9',
'HangulSyllableDyaess': '\xeb\x8d\x8c',
'UnicodeCharacter0xd9c9': '\xed\xa7\x89',
'HangulSyllableJjyegg': '\xec\xaa\xa2',
'LatinSmallLetterZWithSwashTail': '\xc9\x80',
'CjkUnifiedIdeograph3ef8': '\xe3\xbb\xb8',
'CjkUnifiedIdeograph3ef9': '\xe3\xbb\xb9',
'UnicodeCharacter0xf65d': '\xef\x99\x9d',
'CjkUnifiedIdeograph3ef4': '\xe3\xbb\xb4',
'CjkUnifiedIdeograph3ef5': '\xe3\xbb\xb5',
'CjkUnifiedIdeograph3ef6': '\xe3\xbb\xb6',
'CjkUnifiedIdeograph3ef7': '\xe3\xbb\xb7',
'CjkUnifiedIdeograph3ef0': '\xe3\xbb\xb0',
'CjkUnifiedIdeograph3ef1': '\xe3\xbb\xb1',
'CjkUnifiedIdeograph3ef2': '\xe3\xbb\xb2',
'CjkUnifiedIdeograph3ef3': '\xe3\xbb\xb3',
'ChamDigitFive': '\xea\xa9\x95',
'SylotiNagriLetterTo': '\xea\xa0\x94',
'SquareCmSquared': '\xe3\x8e\xa0',
'CjkUnifiedIdeograph9957': '\xe9\xa5\x97',
'CjkUnifiedIdeograph6ea6': '\xe6\xba\xa6',
'FullwidthEqualsSign': '\xef\xbc\x9d',
'HangulSyllableRyeh': '\xeb\xa1\x9b',
'CjkUnifiedIdeograph416a': '\xe4\x85\xaa',
'CjkUnifiedIdeograph416c': '\xe4\x85\xac',
'CjkUnifiedIdeograph416b': '\xe4\x85\xab',
'CjkUnifiedIdeograph416e': '\xe4\x85\xae',
'CjkUnifiedIdeograph416d': '\xe4\x85\xad',
'CjkUnifiedIdeograph416f': '\xe4\x85\xaf',
'CjkUnifiedIdeograph9954': '\xe9\xa5\x94',
'ArabicindicDigitSeven': '\xd9\xa7',
'HangulSyllableSsenj': '\xec\x8e\x89',
'CjkUnifiedIdeograph9953': '\xe9\xa5\x93',
'HangulSyllableRyic': '\xeb\xa6\xa7',
'HangulSyllableRyel': '\xeb\xa1\x88',
'HangulSyllableJu': '\xec\xa3\xbc',
'HangulSyllableRyeo': '\xeb\xa0\xa4',
'OlChikiLetterEn': '\xe1\xb1\xb1',
'MyanmarLetterWa': '\xe1\x80\x9d',
'HangulSyllableRyen': '\xeb\xa1\x84',
'HangulSyllableBbung': '\xeb\xbf\xa1',
'CjkUnifiedIdeograph3efd': '\xe3\xbb\xbd',
'CjkUnifiedIdeograph3efe': '\xe3\xbb\xbe',
'CjkUnifiedIdeograph3eff': '\xe3\xbb\xbf',
'TwoDotsOverOneDotPunctuation': '\xe2\xb8\xaa',
'CjkUnifiedIdeograph3efa': '\xe3\xbb\xba',
'CjkUnifiedIdeograph3efb': '\xe3\xbb\xbb',
'CjkUnifiedIdeograph3efc': '\xe3\xbb\xbc',
'MalayalamSignVirama': '\xe0\xb5\x8d',
'UnicodeCharacter0xf6ce': '\xef\x9b\x8e',
'UnicodeCharacter0xf6cd': '\xef\x9b\x8d',
'CjkUnifiedIdeograph4169': '\xe4\x85\xa9',
'CjkUnifiedIdeograph4168': '\xe4\x85\xa8',
'HangulSyllableA': '\xec\x95\x84',
'HangulSyllableRyeg': '\xeb\xa1\x81',
'CjkUnifiedIdeograph4161': '\xe4\x85\xa1',
'CjkUnifiedIdeograph4160': '\xe4\x85\xa0',
'CjkUnifiedIdeograph4163': '\xe4\x85\xa3',
'CjkUnifiedIdeograph4162': '\xe4\x85\xa2',
'CjkUnifiedIdeograph4165': '\xe4\x85\xa5',
'CjkUnifiedIdeograph4164': '\xe4\x85\xa4',
'CjkUnifiedIdeograph4167': '\xe4\x85\xa7',
'CjkUnifiedIdeograph4166': '\xe4\x85\xa6',
'CjkUnifiedIdeograph5cc7': '\xe5\xb3\x87',
'CjkUnifiedIdeograph5cc6': '\xe5\xb3\x86',
'CjkUnifiedIdeograph5cc5': '\xe5\xb3\x85',
'CjkUnifiedIdeograph5cc4': '\xe5\xb3\x84',
'CjkUnifiedIdeograph5cc3': '\xe5\xb3\x83',
'CjkUnifiedIdeograph5cc2': '\xe5\xb3\x82',
'CjkUnifiedIdeograph5cc1': '\xe5\xb3\x81',
'HebrewAccentYerahBenYomo': '\xd6\xaa',
'HangulSyllableNanh': '\xeb\x82\x9e',
'HangulSyllableNanj': '\xeb\x82\x9d',
'CyrillicCapitalLetterReversedZe': '\xd4\x90',
'CjkUnifiedIdeograph84f8': '\xe8\x93\xb8',
'CjkUnifiedIdeograph5cc9': '\xe5\xb3\x89',
'CjkUnifiedIdeograph5cc8': '\xe5\xb3\x88',
'HangulSyllableHwegs': '\xed\x9b\xbf',
'UnicodeCharacter0xd99c': '\xed\xa6\x9c',
'GreekSmallLetterIotaWithDasiaAndPerispomeni': '\xe1\xbc\xb7',
'CjkUnifiedIdeograph76bc': '\xe7\x9a\xbc',
'CjkUnifiedIdeograph76bb': '\xe7\x9a\xbb',
'CjkUnifiedIdeograph76ba': '\xe7\x9a\xba',
'HangulSyllableI': '\xec\x9d\xb4',
'CjkUnifiedIdeograph76bf': '\xe7\x9a\xbf',
'CjkUnifiedIdeograph76be': '\xe7\x9a\xbe',
'HangulLetterKiyeoksios': '\xe3\x84\xb3',
'HangulSyllableKwilp': '\xed\x81\x82',
'UnicodeCharacter0xeaf1': '\xee\xab\xb1',
'CjkUnifiedIdeograph5b5a': '\xe5\xad\x9a',
'UnicodeCharacter0xe83': '\xe0\xba\x83',
'CjkUnifiedIdeograph569a': '\xe5\x9a\x9a',
'CjkUnifiedIdeograph569b': '\xe5\x9a\x9b',
'CjkUnifiedIdeograph569c': '\xe5\x9a\x9c',
'CjkUnifiedIdeograph569d': '\xe5\x9a\x9d',
'UnicodeCharacter0xe80': '\xe0\xba\x80',
'CjkUnifiedIdeograph569f': '\xe5\x9a\x9f',
'HangulSyllablePeb': '\xed\x8e\xa9',
'VaiSyllableCe': '\xea\x97\xbf',
'EthiopicSyllableXo': '\xe1\x8a\x86',
'EthiopicSyllableXi': '\xe1\x8a\x82',
'VaiSyllableCa': '\xea\x95\xa6',
'EthiopicSyllableXe': '\xe1\x8a\x85',
'UnicodeCharacter0xe86': '\xe0\xba\x86',
'VaiSyllableCo': '\xea\x97\x97',
'EthiopicSyllableXa': '\xe1\x8a\x80',
'VaiSyllableCi': '\xea\x94\xbf',
'CjkUnifiedIdeograph5ccf': '\xe5\xb3\x8f',
'CjkUnifiedIdeograph5cce': '\xe5\xb3\x8e',
'CjkUnifiedIdeograph5ccd': '\xe5\xb3\x8d',
'ModifierLetterEndLowTone': '\xcb\xbc',
'CjkUnifiedIdeograph5ccb': '\xe5\xb3\x8b',
'CjkUnifiedIdeograph5cca': '\xe5\xb3\x8a',
'HangulSyllableBbags': '\xeb\xb9\xa3',
'EthiopicSyllableXu': '\xe1\x8a\x81',
'Comet': '\xe2\x98\x84',
'CjkUnifiedIdeograph8dcf': '\xe8\xb7\x8f',
'CjkUnifiedIdeograph5698': '\xe5\x9a\x98',
'CjkUnifiedIdeograph5699': '\xe5\x9a\x99',
'UnicodeCharacter0xeaf3': '\xee\xab\xb3',
'CjkUnifiedIdeograph5690': '\xe5\x9a\x90',
'CjkUnifiedIdeograph5691': '\xe5\x9a\x91',
'CjkUnifiedIdeograph5692': '\xe5\x9a\x92',
'CjkUnifiedIdeograph5693': '\xe5\x9a\x93',
'CjkUnifiedIdeograph5694': '\xe5\x9a\x94',
'CjkUnifiedIdeograph5695': '\xe5\x9a\x95',
'CjkUnifiedIdeograph5696': '\xe5\x9a\x96',
'CjkUnifiedIdeograph5697': '\xe5\x9a\x97',
'CjkUnifiedIdeograph76b3': '\xe7\x9a\xb3',
'CjkUnifiedIdeograph76b2': '\xe7\x9a\xb2',
'GreekSmallLetterAlphaWithVaria': '\xe1\xbd\xb0',
'CopticCapitalLetterSou': '\xe2\xb2\x8a',
'CjkUnifiedIdeograph76b7': '\xe7\x9a\xb7',
'CjkUnifiedIdeograph76b6': '\xe7\x9a\xb6',
'CjkUnifiedIdeograph76b5': '\xe7\x9a\xb5',
'CjkUnifiedIdeograph76b4': '\xe7\x9a\xb4',
'ArabicLetterSheenIsolatedForm': '\xef\xba\xb5',
'CjkUnifiedIdeograph76b9': '\xe7\x9a\xb9',
'CjkUnifiedIdeograph76b8': '\xe7\x9a\xb8',
'CjkUnifiedIdeograph9852': '\xe9\xa1\x92',
'CjkUnifiedIdeograph9853': '\xe9\xa1\x93',
'CjkUnifiedIdeograph9850': '\xe9\xa1\x90',
'CjkUnifiedIdeograph9851': '\xe9\xa1\x91',
'CjkUnifiedIdeograph9856': '\xe9\xa1\x96',
'CjkUnifiedIdeograph9857': '\xe9\xa1\x97',
'CjkUnifiedIdeograph9854': '\xe9\xa1\x94',
'CjkUnifiedIdeograph9855': '\xe9\xa1\x95',
'CjkUnifiedIdeograph9858': '\xe9\xa1\x98',
'CjkUnifiedIdeograph9859': '\xe9\xa1\x99',
'InfinityNegatedWithVerticalBar': '\xe2\xa7\x9e',
'HexagramForDifficultyAtTheBeginning': '\xe4\xb7\x82',
'ArabicLetterTtehehInitialForm': '\xef\xad\xa0',
'HangulSyllableKwilt': '\xed\x81\x81',
'SinhalaLetterMahaapraanaJayanna': '\xe0\xb6\xa3',
'CjkUnifiedIdeograph9348': '\xe9\x8d\x88',
'FullwidthGreaterthanSign': '\xef\xbc\x9e',
'RightOuterJoin': '\xe2\x9f\x96',
'HangulSyllableJeogg': '\xec\xa0\x82',
'CyrillicCapitalLetterErWithTick': '\xd2\x8e',
'UnicodeCharacter0xdd5f': '\xed\xb5\x9f',
'CjkUnifiedIdeograph359e': '\xe3\x96\x9e',
'CjkUnifiedIdeograph359d': '\xe3\x96\x9d',
'CjkUnifiedIdeograph5340': '\xe5\x8d\x80',
'CjkUnifiedIdeograph359f': '\xe3\x96\x9f',
'CjkUnifiedIdeograph359a': '\xe3\x96\x9a',
'CjkUnifiedIdeograph5347': '\xe5\x8d\x87',
'CjkUnifiedIdeograph5344': '\xe5\x8d\x84',
'CjkUnifiedIdeograph5345': '\xe5\x8d\x85',
'UnicodeCharacter0xf0f0': '\xef\x83\xb0',
'UnicodeCharacter0xf0f1': '\xef\x83\xb1',
'CjkUnifiedIdeograph5348': '\xe5\x8d\x88',
'CjkUnifiedIdeograph5349': '\xe5\x8d\x89',
'UnicodeCharacter0xf0f4': '\xef\x83\xb4',
'CjkUnifiedIdeograph9344': '\xe9\x8d\x84',
'HangulSyllableJeogs': '\xec\xa0\x83',
'UnicodeCharacter0xf0f7': '\xef\x83\xb7',
'UnicodeCharacter0xeaf6': '\xee\xab\xb6',
'UnicodeCharacter0xf6c9': '\xef\x9b\x89',
'CjkUnifiedIdeograph9345': '\xe9\x8d\x85',
'CjkUnifiedIdeograph70a8': '\xe7\x82\xa8',
'UnicodeCharacter0xf6c8': '\xef\x9b\x88',
'HangulSyllableJeolm': '\xec\xa0\x8a',
'CircledIdeographResource': '\xe3\x8a\xae',
'DingbatNegativeCircledSansserifDigitEight': '\xe2\x9e\x91',
'CjkUnifiedIdeograph6131': '\xe6\x84\xb1',
'CjkUnifiedIdeograph985b': '\xe9\xa1\x9b',
'CjkUnifiedIdeograph985c': '\xe9\xa1\x9c',
'SquareKiromeetoru': '\xe3\x8c\x96',
'CjkUnifiedIdeograph985a': '\xe9\xa1\x9a',
'CjkUnifiedIdeograph985f': '\xe9\xa1\x9f',
'CjkUnifiedIdeograph9340': '\xe9\x8d\x80',
'CjkUnifiedIdeograph985d': '\xe9\xa1\x9d',
'CjkUnifiedIdeograph985e': '\xe9\xa1\x9e',
'HangulSyllableMael': '\xeb\xa7\xac',
'CjkUnifiedIdeograph9341': '\xe9\x8d\x81',
'CjkUnifiedIdeograph83d3': '\xe8\x8f\x93',
'UnicodeCharacter0xeaf7': '\xee\xab\xb7',
'CjkUnifiedIdeograph3595': '\xe3\x96\x95',
'CjkUnifiedIdeograph534c': '\xe5\x8d\x8c',
'CjkUnifiedIdeograph3597': '\xe3\x96\x97',
'CjkUnifiedIdeograph3596': '\xe3\x96\x96',
'CjkUnifiedIdeograph534f': '\xe5\x8d\x8f',
'CjkUnifiedIdeograph3590': '\xe3\x96\x90',
'CjkUnifiedIdeograph3593': '\xe3\x96\x93',
'CjkUnifiedIdeograph534e': '\xe5\x8d\x8e',
'SubsetOfOrEqualToWithDotAbove': '\xe2\xab\x83',
'UnicodeCharacter0xf0fa': '\xef\x83\xba',
'UnicodeCharacter0xf0fb': '\xef\x83\xbb',
'UnicodeCharacter0xf0fc': '\xef\x83\xbc',
'CjkUnifiedIdeograph3599': '\xe3\x96\x99',
'CjkUnifiedIdeograph3598': '\xe3\x96\x98',
'UnicodeCharacter0xf0ff': '\xef\x83\xbf',
'VaiSyllableDha': '\xea\x95\x9c',
'CjkUnifiedIdeograph905f': '\xe9\x81\x9f',
'UnicodeCharacter0xdbb3': '\xed\xae\xb3',
'UnicodeCharacter0xeaf8': '\xee\xab\xb8',
'LeftAndRightTack': '\xe2\x9f\x9b',
'UnicodeCharacter0xf175': '\xef\x85\xb5',
'BopomofoLetterBu': '\xe3\x86\xa0',
'UnicodeCharacter0xf6c5': '\xef\x9b\x85',
'AplFunctionalSymbolDelTilde': '\xe2\x8d\xab',
'UnicodeCharacter0xf6c4': '\xef\x9b\x84',
'CjkUnifiedIdeograph71a2': '\xe7\x86\xa2',
'BraillePatternDots346': '\xe2\xa0\xac',
'CjkUnifiedIdeograph490c': '\xe4\xa4\x8c',
'CjkUnifiedIdeograph490b': '\xe4\xa4\x8b',
'CjkUnifiedIdeograph490a': '\xe4\xa4\x8a',
'UnicodeCharacter0xf6c6': '\xef\x9b\x86',
'CjkUnifiedIdeograph490f': '\xe4\xa4\x8f',
'CjkUnifiedIdeograph490e': '\xe4\xa4\x8e',
'CjkUnifiedIdeograph490d': '\xe4\xa4\x8d',
'UnicodeCharacter0xaab0': '\xea\xaa\xb0',
'UnicodeCharacter0xaab1': '\xea\xaa\xb1',
'UnicodeCharacter0xaab2': '\xea\xaa\xb2',
'HangulSyllablePej': '\xed\x8e\xae',
'KangxiRadicalHand': '\xe2\xbc\xbf',
'CjkUnifiedIdeograph8b9f': '\xe8\xae\x9f',
'CjkUnifiedIdeograph8b9e': '\xe8\xae\x9e',
'CjkUnifiedIdeograph8b9d': '\xe8\xae\x9d',
'CjkUnifiedIdeograph8b9c': '\xe8\xae\x9c',
'CjkUnifiedIdeograph8b9b': '\xe8\xae\x9b',
'CjkUnifiedIdeograph8b9a': '\xe8\xae\x9a',
'UnicodeCharacter0xaab4': '\xea\xaa\xb4',
'LatinSmallLetterPhi': '\xc9\xb8',
'UnicodeCharacter0xaab5': '\xea\xaa\xb5',
'CyrillicCapitalLetterAbkhasianChe': '\xd2\xbc',
'CjkUnifiedIdeograph4908': '\xe4\xa4\x88',
'UnicodeCharacter0xaab6': '\xea\xaa\xb6',
'CjkUnifiedIdeograph4903': '\xe4\xa4\x83',
'CjkUnifiedIdeograph4902': '\xe4\xa4\x82',
'CjkUnifiedIdeograph4901': '\xe4\xa4\x81',
'CjkUnifiedIdeograph4900': '\xe4\xa4\x80',
'CjkUnifiedIdeograph4907': '\xe4\xa4\x87',
'QuadrantLowerRight': '\xe2\x96\x97',
'CjkUnifiedIdeograph4905': '\xe4\xa4\x85',
'CjkUnifiedIdeograph4904': '\xe4\xa4\x84',
'SquareWithUpperRightToLowerLeftFill': '\xe2\x96\xa8',
'UnicodeCharacter0xaab8': '\xea\xaa\xb8',
'HangulSyllableNyeok': '\xeb\x85\x98',
'UnicodeCharacter0xaab9': '\xea\xaa\xb9',
'HangulSyllableNyogs': '\xeb\x87\xab',
'CjkUnifiedIdeograph8b97': '\xe8\xae\x97',
'CjkUnifiedIdeograph831b': '\xe8\x8c\x9b',
'CjkUnifiedIdeograph8b95': '\xe8\xae\x95',
'TibetanSubjoinedLetterDzha': '\xe0\xbe\xac',
'CjkUnifiedIdeograph5825': '\xe5\xa0\xa5',
'CjkUnifiedIdeograph8b92': '\xe8\xae\x92',
'CjkUnifiedIdeograph8b91': '\xe8\xae\x91',
'ArabicLetterTheh': '\xd8\xab',
'ArabicLigatureThehWithAlefMaksuraIsolatedForm': '\xef\xb0\x93',
'CjkUnifiedIdeograph8b99': '\xe8\xae\x99',
'CjkUnifiedIdeograph8b98': '\xe8\xae\x98',
'CanadianSyllabicsSouthslaveyKoh': '\xe1\x92\x87',
'CjkUnifiedIdeograph831f': '\xe8\x8c\x9f',
'CjkUnifiedIdeograph7450': '\xe7\x91\x90',
'CjkUnifiedIdeograph7451': '\xe7\x91\x91',
'CjkUnifiedIdeograph7452': '\xe7\x91\x92',
'LatinSmallLetterRWithAcute': '\xc5\x95',
'CjkUnifiedIdeograph7454': '\xe7\x91\x94',
'CjkUnifiedIdeograph7455': '\xe7\x91\x95',
'CjkUnifiedIdeograph7456': '\xe7\x91\x96',
'CjkUnifiedIdeograph7457': '\xe7\x91\x97',
'CjkUnifiedIdeograph7458': '\xe7\x91\x98',
'CjkUnifiedIdeograph7459': '\xe7\x91\x99',
'HalfwidthLeftwardsArrow': '\xef\xbf\xa9',
'CjkUnifiedIdeograph82f7': '\xe8\x8b\xb7',
'CjkUnifiedIdeograph99c5': '\xe9\xa7\x85',
'CjkUnifiedIdeograph99c4': '\xe9\xa7\x84',
'CjkUnifiedIdeograph99c7': '\xe9\xa7\x87',
'CjkUnifiedIdeograph99c6': '\xe9\xa7\x86',
'CjkUnifiedIdeograph99c1': '\xe9\xa7\x81',
'CjkUnifiedIdeograph99c0': '\xe9\xa7\x80',
'CjkUnifiedIdeograph99c3': '\xe9\xa7\x83',
'CjkUnifiedIdeograph99c2': '\xe9\xa7\x82',
'UnicodeCharacter0xf1ad': '\xef\x86\xad',
'CjkUnifiedIdeograph82f5': '\xe8\x8b\xb5',
'CjkUnifiedIdeograph99c9': '\xe9\xa7\x89',
'CjkUnifiedIdeograph99c8': '\xe9\xa7\x88',
'CjkUnifiedIdeograph8a21': '\xe8\xa8\xa1',
'CjkUnifiedIdeograph8a20': '\xe8\xa8\xa0',
'CjkUnifiedIdeograph3d38': '\xe3\xb4\xb8',
'CjkUnifiedIdeograph3d39': '\xe3\xb4\xb9',
'CjkUnifiedIdeograph8a25': '\xe8\xa8\xa5',
'CjkUnifiedIdeograph8a24': '\xe8\xa8\xa4',
'CjkUnifiedIdeograph8a27': '\xe8\xa8\xa7',
'CanadianSyllabicsAivilikB': '\xe1\x96\xaf',
'CjkUnifiedIdeograph3d32': '\xe3\xb4\xb2',
'CjkUnifiedIdeograph3d33': '\xe3\xb4\xb3',
'CjkUnifiedIdeograph3d30': '\xe3\xb4\xb0',
'CjkUnifiedIdeograph3d31': '\xe3\xb4\xb1',
'CjkUnifiedIdeograph3d36': '\xe3\xb4\xb6',
'CjkUnifiedIdeograph3d37': '\xe3\xb4\xb7',
'CjkUnifiedIdeograph3d34': '\xe3\xb4\xb4',
'CjkUnifiedIdeograph3d35': '\xe3\xb4\xb5',
'CircledDigitZero': '\xe2\x93\xaa',
'HangulSyllableHyeot': '\xed\x98\x99',
'NewTaiLueVowelSignOa': '\xe1\xa6\xb8',
'HangulSyllableHyeos': '\xed\x98\x93',
'CjkUnifiedIdeograph99ce': '\xe9\xa7\x8e',
'CjkUnifiedIdeograph99cd': '\xe9\xa7\x8d',
'CjkUnifiedIdeograph99cf': '\xe9\xa7\x8f',
'CjkUnifiedIdeograph99ca': '\xe9\xa7\x8a',
'CjkUnifiedIdeograph99cc': '\xe9\xa7\x8c',
'CjkUnifiedIdeograph99cb': '\xe9\xa7\x8b',
'BuhidLetterPa': '\xe1\x9d\x89',
'CjkUnifiedIdeograph7ca4': '\xe7\xb2\xa4',
'HangulSyllablePyeng': '\xed\x8f\xa5',
'CjkUnifiedIdeograph745a': '\xe7\x91\x9a',
'CjkUnifiedIdeograph70a1': '\xe7\x82\xa1',
'CjkUnifiedIdeograph745c': '\xe7\x91\x9c',
'CjkUnifiedIdeograph745d': '\xe7\x91\x9d',
'CjkUnifiedIdeograph6608': '\xe6\x98\x88',
'CjkUnifiedIdeograph745f': '\xe7\x91\x9f',
'CjkUnifiedIdeograph7ca9': '\xe7\xb2\xa9',
'ArabicLetterPehFinalForm': '\xef\xad\x97',
'HangulSyllableMyigs': '\xeb\xaf\x9f',
'HangulSyllablePwet': '\xed\x93\x89',
'UnicodeCharacter0x137e': '\xe1\x8d\xbe',
'UnicodeCharacter0x137d': '\xe1\x8d\xbd',
'CjkUnifiedIdeograph8a2c': '\xe8\xa8\xac',
'UnicodeCharacter0x137f': '\xe1\x8d\xbf',
'CjkUnifiedIdeograph8a2e': '\xe8\xa8\xae',
'CjkUnifiedIdeograph8a2d': '\xe8\xa8\xad',
'CjkUnifiedIdeograph8a2f': '\xe8\xa8\xaf',
'CjkUnifiedIdeograph3d3b': '\xe3\xb4\xbb',
'CjkUnifiedIdeograph3d3c': '\xe3\xb4\xbc',
'CjkUnifiedIdeograph3d3a': '\xe3\xb4\xba',
'CjkUnifiedIdeograph3d3f': '\xe3\xb4\xbf',
'LaoLetterCo': '\xe0\xba\x88',
'CjkUnifiedIdeograph3d3d': '\xe3\xb4\xbd',
'CjkUnifiedIdeograph3d3e': '\xe3\xb4\xbe',
'PerSign': '\xe2\x85\x8c',
'LatinCapitalLetterAWithTilde': '\xc3\x83',
'HangulSyllableBbyit': '\xec\x82\x8d',
'HangulSyllableSsyuss': '\xec\x93\xa8',
'ArabicLetterSadFinalForm': '\xef\xba\xba',
'ii': '\xe2\x85\x88',
'UnicodeCharacter0xdcd9': '\xed\xb3\x99',
'im': '\xe2\x85\x88',
'HangulSyllableDyeolg': '\xeb\x8e\x95',
'CjkUnifiedIdeograph905d': '\xe9\x81\x9d',
'BoxDrawingsVerticalSingleAndLeftDouble': '\xe2\x95\xa1',
'HangulSyllableYaelm': '\xec\x96\xa2',
'LatinSmallLetterDotlessJWithStroke': '\xc9\x9f',
'CjkUnifiedIdeograph8922': '\xe8\xa4\xa2',
'Gear': '\xe2\x9a\x99',
'HangulSyllableDyeolb': '\xeb\x8e\x97',
'HangulSyllableYaelg': '\xec\x96\xa1',
'CjkUnifiedIdeograph7a60': '\xe7\xa9\xa0',
'UnicodeCharacter0xa973': '\xea\xa5\xb3',
'HangulSyllableYaelb': '\xec\x96\xa3',
'CjkUnifiedIdeograph96cf': '\xe9\x9b\x8f',
'CyrillicCapitalLetterChe': '\xd0\xa7',
'CjkUnifiedIdeograph96cd': '\xe9\x9b\x8d',
'CjkUnifiedIdeograph96ce': '\xe9\x9b\x8e',
'CjkUnifiedIdeograph96cb': '\xe9\x9b\x8b',
'CjkUnifiedIdeograph96cc': '\xe9\x9b\x8c',
'CjkUnifiedIdeograph96ca': '\xe9\x9b\x8a',
'HangulSyllableYaelt': '\xec\x96\xa5',
'HangulSyllableYaelp': '\xec\x96\xa6',
'HangulSyllableYaels': '\xec\x96\xa4',
'YiSyllableKot': '\xea\x88\x86',
'YiSyllableRut': '\xea\x8f\x94',
'HangulSyllableDdyegs': '\xeb\x97\xb7',
'YiSyllableRur': '\xea\x8f\x99',
'SundaneseDigitFive': '\xe1\xae\xb5',
'YiSyllableRup': '\xea\x8f\x97',
'CopticSmallLetterGamma': '\xe2\xb2\x85',
'LatinCapitalLetterOWithMacronAndAcute': '\xe1\xb9\x92',
'YiSyllableKox': '\xea\x88\x87',
'YiSyllableRux': '\xea\x8f\x95',
'ChiRho': '\xe2\x98\xa7',
'BengaliLetterBha': '\xe0\xa6\xad',
'HangulSyllableDdyegg': '\xeb\x97\xb6',
'HebrewMarkUpperDot': '\xd7\x84',
'CjkUnifiedIdeograph96c6': '\xe9\x9b\x86',
'CjkUnifiedIdeograph96c7': '\xe9\x9b\x87',
'CjkUnifiedIdeograph96c4': '\xe9\x9b\x84',
'CjkUnifiedIdeograph8dc8': '\xe8\xb7\x88',
'CjkUnifiedIdeograph96c2': '\xe9\x9b\x82',
'KangxiRadicalLeather': '\xe2\xbe\xb0',
'CjkUnifiedIdeograph96c0': '\xe9\x9b\x80',
'CjkUnifiedIdeograph96c1': '\xe9\x9b\x81',
'HangulSyllableByeonj': '\xeb\xb3\x81',
'HangulSyllableSsyigs': '\xec\x94\x8f',
'HangulSyllableByeonh': '\xeb\xb3\x82',
'ArabicLetterDalWithDotBelowAndSmallTah': '\xda\x8b',
'CjkUnifiedIdeograph96c8': '\xe9\x9b\x88',
'CjkUnifiedIdeograph96c9': '\xe9\x9b\x89',
'YiSyllableNbap': '\xea\x82\x80',
'DecimalSeparatorKeySymbol': '\xe2\x8e\x96',
'YiSyllableNbat': '\xea\x81\xbd',
'YiSyllableNbax': '\xea\x81\xbe',
'HangulSyllableSsyigg': '\xec\x94\x8e',
'SquareMiri': '\xe3\x8d\x89',
'HangulSyllableMeob': '\xeb\xa9\x89',
'CjkUnifiedIdeograph6cdf': '\xe6\xb3\x9f',
'CjkUnifiedIdeograph6cde': '\xe6\xb3\x9e',
'CjkUnifiedIdeograph6cdd': '\xe6\xb3\x9d',
'CjkUnifiedIdeograph6cdc': '\xe6\xb3\x9c',
'CjkUnifiedIdeograph6cdb': '\xe6\xb3\x9b',
'CjkUnifiedIdeograph6cda': '\xe6\xb3\x9a',
'HangulSyllableMeoj': '\xeb\xa9\x8e',
'HangulSyllableMeok': '\xeb\xa9\x90',
'HangulSyllableMeoh': '\xeb\xa9\x93',
'HangulSyllableGeud': '\xea\xb7\xbf',
'HangulSyllableMeon': '\xeb\xa8\xbc',
'CyrillicSmallLetterIeWithBreve': '\xd3\x97',
'HangulSyllableMeom': '\xeb\xa9\x88',
'DottedCircle': '\xe2\x97\x8c',
'CjkUnifiedIdeograph934f': '\xe9\x8d\x8f',
'HangulSyllableMeop': '\xeb\xa9\x92',
'NkoLetterRra': '\xdf\x9a',
'HangulSyllableMeot': '\xeb\xa9\x91',
'coprod': '\xe2\x88\x90',
'CjkUnifiedIdeograph934d': '\xe9\x8d\x8d',
'ModifierLetterSmallCapitalI': '\xe1\xb6\xa6',
'CjkUnifiedIdeograph934e': '\xe9\x8d\x8e',
'ModifierLetterSmallCapitalN': '\xe1\xb6\xb0',
'RejangLetterHa': '\xea\xa5\x81',
'ModifierLetterSmallCapitalL': '\xe1\xb6\xab',
'BoxDrawingsHeavyDoubleDashHorizontal': '\xe2\x95\x8d',
'GlagoliticCapitalLetterVede': '\xe2\xb0\x82',
'CjkUnifiedIdeograph934c': '\xe9\x8d\x8c',
'CjkUnifiedIdeograph934a': '\xe9\x8d\x8a',
'PostalMark': '\xe3\x80\x92',
'ModifierLetterSmallCapitalU': '\xe1\xb6\xb8',
'HangulSyllableRweogs': '\xeb\xa4\x87',
'EthiopicDigitEight': '\xe1\x8d\xb0',
'CjkUnifiedIdeograph6cd7': '\xe6\xb3\x97',
'CjkUnifiedIdeograph5b50': '\xe5\xad\x90',
'CjkUnifiedIdeograph6cd5': '\xe6\xb3\x95',
'CjkUnifiedIdeograph6cd4': '\xe6\xb3\x94',
'CjkUnifiedIdeograph6cd3': '\xe6\xb3\x93',
'CjkUnifiedIdeograph6cd2': '\xe6\xb3\x92',
'CjkUnifiedIdeograph6cd1': '\xe6\xb3\x91',
'CjkUnifiedIdeograph6cd0': '\xe6\xb3\x90',
'CjkUnifiedIdeograph5b52': '\xe5\xad\x92',
'CjkUnifiedIdeograph6cd9': '\xe6\xb3\x99',
'CjkUnifiedIdeograph6cd8': '\xe6\xb3\x98',
'CjkUnifiedIdeograph5b53': '\xe5\xad\x93',
'HangulSyllableMeogs': '\xeb\xa8\xbb',
'YiRadicalZziet': '\xea\x93\x84',
'CjkUnifiedIdeograph5b54': '\xe5\xad\x94',
'CjkUnifiedIdeograph5b55': '\xe5\xad\x95',
'HangulSyllableMeogg': '\xeb\xa8\xba',
'UnicodeCharacter0xe3e0': '\xee\x8f\xa0',
'CjkUnifiedIdeograph5b57': '\xe5\xad\x97',
'KayahLiLetterLa': '\xea\xa4\x9c',
'UnicodeCharacter0xf056': '\xef\x81\x96',
'updownarrow': '\xe2\x86\x95',
'CjkUnifiedIdeograph6bba': '\xe6\xae\xba',
'CjkUnifiedIdeograph6bbb': '\xe6\xae\xbb',
'LatinSmallLetterRWithDoubleGrave': '\xc8\x91',
'CjkUnifiedIdeograph6bbd': '\xe6\xae\xbd',
'CjkUnifiedIdeograph6bbe': '\xe6\xae\xbe',
'CjkUnifiedIdeograph6bbf': '\xe6\xae\xbf',
'HangulSyllableBbyim': '\xec\x82\x84',
'CjkUnifiedIdeograph8fd7': '\xe8\xbf\x97',
'HangulSyllableEuss': '\xec\x9d\x90',
'CanadianSyllabicsFinalGrave': '\xe1\x90\xa0',
'HexagramForMouthCorners': '\xe4\xb7\x9a',
'HangulSyllableJweb': '\xec\xa5\x85',
'OlChikiLetterErr': '\xe1\xb1\xb2',
'RightwardsArrowWithDoubleVerticalStroke': '\xe2\x87\xbb',
'ArabicLetterVeIsolatedForm': '\xef\xaf\x9e',
'HangulSyllableDip': '\xeb\x94\xae',
'PresentationFormForVerticalRightCornerBracket': '\xef\xb9\x82',
'HangulSyllableDdweot': '\xeb\x9b\x91',
'HangulSyllableYac': '\xec\x96\x93',
'VaiSyllableDhu': '\xea\x96\xa6',
'CombiningLeftwardsArrowOverlay': '\xe2\x83\xaa',
'ArabicLetterFehWithTwoDotsBelow': '\xdd\xa0',
'CjkUnifiedIdeograph892d': '\xe8\xa4\xad',
'CjkUnifiedIdeograph6bb8': '\xe6\xae\xb8',
'CjkUnifiedIdeograph6bb9': '\xe6\xae\xb9',
'CanadianSyllabicsNaskapiMwaa': '\xe1\x92\xba',
'CjkUnifiedIdeograph6bb0': '\xe6\xae\xb0',
'CjkUnifiedIdeograph6bb1': '\xe6\xae\xb1',
'CjkUnifiedIdeograph6bb2': '\xe6\xae\xb2',
'CjkUnifiedIdeograph6bb3': '\xe6\xae\xb3',
'CjkUnifiedIdeograph6bb4': '\xe6\xae\xb4',
'CjkUnifiedIdeograph6bb5': '\xe6\xae\xb5',
'CjkUnifiedIdeograph6bb6': '\xe6\xae\xb6',
'CjkUnifiedIdeograph6bb7': '\xe6\xae\xb7',
'HangulSyllableDig': '\xeb\x94\x95',
'HangulSyllableYam': '\xec\x96\x8c',
'UnicodeCharacter0xf840': '\xef\xa1\x80',
'HangulSyllableYaj': '\xec\x96\x92',
'ArabicLigatureSeenWithKhahMedialForm': '\xef\xb4\xb6',
'HangulSyllableYak': '\xec\x96\x94',
'HangulSyllableDid': '\xeb\x94\x9b',
'HangulSyllableYah': '\xec\x96\x97',
'NewTaiLueLetterHighNga': '\xe1\xa6\x84',
'HangulSyllableJegs': '\xec\xa0\x9f',
'HangulSyllableDic': '\xeb\x94\xab',
'UnicodeCharacter0xdcd5': '\xed\xb3\x95',
'CjkUnifiedIdeograph82fe': '\xe8\x8b\xbe',
'CjkUnifiedIdeograph660c': '\xe6\x98\x8c',
'UnicodeCharacter0xeef8': '\xee\xbb\xb8',
'CjkUnifiedIdeograph660a': '\xe6\x98\x8a',
'ArabicLigatureTehWithHahInitialForm': '\xef\xb2\xa2',
'UnicodeCharacter0xe753': '\xee\x9d\x93',
'CjkUnifiedIdeograph8279': '\xe8\x89\xb9',
'CjkRadicalFire': '\xe2\xba\xa3',
'HangulSyllableDin': '\xeb\x94\x98',
'CjkUnifiedIdeograph8275': '\xe8\x89\xb5',
'CjkUnifiedIdeograph8274': '\xe8\x89\xb4',
'CjkUnifiedIdeograph8277': '\xe8\x89\xb7',
'CjkUnifiedIdeograph8276': '\xe8\x89\xb6',
'CjkUnifiedIdeograph8271': '\xe8\x89\xb1',
'CjkUnifiedIdeograph8270': '\xe8\x89\xb0',
'BengaliLetterJa': '\xe0\xa6\x9c',
'CjkUnifiedIdeograph8272': '\xe8\x89\xb2',
'HangulSyllableDil': '\xeb\x94\x9c',
'HangulSyllableGgwibs': '\xeb\x80\x9e',
'BalineseDigitSix': '\xe1\xad\x96',
'SquareRemu': '\xe3\x8d\x95',
'HangulSyllableDij': '\xeb\x94\xaa',
'UnicodeCharacter0xf3bc': '\xef\x8e\xbc',
'UnicodeCharacter0xf3bb': '\xef\x8e\xbb',
'UnicodeCharacter0xf3ba': '\xef\x8e\xba',
'ArabicLetterVe': '\xdb\x8b',
'UnicodeCharacter0xf3be': '\xef\x8e\xbe',
'UnicodeCharacter0xf3bd': '\xef\x8e\xbd',
'UnicodeCharacter0xf1ea': '\xef\x87\xaa',
'HangulSyllableDih': '\xeb\x94\xaf',
'TeluguDigitZero': '\xe0\xb1\xa6',
'YiSyllableNap': '\xea\x86\x86',
'CjkUnifiedIdeograph51c8': '\xe5\x87\x88',
'CjkUnifiedIdeograph827e': '\xe8\x89\xbe',
'CjkUnifiedIdeograph827d': '\xe8\x89\xbd',
'CjkUnifiedIdeograph827f': '\xe8\x89\xbf',
'CjkUnifiedIdeograph827a': '\xe8\x89\xba',
'CjkUnifiedIdeograph827c': '\xe8\x89\xbc',
'CjkUnifiedIdeograph827b': '\xe8\x89\xbb',
'AplFunctionalSymbolQuadNotEqual': '\xe2\x8d\xaf',
'AplFunctionalSymbolQuadDel': '\xe2\x8d\x94',
'UnicodeCharacter0xfb42': '\xef\xad\x82',
'UnicodeCharacter0xf3b3': '\xef\x8e\xb3',
'UnicodeCharacter0xf3b2': '\xef\x8e\xb2',
'UnicodeCharacter0xf3b1': '\xef\x8e\xb1',
'UnicodeCharacter0xf3b0': '\xef\x8e\xb0',
'UnicodeCharacter0xf3b7': '\xef\x8e\xb7',
'UnicodeCharacter0xf3b6': '\xef\x8e\xb6',
'UnicodeCharacter0xf3b5': '\xef\x8e\xb5',
'UnicodeCharacter0xf3b4': '\xef\x8e\xb4',
'CjkUnifiedIdeograph4fff': '\xe4\xbf\xbf',
'UnicodeCharacter0xf3b9': '\xef\x8e\xb9',
'UnicodeCharacter0xf3b8': '\xef\x8e\xb8',
'LeftwardsTwoheadedArrowWithTail': '\xe2\xac\xbb',
'YiSyllableNax': '\xea\x86\x84',
'CanadianSyllabicsNwaa': '\xe1\x93\x8d',
'ArabicLigatureYehWithHamzaAboveWithMeemInitialForm': '\xef\xb2\x9a',
'SyriacHarkleanAsteriscus': '\xdc\x8d',
'CjkUnifiedIdeograph5b5b': '\xe5\xad\x9b',
'IdeographicTelegraphSymbolForNovember': '\xe3\x8b\x8a',
'CjkUnifiedIdeograph5b5c': '\xe5\xad\x9c',
'LongLeftwardsSquiggleArrow': '\xe2\xac\xb3',
'CjkUnifiedIdeograph5b5d': '\xe5\xad\x9d',
'DoesNotProve': '\xe2\x8a\xac',
'HangulSyllableJeulh': '\xec\xa6\x97',
'CjkUnifiedIdeograph5b5e': '\xe5\xad\x9e',
'CjkUnifiedIdeograph5deb': '\xe5\xb7\xab',
'CjkUnifiedIdeograph5dec': '\xe5\xb7\xac',
'CjkUnifiedIdeograph5dea': '\xe5\xb7\xaa',
'CjkUnifiedIdeograph5def': '\xe5\xb7\xaf',
'CjkUnifiedIdeograph5b5f': '\xe5\xad\x9f',
'CjkUnifiedIdeograph5ded': '\xe5\xb7\xad',
'CjkUnifiedIdeograph5dee': '\xe5\xb7\xae',
'UnicodeCharacter0x1a9e': '\xe1\xaa\x9e',
'UnicodeCharacter0x1a9d': '\xe1\xaa\x9d',
'CjkUnifiedIdeograph80f5': '\xe8\x83\xb5',
'UnicodeCharacter0x1a9f': '\xe1\xaa\x9f',
'UnicodeCharacter0x1a9a': '\xe1\xaa\x9a',
'HangulSyllableBwabs': '\xeb\xb4\xa2',
'UnicodeCharacter0x1a9c': '\xe1\xaa\x9c',
'UnicodeCharacter0x1a9b': '\xe1\xaa\x9b',
'CanadianSyllabicsPwaa': '\xe1\x91\x86',
'CjkUnifiedIdeograph8efb': '\xe8\xbb\xbb',
'CjkUnifiedIdeograph4ba9': '\xe4\xae\xa9',
'CjkUnifiedIdeograph4ba8': '\xe4\xae\xa8',
'UnicodeCharacter0xfbd0': '\xef\xaf\x90',
'MyanmarLetterKha': '\xe1\x80\x81',
'MyanmarLetterTta': '\xe1\x80\x8b',
'CjkUnifiedIdeograph4ba3': '\xe4\xae\xa3',
'CjkUnifiedIdeograph4ba2': '\xe4\xae\xa2',
'CjkUnifiedIdeograph4ba1': '\xe4\xae\xa1',
'CjkUnifiedIdeograph4ba0': '\xe4\xae\xa0',
'CjkUnifiedIdeograph4ba7': '\xe4\xae\xa7',
'CjkUnifiedIdeograph4ba6': '\xe4\xae\xa6',
'CjkUnifiedIdeograph4ba5': '\xe4\xae\xa5',
'CjkUnifiedIdeograph4ba4': '\xe4\xae\xa4',
'CjkUnifiedIdeograph5de2': '\xe5\xb7\xa2',
'CjkUnifiedIdeograph5de3': '\xe5\xb7\xa3',
'CjkUnifiedIdeograph5de0': '\xe5\xb7\xa0',
'CjkUnifiedIdeograph5de1': '\xe5\xb7\xa1',
'LatinCapitalLetterAWithMacron': '\xc4\x80',
'UnicodeCharacter0x1a98': '\xe1\xaa\x98',
'HangulJungseongSsangaraea': '\xe1\x86\xa2',
'CjkUnifiedIdeograph5de5': '\xe5\xb7\xa5',
'UnicodeCharacter0x1a95': '\xe1\xaa\x95',
'UnicodeCharacter0x1a94': '\xe1\xaa\x94',
'UnicodeCharacter0x1a97': '\xe1\xaa\x97',
'UnicodeCharacter0x1a96': '\xe1\xaa\x96',
'UnicodeCharacter0x1a91': '\xe1\xaa\x91',
'UnicodeCharacter0x1a90': '\xe1\xaa\x90',
'UnicodeCharacter0x1a93': '\xe1\xaa\x93',
'UnicodeCharacter0x1a92': '\xe1\xaa\x92',
'RunicLetterEhwazEhE': '\xe1\x9b\x96',
'HangulSyllableNelh': '\xeb\x84\xb3',
'HangulLetterSioskiyeok': '\xe3\x85\xba',
'TrigramForEarth': '\xe2\x98\xb7',
'HangulSyllableNelm': '\xeb\x84\xae',
'HangulSyllableNelb': '\xeb\x84\xaf',
'HangulSyllableSu': '\xec\x88\x98',
'HangulSyllableNelg': '\xeb\x84\xad',
'NarrowNobreakSpace': '\xe2\x80\xaf',
'CjkUnifiedIdeograph9eb8': '\xe9\xba\xb8',
'SquareHpa': '\xe3\x8d\xb1',
'CjkUnifiedIdeograph4bac': '\xe4\xae\xac',
'CjkUnifiedIdeograph4bab': '\xe4\xae\xab',
'CjkUnifiedIdeograph4baa': '\xe4\xae\xaa',
'CjkUnifiedIdeograph9eb9': '\xe9\xba\xb9',
'CjkUnifiedIdeograph4baf': '\xe4\xae\xaf',
'CjkUnifiedIdeograph4bae': '\xe4\xae\xae',
'CjkUnifiedIdeograph4bad': '\xe4\xae\xad',
'UnicodeCharacter0xf849': '\xef\xa1\x89',
'CjkUnifiedIdeograph9eb2': '\xe9\xba\xb2',
'ContainsWithOverbar': '\xe2\x8b\xbd',
'RightCornerBracket': '\xe3\x80\x8d',
'CjkUnifiedIdeograph9eb3': '\xe9\xba\xb3',
'IdeographicTelegraphSymbolForHourEighteen': '\xe3\x8d\xaa',
'DownwardsArrowToBar': '\xe2\xa4\x93',
'HexagramForTheAbysmalWater': '\xe4\xb7\x9c',
'CjkUnifiedIdeograph6d7e': '\xe6\xb5\xbe',
'CjkUnifiedIdeograph6d7d': '\xe6\xb5\xbd',
'CjkUnifiedIdeograph6d7f': '\xe6\xb5\xbf',
'CjkUnifiedIdeograph6d7a': '\xe6\xb5\xba',
'CjkUnifiedIdeograph6d7c': '\xe6\xb5\xbc',
'CjkUnifiedIdeograph6d7b': '\xe6\xb5\xbb',
'HangulSyllableGwaelh': '\xea\xb4\xa7',
'HangulSyllableGwaelm': '\xea\xb4\xa2',
'HangulSyllableGwaels': '\xea\xb4\xa4',
'HangulSyllableGwaelp': '\xea\xb4\xa6',
'TibetanMarkGterYigMgoUmGterTshegMa': '\xe0\xbc\x83',
'NorthEastAndSouthWestArrow': '\xe2\xa4\xa2',
'MalayalamLetterDha': '\xe0\xb4\xa7',
'UnicodeCharacter0xbbc': '\xe0\xae\xbc',
'UnicodeCharacter0xbbb': '\xe0\xae\xbb',
'UnicodeCharacter0xbba': '\xe0\xae\xba',
'UnicodeCharacter0xbbd': '\xe0\xae\xbd',
'PhagspaLetterTa': '\xea\xa1\x88',
'DownwardsAncora': '\xe2\xb8\x94',
'CjkUnifiedIdeograph93a2': '\xe9\x8e\xa2',
'HangulSyllableJwigg': '\xec\xa5\x92',
'FullwidthLatinSmallLetterY': '\xef\xbd\x99',
'LatinSmallLigatureFfi': '\xef\xac\x83',
'Intersection': '\xe2\x88\xa9',
'UnicodeCharacter0xf1ec': '\xef\x87\xac',
'DevanagariSignCandrabindu': '\xe0\xa4\x81',
'CjkUnifiedIdeograph6d75': '\xe6\xb5\xb5',
'KhmerLetterKha': '\xe1\x9e\x81',
'CjkUnifiedIdeograph6d77': '\xe6\xb5\xb7',
'CjkUnifiedIdeograph6d76': '\xe6\xb5\xb6',
'CjkUnifiedIdeograph6d71': '\xe6\xb5\xb1',
'CjkUnifiedIdeograph6d70': '\xe6\xb5\xb0',
'CjkUnifiedIdeograph6d73': '\xe6\xb5\xb3',
'CjkUnifiedIdeograph6d72': '\xe6\xb5\xb2',
'CjkUnifiedIdeograph7a08': '\xe7\xa8\x88',
'CjkUnifiedIdeograph7a09': '\xe7\xa8\x89',
'CjkUnifiedIdeograph6d79': '\xe6\xb5\xb9',
'CjkUnifiedIdeograph6d78': '\xe6\xb5\xb8',
'KhmerLetterKho': '\xe1\x9e\x83',
'CjkUnifiedIdeograph5e3e': '\xe5\xb8\xbe',
'CjkUnifiedIdeograph5e3d': '\xe5\xb8\xbd',
'CjkUnifiedIdeograph5e3f': '\xe5\xb8\xbf',
'CjkUnifiedIdeograph5e3a': '\xe5\xb8\xba',
'BalineseSignBisah': '\xe1\xac\x84',
'CjkUnifiedIdeograph5e3c': '\xe5\xb8\xbc',
'CjkUnifiedIdeograph5e3b': '\xe5\xb8\xbb',
'FullwidthLatinSmallLetterP': '\xef\xbd\x90',
'HangulSyllableKegs': '\xec\xbc\x83',
'FullwidthLatinSmallLetterQ': '\xef\xbd\x91',
'CjkUnifiedIdeograph3eb8': '\xe3\xba\xb8',
'FullwidthLatinSmallLetterR': '\xef\xbd\x92',
'CjkUnifiedIdeograph3eb9': '\xe3\xba\xb9',
'UnicodeCharacter0x2bac': '\xe2\xae\xac',
'UnicodeCharacter0x2bab': '\xe2\xae\xab',
'UnicodeCharacter0x2baa': '\xe2\xae\xaa',
'HangulSyllableOegg': '\xec\x99\xba',
'UnicodeCharacter0x2baf': '\xe2\xae\xaf',
'UnicodeCharacter0x2bae': '\xe2\xae\xae',
'UnicodeCharacter0x2bad': '\xe2\xae\xad',
'HangulSyllableMyelp': '\xeb\xaa\x9a',
'HangulSyllableMyels': '\xeb\xaa\x98',
'HangulSyllableBwal': '\xeb\xb4\x98',
'HangulSyllableBwak': '\xeb\xb4\xa8',
'HangulSyllableGgyaenj': '\xea\xba\xa5',
'LatinCapitalLetterSchwa': '\xc6\x8f',
'HangulSyllableGgyaeng': '\xea\xba\xb5',
'HangulSyllableBboegg': '\xeb\xbe\x96',
'HangulSyllableBwae': '\xeb\xb4\xac',
'HangulSyllableBwad': '\xeb\xb4\x97',
'HangulSyllableBwac': '\xeb\xb4\xa7',
'HangulSyllableBwab': '\xeb\xb4\xa1',
'UnicodeCharacter0xd903': '\xed\xa4\x83',
'UnicodeCharacter0xd902': '\xed\xa4\x82',
'HangulSyllableMyelb': '\xeb\xaa\x97',
'BlackRectangle': '\xe2\x96\xac',
'UnicodeCharacter0xcda': '\xe0\xb3\x9a',
'UnicodeCharacter0xcdb': '\xe0\xb3\x9b',
'UnicodeCharacter0xcdc': '\xe0\xb3\x9c',
'UnicodeCharacter0xcdd': '\xe0\xb3\x9d',
'HangulSyllableBboegs': '\xeb\xbe\x97',
'UnicodeCharacter0xcdf': '\xe0\xb3\x9f',
'HangulSyllableBwap': '\xeb\xb4\xaa',
'ArabicPoeticVerseSign': '\xd8\x8e',
'UnicodeCharacter0x2ba9': '\xe2\xae\xa9',
'UnicodeCharacter0x2ba8': '\xe2\xae\xa8',
'FullwidthLatinSmallLetterJ': '\xef\xbd\x8a',
'UnicodeCharacter0x2ba3': '\xe2\xae\xa3',
'UnicodeCharacter0x2ba2': '\xe2\xae\xa2',
'UnicodeCharacter0x2ba1': '\xe2\xae\xa1',
'UnicodeCharacter0x2ba0': '\xe2\xae\xa0',
'UnicodeCharacter0x2ba7': '\xe2\xae\xa7',
'UnicodeCharacter0x2ba6': '\xe2\xae\xa6',
'UnicodeCharacter0x2ba5': '\xe2\xae\xa5',
'UnicodeCharacter0x2ba4': '\xe2\xae\xa4',
'CjkUnifiedIdeograph5e35': '\xe5\xb8\xb5',
'CjkUnifiedIdeograph5e34': '\xe5\xb8\xb4',
'CjkUnifiedIdeograph5e37': '\xe5\xb8\xb7',
'CjkUnifiedIdeograph5e36': '\xe5\xb8\xb6',
'CjkUnifiedIdeograph5e31': '\xe5\xb8\xb1',
'CjkUnifiedIdeograph5e30': '\xe5\xb8\xb0',
'CjkUnifiedIdeograph5e33': '\xe5\xb8\xb3',
'CjkUnifiedIdeograph5e32': '\xe5\xb8\xb2',
'SquareMhz': '\xe3\x8e\x92',
'CjkUnifiedIdeograph5e39': '\xe5\xb8\xb9',
'CjkUnifiedIdeograph5e38': '\xe5\xb8\xb8',
'HangulSyllableRwibs': '\xeb\xa5\x8e',
'UnicodeCharacter0xcd8': '\xe0\xb3\x98',
'UnicodeCharacter0xcd9': '\xe0\xb3\x99',
'FullwidthLatinSmallLetterG': '\xef\xbd\x87',
'HebrewLetterShin': '\xd7\xa9',
'UnicodeCharacter0xcd0': '\xe0\xb3\x90',
'UnicodeCharacter0xcd1': '\xe0\xb3\x91',
'UnicodeCharacter0xcd2': '\xe0\xb3\x92',
'UnicodeCharacter0xcd3': '\xe0\xb3\x93',
'UnicodeCharacter0xcd4': '\xe0\xb3\x94',
'MongolianLetterManchuAliGaliJha': '\xe1\xa2\x9d',
'UnicodeCharacter0xcd7': '\xe0\xb3\x97',
'BalineseLetterDa': '\xe1\xac\xa4',
'HangulSyllableTuj': '\xed\x89\x82',
'HangulSyllableTuh': '\xed\x89\x87',
'HangulSyllableDabs': '\xeb\x8b\xb6',
'HangulSyllableTum': '\xed\x88\xbc',
'HangulSyllableTul': '\xed\x88\xb4',
'HangulSyllableTuc': '\xed\x89\x83',
'HangulSyllableTub': '\xed\x88\xbd',
'UnicodeCharacter0xd90f': '\xed\xa4\x8f',
'HeavyUpperRightshadowedWhiteRightwardsArrow': '\xe2\x9e\xae',
'UnicodeCharacter0xd90c': '\xed\xa4\x8c',
'HangulSyllableTud': '\xed\x88\xb3',
'CjkUnifiedIdeograph429c': '\xe4\x8a\x9c',
'CjkUnifiedIdeograph429b': '\xe4\x8a\x9b',
'CjkUnifiedIdeograph429a': '\xe4\x8a\x9a',
'HangulSyllableJolg': '\xec\xa1\xb9',
'UnicodeCharacter0xe9a5': '\xee\xa6\xa5',
'CjkUnifiedIdeograph429f': '\xe4\x8a\x9f',
'KangxiRadicalRightOpenBox': '\xe2\xbc\x95',
'CjkUnifiedIdeograph429d': '\xe4\x8a\x9d',
'UnicodeCharacter0xe9a9': '\xee\xa6\xa9',
'HangulSyllableJolm': '\xec\xa1\xba',
'HangulSyllableJolh': '\xec\xa1\xbf',
'HangulSyllableJolt': '\xec\xa1\xbd',
'ArabicSmallHighTah': '\xd8\x95',
'UnicodeCharacter0xd84c': '\xed\xa1\x8c',
'HangulSyllableJolp': '\xec\xa1\xbe',
'HangulSyllableJols': '\xec\xa1\xbc',
'CyrillicLetterPalochka': '\xd3\x80',
'HangulSyllableDdwalt': '\xeb\x98\xb9',
'HangulSyllableDdwals': '\xeb\x98\xb8',
'HangulSyllableDdwalp': '\xeb\x98\xba',
'HangulSyllableDdwalg': '\xeb\x98\xb5',
'HangulSyllableJwet': '\xec\xa5\x8d',
'HangulSyllableDdwalb': '\xeb\x98\xb7',
'HangulSyllableJjat': '\xec\xa7\xb5',
'YiSyllableXuo': '\xea\x91\xa6',
'UnicodeCharacter0xdedd': '\xed\xbb\x9d',
'HangulSyllableDdwalm': '\xeb\x98\xb6',
'UnicodeCharacter0xdedf': '\xed\xbb\x9f',
'UnicodeCharacter0xdeda': '\xed\xbb\x9a',
'KhmerIndependentVowelQooTypeTwo': '\xe1\x9e\xb2',
'HangulSyllableDdwalh': '\xeb\x98\xbb',
'NkoDigitEight': '\xdf\x88',
'HebrewLetterTsadi': '\xd7\xa6',
'UnicodeCharacter0xe3d7': '\xee\x8f\x97',
'UnicodeCharacter0xea18': '\xee\xa8\x98',
'CjkUnifiedIdeograph4293': '\xe4\x8a\x93',
'CjkUnifiedIdeograph4292': '\xe4\x8a\x92',
'CjkUnifiedIdeograph4291': '\xe4\x8a\x91',
'CjkUnifiedIdeograph4290': '\xe4\x8a\x90',
'CjkUnifiedIdeograph3964': '\xe3\xa5\xa4',
'CjkUnifiedIdeograph4296': '\xe4\x8a\x96',
'CjkUnifiedIdeograph4295': '\xe4\x8a\x95',
'CjkUnifiedIdeograph4294': '\xe4\x8a\x94',
'CjkUnifiedIdeograph4299': '\xe4\x8a\x99',
'CjkUnifiedIdeograph4298': '\xe4\x8a\x98',
'YiSyllableHmurx': '\xea\x82\xa7',
'CjkRadicalCsimplifiedSalt': '\xe2\xbb\xa7',
'UnicodeCharacter0xe3d0': '\xee\x8f\x90',
'EthiopicSyllableZwa': '\xe1\x8b\x9f',
'UnicodeCharacter0xded8': '\xed\xbb\x98',
'Scales': '\xe2\x9a\x96',
'UnicodeCharacter0xded5': '\xed\xbb\x95',
'UnicodeCharacter0xded4': '\xed\xbb\x94',
'UnicodeCharacter0xded7': '\xed\xbb\x97',
'UnicodeCharacter0xded6': '\xed\xbb\x96',
'UnicodeCharacter0xded1': '\xed\xbb\x91',
'CyrillicSmallLetterBarredO': '\xd3\xa9',
'LatinSmallLetterNWithDotBelow': '\xe1\xb9\x87',
'UnicodeCharacter0xded2': '\xed\xbb\x92',
'UnicodeCharacter0xf051': '\xef\x81\x91',
'UnicodeCharacter0xea10': '\xee\xa8\x90',
'UnicodeCharacter0xa6e2': '\xea\x9b\xa2',
'CjkUnifiedIdeograph53bd': '\xe5\x8e\xbd',
'CjkUnifiedIdeograph53be': '\xe5\x8e\xbe',
'CjkUnifiedIdeograph53bf': '\xe5\x8e\xbf',
'UnicodeCharacter0xea12': '\xee\xa8\x92',
'YiSyllableTit': '\xea\x84\x9a',
'CjkUnifiedIdeograph53ba': '\xe5\x8e\xba',
'CjkUnifiedIdeograph53bb': '\xe5\x8e\xbb',
'CjkUnifiedIdeograph53bc': '\xe5\x8e\xbc',
'YiSyllableTix': '\xea\x84\x9b',
'SquareBar': '\xe3\x8d\xb4',
'UnicodeCharacter0xe694': '\xee\x9a\x94',
'UnicodeCharacter0xa6e1': '\xea\x9b\xa1',
'UnicodeCharacter0xe692': '\xee\x9a\x92',
'UnicodeCharacter0xe693': '\xee\x9a\x93',
'UnicodeCharacter0xe690': '\xee\x9a\x90',
'UnicodeCharacter0xe691': '\xee\x9a\x91',
'CjkUnifiedIdeograph552b': '\xe5\x94\xab',
'CjkUnifiedIdeograph552c': '\xe5\x94\xac',
'CjkUnifiedIdeograph873f': '\xe8\x9c\xbf',
'CjkUnifiedIdeograph552a': '\xe5\x94\xaa',
'CjkUnifiedIdeograph552f': '\xe5\x94\xaf',
'CjkUnifiedIdeograph873a': '\xe8\x9c\xba',
'CjkUnifiedIdeograph552d': '\xe5\x94\xad',
'CjkUnifiedIdeograph552e': '\xe5\x94\xae',
'GeorgianLetterGhan': '\xe1\x83\xa6',
'CjkUnifiedIdeograph3ebd': '\xe3\xba\xbd',
'CjkUnifiedIdeograph8cf6': '\xe8\xb3\xb6',
'UnicodeCharacter0xf419': '\xef\x90\x99',
'UnicodeCharacter0xf418': '\xef\x90\x98',
'CjkUnifiedIdeograph4128': '\xe4\x84\xa8',
'LatinSmallLetterHWithStroke': '\xc4\xa7',
'CjkUnifiedIdeograph38fe': '\xe3\xa3\xbe',
'CjkUnifiedIdeograph38fd': '\xe3\xa3\xbd',
'CjkUnifiedIdeograph38ff': '\xe3\xa3\xbf',
'KhmerSymbolDappiiKoet': '\xe1\xa7\xac',
'CjkUnifiedIdeograph38fc': '\xe3\xa3\xbc',
'CjkUnifiedIdeograph38fb': '\xe3\xa3\xbb',
'LeftAngleBracket': '\xe3\x80\x88',
'UnicodeCharacter0xf573': '\xef\x95\xb3',
'OlChikiLetterAt': '\xe1\xb1\x9b',
'BalineseLetterKaMahaprana': '\xe1\xac\x94',
'UnicodeCharacter0xe3d3': '\xee\x8f\x93',
'CjkUnifiedIdeograph5522': '\xe5\x94\xa2',
'CjkUnifiedIdeograph5523': '\xe5\x94\xa3',
'CjkUnifiedIdeograph5520': '\xe5\x94\xa0',
'AplFunctionalSymbolOmega': '\xe2\x8d\xb5',
'CjkUnifiedIdeograph5526': '\xe5\x94\xa6',
'CjkUnifiedIdeograph5527': '\xe5\x94\xa7',
'CjkUnifiedIdeograph5524': '\xe5\x94\xa4',
'CjkUnifiedIdeograph5525': '\xe5\x94\xa5',
'HexagramForProgress': '\xe4\xb7\xa2',
'CjkUnifiedIdeograph5528': '\xe5\x94\xa8',
'CjkUnifiedIdeograph5529': '\xe5\x94\xa9',
'CjkUnifiedIdeograph8738': '\xe8\x9c\xb8',
'CjkUnifiedIdeograph8739': '\xe8\x9c\xb9',
'ArabicLigatureAinWithJeemWithMeemInitialForm': '\xef\xb7\x84',
'HangulSyllableSyalp': '\xec\x83\xb2',
'CjkUnifiedIdeograph53b4': '\xe5\x8e\xb4',
'CjkUnifiedIdeograph53b5': '\xe5\x8e\xb5',
'CjkUnifiedIdeograph53b6': '\xe5\x8e\xb6',
'CjkUnifiedIdeograph53b7': '\xe5\x8e\xb7',
'CjkUnifiedIdeograph53b0': '\xe5\x8e\xb0',
'CjkUnifiedIdeograph53b1': '\xe5\x8e\xb1',
'CjkUnifiedIdeograph53b2': '\xe5\x8e\xb2',
'CjkUnifiedIdeograph53b3': '\xe5\x8e\xb3',
'HangulSyllableSyalg': '\xec\x83\xad',
'LatinLetterLateralClick': '\xc7\x81',
'GujaratiLetterDha': '\xe0\xaa\xa7',
'CjkUnifiedIdeograph53b8': '\xe5\x8e\xb8',
'CjkUnifiedIdeograph53b9': '\xe5\x8e\xb9',
'UnicodeCharacter0xe69a': '\xee\x9a\x9a',
'CjkUnifiedIdeograph9ebb': '\xe9\xba\xbb',
'CjkUnifiedIdeograph9ebc': '\xe9\xba\xbc',
'HangulSyllableBbyaet': '\xeb\xbb\x8d',
'CjkUnifiedIdeograph38f9': '\xe3\xa3\xb9',
'CjkUnifiedIdeograph38f8': '\xe3\xa3\xb8',
'CjkUnifiedIdeograph38f5': '\xe3\xa3\xb5',
'CjkUnifiedIdeograph38f4': '\xe3\xa3\xb4',
'CjkUnifiedIdeograph38f7': '\xe3\xa3\xb7',
'CjkUnifiedIdeograph38f6': '\xe3\xa3\xb6',
'CjkUnifiedIdeograph38f1': '\xe3\xa3\xb1',
'CjkUnifiedIdeograph38f0': '\xe3\xa3\xb0',
'CjkUnifiedIdeograph38f3': '\xe3\xa3\xb3',
'CjkUnifiedIdeograph38f2': '\xe3\xa3\xb2',
'HangulSyllableDdus': '\xeb\x9a\xaf',
'HangulSyllableDdup': '\xeb\x9a\xb6',
'GlagoliticCapitalLetterTsi': '\xe2\xb0\x9c',
'HangulSyllableDdut': '\xeb\x9a\xb5',
'HangulSyllableDduj': '\xeb\x9a\xb2',
'HangulSyllableDduk': '\xeb\x9a\xb4',
'HangulSyllableDduh': '\xeb\x9a\xb7',
'HangulSyllableDdun': '\xeb\x9a\xa0',
'ArabicSmallWaw': '\xdb\xa5',
'HangulSyllableDdum': '\xeb\x9a\xac',
'HangulSyllableDdub': '\xeb\x9a\xad',
'HangulSyllableDduc': '\xeb\x9a\xb3',
'HangulSyllableDdug': '\xeb\x9a\x9d',
'HangulSyllableDdud': '\xeb\x9a\xa3',
'UnicodeCharacter0xed1c': '\xee\xb4\x9c',
'SyriacEsasaAbove': '\xdc\xbd',
'GreekSubscriptSmallLetterBeta': '\xe1\xb5\xa6',
'ArmenianSmallLigatureMenNow': '\xef\xac\x93',
'TibetanSubjoinedLetterHa': '\xe0\xbe\xb7',
'HangulSyllableNwalt': '\xeb\x86\xa1',
'YiSyllableSsyr': '\xea\x8d\x85',
'HangulSyllableNwalp': '\xeb\x86\xa2',
'HangulSyllableNwals': '\xeb\x86\xa0',
'IdeographicTelegraphSymbolForDayFive': '\xe3\x8f\xa4',
'UnicodeCharacter0xf570': '\xef\x95\xb0',
'BraillePatternDots12678': '\xe2\xa3\xa3',
'YiSyllableSsyt': '\xea\x8d\x80',
'ParallelWithHorizontalStroke': '\xe2\xab\xb2',
'HangulSyllableNwalg': '\xeb\x86\x9d',
'HangulSyllableNwalb': '\xeb\x86\x9f',
'KangxiRadicalInsect': '\xe2\xbe\x8d',
'HangulSyllableNwalh': '\xeb\x86\xa3',
'CanadianSyllabicsFwaa': '\xe1\x95\x9b',
'GurmukhiLetterMa': '\xe0\xa8\xae',
'UnicodeCharacter0xf675': '\xef\x99\xb5',
'OcrBeltBuckle': '\xe2\x91\x84',
'YiRadicalCyt': '\xea\x92\xa3',
'UnicodeCharacter0xf674': '\xef\x99\xb4',
'UnicodeCharacter0xf677': '\xef\x99\xb7',
'YiSyllableGgit': '\xea\x88\x94',
'CjkUnifiedIdeograph727a': '\xe7\x89\xba',
'CjkUnifiedIdeograph727b': '\xe7\x89\xbb',
'CjkUnifiedIdeograph727c': '\xe7\x89\xbc',
'CjkUnifiedIdeograph727d': '\xe7\x89\xbd',
'HeavyTeardropspokedAsterisk': '\xe2\x9c\xbd',
'CjkUnifiedIdeograph727f': '\xe7\x89\xbf',
'CjkUnifiedIdeograph60e8': '\xe6\x83\xa8',
'UnicodeCharacter0xeb1b': '\xee\xac\x9b',
'CjkUnifiedIdeograph396a': '\xe3\xa5\xaa',
'CjkUnifiedIdeograph396b': '\xe3\xa5\xab',
'CjkUnifiedIdeograph396c': '\xe3\xa5\xac',
'CjkUnifiedIdeograph396d': '\xe3\xa5\xad',
'CjkUnifiedIdeograph396e': '\xe3\xa5\xae',
'CjkUnifiedIdeograph396f': '\xe3\xa5\xaf',
'CjkUnifiedIdeograph60e6': '\xe6\x83\xa6',
'CjkUnifiedIdeograph3ded': '\xe3\xb7\xad',
'UnicodeCharacter0x93b': '\xe0\xa4\xbb',
'CjkUnifiedIdeograph3def': '\xe3\xb7\xaf',
'TamilVowelSignUu': '\xe0\xaf\x82',
'CjkUnifiedIdeograph3deb': '\xe3\xb7\xab',
'CjkUnifiedIdeograph3dec': '\xe3\xb7\xac',
'LatinSmallLetterPWithDotAbove': '\xe1\xb9\x97',
'CjkUnifiedIdeograph4739': '\xe4\x9c\xb9',
'UnicodeCharacter0xf00e': '\xef\x80\x8e',
'CjkUnifiedIdeograph4730': '\xe4\x9c\xb0',
'CjkUnifiedIdeograph4731': '\xe4\x9c\xb1',
'CjkUnifiedIdeograph4732': '\xe4\x9c\xb2',
'CjkUnifiedIdeograph4733': '\xe4\x9c\xb3',
'CjkUnifiedIdeograph4734': '\xe4\x9c\xb4',
'CjkUnifiedIdeograph4735': '\xe4\x9c\xb5',
'CjkUnifiedIdeograph4736': '\xe4\x9c\xb6',
'CjkUnifiedIdeograph4737': '\xe4\x9c\xb7',
'UnicodeCharacter0xf5cf': '\xef\x97\x8f',
'UnicodeCharacter0xf5cd': '\xef\x97\x8d',
'UnicodeCharacter0xf5ce': '\xef\x97\x8e',
'UnicodeCharacter0xf5cb': '\xef\x97\x8b',
'CjkUnifiedIdeograph60e1': '\xe6\x83\xa1',
'CyrillicCapitalLetterTwe': '\xea\x9a\x8c',
'CjkUnifiedIdeograph60e0': '\xe6\x83\xa0',
'VaiSyllableYo': '\xea\x97\x9a',
'EthiopicNumberNinety': '\xe1\x8d\xba',
'CjkUnifiedIdeograph3960': '\xe3\xa5\xa0',
'CjkUnifiedIdeograph3961': '\xe3\xa5\xa1',
'CjkUnifiedIdeograph3962': '\xe3\xa5\xa2',
'CjkUnifiedIdeograph3963': '\xe3\xa5\xa3',
'CjkUnifiedIdeograph3de8': '\xe3\xb7\xa8',
'CjkUnifiedIdeograph3965': '\xe3\xa5\xa5',
'CjkUnifiedIdeograph3966': '\xe3\xa5\xa6',
'CjkUnifiedIdeograph3967': '\xe3\xa5\xa7',
'CjkUnifiedIdeograph3de4': '\xe3\xb7\xa4',
'CjkUnifiedIdeograph3969': '\xe3\xa5\xa9',
'CjkUnifiedIdeograph3de6': '\xe3\xb7\xa6',
'CjkUnifiedIdeograph3de7': '\xe3\xb7\xa7',
'CjkUnifiedIdeograph3de0': '\xe3\xb7\xa0',
'CjkUnifiedIdeograph3de1': '\xe3\xb7\xa1',
'CjkUnifiedIdeograph3de2': '\xe3\xb7\xa2',
'CjkUnifiedIdeograph3de3': '\xe3\xb7\xa3',
'CjkUnifiedIdeograph7270': '\xe7\x89\xb0',
'CjkUnifiedIdeograph7271': '\xe7\x89\xb1',
'CjkUnifiedIdeograph7272': '\xe7\x89\xb2',
'SymbolForBell': '\xe2\x90\x87',
'CjkUnifiedIdeograph7274': '\xe7\x89\xb4',
'CjkUnifiedIdeograph7275': '\xe7\x89\xb5',
'CjkUnifiedIdeograph7276': '\xe7\x89\xb6',
'IdeographicDescriptionCharacterSurroundFromAbove': '\xe2\xbf\xb5',
'CjkUnifiedIdeograph7278': '\xe7\x89\xb8',
'CjkUnifiedIdeograph7279': '\xe7\x89\xb9',
'LatinCapitalLetterDWithTopbar': '\xc6\x8b',
'CjkUnifiedIdeograph670a': '\xe6\x9c\x8a',
'UnicodeCharacter0xf5c6': '\xef\x97\x86',
'UnicodeCharacter0xf5c7': '\xef\x97\x87',
'UnicodeCharacter0xf5c4': '\xef\x97\x84',
'UnicodeCharacter0xf5c5': '\xef\x97\x85',
'UnicodeCharacter0xf5c2': '\xef\x97\x82',
'MalayalamLetterNya': '\xe0\xb4\x9e',
'UnicodeCharacter0xf5c0': '\xef\x97\x80',
'UnicodeCharacter0xd841': '\xed\xa1\x81',
'UnicodeCharacter0xebd1': '\xee\xaf\x91',
'HangulSyllableRweonj': '\xeb\xa4\x89',
'UnicodeCharacter0xf5c8': '\xef\x97\x88',
'UnicodeCharacter0xf5c9': '\xef\x97\x89',
'UnicodeCharacter0xd9ca': '\xed\xa7\x8a',
'CjkUnifiedIdeograph473a': '\xe4\x9c\xba',
'CjkUnifiedIdeograph473b': '\xe4\x9c\xbb',
'CjkUnifiedIdeograph473c': '\xe4\x9c\xbc',
'CjkUnifiedIdeograph473d': '\xe4\x9c\xbd',
'CjkUnifiedIdeograph473e': '\xe4\x9c\xbe',
'CjkUnifiedIdeograph473f': '\xe4\x9c\xbf',
'HangulJungseongYi': '\xe1\x85\xb4',
'UnicodeCharacter0xd9cc': '\xed\xa7\x8c',
'HangulJungseongYo': '\xe1\x85\xad',
'UnicodeCharacter0xd846': '\xed\xa1\x86',
'VaiSyllableThee': '\xea\x94\x8f',
'HangulSyllableDdyuj': '\xeb\x9c\xa2',
'CanadianSyllabicsCarrierDzu': '\xe1\x99\x88',
'CyrillicSmallLetterPsi': '\xd1\xb1',
'UnicodeCharacter0xd847': '\xed\xa1\x87',
'UnicodeCharacter0xdd57': '\xed\xb5\x97',
'UnicodeCharacter0xdd56': '\xed\xb5\x96',
'HangulJungseongYe': '\xe1\x85\xa8',
'DevanagariLetterGha': '\xe0\xa4\x98',
'UnicodeCharacter0xd848': '\xed\xa1\x88',
'CanadianSyllabicsCarrierDzo': '\xe1\x99\x89',
'CanadianSyllabicsCarrierDzi': '\xe1\x99\x8c',
'UnicodeCharacter0xd849': '\xed\xa1\x89',
'CanadianSyllabicsCarrierDze': '\xe1\x99\x8a',
'CanadianSyllabicsCarrierDza': '\xe1\x99\x8d',
'HangulJungseongYu': '\xe1\x85\xb2',
'CjkUnifiedIdeograph5067': '\xe5\x81\xa7',
'CjkUnifiedIdeograph5066': '\xe5\x81\xa6',
'CjkUnifiedIdeograph5065': '\xe5\x81\xa5',
'CjkUnifiedIdeograph5064': '\xe5\x81\xa4',
'CjkUnifiedIdeograph5063': '\xe5\x81\xa3',
'CjkUnifiedIdeograph5062': '\xe5\x81\xa2',
'CjkUnifiedIdeograph5061': '\xe5\x81\xa1',
'CjkUnifiedIdeograph5060': '\xe5\x81\xa0',
'HangulSyllableRyep': '\xeb\xa1\x9a',
'HangulSyllableRyes': '\xeb\xa1\x93',
'CjkUnifiedIdeograph98db': '\xe9\xa3\x9b',
'HangulSyllableDdyak': '\xeb\x96\x80',
'CjkUnifiedIdeograph9a64': '\xe9\xa9\xa4',
'CjkUnifiedIdeograph5069': '\xe5\x81\xa9',
'CjkUnifiedIdeograph5068': '\xe5\x81\xa8',
'ThaanaLetterWaavu': '\xde\xa5',
'CjkUnifiedIdeograph9956': '\xe9\xa5\x96',
'CopticCapitalLetterKhi': '\xe2\xb2\xac',
'CjkUnifiedIdeograph669b': '\xe6\x9a\x9b',
'RightWhiteTortoiseShellBracket': '\xe3\x80\x99',
'CjkUnifiedIdeograph9952': '\xe9\xa5\x92',
'CjkUnifiedIdeograph9951': '\xe9\xa5\x91',
'CjkUnifiedIdeograph9950': '\xe9\xa5\x90',
'UnicodeCharacter0xf6ca': '\xef\x9b\x8a',
'HangulSyllableDyaenj': '\xeb\x8c\xbd',
'HangulSyllableRyec': '\xeb\xa1\x97',
'HangulSyllableRyeb': '\xeb\xa1\x91',
'HangulSyllableBbyaec': '\xeb\xbb\x8b',
'HangulSyllableRyed': '\xeb\xa1\x87',
'CjkUnifiedIdeograph9959': '\xe9\xa5\x99',
'CjkUnifiedIdeograph9958': '\xe9\xa5\x98',
'CjkUnifiedIdeograph9a60': '\xe9\xa9\xa0',
'UnicodeCharacter0xda99': '\xed\xaa\x99',
'UnicodeCharacter0xea5a': '\xee\xa9\x9a',
'HangulSyllableByunh': '\xeb\xb7\xb6',
'HangulSyllableByunj': '\xeb\xb7\xb5',
'HebrewLetterTsadiWithDagesh': '\xef\xad\x86',
'UnicodeCharacter0xdd5c': '\xed\xb5\x9c',
'UnicodeCharacter0xdd5b': '\xed\xb5\x9b',
'UnicodeCharacter0xdd5a': '\xed\xb5\x9a',
'HangulSyllableByung': '\xeb\xb8\x85',
'TeluguLetterCha': '\xe0\xb0\x9b',
'HangulSyllableHyobs': '\xed\x9a\xba',
'UnicodeCharacter0xdd5d': '\xed\xb5\x9d',
'CjkUnifiedIdeograph9d9a': '\xe9\xb6\x9a',
'CjkUnifiedIdeograph995f': '\xe9\xa5\x9f',
'CjkUnifiedIdeograph995e': '\xe9\xa5\x9e',
'CjkUnifiedIdeograph995d': '\xe9\xa5\x9d',
'CjkUnifiedIdeograph995c': '\xe9\xa5\x9c',
'CjkUnifiedIdeograph995b': '\xe9\xa5\x9b',
'CjkUnifiedIdeograph995a': '\xe9\xa5\x9a',
'UnicodeCharacter0xf6c1': '\xef\x9b\x81',
'UnicodeCharacter0xf6c0': '\xef\x9b\x80',
'UnicodeCharacter0xf6c3': '\xef\x9b\x83',
'UnicodeCharacter0xf6c2': '\xef\x9b\x82',
'TibetanLetterWa': '\xe0\xbd\x9d',
'CopticSmallLetterDialectpKapa': '\xe2\xb2\xb9',
'UnicodeCharacter0xf6c7': '\xef\x9b\x87',
'UnicodeCharacter0xda93': '\xed\xaa\x93',
'ArabicLigatureFehWithKhahInitialForm': '\xef\xb3\x80',
'CjkUnifiedIdeograph506f': '\xe5\x81\xaf',
'CjkUnifiedIdeograph506e': '\xe5\x81\xae',
'CjkUnifiedIdeograph506d': '\xe5\x81\xad',
'CjkUnifiedIdeograph506c': '\xe5\x81\xac',
'CjkUnifiedIdeograph506b': '\xe5\x81\xab',
'CjkUnifiedIdeograph506a': '\xe5\x81\xaa',
'UnicodeCharacter0xda91': '\xed\xaa\x91',
'SquareHerutu': '\xe3\x8c\xb9',
'CjkUnifiedIdeograph6606': '\xe6\x98\x86',
'CjkUnifiedIdeograph6607': '\xe6\x98\x87',
'CjkUnifiedIdeograph6604': '\xe6\x98\x84',
'CjkUnifiedIdeograph6605': '\xe6\x98\x85',
'CjkUnifiedIdeograph6602': '\xe6\x98\x82',
'CjkUnifiedIdeograph6603': '\xe6\x98\x83',
'CjkUnifiedIdeograph6600': '\xe6\x98\x80',
'CjkUnifiedIdeograph6601': '\xe6\x98\x81',
'CjkUnifiedIdeograph3e7f': '\xe3\xb9\xbf',
'CjkUnifiedIdeograph3e7e': '\xe3\xb9\xbe',
'CjkUnifiedIdeograph3e7d': '\xe3\xb9\xbd',
'CjkUnifiedIdeograph3e7c': '\xe3\xb9\xbc',
'CjkUnifiedIdeograph3e7b': '\xe3\xb9\xbb',
'CjkUnifiedIdeograph3e7a': '\xe3\xb9\xba',
'CjkUnifiedIdeograph6609': '\xe6\x98\x89',
'CjkUnifiedIdeograph8fdd': '\xe8\xbf\x9d',
'CjkUnifiedIdeograph8fde': '\xe8\xbf\x9e',
'CjkUnifiedIdeograph8fdf': '\xe8\xbf\x9f',
'UnicodeCharacter0xf67f': '\xef\x99\xbf',
'CjkUnifiedIdeograph8fda': '\xe8\xbf\x9a',
'CjkUnifiedIdeograph8fdb': '\xe8\xbf\x9b',
'CjkUnifiedIdeograph8fdc': '\xe8\xbf\x9c',
'UnicodeCharacter0xf67a': '\xef\x99\xba',
'CyrillicSmallLetterBe': '\xd0\xb1',
'YiRadicalHmo': '\xea\x92\xad',
'UnicodeCharacter0xf67c': '\xef\x99\xbc',
'NeitherLessthanNorEqualTo': '\xe2\x89\xb0',
'MeasuredAngleWithOpenArmEndingInArrowPointingUpAndLeft': '\xe2\xa6\xa9',
'UnicodeCharacter0xf67b': '\xef\x99\xbb',
'HangulSyllableGgyaess': '\xea\xba\xb4',
'LatinLetterSmallCapitalBarredB': '\xe1\xb4\x83',
'CjkUnifiedIdeograph972c': '\xe9\x9c\xac',
'LatinSmallLetterLWithCaron': '\xc4\xbe',
'CjkUnifiedIdeograph60ee': '\xe6\x83\xae',
'CjkUnifiedIdeograph8fd4': '\xe8\xbf\x94',
'CjkUnifiedIdeograph8fd5': '\xe8\xbf\x95',
'CjkUnifiedIdeograph8fd6': '\xe8\xbf\x96',
'CjkUnifiedIdeograph60ed': '\xe6\x83\xad',
'CjkUnifiedIdeograph8fd0': '\xe8\xbf\x90',
'CjkUnifiedIdeograph8fd1': '\xe8\xbf\x91',
'CjkUnifiedIdeograph8fd2': '\xe8\xbf\x92',
'CjkUnifiedIdeograph8fd3': '\xe8\xbf\x93',
'CjkUnifiedIdeograph60ec': '\xe6\x83\xac',
'CjkUnifiedIdeograph8fd8': '\xe8\xbf\x98',
'CjkUnifiedIdeograph8fd9': '\xe8\xbf\x99',
'SinhalaLetterTaalujaSayanna': '\xe0\xb7\x81',
'CjkUnifiedIdeograph60eb': '\xe6\x83\xab',
'CjkUnifiedIdeograph660f': '\xe6\x98\x8f',
'CjkUnifiedIdeograph660d': '\xe6\x98\x8d',
'CjkUnifiedIdeograph660e': '\xe6\x98\x8e',
'CjkUnifiedIdeograph660b': '\xe6\x98\x8b',
'CjkUnifiedIdeograph60ea': '\xe6\x83\xaa',
'CjkUnifiedIdeograph3e79': '\xe3\xb9\xb9',
'CjkUnifiedIdeograph3e78': '\xe3\xb9\xb8',
'CjkUnifiedIdeograph3e77': '\xe3\xb9\xb7',
'CjkUnifiedIdeograph3e76': '\xe3\xb9\xb6',
'CjkUnifiedIdeograph3e75': '\xe3\xb9\xb5',
'CjkUnifiedIdeograph3e74': '\xe3\xb9\xb4',
'CjkUnifiedIdeograph3e73': '\xe3\xb9\xb3',
'CjkUnifiedIdeograph3e72': '\xe3\xb9\xb2',
'CjkUnifiedIdeograph3e71': '\xe3\xb9\xb1',
'CjkUnifiedIdeograph3e70': '\xe3\xb9\xb0',
'HangulSyllableSyeg': '\xec\x85\xb1',
'HangulSyllableSyed': '\xec\x85\xb7',
'HangulSyllableSyeb': '\xec\x86\x81',
'HangulSyllableSyec': '\xec\x86\x87',
'ArabicLigatureDadWithAlefMaksuraFinalForm': '\xef\xb4\xa3',
'HangulSyllableSyen': '\xec\x85\xb4',
'HangulSyllableSyeo': '\xec\x85\x94',
'HangulSyllableSyel': '\xec\x85\xb8',
'HangulSyllableSyem': '\xec\x86\x80',
'HangulSyllableSyej': '\xec\x86\x86',
'BoxDrawingsLightRight': '\xe2\x95\xb6',
'HangulSyllableSyeh': '\xec\x86\x8b',
'HangulSyllableSyet': '\xec\x86\x89',
'HebrewLetterZayinWithDagesh': '\xef\xac\xb6',
'HangulSyllableSyes': '\xec\x86\x83',
'HangulSyllableSyep': '\xec\x86\x8a',
'UnicodeCharacter0xe0fa': '\xee\x83\xba',
'MongolianLetterManchuAliGaliCya': '\xe1\xa2\xa3',
'LatinSmallLetterAeWithAcute': '\xc7\xbd',
'HangulSyllableJjal': '\xec\xa7\xa4',
'HangulSyllableJwilb': '\xec\xa5\x9b',
'BalineseMusicalSymbolCombiningKempli': '\xe1\xad\xae',
'HangulSyllableJyulp': '\xec\xa5\xba',
'Smile': '\xe2\x8c\xa3',
'HangulChoseongSiospieupkiyeok': '\xe1\x84\xb3',
'CjkUnifiedIdeograph7db8': '\xe7\xb6\xb8',
'HalfwidthHangulLetterTikeut': '\xef\xbe\xa7',
'HangulSyllableJwilg': '\xec\xa5\x99',
'HangulSyllableJwilh': '\xec\xa5\x9f',
'HangulSyllableJyulm': '\xec\xa5\xb6',
'UnicodeCharacter0xe0fc': '\xee\x83\xbc',
'HangulSyllableRwenj': '\xeb\xa4\xa5',
'UnicodeCharacter0xfde6': '\xef\xb7\xa6',
'HangulSyllableGid': '\xea\xb8\xb7',
'CjkUnifiedIdeograph9a6f': '\xe9\xa9\xaf',
'HangulSyllableGig': '\xea\xb8\xb1',
'CjkUnifiedIdeograph6f75': '\xe6\xbd\xb5',
'HangulSyllableGib': '\xea\xb9\x81',
'HangulSyllableGic': '\xea\xb9\x87',
'HangulSyllableGil': '\xea\xb8\xb8',
'HangulSyllableGim': '\xea\xb9\x80',
'HangulSyllableGin': '\xea\xb8\xb4',
'HangulSyllableGih': '\xea\xb9\x8b',
'HangulSyllableGij': '\xea\xb9\x86',
'HangulSyllableGik': '\xea\xb9\x88',
'HangulSyllableGit': '\xea\xb9\x89',
'CjkUnifiedIdeograph7db1': '\xe7\xb6\xb1',
'CjkCompatibilityIdeographf93a': '\xef\xa4\xba',
'KannadaLetterFa': '\xe0\xb3\x9e',
'HangulSyllableCwet': '\xec\xb7\xa5',
'HangulSyllableGis': '\xea\xb9\x83',
'CjkUnifiedIdeograph6695': '\xe6\x9a\x95',
'CyrillicCapitalLetterElWithTail': '\xd3\x85',
'UnicodeCharacter0xe522': '\xee\x94\xa2',
'ArabicLetterDad': '\xd8\xb6',
'UnicodeCharacter0xe523': '\xee\x94\xa3',
'HangulSyllablePeng': '\xed\x8e\xad',
'CjkUnifiedIdeograph7db5': '\xe7\xb6\xb5',
'CjkUnifiedIdeograph9a6a': '\xe9\xa9\xaa',
'HalfwidthKatakanaSemivoicedSoundMark': '\xef\xbe\x9f',
'UnicodeCharacter0xe525': '\xee\x94\xa5',
'TapeDrive': '\xe2\x9c\x87',
'HangulSyllableJwils': '\xec\xa5\x9c',
'UnicodeCharacter0xe526': '\xee\x94\xa6',
'HangulSyllableJwilt': '\xec\xa5\x9d',
'UnicodeCharacter0xe527': '\xee\x94\xa7',
'UnicodeCharacter0xea5c': '\xee\xa9\x9c',
'CjkUnifiedIdeograph98de': '\xe9\xa3\x9e',
'CjkUnifiedIdeograph98dd': '\xe9\xa3\x9d',
'KatakanaLetterKa': '\xe3\x82\xab',
'CjkUnifiedIdeograph98df': '\xe9\xa3\x9f',
'CjkUnifiedIdeograph98da': '\xe9\xa3\x9a',
'KatakanaLetterKe': '\xe3\x82\xb1',
'DoubleCircledDigitThree': '\xe2\x93\xb7',
'KatakanaLetterKi': '\xe3\x82\xad',
'KatakanaLetterKo': '\xe3\x82\xb3',
'CjkUnifiedIdeograph768c': '\xe7\x9a\x8c',
'UnicodeCharacter0xe0d9': '\xee\x83\x99',
'KatakanaLetterKu': '\xe3\x82\xaf',
'UnicodeCharacter0xd94a': '\xed\xa5\x8a',
'UnicodeCharacter0xd9c8': '\xed\xa7\x88',
'HangulSyllableHeugs': '\xed\x9d\x93',
'UnicodeCharacter0xa9e0': '\xea\xa7\xa0',
'CjkUnifiedIdeograph7d7b': '\xe7\xb5\xbb',
'CjkUnifiedIdeograph7d7c': '\xe7\xb5\xbc',
'HangulSyllableJyult': '\xec\xa5\xb9',
'CjkUnifiedIdeograph7d7a': '\xe7\xb5\xba',
'CjkUnifiedIdeograph7d7f': '\xe7\xb5\xbf',
'HangulSyllableJyuls': '\xec\xa5\xb8',
'CjkUnifiedIdeograph7d7d': '\xe7\xb5\xbd',
'CjkUnifiedIdeograph7d7e': '\xe7\xb5\xbe',
'CjkUnifiedIdeograph6f73': '\xe6\xbd\xb3',
'CjkUnifiedIdeograph6f72': '\xe6\xbd\xb2',
'CjkUnifiedIdeograph6f71': '\xe6\xbd\xb1',
'CjkUnifiedIdeograph6f70': '\xe6\xbd\xb0',
'CjkUnifiedIdeograph6f77': '\xe6\xbd\xb7',
'CjkUnifiedIdeograph6f76': '\xe6\xbd\xb6',
'HeavyRightpointingAngleBracketOrnament': '\xe2\x9d\xb1',
'CjkUnifiedIdeograph6f74': '\xe6\xbd\xb4',
'HangulSyllableRweng': '\xeb\xa4\xb5',
'CjkUnifiedIdeograph6f79': '\xe6\xbd\xb9',
'CjkUnifiedIdeograph6f78': '\xe6\xbd\xb8',
'HangulSyllableJyulb': '\xec\xa5\xb7',
'HangulSyllableOelg': '\xec\x9a\x81',
'HangulSyllableOelm': '\xec\x9a\x82',
'HangulSyllableSsaenh': '\xec\x8c\x9a',
'HangulSyllableSsaenj': '\xec\x8c\x99',
'HangulSyllableOelh': '\xec\x9a\x87',
'SaurashtraLetterTha': '\xea\xa2\xa2',
'HangulSyllableDdyum': '\xeb\x9c\x9c',
'AplFunctionalSymbolStarDiaeresis': '\xe2\x8d\xa3',
'UnicodeCharacter0xf54e': '\xef\x95\x8e',
'HangulSyllableSsaeng': '\xec\x8c\xa9',
'VaiSyllableGu': '\xea\x96\xb6',
'CjkUnifiedIdeograph98d5': '\xe9\xa3\x95',
'CjkUnifiedIdeograph98d4': '\xe9\xa3\x94',
'CjkUnifiedIdeograph98d7': '\xe9\xa3\x97',
'CjkUnifiedIdeograph98d6': '\xe9\xa3\x96',
'CjkUnifiedIdeograph98d1': '\xe9\xa3\x91',
'CjkUnifiedIdeograph98d0': '\xe9\xa3\x90',
'CjkUnifiedIdeograph98d3': '\xe9\xa3\x93',
'CjkUnifiedIdeograph98d2': '\xe9\xa3\x92',
'HangulSyllableOelt': '\xec\x9a\x85',
'CjkUnifiedIdeograph9723': '\xe9\x9c\xa3',
'CjkUnifiedIdeograph98d9': '\xe9\xa3\x99',
'CjkUnifiedIdeograph98d8': '\xe9\xa3\x98',
'VulgarFractionSevenEighths': '\xe2\x85\x9e',
'[8]': '\xe2\x82\x88',
'CjkUnifiedIdeograph6f7c': '\xe6\xbd\xbc',
'CjkUnifiedIdeograph6f7b': '\xe6\xbd\xbb',
'CjkUnifiedIdeograph6f7a': '\xe6\xbd\xba',
'BallotBoxWithCheck': '\xe2\x98\x91',
'CjkUnifiedIdeograph6f7e': '\xe6\xbd\xbe',
'CjkUnifiedIdeograph6f7d': '\xe6\xbd\xbd',
'HangulSyllableDeols': '\xeb\x8d\xa0',
'HangulSyllableDeolp': '\xeb\x8d\xa2',
'RightwardsArrowtail': '\xe2\xa4\x9a',
'HangulSyllableDeolt': '\xeb\x8d\xa1',
'BoxDrawingsRightHeavyAndLeftDownLight': '\xe2\x94\xae',
'CjkUnifiedIdeograph7d78': '\xe7\xb5\xb8',
'CjkUnifiedIdeograph7d79': '\xe7\xb5\xb9',
'HangulSyllableDeolm': '\xeb\x8d\x9e',
'CjkUnifiedIdeograph7d72': '\xe7\xb5\xb2',
'CjkUnifiedIdeograph7d73': '\xe7\xb5\xb3',
'CjkUnifiedIdeograph7d70': '\xe7\xb5\xb0',
'CjkUnifiedIdeograph7d71': '\xe7\xb5\xb1',
'CjkUnifiedIdeograph7d76': '\xe7\xb5\xb6',
'CjkUnifiedIdeograph7d77': '\xe7\xb5\xb7',
'CjkUnifiedIdeograph7d74': '\xe7\xb5\xb4',
'CjkUnifiedIdeograph7d75': '\xe7\xb5\xb5',
'HangulSyllableSyebs': '\xec\x86\x82',
'HebrewLetterWideFinalMem': '\xef\xac\xa6',
'TaiLeLetterU': '\xe1\xa5\xa7',
'HangulSyllableTyols': '\xed\x88\x9c',
'HangulSyllableTyolp': '\xed\x88\x9e',
'CjkUnifiedIdeograph7216': '\xe7\x88\x96',
'ParenthesizedIdeographFour': '\xe3\x88\xa3',
'CjkCompatibilityIdeographf936': '\xef\xa4\xb6',
'LeftBarbUpRightBarbDownHarpoon': '\xe2\xa5\x8a',
'VerticalBarTripleRightTurnstile': '\xe2\xab\xa2',
'CjkUnifiedIdeograph4f27': '\xe4\xbc\xa7',
'SquareWithHorizontalFill': '\xe2\x96\xa4',
'CjkUnifiedIdeograph4f25': '\xe4\xbc\xa5',
'TaiLeLetterE': '\xe1\xa5\xab',
'CjkUnifiedIdeograph4f23': '\xe4\xbc\xa3',
'CjkUnifiedIdeograph4f20': '\xe4\xbc\xa0',
'CjkUnifiedIdeograph4f21': '\xe4\xbc\xa1',
'TaiLeLetterI': '\xe1\xa5\xa4',
'HangulSyllableHyibs': '\xed\x9d\xbe',
'HangulSyllableTyolm': '\xed\x88\x9a',
'TaiLeLetterO': '\xe1\xa5\xa9',
'CjkUnifiedIdeograph4f29': '\xe4\xbc\xa9',
'EthiopicSyllableXwi': '\xe1\x8a\x8a',
'ArabicLigatureYehWithHamzaAboveWithOeIsolatedForm': '\xef\xaf\xb2',
'CjkUnifiedIdeograph9f53': '\xe9\xbd\x93',
'KangxiRadicalDistinguish': '\xe2\xbe\xa4',
'HangulSyllableCwes': '\xec\xb7\x9f',
'EthiopicSyllableXwa': '\xe1\x8a\x88',
'BugineseLetterHa': '\xe1\xa8\x96',
'EthiopicSyllableXwe': '\xe1\x8a\x8d',
'HangulSyllableOelp': '\xec\x9a\x86',
'CjkUnifiedIdeograph56b9': '\xe5\x9a\xb9',
'CjkUnifiedIdeograph56b8': '\xe5\x9a\xb8',
'HangulJongseongRieulkhieukh': '\xe1\x87\x98',
'CjkUnifiedIdeograph56b4': '\xe5\x9a\xb4',
'CjkUnifiedIdeograph56b7': '\xe5\x9a\xb7',
'LatinCapitalLetterEWithBreve': '\xc4\x94',
'CjkUnifiedIdeograph56b1': '\xe5\x9a\xb1',
'CjkUnifiedIdeograph56b0': '\xe5\x9a\xb0',
'CjkUnifiedIdeograph56b3': '\xe5\x9a\xb3',
'KhmerSymbolLekAttakPrammuoy': '\xe1\x9f\xb6',
'CanadianSyllabicsCarrierPa': '\xe1\x97\xad',
'UnicodeCharacter0xdf44': '\xed\xbd\x84',
'CjkUnifiedIdeograph4f2d': '\xe4\xbc\xad',
'CjkUnifiedIdeograph4f2e': '\xe4\xbc\xae',
'CanadianSyllabicsCarrierPe': '\xe1\x97\xaa',
'CjkUnifiedIdeograph4f2c': '\xe4\xbc\xac',
'CjkUnifiedIdeograph4f2a': '\xe4\xbc\xaa',
'CanadianSyllabicsCarrierPi': '\xe1\x97\xac',
'HangulSyllableNeult': '\xeb\x8a\x9d',
'HangulSyllableNeuls': '\xeb\x8a\x9c',
'CanadianSyllabicsCarrierPo': '\xe1\x97\xa9',
'HangulSyllableNeulm': '\xeb\x8a\x9a',
'CanadianSyllabicsCarrierPu': '\xe1\x97\xa8',
'HangulSyllableNeulh': '\xeb\x8a\x9f',
'HangulJongseongNieunhieuh': '\xe1\x86\xad',
'HangulSyllableNeulg': '\xeb\x8a\x99',
'HangulSyllableNeulb': '\xeb\x8a\x9b',
'CjkUnifiedIdeograph6f7f': '\xe6\xbd\xbf',
'LatinCapitalLetterIWithAcute': '\xc3\x8d',
'YiSyllableShuop': '\xea\x8e\xb1',
'UnicodeCharacter0xf54d': '\xef\x95\x8d',
'GreekSmallLetterAlphaWithOxiaAndYpogegrammeni': '\xe1\xbe\xb4',
'CjkUnifiedIdeograph56be': '\xe5\x9a\xbe',
'CjkUnifiedIdeograph56bd': '\xe5\x9a\xbd',
'HeavySingleTurnedCommaQuotationMarkOrnament': '\xe2\x9d\x9b',
'CjkUnifiedIdeograph56bf': '\xe5\x9a\xbf',
'CjkUnifiedIdeograph56ba': '\xe5\x9a\xba',
'CjkUnifiedIdeograph56bc': '\xe5\x9a\xbc',
'CjkUnifiedIdeograph56bb': '\xe5\x9a\xbb',
'GurmukhiLetterVa': '\xe0\xa8\xb5',
'HangulSyllableJjyej': '\xec\xaa\xb6',
'GreekCapitalLetterAlphaWithMacron': '\xe1\xbe\xb9',
'CircledNumberFortyTwo': '\xe3\x8a\xb7',
'LeftwardsTwoheadedArrowWithTailWithVerticalStroke': '\xe2\xac\xbc',
'UnicodeCharacter0xe52b': '\xee\x94\xab',
'CjkCompatibilityIdeographf93c': '\xef\xa4\xbc',
'CjkUnifiedIdeograph6ee6': '\xe6\xbb\xa6',
'CjkUnifiedIdeograph6ee7': '\xe6\xbb\xa7',
'CjkUnifiedIdeograph6ee4': '\xe6\xbb\xa4',
'CjkUnifiedIdeograph6ee5': '\xe6\xbb\xa5',
'CjkUnifiedIdeograph6ee2': '\xe6\xbb\xa2',
'CjkUnifiedIdeograph6ee3': '\xe6\xbb\xa3',
'CjkUnifiedIdeograph6ee0': '\xe6\xbb\xa0',
'CjkUnifiedIdeograph6ee1': '\xe6\xbb\xa1',
'UnicodeCharacter0xe52d': '\xee\x94\xad',
'CjkUnifiedIdeograph6ee8': '\xe6\xbb\xa8',
'CjkUnifiedIdeograph6ee9': '\xe6\xbb\xa9',
'HangulSyllableSyul': '\xec\x8a\x90',
'EthiopicSyllableBaa': '\xe1\x89\xa3',
'CjkUnifiedIdeograph70aa': '\xe7\x82\xaa',
'CjkUnifiedIdeograph7eee': '\xe7\xbb\xae',
'CjkUnifiedIdeograph7eed': '\xe7\xbb\xad',
'CjkUnifiedIdeograph7eef': '\xe7\xbb\xaf',
'CjkUnifiedIdeograph7eea': '\xe7\xbb\xaa',
'CjkUnifiedIdeograph7eec': '\xe7\xbb\xac',
'CjkUnifiedIdeograph7eeb': '\xe7\xbb\xab',
'YiSyllableBbie': '\xea\x81\x9c',
'UnicodeCharacter0xdad3': '\xed\xab\x93',
'UnicodeCharacter0xf61c': '\xef\x98\x9c',
'CjkUnifiedIdeograph70ad': '\xe7\x82\xad',
'HangulSyllablePwaebs': '\xed\x90\xb6',
'HangulSyllableDeolh': '\xeb\x8d\xa3',
'YiSyllableBbit': '\xea\x81\x96',
'HangulSyllableCwilt': '\xec\xb7\xb5',
'YiSyllableBbip': '\xea\x81\x99',
'HanunooLetterLa': '\xe1\x9c\xae',
'YiSyllableBbix': '\xea\x81\x97',
'HangulSyllableNyeobs': '\xeb\x85\x92',
'CjkUnifiedIdeograph7ee9': '\xe7\xbb\xa9',
'CjkUnifiedIdeograph7ee8': '\xe7\xbb\xa8',
'CjkUnifiedIdeograph7ee5': '\xe7\xbb\xa5',
'CjkUnifiedIdeograph7ee4': '\xe7\xbb\xa4',
'CjkUnifiedIdeograph7ee7': '\xe7\xbb\xa7',
'CjkUnifiedIdeograph7ee6': '\xe7\xbb\xa6',
'CjkUnifiedIdeograph7ee1': '\xe7\xbb\xa1',
'CjkUnifiedIdeograph7ee0': '\xe7\xbb\xa0',
'CjkUnifiedIdeograph7ee3': '\xe7\xbb\xa3',
'CjkUnifiedIdeograph7ee2': '\xe7\xbb\xa2',
'CjkUnifiedIdeograph6eef': '\xe6\xbb\xaf',
'HangulSyllableDeolb': '\xeb\x8d\x9f',
'CjkUnifiedIdeograph6eed': '\xe6\xbb\xad',
'CjkUnifiedIdeograph6968': '\xe6\xa5\xa8',
'CjkUnifiedIdeograph6eeb': '\xe6\xbb\xab',
'CjkUnifiedIdeograph6eec': '\xe6\xbb\xac',
'CjkUnifiedIdeograph6eea': '\xe6\xbb\xaa',
'CjkUnifiedIdeograph9f59': '\xe9\xbd\x99',
'ArabicLetterTchehehMedialForm': '\xef\xae\x81',
'UnmarriedPartnershipSymbol': '\xe2\x9a\xaf',
'LeftwardsHarpoonWithBarbDownwards': '\xe2\x86\xbd',
'HangulSyllableTwic': '\xed\x8a\x97',
'KangxiRadicalHook': '\xe2\xbc\x85',
'UnicodeCharacter0xdc66': '\xed\xb1\xa6',
'HangulSyllableDeolg': '\xeb\x8d\x9d',
'CjkUnifiedIdeograph6963': '\xe6\xa5\xa3',
'OcrCustomerAccountNumber': '\xe2\x91\x89',
'SyriacLetterRish': '\xdc\xaa',
'CjkUnifiedIdeograph8b4e': '\xe8\xad\x8e',
'HangulSyllableRyugg': '\xeb\xa5\x9a',
'CjkUnifiedIdeograph6960': '\xe6\xa5\xa0',
'UnicodeCharacter0xed02': '\xee\xb4\x82',
'UnicodeCharacter0xe8e6': '\xee\xa3\xa6',
'HangulSyllableJjyeo': '\xec\xaa\x84',
'HangulSyllableTyolt': '\xed\x88\x9d',
'CjkUnifiedIdeograph4af9': '\xe4\xab\xb9',
'CjkUnifiedIdeograph4af8': '\xe4\xab\xb8',
'CjkUnifiedIdeograph6da5': '\xe6\xb6\xa5',
'CjkUnifiedIdeograph6da4': '\xe6\xb6\xa4',
'CjkUnifiedIdeograph6da3': '\xe6\xb6\xa3',
'CjkUnifiedIdeograph6da2': '\xe6\xb6\xa2',
'CjkUnifiedIdeograph6da1': '\xe6\xb6\xa1',
'CjkUnifiedIdeograph6da0': '\xe6\xb6\xa0',
'CjkUnifiedIdeograph4af1': '\xe4\xab\xb1',
'CjkUnifiedIdeograph4af0': '\xe4\xab\xb0',
'CjkUnifiedIdeograph4af3': '\xe4\xab\xb3',
'CjkUnifiedIdeograph4af2': '\xe4\xab\xb2',
'CjkUnifiedIdeograph4af5': '\xe4\xab\xb5',
'CjkUnifiedIdeograph4af4': '\xe4\xab\xb4',
'CjkUnifiedIdeograph4af7': '\xe4\xab\xb7',
'CjkUnifiedIdeograph4af6': '\xe4\xab\xb6',
'GreekCapitalLetterAlphaWithDasiaAndOxia': '\xe1\xbc\x8d',
'CombiningCyrillicLetterEm': '\xe2\xb7\xa8',
'CombiningCyrillicLetterEl': '\xe2\xb7\xa7',
'UnicodeCharacter0xd7cb': '\xed\x9f\x8b',
'UnicodeCharacter0xe8e7': '\xee\xa3\xa7',
'BoxDrawingsHeavyQuadrupleDashVertical': '\xe2\x94\x8b',
'KannadaLetterVocalicR': '\xe0\xb2\x8b',
'KannadaLetterVocalicL': '\xe0\xb2\x8c',
'HangulSyllableJal': '\xec\x9e\x98',
'HangulSyllableSyalb': '\xec\x83\xaf',
'CombiningCyrillicLetterEs': '\xe2\xb7\xad',
'CombiningCyrillicLetterEr': '\xe2\xb7\xac',
'CjkUnifiedIdeograph643a': '\xe6\x90\xba',
'CjkUnifiedIdeograph6dae': '\xe6\xb6\xae',
'CjkUnifiedIdeograph6dad': '\xe6\xb6\xad',
'CjkUnifiedIdeograph6dac': '\xe6\xb6\xac',
'CjkUnifiedIdeograph6dab': '\xe6\xb6\xab',
'CjkUnifiedIdeograph6daa': '\xe6\xb6\xaa',
'CjkUnifiedIdeograph4afa': '\xe4\xab\xba',
'UnicodeCharacter0xf0b0': '\xef\x82\xb0',
'CjkUnifiedIdeograph4afc': '\xe4\xab\xbc',
'CjkUnifiedIdeograph4afb': '\xe4\xab\xbb',
'CjkUnifiedIdeograph4afe': '\xe4\xab\xbe',
'CjkUnifiedIdeograph4afd': '\xe4\xab\xbd',
'CjkUnifiedIdeograph4aff': '\xe4\xab\xbf',
'NkoLetterNya': '\xdf\xa2',
'CircleWithLeftHalfBlack': '\xe2\x97\x90',
'UnicodeCharacter0xf0b2': '\xef\x82\xb2',
'HangulSyllableDdyigg': '\xeb\x9d\x86',
'CjkUnifiedIdeograph4f26': '\xe4\xbc\xa6',
'DingbatCircledSansserifDigitTwo': '\xe2\x9e\x81',
'HangulSyllableTyolg': '\xed\x88\x99',
'BalineseVowelSignRaRepa': '\xe1\xac\xba',
'CjkUnifiedIdeograph4f24': '\xe4\xbc\xa4',
'HangulSyllableSyuss': '\xec\x8a\x9c',
'UnicodeCharacter0xf7d4': '\xef\x9f\x94',
'KangxiRadicalFire': '\xe2\xbd\x95',
'CjkUnifiedIdeograph4f22': '\xe4\xbc\xa2',
'UnicodeCharacter0xf89a': '\xef\xa2\x9a',
'EthiopicSyllableNoa': '\xe2\xb6\x88',
'UnicodeCharacter0xf89c': '\xef\xa2\x9c',
'UnicodeCharacter0xf89b': '\xef\xa2\x9b',
'UnicodeCharacter0xf89e': '\xef\xa2\x9e',
'UnicodeCharacter0xf89d': '\xef\xa2\x9d',
'CjkUnifiedIdeograph70a9': '\xe7\x82\xa9',
'HangulSyllableNwinj': '\xeb\x89\x9d',
'HangulSyllableNwinh': '\xeb\x89\x9e',
'CjkUnifiedIdeograph6566': '\xe6\x95\xa6',
'EthiopicSyllableGye': '\xe2\xb7\x9d',
'HangulSyllableMwelb': '\xeb\xad\xb7',
'CjkUnifiedIdeograph5ad0': '\xe5\xab\x90',
'CjkUnifiedIdeograph5ad1': '\xe5\xab\x91',
'CjkUnifiedIdeograph5ad2': '\xe5\xab\x92',
'CjkUnifiedIdeograph5ad3': '\xe5\xab\x93',
'CjkUnifiedIdeograph5ad4': '\xe5\xab\x94',
'CjkUnifiedIdeograph5ad5': '\xe5\xab\x95',
'CjkUnifiedIdeograph5ad6': '\xe5\xab\x96',
'CjkUnifiedIdeograph5ad7': '\xe5\xab\x97',
'CjkUnifiedIdeograph5ad8': '\xe5\xab\x98',
'CjkUnifiedIdeograph5ad9': '\xe5\xab\x99',
'EthiopicSyllableGya': '\xe2\xb7\x98',
'ArabicLigatureKhahWithYehFinalForm': '\xef\xb4\xa0',
'ArabicLetterAlefMaksura': '\xd9\x89',
'HangulSyllableGgyaep': '\xea\xba\xba',
'HangulSyllableKubs': '\xec\xbf\xb2',
'CjkUnifiedIdeograph4f28': '\xe4\xbc\xa8',
'KhmerLetterDo': '\xe1\x9e\x8c',
'LatinCapitalLetterKWithHook': '\xc6\x98',
'HangulSyllableRoegs': '\xeb\xa2\xb3',
'VaiSyllableNjo': '\xea\x97\x99',
'VaiSyllableNji': '\xea\x95\x81',
'CjkUnifiedIdeograph70a4': '\xe7\x82\xa4',
'HangulSyllableHyoh': '\xed\x9b\x83',
'VaiSyllableNje': '\xea\x98\x81',
'CjkUnifiedIdeograph70a5': '\xe7\x82\xa5',
'UnicodeCharacter0xf891': '\xef\xa2\x91',
'UnicodeCharacter0xf890': '\xef\xa2\x90',
'UnicodeCharacter0xf893': '\xef\xa2\x93',
'KangxiRadicalStop': '\xe2\xbd\x8c',
'UnicodeCharacter0xf895': '\xef\xa2\x95',
'UnicodeCharacter0xf894': '\xef\xa2\x94',
'UnicodeCharacter0xf897': '\xef\xa2\x97',
'UnicodeCharacter0xf896': '\xef\xa2\x96',
'UnicodeCharacter0xf899': '\xef\xa2\x99',
'DegreeCelsius': '\xe2\x84\x83',
'EthiopicSyllableCcaa': '\xe2\xb6\xab',
'YiSyllableBburx': '\xea\x81\xb0',
'CherokeeLetterYi': '\xe1\x8f\xb1',
'CherokeeLetterYo': '\xe1\x8f\xb2',
'CherokeeLetterYa': '\xe1\x8f\xaf',
'YiSyllableTurx': '\xea\x84\xb4',
'CherokeeLetterYe': '\xe1\x8f\xb0',
'UnicodeCharacter0xf209': '\xef\x88\x89',
'CjkUnifiedIdeograph5ada': '\xe5\xab\x9a',
'CjkUnifiedIdeograph5adb': '\xe5\xab\x9b',
'CjkUnifiedIdeograph5adc': '\xe5\xab\x9c',
'CjkUnifiedIdeograph5add': '\xe5\xab\x9d',
'CjkUnifiedIdeograph5ade': '\xe5\xab\x9e',
'CjkUnifiedIdeograph5adf': '\xe5\xab\x9f',
'PresentationFormForVerticalEnDash': '\xef\xb8\xb2',
'HangulSyllableJjyeg': '\xec\xaa\xa1',
'ArabicSukun': '\xd9\x92',
'CherokeeLetterYv': '\xe1\x8f\xb4',
'CherokeeLetterYu': '\xe1\x8f\xb3',
'HangulSyllableGwiss': '\xea\xb7\x94',
'CanadianSyllabicsYwo': '\xe1\x94\xb5',
'HangulSyllableGgol': '\xea\xbc\xb4',
'HangulSyllableGgom': '\xea\xbc\xbc',
'HangulSyllableGgoj': '\xea\xbd\x82',
'HangulSyllableGgok': '\xea\xbd\x84',
'CanadianSyllabicsYwi': '\xe1\x94\xb1',
'HangulSyllableGgog': '\xea\xbc\xad',
'CanadianSyllabicsYwe': '\xe1\x94\xaf',
'HangulSyllableGgoe': '\xea\xbe\x80',
'HangulSyllableGgob': '\xea\xbc\xbd',
'HangulSyllableGgoc': '\xea\xbd\x83',
'CanadianSyllabicsYwa': '\xe1\x94\xb9',
'ChamLetterFinalP': '\xea\xa9\x87',
'HangulSyllableGgot': '\xea\xbd\x85',
'HangulSyllableGgos': '\xea\xbc\xbf',
'HangulSyllableGgop': '\xea\xbd\x86',
'UnicodeCharacter0xf2e3': '\xef\x8b\xa3',
'UnicodeCharacter0xf2e2': '\xef\x8b\xa2',
'UnicodeCharacter0xf2e1': '\xef\x8b\xa1',
'UnicodeCharacter0xf2e0': '\xef\x8b\xa0',
'UnicodeCharacter0xf2e7': '\xef\x8b\xa7',
'UnicodeCharacter0xf2e6': '\xef\x8b\xa6',
'UnicodeCharacter0xf2e5': '\xef\x8b\xa5',
'UnicodeCharacter0xf2e4': '\xef\x8b\xa4',
'LatinSmallLetterGWithCaron': '\xc7\xa7',
'UnicodeCharacter0xf2e9': '\xef\x8b\xa9',
'UnicodeCharacter0xf2e8': '\xef\x8b\xa8',
'ArabicLetterDyehFinalForm': '\xef\xad\xb3',
'LatinCapitalLetterKWithDiagonalStroke': '\xea\x9d\x82',
'OriyaLetterLa': '\xe0\xac\xb2',
'CircledIdeographAlliance': '\xe3\x8a\xaf',
'UnicodeCharacter0xdaec': '\xed\xab\xac',
'ReversedPilcrowSign': '\xe2\x81\x8b',
'DevanagariLetterI': '\xe0\xa4\x87',
'UnicodeCharacter0xdaea': '\xed\xab\xaa',
'DevanagariLetterO': '\xe0\xa4\x93',
'UnicodeCharacter0xeeb3': '\xee\xba\xb3',
'SquareEraNameTaisyou': '\xe3\x8d\xbd',
'DevanagariLetterA': '\xe0\xa4\x85',
'CjkCompatibilityIdeographf92c': '\xef\xa4\xac',
'DevanagariLetterE': '\xe0\xa4\x8f',
'HangulSyllableSswilh': '\xec\x93\x87',
'OriyaLetterPha': '\xe0\xac\xab',
'HangulSyllableSsyeonj': '\xec\x8e\xa5',
'UnicodeCharacter0xdaed': '\xed\xab\xad',
'UnicodeCharacter0xdaee': '\xed\xab\xae',
'CjkUnifiedIdeograph355c': '\xe3\x95\x9c',
'DevanagariLetterU': '\xe0\xa4\x89',
'HangulSyllableGgeong': '\xea\xbb\x91',
'YiSyllableMo': '\xea\x82\xbf',
'CjkUnifiedIdeograph56b6': '\xe5\x9a\xb6',
'YiSyllableMi': '\xea\x82\xb0',
'UnicodeCharacter0xeae4': '\xee\xab\xa4',
'YiSyllableMe': '\xea\x83\x82',
'YiSyllableMa': '\xea\x82\xb7',
'HangulSyllableGgeonj': '\xea\xbb\x81',
'UnicodeCharacter0x1bcc': '\xe1\xaf\x8c',
'UnicodeCharacter0xf2ec': '\xef\x8b\xac',
'HangulSyllableDyeogs': '\xeb\x8e\x8f',
'UnicodeCharacter0xf2ea': '\xef\x8b\xaa',
'YiSyllableMy': '\xea\x83\x8b',
'UnicodeCharacter0xf2ef': '\xef\x8b\xaf',
'UnicodeCharacter0xf2ee': '\xef\x8b\xae',
'UnicodeCharacter0xf2ed': '\xef\x8b\xad',
'YiSyllableMu': '\xea\x83\x85',
'CjkUnifiedIdeograph56b2': '\xe5\x9a\xb2',
'HangulSyllableSswilp': '\xec\x93\x86',
'CjkUnifiedIdeograph8d20': '\xe8\xb4\xa0',
'CjkUnifiedIdeograph4f2f': '\xe4\xbc\xaf',
'CjkUnifiedIdeograph8d22': '\xe8\xb4\xa2',
'CjkUnifiedIdeograph8d23': '\xe8\xb4\xa3',
'CjkUnifiedIdeograph8d24': '\xe8\xb4\xa4',
'CjkUnifiedIdeograph8d25': '\xe8\xb4\xa5',
'CjkUnifiedIdeograph8d26': '\xe8\xb4\xa6',
'CjkUnifiedIdeograph8d27': '\xe8\xb4\xa7',
'CjkUnifiedIdeograph8d28': '\xe8\xb4\xa8',
'CjkUnifiedIdeograph8d29': '\xe8\xb4\xa9',
'UnicodeCharacter0xeae2': '\xee\xab\xa2',
'ChamLetterFinalT': '\xea\xa9\x85',
'HangulSyllableBbweong': '\xeb\xbf\xbd',
'UnicodeCharacter0xe63c': '\xee\x98\xbc',
'EndOfProof': '\xe2\x88\x8e',
'CjkUnifiedIdeograph4f2b': '\xe4\xbc\xab',
'CjkUnifiedIdeograph8c9a': '\xe8\xb2\x9a',
'CjkUnifiedIdeograph8c9f': '\xe8\xb2\x9f',
'CjkUnifiedIdeograph8c9d': '\xe8\xb2\x9d',
'CjkUnifiedIdeograph8c9e': '\xe8\xb2\x9e',
'CyrillicSmallLetterSoftSign': '\xd1\x8c',
'DevanagariLetterRa': '\xe0\xa4\xb0',
'CanadianSyllabicsCarrierTsu': '\xe1\x99\x9b',
'UnicodeCharacter0xeae8': '\xee\xab\xa8',
'SylotiNagriLetterTho': '\xea\xa0\x95',
'ArabicLigatureMeemWithKhahWithMeemInitialForm': '\xef\xb6\x8f',
'CjkUnifiedIdeograph8c98': '\xe8\xb2\x98',
'CjkUnifiedIdeograph8c99': '\xe8\xb2\x99',
'CjkUnifiedIdeograph8c92': '\xe8\xb2\x92',
'CjkUnifiedIdeograph8c93': '\xe8\xb2\x93',
'CjkUnifiedIdeograph8c90': '\xe8\xb2\x90',
'CjkUnifiedIdeograph8c91': '\xe8\xb2\x91',
'CjkUnifiedIdeograph8c96': '\xe8\xb2\x96',
'CjkUnifiedIdeograph8c97': '\xe8\xb2\x97',
'CjkUnifiedIdeograph8c94': '\xe8\xb2\x94',
'CjkUnifiedIdeograph8c95': '\xe8\xb2\x95',
'CjkUnifiedIdeograph8d2a': '\xe8\xb4\xaa',
'CombiningDoubleOverline': '\xcc\xbf',
'CjkUnifiedIdeograph8d2c': '\xe8\xb4\xac',
'CjkUnifiedIdeograph8d2d': '\xe8\xb4\xad',
'CjkUnifiedIdeograph8d2e': '\xe8\xb4\xae',
'CjkUnifiedIdeograph8d2f': '\xe8\xb4\xaf',
'VaiDigitNine': '\xea\x98\xa9',
'LepchaLetterHa': '\xe1\xb0\x9d',
'RightHalfBlock': '\xe2\x96\x90',
'LeftwardsQuadrupleArrow': '\xe2\xad\x85',
'UnicodeCharacter0xdcda': '\xed\xb3\x9a',
'CjkStrokeXg': '\xe3\x87\x82',
'HangulSyllableDweoss': '\xeb\x92\x80',
'ArabicLigatureMeemWithHahWithJeemInitialForm': '\xef\xb6\x89',
'CjkUnifiedIdeograph696d': '\xe6\xa5\xad',
'EquivalentTo': '\xe2\x89\x8d',
'HangulSyllableMyeong': '\xeb\xaa\x85',
'HangulSyllableEud': '\xec\x9d\x83',
'HangulSyllableSweogs': '\xec\x88\xb7',
'CjkUnifiedIdeograph5e71': '\xe5\xb9\xb1',
'HangulSyllableEuc': '\xec\x9d\x93',
'HangulSyllableEun': '\xec\x9d\x80',
'HangulSyllableEul': '\xec\x9d\x84',
'HangulSyllableEum': '\xec\x9d\x8c',
'HangulSyllableEuj': '\xec\x9d\x92',
'HangulSyllableEuk': '\xec\x9d\x94',
'HangulSyllableEuh': '\xec\x9d\x97',
'HangulSyllableEut': '\xec\x9d\x95',
'HangulSyllableDweb': '\xeb\x92\x99',
'BlackChessKing': '\xe2\x99\x9a',
'HangulSyllableEus': '\xec\x9d\x8f',
'HangulSyllableEup': '\xec\x9d\x96',
'PhagspaLetterTta': '\xea\xa1\xa9',
'CjkCompatibilityIdeographf92d': '\xef\xa4\xad',
'HangulSyllableSsaegs': '\xec\x8c\x97',
'CjkUnifiedIdeograph7fee': '\xe7\xbf\xae',
'HangulSyllableRwegg': '\xeb\xa4\xa2',
'ArabicLigatureSheenWithYehIsolatedForm': '\xef\xb3\xbe',
'UnicodeCharacter0xf0ba': '\xef\x82\xba',
'UnicodeCharacter0xf334': '\xef\x8c\xb4',
'UnicodeCharacter0xf335': '\xef\x8c\xb5',
'UnicodeCharacter0xf336': '\xef\x8c\xb6',
'UnicodeCharacter0xf337': '\xef\x8c\xb7',
'UnicodeCharacter0xf330': '\xef\x8c\xb0',
'UnicodeCharacter0xf331': '\xef\x8c\xb1',
'UnicodeCharacter0xf332': '\xef\x8c\xb2',
'UnicodeCharacter0xf333': '\xef\x8c\xb3',
'UnicodeCharacter0xffd8': '\xef\xbf\x98',
'UnicodeCharacter0xffd9': '\xef\xbf\x99',
'UnicodeCharacter0xf338': '\xef\x8c\xb8',
'UnicodeCharacter0xf339': '\xef\x8c\xb9',
'HangulSyllableDwem': '\xeb\x92\x98',
'CopticSmallLetterDalda': '\xe2\xb2\x87',
'HangulSyllableDwel': '\xeb\x92\x90',
'UnicodeCharacter0xe763': '\xee\x9d\xa3',
'LatinCapitalLetterLWithStroke': '\xc5\x81',
'HangulSyllableDwen': '\xeb\x92\x8c',
'UnicodeCharacter0xdcde': '\xed\xb3\x9e',
'UnicodeCharacter0xf33d': '\xef\x8c\xbd',
'HangulSyllableGyes': '\xea\xb3\x97',
'HangulSyllableGyep': '\xea\xb3\x9e',
'UnicodeCharacter0xffdd': '\xef\xbf\x9d',
'HangulSyllableMigs': '\xeb\xaf\xbb',
'LatinCapitalLetterUWithDiaeresisAndMacron': '\xc7\x95',
'HangulSyllableDwep': '\xeb\x92\xa2',
'KannadaSignAnusvara': '\xe0\xb2\x82',
'CanadianSyllabicsWestcreePwo': '\xe1\x91\x81',
'HangulSyllableGyeb': '\xea\xb3\x95',
'AplFunctionalSymbolSquishQuad': '\xe2\x8c\xb7',
'CanadianSyllabicsWaa': '\xe1\x90\x99',
'HangulSyllableGyeg': '\xea\xb3\x85',
'HangulSyllableGyed': '\xea\xb3\x8b',
'CjkUnifiedIdeograph3550': '\xe3\x95\x90',
'HangulSyllableGyej': '\xea\xb3\x9a',
'HangulSyllableGyek': '\xea\xb3\x9c',
'HangulSyllableGyeh': '\xea\xb3\x9f',
'HangulSyllableGyen': '\xea\xb3\x88',
'YiSyllableShuox': '\xea\x8e\xaf',
'HangulSyllableGyel': '\xea\xb3\x8c',
'HangulSyllableGyem': '\xea\xb3\x94',
'HangulSyllableSsit': '\xec\x95\x81',
'HangulSyllableSsip': '\xec\x95\x82',
'HangulSyllableSsis': '\xec\x94\xbb',
'CjkUnifiedIdeograph3554': '\xe3\x95\x94',
'HangulSyllableSsid': '\xec\x94\xaf',
'HangulChoseongYeorinhieuh': '\xe1\x85\x99',
'UnicodeCharacter0xf7a3': '\xef\x9e\xa3',
'HangulSyllableSsib': '\xec\x94\xb9',
'HangulSyllableSsic': '\xec\x94\xbf',
'HangulSyllableSsil': '\xec\x94\xb0',
'ChamLetterMa': '\xea\xa8\xa0',
'HangulSyllableSsin': '\xec\x94\xac',
'HangulSyllableSsih': '\xec\x95\x83',
'HangulSyllableSsij': '\xec\x94\xbe',
'HangulSyllableSsik': '\xec\x95\x80',
'CombiningRightwardsHarpoonWithBarbDownwards': '\xe2\x83\xac',
'CjkUnifiedIdeograph9e7e': '\xe9\xb9\xbe',
'CjkUnifiedIdeograph9e7d': '\xe9\xb9\xbd',
'UnicodeCharacter0xe157': '\xee\x85\x97',
'CjkUnifiedIdeograph9e7f': '\xe9\xb9\xbf',
'BraillePatternDots23467': '\xe2\xa1\xae',
'CjkUnifiedIdeograph7828': '\xe7\xa0\xa8',
'CjkUnifiedIdeograph9e7c': '\xe9\xb9\xbc',
'CjkUnifiedIdeograph9e7b': '\xe9\xb9\xbb',
'CjkUnifiedIdeograph7825': '\xe7\xa0\xa5',
'CjkUnifiedIdeograph7824': '\xe7\xa0\xa4',
'CjkUnifiedIdeograph7827': '\xe7\xa0\xa7',
'BraillePatternDots23468': '\xe2\xa2\xae',
'CjkUnifiedIdeograph7821': '\xe7\xa0\xa1',
'CjkUnifiedIdeograph7820': '\xe7\xa0\xa0',
'CjkUnifiedIdeograph7823': '\xe7\xa0\xa3',
'CjkUnifiedIdeograph7822': '\xe7\xa0\xa2',
'CjkUnifiedIdeograph4198': '\xe4\x86\x98',
'CjkUnifiedIdeograph4199': '\xe4\x86\x99',
'K': '\xce\x9a',
'LatinSmallLetterEWithCircumflexAndDotBelow': '\xe1\xbb\x87',
'HangulSyllableRweolg': '\xeb\xa4\x8d',
'CjkUnifiedIdeograph9117': '\xe9\x84\x97',
'CjkUnifiedIdeograph9e75': '\xe9\xb9\xb5',
'CjkUnifiedIdeograph9e74': '\xe9\xb9\xb4',
'CjkUnifiedIdeograph9e77': '\xe9\xb9\xb7',
'CjkUnifiedIdeograph9e76': '\xe9\xb9\xb6',
'CjkUnifiedIdeograph9e71': '\xe9\xb9\xb1',
'CjkUnifiedIdeograph9115': '\xe9\x84\x95',
'CjkUnifiedIdeograph9e73': '\xe9\xb9\xb3',
'CjkUnifiedIdeograph9e72': '\xe9\xb9\xb2',
'CjkUnifiedIdeograph782e': '\xe7\xa0\xae',
'CjkUnifiedIdeograph782d': '\xe7\xa0\xad',
'CjkUnifiedIdeograph782f': '\xe7\xa0\xaf',
'CjkUnifiedIdeograph782a': '\xe7\xa0\xaa',
'CjkUnifiedIdeograph9e78': '\xe9\xb9\xb8',
'CjkUnifiedIdeograph782c': '\xe7\xa0\xac',
'CjkUnifiedIdeograph782b': '\xe7\xa0\xab',
'UnicodeCharacter0xdae9': '\xed\xab\xa9',
'ModifierLetterSmallHWithHook': '\xca\xb1',
'CjkUnifiedIdeograph9119': '\xe9\x84\x99',
'CjkUnifiedIdeograph606f': '\xe6\x81\xaf',
'CjkUnifiedIdeograph606d': '\xe6\x81\xad',
'CjkUnifiedIdeograph606e': '\xe6\x81\xae',
'CjkUnifiedIdeograph606b': '\xe6\x81\xab',
'CjkUnifiedIdeograph606c': '\xe6\x81\xac',
'CjkUnifiedIdeograph606a': '\xe6\x81\xaa',
'NegativeCircledNumberSixteen': '\xe2\x93\xb0',
'HangulSyllableGgyeb': '\xea\xbc\xa1',
'EthiopicTonalMarkDeret': '\xe1\x8e\x91',
'HangulSyllableJjalh': '\xec\xa7\xab',
'CjkUnifiedIdeograph4196': '\xe4\x86\x96',
'UnicodeCharacter0xe00b': '\xee\x80\x8b',
'HangulSyllableGgyec': '\xea\xbc\xa7',
'UnicodeCharacter0xe7ab': '\xee\x9e\xab',
'HangulSyllableMoelh': '\xeb\xac\x8b',
'UnicodeCharacter0xdae0': '\xed\xab\xa0',
'HangulSyllablePwass': '\xed\x90\x9c',
'UnicodeCharacter0xdae1': '\xed\xab\xa1',
'UnicodeCharacter0xddc9': '\xed\xb7\x89',
'UnicodeCharacter0xddc8': '\xed\xb7\x88',
'HangulSyllableYils': '\xec\x9d\xa4',
'UnicodeCharacter0xdae6': '\xed\xab\xa6',
'UnicodeCharacter0xddc7': '\xed\xb7\x87',
'HangulSyllableYilp': '\xec\x9d\xa6',
'UnicodeCharacter0xddc1': '\xed\xb7\x81',
'UnicodeCharacter0xddc0': '\xed\xb7\x80',
'UnicodeCharacter0xddc3': '\xed\xb7\x83',
'HangulSyllableGgyeg': '\xea\xbc\x91',
'HangulSyllableYilh': '\xec\x9d\xa7',
'HangulSyllableGgyed': '\xea\xbc\x97',
'HangulSyllableYilm': '\xec\x9d\xa2',
'HangulSyllableYilb': '\xec\x9d\xa3',
'UnicodeCharacter0xdae5': '\xed\xab\xa5',
'HangulSyllableYilg': '\xec\x9d\xa1',
'CanadianSyllabicsEn': '\xe1\x90\xab',
'UnicodeCharacter0xe0f4': '\xee\x83\xb4',
'CircledIdeographRest': '\xe3\x8a\xa1',
'UnicodeCharacter0xe59b': '\xee\x96\x9b',
'RightDoubleArrowWithRoundedHead': '\xe2\xa5\xb0',
'CjkUnifiedIdeograph6066': '\xe6\x81\xa6',
'CjkUnifiedIdeograph6067': '\xe6\x81\xa7',
'CjkUnifiedIdeograph6064': '\xe6\x81\xa4',
'CjkUnifiedIdeograph6065': '\xe6\x81\xa5',
'CjkUnifiedIdeograph6062': '\xe6\x81\xa2',
'CjkUnifiedIdeograph6063': '\xe6\x81\xa3',
'CjkUnifiedIdeograph6060': '\xe6\x81\xa0',
'HangulLetterYeo': '\xe3\x85\x95',
'upsilon': '\xcf\x85',
'CjkRadicalCsimplifiedHorse': '\xe2\xbb\xa2',
'CjkUnifiedIdeograph6069': '\xe6\x81\xa9',
'UnicodeCharacter0xdd0c': '\xed\xb4\x8c',
'HeavyRightpointingAngleQuotationMarkOrnament': '\xe2\x9d\xaf',
'VaiSyllableGoo': '\xea\x96\x91',
'LaoToneMaiCatawa': '\xe0\xbb\x8b',
'del': '\xe2\x88\x87',
'PartiallyrecycledPaperSymbol': '\xe2\x99\xbd',
'UnicodeCharacter0xddce': '\xed\xb7\x8e',
'UnicodeCharacter0xddcd': '\xed\xb7\x8d',
'UnicodeCharacter0xddcf': '\xed\xb7\x8f',
'UnicodeCharacter0xddca': '\xed\xb7\x8a',
'UnicodeCharacter0xddcc': '\xed\xb7\x8c',
'UnicodeCharacter0xddcb': '\xed\xb7\x8b',
'CjkUnifiedIdeograph79b0': '\xe7\xa6\xb0',
'CjkUnifiedIdeograph79b1': '\xe7\xa6\xb1',
'CjkUnifiedIdeograph79b2': '\xe7\xa6\xb2',
'CjkUnifiedIdeograph79b3': '\xe7\xa6\xb3',
'CjkUnifiedIdeograph79b4': '\xe7\xa6\xb4',
'CjkUnifiedIdeograph79b5': '\xe7\xa6\xb5',
'CjkUnifiedIdeograph79b6': '\xe7\xa6\xb6',
'CjkUnifiedIdeograph79b7': '\xe7\xa6\xb7',
'CjkUnifiedIdeograph79b8': '\xe7\xa6\xb8',
'CjkUnifiedIdeograph79b9': '\xe7\xa6\xb9',
'BoxDrawingsHeavyDown': '\xe2\x95\xbb',
'CjkUnifiedIdeograph43d5': '\xe4\x8f\x95',
'CjkUnifiedIdeograph43d4': '\xe4\x8f\x94',
'CjkUnifiedIdeograph43d7': '\xe4\x8f\x97',
'CjkUnifiedIdeograph43d6': '\xe4\x8f\x96',
'CjkUnifiedIdeograph43d1': '\xe4\x8f\x91',
'CjkUnifiedIdeograph43d0': '\xe4\x8f\x90',
'CjkUnifiedIdeograph43d3': '\xe4\x8f\x93',
'ArmenianCapitalLetterCha': '\xd5\x89',
'CjkUnifiedIdeograph6398': '\xe6\x8e\x98',
'RomanNumeralTenThousand': '\xe2\x86\x82',
'CjkUnifiedIdeograph9568': '\xe9\x95\xa8',
'CjkUnifiedIdeograph9569': '\xe9\x95\xa9',
'CjkUnifiedIdeograph43d9': '\xe4\x8f\x99',
'CjkUnifiedIdeograph43d8': '\xe4\x8f\x98',
'CjkUnifiedIdeograph7933': '\xe7\xa4\xb3',
'HangulSyllableByeom': '\xeb\xb3\x8c',
'HangulSyllableByeol': '\xeb\xb3\x84',
'HangulSyllableByeok': '\xeb\xb3\x94',
'HangulSyllableByeoj': '\xeb\xb3\x92',
'OhmSign': '\xe2\x84\xa6',
'CjkUnifiedIdeograph7932': '\xe7\xa4\xb2',
'HangulSyllableByeog': '\xeb\xb2\xbd',
'HangulSyllableByeod': '\xeb\xb3\x83',
'HangulSyllableByeoc': '\xeb\xb3\x93',
'HangulSyllableByeob': '\xeb\xb3\x8d',
'CjkUnifiedIdeograph450f': '\xe4\x94\x8f',
'CjkUnifiedIdeograph450e': '\xe4\x94\x8e',
'CjkUnifiedIdeograph450d': '\xe4\x94\x8d',
'CjkUnifiedIdeograph450c': '\xe4\x94\x8c',
'CjkUnifiedIdeograph450b': '\xe4\x94\x8b',
'CjkUnifiedIdeograph450a': '\xe4\x94\x8a',
'UnicodeCharacter0xe0f0': '\xee\x83\xb0',
'HangulSyllableByeot': '\xeb\xb3\x95',
'HangulSyllableByeos': '\xeb\xb3\x8f',
'KhmerSymbolPrambeiKoet': '\xe1\xa7\xa8',
'HangulSyllableByeop': '\xeb\xb3\x96',
'CjkUnifiedIdeograph43de': '\xe4\x8f\x9e',
'CjkUnifiedIdeograph43dd': '\xe4\x8f\x9d',
'CjkUnifiedIdeograph639b': '\xe6\x8e\x9b',
'CjkUnifiedIdeograph43df': '\xe4\x8f\x9f',
'CjkUnifiedIdeograph43da': '\xe4\x8f\x9a',
'CjkUnifiedIdeograph639e': '\xe6\x8e\x9e',
'CjkUnifiedIdeograph43dc': '\xe4\x8f\x9c',
'CjkUnifiedIdeograph43db': '\xe4\x8f\x9b',
'AdiShakti': '\xe2\x98\xac',
'CjkUnifiedIdeograph79ba': '\xe7\xa6\xba',
'CjkUnifiedIdeograph79bb': '\xe7\xa6\xbb',
'CjkUnifiedIdeograph79bc': '\xe7\xa6\xbc',
'CjkUnifiedIdeograph79bd': '\xe7\xa6\xbd',
'CjkUnifiedIdeograph79be': '\xe7\xa6\xbe',
'CjkUnifiedIdeograph79bf': '\xe7\xa6\xbf',
'UnicodeCharacter0xe0d0': '\xee\x83\x90',
'LatinCapitalLetterThornWithStrokeThroughDescender': '\xea\x9d\xa6',
'HangulSyllableMogs': '\xeb\xaa\xab',
'CjkUnifiedIdeograph7939': '\xe7\xa4\xb9',
'CjkUnifiedIdeograph8b64': '\xe8\xad\xa4',
'CjkUnifiedIdeograph4507': '\xe4\x94\x87',
'CjkUnifiedIdeograph4506': '\xe4\x94\x86',
'CjkUnifiedIdeograph4505': '\xe4\x94\x85',
'CjkUnifiedIdeograph4504': '\xe4\x94\x84',
'CjkUnifiedIdeograph4503': '\xe4\x94\x83',
'CjkUnifiedIdeograph4502': '\xe4\x94\x82',
'CjkUnifiedIdeograph4501': '\xe4\x94\x81',
'CjkUnifiedIdeograph4500': '\xe4\x94\x80',
'UnicodeCharacter0xe911': '\xee\xa4\x91',
'CjkUnifiedIdeograph8c89': '\xe8\xb2\x89',
'CyrillicSmallLetterIotifiedClosedLittleYus': '\xea\x99\x9d',
'CjkUnifiedIdeograph4509': '\xe4\x94\x89',
'CjkUnifiedIdeograph4508': '\xe4\x94\x88',
'CjkRadicalWalkTwo': '\xe2\xbb\x8e',
'ModifierLetterMinusSign': '\xcb\x97',
'NorthEastArrowCrossingNorthWestArrow': '\xe2\xa4\xb1',
'HebrewLetterVavWithHolam': '\xef\xad\x8b',
'CjkUnifiedIdeograph69bf': '\xe6\xa6\xbf',
'CjkUnifiedIdeograph8c88': '\xe8\xb2\x88',
'HangulSyllableNyang': '\xeb\x83\xa5',
'LatinCapitalLetterPWithFlourish': '\xea\x9d\x92',
'HangulSyllableNyanh': '\xeb\x83\x96',
'HangulSyllableNyanj': '\xeb\x83\x95',
'HexagramForIncrease': '\xe4\xb7\xa9',
'HangulJongseongSiostikeut': '\xe1\x87\xa8',
'UnicodeCharacter0xe409': '\xee\x90\x89',
'CjkUnifiedIdeograph3bb9': '\xe3\xae\xb9',
'CjkUnifiedIdeograph3bb8': '\xe3\xae\xb8',
'UnicodeCharacter0xde58': '\xed\xb9\x98',
'YiSyllableZyrx': '\xea\x8a\xac',
'CjkUnifiedIdeograph3bb3': '\xe3\xae\xb3',
'CjkUnifiedIdeograph3bb2': '\xe3\xae\xb2',
'CjkUnifiedIdeograph3bb1': '\xe3\xae\xb1',
'CjkUnifiedIdeograph3bb0': '\xe3\xae\xb0',
'CjkUnifiedIdeograph3bb7': '\xe3\xae\xb7',
'CjkUnifiedIdeograph3bb6': '\xe3\xae\xb6',
'CjkUnifiedIdeograph3bb5': '\xe3\xae\xb5',
'CjkUnifiedIdeograph3bb4': '\xe3\xae\xb4',
'UnicodeCharacter0xe597': '\xee\x96\x97',
'ArabicLetterKeheh': '\xda\xa9',
'UnicodeCharacter0xe596': '\xee\x96\x96',
'KangxiRadicalPower': '\xe2\xbc\x92',
'UnicodeCharacter0xe591': '\xee\x96\x91',
'CanadianSyllabicsFinalDoubleShortVerticalStrokes': '\xe1\x90\xa6',
'HangulSyllableRweolh': '\xeb\xa4\x93',
'UnicodeCharacter0xe590': '\xee\x96\x90',
'SundaneseLetterKa': '\xe1\xae\x8a',
'UnicodeCharacter0xdf41': '\xed\xbd\x81',
'UnicodeCharacter0xe593': '\xee\x96\x93',
'CopticSmallLetterPi': '\xe2\xb2\xa1',
'CjkUnifiedIdeograph3bbc': '\xe3\xae\xbc',
'CjkUnifiedIdeograph3bbb': '\xe3\xae\xbb',
'CjkUnifiedIdeograph3bba': '\xe3\xae\xba',
'UnicodeCharacter0xde5e': '\xed\xb9\x9e',
'GreekCapitalLetterOmicronWithTonos': '\xce\x8c',
'CjkUnifiedIdeograph3bbf': '\xe3\xae\xbf',
'CjkUnifiedIdeograph3bbe': '\xe3\xae\xbe',
'CjkUnifiedIdeograph3bbd': '\xe3\xae\xbd',
'ModifierLetterHighDottedLeftstemToneBar': '\xea\x9c\x8e',
'LatinSmallLetterRWithDotAbove': '\xe1\xb9\x99',
'UnicodeCharacter0xe599': '\xee\x96\x99',
'ArabicLetterNgMedialForm': '\xef\xaf\x96',
'UnicodeCharacter0xe0f9': '\xee\x83\xb9',
'UnicodeCharacter0xe598': '\xee\x96\x98',
'CjkUnifiedIdeograph88cf': '\xe8\xa3\x8f',
'CjkUnifiedIdeograph88ce': '\xe8\xa3\x8e',
'CjkUnifiedIdeograph88cd': '\xe8\xa3\x8d',
'CjkUnifiedIdeograph88cc': '\xe8\xa3\x8c',
'CjkUnifiedIdeograph88cb': '\xe8\xa3\x8b',
'CjkUnifiedIdeograph88ca': '\xe8\xa3\x8a',
'CjkUnifiedIdeograph683e': '\xe6\xa0\xbe',
'CjkUnifiedIdeograph683d': '\xe6\xa0\xbd',
'CjkUnifiedIdeograph683f': '\xe6\xa0\xbf',
'CjkUnifiedIdeograph683a': '\xe6\xa0\xba',
'CjkUnifiedIdeograph683c': '\xe6\xa0\xbc',
'CjkUnifiedIdeograph683b': '\xe6\xa0\xbb',
'HangulSyllableCwag': '\xec\xb4\xa5',
'CjkUnifiedIdeograph911a': '\xe9\x84\x9a',
'HangulSyllableBbyilp': '\xec\x82\x82',
'ArabicLetterTcheh': '\xda\x86',
'YiSyllableCho': '\xea\x8d\xab',
'YiSyllableCha': '\xea\x8d\xa3',
'YiSyllableChe': '\xea\x8d\xaf',
'CjkUnifiedIdeograph911f': '\xe9\x84\x9f',
'SquareImageOfOrEqualTo': '\xe2\x8a\x91',
'HangulLetterSsangnieun': '\xe3\x85\xa5',
'CadaUna': '\xe2\x84\x86',
'CjkUnifiedIdeograph88c9': '\xe8\xa3\x89',
'CjkUnifiedIdeograph88c8': '\xe8\xa3\x88',
'CjkUnifiedIdeograph88c7': '\xe8\xa3\x87',
'CjkUnifiedIdeograph88c6': '\xe8\xa3\x86',
'CjkUnifiedIdeograph88c5': '\xe8\xa3\x85',
'CjkUnifiedIdeograph88c4': '\xe8\xa3\x84',
'CjkUnifiedIdeograph88c3': '\xe8\xa3\x83',
'CjkUnifiedIdeograph88c2': '\xe8\xa3\x82',
'CjkUnifiedIdeograph88c1': '\xe8\xa3\x81',
'CjkUnifiedIdeograph88c0': '\xe8\xa3\x80',
'CjkUnifiedIdeograph542d': '\xe5\x90\xad',
'GreekCapitalLetterPsi': '\xce\xa8',
'LatinLetterAin': '\xe1\xb4\xa5',
'CjkUnifiedIdeograph419b': '\xe4\x86\x9b',
'CjkUnifiedIdeograph542b': '\xe5\x90\xab',
'ArabicLigatureSheenWithKhahIsolatedForm': '\xef\xb4\x8b',
'CjkUnifiedIdeograph419c': '\xe4\x86\x9c',
'ChamLetterFinalG': '\xea\xa9\x81',
'CjkUnifiedIdeograph83da': '\xe8\x8f\x9a',
'ChamLetterFinalK': '\xea\xa9\x80',
'CyrillicCapitalLetterZeWithDescender': '\xd2\x98',
'MongolianDigitSix': '\xe1\xa0\x96',
'ChamLetterFinalN': '\xea\xa9\x86',
'ChamLetterFinalL': '\xea\xa9\x8a',
'CjkUnifiedIdeograph6835': '\xe6\xa0\xb5',
'CjkUnifiedIdeograph6834': '\xe6\xa0\xb4',
'CjkUnifiedIdeograph6837': '\xe6\xa0\xb7',
'CjkUnifiedIdeograph6836': '\xe6\xa0\xb6',
'CjkUnifiedIdeograph6831': '\xe6\xa0\xb1',
'CjkUnifiedIdeograph6830': '\xe6\xa0\xb0',
'CjkUnifiedIdeograph6833': '\xe6\xa0\xb3',
'ModifierLetterSmallTurnedM': '\xe1\xb5\x9a',
'ChamLetterFinalY': '\xea\xa9\x88',
'CjkUnifiedIdeograph6839': '\xe6\xa0\xb9',
'CjkUnifiedIdeograph6838': '\xe6\xa0\xb8',
'UnicodeCharacter0x8b9': '\xe0\xa2\xb9',
'UnicodeCharacter0x8b8': '\xe0\xa2\xb8',
'CjkCompatibilityIdeographf938': '\xef\xa4\xb8',
'CjkCompatibilityIdeographf939': '\xef\xa4\xb9',
'SmallLessthanSign': '\xef\xb9\xa4',
'UnicodeCharacter0x8b1': '\xe0\xa2\xb1',
'UnicodeCharacter0x8b0': '\xe0\xa2\xb0',
'UnicodeCharacter0x8b3': '\xe0\xa2\xb3',
'UnicodeCharacter0x8b2': '\xe0\xa2\xb2',
'UnicodeCharacter0x8b5': '\xe0\xa2\xb5',
'UnicodeCharacter0x8b4': '\xe0\xa2\xb4',
'UnicodeCharacter0x8b7': '\xe0\xa2\xb7',
'UnicodeCharacter0x8b6': '\xe0\xa2\xb6',
'YiSyllableNdux': '\xea\x85\xa3',
'CjkUnifiedIdeograph8c83': '\xe8\xb2\x83',
'YiSyllableNdut': '\xea\x85\xa2',
'BraillePatternDots1478': '\xe2\xa3\x89',
'VaiSyllableMgbee': '\xea\x94\x8a',
'YiSyllableNdup': '\xea\x85\xa5',
'YiSyllableNdur': '\xea\x85\xa7',
'GreekSmallLetterOmegaWithDasiaAndOxia': '\xe1\xbd\xa5',
'HangulSyllableRoelm': '\xeb\xa2\xba',
'CyrillicCapitalLetterSchwaWithDiaeresis': '\xd3\x9a',
'BengaliCurrencyNumeratorThree': '\xe0\xa7\xb6',
'ArabicLetterAlefWithHamzaAboveIsolatedForm': '\xef\xba\x83',
'UnicodeCharacter0xec45': '\xee\xb1\x85',
'CjkUnifiedIdeograph3870': '\xe3\xa1\xb0',
'KatakanaLetterSmallNu': '\xe3\x87\xb4',
'CjkUnifiedIdeograph83db': '\xe8\x8f\x9b',
'UnicodeCharacter0xec47': '\xee\xb1\x87',
'ZeroWidthSpace': '\xe2\x80\x8b',
'UnicodeCharacter0xec40': '\xee\xb1\x80',
'UnicodeCharacter0xed09': '\xee\xb4\x89',
'CanadianSyllabicsWestcreeCwi': '\xe1\x92\x95',
'CyrillicSmallLetterKje': '\xd1\x9c',
'UnicodeCharacter0xec41': '\xee\xb1\x81',
'UnicodeCharacter0xee3d': '\xee\xb8\xbd',
'UnicodeCharacter0xec42': '\xee\xb1\x82',
'UnicodeCharacter0x8ba': '\xe0\xa2\xba',
'CjkCompatibilityIdeographf93e': '\xef\xa4\xbe',
'UnicodeCharacter0x8bc': '\xe0\xa2\xbc',
'UnicodeCharacter0x8bb': '\xe0\xa2\xbb',
'UnicodeCharacter0x8be': '\xe0\xa2\xbe',
'UnicodeCharacter0x8bd': '\xe0\xa2\xbd',
'CjkCompatibilityIdeographf93b': '\xef\xa4\xbb',
'UnicodeCharacter0x8bf': '\xe0\xa2\xbf',
'UnicodeCharacter0x2cf3': '\xe2\xb3\xb3',
'UnicodeCharacter0x2cf2': '\xe2\xb3\xb2',
'UnicodeCharacter0x2cf1': '\xe2\xb3\xb1',
'CanadianSyllabicsCarrierTla': '\xe1\x98\xbf',
'UnicodeCharacter0x2cf7': '\xe2\xb3\xb7',
'UnicodeCharacter0x2cf6': '\xe2\xb3\xb6',
'UnicodeCharacter0x2cf5': '\xe2\xb3\xb5',
'UnicodeCharacter0x2cf4': '\xe2\xb3\xb4',
'UnicodeCharacter0x2cf8': '\xe2\xb3\xb8',
'CjkUnifiedIdeograph7188': '\xe7\x86\x88',
'HangulSyllableKeubs': '\xed\x81\xbe',
'ArabicLetterUighurKazakhKirghizAlefMaksuraMedialForm': '\xef\xaf\xa9',
'HangulSyllableBbwalg': '\xeb\xbd\xa5',
'HangulSyllableJjwanj': '\xec\xab\x9d',
'TibetanLogotypeSignChadRtags': '\xe0\xbc\x95',
'HangulSyllableBbwalb': '\xeb\xbd\xa7',
'HangulSyllableBbwalm': '\xeb\xbd\xa6',
'ChamLetterPa': '\xea\xa8\x9a',
'HangulSyllableBbwalh': '\xeb\xbd\xab',
'HangulSyllableBbwalt': '\xeb\xbd\xa9',
'HangulSyllableBbwals': '\xeb\xbd\xa8',
'BoxDrawingsDoubleDownAndLeft': '\xe2\x95\x97',
'HangulSyllableBbyilh': '\xec\x82\x83',
'HangulSyllableSyinh': '\xec\x8b\x86',
'ParenthesizedHangulPhieuph': '\xe3\x88\x8c',
'CanadianSyllabicsCarrierYa': '\xe1\x98\x93',
'UpwardsAncora': '\xe2\xb8\x95',
'ChamLetterMue': '\xea\xa8\x9f',
'SixPetalledBlackAndWhiteFlorette': '\xe2\x9c\xbe',
'HangulSyllableGoenj': '\xea\xb4\xb9',
'CjkUnifiedIdeograph40c3': '\xe4\x83\x83',
'CjkUnifiedIdeograph40c2': '\xe4\x83\x82',
'CjkUnifiedIdeograph40c1': '\xe4\x83\x81',
'CjkUnifiedIdeograph40c0': '\xe4\x83\x80',
'CjkUnifiedIdeograph40c7': '\xe4\x83\x87',
'CjkUnifiedIdeograph40c6': '\xe4\x83\x86',
'CjkUnifiedIdeograph40c5': '\xe4\x83\x85',
'MyanmarLetterNya': '\xe1\x80\x89',
'CjkUnifiedIdeograph83df': '\xe8\x8f\x9f',
'CjkUnifiedIdeograph7e3f': '\xe7\xb8\xbf',
'CjkUnifiedIdeograph40c9': '\xe4\x83\x89',
'LaoDigitFive': '\xe0\xbb\x95',
'CjkUnifiedIdeograph7e3c': '\xe7\xb8\xbc',
'CjkUnifiedIdeograph7e3b': '\xe7\xb8\xbb',
'CjkUnifiedIdeograph7e3a': '\xe7\xb8\xba',
'TamilLetterNa': '\xe0\xae\xa8',
'KangxiRadicalBlack': '\xe2\xbf\x8a',
'UnicodeCharacter0xe64f': '\xee\x99\x8f',
'HangulSyllableDdyeolp': '\xeb\x97\xa6',
'HangulSyllableDdyeols': '\xeb\x97\xa4',
'GujaratiLetterMa': '\xe0\xaa\xae',
'HangulSyllableToep': '\xed\x88\x8e',
'HangulSyllableToes': '\xed\x88\x87',
'HangulSyllableKweogs': '\xec\xbf\xbf',
'HangulSyllableToet': '\xed\x88\x8d',
'CanadianSyllabicsShii': '\xe1\x94\x92',
'HangulSyllableToeh': '\xed\x88\x8f',
'HangulSyllableDdyeolg': '\xeb\x97\xa1',
'HangulSyllableToej': '\xed\x88\x8a',
'HangulSyllableToem': '\xed\x88\x84',
'HangulSyllableToel': '\xed\x87\xbc',
'SquareGiga': '\xe3\x8c\x90',
'HangulSyllableDdyeolm': '\xeb\x97\xa2',
'HangulSyllableToec': '\xed\x88\x8b',
'HangulSyllableToeb': '\xed\x88\x85',
'HangulSyllableDdyeolh': '\xeb\x97\xa7',
'HangulSyllableToeg': '\xed\x87\xb5',
'HangulSyllableKweogg': '\xec\xbf\xbe',
'CjkUnifiedIdeograph40cc': '\xe4\x83\x8c',
'CjkUnifiedIdeograph40cb': '\xe4\x83\x8b',
'CjkUnifiedIdeograph40ca': '\xe4\x83\x8a',
'YiSyllablePyx': '\xea\x81\x91',
'CjkUnifiedIdeograph40cf': '\xe4\x83\x8f',
'CjkUnifiedIdeograph40ce': '\xe4\x83\x8e',
'GreekCapitalLetterIotaWithDasiaAndOxia': '\xe1\xbc\xbd',
'CjkUnifiedIdeograph7e37': '\xe7\xb8\xb7',
'CjkUnifiedIdeograph7e36': '\xe7\xb8\xb6',
'CjkUnifiedIdeograph7e35': '\xe7\xb8\xb5',
'CjkUnifiedIdeograph7e34': '\xe7\xb8\xb4',
'CjkUnifiedIdeograph7e33': '\xe7\xb8\xb3',
'BoxDrawingsLightDoubleDashVertical': '\xe2\x95\x8e',
'CjkUnifiedIdeograph7e31': '\xe7\xb8\xb1',
'CjkUnifiedIdeograph7e30': '\xe7\xb8\xb0',
'UnicodeCharacter0xebdd': '\xee\xaf\x9d',
'CjkRadicalBox': '\xe2\xba\x86',
'ArabicLigatureRehWithSuperscriptAlefIsolatedForm': '\xef\xb1\x9c',
'ThaiCharacterThanthakhat': '\xe0\xb9\x8c',
'GreekCapitalLetterChi': '\xce\xa7',
'CjkUnifiedIdeograph4d5e': '\xe4\xb5\x9e',
'CjkUnifiedIdeograph4d5d': '\xe4\xb5\x9d',
'UnicodeCharacter0xaaff': '\xea\xab\xbf',
'CjkUnifiedIdeograph4d5f': '\xe4\xb5\x9f',
'CjkUnifiedIdeograph4d5a': '\xe4\xb5\x9a',
'UnicodeCharacter0xaafa': '\xea\xab\xba',
'CjkUnifiedIdeograph4d5c': '\xe4\xb5\x9c',
'CjkUnifiedIdeograph4d5b': '\xe4\xb5\x9b',
'HangulSyllableBeuss': '\xeb\xb8\xa0',
'SquarePiko': '\xe3\x8c\xb0',
'UnicodeCharacter0xf13c': '\xef\x84\xbc',
'UnicodeCharacter0xda4b': '\xed\xa9\x8b',
'UnicodeCharacter0xf13a': '\xef\x84\xba',
'UnicodeCharacter0xf13f': '\xef\x84\xbf',
'UnicodeCharacter0xf13d': '\xef\x84\xbd',
'UnicodeCharacter0xf13e': '\xef\x84\xbe',
'UnicodeCharacter0xebdf': '\xee\xaf\x9f',
'CjkUnifiedIdeograph4a89': '\xe4\xaa\x89',
'CjkUnifiedIdeograph4a88': '\xe4\xaa\x88',
'CjkUnifiedIdeograph4a83': '\xe4\xaa\x83',
'CjkUnifiedIdeograph4a82': '\xe4\xaa\x82',
'CjkUnifiedIdeograph4a81': '\xe4\xaa\x81',
'ArmenianCapitalLetterVew': '\xd5\x8e',
'CjkUnifiedIdeograph4a87': '\xe4\xaa\x87',
'CjkUnifiedIdeograph4a86': '\xe4\xaa\x86',
'CjkUnifiedIdeograph4a85': '\xe4\xaa\x85',
'CjkUnifiedIdeograph4a84': '\xe4\xaa\x84',
'CjkUnifiedIdeograph4d55': '\xe4\xb5\x95',
'CjkUnifiedIdeograph4d54': '\xe4\xb5\x94',
'CjkUnifiedIdeograph4d57': '\xe4\xb5\x97',
'CjkUnifiedIdeograph4d56': '\xe4\xb5\x96',
'CjkUnifiedIdeograph4d51': '\xe4\xb5\x91',
'CjkUnifiedIdeograph4d50': '\xe4\xb5\x90',
'CjkUnifiedIdeograph4d53': '\xe4\xb5\x93',
'CjkUnifiedIdeograph4d52': '\xe4\xb5\x92',
'CjkUnifiedIdeograph4d59': '\xe4\xb5\x99',
'CjkUnifiedIdeograph4d58': '\xe4\xb5\x98',
'ArabicLetterThehMedialForm': '\xef\xba\x9c',
'HangulSyllableByaegs': '\xeb\xb1\xab',
'PhagspaLetterYa': '\xea\xa1\x97',
'CjkUnifiedIdeograph6da7': '\xe6\xb6\xa7',
'CjkUnifiedIdeograph6da6': '\xe6\xb6\xa6',
'CjkUnifiedIdeograph4a8c': '\xe4\xaa\x8c',
'CjkUnifiedIdeograph4a8b': '\xe4\xaa\x8b',
'CjkUnifiedIdeograph4a8a': '\xe4\xaa\x8a',
'CjkUnifiedIdeograph4a8f': '\xe4\xaa\x8f',
'CjkUnifiedIdeograph4a8e': '\xe4\xaa\x8e',
'CjkUnifiedIdeograph4a8d': '\xe4\xaa\x8d',
'UnicodeCharacter0xf132': '\xef\x84\xb2',
'SixPointedBlackStar': '\xe2\x9c\xb6',
'UnicodeCharacter0xf130': '\xef\x84\xb0',
'HangulSyllableDwebs': '\xeb\x92\x9a',
'UnicodeCharacter0xf136': '\xef\x84\xb6',
'BraillePatternDots4567': '\xe2\xa1\xb8',
'UnicodeCharacter0xf134': '\xef\x84\xb4',
'UnicodeCharacter0xf135': '\xef\x84\xb5',
'BoxDrawingsHeavyRight': '\xe2\x95\xba',
'UnicodeCharacter0xf139': '\xef\x84\xb9',
'HangulSyllableTwinh': '\xed\x8a\x86',
'HangulSyllableTwinj': '\xed\x8a\x85',
'CjkUnifiedIdeograph5b98': '\xe5\xae\x98',
'CjkUnifiedIdeograph5b99': '\xe5\xae\x99',
'TifinaghLetterYaghh': '\xe2\xb4\xb4',
'HangulSyllableYenj': '\xec\x98\x8d',
'CjkUnifiedIdeograph5b94': '\xe5\xae\x94',
'CjkUnifiedIdeograph5b95': '\xe5\xae\x95',
'CjkUnifiedIdeograph5b96': '\xe5\xae\x96',
'CjkUnifiedIdeograph5b97': '\xe5\xae\x97',
'HexagramForFellowship': '\xe4\xb7\x8c',
'CjkUnifiedIdeograph5b91': '\xe5\xae\x91',
'CjkUnifiedIdeograph5b92': '\xe5\xae\x92',
'CjkUnifiedIdeograph5b93': '\xe5\xae\x93',
'YiSyllableKuop': '\xea\x88\x85',
'CjkRadicalLongTwo': '\xe2\xbb\x92',
'UnicodeCharacter0xe00d': '\xee\x80\x8d',
'ModifierLetterRaisedDownArrow': '\xea\x9c\x9c',
'YiSyllableKuox': '\xea\x88\x83',
'UnicodeCharacter0xebdb': '\xee\xaf\x9b',
'LatinLetterSmallCapitalOe': '\xc9\xb6',
'TeluguLetterGha': '\xe0\xb0\x98',
'UnicodeCharacter0xf688': '\xef\x9a\x88',
'CjkUnifiedIdeograph6da9': '\xe6\xb6\xa9',
'UnicodeCharacter0xa48e': '\xea\x92\x8e',
'UnicodeCharacter0xa48d': '\xea\x92\x8d',
'LatinLetterSmallCapitalOu': '\xe1\xb4\x95',
'CjkUnifiedIdeograph896d': '\xe8\xa5\xad',
'UnicodeCharacter0xf686': '\xef\x9a\x86',
'CjkUnifiedIdeograph6da8': '\xe6\xb6\xa8',
'CjkUnifiedIdeograph896a': '\xe8\xa5\xaa',
'UnicodeCharacter0xf681': '\xef\x9a\x81',
'CjkUnifiedIdeograph896c': '\xe8\xa5\xac',
'CjkUnifiedIdeograph896b': '\xe8\xa5\xab',
'CombiningCyrillicLetterEn': '\xe2\xb7\xa9',
'HangulJungseongYuye': '\xe1\x86\x92',
'HangulSyllableDdwess': '\xeb\x9b\xa8',
'HangulSyllableDdaeng': '\xeb\x95\xa1',
'CjkUnifiedIdeograph5b9d': '\xe5\xae\x9d',
'CjkUnifiedIdeograph5b9e': '\xe5\xae\x9e',
'CjkUnifiedIdeograph5b9f': '\xe5\xae\x9f',
'LeftwardsTripleArrow': '\xe2\x87\x9a',
'CjkUnifiedIdeograph5b9a': '\xe5\xae\x9a',
'CjkUnifiedIdeograph5b9b': '\xe5\xae\x9b',
'CjkUnifiedIdeograph5b9c': '\xe5\xae\x9c',
'HangulSyllableTyagg': '\xed\x83\xba',
'CjkUnifiedIdeograph8969': '\xe8\xa5\xa9',
'CjkUnifiedIdeograph8968': '\xe8\xa5\xa8',
'HangulSyllableHanh': '\xed\x95\x9e',
'CjkUnifiedIdeograph8965': '\xe8\xa5\xa5',
'CjkUnifiedIdeograph8964': '\xe8\xa5\xa4',
'CjkUnifiedIdeograph8967': '\xe8\xa5\xa7',
'CjkUnifiedIdeograph8966': '\xe8\xa5\xa6',
'ThaanaLetterTo': '\xde\xa0',
'CjkUnifiedIdeograph8960': '\xe8\xa5\xa0',
'CjkUnifiedIdeograph8963': '\xe8\xa5\xa3',
'CjkUnifiedIdeograph8962': '\xe8\xa5\xa2',
'TelephoneSign': '\xe2\x84\xa1',
'HangulSyllableNolt': '\xeb\x86\x85',
'HangulSyllableTyags': '\xed\x83\xbb',
'NewTaiLueLetterLowTa': '\xe1\xa6\x91',
'SinhalaLetterOyanna': '\xe0\xb6\x94',
'UnicodeCharacter0xec4d': '\xee\xb1\x8d',
'LatinSmallLetterYWithHookAbove': '\xe1\xbb\xb7',
'IdeographicTelegraphSymbolForHourFourteen': '\xe3\x8d\xa6',
'CjkUnifiedIdeograph387c': '\xe3\xa1\xbc',
'SmallEmDash': '\xef\xb9\x98',
'ArmenianCapitalLetterEch': '\xd4\xb5',
'zeta': '\xce\xb6',
'LatinSmallLetterGWithCedilla': '\xc4\xa3',
'CjkUnifiedIdeograph834c': '\xe8\x8d\x8c',
'CjkUnifiedIdeograph834b': '\xe8\x8d\x8b',
'CjkUnifiedIdeograph834e': '\xe8\x8d\x8e',
'CjkUnifiedIdeograph834d': '\xe8\x8d\x8d',
'CjkUnifiedIdeograph834f': '\xe8\x8d\x8f',
'CjkUnifiedIdeograph75db': '\xe7\x97\x9b',
'CjkUnifiedIdeograph75dc': '\xe7\x97\x9c',
'HangulSyllableRwinh': '\xeb\xa5\x82',
'CjkUnifiedIdeograph75da': '\xe7\x97\x9a',
'CjkUnifiedIdeograph75df': '\xe7\x97\x9f',
'CjkUnifiedIdeograph75dd': '\xe7\x97\x9d',
'CjkUnifiedIdeograph75de': '\xe7\x97\x9e',
'CjkUnifiedIdeograph8a66': '\xe8\xa9\xa6',
'UnicodeCharacter0xec4b': '\xee\xb1\x8b',
'HangulSyllableDonh': '\xeb\x8f\x8a',
'UnicodeCharacter0xec4c': '\xee\xb1\x8c',
'BulletOperator': '\xe2\x88\x99',
'HangulSyllableTweobs': '\xed\x89\x9a',
'RotatedFloralHeartBullet': '\xe2\x9d\xa7',
'CjkUnifiedIdeograph8a62': '\xe8\xa9\xa2',
'WhiteClubSuit': '\xe2\x99\xa7',
'UnicodeCharacter0xda4d': '\xed\xa9\x8d',
'UnicodeCharacter0xf6ea': '\xef\x9b\xaa',
'TopTortoiseShellBracket': '\xe2\x8f\xa0',
'NewTaiLueLetterHighTha': '\xe1\xa6\x8f',
'UpperRightShadowedWhiteSquare': '\xe2\x9d\x92',
'KatakanaLetterRo': '\xe3\x83\xad',
'KatakanaLetterSmallTo': '\xe3\x87\xb3',
'SundaneseLetterPa': '\xe1\xae\x95',
'CjkUnifiedIdeograph8340': '\xe8\x8d\x80',
'CjkUnifiedIdeograph75d8': '\xe7\x97\x98',
'CjkUnifiedIdeograph75d9': '\xe7\x97\x99',
'CjkUnifiedIdeograph8345': '\xe8\x8d\x85',
'CjkUnifiedIdeograph8344': '\xe8\x8d\x84',
'CjkUnifiedIdeograph8347': '\xe8\x8d\x87',
'KatakanaLetterSmallTu': '\xe3\x83\x83',
'CjkUnifiedIdeograph75d2': '\xe7\x97\x92',
'CjkUnifiedIdeograph75d3': '\xe7\x97\x93',
'CjkUnifiedIdeograph75d0': '\xe7\x97\x90',
'CjkUnifiedIdeograph75d1': '\xe7\x97\x91',
'CjkUnifiedIdeograph75d6': '\xe7\x97\x96',
'CjkUnifiedIdeograph75d7': '\xe7\x97\x97',
'CjkUnifiedIdeograph75d4': '\xe7\x97\x94',
'CjkUnifiedIdeograph75d5': '\xe7\x97\x95',
'GreekCapitalLetterIotaWithPsiliAndOxia': '\xe1\xbc\xbc',
'CjkUnifiedIdeograph976a': '\xe9\x9d\xaa',
'CjkUnifiedIdeograph976b': '\xe9\x9d\xab',
'CjkUnifiedIdeograph976c': '\xe9\x9d\xac',
'CjkUnifiedIdeograph7df9': '\xe7\xb7\xb9',
'CjkUnifiedIdeograph7df8': '\xe7\xb7\xb8',
'CjkUnifiedIdeograph976f': '\xe9\x9d\xaf',
'CjkUnifiedIdeograph6daf': '\xe6\xb6\xaf',
'CjkUnifiedIdeograph4fca': '\xe4\xbf\x8a',
'CjkUnifiedIdeograph7df4': '\xe7\xb7\xb4',
'CjkUnifiedIdeograph4fcc': '\xe4\xbf\x8c',
'CjkUnifiedIdeograph4fcb': '\xe4\xbf\x8b',
'CjkUnifiedIdeograph4fce': '\xe4\xbf\x8e',
'CjkUnifiedIdeograph4fcd': '\xe4\xbf\x8d',
'CjkUnifiedIdeograph7df3': '\xe7\xb7\xb3',
'CjkUnifiedIdeograph4fcf': '\xe4\xbf\x8f',
'CjkUnifiedIdeograph561a': '\xe5\x98\x9a',
'CjkUnifiedIdeograph561b': '\xe5\x98\x9b',
'CjkUnifiedIdeograph561c': '\xe5\x98\x9c',
'CjkUnifiedIdeograph561d': '\xe5\x98\x9d',
'CjkUnifiedIdeograph561e': '\xe5\x98\x9e',
'CjkUnifiedIdeograph561f': '\xe5\x98\x9f',
'PresentationFormForVerticalLeftWhiteLenticularBracket': '\xef\xb8\x97',
'EthiopicSyllablePe': '\xe1\x8d\x95',
'HangulSyllableTi': '\xed\x8b\xb0',
'VaiSyllableKo': '\xea\x97\x9b',
'EthiopicSyllablePa': '\xe1\x8d\x90',
'VaiSyllableKi': '\xea\x95\x83',
'VaiSyllableKe': '\xea\x98\x83',
'EthiopicSyllablePo': '\xe1\x8d\x96',
'EthiopicSyllablePi': '\xe1\x8d\x92',
'VaiSyllableKa': '\xea\x95\xaa',
'HangulSyllableTe': '\xed\x85\x8c',
'HangulSyllableGgeuss': '\xeb\x81\x98',
'EthiopicSyllablePu': '\xe1\x8d\x91',
'HangulSyllableGebs': '\xea\xb2\x9e',
'VaiSyllableKu': '\xea\x96\xb4',
'HangulSyllableTu': '\xed\x88\xac',
'CjkUnifiedIdeograph5610': '\xe5\x98\x90',
'CjkUnifiedIdeograph5611': '\xe5\x98\x91',
'CjkUnifiedIdeograph5612': '\xe5\x98\x92',
'CjkUnifiedIdeograph5613': '\xe5\x98\x93',
'CjkUnifiedIdeograph5614': '\xe5\x98\x94',
'CjkUnifiedIdeograph5615': '\xe5\x98\x95',
'CjkUnifiedIdeograph5616': '\xe5\x98\x96',
'CjkUnifiedIdeograph5617': '\xe5\x98\x97',
'CjkUnifiedIdeograph5618': '\xe5\x98\x98',
'CjkUnifiedIdeograph5619': '\xe5\x98\x99',
'CanadianSyllabicsYwii': '\xe1\x94\xb3',
'HangulSyllableGgonh': '\xea\xbc\xb2',
'HangulSyllableGgonj': '\xea\xbc\xb1',
'CjkUnifiedIdeograph4fc9': '\xe4\xbf\x89',
'CjkUnifiedIdeograph4fc8': '\xe4\xbf\x88',
'CjkUnifiedIdeograph9762': '\xe9\x9d\xa2',
'CjkUnifiedIdeograph9763': '\xe9\x9d\xa3',
'CjkUnifiedIdeograph9764': '\xe9\x9d\xa4',
'CjkUnifiedIdeograph9765': '\xe9\x9d\xa5',
'CjkUnifiedIdeograph9766': '\xe9\x9d\xa6',
'CjkUnifiedIdeograph9767': '\xe9\x9d\xa7',
'CjkUnifiedIdeograph4fc1': '\xe4\xbf\x81',
'CjkUnifiedIdeograph4fc0': '\xe4\xbf\x80',
'ParenthesizedNumberNineteen': '\xe2\x92\x86',
'CjkUnifiedIdeograph4fc2': '\xe4\xbf\x82',
'CjkUnifiedIdeograph4fc5': '\xe4\xbf\x85',
'Coffin': '\xe2\x9a\xb0',
'CjkUnifiedIdeograph4fc7': '\xe4\xbf\x87',
'CjkUnifiedIdeograph4fc6': '\xe4\xbf\x86',
'NorthEastArrowAndSouthEastArrow': '\xe2\xa4\xa8',
'HangulSyllableGyilm': '\xea\xb8\x9e',
'y': '\xcf\x88',
'LatinCapitalLetterBWithLineBelow': '\xe1\xb8\x86',
'HangulSyllableGolh': '\xea\xb3\xaf',
'LatinLetterSmallCapitalOpenO': '\xe1\xb4\x90',
'HangulSyllableKwalh': '\xec\xbd\xbf',
'UnicodeCharacter0xda0b': '\xed\xa8\x8b',
'GurmukhiLetterTta': '\xe0\xa8\x9f',
'HangulSyllableKeuc': '\xed\x82\x83',
'CjkUnifiedIdeograph351e': '\xe3\x94\x9e',
'CjkUnifiedIdeograph351d': '\xe3\x94\x9d',
'CjkUnifiedIdeograph351f': '\xe3\x94\x9f',
'CjkUnifiedIdeograph351a': '\xe3\x94\x9a',
'CjkUnifiedIdeograph351c': '\xe3\x94\x9c',
'CjkUnifiedIdeograph351b': '\xe3\x94\x9b',
'WhiteCircle': '\xe2\x97\x8b',
'HangulSyllableBwaet': '\xeb\xb5\x85',
'HangulSyllableNwaep': '\xeb\x87\x8a',
'RightBlackTortoiseShellBracket': '\xe2\xa6\x98',
'RightDottedSubstitutionBracket': '\xe2\xb8\x85',
'CjkUnifiedIdeograph9bcc': '\xe9\xaf\x8c',
'CjkUnifiedIdeograph57e3': '\xe5\x9f\xa3',
'CombiningSquareBelow': '\xcc\xbb',
'HangulSyllableMolt': '\xeb\xaa\xb5',
'HangulSyllableKeud': '\xed\x81\xb3',
'CyrillicCapitalLetterGheWithUpturn': '\xd2\x90',
'UnicodeCharacter0xa4de': '\xea\x93\x9e',
'HangulSyllableKeuj': '\xed\x82\x82',
'CjkUnifiedIdeograph7b35': '\xe7\xac\xb5',
'UnicodeCharacter0xde6a': '\xed\xb9\xaa',
'HangulSyllableHweos': '\xed\x9b\xb3',
'HangulSyllableHweop': '\xed\x9b\xba',
'HangulSyllableHweot': '\xed\x9b\xb9',
'HangulSyllableHweok': '\xed\x9b\xb8',
'HangulSyllableHweoj': '\xed\x9b\xb6',
'YiSyllablePiep': '\xea\x80\xbe',
'HangulSyllableHweoh': '\xed\x9b\xbb',
'CjkUnifiedIdeograph3519': '\xe3\x94\x99',
'CjkUnifiedIdeograph3518': '\xe3\x94\x98',
'HangulSyllableHweom': '\xed\x9b\xb0',
'HangulSyllableHweol': '\xed\x9b\xa8',
'CjkUnifiedIdeograph3515': '\xe3\x94\x95',
'CjkUnifiedIdeograph3514': '\xe3\x94\x94',
'UnicodeCharacter0x8dd': '\xe0\xa3\x9d',
'CjkUnifiedIdeograph3516': '\xe3\x94\x96',
'CjkUnifiedIdeograph3511': '\xe3\x94\x91',
'BraillePatternDots12345678': '\xe2\xa3\xbf',
'CjkUnifiedIdeograph3513': '\xe3\x94\x93',
'CjkUnifiedIdeograph3512': '\xe3\x94\x92',
'OriyaLetterUu': '\xe0\xac\x8a',
'CjkCompatibilityIdeographfa68': '\xef\xa9\xa8',
'LatinCapitalLetterNWithSmallLetterJ': '\xc7\x8b',
'RisingDiagonalCrossingSouthEastArrow': '\xe2\xa4\xb0',
'LatinSmallLetterOWithLongStrokeOverlay': '\xea\x9d\x8b',
'EthiopicSyllableToa': '\xe2\xb6\x86',
'CanadianSyllabicsLii': '\xe1\x93\x96',
'CjkUnifiedIdeograph8b1f': '\xe8\xac\x9f',
'CjkUnifiedIdeograph8b1e': '\xe8\xac\x9e',
'CjkUnifiedIdeograph8b1d': '\xe8\xac\x9d',
'CjkUnifiedIdeograph8b1c': '\xe8\xac\x9c',
'CjkUnifiedIdeograph8b1b': '\xe8\xac\x9b',
'CjkUnifiedIdeograph8b1a': '\xe8\xac\x9a',
'CjkCompatibilityIdeographfa62': '\xef\xa9\xa2',
'UnicodeCharacter0xee79': '\xee\xb9\xb9',
'UnicodeCharacter0xee78': '\xee\xb9\xb8',
'GujaratiLetterSsa': '\xe0\xaa\xb7',
'UnicodeCharacter0xee73': '\xee\xb9\xb3',
'UnicodeCharacter0xee72': '\xee\xb9\xb2',
'LatinSmallLetterYogh': '\xc8\x9d',
'UnicodeCharacter0xee70': '\xee\xb9\xb0',
'UnicodeCharacter0xee77': '\xee\xb9\xb7',
'UnicodeCharacter0xee76': '\xee\xb9\xb6',
'UnicodeCharacter0xee75': '\xee\xb9\xb5',
'YiSyllableMax': '\xea\x82\xb6',
'GujaratiVowelSignO': '\xe0\xab\x8b',
'SmallFullStop': '\xef\xb9\x92',
'GujaratiVowelSignI': '\xe0\xaa\xbf',
'KhmerIndependentVowelQe': '\xe1\x9e\xaf',
'HangulSyllablePac': '\xed\x8c\xa3',
'GujaratiVowelSignE': '\xe0\xab\x87',
'KhmerIndependentVowelQi': '\xe1\x9e\xa5',
'UnicodeCharacter0xf8f6': '\xef\xa3\xb6',
'KhmerIndependentVowelQu': '\xe1\x9e\xa7',
'GujaratiVowelSignU': '\xe0\xab\x81',
'HangulSyllableJwen': '\xec\xa4\xb8',
'CjkUnifiedIdeograph9f5f': '\xe9\xbd\x9f',
'KayahLiLetterSha': '\xea\xa4\x8f',
'UnicodeCharacter0xee7c': '\xee\xb9\xbc',
'UnicodeCharacter0xee7b': '\xee\xb9\xbb',
'UnicodeCharacter0xee7a': '\xee\xb9\xba',
'HangulSyllableJwel': '\xec\xa4\xbc',
'UnicodeCharacter0xf8f7': '\xef\xa3\xb7',
'UnicodeCharacter0xee7f': '\xee\xb9\xbf',
'UnicodeCharacter0xee7e': '\xee\xb9\xbe',
'UnicodeCharacter0xee7d': '\xee\xb9\xbd',
'HangulSyllableJwec': '\xec\xa5\x8b',
'UnicodeCharacter0xd8c0': '\xed\xa3\x80',
'[1]': '\xe2\x82\x81',
'CjkUnifiedIdeograph8b19': '\xe8\xac\x99',
'CjkUnifiedIdeograph8b18': '\xe8\xac\x98',
'CjkUnifiedIdeograph8b17': '\xe8\xac\x97',
'CjkUnifiedIdeograph8b16': '\xe8\xac\x96',
'CjkUnifiedIdeograph8b15': '\xe8\xac\x95',
'CjkUnifiedIdeograph8b14': '\xe8\xac\x94',
'CjkUnifiedIdeograph8b13': '\xe8\xac\x93',
'CjkUnifiedIdeograph8b12': '\xe8\xac\x92',
'CjkUnifiedIdeograph8b11': '\xe8\xac\x91',
'CjkUnifiedIdeograph8b10': '\xe8\xac\x90',
'HangulSyllableDoegg': '\xeb\x90\x9a',
'HangulJongseongMieumchieuch': '\xe1\x87\xa0',
'ArabicLigatureMeemWithMeemIsolatedForm': '\xef\xb1\x88',
'HangulSyllableNwing': '\xeb\x89\xad',
'HangulSyllableJweg': '\xec\xa4\xb5',
'UnicodeCharacter0xf8f8': '\xef\xa3\xb8',
'CjkCompatibilityIdeographfa75': '\xef\xa9\xb5',
'VaiSyllableThi': '\xea\x94\xb4',
'ContinuousUnderlineSymbol': '\xe2\x8e\x81',
'HangulSyllableDoegs': '\xeb\x90\x9b',
'LaoVowelSignUu': '\xe0\xba\xb9',
'CjkUnifiedIdeograph4983': '\xe4\xa6\x83',
'CjkUnifiedIdeograph4982': '\xe4\xa6\x82',
'CjkUnifiedIdeograph4981': '\xe4\xa6\x81',
'CjkUnifiedIdeograph4980': '\xe4\xa6\x80',
'CjkUnifiedIdeograph4987': '\xe4\xa6\x87',
'CjkUnifiedIdeograph4986': '\xe4\xa6\x86',
'CjkUnifiedIdeograph4985': '\xe4\xa6\x85',
'CjkUnifiedIdeograph4984': '\xe4\xa6\x84',
'UnicodeCharacter0xf8f9': '\xef\xa3\xb9',
'CjkUnifiedIdeograph4989': '\xe4\xa6\x89',
'CjkUnifiedIdeograph4988': '\xe4\xa6\x88',
'HangulSyllableTweong': '\xed\x89\x9d',
'TagbanwaLetterMa': '\xe1\x9d\xab',
'HangulSyllableHwiss': '\xed\x9c\xac',
'PhagspaLetterBa': '\xea\xa1\x8e',
'ArabicLigatureLamWithKhahIsolatedForm': '\xef\xb1\x81',
'HangulSyllableMyabs': '\xeb\xa8\x92',
'VaiSyllableTho': '\xea\x97\x8c',
'UnicodeCharacter0xe744': '\xee\x9d\x84',
'CjkUnifiedIdeograph57ea': '\xe5\x9f\xaa',
'LaoLetterKo': '\xe0\xba\x81',
'PresentationFormForVerticalRightDoubleAngleBracket': '\xef\xb8\xbe',
'DiamondWithRightHalfBlack': '\xe2\xac\x97',
'BalinesePameneng': '\xe1\xad\xa0',
'CjkUnifiedIdeograph57ec': '\xe5\x9f\xac',
'CjkUnifiedIdeograph57eb': '\xe5\x9f\xab',
'HangulSyllableBbigg': '\xec\x82\x92',
'CjkUnifiedIdeograph57ee': '\xe5\x9f\xae',
'CjkUnifiedIdeograph57ed': '\xe5\x9f\xad',
'CjkUnifiedIdeograph498c': '\xe4\xa6\x8c',
'CjkUnifiedIdeograph498b': '\xe4\xa6\x8b',
'CjkUnifiedIdeograph498a': '\xe4\xa6\x8a',
'CjkUnifiedIdeograph498f': '\xe4\xa6\x8f',
'CjkUnifiedIdeograph498e': '\xe4\xa6\x8e',
'CjkUnifiedIdeograph498d': '\xe4\xa6\x8d',
'CombiningCyrillicLetterDjerv': '\xe2\xb7\xb8',
'CjkUnifiedIdeograph7800': '\xe7\xa0\x80',
'CjkUnifiedIdeograph57ef': '\xe5\x9f\xaf',
'BopomofoLetterZi': '\xe3\x86\xa1',
'BopomofoLetterZh': '\xe3\x84\x93',
'KatakanaLetterU': '\xe3\x82\xa6',
'ArabicLetterAlefWithHamzaBelowIsolatedForm': '\xef\xba\x87',
'KatakanaLetterE': '\xe3\x82\xa8',
'KatakanaLetterA': '\xe3\x82\xa2',
'HangulSyllableSeult': '\xec\x8a\xb1',
'KangxiRadicalDipper': '\xe2\xbd\x83',
'KatakanaLetterO': '\xe3\x82\xaa',
'KatakanaLetterN': '\xe3\x83\xb3',
'KatakanaLetterI': '\xe3\x82\xa4',
'CjkUnifiedIdeograph7982': '\xe7\xa6\x82',
'CjkUnifiedIdeograph7983': '\xe7\xa6\x83',
'CjkUnifiedIdeograph7980': '\xe7\xa6\x80',
'CjkUnifiedIdeograph7981': '\xe7\xa6\x81',
'CjkUnifiedIdeograph7986': '\xe7\xa6\x86',
'CjkUnifiedIdeograph7987': '\xe7\xa6\x87',
'CjkUnifiedIdeograph7984': '\xe7\xa6\x84',
'CjkUnifiedIdeograph7985': '\xe7\xa6\x85',
'ModifierLetterChineseToneYinRu': '\xea\x9c\x86',
'CjkUnifiedIdeograph7988': '\xe7\xa6\x88',
'CjkUnifiedIdeograph7989': '\xe7\xa6\x89',
'BlackChessKnight': '\xe2\x99\x9e',
'HangulSyllableKyoh': '\xec\xbf\x9f',
'UnicodeCharacter0x20c7': '\xe2\x83\x87',
'UnicodeCharacter0x20c6': '\xe2\x83\x86',
'UnicodeCharacter0x20c5': '\xe2\x83\x85',
'CjkUnifiedIdeograph4324': '\xe4\x8c\xa4',
'CjkUnifiedIdeograph4323': '\xe4\x8c\xa3',
'CjkUnifiedIdeograph4322': '\xe4\x8c\xa2',
'UnicodeCharacter0x20c1': '\xe2\x83\x81',
'CjkUnifiedIdeograph4320': '\xe4\x8c\xa0',
'UnicodeCharacter0xe512': '\xee\x94\x92',
'UnicodeCharacter0x20c9': '\xe2\x83\x89',
'UnicodeCharacter0x20c8': '\xe2\x83\x88',
'YiSyllableNbyx': '\xea\x82\x8c',
'ArabicLigatureSeenWithKhahIsolatedForm': '\xef\xb0\x9e',
'UnicodeCharacter0xe82e': '\xee\xa0\xae',
'YiSyllableNbyr': '\xea\x82\x90',
'YiSyllableNbyp': '\xea\x82\x8e',
'YiSyllableNbyt': '\xea\x82\x8b',
'UnicodeCharacter0x20cf': '\xe2\x83\x8f',
'CjkUnifiedIdeograph432e': '\xe4\x8c\xae',
'UnicodeCharacter0x20cd': '\xe2\x83\x8d',
'CjkUnifiedIdeograph432c': '\xe4\x8c\xac',
'CjkUnifiedIdeograph432b': '\xe4\x8c\xab',
'CjkUnifiedIdeograph432a': '\xe4\x8c\xaa',
'HangulSyllableTeogs': '\xed\x84\xb3',
'CjkUnifiedIdeograph798b': '\xe7\xa6\x8b',
'CjkUnifiedIdeograph798c': '\xe7\xa6\x8c',
'VaiSyllableWoon': '\xea\x95\xb5',
'CjkUnifiedIdeograph798a': '\xe7\xa6\x8a',
'CjkUnifiedIdeograph798f': '\xe7\xa6\x8f',
'CjkUnifiedIdeograph798d': '\xe7\xa6\x8d',
'CjkUnifiedIdeograph798e': '\xe7\xa6\x8e',
'HangulSyllableDej': '\xeb\x8e\x86',
'HangulSyllableTeogg': '\xed\x84\xb2',
'UnicodeCharacter0x3249': '\xe3\x89\x89',
'UnicodeCharacter0x3248': '\xe3\x89\x88',
'HangulSyllableJels': '\xec\xa0\xa8',
'HangulSyllableJelp': '\xec\xa0\xaa',
'UnicodeCharacter0xe6a7': '\xee\x9a\xa7',
'PhagspaSubjoinedLetterRa': '\xea\xa1\xb1',
'UnicodeCharacter0x3247': '\xe3\x89\x87',
'UnicodeCharacter0x3246': '\xe3\x89\x86',
'UnicodeCharacter0x3245': '\xe3\x89\x85',
'UnicodeCharacter0x3244': '\xe3\x89\x84',
'HangulSyllableJelg': '\xec\xa0\xa5',
'CjkUnifiedIdeograph919f': '\xe9\x86\x9f',
'HangulSyllableJelb': '\xec\xa0\xa7',
'LatinSmallLetterAWithMacron': '\xc4\x81',
'HangulSyllableJelm': '\xec\xa0\xa6',
'LatinCapitalLetterUWithInvertedBreve': '\xc8\x96',
'UnicodeCharacter0xe6a3': '\xee\x9a\xa3',
'MalayalamLetterVocalicL': '\xe0\xb4\x8c',
'TeluguFractionDigitTwoForOddPowersOfFour': '\xe0\xb1\xba',
'HangulSyllableDeh': '\xeb\x8e\x8b',
'HangulSyllableRyibs': '\xeb\xa6\xa2',
'LatinSmallLetterUWithGrave': '\xc3\xb9',
'LatinSmallLetterEWithCircumflexBelow': '\xe1\xb8\x99',
'LatinSmallLetterOWithCircumflexAndDotBelow': '\xe1\xbb\x99',
'HangulSyllableHweoss': '\xed\x9b\xb4',
'UnicodeCharacter0x324c': '\xe3\x89\x8c',
'UnicodeCharacter0x324b': '\xe3\x89\x8b',
'UnicodeCharacter0x324a': '\xe3\x89\x8a',
'UnicodeCharacter0x324f': '\xe3\x89\x8f',
'UnicodeCharacter0x324e': '\xe3\x89\x8e',
'UnicodeCharacter0x324d': '\xe3\x89\x8d',
'HangulSyllableSsulb': '\xec\x91\xaf',
'KatakanaLetterSmallKu': '\xe3\x87\xb0',
'TibetanDigitHalfFive': '\xe0\xbc\xae',
'MongolianDigitFour': '\xe1\xa0\x94',
'LatinCapitalLetterIWithDiaeresisAndAcute': '\xe1\xb8\xae',
'CjkUnifiedIdeograph57e1': '\xe5\x9f\xa1',
'BraillePatternDots13567': '\xe2\xa1\xb5',
'YiSyllableSep': '\xea\x8c\x9d',
'KatakanaLetterSmallKe': '\xe3\x83\xb6',
'CjkUnifiedIdeograph57e0': '\xe5\x9f\xa0',
'HangulSyllableYubs': '\xec\x9c\xb2',
'ArabicNumberSign': '\xd8\x80',
'BraillePatternDots13568': '\xe2\xa2\xb5',
'CjkUnifiedIdeograph6215': '\xe6\x88\x95',
'CjkUnifiedIdeograph6214': '\xe6\x88\x94',
'CjkUnifiedIdeograph6217': '\xe6\x88\x97',
'CjkUnifiedIdeograph6216': '\xe6\x88\x96',
'CjkUnifiedIdeograph6211': '\xe6\x88\x91',
'CjkUnifiedIdeograph6210': '\xe6\x88\x90',
'CjkUnifiedIdeograph6213': '\xe6\x88\x93',
'CjkUnifiedIdeograph6212': '\xe6\x88\x92',
'KhmerSignPhnaekMuan': '\xe1\x9f\x99',
'CjkUnifiedIdeograph6219': '\xe6\x88\x99',
'CjkUnifiedIdeograph6218': '\xe6\x88\x98',
'CjkUnifiedIdeograph9249': '\xe9\x89\x89',
'CjkUnifiedIdeograph57e4': '\xe5\x9f\xa4',
'CjkUnifiedIdeograph68e7': '\xe6\xa3\xa7',
'CjkUnifiedIdeograph68e6': '\xe6\xa3\xa6',
'CjkUnifiedIdeograph68e5': '\xe6\xa3\xa5',
'CjkUnifiedIdeograph68e4': '\xe6\xa3\xa4',
'CjkUnifiedIdeograph68e3': '\xe6\xa3\xa3',
'CjkUnifiedIdeograph57e7': '\xe5\x9f\xa7',
'CjkUnifiedIdeograph68e1': '\xe6\xa3\xa1',
'CjkUnifiedIdeograph68e0': '\xe6\xa3\xa0',
'CjkUnifiedIdeograph57e6': '\xe5\x9f\xa6',
'CjkUnifiedIdeograph4c58': '\xe4\xb1\x98',
'CjkUnifiedIdeograph68e9': '\xe6\xa3\xa9',
'CjkUnifiedIdeograph68e8': '\xe6\xa3\xa8',
'UnicodeCharacter0xa4da': '\xea\x93\x9a',
'YiSyllableSex': '\xea\x8c\x9b',
'HangulSyllableMoeng': '\xeb\xac\x91',
'HangulSyllableMoenh': '\xeb\xac\x82',
'BoxDrawingsUpDoubleAndHorizontalSingle': '\xe2\x95\xa8',
'HangulSyllableMoenj': '\xeb\xac\x81',
'HangulSyllableKinj': '\xed\x82\xa9',
'ArabicLetterKafMedialForm': '\xef\xbb\x9c',
'ArabicLigatureShaddaWithDammatanIsolatedForm': '\xef\xb1\x9e',
'BengaliDigitEight': '\xe0\xa7\xae',
'FullwidthCircumflexAccent': '\xef\xbc\xbe',
'UnicodeCharacter0xf251': '\xef\x89\x91',
'CanadianSyllabicsKwii': '\xe1\x91\xb8',
'CjkUnifiedIdeograph68ef': '\xe6\xa3\xaf',
'CjkUnifiedIdeograph68ee': '\xe6\xa3\xae',
'CjkUnifiedIdeograph68ed': '\xe6\xa3\xad',
'CjkUnifiedIdeograph68ec': '\xe6\xa3\xac',
'CjkUnifiedIdeograph5d07': '\xe5\xb4\x87',
'CjkUnifiedIdeograph68ea': '\xe6\xa3\xaa',
'HangulSyllableGoelb': '\xea\xb4\xbf',
'LepchaLetterSha': '\xe1\xb0\xa1',
'HangulSyllableGoelg': '\xea\xb4\xbd',
'SubsetAboveRightwardsArrow': '\xe2\xa5\xb9',
'CjkUnifiedIdeograph621e': '\xe6\x88\x9e',
'CjkUnifiedIdeograph621d': '\xe6\x88\x9d',
'CjkUnifiedIdeograph924e': '\xe9\x89\x8e',
'CjkUnifiedIdeograph621f': '\xe6\x88\x9f',
'CjkUnifiedIdeograph621a': '\xe6\x88\x9a',
'CjkUnifiedIdeograph924b': '\xe9\x89\x8b',
'CjkUnifiedIdeograph621c': '\xe6\x88\x9c',
'CjkUnifiedIdeograph621b': '\xe6\x88\x9b',
'HangulSyllableGoelp': '\xea\xb5\x82',
'HangulSyllableGoels': '\xea\xb5\x80',
'IdeographicTelegraphSymbolForDayEleven': '\xe3\x8f\xaa',
'HangulSyllableGoelt': '\xea\xb5\x81',
'LepchaPunctuationCerwa': '\xe1\xb0\xbd',
'CjkUnifiedIdeograph83d2': '\xe8\x8f\x92',
'HangulSyllableGyigs': '\xea\xb8\x97',
'LatinCapitalLetterCuatrilloWithComma': '\xea\x9c\xae',
'DivisionTimes': '\xe2\x8b\x87',
'CjkUnifiedIdeograph9cd6': '\xe9\xb3\x96',
'UnicodeCharacter0xf892': '\xef\xa2\x92',
'UnicodeCharacter0xf06f': '\xef\x81\xaf',
'CjkUnifiedIdeograph9cd7': '\xe9\xb3\x97',
'GreekCapitalLetterOmegaWithPsiliAndVariaAndProsgegrammeni': '\xe1\xbe\xaa',
'GurmukhiSignAdakBindi': '\xe0\xa8\x81',
'UnicodeCharacter0xdadb': '\xed\xab\x9b',
'CjkUnifiedIdeograph9cd4': '\xe9\xb3\x94',
'CjkUnifiedIdeograph4ce2': '\xe4\xb3\xa2',
'CjkUnifiedIdeograph4ce3': '\xe4\xb3\xa3',
'CjkUnifiedIdeograph4ce0': '\xe4\xb3\xa0',
'CjkUnifiedIdeograph4ce1': '\xe4\xb3\xa1',
'CjkUnifiedIdeograph4ce6': '\xe4\xb3\xa6',
'CjkUnifiedIdeograph4ce7': '\xe4\xb3\xa7',
'CjkUnifiedIdeograph4ce4': '\xe4\xb3\xa4',
'CjkUnifiedIdeograph4ce5': '\xe4\xb3\xa5',
'CjkUnifiedIdeograph4ce8': '\xe4\xb3\xa8',
'SucceedsAboveNotEqualTo': '\xe2\xaa\xb6',
'HangulSyllableSseuss': '\xec\x94\x84',
'CirculationFunction': '\xe2\xa8\x90',
'CjkUnifiedIdeograph9cd3': '\xe9\xb3\x93',
'UnicodeCharacter0xe1df': '\xee\x87\x9f',
'CjkUnifiedIdeograph78aa': '\xe7\xa2\xaa',
'CjkUnifiedIdeograph78ab': '\xe7\xa2\xab',
'CjkUnifiedIdeograph78ac': '\xe7\xa2\xac',
'CjkUnifiedIdeograph78ad': '\xe7\xa2\xad',
'CjkUnifiedIdeograph78ae': '\xe7\xa2\xae',
'CjkUnifiedIdeograph78af': '\xe7\xa2\xaf',
'UnicodeCharacter0xe1da': '\xee\x87\x9a',
'HangulSyllableJiss': '\xec\xa7\x94',
'CjkUnifiedIdeograph9cd1': '\xe9\xb3\x91',
'ArabicLetterNg': '\xda\xad',
'SquareYaaru': '\xe3\x8d\x8f',
'CjkUnifiedIdeograph3895': '\xe3\xa2\x95',
'CjkUnifiedIdeograph3896': '\xe3\xa2\x96',
'CjkUnifiedIdeograph3897': '\xe3\xa2\x97',
'CjkUnifiedIdeograph3890': '\xe3\xa2\x90',
'CjkUnifiedIdeograph3891': '\xe3\xa2\x91',
'CjkUnifiedIdeograph3892': '\xe3\xa2\x92',
'CjkUnifiedIdeograph3893': '\xe3\xa2\x93',
'SubscriptFive': '\xe2\x82\x85',
'CjkUnifiedIdeograph3898': '\xe3\xa2\x98',
'CjkUnifiedIdeograph3899': '\xe3\xa2\x99',
'CjkUnifiedIdeograph7164': '\xe7\x85\xa4',
'CjkUnifiedIdeograph7165': '\xe7\x85\xa5',
'CjkUnifiedIdeograph7166': '\xe7\x85\xa6',
'CjkUnifiedIdeograph7167': '\xe7\x85\xa7',
'CjkUnifiedIdeograph7160': '\xe7\x85\xa0',
'CjkUnifiedIdeograph7161': '\xe7\x85\xa1',
'CjkUnifiedIdeograph7162': '\xe7\x85\xa2',
'BalineseLetterZalSasak': '\xe1\xad\x8a',
'CjkUnifiedIdeograph7168': '\xe7\x85\xa8',
'CjkUnifiedIdeograph7169': '\xe7\x85\xa9',
'DevanagariSignVirama': '\xe0\xa5\x8d',
'CjkUnifiedIdeograph78a0': '\xe7\xa2\xa0',
'CjkUnifiedIdeograph78a1': '\xe7\xa2\xa1',
'CjkUnifiedIdeograph78a2': '\xe7\xa2\xa2',
'CjkUnifiedIdeograph78a3': '\xe7\xa2\xa3',
'CjkUnifiedIdeograph78a4': '\xe7\xa2\xa4',
'CjkUnifiedIdeograph78a5': '\xe7\xa2\xa5',
'CjkUnifiedIdeograph78a6': '\xe7\xa2\xa6',
'CjkUnifiedIdeograph78a7': '\xe7\xa2\xa7',
'CjkUnifiedIdeograph78a8': '\xe7\xa2\xa8',
'CjkUnifiedIdeograph78a9': '\xe7\xa2\xa9',
'CjkCompatibilityIdeographf9bd': '\xef\xa6\xbd',
'UnicodeCharacter0xe1d8': '\xee\x87\x98',
'UnicodeCharacter0xe1d9': '\xee\x87\x99',
'CjkUnifiedIdeograph4ceb': '\xe4\xb3\xab',
'CjkUnifiedIdeograph4cec': '\xe4\xb3\xac',
'CjkUnifiedIdeograph4cea': '\xe4\xb3\xaa',
'CjkUnifiedIdeograph4cef': '\xe4\xb3\xaf',
'UnicodeCharacter0xf254': '\xef\x89\x94',
'CjkUnifiedIdeograph4ced': '\xe4\xb3\xad',
'CjkUnifiedIdeograph4cee': '\xe4\xb3\xae',
'CjkUnifiedIdeograph83d6': '\xe8\x8f\x96',
'CjkUnifiedIdeograph716d': '\xe7\x85\xad',
'CjkUnifiedIdeograph716e': '\xe7\x85\xae',
'CjkUnifiedIdeograph716f': '\xe7\x85\xaf',
'UnicodeCharacter0xef33': '\xee\xbc\xb3',
'UnicodeCharacter0xef34': '\xee\xbc\xb4',
'ArmenianCapitalLetterAyb': '\xd4\xb1',
'CjkUnifiedIdeograph716b': '\xe7\x85\xab',
'CanadianSyllabicsYcreeW': '\xe1\x90\x9d',
'UnicodeCharacter0xef38': '\xee\xbc\xb8',
'CjkUnifiedIdeograph87f7': '\xe8\x9f\xb7',
'CopticSmallLetterSampi': '\xe2\xb3\x81',
'CjkUnifiedIdeograph389d': '\xe3\xa2\x9d',
'CjkUnifiedIdeograph389e': '\xe3\xa2\x9e',
'ParenthesizedHangulPhieuphA': '\xe3\x88\x9a',
'CjkUnifiedIdeograph389a': '\xe3\xa2\x9a',
'CjkUnifiedIdeograph389b': '\xe3\xa2\x9b',
'CjkUnifiedIdeograph389c': '\xe3\xa2\x9c',
'CjkUnifiedIdeograph87f4': '\xe8\x9f\xb4',
'BlackFourPointedStar': '\xe2\x9c\xa6',
'UnicodeCharacter0xa4e6': '\xea\x93\xa6',
'CjkUnifiedIdeograph64fb': '\xe6\x93\xbb',
'CjkUnifiedIdeograph4c5b': '\xe4\xb1\x9b',
'CjkUnifiedIdeograph9cba': '\xe9\xb2\xba',
'CjkUnifiedIdeograph64fa': '\xe6\x93\xba',
'CjkUnifiedIdeograph64ff': '\xe6\x93\xbf',
'CjkUnifiedIdeograph64fd': '\xe6\x93\xbd',
'CjkUnifiedIdeograph64fe': '\xe6\x93\xbe',
'CyrillicSmallLetterKaWithStroke': '\xd2\x9f',
'HangulSyllableKoelb': '\xec\xbe\xb3',
'HangulSyllableDed': '\xeb\x8d\xb7',
'CjkUnifiedIdeograph7113': '\xe7\x84\x93',
'RomanNumeralEight': '\xe2\x85\xa7',
'UnicodeCharacter0xf26e': '\xef\x89\xae',
'UnicodeCharacter0xeff6': '\xee\xbf\xb6',
'CjkUnifiedIdeograph9f24': '\xe9\xbc\xa4',
'HangulSyllableSeulb': '\xec\x8a\xaf',
'CjkUnifiedIdeograph4b91': '\xe4\xae\x91',
'LatinCapitalLetterPWithDotAbove': '\xe1\xb9\x96',
'UnicodeCharacter0xe8bd': '\xee\xa2\xbd',
'HangulSyllableTalt': '\xed\x83\x8d',
'frac': '\xe2\x81\x84',
'LatinCapitalLetterHWithDescender': '\xe2\xb1\xa7',
'ParenthesizedIdeographThree': '\xe3\x88\xa2',
'HangulSyllableGoeb': '\xea\xb5\x85',
'HangulSyllableBbwa': '\xeb\xbd\x9c',
'CjkUnifiedIdeograph64f2': '\xe6\x93\xb2',
'CjkUnifiedIdeograph64f3': '\xe6\x93\xb3',
'CjkUnifiedIdeograph64f0': '\xe6\x93\xb0',
'CjkUnifiedIdeograph64f1': '\xe6\x93\xb1',
'LatinSmallLetterSWithPalatalHook': '\xe1\xb6\x8a',
'CjkUnifiedIdeograph64f7': '\xe6\x93\xb7',
'CjkUnifiedIdeograph64f4': '\xe6\x93\xb4',
'CjkUnifiedIdeograph64f5': '\xe6\x93\xb5',
'CjkCompatibilityIdeographf929': '\xef\xa4\xa9',
'CjkUnifiedIdeograph64f8': '\xe6\x93\xb8',
'CjkUnifiedIdeograph64f9': '\xe6\x93\xb9',
'EthiopicSyllableDdaa': '\xe1\x8b\xbb',
'BengaliLetterBa': '\xe0\xa6\xac',
'GreekCapitalReversedLunateSigmaSymbol': '\xcf\xbd',
'CopticCapitalLetterDialectpAlef': '\xe2\xb2\xb2',
'BlackChessRook': '\xe2\x99\x9c',
'HangulSyllableKoelp': '\xec\xbe\xb6',
'HangulSyllableGoeh': '\xea\xb5\x8f',
'HangulSyllableKoels': '\xec\xbe\xb4',
'HalfwidthHangulLetterRieul': '\xef\xbe\xa9',
'UnicodeCharacter0xe6b4': '\xee\x9a\xb4',
'HangulSyllableGoej': '\xea\xb5\x8a',
'Fisheye': '\xe2\x97\x89',
'SylotiNagriSignDvisvara': '\xea\xa0\x82',
'CjkUnifiedIdeograph6a63': '\xe6\xa9\xa3',
'CjkUnifiedIdeograph6a62': '\xe6\xa9\xa2',
'CjkUnifiedIdeograph6a61': '\xe6\xa9\xa1',
'CjkUnifiedIdeograph6a60': '\xe6\xa9\xa0',
'CjkUnifiedIdeograph6a67': '\xe6\xa9\xa7',
'CjkUnifiedIdeograph6a66': '\xe6\xa9\xa6',
'CjkUnifiedIdeograph6a65': '\xe6\xa9\xa5',
'CjkUnifiedIdeograph6a64': '\xe6\xa9\xa4',
'HangulSyllableGoel': '\xea\xb4\xbc',
'CjkUnifiedIdeograph6a69': '\xe6\xa9\xa9',
'CjkUnifiedIdeograph6a68': '\xe6\xa9\xa8',
'CyrillicSmallLetterOmega': '\xd1\xa1',
'HangulSyllableGoem': '\xea\xb5\x84',
'_1': '\xe2\x82\x81',
'HangulSyllableGoen': '\xea\xb4\xb8',
'CjkUnifiedIdeograph7a8b': '\xe7\xaa\x8b',
'CjkUnifiedIdeograph7a8c': '\xe7\xaa\x8c',
'CjkUnifiedIdeograph7a8a': '\xe7\xaa\x8a',
'GeorgianCapitalLetterHe': '\xe1\x83\x81',
'LatinSmallLetterZWithDotAbove': '\xc5\xbc',
'CjkUnifiedIdeograph7a8e': '\xe7\xaa\x8e',
'HangulSyllableGgoh': '\xea\xbd\x87',
'ArabicLigatureNoonWithHahWithMeemInitialForm': '\xef\xb6\x95',
'LatinSmallLetterJWithCaron': '\xc7\xb0',
'UnicodeCharacter0xfe1a': '\xef\xb8\x9a',
'CjkUnifiedIdeograph86f8': '\xe8\x9b\xb8',
'CjkUnifiedIdeograph86f9': '\xe8\x9b\xb9',
'CjkUnifiedIdeograph86f6': '\xe8\x9b\xb6',
'CjkUnifiedIdeograph86f7': '\xe8\x9b\xb7',
'CjkUnifiedIdeograph86f4': '\xe8\x9b\xb4',
'CjkUnifiedIdeograph86f5': '\xe8\x9b\xb5',
'CjkUnifiedIdeograph86f2': '\xe8\x9b\xb2',
'CjkUnifiedIdeograph86f3': '\xe8\x9b\xb3',
'CjkUnifiedIdeograph86f0': '\xe8\x9b\xb0',
'CjkUnifiedIdeograph86f1': '\xe8\x9b\xb1',
'CjkUnifiedIdeograph7a88': '\xe7\xaa\x88',
'CjkUnifiedIdeograph7a89': '\xe7\xaa\x89',
'LaoLetterPo': '\xe0\xba\x9b',
'UnicodeCharacter0xd7d5': '\xed\x9f\x95',
'CjkCompatibilityIdeographf9b2': '\xef\xa6\xb2',
'CjkUnifiedIdeograph7a82': '\xe7\xaa\x82',
'CjkUnifiedIdeograph7a83': '\xe7\xaa\x83',
'CjkUnifiedIdeograph7a80': '\xe7\xaa\x80',
'CjkUnifiedIdeograph7a81': '\xe7\xaa\x81',
'CjkUnifiedIdeograph7a86': '\xe7\xaa\x86',
'CjkUnifiedIdeograph7a87': '\xe7\xaa\x87',
'CjkUnifiedIdeograph7a84': '\xe7\xaa\x84',
'CircledDash': '\xe2\x8a\x9d',
'CjkUnifiedIdeograph6a6c': '\xe6\xa9\xac',
'CjkUnifiedIdeograph6a6b': '\xe6\xa9\xab',
'NaryCircledTimesOperator': '\xe2\xa8\x82',
'CjkCompatibilityIdeographf9b0': '\xef\xa6\xb0',
'BalineseMusicalSymbolLefthandOpenPung': '\xe1\xad\xb9',
'CjkUnifiedIdeograph6a6f': '\xe6\xa9\xaf',
'CjkUnifiedIdeograph6a6e': '\xe6\xa9\xae',
'CjkUnifiedIdeograph6a6d': '\xe6\xa9\xad',
'CjkCompatibilityIdeographf9b7': '\xef\xa6\xb7',
'SymbolForEndOfTransmissionBlock': '\xe2\x90\x97',
'RightArrowWithSmallCircle': '\xe2\x87\xb4',
'EthiopicSyllableQee': '\xe1\x89\x84',
'AplFunctionalSymbolDownTackUnderbar': '\xe2\x8d\x8a',
'CjkUnifiedIdeograph86ff': '\xe8\x9b\xbf',
'CjkUnifiedIdeograph86fd': '\xe8\x9b\xbd',
'CjkUnifiedIdeograph86fe': '\xe8\x9b\xbe',
'CjkUnifiedIdeograph86fb': '\xe8\x9b\xbb',
'CjkUnifiedIdeograph86fc': '\xe8\x9b\xbc',
'CjkUnifiedIdeograph86fa': '\xe8\x9b\xba',
'HexagramForGatheringTogether': '\xe4\xb7\xac',
'HangulSyllableSeonj': '\xec\x84\xa1',
'HangulSyllableSeonh': '\xec\x84\xa2',
'CanadianSyllabicsFinalMiddleDot': '\xe1\x90\xa7',
'SectionSign': '\xc2\xa7',
'UnicodeCharacter0xfe1d': '\xef\xb8\x9d',
'UnicodeCharacter0xdfd1': '\xed\xbf\x91',
'HangulSyllableSeong': '\xec\x84\xb1',
'HangulSyllableSsweb': '\xec\x92\xad',
'HangulSyllableSswec': '\xec\x92\xb3',
'ArabicLigatureZahWithMeemInitialForm': '\xef\xb2\xb9',
'HangulSyllableTals': '\xed\x83\x8c',
'HangulSyllableSswed': '\xec\x92\xa3',
'KhmerLetterNgo': '\xe1\x9e\x84',
'HangulSyllableOen': '\xec\x99\xbc',
'CjkUnifiedIdeograph8887': '\xe8\xa2\x87',
'SquareMpa': '\xe3\x8e\xab',
'HangulSyllableSsweg': '\xec\x92\x9d',
'HangulSyllableSsweh': '\xec\x92\xb7',
'CjkUnifiedIdeograph8278': '\xe8\x89\xb8',
'MalayalamVowelSignOo': '\xe0\xb5\x8b',
'HangulSyllableSswej': '\xec\x92\xb2',
'UnicodeCharacter0xec1c': '\xee\xb0\x9c',
'HangulSyllableSswek': '\xec\x92\xb4',
'CjkUnifiedIdeograph8884': '\xe8\xa2\x84',
'LatinCapitalLetterEWithDotBelow': '\xe1\xba\xb8',
'CjkUnifiedIdeograph99d8': '\xe9\xa7\x98',
'GlagoliticSmallLetterMyslite': '\xe2\xb0\xbf',
'OriyaLetterNna': '\xe0\xac\xa3',
'EthiopicSyllablePaa': '\xe1\x8d\x93',
'CjkUnifiedIdeograph9cdf': '\xe9\xb3\x9f',
'HangulSyllableKaeng': '\xec\xba\xa5',
'ArabicLigatureJeemWithHahInitialForm': '\xef\xb2\xa7',
'GreekSmallLetterEtaWithPsiliAndOxiaAndYpogegrammeni': '\xe1\xbe\x94',
'HangulSyllableSsweo': '\xec\x92\x80',
'UnicodeCharacter0xf532': '\xef\x94\xb2',
'CjkUnifiedIdeograph9cdd': '\xe9\xb3\x9d',
'CjkUnifiedIdeograph9cde': '\xe9\xb3\x9e',
'CjkUnifiedIdeograph8885': '\xe8\xa2\x85',
'CjkUnifiedIdeograph8ab0': '\xe8\xaa\xb0',
'CjkUnifiedIdeograph9cdb': '\xe9\xb3\x9b',
'MyanmarShanDigitFive': '\xe1\x82\x95',
'CjkUnifiedIdeograph4c94': '\xe4\xb2\x94',
'CanadianSyllabicsFinalRing': '\xe1\x90\xa4',
'HangulSyllableBbyum': '\xec\x81\x8c',
'HangulSyllableDdyaes': '\xeb\x96\x97',
'HangulSyllableSswet': '\xec\x92\xb5',
'CjkUnifiedIdeograph9cda': '\xe9\xb3\x9a',
'SaurashtraLetterKha': '\xea\xa2\x93',
'ModifierLetterSmallT': '\xe1\xb5\x97',
'ModifierLetterSmallU': '\xe1\xb5\x98',
'ModifierLetterSmallV': '\xe1\xb5\x9b',
'ArmenianSmallLetterTiwn': '\xd5\xbf',
'ModifierLetterSmallP': '\xe1\xb5\x96',
'ModifierLetterSmallR': '\xca\xb3',
'ModifierLetterSmallS': '\xcb\xa2',
'CjkUnifiedIdeograph52dc': '\xe5\x8b\x9c',
'CjkUnifiedIdeograph52db': '\xe5\x8b\x9b',
'CjkUnifiedIdeograph52da': '\xe5\x8b\x9a',
'HangulSyllableAss': '\xec\x95\x98',
'ModifierLetterSmallX': '\xcb\xa3',
'ModifierLetterSmallY': '\xca\xb8',
'CjkUnifiedIdeograph52de': '\xe5\x8b\x9e',
'CjkUnifiedIdeograph52dd': '\xe5\x8b\x9d',
'ModifierLetterSmallD': '\xe1\xb5\x88',
'ModifierLetterSmallE': '\xe1\xb5\x89',
'ModifierLetterSmallF': '\xe1\xb6\xa0',
'ModifierLetterSmallG': '\xe1\xb5\x8d',
'ModifierLetterSmallA': '\xe1\xb5\x83',
'ModifierLetterSmallB': '\xe1\xb5\x87',
'ModifierLetterSmallC': '\xe1\xb6\x9c',
'ModifierLetterSmallL': '\xcb\xa1',
'ModifierLetterSmallM': '\xe1\xb5\x90',
'ArabicLigatureJeemWithAlefMaksuraFinalForm': '\xef\xb4\x9d',
'ModifierLetterSmallO': '\xe1\xb5\x92',
'ModifierLetterSmallH': '\xca\xb0',
'HangulSyllableDyags': '\xeb\x8c\x9f',
'ModifierLetterSmallJ': '\xca\xb2',
'ModifierLetterSmallK': '\xe1\xb5\x8f',
'CjkUnifiedIdeograph8883': '\xe8\xa2\x83',
'ArabicLigatureMeemWithJeemWithHahInitialForm': '\xef\xb6\x8c',
'UnicodeCharacter0xf074': '\xef\x81\xb4',
'CjkUnifiedIdeograph4c92': '\xe4\xb2\x92',
'CjkUnifiedIdeograph5d92': '\xe5\xb6\x92',
'HangulSyllableJjyubs': '\xec\xaf\x8a',
'OriyaSignVisarga': '\xe0\xac\x83',
'UnicodeCharacter0xf50f': '\xef\x94\x8f',
'LatinCapitalLetterSWithAcuteAndDotAbove': '\xe1\xb9\xa4',
'HangulSyllableUgs': '\xec\x9a\xb3',
'UnicodeCharacter0xe1a3': '\xee\x86\xa3',
'CjkUnifiedIdeograph8880': '\xe8\xa2\x80',
'ArabicLigatureSheenWithMeemWithMeemFinalForm': '\xef\xb5\xac',
'HangulSyllableBbyeolg': '\xeb\xbc\x91',
'UnicodeCharacter0xe1a2': '\xee\x86\xa2',
'HangulSyllableBbyeolb': '\xeb\xbc\x93',
'HangulSyllableBbyeolm': '\xeb\xbc\x92',
'UnicodeCharacter0xf50c': '\xef\x94\x8c',
'HangulSyllableJjenj': '\xec\xa9\xad',
'UnicodeCharacter0xde67': '\xed\xb9\xa7',
'HangulSyllableBbyeolh': '\xeb\xbc\x97',
'UnicodeCharacter0xf50b': '\xef\x94\x8b',
'HangulSyllableBbyeolt': '\xeb\xbc\x95',
'CjkUnifiedIdeograph52d9': '\xe5\x8b\x99',
'CjkUnifiedIdeograph52d8': '\xe5\x8b\x98',
'CjkUnifiedIdeograph7086': '\xe7\x82\x86',
'HangulSyllableBbyeolp': '\xeb\xbc\x96',
'ParenthesizedIdeographEarth': '\xe3\x88\xaf',
'TrigramForWind': '\xe2\x98\xb4',
'CjkUnifiedIdeograph52d3': '\xe5\x8b\x93',
'CjkUnifiedIdeograph52d2': '\xe5\x8b\x92',
'CjkUnifiedIdeograph52d1': '\xe5\x8b\x91',
'CjkUnifiedIdeograph52d0': '\xe5\x8b\x90',
'CjkUnifiedIdeograph52d7': '\xe5\x8b\x97',
'CjkUnifiedIdeograph52d6': '\xe5\x8b\x96',
'CjkUnifiedIdeograph52d5': '\xe5\x8b\x95',
'CjkUnifiedIdeograph52d4': '\xe5\x8b\x94',
'HangulSyllableTeulp': '\xed\x8b\x86',
'HangulSyllableTeuls': '\xed\x8b\x84',
'HangulSyllableTeult': '\xed\x8b\x85',
'UnicodeCharacter0xeb66': '\xee\xad\xa6',
'UnicodeCharacter0xde66': '\xed\xb9\xa6',
'VaiSyllableDhe': '\xea\x97\xb5',
'CjkUnifiedIdeograph7c27': '\xe7\xb0\xa7',
'HangulSyllableDduss': '\xeb\x9a\xb0',
'HangulSyllableTeulb': '\xed\x8b\x83',
'QuadrantUpperLeft': '\xe2\x96\x98',
'CjkUnifiedIdeograph72d4': '\xe7\x8b\x94',
'HangulSyllableTeulh': '\xed\x8b\x87',
'HangulSyllableTeulm': '\xed\x8b\x82',
'HangulSyllableDaelg': '\xeb\x8c\x89',
'GeorgianSmallLetterShin': '\xe2\xb4\x98',
'HangulSyllableDaelb': '\xeb\x8c\x8b',
'KangxiRadicalBitter': '\xe2\xbe\x9f',
'HexagramForAfterCompletion': '\xe4\xb7\xbe',
'HangulSyllableDaelm': '\xeb\x8c\x8a',
'BengaliLetterGha': '\xe0\xa6\x98',
'UnicodeCharacter0xe94c': '\xee\xa5\x8c',
'VaiSyllableDho': '\xea\x97\x8d',
'CjkUnifiedIdeograph5830': '\xe5\xa0\xb0',
'CjkUnifiedIdeograph5831': '\xe5\xa0\xb1',
'CjkUnifiedIdeograph5832': '\xe5\xa0\xb2',
'CjkUnifiedIdeograph5833': '\xe5\xa0\xb3',
'CjkUnifiedIdeograph5834': '\xe5\xa0\xb4',
'CjkUnifiedIdeograph5835': '\xe5\xa0\xb5',
'CjkUnifiedIdeograph5836': '\xe5\xa0\xb6',
'CjkUnifiedIdeograph5837': '\xe5\xa0\xb7',
'CjkUnifiedIdeograph5838': '\xe5\xa0\xb8',
'CjkUnifiedIdeograph5839': '\xe5\xa0\xb9',
'CjkUnifiedIdeograph72d2': '\xe7\x8b\x92',
'HangulSyllableSwabs': '\xec\x86\xba',
'CjkUnifiedIdeograph65ca': '\xe6\x97\x8a',
'CjkUnifiedIdeograph65cb': '\xe6\x97\x8b',
'CjkUnifiedIdeograph65cc': '\xe6\x97\x8c',
'CjkUnifiedIdeograph65cd': '\xe6\x97\x8d',
'CjkUnifiedIdeograph65ce': '\xe6\x97\x8e',
'TagbanwaLetterNga': '\xe1\x9d\xa5',
'CjkUnifiedIdeograph869b': '\xe8\x9a\x9b',
'HangulSyllableYu': '\xec\x9c\xa0',
'CjkUnifiedIdeograph869a': '\xe8\x9a\x9a',
'CjkUnifiedIdeograph5c11': '\xe5\xb0\x91',
'CjkUnifiedIdeograph5c10': '\xe5\xb0\x90',
'CjkUnifiedIdeograph5c13': '\xe5\xb0\x93',
'EthiopicSyllableHhwa': '\xe1\x88\x97',
'CjkUnifiedIdeograph5c15': '\xe5\xb0\x95',
'CjkUnifiedIdeograph5c14': '\xe5\xb0\x94',
'CjkUnifiedIdeograph5c17': '\xe5\xb0\x97',
'CjkUnifiedIdeograph5c16': '\xe5\xb0\x96',
'CjkUnifiedIdeograph5c19': '\xe5\xb0\x99',
'KangxiRadicalBranch': '\xe2\xbd\x80',
'UnicodeCharacter0xf413': '\xef\x90\x93',
'UnicodeCharacter0xe36d': '\xee\x8d\xad',
'UnicodeCharacter0xf49c': '\xef\x92\x9c',
'CjkUnifiedIdeograph869f': '\xe8\x9a\x9f',
'CjkUnifiedIdeograph583a': '\xe5\xa0\xba',
'CjkUnifiedIdeograph583b': '\xe5\xa0\xbb',
'HiraganaLetterMe': '\xe3\x82\x81',
'CjkUnifiedIdeograph583d': '\xe5\xa0\xbd',
'LatinSmallLetterIota': '\xc9\xa9',
'CjkUnifiedIdeograph583f': '\xe5\xa0\xbf',
'HiraganaLetterMa': '\xe3\x81\xbe',
'HiraganaLetterMo': '\xe3\x82\x82',
'CjkUnifiedIdeograph869d': '\xe8\x9a\x9d',
'HiraganaLetterMi': '\xe3\x81\xbf',
'HangulSyllableJih': '\xec\xa7\x9b',
'HiraganaLetterMu': '\xe3\x82\x80',
'CjkUnifiedIdeograph5c1a': '\xe5\xb0\x9a',
'CjkUnifiedIdeograph5c1c': '\xe5\xb0\x9c',
'KangxiRadicalStone': '\xe2\xbd\xaf',
'CjkUnifiedIdeograph5c1e': '\xe5\xb0\x9e',
'CjkUnifiedIdeograph5c1d': '\xe5\xb0\x9d',
'CjkUnifiedIdeograph5c1f': '\xe5\xb0\x9f',
'UnicodeCharacter0xf495': '\xef\x92\x95',
'UnicodeCharacter0xf494': '\xef\x92\x94',
'CyrillicCapitalLetterEnWithHook': '\xd3\x87',
'UnicodeCharacter0xf496': '\xef\x92\x96',
'UnicodeCharacter0xf491': '\xef\x92\x91',
'UnicodeCharacter0xf490': '\xef\x92\x90',
'UnicodeCharacter0xf493': '\xef\x92\x93',
'UnicodeCharacter0xf492': '\xef\x92\x92',
'CjkUnifiedIdeograph65c0': '\xe6\x97\x80',
'CjkUnifiedIdeograph65c1': '\xe6\x97\x81',
'CjkUnifiedIdeograph65c2': '\xe6\x97\x82',
'CjkUnifiedIdeograph65c3': '\xe6\x97\x83',
'DevanagariLetterTha': '\xe0\xa4\xa5',
'CjkUnifiedIdeograph65c5': '\xe6\x97\x85',
'CjkUnifiedIdeograph65c6': '\xe6\x97\x86',
'TamilDigitTwo': '\xe0\xaf\xa8',
'CjkUnifiedIdeograph65c8': '\xe6\x97\x88',
'CjkUnifiedIdeograph65c9': '\xe6\x97\x89',
'UnicodeCharacter0xd9b0': '\xed\xa6\xb0',
'CircledLatinCapitalLetterK': '\xe2\x93\x80',
'HangulSyllableSsugs': '\xec\x91\xa7',
'CjkUnifiedIdeograph3cec': '\xe3\xb3\xac',
'CjkUnifiedIdeograph3ceb': '\xe3\xb3\xab',
'CjkUnifiedIdeograph3cea': '\xe3\xb3\xaa',
'CjkUnifiedIdeograph3cef': '\xe3\xb3\xaf',
'CjkUnifiedIdeograph3cee': '\xe3\xb3\xae',
'CjkUnifiedIdeograph3ced': '\xe3\xb3\xad',
'HangulSyllableJij': '\xec\xa7\x96',
'HangulSyllablePobs': '\xed\x8f\xbe',
'CjkUnifiedIdeograph404f': '\xe4\x81\x8f',
'CjkUnifiedIdeograph404d': '\xe4\x81\x8d',
'CjkUnifiedIdeograph404e': '\xe4\x81\x8e',
'CjkUnifiedIdeograph404b': '\xe4\x81\x8b',
'CjkUnifiedIdeograph404c': '\xe4\x81\x8c',
'CjkUnifiedIdeograph404a': '\xe4\x81\x8a',
'CjkUnifiedIdeograph8f8d': '\xe8\xbe\x8d',
'UnicodeCharacter0xeb60': '\xee\xad\xa0',
'TibetanSubjoinedLetterPa': '\xe0\xbe\xa4',
'UnicodeCharacter0xef62': '\xee\xbd\xa2',
'CjkUnifiedIdeograph93fb': '\xe9\x8f\xbb',
'GeorgianSmallLetterCil': '\xe2\xb4\x9c',
'UnicodeCharacter0x2c70': '\xe2\xb1\xb0',
'KatakanaLetterHu': '\xe3\x83\x95',
'HangulSyllableDdweong': '\xeb\x9b\x8d',
'CjkUnifiedIdeograph4046': '\xe4\x81\x86',
'CjkUnifiedIdeograph4047': '\xe4\x81\x87',
'CjkUnifiedIdeograph4044': '\xe4\x81\x84',
'CjkUnifiedIdeograph4045': '\xe4\x81\x85',
'CjkUnifiedIdeograph4042': '\xe4\x81\x82',
'CjkUnifiedIdeograph4043': '\xe4\x81\x83',
'CjkUnifiedIdeograph4040': '\xe4\x81\x80',
'CjkUnifiedIdeograph4041': '\xe4\x81\x81',
'HangulSyllableDdags': '\xeb\x94\xb3',
'CjkUnifiedIdeograph4048': '\xe4\x81\x88',
'CjkUnifiedIdeograph4049': '\xe4\x81\x89',
'CjkUnifiedIdeograph3ce3': '\xe3\xb3\xa3',
'CjkUnifiedIdeograph3ce2': '\xe3\xb3\xa2',
'CjkUnifiedIdeograph3ce1': '\xe3\xb3\xa1',
'CjkUnifiedIdeograph3ce0': '\xe3\xb3\xa0',
'CjkUnifiedIdeograph3ce7': '\xe3\xb3\xa7',
'CjkUnifiedIdeograph3ce6': '\xe3\xb3\xa6',
'CjkUnifiedIdeograph3ce5': '\xe3\xb3\xa5',
'CjkUnifiedIdeograph3ce4': '\xe3\xb3\xa4',
'CjkUnifiedIdeograph3ce9': '\xe3\xb3\xa9',
'CjkUnifiedIdeograph3ce8': '\xe3\xb3\xa8',
'Intercalate': '\xe2\x8a\xba',
'UnicodeCharacter0x2c7e': '\xe2\xb1\xbe',
'UnicodeCharacter0x2c7f': '\xe2\xb1\xbf',
'HangulSyllableGyaelt': '\xea\xb1\xa1',
'HangulSyllableHyeoss': '\xed\x98\x94',
'VaiSyllableZha': '\xea\x95\xa5',
'CjkUnifiedIdeograph9cbb': '\xe9\xb2\xbb',
'Sampi': '\xcf\xa0',
'HangulSyllableGyaelb': '\xea\xb1\x9f',
'HangulSyllableGyaelg': '\xea\xb1\x9d',
'TibetanMarkGterYigMgoUmRnamBcadMa': '\xe0\xbc\x82',
'HangulSyllableRyeoss': '\xeb\xa0\xb8',
'HangulSyllableGyaelm': '\xea\xb1\x9e',
'CanadianSyllabicsCarrierTlee': '\xe1\x98\xbd',
'HangulSyllableDyagg': '\xeb\x8c\x9e',
'UnicodeCharacter0xde84': '\xed\xba\x84',
'UnicodeCharacter0xda6b': '\xed\xa9\xab',
'CjkUnifiedIdeograph948e': '\xe9\x92\x8e',
'CjkUnifiedIdeograph948d': '\xe9\x92\x8d',
'CjkUnifiedIdeograph948f': '\xe9\x92\x8f',
'CjkUnifiedIdeograph948a': '\xe9\x92\x8a',
'CjkUnifiedIdeograph948c': '\xe9\x92\x8c',
'CjkUnifiedIdeograph948b': '\xe9\x92\x8b',
'UnicodeCharacter0x191e': '\xe1\xa4\x9e',
'UnicodeCharacter0x191d': '\xe1\xa4\x9d',
'UnicodeCharacter0x191f': '\xe1\xa4\x9f',
'CjkUnifiedIdeograph579e': '\xe5\x9e\x9e',
'CjkUnifiedIdeograph579d': '\xe5\x9e\x9d',
'CjkUnifiedIdeograph579f': '\xe5\x9e\x9f',
'CjkUnifiedIdeograph579a': '\xe5\x9e\x9a',
'ReplacementCharacter': '\xef\xbf\xbd',
'SyriacLetterSogdianFe': '\xdd\x8f',
'CjkUnifiedIdeograph579b': '\xe5\x9e\x9b',
'LatinCapitalLetterUWithHookAbove': '\xe1\xbb\xa6',
'HangulSyllableDyolm': '\xeb\x90\xbe',
'HangulSyllableDyolh': '\xeb\x91\x83',
'UnicodeCharacter0x7ff': '\xdf\xbf',
'HangulSyllableGgaess': '\xea\xb9\xbc',
'UnicodeCharacter0x7fd': '\xdf\xbd',
'UnicodeCharacter0x7fe': '\xdf\xbe',
'UnicodeCharacter0x7fb': '\xdf\xbb',
'UnicodeCharacter0x7fc': '\xdf\xbc',
'HangulSyllableDyolp': '\xeb\x91\x82',
'CanadianSyllabicsAai': '\xe1\x90\x82',
'CjkUnifiedIdeograph9485': '\xe9\x92\x85',
'CjkUnifiedIdeograph9484': '\xe9\x92\x84',
'CjkUnifiedIdeograph9487': '\xe9\x92\x87',
'CjkUnifiedIdeograph9486': '\xe9\x92\x86',
'CjkUnifiedIdeograph9481': '\xe9\x92\x81',
'CyrillicCapitalLetterZe': '\xd0\x97',
'CjkUnifiedIdeograph9483': '\xe9\x92\x83',
'CjkUnifiedIdeograph9482': '\xe9\x92\x82',
'UnicodeCharacter0xab10': '\xea\xac\x90',
'HangulSyllableYunj': '\xec\x9c\xa5',
'TagalogLetterU': '\xe1\x9c\x82',
'CjkUnifiedIdeograph9488': '\xe9\x92\x88',
'TagalogLetterI': '\xe1\x9c\x81',
'HangulSyllableKoegs': '\xec\xbe\xab',
'TagalogLetterA': '\xe1\x9c\x80',
'UnicodeCharacter0xe7e6': '\xee\x9f\xa6',
'UnicodeCharacter0xeb69': '\xee\xad\xa9',
'CjkUnifiedIdeograph5795': '\xe5\x9e\x95',
'CjkUnifiedIdeograph5794': '\xe5\x9e\x94',
'CjkUnifiedIdeograph5797': '\xe5\x9e\x97',
'CjkUnifiedIdeograph5796': '\xe5\x9e\x96',
'CjkUnifiedIdeograph5791': '\xe5\x9e\x91',
'CjkUnifiedIdeograph5790': '\xe5\x9e\x90',
'CjkUnifiedIdeograph5793': '\xe5\x9e\x93',
'CjkUnifiedIdeograph5792': '\xe5\x9e\x92',
'CjkUnifiedIdeograph5799': '\xe5\x9e\x99',
'CjkUnifiedIdeograph5798': '\xe5\x9e\x98',
'UnicodeCharacter0xda6f': '\xed\xa9\xaf',
'DotOperator': '\xe2\x8b\x85',
'CjkUnifiedIdeograph3e6e': '\xe3\xb9\xae',
'CjkUnifiedIdeograph8d21': '\xe8\xb4\xa1',
'HangulSyllableDdyelm': '\xeb\x97\xbe',
'HebrewLetterNunWithDagesh': '\xef\xad\x80',
'UnicodeCharacter0xde68': '\xed\xb9\xa8',
'UnicodeCharacter0xe7af': '\xee\x9e\xaf',
'UnicodeCharacter0xf507': '\xef\x94\x87',
'CjkCompatibilityIdeographf98b': '\xef\xa6\x8b',
'UnicodeCharacter0xe7ae': '\xee\x9e\xae',
'UnicodeCharacter0xe7ad': '\xee\x9e\xad',
'HexagramForTheClingingFire': '\xe4\xb7\x9d',
'UnicodeCharacter0xf505': '\xef\x94\x85',
'UnicodeCharacter0xda6e': '\xed\xa9\xae',
'CjkUnifiedIdeograph8037': '\xe8\x80\xb7',
'UnicodeCharacter0xf504': '\xef\x94\x84',
'HeavyEightTeardropspokedPropellerAsterisk': '\xe2\x9d\x8b',
'UnicodeCharacter0xf503': '\xef\x94\x83',
'HangulSyllableNak': '\xeb\x82\xb0',
'UnicodeCharacter0xe1ad': '\xee\x86\xad',
'HangulSyllableNaj': '\xeb\x82\xae',
'HangulSyllableNam': '\xeb\x82\xa8',
'UnicodeCharacter0xf500': '\xef\x94\x80',
'UnicodeCharacter0xda6d': '\xed\xa9\xad',
'CjkUnifiedIdeograph8032': '\xe8\x80\xb2',
'GujaratiVowelSignVocalicR': '\xe0\xab\x83',
'CjkUnifiedIdeograph8727': '\xe8\x9c\xa7',
'E': '\xce\x95',
'UnicodeCharacter0xef6b': '\xee\xbd\xab',
'HangulSyllableNan': '\xeb\x82\x9c',
'SquareMikuron': '\xe3\x8d\x88',
'UnicodeCharacter0xe740': '\xee\x9d\x80',
'UnicodeCharacter0xe741': '\xee\x9d\x81',
'UnicodeCharacter0xaa5a': '\xea\xa9\x9a',
'UnicodeCharacter0xaa5b': '\xea\xa9\x9b',
'HangulSyllableNac': '\xeb\x82\xaf',
'GujaratiVowelSignVocalicL': '\xe0\xab\xa2',
'HangulSyllableNab': '\xeb\x82\xa9',
'HangulSyllableNae': '\xeb\x82\xb4',
'UnicodeCharacter0xef6d': '\xee\xbd\xad',
'UnicodeCharacter0xda46': '\xed\xa9\x86',
'HeavyBallotX': '\xe2\x9c\x98',
'HangulSyllableNad': '\xeb\x82\x9f',
'CjkUnifiedIdeograph8039': '\xe8\x80\xb9',
'GurmukhiDigitSix': '\xe0\xa9\xac',
'CjkUnifiedIdeograph8038': '\xe8\x80\xb8',
'CjkUnifiedIdeograph8725': '\xe8\x9c\xa5',
'HangulSyllableHwaess': '\xed\x9a\x84',
'HangulSyllableYe': '\xec\x98\x88',
'CjkUnifiedIdeograph8c9b': '\xe8\xb2\x9b',
'_+': '\xe2\x82\x8a',
'CjkUnifiedIdeograph8c9c': '\xe8\xb2\x9c',
'CjkUnifiedIdeograph482a': '\xe4\xa0\xaa',
'CjkUnifiedIdeograph482b': '\xe4\xa0\xab',
'CjkUnifiedIdeograph482c': '\xe4\xa0\xac',
'CjkUnifiedIdeograph482d': '\xe4\xa0\xad',
'CjkUnifiedIdeograph482e': '\xe4\xa0\xae',
'CjkUnifiedIdeograph482f': '\xe4\xa0\xaf',
'UnicodeCharacter0xdb3f': '\xed\xac\xbf',
'UnicodeCharacter0xdb3e': '\xed\xac\xbe',
'UnicodeCharacter0xdb3d': '\xed\xac\xbd',
'UnicodeCharacter0xdb3c': '\xed\xac\xbc',
'UnicodeCharacter0xdb3b': '\xed\xac\xbb',
'UnicodeCharacter0xdb3a': '\xed\xac\xba',
'UnicodeCharacter0xed39': '\xee\xb4\xb9',
'UnicodeCharacter0xed36': '\xee\xb4\xb6',
'UnicodeCharacter0xed37': '\xee\xb4\xb7',
'UnicodeCharacter0xed34': '\xee\xb4\xb4',
'UnicodeCharacter0xed35': '\xee\xb4\xb5',
'UnicodeCharacter0xed32': '\xee\xb4\xb2',
'UnicodeCharacter0xed33': '\xee\xb4\xb3',
'UnicodeCharacter0xed30': '\xee\xb4\xb0',
'UnicodeCharacter0xed31': '\xee\xb4\xb1',
'UnicodeCharacter0xde86': '\xed\xba\x86',
'AplFunctionalSymbolAlphaUnderbar': '\xe2\x8d\xb6',
'HangulSyllableGgwalp': '\xea\xbd\x96',
'GurmukhiDigitNine': '\xe0\xa9\xaf',
'CjkUnifiedIdeograph4828': '\xe4\xa0\xa8',
'CjkUnifiedIdeograph4829': '\xe4\xa0\xa9',
'HangulSyllableDam': '\xeb\x8b\xb4',
'HangulSyllableDal': '\xeb\x8b\xac',
'HangulSyllableDak': '\xeb\x8b\xbc',
'HangulSyllableDaj': '\xeb\x8b\xba',
'HangulSyllableDah': '\xeb\x8b\xbf',
'CjkUnifiedIdeograph4820': '\xe4\xa0\xa0',
'CjkUnifiedIdeograph4821': '\xe4\xa0\xa1',
'CjkUnifiedIdeograph4822': '\xe4\xa0\xa2',
'CjkUnifiedIdeograph4823': '\xe4\xa0\xa3',
'CjkUnifiedIdeograph4824': '\xe4\xa0\xa4',
'CjkUnifiedIdeograph4825': '\xe4\xa0\xa5',
'CjkUnifiedIdeograph4826': '\xe4\xa0\xa6',
'CjkUnifiedIdeograph4827': '\xe4\xa0\xa7',
'ThaanaAabaafili': '\xde\xa7',
'CjkUnifiedIdeograph8722': '\xe8\x9c\xa2',
'SaurashtraLetterBa': '\xea\xa2\xa8',
'HangulSyllableDat': '\xeb\x8b\xbd',
'HangulSyllableDas': '\xeb\x8b\xb7',
'HangulSyllableDap': '\xeb\x8b\xbe',
'UnicodeCharacter0xdb37': '\xed\xac\xb7',
'UnicodeCharacter0xdb36': '\xed\xac\xb6',
'UnicodeCharacter0xdb35': '\xed\xac\xb5',
'UnicodeCharacter0xdb34': '\xed\xac\xb4',
'UnicodeCharacter0xdb33': '\xed\xac\xb3',
'UnicodeCharacter0xdb32': '\xed\xac\xb2',
'UnicodeCharacter0xdb31': '\xed\xac\xb1',
'UnicodeCharacter0xdb30': '\xed\xac\xb0',
'UnicodeCharacter0xed3f': '\xee\xb4\xbf',
'MetricalPentaseme': '\xe2\x8f\x99',
'UnicodeCharacter0xed3e': '\xee\xb4\xbe',
'HebrewPointHolam': '\xd6\xb9',
'UnicodeCharacter0xed3c': '\xee\xb4\xbc',
'UnicodeCharacter0xdb39': '\xed\xac\xb9',
'UnicodeCharacter0xdb38': '\xed\xac\xb8',
'UnicodeCharacter0xf8d6': '\xef\xa3\x96',
'UnicodeCharacter0xf8d7': '\xef\xa3\x97',
'UnicodeCharacter0xf8d4': '\xef\xa3\x94',
'UnicodeCharacter0xf8d5': '\xef\xa3\x95',
'CjkUnifiedIdeograph7559': '\xe7\x95\x99',
'CjkUnifiedIdeograph7558': '\xe7\x95\x98',
'UnicodeCharacter0xf8d0': '\xef\xa3\x90',
'UnicodeCharacter0xf8d1': '\xef\xa3\x91',
'CjkUnifiedIdeograph7555': '\xe7\x95\x95',
'CjkUnifiedIdeograph7554': '\xe7\x95\x94',
'CjkUnifiedIdeograph7557': '\xe7\x95\x97',
'CjkUnifiedIdeograph7556': '\xe7\x95\x96',
'CjkUnifiedIdeograph7551': '\xe7\x95\x91',
'CjkUnifiedIdeograph7550': '\xe7\x95\x90',
'CjkUnifiedIdeograph7553': '\xe7\x95\x93',
'CyrillicSmallLetterLha': '\xd4\x95',
'DownwardsQuadrupleArrow': '\xe2\x9f\xb1',
'EthiopicSyllableKxwaa': '\xe1\x8b\x83',
'CjkUnifiedIdeograph98dc': '\xe9\xa3\x9c',
'SquareRittoru': '\xe3\x8d\x91',
'UnicodeCharacter0xf25c': '\xef\x89\x9c',
'HangulSyllableWe': '\xec\x9b\xa8',
'HangulSyllableGat': '\xea\xb0\x99',
'DevanagariSignNukta': '\xe0\xa4\xbc',
'HangulSyllableGap': '\xea\xb0\x9a',
'HangulSyllableGas': '\xea\xb0\x93',
'HangulSyllableGal': '\xea\xb0\x88',
'HangulSyllableGam': '\xea\xb0\x90',
'HangulSyllableGan': '\xea\xb0\x84',
'ParenthesizedDigitFour': '\xe2\x91\xb7',
'HangulSyllableGah': '\xea\xb0\x9b',
'HangulSyllableGaj': '\xea\xb0\x96',
'HangulSyllableGak': '\xea\xb0\x98',
'CjkUnifiedIdeograph4eaa': '\xe4\xba\xaa',
'HangulSyllableGae': '\xea\xb0\x9c',
'HangulSyllableGag': '\xea\xb0\x81',
'CjkUnifiedIdeograph8664': '\xe8\x99\xa4',
'HangulSyllableGab': '\xea\xb0\x91',
'HangulSyllableGac': '\xea\xb0\x97',
'HangulSyllableDwass': '\xeb\x8f\xb4',
'UnicodeCharacter0xf0ed': '\xef\x83\xad',
'SquareSv': '\xe3\x8f\x9c',
'SquareSr': '\xe3\x8f\x9b',
'UnicodeCharacter0xf8df': '\xef\xa3\x9f',
'UnicodeCharacter0xf8dd': '\xef\xa3\x9d',
'CjkUnifiedIdeograph803d': '\xe8\x80\xbd',
'UnicodeCharacter0xf8db': '\xef\xa3\x9b',
'UnicodeCharacter0xf8dc': '\xef\xa3\x9c',
'ArabicLetterNoonWithThreeDotsAbove': '\xda\xbd',
'CjkUnifiedIdeograph755e': '\xe7\x95\x9e',
'CjkUnifiedIdeograph755d': '\xe7\x95\x9d',
'CjkUnifiedIdeograph755f': '\xe7\x95\x9f',
'CjkUnifiedIdeograph755a': '\xe7\x95\x9a',
'CombiningLongDoubleSolidusOverlay': '\xe2\x83\xab',
'CjkUnifiedIdeograph755c': '\xe7\x95\x9c',
'CjkUnifiedIdeograph755b': '\xe7\x95\x9b',
'UnicodeCharacter0xf25a': '\xef\x89\x9a',
'CjkUnifiedIdeograph7fda': '\xe7\xbf\x9a',
'CyrillicSmallLetterZe': '\xd0\xb7',
'NeitherGreaterthanNorEqualTo': '\xe2\x89\xb1',
'UnicodeCharacter0xf1fd': '\xef\x87\xbd',
'UnicodeCharacter0xe74b': '\xee\x9d\x8b',
'CyrillicCapitalLetterNeutralYer': '\xea\x99\x8e',
'HangulSyllableDol': '\xeb\x8f\x8c',
'CjkUnifiedIdeograph7fdb': '\xe7\xbf\x9b',
'ArabicLetterKaf': '\xd9\x83',
'HangulSyllableDaess': '\xeb\x8c\x94',
'CjkUnifiedIdeograph5d1e': '\xe5\xb4\x9e',
'CjkUnifiedIdeograph7fde': '\xe7\xbf\x9e',
'VaiSyllableNdoleSoo': '\xea\x98\x92',
'UnicodeCharacter0xe74e': '\xee\x9d\x8e',
'HangulSyllableDwaem': '\xeb\x90\x8c',
'HangulSyllableJwang': '\xec\xa2\xa1',
'HangulSyllableDyum': '\xeb\x93\x90',
'HangulSyllableJwanh': '\xec\xa2\x92',
'UnicodeCharacter0xf1fb': '\xef\x87\xbb',
'KhmerDigitZero': '\xe1\x9f\xa0',
'HangulSyllableUnh': '\xec\x9a\xb6',
'CjkUnifiedIdeograph7da6': '\xe7\xb6\xa6',
'CjkUnifiedIdeograph5d1b': '\xe5\xb4\x9b',
'UnicodeCharacter0x52a': '\xd4\xaa',
'UnicodeCharacter0x52b': '\xd4\xab',
'UnicodeCharacter0x52c': '\xd4\xac',
'UnicodeCharacter0x52d': '\xd4\xad',
'UnicodeCharacter0x52e': '\xd4\xae',
'CircledNumberThirtySeven': '\xe3\x8a\xb2',
'UnicodeCharacter0xef7b': '\xee\xbd\xbb',
'HangulSyllableDwael': '\xeb\x90\x84',
'HangulSyllableAebs': '\xec\x95\xb2',
'UnicodeCharacter0xe7a8': '\xee\x9e\xa8',
'UnicodeCharacter0xf25f': '\xef\x89\x9f',
'HangulSyllableYom': '\xec\x9a\xa4',
'HangulSyllableSsobs': '\xec\x8f\xaa',
'CjkUnifiedIdeograph7f12': '\xe7\xbc\x92',
'UnicodeCharacter0xe7a2': '\xee\x9e\xa2',
'UnicodeCharacter0xe7a1': '\xee\x9e\xa1',
'subphi': '\xe1\xb5\xa9',
'LeftRightArrowWithDoubleVerticalStroke': '\xe2\x87\xbc',
'UnicodeCharacter0xe7a0': '\xee\x9e\xa0',
'WhiteLeftpointingTriangle': '\xe2\x97\x81',
'UnicodeCharacter0xe7a7': '\xee\x9e\xa7',
'AplFunctionalSymbolIbeam': '\xe2\x8c\xb6',
'CjkUnifiedIdeograph8d2b': '\xe8\xb4\xab',
'UnicodeCharacter0xe7a6': '\xee\x9e\xa6',
'UnicodeCharacter0x528': '\xd4\xa8',
'UnicodeCharacter0x529': '\xd4\xa9',
'UnicodeCharacter0xe7a5': '\xee\x9e\xa5',
'UnicodeCharacter0xf25d': '\xef\x89\x9d',
'UnicodeCharacter0xdca0': '\xed\xb2\xa0',
'UnicodeCharacter0xdca1': '\xed\xb2\xa1',
'UnicodeCharacter0xdca2': '\xed\xb2\xa2',
'UnicodeCharacter0xdca3': '\xed\xb2\xa3',
'UnicodeCharacter0x524': '\xd4\xa4',
'UnicodeCharacter0x525': '\xd4\xa5',
'UnicodeCharacter0x526': '\xd4\xa6',
'UnicodeCharacter0x527': '\xd4\xa7',
'GreekSmallLetterRhoWithDasia': '\xe1\xbf\xa5',
'EthiopicSyllableXoa': '\xe1\x8a\x87',
'ArabicLetterMeem': '\xd9\x85',
'KangxiRadicalOne': '\xe2\xbc\x80',
'UnicodeCharacter0x38d': '\xce\x8d',
'!=': '\xe2\x89\xa0',
'UnicodeCharacter0x38b': '\xce\x8b',
'CanadianSyllabicsCarrierHi': '\xe1\x90\xb7',
'HangulSyllableHogg': '\xed\x98\xba',
'HangulSyllablePeonh': '\xed\x8e\x82',
'HangulSyllableDwaeh': '\xeb\x90\x97',
'HangulJungseongIeu': '\xe1\x86\x9c',
'YiSyllableYyp': '\xea\x92\x8a',
'UnicodeCharacter0xebb8': '\xee\xae\xb8',
'IdeographicTelegraphSymbolForHourTwentyfour': '\xe3\x8d\xb0',
'UnicodeCharacter0xf61e': '\xef\x98\x9e',
'LatinSmallLetterSquatReversedEsh': '\xca\x85',
'LatinCapitalLetterZWithCircumflex': '\xe1\xba\x90',
'YiSyllableNzax': '\xea\x8b\xb5',
'YiSyllableNzap': '\xea\x8b\xb7',
'UnicodeCharacter0x380': '\xce\x80',
'UnicodeCharacter0x381': '\xce\x81',
'UnicodeCharacter0x382': '\xce\x82',
'UnicodeCharacter0x383': '\xce\x83',
'UnicodeCharacter0xebb3': '\xee\xae\xb3',
'HangulSyllableJyaebs': '\xec\x9f\xb6',
'CjkUnifiedIdeograph9387': '\xe9\x8e\x87',
'UnicodeCharacter0xebb1': '\xee\xae\xb1',
'HangulSyllableReolh': '\xeb\x9f\xbb',
'UnicodeCharacter0xebb0': '\xee\xae\xb0',
'HangulSyllableReolm': '\xeb\x9f\xb6',
'ArabicLigatureSeenWithAlefMaksuraFinalForm': '\xef\xb4\x97',
'HangulSyllableReolb': '\xeb\x9f\xb7',
'HangulSyllableReolg': '\xeb\x9f\xb5',
'UnicodeCharacter0xebb5': '\xee\xae\xb5',
'HangulSyllableKiss': '\xed\x82\xb8',
'HangulSyllableReolp': '\xeb\x9f\xba',
'CjkUnifiedIdeograph9632': '\xe9\x98\xb2',
'HangulSyllableReols': '\xeb\x9f\xb8',
'HangulSyllableReolt': '\xeb\x9f\xb9',
'DevanagariLetterNnna': '\xe0\xa4\xa9',
'UnicodeCharacter0xf390': '\xef\x8e\x90',
'CjkUnifiedIdeograph9633': '\xe9\x98\xb3',
'HanunooLetterDa': '\xe1\x9c\xa7',
'CjkUnifiedIdeograph9630': '\xe9\x98\xb0',
'HangulSyllableBbubs': '\xeb\xbf\x9e',
'HangulSyllableHyagg': '\xed\x96\x92',
'YiSyllableBbax': '\xea\x81\x9f',
'HangulSyllableHyags': '\xed\x96\x93',
'YiSyllableBbat': '\xea\x81\x9e',
'CjkUnifiedIdeograph9f91': '\xe9\xbe\x91',
'CjkUnifiedIdeograph640d': '\xe6\x90\x8d',
'CjkUnifiedIdeograph640e': '\xe6\x90\x8e',
'CjkUnifiedIdeograph640f': '\xe6\x90\x8f',
'CjkUnifiedIdeograph640a': '\xe6\x90\x8a',
'CjkUnifiedIdeograph640b': '\xe6\x90\x8b',
'LatinSmallLetterYWithRingAbove': '\xe1\xba\x99',
'ArabicLetterDalIsolatedForm': '\xef\xba\xa9',
'CombiningCyrillicHundredMillionsSign': '\xea\x99\xb1',
'CjkUnifiedIdeograph9638': '\xe9\x98\xb8',
'CjkUnifiedIdeograph9639': '\xe9\x98\xb9',
'HangulSyllableDwaeg': '\xeb\x8f\xbd',
'OlChikiLetterAak': '\xe1\xb1\xa0',
'OlChikiLetterAaj': '\xe1\xb1\xa1',
'OlChikiLetterAam': '\xe1\xb1\xa2',
'CjkCompatibilityIdeographf9c2': '\xef\xa7\x82',
'HangulSyllableTeob': '\xed\x85\x81',
'UnicodeCharacter0xf8fc': '\xef\xa3\xbc',
'CyrillicCapitalLetterEf': '\xd0\xa4',
'CyrillicCapitalLetterEm': '\xd0\x9c',
'CyrillicCapitalLetterEl': '\xd0\x9b',
'OlChikiLetterAaw': '\xe1\xb1\xa3',
'CyrillicCapitalLetterEn': '\xd0\x9d',
'CjkUnifiedIdeograph6404': '\xe6\x90\x84',
'CjkUnifiedIdeograph6405': '\xe6\x90\x85',
'CjkUnifiedIdeograph6406': '\xe6\x90\x86',
'CjkUnifiedIdeograph6407': '\xe6\x90\x87',
'CjkUnifiedIdeograph6400': '\xe6\x90\x80',
'CjkUnifiedIdeograph6401': '\xe6\x90\x81',
'LatinSmallLetterBWithDotAbove': '\xe1\xb8\x83',
'CjkUnifiedIdeograph6403': '\xe6\x90\x83',
'CjkUnifiedIdeograph791b': '\xe7\xa4\x9b',
'CjkUnifiedIdeograph6408': '\xe6\x90\x88',
'CjkUnifiedIdeograph6409': '\xe6\x90\x89',
'UnicodeCharacter0xe5c0': '\xee\x97\x80',
'CherokeeLetterQuv': '\xe1\x8f\x8b',
'CherokeeLetterQuu': '\xe1\x8f\x8a',
'HangulSyllableEug': '\xec\x9c\xbd',
'HangulSyllableJjwigg': '\xec\xae\x9e',
'SinhalaLetterAlpapraanaBayanna': '\xe0\xb6\xb6',
'~!==': '\xe2\x89\x86',
'CherokeeLetterQua': '\xe1\x8f\x86',
'CjkUnifiedIdeograph8539': '\xe8\x94\xb9',
'UnicodeCharacter0xb45': '\xe0\xad\x85',
'CherokeeLetterQue': '\xe1\x8f\x87',
'UnicodeCharacter0xb46': '\xe0\xad\x86',
'UnicodeCharacter0xb49': '\xe0\xad\x89',
'CjkUnifiedIdeograph8533': '\xe8\x94\xb3',
'CherokeeLetterQui': '\xe1\x8f\x88',
'MongolianLetterAliGaliDda': '\xe1\xa2\x8e',
'CherokeeLetterQuo': '\xe1\x8f\x89',
'CjkUnifiedIdeograph8537': '\xe8\x94\xb7',
'CjkUnifiedIdeograph8534': '\xe8\x94\xb4',
'CjkUnifiedIdeograph8535': '\xe8\x94\xb5',
'UnicodeCharacter0xd8f3': '\xed\xa3\xb3',
'HangulSyllableMwaegg': '\xeb\xab\xa2',
'PresentationFormForVerticalColon': '\xef\xb8\x93',
'SimilarAboveGreaterthanAboveEqualsSign': '\xe2\xaa\xa0',
'LepchaDigitSeven': '\xe1\xb1\x87',
'UnicodeCharacter0xf809': '\xef\xa0\x89',
'HangulSyllableDwaec': '\xeb\x90\x93',
'SimilarMinusSimilar': '\xe2\xa9\xac',
'UnicodeCharacter0xb4a': '\xe0\xad\x8a',
'KangxiRadicalEar': '\xe2\xbd\xbf',
'KangxiRadicalEat': '\xe2\xbe\xb7',
'HexagramForReturn': '\xe4\xb7\x97',
'UnicodeCharacter0xb4f': '\xe0\xad\x8f',
'CjkUnifiedIdeograph853b': '\xe8\x94\xbb',
'CjkUnifiedIdeograph853c': '\xe8\x94\xbc',
'CjkUnifiedIdeograph853a': '\xe8\x94\xba',
'CjkUnifiedIdeograph853f': '\xe8\x94\xbf',
'CjkUnifiedIdeograph853d': '\xe8\x94\xbd',
'CjkUnifiedIdeograph853e': '\xe8\x94\xbe',
'UnicodeCharacter0xf802': '\xef\xa0\x82',
'HangulSyllableDwaeb': '\xeb\x90\x8d',
'CopticOldNubianIndirectQuestionMark': '\xe2\xb3\xbb',
'UnicodeCharacter0xf803': '\xef\xa0\x83',
'CjkUnifiedIdeograph67a8': '\xe6\x9e\xa8',
'ModifierLetterLowTilde': '\xcb\xb7',
'TibetanLetterBa': '\xe0\xbd\x96',
'UnicodeCharacter0xe7e0': '\xee\x9f\xa0',
'UnicodeCharacter0xf807': '\xef\xa0\x87',
'UnicodeCharacter0xebbc': '\xee\xae\xbc',
'UnicodeCharacter0xebbb': '\xee\xae\xbb',
'UnicodeCharacter0xebba': '\xee\xae\xba',
'YiSyllableBiep': '\xea\x80\x9d',
'CjkUnifiedIdeograph67a7': '\xe6\x9e\xa7',
'UnicodeCharacter0xef92': '\xee\xbe\x92',
'HangulSyllableSweogg': '\xec\x88\xb6',
'CjkUnifiedIdeograph963d': '\xe9\x98\xbd',
'UnicodeCharacter0xebbf': '\xee\xae\xbf',
'YiSyllableDi': '\xea\x84\x82',
'UnicodeCharacter0xf14e': '\xef\x85\x8e',
'UnicodeCharacter0xebbe': '\xee\xae\xbe',
'YiSyllableBiet': '\xea\x80\x9a',
'UnicodeCharacter0xebbd': '\xee\xae\xbd',
'YiSyllableDo': '\xea\x84\x8f',
'HangulSyllableBbeulg': '\xec\x81\xa1',
'YiSyllableSiex': '\xea\x8c\x8d',
'HangulSyllableBbeulb': '\xec\x81\xa3',
'HangulSyllableBbeulm': '\xec\x81\xa2',
'YiSyllableSiep': '\xea\x8c\x8f',
'HangulSyllableBbeulh': '\xec\x81\xa7',
'YiSyllableBiex': '\xea\x80\x9b',
'HangulSyllableBbeulp': '\xec\x81\xa6',
'HangulSyllableBbeuls': '\xec\x81\xa4',
'CjkUnifiedIdeograph5fee': '\xe5\xbf\xae',
'UnicodeCharacter0xd8e5': '\xed\xa3\xa5',
'GujaratiDigitTwo': '\xe0\xab\xa8',
'BlackUppointingTriangle': '\xe2\x96\xb2',
'DevanagariLetterTtha': '\xe0\xa4\xa0',
'LatinCapitalLetterYogh': '\xc8\x9c',
'UnicodeCharacter0xdbb1': '\xed\xae\xb1',
'LatinSmallLetterBWithMiddleTilde': '\xe1\xb5\xac',
'HangulSyllableGgwing': '\xeb\x80\xa1',
'ThaiCharacterLoLing': '\xe0\xb8\xa5',
'FullwidthQuestionMark': '\xef\xbc\x9f',
'CjkUnifiedIdeograph5f28': '\xe5\xbc\xa8',
'KhmerIndependentVowelQaa': '\xe1\x9e\xa4',
'GujaratiVowelSignIi': '\xe0\xab\x80',
'KhmerIndependentVowelQai': '\xe1\x9e\xb0',
'KhmerIndependentVowelQau': '\xe1\x9e\xb3',
'KhmerIndependentVowelQaq': '\xe1\x9e\xa3',
'UnicodeCharacter0xee9': '\xe0\xbb\xa9',
'OriyaLetterTa': '\xe0\xac\xa4',
'HangulLetterTikeut': '\xe3\x84\xb7',
'IdeographicDescriptionCharacterOverlaid': '\xe2\xbf\xbb',
'CjkUnifiedIdeograph89ee': '\xe8\xa7\xae',
'CjkUnifiedIdeograph6c3b': '\xe6\xb0\xbb',
'HangulChoseongPieupsioscieuc': '\xe1\x84\xa6',
'HangulSyllableJjyin': '\xec\xaf\xb4',
'CjkUnifiedIdeograph6c3a': '\xe6\xb0\xba',
'CjkUnifiedIdeograph6c3f': '\xe6\xb0\xbf',
'LargeUpTack': '\xe2\x9f\x98',
'CjkUnifiedIdeograph6c3d': '\xe6\xb0\xbd',
'CjkUnifiedIdeograph6c3e': '\xe6\xb0\xbe',
'CombiningHorn': '\xcc\x9b',
'CjkUnifiedIdeograph7b2f': '\xe7\xac\xaf',
'HangulChoseongKapyeounssangpieup': '\xe1\x84\xac',
'CjkUnifiedIdeograph7b2d': '\xe7\xac\xad',
'CjkUnifiedIdeograph7b2c': '\xe7\xac\xac',
'CjkUnifiedIdeograph7b2b': '\xe7\xac\xab',
'CjkUnifiedIdeograph7b2a': '\xe7\xac\xaa',
'KannadaLetterSa': '\xe0\xb2\xb8',
'ArabicLigatureDadWithMeemInitialForm': '\xef\xb2\xb7',
'UnicodeCharacter0xee5': '\xe0\xbb\xa5',
'UnicodeCharacter0xe939': '\xee\xa4\xb9',
'UnicodeCharacter0xe938': '\xee\xa4\xb8',
'UnicodeCharacter0xf0ec': '\xef\x83\xac',
'KangxiRadicalWalkEnclosure': '\xe2\xbe\x8f',
'CjkUnifiedIdeograph3a07': '\xe3\xa8\x87',
'CjkUnifiedIdeograph84ae': '\xe8\x92\xae',
'CjkUnifiedIdeograph84ad': '\xe8\x92\xad',
'UnicodeCharacter0xa8cd': '\xea\xa3\x8d',
'CjkUnifiedIdeograph84af': '\xe8\x92\xaf',
'CjkUnifiedIdeograph84aa': '\xe8\x92\xaa',
'DigitNineFullStop': '\xe2\x92\x90',
'CjkUnifiedIdeograph84ac': '\xe8\x92\xac',
'CjkUnifiedIdeograph84ab': '\xe8\x92\xab',
'CjkUnifiedIdeograph3a05': '\xe3\xa8\x85',
'SquareNm': '\xe3\x8e\x9a',
'UnicodeCharacter0xee1': '\xe0\xbb\xa1',
'SquareNa': '\xe3\x8e\x81',
'SquareNf': '\xe3\x8e\x8b',
'CjkUnifiedIdeograph3a03': '\xe3\xa8\x83',
'HangulSyllableBaelm': '\xeb\xb0\xba',
'CjkUnifiedIdeograph3a00': '\xe3\xa8\x80',
'SquareNs': '\xe3\x8e\xb1',
'CjkUnifiedIdeograph3a01': '\xe3\xa8\x81',
'SquareNw': '\xe3\x8e\xbb',
'SquareNv': '\xe3\x8e\xb5',
'UnicodeCharacter0xdbb5': '\xed\xae\xb5',
'CjkUnifiedIdeograph84a9': '\xe8\x92\xa9',
'CjkUnifiedIdeograph84a8': '\xe8\x92\xa8',
'UnicodeCharacter0xa8c8': '\xea\xa3\x88',
'UnicodeCharacter0xa8c9': '\xea\xa3\x89',
'CjkUnifiedIdeograph84a5': '\xe8\x92\xa5',
'CjkUnifiedIdeograph84a4': '\xe8\x92\xa4',
'CjkUnifiedIdeograph84a7': '\xe8\x92\xa7',
'CjkUnifiedIdeograph84a6': '\xe8\x92\xa6',
'CjkUnifiedIdeograph84a1': '\xe8\x92\xa1',
'CjkUnifiedIdeograph84a0': '\xe8\x92\xa0',
'CjkUnifiedIdeograph84a3': '\xe8\x92\xa3',
'KhmerLetterRo': '\xe1\x9e\x9a',
'KatakanaLetterSmallKa': '\xe3\x83\xb5',
'CjkUnifiedIdeograph6c33': '\xe6\xb0\xb3',
'CjkUnifiedIdeograph6c30': '\xe6\xb0\xb0',
'CjkUnifiedIdeograph6c31': '\xe6\xb0\xb1',
'CjkUnifiedIdeograph6c36': '\xe6\xb0\xb6',
'CjkUnifiedIdeograph6c37': '\xe6\xb0\xb7',
'CjkUnifiedIdeograph6c34': '\xe6\xb0\xb4',
'CjkUnifiedIdeograph6c35': '\xe6\xb0\xb5',
'CjkUnifiedIdeograph7b27': '\xe7\xac\xa7',
'CjkUnifiedIdeograph7b26': '\xe7\xac\xa6',
'CjkUnifiedIdeograph6c38': '\xe6\xb0\xb8',
'CjkUnifiedIdeograph6c39': '\xe6\xb0\xb9',
'CjkUnifiedIdeograph7b23': '\xe7\xac\xa3',
'CjkStrokeHxwg': '\xe3\x87\xa0',
'CjkUnifiedIdeograph7b21': '\xe7\xac\xa1',
'CjkUnifiedIdeograph7b20': '\xe7\xac\xa0',
'CjkUnifiedIdeograph44aa': '\xe4\x92\xaa',
'CjkUnifiedIdeograph4c4d': '\xe4\xb1\x8d',
'CjkUnifiedIdeograph44ac': '\xe4\x92\xac',
'CjkUnifiedIdeograph44ab': '\xe4\x92\xab',
'CjkUnifiedIdeograph4c4a': '\xe4\xb1\x8a',
'CjkUnifiedIdeograph44ad': '\xe4\x92\xad',
'CjkUnifiedIdeograph4c4c': '\xe4\xb1\x8c',
'CjkUnifiedIdeograph4c4b': '\xe4\xb1\x8b',
'UnicodeCharacter0xf80f': '\xef\xa0\x8f',
'HangulJungseongYa': '\xe1\x85\xa3',
'MyanmarLetterMa': '\xe1\x80\x99',
'UnicodeCharacter0xe826': '\xee\xa0\xa6',
'YiSyllableNbot': '\xea\x82\x81',
'NorthWestBlackArrow': '\xe2\xac\x89',
'HangulSyllableNwaegs': '\xeb\x86\xb3',
'CjkUnifiedIdeograph87af': '\xe8\x9e\xaf',
'SmallAsterisk': '\xef\xb9\xa1',
'CjkUnifiedIdeograph87ad': '\xe8\x9e\xad',
'CjkUnifiedIdeograph87ae': '\xe8\x9e\xae',
'CjkUnifiedIdeograph87ab': '\xe8\x9e\xab',
'CjkUnifiedIdeograph87ac': '\xe8\x9e\xac',
'CjkUnifiedIdeograph87aa': '\xe8\x9e\xaa',
'HangulSyllableNwaegg': '\xeb\x86\xb2',
'CyrillicSmallLetterEsWithDescender': '\xd2\xab',
'UnicodeCharacter0xf274': '\xef\x89\xb4',
'CjkUnifiedIdeograph9296': '\xe9\x8a\x96',
'UnicodeCharacter0xe8cc': '\xee\xa3\x8c',
'CjkUnifiedIdeograph4c45': '\xe4\xb1\x85',
'CjkUnifiedIdeograph4c44': '\xe4\xb1\x84',
'CjkUnifiedIdeograph4c47': '\xe4\xb1\x87',
'CjkUnifiedIdeograph4c46': '\xe4\xb1\x86',
'CjkUnifiedIdeograph4c41': '\xe4\xb1\x81',
'CjkUnifiedIdeograph4c40': '\xe4\xb1\x80',
'CjkUnifiedIdeograph4c43': '\xe4\xb1\x83',
'CjkUnifiedIdeograph4c42': '\xe4\xb1\x82',
'CjkUnifiedIdeograph44a9': '\xe4\x92\xa9',
'CjkUnifiedIdeograph44a8': '\xe4\x92\xa8',
'GeorgianSmallLetterMan': '\xe2\xb4\x8b',
'CjkUnifiedIdeograph4c49': '\xe4\xb1\x89',
'CjkUnifiedIdeograph4c48': '\xe4\xb1\x88',
'HangulSyllableGgyeoc': '\xea\xbc\x8b',
'CanadianSyllabicsYoo': '\xe1\x94\xab',
'CjkUnifiedIdeograph573e': '\xe5\x9c\xbe',
'UnicodeCharacter0xfefe': '\xef\xbb\xbe',
'UnicodeCharacter0xfefd': '\xef\xbb\xbd',
'CjkStrokePz': '\xe3\x87\x9c',
'CjkStrokePd': '\xe3\x87\x9b',
'CjkStrokePg': '\xe3\x87\xa2',
'CjkUnifiedIdeograph87a8': '\xe8\x9e\xa8',
'CjkUnifiedIdeograph87a9': '\xe8\x9e\xa9',
'CjkUnifiedIdeograph87a6': '\xe8\x9e\xa6',
'CjkUnifiedIdeograph87a7': '\xe8\x9e\xa7',
'CjkUnifiedIdeograph87a4': '\xe8\x9e\xa4',
'CjkUnifiedIdeograph87a5': '\xe8\x9e\xa5',
'CjkUnifiedIdeograph87a2': '\xe8\x9e\xa2',
'CjkUnifiedIdeograph87a3': '\xe8\x9e\xa3',
'CjkUnifiedIdeograph87a0': '\xe8\x9e\xa0',
'CjkUnifiedIdeograph87a1': '\xe8\x9e\xa1',
'HangulSyllableGgyeom': '\xea\xbc\x84',
'Scorpius': '\xe2\x99\x8f',
'CjkUnifiedIdeograph94d8': '\xe9\x93\x98',
'YiSyllableBbup': '\xea\x81\xaf',
'ArabicLigatureQafWithMeemWithYehFinalForm': '\xef\xb6\xb2',
'ArabicLigatureZahWithMeemMedialForm': '\xef\xb4\xbb',
'KangxiRadicalFoot': '\xe2\xbe\x9c',
'HangulSyllableKyeolb': '\xec\xbc\xa7',
'HangulSyllableCwags': '\xec\xb4\xa7',
'HangulLetterCieuc': '\xe3\x85\x88',
'CjkRadicalJsimplifiedEven': '\xe2\xbb\xab',
'MyanmarSignShanCouncilEmphaticTone': '\xe1\x82\x8d',
'KangxiRadicalChild': '\xe2\xbc\xa6',
'UnicodeCharacter0xecce': '\xee\xb3\x8e',
'UnicodeCharacter0xeccd': '\xee\xb3\x8d',
'UnicodeCharacter0xeccf': '\xee\xb3\x8f',
'UnicodeCharacter0xecca': '\xee\xb3\x8a',
'UnicodeCharacter0xeccc': '\xee\xb3\x8c',
'UnicodeCharacter0xeccb': '\xee\xb3\x8b',
'HangulSyllableYoc': '\xec\x9a\xab',
'UnicodeCharacter0xf33e': '\xef\x8c\xbe',
'CanadianSyllabicsNgoo': '\xe1\x96\x92',
'UnicodeCharacter0xf33f': '\xef\x8c\xbf',
'HangulSyllablePeubs': '\xed\x94\x96',
'ArabicLetterAlefMaksuraFinalForm': '\xef\xbb\xb0',
'MongolianLetterSibeFa': '\xe1\xa1\xab',
'ThaiCharacterKhoKhon': '\xe0\xb8\x85',
'BraillePatternDots4678': '\xe2\xa3\xa8',
'UnicodeCharacter0xf33a': '\xef\x8c\xba',
'UnicodeCharacter0xecc9': '\xee\xb3\x89',
'HangulSyllableGyet': '\xea\xb3\x9d',
'HangulLetterNieunpansios': '\xe3\x85\xa8',
'EthiopicSectionMark': '\xe1\x8d\xa0',
'CjkUnifiedIdeograph904a': '\xe9\x81\x8a',
'UnicodeCharacter0xecc7': '\xee\xb3\x87',
'HangulSyllableNyeot': '\xeb\x85\x99',
'SinhalaLetterFayanna': '\xe0\xb7\x86',
'UnicodeCharacter0xecc0': '\xee\xb3\x80',
'UnicodeCharacter0xecc3': '\xee\xb3\x83',
'EthiopicSyllableKyo': '\xe2\xb7\x8e',
'HangulSyllableNwik': '\xeb\x89\xb0',
'HangulSyllableNwij': '\xeb\x89\xae',
'HangulSyllableNwih': '\xeb\x89\xb3',
'HangulSyllableNwin': '\xeb\x89\x9c',
'HangulSyllableNwim': '\xeb\x89\xa8',
'HangulSyllableNwil': '\xeb\x89\xa0',
'HangulSyllableNwic': '\xeb\x89\xaf',
'HangulSyllableNwib': '\xeb\x89\xa9',
'HangulSyllableNwig': '\xeb\x89\x99',
'HangulSyllableNwid': '\xeb\x89\x9f',
'HangulSyllableNaegs': '\xeb\x82\xb7',
'HangulSyllableSsam': '\xec\x8c\x88',
'GreekSmallLetterOmegaWithPsiliAndPerispomeniAndYpogegrammeni': '\xe1\xbe\xa6',
'YiSyllableBbut': '\xea\x81\xac',
'HangulSyllableSsah': '\xec\x8c\x93',
'HangulSyllableSsaj': '\xec\x8c\x8e',
'HangulSyllableSsak': '\xec\x8c\x90',
'MathematicalLeftFlattenedParenthesis': '\xe2\x9f\xae',
'HangulSyllableSsae': '\xec\x8c\x94',
'HangulSyllableSsag': '\xec\x8b\xb9',
'HangulSyllableSsab': '\xec\x8c\x89',
'HangulSyllableSsac': '\xec\x8c\x8f',
'LatinSmallLetterMWithHook': '\xc9\xb1',
'UnicodeCharacter0xf511': '\xef\x94\x91',
'CjkUnifiedIdeograph867c': '\xe8\x99\xbc',
'CjkUnifiedIdeograph867b': '\xe8\x99\xbb',
'CjkUnifiedIdeograph867e': '\xe8\x99\xbe',
'CjkUnifiedIdeograph867d': '\xe8\x99\xbd',
'UnicodeCharacter0xf516': '\xef\x94\x96',
'CjkUnifiedIdeograph867f': '\xe8\x99\xbf',
'HangulSyllableSsat': '\xec\x8c\x91',
'UnicodeCharacter0xf519': '\xef\x94\x99',
'HangulSyllableBubs': '\xeb\xb6\x92',
'HangulSyllableSsap': '\xec\x8c\x92',
'ArabicLigatureGhainWithJeemInitialForm': '\xef\xb2\xbc',
'HangulSyllableSsas': '\xec\x8c\x8b',
'CjkUnifiedIdeograph47a2': '\xe4\x9e\xa2',
'CjkUnifiedIdeograph47a3': '\xe4\x9e\xa3',
'CjkUnifiedIdeograph47a0': '\xe4\x9e\xa0',
'CjkUnifiedIdeograph47a1': '\xe4\x9e\xa1',
'CjkUnifiedIdeograph47a6': '\xe4\x9e\xa6',
'CjkUnifiedIdeograph47a7': '\xe4\x9e\xa7',
'CjkUnifiedIdeograph47a4': '\xe4\x9e\xa4',
'CjkUnifiedIdeograph47a5': '\xe4\x9e\xa5',
'UnicodeCharacter0xebcd': '\xee\xaf\x8d',
'UnicodeCharacter0xebce': '\xee\xaf\x8e',
'CjkUnifiedIdeograph47a8': '\xe4\x9e\xa8',
'CjkUnifiedIdeograph47a9': '\xe4\x9e\xa9',
'UnicodeCharacter0xebca': '\xee\xaf\x8a',
'UnicodeCharacter0xebcb': '\xee\xaf\x8b',
'UnicodeCharacter0xebcc': '\xee\xaf\x8c',
'HangulSyllablePogg': '\xed\x8f\xae',
'HangulSyllableNyeog': '\xeb\x85\x81',
'CjkUnifiedIdeograph78e9': '\xe7\xa3\xa9',
'CjkUnifiedIdeograph8671': '\xe8\x99\xb1',
'LatinSmallLetterLWithRetroflexHook': '\xc9\xad',
'CjkUnifiedIdeograph8673': '\xe8\x99\xb3',
'CjkUnifiedIdeograph8672': '\xe8\x99\xb2',
'CjkUnifiedIdeograph8675': '\xe8\x99\xb5',
'CjkUnifiedIdeograph8674': '\xe8\x99\xb4',
'CjkUnifiedIdeograph8677': '\xe8\x99\xb7',
'CjkUnifiedIdeograph8676': '\xe8\x99\xb6',
'CjkUnifiedIdeograph8679': '\xe8\x99\xb9',
'CjkUnifiedIdeograph8678': '\xe8\x99\xb8',
'HangulSyllableRwim': '\xeb\xa5\x8c',
'HangulSyllableRwil': '\xeb\xa5\x84',
'HangulSyllableRwik': '\xeb\xa5\x94',
'HangulSyllableRwij': '\xeb\xa5\x92',
'HangulSyllableRwih': '\xeb\xa5\x97',
'HexagramForDispersion': '\xe4\xb7\xba',
'HangulSyllableRwit': '\xeb\xa5\x95',
'HangulSyllableRwis': '\xeb\xa5\x8f',
'HangulSyllableNyeoj': '\xeb\x85\x96',
'CjkUnifiedIdeograph9112': '\xe9\x84\x92',
'ChamDigitZero': '\xea\xa9\x90',
'CjkUnifiedIdeograph89e0': '\xe8\xa7\xa0',
'HangulSyllableNyeoh': '\xeb\x85\x9b',
'HangulSyllableKunh': '\xec\xbf\xa6',
'HangulSyllableGyeo': '\xea\xb2\xa8',
'CjkUnifiedIdeograph47ab': '\xe4\x9e\xab',
'CjkUnifiedIdeograph47ac': '\xe4\x9e\xac',
'CjkUnifiedIdeograph47aa': '\xe4\x9e\xaa',
'CjkUnifiedIdeograph47af': '\xe4\x9e\xaf',
'HangulSyllableNyeom': '\xeb\x85\x90',
'NumberTenFullStop': '\xe2\x92\x91',
'CjkUnifiedIdeograph47ae': '\xe4\x9e\xae',
'UnicodeCharacter0xebc4': '\xee\xaf\x84',
'UnicodeCharacter0xebc5': '\xee\xaf\x85',
'RightSquareBracketWithTickInTopCorner': '\xe2\xa6\x90',
'HangulSyllableNyeol': '\xeb\x85\x88',
'UnicodeCharacter0xebc0': '\xee\xaf\x80',
'UnicodeCharacter0xebc1': '\xee\xaf\x81',
'UnicodeCharacter0xebc2': '\xee\xaf\x82',
'UnicodeCharacter0xebc3': '\xee\xaf\x83',
'CjkUnifiedIdeograph9f87': '\xe9\xbe\x87',
'CjkUnifiedIdeograph9f86': '\xe9\xbe\x86',
'IdeographicTelegraphSymbolForDayFour': '\xe3\x8f\xa3',
'CjkUnifiedIdeograph9f84': '\xe9\xbe\x84',
'CjkUnifiedIdeograph9f83': '\xe9\xbe\x83',
'CjkUnifiedIdeograph9f82': '\xe9\xbe\x82',
'CjkUnifiedIdeograph9f81': '\xe9\xbe\x81',
'CjkUnifiedIdeograph9f80': '\xe9\xbe\x80',
'CombiningEnclosingSquare': '\xe2\x83\x9e',
'CjkUnifiedIdeograph9f89': '\xe9\xbe\x89',
'CjkUnifiedIdeograph9f88': '\xe9\xbe\x88',
'CjkUnifiedIdeograph3a0f': '\xe3\xa8\x8f',
'UnicodeCharacter0xec00': '\xee\xb0\x80',
'CjkUnifiedIdeograph89e9': '\xe8\xa7\xa9',
'HangulSyllableHelp': '\xed\x97\xb2',
'UnicodeCharacter0xdc5c': '\xed\xb1\x9c',
'CanadianSyllabicsMi': '\xe1\x92\xa5',
'CanadianSyllabicsMh': '\xe1\x92\xbd',
'UnicodeCharacter0xeef': '\xe0\xbb\xaf',
'CanadianSyllabicsMo': '\xe1\x92\xa7',
'CanadianSyllabicsMa': '\xe1\x92\xaa',
'CjkUnifiedIdeograph3a0b': '\xe3\xa8\x8b',
'HangulSyllableHels': '\xed\x97\xb0',
'CanadianSyllabicsMe': '\xe1\x92\xa3',
'YiSyllableHnot': '\xea\x85\xb6',
'LatinCapitalLetterOWithHorn': '\xc6\xa0',
'UnicodeCharacter0xeeb': '\xe0\xbb\xab',
'YiSyllableHnop': '\xea\x85\xb8',
'UnicodeCharacter0xe278': '\xee\x89\xb8',
'LatinCapitalLetterEng': '\xc5\x8a',
'CjkUnifiedIdeograph9f8f': '\xe9\xbe\x8f',
'CjkUnifiedIdeograph9f8e': '\xe9\xbe\x8e',
'CjkUnifiedIdeograph9f8d': '\xe9\xbe\x8d',
'CjkUnifiedIdeograph9f8c': '\xe9\xbe\x8c',
'CjkUnifiedIdeograph9f8b': '\xe9\xbe\x8b',
'CjkUnifiedIdeograph9f8a': '\xe9\xbe\x8a',
'CjkUnifiedIdeograph9f6c': '\xe9\xbd\xac',
'LimbuDigitTwo': '\xe1\xa5\x88',
'HangulLetterSsangsios': '\xe3\x85\x86',
'HangulSyllableSsyek': '\xec\x8f\x94',
'HangulSyllableSsyeh': '\xec\x8f\x97',
'HangulSyllableJjogs': '\xec\xaa\xbf',
'HangulSyllableSsyen': '\xec\x8f\x80',
'HangulSyllableSsyeo': '\xec\x8e\xa0',
'HangulSyllableSsyel': '\xec\x8f\x84',
'HangulSyllableSsyem': '\xec\x8f\x8c',
'HangulSyllableSsyeb': '\xec\x8f\x8d',
'HangulSyllableSsyec': '\xec\x8f\x93',
'CyrillicSmallLetterHaWithStroke': '\xd3\xbf',
'UnicodeCharacter0xe279': '\xee\x89\xb9',
'HangulSyllableSsyeg': '\xec\x8e\xbd',
'HangulSyllableSsyed': '\xec\x8f\x83',
'UnicodeCharacter0xdd03': '\xed\xb4\x83',
'YiRadicalNbie': '\xea\x93\x85',
'HangulSyllableSsig': '\xec\x94\xa9',
'BlackDiamond': '\xe2\x97\x86',
'HangulSyllableSsyes': '\xec\x8f\x8f',
'HangulSyllableSsyep': '\xec\x8f\x96',
'HangulSyllableSsyet': '\xec\x8f\x95',
'HangulSyllableDwibs': '\xeb\x92\xb6',
'CjkUnifiedIdeograph3ad2': '\xe3\xab\x92',
'CjkUnifiedIdeograph3ad3': '\xe3\xab\x93',
'CjkUnifiedIdeograph3ad0': '\xe3\xab\x90',
'CjkUnifiedIdeograph3ad1': '\xe3\xab\x91',
'CjkUnifiedIdeograph3ad6': '\xe3\xab\x96',
'CjkUnifiedIdeograph3ad7': '\xe3\xab\x97',
'CjkUnifiedIdeograph3ad4': '\xe3\xab\x94',
'CjkUnifiedIdeograph3ad5': '\xe3\xab\x95',
'CjkUnifiedIdeograph733e': '\xe7\x8c\xbe',
'CjkUnifiedIdeograph733d': '\xe7\x8c\xbd',
'CjkUnifiedIdeograph3ad8': '\xe3\xab\x98',
'CjkUnifiedIdeograph3ad9': '\xe3\xab\x99',
'CjkUnifiedIdeograph733a': '\xe7\x8c\xba',
'UnicodeCharacter0xdd04': '\xed\xb4\x84',
'CjkUnifiedIdeograph733c': '\xe7\x8c\xbc',
'CjkUnifiedIdeograph733b': '\xe7\x8c\xbb',
'HangulSyllableKaeg': '\xec\xba\x91',
'HangulSyllableKaed': '\xec\xba\x97',
'HangulSyllableGgweonh': '\xea\xbf\x9a',
'HangulSyllableByegs': '\xeb\xb3\x9b',
'HangulSyllableJjeogg': '\xec\xa9\x8e',
'HangulSyllableSsim': '\xec\x94\xb8',
'HangulSyllableKaen': '\xec\xba\x94',
'HangulSyllableKael': '\xec\xba\x98',
'CjkUnifiedIdeograph9042': '\xe9\x81\x82',
'HangulSyllableKaej': '\xec\xba\xa6',
'HangulSyllableKaek': '\xec\xba\xa8',
'HangulSyllableKaeh': '\xec\xba\xab',
'CjkUnifiedIdeograph9070': '\xe9\x81\xb0',
'CjkUnifiedIdeograph9071': '\xe9\x81\xb1',
'EqualsSignWithDotBelow': '\xe2\xa9\xa6',
'CjkUnifiedIdeograph9073': '\xe9\x81\xb3',
'CjkUnifiedIdeograph9074': '\xe9\x81\xb4',
'CjkUnifiedIdeograph9075': '\xe9\x81\xb5',
'BalineseLetterTaLatik': '\xe1\xac\x9d',
'CjkUnifiedIdeograph9077': '\xe9\x81\xb7',
'CjkUnifiedIdeograph8f6f': '\xe8\xbd\xaf',
'CjkUnifiedIdeograph458f': '\xe4\x96\x8f',
'CjkUnifiedIdeograph458e': '\xe4\x96\x8e',
'CjkUnifiedIdeograph458d': '\xe4\x96\x8d',
'CjkUnifiedIdeograph458c': '\xe4\x96\x8c',
'CjkUnifiedIdeograph458b': '\xe4\x96\x8b',
'CjkUnifiedIdeograph458a': '\xe4\x96\x8a',
'CjkUnifiedIdeograph8f6a': '\xe8\xbd\xaa',
'CjkUnifiedIdeograph66f0': '\xe6\x9b\xb0',
'CjkUnifiedIdeograph66f1': '\xe6\x9b\xb1',
'CjkUnifiedIdeograph66f2': '\xe6\x9b\xb2',
'CjkUnifiedIdeograph66f3': '\xe6\x9b\xb3',
'CjkUnifiedIdeograph66f4': '\xe6\x9b\xb4',
'CjkUnifiedIdeograph66f5': '\xe6\x9b\xb5',
'CjkUnifiedIdeograph66f6': '\xe6\x9b\xb6',
'CjkUnifiedIdeograph66f7': '\xe6\x9b\xb7',
'CjkUnifiedIdeograph66f8': '\xe6\x9b\xb8',
'CjkUnifiedIdeograph66f9': '\xe6\x9b\xb9',
'Omega': '\xce\xa9',
'MongolianLetterMa': '\xe1\xa0\xae',
'HebrewLetterFinalTsadi': '\xd7\xa5',
'CjkUnifiedIdeograph3adb': '\xe3\xab\x9b',
'CjkUnifiedIdeograph3adc': '\xe3\xab\x9c',
'CjkUnifiedIdeograph9afd': '\xe9\xab\xbd',
'CjkUnifiedIdeograph3ada': '\xe3\xab\x9a',
'CjkUnifiedIdeograph3adf': '\xe3\xab\x9f',
'CjkUnifiedIdeograph7338': '\xe7\x8c\xb8',
'CjkUnifiedIdeograph3add': '\xe3\xab\x9d',
'CjkUnifiedIdeograph3ade': '\xe3\xab\x9e',
'CjkUnifiedIdeograph7335': '\xe7\x8c\xb5',
'CjkUnifiedIdeograph7334': '\xe7\x8c\xb4',
'CjkUnifiedIdeograph7337': '\xe7\x8c\xb7',
'CjkUnifiedIdeograph7336': '\xe7\x8c\xb6',
'CjkUnifiedIdeograph7331': '\xe7\x8c\xb1',
'CjkUnifiedIdeograph7330': '\xe7\x8c\xb0',
'CjkUnifiedIdeograph7333': '\xe7\x8c\xb3',
'CjkUnifiedIdeograph7332': '\xe7\x8c\xb2',
'CjkUnifiedIdeograph66fa': '\xe6\x9b\xba',
'CjkUnifiedIdeograph66fb': '\xe6\x9b\xbb',
'CjkUnifiedIdeograph66fc': '\xe6\x9b\xbc',
'SquareKOhm': '\xe3\x8f\x80',
'CjkUnifiedIdeograph66fe': '\xe6\x9b\xbe',
'ThaanaLetterChaviyani': '\xde\x97',
'ParenthesizedIdeographSelf': '\xe3\x89\x82',
'HangulSyllablePagg': '\xed\x8c\x8e',
'CjkUnifiedIdeograph888f': '\xe8\xa2\x8f',
'CjkUnifiedIdeograph5d17': '\xe5\xb4\x97',
'CjkUnifiedIdeograph907a': '\xe9\x81\xba',
'CjkUnifiedIdeograph907b': '\xe9\x81\xbb',
'CjkUnifiedIdeograph907c': '\xe9\x81\xbc',
'CjkUnifiedIdeograph907d': '\xe9\x81\xbd',
'CjkUnifiedIdeograph907e': '\xe9\x81\xbe',
'CjkUnifiedIdeograph4589': '\xe4\x96\x89',
'CjkUnifiedIdeograph4588': '\xe4\x96\x88',
'CjkUnifiedIdeograph4587': '\xe4\x96\x87',
'CjkUnifiedIdeograph4586': '\xe4\x96\x86',
'CjkUnifiedIdeograph4585': '\xe4\x96\x85',
'CjkUnifiedIdeograph4584': '\xe4\x96\x84',
'CjkUnifiedIdeograph4583': '\xe4\x96\x83',
'CjkUnifiedIdeograph4582': '\xe4\x96\x82',
'CjkUnifiedIdeograph4581': '\xe4\x96\x81',
'CjkUnifiedIdeograph4580': '\xe4\x96\x80',
'HangulSyllableGyonh': '\xea\xb5\x96',
'HangulSyllableHwaep': '\xed\x9a\x8a',
'HangulSyllableHwaes': '\xed\x9a\x83',
'HangulSyllableHwaet': '\xed\x9a\x89',
'HangulSyllableHwaeh': '\xed\x9a\x8b',
'HangulSyllableTaelg': '\xed\x83\xa5',
'HangulSyllableHwaej': '\xed\x9a\x86',
'HangulSyllableHwaem': '\xed\x9a\x80',
'HangulSyllableHwael': '\xed\x99\xb8',
'HangulSyllableHwaen': '\xed\x99\xb4',
'HangulSyllableBbyaess': '\xeb\xbb\x88',
'HangulSyllableHwaec': '\xed\x9a\x87',
'HangulSyllableHwaeb': '\xed\x9a\x81',
'HangulSyllableHwaed': '\xed\x99\xb7',
'HangulSyllableHwaeg': '\xed\x99\xb1',
'CjkUnifiedIdeograph5184': '\xe5\x86\x84',
'CjkUnifiedIdeograph5185': '\xe5\x86\x85',
'CjkUnifiedIdeograph5186': '\xe5\x86\x86',
'CjkUnifiedIdeograph5187': '\xe5\x86\x87',
'CjkUnifiedIdeograph5180': '\xe5\x86\x80',
'CjkUnifiedIdeograph5181': '\xe5\x86\x81',
'CjkUnifiedIdeograph5182': '\xe5\x86\x82',
'CjkUnifiedIdeograph5183': '\xe5\x86\x83',
'CjkUnifiedIdeograph888d': '\xe8\xa2\x8d',
'ArabicLetterQafIsolatedForm': '\xef\xbb\x95',
'CjkUnifiedIdeograph5188': '\xe5\x86\x88',
'CjkUnifiedIdeograph5189': '\xe5\x86\x89',
'UnicodeCharacter0xf398': '\xef\x8e\x98',
'CjkUnifiedIdeograph9694': '\xe9\x9a\x94',
'CjkUnifiedIdeograph9695': '\xe9\x9a\x95',
'CjkUnifiedIdeograph9696': '\xe9\x9a\x96',
'CjkUnifiedIdeograph9697': '\xe9\x9a\x97',
'CjkUnifiedIdeograph9690': '\xe9\x9a\x90',
'CjkUnifiedIdeograph9691': '\xe9\x9a\x91',
'CanadianSyllabicsWestcreeMwoo': '\xe1\x92\xb5',
'CjkUnifiedIdeograph9693': '\xe9\x9a\x93',
'CjkUnifiedIdeograph523f': '\xe5\x88\xbf',
'CjkUnifiedIdeograph7829': '\xe7\xa0\xa9',
'CjkUnifiedIdeograph523d': '\xe5\x88\xbd',
'CjkUnifiedIdeograph523e': '\xe5\x88\xbe',
'CjkUnifiedIdeograph523b': '\xe5\x88\xbb',
'CjkUnifiedIdeograph523c': '\xe5\x88\xbc',
'LatinSmallLetterVy': '\xea\x9d\xa1',
'CjkUnifiedIdeograph523a': '\xe5\x88\xba',
'HangulSyllablePug': '\xed\x91\xb9',
'ArmenianEmphasisMark': '\xd5\x9b',
'HangulSyllablePud': '\xed\x91\xbf',
'HangulSyllablePuc': '\xed\x92\x8f',
'HangulSyllablePub': '\xed\x92\x89',
'UnicodeCharacter0xf05f': '\xef\x81\x9f',
'HangulSyllablePun': '\xed\x91\xbc',
'HangulSyllablePum': '\xed\x92\x88',
'HangulSyllablePul': '\xed\x92\x80',
'HangulSyllablePuk': '\xed\x92\x90',
'HangulSyllablePeolt': '\xed\x8e\x89',
'HangulSyllablePuh': '\xed\x92\x93',
'HangulSyllablePut': '\xed\x92\x91',
'HangulSyllablePus': '\xed\x92\x8b',
'GreaterthanWithQuestionMarkAbove': '\xe2\xa9\xbc',
'HangulSyllablePeols': '\xed\x8e\x88',
'CjkUnifiedIdeograph969d': '\xe9\x9a\x9d',
'CjkUnifiedIdeograph969e': '\xe9\x9a\x9e',
'CjkUnifiedIdeograph969f': '\xe9\x9a\x9f',
'CjkUnifiedIdeograph7826': '\xe7\xa0\xa6',
'LatinSmallLetterSWithSwashTail': '\xc8\xbf',
'RightSquareBracketLowerCorner': '\xe2\x8e\xa6',
'CjkUnifiedIdeograph5238': '\xe5\x88\xb8',
'CjkUnifiedIdeograph5239': '\xe5\x88\xb9',
'CjkUnifiedIdeograph5236': '\xe5\x88\xb6',
'CjkUnifiedIdeograph5237': '\xe5\x88\xb7',
'CjkUnifiedIdeograph5234': '\xe5\x88\xb4',
'VerticalIdeographicIterationMark': '\xe3\x80\xbb',
'CjkUnifiedIdeograph5232': '\xe5\x88\xb2',
'CjkUnifiedIdeograph5233': '\xe5\x88\xb3',
'UnicodeCharacter0x176d': '\xe1\x9d\xad',
'CjkUnifiedIdeograph5231': '\xe5\x88\xb1',
'CjkUnifiedIdeograph518d': '\xe5\x86\x8d',
'CjkUnifiedIdeograph518e': '\xe5\x86\x8e',
'CjkUnifiedIdeograph518f': '\xe5\x86\x8f',
'CjkUnifiedIdeograph518a': '\xe5\x86\x8a',
'CjkUnifiedIdeograph518b': '\xe5\x86\x8b',
'CjkUnifiedIdeograph518c': '\xe5\x86\x8c',
'CjkUnifiedIdeograph6523': '\xe6\x94\xa3',
'CjkUnifiedIdeograph6522': '\xe6\x94\xa2',
'CjkUnifiedIdeograph6521': '\xe6\x94\xa1',
'CjkUnifiedIdeograph6520': '\xe6\x94\xa0',
'CjkUnifiedIdeograph5d10': '\xe5\xb4\x90',
'CjkUnifiedIdeograph6526': '\xe6\x94\xa6',
'CjkUnifiedIdeograph6525': '\xe6\x94\xa5',
'CjkUnifiedIdeograph6524': '\xe6\x94\xa4',
'CjkUnifiedIdeograph6529': '\xe6\x94\xa9',
'CjkUnifiedIdeograph6528': '\xe6\x94\xa8',
'UpwardsHarpoonWithBarbLeftToBar': '\xe2\xa5\x98',
'GreekSmallLetterOmegaWithPsiliAndOxia': '\xe1\xbd\xa4',
'CjkUnifiedIdeograph7323': '\xe7\x8c\xa3',
'HangulSyllableKyeon': '\xec\xbc\xa0',
'CopticSmallLetterDei': '\xcf\xaf',
'CjkUnifiedIdeograph4c11': '\xe4\xb0\x91',
'CjkUnifiedIdeograph58b2': '\xe5\xa2\xb2',
'CjkUnifiedIdeograph58b1': '\xe5\xa2\xb1',
'CjkUnifiedIdeograph58b0': '\xe5\xa2\xb0',
'CjkUnifiedIdeograph58b7': '\xe5\xa2\xb7',
'CjkUnifiedIdeograph58b6': '\xe5\xa2\xb6',
'CjkUnifiedIdeograph58b5': '\xe5\xa2\xb5',
'CjkUnifiedIdeograph58b4': '\xe5\xa2\xb4',
'CjkUnifiedIdeograph58b9': '\xe5\xa2\xb9',
'CjkUnifiedIdeograph58b8': '\xe5\xa2\xb8',
'HebrewLetterYodWithDagesh': '\xef\xac\xb9',
'HangulSyllableKyeob': '\xec\xbc\xad',
'HangulSyllableBwing': '\xeb\xb7\xa9',
'CjkUnifiedIdeograph64b7': '\xe6\x92\xb7',
'HangulSyllableBwinj': '\xeb\xb7\x99',
'UnicodeCharacter0xdc58': '\xed\xb1\x98',
'HangulSyllableBwinh': '\xeb\xb7\x9a',
'EthiopicSyllablePwee': '\xe1\x8e\x8e',
'UnicodeCharacter0xfbce': '\xef\xaf\x8e',
'HangulSyllableTaelm': '\xed\x83\xa6',
'GreekCapitalLetterEpsilonWithVaria': '\xe1\xbf\x88',
'UnicodeCharacter0xeb6f': '\xee\xad\xaf',
'HangulSyllableKyeog': '\xec\xbc\x9d',
'CjkUnifiedIdeograph652c': '\xe6\x94\xac',
'CjkUnifiedIdeograph652b': '\xe6\x94\xab',
'CjkUnifiedIdeograph652a': '\xe6\x94\xaa',
'CjkUnifiedIdeograph652f': '\xe6\x94\xaf',
'CjkUnifiedIdeograph652e': '\xe6\x94\xae',
'CjkUnifiedIdeograph652d': '\xe6\x94\xad',
'UnicodeCharacter0xe7b7': '\xee\x9e\xb7',
'UnicodeCharacter0xdc56': '\xed\xb1\x96',
'CjkUnifiedIdeograph7f80': '\xe7\xbe\x80',
'HangulSyllablePeolm': '\xed\x8e\x86',
'HangulSyllableCyess': '\xec\xb4\x80',
'UnicodeCharacter0xeb6e': '\xee\xad\xae',
'HalfwidthLeftCornerBracket': '\xef\xbd\xa2',
'UnicodeCharacter0xdc51': '\xed\xb1\x91',
'CjkUnifiedIdeograph58bc': '\xe5\xa2\xbc',
'CjkUnifiedIdeograph58bb': '\xe5\xa2\xbb',
'CjkUnifiedIdeograph58ba': '\xe5\xa2\xba',
'CjkUnifiedIdeograph58bf': '\xe5\xa2\xbf',
'CjkUnifiedIdeograph58be': '\xe5\xa2\xbe',
'CjkUnifiedIdeograph58bd': '\xe5\xa2\xbd',
'YiSyllableBbuo': '\xea\x81\xa3',
'KhmerSignKhan': '\xe1\x9f\x94',
'AplFunctionalSymbolBackslashBar': '\xe2\x8d\x80',
'CjkCompatibilityIdeographfa26': '\xef\xa8\xa6',
'HangulJongseongSios': '\xe1\x86\xba',
'UnicodeCharacter0xe7b5': '\xee\x9e\xb5',
'HangulSyllableSsyaess': '\xec\x8d\xa0',
'HangulSyllableEbs': '\xec\x97\xa2',
'HangulSyllableDinj': '\xeb\x94\x99',
'HangulSyllableDinh': '\xeb\x94\x9a',
'HangulSyllableYeogg': '\xec\x97\xae',
'HangulSyllableHeogs': '\xed\x97\x8b',
'HangulSyllableDing': '\xeb\x94\xa9',
'LowerRightSemicircularClockwiseArrow': '\xe2\xa4\xbe',
'CjkUnifiedIdeograph3c2f': '\xe3\xb0\xaf',
'CjkUnifiedIdeograph7693': '\xe7\x9a\x93',
'CjkUnifiedIdeograph3c2d': '\xe3\xb0\xad',
'CjkUnifiedIdeograph3c2e': '\xe3\xb0\xae',
'CjkUnifiedIdeograph3c2b': '\xe3\xb0\xab',
'CjkUnifiedIdeograph3c2c': '\xe3\xb0\xac',
'CjkUnifiedIdeograph7694': '\xe7\x9a\x94',
'CjkUnifiedIdeograph3c2a': '\xe3\xb0\xaa',
'HangulSyllableHeogg': '\xed\x97\x8a',
'CjkUnifiedIdeograph7698': '\xe7\x9a\x98',
'CjkUnifiedIdeograph7699': '\xe7\x9a\x99',
'UnicodeCharacter0xe15e': '\xee\x85\x9e',
'UnicodeCharacter0xe15d': '\xee\x85\x9d',
'UnicodeCharacter0xf57b': '\xef\x95\xbb',
'CjkUnifiedIdeograph8f13': '\xe8\xbc\x93',
'UnicodeCharacter0xe15f': '\xee\x85\x9f',
'ArabicLetterHehDoachashmeeIsolatedForm': '\xef\xae\xaa',
'CjkUnifiedIdeograph8f12': '\xe8\xbc\x92',
'UnicodeCharacter0xe15a': '\xee\x85\x9a',
'CjkUnifiedIdeograph8f11': '\xe8\xbc\x91',
'HangulSyllableSyeobs': '\xec\x85\xa6',
'CjkUnifiedIdeograph9e70': '\xe9\xb9\xb0',
'CjkUnifiedIdeograph3c26': '\xe3\xb0\xa6',
'CjkUnifiedIdeograph3c27': '\xe3\xb0\xa7',
'CjkUnifiedIdeograph3c24': '\xe3\xb0\xa4',
'CjkUnifiedIdeograph3c25': '\xe3\xb0\xa5',
'CjkUnifiedIdeograph3c22': '\xe3\xb0\xa2',
'CjkUnifiedIdeograph3c23': '\xe3\xb0\xa3',
'CjkUnifiedIdeograph3c20': '\xe3\xb0\xa0',
'CjkUnifiedIdeograph3c21': '\xe3\xb0\xa1',
'CjkUnifiedIdeograph4590': '\xe4\x96\x90',
'UnicodeCharacter0xe15b': '\xee\x85\x9b',
'CjkUnifiedIdeograph3c28': '\xe3\xb0\xa8',
'CjkUnifiedIdeograph3c29': '\xe3\xb0\xa9',
'SaurashtraDigitEight': '\xea\xa3\x98',
'HangulSyllableDyils': '\xeb\x94\x84',
'CjkUnifiedIdeograph8f15': '\xe8\xbc\x95',
'CjkUnifiedIdeograph8f14': '\xe8\xbc\x94',
'HangulSyllableHyug': '\xed\x9c\xb5',
'FullwidthLatinCapitalLetterX': '\xef\xbc\xb8',
'HangulSyllableHyud': '\xed\x9c\xbb',
'TagalogLetterYa': '\xe1\x9c\x8c',
'HangulSyllableHyub': '\xed\x9d\x85',
'CjkUnifiedIdeograph9e79': '\xe9\xb9\xb9',
'HangulSyllableHyum': '\xed\x9d\x84',
'HangulSyllableHyul': '\xed\x9c\xbc',
'ThaanaLetterJaviyani': '\xde\x96',
'HangulSyllableHyuj': '\xed\x9d\x8a',
'HangulSyllableHyuh': '\xed\x9d\x8f',
'HangulSyllableHyut': '\xed\x9d\x8d',
'CanadianSyllabicsThwaa': '\xe1\x95\xa8',
'CjkUnifiedIdeograph820e': '\xe8\x88\x8e',
'ArabicLigatureShaddaWithKasraMedialForm': '\xef\xb3\xb4',
'CjkUnifiedIdeograph6e30': '\xe6\xb8\xb0',
'CjkUnifiedIdeograph6e31': '\xe6\xb8\xb1',
'CjkUnifiedIdeograph6e32': '\xe6\xb8\xb2',
'CjkUnifiedIdeograph69e5': '\xe6\xa7\xa5',
'LatinCapitalLetterAWithStroke': '\xc8\xba',
'CjkUnifiedIdeograph6e35': '\xe6\xb8\xb5',
'CjkUnifiedIdeograph6e36': '\xe6\xb8\xb6',
'CjkUnifiedIdeograph6e37': '\xe6\xb8\xb7',
'CjkUnifiedIdeograph6e38': '\xe6\xb8\xb8',
'CjkUnifiedIdeograph6e39': '\xe6\xb8\xb9',
'CjkUnifiedIdeograph820c': '\xe8\x88\x8c',
'CanadianSyllabicsShaa': '\xe1\x94\x96',
'UnicodeCharacter0xe5b0': '\xee\x96\xb0',
'HangulSyllableSyeolb': '\xec\x85\x9f',
'PhagspaLetterSha': '\xea\xa1\x9a',
'AplFunctionalSymbolCircleBackslash': '\xe2\x8d\x89',
'HangulJungseongYoyeo': '\xe1\x86\x86',
'ArabicLigatureDadWithHahWithYehFinalForm': '\xef\xb6\xab',
'CjkUnifiedIdeograph6e3a': '\xe6\xb8\xba',
'CjkUnifiedIdeograph6e3b': '\xe6\xb8\xbb',
'CjkUnifiedIdeograph6e3c': '\xe6\xb8\xbc',
'CjkUnifiedIdeograph6e3d': '\xe6\xb8\xbd',
'CjkUnifiedIdeograph6e3e': '\xe6\xb8\xbe',
'CjkUnifiedIdeograph6e3f': '\xe6\xb8\xbf',
'NewTaiLueDigitSix': '\xe1\xa7\x96',
'UnicodeCharacter0xda62': '\xed\xa9\xa2',
'^1': '\xc2\xb9',
'/\\|\\/': '\xe2\x86\x95',
'VaiDigitFour': '\xea\x98\xa4',
'UnicodeCharacter0xe5b2': '\xee\x96\xb2',
'plusminus': '\xc2\xb1',
'HangulSyllableSyeolh': '\xec\x85\xa3',
'CjkUnifiedIdeograph5a6b': '\xe5\xa9\xab',
'CjkUnifiedIdeograph5a6c': '\xe5\xa9\xac',
'BoxDrawingsLightArcUpAndLeft': '\xe2\x95\xaf',
'CjkUnifiedIdeograph5a6f': '\xe5\xa9\xaf',
'TagalogVowelSignI': '\xe1\x9c\x92',
'CjkUnifiedIdeograph5a6d': '\xe5\xa9\xad',
'CjkUnifiedIdeograph5a6e': '\xe5\xa9\xae',
'CjkUnifiedIdeograph4a03': '\xe4\xa8\x83',
'CjkUnifiedIdeograph4a02': '\xe4\xa8\x82',
'CjkUnifiedIdeograph4a01': '\xe4\xa8\x81',
'CjkUnifiedIdeograph4a00': '\xe4\xa8\x80',
'CjkUnifiedIdeograph4a07': '\xe4\xa8\x87',
'CjkUnifiedIdeograph4a06': '\xe4\xa8\x86',
'CjkUnifiedIdeograph4a05': '\xe4\xa8\x85',
'CjkUnifiedIdeograph4a04': '\xe4\xa8\x84',
'ObliqueAngleOpeningDown': '\xe2\xa6\xa7',
'CjkUnifiedIdeograph4a09': '\xe4\xa8\x89',
'CjkUnifiedIdeograph4a08': '\xe4\xa8\x88',
'UnicodeCharacter0xe5b3': '\xee\x96\xb3',
'YiSyllableWuox': '\xea\x8a\x84',
'multiply': '\xc3\x97',
'HangulSyllableGass': '\xea\xb0\x94',
'YiSyllableWuop': '\xea\x8a\x86',
'CyrillicCapitalLetterUkrainianIe': '\xd0\x84',
'HangulSyllableJjweoss': '\xec\xad\xb8',
'TaiLeLetterAi': '\xe1\xa5\xad',
'ArabicLigatureMeemWithMeemInitialForm': '\xef\xb3\x91',
'CanadianSyllabicsPi': '\xe1\x90\xb1',
'HangulSyllableKoem': '\xec\xbe\xb8',
'CanadianSyllabicsPo': '\xe1\x90\xb3',
'BraillePatternDots2467': '\xe2\xa1\xaa',
'BraillePatternDots2468': '\xe2\xa2\xaa',
'CanadianSyllabicsPa': '\xe1\x90\xb8',
'CjkUnifiedIdeograph864c': '\xe8\x99\x8c',
'CanadianSyllabicsPe': '\xe1\x90\xaf',
'CjkUnifiedIdeograph4a0c': '\xe4\xa8\x8c',
'CjkUnifiedIdeograph4a0b': '\xe4\xa8\x8b',
'CjkUnifiedIdeograph4a0a': '\xe4\xa8\x8a',
'CjkUnifiedIdeograph4a0f': '\xe4\xa8\x8f',
'CjkUnifiedIdeograph4a0e': '\xe4\xa8\x8e',
'CjkUnifiedIdeograph4a0d': '\xe4\xa8\x8d',
'CjkUnifiedIdeograph9f6b': '\xe9\xbd\xab',
'KhmerVowelSignOo': '\xe1\x9f\x84',
'YiSyllableHxot': '\xea\x89\x96',
'CjkUnifiedIdeograph5a68': '\xe5\xa9\xa8',
'CjkUnifiedIdeograph5a69': '\xe5\xa9\xa9',
'YiSyllableHxop': '\xea\x89\x99',
'CjkUnifiedIdeograph5a62': '\xe5\xa9\xa2',
'CjkUnifiedIdeograph5a63': '\xe5\xa9\xa3',
'CjkUnifiedIdeograph5a60': '\xe5\xa9\xa0',
'CjkUnifiedIdeograph5a61': '\xe5\xa9\xa1',
'CjkUnifiedIdeograph5a66': '\xe5\xa9\xa6',
'CjkUnifiedIdeograph5a67': '\xe5\xa9\xa7',
'CjkUnifiedIdeograph5a64': '\xe5\xa9\xa4',
'CjkUnifiedIdeograph5a65': '\xe5\xa9\xa5',
'LeftwardsArrowAboveReverseTildeOperator': '\xe2\xad\x8b',
'HangulSyllableKyeogg': '\xec\xbc\x9e',
'HangulSyllableCyaess': '\xec\xb2\x90',
'HangulSyllableSyugg': '\xec\x8a\x8a',
'CjkUnifiedIdeograph8f1c': '\xe8\xbc\x9c',
'CircleDividedByHorizontalBarAndTopHalfDividedByVerticalBar': '\xe2\xa6\xba',
'CjkUnifiedIdeograph8f1b': '\xe8\xbc\x9b',
'HangulSyllableNeuj': '\xeb\x8a\xa6',
'CjkUnifiedIdeograph8f1a': '\xe8\xbc\x9a',
'UnicodeCharacter0xda65': '\xed\xa9\xa5',
'EthiopicDigitThree': '\xe1\x8d\xab',
'ArabicKasraIsolatedForm': '\xef\xb9\xba',
'CjkUnifiedIdeograph9cd0': '\xe9\xb3\x90',
'CjkUnifiedIdeograph8f1f': '\xe8\xbc\x9f',
'CjkUnifiedIdeograph7b3f': '\xe7\xac\xbf',
'CjkUnifiedIdeograph8f1e': '\xe8\xbc\x9e',
'UnicodeCharacter0xf412': '\xef\x90\x92',
'HangulSyllableCabs': '\xec\xb0\xba',
'CjkUnifiedIdeograph459c': '\xe4\x96\x9c',
'LatinSmallLetterAWithRingAbove': '\xc3\xa5',
'ArabicLigatureKafWithJeemInitialForm': '\xef\xb3\x84',
'UnicodeCharacter0xdbb8': '\xed\xae\xb8',
'UnicodeCharacter0xdbb9': '\xed\xae\xb9',
'CjkUnifiedIdeograph8205': '\xe8\x88\x85',
'UnicodeCharacter0xdbb0': '\xed\xae\xb0',
'HangulSyllableCug': '\xec\xb6\x95',
'UnicodeCharacter0xdbb2': '\xed\xae\xb2',
'HangulSyllableJjeoj': '\xec\xa9\xa2',
'LatinCapitalLetterYWithCircumflex': '\xc5\xb6',
'CjkUnifiedIdeograph8206': '\xe8\x88\x86',
'UnicodeCharacter0xdbb6': '\xed\xae\xb6',
'UnicodeCharacter0xdbb7': '\xed\xae\xb7',
'TibetanMarkNyisTshegShad': '\xe0\xbc\x90',
'HangulSyllableBoss': '\xeb\xb4\x88',
'KayahLiLetterTa': '\xea\xa4\x92',
'LatinLetterInvertedGlottalStop': '\xca\x96',
'MediumFlattenedLeftParenthesisOrnament': '\xe2\x9d\xaa',
'DoublelineSlantedLessthanOrEqualTo': '\xe2\xab\xb9',
'UnicodeCharacter0xea56': '\xee\xa9\x96',
'HangulSyllableKoek': '\xec\xbf\x80',
'UnicodeCharacter0xdbba': '\xed\xae\xba',
'UnicodeCharacter0xdbbb': '\xed\xae\xbb',
'UnicodeCharacter0xdbbc': '\xed\xae\xbc',
'UnicodeCharacter0xdbbd': '\xed\xae\xbd',
'UnicodeCharacter0xdbbe': '\xed\xae\xbe',
'UnicodeCharacter0xdbbf': '\xed\xae\xbf',
'HangulSyllableBweobs': '\xeb\xb6\xae',
'UnicodeCharacter0xddc5': '\xed\xb7\x85',
'UnicodeCharacter0xddc4': '\xed\xb7\x84',
'UnicodeCharacter0xda69': '\xed\xa9\xa9',
'HangulSyllableNyap': '\xeb\x83\xaa',
'HangulSyllableNyas': '\xeb\x83\xa3',
'UnicodeCharacter0xddc6': '\xed\xb7\x86',
'UnicodeCharacter0xec8d': '\xee\xb2\x8d',
'HangulSyllableNyat': '\xeb\x83\xa9',
'CjkUnifiedIdeograph8479': '\xe8\x91\xb9',
'CjkUnifiedIdeograph8478': '\xe8\x91\xb8',
'CjkUnifiedIdeograph8477': '\xe8\x91\xb7',
'CjkUnifiedIdeograph8476': '\xe8\x91\xb6',
'CjkUnifiedIdeograph8475': '\xe8\x91\xb5',
'CjkUnifiedIdeograph8474': '\xe8\x91\xb4',
'CjkUnifiedIdeograph8473': '\xe8\x91\xb3',
'LatinSmallLetterCWithCaron': '\xc4\x8d',
'CjkUnifiedIdeograph8471': '\xe8\x91\xb1',
'CjkUnifiedIdeograph8470': '\xe8\x91\xb0',
'HangulSyllableBbyaeb': '\xeb\xbb\x85',
'HangulSyllableNeum': '\xeb\x8a\xa0',
'HangulSyllableNyac': '\xeb\x83\xa7',
'HangulSyllableNyab': '\xeb\x83\xa1',
'ArabicLetterRehWithFourDotsAbove': '\xda\x99',
'HangulSyllableNyad': '\xeb\x83\x97',
'HangulSyllableNyag': '\xeb\x83\x91',
'UnicodeCharacter0xda68': '\xed\xa9\xa8',
'HangulSyllableBbyaej': '\xeb\xbb\x8a',
'HangulSyllableNyah': '\xeb\x83\xab',
'HangulSyllableNyak': '\xeb\x83\xa8',
'HangulSyllableNyaj': '\xeb\x83\xa6',
'HangulSyllableNyam': '\xeb\x83\xa0',
'HangulSyllableNyal': '\xeb\x83\x98',
'HangulSyllableByolb': '\xeb\xb5\xaf',
'HangulSyllableNyan': '\xeb\x83\x94',
'OlChikiLetterUy': '\xe1\xb1\xad',
'UnicodeCharacter0x12bf': '\xe1\x8a\xbf',
'HangulSyllableMon': '\xeb\xaa\xac',
'HangulSyllableMoh': '\xeb\xab\x83',
'HangulSyllableMoj': '\xeb\xaa\xbe',
'HangulSyllableMok': '\xeb\xab\x80',
'CjkUnifiedIdeograph8ef1': '\xe8\xbb\xb1',
'CjkUnifiedIdeograph81f9': '\xe8\x87\xb9',
'CjkUnifiedIdeograph8ef3': '\xe8\xbb\xb3',
'CjkUnifiedIdeograph8ef2': '\xe8\xbb\xb2',
'CjkUnifiedIdeograph8ef5': '\xe8\xbb\xb5',
'CjkUnifiedIdeograph8ef4': '\xe8\xbb\xb4',
'CjkUnifiedIdeograph8ef7': '\xe8\xbb\xb7',
'CjkUnifiedIdeograph8ef6': '\xe8\xbb\xb6',
'HangulSyllablePyun': '\xed\x93\xac',
'HangulSyllablePyum': '\xed\x93\xb8',
'HangulSyllablePyul': '\xed\x93\xb0',
'HangulSyllablePyuk': '\xed\x94\x80',
'HangulSyllableKweos': '\xed\x80\x8f',
'PhagspaLetterCha': '\xea\xa1\x85',
'HangulSyllablePyuh': '\xed\x94\x83',
'HangulSyllableMot': '\xeb\xab\x81',
'OlChikiLetterUc': '\xe1\xb1\xaa',
'UpwardsArrowWithHorizontalStroke': '\xe2\xa4\x89',
'HangulSyllableMop': '\xeb\xab\x82',
'OlChikiLetterUd': '\xe1\xb1\xab',
'HangulSyllableMos': '\xeb\xaa\xbb',
'IdeographicTelegraphSymbolForDayTwentythree': '\xe3\x8f\xb6',
'HebrewLetterKafWithRafe': '\xef\xad\x8d',
'HangulSyllableDeot': '\xeb\x8d\xad',
'HangulSyllableDeop': '\xeb\x8d\xae',
'HangulSyllableDeos': '\xeb\x8d\xa7',
'HangulSyllableDeom': '\xeb\x8d\xa4',
'HangulSyllableDeol': '\xeb\x8d\x9c',
'HangulSyllableDeon': '\xeb\x8d\x98',
'HangulSyllableDeoh': '\xeb\x8d\xaf',
'HangulSyllableDeok': '\xeb\x8d\xac',
'CjkUnifiedIdeograph6a26': '\xe6\xa8\xa6',
'CjkUnifiedIdeograph847f': '\xe8\x91\xbf',
'CjkUnifiedIdeograph847e': '\xe8\x91\xbe',
'CjkUnifiedIdeograph847d': '\xe8\x91\xbd',
'CjkUnifiedIdeograph847c': '\xe8\x91\xbc',
'CjkUnifiedIdeograph6a25': '\xe6\xa8\xa5',
'CjkUnifiedIdeograph847a': '\xe8\x91\xba',
'HangulSyllableDeob': '\xeb\x8d\xa5',
'CjkUnifiedIdeograph6a24': '\xe6\xa8\xa4',
'UnicodeCharacter0xf745': '\xef\x9d\x85',
'SundaneseLetterXa': '\xe1\xae\x9f',
'CjkUnifiedIdeograph6a23': '\xe6\xa8\xa3',
'ElementOf': '\xe2\x88\x88',
'CjkUnifiedIdeograph6a22': '\xe6\xa8\xa2',
'UnicodeCharacter0x12b7': '\xe1\x8a\xb7',
'UnicodeCharacter0x12b6': '\xe1\x8a\xb6',
'CjkUnifiedIdeograph5c4a': '\xe5\xb1\x8a',
'CjkUnifiedIdeograph6a21': '\xe6\xa8\xa1',
'CyrillicSmallLetterIotifiedLittleYus': '\xd1\xa9',
'CjkUnifiedIdeograph8efa': '\xe8\xbb\xba',
'CjkUnifiedIdeograph8efc': '\xe8\xbb\xbc',
'CjkUnifiedIdeograph6a20': '\xe6\xa8\xa0',
'CjkUnifiedIdeograph8efe': '\xe8\xbb\xbe',
'CjkUnifiedIdeograph8efd': '\xe8\xbb\xbd',
'CyrillicCapitalLetterTshe': '\xd0\x8b',
'CjkUnifiedIdeograph8eff': '\xe8\xbb\xbf',
'LatinSmallLetterUWithCircumflex': '\xc3\xbb',
'CopticCapitalLetterThethe': '\xe2\xb2\x90',
'UnicodeCharacter0x5c9': '\xd7\x89',
'UnicodeCharacter0x5c8': '\xd7\x88',
'CanadianSyllabicsWestcreeM': '\xe1\x92\xbc',
'CanadianSyllabicsWestcreeL': '\xe1\x93\xab',
'UnicodeCharacter0xe136': '\xee\x84\xb6',
'GreekSmallLetterPamphylianDigamma': '\xcd\xb7',
'CanadianSyllabicsWestcreeP': '\xe1\x91\x8a',
'CjkCompatibilityIdeographf9d0': '\xef\xa7\x90',
'CanadianSyllabicsWestcreeR': '\xe1\x95\x91',
'CopticSmallLetterShei': '\xcf\xa3',
'CanadianSyllabicsWestcreeY': '\xe1\x95\x80',
'GuaraniSign': '\xe2\x82\xb2',
'UnicodeCharacter0xf57f': '\xef\x95\xbf',
'UnicodeCharacter0xf747': '\xef\x9d\x87',
'SaurashtraVowelSignAu': '\xea\xa3\x83',
'LatinSmallLetterDWithTail': '\xc9\x96',
'UnicodeCharacter0xe135': '\xee\x84\xb5',
'BalineseCarikPareren': '\xe1\xad\x9f',
'VaiSyllableSe': '\xea\x97\xbb',
'VaiSyllableSa': '\xea\x95\xa2',
'SaurashtraVowelSignAi': '\xea\xa3\x80',
'CanadianSyllabicsYwaa': '\xe1\x94\xbb',
'VaiSyllableSo': '\xea\x97\x93',
'VaiSyllableSi': '\xea\x94\xbb',
'CyrillicCapitalLetterEWithDiaeresis': '\xd3\xac',
'HangulSyllableSswalg': '\xec\x8f\xbd',
'CjkUnifiedIdeograph9025': '\xe9\x80\xa5',
'HangulSyllableColh': '\xec\xb4\x97',
'UnicodeCharacter0x5cf': '\xd7\x8f',
'UnicodeCharacter0x5ce': '\xd7\x8e',
'UnicodeCharacter0x5cd': '\xd7\x8d',
'UnicodeCharacter0x5cc': '\xd7\x8c',
'UnicodeCharacter0x5cb': '\xd7\x8b',
'UnicodeCharacter0x5ca': '\xd7\x8a',
'HangulSyllableHeung': '\xed\x9d\xa5',
'bullet': '\xe2\x88\x99',
'TeluguLetterHa': '\xe0\xb0\xb9',
'pipi': '\xe2\x84\xbc',
'ThaiDigitFour': '\xe0\xb9\x94',
'HangulSyllableJja': '\xec\xa7\x9c',
'UnicodeCharacter0xe3d4': '\xee\x8f\x94',
'UnicodeCharacter0xe5c5': '\xee\x97\x85',
'HangulSyllableColb': '\xec\xb4\x93',
'UnicodeCharacter0xe3d6': '\xee\x8f\x96',
'CjkUnifiedIdeograph5948': '\xe5\xa5\x88',
'HangulSyllableBwelg': '\xeb\xb7\x81',
'HangulSyllableBwelb': '\xeb\xb7\x83',
'CjkUnifiedIdeograph5949': '\xe5\xa5\x89',
'CjkUnifiedIdeograph5247': '\xe5\x89\x87',
'UnicodeCharacter0xa6fe': '\xea\x9b\xbe',
'CjkUnifiedIdeograph6061': '\xe6\x81\xa1',
'UnicodeCharacter0xa6fc': '\xea\x9b\xbc',
'UnicodeCharacter0xa6fb': '\xea\x9b\xbb',
'CjkUnifiedIdeograph9023': '\xe9\x80\xa3',
'GreekSmallLetterEtaWithOxia': '\xe1\xbd\xb5',
'HangulSyllableBwelt': '\xeb\xb7\x85',
'UnicodeCharacter0xea50': '\xee\xa9\x90',
'HangulSyllableBwelp': '\xeb\xb7\x86',
'HangulSyllableBwaeng': '\xeb\xb5\x81',
'HangulSyllableBwels': '\xeb\xb7\x84',
'VaiSyllableMoo': '\xea\x96\x92',
'CjkUnifiedIdeograph9300': '\xe9\x8c\x80',
'CjkUnifiedIdeograph867a': '\xe8\x99\xba',
'UnicodeCharacter0xe139': '\xee\x84\xb9',
'UnicodeCharacter0xe878': '\xee\xa1\xb8',
'LatinSmallLetterLongSWithHighStroke': '\xe1\xba\x9d',
'HangulChoseongCeongchieumssangcieuc': '\xe1\x85\x91',
'UnicodeCharacter0xe877': '\xee\xa1\xb7',
'HangulSyllableDweogg': '\xeb\x91\xae',
'UnicodeCharacter0xe875': '\xee\xa1\xb5',
'CjkUnifiedIdeograph9022': '\xe9\x80\xa2',
'UnicodeCharacter0xe873': '\xee\xa1\xb3',
'ThaiCharacterMaiTri': '\xe0\xb9\x8a',
'UnicodeCharacter0xe871': '\xee\xa1\xb1',
'CjkUnifiedIdeograph6b7f': '\xe6\xad\xbf',
'CjkUnifiedIdeograph6b7e': '\xe6\xad\xbe',
'CjkUnifiedIdeograph6b7d': '\xe6\xad\xbd',
'CjkUnifiedIdeograph6b7c': '\xe6\xad\xbc',
'CjkUnifiedIdeograph6068': '\xe6\x81\xa8',
'CjkUnifiedIdeograph6b7a': '\xe6\xad\xba',
'CjkUnifiedIdeograph9301': '\xe9\x8c\x81',
'HangulSyllableYass': '\xec\x96\x90',
'HangulSyllableDweogs': '\xeb\x91\xaf',
'HangulSyllableGgya': '\xea\xba\x84',
'HangulSyllableDyinh': '\xeb\x93\xbe',
'CircledIdeographSpecial': '\xe3\x8a\x95',
'CjkUnifiedIdeograph5946': '\xe5\xa5\x86',
'CjkUnifiedIdeograph9306': '\xe9\x8c\x86',
'UnicodeCharacter0xa6f8': '\xea\x9b\xb8',
'UnicodeCharacter0xa6f7': '\xea\x9b\xb7',
'UnicodeCharacter0xa6f6': '\xea\x9b\xb6',
'UnicodeCharacter0xa6f5': '\xea\x9b\xb5',
'CjkUnifiedIdeograph5947': '\xe5\xa5\x87',
'UnicodeCharacter0xa6f3': '\xea\x9b\xb3',
'UnicodeCharacter0xa6f2': '\xea\x9b\xb2',
'BoxDrawingsRightLightAndLeftVerticalHeavy': '\xe2\x95\x89',
'DingbatNegativeCircledSansserifDigitSeven': '\xe2\x9e\x90',
'CjkUnifiedIdeograph6b77': '\xe6\xad\xb7',
'CjkUnifiedIdeograph6b76': '\xe6\xad\xb6',
'CjkUnifiedIdeograph6b75': '\xe6\xad\xb5',
'CjkUnifiedIdeograph6b74': '\xe6\xad\xb4',
'CjkUnifiedIdeograph6b73': '\xe6\xad\xb3',
'CjkUnifiedIdeograph6b72': '\xe6\xad\xb2',
'CjkUnifiedIdeograph6b71': '\xe6\xad\xb1',
'CjkUnifiedIdeograph6b70': '\xe6\xad\xb0',
'GG': '\xe2\x85\x81',
'HangulSyllableSsyulg': '\xec\x93\x9d',
'YiSyllableNdu': '\xea\x85\xa4',
'CjkUnifiedIdeograph6b79': '\xe6\xad\xb9',
'CjkUnifiedIdeograph5f95': '\xe5\xbe\x95',
'YiSyllableNdi': '\xea\x85\x93',
'CjkUnifiedIdeograph9307': '\xe9\x8c\x87',
'YiSyllableNdo': '\xea\x85\x9d',
'YiSyllableNda': '\xea\x85\x99',
'UnicodeCharacter0xe87d': '\xee\xa1\xbd',
'UnicodeCharacter0xe87e': '\xee\xa1\xbe',
'YiSyllableNde': '\xea\x85\xa0',
'UnicodeCharacter0xe87c': '\xee\xa1\xbc',
'UnicodeCharacter0xe87a': '\xee\xa1\xba',
'CjkRadicalNetFour': '\xe2\xba\xb4',
'CanadianSyllabicsLaa': '\xe1\x93\x9b',
'UnicodeCharacter0xffd0': '\xef\xbf\x90',
'UnicodeCharacter0xeede': '\xee\xbb\x9e',
'UnicodeCharacter0xe5c4': '\xee\x97\x84',
'UnicodeCharacter0xd839': '\xed\xa0\xb9',
'UnicodeCharacter0xd838': '\xed\xa0\xb8',
'KangxiRadicalLack': '\xe2\xbd\x8b',
'UnicodeCharacter0xd830': '\xed\xa0\xb0',
'OghamLetterStraif': '\xe1\x9a\x8e',
'UnicodeCharacter0xd832': '\xed\xa0\xb2',
'UnicodeCharacter0xd835': '\xed\xa0\xb5',
'UnicodeCharacter0xd834': '\xed\xa0\xb4',
'UnicodeCharacter0xd837': '\xed\xa0\xb7',
'UnicodeCharacter0xd836': '\xed\xa0\xb6',
'CjkUnifiedIdeograph93ea': '\xe9\x8f\xaa',
'YiSyllableBbuox': '\xea\x81\xa2',
'CjkUnifiedIdeograph93ec': '\xe9\x8f\xac',
'CjkUnifiedIdeograph93eb': '\xe9\x8f\xab',
'CjkUnifiedIdeograph93ee': '\xe9\x8f\xae',
'CjkUnifiedIdeograph93ed': '\xe9\x8f\xad',
'CjkUnifiedIdeograph93ef': '\xe9\x8f\xaf',
'YiSyllableBbuop': '\xea\x81\xa4',
'CjkUnifiedIdeograph9305': '\xe9\x8c\x85',
'HangulSyllableGyenh': '\xea\xb3\x8a',
'ArabicLetterAlefWithMaddaAbove': '\xd8\xa2',
'HangulSyllableGyenj': '\xea\xb3\x89',
'SquareAruhua': '\xe3\x8c\x81',
'MalayalamLetterNa': '\xe0\xb4\xa8',
'HangulSyllableGyeng': '\xea\xb3\x99',
'EthiopicSyllableQya': '\xe2\xb7\x80',
'LatinCapitalLetterTWithCedilla': '\xc5\xa2',
'UnicodeCharacter0xd83a': '\xed\xa0\xba',
'UnicodeCharacter0xd83c': '\xed\xa0\xbc',
'UnicodeCharacter0x1400': '\xe1\x90\x80',
'UnicodeCharacter0xd83e': '\xed\xa0\xbe',
'UnicodeCharacter0xd83d': '\xed\xa0\xbd',
'UnicodeCharacter0xd83f': '\xed\xa0\xbf',
'UnicodeCharacter0xf356': '\xef\x8d\x96',
'HangulSyllableRwelm': '\xeb\xa4\xaa',
'HangulSyllableRyagg': '\xeb\x9e\xb6',
'CjkUnifiedIdeograph93e1': '\xe9\x8f\xa1',
'CjkUnifiedIdeograph93e0': '\xe9\x8f\xa0',
'CjkUnifiedIdeograph93e3': '\xe9\x8f\xa3',
'CjkUnifiedIdeograph93e2': '\xe9\x8f\xa2',
'CjkUnifiedIdeograph93e5': '\xe9\x8f\xa5',
'CjkUnifiedIdeograph93e4': '\xe9\x8f\xa4',
'CjkUnifiedIdeograph93e7': '\xe9\x8f\xa7',
'CjkUnifiedIdeograph93e6': '\xe9\x8f\xa6',
'CjkUnifiedIdeograph93e9': '\xe9\x8f\xa9',
'CjkUnifiedIdeograph93e8': '\xe9\x8f\xa8',
'deg': '\xc2\xb0',
'HangulSyllableTyunj': '\xed\x8a\xa1',
'UnicodeCharacter0xefaa': '\xee\xbe\xaa',
'HangulSyllableTyunh': '\xed\x8a\xa2',
'CjkUnifiedIdeograph5d01': '\xe5\xb4\x81',
'CjkUnifiedIdeograph5d00': '\xe5\xb4\x80',
'UnicodeCharacter0x1ad8': '\xe1\xab\x98',
'UnicodeCharacter0x1ad9': '\xe1\xab\x99',
'CjkUnifiedIdeograph5d05': '\xe5\xb4\x85',
'CjkUnifiedIdeograph5d04': '\xe5\xb4\x84',
'LatinCapitalLetterEWithCircumflexAndGrave': '\xe1\xbb\x80',
'CjkUnifiedIdeograph5d06': '\xe5\xb4\x86',
'UnicodeCharacter0x1ad2': '\xe1\xab\x92',
'UnicodeCharacter0x1ad3': '\xe1\xab\x93',
'UnicodeCharacter0x1ad0': '\xe1\xab\x90',
'UnicodeCharacter0x1ad1': '\xe1\xab\x91',
'UnicodeCharacter0x1ad6': '\xe1\xab\x96',
'UnicodeCharacter0x1ad7': '\xe1\xab\x97',
'UnicodeCharacter0x1ad4': '\xe1\xab\x94',
'UnicodeCharacter0x1ad5': '\xe1\xab\x95',
'LeftRightArrow': '\xe2\x86\x94',
'MathematicalRightWhiteSquareBracket': '\xe2\x9f\xa7',
'UnicodeCharacter0xe2be': '\xee\x8a\xbe',
'CjkUnifiedIdeograph85b9': '\xe8\x96\xb9',
'CjkUnifiedIdeograph85b8': '\xe8\x96\xb8',
'UnicodeCharacter0xfbcd': '\xef\xaf\x8d',
'CjkUnifiedIdeograph85b5': '\xe8\x96\xb5',
'CjkUnifiedIdeograph85b4': '\xe8\x96\xb4',
'CjkUnifiedIdeograph85b7': '\xe8\x96\xb7',
'CjkUnifiedIdeograph85b6': '\xe8\x96\xb6',
'CjkUnifiedIdeograph85b1': '\xe8\x96\xb1',
'CjkUnifiedIdeograph85b0': '\xe8\x96\xb0',
'CjkUnifiedIdeograph85b3': '\xe8\x96\xb3',
'CjkUnifiedIdeograph85b2': '\xe8\x96\xb2',
'MinusSignWithDotBelow': '\xe2\xa8\xaa',
'UnicodeCharacter0xf58d': '\xef\x96\x8d',
'IdeographicTelegraphSymbolForHourFour': '\xe3\x8d\x9c',
'CombiningThreeDotsAbove': '\xe2\x83\x9b',
'UnicodeCharacter0xfbcb': '\xef\xaf\x8b',
'HebrewLetterPeWithDagesh': '\xef\xad\x84',
'CjkUnifiedIdeograph5d0a': '\xe5\xb4\x8a',
'CjkUnifiedIdeograph9a89': '\xe9\xaa\x89',
'CjkUnifiedIdeograph5d0c': '\xe5\xb4\x8c',
'CjkUnifiedIdeograph5d0b': '\xe5\xb4\x8b',
'CjkUnifiedIdeograph5d0e': '\xe5\xb4\x8e',
'CjkUnifiedIdeograph5d0d': '\xe5\xb4\x8d',
'UnicodeCharacter0xf50e': '\xef\x94\x8e',
'CjkUnifiedIdeograph5d0f': '\xe5\xb4\x8f',
'UnicodeCharacter0x1adb': '\xe1\xab\x9b',
'UnicodeCharacter0x1adc': '\xe1\xab\x9c',
'CjkUnifiedIdeograph9a82': '\xe9\xaa\x82',
'UnicodeCharacter0x1ada': '\xe1\xab\x9a',
'UnicodeCharacter0x1adf': '\xe1\xab\x9f',
'CjkUnifiedIdeograph9a85': '\xe9\xaa\x85',
'UnicodeCharacter0x1add': '\xe1\xab\x9d',
'UnicodeCharacter0x1ade': '\xe1\xab\x9e',
'CjkUnifiedIdeograph6390': '\xe6\x8e\x90',
'GreekSmallLetterAlphaWithVariaAndYpogegrammeni': '\xe1\xbe\xb2',
'HangulSyllableGwinh': '\xea\xb7\x86',
'CjkUnifiedIdeograph6392': '\xe6\x8e\x92',
'UnicodeCharacter0xfde9': '\xef\xb7\xa9',
'UnicodeCharacter0xfde8': '\xef\xb7\xa8',
'CjkUnifiedIdeograph6393': '\xe6\x8e\x93',
'UnicodeCharacter0xf50d': '\xef\x94\x8d',
'CjkUnifiedIdeograph85be': '\xe8\x96\xbe',
'CjkUnifiedIdeograph6394': '\xe6\x8e\x94',
'UnicodeCharacter0xfde3': '\xef\xb7\xa3',
'CjkUnifiedIdeograph85bf': '\xe8\x96\xbf',
'CjkUnifiedIdeograph85ba': '\xe8\x96\xba',
'CjkUnifiedIdeograph5f9b': '\xe5\xbe\x9b',
'CjkUnifiedIdeograph85bc': '\xe8\x96\xbc',
'CjkUnifiedIdeograph6395': '\xe6\x8e\x95',
'HangulSyllableBeolm': '\xeb\xb2\x8e',
'HalfwidthKatakanaLetterSmallYa': '\xef\xbd\xac',
'nwarrow': '\xe2\x86\x96',
'CjkUnifiedIdeograph6396': '\xe6\x8e\x96',
'UnicodeCharacter0xeed3': '\xee\xbb\x93',
'MyanmarDigitZero': '\xe1\x81\x80',
'CjkUnifiedIdeograph43d2': '\xe4\x8f\x92',
'KhmerSignToandakhiat': '\xe1\x9f\x8d',
'HangulSyllableSswalt': '\xec\x90\x81',
'CjkUnifiedIdeograph45b1': '\xe4\x96\xb1',
'CjkUnifiedIdeograph45b0': '\xe4\x96\xb0',
'CjkUnifiedIdeograph45b3': '\xe4\x96\xb3',
'CjkUnifiedIdeograph45b2': '\xe4\x96\xb2',
'CjkUnifiedIdeograph45b5': '\xe4\x96\xb5',
'CjkUnifiedIdeograph45b4': '\xe4\x96\xb4',
'LeftRightArrowThroughSmallCircle': '\xe2\xa5\x88',
'CjkUnifiedIdeograph45b6': '\xe4\x96\xb6',
'CjkUnifiedIdeograph45b9': '\xe4\x96\xb9',
'CjkUnifiedIdeograph45b8': '\xe4\x96\xb8',
'CjkUnifiedIdeograph9542': '\xe9\x95\x82',
'LatinSmallLetterLongSWithDiagonalStroke': '\xe1\xba\x9c',
'UnicodeCharacter0xd908': '\xed\xa4\x88',
'IntersectionWithOverbar': '\xe2\xa9\x83',
'UnicodeCharacter0xf351': '\xef\x8d\x91',
'CjkUnifiedIdeograph9b3b': '\xe9\xac\xbb',
'RightCurlyBracketLowerHook': '\xe2\x8e\xad',
'CjkUnifiedIdeograph7908': '\xe7\xa4\x88',
'CjkUnifiedIdeograph7909': '\xe7\xa4\x89',
'CjkUnifiedIdeograph9b3f': '\xe9\xac\xbf',
'UnicodeCharacter0xdaff': '\xed\xab\xbf',
'CjkUnifiedIdeograph9b3d': '\xe9\xac\xbd',
'CjkUnifiedIdeograph9b3e': '\xe9\xac\xbe',
'CjkUnifiedIdeograph7902': '\xe7\xa4\x82',
'CjkUnifiedIdeograph7903': '\xe7\xa4\x83',
'CjkUnifiedIdeograph7900': '\xe7\xa4\x80',
'CjkUnifiedIdeograph7901': '\xe7\xa4\x81',
'CjkUnifiedIdeograph7906': '\xe7\xa4\x86',
'CjkUnifiedIdeograph7907': '\xe7\xa4\x87',
'MongolianDigitThree': '\xe1\xa0\x93',
'CjkUnifiedIdeograph7905': '\xe7\xa4\x85',
'CjkUnifiedIdeograph45ba': '\xe4\x96\xba',
'CjkUnifiedIdeograph9bb8': '\xe9\xae\xb8',
'CjkUnifiedIdeograph45bc': '\xe4\x96\xbc',
'CjkUnifiedIdeograph45bb': '\xe4\x96\xbb',
'CjkUnifiedIdeograph45be': '\xe4\x96\xbe',
'CjkUnifiedIdeograph45bd': '\xe4\x96\xbd',
'CjkUnifiedIdeograph45bf': '\xe4\x96\xbf',
'HangulSyllableHugs': '\xed\x9b\x87',
'UnicodeCharacter0xf50a': '\xef\x94\x8a',
'CjkUnifiedIdeograph789e': '\xe7\xa2\x9e',
'UnicodeCharacter0xe3dd': '\xee\x8f\x9d',
'UnicodeCharacter0xd98d': '\xed\xa6\x8d',
'HalfwidthKatakanaLetterSmallYo': '\xef\xbd\xae',
'HangulSyllableHep': '\xed\x97\xbe',
'UnicodeCharacter0xe3de': '\xee\x8f\x9e',
'UnicodeCharacter0xf220': '\xef\x88\xa0',
'UnicodeCharacter0xe3df': '\xee\x8f\x9f',
'HangulSyllableHugg': '\xed\x9b\x86',
'CjkUnifiedIdeograph9b32': '\xe9\xac\xb2',
'CjkUnifiedIdeograph9b33': '\xe9\xac\xb3',
'CjkUnifiedIdeograph9b30': '\xe9\xac\xb0',
'Ceres': '\xe2\x9a\xb3',
'CjkUnifiedIdeograph9b36': '\xe9\xac\xb6',
'CjkUnifiedIdeograph9b37': '\xe9\xac\xb7',
'CjkUnifiedIdeograph9b34': '\xe9\xac\xb4',
'CjkUnifiedIdeograph9b35': '\xe9\xac\xb5',
'CjkUnifiedIdeograph401a': '\xe4\x80\x9a',
'CjkUnifiedIdeograph790c': '\xe7\xa4\x8c',
'CjkUnifiedIdeograph9b38': '\xe9\xac\xb8',
'CjkUnifiedIdeograph790a': '\xe7\xa4\x8a',
'CjkUnifiedIdeograph790f': '\xe7\xa4\x8f',
'CjkUnifiedIdeograph790d': '\xe7\xa4\x8d',
'CjkUnifiedIdeograph790e': '\xe7\xa4\x8e',
'HexagramForConflict': '\xe4\xb7\x85',
'CjkUnifiedIdeograph9545': '\xe9\x95\x85',
'GreekSmallLetterEpsilonWithDasiaAndVaria': '\xe1\xbc\x93',
'SquaredDotOperator': '\xe2\x8a\xa1',
'HalfwidthKatakanaLetterSmallYu': '\xef\xbd\xad',
'UnicodeCharacter0xd905': '\xed\xa4\x85',
'HangulSyllableDags': '\xeb\x8b\xa7',
'HangulSyllableSsyaebs': '\xec\x8d\x9e',
'UnicodeCharacter0x2bff': '\xe2\xaf\xbf',
'GujaratiLetterDda': '\xe0\xaa\xa1',
'GreekCapitalLetterAlphaWithDasiaAndPerispomeniAndProsgegrammeni': '\xe1\xbe\x8f',
'CjkUnifiedIdeograph9546': '\xe9\x95\x86',
'HangulSyllableDdalh': '\xeb\x94\xbf',
'CjkUnifiedIdeograph69a8': '\xe6\xa6\xa8',
'CjkUnifiedIdeograph69a9': '\xe6\xa6\xa9',
'CjkUnifiedIdeograph69a6': '\xe6\xa6\xa6',
'CjkUnifiedIdeograph69a7': '\xe6\xa6\xa7',
'CjkUnifiedIdeograph69a4': '\xe6\xa6\xa4',
'CjkUnifiedIdeograph69a5': '\xe6\xa6\xa5',
'CjkUnifiedIdeograph69a2': '\xe6\xa6\xa2',
'GreekCapitalLetterOmegaWithDasiaAndProsgegrammeni': '\xe1\xbe\xa9',
'CjkUnifiedIdeograph69a0': '\xe6\xa6\xa0',
'KangxiRadicalWrap': '\xe2\xbc\x93',
'HangulSyllableBbuss': '\xeb\xbf\xa0',
'HangulSyllableSswals': '\xec\x90\x80',
'UnicodeCharacter0xd98a': '\xed\xa6\x8a',
'CjkUnifiedIdeograph9547': '\xe9\x95\x87',
'CjkUnifiedIdeograph5418': '\xe5\x90\x98',
'CjkUnifiedIdeograph5419': '\xe5\x90\x99',
'CjkUnifiedIdeograph5416': '\xe5\x90\x96',
'CjkUnifiedIdeograph5417': '\xe5\x90\x97',
'CjkUnifiedIdeograph5414': '\xe5\x90\x94',
'CjkUnifiedIdeograph5415': '\xe5\x90\x95',
'CjkUnifiedIdeograph5412': '\xe5\x90\x92',
'CjkUnifiedIdeograph5413': '\xe5\x90\x93',
'CjkUnifiedIdeograph5410': '\xe5\x90\x90',
'CjkUnifiedIdeograph5411': '\xe5\x90\x91',
'CopticSmallLetterUa': '\xe2\xb2\xa9',
'UnicodeCharacter0xfbc4': '\xef\xaf\x84',
'CjkUnifiedIdeograph69af': '\xe6\xa6\xaf',
'CjkUnifiedIdeograph69ad': '\xe6\xa6\xad',
'CjkUnifiedIdeograph69ae': '\xe6\xa6\xae',
'CjkUnifiedIdeograph69ab': '\xe6\xa6\xab',
'CjkUnifiedIdeograph69ac': '\xe6\xa6\xac',
'CjkUnifiedIdeograph7df0': '\xe7\xb7\xb0',
'CjkUnifiedIdeograph69aa': '\xe6\xa6\xaa',
'HangulSyllablePoeh': '\xed\x91\x9b',
'HangulSyllableDdwaegs': '\xeb\x99\x8b',
'UnicodeCharacter0xd93c': '\xed\xa4\xbc',
'ArabicLetterGhainMedialForm': '\xef\xbb\x90',
'UnicodeCharacter0xfbc0': '\xef\xaf\x80',
'HangulSyllableYegs': '\xec\x98\x8b',
'ArmenianCapitalLetterCa': '\xd4\xbe',
'UnicodeCharacter0xfbc3': '\xef\xaf\x83',
'CjkUnifiedIdeograph541f': '\xe5\x90\x9f',
'ArmenianCapitalLetterCo': '\xd5\x91',
'CjkUnifiedIdeograph541e': '\xe5\x90\x9e',
'CjkUnifiedIdeograph541b': '\xe5\x90\x9b',
'CjkUnifiedIdeograph541c': '\xe5\x90\x9c',
'CjkUnifiedIdeograph541a': '\xe5\x90\x9a',
'KannadaVowelSignAa': '\xe0\xb2\xbe',
'UnicodeCharacter0xd901': '\xed\xa4\x81',
'KannadaVowelSignAi': '\xe0\xb3\x88',
'elem': '\xe2\x88\x88',
'UnicodeCharacter0xe989': '\xee\xa6\x89',
'CjkUnifiedIdeograph956b': '\xe9\x95\xab',
'HangulSyllableDyat': '\xeb\x8c\xb5',
'HangulSyllableDyas': '\xeb\x8c\xaf',
'CjkUnifiedIdeograph639a': '\xe6\x8e\x9a',
'UnicodeCharacter0xf314': '\xef\x8c\x94',
'HangulSyllableDyag': '\xeb\x8c\x9d',
'HangulSyllableDdals': '\xeb\x94\xbc',
'HangulSyllableDyae': '\xeb\x8c\xb8',
'CjkUnifiedIdeograph639c': '\xe6\x8e\x9c',
'CjkUnifiedIdeograph6299': '\xe6\x8a\x99',
'CjkUnifiedIdeograph6298': '\xe6\x8a\x98',
'CjkUnifiedIdeograph6295': '\xe6\x8a\x95',
'CjkUnifiedIdeograph6294': '\xe6\x8a\x94',
'CjkUnifiedIdeograph6297': '\xe6\x8a\x97',
'CjkUnifiedIdeograph6296': '\xe6\x8a\x96',
'CjkUnifiedIdeograph6291': '\xe6\x8a\x91',
'CjkUnifiedIdeograph6290': '\xe6\x8a\x90',
'CjkUnifiedIdeograph6293': '\xe6\x8a\x93',
'CjkUnifiedIdeograph6292': '\xe6\x8a\x92',
'CanadianSyllabicsKwaa': '\xe1\x92\x80',
'KangxiRadicalYellow': '\xe2\xbf\x88',
'EthiopicSyllableGo': '\xe1\x8c\x8e',
'CjkUnifiedIdeograph9caa': '\xe9\xb2\xaa',
'EthiopicSyllableGi': '\xe1\x8c\x8a',
'CjkUnifiedIdeograph9cac': '\xe9\xb2\xac',
'CjkUnifiedIdeograph9cab': '\xe9\xb2\xab',
'EthiopicSyllableGe': '\xe1\x8c\x8d',
'CjkUnifiedIdeograph956e': '\xe9\x95\xae',
'EthiopicSyllableGa': '\xe1\x8c\x88',
'CjkUnifiedIdeograph442d': '\xe4\x90\xad',
'CjkUnifiedIdeograph442e': '\xe4\x90\xae',
'CjkUnifiedIdeograph442f': '\xe4\x90\xaf',
'CjkUnifiedIdeograph442a': '\xe4\x90\xaa',
'CjkUnifiedIdeograph442b': '\xe4\x90\xab',
'CjkUnifiedIdeograph442c': '\xe4\x90\xac',
'EthiopicSyllableGu': '\xe1\x8c\x89',
'CjkUnifiedIdeograph9d46': '\xe9\xb5\x86',
'HangulSyllableBboej': '\xeb\xbe\xaa',
'CjkUnifiedIdeograph8472': '\xe8\x91\xb2',
'UnicodeCharacter0xe6b9': '\xee\x9a\xb9',
'UnicodeCharacter0xe6b8': '\xee\x9a\xb8',
'CjkUnifiedIdeograph629e': '\xe6\x8a\x9e',
'CjkUnifiedIdeograph629d': '\xe6\x8a\x9d',
'UnicodeCharacter0xe6b5': '\xee\x9a\xb5',
'CjkUnifiedIdeograph629f': '\xe6\x8a\x9f',
'CjkUnifiedIdeograph629a': '\xe6\x8a\x9a',
'CjkUnifiedIdeograph8502': '\xe8\x94\x82',
'CjkUnifiedIdeograph629c': '\xe6\x8a\x9c',
'CjkUnifiedIdeograph629b': '\xe6\x8a\x9b',
'ArmenianCapitalLetterReh': '\xd5\x90',
'HangulSyllableKyeolg': '\xec\xbc\xa5',
'VaiSyllableNgan': '\xea\x95\x8b',
'CanadianSyllabicsT': '\xe1\x91\xa6',
'HangulSyllableBboek': '\xeb\xbe\xac',
'CjkUnifiedIdeograph4424': '\xe4\x90\xa4',
'CjkUnifiedIdeograph4425': '\xe4\x90\xa5',
'CjkUnifiedIdeograph4426': '\xe4\x90\xa6',
'CjkUnifiedIdeograph4427': '\xe4\x90\xa7',
'CjkUnifiedIdeograph4420': '\xe4\x90\xa0',
'CjkUnifiedIdeograph4421': '\xe4\x90\xa1',
'CjkUnifiedIdeograph4422': '\xe4\x90\xa2',
'CjkUnifiedIdeograph4423': '\xe4\x90\xa3',
'HangulSyllableGyaegg': '\xea\xb1\x96',
'CjkUnifiedIdeograph4428': '\xe4\x90\xa8',
'CjkUnifiedIdeograph4429': '\xe4\x90\xa9',
'CjkUnifiedIdeograph9ca5': '\xe9\xb2\xa5',
'CjkUnifiedIdeograph9ca4': '\xe9\xb2\xa4',
'CjkUnifiedIdeograph9ca7': '\xe9\xb2\xa7',
'CjkUnifiedIdeograph9ca6': '\xe9\xb2\xa6',
'CjkUnifiedIdeograph9ca1': '\xe9\xb2\xa1',
'CjkUnifiedIdeograph9ca0': '\xe9\xb2\xa0',
'CjkUnifiedIdeograph9ca3': '\xe9\xb2\xa3',
'CjkUnifiedIdeograph9ca2': '\xe9\xb2\xa2',
'HangulSyllableBboeh': '\xeb\xbe\xaf',
'CjkUnifiedIdeograph9ca9': '\xe9\xb2\xa9',
'CjkUnifiedIdeograph9ca8': '\xe9\xb2\xa8',
'CjkUnifiedIdeograph355a': '\xe3\x95\x9a',
'UnicodeCharacter0x807': '\xe0\xa0\x87',
'UnicodeCharacter0x806': '\xe0\xa0\x86',
'UnicodeCharacter0x805': '\xe0\xa0\x85',
'UnicodeCharacter0x804': '\xe0\xa0\x84',
'UnicodeCharacter0x803': '\xe0\xa0\x83',
'UnicodeCharacter0x802': '\xe0\xa0\x82',
'UnicodeCharacter0x801': '\xe0\xa0\x81',
'UnicodeCharacter0x800': '\xe0\xa0\x80',
'HangulSyllableKwelb': '\xed\x80\xa3',
'UnicodeCharacter0x809': '\xe0\xa0\x89',
'UnicodeCharacter0x808': '\xe0\xa0\x88',
'CjkUnifiedIdeograph766a': '\xe7\x99\xaa',
'LatinLetterSmallCapitalO': '\xe1\xb4\x8f',
'LeftThreeQuartersBlock': '\xe2\x96\x8a',
'Ankh': '\xe2\x98\xa5',
'UnicodeCharacter0xfaea': '\xef\xab\xaa',
'YiSyllableNbyrx': '\xea\x82\x8f',
'HangulSyllableGyaegs': '\xea\xb1\x97',
'CjkUnifiedIdeograph3818': '\xe3\xa0\x98',
'CjkUnifiedIdeograph3819': '\xe3\xa0\x99',
'HangulSyllableBboen': '\xeb\xbe\x98',
'HangulLetterMieumpansios': '\xe3\x85\xb0',
'CjkUnifiedIdeograph3814': '\xe3\xa0\x94',
'CjkUnifiedIdeograph3815': '\xe3\xa0\x95',
'CjkUnifiedIdeograph3816': '\xe3\xa0\x96',
'CjkUnifiedIdeograph3817': '\xe3\xa0\x97',
'CjkUnifiedIdeograph3810': '\xe3\xa0\x90',
'CjkUnifiedIdeograph3811': '\xe3\xa0\x91',
'CjkUnifiedIdeograph3812': '\xe3\xa0\x92',
'CjkUnifiedIdeograph3813': '\xe3\xa0\x93',
'HangulSyllableTig': '\xed\x8b\xb1',
'UnicodeCharacter0xeff8': '\xee\xbf\xb8',
'IdeographicAnnotationTopMark': '\xe3\x86\x96',
'WhiteSquareWithCentreVerticalLine': '\xe2\x8e\x85',
'HangulSyllableBolt': '\xeb\xb4\x81',
'HangulSyllableBolp': '\xeb\xb4\x82',
'HangulSyllableBols': '\xeb\xb4\x80',
'YiSyllableViet': '\xea\x83\xa6',
'UnicodeCharacter0x80f': '\xe0\xa0\x8f',
'UnicodeCharacter0x80e': '\xe0\xa0\x8e',
'UnicodeCharacter0x80d': '\xe0\xa0\x8d',
'UnicodeCharacter0x80c': '\xe0\xa0\x8c',
'UnicodeCharacter0x80b': '\xe0\xa0\x8b',
'UnicodeCharacter0x80a': '\xe0\xa0\x8a',
'UnicodeCharacter0xd94e': '\xed\xa5\x8e',
'HangulSyllableBolg': '\xeb\xb3\xbd',
'YiSyllableViex': '\xea\x83\xa7',
'HangulSyllableBolb': '\xeb\xb3\xbf',
'UnicodeCharacter0xd94d': '\xed\xa5\x8d',
'SquaredAsterisk': '\xe2\xa7\x86',
'YiSyllableNyo': '\xea\x91\x97',
'ArabicLetterHahWithThreeDotsAbove': '\xda\x85',
'CjkUnifiedIdeograph381d': '\xe3\xa0\x9d',
'CjkUnifiedIdeograph381e': '\xe3\xa0\x9e',
'CjkUnifiedIdeograph381f': '\xe3\xa0\x9f',
'UnicodeCharacter0xd94f': '\xed\xa5\x8f',
'UnicodeCharacter0xec2b': '\xee\xb0\xab',
'CjkUnifiedIdeograph381a': '\xe3\xa0\x9a',
'CjkUnifiedIdeograph381b': '\xe3\xa0\x9b',
'CjkUnifiedIdeograph381c': '\xe3\xa0\x9c',
'HangulSyllableBboel': '\xeb\xbe\x9c',
'r': '\xcf\x81',
'HangulSyllableJyabs': '\xec\x9f\x9a',
'YiSyllableNyu': '\xea\x91\x9b',
'LatinCapitalLetterDWithDotAbove': '\xe1\xb8\x8a',
'and': '\xe2\x88\xa7',
'CyrillicSmallLetterZemlya': '\xea\x99\x81',
'LimbuSmallLetterMa': '\xe1\xa4\xb6',
'HangulSyllableTij': '\xed\x8c\x86',
'HangulSyllableBboem': '\xeb\xbe\xa4',
'HangulSyllableTwaebs': '\xed\x87\xaa',
'HangulSyllableGyobs': '\xea\xb5\xa2',
'HangulSyllableGek': '\xea\xb2\xa4',
'MongolianLetterUe': '\xe1\xa0\xa6',
'HangulSyllableBbwaes': '\xeb\xbe\x8b',
'UnicodeCharacter0xef6a': '\xee\xbd\xaa',
'HangulSyllableBboeb': '\xeb\xbe\xa5',
'CjkUnifiedIdeograph9d4a': '\xe9\xb5\x8a',
'ArabicLigatureDadWithJeemInitialForm': '\xef\xb2\xb4',
'VaiSyllableGboo': '\xea\x95\xbc',
'HangulSyllableGel': '\xea\xb2\x94',
'HangulSyllableJoebs': '\xec\xa3\x96',
'VaiSyllableGbon': '\xea\x97\x88',
'ArabicLigatureBehWithHahWithYehFinalForm': '\xef\xb7\x82',
'UnicodeCharacter0xd86a': '\xed\xa1\xaa',
'CjkUnifiedIdeograph465d': '\xe4\x99\x9d',
'HangulSyllableWag': '\xec\x99\x81',
'HangulSyllableBil': '\xeb\xb9\x8c',
'HangulSyllableBboec': '\xeb\xbe\xab',
'ModifierLetterLowAcuteAccent': '\xcb\x8f',
'HangulSyllableSseub': '\xec\x94\x81',
'CjkUnifiedIdeograph9d4f': '\xe9\xb5\x8f',
'HangulSyllableBbwaem': '\xeb\xbe\x88',
'HangulSyllableCyal': '\xec\xb1\xa8',
'HangulSyllableGeogs': '\xea\xb1\xb3',
'ArabicLigatureYehWithHamzaAboveWithAeIsolatedForm': '\xef\xaf\xac',
'CjkUnifiedIdeograph8cd2': '\xe8\xb3\x92',
'VaiSyllablePee': '\xea\x94\x85',
'HangulSyllableJjyaenh': '\xec\xa8\xb6',
'SupersetOfAboveTildeOperator': '\xe2\xab\x88',
'UnicodeCharacter0xdd22': '\xed\xb4\xa2',
'TibetanSubjoinedLetterSsa': '\xe0\xbe\xb5',
'YiSyllableVat': '\xea\x83\xaa',
'LowerThreeQuartersBlock': '\xe2\x96\x86',
'CjkUnifiedIdeograph8cd4': '\xe8\xb3\x94',
'CjkUnifiedIdeograph3a5e': '\xe3\xa9\x9e',
'CjkUnifiedIdeograph3a5d': '\xe3\xa9\x9d',
'CjkUnifiedIdeograph3d89': '\xe3\xb6\x89',
'CjkUnifiedIdeograph3a5f': '\xe3\xa9\x9f',
'CjkUnifiedIdeograph3a5a': '\xe3\xa9\x9a',
'UnicodeCharacter0xefa': '\xe0\xbb\xba',
'CjkUnifiedIdeograph3a5c': '\xe3\xa9\x9c',
'CjkUnifiedIdeograph3a5b': '\xe3\xa9\x9b',
'CjkUnifiedIdeograph3d83': '\xe3\xb6\x83',
'CjkUnifiedIdeograph3d82': '\xe3\xb6\x82',
'CjkUnifiedIdeograph3d81': '\xe3\xb6\x81',
'CjkUnifiedIdeograph3d80': '\xe3\xb6\x80',
'CjkUnifiedIdeograph3d87': '\xe3\xb6\x87',
'CjkUnifiedIdeograph3d86': '\xe3\xb6\x86',
'CjkUnifiedIdeograph3d85': '\xe3\xb6\x85',
'CjkUnifiedIdeograph3d84': '\xe3\xb6\x84',
'HangulSyllablePyaess': '\xed\x8d\xb4',
'LatinSmallLetterRWithFishhookAndMiddleTilde': '\xe1\xb5\xb3',
'YiSyllableVap': '\xea\x83\xad',
'CjkUnifiedIdeograph72b9': '\xe7\x8a\xb9',
'CjkUnifiedIdeograph72b8': '\xe7\x8a\xb8',
'CanadianSyllabicsWestcreeKwoo': '\xe1\x91\xbd',
'CjkUnifiedIdeograph72b6': '\xe7\x8a\xb6',
'CjkUnifiedIdeograph72b5': '\xe7\x8a\xb5',
'CjkUnifiedIdeograph72b4': '\xe7\x8a\xb4',
'CjkUnifiedIdeograph72b3': '\xe7\x8a\xb3',
'CjkUnifiedIdeograph72b2': '\xe7\x8a\xb2',
'CjkUnifiedIdeograph72b1': '\xe7\x8a\xb1',
'CjkUnifiedIdeograph72b0': '\xe7\x8a\xb0',
'TibetanLetterPha': '\xe0\xbd\x95',
'SundaneseDigitFour': '\xe1\xae\xb4',
'HangulSyllableNyung': '\xeb\x8a\x89',
'WhiteDownpointingTriangle': '\xe2\x96\xbd',
'CjkUnifiedIdeograph72bf': '\xe7\x8a\xbf',
'CjkUnifiedIdeograph72be': '\xe7\x8a\xbe',
'CjkUnifiedIdeograph72bd': '\xe7\x8a\xbd',
'CjkUnifiedIdeograph72bc': '\xe7\x8a\xbc',
'CjkUnifiedIdeograph72bb': '\xe7\x8a\xbb',
'CjkUnifiedIdeograph72ba': '\xe7\x8a\xba',
'CjkUnifiedIdeograph3a55': '\xe3\xa9\x95',
'CjkUnifiedIdeograph3a54': '\xe3\xa9\x94',
'CjkUnifiedIdeograph3a57': '\xe3\xa9\x97',
'CjkUnifiedIdeograph3a56': '\xe3\xa9\x96',
'CjkUnifiedIdeograph3a51': '\xe3\xa9\x91',
'CjkUnifiedIdeograph3a50': '\xe3\xa9\x90',
'CjkUnifiedIdeograph3a53': '\xe3\xa9\x93',
'CjkUnifiedIdeograph3a52': '\xe3\xa9\x92',
'CjkUnifiedIdeograph3d8c': '\xe3\xb6\x8c',
'CjkUnifiedIdeograph3d8b': '\xe3\xb6\x8b',
'BlackMediumDiamond': '\xe2\xac\xa5',
'varepsilon': '\xce\xb5',
'CjkUnifiedIdeograph3a59': '\xe3\xa9\x99',
'CjkUnifiedIdeograph3a58': '\xe3\xa9\x98',
'CjkUnifiedIdeograph3d8e': '\xe3\xb6\x8e',
'CjkUnifiedIdeograph3d8d': '\xe3\xb6\x8d',
'HangulSyllableBboed': '\xeb\xbe\x9b',
'UnicodeCharacter0xf02e': '\xef\x80\xae',
'BopomofoLetterIu': '\xe3\x84\xa9',
'BopomofoLetterIr': '\xe3\x86\xa8',
'HangulSyllableBip': '\xeb\xb9\x9e',
'BopomofoLetterIm': '\xe3\x86\xac',
'BopomofoLetterIh': '\xe3\x84\xad',
'HangulSyllableBis': '\xeb\xb9\x97',
'BengaliLetterTha': '\xe0\xa6\xa5',
'Gamma': '\xce\x93',
'UnicodeCharacter0x3040': '\xe3\x81\x80',
'HangulSyllableTweos': '\xed\x89\x9b',
'HangulSyllableBbyuc': '\xec\x81\x93',
'HangulLetterKhieukh': '\xe3\x85\x8b',
'UnicodeCharacter0xf74a': '\xef\x9d\x8a',
'BraillePatternDots124578': '\xe2\xa3\x9b',
'UnicodeCharacter0xe30a': '\xee\x8c\x8a',
'HangulSyllableSwags': '\xec\x86\xab',
'SaurashtraDigitFive': '\xea\xa3\x95',
'CjkUnifiedIdeograph887c': '\xe8\xa1\xbc',
'LowerOneEighthBlock': '\xe2\x96\x81',
'UnicodeCharacter0xe30c': '\xee\x8c\x8c',
'AplFunctionalSymbolQuadGreaterthan': '\xe2\x8d\x84',
'CjkUnifiedIdeograph887b': '\xe8\xa1\xbb',
'CjkUnifiedIdeograph46fb': '\xe4\x9b\xbb',
'CjkUnifiedIdeograph46fc': '\xe4\x9b\xbc',
'UnicodeCharacter0xe003': '\xee\x80\x83',
'CjkUnifiedIdeograph46fa': '\xe4\x9b\xba',
'CjkUnifiedIdeograph46ff': '\xe4\x9b\xbf',
'CjkUnifiedIdeograph46fd': '\xe4\x9b\xbd',
'CjkUnifiedIdeograph46fe': '\xe4\x9b\xbe',
'UnicodeCharacter0xe30f': '\xee\x8c\x8f',
'CjkUnifiedIdeograph7f7d': '\xe7\xbd\xbd',
'CjkUnifiedIdeograph7f7e': '\xe7\xbd\xbe',
'ParenthesizedHangulNieun': '\xe3\x88\x81',
'CopticSmallLetterOldCopticHa': '\xe2\xb3\x8f',
'CjkUnifiedIdeograph7f7b': '\xe7\xbd\xbb',
'CjkUnifiedIdeograph7f7c': '\xe7\xbd\xbc',
'BalineseLetterTa': '\xe1\xac\xa2',
'HangulSyllableTej': '\xed\x85\xa2',
'HangulSyllableTeh': '\xed\x85\xa7',
'HangulSyllableDunj': '\xeb\x91\x95',
'CjkUnifiedIdeograph7669': '\xe7\x99\xa9',
'HangulSyllableDunh': '\xeb\x91\x96',
'HangulSyllableTel': '\xed\x85\x94',
'HangulSyllableTec': '\xed\x85\xa3',
'HangulSyllableDung': '\xeb\x91\xa5',
'CjkUnifiedIdeograph7668': '\xe7\x99\xa8',
'HangulSyllableTeg': '\xed\x85\x8d',
'UnicodeCharacter0xf74c': '\xef\x9d\x8c',
'HangulSyllableTed': '\xed\x85\x93',
'HangulSyllableSeuss': '\xec\x8a\xb8',
'4rt': '\xe2\x88\x9c',
'HangulSyllableTes': '\xed\x85\x9f',
'HangulSyllableTweop': '\xed\x89\xa2',
'HangulSyllableTep': '\xed\x85\xa6',
'HangulSyllableTeu': '\xed\x8a\xb8',
'HangulSyllableTet': '\xed\x85\xa5',
'DoubleExclamationMark': '\xe2\x80\xbc',
'UnicodeCharacter0xf255': '\xef\x89\x95',
'HangulSyllableSwagg': '\xec\x86\xaa',
'CjkUnifiedIdeograph7f78': '\xe7\xbd\xb8',
'CjkUnifiedIdeograph7f79': '\xe7\xbd\xb9',
'CjkUnifiedIdeograph7f74': '\xe7\xbd\xb4',
'CjkUnifiedIdeograph7f75': '\xe7\xbd\xb5',
'CjkUnifiedIdeograph7f76': '\xe7\xbd\xb6',
'CjkUnifiedIdeograph7f77': '\xe7\xbd\xb7',
'CjkUnifiedIdeograph7f70': '\xe7\xbd\xb0',
'CjkUnifiedIdeograph7f71': '\xe7\xbd\xb1',
'CjkUnifiedIdeograph7f72': '\xe7\xbd\xb2',
'CjkUnifiedIdeograph7f73': '\xe7\xbd\xb3',
'CjkUnifiedIdeograph46f8': '\xe4\x9b\xb8',
'CjkUnifiedIdeograph46f9': '\xe4\x9b\xb9',
'CjkUnifiedIdeograph46f2': '\xe4\x9b\xb2',
'CjkUnifiedIdeograph46f3': '\xe4\x9b\xb3',
'CjkUnifiedIdeograph46f0': '\xe4\x9b\xb0',
'CjkUnifiedIdeograph46f1': '\xe4\x9b\xb1',
'CjkUnifiedIdeograph46f6': '\xe4\x9b\xb6',
'LatinCapitalLetterOWithBreve': '\xc5\x8e',
'ParenthesizedNumberFourteen': '\xe2\x92\x81',
'CjkUnifiedIdeograph46f5': '\xe4\x9b\xb5',
'EthiopicSyllablePharyngealAa': '\xe1\x8b\x93',
'UpwardsArrow': '\xe2\x86\x91',
'UnicodeCharacter0xf74e': '\xef\x9d\x8e',
'TibetanMarkClosingBrdaRnyingYigMgoSgabMa': '\xe0\xbf\x94',
'HangulSyllableBbwaeg': '\xeb\xbd\xb9',
'LimbuDigitFour': '\xe1\xa5\x8a',
'ArabicLetterJeemFinalForm': '\xef\xba\x9e',
'CjkUnifiedIdeograph5f4f': '\xe5\xbd\x8f',
'CjkUnifiedIdeograph5f4e': '\xe5\xbd\x8e',
'CjkUnifiedIdeograph5f4d': '\xe5\xbd\x8d',
'CjkUnifiedIdeograph5f4c': '\xe5\xbd\x8c',
'CjkUnifiedIdeograph5f4b': '\xe5\xbd\x8b',
'CjkUnifiedIdeograph5f4a': '\xe5\xbd\x8a',
'UnicodeCharacter0xf5f5': '\xef\x97\xb5',
'UnicodeCharacter0xf74d': '\xef\x9d\x8d',
'HalfwidthKatakanaLetterE': '\xef\xbd\xb4',
'UnicodeCharacter0xe9bd': '\xee\xa6\xbd',
'ArabicLigatureBehWithNoonFinalForm': '\xef\xb1\xad',
'ArabicLetterFarsiYehWithTwoDotsAbove': '\xd8\xbe',
'LimbuQuestionMark': '\xe1\xa5\x85',
'HangulJongseongRieulpansios': '\xe1\x87\x97',
'ModifierLetterTurnedComma': '\xca\xbb',
'ArabicLetterZah': '\xd8\xb8',
'HalfwidthHangulLetterSsangsios': '\xef\xbe\xb6',
'CjkUnifiedIdeograph5f49': '\xe5\xbd\x89',
'CjkUnifiedIdeograph5f48': '\xe5\xbd\x88',
'CjkUnifiedIdeograph5f47': '\xe5\xbd\x87',
'CjkUnifiedIdeograph5f46': '\xe5\xbd\x86',
'CjkUnifiedIdeograph5f45': '\xe5\xbd\x85',
'CjkUnifiedIdeograph5f44': '\xe5\xbd\x84',
'CjkUnifiedIdeograph5f43': '\xe5\xbd\x83',
'CjkUnifiedIdeograph5f42': '\xe5\xbd\x82',
'CjkUnifiedIdeograph5f41': '\xe5\xbd\x81',
'CjkUnifiedIdeograph5f40': '\xe5\xbd\x80',
'UnicodeCharacter0xde56': '\xed\xb9\x96',
'UnicodeCharacter0xde57': '\xed\xb9\x97',
'UnicodeCharacter0xde54': '\xed\xb9\x94',
'WhiteCircleWithUpperRightQuadrant': '\xe2\x97\xb7',
'HangulSyllableCwaet': '\xec\xb5\x99',
'CircledKatakanaYo': '\xe3\x8b\xb5',
'HangulSyllableCwaes': '\xec\xb5\x93',
'HangulSyllableCwaep': '\xec\xb5\x9a',
'CircleWithSuperimposedX': '\xe2\xa6\xbb',
'UnicodeCharacter0xde52': '\xed\xb9\x92',
'CircledKatakanaYa': '\xe3\x8b\xb3',
'UnicodeCharacter0xde53': '\xed\xb9\x93',
'YiSyllableYiet': '\xea\x91\xb5',
'HangulSyllableCwaed': '\xec\xb5\x87',
'HangulSyllableCwaeb': '\xec\xb5\x91',
'YiSyllableYiep': '\xea\x91\xb8',
'HangulSyllableCwaen': '\xec\xb5\x84',
'CircledKatakanaYu': '\xe3\x8b\xb4',
'HangulSyllableCwael': '\xec\xb5\x88',
'HangulSyllableCwaem': '\xec\xb5\x90',
'HangulSyllableCwaej': '\xec\xb5\x96',
'YiSyllableYiex': '\xea\x91\xb6',
'HangulSyllableCwaeh': '\xec\xb5\x9b',
'CjkUnifiedIdeograph5c99': '\xe5\xb2\x99',
'CjkUnifiedIdeograph5c98': '\xe5\xb2\x98',
'JapaneseIndustrialStandardSymbol': '\xe3\x80\x84',
'NaryUnion': '\xe2\x8b\x83',
'UnicodeCharacter0xe309': '\xee\x8c\x89',
'CjkUnifiedIdeograph5c91': '\xe5\xb2\x91',
'UnicodeCharacter0x1a4f': '\xe1\xa9\x8f',
'CjkUnifiedIdeograph5c93': '\xe5\xb2\x93',
'CjkUnifiedIdeograph5c92': '\xe5\xb2\x92',
'CjkUnifiedIdeograph5c95': '\xe5\xb2\x95',
'CjkUnifiedIdeograph5c94': '\xe5\xb2\x94',
'CjkUnifiedIdeograph5c97': '\xe5\xb2\x97',
'CjkUnifiedIdeograph5c96': '\xe5\xb2\x96',
'HangulSyllableJjyunh': '\xec\xae\xbe',
'ArabicLetterAinMedialForm': '\xef\xbb\x8c',
'ArabicLetterNgoehFinalForm': '\xef\xae\x9b',
'UnicodeCharacter0xf769': '\xef\x9d\xa9',
'CircledNumberFortyFour': '\xe3\x8a\xb9',
'UnicodeCharacter0xf5f4': '\xef\x97\xb4',
'CjkUnifiedIdeograph884b': '\xe8\xa1\x8b',
'CjkUnifiedIdeograph884c': '\xe8\xa1\x8c',
'UnicodeCharacter0xf761': '\xef\x9d\xa1',
'CjkUnifiedIdeograph884a': '\xe8\xa1\x8a',
'CjkUnifiedIdeograph884f': '\xe8\xa1\x8f',
'UnicodeCharacter0xf766': '\xef\x9d\xa6',
'CjkUnifiedIdeograph884d': '\xe8\xa1\x8d',
'CjkUnifiedIdeograph884e': '\xe8\xa1\x8e',
'UnicodeCharacter0xf11b': '\xef\x84\x9b',
'YiRadicalBur': '\xea\x92\xa5',
'HangulSyllableJjyung': '\xec\xaf\x8d',
'ArabicLigatureMeemWithAlefFinalForm': '\xef\xb2\x88',
'TrigramForThunder': '\xe2\x98\xb3',
'UnicodeCharacter0xe305': '\xee\x8c\x85',
'CjkUnifiedIdeograph5c9a': '\xe5\xb2\x9a',
'CjkUnifiedIdeograph5c9c': '\xe5\xb2\x9c',
'CjkUnifiedIdeograph5c9b': '\xe5\xb2\x9b',
'CjkUnifiedIdeograph5c9e': '\xe5\xb2\x9e',
'CjkUnifiedIdeograph5c9d': '\xe5\xb2\x9d',
'CjkUnifiedIdeograph82ae': '\xe8\x8a\xae',
'CjkUnifiedIdeograph5c9f': '\xe5\xb2\x9f',
'ArmenianSmallLetterVew': '\xd5\xbe',
'CjkUnifiedIdeograph8849': '\xe8\xa1\x89',
'CjkUnifiedIdeograph8842': '\xe8\xa1\x82',
'CjkUnifiedIdeograph8843': '\xe8\xa1\x83',
'SquareCup': '\xe2\x8a\x94',
'CjkUnifiedIdeograph8841': '\xe8\xa1\x81',
'CjkUnifiedIdeograph8846': '\xe8\xa1\x86',
'CjkUnifiedIdeograph8847': '\xe8\xa1\x87',
'CjkUnifiedIdeograph8844': '\xe8\xa1\x84',
'CjkUnifiedIdeograph8845': '\xe8\xa1\x85',
'HangulSyllableBess': '\xeb\xb2\xb4',
'UnicodeCharacter0xe27c': '\xee\x89\xbc',
'CjkUnifiedIdeograph6e0f': '\xe6\xb8\x8f',
'TifinaghLetterBerberAcademyYah': '\xe2\xb5\x81',
'TifinaghLetterBerberAcademyYaj': '\xe2\xb4\xb5',
'CjkUnifiedIdeograph8cc8': '\xe8\xb3\x88',
'CjkUnifiedIdeograph8cc9': '\xe8\xb3\x89',
'HangulSyllableNyaelm': '\xeb\x83\xb6',
'HangulSyllableTwelt': '\xed\x89\xb1',
'CjkUnifiedIdeograph8cc0': '\xe8\xb3\x80',
'GeorgianSmallLetterCan': '\xe2\xb4\x9a',
'CjkUnifiedIdeograph8cc2': '\xe8\xb3\x82',
'CjkUnifiedIdeograph8cc3': '\xe8\xb3\x83',
'CjkUnifiedIdeograph8cc4': '\xe8\xb3\x84',
'CjkUnifiedIdeograph8cc5': '\xe8\xb3\x85',
'CjkUnifiedIdeograph8cc6': '\xe8\xb3\x86',
'CjkUnifiedIdeograph8cc7': '\xe8\xb3\x87',
'CjkUnifiedIdeograph9d30': '\xe9\xb4\xb0',
'CjkUnifiedIdeograph8297': '\xe8\x8a\x97',
'CjkUnifiedIdeograph9d32': '\xe9\xb4\xb2',
'CjkUnifiedIdeograph9d33': '\xe9\xb4\xb3',
'CjkUnifiedIdeograph9d34': '\xe9\xb4\xb4',
'CjkUnifiedIdeograph9d35': '\xe9\xb4\xb5',
'CjkUnifiedIdeograph9d36': '\xe9\xb4\xb6',
'CjkUnifiedIdeograph9d37': '\xe9\xb4\xb7',
'CjkUnifiedIdeograph9d38': '\xe9\xb4\xb8',
'CjkUnifiedIdeograph9d39': '\xe9\xb4\xb9',
'ArabicLigatureTahWithMeemMedialForm': '\xef\xb4\xba',
'ArabicSemicolon': '\xd8\x9b',
'SyriacLetterPersianBheth': '\xdc\xad',
'CjkUnifiedIdeograph8cca': '\xe8\xb3\x8a',
'CjkUnifiedIdeograph8ccb': '\xe8\xb3\x8b',
'CjkUnifiedIdeograph8ccc': '\xe8\xb3\x8c',
'CjkUnifiedIdeograph8ccd': '\xe8\xb3\x8d',
'CjkUnifiedIdeograph8cce': '\xe8\xb3\x8e',
'CjkUnifiedIdeograph8ccf': '\xe8\xb3\x8f',
'ArabicLetterBehehFinalForm': '\xef\xad\x9b',
'HangulSyllableSsyolb': '\xec\x91\x93',
'InvisibleSeparator': '\xe2\x81\xa3',
'HangulSyllableDdwim': '\xeb\x9c\x80',
'HangulSyllableSsyolg': '\xec\x91\x91',
'DigramForLesserYang': '\xe2\x9a\x8e',
'HangulSyllableMyalg': '\xeb\xa8\x89',
'ArabicLetterNyehInitialForm': '\xef\xad\xb8',
'CjkUnifiedIdeograph9d3a': '\xe9\xb4\xba',
'CjkUnifiedIdeograph9d3b': '\xe9\xb4\xbb',
'CjkUnifiedIdeograph9d3c': '\xe9\xb4\xbc',
'CjkUnifiedIdeograph9d3d': '\xe9\xb4\xbd',
'CjkUnifiedIdeograph9d3e': '\xe9\xb4\xbe',
'CjkUnifiedIdeograph9d3f': '\xe9\xb4\xbf',
'HangulSyllableSsyolh': '\xec\x91\x97',
'NkoLetterJa': '\xdf\x96',
'CyrillicCapitalLetterUWithDoubleAcute': '\xd3\xb2',
'UnicodeCharacter0xde5d': '\xed\xb9\x9d',
'HangulSyllableTweob': '\xed\x89\x99',
'UnicodeCharacter0xe472': '\xee\x91\xb2',
'UnicodeCharacter0xe449': '\xee\x91\x89',
'UnicodeCharacter0xe448': '\xee\x91\x88',
'HalfwidthKatakanaLetterI': '\xef\xbd\xb2',
'UnicodeCharacter0xe473': '\xee\x91\xb3',
'CjkUnifiedIdeograph940e': '\xe9\x90\x8e',
'CjkUnifiedIdeograph940d': '\xe9\x90\x8d',
'UnicodeCharacter0xe441': '\xee\x91\x81',
'CjkUnifiedIdeograph940f': '\xe9\x90\x8f',
'CjkUnifiedIdeograph940a': '\xe9\x90\x8a',
'UnicodeCharacter0xe446': '\xee\x91\x86',
'CjkUnifiedIdeograph940c': '\xe9\x90\x8c',
'CjkUnifiedIdeograph940b': '\xe9\x90\x8b',
'HangulSyllableBbyeop': '\xeb\xbc\xa2',
'UnicodeCharacter0xe471': '\xee\x91\xb1',
'LeftNormalFactorSemidirectProduct': '\xe2\x8b\x89',
'ModifierLetterStressAndHighTone': '\xea\x9c\xa0',
'CjkUnifiedIdeograph571e': '\xe5\x9c\x9e',
'CjkUnifiedIdeograph571d': '\xe5\x9c\x9d',
'CjkUnifiedIdeograph571f': '\xe5\x9c\x9f',
'CjkUnifiedIdeograph571a': '\xe5\x9c\x9a',
'CjkUnifiedIdeograph571c': '\xe5\x9c\x9c',
'CjkUnifiedIdeograph571b': '\xe5\x9c\x9b',
'RightwardsArrowWithTailWithDoubleVerticalStroke': '\xe2\xa4\x95',
'HangulSyllableBbyeos': '\xeb\xbc\x9b',
'BengaliSignNukta': '\xe0\xa6\xbc',
'HangulLetterKapyeounpieup': '\xe3\x85\xb8',
'CjkUnifiedIdeograph59ea': '\xe5\xa7\xaa',
'CjkUnifiedIdeograph59ef': '\xe5\xa7\xaf',
'CjkUnifiedIdeograph59ee': '\xe5\xa7\xae',
'CjkUnifiedIdeograph59ed': '\xe5\xa7\xad',
'CjkUnifiedIdeograph8158': '\xe8\x85\x98',
'CjkUnifiedIdeograph8159': '\xe8\x85\x99',
'CjkUnifiedIdeograph8e3d': '\xe8\xb8\xbd',
'CjkUnifiedIdeograph8150': '\xe8\x85\x90',
'CjkUnifiedIdeograph8151': '\xe8\x85\x91',
'CjkUnifiedIdeograph8152': '\xe8\x85\x92',
'CjkUnifiedIdeograph8153': '\xe8\x85\x93',
'CjkUnifiedIdeograph8154': '\xe8\x85\x94',
'CjkUnifiedIdeograph8155': '\xe8\x85\x95',
'CjkUnifiedIdeograph8156': '\xe8\x85\x96',
'CjkUnifiedIdeograph8157': '\xe8\x85\x97',
'CjkUnifiedIdeograph7a51': '\xe7\xa9\x91',
'UnicodeCharacter0xe479': '\xee\x91\xb9',
'CjkUnifiedIdeograph5719': '\xe5\x9c\x99',
'CjkUnifiedIdeograph5718': '\xe5\x9c\x98',
'CjkUnifiedIdeograph7a50': '\xe7\xa9\x90',
'CjkUnifiedIdeograph5715': '\xe5\x9c\x95',
'CjkUnifiedIdeograph5714': '\xe5\x9c\x94',
'CjkUnifiedIdeograph5717': '\xe5\x9c\x97',
'HangulChoseongKapyeounrieul': '\xe1\x84\x9b',
'CjkUnifiedIdeograph5711': '\xe5\x9c\x91',
'CjkUnifiedIdeograph5710': '\xe5\x9c\x90',
'CjkUnifiedIdeograph5713': '\xe5\x9c\x93',
'CjkUnifiedIdeograph5712': '\xe5\x9c\x92',
'CjkUnifiedIdeograph7a52': '\xe7\xa9\x92',
'CjkUnifiedIdeograph9409': '\xe9\x90\x89',
'CjkUnifiedIdeograph9408': '\xe9\x90\x88',
'YiSyllableZzyrx': '\xea\x8b\xab',
'CjkUnifiedIdeograph9405': '\xe9\x90\x85',
'CjkUnifiedIdeograph7a55': '\xe7\xa9\x95',
'CjkUnifiedIdeograph9407': '\xe9\x90\x87',
'CjkUnifiedIdeograph9406': '\xe9\x90\x86',
'CjkUnifiedIdeograph9401': '\xe9\x90\x81',
'CjkUnifiedIdeograph9400': '\xe9\x90\x80',
'CjkUnifiedIdeograph9403': '\xe9\x90\x83',
'CjkUnifiedIdeograph7a54': '\xe7\xa9\x94',
'CjkUnifiedIdeograph8bc9': '\xe8\xaf\x89',
'CjkUnifiedIdeograph8bc8': '\xe8\xaf\x88',
'om': '\xcf\x89',
'UnicodeCharacter0xf5f6': '\xef\x97\xb6',
'CjkUnifiedIdeograph7a57': '\xe7\xa9\x97',
'CjkUnifiedIdeograph8bc1': '\xe8\xaf\x81',
'CjkUnifiedIdeograph815a': '\xe8\x85\x9a',
'CjkUnifiedIdeograph815b': '\xe8\x85\x9b',
'CjkUnifiedIdeograph7a56': '\xe7\xa9\x96',
'CjkUnifiedIdeograph815d': '\xe8\x85\x9d',
'CjkUnifiedIdeograph815e': '\xe8\x85\x9e',
'CjkUnifiedIdeograph815f': '\xe8\x85\x9f',
'CjkUnifiedIdeograph8bc6': '\xe8\xaf\x86',
'CjkUnifiedIdeograph59e9': '\xe5\xa7\xa9',
'CjkUnifiedIdeograph59e8': '\xe5\xa7\xa8',
'HangulSyllableTweon': '\xed\x89\x8c',
'CjkUnifiedIdeograph59e3': '\xe5\xa7\xa3',
'ParenthesizedHangulKiyeokA': '\xe3\x88\x8e',
'CjkUnifiedIdeograph59e1': '\xe5\xa7\xa1',
'CjkUnifiedIdeograph59e0': '\xe5\xa7\xa0',
'WheelchairSymbol': '\xe2\x99\xbf',
'CjkUnifiedIdeograph59e6': '\xe5\xa7\xa6',
'CjkUnifiedIdeograph59e5': '\xe5\xa7\xa5',
'CjkUnifiedIdeograph59e4': '\xe5\xa7\xa4',
'CyrillicSmallLetterSoftDe': '\xea\x99\xa3',
'RightwardsArrowAboveAlmostEqualTo': '\xe2\xa5\xb5',
'GeorgianCapitalLetterUn': '\xe1\x82\xb3',
'CjkUnifiedIdeograph922f': '\xe9\x88\xaf',
'CjkUnifiedIdeograph630e': '\xe6\x8c\x8e',
'CjkUnifiedIdeograph5878': '\xe5\xa1\xb8',
'HebrewLetterFinalPeWithDagesh': '\xef\xad\x83',
'CjkUnifiedIdeograph5879': '\xe5\xa1\xb9',
'UnicodeCharacter0xab27': '\xea\xac\xa7',
'UnicodeCharacter0xab26': '\xea\xac\xa6',
'UnicodeCharacter0xab25': '\xea\xac\xa5',
'UnicodeCharacter0xab24': '\xea\xac\xa4',
'UnicodeCharacter0xab23': '\xea\xac\xa3',
'UnicodeCharacter0xab22': '\xea\xac\xa2',
'LatinCapitalLetterOWithDotBelow': '\xe1\xbb\x8c',
'UnicodeCharacter0xab20': '\xea\xac\xa0',
'UnicodeCharacter0xab29': '\xea\xac\xa9',
'UnicodeCharacter0xab28': '\xea\xac\xa8',
'CjkUnifiedIdeograph8886': '\xe8\xa2\x86',
'HangulSyllableNoss': '\xeb\x86\x8c',
'CjkUnifiedIdeograph5875': '\xe5\xa1\xb5',
'CjkUnifiedIdeograph5876': '\xe5\xa1\xb6',
'CjkUnifiedIdeograph53fc': '\xe5\x8f\xbc',
'CjkUnifiedIdeograph5870': '\xe5\xa1\xb0',
'LatinCapitalLetterYWithTilde': '\xe1\xbb\xb8',
'CjkUnifiedIdeograph5871': '\xe5\xa1\xb1',
'UnicodeCharacter0xe33a': '\xee\x8c\xba',
'KannadaSignUpadhmaniya': '\xe0\xb3\xb2',
'CjkUnifiedIdeograph5872': '\xe5\xa1\xb2',
'CjkUnifiedIdeograph8881': '\xe8\xa2\x81',
'TamilVowelSignEe': '\xe0\xaf\x87',
'RightSshapedBagDelimiter': '\xe2\x9f\x86',
'CjkRadicalCsimplifiedShell': '\xe2\xbb\x89',
'AplFunctionalSymbolQuadDelta': '\xe2\x8d\x8d',
'CyrillicSmallLetterEmWithTail': '\xd3\x8e',
'UnicodeCharacter0xab2f': '\xea\xac\xaf',
'UnicodeCharacter0xab2e': '\xea\xac\xae',
'UnicodeCharacter0xab2d': '\xea\xac\xad',
'UnicodeCharacter0xab2c': '\xea\xac\xac',
'UnicodeCharacter0xab2b': '\xea\xac\xab',
'UnicodeCharacter0xab2a': '\xea\xac\xaa',
'HangulSyllableJjing': '\xec\xb0\xa1',
'HangulSyllablePyael': '\xed\x8d\xa8',
'ArabicLetterAlefWithHamzaBelowFinalForm': '\xef\xba\x88',
'UnicodeCharacter0xe5bc': '\xee\x96\xbc',
'HangulSyllableDdwik': '\xeb\x9c\x88',
'LatinSmallLetterAWithCircumflexAndGrave': '\xe1\xba\xa7',
'SquarePeso': '\xe3\x8c\xb7',
'ArabicLigatureSheenWithYehFinalForm': '\xef\xb4\x9a',
'UnicodeCharacter0xf450': '\xef\x91\x90',
'HangulSyllablePyaek': '\xed\x8d\xb8',
'YiSyllableXy': '\xea\x91\xad',
'UnicodeCharacter0xe5bd': '\xee\x96\xbd',
'BallotBox': '\xe2\x98\x90',
'HangulSyllableDye': '\xeb\x8e\xa8',
'UnicodeCharacter0xf452': '\xef\x91\x92',
'CjkUnifiedIdeograph55f8': '\xe5\x97\xb8',
'HangulSyllableDya': '\xeb\x8c\x9c',
'CjkUnifiedIdeograph8120': '\xe8\x84\xa0',
'EthiopicSyllableShee': '\xe1\x88\xbc',
'CjkUnifiedIdeograph8e5e': '\xe8\xb9\x9e',
'CjkUnifiedIdeograph8e5f': '\xe8\xb9\x9f',
'CjkUnifiedIdeograph8e5a': '\xe8\xb9\x9a',
'HangulChoseongIeungmieum': '\xe1\x85\x83',
'CjkUnifiedIdeograph877e': '\xe8\x9d\xbe',
'HangulSyllableDyu': '\xeb\x93\x80',
'CyrillicSmallLetterYn': '\xea\x99\x9f',
'UnicodeCharacter0xf457': '\xef\x91\x97',
'UnicodeCharacter0xda43': '\xed\xa9\x83',
'HangulSyllableSeunh': '\xec\x8a\xaa',
'CjkUnifiedIdeograph6e09': '\xe6\xb8\x89',
'UnicodeCharacter0xe5be': '\xee\x96\xbe',
'TibetanLetterSsa': '\xe0\xbd\xa5',
'CjkUnifiedIdeograph6e08': '\xe6\xb8\x88',
'CjkUnifiedIdeograph8e31': '\xe8\xb8\xb1',
'MeasuredAngleWithOpenArmEndingInArrowPointingUpAndRight': '\xe2\xa6\xa8',
'YiSyllableChy': '\xea\x8d\xb8',
'MongolianLetterTodoQa': '\xe1\xa1\x8d',
'CjkUnifiedIdeograph9cdc': '\xe9\xb3\x9c',
'CanadianSyllabicsWestcreeCwe': '\xe1\x92\x93',
'HangulSyllableBbeb': '\xeb\xbb\xbd',
'CanadianSyllabicsWestcreeCwa': '\xe1\x92\x9d',
'PresentationFormForVerticalRightAngleBracket': '\xef\xb9\x80',
'HangulSyllableBbec': '\xeb\xbc\x83',
'CanadianSyllabicsWestcreeCwo': '\xe1\x92\x99',
'RunicLetterIngwaz': '\xe1\x9b\x9c',
'CjkUnifiedIdeograph9315': '\xe9\x8c\x95',
'CjkUnifiedIdeograph55f6': '\xe5\x97\xb6',
'ArabicLetterWawWithDotAbove': '\xdb\x8f',
'ArabicLigatureGhainWithJeemIsolatedForm': '\xef\xb0\xab',
'UnicodeCharacter0xca9': '\xe0\xb2\xa9',
'HangulSyllableBbeg': '\xeb\xbb\xad',
'UnicodeCharacter0xe99a': '\xee\xa6\x9a',
'NeitherGreaterthanNorLessthan': '\xe2\x89\xb9',
'BraillePatternDots178': '\xe2\xa3\x81',
'CjkUnifiedIdeograph8e59': '\xe8\xb9\x99',
'CjkUnifiedIdeograph8e54': '\xe8\xb9\x94',
'CjkUnifiedIdeograph8e55': '\xe8\xb9\x95',
'CjkUnifiedIdeograph8e56': '\xe8\xb9\x96',
'CjkUnifiedIdeograph8e57': '\xe8\xb9\x97',
'CjkUnifiedIdeograph8e50': '\xe8\xb9\x90',
'CjkUnifiedIdeograph8e51': '\xe8\xb9\x91',
'CjkUnifiedIdeograph8e52': '\xe8\xb9\x92',
'CjkUnifiedIdeograph8e53': '\xe8\xb9\x93',
'HangulSyllableGyu': '\xea\xb7\x9c',
'HangulSyllableBbyol': '\xeb\xbe\xb8',
'CjkUnifiedIdeograph998a': '\xe9\xa6\x8a',
'HangulSyllableTwat': '\xed\x87\x95',
'CjkUnifiedIdeograph998b': '\xe9\xa6\x8b',
'HangulSyllableTwas': '\xed\x87\x8f',
'YiSyllableChu': '\xea\x8d\xb2',
'HangulSyllableTwam': '\xed\x87\x8c',
'HangulSyllableGye': '\xea\xb3\x84',
'ArabicLetterJeh': '\xda\x98',
'CjkUnifiedIdeograph998c': '\xe9\xa6\x8c',
'HangulSyllableGya': '\xea\xb0\xb8',
'HangulSyllableTwak': '\xed\x87\x94',
'HangulSyllableTwaj': '\xed\x87\x92',
'HangulSyllableTwae': '\xed\x87\x98',
'CjkUnifiedIdeograph998d': '\xe9\xa6\x8d',
'HangulSyllableTwag': '\xed\x86\xbd',
'HangulSyllableGyo': '\xea\xb5\x90',
'HangulSyllableGyi': '\xea\xb8\x94',
'CjkUnifiedIdeograph7da1': '\xe7\xb6\xa1',
'CjkUnifiedIdeograph5743': '\xe5\x9d\x83',
'HangulSyllableGgyang': '\xea\xba\x99',
'LatinSmallLetterOWithDoubleAcute': '\xc5\x91',
'CjkUnifiedIdeograph998f': '\xe9\xa6\x8f',
'HangulSyllableGgyanh': '\xea\xba\x8a',
'UnicodeCharacter0xe586': '\xee\x96\x86',
'HangulSyllableGgyanj': '\xea\xba\x89',
'UnicodeCharacter0xf20a': '\xef\x88\x8a',
'UnicodeCharacter0xf20b': '\xef\x88\x8b',
'UnicodeCharacter0xf20c': '\xef\x88\x8c',
'UnicodeCharacter0xf20d': '\xef\x88\x8d',
'HangulSyllableHoelt': '\xed\x9a\x99',
'UnicodeCharacter0xf20f': '\xef\x88\x8f',
'UnicodeCharacter0xfaf4': '\xef\xab\xb4',
'HangulSyllableSsalg': '\xec\x8c\x81',
'**9': '\xe2\x81\xb9',
'HangulSyllableSsalb': '\xec\x8c\x83',
'HangulSyllableSsalm': '\xec\x8c\x82',
'**6': '\xe2\x81\xb6',
'**5': '\xe2\x81\xb5',
'UnicodeCharacter0xa9b8': '\xea\xa6\xb8',
'HangulSyllableDyigs': '\xeb\x93\xbb',
'HangulSyllableSsalh': '\xec\x8c\x87',
'**1': '\xc2\xb9',
'**0': '\xe2\x81\xb0',
'VerticalKanaRepeatMark': '\xe3\x80\xb1',
'**-': '\xe2\x81\xbb',
'TibetanMarkNyisTsheg': '\xe0\xbf\x92',
'HangulSyllableSsalp': '\xec\x8c\x86',
'HangulSyllableSsals': '\xec\x8c\x84',
'**(': '\xe2\x81\xbd',
'WhiteDiamond': '\xe2\x97\x87',
'HangulSyllableBbis': '\xec\x82\xa3',
'UnicodeCharacter0xf200': '\xef\x88\x80',
'UnicodeCharacter0xf201': '\xef\x88\x81',
'UnicodeCharacter0xf202': '\xef\x88\x82',
'CjkUnifiedIdeograph9c81': '\xe9\xb2\x81',
'UnicodeCharacter0xf204': '\xef\x88\x84',
'UnicodeCharacter0xf205': '\xef\x88\x85',
'UnicodeCharacter0xf206': '\xef\x88\x86',
'UnicodeCharacter0xf207': '\xef\x88\x87',
'UnicodeCharacter0xf208': '\xef\x88\x88',
'UnicodeCharacter0xa9b9': '\xea\xa6\xb9',
'CjkUnifiedIdeograph91ac': '\xe9\x86\xac',
'HangulSyllableNyaegg': '\xeb\x83\xae',
'HangulSyllableJjinh': '\xec\xb0\x92',
'LatinSmallLetterOWithCaron': '\xc7\x92',
'FullwidthPercentSign': '\xef\xbc\x85',
'HangulSyllableNyaegs': '\xeb\x83\xaf',
'HangulSyllableJjinj': '\xec\xb0\x91',
'UnicodeCharacter0xfaf6': '\xef\xab\xb6',
'ArabicLetterNgoehIsolatedForm': '\xef\xae\x9a',
'HangulSyllableHed': '\xed\x97\xab',
'UnicodeCharacter0xdf4f': '\xed\xbd\x8f',
'UnicodeCharacter0xab18': '\xea\xac\x98',
'UnicodeCharacter0xe548': '\xee\x95\x88',
'CjkUnifiedIdeograph91ab': '\xe9\x86\xab',
'UnicodeCharacter0xab19': '\xea\xac\x99',
'HangulSyllableJjyic': '\xec\xb0\x87',
'CjkUnifiedIdeograph9c77': '\xe9\xb1\xb7',
'CjkUnifiedIdeograph9c76': '\xe9\xb1\xb6',
'CjkUnifiedIdeograph9c75': '\xe9\xb1\xb5',
'CjkUnifiedIdeograph9c74': '\xe9\xb1\xb4',
'CjkUnifiedIdeograph9c73': '\xe9\xb1\xb3',
'CjkUnifiedIdeograph9c72': '\xe9\xb1\xb2',
'CjkUnifiedIdeograph9c71': '\xe9\xb1\xb1',
'CjkUnifiedIdeograph9c70': '\xe9\xb1\xb0',
'CjkUnifiedIdeograph9c79': '\xe9\xb1\xb9',
'CjkUnifiedIdeograph9c78': '\xe9\xb1\xb8',
'_gamma': '\xe1\xb5\xa7',
'CjkUnifiedIdeograph7a5a': '\xe7\xa9\x9a',
'UnicodeCharacter0xab15': '\xea\xac\x95',
'CjkUnifiedIdeograph91aa': '\xe9\x86\xaa',
'UnicodeCharacter0xab12': '\xea\xac\x92',
'CjkUnifiedIdeograph7a5c': '\xe7\xa9\x9c',
'CjkUnifiedIdeograph4244': '\xe4\x89\x84',
'CjkUnifiedIdeograph4245': '\xe4\x89\x85',
'CjkUnifiedIdeograph4246': '\xe4\x89\x86',
'CjkUnifiedIdeograph4247': '\xe4\x89\x87',
'CjkUnifiedIdeograph4240': '\xe4\x89\x80',
'CjkUnifiedIdeograph4241': '\xe4\x89\x81',
'CjkUnifiedIdeograph4242': '\xe4\x89\x82',
'CjkUnifiedIdeograph4243': '\xe4\x89\x83',
'CjkUnifiedIdeograph92f9': '\xe9\x8b\xb9',
'CjkUnifiedIdeograph92f8': '\xe9\x8b\xb8',
'UnicodeCharacter0xe874': '\xee\xa1\xb4',
'UnicodeCharacter0xab11': '\xea\xac\x91',
'CjkUnifiedIdeograph4248': '\xe4\x89\x88',
'CjkUnifiedIdeograph4249': '\xe4\x89\x89',
'CjkUnifiedIdeograph7a5d': '\xe7\xa9\x9d',
'YiSyllableNyip': '\xea\x91\x8d',
'CjkUnifiedIdeograph9c7f': '\xe9\xb1\xbf',
'CjkUnifiedIdeograph9c7e': '\xe9\xb1\xbe',
'CjkUnifiedIdeograph9c7d': '\xe9\xb1\xbd',
'CjkUnifiedIdeograph9c7c': '\xe9\xb1\xbc',
'CjkUnifiedIdeograph9c7b': '\xe9\xb1\xbb',
'CjkUnifiedIdeograph9c7a': '\xe9\xb1\xba',
'YiSyllableNyix': '\xea\x91\x8b',
'UnicodeCharacter0xf45e': '\xef\x91\x9e',
'CjkUnifiedIdeograph7a5f': '\xe7\xa9\x9f',
'UnicodeCharacter0xf45d': '\xef\x91\x9d',
'YiSyllableNyie': '\xea\x91\x90',
'UnicodeCharacter0xf45f': '\xef\x91\x9f',
'CjkUnifiedIdeograph424d': '\xe4\x89\x8d',
'CjkUnifiedIdeograph424e': '\xe4\x89\x8e',
'CjkUnifiedIdeograph424f': '\xe4\x89\x8f',
'CjkUnifiedIdeograph92fb': '\xe9\x8b\xbb',
'CjkUnifiedIdeograph92fe': '\xe9\x8b\xbe',
'CjkUnifiedIdeograph424a': '\xe4\x89\x8a',
'MediumMathematicalSpace': '\xe2\x81\x9f',
'CjkUnifiedIdeograph424c': '\xe4\x89\x8c',
'KhmerLetterPho': '\xe1\x9e\x97',
'CanadianSyllabicsTya': '\xe1\x95\xb3',
'UnicodeCharacter0xfaf2': '\xef\xab\xb2',
'LeftwardsHarpoonWithBarbUpwards': '\xe2\x86\xbc',
'HangulSyllableBbwags': '\xeb\xbd\x9f',
'CanadianSyllabicsTyi': '\xe1\x95\xb1',
'RightDoubleWigglyFence': '\xe2\xa7\x9b',
'CjkUnifiedIdeograph91af': '\xe9\x86\xaf',
'HangulSyllableHibs': '\xed\x9e\x9a',
'LeftwardsArrowWithTipDownwards': '\xe2\xac\x90',
'CjkUnifiedIdeograph5e45': '\xe5\xb9\x85',
'GreekCapitalLetterEtaWithDasia': '\xe1\xbc\xa9',
'TibetanLogotypeSignLhagRtags': '\xe0\xbc\x96',
'HangulSyllableKed': '\xec\xbc\x87',
'CjkUnifiedIdeograph587d': '\xe5\xa1\xbd',
'UnicodeCharacter0xefe8': '\xee\xbf\xa8',
'UnicodeCharacter0xefe9': '\xee\xbf\xa9',
'CjkUnifiedIdeograph587e': '\xe5\xa1\xbe',
'UnicodeCharacter0xefe2': '\xee\xbf\xa2',
'UnicodeCharacter0xefe3': '\xee\xbf\xa3',
'UnicodeCharacter0xefe0': '\xee\xbf\xa0',
'UnicodeCharacter0xefe1': '\xee\xbf\xa1',
'UnicodeCharacter0xefe6': '\xee\xbf\xa6',
'CjkUnifiedIdeograph587f': '\xe5\xa1\xbf',
'CjkUnifiedIdeograph91ae': '\xe9\x86\xae',
'UnicodeCharacter0xefe5': '\xee\xbf\xa5',
'CjkUnifiedIdeograph888e': '\xe8\xa2\x8e',
'YiSyllableNzie': '\xea\x8b\xb2',
'CjkUnifiedIdeograph53f4': '\xe5\x8f\xb4',
'CjkUnifiedIdeograph5e43': '\xe5\xb9\x83',
'CjkUnifiedIdeograph89a3': '\xe8\xa6\xa3',
'CjkUnifiedIdeograph587a': '\xe5\xa1\xba',
'YiSyllableNzip': '\xea\x8b\xb0',
'CjkUnifiedIdeograph587b': '\xe5\xa1\xbb',
'YiSyllableNzit': '\xea\x8b\xad',
'CjkUnifiedIdeograph587c': '\xe5\xa1\xbc',
'MyanmarDigitNine': '\xe1\x81\x89',
'ArabicLetterQafMedialForm': '\xef\xbb\x98',
'HangulSyllableJags': '\xec\x9e\x93',
'GurmukhiLetterFa': '\xe0\xa9\x9e',
'UnicodeCharacter0xefeb': '\xee\xbf\xab',
'UnicodeCharacter0xefec': '\xee\xbf\xac',
'CjkUnifiedIdeograph7da3': '\xe7\xb6\xa3',
'UnicodeCharacter0xefea': '\xee\xbf\xaa',
'UnicodeCharacter0xefef': '\xee\xbf\xaf',
'UnicodeCharacter0xefed': '\xee\xbf\xad',
'CjkUnifiedIdeograph5e49': '\xe5\xb9\x89',
'HangulSyllableJagg': '\xec\x9e\x92',
'UnicodeCharacter0xdf4a': '\xed\xbd\x8a',
'HangulSyllableBbaeb': '\xeb\xba\x8d',
'HangulSyllableBbaec': '\xeb\xba\x93',
'HangulSyllableBbaed': '\xeb\xba\x83',
'CjkUnifiedIdeograph5ecd': '\xe5\xbb\x8d',
'HangulSyllableBbaeh': '\xeb\xba\x97',
'MeasuredBy': '\xe2\x89\x9e',
'HangulSyllableBbaej': '\xeb\xba\x92',
'HangulSyllableBbaek': '\xeb\xba\x94',
'HangulSyllableBbael': '\xeb\xba\x84',
'HangulSyllableBbaem': '\xeb\xba\x8c',
'EthiopicSyllableTsaa': '\xe1\x8c\xbb',
'HangulSyllableBbaep': '\xeb\xba\x96',
'HangulSyllableBbaes': '\xeb\xba\x8f',
'HangulSyllableBbaet': '\xeb\xba\x95',
'HangulSyllableJaegs': '\xec\x9e\xaf',
'CanadianSyllabicsSaai': '\xe1\x93\xae',
'SyriacHorizontalColon': '\xdc\x85',
'LeftCurlyBracketMiddlePiece': '\xe2\x8e\xa8',
'CjkUnifiedIdeograph9364': '\xe9\x8d\xa4',
'CjkUnifiedIdeograph9365': '\xe9\x8d\xa5',
'CjkUnifiedIdeograph9366': '\xe9\x8d\xa6',
'CjkUnifiedIdeograph9367': '\xe9\x8d\xa7',
'CjkUnifiedIdeograph9360': '\xe9\x8d\xa0',
'CjkUnifiedIdeograph9361': '\xe9\x8d\xa1',
'CjkUnifiedIdeograph9362': '\xe9\x8d\xa2',
'CjkUnifiedIdeograph9363': '\xe9\x8d\xa3',
'CjkUnifiedIdeograph9c8f': '\xe9\xb2\x8f',
'CjkUnifiedIdeograph9368': '\xe9\x8d\xa8',
'CjkUnifiedIdeograph9369': '\xe9\x8d\xa9',
'HiraganaLetterSmallWa': '\xe3\x82\x8e',
'DoublelineEqualToOrLessthan': '\xe2\xaa\x99',
'ArabicLigatureHahWithAlefMaksuraFinalForm': '\xef\xb4\x9b',
'CjkUnifiedIdeograph648d': '\xe6\x92\x8d',
'CjkUnifiedIdeograph648e': '\xe6\x92\x8e',
'CjkUnifiedIdeograph648f': '\xe6\x92\x8f',
'CjkUnifiedIdeograph648a': '\xe6\x92\x8a',
'CjkUnifiedIdeograph648b': '\xe6\x92\x8b',
'CjkUnifiedIdeograph648c': '\xe6\x92\x8c',
'UnicodeCharacter0xd8b8': '\xed\xa2\xb8',
'UnicodeCharacter0xd8b9': '\xed\xa2\xb9',
'PhagspaMarkDoubleShad': '\xea\xa1\xb7',
'ChamLetterFinalR': '\xea\xa9\x89',
'UnicodeCharacter0xd8b2': '\xed\xa2\xb2',
'UnicodeCharacter0xd8b3': '\xed\xa2\xb3',
'UnicodeCharacter0xd8b0': '\xed\xa2\xb0',
'UnicodeCharacter0xd8b1': '\xed\xa2\xb1',
'UnicodeCharacter0xd8b6': '\xed\xa2\xb6',
'UnicodeCharacter0xd8b7': '\xed\xa2\xb7',
'UnicodeCharacter0xd8b4': '\xed\xa2\xb4',
'UnicodeCharacter0xd8b5': '\xed\xa2\xb5',
'LatinCapitalLetterOWithTildeAndMacron': '\xc8\xac',
'UnicodeCharacter0xab1d': '\xea\xac\x9d',
'HangulSyllableKwaep': '\xec\xbe\xa6',
'CjkUnifiedIdeograph6488': '\xe6\x92\x88',
'CjkUnifiedIdeograph6489': '\xe6\x92\x89',
'GreekCapitalLetterAlphaWithVrachy': '\xe1\xbe\xb8',
'CjkUnifiedIdeograph6485': '\xe6\x92\x85',
'CjkUnifiedIdeograph6486': '\xe6\x92\x86',
'CjkUnifiedIdeograph6487': '\xe6\x92\x87',
'CjkUnifiedIdeograph6480': '\xe6\x92\x80',
'CjkUnifiedIdeograph6481': '\xe6\x92\x81',
'CjkUnifiedIdeograph6482': '\xe6\x92\x82',
'CjkUnifiedIdeograph6483': '\xe6\x92\x83',
'CjkUnifiedIdeograph936d': '\xe9\x8d\xad',
'CjkUnifiedIdeograph936e': '\xe9\x8d\xae',
'CjkUnifiedIdeograph936f': '\xe9\x8d\xaf',
'CjkUnifiedIdeograph936a': '\xe9\x8d\xaa',
'CjkUnifiedIdeograph936b': '\xe9\x8d\xab',
'CjkUnifiedIdeograph936c': '\xe9\x8d\xac',
'CjkUnifiedIdeograph7bff': '\xe7\xaf\xbf',
'VaiSyllableWen': '\xea\x97\xa7',
'CjkUnifiedIdeograph9980': '\xe9\xa6\x80',
'MongolianLetterTodoLongVowelSign': '\xe1\xa1\x83',
'UnicodeCharacter0xe872': '\xee\xa1\xb2',
'CjkRadicalLameTwo': '\xe2\xba\x8f',
'VaiSyllableWee': '\xea\x94\x83',
'CjkUnifiedIdeograph9982': '\xe9\xa6\x82',
'CircledNumberFifteen': '\xe2\x91\xae',
'LatinCapitalLetterPWithHook': '\xc6\xa4',
'CjkUnifiedIdeograph9983': '\xe9\xa6\x83',
'UnicodeCharacter0xd8bb': '\xed\xa2\xbb',
'CjkUnifiedIdeograph9984': '\xe9\xa6\x84',
'UnicodeCharacter0xd8ba': '\xed\xa2\xba',
'UnicodeCharacter0xd8bf': '\xed\xa2\xbf',
'UnicodeCharacter0xd8bd': '\xed\xa2\xbd',
'CjkUnifiedIdeograph9985': '\xe9\xa6\x85',
'UnicodeCharacter0xe9de': '\xee\xa7\x9e',
'HangulChoseongSiosieung': '\xe1\x84\xb5',
'KhmerLetterPha': '\xe1\x9e\x95',
'HangulSyllableTwep': '\xed\x89\xbe',
'TibetanSignSnaLdan': '\xe0\xbe\x83',
'UnicodeCharacter0xe9dc': '\xee\xa7\x9c',
'CjkUnifiedIdeograph4a18': '\xe4\xa8\x98',
'HangulSyllableSogs': '\xec\x86\x8f',
'GlagoliticSmallLetterOtu': '\xe2\xb1\x89',
'UnicodeCharacter0xde0b': '\xed\xb8\x8b',
'CjkUnifiedIdeograph3991': '\xe3\xa6\x91',
'CjkUnifiedIdeograph3990': '\xe3\xa6\x90',
'CjkUnifiedIdeograph3993': '\xe3\xa6\x93',
'CjkUnifiedIdeograph3992': '\xe3\xa6\x92',
'CjkUnifiedIdeograph3995': '\xe3\xa6\x95',
'CjkUnifiedIdeograph3994': '\xe3\xa6\x94',
'CjkUnifiedIdeograph3997': '\xe3\xa6\x97',
'CjkUnifiedIdeograph3996': '\xe3\xa6\x96',
'CjkUnifiedIdeograph3999': '\xe3\xa6\x99',
'CjkUnifiedIdeograph3998': '\xe3\xa6\x98',
'CjkUnifiedIdeograph775a': '\xe7\x9d\x9a',
'CjkUnifiedIdeograph651e': '\xe6\x94\x9e',
'BlackCircleWithDownArrow': '\xe2\xa7\xad',
'CjkUnifiedIdeograph775f': '\xe7\x9d\x9f',
'CjkUnifiedIdeograph775e': '\xe7\x9d\x9e',
'CjkUnifiedIdeograph775d': '\xe7\x9d\x9d',
'CjkUnifiedIdeograph7281': '\xe7\x8a\x81',
'CjkUnifiedIdeograph7280': '\xe7\x8a\x80',
'CjkUnifiedIdeograph7283': '\xe7\x8a\x83',
'CjkUnifiedIdeograph7282': '\xe7\x8a\x82',
'CjkUnifiedIdeograph7285': '\xe7\x8a\x85',
'CjkUnifiedIdeograph7284': '\xe7\x8a\x84',
'CjkUnifiedIdeograph7287': '\xe7\x8a\x87',
'CjkUnifiedIdeograph7286': '\xe7\x8a\x86',
'CjkUnifiedIdeograph366c': '\xe3\x99\xac',
'CjkUnifiedIdeograph366b': '\xe3\x99\xab',
'CjkUnifiedIdeograph366a': '\xe3\x99\xaa',
'UnicodeCharacter0xea7d': '\xee\xa9\xbd',
'UnicodeCharacter0xea7c': '\xee\xa9\xbc',
'CjkUnifiedIdeograph366f': '\xe3\x99\xaf',
'CjkUnifiedIdeograph366e': '\xe3\x99\xae',
'CjkUnifiedIdeograph366d': '\xe3\x99\xad',
'HangulSyllableBbaelh': '\xeb\xba\x8b',
'UnicodeCharacter0xeb62': '\xee\xad\xa2',
'LatinSmallLetterRWithStroke': '\xc9\x8d',
'HangulSyllableDwaek': '\xeb\x90\x94',
'CjkCompatibilityIdeographf930': '\xef\xa4\xb0',
'HangulSyllableDwaej': '\xeb\x90\x92',
'HangulSyllableSeod': '\xec\x84\xa3',
'HangulSyllableBbaelg': '\xeb\xba\x85',
'TibetanLetterJa': '\xe0\xbd\x87',
'CjkUnifiedIdeograph95b6': '\xe9\x96\xb6',
'CjkCompatibilityIdeographf932': '\xef\xa4\xb2',
'CjkUnifiedIdeograph95b7': '\xe9\x96\xb7',
'UnicodeCharacter0xe9b8': '\xee\xa6\xb8',
'HangulSyllableBbaelp': '\xeb\xba\x8a',
'HangulSyllableBbaels': '\xeb\xba\x88',
'CjkCompatibilityIdeographf933': '\xef\xa4\xb3',
'CjkUnifiedIdeograph95b4': '\xe9\x96\xb4',
'CircledIdeographLow': '\xe3\x8a\xa6',
'CjkUnifiedIdeograph728a': '\xe7\x8a\x8a',
'CjkUnifiedIdeograph3669': '\xe3\x99\xa9',
'CjkUnifiedIdeograph3668': '\xe3\x99\xa8',
'CjkUnifiedIdeograph728e': '\xe7\x8a\x8e',
'CjkUnifiedIdeograph728d': '\xe7\x8a\x8d',
'UnicodeCharacter0xea79': '\xee\xa9\xb9',
'CjkUnifiedIdeograph728f': '\xe7\x8a\x8f',
'CjkUnifiedIdeograph3663': '\xe3\x99\xa3',
'CjkUnifiedIdeograph3662': '\xe3\x99\xa2',
'CjkUnifiedIdeograph3661': '\xe3\x99\xa1',
'CjkUnifiedIdeograph3660': '\xe3\x99\xa0',
'CjkUnifiedIdeograph3667': '\xe3\x99\xa7',
'CjkUnifiedIdeograph3666': '\xe3\x99\xa6',
'CjkUnifiedIdeograph3665': '\xe3\x99\xa5',
'CjkUnifiedIdeograph3664': '\xe3\x99\xa4',
'CjkUnifiedIdeograph399a': '\xe3\xa6\x9a',
'HangulSyllableRil': '\xeb\xa6\xb4',
'CjkUnifiedIdeograph399c': '\xe3\xa6\x9c',
'CjkUnifiedIdeograph399b': '\xe3\xa6\x9b',
'CjkUnifiedIdeograph399e': '\xe3\xa6\x9e',
'CjkUnifiedIdeograph399d': '\xe3\xa6\x9d',
'HangulSyllableRik': '\xeb\xa7\x84',
'CjkUnifiedIdeograph399f': '\xe3\xa6\x9f',
'CjkUnifiedIdeograph7753': '\xe7\x9d\x93',
'CjkUnifiedIdeograph7752': '\xe7\x9d\x92',
'CjkUnifiedIdeograph7751': '\xe7\x9d\x91',
'CjkUnifiedIdeograph7750': '\xe7\x9d\x90',
'CjkUnifiedIdeograph7757': '\xe7\x9d\x97',
'CjkUnifiedIdeograph7756': '\xe7\x9d\x96',
'CjkUnifiedIdeograph7755': '\xe7\x9d\x95',
'CjkUnifiedIdeograph7754': '\xe7\x9d\x94',
'UnicodeCharacter0xe389': '\xee\x8e\x89',
'YiSyllableQux': '\xea\x90\x8d',
'ThaanaLetterFaafu': '\xde\x8a',
'HangulSyllableKwils': '\xed\x81\x80',
'CjkUnifiedIdeograph7d45': '\xe7\xb5\x85',
'YiSyllableQup': '\xea\x90\x8f',
'YiSyllableQur': '\xea\x90\x91',
'HangulSyllableKwilh': '\xed\x81\x83',
'YiSyllableQuo': '\xea\x90\x86',
'HangulSyllableKwilm': '\xed\x80\xbe',
'UnicodeCharacter0xf35e': '\xef\x8d\x9e',
'HangulSyllableSsilb': '\xec\x94\xb3',
'HangulSyllablePonj': '\xed\x8f\xb1',
'HangulSyllableKwilg': '\xed\x80\xbd',
'CjkUnifiedIdeograph5ec0': '\xe5\xbb\x80',
'UnicodeCharacter0xa9cf': '\xea\xa7\x8f',
'UnicodeCharacter0xa9ce': '\xea\xa7\x8e',
'UnicodeCharacter0xa9cd': '\xea\xa7\x8d',
'UnicodeCharacter0xa9cc': '\xea\xa7\x8c',
'SundaneseLetterBa': '\xe1\xae\x98',
'UnicodeCharacter0xa9ca': '\xea\xa7\x8a',
'HangulSyllableJag': '\xec\x9e\x91',
'KangxiRadicalBig': '\xe2\xbc\xa4',
'CjkUnifiedIdeograph9bb4': '\xe9\xae\xb4',
'CjkUnifiedIdeograph9bb7': '\xe9\xae\xb7',
'CjkUnifiedIdeograph9bb6': '\xe9\xae\xb6',
'CjkUnifiedIdeograph9bb1': '\xe9\xae\xb1',
'CjkUnifiedIdeograph9bb0': '\xe9\xae\xb0',
'CjkUnifiedIdeograph9bb3': '\xe9\xae\xb3',
'CjkUnifiedIdeograph9bb2': '\xe9\xae\xb2',
'HangulSyllableKweong': '\xed\x80\x91',
'HangulSyllableSsoess': '\xec\x91\x80',
'CjkUnifiedIdeograph9bb9': '\xe9\xae\xb9',
'TibetanSignRdelNagGcig': '\xe0\xbc\x9d',
'CyrillicSmallLetterShortIWithTail': '\xd2\x8b',
'GlagoliticSmallLetterSmallYusWithTail': '\xe2\xb1\x95',
'CjkUnifiedIdeograph90eb': '\xe9\x83\xab',
'CjkUnifiedIdeograph5ec8': '\xe5\xbb\x88',
'GeorgianSmallLetterHoe': '\xe2\xb4\xa5',
'HangulSyllableJjyeong': '\xec\xaa\x99',
'HangulSyllableSsilt': '\xec\x94\xb5',
'HangulSyllableBeugg': '\xeb\xb8\x8e',
'HangulSyllableJjyeonh': '\xec\xaa\x8a',
'CjkUnifiedIdeograph9bbe': '\xe9\xae\xbe',
'CjkUnifiedIdeograph9bbd': '\xe9\xae\xbd',
'UnicodeCharacter0xda7f': '\xed\xa9\xbf',
'CjkUnifiedIdeograph9bbf': '\xe9\xae\xbf',
'CjkUnifiedIdeograph9bba': '\xe9\xae\xba',
'UnicodeCharacter0xda7a': '\xed\xa9\xba',
'CjkRadicalGrassThree': '\xe2\xbb\x80',
'CjkUnifiedIdeograph5e4e': '\xe5\xb9\x8e',
'GreekSmallLetterNu': '\xce\xbd',
'CjkUnifiedIdeograph90ec': '\xe9\x83\xac',
'SinhalaVowelSignGaettapilla': '\xe0\xb7\x98',
'HangulSyllableTelm': '\xed\x85\x96',
'UnicodeCharacter0xa9c7': '\xea\xa7\x87',
'UnicodeCharacter0xa9c6': '\xea\xa7\x86',
'UnicodeCharacter0xa9c5': '\xea\xa7\x85',
'UnicodeCharacter0xa9c4': '\xea\xa7\x84',
'UnicodeCharacter0xa9c3': '\xea\xa7\x83',
'UnicodeCharacter0xa9c2': '\xea\xa7\x82',
'UnicodeCharacter0xa9c1': '\xea\xa7\x81',
'UnicodeCharacter0xa9c0': '\xea\xa7\x80',
'UnicodeCharacter0xd87e': '\xed\xa1\xbe',
'UnicodeCharacter0xa9c9': '\xea\xa7\x89',
'LepchaLetterA': '\xe1\xb0\xa3',
'HangulSyllableJjyad': '\xec\xa8\x9b',
'HangulSyllableJjyae': '\xec\xa8\xb0',
'HangulSyllableJjyag': '\xec\xa8\x95',
'HangulSyllableJjyab': '\xec\xa8\xa5',
'HangulSyllableJjyac': '\xec\xa8\xab',
'CjkUnifiedIdeograph87e8': '\xe8\x9f\xa8',
'HangulSyllableJjyam': '\xec\xa8\xa4',
'HangulSyllableJjyan': '\xec\xa8\x98',
'HangulSyllableJjyah': '\xec\xa8\xaf',
'BraillePatternDots1678': '\xe2\xa3\xa1',
'HangulSyllableJjyaj': '\xec\xa8\xaa',
'HangulSyllableJjyak': '\xec\xa8\xac',
'UnicodeCharacter0x2ef5': '\xe2\xbb\xb5',
'UnicodeCharacter0x2ef4': '\xe2\xbb\xb4',
'UnicodeCharacter0x2ef7': '\xe2\xbb\xb7',
'UnicodeCharacter0x2ef6': '\xe2\xbb\xb6',
'HangulSyllableJjyap': '\xec\xa8\xae',
'SylotiNagriLetterRro': '\xea\xa0\xa0',
'HangulSyllableJjyas': '\xec\xa8\xa7',
'UnicodeCharacter0xd985': '\xed\xa6\x85',
'UnicodeCharacter0x2ef9': '\xe2\xbb\xb9',
'UnicodeCharacter0x2ef8': '\xe2\xbb\xb8',
'CjkUnifiedIdeograph6901': '\xe6\xa4\x81',
'CjkUnifiedIdeograph6900': '\xe6\xa4\x80',
'CjkUnifiedIdeograph6903': '\xe6\xa4\x83',
'CjkUnifiedIdeograph6902': '\xe6\xa4\x82',
'CjkUnifiedIdeograph6905': '\xe6\xa4\x85',
'CjkUnifiedIdeograph6904': '\xe6\xa4\x84',
'CjkUnifiedIdeograph6907': '\xe6\xa4\x87',
'CjkUnifiedIdeograph6906': '\xe6\xa4\x86',
'CjkUnifiedIdeograph6909': '\xe6\xa4\x89',
'CjkUnifiedIdeograph6908': '\xe6\xa4\x88',
'SymbolForShiftIn': '\xe2\x90\x8f',
'UnicodeCharacter0xfe6': '\xe0\xbf\xa6',
'UnicodeCharacter0xfe7': '\xe0\xbf\xa7',
'UnicodeCharacter0xfe4': '\xe0\xbf\xa4',
'UnicodeCharacter0xfe5': '\xe0\xbf\xa5',
'UnicodeCharacter0xfe2': '\xe0\xbf\xa2',
'UnicodeCharacter0xfe3': '\xe0\xbf\xa3',
'UnicodeCharacter0xfe0': '\xe0\xbf\xa0',
'KannadaLetterKa': '\xe0\xb2\x95',
'GurmukhiLetterNga': '\xe0\xa8\x99',
'UnicodeCharacter0xfe8': '\xe0\xbf\xa8',
'UnicodeCharacter0xfe9': '\xe0\xbf\xa9',
'UnicodeCharacter0x2efe': '\xe2\xbb\xbe',
'UnicodeCharacter0x2efd': '\xe2\xbb\xbd',
'UnicodeCharacter0x2eff': '\xe2\xbb\xbf',
'UnicodeCharacter0x2efa': '\xe2\xbb\xba',
'UnicodeCharacter0x2efc': '\xe2\xbb\xbc',
'UnicodeCharacter0x2efb': '\xe2\xbb\xbb',
'SymbolForSpace': '\xe2\x90\xa0',
'UnicodeCharacter0xd987': '\xed\xa6\x87',
'IdeographicTelegraphSymbolForDayTwentyeight': '\xe3\x8f\xbb',
'CjkUnifiedIdeograph954e': '\xe9\x95\x8e',
'HangulSyllableEogs': '\xec\x96\xb7',
'TibetanSignRdelDkarGnyis': '\xe0\xbc\x9b',
'CombiningCyrillicHundredThousandsSign': '\xd2\x88',
'UnicodeCharacter0xfef': '\xe0\xbf\xaf',
'UnicodeCharacter0xfed': '\xe0\xbf\xad',
'HeavyTeardropspokedPinwheelAsterisk': '\xe2\x9d\x83',
'UnicodeCharacter0xfeb': '\xe0\xbf\xab',
'UnicodeCharacter0xfec': '\xe0\xbf\xac',
'UnicodeCharacter0xfea': '\xe0\xbf\xaa',
'CyrillicSmallLetterOt': '\xd1\xbf',
'HangulSyllableCalb': '\xec\xb0\xb3',
'CjkUnifiedIdeograph80bf': '\xe8\x82\xbf',
'CjkUnifiedIdeograph690a': '\xe6\xa4\x8a',
'CjkUnifiedIdeograph690c': '\xe6\xa4\x8c',
'CjkUnifiedIdeograph690b': '\xe6\xa4\x8b',
'CjkUnifiedIdeograph690e': '\xe6\xa4\x8e',
'CjkUnifiedIdeograph690d': '\xe6\xa4\x8d',
'AplFunctionalSymbolQuadDivide': '\xe2\x8c\xb9',
'UnicodeCharacter0xa4dc': '\xea\x93\x9c',
'UnicodeCharacter0xd90d': '\xed\xa4\x8d',
'CjkUnifiedIdeograph95bd': '\xe9\x96\xbd',
'UnicodeCharacter0xa4db': '\xea\x93\x9b',
'CjkUnifiedIdeograph95be': '\xe9\x96\xbe',
'CjkUnifiedIdeograph80bb': '\xe8\x82\xbb',
'HangulSyllableJjyanh': '\xec\xa8\x9a',
'UnicodeCharacter0xe5d4': '\xee\x97\x94',
'BraillePatternDots2378': '\xe2\xa3\x86',
'HangulSyllableTyih': '\xed\x8b\xaf',
'HangulSyllableJyugs': '\xec\xa5\xaf',
'CjkUnifiedIdeograph95bc': '\xe9\x96\xbc',
'CjkUnifiedIdeograph91b8': '\xe9\x86\xb8',
'CjkUnifiedIdeograph91b9': '\xe9\x86\xb9',
'CjkUnifiedIdeograph91b2': '\xe9\x86\xb2',
'CjkUnifiedIdeograph91b3': '\xe9\x86\xb3',
'CjkUnifiedIdeograph91b0': '\xe9\x86\xb0',
'CjkUnifiedIdeograph91b1': '\xe9\x86\xb1',
'CjkUnifiedIdeograph91b6': '\xe9\x86\xb6',
'CjkUnifiedIdeograph91b7': '\xe9\x86\xb7',
'CjkUnifiedIdeograph91b4': '\xe9\x86\xb4',
'TibetanSignGruCanRgyings': '\xe0\xbe\x8a',
'UnicodeCharacter0xa7bd': '\xea\x9e\xbd',
'ArabicLetterYehWithHamzaAbove': '\xd8\xa6',
'CjkUnifiedIdeograph7004': '\xe7\x80\x84',
'UnicodeCharacter0xf428': '\xef\x90\xa8',
'UnicodeCharacter0xdd80': '\xed\xb6\x80',
'HangulSyllablePibs': '\xed\x95\x8e',
'HangulSyllableOess': '\xec\x9a\x8c',
'UnicodeCharacter0xa7be': '\xea\x9e\xbe',
'HangulSyllableKes': '\xec\xbc\x93',
'HangulSyllableCals': '\xec\xb0\xb4',
'CjkStrokeHz': '\xe3\x87\x95',
'CjkUnifiedIdeograph91bb': '\xe9\x86\xbb',
'CjkUnifiedIdeograph91bc': '\xe9\x86\xbc',
'HangulSyllableMoep': '\xeb\xac\x96',
'MyanmarDigitSix': '\xe1\x81\x86',
'CjkUnifiedIdeograph91bf': '\xe9\x86\xbf',
'CjkStrokeHp': '\xe3\x87\x87',
'CjkUnifiedIdeograph91bd': '\xe9\x86\xbd',
'CjkUnifiedIdeograph91be': '\xe9\x86\xbe',
'HangulSyllableMoej': '\xeb\xac\x92',
'HangulSyllableMoek': '\xeb\xac\x94',
'HangulSyllableMoeh': '\xeb\xac\x97',
'UnicodeCharacter0xdd81': '\xed\xb6\x81',
'HangulSyllableMoen': '\xeb\xac\x80',
'LatinCapitalLetterNj': '\xc7\x8a',
'OlChikiLetterUnn': '\xe1\xb1\xac',
'HangulSyllableMoem': '\xeb\xac\x8c',
'HangulSyllableMoeb': '\xeb\xac\x8d',
'HangulSyllableMoec': '\xeb\xac\x93',
'CjkStrokeHg': '\xe3\x87\x96',
'HangulSyllableMoeg': '\xeb\xab\xbd',
'HangulSyllableMoed': '\xeb\xac\x83',
'KhmerIndependentVowelQii': '\xe1\x9e\xa6',
'GujaratiVowelSignAa': '\xe0\xaa\xbe',
'UnicodeCharacter0xd982': '\xed\xa6\x82',
'GujaratiVowelSignAi': '\xe0\xab\x88',
'UnicodeCharacter0xe9d3': '\xee\xa7\x93',
'CjkUnifiedIdeograph64fc': '\xe6\x93\xbc',
'UnicodeCharacter0xdc54': '\xed\xb1\x94',
'GujaratiVowelSignAu': '\xe0\xab\x8c',
'UnicodeCharacter0xe9d1': '\xee\xa7\x91',
'HangulSyllableCaelb': '\xec\xb1\x8f',
'UnicodeCharacter0xe2d8': '\xee\x8b\x98',
'UnicodeCharacter0xe22c': '\xee\x88\xac',
'UnicodeCharacter0xe22b': '\xee\x88\xab',
'UnicodeCharacter0xe22a': '\xee\x88\xaa',
'CanadianSyllabicsCarrierLhu': '\xe1\x98\xae',
'UnicodeCharacter0xef86': '\xee\xbe\x86',
'HangulSyllableAelm': '\xec\x95\xaa',
'UnicodeCharacter0xe22e': '\xee\x88\xae',
'UnicodeCharacter0xdd83': '\xed\xb6\x83',
'CanadianSyllabicsCarrierLho': '\xe1\x98\xaf',
'WhiteFourPointedStar': '\xe2\x9c\xa7',
'CanadianSyllabicsCarrierLhi': '\xe1\x98\xb2',
'CanadianSyllabicsCarrierLhe': '\xe1\x98\xb0',
'CjkCompatibilityIdeographf93f': '\xef\xa4\xbf',
'CanadianSyllabicsCarrierLha': '\xe1\x98\xb3',
'CjkUnifiedIdeograph4f9f': '\xe4\xbe\x9f',
'CjkUnifiedIdeograph4f9e': '\xe4\xbe\x9e',
'CjkUnifiedIdeograph4f9d': '\xe4\xbe\x9d',
'CjkUnifiedIdeograph4f9c': '\xe4\xbe\x9c',
'CjkUnifiedIdeograph4f9b': '\xe4\xbe\x9b',
'CjkUnifiedIdeograph4f9a': '\xe4\xbe\x9a',
'GreekCapitalDottedLunateSigmaSymbol': '\xcf\xbe',
'HangulSyllableAelh': '\xec\x95\xaf',
'HangulSyllableNeugs': '\xeb\x8a\x93',
'UnicodeCharacter0xdd84': '\xed\xb6\x84',
'UnicodeCharacter0xd90b': '\xed\xa4\x8b',
'HangulSyllableAelg': '\xec\x95\xa9',
'GreekDasiaAndOxia': '\xe1\xbf\x9e',
'HangulSyllableNeugg': '\xeb\x8a\x92',
'UnicodeCharacter0xe2d2': '\xee\x8b\x92',
'HangulSyllableGyub': '\xea\xb7\xad',
'HangulSyllableGyuc': '\xea\xb7\xb3',
'LeftParenthesisUpperHook': '\xe2\x8e\x9b',
'HangulSyllableNwap': '\xeb\x86\xae',
'ArabicLetterBehWithThreeDotsHorizontallyBelow': '\xdd\x90',
'HangulSyllableGyug': '\xea\xb7\x9d',
'HangulSyllableGyud': '\xea\xb7\xa3',
'HangulSyllableNwat': '\xeb\x86\xad',
'HangulSyllableGyuj': '\xea\xb7\xb2',
'HangulSyllableGyuk': '\xea\xb7\xb4',
'HangulSyllableGyuh': '\xea\xb7\xb7',
'HangulSyllableCyolb': '\xec\xb6\x83',
'HangulSyllableGyun': '\xea\xb7\xa0',
'ArabicLetterJeem': '\xd8\xac',
'HangulSyllableGyul': '\xea\xb7\xa4',
'HangulSyllableGyum': '\xea\xb7\xac',
'HangulSyllableNwac': '\xeb\x86\xab',
'HangulSyllableGyus': '\xea\xb7\xaf',
'HangulSyllableGyup': '\xea\xb7\xb6',
'UnicodeCharacter0x2cf0': '\xe2\xb3\xb0',
'GlagoliticCapitalLetterSmallYus': '\xe2\xb0\xa4',
'HangulSyllableRoss': '\xeb\xa1\xb0',
'HangulSyllableGyut': '\xea\xb7\xb5',
'HangulSyllableNwad': '\xeb\x86\x9b',
'HangulSyllableNwak': '\xeb\x86\xac',
'HangulSyllableNwaj': '\xeb\x86\xaa',
'HangulSyllableCyols': '\xec\xb6\x84',
'HangulSyllableNwah': '\xeb\x86\xaf',
'UnicodeCharacter0xe227': '\xee\x88\xa7',
'HangulSyllableNwan': '\xeb\x86\x98',
'HangulSyllableNwam': '\xeb\x86\xa4',
'HangulSyllableNwal': '\xeb\x86\x9c',
'MyanmarLetterUu': '\xe1\x80\xa6',
'UnicodeCharacter0xe8bc': '\xee\xa2\xbc',
'UnicodeCharacter0xeb3e': '\xee\xac\xbe',
'ArabicLigatureYehWithAlefMaksuraFinalForm': '\xef\xb2\x95',
'CjkUnifiedIdeograph4f97': '\xe4\xbe\x97',
'CjkUnifiedIdeograph4f96': '\xe4\xbe\x96',
'CjkUnifiedIdeograph4f95': '\xe4\xbe\x95',
'CjkUnifiedIdeograph4f94': '\xe4\xbe\x94',
'CjkUnifiedIdeograph4f93': '\xe4\xbe\x93',
'CjkUnifiedIdeograph4f92': '\xe4\xbe\x92',
'CjkUnifiedIdeograph4f91': '\xe4\xbe\x91',
'CjkUnifiedIdeograph4f90': '\xe4\xbe\x90',
'DevanagariLetterBa': '\xe0\xa4\xac',
'CjkUnifiedIdeograph4f99': '\xe4\xbe\x99',
'CjkUnifiedIdeograph4f98': '\xe4\xbe\x98',
'CjkCompatibilityIdeographfa41': '\xef\xa9\x81',
'UnicodeCharacter0xa4d7': '\xea\x93\x97',
'CanadianSyllabicsNunavikHi': '\xe1\x95\xb5',
'CjkCompatibilityIdeographfa42': '\xef\xa9\x82',
'CanadianSyllabicsNunavikHo': '\xe1\x95\xb7',
'GreekSmallLetterUpsilonWithOxia': '\xe1\xbd\xbb',
'CjkCompatibilityIdeographfa47': '\xef\xa9\x87',
'UnicodeCharacter0xa4d6': '\xea\x93\x96',
'CjkCompatibilityIdeographfa49': '\xef\xa9\x89',
'CjkCompatibilityIdeographfa48': '\xef\xa9\x88',
'CanadianSyllabicsNunavikHa': '\xe1\x95\xb9',
'UnicodeCharacter0xa4d5': '\xea\x93\x95',
'CanadianSyllabicsNunavikHe': '\xe1\x95\xb4',
'UnicodeCharacter0xa4d4': '\xea\x93\x94',
'LatinCapitalLetterMWithAcute': '\xe1\xb8\xbe',
'UnicodeCharacter0xa4d3': '\xea\x93\x93',
'ParenthesizedIdeographSpecial': '\xe3\x88\xb5',
'BraillePatternDots234578': '\xe2\xa3\x9e',
'UnicodeCharacter0xa4d2': '\xea\x93\x92',
'CombiningCaron': '\xcc\x8c',
'HangulSyllableAelt': '\xec\x95\xad',
'UnicodeCharacter0xe4b9': '\xee\x92\xb9',
'UnicodeCharacter0xa4d1': '\xea\x93\x91',
'UnicodeCharacter0xf048': '\xef\x81\x88',
'UnicodeCharacter0xf049': '\xef\x81\x89',
'UnicodeCharacter0xe4b5': '\xee\x92\xb5',
'UnicodeCharacter0xe4b4': '\xee\x92\xb4',
'UnicodeCharacter0xe4b7': '\xee\x92\xb7',
'UnicodeCharacter0xa4d0': '\xea\x93\x90',
'HangulSyllableRwas': '\xeb\xa2\x8b',
'UnicodeCharacter0xe4b0': '\xee\x92\xb0',
'ArabicLetterAlefWithExtendedArabicindicDigitTwoAbove': '\xdd\xb3',
'HangulSyllableRwap': '\xeb\xa2\x92',
'CjkUnifiedIdeograph4e21': '\xe4\xb8\xa1',
'CjkUnifiedIdeograph4e20': '\xe4\xb8\xa0',
'CjkUnifiedIdeograph4e23': '\xe4\xb8\xa3',
'CjkUnifiedIdeograph4e22': '\xe4\xb8\xa2',
'CjkUnifiedIdeograph4e25': '\xe4\xb8\xa5',
'CjkUnifiedIdeograph4e24': '\xe4\xb8\xa4',
'CjkUnifiedIdeograph4e27': '\xe4\xb8\xa7',
'CjkUnifiedIdeograph4e26': '\xe4\xb8\xa6',
'CjkUnifiedIdeograph4e29': '\xe4\xb8\xa9',
'CjkUnifiedIdeograph4e28': '\xe4\xb8\xa8',
'HangulSyllableRwae': '\xeb\xa2\x94',
'HangulSyllableRwad': '\xeb\xa1\xbf',
'HangulSyllableRwac': '\xeb\xa2\x8f',
'HangulSyllableRwab': '\xeb\xa2\x89',
'UnicodeCharacter0xf59f': '\xef\x96\x9f',
'UnicodeCharacter0xed53': '\xee\xb5\x93',
'UnicodeCharacter0xdf54': '\xed\xbd\x94',
'UnicodeCharacter0xd862': '\xed\xa1\xa2',
'CjkUnifiedIdeograph80b8': '\xe8\x82\xb8',
'UnicodeCharacter0xe946': '\xee\xa5\x86',
'SquareKoruna': '\xe3\x8c\x9d',
'UnicodeCharacter0xdf57': '\xed\xbd\x97',
'UnicodeCharacter0xa60': '\xe0\xa9\xa0',
'UnicodeCharacter0xa61': '\xe0\xa9\xa1',
'UnicodeCharacter0xa62': '\xe0\xa9\xa2',
'UnicodeCharacter0xa63': '\xe0\xa9\xa3',
'UnicodeCharacter0xa64': '\xe0\xa9\xa4',
'UnicodeCharacter0xa65': '\xe0\xa9\xa5',
'CjkCompatibilityIdeographfa4f': '\xef\xa9\x8f',
'RightParenthesisUpperHook': '\xe2\x8e\x9e',
'YiSyllableNgiex': '\xea\x89\x9d',
'CyrillicCapitalLetterBashkirKa': '\xd2\xa0',
'UnicodeCharacter0xdf50': '\xed\xbd\x90',
'CjkUnifiedIdeograph4e2a': '\xe4\xb8\xaa',
'UnicodeCharacter0xf599': '\xef\x96\x99',
'CjkUnifiedIdeograph4e2c': '\xe4\xb8\xac',
'CjkUnifiedIdeograph4e2b': '\xe4\xb8\xab',
'CjkUnifiedIdeograph4e2e': '\xe4\xb8\xae',
'CjkUnifiedIdeograph4e2d': '\xe4\xb8\xad',
'CjkUnifiedIdeograph4e2f': '\xe4\xb8\xaf',
'UnicodeCharacter0xf590': '\xef\x96\x90',
'UnicodeCharacter0xf591': '\xef\x96\x91',
'LessthanClosedByCurveAboveSlantedEqual': '\xe2\xaa\xa8',
'HangulSyllableSuss': '\xec\x88\xac',
'UnicodeCharacter0xf594': '\xef\x96\x94',
'UnicodeCharacter0xf595': '\xef\x96\x95',
'UnicodeCharacter0xf596': '\xef\x96\x96',
'UnicodeCharacter0xf597': '\xef\x96\x97',
'UnicodeCharacter0xe62f': '\xee\x98\xaf',
'UnicodeCharacter0xe4be': '\xee\x92\xbe',
'UnicodeCharacter0xe4bd': '\xee\x92\xbd',
'UnicodeCharacter0xf04d': '\xef\x81\x8d',
'UnicodeCharacter0xe4bf': '\xee\x92\xbf',
'UnicodeCharacter0xe4ba': '\xee\x92\xba',
'HangulSyllableGoess': '\xea\xb5\x88',
'UnicodeCharacter0xe4bc': '\xee\x92\xbc',
'UnicodeCharacter0xe4bb': '\xee\x92\xbb',
'HangulSyllableKeolb': '\xec\xbb\xaf',
'CombiningCyrillicLetterPe': '\xe2\xb7\xab',
'ThaiCharacterChoChoe': '\xe0\xb8\x8c',
'UnicodeCharacter0xdf58': '\xed\xbd\x98',
'YiSyllableJjop': '\xea\x90\xa6',
'YiSyllableJjot': '\xea\x90\xa3',
'CjkUnifiedIdeograph9f09': '\xe9\xbc\x89',
'CjkUnifiedIdeograph9f08': '\xe9\xbc\x88',
'CjkUnifiedIdeograph9f07': '\xe9\xbc\x87',
'KangxiRadicalHemp': '\xe2\xbf\x87',
'CjkUnifiedIdeograph9f05': '\xe9\xbc\x85',
'CjkUnifiedIdeograph9f04': '\xe9\xbc\x84',
'CjkUnifiedIdeograph9f03': '\xe9\xbc\x83',
'CjkUnifiedIdeograph9f02': '\xe9\xbc\x82',
'CjkUnifiedIdeograph9f01': '\xe9\xbc\x81',
'CjkUnifiedIdeograph9f00': '\xe9\xbc\x80',
'CjkUnifiedIdeograph49fa': '\xe4\xa7\xba',
'CjkUnifiedIdeograph9c1e': '\xe9\xb0\x9e',
'CjkUnifiedIdeograph49fc': '\xe4\xa7\xbc',
'CjkUnifiedIdeograph49fb': '\xe4\xa7\xbb',
'CjkUnifiedIdeograph49fe': '\xe4\xa7\xbe',
'CjkUnifiedIdeograph49fd': '\xe4\xa7\xbd',
'CjkUnifiedIdeograph49ff': '\xe4\xa7\xbf',
'CjkUnifiedIdeograph4e67': '\xe4\xb9\xa7',
'MyanmarSignVirama': '\xe1\x80\xb9',
'ArabicLetterNoonMedialForm': '\xef\xbb\xa8',
'CjkUnifiedIdeograph9c1f': '\xe9\xb0\x9f',
'pd': '\xe2\x88\x82',
'CjkUnifiedIdeograph9c1a': '\xe9\xb0\x9a',
'ph': '\xcf\x95',
'UnicodeCharacter0xdc7c': '\xed\xb1\xbc',
'CjkUnifiedIdeograph9c1c': '\xe9\xb0\x9c',
'f': '\xcf\x95',
'HangulSyllableCyeogg': '\xec\xb3\x92',
'CjkUnifiedIdeograph9f0f': '\xe9\xbc\x8f',
'CjkUnifiedIdeograph9f0e': '\xe9\xbc\x8e',
'CjkUnifiedIdeograph9f0d': '\xe9\xbc\x8d',
'CjkUnifiedIdeograph9f0c': '\xe9\xbc\x8c',
'CjkUnifiedIdeograph9f0b': '\xe9\xbc\x8b',
'CjkUnifiedIdeograph9f0a': '\xe9\xbc\x8a',
'UnicodeCharacter0xe8b5': '\xee\xa2\xb5',
'HangulSyllableSigs': '\xec\x8b\x9f',
'UpwardsWhiteDoubleArrow': '\xe2\x87\xae',
'TibetanMarkCheMgo': '\xe0\xbc\xb8',
'HiraganaLetterZe': '\xe3\x81\x9c',
'UnicodeCharacter0xe947': '\xee\xa5\x87',
'VaiSyllableE': '\xea\x97\xa1',
'HangulSyllableSsya': '\xec\x8c\xb0',
'HangulSyllableSsyo': '\xec\x91\x88',
'HangulSyllableSsyi': '\xec\x94\x8c',
'CjkUnifiedIdeograph49f1': '\xe4\xa7\xb1',
'CjkUnifiedIdeograph49f0': '\xe4\xa7\xb0',
'CjkUnifiedIdeograph49f3': '\xe4\xa7\xb3',
'CjkUnifiedIdeograph49f2': '\xe4\xa7\xb2',
'CjkUnifiedIdeograph49f5': '\xe4\xa7\xb5',
'CjkUnifiedIdeograph49f4': '\xe4\xa7\xb4',
'CjkUnifiedIdeograph49f7': '\xe4\xa7\xb7',
'CjkUnifiedIdeograph49f6': '\xe4\xa7\xb6',
'CjkUnifiedIdeograph49f9': '\xe4\xa7\xb9',
'CjkUnifiedIdeograph49f8': '\xe4\xa7\xb8',
'LatinSmallLetterTurnedHWithFishhook': '\xca\xae',
'LatinCapitalLetterWWithDotBelow': '\xe1\xba\x88',
'CjkUnifiedIdeograph4679': '\xe4\x99\xb9',
'CjkUnifiedIdeograph4678': '\xe4\x99\xb8',
'LatinSmallLetterAeWithMacron': '\xc7\xa3',
'CjkUnifiedIdeograph4675': '\xe4\x99\xb5',
'CjkUnifiedIdeograph4674': '\xe4\x99\xb4',
'CjkUnifiedIdeograph4677': '\xe4\x99\xb7',
'CjkUnifiedIdeograph4676': '\xe4\x99\xb6',
'CjkUnifiedIdeograph4671': '\xe4\x99\xb1',
'CjkUnifiedIdeograph4670': '\xe4\x99\xb0',
'CjkUnifiedIdeograph4673': '\xe4\x99\xb3',
'CjkUnifiedIdeograph4672': '\xe4\x99\xb2',
'BlackRightpointingSmallTriangle': '\xe2\x96\xb8',
'DiamondWithLeftHalfBlack': '\xe2\xac\x96',
'UnicodeCharacter0x181a': '\xe1\xa0\x9a',
'UnicodeCharacter0x181b': '\xe1\xa0\x9b',
'UnicodeCharacter0x181c': '\xe1\xa0\x9c',
'UnicodeCharacter0x181d': '\xe1\xa0\x9d',
'UnicodeCharacter0x181e': '\xe1\xa0\x9e',
'UnicodeCharacter0x181f': '\xe1\xa0\x9f',
'ArabicLigatureTahWithAlefMaksuraFinalForm': '\xef\xb4\x91',
'HangulSyllableNos': '\xeb\x86\x8b',
'UnicodeCharacter0xed61': '\xee\xb5\xa1',
'HangulSyllableNop': '\xeb\x86\x92',
'HangulSyllableKyad': '\xec\xba\xb3',
'HangulSyllableNot': '\xeb\x86\x91',
'HangulSyllableKyae': '\xec\xbb\x88',
'UnicodeCharacter0xed63': '\xee\xb5\xa3',
'HangulSyllableNoc': '\xeb\x86\x8f',
'HangulSyllableNob': '\xeb\x86\x89',
'UnicodeCharacter0xed62': '\xee\xb5\xa2',
'CyrillicCapitalLetterAbkhasianCheWithDescender': '\xd2\xbe',
'CjkUnifiedIdeograph7ff9': '\xe7\xbf\xb9',
'CjkUnifiedIdeograph7ff8': '\xe7\xbf\xb8',
'CjkUnifiedIdeograph7ff7': '\xe7\xbf\xb7',
'CjkUnifiedIdeograph7ff6': '\xe7\xbf\xb6',
'CjkUnifiedIdeograph7ff5': '\xe7\xbf\xb5',
'CjkUnifiedIdeograph7ff4': '\xe7\xbf\xb4',
'CjkUnifiedIdeograph7ff3': '\xe7\xbf\xb3',
'CjkUnifiedIdeograph7ff2': '\xe7\xbf\xb2',
'CjkUnifiedIdeograph7ff1': '\xe7\xbf\xb1',
'MongolianLetterEe': '\xe1\xa0\xa7',
'HangulSyllableGwilm': '\xea\xb7\x8a',
'HangulSyllableByaj': '\xeb\xb1\xa2',
'HangulSyllableGwilh': '\xea\xb7\x8f',
'HangulSyllableSseus': '\xec\x94\x83',
'HangulSyllableKyan': '\xec\xba\xb0',
'HangulSyllableGwilg': '\xea\xb7\x89',
'OriyaLetterJha': '\xe0\xac\x9d',
'HangulSyllableGwilb': '\xea\xb7\x8b',
'UnicodeCharacter0xed69': '\xee\xb5\xa9',
'HangulSyllableSyinj': '\xec\x8b\x85',
'BoxDrawingsUpHeavyAndLeftDownLight': '\xe2\x94\xa6',
'HangulSyllableKyal': '\xec\xba\xb4',
'UnicodeCharacter0xed68': '\xee\xb5\xa8',
'CjkUnifiedIdeograph467e': '\xe4\x99\xbe',
'CjkUnifiedIdeograph467d': '\xe4\x99\xbd',
'CjkUnifiedIdeograph467f': '\xe4\x99\xbf',
'CjkUnifiedIdeograph467a': '\xe4\x99\xba',
'HangulSyllableGwilp': '\xea\xb7\x8e',
'CjkUnifiedIdeograph467c': '\xe4\x99\xbc',
'CjkUnifiedIdeograph467b': '\xe4\x99\xbb',
'HangulSyllableReog': '\xeb\x9f\xad',
'UnicodeCharacter0xe76f': '\xee\x9d\xaf',
'CjkUnifiedIdeograph7fff': '\xe7\xbf\xbf',
'SinhalaLetterHayanna': '\xe0\xb7\x84',
'CjkUnifiedIdeograph7ffd': '\xe7\xbf\xbd',
'CjkUnifiedIdeograph7ffc': '\xe7\xbf\xbc',
'CjkUnifiedIdeograph7ffb': '\xe7\xbf\xbb',
'CjkUnifiedIdeograph7ffa': '\xe7\xbf\xba',
'UnicodeCharacter0xe76a': '\xee\x9d\xaa',
'CjkUnifiedIdeograph6cfd': '\xe6\xb3\xbd',
'HangulSyllableReod': '\xeb\x9f\xb3',
'MyanmarConsonantSignShanMedialWa': '\xe1\x82\x82',
'HangulSyllableReoc': '\xeb\xa0\x83',
'CjkUnifiedIdeograph8a4f': '\xe8\xa9\x8f',
'HangulSyllableReob': '\xeb\x9f\xbd',
'UnicodeCharacter0xe484': '\xee\x92\x84',
'CjkUnifiedIdeograph9618': '\xe9\x98\x98',
'CjkUnifiedIdeograph9619': '\xe9\x98\x99',
'HangulSyllablePeonj': '\xed\x8e\x81',
'CjkUnifiedIdeograph9614': '\xe9\x98\x94',
'CjkUnifiedIdeograph9615': '\xe9\x98\x95',
'WhiteChessBishop': '\xe2\x99\x97',
'CjkUnifiedIdeograph9617': '\xe9\x98\x97',
'CjkUnifiedIdeograph9610': '\xe9\x98\x90',
'CjkUnifiedIdeograph9611': '\xe9\x98\x91',
'CjkUnifiedIdeograph9612': '\xe9\x98\x92',
'CjkUnifiedIdeograph9613': '\xe9\x98\x93',
'HangulSyllableDdubs': '\xeb\x9a\xae',
'HangulSyllableSying': '\xec\x8b\x95',
'SouthWestBlackArrow': '\xe2\xac\x8b',
'EthiopicSyllableSebatbeitBwa': '\xe1\x8e\x84',
'UnicodeCharacter0xe480': '\xee\x92\x80',
'ArabicLetterFehWithThreeDotsBelow': '\xda\xa5',
'KhmerIndependentVowelQuuv': '\xe1\x9e\xaa',
'UnicodeCharacter0xea46': '\xee\xa9\x86',
'UnicodeCharacter0xdf46': '\xed\xbd\x86',
'HangulSyllableDwang': '\xeb\x8f\xb5',
'ArabicLigatureLamWithMeemMedialForm': '\xef\xb3\xad',
'UnicodeCharacter0x11fd': '\xe1\x87\xbd',
'UnicodeCharacter0x11fe': '\xe1\x87\xbe',
'UnicodeCharacter0x11ff': '\xe1\x87\xbf',
'UnicodeCharacter0xf863': '\xef\xa1\xa3',
'CjkUnifiedIdeograph5108': '\xe5\x84\x88',
'CjkUnifiedIdeograph5109': '\xe5\x84\x89',
'UnicodeCharacter0x11fb': '\xe1\x87\xbb',
'UnicodeCharacter0x11fc': '\xe1\x87\xbc',
'CjkUnifiedIdeograph5104': '\xe5\x84\x84',
'CjkUnifiedIdeograph5105': '\xe5\x84\x85',
'CjkUnifiedIdeograph5106': '\xe5\x84\x86',
'CjkUnifiedIdeograph5107': '\xe5\x84\x87',
'CjkUnifiedIdeograph5100': '\xe5\x84\x80',
'CjkUnifiedIdeograph5101': '\xe5\x84\x81',
'CjkUnifiedIdeograph5102': '\xe5\x84\x82',
'CjkUnifiedIdeograph5103': '\xe5\x84\x83',
'KangxiRadicalBoltOfCloth': '\xe2\xbd\xa6',
'YiSyllableFo': '\xea\x83\x96',
'YiSyllableFi': '\xea\x83\x8f',
'UnicodeCharacter0xf43d': '\xef\x90\xbd',
'CjkUnifiedIdeograph9c14': '\xe9\xb0\x94',
'CjkUnifiedIdeograph40c4': '\xe4\x83\x84',
'YiSyllableFa': '\xea\x83\x93',
'HebrewAccentEtnahta': '\xd6\x91',
'DevanagariLetterPha': '\xe0\xa4\xab',
'YiSyllableFy': '\xea\x83\xa0',
'CjkUnifiedIdeograph961d': '\xe9\x98\x9d',
'VerticalTilde': '\xe2\xb8\xaf',
'CjkUnifiedIdeograph961f': '\xe9\x98\x9f',
'CjkUnifiedIdeograph7e3e': '\xe7\xb8\xbe',
'CjkUnifiedIdeograph961b': '\xe9\x98\x9b',
'CjkUnifiedIdeograph961c': '\xe9\x98\x9c',
'HangulSyllableDelp': '\xeb\x8d\xbe',
'UnicodeCharacter0xf86b': '\xef\xa1\xab',
'CjkUnifiedIdeograph40c8': '\xe4\x83\x88',
'UnicodeCharacter0xf86d': '\xef\xa1\xad',
'CjkUnifiedIdeograph9c13': '\xe9\xb0\x93',
'UnicodeCharacter0xf86f': '\xef\xa1\xaf',
'CjkUnifiedIdeograph510d': '\xe5\x84\x8d',
'CjkUnifiedIdeograph510e': '\xe5\x84\x8e',
'CjkUnifiedIdeograph510f': '\xe5\x84\x8f',
'CjkUnifiedIdeograph9c12': '\xe9\xb0\x92',
'HangulSyllableBaenh': '\xeb\xb0\xb6',
'CjkUnifiedIdeograph510a': '\xe5\x84\x8a',
'CjkUnifiedIdeograph510b': '\xe5\x84\x8b',
'CjkUnifiedIdeograph510c': '\xe5\x84\x8c',
'GlagoliticCapitalLetterHeru': '\xe2\xb0\x98',
'HangulSyllableBenh': '\xeb\xb2\xa6',
'LatinCapitalLetterMiddlewelshLl': '\xe1\xbb\xba',
'HangulSyllableDdyeolt': '\xeb\x97\xa5',
'CjkUnifiedIdeograph9c19': '\xe9\xb0\x99',
'UnicodeCharacter0xe945': '\xee\xa5\x85',
'CjkUnifiedIdeograph9c18': '\xe9\xb0\x98',
'GeorgianSmallLetterChar': '\xe2\xb4\x9d',
'CjkUnifiedIdeograph846d': '\xe8\x91\xad',
'UnicodeCharacter0xe057': '\xee\x81\x97',
'CjkUnifiedIdeograph846e': '\xe8\x91\xae',
'UnicodeCharacter0xea42': '\xee\xa9\x82',
'CjkUnifiedIdeograph3469': '\xe3\x91\xa9',
'CjkUnifiedIdeograph3468': '\xe3\x91\xa8',
'GeorgianLetterWe': '\xe1\x83\xb3',
'UnicodeCharacter0xdc7a': '\xed\xb1\xba',
'UnicodeCharacter0xdc7f': '\xed\xb1\xbf',
'HangulSyllableDdeos': '\xeb\x96\xb3',
'HangulSyllableDdeop': '\xeb\x96\xba',
'UnicodeCharacter0xdc7e': '\xed\xb1\xbe',
'CjkUnifiedIdeograph3461': '\xe3\x91\xa1',
'CjkUnifiedIdeograph3460': '\xe3\x91\xa0',
'CjkUnifiedIdeograph3463': '\xe3\x91\xa3',
'CjkUnifiedIdeograph3462': '\xe3\x91\xa2',
'CjkUnifiedIdeograph3465': '\xe3\x91\xa5',
'CjkUnifiedIdeograph3464': '\xe3\x91\xa4',
'CjkUnifiedIdeograph3467': '\xe3\x91\xa7',
'CjkUnifiedIdeograph3466': '\xe3\x91\xa6',
'SupersetOfOrEqualTo': '\xe2\x8a\x87',
'HangulSyllableDdeod': '\xeb\x96\xa7',
'RightshadedWhiteRightwardsArrow': '\xe2\x9e\xa9',
'CjkUnifiedIdeograph5b4b': '\xe5\xad\x8b',
'CjkUnifiedIdeograph846b': '\xe8\x91\xab',
'HangulSyllableDdeon': '\xeb\x96\xa4',
'UnicodeCharacter0xea43': '\xee\xa9\x83',
'HangulSyllableDdeol': '\xeb\x96\xa8',
'HangulSyllableDdeom': '\xeb\x96\xb0',
'HangulSyllableDdeoj': '\xeb\x96\xb6',
'HangulSyllableDdeok': '\xeb\x96\xb8',
'HangulSyllableDdeoh': '\xeb\x96\xbb',
'HangulSyllableGgweolm': '\xea\xbf\x9e',
'HangulSyllableGgweolh': '\xea\xbf\xa3',
'HangulSyllableCoet': '\xec\xb5\xb5',
'LatinLetterYr': '\xc6\xa6',
'UnicodeCharacter0xe628': '\xee\x98\xa8',
'HangulSyllableCoep': '\xec\xb5\xb6',
'HangulSyllableGgeugg': '\xeb\x81\x86',
'GreekCapitalLetterEtaWithPsili': '\xe1\xbc\xa8',
'HangulSyllableCoel': '\xec\xb5\xa4',
'HangulSyllableCoem': '\xec\xb5\xac',
'HangulSyllableCoen': '\xec\xb5\xa0',
'HangulSyllableCoeh': '\xec\xb5\xb7',
'HangulSyllableCoej': '\xec\xb5\xb2',
'HangulSyllableCoek': '\xec\xb5\xb4',
'HangulSyllableCoed': '\xec\xb5\xa3',
'HangulSyllableGgweolt': '\xea\xbf\xa1',
'HangulSyllableCoeg': '\xec\xb5\x9d',
'HangulSyllableGgweolp': '\xea\xbf\xa2',
'HangulSyllableGgweols': '\xea\xbf\xa0',
'HangulSyllableCoec': '\xec\xb5\xb3',
'LatinCapitalLetterThornWithStroke': '\xea\x9d\xa4',
'UnicodeCharacter0xed8f': '\xee\xb6\x8f',
'TibetanDigitEight': '\xe0\xbc\xa8',
'LatinSmallLetterXWithPalatalHook': '\xe1\xb6\x8d',
'EthiopicSyllableGgaa': '\xe1\x8c\x9b',
'HangulSyllableNeuc': '\xeb\x8a\xa7',
'UnicodeCharacter0xe622': '\xee\x98\xa2',
'UnicodeCharacter0xdc72': '\xed\xb1\xb2',
'UnicodeCharacter0xdc73': '\xed\xb1\xb3',
'UnicodeCharacter0xdc70': '\xed\xb1\xb0',
'HangulSyllableNeub': '\xeb\x8a\xa1',
'UnicodeCharacter0xdc76': '\xed\xb1\xb6',
'UnicodeCharacter0xdc77': '\xed\xb1\xb7',
'UnicodeCharacter0xdc74': '\xed\xb1\xb4',
'UnicodeCharacter0xdc75': '\xed\xb1\xb5',
'CjkUnifiedIdeograph346a': '\xe3\x91\xaa',
'CjkUnifiedIdeograph346c': '\xe3\x91\xac',
'CjkUnifiedIdeograph346b': '\xe3\x91\xab',
'CjkUnifiedIdeograph346e': '\xe3\x91\xae',
'CjkUnifiedIdeograph346d': '\xe3\x91\xad',
'CjkUnifiedIdeograph3da5': '\xe3\xb6\xa5',
'CjkUnifiedIdeograph7618': '\xe7\x98\x98',
'CjkUnifiedIdeograph7619': '\xe7\x98\x99',
'CjkUnifiedIdeograph3da6': '\xe3\xb6\xa6',
'CjkUnifiedIdeograph7612': '\xe7\x98\x92',
'CjkUnifiedIdeograph7613': '\xe7\x98\x93',
'CjkUnifiedIdeograph7610': '\xe7\x98\x90',
'CjkUnifiedIdeograph3da7': '\xe3\xb6\xa7',
'CjkUnifiedIdeograph7616': '\xe7\x98\x96',
'CjkUnifiedIdeograph7617': '\xe7\x98\x97',
'CjkUnifiedIdeograph7614': '\xe7\x98\x94',
'CjkUnifiedIdeograph7615': '\xe7\x98\x95',
'CjkUnifiedIdeograph3da8': '\xe3\xb6\xa8',
'HangulSyllableCwess': '\xec\xb7\xa0',
'CjkUnifiedIdeograph97c9': '\xe9\x9f\x89',
'CjkUnifiedIdeograph3da9': '\xe3\xb6\xa9',
'CjkUnifiedIdeograph81dc': '\xe8\x87\x9c',
'CjkUnifiedIdeograph81db': '\xe8\x87\x9b',
'CjkUnifiedIdeograph81da': '\xe8\x87\x9a',
'CjkUnifiedIdeograph97c4': '\xe9\x9f\x84',
'CjkUnifiedIdeograph97c3': '\xe9\x9f\x83',
'CjkUnifiedIdeograph81df': '\xe8\x87\x9f',
'CjkUnifiedIdeograph81de': '\xe8\x87\x9e',
'CjkUnifiedIdeograph81dd': '\xe8\x87\x9d',
'CjkUnifiedIdeograph8c29': '\xe8\xb0\xa9',
'CjkUnifiedIdeograph8c28': '\xe8\xb0\xa8',
'i': '\xce\xb9',
'YiSyllablePurx': '\xea\x81\x8e',
'HangulSyllablePunh': '\xed\x91\xbe',
'CjkUnifiedIdeograph8c23': '\xe8\xb0\xa3',
'CjkUnifiedIdeograph8c22': '\xe8\xb0\xa2',
'CjkUnifiedIdeograph8c21': '\xe8\xb0\xa1',
'CjkUnifiedIdeograph8c20': '\xe8\xb0\xa0',
'CjkUnifiedIdeograph8c27': '\xe8\xb0\xa7',
'CjkUnifiedIdeograph8c26': '\xe8\xb0\xa6',
'CjkUnifiedIdeograph8c25': '\xe8\xb0\xa5',
'CjkUnifiedIdeograph8c24': '\xe8\xb0\xa4',
'HangulSyllableSseoss': '\xec\x8d\xbc',
'HangulSyllableRyaeh': '\xeb\x9f\xab',
'CjkUnifiedIdeograph61bf': '\xe6\x86\xbf',
'CjkUnifiedIdeograph9438': '\xe9\x90\xb8',
'PresentationFormForVerticalLeftWhiteCornerBracket': '\xef\xb9\x83',
'KangxiRadicalLeek': '\xe2\xbe\xb2',
'UnicodeCharacter0xe9a7': '\xee\xa6\xa7',
'CjkUnifiedIdeograph81d9': '\xe8\x87\x99',
'CjkUnifiedIdeograph81d8': '\xe8\x87\x98',
'YiSyllableSyx': '\xea\x8c\xa5',
'CjkUnifiedIdeograph81d3': '\xe8\x87\x93',
'CjkUnifiedIdeograph81d2': '\xe8\x87\x92',
'CjkUnifiedIdeograph81d1': '\xe8\x87\x91',
'CjkUnifiedIdeograph81d0': '\xe8\x87\x90',
'CjkUnifiedIdeograph81d7': '\xe8\x87\x97',
'CjkUnifiedIdeograph81d6': '\xe8\x87\x96',
'CjkUnifiedIdeograph81d5': '\xe8\x87\x95',
'CjkUnifiedIdeograph81d4': '\xe8\x87\x94',
'LatinSmallLetterTurnedRWithTail': '\xe2\xb1\xb9',
'CjkUnifiedIdeograph761c': '\xe7\x98\x9c',
'CjkUnifiedIdeograph761a': '\xe7\x98\x9a',
'CjkUnifiedIdeograph761f': '\xe7\x98\x9f',
'CjkUnifiedIdeograph761d': '\xe7\x98\x9d',
'CjkUnifiedIdeograph761e': '\xe7\x98\x9e',
'ArabicLigatureSheenWithHahIsolatedForm': '\xef\xb4\x8a',
'ArabicLigatureJeemWithMeemIsolatedForm': '\xef\xb0\x96',
'CjkUnifiedIdeograph7e39': '\xe7\xb8\xb9',
'HangulSyllableSsubs': '\xec\x91\xb6',
'CjkUnifiedIdeograph40cd': '\xe4\x83\x8d',
'TriangleWithUnderbar': '\xe2\xa7\x8b',
'UnicodeCharacter0xe5c8': '\xee\x97\x88',
'YiSyllablePyt': '\xea\x81\x90',
'UnicodeCharacter0xf49e': '\xef\x92\x9e',
'HangulJongseongPieupphieuph': '\xe1\x87\xa4',
'CjkUnifiedIdeograph8c2b': '\xe8\xb0\xab',
'CjkUnifiedIdeograph8c2a': '\xe8\xb0\xaa',
'CjkUnifiedIdeograph8c2f': '\xe8\xb0\xaf',
'CjkUnifiedIdeograph8c2e': '\xe8\xb0\xae',
'CjkUnifiedIdeograph8c2d': '\xe8\xb0\xad',
'UnicodeCharacter0xf8a9': '\xef\xa2\xa9',
'CjkUnifiedIdeograph5928': '\xe5\xa4\xa8',
'CjkUnifiedIdeograph5929': '\xe5\xa4\xa9',
'CjkUnifiedIdeograph5926': '\xe5\xa4\xa6',
'CjkUnifiedIdeograph5927': '\xe5\xa4\xa7',
'CjkUnifiedIdeograph5924': '\xe5\xa4\xa4',
'CjkUnifiedIdeograph5925': '\xe5\xa4\xa5',
'CjkUnifiedIdeograph5922': '\xe5\xa4\xa2',
'CjkUnifiedIdeograph5923': '\xe5\xa4\xa3',
'CjkUnifiedIdeograph5920': '\xe5\xa4\xa0',
'CjkUnifiedIdeograph5921': '\xe5\xa4\xa1',
'PunctuationSpace': '\xe2\x80\x88',
'LaoLetterKhoSung': '\xe0\xba\x82',
'HangulSyllableCaej': '\xec\xb1\x9a',
'CjkUnifiedIdeograph8469': '\xe8\x91\xa9',
'ArabicFathaIsolatedForm': '\xef\xb9\xb6',
'UnicodeCharacter0xeeba': '\xee\xba\xba',
'GleichStark': '\xe2\xa7\xa6',
'KhmerSymbolDappramRoc': '\xe1\xa7\xbf',
'UnicodeCharacter0xeebd': '\xee\xba\xbd',
'UnicodeCharacter0xeebe': '\xee\xba\xbe',
'HangulSyllableHwalt': '\xed\x99\xa1',
'CjkUnifiedIdeograph94ca': '\xe9\x93\x8a',
'CjkUnifiedIdeograph94cb': '\xe9\x93\x8b',
'CjkUnifiedIdeograph94cc': '\xe9\x93\x8c',
'CjkUnifiedIdeograph35f8': '\xe3\x97\xb8',
'CjkUnifiedIdeograph35f9': '\xe3\x97\xb9',
'CjkUnifiedIdeograph94cf': '\xe9\x93\x8f',
'HangulSyllableCaeg': '\xec\xb1\x85',
'CjkUnifiedIdeograph35f4': '\xe3\x97\xb4',
'CjkUnifiedIdeograph35f5': '\xe3\x97\xb5',
'CjkUnifiedIdeograph35f6': '\xe3\x97\xb6',
'CjkUnifiedIdeograph35f7': '\xe3\x97\xb7',
'CjkUnifiedIdeograph35f0': '\xe3\x97\xb0',
'CjkUnifiedIdeograph35f1': '\xe3\x97\xb1',
'CjkUnifiedIdeograph35f2': '\xe3\x97\xb2',
'CjkUnifiedIdeograph35f3': '\xe3\x97\xb3',
'HangulSyllableCyigg': '\xec\xb8\xbe',
'TibetanVowelSignEe': '\xe0\xbd\xbb',
'HangulSyllableCaec': '\xec\xb1\x9b',
'CjkUnifiedIdeograph7c4a': '\xe7\xb1\x8a',
'YiSyllableHluop': '\xea\x86\xa4',
'TurnedSansserifCapitalG': '\xe2\x85\x81',
'CjkUnifiedIdeograph592f': '\xe5\xa4\xaf',
'ServiceMark': '\xe2\x84\xa0',
'CjkUnifiedIdeograph592d': '\xe5\xa4\xad',
'CjkUnifiedIdeograph592e': '\xe5\xa4\xae',
'CjkUnifiedIdeograph592b': '\xe5\xa4\xab',
'CjkUnifiedIdeograph592c': '\xe5\xa4\xac',
'CjkUnifiedIdeograph592a': '\xe5\xa4\xaa',
'NewTaiLueDigitTwo': '\xe1\xa7\x92',
'UpperRightTriangle': '\xe2\x97\xb9',
'CjkUnifiedIdeograph94c2': '\xe9\x93\x82',
'CjkUnifiedIdeograph94c3': '\xe9\x93\x83',
'CjkUnifiedIdeograph94c4': '\xe9\x93\x84',
'CjkUnifiedIdeograph94c5': '\xe9\x93\x85',
'CjkUnifiedIdeograph94c6': '\xe9\x93\x86',
'CjkUnifiedIdeograph94c7': '\xe9\x93\x87',
'CjkUnifiedIdeograph35fd': '\xe3\x97\xbd',
'CjkUnifiedIdeograph35fe': '\xe3\x97\xbe',
'CjkUnifiedIdeograph35ff': '\xe3\x97\xbf',
'CircledHangulIeungU': '\xe3\x89\xbe',
'HangulSyllablePulp': '\xed\x92\x86',
'CjkUnifiedIdeograph35fa': '\xe3\x97\xba',
'CjkUnifiedIdeograph35fb': '\xe3\x97\xbb',
'CjkUnifiedIdeograph35fc': '\xe3\x97\xbc',
'UnicodeCharacter0xeeb8': '\xee\xba\xb8',
'HangulSyllablePulm': '\xed\x92\x82',
'HangulSyllablePulh': '\xed\x92\x87',
'UnicodeCharacter0xeeb0': '\xee\xba\xb0',
'UnicodeCharacter0xeeb1': '\xee\xba\xb1',
'CjkUnifiedIdeograph6ddb': '\xe6\xb7\x9b',
'HangulSyllablePulg': '\xed\x92\x81',
'UnicodeCharacter0xeeb4': '\xee\xba\xb4',
'UnicodeCharacter0xeeb5': '\xee\xba\xb5',
'HangulSyllablePulb': '\xed\x92\x83',
'CircledHangulIeungA': '\xe3\x89\xb5',
'CjkUnifiedIdeograph4db4': '\xe4\xb6\xb4',
'EthiopicSyllableNyoa': '\xe2\xb6\x89',
'CyrillicCapitalLetterDjerv': '\xea\x99\x88',
'ArabicLigatureSadWithAlefMaksuraIsolatedForm': '\xef\xb4\x85',
'UnicodeCharacter0x2796': '\xe2\x9e\x96',
'UnicodeCharacter0xeddc': '\xee\xb7\x9c',
'HangulSyllableGgwip': '\xeb\x80\xa6',
'UnicodeCharacter0xedda': '\xee\xb7\x9a',
'UnicodeCharacter0xeddf': '\xee\xb7\x9f',
'SmallRomanNumeralFive': '\xe2\x85\xb4',
'EthiopicSyllableKwaa': '\xe1\x8a\xb3',
'UnicodeCharacter0xf7f0': '\xef\x9f\xb0',
'HangulSyllableCyob': '\xec\xb6\x89',
'CyrillicCapitalLetterGje': '\xd0\x83',
'HangulSyllableCyod': '\xec\xb5\xbf',
'HangulSyllableCyog': '\xec\xb5\xb9',
'HangulSyllableCyoh': '\xec\xb6\x93',
'HangulSyllableSeon': '\xec\x84\xa0',
'HangulSyllableCyoj': '\xec\xb6\x8e',
'HangulSyllableGgwis': '\xeb\x80\x9f',
'HangulSyllableCyol': '\xec\xb6\x80',
'HangulSyllableCyom': '\xec\xb6\x88',
'HangulSyllableCyon': '\xec\xb5\xbc',
'KangxiRadicalInch': '\xe2\xbc\xa8',
'HangulSyllableCyop': '\xec\xb6\x92',
'SquareMegaton': '\xe3\x8d\x8c',
'HangulSyllableCyot': '\xec\xb6\x91',
'HexagramForDuration': '\xe4\xb7\x9f',
'ArabicLigatureJeemWithHahIsolatedForm': '\xef\xb0\x95',
'UnicodeCharacter0xf7f7': '\xef\x9f\xb7',
'UnicodeCharacter0xedd9': '\xee\xb7\x99',
'UnicodeCharacter0xedd8': '\xee\xb7\x98',
'UnicodeCharacter0xedd3': '\xee\xb7\x93',
'UnicodeCharacter0xedd2': '\xee\xb7\x92',
'UnicodeCharacter0xedd1': '\xee\xb7\x91',
'UnicodeCharacter0xedd0': '\xee\xb7\x90',
'UnicodeCharacter0xedd7': '\xee\xb7\x97',
'UnicodeCharacter0xedd6': '\xee\xb7\x96',
'UnicodeCharacter0xedd5': '\xee\xb7\x95',
'UnicodeCharacter0xedd4': '\xee\xb7\x94',
'GlagoliticSmallLetterKako': '\xe2\xb0\xbd',
'ArabicLigatureTehWithMeemWithAlefMaksuraFinalForm': '\xef\xb6\xa4',
'HangulJongseongKapyeounphieuph': '\xe1\x87\xb4',
'ArabicLetterUWithHamzaAboveIsolatedForm': '\xef\xaf\x9d',
'UnicodeCharacter0xaafd': '\xea\xab\xbd',
'UnicodeCharacter0xe655': '\xee\x99\x95',
'UnicodeCharacter0xaafe': '\xea\xab\xbe',
'ArabicLigatureYehWithHamzaAboveWithMeemIsolatedForm': '\xef\xb0\x82',
'HangulSyllableOels': '\xec\x9a\x84',
'HangulSyllableGgwig': '\xeb\x80\x8d',
'UnicodeCharacter0xa8de': '\xea\xa3\x9e',
'UnicodeCharacter0xaafb': '\xea\xab\xbb',
'GreekSubscriptSmallLetterPhi': '\xe1\xb5\xa9',
'UnicodeCharacter0xf6b0': '\xef\x9a\xb0',
'UnicodeCharacter0xaafc': '\xea\xab\xbc',
'YiSyllableZze': '\xea\x8b\xa0',
'CjkUnifiedIdeograph7ab8': '\xe7\xaa\xb8',
'CjkUnifiedIdeograph7ab9': '\xe7\xaa\xb9',
'VictoryHand': '\xe2\x9c\x8c',
'HangulSyllableGgwim': '\xeb\x80\x9c',
'CjkUnifiedIdeograph7ab0': '\xe7\xaa\xb0',
'CjkUnifiedIdeograph7ab1': '\xe7\xaa\xb1',
'CjkUnifiedIdeograph7ab2': '\xe7\xaa\xb2',
'CjkUnifiedIdeograph7ab3': '\xe7\xaa\xb3',
'CjkUnifiedIdeograph7ab4': '\xe7\xaa\xb4',
'CjkUnifiedIdeograph7ab5': '\xe7\xaa\xb5',
'CjkUnifiedIdeograph7ab6': '\xe7\xaa\xb6',
'CjkUnifiedIdeograph7ab7': '\xe7\xaa\xb7',
'CjkUnifiedIdeograph636a': '\xe6\x8d\xaa',
'CjkUnifiedIdeograph636c': '\xe6\x8d\xac',
'CjkUnifiedIdeograph636b': '\xe6\x8d\xab',
'CjkUnifiedIdeograph636e': '\xe6\x8d\xae',
'CjkUnifiedIdeograph636d': '\xe6\x8d\xad',
'CjkUnifiedIdeograph636f': '\xe6\x8d\xaf',
'UnicodeCharacter0x26fb': '\xe2\x9b\xbb',
'UnicodeCharacter0x26fc': '\xe2\x9b\xbc',
'UnicodeCharacter0x26fa': '\xe2\x9b\xba',
'UnicodeCharacter0x26ff': '\xe2\x9b\xbf',
'HangulSyllableGgwij': '\xeb\x80\xa2',
'UnicodeCharacter0x26fd': '\xe2\x9b\xbd',
'UnicodeCharacter0x26fe': '\xe2\x9b\xbe',
'HangulSyllableGgwik': '\xeb\x80\xa4',
'LongDivision': '\xe2\x9f\x8c',
'UnicodeCharacter0xf13b': '\xef\x84\xbb',
'HangulSyllableHoeng': '\xed\x9a\xa1',
'HangulSyllableHoenh': '\xed\x9a\x92',
'CjkUnifiedIdeograph7aba': '\xe7\xaa\xba',
'CjkUnifiedIdeograph7abb': '\xe7\xaa\xbb',
'TibetanMarkRinChenSpungsShad': '\xe0\xbc\x91',
'CjkUnifiedIdeograph7abd': '\xe7\xaa\xbd',
'CjkUnifiedIdeograph7abe': '\xe7\xaa\xbe',
'ModifierLetterSmallIota': '\xe1\xb6\xa5',
'YiSyllableZzi': '\xea\x8b\x92',
'SundaneseLetterNya': '\xe1\xae\x91',
'UnicodeCharacter0x26f2': '\xe2\x9b\xb2',
'UnicodeCharacter0x26f3': '\xe2\x9b\xb3',
'UnicodeCharacter0x26f0': '\xe2\x9b\xb0',
'UnicodeCharacter0x26f1': '\xe2\x9b\xb1',
'UnicodeCharacter0x26f6': '\xe2\x9b\xb6',
'UnicodeCharacter0x26f7': '\xe2\x9b\xb7',
'UnicodeCharacter0x26f4': '\xe2\x9b\xb4',
'UnicodeCharacter0x26f5': '\xe2\x9b\xb5',
'UnicodeCharacter0x26f8': '\xe2\x9b\xb8',
'UnicodeCharacter0x26f9': '\xe2\x9b\xb9',
'CyrillicCapitalLetterYae': '\xd4\x98',
'CjkUnifiedIdeograph6361': '\xe6\x8d\xa1',
'CjkUnifiedIdeograph6360': '\xe6\x8d\xa0',
'CjkUnifiedIdeograph6363': '\xe6\x8d\xa3',
'CjkUnifiedIdeograph6362': '\xe6\x8d\xa2',
'CjkUnifiedIdeograph6365': '\xe6\x8d\xa5',
'CjkUnifiedIdeograph6364': '\xe6\x8d\xa4',
'CjkUnifiedIdeograph6367': '\xe6\x8d\xa7',
'CjkUnifiedIdeograph6366': '\xe6\x8d\xa6',
'Capricorn': '\xe2\x99\x91',
'ArrowPointingRightwardsThenCurvingDownwards': '\xe2\xa4\xb5',
'CyrillicCapitalLetterYat': '\xd1\xa2',
'HangulSyllableMyubs': '\xeb\xae\xb6',
'HangulSyllableNyih': '\xeb\x8b\x87',
'HangulSyllableNyik': '\xeb\x8b\x84',
'HangulSyllableGgoelb': '\xea\xbe\x8b',
'HangulSyllableNyim': '\xeb\x8a\xbc',
'HangulSyllableNyil': '\xeb\x8a\xb4',
'HangulSyllableGgoelg': '\xea\xbe\x89',
'HangulSyllableNyin': '\xeb\x8a\xb0',
'HangulSyllableGgoelh': '\xea\xbe\x8f',
'LatinCapitalLetterSWithCaron': '\xc5\xa0',
'CjkUnifiedIdeograph61b8': '\xe6\x86\xb8',
'HangulSyllableGgoelm': '\xea\xbe\x8a',
'HangulSyllableNyid': '\xeb\x8a\xb3',
'MalayalamLetterSsa': '\xe0\xb4\xb7',
'HangulSyllableRyah': '\xeb\x9f\x8f',
'CjkUnifiedIdeograph42ef': '\xe4\x8b\xaf',
'CjkUnifiedIdeograph42ee': '\xe4\x8b\xae',
'CjkUnifiedIdeograph42ed': '\xe4\x8b\xad',
'CjkUnifiedIdeograph42ec': '\xe4\x8b\xac',
'CjkUnifiedIdeograph42eb': '\xe4\x8b\xab',
'CjkUnifiedIdeograph42ea': '\xe4\x8b\xaa',
'UnicodeCharacter0xc65': '\xe0\xb1\xa5',
'CjkUnifiedIdeograph78c4': '\xe7\xa3\x84',
'HangulSyllableNyip': '\xeb\x8b\x86',
'LatinCapitalLetterEWithCedillaAndBreve': '\xe1\xb8\x9c',
'YiSyllableZhuop': '\xea\x8d\x8c',
'HangulSyllableNyit': '\xeb\x8b\x85',
'HangulSyllableDegs': '\xeb\x8d\xb3',
'HangulSyllableRyat': '\xeb\x9f\x8d',
'CjkUnifiedIdeograph5eac': '\xe5\xba\xac',
'CjkUnifiedIdeograph5eab': '\xe5\xba\xab',
'TamilNumberOneThousand': '\xe0\xaf\xb2',
'FullwidthDollarSign': '\xef\xbc\x84',
'CjkUnifiedIdeograph5eaf': '\xe5\xba\xaf',
'CjkUnifiedIdeograph5eae': '\xe5\xba\xae',
'CjkUnifiedIdeograph5ead': '\xe5\xba\xad',
'HangulSyllableDegg': '\xeb\x8d\xb2',
'CjkUnifiedIdeograph9918': '\xe9\xa4\x98',
'YiSyllableZzy': '\xea\x8b\xa9',
'MongolianLetterManchuAliGaliNga': '\xe1\xa2\x9b',
'UnicodeCharacter0xed8b': '\xee\xb6\x8b',
'HangulSyllableRyap': '\xeb\x9f\x8e',
'CjkUnifiedIdeograph42e7': '\xe4\x8b\xa7',
'CjkUnifiedIdeograph42e6': '\xe4\x8b\xa6',
'CjkUnifiedIdeograph42e5': '\xe4\x8b\xa5',
'CjkUnifiedIdeograph42e4': '\xe4\x8b\xa4',
'CjkUnifiedIdeograph42e3': '\xe4\x8b\xa3',
'CjkUnifiedIdeograph42e2': '\xe4\x8b\xa2',
'CjkUnifiedIdeograph42e1': '\xe4\x8b\xa1',
'CjkUnifiedIdeograph42e0': '\xe4\x8b\xa0',
'CjkUnifiedIdeograph9a10': '\xe9\xa8\x90',
'CjkUnifiedIdeograph42e9': '\xe4\x8b\xa9',
'CjkUnifiedIdeograph42e8': '\xe4\x8b\xa8',
'NominalDigitShapes': '\xe2\x81\xaf',
'KhmerSymbolLekAttakPrambei': '\xe1\x9f\xb8',
'CjkUnifiedIdeograph9912': '\xe9\xa4\x92',
'MathematicalRightAngleBracket': '\xe2\x9f\xa9',
'HangulSyllableKaess': '\xec\xba\xa4',
'HangulSyllableMaep': '\xeb\xa7\xbe',
'HangulSyllableDyung': '\xeb\x93\x95',
'HalfwidthKatakanaLetterSi': '\xef\xbd\xbc',
'CjkUnifiedIdeograph4a80': '\xe4\xaa\x80',
'HangulSyllableMaet': '\xeb\xa7\xbd',
'ArabicLigatureHehWithSuperscriptAlefInitialForm': '\xef\xb3\x99',
'HalfwidthKatakanaLetterSa': '\xef\xbd\xbb',
'HangulSyllableDyunj': '\xeb\x93\x85',
'HangulSyllableDyunh': '\xeb\x93\x86',
'CjkUnifiedIdeograph5ea9': '\xe5\xba\xa9',
'SyriacQushshaya': '\xdd\x81',
'HangulSyllableMaed': '\xeb\xa7\xab',
'CjkUnifiedIdeograph9915': '\xe9\xa4\x95',
'HangulSyllableMaeg': '\xeb\xa7\xa5',
'CjkUnifiedIdeograph5ea3': '\xe5\xba\xa3',
'CjkUnifiedIdeograph5ea2': '\xe5\xba\xa2',
'CjkUnifiedIdeograph5ea1': '\xe5\xba\xa1',
'MyanmarLetterShanHa': '\xe1\x82\x81',
'CjkUnifiedIdeograph5ea7': '\xe5\xba\xa7',
'CjkUnifiedIdeograph5ea6': '\xe5\xba\xa6',
'CjkUnifiedIdeograph5ea5': '\xe5\xba\xa5',
'CjkUnifiedIdeograph5ea4': '\xe5\xba\xa4',
'UnicodeCharacter0xaaf4': '\xea\xab\xb4',
'UnicodeCharacter0xaaf5': '\xea\xab\xb5',
'CyrillicSmallLetterTche': '\xea\x9a\x93',
'HangulSyllablePeoh': '\xed\x8e\x97',
'KannadaVowelSignVocalicRr': '\xe0\xb3\x84',
'CjkUnifiedIdeograph62fd': '\xe6\x8b\xbd',
'CjkUnifiedIdeograph62fe': '\xe6\x8b\xbe',
'CjkUnifiedIdeograph62ff': '\xe6\x8b\xbf',
'CjkRadicalBoltOfCloth': '\xe2\xba\xaa',
'CjkUnifiedIdeograph62fa': '\xe6\x8b\xba',
'CjkUnifiedIdeograph62fb': '\xe6\x8b\xbb',
'CjkUnifiedIdeograph62fc': '\xe6\x8b\xbc',
'UnicodeCharacter0xaaf0': '\xea\xab\xb0',
'UnicodeCharacter0xaaf1': '\xea\xab\xb1',
'SaurashtraVowelSignIi': '\xea\xa2\xb7',
'MyanmarLetterNnya': '\xe1\x80\x8a',
'UnicodeCharacter0xaaf2': '\xea\xab\xb2',
'YiSyllableNjuo': '\xea\x90\xba',
'HangulSyllablePeok': '\xed\x8e\x94',
'UnicodeCharacter0xaaf3': '\xea\xab\xb3',
'HangulSyllableSaenh': '\xec\x83\x8e',
'NumberThirteenFullStop': '\xe2\x92\x94',
'EthiopicSyllableKxaa': '\xe1\x8a\xbb',
'GeorgianLetterHe': '\xe1\x83\xb1',
'YiSyllableNjur': '\xea\x91\x83',
'UnicodeCharacter0xf6b6': '\xef\x9a\xb6',
'YiSyllableNjup': '\xea\x91\x81',
'GreekCapitalLetterPhi': '\xce\xa6',
'CircledIdeographSeven': '\xe3\x8a\x86',
'YiSyllableNjux': '\xea\x90\xbf',
'RightwardsArrowThroughSuperset': '\xe2\xad\x84',
'CjkUnifiedIdeograph62f5': '\xe6\x8b\xb5',
'CjkUnifiedIdeograph62f6': '\xe6\x8b\xb6',
'CjkUnifiedIdeograph62f7': '\xe6\x8b\xb7',
'CjkUnifiedIdeograph62f0': '\xe6\x8b\xb0',
'CjkUnifiedIdeograph62f1': '\xe6\x8b\xb1',
'CjkUnifiedIdeograph62f2': '\xe6\x8b\xb2',
'CjkUnifiedIdeograph62f3': '\xe6\x8b\xb3',
'UnicodeCharacter0xaaf8': '\xea\xab\xb8',
'CjkUnifiedIdeograph62f8': '\xe6\x8b\xb8',
'UnicodeCharacter0x2b86': '\xe2\xae\x86',
'RecyclingSymbolForType7Plastics': '\xe2\x99\xb9',
'UnicodeCharacter0xaaf9': '\xea\xab\xb9',
'UnicodeCharacter0xed8a': '\xee\xb6\x8a',
'UnicodeCharacter0xebd0': '\xee\xaf\x90',
'CjkUnifiedIdeograph5bd5': '\xe5\xaf\x95',
'ArabicLigatureYehWithHamzaAboveWithRehFinalForm': '\xef\xb1\xa4',
'UnicodeCharacter0xeab1': '\xee\xaa\xb1',
'BalineseLetterAikara': '\xe1\xac\x90',
'PresentationFormForVerticalRightWhiteLenticularBrakcet': '\xef\xb8\x98',
'ArabicLigatureYehWithHamzaAboveWithAlefMaksuraFinalForm': '\xef\xb1\xa8',
'CjkUnifiedIdeograph91a2': '\xe9\x86\xa2',
'UnicodeCharacter0x2b83': '\xe2\xae\x83',
'RunicLetterNaudizNydNaudN': '\xe1\x9a\xbe',
'CjkUnifiedIdeograph91a1': '\xe9\x86\xa1',
'UnicodeCharacter0x2b88': '\xe2\xae\x88',
'LatinCapitalLetterAWithDotAboveAndMacron': '\xc7\xa0',
'WhiteConcavesidedDiamondWithRightwardsTick': '\xe2\x9f\xa3',
'SmallColon': '\xef\xb9\x95',
'LaoDigitFour': '\xe0\xbb\x94',
'CjkUnifiedIdeograph9a19': '\xe9\xa8\x99',
'CjkCompatibilityIdeographf9f7': '\xef\xa7\xb7',
'YiSyllablePyrx': '\xea\x81\x94',
'CjkUnifiedIdeograph3f45': '\xe3\xbd\x85',
'CjkUnifiedIdeograph3f44': '\xe3\xbd\x84',
'CjkUnifiedIdeograph3f47': '\xe3\xbd\x87',
'CjkUnifiedIdeograph3f46': '\xe3\xbd\x86',
'CjkUnifiedIdeograph3f41': '\xe3\xbd\x81',
'CjkUnifiedIdeograph3f40': '\xe3\xbd\x80',
'CjkUnifiedIdeograph3f43': '\xe3\xbd\x83',
'CjkUnifiedIdeograph3f42': '\xe3\xbd\x82',
'CjkUnifiedIdeograph7c4d': '\xe7\xb1\x8d',
'CjkUnifiedIdeograph7c4e': '\xe7\xb1\x8e',
'CjkUnifiedIdeograph7c4f': '\xe7\xb1\x8f',
'HangulSyllableBbyu': '\xec\x80\xbc',
'CjkUnifiedIdeograph3f49': '\xe3\xbd\x89',
'CjkUnifiedIdeograph3f48': '\xe3\xbd\x88',
'CjkUnifiedIdeograph7c4b': '\xe7\xb1\x8b',
'CjkUnifiedIdeograph7c4c': '\xe7\xb1\x8c',
'HangulSyllableSseong': '\xec\x8d\xbd',
'HangulSyllableBbyo': '\xeb\xbe\xb0',
'HangulSyllableTaegs': '\xed\x83\x9f',
'UnicodeCharacter0xee5b': '\xee\xb9\x9b',
'HangulSyllableBbyi': '\xec\x81\xb4',
'CjkCompatibilityIdeographf98e': '\xef\xa6\x8e',
'CjkCompatibilityIdeographf98d': '\xef\xa6\x8d',
'CjkUnifiedIdeograph9a18': '\xe9\xa8\x98',
'CjkCompatibilityIdeographf98a': '\xef\xa6\x8a',
'HangulSyllableSseonj': '\xec\x8d\xad',
'CjkCompatibilityIdeographf98c': '\xef\xa6\x8c',
'HangulSyllableBbya': '\xeb\xba\x98',
'HangulSyllableKyael': '\xec\xbb\x90',
'HangulSyllableKyaem': '\xec\xbb\x98',
'HangulSyllableKyaen': '\xec\xbb\x8c',
'CjkUnifiedIdeograph91a7': '\xe9\x86\xa7',
'HangulSyllableKyaej': '\xec\xbb\x9e',
'HangulSyllableKyaek': '\xec\xbb\xa0',
'HangulSyllableKyaed': '\xec\xbb\x8f',
'SquareErg': '\xe3\x8b\x8d',
'HangulSyllableKyaeg': '\xec\xbb\x89',
'UnicodeCharacter0xf6b7': '\xef\x9a\xb7',
'HalfwidthHangulLetterKiyeoksios': '\xef\xbe\xa3',
'HangulSyllableKyaeb': '\xec\xbb\x99',
'HangulSyllableKyaec': '\xec\xbb\x9f',
'HangulSyllableCunj': '\xec\xb6\x99',
'HangulSyllableCunh': '\xec\xb6\x9a',
'HangulSyllableKyaet': '\xec\xbb\xa1',
'HangulSyllableCung': '\xec\xb6\xa9',
'HangulSyllableKyaes': '\xec\xbb\x9b',
'VaiSyllableLoo': '\xea\x96\x83',
'CjkUnifiedIdeograph9302': '\xe9\x8c\x82',
'CjkUnifiedIdeograph91a6': '\xe9\x86\xa6',
'CjkCompatibilityIdeographf989': '\xef\xa6\x89',
'CjkCompatibilityIdeographf988': '\xef\xa6\x88',
'CjkUnifiedIdeograph9303': '\xe9\x8c\x83',
'CjkCompatibilityIdeographf985': '\xef\xa6\x85',
'CjkCompatibilityIdeographf984': '\xef\xa6\x84',
'CjkCompatibilityIdeographf987': '\xef\xa6\x87',
'CjkUnifiedIdeograph74ca': '\xe7\x93\x8a',
'CjkCompatibilityIdeographf981': '\xef\xa6\x81',
'CjkUnifiedIdeograph6172': '\xe6\x85\xb2',
'HangulSyllableDu': '\xeb\x91\x90',
'CjkCompatibilityIdeographf982': '\xef\xa6\x82',
'CjkUnifiedIdeograph3f4e': '\xe3\xbd\x8e',
'CjkUnifiedIdeograph3f4d': '\xe3\xbd\x8d',
'UnicodeCharacter0xa7df': '\xea\x9f\x9f',
'CjkUnifiedIdeograph3f4f': '\xe3\xbd\x8f',
'CjkUnifiedIdeograph3f4a': '\xe3\xbd\x8a',
'CjkUnifiedIdeograph7c49': '\xe7\xb1\x89',
'CjkUnifiedIdeograph3f4c': '\xe3\xbd\x8c',
'CjkUnifiedIdeograph3f4b': '\xe3\xbd\x8b',
'CjkUnifiedIdeograph7c44': '\xe7\xb1\x84',
'CjkUnifiedIdeograph7c45': '\xe7\xb1\x85',
'CjkUnifiedIdeograph7c46': '\xe7\xb1\x86',
'CjkUnifiedIdeograph7c47': '\xe7\xb1\x87',
'CjkUnifiedIdeograph7c40': '\xe7\xb1\x80',
'CjkUnifiedIdeograph7c41': '\xe7\xb1\x81',
'CjkUnifiedIdeograph7c42': '\xe7\xb1\x82',
'CjkUnifiedIdeograph7c43': '\xe7\xb1\x83',
'CjkUnifiedIdeograph4b2b': '\xe4\xac\xab',
'CjkUnifiedIdeograph4b2c': '\xe4\xac\xac',
'CjkUnifiedIdeograph4b2a': '\xe4\xac\xaa',
'CjkUnifiedIdeograph4b2f': '\xe4\xac\xaf',
'CjkUnifiedIdeograph9304': '\xe9\x8c\x84',
'CjkUnifiedIdeograph4b2d': '\xe4\xac\xad',
'CjkUnifiedIdeograph4b2e': '\xe4\xac\xae',
'CjkUnifiedIdeograph6177': '\xe6\x85\xb7',
'HangulSyllableDdyagg': '\xeb\x95\xaa',
'HangulSyllableGgwej': '\xeb\x80\x86',
'TamilLetterYa': '\xe0\xae\xaf',
'CjkRadicalCsimplifiedGold': '\xe2\xbb\x90',
'ArabicLigatureYehWithHamzaAboveWithAlefMaksuraIsolatedForm': '\xef\xb0\x83',
'UnicodeCharacter0xf133': '\xef\x84\xb3',
'CjkUnifiedIdeograph8de7': '\xe8\xb7\xa7',
'UnicodeCharacter0xf131': '\xef\x84\xb1',
'HangulSyllableGgwek': '\xeb\x80\x88',
'UnicodeCharacter0xdcf0': '\xed\xb3\xb0',
'UnicodeCharacter0xf137': '\xef\x84\xb7',
'Neptune': '\xe2\x99\x86',
'HangulSyllableNyeonh': '\xeb\x85\x86',
'CjkUnifiedIdeograph4b22': '\xe4\xac\xa2',
'CjkUnifiedIdeograph502c': '\xe5\x80\xac',
'CjkUnifiedIdeograph4b20': '\xe4\xac\xa0',
'CjkUnifiedIdeograph4b21': '\xe4\xac\xa1',
'CjkUnifiedIdeograph4b26': '\xe4\xac\xa6',
'CjkUnifiedIdeograph4b27': '\xe4\xac\xa7',
'CjkUnifiedIdeograph4b24': '\xe4\xac\xa4',
'CyrillicCapitalLetterGheWithMiddleHook': '\xd2\x94',
'CjkUnifiedIdeograph4b28': '\xe4\xac\xa8',
'CjkUnifiedIdeograph4b29': '\xe4\xac\xa9',
'BraillePatternDots4568': '\xe2\xa2\xb8',
'HangulSyllableGgeol': '\xea\xbb\x84',
'HangulSyllableGgeom': '\xea\xbb\x8c',
'HangulSyllableGgeon': '\xea\xbb\x80',
'HangulSyllableGgeoh': '\xea\xbb\x97',
'HangulSyllableGgeoj': '\xea\xbb\x92',
'HangulSyllableGgeok': '\xea\xbb\x94',
'HangulSyllableGgeod': '\xea\xbb\x83',
'HangulSyllableSsu': '\xec\x91\xa4',
'HangulChoseongCeongchieumsios': '\xe1\x84\xbe',
'HangulSyllableGgeob': '\xea\xbb\x8d',
'HangulSyllableGgeoc': '\xea\xbb\x93',
'UnicodeCharacter0xd9b6': '\xed\xa6\xb6',
'UnicodeCharacter0xd89b': '\xed\xa2\x9b',
'CjkUnifiedIdeograph4ea2': '\xe4\xba\xa2',
'HangulSyllableGgeop': '\xea\xbb\x96',
'IdeographicTelegraphSymbolForHourFifteen': '\xe3\x8d\xa7',
'HangulSyllableGgeos': '\xea\xbb\x8f',
'BuhidLetterHa': '\xe1\x9d\x91',
'FiveDotPunctuation': '\xe2\x81\x99',
'CjkUnifiedIdeograph77b8': '\xe7\x9e\xb8',
'CjkUnifiedIdeograph77b9': '\xe7\x9e\xb9',
'CjkUnifiedIdeograph77b6': '\xe7\x9e\xb6',
'CjkUnifiedIdeograph77b7': '\xe7\x9e\xb7',
'CjkUnifiedIdeograph77b4': '\xe7\x9e\xb4',
'CjkUnifiedIdeograph77b5': '\xe7\x9e\xb5',
'CjkUnifiedIdeograph77b2': '\xe7\x9e\xb2',
'CjkUnifiedIdeograph77b3': '\xe7\x9e\xb3',
'CjkUnifiedIdeograph77b0': '\xe7\x9e\xb0',
'CjkUnifiedIdeograph77b1': '\xe7\x9e\xb1',
'CjkUnifiedIdeograph5d89': '\xe5\xb6\x89',
'CjkUnifiedIdeograph5d88': '\xe5\xb6\x88',
'DoublePlus': '\xe2\xa7\xba',
'SmallElementOf': '\xe2\x88\x8a',
'CjkUnifiedIdeograph5d81': '\xe5\xb6\x81',
'CjkUnifiedIdeograph5d80': '\xe5\xb6\x80',
'CjkUnifiedIdeograph5d83': '\xe5\xb6\x83',
'CjkUnifiedIdeograph5d82': '\xe5\xb6\x82',
'CjkUnifiedIdeograph5d85': '\xe5\xb6\x85',
'CjkUnifiedIdeograph5d84': '\xe5\xb6\x84',
'CjkUnifiedIdeograph5d87': '\xe5\xb6\x87',
'CjkUnifiedIdeograph5d86': '\xe5\xb6\x86',
'HangulSyllableJjweolg': '\xec\xad\xad',
'MyanmarLetterMonBbe': '\xe1\x81\x9d',
'HangulSyllableBbyulb': '\xec\x81\x87',
'MyanmarLetterMonBba': '\xe1\x81\x9c',
'HangulSyllableBbyulg': '\xec\x81\x85',
'HangulSyllableBbyulh': '\xec\x81\x8b',
'HangulSyllableJjweolm': '\xec\xad\xae',
'HangulSyllableBbyulm': '\xec\x81\x86',
'HangulSyllableTolp': '\xed\x86\xae',
'HangulSyllableJjweolh': '\xec\xad\xb3',
'HangulSyllableBbyulp': '\xec\x81\x8a',
'PhagspaLetterRa': '\xea\xa1\x98',
'UnicodeCharacter0xa9be': '\xea\xa6\xbe',
'SymbolForDeviceControlFour': '\xe2\x90\x94',
'HangulSyllableBbyult': '\xec\x81\x89',
'TaiLeLetterTa': '\xe1\xa5\x96',
'KangxiRadicalRun': '\xe2\xbe\x9b',
'RightSingleQuotationMark': '\xe2\x80\x99',
'ArabicLigatureNoonWithHahWithAlefMaksuraFinalForm': '\xef\xb6\x96',
'HangulSyllableTolt': '\xed\x86\xad',
'Neuter': '\xe2\x9a\xb2',
'CjkUnifiedIdeograph5d8a': '\xe5\xb6\x8a',
'CjkUnifiedIdeograph5b90': '\xe5\xae\x90',
'CjkUnifiedIdeograph5d8c': '\xe5\xb6\x8c',
'CjkUnifiedIdeograph5d8b': '\xe5\xb6\x8b',
'CjkUnifiedIdeograph5d8e': '\xe5\xb6\x8e',
'CjkUnifiedIdeograph5d8d': '\xe5\xb6\x8d',
'CjkUnifiedIdeograph5d8f': '\xe5\xb6\x8f',
'CjkUnifiedIdeograph90e8': '\xe9\x83\xa8',
'CjkUnifiedIdeograph90e9': '\xe9\x83\xa9',
'HangulSyllableSyeolt': '\xec\x85\xa1',
'CjkUnifiedIdeograph77bf': '\xe7\x9e\xbf',
'CjkUnifiedIdeograph90e3': '\xe9\x83\xa3',
'CjkUnifiedIdeograph77bd': '\xe7\x9e\xbd',
'CjkUnifiedIdeograph77be': '\xe7\x9e\xbe',
'CjkUnifiedIdeograph77bb': '\xe7\x9e\xbb',
'CjkUnifiedIdeograph77bc': '\xe7\x9e\xbc',
'CjkUnifiedIdeograph90e4': '\xe9\x83\xa4',
'HebrewLetterTet': '\xd7\x98',
'HangulSyllableJugg': '\xec\xa3\xbe',
'YiSyllableMgux': '\xea\x89\x81',
'BraillePatternDots3678': '\xe2\xa3\xa4',
'UnicodeCharacter0xd9b2': '\xed\xa6\xb2',
'YiSyllableMgur': '\xea\x89\x85',
'YiSyllableMgup': '\xea\x89\x83',
'YiSyllableMgut': '\xea\x89\x80',
'AngleWithSInside': '\xe2\xa6\x9e',
'HangulSyllableJjyug': '\xec\xae\xb9',
'TeluguLetterSsa': '\xe0\xb0\xb7',
'CjkUnifiedIdeograph930c': '\xe9\x8c\x8c',
'UnicodeCharacter0xd9eb': '\xed\xa7\xab',
'UnicodeCharacter0xd9ec': '\xed\xa7\xac',
'CjkUnifiedIdeograph930a': '\xe9\x8c\x8a',
'UnicodeCharacter0xd9ef': '\xed\xa7\xaf',
'HangulSyllableByolh': '\xeb\xb5\xb3',
'GlagoliticSmallLetterDjervi': '\xe2\xb0\xbc',
'CircledNumberSeventeen': '\xe2\x91\xb0',
'CjkUnifiedIdeograph930f': '\xe9\x8c\x8f',
'HangulSyllableCaelm': '\xec\xb1\x8e',
'ParenthesizedIdeographWater': '\xe3\x88\xac',
'UnicodeCharacter0xf653': '\xef\x99\x93',
'UnicodeCharacter0xf652': '\xef\x99\x92',
'UnicodeCharacter0xf651': '\xef\x99\x91',
'HangulSyllableRubs': '\xeb\xa3\xba',
'HangulSyllableByi': '\xeb\xb8\xa8',
'CjkUnifiedIdeograph930d': '\xe9\x8c\x8d',
'EthiopicSyllableThoa': '\xe2\xb6\x8f',
'HangulChoseongFiller': '\xe1\x85\x9f',
'HangulSyllableDdoegs': '\xeb\x99\xa7',
'CjkUnifiedIdeograph9a00': '\xe9\xa8\x80',
'CjkUnifiedIdeograph9a01': '\xe9\xa8\x81',
'LeftwardsDashedArrow': '\xe2\x87\xa0',
'CjkUnifiedIdeograph9a03': '\xe9\xa8\x83',
'CjkUnifiedIdeograph9a04': '\xe9\xa8\x84',
'CjkUnifiedIdeograph9a05': '\xe9\xa8\x85',
'CjkUnifiedIdeograph9a06': '\xe9\xa8\x86',
'CjkUnifiedIdeograph9a07': '\xe9\xa8\x87',
'CjkUnifiedIdeograph9a08': '\xe9\xa8\x88',
'CjkUnifiedIdeograph9a09': '\xe9\xa8\x89',
'HalfwidthRightwardsArrow': '\xef\xbf\xab',
'HangulSyllableBeulm': '\xeb\xb8\x96',
'UnicodeCharacter0xd9e2': '\xed\xa7\xa2',
'LatinCapitalLetterRWithTail': '\xe2\xb1\xa4',
'UnicodeCharacter0xd9e0': '\xed\xa7\xa0',
'UnicodeCharacter0xd9e1': '\xed\xa7\xa1',
'UnicodeCharacter0xd9e6': '\xed\xa7\xa6',
'CopticSmallLetterOou': '\xe2\xb2\xb1',
'UnicodeCharacter0xd9e4': '\xed\xa7\xa4',
'UnicodeCharacter0xd9e5': '\xed\xa7\xa5',
'UnicodeCharacter0xf65f': '\xef\x99\x9f',
'UnicodeCharacter0xd9e8': '\xed\xa7\xa8',
'UnicodeCharacter0xd9e9': '\xed\xa7\xa9',
'UnicodeCharacter0xf65c': '\xef\x99\x9c',
'UnicodeCharacter0xf48': '\xe0\xbd\x88',
'UnicodeCharacter0xf65a': '\xef\x99\x9a',
'HangulSyllableHass': '\xed\x95\xac',
'DevanagariLetterTa': '\xe0\xa4\xa4',
'CjkUnifiedIdeograph9a0a': '\xe9\xa8\x8a',
'CjkUnifiedIdeograph9a0b': '\xe9\xa8\x8b',
'CjkUnifiedIdeograph9a0c': '\xe9\xa8\x8c',
'CjkUnifiedIdeograph9a0d': '\xe9\xa8\x8d',
'CjkUnifiedIdeograph9a0e': '\xe9\xa8\x8e',
'CjkUnifiedIdeograph9a0f': '\xe9\xa8\x8f',
'ArabicLigatureBehWithYehIsolatedForm': '\xef\xb0\x8a',
'MyanmarLetterWesternPwoKarenTha': '\xe1\x81\xa5',
'LaoVowelSignEi': '\xe0\xbb\x81',
'KangxiRadicalStand': '\xe2\xbd\xb4',
'UnicodeCharacter0xd89a': '\xed\xa2\x9a',
'HangulJongseongNieunkiyeok': '\xe1\x87\x85',
'LatinCapitalLetterIWithStroke': '\xc6\x97',
'CyrillicSmallLetterElWithHook': '\xd4\x93',
'CjkUnifiedIdeograph943a': '\xe9\x90\xba',
'RightRaisedOmissionBracket': '\xe2\xb8\x8d',
'CjkUnifiedIdeograph5496': '\xe5\x92\x96',
'CjkUnifiedIdeograph5497': '\xe5\x92\x97',
'CjkUnifiedIdeograph5494': '\xe5\x92\x94',
'CjkUnifiedIdeograph5495': '\xe5\x92\x95',
'CjkUnifiedIdeograph5492': '\xe5\x92\x92',
'CjkUnifiedIdeograph5493': '\xe5\x92\x93',
'CjkUnifiedIdeograph5490': '\xe5\x92\x90',
'CjkUnifiedIdeograph5491': '\xe5\x92\x91',
'HangulSyllableMyung': '\xeb\xae\xb9',
'CjkUnifiedIdeograph5498': '\xe5\x92\x98',
'CjkUnifiedIdeograph5499': '\xe5\x92\x99',
'UnicodeCharacter0x2e76': '\xe2\xb9\xb6',
'UnicodeCharacter0x2e77': '\xe2\xb9\xb7',
'UnicodeCharacter0x2e74': '\xe2\xb9\xb4',
'UnicodeCharacter0x2e75': '\xe2\xb9\xb5',
'UnicodeCharacter0x2e72': '\xe2\xb9\xb2',
'UnicodeCharacter0x2e73': '\xe2\xb9\xb3',
'UnicodeCharacter0x2e70': '\xe2\xb9\xb0',
'UnicodeCharacter0x2e71': '\xe2\xb9\xb1',
'CjkUnifiedIdeograph943b': '\xe9\x90\xbb',
'UnicodeCharacter0x2e78': '\xe2\xb9\xb8',
'UnicodeCharacter0x2e79': '\xe2\xb9\xb9',
'GreekRhoSymbol': '\xcf\xb1',
'CyrillicSmallLetterIeWithGrave': '\xd1\x90',
'CjkUnifiedIdeograph549f': '\xe5\x92\x9f',
'CjkUnifiedIdeograph549d': '\xe5\x92\x9d',
'CjkUnifiedIdeograph549e': '\xe5\x92\x9e',
'CjkUnifiedIdeograph549b': '\xe5\x92\x9b',
'CjkUnifiedIdeograph549c': '\xe5\x92\x9c',
'CjkUnifiedIdeograph943c': '\xe9\x90\xbc',
'CjkUnifiedIdeograph549a': '\xe5\x92\x9a',
'heta': '\xcd\xb1',
'HangulSyllableHya': '\xed\x96\x90',
'HangulSyllableBbweolg': '\xeb\xbf\xb1',
'CjkUnifiedIdeograph896e': '\xe8\xa5\xae',
'HangulSyllableBbweolb': '\xeb\xbf\xb3',
'UnicodeCharacter0xf685': '\xef\x9a\x85',
'HangulSyllableBbweolm': '\xeb\xbf\xb2',
'HangulSyllableBbweolh': '\xeb\xbf\xb7',
'KannadaVowelSignIi': '\xe0\xb3\x80',
'SaurashtraLetterU': '\xea\xa2\x86',
'CjkUnifiedIdeograph896f': '\xe8\xa5\xaf',
'LatinSmallLetterEWithHookAbove': '\xe1\xba\xbb',
'LatinCapitalLetterNWithCaron': '\xc5\x87',
'HangulSyllableGyeobs': '\xea\xb2\xba',
'HangulSyllableWaet': '\xec\x99\xb5',
'UnicodeCharacter0x2e7f': '\xe2\xb9\xbf',
'HangulSyllableWaek': '\xec\x99\xb4',
'UnicodeCharacter0x2e7d': '\xe2\xb9\xbd',
'UnicodeCharacter0x2e7e': '\xe2\xb9\xbe',
'UnicodeCharacter0x2e7b': '\xe2\xb9\xbb',
'UnicodeCharacter0x2e7c': '\xe2\xb9\xbc',
'SquareEsukuudo': '\xe3\x8c\x87',
'UnicodeCharacter0x2e7a': '\xe2\xb9\xba',
'SaurashtraLetterO': '\xea\xa2\x8f',
'HangulSyllableWaec': '\xec\x99\xb3',
'UnicodeCharacter0xf683': '\xef\x9a\x83',
'HangulSyllableWaeg': '\xec\x99\x9d',
'SaurashtraLetterI': '\xea\xa2\x84',
'HangulSyllableDyin': '\xeb\x93\xbc',
'CyrillicSmallLetterEnWithTail': '\xd3\x8a',
'HangulSyllableDyil': '\xeb\x94\x80',
'HangulSyllableDyik': '\xeb\x94\x90',
'HangulSyllableDyij': '\xeb\x94\x8e',
'HangulSyllableDyih': '\xeb\x94\x93',
'HangulSyllableDyig': '\xeb\x93\xb9',
'HangulSyllableDyid': '\xeb\x93\xbf',
'ThreeDotPunctuation': '\xe2\x81\x96',
'HangulSyllableDyib': '\xeb\x94\x89',
'HangulSyllableBya': '\xeb\xb1\x8c',
'HangulSyllableSop': '\xec\x86\xa6',
'TeluguDigitFive': '\xe0\xb1\xab',
'HangulSyllableDyit': '\xeb\x94\x91',
'HangulSyllableDyis': '\xeb\x94\x8b',
'HangulSyllableDyip': '\xeb\x94\x92',
'CjkUnifiedIdeograph9139': '\xe9\x84\xb9',
'CjkUnifiedIdeograph4862': '\xe4\xa1\xa2',
'CjkUnifiedIdeograph90e6': '\xe9\x83\xa6',
'CjkUnifiedIdeograph9134': '\xe9\x84\xb4',
'CjkUnifiedIdeograph9137': '\xe9\x84\xb7',
'CjkUnifiedIdeograph9136': '\xe9\x84\xb6',
'CjkUnifiedIdeograph9131': '\xe9\x84\xb1',
'CjkUnifiedIdeograph9130': '\xe9\x84\xb0',
'CjkUnifiedIdeograph9133': '\xe9\x84\xb3',
'CjkUnifiedIdeograph9132': '\xe9\x84\xb2',
'UnicodeCharacter0xd898': '\xed\xa2\x98',
'HangulSyllableJjweonj': '\xec\xad\xa9',
'CjkUnifiedIdeograph7552': '\xe7\x95\x92',
'HangulSyllableKoebs': '\xec\xbe\xba',
'CopticSmallLetterOldNubianNgi': '\xe2\xb3\x9f',
'HangulSyllableHyu': '\xed\x9c\xb4',
'CjkUnifiedIdeograph90e7': '\xe9\x83\xa7',
'HangulSyllableYogs': '\xec\x9a\x97',
'UnicodeCharacter0xab4': '\xe0\xaa\xb4',
'UnicodeCharacter0xdd54': '\xed\xb5\x94',
'HangulSyllablePeot': '\xed\x8e\x95',
'LatinCapitalLetterDWithCedilla': '\xe1\xb8\x90',
'HangulSyllableSog': '\xec\x86\x8d',
'HangulSyllableYogg': '\xec\x9a\x96',
'ArabicLetterFehIsolatedForm': '\xef\xbb\x91',
'NewTaiLueLetterLowYa': '\xe1\xa6\x8d',
'WhiteRightpointingSmallTriangle': '\xe2\x96\xb9',
'UnicodeCharacter0xd896': '\xed\xa2\x96',
'CanadianSyllabicsBlackfootWi': '\xe1\x96\xb5',
'TeluguLetterUu': '\xe0\xb0\x8a',
'EthiopicSyllableSzee': '\xe1\x88\xa4',
'GlagoliticCapitalLetterLatinateMyslite': '\xe2\xb0\xae',
'CjkUnifiedIdeograph913e': '\xe9\x84\xbe',
'CjkUnifiedIdeograph913d': '\xe9\x84\xbd',
'CanadianSyllabicsBlackfootWa': '\xe1\x96\xb7',
'CjkUnifiedIdeograph913f': '\xe9\x84\xbf',
'CjkUnifiedIdeograph913a': '\xe9\x84\xba',
'YiSyllableNep': '\xea\x86\x90',
'CanadianSyllabicsBlackfootWe': '\xe1\x96\xb4',
'CjkUnifiedIdeograph913b': '\xe9\x84\xbb',
'HangulSyllableDonj': '\xeb\x8f\x89',
'CjkUnifiedIdeograph90e5': '\xe9\x83\xa5',
'CjkUnifiedIdeograph77d9': '\xe7\x9f\x99',
'GreekCapitalLetterIotaWithVaria': '\xe1\xbf\x9a',
'CjkCompatibilityIdeographf9f6': '\xef\xa7\xb6',
'CjkUnifiedIdeograph664b': '\xe6\x99\x8b',
'UnicodeCharacter0x889': '\xe0\xa2\x89',
'UnicodeCharacter0x888': '\xe0\xa2\x88',
'UnicodeCharacter0x887': '\xe0\xa2\x87',
'DentistrySymbolLightUpAndHorizontalWithWave': '\xe2\x8f\x88',
'UnicodeCharacter0x885': '\xe0\xa2\x85',
'CjkUnifiedIdeograph4512': '\xe4\x94\x92',
'UnicodeCharacter0x883': '\xe0\xa2\x83',
'UnicodeCharacter0x882': '\xe0\xa2\x82',
'UnicodeCharacter0x881': '\xe0\xa2\x81',
'UnicodeCharacter0x880': '\xe0\xa2\x80',
'HangulSyllableDdaenh': '\xeb\x95\x92',
'CjkUnifiedIdeograph986c': '\xe9\xa1\xac',
'MongolianDigitNine': '\xe1\xa0\x99',
'CjkUnifiedIdeograph986a': '\xe9\xa1\xaa',
'CjkUnifiedIdeograph986f': '\xe9\xa1\xaf',
'CjkUnifiedIdeograph986e': '\xe9\xa1\xae',
'CjkUnifiedIdeograph986d': '\xe9\xa1\xad',
'CjkUnifiedIdeograph9791': '\xe9\x9e\x91',
'CjkUnifiedIdeograph9790': '\xe9\x9e\x90',
'CjkUnifiedIdeograph9793': '\xe9\x9e\x93',
'CjkUnifiedIdeograph9792': '\xe9\x9e\x92',
'CjkUnifiedIdeograph9795': '\xe9\x9e\x95',
'CjkUnifiedIdeograph9794': '\xe9\x9e\x94',
'CjkUnifiedIdeograph9797': '\xe9\x9e\x97',
'ZNotationRightBindingBracket': '\xe2\xa6\x8a',
'CjkUnifiedIdeograph537c': '\xe5\x8d\xbc',
'CjkUnifiedIdeograph537b': '\xe5\x8d\xbb',
'CjkUnifiedIdeograph537a': '\xe5\x8d\xba',
'CjkUnifiedIdeograph537f': '\xe5\x8d\xbf',
'CjkUnifiedIdeograph537e': '\xe5\x8d\xbe',
'CjkUnifiedIdeograph537d': '\xe5\x8d\xbd',
'UnicodeCharacter0xa7b2': '\xea\x9e\xb2',
'TibetanSignYarTshes': '\xe0\xbc\xbe',
'HexagramForTheReceptiveEarth': '\xe4\xb7\x81',
'YiSyllableNbiex': '\xea\x81\xba',
'UnicodeCharacter0x88f': '\xe0\xa2\x8f',
'UnicodeCharacter0x88e': '\xe0\xa2\x8e',
'UnicodeCharacter0x88d': '\xe0\xa2\x8d',
'UnicodeCharacter0x88c': '\xe0\xa2\x8c',
'UnicodeCharacter0x88b': '\xe0\xa2\x8b',
'UnicodeCharacter0x88a': '\xe0\xa2\x8a',
'CjkUnifiedIdeograph77d5': '\xe7\x9f\x95',
'HangulSyllableHanj': '\xed\x95\x9d',
'CjkUnifiedIdeograph7220': '\xe7\x88\xa0',
'CjkUnifiedIdeograph979a': '\xe9\x9e\x9a',
'CjkUnifiedIdeograph5379': '\xe5\x8d\xb9',
'CjkUnifiedIdeograph5378': '\xe5\x8d\xb8',
'CjkUnifiedIdeograph979e': '\xe9\x9e\x9e',
'CjkUnifiedIdeograph979d': '\xe9\x9e\x9d',
'CjkUnifiedIdeograph979f': '\xe9\x9e\x9f',
'CjkUnifiedIdeograph5373': '\xe5\x8d\xb3',
'CjkUnifiedIdeograph5372': '\xe5\x8d\xb2',
'CjkUnifiedIdeograph5371': '\xe5\x8d\xb1',
'CjkUnifiedIdeograph5370': '\xe5\x8d\xb0',
'CjkUnifiedIdeograph5377': '\xe5\x8d\xb7',
'CjkUnifiedIdeograph5376': '\xe5\x8d\xb6',
'CjkUnifiedIdeograph5375': '\xe5\x8d\xb5',
'CjkUnifiedIdeograph5374': '\xe5\x8d\xb4',
'UnicodeCharacter0xf68d': '\xef\x9a\x8d',
'CjkUnifiedIdeograph9869': '\xe9\xa1\xa9',
'CjkUnifiedIdeograph9868': '\xe9\xa1\xa8',
'HangulSyllableHang': '\xed\x95\xad',
'CjkUnifiedIdeograph9863': '\xe9\xa1\xa3',
'CjkUnifiedIdeograph9862': '\xe9\xa1\xa2',
'EnQuad': '\xe2\x80\x80',
'CjkUnifiedIdeograph9860': '\xe9\xa1\xa0',
'CjkUnifiedIdeograph9867': '\xe9\xa1\xa7',
'CjkUnifiedIdeograph9866': '\xe9\xa1\xa6',
'CjkUnifiedIdeograph9865': '\xe9\xa1\xa5',
'CjkUnifiedIdeograph9864': '\xe9\xa1\xa4',
'YiSyllableDdot': '\xea\x85\x84',
'CjkUnifiedIdeograph8961': '\xe8\xa5\xa1',
'UnicodeCharacter0xaade': '\xea\xab\x9e',
'HangulSyllableOd': '\xec\x98\xab',
'UnicodeCharacter0xf68a': '\xef\x9a\x8a',
'ArabicLigatureYehWithJeemIsolatedForm': '\xef\xb1\x95',
'GujaratiLetterSha': '\xe0\xaa\xb6',
'CjkUnifiedIdeograph9e91': '\xe9\xba\x91',
'SouthWestWhiteArrow': '\xe2\xac\x83',
'CjkUnifiedIdeograph677b': '\xe6\x9d\xbb',
'CjkUnifiedIdeograph677c': '\xe6\x9d\xbc',
'CjkUnifiedIdeograph677a': '\xe6\x9d\xba',
'CjkUnifiedIdeograph677f': '\xe6\x9d\xbf',
'CjkUnifiedIdeograph677d': '\xe6\x9d\xbd',
'CjkUnifiedIdeograph677e': '\xe6\x9d\xbe',
'HangulSyllableCilm': '\xec\xb9\xa2',
'HangulSyllableCilh': '\xec\xb9\xa7',
'HexagramForWaiting': '\xe4\xb7\x84',
'HangulSyllableCilg': '\xec\xb9\xa1',
'HangulSyllableCilb': '\xec\xb9\xa3',
'HangulSyllableCilt': '\xec\xb9\xa5',
'HangulSyllableCilp': '\xec\xb9\xa6',
'HangulSyllableCils': '\xec\xb9\xa4',
'CjkUnifiedIdeograph6778': '\xe6\x9d\xb8',
'CjkUnifiedIdeograph6779': '\xe6\x9d\xb9',
'CjkUnifiedIdeograph6772': '\xe6\x9d\xb2',
'CjkUnifiedIdeograph6773': '\xe6\x9d\xb3',
'CjkUnifiedIdeograph6770': '\xe6\x9d\xb0',
'CjkUnifiedIdeograph6771': '\xe6\x9d\xb1',
'CjkUnifiedIdeograph6776': '\xe6\x9d\xb6',
'CjkUnifiedIdeograph6777': '\xe6\x9d\xb7',
'CjkUnifiedIdeograph6774': '\xe6\x9d\xb4',
'CjkUnifiedIdeograph6775': '\xe6\x9d\xb5',
'OghamReversedFeatherMark': '\xe1\x9a\x9c',
'HangulSyllableJwep': '\xec\xa5\x8e',
'UnicodeCharacter0xdd7': '\xe0\xb7\x97',
'UnicodeCharacter0xdd5': '\xe0\xb7\x95',
'LatinCapitalLetterFWithHook': '\xc6\x91',
'KhmerVowelSignY': '\xe1\x9e\xb9',
'HangulSyllableRyunh': '\xeb\xa5\x9e',
'HangulSyllableRyunj': '\xeb\xa5\x9d',
'HangulSyllablePwaeg': '\xed\x90\xa5',
'HangulSyllableRyung': '\xeb\xa5\xad',
'HangulSyllableOt': '\xec\x98\xbd',
'HangulChoseongSsangrieul': '\xe1\x84\x99',
'BottomLeftHalfBracket': '\xe2\xb8\xa4',
'BalineseLetterNga': '\xe1\xac\x97',
'UnicodeCharacter0xdd0b': '\xed\xb4\x8b',
'HangulSyllableJjweong': '\xec\xad\xb9',
'CjkUnifiedIdeograph4954': '\xe4\xa5\x94',
'CjkUnifiedIdeograph4955': '\xe4\xa5\x95',
'CjkUnifiedIdeograph4956': '\xe4\xa5\x96',
'CjkUnifiedIdeograph4957': '\xe4\xa5\x97',
'CjkUnifiedIdeograph4950': '\xe4\xa5\x90',
'CjkUnifiedIdeograph4951': '\xe4\xa5\x91',
'CjkUnifiedIdeograph4952': '\xe4\xa5\x92',
'CjkUnifiedIdeograph4953': '\xe4\xa5\x93',
'HangulSyllableSyaebs': '\xec\x84\x92',
'UnicodeCharacter0xdd0d': '\xed\xb4\x8d',
'CjkUnifiedIdeograph4958': '\xe4\xa5\x98',
'CjkUnifiedIdeograph4959': '\xe4\xa5\x99',
'UnicodeCharacter0xdd0e': '\xed\xb4\x8e',
'LaoVowelSignE': '\xe0\xbb\x80',
'CombiningCyrillicTenMillionsSign': '\xea\x99\xb0',
'TifinaghLetterTuaregYazh': '\xe2\xb5\x8c',
'CjkUnifiedIdeograph99ff': '\xe9\xa7\xbf',
'LaoVowelSignO': '\xe0\xbb\x82',
'CjkUnifiedIdeograph99fe': '\xe9\xa7\xbe',
'LaoVowelSignI': '\xe0\xba\xb4',
'CjkUnifiedIdeograph99fc': '\xe9\xa7\xbc',
'CjkUnifiedIdeograph99fa': '\xe9\xa7\xba',
'NewTaiLueLetterLowTha': '\xe1\xa6\x92',
'CjkUnifiedIdeograph9ef2': '\xe9\xbb\xb2',
'HebrewLetterLamed': '\xd7\x9c',
'CjkUnifiedIdeograph742a': '\xe7\x90\xaa',
'CjkUnifiedIdeograph742c': '\xe7\x90\xac',
'CjkUnifiedIdeograph742b': '\xe7\x90\xab',
'CjkUnifiedIdeograph742e': '\xe7\x90\xae',
'CjkUnifiedIdeograph742d': '\xe7\x90\xad',
'CjkUnifiedIdeograph742f': '\xe7\x90\xaf',
'CjkUnifiedIdeograph3d0c': '\xe3\xb4\x8c',
'CjkUnifiedIdeograph3d0b': '\xe3\xb4\x8b',
'CjkUnifiedIdeograph3d0a': '\xe3\xb4\x8a',
'HangulSyllableSswem': '\xec\x92\xac',
'Heta': '\xcd\xb0',
'CjkUnifiedIdeograph3d0f': '\xe3\xb4\x8f',
'CjkUnifiedIdeograph3d0e': '\xe3\xb4\x8e',
'CjkUnifiedIdeograph3d0d': '\xe3\xb4\x8d',
'HangulSyllableJjaep': '\xec\xa8\x92',
'BopomofoLetterAu': '\xe3\x84\xa0',
'HangulSyllableJjaes': '\xec\xa8\x8b',
'HangulSyllableBbilg': '\xec\x82\x99',
'ParenthesizedIdeographRepresent': '\xe3\x88\xb9',
'CjkUnifiedIdeograph99f8': '\xe9\xa7\xb8',
'CjkUnifiedIdeograph99f9': '\xe9\xa7\xb9',
'CjkUnifiedIdeograph99f6': '\xe9\xa7\xb6',
'CjkUnifiedIdeograph99f7': '\xe9\xa7\xb7',
'CjkUnifiedIdeograph99f4': '\xe9\xa7\xb4',
'CjkUnifiedIdeograph99f5': '\xe9\xa7\xb5',
'CjkUnifiedIdeograph99f2': '\xe9\xa7\xb2',
'CjkUnifiedIdeograph99f3': '\xe9\xa7\xb3',
'TaiLeLetterA': '\xe1\xa5\xa3',
'CjkUnifiedIdeograph99f1': '\xe9\xa7\xb1',
'CjkUnifiedIdeograph495d': '\xe4\xa5\x9d',
'CjkUnifiedIdeograph495e': '\xe4\xa5\x9e',
'CjkUnifiedIdeograph495f': '\xe4\xa5\x9f',
'HangulSyllableBbilp': '\xec\x82\x9e',
'HangulSyllableJjaed': '\xec\xa7\xbf',
'CjkUnifiedIdeograph495a': '\xe4\xa5\x9a',
'CjkUnifiedIdeograph495b': '\xe4\xa5\x9b',
'CjkUnifiedIdeograph495c': '\xe4\xa5\x9c',
'CyrillicCapitalLetterEsWithDescender': '\xd2\xaa',
'BopomofoLetterAm': '\xe3\x86\xb0',
'BopomofoLetterAn': '\xe3\x84\xa2',
'HangulSyllableJjaek': '\xec\xa8\x90',
'HangulSyllableJjael': '\xec\xa8\x80',
'BopomofoLetterAi': '\xe3\x84\x9e',
'HangulSyllableJjaen': '\xec\xa7\xbc',
'UnicodeCharacter0xe868': '\xee\xa1\xa8',
'CjkUnifiedIdeograph3d03': '\xe3\xb4\x83',
'CjkUnifiedIdeograph3d02': '\xe3\xb4\x82',
'CjkUnifiedIdeograph3d01': '\xe3\xb4\x81',
'CjkUnifiedIdeograph3d00': '\xe3\xb4\x80',
'CjkUnifiedIdeograph3d07': '\xe3\xb4\x87',
'CjkUnifiedIdeograph3d06': '\xe3\xb4\x86',
'CjkUnifiedIdeograph3d05': '\xe3\xb4\x85',
'CjkUnifiedIdeograph3d04': '\xe3\xb4\x84',
'CjkUnifiedIdeograph3d09': '\xe3\xb4\x89',
'CjkUnifiedIdeograph3d08': '\xe3\xb4\x88',
'CjkUnifiedIdeograph7429': '\xe7\x90\xa9',
'CjkUnifiedIdeograph7428': '\xe7\x90\xa8',
'CjkRadicalHeartOne': '\xe2\xba\x96',
'HangulSyllableHwih': '\xed\x9c\xb3',
'CjkUnifiedIdeograph7421': '\xe7\x90\xa1',
'CjkUnifiedIdeograph7420': '\xe7\x90\xa0',
'CjkUnifiedIdeograph7423': '\xe7\x90\xa3',
'CjkUnifiedIdeograph7422': '\xe7\x90\xa2',
'CjkUnifiedIdeograph7425': '\xe7\x90\xa5',
'CjkUnifiedIdeograph7424': '\xe7\x90\xa4',
'CjkUnifiedIdeograph7427': '\xe7\x90\xa7',
'CjkUnifiedIdeograph7426': '\xe7\x90\xa6',
'CjkUnifiedIdeograph4946': '\xe4\xa5\x86',
'UnicodeCharacter0xaa8f': '\xea\xaa\x8f',
'CjkUnifiedIdeograph9310': '\xe9\x8c\x90',
'NeitherLessthanNorGreaterthan': '\xe2\x89\xb8',
'HangulSyllableHwil': '\xed\x9c\xa0',
'UnicodeCharacter0x18b9': '\xe1\xa2\xb9',
'UnicodeCharacter0x18b8': '\xe1\xa2\xb8',
'UnicodeCharacter0x18b5': '\xe1\xa2\xb5',
'UnicodeCharacter0x18b4': '\xe1\xa2\xb4',
'UnicodeCharacter0x18b7': '\xe1\xa2\xb7',
'UnicodeCharacter0x18b6': '\xe1\xa2\xb6',
'UnicodeCharacter0x18b1': '\xe1\xa2\xb1',
'UnicodeCharacter0x18b0': '\xe1\xa2\xb0',
'UnicodeCharacter0x18b3': '\xe1\xa2\xb3',
'UnicodeCharacter0x18b2': '\xe1\xa2\xb2',
'HangulSyllableKot': '\xec\xbd\xad',
'HangulSyllableKos': '\xec\xbd\xa7',
'HangulSyllableKop': '\xec\xbd\xae',
'HangulSyllableHwip': '\xed\x9c\xb2',
'HangulSyllableByeoss': '\xeb\xb3\x90',
'HangulSyllableHwis': '\xed\x9c\xab',
'HangulSyllableKog': '\xec\xbd\x95',
'LepchaVowelSignUu': '\xe1\xb0\xab',
'HangulSyllableKoe': '\xec\xbe\xa8',
'HangulSyllableKob': '\xec\xbd\xa5',
'HangulSyllableKoc': '\xec\xbd\xab',
'HangulSyllableKon': '\xec\xbd\x98',
'CyrillicCapitalLetterSoftEm': '\xea\x99\xa6',
'HangulSyllableKol': '\xec\xbd\x9c',
'HangulSyllableKom': '\xec\xbd\xa4',
'HangulSyllableDdyebs': '\xeb\x98\x86',
'HangulSyllableKok': '\xec\xbd\xac',
'HangulSyllableKoh': '\xec\xbd\xaf',
'HangulSyllableHwit': '\xed\x9c\xb1',
'HangulSyllablePwem': '\xed\x93\x80',
'CjkUnifiedIdeograph834a': '\xe8\x8d\x8a',
'HangulSyllableDyeos': '\xeb\x8e\x9f',
'HangulSyllablePwen': '\xed\x92\xb4',
'HangulSyllableDyeot': '\xeb\x8e\xa5',
'HangulSyllablePwek': '\xed\x93\x88',
'HangulSyllablePwej': '\xed\x93\x86',
'UnicodeCharacter0x18be': '\xe1\xa2\xbe',
'UnicodeCharacter0x18bd': '\xe1\xa2\xbd',
'HangulSyllablePweg': '\xed\x92\xb1',
'UnicodeCharacter0x18bf': '\xe1\xa2\xbf',
'UnicodeCharacter0x18ba': '\xe1\xa2\xba',
'CircledIdeographEarth': '\xe3\x8a\x8f',
'UnicodeCharacter0x18bb': '\xe1\xa2\xbb',
'CombiningEnclosingCircleBackslash': '\xe2\x83\xa0',
'HangulSyllableDyeoc': '\xeb\x8e\xa3',
'HangulSyllableGes': '\xea\xb2\x9f',
'AplFunctionalSymbolIotaUnderbar': '\xe2\x8d\xb8',
'HangulSyllableDyeog': '\xeb\x8e\x8d',
'GreekCapitalLetterTheta': '\xce\x98',
'HangulSyllableGet': '\xea\xb2\xa5',
'HangulSyllableDyeok': '\xeb\x8e\xa4',
'HangulSyllableDyeoj': '\xeb\x8e\xa2',
'HangulSyllableDyeom': '\xeb\x8e\x9c',
'HangulSyllableDyeol': '\xeb\x8e\x94',
'HangulSyllablePwes': '\xed\x93\x83',
'HangulSyllableGeu': '\xea\xb7\xb8',
'ArmenianSmallLetterEh': '\xd5\xa7',
'UnicodeCharacter0xd89d': '\xed\xa2\x9d',
'UnicodeCharacter0xf4d9': '\xef\x93\x99',
'HangulSyllablePags': '\xed\x8c\x8f',
'LepchaDigitZero': '\xe1\xb1\x80',
'UnicodeCharacter0xe077': '\xee\x81\xb7',
'UnicodeCharacter0xe8de': '\xee\xa3\x9e',
'HangulSyllableRwinj': '\xeb\xa5\x81',
'ArabicLetterFarsiYehFinalForm': '\xef\xaf\xbd',
'HangulSyllableHyebs': '\xed\x98\xae',
'LatinSmallLetterDWithCaron': '\xc4\x8f',
'ArmenianSmallLetterEt': '\xd5\xa8',
'ArabicSmallLowSeen': '\xdb\xa3',
'HangulSyllableBbeog': '\xeb\xbb\x91',
'TeluguLetterTha': '\xe0\xb0\xa5',
'SylotiNagriPoetryMark4': '\xea\xa0\xab',
'SylotiNagriPoetryMark3': '\xea\xa0\xaa',
'SylotiNagriPoetryMark2': '\xea\xa0\xa9',
'CjkUnifiedIdeograph4397': '\xe4\x8e\x97',
'HangulSyllableBbeon': '\xeb\xbb\x94',
'HangulSyllableBbeol': '\xeb\xbb\x98',
'HangulSyllableBbeom': '\xeb\xbb\xa0',
'HangulSyllableBbeoj': '\xeb\xbb\xa6',
'HangulSyllableBbeok': '\xeb\xbb\xa8',
'HangulSyllableBbeoh': '\xeb\xbb\xab',
'HangulSyllableBbeot': '\xeb\xbb\xa9',
'HangulSyllableBbeos': '\xeb\xbb\xa3',
'HangulSyllableBbeop': '\xeb\xbb\xaa',
'ArabicLigatureQafWithMeemWithHahInitialForm': '\xef\xb6\xb4',
'AlmostEqualTo': '\xe2\x89\x88',
'GlagoliticSmallLetterNashi': '\xe2\xb1\x80',
'UnicodeCharacter0xdd01': '\xed\xb4\x81',
'MasuMark': '\xe3\x80\xbc',
'HangulSyllableRyaep': '\xeb\x9f\xaa',
'CjkUnifiedIdeograph34ed': '\xe3\x93\xad',
'CjkUnifiedIdeograph34ee': '\xe3\x93\xae',
'CjkUnifiedIdeograph34ef': '\xe3\x93\xaf',
'CjkUnifiedIdeograph34ea': '\xe3\x93\xaa',
'CjkUnifiedIdeograph34eb': '\xe3\x93\xab',
'CjkUnifiedIdeograph34ec': '\xe3\x93\xac',
'ParenthesizedIdeographSupervise': '\xe3\x88\xbc',
'CjkCompatibilityIdeographf9a5': '\xef\xa6\xa5',
'CjkUnifiedIdeograph77df': '\xe7\x9f\x9f',
'RunicLetterIcelandicyr': '\xe1\x9b\xa8',
'HangulSyllableDeugg': '\xeb\x93\x9e',
'CjkUnifiedIdeograph50f8': '\xe5\x83\xb8',
'VaiSyllableEen': '\xea\x94\x81',
'EthiopicSyllableGlottalEe': '\xe1\x8a\xa4',
'UnicodeCharacter0xf2d1': '\xef\x8b\x91',
'UnicodeCharacter0xe072': '\xee\x81\xb2',
'CjkUnifiedIdeograph7511': '\xe7\x94\x91',
'CjkUnifiedIdeograph6cf4': '\xe6\xb3\xb4',
'CjkUnifiedIdeograph9efa': '\xe9\xbb\xba',
'HangulChoseongSsangsios': '\xe1\x84\x8a',
'CjkUnifiedIdeograph7510': '\xe7\x94\x90',
'GeorgianLetterPhar': '\xe1\x83\xa4',
'HangulSyllableGej': '\xea\xb2\xa2',
'CjkUnifiedIdeograph7512': '\xe7\x94\x92',
'UnicodeCharacter0xe073': '\xee\x81\xb3',
'HebrewAccentRevia': '\xd6\x97',
'CjkUnifiedIdeograph7515': '\xe7\x94\x95',
'SylotiNagriSignHasanta': '\xea\xa0\x86',
'UnicodeCharacter0xf7a2': '\xef\x9e\xa2',
'HangulSyllableGem': '\xea\xb2\x9c',
'BoxDrawingsDoubleUpAndLeft': '\xe2\x95\x9d',
'BraillePatternDots12578': '\xe2\xa3\x93',
'UnicodeCharacter0xa6a9': '\xea\x9a\xa9',
'CjkUnifiedIdeograph34e8': '\xe3\x93\xa8',
'CjkUnifiedIdeograph34e9': '\xe3\x93\xa9',
'CjkUnifiedIdeograph34e4': '\xe3\x93\xa4',
'CjkUnifiedIdeograph34e5': '\xe3\x93\xa5',
'CjkUnifiedIdeograph34e6': '\xe3\x93\xa6',
'CjkUnifiedIdeograph34e7': '\xe3\x93\xa7',
'CjkUnifiedIdeograph34e0': '\xe3\x93\xa0',
'CjkUnifiedIdeograph34e1': '\xe3\x93\xa1',
'CjkUnifiedIdeograph34e2': '\xe3\x93\xa2',
'CjkUnifiedIdeograph34e3': '\xe3\x93\xa3',
'UnicodeCharacter0xe070': '\xee\x81\xb0',
'CjkCompatibilityIdeographf9a0': '\xef\xa6\xa0',
'UnicodeCharacter0x17df': '\xe1\x9f\x9f',
'HangulSyllableDdong': '\xeb\x98\xa5',
'UnicodeCharacter0xa6a2': '\xea\x9a\xa2',
'UnicodeCharacter0x9fd9': '\xe9\xbf\x99',
'UnicodeCharacter0x9fd8': '\xe9\xbf\x98',
'LatinSmallLetterIWithTildeBelow': '\xe1\xb8\xad',
'MetricalTriseme': '\xe2\x8f\x97',
'UnicodeCharacter0x9fd5': '\xe9\xbf\x95',
'UnicodeCharacter0x9fd4': '\xe9\xbf\x94',
'UnicodeCharacter0x9fd7': '\xe9\xbf\x97',
'UnicodeCharacter0x9fd6': '\xe9\xbf\x96',
'UnicodeCharacter0x9fd1': '\xe9\xbf\x91',
'UnicodeCharacter0x9fd0': '\xe9\xbf\x90',
'UnicodeCharacter0x9fd3': '\xe9\xbf\x93',
'UnicodeCharacter0x9fd2': '\xe9\xbf\x92',
'UnicodeCharacter0xdd08': '\xed\xb4\x88',
'ModifierLetterMiddleDoubleAcuteAccent': '\xcb\xb6',
'UnicodeCharacter0xa6a1': '\xea\x9a\xa1',
'UnicodeCharacter0xf0e7': '\xef\x83\xa7',
'UnicodeCharacter0xdd09': '\xed\xb4\x89',
'UnicodeCharacter0xa6a6': '\xea\x9a\xa6',
'UnicodeCharacter0xf18a': '\xef\x86\x8a',
'UnicodeCharacter0xa6a7': '\xea\x9a\xa7',
'UnicodeCharacter0xa6a4': '\xea\x9a\xa4',
'HangulSyllableGgwalt': '\xea\xbd\x95',
'UnicodeCharacter0xa6a5': '\xea\x9a\xa5',
'HangulSyllableCeod': '\xec\xb2\x9f',
'CjkUnifiedIdeograph6b64': '\xe6\xad\xa4',
'HangulSyllableCeog': '\xec\xb2\x99',
'HangulSyllableCeob': '\xec\xb2\xa9',
'HangulSyllableCeoc': '\xec\xb2\xaf',
'UnicodeCharacter0x9fde': '\xe9\xbf\x9e',
'UnicodeCharacter0x9fdd': '\xe9\xbf\x9d',
'HangulSyllableCeon': '\xec\xb2\x9c',
'UnicodeCharacter0x9fdf': '\xe9\xbf\x9f',
'UnicodeCharacter0x9fda': '\xe9\xbf\x9a',
'UnicodeCharacter0x9fdc': '\xe9\xbf\x9c',
'UnicodeCharacter0x9fdb': '\xe9\xbf\x9b',
'HangulSyllableCeot': '\xec\xb2\xb1',
'UnicodeCharacter0xdd00': '\xed\xb4\x80',
'HangulSyllableCeop': '\xec\xb2\xb2',
'HangulSyllableCeos': '\xec\xb2\xab',
'UnicodeCharacter0xdd02': '\xed\xb4\x82',
'ArabicLetterDadMedialForm': '\xef\xbb\x80',
'CircledNumberThirteen': '\xe2\x91\xac',
'MongolianLetterHaa': '\xe1\xa0\xbe',
'PhagspaSubjoinedLetterYa': '\xea\xa1\xa8',
'RightwardsArrowWithDottedStem': '\xe2\xa4\x91',
'CombiningMinusSignBelow': '\xcc\xa0',
'TibetanLetterDda': '\xe0\xbd\x8c',
'HangulSyllableJying': '\xec\xa6\xb9',
'UnicodeCharacter0xdd06': '\xed\xb4\x86',
'AfghaniSign': '\xd8\x8b',
'HangulSyllableUss': '\xec\x9b\x84',
'UnicodeCharacter0xdd07': '\xed\xb4\x87',
'ArabicLigatureKhahWithMeemInitialForm': '\xef\xb2\xac',
'HangulSyllableGgaelh': '\xea\xb9\xb7',
'CjkUnifiedIdeograph822f': '\xe8\x88\xaf',
'CjkUnifiedIdeograph822d': '\xe8\x88\xad',
'CjkUnifiedIdeograph822e': '\xe8\x88\xae',
'CjkUnifiedIdeograph822b': '\xe8\x88\xab',
'CjkUnifiedIdeograph822c': '\xe8\x88\xac',
'CjkUnifiedIdeograph822a': '\xe8\x88\xaa',
'UnicodeCharacter0xf3ab': '\xef\x8e\xab',
'UnicodeCharacter0xf3ac': '\xef\x8e\xac',
'UnicodeCharacter0xf3aa': '\xef\x8e\xaa',
'UnicodeCharacter0xf3af': '\xef\x8e\xaf',
'HangulSyllableDying': '\xeb\x94\x8d',
'UnicodeCharacter0xf3ad': '\xef\x8e\xad',
'LatinSmallLetterNWithAcute': '\xc5\x84',
'CjkUnifiedIdeograph6be1': '\xe6\xaf\xa1',
'CjkUnifiedIdeograph6be0': '\xe6\xaf\xa0',
'CjkUnifiedIdeograph6be3': '\xe6\xaf\xa3',
'CjkUnifiedIdeograph6be2': '\xe6\xaf\xa2',
'CjkUnifiedIdeograph6be5': '\xe6\xaf\xa5',
'CjkUnifiedIdeograph6be4': '\xe6\xaf\xa4',
'CjkUnifiedIdeograph6be7': '\xe6\xaf\xa7',
'CjkUnifiedIdeograph6be6': '\xe6\xaf\xa6',
'CjkUnifiedIdeograph6be9': '\xe6\xaf\xa9',
'CjkUnifiedIdeograph6be8': '\xe6\xaf\xa8',
'HexagramForPeace': '\xe4\xb7\x8a',
'NkoLetterBa': '\xdf\x93',
'CjkUnifiedIdeograph6bec': '\xe6\xaf\xac',
'CjkUnifiedIdeograph6beb': '\xe6\xaf\xab',
'CjkUnifiedIdeograph6bee': '\xe6\xaf\xae',
'CjkUnifiedIdeograph6bed': '\xe6\xaf\xad',
'CjkUnifiedIdeograph6bef': '\xe6\xaf\xaf',
'CjkUnifiedIdeograph8226': '\xe8\x88\xa6',
'CjkUnifiedIdeograph8227': '\xe8\x88\xa7',
'CjkUnifiedIdeograph8224': '\xe8\x88\xa4',
'CjkUnifiedIdeograph8225': '\xe8\x88\xa5',
'CjkUnifiedIdeograph8222': '\xe8\x88\xa2',
'CjkUnifiedIdeograph8223': '\xe8\x88\xa3',
'CjkUnifiedIdeograph8220': '\xe8\x88\xa0',
'CjkUnifiedIdeograph8221': '\xe8\x88\xa1',
'GlagoliticCapitalLetterUku': '\xe2\xb0\x96',
'HangulSyllableDoem': '\xeb\x90\xa8',
'UnicodeCharacter0xf3a0': '\xef\x8e\xa0',
'UnicodeCharacter0xf3a1': '\xef\x8e\xa1',
'UnicodeCharacter0xf3a6': '\xef\x8e\xa6',
'UnicodeCharacter0xf3a7': '\xef\x8e\xa7',
'TriplePrime': '\xe2\x80\xb4',
'CjkUnifiedIdeograph8229': '\xe8\x88\xa9',
'UnicodeCharacter0xf639': '\xef\x98\xb9',
'UnicodeCharacter0xd7ea': '\xed\x9f\xaa',
'CjkUnifiedIdeograph751a': '\xe7\x94\x9a',
'UnicodeCharacter0xe078': '\xee\x81\xb8',
'HangulSyllableIbs': '\xec\x9e\x86',
'ModifierLetterLowLeftArrow': '\xcb\xbf',
'CjkUnifiedIdeograph751c': '\xe7\x94\x9c',
'CjkUnifiedIdeograph751b': '\xe7\x94\x9b',
'HangulSyllableBeolp': '\xeb\xb2\x92',
'LatinSmallLetterTWithCedilla': '\xc5\xa3',
'HangulSyllableBeols': '\xeb\xb2\x90',
'HangulSyllableBeolt': '\xeb\xb2\x91',
'CjkUnifiedIdeograph751d': '\xe7\x94\x9d',
'BraillePatternDots1345': '\xe2\xa0\x9d',
'BraillePatternDots1347': '\xe2\xa1\x8d',
'BraillePatternDots1346': '\xe2\xa0\xad',
'HangulSyllableDoeg': '\xeb\x90\x99',
'UnicodeCharacter0xf630': '\xef\x98\xb0',
'HangulSyllableBeolb': '\xeb\xb2\x8f',
'CjkUnifiedIdeograph751f': '\xe7\x94\x9f',
'BraillePatternDots1348': '\xe2\xa2\x8d',
'TeluguLetterVocalicRr': '\xe0\xb1\xa0',
'HangulSyllableBeolg': '\xeb\xb2\x8d',
'CjkUnifiedIdeograph4bdd': '\xe4\xaf\x9d',
'CjkUnifiedIdeograph4bde': '\xe4\xaf\x9e',
'SquareKyurii': '\xe3\x8c\x92',
'MongolianLetterChi': '\xe1\xa1\x82',
'CjkUnifiedIdeograph4bda': '\xe4\xaf\x9a',
'CjkUnifiedIdeograph4bdb': '\xe4\xaf\x9b',
'CjkUnifiedIdeograph4bdc': '\xe4\xaf\x9c',
'UnicodeCharacter0xd939': '\xed\xa4\xb9',
'MongolianLetterCha': '\xe1\xa0\xb4',
'CjkUnifiedIdeograph8343': '\xe8\x8d\x83',
'UnicodeCharacter0xa6ab': '\xea\x9a\xab',
'HebrewAccentDehi': '\xd6\xad',
'HangulSyllableBweoc': '\xeb\xb6\xb3',
'YiSyllableDiep': '\xea\x84\x86',
'UnicodeCharacter0xa6aa': '\xea\x9a\xaa',
'UnicodeCharacter0xa6af': '\xea\x9a\xaf',
'CjkUnifiedIdeograph8346': '\xe8\x8d\x86',
'RightwardsDoubleArrowtail': '\xe2\xa4\x9c',
'HangulSyllableBweod': '\xeb\xb6\xa3',
'CjkUnifiedIdeograph8349': '\xe8\x8d\x89',
'IdeographicDescriptionCharacterSurroundFromUpperRight': '\xe2\xbf\xb9',
'HangulSyllableBweog': '\xeb\xb6\x9d',
'CjkUnifiedIdeograph8348': '\xe8\x8d\x88',
'HangulSyllableEobs': '\xec\x97\x86',
'UnicodeCharacter0xa6ae': '\xea\x9a\xae',
'UnicodeCharacter0xf6f3': '\xef\x9b\xb3',
'HangulSyllableJjyaess': '\xec\xa9\x84',
'k': '\xce\xba',
'CjkUnifiedIdeograph4bd4': '\xe4\xaf\x94',
'CjkUnifiedIdeograph4bd5': '\xe4\xaf\x95',
'CjkUnifiedIdeograph4bd6': '\xe4\xaf\x96',
'CjkUnifiedIdeograph4bd7': '\xe4\xaf\x97',
'CjkUnifiedIdeograph4bd0': '\xe4\xaf\x90',
'CjkUnifiedIdeograph4bd1': '\xe4\xaf\x91',
'CjkUnifiedIdeograph4bd2': '\xe4\xaf\x92',
'CjkUnifiedIdeograph4bd3': '\xe4\xaf\x93',
'ChamLetterBha': '\xea\xa8\x9e',
'CjkUnifiedIdeograph4bd8': '\xe4\xaf\x98',
'CjkUnifiedIdeograph4bd9': '\xe4\xaf\x99',
'HangulSyllableSselt': '\xec\x8e\x91',
'UnicodeCharacter0x1a64': '\xe1\xa9\xa4',
'UnicodeCharacter0x1a65': '\xe1\xa9\xa5',
'UnicodeCharacter0x1a66': '\xe1\xa9\xa6',
'UnicodeCharacter0x1a67': '\xe1\xa9\xa7',
'UnicodeCharacter0x1a60': '\xe1\xa9\xa0',
'UnicodeCharacter0x1a61': '\xe1\xa9\xa1',
'UnicodeCharacter0x1a62': '\xe1\xa9\xa2',
'UnicodeCharacter0x1a63': '\xe1\xa9\xa3',
'TopCurlyBracket': '\xe2\x8f\x9e',
'UnicodeCharacter0x1a68': '\xe1\xa9\xa8',
'UnicodeCharacter0x1a69': '\xe1\xa9\xa9',
'UnicodeCharacter0xf07d': '\xef\x81\xbd',
'ArabicLetterTehInitialForm': '\xef\xba\x97',
'CjkUnifiedIdeograph976d': '\xe9\x9d\xad',
'CjkUnifiedIdeograph976e': '\xe9\x9d\xae',
'HangulSyllableKyaebs': '\xec\xbb\x9a',
'CjkUnifiedIdeograph4999': '\xe4\xa6\x99',
'UnicodeCharacter0xf7d7': '\xef\x9f\x97',
'UnicodeCharacter0xf7fc': '\xef\x9f\xbc',
'KangxiRadicalTooth': '\xe2\xbf\x92',
'UnicodeCharacter0xf7d6': '\xef\x9f\x96',
'ArabicLetterSeenWithThreeDotsBelow': '\xda\x9b',
'CjkUnifiedIdeograph7df5': '\xe7\xb7\xb5',
'CjkUnifiedIdeograph5df9': '\xe5\xb7\xb9',
'CjkUnifiedIdeograph5df8': '\xe5\xb7\xb8',
'UnicodeCharacter0xe565': '\xee\x95\xa5',
'CjkUnifiedIdeograph5df3': '\xe5\xb7\xb3',
'CjkUnifiedIdeograph5df2': '\xe5\xb7\xb2',
'CjkUnifiedIdeograph5df1': '\xe5\xb7\xb1',
'CjkUnifiedIdeograph5df0': '\xe5\xb7\xb0',
'CjkUnifiedIdeograph5df7': '\xe5\xb7\xb7',
'CjkUnifiedIdeograph5df6': '\xe5\xb7\xb6',
'CjkUnifiedIdeograph5df5': '\xe5\xb7\xb5',
'CjkUnifiedIdeograph5df4': '\xe5\xb7\xb4',
'HangulSyllableHwaek': '\xed\x9a\x88',
'CjkUnifiedIdeograph7df6': '\xe7\xb7\xb6',
'CombiningLatinLetterSmallCapitalL': '\xe1\xb7\x9e',
'CombiningLatinLetterSmallCapitalM': '\xe1\xb7\x9f',
'CombiningLatinLetterSmallCapitalN': '\xe1\xb7\xa1',
'ThaiCharacterPhoPhan': '\xe0\xb8\x9e',
'CjkUnifiedIdeograph7df1': '\xe7\xb7\xb1',
'BengaliVowelSignVocalicR': '\xe0\xa7\x83',
'SinhalaLetterAiyanna': '\xe0\xb6\x93',
'UnicodeCharacter0x1a6d': '\xe1\xa9\xad',
'UnicodeCharacter0x1a6e': '\xe1\xa9\xae',
'UnicodeCharacter0x1a6f': '\xe1\xa9\xaf',
'BengaliVowelSignVocalicL': '\xe0\xa7\xa2',
'UnicodeCharacter0x1a6a': '\xe1\xa9\xaa',
'UnicodeCharacter0x1a6b': '\xe1\xa9\xab',
'UnicodeCharacter0x1a6c': '\xe1\xa9\xac',
'CombiningLatinLetterSmallCapitalR': '\xe1\xb7\xa2',
'CjkUnifiedIdeograph7df2': '\xe7\xb7\xb2',
'YiSyllableLiex': '\xea\x86\xbc',
'UnicodeCharacter0xf7fa': '\xef\x9f\xba',
'CjkUnifiedIdeograph5dfc': '\xe5\xb7\xbc',
'CjkUnifiedIdeograph5dfb': '\xe5\xb7\xbb',
'CjkUnifiedIdeograph5dfa': '\xe5\xb7\xba',
'CjkUnifiedIdeograph5dff': '\xe5\xb7\xbf',
'CjkUnifiedIdeograph5dfe': '\xe5\xb7\xbe',
'CjkUnifiedIdeograph5dfd': '\xe5\xb7\xbd',
'UnicodeCharacter0xf3a8': '\xef\x8e\xa8',
'UnicodeCharacter0xdd05': '\xed\xb4\x85',
'HangulSyllableDdaegg': '\xeb\x95\x8e',
'CjkUnifiedIdeograph63e4': '\xe6\x8f\xa4',
'CjkUnifiedIdeograph63e5': '\xe6\x8f\xa5',
'CjkUnifiedIdeograph63e6': '\xe6\x8f\xa6',
'CjkUnifiedIdeograph63e7': '\xe6\x8f\xa7',
'CjkUnifiedIdeograph63e0': '\xe6\x8f\xa0',
'CjkUnifiedIdeograph63e1': '\xe6\x8f\xa1',
'BraillePatternDots248': '\xe2\xa2\x8a',
'CjkUnifiedIdeograph63e3': '\xe6\x8f\xa3',
'BraillePatternDots246': '\xe2\xa0\xaa',
'BraillePatternDots247': '\xe2\xa1\x8a',
'BraillePatternDots245': '\xe2\xa0\x9a',
'LatinCapitalLetterOWithTildeAndAcute': '\xe1\xb9\x8c',
'CjkUnifiedIdeograph63e9': '\xe6\x8f\xa9',
'UnicodeCharacter0xe3ee': '\xee\x8f\xae',
'UnicodeCharacter0xd7e3': '\xed\x9f\xa3',
'CjkRadicalSimplifiedHalfTreeTrunk': '\xe2\xba\xa6',
'UnicodeCharacter0xf050': '\xef\x81\x90',
'UnicodeCharacter0xd7e5': '\xed\x9f\xa5',
'CjkUnifiedIdeograph7a39': '\xe7\xa8\xb9',
'MongolianLetterTodoYa': '\xe1\xa1\x95',
'HangulLetterNieun': '\xe3\x84\xb4',
'CjkUnifiedIdeograph6d28': '\xe6\xb4\xa8',
'CjkUnifiedIdeograph6d29': '\xe6\xb4\xa9',
'CjkUnifiedIdeograph6d26': '\xe6\xb4\xa6',
'CjkUnifiedIdeograph6d27': '\xe6\xb4\xa7',
'CjkUnifiedIdeograph6d24': '\xe6\xb4\xa4',
'CjkUnifiedIdeograph6d25': '\xe6\xb4\xa5',
'CjkUnifiedIdeograph6d22': '\xe6\xb4\xa2',
'CjkUnifiedIdeograph6d23': '\xe6\xb4\xa3',
'CjkUnifiedIdeograph6d20': '\xe6\xb4\xa0',
'CjkUnifiedIdeograph6d21': '\xe6\xb4\xa1',
'HexagramForFollowing': '\xe4\xb7\x90',
'rightarrow': '\xe2\x86\x92',
'HalfwidthHangulLetterKiyeok': '\xef\xbe\xa1',
'UnicodeCharacter0xf63c': '\xef\x98\xbc',
'HangulSyllableDagg': '\xeb\x8b\xa6',
'RejangLetterGa': '\xea\xa4\xb1',
'BopomofoLetterOnn': '\xe3\x86\xa7',
'HangulSyllableGweoss': '\xea\xb6\x9c',
'BalineseLetterGa': '\xe1\xac\x95',
'CjkUnifiedIdeograph6b6c': '\xe6\xad\xac',
'BopomofoLetterOng': '\xe3\x86\xb2',
'CyrillicCapitalLigatureEnGhe': '\xd2\xa4',
'CjkUnifiedIdeograph63ed': '\xe6\x8f\xad',
'CjkUnifiedIdeograph63ee': '\xe6\x8f\xae',
'CjkUnifiedIdeograph63ef': '\xe6\x8f\xaf',
'CjkUnifiedIdeograph63ea': '\xe6\x8f\xaa',
'CjkUnifiedIdeograph63eb': '\xe6\x8f\xab',
'SinhalaLetterSanyakaJayanna': '\xe0\xb6\xa6',
'HangulSyllableTo': '\xed\x86\xa0',
'UnicodeCharacter0xf63f': '\xef\x98\xbf',
'UnicodeCharacter0xa9f7': '\xea\xa7\xb7',
'BraillePatternDots1234578': '\xe2\xa3\x9f',
'ArabicLetterAlefWithWavyHamzaAbove': '\xd9\xb2',
'CjkUnifiedIdeograph6dd7': '\xe6\xb7\x97',
'CjkUnifiedIdeograph6d2f': '\xe6\xb4\xaf',
'CjkUnifiedIdeograph7a3b': '\xe7\xa8\xbb',
'CjkUnifiedIdeograph6d2d': '\xe6\xb4\xad',
'CjkUnifiedIdeograph6d2e': '\xe6\xb4\xae',
'CjkUnifiedIdeograph6d2b': '\xe6\xb4\xab',
'LatinCapitalLetterHWithCircumflex': '\xc4\xa4',
'CjkUnifiedIdeograph7a3e': '\xe7\xa8\xbe',
'CjkUnifiedIdeograph6d2a': '\xe6\xb4\xaa',
'UnicodeCharacter0x2bb2': '\xe2\xae\xb2',
'UnicodeCharacter0x2bb3': '\xe2\xae\xb3',
'UnicodeCharacter0x2bb0': '\xe2\xae\xb0',
'UnicodeCharacter0x2bb1': '\xe2\xae\xb1',
'UnicodeCharacter0x2bb6': '\xe2\xae\xb6',
'UnicodeCharacter0x2bb7': '\xe2\xae\xb7',
'UnicodeCharacter0x2bb4': '\xe2\xae\xb4',
'UnicodeCharacter0x2bb5': '\xe2\xae\xb5',
'CjkUnifiedIdeograph5e66': '\xe5\xb9\xa6',
'CjkUnifiedIdeograph5e67': '\xe5\xb9\xa7',
'UnicodeCharacter0x2bb8': '\xe2\xae\xb8',
'UnicodeCharacter0x2bb9': '\xe2\xae\xb9',
'CjkUnifiedIdeograph5e62': '\xe5\xb9\xa2',
'CjkUnifiedIdeograph5e63': '\xe5\xb9\xa3',
'CjkUnifiedIdeograph5e60': '\xe5\xb9\xa0',
'CjkUnifiedIdeograph5e61': '\xe5\xb9\xa1',
'HangulSyllableJoelg': '\xec\xa3\x8d',
'ParenthesizedHangulPieup': '\xe3\x88\x85',
'xi': '\xce\xbe',
'DevanagariLetterShortO': '\xe0\xa4\x92',
'UnicodeCharacter0xf18f': '\xef\x86\x8f',
'DevanagariLetterShortA': '\xe0\xa4\x84',
'DevanagariLetterShortE': '\xe0\xa4\x8e',
'ArabicEmptyCentreLowStop': '\xdb\xaa',
'UnicodeCharacter0xe9fa': '\xee\xa7\xba',
'UnicodeCharacter0xe9fb': '\xee\xa7\xbb',
'UnicodeCharacter0xe9fc': '\xee\xa7\xbc',
'UnicodeCharacter0xe9fd': '\xee\xa7\xbd',
'UnicodeCharacter0xe9fe': '\xee\xa7\xbe',
'LatinCapitalLetterVWithHook': '\xc6\xb2',
'CjkUnifiedIdeograph5dc2': '\xe5\xb7\x82',
'CjkUnifiedIdeograph95d9': '\xe9\x97\x99',
'CjkUnifiedIdeograph72b7': '\xe7\x8a\xb7',
'CjkUnifiedIdeograph95d0': '\xe9\x97\x90',
'CjkUnifiedIdeograph95d1': '\xe9\x97\x91',
'CjkUnifiedIdeograph95d2': '\xe9\x97\x92',
'CjkUnifiedIdeograph95d3': '\xe9\x97\x93',
'CjkUnifiedIdeograph95d4': '\xe9\x97\x94',
'CjkUnifiedIdeograph95d5': '\xe9\x97\x95',
'CjkUnifiedIdeograph95d6': '\xe9\x97\x96',
'CjkUnifiedIdeograph95d7': '\xe9\x97\x97',
'HangulSyllableHyeolb': '\xed\x98\x8b',
'UnicodeCharacter0x2bbb': '\xe2\xae\xbb',
'UnicodeCharacter0x2bbc': '\xe2\xae\xbc',
'CjkUnifiedIdeograph9e69': '\xe9\xb9\xa9',
'UnicodeCharacter0x2bba': '\xe2\xae\xba',
'UnicodeCharacter0x2bbf': '\xe2\xae\xbf',
'UnicodeCharacter0x2bbd': '\xe2\xae\xbd',
'UnicodeCharacter0x2bbe': '\xe2\xae\xbe',
'CyrillicSmallLetterRoundOmega': '\xd1\xbb',
'ThreeDimensionalAngle': '\xe2\x9f\x80',
'CjkUnifiedIdeograph5e6d': '\xe5\xb9\xad',
'CjkUnifiedIdeograph5e6e': '\xe5\xb9\xae',
'CjkUnifiedIdeograph5e6b': '\xe5\xb9\xab',
'CjkUnifiedIdeograph5e6c': '\xe5\xb9\xac',
'CjkUnifiedIdeograph5e6a': '\xe5\xb9\xaa',
'HangulSyllableGwit': '\xea\xb7\x99',
'HangulSyllableGwip': '\xea\xb7\x9a',
'HangulSyllableGwis': '\xea\xb7\x93',
'UnicodeCharacter0xebd4': '\xee\xaf\x94',
'CjkUnifiedIdeograph95da': '\xe9\x97\x9a',
'RightwardsArrowOverLeftwardsArrow': '\xe2\x87\x84',
'CjkUnifiedIdeograph95dc': '\xe9\x97\x9c',
'CjkUnifiedIdeograph95dd': '\xe9\x97\x9d',
'CjkUnifiedIdeograph95de': '\xe9\x97\x9e',
'CjkUnifiedIdeograph95df': '\xe9\x97\x9f',
'HangulSyllableGwid': '\xea\xb7\x87',
'UnicodeCharacter0xe9f9': '\xee\xa7\xb9',
'HangulSyllableGwig': '\xea\xb7\x81',
'LefttorightMark': '\xe2\x80\x8e',
'HangulSyllableGwic': '\xea\xb7\x97',
'HangulSyllableGwil': '\xea\xb7\x88',
'HangulSyllableGwim': '\xea\xb7\x90',
'HangulSyllableGwin': '\xea\xb7\x84',
'UnicodeCharacter0xe9f3': '\xee\xa7\xb3',
'HangulSyllableGwih': '\xea\xb7\x9b',
'HangulSyllableHyeolh': '\xed\x98\x8f',
'HangulSyllableGwij': '\xea\xb7\x96',
'HangulSyllableGwik': '\xea\xb7\x98',
'CjkUnifiedIdeograph9a2a': '\xe9\xa8\xaa',
'UnicodeCharacter0xa7a5': '\xea\x9e\xa5',
'HangulChoseongSsangieung': '\xe1\x85\x87',
'HangulJongseongRieulpieupsios': '\xe1\x87\x93',
'CjkUnifiedIdeograph5d1f': '\xe5\xb4\x9f',
'HangulSyllableDdoelp': '\xeb\x99\xb2',
'UnicodeCharacter0xdac2': '\xed\xab\x82',
'YiSyllableHxuo': '\xea\x89\x94',
'CjkUnifiedIdeograph5d1d': '\xe5\xb4\x9d',
'CjkUnifiedIdeograph37c5': '\xe3\x9f\x85',
'CjkUnifiedIdeograph37c4': '\xe3\x9f\x84',
'CjkUnifiedIdeograph37c7': '\xe3\x9f\x87',
'CjkUnifiedIdeograph37c6': '\xe3\x9f\x86',
'CjkUnifiedIdeograph37c1': '\xe3\x9f\x81',
'CjkUnifiedIdeograph37c0': '\xe3\x9f\x80',
'CjkUnifiedIdeograph37c3': '\xe3\x9f\x83',
'CjkUnifiedIdeograph37c2': '\xe3\x9f\x82',
'UnicodeCharacter0xe64a': '\xee\x99\x8a',
'CanadianSyllabicsRaai': '\xe1\x95\x85',
'UnicodeCharacter0xe64c': '\xee\x99\x8c',
'ModifierLetterHighToneBar': '\xcb\xa6',
'CjkUnifiedIdeograph37c9': '\xe3\x9f\x89',
'CjkUnifiedIdeograph37c8': '\xe3\x9f\x88',
'CjkUnifiedIdeograph5d1c': '\xe5\xb4\x9c',
'UnicodeCharacter0xe399': '\xee\x8e\x99',
'UnicodeCharacter0xe398': '\xee\x8e\x98',
'UnicodeCharacter0xe397': '\xee\x8e\x97',
'UnicodeCharacter0xe396': '\xee\x8e\x96',
'UnicodeCharacter0xe395': '\xee\x8e\x95',
'CjkUnifiedIdeograph5d1a': '\xe5\xb4\x9a',
'UnicodeCharacter0xe393': '\xee\x8e\x93',
'UnicodeCharacter0xe392': '\xee\x8e\x92',
'UnicodeCharacter0xe391': '\xee\x8e\x91',
'UnicodeCharacter0xe390': '\xee\x8e\x90',
'LimbuLetterHa': '\xe1\xa4\x9c',
'RunicLetterC': '\xe1\x9b\x8d',
'RunicLetterE': '\xe1\x9b\x82',
'RunicLetterD': '\xe1\x9b\x91',
'RunicLetterG': '\xe1\x9a\xb5',
'CjkUnifiedIdeograph5b00': '\xe5\xac\x80',
'RunicLetterO': '\xe1\x9a\xae',
'RunicLetterQ': '\xe1\x9b\xa9',
'SinhalaLetterMuurdhajaSayanna': '\xe0\xb7\x82',
'RunicLetterW': '\xe1\x9a\xa5',
'RunicLetterV': '\xe1\x9a\xa1',
'RunicLetterY': '\xe1\x9a\xa4',
'RunicLetterX': '\xe1\x9b\xaa',
'RunicLetterZ': '\xe1\x9b\x8e',
'CjkUnifiedIdeograph657f': '\xe6\x95\xbf',
'CjkUnifiedIdeograph9760': '\xe9\x9d\xa0',
'HangulSyllableDdyalg': '\xeb\x95\xb1',
'HangulSyllableTwip': '\xed\x8a\x9a',
'HangulSyllableTwis': '\xed\x8a\x93',
'CjkUnifiedIdeograph9761': '\xe9\x9d\xa1',
'HangulSyllableDdyalm': '\xeb\x95\xb2',
'CanadianSyllabicsTaa': '\xe1\x91\x96',
'UnicodeCharacter0xe39e': '\xee\x8e\x9e',
'UnicodeCharacter0xe39d': '\xee\x8e\x9d',
'UnicodeCharacter0xe39c': '\xee\x8e\x9c',
'HangulSyllableDdyalh': '\xeb\x95\xb7',
'UnicodeCharacter0xe39a': '\xee\x8e\x9a',
'HangulSyllableByabs': '\xeb\xb1\x9e',
'CjkUnifiedIdeograph37ce': '\xe3\x9f\x8e',
'CjkUnifiedIdeograph37cd': '\xe3\x9f\x8d',
'HangulSyllableTwig': '\xed\x8a\x81',
'CjkUnifiedIdeograph37cf': '\xe3\x9f\x8f',
'CjkUnifiedIdeograph37ca': '\xe3\x9f\x8a',
'HangulSyllableDdyalp': '\xeb\x95\xb6',
'CjkUnifiedIdeograph37cc': '\xe3\x9f\x8c',
'CjkUnifiedIdeograph37cb': '\xe3\x9f\x8b',
'HangulSyllableTwim': '\xed\x8a\x90',
'HangulSyllableTwil': '\xed\x8a\x88',
'UnicodeCharacter0xe643': '\xee\x99\x83',
'HangulSyllableTwin': '\xed\x8a\x84',
'UnicodeCharacter0xe645': '\xee\x99\x85',
'HangulSyllableTwih': '\xed\x8a\x9b',
'HangulSyllableTwik': '\xed\x8a\x98',
'CjkUnifiedIdeograph56b5': '\xe5\x9a\xb5',
'TibetanSubjoinedLetterMa': '\xe0\xbe\xa8',
'BoxDrawingsDownLightAndLeftHeavy': '\xe2\x94\x91',
'HexagramForGreatPossession': '\xe4\xb7\x8d',
'UnicodeCharacter0xf7df': '\xef\x9f\x9f',
'LatinSmallLetterAWithBreveAndGrave': '\xe1\xba\xb1',
'CjkUnifiedIdeograph7dfe': '\xe7\xb7\xbe',
'GreekCapitalLetterUpsilonWithVaria': '\xe1\xbf\xaa',
'CjkUnifiedIdeograph4fc3': '\xe4\xbf\x83',
'CjkUnifiedIdeograph7dff': '\xe7\xb7\xbf',
'CjkUnifiedIdeograph7dfa': '\xe7\xb7\xba',
'CjkUnifiedIdeograph4fc4': '\xe4\xbf\x84',
'CjkUnifiedIdeograph38c6': '\xe3\xa3\x86',
'CjkUnifiedIdeograph38c7': '\xe3\xa3\x87',
'CjkUnifiedIdeograph38c4': '\xe3\xa3\x84',
'CjkUnifiedIdeograph38c5': '\xe3\xa3\x85',
'CjkUnifiedIdeograph38c2': '\xe3\xa3\x82',
'CjkUnifiedIdeograph38c3': '\xe3\xa3\x83',
'CjkUnifiedIdeograph38c0': '\xe3\xa3\x80',
'CjkUnifiedIdeograph38c1': '\xe3\xa3\x81',
'CjkUnifiedIdeograph5513': '\xe5\x94\x93',
'GurmukhiLetterNa': '\xe0\xa8\xa8',
'CjkUnifiedIdeograph5511': '\xe5\x94\x91',
'CjkUnifiedIdeograph5510': '\xe5\x94\x90',
'CjkUnifiedIdeograph5517': '\xe5\x94\x97',
'CjkUnifiedIdeograph5516': '\xe5\x94\x96',
'CjkUnifiedIdeograph38c8': '\xe3\xa3\x88',
'CjkUnifiedIdeograph38c9': '\xe3\xa3\x89',
'CjkUnifiedIdeograph93d6': '\xe9\x8f\x96',
'CjkUnifiedIdeograph85cc': '\xe8\x97\x8c',
'CjkUnifiedIdeograph38cf': '\xe3\xa3\x8f',
'CjkUnifiedIdeograph38cd': '\xe3\xa3\x8d',
'CjkUnifiedIdeograph38ce': '\xe3\xa3\x8e',
'CjkUnifiedIdeograph38cb': '\xe3\xa3\x8b',
'CjkUnifiedIdeograph38cc': '\xe3\xa3\x8c',
'CjkUnifiedIdeograph38ca': '\xe3\xa3\x8a',
'CjkUnifiedIdeograph551c': '\xe5\x94\x9c',
'CjkUnifiedIdeograph551b': '\xe5\x94\x9b',
'CjkUnifiedIdeograph551a': '\xe5\x94\x9a',
'CjkUnifiedIdeograph551f': '\xe5\x94\x9f',
'CjkUnifiedIdeograph551e': '\xe5\x94\x9e',
'CjkUnifiedIdeograph551d': '\xe5\x94\x9d',
'GreekSmallLetterAlpha': '\xce\xb1',
'HangulLetterWi': '\xe3\x85\x9f',
'ArabicLigatureAkbarIsolatedForm': '\xef\xb7\xb3',
'ArabicLetterTchehehInitialForm': '\xef\xae\x80',
'HangulLetterWe': '\xe3\x85\x9e',
'CjkUnifiedIdeograph93d1': '\xe9\x8f\x91',
'HangulLetterWa': '\xe3\x85\x98',
'GreekAnoTeleia': '\xce\x87',
'UnicodeCharacter0x1ae9': '\xe1\xab\xa9',
'KayahLiSignCwi': '\xea\xa4\xae',
'RejangVowelSignEu': '\xea\xa5\x8d',
'ArabicLigatureTehWithMeemWithKhahInitialForm': '\xef\xb5\x97',
'YiSyllableQo': '\xea\x90\x8a',
'CjkUnifiedIdeograph9c51': '\xe9\xb1\x91',
'CjkUnifiedIdeograph6158': '\xe6\x85\x98',
'CjkUnifiedIdeograph6159': '\xe6\x85\x99',
'HangulSyllableDdyoss': '\xeb\x9a\x94',
'CjkUnifiedIdeograph6152': '\xe6\x85\x92',
'CjkUnifiedIdeograph6153': '\xe6\x85\x93',
'CjkUnifiedIdeograph6150': '\xe6\x85\x90',
'CjkUnifiedIdeograph6151': '\xe6\x85\x91',
'CjkUnifiedIdeograph6156': '\xe6\x85\x96',
'CjkUnifiedIdeograph6157': '\xe6\x85\x97',
'CjkUnifiedIdeograph6154': '\xe6\x85\x94',
'CjkUnifiedIdeograph6155': '\xe6\x85\x95',
'CyrillicCapitalLetterUk': '\xd1\xb8',
'BowtieWithLeftHalfBlack': '\xe2\xa7\x91',
'CjkUnifiedIdeograph474a': '\xe4\x9d\x8a',
'CjkUnifiedIdeograph5d16': '\xe5\xb4\x96',
'CjkUnifiedIdeograph474c': '\xe4\x9d\x8c',
'CjkUnifiedIdeograph474b': '\xe4\x9d\x8b',
'CjkUnifiedIdeograph474e': '\xe4\x9d\x8e',
'CjkUnifiedIdeograph474d': '\xe4\x9d\x8d',
'CjkUnifiedIdeograph474f': '\xe4\x9d\x8f',
'ArabicLigatureHahWithMeemIsolatedForm': '\xef\xb0\x98',
'CediSign': '\xe2\x82\xb5',
'CjkUnifiedIdeograph5d14': '\xe5\xb4\x94',
'CjkUnifiedIdeograph5d15': '\xe5\xb4\x95',
'CjkUnifiedIdeograph615b': '\xe6\x85\x9b',
'CjkUnifiedIdeograph5d12': '\xe5\xb4\x92',
'CjkUnifiedIdeograph615a': '\xe6\x85\x9a',
'CjkUnifiedIdeograph615f': '\xe6\x85\x9f',
'CjkUnifiedIdeograph615d': '\xe6\x85\x9d',
'CjkUnifiedIdeograph5d13': '\xe5\xb4\x93',
'CjkUnifiedIdeograph9f36': '\xe9\xbc\xb6',
'SyriacLetterSogdianKhaph': '\xdd\x8e',
'HangulSyllableDdyaelb': '\xeb\x96\x8f',
'CjkUnifiedIdeograph5d11': '\xe5\xb4\x91',
'BottomRightCrop': '\xe2\x8c\x8c',
'CjkUnifiedIdeograph4740': '\xe4\x9d\x80',
'CjkUnifiedIdeograph4743': '\xe4\x9d\x83',
'CjkUnifiedIdeograph4742': '\xe4\x9d\x82',
'CjkUnifiedIdeograph4745': '\xe4\x9d\x85',
'CjkUnifiedIdeograph4744': '\xe4\x9d\x84',
'CjkUnifiedIdeograph4747': '\xe4\x9d\x87',
'CjkUnifiedIdeograph4746': '\xe4\x9d\x86',
'CjkUnifiedIdeograph4749': '\xe4\x9d\x89',
'CjkUnifiedIdeograph4748': '\xe4\x9d\x88',
'HangulSyllableHwe': '\xed\x9b\xbc',
'HangulSyllableJjaet': '\xec\xa8\x91',
'CjkUnifiedIdeograph7209': '\xe7\x88\x89',
'CjkUnifiedIdeograph7208': '\xe7\x88\x88',
'CjkUnifiedIdeograph7201': '\xe7\x88\x81',
'EthiopicSyllableGyaa': '\xe2\xb7\x9b',
'CjkUnifiedIdeograph7203': '\xe7\x88\x83',
'HanunooLetterYa': '\xe1\x9c\xac',
'MinusSignWithFallingDots': '\xe2\xa8\xab',
'CjkUnifiedIdeograph7204': '\xe7\x88\x84',
'CjkUnifiedIdeograph7207': '\xe7\x88\x87',
'CjkUnifiedIdeograph7206': '\xe7\x88\x86',
'CjkUnifiedIdeograph761b': '\xe7\x98\x9b',
'TibetanMarkBsdusRtags': '\xe0\xbc\xb4',
'UnicodeCharacter0xdd6a': '\xed\xb5\xaa',
'UnicodeCharacter0xdd6f': '\xed\xb5\xaf',
'UnicodeCharacter0xdd6d': '\xed\xb5\xad',
'UnicodeCharacter0xdd6e': '\xed\xb5\xae',
'ThaanaLetterLaamu': '\xde\x8d',
'UnicodeCharacter0xf856': '\xef\xa1\x96',
'BraillePatternDots14568': '\xe2\xa2\xb9',
'VaiSyllableThu': '\xea\x96\xa5',
'BraillePatternDots14567': '\xe2\xa1\xb9',
'CjkUnifiedIdeograph3919': '\xe3\xa4\x99',
'CjkUnifiedIdeograph3918': '\xe3\xa4\x98',
'VaiSyllableTha': '\xea\x95\x9b',
'UnicodeCharacter0xeb49': '\xee\xad\x89',
'UnicodeCharacter0xeb48': '\xee\xad\x88',
'BalineseMusicalSymbolRighthandOpenDag': '\xe1\xad\xb5',
'CjkUnifiedIdeograph3911': '\xe3\xa4\x91',
'CjkUnifiedIdeograph3910': '\xe3\xa4\x90',
'AngstromSign': '\xe2\x84\xab',
'LatinCapitalLetterKWithAcute': '\xe1\xb8\xb0',
'CjkUnifiedIdeograph3915': '\xe3\xa4\x95',
'CjkUnifiedIdeograph3914': '\xe3\xa4\x94',
'PlusminusSign': '\xc2\xb1',
'CjkUnifiedIdeograph3916': '\xe3\xa4\x96',
'UnicodeCharacter0xdd62': '\xed\xb5\xa2',
'HangulSyllableRat': '\xeb\x9e\x95',
'LepchaLetterTtha': '\xe1\xb1\x8e',
'CyrillicSmallLetterIzhitsaWithDoubleGraveAccent': '\xd1\xb7',
'UnicodeCharacter0xdd66': '\xed\xb5\xa6',
'HangulSyllableRap': '\xeb\x9e\x96',
'CyrillicSmallLetterCheWithDescender': '\xd2\xb7',
'HangulChoseongNieunpieup': '\xe1\x84\x96',
'UnicodeCharacter0xdd68': '\xed\xb5\xa8',
'HangulSyllableGweogg': '\xea\xb6\x8a',
'HangulSyllableRae': '\xeb\x9e\x98',
'HangulSyllableRad': '\xeb\x9e\x83',
'HangulSyllableRag': '\xeb\x9d\xbd',
'SupersetOfOrEqualToWithDotAbove': '\xe2\xab\x84',
'HangulSyllableRac': '\xeb\x9e\x93',
'TibetanLetterRa': '\xe0\xbd\xa2',
'CjkUnifiedIdeograph720a': '\xe7\x88\x8a',
'HangulSyllableRal': '\xeb\x9e\x84',
'CjkUnifiedIdeograph720c': '\xe7\x88\x8c',
'CjkUnifiedIdeograph720b': '\xe7\x88\x8b',
'CjkUnifiedIdeograph720e': '\xe7\x88\x8e',
'CjkUnifiedIdeograph720d': '\xe7\x88\x8d',
'HangulSyllableRak': '\xeb\x9e\x94',
'CjkUnifiedIdeograph720f': '\xe7\x88\x8f',
'HangulSyllablePeuk': '\xed\x94\x9c',
'HangulSyllablePeuj': '\xed\x94\x9a',
'CanadianSyllabicsFinalBottomHalfRing': '\xe1\x90\xa1',
'HangulSyllablePeun': '\xed\x94\x88',
'HangulSyllablePeum': '\xed\x94\x94',
'HangulSyllablePeul': '\xed\x94\x8c',
'CjkUnifiedIdeograph391a': '\xe3\xa4\x9a',
'HangulSyllablePeub': '\xed\x94\x95',
'CjkUnifiedIdeograph391c': '\xe3\xa4\x9c',
'CjkUnifiedIdeograph391b': '\xe3\xa4\x9b',
'CombiningLeftArrowBelow': '\xe2\x83\xae',
'CjkUnifiedIdeograph391d': '\xe3\xa4\x9d',
'UnicodeCharacter0xeb4c': '\xee\xad\x8c',
'CjkUnifiedIdeograph391f': '\xe3\xa4\x9f',
'CjkUnifiedIdeograph93df': '\xe9\x8f\x9f',
'SaurashtraVowelSignVocalicR': '\xea\xa2\xba',
'HanunooLetterTa': '\xe1\x9c\xa6',
'HangulSyllablePeus': '\xed\x94\x97',
'UnicodeCharacter0xf853': '\xef\xa1\x93',
'HangulSyllablePeup': '\xed\x94\x9e',
'CjkUnifiedIdeograph93dd': '\xe9\x8f\x9d',
'HangulSyllablePeut': '\xed\x94\x9d',
'UnicodeCharacter0xdb8f': '\xed\xae\x8f',
'HangulSyllableTwigs': '\xed\x8a\x83',
'UnicodeCharacter0xdb8d': '\xed\xae\x8d',
'CjkUnifiedIdeograph93de': '\xe9\x8f\x9e',
'UnicodeCharacter0xdb8b': '\xed\xae\x8b',
'UnicodeCharacter0xdb8c': '\xed\xae\x8c',
'HangulSyllableDdigs': '\xeb\x9d\xa3',
'CjkUnifiedIdeograph93db': '\xe9\x8f\x9b',
'Proportion': '\xe2\x88\xb7',
'HangulSyllableSsweolp': '\xec\x92\x8e',
'HangulSyllableSsweols': '\xec\x92\x8c',
'CjkUnifiedIdeograph93dc': '\xe9\x8f\x9c',
'LatinLetterInvertedGlottalStopWithStroke': '\xc6\xbe',
'CyrillicSmallLetterShha': '\xd2\xbb',
'HangulSyllableGyeop': '\xea\xb3\x82',
'CjkUnifiedIdeograph4f63': '\xe4\xbd\xa3',
'HangulJongseongHieuhmieum': '\xe1\x87\xb7',
'UnicodeCharacter0xd7cc': '\xed\x9f\x8c',
'HangulSyllableSsweolg': '\xec\x92\x89',
'CjkUnifiedIdeograph93da': '\xe9\x8f\x9a',
'UnicodeCharacter0xd7cf': '\xed\x9f\x8f',
'HangulSyllableSyeng': '\xec\x86\x85',
'UnicodeCharacter0xd7cd': '\xed\x9f\x8d',
'HangulSyllableSsweolb': '\xec\x92\x8b',
'UnicodeCharacter0xf6d0': '\xef\x9b\x90',
'HangulSyllableTwegs': '\xed\x89\xa7',
'UnicodeCharacter0xf6d2': '\xef\x9b\x92',
'HangulSyllableJjelt': '\xec\xa9\xb5',
'ArabicLetterPeh': '\xd9\xbe',
'UnicodeCharacter0xf6d5': '\xef\x9b\x95',
'UnicodeCharacter0xf6d6': '\xef\x9b\x96',
'UnicodeCharacter0xf6d7': '\xef\x9b\x97',
'UnicodeCharacter0xf6d8': '\xef\x9b\x98',
'Thermodynamic': '\xe2\xa7\xa7',
'CjkUnifiedIdeograph663e': '\xe6\x98\xbe',
'CjkUnifiedIdeograph663d': '\xe6\x98\xbd',
'CjkUnifiedIdeograph663c': '\xe6\x98\xbc',
'EthiopicColon': '\xe1\x8d\xa5',
'CjkUnifiedIdeograph663a': '\xe6\x98\xba',
'GreekSmallReversedDottedLunateSigmaSymbol': '\xcd\xbd',
'HangulSyllablePoes': '\xed\x91\x93',
'CjkUnifiedIdeograph60ce': '\xe6\x83\x8e',
'CjkUnifiedIdeograph60cd': '\xe6\x83\x8d',
'CjkUnifiedIdeograph60cf': '\xe6\x83\x8f',
'CjkUnifiedIdeograph60ca': '\xe6\x83\x8a',
'CjkUnifiedIdeograph60cc': '\xe6\x83\x8c',
'CjkUnifiedIdeograph60cb': '\xe6\x83\x8b',
'UnicodeCharacter0xd7c8': '\xed\x9f\x88',
'UnicodeCharacter0xd7c9': '\xed\x9f\x89',
'YiSyllableJjit': '\xea\x90\x98',
'UnicodeCharacter0xd80b': '\xed\xa0\x8b',
'UnicodeCharacter0xd7c2': '\xed\x9f\x82',
'UnicodeCharacter0xd7c3': '\xed\x9f\x83',
'UnicodeCharacter0xd7c0': '\xed\x9f\x80',
'UnicodeCharacter0xd7c1': '\xed\x9f\x81',
'UnicodeCharacter0xd7c6': '\xed\x9f\x86',
'UnicodeCharacter0xd7c7': '\xed\x9f\x87',
'UnicodeCharacter0xd7c4': '\xed\x9f\x84',
'UnicodeCharacter0xd7c5': '\xed\x9f\x85',
'UnicodeCharacter0xdb86': '\xed\xae\x86',
'UnicodeCharacter0xdb87': '\xed\xae\x87',
'UnicodeCharacter0xdb84': '\xed\xae\x84',
'UnicodeCharacter0xdb85': '\xed\xae\x85',
'UnicodeCharacter0xdb82': '\xed\xae\x82',
'UnicodeCharacter0xdb83': '\xed\xae\x83',
'LatinCapitalLetterAWithGrave': '\xc3\x80',
'UnicodeCharacter0xdb81': '\xed\xae\x81',
'ModifierLetterCapitalOu': '\xe1\xb4\xbd',
'UnicodeCharacter0xdb88': '\xed\xae\x88',
'UnicodeCharacter0xdb89': '\xed\xae\x89',
'DashedOverline': '\xef\xb9\x89',
'CjkUnifiedIdeograph60c9': '\xe6\x83\x89',
'IdeographicAnnotationFirstMark': '\xe3\x86\x99',
'CjkUnifiedIdeograph60c5': '\xe6\x83\x85',
'CjkUnifiedIdeograph60c4': '\xe6\x83\x84',
'CjkUnifiedIdeograph60c7': '\xe6\x83\x87',
'CjkUnifiedIdeograph60c6': '\xe6\x83\x86',
'CjkUnifiedIdeograph60c1': '\xe6\x83\x81',
'CjkUnifiedIdeograph60c0': '\xe6\x83\x80',
'CjkUnifiedIdeograph60c3': '\xe6\x83\x83',
'CjkUnifiedIdeograph60c2': '\xe6\x83\x82',
'GreekSmallLetterAlphaWithDasiaAndOxia': '\xe1\xbc\x85',
'UnicodeCharacter0xf6da': '\xef\x9b\x9a',
'UnicodeCharacter0xf6db': '\xef\x9b\x9b',
'UnicodeCharacter0xf6dc': '\xef\x9b\x9c',
'UnicodeCharacter0xf6dd': '\xef\x9b\x9d',
'UnicodeCharacter0xf6de': '\xef\x9b\x9e',
'CjkUnifiedIdeograph6639': '\xe6\x98\xb9',
'CjkUnifiedIdeograph6638': '\xe6\x98\xb8',
'CjkUnifiedIdeograph6637': '\xe6\x98\xb7',
'CjkUnifiedIdeograph6636': '\xe6\x98\xb6',
'CjkUnifiedIdeograph6635': '\xe6\x98\xb5',
'CjkUnifiedIdeograph6634': '\xe6\x98\xb4',
'CjkUnifiedIdeograph6633': '\xe6\x98\xb3',
'CjkUnifiedIdeograph6632': '\xe6\x98\xb2',
'CjkUnifiedIdeograph6631': '\xe6\x98\xb1',
'CjkUnifiedIdeograph6630': '\xe6\x98\xb0',
'CjkUnifiedIdeograph3b1b': '\xe3\xac\x9b',
'CjkUnifiedIdeograph3b1c': '\xe3\xac\x9c',
'CjkUnifiedIdeograph3b1a': '\xe3\xac\x9a',
'CjkUnifiedIdeograph3b1f': '\xe3\xac\x9f',
'CjkUnifiedIdeograph3b1d': '\xe3\xac\x9d',
'CjkUnifiedIdeograph3b1e': '\xe3\xac\x9e',
'UnicodeCharacter0xf0b9': '\xef\x82\xb9',
'HangulSyllableBbweoc': '\xeb\xbf\xbf',
'CombiningAnnuitySymbol': '\xe2\x83\xa7',
'KannadaLetterCa': '\xe0\xb2\x9a',
'UnicodeCharacter0xf879': '\xef\xa1\xb9',
'HangulSyllableDdoeng': '\xeb\x99\xb9',
'CjkUnifiedIdeograph6989': '\xe6\xa6\x89',
'CjkUnifiedIdeograph6988': '\xe6\xa6\x88',
'EthiopicSyllableNee': '\xe1\x8a\x94',
'HangulSyllableDong': '\xeb\x8f\x99',
'HangulSyllableDdoenh': '\xeb\x99\xaa',
'CjkUnifiedIdeograph6981': '\xe6\xa6\x81',
'CjkUnifiedIdeograph6980': '\xe6\xa6\x80',
'CjkUnifiedIdeograph6983': '\xe6\xa6\x83',
'CjkUnifiedIdeograph6982': '\xe6\xa6\x82',
'CjkUnifiedIdeograph6985': '\xe6\xa6\x85',
'CjkUnifiedIdeograph6984': '\xe6\xa6\x84',
'MongolianLetterTodoJia': '\xe1\xa1\x9a',
'CjkUnifiedIdeograph6986': '\xe6\xa6\x86',
'CjkUnifiedIdeograph90b4': '\xe9\x82\xb4',
'ArabicLigatureQafWithYehIsolatedForm': '\xef\xb0\xb6',
'CjkUnifiedIdeograph45cb': '\xe4\x97\x8b',
'UnicodeCharacter0xf7a0': '\xef\x9e\xa0',
'CjkUnifiedIdeograph90b2': '\xe9\x82\xb2',
'CjkUnifiedIdeograph3b12': '\xe3\xac\x92',
'KhmerLetterBa': '\xe1\x9e\x94',
'CjkUnifiedIdeograph3b10': '\xe3\xac\x90',
'CjkUnifiedIdeograph3b11': '\xe3\xac\x91',
'CjkUnifiedIdeograph3b16': '\xe3\xac\x96',
'CjkUnifiedIdeograph3b17': '\xe3\xac\x97',
'CjkUnifiedIdeograph3b14': '\xe3\xac\x94',
'CjkUnifiedIdeograph3b15': '\xe3\xac\x95',
'CjkUnifiedIdeograph3b18': '\xe3\xac\x98',
'CjkUnifiedIdeograph3b19': '\xe3\xac\x99',
'TaiLeLetterTsha': '\xe1\xa5\xa1',
'YiRadicalShur': '\xea\x92\xba',
'UnicodeCharacter0xe2a9': '\xee\x8a\xa9',
'HangulSyllableDdeunj': '\xeb\x9c\xad',
'CjkUnifiedIdeograph7588': '\xe7\x96\x88',
'CjkUnifiedIdeograph7589': '\xe7\x96\x89',
'CjkUnifiedIdeograph698a': '\xe6\xa6\x8a',
'CjkUnifiedIdeograph7587': '\xe7\x96\x87',
'CjkUnifiedIdeograph698c': '\xe6\xa6\x8c',
'CjkUnifiedIdeograph698b': '\xe6\xa6\x8b',
'CjkUnifiedIdeograph698e': '\xe6\xa6\x8e',
'CjkUnifiedIdeograph698d': '\xe6\xa6\x8d',
'CjkUnifiedIdeograph7580': '\xe7\x96\x80',
'CjkUnifiedIdeograph698f': '\xe6\xa6\x8f',
'UnicodeCharacter0xea25': '\xee\xa8\xa5',
'TibetanSymbolDrilBu': '\xe0\xbf\x84',
'UnicodeCharacter0xf0e1': '\xef\x83\xa1',
'CjkUnifiedIdeograph9dab': '\xe9\xb6\xab',
'CjkUnifiedIdeograph9dac': '\xe9\xb6\xac',
'CjkUnifiedIdeograph9daa': '\xe9\xb6\xaa',
'CjkUnifiedIdeograph9daf': '\xe9\xb6\xaf',
'CjkUnifiedIdeograph9dad': '\xe9\xb6\xad',
'CjkUnifiedIdeograph9dae': '\xe9\xb6\xae',
'HangulSyllableMelt': '\xeb\xa9\xa1',
'ArabicLetterGuehInitialForm': '\xef\xae\x98',
'HangulChoseongChieuchkhieukh': '\xe1\x85\x92',
'YiSyllableNguo': '\xea\x89\xa6',
'CircledDigitTwo': '\xe2\x91\xa1',
'UnicodeCharacter0x245e': '\xe2\x91\x9e',
'UnicodeCharacter0x245d': '\xe2\x91\x9d',
'UnicodeCharacter0x245f': '\xe2\x91\x9f',
'UnicodeCharacter0x245a': '\xe2\x91\x9a',
'UnicodeCharacter0x245c': '\xe2\x91\x9c',
'UnicodeCharacter0x245b': '\xe2\x91\x9b',
'UnicodeCharacter0x2fdd': '\xe2\xbf\x9d',
'UnicodeCharacter0x2fde': '\xe2\xbf\x9e',
'UnicodeCharacter0x2fdf': '\xe2\xbf\x9f',
'UnicodeCharacter0x2fda': '\xe2\xbf\x9a',
'UnicodeCharacter0x2fdb': '\xe2\xbf\x9b',
'UnicodeCharacter0x2fdc': '\xe2\xbf\x9c',
'HangulSyllableJjaec': '\xec\xa8\x8f',
'FullwidthRightSquareBracket': '\xef\xbc\xbd',
'CjkUnifiedIdeograph6f2d': '\xe6\xbc\xad',
'AplFunctionalSymbolQuadRightwardsArrow': '\xe2\x8d\x88',
'CjkUnifiedIdeograph6f2f': '\xe6\xbc\xaf',
'UnicodeCharacter0xeaa6': '\xee\xaa\xa6',
'CjkUnifiedIdeograph6f2a': '\xe6\xbc\xaa',
'CjkUnifiedIdeograph6f2b': '\xe6\xbc\xab',
'CjkUnifiedIdeograph6f2c': '\xe6\xbc\xac',
'UnicodeCharacter0xec9f': '\xee\xb2\x9f',
'HangulJungseongWa': '\xe1\x85\xaa',
'HangulSyllableWaelt': '\xec\x99\xa9',
'UnicodeCharacter0xea47': '\xee\xa9\x87',
'CjkUnifiedIdeograph9f6d': '\xe9\xbd\xad',
'SymbolForNegativeAcknowledge': '\xe2\x90\x95',
'ReverseSolidusPrecedingSubset': '\xe2\x9f\x88',
'CyrillicSmallLigatureAIe': '\xd3\x95',
'CjkUnifiedIdeograph9da9': '\xe9\xb6\xa9',
'CjkUnifiedIdeograph9da2': '\xe9\xb6\xa2',
'UnicodeCharacter0x590': '\xd6\x90',
'CjkUnifiedIdeograph9da0': '\xe9\xb6\xa0',
'TeluguVowelSignVocalicLl': '\xe0\xb1\xa3',
'CjkUnifiedIdeograph9da6': '\xe9\xb6\xa6',
'CjkUnifiedIdeograph9da7': '\xe9\xb6\xa7',
'CjkUnifiedIdeograph9da4': '\xe9\xb6\xa4',
'CjkUnifiedIdeograph9da5': '\xe9\xb6\xa5',
'CjkUnifiedIdeograph6f28': '\xe6\xbc\xa8',
'CjkUnifiedIdeograph6f29': '\xe6\xbc\xa9',
'CjkUnifiedIdeograph6f24': '\xe6\xbc\xa4',
'CjkUnifiedIdeograph6f25': '\xe6\xbc\xa5',
'CjkUnifiedIdeograph6f26': '\xe6\xbc\xa6',
'CjkUnifiedIdeograph6f27': '\xe6\xbc\xa7',
'CjkUnifiedIdeograph6f20': '\xe6\xbc\xa0',
'CjkUnifiedIdeograph6f21': '\xe6\xbc\xa1',
'CjkUnifiedIdeograph6f22': '\xe6\xbc\xa2',
'CjkUnifiedIdeograph6f23': '\xe6\xbc\xa3',
'UnicodeCharacter0x2455': '\xe2\x91\x95',
'UnicodeCharacter0x2454': '\xe2\x91\x94',
'UnicodeCharacter0x2457': '\xe2\x91\x97',
'UnicodeCharacter0x2456': '\xe2\x91\x96',
'UnicodeCharacter0x2451': '\xe2\x91\x91',
'UnicodeCharacter0x2450': '\xe2\x91\x90',
'UnicodeCharacter0x2453': '\xe2\x91\x93',
'UnicodeCharacter0x2452': '\xe2\x91\x92',
'HangulSyllableByid': '\xeb\xb8\xaf',
'UnicodeCharacter0x2fd6': '\xe2\xbf\x96',
'UnicodeCharacter0x2fd7': '\xe2\xbf\x97',
'UnicodeCharacter0x2459': '\xe2\x91\x99',
'UnicodeCharacter0x2458': '\xe2\x91\x98',
'HangulSyllableByic': '\xeb\xb8\xbf',
'HangulSyllableByib': '\xeb\xb8\xb9',
'RightSquareBracketUpperCorner': '\xe2\x8e\xa4',
'UnicodeCharacter0xf55a': '\xef\x95\x9a',
'BraillePatternDots123467': '\xe2\xa1\xaf',
'CjkUnifiedIdeograph98a9': '\xe9\xa2\xa9',
'CjkUnifiedIdeograph98a6': '\xe9\xa2\xa6',
'BraillePatternDots123468': '\xe2\xa2\xaf',
'LatinCapitalLetterGamma': '\xc6\x94',
'CjkUnifiedIdeograph98a5': '\xe9\xa2\xa5',
'CjkUnifiedIdeograph98a2': '\xe9\xa2\xa2',
'CjkUnifiedIdeograph98a3': '\xe9\xa2\xa3',
'CjkUnifiedIdeograph98a0': '\xe9\xa2\xa0',
'CjkUnifiedIdeograph98a1': '\xe9\xa2\xa1',
'HangulSyllableWaelg': '\xec\x99\xa5',
'HangulSyllableDwan': '\xeb\x8f\xa4',
'CjkCompatibilityIdeographf9db': '\xef\xa7\x9b',
'UnicodeCharacter0xe8d8': '\xee\xa3\x98',
'CjkUnifiedIdeograph4f1f': '\xe4\xbc\x9f',
'CjkUnifiedIdeograph4f1e': '\xe4\xbc\x9e',
'CjkUnifiedIdeograph4f1d': '\xe4\xbc\x9d',
'CjkUnifiedIdeograph4f1c': '\xe4\xbc\x9c',
'CjkUnifiedIdeograph4f1b': '\xe4\xbc\x9b',
'CjkUnifiedIdeograph4f1a': '\xe4\xbc\x9a',
'HangulSyllableDwah': '\xeb\x8f\xbb',
'HangulSyllableDwak': '\xeb\x8f\xb8',
'CjkUnifiedIdeograph745e': '\xe7\x91\x9e',
'HangulSyllableJjaeg': '\xec\xa7\xb9',
'CanadianSyllabicsWestcreeLe': '\xe1\x95\x84',
'HangulSyllableWaelb': '\xec\x99\xa7',
'CjkUnifiedIdeograph7d43': '\xe7\xb5\x83',
'CjkUnifiedIdeograph7d42': '\xe7\xb5\x82',
'CjkUnifiedIdeograph7d41': '\xe7\xb5\x81',
'CjkUnifiedIdeograph7d40': '\xe7\xb5\x80',
'CjkUnifiedIdeograph7d47': '\xe7\xb5\x87',
'CjkUnifiedIdeograph7d46': '\xe7\xb5\x86',
'NewTaiLueVowelSignOay': '\xe1\xa6\xbe',
'CjkUnifiedIdeograph7d44': '\xe7\xb5\x84',
'CanadianSyllabicsNngaa': '\xe1\x99\xb6',
'CjkUnifiedIdeograph7700': '\xe7\x9c\x80',
'CjkUnifiedIdeograph7d49': '\xe7\xb5\x89',
'CjkUnifiedIdeograph7d48': '\xe7\xb5\x88',
'DittoMark': '\xe3\x80\x83',
'EthiopicTonalMarkDerethidet': '\xe1\x8e\x98',
'UnicodeCharacter0xf701': '\xef\x9c\x81',
'CjkUnifiedIdeograph4f19': '\xe4\xbc\x99',
'CjkUnifiedIdeograph4f18': '\xe4\xbc\x98',
'CjkUnifiedIdeograph4f17': '\xe4\xbc\x97',
'CjkUnifiedIdeograph4f16': '\xe4\xbc\x96',
'CjkUnifiedIdeograph4f15': '\xe4\xbc\x95',
'CjkUnifiedIdeograph4f14': '\xe4\xbc\x94',
'CjkUnifiedIdeograph4f13': '\xe4\xbc\x93',
'CjkUnifiedIdeograph4f12': '\xe4\xbc\x92',
'CjkUnifiedIdeograph4f11': '\xe4\xbc\x91',
'KatakanaLetterTu': '\xe3\x83\x84',
'SmallCommercialAt': '\xef\xb9\xab',
'TibetanSubjoinedLetterFixedformYa': '\xe0\xbe\xbb',
'KatakanaLetterTo': '\xe3\x83\x88',
'RightSquareBracketWithQuill': '\xe2\x81\x86',
'CjkUnifiedIdeograph98af': '\xe9\xa2\xaf',
'YiSyllableMat': '\xea\x82\xb5',
'CjkUnifiedIdeograph98ad': '\xe9\xa2\xad',
'KatakanaLetterTa': '\xe3\x82\xbf',
'CjkUnifiedIdeograph98ab': '\xe9\xa2\xab',
'CjkUnifiedIdeograph98ac': '\xe9\xa2\xac',
'KatakanaLetterTe': '\xe3\x83\x86',
'CjkUnifiedIdeograph7d4c': '\xe7\xb5\x8c',
'CjkUnifiedIdeograph7d4b': '\xe7\xb5\x8b',
'CjkUnifiedIdeograph7d4a': '\xe7\xb5\x8a',
'HangulSyllableJwaep': '\xec\xa3\x82',
'UnicodeCharacter0xe5df': '\xee\x97\x9f',
'CjkUnifiedIdeograph7d4f': '\xe7\xb5\x8f',
'CjkUnifiedIdeograph7d4e': '\xe7\xb5\x8e',
'UnicodeCharacter0x620': '\xd8\xa0',
'UnicodeCharacter0xda2c': '\xed\xa8\xac',
'HangulSyllableRyeols': '\xeb\xa0\xb0',
'ArabicLetterBeehInitialForm': '\xef\xad\x94',
'UnicodeCharacter0xda2b': '\xed\xa8\xab',
'HangulSyllableJwaec': '\xec\xa2\xbf',
'HangulSyllableJwaeb': '\xec\xa2\xb9',
'HangulSyllableJwaeg': '\xec\xa2\xa9',
'HangulSyllableJwaed': '\xec\xa2\xaf',
'HangulSyllableJwaek': '\xec\xa3\x80',
'HangulSyllableJwaej': '\xec\xa2\xbe',
'HangulSyllableJwaeh': '\xec\xa3\x83',
'MyanmarSymbolLocative': '\xe1\x81\x8c',
'HangulSyllableJwaem': '\xec\xa2\xb8',
'HangulSyllableJwael': '\xec\xa2\xb0',
'BugineseLetterKa': '\xe1\xa8\x80',
'UnicodeCharacter0xabdc': '\xea\xaf\x9c',
'UnicodeCharacter0xabdb': '\xea\xaf\x9b',
'UnicodeCharacter0xabde': '\xea\xaf\x9e',
'UnicodeCharacter0xabdd': '\xea\xaf\x9d',
'MalayalamLetterKha': '\xe0\xb4\x96',
'HangulSyllableDwat': '\xeb\x8f\xb9',
'HalfwidthKatakanaLetterRi': '\xef\xbe\x98',
'GeorgianCapitalLetterZen': '\xe1\x82\xa6',
'CjkUnifiedIdeograph6eff': '\xe6\xbb\xbf',
'CjkUnifiedIdeograph6efe': '\xe6\xbb\xbe',
'TeluguVowelSignIi': '\xe0\xb1\x80',
'CjkUnifiedIdeograph6efc': '\xe6\xbb\xbc',
'CjkUnifiedIdeograph6efb': '\xe6\xbb\xbb',
'CjkUnifiedIdeograph6efa': '\xe6\xbb\xba',
'UnicodeCharacter0xeaaa': '\xee\xaa\xaa',
'HangulSyllableRyeolg': '\xeb\xa0\xad',
'UnicodeCharacter0xeaac': '\xee\xaa\xac',
'RomanNumeralFiveHundred': '\xe2\x85\xae',
'HangulSyllableHweon': '\xed\x9b\xa4',
'SquareDmCubed': '\xe3\x8d\xb9',
'UnicodeCharacter0xeaab': '\xee\xaa\xab',
'HangulSyllableJjeup': '\xec\xaf\xae',
'HangulSyllableJjeus': '\xec\xaf\xa7',
'UnicodeCharacter0xeaae': '\xee\xaa\xae',
'HangulSyllableGgyak': '\xea\xba\x9c',
'UnicodeCharacter0xabd1': '\xea\xaf\x91',
'UnicodeCharacter0xabd0': '\xea\xaf\x90',
'UnicodeCharacter0xabd3': '\xea\xaf\x93',
'BengaliLetterPha': '\xe0\xa6\xab',
'UnicodeCharacter0xabd5': '\xea\xaf\x95',
'UnicodeCharacter0xabd4': '\xea\xaf\x94',
'UnicodeCharacter0xabd7': '\xea\xaf\x97',
'UnicodeCharacter0xabd6': '\xea\xaf\x96',
'UnicodeCharacter0xabd9': '\xea\xaf\x99',
'UnicodeCharacter0x2fff': '\xe2\xbf\xbf',
'HangulSyllableJjeun': '\xec\xaf\x98',
'HangulSyllableJjeuh': '\xec\xaf\xaf',
'CjkUnifiedIdeograph3517': '\xe3\x94\x97',
'HangulSyllableJjaem': '\xec\xa8\x88',
'HangulSyllableJjeuk': '\xec\xaf\xac',
'BugineseLetterMa': '\xe1\xa8\x86',
'UnicodeCharacter0xd76': '\xe0\xb5\xb6',
'UnicodeCharacter0xd77': '\xe0\xb5\xb7',
'UnicodeCharacter0xfdfe': '\xef\xb7\xbe',
'HangulSyllableHweog': '\xed\x9b\xa1',
'UnicodeCharacter0xd78': '\xe0\xb5\xb8',
'HangulSyllableMwanj': '\xeb\xab\x89',
'BoxDrawingsHeavyLeft': '\xe2\x95\xb8',
'HangulSyllableSyegg': '\xec\x85\xb2',
'CjkUnifiedIdeograph3510': '\xe3\x94\x90',
'YiSyllableZuox': '\xea\x8a\x98',
'CjkUnifiedIdeograph6ef9': '\xe6\xbb\xb9',
'CjkUnifiedIdeograph6ef8': '\xe6\xbb\xb8',
'CjkUnifiedIdeograph6ef7': '\xe6\xbb\xb7',
'CjkUnifiedIdeograph6ef6': '\xe6\xbb\xb6',
'CjkUnifiedIdeograph6ef5': '\xe6\xbb\xb5',
'CjkUnifiedIdeograph6ef4': '\xe6\xbb\xb4',
'CjkUnifiedIdeograph6ef3': '\xe6\xbb\xb3',
'CjkUnifiedIdeograph6ef2': '\xe6\xbb\xb2',
'CjkUnifiedIdeograph6ef1': '\xe6\xbb\xb1',
'CjkUnifiedIdeograph6ef0': '\xe6\xbb\xb0',
'GeorgianLetterSan': '\xe1\x83\xa1',
'ArabicLetterZahInitialForm': '\xef\xbb\x87',
'SundaneseLetterE': '\xe1\xae\x88',
'LatinSmallLetterUWithDiaeresis': '\xc3\xbc',
'CjkUnifiedIdeograph80e8': '\xe8\x83\xa8',
'SundaneseLetterO': '\xe1\xae\x87',
'SundaneseLetterU': '\xe1\xae\x85',
'CjkUnifiedIdeograph90bf': '\xe9\x82\xbf',
'AplFunctionalSymbolQuadColon': '\xe2\x8d\xa0',
'CjkUnifiedIdeograph90be': '\xe9\x82\xbe',
'SymbolForSynchronousIdle': '\xe2\x90\x96',
'UnicodeCharacter0xb9d': '\xe0\xae\x9d',
'CjkUnifiedIdeograph90bd': '\xe9\x82\xbd',
'UnicodeCharacter0xb9b': '\xe0\xae\x9b',
'CjkUnifiedIdeograph858c': '\xe8\x96\x8c',
'CjkUnifiedIdeograph858b': '\xe8\x96\x8b',
'CjkUnifiedIdeograph858a': '\xe8\x96\x8a',
'CjkUnifiedIdeograph858f': '\xe8\x96\x8f',
'CjkUnifiedIdeograph858e': '\xe8\x96\x8e',
'CjkUnifiedIdeograph858d': '\xe8\x96\x8d',
'CjkUnifiedIdeograph90ba': '\xe9\x82\xba',
'turnednot': '\xe2\x8c\x99',
'CjkUnifiedIdeograph7bb4': '\xe7\xae\xb4',
'KangxiRadicalDoubleX': '\xe2\xbd\x98',
'CjkCompatibilityIdeographf9dd': '\xef\xa7\x9d',
'LatinSmallLetterDotlessI': '\xc4\xb1',
'LatinSmallLetterDotlessJ': '\xc8\xb7',
'GreekSmallLetterOmicronWithDasiaAndVaria': '\xe1\xbd\x83',
'BoxDrawingsVerticalHeavyAndHorizontalLight': '\xe2\x95\x82',
'UnicodeCharacter0xe3e4': '\xee\x8f\xa4',
'CjkUnifiedIdeograph7bb3': '\xe7\xae\xb3',
'HangulSyllablePeugs': '\xed\x94\x87',
'CjkUnifiedIdeograph4155': '\xe4\x85\x95',
'CircledNumberForty': '\xe3\x8a\xb5',
'CjkUnifiedIdeograph770a': '\xe7\x9c\x8a',
'ChamLetterGha': '\xea\xa8\x89',
'HangulSyllableCeult': '\xec\xb8\xad',
'CjkUnifiedIdeograph770b': '\xe7\x9c\x8b',
'HangulSyllableCwigs': '\xec\xb7\xab',
'CyrillicSmallLetterYae': '\xd4\x99',
'UnicodeCharacter0xda28': '\xed\xa8\xa8',
'UnicodeCharacter0xb96': '\xe0\xae\x96',
'UnicodeCharacter0xb97': '\xe0\xae\x97',
'CjkUnifiedIdeograph8589': '\xe8\x96\x89',
'CjkUnifiedIdeograph8588': '\xe8\x96\x88',
'UnicodeCharacter0xb91': '\xe0\xae\x91',
'CyrillicSmallLetterYat': '\xd1\xa3',
'CjkUnifiedIdeograph8582': '\xe8\x96\x82',
'CjkUnifiedIdeograph8581': '\xe8\x96\x81',
'GeorgianSmallLetterJhan': '\xe2\xb4\x9f',
'CjkUnifiedIdeograph8587': '\xe8\x96\x87',
'CjkUnifiedIdeograph8586': '\xe8\x96\x86',
'UnicodeCharacter0xb98': '\xe0\xae\x98',
'CjkUnifiedIdeograph8584': '\xe8\x96\x84',
'CjkUnifiedIdeograph8055': '\xe8\x81\x95',
'CjkUnifiedIdeograph8054': '\xe8\x81\x94',
'CjkUnifiedIdeograph8057': '\xe8\x81\x97',
'CjkUnifiedIdeograph8056': '\xe8\x81\x96',
'CjkUnifiedIdeograph8051': '\xe8\x81\x91',
'CjkUnifiedIdeograph8050': '\xe8\x81\x90',
'ThaanaLetterHaa': '\xde\x80',
'CjkUnifiedIdeograph8052': '\xe8\x81\x92',
'UnicodeCharacter0xda23': '\xed\xa8\xa3',
'CjkUnifiedIdeograph8059': '\xe8\x81\x99',
'CjkUnifiedIdeograph8058': '\xe8\x81\x98',
'SinhalaLetterRayanna': '\xe0\xb6\xbb',
'UnicodeCharacter0x23f2': '\xe2\x8f\xb2',
'UnicodeCharacter0x23f1': '\xe2\x8f\xb1',
'MongolianLetterZra': '\xe1\xa0\xbf',
'UnicodeCharacter0x23f7': '\xe2\x8f\xb7',
'UnicodeCharacter0x23f6': '\xe2\x8f\xb6',
'UnicodeCharacter0x23f5': '\xe2\x8f\xb5',
'UnicodeCharacter0x23f4': '\xe2\x8f\xb4',
'CjkUnifiedIdeograph5ac1': '\xe5\xab\x81',
'CjkUnifiedIdeograph5ac0': '\xe5\xab\x80',
'UnicodeCharacter0x23f9': '\xe2\x8f\xb9',
'UnicodeCharacter0x23f8': '\xe2\x8f\xb8',
'CjkUnifiedIdeograph5ac5': '\xe5\xab\x85',
'CjkUnifiedIdeograph5ac4': '\xe5\xab\x84',
'CjkUnifiedIdeograph5ac7': '\xe5\xab\x87',
'SinhalaLetterMahaapraanaKayanna': '\xe0\xb6\x9b',
'UnicodeCharacter0xea4f': '\xee\xa9\x8f',
'UnicodeCharacter0xea4d': '\xee\xa9\x8d',
'LatinSmallLetterTurnedRWithHook': '\xc9\xbb',
'HangulSyllableJjweog': '\xec\xad\xa5',
'UnicodeCharacter0xea4c': '\xee\xa9\x8c',
'UnicodeCharacter0x23fc': '\xe2\x8f\xbc',
'UnicodeCharacter0x23fb': '\xe2\x8f\xbb',
'UnicodeCharacter0x23fa': '\xe2\x8f\xba',
'UnicodeCharacter0x23ff': '\xe2\x8f\xbf',
'UnicodeCharacter0x23fe': '\xe2\x8f\xbe',
'UnicodeCharacter0x23fd': '\xe2\x8f\xbd',
'CjkUnifiedIdeograph5aca': '\xe5\xab\x8a',
'CjkUnifiedIdeograph5acc': '\xe5\xab\x8c',
'CjkUnifiedIdeograph5acb': '\xe5\xab\x8b',
'CjkUnifiedIdeograph5ace': '\xe5\xab\x8e',
'CjkUnifiedIdeograph5acd': '\xe5\xab\x8d',
'CjkUnifiedIdeograph5acf': '\xe5\xab\x8f',
'CjkUnifiedIdeograph805e': '\xe8\x81\x9e',
'CjkUnifiedIdeograph805d': '\xe8\x81\x9d',
'CjkUnifiedIdeograph805f': '\xe8\x81\x9f',
'CjkUnifiedIdeograph805a': '\xe8\x81\x9a',
'CjkUnifiedIdeograph805c': '\xe8\x81\x9c',
'CjkUnifiedIdeograph805b': '\xe8\x81\x9b',
'HangulSyllablePyalg': '\xed\x8d\x8d',
'HangulSyllablePyalb': '\xed\x8d\x8f',
'HangulSyllablePyalm': '\xed\x8d\x8e',
'CjkUnifiedIdeograph9a9a': '\xe9\xaa\x9a',
'UnicodeCharacter0xe042': '\xee\x81\x82',
'HangulSyllablePyalt': '\xed\x8d\x91',
'HangulSyllablePyalp': '\xed\x8d\x92',
'HangulSyllableSyiss': '\xec\x8b\x94',
'CjkUnifiedIdeograph9dc4': '\xe9\xb7\x84',
'HangulSyllableDdyaegs': '\xeb\x96\x87',
'HangulSyllableTyaelb': '\xed\x84\x9f',
'DoubleCircledDigitTwo': '\xe2\x93\xb6',
'VaiSyllableOon': '\xea\x95\xb2',
'HangulSyllableTyaelg': '\xed\x84\x9d',
'HangulSyllableTyaelh': '\xed\x84\xa3',
'HangulSyllableJjyenj': '\xec\xaa\xa5',
'HangulSyllableTyaelp': '\xed\x84\xa2',
'EthiopicSyllableWoa': '\xe1\x8b\x8f',
'UnicodeCharacter0xe988': '\xee\xa6\x88',
'HangulSyllableSswalb': '\xec\x8f\xbf',
'HangulSyllableTyaelt': '\xed\x84\xa1',
'ArabicLigatureSeenWithMeemWithJeemInitialForm': '\xef\xb5\xa1',
'UnicodeCharacter0xe982': '\xee\xa6\x82',
'UnicodeCharacter0xe983': '\xee\xa6\x83',
'UnicodeCharacter0xe980': '\xee\xa6\x80',
'UnicodeCharacter0xe981': '\xee\xa6\x81',
'UnicodeCharacter0xe986': '\xee\xa6\x86',
'UnicodeCharacter0xe987': '\xee\xa6\x87',
'UnicodeCharacter0xe984': '\xee\xa6\x84',
'UnicodeCharacter0xe985': '\xee\xa6\x85',
'ArabicLetterRehWithSmallV': '\xda\x92',
'CjkUnifiedIdeograph6975': '\xe6\xa5\xb5',
'CjkUnifiedIdeograph5fcb': '\xe5\xbf\x8b',
'BraillePatternDots245678': '\xe2\xa3\xba',
'CjkUnifiedIdeograph5fca': '\xe5\xbf\x8a',
'CjkUnifiedIdeograph5fcf': '\xe5\xbf\x8f',
'CjkUnifiedIdeograph69fa': '\xe6\xa7\xba',
'CjkUnifiedIdeograph5fcd': '\xe5\xbf\x8d',
'CjkUnifiedIdeograph5fce': '\xe5\xbf\x8e',
'YiSyllableNu': '\xea\x86\x93',
'HalfwidthHangulLetterWa': '\xef\xbf\x8d',
'UnicodeCharacter0xe98b': '\xee\xa6\x8b',
'HangulSyllableJjyenh': '\xec\xaa\xa6',
'UnicodeCharacter0xe98a': '\xee\xa6\x8a',
'UnicodeCharacter0xe98f': '\xee\xa6\x8f',
'LatinSmallLetterDzDigraph': '\xca\xa3',
'KhmerLetterYo': '\xe1\x9e\x99',
'UnicodeCharacter0xe98e': '\xee\xa6\x8e',
'YiSyllableNe': '\xea\x86\x8f',
'HangulSyllableJjweod': '\xec\xad\xab',
'YiSyllableNa': '\xea\x86\x85',
'CjkUnifiedIdeograph8f74': '\xe8\xbd\xb4',
'ArabicLetterEInitialForm': '\xef\xaf\xa6',
'YiSyllableNo': '\xea\x86\x8c',
'YiSyllableNi': '\xea\x85\xbf',
'UnicodeCharacter0xea22': '\xee\xa8\xa2',
'CjkUnifiedIdeograph5fc8': '\xe5\xbf\x88',
'CjkUnifiedIdeograph5fc9': '\xe5\xbf\x89',
'CjkUnifiedIdeograph5fc2': '\xe5\xbf\x82',
'CjkUnifiedIdeograph5fc3': '\xe5\xbf\x83',
'CjkUnifiedIdeograph5fc0': '\xe5\xbf\x80',
'CjkUnifiedIdeograph5fc1': '\xe5\xbf\x81',
'CjkUnifiedIdeograph5fc6': '\xe5\xbf\x86',
'CjkUnifiedIdeograph5fc7': '\xe5\xbf\x87',
'CjkUnifiedIdeograph5fc4': '\xe5\xbf\x84',
'CjkUnifiedIdeograph5fc5': '\xe5\xbf\x85',
'CanadianSyllabicsNaskapiSkw': '\xe1\x94\x8a',
'UnicodeCharacter0xee74': '\xee\xb9\xb4',
'HangulSyllableJyak': '\xec\x9f\xa0',
'EthiopicSyllableVwa': '\xe1\x89\xaf',
'HangulSyllableCyubs': '\xec\xb8\x96',
'el': '\xe2\x88\x88',
'HangulSyllableNwaelp': '\xeb\x86\xbe',
'LepchaLetterWa': '\xe1\xb0\xa2',
'HangulSyllableNwaels': '\xeb\x86\xbc',
'GreaterthanAboveSlantedEqualAboveLessthanAboveSlantedEqual': '\xe2\xaa\x94',
'UnicodeCharacter0xf0b7': '\xef\x82\xb7',
'HangulSyllableNwaelg': '\xeb\x86\xb9',
'CjkUnifiedIdeograph415b': '\xe4\x85\x9b',
'HangulSyllableNwaelb': '\xeb\x86\xbb',
'HangulSyllableNwaelm': '\xeb\x86\xba',
'HangulSyllableNwaelh': '\xeb\x86\xbf',
'YiSyllableHnep': '\xea\x85\xbb',
'UnicodeCharacter0xe132': '\xee\x84\xb2',
'DigitTwoFullStop': '\xe2\x92\x89',
'HangulSyllableDdegg': '\xeb\x96\xbe',
'CjkUnifiedIdeograph8d5c': '\xe8\xb5\x9c',
'ArabicLetterYehWithTail': '\xdb\x8d',
'CjkUnifiedIdeograph8d5e': '\xe8\xb5\x9e',
'TopHalfIntegral': '\xe2\x8c\xa0',
'CjkUnifiedIdeograph8d5f': '\xe8\xb5\x9f',
'LatinCapitalLetterNWithLongRightLeg': '\xc8\xa0',
'HangulSyllableDdegs': '\xeb\x96\xbf',
'HangulSyllableByon': '\xeb\xb5\xa8',
'CjkCompatibilityIdeographf9f2': '\xef\xa7\xb2',
'ArabicFathatan': '\xd9\x8b',
'ArabicShadda': '\xd9\x91',
'CanadianSyllabicsYcreeYoo': '\xe1\x94\xac',
'LatinSmallLigatureFfl': '\xef\xac\x84',
'CjkCompatibilityIdeographf95c': '\xef\xa5\x9c',
'HangulSyllableDyelb': '\xeb\x8e\xb3',
'HangulSyllableCyok': '\xec\xb6\x90',
'OriyaLetterAu': '\xe0\xac\x94',
'CjkUnifiedIdeograph8d59': '\xe8\xb5\x99',
'OriyaLetterAi': '\xe0\xac\x90',
'CjkUnifiedIdeograph7e2d': '\xe7\xb8\xad',
'UnicodeCharacter0xea23': '\xee\xa8\xa3',
'CopticSmallLetterEie': '\xe2\xb2\x89',
'SquareMOverSSquared': '\xe3\x8e\xa8',
'TibetanLetterNna': '\xe0\xbd\x8e',
'OriyaLetterAa': '\xe0\xac\x86',
'CjkUnifiedIdeograph8d53': '\xe8\xb5\x93',
'CjkUnifiedIdeograph8d52': '\xe8\xb5\x92',
'CjkUnifiedIdeograph8d55': '\xe8\xb5\x95',
'CjkUnifiedIdeograph8d54': '\xe8\xb5\x94',
'CjkUnifiedIdeograph8d57': '\xe8\xb5\x97',
'CjkUnifiedIdeograph8d56': '\xe8\xb5\x96',
'CjkUnifiedIdeograph98c5': '\xe9\xa3\x85',
'LimbuSignLoo': '\xe1\xa5\x80',
'LatinSmallLetterCuatrillo': '\xea\x9c\xad',
'CjkCompatibilityIdeographf9aa': '\xef\xa6\xaa',
'BenzeneRingWithCircle': '\xe2\x8f\xa3',
'CjkUnifiedIdeograph3700': '\xe3\x9c\x80',
'ModifierLetterApostrophe': '\xca\xbc',
'OghamLetterEamhancholl': '\xe1\x9a\x99',
'YiRadicalDu': '\xea\x92\xb9',
'CjkRadicalCsimplifiedBird': '\xe2\xbb\xa6',
'UnicodeCharacter0xf7c8': '\xef\x9f\x88',
'UnicodeCharacter0xf36f': '\xef\x8d\xaf',
'LatinSmallLetterSidewaysOWithStroke': '\xe1\xb4\x93',
'UnicodeCharacter0xf36d': '\xef\x8d\xad',
'UnicodeCharacter0xf36c': '\xef\x8d\xac',
'UnicodeCharacter0xf36b': '\xef\x8d\xab',
'UnicodeCharacter0xf36a': '\xef\x8d\xaa',
'SingleLow9QuotationMark': '\xe2\x80\x9a',
'VaiSyllableShee': '\xea\x94\x97',
'HangulSyllableMwak': '\xeb\xab\x9c',
'IntegralAverageWithSlash': '\xe2\xa8\x8f',
'HangulSyllableJjyeng': '\xec\xaa\xb5',
'HangulSyllableMibs': '\xeb\xb0\x8a',
'HangulSyllableGgaes': '\xea\xb9\xbb',
'HangulSyllableGgaep': '\xea\xba\x82',
'HangulSyllableGgaet': '\xea\xba\x81',
'UnicodeCharacter0xf368': '\xef\x8d\xa8',
'UnicodeCharacter0xf367': '\xef\x8d\xa7',
'UnicodeCharacter0xf366': '\xef\x8d\xa6',
'UnicodeCharacter0xf365': '\xef\x8d\xa5',
'UnicodeCharacter0xf364': '\xef\x8d\xa4',
'UnicodeCharacter0xf363': '\xef\x8d\xa3',
'UnicodeCharacter0xf362': '\xef\x8d\xa2',
'UnicodeCharacter0xf361': '\xef\x8d\xa1',
'UnicodeCharacter0xf360': '\xef\x8d\xa0',
'CircledMultiplicationSignWithCircumflexAccent': '\xe2\xa8\xb6',
'HangulSyllableGgaec': '\xea\xb9\xbf',
'CjkUnifiedIdeograph984f': '\xe9\xa1\x8f',
'GeorgianSmallLetterChin': '\xe2\xb4\x99',
'HangulSyllableGgaeg': '\xea\xb9\xa9',
'HangulSyllableGgaed': '\xea\xb9\xaf',
'BalineseLetterDaMurdaMahaprana': '\xe1\xac\xa0',
'HangulSyllableGgaek': '\xea\xba\x80',
'HangulSyllableGgaeh': '\xea\xba\x83',
'HangulSyllableDdinj': '\xeb\x9d\xa5',
'HangulSyllableGgaen': '\xea\xb9\xac',
'HangulSyllableGgael': '\xea\xb9\xb0',
'HangulSyllableGgaem': '\xea\xb9\xb8',
'GlagoliticCapitalLetterFita': '\xe2\xb0\xaa',
'CjkUnifiedIdeograph9e2f': '\xe9\xb8\xaf',
'CjkUnifiedIdeograph9e2d': '\xe9\xb8\xad',
'CjkUnifiedIdeograph9e2e': '\xe9\xb8\xae',
'CjkUnifiedIdeograph9e2b': '\xe9\xb8\xab',
'CjkUnifiedIdeograph9e2c': '\xe9\xb8\xac',
'CjkUnifiedIdeograph9e2a': '\xe9\xb8\xaa',
'UnicodeCharacter0xedfd': '\xee\xb7\xbd',
'RightwardsArrowToBar': '\xe2\x87\xa5',
'CjkUnifiedIdeograph787f': '\xe7\xa1\xbf',
'CjkUnifiedIdeograph787d': '\xe7\xa1\xbd',
'CjkUnifiedIdeograph787e': '\xe7\xa1\xbe',
'CjkUnifiedIdeograph787b': '\xe7\xa1\xbb',
'CjkUnifiedIdeograph787c': '\xe7\xa1\xbc',
'CjkUnifiedIdeograph787a': '\xe7\xa1\xba',
'DoubleCircledDigitOne': '\xe2\x93\xb5',
'HangulSyllableSswel': '\xec\x92\xa4',
'UnicodeCharacter0xe5b8': '\xee\x96\xb8',
'UnicodeCharacter0xe5b9': '\xee\x96\xb9',
'CjkUnifiedIdeograph5689': '\xe5\x9a\x89',
'UnicodeCharacter0xf871': '\xef\xa1\xb1',
'CjkUnifiedIdeograph5688': '\xe5\x9a\x88',
'CjkUnifiedIdeograph9e28': '\xe9\xb8\xa8',
'CjkUnifiedIdeograph9e29': '\xe9\xb8\xa9',
'CjkUnifiedIdeograph9e26': '\xe9\xb8\xa6',
'CjkUnifiedIdeograph9e27': '\xe9\xb8\xa7',
'CjkUnifiedIdeograph9e24': '\xe9\xb8\xa4',
'CjkUnifiedIdeograph9e25': '\xe9\xb8\xa5',
'CjkUnifiedIdeograph9e22': '\xe9\xb8\xa2',
'CjkUnifiedIdeograph9e23': '\xe9\xb8\xa3',
'CjkUnifiedIdeograph9e20': '\xe9\xb8\xa0',
'CjkUnifiedIdeograph9e21': '\xe9\xb8\xa1',
'TibetanSubjoinedLetterPha': '\xe0\xbe\xa5',
'HangulSyllableCwic': '\xec\xb7\xbf',
'HangulSyllableWals': '\xec\x99\x8c',
'CjkUnifiedIdeograph7876': '\xe7\xa1\xb6',
'CjkUnifiedIdeograph7877': '\xe7\xa1\xb7',
'CjkUnifiedIdeograph7874': '\xe7\xa1\xb4',
'CjkUnifiedIdeograph7875': '\xe7\xa1\xb5',
'CjkUnifiedIdeograph7872': '\xe7\xa1\xb2',
'CjkUnifiedIdeograph7873': '\xe7\xa1\xb3',
'CjkUnifiedIdeograph7870': '\xe7\xa1\xb0',
'CjkUnifiedIdeograph7871': '\xe7\xa1\xb1',
'HangulSyllableSweobs': '\xec\x89\x86',
'HangulSyllableBalp': '\xeb\xb0\xa2',
'CjkUnifiedIdeograph7878': '\xe7\xa1\xb8',
'CjkUnifiedIdeograph7879': '\xe7\xa1\xb9',
'CentSign': '\xc2\xa2',
'HangulSyllableBwaels': '\xeb\xb4\xb8',
'HangulSyllableBwaelt': '\xeb\xb4\xb9',
'CjkUnifiedIdeograph5686': '\xe5\x9a\x86',
'UnicodeCharacter0xddc2': '\xed\xb7\x82',
'HangulSyllableBalt': '\xeb\xb0\xa1',
'HangulSyllableJjweoc': '\xec\xad\xbb',
'UnicodeCharacter0xe5b7': '\xee\x96\xb7',
'CjkUnifiedIdeograph69f1': '\xe6\xa7\xb1',
'HangulSyllableBwaelh': '\xeb\xb4\xbb',
'ArabicLetterDdahal': '\xda\x8d',
'VaiSyllableGbo': '\xea\x97\x87',
'ArabicLigatureLamWithMeemFinalForm': '\xef\xb2\x85',
'VaiSyllableGba': '\xea\x95\x97',
'VaiSyllableGbe': '\xea\x97\xaf',
'CjkUnifiedIdeograph36de': '\xe3\x9b\x9e',
'CjkUnifiedIdeograph36dd': '\xe3\x9b\x9d',
'CjkUnifiedIdeograph36df': '\xe3\x9b\x9f',
'CjkUnifiedIdeograph36da': '\xe3\x9b\x9a',
'CjkUnifiedIdeograph36dc': '\xe3\x9b\x9c',
'CjkUnifiedIdeograph36db': '\xe3\x9b\x9b',
'VaiSyllableGbu': '\xea\x96\xa1',
'HangulSyllableBwaelm': '\xeb\xb4\xb6',
'LatinSmallLetterGWithCircumflex': '\xc4\x9d',
'HangulSyllableHyes': '\xed\x98\xaf',
'ChamLetterHa': '\xea\xa8\xa8',
'HangulSyllableHyep': '\xed\x98\xb6',
'UnicodeCharacter0xa4df': '\xea\x93\x9f',
'HangulChoseongPieupkiyeok': '\xe1\x84\x9e',
'HangulSyllableHyeg': '\xed\x98\x9d',
'HangulSyllableBalb': '\xeb\xb0\x9f',
'HangulSyllableHyed': '\xed\x98\xa3',
'HangulSyllableHyec': '\xed\x98\xb3',
'HangulSyllableHyeb': '\xed\x98\xad',
'HangulSyllableHyeo': '\xed\x98\x80',
'HangulSyllableHyen': '\xed\x98\xa0',
'HangulSyllableHyem': '\xed\x98\xac',
'HangulSyllableHyel': '\xed\x98\xa4',
'HangulSyllableHyek': '\xed\x98\xb4',
'HangulSyllableHyej': '\xed\x98\xb2',
'HangulSyllableHyeh': '\xed\x98\xb7',
'CjkUnifiedIdeograph36d5': '\xe3\x9b\x95',
'CjkUnifiedIdeograph36d4': '\xe3\x9b\x94',
'CjkUnifiedIdeograph36d7': '\xe3\x9b\x97',
'CjkUnifiedIdeograph36d6': '\xe3\x9b\x96',
'CjkUnifiedIdeograph36d1': '\xe3\x9b\x91',
'CjkUnifiedIdeograph36d0': '\xe3\x9b\x90',
'CjkUnifiedIdeograph36d3': '\xe3\x9b\x93',
'Succeeds': '\xe2\x89\xbb',
'CjkUnifiedIdeograph36d9': '\xe3\x9b\x99',
'CjkUnifiedIdeograph36d8': '\xe3\x9b\x98',
'HangulSyllableBalg': '\xeb\xb0\x9d',
'HangulSyllableNwelb': '\xeb\x89\x87',
'CjkUnifiedIdeograph6de8': '\xe6\xb7\xa8',
'HangulSyllableNwelg': '\xeb\x89\x85',
'HangulSyllableNwelh': '\xeb\x89\x8b',
'CjkUnifiedIdeograph56ec': '\xe5\x9b\xac',
'HangulSyllableNwelm': '\xeb\x89\x86',
'CjkUnifiedIdeograph8573': '\xe8\x95\xb3',
'CjkUnifiedIdeograph9081': '\xe9\x82\x81',
'CjkUnifiedIdeograph9080': '\xe9\x82\x80',
'CjkUnifiedIdeograph9083': '\xe9\x82\x83',
'CjkUnifiedIdeograph9082': '\xe9\x82\x82',
'LatinCapitalLetterAWithRingAbove': '\xc3\x85',
'CjkUnifiedIdeograph9084': '\xe9\x82\x84',
'CjkUnifiedIdeograph6059': '\xe6\x81\x99',
'CjkUnifiedIdeograph6058': '\xe6\x81\x98',
'CjkUnifiedIdeograph6057': '\xe6\x81\x97',
'CjkUnifiedIdeograph6056': '\xe6\x81\x96',
'CjkUnifiedIdeograph6055': '\xe6\x81\x95',
'CjkUnifiedIdeograph6054': '\xe6\x81\x94',
'CjkUnifiedIdeograph6053': '\xe6\x81\x93',
'CjkUnifiedIdeograph6052': '\xe6\x81\x92',
'CjkUnifiedIdeograph6051': '\xe6\x81\x91',
'CjkUnifiedIdeograph6050': '\xe6\x81\x90',
'HangulSyllableJjweb': '\xec\xae\x91',
'HangulSyllableJjwec': '\xec\xae\x97',
'CjkUnifiedIdeograph9846': '\xe9\xa1\x86',
'HangulSyllableJjweg': '\xec\xae\x81',
'HangulSyllableJjwed': '\xec\xae\x87',
'CjkUnifiedIdeograph6de2': '\xe6\xb7\xa2',
'HangulJongseongRieulkiyeok': '\xe1\x86\xb0',
'CjkUnifiedIdeograph955b': '\xe9\x95\x9b',
'CjkUnifiedIdeograph43ad': '\xe4\x8e\xad',
'CjkUnifiedIdeograph43ae': '\xe4\x8e\xae',
'CjkUnifiedIdeograph43ab': '\xe4\x8e\xab',
'CjkUnifiedIdeograph43ac': '\xe4\x8e\xac',
'CjkUnifiedIdeograph955e': '\xe9\x95\x9e',
'CjkUnifiedIdeograph43aa': '\xe4\x8e\xaa',
'MongolianLetterZa': '\xe1\xa0\xbd',
'CjkUnifiedIdeograph79ea': '\xe7\xa7\xaa',
'CjkUnifiedIdeograph79ec': '\xe7\xa7\xac',
'CjkUnifiedIdeograph79eb': '\xe7\xa7\xab',
'CjkUnifiedIdeograph79ee': '\xe7\xa7\xae',
'CjkUnifiedIdeograph79ed': '\xe7\xa7\xad',
'CjkUnifiedIdeograph79ef': '\xe7\xa7\xaf',
'SyriacLetterSadhe': '\xdc\xa8',
'YiSyllableBbiex': '\xea\x81\x9b',
'HangulSyllableJwaenj': '\xec\xa2\xad',
'CjkUnifiedIdeograph8578': '\xe8\x95\xb8',
'HangulSyllableJwaenh': '\xec\xa2\xae',
'CjkUnifiedIdeograph5a87': '\xe5\xaa\x87',
'HangulSyllableJwaeng': '\xec\xa2\xbd',
'UnicodeCharacter0xf8ad': '\xef\xa2\xad',
'CjkUnifiedIdeograph6de4': '\xe6\xb7\xa4',
'CjkUnifiedIdeograph84b7': '\xe8\x92\xb7',
'LatinCapitalLetterOWithHookAbove': '\xe1\xbb\x8e',
'HangulSyllableBbyeh': '\xeb\xbc\xbf',
'OriyaVowelSignVocalicL': '\xe0\xad\xa2',
'CjkUnifiedIdeograph9559': '\xe9\x95\x99',
'CjkUnifiedIdeograph9558': '\xe9\x95\x98',
'UnicodeCharacter0x2068': '\xe2\x81\xa8',
'UnicodeCharacter0x2069': '\xe2\x81\xa9',
'CjkUnifiedIdeograph43a8': '\xe4\x8e\xa8',
'CjkUnifiedIdeograph43a9': '\xe4\x8e\xa9',
'CjkUnifiedIdeograph43a6': '\xe4\x8e\xa6',
'UnicodeCharacter0x2065': '\xe2\x81\xa5',
'UnicodeCharacter0x2066': '\xe2\x81\xa6',
'UnicodeCharacter0x2067': '\xe2\x81\xa7',
'CjkUnifiedIdeograph43a2': '\xe4\x8e\xa2',
'CjkUnifiedIdeograph43a3': '\xe4\x8e\xa3',
'CjkUnifiedIdeograph43a0': '\xe4\x8e\xa0',
'CjkUnifiedIdeograph43a1': '\xe4\x8e\xa1',
'CjkUnifiedIdeograph908a': '\xe9\x82\x8a',
'CjkUnifiedIdeograph908c': '\xe9\x82\x8c',
'CjkUnifiedIdeograph908b': '\xe9\x82\x8b',
'CjkUnifiedIdeograph908e': '\xe9\x82\x8e',
'CjkUnifiedIdeograph908d': '\xe9\x82\x8d',
'CjkUnifiedIdeograph908f': '\xe9\x82\x8f',
'HangulSyllableBbyep': '\xeb\xbc\xbe',
'EthiopicTonalMarkKurt': '\xe1\x8e\x99',
'CjkUnifiedIdeograph605e': '\xe6\x81\x9e',
'CjkUnifiedIdeograph605d': '\xe6\x81\x9d',
'CjkUnifiedIdeograph605c': '\xe6\x81\x9c',
'SyriacTwoVerticalDotsBelow': '\xdd\x84',
'CjkUnifiedIdeograph605a': '\xe6\x81\x9a',
'OriyaVowelSignVocalicR': '\xe0\xad\x83',
'TifinaghLetterYash': '\xe2\xb5\x9b',
'YiSyllableBbiep': '\xea\x81\x9d',
'UnicodeCharacter0xf0b3': '\xef\x82\xb3',
'HangulSyllablePyass': '\xed\x8d\x98',
'NotEquivalentTo': '\xe2\x89\xad',
'GreekCapitalLetterEtaWithDasiaAndOxia': '\xe1\xbc\xad',
'CjkUnifiedIdeograph98cf': '\xe9\xa3\x8f',
'VaiSyllableNdoleDo': '\xea\x98\xab',
'CjkUnifiedIdeograph79e9': '\xe7\xa7\xa9',
'CjkUnifiedIdeograph79e8': '\xe7\xa7\xa8',
'CjkUnifiedIdeograph9f68': '\xe9\xbd\xa8',
'ModifierLetterSmallCapitalInvertedR': '\xca\xb6',
'CjkUnifiedIdeograph79e1': '\xe7\xa7\xa1',
'CjkUnifiedIdeograph79e0': '\xe7\xa7\xa0',
'TifinaghLetterYass': '\xe2\xb5\x9a',
'CjkUnifiedIdeograph79e2': '\xe7\xa7\xa2',
'CjkUnifiedIdeograph79e5': '\xe7\xa7\xa5',
'CjkUnifiedIdeograph79e4': '\xe7\xa7\xa4',
'CjkUnifiedIdeograph79e7': '\xe7\xa7\xa7',
'GreekSmallLetterIotaWithDialytikaAndVaria': '\xe1\xbf\x92',
'HangulSyllablePyeo': '\xed\x8e\xb4',
'CjkUnifiedIdeograph70d7': '\xe7\x83\x97',
'HangulSyllablePyem': '\xed\x8f\xa0',
'LatinSmallLetterZWithAcute': '\xc5\xba',
'HangulSyllablePyek': '\xed\x8f\xa8',
'HangulSyllablePyej': '\xed\x8f\xa6',
'CjkUnifiedIdeograph6c82': '\xe6\xb2\x82',
'HangulSyllablePyeg': '\xed\x8f\x91',
'HangulSyllablePyed': '\xed\x8f\x97',
'HangulSyllablePyec': '\xed\x8f\xa7',
'CjkUnifiedIdeograph70d5': '\xe7\x83\x95',
'CjkUnifiedIdeograph5a82': '\xe5\xaa\x82',
'CjkUnifiedIdeograph70d4': '\xe7\x83\x94',
'UnicodeCharacter0xe7dd': '\xee\x9f\x9d',
'CjkUnifiedIdeograph5ac9': '\xe5\xab\x89',
'CjkUnifiedIdeograph70d3': '\xe7\x83\x93',
'HangulSyllablePyet': '\xed\x8f\xa9',
'CjkUnifiedIdeograph9b5b': '\xe9\xad\x9b',
'CjkUnifiedIdeograph70d2': '\xe7\x83\x92',
'IdenticalToAndSlantedParallel': '\xe2\xa7\xa5',
'CjkUnifiedIdeograph70d1': '\xe7\x83\x91',
'v': '\xce\xbd',
'CjkUnifiedIdeograph701a': '\xe7\x80\x9a',
'CjkUnifiedIdeograph701b': '\xe7\x80\x9b',
'CjkUnifiedIdeograph701c': '\xe7\x80\x9c',
'CjkUnifiedIdeograph701d': '\xe7\x80\x9d',
'CjkUnifiedIdeograph701e': '\xe7\x80\x9e',
'CjkUnifiedIdeograph701f': '\xe7\x80\x9f',
'UnicodeCharacter0xde5c': '\xed\xb9\x9c',
'CjkUnifiedIdeograph7b79': '\xe7\xad\xb9',
'SundaneseLetterHa': '\xe1\xae\xa0',
'CjkUnifiedIdeograph370a': '\xe3\x9c\x8a',
'CjkUnifiedIdeograph370b': '\xe3\x9c\x8b',
'CjkUnifiedIdeograph370c': '\xe3\x9c\x8c',
'CjkUnifiedIdeograph370d': '\xe3\x9c\x8d',
'CjkUnifiedIdeograph370e': '\xe3\x9c\x8e',
'CjkUnifiedIdeograph370f': '\xe3\x9c\x8f',
'UnicodeCharacter0xde8f': '\xed\xba\x8f',
'WhiteVerticalRectangle': '\xe2\x96\xaf',
'UnicodeCharacter0xdfc6': '\xed\xbf\x86',
'LeftRaisedOmissionBracket': '\xe2\xb8\x8c',
'BalineseVowelSignTalingRepa': '\xe1\xac\xbf',
'CanadianSyllabicsCwa': '\xe1\x92\x9c',
'CjkUnifiedIdeograph3709': '\xe3\x9c\x89',
'CanadianSyllabicsCwe': '\xe1\x92\x92',
'UnicodeCharacter0xee0c': '\xee\xb8\x8c',
'HangulJongseongMieumhieuh': '\xe1\x87\xa1',
'CjkUnifiedIdeograph3701': '\xe3\x9c\x81',
'CjkUnifiedIdeograph3702': '\xe3\x9c\x82',
'CjkUnifiedIdeograph3703': '\xe3\x9c\x83',
'CjkUnifiedIdeograph3704': '\xe3\x9c\x84',
'CjkUnifiedIdeograph3705': '\xe3\x9c\x85',
'CanadianSyllabicsCwo': '\xe1\x92\x98',
'CjkUnifiedIdeograph3707': '\xe3\x9c\x87',
'CjkUnifiedIdeograph7018': '\xe7\x80\x98',
'LatinSmallLetterUWithHornAndTilde': '\xe1\xbb\xaf',
'HangulSyllablePwalp': '\xed\x90\x96',
'CjkUnifiedIdeograph56e5': '\xe5\x9b\xa5',
'CjkUnifiedIdeograph7010': '\xe7\x80\x90',
'CjkUnifiedIdeograph7011': '\xe7\x80\x91',
'CjkUnifiedIdeograph7012': '\xe7\x80\x92',
'CjkUnifiedIdeograph7013': '\xe7\x80\x93',
'CjkUnifiedIdeograph7014': '\xe7\x80\x94',
'CjkUnifiedIdeograph7015': '\xe7\x80\x95',
'CjkUnifiedIdeograph7016': '\xe7\x80\x96',
'CjkUnifiedIdeograph7017': '\xe7\x80\x97',
'TibetanMarkSbrulShad': '\xe0\xbc\x88',
'RightAngleVariantWithSquare': '\xe2\xa6\x9c',
'SymbolForDeviceControlThree': '\xe2\x90\x93',
'HangulSyllableMwaep': '\xeb\xab\xba',
'HangulSyllableMwaes': '\xeb\xab\xb3',
'HangulSyllableMwaet': '\xeb\xab\xb9',
'CjkUnifiedIdeograph857d': '\xe8\x95\xbd',
'IntegralWithUnion': '\xe2\xa8\x9a',
'HangulSyllableMwaej': '\xeb\xab\xb6',
'CjkUnifiedIdeograph857e': '\xe8\x95\xbe',
'HangulSyllableMwael': '\xeb\xab\xa8',
'HangulSyllableMwaem': '\xeb\xab\xb0',
'HangulSyllableMwaen': '\xeb\xab\xa4',
'LowerFiveEighthsBlock': '\xe2\x96\x85',
'HangulSyllableMwaeb': '\xeb\xab\xb1',
'HangulSyllableMwaec': '\xeb\xab\xb7',
'HangulSyllableMwaed': '\xeb\xab\xa7',
'HangulJungseongYiu': '\xe1\x86\x97',
'HangulSyllableMwaeg': '\xeb\xab\xa1',
'DingbatCircledSansserifDigitThree': '\xe2\x9e\x82',
'HangulSyllableJjuss': '\xec\xad\x9c',
'SylotiNagriLetterU': '\xea\xa0\x83',
'SylotiNagriLetterO': '\xea\xa0\x85',
'LatinCapitalLetterRWithLineBelow': '\xe1\xb9\x9e',
'CircledReverseSolidus': '\xe2\xa6\xb8',
'UnicodeCharacter0xdf30': '\xed\xbc\xb0',
'UnicodeCharacter0xdf37': '\xed\xbc\xb7',
'UnicodeCharacter0xdf36': '\xed\xbc\xb6',
'SylotiNagriLetterI': '\xea\xa0\x81',
'UnicodeCharacter0xdf34': '\xed\xbc\xb4',
'UnicodeCharacter0xf0e9': '\xef\x83\xa9',
'OriyaDigitOne': '\xe0\xad\xa7',
'UnicodeCharacter0xdf38': '\xed\xbc\xb8',
'SylotiNagriLetterA': '\xea\xa0\x80',
'UnicodeCharacter0xf0e8': '\xef\x83\xa8',
'ReversedEmptySet': '\xe2\xa6\xb0',
'BalineseDigitOne': '\xe1\xad\x91',
'CjkUnifiedIdeograph6def': '\xe6\xb7\xaf',
'UnicodeCharacter0xf0e5': '\xef\x83\xa5',
'UnicodeCharacter0xe5ba': '\xee\x96\xba',
'ArmenianModifierLetterLeftHalfRing': '\xd5\x99',
'ArabicLetterAlefIsolatedForm': '\xef\xba\x8d',
'UnicodeCharacter0xe5bb': '\xee\x96\xbb',
'UnicodeCharacter0xdf3c': '\xed\xbc\xbc',
'UnicodeCharacter0xdf3b': '\xed\xbc\xbb',
'UnicodeCharacter0xdf3a': '\xed\xbc\xba',
'CjkUnifiedIdeograph9f60': '\xe9\xbd\xa0',
'UnicodeCharacter0xdf3f': '\xed\xbc\xbf',
'CombiningUrAbove': '\xe1\xb7\x91',
'UnicodeCharacter0xdf3d': '\xed\xbc\xbd',
'CjkUnifiedIdeograph7d2e': '\xe7\xb4\xae',
'HangulSyllableBbyaelg': '\xeb\xba\xbd',
'neg': '\xc2\xac',
'CjkUnifiedIdeograph568f': '\xe5\x9a\x8f',
'LatinCapitalLetterOWithLoop': '\xea\x9d\x8c',
'BoxDrawingsLightDown': '\xe2\x95\xb7',
'UnicodeCharacter0xf0e3': '\xef\x83\xa3',
'HangulSyllableBbyaels': '\xeb\xbb\x80',
'HangulSyllableBbyaelp': '\xeb\xbb\x82',
'HangulSyllableBbyaelt': '\xeb\xbb\x81',
'HangulSyllableYeolb': '\xec\x97\xb7',
'HangulSyllableYeolg': '\xec\x97\xb5',
'HangulSyllableRyubs': '\xeb\xa5\xaa',
'UnicodeCharacter0x2ceb': '\xe2\xb3\xab',
'UnicodeCharacter0x2cec': '\xe2\xb3\xac',
'HangulSyllableYeolh': '\xec\x97\xbb',
'UnicodeCharacter0x2cef': '\xe2\xb3\xaf',
'HangulSyllableDyuss': '\xeb\x93\x94',
'UnicodeCharacter0x2ced': '\xe2\xb3\xad',
'UnicodeCharacter0x2cee': '\xe2\xb3\xae',
'HangulSyllableGgeugs': '\xeb\x81\x87',
'CjkUnifiedIdeograph3c9f': '\xe3\xb2\x9f',
'CjkUnifiedIdeograph3c9e': '\xe3\xb2\x9e',
'CjkUnifiedIdeograph3c9d': '\xe3\xb2\x9d',
'CjkUnifiedIdeograph3c9c': '\xe3\xb2\x9c',
'CjkUnifiedIdeograph3c9b': '\xe3\xb2\x9b',
'CjkUnifiedIdeograph3c9a': '\xe3\xb2\x9a',
'HangulSyllableYeolt': '\xec\x97\xb9',
'HangulSyllablePyinj': '\xed\x94\xa5',
'UnicodeCharacter0xa8f6': '\xea\xa3\xb6',
'HangulSyllablePyinh': '\xed\x94\xa6',
'CjkUnifiedIdeograph9f62': '\xe9\xbd\xa2',
'CjkCompatibilityIdeographf90e': '\xef\xa4\x8e',
'HangulSyllableDdonj': '\xeb\x98\x95',
'HangulSyllableDdonh': '\xeb\x98\x96',
'CjkCompatibilityIdeographf90a': '\xef\xa4\x8a',
'UnicodeCharacter0xabba': '\xea\xae\xba',
'CjkUnifiedIdeograph6868': '\xe6\xa1\xa8',
'CjkUnifiedIdeograph6869': '\xe6\xa1\xa9',
'CjkUnifiedIdeograph6866': '\xe6\xa1\xa6',
'CjkUnifiedIdeograph6867': '\xe6\xa1\xa7',
'CjkUnifiedIdeograph6864': '\xe6\xa1\xa4',
'CjkUnifiedIdeograph6865': '\xe6\xa1\xa5',
'CjkUnifiedIdeograph6862': '\xe6\xa1\xa2',
'CjkUnifiedIdeograph6863': '\xe6\xa1\xa3',
'CjkUnifiedIdeograph6860': '\xe6\xa1\xa0',
'CjkUnifiedIdeograph6861': '\xe6\xa1\xa1',
'HebrewLetterFinalMem': '\xd7\x9d',
'HangulSyllableCwig': '\xec\xb7\xa9',
'CjkUnifiedIdeograph3c97': '\xe3\xb2\x97',
'CjkUnifiedIdeograph3c96': '\xe3\xb2\x96',
'CjkUnifiedIdeograph3c95': '\xe3\xb2\x95',
'CjkUnifiedIdeograph3c94': '\xe3\xb2\x94',
'CjkUnifiedIdeograph3c93': '\xe3\xb2\x93',
'CjkUnifiedIdeograph3c92': '\xe3\xb2\x92',
'CjkUnifiedIdeograph3c91': '\xe3\xb2\x91',
'CjkUnifiedIdeograph3c90': '\xe3\xb2\x90',
'CjkUnifiedIdeograph4e83': '\xe4\xba\x83',
'LatinSmallLetterTurnedRWithLongLeg': '\xc9\xba',
'CjkUnifiedIdeograph3c99': '\xe3\xb2\x99',
'CjkUnifiedIdeograph3c98': '\xe3\xb2\x98',
'HangulSyllableJoelt': '\xec\xa3\x91',
'ArabicLetterTtehehIsolatedForm': '\xef\xad\x9e',
'CjkCompatibilityIdeographf905': '\xef\xa4\x85',
'CjkCompatibilityIdeographf904': '\xef\xa4\x84',
'CjkCompatibilityIdeographf907': '\xef\xa4\x87',
'ColonSign': '\xe2\x82\xa1',
'CjkCompatibilityIdeographf901': '\xef\xa4\x81',
'CjkCompatibilityIdeographf900': '\xef\xa4\x80',
'CjkCompatibilityIdeographf903': '\xef\xa4\x83',
'CjkCompatibilityIdeographf902': '\xef\xa4\x82',
'ArmenianSmallLetterJheh': '\xd5\xbb',
'HangulSyllableJoelp': '\xec\xa3\x92',
'CjkUnifiedIdeograph686d': '\xe6\xa1\xad',
'CjkUnifiedIdeograph686e': '\xe6\xa1\xae',
'CjkUnifiedIdeograph686b': '\xe6\xa1\xab',
'CjkUnifiedIdeograph686c': '\xe6\xa1\xac',
'CjkUnifiedIdeograph686a': '\xe6\xa1\xaa',
'HangulSyllableJeum': '\xec\xa6\x98',
'HangulSyllableTeul': '\xed\x8b\x80',
'HangulSyllableKweobs': '\xed\x80\x8e',
'MongolianLetterSibeGaa': '\xe1\xa1\xac',
'ReversedTildeEquals': '\xe2\x8b\x8d',
'ArabicLigatureGhainWithAlefMaksuraFinalForm': '\xef\xb4\x95',
'GujaratiLetterPa': '\xe0\xaa\xaa',
'HangulSyllableJeuh': '\xec\xa6\xa3',
'_phi': '\xe1\xb5\xa9',
'CjkUnifiedIdeograph61d9': '\xe6\x87\x99',
'CjkUnifiedIdeograph61d8': '\xe6\x87\x98',
'GeorgianLetterAin': '\xe1\x83\xba',
'CjkUnifiedIdeograph61d5': '\xe6\x87\x95',
'CjkUnifiedIdeograph61d4': '\xe6\x87\x94',
'CjkUnifiedIdeograph61d7': '\xe6\x87\x97',
'CjkUnifiedIdeograph61d6': '\xe6\x87\x96',
'CjkUnifiedIdeograph61d1': '\xe6\x87\x91',
'CjkUnifiedIdeograph61d0': '\xe6\x87\x90',
'CjkUnifiedIdeograph61d3': '\xe6\x87\x93',
'CjkUnifiedIdeograph61d2': '\xe6\x87\x92',
'CjkUnifiedIdeograph40fd': '\xe4\x83\xbd',
'CjkUnifiedIdeograph40fe': '\xe4\x83\xbe',
'CjkUnifiedIdeograph40ff': '\xe4\x83\xbf',
'HangulSyllableCinh': '\xec\xb9\x9e',
'CjkUnifiedIdeograph910a': '\xe9\x84\x8a',
'CjkUnifiedIdeograph40fa': '\xe4\x83\xba',
'CjkUnifiedIdeograph40fb': '\xe4\x83\xbb',
'KangxiRadicalEven': '\xe2\xbf\x91',
'YiSyllableHlurx': '\xea\x86\xaf',
'HangulSyllableGgegg': '\xea\xbb\x9a',
'HangulSyllableNwalm': '\xeb\x86\x9e',
'GeorgianSmallLetterVin': '\xe2\xb4\x85',
'HangulSyllableRwanh': '\xeb\xa1\xbe',
'HangulSyllableMyig': '\xeb\xaf\x9d',
'UnicodeCharacter0xf018': '\xef\x80\x98',
'HangulSyllableGgegs': '\xea\xbb\x9b',
'HangulSyllableRwang': '\xeb\xa2\x8d',
'EthiopicSyllableRee': '\xe1\x88\xac',
'CjkUnifiedIdeograph91a3': '\xe9\x86\xa3',
'CjkUnifiedIdeograph40f8': '\xe4\x83\xb8',
'CjkUnifiedIdeograph40f9': '\xe4\x83\xb9',
'CjkUnifiedIdeograph70df': '\xe7\x83\x9f',
'CjkUnifiedIdeograph40f4': '\xe4\x83\xb4',
'CjkUnifiedIdeograph40f5': '\xe4\x83\xb5',
'CjkUnifiedIdeograph40f6': '\xe4\x83\xb6',
'CjkUnifiedIdeograph40f7': '\xe4\x83\xb7',
'CjkUnifiedIdeograph40f0': '\xe4\x83\xb0',
'CjkUnifiedIdeograph40f1': '\xe4\x83\xb1',
'CjkUnifiedIdeograph40f2': '\xe4\x83\xb2',
'CjkUnifiedIdeograph40f3': '\xe4\x83\xb3',
'CjkUnifiedIdeograph7b7c': '\xe7\xad\xbc',
'LatinSmallLetterUWithHookAbove': '\xe1\xbb\xa7',
'_v': '\xe1\xb5\xa5',
'CjkUnifiedIdeograph61de': '\xe6\x87\x9e',
'CjkUnifiedIdeograph61dd': '\xe6\x87\x9d',
'CjkUnifiedIdeograph61df': '\xe6\x87\x9f',
'CjkUnifiedIdeograph61da': '\xe6\x87\x9a',
'CjkUnifiedIdeograph61dc': '\xe6\x87\x9c',
'CjkUnifiedIdeograph61db': '\xe6\x87\x9b',
'CjkUnifiedIdeograph9fb9': '\xe9\xbe\xb9',
'CharacterTie': '\xe2\x81\x80',
'CjkUnifiedIdeograph70da': '\xe7\x83\x9a',
'CjkUnifiedIdeograph9fb1': '\xe9\xbe\xb1',
'CjkUnifiedIdeograph9fb0': '\xe9\xbe\xb0',
'CjkUnifiedIdeograph9fb3': '\xe9\xbe\xb3',
'CjkUnifiedIdeograph91a4': '\xe9\x86\xa4',
'CjkUnifiedIdeograph9fb5': '\xe9\xbe\xb5',
'CjkUnifiedIdeograph9fb4': '\xe9\xbe\xb4',
'CjkUnifiedIdeograph9fb7': '\xe9\xbe\xb7',
'CjkUnifiedIdeograph9fb6': '\xe9\xbe\xb6',
'HangulSyllableJeut': '\xec\xa6\xa1',
'PhagspaLetterZa': '\xea\xa1\x95',
'HangulSyllableMyik': '\xeb\xaf\xb4',
'UnicodeCharacter0xfe6c': '\xef\xb9\xac',
'DevanagariLetterKhha': '\xe0\xa5\x99',
'TaiLeLetterLa': '\xe1\xa5\x98',
'HangulSyllableSswaebs': '\xec\x90\xa2',
'ArabicLigatureTehWithJeemInitialForm': '\xef\xb2\xa1',
'CjkUnifiedIdeograph6517': '\xe6\x94\x97',
'HangulSyllableJeup': '\xec\xa6\xa2',
'UnicodeCharacter0xf0e4': '\xef\x83\xa4',
'YiSyllableZziex': '\xea\x8b\x95',
'KayahLiLetterNya': '\xea\xa4\x91',
'CjkUnifiedIdeograph6285': '\xe6\x8a\x85',
'YiSyllableZziet': '\xea\x8b\x94',
'_i': '\xe1\xb5\xa2',
'EthiopicSyllableTwa': '\xe1\x89\xb7',
'HangulSyllablePim': '\xed\x95\x8c',
'HangulSyllableBbwass': '\xeb\xbd\xb0',
'UnicodeCharacter0xf024': '\xef\x80\xa4',
'LatinCapitalLetterGWithCircumflex': '\xc4\x9c',
'CjkCompatibilityIdeographf954': '\xef\xa5\x94',
'LatinCapitalLetterPWithStrokeThroughDescender': '\xea\x9d\x90',
'UnicodeCharacter0xf025': '\xef\x80\xa5',
'CjkUnifiedIdeograph9fba': '\xe9\xbe\xba',
'CjkUnifiedIdeograph9fbb': '\xe9\xbe\xbb',
'UnicodeCharacter0xf026': '\xef\x80\xa6',
'UnicodeCharacter0xf578': '\xef\x95\xb8',
'CombiningCyrillicLetterChe': '\xe2\xb7\xb1',
'TagalogLetterTa': '\xe1\x9c\x86',
'ReversedQuestionMark': '\xe2\xb8\xae',
'CircleWithVerticalFill': '\xe2\x97\x8d',
'NewTaiLueLetterHighHa': '\xe1\xa6\xa0',
'MathematicalLeftWhiteSquareBracket': '\xe2\x9f\xa6',
'UnicodeCharacter0xf8a7': '\xef\xa2\xa7',
'UnicodeCharacter0xf022': '\xef\x80\xa2',
'HangulSyllableRwelh': '\xeb\xa4\xaf',
'UnicodeCharacter0xf023': '\xef\x80\xa3',
'HangulSyllableJjwaeg': '\xec\xab\xb5',
'KangxiRadicalWhite': '\xe2\xbd\xa9',
'CjkUnifiedIdeograph5b6e': '\xe5\xad\xae',
'CjkUnifiedIdeograph5b6d': '\xe5\xad\xad',
'CjkUnifiedIdeograph5b6f': '\xe5\xad\xaf',
'CjkUnifiedIdeograph5b6a': '\xe5\xad\xaa',
'HangulSyllableJjwaeb': '\xec\xac\x85',
'CjkUnifiedIdeograph5b6c': '\xe5\xad\xac',
'CjkUnifiedIdeograph5b6b': '\xe5\xad\xab',
'MyanmarLetterShanPha': '\xe1\x81\xbd',
'HangulSyllableRyuls': '\xeb\xa5\xa4',
'CyrillicCapitalLetterMonographUk': '\xea\x99\x8a',
'HangulSyllableCweonj': '\xec\xb6\xb5',
'DoubleObliqueHyphen': '\xe2\xb8\x97',
'UnicodeCharacter0xe9ed': '\xee\xa7\xad',
'UnicodeCharacter0xf168': '\xef\x85\xa8',
'HangulSyllableJjyelg': '\xec\xaa\xa9',
'UnicodeCharacter0xf165': '\xef\x85\xa5',
'HangulSyllableJjyelh': '\xec\xaa\xaf',
'UnicodeCharacter0xf167': '\xef\x85\xa7',
'UnicodeCharacter0xf166': '\xef\x85\xa6',
'HangulSyllableJjyelm': '\xec\xaa\xaa',
'UnicodeCharacter0xf160': '\xef\x85\xa0',
'UnicodeCharacter0xf163': '\xef\x85\xa3',
'UnicodeCharacter0xf162': '\xef\x85\xa2',
'HangulSyllableJjyelp': '\xec\xaa\xae',
'HangulSyllableJjyels': '\xec\xaa\xac',
'HangulSyllableJjyelt': '\xec\xaa\xad',
'CjkUnifiedIdeograph4d08': '\xe4\xb4\x88',
'CjkUnifiedIdeograph4d09': '\xe4\xb4\x89',
'CjkUnifiedIdeograph4d06': '\xe4\xb4\x86',
'CjkUnifiedIdeograph4d07': '\xe4\xb4\x87',
'CjkUnifiedIdeograph4d04': '\xe4\xb4\x84',
'CjkUnifiedIdeograph4d05': '\xe4\xb4\x85',
'CjkUnifiedIdeograph4d02': '\xe4\xb4\x82',
'CjkUnifiedIdeograph4d03': '\xe4\xb4\x83',
'CjkUnifiedIdeograph4d00': '\xe4\xb4\x80',
'CjkUnifiedIdeograph4d01': '\xe4\xb4\x81',
'CjkUnifiedIdeograph5b65': '\xe5\xad\xa5',
'CjkUnifiedIdeograph5b64': '\xe5\xad\xa4',
'CjkUnifiedIdeograph5b67': '\xe5\xad\xa7',
'CjkUnifiedIdeograph5b66': '\xe5\xad\xa6',
'CjkUnifiedIdeograph5b61': '\xe5\xad\xa1',
'CjkUnifiedIdeograph5b60': '\xe5\xad\xa0',
'CjkUnifiedIdeograph5b63': '\xe5\xad\xa3',
'CjkUnifiedIdeograph5b62': '\xe5\xad\xa2',
'MalayalamSignAnusvara': '\xe0\xb4\x82',
'CjkUnifiedIdeograph5b69': '\xe5\xad\xa9',
'CjkUnifiedIdeograph5b68': '\xe5\xad\xa8',
'UnicodeCharacter0xe515': '\xee\x94\x95',
'ArmenianAbbreviationMark': '\xd5\x9f',
'TeluguDigitNine': '\xe0\xb1\xaf',
'HangulSyllableHyung': '\xed\x9d\x89',
'LatinSmallLetterOWithDoubleGrave': '\xc8\x8d',
'UnicodeCharacter0xaacf': '\xea\xab\x8f',
'CjkUnifiedIdeograph4d0d': '\xe4\xb4\x8d',
'CjkUnifiedIdeograph4d0e': '\xe4\xb4\x8e',
'CjkUnifiedIdeograph4d0b': '\xe4\xb4\x8b',
'CjkUnifiedIdeograph4d0c': '\xe4\xb4\x8c',
'UnicodeCharacter0xaaca': '\xea\xab\x8a',
'CjkUnifiedIdeograph4d0a': '\xe4\xb4\x8a',
'UnicodeCharacter0xf16e': '\xef\x85\xae',
'DevanagariAcuteAccent': '\xe0\xa5\x94',
'GlagoliticSmallLetterAzu': '\xe2\xb0\xb0',
'UnicodeCharacter0xf16f': '\xef\x85\xaf',
'UnicodeCharacter0xf16a': '\xef\x85\xaa',
'UnicodeCharacter0xf16c': '\xef\x85\xac',
'UnicodeCharacter0xf16b': '\xef\x85\xab',
'CjkUnifiedIdeograph3eae': '\xe3\xba\xae',
'CjkUnifiedIdeograph3ead': '\xe3\xba\xad',
'CjkUnifiedIdeograph3eaf': '\xe3\xba\xaf',
'CjkUnifiedIdeograph3eaa': '\xe3\xba\xaa',
'UnicodeCharacter0xe6e8': '\xee\x9b\xa8',
'CjkUnifiedIdeograph3eac': '\xe3\xba\xac',
'CjkUnifiedIdeograph3eab': '\xe3\xba\xab',
'HangulSyllableGiss': '\xea\xb9\x84',
'UnicodeCharacter0xe6e9': '\xee\x9b\xa9',
'HangulSyllableCwabs': '\xec\xb4\xb6',
'CjkUnifiedIdeograph8936': '\xe8\xa4\xb6',
'CjkUnifiedIdeograph4327': '\xe4\x8c\xa7',
'CjkUnifiedIdeograph8934': '\xe8\xa4\xb4',
'CjkUnifiedIdeograph8935': '\xe8\xa4\xb5',
'CjkUnifiedIdeograph8932': '\xe8\xa4\xb2',
'CjkUnifiedIdeograph8933': '\xe8\xa4\xb3',
'CjkUnifiedIdeograph8930': '\xe8\xa4\xb0',
'CjkUnifiedIdeograph4326': '\xe4\x8c\xa6',
'YiSyllableSap': '\xea\x8c\x93',
'CjkUnifiedIdeograph4325': '\xe4\x8c\xa5',
'KatakanaLetterSmallSu': '\xe3\x87\xb2',
'CjkUnifiedIdeograph8939': '\xe8\xa4\xb9',
'CjkUnifiedIdeograph51b6': '\xe5\x86\xb6',
'CjkUnifiedIdeograph51b7': '\xe5\x86\xb7',
'CjkUnifiedIdeograph51b4': '\xe5\x86\xb4',
'UnicodeCharacter0x20c4': '\xe2\x83\x84',
'CjkUnifiedIdeograph51b2': '\xe5\x86\xb2',
'CjkUnifiedIdeograph51b3': '\xe5\x86\xb3',
'CjkUnifiedIdeograph51b0': '\xe5\x86\xb0',
'CjkUnifiedIdeograph51b1': '\xe5\x86\xb1',
'GreekSmallLetterUpsilonWithDasiaAndOxia': '\xe1\xbd\x95',
'UnicodeCharacter0x20c3': '\xe2\x83\x83',
'UnicodeCharacter0xe6e0': '\xee\x9b\xa0',
'CjkUnifiedIdeograph51b8': '\xe5\x86\xb8',
'CjkUnifiedIdeograph51b9': '\xe5\x86\xb9',
'UnicodeCharacter0xe6e1': '\xee\x9b\xa1',
'CjkUnifiedIdeograph4321': '\xe4\x8c\xa1',
'YiSyllableShur': '\xea\x8e\xbf',
'UnicodeCharacter0x20c0': '\xe2\x83\x80',
'UnicodeCharacter0xf0e2': '\xef\x83\xa2',
'CjkUnifiedIdeograph3ea9': '\xe3\xba\xa9',
'CjkUnifiedIdeograph3ea8': '\xe3\xba\xa8',
'CjkUnifiedIdeograph3ea5': '\xe3\xba\xa5',
'CjkUnifiedIdeograph3ea4': '\xe3\xba\xa4',
'CjkUnifiedIdeograph3ea7': '\xe3\xba\xa7',
'CjkUnifiedIdeograph3ea6': '\xe3\xba\xa6',
'CjkUnifiedIdeograph3ea1': '\xe3\xba\xa1',
'CjkUnifiedIdeograph3ea0': '\xe3\xba\xa0',
'CjkUnifiedIdeograph3ea3': '\xe3\xba\xa3',
'CjkUnifiedIdeograph3ea2': '\xe3\xba\xa2',
'CjkUnifiedIdeograph51bf': '\xe5\x86\xbf',
'SuperscriptLatinSmallLetterI': '\xe2\x81\xb1',
'CjkUnifiedIdeograph51bd': '\xe5\x86\xbd',
'CjkUnifiedIdeograph51be': '\xe5\x86\xbe',
'CjkUnifiedIdeograph51bb': '\xe5\x86\xbb',
'CjkUnifiedIdeograph51bc': '\xe5\x86\xbc',
'CjkUnifiedIdeograph51ba': '\xe5\x86\xba',
'TibetanLetterTtha': '\xe0\xbd\x8b',
'CjkUnifiedIdeograph893f': '\xe8\xa4\xbf',
'UnicodeCharacter0xf02d': '\xef\x80\xad',
'CjkUnifiedIdeograph893d': '\xe8\xa4\xbd',
'CjkUnifiedIdeograph893e': '\xe8\xa4\xbe',
'CjkUnifiedIdeograph893b': '\xe8\xa4\xbb',
'CjkUnifiedIdeograph4329': '\xe4\x8c\xa9',
'CjkUnifiedIdeograph893a': '\xe8\xa4\xba',
'UnicodeCharacter0xf0d6': '\xef\x83\x96',
'CjkUnifiedIdeograph4328': '\xe4\x8c\xa8',
'UnicodeCharacter0xf02f': '\xef\x80\xaf',
'VaiSyllableHu': '\xea\x96\x97',
'CanadianSyllabicsCarrierGho': '\xe1\x97\x85',
'CjkUnifiedIdeograph4139': '\xe4\x84\xb9',
'CanadianSyllabicsCarrierGhi': '\xe1\x97\x88',
'UnicodeCharacter0xf1f4': '\xef\x87\xb4',
'CanadianSyllabicsCarrierGhe': '\xe1\x97\x86',
'CjkUnifiedIdeograph4133': '\xe4\x84\xb3',
'CjkUnifiedIdeograph4130': '\xe4\x84\xb0',
'CjkUnifiedIdeograph4131': '\xe4\x84\xb1',
'CjkUnifiedIdeograph4136': '\xe4\x84\xb6',
'CjkUnifiedIdeograph4137': '\xe4\x84\xb7',
'CjkUnifiedIdeograph4134': '\xe4\x84\xb4',
'CjkUnifiedIdeograph4135': '\xe4\x84\xb5',
'CjkUnifiedIdeograph7dcf': '\xe7\xb7\x8f',
'UnicodeCharacter0xe53f': '\xee\x94\xbf',
'ArabicStartOfRubElHizb': '\xdb\x9e',
'CjkUnifiedIdeograph7dce': '\xe7\xb7\x8e',
'CjkUnifiedIdeograph7dcb': '\xe7\xb7\x8b',
'CjkUnifiedIdeograph7dcc': '\xe7\xb7\x8c',
'UnicodeCharacter0xe53a': '\xee\x94\xba',
'CjkUnifiedIdeograph7dca': '\xe7\xb7\x8a',
'CanadianSyllabicsCarrierGhu': '\xe1\x97\x84',
'CjkUnifiedIdeograph5cfa': '\xe5\xb3\xba',
'CjkUnifiedIdeograph5cfb': '\xe5\xb3\xbb',
'CjkUnifiedIdeograph5cfc': '\xe5\xb3\xbc',
'CjkUnifiedIdeograph5cfd': '\xe5\xb3\xbd',
'CjkUnifiedIdeograph5cfe': '\xe5\xb3\xbe',
'CjkUnifiedIdeograph5cff': '\xe5\xb3\xbf',
'EthiopicSyllableCci': '\xe2\xb6\xaa',
'UnicodeCharacter0xe742': '\xee\x9d\x82',
'CyrillicSmallLetterShortI': '\xd0\xb9',
'EthiopicSyllableCco': '\xe2\xb6\xae',
'EthiopicSyllableCca': '\xe2\xb6\xa8',
'UnicodeCharacter0xf7cb': '\xef\x9f\x8b',
'UnicodeCharacter0xf7cc': '\xef\x9f\x8c',
'UnicodeCharacter0xf7cd': '\xef\x9f\x8d',
'EthiopicSyllableCce': '\xe2\xb6\xad',
'UnicodeCharacter0xf7cf': '\xef\x9f\x8f',
'ArabicLigatureYehWithHamzaAboveWithKhahInitialForm': '\xef\xb2\x99',
'GujaratiDigitNine': '\xe0\xab\xaf',
'CjkUnifiedIdeograph6fc9': '\xe6\xbf\x89',
'LaoToneMaiTho': '\xe0\xbb\x89',
'CjkUnifiedIdeograph6fc7': '\xe6\xbf\x87',
'CjkUnifiedIdeograph57a9': '\xe5\x9e\xa9',
'CyrillicSmallLetterShortU': '\xd1\x9e',
'CjkUnifiedIdeograph6fc4': '\xe6\xbf\x84',
'CjkUnifiedIdeograph6fc3': '\xe6\xbf\x83',
'EthiopicSyllableCcu': '\xe2\xb6\xa9',
'CjkUnifiedIdeograph6fc1': '\xe6\xbf\x81',
'CjkUnifiedIdeograph6fc0': '\xe6\xbf\x80',
'ObliqueAngleOpeningUp': '\xe2\xa6\xa6',
'CjkUnifiedIdeograph5cf9': '\xe5\xb3\xb9',
'CjkUnifiedIdeograph7dc4': '\xe7\xb7\x84',
'RightTriangle': '\xe2\x8a\xbf',
'CjkUnifiedIdeograph719d': '\xe7\x86\x9d',
'CjkUnifiedIdeograph7dc3': '\xe7\xb7\x83',
'CjkUnifiedIdeograph7dc0': '\xe7\xb7\x80',
'CjkUnifiedIdeograph4410': '\xe4\x90\x90',
'CjkUnifiedIdeograph5cf0': '\xe5\xb3\xb0',
'CjkUnifiedIdeograph5cf1': '\xe5\xb3\xb1',
'CjkUnifiedIdeograph5cf2': '\xe5\xb3\xb2',
'CjkUnifiedIdeograph5cf3': '\xe5\xb3\xb3',
'CjkUnifiedIdeograph5cf4': '\xe5\xb3\xb4',
'BlackSmilingFace': '\xe2\x98\xbb',
'CjkUnifiedIdeograph5cf6': '\xe5\xb3\xb6',
'CjkUnifiedIdeograph5cf7': '\xe5\xb3\xb7',
'HangulSyllableDyaelt': '\xeb\x8d\x85',
'SquaredMinus': '\xe2\x8a\x9f',
'CjkUnifiedIdeograph413b': '\xe4\x84\xbb',
'CjkUnifiedIdeograph413c': '\xe4\x84\xbc',
'CjkUnifiedIdeograph413a': '\xe4\x84\xba',
'CjkUnifiedIdeograph413f': '\xe4\x84\xbf',
'NewTaiLueLetterLowQa': '\xe1\xa6\x81',
'CjkUnifiedIdeograph4414': '\xe4\x90\x94',
'HangulSyllableJyess': '\xec\xa1\xa8',
'GreekSmallLetterMu': '\xce\xbc',
'CjkCompatibilityIdeographf9fa': '\xef\xa7\xba',
'ParenthesizedHangulChieuchA': '\xe3\x88\x97',
'CjkUnifiedIdeograph57a1': '\xe5\x9e\xa1',
'CjkUnifiedIdeograph6fce': '\xe6\xbf\x8e',
'CjkUnifiedIdeograph6fcd': '\xe6\xbf\x8d',
'CjkUnifiedIdeograph6fcc': '\xe6\xbf\x8c',
'ArabicLetterWawWithTwoDotsAbove': '\xdb\x8a',
'CjkUnifiedIdeograph6fca': '\xe6\xbf\x8a',
'HangulSyllableDyeong': '\xeb\x8e\xa1',
'CjkUnifiedIdeograph382a': '\xe3\xa0\xaa',
'HangulJungseongEueu': '\xe1\x86\x96',
'UnicodeCharacter0xec3d': '\xee\xb0\xbd',
'UnicodeCharacter0xf7c0': '\xef\x9f\x80',
'UnicodeCharacter0xf7c1': '\xef\x9f\x81',
'UnicodeCharacter0xf7c2': '\xef\x9f\x82',
'UnicodeCharacter0xf7c3': '\xef\x9f\x83',
'UnicodeCharacter0xf7c4': '\xef\x9f\x84',
'HangulChoseongNieun': '\xe1\x84\x82',
'UnicodeCharacter0xf7c6': '\xef\x9f\x86',
'UnicodeCharacter0xf7c7': '\xef\x9f\x87',
'CjkUnifiedIdeograph9719': '\xe9\x9c\x99',
'StarEquals': '\xe2\x89\x9b',
'HangulSyllableYanj': '\xec\x96\x81',
'ParallelTo': '\xe2\x88\xa5',
'GujaratiSignVisarga': '\xe0\xaa\x83',
'CjkUnifiedIdeograph9711': '\xe9\x9c\x91',
'CjkUnifiedIdeograph9710': '\xe9\x9c\x90',
'CjkUnifiedIdeograph9713': '\xe9\x9c\x93',
'CjkUnifiedIdeograph9712': '\xe9\x9c\x92',
'CjkUnifiedIdeograph9715': '\xe9\x9c\x95',
'CjkUnifiedIdeograph9714': '\xe9\x9c\x94',
'CjkUnifiedIdeograph9717': '\xe9\x9c\x97',
'CjkUnifiedIdeograph9716': '\xe9\x9c\x96',
'UnicodeCharacter0xe0e3': '\xee\x83\xa3',
'GlagoliticSmallLetterTvrido': '\xe2\xb1\x85',
'UnicodeCharacter0xec3c': '\xee\xb0\xbc',
'UnicodeCharacter0xf0cc': '\xef\x83\x8c',
'UnicodeCharacter0xf0cb': '\xef\x83\x8b',
'UnicodeCharacter0xf0ca': '\xef\x83\x8a',
'UnicodeCharacter0xe746': '\xee\x9d\x86',
'UnicodeCharacter0xf0cf': '\xef\x83\x8f',
'UnicodeCharacter0xf0ce': '\xef\x83\x8e',
'UnicodeCharacter0xf0cd': '\xef\x83\x8d',
'HangulSyllableKwaen': '\xec\xbe\x90',
'CjkUnifiedIdeograph4c9f': '\xe4\xb2\x9f',
'HangulSyllableKwael': '\xec\xbe\x94',
'HangulSyllableKwaem': '\xec\xbe\x9c',
'HangulSyllableKwaej': '\xec\xbe\xa2',
'HangulSyllableKwaek': '\xec\xbe\xa4',
'MyanmarVowelSignVocalicL': '\xe1\x81\x98',
'HangulSyllableTaes': '\xed\x83\xaf',
'HangulSyllableKwaeg': '\xec\xbe\x8d',
'HangulSyllableKwaed': '\xec\xbe\x93',
'HangulSyllableKwaeb': '\xec\xbe\x9d',
'HangulSyllableKwaec': '\xec\xbe\xa3',
'CanadianSyllabicsFaa': '\xe1\x95\x9a',
'CjkUnifiedIdeograph5669': '\xe5\x99\xa9',
'CjkUnifiedIdeograph5668': '\xe5\x99\xa8',
'CjkUnifiedIdeograph4c9e': '\xe4\xb2\x9e',
'HangulSyllableNyess': '\xeb\x85\xb0',
'MyanmarVowelSignVocalicR': '\xe1\x81\x96',
'CjkUnifiedIdeograph44df': '\xe4\x93\x9f',
'CjkUnifiedIdeograph5663': '\xe5\x99\xa3',
'CjkUnifiedIdeograph5662': '\xe5\x99\xa2',
'CjkUnifiedIdeograph5665': '\xe5\x99\xa5',
'CjkUnifiedIdeograph5664': '\xe5\x99\xa4',
'CjkUnifiedIdeograph5667': '\xe5\x99\xa7',
'CjkUnifiedIdeograph5666': '\xe5\x99\xa6',
'UnicodeCharacter0xf0c9': '\xef\x83\x89',
'UnicodeCharacter0xf0c8': '\xef\x83\x88',
'CjkUnifiedIdeograph44dd': '\xe4\x93\x9d',
'YiSyllableNix': '\xea\x85\xbe',
'CjkUnifiedIdeograph432f': '\xe4\x8c\xaf',
'UnicodeCharacter0xf0c3': '\xef\x83\x83',
'LaoHoNo': '\xe0\xbb\x9c',
'KannadaAiLengthMark': '\xe0\xb3\x96',
'CjkUnifiedIdeograph4c9a': '\xe4\xb2\x9a',
'UnicodeCharacter0xf0c7': '\xef\x83\x87',
'UnicodeCharacter0x20ce': '\xe2\x83\x8e',
'YiSyllableNip': '\xea\x86\x80',
'UnicodeCharacter0xf0c4': '\xef\x83\x84',
'ArabicLigatureBehWithRehFinalForm': '\xef\xb1\xaa',
'CjkUnifiedIdeograph432d': '\xe4\x8c\xad',
'VulgarFractionFiveEighths': '\xe2\x85\x9d',
'CjkUnifiedIdeograph971a': '\xe9\x9c\x9a',
'UnicodeCharacter0x20cc': '\xe2\x83\x8c',
'CjkUnifiedIdeograph971c': '\xe9\x9c\x9c',
'CjkUnifiedIdeograph4971': '\xe4\xa5\xb1',
'CjkUnifiedIdeograph971e': '\xe9\x9c\x9e',
'CjkUnifiedIdeograph971d': '\xe9\x9c\x9d',
'UnicodeCharacter0x20cb': '\xe2\x83\x8b',
'UnicodeCharacter0xdac0': '\xed\xab\x80',
'UnicodeCharacter0x20ca': '\xe2\x83\x8a',
'CjkUnifiedIdeograph566a': '\xe5\x99\xaa',
'EthiopicSyllableWu': '\xe1\x8b\x89',
'CjkUnifiedIdeograph566c': '\xe5\x99\xac',
'CjkUnifiedIdeograph566b': '\xe5\x99\xab',
'CjkUnifiedIdeograph566e': '\xe5\x99\xae',
'CjkUnifiedIdeograph566d': '\xe5\x99\xad',
'CjkUnifiedIdeograph566f': '\xe5\x99\xaf',
'EthiopicSyllableWo': '\xe1\x8b\x8e',
'EthiopicSyllableWi': '\xe1\x8b\x8a',
'EthiopicSyllableWe': '\xe1\x8b\x8d',
'EthiopicSyllableWa': '\xe1\x8b\x88',
'CjkUnifiedIdeograph7d9d': '\xe7\xb6\x9d',
'KhmerIndependentVowelRy': '\xe1\x9e\xab',
'UnicodeCharacter0xf034': '\xef\x80\xb4',
'GujaratiLetterLla': '\xe0\xaa\xb3',
'HangulSyllableJyubs': '\xec\xa5\xbe',
'GreekDasiaAndVaria': '\xe1\xbf\x9d',
'HangulSyllableBbwaess': '\xeb\xbe\x8c',
'MalayalamLetterYa': '\xe0\xb4\xaf',
'LeftwardsArrowWithTailWithVerticalStroke': '\xe2\xac\xb9',
'UnicodeCharacter0xee4b': '\xee\xb9\x8b',
'UnicodeCharacter0xee4c': '\xee\xb9\x8c',
'UnicodeCharacter0xee4a': '\xee\xb9\x8a',
'UnicodeCharacter0xee4f': '\xee\xb9\x8f',
'EthiopicSyllableFu': '\xe1\x8d\x89',
'UnicodeCharacter0xee4d': '\xee\xb9\x8d',
'UnicodeCharacter0xee4e': '\xee\xb9\x8e',
'CjkUnifiedIdeograph55f9': '\xe5\x97\xb9',
'CjkUnifiedIdeograph3546': '\xe3\x95\x86',
'CjkUnifiedIdeograph3547': '\xe3\x95\x87',
'CjkUnifiedIdeograph3544': '\xe3\x95\x84',
'CjkUnifiedIdeograph3545': '\xe3\x95\x85',
'CjkUnifiedIdeograph3542': '\xe3\x95\x82',
'CjkUnifiedIdeograph3543': '\xe3\x95\x83',
'CjkUnifiedIdeograph3540': '\xe3\x95\x80',
'CjkUnifiedIdeograph3541': '\xe3\x95\x81',
'UnicodeCharacter0xdc35': '\xed\xb0\xb5',
'UnicodeCharacter0xe424': '\xee\x90\xa4',
'CjkUnifiedIdeograph3548': '\xe3\x95\x88',
'CjkUnifiedIdeograph3549': '\xe3\x95\x89',
'HangulSyllableGgwess': '\xeb\x80\x84',
'HangulSyllableJeobs': '\xec\xa0\x92',
'HangulSyllableMoegs': '\xeb\xab\xbf',
'UnicodeCharacter0xe05b': '\xee\x81\x9b',
'RightSquareBracketWithUnderbar': '\xe2\xa6\x8c',
'DingbatNegativeCircledDigitOne': '\xe2\x9d\xb6',
'CyrillicCapitalLetterIeWithBreve': '\xd3\x96',
'CjkUnifiedIdeograph354f': '\xe3\x95\x8f',
'CjkUnifiedIdeograph354d': '\xe3\x95\x8d',
'CjkUnifiedIdeograph354e': '\xe3\x95\x8e',
'CjkUnifiedIdeograph354b': '\xe3\x95\x8b',
'CjkUnifiedIdeograph354c': '\xe3\x95\x8c',
'CjkUnifiedIdeograph354a': '\xe3\x95\x8a',
'UnicodeCharacter0xee42': '\xee\xb9\x82',
'UnicodeCharacter0xee43': '\xee\xb9\x83',
'UnicodeCharacter0xee40': '\xee\xb9\x80',
'UnicodeCharacter0xee41': '\xee\xb9\x81',
'UnicodeCharacter0xee46': '\xee\xb9\x86',
'UnicodeCharacter0xee47': '\xee\xb9\x87',
'UnicodeCharacter0xdac1': '\xed\xab\x81',
'UnicodeCharacter0xee45': '\xee\xb9\x85',
'UnicodeCharacter0xe426': '\xee\x90\xa6',
'HangulSyllableCaebs': '\xec\xb1\x96',
'UnicodeCharacter0xee49': '\xee\xb9\x89',
'LatinCapitalLetterAWithRingBelow': '\xe1\xb8\x80',
'StrictlyEquivalentTo': '\xe2\x89\xa3',
'PhagspaLetterGa': '\xea\xa1\x82',
'HangulSyllableGoss': '\xea\xb3\xb4',
'UnicodeCharacter0xe699': '\xee\x9a\x99',
'CjkUnifiedIdeograph8a7b': '\xe8\xa9\xbb',
'CjkUnifiedIdeograph8a7c': '\xe8\xa9\xbc',
'HangulSyllableByegg': '\xeb\xb3\x9a',
'CjkUnifiedIdeograph8a7a': '\xe8\xa9\xba',
'CjkUnifiedIdeograph8a7f': '\xe8\xa9\xbf',
'HangulSyllableAeg': '\xec\x95\xa1',
'CjkUnifiedIdeograph8a7d': '\xe8\xa9\xbd',
'CjkUnifiedIdeograph8a7e': '\xe8\xa9\xbe',
'HangulSyllableAej': '\xec\x95\xb6',
'GurmukhiLetterDdha': '\xe0\xa8\xa2',
'HangulSyllableAeh': '\xec\x95\xbb',
'HangulSyllableAen': '\xec\x95\xa4',
'CjkUnifiedIdeograph8a1f': '\xe8\xa8\x9f',
'HangulSyllableAel': '\xec\x95\xa8',
'HangulSyllableMyagg': '\xeb\xa8\x82',
'UnicodeCharacter0x19db': '\xe1\xa7\x9b',
'UnicodeCharacter0x19dc': '\xe1\xa7\x9c',
'HangulSyllableAep': '\xec\x95\xba',
'UnicodeCharacter0x19da': '\xe1\xa7\x9a',
'CjkUnifiedIdeograph71bf': '\xe7\x86\xbf',
'UnicodeCharacter0x19dd': '\xe1\xa7\x9d',
'LatinCapitalLetterEWithCircumflex': '\xc3\x8a',
'HangulSyllableMyags': '\xeb\xa8\x83',
'[0]': '\xe2\x82\x80',
'CombiningUpwardsArrowBelow': '\xcd\x8e',
'CjkUnifiedIdeograph4c96': '\xe4\xb2\x96',
'HangulSyllableGgyonh': '\xea\xbe\xa2',
'CjkUnifiedIdeograph6efd': '\xe6\xbb\xbd',
'CjkUnifiedIdeograph44d3': '\xe4\x93\x93',
'varrho': '\xcf\xb1',
'HangulSyllableGgyong': '\xea\xbe\xb1',
'MalayalamLetterChilluLl': '\xe0\xb5\xbe',
'CjkUnifiedIdeograph9f6e': '\xe9\xbd\xae',
'CjkUnifiedIdeograph4c95': '\xe4\xb2\x95',
'SylotiNagriLetterKo': '\xea\xa0\x87',
'CjkUnifiedIdeograph7eb5': '\xe7\xba\xb5',
'CjkUnifiedIdeograph44d6': '\xe4\x93\x96',
'HangulSyllableDdyalt': '\xeb\x95\xb5',
'ChamDigitTwo': '\xea\xa9\x92',
'GujaratiRupeeSign': '\xe0\xab\xb1',
'CjkUnifiedIdeograph4c93': '\xe4\xb2\x93',
'HangulSyllableHwilg': '\xed\x9c\xa1',
'CjkUnifiedIdeograph9f6f': '\xe9\xbd\xaf',
'CjkUnifiedIdeograph4c90': '\xe4\xb2\x90',
'LastQuarterMoon': '\xe2\x98\xbe',
'CjkUnifiedIdeograph9f6a': '\xe9\xbd\xaa',
'CjkUnifiedIdeograph44d5': '\xe4\x93\x95',
'HangulSyllableNyobs': '\xeb\x87\xba',
'HangulSyllableDdyilp': '\xeb\x9d\x92',
'CjkUnifiedIdeograph8a72': '\xe8\xa9\xb2',
'CjkUnifiedIdeograph8a73': '\xe8\xa9\xb3',
'CjkUnifiedIdeograph8a70': '\xe8\xa9\xb0',
'CjkUnifiedIdeograph8a71': '\xe8\xa9\xb1',
'CjkUnifiedIdeograph8a76': '\xe8\xa9\xb6',
'CjkUnifiedIdeograph8a77': '\xe8\xa9\xb7',
'CjkUnifiedIdeograph8a74': '\xe8\xa9\xb4',
'CjkUnifiedIdeograph8a75': '\xe8\xa9\xb5',
'CjkUnifiedIdeograph8a78': '\xe8\xa9\xb8',
'CjkUnifiedIdeograph8a79': '\xe8\xa9\xb9',
'YiSyllableHnuo': '\xea\x85\xb5',
'YiSyllableNdiex': '\xea\x85\x95',
'ArabicLigatureTehWithMeemWithJeemInitialForm': '\xef\xb5\x95',
'CjkUnifiedIdeograph7442': '\xe7\x91\x82',
'HangulSyllableBbing': '\xec\x82\xa5',
'ArabicLigatureYehWithHamzaAboveWithHehMedialForm': '\xef\xb3\xa0',
'UnicodeCharacter0xea76': '\xee\xa9\xb6',
'SyriacLetterFinalSemkath': '\xdc\xa4',
'HangulSyllableSswah': '\xec\x90\x8f',
'CjkUnifiedIdeograph4c99': '\xe4\xb2\x99',
'CjkUnifiedIdeograph3829': '\xe3\xa0\xa9',
'HangulSyllableSswaj': '\xec\x90\x8a',
'HangulSyllableKwi': '\xed\x80\xb4',
'CjkUnifiedIdeograph84e5': '\xe8\x93\xa5',
'HangulSyllableSswak': '\xec\x90\x8c',
'CjkCompatibilityIdeographf9f5': '\xef\xa7\xb5',
'CjkUnifiedIdeograph84e4': '\xe8\x93\xa4',
'ZeroWidthJoiner': '\xe2\x80\x8d',
'CjkCompatibilityIdeographf9f4': '\xef\xa7\xb4',
'HangulSyllableGgweng': '\xeb\x80\x85',
'HangulSyllableJelh': '\xec\xa0\xab',
'HangulSyllableJjags': '\xec\xa7\x9f',
'HangulSyllableGgwenj': '\xea\xbf\xb5',
'HangulSyllableSsyibs': '\xec\x94\x9e',
'SinhalaSignAnusvaraya': '\xe0\xb6\x82',
'UnicodeCharacter0xec38': '\xee\xb0\xb8',
'CjkUnifiedIdeograph901d': '\xe9\x80\x9d',
'FullwidthRightCurlyBracket': '\xef\xbd\x9d',
'HangulSyllableSswag': '\xec\x8f\xb5',
'HangulSyllableDoebs': '\xeb\x90\xaa',
'HangulSyllableDyegg': '\xeb\x8e\xaa',
'UnicodeCharacter0xec35': '\xee\xb0\xb5',
'EthiopicSyllableQyee': '\xe2\xb7\x84',
'HangulSyllableDyegs': '\xeb\x8e\xab',
'HangulSyllableSswac': '\xec\x90\x8b',
'UnicodeCharacter0xec37': '\xee\xb0\xb7',
'HangulChoseongCeongchieumchieuch': '\xe1\x85\x95',
'CjkUnifiedIdeograph3822': '\xe3\xa0\xa2',
'LatinSmallLetterSWithMiddleTilde': '\xe1\xb5\xb4',
'UnicodeCharacter0xec31': '\xee\xb0\xb1',
'UnicodeCharacter0xec30': '\xee\xb0\xb0',
'HangulSyllableDdyals': '\xeb\x95\xb4',
'LatinSmallLetterRWithDotBelow': '\xe1\xb9\x9b',
'UnicodeCharacter0xec33': '\xee\xb0\xb3',
'Airplane': '\xe2\x9c\x88',
'UnicodeCharacter0xe913': '\xee\xa4\x93',
'HangulSyllableMweonj': '\xeb\xad\x95',
'UnicodeCharacter0xdfee': '\xed\xbf\xae',
'UnicodeCharacter0xdfed': '\xed\xbf\xad',
'HangulSyllableMweot': '\xeb\xad\xa9',
'CjkUnifiedIdeograph9144': '\xe9\x85\x84',
'UnicodeCharacter0xdfea': '\xed\xbf\xaa',
'UnicodeCharacter0xdfec': '\xed\xbf\xac',
'UnicodeCharacter0xdfeb': '\xed\xbf\xab',
'UnicodeCharacter0xe33d': '\xee\x8c\xbd',
'HangulSyllableDyaelg': '\xeb\x8d\x81',
'HangulSyllableSswat': '\xec\x90\x8d',
'HangulSyllableJyip': '\xec\xa6\xbe',
'ArabicLetterGafWithTwoDotsBelow': '\xda\xb2',
'CjkUnifiedIdeograph395f': '\xe3\xa5\x9f',
'HangulSyllableDyelp': '\xeb\x8e\xb6',
'HangulSyllableMeolg': '\xeb\xa9\x81',
'HangulSyllableDdwaelh': '\xeb\x99\x97',
'HangulJungseongWae': '\xe1\x85\xab',
'HangulSyllableMeolb': '\xeb\xa9\x83',
'HangulSyllableTaeb': '\xed\x83\xad',
'HangulSyllableSswap': '\xec\x90\x8e',
'HangulSyllableMeolm': '\xeb\xa9\x82',
'HangulSyllableDdwaelb': '\xeb\x99\x93',
'UnicodeCharacter0xea21': '\xee\xa8\xa1',
'LeftBlackLenticularBracket': '\xe3\x80\x90',
'ArabicLetterHahWithSmallArabicLetterTahAbove': '\xdd\xb2',
'HangulSyllableMeolh': '\xeb\xa9\x87',
'UnicodeCharacter0xe505': '\xee\x94\x85',
'HangulSyllableMeolt': '\xeb\xa9\x85',
'HangulSyllableMeols': '\xeb\xa9\x84',
'LongRightTack': '\xe2\x9f\x9d',
'HangulSyllableMeolp': '\xeb\xa9\x86',
'HangulSyllableDdwaelp': '\xeb\x99\x96',
'HangulSyllableDdwaels': '\xeb\x99\x94',
'HangulSyllableSswas': '\xec\x90\x87',
'HangulSyllableDdwaelt': '\xeb\x99\x95',
'UnicodeCharacter0xe972': '\xee\xa5\xb2',
'UnicodeCharacter0xf499': '\xef\x92\x99',
'UnicodeCharacter0xdfe5': '\xed\xbf\xa5',
'UnicodeCharacter0xdfe4': '\xed\xbf\xa4',
'UnicodeCharacter0xdfe7': '\xed\xbf\xa7',
'CjkUnifiedIdeograph8a17': '\xe8\xa8\x97',
'UnicodeCharacter0xdfe1': '\xed\xbf\xa1',
'UnicodeCharacter0xdfe0': '\xed\xbf\xa0',
'ArmenianQuestionMark': '\xd5\x9e',
'UnicodeCharacter0xdfe2': '\xed\xbf\xa2',
'CjkUnifiedIdeograph8a10': '\xe8\xa8\x90',
'UnicodeCharacter0xdfe9': '\xed\xbf\xa9',
'UnicodeCharacter0xdfe8': '\xed\xbf\xa8',
'CjkUnifiedIdeograph8a11': '\xe8\xa8\x91',
'UnicodeCharacter0xf099': '\xef\x82\x99',
'YiSyllableRre': '\xea\x8e\x86',
'VaiSyllableNja': '\xea\x95\xa8',
'YiSyllableRra': '\xea\x8d\xbd',
'UnicodeCharacter0xe213': '\xee\x88\x93',
'YiSyllableRro': '\xea\x8e\x82',
'CanadianSyllabicsFinalPlus': '\xe1\x90\xa9',
'CjkUnifiedIdeograph8a13': '\xe8\xa8\x93',
'UnicodeCharacter0xe210': '\xee\x88\x90',
'MalayalamLetterVocalicR': '\xe0\xb4\x8b',
'CjkUnifiedIdeograph9f65': '\xe9\xbd\xa5',
'YiSyllableRru': '\xea\x8e\x8a',
'CjkUnifiedIdeograph9f64': '\xe9\xbd\xa4',
'UnicodeCharacter0xe978': '\xee\xa5\xb8',
'CjkUnifiedIdeograph9f67': '\xe9\xbd\xa7',
'YiSyllableRry': '\xea\x8e\x90',
'HangulSyllableTaeg': '\xed\x83\x9d',
'CjkUnifiedIdeograph9f66': '\xe9\xbd\xa6',
'UnicodeCharacter0xf5f0': '\xef\x97\xb0',
'CjkUnifiedIdeograph8a18': '\xe8\xa8\x98',
'UnicodeCharacter0xe215': '\xee\x88\x95',
'CjkCompatibilityIdeographf934': '\xef\xa4\xb4',
'CjkUnifiedIdeograph7eba': '\xe7\xba\xba',
'CjkUnifiedIdeograph6ca8': '\xe6\xb2\xa8',
'BlackRightpointingTriangle': '\xe2\x96\xb6',
'UnicodeCharacter0xe05e': '\xee\x81\x9e',
'CjkUnifiedIdeograph9f63': '\xe9\xbd\xa3',
'CjkUnifiedIdeograph6ca0': '\xe6\xb2\xa0',
'CjkUnifiedIdeograph6ca1': '\xe6\xb2\xa1',
'CjkUnifiedIdeograph6ca2': '\xe6\xb2\xa2',
'CjkUnifiedIdeograph6ca3': '\xe6\xb2\xa3',
'CjkUnifiedIdeograph6ca4': '\xe6\xb2\xa4',
'CjkUnifiedIdeograph6ca5': '\xe6\xb2\xa5',
'CjkUnifiedIdeograph6ca6': '\xe6\xb2\xa6',
'KangxiRadicalTile': '\xe2\xbd\xa1',
'UnicodeCharacter0xf092': '\xef\x82\x92',
'HangulSyllableJjas': '\xec\xa7\xaf',
'CjkUnifiedIdeograph624f': '\xe6\x89\x8f',
'CjkCompatibilityIdeographfa1b': '\xef\xa8\x9b',
'CjkUnifiedIdeograph624d': '\xe6\x89\x8d',
'CjkUnifiedIdeograph624e': '\xe6\x89\x8e',
'CjkUnifiedIdeograph624b': '\xe6\x89\x8b',
'CjkUnifiedIdeograph624c': '\xe6\x89\x8c',
'CjkUnifiedIdeograph624a': '\xe6\x89\x8a',
'HangulSyllableCegg': '\xec\xb2\xb6',
'CombiningCyrillicTitlo': '\xd2\x83',
'UnicodeCharacter0xe6bf': '\xee\x9a\xbf',
'ArabicLigatureLamWithMeemWithHahFinalForm': '\xef\xb6\x87',
'UnicodeCharacter0xa9ac': '\xea\xa6\xac',
'UnicodeCharacter0xf5f2': '\xef\x97\xb2',
'CjkUnifiedIdeograph911b': '\xe9\x84\x9b',
'UnicodeCharacter0xdbd3': '\xed\xaf\x93',
'CjkUnifiedIdeograph6caa': '\xe6\xb2\xaa',
'CjkUnifiedIdeograph6cab': '\xe6\xb2\xab',
'CjkUnifiedIdeograph6cac': '\xe6\xb2\xac',
'CjkUnifiedIdeograph6cad': '\xe6\xb2\xad',
'CjkUnifiedIdeograph6cae': '\xe6\xb2\xae',
'CjkUnifiedIdeograph6caf': '\xe6\xb2\xaf',
'SylotiNagriLetterNo': '\xea\xa0\x98',
'KangxiRadicalSpirit': '\xe2\xbd\xb0',
'EmSpace': '\xe2\x80\x83',
'ClosedUnionWithSerifsAndSmashProduct': '\xe2\xa9\x90',
'CjkCompatibilityIdeographf937': '\xef\xa4\xb7',
'HangulSyllableBbegs': '\xeb\xbb\xaf',
'YiSyllableHxi': '\xea\x89\x88',
'YiSyllableHxo': '\xea\x89\x98',
'HangulSyllableJjyanj': '\xec\xa8\x99',
'CjkUnifiedIdeograph6248': '\xe6\x89\x88',
'CjkUnifiedIdeograph6249': '\xe6\x89\x89',
'CjkUnifiedIdeograph6246': '\xe6\x89\x86',
'CjkUnifiedIdeograph6247': '\xe6\x89\x87',
'CjkUnifiedIdeograph6244': '\xe6\x89\x84',
'CjkUnifiedIdeograph6245': '\xe6\x89\x85',
'CjkUnifiedIdeograph6242': '\xe6\x89\x82',
'CjkUnifiedIdeograph6243': '\xe6\x89\x83',
'GeorgianCapitalLetterRae': '\xe1\x82\xb0',
'DevanagariDigitThree': '\xe0\xa5\xa9',
'KangxiRadicalBoat': '\xe2\xbe\x88',
'HangulSyllablePyaenj': '\xed\x8d\xa5',
'HangulSyllablePyaenh': '\xed\x8d\xa6',
'CjkUnifiedIdeograph97ef': '\xe9\x9f\xaf',
'LatinCapitalLetterEWithCircumflexBelow': '\xe1\xb8\x98',
'UnicodeCharacter0xf1b5': '\xef\x86\xb5',
'UnicodeCharacter0xf5f7': '\xef\x97\xb7',
'SmallRomanNumeralFiveHundred': '\xe2\x85\xbe',
'UnicodeCharacter0xf1b4': '\xef\x86\xb4',
'CjkCompatibilityIdeographf931': '\xef\xa4\xb1',
'CjkUnifiedIdeograph4cfc': '\xe4\xb3\xbc',
'CjkUnifiedIdeograph4cfb': '\xe4\xb3\xbb',
'CjkUnifiedIdeograph4cfa': '\xe4\xb3\xba',
'CjkUnifiedIdeograph4cff': '\xe4\xb3\xbf',
'CjkUnifiedIdeograph4cfe': '\xe4\xb3\xbe',
'CjkUnifiedIdeograph4cfd': '\xe4\xb3\xbd',
'YiSyllableGuox': '\xea\x87\xa7',
'YiSyllableShap': '\xea\x8e\xae',
'YiSyllableShat': '\xea\x8e\xab',
'PhagspaLetterSmallA': '\xea\xa1\x96',
'HangulSyllableNyenh': '\xeb\x85\xa2',
'YiSyllableGuop': '\xea\x87\xa9',
'YiSyllableShax': '\xea\x8e\xac',
'YiSyllableGuot': '\xea\x87\xa6',
'UnicodeCharacter0xecd9': '\xee\xb3\x99',
'HangulSyllableGwaeb': '\xea\xb4\xa9',
'HangulSyllableGwaec': '\xea\xb4\xaf',
'HangulSyllableGwaeg': '\xea\xb4\x99',
'HangulSyllableGwaed': '\xea\xb4\x9f',
'UnicodeCharacter0xe97f': '\xee\xa5\xbf',
'HangulSyllableGwaej': '\xea\xb4\xae',
'CjkUnifiedIdeograph96be': '\xe9\x9a\xbe',
'HangulSyllableGwaeh': '\xea\xb4\xb3',
'HangulSyllableGwaen': '\xea\xb4\x9c',
'UnicodeCharacter0xe97e': '\xee\xa5\xbe',
'HangulSyllableGwael': '\xea\xb4\xa0',
'HangulChoseongMieumpieup': '\xe1\x84\x9c',
'HangulSyllableGwaes': '\xea\xb4\xab',
'HangulSyllableGwaep': '\xea\xb4\xb2',
'UnicodeCharacter0xe97d': '\xee\xa5\xbd',
'RunicLetterIor': '\xe1\x9b\xa1',
'YiSyllableDiex': '\xea\x84\x84',
'UnicodeCharacter0xea82': '\xee\xaa\x82',
'UnicodeCharacter0xe97c': '\xee\xa5\xbc',
'UnicodeCharacter0xe703': '\xee\x9c\x83',
'ArabicLigatureTehWithHehInitialForm': '\xef\xb2\xa5',
'UnicodeCharacter0xffde': '\xef\xbf\x9e',
'UnicodeCharacter0xecd7': '\xee\xb3\x97',
'LeftVerticalBoxLine': '\xe2\x8e\xb8',
'HangulSyllableJjung': '\xec\xad\x9d',
'HangulSyllableJjunh': '\xec\xad\x8e',
'HangulSyllableJjunj': '\xec\xad\x8d',
'VaiSyllableFee': '\xea\x94\x8c',
'CjkUnifiedIdeograph4cf9': '\xe4\xb3\xb9',
'CjkUnifiedIdeograph4cf8': '\xe4\xb3\xb8',
'MyanmarDigitEight': '\xe1\x81\x88',
'CjkUnifiedIdeograph4cf3': '\xe4\xb3\xb3',
'CjkUnifiedIdeograph4cf2': '\xe4\xb3\xb2',
'CjkUnifiedIdeograph4cf1': '\xe4\xb3\xb1',
'CjkUnifiedIdeograph4cf0': '\xe4\xb3\xb0',
'CjkUnifiedIdeograph4cf7': '\xe4\xb3\xb7',
'CjkUnifiedIdeograph4cf6': '\xe4\xb3\xb6',
'CjkUnifiedIdeograph4cf5': '\xe4\xb3\xb5',
'CjkUnifiedIdeograph4cf4': '\xe4\xb3\xb4',
'LatinSmallLetterAWithBreveAndDotBelow': '\xe1\xba\xb7',
'UnicodeCharacter0xe21b': '\xee\x88\x9b',
'CjkUnifiedIdeograph6c75': '\xe6\xb1\xb5',
'UnicodeCharacter0xe21c': '\xee\x88\x9c',
'UnicodeCharacter0xffd1': '\xef\xbf\x91',
'UnicodeCharacter0xe21a': '\xee\x88\x9a',
'IdeographicTelegraphSymbolForHourTwenty': '\xe3\x8d\xac',
'CjkCompatibilityIdeographfa18': '\xef\xa8\x98',
'YiSyllableIp': '\xea\x80\x83',
'UnicodeCharacter0xe21e': '\xee\x88\x9e',
'HangulSyllableTaem': '\xed\x83\xac',
'TibetanSignRjesSuNgaRo': '\xe0\xbd\xbe',
'UnicodeCharacter0xdb1c': '\xed\xac\x9c',
'CjkUnifiedIdeograph78f9': '\xe7\xa3\xb9',
'CjkUnifiedIdeograph78f8': '\xe7\xa3\xb8',
'CyrillicCapitalLetterReversedDze': '\xea\x99\x84',
'CjkUnifiedIdeograph78f1': '\xe7\xa3\xb1',
'CjkUnifiedIdeograph78f0': '\xe7\xa3\xb0',
'CjkUnifiedIdeograph78f3': '\xe7\xa3\xb3',
'CjkUnifiedIdeograph78f2': '\xe7\xa3\xb2',
'CjkUnifiedIdeograph78f5': '\xe7\xa3\xb5',
'CjkUnifiedIdeograph78f4': '\xe7\xa3\xb4',
'CombiningAcuteAccentBelow': '\xcc\x97',
'CjkUnifiedIdeograph78f6': '\xe7\xa3\xb6',
'UnicodeCharacter0xdbcb': '\xed\xaf\x8b',
'WhiteVerySmallSquare': '\xe2\xac\x9e',
'CjkCompatibilityIdeographfa15': '\xef\xa8\x95',
'HangulSyllableNed': '\xeb\x84\xab',
'BengaliLetterVocalicR': '\xe0\xa6\x8b',
'ZNotationRightImageBracket': '\xe2\xa6\x88',
'OcrHook': '\xe2\x91\x80',
'HangulSyllableNeg': '\xeb\x84\xa5',
'CjkCompatibilityIdeographfa13': '\xef\xa8\x93',
'CjkUnifiedIdeograph8af6': '\xe8\xab\xb6',
'SinhalaLetterAlpapraanaTtayanna': '\xe0\xb6\xa7',
'CjkCompatibilityIdeographfaa0': '\xef\xaa\xa0',
'BengaliLetterVocalicL': '\xe0\xa6\x8c',
'UnicodeCharacter0xf09f': '\xef\x82\x9f',
'UnicodeCharacter0xdbca': '\xed\xaf\x8a',
'HangulSyllableBeubs': '\xeb\xb8\x9e',
'ThaanaLetterTaviyani': '\xde\x93',
'HangulSyllableNec': '\xeb\x84\xbb',
'CjkUnifiedIdeograph78fa': '\xe7\xa3\xba',
'CjkUnifiedIdeograph78fc': '\xe7\xa3\xbc',
'CjkUnifiedIdeograph78fb': '\xe7\xa3\xbb',
'CjkUnifiedIdeograph78fe': '\xe7\xa3\xbe',
'CjkUnifiedIdeograph78fd': '\xe7\xa3\xbd',
'CjkUnifiedIdeograph78ff': '\xe7\xa3\xbf',
'LatinCapitalLetterDWithSmallLetterZWithCaron': '\xc7\x85',
'CircledNumberNineteen': '\xe2\x91\xb2',
'UnicodeCharacter0xaa6d': '\xea\xa9\xad',
'CjkUnifiedIdeograph9247': '\xe9\x89\x87',
'CjkUnifiedIdeograph64e9': '\xe6\x93\xa9',
'CjkUnifiedIdeograph64e8': '\xe6\x93\xa8',
'HangulSyllableCwad': '\xec\xb4\xab',
'HangulSyllableCwae': '\xec\xb5\x80',
'CjkUnifiedIdeograph9246': '\xe9\x89\x86',
'CjkUnifiedIdeograph64e3': '\xe6\x93\xa3',
'CjkUnifiedIdeograph64e2': '\xe6\x93\xa2',
'CjkUnifiedIdeograph64e1': '\xe6\x93\xa1',
'BalineseLetterKhotSasak': '\xe1\xad\x86',
'CjkUnifiedIdeograph64e7': '\xe6\x93\xa7',
'CjkUnifiedIdeograph64e6': '\xe6\x93\xa6',
'CjkUnifiedIdeograph64e5': '\xe6\x93\xa5',
'CjkUnifiedIdeograph64e4': '\xe6\x93\xa4',
'HangulSyllableCwap': '\xec\xb4\xbe',
'CjkUnifiedIdeograph9244': '\xe9\x89\x84',
'HangulSyllableCwat': '\xec\xb4\xbd',
'CjkUnifiedIdeograph9243': '\xe9\x89\x83',
'UnicodeCharacter0xaa6e': '\xea\xa9\xae',
'CjkUnifiedIdeograph9242': '\xe9\x89\x82',
'CjkUnifiedIdeograph86ef': '\xe8\x9b\xaf',
'CjkUnifiedIdeograph86ee': '\xe8\x9b\xae',
'CjkUnifiedIdeograph86ed': '\xe8\x9b\xad',
'CjkUnifiedIdeograph86ec': '\xe8\x9b\xac',
'ThaiCharacterSoRusi': '\xe0\xb8\xa9',
'CjkUnifiedIdeograph86ea': '\xe8\x9b\xaa',
'CjkUnifiedIdeograph6a3d': '\xe6\xa8\xbd',
'CjkUnifiedIdeograph6a3e': '\xe6\xa8\xbe',
'CjkUnifiedIdeograph6a3f': '\xe6\xa8\xbf',
'CjkUnifiedIdeograph9240': '\xe9\x89\x80',
'CjkUnifiedIdeograph6a3a': '\xe6\xa8\xba',
'CjkUnifiedIdeograph6a3b': '\xe6\xa8\xbb',
'CjkUnifiedIdeograph6a3c': '\xe6\xa8\xbc',
'ModifierLetterLowDottedLeftstemToneBar': '\xea\x9c\x90',
'ThaiCharacterToTao': '\xe0\xb8\x95',
'CircledHangulThieuthA': '\xe3\x89\xb9',
'HangulSyllableSsyanj': '\xec\x8c\xb5',
'GreekCapitalLetterPamphylianDigamma': '\xcd\xb6',
'HangulSyllableNep': '\xeb\x84\xbe',
'HangulSyllableSebs': '\xec\x85\x8a',
'UnicodeCharacter0xfb45': '\xef\xad\x85',
'MathematicalLeftAngleBracket': '\xe2\x9f\xa8',
'HebrewLetterBetWithRafe': '\xef\xad\x8c',
'CjkUnifiedIdeograph8072': '\xe8\x81\xb2',
'CjkUnifiedIdeograph64ec': '\xe6\x93\xac',
'CjkUnifiedIdeograph64eb': '\xe6\x93\xab',
'CjkUnifiedIdeograph64ea': '\xe6\x93\xaa',
'CjkUnifiedIdeograph9248': '\xe9\x89\x88',
'CjkUnifiedIdeograph64ef': '\xe6\x93\xaf',
'CjkUnifiedIdeograph64ee': '\xe6\x93\xae',
'CjkUnifiedIdeograph64ed': '\xe6\x93\xad',
'ModifierLetterYinDepartingToneMark': '\xcb\xaa',
'HangulSyllablePyij': '\xed\x94\xb6',
'ArabicLigatureGhainWithAlefMaksuraIsolatedForm': '\xef\xb3\xb9',
'CjkUnifiedIdeograph8076': '\xe8\x81\xb6',
'CjkUnifiedIdeograph86e7': '\xe8\x9b\xa7',
'CjkUnifiedIdeograph86e6': '\xe8\x9b\xa6',
'CjkUnifiedIdeograph86e5': '\xe8\x9b\xa5',
'CjkUnifiedIdeograph86e4': '\xe8\x9b\xa4',
'CjkUnifiedIdeograph6a38': '\xe6\xa8\xb8',
'NarySquareUnionOperator': '\xe2\xa8\x86',
'CjkUnifiedIdeograph86e1': '\xe8\x9b\xa1',
'CjkUnifiedIdeograph86e0': '\xe8\x9b\xa0',
'CjkUnifiedIdeograph6a34': '\xe6\xa8\xb4',
'CjkUnifiedIdeograph6a35': '\xe6\xa8\xb5',
'CjkUnifiedIdeograph6a36': '\xe6\xa8\xb6',
'CjkUnifiedIdeograph6a37': '\xe6\xa8\xb7',
'CjkUnifiedIdeograph6a30': '\xe6\xa8\xb0',
'CjkUnifiedIdeograph6a31': '\xe6\xa8\xb1',
'CjkUnifiedIdeograph6a32': '\xe6\xa8\xb2',
'CjkUnifiedIdeograph68e2': '\xe6\xa3\xa2',
'HebrewLetterKaf': '\xd7\x9b',
'DevanagariLetterVa': '\xe0\xa4\xb5',
'ArabicEmptyCentreHighStop': '\xdb\xab',
'UnicodeCharacter0xeb1a': '\xee\xac\x9a',
'GreekSmallLetterEtaWithPsiliAndYpogegrammeni': '\xe1\xbe\x90',
'BottomCurlyBracket': '\xe2\x8f\x9f',
'SquareMuS': '\xe3\x8e\xb2',
'SquareMuV': '\xe3\x8e\xb6',
'SmallRomanNumeralFour': '\xe2\x85\xb3',
'UnicodeCharacter0xd97d': '\xed\xa5\xbd',
'UnicodeCharacter0xd97e': '\xed\xa5\xbe',
'UnicodeCharacter0xd97f': '\xed\xa5\xbf',
'UnicodeCharacter0xd97a': '\xed\xa5\xba',
'UnicodeCharacter0xd97b': '\xed\xa5\xbb',
'UnicodeCharacter0xd97c': '\xed\xa5\xbc',
'SquareMuA': '\xe3\x8e\x82',
'SquareMuF': '\xe3\x8e\x8c',
'SquareMuG': '\xe3\x8e\x8d',
'HangulSyllableDuss': '\xeb\x91\xa4',
'ArmenianCapitalLetterGhad': '\xd5\x82',
'SquareMuM': '\xe3\x8e\x9b',
'LatinCapitalLetterUWithDiaeresisAndGrave': '\xc7\x9b',
'UnicodeCharacter0xdccd': '\xed\xb3\x8d',
'HangulSyllableSoess': '\xec\x87\xb4',
'M': '\xce\x9c',
'UnicodeCharacter0xaa6c': '\xea\xa9\xac',
'GreekSmallLetterOmegaWithVariaAndYpogegrammeni': '\xe1\xbf\xb2',
'HangulSyllableSwan': '\xec\x86\xac',
'ArabicLetterTchehFinalForm': '\xef\xad\xbb',
'GeorgianCapitalLetterEn': '\xe1\x82\xa4',
'ArmenianSmallLetterVo': '\xd5\xb8',
'HangzhouNumeralTen': '\xe3\x80\xb8',
'UnicodeCharacter0xeb1d': '\xee\xac\x9d',
'UnicodeCharacter0xf1bb': '\xef\x86\xbb',
'UnicodeCharacter0xd978': '\xed\xa5\xb8',
'UnicodeCharacter0xd979': '\xed\xa5\xb9',
'OriyaDigitTwo': '\xe0\xad\xa8',
'UnicodeCharacter0xd975': '\xed\xa5\xb5',
'LatinCapitalLetterYWithDiaeresis': '\xc5\xb8',
'UnicodeCharacter0xd977': '\xed\xa5\xb7',
'UnicodeCharacter0xd970': '\xed\xa5\xb0',
'UnicodeCharacter0xd971': '\xed\xa5\xb1',
'Cloud': '\xe2\x98\x81',
'UnicodeCharacter0xd973': '\xed\xa5\xb3',
'DottedCross': '\xe2\x81\x9c',
'UnicodeCharacter0xf1fe': '\xef\x87\xbe',
'HalfwidthHangulLetterChieuch': '\xef\xbe\xba',
'CjkStrokeSzwg': '\xe3\x87\x89',
'UnicodeCharacter0xe70e': '\xee\x9c\x8e',
'CircledKoreanCharacterJueui': '\xe3\x89\xbd',
'CjkUnifiedIdeograph96b4': '\xe9\x9a\xb4',
'CircledKatakanaRo': '\xe3\x8b\xba',
'CircledKatakanaRi': '\xe3\x8b\xb7',
'CircledKatakanaRe': '\xe3\x8b\xb9',
'HangulSyllableCi': '\xec\xb9\x98',
'CircledKatakanaRa': '\xe3\x8b\xb6',
'PrecedesAboveSinglelineNotEqualTo': '\xe2\xaa\xb1',
'CjkUnifiedIdeograph72ca': '\xe7\x8b\x8a',
'LatinSmallLetterRRotunda': '\xea\x9d\x9b',
'CircledKatakanaRu': '\xe3\x8b\xb8',
'UnicodeCharacter0xdcca': '\xed\xb3\x8a',
'HangulSyllableHyelm': '\xed\x98\xa6',
'HangulSyllableGwal': '\xea\xb4\x84',
'HangulSyllableGwam': '\xea\xb4\x8c',
'HangulSyllableGwan': '\xea\xb4\x80',
'UnicodeCharacter0xe74c': '\xee\x9d\x8c',
'HangulSyllableGwah': '\xea\xb4\x97',
'HangulSyllableGwaj': '\xea\xb4\x92',
'LatinCapitalLetterBWithDotAbove': '\xe1\xb8\x82',
'HangulSyllableGwad': '\xea\xb4\x83',
'HangulSyllableGwae': '\xea\xb4\x98',
'HangulSyllableGwag': '\xea\xb3\xbd',
'HangulSyllableGwab': '\xea\xb4\x8d',
'HangulSyllableGwac': '\xea\xb4\x93',
'HangulSyllableTigs': '\xed\x8b\xb3',
'HangulSyllableGwat': '\xea\xb4\x95',
'HangulSyllableGwap': '\xea\xb4\x96',
'YiSyllableNjit': '\xea\x90\xb1',
'HangulSyllableGwas': '\xea\xb4\x8f',
'CjkUnifiedIdeograph4c01': '\xe4\xb0\x81',
'HangulSyllablePalg': '\xed\x8c\x95',
'HangulSyllableCo': '\xec\xb4\x88',
'TibetanMarkBskurYigMgo': '\xe0\xbc\x89',
'HiraganaLetterPu': '\xe3\x81\xb7',
'CjkUnifiedIdeograph52a4': '\xe5\x8a\xa4',
'CjkUnifiedIdeograph52a5': '\xe5\x8a\xa5',
'HiraganaLetterPo': '\xe3\x81\xbd',
'CjkUnifiedIdeograph52a7': '\xe5\x8a\xa7',
'HiraganaLetterPi': '\xe3\x81\xb4',
'CjkUnifiedIdeograph52a1': '\xe5\x8a\xa1',
'CjkUnifiedIdeograph52a2': '\xe5\x8a\xa2',
'HangulChoseongSiospieup': '\xe1\x84\xb2',
'HiraganaLetterPe': '\xe3\x81\xba',
'HangulSyllableGoelm': '\xea\xb4\xbe',
'CjkUnifiedIdeograph584c': '\xe5\xa1\x8c',
'CjkUnifiedIdeograph584b': '\xe5\xa1\x8b',
'HiraganaLetterPa': '\xe3\x81\xb1',
'Snowflake': '\xe2\x9d\x84',
'UnicodeCharacter0xe319': '\xee\x8c\x99',
'CjkUnifiedIdeograph584f': '\xe5\xa1\x8f',
'CjkUnifiedIdeograph41e8': '\xe4\x87\xa8',
'CjkUnifiedIdeograph41e9': '\xe4\x87\xa9',
'CjkUnifiedIdeograph41e4': '\xe4\x87\xa4',
'LimbuLetterPa': '\xe1\xa4\x90',
'CjkUnifiedIdeograph41e6': '\xe4\x87\xa6',
'CjkUnifiedIdeograph41e7': '\xe4\x87\xa7',
'CjkUnifiedIdeograph41e0': '\xe4\x87\xa0',
'CjkUnifiedIdeograph41e1': '\xe4\x87\xa1',
'CjkUnifiedIdeograph41e2': '\xe4\x87\xa2',
'CjkUnifiedIdeograph41e3': '\xe4\x87\xa3',
'UnicodeCharacter0x1cc1': '\xe1\xb3\x81',
'HangulSyllablePwaeb': '\xed\x90\xb5',
'UnicodeCharacter0xdec8': '\xed\xbb\x88',
'UnicodeCharacter0xdec9': '\xed\xbb\x89',
'UnicodeCharacter0x1cc0': '\xe1\xb3\x80',
'HangulSyllableByeon': '\xeb\xb3\x80',
'UnicodeCharacter0xdec5': '\xed\xbb\x85',
'UnicodeCharacter0xdec6': '\xed\xbb\x86',
'UnicodeCharacter0xdec7': '\xed\xbb\x87',
'UnicodeCharacter0xdec0': '\xed\xbb\x80',
'LatinSmallLetterRum': '\xea\x9d\xb5',
'UnicodeCharacter0xdec2': '\xed\xbb\x82',
'HangulSyllablePwaed': '\xed\x90\xab',
'CjkUnifiedIdeograph52ad': '\xe5\x8a\xad',
'CjkUnifiedIdeograph52ae': '\xe5\x8a\xae',
'CjkUnifiedIdeograph52af': '\xe5\x8a\xaf',
'UnicodeCharacter0xe31d': '\xee\x8c\x9d',
'UnicodeCharacter0xe31c': '\xee\x8c\x9c',
'CjkUnifiedIdeograph52aa': '\xe5\x8a\xaa',
'CjkUnifiedIdeograph52ab': '\xe5\x8a\xab',
'CjkUnifiedIdeograph52ac': '\xe5\x8a\xac',
'CjkUnifiedIdeograph5841': '\xe5\xa1\x81',
'CanadianSyllabicsTii': '\xe1\x91\x8f',
'CjkUnifiedIdeograph5843': '\xe5\xa1\x83',
'CjkUnifiedIdeograph5842': '\xe5\xa1\x82',
'CjkUnifiedIdeograph5845': '\xe5\xa1\x85',
'CjkUnifiedIdeograph5844': '\xe5\xa1\x84',
'CjkUnifiedIdeograph5847': '\xe5\xa1\x87',
'CjkUnifiedIdeograph5846': '\xe5\xa1\x86',
'CjkRadicalMoundOne': '\xe2\xbb\x95',
'HangulSyllableKeong': '\xec\xbb\xb9',
'CjkUnifiedIdeograph67ed': '\xe6\x9f\xad',
'HangulSyllableKeonh': '\xec\xbb\xaa',
'HangulSyllablePwaek': '\xed\x90\xbc',
'UnicodeCharacter0xf546': '\xef\x95\x86',
'CombiningCyrillicLetterZe': '\xe2\xb7\xa5',
'UnicodeCharacter0xf545': '\xef\x95\x85',
'HangulSyllablePwagg': '\xed\x90\x8a',
'CjkUnifiedIdeograph924f': '\xe9\x89\x8f',
'UnicodeCharacter0xdecd': '\xed\xbb\x8d',
'UnicodeCharacter0xdece': '\xed\xbb\x8e',
'UnicodeCharacter0xdecf': '\xed\xbb\x8f',
'KangxiRadicalTree': '\xe2\xbd\x8a',
'UnicodeCharacter0xdeca': '\xed\xbb\x8a',
'UnicodeCharacter0xdecb': '\xed\xbb\x8b',
'UnicodeCharacter0xdecc': '\xed\xbb\x8c',
'CjkUnifiedIdeograph924d': '\xe9\x89\x8d',
'LatinCapitalLetterEzhWithCaron': '\xc7\xae',
'UnicodeCharacter0xeca8': '\xee\xb2\xa8',
'LaoDigitTwo': '\xe0\xbb\x92',
'CjkUnifiedIdeograph41ee': '\xe4\x87\xae',
'CjkUnifiedIdeograph41ef': '\xe4\x87\xaf',
'FullwidthDigitFive': '\xef\xbc\x95',
'CjkUnifiedIdeograph41ea': '\xe4\x87\xaa',
'CjkUnifiedIdeograph41eb': '\xe4\x87\xab',
'CjkUnifiedIdeograph41ec': '\xe4\x87\xac',
'YiSyllableNzyr': '\xea\x8c\x88',
'YiSyllableNzyp': '\xea\x8c\x86',
'ArabicLetterNoonWithDotBelow': '\xda\xb9',
'YiSyllableNzyt': '\xea\x8c\x83',
'HangulJongseongRieulkiyeoksios': '\xe1\x87\x8c',
'YiSyllableNzyx': '\xea\x8c\x84',
'CjkUnifiedIdeograph5593': '\xe5\x96\x93',
'CjkUnifiedIdeograph5592': '\xe5\x96\x92',
'CjkUnifiedIdeograph5591': '\xe5\x96\x91',
'CjkUnifiedIdeograph5590': '\xe5\x96\x90',
'CjkUnifiedIdeograph5597': '\xe5\x96\x97',
'CjkUnifiedIdeograph5596': '\xe5\x96\x96',
'CjkUnifiedIdeograph5595': '\xe5\x96\x95',
'CjkUnifiedIdeograph5594': '\xe5\x96\x94',
'HangulSyllableAgg': '\xec\x95\x86',
'CjkUnifiedIdeograph5599': '\xe5\x96\x99',
'CjkUnifiedIdeograph5598': '\xe5\x96\x98',
'CjkUnifiedIdeograph8749': '\xe8\x9d\x89',
'CjkUnifiedIdeograph8748': '\xe8\x9d\x88',
'CjkUnifiedIdeograph65d9': '\xe6\x97\x99',
'CjkUnifiedIdeograph65d8': '\xe6\x97\x98',
'UnicodeCharacter0xf466': '\xef\x91\xa6',
'GeorgianLetterMan': '\xe1\x83\x9b',
'UnicodeCharacter0xf460': '\xef\x91\xa0',
'DentistrySymbolLightVerticalAndTopRight': '\xe2\x8e\xbe',
'UnicodeCharacter0xf462': '\xef\x91\xa2',
'UnicodeCharacter0xf463': '\xef\x91\xa3',
'CjkUnifiedIdeograph65d1': '\xe6\x97\x91',
'CjkUnifiedIdeograph65d0': '\xe6\x97\x90',
'CjkUnifiedIdeograph65d3': '\xe6\x97\x93',
'CjkUnifiedIdeograph65d2': '\xe6\x97\x92',
'CjkUnifiedIdeograph65d5': '\xe6\x97\x95',
'CjkUnifiedIdeograph65d4': '\xe6\x97\x94',
'CjkUnifiedIdeograph65d7': '\xe6\x97\x97',
'CjkUnifiedIdeograph65d6': '\xe6\x97\x96',
'CjkUnifiedIdeograph53ee': '\xe5\x8f\xae',
'CjkUnifiedIdeograph5162': '\xe5\x85\xa2',
'CjkUnifiedIdeograph873d': '\xe8\x9c\xbd',
'CjkUnifiedIdeograph53ef': '\xe5\x8f\xaf',
'CjkUnifiedIdeograph53ea': '\xe5\x8f\xaa',
'CjkUnifiedIdeograph53ec': '\xe5\x8f\xac',
'CjkUnifiedIdeograph5163': '\xe5\x85\xa3',
'UnicodeCharacter0x2c2f': '\xe2\xb0\xaf',
'CjkUnifiedIdeograph71d6': '\xe7\x87\x96',
'CjkUnifiedIdeograph5160': '\xe5\x85\xa0',
'CjkUnifiedIdeograph559c': '\xe5\x96\x9c',
'CjkUnifiedIdeograph559b': '\xe5\x96\x9b',
'CjkUnifiedIdeograph559a': '\xe5\x96\x9a',
'CjkUnifiedIdeograph874f': '\xe8\x9d\x8f',
'CjkUnifiedIdeograph874a': '\xe8\x9d\x8a',
'CjkUnifiedIdeograph559f': '\xe5\x96\x9f',
'CjkUnifiedIdeograph559e': '\xe5\x96\x9e',
'CjkUnifiedIdeograph559d': '\xe5\x96\x9d',
'InvertedOhmSign': '\xe2\x84\xa7',
'HangulSyllableDyal': '\xeb\x8c\xa4',
'CjkUnifiedIdeograph873e': '\xe8\x9c\xbe',
'HangulSyllableSwels': '\xec\x89\x9c',
'UnicodeCharacter0xe42d': '\xee\x90\xad',
'CjkUnifiedIdeograph53e5': '\xe5\x8f\xa5',
'CjkUnifiedIdeograph53e4': '\xe5\x8f\xa4',
'CjkUnifiedIdeograph53e7': '\xe5\x8f\xa7',
'CjkUnifiedIdeograph53e6': '\xe5\x8f\xa6',
'CjkUnifiedIdeograph53e1': '\xe5\x8f\xa1',
'CjkUnifiedIdeograph53e0': '\xe5\x8f\xa0',
'CjkUnifiedIdeograph53e3': '\xe5\x8f\xa3',
'CjkUnifiedIdeograph53e2': '\xe5\x8f\xa2',
'GurmukhiLetterDha': '\xe0\xa8\xa7',
'CjkUnifiedIdeograph53e9': '\xe5\x8f\xa9',
'CjkUnifiedIdeograph53e8': '\xe5\x8f\xa8',
'UnicodeCharacter0xf46d': '\xef\x91\xad',
'UnicodeCharacter0xf46e': '\xef\x91\xae',
'UnicodeCharacter0xf46f': '\xef\x91\xaf',
'CjkUnifiedIdeograph9679': '\xe9\x99\xb9',
'UnicodeCharacter0xf46a': '\xef\x91\xaa',
'UnicodeCharacter0xf46b': '\xef\x91\xab',
'HangulSyllableMegs': '\xeb\xa9\x97',
'CjkUnifiedIdeograph65da': '\xe6\x97\x9a',
'CjkUnifiedIdeograph65dc': '\xe6\x97\x9c',
'CjkUnifiedIdeograph65db': '\xe6\x97\x9b',
'CjkUnifiedIdeograph65de': '\xe6\x97\x9e',
'CjkUnifiedIdeograph65dd': '\xe6\x97\x9d',
'CjkUnifiedIdeograph65df': '\xe6\x97\x9f',
'WhiteTriangleContainingSmallWhiteTriangle': '\xe2\x9f\x81',
'LessthanWithDot': '\xe2\x8b\x96',
'HangulSyllableBeoss': '\xeb\xb2\x98',
'psi': '\xcf\x88',
'GeorgianLetterBan': '\xe1\x83\x91',
'UnicodeCharacter0xe42f': '\xee\x90\xaf',
'HangulSyllableSwelg': '\xec\x89\x99',
'CanadianSyllabicsLhii': '\xe1\x96\xa1',
'CjkUnifiedIdeograph9671': '\xe9\x99\xb1',
'TifinaghLetterYi': '\xe2\xb5\x89',
'CjkUnifiedIdeograph862e': '\xe8\x98\xae',
'TifinaghLetterYa': '\xe2\xb4\xb0',
'EthiopicSyllableHee': '\xe1\x88\x84',
'CjkUnifiedIdeograph4079': '\xe4\x81\xb9',
'CjkUnifiedIdeograph4078': '\xe4\x81\xb8',
'CjkUnifiedIdeograph4077': '\xe4\x81\xb7',
'CjkUnifiedIdeograph4076': '\xe4\x81\xb6',
'TifinaghLetterYu': '\xe2\xb5\x93',
'CjkUnifiedIdeograph4074': '\xe4\x81\xb4',
'CjkUnifiedIdeograph4073': '\xe4\x81\xb3',
'CjkUnifiedIdeograph4072': '\xe4\x81\xb2',
'CjkUnifiedIdeograph4071': '\xe4\x81\xb1',
'CjkUnifiedIdeograph4070': '\xe4\x81\xb0',
'HangulSyllableHoc': '\xed\x99\x8f',
'HangulSyllableHob': '\xed\x99\x89',
'HangulSyllableHoe': '\xed\x9a\x8c',
'HangulSyllableTyaeh': '\xed\x84\xaf',
'HangulSyllableHog': '\xed\x98\xb9',
'HangulSyllableSet': '\xec\x85\x91',
'HangulSyllableDeoss': '\xeb\x8d\xa8',
'HangulSyllableHok': '\xed\x99\x90',
'HangulSyllableSsess': '\xec\x8e\x98',
'HangulSyllableSep': '\xec\x85\x92',
'HangulSyllableHol': '\xed\x99\x80',
'HangulSyllableSes': '\xec\x85\x8b',
'VaiSyllableWun': '\xea\x96\x9a',
'HangulSyllableSem': '\xec\x85\x88',
'HangulSyllableSen': '\xec\x84\xbc',
'HangulSyllableSeo': '\xec\x84\x9c',
'HangulSyllableSeh': '\xec\x85\x93',
'HangulSyllableHot': '\xed\x99\x91',
'HangulSyllableSej': '\xec\x85\x8e',
'HangulSyllableSek': '\xec\x85\x90',
'HangulSyllableSed': '\xec\x84\xbf',
'HangulSyllableSeg': '\xec\x84\xb9',
'KhmerSymbolLekAttakPrambuon': '\xe1\x9f\xb9',
'HangulSyllableSeb': '\xec\x85\x89',
'HangulSyllableSec': '\xec\x85\x8f',
'HangulSyllableSsyae': '\xec\x8d\x8c',
'CjkUnifiedIdeograph407f': '\xe4\x81\xbf',
'CjkUnifiedIdeograph407e': '\xe4\x81\xbe',
'CjkUnifiedIdeograph407d': '\xe4\x81\xbd',
'CjkUnifiedIdeograph407c': '\xe4\x81\xbc',
'CjkUnifiedIdeograph407b': '\xe4\x81\xbb',
'CjkUnifiedIdeograph407a': '\xe4\x81\xba',
'HangulSyllableGyigg': '\xea\xb8\x96',
'ArabicLetterVehMedialForm': '\xef\xad\xad',
'WhiteTrapezium': '\xe2\x8f\xa2',
'UnicodeCharacter0xab97': '\xea\xae\x97',
'UnicodeCharacter0xab94': '\xea\xae\x94',
'UnicodeCharacter0xab95': '\xea\xae\x95',
'UnicodeCharacter0xab92': '\xea\xae\x92',
'UnicodeCharacter0xab93': '\xea\xae\x93',
'UnicodeCharacter0xab90': '\xea\xae\x90',
'UnicodeCharacter0xab91': '\xea\xae\x91',
'ThaiCharacterSoSo': '\xe0\xb8\x8b',
'ArabicLetterNgoehInitialForm': '\xef\xae\x9c',
'HangulSyllableDdwinh': '\xeb\x9b\xb6',
'UnicodeCharacter0xab98': '\xea\xae\x98',
'CjkUnifiedIdeograph86b2': '\xe8\x9a\xb2',
'CjkUnifiedIdeograph4daa': '\xe4\xb6\xaa',
'UnicodeCharacter0x13f7': '\xe1\x8f\xb7',
'CjkUnifiedIdeograph4dac': '\xe4\xb6\xac',
'UnicodeCharacter0x13f5': '\xe1\x8f\xb5',
'CjkUnifiedIdeograph3db9': '\xe3\xb6\xb9',
'CjkUnifiedIdeograph4dad': '\xe4\xb6\xad',
'CjkUnifiedIdeograph4daf': '\xe4\xb6\xaf',
'CjkUnifiedIdeograph3db5': '\xe3\xb6\xb5',
'CjkUnifiedIdeograph3db4': '\xe3\xb6\xb4',
'CjkUnifiedIdeograph3db7': '\xe3\xb6\xb7',
'CjkUnifiedIdeograph3db6': '\xe3\xb6\xb6',
'CjkUnifiedIdeograph3db1': '\xe3\xb6\xb1',
'KangxiRadicalEye': '\xe2\xbd\xac',
'CjkUnifiedIdeograph3db3': '\xe3\xb6\xb3',
'UnicodeCharacter0x13f9': '\xe1\x8f\xb9',
'CjkUnifiedIdeograph74dc': '\xe7\x93\x9c',
'CjkUnifiedIdeograph74db': '\xe7\x93\x9b',
'CjkUnifiedIdeograph74da': '\xe7\x93\x9a',
'CjkUnifiedIdeograph74df': '\xe7\x93\x9f',
'CjkUnifiedIdeograph74de': '\xe7\x93\x9e',
'CjkUnifiedIdeograph74dd': '\xe7\x93\x9d',
'UnicodeCharacter0xf5f1': '\xef\x97\xb1',
'CjkUnifiedIdeograph990a': '\xe9\xa4\x8a',
'CjkUnifiedIdeograph990b': '\xe9\xa4\x8b',
'CjkUnifiedIdeograph990c': '\xe9\xa4\x8c',
'CjkUnifiedIdeograph990d': '\xe9\xa4\x8d',
'CjkUnifiedIdeograph990e': '\xe9\xa4\x8e',
'CjkUnifiedIdeograph990f': '\xe9\xa4\x8f',
'HangulSyllableJjyilb': '\xec\xaf\xbb',
'CjkUnifiedIdeograph5fac': '\xe5\xbe\xac',
'UnicodeCharacter0x1bca': '\xe1\xaf\x8a',
'CjkUnifiedIdeograph503a': '\xe5\x80\xba',
'CjkUnifiedIdeograph503b': '\xe5\x80\xbb',
'CjkUnifiedIdeograph503c': '\xe5\x80\xbc',
'CjkUnifiedIdeograph503d': '\xe5\x80\xbd',
'CjkUnifiedIdeograph503e': '\xe5\x80\xbe',
'CjkUnifiedIdeograph503f': '\xe5\x80\xbf',
'TibetanLetterZa': '\xe0\xbd\x9f',
'UnicodeCharacter0x13ff': '\xe1\x8f\xbf',
'CjkUnifiedIdeograph4da0': '\xe4\xb6\xa0',
'CjkUnifiedIdeograph4da3': '\xe4\xb6\xa3',
'UnicodeCharacter0x13fe': '\xe1\x8f\xbe',
'UnicodeCharacter0x13fb': '\xe1\x8f\xbb',
'UnicodeCharacter0x13fc': '\xe1\x8f\xbc',
'CjkUnifiedIdeograph4da7': '\xe4\xb6\xa7',
'CjkUnifiedIdeograph4da6': '\xe4\xb6\xa6',
'CjkUnifiedIdeograph4da9': '\xe4\xb6\xa9',
'CjkUnifiedIdeograph3dbd': '\xe3\xb6\xbd',
'CjkUnifiedIdeograph3dbf': '\xe3\xb6\xbf',
'CjkUnifiedIdeograph3dba': '\xe3\xb6\xba',
'CjkUnifiedIdeograph3dbc': '\xe3\xb6\xbc',
'CjkUnifiedIdeograph3dbb': '\xe3\xb6\xbb',
'UnicodeCharacter0xab9f': '\xea\xae\x9f',
'UnicodeCharacter0xab9d': '\xea\xae\x9d',
'UnicodeCharacter0xab9e': '\xea\xae\x9e',
'UnicodeCharacter0xab9b': '\xea\xae\x9b',
'CyrillicSmallLetterBigYus': '\xd1\xab',
'UnicodeCharacter0xab9a': '\xea\xae\x9a',
'HangulSyllableRyu': '\xeb\xa5\x98',
'CyrillicCapitalLetterCheWithVerticalStroke': '\xd2\xb8',
'CjkUnifiedIdeograph5038': '\xe5\x80\xb8',
'CjkUnifiedIdeograph5039': '\xe5\x80\xb9',
'ModifierLetterSmallUpsilon': '\xe1\xb6\xb7',
'd': '\xce\xb4',
'CjkUnifiedIdeograph5030': '\xe5\x80\xb0',
'CjkUnifiedIdeograph5031': '\xe5\x80\xb1',
'GreekSmallLetterOmicronWithOxia': '\xe1\xbd\xb9',
'CjkUnifiedIdeograph5033': '\xe5\x80\xb3',
'CjkUnifiedIdeograph5034': '\xe5\x80\xb4',
'CjkUnifiedIdeograph5035': '\xe5\x80\xb5',
'CjkUnifiedIdeograph5036': '\xe5\x80\xb6',
'CjkUnifiedIdeograph5037': '\xe5\x80\xb7',
'CjkUnifiedIdeograph74d3': '\xe7\x93\x93',
'CjkUnifiedIdeograph74d2': '\xe7\x93\x92',
'CjkUnifiedIdeograph74d1': '\xe7\x93\x91',
'CjkUnifiedIdeograph74d0': '\xe7\x93\x90',
'CjkUnifiedIdeograph74d7': '\xe7\x93\x97',
'CjkUnifiedIdeograph74d6': '\xe7\x93\x96',
'CjkUnifiedIdeograph74d5': '\xe7\x93\x95',
'CjkUnifiedIdeograph74d4': '\xe7\x93\x94',
'CjkUnifiedIdeograph9900': '\xe9\xa4\x80',
'CjkUnifiedIdeograph9901': '\xe9\xa4\x81',
'CjkUnifiedIdeograph74d9': '\xe7\x93\x99',
'CjkUnifiedIdeograph74d8': '\xe7\x93\x98',
'CjkUnifiedIdeograph9904': '\xe9\xa4\x84',
'CjkUnifiedIdeograph9905': '\xe9\xa4\x85',
'CjkUnifiedIdeograph9906': '\xe9\xa4\x86',
'CjkUnifiedIdeograph9907': '\xe9\xa4\x87',
'CjkUnifiedIdeograph5be8': '\xe5\xaf\xa8',
'CjkUnifiedIdeograph5be9': '\xe5\xaf\xa9',
'HangulSyllableDenj': '\xeb\x8d\xb5',
'ArabicLigatureTahWithYehIsolatedForm': '\xef\xb3\xb6',
'CjkUnifiedIdeograph92a6': '\xe9\x8a\xa6',
'RomanNumeralTwelve': '\xe2\x85\xab',
'HeavyFourBalloonspokedAsterisk': '\xe2\x9c\xa4',
'CjkUnifiedIdeograph5be1': '\xe5\xaf\xa1',
'CjkUnifiedIdeograph5be2': '\xe5\xaf\xa2',
'CjkUnifiedIdeograph5be3': '\xe5\xaf\xa3',
'CjkUnifiedIdeograph5be4': '\xe5\xaf\xa4',
'CjkUnifiedIdeograph5be5': '\xe5\xaf\xa5',
'CjkUnifiedIdeograph5be6': '\xe5\xaf\xa6',
'CjkUnifiedIdeograph5be7': '\xe5\xaf\xa7',
'CjkUnifiedIdeograph95a1': '\xe9\x96\xa1',
'SquareMeetoru': '\xe3\x8d\x8d',
'HangulSyllableMeulh': '\xeb\xaf\x8f',
'HangulSyllableTyag': '\xed\x83\xb9',
'UnicodeCharacter0xf54c': '\xef\x95\x8c',
'HangulSyllableTyae': '\xed\x84\x94',
'CjkUnifiedIdeograph92a7': '\xe9\x8a\xa7',
'HangulSyllableTyac': '\xed\x84\x8f',
'HangulSyllableTyab': '\xed\x84\x89',
'UnicodeCharacter0xf54b': '\xef\x95\x8b',
'HangulSyllableTyan': '\xed\x83\xbc',
'DevanagariOm': '\xe0\xa5\x90',
'HangulSyllableTyal': '\xed\x84\x80',
'HangulSyllableTyak': '\xed\x84\x90',
'HangulSyllableTyaj': '\xed\x84\x8e',
'HangulSyllableGaebs': '\xea\xb0\xae',
'HangulSyllableTyah': '\xed\x84\x93',
'YiSyllableRrep': '\xea\x8e\x87',
'HangulSyllableTyas': '\xed\x84\x8b',
'UnicodeCharacter0xe50d': '\xee\x94\x8d',
'HangulJongseongRieultikeut': '\xe1\x87\x8e',
'HangulSyllableTyap': '\xed\x84\x92',
'YiSyllableGgurx': '\xea\x88\xae',
'UnicodeCharacter0xf54f': '\xef\x95\x8f',
'CjkUnifiedIdeograph3894': '\xe3\xa2\x94',
'CyrillicCapitalLetterShcha': '\xd0\xa9',
'HangulSyllableTaels': '\xed\x83\xa8',
'CanadianSyllabicsCarrierDee': '\xe1\x91\x93',
'UnicodeCharacter0xf8ef': '\xef\xa3\xaf',
'YiSyllableRret': '\xea\x8e\x84',
'HangulSyllableHyeobs': '\xed\x98\x92',
'CjkUnifiedIdeograph967d': '\xe9\x99\xbd',
'BalineseLetterIkara': '\xe1\xac\x87',
'CjkUnifiedIdeograph5bea': '\xe5\xaf\xaa',
'CjkUnifiedIdeograph5beb': '\xe5\xaf\xab',
'CjkUnifiedIdeograph5bec': '\xe5\xaf\xac',
'CjkUnifiedIdeograph5bed': '\xe5\xaf\xad',
'CjkUnifiedIdeograph5bee': '\xe5\xaf\xae',
'CjkUnifiedIdeograph5bef': '\xe5\xaf\xaf',
'CjkUnifiedIdeograph7dfd': '\xe7\xb7\xbd',
'CopticCapitalLetterKsi': '\xe2\xb2\x9c',
'CjkUnifiedIdeograph97e9': '\xe9\x9f\xa9',
'CircledHangulNieun': '\xe3\x89\xa1',
'HangulSyllablePyit': '\xed\x94\xb9',
'HangulJongseongIeungkhieukh': '\xe1\x87\xaf',
'CjkUnifiedIdeograph83e9': '\xe8\x8f\xa9',
'UnicodeCharacter0x17ec': '\xe1\x9f\xac',
'UnicodeCharacter0x17eb': '\xe1\x9f\xab',
'CjkUnifiedIdeograph4853': '\xe4\xa1\x93',
'CjkUnifiedIdeograph4852': '\xe4\xa1\x92',
'CjkUnifiedIdeograph4855': '\xe4\xa1\x95',
'UnicodeCharacter0x17ef': '\xe1\x9f\xaf',
'UnicodeCharacter0x17ee': '\xe1\x9f\xae',
'CjkUnifiedIdeograph4856': '\xe4\xa1\x96',
'CjkUnifiedIdeograph4859': '\xe4\xa1\x99',
'CjkUnifiedIdeograph4858': '\xe4\xa1\x98',
'HangulSyllableGgyaen': '\xea\xba\xa4',
'HangulSyllableGgyaeh': '\xea\xba\xbb',
'HangulSyllableBboeg': '\xeb\xbe\x95',
'HangulSyllableGgyaej': '\xea\xba\xb6',
'HangulSyllableGgyaek': '\xea\xba\xb8',
'CjkUnifiedIdeograph750f': '\xe7\x94\x8f',
'CjkUnifiedIdeograph8fc4': '\xe8\xbf\x84',
'CjkUnifiedIdeograph750d': '\xe7\x94\x8d',
'CjkUnifiedIdeograph750e': '\xe7\x94\x8e',
'CjkUnifiedIdeograph750b': '\xe7\x94\x8b',
'CjkUnifiedIdeograph750c': '\xe7\x94\x8c',
'CjkUnifiedIdeograph8fc3': '\xe8\xbf\x83',
'TibetanSubjoinedLetterKha': '\xe0\xbe\x91',
'HangulSyllableDeng': '\xeb\x8e\x85',
'HangulSyllableBboes': '\xeb\xbe\xa7',
'HangulSyllableBboep': '\xeb\xbe\xae',
'CjkUnifiedIdeograph8fc9': '\xe8\xbf\x89',
'CjkUnifiedIdeograph8fc8': '\xe8\xbf\x88',
'HangulSyllableBboet': '\xeb\xbe\xad',
'UnicodeCharacter0xdb08': '\xed\xac\x88',
'UnicodeCharacter0xdb09': '\xed\xac\x89',
'UnicodeCharacter0xdb06': '\xed\xac\x86',
'UnicodeCharacter0xdb07': '\xed\xac\x87',
'UnicodeCharacter0xdb04': '\xed\xac\x84',
'EthiopicSyllablePhi': '\xe1\x8c\xb2',
'UnicodeCharacter0xdb02': '\xed\xac\x82',
'UnicodeCharacter0xdb03': '\xed\xac\x83',
'UnicodeCharacter0xdb00': '\xed\xac\x80',
'UnicodeCharacter0xdb01': '\xed\xac\x81',
'YiSyllableZza': '\xea\x8b\x9a',
'CjkUnifiedIdeograph3e2a': '\xe3\xb8\xaa',
'CjkUnifiedIdeograph3e2b': '\xe3\xb8\xab',
'CjkUnifiedIdeograph3e2c': '\xe3\xb8\xac',
'CjkUnifiedIdeograph3e2d': '\xe3\xb8\xad',
'CjkUnifiedIdeograph3e2e': '\xe3\xb8\xae',
'CjkUnifiedIdeograph3e2f': '\xe3\xb8\xaf',
'UnicodeCharacter0xef3d': '\xee\xbc\xbd',
'HangulSyllableSsyagg': '\xec\x8c\xb2',
'CjkUnifiedIdeograph7506': '\xe7\x94\x86',
'KhmerLetterNyo': '\xe1\x9e\x89',
'CjkUnifiedIdeograph7504': '\xe7\x94\x84',
'CjkUnifiedIdeograph7505': '\xe7\x94\x85',
'CjkUnifiedIdeograph7502': '\xe7\x94\x82',
'CjkUnifiedIdeograph7503': '\xe7\x94\x83',
'CjkUnifiedIdeograph7500': '\xe7\x94\x80',
'CjkUnifiedIdeograph7501': '\xe7\x94\x81',
'HangulSyllableJyilh': '\xec\xa6\xb3',
'CjkUnifiedIdeograph7163': '\xe7\x85\xa3',
'ArabicLetterDotlessQaf': '\xd9\xaf',
'CjkUnifiedIdeograph7508': '\xe7\x94\x88',
'CjkUnifiedIdeograph7509': '\xe7\x94\x89',
'CjkUnifiedIdeograph485a': '\xe4\xa1\x9a',
'HangulSyllableJyils': '\xec\xa6\xb0',
'CjkUnifiedIdeograph485c': '\xe4\xa1\x9c',
'CjkUnifiedIdeograph485b': '\xe4\xa1\x9b',
'CjkUnifiedIdeograph485e': '\xe4\xa1\x9e',
'CjkUnifiedIdeograph485d': '\xe4\xa1\x9d',
'HangulSyllableJyilt': '\xec\xa6\xb1',
'CjkUnifiedIdeograph485f': '\xe4\xa1\x9f',
'ArabicLigatureKafWithAlefMaksuraIsolatedForm': '\xef\xb0\xbd',
'CjkUnifiedIdeograph3e20': '\xe3\xb8\xa0',
'CjkUnifiedIdeograph3e21': '\xe3\xb8\xa1',
'CjkUnifiedIdeograph3e22': '\xe3\xb8\xa2',
'CjkUnifiedIdeograph3e23': '\xe3\xb8\xa3',
'CjkUnifiedIdeograph3e24': '\xe3\xb8\xa4',
'CjkUnifiedIdeograph3e25': '\xe3\xb8\xa5',
'CjkUnifiedIdeograph3e26': '\xe3\xb8\xa6',
'UnicodeCharacter0xea0': '\xe0\xba\xa0',
'CjkUnifiedIdeograph3e28': '\xe3\xb8\xa8',
'CjkUnifiedIdeograph3e29': '\xe3\xb8\xa9',
'HangulSyllableKugs': '\xec\xbf\xa3',
'HangulSyllableBbyuj': '\xec\x81\x92',
'CyrillicSmallLetterSemisoftSign': '\xd2\x8d',
'LatinSmallLetterRumRotunda': '\xea\x9d\x9d',
'HangulSyllableBbyuk': '\xec\x81\x94',
'YiSyllableRryrx': '\xea\x8e\x92',
'BottomHalfIntegral': '\xe2\x8c\xa1',
'ParagraphSeparator': '\xe2\x80\xa9',
'ArabicLetterUWithHamzaAbove': '\xd9\xb7',
'UnicodeCharacter0xdb0e': '\xed\xac\x8e',
'UnicodeCharacter0xdb0b': '\xed\xac\x8b',
'UnicodeCharacter0xdb0c': '\xed\xac\x8c',
'CjkUnifiedIdeograph3a47': '\xe3\xa9\x87',
'HangulSyllableSsogs': '\xec\x8f\x9b',
'UnicodeCharacter0xe1d7': '\xee\x87\x97',
'UnicodeCharacter0xea6': '\xe0\xba\xa6',
'UnicodeCharacter0xe1d4': '\xee\x87\x94',
'CanadianSyllabicsWestcreeThwaa': '\xe1\x95\xa9',
'UnicodeCharacter0xe1d5': '\xee\x87\x95',
'CanadianSyllabicsThcreeTh': '\xe1\x96\xae',
'CjkUnifiedIdeograph97e7': '\xe9\x9f\xa7',
'UnicodeCharacter0xe1d3': '\xee\x87\x93',
'CjkUnifiedIdeograph83e3': '\xe8\x8f\xa3',
'CjkUnifiedIdeograph3b98': '\xe3\xae\x98',
'CjkUnifiedIdeograph3b99': '\xe3\xae\x99',
'UnicodeCharacter0xe1d0': '\xee\x87\x90',
'ArabicLigatureNoonWithJeemWithHahFinalForm': '\xef\xb6\xbd',
'CjkUnifiedIdeograph3b92': '\xe3\xae\x92',
'CjkUnifiedIdeograph3b93': '\xe3\xae\x93',
'CjkUnifiedIdeograph3b90': '\xe3\xae\x90',
'CjkUnifiedIdeograph3b91': '\xe3\xae\x91',
'CjkUnifiedIdeograph3b96': '\xe3\xae\x96',
'CjkUnifiedIdeograph3b97': '\xe3\xae\x97',
'CjkUnifiedIdeograph3b94': '\xe3\xae\x94',
'CjkUnifiedIdeograph3b95': '\xe3\xae\x95',
'HangulSyllableKugg': '\xec\xbf\xa2',
'ArabicLigatureKafWithMeemInitialForm': '\xef\xb3\x88',
'HangulSyllableDeogs': '\xeb\x8d\x97',
'ArabicLetterRreh': '\xda\x91',
'HangulSyllableByat': '\xeb\xb1\xa5',
'SimilarOrLessthan': '\xe2\xaa\x9d',
'HangulSyllableByap': '\xeb\xb1\xa6',
'HangulSyllableByas': '\xeb\xb1\x9f',
'ArabicLigatureJeemWithMeemWithAlefMaksuraFinalForm': '\xef\xb6\xa7',
'HangulSyllableByae': '\xeb\xb1\xa8',
'HangulSyllableByad': '\xeb\xb1\x93',
'CanadianSyllabicsTte': '\xe1\x91\xa7',
'CanadianSyllabicsTta': '\xe1\x91\xaa',
'HangulSyllableByab': '\xeb\xb1\x9d',
'CanadianSyllabicsTto': '\xe1\x91\xa9',
'HangulSyllableByal': '\xeb\xb1\x94',
'HangulSyllableByan': '\xeb\xb1\x90',
'HangulSyllableByah': '\xeb\xb1\xa7',
'CanadianSyllabicsTti': '\xe1\x91\xa8',
'CanadianSyllabicsTth': '\xe1\x95\xaf',
'HangulSyllableBbweols': '\xeb\xbf\xb4',
'ArabicSmallDamma': '\xd8\x99',
'HangulSyllableBbyong': '\xeb\xbf\x85',
'HangulSyllablePeung': '\xed\x94\x99',
'HangulSyllableBbyonh': '\xeb\xbe\xb6',
'HangulSyllableBbyonj': '\xeb\xbe\xb5',
'HangulSyllableHwelg': '\xed\x9c\x85',
'ArabicLetterAinFinalForm': '\xef\xbb\x8a',
'GlagoliticSmallLetterTrokutastiA': '\xe2\xb1\x9d',
'CjkUnifiedIdeograph3b9b': '\xe3\xae\x9b',
'CjkUnifiedIdeograph3b9c': '\xe3\xae\x9c',
'CjkUnifiedIdeograph3b9a': '\xe3\xae\x9a',
'CjkUnifiedIdeograph3b9f': '\xe3\xae\x9f',
'CjkUnifiedIdeograph3b9d': '\xe3\xae\x9d',
'CjkUnifiedIdeograph3b9e': '\xe3\xae\x9e',
'UnicodeCharacter0xe189': '\xee\x86\x89',
'UnicodeCharacter0x1bc0': '\xe1\xaf\x80',
'UnicodeCharacter0x1bc1': '\xe1\xaf\x81',
'CanadianSyllabicsNngii': '\xe1\x99\xb2',
'HangulSyllableKeulh': '\xed\x81\xbb',
'SaurashtraLetterCa': '\xea\xa2\x97',
'UnicodeCharacter0x1bc2': '\xe1\xaf\x82',
'LeftTranspositionBracket': '\xe2\xb8\x89',
'HangulSyllableJals': '\xec\x9e\x9c',
'HangulSyllableJalp': '\xec\x9e\x9e',
'UnicodeCharacter0x1bc3': '\xe1\xaf\x83',
'HangulSyllableJalt': '\xec\x9e\x9d',
'UnicodeCharacter0x1bc4': '\xe1\xaf\x84',
'UnicodeCharacter0x1bc5': '\xe1\xaf\x85',
'HangulSyllableJalb': '\xec\x9e\x9b',
'IdeographicTelegraphSymbolForMay': '\xe3\x8b\x84',
'UnicodeCharacter0x1bc6': '\xe1\xaf\x86',
'HangulSyllableKinh': '\xed\x82\xaa',
'HangulSyllableKing': '\xed\x82\xb9',
'HangulSyllableJalh': '\xec\x9e\x9f',
'UnicodeCharacter0x1bc7': '\xe1\xaf\x87',
'GlagoliticCapitalLetterMyslite': '\xe2\xb0\x8f',
'GurmukhiLetterSa': '\xe0\xa8\xb8',
'HangulSyllableJalm': '\xec\x9e\x9a',
'YiSyllableMit': '\xea\x82\xae',
'GlagoliticSmallLetterBuky': '\xe2\xb0\xb1',
'CjkUnifiedIdeograph7197': '\xe7\x86\x97',
'YiSyllableMip': '\xea\x82\xb1',
'UnicodeCharacter0xef31': '\xee\xbc\xb1',
'HangulSyllableRolg': '\xeb\xa1\xa5',
'YiSyllableMix': '\xea\x82\xaf',
'HangulSyllableRolb': '\xeb\xa1\xa7',
'YiSyllableMie': '\xea\x82\xb3',
'HangulSyllableRolt': '\xeb\xa1\xa9',
'WhiteCircleWithUpperLeftQuadrant': '\xe2\x97\xb4',
'HangulSyllableGgagg': '\xea\xb9\x8e',
'HangulSyllableRolp': '\xeb\xa1\xaa',
'ArabicLigatureSallallahouAlayheWasallam': '\xef\xb7\xba',
'CjkUnifiedIdeograph716a': '\xe7\x85\xaa',
'ArabicLigatureFehWithAlefMaksuraFinalForm': '\xef\xb1\xbc',
'YiSyllableGurx': '\xea\x87\xb6',
'HangulSyllableRaelt': '\xeb\x9e\xa5',
'KangxiRadicalNet': '\xe2\xbd\xb9',
'RunicLetterUruzUrU': '\xe1\x9a\xa2',
'HangulSyllableRaelp': '\xeb\x9e\xa6',
'HangulSyllableRaelm': '\xeb\x9e\xa2',
'HangulSyllableRaelh': '\xeb\x9e\xa7',
'SingleLeftpointingAngleQuotationMark': '\xe2\x80\xb9',
'HangulSyllableRaelg': '\xeb\x9e\xa1',
'HangulSyllableSoegg': '\xec\x87\xa2',
'HangulSyllableRaelb': '\xeb\x9e\xa3',
'SymbolForDeviceControlOne': '\xe2\x90\x91',
'HangulSyllableJyam': '\xec\x9f\x98',
'HangulSyllableJyal': '\xec\x9f\x90',
'UnicodeCharacter0xe85e': '\xee\xa1\x9e',
'HangulSyllableJyan': '\xec\x9f\x8c',
'VaiSyllableKpoo': '\xea\x95\xba',
'HangulSyllableJyah': '\xec\x9f\xa3',
'LatinSmallLetterJWithCircumflex': '\xc4\xb5',
'HangulSyllableJyaj': '\xec\x9f\x9e',
'HangulSyllableJyae': '\xec\x9f\xa4',
'HangulSyllableJyad': '\xec\x9f\x8f',
'HangulSyllableJyag': '\xec\x9f\x89',
'HangulSyllableJyac': '\xec\x9f\x9f',
'TeluguVowelSignAu': '\xe0\xb1\x8c',
'HangulSyllableNyeogs': '\xeb\x85\x83',
'TeluguVowelSignAi': '\xe0\xb1\x88',
'SouthEastDoubleArrow': '\xe2\x87\x98',
'HangulSyllableJyat': '\xec\x9f\xa1',
'CyrillicSmallLetterCheWithDiaeresis': '\xd3\xb5',
'TeluguVowelSignAa': '\xe0\xb0\xbe',
'HangulSyllableJyap': '\xec\x9f\xa2',
'HangulSyllableJyas': '\xec\x9f\x9b',
'HangulSyllableGyan': '\xea\xb0\xbc',
'HangulSyllableGgags': '\xea\xb9\x8f',
'LatinCapitalLetterEgyptologicalAlef': '\xea\x9c\xa2',
'CrossOfLorraine': '\xe2\x98\xa8',
'LatinSmallLetterAWithRingBelow': '\xe1\xb8\x81',
'CjkUnifiedIdeograph389f': '\xe3\xa2\x9f',
'YiSyllableJie': '\xea\x8f\xa6',
'YiSyllableJix': '\xea\x8f\xa1',
'BlackUppointingSmallTriangle': '\xe2\x96\xb4',
'YiSyllableJip': '\xea\x8f\xa3',
'CjkUnifiedIdeograph7193': '\xe7\x86\x93',
'IdeographicTelegraphSymbolForHourThirteen': '\xe3\x8d\xa5',
'HangulSyllableDdot': '\xeb\x98\xa9',
'HangulSyllableRyolm': '\xeb\xa3\x96',
'HangulSyllableDdop': '\xeb\x98\xaa',
'HangulSyllableSyeog': '\xec\x85\x95',
'HangulSyllableSyeod': '\xec\x85\x9b',
'HangulSyllableDdos': '\xeb\x98\xa3',
'HangulSyllableSyeoj': '\xec\x85\xaa',
'HangulSyllableSyeok': '\xec\x85\xac',
'HangulSyllableSyeoh': '\xec\x85\xaf',
'HangulSyllableRyolg': '\xeb\xa3\x95',
'HangulSyllableSyeon': '\xec\x85\x98',
'HangulSyllableRyolb': '\xeb\xa3\x97',
'HangulSyllableSyeom': '\xec\x85\xa4',
'HangulSyllableDdod': '\xeb\x98\x97',
'HangulSyllableDdoe': '\xeb\x99\xa4',
'HangulSyllableSyeop': '\xec\x85\xae',
'HangulSyllableDdog': '\xeb\x98\x91',
'HangulSyllableTyogs': '\xed\x88\x93',
'HangulSyllableDdob': '\xeb\x98\xa1',
'HangulSyllableDdoc': '\xeb\x98\xa7',
'HangulSyllableDdol': '\xeb\x98\x98',
'HangulSyllableDdom': '\xeb\x98\xa0',
'HangulSyllableDdon': '\xeb\x98\x94',
'HangulSyllableDdoh': '\xeb\x98\xab',
'HangulSyllableDdoj': '\xeb\x98\xa6',
'HangulSyllableDdok': '\xeb\x98\xa8',
'HangulSyllableBbwenh': '\xec\x80\x8a',
'NeitherGreaterthanNorEquivalentTo': '\xe2\x89\xb5',
'CanadianSyllabicsCarrierWu': '\xe1\x97\x90',
'CjkUnifiedIdeograph769f': '\xe7\x9a\x9f',
'HangulSyllableBbweng': '\xec\x80\x99',
'LeftwardsBlackArrow': '\xe2\xac\x85',
'CanadianSyllabicsCarrierWa': '\xe1\x97\x95',
'ArabicLigatureBehWithMeemFinalForm': '\xef\xb1\xac',
'CanadianSyllabicsCarrierWe': '\xe1\x97\x92',
'CanadianSyllabicsCarrierWi': '\xe1\x97\x94',
'CjkUnifiedIdeograph8261': '\xe8\x89\xa1',
'BalineseSignCecek': '\xe1\xac\x82',
'CanadianSyllabicsCarrierWo': '\xe1\x97\x91',
'CjkUnifiedIdeograph850c': '\xe8\x94\x8c',
'CjkUnifiedIdeograph850b': '\xe8\x94\x8b',
'ScriptCapitalM': '\xe2\x84\xb3',
'ScriptCapitalL': '\xe2\x84\x92',
'KangxiRadicalBadger': '\xe2\xbe\x98',
'GujaratiLetterDdha': '\xe0\xaa\xa2',
'ScriptCapitalI': '\xe2\x84\x90',
'ScriptCapitalH': '\xe2\x84\x8b',
'ScriptCapitalF': '\xe2\x84\xb1',
'ScriptCapitalE': '\xe2\x84\xb0',
'CjkUnifiedIdeograph6ddc': '\xe6\xb7\x9c',
'CjkUnifiedIdeograph6ddd': '\xe6\xb7\x9d',
'ScriptCapitalB': '\xe2\x84\xac',
'CurrencySign': '\xc2\xa4',
'HangulSyllableSsye': '\xec\x8e\xbc',
'HangulSyllableDdyibs': '\xeb\x9d\x96',
'FeminineOrdinalIndicator': '\xc2\xaa',
'CyrillicCapitalLetterStraightU': '\xd2\xae',
'HalfwidthHangulLetterSios': '\xef\xbe\xb5',
'UnicodeCharacter0xeb10': '\xee\xac\x90',
'UnicodeCharacter0xde10': '\xed\xb8\x90',
'ScriptCapitalR': '\xe2\x84\x9b',
'ScriptCapitalP': '\xe2\x84\x98',
'UnicodeCharacter0xf8eb': '\xef\xa3\xab',
'PlusSignAboveEqualsSign': '\xe2\xa9\xb2',
'HangulSyllableBbaegg': '\xeb\xb9\xbe',
'VariationSelector14': '\xef\xb8\x8d',
'VariationSelector15': '\xef\xb8\x8e',
'VariationSelector16': '\xef\xb8\x8f',
'VariationSelector10': '\xef\xb8\x89',
'VariationSelector11': '\xef\xb8\x8a',
'VariationSelector12': '\xef\xb8\x8b',
'VariationSelector13': '\xef\xb8\x8c',
'UnicodeCharacter0xde11': '\xed\xb8\x91',
'HangulSyllableBbaegs': '\xeb\xb9\xbf',
'EthiopicSyllableHhee': '\xe1\x88\x94',
'UnicodeCharacter0xb12': '\xe0\xac\x92',
'UnicodeCharacter0xb11': '\xe0\xac\x91',
'CjkUnifiedIdeograph6dd8': '\xe6\xb7\x98',
'CanadianSyllabicsSayisiSha': '\xe1\x96\x9a',
'CjkUnifiedIdeograph8501': '\xe8\x94\x81',
'CjkUnifiedIdeograph8500': '\xe8\x94\x80',
'CjkUnifiedIdeograph8507': '\xe8\x94\x87',
'CanadianSyllabicsSayisiShe': '\xe1\x96\x97',
'CjkUnifiedIdeograph8505': '\xe8\x94\x85',
'CjkUnifiedIdeograph8504': '\xe8\x94\x84',
'CjkUnifiedIdeograph6dd0': '\xe6\xb7\x90',
'CanadianSyllabicsSayisiShi': '\xe1\x96\x98',
'CjkUnifiedIdeograph6dd2': '\xe6\xb7\x92',
'CjkUnifiedIdeograph6dd3': '\xe6\xb7\x93',
'CjkUnifiedIdeograph6dd4': '\xe6\xb7\x94',
'CjkUnifiedIdeograph6dd5': '\xe6\xb7\x95',
'CjkUnifiedIdeograph6dd6': '\xe6\xb7\x96',
'CanadianSyllabicsSayisiSho': '\xe1\x96\x99',
'CanadianSyllabicsMedialR': '\xe1\x95\x92',
'YiSyllableShuo': '\xea\x8e\xb0',
'TibetanVowelSignReversedI': '\xe0\xbe\x80',
'UnicodeCharacter0xeb17': '\xee\xac\x97',
'FullwidthAmpersand': '\xef\xbc\x86',
'UnicodeCharacter0xe26f': '\xee\x89\xaf',
'YiSyllableNburx': '\xea\x82\x89',
'CanadianSyllabicsMedialL': '\xe1\x93\xac',
'ArmenianSmallLigatureVewNow': '\xef\xac\x96',
'HiraganaDigraphYori': '\xe3\x82\x9f',
'HangulSyllableKyaess': '\xec\xbb\x9c',
'TotalRunout': '\xe2\x8c\xb0',
'CopticCapitalLetterCryptogrammicNi': '\xe2\xb2\xbc',
'HangulSyllableHobs': '\xed\x99\x8a',
'VaiSyllableNoo': '\xea\x96\x93',
'HangulSyllableWeog': '\xec\x9b\x8d',
'ctheta': '\xcf\x91',
'CjkUnifiedIdeograph4ac8': '\xe4\xab\x88',
'CjkUnifiedIdeograph4ac9': '\xe4\xab\x89',
'CjkUnifiedIdeograph4ac2': '\xe4\xab\x82',
'CjkUnifiedIdeograph4ac3': '\xe4\xab\x83',
'CjkUnifiedIdeograph4ac0': '\xe4\xab\x80',
'CjkUnifiedIdeograph4ac1': '\xe4\xab\x81',
'AplFunctionalSymbolQuad': '\xe2\x8e\x95',
'CjkUnifiedIdeograph4ac7': '\xe4\xab\x87',
'CjkUnifiedIdeograph4ac4': '\xe4\xab\x84',
'CjkUnifiedIdeograph4ac5': '\xe4\xab\x85',
'UnicodeCharacter0xf5fa': '\xef\x97\xba',
'UnicodeCharacter0xe26d': '\xee\x89\xad',
'BraillePatternBlank': '\xe2\xa0\x80',
'HangulJungseongOo': '\xe1\x86\x82',
'HangulJungseongOe': '\xe1\x86\x80',
'HangulJungseongOu': '\xe1\x86\x83',
'HangulSyllableKyeogs': '\xec\xbc\x9f',
'HangulSyllableWeos': '\xec\x9b\x9f',
'UnicodeCharacter0xab07': '\xea\xac\x87',
'CjkUnifiedIdeograph96f3': '\xe9\x9b\xb3',
'ReversedAngleWithUnderbar': '\xe2\xa6\xa5',
'CjkUnifiedIdeograph4acb': '\xe4\xab\x8b',
'CjkUnifiedIdeograph4acc': '\xe4\xab\x8c',
'CjkUnifiedIdeograph4aca': '\xe4\xab\x8a',
'CjkUnifiedIdeograph4acf': '\xe4\xab\x8f',
'CjkUnifiedIdeograph4acd': '\xe4\xab\x8d',
'CjkUnifiedIdeograph4ace': '\xe4\xab\x8e',
'ArabicLetterHahWithThreeDotsPointingUpwardsBelow': '\xdd\x98',
'HangulSyllableDeunj': '\xeb\x93\xa1',
'LatinCapitalLetterKWithLineBelow': '\xe1\xb8\xb4',
'GlagoliticSmallLetterIotatedSmallYus': '\xe2\xb1\x97',
'YiSyllableHiex': '\xea\x89\xaf',
'CjkUnifiedIdeograph464e': '\xe4\x99\x8e',
'KannadaLetterPa': '\xe0\xb2\xaa',
'ArabicLetterYehWithHamzaAboveMedialForm': '\xef\xba\x8c',
'HangulSyllableEolt': '\xec\x97\x81',
'UnicodeCharacter0xe90b': '\xee\xa4\x8b',
'UnicodeCharacter0xe90c': '\xee\xa4\x8c',
'UnicodeCharacter0xe90a': '\xee\xa4\x8a',
'LatinCapitalLetterOWithGrave': '\xc3\x92',
'YiSyllableVy': '\xea\x83\xbc',
'UnicodeCharacter0xe90d': '\xee\xa4\x8d',
'UnicodeCharacter0xe90e': '\xee\xa4\x8e',
'KhmerLetterQa': '\xe1\x9e\xa2',
'YiSyllableVu': '\xea\x83\xb6',
'CjkUnifiedIdeograph893c': '\xe8\xa4\xbc',
'YiSyllableVo': '\xea\x83\xb0',
'YiSyllableVi': '\xea\x83\xa4',
'YiSyllableVa': '\xea\x83\xac',
'UnicodeCharacter0x9ff7': '\xe9\xbf\xb7',
'ArmenianSmallLetterReh': '\xd6\x80',
'UnicodeCharacter0xdd15': '\xed\xb4\x95',
'OriyaLetterCa': '\xe0\xac\x9a',
'UnicodeCharacter0xf702': '\xef\x9c\x82',
'CjkUnifiedIdeograph692e': '\xe6\xa4\xae',
'EthiopicSyllableCchaa': '\xe2\xb6\xbb',
'GreekLetterYot': '\xcf\xb3',
'CjkUnifiedIdeograph7dfb': '\xe7\xb7\xbb',
'RightwardsArrowFromBar': '\xe2\x86\xa6',
'UnicodeCharacter0xec14': '\xee\xb0\x94',
'UnicodeCharacter0xe902': '\xee\xa4\x82',
'UnicodeCharacter0xe903': '\xee\xa4\x83',
'UnicodeCharacter0xe900': '\xee\xa4\x80',
'UnicodeCharacter0xe901': '\xee\xa4\x81',
'UnicodeCharacter0xe906': '\xee\xa4\x86',
'UnicodeCharacter0xe907': '\xee\xa4\x87',
'UnicodeCharacter0xe904': '\xee\xa4\x84',
'UnicodeCharacter0xe905': '\xee\xa4\x85',
'UnicodeCharacter0xe908': '\xee\xa4\x88',
'UnicodeCharacter0xe116': '\xee\x84\x96',
'UnicodeCharacter0xdd8f': '\xed\xb6\x8f',
'HangulSyllableRyaels': '\xeb\x9f\x9c',
'HangulSyllableRyaelp': '\xeb\x9f\x9e',
'UnicodeCharacter0xec13': '\xee\xb0\x93',
'HangulSyllableRyaelt': '\xeb\x9f\x9d',
'HangulSyllableRyaelh': '\xeb\x9f\x9f',
'KhmerSymbolDapmuoyRoc': '\xe1\xa7\xbb',
'HangulSyllableRyaelm': '\xeb\x9f\x9a',
'HangulSyllableRyaelb': '\xeb\x9f\x9b',
'ShortUpTack': '\xe2\xab\xa0',
'MongolianLetterAliGaliVisargaOne': '\xe1\xa2\x81',
'HangulSyllableRyaelg': '\xeb\x9f\x99',
'CjkUnifiedIdeograph84d6': '\xe8\x93\x96',
'CjkUnifiedIdeograph84d7': '\xe8\x93\x97',
'CjkUnifiedIdeograph6c09': '\xe6\xb0\x89',
'CjkUnifiedIdeograph6c08': '\xe6\xb0\x88',
'CjkUnifiedIdeograph84d2': '\xe8\x93\x92',
'CjkUnifiedIdeograph84d3': '\xe8\x93\x93',
'RightAngleSubstitutionMarker': '\xe2\xb8\x80',
'CjkUnifiedIdeograph84d1': '\xe8\x93\x91',
'CjkUnifiedIdeograph6c03': '\xe6\xb0\x83',
'CjkUnifiedIdeograph6c02': '\xe6\xb0\x82',
'CjkUnifiedIdeograph6c01': '\xe6\xb0\x81',
'CjkUnifiedIdeograph6c00': '\xe6\xb0\x80',
'CjkUnifiedIdeograph6c07': '\xe6\xb0\x87',
'CjkUnifiedIdeograph6c06': '\xe6\xb0\x86',
'CjkUnifiedIdeograph6c05': '\xe6\xb0\x85',
'CjkUnifiedIdeograph6c04': '\xe6\xb0\x84',
'UnicodeCharacter0xfafc': '\xef\xab\xbc',
'LeftwardsTwoHeadedArrow': '\xe2\x86\x9e',
'CjkRadicalTiger': '\xe2\xbb\x81',
'UnicodeCharacter0xe7d9': '\xee\x9f\x99',
'UnicodeCharacter0xec11': '\xee\xb0\x91',
'OriyaLetterIi': '\xe0\xac\x88',
'HalfwidthKatakanaLetterKa': '\xef\xbd\xb6',
'HangulSyllableMweoss': '\xeb\xad\xa4',
'YiSyllableChop': '\xea\x8d\xac',
'CjkUnifiedIdeograph8734': '\xe8\x9c\xb4',
'CjkUnifiedIdeograph8d76': '\xe8\xb5\xb6',
'YiSyllableChox': '\xea\x8d\xaa',
'CjkUnifiedIdeograph73c8': '\xe7\x8f\x88',
'LatinSmallLetterNj': '\xc7\x8c',
'NarySummation': '\xe2\x88\x91',
'HalfwidthHangulLetterRieulhieuh': '\xef\xbe\xb0',
'ParenthesizedNumberTwenty': '\xe2\x92\x87',
'HangulSyllableSsyu': '\xec\x93\x94',
'CjkUnifiedIdeograph84df': '\xe8\x93\x9f',
'CjkUnifiedIdeograph7f41': '\xe7\xbd\x81',
'CjkUnifiedIdeograph84dd': '\xe8\x93\x9d',
'CjkUnifiedIdeograph84de': '\xe8\x93\x9e',
'CjkUnifiedIdeograph84db': '\xe8\x93\x9b',
'CjkUnifiedIdeograph84dc': '\xe8\x93\x9c',
'CjkUnifiedIdeograph7f40': '\xe7\xbd\x80',
'CjkUnifiedIdeograph6c0c': '\xe6\xb0\x8c',
'CjkUnifiedIdeograph64f6': '\xe6\x93\xb6',
'CjkUnifiedIdeograph6c0a': '\xe6\xb0\x8a',
'CjkUnifiedIdeograph8d70': '\xe8\xb5\xb0',
'CjkUnifiedIdeograph6c0f': '\xe6\xb0\x8f',
'CjkUnifiedIdeograph6c0e': '\xe6\xb0\x8e',
'CjkUnifiedIdeograph6c0d': '\xe6\xb0\x8d',
'CjkUnifiedIdeograph7f42': '\xe7\xbd\x82',
'UnicodeCharacter0xf5fc': '\xef\x97\xbc',
'CjkUnifiedIdeograph4c18': '\xe4\xb0\x98',
'CjkUnifiedIdeograph4c19': '\xe4\xb0\x99',
'CjkUnifiedIdeograph4c16': '\xe4\xb0\x96',
'CjkUnifiedIdeograph4c17': '\xe4\xb0\x97',
'CjkUnifiedIdeograph4c14': '\xe4\xb0\x94',
'CjkUnifiedIdeograph4c15': '\xe4\xb0\x95',
'CjkUnifiedIdeograph4c12': '\xe4\xb0\x92',
'CjkUnifiedIdeograph4c13': '\xe4\xb0\x93',
'CjkUnifiedIdeograph4c10': '\xe4\xb0\x90',
'LatinCapitalLetterOWithHornAndDotBelow': '\xe1\xbb\xa2',
'UnicodeCharacter0xf03a': '\xef\x80\xba',
'CjkUnifiedIdeograph73c3': '\xe7\x8f\x83',
'YiSyllableSix': '\xea\x8c\x8a',
'CjkUnifiedIdeograph7f46': '\xe7\xbd\x86',
'YiSyllableSip': '\xea\x8c\x8c',
'UnicodeCharacter0xf5fb': '\xef\x97\xbb',
'UnicodeCharacter0xdf1d': '\xed\xbc\x9d',
'HangulSyllableBugg': '\xeb\xb6\x82',
'ArabicLetterRehWithDotBelow': '\xda\x94',
'CjkUnifiedIdeograph87b7': '\xe8\x9e\xb7',
'CjkUnifiedIdeograph87b6': '\xe8\x9e\xb6',
'CjkUnifiedIdeograph87b5': '\xe8\x9e\xb5',
'CjkUnifiedIdeograph87b4': '\xe8\x9e\xb4',
'CjkUnifiedIdeograph87b3': '\xe8\x9e\xb3',
'CjkUnifiedIdeograph87b2': '\xe8\x9e\xb2',
'CjkUnifiedIdeograph87b1': '\xe8\x9e\xb1',
'CjkUnifiedIdeograph87b0': '\xe8\x9e\xb0',
'HangulSyllablePweolt': '\xed\x92\xa1',
'HangulSyllableBugs': '\xeb\xb6\x83',
'UnicodeCharacter0xe5cd': '\xee\x97\x8d',
'CjkUnifiedIdeograph87b9': '\xe8\x9e\xb9',
'CjkUnifiedIdeograph87b8': '\xe8\x9e\xb8',
'UnicodeCharacter0xf5fe': '\xef\x97\xbe',
'LatinCapitalLetterEWithDoubleGrave': '\xc8\x84',
'CjkUnifiedIdeograph4c1f': '\xe4\xb0\x9f',
'CjkUnifiedIdeograph4c1d': '\xe4\xb0\x9d',
'CjkUnifiedIdeograph4c1e': '\xe4\xb0\x9e',
'CjkUnifiedIdeograph4c1b': '\xe4\xb0\x9b',
'CjkUnifiedIdeograph4c1c': '\xe4\xb0\x9c',
'CjkUnifiedIdeograph4c1a': '\xe4\xb0\x9a',
'CjkUnifiedIdeograph87bf': '\xe8\x9e\xbf',
'CjkUnifiedIdeograph87be': '\xe8\x9e\xbe',
'CjkUnifiedIdeograph87bd': '\xe8\x9e\xbd',
'CjkUnifiedIdeograph87bc': '\xe8\x9e\xbc',
'CjkUnifiedIdeograph87bb': '\xe8\x9e\xbb',
'CopticCapitalLetterRo': '\xe2\xb2\xa2',
'PresentationFormForVerticalRightBlackLenticularBracket': '\xef\xb8\xbc',
'UnicodeCharacter0xf2a8': '\xef\x8a\xa8',
'UnicodeCharacter0xe5cb': '\xee\x97\x8b',
'UnicodeCharacter0xe11c': '\xee\x84\x9c',
'UnicodeCharacter0xe11b': '\xee\x84\x9b',
'Angle': '\xe2\x88\xa0',
'HangulSyllableNoet': '\xeb\x87\xa5',
'TibetanVowelSignUu': '\xe0\xbd\xb5',
'CircledDigitNine': '\xe2\x91\xa8',
'LatinSmallLetterLWithMiddleTilde': '\xc9\xab',
'UnicodeCharacter0xe11d': '\xee\x84\x9d',
'UnicodeCharacter0xe12d': '\xee\x84\xad',
'SaurashtraLetterDha': '\xea\xa2\xa4',
'CjkUnifiedIdeograph47b9': '\xe4\x9e\xb9',
'CjkUnifiedIdeograph47b8': '\xe4\x9e\xb8',
'UnicodeCharacter0xe12a': '\xee\x84\xaa',
'UnicodeCharacter0xe12b': '\xee\x84\xab',
'UnicodeCharacter0xe12c': '\xee\x84\xac',
'CjkUnifiedIdeograph47b3': '\xe4\x9e\xb3',
'CjkUnifiedIdeograph47b2': '\xe4\x9e\xb2',
'CjkUnifiedIdeograph47b1': '\xe4\x9e\xb1',
'CjkUnifiedIdeograph47b0': '\xe4\x9e\xb0',
'CjkUnifiedIdeograph47b7': '\xe4\x9e\xb7',
'CjkUnifiedIdeograph47b6': '\xe4\x9e\xb6',
'CjkUnifiedIdeograph47b5': '\xe4\x9e\xb5',
'CjkUnifiedIdeograph47b4': '\xe4\x9e\xb4',
'UnicodeCharacter0xa63d': '\xea\x98\xbd',
'UnicodeCharacter0xa63e': '\xea\x98\xbe',
'UnicodeCharacter0xa63f': '\xea\x98\xbf',
'HangulSyllableNoes': '\xeb\x87\x9f',
'UnicodeCharacter0xa63a': '\xea\x98\xba',
'UnicodeCharacter0xa63b': '\xea\x98\xbb',
'UnicodeCharacter0xa63c': '\xea\x98\xbc',
'UnicodeCharacter0x1bbf': '\xe1\xae\xbf',
'UnicodeCharacter0x1bbe': '\xe1\xae\xbe',
'UnicodeCharacter0x1bbd': '\xe1\xae\xbd',
'UnicodeCharacter0x1bbc': '\xe1\xae\xbc',
'UnicodeCharacter0x1bbb': '\xe1\xae\xbb',
'UnicodeCharacter0x1bba': '\xe1\xae\xba',
'UnicodeCharacter0xe292': '\xee\x8a\x92',
'UnicodeCharacter0xe293': '\xee\x8a\x93',
'UnicodeCharacter0xe290': '\xee\x8a\x90',
'UnicodeCharacter0xe291': '\xee\x8a\x91',
'UnicodeCharacter0xe296': '\xee\x8a\x96',
'UnicodeCharacter0xe297': '\xee\x8a\x97',
'UnicodeCharacter0xe294': '\xee\x8a\x94',
'UnicodeCharacter0xe295': '\xee\x8a\x95',
'UnicodeCharacter0xe124': '\xee\x84\xa4',
'UnicodeCharacter0xe125': '\xee\x84\xa5',
'UnicodeCharacter0xe126': '\xee\x84\xa6',
'UnicodeCharacter0xe127': '\xee\x84\xa7',
'UnicodeCharacter0xe120': '\xee\x84\xa0',
'UnicodeCharacter0xe121': '\xee\x84\xa1',
'UnicodeCharacter0xe122': '\xee\x84\xa2',
'UnicodeCharacter0xe123': '\xee\x84\xa3',
'CjkUnifiedIdeograph47bc': '\xe4\x9e\xbc',
'CjkUnifiedIdeograph47bb': '\xe4\x9e\xbb',
'CjkUnifiedIdeograph47ba': '\xe4\x9e\xba',
'UnicodeCharacter0xe128': '\xee\x84\xa8',
'CjkUnifiedIdeograph47bf': '\xe4\x9e\xbf',
'CjkUnifiedIdeograph47be': '\xe4\x9e\xbe',
'CjkUnifiedIdeograph47bd': '\xe4\x9e\xbd',
'CjkUnifiedIdeograph5abe': '\xe5\xaa\xbe',
'IdeographicTelegraphSymbolForAugust': '\xe3\x8b\x87',
'HangulChoseongSiosssangsios': '\xe1\x84\xb4',
'KelvinSign': '\xe2\x84\xaa',
'CopticSmallLetterOldCopticAin': '\xe2\xb2\xb5',
'CjkUnifiedIdeograph9439': '\xe9\x90\xb9',
'UnicodeCharacter0xe29b': '\xee\x8a\x9b',
'HangulSyllableCwebs': '\xec\xb7\x9e',
'UnicodeCharacter0xe29a': '\xee\x8a\x9a',
'UnicodeCharacter0xe29f': '\xee\x8a\x9f',
'UnicodeCharacter0xe29d': '\xee\x8a\x9d',
'UnicodeCharacter0xe29e': '\xee\x8a\x9e',
'UnicodeCharacter0xa634': '\xea\x98\xb4',
'UnicodeCharacter0xa635': '\xea\x98\xb5',
'UnicodeCharacter0xa636': '\xea\x98\xb6',
'UnicodeCharacter0xa637': '\xea\x98\xb7',
'UnicodeCharacter0xa630': '\xea\x98\xb0',
'UnicodeCharacter0xa631': '\xea\x98\xb1',
'UnicodeCharacter0xa632': '\xea\x98\xb2',
'UnicodeCharacter0xa633': '\xea\x98\xb3',
'CjkUnifiedIdeograph8d5a': '\xe8\xb5\x9a',
'UnicodeCharacter0xa638': '\xea\x98\xb8',
'UnicodeCharacter0xa639': '\xea\x98\xb9',
'HangulSyllableNoep': '\xeb\x87\xa6',
'YiSyllableVop': '\xea\x83\xb1',
'YiSyllableVot': '\xea\x83\xae',
'YiSyllableVox': '\xea\x83\xaf',
'CjkCompatibilityIdeographfa3a': '\xef\xa8\xba',
'HangulSyllablePok': '\xed\x90\x84',
'SyriacLetterYudhHe': '\xdc\x9e',
'ThereDoesNotExist': '\xe2\x88\x84',
'HangulSyllableSoeng': '\xec\x87\xb5',
'UnicodeCharacter0xf34e': '\xef\x8d\x8e',
'GreekSmallLetterUpsilonWithDialytikaAndOxia': '\xe1\xbf\xa3',
'YiRadicalNyip': '\xea\x92\x93',
'UnicodeCharacter0xdddd': '\xed\xb7\x9d',
'UnicodeCharacter0xddde': '\xed\xb7\x9e',
'UnicodeCharacter0xdddf': '\xed\xb7\x9f',
'UnicodeCharacter0xddda': '\xed\xb7\x9a',
'FourClubspokedAsterisk': '\xe2\x9c\xa5',
'UnicodeCharacter0xdddc': '\xed\xb7\x9c',
'UnicodeCharacter0xebf7': '\xee\xaf\xb7',
'CjkUnifiedIdeograph903d': '\xe9\x80\xbd',
'UnicodeCharacter0xebf5': '\xee\xaf\xb5',
'UnicodeCharacter0xebf4': '\xee\xaf\xb4',
'BoxDrawingsLightDiagonalCross': '\xe2\x95\xb3',
'UnicodeCharacter0xebf2': '\xee\xaf\xb2',
'UnicodeCharacter0xebf1': '\xee\xaf\xb1',
'CjkUnifiedIdeograph903e': '\xe9\x80\xbe',
'UnicodeCharacter0xf34a': '\xef\x8d\x8a',
'HangulSyllableHwi': '\xed\x9c\x98',
'UnicodeCharacter0xebf9': '\xee\xaf\xb9',
'UnicodeCharacter0xebf8': '\xee\xaf\xb8',
'CjkUnifiedIdeograph602a': '\xe6\x80\xaa',
'ModifierLetterSmallTheta': '\xe1\xb6\xbf',
'HexagramForGrace': '\xe4\xb7\x95',
'DoesNotContainAsNormalSubgroupOrEqual': '\xe2\x8b\xad',
'HangulSyllableTyelp': '\xed\x86\x92',
'UnicodeCharacter0xf672': '\xef\x99\xb2',
'HangulSyllableTyels': '\xed\x86\x90',
'HangulSyllableTyelm': '\xed\x86\x8e',
'HangulSyllableTyelh': '\xed\x86\x93',
'Bullseye': '\xe2\x97\x8e',
'CjkUnifiedIdeograph903c': '\xe9\x80\xbc',
'HangulSyllableTyelb': '\xed\x86\x8f',
'UnicodeCharacter0xaa64': '\xea\xa9\xa4',
'UnicodeCharacter0xebff': '\xee\xaf\xbf',
'UnicodeCharacter0xebfe': '\xee\xaf\xbe',
'Because': '\xe2\x88\xb5',
'UnicodeCharacter0xebfc': '\xee\xaf\xbc',
'UnicodeCharacter0xebfb': '\xee\xaf\xbb',
'UnicodeCharacter0xebfa': '\xee\xaf\xba',
'YiSyllableSsu': '\xea\x8c\xbe',
'CjkUnifiedIdeograph9bcb': '\xe9\xaf\x8b',
'UnicodeCharacter0xddd8': '\xed\xb7\x98',
'UnicodeCharacter0xddd9': '\xed\xb7\x99',
'CjkUnifiedIdeograph48cb': '\xe4\xa3\x8b',
'UnicodeCharacter0xddd4': '\xed\xb7\x94',
'UnicodeCharacter0xddd5': '\xed\xb7\x95',
'CjkUnifiedIdeograph903b': '\xe9\x80\xbb',
'UnicodeCharacter0xddd7': '\xed\xb7\x97',
'UnicodeCharacter0xddd0': '\xed\xb7\x90',
'UnicodeCharacter0xddd1': '\xed\xb7\x91',
'UnicodeCharacter0xaa65': '\xea\xa9\xa5',
'UnicodeCharacter0xddd3': '\xed\xb7\x93',
'UpwardsDashedArrow': '\xe2\x87\xa1',
'HangulSyllableJjwelp': '\xec\xae\x8e',
'MyanmarVowelSignKayahEe': '\xe1\x81\xb4',
'HangulSyllableJjwelt': '\xec\xae\x8d',
'HangulLetterU': '\xe3\x85\x9c',
'ArabicLetterAinIsolatedForm': '\xef\xbb\x89',
'CjkUnifiedIdeograph9009': '\xe9\x80\x89',
'CjkUnifiedIdeograph9008': '\xe9\x80\x88',
'HangulLetterO': '\xe3\x85\x97',
'HangulSyllableJjwelg': '\xec\xae\x89',
'HangulLetterI': '\xe3\x85\xa3',
'EthiopicSyllableChee': '\xe1\x8c\xac',
'HangulLetterE': '\xe3\x85\x94',
'MongolianLetterRa': '\xe1\xa0\xb7',
'CjkUnifiedIdeograph9002': '\xe9\x80\x82',
'CjkUnifiedIdeograph9005': '\xe9\x80\x85',
'HangulLetterA': '\xe3\x85\x8f',
'CjkUnifiedIdeograph9007': '\xe9\x80\x87',
'CjkUnifiedIdeograph9006': '\xe9\x80\x86',
'UnicodeCharacter0xe63f': '\xee\x98\xbf',
'RightwardsDoubleArrowWithVerticalStroke': '\xe2\xa4\x83',
'LatinSmallLetterAWithGrave': '\xc3\xa0',
'CjkUnifiedIdeograph9aee': '\xe9\xab\xae',
'CjkUnifiedIdeograph9aed': '\xe9\xab\xad',
'CjkUnifiedIdeograph9aec': '\xe9\xab\xac',
'CanadianSyllabicsYcreePoo': '\xe1\x90\xb5',
'CjkUnifiedIdeograph9aea': '\xe9\xab\xaa',
'CjkUnifiedIdeograph7a8d': '\xe7\xaa\x8d',
'HangulSyllableJjeugg': '\xec\xaf\x96',
'CjkUnifiedIdeograph66aa': '\xe6\x9a\xaa',
'CjkUnifiedIdeograph66ac': '\xe6\x9a\xac',
'CjkUnifiedIdeograph66ab': '\xe6\x9a\xab',
'CjkUnifiedIdeograph66ae': '\xe6\x9a\xae',
'CjkUnifiedIdeograph66ad': '\xe6\x9a\xad',
'CjkUnifiedIdeograph66af': '\xe6\x9a\xaf',
'UnicodeCharacter0xe0b9': '\xee\x82\xb9',
'UnicodeCharacter0xe0b8': '\xee\x82\xb8',
'CjkUnifiedIdeograph900a': '\xe9\x80\x8a',
'TifinaghLetterYakh': '\xe2\xb5\x85',
'CjkUnifiedIdeograph900c': '\xe9\x80\x8c',
'CjkUnifiedIdeograph900b': '\xe9\x80\x8b',
'CjkUnifiedIdeograph900e': '\xe9\x80\x8e',
'CjkUnifiedIdeograph900d': '\xe9\x80\x8d',
'UnicodeCharacter0xe0b7': '\xee\x82\xb7',
'CjkUnifiedIdeograph900f': '\xe9\x80\x8f',
'CjkUnifiedIdeograph7f4a': '\xe7\xbd\x8a',
'UnicodeCharacter0xe7da': '\xee\x9f\x9a',
'HeavyBlackHeart': '\xe2\x9d\xa4',
'HangulSyllableJjwaegg': '\xec\xab\xb6',
'UnicodeCharacter0xe7db': '\xee\x9f\x9b',
'UnicodeCharacter0xa9fe': '\xea\xa7\xbe',
'HangulSyllableEolp': '\xec\x97\x82',
'LaoLetterThoTam': '\xe0\xba\x97',
'CjkUnifiedIdeograph66a8': '\xe6\x9a\xa8',
'CjkUnifiedIdeograph7f4b': '\xe7\xbd\x8b',
'HeavyRoundtippedRightwardsArrow': '\xe2\x9e\x9c',
'HangulSyllableMyinj': '\xeb\xaf\xa1',
'CjkUnifiedIdeograph66a1': '\xe6\x9a\xa1',
'CanadianSyllabicsHk': '\xe1\x95\xbd',
'CjkUnifiedIdeograph66a3': '\xe6\x9a\xa3',
'CjkUnifiedIdeograph66a2': '\xe6\x9a\xa2',
'CjkUnifiedIdeograph66a5': '\xe6\x9a\xa5',
'CjkUnifiedIdeograph66a4': '\xe6\x9a\xa4',
'CjkUnifiedIdeograph66a7': '\xe6\x9a\xa7',
'CjkUnifiedIdeograph66a6': '\xe6\x9a\xa6',
'HangulSyllableMyinh': '\xeb\xaf\xa2',
'KatakanaIterationMark': '\xe3\x83\xbd',
'CjkUnifiedIdeograph9ae9': '\xe9\xab\xa9',
'CjkUnifiedIdeograph9ae8': '\xe9\xab\xa8',
'CjkUnifiedIdeograph9ae7': '\xe9\xab\xa7',
'CjkUnifiedIdeograph9ae6': '\xe9\xab\xa6',
'CjkUnifiedIdeograph9ae5': '\xe9\xab\xa5',
'CjkUnifiedIdeograph7f4f': '\xe7\xbd\x8f',
'CjkUnifiedIdeograph9ae3': '\xe9\xab\xa3',
'CjkUnifiedIdeograph9ae2': '\xe9\xab\xa2',
'CjkUnifiedIdeograph9ae1': '\xe9\xab\xa1',
'CjkUnifiedIdeograph9ae0': '\xe9\xab\xa0',
'CjkUnifiedIdeograph709a': '\xe7\x82\x9a',
'CjkUnifiedIdeograph709b': '\xe7\x82\x9b',
'CjkUnifiedIdeograph709c': '\xe7\x82\x9c',
'CjkUnifiedIdeograph709d': '\xe7\x82\x9d',
'CjkUnifiedIdeograph709e': '\xe7\x82\x9e',
'CjkUnifiedIdeograph709f': '\xe7\x82\x9f',
'UnicodeCharacter0xdcc7': '\xed\xb3\x87',
'CjkUnifiedIdeograph7366': '\xe7\x8d\xa6',
'CjkUnifiedIdeograph378a': '\xe3\x9e\x8a',
'CjkUnifiedIdeograph378b': '\xe3\x9e\x8b',
'CjkUnifiedIdeograph378c': '\xe3\x9e\x8c',
'CjkUnifiedIdeograph378d': '\xe3\x9e\x8d',
'CjkUnifiedIdeograph378e': '\xe3\x9e\x8e',
'CjkUnifiedIdeograph378f': '\xe3\x9e\x8f',
'CjkUnifiedIdeograph7361': '\xe7\x8d\xa1',
'CjkUnifiedIdeograph5f58': '\xe5\xbd\x98',
'CjkUnifiedIdeograph8d1d': '\xe8\xb4\x9d',
'CjkUnifiedIdeograph7368': '\xe7\x8d\xa8',
'CjkUnifiedIdeograph5f59': '\xe5\xbd\x99',
'CjkUnifiedIdeograph4558': '\xe4\x95\x98',
'UnicodeCharacter0x173d': '\xe1\x9c\xbd',
'UnicodeCharacter0x173f': '\xe1\x9c\xbf',
'UnicodeCharacter0x173a': '\xe1\x9c\xba',
'UnicodeCharacter0x173c': '\xe1\x9c\xbc',
'UnicodeCharacter0x173b': '\xe1\x9c\xbb',
'CjkUnifiedIdeograph4550': '\xe4\x95\x90',
'CjkUnifiedIdeograph4551': '\xe4\x95\x91',
'CjkUnifiedIdeograph4552': '\xe4\x95\x92',
'OghamLetterIodhadh': '\xe1\x9a\x94',
'CjkUnifiedIdeograph4554': '\xe4\x95\x94',
'CjkUnifiedIdeograph4555': '\xe4\x95\x95',
'CjkUnifiedIdeograph4556': '\xe4\x95\x96',
'CjkUnifiedIdeograph4557': '\xe4\x95\x97',
'CjkUnifiedIdeograph8f57': '\xe8\xbd\x97',
'CjkUnifiedIdeograph8f56': '\xe8\xbd\x96',
'CjkUnifiedIdeograph8f55': '\xe8\xbd\x95',
'CjkUnifiedIdeograph8f54': '\xe8\xbd\x94',
'CjkUnifiedIdeograph8f53': '\xe8\xbd\x93',
'CjkUnifiedIdeograph8f52': '\xe8\xbd\x92',
'CjkUnifiedIdeograph8f51': '\xe8\xbd\x91',
'CjkUnifiedIdeograph5f55': '\xe5\xbd\x95',
'CjkUnifiedIdeograph8f59': '\xe8\xbd\x99',
'CjkUnifiedIdeograph8f58': '\xe8\xbd\x98',
'CjkUnifiedIdeograph3780': '\xe3\x9e\x80',
'CjkUnifiedIdeograph3781': '\xe3\x9e\x81',
'CopticCapitalLetterOldCopticAin': '\xe2\xb2\xb4',
'CjkUnifiedIdeograph3783': '\xe3\x9e\x83',
'CjkUnifiedIdeograph3784': '\xe3\x9e\x84',
'CjkUnifiedIdeograph3785': '\xe3\x9e\x85',
'CjkUnifiedIdeograph3786': '\xe3\x9e\x86',
'CjkUnifiedIdeograph3787': '\xe3\x9e\x87',
'CjkUnifiedIdeograph3788': '\xe3\x9e\x88',
'CjkUnifiedIdeograph3789': '\xe3\x9e\x89',
'HangulSyllableByebs': '\xeb\xb3\xaa',
'GreekCapitalLetterBeta': '\xce\x92',
'HangulSyllableBoegg': '\xeb\xb5\x8a',
'CjkUnifiedIdeograph7090': '\xe7\x82\x90',
'CjkUnifiedIdeograph7091': '\xe7\x82\x91',
'CjkUnifiedIdeograph7092': '\xe7\x82\x92',
'CjkUnifiedIdeograph7093': '\xe7\x82\x93',
'CjkUnifiedIdeograph7094': '\xe7\x82\x94',
'CjkUnifiedIdeograph7095': '\xe7\x82\x95',
'CjkUnifiedIdeograph7096': '\xe7\x82\x96',
'CjkUnifiedIdeograph7097': '\xe7\x82\x97',
'CjkUnifiedIdeograph7098': '\xe7\x82\x98',
'ArabicLetterSeenWithInvertedV': '\xdd\xbe',
'HangulSyllableBoegs': '\xeb\xb5\x8b',
'SquareRadOverS': '\xe3\x8e\xae',
'UnicodeCharacter0xf340': '\xef\x8d\x80',
'CjkUnifiedIdeograph8f5f': '\xe8\xbd\x9f',
'CjkUnifiedIdeograph8f5e': '\xe8\xbd\x9e',
'CjkUnifiedIdeograph8f5d': '\xe8\xbd\x9d',
'CjkUnifiedIdeograph8f5c': '\xe8\xbd\x9c',
'CjkUnifiedIdeograph8f5b': '\xe8\xbd\x9b',
'CjkUnifiedIdeograph8f5a': '\xe8\xbd\x9a',
'ArabicLigatureAlefMaksuraWithSuperscriptAlefIsolatedForm': '\xef\xb1\x9d',
'UnicodeCharacter0x1737': '\xe1\x9c\xb7',
'CjkUnifiedIdeograph455a': '\xe4\x95\x9a',
'CjkUnifiedIdeograph455b': '\xe4\x95\x9b',
'CjkUnifiedIdeograph455c': '\xe4\x95\x9c',
'UnicodeCharacter0x1739': '\xe1\x9c\xb9',
'CjkUnifiedIdeograph455e': '\xe4\x95\x9e',
'CjkUnifiedIdeograph455f': '\xe4\x95\x9f',
'EthiopicSyllableQhee': '\xe1\x89\x94',
'UnicodeCharacter0xf8e7': '\xef\xa3\xa7',
'UnicodeCharacter0xf349': '\xef\x8d\x89',
'QuadruplePrime': '\xe2\x81\x97',
'UnicodeCharacter0xf348': '\xef\x8d\x88',
'HangulChoseongYesieung': '\xe1\x85\x8c',
'HebrewLetterShinWithShinDot': '\xef\xac\xaa',
'CjkUnifiedIdeograph5207': '\xe5\x88\x87',
'CjkUnifiedIdeograph5206': '\xe5\x88\x86',
'CjkUnifiedIdeograph5205': '\xe5\x88\x85',
'CjkUnifiedIdeograph5204': '\xe5\x88\x84',
'CjkUnifiedIdeograph5203': '\xe5\x88\x83',
'CjkUnifiedIdeograph5202': '\xe5\x88\x82',
'CjkUnifiedIdeograph5201': '\xe5\x88\x81',
'CjkUnifiedIdeograph5200': '\xe5\x88\x80',
'CjkUnifiedIdeograph657d': '\xe6\x95\xbd',
'CjkUnifiedIdeograph657e': '\xe6\x95\xbe',
'AlternativeKeySymbol': '\xe2\x8e\x87',
'HangulSyllableKyoc': '\xec\xbf\x9b',
'HangulSyllableKyod': '\xec\xbf\x8b',
'CjkUnifiedIdeograph657a': '\xe6\x95\xba',
'CjkUnifiedIdeograph5209': '\xe5\x88\x89',
'AplFunctionalSymbolQuadUpwardsArrow': '\xe2\x8d\x90',
'ParenthesizedHangulMieum': '\xe3\x88\x84',
'YiSyllableChot': '\xea\x8d\xa9',
'HangulSyllableKyop': '\xec\xbf\x9e',
'UnicodeCharacter0xf4ec': '\xef\x93\xac',
'HangulSyllableKyos': '\xec\xbf\x97',
'HangulSyllableKyot': '\xec\xbf\x9d',
'UnicodeCharacter0xf4ed': '\xef\x93\xad',
'UnicodeCharacter0xf4ef': '\xef\x93\xaf',
'infinity': '\xe2\x88\x9e',
'UnicodeCharacter0xf5ff': '\xef\x97\xbf',
'HangulSyllableSsolp': '\xec\x8f\xa6',
'EqualsSignWithTwoDotsAboveAndTwoDotsBelow': '\xe2\xa9\xb7',
'CjkUnifiedIdeograph7a85': '\xe7\xaa\x85',
'UnicodeCharacter0xe30d': '\xee\x8c\x8d',
'UnicodeCharacter0xf4e9': '\xef\x93\xa9',
'UnicodeCharacter0xf4e8': '\xef\x93\xa8',
'CjkUnifiedIdeograph9ccb': '\xe9\xb3\x8b',
'UnicodeCharacter0xaa69': '\xea\xa9\xa9',
'UnicodeCharacter0xf4e1': '\xef\x93\xa1',
'UnicodeCharacter0xe7cc': '\xee\x9f\x8c',
'UnicodeCharacter0xf4e3': '\xef\x93\xa3',
'UnicodeCharacter0xf4e2': '\xef\x93\xa2',
'UnicodeCharacter0xf4e5': '\xef\x93\xa5',
'CjkUnifiedIdeograph6a6a': '\xe6\xa9\xaa',
'UnicodeCharacter0xf4e7': '\xef\x93\xa7',
'UnicodeCharacter0xf4e6': '\xef\x93\xa6',
'CjkUnifiedIdeograph520f': '\xe5\x88\x8f',
'CjkUnifiedIdeograph520e': '\xe5\x88\x8e',
'CjkUnifiedIdeograph520d': '\xe5\x88\x8d',
'CjkUnifiedIdeograph520c': '\xe5\x88\x8c',
'CjkUnifiedIdeograph520b': '\xe5\x88\x8b',
'CjkUnifiedIdeograph520a': '\xe5\x88\x8a',
'CjkUnifiedIdeograph6574': '\xe6\x95\xb4',
'CjkUnifiedIdeograph6575': '\xe6\x95\xb5',
'CjkUnifiedIdeograph6576': '\xe6\x95\xb6',
'LatinSmallLetterUe': '\xe1\xb5\xab',
'CjkUnifiedIdeograph6570': '\xe6\x95\xb0',
'CjkUnifiedIdeograph6571': '\xe6\x95\xb1',
'CjkUnifiedIdeograph6572': '\xe6\x95\xb2',
'CjkUnifiedIdeograph6573': '\xe6\x95\xb3',
'HangulSyllableBbin': '\xec\x82\x94',
'HangulSyllableBbil': '\xec\x82\x98',
'HangulSyllableBbim': '\xec\x82\xa0',
'HangulSyllableBbij': '\xec\x82\xa6',
'HangulSyllableBbik': '\xec\x82\xa8',
'HangulSyllableBbih': '\xec\x82\xab',
'CjkUnifiedIdeograph3c1f': '\xe3\xb0\x9f',
'CjkUnifiedIdeograph3c1e': '\xe3\xb0\x9e',
'CjkUnifiedIdeograph3c1d': '\xe3\xb0\x9d',
'CjkUnifiedIdeograph3c1c': '\xe3\xb0\x9c',
'CjkUnifiedIdeograph3c1b': '\xe3\xb0\x9b',
'CjkUnifiedIdeograph3c1a': '\xe3\xb0\x9a',
'UnicodeCharacter0xde03': '\xed\xb8\x83',
'CjkUnifiedIdeograph88f0': '\xe8\xa3\xb0',
'CjkUnifiedIdeograph6022': '\xe6\x80\xa2',
'CjkUnifiedIdeograph88f2': '\xe8\xa3\xb2',
'CjkUnifiedIdeograph88f3': '\xe8\xa3\xb3',
'CyrillicCapitalLetterKsi': '\xd1\xae',
'CjkUnifiedIdeograph88f5': '\xe8\xa3\xb5',
'CjkUnifiedIdeograph88f6': '\xe8\xa3\xb6',
'CjkUnifiedIdeograph88f7': '\xe8\xa3\xb7',
'CjkUnifiedIdeograph88f8': '\xe8\xa3\xb8',
'CjkUnifiedIdeograph88f9': '\xe8\xa3\xb9',
'CombiningRightHarpoonAbove': '\xe2\x83\x91',
'HangulSyllableYigs': '\xec\x9d\x9b',
'CjkUnifiedIdeograph9036': '\xe9\x80\xb6',
'HangulSyllableBbip': '\xec\x82\xaa',
'YiSyllableZhy': '\xea\x8d\x9d',
'CjkUnifiedIdeograph9037': '\xe9\x80\xb7',
'CjkUnifiedIdeograph9030': '\xe9\x80\xb0',
'YiSyllableZhu': '\xea\x8d\x97',
'CjkUnifiedIdeograph9031': '\xe9\x80\xb1',
'CjkUnifiedIdeograph9032': '\xe9\x80\xb2',
'YiSyllableZho': '\xea\x8d\x8f',
'YiSyllableZha': '\xea\x8d\x88',
'HangulSyllableIss': '\xec\x9e\x88',
'CjkUnifiedIdeograph9033': '\xe9\x80\xb3',
'YiSyllableZhe': '\xea\x8d\x93',
'HangulSyllableBeonh': '\xeb\xb2\x8a',
'CjkUnifiedIdeograph88fa': '\xe8\xa3\xba',
'LeftFloor': '\xe2\x8c\x8a',
'EthiopicSyllableSaa': '\xe1\x88\xb3',
'CjkUnifiedIdeograph88fd': '\xe8\xa3\xbd',
'CjkUnifiedIdeograph88fe': '\xe8\xa3\xbe',
'CjkUnifiedIdeograph88ff': '\xe8\xa3\xbf',
'HangulSyllableWeg': '\xec\x9b\xa9',
'HebrewPunctuationGershayim': '\xd7\xb4',
'HangulSyllableWej': '\xec\x9b\xbe',
'HangulSyllableWek': '\xec\x9c\x80',
'HangulSyllableWel': '\xec\x9b\xb0',
'HangulSyllableWem': '\xec\x9b\xb8',
'HangulSyllableWen': '\xec\x9b\xac',
'UnicodeCharacter0xab17': '\xea\xac\x97',
'HangulSyllableWep': '\xec\x9c\x82',
'HangulSyllableCyi': '\xec\xb8\xbc',
'HangulSyllableWes': '\xec\x9b\xbb',
'HangulSyllableWet': '\xec\x9c\x81',
'CjkUnifiedIdeograph3c19': '\xe3\xb0\x99',
'CjkUnifiedIdeograph3c18': '\xe3\xb0\x98',
'CjkUnifiedIdeograph3c17': '\xe3\xb0\x97',
'CjkUnifiedIdeograph3c16': '\xe3\xb0\x96',
'CjkUnifiedIdeograph3c15': '\xe3\xb0\x95',
'CjkUnifiedIdeograph3c14': '\xe3\xb0\x94',
'CjkUnifiedIdeograph3c13': '\xe3\xb0\x93',
'CjkUnifiedIdeograph3c12': '\xe3\xb0\x92',
'IdeographicTelegraphSymbolForDayFifteen': '\xe3\x8f\xae',
'CjkUnifiedIdeograph3c10': '\xe3\xb0\x90',
'UnicodeCharacter0xdca5': '\xed\xb2\xa5',
'HangulJungseongYae': '\xe1\x85\xa4',
'TeluguSignAvagraha': '\xe0\xb0\xbd',
'HangulSyllableCyaelb': '\xec\xb2\x87',
'HangulJungseongYao': '\xe1\x85\xb8',
'UnicodeCharacter0xe315': '\xee\x8c\x95',
'IdeographicTelegraphSymbolForDayTen': '\xe3\x8f\xa9',
'OriyaLetterLla': '\xe0\xac\xb3',
'HangulSyllableRweg': '\xeb\xa4\xa1',
'e': '\xcf\xb5',
'CjkUnifiedIdeograph957e': '\xe9\x95\xbe',
'CjkUnifiedIdeograph5bd8': '\xe5\xaf\x98',
'AsteriskOperator': '\xe2\x88\x97',
'ArabicLetterHehGoalInitialForm': '\xef\xae\xa8',
'HangulSyllableCyaelh': '\xec\xb2\x8b',
'MongolianLetterManchuAliGaliZa': '\xe1\xa2\xa5',
'CjkUnifiedIdeograph5985': '\xe5\xa6\x85',
'HangulSyllableTobs': '\xed\x86\xb2',
'CjkUnifiedIdeograph957d': '\xe9\x95\xbd',
'LeftwardsDoubleArrow': '\xe2\x87\x90',
'HexagramForTheJoyousLake': '\xe4\xb7\xb9',
'CyrillicSmallLetterBarredOWithDiaeresis': '\xd3\xab',
'UnicodeCharacter0xe4e7': '\xee\x93\xa7',
'CopticSmallLetterGangia': '\xcf\xab',
'LaoKoLa': '\xe0\xbb\x86',
'CjkUnifiedIdeograph6e4a': '\xe6\xb9\x8a',
'CjkUnifiedIdeograph6e4c': '\xe6\xb9\x8c',
'CjkUnifiedIdeograph6e4b': '\xe6\xb9\x8b',
'CjkUnifiedIdeograph6e4e': '\xe6\xb9\x8e',
'CjkUnifiedIdeograph6e4d': '\xe6\xb9\x8d',
'CjkUnifiedIdeograph6e4f': '\xe6\xb9\x8f',
'HangulSyllableEod': '\xec\x96\xbb',
'HangulSyllableEog': '\xec\x96\xb5',
'HangulSyllableEob': '\xec\x97\x85',
'HangulSyllableEoc': '\xec\x97\x8b',
'HangulSyllableEol': '\xec\x96\xbc',
'HangulSyllableEom': '\xec\x97\x84',
'HangulSyllableEon': '\xec\x96\xb8',
'HangulSyllableEoh': '\xec\x97\x8f',
'HangulSyllableEoj': '\xec\x97\x8a',
'HangulSyllableEok': '\xec\x97\x8c',
'HangulSyllableEot': '\xec\x97\x8d',
'ArabicLigatureLamWithJeemWithJeemFinalForm': '\xef\xb6\x84',
'HangulSyllableEop': '\xec\x97\x8e',
'HangulSyllableEos': '\xec\x97\x87',
'CjkUnifiedIdeograph6e49': '\xe6\xb9\x89',
'CjkUnifiedIdeograph6e48': '\xe6\xb9\x88',
'HangulSyllableHeobs': '\xed\x97\x9a',
'HangulSyllableTwaej': '\xed\x87\xae',
'CjkUnifiedIdeograph6e41': '\xe6\xb9\x81',
'CjkUnifiedIdeograph6e40': '\xe6\xb9\x80',
'ElementOfWithVerticalBarAtEndOfHorizontalStroke': '\xe2\x8b\xb3',
'CjkUnifiedIdeograph6e42': '\xe6\xb9\x82',
'CjkUnifiedIdeograph6e45': '\xe6\xb9\x85',
'FemaleSign': '\xe2\x99\x80',
'CjkUnifiedIdeograph6e47': '\xe6\xb9\x87',
'CjkUnifiedIdeograph6e46': '\xe6\xb9\x86',
'ArabicLigatureNoonWithNoonFinalForm': '\xef\xb2\x8d',
'ArabicLetterHahMedialForm': '\xef\xba\xa4',
'CopyrightSign': '\xc2\xa9',
'HorizontalBar': '\xe2\x80\x95',
'BlackLeftwardsBullet': '\xe2\x81\x8c',
'TibetanSignRdelNagGsum': '\xe0\xbf\x8f',
'KangxiRadicalRed': '\xe2\xbe\x9a',
'UnicodeCharacter0xf3c8': '\xef\x8f\x88',
'YiSyllableZhot': '\xea\x8d\x8d',
'YiSyllableZhop': '\xea\x8d\x90',
'LatinCapitalLetterEWithMacron': '\xc4\x92',
'UnicodeCharacter0xef3b': '\xee\xbc\xbb',
'CjkUnifiedIdeograph7b06': '\xe7\xac\x86',
'CjkUnifiedIdeograph4a5d': '\xe4\xa9\x9d',
'CjkUnifiedIdeograph4a5e': '\xe4\xa9\x9e',
'CjkUnifiedIdeograph4a5f': '\xe4\xa9\x9f',
'KhmerSymbolDapbuonKoet': '\xe1\xa7\xae',
'CjkUnifiedIdeograph4a5b': '\xe4\xa9\x9b',
'CjkUnifiedIdeograph4a5c': '\xe4\xa9\x9c',
'LatinCapitalLetterOWithHornAndAcute': '\xe1\xbb\x9a',
'HangulSyllableJjaess': '\xec\xa8\x8c',
'ArabicLigatureMeemWithJeemWithKhahInitialForm': '\xef\xb6\x92',
'CjkUnifiedIdeograph7e60': '\xe7\xb9\xa0',
'CjkUnifiedIdeograph7e61': '\xe7\xb9\xa1',
'CjkUnifiedIdeograph7e62': '\xe7\xb9\xa2',
'CjkUnifiedIdeograph7e63': '\xe7\xb9\xa3',
'ThaiCharacterPhoSamphao': '\xe0\xb8\xa0',
'UnicodeCharacter0x70e': '\xdc\x8e',
'CjkUnifiedIdeograph7e66': '\xe7\xb9\xa6',
'CjkUnifiedIdeograph7e67': '\xe7\xb9\xa7',
'CjkUnifiedIdeograph7e68': '\xe7\xb9\xa8',
'CjkUnifiedIdeograph7e69': '\xe7\xb9\xa9',
'UpwardsArrowWithTipLeftwards': '\xe2\x86\xb0',
'CjkUnifiedIdeograph5c89': '\xe5\xb2\x89',
'HangulSyllableMaenj': '\xeb\xa7\xa9',
'HangulSyllableMaenh': '\xeb\xa7\xaa',
'SupersetOfAboveAlmostEqualTo': '\xe2\xab\x8a',
'CanadianSyllabicsCarrierKko': '\xe1\x97\xbc',
'CjkUnifiedIdeograph4a58': '\xe4\xa9\x98',
'CanadianSyllabicsCarrierKki': '\xe1\x97\xbf',
'TamilLetterIi': '\xe0\xae\x88',
'CjkUnifiedIdeograph4a54': '\xe4\xa9\x94',
'CanadianSyllabicsCarrierKke': '\xe1\x97\xbd',
'CjkUnifiedIdeograph4a56': '\xe4\xa9\x96',
'CjkUnifiedIdeograph4a57': '\xe4\xa9\x97',
'CjkUnifiedIdeograph4a50': '\xe4\xa9\x90',
'CanadianSyllabicsCarrierKka': '\xe1\x98\x80',
'CjkUnifiedIdeograph4a52': '\xe4\xa9\x92',
'CjkUnifiedIdeograph4a53': '\xe4\xa9\x93',
'CjkUnifiedIdeograph957b': '\xe9\x95\xbb',
'ChamLetterTha': '\xea\xa8\x94',
'CanadianSyllabicsCarrierKku': '\xe1\x97\xbb',
'CjkUnifiedIdeograph7e6a': '\xe7\xb9\xaa',
'CjkUnifiedIdeograph7e6b': '\xe7\xb9\xab',
'CjkUnifiedIdeograph7e6c': '\xe7\xb9\xac',
'CjkUnifiedIdeograph7e6d': '\xe7\xb9\xad',
'CjkUnifiedIdeograph7e6e': '\xe7\xb9\xae',
'CjkUnifiedIdeograph7e6f': '\xe7\xb9\xaf',
'ArabicLigatureYehWithHamzaAboveWithHahIsolatedForm': '\xef\xb0\x81',
'GujaratiLetterHa': '\xe0\xaa\xb9',
'CopticCapitalLetterOldCopticDja': '\xe2\xb3\x98',
'CanadianSyllabicsWestcreeLwii': '\xe1\x93\xa1',
'LatinCapitalLetterVWithDiagonalStroke': '\xea\x9d\x9e',
'HangulSyllableTyabs': '\xed\x84\x8a',
'int': '\xe2\x88\xab',
'HangulSyllableDdweng': '\xeb\x9b\xa9',
'HangulSyllableHwelp': '\xed\x9c\x8a',
'LatinSmallLetterTurnedOpenE': '\xe1\xb4\x88',
'CjkUnifiedIdeograph4d86': '\xe4\xb6\x86',
'CjkUnifiedIdeograph4d87': '\xe4\xb6\x87',
'MongolianLetterTodoNia': '\xe1\xa1\x9b',
'CjkUnifiedIdeograph4d85': '\xe4\xb6\x85',
'CjkUnifiedIdeograph4d82': '\xe4\xb6\x82',
'LatinCapitalLetterUWithDiaeresis': '\xc3\x9c',
'CjkUnifiedIdeograph4d80': '\xe4\xb6\x80',
'CjkUnifiedIdeograph4d81': '\xe4\xb6\x81',
'CjkUnifiedIdeograph5a59': '\xe5\xa9\x99',
'CjkUnifiedIdeograph5a58': '\xe5\xa9\x98',
'HangulSyllableJweonj': '\xec\xa4\x9d',
'CjkUnifiedIdeograph4d88': '\xe4\xb6\x88',
'CjkUnifiedIdeograph4d89': '\xe4\xb6\x89',
'HangulSyllableRwem': '\xeb\xa4\xb0',
'UnicodeCharacter0xe910': '\xee\xa4\x90',
'HangulSyllableGwegs': '\xea\xb6\xa7',
'MyanmarSymbolGenitive': '\xe1\x81\x8f',
'GreekCapitalLetterDelta': '\xce\x94',
'UnicodeCharacter0xe3e': '\xe0\xb8\xbe',
'UnicodeCharacter0xe3d': '\xe0\xb8\xbd',
'UnicodeCharacter0xe3c': '\xe0\xb8\xbc',
'UnicodeCharacter0xe3b': '\xe0\xb8\xbb',
'CjkUnifiedIdeograph4d8f': '\xe4\xb6\x8f',
'CanadianSyllabicsWestcreePwii': '\xe1\x90\xbf',
'CjkUnifiedIdeograph4d8d': '\xe4\xb6\x8d',
'CjkUnifiedIdeograph4d8e': '\xe4\xb6\x8e',
'CjkUnifiedIdeograph4d8b': '\xe4\xb6\x8b',
'CjkUnifiedIdeograph4d8c': '\xe4\xb6\x8c',
'CjkUnifiedIdeograph5a5e': '\xe5\xa9\x9e',
'CjkUnifiedIdeograph4d8a': '\xe4\xb6\x8a',
'HangulSyllableDdyub': '\xeb\x9c\x9d',
'HangulSyllableDdyuc': '\xeb\x9c\xa3',
'UnicodeCharacter0xa9fb': '\xea\xa7\xbb',
'HangulSyllableDdyug': '\xeb\x9c\x8d',
'SymbolForGroupSeparator': '\xe2\x90\x9d',
'GlagoliticCapitalLetterSmallYusWithTail': '\xe2\xb0\xa5',
'HangulSyllableGgyop': '\xea\xbe\xb6',
'UnicodeCharacter0xe811': '\xee\xa0\x91',
'HangulSyllableDdyus': '\xeb\x9c\x9f',
'HangulSyllableDdyut': '\xeb\x9c\xa5',
'OpenCentreBlackStar': '\xe2\x9c\xab',
'HangulSyllableRwass': '\xeb\xa2\x8c',
'CyrillicCapitalLetterYeru': '\xd0\xab',
'BengaliCurrencyNumeratorOne': '\xe0\xa7\xb4',
'CanadianSyllabicsSw': '\xe1\x94\x87',
'HangulSyllableGgyaelp': '\xea\xba\xae',
'HangulSyllableSsan': '\xec\x8b\xbc',
'HangulSyllableMunj': '\xeb\xac\xb9',
'CanadianSyllabicsSa': '\xe1\x93\xb4',
'HangulSyllableGwegg': '\xea\xb6\xa6',
'CjkRadicalSilk': '\xe2\xba\xaf',
'CanadianSyllabicsSe': '\xe1\x93\xad',
'HangulSyllableSoelt': '\xec\x87\xad',
'CanadianSyllabicsSi': '\xe1\x93\xaf',
'CanadianSyllabicsSh': '\xe1\x94\xa5',
'CanadianSyllabicsSo': '\xe1\x93\xb1',
'ArabicLigatureBehWithMeemMedialForm': '\xef\xb3\xa1',
'LatinCapitalLetterNWithAcute': '\xc5\x83',
'UnicodeCharacter0xe915': '\xee\xa4\x95',
'ThaanaOoboofili': '\xde\xab',
'LessthanAboveDoublelineEqualAboveGreaterthan': '\xe2\xaa\x8b',
'YiSyllableZzax': '\xea\x8b\x99',
'ModifierLetterLowLeftstemToneBar': '\xea\x9c\x95',
'CyrillicCapitalLigatureAIe': '\xd3\x94',
'UnicodeCharacter0xe99f': '\xee\xa6\x9f',
'ModifierLetterLowMacron': '\xcb\x8d',
'UnicodeCharacter0xe914': '\xee\xa4\x94',
'HangulSyllableKwaenh': '\xec\xbe\x92',
'UnicodeCharacter0xf027': '\xef\x80\xa7',
'HangulSyllableKwaenj': '\xec\xbe\x91',
'HangulSyllableKwaeng': '\xec\xbe\xa1',
'GreekCapitalReversedDottedLunateSigmaSymbol': '\xcf\xbf',
'HangulSyllableDdeolm': '\xeb\x96\xaa',
'HangulSyllableDdeolh': '\xeb\x96\xaf',
'HangulSyllableDdeolg': '\xeb\x96\xa9',
'HangulSyllableDdeolb': '\xeb\x96\xab',
'HangulSyllableGgyot': '\xea\xbe\xb5',
'LatinSmallLetterGamma': '\xc9\xa3',
'HangulSyllablePyogs': '\xed\x91\x9f',
'HangulSyllableDdeolt': '\xeb\x96\xad',
'HangulSyllableDdeols': '\xeb\x96\xac',
'UnicodeCharacter0xe916': '\xee\xa4\x96',
'HexagramForRevolution': '\xe4\xb7\xb0',
'HangulSyllableDdeolp': '\xeb\x96\xae',
'HangulSyllableMyolb': '\xeb\xac\xa3',
'HangulSyllableMyolg': '\xeb\xac\xa1',
'LatinSmallLetterQpDigraph': '\xc8\xb9',
'HangulSyllableMyolh': '\xeb\xac\xa7',
'CjkUnifiedIdeograph762a': '\xe7\x98\xaa',
'HangulSyllableMyolm': '\xeb\xac\xa2',
'HangulSyllableTibs': '\xed\x8c\x82',
'HangulSyllableMyols': '\xeb\xac\xa4',
'UnicodeCharacter0xc91': '\xe0\xb2\x91',
'HangulSyllableMyolp': '\xeb\xac\xa6',
'CjkUnifiedIdeograph91c9': '\xe9\x87\x89',
'HangulSyllableMyolt': '\xeb\xac\xa5',
'LeftwardsTwoheadedArrowFromBar': '\xe2\xac\xb6',
'CjkUnifiedIdeograph7856': '\xe7\xa1\x96',
'CjkUnifiedIdeograph8312': '\xe8\x8c\x92',
'CjkUnifiedIdeograph8313': '\xe8\x8c\x93',
'CjkUnifiedIdeograph8310': '\xe8\x8c\x90',
'CjkUnifiedIdeograph8311': '\xe8\x8c\x91',
'CjkUnifiedIdeograph8316': '\xe8\x8c\x96',
'CjkUnifiedIdeograph8317': '\xe8\x8c\x97',
'CjkUnifiedIdeograph8314': '\xe8\x8c\x94',
'CjkUnifiedIdeograph7378': '\xe7\x8d\xb8',
'HangulSyllableSweolm': '\xec\x88\xbe',
'CjkUnifiedIdeograph8318': '\xe8\x8c\x98',
'CjkUnifiedIdeograph8319': '\xe8\x8c\x99',
'ArabicLetterTtehIsolatedForm': '\xef\xad\xa6',
'UnicodeCharacter0xe745': '\xee\x9d\x85',
'UnicodeCharacter0x5fd': '\xd7\xbd',
'UnicodeCharacter0x5fe': '\xd7\xbe',
'UnicodeCharacter0x5ff': '\xd7\xbf',
'UnicodeCharacter0x5fa': '\xd7\xba',
'UnicodeCharacter0x5fb': '\xd7\xbb',
'UnicodeCharacter0x5fc': '\xd7\xbc',
'UnicodeCharacter0xe918': '\xee\xa4\x98',
'ArabicLigatureSeenWithJeemWithHahInitialForm': '\xef\xb5\x9d',
'CjkUnifiedIdeograph8dcc': '\xe8\xb7\x8c',
'CjkUnifiedIdeograph8dcb': '\xe8\xb7\x8b',
'CjkUnifiedIdeograph8dca': '\xe8\xb7\x8a',
'HangulSyllableKeng': '\xec\xbc\x95',
'GreekSmallLetterUpsilonWithDialytikaAndTonos': '\xce\xb0',
'CjkUnifiedIdeograph8dce': '\xe8\xb7\x8e',
'CjkUnifiedIdeograph8dcd': '\xe8\xb7\x8d',
'HangulSyllableKenj': '\xec\xbc\x85',
'CyrillicPayerok': '\xea\x99\xbf',
'HangulSyllableBbyolh': '\xeb\xbe\xbf',
'ElementOfWithDotAbove': '\xe2\x8b\xb5',
'RightwardsHarpoonWithBarbDownFromBar': '\xe2\xa5\x9f',
'UnicodeCharacter0xf020': '\xef\x80\xa0',
'CjkUnifiedIdeograph5e05': '\xe5\xb8\x85',
'UnicodeCharacter0x5f5': '\xd7\xb5',
'UnicodeCharacter0x5f6': '\xd7\xb6',
'UnicodeCharacter0x5f7': '\xd7\xb7',
'UnicodeCharacter0xf3cb': '\xef\x8f\x8b',
'SinhalaVowelSignDigaAedapilla': '\xe0\xb7\x91',
'GlagoliticCapitalLetterAzu': '\xe2\xb0\x80',
'UnicodeCharacter0x5f8': '\xd7\xb8',
'UnicodeCharacter0x5f9': '\xd7\xb9',
'DevanagariVowelSignVocalicR': '\xe0\xa5\x83',
'DevanagariVowelSignVocalicL': '\xe0\xa5\xa2',
'CjkUnifiedIdeograph831c': '\xe8\x8c\x9c',
'CjkUnifiedIdeograph831a': '\xe8\x8c\x9a',
'TeluguLetterEe': '\xe0\xb0\x8f',
'CjkUnifiedIdeograph831d': '\xe8\x8c\x9d',
'CjkUnifiedIdeograph831e': '\xe8\x8c\x9e',
'CjkUnifiedIdeograph7376': '\xe7\x8d\xb6',
'CjkUnifiedIdeograph8dc3': '\xe8\xb7\x83',
'CjkUnifiedIdeograph8dc2': '\xe8\xb7\x82',
'CjkUnifiedIdeograph8dc1': '\xe8\xb7\x81',
'CjkUnifiedIdeograph8dc0': '\xe8\xb7\x80',
'CjkUnifiedIdeograph8dc7': '\xe8\xb7\x87',
'CjkUnifiedIdeograph8dc6': '\xe8\xb7\x86',
'CjkUnifiedIdeograph8dc5': '\xe8\xb7\x85',
'CjkUnifiedIdeograph8dc4': '\xe8\xb7\x84',
'CjkUnifiedIdeograph8dc9': '\xe8\xb7\x89',
'TibetanLetterCha': '\xe0\xbd\x86',
'CjkUnifiedIdeograph4b8e': '\xe4\xae\x8e',
'CjkUnifiedIdeograph88df': '\xe8\xa3\x9f',
'HangulSyllableTuss': '\xed\x89\x80',
'UnicodeCharacter0xdc20': '\xed\xb0\xa0',
'BengaliDigitOne': '\xe0\xa7\xa7',
'HangulJongseongMieum': '\xe1\x86\xb7',
'UnicodeCharacter0xdc22': '\xed\xb0\xa2',
'CjkUnifiedIdeograph6130': '\xe6\x84\xb0',
'YiSyllableZuo': '\xea\x8a\x99',
'CjkUnifiedIdeograph66d8': '\xe6\x9b\x98',
'CjkUnifiedIdeograph5dd3': '\xe5\xb7\x93',
'HangulSyllableDdyeogs': '\xeb\x97\x9b',
'CjkUnifiedIdeograph5dd2': '\xe5\xb7\x92',
'HangulSyllableDigs': '\xeb\x94\x97',
'CjkUnifiedIdeograph9515': '\xe9\x94\x95',
'HangulSyllableBep': '\xeb\xb2\xba',
'CjkUnifiedIdeograph4ffb': '\xe4\xbf\xbb',
'CjkUnifiedIdeograph4ffc': '\xe4\xbf\xbc',
'YiSyllableZut': '\xea\x8a\xa2',
'CjkUnifiedIdeograph4ffa': '\xe4\xbf\xba',
'CircleWithHorizontalBar': '\xe2\xa6\xb5',
'UnicodeCharacter0x1a84': '\xe1\xaa\x84',
'CjkUnifiedIdeograph4ffd': '\xe4\xbf\xbd',
'CjkUnifiedIdeograph4ffe': '\xe4\xbf\xbe',
'HangulSyllableGyubs': '\xea\xb7\xae',
'EthiopicSyllableSzu': '\xe1\x88\xa1',
'HangulSyllableJeulm': '\xec\xa6\x92',
'HangulSyllableJeulb': '\xec\xa6\x93',
'UnicodeCharacter0x1a82': '\xe1\xaa\x82',
'HangulSyllableJeulg': '\xec\xa6\x91',
'UnicodeCharacter0x1a83': '\xe1\xaa\x83',
'EthiopicSyllableSza': '\xe1\x88\xa0',
'HangulSyllableSi': '\xec\x8b\x9c',
'HangulSyllableSo': '\xec\x86\x8c',
'EthiopicSyllableSze': '\xe1\x88\xa5',
'HangulSyllableJeuls': '\xec\xa6\x94',
'EthiopicSyllableSzi': '\xe1\x88\xa2',
'HangulSyllableSa': '\xec\x82\xac',
'EthiopicSyllableSzo': '\xe1\x88\xa6',
'HangulSyllableJeult': '\xec\xa6\x95',
'HangulSyllableSe': '\xec\x84\xb8',
'CjkUnifiedIdeograph4ff8': '\xe4\xbf\xb8',
'CjkUnifiedIdeograph4ff9': '\xe4\xbf\xb9',
'FullwidthLatinSmallLetterX': '\xef\xbd\x98',
'HangulSyllableKaenh': '\xec\xba\x96',
'FullwidthLatinSmallLetterZ': '\xef\xbd\x9a',
'HangulSyllableCyeg': '\xec\xb3\xad',
'CjkUnifiedIdeograph4ff2': '\xe4\xbf\xb2',
'CjkUnifiedIdeograph4ff3': '\xe4\xbf\xb3',
'CjkUnifiedIdeograph4ff0': '\xe4\xbf\xb0',
'CjkUnifiedIdeograph4ff1': '\xe4\xbf\xb1',
'CjkUnifiedIdeograph4ff6': '\xe4\xbf\xb6',
'CjkUnifiedIdeograph4ff7': '\xe4\xbf\xb7',
'CjkUnifiedIdeograph4ff4': '\xe4\xbf\xb4',
'CjkUnifiedIdeograph4ff5': '\xe4\xbf\xb5',
'FullwidthLatinSmallLetterL': '\xef\xbd\x8c',
'FullwidthLatinSmallLetterM': '\xef\xbd\x8d',
'VaiSyllablePo': '\xea\x97\x81',
'FullwidthLatinSmallLetterO': '\xef\xbd\x8f',
'VaiSyllablePi': '\xea\x94\xaa',
'FullwidthLatinSmallLetterI': '\xef\xbd\x89',
'TibetanSubjoinedLetterBha': '\xe0\xbe\xa7',
'LatinSmallLetterCWithCedillaAndAcute': '\xe1\xb8\x89',
'VaiSyllablePe': '\xea\x97\xa8',
'HangulSyllableBem': '\xeb\xb2\xb0',
'FullwidthLatinSmallLetterF': '\xef\xbd\x86',
'CjkUnifiedIdeograph9d0b': '\xe9\xb4\x8b',
'GreekSmallLetterEtaWithPsiliAndPerispomeni': '\xe1\xbc\xa6',
'FullwidthLatinSmallLetterA': '\xef\xbd\x81',
'FullwidthLatinSmallLetterB': '\xef\xbd\x82',
'HangulSyllableBel': '\xeb\xb2\xa8',
'UnicodeCharacter0xd84a': '\xed\xa1\x8a',
'UnicodeCharacter0xd84b': '\xed\xa1\x8b',
'DownwardsArrowWithTipRightwards': '\xe2\x86\xb3',
'UnicodeCharacter0xd84d': '\xed\xa1\x8d',
'UnicodeCharacter0xd84e': '\xed\xa1\x8e',
'UnicodeCharacter0xd84f': '\xed\xa1\x8f',
'HangulSyllableJon': '\xec\xa1\xb4',
'UnicodeCharacter0xd968': '\xed\xa5\xa8',
'HangulSyllableJom': '\xec\xa2\x80',
'UnicodeCharacter0xa6e6': '\xea\x9b\xa6',
'CjkUnifiedIdeograph6d67': '\xe6\xb5\xa7',
'CjkUnifiedIdeograph8f7a': '\xe8\xbd\xba',
'UnicodeCharacter0xa6e5': '\xea\x9b\xa5',
'RunicLetterLongbranchossO': '\xe1\x9a\xac',
'UnicodeCharacter0xa6e3': '\xea\x9b\xa3',
'LeftSquareBracketLowerCorner': '\xe2\x8e\xa3',
'YiSyllableSsiep': '\xea\x8c\xb0',
'HangulSyllableJjagg': '\xec\xa7\x9e',
'HangulSyllableJok': '\xec\xa2\x88',
'UnicodeCharacter0xea2e': '\xee\xa8\xae',
'CyrillicSmallLetterReversedDze': '\xea\x99\x85',
'EthiopicSyllableKyee': '\xe2\xb7\x8c',
'UnicodeCharacter0xa6e8': '\xea\x9b\xa8',
'CjkUnifiedIdeograph7626': '\xe7\x98\xa6',
'HangulSyllableJjyilg': '\xec\xaf\xb9',
'HiraganaIterationMark': '\xe3\x82\x9d',
'HangulSyllableCyej': '\xec\xb4\x82',
'HangulSyllableJoh': '\xec\xa2\x8b',
'HangulSyllableCyek': '\xec\xb4\x84',
'HebrewLetterAlefWithPatah': '\xef\xac\xae',
'PlusSignWithCircumflexAccentAbove': '\xe2\xa8\xa3',
'HangulSyllableCyeh': '\xec\xb4\x87',
'HangulSyllableJoe': '\xec\xa3\x84',
'VaiSyllableMba': '\xea\x95\x93',
'GreekCapitalLetterAlphaWithVaria': '\xe1\xbe\xba',
'UnicodeCharacter0xa6ed': '\xea\x9b\xad',
'UnicodeCharacter0xa6ee': '\xea\x9b\xae',
'ArabicSmallHighNoon': '\xdb\xa8',
'UnicodeCharacter0xa6ec': '\xea\x9b\xac',
'NkoDigitFive': '\xdf\x85',
'VaiSyllableMbi': '\xea\x94\xad',
'CanadianSyllabicsNunavikHaa': '\xe1\x95\xba',
'VaiSyllableMbo': '\xea\x97\x84',
'HangulSyllableBaenj': '\xeb\xb0\xb5',
'UnicodeCharacter0xd840': '\xed\xa1\x80',
'CjkUnifiedIdeograph66bd': '\xe6\x9a\xbd',
'UnicodeCharacter0xd842': '\xed\xa1\x82',
'UnicodeCharacter0xd843': '\xed\xa1\x83',
'VaiSyllableMbu': '\xea\x96\x9e',
'UnicodeCharacter0xd845': '\xed\xa1\x85',
'BraillePatternDots34678': '\xe2\xa3\xac',
'CjkUnifiedIdeograph66d2': '\xe6\x9b\x92',
'WhiteChessRook': '\xe2\x99\x96',
'BopomofoLetterInn': '\xe3\x86\xaa',
'UnicodeCharacter0xf779': '\xef\x9d\xb9',
'HangulSyllableNyulh': '\xeb\x8a\x83',
'HiraganaLetterSmallI': '\xe3\x81\x83',
'HiraganaLetterSmallO': '\xe3\x81\x89',
'HangulSyllableNyulm': '\xeb\x89\xbe',
'HangulSyllableNyulb': '\xeb\x89\xbf',
'HiraganaLetterSmallA': '\xe3\x81\x81',
'HangulSyllableNyulg': '\xeb\x89\xbd',
'HiraganaLetterSmallE': '\xe3\x81\x87',
'GreekCapitalLetterEtaWithDasiaAndVariaAndProsgegrammeni': '\xe1\xbe\x9b',
'UnicodeCharacter0xe7c0': '\xee\x9f\x80',
'MongolianBirga': '\xe1\xa0\x80',
'HangulSyllableNyuls': '\xeb\x8a\x80',
'HangulSyllableNyulp': '\xeb\x8a\x82',
'HangulSyllableDwal': '\xeb\x8f\xa8',
'HangulSyllableNyult': '\xeb\x8a\x81',
'HiraganaLetterSmallU': '\xe3\x81\x85',
'CjkUnifiedIdeograph8b40': '\xe8\xad\x80',
'CjkUnifiedIdeograph8b41': '\xe8\xad\x81',
'CjkUnifiedIdeograph8b42': '\xe8\xad\x82',
'CjkUnifiedIdeograph8b43': '\xe8\xad\x83',
'CjkUnifiedIdeograph8b44': '\xe8\xad\x84',
'CjkUnifiedIdeograph8b45': '\xe8\xad\x85',
'CjkUnifiedIdeograph8b46': '\xe8\xad\x86',
'CjkUnifiedIdeograph8b47': '\xe8\xad\x87',
'CjkUnifiedIdeograph8b48': '\xe8\xad\x88',
'CjkUnifiedIdeograph8b49': '\xe8\xad\x89',
'HangulSyllableJwilm': '\xec\xa5\x9a',
'CircledHangulCieucA': '\xe3\x89\xb6',
'UnicodeCharacter0x1aae': '\xe1\xaa\xae',
'UnicodeCharacter0x1aad': '\xe1\xaa\xad',
'HangulSyllableGweg': '\xea\xb6\xa5',
'UnicodeCharacter0x1aaf': '\xe1\xaa\xaf',
'UnicodeCharacter0x1aaa': '\xe1\xaa\xaa',
'TibetanMarkBskaShogGiMgoRgyan': '\xe0\xbf\x90',
'UnicodeCharacter0x1aac': '\xe1\xaa\xac',
'UnicodeCharacter0x1aab': '\xe1\xaa\xab',
'ArabicLetterDal': '\xd8\xaf',
'LessthanOrEquivalentTo': '\xe2\x89\xb2',
'CircledIdeographCentre': '\xe3\x8a\xa5',
'UnicodeCharacter0xde17': '\xed\xb8\x97',
'HangulSyllableJot': '\xec\xa2\x89',
'UnicodeCharacter0xde14': '\xed\xb8\x94',
'UnicodeCharacter0xde15': '\xed\xb8\x95',
'UnicodeCharacter0xe34d': '\xee\x8d\x8d',
'HangulSyllableSweog': '\xec\x88\xb5',
'UnicodeCharacter0xe34e': '\xee\x8d\x8e',
'CjkUnifiedIdeograph58fe': '\xe5\xa3\xbe',
'UnicodeCharacter0xf22d': '\xef\x88\xad',
'UnicodeCharacter0x1aa5': '\xe1\xaa\xa5',
'UnicodeCharacter0x1aa4': '\xe1\xaa\xa4',
'UnicodeCharacter0x1aa7': '\xe1\xaa\xa7',
'UnicodeCharacter0x1aa6': '\xe1\xaa\xa6',
'UnicodeCharacter0x1aa1': '\xe1\xaa\xa1',
'UnicodeCharacter0x1aa0': '\xe1\xaa\xa0',
'UnicodeCharacter0x1aa3': '\xe1\xaa\xa3',
'UnicodeCharacter0x1aa2': '\xe1\xaa\xa2',
'HangulSyllableCyiss': '\xec\xb9\x90',
'UnicodeCharacter0x1aa9': '\xe1\xaa\xa9',
'UnicodeCharacter0x1aa8': '\xe1\xaa\xa8',
'CjkUnifiedIdeograph52df': '\xe5\x8b\x9f',
'CjkUnifiedIdeograph8b4a': '\xe8\xad\x8a',
'LatinCapitalLetterUWithMacronAndDiaeresis': '\xe1\xb9\xba',
'CjkUnifiedIdeograph8b4c': '\xe8\xad\x8c',
'CjkUnifiedIdeograph8b4d': '\xe8\xad\x8d',
'ModifierLetterSmallZ': '\xe1\xb6\xbb',
'CjkUnifiedIdeograph8b4f': '\xe8\xad\x8f',
'FullwidthCentSign': '\xef\xbf\xa0',
'CjkUnifiedIdeograph9d09': '\xe9\xb4\x89',
'UnicodeCharacter0xe34c': '\xee\x8d\x8c',
'BoxDrawingsHeavyVerticalAndRight': '\xe2\x94\xa3',
'UnicodeCharacter0xde1c': '\xed\xb8\x9c',
'CjkUnifiedIdeograph8c71': '\xe8\xb1\xb1',
'LatinSmallLetterOWithStrokeAndAcute': '\xc7\xbf',
'GreekLetterSampi': '\xcf\xa0',
'UnicodeCharacter0xf39c': '\xef\x8e\x9c',
'MalayalamLetterAu': '\xe0\xb4\x94',
'SuperscriptLeftParenthesis': '\xe2\x81\xbd',
'GreekCapitalKaiSymbol': '\xcf\x8f',
'IdenticalWithDotAbove': '\xe2\xa9\xa7',
'CjkUnifiedIdeograph9d05': '\xe9\xb4\x85',
'MalayalamLetterAa': '\xe0\xb4\x86',
'CjkUnifiedIdeograph3614': '\xe3\x98\x94',
'HangulSyllableNeong': '\xeb\x84\x9d',
'UnicodeCharacter0xe3c9': '\xee\x8f\x89',
'MalayalamLetterAi': '\xe0\xb4\x90',
'BacktiltedShadowedWhiteRightwardsArrow': '\xe2\x9e\xab',
'OriyaDigitFour': '\xe0\xad\xaa',
'HangulSyllableSswilg': '\xec\x93\x81',
'HangulSyllableSswilb': '\xec\x93\x83',
'CjkUnifiedIdeograph9b0c': '\xe9\xac\x8c',
'CjkUnifiedIdeograph9b0b': '\xe9\xac\x8b',
'CjkUnifiedIdeograph9b0a': '\xe9\xac\x8a',
'CjkUnifiedIdeograph7084': '\xe7\x82\x84',
'UnicodeCharacter0xdaef': '\xed\xab\xaf',
'CjkUnifiedIdeograph9b0f': '\xe9\xac\x8f',
'CjkUnifiedIdeograph9b0e': '\xe9\xac\x8e',
'CjkUnifiedIdeograph9b0d': '\xe9\xac\x8d',
'UnicodeCharacter0xeae5': '\xee\xab\xa5',
'HangulSyllableSswilt': '\xec\x93\x85',
'UnicodeCharacter0xeae7': '\xee\xab\xa7',
'UnicodeCharacter0xeae6': '\xee\xab\xa6',
'UnicodeCharacter0xeae1': '\xee\xab\xa1',
'CjkUnifiedIdeograph9d03': '\xe9\xb4\x83',
'HangulSyllableSswils': '\xec\x93\x84',
'UnicodeCharacter0xe3c4': '\xee\x8f\x84',
'UnicodeCharacter0xeae9': '\xee\xab\xa9',
'UnicodeCharacter0xe3c7': '\xee\x8f\x87',
'CjkUnifiedIdeograph793c': '\xe7\xa4\xbc',
'CjkUnifiedIdeograph793b': '\xe7\xa4\xbb',
'CjkUnifiedIdeograph793a': '\xe7\xa4\xba',
'UnicodeCharacter0xe3c6': '\xee\x8f\x86',
'HangulSyllablePweob': '\xed\x92\xa5',
'CjkUnifiedIdeograph793f': '\xe7\xa4\xbf',
'CjkUnifiedIdeograph793e': '\xe7\xa4\xbe',
'CjkUnifiedIdeograph793d': '\xe7\xa4\xbd',
'CjkUnifiedIdeograph437a': '\xe4\x8d\xba',
'CjkUnifiedIdeograph437b': '\xe4\x8d\xbb',
'CjkUnifiedIdeograph437c': '\xe4\x8d\xbc',
'CjkUnifiedIdeograph437d': '\xe4\x8d\xbd',
'CjkUnifiedIdeograph437e': '\xe4\x8d\xbe',
'CjkUnifiedIdeograph437f': '\xe4\x8d\xbf',
'HangulLetterRieulpieupsios': '\xe3\x85\xab',
'CjkUnifiedIdeograph846a': '\xe8\x91\xaa',
'HangulSyllableTeobs': '\xed\x85\x82',
'CjkRadicalCsimplifiedEven': '\xe2\xbb\xac',
'CjkUnifiedIdeograph36fa': '\xe3\x9b\xba',
'LatinSmallLigatureIj': '\xc4\xb3',
'UnicodeCharacter0xeaee': '\xee\xab\xae',
'UnicodeCharacter0xeaed': '\xee\xab\xad',
'UnicodeCharacter0xeaef': '\xee\xab\xaf',
'UnicodeCharacter0xeaea': '\xee\xab\xaa',
'UnicodeCharacter0xeaec': '\xee\xab\xac',
'UnicodeCharacter0xeaeb': '\xee\xab\xab',
'HangulSyllableGgyes': '\xea\xbc\xa3',
'HangulSyllableGgyep': '\xea\xbc\xaa',
'CjkUnifiedIdeograph99a4': '\xe9\xa6\xa4',
'HangulSyllableGgyet': '\xea\xbc\xa9',
'HangulSyllableGgyej': '\xea\xbc\xa6',
'HangulSyllableGgyek': '\xea\xbc\xa8',
'CjkUnifiedIdeograph9b09': '\xe9\xac\x89',
'CjkUnifiedIdeograph9b08': '\xe9\xac\x88',
'HangulSyllableGgyen': '\xea\xbc\x94',
'HangulSyllableGgyeo': '\xea\xbb\xb4',
'HangulSyllableGgyel': '\xea\xbc\x98',
'HangulSyllableGgyem': '\xea\xbc\xa0',
'CjkUnifiedIdeograph9b03': '\xe9\xac\x83',
'CjkUnifiedIdeograph9b02': '\xe9\xac\x82',
'CjkUnifiedIdeograph9b01': '\xe9\xac\x81',
'CjkUnifiedIdeograph9b00': '\xe9\xac\x80',
'CjkUnifiedIdeograph9b07': '\xe9\xac\x87',
'CjkUnifiedIdeograph9b06': '\xe9\xac\x86',
'CjkUnifiedIdeograph9b05': '\xe9\xac\x85',
'CjkUnifiedIdeograph9b04': '\xe9\xac\x84',
'BraillePatternDots23578': '\xe2\xa3\x96',
'UnicodeCharacter0xdc2f': '\xed\xb0\xaf',
'UnicodeCharacter0xdfba': '\xed\xbe\xba',
'LatinSmallLetterVWithDiagonalStroke': '\xea\x9d\x9f',
'+-': '\xc2\xb1',
'CjkUnifiedIdeograph4378': '\xe4\x8d\xb8',
'CjkUnifiedIdeograph4379': '\xe4\x8d\xb9',
'CjkUnifiedIdeograph7931': '\xe7\xa4\xb1',
'CjkUnifiedIdeograph7930': '\xe7\xa4\xb0',
'CjkUnifiedIdeograph7937': '\xe7\xa4\xb7',
'CjkUnifiedIdeograph7936': '\xe7\xa4\xb6',
'CjkUnifiedIdeograph7935': '\xe7\xa4\xb5',
'CjkUnifiedIdeograph7934': '\xe7\xa4\xb4',
'CjkUnifiedIdeograph4370': '\xe4\x8d\xb0',
'CjkUnifiedIdeograph4371': '\xe4\x8d\xb1',
'CjkUnifiedIdeograph4372': '\xe4\x8d\xb2',
'CjkUnifiedIdeograph4373': '\xe4\x8d\xb3',
'CjkUnifiedIdeograph4374': '\xe4\x8d\xb4',
'CjkUnifiedIdeograph4375': '\xe4\x8d\xb5',
'CjkUnifiedIdeograph4376': '\xe4\x8d\xb6',
'CjkUnifiedIdeograph4377': '\xe4\x8d\xb7',
'SuperscriptRightParenthesis': '\xe2\x81\xbe',
'CjkUnifiedIdeograph69be': '\xe6\xa6\xbe',
'CjkUnifiedIdeograph69bd': '\xe6\xa6\xbd',
'CjkUnifiedIdeograph69bc': '\xe6\xa6\xbc',
'CjkUnifiedIdeograph69bb': '\xe6\xa6\xbb',
'CjkUnifiedIdeograph69ba': '\xe6\xa6\xba',
'QuadrantUpperLeftAndLowerRight': '\xe2\x96\x9a',
'HangulSyllableSiss': '\xec\x8b\xb0',
'UnicodeCharacter0x1a8f': '\xe1\xaa\x8f',
'UnicodeCharacter0xde1e': '\xed\xb8\x9e',
'ArabicLetterDotlessBeh': '\xd9\xae',
'CjkUnifiedIdeograph81a1': '\xe8\x86\xa1',
'CjkUnifiedIdeograph542f': '\xe5\x90\xaf',
'CjkUnifiedIdeograph542e': '\xe5\x90\xae',
'UnicodeCharacter0x1a8e': '\xe1\xaa\x8e',
'CjkUnifiedIdeograph542c': '\xe5\x90\xac',
'GujaratiDigitFive': '\xe0\xab\xab',
'CjkUnifiedIdeograph542a': '\xe5\x90\xaa',
'HangulSyllableRoelt': '\xeb\xa2\xbd',
'UnicodeCharacter0x1a8b': '\xe1\xaa\x8b',
'CjkUnifiedIdeograph81a3': '\xe8\x86\xa3',
'ModifierLetterSmallTurnedH': '\xe1\xb6\xa3',
'ModifierLetterSmallTurnedI': '\xe1\xb5\x8e',
'CanadianSyllabicsCarrierTlu': '\xe1\x98\xba',
'LatinSmallLetterTeshDigraph': '\xca\xa7',
'ModifierLetterSmallTurnedV': '\xe1\xb6\xba',
'CanadianSyllabicsCarrierTli': '\xe1\x98\xbe',
'HangulSyllableRoelh': '\xeb\xa2\xbf',
'CanadianSyllabicsCarrierTlo': '\xe1\x98\xbb',
'ModifierLetterSmallTurnedR': '\xca\xb4',
'UnicodeCharacter0xe263': '\xee\x89\xa3',
'CircledDigitOne': '\xe2\x91\xa0',
'UnicodeCharacter0x1a8a': '\xe1\xaa\x8a',
'HangulSyllableRoelb': '\xeb\xa2\xbb',
'CanadianSyllabicsCarrierTle': '\xe1\x98\xbc',
'UnicodeCharacter0xf6d3': '\xef\x9b\x93',
'CjkUnifiedIdeograph94de': '\xe9\x93\x9e',
'GujaratiSignVirama': '\xe0\xab\x8d',
'CjkUnifiedIdeograph94dd': '\xe9\x93\x9d',
'CjkUnifiedIdeograph69b7': '\xe6\xa6\xb7',
'RunicLetterOpenp': '\xe1\x9b\x95',
'CjkUnifiedIdeograph69b5': '\xe6\xa6\xb5',
'CjkUnifiedIdeograph69b4': '\xe6\xa6\xb4',
'CjkUnifiedIdeograph69b3': '\xe6\xa6\xb3',
'CjkUnifiedIdeograph69b2': '\xe6\xa6\xb2',
'CjkUnifiedIdeograph69b1': '\xe6\xa6\xb1',
'CjkUnifiedIdeograph69b0': '\xe6\xa6\xb0',
'CjkUnifiedIdeograph69b9': '\xe6\xa6\xb9',
'CjkUnifiedIdeograph69b8': '\xe6\xa6\xb8',
'HangulSyllableDwalb': '\xeb\x8f\xab',
'UnicodeCharacter0xf6d4': '\xef\x9b\x94',
'HangulSyllableJjyilt': '\xec\xaf\xbd',
'HangulSyllableSsad': '\xec\x8b\xbf',
'CjkUnifiedIdeograph5427': '\xe5\x90\xa7',
'CjkUnifiedIdeograph5426': '\xe5\x90\xa6',
'CjkUnifiedIdeograph5425': '\xe5\x90\xa5',
'CjkUnifiedIdeograph5424': '\xe5\x90\xa4',
'CjkUnifiedIdeograph5423': '\xe5\x90\xa3',
'CjkUnifiedIdeograph5422': '\xe5\x90\xa2',
'CjkUnifiedIdeograph5421': '\xe5\x90\xa1',
'CjkUnifiedIdeograph5420': '\xe5\x90\xa0',
'ArabicLetterHehWithInvertedV': '\xdb\xbf',
'CjkUnifiedIdeograph5429': '\xe5\x90\xa9',
'CjkUnifiedIdeograph5428': '\xe5\x90\xa8',
'UnicodeCharacter0xe304': '\xee\x8c\x84',
'UnicodeCharacter0xfe27': '\xef\xb8\xa7',
'KangxiRadicalGold': '\xe2\xbe\xa6',
'CjkUnifiedIdeograph708b': '\xe7\x82\x8b',
'LepchaLetterNga': '\xe1\xb0\x85',
'UnicodeCharacter0xfe29': '\xef\xb8\xa9',
'UnicodeCharacter0xfe28': '\xef\xb8\xa8',
'UnicodeCharacter0xfbd1': '\xef\xaf\x91',
'RightSubstitutionBracket': '\xe2\xb8\x83',
'HangulSyllableTwaeb': '\xed\x87\xa9',
'BoxDrawingsLightArcUpAndRight': '\xe2\x95\xb0',
'CjkUnifiedIdeograph921a': '\xe9\x88\x9a',
'CjkUnifiedIdeograph921b': '\xe9\x88\x9b',
'CjkUnifiedIdeograph921c': '\xe9\x88\x9c',
'CjkUnifiedIdeograph921d': '\xe9\x88\x9d',
'CjkUnifiedIdeograph921e': '\xe9\x88\x9e',
'CjkUnifiedIdeograph921f': '\xe9\x88\x9f',
'HangulSyllableKeus': '\xed\x81\xbf',
'HangulSyllableKeup': '\xed\x82\x86',
'HangulSyllableGyilh': '\xea\xb8\xa3',
'ThreedBottomlightedRightwardsArrowhead': '\xe2\x9e\xa3',
'HangulSyllableGyilb': '\xea\xb8\x9f',
'CjkUnifiedIdeograph708d': '\xe7\x82\x8d',
'HangulSyllableGyilg': '\xea\xb8\x9d',
'HangulSyllableKeub': '\xed\x81\xbd',
'MongolianLetterManchuAliGaliGha': '\xe1\xa2\x9a',
'HangulSyllableKeug': '\xed\x81\xad',
'HangulSyllableMebs': '\xeb\xa9\xa6',
'UnicodeCharacter0xe3cd': '\xee\x8f\x8d',
'HangulSyllableGyils': '\xea\xb8\xa0',
'HangulSyllableKeuk': '\xed\x82\x84',
'HangulSyllableKeuh': '\xed\x82\x87',
'HangulSyllableGyilp': '\xea\xb8\xa2',
'HangulSyllableKeun': '\xed\x81\xb0',
'BugineseLetterNya': '\xe1\xa8\x8e',
'HangulSyllableKeul': '\xed\x81\xb4',
'HangulSyllableGyilt': '\xea\xb8\xa1',
'CjkUnifiedIdeograph9218': '\xe9\x88\x98',
'CjkUnifiedIdeograph9219': '\xe9\x88\x99',
'HangulSyllablePip': '\xed\x95\x96',
'NewTaiLueLetterLowNa': '\xe1\xa6\x93',
'CjkUnifiedIdeograph9210': '\xe9\x88\x90',
'CjkUnifiedIdeograph9211': '\xe9\x88\x91',
'CjkUnifiedIdeograph9212': '\xe9\x88\x92',
'CjkUnifiedIdeograph9213': '\xe9\x88\x93',
'CjkUnifiedIdeograph9214': '\xe9\x88\x94',
'CjkUnifiedIdeograph9215': '\xe9\x88\x95',
'CjkUnifiedIdeograph9216': '\xe9\x88\x96',
'CjkUnifiedIdeograph9217': '\xe9\x88\x97',
'UnicodeCharacter0xfe2e': '\xef\xb8\xae',
'UnicodeCharacter0xfe2d': '\xef\xb8\xad',
'HangulSyllableKying': '\xed\x82\x9d',
'CombiningDotAbove': '\xcc\x87',
'UnicodeCharacter0xfe2a': '\xef\xb8\xaa',
'MinusSignInTriangle': '\xe2\xa8\xba',
'UnicodeCharacter0xfe2c': '\xef\xb8\xac',
'UnicodeCharacter0xfe2b': '\xef\xb8\xab',
'UnicodeCharacter0xe3cb': '\xee\x8f\x8b',
'HangulSyllableJjyilp': '\xec\xaf\xbe',
'DiamondWithTopHalfBlack': '\xe2\xac\x98',
'HangulSyllablePik': '\xed\x95\x94',
'HangulSyllablePij': '\xed\x95\x92',
'HangulSyllableHulp': '\xed\x9b\x92',
'NotSquareImageOfOrEqualTo': '\xe2\x8b\xa2',
'GlagoliticSmallLetterLjudije': '\xe2\xb0\xbe',
'UnicodeCharacter0xde1a': '\xed\xb8\x9a',
'TeluguDigitFour': '\xe0\xb1\xaa',
'UnicodeCharacter0xe348': '\xee\x8d\x88',
'HangulSyllableJjyils': '\xec\xaf\xbc',
'CjkUnifiedIdeograph9cc6': '\xe9\xb3\x86',
'UnicodeCharacter0xe6a6': '\xee\x9a\xa6',
'HangulSyllableMyeol': '\xeb\xa9\xb8',
'UnicodeCharacter0xe6a4': '\xee\x9a\xa4',
'UnicodeCharacter0xe6a5': '\xee\x9a\xa5',
'UnicodeCharacter0xe6a2': '\xee\x9a\xa2',
'CjkUnifiedIdeograph58f9': '\xe5\xa3\xb9',
'UnicodeCharacter0xe6a0': '\xee\x9a\xa0',
'UnicodeCharacter0xe6a1': '\xee\x9a\xa1',
'HangulSyllableSsoeng': '\xec\x91\x81',
'CjkUnifiedIdeograph58f8': '\xe5\xa3\xb8',
'LatinSmallLetterEWithGrave': '\xc3\xa8',
'UnicodeCharacter0xe6a8': '\xee\x9a\xa8',
'UnicodeCharacter0xe6a9': '\xee\x9a\xa9',
'CjkUnifiedIdeograph711e': '\xe7\x84\x9e',
'OghamLetterAilm': '\xe1\x9a\x90',
'CjkUnifiedIdeograph711f': '\xe7\x84\x9f',
'CjkUnifiedIdeograph711a': '\xe7\x84\x9a',
'CjkUnifiedIdeograph711c': '\xe7\x84\x9c',
'CjkUnifiedIdeograph711b': '\xe7\x84\x9b',
'TamilLetterNna': '\xe0\xae\xa3',
'HangulSyllableMyeoh': '\xeb\xaa\x8b',
'UnicodeCharacter0xe346': '\xee\x8d\x86',
'UnicodeCharacter0xe347': '\xee\x8d\x87',
'CjkUnifiedIdeograph9cd8': '\xe9\xb3\x98',
'CjkUnifiedIdeograph9cd9': '\xe9\xb3\x99',
'UnicodeCharacter0x85d': '\xe0\xa1\x9d',
'UnicodeCharacter0x85e': '\xe0\xa1\x9e',
'UnicodeCharacter0x85f': '\xe0\xa1\x9f',
'CjkUnifiedIdeograph9cd5': '\xe9\xb3\x95',
'CjkUnifiedIdeograph9cd2': '\xe9\xb3\x92',
'UnicodeCharacter0x85a': '\xe0\xa1\x9a',
'UnicodeCharacter0x85b': '\xe0\xa1\x9b',
'UnicodeCharacter0x85c': '\xe0\xa1\x9c',
'NkoLetterRa': '\xdf\x99',
'HangulSyllableJyael': '\xec\x9f\xac',
'CjkCompatibilityIdeographf9ba': '\xef\xa6\xba',
'HangulSyllableJyaen': '\xec\x9f\xa8',
'HangulSyllableJyaeh': '\xec\x9f\xbf',
'HangulSyllableJyaek': '\xec\x9f\xbc',
'HangulSyllableJyaej': '\xec\x9f\xba',
'HangulSyllableJyaed': '\xec\x9f\xab',
'HangulSyllableJyaeg': '\xec\x9f\xa5',
'HangulSyllablePid': '\xed\x95\x83',
'LatinSmallLetterOWithCircumflexAndTilde': '\xe1\xbb\x97',
'HangulSyllableJyaec': '\xec\x9f\xbb',
'HangulSyllableJyaeb': '\xec\x9f\xb5',
'CjkUnifiedIdeograph7115': '\xe7\x84\x95',
'CjkUnifiedIdeograph7114': '\xe7\x84\x94',
'CjkUnifiedIdeograph7117': '\xe7\x84\x97',
'CjkUnifiedIdeograph7116': '\xe7\x84\x96',
'CjkUnifiedIdeograph7111': '\xe7\x84\x91',
'CjkUnifiedIdeograph7110': '\xe7\x84\x90',
'CanadianSyllabicsCarrierHwo': '\xe1\x97\x97',
'CjkUnifiedIdeograph7112': '\xe7\x84\x92',
'BalineseMusicalSymbolCombiningEndep': '\xe1\xad\xac',
'CjkUnifiedIdeograph9172': '\xe9\x85\xb2',
'CjkUnifiedIdeograph7119': '\xe7\x84\x99',
'CjkUnifiedIdeograph7118': '\xe7\x84\x98',
'UnicodeCharacter0xe6af': '\xee\x9a\xaf',
'UnicodeCharacter0xe6ad': '\xee\x9a\xad',
'UnicodeCharacter0xe6ae': '\xee\x9a\xae',
'UnicodeCharacter0xe6ab': '\xee\x9a\xab',
'UnicodeCharacter0xe6ac': '\xee\x9a\xac',
'UnicodeCharacter0xe6aa': '\xee\x9a\xaa',
'ArabicLigatureAinWithJeemInitialForm': '\xef\xb2\xba',
'CjkCompatibilityIdeographf9b3': '\xef\xa6\xb3',
'CjkUnifiedIdeograph953a': '\xe9\x94\xba',
'EthiopicSyllableJe': '\xe1\x8c\x85',
'UnicodeCharacter0xe5c6': '\xee\x97\x86',
'HangulSyllableGyih': '\xea\xb8\xaf',
'CjkCompatibilityIdeographf9b6': '\xef\xa6\xb6',
'EthiopicSyllableJa': '\xe1\x8c\x80',
'CjkCompatibilityIdeographf9b4': '\xef\xa6\xb4',
'EthiopicSyllableJo': '\xe1\x8c\x86',
'CjkCompatibilityIdeographf9b9': '\xef\xa6\xb9',
'CjkCompatibilityIdeographf9b8': '\xef\xa6\xb8',
'CjkUnifiedIdeograph953c': '\xe9\x94\xbc',
'EthiopicSyllableJi': '\xe1\x8c\x82',
'EthiopicSyllableJu': '\xe1\x8c\x81',
'BraillePatternDots267': '\xe2\xa1\xa2',
'UnicodeCharacter0x858': '\xe0\xa1\x98',
'UnicodeCharacter0x859': '\xe0\xa1\x99',
'UnicodeCharacter0x854': '\xe0\xa1\x94',
'UnicodeCharacter0x855': '\xe0\xa1\x95',
'UnicodeCharacter0x856': '\xe0\xa1\x96',
'UnicodeCharacter0x857': '\xe0\xa1\x97',
'UnicodeCharacter0x850': '\xe0\xa1\x90',
'UnicodeCharacter0x851': '\xe0\xa1\x91',
'UnicodeCharacter0x852': '\xe0\xa1\x92',
'UnicodeCharacter0x853': '\xe0\xa1\x93',
'UnicodeCharacter0xe1a9': '\xee\x86\xa9',
'UnicodeCharacter0xe1a8': '\xee\x86\xa8',
'HangulSyllableMweonh': '\xeb\xad\x96',
'HangulSyllableDdweogs': '\xeb\x9a\xbb',
'UnicodeCharacter0xe1a1': '\xee\x86\xa1',
'UnicodeCharacter0xe1a0': '\xee\x86\xa0',
'HangulSyllableMweong': '\xeb\xad\xa5',
'CjkUnifiedIdeograph953f': '\xe9\x94\xbf',
'UnicodeCharacter0xe1a5': '\xee\x86\xa5',
'UnicodeCharacter0xe1a4': '\xee\x86\xa4',
'UnicodeCharacter0xe1a7': '\xee\x86\xa7',
'UnicodeCharacter0xe1a6': '\xee\x86\xa6',
'UnicodeCharacter0xf030': '\xef\x80\xb0',
'TibetanSubjoinedLetterDdha': '\xe0\xbe\x9d',
'CjkUnifiedIdeograph5913': '\xe5\xa4\x93',
'HangulSyllableCiss': '\xec\xb9\xac',
'YiSyllableDdi': '\xea\x84\xb8',
'YiSyllableDdo': '\xea\x85\x86',
'CjkUnifiedIdeograph8163': '\xe8\x85\xa3',
'UnicodeCharacter0xda4e': '\xed\xa9\x8e',
'HangulSyllableMyeop': '\xeb\xaa\x8a',
'YiSyllableDda': '\xea\x84\xbf',
'YiSyllableDde': '\xea\x85\x89',
'UnicodeCharacter0xef63': '\xee\xbd\xa3',
'LatinCapitalLetterEWithMacronAndAcute': '\xe1\xb8\x96',
'UnicodeCharacter0xef61': '\xee\xbd\xa1',
'UnicodeCharacter0xef60': '\xee\xbd\xa0',
'UnicodeCharacter0xef67': '\xee\xbd\xa7',
'UnicodeCharacter0xef66': '\xee\xbd\xa6',
'UnicodeCharacter0xef65': '\xee\xbd\xa5',
'UnicodeCharacter0xef64': '\xee\xbd\xa4',
'BengaliLetterGa': '\xe0\xa6\x97',
'UnicodeCharacter0xef69': '\xee\xbd\xa9',
'UnicodeCharacter0xef68': '\xee\xbd\xa8',
'HangulSyllableNaelb': '\xeb\x82\xbf',
'UnicodeCharacter0xe5c3': '\xee\x97\x83',
'YiSyllableDdu': '\xea\x85\x8d',
'HangulSyllableDaec': '\xeb\x8c\x97',
'HangulSyllableSseunh': '\xec\x93\xb6',
'HangulSyllableDaeb': '\xeb\x8c\x91',
'HalfwidthHangulLetterYi': '\xef\xbf\x9b',
'HangulSyllableSseung': '\xec\x94\x85',
'HangulSyllableDdo': '\xeb\x98\x90',
'ArabicLigatureBehWithJeemIsolatedForm': '\xef\xb0\x85',
'UnicodeCharacter0xeea2': '\xee\xba\xa2',
'UnicodeCharacter0xe1aa': '\xee\x86\xaa',
'UnicodeCharacter0xe1ac': '\xee\x86\xac',
'UnicodeCharacter0xe1ab': '\xee\x86\xab',
'UnicodeCharacter0xe1ae': '\xee\x86\xae',
'HangulSyllableGgyiss': '\xeb\x81\xb4',
'UnicodeCharacter0xe1af': '\xee\x86\xaf',
'UnicodeCharacter0xef6c': '\xee\xbd\xac',
'CjkUnifiedIdeograph94d9': '\xe9\x93\x99',
'DevanagariLetterDddha': '\xe0\xa5\x9c',
'HangulSyllableNwaeng': '\xeb\x87\x85',
'DentistrySymbolLightUpAndHorizontalWithCircle': '\xe2\x8f\x82',
'UnicodeCharacter0xef6e': '\xee\xbd\xae',
'CjkUnifiedIdeograph81ae': '\xe8\x86\xae',
'imaginaryJ': '\xe2\x85\x89',
'LepchaSubjoinedLetterRa': '\xe1\xb0\xa5',
'HangulSyllableDaek': '\xeb\x8c\x98',
'HangulSyllableDaej': '\xeb\x8c\x96',
'UnicodeCharacter0xe5c1': '\xee\x97\x81',
'HangulSyllableDaeh': '\xeb\x8c\x9b',
'EthiopicSyllableFwee': '\xe1\x8e\x8a',
'HangulSyllableJjwilt': '\xec\xae\xa9',
'HangulSyllableJjwils': '\xec\xae\xa8',
'HangulSyllableDaen': '\xeb\x8c\x84',
'HangulSyllableJjwilm': '\xec\xae\xa6',
'HangulSyllableUng': '\xec\x9b\x85',
'HangulSyllableDaem': '\xeb\x8c\x90',
'HangulSyllableJjwilh': '\xec\xae\xab',
'HangulSyllableJjwilg': '\xec\xae\xa5',
'HangulSyllableUnj': '\xec\x9a\xb5',
'HangulSyllableDael': '\xeb\x8c\x88',
'HangulSyllableJjwilb': '\xec\xae\xa7',
'HangulSyllableDaes': '\xeb\x8c\x93',
'CjkUnifiedIdeograph94d2': '\xe9\x93\x92',
'HangulSyllableHogs': '\xed\x98\xbb',
'CjkUnifiedIdeograph91c1': '\xe9\x87\x81',
'CjkUnifiedIdeograph9034': '\xe9\x80\xb4',
'UnicodeCharacter0xe91a': '\xee\xa4\x9a',
'CjkUnifiedIdeograph830f': '\xe8\x8c\x8f',
'HangulSyllableDaep': '\xeb\x8c\x9a',
'HangulSyllableJjigg': '\xec\xb0\x8e',
'BraillePatternDots1568': '\xe2\xa2\xb1',
'LimbuSmallLetterNa': '\xe1\xa4\xb4',
'CjkUnifiedIdeograph9035': '\xe9\x80\xb5',
'HangulSyllableBbag': '\xeb\xb9\xa1',
'KayahLiLetterTha': '\xea\xa4\x9e',
'HangulSyllableGyogs': '\xea\xb5\x93',
'HangulSyllableTwaegs': '\xed\x87\x9b',
'BraillePatternDots1567': '\xe2\xa1\xb1',
'IntersectionWithDot': '\xe2\xa9\x80',
'HangulSyllableTwaegg': '\xed\x87\x9a',
'RightSidewaysUBracket': '\xe2\xb8\xa7',
'HangulSyllableGyogg': '\xea\xb5\x92',
'HangulSyllableDaelt': '\xeb\x8c\x8d',
'UnicodeCharacter0xd91b': '\xed\xa4\x9b',
'HangulSyllableJwaelm': '\xec\xa2\xb2',
'UnicodeCharacter0xe206': '\xee\x88\x86',
'GujaratiSignCandrabindu': '\xe0\xaa\x81',
'LeftBarbUpRightBarbUpHarpoon': '\xe2\xa5\x8e',
'HangulSyllableDaelp': '\xeb\x8c\x8e',
'CjkUnifiedIdeograph8d50': '\xe8\xb5\x90',
'CjkUnifiedIdeograph89ea': '\xe8\xa7\xaa',
'CanadianSyllabicsMwoo': '\xe1\x92\xb4',
'CjkUnifiedIdeograph89ec': '\xe8\xa7\xac',
'LatinSmallLetterDWithMiddleTilde': '\xe1\xb5\xad',
'UnicodeCharacter0xee8': '\xe0\xbb\xa8',
'CjkUnifiedIdeograph3a08': '\xe3\xa8\x88',
'CjkUnifiedIdeograph3a09': '\xe3\xa8\x89',
'CjkUnifiedIdeograph3a06': '\xe3\xa8\x86',
'UnicodeCharacter0xee4': '\xe0\xbb\xa4',
'UnicodeCharacter0xee7': '\xe0\xbb\xa7',
'UnicodeCharacter0xee6': '\xe0\xbb\xa6',
'CjkUnifiedIdeograph3a02': '\xe3\xa8\x82',
'UnicodeCharacter0xee0': '\xe0\xbb\xa0',
'UnicodeCharacter0xee3': '\xe0\xbb\xa3',
'UnicodeCharacter0xee2': '\xe0\xbb\xa2',
'CjkUnifiedIdeograph9532': '\xe9\x94\xb2',
'YiSyllableNbox': '\xea\x82\x82',
'BalineseLetterWa': '\xe1\xac\xaf',
'BalineseLetterTzirSasak': '\xe1\xad\x87',
'YiSyllableNbop': '\xea\x82\x84',
'HangulSyllableJyunh': '\xec\xa5\xb2',
'CjkUnifiedIdeograph9534': '\xe9\x94\xb4',
'HangulSyllableGgyeog': '\xea\xbb\xb5',
'HangulSyllableGgyeod': '\xea\xbb\xbb',
'HangulSyllableGgyeob': '\xea\xbc\x85',
'CjkUnifiedIdeograph9537': '\xe9\x94\xb7',
'EthiopicSyllableKyu': '\xe2\xb7\x89',
'HangulSyllableGgyeon': '\xea\xbb\xb8',
'HangulSyllableGgyeol': '\xea\xbb\xbc',
'CjkUnifiedIdeograph9536': '\xe9\x94\xb6',
'HangulSyllableGgyeoj': '\xea\xbc\x8a',
'HangulSyllableGgyeok': '\xea\xbc\x8c',
'HangulSyllableGgyeoh': '\xea\xbc\x8f',
'ArabicindicDigitOne': '\xd9\xa1',
'HangulSyllableGgyeot': '\xea\xbc\x8d',
'EthiopicSyllableKya': '\xe2\xb7\x88',
'LeftwardsOpenheadedArrow': '\xe2\x87\xbd',
'HangulSyllableGgyeos': '\xea\xbc\x87',
'HangulSyllableGgyeop': '\xea\xbc\x8e',
'EthiopicSyllableKye': '\xe2\xb7\x8d',
'EthiopicSyllableKyi': '\xe2\xb7\x8a',
'CircledGreaterthan': '\xe2\xa7\x81',
'HangulSyllableBbas': '\xeb\xb9\xb3',
'UnicodeCharacter0xfa2f': '\xef\xa8\xaf',
'HangulSyllableTyilp': '\xed\x8b\xa2',
'UnicodeCharacter0xe91d': '\xee\xa4\x9d',
'HangulSyllableNeogs': '\xeb\x84\x8b',
'UnicodeCharacter0xee04': '\xee\xb8\x84',
'PrecedesButNotEquivalentTo': '\xe2\x8b\xa8',
'CjkUnifiedIdeograph89e1': '\xe8\xa7\xa1',
'CjkUnifiedIdeograph89e2': '\xe8\xa7\xa2',
'CjkUnifiedIdeograph89e3': '\xe8\xa7\xa3',
'CjkUnifiedIdeograph89e4': '\xe8\xa7\xa4',
'CjkUnifiedIdeograph89e5': '\xe8\xa7\xa5',
'CjkUnifiedIdeograph89e6': '\xe8\xa7\xa6',
'CjkUnifiedIdeograph89e7': '\xe8\xa7\xa7',
'UnicodeCharacter0xeee': '\xe0\xbb\xae',
'UnicodeCharacter0xeed': '\xe0\xbb\xad',
'CjkUnifiedIdeograph3a0d': '\xe3\xa8\x8d',
'CjkUnifiedIdeograph3a0e': '\xe3\xa8\x8e',
'UnicodeCharacter0xeea': '\xe0\xbb\xaa',
'CjkUnifiedIdeograph3a0c': '\xe3\xa8\x8c',
'UnicodeCharacter0xeec': '\xe0\xbb\xac',
'ArabicLetterTehWithThreeDotsAboveDownwards': '\xd9\xbd',
'MaleAndFemaleSign': '\xe2\x9a\xa5',
'CjkUnifiedIdeograph65cf': '\xe6\x97\x8f',
'HangulSyllableCwip': '\xec\xb8\x82',
'UnicodeCharacter0xe5c9': '\xee\x97\x89',
'HangulSyllableCwis': '\xec\xb7\xbb',
'HangulSyllableCwit': '\xec\xb8\x81',
'HangulSyllablePyalh': '\xed\x8d\x93',
'HangulSyllableCwih': '\xec\xb8\x83',
'CombiningGraveacutegrave': '\xe1\xb7\x88',
'HangulSyllableCwik': '\xec\xb8\x80',
'HangulSyllableCwil': '\xec\xb7\xb0',
'HangulSyllableCwim': '\xec\xb7\xb8',
'HangulSyllableCwin': '\xec\xb7\xac',
'CjkUnifiedIdeograph9ec8': '\xe9\xbb\x88',
'HangulSyllableCwib': '\xec\xb7\xb9',
'RomanNumeralTen': '\xe2\x85\xa9',
'HangulSyllableCwid': '\xec\xb7\xaf',
'HangulSyllableRaebs': '\xeb\x9e\xaa',
'EthiopicSyllableZzo': '\xe2\xb6\xb6',
'AplFunctionalSymbolCircleUnderbar': '\xe2\x8d\x9c',
'EthiopicSyllableZzi': '\xe2\xb6\xb2',
'EthiopicSyllableZze': '\xe2\xb6\xb5',
'EthiopicSyllableZza': '\xe2\xb6\xb0',
'LatinCapitalLetterBWithHook': '\xc6\x81',
'ArabicLigatureNoonWithJeemIsolatedForm': '\xef\xb1\x8b',
'TwoLogicalOrOperator': '\xe2\xa8\x88',
'EthiopicSyllableZzu': '\xe2\xb6\xb1',
'CjkUnifiedIdeograph8e58': '\xe8\xb9\x98',
'CjkUnifiedIdeograph7f85': '\xe7\xbe\x85',
'CjkUnifiedIdeograph7f84': '\xe7\xbe\x84',
'CjkUnifiedIdeograph7f87': '\xe7\xbe\x87',
'CjkUnifiedIdeograph7f86': '\xe7\xbe\x86',
'CjkUnifiedIdeograph7f81': '\xe7\xbe\x81',
'CjkUnifiedIdeograph7d5a': '\xe7\xb5\x9a',
'CjkUnifiedIdeograph7f83': '\xe7\xbe\x83',
'CjkUnifiedIdeograph7f82': '\xe7\xbe\x82',
'CjkUnifiedIdeograph7f89': '\xe7\xbe\x89',
'CjkUnifiedIdeograph7f88': '\xe7\xbe\x88',
'HangulLetterNieunhieuh': '\xe3\x84\xb6',
'CjkUnifiedIdeograph46e3': '\xe4\x9b\xa3',
'CjkUnifiedIdeograph46e2': '\xe4\x9b\xa2',
'CjkUnifiedIdeograph46e1': '\xe4\x9b\xa1',
'CjkUnifiedIdeograph46e0': '\xe4\x9b\xa0',
'CjkUnifiedIdeograph46e7': '\xe4\x9b\xa7',
'CjkUnifiedIdeograph46e6': '\xe4\x9b\xa6',
'CjkUnifiedIdeograph46e5': '\xe4\x9b\xa5',
'CjkUnifiedIdeograph46e4': '\xe4\x9b\xa4',
'UnicodeCharacter0xe4d6': '\xee\x93\x96',
'CjkUnifiedIdeograph46e9': '\xe4\x9b\xa9',
'CjkUnifiedIdeograph46e8': '\xe4\x9b\xa8',
'EthiopicSyllableMoa': '\xe2\xb6\x81',
'HangulSyllableNeogg': '\xeb\x84\x8a',
'ModifierLetterOpenShelf': '\xcb\xbe',
'HangulSyllableTyilg': '\xed\x8b\x9d',
'HangulJongseongIeungkiyeok': '\xe1\x87\xac',
'RightwardsQuadrupleArrow': '\xe2\xad\x86',
'ArabicLigatureLamWithAlefWithMaddaAboveFinalForm': '\xef\xbb\xb6',
'HangulSyllableMyi': '\xeb\xaf\x9c',
'TibetanDigitHalfFour': '\xe0\xbc\xad',
'CjkUnifiedIdeograph46ec': '\xe4\x9b\xac',
'CjkUnifiedIdeograph46eb': '\xe4\x9b\xab',
'CjkUnifiedIdeograph46ea': '\xe4\x9b\xaa',
'UnicodeCharacter0xf49f': '\xef\x92\x9f',
'CjkUnifiedIdeograph46ef': '\xe4\x9b\xaf',
'CjkUnifiedIdeograph46ee': '\xe4\x9b\xae',
'CjkUnifiedIdeograph46ed': '\xe4\x9b\xad',
'BlackMediumSquare': '\xe2\x97\xbc',
'UnicodeCharacter0xf49a': '\xef\x92\x9a',
'UnicodeCharacter0xd919': '\xed\xa4\x99',
'HangulSyllableTyilm': '\xed\x8b\x9e',
'CjkUnifiedIdeograph7f8e': '\xe7\xbe\x8e',
'CjkUnifiedIdeograph7f8d': '\xe7\xbe\x8d',
'CjkUnifiedIdeograph7f8f': '\xe7\xbe\x8f',
'CjkUnifiedIdeograph7f8a': '\xe7\xbe\x8a',
'CjkUnifiedIdeograph7f8c': '\xe7\xbe\x8c',
'CjkUnifiedIdeograph7f8b': '\xe7\xbe\x8b',
'UnicodeCharacter0xf49b': '\xef\x92\x9b',
'UnicodeCharacter0xd915': '\xed\xa4\x95',
'HangulSyllableDdess': '\xeb\x97\x90',
'ArabicLigatureMeemWithJeemWithMeemInitialForm': '\xef\xb6\x8d',
'UnicodeCharacter0xf510': '\xef\x94\x90',
'UnicodeCharacter0xd913': '\xed\xa4\x93',
'CjkUnifiedIdeograph5c4b': '\xe5\xb1\x8b',
'CjkUnifiedIdeograph5c4c': '\xe5\xb1\x8c',
'CjkUnifiedIdeograph583c': '\xe5\xa0\xbc',
'CjkUnifiedIdeograph5c4f': '\xe5\xb1\x8f',
'CjkUnifiedIdeograph5c4d': '\xe5\xb1\x8d',
'CjkUnifiedIdeograph5c4e': '\xe5\xb1\x8e',
'UnicodeCharacter0xd911': '\xed\xa4\x91',
'ArabicLetterFehWithDotBelow': '\xda\xa3',
'CjkUnifiedIdeograph583e': '\xe5\xa0\xbe',
'UnicodeCharacter0xf75b': '\xef\x9d\x9b',
'UnicodeCharacter0xf75c': '\xef\x9d\x9c',
'UnicodeCharacter0xf75a': '\xef\x9d\x9a',
'BlackletterCapitalZ': '\xe2\x84\xa8',
'UnicodeCharacter0xf75d': '\xef\x9d\x9d',
'UnicodeCharacter0xf75e': '\xef\x9d\x9e',
'HangulSyllableSsyuls': '\xec\x93\xa0',
'HangulSyllableSsyulp': '\xec\x93\xa2',
'BlackletterCapitalR': '\xe2\x84\x9c',
'CyrillicCapitalLetterSoftEl': '\xea\x99\xa4',
'/': '\xe2\x81\x84',
'HangulSyllableSsyult': '\xec\x93\xa1',
'CjkUnifiedIdeograph5f98': '\xe5\xbe\x98',
'CjkUnifiedIdeograph5f99': '\xe5\xbe\x99',
'HangulSyllableSsyulh': '\xec\x93\xa3',
'SquareMmCubed': '\xe3\x8e\xa3',
'BlackletterCapitalH': '\xe2\x84\x8c',
'BlackletterCapitalI': '\xe2\x84\x91',
'CjkUnifiedIdeograph5f90': '\xe5\xbe\x90',
'CjkUnifiedIdeograph5f91': '\xe5\xbe\x91',
'CjkUnifiedIdeograph5f92': '\xe5\xbe\x92',
'CjkUnifiedIdeograph5f93': '\xe5\xbe\x93',
'CjkUnifiedIdeograph5f94': '\xe5\xbe\x94',
'BlackletterCapitalC': '\xe2\x84\xad',
'CjkUnifiedIdeograph5f96': '\xe5\xbe\x96',
'CjkUnifiedIdeograph5f97': '\xe5\xbe\x97',
'CjkUnifiedIdeograph5c42': '\xe5\xb1\x82',
'CjkUnifiedIdeograph5c43': '\xe5\xb1\x83',
'CjkUnifiedIdeograph5c40': '\xe5\xb1\x80',
'CjkUnifiedIdeograph5c41': '\xe5\xb1\x81',
'CjkUnifiedIdeograph5c46': '\xe5\xb1\x86',
'CjkUnifiedIdeograph5c47': '\xe5\xb1\x87',
'CjkUnifiedIdeograph5c44': '\xe5\xb1\x84',
'CjkUnifiedIdeograph5c45': '\xe5\xb1\x85',
'CjkUnifiedIdeograph5c48': '\xe5\xb1\x88',
'CjkUnifiedIdeograph5c49': '\xe5\xb1\x89',
'SundaneseConsonantSignPanyiku': '\xe1\xae\xa3',
'LimbuSmallLetterAnusvara': '\xe1\xa4\xb2',
'UnicodeCharacter0xf458': '\xef\x91\x98',
'UnicodeCharacter0xf341': '\xef\x8d\x81',
'CjkUnifiedIdeograph5f9a': '\xe5\xbe\x9a',
'LatinCapitalLetterEWithStroke': '\xc9\x86',
'CjkUnifiedIdeograph5f9c': '\xe5\xbe\x9c',
'CjkUnifiedIdeograph5f9d': '\xe5\xbe\x9d',
'CjkUnifiedIdeograph5f9e': '\xe5\xbe\x9e',
'CjkUnifiedIdeograph5f9f': '\xe5\xbe\x9f',
'CjkUnifiedIdeograph6abb': '\xe6\xaa\xbb',
'UnicodeCharacter0xf752': '\xef\x9d\x92',
'UnicodeCharacter0xf753': '\xef\x9d\x93',
'UnicodeCharacter0xf750': '\xef\x9d\x90',
'UnicodeCharacter0xf751': '\xef\x9d\x91',
'UnicodeCharacter0xf756': '\xef\x9d\x96',
'UnicodeCharacter0xf757': '\xef\x9d\x97',
'UnicodeCharacter0xf754': '\xef\x9d\x94',
'UnicodeCharacter0xf755': '\xef\x9d\x95',
'CjkUnifiedIdeograph9bc8': '\xe9\xaf\x88',
'UnicodeCharacter0xf758': '\xef\x9d\x98',
'UnicodeCharacter0xf759': '\xef\x9d\x99',
'KangxiRadicalJade': '\xe2\xbd\x9f',
'HangulSyllableBwibs': '\xeb\xb7\xa6',
'CjkUnifiedIdeograph8cda': '\xe8\xb3\x9a',
'CjkUnifiedIdeograph95f2': '\xe9\x97\xb2',
'CjkUnifiedIdeograph8cdc': '\xe8\xb3\x9c',
'CjkUnifiedIdeograph8cdb': '\xe8\xb3\x9b',
'CjkUnifiedIdeograph8cde': '\xe8\xb3\x9e',
'CjkUnifiedIdeograph8cdd': '\xe8\xb3\x9d',
'HangulSyllableDdalm': '\xeb\x94\xba',
'CjkUnifiedIdeograph8cdf': '\xe8\xb3\x9f',
'HangulSyllableDdalb': '\xeb\x94\xbb',
'HangulSyllableDdalg': '\xeb\x94\xb9',
'CjkUnifiedIdeograph9d49': '\xe9\xb5\x89',
'CjkUnifiedIdeograph9d48': '\xe9\xb5\x88',
'CjkUnifiedIdeograph9513': '\xe9\x94\x93',
'CjkUnifiedIdeograph42cf': '\xe4\x8b\x8f',
'ChamVowelSignUe': '\xea\xa8\xb2',
'CombiningMacrongrave': '\xe1\xb7\x86',
'CjkUnifiedIdeograph9d41': '\xe9\xb5\x81',
'CjkUnifiedIdeograph9d40': '\xe9\xb5\x80',
'CjkUnifiedIdeograph9d43': '\xe9\xb5\x83',
'CjkUnifiedIdeograph9d42': '\xe9\xb5\x82',
'CjkUnifiedIdeograph9d45': '\xe9\xb5\x85',
'CjkUnifiedIdeograph9d44': '\xe9\xb5\x84',
'CjkUnifiedIdeograph9d47': '\xe9\xb5\x87',
'SquaredFallingDiagonalSlash': '\xe2\xa7\x85',
'CjkUnifiedIdeograph8873': '\xe8\xa1\xb3',
'CjkUnifiedIdeograph8872': '\xe8\xa1\xb2',
'CjkUnifiedIdeograph8871': '\xe8\xa1\xb1',
'CjkUnifiedIdeograph8870': '\xe8\xa1\xb0',
'CjkUnifiedIdeograph8877': '\xe8\xa1\xb7',
'CjkUnifiedIdeograph8876': '\xe8\xa1\xb6',
'CjkUnifiedIdeograph8875': '\xe8\xa1\xb5',
'CjkUnifiedIdeograph8874': '\xe8\xa1\xb4',
'CjkUnifiedIdeograph8879': '\xe8\xa1\xb9',
'CjkUnifiedIdeograph8878': '\xe8\xa1\xb8',
'NkoLetterOo': '\xdf\x8f',
'LatinSmallLetterDWithCurl': '\xc8\xa1',
'gammagamma': '\xe2\x84\xbd',
'GreekCapitalLetterEtaWithVaria': '\xe1\xbf\x8a',
'CjkUnifiedIdeograph5e01': '\xe5\xb8\x81',
'KangxiRadicalSelf': '\xe2\xbe\x83',
'DownwardsTripleArrow': '\xe2\xa4\x8b',
'CjkUnifiedIdeograph5e02': '\xe5\xb8\x82',
'HangulSyllableJjigs': '\xec\xb0\x8f',
'CjkUnifiedIdeograph5e03': '\xe5\xb8\x83',
'UnicodeCharacter0xab5d': '\xea\xad\x9d',
'HeavyBlackfeatheredNorthEastArrow': '\xe2\x9e\xb9',
'CjkUnifiedIdeograph5e04': '\xe5\xb8\x84',
'CjkUnifiedIdeograph9d4c': '\xe9\xb5\x8c',
'CjkUnifiedIdeograph9d4b': '\xe9\xb5\x8b',
'CjkUnifiedIdeograph9d4e': '\xe9\xb5\x8e',
'CjkUnifiedIdeograph9d4d': '\xe9\xb5\x8d',
'MetricalBreve': '\xe2\x8f\x91',
'BoxDrawingsRightDownHeavyAndLeftUpLight': '\xe2\x95\x86',
'CjkUnifiedIdeograph8cd1': '\xe8\xb3\x91',
'CjkUnifiedIdeograph8cd0': '\xe8\xb3\x90',
'CjkUnifiedIdeograph8cd3': '\xe8\xb3\x93',
'CjkUnifiedIdeograph5c1b': '\xe5\xb0\x9b',
'CjkUnifiedIdeograph8cd5': '\xe8\xb3\x95',
'CjkUnifiedIdeograph5e06': '\xe5\xb8\x86',
'CjkUnifiedIdeograph8cd7': '\xe8\xb3\x97',
'CjkUnifiedIdeograph8cd6': '\xe8\xb3\x96',
'CjkUnifiedIdeograph8cd9': '\xe8\xb3\x99',
'CjkUnifiedIdeograph8cd8': '\xe8\xb3\x98',
'UnicodeCharacter0xf498': '\xef\x92\x98',
'HiraganaLetterHu': '\xe3\x81\xb5',
'CopticCapitalLetterHori': '\xcf\xa8',
'CjkUnifiedIdeograph59d5': '\xe5\xa7\x95',
'HiraganaLetterHe': '\xe3\x81\xb8',
'LatinSmallLetterWWithRingAbove': '\xe1\xba\x98',
'CjkUnifiedIdeograph887a': '\xe8\xa1\xba',
'HiraganaLetterHa': '\xe3\x81\xaf',
'CjkUnifiedIdeograph887f': '\xe8\xa1\xbf',
'CjkUnifiedIdeograph887e': '\xe8\xa1\xbe',
'CjkUnifiedIdeograph887d': '\xe8\xa1\xbd',
'LatinSmallLetterSWithDotBelowAndDotAbove': '\xe1\xb9\xa9',
'HiraganaLetterHo': '\xe3\x81\xbb',
'CjkUnifiedIdeograph8bb9': '\xe8\xae\xb9',
'HiraganaLetterHi': '\xe3\x81\xb2',
'UnicodeCharacter0xf497': '\xef\x92\x97',
'ArabicLetterRehWithHamzaAbove': '\xdd\xac',
'UnicodeCharacter0xfb3d': '\xef\xac\xbd',
'TamilLetterAu': '\xe0\xae\x94',
'CanadianSyllabicsLhaa': '\xe1\x96\xa5',
'UnicodeCharacter0xe47b': '\xee\x91\xbb',
'UnicodeCharacter0xe47c': '\xee\x91\xbc',
'UnicodeCharacter0xe47a': '\xee\x91\xba',
'UnicodeCharacter0xe47f': '\xee\x91\xbf',
'UnicodeCharacter0xe47d': '\xee\x91\xbd',
'UnicodeCharacter0xe47e': '\xee\x91\xbe',
'CjkUnifiedIdeograph4eea': '\xe4\xbb\xaa',
'CjkUnifiedIdeograph4eeb': '\xe4\xbb\xab',
'CjkUnifiedIdeograph4eec': '\xe4\xbb\xac',
'CjkUnifiedIdeograph4eed': '\xe4\xbb\xad',
'CjkUnifiedIdeograph4eee': '\xe4\xbb\xae',
'CjkUnifiedIdeograph4eef': '\xe4\xbb\xaf',
'MeasuredAngleWithOpenArmEndingInArrowPointingRightAndUp': '\xe2\xa6\xac',
'CjkUnifiedIdeograph812a': '\xe8\x84\xaa',
'CjkUnifiedIdeograph812c': '\xe8\x84\xac',
'CjkUnifiedIdeograph812b': '\xe8\x84\xab',
'CjkUnifiedIdeograph812e': '\xe8\x84\xae',
'CjkUnifiedIdeograph812d': '\xe8\x84\xad',
'CjkUnifiedIdeograph812f': '\xe8\x84\xaf',
'KayahLiLetterKa': '\xea\xa4\x8a',
'EthiopicSyllableSzwa': '\xe1\x88\xa7',
'CjkUnifiedIdeograph8296': '\xe8\x8a\x96',
'CjkUnifiedIdeograph8295': '\xe8\x8a\x95',
'CjkUnifiedIdeograph8294': '\xe8\x8a\x94',
'CjkUnifiedIdeograph8293': '\xe8\x8a\x93',
'CjkUnifiedIdeograph8292': '\xe8\x8a\x92',
'CjkUnifiedIdeograph8291': '\xe8\x8a\x91',
'CjkUnifiedIdeograph8290': '\xe8\x8a\x90',
'CjkUnifiedIdeograph65c4': '\xe6\x97\x84',
'CjkUnifiedIdeograph8299': '\xe8\x8a\x99',
'CjkUnifiedIdeograph8298': '\xe8\x8a\x98',
'CjkUnifiedIdeograph4ee8': '\xe4\xbb\xa8',
'CjkUnifiedIdeograph4ee9': '\xe4\xbb\xa9',
'UnicodeCharacter0xe470': '\xee\x91\xb0',
'SquareMwMega': '\xe3\x8e\xbf',
'UnicodeCharacter0xe476': '\xee\x91\xb6',
'UnicodeCharacter0xe477': '\xee\x91\xb7',
'UnicodeCharacter0xe474': '\xee\x91\xb4',
'UnicodeCharacter0xe475': '\xee\x91\xb5',
'CjkUnifiedIdeograph4ee0': '\xe4\xbb\xa0',
'CjkUnifiedIdeograph4ee1': '\xe4\xbb\xa1',
'CjkUnifiedIdeograph4ee2': '\xe4\xbb\xa2',
'CjkUnifiedIdeograph4ee3': '\xe4\xbb\xa3',
'CjkUnifiedIdeograph4ee4': '\xe4\xbb\xa4',
'CjkUnifiedIdeograph4ee5': '\xe4\xbb\xa5',
'CjkUnifiedIdeograph4ee6': '\xe4\xbb\xa6',
'CjkUnifiedIdeograph4ee7': '\xe4\xbb\xa7',
'HangulSyllableBeunh': '\xeb\xb8\x92',
'HangulSyllableBeunj': '\xeb\xb8\x91',
'CjkUnifiedIdeograph9238': '\xe9\x88\xb8',
'HangulSyllableGgyinh': '\xeb\x81\xa6',
'HangulSyllableGgyinj': '\xeb\x81\xa5',
'HangulSyllableGgying': '\xeb\x81\xb5',
'HangulSyllablePogs': '\xed\x8f\xaf',
'CjkUnifiedIdeograph9da1': '\xe9\xb6\xa1',
'HangulSyllableBeung': '\xeb\xb8\xa1',
'CjkUnifiedIdeograph9575': '\xe9\x95\xb5',
'CjkUnifiedIdeograph829f': '\xe8\x8a\x9f',
'CjkUnifiedIdeograph829e': '\xe8\x8a\x9e',
'CjkUnifiedIdeograph829d': '\xe8\x8a\x9d',
'CjkUnifiedIdeograph829c': '\xe8\x8a\x9c',
'CjkUnifiedIdeograph829b': '\xe8\x8a\x9b',
'CombiningCyrillicLetterIotifiedBigYus': '\xe2\xb7\xbf',
'UnicodeCharacter0xd933': '\xed\xa4\xb3',
'CjkUnifiedIdeograph75b0': '\xe7\x96\xb0',
'CjkUnifiedIdeograph8121': '\xe8\x84\xa1',
'CjkUnifiedIdeograph66df': '\xe6\x9b\x9f',
'CjkUnifiedIdeograph8123': '\xe8\x84\xa3',
'CjkUnifiedIdeograph8122': '\xe8\x84\xa2',
'CjkUnifiedIdeograph8125': '\xe8\x84\xa5',
'CjkUnifiedIdeograph8124': '\xe8\x84\xa4',
'CjkUnifiedIdeograph8127': '\xe8\x84\xa7',
'GreekSmallLetterRhoWithPsili': '\xe1\xbf\xa4',
'CjkUnifiedIdeograph8129': '\xe8\x84\xa9',
'CjkUnifiedIdeograph8128': '\xe8\x84\xa8',
'ArabicLetterYehWithHamzaAboveIsolatedForm': '\xef\xba\x89',
'LatinSubscriptSmallLetterO': '\xe2\x82\x92',
'CjkUnifiedIdeograph5746': '\xe5\x9d\x86',
'CjkUnifiedIdeograph5747': '\xe5\x9d\x87',
'CjkUnifiedIdeograph5744': '\xe5\x9d\x84',
'CjkUnifiedIdeograph5745': '\xe5\x9d\x85',
'CjkUnifiedIdeograph5742': '\xe5\x9d\x82',
'TamilLetterAa': '\xe0\xae\x86',
'CjkUnifiedIdeograph5740': '\xe5\x9d\x80',
'CjkUnifiedIdeograph5741': '\xe5\x9d\x81',
'LatinSmallLetterTWithRetroflexHook': '\xca\x88',
'RunicLetterSowiloS': '\xe1\x9b\x8a',
'CjkUnifiedIdeograph5748': '\xe5\x9d\x88',
'CjkUnifiedIdeograph5749': '\xe5\x9d\x89',
'CjkUnifiedIdeograph9456': '\xe9\x91\x96',
'CjkUnifiedIdeograph9457': '\xe9\x91\x97',
'CjkUnifiedIdeograph9454': '\xe9\x91\x94',
'CjkUnifiedIdeograph9455': '\xe9\x91\x95',
'CjkUnifiedIdeograph9452': '\xe9\x91\x92',
'CjkUnifiedIdeograph9453': '\xe9\x91\x93',
'CjkUnifiedIdeograph9450': '\xe9\x91\x90',
'CjkUnifiedIdeograph9451': '\xe9\x91\x91',
'UnicodeCharacter0xab16': '\xea\xac\x96',
'WhiteUppointingTriangle': '\xe2\x96\xb3',
'UnicodeCharacter0xab14': '\xea\xac\x94',
'CjkUnifiedIdeograph707a': '\xe7\x81\xba',
'CjkUnifiedIdeograph9577': '\xe9\x95\xb7',
'UnicodeCharacter0xab13': '\xea\xac\x93',
'CjkUnifiedIdeograph9458': '\xe9\x91\x98',
'CjkUnifiedIdeograph9459': '\xe9\x91\x99',
'HangulSyllableBwalb': '\xeb\xb4\x9b',
'CjkUnifiedIdeograph570a': '\xe5\x9c\x8a',
'HangulSyllableByeoh': '\xeb\xb3\x97',
'HangulSyllableBwalg': '\xeb\xb4\x99',
'HangulSyllableBwalh': '\xeb\xb4\x9f',
'LeftwardsArrowFromBar': '\xe2\x86\xa4',
'HangulSyllableBwalm': '\xeb\xb4\x9a',
'HangulSyllableBwalp': '\xeb\xb4\x9e',
'HangulSyllableBwals': '\xeb\xb4\x9c',
'HangulSyllableBwalt': '\xeb\xb4\x9d',
'CjkCompatibilityIdeographf979': '\xef\xa5\xb9',
'EthiopicSyllableZzee': '\xe2\xb6\xb4',
'MyanmarDigitOne': '\xe1\x81\x81',
'CjkUnifiedIdeograph945f': '\xe9\x91\x9f',
'UnicodeCharacter0xab52': '\xea\xad\x92',
'CjkUnifiedIdeograph945d': '\xe9\x91\x9d',
'CjkUnifiedIdeograph945e': '\xe9\x91\x9e',
'CjkUnifiedIdeograph945b': '\xe9\x91\x9b',
'CjkUnifiedIdeograph945c': '\xe9\x91\x9c',
'CjkUnifiedIdeograph945a': '\xe9\x91\x9a',
'UnicodeCharacter0xab1f': '\xea\xac\x9f',
'UnicodeCharacter0x196e': '\xe1\xa5\xae',
'UnicodeCharacter0x196f': '\xe1\xa5\xaf',
'UnicodeCharacter0xab1e': '\xea\xac\x9e',
'UnicodeCharacter0xab1b': '\xea\xac\x9b',
'UnicodeCharacter0xab1c': '\xea\xac\x9c',
'UnicodeCharacter0xab1a': '\xea\xac\x9a',
'CjkUnifiedIdeograph574f': '\xe5\x9d\x8f',
'CjkUnifiedIdeograph9981': '\xe9\xa6\x81',
'CjkUnifiedIdeograph574d': '\xe5\x9d\x8d',
'CjkUnifiedIdeograph574e': '\xe5\x9d\x8e',
'CjkUnifiedIdeograph574b': '\xe5\x9d\x8b',
'CjkUnifiedIdeograph574c': '\xe5\x9d\x8c',
'CjkUnifiedIdeograph9986': '\xe9\xa6\x86',
'CjkUnifiedIdeograph574a': '\xe5\x9d\x8a',
'CjkUnifiedIdeograph9988': '\xe9\xa6\x88',
'CjkUnifiedIdeograph9989': '\xe9\xa6\x89',
'UnicodeCharacter0xab57': '\xea\xad\x97',
'HangulSyllableTelp': '\xed\x85\x9a',
'HangulSyllableTels': '\xed\x85\x98',
'HangulSyllableTelt': '\xed\x85\x99',
'HangulSyllableKyaj': '\xec\xbb\x82',
'HangulSyllableDoss': '\xeb\x8f\x98',
'UnicodeCharacter0xab55': '\xea\xad\x95',
'UnicodeCharacter0xfaec': '\xef\xab\xac',
'HangulSyllableTelb': '\xed\x85\x97',
'HangulSyllableTelg': '\xed\x85\x95',
'HangulSyllableTelh': '\xed\x85\x9b',
'UnicodeCharacter0xab58': '\xea\xad\x98',
'VaiSyllableKan': '\xea\x95\xab',
'VaiSyllableNee': '\xea\x94\xa2',
'UnicodeCharacter0xab59': '\xea\xad\x99',
'GreekSmallLetterUpsilonWithDasiaAndVaria': '\xe1\xbd\x93',
'HangulSyllableKyak': '\xec\xbb\x84',
'HalfwidthHangulLetterYeo': '\xef\xbf\x8a',
'UnicodeCharacter0xfaed': '\xef\xab\xad',
'HangulSyllableTyit': '\xed\x8b\xad',
'HangulSyllableTyis': '\xed\x8b\xa7',
'HangulSyllableTyip': '\xed\x8b\xae',
'CjkUnifiedIdeograph80bd': '\xe8\x82\xbd',
'CjkUnifiedIdeograph80be': '\xe8\x82\xbe',
'EthiopicSyllablePwa': '\xe1\x8d\x97',
'UnicodeCharacter0xa4dd': '\xea\x93\x9d',
'ThaanaLetterAinu': '\xde\xa2',
'CjkUnifiedIdeograph80ba': '\xe8\x82\xba',
'EthiopicSyllablePwe': '\xe1\x8e\x8f',
'CjkUnifiedIdeograph80bc': '\xe8\x82\xbc',
'HangulSyllableTyig': '\xed\x8b\x95',
'EthiopicSyllablePwi': '\xe1\x8e\x8d',
'HangulSyllableTyid': '\xed\x8b\x9b',
'HangulSyllableTyic': '\xed\x8b\xab',
'HangulSyllableTyib': '\xed\x8b\xa5',
'CjkUnifiedIdeograph96ae': '\xe9\x9a\xae',
'UnicodeCharacter0xed6a': '\xee\xb5\xaa',
'MongolianLetterSibeSha': '\xe1\xa1\xa7',
'SylotiNagriLetterMo': '\xea\xa0\x9d',
'UnicodeCharacter0xed6b': '\xee\xb5\xab',
'HangulSyllableKyah': '\xec\xbb\x87',
'UnicodeCharacter0xed6d': '\xee\xb5\xad',
'UnicodeCharacter0xed6f': '\xee\xb5\xaf',
'GreaterthanOrSlantedEqualToWithDotAboveLeft': '\xe2\xaa\x84',
'AplFunctionalSymbolQuadBackslash': '\xe2\x8d\x82',
'CommercialMinusSign': '\xe2\x81\x92',
'CjkUnifiedIdeograph80b4': '\xe8\x82\xb4',
'CjkUnifiedIdeograph80b5': '\xe8\x82\xb5',
'CjkUnifiedIdeograph80b6': '\xe8\x82\xb6',
'CjkUnifiedIdeograph80b7': '\xe8\x82\xb7',
'CjkUnifiedIdeograph80b0': '\xe8\x82\xb0',
'CjkUnifiedIdeograph80b1': '\xe8\x82\xb1',
'CjkUnifiedIdeograph80b2': '\xe8\x82\xb2',
'CjkUnifiedIdeograph80b3': '\xe8\x82\xb3',
'IdeographicTelegraphSymbolForDecember': '\xe3\x8b\x8b',
'CjkUnifiedIdeograph80b9': '\xe8\x82\xb9',
'UnicodeCharacter0xa4d9': '\xea\x93\x99',
'UnicodeCharacter0xa4d8': '\xea\x93\x98',
'HangulSyllableGyang': '\xea\xb1\x8d',
'KangxiRadicalGoSlowly': '\xe2\xbc\xa2',
'ArabicLetterTtehInitialForm': '\xef\xad\xa8',
'HangulSyllableGyanh': '\xea\xb0\xbe',
'LatinCapitalLetterYWithGrave': '\xe1\xbb\xb2',
'HangulSyllableGyanj': '\xea\xb0\xbd',
'Slope': '\xe2\x8c\xb3',
'CjkUnifiedIdeograph95fb': '\xe9\x97\xbb',
'UnicodeCharacter0x1de9': '\xe1\xb7\xa9',
'HangulSyllableNolp': '\xeb\x86\x86',
'HangulSyllableNols': '\xeb\x86\x84',
'CanadianSyllabicsAthapascanS': '\xe1\x94\x86',
'HalfwidthKatakanaLetterYo': '\xef\xbe\x96',
'HangulSyllableNolh': '\xeb\x86\x87',
'UnicodeCharacter0xed4d': '\xee\xb5\x8d',
'CanadianSyllabicsAthapascanM': '\xe1\x92\xbe',
'CjkUnifiedIdeograph95fa': '\xe9\x97\xba',
'UnicodeCharacter0xed65': '\xee\xb5\xa5',
'HangulSyllableNolm': '\xeb\x86\x82',
'UnicodeCharacter0xed67': '\xee\xb5\xa7',
'UnicodeCharacter0xed66': '\xee\xb5\xa6',
'CjkUnifiedIdeograph8547': '\xe8\x95\x87',
'CjkUnifiedIdeograph95ff': '\xe9\x97\xbf',
'HangulSyllableNolb': '\xeb\x86\x83',
'HangulSyllableNolg': '\xeb\x86\x81',
'HangulSyllableHaenj': '\xed\x95\xb9',
'YiSyllablePiex': '\xea\x80\xbc',
'LatinCapitalLetterAWithRingAboveAndAcute': '\xc7\xba',
'HangulSyllableDwanh': '\xeb\x8f\xa6',
'HangulSyllableDwanj': '\xeb\x8f\xa5',
'CjkUnifiedIdeograph42c2': '\xe4\x8b\x82',
'HangulSyllableHaenh': '\xed\x95\xba',
'HangulSyllableTyuh': '\xed\x8a\xb7',
'CjkUnifiedIdeograph5e0a': '\xe5\xb8\x8a',
'UnicodeCharacter0xf8aa': '\xef\xa2\xaa',
'UnicodeCharacter0xf8ac': '\xef\xa2\xac',
'UnicodeCharacter0xf8ab': '\xef\xa2\xab',
'UnicodeCharacter0xf8ae': '\xef\xa2\xae',
'CircledKatakanaHa': '\xe3\x8b\xa9',
'EthiopicSyllableZaa': '\xe1\x8b\x9b',
'CjkUnifiedIdeograph7aed': '\xe7\xab\xad',
'CyrillicSmallLetterWe': '\xd4\x9d',
'KhmerIndependentVowelLyy': '\xe1\x9e\xae',
'HangulSyllableTyum': '\xed\x8a\xac',
'HangulSyllableDdweonh': '\xeb\x9a\xbe',
'HalfwidthKatakanaLetterYa': '\xef\xbe\x94',
'VaiSyllableKee': '\xea\x94\x9e',
'ArabicLigatureFehWithHahIsolatedForm': '\xef\xb0\xae',
'BraillePatternDots145': '\xe2\xa0\x99',
'BraillePatternDots147': '\xe2\xa1\x89',
'BraillePatternDots146': '\xe2\xa0\xa9',
'CjkUnifiedIdeograph96ab': '\xe9\x9a\xab',
'CjkUnifiedIdeograph5e0f': '\xe5\xb8\x8f',
'HangulSyllableTyub': '\xed\x8a\xad',
'HangulSyllableKyam': '\xec\xba\xbc',
'SaurashtraLetterTtha': '\xea\xa2\x9d',
'BraillePatternDots148': '\xe2\xa2\x89',
'OriyaVowelSignAu': '\xe0\xad\x8c',
'UnicodeCharacter0xf8a8': '\xef\xa2\xa8',
'TurnedSansserifCapitalL': '\xe2\x85\x82',
'F': '\xce\xa6',
'CjkUnifiedIdeograph9579': '\xe9\x95\xb9',
'HangulSyllableTyug': '\xed\x8a\x9d',
'UnicodeCharacter0xf8a3': '\xef\xa2\xa3',
'UnicodeCharacter0xf8a2': '\xef\xa2\xa2',
'UnicodeCharacter0xf8a5': '\xef\xa2\xa5',
'UnicodeCharacter0xf8a4': '\xef\xa2\xa4',
'DingbatNegativeCircledSansserifDigitOne': '\xe2\x9e\x8a',
'HangulSyllableHaeng': '\xed\x96\x89',
'TurnedSansserifCapitalY': '\xe2\x85\x84',
'HangulSyllableBing': '\xeb\xb9\x99',
'OriyaVowelSignAa': '\xe0\xac\xbe',
'GreaterthanOrLessthan': '\xe2\x89\xb7',
'OriyaVowelSignAi': '\xe0\xad\x88',
'HangulSyllableDdyilt': '\xeb\x9d\x91',
'VaiSyllableMgbo': '\xea\x97\x86',
'CjkUnifiedIdeograph9578': '\xe9\x95\xb8',
'HangulSyllableDdu': '\xeb\x9a\x9c',
'BalineseLetterNa': '\xe1\xac\xa6',
'VaiSyllableMgba': '\xea\x95\x96',
'MyanmarToneMarkSgawKarenHathi': '\xe1\x81\xa3',
'DraftingPointRightwardsArrow': '\xe2\x9e\x9b',
'HangulSyllableDdi': '\xeb\x9d\xa0',
'HangulSyllableYenh': '\xec\x98\x8e',
'CjkUnifiedIdeograph3767': '\xe3\x9d\xa7',
'CjkUnifiedIdeograph9582': '\xe9\x96\x82',
'HangulSyllableDde': '\xeb\x96\xbc',
'HangulSyllableDda': '\xeb\x94\xb0',
'HangulSyllableGeunj': '\xea\xb7\xbd',
'VaiSyllableMgbe': '\xea\x97\xae',
'UnicodeCharacter0xe5ee': '\xee\x97\xae',
'HangulSyllableSwinj': '\xec\x89\xb1',
'SubsetAboveSuperset': '\xe2\xab\x93',
'UnicodeCharacter0x2728': '\xe2\x9c\xa8',
'HangulSyllableBinj': '\xeb\xb9\x89',
'CjkUnifiedIdeograph80fa': '\xe8\x83\xba',
'FullwidthLatinSmallLetterT': '\xef\xbd\x94',
'LimbuLetterCa': '\xe1\xa4\x86',
'CjkUnifiedIdeograph80fc': '\xe8\x83\xbc',
'CanadianSyllabicsSwoo': '\xe1\x93\xbe',
'MongolianLetterAliGaliPa': '\xe1\xa2\x92',
'CjkUnifiedIdeograph80ff': '\xe8\x83\xbf',
'ClockwiseIntegral': '\xe2\x88\xb1',
'CanadianSyllabicsSho': '\xe1\x94\x93',
'CanadianSyllabicsShi': '\xe1\x94\x91',
'CjkUnifiedIdeograph5e0b': '\xe5\xb8\x8b',
'CanadianSyllabicsShe': '\xe1\x94\x90',
'HangulSyllableSwinh': '\xec\x89\xb2',
'CanadianSyllabicsSha': '\xe1\x94\x95',
'CjkUnifiedIdeograph7076': '\xe7\x81\xb6',
'GeorgianCapitalLetterKhar': '\xe1\x82\xb5',
'HangulSyllableDdyags': '\xeb\x95\xab',
'HangulSyllableBweogg': '\xeb\xb6\x9e',
'HangulSyllableWeonj': '\xec\x9b\x91',
'CjkCompatibilityIdeographf97a': '\xef\xa5\xba',
'CombiningDoubleMacronBelow': '\xcd\x9f',
'VaiSyllableMgbu': '\xea\x96\xa0',
'UnicodeCharacter0xda9f': '\xed\xaa\x9f',
'CjkCompatibilityIdeographf97c': '\xef\xa5\xbc',
'UnicodeCharacter0xdf09': '\xed\xbc\x89',
'UnicodeCharacter0xdc79': '\xed\xb1\xb9',
'LimbuLetterSsa': '\xe1\xa4\x9a',
'UnicodeCharacter0xdcd8': '\xed\xb3\x98',
'ThaiCharacterChoChang': '\xe0\xb8\x8a',
'UnicodeCharacter0xdcd3': '\xed\xb3\x93',
'UnicodeCharacter0xdcd2': '\xed\xb3\x92',
'UnicodeCharacter0xdcd1': '\xed\xb3\x91',
'UnicodeCharacter0xdcd0': '\xed\xb3\x90',
'UnicodeCharacter0xdcd7': '\xed\xb3\x97',
'UnicodeCharacter0xdcd6': '\xed\xb3\x96',
'OlChikiAhad': '\xe1\xb1\xbd',
'UnicodeCharacter0xdcd4': '\xed\xb3\x94',
'UnicodeCharacter0xf40e': '\xef\x90\x8e',
'SmallDollarSign': '\xef\xb9\xa9',
'HangulSyllablePwik': '\xed\x93\xa4',
'KatakanaLetterDi': '\xe3\x83\x82',
'KatakanaLetterDo': '\xe3\x83\x89',
'UnicodeCharacter0xf7b3': '\xef\x9e\xb3',
'KatakanaLetterDa': '\xe3\x83\x80',
'KatakanaLetterDe': '\xe3\x83\x87',
'LatinSmallLetterSchwaWithHook': '\xc9\x9a',
'PhagspaLetterJa': '\xea\xa1\x86',
'KatakanaLetterDu': '\xe3\x83\x85',
'UnicodeCharacter0x3130': '\xe3\x84\xb0',
'Forking': '\xe2\xab\x9c',
'YiSyllableDdop': '\xea\x85\x87',
'HangulSyllableGyaelp': '\xea\xb1\xa2',
'UnicodeCharacter0xda9e': '\xed\xaa\x9e',
'YiSyllableDdox': '\xea\x85\x85',
'YiSyllableNgep': '\xea\x89\xad',
'UnicodeCharacter0xdcdc': '\xed\xb3\x9c',
'UnicodeCharacter0xdcdb': '\xed\xb3\x9b',
'CjkUnifiedIdeograph98ea': '\xe9\xa3\xaa',
'HangulSyllableRunj': '\xeb\xa3\xad',
'UnicodeCharacter0xdcdf': '\xed\xb3\x9f',
'YiSyllableNgex': '\xea\x89\xab',
'UnicodeCharacter0xdcdd': '\xed\xb3\x9d',
'HangulSyllableHwic': '\xed\x9c\xaf',
'HangulSyllableHwib': '\xed\x9c\xa9',
'HangulSyllableHwid': '\xed\x9c\x9f',
'HangulSyllableHwig': '\xed\x9c\x99',
'HangulSyllableCeolt': '\xec\xb2\xa5',
'HangulSyllableHwik': '\xed\x9c\xb0',
'HangulSyllableHwij': '\xed\x9c\xae',
'HangulSyllableHwim': '\xed\x9c\xa8',
'HangulSyllableCeolp': '\xec\xb2\xa6',
'HangulSyllableCeols': '\xec\xb2\xa4',
'HangulSyllableHwin': '\xed\x9c\x9c',
'HangulSyllableCeolm': '\xec\xb2\xa2',
'YiSyllableZot': '\xea\x8a\x9b',
'MeasuredAngleWithOpenArmEndingInArrowPointingDownAndLeft': '\xe2\xa6\xab',
'HangulSyllableCeolh': '\xec\xb2\xa7',
'HangulSyllableCeolg': '\xec\xb2\xa1',
'HangulSyllableCeolb': '\xec\xb2\xa3',
'CanadianSyllabicsA': '\xe1\x90\x8a',
'CanadianSyllabicsC': '\xe1\x92\xa1',
'CanadianSyllabicsE': '\xe1\x90\x81',
'UnicodeCharacter0xe66e': '\xee\x99\xae',
'CanadianSyllabicsF': '\xe1\x95\x9d',
'CanadianSyllabicsI': '\xe1\x90\x83',
'CanadianSyllabicsK': '\xe1\x92\x83',
'CanadianSyllabicsM': '\xe1\x92\xbb',
'CanadianSyllabicsL': '\xe1\x93\xaa',
'CanadianSyllabicsO': '\xe1\x90\x85',
'CanadianSyllabicsN': '\xe1\x93\x90',
'CanadianSyllabicsQ': '\xe1\x96\x85',
'CanadianSyllabicsP': '\xe1\x91\x89',
'CanadianSyllabicsS': '\xe1\x94\x85',
'CanadianSyllabicsR': '\xe1\x95\x90',
'GeorgianCapitalLetterNar': '\xe1\x82\xac',
'CanadianSyllabicsY': '\xe1\x94\xbe',
'CjkUnifiedIdeograph7a6e': '\xe7\xa9\xae',
'BraillePatternDots13478': '\xe2\xa3\x8d',
'UnicodeCharacter0xf55e': '\xef\x95\x9e',
'ArabicSmallFatha': '\xd8\x98',
'ThreeConsecutiveEqualsSigns': '\xe2\xa9\xb6',
'CombiningLeftRightArrowAbove': '\xe2\x83\xa1',
'KhmerDigitTwo': '\xe1\x9f\xa2',
'HangulSyllableSselb': '\xec\x8e\x8f',
'HangulSyllableSselg': '\xec\x8e\x8d',
'CjkUnifiedIdeograph3fec': '\xe3\xbf\xac',
'HangulSyllableSselh': '\xec\x8e\x93',
'LeftwardsArrowAboveReverseAlmostEqualTo': '\xe2\xad\x82',
'DownwardsArrowWithTipLeftwards': '\xe2\x86\xb2',
'HangulSyllableSselm': '\xec\x8e\x8e',
'SquareWithUpperLeftDiagonalHalfBlack': '\xe2\x97\xa9',
'HangulSyllableSselp': '\xec\x8e\x92',
'HangulSyllableSsels': '\xec\x8e\x90',
'CyrillicSmallLetterSoftEl': '\xea\x99\xa5',
'GreekCapitalLetterAlphaWithProsgegrammeni': '\xe1\xbe\xbc',
'CyrillicSmallLetterSoftEm': '\xea\x99\xa7',
'HangulSyllableRweobs': '\xeb\xa4\x96',
'UnicodeCharacter0xdee8': '\xed\xbb\xa8',
'HangulSyllableSoelm': '\xec\x87\xaa',
'UnicodeCharacter0xdee9': '\xed\xbb\xa9',
'SaurashtraLetterKa': '\xea\xa2\x92',
'CjkUnifiedIdeograph8751': '\xe8\x9d\x91',
'CjkUnifiedIdeograph7ae5': '\xe7\xab\xa5',
'CjkUnifiedIdeograph7ae4': '\xe7\xab\xa4',
'CjkUnifiedIdeograph6475': '\xe6\x91\xb5',
'CjkUnifiedIdeograph6474': '\xe6\x91\xb4',
'CjkUnifiedIdeograph6477': '\xe6\x91\xb7',
'CjkUnifiedIdeograph6476': '\xe6\x91\xb6',
'CjkUnifiedIdeograph6471': '\xe6\x91\xb1',
'CjkUnifiedIdeograph6470': '\xe6\x91\xb0',
'CjkUnifiedIdeograph6473': '\xe6\x91\xb3',
'CjkUnifiedIdeograph6472': '\xe6\x91\xb2',
'CjkUnifiedIdeograph7ae6': '\xe7\xab\xa6',
'CjkUnifiedIdeograph6479': '\xe6\x91\xb9',
'CjkUnifiedIdeograph6478': '\xe6\x91\xb8',
'CjkUnifiedIdeograph7a6a': '\xe7\xa9\xaa',
'UnicodeCharacter0xe58d': '\xee\x96\x8d',
'CjkUnifiedIdeograph7ae8': '\xe7\xab\xa8',
'UnicodeCharacter0xdf16': '\xed\xbc\x96',
'HangulSyllableGgyulh': '\xeb\x80\xb7',
'HangulSyllableGgyulm': '\xeb\x80\xb2',
'HangulSyllableGgyulb': '\xeb\x80\xb3',
'UnicodeCharacter0xf1f8': '\xef\x87\xb8',
'HangulSyllableGgyulg': '\xeb\x80\xb1',
'CjkUnifiedIdeograph7a6b': '\xe7\xa9\xab',
'UnicodeCharacter0xe58e': '\xee\x96\x8e',
'UnicodeCharacter0xf680': '\xef\x9a\x80',
'HangulSyllableGgyuls': '\xeb\x80\xb4',
'HangulSyllableGgyulp': '\xeb\x80\xb6',
'HangulSyllableGgyult': '\xeb\x80\xb5',
'HangulSyllablePenj': '\xed\x8e\x9d',
'HangulSyllablePenh': '\xed\x8e\x9e',
'HangulSyllableSolp': '\xec\x86\x9a',
'GreekCapitalLetterIotaWithPsiliAndVaria': '\xe1\xbc\xba',
'LargerThanOrEqualTo': '\xe2\xaa\xad',
'CjkUnifiedIdeograph647e': '\xe6\x91\xbe',
'CjkUnifiedIdeograph647d': '\xe6\x91\xbd',
'CjkUnifiedIdeograph647f': '\xe6\x91\xbf',
'CjkUnifiedIdeograph647a': '\xe6\x91\xba',
'CjkUnifiedIdeograph647c': '\xe6\x91\xbc',
'MongolianTodoSoftHyphen': '\xe1\xa0\x86',
'HangulSyllableJyid': '\xec\xa6\xab',
'HangulSyllableJyig': '\xec\xa6\xa5',
'HangulSyllableJyic': '\xec\xa6\xbb',
'HangulSyllableJyib': '\xec\xa6\xb5',
'HangulSyllableJyim': '\xec\xa6\xb4',
'HangulSyllableJyil': '\xec\xa6\xac',
'HangulSyllableJyin': '\xec\xa6\xa8',
'HangulSyllableJyih': '\xec\xa6\xbf',
'HangulSyllableJyik': '\xec\xa6\xbc',
'LatinSmallLetterUm': '\xea\x9d\xb8',
'ReversedTriplePrime': '\xe2\x80\xb7',
'HangulSyllableJyit': '\xec\xa6\xbd',
'HiraganaLetterVu': '\xe3\x82\x94',
'HangulSyllableJyis': '\xec\xa6\xb7',
'HangulSyllableJjwan': '\xec\xab\x9c',
'ArabicLetterWawWithHamzaAboveIsolatedForm': '\xef\xba\x85',
'HangulSyllableKyem': '\xec\xbd\x88',
'YiSyllableCyt': '\xea\x8b\x8a',
'FullwidthMacron': '\xef\xbf\xa3',
'CjkUnifiedIdeograph3634': '\xe3\x98\xb4',
'CjkUnifiedIdeograph3635': '\xe3\x98\xb5',
'CjkUnifiedIdeograph3636': '\xe3\x98\xb6',
'CjkUnifiedIdeograph3637': '\xe3\x98\xb7',
'CjkUnifiedIdeograph3630': '\xe3\x98\xb0',
'CjkUnifiedIdeograph3631': '\xe3\x98\xb1',
'CjkUnifiedIdeograph3632': '\xe3\x98\xb2',
'CjkUnifiedIdeograph3633': '\xe3\x98\xb3',
'HangulJungseongWi': '\xe1\x85\xb1',
'UnicodeCharacter0xe58a': '\xee\x96\x8a',
'CjkUnifiedIdeograph3638': '\xe3\x98\xb8',
'CjkUnifiedIdeograph3639': '\xe3\x98\xb9',
'UnicodeCharacter0xea48': '\xee\xa9\x88',
'UnicodeCharacter0xea49': '\xee\xa9\x89',
'CjkUnifiedIdeograph7704': '\xe7\x9c\x84',
'CjkUnifiedIdeograph7705': '\xe7\x9c\x85',
'CjkUnifiedIdeograph7706': '\xe7\x9c\x86',
'CjkUnifiedIdeograph7707': '\xe7\x9c\x87',
'CopticSmallLetterO': '\xe2\xb2\x9f',
'CjkUnifiedIdeograph7701': '\xe7\x9c\x81',
'CjkUnifiedIdeograph7702': '\xe7\x9c\x82',
'CircledDigitFive': '\xe2\x91\xa4',
'UnicodeCharacter0xda2f': '\xed\xa8\xaf',
'UnicodeCharacter0xda2e': '\xed\xa8\xae',
'UnicodeCharacter0xda2d': '\xed\xa8\xad',
'CjkUnifiedIdeograph7708': '\xe7\x9c\x88',
'CjkUnifiedIdeograph7709': '\xe7\x9c\x89',
'UnicodeCharacter0xda2a': '\xed\xa8\xaa',
'CircledDivisionSlash': '\xe2\x8a\x98',
'CombiningCyrillicLetterHa': '\xe2\xb7\xaf',
'HangulSyllableDyolg': '\xeb\x90\xbd',
'ArmenianSmallLetterAyb': '\xd5\xa1',
'RomanNumeralFiftyEarlyForm': '\xe2\x86\x86',
'FullwidthLatinSmallLetterU': '\xef\xbd\x95',
'CjkUnifiedIdeograph6e78': '\xe6\xb9\xb8',
'CjkUnifiedIdeograph7e7f': '\xe7\xb9\xbf',
'CjkUnifiedIdeograph770d': '\xe7\x9c\x8d',
'CjkUnifiedIdeograph770e': '\xe7\x9c\x8e',
'CjkUnifiedIdeograph770f': '\xe7\x9c\x8f',
'CjkUnifiedIdeograph579c': '\xe5\x9e\x9c',
'LatinCapitalLetterOpenO': '\xc6\x86',
'CjkUnifiedIdeograph770c': '\xe7\x9c\x8c',
'UnicodeCharacter0xda27': '\xed\xa8\xa7',
'UnicodeCharacter0xda26': '\xed\xa8\xa6',
'UnicodeCharacter0xda25': '\xed\xa8\xa5',
'UnicodeCharacter0xda24': '\xed\xa8\xa4',
'LatinCapitalLetterOpenE': '\xc6\x90',
'UnicodeCharacter0xda22': '\xed\xa8\xa2',
'UnicodeCharacter0xda21': '\xed\xa8\xa1',
'UnicodeCharacter0xda20': '\xed\xa8\xa0',
'CjkUnifiedIdeograph363d': '\xe3\x98\xbd',
'CjkUnifiedIdeograph363e': '\xe3\x98\xbe',
'CjkUnifiedIdeograph363f': '\xe3\x98\xbf',
'UnicodeCharacter0xea4e': '\xee\xa9\x8e',
'UnicodeCharacter0xea4b': '\xee\xa9\x8b',
'CjkUnifiedIdeograph363a': '\xe3\x98\xba',
'CjkUnifiedIdeograph363b': '\xe3\x98\xbb',
'CjkUnifiedIdeograph363c': '\xe3\x98\xbc',
'GreekCapitalLetterEpsilonWithDasiaAndOxia': '\xe1\xbc\x9d',
'HangulSyllableJjwaj': '\xec\xab\xae',
'UnicodeCharacter0xf1ce': '\xef\x87\x8e',
'CjkUnifiedIdeograph8308': '\xe8\x8c\x88',
'UnicodeCharacter0xe076': '\xee\x81\xb6',
'ArabicLigatureLamWithYehFinalForm': '\xef\xb2\x87',
'HangulSyllableMwegs': '\xeb\xad\xaf',
'CjkUnifiedIdeograph6e70': '\xe6\xb9\xb0',
'UnicodeCharacter0xf7ff': '\xef\x9f\xbf',
'SquareIu': '\xe3\x8d\xba',
'HangulSyllableMwegg': '\xeb\xad\xae',
'SquareIn': '\xe3\x8f\x8c',
'HangulSyllableJjwak': '\xec\xab\xb0',
'CjkUnifiedIdeograph6e73': '\xe6\xb9\xb3',
'BalineseLetterKafSasak': '\xe1\xad\x85',
'MongolianLetterSibeZha': '\xe1\xa1\xb2',
'HangulSyllablePyong': '\xed\x91\xb1',
'HangulSyllablePyonh': '\xed\x91\xa2',
'HangulSyllablePyonj': '\xed\x91\xa1',
'HangulSyllableDyolt': '\xeb\x91\x81',
'HangulSyllableDding': '\xeb\x9d\xb5',
'UnicodeCharacter0x9da': '\xe0\xa7\x9a',
'CjkRadicalCsimplifiedLong': '\xe2\xbb\x93',
'ArabicLigatureNoonWithMeemMedialForm': '\xef\xb3\xae',
'CjkUnifiedIdeograph633d': '\xe6\x8c\xbd',
'HangulSyllableDdinh': '\xeb\x9d\xa6',
'MyanmarShanDigitThree': '\xe1\x82\x93',
'HangulSyllableGugg': '\xea\xb5\xae',
'ArabicLetterTtehehFinalForm': '\xef\xad\x9f',
'HexagramForDarkeningOfTheLight': '\xe4\xb7\xa3',
'ChamLetterNgue': '\xea\xa8\x8a',
'SquareMOhm': '\xe3\x8f\x81',
'HangulSyllableCyenh': '\xec\xb3\xb2',
'InvertedLazyS': '\xe2\x88\xbe',
'HangulSyllableCyenj': '\xec\xb3\xb1',
'HangulSyllableCyeng': '\xec\xb4\x81',
'CjkUnifiedIdeograph6c83': '\xe6\xb2\x83',
'HebrewLetterFinalPe': '\xd7\xa3',
'CjkUnifiedIdeograph6c81': '\xe6\xb2\x81',
'CjkUnifiedIdeograph6c80': '\xe6\xb2\x80',
'CjkUnifiedIdeograph6c87': '\xe6\xb2\x87',
'CjkUnifiedIdeograph6c86': '\xe6\xb2\x86',
'CjkUnifiedIdeograph6c85': '\xe6\xb2\x85',
'CjkUnifiedIdeograph6c84': '\xe6\xb2\x84',
'CjkUnifiedIdeograph7b78': '\xe7\xad\xb8',
'LatinCapitalLetterAeWithAcute': '\xc7\xbc',
'ModifierLetterUnaspirated': '\xcb\xad',
'CjkUnifiedIdeograph6c88': '\xe6\xb2\x88',
'HangulSyllableRelg': '\xeb\xa0\x91',
'CjkUnifiedIdeograph70d9': '\xe7\x83\x99',
'BraillePatternDots68': '\xe2\xa2\xa0',
'HangulSyllableKyinh': '\xed\x82\x8e',
'CjkUnifiedIdeograph69b6': '\xe6\xa6\xb6',
'CjkUnifiedIdeograph9480': '\xe9\x92\x80',
'HangulSyllablePwegg': '\xed\x92\xb2',
'LatinCapitalLetterToneFive': '\xc6\xbc',
'HangulSyllableGgyeolh': '\xea\xbc\x83',
'UnicodeCharacter0xa8f1': '\xea\xa3\xb1',
'UnicodeCharacter0xa8f0': '\xea\xa3\xb0',
'UnicodeCharacter0xa8f3': '\xea\xa3\xb3',
'UnicodeCharacter0xa8f2': '\xea\xa3\xb2',
'UnicodeCharacter0xa8f5': '\xea\xa3\xb5',
'UnicodeCharacter0xa8f4': '\xea\xa3\xb4',
'UnicodeCharacter0xa8f7': '\xea\xa3\xb7',
'TransversalIntersection': '\xe2\xab\x9b',
'UnicodeCharacter0xa8f9': '\xea\xa3\xb9',
'UnicodeCharacter0xa8f8': '\xea\xa3\xb8',
'CjkUnifiedIdeograph8754': '\xe8\x9d\x94',
'CopticCapitalLetterO': '\xe2\xb2\x9e',
'ArabicLigatureTehWithMeemWithYehFinalForm': '\xef\xb6\xa3',
'ArabicThousandsSeparator': '\xd9\xac',
'CjkUnifiedIdeograph9489': '\xe9\x92\x89',
'LatinSmallCapitalLetterUWithStroke': '\xe1\xb5\xbe',
'YiSyllableZiep': '\xea\x8a\x93',
'CjkUnifiedIdeograph6c8c': '\xe6\xb2\x8c',
'CjkUnifiedIdeograph6c8b': '\xe6\xb2\x8b',
'CjkUnifiedIdeograph6c8a': '\xe6\xb2\x8a',
'CjkUnifiedIdeograph70dd': '\xe7\x83\x9d',
'CjkUnifiedIdeograph70dc': '\xe7\x83\x9c',
'CjkUnifiedIdeograph6c8f': '\xe6\xb2\x8f',
'CjkUnifiedIdeograph6c8e': '\xe6\xb2\x8e',
'CjkUnifiedIdeograph6c8d': '\xe6\xb2\x8d',
'CjkUnifiedIdeograph91a9': '\xe9\x86\xa9',
'CjkUnifiedIdeograph91a8': '\xe9\x86\xa8',
'UnicodeCharacter0xdeed': '\xed\xbb\xad',
'UnicodeCharacter0xa8fa': '\xea\xa3\xba',
'UnicodeCharacter0xa8fc': '\xea\xa3\xbc',
'UnicodeCharacter0xa8fb': '\xea\xa3\xbb',
'UnicodeCharacter0xa8fe': '\xea\xa3\xbe',
'UnicodeCharacter0xa8fd': '\xea\xa3\xbd',
'UnicodeCharacter0xa8ff': '\xea\xa3\xbf',
'BeamedSixteenthNotes': '\xe2\x99\xac',
'HangulSyllableNyob': '\xeb\x87\xb9',
'YiSyllableGax': '\xea\x87\xa3',
'YiSyllableGap': '\xea\x87\xa5',
'KannadaLetterHa': '\xe0\xb2\xb9',
'YiSyllableGat': '\xea\x87\xa2',
'CjkUnifiedIdeograph71b8': '\xe7\x86\xb8',
'CjkUnifiedIdeograph71b9': '\xe7\x86\xb9',
'YiSyllableSat': '\xea\x8c\x90',
'CjkUnifiedIdeograph8726': '\xe8\x9c\xa6',
'CjkUnifiedIdeograph71b0': '\xe7\x86\xb0',
'CjkUnifiedIdeograph71b1': '\xe7\x86\xb1',
'CjkUnifiedIdeograph71b2': '\xe7\x86\xb2',
'CjkUnifiedIdeograph71b3': '\xe7\x86\xb3',
'CjkUnifiedIdeograph71b4': '\xe7\x86\xb4',
'CjkUnifiedIdeograph71b5': '\xe7\x86\xb5',
'CjkUnifiedIdeograph71b6': '\xe7\x86\xb6',
'CjkUnifiedIdeograph71b7': '\xe7\x86\xb7',
'ThaiCharacterKhoKhwai': '\xe0\xb8\x84',
'CjkUnifiedIdeograph7d95': '\xe7\xb6\x95',
'CjkUnifiedIdeograph7d96': '\xe7\xb6\x96',
'CjkUnifiedIdeograph7d97': '\xe7\xb6\x97',
'CjkUnifiedIdeograph7d90': '\xe7\xb6\x90',
'CjkUnifiedIdeograph7d91': '\xe7\xb6\x91',
'CjkUnifiedIdeograph7d92': '\xe7\xb6\x92',
'SuperscriptLatinSmallLetterN': '\xe2\x81\xbf',
'CjkUnifiedIdeograph8723': '\xe8\x9c\xa3',
'CjkUnifiedIdeograph7d98': '\xe7\xb6\x98',
'CjkUnifiedIdeograph7d99': '\xe7\xb6\x99',
'IdeographicTelegraphSymbolForSeptember': '\xe3\x8b\x88',
'CjkUnifiedIdeograph7b33': '\xe7\xac\xb3',
'MyanmarLetterHa': '\xe1\x80\x9f',
'CjkUnifiedIdeograph8721': '\xe8\x9c\xa1',
'CjkUnifiedIdeograph8720': '\xe8\x9c\xa0',
'KatakanaLetterYu': '\xe3\x83\xa6',
'CjkUnifiedIdeograph44db': '\xe4\x93\x9b',
'CjkUnifiedIdeograph44dc': '\xe4\x93\x9c',
'CjkUnifiedIdeograph4c9d': '\xe4\xb2\x9d',
'CjkUnifiedIdeograph44da': '\xe4\x93\x9a',
'CjkUnifiedIdeograph4c9b': '\xe4\xb2\x9b',
'CjkUnifiedIdeograph4c9c': '\xe4\xb2\x9c',
'KatakanaLetterYo': '\xe3\x83\xa8',
'CjkUnifiedIdeograph44de': '\xe4\x93\x9e',
'KatakanaLetterYa': '\xe3\x83\xa4',
'UnicodeCharacter0xf7fd': '\xef\x9f\xbd',
'HangulSyllableGgyeolp': '\xea\xbc\x82',
'EthiopicTonalMarkHidet': '\xe1\x8e\x97',
'CjkUnifiedIdeograph7d9e': '\xe7\xb6\x9e',
'CjkUnifiedIdeograph7d9f': '\xe7\xb6\x9f',
'HangulSyllableMyeoss': '\xeb\xaa\x84',
'CjkUnifiedIdeograph7d9a': '\xe7\xb6\x9a',
'CjkUnifiedIdeograph7d9b': '\xe7\xb6\x9b',
'CjkUnifiedIdeograph7d9c': '\xe7\xb6\x9c',
'CjkUnifiedIdeograph8e61': '\xe8\xb9\xa1',
'CjkUnifiedIdeograph8e60': '\xe8\xb9\xa0',
'CjkUnifiedIdeograph71ba': '\xe7\x86\xba',
'CjkUnifiedIdeograph71bb': '\xe7\x86\xbb',
'CjkUnifiedIdeograph71bc': '\xe7\x86\xbc',
'CjkUnifiedIdeograph71bd': '\xe7\x86\xbd',
'CjkUnifiedIdeograph71be': '\xe7\x86\xbe',
'LatinSmallLetterEWithRetroflexHook': '\xe1\xb6\x92',
'CjkUnifiedIdeograph44d2': '\xe4\x93\x92',
'CjkUnifiedIdeograph4c97': '\xe4\xb2\x97',
'CjkUnifiedIdeograph44d0': '\xe4\x93\x90',
'CjkUnifiedIdeograph44d1': '\xe4\x93\x91',
'CombiningVerticalTilde': '\xcc\xbe',
'CjkUnifiedIdeograph44d7': '\xe4\x93\x97',
'CjkUnifiedIdeograph44d4': '\xe4\x93\x94',
'CjkUnifiedIdeograph4c91': '\xe4\xb2\x91',
'UndoSymbol': '\xe2\x8e\x8c',
'CjkUnifiedIdeograph8e65': '\xe8\xb9\xa5',
'CjkUnifiedIdeograph44d8': '\xe4\x93\x98',
'CjkUnifiedIdeograph44d9': '\xe4\x93\x99',
'CjkUnifiedIdeograph4c98': '\xe4\xb2\x98',
'GujaratiDigitEight': '\xe0\xab\xae',
'YiSyllableHnut': '\xea\x85\xbc',
'BugineseLetterA': '\xe1\xa8\x95',
'LepchaLetterGa': '\xe1\xb0\x83',
'CjkUnifiedIdeograph8e66': '\xe8\xb9\xa6',
'CjkUnifiedIdeograph7bb6': '\xe7\xae\xb6',
'UnicodeCharacter0xe212': '\xee\x88\x92',
'CjkUnifiedIdeograph7c2b': '\xe7\xb0\xab',
'CjkUnifiedIdeograph647b': '\xe6\x91\xbb',
'UnicodeCharacter0xe211': '\xee\x88\x91',
'UnicodeCharacter0xe216': '\xee\x88\x96',
'UnicodeCharacter0xe217': '\xee\x88\x97',
'UnicodeCharacter0xe214': '\xee\x88\x94',
'CjkUnifiedIdeograph7c2c': '\xe7\xb0\xac',
'UnicodeCharacter0xf091': '\xef\x82\x91',
'UnicodeCharacter0xf090': '\xef\x82\x90',
'UnicodeCharacter0xe218': '\xee\x88\x98',
'UnicodeCharacter0xe219': '\xee\x88\x99',
'UnicodeCharacter0xf095': '\xef\x82\x95',
'UnicodeCharacter0xf094': '\xef\x82\x94',
'UnicodeCharacter0xf097': '\xef\x82\x97',
'UnicodeCharacter0xf096': '\xef\x82\x96',
'CjkUnifiedIdeograph7c2a': '\xe7\xb0\xaa',
'CjkUnifiedIdeograph7c2f': '\xe7\xb0\xaf',
'HangulSyllableNyaelp': '\xeb\x83\xba',
'UnicodeCharacter0xfaee': '\xef\xab\xae',
'UnicodeCharacter0xecd8': '\xee\xb3\x98',
'CjkUnifiedIdeograph7c2d': '\xe7\xb0\xad',
'Caron': '\xcb\x87',
'UnicodeCharacter0xecd5': '\xee\xb3\x95',
'UnicodeCharacter0xecd6': '\xee\xb3\x96',
'CjkUnifiedIdeograph7c2e': '\xe7\xb0\xae',
'UnicodeCharacter0xecd0': '\xee\xb3\x90',
'UnicodeCharacter0xecd1': '\xee\xb3\x91',
'UnicodeCharacter0xecd2': '\xee\xb3\x92',
'CjkUnifiedIdeograph7e76': '\xe7\xb9\xb6',
'HangulSyllableGwigg': '\xea\xb7\x82',
'TibetanMarkIntersyllabicTsheg': '\xe0\xbc\x8b',
'UnicodeCharacter0xd81': '\xe0\xb6\x81',
'UnicodeCharacter0xd80': '\xe0\xb6\x80',
'UnicodeCharacter0xe21f': '\xee\x88\x9f',
'UnicodeCharacter0xe21d': '\xee\x88\x9d',
'UnicodeCharacter0xd84': '\xe0\xb6\x84',
'UnicodeCharacter0xf09a': '\xef\x82\x9a',
'CjkCompatibilityIdeographfa17': '\xef\xa8\x97',
'UnicodeCharacter0xf09c': '\xef\x82\x9c',
'UnicodeCharacter0xf09b': '\xef\x82\x9b',
'UnicodeCharacter0xf09e': '\xef\x82\x9e',
'UnicodeCharacter0xf09d': '\xef\x82\x9d',
'CjkCompatibilityIdeographfa10': '\xef\xa8\x90',
'UnicodeCharacter0xd894': '\xed\xa2\x94',
'HangulSyllableSsyang': '\xec\x8d\x85',
'HangulSyllableRyalh': '\xeb\x9f\x83',
'HangulSyllableNeb': '\xeb\x84\xb5',
'HangulSyllableSoelh': '\xec\x87\xaf',
'HangulSyllablePyilh': '\xed\x94\xaf',
'CjkUnifiedIdeograph7e79': '\xe7\xb9\xb9',
'HangulSyllableBbwi': '\xec\x80\xa0',
'CjkUnifiedIdeograph8420': '\xe8\x90\xa0',
'HangulSyllableNeubs': '\xeb\x8a\xa2',
'UnicodeCharacter0xecdd': '\xee\xb3\x9d',
'UnicodeCharacter0xecde': '\xee\xb3\x9e',
'UnicodeCharacter0xecdf': '\xee\xb3\x9f',
'UnicodeCharacter0xecda': '\xee\xb3\x9a',
'UnicodeCharacter0xecdb': '\xee\xb3\x9b',
'CjkUnifiedIdeograph6e7a': '\xe6\xb9\xba',
'UnicodeCharacter0xfae2': '\xef\xab\xa2',
'VulgarFractionOneEighth': '\xe2\x85\x9b',
'UnicodeCharacter0xa58': '\xe0\xa9\x98',
'HangulSyllablePyilg': '\xed\x94\xa9',
'cphi': '\xcf\x86',
'CjkUnifiedIdeograph572b': '\xe5\x9c\xab',
'UnicodeCharacter0xa50': '\xe0\xa9\x90',
'UnicodeCharacter0xa53': '\xe0\xa9\x93',
'UnicodeCharacter0xa52': '\xe0\xa9\x92',
'UnicodeCharacter0xa55': '\xe0\xa9\x95',
'UnicodeCharacter0xa54': '\xe0\xa9\x94',
'UnicodeCharacter0xa57': '\xe0\xa9\x97',
'UnicodeCharacter0xa56': '\xe0\xa9\x96',
'CjkUnifiedIdeograph8622': '\xe8\x98\xa2',
'CjkUnifiedIdeograph8623': '\xe8\x98\xa3',
'CjkUnifiedIdeograph8620': '\xe8\x98\xa0',
'CjkUnifiedIdeograph8621': '\xe8\x98\xa1',
'CjkUnifiedIdeograph8626': '\xe8\x98\xa6',
'CjkUnifiedIdeograph8627': '\xe8\x98\xa7',
'CjkUnifiedIdeograph8624': '\xe8\x98\xa4',
'CjkUnifiedIdeograph8625': '\xe8\x98\xa5',
'CjkUnifiedIdeograph8628': '\xe8\x98\xa8',
'CjkUnifiedIdeograph8629': '\xe8\x98\xa9',
'ThaanaLetterRaa': '\xde\x83',
'CjkUnifiedIdeograph67ea': '\xe6\x9f\xaa',
'CjkUnifiedIdeograph67eb': '\xe6\x9f\xab',
'CjkUnifiedIdeograph67ec': '\xe6\x9f\xac',
'LatinSmallLetterOWithLowRingInside': '\xe2\xb1\xba',
'CjkUnifiedIdeograph67ee': '\xe6\x9f\xae',
'CjkUnifiedIdeograph67ef': '\xe6\x9f\xaf',
'ModifierLetterSmallUBar': '\xe1\xb6\xb6',
'UnicodeCharacter0xf549': '\xef\x95\x89',
'CjkUnifiedIdeograph5a2a': '\xe5\xa8\xaa',
'UnicodeCharacter0xfae4': '\xef\xab\xa4',
'HangulSyllableSyong': '\xec\x88\x91',
'SyriacLetterZain': '\xdc\x99',
'HangulSyllableSyonj': '\xec\x88\x81',
'HangulSyllableSyonh': '\xec\x88\x82',
'CjkUnifiedIdeograph862b': '\xe8\x98\xab',
'CjkUnifiedIdeograph862c': '\xe8\x98\xac',
'ErrorbarredWhiteSquare': '\xe2\xa7\xae',
'CjkUnifiedIdeograph862a': '\xe8\x98\xaa',
'CjkUnifiedIdeograph862f': '\xe8\x98\xaf',
'CjkUnifiedIdeograph862d': '\xe8\x98\xad',
'SquareCmCubed': '\xe3\x8e\xa4',
'UnicodeCharacter0xfae5': '\xef\xab\xa5',
'HangulSyllableNaebs': '\xeb\x83\x86',
'UnicodeCharacter0xe91e': '\xee\xa4\x9e',
'Straightness': '\xe2\x8f\xa4',
'SinhalaVowelSignKettiPaapilla': '\xe0\xb7\x94',
'HangulSyllableBweong': '\xeb\xb6\xb1',
'UnicodeCharacter0xa5d': '\xe0\xa9\x9d',
'UnicodeCharacter0xa5f': '\xe0\xa9\x9f',
'HangulSyllableKyonj': '\xec\xbf\x89',
'HangulSyllableKyonh': '\xec\xbf\x8a',
'HangulSyllableKyong': '\xec\xbf\x99',
'HangulSyllablePyilp': '\xed\x94\xae',
'HangulSyllableSswal': '\xec\x8f\xbc',
'CjkUnifiedIdeograph67e0': '\xe6\x9f\xa0',
'CjkUnifiedIdeograph67e1': '\xe6\x9f\xa1',
'CjkUnifiedIdeograph67e2': '\xe6\x9f\xa2',
'CjkUnifiedIdeograph67e3': '\xe6\x9f\xa3',
'CjkUnifiedIdeograph67e4': '\xe6\x9f\xa4',
'CjkUnifiedIdeograph67e5': '\xe6\x9f\xa5',
'CjkUnifiedIdeograph67e6': '\xe6\x9f\xa6',
'CjkUnifiedIdeograph67e7': '\xe6\x9f\xa7',
'CjkUnifiedIdeograph67e8': '\xe6\x9f\xa8',
'CjkUnifiedIdeograph67e9': '\xe6\x9f\xa9',
'RightwardsArrowWithTail': '\xe2\x86\xa3',
'CjkUnifiedIdeograph8e6c': '\xe8\xb9\xac',
'GujaratiLetterVocalicLl': '\xe0\xab\xa1',
'CjkUnifiedIdeograph8e6b': '\xe8\xb9\xab',
'UnicodeCharacter0xfae7': '\xef\xab\xa7',
'HangulSyllableBbyup': '\xec\x81\x96',
'CjkUnifiedIdeograph8e6e': '\xe8\xb9\xae',
'ArabicLetterKhah': '\xd8\xae',
'HangulSyllableRwaels': '\xeb\xa2\xa0',
'HangulSyllableBbyut': '\xec\x81\x95',
'UnicodeCharacter0x1f58': '\xe1\xbd\x98',
'CopticSmallLetterCryptogrammicNi': '\xe2\xb2\xbd',
'CjkUnifiedIdeograph462f': '\xe4\x98\xaf',
'CjkUnifiedIdeograph462d': '\xe4\x98\xad',
'CjkUnifiedIdeograph462e': '\xe4\x98\xae',
'CjkUnifiedIdeograph462b': '\xe4\x98\xab',
'CjkUnifiedIdeograph462c': '\xe4\x98\xac',
'HangulSyllableBbyun': '\xec\x81\x80',
'CjkUnifiedIdeograph462a': '\xe4\x98\xaa',
'HangulSyllableBbyub': '\xec\x81\x8d',
'CjkUnifiedIdeograph8e6f': '\xe8\xb9\xaf',
'RightwardsOpenheadedArrow': '\xe2\x87\xbe',
'IntegralAroundAPointOperator': '\xe2\xa8\x95',
'HangulSyllableBbyug': '\xec\x80\xbd',
'CyrillicCapitalLetterIotifiedBigYus': '\xd1\xac',
'TifinaghLetterYach': '\xe2\xb5\x9e',
'CjkUnifiedIdeograph7c23': '\xe7\xb0\xa3',
'ArabicLetterTehehIsolatedForm': '\xef\xad\xa2',
'CircledHangulKiyeok': '\xe3\x89\xa0',
'CjkUnifiedIdeograph7c20': '\xe7\xb0\xa0',
'CjkUnifiedIdeograph4626': '\xe4\x98\xa6',
'CjkUnifiedIdeograph4627': '\xe4\x98\xa7',
'CjkUnifiedIdeograph4624': '\xe4\x98\xa4',
'CjkUnifiedIdeograph4625': '\xe4\x98\xa5',
'CjkUnifiedIdeograph4622': '\xe4\x98\xa2',
'CjkUnifiedIdeograph4623': '\xe4\x98\xa3',
'CjkUnifiedIdeograph4620': '\xe4\x98\xa0',
'CjkUnifiedIdeograph4621': '\xe4\x98\xa1',
'CjkUnifiedIdeograph7c26': '\xe7\xb0\xa6',
'CjkUnifiedIdeograph4628': '\xe4\x98\xa8',
'CjkUnifiedIdeograph4629': '\xe4\x98\xa9',
'UnicodeCharacter0x1f5e': '\xe1\xbd\x9e',
'UnicodeCharacter0x1f5c': '\xe1\xbd\x9c',
'HangulSyllableYeng': '\xec\x98\x9d',
'UnicodeCharacter0x1f5a': '\xe1\xbd\x9a',
'CjkUnifiedIdeograph7c25': '\xe7\xb0\xa5',
'HangulSyllableWeoh': '\xec\x9b\xa7',
'HangulSyllableWeoj': '\xec\x9b\xa2',
'CanadianSyllabicsWestcreeNwaa': '\xe1\x93\x8e',
'HangulSyllableWeol': '\xec\x9b\x94',
'HangulSyllableWeom': '\xec\x9b\x9c',
'HangulSyllableWeon': '\xec\x9b\x90',
'CjkUnifiedIdeograph872f': '\xe8\x9c\xaf',
'HangulSyllableWeob': '\xec\x9b\x9d',
'HangulSyllableWeoc': '\xec\x9b\xa3',
'HangulSyllableWeod': '\xec\x9b\x93',
'CjkUnifiedIdeograph8ec7': '\xe8\xbb\x87',
'CjkUnifiedIdeograph872d': '\xe8\x9c\xad',
'TibetanCantillationSignHeavyBeat': '\xe0\xbf\x80',
'HangulSyllableWeop': '\xec\x9b\xa6',
'CjkUnifiedIdeograph872c': '\xe8\x9c\xac',
'TaiLeLetterTone6': '\xe1\xa5\xb4',
'HangulSyllableWeot': '\xec\x9b\xa5',
'VulgarFractionThreeQuarters': '\xc2\xbe',
'HangulSyllableCoebs': '\xec\xb5\xae',
'CjkUnifiedIdeograph872a': '\xe8\x9c\xaa',
'UnicodeCharacter0xe5f7': '\xee\x97\xb7',
'CjkUnifiedIdeograph6f55': '\xe6\xbd\x95',
'CjkUnifiedIdeograph7e55': '\xe7\xb9\x95',
'CjkCompatibilityIdeographf9d8': '\xef\xa7\x98',
'GreekSmallLetterSampi': '\xcf\xa1',
'HalfwidthKatakanaLetterKe': '\xef\xbd\xb9',
'KangxiRadicalBird': '\xe2\xbf\x83',
'HalfwidthKatakanaLetterKo': '\xef\xbd\xba',
'CjkUnifiedIdeograph48c9': '\xe4\xa3\x89',
'CjkUnifiedIdeograph48c8': '\xe4\xa3\x88',
'CanadianSyllabicsNaai': '\xe1\x93\x81',
'HalfwidthKatakanaLetterKu': '\xef\xbd\xb8',
'CjkUnifiedIdeograph48c3': '\xe4\xa3\x83',
'CjkUnifiedIdeograph48c2': '\xe4\xa3\x82',
'CjkUnifiedIdeograph48c1': '\xe4\xa3\x81',
'CjkUnifiedIdeograph48c0': '\xe4\xa3\x80',
'CjkUnifiedIdeograph48c7': '\xe4\xa3\x87',
'CjkUnifiedIdeograph48c6': '\xe4\xa3\x86',
'CjkUnifiedIdeograph48c5': '\xe4\xa3\x85',
'CjkUnifiedIdeograph48c4': '\xe4\xa3\x84',
'YiSyllableLuo': '\xea\x87\x85',
'CjkRadicalCsimplifiedFrog': '\xe2\xbb\xaa',
'LatinSmallLetterLWithLineBelow': '\xe1\xb8\xbb',
'HangulSyllableJjeobs': '\xec\xa9\x9e',
'HangulLetterYu': '\xe3\x85\xa0',
'YiSyllableLux': '\xea\x87\x8f',
'CjkUnifiedIdeograph7143': '\xe7\x85\x83',
'YiSyllableLut': '\xea\x87\x8e',
'HangulSyllableBi': '\xeb\xb9\x84',
'HangulSyllableDdoebs': '\xeb\x99\xb6',
'YiSyllableLup': '\xea\x87\x91',
'YiSyllableLur': '\xea\x87\x93',
'IdeographicTelegraphSymbolForDaySeven': '\xe3\x8f\xa6',
'BoxDrawingsUpLightAndHorizontalHeavy': '\xe2\x94\xb7',
'CjkUnifiedIdeograph48cc': '\xe4\xa3\x8c',
'BeamedEighthNotes': '\xe2\x99\xab',
'CjkUnifiedIdeograph48ca': '\xe4\xa3\x8a',
'CjkUnifiedIdeograph48cf': '\xe4\xa3\x8f',
'CjkUnifiedIdeograph48ce': '\xe4\xa3\x8e',
'CjkUnifiedIdeograph48cd': '\xe4\xa3\x8d',
'MongolianLetterJa': '\xe1\xa0\xb5',
'BengaliLetterVocalicLl': '\xe0\xa7\xa1',
'ChamConsonantSignFinalM': '\xea\xa9\x8c',
'UnicodeCharacter0xf579': '\xef\x95\xb9',
'FullwidthLatinSmallLetterV': '\xef\xbd\x96',
'UnicodeCharacter0xe303': '\xee\x8c\x83',
'HangulSyllableDdelm': '\xeb\x97\x86',
'CjkUnifiedIdeograph4a76': '\xe4\xa9\xb6',
'HangulSyllableDdelh': '\xeb\x97\x8b',
'HangulSyllableDdelg': '\xeb\x97\x85',
'YiRadicalShy': '\xea\x92\xb0',
'HangulSyllableDdelb': '\xeb\x97\x87',
'HangulSyllableHwilm': '\xed\x9c\xa2',
'HangulSyllableNwelp': '\xeb\x89\x8a',
'HangulSyllableDdelt': '\xeb\x97\x89',
'HangulSyllableDdels': '\xeb\x97\x88',
'HangulSyllableDdelp': '\xeb\x97\x8a',
'UnicodeCharacter0xdae2': '\xed\xab\xa2',
'CjkUnifiedIdeograph8396': '\xe8\x8e\x96',
'HangulLetterYe': '\xe3\x85\x96',
'CjkUnifiedIdeograph8394': '\xe8\x8e\x94',
'CjkUnifiedIdeograph5289': '\xe5\x8a\x89',
'CjkUnifiedIdeograph5288': '\xe5\x8a\x88',
'CjkUnifiedIdeograph5287': '\xe5\x8a\x87',
'CjkUnifiedIdeograph5286': '\xe5\x8a\x86',
'CjkUnifiedIdeograph5285': '\xe5\x8a\x85',
'CjkUnifiedIdeograph5284': '\xe5\x8a\x84',
'CjkUnifiedIdeograph5283': '\xe5\x8a\x83',
'CjkUnifiedIdeograph5282': '\xe5\x8a\x82',
'CjkUnifiedIdeograph5281': '\xe5\x8a\x81',
'CjkUnifiedIdeograph5280': '\xe5\x8a\x80',
'HangulSyllableHwilb': '\xed\x9c\xa3',
'HangulChoseongPieupcieuc': '\xe1\x84\xa7',
'HangulSyllablePiss': '\xed\x95\x90',
'HangulSyllableSwegg': '\xec\x89\x92',
'CjkUnifiedIdeograph528f': '\xe5\x8a\x8f',
'CjkUnifiedIdeograph528e': '\xe5\x8a\x8e',
'CjkUnifiedIdeograph528d': '\xe5\x8a\x8d',
'CjkUnifiedIdeograph528c': '\xe5\x8a\x8c',
'CjkUnifiedIdeograph528b': '\xe5\x8a\x8b',
'CjkUnifiedIdeograph528a': '\xe5\x8a\x8a',
'UnicodeCharacter0xf217': '\xef\x88\x97',
'HangulSyllableSwegs': '\xec\x89\x93',
'YiSyllableNiex': '\xea\x86\x81',
'UnicodeCharacter0xf216': '\xef\x88\x96',
'ModifierLetterMidDottedToneBar': '\xea\x9c\x8a',
'YiSyllableNiep': '\xea\x86\x83',
'UnicodeCharacter0xf215': '\xef\x88\x95',
'CjkUnifiedIdeograph5f72': '\xe5\xbd\xb2',
'HangulSyllableSelp': '\xec\x85\x86',
'HangulSyllableNoelt': '\xeb\x87\x99',
'WhiteTelephone': '\xe2\x98\x8f',
'UnicodeCharacter0xe7f9': '\xee\x9f\xb9',
'HangulSyllableJjyinh': '\xec\xaf\xb6',
'HangulSyllableByut': '\xeb\xb8\x89',
'CjkUnifiedIdeograph762c': '\xe7\x98\xac',
'CjkUnifiedIdeograph762b': '\xe7\x98\xab',
'DoubleDownTack': '\xe2\xab\xaa',
'UnicodeCharacter0xe91f': '\xee\xa4\x9f',
'CjkUnifiedIdeograph762f': '\xe7\x98\xaf',
'CjkUnifiedIdeograph762e': '\xe7\x98\xae',
'CjkUnifiedIdeograph762d': '\xe7\x98\xad',
'HangulSyllableJjwaenj': '\xec\xab\xb9',
'HangulSyllableJjwaenh': '\xec\xab\xba',
'BlackLeftpointingSmallTriangle': '\xe2\x97\x82',
'UnicodeCharacter0xdc29': '\xed\xb0\xa9',
'UnicodeCharacter0xdc28': '\xed\xb0\xa8',
'HangulSyllableByuh': '\xeb\xb8\x8b',
'UnicodeCharacter0xdc25': '\xed\xb0\xa5',
'TopLeftHalfBracket': '\xe2\xb8\xa2',
'UnicodeCharacter0xdc27': '\xed\xb0\xa7',
'SinhalaLetterAeyanna': '\xe0\xb6\x87',
'UnicodeCharacter0xdc21': '\xed\xb0\xa1',
'HangulSyllableKwelg': '\xed\x80\xa1',
'UnicodeCharacter0xdc23': '\xed\xb0\xa3',
'HangulSyllableHwils': '\xed\x9c\xa4',
'CjkUnifiedIdeograph8c7d': '\xe8\xb1\xbd',
'CjkUnifiedIdeograph8c7e': '\xe8\xb1\xbe',
'CjkUnifiedIdeograph8c7f': '\xe8\xb1\xbf',
'TeluguLetterBha': '\xe0\xb0\xad',
'CjkUnifiedIdeograph5a28': '\xe5\xa8\xa8',
'CjkUnifiedIdeograph8c7b': '\xe8\xb1\xbb',
'CjkUnifiedIdeograph8c7c': '\xe8\xb1\xbc',
'CjkUnifiedIdeograph99a5': '\xe9\xa6\xa5',
'GreekCapitalLetterAlphaWithPsiliAndVaria': '\xe1\xbc\x8a',
'HangulSyllableSsyess': '\xec\x8f\x90',
'CjkUnifiedIdeograph7a65': '\xe7\xa9\xa5',
'HangulSyllableNoelm': '\xeb\x87\x96',
'CjkUnifiedIdeograph7623': '\xe7\x98\xa3',
'CjkUnifiedIdeograph7622': '\xe7\x98\xa2',
'CjkUnifiedIdeograph7621': '\xe7\x98\xa1',
'CjkUnifiedIdeograph7620': '\xe7\x98\xa0',
'CjkUnifiedIdeograph7627': '\xe7\x98\xa7',
'CjkUnifiedIdeograph5ed2': '\xe5\xbb\x92',
'CjkUnifiedIdeograph7625': '\xe7\x98\xa5',
'CjkUnifiedIdeograph7624': '\xe7\x98\xa4',
'HangulSyllableJog': '\xec\xa1\xb1',
'UnicodeCharacter0xeb98': '\xee\xae\x98',
'CjkUnifiedIdeograph7629': '\xe7\x98\xa9',
'CjkUnifiedIdeograph7628': '\xe7\x98\xa8',
'HangulSyllableJoc': '\xec\xa2\x87',
'HangulSyllableJob': '\xec\xa2\x81',
'HangulSyllableKwelm': '\xed\x80\xa2',
'HangulSyllableGgang': '\xea\xb9\xa1',
'HangulSyllableGganj': '\xea\xb9\x91',
'HangulSyllableGganh': '\xea\xb9\x92',
'HangulSyllableJos': '\xec\xa2\x83',
'HangulSyllableNoelb': '\xeb\x87\x97',
'HangulSyllableJop': '\xec\xa2\x8a',
'CjkUnifiedIdeograph8c74': '\xe8\xb1\xb4',
'CjkUnifiedIdeograph8c75': '\xe8\xb1\xb5',
'CjkUnifiedIdeograph8c76': '\xe8\xb1\xb6',
'LeftFishTail': '\xe2\xa5\xbc',
'CjkUnifiedIdeograph8c70': '\xe8\xb1\xb0',
'LatinCapitalLetterOWithTilde': '\xc3\x95',
'CjkUnifiedIdeograph8c72': '\xe8\xb1\xb2',
'CjkUnifiedIdeograph8c73': '\xe8\xb1\xb3',
'HangulSyllableTyigg': '\xed\x8b\x96',
'HalfwidthKatakanaLetterKi': '\xef\xbd\xb7',
'CjkUnifiedIdeograph8c78': '\xe8\xb1\xb8',
'CjkUnifiedIdeograph8c79': '\xe8\xb1\xb9',
'KangxiRadicalCliff': '\xe2\xbc\x9a',
'HangulSyllableKwelh': '\xed\x80\xa7',
'HangulLetterMieum': '\xe3\x85\x81',
'KhmerSymbolLekAttakMuoy': '\xe1\x9f\xb1',
'UnicodeCharacter0xdc2e': '\xed\xb0\xae',
'UnicodeCharacter0xdc2d': '\xed\xb0\xad',
'HangulSyllableSelt': '\xec\x85\x85',
'UnicodeCharacter0xdc2a': '\xed\xb0\xaa',
'HangulSyllableTyigs': '\xed\x8b\x97',
'UnicodeCharacter0xdc2b': '\xed\xb0\xab',
'CjkUnifiedIdeograph81a4': '\xe8\x86\xa4',
'CjkUnifiedIdeograph81a5': '\xe8\x86\xa5',
'CjkUnifiedIdeograph81a6': '\xe8\x86\xa6',
'CjkUnifiedIdeograph81a7': '\xe8\x86\xa7',
'CjkUnifiedIdeograph81a0': '\xe8\x86\xa0',
'GujaratiLetterRa': '\xe0\xaa\xb0',
'CjkUnifiedIdeograph81a2': '\xe8\x86\xa2',
'KhmerSymbolBuonRoc': '\xe1\xa7\xb4',
'CjkUnifiedIdeograph94da': '\xe9\x93\x9a',
'CjkUnifiedIdeograph94dc': '\xe9\x93\x9c',
'CjkUnifiedIdeograph94db': '\xe9\x93\x9b',
'CjkUnifiedIdeograph81a8': '\xe8\x86\xa8',
'CjkUnifiedIdeograph81a9': '\xe8\x86\xa9',
'CjkUnifiedIdeograph94df': '\xe9\x93\x9f',
'CjkUnifiedIdeograph591f': '\xe5\xa4\x9f',
'CjkUnifiedIdeograph591e': '\xe5\xa4\x9e',
'CjkUnifiedIdeograph591d': '\xe5\xa4\x9d',
'CjkUnifiedIdeograph591c': '\xe5\xa4\x9c',
'LatinSmallLetterKWithCaron': '\xc7\xa9',
'CjkUnifiedIdeograph591a': '\xe5\xa4\x9a',
'HangulSyllableMoelb': '\xeb\xac\x87',
'UnicodeCharacter0xe9c1': '\xee\xa7\x81',
'SyriacSublinearFullStop': '\xdc\x82',
'HangulSyllableSsoenj': '\xec\x90\xb1',
'HangulSyllableSsoenh': '\xec\x90\xb2',
'UnicodeCharacter0xf577': '\xef\x95\xb7',
'LatinSmallLetterEWithTilde': '\xe1\xba\xbd',
'HangulSyllableEgs': '\xec\x97\x93',
'LatinSmallLigatureFi': '\xef\xac\x81',
'CjkUnifiedIdeograph7a61': '\xe7\xa9\xa1',
'HangulSyllableGwenj': '\xea\xb6\xa9',
'HangulSyllableGwenh': '\xea\xb6\xaa',
'HangulSyllableGweng': '\xea\xb6\xb9',
'HangulSyllableEgg': '\xec\x97\x92',
'BoxDrawingsRightHeavyAndLeftUpLight': '\xe2\x94\xb6',
'CjkUnifiedIdeograph5917': '\xe5\xa4\x97',
'CjkUnifiedIdeograph5916': '\xe5\xa4\x96',
'CjkUnifiedIdeograph5915': '\xe5\xa4\x95',
'CjkUnifiedIdeograph5914': '\xe5\xa4\x94',
'SinhalaLetterIluyanna': '\xe0\xb6\x8f',
'CjkUnifiedIdeograph5912': '\xe5\xa4\x92',
'CjkUnifiedIdeograph5911': '\xe5\xa4\x91',
'CjkUnifiedIdeograph5910': '\xe5\xa4\x90',
'!~=': '\xe2\x89\x84',
'UnicodeCharacter0xe337': '\xee\x8c\xb7',
'lessthanequal': '\xe2\x89\xa4',
'CjkUnifiedIdeograph5919': '\xe5\xa4\x99',
'CjkUnifiedIdeograph5918': '\xe5\xa4\x98',
'CjkUnifiedIdeograph81ad': '\xe8\x86\xad',
'LatinSmallLetterCWithStroke': '\xc8\xbc',
'CjkUnifiedIdeograph81af': '\xe8\x86\xaf',
'CjkUnifiedIdeograph81aa': '\xe8\x86\xaa',
'CjkUnifiedIdeograph81ab': '\xe8\x86\xab',
'CjkUnifiedIdeograph81ac': '\xe8\x86\xac',
'CjkUnifiedIdeograph94d1': '\xe9\x93\x91',
'CjkUnifiedIdeograph94d0': '\xe9\x93\x90',
'CjkUnifiedIdeograph94d3': '\xe9\x93\x93',
'GreekSmallLetterOmegaWithOxiaAndYpogegrammeni': '\xe1\xbf\xb4',
'CjkUnifiedIdeograph94d5': '\xe9\x93\x95',
'CjkUnifiedIdeograph94d4': '\xe9\x93\x94',
'CjkUnifiedIdeograph94d7': '\xe9\x93\x97',
'CjkUnifiedIdeograph94d6': '\xe9\x93\x96',
'UnicodeCharacter0xdd6b': '\xed\xb5\xab',
'CjkUnifiedIdeograph9c64': '\xe9\xb1\xa4',
'HangulSyllableBbad': '\xeb\xb9\xa7',
'HangulSyllableBbae': '\xeb\xb9\xbc',
'HangulSyllableBbab': '\xeb\xb9\xb1',
'HangulSyllableBbac': '\xeb\xb9\xb7',
'HangulSyllableJaelg': '\xec\x9e\xb5',
'HangulSyllableBban': '\xeb\xb9\xa4',
'HangulSyllableBbal': '\xeb\xb9\xa8',
'HangulSyllableBbam': '\xeb\xb9\xb0',
'HangulSyllableBbaj': '\xeb\xb9\xb6',
'HangulSyllableBbak': '\xeb\xb9\xb8',
'HangulSyllableBbah': '\xeb\xb9\xbb',
'PresentationFormForVerticalTwoDotLeader': '\xef\xb8\xb0',
'HangulSyllableBbat': '\xeb\xb9\xb9',
'CjkUnifiedIdeograph9c67': '\xe9\xb1\xa7',
'LiraSign': '\xe2\x82\xa4',
'HangulSyllableBbap': '\xeb\xb9\xba',
'CjkUnifiedIdeograph9c61': '\xe9\xb1\xa1',
'**4': '\xe2\x81\xb4',
'SquareBaareru': '\xe3\x8c\xad',
'UnicodeCharacter0xec12': '\xee\xb0\x92',
'ArabicLetterTeh': '\xd8\xaa',
'UnicodeCharacter0xfb37': '\xef\xac\xb7',
'CjkUnifiedIdeograph7758': '\xe7\x9d\x98',
'CyrillicSmallLetterPeWithMiddleHook': '\xd2\xa7',
'UnicodeCharacter0xeb93': '\xee\xae\x93',
'UnicodeCharacter0xde93': '\xed\xba\x93',
'SyriacRbasaAbove': '\xdc\xb6',
'CjkCompatibilityIdeographfad0': '\xef\xab\x90',
'HangulSyllableBoelm': '\xeb\xb5\x92',
'HangulSyllableHyun': '\xed\x9c\xb8',
'BuhidVowelSignI': '\xe1\x9d\x92',
'BuhidVowelSignU': '\xe1\x9d\x93',
'UnicodeCharacter0xf8e5': '\xef\xa3\xa5',
'CanadianSyllabicsWestcreeShwoo': '\xe1\x94\xa0',
'UnicodeCharacter0xe7f7': '\xee\x9f\xb7',
'TeluguSignCandrabindu': '\xe0\xb0\x81',
'NewTaiLueLetterHighXa': '\xe1\xa6\x83',
'UnicodeCharacter0xfb3f': '\xef\xac\xbf',
'UnicodeCharacter0xf437': '\xef\x90\xb7',
'nexists': '\xe2\x88\x84',
'GreaterthanWithDot': '\xe2\x8b\x97',
'TagalogLetterDa': '\xe1\x9c\x87',
'ArabicLigatureKhahWithHahIsolatedForm': '\xef\xb0\x9a',
'CanadianSyllabicsWestcreeLwaa': '\xe1\x93\xa9',
'TamilLetterAi': '\xe0\xae\x90',
'HangulSyllableGunj': '\xea\xb5\xb1',
'HangulSyllableGunh': '\xea\xb5\xb2',
'HangulSyllableGung': '\xea\xb6\x81',
'UnicodeCharacter0xeb96': '\xee\xae\x96',
'CyrillicCapitalLetterIzhitsaWithDoubleGraveAccent': '\xd1\xb6',
'HangulSyllableSswilm': '\xec\x93\x82',
'HangulSyllablePyaed': '\xed\x8d\xa7',
'CjkUnifiedIdeograph7aea': '\xe7\xab\xaa',
'GreekCapitalLetterOmegaWithDasiaAndVariaAndProsgegrammeni': '\xe1\xbe\xab',
'CjkUnifiedIdeograph7aec': '\xe7\xab\xac',
'CjkUnifiedIdeograph7aeb': '\xe7\xab\xab',
'CjkUnifiedIdeograph7aee': '\xe7\xab\xae',
'GreekCapitalLetterAlphaWithPsiliAndPerispomeniAndProsgegrammeni': '\xe1\xbe\x8e',
'CjkUnifiedIdeograph7aef': '\xe7\xab\xaf',
'UnicodeCharacter0x26ce': '\xe2\x9b\x8e',
'UnicodeCharacter0x26cd': '\xe2\x9b\x8d',
'UnicodeCharacter0x26cf': '\xe2\x9b\x8f',
'UnicodeCharacter0x26ca': '\xe2\x9b\x8a',
'TibetanVowelSignVocalicRr': '\xe0\xbd\xb7',
'UnicodeCharacter0x26cc': '\xe2\x9b\x8c',
'UnicodeCharacter0x26cb': '\xe2\x9b\x8b',
'VaiSyllableHin': '\xea\x94\xa7',
'UnicodeCharacter0xde97': '\xed\xba\x97',
'GreekPsiliAndVaria': '\xe1\xbf\x8d',
'HangulSyllableRwalt': '\xeb\xa2\x85',
'CjkUnifiedIdeograph7a69': '\xe7\xa9\xa9',
'MalayalamFractionOneQuarter': '\xe0\xb5\xb3',
'BlackFlag': '\xe2\x9a\x91',
'UnicodeCharacter0xffdf': '\xef\xbf\x9f',
'LatinCapitalLetterAWithCircumflexAndGrave': '\xe1\xba\xa6',
'BlackRightwardsArrowhead': '\xe2\x9e\xa4',
'HangulSyllableBweogs': '\xeb\xb6\x9f',
'CanadianSyllabicsWestcreePwaa': '\xe1\x91\x87',
'UnicodeCharacter0xde94': '\xed\xba\x94',
'ArabicLigatureSadWithKhahInitialForm': '\xef\xb2\xb2',
'CircledNumberThirtyEight': '\xe3\x8a\xb3',
'UnicodeCharacter0xf3a9': '\xef\x8e\xa9',
'YiSyllableGiex': '\xea\x87\x9f',
'ArabicLigatureLamWithJeemIsolatedForm': '\xef\xb0\xbf',
'YiSyllableGiet': '\xea\x87\x9e',
'YiSyllableGiep': '\xea\x87\xa1',
'CjkUnifiedIdeograph7ae1': '\xe7\xab\xa1',
'CjkUnifiedIdeograph7ae0': '\xe7\xab\xa0',
'CjkUnifiedIdeograph7ae3': '\xe7\xab\xa3',
'CjkUnifiedIdeograph7ae2': '\xe7\xab\xa2',
'UnicodeCharacter0x26c9': '\xe2\x9b\x89',
'UnicodeCharacter0x26c8': '\xe2\x9b\x88',
'CjkUnifiedIdeograph7ae7': '\xe7\xab\xa7',
'BugineseLetterNyca': '\xe1\xa8\x8f',
'UnicodeCharacter0x26c5': '\xe2\x9b\x85',
'UnicodeCharacter0x26c4': '\xe2\x9b\x84',
'UnicodeCharacter0x26c7': '\xe2\x9b\x87',
'UnicodeCharacter0x26c6': '\xe2\x9b\x86',
'UnicodeCharacter0xd875': '\xed\xa1\xb5',
'UnicodeCharacter0xed38': '\xee\xb4\xb8',
'ModifierLetterCrossAccent': '\xcb\x9f',
'CjkUnifiedIdeograph6c42': '\xe6\xb1\x82',
'UnicodeCharacter0xeb46': '\xee\xad\x86',
'UnicodeCharacter0x27cb': '\xe2\x9f\x8b',
'UnicodeCharacter0xdc47': '\xed\xb1\x87',
'CjkUnifiedIdeograph842a': '\xe8\x90\xaa',
'CjkUnifiedIdeograph842b': '\xe8\x90\xab',
'CjkUnifiedIdeograph842c': '\xe8\x90\xac',
'CjkUnifiedIdeograph842d': '\xe8\x90\xad',
'CjkUnifiedIdeograph842e': '\xe8\x90\xae',
'CjkUnifiedIdeograph842f': '\xe8\x90\xaf',
'HangulSyllableRwalg': '\xeb\xa2\x81',
'CanadianSyllabicsFaai': '\xe1\x95\x94',
'FullwidthLatinSmallLetterC': '\xef\xbd\x83',
'HangulSyllableRwalh': '\xeb\xa2\x87',
'HangulSyllablePyaec': '\xed\x8d\xb7',
'UnicodeCharacter0xdd98': '\xed\xb6\x98',
'UnicodeCharacter0xd877': '\xed\xa1\xb7',
'CjkUnifiedIdeograph7cc3': '\xe7\xb3\x83',
'HangulSyllableByogg': '\xeb\xb5\xa6',
'CjkUnifiedIdeograph8ecb': '\xe8\xbb\x8b',
'CjkUnifiedIdeograph8ecc': '\xe8\xbb\x8c',
'CjkUnifiedIdeograph8eca': '\xe8\xbb\x8a',
'CjkUnifiedIdeograph8ecf': '\xe8\xbb\x8f',
'CjkUnifiedIdeograph8ecd': '\xe8\xbb\x8d',
'CjkUnifiedIdeograph8ece': '\xe8\xbb\x8e',
'NkoLowToneApostrophe': '\xdf\xb5',
'MyanmarLetterShanKa': '\xe1\x81\xb5',
'UnicodeCharacter0xec97': '\xee\xb2\x97',
'UnicodeCharacter0xdd97': '\xed\xb6\x97',
'HangulSyllableHwelt': '\xed\x9c\x89',
'UnicodeCharacter0xee68': '\xee\xb9\xa8',
'HangulSyllableRyoss': '\xeb\xa3\xa0',
'UnicodeCharacter0xc11': '\xe0\xb0\x91',
'UnicodeCharacter0x2c5f': '\xe2\xb1\x9f',
'UnicodeCharacter0xefdd': '\xee\xbf\x9d',
'LatinSmallLetterUWithDiaeresisAndAcute': '\xc7\x98',
'CjkUnifiedIdeograph8421': '\xe8\x90\xa1',
'CjkUnifiedIdeograph8422': '\xe8\x90\xa2',
'CjkUnifiedIdeograph8423': '\xe8\x90\xa3',
'CjkUnifiedIdeograph8424': '\xe8\x90\xa4',
'CjkUnifiedIdeograph8425': '\xe8\x90\xa5',
'CjkUnifiedIdeograph8426': '\xe8\x90\xa6',
'AplFunctionalSymbolQuadUpCaret': '\xe2\x8d\x93',
'CjkUnifiedIdeograph8428': '\xe8\x90\xa8',
'CjkUnifiedIdeograph8429': '\xe8\x90\xa9',
'UnicodeCharacter0xefdf': '\xee\xbf\x9f',
'UnicodeCharacter0xe38e': '\xee\x8e\x8e',
'UnicodeCharacter0xdb69': '\xed\xad\xa9',
'GlagoliticCapitalLetterZemlja': '\xe2\xb0\x88',
'EthiopicSyllableThwa': '\xe1\x8c\xa7',
'HangulSyllableHug': '\xed\x9b\x85',
'CjkUnifiedIdeograph8ec8': '\xe8\xbb\x88',
'CjkUnifiedIdeograph8ec9': '\xe8\xbb\x89',
'UnicodeCharacter0xf3a3': '\xef\x8e\xa3',
'HalfwidthHangulFiller': '\xef\xbe\xa0',
'CjkUnifiedIdeograph8ec2': '\xe8\xbb\x82',
'CjkUnifiedIdeograph8ec3': '\xe8\xbb\x83',
'CjkUnifiedIdeograph8ec0': '\xe8\xbb\x80',
'CjkUnifiedIdeograph8ec1': '\xe8\xbb\x81',
'CjkUnifiedIdeograph8ec6': '\xe8\xbb\x86',
'SubsetOfAboveNotEqualTo': '\xe2\xab\x8b',
'CjkUnifiedIdeograph8ec4': '\xe8\xbb\x84',
'CjkUnifiedIdeograph8ec5': '\xe8\xbb\x85',
'SyriacLetterShin': '\xdc\xab',
'UnicodeCharacter0xa7a9': '\xea\x9e\xa9',
'HangulSyllableDan': '\xeb\x8b\xa8',
'HangulSyllableSoelp': '\xec\x87\xae',
'HangulSyllableHuc': '\xed\x9b\x9b',
'UnicodeCharacter0xa7a8': '\xea\x9e\xa8',
'HangulSyllableHub': '\xed\x9b\x95',
'DingbatNegativeCircledSansserifDigitTwo': '\xe2\x9e\x8b',
'UnicodeCharacter0xa7a7': '\xea\x9e\xa7',
'CjkUnifiedIdeograph8398': '\xe8\x8e\x98',
'CjkUnifiedIdeograph8399': '\xe8\x8e\x99',
'SquareSantiimu': '\xe3\x8c\xa0',
'UnicodeCharacter0xd873': '\xed\xa1\xb3',
'UnicodeCharacter0xa7a6': '\xea\x9e\xa6',
'CjkUnifiedIdeograph8392': '\xe8\x8e\x92',
'CjkUnifiedIdeograph8393': '\xe8\x8e\x93',
'CjkUnifiedIdeograph8390': '\xe8\x8e\x90',
'CjkUnifiedIdeograph8391': '\xe8\x8e\x91',
'InsertionSymbol': '\xe2\x8e\x80',
'CjkUnifiedIdeograph8397': '\xe8\x8e\x97',
'ArmenianSmallLetterCha': '\xd5\xb9',
'CjkUnifiedIdeograph8395': '\xe8\x8e\x95',
'HangulSyllableKwelt': '\xed\x80\xa5',
'UnicodeCharacter0xa7a4': '\xea\x9e\xa4',
'HangulSyllableKwelp': '\xed\x80\xa6',
'HangulSyllableKwels': '\xed\x80\xa4',
'HalfwidthHangulLetterEu': '\xef\xbf\x9a',
'UnicodeCharacter0xa7a3': '\xea\x9e\xa3',
'UnicodeCharacter0xa7a2': '\xea\x9e\xa2',
'CjkUnifiedIdeograph5ed4': '\xe5\xbb\x94',
'CjkUnifiedIdeograph5ed5': '\xe5\xbb\x95',
'CjkUnifiedIdeograph5ed6': '\xe5\xbb\x96',
'CjkUnifiedIdeograph5ed7': '\xe5\xbb\x97',
'CjkUnifiedIdeograph5ed0': '\xe5\xbb\x90',
'CjkUnifiedIdeograph5ed1': '\xe5\xbb\x91',
'CombiningSnakeBelow': '\xe1\xb7\x82',
'CjkUnifiedIdeograph5ed3': '\xe5\xbb\x93',
'TeluguLetterMa': '\xe0\xb0\xae',
'UnicodeCharacter0xa7a0': '\xea\x9e\xa0',
'CjkUnifiedIdeograph5ed8': '\xe5\xbb\x98',
'CjkUnifiedIdeograph5ed9': '\xe5\xbb\x99',
'CjkUnifiedIdeograph988d': '\xe9\xa2\x8d',
'HangulSyllableJaelt': '\xec\x9e\xb9',
'TeluguAiLengthMark': '\xe0\xb1\x96',
'HangulSyllableJaels': '\xec\x9e\xb8',
'HangulSyllableJaelp': '\xec\x9e\xba',
'CjkUnifiedIdeograph839b': '\xe8\x8e\x9b',
'CjkUnifiedIdeograph839c': '\xe8\x8e\x9c',
'WhiteDiamondSuit': '\xe2\x99\xa2',
'CjkUnifiedIdeograph839a': '\xe8\x8e\x9a',
'CjkUnifiedIdeograph839f': '\xe8\x8e\x9f',
'CjkUnifiedIdeograph839d': '\xe8\x8e\x9d',
'CjkUnifiedIdeograph839e': '\xe8\x8e\x9e',
'BraillePatternDots1278': '\xe2\xa3\x83',
'HangulSyllableJaelb': '\xec\x9e\xb7',
'KayahLiLetterWa': '\xea\xa4\x9d',
'UnicodeCharacter0xa6ea': '\xea\x9b\xaa',
'HangulSyllableJaelm': '\xec\x9e\xb6',
'HangulSyllableJaelh': '\xec\x9e\xbb',
'UnicodeCharacter0xe82d': '\xee\xa0\xad',
'LatinCapitalLetterWynn': '\xc7\xb7',
'CjkUnifiedIdeograph5ede': '\xe5\xbb\x9e',
'CjkUnifiedIdeograph5edf': '\xe5\xbb\x9f',
'CjkUnifiedIdeograph5eda': '\xe5\xbb\x9a',
'CjkUnifiedIdeograph5edb': '\xe5\xbb\x9b',
'CjkUnifiedIdeograph5edc': '\xe5\xbb\x9c',
'UnicodeCharacter0xe82f': '\xee\xa0\xaf',
'HangulSyllableKeunh': '\xed\x81\xb2',
'HangulSyllableKeunj': '\xed\x81\xb1',
'UnicodeCharacter0x27cd': '\xe2\x9f\x8d',
'UnicodeCharacter0x27ce': '\xe2\x9f\x8e',
'UnicodeCharacter0x27cf': '\xe2\x9f\x8f',
'HangulSyllableHup': '\xed\x9b\x9e',
'UnicodeCharacter0x2b4d': '\xe2\xad\x8d',
'UnicodeCharacter0x2b4e': '\xe2\xad\x8e',
'UnicodeCharacter0x2b4f': '\xe2\xad\x8f',
'UnicodeCharacter0xe06e': '\xee\x81\xae',
'UnicodeCharacter0xeb67': '\xee\xad\xa7',
'HangulSyllableRang': '\xeb\x9e\x91',
'HangulSyllableNobs': '\xeb\x86\x8a',
'UnicodeCharacter0xdd90': '\xed\xb6\x90',
'UnicodeCharacter0xf3a4': '\xef\x8e\xa4',
'CjkUnifiedIdeograph3971': '\xe3\xa5\xb1',
'HangulSyllableNass': '\xeb\x82\xac',
'CyrillicSmallLetterGheWithDescender': '\xd3\xb7',
'UnicodeCharacter0xeb64': '\xee\xad\xa4',
'PresentationFormForVerticalWavyLowLine': '\xef\xb8\xb4',
'CjkUnifiedIdeograph3f18': '\xe3\xbc\x98',
'CjkUnifiedIdeograph3f19': '\xe3\xbc\x99',
'CjkUnifiedIdeograph3f16': '\xe3\xbc\x96',
'CjkUnifiedIdeograph3f17': '\xe3\xbc\x97',
'CjkUnifiedIdeograph3f14': '\xe3\xbc\x94',
'CjkUnifiedIdeograph3f15': '\xe3\xbc\x95',
'CjkUnifiedIdeograph3f12': '\xe3\xbc\x92',
'CjkUnifiedIdeograph3f13': '\xe3\xbc\x93',
'CjkUnifiedIdeograph3f10': '\xe3\xbc\x90',
'CjkUnifiedIdeograph3f11': '\xe3\xbc\x91',
'UnicodeCharacter0xe82a': '\xee\xa0\xaa',
'UnicodeCharacter0xe82c': '\xee\xa0\xac',
'UnicodeCharacter0xe82b': '\xee\xa0\xab',
'HangulSyllableRanh': '\xeb\x9e\x82',
'CjkUnifiedIdeograph3dd9': '\xe3\xb7\x99',
'HangulSyllableRanj': '\xeb\x9e\x81',
'HangulSyllableKu': '\xec\xbf\xa0',
'CjkUnifiedIdeograph3974': '\xe3\xa5\xb4',
'TeardropspokedAsterisk': '\xe2\x9c\xbb',
'HangulSyllableKa': '\xec\xb9\xb4',
'HangulSyllableKe': '\xec\xbc\x80',
'HangulSyllableKi': '\xed\x82\xa4',
'CjkUnifiedIdeograph3dd5': '\xe3\xb7\x95',
'CjkUnifiedIdeograph3dd4': '\xe3\xb7\x94',
'CjkUnifiedIdeograph3f1f': '\xe3\xbc\x9f',
'UnicodeCharacter0xa7af': '\xea\x9e\xaf',
'CjkUnifiedIdeograph3f1d': '\xe3\xbc\x9d',
'CjkUnifiedIdeograph3f1e': '\xe3\xbc\x9e',
'IdeographicTelegraphSymbolForHourThree': '\xe3\x8d\x9b',
'CjkUnifiedIdeograph3f1c': '\xe3\xbc\x9c',
'UnicodeCharacter0xa7aa': '\xea\x9e\xaa',
'CjkUnifiedIdeograph3f1a': '\xe3\xbc\x9a',
'CjkUnifiedIdeograph3dd1': '\xe3\xb7\x91',
'UnicodeCharacter0xe06f': '\xee\x81\xaf',
'UnicodeCharacter0xeb68': '\xee\xad\xa8',
'UnicodeCharacter0xea9a': '\xee\xaa\x9a',
'HangulSyllableNaelg': '\xeb\x82\xbd',
'YiSyllableYut': '\xea\x92\x81',
'CombiningInvertedBreveBelow': '\xcc\xaf',
'UnicodeCharacter0xea9c': '\xee\xaa\x9c',
'HangulSyllableKo': '\xec\xbd\x94',
'UnicodeCharacter0xe821': '\xee\xa0\xa1',
'UnicodeCharacter0xe820': '\xee\xa0\xa0',
'UnicodeCharacter0xe823': '\xee\xa0\xa3',
'UnicodeCharacter0xe822': '\xee\xa0\xa2',
'UnicodeCharacter0xe825': '\xee\xa0\xa5',
'UnicodeCharacter0xe824': '\xee\xa0\xa4',
'UnicodeCharacter0xe827': '\xee\xa0\xa7',
'UnicodeCharacter0xe06a': '\xee\x81\xaa',
'UnicodeCharacter0xe829': '\xee\xa0\xa9',
'CyrillicCapitalLetterBinocularO': '\xea\x99\xaa',
'UnicodeCharacter0xea9d': '\xee\xaa\x9d',
'DivorceSymbol': '\xe2\x9a\xae',
'UnicodeCharacter0xf8e1': '\xef\xa3\xa1',
'UnicodeCharacter0xea9f': '\xee\xaa\x9f',
'YiSyllablePox': '\xea\x81\x87',
'UnicodeCharacter0xd9ee': '\xed\xa7\xae',
'UnicodeCharacter0xf447': '\xef\x91\x87',
'YiSyllablePop': '\xea\x81\x89',
'MalayalamFractionThreeQuarters': '\xe0\xb5\xb5',
'UnicodeCharacter0xdd4f': '\xed\xb5\x8f',
'YiSyllablePot': '\xea\x81\x86',
'CjkUnifiedIdeograph6b28': '\xe6\xac\xa8',
'CjkUnifiedIdeograph6b29': '\xe6\xac\xa9',
'UnicodeCharacter0xda6c': '\xed\xa9\xac',
'SylotiNagriLetterDdo': '\xea\xa0\x92',
'CjkUnifiedIdeograph6b20': '\xe6\xac\xa0',
'CjkUnifiedIdeograph6b21': '\xe6\xac\xa1',
'CjkUnifiedIdeograph6b22': '\xe6\xac\xa2',
'CjkUnifiedIdeograph6b23': '\xe6\xac\xa3',
'CjkUnifiedIdeograph6b24': '\xe6\xac\xa4',
'CjkUnifiedIdeograph6b25': '\xe6\xac\xa5',
'CjkUnifiedIdeograph6b26': '\xe6\xac\xa6',
'CjkUnifiedIdeograph6b27': '\xe6\xac\xa7',
'UnicodeCharacter0xee86': '\xee\xba\x86',
'UnicodeCharacter0xea55': '\xee\xa9\x95',
'FullwidthApostrophe': '\xef\xbc\x87',
'CanadianSyllabicsNunavikHii': '\xe1\x95\xb6',
'UnicodeCharacter0xa7ae': '\xea\x9e\xae',
'HangulSyllableRyabs': '\xeb\x9f\x86',
'HangulSyllableSalt': '\xec\x82\xb9',
'UnicodeCharacter0xa7ad': '\xea\x9e\xad',
'HangulSyllableSalp': '\xec\x82\xba',
'HangulSyllableSals': '\xec\x82\xb8',
'HangulSyllableDweolt': '\xeb\x91\xb9',
'CjkUnifiedIdeograph3f1b': '\xe3\xbc\x9b',
'kai': '\xcf\x97',
'SymbolForEscape': '\xe2\x90\x9b',
'HangulSyllableDweolp': '\xeb\x91\xba',
'GreekSmallLetterUpsilonWithVrachy': '\xe1\xbf\xa0',
'LatinSmallLetterBottomHalfO': '\xe1\xb4\x97',
'HangulSyllableDweolm': '\xeb\x91\xb6',
'HangulSyllableSalg': '\xec\x82\xb5',
'CjkUnifiedIdeograph4b83': '\xe4\xae\x83',
'HangulSyllableDweolh': '\xeb\x91\xbb',
'YiSyllableNgat': '\xea\x89\xa0',
'HangulSyllableSalb': '\xec\x82\xb7',
'HangulSyllableSalm': '\xec\x82\xb6',
'CjkUnifiedIdeograph4b84': '\xe4\xae\x84',
'BraillePatternDots378': '\xe2\xa3\x84',
'HangulSyllableDweolg': '\xeb\x91\xb5',
'ArabicLigatureDadWithJeemIsolatedForm': '\xef\xb0\xa2',
'HangulSyllableSalh': '\xec\x82\xbb',
'HangulSyllableDweolb': '\xeb\x91\xb7',
'YiSyllableZhex': '\xea\x8d\x92',
'LatinCapitalLetterIWithMacron': '\xc4\xaa',
'**3': '\xc2\xb3',
'UnicodeCharacter0xea57': '\xee\xa9\x97',
'CjkUnifiedIdeograph4b89': '\xe4\xae\x89',
'CjkUnifiedIdeograph6b2a': '\xe6\xac\xaa',
'CjkUnifiedIdeograph6b2b': '\xe6\xac\xab',
'CjkUnifiedIdeograph6b2c': '\xe6\xac\xac',
'CjkUnifiedIdeograph6b2d': '\xe6\xac\xad',
'CjkUnifiedIdeograph6b2e': '\xe6\xac\xae',
'CjkUnifiedIdeograph6b2f': '\xe6\xac\xaf',
'CjkUnifiedIdeograph5d5b': '\xe5\xb5\x9b',
'CjkUnifiedIdeograph5d5c': '\xe5\xb5\x9c',
'CjkUnifiedIdeograph4b19': '\xe4\xac\x99',
'CjkUnifiedIdeograph4b18': '\xe4\xac\x98',
'CjkUnifiedIdeograph5d5f': '\xe5\xb5\x9f',
'CjkUnifiedIdeograph5d5d': '\xe5\xb5\x9d',
'CjkUnifiedIdeograph5d5e': '\xe5\xb5\x9e',
'NotGreaterthan': '\xe2\x89\xaf',
'CjkUnifiedIdeograph4b12': '\xe4\xac\x92',
'CjkUnifiedIdeograph4b11': '\xe4\xac\x91',
'CjkUnifiedIdeograph4b10': '\xe4\xac\x90',
'CjkUnifiedIdeograph4b17': '\xe4\xac\x97',
'CjkUnifiedIdeograph4b16': '\xe4\xac\x96',
'CjkUnifiedIdeograph4b15': '\xe4\xac\x95',
'CjkUnifiedIdeograph4b14': '\xe4\xac\x94',
'CjkUnifiedIdeograph77af': '\xe7\x9e\xaf',
'CjkUnifiedIdeograph77ae': '\xe7\x9e\xae',
'CjkUnifiedIdeograph77ad': '\xe7\x9e\xad',
'CjkUnifiedIdeograph77ac': '\xe7\x9e\xac',
'CjkUnifiedIdeograph77ab': '\xe7\x9e\xab',
'CjkUnifiedIdeograph77aa': '\xe7\x9e\xaa',
'KhmerVowelSignUu': '\xe1\x9e\xbc',
'UnicodeCharacter0xf8d2': '\xef\xa3\x92',
'UnicodeCharacter0xf7ea': '\xef\x9f\xaa',
'UnicodeCharacter0xf8d3': '\xef\xa3\x93',
'HangulSyllableHyolb': '\xed\x9a\xb3',
'EthiopicSyllableTza': '\xe1\x8d\x80',
'EthiopicSyllableTze': '\xe1\x8d\x85',
'CombiningPlusSignBelow': '\xcc\x9f',
'EthiopicSyllableTzi': '\xe1\x8d\x82',
'HangulSyllableHyolh': '\xed\x9a\xb7',
'UnicodeCharacter0xea51': '\xee\xa9\x91',
'EthiopicSyllableTzo': '\xe1\x8d\x86',
'LatinSmallLetterOWithHorn': '\xc6\xa1',
'HangulSyllableHyols': '\xed\x9a\xb4',
'HangulSyllableHyolp': '\xed\x9a\xb6',
'EthiopicSyllableTzu': '\xe1\x8d\x81',
'HangulSyllableHyolt': '\xed\x9a\xb5',
'DownTack': '\xe2\x8a\xa4',
'CjkUnifiedIdeograph77a7': '\xe7\x9e\xa7',
'CjkUnifiedIdeograph77a6': '\xe7\x9e\xa6',
'CjkUnifiedIdeograph77a5': '\xe7\x9e\xa5',
'CjkUnifiedIdeograph77a4': '\xe7\x9e\xa4',
'CjkUnifiedIdeograph77a3': '\xe7\x9e\xa3',
'CjkUnifiedIdeograph77a2': '\xe7\x9e\xa2',
'CjkUnifiedIdeograph77a1': '\xe7\x9e\xa1',
'CjkUnifiedIdeograph77a0': '\xe7\x9e\xa0',
'UnicodeCharacter0xf722': '\xef\x9c\xa2',
'HangulSyllableSwaelt': '\xec\x87\x91',
'HangulSyllableSwaels': '\xec\x87\x90',
'OriyaLetterVocalicRr': '\xe0\xad\xa0',
'CjkUnifiedIdeograph77a9': '\xe7\x9e\xa9',
'CjkUnifiedIdeograph77a8': '\xe7\x9e\xa8',
'CjkUnifiedIdeograph5d52': '\xe5\xb5\x92',
'CjkUnifiedIdeograph5d53': '\xe5\xb5\x93',
'CjkUnifiedIdeograph5d50': '\xe5\xb5\x90',
'CjkUnifiedIdeograph5d51': '\xe5\xb5\x91',
'CjkUnifiedIdeograph5d56': '\xe5\xb5\x96',
'CjkUnifiedIdeograph5d57': '\xe5\xb5\x97',
'CjkUnifiedIdeograph5d54': '\xe5\xb5\x94',
'CjkUnifiedIdeograph5d55': '\xe5\xb5\x95',
'CjkUnifiedIdeograph4b1c': '\xe4\xac\x9c',
'CjkUnifiedIdeograph4b1b': '\xe4\xac\x9b',
'CjkUnifiedIdeograph4b1a': '\xe4\xac\x9a',
'EthiopicNumberForty': '\xe1\x8d\xb5',
'CjkUnifiedIdeograph4b1f': '\xe4\xac\x9f',
'CjkUnifiedIdeograph4b1e': '\xe4\xac\x9e',
'CjkUnifiedIdeograph4b1d': '\xe4\xac\x9d',
'MalayalamLetterIi': '\xe0\xb4\x88',
'CjkCompatibilityIdeographf9d3': '\xef\xa7\x93',
'UnicodeCharacter0xea53': '\xee\xa9\x93',
'CyrillicSmallLetterZheWithDescender': '\xd2\x97',
'RomanNumeralOneThousandCD': '\xe2\x86\x80',
'HangulSyllableGyess': '\xea\xb3\x98',
'OlChikiPunctuationDoubleMucaad': '\xe1\xb1\xbf',
'UnicodeCharacter0xf66f': '\xef\x99\xaf',
'UnicodeCharacter0xea91': '\xee\xaa\x91',
'ThaanaLetterZo': '\xde\xa1',
'HangulSyllableEogg': '\xec\x96\xb6',
'UnicodeCharacter0xf66b': '\xef\x99\xab',
'HangulSyllableDdyong': '\xeb\x9a\x95',
'UnicodeCharacter0xea52': '\xee\xa9\x92',
'KhmerSymbolPiiRoc': '\xe1\xa7\xb2',
'UnicodeCharacter0xea93': '\xee\xaa\x93',
'CjkUnifiedIdeograph90f3': '\xe9\x83\xb3',
'CjkUnifiedIdeograph6fdb': '\xe6\xbf\x9b',
'GreekCapitalLetterIotaWithDasiaAndPerispomeni': '\xe1\xbc\xbf',
'CjkUnifiedIdeograph90f0': '\xe9\x83\xb0',
'CjkUnifiedIdeograph90f7': '\xe9\x83\xb7',
'CjkUnifiedIdeograph90f6': '\xe9\x83\xb6',
'CjkUnifiedIdeograph90f5': '\xe9\x83\xb5',
'CjkUnifiedIdeograph90f4': '\xe9\x83\xb4',
'UnicodeCharacter0xea95': '\xee\xaa\x95',
'CjkUnifiedIdeograph90f9': '\xe9\x83\xb9',
'CjkUnifiedIdeograph90f8': '\xe9\x83\xb8',
'HangulSyllableMeubs': '\xeb\xaf\x92',
'UnicodeCharacter0xdd49': '\xed\xb5\x89',
'ArabicLetterMeemIsolatedForm': '\xef\xbb\xa1',
'HangulSyllableTweoss': '\xed\x89\x9c',
'HangulSyllableDdyonj': '\xeb\x9a\x85',
'UnicodeCharacter0xea97': '\xee\xaa\x97',
'CjkUnifiedIdeograph39c9': '\xe3\xa7\x89',
'CjkUnifiedIdeograph39c8': '\xe3\xa7\x88',
'CjkUnifiedIdeograph39c7': '\xe3\xa7\x87',
'TripleTilde': '\xe2\x89\x8b',
'CjkUnifiedIdeograph39c5': '\xe3\xa7\x85',
'CjkUnifiedIdeograph39c4': '\xe3\xa7\x84',
'CjkUnifiedIdeograph39c3': '\xe3\xa7\x83',
'CjkUnifiedIdeograph39c2': '\xe3\xa7\x82',
'CjkUnifiedIdeograph39c1': '\xe3\xa7\x81',
'CjkUnifiedIdeograph39c0': '\xe3\xa7\x80',
'UnicodeCharacter0xea99': '\xee\xaa\x99',
'HangulSyllableKyaegs': '\xec\xbb\x8b',
'UnicodeCharacter0xf8e3': '\xef\xa3\xa3',
'UnicodeCharacter0xea98': '\xee\xaa\x98',
'CjkUnifiedIdeograph9a7a': '\xe9\xa9\xba',
'CjkUnifiedIdeograph9a7c': '\xe9\xa9\xbc',
'CjkUnifiedIdeograph9a7b': '\xe9\xa9\xbb',
'CjkUnifiedIdeograph9a7e': '\xe9\xa9\xbe',
'CjkUnifiedIdeograph9a7d': '\xe9\xa9\xbd',
'CjkUnifiedIdeograph9a7f': '\xe9\xa9\xbf',
'CjkUnifiedIdeograph90fc': '\xe9\x83\xbc',
'CjkUnifiedIdeograph90fb': '\xe9\x83\xbb',
'CjkUnifiedIdeograph90fa': '\xe9\x83\xba',
'UnicodeCharacter0xe000': '\xee\x80\x80',
'UnicodeCharacter0xe007': '\xee\x80\x87',
'CjkUnifiedIdeograph90ff': '\xe9\x83\xbf',
'CjkUnifiedIdeograph90fe': '\xe9\x83\xbe',
'CjkUnifiedIdeograph90fd': '\xe9\x83\xbd',
'UnicodeCharacter0xe009': '\xee\x80\x89',
'UnicodeCharacter0xe008': '\xee\x80\x88',
'UnicodeCharacter0xe14c': '\xee\x85\x8c',
'UnicodeCharacter0xf666': '\xef\x99\xa6',
'UnicodeCharacter0xf667': '\xef\x99\xa7',
'UnicodeCharacter0xf664': '\xef\x99\xa4',
'UnicodeCharacter0xf665': '\xef\x99\xa5',
'UnicodeCharacter0xf662': '\xef\x99\xa2',
'UnicodeCharacter0xf663': '\xef\x99\xa3',
'UnicodeCharacter0xf660': '\xef\x99\xa0',
'UnicodeCharacter0xe828': '\xee\xa0\xa8',
'UnicodeCharacter0xdd43': '\xed\xb5\x83',
'UnicodeCharacter0xeb41': '\xee\xad\x81',
'UnicodeCharacter0xf668': '\xef\x99\xa8',
'UnicodeCharacter0xec9c': '\xee\xb2\x9c',
'CjkUnifiedIdeograph9a79': '\xe9\xa9\xb9',
'CjkUnifiedIdeograph9a78': '\xe9\xa9\xb8',
'HangulSyllableCwelh': '\xec\xb7\x9b',
'CjkUnifiedIdeograph397b': '\xe3\xa5\xbb',
'CjkUnifiedIdeograph9a71': '\xe9\xa9\xb1',
'CjkUnifiedIdeograph9a70': '\xe9\xa9\xb0',
'CjkUnifiedIdeograph9404': '\xe9\x90\x84',
'CjkUnifiedIdeograph9a72': '\xe9\xa9\xb2',
'CjkUnifiedIdeograph9a75': '\xe9\xa9\xb5',
'CjkUnifiedIdeograph9a74': '\xe9\xa9\xb4',
'CjkUnifiedIdeograph9a77': '\xe9\xa9\xb7',
'CjkUnifiedIdeograph9a76': '\xe9\xa9\xb6',
'HangulSyllableBbyiss': '\xec\x82\x88',
'CjkUnifiedIdeograph39cf': '\xe3\xa7\x8f',
'CjkUnifiedIdeograph39ce': '\xe3\xa7\x8e',
'CjkUnifiedIdeograph39cd': '\xe3\xa7\x8d',
'CjkUnifiedIdeograph39cc': '\xe3\xa7\x8c',
'CjkUnifiedIdeograph39cb': '\xe3\xa7\x8b',
'CjkUnifiedIdeograph39ca': '\xe3\xa7\x8a',
'UnicodeCharacter0xeb6b': '\xee\xad\xab',
'CjkUnifiedIdeograph8987': '\xe8\xa6\x87',
'CjkUnifiedIdeograph8986': '\xe8\xa6\x86',
'CjkUnifiedIdeograph8985': '\xe8\xa6\x85',
'CjkUnifiedIdeograph8984': '\xe8\xa6\x84',
'CjkUnifiedIdeograph8983': '\xe8\xa6\x83',
'CjkUnifiedIdeograph8982': '\xe8\xa6\x82',
'CjkUnifiedIdeograph8981': '\xe8\xa6\x81',
'CjkUnifiedIdeograph8980': '\xe8\xa6\x80',
'OghamLetterUath': '\xe1\x9a\x86',
'MyanmarVowelSignVocalicRr': '\xe1\x81\x97',
'UnicodeCharacter0xda17': '\xed\xa8\x97',
'CjkUnifiedIdeograph8989': '\xe8\xa6\x89',
'CjkUnifiedIdeograph8988': '\xe8\xa6\x88',
'HangulSyllableMwinh': '\xeb\xae\x8e',
'SummationBottom': '\xe2\x8e\xb3',
'UnicodeCharacter0xdf59': '\xed\xbd\x99',
'CanadianSyllabicsCarrierTta': '\xe1\x97\xa7',
'CanadianSyllabicsCarrierTte': '\xe1\x97\xa4',
'CjkUnifiedIdeograph452f': '\xe4\x94\xaf',
'SymbolForSamaritanSource': '\xe2\x85\x8f',
'CanadianSyllabicsCarrierTti': '\xe1\x97\xa6',
'CanadianSyllabicsCarrierTto': '\xe1\x97\xa3',
'CjkUnifiedIdeograph3ddb': '\xe3\xb7\x9b',
'CjkUnifiedIdeograph9b83': '\xe9\xae\x83',
'CjkUnifiedIdeograph9b82': '\xe9\xae\x82',
'CjkUnifiedIdeograph9b81': '\xe9\xae\x81',
'CjkUnifiedIdeograph9b80': '\xe9\xae\x80',
'CjkUnifiedIdeograph9b87': '\xe9\xae\x87',
'CjkUnifiedIdeograph9b86': '\xe9\xae\x86',
'CjkUnifiedIdeograph9b85': '\xe9\xae\x85',
'CanadianSyllabicsCarrierTtu': '\xe1\x97\xa2',
'UnicodeCharacter0xe733': '\xee\x9c\xb3',
'CjkUnifiedIdeograph9b89': '\xe9\xae\x89',
'CjkUnifiedIdeograph9b88': '\xe9\xae\x88',
'UnicodeCharacter0xda15': '\xed\xa8\x95',
'UnicodeCharacter0xf8e2': '\xef\xa3\xa2',
'CjkUnifiedIdeograph898f': '\xe8\xa6\x8f',
'CjkUnifiedIdeograph898e': '\xe8\xa6\x8e',
'CjkUnifiedIdeograph898d': '\xe8\xa6\x8d',
'CjkUnifiedIdeograph898c': '\xe8\xa6\x8c',
'CjkUnifiedIdeograph898b': '\xe8\xa6\x8b',
'GlagoliticCapitalLetterIzhe': '\xe2\xb0\x89',
'KannadaLetterTtha': '\xe0\xb2\xa0',
'HangulSyllableRiss': '\xeb\xa7\x80',
'CjkUnifiedIdeograph9b8c': '\xe9\xae\x8c',
'CjkUnifiedIdeograph9b8b': '\xe9\xae\x8b',
'CjkUnifiedIdeograph9b8a': '\xe9\xae\x8a',
'CyrillicCapitalLetterSchwa': '\xd3\x98',
'CjkUnifiedIdeograph9b8f': '\xe9\xae\x8f',
'CjkUnifiedIdeograph9b8e': '\xe9\xae\x8e',
'CjkUnifiedIdeograph9b8d': '\xe9\xae\x8d',
'CjkUnifiedIdeograph659b': '\xe6\x96\x9b',
'MyanmarSignAsat': '\xe1\x80\xba',
'HangulChoseongPieupchieuch': '\xe1\x84\xa8',
'HangulSyllableYobs': '\xec\x9a\xa6',
'B': '\xce\x92',
'CjkUnifiedIdeograph8307': '\xe8\x8c\x87',
'HangulSyllableJyaess': '\xec\x9f\xb8',
'ModifierLetterSmallGreekGamma': '\xe1\xb5\x9e',
'CjkUnifiedIdeograph8306': '\xe8\x8c\x86',
'EthiopicSyllableXyee': '\xe2\xb7\x94',
'CjkUnifiedIdeograph68d1': '\xe6\xa3\x91',
'CopticSmallLetterTau': '\xe2\xb2\xa7',
'RejangLetterSa': '\xea\xa4\xbc',
'TeluguLetterRa': '\xe0\xb0\xb0',
'HangulSyllableMaelb': '\xeb\xa7\xaf',
'HangulSyllableMwing': '\xeb\xae\x9d',
'CjkUnifiedIdeograph916f': '\xe9\x85\xaf',
'CjkUnifiedIdeograph916d': '\xe9\x85\xad',
'CjkUnifiedIdeograph916e': '\xe9\x85\xae',
'CjkUnifiedIdeograph916b': '\xe9\x85\xab',
'CjkUnifiedIdeograph916c': '\xe9\x85\xac',
'YiSyllableSyrx': '\xea\x8c\xa8',
'CjkUnifiedIdeograph916a': '\xe9\x85\xaa',
'CjkUnifiedIdeograph418c': '\xe4\x86\x8c',
'CjkUnifiedIdeograph418b': '\xe4\x86\x8b',
'CjkUnifiedIdeograph418a': '\xe4\x86\x8a',
'CjkUnifiedIdeograph418f': '\xe4\x86\x8f',
'CjkUnifiedIdeograph418e': '\xe4\x86\x8e',
'NewTaiLueLetterLowFa': '\xe1\xa6\x9d',
'UnicodeCharacter0xda11': '\xed\xa8\x91',
'CjkUnifiedIdeograph6fd1': '\xe6\xbf\x91',
'HangulSyllableSsyaegg': '\xec\x8d\x8e',
'CjkUnifiedIdeograph8309': '\xe8\x8c\x89',
'HangulSyllableNoebs': '\xeb\x87\x9e',
'UnicodeCharacter0xf57c': '\xef\x95\xbc',
'ArabicSmallHighLamAlef': '\xdb\x99',
'HangulSyllableSsyaegs': '\xec\x8d\x8f',
'UnicodeCharacter0xecd3': '\xee\xb3\x93',
'ArabicLetterDadInitialForm': '\xef\xba\xbf',
'HangulSyllablePoelg': '\xed\x91\x89',
'UnicodeCharacter0xf7ed': '\xef\x9f\xad',
'CjkUnifiedIdeograph9166': '\xe9\x85\xa6',
'CjkUnifiedIdeograph9167': '\xe9\x85\xa7',
'CjkUnifiedIdeograph4189': '\xe4\x86\x89',
'CjkUnifiedIdeograph4188': '\xe4\x86\x88',
'CjkUnifiedIdeograph9162': '\xe9\x85\xa2',
'CjkUnifiedIdeograph9163': '\xe9\x85\xa3',
'CjkUnifiedIdeograph9160': '\xe9\x85\xa0',
'CjkUnifiedIdeograph9161': '\xe9\x85\xa1',
'CjkUnifiedIdeograph4183': '\xe4\x86\x83',
'CjkUnifiedIdeograph4182': '\xe4\x86\x82',
'CjkUnifiedIdeograph4181': '\xe4\x86\x81',
'CjkUnifiedIdeograph4180': '\xe4\x86\x80',
'CjkUnifiedIdeograph4187': '\xe4\x86\x87',
'CjkUnifiedIdeograph4186': '\xe4\x86\x86',
'CjkUnifiedIdeograph4185': '\xe4\x86\x85',
'CjkUnifiedIdeograph4184': '\xe4\x86\x84',
'LatinSmallLetterTcDigraphWithCurl': '\xca\xa8',
'UnicodeCharacter0xf8de': '\xef\xa3\x9e',
'CombiningOgonekAbove': '\xe1\xb7\x8e',
'HangulSyllableMyu': '\xeb\xae\xa4',
'CjkUnifiedIdeograph55bb': '\xe5\x96\xbb',
'CjkUnifiedIdeograph55bc': '\xe5\x96\xbc',
'CjkUnifiedIdeograph55ba': '\xe5\x96\xba',
'MinusSignWithCommaAbove': '\xe2\xa8\xa9',
'HangulSyllableKwanj': '\xec\xbd\xb5',
'CjkUnifiedIdeograph55bd': '\xe5\x96\xbd',
'CjkUnifiedIdeograph55be': '\xe5\x96\xbe',
'CjkUnifiedIdeograph719e': '\xe7\x86\x9e',
'RunicLetterDottedn': '\xe1\x9b\x80',
'UnicodeCharacter0xf57a': '\xef\x95\xba',
'RunicLetterDottedl': '\xe1\x9b\x9b',
'CjkUnifiedIdeograph719a': '\xe7\x86\x9a',
'UnicodeCharacter0xd7af': '\xed\x9e\xaf',
'CjkUnifiedIdeograph719c': '\xe7\x86\x9c',
'CjkUnifiedIdeograph719b': '\xe7\x86\x9b',
'YiSyllableJiex': '\xea\x8f\xa5',
'UnicodeCharacter0xec7c': '\xee\xb1\xbc',
'UnicodeCharacter0xec7b': '\xee\xb1\xbb',
'UnicodeCharacter0xec7e': '\xee\xb1\xbe',
'UnicodeCharacter0xec7d': '\xee\xb1\xbd',
'RunicLetterDottedp': '\xe1\x9b\x94',
'CjkUnifiedIdeograph386e': '\xe3\xa1\xae',
'CjkUnifiedIdeograph386d': '\xe3\xa1\xad',
'CjkUnifiedIdeograph386f': '\xe3\xa1\xaf',
'CjkUnifiedIdeograph386a': '\xe3\xa1\xaa',
'UnicodeCharacter0xd938': '\xed\xa4\xb8',
'CjkUnifiedIdeograph386c': '\xe3\xa1\xac',
'CjkUnifiedIdeograph386b': '\xe3\xa1\xab',
'CjkUnifiedIdeograph9290': '\xe9\x8a\x90',
'UnicodeCharacter0x167a': '\xe1\x99\xba',
'UnicodeCharacter0x167b': '\xe1\x99\xbb',
'UnicodeCharacter0x167c': '\xe1\x99\xbc',
'KannadaDigitTwo': '\xe0\xb3\xa8',
'UnicodeCharacter0x167e': '\xe1\x99\xbe',
'UnicodeCharacter0x167f': '\xe1\x99\xbf',
'CjkUnifiedIdeograph9297': '\xe9\x8a\x97',
'CjkUnifiedIdeograph9298': '\xe9\x8a\x98',
'CjkUnifiedIdeograph9299': '\xe9\x8a\x99',
'UnicodeCharacter0xd7ac': '\xed\x9e\xac',
'UnicodeCharacter0xe588': '\xee\x96\x88',
'UnicodeCharacter0xe589': '\xee\x96\x89',
'CjkUnifiedIdeograph4455': '\xe4\x91\x95',
'TeluguLetterTtha': '\xe0\xb0\xa0',
'CjkUnifiedIdeograph4457': '\xe4\x91\x97',
'LatinCapitalLetterAWithOgonek': '\xc4\x84',
'CjkUnifiedIdeograph4451': '\xe4\x91\x91',
'CjkUnifiedIdeograph4450': '\xe4\x91\x90',
'CjkUnifiedIdeograph4453': '\xe4\x91\x93',
'CjkUnifiedIdeograph4452': '\xe4\x91\x92',
'CjkUnifiedIdeograph4459': '\xe4\x91\x99',
'CjkUnifiedIdeograph4458': '\xe4\x91\x98',
'UnicodeCharacter0xec71': '\xee\xb1\xb1',
'UnicodeCharacter0xec70': '\xee\xb1\xb0',
'UnicodeCharacter0xec73': '\xee\xb1\xb3',
'UnicodeCharacter0xec72': '\xee\xb1\xb2',
'CjkUnifiedIdeograph3869': '\xe3\xa1\xa9',
'CjkUnifiedIdeograph3868': '\xe3\xa1\xa8',
'UnicodeCharacter0xec77': '\xee\xb1\xb7',
'UnicodeCharacter0xec76': '\xee\xb1\xb6',
'CjkUnifiedIdeograph3865': '\xe3\xa1\xa5',
'CjkUnifiedIdeograph3864': '\xe3\xa1\xa4',
'EthiopicSyllableBu': '\xe1\x89\xa1',
'LimbuSmallLetterNga': '\xe1\xa4\xb1',
'CjkUnifiedIdeograph3861': '\xe3\xa1\xa1',
'CjkUnifiedIdeograph3860': '\xe3\xa1\xa0',
'CjkUnifiedIdeograph3863': '\xe3\xa1\xa3',
'CjkUnifiedIdeograph3862': '\xe3\xa1\xa2',
'EthiopicSyllableBo': '\xe1\x89\xa6',
'CjkUnifiedIdeograph55b3': '\xe5\x96\xb3',
'CjkUnifiedIdeograph55b0': '\xe5\x96\xb0',
'CjkUnifiedIdeograph55b1': '\xe5\x96\xb1',
'CjkUnifiedIdeograph55b6': '\xe5\x96\xb6',
'CjkUnifiedIdeograph55b7': '\xe5\x96\xb7',
'EthiopicSyllableBi': '\xe1\x89\xa2',
'CjkUnifiedIdeograph55b5': '\xe5\x96\xb5',
'CjkUnifiedIdeograph7195': '\xe7\x86\x95',
'CjkUnifiedIdeograph7194': '\xe7\x86\x94',
'EthiopicSyllableBe': '\xe1\x89\xa5',
'CjkUnifiedIdeograph55b9': '\xe5\x96\xb9',
'CjkUnifiedIdeograph7191': '\xe7\x86\x91',
'CjkUnifiedIdeograph7190': '\xe7\x86\x90',
'EthiopicSyllableBa': '\xe1\x89\xa0',
'CjkUnifiedIdeograph7192': '\xe7\x86\x92',
'CjkUnifiedIdeograph445e': '\xe4\x91\x9e',
'CjkUnifiedIdeograph445d': '\xe4\x91\x9d',
'CjkUnifiedIdeograph445f': '\xe4\x91\x9f',
'CjkUnifiedIdeograph445a': '\xe4\x91\x9a',
'CjkUnifiedIdeograph445c': '\xe4\x91\x9c',
'CjkUnifiedIdeograph445b': '\xe4\x91\x9b',
'HangulSyllableMeunj': '\xeb\xaf\x85',
'HangulSyllableMeunh': '\xeb\xaf\x86',
'UnicodeCharacter0xe58f': '\xee\x96\x8f',
'CjkUnifiedIdeograph929a': '\xe9\x8a\x9a',
'CjkUnifiedIdeograph929b': '\xe9\x8a\x9b',
'CjkUnifiedIdeograph929c': '\xe9\x8a\x9c',
'CjkUnifiedIdeograph929d': '\xe9\x8a\x9d',
'CjkUnifiedIdeograph929e': '\xe9\x8a\x9e',
'CjkUnifiedIdeograph929f': '\xe9\x8a\x9f',
'UnicodeCharacter0x1677': '\xe1\x99\xb7',
'UnicodeCharacter0x1678': '\xe1\x99\xb8',
'UnicodeCharacter0x1679': '\xe1\x99\xb9',
'UnicodeCharacter0xd8cf': '\xed\xa3\x8f',
'UnicodeCharacter0xee9e': '\xee\xba\x9e',
'UnicodeCharacter0xee9d': '\xee\xba\x9d',
'UnicodeCharacter0xee9f': '\xee\xba\x9f',
'UnicodeCharacter0xee9a': '\xee\xba\x9a',
'CanadianSyllabicsSouthslaveyKeh': '\xe1\x92\x85',
'UnicodeCharacter0xee9c': '\xee\xba\x9c',
'UnicodeCharacter0xee9b': '\xee\xba\x9b',
'PlusSignWithSubscriptTwo': '\xe2\xa8\xa7',
'UnicodeCharacter0xef18': '\xee\xbc\x98',
'UnicodeCharacter0xee83': '\xee\xba\x83',
'CjkUnifiedIdeograph5324': '\xe5\x8c\xa4',
'CjkUnifiedIdeograph5325': '\xe5\x8c\xa5',
'CjkUnifiedIdeograph5326': '\xe5\x8c\xa6',
'CjkUnifiedIdeograph5327': '\xe5\x8c\xa7',
'CjkUnifiedIdeograph5320': '\xe5\x8c\xa0',
'CjkUnifiedIdeograph5321': '\xe5\x8c\xa1',
'CjkUnifiedIdeograph5322': '\xe5\x8c\xa2',
'CjkUnifiedIdeograph5323': '\xe5\x8c\xa3',
'HangulSyllablePoelh': '\xed\x91\x8f',
'CjkUnifiedIdeograph6128': '\xe6\x84\xa8',
'CjkUnifiedIdeograph5328': '\xe5\x8c\xa8',
'CjkUnifiedIdeograph5329': '\xe5\x8c\xa9',
'HangulSyllablePoelm': '\xed\x91\x8a',
'CjkUnifiedIdeograph9834': '\xe9\xa0\xb4',
'CjkUnifiedIdeograph9359': '\xe9\x8d\x99',
'CjkUnifiedIdeograph9836': '\xe9\xa0\xb6',
'CjkUnifiedIdeograph9837': '\xe9\xa0\xb7',
'CjkUnifiedIdeograph9830': '\xe9\xa0\xb0',
'CjkUnifiedIdeograph9831': '\xe9\xa0\xb1',
'CjkUnifiedIdeograph9832': '\xe9\xa0\xb2',
'CjkUnifiedIdeograph9358': '\xe9\x8d\x98',
'CjkUnifiedIdeograph9838': '\xe9\xa0\xb8',
'CjkUnifiedIdeograph9839': '\xe9\xa0\xb9',
'CjkCompatibilityIdeographf9d7': '\xef\xa7\x97',
'TimesWithLeftHalfBlack': '\xe2\xa7\x94',
'GurmukhiVowelSignAa': '\xe0\xa8\xbe',
'GurmukhiVowelSignAi': '\xe0\xa9\x88',
'UnicodeCharacter0xee95': '\xee\xba\x95',
'UnicodeCharacter0xee94': '\xee\xba\x94',
'UnicodeCharacter0xee97': '\xee\xba\x97',
'GurmukhiVowelSignAu': '\xe0\xa9\x8c',
'UnicodeCharacter0xee91': '\xee\xba\x91',
'UnicodeCharacter0xee90': '\xee\xba\x90',
'UnicodeCharacter0xee93': '\xee\xba\x93',
'UnicodeCharacter0xee92': '\xee\xba\x92',
'MeasuredAngleWithOpenArmEndingInArrowPointingDownAndRight': '\xe2\xa6\xaa',
'UnicodeCharacter0xf187': '\xef\x86\x87',
'HangulSyllableDweols': '\xeb\x91\xb8',
'UnicodeCharacter0xee99': '\xee\xba\x99',
'UnicodeCharacter0xee98': '\xee\xba\x98',
'CjkUnifiedIdeograph983d': '\xe9\xa0\xbd',
'CjkUnifiedIdeograph983e': '\xe9\xa0\xbe',
'CjkUnifiedIdeograph983f': '\xe9\xa0\xbf',
'CjkUnifiedIdeograph983a': '\xe9\xa0\xba',
'CjkUnifiedIdeograph983b': '\xe9\xa0\xbb',
'CjkUnifiedIdeograph983c': '\xe9\xa0\xbc',
'LatinSmallLetterUWithOgonek': '\xc5\xb3',
'ThaiCharacterMaiTho': '\xe0\xb9\x89',
'CjkUnifiedIdeograph532d': '\xe5\x8c\xad',
'CjkUnifiedIdeograph532e': '\xe5\x8c\xae',
'CjkUnifiedIdeograph532f': '\xe5\x8c\xaf',
'HangulSyllableToenh': '\xed\x87\xba',
'CjkUnifiedIdeograph532a': '\xe5\x8c\xaa',
'CjkUnifiedIdeograph532b': '\xe5\x8c\xab',
'CjkUnifiedIdeograph532c': '\xe5\x8c\xac',
'HangulSyllableGgubs': '\xea\xbf\x8a',
'HangulSyllableToeng': '\xed\x88\x89',
'UnicodeCharacter0xf189': '\xef\x86\x89',
'CjkUnifiedIdeograph6743': '\xe6\x9d\x83',
'CjkUnifiedIdeograph6742': '\xe6\x9d\x82',
'CjkUnifiedIdeograph6741': '\xe6\x9d\x81',
'CjkUnifiedIdeograph6740': '\xe6\x9d\x80',
'CjkUnifiedIdeograph6747': '\xe6\x9d\x87',
'CjkUnifiedIdeograph6746': '\xe6\x9d\x86',
'CjkUnifiedIdeograph6745': '\xe6\x9d\x85',
'CjkUnifiedIdeograph6744': '\xe6\x9d\x84',
'CjkUnifiedIdeograph3d5d': '\xe3\xb5\x9d',
'CjkUnifiedIdeograph3d5e': '\xe3\xb5\x9e',
'IntegralExtension': '\xe2\x8e\xae',
'CjkUnifiedIdeograph6748': '\xe6\x9d\x88',
'CjkUnifiedIdeograph3d5a': '\xe3\xb5\x9a',
'CjkUnifiedIdeograph3d5b': '\xe3\xb5\x9b',
'CjkUnifiedIdeograph3d5c': '\xe3\xb5\x9c',
'CjkUnifiedIdeograph661f': '\xe6\x98\x9f',
'HangulJongseongNieunthieuth': '\xe1\x87\x89',
'UnicodeCharacter0xaaa5': '\xea\xaa\xa5',
'.+': '\xe2\x88\x94',
'.-': '\xe2\x88\xb8',
'UnicodeCharacter0xaaa4': '\xea\xaa\xa4',
'UnicodeCharacter0xaaa7': '\xea\xaa\xa7',
'HangulChoseongPieupnieun': '\xe1\x84\x9f',
'UnicodeCharacter0xeb65': '\xee\xad\xa5',
'UnicodeCharacter0xaaa6': '\xea\xaa\xa6',
'UnicodeCharacter0xee80': '\xee\xba\x80',
'EthiopicSyllableHaa': '\xe1\x88\x83',
'UnicodeCharacter0xaaa1': '\xea\xaa\xa1',
'LaoToneMaiEk': '\xe0\xbb\x88',
'UnicodeCharacter0xaaa0': '\xea\xaa\xa0',
'UnicodeCharacter0xd8c9': '\xed\xa3\x89',
'UnicodeCharacter0xecef': '\xee\xb3\xaf',
'UnicodeCharacter0xaaa3': '\xea\xaa\xa3',
'UnicodeCharacter0xd8c8': '\xed\xa3\x88',
'UnicodeCharacter0xaaa2': '\xea\xaa\xa2',
'CjkUnifiedIdeograph674c': '\xe6\x9d\x8c',
'CjkUnifiedIdeograph674b': '\xe6\x9d\x8b',
'CjkUnifiedIdeograph674a': '\xe6\x9d\x8a',
'CjkUnifiedIdeograph3d58': '\xe3\xb5\x98',
'CjkUnifiedIdeograph3d59': '\xe3\xb5\x99',
'CjkUnifiedIdeograph674e': '\xe6\x9d\x8e',
'CjkUnifiedIdeograph674d': '\xe6\x9d\x8d',
'CjkUnifiedIdeograph3d54': '\xe3\xb5\x94',
'CjkUnifiedIdeograph3d55': '\xe3\xb5\x95',
'CjkUnifiedIdeograph3d56': '\xe3\xb5\x96',
'CjkUnifiedIdeograph3d57': '\xe3\xb5\x97',
'CjkUnifiedIdeograph3d50': '\xe3\xb5\x90',
'CjkUnifiedIdeograph3d51': '\xe3\xb5\x91',
'CjkUnifiedIdeograph3d52': '\xe3\xb5\x92',
'CjkUnifiedIdeograph3d53': '\xe3\xb5\x93',
'SaurashtraVowelSignVocalicRr': '\xea\xa2\xbb',
'VolumeIntegral': '\xe2\x88\xb0',
'SquareHon': '\xe3\x8c\xbf',
'HebrewAccentSegol': '\xd6\x92',
'HangulSyllableMaelg': '\xeb\xa7\xad',
'ArabicLetterHehGoalIsolatedForm': '\xef\xae\xa6',
'VaiDigitZero': '\xea\x98\xa0',
'UnicodeCharacter0xd8c1': '\xed\xa3\x81',
'HangulSyllableGyuss': '\xea\xb7\xb0',
'MalayalamLetterTa': '\xe0\xb4\xa4',
'CjkCompatibilityIdeographf9be': '\xef\xa6\xbe',
'CjkUnifiedIdeograph99e7': '\xe9\xa7\xa7',
'CjkUnifiedIdeograph99e6': '\xe9\xa7\xa6',
'CjkUnifiedIdeograph99e5': '\xe9\xa7\xa5',
'CjkUnifiedIdeograph99e4': '\xe9\xa7\xa4',
'CjkUnifiedIdeograph99e3': '\xe9\xa7\xa3',
'CjkUnifiedIdeograph99e2': '\xe9\xa7\xa2',
'CjkUnifiedIdeograph99e1': '\xe9\xa7\xa1',
'CjkUnifiedIdeograph99e0': '\xe9\xa7\xa0',
'HangulSyllableGgyegs': '\xea\xbc\x93',
'CjkUnifiedIdeograph99e9': '\xe9\xa7\xa9',
'CjkRadicalKnifeOne': '\xe2\xba\x88',
'HangulSyllableJwanj': '\xec\xa2\x91',
'HangulJongseongRieulyeorinhieuh': '\xe1\x87\x99',
'HangulSyllableGgyegg': '\xea\xbc\x92',
'UnicodeCharacter0xd87d': '\xed\xa1\xbd',
'iiint': '\xe2\x88\xad',
'UnicodeCharacter0x269f': '\xe2\x9a\x9f',
'UnicodeCharacter0x269e': '\xe2\x9a\x9e',
'EthiopicSyllableKaa': '\xe1\x8a\xab',
'CjkUnifiedIdeograph3a8f': '\xe3\xaa\x8f',
'CjkUnifiedIdeograph3a8d': '\xe3\xaa\x8d',
'CjkUnifiedIdeograph3a8e': '\xe3\xaa\x8e',
'CjkUnifiedIdeograph3a8b': '\xe3\xaa\x8b',
'CjkUnifiedIdeograph3a8c': '\xe3\xaa\x8c',
'CjkUnifiedIdeograph3a8a': '\xe3\xaa\x8a',
'LatinCapitalLetterInsularF': '\xea\x9d\xbb',
'LatinCapitalLetterInsularG': '\xea\x9d\xbd',
'LaoDigitSix': '\xe0\xbb\x96',
'LatinCapitalLetterInsularD': '\xea\x9d\xb9',
'CjkUnifiedIdeograph7cf1': '\xe7\xb3\xb1',
'BalineseLetterGaGora': '\xe1\xac\x96',
'CjkUnifiedIdeograph99ef': '\xe9\xa7\xaf',
'CjkUnifiedIdeograph99ee': '\xe9\xa7\xae',
'CjkUnifiedIdeograph99ed': '\xe9\xa7\xad',
'CjkUnifiedIdeograph99ec': '\xe9\xa7\xac',
'CjkUnifiedIdeograph99eb': '\xe9\xa7\xab',
'CjkUnifiedIdeograph99ea': '\xe9\xa7\xaa',
'HangulSyllableSsweng': '\xec\x92\xb1',
'UnicodeCharacter0xd87f': '\xed\xa1\xbf',
'ArabicLetterNyeh': '\xda\x83',
'CjkUnifiedIdeograph3a87': '\xe3\xaa\x87',
'CjkUnifiedIdeograph3a84': '\xe3\xaa\x84',
'CjkUnifiedIdeograph3a85': '\xe3\xaa\x85',
'CjkUnifiedIdeograph3a82': '\xe3\xaa\x82',
'CjkUnifiedIdeograph3a83': '\xe3\xaa\x83',
'DoubledMaleSign': '\xe2\x9a\xa3',
'CjkUnifiedIdeograph3a81': '\xe3\xaa\x81',
'CjkUnifiedIdeograph3a88': '\xe3\xaa\x88',
'CjkUnifiedIdeograph3a89': '\xe3\xaa\x89',
'HangulSyllableBbying': '\xec\x82\x89',
'CjkUnifiedIdeograph7cf8': '\xe7\xb3\xb8',
'HangulSyllableSswenj': '\xec\x92\xa1',
'CjkUnifiedIdeograph8918': '\xe8\xa4\x98',
'HangulSyllableBbyinj': '\xec\x81\xb9',
'HangulSyllableBbyinh': '\xec\x81\xba',
'HangulSyllableSing': '\xec\x8b\xb1',
'UnicodeCharacter0xdcac': '\xed\xb2\xac',
'YiSyllableZhuox': '\xea\x8d\x8a',
'UnicodeCharacter0xdcad': '\xed\xb2\xad',
'HangulSyllableSinj': '\xec\x8b\xa1',
'HangulSyllableSinh': '\xec\x8b\xa2',
'VerticalLineExtension': '\xe2\x8f\x90',
'KhmerVowelSignUa': '\xe1\x9e\xbd',
'UnicodeCharacter0x52f': '\xd4\xaf',
'EthiopicSyllableMwa': '\xe1\x88\x9f',
'UnicodeCharacter0xde7e': '\xed\xb9\xbe',
'CjkUnifiedIdeograph9caf': '\xe9\xb2\xaf',
'EthiopicSyllableMwi': '\xe1\x8e\x81',
'HangulSyllableCwang': '\xec\xb4\xb9',
'CjkUnifiedIdeograph8910': '\xe8\xa4\x90',
'HangulSyllableYaebs': '\xec\x96\xaa',
'HangulSyllableCwanj': '\xec\xb4\xa9',
'HangulSyllableCwanh': '\xec\xb4\xaa',
'CjkUnifiedIdeograph4b13': '\xe4\xac\x93',
'CjkUnifiedIdeograph8dd5': '\xe8\xb7\x95',
'CjkUnifiedIdeograph7f09': '\xe7\xbc\x89',
'CjkUnifiedIdeograph7f08': '\xe7\xbc\x88',
'CjkUnifiedIdeograph7f05': '\xe7\xbc\x85',
'CjkUnifiedIdeograph7f04': '\xe7\xbc\x84',
'CjkUnifiedIdeograph7f07': '\xe7\xbc\x87',
'CjkUnifiedIdeograph7f06': '\xe7\xbc\x86',
'CjkUnifiedIdeograph7f01': '\xe7\xbc\x81',
'CjkUnifiedIdeograph7f00': '\xe7\xbc\x80',
'CjkUnifiedIdeograph7f03': '\xe7\xbc\x83',
'CjkUnifiedIdeograph7f02': '\xe7\xbc\x82',
'UnicodeCharacter0xaaae': '\xea\xaa\xae',
'OriyaLetterI': '\xe0\xac\x87',
'OriyaLetterO': '\xe0\xac\x93',
'UnicodeCharacter0xaaad': '\xea\xaa\xad',
'OriyaLetterA': '\xe0\xac\x85',
'HangulSyllableKong': '\xec\xbd\xa9',
'OriyaLetterE': '\xe0\xac\x8f',
'HeavyCheckMark': '\xe2\x9c\x94',
'YiSyllableKex': '\xea\x88\x8b',
'CanadianSyllabicsNaskapiSwaa': '\xe1\x94\x84',
'YiSyllableSsex': '\xea\x8c\xb9',
'UnicodeCharacter0xd87b': '\xed\xa1\xbb',
'YiSyllableKep': '\xea\x88\x8d',
'YiSyllableSsep': '\xea\x8c\xbb',
'HangulChoseongSioscieuc': '\xe1\x84\xb6',
'UnicodeCharacter0xaaac': '\xea\xaa\xac',
'CjkUnifiedIdeograph7f0e': '\xe7\xbc\x8e',
'CjkUnifiedIdeograph7f0d': '\xe7\xbc\x8d',
'DoubleQuestionMark': '\xe2\x81\x87',
'CjkUnifiedIdeograph7f0f': '\xe7\xbc\x8f',
'CjkUnifiedIdeograph7f0a': '\xe7\xbc\x8a',
'CjkUnifiedIdeograph7f0c': '\xe7\xbc\x8c',
'CjkUnifiedIdeograph7f0b': '\xe7\xbc\x8b',
'UpArrowheadBetweenTwoHorizontalBars': '\xe2\x8c\xa4',
'ArabicLetterSheenMedialForm': '\xef\xba\xb8',
'ArabicLigatureSheenWithJeemFinalForm': '\xef\xb4\xa5',
'HangulSyllableJyaels': '\xec\x9f\xb0',
'HangulSyllableJyaelp': '\xec\x9f\xb2',
'HangulSyllableJyaelt': '\xec\x9f\xb1',
'GreekCapitalLetterOmicronWithPsiliAndOxia': '\xe1\xbd\x8c',
'KhmerSignLekToo': '\xe1\x9f\x97',
'CjkUnifiedIdeograph5f10': '\xe5\xbc\x90',
'CjkUnifiedIdeograph5f11': '\xe5\xbc\x91',
'CjkUnifiedIdeograph5f12': '\xe5\xbc\x92',
'CjkUnifiedIdeograph5f13': '\xe5\xbc\x93',
'CjkUnifiedIdeograph5f14': '\xe5\xbc\x94',
'CjkUnifiedIdeograph5f15': '\xe5\xbc\x95',
'CjkUnifiedIdeograph5f16': '\xe5\xbc\x96',
'CjkUnifiedIdeograph5f17': '\xe5\xbc\x97',
'CjkUnifiedIdeograph5f18': '\xe5\xbc\x98',
'CjkUnifiedIdeograph5f19': '\xe5\xbc\x99',
'HangulSyllableJyaelh': '\xec\x9f\xb3',
'NorthEastArrowWithHook': '\xe2\xa4\xa4',
'GreekLetterSmallCapitalRho': '\xe1\xb4\xa9',
'HangulSyllableJyaelm': '\xec\x9f\xae',
'HangulSyllableMeul': '\xeb\xaf\x88',
'HangulSyllableMeum': '\xeb\xaf\x90',
'HangulSyllableMeun': '\xeb\xaf\x84',
'HangulSyllableMeuh': '\xeb\xaf\x9b',
'HangulSyllableMeuj': '\xeb\xaf\x96',
'HangulSyllableMeuk': '\xeb\xaf\x98',
'HangulSyllableMeud': '\xeb\xaf\x87',
'GeorgianCapitalLetterXan': '\xe1\x82\xbe',
'CjkUnifiedIdeograph5b49': '\xe5\xad\x89',
'HangulSyllableMeub': '\xeb\xaf\x91',
'TildeOperatorWithRisingDots': '\xe2\xa9\xab',
'UnicodeCharacter0x9fe6': '\xe9\xbf\xa6',
'HangulSyllableMeut': '\xeb\xaf\x99',
'HangulSyllableGweonh': '\xea\xb6\x8e',
'HangulSyllableMeup': '\xeb\xaf\x9a',
'HangulSyllableMeus': '\xeb\xaf\x93',
'CjkUnifiedIdeograph5f1a': '\xe5\xbc\x9a',
'CjkUnifiedIdeograph5f1b': '\xe5\xbc\x9b',
'CjkUnifiedIdeograph5f1c': '\xe5\xbc\x9c',
'CjkUnifiedIdeograph5f1d': '\xe5\xbc\x9d',
'CjkUnifiedIdeograph5f1e': '\xe5\xbc\x9e',
'CjkUnifiedIdeograph5f1f': '\xe5\xbc\x9f',
'HangulChoseongCeongchieumcieuc': '\xe1\x85\x90',
'HangulSyllableBbelt': '\xeb\xbb\xb9',
'HangulSyllableBbels': '\xeb\xbb\xb8',
'CombiningRightArrowheadAndUpArrowheadBelow': '\xcd\x96',
'HangulSyllableBbelp': '\xeb\xbb\xba',
'CjkUnifiedIdeograph5b43': '\xe5\xad\x83',
'HangulSyllableBbelm': '\xeb\xbb\xb6',
'YiSyllableHmo': '\xea\x82\xa1',
'HangulSyllableBbelg': '\xeb\xbb\xb5',
'YiSyllableHma': '\xea\x82\x9a',
'HangulSyllableBbelb': '\xeb\xbb\xb7',
'HangulSyllableHyolg': '\xed\x9a\xb1',
'MinusorplusSign': '\xe2\x88\x93',
'ElementOfOpeningUpwards': '\xe2\x9f\x92',
'GreekCapitalLetterEpsilonWithTonos': '\xce\x88',
'CjkUnifiedIdeograph6615': '\xe6\x98\x95',
'HangulSyllableTyaet': '\xed\x84\xad',
'HangulSyllableSsyoss': '\xec\x91\x9c',
'UnicodeCharacter0xdb6a': '\xed\xad\xaa',
'BoxDrawingsLeftDownHeavyAndRightUpLight': '\xe2\x95\x85',
'CjkUnifiedIdeograph5b44': '\xe5\xad\x84',
'CjkUnifiedIdeograph9e59': '\xe9\xb9\x99',
'ArabicLetterKehehFinalForm': '\xef\xae\x8f',
'TamilLetterNya': '\xe0\xae\x9e',
'LeftwardsHarpoonWithBarbUpFromBar': '\xe2\xa5\x9a',
'HangulSyllableHyolm': '\xed\x9a\xb2',
'HangulSyllableGwang': '\xea\xb4\x91',
'HangulSyllableIgg': '\xec\x9d\xb6',
'HangulSyllableGwanj': '\xea\xb4\x81',
'HangulSyllableGwanh': '\xea\xb4\x82',
'UnicodeCharacter0x193f': '\xe1\xa4\xbf',
'HangulLetterOe': '\xe3\x85\x9a',
'HangulSyllableIgs': '\xec\x9d\xb7',
'ArabicLetterHehGoalFinalForm': '\xef\xae\xa7',
'HangulSyllableIlb': '\xec\x9d\xbf',
'HangulSyllableBbyilt': '\xec\x82\x81',
'UnicodeCharacter0x193e': '\xe1\xa4\xbe',
'CjkUnifiedIdeograph9ecf': '\xe9\xbb\x8f',
'NkoCombiningShortLowTone': '\xdf\xac',
'ArabicLetterSheenFinalForm': '\xef\xba\xb6',
'TibetanKuRuKha': '\xe0\xbe\xbe',
'UnicodeCharacter0xe738': '\xee\x9c\xb8',
'SylotiNagriLetterBho': '\xea\xa0\x9c',
'LatinCapitalLetterTWithDiagonalStroke': '\xc8\xbe',
'CjkUnifiedIdeograph3e83': '\xe3\xba\x83',
'UnicodeCharacter0x193d': '\xe1\xa4\xbd',
'HangulSyllableGoebs': '\xea\xb5\x86',
'CjkUnifiedIdeograph3e81': '\xe3\xba\x81',
'CjkUnifiedIdeograph8219': '\xe8\x88\x99',
'CjkUnifiedIdeograph8218': '\xe8\x88\x98',
'CjkUnifiedIdeograph8217': '\xe8\x88\x97',
'CjkUnifiedIdeograph8216': '\xe8\x88\x96',
'CjkUnifiedIdeograph8215': '\xe8\x88\x95',
'CjkUnifiedIdeograph8214': '\xe8\x88\x94',
'CjkUnifiedIdeograph8213': '\xe8\x88\x93',
'CjkUnifiedIdeograph8212': '\xe8\x88\x92',
'CjkUnifiedIdeograph8211': '\xe8\x88\x91',
'CjkUnifiedIdeograph8210': '\xe8\x88\x90',
'UnicodeCharacter0xf3d5': '\xef\x8f\x95',
'UnicodeCharacter0xf3d4': '\xef\x8f\x94',
'UnicodeCharacter0xf3d7': '\xef\x8f\x97',
'UnicodeCharacter0xf3d6': '\xef\x8f\x96',
'UnicodeCharacter0xf3d1': '\xef\x8f\x91',
'UnicodeCharacter0xf3d0': '\xef\x8f\x90',
'UnicodeCharacter0xf3d3': '\xef\x8f\x93',
'UnicodeCharacter0xf3d2': '\xef\x8f\x92',
'UnicodeCharacter0xf3d9': '\xef\x8f\x99',
'UnicodeCharacter0xf3d8': '\xef\x8f\x98',
'SquareWithContouredOutline': '\xe2\xa7\xa0',
'HangulSyllableSul': '\xec\x88\xa0',
'HangulSyllableSum': '\xec\x88\xa8',
'HangulSyllableSun': '\xec\x88\x9c',
'LeftVerticalBarWithQuill': '\xe2\xb8\xa0',
'HangulSyllableSuh': '\xec\x88\xb3',
'HangulSyllableHyunj': '\xed\x9c\xb9',
'HangulSyllableSuj': '\xec\x88\xae',
'HangulSyllableSuk': '\xec\x88\xb0',
'HangulSyllableSud': '\xec\x88\x9f',
'HangulSyllableSug': '\xec\x88\x99',
'HangulSyllableKeon': '\xec\xbb\xa8',
'HangulSyllableSub': '\xec\x88\xa9',
'HangulSyllableSuc': '\xec\x88\xaf',
'BengaliLetterPa': '\xe0\xa6\xaa',
'TibetanSubjoinedLetterNga': '\xe0\xbe\x94',
'UnicodeCharacter0xea5f': '\xee\xa9\x9f',
'HangulJongseongKiyeoksioskiyeok': '\xe1\x87\x84',
'HangulSyllableSut': '\xec\x88\xb1',
'HangulSyllableSup': '\xec\x88\xb2',
'HangulSyllableSus': '\xec\x88\xab',
'UnicodeCharacter0xf3de': '\xef\x8f\x9e',
'UnicodeCharacter0xf3dd': '\xef\x8f\x9d',
'UnicodeCharacter0xf3df': '\xef\x8f\x9f',
'UnicodeCharacter0xf3da': '\xef\x8f\x9a',
'HangulSyllableJjwaebs': '\xec\xac\x86',
'UnicodeCharacter0xf3dc': '\xef\x8f\x9c',
'UnicodeCharacter0xf3db': '\xef\x8f\x9b',
'UnicodeCharacter0xfdea': '\xef\xb7\xaa',
'SquareRadOverSSquared': '\xe3\x8e\xaf',
'CjkUnifiedIdeograph821f': '\xe8\x88\x9f',
'CjkUnifiedIdeograph821e': '\xe8\x88\x9e',
'CjkUnifiedIdeograph821d': '\xe8\x88\x9d',
'CjkUnifiedIdeograph821c': '\xe8\x88\x9c',
'CjkUnifiedIdeograph821b': '\xe8\x88\x9b',
'CjkUnifiedIdeograph821a': '\xe8\x88\x9a',
'HangulSyllableHwaelg': '\xed\x99\xb9',
'HangulSyllableHwaelb': '\xed\x99\xbb',
'NormalSubgroupOf': '\xe2\x8a\xb2',
'UnicodeCharacter0xdca4': '\xed\xb2\xa4',
'HangulSyllableHwaelm': '\xed\x99\xba',
'HangulSyllableSwaelm': '\xec\x87\x8e',
'HangulSyllableHwaelh': '\xed\x99\xbf',
'EqualsSignAbovePlusSign': '\xe2\xa9\xb1',
'HangulSyllableHwaelt': '\xed\x99\xbd',
'HangulSyllableHwaels': '\xed\x99\xbc',
'HangulSyllableHwaelp': '\xed\x99\xbe',
'UnicodeCharacter0xdca7': '\xed\xb2\xa7',
'HangulSyllableSwaelh': '\xec\x87\x93',
'DoubleSolidusOperator': '\xe2\xab\xbd',
'YiSyllableHlyrx': '\xea\x86\xb5',
'BoxDrawingsHeavyVerticalAndHorizontal': '\xe2\x95\x8b',
'IncompleteInfinity': '\xe2\xa7\x9c',
'EthiopicSyllableQwi': '\xe1\x89\x8a',
'HangulSyllableJjwiss': '\xec\xae\xb0',
'EthiopicSyllableQwa': '\xe1\x89\x88',
'CjkUnifiedIdeograph5d58': '\xe5\xb5\x98',
'EthiopicSyllableQwe': '\xe1\x89\x8d',
'CjkUnifiedIdeograph5dcd': '\xe5\xb7\x8d',
'CjkUnifiedIdeograph5dce': '\xe5\xb7\x8e',
'CjkUnifiedIdeograph5dcf': '\xe5\xb7\x8f',
'CjkUnifiedIdeograph5d59': '\xe5\xb5\x99',
'UnicodeCharacter0xfbb2': '\xef\xae\xb2',
'CjkUnifiedIdeograph5dca': '\xe5\xb7\x8a',
'CjkUnifiedIdeograph5dcb': '\xe5\xb7\x8b',
'CjkUnifiedIdeograph5dcc': '\xe5\xb7\x8c',
'UnicodeCharacter0x1a3f': '\xe1\xa8\xbf',
'UnicodeCharacter0x1a3e': '\xe1\xa8\xbe',
'UnicodeCharacter0x1a3d': '\xe1\xa8\xbd',
'UnicodeCharacter0x1a3c': '\xe1\xa8\xbc',
'UnicodeCharacter0x1a3b': '\xe1\xa8\xbb',
'UnicodeCharacter0x1a3a': '\xe1\xa8\xba',
'HangulSyllableSwaelb': '\xec\x87\x8f',
'HangulSyllableGgilm': '\xeb\x82\x86',
'CjkUnifiedIdeograph8bf8': '\xe8\xaf\xb8',
'NotchedLowerRightshadowedWhiteRightwardsArrow': '\xe2\x9e\xaf',
'CyrillicSmallLetterKomiLje': '\xd4\x89',
'HangulSyllableBam': '\xeb\xb0\xa4',
'CjkUnifiedIdeograph8bf2': '\xe8\xaf\xb2',
'CjkUnifiedIdeograph8bf3': '\xe8\xaf\xb3',
'CjkUnifiedIdeograph8bf0': '\xe8\xaf\xb0',
'CjkUnifiedIdeograph8bf1': '\xe8\xaf\xb1',
'CjkUnifiedIdeograph8bf6': '\xe8\xaf\xb6',
'CjkUnifiedIdeograph8bf7': '\xe8\xaf\xb7',
'CjkUnifiedIdeograph8bf4': '\xe8\xaf\xb4',
'CjkUnifiedIdeograph8bf5': '\xe8\xaf\xb5',
'HangulChoseongPieuptikeut': '\xe1\x84\xa0',
'ArabicLigatureLamWithAlefMaksuraIsolatedForm': '\xef\xb1\x83',
'CanadianSyllabicsPwoo': '\xe1\x91\x82',
'GeorgianSmallLetterKhar': '\xe2\xb4\x95',
'CjkUnifiedIdeograph9ec2': '\xe9\xbb\x82',
'ChamLetterCha': '\xea\xa8\x8c',
'UnicodeCharacter0xa7c5': '\xea\x9f\x85',
'DoubleLow9QuotationMark': '\xe2\x80\x9e',
'CjkUnifiedIdeograph9ec7': '\xe9\xbb\x87',
'CjkUnifiedIdeograph8bfb': '\xe8\xaf\xbb',
'CjkUnifiedIdeograph8bfc': '\xe8\xaf\xbc',
'LatinSmallLetterBrokenL': '\xea\x9d\x87',
'CjkUnifiedIdeograph8bfa': '\xe8\xaf\xba',
'CjkUnifiedIdeograph8bff': '\xe8\xaf\xbf',
'VaiSyllableJoo': '\xea\x96\x8c',
'CjkUnifiedIdeograph8bfd': '\xe8\xaf\xbd',
'CjkUnifiedIdeograph8bfe': '\xe8\xaf\xbe',
'CjkUnifiedIdeograph5dc4': '\xe5\xb7\x84',
'CyrillicSmallLetterGhe': '\xd0\xb3',
'CjkUnifiedIdeograph5dc6': '\xe5\xb7\x86',
'CjkUnifiedIdeograph5dc7': '\xe5\xb7\x87',
'CjkUnifiedIdeograph5dc0': '\xe5\xb7\x80',
'CjkRadicalGrassTwo': '\xe2\xba\xbf',
'UnicodeCharacter0x1a39': '\xe1\xa8\xb9',
'UnicodeCharacter0x1a38': '\xe1\xa8\xb8',
'UnicodeCharacter0x1a37': '\xe1\xa8\xb7',
'UnicodeCharacter0x1a36': '\xe1\xa8\xb6',
'UnicodeCharacter0x1a35': '\xe1\xa8\xb5',
'UnicodeCharacter0x1a34': '\xe1\xa8\xb4',
'UnicodeCharacter0x1a33': '\xe1\xa8\xb3',
'UnicodeCharacter0x1a32': '\xe1\xa8\xb2',
'UnicodeCharacter0x1a31': '\xe1\xa8\xb1',
'UnicodeCharacter0x1a30': '\xe1\xa8\xb0',
'HangulSyllableMyaels': '\xeb\xa8\xa8',
'HangulSyllableMyaelp': '\xeb\xa8\xaa',
'VaiSyllableBo': '\xea\x97\x83',
'HangulSyllableDdeult': '\xeb\x9c\xb5',
'HangulSyllableWeobs': '\xec\x9b\x9e',
'BalineseMusicalSymbolRighthandClosedTuk': '\xe1\xad\xb6',
'YiSyllableBa': '\xea\x80\xa0',
'HebrewLetterSamekhWithDagesh': '\xef\xad\x81',
'HangulSyllableMyaelb': '\xeb\xa8\xa7',
'HangulSyllableMyaelg': '\xeb\xa8\xa5',
'SquareDoru': '\xe3\x8c\xa6',
'SquaredSquare': '\xe2\xa7\x88',
'YiSyllableHlyr': '\xea\x86\xb6',
'BalineseLetterJa': '\xe1\xac\x9a',
'YiSyllableHlyp': '\xea\x86\xb4',
'CjkRadicalSnake': '\xe2\xba\x92',
'HangulSyllableDdeuls': '\xeb\x9c\xb4',
'HangulSyllablePyeob': '\xed\x8f\x85',
'HangulJongseongKapyeounmieum': '\xe1\x87\xa2',
'CjkUnifiedIdeograph7a6d': '\xe7\xa9\xad',
'CjkUnifiedIdeograph6d1f': '\xe6\xb4\x9f',
'CjkUnifiedIdeograph6d1e': '\xe6\xb4\x9e',
'CjkUnifiedIdeograph6d1d': '\xe6\xb4\x9d',
'CjkUnifiedIdeograph6d1c': '\xe6\xb4\x9c',
'CjkUnifiedIdeograph6d1b': '\xe6\xb4\x9b',
'CjkUnifiedIdeograph6d1a': '\xe6\xb4\x9a',
'CjkUnifiedIdeograph7a6c': '\xe7\xa9\xac',
'UnicodeCharacter0xde9a': '\xed\xba\x9a',
'CjkUnifiedIdeograph4851': '\xe4\xa1\x91',
'WavyDash': '\xe3\x80\xb0',
'CjkUnifiedIdeograph7299': '\xe7\x8a\x99',
'UnicodeCharacter0xa7c1': '\xea\x9f\x81',
'HangulSyllablePyeok': '\xed\x8f\x8c',
'LatinCapitalLetterBWithDotBelow': '\xe1\xb8\x84',
'ModifierLetterRaisedInvertedExclamationMark': '\xea\x9c\x9e',
'CjkUnifiedIdeograph6d17': '\xe6\xb4\x97',
'CjkUnifiedIdeograph3ec3': '\xe3\xbb\x83',
'CjkUnifiedIdeograph6d15': '\xe6\xb4\x95',
'CjkUnifiedIdeograph6d14': '\xe6\xb4\x94',
'CjkUnifiedIdeograph6d13': '\xe6\xb4\x93',
'CjkUnifiedIdeograph6d12': '\xe6\xb4\x92',
'CjkUnifiedIdeograph6d11': '\xe6\xb4\x91',
'CjkUnifiedIdeograph6d10': '\xe6\xb4\x90',
'CjkUnifiedIdeograph7a68': '\xe7\xa9\xa8',
'CanadianSyllabicsFinalRightHalfRing': '\xe1\x90\xa3',
'CjkUnifiedIdeograph6d19': '\xe6\xb4\x99',
'CjkUnifiedIdeograph6d18': '\xe6\xb4\x98',
'UnicodeCharacter0xa7c0': '\xea\x9f\x80',
'HangulSyllableNyeoss': '\xeb\x85\x94',
'BlackDownpointingTriangle': '\xe2\x96\xbc',
'HangulSyllableIlg': '\xec\x9d\xbd',
'CyrillicCapitalLetterShwe': '\xea\x9a\x96',
'HangulSyllablePyeop': '\xed\x8f\x8e',
'ParenthesizedIdeographSun': '\xe3\x88\xb0',
'GreekCapitalLetterIotaWithMacron': '\xe1\xbf\x99',
'CjkUnifiedIdeograph8e29': '\xe8\xb8\xa9',
'CjkUnifiedIdeograph8e28': '\xe8\xb8\xa8',
'CjkUnifiedIdeograph8e25': '\xe8\xb8\xa5',
'CjkUnifiedIdeograph8e24': '\xe8\xb8\xa4',
'CjkUnifiedIdeograph8e27': '\xe8\xb8\xa7',
'CjkUnifiedIdeograph8e26': '\xe8\xb8\xa6',
'CjkUnifiedIdeograph8e21': '\xe8\xb8\xa1',
'CjkUnifiedIdeograph8e20': '\xe8\xb8\xa0',
'CjkUnifiedIdeograph8e23': '\xe8\xb8\xa3',
'CjkUnifiedIdeograph8e22': '\xe8\xb8\xa2',
'CjkUnifiedIdeograph5e5f': '\xe5\xb9\x9f',
'CjkUnifiedIdeograph5e5e': '\xe5\xb9\x9e',
'CjkUnifiedIdeograph5e5d': '\xe5\xb9\x9d',
'CjkUnifiedIdeograph5e5c': '\xe5\xb9\x9c',
'CjkUnifiedIdeograph5e5b': '\xe5\xb9\x9b',
'CjkUnifiedIdeograph5e5a': '\xe5\xb9\x9a',
'UnicodeCharacter0xcb4': '\xe0\xb2\xb4',
'VulgarFractionOneHalf': '\xc2\xbd',
'CjkUnifiedIdeograph95ca': '\xe9\x97\x8a',
'CjkUnifiedIdeograph95cc': '\xe9\x97\x8c',
'CjkUnifiedIdeograph95cb': '\xe9\x97\x8b',
'CjkUnifiedIdeograph95ce': '\xe9\x97\x8e',
'CjkUnifiedIdeograph95cd': '\xe9\x97\x8d',
'CjkUnifiedIdeograph95cf': '\xe9\x97\x8f',
'FuneralUrn': '\xe2\x9a\xb1',
'UnicodeCharacter0xf66a': '\xef\x99\xaa',
'HangulSyllableGyae': '\xea\xb1\x94',
'CjkUnifiedIdeograph5e57': '\xe5\xb9\x97',
'CjkUnifiedIdeograph5e56': '\xe5\xb9\x96',
'CjkUnifiedIdeograph4280': '\xe4\x8a\x80',
'CjkUnifiedIdeograph5e54': '\xe5\xb9\x94',
'OriyaVowelSignIi': '\xe0\xad\x80',
'CjkUnifiedIdeograph5e52': '\xe5\xb9\x92',
'CjkUnifiedIdeograph5e51': '\xe5\xb9\x91',
'CjkUnifiedIdeograph5e50': '\xe5\xb9\x90',
'UnicodeCharacter0xcbb': '\xe0\xb2\xbb',
'CjkUnifiedIdeograph5e59': '\xe5\xb9\x99',
'UnicodeCharacter0xcba': '\xe0\xb2\xba',
'BoxDrawingsHeavyDownAndRight': '\xe2\x94\x8f',
'CopticMorphologicalDivider': '\xe2\xb3\xbf',
'CjkUnifiedIdeograph8e2e': '\xe8\xb8\xae',
'CjkUnifiedIdeograph8e2d': '\xe8\xb8\xad',
'CjkUnifiedIdeograph8e2f': '\xe8\xb8\xaf',
'CjkUnifiedIdeograph8e2a': '\xe8\xb8\xaa',
'CjkUnifiedIdeograph8e2c': '\xe8\xb8\xac',
'CjkUnifiedIdeograph8e2b': '\xe8\xb8\xab',
'UnicodeCharacter0xe00c': '\xee\x80\x8c',
'YiRadicalYo': '\xea\x92\x9d',
'CjkUnifiedIdeograph90f2': '\xe9\x83\xb2',
'CjkUnifiedIdeograph90f1': '\xe9\x83\xb1',
'CjkUnifiedIdeograph95c1': '\xe9\x97\x81',
'CjkUnifiedIdeograph95c0': '\xe9\x97\x80',
'CjkUnifiedIdeograph95c3': '\xe9\x97\x83',
'CjkUnifiedIdeograph95c2': '\xe9\x97\x82',
'CjkUnifiedIdeograph95c5': '\xe9\x97\x85',
'CjkUnifiedIdeograph95c4': '\xe9\x97\x84',
'CjkUnifiedIdeograph95c7': '\xe9\x97\x87',
'AplFunctionalSymbolJotDiaeresis': '\xe2\x8d\xa4',
'CjkUnifiedIdeograph95c9': '\xe9\x97\x89',
'CjkUnifiedIdeograph95c8': '\xe9\x97\x88',
'UnicodeCharacter0xf1cb': '\xef\x87\x8b',
'NkoSymbolGbakurunen': '\xdf\xb7',
'UnicodeCharacter0xe00f': '\xee\x80\x8f',
'UnicodeCharacter0xe00e': '\xee\x80\x8e',
'LimbuLetterKa': '\xe1\xa4\x81',
'DevanagariDigitOne': '\xe0\xa5\xa7',
'CjkUnifiedIdeograph9c2b': '\xe9\xb0\xab',
'CjkUnifiedIdeograph9c2c': '\xe9\xb0\xac',
'CjkUnifiedIdeograph9c2d': '\xe9\xb0\xad',
'CjkUnifiedIdeograph9c2e': '\xe9\xb0\xae',
'CyrillicCapitalLetterKomiZje': '\xd4\x84',
'uparrow': '\xe2\x86\x91',
'CjkUnifiedIdeograph423e': '\xe4\x88\xbe',
'CjkUnifiedIdeograph423d': '\xe4\x88\xbd',
'CjkUnifiedIdeograph423f': '\xe4\x88\xbf',
'CjkUnifiedIdeograph423a': '\xe4\x88\xba',
'CjkUnifiedIdeograph423c': '\xe4\x88\xbc',
'CjkUnifiedIdeograph423b': '\xe4\x88\xbb',
'CjkUnifiedIdeograph37f6': '\xe3\x9f\xb6',
'CjkUnifiedIdeograph37f7': '\xe3\x9f\xb7',
'CjkUnifiedIdeograph37f4': '\xe3\x9f\xb4',
'CjkUnifiedIdeograph37f5': '\xe3\x9f\xb5',
'CjkUnifiedIdeograph37f2': '\xe3\x9f\xb2',
'CjkUnifiedIdeograph37f3': '\xe3\x9f\xb3',
'ModifierLetterLowDownArrowhead': '\xcb\xaf',
'CjkUnifiedIdeograph37f1': '\xe3\x9f\xb1',
'KannadaLetterNya': '\xe0\xb2\x9e',
'HangulSyllableByaet': '\xeb\xb2\x81',
'HangulSyllableBilt': '\xeb\xb9\x91',
'HangulSyllableByaep': '\xeb\xb2\x82',
'HangulSyllableByaes': '\xeb\xb1\xbb',
'RunicLetterCealc': '\xe1\x9b\xa4',
'ArabicLigatureSadWithRehFinalForm': '\xef\xb4\xab',
'UnicodeCharacter0xf253': '\xef\x89\x93',
'GeorgianSmallLetterPhar': '\xe2\xb4\x94',
'HangulSyllableByaeg': '\xeb\xb1\xa9',
'UnicodeCharacter0xf250': '\xef\x89\x90',
'UnicodeCharacter0xf257': '\xef\x89\x97',
'UnicodeCharacter0xf256': '\xef\x89\x96',
'HangulSyllableByaec': '\xeb\xb1\xbf',
'HangulSyllableByaeb': '\xeb\xb1\xb9',
'HangulSyllableByaem': '\xeb\xb1\xb8',
'HangulSyllableByael': '\xeb\xb1\xb0',
'UnicodeCharacter0xf259': '\xef\x89\x99',
'HangulSyllableByaen': '\xeb\xb1\xac',
'HangulSyllableByaeh': '\xeb\xb2\x83',
'AcCurrent': '\xe2\x8f\xa6',
'HangulSyllableByaej': '\xeb\xb1\xbe',
'CjkUnifiedIdeograph4235': '\xe4\x88\xb5',
'CjkUnifiedIdeograph4234': '\xe4\x88\xb4',
'CjkUnifiedIdeograph4237': '\xe4\x88\xb7',
'CjkUnifiedIdeograph4236': '\xe4\x88\xb6',
'CjkUnifiedIdeograph4231': '\xe4\x88\xb1',
'CjkUnifiedIdeograph4230': '\xe4\x88\xb0',
'CjkUnifiedIdeograph4233': '\xe4\x88\xb3',
'CjkUnifiedIdeograph4232': '\xe4\x88\xb2',
'CjkUnifiedIdeograph37ff': '\xe3\x9f\xbf',
'CjkUnifiedIdeograph37fd': '\xe3\x9f\xbd',
'CjkUnifiedIdeograph37fe': '\xe3\x9f\xbe',
'CjkUnifiedIdeograph37fb': '\xe3\x9f\xbb',
'CjkUnifiedIdeograph4238': '\xe4\x88\xb8',
'HangulSyllableKenh': '\xec\xbc\x86',
'CjkUnifiedIdeograph37fa': '\xe3\x9f\xba',
'CjkUnifiedIdeograph9c28': '\xe9\xb0\xa8',
'CjkUnifiedIdeograph9c29': '\xe9\xb0\xa9',
'YiSyllableZuop': '\xea\x8a\x9a',
'UnicodeCharacter0xea09': '\xee\xa8\x89',
'CjkUnifiedIdeograph9c20': '\xe9\xb0\xa0',
'CjkUnifiedIdeograph9c21': '\xe9\xb0\xa1',
'CjkUnifiedIdeograph9c22': '\xe9\xb0\xa2',
'CjkUnifiedIdeograph9c23': '\xe9\xb0\xa3',
'CjkUnifiedIdeograph9c24': '\xe9\xb0\xa4',
'CjkUnifiedIdeograph9c25': '\xe9\xb0\xa5',
'CjkUnifiedIdeograph9c26': '\xe9\xb0\xa6',
'CjkUnifiedIdeograph9c27': '\xe9\xb0\xa7',
'UnicodeCharacter0xa7c8': '\xea\x9f\x88',
'UnicodeCharacter0xf25b': '\xef\x89\x9b',
'HangulSyllableHyigs': '\xed\x9d\xaf',
'YiSyllableNuop': '\xea\x86\x89',
'UnicodeCharacter0xf25e': '\xef\x89\x9e',
'KangxiRadicalMound': '\xe2\xbe\xa9',
'NewTaiLueLetterLowKva': '\xe1\xa6\xa8',
'CjkUnifiedIdeograph9bce': '\xe9\xaf\x8e',
'ArabicLigatureThehWithMeemMedialForm': '\xef\xb3\xa5',
'UnicodeCharacter0xea02': '\xee\xa8\x82',
'HangulSyllableHyigg': '\xed\x9d\xae',
'CjkUnifiedIdeograph39c6': '\xe3\xa7\x86',
'HangulSyllableJyibs': '\xec\xa6\xb6',
'UnicodeCharacter0xea00': '\xee\xa8\x80',
'UnicodeCharacter0xe8c2': '\xee\xa3\x82',
'UnicodeCharacter0xe8c3': '\xee\xa3\x83',
'HangulSyllableBbabs': '\xeb\xb9\xb2',
'UnicodeCharacter0xe8c1': '\xee\xa3\x81',
'EqualsSignWithBumpyAbove': '\xe2\xaa\xae',
'UnicodeCharacter0xe8c7': '\xee\xa3\x87',
'UnicodeCharacter0xe8c4': '\xee\xa3\x84',
'UnicodeCharacter0xe8c5': '\xee\xa3\x85',
'CjkUnifiedIdeograph3674': '\xe3\x99\xb4',
'KhmerLetterSso': '\xe1\x9e\x9e',
'UnicodeCharacter0xe8c9': '\xee\xa3\x89',
'GurmukhiLetterCa': '\xe0\xa8\x9a',
'RunicLetterThurisazThursThorn': '\xe1\x9a\xa6',
'YiSyllableMyt': '\xea\x83\x89',
'YiSyllableTot': '\xea\x84\xa9',
'HangulSyllableNeunh': '\xeb\x8a\x96',
'YiSyllableMyp': '\xea\x83\x8c',
'YiSyllableTop': '\xea\x84\xac',
'UnicodeCharacter0xa8cb': '\xea\xa3\x8b',
'CjkUnifiedIdeograph4114': '\xe4\x84\x94',
'CjkUnifiedIdeograph3677': '\xe3\x99\xb7',
'CjkUnifiedIdeograph3dd8': '\xe3\xb7\x98',
'YiSyllableMyx': '\xea\x83\x8a',
'YiSyllableTox': '\xea\x84\xaa',
'CjkUnifiedIdeograph554d': '\xe5\x95\x8d',
'CanadianSyllabicsTwaa': '\xe1\x91\xa3',
'CjkUnifiedIdeograph554f': '\xe5\x95\x8f',
'CjkUnifiedIdeograph554a': '\xe5\x95\x8a',
'CjkUnifiedIdeograph554b': '\xe5\x95\x8b',
'CjkUnifiedIdeograph554c': '\xe5\x95\x8c',
'CjkUnifiedIdeograph4117': '\xe4\x84\x97',
'NotLessthan': '\xe2\x89\xae',
'HanunooLetterNga': '\xe1\x9c\xa5',
'UnicodeCharacter0xe631': '\xee\x98\xb1',
'UnicodeCharacter0xe632': '\xee\x98\xb2',
'UnicodeCharacter0xe633': '\xee\x98\xb3',
'UnicodeCharacter0xe634': '\xee\x98\xb4',
'UnicodeCharacter0xa8cc': '\xea\xa3\x8c',
'UnicodeCharacter0xe636': '\xee\x98\xb6',
'KangxiRadicalIce': '\xe2\xbc\x8e',
'UnicodeCharacter0xe638': '\xee\x98\xb8',
'UnicodeCharacter0xe639': '\xee\x98\xb9',
'SquareHuran': '\xe3\x8c\xb5',
'PilcrowSign': '\xc2\xb6',
'HangulSyllableMeugs': '\xeb\xaf\x83',
'HangulSyllableHwap': '\xed\x99\xae',
'HangulSyllableHwas': '\xed\x99\xa7',
'HangulSyllableHwat': '\xed\x99\xad',
'HangulSyllableNeunj': '\xeb\x8a\x95',
'CjkUnifiedIdeograph7740': '\xe7\x9d\x80',
'HangulSyllableHwak': '\xed\x99\xac',
'HangulSyllableHwaj': '\xed\x99\xaa',
'HangulSyllableHwam': '\xed\x99\xa4',
'HangulSyllableHwal': '\xed\x99\x9c',
'UnicodeCharacter0xe8cd': '\xee\xa3\x8d',
'HangulSyllableHwan': '\xed\x99\x98',
'HangulSyllableHwac': '\xed\x99\xab',
'HangulSyllableSswegg': '\xec\x92\x9e',
'HangulSyllableHwae': '\xed\x99\xb0',
'HangulSyllableHwad': '\xed\x99\x9b',
'HangulSyllableHwag': '\xed\x99\x95',
'HangulSyllableBbwigs': '\xec\x80\xa3',
'UnicodeCharacter0xe63b': '\xee\x98\xbb',
'HangulSyllableDwaebs': '\xeb\x90\x8e',
'UnicodeCharacter0xe63d': '\xee\x98\xbd',
'UnicodeCharacter0xe63e': '\xee\x98\xbe',
'VulgarFractionThreeFifths': '\xe2\x85\x97',
'CjkUnifiedIdeograph5544': '\xe5\x95\x84',
'CjkUnifiedIdeograph5545': '\xe5\x95\x85',
'CjkUnifiedIdeograph5546': '\xe5\x95\x86',
'CjkUnifiedIdeograph5547': '\xe5\x95\x87',
'CjkUnifiedIdeograph5540': '\xe5\x95\x80',
'CjkUnifiedIdeograph5541': '\xe5\x95\x81',
'CjkUnifiedIdeograph5542': '\xe5\x95\x82',
'CjkUnifiedIdeograph5543': '\xe5\x95\x83',
'CjkUnifiedIdeograph5548': '\xe5\x95\x88',
'CjkUnifiedIdeograph5549': '\xe5\x95\x89',
'CjkUnifiedIdeograph616c': '\xe6\x85\xac',
'CjkUnifiedIdeograph616b': '\xe6\x85\xab',
'CjkUnifiedIdeograph616a': '\xe6\x85\xaa',
'CjkUnifiedIdeograph931f': '\xe9\x8c\x9f',
'CjkUnifiedIdeograph931a': '\xe9\x8c\x9a',
'CjkUnifiedIdeograph616f': '\xe6\x85\xaf',
'CjkUnifiedIdeograph616e': '\xe6\x85\xae',
'CjkUnifiedIdeograph616d': '\xe6\x85\xad',
'YiSyllableJyr': '\xea\x8f\xbb',
'YiSyllableJyp': '\xea\x8f\xb9',
'UnicodeCharacter0xe001': '\xee\x80\x81',
'YiSyllableJyt': '\xea\x8f\xb6',
'UnicodeCharacter0xeb5b': '\xee\xad\x9b',
'CombiningInvertedDoubleArchBelow': '\xcc\xab',
'AplFunctionalSymbolQuadQuestion': '\xe2\x8d\xb0',
'IdeographicTelegraphSymbolForDayOne': '\xe3\x8f\xa0',
'UnicodeCharacter0xe006': '\xee\x80\x86',
'UnicodeCharacter0xe005': '\xee\x80\x85',
'CjkUnifiedIdeograph7cbb': '\xe7\xb2\xbb',
'CjkUnifiedIdeograph7cbc': '\xe7\xb2\xbc',
'CjkUnifiedIdeograph7cba': '\xe7\xb2\xba',
'CjkUnifiedIdeograph7cbf': '\xe7\xb2\xbf',
'CjkUnifiedIdeograph7cbd': '\xe7\xb2\xbd',
'CjkUnifiedIdeograph7cbe': '\xe7\xb2\xbe',
'HangulSyllableMeugg': '\xeb\xaf\x82',
'CanadianSyllabicsCarrierGa': '\xe1\x97\xb4',
'CanadianSyllabicsCarrierGe': '\xe1\x97\xb1',
'CanadianSyllabicsCarrierGi': '\xe1\x97\xb3',
'TibetanSubjoinedLetterBa': '\xe0\xbe\xa6',
'WhiteRectangle': '\xe2\x96\xad',
'HangulSyllableHigs': '\xed\x9e\x8b',
'CanadianSyllabicsCarrierGo': '\xe1\x97\xb0',
'CjkUnifiedIdeograph6163': '\xe6\x85\xa3',
'CjkUnifiedIdeograph6162': '\xe6\x85\xa2',
'SquatBlackRightwardsArrow': '\xe2\x9e\xa7',
'CjkUnifiedIdeograph6160': '\xe6\x85\xa0',
'CjkUnifiedIdeograph6167': '\xe6\x85\xa7',
'SyriacLetterGamal': '\xdc\x93',
'CjkUnifiedIdeograph6165': '\xe6\x85\xa5',
'CjkUnifiedIdeograph6164': '\xe6\x85\xa4',
'UnicodeCharacter0xe061': '\xee\x81\xa1',
'CjkUnifiedIdeograph6169': '\xe6\x85\xa9',
'CjkUnifiedIdeograph6168': '\xe6\x85\xa8',
'CjkUnifiedIdeograph9319': '\xe9\x8c\x99',
'CjkUnifiedIdeograph9318': '\xe9\x8c\x98',
'HangulSyllableReoss': '\xeb\xa0\x80',
'ExtendedArabicindicDigitFour': '\xdb\xb4',
'LatinCapitalLetterOWithHornAndGrave': '\xe1\xbb\x9c',
'SaurashtraLetterSa': '\xea\xa2\xb1',
'HangulSyllableJjil': '\xec\xb0\x94',
'CjkUnifiedIdeograph7dad': '\xe7\xb6\xad',
'CjkUnifiedIdeograph7cb8': '\xe7\xb2\xb8',
'CjkUnifiedIdeograph7cb9': '\xe7\xb2\xb9',
'CjkUnifiedIdeograph7dae': '\xe7\xb6\xae',
'CjkUnifiedIdeograph7cb2': '\xe7\xb2\xb2',
'CjkUnifiedIdeograph7cb3': '\xe7\xb2\xb3',
'CjkUnifiedIdeograph7cb0': '\xe7\xb2\xb0',
'CjkUnifiedIdeograph7cb1': '\xe7\xb2\xb1',
'CjkUnifiedIdeograph7cb6': '\xe7\xb2\xb6',
'LightVerticalBar': '\xe2\x9d\x98',
'CjkUnifiedIdeograph7cb4': '\xe7\xb2\xb4',
'CjkUnifiedIdeograph7cb5': '\xe7\xb2\xb5',
'SyriacLetterTethGarshuni': '\xdc\x9c',
'CjkUnifiedIdeograph75a2': '\xe7\x96\xa2',
'PartnershipSign': '\xe3\x89\x90',
'UnicodeCharacter0xe51a': '\xee\x94\x9a',
'UnicodeCharacter0xf661': '\xef\x99\xa1',
'CjkUnifiedIdeograph7daa': '\xe7\xb6\xaa',
'CjkUnifiedIdeograph7dab': '\xe7\xb6\xab',
'UnicodeCharacter0xe44a': '\xee\x91\x8a',
'CjkUnifiedIdeograph7dac': '\xe7\xb6\xac',
'CjkCompatibilityIdeographf9b5': '\xef\xa6\xb5',
'HangulSyllableBah': '\xeb\xb0\xaf',
'UnicodeCharacter0xeb5c': '\xee\xad\x9c',
'HangulSyllableNyeon': '\xeb\x85\x84',
'UnicodeCharacter0xd85e': '\xed\xa1\x9e',
'BraillePatternDots12346': '\xe2\xa0\xaf',
'BraillePatternDots12347': '\xe2\xa1\x8f',
'BraillePatternDots12345': '\xe2\xa0\x9f',
'BraillePatternDots12348': '\xe2\xa2\x8f',
'UnicodeCharacter0xf669': '\xef\x99\xa9',
'LeftWigglyFence': '\xe2\xa7\x98',
'UnicodeCharacter0xa6ce': '\xea\x9b\x8e',
'HangulSyllableCwelm': '\xec\xb7\x96',
'SupersetWithDot': '\xe2\xaa\xbe',
'UnicodeCharacter0xed64': '\xee\xb5\xa4',
'HangulSyllableGeuk': '\xea\xb8\x90',
'GreaterthanButNotEquivalentTo': '\xe2\x8b\xa7',
'OcrBranchBankIdentification': '\xe2\x91\x86',
'EjectSymbol': '\xe2\x8f\x8f',
'UnicodeCharacter0xd874': '\xed\xa1\xb4',
'KangxiRadicalCauldron': '\xe2\xbf\x80',
'HangulSyllableNugs': '\xeb\x88\x87',
'GlagoliticSmallLetterChrivi': '\xe2\xb1\x8d',
'MalayalamLetterNga': '\xe0\xb4\x99',
'HangulSyllableNyiss': '\xeb\x8b\x80',
'ArabicLetterTehMarbutaIsolatedForm': '\xef\xba\x93',
'CyrillicCapitalLetterHa': '\xd0\xa5',
'TibetanDigitHalfOne': '\xe0\xbc\xaa',
'CjkUnifiedIdeograph9a73': '\xe9\xa9\xb3',
'UnicodeCharacter0xd870': '\xed\xa1\xb0',
'CjkUnifiedIdeograph7788': '\xe7\x9e\x88',
'CjkUnifiedIdeograph7789': '\xe7\x9e\x89',
'CjkUnifiedIdeograph7784': '\xe7\x9e\x84',
'CjkUnifiedIdeograph7785': '\xe7\x9e\x85',
'CjkUnifiedIdeograph7786': '\xe7\x9e\x86',
'CjkUnifiedIdeograph7787': '\xe7\x9e\x87',
'CjkUnifiedIdeograph7780': '\xe7\x9e\x80',
'CjkUnifiedIdeograph7781': '\xe7\x9e\x81',
'CjkUnifiedIdeograph7782': '\xe7\x9e\x82',
'CjkUnifiedIdeograph7783': '\xe7\x9e\x83',
'LatinCapitalLetterYWithDotAbove': '\xe1\xba\x8e',
'KayahLiLetterGa': '\xea\xa4\x8c',
'/\\': '\xe2\x88\xa7',
'LatinSmallLetterKWithDiagonalStroke': '\xea\x9d\x83',
'HangulSyllableCwelb': '\xec\xb7\x97',
'NaryCoproduct': '\xe2\x88\x90',
'NumberTwentyFullStop': '\xe2\x92\x9b',
'UnicodeCharacter0xda63': '\xed\xa9\xa3',
'SquaredTimes': '\xe2\x8a\xa0',
'LatinCapitalLetterOWithDiaeresisAndMacron': '\xc8\xaa',
'CjkUnifiedIdeograph9bc0': '\xe9\xaf\x80',
'CjkUnifiedIdeograph5883': '\xe5\xa2\x83',
'UnicodeCharacter0xda60': '\xed\xa9\xa0',
'UnicodeCharacter0xdd39': '\xed\xb4\xb9',
'UnicodeCharacter0xdd38': '\xed\xb4\xb8',
'UnicodeCharacter0xe147': '\xee\x85\x87',
'CjkUnifiedIdeograph9cb9': '\xe9\xb2\xb9',
'UnicodeCharacter0xda67': '\xed\xa9\xa7',
'UnicodeCharacter0xdd37': '\xed\xb4\xb7',
'YiSyllableBbap': '\xea\x81\xa1',
'UnicodeCharacter0xdd31': '\xed\xb4\xb1',
'MongolianLetterManchuAliGaliZha': '\xe1\xa2\xa4',
'UnicodeCharacter0xdd33': '\xed\xb4\xb3',
'UnicodeCharacter0xda66': '\xed\xa9\xa6',
'HangulSyllableRyok': '\xeb\xa3\xa4',
'HangulSyllableRyoj': '\xeb\xa3\xa2',
'HangulSyllableJjeh': '\xec\xaa\x83',
'HangulSyllableRyoh': '\xeb\xa3\xa7',
'HangulSyllableJjen': '\xec\xa9\xac',
'HangulSyllableRyon': '\xeb\xa3\x90',
'HangulSyllableRyom': '\xeb\xa3\x9c',
'HangulSyllableRyol': '\xeb\xa3\x94',
'HangulSyllableRyoc': '\xeb\xa3\xa3',
'HangulSyllableRyob': '\xeb\xa3\x9d',
'PassivepullupoutputSymbol': '\xe2\x8e\x92',
'HangulSyllableRyog': '\xeb\xa3\x8d',
'HangulSyllableJjeg': '\xec\xa9\xa9',
'HangulSyllableJjed': '\xec\xa9\xaf',
'HangulSyllableRyod': '\xeb\xa3\x93',
'HangulSyllableCwelt': '\xec\xb7\x99',
'CjkUnifiedIdeograph778d': '\xe7\x9e\x8d',
'CjkUnifiedIdeograph778e': '\xe7\x9e\x8e',
'CjkUnifiedIdeograph778f': '\xe7\x9e\x8f',
'HangulSyllableRyop': '\xeb\xa3\xa6',
'CjkUnifiedIdeograph778a': '\xe7\x9e\x8a',
'CjkUnifiedIdeograph778b': '\xe7\x9e\x8b',
'CjkUnifiedIdeograph778c': '\xe7\x9e\x8c',
'CjkUnifiedIdeograph9bc9': '\xe9\xaf\x89',
'/*': '\xe2\x8b\x87',
'CjkUnifiedIdeograph411a': '\xe4\x84\x9a',
'UnicodeCharacter0xdd3e': '\xed\xb4\xbe',
'HangulSyllableNugg': '\xeb\x88\x86',
'CjkUnifiedIdeograph640c': '\xe6\x90\x8c',
'UnicodeCharacter0xdd3a': '\xed\xb4\xba',
'ArabicLetterAlefWithExtendedArabicindicDigitThreeAbove': '\xdd\xb4',
'PhagspaLetterWa': '\xea\xa1\x93',
'UnicodeCharacter0xa6cc': '\xea\x9b\x8c',
'DownRightDiagonalEllipsis': '\xe2\x8b\xb1',
'CombiningLongVerticalLineOverlay': '\xe2\x83\x92',
'HangulSyllableGeup': '\xea\xb8\x92',
'UnicodeCharacter0xe769': '\xee\x9d\xa9',
'CjkUnifiedIdeograph411d': '\xe4\x84\x9d',
'HangulSyllableMweon': '\xeb\xad\x94',
'SquareEv': '\xe3\x8b\x8e',
'HangulSyllableMweol': '\xeb\xad\x98',
'HangulSyllableMweom': '\xeb\xad\xa0',
'HangulSyllableMweoj': '\xeb\xad\xa6',
'HangulSyllableMweok': '\xeb\xad\xa8',
'HangulSyllableMweoh': '\xeb\xad\xab',
'UnicodeCharacter0xd7f2': '\xed\x9f\xb2',
'HangulSyllableMweog': '\xeb\xad\x91',
'HangulSyllableMweod': '\xeb\xad\x97',
'HangulSyllableMweob': '\xeb\xad\xa1',
'HangulSyllableMweoc': '\xeb\xad\xa7',
'CjkUnifiedIdeograph54e2': '\xe5\x93\xa2',
'CjkUnifiedIdeograph54e3': '\xe5\x93\xa3',
'CjkUnifiedIdeograph54e0': '\xe5\x93\xa0',
'CjkUnifiedIdeograph54e1': '\xe5\x93\xa1',
'CjkUnifiedIdeograph54e6': '\xe5\x93\xa6',
'CjkUnifiedIdeograph54e7': '\xe5\x93\xa7',
'CjkUnifiedIdeograph54e4': '\xe5\x93\xa4',
'CjkUnifiedIdeograph54e5': '\xe5\x93\xa5',
'CjkUnifiedIdeograph548e': '\xe5\x92\x8e',
'CjkUnifiedIdeograph54e9': '\xe5\x93\xa9',
'HangulSyllableMweos': '\xeb\xad\xa3',
'HangulSyllableMweop': '\xeb\xad\xaa',
'CjkUnifiedIdeograph60f6': '\xe6\x83\xb6',
'CjkUnifiedIdeograph60f7': '\xe6\x83\xb7',
'CjkUnifiedIdeograph60f4': '\xe6\x83\xb4',
'CjkUnifiedIdeograph60f5': '\xe6\x83\xb5',
'CjkUnifiedIdeograph60f2': '\xe6\x83\xb2',
'CjkUnifiedIdeograph60f3': '\xe6\x83\xb3',
'CjkUnifiedIdeograph60f0': '\xe6\x83\xb0',
'CjkUnifiedIdeograph60f1': '\xe6\x83\xb1',
'CyrillicSmallLetterDe': '\xd0\xb4',
'CjkUnifiedIdeograph60f8': '\xe6\x83\xb8',
'CjkUnifiedIdeograph60f9': '\xe6\x83\xb9',
'HangulSyllableJyigs': '\xec\xa6\xa7',
'TibetanLetterTha': '\xe0\xbd\x90',
'CircledEquals': '\xe2\x8a\x9c',
'ShadowedWhiteLatinCross': '\xe2\x9c\x9e',
'CjkUnifiedIdeograph54eb': '\xe5\x93\xab',
'CjkUnifiedIdeograph54ec': '\xe5\x93\xac',
'CjkUnifiedIdeograph54ea': '\xe5\x93\xaa',
'CjkUnifiedIdeograph54ef': '\xe5\x93\xaf',
'UnicodeCharacter0xf263': '\xef\x89\xa3',
'CjkUnifiedIdeograph54ed': '\xe5\x93\xad',
'CjkUnifiedIdeograph54ee': '\xe5\x93\xae',
'UnicodeCharacter0xe140': '\xee\x85\x80',
'YiSyllableZu': '\xea\x8a\xa4',
'UnicodeCharacter0xd7fe': '\xed\x9f\xbe',
'UnicodeCharacter0xd7fd': '\xed\x9f\xbd',
'UnicodeCharacter0xd7ff': '\xed\x9f\xbf',
'LatinSmallLetterKWithCedilla': '\xc4\xb7',
'UnicodeCharacter0xd7fc': '\xed\x9f\xbc',
'UnicodeCharacter0xd7fb': '\xed\x9f\xbb',
'HangulSyllableSyop': '\xec\x88\x96',
'CherokeeLetterNi': '\xe1\x8f\x82',
'HangulSyllableSyot': '\xec\x88\x95',
'CherokeeLetterNo': '\xe1\x8f\x83',
'EightTeardropspokedPropellerAsterisk': '\xe2\x9d\x8a',
'CherokeeLetterNa': '\xe1\x8e\xbe',
'ArabicLigatureYehWithHamzaAboveWithAlefIsolatedForm': '\xef\xaf\xaa',
'CherokeeLetterNe': '\xe1\x8f\x81',
'CjkUnifiedIdeograph60ff': '\xe6\x83\xbf',
'CjkUnifiedIdeograph6953': '\xe6\xa5\x93',
'CjkUnifiedIdeograph60fd': '\xe6\x83\xbd',
'TugrikSign': '\xe2\x82\xae',
'CjkUnifiedIdeograph60fb': '\xe6\x83\xbb',
'CjkUnifiedIdeograph60fc': '\xe6\x83\xbc',
'CjkUnifiedIdeograph6954': '\xe6\xa5\x94',
'CjkUnifiedIdeograph60fa': '\xe6\x83\xba',
'HangulSyllableSyoh': '\xec\x88\x97',
'CjkUnifiedIdeograph6958': '\xe6\xa5\x98',
'GeorgianSmallLetterQar': '\xe2\xb4\x97',
'CherokeeLetterNv': '\xe1\x8f\x85',
'HangulSyllableSyom': '\xec\x88\x8c',
'HangulSyllableSyon': '\xec\x88\x80',
'CherokeeLetterNu': '\xe1\x8f\x84',
'CjkUnifiedIdeograph8b53': '\xe8\xad\x93',
'CjkUnifiedIdeograph3b29': '\xe3\xac\xa9',
'CjkUnifiedIdeograph3b28': '\xe3\xac\xa8',
'LepchaDigitTwo': '\xe1\xb1\x82',
'TeluguLetterNya': '\xe0\xb0\x9e',
'CjkUnifiedIdeograph3b23': '\xe3\xac\xa3',
'CjkUnifiedIdeograph3b22': '\xe3\xac\xa2',
'CjkUnifiedIdeograph3b21': '\xe3\xac\xa1',
'CjkUnifiedIdeograph3b20': '\xe3\xac\xa0',
'CjkUnifiedIdeograph3b27': '\xe3\xac\xa7',
'CjkUnifiedIdeograph3b26': '\xe3\xac\xa6',
'CjkUnifiedIdeograph3b25': '\xe3\xac\xa5',
'CjkUnifiedIdeograph3b24': '\xe3\xac\xa4',
'Diaeresis': '\xc2\xa8',
'CyrillicCapitalLetterEr': '\xd0\xa0',
'UnicodeCharacter0xd87a': '\xed\xa1\xba',
'CjkUnifiedIdeograph8b55': '\xe8\xad\x95',
'HalfwidthHangulLetterWi': '\xef\xbf\x96',
'UnicodeCharacter0xf02c': '\xef\x80\xac',
'HangulSyllableDdwel': '\xeb\x9b\x9c',
'GujaratiLetterTha': '\xe0\xaa\xa5',
'KangxiRadicalFather': '\xe2\xbd\x97',
'OriyaLetterYa': '\xe0\xac\xaf',
'ArabicLetterKehehWithThreeDotsAbove': '\xdd\xa3',
'UnicodeCharacter0xe3f5': '\xee\x8f\xb5',
'UnicodeCharacter0xe519': '\xee\x94\x99',
'UnicodeCharacter0xe518': '\xee\x94\x98',
'LepchaPunctuationTshook': '\xe1\xb0\xbf',
'HangulSyllableGop': '\xea\xb3\xba',
'HangulSyllableDdyunh': '\xeb\x9c\x92',
'HangulSyllableGot': '\xea\xb3\xb9',
'HangulSyllableDdwem': '\xeb\x9b\xa4',
'CjkUnifiedIdeograph9700': '\xe9\x9c\x80',
'HangulSyllableDdyung': '\xeb\x9c\xa1',
'HangulSyllableGob': '\xea\xb3\xb1',
'CjkUnifiedIdeograph7da4': '\xe7\xb6\xa4',
'HangulSyllableBib': '\xeb\xb9\x95',
'InvertedQuestionMark': '\xc2\xbf',
'HangulSyllableGog': '\xea\xb3\xa1',
'HangulSyllableGod': '\xea\xb3\xa7',
'HangulSyllableGoe': '\xea\xb4\xb4',
'CjkUnifiedIdeograph3b2c': '\xe3\xac\xac',
'CjkUnifiedIdeograph3b2b': '\xe3\xac\xab',
'CjkUnifiedIdeograph3b2a': '\xe3\xac\xaa',
'MalayalamLetterBa': '\xe0\xb4\xac',
'HangulSyllableGon': '\xea\xb3\xa4',
'CjkUnifiedIdeograph3b2f': '\xe3\xac\xaf',
'ArabicindicDigitEight': '\xd9\xa8',
'CjkUnifiedIdeograph3b2d': '\xe3\xac\xad',
'SaurashtraDigitZero': '\xea\xa3\x90',
'CjkUnifiedIdeograph7da7': '\xe7\xb6\xa7',
'YiSyllableGit': '\xea\x87\x9a',
'CjkUnifiedIdeograph7da0': '\xe7\xb6\xa0',
'YiSyllableGix': '\xea\x87\x9b',
'SquareAm': '\xe3\x8f\x82',
'HangulChoseongSsangnieun': '\xe1\x84\x94',
'CjkUnifiedIdeograph6f08': '\xe6\xbc\x88',
'YiSyllableGie': '\xea\x87\xa0',
'SquareAu': '\xe3\x8d\xb3',
'CjkUnifiedIdeograph6f09': '\xe6\xbc\x89',
'HangulSyllableWis': '\xec\x9c\x97',
'MyanmarLetterPa': '\xe1\x80\x95',
'CjkUnifiedIdeograph9db3': '\xe9\xb6\xb3',
'CjkUnifiedIdeograph98bf': '\xe9\xa2\xbf',
'CjkUnifiedIdeograph98be': '\xe9\xa2\xbe',
'CjkUnifiedIdeograph98bd': '\xe9\xa2\xbd',
'CjkUnifiedIdeograph98bc': '\xe9\xa2\xbc',
'CjkUnifiedIdeograph98bb': '\xe9\xa2\xbb',
'CjkUnifiedIdeograph98ba': '\xe9\xa2\xba',
'CjkUnifiedIdeograph9db4': '\xe9\xb6\xb4',
'CjkUnifiedIdeograph9db9': '\xe9\xb6\xb9',
'CjkUnifiedIdeograph9db8': '\xe9\xb6\xb8',
'CjkUnifiedIdeograph85f2': '\xe8\x97\xb2',
'DevanagariLetterGa': '\xe0\xa4\x97',
'BlackNib': '\xe2\x9c\x92',
'CjkUnifiedIdeograph7d1d': '\xe7\xb4\x9d',
'CjkUnifiedIdeograph7d1e': '\xe7\xb4\x9e',
'CjkUnifiedIdeograph7d1f': '\xe7\xb4\x9f',
'CjkUnifiedIdeograph85f6': '\xe8\x97\xb6',
'UnicodeCharacter0xe5aa': '\xee\x96\xaa',
'CjkUnifiedIdeograph7d1a': '\xe7\xb4\x9a',
'CjkUnifiedIdeograph7d1b': '\xe7\xb4\x9b',
'CjkUnifiedIdeograph7d1c': '\xe7\xb4\x9c',
'BraillePatternDots123458': '\xe2\xa2\x9f',
'CjkUnifiedIdeograph98b6': '\xe9\xa2\xb6',
'CjkUnifiedIdeograph98b5': '\xe9\xa2\xb5',
'CjkUnifiedIdeograph9734': '\xe9\x9c\xb4',
'CjkUnifiedIdeograph98b3': '\xe9\xa2\xb3',
'LepchaConsonantSignNyindo': '\xe1\xb0\xb4',
'CjkUnifiedIdeograph98b1': '\xe9\xa2\xb1',
'CjkUnifiedIdeograph98b0': '\xe9\xa2\xb0',
'BraillePatternDots3478': '\xe2\xa3\x8c',
'CjkUnifiedIdeograph9b84': '\xe9\xae\x84',
'HangulSyllableRaegs': '\xeb\x9e\x9b',
'BraillePatternDots123456': '\xe2\xa0\xbf',
'BraillePatternDots123457': '\xe2\xa1\x9f',
'CjkUnifiedIdeograph691f': '\xe6\xa4\x9f',
'HangulSyllablePigg': '\xed\x94\xbe',
'YiSyllableDdur': '\xea\x85\x90',
'CombiningCircumflexAccentBelow': '\xcc\xad',
'HangulSyllableRaegg': '\xeb\x9e\x9a',
'MalayalamVowelSignVocalicRr': '\xe0\xb5\x84',
'UnicodeCharacter0xa9d8': '\xea\xa7\x98',
'HangulSyllableSsols': '\xec\x8f\xa4',
'SundaneseDigitThree': '\xe1\xae\xb3',
'UnicodeCharacter0xa9d9': '\xea\xa7\x99',
'CjkUnifiedIdeograph7d18': '\xe7\xb4\x98',
'CjkUnifiedIdeograph7d19': '\xe7\xb4\x99',
'HangulSyllableSsolt': '\xec\x8f\xa5',
'CjkUnifiedIdeograph7d14': '\xe7\xb4\x94',
'CjkUnifiedIdeograph7d15': '\xe7\xb4\x95',
'CjkUnifiedIdeograph7d16': '\xe7\xb4\x96',
'CjkUnifiedIdeograph7d17': '\xe7\xb4\x97',
'CjkUnifiedIdeograph7d10': '\xe7\xb4\x90',
'CjkUnifiedIdeograph7d11': '\xe7\xb4\x91',
'CjkUnifiedIdeograph7d12': '\xe7\xb4\x92',
'CjkUnifiedIdeograph7d13': '\xe7\xb4\x93',
'HangulSyllableSsolb': '\xec\x8f\xa3',
'LepchaVowelSignE': '\xe1\xb0\xac',
'UnicodeCharacter0xf0ab': '\xef\x82\xab',
'HangulSyllableSsolg': '\xec\x8f\xa1',
'SquareMahha': '\xe3\x8d\x85',
'LepchaVowelSignO': '\xe1\xb0\xa8',
'HangulSyllableSsolh': '\xec\x8f\xa7',
'LepchaVowelSignI': '\xe1\xb0\xa7',
'CjkCompatibilityIdeographfa9f': '\xef\xaa\x9f',
'CjkCompatibilityIdeographfa9d': '\xef\xaa\x9d',
'Sesquiquadrate': '\xe2\x9a\xbc',
'CjkCompatibilityIdeographfa9b': '\xef\xaa\x9b',
'CjkCompatibilityIdeographfa9c': '\xef\xaa\x9c',
'CjkCompatibilityIdeographfa9a': '\xef\xaa\x9a',
'UnicodeCharacter0xd0d': '\xe0\xb4\x8d',
'CjkUnifiedIdeograph4e7b': '\xe4\xb9\xbb',
'CjkUnifiedIdeograph4e7c': '\xe4\xb9\xbc',
'CjkUnifiedIdeograph4e7a': '\xe4\xb9\xba',
'CjkUnifiedIdeograph4e7f': '\xe4\xb9\xbf',
'CjkUnifiedIdeograph4e7d': '\xe4\xb9\xbd',
'KhmerDigitSix': '\xe1\x9f\xa6',
'HangulSyllableNwaeb': '\xeb\x87\x81',
'UnicodeCharacter0xf01a': '\xef\x80\x9a',
'HangulSyllableSunj': '\xec\x88\x9d',
'UnicodeCharacter0xf01c': '\xef\x80\x9c',
'HangulSyllableSunh': '\xec\x88\x9e',
'UnicodeCharacter0xf01e': '\xef\x80\x9e',
'UnicodeCharacter0xf01d': '\xef\x80\x9d',
'CjkUnifiedIdeograph56d9': '\xe5\x9b\x99',
'CjkUnifiedIdeograph56d8': '\xe5\x9b\x98',
'CjkUnifiedIdeograph56d7': '\xe5\x9b\x97',
'CjkUnifiedIdeograph56d6': '\xe5\x9b\x96',
'CjkUnifiedIdeograph56d5': '\xe5\x9b\x95',
'CjkUnifiedIdeograph56d4': '\xe5\x9b\x94',
'CjkUnifiedIdeograph56d3': '\xe5\x9b\x93',
'CjkUnifiedIdeograph56d2': '\xe5\x9b\x92',
'CjkUnifiedIdeograph56d1': '\xe5\x9b\x91',
'CjkUnifiedIdeograph56d0': '\xe5\x9b\x90',
'SmallQuestionMark': '\xef\xb9\x96',
'HangulSyllableDils': '\xeb\x94\xa0',
'HangulSyllableWibs': '\xec\x9c\x96',
'BugineseLetterRa': '\xe1\xa8\x91',
'CjkUnifiedIdeograph35b6': '\xe3\x96\xb6',
'CjkUnifiedIdeograph4e72': '\xe4\xb9\xb2',
'CjkUnifiedIdeograph4e73': '\xe4\xb9\xb3',
'CjkUnifiedIdeograph4e70': '\xe4\xb9\xb0',
'CjkUnifiedIdeograph4e71': '\xe4\xb9\xb1',
'CjkUnifiedIdeograph4e76': '\xe4\xb9\xb6',
'CjkUnifiedIdeograph4e77': '\xe4\xb9\xb7',
'CanadianSyllabicsCarrierZu': '\xe1\x99\x80',
'CjkUnifiedIdeograph4e75': '\xe4\xb9\xb5',
'CjkUnifiedIdeograph4e78': '\xe4\xb9\xb8',
'CjkUnifiedIdeograph4e79': '\xe4\xb9\xb9',
'CjkCompatibilityIdeographfa96': '\xef\xaa\x96',
'CjkCompatibilityIdeographfa97': '\xef\xaa\x97',
'CanadianSyllabicsCarrierZa': '\xe1\x99\x85',
'CjkUnifiedIdeograph56ac': '\xe5\x9a\xac',
'CjkCompatibilityIdeographfa92': '\xef\xaa\x92',
'WhiteRightpointingPointer': '\xe2\x96\xbb',
'CanadianSyllabicsCarrierZe': '\xe1\x99\x82',
'CjkCompatibilityIdeographfa91': '\xef\xaa\x91',
'UnicodeCharacter0xd01': '\xe0\xb4\x81',
'UnicodeCharacter0xd00': '\xe0\xb4\x80',
'CanadianSyllabicsCarrierZo': '\xe1\x99\x81',
'CjkCompatibilityIdeographfa98': '\xef\xaa\x98',
'UnicodeCharacter0xd04': '\xe0\xb4\x84',
'UnicodeCharacter0xf011': '\xef\x80\x91',
'UnicodeCharacter0xf010': '\xef\x80\x90',
'UnicodeCharacter0xf013': '\xef\x80\x93',
'UnicodeCharacter0xf012': '\xef\x80\x92',
'UnicodeCharacter0xe4e8': '\xee\x93\xa8',
'UnicodeCharacter0xe4e9': '\xee\x93\xa9',
'UnicodeCharacter0xdf64': '\xed\xbd\xa4',
'UnicodeCharacter0xf016': '\xef\x80\x96',
'MongolianLetterSibeUe': '\xe1\xa1\xa0',
'CjkUnifiedIdeograph56df': '\xe5\x9b\x9f',
'CjkUnifiedIdeograph56de': '\xe5\x9b\x9e',
'CjkUnifiedIdeograph56dd': '\xe5\x9b\x9d',
'CjkUnifiedIdeograph56dc': '\xe5\x9b\x9c',
'CjkUnifiedIdeograph56db': '\xe5\x9b\x9b',
'CjkUnifiedIdeograph56da': '\xe5\x9b\x9a',
'UnicodeCharacter0xe4e3': '\xee\x93\xa3',
'UnicodeCharacter0xabc8': '\xea\xaf\x88',
'UnicodeCharacter0xabc9': '\xea\xaf\x89',
'CjkUnifiedIdeograph898a': '\xe8\xa6\x8a',
'HangulSyllableTeus': '\xed\x8b\x8b',
'UnicodeCharacter0xabc0': '\xea\xaf\x80',
'UnicodeCharacter0xabc1': '\xea\xaf\x81',
'UnicodeCharacter0xabc2': '\xea\xaf\x82',
'UnicodeCharacter0xabc3': '\xea\xaf\x83',
'UnicodeCharacter0xabc4': '\xea\xaf\x84',
'UnicodeCharacter0xabc5': '\xea\xaf\x85',
'UnicodeCharacter0xabc6': '\xea\xaf\x86',
'UnicodeCharacter0xabc7': '\xea\xaf\x87',
'ArabicLigatureQafWithAlefMaksuraFinalForm': '\xef\xb1\xbe',
'CjkUnifiedIdeograph539f': '\xe5\x8e\x9f',
'UnicodeCharacter0xeadc': '\xee\xab\x9c',
'UnicodeCharacter0xe75d': '\xee\x9d\x9d',
'CjkUnifiedIdeograph7ecf': '\xe7\xbb\x8f',
'CjkUnifiedIdeograph7ece': '\xe7\xbb\x8e',
'CjkUnifiedIdeograph7ecd': '\xe7\xbb\x8d',
'CjkUnifiedIdeograph7ecc': '\xe7\xbb\x8c',
'CjkUnifiedIdeograph7ecb': '\xe7\xbb\x8b',
'CjkUnifiedIdeograph7eca': '\xe7\xbb\x8a',
'CjkUnifiedIdeograph57b0': '\xe5\x9e\xb0',
'BoxDrawingsDownDoubleAndRightSingle': '\xe2\x95\x93',
'CjkUnifiedIdeograph57b2': '\xe5\x9e\xb2',
'CjkUnifiedIdeograph57b3': '\xe5\x9e\xb3',
'CjkUnifiedIdeograph57b4': '\xe5\x9e\xb4',
'CjkUnifiedIdeograph57b5': '\xe5\x9e\xb5',
'CjkUnifiedIdeograph57b6': '\xe5\x9e\xb6',
'CjkUnifiedIdeograph57b7': '\xe5\x9e\xb7',
'CjkUnifiedIdeograph57b8': '\xe5\x9e\xb8',
'CjkUnifiedIdeograph57b9': '\xe5\x9e\xb9',
'UnicodeCharacter0xe75c': '\xee\x9d\x9c',
'UnicodeCharacter0xe5f9': '\xee\x97\xb9',
'LatinCapitalLetterLWithBar': '\xc8\xbd',
'CjkUnifiedIdeograph76a2': '\xe7\x9a\xa2',
'UnicodeCharacter0xa9da': '\xea\xa7\x9a',
'UnicodeCharacter0xa7cf': '\xea\x9f\x8f',
'CjkUnifiedIdeograph76a3': '\xe7\x9a\xa3',
'UnicodeCharacter0xabca': '\xea\xaf\x8a',
'UnicodeCharacter0xabcb': '\xea\xaf\x8b',
'UnicodeCharacter0xabcc': '\xea\xaf\x8c',
'UnicodeCharacter0xabcd': '\xea\xaf\x8d',
'CjkUnifiedIdeograph76a0': '\xe7\x9a\xa0',
'UnicodeCharacter0xabcf': '\xea\xaf\x8f',
'CjkUnifiedIdeograph57ba': '\xe5\x9e\xba',
'CjkUnifiedIdeograph57bb': '\xe5\x9e\xbb',
'CjkUnifiedIdeograph5644': '\xe5\x99\x84',
'CjkUnifiedIdeograph57bd': '\xe5\x9e\xbd',
'CjkUnifiedIdeograph57be': '\xe5\x9e\xbe',
'CjkUnifiedIdeograph57bf': '\xe5\x9e\xbf',
'CjkUnifiedIdeograph7a3f': '\xe7\xa8\xbf',
'CjkUnifiedIdeograph76a6': '\xe7\x9a\xa6',
'HebrewLetterReshWithDagesh': '\xef\xad\x88',
'UnicodeCharacter0xa9de': '\xea\xa7\x9e',
'CjkUnifiedIdeograph8538': '\xe8\x94\xb8',
'CjkUnifiedIdeograph5642': '\xe5\x99\x82',
'CjkUnifiedIdeograph76a4': '\xe7\x9a\xa4',
'CjkUnifiedIdeograph7ec9': '\xe7\xbb\x89',
'CjkUnifiedIdeograph7ec8': '\xe7\xbb\x88',
'CjkUnifiedIdeograph7ec7': '\xe7\xbb\x87',
'CjkUnifiedIdeograph7ec6': '\xe7\xbb\x86',
'CjkUnifiedIdeograph7ec5': '\xe7\xbb\x85',
'CjkUnifiedIdeograph76a5': '\xe7\x9a\xa5',
'CjkUnifiedIdeograph7ec3': '\xe7\xbb\x83',
'CjkUnifiedIdeograph7ec2': '\xe7\xbb\x82',
'CjkUnifiedIdeograph7ec1': '\xe7\xbb\x81',
'CjkUnifiedIdeograph7ec0': '\xe7\xbb\x80',
'CjkUnifiedIdeograph49c2': '\xe4\xa7\x82',
'CjkUnifiedIdeograph49c3': '\xe4\xa7\x83',
'CjkUnifiedIdeograph49c0': '\xe4\xa7\x80',
'CjkUnifiedIdeograph49c1': '\xe4\xa7\x81',
'CjkUnifiedIdeograph49c6': '\xe4\xa7\x86',
'CjkUnifiedIdeograph49c7': '\xe4\xa7\x87',
'CjkUnifiedIdeograph49c4': '\xe4\xa7\x84',
'CjkUnifiedIdeograph49c5': '\xe4\xa7\x85',
'CjkUnifiedIdeograph49c8': '\xe4\xa7\x88',
'CjkUnifiedIdeograph49c9': '\xe4\xa7\x89',
'CyrillicCapitalLetterCheWithDescender': '\xd2\xb6',
'GreekCapitalLetterOmegaWithDasiaAndOxiaAndProsgegrammeni': '\xe1\xbe\xad',
'CjkUnifiedIdeograph8ae8': '\xe8\xab\xa8',
'CjkUnifiedIdeograph7fca': '\xe7\xbf\x8a',
'CjkUnifiedIdeograph7fcb': '\xe7\xbf\x8b',
'CjkUnifiedIdeograph7fcc': '\xe7\xbf\x8c',
'CjkUnifiedIdeograph7fcd': '\xe7\xbf\x8d',
'CjkUnifiedIdeograph7fce': '\xe7\xbf\x8e',
'CjkUnifiedIdeograph7fcf': '\xe7\xbf\x8f',
'UnicodeCharacter0xe73f': '\xee\x9c\xbf',
'UnicodeCharacter0x23eb': '\xe2\x8f\xab',
'UnicodeCharacter0x23ec': '\xe2\x8f\xac',
'CjkUnifiedIdeograph8ae2': '\xe8\xab\xa2',
'UnicodeCharacter0x23ea': '\xe2\x8f\xaa',
'UnicodeCharacter0x23ef': '\xe2\x8f\xaf',
'CjkUnifiedIdeograph8530': '\xe8\x94\xb0',
'UnicodeCharacter0x23ed': '\xe2\x8f\xad',
'UnicodeCharacter0x23ee': '\xe2\x8f\xae',
'CjkUnifiedIdeograph9f50': '\xe9\xbd\x90',
'CjkUnifiedIdeograph9f51': '\xe9\xbd\x91',
'CjkUnifiedIdeograph9f52': '\xe9\xbd\x92',
'CjkUnifiedIdeograph8531': '\xe8\x94\xb1',
'CjkUnifiedIdeograph9f54': '\xe9\xbd\x94',
'CjkUnifiedIdeograph9f55': '\xe9\xbd\x95',
'CjkUnifiedIdeograph9f56': '\xe9\xbd\x96',
'CjkUnifiedIdeograph9f57': '\xe9\xbd\x97',
'CjkUnifiedIdeograph800f': '\xe8\x80\x8f',
'CjkUnifiedIdeograph8536': '\xe8\x94\xb6',
'CjkUnifiedIdeograph800d': '\xe8\x80\x8d',
'CjkUnifiedIdeograph800e': '\xe8\x80\x8e',
'CjkUnifiedIdeograph800b': '\xe8\x80\x8b',
'CjkUnifiedIdeograph800c': '\xe8\x80\x8c',
'CjkUnifiedIdeograph800a': '\xe8\x80\x8a',
'UnicodeCharacter0xa7cc': '\xea\x9f\x8c',
'UnicodeCharacter0xdbdc': '\xed\xaf\x9c',
'HangulJongseongPieupsios': '\xe1\x86\xb9',
'UnicodeCharacter0xf0a8': '\xef\x82\xa8',
'UnicodeCharacter0xdab8': '\xed\xaa\xb8',
'CjkUnifiedIdeograph7fc0': '\xe7\xbf\x80',
'CjkUnifiedIdeograph7fc1': '\xe7\xbf\x81',
'UnicodeCharacter0x23e8': '\xe2\x8f\xa8',
'UnicodeCharacter0x23e9': '\xe2\x8f\xa9',
'CjkUnifiedIdeograph7fc4': '\xe7\xbf\x84',
'CjkUnifiedIdeograph7fc5': '\xe7\xbf\x85',
'CjkUnifiedIdeograph7fc6': '\xe7\xbf\x86',
'CjkUnifiedIdeograph7fc7': '\xe7\xbf\x87',
'CjkUnifiedIdeograph7fc8': '\xe7\xbf\x88',
'CjkUnifiedIdeograph7fc9': '\xe7\xbf\x89',
'CjkUnifiedIdeograph8aeb': '\xe8\xab\xab',
'CjkUnifiedIdeograph8aec': '\xe8\xab\xac',
'CjkUnifiedIdeograph8aed': '\xe8\xab\xad',
'CjkUnifiedIdeograph8aee': '\xe8\xab\xae',
'CjkUnifiedIdeograph8aef': '\xe8\xab\xaf',
'LatinCapitalLetterRumRotunda': '\xea\x9d\x9c',
'CjkUnifiedIdeograph49cb': '\xe4\xa7\x8b',
'CjkUnifiedIdeograph49cc': '\xe4\xa7\x8c',
'CjkUnifiedIdeograph49ca': '\xe4\xa7\x8a',
'CjkUnifiedIdeograph49cf': '\xe4\xa7\x8f',
'CjkUnifiedIdeograph49cd': '\xe4\xa7\x8d',
'CjkUnifiedIdeograph49ce': '\xe4\xa7\x8e',
'GlagoliticSmallLetterOnu': '\xe2\xb1\x81',
'LatinCapitalLetterDzWithCaron': '\xc7\x84',
'YiSyllableFat': '\xea\x83\x91',
'Snowman': '\xe2\x98\x83',
'HangulSyllableKyaegg': '\xec\xbb\x8a',
'SundaneseVowelSignPanolong': '\xe1\xae\xa7',
'CjkUnifiedIdeograph9f5a': '\xe9\xbd\x9a',
'CjkUnifiedIdeograph9f5b': '\xe9\xbd\x9b',
'CjkUnifiedIdeograph9f5c': '\xe9\xbd\x9c',
'CjkUnifiedIdeograph9f5d': '\xe9\xbd\x9d',
'CjkUnifiedIdeograph9f5e': '\xe9\xbd\x9e',
'CjkUnifiedIdeograph8008': '\xe8\x80\x88',
'CjkUnifiedIdeograph8009': '\xe8\x80\x89',
'CjkUnifiedIdeograph8006': '\xe8\x80\x86',
'CjkUnifiedIdeograph8007': '\xe8\x80\x87',
'CjkUnifiedIdeograph8004': '\xe8\x80\x84',
'CjkUnifiedIdeograph8005': '\xe8\x80\x85',
'CjkUnifiedIdeograph8002': '\xe8\x80\x82',
'CjkUnifiedIdeograph8003': '\xe8\x80\x83',
'CjkUnifiedIdeograph8000': '\xe8\x80\x80',
'CjkUnifiedIdeograph8001': '\xe8\x80\x81',
'UnicodeCharacter0xf2ce': '\xef\x8b\x8e',
'CjkUnifiedIdeograph6f9a': '\xe6\xbe\x9a',
'YiSyllableCa': '\xea\x8a\xb8',
'UnicodeCharacter0xf2cf': '\xef\x8b\x8f',
'UnicodeCharacter0xf2ca': '\xef\x8b\x8a',
'YiSyllableCe': '\xea\x8b\x82',
'UnicodeCharacter0xf2cb': '\xef\x8b\x8b',
'CjkUnifiedIdeograph966e': '\xe9\x99\xae',
'CjkUnifiedIdeograph966d': '\xe9\x99\xad',
'YiSyllableCi': '\xea\x8a\xb0',
'CjkUnifiedIdeograph966f': '\xe9\x99\xaf',
'CjkUnifiedIdeograph966a': '\xe9\x99\xaa',
'CjkUnifiedIdeograph6f9c': '\xe6\xbe\x9c',
'CjkUnifiedIdeograph966c': '\xe9\x99\xac',
'CjkUnifiedIdeograph966b': '\xe9\x99\xab',
'UnicodeCharacter0xf83c': '\xef\xa0\xbc',
'UnicodeCharacter0xf83b': '\xef\xa0\xbb',
'UnicodeCharacter0xf83a': '\xef\xa0\xba',
'CjkUnifiedIdeograph6f9b': '\xe6\xbe\x9b',
'UnicodeCharacter0xf83f': '\xef\xa0\xbf',
'YiSyllableCu': '\xea\x8b\x86',
'CjkUnifiedIdeograph973f': '\xe9\x9c\xbf',
'CjkUnifiedIdeograph517e': '\xe5\x85\xbe',
'CjkUnifiedIdeograph517d': '\xe5\x85\xbd',
'YiSyllableCy': '\xea\x8b\x8c',
'CjkUnifiedIdeograph517f': '\xe5\x85\xbf',
'KhmerSymbolDappiiRoc': '\xe1\xa7\xbc',
'CjkUnifiedIdeograph517c': '\xe5\x85\xbc',
'CjkUnifiedIdeograph517b': '\xe5\x85\xbb',
'MyanmarLetterI': '\xe1\x80\xa3',
'ExtendedArabicindicDigitOne': '\xdb\xb1',
'ArabicLigatureYehWithMeemIsolatedForm': '\xef\xb1\x98',
'CanadianSyllabicsCoo': '\xe1\x92\x8e',
'MyanmarLetterA': '\xe1\x80\xa1',
'CjkUnifiedIdeograph6f9f': '\xe6\xbe\x9f',
'MyanmarLetterE': '\xe1\x80\xa7',
'UnicodeCharacter0xe0b6': '\xee\x82\xb6',
'CombiningCyrillicLetterFita': '\xe2\xb7\xb4',
'MyanmarLetterU': '\xe1\x80\xa5',
'UnicodeCharacter0xf833': '\xef\xa0\xb3',
'UnicodeCharacter0xf832': '\xef\xa0\xb2',
'UnicodeCharacter0xf831': '\xef\xa0\xb1',
'UnicodeCharacter0xf830': '\xef\xa0\xb0',
'CjkUnifiedIdeograph5179': '\xe5\x85\xb9',
'CjkUnifiedIdeograph5178': '\xe5\x85\xb8',
'UnicodeCharacter0xf835': '\xef\xa0\xb5',
'UnicodeCharacter0xf834': '\xef\xa0\xb4',
'CjkUnifiedIdeograph5175': '\xe5\x85\xb5',
'CjkUnifiedIdeograph5174': '\xe5\x85\xb4',
'CjkUnifiedIdeograph5177': '\xe5\x85\xb7',
'CjkUnifiedIdeograph5176': '\xe5\x85\xb6',
'CjkUnifiedIdeograph5171': '\xe5\x85\xb1',
'CjkUnifiedIdeograph5170': '\xe5\x85\xb0',
'CjkUnifiedIdeograph5173': '\xe5\x85\xb3',
'CjkUnifiedIdeograph5172': '\xe5\x85\xb2',
'UnicodeCharacter0xf2c5': '\xef\x8b\x85',
'UnicodeCharacter0xf2c4': '\xef\x8b\x84',
'UnicodeCharacter0xdab3': '\xed\xaa\xb3',
'UnicodeCharacter0xf2c6': '\xef\x8b\x86',
'CjkUnifiedIdeograph9669': '\xe9\x99\xa9',
'CjkUnifiedIdeograph9668': '\xe9\x99\xa8',
'UnicodeCharacter0xf2c3': '\xef\x8b\x83',
'GreekSmallLetterEtaWithYpogegrammeni': '\xe1\xbf\x83',
'CjkUnifiedIdeograph9665': '\xe9\x99\xa5',
'CjkUnifiedIdeograph9664': '\xe9\x99\xa4',
'CjkUnifiedIdeograph9667': '\xe9\x99\xa7',
'CjkUnifiedIdeograph9666': '\xe9\x99\xa6',
'CjkUnifiedIdeograph9661': '\xe9\x99\xa1',
'CjkUnifiedIdeograph9660': '\xe9\x99\xa0',
'CjkUnifiedIdeograph9663': '\xe9\x99\xa3',
'CjkUnifiedIdeograph9662': '\xe9\x99\xa2',
'MongolianLetterBa': '\xe1\xa0\xaa',
'CherokeeLetterSu': '\xe1\x8f\x91',
'CherokeeLetterSv': '\xe1\x8f\x92',
'CherokeeLetterSi': '\xe1\x8f\x8f',
'CjkUnifiedIdeograph7d6e': '\xe7\xb5\xae',
'UnicodeCharacter0xf861': '\xef\xa1\xa1',
'CherokeeLetterSo': '\xe1\x8f\x90',
'CjkUnifiedIdeograph76ac': '\xe7\x9a\xac',
'CherokeeLetterSa': '\xe1\x8f\x8c',
'KangxiRadicalOppose': '\xe2\xbe\x87',
'CherokeeLetterSe': '\xe1\x8f\x8e',
'UnicodeCharacter0xe5ff': '\xee\x97\xbf',
'CjkUnifiedIdeograph76aa': '\xe7\x9a\xaa',
'HangulSyllableGgup': '\xea\xbf\x92',
'CjkUnifiedIdeograph76af': '\xe7\x9a\xaf',
'HangulSyllableGgus': '\xea\xbf\x8b',
'HangulSyllableGgut': '\xea\xbf\x91',
'UnicodeCharacter0xe5fa': '\xee\x97\xba',
'HangulSyllableGguh': '\xea\xbf\x93',
'HangulSyllableGguj': '\xea\xbf\x8e',
'HangulSyllableGguk': '\xea\xbf\x90',
'HangulSyllableGgul': '\xea\xbf\x80',
'CjkUnifiedIdeograph76ad': '\xe7\x9a\xad',
'HangulSyllableGgun': '\xea\xbe\xbc',
'YiRadicalMop': '\xea\x92\x9c',
'UnicodeCharacter0xb4e': '\xe0\xad\x8e',
'HangulSyllableGgub': '\xea\xbf\x89',
'ThaiCharacterYoYing': '\xe0\xb8\x8d',
'HangulSyllableGgud': '\xea\xbe\xbf',
'SquareBeeta': '\xe3\x8c\xbc',
'HangulSyllableGgug': '\xea\xbe\xb9',
'OriyaLetterBa': '\xe0\xac\xac',
'KangxiRadicalEmbroidery': '\xe2\xbf\x8b',
'HangulSyllableTan': '\xed\x83\x84',
'HangulSyllableTwaelt': '\xed\x87\xa5',
'HangulSyllableRweoh': '\xeb\xa4\x9f',
'CjkUnifiedIdeograph343b': '\xe3\x90\xbb',
'CjkUnifiedIdeograph343c': '\xe3\x90\xbc',
'Kai': '\xcf\x8f',
'CjkUnifiedIdeograph343a': '\xe3\x90\xba',
'CjkUnifiedIdeograph343f': '\xe3\x90\xbf',
'HangulSyllableRweok': '\xeb\xa4\x9c',
'CjkUnifiedIdeograph343d': '\xe3\x90\xbd',
'CjkUnifiedIdeograph343e': '\xe3\x90\xbe',
'HangulSyllableTulh': '\xed\x88\xbb',
'HangulSyllableRweoj': '\xeb\xa4\x9a',
'HangulSyllableWij': '\xec\x9c\x9a',
'UnicodeCharacter0xdf67': '\xed\xbd\xa7',
'HangulSyllableRweom': '\xeb\xa4\x94',
'HangulSyllableRweol': '\xeb\xa4\x8c',
'HangulSyllableSweop': '\xec\x89\x8e',
'HangulSyllableJjyogg': '\xec\xac\xae',
'HalfwidthKatakanaLetterWo': '\xef\xbd\xa6',
'HangulSyllableRweon': '\xeb\xa4\x88',
'HangulSyllableSweot': '\xec\x89\x8d',
'SupersetOfAboveEqualsSign': '\xe2\xab\x86',
'HangulSyllableTulm': '\xed\x88\xb6',
'HangulSyllableSweoh': '\xec\x89\x8f',
'CanadianSyllabicsThcreeTho': '\xe1\x96\xaa',
'HangulSyllableSweoj': '\xec\x89\x8a',
'HangulSyllableSweok': '\xec\x89\x8c',
'HangulSyllableSweol': '\xec\x88\xbc',
'HangulSyllableSweom': '\xec\x89\x84',
'HangulSyllableSweon': '\xec\x88\xb8',
'CanadianSyllabicsThcreeThi': '\xe1\x96\xa8',
'LatinSmallLetterEt': '\xea\x9d\xab',
'CanadianSyllabicsThcreeThe': '\xe1\x96\xa7',
'HangulSyllableSweod': '\xec\x88\xbb',
'HangulSyllableJjyogs': '\xec\xac\xaf',
'CjkUnifiedIdeograph70c7': '\xe7\x83\x87',
'CanadianSyllabicsThcreeTha': '\xe1\x96\xac',
'HangulSyllableTulp': '\xed\x88\xba',
'PhagspaLetterNga': '\xea\xa1\x83',
'HangulSyllableAlh': '\xec\x95\x93',
'ArabicLigatureShaddaWithKasraIsolatedForm': '\xef\xb1\xa2',
'UnicodeCharacter0xead4': '\xee\xab\x94',
'HangulSyllableYa': '\xec\x95\xbc',
'ArabicLigatureYehWithHahIsolatedForm': '\xef\xb1\x96',
'CjkUnifiedIdeograph3438': '\xe3\x90\xb8',
'CjkUnifiedIdeograph3439': '\xe3\x90\xb9',
'CjkUnifiedIdeograph5397': '\xe5\x8e\x97',
'CjkUnifiedIdeograph3432': '\xe3\x90\xb2',
'CjkUnifiedIdeograph3433': '\xe3\x90\xb3',
'CjkUnifiedIdeograph3430': '\xe3\x90\xb0',
'CjkUnifiedIdeograph3431': '\xe3\x90\xb1',
'CjkUnifiedIdeograph3436': '\xe3\x90\xb6',
'CjkUnifiedIdeograph3437': '\xe3\x90\xb7',
'CjkUnifiedIdeograph3434': '\xe3\x90\xb4',
'CjkUnifiedIdeograph3435': '\xe3\x90\xb5',
'RomanNumeralSeven': '\xe2\x85\xa6',
'UnicodeCharacter0xe5a6': '\xee\x96\xa6',
'HangulSyllableBulh': '\xeb\xb6\x8f',
'HangulSyllableBulg': '\xeb\xb6\x89',
'ThaiCharacterYoYak': '\xe0\xb8\xa2',
'UnicodeCharacter0xead1': '\xee\xab\x91',
'HangulSyllableBulb': '\xeb\xb6\x8b',
'CyrillicCapitalLetterKomiDje': '\xd4\x82',
'LatinSmallLetterSchwa': '\xc9\x99',
'HangulSyllableOep': '\xec\x9a\x92',
'HangulSyllableBult': '\xeb\xb6\x8d',
'HangulSyllableBuls': '\xeb\xb6\x8c',
'HangulSyllableBulp': '\xeb\xb6\x8e',
'HangulSyllableCih': '\xec\xb9\xb3',
'HangulSyllableJwe': '\xec\xa4\xb4',
'HangulSyllableSsugg': '\xec\x91\xa6',
'HangulSyllableCil': '\xec\xb9\xa0',
'CjkRadicalCsimplifiedEat': '\xe2\xbb\xa0',
'IdeographicTelegraphSymbolForDayTwentysix': '\xe3\x8f\xb9',
'UnicodeCharacter0xf867': '\xef\xa1\xa7',
'HangulSyllableCib': '\xec\xb9\xa9',
'HangulSyllableCic': '\xec\xb9\xaf',
'HangulSyllableCid': '\xec\xb9\x9f',
'HangulSyllableJwi': '\xec\xa5\x90',
'HangulSyllableCig': '\xec\xb9\x99',
'CjkUnifiedIdeograph8d82': '\xe8\xb6\x82',
'CjkUnifiedIdeograph8d83': '\xe8\xb6\x83',
'CjkUnifiedIdeograph8d80': '\xe8\xb6\x80',
'CjkUnifiedIdeograph8d81': '\xe8\xb6\x81',
'CjkUnifiedIdeograph8d86': '\xe8\xb6\x86',
'CjkUnifiedIdeograph8d87': '\xe8\xb6\x87',
'CjkUnifiedIdeograph8d84': '\xe8\xb6\x84',
'CjkUnifiedIdeograph8d85': '\xe8\xb6\x85',
'HangulSyllableCip': '\xec\xb9\xb2',
'CjkUnifiedIdeograph8d88': '\xe8\xb6\x88',
'CjkUnifiedIdeograph8d89': '\xe8\xb6\x89',
'HangulSyllableCit': '\xec\xb9\xb1',
'BalineseSignSurang': '\xe1\xac\x83',
'GreekSmallLetterOmegaWithTonos': '\xcf\x8e',
'LepchaLetterRa': '\xe1\xb0\x9b',
'UnicodeCharacter0xabff': '\xea\xaf\xbf',
'UnicodeCharacter0xf41c': '\xef\x90\x9c',
'TeluguDigitTwo': '\xe0\xb1\xa8',
'CjkUnifiedIdeograph8d8b': '\xe8\xb6\x8b',
'CjkUnifiedIdeograph8d8c': '\xe8\xb6\x8c',
'CjkUnifiedIdeograph8d8a': '\xe8\xb6\x8a',
'CjkUnifiedIdeograph8d8f': '\xe8\xb6\x8f',
'NorthWestArrowToCorner': '\xe2\x87\xb1',
'CjkUnifiedIdeograph8d8d': '\xe8\xb6\x8d',
'CjkUnifiedIdeograph8d8e': '\xe8\xb6\x8e',
'CjkUnifiedIdeograph9cb1': '\xe9\xb2\xb1',
'HangulSyllableGgeubs': '\xeb\x81\x96',
'HangulSyllableGgyogg': '\xea\xbe\x9e',
'HangulSyllableCyugg': '\xec\xb8\x86',
'HangulSyllableAek': '\xec\x95\xb8',
'LatinSmallLetterBWithTopbar': '\xc6\x83',
'GujaratiLetterJha': '\xe0\xaa\x9d',
'HangulSyllableCyugs': '\xec\xb8\x87',
'CjkUnifiedIdeograph418d': '\xe4\x86\x8d',
'!->>': '\xe2\x87\x8f',
'UnicodeCharacter0x3a2': '\xce\xa2',
'CjkUnifiedIdeograph599e': '\xe5\xa6\x9e',
'CjkUnifiedIdeograph599d': '\xe5\xa6\x9d',
'CjkUnifiedIdeograph599c': '\xe5\xa6\x9c',
'CjkUnifiedIdeograph599b': '\xe5\xa6\x9b',
'CjkUnifiedIdeograph599a': '\xe5\xa6\x9a',
'HangulSyllableDaels': '\xeb\x8c\x8c',
'CjkUnifiedIdeograph35ae': '\xe3\x96\xae',
'CjkUnifiedIdeograph35ad': '\xe3\x96\xad',
'CjkUnifiedIdeograph35af': '\xe3\x96\xaf',
'CjkUnifiedIdeograph35aa': '\xe3\x96\xaa',
'CjkUnifiedIdeograph35ac': '\xe3\x96\xac',
'CjkUnifiedIdeograph35ab': '\xe3\x96\xab',
'CjkUnifiedIdeograph97f0': '\xe9\x9f\xb0',
'CjkUnifiedIdeograph97f1': '\xe9\x9f\xb1',
'CjkUnifiedIdeograph97f2': '\xe9\x9f\xb2',
'CjkUnifiedIdeograph97f3': '\xe9\x9f\xb3',
'CjkUnifiedIdeograph97f4': '\xe9\x9f\xb4',
'CjkUnifiedIdeograph97f5': '\xe9\x9f\xb5',
'CjkUnifiedIdeograph97f6': '\xe9\x9f\xb6',
'CjkUnifiedIdeograph97f7': '\xe9\x9f\xb7',
'CjkUnifiedIdeograph97f8': '\xe9\x9f\xb8',
'LatinSmallLetterEzh': '\xca\x92',
'UnicodeCharacter0xf358': '\xef\x8d\x98',
'UnicodeCharacter0xf359': '\xef\x8d\x99',
'LeftwardsArrowWithPlusBelow': '\xe2\xa5\x86',
'OpencircuitoutputHtypeSymbol': '\xe2\x8e\x8f',
'KhmerSignTriisap': '\xe1\x9f\x8a',
'SylotiNagriLetterKho': '\xea\xa0\x88',
'UnicodeCharacter0xa6c4': '\xea\x9b\x84',
'CjkUnifiedIdeograph35a9': '\xe3\x96\xa9',
'CjkUnifiedIdeograph35a8': '\xe3\x96\xa8',
'HalfwidthKatakanahiraganaProlongedSoundMark': '\xef\xbd\xb0',
'CjkUnifiedIdeograph35a5': '\xe3\x96\xa5',
'CjkUnifiedIdeograph35a4': '\xe3\x96\xa4',
'CjkUnifiedIdeograph35a7': '\xe3\x96\xa7',
'CjkUnifiedIdeograph35a6': '\xe3\x96\xa6',
'CjkUnifiedIdeograph35a1': '\xe3\x96\xa1',
'CjkUnifiedIdeograph35a0': '\xe3\x96\xa0',
'CjkUnifiedIdeograph35a3': '\xe3\x96\xa3',
'CjkUnifiedIdeograph35a2': '\xe3\x96\xa2',
'CjkUnifiedIdeograph89f6': '\xe8\xa7\xb6',
'DoubleColonEqual': '\xe2\xa9\xb4',
'CjkUnifiedIdeograph5999': '\xe5\xa6\x99',
'CyrillicSmallLetterOWithDiaeresis': '\xd3\xa7',
'CjkUnifiedIdeograph5997': '\xe5\xa6\x97',
'CjkUnifiedIdeograph5996': '\xe5\xa6\x96',
'CjkUnifiedIdeograph5995': '\xe5\xa6\x95',
'CjkUnifiedIdeograph5994': '\xe5\xa6\x94',
'CjkUnifiedIdeograph5993': '\xe5\xa6\x93',
'CjkUnifiedIdeograph5992': '\xe5\xa6\x92',
'CjkUnifiedIdeograph5991': '\xe5\xa6\x91',
'CjkUnifiedIdeograph5990': '\xe5\xa6\x90',
'CjkUnifiedIdeograph6f90': '\xe6\xbe\x90',
'UnicodeCharacter0x109a': '\xe1\x82\x9a',
'UnicodeCharacter0x109b': '\xe1\x82\x9b',
'BuhidLetterU': '\xe1\x9d\x82',
'UnicodeCharacter0x109d': '\xe1\x82\x9d',
'CjkUnifiedIdeograph6f93': '\xe6\xbe\x93',
'HangulSyllableJjwit': '\xec\xae\xb5',
'UnicodeCharacter0xf35f': '\xef\x8d\x9f',
'CjkUnifiedIdeograph97fa': '\xe9\x9f\xba',
'CjkUnifiedIdeograph97fb': '\xe9\x9f\xbb',
'CjkUnifiedIdeograph6f92': '\xe6\xbe\x92',
'CjkUnifiedIdeograph97fd': '\xe9\x9f\xbd',
'CjkUnifiedIdeograph97fe': '\xe9\x9f\xbe',
'CjkUnifiedIdeograph97ff': '\xe9\x9f\xbf',
'BuhidLetterI': '\xe1\x9d\x81',
'ArabicLetterNoonGhunnaIsolatedForm': '\xef\xae\x9e',
'CjkUnifiedIdeograph6f95': '\xe6\xbe\x95',
'ThaiCharacterKhoKhai': '\xe0\xb8\x82',
'BuhidLetterA': '\xe1\x9d\x80',
'ModifierLetterTriangularColon': '\xcb\x90',
'CjkUnifiedIdeograph6f97': '\xe6\xbe\x97',
'CjkUnifiedIdeograph36af': '\xe3\x9a\xaf',
'CjkUnifiedIdeograph6ede': '\xe6\xbb\x9e',
'CjkUnifiedIdeograph36ad': '\xe3\x9a\xad',
'CjkUnifiedIdeograph36ae': '\xe3\x9a\xae',
'CjkUnifiedIdeograph36ab': '\xe3\x9a\xab',
'CjkUnifiedIdeograph36ac': '\xe3\x9a\xac',
'CjkUnifiedIdeograph36aa': '\xe3\x9a\xaa',
'CjkUnifiedIdeograph9e17': '\xe9\xb8\x97',
'CjkUnifiedIdeograph9e16': '\xe9\xb8\x96',
'CjkUnifiedIdeograph9e15': '\xe9\xb8\x95',
'CjkUnifiedIdeograph9e14': '\xe9\xb8\x94',
'CjkUnifiedIdeograph9e13': '\xe9\xb8\x93',
'SyriacLetterGamalGarshuni': '\xdc\x94',
'CjkUnifiedIdeograph9e11': '\xe9\xb8\x91',
'CjkUnifiedIdeograph9e10': '\xe9\xb8\x90',
'ArabicLigatureSeenWithKhahInitialForm': '\xef\xb2\xaf',
'CjkUnifiedIdeograph9e19': '\xe9\xb8\x99',
'CjkUnifiedIdeograph9e18': '\xe9\xb8\x98',
'CjkUnifiedIdeograph7849': '\xe7\xa1\x89',
'CjkUnifiedIdeograph7848': '\xe7\xa1\x88',
'CjkUnifiedIdeograph7847': '\xe7\xa1\x87',
'CjkUnifiedIdeograph7846': '\xe7\xa1\x86',
'CjkUnifiedIdeograph7845': '\xe7\xa1\x85',
'CjkUnifiedIdeograph7844': '\xe7\xa1\x84',
'CjkUnifiedIdeograph7843': '\xe7\xa1\x83',
'CjkUnifiedIdeograph7842': '\xe7\xa1\x82',
'CjkUnifiedIdeograph7841': '\xe7\xa1\x81',
'LatinCapitalLetterIWithBreve': '\xc4\xac',
'NewTaiLueLetterHighPa': '\xe1\xa6\x94',
'YiSyllableCax': '\xea\x8a\xb7',
'TagalogLetterLa': '\xe1\x9c\x8e',
'CjkUnifiedIdeograph9e1f': '\xe9\xb8\x9f',
'CjkUnifiedIdeograph9e1e': '\xe9\xb8\x9e',
'CjkUnifiedIdeograph9e1d': '\xe9\xb8\x9d',
'CjkUnifiedIdeograph9e1c': '\xe9\xb8\x9c',
'CjkUnifiedIdeograph9e1b': '\xe9\xb8\x9b',
'CjkUnifiedIdeograph9e1a': '\xe9\xb8\x9a',
'UnicodeCharacter0xf1ef': '\xef\x87\xaf',
'CjkUnifiedIdeograph9164': '\xe9\x85\xa4',
'CjkUnifiedIdeograph9165': '\xe9\x85\xa5',
'CjkUnifiedIdeograph36a8': '\xe3\x9a\xa8',
'CjkUnifiedIdeograph36a9': '\xe3\x9a\xa9',
'CjkUnifiedIdeograph36a6': '\xe3\x9a\xa6',
'CjkUnifiedIdeograph36a7': '\xe3\x9a\xa7',
'CjkUnifiedIdeograph36a4': '\xe3\x9a\xa4',
'CjkUnifiedIdeograph36a5': '\xe3\x9a\xa5',
'CjkUnifiedIdeograph36a2': '\xe3\x9a\xa2',
'CjkUnifiedIdeograph36a3': '\xe3\x9a\xa3',
'CjkUnifiedIdeograph36a0': '\xe3\x9a\xa0',
'CjkUnifiedIdeograph36a1': '\xe3\x9a\xa1',
'HangulSyllableCyilt': '\xec\xb9\x89',
'HangulSyllableCyils': '\xec\xb9\x88',
'HangulSyllableCyilp': '\xec\xb9\x8a',
'ThaiCharacterSaraAiMaimalai': '\xe0\xb9\x84',
'HangulSyllableCyilg': '\xec\xb9\x85',
'HangulSyllableCyilb': '\xec\xb9\x87',
'CjkUnifiedIdeograph784f': '\xe7\xa1\x8f',
'CjkUnifiedIdeograph784e': '\xe7\xa1\x8e',
'CjkUnifiedIdeograph784d': '\xe7\xa1\x8d',
'CjkUnifiedIdeograph784c': '\xe7\xa1\x8c',
'CjkUnifiedIdeograph784b': '\xe7\xa1\x8b',
'CjkUnifiedIdeograph784a': '\xe7\xa1\x8a',
'HangulSyllableCyilh': '\xec\xb9\x8b',
'YiSyllableDut': '\xea\x84\x94',
'HangulSyllableMya': '\xeb\xa8\x80',
'SaurashtraLetterLla': '\xea\xa2\xb3',
'UnicodeCharacter0xdab9': '\xed\xaa\xb9',
'HangulSyllableDdyep': '\xeb\x98\x8e',
'HangulSyllableGyeonj': '\xea\xb2\xad',
'HangulSyllableDdyet': '\xeb\x98\x8d',
'HangulSyllableDdyess': '\xeb\x98\x88',
'HangulSyllableDdyeh': '\xeb\x98\x8f',
'UnicodeCharacter0xdab7': '\xed\xaa\xb7',
'HangulSyllableDdyej': '\xeb\x98\x8a',
'HangulSyllableDdyek': '\xeb\x98\x8c',
'CjkUnifiedIdeograph9cb3': '\xe9\xb2\xb3',
'CjkUnifiedIdeograph9168': '\xe9\x85\xa8',
'HangulSyllableDdyen': '\xeb\x97\xb8',
'HangulSyllableDdyeo': '\xeb\x97\x98',
'HangulSyllableDdyeb': '\xeb\x98\x85',
'ReverseTildeOperatorAboveRightwardsArrow': '\xe2\xad\x87',
'HangulSyllableDdyed': '\xeb\x97\xbb',
'HangulSyllableGyeong': '\xea\xb2\xbd',
'HangulSyllableDdyeg': '\xeb\x97\xb5',
'UnicodeCharacter0xdab4': '\xed\xaa\xb4',
'YiSyllableDur': '\xea\x84\x99',
'CjkUnifiedIdeograph608a': '\xe6\x82\x8a',
'CjkUnifiedIdeograph608b': '\xe6\x82\x8b',
'CjkUnifiedIdeograph608c': '\xe6\x82\x8c',
'CjkUnifiedIdeograph608d': '\xe6\x82\x8d',
'CjkUnifiedIdeograph608e': '\xe6\x82\x8e',
'CjkUnifiedIdeograph608f': '\xe6\x82\x8f',
'UnicodeCharacter0xdab2': '\xed\xaa\xb2',
'YiSyllableZhyp': '\xea\x8d\x9e',
'UnicodeCharacter0xdab1': '\xed\xaa\xb1',
'UnicodeCharacter0xdab0': '\xed\xaa\xb0',
'CircledNumberThirtyOne': '\xe3\x89\x9b',
'CanadianSyllabicsCa': '\xe1\x92\x90',
'HangulLetterYoi': '\xe3\x86\x89',
'CanadianSyllabicsCe': '\xe1\x92\x89',
'CanadianSyllabicsCi': '\xe1\x92\x8b',
'CanadianSyllabicsCo': '\xe1\x92\x8d',
'DownTackWithCircleBelow': '\xe2\xab\xb1',
'LatinCapitalLetterMiddlewelshV': '\xe1\xbb\xbc',
'NegativeCircledDigitZero': '\xe2\x93\xbf',
'HangulSyllableSwilp': '\xec\x89\xba',
'HangulSyllableSwils': '\xec\x89\xb8',
'HangulSyllableSwilt': '\xec\x89\xb9',
'm': '\xce\xbc',
'YiSyllableZhyr': '\xea\x8d\xa0',
'HangulSyllableBbeolb': '\xeb\xbb\x9b',
'CjkUnifiedIdeograph6088': '\xe6\x82\x88',
'CjkUnifiedIdeograph6089': '\xe6\x82\x89',
'HangulSyllableSwilb': '\xec\x89\xb7',
'ParenthesizedNumberSixteen': '\xe2\x92\x83',
'HangulSyllableSwilg': '\xec\x89\xb5',
'CjkUnifiedIdeograph6080': '\xe6\x82\x80',
'CjkUnifiedIdeograph6081': '\xe6\x82\x81',
'CjkUnifiedIdeograph6082': '\xe6\x82\x82',
'CjkUnifiedIdeograph6083': '\xe6\x82\x83',
'CjkUnifiedIdeograph6084': '\xe6\x82\x84',
'CjkUnifiedIdeograph6085': '\xe6\x82\x85',
'CjkUnifiedIdeograph6086': '\xe6\x82\x86',
'CjkUnifiedIdeograph6087': '\xe6\x82\x87',
'CjkUnifiedIdeograph43b7': '\xe4\x8e\xb7',
'CjkUnifiedIdeograph43b6': '\xe4\x8e\xb6',
'CjkUnifiedIdeograph43b5': '\xe4\x8e\xb5',
'CjkUnifiedIdeograph43b4': '\xe4\x8e\xb4',
'CjkUnifiedIdeograph43b3': '\xe4\x8e\xb3',
'CjkUnifiedIdeograph43b2': '\xe4\x8e\xb2',
'CjkUnifiedIdeograph43b1': '\xe4\x8e\xb1',
'CjkUnifiedIdeograph43b0': '\xe4\x8e\xb0',
'LessthanOrEqualTo': '\xe2\x89\xa4',
'CjkUnifiedIdeograph6338': '\xe6\x8c\xb8',
'CjkUnifiedIdeograph6339': '\xe6\x8c\xb9',
'LowerRightPencil': '\xe2\x9c\x8e',
'CjkUnifiedIdeograph9509': '\xe9\x94\x89',
'CjkUnifiedIdeograph43b9': '\xe4\x8e\xb9',
'CjkUnifiedIdeograph43b8': '\xe4\x8e\xb8',
'FullwidthLatinSmallLetterS': '\xef\xbd\x93',
'TripleHorizontalBarWithTripleVerticalStroke': '\xe2\xa9\xa9',
'CjkUnifiedIdeograph55bf': '\xe5\x96\xbf',
'SundaneseLetterEu': '\xe1\xae\x89',
'HangulSyllableTabs': '\xed\x83\x92',
'HangulChoseongPieupsiospieup': '\xe1\x84\xa4',
'HangulSyllableBbeult': '\xec\x81\xa5',
'UnicodeCharacter0xe41c': '\xee\x90\x9c',
'MyanmarLetterShanCa': '\xe1\x81\xb8',
'HangulSyllableKwanh': '\xec\xbd\xb6',
'HangulSyllableGabs': '\xea\xb0\x92',
'CjkUnifiedIdeograph7944': '\xe7\xa5\x84',
'LatinCapitalLetterOWithAcute': '\xc3\x93',
'HangulSyllableKwang': '\xec\xbe\x85',
'CjkUnifiedIdeograph7945': '\xe7\xa5\x85',
'KangxiRadicalBamboo': '\xe2\xbd\xb5',
'GreekSmallLetterOmegaWithDasiaAndOxiaAndYpogegrammeni': '\xe1\xbe\xa5',
'AsymptoticallyEqualTo': '\xe2\x89\x83',
'counterclockwiseintegral': '\xe2\x88\xb2',
'YiSyllableLiet': '\xea\x86\xbb',
'CjkUnifiedIdeograph719f': '\xe7\x86\x9f',
'UnicodeCharacter0xee29': '\xee\xb8\xa9',
'TibetanAstrologicalSignKhyudPa': '\xe0\xbc\x98',
'HangulSyllableBagg': '\xeb\xb0\x96',
'CjkUnifiedIdeograph7941': '\xe7\xa5\x81',
'HangulSyllableTalp': '\xed\x83\x8e',
'CjkRadicalSpiritTwo': '\xe2\xba\xad',
'CjkUnifiedIdeograph43bf': '\xe4\x8e\xbf',
'CjkUnifiedIdeograph43be': '\xe4\x8e\xbe',
'CjkUnifiedIdeograph43bd': '\xe4\x8e\xbd',
'CjkUnifiedIdeograph43bc': '\xe4\x8e\xbc',
'CjkUnifiedIdeograph43bb': '\xe4\x8e\xbb',
'CopticCapitalLetterDialectpNi': '\xe2\xb2\xba',
'LaoVowelSignMaiKan': '\xe0\xba\xb1',
'YiSyllableHxe': '\xea\x89\x9b',
'CjkUnifiedIdeograph9df0': '\xe9\xb7\xb0',
'ArabicLigatureNoonWithJeemWithMeemInitialForm': '\xef\xb6\x98',
'ChamLetterA': '\xea\xa8\x80',
'UnicodeCharacter0xf07f': '\xef\x81\xbf',
'CircledNumberSixteen': '\xe2\x91\xaf',
'HangulSyllableNyu': '\xeb\x89\xb4',
'VaiSyllableHan': '\xea\x95\x8d',
'FullwidthLatinSmallLetterN': '\xef\xbd\x8e',
'HangulSyllableNya': '\xeb\x83\x90',
'UnicodeCharacter0xebd8': '\xee\xaf\x98',
'GeorgianCapitalLetterQar': '\xe1\x82\xb7',
'HangulSyllableNye': '\xeb\x85\x9c',
'CjkUnifiedIdeograph7948': '\xe7\xa5\x88',
'UnicodeCharacter0xee21': '\xee\xb8\xa1',
'HangulSyllableNyi': '\xeb\x8a\xac',
'CjkUnifiedIdeograph7949': '\xe7\xa5\x89',
'UnicodeCharacter0xee22': '\xee\xb8\xa2',
'HangulSyllableNyo': '\xeb\x87\xa8',
'UnicodeCharacter0xee23': '\xee\xb8\xa3',
'BalineseLetterLaLenga': '\xe1\xac\x8d',
'UnicodeCharacter0xec7f': '\xee\xb1\xbf',
'GreekCapitalLetterAlphaWithPsiliAndProsgegrammeni': '\xe1\xbe\x88',
'CjkUnifiedIdeograph3ab6': '\xe3\xaa\xb6',
'HebrewLetterLamedWithDagesh': '\xef\xac\xbc',
'GlagoliticSmallLetterIzhe': '\xe2\xb0\xb9',
'UnicodeCharacter0xe69c': '\xee\x9a\x9c',
'UnicodeCharacter0xda33': '\xed\xa8\xb3',
'CjkUnifiedIdeograph3771': '\xe3\x9d\xb1',
'CjkUnifiedIdeograph3770': '\xe3\x9d\xb0',
'CjkUnifiedIdeograph3773': '\xe3\x9d\xb3',
'CjkUnifiedIdeograph3772': '\xe3\x9d\xb2',
'CjkUnifiedIdeograph3775': '\xe3\x9d\xb5',
'CjkUnifiedIdeograph3774': '\xe3\x9d\xb4',
'CjkUnifiedIdeograph3777': '\xe3\x9d\xb7',
'CjkUnifiedIdeograph3776': '\xe3\x9d\xb6',
'CjkUnifiedIdeograph3779': '\xe3\x9d\xb9',
'CjkUnifiedIdeograph3778': '\xe3\x9d\xb8',
'LatinSmallLetterRWithTail': '\xc9\xbd',
'CjkUnifiedIdeograph7060': '\xe7\x81\xa0',
'CjkUnifiedIdeograph7063': '\xe7\x81\xa3',
'CjkUnifiedIdeograph7062': '\xe7\x81\xa2',
'CjkUnifiedIdeograph7065': '\xe7\x81\xa5',
'CjkUnifiedIdeograph7064': '\xe7\x81\xa4',
'CjkUnifiedIdeograph7067': '\xe7\x81\xa7',
'CjkUnifiedIdeograph7066': '\xe7\x81\xa6',
'CjkUnifiedIdeograph7069': '\xe7\x81\xa9',
'CjkUnifiedIdeograph7068': '\xe7\x81\xa8',
'CjkUnifiedIdeograph7be0': '\xe7\xaf\xa0',
'CjkUnifiedIdeograph7be1': '\xe7\xaf\xa1',
'CjkUnifiedIdeograph7be6': '\xe7\xaf\xa6',
'CjkUnifiedIdeograph7be7': '\xe7\xaf\xa7',
'CjkUnifiedIdeograph7be4': '\xe7\xaf\xa4',
'CjkUnifiedIdeograph7be5': '\xe7\xaf\xa5',
'CjkUnifiedIdeograph9cb5': '\xe9\xb2\xb5',
'HangulChoseongChieuchhieuh': '\xe1\x85\x93',
'YiSyllableNbut': '\xea\x82\x85',
'YiSyllableZhurx': '\xea\x8d\x99',
'CjkUnifiedIdeograph9291': '\xe9\x8a\x91',
'UnicodeCharacter0xdabc': '\xed\xaa\xbc',
'HangulSyllableByigg': '\xeb\xb8\xaa',
'CjkUnifiedIdeograph9292': '\xe9\x8a\x92',
'YiSyllableNbur': '\xea\x82\x8a',
'BraillePatternDots78': '\xe2\xa3\x80',
'CjkUnifiedIdeograph9293': '\xe9\x8a\x93',
'CjkUnifiedIdeograph6821': '\xe6\xa0\xa1',
'HangulSyllableNyeong': '\xeb\x85\x95',
'UnicodeCharacter0x167d': '\xe1\x99\xbd',
'YiSyllableNbup': '\xea\x82\x88',
'Koppa': '\xcf\x9e',
'CjkUnifiedIdeograph9295': '\xe9\x8a\x95',
'CjkUnifiedIdeograph706a': '\xe7\x81\xaa',
'CjkUnifiedIdeograph706c': '\xe7\x81\xac',
'CjkUnifiedIdeograph706b': '\xe7\x81\xab',
'CjkUnifiedIdeograph706e': '\xe7\x81\xae',
'EthiopicSyllableYaa': '\xe1\x8b\xab',
'CjkUnifiedIdeograph706f': '\xe7\x81\xaf',
'CjkUnifiedIdeograph7beb': '\xe7\xaf\xab',
'CjkUnifiedIdeograph7bec': '\xe7\xaf\xac',
'UnicodeCharacter0xdf00': '\xed\xbc\x80',
'CjkUnifiedIdeograph7bea': '\xe7\xaf\xaa',
'CjkUnifiedIdeograph7bef': '\xe7\xaf\xaf',
'UnicodeCharacter0xdf07': '\xed\xbc\x87',
'CjkUnifiedIdeograph7bed': '\xe7\xaf\xad',
'CjkUnifiedIdeograph7bee': '\xe7\xaf\xae',
'CjkUnifiedIdeograph377a': '\xe3\x9d\xba',
'Lozenge': '\xe2\x97\x8a',
'CjkUnifiedIdeograph377c': '\xe3\x9d\xbc',
'CjkUnifiedIdeograph377b': '\xe3\x9d\xbb',
'CjkUnifiedIdeograph377e': '\xe3\x9d\xbe',
'CjkUnifiedIdeograph377d': '\xe3\x9d\xbd',
'UnicodeCharacter0xe3bd': '\xee\x8e\xbd',
'CjkUnifiedIdeograph377f': '\xe3\x9d\xbf',
'CjkUnifiedIdeograph356c': '\xe3\x95\xac',
'MyanmarLetterSsa': '\xe1\x81\x91',
'FullwidthLatinSmallLetterK': '\xef\xbd\x8b',
'CanadianSyllabicsNaskapiSkwa': '\xe1\x94\x8e',
'UnicodeCharacter0xe51e': '\xee\x94\x9e',
'HangulSyllableMuls': '\xeb\xad\x80',
'HangulSyllableMulp': '\xeb\xad\x82',
'YiSyllableZep': '\xea\x8a\xa1',
'HangulSyllableMult': '\xeb\xad\x81',
'CjkUnifiedIdeograph62a5': '\xe6\x8a\xa5',
'CjkUnifiedIdeograph62a4': '\xe6\x8a\xa4',
'CjkUnifiedIdeograph62a7': '\xe6\x8a\xa7',
'CjkUnifiedIdeograph62a6': '\xe6\x8a\xa6',
'CjkUnifiedIdeograph62a1': '\xe6\x8a\xa1',
'CjkUnifiedIdeograph62a0': '\xe6\x8a\xa0',
'CjkUnifiedIdeograph62a3': '\xe6\x8a\xa3',
'CjkUnifiedIdeograph62a2': '\xe6\x8a\xa2',
'HangulSyllableMulb': '\xeb\xac\xbf',
'CjkUnifiedIdeograph62a9': '\xe6\x8a\xa9',
'CjkUnifiedIdeograph62a8': '\xe6\x8a\xa8',
'san': '\xcf\xbb',
'CjkUnifiedIdeograph3f9f': '\xe3\xbe\x9f',
'FullwidthLatinSmallLetterD': '\xef\xbd\x84',
'CjkUnifiedIdeograph3f9d': '\xe3\xbe\x9d',
'CjkUnifiedIdeograph3f9e': '\xe3\xbe\x9e',
'CjkUnifiedIdeograph3f9b': '\xe3\xbe\x9b',
'CjkUnifiedIdeograph3f9c': '\xe3\xbe\x9c',
'CjkUnifiedIdeograph3f9a': '\xe3\xbe\x9a',
'VaiSyllableShu': '\xea\x96\xad',
'UnicodeCharacter0xa7ce': '\xea\x9f\x8e',
'DoubledFemaleSign': '\xe2\x9a\xa2',
'SmallElementOfWithOverbar': '\xe2\x8b\xb7',
'HangulSyllableCe': '\xec\xb2\xb4',
'CjkUnifiedIdeograph685f': '\xe6\xa1\x9f',
'CjkUnifiedIdeograph685e': '\xe6\xa1\x9e',
'CjkUnifiedIdeograph4456': '\xe4\x91\x96',
'CjkUnifiedIdeograph685c': '\xe6\xa1\x9c',
'LatinCapitalLetterIWithHookAbove': '\xe1\xbb\x88',
'CjkUnifiedIdeograph685a': '\xe6\xa1\x9a',
'CjkUnifiedIdeograph7ef9': '\xe7\xbb\xb9',
'VaiSyllableShe': '\xea\x97\xbc',
'HammerAndSickle': '\xe2\x98\xad',
'VaiSyllableSha': '\xea\x95\xa3',
'HangulSyllableCu': '\xec\xb6\x94',
'CjkUnifiedIdeograph62ae': '\xe6\x8a\xae',
'CjkUnifiedIdeograph62ad': '\xe6\x8a\xad',
'CjkUnifiedIdeograph62af': '\xe6\x8a\xaf',
'CjkUnifiedIdeograph62aa': '\xe6\x8a\xaa',
'CjkUnifiedIdeograph62ac': '\xe6\x8a\xac',
'CjkUnifiedIdeograph62ab': '\xe6\x8a\xab',
'HangulSyllableHyess': '\xed\x98\xb0',
'DingbatNegativeCircledDigitSix': '\xe2\x9d\xbb',
'UnicodeCharacter0xe3bc': '\xee\x8e\xbc',
'CjkUnifiedIdeograph8ed4': '\xe8\xbb\x94',
'CjkUnifiedIdeograph9cb6': '\xe9\xb2\xb6',
'HangulSyllableByih': '\xeb\xb9\x83',
'CjkUnifiedIdeograph6857': '\xe6\xa1\x97',
'CjkUnifiedIdeograph6856': '\xe6\xa1\x96',
'CjkUnifiedIdeograph6855': '\xe6\xa1\x95',
'CjkUnifiedIdeograph6854': '\xe6\xa1\x94',
'CjkUnifiedIdeograph6853': '\xe6\xa1\x93',
'CjkUnifiedIdeograph6852': '\xe6\xa1\x92',
'CopticSmallLetterHate': '\xe2\xb2\x8f',
'CjkUnifiedIdeograph6850': '\xe6\xa1\x90',
'HangulSyllableSaebs': '\xec\x83\x9a',
'CjkUnifiedIdeograph794a': '\xe7\xa5\x8a',
'ArabicLigatureJeemWithMeemWithYehFinalForm': '\xef\xb6\xa5',
'CjkUnifiedIdeograph6859': '\xe6\xa1\x99',
'CjkUnifiedIdeograph6858': '\xe6\xa1\x98',
'CjkUnifiedIdeograph3f96': '\xe3\xbe\x96',
'CjkUnifiedIdeograph3f97': '\xe3\xbe\x97',
'CjkUnifiedIdeograph3f94': '\xe3\xbe\x94',
'CjkUnifiedIdeograph3f95': '\xe3\xbe\x95',
'CjkUnifiedIdeograph3f92': '\xe3\xbe\x92',
'CjkUnifiedIdeograph3f93': '\xe3\xbe\x93',
'CjkUnifiedIdeograph3f90': '\xe3\xbe\x90',
'CjkUnifiedIdeograph3f91': '\xe3\xbe\x91',
'HangulSyllablePaelt': '\xed\x8c\xb5',
'CjkUnifiedIdeograph3f98': '\xe3\xbe\x98',
'CjkUnifiedIdeograph3f99': '\xe3\xbe\x99',
'HalfwidthHangulLetterRieulthieuth': '\xef\xbe\xae',
'CjkUnifiedIdeograph7c39': '\xe7\xb0\xb9',
'CjkUnifiedIdeograph7c38': '\xe7\xb0\xb8',
'BoxDrawingsLightArcDownAndRight': '\xe2\x95\xad',
'CjkUnifiedIdeograph7c35': '\xe7\xb0\xb5',
'CjkUnifiedIdeograph5dae': '\xe5\xb6\xae',
'CjkUnifiedIdeograph7c37': '\xe7\xb0\xb7',
'CjkUnifiedIdeograph7c36': '\xe7\xb0\xb6',
'CjkUnifiedIdeograph7c31': '\xe7\xb0\xb1',
'CjkUnifiedIdeograph7c30': '\xe7\xb0\xb0',
'CjkUnifiedIdeograph7c33': '\xe7\xb0\xb3',
'CjkUnifiedIdeograph7c32': '\xe7\xb0\xb2',
'ArabicLigatureThehWithMeemInitialForm': '\xef\xb2\xa6',
'LeftSquareBracketUpperCorner': '\xe2\x8e\xa1',
'LogicalOrWithDoubleOverbar': '\xe2\xa9\xa2',
'UnicodeCharacter0xec75': '\xee\xb1\xb5',
'HangulSyllableJweng': '\xec\xa5\x89',
'UnicodeCharacter0xf7ac': '\xef\x9e\xac',
'KangxiRadicalClothes': '\xe2\xbe\x90',
'UnicodeCharacter0xec74': '\xee\xb1\xb4',
'CjkCompatibilityIdeographfacd': '\xef\xab\x8d',
'CjkCompatibilityIdeographface': '\xef\xab\x8e',
'CjkCompatibilityIdeographfacf': '\xef\xab\x8f',
'HangulSyllableMyod': '\xeb\xac\x9f',
'UnicodeCharacter0xdcaa': '\xed\xb2\xaa',
'CjkCompatibilityIdeographfacb': '\xef\xab\x8b',
'CjkCompatibilityIdeographfacc': '\xef\xab\x8c',
'CjkUnifiedIdeograph5daf': '\xe5\xb6\xaf',
'CjkUnifiedIdeograph61af': '\xe6\x86\xaf',
'UnicodeCharacter0xec79': '\xee\xb1\xb9',
'CjkUnifiedIdeograph61ad': '\xe6\x86\xad',
'CjkUnifiedIdeograph61ae': '\xe6\x86\xae',
'CjkUnifiedIdeograph61ab': '\xe6\x86\xab',
'CjkUnifiedIdeograph61ac': '\xe6\x86\xac',
'CjkUnifiedIdeograph61aa': '\xe6\x86\xaa',
'UnicodeCharacter0xe51c': '\xee\x94\x9c',
'LongRightwardsDoubleArrowFromBar': '\xe2\x9f\xbe',
'CjkUnifiedIdeograph3867': '\xe3\xa1\xa7',
'ParenthesizedDigitSix': '\xe2\x91\xb9',
'UnicodeCharacter0xf2eb': '\xef\x8b\xab',
'UnicodeCharacter0xeb61': '\xee\xad\xa1',
'CjkUnifiedIdeograph3866': '\xe3\xa1\xa6',
'PresentationFormForVerticalSemicolon': '\xef\xb8\x94',
'UnicodeCharacter0xf1c1': '\xef\x87\x81',
'BlackLeftpointingTriangle': '\xe2\x97\x80',
'CjkUnifiedIdeograph7c3e': '\xe7\xb0\xbe',
'CjkUnifiedIdeograph7c3d': '\xe7\xb0\xbd',
'CjkUnifiedIdeograph7c3f': '\xe7\xb0\xbf',
'CjkUnifiedIdeograph7c3a': '\xe7\xb0\xba',
'CjkUnifiedIdeograph89c0': '\xe8\xa7\x80',
'CjkUnifiedIdeograph7c3c': '\xe7\xb0\xbc',
'CjkUnifiedIdeograph7c3b': '\xe7\xb0\xbb',
'CjkUnifiedIdeograph61a6': '\xe6\x86\xa6',
'CjkUnifiedIdeograph55b2': '\xe5\x96\xb2',
'CjkUnifiedIdeograph61a4': '\xe6\x86\xa4',
'CjkUnifiedIdeograph61a5': '\xe6\x86\xa5',
'CjkUnifiedIdeograph61a2': '\xe6\x86\xa2',
'CjkUnifiedIdeograph61a3': '\xe6\x86\xa3',
'CjkUnifiedIdeograph61a0': '\xe6\x86\xa0',
'CjkUnifiedIdeograph5dad': '\xe5\xb6\xad',
'CjkUnifiedIdeograph61a8': '\xe6\x86\xa8',
'CjkUnifiedIdeograph61a9': '\xe6\x86\xa9',
'LatinCapitalLetterTz': '\xea\x9c\xa8',
'CjkCompatibilityIdeographfac5': '\xef\xab\x85',
'CjkCompatibilityIdeographfac6': '\xef\xab\x86',
'HebrewLetterHet': '\xd7\x97',
'CjkCompatibilityIdeographfac0': '\xef\xab\x80',
'CjkCompatibilityIdeographfac1': '\xef\xab\x81',
'CjkCompatibilityIdeographfac2': '\xef\xab\x82',
'CjkCompatibilityIdeographfac3': '\xef\xab\x83',
'ArabicLigatureSeenWithHahMedialForm': '\xef\xb4\xb5',
'CjkUnifiedIdeograph7199': '\xe7\x86\x99',
'HangulSyllableGgwiss': '\xeb\x80\xa0',
'CjkCompatibilityIdeographfac8': '\xef\xab\x88',
'CjkCompatibilityIdeographfac9': '\xef\xab\x89',
'UnicodeCharacter0xe752': '\xee\x9d\x92',
'CjkUnifiedIdeograph7198': '\xe7\x86\x98',
'UnicodeCharacter0xfaef': '\xef\xab\xaf',
'EthiopicSyllableGlottalOa': '\xe2\xb6\x8a',
'CjkUnifiedIdeograph55b4': '\xe5\x96\xb4',
'HeavyWedgetailedRightwardsArrow': '\xe2\x9e\xbd',
'HangulSyllableSswaegg': '\xec\x90\x92',
'RejangLetterDa': '\xea\xa4\xb4',
'ArabicDammaMedialForm': '\xef\xb9\xb9',
'CjkUnifiedIdeograph7e91': '\xe7\xba\x91',
'CjkUnifiedIdeograph7e90': '\xe7\xba\x90',
'GreekSmallLetterAlphaWithPerispomeni': '\xe1\xbe\xb6',
'CjkUnifiedIdeograph7e92': '\xe7\xba\x92',
'CjkUnifiedIdeograph7e95': '\xe7\xba\x95',
'CjkUnifiedIdeograph7e94': '\xe7\xba\x94',
'CjkUnifiedIdeograph7e97': '\xe7\xba\x97',
'HangulLetterRieulkiyeoksios': '\xe3\x85\xa9',
'CjkUnifiedIdeograph7e99': '\xe7\xba\x99',
'CjkUnifiedIdeograph7e98': '\xe7\xba\x98',
'UnicodeCharacter0xe751': '\xee\x9d\x91',
'CjkUnifiedIdeograph55b8': '\xe5\x96\xb8',
'DevanagariLetterGlottalStop': '\xe0\xa5\xbd',
'HangulSyllablePelm': '\xed\x8e\xa2',
'CjkUnifiedIdeograph7196': '\xe7\x86\x96',
'Aquarius': '\xe2\x99\x92',
'TamilLetterTa': '\xe0\xae\xa4',
'TagalogLetterWa': '\xe1\x9c\x8f',
'CjkUnifiedIdeograph4b9c': '\xe4\xae\x9c',
'CjkUnifiedIdeograph4b9b': '\xe4\xae\x9b',
'CjkUnifiedIdeograph4b9a': '\xe4\xae\x9a',
'CjkUnifiedIdeograph4b9f': '\xe4\xae\x9f',
'CjkUnifiedIdeograph4b9e': '\xe4\xae\x9e',
'CjkUnifiedIdeograph4b9d': '\xe4\xae\x9d',
'ArabicLetterThalIsolatedForm': '\xef\xba\xab',
'UnicodeCharacter0xe1bd': '\xee\x86\xbd',
'UnicodeCharacter0xd7f8': '\xed\x9f\xb8',
'CjkUnifiedIdeograph7e9a': '\xe7\xba\x9a',
'CjkUnifiedIdeograph7e9c': '\xe7\xba\x9c',
'CjkUnifiedIdeograph7e9b': '\xe7\xba\x9b',
'CjkUnifiedIdeograph7e9e': '\xe7\xba\x9e',
'CjkUnifiedIdeograph7e9d': '\xe7\xba\x9d',
'CjkUnifiedIdeograph7e9f': '\xe7\xba\x9f',
'ElectricalIntersection': '\xe2\x8f\xa7',
'GujaratiLetterSa': '\xe0\xaa\xb8',
'UnicodeCharacter0xe1be': '\xee\x86\xbe',
'HangulSyllableMeung': '\xeb\xaf\x95',
'HangulSyllableReugg': '\xeb\xa5\xb6',
'UnicodeCharacter0xda36': '\xed\xa8\xb6',
'CjkUnifiedIdeograph4b93': '\xe4\xae\x93',
'CjkUnifiedIdeograph4b92': '\xe4\xae\x92',
'DownpointingTriangleWithLeftHalfBlack': '\xe2\xa7\xa8',
'CjkUnifiedIdeograph4b90': '\xe4\xae\x90',
'CjkUnifiedIdeograph4b97': '\xe4\xae\x97',
'CjkUnifiedIdeograph4b96': '\xe4\xae\x96',
'CjkUnifiedIdeograph4b95': '\xe4\xae\x95',
'CjkUnifiedIdeograph4b94': '\xe4\xae\x94',
'UnicodeCharacter0xf019': '\xef\x80\x99',
'CjkUnifiedIdeograph4b99': '\xe4\xae\x99',
'CjkUnifiedIdeograph4b98': '\xe4\xae\x98',
'HangulSyllableHyaeng': '\xed\x97\x81',
'CjkUnifiedIdeograph89c1': '\xe8\xa7\x81',
'UnicodeCharacter0xe1bf': '\xee\x86\xbf',
'CjkUnifiedIdeograph5daa': '\xe5\xb6\xaa',
'SyriacLetterNun': '\xdc\xa2',
'RunicLetterIwazEoh': '\xe1\x9b\x87',
'TopArcAnticlockwiseArrow': '\xe2\xa4\xba',
'KangxiRadicalMinister': '\xe2\xbe\x82',
'UnicodeCharacter0xe08c': '\xee\x82\x8c',
'UnicodeCharacter0xe08b': '\xee\x82\x8b',
'UnicodeCharacter0xe08a': '\xee\x82\x8a',
'CjkUnifiedIdeograph980a': '\xe9\xa0\x8a',
'UnicodeCharacter0xe08f': '\xee\x82\x8f',
'UnicodeCharacter0xe08e': '\xee\x82\x8e',
'UnicodeCharacter0xe08d': '\xee\x82\x8d',
'BengaliDigitTwo': '\xe0\xa7\xa8',
'CjkUnifiedIdeograph50e9': '\xe5\x83\xa9',
'CjkUnifiedIdeograph50e6': '\xe5\x83\xa6',
'CjkUnifiedIdeograph50e7': '\xe5\x83\xa7',
'CjkUnifiedIdeograph50e4': '\xe5\x83\xa4',
'CjkUnifiedIdeograph50e5': '\xe5\x83\xa5',
'CjkUnifiedIdeograph50e2': '\xe5\x83\xa2',
'CjkUnifiedIdeograph50e3': '\xe5\x83\xa3',
'CjkUnifiedIdeograph50e0': '\xe5\x83\xa0',
'CjkUnifiedIdeograph50e1': '\xe5\x83\xa1',
'CjkUnifiedIdeograph5a8d': '\xe5\xaa\x8d',
'CjkUnifiedIdeograph4d3f': '\xe4\xb4\xbf',
'CjkUnifiedIdeograph4d3e': '\xe4\xb4\xbe',
'CjkUnifiedIdeograph4d3d': '\xe4\xb4\xbd',
'CjkUnifiedIdeograph4d3c': '\xe4\xb4\xbc',
'CjkUnifiedIdeograph4d3b': '\xe4\xb4\xbb',
'CjkUnifiedIdeograph4d3a': '\xe4\xb4\xba',
'CjkUnifiedIdeograph5a8c': '\xe5\xaa\x8c',
'HangulSyllableGgyub': '\xeb\x80\xb9',
'HangulSyllableGgyuc': '\xeb\x80\xbf',
'HangulSyllableGgyug': '\xeb\x80\xa9',
'HangulSyllableGgyud': '\xeb\x80\xaf',
'UnicodeCharacter0xf19d': '\xef\x86\x9d',
'UnicodeCharacter0xf19e': '\xef\x86\x9e',
'UnicodeCharacter0xf19f': '\xef\x86\x9f',
'LatinCapitalLetterAWithCircumflexAndTilde': '\xe1\xba\xaa',
'UnicodeCharacter0xf19a': '\xef\x86\x9a',
'UnicodeCharacter0xf19b': '\xef\x86\x9b',
'UnicodeCharacter0xf19c': '\xef\x86\x9c',
'UnicodeCharacter0xdabe': '\xed\xaa\xbe',
'HangulSyllableGgyus': '\xeb\x80\xbb',
'TagbanwaLetterPa': '\xe1\x9d\xa9',
'HangulSyllableGgyut': '\xeb\x81\x81',
'UnicodeCharacter0xe58c': '\xee\x96\x8c',
'HangulSyllableJjyil': '\xec\xaf\xb8',
'UnicodeCharacter0xa979': '\xea\xa5\xb9',
'UnicodeCharacter0xa978': '\xea\xa5\xb8',
'TaiLeLetterSa': '\xe1\xa5\x94',
'CjkUnifiedIdeograph6c3c': '\xe6\xb0\xbc',
'CjkUnifiedIdeograph50ef': '\xe5\x83\xaf',
'CyrillicSmallLetterUWithDiaeresis': '\xd3\xb1',
'CjkUnifiedIdeograph50ed': '\xe5\x83\xad',
'CjkUnifiedIdeograph50ee': '\xe5\x83\xae',
'CjkUnifiedIdeograph50eb': '\xe5\x83\xab',
'CjkUnifiedIdeograph50ec': '\xe5\x83\xac',
'UnicodeCharacter0xa975': '\xea\xa5\xb5',
'CjkUnifiedIdeograph50ea': '\xe5\x83\xaa',
'UnicodeCharacter0xe1ba': '\xee\x86\xba',
'BuhidLetterMa': '\xe1\x9d\x8b',
'HangulSyllableKyung': '\xed\x81\xa5',
'UnicodeCharacter0xe088': '\xee\x82\x88',
'UnicodeCharacter0xdabd': '\xed\xaa\xbd',
'HangulSyllableBbyugg': '\xec\x80\xbe',
'UnicodeCharacter0xe083': '\xee\x82\x83',
'HangulSyllableJjyih': '\xec\xb0\x8b',
'UnicodeCharacter0xe081': '\xee\x82\x81',
'UnicodeCharacter0xe080': '\xee\x82\x80',
'YiSyllableSuop': '\xea\x8c\x96',
'HangulSyllableKyunh': '\xed\x81\x96',
'UnicodeCharacter0xe085': '\xee\x82\x85',
'HangulSyllableKyunj': '\xed\x81\x95',
'UnicodeCharacter0xf194': '\xef\x86\x94',
'UnicodeCharacter0xf195': '\xef\x86\x95',
'UnicodeCharacter0xf196': '\xef\x86\x96',
'UnicodeCharacter0xf197': '\xef\x86\x97',
'UnicodeCharacter0xf190': '\xef\x86\x90',
'HangulSyllableJjyij': '\xec\xb0\x86',
'UnicodeCharacter0xf192': '\xef\x86\x92',
'UnicodeCharacter0xf193': '\xef\x86\x93',
'BoxDrawingsDownSingleAndLeftDouble': '\xe2\x95\x95',
'UnicodeCharacter0xf198': '\xef\x86\x98',
'UnicodeCharacter0xf199': '\xef\x86\x99',
'UnicodeCharacter0xe1bb': '\xee\x86\xbb',
'CjkUnifiedIdeograph4d37': '\xe4\xb4\xb7',
'CjkUnifiedIdeograph4d36': '\xe4\xb4\xb6',
'CjkUnifiedIdeograph4d35': '\xe4\xb4\xb5',
'CjkUnifiedIdeograph4d34': '\xe4\xb4\xb4',
'CjkUnifiedIdeograph4d33': '\xe4\xb4\xb3',
'CjkUnifiedIdeograph4d32': '\xe4\xb4\xb2',
'CjkUnifiedIdeograph4d31': '\xe4\xb4\xb1',
'CjkUnifiedIdeograph4d30': '\xe4\xb4\xb0',
'EqualsWithAsterisk': '\xe2\xa9\xae',
'UnicodeCharacter0xf0a0': '\xef\x82\xa0',
'UnicodeCharacter0xf1c7': '\xef\x87\x87',
'CjkUnifiedIdeograph5a88': '\xe5\xaa\x88',
'CjkUnifiedIdeograph5a89': '\xe5\xaa\x89',
'CjkUnifiedIdeograph4d39': '\xe4\xb4\xb9',
'UpperRightOrLowerLeftCurlyBracketSection': '\xe2\x8e\xb1',
'ArmenianCapitalLetterVo': '\xd5\x88',
'HangulSyllablePyelb': '\xed\x8f\x9b',
'HangulSyllableJjyig': '\xec\xaf\xb1',
'NewTaiLueVowelSignIi': '\xe1\xa6\xb2',
'BraillePatternDots458': '\xe2\xa2\x98',
'BraillePatternDots457': '\xe2\xa1\x98',
'BraillePatternDots456': '\xe2\xa0\xb8',
'UnicodeCharacter0xe1bc': '\xee\x86\xbc',
'CjkUnifiedIdeograph886a': '\xe8\xa1\xaa',
'HangulSyllablePyelp': '\xed\x8f\x9e',
'UnicodeCharacter0xdabb': '\xed\xaa\xbb',
'HangulSyllablePyels': '\xed\x8f\x9c',
'NewTaiLueVowelSignIy': '\xe1\xa7\x80',
'HangulSyllableJjyib': '\xec\xb0\x81',
'CjkUnifiedIdeograph8909': '\xe8\xa4\x89',
'CjkUnifiedIdeograph8908': '\xe8\xa4\x88',
'CjkUnifiedIdeograph8907': '\xe8\xa4\x87',
'CjkUnifiedIdeograph8906': '\xe8\xa4\x86',
'CjkUnifiedIdeograph8905': '\xe8\xa4\x85',
'CjkUnifiedIdeograph8904': '\xe8\xa4\x84',
'CjkUnifiedIdeograph8903': '\xe8\xa4\x83',
'CjkUnifiedIdeograph8902': '\xe8\xa4\x82',
'CjkUnifiedIdeograph8901': '\xe8\xa4\x81',
'CjkUnifiedIdeograph8900': '\xe8\xa4\x80',
'LatinSmallLetterRWithInvertedBreve': '\xc8\x93',
'HangulSyllableMass': '\xeb\xa7\x9c',
'LatinSmallLetterKWithLineBelow': '\xe1\xb8\xb5',
'ArabicLetterZahFinalForm': '\xef\xbb\x86',
'UnicodeCharacter0xe758': '\xee\x9d\x98',
'MediumLeftParenthesisOrnament': '\xe2\x9d\xa8',
'HangulSyllableRugs': '\xeb\xa3\xab',
'UnicodeCharacter0xf1cf': '\xef\x87\x8f',
'CjkUnifiedIdeograph5b38': '\xe5\xac\xb8',
'CjkUnifiedIdeograph5b39': '\xe5\xac\xb9',
'CjkUnifiedIdeograph5b36': '\xe5\xac\xb6',
'CjkUnifiedIdeograph51af': '\xe5\x86\xaf',
'CjkUnifiedIdeograph51ae': '\xe5\x86\xae',
'CjkUnifiedIdeograph51ad': '\xe5\x86\xad',
'CjkUnifiedIdeograph51ac': '\xe5\x86\xac',
'CjkUnifiedIdeograph51ab': '\xe5\x86\xab',
'CjkUnifiedIdeograph51aa': '\xe5\x86\xaa',
'CjkUnifiedIdeograph5b31': '\xe5\xac\xb1',
'CombiningTildeBelow': '\xcc\xb0',
'HangulSyllableSsyolm': '\xec\x91\x92',
'RejangLetterNgga': '\xea\xa5\x83',
'CjkUnifiedIdeograph8a59': '\xe8\xa9\x99',
'CjkUnifiedIdeograph890f': '\xe8\xa4\x8f',
'CjkUnifiedIdeograph890e': '\xe8\xa4\x8e',
'CjkUnifiedIdeograph890d': '\xe8\xa4\x8d',
'CjkUnifiedIdeograph890c': '\xe8\xa4\x8c',
'CjkUnifiedIdeograph890b': '\xe8\xa4\x8b',
'CjkUnifiedIdeograph890a': '\xe8\xa4\x8a',
'HangulSyllableJjyit': '\xec\xb0\x89',
'UnicodeCharacter0xda4f': '\xed\xa9\x8f',
'UnicodeCharacter0xf86c': '\xef\xa1\xac',
'HangulSyllableToebs': '\xed\x88\x86',
'UnicodeCharacter0xebf0': '\xee\xaf\xb0',
'UnicodeCharacter0xe931': '\xee\xa4\xb1',
'CircledDotOperator': '\xe2\x8a\x99',
'UnicodeCharacter0xe930': '\xee\xa4\xb0',
'CjkUnifiedIdeograph51a9': '\xe5\x86\xa9',
'CjkUnifiedIdeograph51a8': '\xe5\x86\xa8',
'CjkUnifiedIdeograph51a7': '\xe5\x86\xa7',
'CjkUnifiedIdeograph51a6': '\xe5\x86\xa6',
'CjkUnifiedIdeograph51a5': '\xe5\x86\xa5',
'CjkUnifiedIdeograph51a4': '\xe5\x86\xa4',
'CjkUnifiedIdeograph51a3': '\xe5\x86\xa3',
'CjkUnifiedIdeograph51a2': '\xe5\x86\xa2',
'CjkUnifiedIdeograph51a1': '\xe5\x86\xa1',
'CjkUnifiedIdeograph51a0': '\xe5\x86\xa0',
'HangulSyllableJjyeoh': '\xec\xaa\x9f',
'HangulSyllableJjyeoj': '\xec\xaa\x9a',
'HangulSyllableJjyeok': '\xec\xaa\x9c',
'HangulSyllableJjyeol': '\xec\xaa\x8c',
'HangulSyllableJjyeom': '\xec\xaa\x94',
'HangulSyllableJjyeon': '\xec\xaa\x88',
'HangulSyllableJjyeob': '\xec\xaa\x95',
'UnicodeCharacter0xa8ca': '\xea\xa3\x8a',
'HangulSyllableJjyeod': '\xec\xaa\x8b',
'UnicodeCharacter0xe068': '\xee\x81\xa8',
'HangulSyllableJjyeog': '\xec\xaa\x85',
'HangulSyllableHyalg': '\xed\x96\x99',
'ParenthesizedHangulIeung': '\xe3\x88\x87',
'CjkUnifiedIdeograph8a54': '\xe8\xa9\x94',
'UnicodeCharacter0xda38': '\xed\xa8\xb8',
'CjkUnifiedIdeograph410c': '\xe4\x84\x8c',
'CjkUnifiedIdeograph410b': '\xe4\x84\x8b',
'CjkUnifiedIdeograph410a': '\xe4\x84\x8a',
'CjkUnifiedIdeograph3ed5': '\xe3\xbb\x95',
'CjkUnifiedIdeograph3ed2': '\xe3\xbb\x92',
'CjkUnifiedIdeograph410f': '\xe4\x84\x8f',
'CjkUnifiedIdeograph410e': '\xe4\x84\x8e',
'CjkUnifiedIdeograph410d': '\xe4\x84\x8d',
'CjkUnifiedIdeograph3ed8': '\xe3\xbb\x98',
'CjkUnifiedIdeograph3ed9': '\xe3\xbb\x99',
'KannadaLetterTta': '\xe0\xb2\x9f',
'KannadaLetterKha': '\xe0\xb2\x96',
'TibetanMarkRgyaGramShad': '\xe0\xbc\x92',
'HangulSyllablePelb': '\xed\x8e\xa3',
'UnicodeCharacter0xe0df': '\xee\x83\x9f',
'UnicodeCharacter0xf41f': '\xef\x90\x9f',
'LatinSmallLetterIWithBreve': '\xc4\xad',
'CjkUnifiedIdeograph75be': '\xe7\x96\xbe',
'CjkUnifiedIdeograph75bf': '\xe7\x96\xbf',
'UpperLeftCornerWithDot': '\xe2\x9f\x94',
'UnicodeCharacter0xec7a': '\xee\xb1\xba',
'CjkUnifiedIdeograph75ba': '\xe7\x96\xba',
'CjkUnifiedIdeograph75bb': '\xe7\x96\xbb',
'CjkUnifiedIdeograph75bc': '\xe7\x96\xbc',
'CjkUnifiedIdeograph4103': '\xe4\x84\x83',
'CjkUnifiedIdeograph4102': '\xe4\x84\x82',
'CjkUnifiedIdeograph4101': '\xe4\x84\x81',
'CjkUnifiedIdeograph4100': '\xe4\x84\x80',
'CjkUnifiedIdeograph4107': '\xe4\x84\x87',
'CjkUnifiedIdeograph4106': '\xe4\x84\x86',
'CjkUnifiedIdeograph4105': '\xe4\x84\x85',
'CjkUnifiedIdeograph4104': '\xe4\x84\x84',
'HangulSyllableMaj': '\xeb\xa7\x9e',
'HangulSyllableMak': '\xeb\xa7\xa0',
'CjkUnifiedIdeograph4109': '\xe4\x84\x89',
'CjkUnifiedIdeograph4108': '\xe4\x84\x88',
'HangulSyllableMan': '\xeb\xa7\x8c',
'UnicodeCharacter0xde12': '\xed\xb8\x92',
'HangulSyllableMal': '\xeb\xa7\x90',
'HangulSyllableMam': '\xeb\xa7\x98',
'HangulSyllableMas': '\xeb\xa7\x9b',
'HangulSyllableMap': '\xeb\xa7\xa2',
'HangulSyllableMat': '\xeb\xa7\xa1',
'SundaneseLetterKha': '\xe1\xae\xae',
'GreekSmallLetterUpsilonWithDasiaAndPerispomeni': '\xe1\xbd\x97',
'UnicodeCharacter0xe508': '\xee\x94\x88',
'ArabicLigatureAinWithMeemWithAlefMaksuraFinalForm': '\xef\xb5\xb8',
'HangulSyllablePoj': '\xed\x90\x82',
'CjkUnifiedIdeograph7a6f': '\xe7\xa9\xaf',
'UnicodeCharacter0xe431': '\xee\x90\xb1',
'CjkUnifiedIdeograph75b8': '\xe7\x96\xb8',
'CjkUnifiedIdeograph75b9': '\xe7\x96\xb9',
'ArchaicSampi': '\xcd\xb2',
'CjkUnifiedIdeograph75b4': '\xe7\x96\xb4',
'CjkUnifiedIdeograph75b5': '\xe7\x96\xb5',
'CjkUnifiedIdeograph75b6': '\xe7\x96\xb6',
'CjkUnifiedIdeograph75b7': '\xe7\x96\xb7',
'BoxDrawingsLightUpAndRight': '\xe2\x94\x94',
'CjkUnifiedIdeograph75b1': '\xe7\x96\xb1',
'CjkUnifiedIdeograph75b2': '\xe7\x96\xb2',
'CjkUnifiedIdeograph75b3': '\xe7\x96\xb3',
'UnicodeCharacter0xe0dd': '\xee\x83\x9d',
'HangulSyllableHyals': '\xed\x96\x9c',
'UnicodeCharacter0xdcae': '\xed\xb2\xae',
'CjkUnifiedIdeograph5ca1': '\xe5\xb2\xa1',
'CjkUnifiedIdeograph5ca0': '\xe5\xb2\xa0',
'CjkUnifiedIdeograph5ca3': '\xe5\xb2\xa3',
'CjkUnifiedIdeograph5ca2': '\xe5\xb2\xa2',
'CjkUnifiedIdeograph5ca5': '\xe5\xb2\xa5',
'CjkUnifiedIdeograph5ca4': '\xe5\xb2\xa4',
'CjkUnifiedIdeograph5ca7': '\xe5\xb2\xa7',
'CjkUnifiedIdeograph5ca6': '\xe5\xb2\xa6',
'CjkUnifiedIdeograph5ca9': '\xe5\xb2\xa9',
'CjkUnifiedIdeograph5ca8': '\xe5\xb2\xa8',
'IdeographicAnnotationHeavenMark': '\xe3\x86\x9d',
'IdeographicTelegraphSymbolForHourEight': '\xe3\x8d\xa0',
'HangulSyllableHyaegg': '\xed\x96\xae',
'DevanagariDigitNine': '\xe0\xa5\xaf',
'LatinSmallLetterReversedOpenEWithRetroflexHook': '\xe1\xb6\x94',
'EthiopicSyllableZu': '\xe1\x8b\x99',
'CjkUnifiedIdeograph5e14': '\xe5\xb8\x94',
'CjkUnifiedIdeograph9835': '\xe9\xa0\xb5',
'UnicodeCharacter0xe0db': '\xee\x83\x9b',
'HangulSyllablePoelp': '\xed\x91\x8e',
'varpi': '\xcf\x96',
'LaoHoMo': '\xe0\xbb\x9d',
'HangulSyllableMoess': '\xeb\xac\x90',
'EthiopicSyllableZe': '\xe1\x8b\x9d',
'MongolianLetterManchuAliGaliBha': '\xe1\xa2\xa8',
'EthiopicSyllableZa': '\xe1\x8b\x98',
'UnicodeCharacter0xe509': '\xee\x94\x89',
'EthiopicSyllableZo': '\xe1\x8b\x9e',
'UnicodeCharacter0xe507': '\xee\x94\x87',
'UnicodeCharacter0xe504': '\xee\x94\x84',
'ArabicLetterQaf': '\xd9\x82',
'UnicodeCharacter0xe502': '\xee\x94\x82',
'CircledNumberThirtyThree': '\xe3\x89\x9d',
'EthiopicSyllableZi': '\xe1\x8b\x9a',
'UnicodeCharacter0xe501': '\xee\x94\x81',
'HangulSyllablePoelt': '\xed\x91\x8d',
'VaiSyllableEe': '\xea\x94\x80',
'UnicodeCharacter0xe0da': '\xee\x83\x9a',
'CjkUnifiedIdeograph9833': '\xe9\xa0\xb3',
'VaiSyllableEn': '\xea\x97\xa2',
'BoxDrawingsDoubleVerticalAndRight': '\xe2\x95\xa0',
'HangulSyllableSsyolt': '\xec\x91\x95',
'CjkUnifiedIdeograph5caa': '\xe5\xb2\xaa',
'CjkUnifiedIdeograph5cac': '\xe5\xb2\xac',
'CjkUnifiedIdeograph5cab': '\xe5\xb2\xab',
'CjkUnifiedIdeograph5cae': '\xe5\xb2\xae',
'CjkUnifiedIdeograph5cad': '\xe5\xb2\xad',
'CjkUnifiedIdeograph5caf': '\xe5\xb2\xaf',
'HangulSyllablePyoss': '\xed\x91\xb0',
'UnicodeCharacter0xeb73': '\xee\xad\xb3',
'CyrillicSmallLetterKomiNje': '\xd4\x8b',
'UnicodeCharacter0xde13': '\xed\xb8\x93',
'UnicodeCharacter0xe50f': '\xee\x94\x8f',
'UnicodeCharacter0xe4ad': '\xee\x92\xad',
'HangulSyllableNulh': '\xeb\x88\x93',
'UnicodeCharacter0xe50e': '\xee\x94\x8e',
'UnicodeCharacter0xe50b': '\xee\x94\x8b',
'UnicodeCharacter0xe50c': '\xee\x94\x8c',
'UnicodeCharacter0xe4ae': '\xee\x92\xae',
'UnicodeCharacter0xa8c6': '\xea\xa3\x86',
'UnicodeCharacter0xf05c': '\xef\x81\x9c',
'UnicodeCharacter0xa8c7': '\xea\xa3\x87',
'UnicodeCharacter0xe93a': '\xee\xa4\xba',
'CjkCompatibilityIdeographfa52': '\xef\xa9\x92',
'HangulSyllableRwess': '\xeb\xa4\xb4',
'UnicodeCharacter0xa8c5': '\xea\xa3\x85',
'TibetanSignRdelNagRdelDkar': '\xe0\xbf\x8e',
'UnicodeCharacter0xfbc2': '\xef\xaf\x82',
'HangulSyllableHeong': '\xed\x97\x9d',
'UnicodeCharacter0xe93f': '\xee\xa4\xbf',
'UnicodeCharacter0xdcaf': '\xed\xb2\xaf',
'UnicodeCharacter0xe93e': '\xee\xa4\xbe',
'HangulLetterChieuch': '\xe3\x85\x8a',
'SuperscriptSix': '\xe2\x81\xb6',
'CjkCompatibilityIdeographfa57': '\xef\xa9\x97',
'CjkUnifiedIdeograph6c32': '\xe6\xb0\xb2',
'HangulSyllableCaegg': '\xec\xb1\x86',
'HexagramForLimitation': '\xe4\xb7\xbb',
'HangulSyllableCaegs': '\xec\xb1\x87',
'GurmukhiVowelSignIi': '\xe0\xa9\x80',
'CjkUnifiedIdeograph357f': '\xe3\x95\xbf',
'CjkUnifiedIdeograph357e': '\xe3\x95\xbe',
'CjkUnifiedIdeograph357d': '\xe3\x95\xbd',
'CjkUnifiedIdeograph357c': '\xe3\x95\xbc',
'CjkUnifiedIdeograph357b': '\xe3\x95\xbb',
'CjkUnifiedIdeograph357a': '\xe3\x95\xba',
'CyrillicCapitalLetterIzhitsa': '\xd1\xb4',
'CjkCompatibilityIdeographfa59': '\xef\xa9\x99',
'CjkUnifiedIdeograph7b29': '\xe7\xac\xa9',
'HeavyChevronSnowflake': '\xe2\x9d\x86',
'CjkUnifiedIdeograph57cb': '\xe5\x9f\x8b',
'HangulSyllableAegs': '\xec\x95\xa3',
'CjkUnifiedIdeograph7b25': '\xe7\xac\xa5',
'AplFunctionalSymbolUpCaretTilde': '\xe2\x8d\xb2',
'HangulSyllableReuls': '\xeb\xa6\x80',
'YiSyllableShyx': '\xea\x8f\x81',
'CjkUnifiedIdeograph7b24': '\xe7\xac\xa4',
'CjkUnifiedIdeograph3579': '\xe3\x95\xb9',
'CjkUnifiedIdeograph3578': '\xe3\x95\xb8',
'CjkUnifiedIdeograph3577': '\xe3\x95\xb7',
'CjkUnifiedIdeograph3576': '\xe3\x95\xb6',
'CjkUnifiedIdeograph3575': '\xe3\x95\xb5',
'EthiopicSyllableGwee': '\xe1\x8c\x94',
'CjkUnifiedIdeograph3573': '\xe3\x95\xb3',
'CjkUnifiedIdeograph3572': '\xe3\x95\xb2',
'CjkUnifiedIdeograph3571': '\xe3\x95\xb1',
'CjkUnifiedIdeograph3570': '\xe3\x95\xb0',
'HangulSyllableReulb': '\xeb\xa5\xbf',
'UnicodeCharacter0xee96': '\xee\xba\x96',
'CyrillicSmallLetterIWithMacron': '\xd3\xa3',
'HangulSyllableReulh': '\xeb\xa6\x83',
'CjkUnifiedIdeograph9f73': '\xe9\xbd\xb3',
'HangulSyllableReulm': '\xeb\xa5\xbe',
'CjkUnifiedIdeograph492e': '\xe4\xa4\xae',
'CjkUnifiedIdeograph4c4e': '\xe4\xb1\x8e',
'CjkUnifiedIdeograph492f': '\xe4\xa4\xaf',
'CjkUnifiedIdeograph492a': '\xe4\xa4\xaa',
'CjkUnifiedIdeograph492c': '\xe4\xa4\xac',
'LessthanOrSlantedEqualTo': '\xe2\xa9\xbd',
'UnicodeCharacter0x19ae': '\xe1\xa6\xae',
'UnicodeCharacter0x19ad': '\xe1\xa6\xad',
'UnicodeCharacter0x19af': '\xe1\xa6\xaf',
'UnicodeCharacter0x19aa': '\xe1\xa6\xaa',
'GreekSmallLetterEpsilon': '\xce\xb5',
'UnicodeCharacter0x19ac': '\xe1\xa6\xac',
'WhiteUppointingSmallTriangle': '\xe2\x96\xb5',
'CombiningCyrillicLetterMonographUk': '\xe2\xb7\xb9',
'DevanagariLetterJa': '\xe0\xa4\x9c',
'BuhidLetterNga': '\xe1\x9d\x85',
'CjkUnifiedIdeograph44ae': '\xe4\x92\xae',
'CjkUnifiedIdeograph9f70': '\xe9\xbd\xb0',
'HangulSyllableKeo': '\xec\xbb\xa4',
'UnicodeCharacter0xf68c': '\xef\x9a\x8c',
'UnicodeCharacter0xee19': '\xee\xb8\x99',
'UnicodeCharacter0xee18': '\xee\xb8\x98',
'UnicodeCharacter0xee15': '\xee\xb8\x95',
'UnicodeCharacter0xee14': '\xee\xb8\x94',
'UnicodeCharacter0xee17': '\xee\xb8\x97',
'HangulChoseongPieupphieuph': '\xe1\x84\xaa',
'UnicodeCharacter0xee11': '\xee\xb8\x91',
'UnicodeCharacter0xee10': '\xee\xb8\x90',
'UnicodeCharacter0xee13': '\xee\xb8\x93',
'UnicodeCharacter0xee12': '\xee\xb8\x92',
'YiRadicalQot': '\xea\x92\x90',
'ArabicLigatureYehWithHamzaAboveWithAlefFinalForm': '\xef\xaf\xab',
'CjkUnifiedIdeograph9f71': '\xe9\xbd\xb1',
'YiRadicalZur': '\xea\x93\x81',
'KangxiRadicalTall': '\xe2\xbe\xbc',
'UnicodeCharacter0xe4a8': '\xee\x92\xa8',
'UnicodeCharacter0xe4a9': '\xee\x92\xa9',
'CjkUnifiedIdeograph9207': '\xe9\x88\x87',
'HangulSyllablePelp': '\xed\x8e\xa6',
'ArabicLigatureYehWithHamzaAboveWithYehIsolatedForm': '\xef\xb0\x84',
'CjkUnifiedIdeograph9206': '\xe9\x88\x86',
'CjkUnifiedIdeograph4925': '\xe4\xa4\xa5',
'CjkUnifiedIdeograph4924': '\xe4\xa4\xa4',
'CjkUnifiedIdeograph4927': '\xe4\xa4\xa7',
'CjkUnifiedIdeograph4926': '\xe4\xa4\xa6',
'CjkUnifiedIdeograph4921': '\xe4\xa4\xa1',
'EthiopicSyllableQwee': '\xe1\x89\x8c',
'CjkUnifiedIdeograph4923': '\xe4\xa4\xa3',
'CjkUnifiedIdeograph4922': '\xe4\xa4\xa2',
'CjkUnifiedIdeograph4929': '\xe4\xa4\xa9',
'CjkUnifiedIdeograph4928': '\xe4\xa4\xa8',
'UnicodeCharacter0xf058': '\xef\x81\x98',
'CjkUnifiedIdeograph7f57': '\xe7\xbd\x97',
'BoxDrawingsHeavyUpAndLeft': '\xe2\x94\x9b',
'UnicodeCharacter0xf055': '\xef\x81\x95',
'OutlinedLatinCross': '\xe2\x9c\x9f',
'CjkUnifiedIdeograph9f77': '\xe9\xbd\xb7',
'UnicodeCharacter0xf054': '\xef\x81\x94',
'UnicodeCharacter0xee1e': '\xee\xb8\x9e',
'UnicodeCharacter0xee1d': '\xee\xb8\x9d',
'UnicodeCharacter0xee1f': '\xee\xb8\x9f',
'UnicodeCharacter0xee1a': '\xee\xb8\x9a',
'HangulSyllableHoegs': '\xed\x9a\x8f',
'UnicodeCharacter0xee1c': '\xee\xb8\x9c',
'UnicodeCharacter0xee1b': '\xee\xb8\x9b',
'CjkUnifiedIdeograph7478': '\xe7\x91\xb8',
'CjkUnifiedIdeograph7479': '\xe7\x91\xb9',
'HangulSyllableGgyuss': '\xeb\x80\xbc',
'UnicodeCharacter0xeff3': '\xee\xbf\xb3',
'CjkUnifiedIdeograph7472': '\xe7\x91\xb2',
'CjkUnifiedIdeograph7473': '\xe7\x91\xb3',
'CjkUnifiedIdeograph7470': '\xe7\x91\xb0',
'CjkUnifiedIdeograph7471': '\xe7\x91\xb1',
'CjkUnifiedIdeograph7476': '\xe7\x91\xb6',
'CjkUnifiedIdeograph7477': '\xe7\x91\xb7',
'CjkUnifiedIdeograph7474': '\xe7\x91\xb4',
'CjkUnifiedIdeograph7475': '\xe7\x91\xb5',
'HangulSyllableDyebs': '\xeb\x8e\xba',
'UnicodeCharacter0xf053': '\xef\x81\x93',
'MyanmarVowelSignUu': '\xe1\x80\xb0',
'MongolianLetterAliGaliAnusvaraOne': '\xe1\xa2\x80',
'UnicodeCharacter0xe4a7': '\xee\x92\xa7',
'HangulSyllableHeonh': '\xed\x97\x8e',
'ArabicLetterMeemWithDotBelow': '\xdd\xa6',
'HangulSyllableBwaj': '\xeb\xb4\xa6',
'CjkUnifiedIdeograph8a49': '\xe8\xa9\x89',
'CjkUnifiedIdeograph8a48': '\xe8\xa9\x88',
'HangulSyllablePels': '\xed\x8e\xa4',
'HangulSyllableDdeung': '\xeb\x9c\xbd',
'CjkUnifiedIdeograph8a43': '\xe8\xa9\x83',
'CjkUnifiedIdeograph8a42': '\xe8\xa9\x82',
'CjkUnifiedIdeograph8a41': '\xe8\xa9\x81',
'CjkUnifiedIdeograph8a40': '\xe8\xa9\x80',
'CjkUnifiedIdeograph8a47': '\xe8\xa9\x87',
'CjkUnifiedIdeograph8a46': '\xe8\xa9\x86',
'CjkUnifiedIdeograph8a45': '\xe8\xa9\x85',
'CjkUnifiedIdeograph8a44': '\xe8\xa9\x84',
'LongRightwardsSquiggleArrow': '\xe2\x9f\xbf',
'LatinCapitalLetterSWithDotBelow': '\xe1\xb9\xa2',
'TifinaghLetterTuaregYagh': '\xe2\xb5\x97',
'TifinaghLetterTuaregYagn': '\xe2\xb5\x90',
'HangulSyllableDyej': '\xeb\x8e\xbe',
'HangulSyllableGong': '\xea\xb3\xb5',
'CjkUnifiedIdeograph747b': '\xe7\x91\xbb',
'CjkUnifiedIdeograph747c': '\xe7\x91\xbc',
'CjkUnifiedIdeograph747a': '\xe7\x91\xba',
'CjkUnifiedIdeograph747f': '\xe7\x91\xbf',
'HangulSyllableGonh': '\xea\xb3\xa6',
'CjkUnifiedIdeograph747d': '\xe7\x91\xbd',
'LeftwardsArrowAboveAlmostEqualTo': '\xe2\xad\x8a',
'UnicodeCharacter0xe1f0': '\xee\x87\xb0',
'CyrillicSmallLetterGheWithStroke': '\xd2\x93',
'CjkUnifiedIdeograph4e30': '\xe4\xb8\xb0',
'CjkCompatibilityIdeographf94e': '\xef\xa5\x8e',
'HangulSyllableDyeb': '\xeb\x8e\xb9',
'CjkUnifiedIdeograph8a4c': '\xe8\xa9\x8c',
'CjkUnifiedIdeograph8a4b': '\xe8\xa9\x8b',
'UnicodeCharacter0x135e': '\xe1\x8d\x9e',
'UnicodeCharacter0x135d': '\xe1\x8d\x9d',
'UnicodeCharacter0x135c': '\xe1\x8d\x9c',
'UnicodeCharacter0x135b': '\xe1\x8d\x9b',
'CjkUnifiedIdeograph8a4e': '\xe8\xa9\x8e',
'CjkUnifiedIdeograph8a4d': '\xe8\xa9\x8d',
'HangulSyllableOgg': '\xec\x98\xa6',
'GreekSmallLetterOmicronWithDasiaAndOxia': '\xe1\xbd\x85',
'CjkCompatibilityIdeographf94d': '\xef\xa5\x8d',
'HangulSyllableDyed': '\xeb\x8e\xaf',
'UnicodeCharacter0xd8af': '\xed\xa2\xaf',
'LatinSmallLetterKWithHook': '\xc6\x99',
'HangulSyllableDoelm': '\xeb\x90\xa2',
'UnicodeCharacter0x18e8': '\xe1\xa3\xa8',
'UnicodeCharacter0x18e9': '\xe1\xa3\xa9',
'UnicodeCharacter0x18e4': '\xe1\xa3\xa4',
'UnicodeCharacter0x18e5': '\xe1\xa3\xa5',
'UnicodeCharacter0x18e6': '\xe1\xa3\xa6',
'UnicodeCharacter0x18e7': '\xe1\xa3\xa7',
'UnicodeCharacter0x18e0': '\xe1\xa3\xa0',
'UnicodeCharacter0x18e1': '\xe1\xa3\xa1',
'UnicodeCharacter0x18e2': '\xe1\xa3\xa2',
'UnicodeCharacter0x18e3': '\xe1\xa3\xa3',
'CjkUnifiedIdeograph96ea': '\xe9\x9b\xaa',
'CjkUnifiedIdeograph96eb': '\xe9\x9b\xab',
'CjkUnifiedIdeograph96ec': '\xe9\x9b\xac',
'CjkUnifiedIdeograph96ed': '\xe9\x9b\xad',
'CjkUnifiedIdeograph96ee': '\xe9\x9b\xae',
'CjkUnifiedIdeograph96ef': '\xe9\x9b\xaf',
'SinhalaLetterMahaapraanaCayanna': '\xe0\xb6\xa1',
'UnicodeCharacter0xeff5': '\xee\xbf\xb5',
'CjkUnifiedIdeograph3d5f': '\xe3\xb5\x9f',
'CjkUnifiedIdeograph9ced': '\xe9\xb3\xad',
'BalineseLetterVeSasak': '\xe1\xad\x89',
'CjkUnifiedIdeograph9f1b': '\xe9\xbc\x9b',
'HangulSyllablePyanj': '\xed\x8d\x89',
'VaiSymbolBang': '\xea\x98\x97',
'CjkUnifiedIdeograph96e0': '\xe9\x9b\xa0',
'CjkUnifiedIdeograph96e1': '\xe9\x9b\xa1',
'CjkUnifiedIdeograph96e2': '\xe9\x9b\xa2',
'CjkUnifiedIdeograph96e3': '\xe9\x9b\xa3',
'CjkUnifiedIdeograph96e4': '\xe9\x9b\xa4',
'CjkUnifiedIdeograph96e5': '\xe9\x9b\xa5',
'CjkUnifiedIdeograph96e6': '\xe9\x9b\xa6',
'ModifierLetterSmallReversedGlottalStop': '\xcb\xa4',
'CjkUnifiedIdeograph96e8': '\xe9\x9b\xa8',
'CjkUnifiedIdeograph96e9': '\xe9\x9b\xa9',
'KangxiRadicalPrivate': '\xe2\xbc\x9b',
'HangulSyllableKoeng': '\xec\xbe\xbd',
'ModifierLetterStressAndLowTone': '\xea\x9c\xa1',
'HangulSyllableSswigs': '\xec\x92\xbb',
'UnicodeCharacter0x18ed': '\xe1\xa3\xad',
'UnicodeCharacter0x18ee': '\xe1\xa3\xae',
'UnicodeCharacter0x18ef': '\xe1\xa3\xaf',
'HangulSyllableDyet': '\xeb\x8f\x81',
'UnicodeCharacter0x18ea': '\xe1\xa3\xaa',
'UnicodeCharacter0x18eb': '\xe1\xa3\xab',
'UnicodeCharacter0x18ec': '\xe1\xa3\xac',
'CjkUnifiedIdeograph6cfa': '\xe6\xb3\xba',
'CanadianSyllabicsWestcreeShwe': '\xe1\x94\x98',
'CjkUnifiedIdeograph6cfc': '\xe6\xb3\xbc',
'CjkUnifiedIdeograph6cfb': '\xe6\xb3\xbb',
'CjkUnifiedIdeograph34b9': '\xe3\x92\xb9',
'TeluguLetterKha': '\xe0\xb0\x96',
'CjkUnifiedIdeograph6cff': '\xe6\xb3\xbf',
'CjkUnifiedIdeograph34b5': '\xe3\x92\xb5',
'CjkUnifiedIdeograph34b4': '\xe3\x92\xb4',
'CjkUnifiedIdeograph34b7': '\xe3\x92\xb7',
'CjkUnifiedIdeograph34b6': '\xe3\x92\xb6',
'CjkUnifiedIdeograph34b1': '\xe3\x92\xb1',
'CjkUnifiedIdeograph34b0': '\xe3\x92\xb0',
'CjkUnifiedIdeograph34b3': '\xe3\x92\xb3',
'CjkUnifiedIdeograph34b2': '\xe3\x92\xb2',
'YiRadicalGep': '\xea\x92\x97',
'BugineseLetterNra': '\xe1\xa8\x8b',
'RejangLetterNa': '\xea\xa4\xb5',
'YiSyllableHep': '\xea\x89\xbf',
'-+': '\xe2\x88\x93',
'3rt': '\xe2\x88\x9b',
'YiSyllableHex': '\xea\x89\xbd',
'HangulSyllableDdwass': '\xeb\x99\x80',
'MongolianLetterTodoCha': '\xe1\xa1\x92',
'CjkUnifiedIdeograph6cf1': '\xe6\xb3\xb1',
'CjkUnifiedIdeograph6cf0': '\xe6\xb3\xb0',
'CjkUnifiedIdeograph6cf3': '\xe6\xb3\xb3',
'CjkUnifiedIdeograph6cf2': '\xe6\xb3\xb2',
'CjkUnifiedIdeograph6cf5': '\xe6\xb3\xb5',
'EthiopicSyllableDu': '\xe1\x8b\xb1',
'CjkUnifiedIdeograph6cf7': '\xe6\xb3\xb7',
'CjkUnifiedIdeograph6cf6': '\xe6\xb3\xb6',
'CjkUnifiedIdeograph34be': '\xe3\x92\xbe',
'CjkUnifiedIdeograph34bd': '\xe3\x92\xbd',
'CjkUnifiedIdeograph34bf': '\xe3\x92\xbf',
'CjkUnifiedIdeograph34ba': '\xe3\x92\xba',
'CjkUnifiedIdeograph34bc': '\xe3\x92\xbc',
'CjkUnifiedIdeograph34bb': '\xe3\x92\xbb',
'FullwidthLatinCapitalLetterU': '\xef\xbc\xb5',
'CjkUnifiedIdeograph44a1': '\xe4\x92\xa1',
'FullwidthLatinCapitalLetterW': '\xef\xbc\xb7',
'FullwidthLatinCapitalLetterV': '\xef\xbc\xb6',
'FullwidthLatinCapitalLetterQ': '\xef\xbc\xb1',
'FullwidthLatinCapitalLetterP': '\xef\xbc\xb0',
'FullwidthLatinCapitalLetterS': '\xef\xbc\xb3',
'CjkUnifiedIdeograph44a0': '\xe4\x92\xa0',
'FullwidthLatinCapitalLetterY': '\xef\xbc\xb9',
'CjkUnifiedIdeograph44a3': '\xe4\x92\xa3',
'CircledHangulThieuth': '\xe3\x89\xab',
'HangulSyllableSyaelh': '\xec\x84\x8f',
'FullwidthLatinCapitalLetterE': '\xef\xbc\xa5',
'VaiSyllableDoo': '\xea\x96\x85',
'FullwidthLatinCapitalLetterG': '\xef\xbc\xa7',
'CjkUnifiedIdeograph44a2': '\xe4\x92\xa2',
'FullwidthLatinCapitalLetterA': '\xef\xbc\xa1',
'FullwidthLatinCapitalLetterC': '\xef\xbc\xa3',
'FullwidthLatinCapitalLetterB': '\xef\xbc\xa2',
'FullwidthLatinCapitalLetterM': '\xef\xbc\xad',
'CjkUnifiedIdeograph44a5': '\xe4\x92\xa5',
'FullwidthLatinCapitalLetterO': '\xef\xbc\xaf',
'FullwidthLatinCapitalLetterN': '\xef\xbc\xae',
'FullwidthLatinCapitalLetterI': '\xef\xbc\xa9',
'HangulSyllableDeubs': '\xeb\x93\xae',
'FullwidthLatinCapitalLetterK': '\xef\xbc\xab',
'CjkUnifiedIdeograph44a4': '\xe4\x92\xa4',
'BoxDrawingsHeavyUpAndHorizontal': '\xe2\x94\xbb',
'HangulSyllableMwilp': '\xeb\xae\x96',
'CjkUnifiedIdeograph44a7': '\xe4\x92\xa7',
'UnicodeCharacter0xf1e8': '\xef\x87\xa8',
'CjkUnifiedIdeograph44a6': '\xe4\x92\xa6',
'HebrewLigatureYiddishYodYodPatah': '\xef\xac\x9f',
'HangulSyllableBeogg': '\xeb\xb2\x86',
'NaryWhiteVerticalBar': '\xe2\xab\xbf',
'CjkRadicalEatThree': '\xe2\xbb\x9f',
'CjkUnifiedIdeograph57f2': '\xe5\x9f\xb2',
'VaiSyllableLengthener': '\xea\x98\x8c',
'HangulSyllableMwilt': '\xeb\xae\x95',
'UnicodeCharacter0xf5f9': '\xef\x97\xb9',
'UnicodeCharacter0x9fc8': '\xe9\xbf\x88',
'UnicodeCharacter0x9fc9': '\xe9\xbf\x89',
'UnicodeCharacter0xf586': '\xef\x96\x86',
'UnicodeCharacter0x9fc4': '\xe9\xbf\x84',
'UnicodeCharacter0x9fc5': '\xe9\xbf\x85',
'UnicodeCharacter0x9fc6': '\xe9\xbf\x86',
'TibetanSignGruMedRgyings': '\xe0\xbe\x8b',
'UnicodeCharacter0x9fc0': '\xe9\xbf\x80',
'UnicodeCharacter0x9fc1': '\xe9\xbf\x81',
'UnicodeCharacter0x9fc2': '\xe9\xbf\x82',
'UnicodeCharacter0x9fc3': '\xe9\xbf\x83',
'CjkUnifiedIdeograph6277': '\xe6\x89\xb7',
'CjkUnifiedIdeograph6276': '\xe6\x89\xb6',
'CjkUnifiedIdeograph6275': '\xe6\x89\xb5',
'MetricalLongOverTwoShorts': '\xe2\x8f\x94',
'CjkUnifiedIdeograph6273': '\xe6\x89\xb3',
'CjkUnifiedIdeograph6272': '\xe6\x89\xb2',
'CjkUnifiedIdeograph6271': '\xe6\x89\xb1',
'CjkUnifiedIdeograph6270': '\xe6\x89\xb0',
'UnicodeCharacter0xf583': '\xef\x96\x83',
'CjkUnifiedIdeograph6279': '\xe6\x89\xb9',
'CjkUnifiedIdeograph6278': '\xe6\x89\xb8',
'HangulSyllableByep': '\xeb\xb3\xb2',
'CjkUnifiedIdeograph9ce1': '\xe9\xb3\xa1',
'CjkUnifiedIdeograph588e': '\xe5\xa2\x8e',
'SyriacLetterKaph': '\xdc\x9f',
'UnicodeCharacter0xf581': '\xef\x96\x81',
'BalineseMusicalSymbolDong': '\xe1\xad\xa1',
'CjkUnifiedIdeograph9ce0': '\xe9\xb3\xa0',
'UnicodeCharacter0xe8cb': '\xee\xa3\x8b',
'BalineseMusicalSymbolCombiningKempliWithJegogan': '\xe1\xad\xb1',
'UnicodeCharacter0xebe1': '\xee\xaf\xa1',
'CjkUnifiedIdeograph90b7': '\xe9\x82\xb7',
'CjkUnifiedIdeograph98ca': '\xe9\xa3\x8a',
'HangulSyllableSyob': '\xec\x88\x8d',
'HangulSyllableJjolt': '\xec\xab\x89',
'CjkUnifiedIdeograph588d': '\xe5\xa2\x8d',
'CjkUnifiedIdeograph627f': '\xe6\x89\xbf',
'CjkUnifiedIdeograph627e': '\xe6\x89\xbe',
'CjkUnifiedIdeograph627d': '\xe6\x89\xbd',
'CjkUnifiedIdeograph627c': '\xe6\x89\xbc',
'CjkUnifiedIdeograph627b': '\xe6\x89\xbb',
'CjkUnifiedIdeograph627a': '\xe6\x89\xba',
'CjkUnifiedIdeograph9ce6': '\xe9\xb3\xa6',
'UnicodeCharacter0xf880': '\xef\xa2\x80',
'GeorgianLetterPar': '\xe1\x83\x9e',
'LatinSmallLetterEWithInvertedBreve': '\xc8\x87',
'HangulSyllableSyoc': '\xec\x88\x93',
'UnicodeCharacter0xf589': '\xef\x96\x89',
'CyrillicSmallLetterZhwe': '\xea\x9a\x85',
'CopticSmallLetterOldNubianNyi': '\xe2\xb3\xa1',
'UnicodeCharacter0x9fce': '\xe9\xbf\x8e',
'UnicodeCharacter0x9fcf': '\xe9\xbf\x8f',
'PresentationFormForVerticalLeftDoubleAngleBracket': '\xef\xb8\xbd',
'UnicodeCharacter0x23f0': '\xe2\x8f\xb0',
'UnicodeCharacter0x9fca': '\xe9\xbf\x8a',
'UnicodeCharacter0x9fcb': '\xe9\xbf\x8b',
'UnicodeCharacter0x9fcc': '\xe9\xbf\x8c',
'UnicodeCharacter0xf885': '\xef\xa2\x85',
'CircledIdeographLabor': '\xe3\x8a\x98',
'HangulSyllableNaenj': '\xeb\x82\xb9',
'CjkUnifiedIdeograph90b5': '\xe9\x82\xb5',
'HangulSyllableSyod': '\xec\x88\x83',
'HebrewLetterAlternativePlusSign': '\xef\xac\xa9',
'BengaliLetterHa': '\xe0\xa6\xb9',
'CjkUnifiedIdeograph588f': '\xe5\xa2\x8f',
'UnicodeCharacter0xfff3': '\xef\xbf\xb3',
'UnicodeCharacter0xe7c1': '\xee\x9f\x81',
'CjkUnifiedIdeograph8e18': '\xe8\xb8\x98',
'HangulSyllableDdweoss': '\xeb\x9b\x8c',
'CjkUnifiedIdeograph4cc4': '\xe4\xb3\x84',
'CjkUnifiedIdeograph4cc5': '\xe4\xb3\x85',
'CjkUnifiedIdeograph4cc6': '\xe4\xb3\x86',
'CjkUnifiedIdeograph4cc7': '\xe4\xb3\x87',
'CjkUnifiedIdeograph4cc0': '\xe4\xb3\x80',
'CjkUnifiedIdeograph4cc1': '\xe4\xb3\x81',
'CjkUnifiedIdeograph4cc2': '\xe4\xb3\x82',
'CjkUnifiedIdeograph4cc3': '\xe4\xb3\x83',
'ArabicLigatureJeemWithMeemWithHahInitialForm': '\xef\xb5\x99',
'CjkUnifiedIdeograph588a': '\xe5\xa2\x8a',
'CjkUnifiedIdeograph4cc8': '\xe4\xb3\x88',
'CjkUnifiedIdeograph4cc9': '\xe4\xb3\x89',
'LatinCapitalLetterBrokenL': '\xea\x9d\x86',
'ChamVowelSignEi': '\xea\xa8\xac',
'CjkUnifiedIdeograph90b3': '\xe9\x82\xb3',
'DevanagariLetterBba': '\xe0\xa5\xbf',
'WhitefeatheredRightwardsArrow': '\xe2\x9e\xb3',
'LeftSshapedBagDelimiter': '\xe2\x9f\x85',
'CjkUnifiedIdeograph4ccd': '\xe4\xb3\x8d',
'CjkUnifiedIdeograph4cce': '\xe4\xb3\x8e',
'CjkUnifiedIdeograph4ccf': '\xe4\xb3\x8f',
'HangulSyllableNwegg': '\xeb\x88\xbe',
'CjkUnifiedIdeograph4cca': '\xe4\xb3\x8a',
'CjkUnifiedIdeograph4ccb': '\xe4\xb3\x8b',
'CjkUnifiedIdeograph4ccc': '\xe4\xb3\x8c',
'HangulSyllableSyog': '\xec\x87\xbd',
'UnicodeCharacter0xe503': '\xee\x94\x83',
'CjkUnifiedIdeograph674f': '\xe6\x9d\x8f',
'CjkUnifiedIdeograph4bc5': '\xe4\xaf\x85',
'CjkUnifiedIdeograph4bc4': '\xe4\xaf\x84',
'CjkUnifiedIdeograph4bc7': '\xe4\xaf\x87',
'CjkUnifiedIdeograph4bc6': '\xe4\xaf\x86',
'CjkUnifiedIdeograph4bc1': '\xe4\xaf\x81',
'CjkUnifiedIdeograph4bc0': '\xe4\xaf\x80',
'CjkUnifiedIdeograph4bc3': '\xe4\xaf\x83',
'CjkUnifiedIdeograph4bc2': '\xe4\xaf\x82',
'CjkUnifiedIdeograph4bc9': '\xe4\xaf\x89',
'CjkUnifiedIdeograph4bc8': '\xe4\xaf\x88',
'ArabicLigatureTahWithHahInitialForm': '\xef\xb2\xb8',
'HangulSyllableIlh': '\xec\x9e\x83',
'HangulChoseongKapyeounpieup': '\xe1\x84\xab',
'UnicodeCharacter0xf1e1': '\xef\x87\xa1',
'CjkUnifiedIdeograph9f16': '\xe9\xbc\x96',
'CjkUnifiedIdeograph90b0': '\xe9\x82\xb0',
'LeftwardsArrow': '\xe2\x86\x90',
'TriplePlus': '\xe2\xa7\xbb',
'CombiningKatakanahiraganaSemivoicedSoundMark': '\xe3\x82\x9a',
'CyrillicSmallLetterNeutralYer': '\xea\x99\x8f',
'CjkUnifiedIdeograph9f13': '\xe9\xbc\x93',
'YiSyllableJuox': '\xea\x8f\xa9',
'UnicodeCharacter0xf2db': '\xef\x8b\x9b',
'KangxiRadicalSkin': '\xe2\xbd\xaa',
'CjkUnifiedIdeograph63a0': '\xe6\x8e\xa0',
'UnicodeCharacter0xf2dc': '\xef\x8b\x9c',
'HangulSyllableSegg': '\xec\x84\xba',
'YiSyllableJuot': '\xea\x8f\xa8',
'CjkUnifiedIdeograph4bce': '\xe4\xaf\x8e',
'CjkUnifiedIdeograph4bcd': '\xe4\xaf\x8d',
'CjkUnifiedIdeograph4bcf': '\xe4\xaf\x8f',
'CjkUnifiedIdeograph4bca': '\xe4\xaf\x8a',
'CjkUnifiedIdeograph4bcc': '\xe4\xaf\x8c',
'CjkUnifiedIdeograph4bcb': '\xe4\xaf\x8b',
'HangulSyllableNeng': '\xeb\x84\xb9',
'LeftwardsWhiteArrow': '\xe2\x87\xa6',
'HangulSyllableHaelh': '\xed\x96\x83',
'UnicodeCharacter0xf58b': '\xef\x96\x8b',
'CombiningDoubleTilde': '\xcd\xa0',
'HangulSyllableSyok': '\xec\x88\x94',
'HangulSyllableSswabs': '\xec\x90\x86',
'cf': '\xcf\x86',
'SyriacLetterQaph': '\xdc\xa9',
'SmallRightTortoiseShellBracket': '\xef\xb9\x9e',
'CjkUnifiedIdeograph63be': '\xe6\x8e\xbe',
'CjkUnifiedIdeograph63bd': '\xe6\x8e\xbd',
'CjkUnifiedIdeograph63bf': '\xe6\x8e\xbf',
'CjkUnifiedIdeograph63ba': '\xe6\x8e\xba',
'CjkUnifiedIdeograph63bc': '\xe6\x8e\xbc',
'CjkUnifiedIdeograph63bb': '\xe6\x8e\xbb',
'HangulSyllableSsilm': '\xec\x94\xb2',
'UnicodeCharacter0xe500': '\xee\x94\x80',
'CyrillicCapitalLetterEnWithMiddleHook': '\xd4\xa2',
'YiSyllableHlax': '\xea\x86\x9f',
'ArabicLigatureTahWithAlefMaksuraIsolatedForm': '\xef\xb3\xb5',
'ArabicLetterGhainInitialForm': '\xef\xbb\x8f',
'HangulSyllableGwaenh': '\xea\xb4\x9e',
'HangulSyllableGwaenj': '\xea\xb4\x9d',
'HalfwidthKatakanaLetterN': '\xef\xbe\x9d',
'HangulSyllableCeuj': '\xec\xb8\xb6',
'CombiningCyrillicDasiaPneumata': '\xd2\x85',
'CjkUnifiedIdeograph5ee7': '\xe5\xbb\xa7',
'CjkUnifiedIdeograph6d9e': '\xe6\xb6\x9e',
'CjkUnifiedIdeograph6d9d': '\xe6\xb6\x9d',
'CjkUnifiedIdeograph6d9c': '\xe6\xb6\x9c',
'CjkUnifiedIdeograph6d9b': '\xe6\xb6\x9b',
'CjkUnifiedIdeograph6d9a': '\xe6\xb6\x9a',
'UnicodeCharacter0xbda': '\xe0\xaf\x9a',
'UnicodeCharacter0xbdc': '\xe0\xaf\x9c',
'UnicodeCharacter0xbdb': '\xe0\xaf\x9b',
'UnicodeCharacter0xbde': '\xe0\xaf\x9e',
'UnicodeCharacter0xbdd': '\xe0\xaf\x9d',
'UnicodeCharacter0xbdf': '\xe0\xaf\x9f',
'BalineseDigitSeven': '\xe1\xad\x97',
'CanadianSyllabicsWestcreeKwaa': '\xe1\x92\x81',
'GlagoliticCapitalLetterLjudije': '\xe2\xb0\x8e',
'CjkCompatibilityIdeographf9a6': '\xef\xa6\xa6',
'CjkCompatibilityIdeographf9a7': '\xef\xa6\xa7',
'HangulSyllableJigs': '\xec\xa7\x83',
'CjkCompatibilityIdeographf9a4': '\xef\xa6\xa4',
'CjkUnifiedIdeograph63b5': '\xe6\x8e\xb5',
'CjkUnifiedIdeograph63b4': '\xe6\x8e\xb4',
'CjkUnifiedIdeograph63b7': '\xe6\x8e\xb7',
'CjkUnifiedIdeograph63b6': '\xe6\x8e\xb6',
'CjkUnifiedIdeograph63b1': '\xe6\x8e\xb1',
'CjkUnifiedIdeograph34c4': '\xe3\x93\x84',
'CjkUnifiedIdeograph63b3': '\xe6\x8e\xb3',
'BraillePatternDots238': '\xe2\xa2\x86',
'BraillePatternDots237': '\xe2\xa1\x86',
'BraillePatternDots236': '\xe2\xa0\xa6',
'BraillePatternDots235': '\xe2\xa0\x96',
'BraillePatternDots234': '\xe2\xa0\x8e',
'CjkUnifiedIdeograph63b9': '\xe6\x8e\xb9',
'CjkUnifiedIdeograph63b8': '\xe6\x8e\xb8',
'CjkCompatibilityIdeographf9a3': '\xef\xa6\xa3',
'UnicodeCharacter0xbd1': '\xe0\xaf\x91',
'UnicodeCharacter0xbd3': '\xe0\xaf\x93',
'UnicodeCharacter0xbd2': '\xe0\xaf\x92',
'UnicodeCharacter0xbd5': '\xe0\xaf\x95',
'UnicodeCharacter0xbd4': '\xe0\xaf\x94',
'UnicodeCharacter0xbd6': '\xe0\xaf\x96',
'UnicodeCharacter0xbd9': '\xe0\xaf\x99',
'UnicodeCharacter0xbd8': '\xe0\xaf\x98',
'CjkCompatibilityIdeographf9a1': '\xef\xa6\xa1',
'ArabicLetterAlefMaksuraIsolatedForm': '\xef\xbb\xaf',
'ArabicLigatureKhahWithJeemInitialForm': '\xef\xb2\xab',
'YiSyllableJjiex': '\xea\x90\x9d',
'HangulSyllableJyeoh': '\xec\xa1\x93',
'CjkUnifiedIdeograph6d99': '\xe6\xb6\x99',
'GreekSmallLetterUpsilonWithPsiliAndOxia': '\xe1\xbd\x94',
'CjkUnifiedIdeograph6d97': '\xe6\xb6\x97',
'CjkUnifiedIdeograph6d96': '\xe6\xb6\x96',
'CjkUnifiedIdeograph6d95': '\xe6\xb6\x95',
'CjkUnifiedIdeograph57fc': '\xe5\x9f\xbc',
'CjkUnifiedIdeograph6d93': '\xe6\xb6\x93',
'CjkUnifiedIdeograph6d92': '\xe6\xb6\x92',
'CjkUnifiedIdeograph6d91': '\xe6\xb6\x91',
'CjkUnifiedIdeograph6d90': '\xe6\xb6\x90',
'CircledKatakanaWa': '\xe3\x8b\xbb',
'CjkRadicalCsimplifiedLeaf': '\xe2\xbb\x9a',
'CircledKatakanaWe': '\xe3\x8b\xbd',
'CircledKatakanaWi': '\xe3\x8b\xbc',
'CircledKatakanaWo': '\xe3\x8b\xbe',
'LatinLetterBidentalPercussive': '\xca\xad',
'KhmerSymbolMuoyKoet': '\xe1\xa7\xa1',
'HangulSyllablePalm': '\xed\x8c\x96',
'CjkUnifiedIdeograph7128': '\xe7\x84\xa8',
'HangulSyllableIls': '\xec\x9e\x80',
'YiSyllableJjiep': '\xea\x90\x9f',
'HangulSyllableTyuss': '\xed\x8a\xb0',
'HangulSyllablePaegs': '\xed\x8c\xab',
'CjkUnifiedIdeograph7129': '\xe7\x84\xa9',
'HangulSyllableMyebs': '\xeb\xaa\x9e',
'UnicodeCharacter0xd927': '\xed\xa4\xa7',
'UnicodeCharacter0xd926': '\xed\xa4\xa6',
'HangulSyllablePaegg': '\xed\x8c\xaa',
'UnicodeCharacter0xd924': '\xed\xa4\xa4',
'UnicodeCharacter0xd923': '\xed\xa4\xa3',
'UnicodeCharacter0xd922': '\xed\xa4\xa2',
'UnicodeCharacter0xd921': '\xed\xa4\xa1',
'UnicodeCharacter0xd920': '\xed\xa4\xa0',
'UnicodeCharacter0xe0d7': '\xee\x83\x97',
'ArabicLigatureBehWithKhahIsolatedForm': '\xef\xb0\x87',
'UnicodeCharacter0xd929': '\xed\xa4\xa9',
'UnicodeCharacter0xd928': '\xed\xa4\xa8',
'MalayalamVowelSignAi': '\xe0\xb5\x88',
'MalayalamVowelSignAa': '\xe0\xb4\xbe',
'UnicodeCharacter0xe0d6': '\xee\x83\x96',
'CjkUnifiedIdeograph99e8': '\xe9\xa7\xa8',
'MalayalamVowelSignAu': '\xe0\xb5\x8c',
'BoxDrawingsLightUpAndHorizontal': '\xe2\x94\xb4',
'ArabicLetterNyehMedialForm': '\xef\xad\xb9',
'UnicodeCharacter0xd92f': '\xed\xa4\xaf',
'UnicodeCharacter0xd92e': '\xed\xa4\xae',
'CjkUnifiedIdeograph7123': '\xe7\x84\xa3',
'UnicodeCharacter0xd92c': '\xed\xa4\xac',
'UnicodeCharacter0xd92b': '\xed\xa4\xab',
'UnicodeCharacter0xd92a': '\xed\xa4\xaa',
'HangulSyllableGoelh': '\xea\xb5\x83',
'CjkUnifiedIdeograph8aa6': '\xe8\xaa\xa6',
'ArabicLetterEFinalForm': '\xef\xaf\xa5',
'CjkUnifiedIdeograph6888': '\xe6\xa2\x88',
'CjkUnifiedIdeograph7126': '\xe7\x84\xa6',
'BalineseLetterBa': '\xe1\xac\xa9',
'ArabicLetterTehMedialForm': '\xef\xba\x98',
'Hourglass': '\xe2\x8c\x9b',
'UnicodeCharacter0xf78e': '\xef\x9e\x8e',
'LatinSmallLetterTurnedY': '\xca\x8e',
'MyanmarSymbolCompleted': '\xe1\x81\x8d',
'UnicodeCharacter0xe9c8': '\xee\xa7\x88',
'UnicodeCharacter0xf78a': '\xef\x9e\x8a',
'Th': '\xce\x98',
'UnicodeCharacter0xf78c': '\xef\x9e\x8c',
'UnicodeCharacter0xf78b': '\xef\x9e\x8b',
'UnicodeCharacter0xe9c3': '\xee\xa7\x83',
'YiSyllableWop': '\xea\x8a\x89',
'LatinSmallLetterTurnedR': '\xc9\xb9',
'HangulSyllableJyeos': '\xec\xa1\x8b',
'LatinSmallLetterTurnedT': '\xca\x87',
'UnicodeCharacter0xe9c6': '\xee\xa7\x86',
'LatinSmallLetterTurnedV': '\xca\x8c',
'LatinSmallLetterTurnedW': '\xca\x8d',
'LatinSmallLetterTurnedH': '\xc9\xa5',
'LatinSmallLetterTurnedI': '\xe1\xb4\x89',
'UnicodeCharacter0xe0d4': '\xee\x83\x94',
'LatinSmallLetterTurnedK': '\xca\x9e',
'LatinSmallLetterTurnedL': '\xea\x9e\x81',
'LatinSmallLetterTurnedM': '\xc9\xaf',
'LatinSmallLetterSWithDotBelow': '\xe1\xb9\xa3',
'UnicodeCharacter0xe32f': '\xee\x8c\xaf',
'LatinSmallLetterTurnedA': '\xc9\x90',
'UnicodeCharacter0xe32d': '\xee\x8c\xad',
'UnicodeCharacter0xe32e': '\xee\x8c\xae',
'UnicodeCharacter0xe32b': '\xee\x8c\xab',
'LatinSmallLetterTurnedE': '\xc7\x9d',
'LatinSmallLetterTurnedG': '\xe1\xb5\xb7',
'UnicodeCharacter0xdeff': '\xed\xbb\xbf',
'UnicodeCharacter0xdefe': '\xed\xbb\xbe',
'HangulSyllablePals': '\xed\x8c\x98',
'ChamVowelSignU': '\xea\xa8\xad',
'UnicodeCharacter0xdefb': '\xed\xbb\xbb',
'UnicodeCharacter0xdefa': '\xed\xbb\xba',
'UnicodeCharacter0xa83f': '\xea\xa0\xbf',
'SyriacEndOfParagraph': '\xdc\x80',
'UnicodeCharacter0xa83d': '\xea\xa0\xbd',
'UnicodeCharacter0xa83e': '\xea\xa0\xbe',
'UnicodeCharacter0xa83b': '\xea\xa0\xbb',
'UnicodeCharacter0xa83c': '\xea\xa0\xbc',
'UnicodeCharacter0xa83a': '\xea\xa0\xba',
'UnicodeCharacter0xe0d3': '\xee\x83\x93',
'HangulSyllableMwaenj': '\xeb\xab\xa5',
'HangulSyllableMwaenh': '\xeb\xab\xa6',
'CyrillicSmallLetterTsse': '\xea\x9a\x91',
'HangulSyllableDdwanh': '\xeb\x98\xb2',
'ThaanaIbifili': '\xde\xa8',
'HangulSyllableDdwanj': '\xeb\x98\xb1',
'ChamVowelSignI': '\xea\xa8\xaa',
'HangulSyllableDdwang': '\xeb\x99\x81',
'HangulSyllableMwaeng': '\xeb\xab\xb5',
'HangulJongseongSsangieung': '\xe1\x87\xae',
'UnicodeCharacter0xebee': '\xee\xaf\xae',
'UnicodeCharacter0xe328': '\xee\x8c\xa8',
'UnicodeCharacter0xe329': '\xee\x8c\xa9',
'UnicodeCharacter0xe326': '\xee\x8c\xa6',
'ReverseSolidusOperator': '\xe2\xa7\xb5',
'UnicodeCharacter0xe324': '\xee\x8c\xa4',
'UnicodeCharacter0xe325': '\xee\x8c\xa5',
'UnicodeCharacter0xe322': '\xee\x8c\xa2',
'UnicodeCharacter0xe323': '\xee\x8c\xa3',
'UnicodeCharacter0xe0d2': '\xee\x83\x92',
'UnicodeCharacter0xe321': '\xee\x8c\xa1',
'UnicodeCharacter0xf785': '\xef\x9e\x85',
'UnicodeCharacter0xf784': '\xef\x9e\x84',
'UnicodeCharacter0xf787': '\xef\x9e\x87',
'HangulSyllableGgwass': '\xea\xbd\x9c',
'UnicodeCharacter0xf781': '\xef\x9e\x81',
'UnicodeCharacter0xf780': '\xef\x9e\x80',
'UnicodeCharacter0xf783': '\xef\x9e\x83',
'UnicodeCharacter0xf782': '\xef\x9e\x82',
'UnicodeCharacter0xe9cc': '\xee\xa7\x8c',
'UnicodeCharacter0xe9cb': '\xee\xa7\x8b',
'UnicodeCharacter0xe9ca': '\xee\xa7\x8a',
'UnicodeCharacter0xf789': '\xef\x9e\x89',
'UnicodeCharacter0xe9cf': '\xee\xa7\x8f',
'UnicodeCharacter0xe9ce': '\xee\xa7\x8e',
'UnicodeCharacter0xe9cd': '\xee\xa7\x8d',
'LatinCapitalLetterReversedE': '\xc6\x8e',
'TibetanDigitHalfSeven': '\xe0\xbc\xb0',
'ArabicLetterDalFinalForm': '\xef\xba\xaa',
'UnicodeCharacter0xe0d1': '\xee\x83\x91',
'UnicodeCharacter0xdef7': '\xed\xbb\xb7',
'HangulSyllableSsyonj': '\xec\x91\x8d',
'UnicodeCharacter0xdef5': '\xed\xbb\xb5',
'HangulSyllableSsyonh': '\xec\x91\x8e',
'UnicodeCharacter0xa4c7': '\xea\x93\x87',
'UnicodeCharacter0xdef2': '\xed\xbb\xb2',
'UnicodeCharacter0xa838': '\xea\xa0\xb8',
'UnicodeCharacter0xa839': '\xea\xa0\xb9',
'UnicodeCharacter0xa836': '\xea\xa0\xb6',
'UnicodeCharacter0xa837': '\xea\xa0\xb7',
'UnicodeCharacter0xa834': '\xea\xa0\xb4',
'UnicodeCharacter0xa835': '\xea\xa0\xb5',
'UnicodeCharacter0xa832': '\xea\xa0\xb2',
'UnicodeCharacter0xa833': '\xea\xa0\xb3',
'UnicodeCharacter0xa830': '\xea\xa0\xb0',
'UnicodeCharacter0xa831': '\xea\xa0\xb1',
'CjkUnifiedIdeograph5892': '\xe5\xa2\x92',
'CjkUnifiedIdeograph5893': '\xe5\xa2\x93',
'CjkUnifiedIdeograph5890': '\xe5\xa2\x90',
'CjkUnifiedIdeograph5891': '\xe5\xa2\x91',
'CjkUnifiedIdeograph5896': '\xe5\xa2\x96',
'CjkUnifiedIdeograph5897': '\xe5\xa2\x97',
'CjkUnifiedIdeograph5894': '\xe5\xa2\x94',
'CjkUnifiedIdeograph5895': '\xe5\xa2\x95',
'CjkUnifiedIdeograph5898': '\xe5\xa2\x98',
'CjkUnifiedIdeograph5899': '\xe5\xa2\x99',
'ModifierLetterSmallVWithHook': '\xe1\xb6\xb9',
'CanadianSyllabicsTwii': '\xe1\x91\x9b',
'UnicodeCharacter0xf439': '\xef\x90\xb9',
'UnicodeCharacter0xf438': '\xef\x90\xb8',
'CjkUnifiedIdeograph871f': '\xe8\x9c\x9f',
'UnicodeCharacter0xf436': '\xef\x90\xb6',
'CjkUnifiedIdeograph871d': '\xe8\x9c\x9d',
'CjkUnifiedIdeograph871e': '\xe8\x9c\x9e',
'CjkUnifiedIdeograph871b': '\xe8\x9c\x9b',
'CjkUnifiedIdeograph871c': '\xe8\x9c\x9c',
'UnicodeCharacter0xf431': '\xef\x90\xb1',
'CjkUnifiedIdeograph871a': '\xe8\x9c\x9a',
'HangulSyllableJjils': '\xec\xb0\x98',
'HangulSyllableJjilp': '\xec\xb0\x9a',
'KayahLiToneCalyaPlophu': '\xea\xa4\xad',
'HangulSyllableJjilt': '\xec\xb0\x99',
'ContourIntegral': '\xe2\x88\xae',
'HangulSyllableSwelt': '\xec\x89\x9d',
'HangulSyllableJjilb': '\xec\xb0\x97',
'HangulSyllableJjilg': '\xec\xb0\x95',
'UnicodeCharacter0xe3b7': '\xee\x8e\xb7',
'HangulSyllableJjilh': '\xec\xb0\x9b',
'HangulSyllableJjilm': '\xec\xb0\x96',
'UnicodeCharacter0xde19': '\xed\xb8\x99',
'HiraganaLetterSu': '\xe3\x81\x99',
'IdeographicDescriptionCharacterSurroundFromBelow': '\xe2\xbf\xb6',
'CjkUnifiedIdeograph8718': '\xe8\x9c\x98',
'CjkUnifiedIdeograph8719': '\xe8\x9c\x99',
'CjkUnifiedIdeograph8716': '\xe8\x9c\x96',
'CjkUnifiedIdeograph8717': '\xe8\x9c\x97',
'CjkUnifiedIdeograph8714': '\xe8\x9c\x94',
'CjkUnifiedIdeograph8715': '\xe8\x9c\x95',
'CjkUnifiedIdeograph8712': '\xe8\x9c\x92',
'CjkUnifiedIdeograph8713': '\xe8\x9c\x93',
'CjkUnifiedIdeograph8710': '\xe8\x9c\x90',
'KangxiRadicalSpeech': '\xe2\xbe\x94',
'CjkUnifiedIdeograph589b': '\xe5\xa2\x9b',
'HiraganaLetterSe': '\xe3\x81\x9b',
'CjkUnifiedIdeograph589a': '\xe5\xa2\x9a',
'CjkUnifiedIdeograph589f': '\xe5\xa2\x9f',
'HiraganaLetterSa': '\xe3\x81\x95',
'CjkUnifiedIdeograph589d': '\xe5\xa2\x9d',
'CjkUnifiedIdeograph589e': '\xe5\xa2\x9e',
'UnicodeCharacter0xe3b4': '\xee\x8e\xb4',
'HiraganaLetterSo': '\xe3\x81\x9d',
'TibetanDigitHalfSix': '\xe0\xbc\xaf',
'HiraganaLetterSi': '\xe3\x81\x97',
'UnicodeCharacter0xecc8': '\xee\xb3\x88',
'ArabicLetterKirghizYuFinalForm': '\xef\xaf\xa3',
'CjkUnifiedIdeograph8d3c': '\xe8\xb4\xbc',
'UnicodeCharacter0xaaab': '\xea\xaa\xab',
'NotIdenticalTo': '\xe2\x89\xa2',
'UnicodeCharacter0xa78d': '\xea\x9e\x8d',
'UnicodeCharacter0xa78e': '\xea\x9e\x8e',
'UnicodeCharacter0xa78f': '\xea\x9e\x8f',
'HangulSyllableNwis': '\xeb\x89\xab',
'HangulSyllableCyulh': '\xec\xb8\x93',
'UnicodeCharacter0xecc4': '\xee\xb3\x84',
'SaurashtraLetterTta': '\xea\xa2\x9c',
'LatinSmallLetterEshWithCurl': '\xca\x86',
'UnicodeCharacter0xe105': '\xee\x84\x85',
'CjkUnifiedIdeograph8d3f': '\xe8\xb4\xbf',
'BoxDrawingsUpSingleAndRightDouble': '\xe2\x95\x98',
'LatinSmallLetterAWithRetroflexHook': '\xe1\xb6\x8f',
'HangulSyllableNwip': '\xeb\x89\xb2',
'SinhalaLetterMuurdhajaNayanna': '\xe0\xb6\xab',
'HangulSyllableCyulm': '\xec\xb8\x8e',
'UnicodeCharacter0xe514': '\xee\x94\x94',
'HangulSyllableNyegg': '\xeb\x85\x9e',
'CircledNumberFortySeven': '\xe3\x8a\xbc',
'CjkUnifiedIdeograph8aaf': '\xe8\xaa\xaf',
'HangulSyllableDdwi': '\xeb\x9b\xb0',
'HangulSyllableCyulb': '\xec\xb8\x8f',
'HangulSyllableDdwe': '\xeb\x9b\x94',
'HangulSyllableNwit': '\xeb\x89\xb1',
'HangulSyllableSwec': '\xec\x89\xa7',
'HangulSyllableDdwa': '\xeb\x98\xac',
'HangulSyllableCeulh': '\xec\xb8\xaf',
'SouthEastArrowToCorner': '\xe2\x87\xb2',
'HangulSyllablePolm': '\xed\x8f\xb6',
'HangulSyllableSegs': '\xec\x84\xbb',
'HangulSyllablePolh': '\xed\x8f\xbb',
'UnicodeCharacter0xef75': '\xee\xbd\xb5',
'HangulSyllableSwelp': '\xec\x89\x9e',
'HangulSyllablePolg': '\xed\x8f\xb5',
'HangulSyllablePolb': '\xed\x8f\xb7',
'CjkCompatibilityIdeographf9ad': '\xef\xa6\xad',
'UnicodeCharacter0xe3b3': '\xee\x8e\xb3',
'TibetanSubjoinedLetterJa': '\xe0\xbe\x97',
'CjkCompatibilityIdeographf9ab': '\xef\xa6\xab',
'HangulSyllablePolt': '\xed\x8f\xb9',
'HangulSyllablePols': '\xed\x8f\xb8',
'HangulSyllablePolp': '\xed\x8f\xba',
'CjkCompatibilityIdeographf9ac': '\xef\xa6\xac',
'UnicodeCharacter0xe516': '\xee\x94\x96',
'SaurashtraLetterNya': '\xea\xa2\x9b',
'YiSyllableZhur': '\xea\x8d\x9a',
'UnicodeCharacter0xe3b0': '\xee\x8e\xb0',
'HangulSyllableTwags': '\xed\x86\xbf',
'HangulSyllableNwanj': '\xeb\x86\x99',
'HangulSyllableNwanh': '\xeb\x86\x9a',
'HangulSyllableNwang': '\xeb\x86\xa9',
'CjkUnifiedIdeograph957c': '\xe9\x95\xbc',
'UnicodeCharacter0xe511': '\xee\x94\x91',
'GurmukhiLetterKa': '\xe0\xa8\x95',
'UnicodeCharacter0xe3b1': '\xee\x8e\xb1',
'UnicodeCharacter0xef79': '\xee\xbd\xb9',
'YiSyllableXuox': '\xea\x91\xa5',
'UnicodeCharacter0xe510': '\xee\x94\x90',
'HangulSyllableSsal': '\xec\x8c\x80',
'CjkUnifiedIdeograph725b': '\xe7\x89\x9b',
'CjkUnifiedIdeograph725c': '\xe7\x89\x9c',
'CjkUnifiedIdeograph725a': '\xe7\x89\x9a',
'CjkUnifiedIdeograph725f': '\xe7\x89\x9f',
'CjkUnifiedIdeograph6948': '\xe6\xa5\x88',
'CjkUnifiedIdeograph725d': '\xe7\x89\x9d',
'CjkUnifiedIdeograph725e': '\xe7\x89\x9e',
'CjkUnifiedIdeograph394b': '\xe3\xa5\x8b',
'CjkUnifiedIdeograph394c': '\xe3\xa5\x8c',
'UnicodeCharacter0xeb3f': '\xee\xac\xbf',
'LatinCapitalLetterNWithDotBelow': '\xe1\xb9\x86',
'CjkUnifiedIdeograph394f': '\xe3\xa5\x8f',
'TeluguSignVirama': '\xe0\xb1\x8d',
'CjkUnifiedIdeograph394d': '\xe3\xa5\x8d',
'CjkUnifiedIdeograph394e': '\xe3\xa5\x8e',
'CjkCompatibilityIdeographf941': '\xef\xa5\x81',
'CjkUnifiedIdeograph9399': '\xe9\x8e\x99',
'CjkUnifiedIdeograph9398': '\xe9\x8e\x98',
'DigitOneFullStop': '\xe2\x92\x88',
'CjkUnifiedIdeograph9395': '\xe9\x8e\x95',
'CjkUnifiedIdeograph9394': '\xe9\x8e\x94',
'CjkUnifiedIdeograph9397': '\xe9\x8e\x97',
'CjkUnifiedIdeograph9396': '\xe9\x8e\x96',
'CjkUnifiedIdeograph9391': '\xe9\x8e\x91',
'CjkUnifiedIdeograph9390': '\xe9\x8e\x90',
'CjkUnifiedIdeograph9393': '\xe9\x8e\x93',
'CjkUnifiedIdeograph9392': '\xe9\x8e\x92',
'CjkUnifiedIdeograph4712': '\xe4\x9c\x92',
'CjkUnifiedIdeograph4713': '\xe4\x9c\x93',
'CjkUnifiedIdeograph4710': '\xe4\x9c\x90',
'CjkUnifiedIdeograph4711': '\xe4\x9c\x91',
'CjkUnifiedIdeograph4716': '\xe4\x9c\x96',
'CjkUnifiedIdeograph4717': '\xe4\x9c\x97',
'CjkUnifiedIdeograph4714': '\xe4\x9c\x94',
'CjkUnifiedIdeograph4715': '\xe4\x9c\x95',
'CjkUnifiedIdeograph4718': '\xe4\x9c\x98',
'CjkUnifiedIdeograph4719': '\xe4\x9c\x99',
'CombiningCaronBelow': '\xcc\xac',
'CjkCompatibilityIdeographf940': '\xef\xa5\x80',
'CjkUnifiedIdeograph77cd': '\xe7\x9f\x8d',
'CjkUnifiedIdeograph3a86': '\xe3\xaa\x86',
'CjkUnifiedIdeograph3948': '\xe3\xa5\x88',
'CjkUnifiedIdeograph3949': '\xe3\xa5\x89',
'UnicodeCharacter0xeb38': '\xee\xac\xb8',
'UnicodeCharacter0xeb39': '\xee\xac\xb9',
'CjkUnifiedIdeograph3942': '\xe3\xa5\x82',
'CjkUnifiedIdeograph3943': '\xe3\xa5\x83',
'CjkUnifiedIdeograph3940': '\xe3\xa5\x80',
'CjkUnifiedIdeograph3941': '\xe3\xa5\x81',
'CjkUnifiedIdeograph3946': '\xe3\xa5\x86',
'CjkUnifiedIdeograph3947': '\xe3\xa5\x87',
'CjkUnifiedIdeograph3944': '\xe3\xa5\x84',
'CjkUnifiedIdeograph3945': '\xe3\xa5\x85',
'CjkUnifiedIdeograph7258': '\xe7\x89\x98',
'CjkUnifiedIdeograph7259': '\xe7\x89\x99',
'UnicodeCharacter0x955': '\xe0\xa5\x95',
'YiSyllableJurx': '\xea\x8f\xb4',
'UnicodeCharacter0x957': '\xe0\xa5\x97',
'UnicodeCharacter0x956': '\xe0\xa5\x96',
'CjkUnifiedIdeograph7252': '\xe7\x89\x92',
'CjkUnifiedIdeograph7253': '\xe7\x89\x93',
'CjkUnifiedIdeograph41c7': '\xe4\x87\x87',
'CjkUnifiedIdeograph7251': '\xe7\x89\x91',
'CjkUnifiedIdeograph7256': '\xe7\x89\x96',
'CjkUnifiedIdeograph7257': '\xe7\x89\x97',
'CjkUnifiedIdeograph7254': '\xe7\x89\x94',
'CjkUnifiedIdeograph7255': '\xe7\x89\x95',
'CjkUnifiedIdeograph471b': '\xe4\x9c\x9b',
'CjkUnifiedIdeograph471c': '\xe4\x9c\x9c',
'UnicodeCharacter0xf5a2': '\xef\x96\xa2',
'CjkUnifiedIdeograph471a': '\xe4\x9c\x9a',
'CjkUnifiedIdeograph471f': '\xe4\x9c\x9f',
'CjkUnifiedIdeograph3a80': '\xe3\xaa\x80',
'CjkUnifiedIdeograph471d': '\xe4\x9c\x9d',
'CjkUnifiedIdeograph471e': '\xe4\x9c\x9e',
'UnicodeCharacter0xf5a8': '\xef\x96\xa8',
'UnicodeCharacter0xf5a9': '\xef\x96\xa9',
'UnicodeCharacter0xf512': '\xef\x94\x92',
'LatinSmallLetterTurnedInsularG': '\xea\x9d\xbf',
'ArabicLigatureKafWithKhahIsolatedForm': '\xef\xb0\xba',
'UnicodeCharacter0xf513': '\xef\x94\x93',
'CjkUnifiedIdeograph939e': '\xe9\x8e\x9e',
'CjkUnifiedIdeograph939d': '\xe9\x8e\x9d',
'CjkUnifiedIdeograph939f': '\xe9\x8e\x9f',
'CjkUnifiedIdeograph939a': '\xe9\x8e\x9a',
'DifferenceBetween': '\xe2\x89\x8f',
'CjkUnifiedIdeograph939b': '\xe9\x8e\x9b',
'UnicodeCharacter0xed9a': '\xee\xb6\x9a',
'UnicodeCharacter0xed9b': '\xee\xb6\x9b',
'UnicodeCharacter0xed9c': '\xee\xb6\x9c',
'UnicodeCharacter0xed9d': '\xee\xb6\x9d',
'UnicodeCharacter0xed9e': '\xee\xb6\x9e',
'UnicodeCharacter0xed9f': '\xee\xb6\x9f',
'CjkUnifiedIdeograph8d32': '\xe8\xb4\xb2',
'UnicodeCharacter0xf517': '\xef\x94\x97',
'HangulSyllableHyeogs': '\xed\x98\x83',
'CjkUnifiedIdeograph5bba': '\xe5\xae\xba',
'NegativeCircledNumberFourteen': '\xe2\x93\xae',
'CjkUnifiedIdeograph5bbc': '\xe5\xae\xbc',
'CjkUnifiedIdeograph5bbb': '\xe5\xae\xbb',
'CjkUnifiedIdeograph5bbe': '\xe5\xae\xbe',
'CjkUnifiedIdeograph5bbd': '\xe5\xae\xbd',
'CjkUnifiedIdeograph5bbf': '\xe5\xae\xbf',
'HangulSyllableHyeogg': '\xed\x98\x82',
'DigitSixFullStop': '\xe2\x92\x8d',
'UnicodeCharacter0xe1b4': '\xee\x86\xb4',
'LatinSmallLetterHeng': '\xea\x9c\xa7',
'CjkUnifiedIdeograph5041': '\xe5\x81\x81',
'CjkUnifiedIdeograph5040': '\xe5\x81\x80',
'CjkUnifiedIdeograph5043': '\xe5\x81\x83',
'CjkUnifiedIdeograph5042': '\xe5\x81\x82',
'CjkUnifiedIdeograph5045': '\xe5\x81\x85',
'CjkUnifiedIdeograph5044': '\xe5\x81\x84',
'CjkUnifiedIdeograph5047': '\xe5\x81\x87',
'CjkUnifiedIdeograph5046': '\xe5\x81\x86',
'CjkUnifiedIdeograph5049': '\xe5\x81\x89',
'CjkUnifiedIdeograph5048': '\xe5\x81\x88',
'CjkUnifiedIdeograph74a6': '\xe7\x92\xa6',
'CjkUnifiedIdeograph74a7': '\xe7\x92\xa7',
'CjkUnifiedIdeograph74a0': '\xe7\x92\xa0',
'CjkUnifiedIdeograph74a1': '\xe7\x92\xa1',
'CjkUnifiedIdeograph74a2': '\xe7\x92\xa2',
'CjkUnifiedIdeograph74a3': '\xe7\x92\xa3',
'CjkUnifiedIdeograph9971': '\xe9\xa5\xb1',
'CjkUnifiedIdeograph9970': '\xe9\xa5\xb0',
'CjkUnifiedIdeograph9973': '\xe9\xa5\xb3',
'CjkUnifiedIdeograph9972': '\xe9\xa5\xb2',
'CjkUnifiedIdeograph9975': '\xe9\xa5\xb5',
'CjkUnifiedIdeograph9974': '\xe9\xa5\xb4',
'CjkUnifiedIdeograph9977': '\xe9\xa5\xb7',
'CjkUnifiedIdeograph9976': '\xe9\xa5\xb6',
'CjkUnifiedIdeograph9979': '\xe9\xa5\xb9',
'CjkUnifiedIdeograph9978': '\xe9\xa5\xb8',
'UnicodeCharacter0xe1b5': '\xee\x86\xb5',
'CjkUnifiedIdeograph5bb1': '\xe5\xae\xb1',
'CjkUnifiedIdeograph5bb0': '\xe5\xae\xb0',
'CjkUnifiedIdeograph5bb3': '\xe5\xae\xb3',
'CjkUnifiedIdeograph5bb2': '\xe5\xae\xb2',
'CjkUnifiedIdeograph5bb5': '\xe5\xae\xb5',
'CjkUnifiedIdeograph5bb4': '\xe5\xae\xb4',
'CjkUnifiedIdeograph5bb7': '\xe5\xae\xb7',
'CjkUnifiedIdeograph5bb6': '\xe5\xae\xb6',
'CjkUnifiedIdeograph5bb9': '\xe5\xae\xb9',
'CjkUnifiedIdeograph5bb8': '\xe5\xae\xb8',
'HangulSyllableByulm': '\xeb\xb7\xba',
'HangulSyllableByulh': '\xeb\xb7\xbf',
'UnicodeCharacter0xed90': '\xee\xb6\x90',
'UnicodeCharacter0xed91': '\xee\xb6\x91',
'HangulSyllableByult': '\xeb\xb7\xbd',
'UnicodeCharacter0xed93': '\xee\xb6\x93',
'LessthanOverEqualTo': '\xe2\x89\xa6',
'HangulSyllableByuls': '\xeb\xb7\xbc',
'HangulSyllableByulp': '\xeb\xb7\xbe',
'UnicodeCharacter0xed97': '\xee\xb6\x97',
'UnicodeCharacter0xe1b6': '\xee\x86\xb6',
'UnicodeCharacter0xed99': '\xee\xb6\x99',
'CjkUnifiedIdeograph5da1': '\xe5\xb6\xa1',
'CjkUnifiedIdeograph997a': '\xe9\xa5\xba',
'CjkUnifiedIdeograph997c': '\xe9\xa5\xbc',
'CjkUnifiedIdeograph997b': '\xe9\xa5\xbb',
'CjkUnifiedIdeograph997e': '\xe9\xa5\xbe',
'CjkUnifiedIdeograph997d': '\xe9\xa5\xbd',
'TibetanLetterYa': '\xe0\xbd\xa1',
'CjkUnifiedIdeograph997f': '\xe9\xa5\xbf',
'UnicodeCharacter0xe54c': '\xee\x95\x8c',
'CjkUnifiedIdeograph504a': '\xe5\x81\x8a',
'UnicodeCharacter0xf6a2': '\xef\x9a\xa2',
'CjkUnifiedIdeograph504c': '\xe5\x81\x8c',
'CjkUnifiedIdeograph504b': '\xe5\x81\x8b',
'CjkUnifiedIdeograph504e': '\xe5\x81\x8e',
'CjkUnifiedIdeograph504d': '\xe5\x81\x8d',
'UnicodeCharacter0xe1b7': '\xee\x86\xb7',
'CjkUnifiedIdeograph504f': '\xe5\x81\x8f',
'CjkUnifiedIdeograph74ad': '\xe7\x92\xad',
'CjkUnifiedIdeograph74ae': '\xe7\x92\xae',
'CjkUnifiedIdeograph74af': '\xe7\x92\xaf',
'HangulChoseongRieul': '\xe1\x84\x85',
'CjkUnifiedIdeograph74aa': '\xe7\x92\xaa',
'CjkUnifiedIdeograph74ab': '\xe7\x92\xab',
'CjkUnifiedIdeograph74ac': '\xe7\x92\xac',
'CjkUnifiedIdeograph6660': '\xe6\x99\xa0',
'CjkUnifiedIdeograph6661': '\xe6\x99\xa1',
'CjkUnifiedIdeograph6662': '\xe6\x99\xa2',
'CjkUnifiedIdeograph6663': '\xe6\x99\xa3',
'CjkUnifiedIdeograph6664': '\xe6\x99\xa4',
'CjkUnifiedIdeograph6665': '\xe6\x99\xa5',
'CjkUnifiedIdeograph6666': '\xe6\x99\xa6',
'CjkUnifiedIdeograph6667': '\xe6\x99\xa7',
'CjkUnifiedIdeograph3e5a': '\xe3\xb9\x9a',
'CjkUnifiedIdeograph6669': '\xe6\x99\xa9',
'CjkUnifiedIdeograph3e5c': '\xe3\xb9\x9c',
'CjkUnifiedIdeograph3e5b': '\xe3\xb9\x9b',
'CjkUnifiedIdeograph3e5e': '\xe3\xb9\x9e',
'CjkUnifiedIdeograph3e5d': '\xe3\xb9\x9d',
'ModifierLetterGlottalStop': '\xcb\x80',
'UnicodeCharacter0xe1b0': '\xee\x86\xb0',
'CjkUnifiedIdeograph8fff': '\xe8\xbf\xbf',
'CjkUnifiedIdeograph8ffd': '\xe8\xbf\xbd',
'CjkUnifiedIdeograph8ffe': '\xe8\xbf\xbe',
'CjkUnifiedIdeograph8ffb': '\xe8\xbf\xbb',
'LatinCapitalLetterRWithStroke': '\xc9\x8c',
'CjkUnifiedIdeograph8ffa': '\xe8\xbf\xba',
'HangulSyllableMweb': '\xeb\xad\xbd',
'HangulSyllableMwec': '\xeb\xae\x83',
'HangulSyllableMweg': '\xeb\xad\xad',
'TeluguLetterDda': '\xe0\xb0\xa1',
'ZeroWidthNobreakSpace': '\xef\xbb\xbf',
'HangulSyllableMwej': '\xeb\xae\x82',
'HangulSyllableMwek': '\xeb\xae\x84',
'HangulSyllableMweh': '\xeb\xae\x87',
'HangulSyllableMwen': '\xeb\xad\xb0',
'HangulSyllableMweo': '\xeb\xad\x90',
'HangulSyllableMwel': '\xeb\xad\xb4',
'HangulSyllableMwem': '\xeb\xad\xbc',
'HangulSyllableMwes': '\xeb\xad\xbf',
'HangulSyllableMwep': '\xeb\xae\x86',
'HangulSyllableMwet': '\xeb\xae\x85',
'OriyaDigitSix': '\xe0\xad\xac',
'CombiningCyrillicVzmet': '\xea\x99\xaf',
'YiSyllableGe': '\xea\x87\xb0',
'CjkUnifiedIdeograph8ff8': '\xe8\xbf\xb8',
'LatinSmallLetterAWithDiaeresisAndMacron': '\xc7\x9f',
'HangulChoseongIeungchieuch': '\xe1\x85\x89',
'CjkUnifiedIdeograph8ff7': '\xe8\xbf\xb7',
'CjkUnifiedIdeograph8ff4': '\xe8\xbf\xb4',
'CjkUnifiedIdeograph8ff5': '\xe8\xbf\xb5',
'CjkUnifiedIdeograph8ff2': '\xe8\xbf\xb2',
'IdeographicSpace': '\xe3\x80\x80',
'CjkUnifiedIdeograph8ff0': '\xe8\xbf\xb0',
'CjkUnifiedIdeograph8ff1': '\xe8\xbf\xb1',
'LeftParenthesisExtension': '\xe2\x8e\x9c',
'CjkUnifiedIdeograph3e58': '\xe3\xb9\x98',
'CjkUnifiedIdeograph666b': '\xe6\x99\xab',
'CjkUnifiedIdeograph666c': '\xe6\x99\xac',
'CjkUnifiedIdeograph666d': '\xe6\x99\xad',
'CjkUnifiedIdeograph666e': '\xe6\x99\xae',
'CjkUnifiedIdeograph666f': '\xe6\x99\xaf',
'HangulSyllableDoj': '\xeb\x8f\x9a',
'CjkUnifiedIdeograph3e51': '\xe3\xb9\x91',
'CjkUnifiedIdeograph3e50': '\xe3\xb9\x90',
'CjkUnifiedIdeograph3e53': '\xe3\xb9\x93',
'CjkUnifiedIdeograph3e52': '\xe3\xb9\x92',
'CjkUnifiedIdeograph3e55': '\xe3\xb9\x95',
'CjkUnifiedIdeograph3e54': '\xe3\xb9\x94',
'CjkUnifiedIdeograph3e57': '\xe3\xb9\x97',
'CjkUnifiedIdeograph3e56': '\xe3\xb9\x96',
'UnicodeCharacter0xf5e0': '\xef\x97\xa0',
'AplFunctionalSymbolSlashBar': '\xe2\x8c\xbf',
'CjkUnifiedIdeograph8f92': '\xe8\xbe\x92',
'EthiopicSyllableMwe': '\xe1\x8e\x83',
'HangulSyllableDyang': '\xeb\x8c\xb1',
'HangulSyllableRwig': '\xeb\xa4\xbd',
'ArabicLetterNyehIsolatedForm': '\xef\xad\xb6',
'CjkUnifiedIdeograph8670': '\xe8\x99\xb0',
'UnicodeCharacter0xf51b': '\xef\x94\x9b',
'HangulSyllableDyanj': '\xeb\x8c\xa1',
'HangulSyllableRwid': '\xeb\xa5\x83',
'HangulSyllableHeulh': '\xed\x9d\x9f',
'HangulSyllableRwic': '\xeb\xa5\x93',
'ArabicRoundedHighStopWithFilledCentre': '\xdb\xac',
'CjkUnifiedIdeograph962c': '\xe9\x98\xac',
'HangulSyllableRwib': '\xeb\xa5\x8d',
'ParenthesizedHangulThieuthA': '\xe3\x88\x99',
'CjkUnifiedIdeograph962b': '\xe9\x98\xab',
'UnicodeCharacter0xf51f': '\xef\x94\x9f',
'UnicodeCharacter0xe5a1': '\xee\x96\xa1',
'ArabicLigatureTahWithMeemWithMeemInitialForm': '\xef\xb5\xb3',
'GreekSmallLetterAlphaWithYpogegrammeni': '\xe1\xbe\xb3',
'CjkUnifiedIdeograph6267': '\xe6\x89\xa7',
'HangulSyllableSagg': '\xec\x82\xae',
'CjkUnifiedIdeograph7539': '\xe7\x94\xb9',
'CjkUnifiedIdeograph7538': '\xe7\x94\xb8',
'CjkUnifiedIdeograph7537': '\xe7\x94\xb7',
'RaisedInterpolationMarker': '\xe2\xb8\x86',
'SupersetAboveLeftwardsArrow': '\xe2\xa5\xbb',
'CjkUnifiedIdeograph7534': '\xe7\x94\xb4',
'CjkUnifiedIdeograph7533': '\xe7\x94\xb3',
'CjkUnifiedIdeograph7532': '\xe7\x94\xb2',
'CjkUnifiedIdeograph7531': '\xe7\x94\xb1',
'CjkUnifiedIdeograph7530': '\xe7\x94\xb0',
'HangulSyllableGgi': '\xeb\x81\xbc',
'HangulSyllableSwelb': '\xec\x89\x9b',
'HangulSyllableGgo': '\xea\xbc\xac',
'MinusSignWithRisingDots': '\xe2\xa8\xac',
'CjkUnifiedIdeograph6263': '\xe6\x89\xa3',
'HangulSyllableGge': '\xea\xbb\x98',
'HangulSyllableGgwang': '\xea\xbd\x9d',
'CjkCompatibilityIdeographf9bc': '\xef\xa6\xbc',
'UnicodeCharacter0x17fb': '\xe1\x9f\xbb',
'SyriacLetterMim': '\xdc\xa1',
'UnicodeCharacter0x17fa': '\xe1\x9f\xba',
'UnicodeCharacter0x17ff': '\xe1\x9f\xbf',
'HangulSyllableGgwanj': '\xea\xbd\x8d',
'UnicodeCharacter0x17fd': '\xe1\x9f\xbd',
'UnicodeCharacter0x17fe': '\xe1\x9f\xbe',
'CjkUnifiedIdeograph7fc3': '\xe7\xbf\x83',
'CjkUnifiedIdeograph753f': '\xe7\x94\xbf',
'CjkUnifiedIdeograph753e': '\xe7\x94\xbe',
'CjkUnifiedIdeograph753d': '\xe7\x94\xbd',
'CjkUnifiedIdeograph753c': '\xe7\x94\xbc',
'CjkUnifiedIdeograph753b': '\xe7\x94\xbb',
'CjkUnifiedIdeograph753a': '\xe7\x94\xba',
'RejangLetterMa': '\xea\xa4\xb8',
'UnicodeCharacter0xf87f': '\xef\xa1\xbf',
'DotAbove': '\xcb\x99',
'UnicodeCharacter0x2797': '\xe2\x9e\x97',
'UnicodeCharacter0x2795': '\xe2\x9e\x95',
'ArabicLetterJeemIsolatedForm': '\xef\xba\x9d',
'UnicodeCharacter0x2427': '\xe2\x90\xa7',
'HiraganaLetterU': '\xe3\x81\x86',
'CjkUnifiedIdeograph662a': '\xe6\x98\xaa',
'UnicodeCharacter0xf87c': '\xef\xa1\xbc',
'UnicodeCharacter0xde0c': '\xed\xb8\x8c',
'VaiDigitSeven': '\xea\x98\xa7',
'UnicodeCharacter0x2428': '\xe2\x90\xa8',
'UnicodeCharacter0x2429': '\xe2\x90\xa9',
'KangxiRadicalPig': '\xe2\xbe\x97',
'UnicodeCharacter0xf87b': '\xef\xa1\xbb',
'CjkUnifiedIdeograph81bd': '\xe8\x86\xbd',
'HiraganaLetterA': '\xe3\x81\x82',
'CanadianSyllabicsRaa': '\xe1\x95\x8c',
'HiraganaLetterE': '\xe3\x81\x88',
'HangulSyllableCeuls': '\xec\xb8\xac',
'HiraganaLetterI': '\xe3\x81\x84',
'MultiplicationSignInLeftHalfCircle': '\xe2\xa8\xb4',
'HiraganaLetterN': '\xe3\x82\x93',
'HiraganaLetterO': '\xe3\x81\x8a',
'UnicodeCharacter0xe1b9': '\xee\x86\xb9',
'HangulSyllableCyeonh': '\xec\xb3\x96',
'TamilNumberOneHundred': '\xe0\xaf\xb1',
'ParenthesizedHangulRieul': '\xe3\x88\x83',
'YiSyllableSax': '\xea\x8c\x91',
'CjkUnifiedIdeograph6f59': '\xe6\xbd\x99',
'CjkUnifiedIdeograph6f58': '\xe6\xbd\x98',
'SinhalaLetterMahaapraanaPayanna': '\xe0\xb6\xb5',
'CjkUnifiedIdeograph6f54': '\xe6\xbd\x94',
'CjkUnifiedIdeograph6f57': '\xe6\xbd\x97',
'CjkUnifiedIdeograph6f56': '\xe6\xbd\x96',
'CjkUnifiedIdeograph6f51': '\xe6\xbd\x91',
'CjkUnifiedIdeograph6f50': '\xe6\xbd\x90',
'CjkUnifiedIdeograph6f53': '\xe6\xbd\x93',
'CjkUnifiedIdeograph6f52': '\xe6\xbd\x92',
'GreekCapitalLetterEtaWithPsiliAndVaria': '\xe1\xbc\xaa',
'HangulSyllableCah': '\xec\xb1\x83',
'UnicodeCharacter0xde0e': '\xed\xb8\x8e',
'UnicodeCharacter0xda04': '\xed\xa8\x84',
'UnicodeCharacter0x242d': '\xe2\x90\xad',
'UnicodeCharacter0x242e': '\xe2\x90\xae',
'UnicodeCharacter0x242f': '\xe2\x90\xaf',
'HangulSyllableDyulb': '\xeb\x93\x8b',
'UnicodeCharacter0x242a': '\xe2\x90\xaa',
'UnicodeCharacter0x242b': '\xe2\x90\xab',
'UnicodeCharacter0x242c': '\xe2\x90\xac',
'ArabicLigatureNoonWithZainFinalForm': '\xef\xb2\x8b',
'CjkUnifiedIdeograph8c82': '\xe8\xb2\x82',
'UnicodeCharacter0xde0d': '\xed\xb8\x8d',
'CjkUnifiedIdeograph6f5e': '\xe6\xbd\x9e',
'CjkUnifiedIdeograph6f5d': '\xe6\xbd\x9d',
'CjkUnifiedIdeograph6f5f': '\xe6\xbd\x9f',
'CjkUnifiedIdeograph6f5a': '\xe6\xbd\x9a',
'CjkUnifiedIdeograph6f5c': '\xe6\xbd\x9c',
'CjkUnifiedIdeograph6f5b': '\xe6\xbd\x9b',
'HangulSyllableCeulp': '\xec\xb8\xae',
'HangulSyllableDdyaenh': '\xeb\x96\x8a',
'HangulSyllableDdyaenj': '\xeb\x96\x89',
'HangulSyllableDdyaeng': '\xeb\x96\x99',
'HangulSyllableSeoss': '\xec\x84\xb0',
'CjkUnifiedIdeograph9629': '\xe9\x98\xa9',
'CjkUnifiedIdeograph8f9b': '\xe8\xbe\x9b',
'HangulSyllableSyels': '\xec\x85\xbc',
'HangulSyllableSyelp': '\xec\x85\xbe',
'LepchaLetterGla': '\xe1\xb0\x84',
'HangulSyllableSyelt': '\xec\x85\xbd',
'CjkUnifiedIdeograph4f40': '\xe4\xbd\x80',
'CjkUnifiedIdeograph4f41': '\xe4\xbd\x81',
'CjkUnifiedIdeograph4f42': '\xe4\xbd\x82',
'BugineseLetterJa': '\xe1\xa8\x8d',
'CjkUnifiedIdeograph4f44': '\xe4\xbd\x84',
'CjkUnifiedIdeograph4f45': '\xe4\xbd\x85',
'CjkUnifiedIdeograph4f46': '\xe4\xbd\x86',
'CjkUnifiedIdeograph4f47': '\xe4\xbd\x87',
'CjkUnifiedIdeograph4f48': '\xe4\xbd\x88',
'CjkUnifiedIdeograph4f49': '\xe4\xbd\x89',
'HangulSyllableSyelg': '\xec\x85\xb9',
'UnicodeCharacter0xec92': '\xee\xb2\x92',
'GlagoliticCapitalLetterBigYus': '\xe2\xb0\xa8',
'YiSyllableLyr': '\xea\x87\x99',
'KangxiRadicalOpenBox': '\xe2\xbc\x90',
'CircledOpenCentreEightPointedStar': '\xe2\x9d\x82',
'KannadaDigitNine': '\xe0\xb3\xaf',
'CanadianSyllabicsCarrierRi': '\xe1\x97\x8e',
'HangulSyllableNwags': '\xeb\x86\x97',
'CanadianSyllabicsCarrierRo': '\xe1\x97\x8b',
'TamilLetterLlla': '\xe0\xae\xb4',
'UnicodeCharacter0xebc8': '\xee\xaf\x88',
'CanadianSyllabicsCarrierRa': '\xe1\x97\x8f',
'BraillePatternDots2567': '\xe2\xa1\xb2',
'CanadianSyllabicsCarrierRe': '\xe1\x97\x8c',
'YiRadicalCy': '\xea\x92\xbc',
'CjkUnifiedIdeograph4f4a': '\xe4\xbd\x8a',
'CjkUnifiedIdeograph4f4b': '\xe4\xbd\x8b',
'CjkUnifiedIdeograph4f4c': '\xe4\xbd\x8c',
'CjkUnifiedIdeograph4f4d': '\xe4\xbd\x8d',
'CjkUnifiedIdeograph47ad': '\xe4\x9e\xad',
'CjkUnifiedIdeograph4f4f': '\xe4\xbd\x8f',
'HangulJungseongAraeaeo': '\xe1\x86\x9f',
'MalayalamDigitTwo': '\xe0\xb5\xa8',
'SaurashtraLetterDa': '\xea\xa2\xa3',
'ArabicLetterSeenWithFourDotsAbove': '\xdd\x9c',
'ParenthesizedHangulIeungA': '\xe3\x88\x95',
'UnicodeCharacter0xebc6': '\xee\xaf\x86',
'UnicodeCharacter0xebc7': '\xee\xaf\x87',
'UnicodeCharacter0xe131': '\xee\x84\xb1',
'BoxDrawingsLightQuadrupleDashHorizontal': '\xe2\x94\x88',
'WhiteLeftPointingIndex': '\xe2\x98\x9c',
'GurmukhiLetterTa': '\xe0\xa8\xa4',
'CombiningRetroflexHookBelow': '\xcc\xa2',
'CjkUnifiedIdeograph6ec0': '\xe6\xbb\x80',
'CjkUnifiedIdeograph6ec1': '\xe6\xbb\x81',
'CjkUnifiedIdeograph6ec2': '\xe6\xbb\x82',
'CjkUnifiedIdeograph6ec3': '\xe6\xbb\x83',
'CjkUnifiedIdeograph6ec4': '\xe6\xbb\x84',
'CjkUnifiedIdeograph6ec5': '\xe6\xbb\x85',
'CjkUnifiedIdeograph6ec6': '\xe6\xbb\x86',
'CjkUnifiedIdeograph6ec7': '\xe6\xbb\x87',
'CjkUnifiedIdeograph6ec8': '\xe6\xbb\x88',
'CjkUnifiedIdeograph6ec9': '\xe6\xbb\x89',
'EthiopicSyllableKxo': '\xe1\x8a\xbe',
'CjkUnifiedIdeograph682c': '\xe6\xa0\xac',
'CombiningBreveBelow': '\xcc\xae',
'CjkUnifiedIdeograph670c': '\xe6\x9c\x8c',
'CyrillicSmallLetterClosedLittleYus': '\xea\x99\x99',
'UnicodeCharacter0xdd6c': '\xed\xb5\xac',
'CircledNumberEighteen': '\xe2\x91\xb1',
'BalineseMusicalSymbolLefthandOpenPang': '\xe1\xad\xb8',
'KangxiRadicalWeapon': '\xe2\xbd\x8e',
'YiSyllableBbop': '\xea\x81\xa8',
'KangxiRadicalCompare': '\xe2\xbd\x90',
'CjkUnifiedIdeograph5fd1': '\xe5\xbf\x91',
'HanunooLetterBa': '\xe1\x9c\xaa',
'CircledDigitFour': '\xe2\x91\xa3',
'YiSyllableBbox': '\xea\x81\xa6',
'OriyaLetterU': '\xe0\xac\x89',
'YiSyllableRrur': '\xea\x8e\x8d',
'YiSyllableRrup': '\xea\x8e\x8b',
'YiSyllableRrut': '\xea\x8e\x88',
'CjkUnifiedIdeograph93d5': '\xe9\x8f\x95',
'LatinSmallLetterRWithLineBelow': '\xe1\xb9\x9f',
'YiSyllableRrux': '\xea\x8e\x89',
'UnicodeCharacter0xf261': '\xef\x89\xa1',
'CjkUnifiedIdeograph6eca': '\xe6\xbb\x8a',
'CjkUnifiedIdeograph6ecb': '\xe6\xbb\x8b',
'CjkUnifiedIdeograph6ecc': '\xe6\xbb\x8c',
'CjkUnifiedIdeograph6ecd': '\xe6\xbb\x8d',
'CjkUnifiedIdeograph6ece': '\xe6\xbb\x8e',
'CjkUnifiedIdeograph6ecf': '\xe6\xbb\x8f',
'LatinSmallLetterVWithRightHook': '\xe2\xb1\xb1',
'ExtendedArabicindicDigitSeven': '\xdb\xb7',
'ArabicLetterDalWithDotBelow': '\xda\x8a',
'GlagoliticCapitalLetterRitsi': '\xe2\xb0\x93',
'CjkUnifiedIdeograph93d2': '\xe9\x8f\x92',
'ArabicLigatureHehWithMeemWithMeemInitialForm': '\xef\xb6\x94',
'LeftwardsArrowWithLoop': '\xe2\x86\xab',
'LatinCapitalLetterRWithDotBelowAndMacron': '\xe1\xb9\x9c',
'CjkUnifiedIdeograph808f': '\xe8\x82\x8f',
'CjkUnifiedIdeograph808d': '\xe8\x82\x8d',
'CjkUnifiedIdeograph808e': '\xe8\x82\x8e',
'CjkUnifiedIdeograph808b': '\xe8\x82\x8b',
'CjkUnifiedIdeograph808c': '\xe8\x82\x8c',
'CjkUnifiedIdeograph808a': '\xe8\x82\x8a',
'CjkUnifiedIdeograph8554': '\xe8\x95\x94',
'CjkUnifiedIdeograph8555': '\xe8\x95\x95',
'CjkUnifiedIdeograph8556': '\xe8\x95\x96',
'CjkUnifiedIdeograph8557': '\xe8\x95\x97',
'CjkUnifiedIdeograph8550': '\xe8\x95\x90',
'CjkUnifiedIdeograph8551': '\xe8\x95\x91',
'GreekSmallLetterUpsilonWithPsili': '\xe1\xbd\x90',
'CjkUnifiedIdeograph8553': '\xe8\x95\x93',
'CjkUnifiedIdeograph6dc1': '\xe6\xb7\x81',
'CjkUnifiedIdeograph6dc0': '\xe6\xb7\x80',
'CjkUnifiedIdeograph6dc3': '\xe6\xb7\x83',
'CjkUnifiedIdeograph6dc2': '\xe6\xb7\x82',
'CjkUnifiedIdeograph6dc5': '\xe6\xb7\x85',
'CjkUnifiedIdeograph6dc4': '\xe6\xb7\x84',
'CjkUnifiedIdeograph6dc7': '\xe6\xb7\x87',
'CjkUnifiedIdeograph6dc6': '\xe6\xb7\x86',
'CjkUnifiedIdeograph6dc9': '\xe6\xb7\x89',
'CjkUnifiedIdeograph6dc8': '\xe6\xb7\x88',
'CjkUnifiedIdeograph9f7a': '\xe9\xbd\xba',
'CjkUnifiedIdeograph93d0': '\xe9\x8f\x90',
'NkoDigitNine': '\xdf\x89',
'RightsideArcClockwiseArrow': '\xe2\xa4\xb8',
'HangulSyllablePwanh': '\xed\x90\x8e',
'HangulSyllablePwanj': '\xed\x90\x8d',
'UnicodeCharacter0x173e': '\xe1\x9c\xbe',
'PhagspaMarkShad': '\xea\xa1\xb6',
'HalfwidthHangulLetterWeo': '\xef\xbf\x94',
'HangulSyllableSeunj': '\xec\x8a\xa9',
'CjkUnifiedIdeograph69ed': '\xe6\xa7\xad',
'CjkUnifiedIdeograph6dca': '\xe6\xb7\x8a',
'CjkUnifiedIdeograph6dcc': '\xe6\xb7\x8c',
'CjkUnifiedIdeograph6dcb': '\xe6\xb7\x8b',
'CjkUnifiedIdeograph6dce': '\xe6\xb7\x8e',
'CjkUnifiedIdeograph6dcd': '\xe6\xb7\x8d',
'HangulSyllableNeolg': '\xeb\x84\x91',
'CjkUnifiedIdeograph6dcf': '\xe6\xb7\x8f',
'CjkUnifiedIdeograph8086': '\xe8\x82\x86',
'CjkUnifiedIdeograph8087': '\xe8\x82\x87',
'CjkUnifiedIdeograph8084': '\xe8\x82\x84',
'CjkUnifiedIdeograph8085': '\xe8\x82\x85',
'CjkUnifiedIdeograph8082': '\xe8\x82\x82',
'CjkUnifiedIdeograph8083': '\xe8\x82\x83',
'CjkUnifiedIdeograph8080': '\xe8\x82\x80',
'CjkUnifiedIdeograph8081': '\xe8\x82\x81',
'CjkUnifiedIdeograph855d': '\xe8\x95\x9d',
'CjkUnifiedIdeograph855e': '\xe8\x95\x9e',
'CjkUnifiedIdeograph855f': '\xe8\x95\x9f',
'CjkUnifiedIdeograph855a': '\xe8\x95\x9a',
'CjkUnifiedIdeograph8088': '\xe8\x82\x88',
'CjkUnifiedIdeograph8089': '\xe8\x82\x89',
'UnicodeCharacter0xd9d7': '\xed\xa7\x97',
'UnicodeCharacter0xf874': '\xef\xa1\xb4',
'c/': '\xe2\x8a\x98',
'CjkUnifiedIdeograph5135': '\xe5\x84\xb5',
'UnicodeCharacter0xf872': '\xef\xa1\xb2',
'ParenthesizedDigitTwo': '\xe2\x91\xb5',
'MongolianLetterFa': '\xe1\xa0\xb9',
'UnicodeCharacter0xf870': '\xef\xa1\xb0',
'CjkUnifiedIdeograph7dc6': '\xe7\xb7\x86',
'UnicodeCharacter0xeffb': '\xee\xbf\xbb',
'HangulSyllableSyaenh': '\xec\x84\x86',
'KhmerLetterTa': '\xe1\x9e\x8f',
'KhmerLetterTo': '\xe1\x9e\x91',
'CjkUnifiedIdeograph7dc7': '\xe7\xb7\x87',
'RightAngleDottedSubstitutionMarker': '\xe2\xb8\x81',
'HangulSyllableKulh': '\xec\xbf\xaf',
'CjkUnifiedIdeograph5af8': '\xe5\xab\xb8',
'CjkUnifiedIdeograph5af9': '\xe5\xab\xb9',
'YiSyllableLex': '\xea\x87\x8b',
'CjkUnifiedIdeograph5af2': '\xe5\xab\xb2',
'CjkUnifiedIdeograph5af3': '\xe5\xab\xb3',
'CjkUnifiedIdeograph5af0': '\xe5\xab\xb0',
'CjkUnifiedIdeograph5af1': '\xe5\xab\xb1',
'CjkUnifiedIdeograph5af6': '\xe5\xab\xb6',
'CjkUnifiedIdeograph5af7': '\xe5\xab\xb7',
'CjkUnifiedIdeograph5af4': '\xe5\xab\xb4',
'CjkUnifiedIdeograph5af5': '\xe5\xab\xb5',
'YiSyllableXyt': '\xea\x91\xab',
'YiSyllableKiep': '\xea\x87\xbe',
'YiSyllableXyp': '\xea\x91\xae',
'LatinCapitalLetterAlpha': '\xe2\xb1\xad',
'YiSyllableXyr': '\xea\x91\xb0',
'HangulSyllableKulp': '\xec\xbf\xae',
'HangulSyllableKuls': '\xec\xbf\xac',
'YiSyllableXyx': '\xea\x91\xac',
'HangulSyllableKult': '\xec\xbf\xad',
'TibetanSymbolNorBuBzhiKhyil': '\xe0\xbf\x8c',
'SquareKiroguramu': '\xe3\x8c\x95',
'LatinSmallLetterAWithRingAboveAndAcute': '\xc7\xbb',
'YiSyllableTip': '\xea\x84\x9d',
'CjkUnifiedIdeograph7dc5': '\xe7\xb7\x85',
'HangulSyllableBbeubs': '\xec\x81\xaa',
'HangulSyllableJyaelb': '\xec\x9f\xaf',
'YiSyllableKiex': '\xea\x87\xbc',
'HangulSyllableByuc': '\xeb\xb8\x87',
'HangulSyllableTeong': '\xed\x85\x85',
'UnicodeCharacter0xe80c': '\xee\xa0\x8c',
'CjkUnifiedIdeograph5afb': '\xe5\xab\xbb',
'CjkUnifiedIdeograph5afc': '\xe5\xab\xbc',
'CjkUnifiedIdeograph5afa': '\xe5\xab\xba',
'CjkUnifiedIdeograph5aff': '\xe5\xab\xbf',
'CjkUnifiedIdeograph5afd': '\xe5\xab\xbd',
'CjkUnifiedIdeograph5afe': '\xe5\xab\xbe',
'~~': '\xe2\x89\x88',
'OriyaLetterJa': '\xe0\xac\x9c',
'UnicodeCharacter0xf305': '\xef\x8c\x85',
'HangulSyllablePyugg': '\xed\x93\xaa',
'SubscriptOne': '\xe2\x82\x81',
'CjkCompatibilityIdeographfa84': '\xef\xaa\x84',
'HangulSyllableDdwenh': '\xeb\x9b\x9a',
'Theta': '\xce\x98',
'UnicodeCharacter0xf306': '\xef\x8c\x86',
'HangulSyllablePyugs': '\xed\x93\xab',
'HangulSyllableDad': '\xeb\x8b\xab',
'UnicodeCharacter0xe959': '\xee\xa5\x99',
'UnicodeCharacter0xe958': '\xee\xa5\x98',
'UnicodeCharacter0xe955': '\xee\xa5\x95',
'UnicodeCharacter0xe954': '\xee\xa5\x94',
'UnicodeCharacter0xe957': '\xee\xa5\x97',
'UnicodeCharacter0xe956': '\xee\xa5\x96',
'WhiteFlag': '\xe2\x9a\x90',
'UnicodeCharacter0xe950': '\xee\xa5\x90',
'UnicodeCharacter0xe953': '\xee\xa5\x93',
'UnicodeCharacter0xe952': '\xee\xa5\x92',
'CjkUnifiedIdeograph6c5d': '\xe6\xb1\x9d',
'LaoLetterTo': '\xe0\xba\x95',
'CjkUnifiedIdeograph6c5f': '\xe6\xb1\x9f',
'CjkUnifiedIdeograph7b8b': '\xe7\xae\x8b',
'CjkUnifiedIdeograph7b8e': '\xe7\xae\x8e',
'CjkUnifiedIdeograph6c5a': '\xe6\xb1\x9a',
'CjkUnifiedIdeograph6c5b': '\xe6\xb1\x9b',
'CjkUnifiedIdeograph6c5c': '\xe6\xb1\x9c',
'Alpha': '\xce\x91',
'UnicodeCharacter0xeffd': '\xee\xbf\xbd',
'ArabicLigatureFehWithHahInitialForm': '\xef\xb2\xbf',
'Versicle': '\xe2\x84\xa3',
'~=': '\xe2\x89\x83',
'~>': '\xe2\x86\x9d',
'ArabicLigatureFehWithYehIsolatedForm': '\xef\xb0\xb2',
'CjkUnifiedIdeograph6c54': '\xe6\xb1\x94',
'CjkUnifiedIdeograph6c55': '\xe6\xb1\x95',
'CjkUnifiedIdeograph6c56': '\xe6\xb1\x96',
'CjkUnifiedIdeograph6c57': '\xe6\xb1\x97',
'CjkUnifiedIdeograph6c50': '\xe6\xb1\x90',
'CjkUnifiedIdeograph6c51': '\xe6\xb1\x91',
'CjkUnifiedIdeograph6c52': '\xe6\xb1\x92',
'CjkUnifiedIdeograph6c53': '\xe6\xb1\x93',
'CjkUnifiedIdeograph7b89': '\xe7\xae\x89',
'CjkUnifiedIdeograph7b88': '\xe7\xae\x88',
'YiSyllableKa': '\xea\x88\x81',
'CurlyLogicalAnd': '\xe2\x8b\x8f',
'CjkUnifiedIdeograph6c59': '\xe6\xb1\x99',
'YiSyllableKe': '\xea\x88\x8c',
'HangulSyllableGgeolb': '\xea\xbb\x87',
'UnicodeCharacter0xee8a': '\xee\xba\x8a',
'UnicodeCharacter0xea9b': '\xee\xaa\x9b',
'UnicodeCharacter0xe95e': '\xee\xa5\x9e',
'HangulSyllableGgeolt': '\xea\xbb\x89',
'UnicodeCharacter0xe95f': '\xee\xa5\x9f',
'UnicodeCharacter0xe95a': '\xee\xa5\x9a',
'CopticCapitalLetterGamma': '\xe2\xb2\x84',
'YiSyllableKu': '\xea\x88\x90',
'UnicodeCharacter0xe95b': '\xee\xa5\x9b',
'MyanmarLetterCa': '\xe1\x80\x85',
'CjkUnifiedIdeograph8d09': '\xe8\xb4\x89',
'HangulSyllableCad': '\xec\xb0\xaf',
'DevanagariVowelSignAi': '\xe0\xa5\x88',
'LatinSmallLetterSchwaWithRetroflexHook': '\xe1\xb6\x95',
'CjkUnifiedIdeograph8d02': '\xe8\xb4\x82',
'CjkUnifiedIdeograph8d03': '\xe8\xb4\x83',
'CjkUnifiedIdeograph8d00': '\xe8\xb4\x80',
'CjkUnifiedIdeograph8d01': '\xe8\xb4\x81',
'CjkUnifiedIdeograph8d06': '\xe8\xb4\x86',
'DevanagariVowelSignAa': '\xe0\xa4\xbe',
'CjkUnifiedIdeograph8d04': '\xe8\xb4\x84',
'CjkUnifiedIdeograph8d05': '\xe8\xb4\x85',
'HangulSyllableCap': '\xec\xb1\x82',
'CjkUnifiedIdeograph4c2f': '\xe4\xb0\xaf',
'CjkUnifiedIdeograph4c2e': '\xe4\xb0\xae',
'CjkUnifiedIdeograph4c2d': '\xe4\xb0\xad',
'CjkUnifiedIdeograph4c2c': '\xe4\xb0\xac',
'CjkUnifiedIdeograph4c2b': '\xe4\xb0\xab',
'CjkUnifiedIdeograph4c2a': '\xe4\xb0\xaa',
'DevanagariVowelSignAu': '\xe0\xa5\x8c',
'UnicodeCharacter0x1cee': '\xe1\xb3\xae',
'UnicodeCharacter0x1ced': '\xe1\xb3\xad',
'UnicodeCharacter0x1cec': '\xe1\xb3\xac',
'UnicodeCharacter0x1ceb': '\xe1\xb3\xab',
'HebrewLigatureYiddishVavYod': '\xd7\xb1',
'LatinSmallLetterIWithDotBelow': '\xe1\xbb\x8b',
'ArabicLetterLamInitialForm': '\xef\xbb\x9f',
'HangulSyllableNeod': '\xeb\x84\x8f',
'DevanagariLetterPa': '\xe0\xa4\xaa',
'UnicodeCharacter0xeeea': '\xee\xbb\xaa',
'HangulSyllableCam': '\xec\xb0\xb8',
'HangulSyllablePigs': '\xed\x94\xbf',
'CjkRadicalCsimplifiedTooth': '\xe2\xbb\xae',
'GreekSubscriptSmallLetterGamma': '\xe1\xb5\xa7',
'UnicodeCharacter0xd7b9': '\xed\x9e\xb9',
'CjkUnifiedIdeograph4c27': '\xe4\xb0\xa7',
'CjkUnifiedIdeograph4c26': '\xe4\xb0\xa6',
'CjkUnifiedIdeograph4c25': '\xe4\xb0\xa5',
'CjkUnifiedIdeograph4c24': '\xe4\xb0\xa4',
'CjkUnifiedIdeograph4c23': '\xe4\xb0\xa3',
'CjkUnifiedIdeograph4c22': '\xe4\xb0\xa2',
'CjkUnifiedIdeograph4c21': '\xe4\xb0\xa1',
'UnicodeCharacter0x1ce8': '\xe1\xb3\xa8',
'UnicodeCharacter0x1ce7': '\xe1\xb3\xa7',
'UnicodeCharacter0x1ce6': '\xe1\xb3\xa6',
'UnicodeCharacter0x1ce5': '\xe1\xb3\xa5',
'UnicodeCharacter0x1ce4': '\xe1\xb3\xa4',
'UnicodeCharacter0x1ce3': '\xe1\xb3\xa3',
'UnicodeCharacter0x1ce2': '\xe1\xb3\xa2',
'CjkUnifiedIdeograph4c29': '\xe4\xb0\xa9',
'CjkUnifiedIdeograph4c28': '\xe4\xb0\xa8',
'HangulSyllableSulg': '\xec\x88\xa1',
'LepchaLetterJa': '\xe1\xb0\x88',
'CjkUnifiedIdeograph8d0b': '\xe8\xb4\x8b',
'BalineseDigitFour': '\xe1\xad\x94',
'CjkUnifiedIdeograph8d0a': '\xe8\xb4\x8a',
'CjkUnifiedIdeograph8d0f': '\xe8\xb4\x8f',
'CjkUnifiedIdeograph8d0d': '\xe8\xb4\x8d',
'CjkUnifiedIdeograph8d0e': '\xe8\xb4\x8e',
'HebrewLetterWideLamed': '\xef\xac\xa5',
'HangulSyllableNeos': '\xeb\x84\x9b',
'BalineseLetterDaMadu': '\xe1\xac\xa5',
'KangxiRadicalLong': '\xe2\xbe\xa7',
'HangulSyllableNeop': '\xeb\x84\xa2',
'UnicodeCharacter0xedf2': '\xee\xb7\xb2',
'YiSyllableHmy': '\xea\x82\xaa',
'UnicodeCharacter0x1bad': '\xe1\xae\xad',
'HangulSyllableNeot': '\xeb\x84\xa1',
'UnicodeCharacter0x1bab': '\xe1\xae\xab',
'UnicodeCharacter0x1bac': '\xe1\xae\xac',
'LatinCapitalLetterZWithDescender': '\xe2\xb1\xab',
'CopticCapitalLetterDalda': '\xe2\xb2\x86',
'CjkUnifiedIdeograph7dc8': '\xe7\xb7\x88',
'HangulSyllableSsyaep': '\xec\x8d\xa6',
'CjkUnifiedIdeograph77c2': '\xe7\x9f\x82',
'ChamLetterJha': '\xea\xa8\x8f',
'EthiopicSyllableGgwaa': '\xe1\x8c\x9f',
'HangulSyllableSsyaed': '\xec\x8d\x93',
'HangulSyllableSsyaeg': '\xec\x8d\x8d',
'NotchedUpperRightshadowedWhiteRightwardsArrow': '\xe2\x9e\xb1',
'HangulSyllableSsyaec': '\xec\x8d\xa3',
'HangulSyllableSsyael': '\xec\x8d\x94',
'HangulSyllableSsyaem': '\xec\x8d\x9c',
'HangulSyllableSsyaen': '\xec\x8d\x90',
'HangulSyllableSsyaeh': '\xec\x8d\xa7',
'UpperLeftTriangle': '\xe2\x97\xb8',
'HangulSyllableSsyaek': '\xec\x8d\xa4',
'UnicodeCharacter0xe48d': '\xee\x92\x8d',
'LatinCapitalLetterGWithCaron': '\xc7\xa6',
'HangulSyllableCan': '\xec\xb0\xac',
'UnicodeCharacter0xe92e': '\xee\xa4\xae',
'complement': '\xe2\x88\x81',
'HangulSyllableJjogg': '\xec\xaa\xbe',
'HangulSyllableKwep': '\xed\x80\xb2',
'UnicodeCharacter0xe14d': '\xee\x85\x8d',
'YiSyllableHmu': '\xea\x82\xa5',
'HangulSyllableKwel': '\xed\x80\xa0',
'HangulSyllableKwem': '\xed\x80\xa8',
'HangulSyllableKwen': '\xed\x80\x9c',
'HangulSyllableKweo': '\xec\xbf\xbc',
'HangulSyllableKweh': '\xed\x80\xb3',
'HangulSyllableKwej': '\xed\x80\xae',
'HangulSyllableKwek': '\xed\x80\xb0',
'HangulSyllableKwed': '\xed\x80\x9f',
'UnicodeCharacter0xe14b': '\xee\x85\x8b',
'HangulSyllableKweg': '\xed\x80\x99',
'HangulSyllableGyaenj': '\xea\xb1\x99',
'HangulSyllableKweb': '\xed\x80\xa9',
'HangulSyllableKwec': '\xed\x80\xaf',
'HangulSyllablePeulm': '\xed\x94\x8e',
'HangulSyllablePeulh': '\xed\x94\x93',
'ArabicLetterFeh': '\xd9\x81',
'YiSyllableHmi': '\xea\x82\x93',
'HangulSyllablePeulg': '\xed\x94\x8d',
'UnicodeCharacter0xffbf': '\xef\xbe\xbf',
'HangulSyllableGyec': '\xea\xb3\x9b',
'HangulSyllableCyobs': '\xec\xb6\x8a',
'UnicodeCharacter0xedf6': '\xee\xb7\xb6',
'HangulSyllablePeult': '\xed\x94\x91',
'MongolianLetterSibeHa': '\xe1\xa1\xa5',
'HangulSyllablePeulp': '\xed\x94\x92',
'HangulSyllablePeuls': '\xed\x94\x90',
'CjkUnifiedIdeograph9af6': '\xe9\xab\xb6',
'CjkUnifiedIdeograph9af7': '\xe9\xab\xb7',
'ArabicLigatureLamWithJeemWithMeemInitialForm': '\xef\xb6\xba',
'HangulSyllableSsot': '\xec\x8f\xb1',
'HangulSyllableBbelh': '\xeb\xbb\xbb',
'CjkUnifiedIdeograph9af4': '\xe9\xab\xb4',
'HangulSyllableSsop': '\xec\x8f\xb2',
'HangulSyllableSson': '\xec\x8f\x9c',
'HangulSyllableSsol': '\xec\x8f\xa0',
'CjkUnifiedIdeograph9af5': '\xe9\xab\xb5',
'HangulSyllableSsoj': '\xec\x8f\xae',
'HangulSyllableSoss': '\xec\x86\xa0',
'HangulSyllableSsoh': '\xec\x8f\xb3',
'HangulSyllableGyaenh': '\xea\xb1\x9a',
'CjkUnifiedIdeograph9af2': '\xe9\xab\xb2',
'HangulSyllableSsod': '\xec\x8f\x9f',
'HangulSyllableSsoe': '\xec\x90\xac',
'HangulSyllableSsob': '\xec\x8f\xa9',
'HangulSyllableSsoc': '\xec\x8f\xaf',
'CjkRadicalCsimplifiedWind': '\xe2\xbb\x9b',
'CjkUnifiedIdeograph9af3': '\xe9\xab\xb3',
'VaiSyllableMboo': '\xea\x95\xb9',
'TamilDebitSign': '\xe0\xaf\xb6',
'CjkUnifiedIdeograph9af0': '\xe9\xab\xb0',
'MongolianDigitFive': '\xe1\xa0\x95',
'CombiningTilde': '\xcc\x83',
'CjkUnifiedIdeograph9af1': '\xe9\xab\xb1',
'CjkUnifiedIdeograph90bc': '\xe9\x82\xbc',
'UnicodeCharacter0xa661': '\xea\x99\xa1',
'UnicodeCharacter0xa660': '\xea\x99\xa0',
'HangulSyllableKeuls': '\xed\x81\xb8',
'CanadianSyllabicsThaa': '\xe1\x95\xa7',
'HangulSyllableCyagg': '\xec\xb1\xa2',
'UnicodeCharacter0xe177': '\xee\x85\xb7',
'CjkUnifiedIdeograph9e9f': '\xe9\xba\x9f',
'CjkUnifiedIdeograph9e9e': '\xe9\xba\x9e',
'CjkUnifiedIdeograph9e9d': '\xe9\xba\x9d',
'CjkUnifiedIdeograph9e9c': '\xe9\xba\x9c',
'CjkUnifiedIdeograph9e9b': '\xe9\xba\x9b',
'CjkUnifiedIdeograph9e9a': '\xe9\xba\x9a',
'UnicodeCharacter0xe170': '\xee\x85\xb0',
'CjkUnifiedIdeograph733f': '\xe7\x8c\xbf',
'CjkUnifiedIdeograph78d9': '\xe7\xa3\x99',
'CjkUnifiedIdeograph90bb': '\xe9\x82\xbb',
'CjkUnifiedIdeograph9af8': '\xe9\xab\xb8',
'CjkUnifiedIdeograph7c18': '\xe7\xb0\x98',
'AplFunctionalSymbolDiamondUnderbar': '\xe2\x8d\x9a',
'CjkUnifiedIdeograph9af9': '\xe9\xab\xb9',
'UnicodeCharacter0xdbd2': '\xed\xaf\x92',
'LatinSmallLetterMWithDotAbove': '\xe1\xb9\x81',
'HangulSyllableTinj': '\xed\x8b\xb5',
'CjkUnifiedIdeograph9e99': '\xe9\xba\x99',
'CjkUnifiedIdeograph9e98': '\xe9\xba\x98',
'CjkUnifiedIdeograph9e97': '\xe9\xba\x97',
'CjkUnifiedIdeograph9e96': '\xe9\xba\x96',
'CjkUnifiedIdeograph9e95': '\xe9\xba\x95',
'CjkUnifiedIdeograph9e94': '\xe9\xba\x94',
'CjkUnifiedIdeograph9e93': '\xe9\xba\x93',
'CjkUnifiedIdeograph9e92': '\xe9\xba\x92',
'SinhalaLetterSanyakaGayanna': '\xe0\xb6\x9f',
'CjkUnifiedIdeograph9e90': '\xe9\xba\x90',
'BalineseMusicalSymbolDangSurang': '\xe1\xad\xa5',
'ArabicLetterGaf': '\xda\xaf',
'VaiSyllableNdoleFa': '\xea\x98\x90',
'CjkUnifiedIdeograph89d6': '\xe8\xa7\x96',
'CjkUnifiedIdeograph6a14': '\xe6\xa8\x94',
'LeftCurlyBracketLowerHook': '\xe2\x8e\xa9',
'CjkUnifiedIdeograph600b': '\xe6\x80\x8b',
'CjkUnifiedIdeograph600c': '\xe6\x80\x8c',
'CjkUnifiedIdeograph600d': '\xe6\x80\x8d',
'CjkUnifiedIdeograph600e': '\xe6\x80\x8e',
'CjkUnifiedIdeograph600f': '\xe6\x80\x8f',
'GreekSmallLetterSan': '\xcf\xbb',
'TifinaghLetterYatt': '\xe2\xb5\x9f',
'TifinaghLetterYath': '\xe2\xb5\x9d',
'KangxiRadicalHidingEnclosure': '\xe2\xbc\x96',
'CircledHangulKiyeokA': '\xe3\x89\xae',
'UnicodeCharacter0xdbd0': '\xed\xaf\x90',
'CjkUnifiedIdeograph89d9': '\xe8\xa7\x99',
'SuperscriptFour': '\xe2\x81\xb4',
'ArabicLigatureSheenWithMeemInitialForm': '\xef\xb4\xb0',
'CjkUnifiedIdeograph89d8': '\xe8\xa7\x98',
'LatinCapitalLetterHalfH': '\xe2\xb1\xb5',
'CjkUnifiedIdeograph6aeb': '\xe6\xab\xab',
'ThaiCharacterChoChan': '\xe0\xb8\x88',
'CjkUnifiedIdeograph6aea': '\xe6\xab\xaa',
'CanadianSyllabicsKw': '\xe1\x92\x84',
'CjkUnifiedIdeograph6aed': '\xe6\xab\xad',
'CjkUnifiedIdeograph6aee': '\xe6\xab\xae',
'UnicodeCharacter0xdda7': '\xed\xb6\xa7',
'UnicodeCharacter0xdda6': '\xed\xb6\xa6',
'CanadianSyllabicsKi': '\xe1\x91\xad',
'UnicodeCharacter0xdda4': '\xed\xb6\xa4',
'CanadianSyllabicsKo': '\xe1\x91\xaf',
'UnicodeCharacter0xdda2': '\xed\xb6\xa2',
'UnicodeCharacter0xdda1': '\xed\xb6\xa1',
'UnicodeCharacter0xdda0': '\xed\xb6\xa0',
'CanadianSyllabicsKa': '\xe1\x91\xb2',
'LatinSmallLetterEWithCaron': '\xc4\x9b',
'CanadianSyllabicsKe': '\xe1\x91\xab',
'UnicodeCharacter0xdda8': '\xed\xb6\xa8',
'CjkUnifiedIdeograph97d5': '\xe9\x9f\x95',
'UnicodeCharacter0xd7b4': '\xed\x9e\xb4',
'HangulJongseongCieuc': '\xe1\x86\xbd',
'CjkUnifiedIdeograph9072': '\xe9\x81\xb2',
'KangxiRadicalBristle': '\xe2\xbc\xba',
'CjkUnifiedIdeograph6000': '\xe6\x80\x80',
'CjkUnifiedIdeograph6001': '\xe6\x80\x81',
'CjkUnifiedIdeograph6002': '\xe6\x80\x82',
'CjkUnifiedIdeograph6003': '\xe6\x80\x83',
'CjkUnifiedIdeograph6004': '\xe6\x80\x84',
'CjkUnifiedIdeograph6005': '\xe6\x80\x85',
'CjkUnifiedIdeograph6006': '\xe6\x80\x86',
'CjkUnifiedIdeograph6007': '\xe6\x80\x87',
'CjkUnifiedIdeograph6008': '\xe6\x80\x88',
'CjkUnifiedIdeograph6009': '\xe6\x80\x89',
'HangulSyllableJjalt': '\xec\xa7\xa9',
'UnicodeCharacter0xddaf': '\xed\xb6\xaf',
'UnicodeCharacter0xddae': '\xed\xb6\xae',
'UnicodeCharacter0xddad': '\xed\xb6\xad',
'UnicodeCharacter0xddac': '\xed\xb6\xac',
'CjkUnifiedIdeograph9076': '\xe9\x81\xb6',
'UnicodeCharacter0xddaa': '\xed\xb6\xaa',
'ArabicLetterJeemMedialForm': '\xef\xba\xa0',
'CjkUnifiedIdeograph9078': '\xe9\x81\xb8',
'CjkUnifiedIdeograph6ae8': '\xe6\xab\xa8',
'CjkUnifiedIdeograph6ae9': '\xe6\xab\xa9',
'HangulSyllableCyaenj': '\xec\xb2\x81',
'CjkUnifiedIdeograph9079': '\xe9\x81\xb9',
'CjkUnifiedIdeograph6ae2': '\xe6\xab\xa2',
'CjkUnifiedIdeograph6ae3': '\xe6\xab\xa3',
'CjkUnifiedIdeograph6ae0': '\xe6\xab\xa0',
'CjkUnifiedIdeograph6ae1': '\xe6\xab\xa1',
'CjkUnifiedIdeograph6ae6': '\xe6\xab\xa6',
'CjkUnifiedIdeograph6ae7': '\xe6\xab\xa7',
'CjkUnifiedIdeograph6ae4': '\xe6\xab\xa4',
'CjkUnifiedIdeograph6ae5': '\xe6\xab\xa5',
'UnicodeCharacter0xe0e0': '\xee\x83\xa0',
'UnicodeCharacter0xe0e1': '\xee\x83\xa1',
'UnicodeCharacter0xe0e2': '\xee\x83\xa2',
'CjkUnifiedIdeograph8f6e': '\xe8\xbd\xae',
'UnicodeCharacter0xe0e4': '\xee\x83\xa4',
'UnicodeCharacter0xe0e5': '\xee\x83\xa5',
'UnicodeCharacter0xe0e6': '\xee\x83\xa6',
'UnicodeCharacter0xe0e7': '\xee\x83\xa7',
'UnicodeCharacter0xe0e8': '\xee\x83\xa8',
'CjkUnifiedIdeograph8f6b': '\xe8\xbd\xab',
'CjkUnifiedIdeograph8f6c': '\xe8\xbd\xac',
'HangulSyllableBoebs': '\xeb\xb5\x9a',
'CjkUnifiedIdeograph9588': '\xe9\x96\x88',
'CjkUnifiedIdeograph9589': '\xe9\x96\x89',
'CjkUnifiedIdeograph9584': '\xe9\x96\x84',
'CjkUnifiedIdeograph735f': '\xe7\x8d\x9f',
'UpTack': '\xe2\x8a\xa5',
'CjkUnifiedIdeograph735d': '\xe7\x8d\x9d',
'CjkUnifiedIdeograph735c': '\xe7\x8d\x9c',
'CjkUnifiedIdeograph735b': '\xe7\x8d\x9b',
'CjkUnifiedIdeograph735a': '\xe7\x8d\x9a',
'CjkUnifiedIdeograph9583': '\xe9\x96\x83',
'CjkUnifiedIdeograph4697': '\xe4\x9a\x97',
'CjkUnifiedIdeograph4696': '\xe4\x9a\x96',
'CjkUnifiedIdeograph4695': '\xe4\x9a\x95',
'CjkUnifiedIdeograph4694': '\xe4\x9a\x94',
'CjkUnifiedIdeograph4693': '\xe4\x9a\x93',
'CjkUnifiedIdeograph4692': '\xe4\x9a\x92',
'CjkUnifiedIdeograph4691': '\xe4\x9a\x91',
'CjkUnifiedIdeograph4690': '\xe4\x9a\x90',
'~~=': '\xe2\x89\x8a',
'CjkUnifiedIdeograph4699': '\xe4\x9a\x99',
'CjkUnifiedIdeograph4698': '\xe4\x9a\x98',
'CjkUnifiedIdeograph3ab4': '\xe3\xaa\xb4',
'CjkUnifiedIdeograph3ab5': '\xe3\xaa\xb5',
'CjkUnifiedIdeograph452c': '\xe4\x94\xac',
'CjkUnifiedIdeograph452b': '\xe4\x94\xab',
'CjkUnifiedIdeograph452e': '\xe4\x94\xae',
'CjkUnifiedIdeograph452d': '\xe4\x94\xad',
'CjkUnifiedIdeograph3ab2': '\xe3\xaa\xb2',
'CjkUnifiedIdeograph3ab3': '\xe3\xaa\xb3',
'CjkUnifiedIdeograph3ab8': '\xe3\xaa\xb8',
'CjkUnifiedIdeograph3ab9': '\xe3\xaa\xb9',
'CjkUnifiedIdeograph7359': '\xe7\x8d\x99',
'CjkUnifiedIdeograph7358': '\xe7\x8d\x98',
'CjkUnifiedIdeograph7357': '\xe7\x8d\x97',
'CjkUnifiedIdeograph7356': '\xe7\x8d\x96',
'CjkUnifiedIdeograph7355': '\xe7\x8d\x95',
'CjkUnifiedIdeograph7354': '\xe7\x8d\x94',
'CjkUnifiedIdeograph7353': '\xe7\x8d\x93',
'CjkUnifiedIdeograph7352': '\xe7\x8d\x92',
'CjkUnifiedIdeograph7351': '\xe7\x8d\x91',
'LatinSmallLetterEWithCircumflexAndGrave': '\xe1\xbb\x81',
'MongolianLetterWa': '\xe1\xa0\xb8',
'UnicodeCharacter0xe0eb': '\xee\x83\xab',
'UnicodeCharacter0xe0ec': '\xee\x83\xac',
'UnicodeCharacter0xe0ed': '\xee\x83\xad',
'UnicodeCharacter0xe0ee': '\xee\x83\xae',
'UnicodeCharacter0xe0ef': '\xee\x83\xaf',
'BlackfeatheredRightwardsArrow': '\xe2\x9e\xb5',
'LatinCapitalLetterOWithCaron': '\xc7\x91',
'CjkUnifiedIdeograph4521': '\xe4\x94\xa1',
'CjkUnifiedIdeograph3abe': '\xe3\xaa\xbe',
'CjkUnifiedIdeograph4523': '\xe4\x94\xa3',
'CjkUnifiedIdeograph4522': '\xe4\x94\xa2',
'CjkUnifiedIdeograph4525': '\xe4\x94\xa5',
'CjkUnifiedIdeograph4524': '\xe4\x94\xa4',
'CjkUnifiedIdeograph4527': '\xe4\x94\xa7',
'CjkUnifiedIdeograph4526': '\xe4\x94\xa6',
'CjkUnifiedIdeograph4529': '\xe4\x94\xa9',
'CjkUnifiedIdeograph4528': '\xe4\x94\xa8',
'RomanNumeralFifty': '\xe2\x85\xac',
'CyrillicCapitalLetterA': '\xd0\x90',
'CjkUnifiedIdeograph469f': '\xe4\x9a\x9f',
'CjkUnifiedIdeograph469e': '\xe4\x9a\x9e',
'CjkUnifiedIdeograph469d': '\xe4\x9a\x9d',
'CyrillicCapitalLetterE': '\xd0\xad',
'CjkUnifiedIdeograph469b': '\xe4\x9a\x9b',
'CjkUnifiedIdeograph469a': '\xe4\x9a\x9a',
'CyrillicCapitalLetterI': '\xd0\x98',
'CyrillicCapitalLetterO': '\xd0\x9e',
'GreekProsgegrammeni': '\xe1\xbe\xbe',
'KangxiRadicalEnter': '\xe2\xbc\x8a',
'UnicodeCharacter0xeb00': '\xee\xac\x80',
'LepchaVowelSignAa': '\xe1\xb0\xa6',
'CjkUnifiedIdeograph4eab': '\xe4\xba\xab',
'UnicodeCharacter0xdf8b': '\xed\xbe\x8b',
'UnicodeCharacter0xdf8c': '\xed\xbe\x8c',
'HangulSyllableNyalm': '\xeb\x83\x9a',
'UnicodeCharacter0xdf8f': '\xed\xbe\x8f',
'HangulSyllableNyalh': '\xeb\x83\x9f',
'ArmenianSmallLetterXeh': '\xd5\xad',
'HangulSyllableNyalg': '\xeb\x83\x99',
'HangulSyllableNyalb': '\xeb\x83\x9b',
'SquareHaitu': '\xe3\x8c\xaa',
'UnicodeCharacter0xe146': '\xee\x85\x86',
'CjkUnifiedIdeograph7811': '\xe7\xa0\x91',
'CjkUnifiedIdeograph8419': '\xe8\x90\x99',
'SinhalaVowelSignGayanukitta': '\xe0\xb7\x9f',
'CjkUnifiedIdeograph4eac': '\xe4\xba\xac',
'UnicodeCharacter0xe144': '\xee\x85\x84',
'UnicodeCharacter0xde78': '\xed\xb9\xb8',
'CjkUnifiedIdeograph73dd': '\xe7\x8f\x9d',
'UnicodeCharacter0xe145': '\xee\x85\x85',
'UnicodeCharacter0xe5e0': '\xee\x97\xa0',
'UnicodeCharacter0xde70': '\xed\xb9\xb0',
'UnicodeCharacter0xde71': '\xed\xb9\xb1',
'UnicodeCharacter0xde72': '\xed\xb9\xb2',
'UnicodeCharacter0xde73': '\xed\xb9\xb3',
'UnicodeCharacter0xde74': '\xed\xb9\xb4',
'UnicodeCharacter0xde75': '\xed\xb9\xb5',
'UnicodeCharacter0xde76': '\xed\xb9\xb6',
'UnicodeCharacter0xde77': '\xed\xb9\xb7',
'UnicodeCharacter0xa4cb': '\xea\x93\x8b',
'UnicodeCharacter0xdf83': '\xed\xbe\x83',
'UnicodeCharacter0xdf80': '\xed\xbe\x80',
'HangulSyllableTat': '\xed\x83\x99',
'UnicodeCharacter0xdf86': '\xed\xbe\x86',
'UnicodeCharacter0xdf87': '\xed\xbe\x87',
'UnicodeCharacter0xdf84': '\xed\xbe\x84',
'UnicodeCharacter0xdf85': '\xed\xbe\x85',
'HangulSyllableSsyeng': '\xec\x8f\x91',
'UnicodeCharacter0xdf89': '\xed\xbe\x89',
'O': '\xce\x9f',
'SubsetOfAboveEqualsSign': '\xe2\xab\x85',
'CjkUnifiedIdeograph7818': '\xe7\xa0\x98',
'HangulSyllableTap': '\xed\x83\x9a',
'LatinCapitalLetterAWithBreveAndTilde': '\xe1\xba\xb4',
'UnicodeCharacter0xeb05': '\xee\xac\x85',
'SundaneseLetterMa': '\xe1\xae\x99',
'UnicodeCharacter0xde05': '\xed\xb8\x85',
'UnicodeCharacter0xde7a': '\xed\xb9\xba',
'UnicodeCharacter0xde7b': '\xed\xb9\xbb',
'UnicodeCharacter0xde7c': '\xed\xb9\xbc',
'UnicodeCharacter0xde7d': '\xed\xb9\xbd',
'HangulSyllableNwi': '\xeb\x89\x98',
'UnicodeCharacter0xde7f': '\xed\xb9\xbf',
'CjkUnifiedIdeograph9afe': '\xe9\xab\xbe',
'CjkRadicalEatTwo': '\xe2\xbb\x9e',
'LeftwardsHarpoonWithBarbDownFromBar': '\xe2\xa5\x9e',
'CjkUnifiedIdeograph9afc': '\xe9\xab\xbc',
'HangulSyllableBae': '\xeb\xb0\xb0',
'UnicodeCharacter0xde04': '\xed\xb8\x84',
'HangulSyllableBad': '\xeb\xb0\x9b',
'CjkUnifiedIdeograph9afa': '\xe9\xab\xba',
'HangulSyllableBag': '\xeb\xb0\x95',
'SlantedEqualToOrGreaterthan': '\xe2\xaa\x96',
'CjkUnifiedIdeograph9257': '\xe9\x89\x97',
'VaiSyllableZhoo': '\xea\x96\x8a',
'CjkUnifiedIdeograph88aa': '\xe8\xa2\xaa',
'CjkUnifiedIdeograph88ac': '\xe8\xa2\xac',
'CjkUnifiedIdeograph88ab': '\xe8\xa2\xab',
'CjkUnifiedIdeograph88ae': '\xe8\xa2\xae',
'CjkUnifiedIdeograph88ad': '\xe8\xa2\xad',
'CjkUnifiedIdeograph88af': '\xe8\xa2\xaf',
'UnicodeCharacter0xf4b0': '\xef\x92\xb0',
'UnicodeCharacter0xdc61': '\xed\xb1\xa1',
'UnicodeCharacter0xf4b2': '\xef\x92\xb2',
'UnicodeCharacter0xf4b3': '\xef\x92\xb3',
'UnicodeCharacter0xf4b4': '\xef\x92\xb4',
'HangulSyllableBac': '\xeb\xb0\xab',
'UnicodeCharacter0xf4b6': '\xef\x92\xb6',
'UnicodeCharacter0xdc60': '\xed\xb1\xa0',
'UnicodeCharacter0xf4b8': '\xef\x92\xb8',
'HangulSyllableTac': '\xed\x83\x97',
'HangulSyllableBab': '\xeb\xb0\xa5',
'UnicodeCharacter0xdc63': '\xed\xb1\xa3',
'HangulSyllableKeulb': '\xed\x81\xb7',
'CjkUnifiedIdeograph6585': '\xe6\x96\x85',
'CjkUnifiedIdeograph6584': '\xe6\x96\x84',
'CjkUnifiedIdeograph6587': '\xe6\x96\x87',
'CjkUnifiedIdeograph6586': '\xe6\x96\x86',
'CjkUnifiedIdeograph6581': '\xe6\x96\x81',
'CjkUnifiedIdeograph6580': '\xe6\x96\x80',
'CjkUnifiedIdeograph6583': '\xe6\x96\x83',
'CjkUnifiedIdeograph6582': '\xe6\x96\x82',
'UnicodeCharacter0xa4cf': '\xea\x93\x8f',
'LepchaDigitFive': '\xe1\xb1\x85',
'UnicodeCharacter0xab30': '\xea\xac\xb0',
'CjkUnifiedIdeograph6589': '\xe6\x96\x89',
'CjkUnifiedIdeograph6588': '\xe6\x96\x88',
'UnicodeCharacter0xdc64': '\xed\xb1\xa4',
'CjkUnifiedIdeograph88a9': '\xe8\xa2\xa9',
'CjkUnifiedIdeograph88a8': '\xe8\xa2\xa8',
'UnicodeCharacter0xde06': '\xed\xb8\x86',
'UnicodeCharacter0xdc67': '\xed\xb1\xa7',
'CjkUnifiedIdeograph88a1': '\xe8\xa2\xa1',
'CjkUnifiedIdeograph88a0': '\xe8\xa2\xa0',
'CjkUnifiedIdeograph88a3': '\xe8\xa2\xa3',
'CjkUnifiedIdeograph88a2': '\xe8\xa2\xa2',
'CjkUnifiedIdeograph88a5': '\xe8\xa2\xa5',
'CjkUnifiedIdeograph88a4': '\xe8\xa2\xa4',
'CjkUnifiedIdeograph88a7': '\xe8\xa2\xa7',
'CjkUnifiedIdeograph88a6': '\xe8\xa2\xa6',
'UnicodeCharacter0xdc69': '\xed\xb1\xa9',
'GeorgianLetterEn': '\xe1\x83\x94',
'UnicodeCharacter0xdc68': '\xed\xb1\xa8',
'UnicodeCharacter0xab31': '\xea\xac\xb1',
'CjkUnifiedIdeograph66fd': '\xe6\x9b\xbd',
'HangulSyllableMwabs': '\xeb\xab\x96',
'GeorgianSmallLetterNar': '\xe2\xb4\x8c',
'CombiningDottedAcuteAccent': '\xe1\xb7\x81',
'CjkUnifiedIdeograph658e': '\xe6\x96\x8e',
'CjkUnifiedIdeograph658d': '\xe6\x96\x8d',
'CjkUnifiedIdeograph658f': '\xe6\x96\x8f',
'CjkUnifiedIdeograph658a': '\xe6\x96\x8a',
'NeitherASubsetOfNorEqualTo': '\xe2\x8a\x88',
'CjkUnifiedIdeograph658c': '\xe6\x96\x8c',
'SyriacColonSkewedRight': '\xdc\x87',
'YiSyllableVit': '\xea\x83\xa2',
'HangulChoseongNieunkiyeok': '\xe1\x84\x93',
'UnicodeCharacter0xf4ba': '\xef\x92\xba',
'UnicodeCharacter0xf4bb': '\xef\x92\xbb',
'UnicodeCharacter0xf4bc': '\xef\x92\xbc',
'UnicodeCharacter0xf4bd': '\xef\x92\xbd',
'HangulSyllableCub': '\xec\xb6\xa5',
'UnicodeCharacter0xf4bf': '\xef\x92\xbf',
'GreekSmallReversedLunateSigmaSymbol': '\xcd\xbb',
'UnicodeCharacter0xeb08': '\xee\xac\x88',
'HangulSyllableRwaek': '\xeb\xa2\xac',
'HangulSyllableRwaej': '\xeb\xa2\xaa',
'CjkCompatibilityIdeographf958': '\xef\xa5\x98',
'HangulSyllableRwaeh': '\xeb\xa2\xaf',
'KhmerSignBathamasat': '\xe1\x9f\x93',
'ModifierLetterSmallAin': '\xe1\xb5\x9c',
'HangulSyllableRwael': '\xeb\xa2\x9c',
'HangulSyllableRwaec': '\xeb\xa2\xab',
'HangulSyllableRwaeb': '\xeb\xa2\xa5',
'CjkCompatibilityIdeographf950': '\xef\xa5\x90',
'CjkCompatibilityIdeographf951': '\xef\xa5\x91',
'HangulSyllableRwaeg': '\xeb\xa2\x95',
'HangulSyllableElp': '\xec\x97\x9e',
'HangulSyllableEls': '\xec\x97\x9c',
'HangulSyllableRwaed': '\xeb\xa2\x9b',
'HangulSyllableElm': '\xec\x97\x9a',
'HangulSyllableElh': '\xec\x97\x9f',
'HangulSyllableRwaes': '\xeb\xa2\xa7',
'HangulSyllableElg': '\xec\x97\x99',
'HangulSyllableRwaep': '\xeb\xa2\xae',
'HangulSyllableRwaet': '\xeb\xa2\xad',
'CjkUnifiedIdeograph3a49': '\xe3\xa9\x89',
'~~~': '\xe2\x89\x8b',
'CjkUnifiedIdeograph9250': '\xe9\x89\x90',
'CombiningClockwiseRingOverlay': '\xe2\x83\x99',
'MongolianComma': '\xe1\xa0\x82',
'HangulSyllableKyibs': '\xed\x82\x9a',
'EthiopicSyllableLoa': '\xe2\xb6\x80',
'CjkUnifiedIdeograph8f68': '\xe8\xbd\xa8',
'UnicodeCharacter0xdd63': '\xed\xb5\xa3',
'LatinCapitalLetterLj': '\xc7\x87',
'CjkUnifiedIdeograph8f69': '\xe8\xbd\xa9',
'CjkUnifiedIdeograph8f66': '\xe8\xbd\xa6',
'DoublestruckItalicCapitalD': '\xe2\x85\x85',
'CjkUnifiedIdeograph8f67': '\xe8\xbd\xa7',
'CjkCompatibilityIdeographf95b': '\xef\xa5\x9b',
'SquaredRisingDiagonalSlash': '\xe2\xa7\x84',
'CjkCompatibilityIdeographf95a': '\xef\xa5\x9a',
'CjkCompatibilityIdeographf95f': '\xef\xa5\x9f',
'CjkUnifiedIdeograph8f64': '\xe8\xbd\xa4',
'CjkCompatibilityIdeographf95d': '\xef\xa5\x9d',
'CjkCompatibilityIdeographf95e': '\xef\xa5\x9e',
'UnicodeCharacter0xdf12': '\xed\xbc\x92',
'UnicodeCharacter0xec60': '\xee\xb1\xa0',
'CjkUnifiedIdeograph8f65': '\xe8\xbd\xa5',
'UnicodeCharacter0xdd60': '\xed\xb5\xa0',
'HangulSyllableJjwaess': '\xec\xac\x88',
'CjkUnifiedIdeograph8f62': '\xe8\xbd\xa2',
'RunicLetterShorttwigbjarkanB': '\xe1\x9b\x93',
'CjkUnifiedIdeograph8f63': '\xe8\xbd\xa3',
'CyrillicCapitalLetterCche': '\xea\x9a\x86',
'CjkUnifiedIdeograph8f60': '\xe8\xbd\xa0',
'LessthanAndSinglelineNotEqualTo': '\xe2\xaa\x87',
'HangulSyllableJyobs': '\xec\xa3\xb2',
'CjkUnifiedIdeograph8f61': '\xe8\xbd\xa1',
'HangulSyllableRyels': '\xeb\xa1\x8c',
'CjkUnifiedIdeograph8669': '\xe8\x99\xa9',
'finalsigma': '\xcf\x82',
'ChamLetterRa': '\xea\xa8\xa3',
'LatinSmallLetterLWithAcute': '\xc4\xba',
'GreekPerispomeni': '\xe1\xbf\x80',
'HangulSyllableBbwang': '\xeb\xbd\xb1',
'TamilDigitZero': '\xe0\xaf\xa6',
'HangulSyllableJjolh': '\xec\xab\x8b',
'HangulSyllableSong': '\xec\x86\xa1',
'HangulSyllableSonh': '\xec\x86\x92',
'HangulSyllableSonj': '\xec\x86\x91',
'CjkUnifiedIdeograph6e92': '\xe6\xba\x92',
'NewTaiLueLetterHighMa': '\xe1\xa6\x96',
'CjkUnifiedIdeograph6e90': '\xe6\xba\x90',
'CjkUnifiedIdeograph6e91': '\xe6\xba\x91',
'CjkUnifiedIdeograph6e96': '\xe6\xba\x96',
'CjkUnifiedIdeograph6e97': '\xe6\xba\x97',
'CjkUnifiedIdeograph6e94': '\xe6\xba\x94',
'CjkUnifiedIdeograph6e95': '\xe6\xba\x95',
'UnicodeCharacter0xdaf5': '\xed\xab\xb5',
'CjkUnifiedIdeograph6e98': '\xe6\xba\x98',
'CjkUnifiedIdeograph6e99': '\xe6\xba\x99',
'HalfwidthHangulLetterIeung': '\xef\xbe\xb7',
'CjkUnifiedIdeograph40a9': '\xe4\x82\xa9',
'UnicodeCharacter0x9ba': '\xe0\xa6\xba',
'UnicodeCharacter0x9bb': '\xe0\xa6\xbb',
'CjkUnifiedIdeograph40a5': '\xe4\x82\xa5',
'CjkUnifiedIdeograph40a4': '\xe4\x82\xa4',
'CjkUnifiedIdeograph40a7': '\xe4\x82\xa7',
'CjkUnifiedIdeograph40a6': '\xe4\x82\xa6',
'CjkUnifiedIdeograph40a1': '\xe4\x82\xa1',
'CjkUnifiedIdeograph40a0': '\xe4\x82\xa0',
'CjkUnifiedIdeograph40a3': '\xe4\x82\xa3',
'CjkUnifiedIdeograph40a2': '\xe4\x82\xa2',
'HangulSyllableJyung': '\xec\xa6\x81',
'ParenthesizedIdeographHave': '\xe3\x88\xb2',
'HangulSyllableTogg': '\xed\x86\xa2',
'HangulSyllableGgyeong': '\xea\xbc\x89',
'HangulSyllableGgyeonh': '\xea\xbb\xba',
'HangulSyllableElb': '\xec\x97\x9b',
'HangulSyllableGgyeonj': '\xea\xbb\xb9',
'HangulSyllableCat': '\xec\xb1\x81',
'GujaratiLetterKa': '\xe0\xaa\x95',
'HangulSyllableTogs': '\xed\x86\xa3',
'CopticSmallLetterKhei': '\xcf\xa7',
'HangulSyllableJoel': '\xec\xa3\x8c',
'UnicodeCharacter0xdc6b': '\xed\xb1\xab',
'HangulSyllableDdyeong': '\xeb\x97\xad',
'VaiDigitTwo': '\xea\x98\xa2',
'HangulSyllableDdyeonj': '\xeb\x97\x9d',
'ArabicLetterSeenWithExtendedArabicindicDigitFourAbove': '\xdd\xbd',
'HangulSyllableDdyeonh': '\xeb\x97\x9e',
'UnicodeCharacter0xdd64': '\xed\xb5\xa4',
'UnicodeCharacter0x9b4': '\xe0\xa6\xb4',
'UnicodeCharacter0x9b5': '\xe0\xa6\xb5',
'UnicodeCharacter0x9b1': '\xe0\xa6\xb1',
'UnicodeCharacter0x9b3': '\xe0\xa6\xb3',
'CjkUnifiedIdeograph40ae': '\xe4\x82\xae',
'CjkUnifiedIdeograph40ad': '\xe4\x82\xad',
'CjkUnifiedIdeograph7e13': '\xe7\xb8\x93',
'HangulLetterYoyae': '\xe3\x86\x88',
'CjkUnifiedIdeograph40aa': '\xe4\x82\xaa',
'SymbolForSubstituteFormTwo': '\xe2\x90\xa6',
'CjkUnifiedIdeograph40ac': '\xe4\x82\xac',
'CjkUnifiedIdeograph40ab': '\xe4\x82\xab',
'CjkUnifiedIdeograph6e9b': '\xe6\xba\x9b',
'CjkUnifiedIdeograph6e9c': '\xe6\xba\x9c',
'CjkUnifiedIdeograph6e9a': '\xe6\xba\x9a',
'ArabicLetterRehWithRing': '\xda\x93',
'CjkUnifiedIdeograph6e9d': '\xe6\xba\x9d',
'CjkUnifiedIdeograph6e9e': '\xe6\xba\x9e',
'UnicodeCharacter0xddba': '\xed\xb6\xba',
'UnicodeCharacter0xda8e': '\xed\xaa\x8e',
'YiSyllableNjix': '\xea\x90\xb2',
'VaiSyllableWeen': '\xea\x94\x84',
'CjkUnifiedIdeograph5a0d': '\xe5\xa8\x8d',
'BengaliDigitNine': '\xe0\xa7\xaf',
'CjkUnifiedIdeograph5a0f': '\xe5\xa8\x8f',
'CjkUnifiedIdeograph5a0a': '\xe5\xa8\x8a',
'CjkUnifiedIdeograph5a0b': '\xe5\xa8\x8b',
'CjkUnifiedIdeograph5a0c': '\xe5\xa8\x8c',
'WhiteSesameDot': '\xef\xb9\x86',
'SymbolForSubstitute': '\xe2\x90\x9a',
'LatinCapitalLetterTWithCaron': '\xc5\xa4',
'EthiopicSyllableNyaa': '\xe1\x8a\x9b',
'UnicodeCharacter0xe49d': '\xee\x92\x9d',
'UnicodeCharacter0xf11d': '\xef\x84\x9d',
'UnicodeCharacter0xf11e': '\xef\x84\x9e',
'UnicodeCharacter0xf11f': '\xef\x84\x9f',
'HangulSyllableJjolg': '\xec\xab\x85',
'UnicodeCharacter0xf11a': '\xef\x84\x9a',
'TaiLeLetterKa': '\xe1\xa5\x90',
'UnicodeCharacter0xf11c': '\xef\x84\x9c',
'HangulSyllableHwing': '\xed\x9c\xad',
'CjkUnifiedIdeograph5a08': '\xe5\xa8\x88',
'CjkUnifiedIdeograph5a09': '\xe5\xa8\x89',
'BoxDrawingsDownDoubleAndLeftSingle': '\xe2\x95\x96',
'CjkUnifiedIdeograph5a04': '\xe5\xa8\x84',
'CjkUnifiedIdeograph5a05': '\xe5\xa8\x85',
'CjkUnifiedIdeograph5a06': '\xe5\xa8\x86',
'CjkUnifiedIdeograph5a07': '\xe5\xa8\x87',
'CjkUnifiedIdeograph5a00': '\xe5\xa8\x80',
'CjkUnifiedIdeograph5a01': '\xe5\xa8\x81',
'CjkUnifiedIdeograph5a02': '\xe5\xa8\x82',
'IdeographicAnnotationThreeMark': '\xe3\x86\x94',
'HangulSyllableMeuc': '\xeb\xaf\x97',
'UnicodeCharacter0xdfda': '\xed\xbf\x9a',
'RightwardsHarpoonOverLeftwardsHarpoon': '\xe2\x87\x8c',
'TibetanSignRdelNagGnyis': '\xe0\xbc\x9e',
'Digamma': '\xcf\x9c',
'UnicodeCharacter0xe49f': '\xee\x92\x9f',
'YiSyllableHxep': '\xea\x89\x9c',
'UnicodeCharacter0xe050': '\xee\x81\x90',
'EthiopicSyllableGlottalWa': '\xe1\x8a\xa7',
'HangulSyllableHeoc': '\xed\x97\x9f',
'YiSyllableHxex': '\xea\x89\x9a',
'HangulSyllableHeob': '\xed\x97\x99',
'UnicodeCharacter0xf118': '\xef\x84\x98',
'UnicodeCharacter0xf119': '\xef\x84\x99',
'UnicodeCharacter0xf114': '\xef\x84\x94',
'UnicodeCharacter0xf115': '\xef\x84\x95',
'GlagoliticSmallLetterYati': '\xe2\xb1\x91',
'UnicodeCharacter0xf117': '\xef\x84\x97',
'UnicodeCharacter0xf110': '\xef\x84\x90',
'UnicodeCharacter0xf111': '\xef\x84\x91',
'UnicodeCharacter0xf112': '\xef\x84\x92',
'HangulSyllableHeod': '\xed\x97\x8f',
'CjkStrokeP': '\xe3\x87\x92',
'CjkStrokeQ': '\xe3\x87\xa3',
'CjkStrokeS': '\xe3\x87\x91',
'CjkStrokeT': '\xe3\x87\x80',
'HangulSyllableHeog': '\xed\x97\x89',
'CjkUnifiedIdeograph993b': '\xe9\xa4\xbb',
'UnicodeCharacter0xe5e4': '\xee\x97\xa4',
'CjkStrokeD': '\xe3\x87\x94',
'CjkStrokeH': '\xe3\x87\x90',
'ZNotationRelationalComposition': '\xe2\xa8\xbe',
'HangulSyllableHeok': '\xed\x97\xa0',
'CjkStrokeN': '\xe3\x87\x8f',
'UnicodeCharacter0xe64': '\xe0\xb9\xa4',
'UnicodeCharacter0xe65': '\xe0\xb9\xa5',
'UnicodeCharacter0xe66': '\xe0\xb9\xa6',
'UnicodeCharacter0xe67': '\xe0\xb9\xa7',
'UnicodeCharacter0xe60': '\xe0\xb9\xa0',
'UnicodeCharacter0xe61': '\xe0\xb9\xa1',
'SundaneseVowelSignPanghulu': '\xe1\xae\xa4',
'UnicodeCharacter0xe63': '\xe0\xb9\xa3',
'ChamLetterJa': '\xea\xa8\x8e',
'UnicodeCharacter0xe68': '\xe0\xb9\xa8',
'UnicodeCharacter0xe69': '\xe0\xb9\xa9',
'HangulSyllableHeol': '\xed\x97\x90',
'CjkUnifiedIdeograph81e0': '\xe8\x87\xa0',
'DentistrySymbolLightVerticalAndBottomLeft': '\xe2\x8f\x8c',
'CjkUnifiedIdeograph81e1': '\xe8\x87\xa1',
'MongolianLetterLha': '\xe1\xa1\x80',
'CjkUnifiedIdeograph81e2': '\xe8\x87\xa2',
'HangulSyllableSwap': '\xec\x87\x82',
'HangulSyllableJjolb': '\xec\xab\x87',
'HangulSyllableSwas': '\xec\x86\xbb',
'CircledIdeographOne': '\xe3\x8a\x80',
'CjkUnifiedIdeograph9692': '\xe9\x9a\x92',
'HangulSyllableDdaelp': '\xeb\x95\x9a',
'HangulSyllableDdaels': '\xeb\x95\x98',
'HangulSyllableDdaelt': '\xeb\x95\x99',
'CjkUnifiedIdeograph81e5': '\xe8\x87\xa5',
'HangulSyllableDdaelh': '\xeb\x95\x9b',
'HangulSyllableSwab': '\xec\x86\xb9',
'HangulSyllableSwac': '\xec\x86\xbf',
'HangulSyllableDdaelm': '\xeb\x95\x96',
'CjkUnifiedIdeograph81e6': '\xe8\x87\xa6',
'HangulSyllableSwag': '\xec\x86\xa9',
'HangulSyllableSwah': '\xec\x87\x83',
'HangulSyllableSwaj': '\xec\x86\xbe',
'CjkUnifiedIdeograph81e7': '\xe8\x87\xa7',
'HangulSyllableSwal': '\xec\x86\xb0',
'HangulSyllableSwam': '\xec\x86\xb8',
'HangulSyllableDdaelg': '\xeb\x95\x95',
'Lambda': '\xce\x9b',
'ArabicLigatureThehWithNoonFinalForm': '\xef\xb1\xb9',
'CjkUnifiedIdeograph9698': '\xe9\x9a\x98',
'UnicodeCharacter0xf98': '\xe0\xbe\x98',
'UnicodeCharacter0xddb9': '\xed\xb6\xb9',
'RightwardsArrowWithVerticalStroke': '\xe2\x87\xb8',
'CjkUnifiedIdeograph9699': '\xe9\x9a\x99',
'TurnedAmpersand': '\xe2\x85\x8b',
'UnicodeCharacter0xe6e': '\xe0\xb9\xae',
'UnicodeCharacter0xe6f': '\xe0\xb9\xaf',
'ArabicLigatureYehWithMeemInitialForm': '\xef\xb3\x9d',
'UnicodeCharacter0xe6a': '\xe0\xb9\xaa',
'UnicodeCharacter0xe6b': '\xe0\xb9\xab',
'UnicodeCharacter0xe6c': '\xe0\xb9\xac',
'HangulSyllableByilp': '\xeb\xb8\xb6',
'RunicLetterGar': '\xe1\x9a\xb8',
'YiSyllableMgurx': '\xea\x89\x84',
'**n': '\xe2\x81\xbf',
'CjkUnifiedIdeograph5d7b': '\xe5\xb5\xbb',
'NewTaiLueLetterLowVa': '\xe1\xa6\x9e',
'CjkUnifiedIdeograph6ffa': '\xe6\xbf\xba',
'CjkUnifiedIdeograph6ffb': '\xe6\xbf\xbb',
'CjkUnifiedIdeograph6ffc': '\xe6\xbf\xbc',
'CjkUnifiedIdeograph6ffd': '\xe6\xbf\xbd',
'CjkUnifiedIdeograph6ffe': '\xe6\xbf\xbe',
'CjkUnifiedIdeograph6fff': '\xe6\xbf\xbf',
'TeluguLetterBa': '\xe0\xb0\xac',
'HangulSyllableDdyolh': '\xeb\x9a\x8f',
'HangulSyllableDdyolm': '\xeb\x9a\x8a',
'PostalMarkFace': '\xe3\x80\xa0',
'CjkUnifiedIdeograph832c': '\xe8\x8c\xac',
'CjkUnifiedIdeograph832b': '\xe8\x8c\xab',
'CjkUnifiedIdeograph832a': '\xe8\x8c\xaa',
'UnicodeCharacter0xf4a3': '\xef\x92\xa3',
'HangulSyllableDdyolg': '\xeb\x9a\x89',
'CjkUnifiedIdeograph832f': '\xe8\x8c\xaf',
'CjkUnifiedIdeograph832e': '\xe8\x8c\xae',
'CjkUnifiedIdeograph832d': '\xe8\x8c\xad',
'HangulSyllableMij': '\xeb\xb0\x8e',
'HangulSyllableMik': '\xeb\xb0\x90',
'HangulSyllableMih': '\xeb\xb0\x93',
'HangulSyllableMin': '\xeb\xaf\xbc',
'HangulSyllableMil': '\xeb\xb0\x80',
'HangulSyllableMim': '\xeb\xb0\x88',
'HangulSyllableDdyols': '\xeb\x9a\x8c',
'HangulSyllableMic': '\xeb\xb0\x8f',
'FullwidthNumberSign': '\xef\xbc\x83',
'EthiopicSyllableSsaa': '\xe2\xb6\xa3',
'LatinCapitalLetterSaltillo': '\xea\x9e\x8b',
'HangulSyllableMig': '\xeb\xaf\xb9',
'HangulSyllableMid': '\xeb\xaf\xbf',
'HangulSyllableDdyolt': '\xeb\x9a\x8d',
'CjkUnifiedIdeograph6ff0': '\xe6\xbf\xb0',
'CjkUnifiedIdeograph6ff1': '\xe6\xbf\xb1',
'ArabicLetterYehBarree': '\xdb\x92',
'CjkUnifiedIdeograph6ff3': '\xe6\xbf\xb3',
'CjkUnifiedIdeograph6ff4': '\xe6\xbf\xb4',
'CjkUnifiedIdeograph6ff5': '\xe6\xbf\xb5',
'CjkUnifiedIdeograph6ff6': '\xe6\xbf\xb6',
'CjkUnifiedIdeograph6ff7': '\xe6\xbf\xb7',
'CjkUnifiedIdeograph6ff8': '\xe6\xbf\xb8',
'CjkUnifiedIdeograph6ff9': '\xe6\xbf\xb9',
'HangulSyllablePuj': '\xed\x92\x8e',
'CloseUp': '\xe2\x81\x90',
'MongolianLetterManchuAliGaliSsa': '\xe1\xa2\xa2',
'BoxDrawingsLightTripleDashHorizontal': '\xe2\x94\x84',
'HangulSyllableBbweos': '\xeb\xbf\xbb',
'NewTaiLueVowelSignAe': '\xe1\xa6\xb6',
'HangulSyllableTwils': '\xed\x8a\x8c',
'HangulSyllableTwilp': '\xed\x8a\x8e',
'HangulSyllableYelt': '\xec\x98\x95',
'NewTaiLueVowelSignAa': '\xe1\xa6\xb1',
'HangulSyllableTwilt': '\xed\x8a\x8d',
'HangulSyllableYelp': '\xec\x98\x96',
'CombiningLatinSmallLetterInsularD': '\xe1\xb7\x98',
'UnicodeCharacter0xf87d': '\xef\xa1\xbd',
'HangulSyllableYelg': '\xec\x98\x91',
'CjkUnifiedIdeograph8329': '\xe8\x8c\xa9',
'CjkUnifiedIdeograph8328': '\xe8\x8c\xa8',
'HangulSyllableYelb': '\xec\x98\x93',
'SundaneseLetterRa': '\xe1\xae\x9b',
'CjkUnifiedIdeograph8323': '\xe8\x8c\xa3',
'CjkUnifiedIdeograph8322': '\xe8\x8c\xa2',
'CjkUnifiedIdeograph8321': '\xe8\x8c\xa1',
'CjkUnifiedIdeograph8320': '\xe8\x8c\xa0',
'NewTaiLueVowelSignAy': '\xe1\xa6\xba',
'CjkUnifiedIdeograph8326': '\xe8\x8c\xa6',
'CjkUnifiedIdeograph8325': '\xe8\x8c\xa5',
'CjkUnifiedIdeograph8324': '\xe8\x8c\xa4',
'CjkUnifiedIdeograph638c': '\xe6\x8e\x8c',
'UnicodeCharacter0xf7f9': '\xef\x9f\xb9',
'UnicodeCharacter0xf7f8': '\xef\x9f\xb8',
'CjkUnifiedIdeograph638b': '\xe6\x8e\x8b',
'CjkUnifiedIdeograph974b': '\xe9\x9d\x8b',
'CjkUnifiedIdeograph974c': '\xe9\x9d\x8c',
'UnicodeCharacter0xf7f1': '\xef\x9f\xb1',
'CjkUnifiedIdeograph974a': '\xe9\x9d\x8a',
'CjkUnifiedIdeograph974f': '\xe9\x9d\x8f',
'UnicodeCharacter0xf7f6': '\xef\x9f\xb6',
'CjkUnifiedIdeograph974d': '\xe9\x9d\x8d',
'CjkUnifiedIdeograph974e': '\xe9\x9d\x8e',
'CjkUnifiedIdeograph563b': '\xe5\x98\xbb',
'CjkUnifiedIdeograph563c': '\xe5\x98\xbc',
'HangulSyllableGeubs': '\xea\xb8\x8a',
'CjkUnifiedIdeograph563a': '\xe5\x98\xba',
'CjkUnifiedIdeograph563f': '\xe5\x98\xbf',
'CjkUnifiedIdeograph563d': '\xe5\x98\xbd',
'CjkUnifiedIdeograph563e': '\xe5\x98\xbe',
'EthiopicSyllableRo': '\xe1\x88\xae',
'YiSyllableHmuo': '\xea\x82\x9d',
'VaiSyllableMi': '\xea\x95\x86',
'CjkUnifiedIdeograph8df8': '\xe8\xb7\xb8',
'CjkUnifiedIdeograph8df9': '\xe8\xb7\xb9',
'EthiopicSyllableRi': '\xe1\x88\xaa',
'CjkUnifiedIdeograph8df4': '\xe8\xb7\xb4',
'CjkUnifiedIdeograph8df5': '\xe8\xb7\xb5',
'EthiopicSyllableRe': '\xe1\x88\xad',
'CjkUnifiedIdeograph8df7': '\xe8\xb7\xb7',
'CjkUnifiedIdeograph8df0': '\xe8\xb7\xb0',
'CjkUnifiedIdeograph8df1': '\xe8\xb7\xb1',
'EthiopicSyllableRa': '\xe1\x88\xa8',
'CjkUnifiedIdeograph8df3': '\xe8\xb7\xb3',
'ParenthesizedIdeographCongratulation': '\xe3\x88\xb7',
'HangulLetterNieuntikeut': '\xe3\x85\xa6',
'YiSyllableHmux': '\xea\x82\xa4',
'EthiopicSyllableRu': '\xe1\x88\xa9',
'CjkUnifiedIdeograph7c6a': '\xe7\xb1\xaa',
'YiSyllableHmup': '\xea\x82\xa6',
'VaiSyllableMu': '\xea\x96\xb7',
'CjkUnifiedIdeograph5638': '\xe5\x98\xb8',
'CjkUnifiedIdeograph5639': '\xe5\x98\xb9',
'CanadianSyllabicsYwoo': '\xe1\x94\xb7',
'CjkUnifiedIdeograph5632': '\xe5\x98\xb2',
'CjkUnifiedIdeograph5633': '\xe5\x98\xb3',
'CjkUnifiedIdeograph5630': '\xe5\x98\xb0',
'CjkUnifiedIdeograph5631': '\xe5\x98\xb1',
'CjkUnifiedIdeograph5636': '\xe5\x98\xb6',
'CjkUnifiedIdeograph5637': '\xe5\x98\xb7',
'CjkUnifiedIdeograph5634': '\xe5\x98\xb4',
'CjkUnifiedIdeograph5635': '\xe5\x98\xb5',
'CjkUnifiedIdeograph9748': '\xe9\x9d\x88',
'CjkUnifiedIdeograph9749': '\xe9\x9d\x89',
'UnicodeCharacter0xea75': '\xee\xa9\xb5',
'HangulSyllableSsyeolh': '\xec\x8e\xaf',
'CjkUnifiedIdeograph9742': '\xe9\x9d\x82',
'CombiningBridgeBelow': '\xcc\xaa',
'CjkUnifiedIdeograph9740': '\xe9\x9d\x80',
'CjkUnifiedIdeograph9741': '\xe9\x9d\x81',
'CjkUnifiedIdeograph9746': '\xe9\x9d\x86',
'CjkUnifiedIdeograph9747': '\xe9\x9d\x87',
'CjkUnifiedIdeograph9744': '\xe9\x9d\x84',
'CjkUnifiedIdeograph969a': '\xe9\x9a\x9a',
'CjkUnifiedIdeograph79cc': '\xe7\xa7\x8c',
'CjkUnifiedIdeograph969b': '\xe9\x9a\x9b',
'UnicodeCharacter0x5ee': '\xd7\xae',
'UnicodeCharacter0x5ed': '\xd7\xad',
'UnicodeCharacter0x5ef': '\xd7\xaf',
'ThaanaEbefili': '\xde\xac',
'UnicodeCharacter0x5eb': '\xd7\xab',
'LowerThreeEighthsBlock': '\xe2\x96\x83',
'NegativeCircledNumberFifteen': '\xe2\x93\xaf',
'UnicodeCharacter0xd9a7': '\xed\xa6\xa7',
'CjkUnifiedIdeograph8dfd': '\xe8\xb7\xbd',
'CjkUnifiedIdeograph8dfe': '\xe8\xb7\xbe',
'CjkUnifiedIdeograph8dff': '\xe8\xb7\xbf',
'SyriacPthahaBelow': '\xdc\xb1',
'CjkUnifiedIdeograph8dfa': '\xe8\xb7\xba',
'CjkUnifiedIdeograph8dfb': '\xe8\xb7\xbb',
'CjkUnifiedIdeograph8dfc': '\xe8\xb7\xbc',
'BlackMediumSmallSquare': '\xe2\x97\xbe',
'CjkUnifiedIdeograph5235': '\xe5\x88\xb5',
'OlChikiDigitSix': '\xe1\xb1\x96',
'VaiSyllableMee': '\xea\x94\xa1',
'ParenthesizedNumberTwelve': '\xe2\x91\xbf',
'CjkUnifiedIdeograph5958': '\xe5\xa5\x98',
'HeavyLeftpointingAngleQuotationMarkOrnament': '\xe2\x9d\xae',
'CjkUnifiedIdeograph4fe3': '\xe4\xbf\xa3',
'CjkUnifiedIdeograph4fe2': '\xe4\xbf\xa2',
'UnicodeCharacter0x31ee': '\xe3\x87\xae',
'CjkUnifiedIdeograph4fe0': '\xe4\xbf\xa0',
'CjkUnifiedIdeograph4fe7': '\xe4\xbf\xa7',
'CjkUnifiedIdeograph4fe6': '\xe4\xbf\xa6',
'CjkUnifiedIdeograph4fe5': '\xe4\xbf\xa5',
'CjkUnifiedIdeograph4fe4': '\xe4\xbf\xa4',
'CjkUnifiedIdeograph4fe9': '\xe4\xbf\xa9',
'CjkUnifiedIdeograph4fe8': '\xe4\xbf\xa8',
'ReversedTilde': '\xe2\x88\xbd',
'HangulSyllableJjwem': '\xec\xae\x90',
'HangulSyllableJyags': '\xec\x9f\x8b',
'ArabicLetterTehMarbutaGoal': '\xdb\x83',
'CjkUnifiedIdeograph5951': '\xe5\xa5\x91',
'LatinCapitalLetterOWithStrokeAndAcute': '\xc7\xbe',
'EthiopicQuestionMark': '\xe1\x8d\xa7',
'UnicodeCharacter0xd95b': '\xed\xa5\x9b',
'LeftwardsTwoheadedTripleDashArrow': '\xe2\xac\xb7',
'HangulSyllableBwaegs': '\xeb\xb4\xaf',
'CyrillicSmallLetterKsi': '\xd1\xaf',
'HangulSyllableTa': '\xed\x83\x80',
'HeavyEightPointedRectilinearBlackStar': '\xe2\x9c\xb8',
'HangulSyllableBwaegg': '\xeb\xb4\xae',
'UnicodeCharacter0xd95d': '\xed\xa5\x9d',
'UnicodeCharacter0xd95e': '\xed\xa5\x9e',
'UnicodeCharacter0x31e7': '\xe3\x87\xa7',
'CjkUnifiedIdeograph4feb': '\xe4\xbf\xab',
'CjkUnifiedIdeograph4fea': '\xe4\xbf\xaa',
'UnicodeCharacter0x31e4': '\xe3\x87\xa4',
'CjkUnifiedIdeograph4fef': '\xe4\xbf\xaf',
'CjkUnifiedIdeograph4fee': '\xe4\xbf\xae',
'CjkUnifiedIdeograph4fed': '\xe4\xbf\xad',
'UnicodeCharacter0x31e9': '\xe3\x87\xa9',
'UnicodeCharacter0x31e8': '\xe3\x87\xa8',
'HangulSyllableIng': '\xec\x9e\x89',
'HangulSyllableHyogg': '\xed\x9a\xaa',
'CjkUnifiedIdeograph85df': '\xe8\x97\x9f',
'CjkUnifiedIdeograph85de': '\xe8\x97\x9e',
'CjkUnifiedIdeograph85dd': '\xe8\x97\x9d',
'CjkUnifiedIdeograph85dc': '\xe8\x97\x9c',
'CjkUnifiedIdeograph85db': '\xe8\x97\x9b',
'RightCurlyBracketUpperHook': '\xe2\x8e\xab',
'HangulSyllableJjoegs': '\xec\xac\x93',
'HangulSyllableHyogs': '\xed\x9a\xab',
'UnicodeCharacter0xdcb0': '\xed\xb2\xb0',
'UnicodeCharacter0xdaf8': '\xed\xab\xb8',
'CjkUnifiedIdeograph81ea': '\xe8\x87\xaa',
'CjkUnifiedIdeograph6527': '\xe6\x94\xa7',
'CjkUnifiedIdeograph5566': '\xe5\x95\xa6',
'HangulSyllableDdwig': '\xeb\x9b\xb1',
'MyanmarDigitSeven': '\xe1\x81\x87',
'CjkUnifiedIdeograph81ec': '\xe8\x87\xac',
'HangulSyllableDdwib': '\xeb\x9c\x81',
'HangulSyllableDdwic': '\xeb\x9c\x87',
'CjkUnifiedIdeograph9505': '\xe9\x94\x85',
'CjkUnifiedIdeograph8b3a': '\xe8\xac\xba',
'CjkUnifiedIdeograph81ed': '\xe8\x87\xad',
'CjkUnifiedIdeograph8b3c': '\xe8\xac\xbc',
'CjkUnifiedIdeograph8b3b': '\xe8\xac\xbb',
'CjkUnifiedIdeograph8b3e': '\xe8\xac\xbe',
'CjkUnifiedIdeograph8b3d': '\xe8\xac\xbd',
'HangulSyllableDdwih': '\xeb\x9c\x8b',
'CjkUnifiedIdeograph81ee': '\xe8\x87\xae',
'HangulSyllableDdwit': '\xeb\x9c\x89',
'CjkUnifiedIdeograph81ef': '\xe8\x87\xaf',
'HangulSyllableDdwip': '\xeb\x9c\x8a',
'VaiSyllableKpi': '\xea\x94\xae',
'CjkUnifiedIdeograph5567': '\xe5\x95\xa7',
'ArabicLetterTehehFinalForm': '\xef\xad\xa3',
'CombiningReversedCommaAbove': '\xcc\x94',
'HangulSyllableGgyeoss': '\xea\xbc\x88',
'MalayalamLetterDa': '\xe0\xb4\xa6',
'CjkUnifiedIdeograph6af4': '\xe6\xab\xb4',
'HangulSyllableGgugg': '\xea\xbe\xba',
'UnicodeCharacter0xdcb2': '\xed\xb2\xb2',
'CjkUnifiedIdeograph9d7a': '\xe9\xb5\xba',
'VaiSyllableKpe': '\xea\x97\xac',
'ChamConsonantSignFinalH': '\xea\xa9\x8d',
'CjkUnifiedIdeograph85d9': '\xe8\x97\x99',
'CjkUnifiedIdeograph85d8': '\xe8\x97\x98',
'CjkUnifiedIdeograph85d7': '\xe8\x97\x97',
'CjkUnifiedIdeograph85d6': '\xe8\x97\x96',
'CjkUnifiedIdeograph85d5': '\xe8\x97\x95',
'CjkUnifiedIdeograph85d4': '\xe8\x97\x94',
'CjkUnifiedIdeograph85d3': '\xe8\x97\x93',
'KangxiRadicalCorpse': '\xe2\xbc\xab',
'CjkUnifiedIdeograph85d1': '\xe8\x97\x91',
'CjkUnifiedIdeograph85d0': '\xe8\x97\x90',
'HangulSyllableHwah': '\xed\x99\xaf',
'LatinCapitalLetterDWithCircumflexBelow': '\xe1\xb8\x92',
'CjkUnifiedIdeograph8b39': '\xe8\xac\xb9',
'CjkUnifiedIdeograph8b38': '\xe8\xac\xb8',
'CjkUnifiedIdeograph6d82': '\xe6\xb6\x82',
'|->': '\xe2\x86\xa6',
'HangulSyllableTyus': '\xed\x8a\xaf',
'CjkUnifiedIdeograph8b31': '\xe8\xac\xb1',
'CjkUnifiedIdeograph8b30': '\xe8\xac\xb0',
'CjkUnifiedIdeograph8b33': '\xe8\xac\xb3',
'CjkUnifiedIdeograph8b32': '\xe8\xac\xb2',
'CjkUnifiedIdeograph8b35': '\xe8\xac\xb5',
'CjkUnifiedIdeograph8b34': '\xe8\xac\xb4',
'CjkUnifiedIdeograph8b37': '\xe8\xac\xb7',
'CjkUnifiedIdeograph8b36': '\xe8\xac\xb6',
'UnicodeCharacter0xdbc4': '\xed\xaf\x84',
'UnicodeCharacter0xebaf': '\xee\xae\xaf',
'CjkRadicalCsimplifiedTannedLeather': '\xe2\xbb\x99',
'HangulSyllableGgwaes': '\xea\xbd\xb7',
'LatinSmallLetterUWithRingAbove': '\xc5\xaf',
'ArabicLigatureMeemWithKhahIsolatedForm': '\xef\xb1\x87',
'ThaanaLetterZaviyani': '\xde\x92',
'DoubleendedMultimap': '\xe2\xa7\x9f',
'HangulSyllableAbs': '\xec\x95\x96',
'CjkUnifiedIdeograph58b3': '\xe5\xa2\xb3',
'ModifierLetterSmallEsh': '\xe1\xb6\xb4',
'BraillePatternDots1245678': '\xe2\xa3\xbb',
'HangulSyllableDwilm': '\xeb\x92\xae',
'LaoLetterMo': '\xe0\xba\xa1',
'HangulSyllableTunj': '\xed\x88\xb1',
'HangulSyllableTunh': '\xed\x88\xb2',
'BraillePatternDots12567': '\xe2\xa1\xb3',
'HangulSyllableTung': '\xed\x89\x81',
'CjkUnifiedIdeograph6383': '\xe6\x8e\x83',
'UnicodeCharacter0xe76d': '\xee\x9d\xad',
'CjkUnifiedIdeograph9574': '\xe9\x95\xb4',
'HangulSyllableBbwalp': '\xeb\xbd\xaa',
'[7]': '\xe2\x82\x87',
'CjkUnifiedIdeograph6381': '\xe6\x8e\x81',
'CjkUnifiedIdeograph5561': '\xe5\x95\xa1',
'CjkUnifiedIdeograph9576': '\xe9\x95\xb6',
'CyrillicSmallLetterBlendedYus': '\xea\x99\x9b',
'CjkUnifiedIdeograph9571': '\xe9\x95\xb1',
'CjkUnifiedIdeograph9570': '\xe9\x95\xb0',
'BoxDrawingsHeavyTripleDashVertical': '\xe2\x94\x87',
'HangulSyllableGyeols': '\xea\xb2\xb4',
'CjkUnifiedIdeograph9573': '\xe9\x95\xb3',
'HangulSyllableGyeolp': '\xea\xb2\xb6',
'CjkUnifiedIdeograph9572': '\xe9\x95\xb2',
'CjkUnifiedIdeograph9b54': '\xe9\xad\x94',
'CjkUnifiedIdeograph9b55': '\xe9\xad\x95',
'CjkUnifiedIdeograph9b56': '\xe9\xad\x96',
'CjkUnifiedIdeograph9b57': '\xe9\xad\x97',
'CjkUnifiedIdeograph9b50': '\xe9\xad\x90',
'CjkUnifiedIdeograph9b51': '\xe9\xad\x91',
'CjkUnifiedIdeograph9b52': '\xe9\xad\x92',
'CjkUnifiedIdeograph9b53': '\xe9\xad\x93',
'UnicodeCharacter0xea72': '\xee\xa9\xb2',
'HangulSyllableGyeolm': '\xea\xb2\xb2',
'CjkUnifiedIdeograph9b58': '\xe9\xad\x98',
'CjkUnifiedIdeograph6389': '\xe6\x8e\x89',
'HangulSyllableGyeolh': '\xea\xb2\xb7',
'YiSyllableRot': '\xea\x8f\x8d',
'VaiSyllableSu': '\xea\x96\xac',
'CjkUnifiedIdeograph6388': '\xe6\x8e\x88',
'CjkUnifiedIdeograph7968': '\xe7\xa5\xa8',
'CjkUnifiedIdeograph7969': '\xe7\xa5\xa9',
'CjkUnifiedIdeograph7964': '\xe7\xa5\xa4',
'CjkUnifiedIdeograph7965': '\xe7\xa5\xa5',
'CjkUnifiedIdeograph7966': '\xe7\xa5\xa6',
'CjkUnifiedIdeograph7967': '\xe7\xa5\xa7',
'CjkUnifiedIdeograph7960': '\xe7\xa5\xa0',
'CjkUnifiedIdeograph7961': '\xe7\xa5\xa1',
'CjkUnifiedIdeograph7962': '\xe7\xa5\xa2',
'CjkUnifiedIdeograph7963': '\xe7\xa5\xa3',
'CjkUnifiedIdeograph4301': '\xe4\x8c\x81',
'CjkUnifiedIdeograph4300': '\xe4\x8c\x80',
'CjkUnifiedIdeograph4303': '\xe4\x8c\x83',
'CjkUnifiedIdeograph4302': '\xe4\x8c\x82',
'CjkUnifiedIdeograph4305': '\xe4\x8c\x85',
'CjkUnifiedIdeograph4304': '\xe4\x8c\x84',
'CjkUnifiedIdeograph4307': '\xe4\x8c\x87',
'CjkUnifiedIdeograph4306': '\xe4\x8c\x86',
'CjkUnifiedIdeograph4309': '\xe4\x8c\x89',
'CjkUnifiedIdeograph4308': '\xe4\x8c\x88',
'UnicodeCharacter0xe808': '\xee\xa0\x88',
'CjkUnifiedIdeograph9b5d': '\xe9\xad\x9d',
'CjkUnifiedIdeograph9b5e': '\xe9\xad\x9e',
'CjkUnifiedIdeograph9b5f': '\xe9\xad\x9f',
'CjkUnifiedIdeograph9b5a': '\xe9\xad\x9a',
'UnicodeCharacter0x1c9a': '\xe1\xb2\x9a',
'CjkUnifiedIdeograph9b5c': '\xe9\xad\x9c',
'ArabicLetterKafWithDotAbove': '\xda\xac',
'UnicodeCharacter0xf720': '\xef\x9c\xa0',
'KangxiRadicalLid': '\xe2\xbc\x87',
'DigitFourFullStop': '\xe2\x92\x8b',
'RunicLetterWunjoWynnW': '\xe1\x9a\xb9',
'CjkUnifiedIdeograph430c': '\xe4\x8c\x8c',
'CjkUnifiedIdeograph430b': '\xe4\x8c\x8b',
'CjkUnifiedIdeograph430e': '\xe4\x8c\x8e',
'CjkUnifiedIdeograph430d': '\xe4\x8c\x8d',
'CjkUnifiedIdeograph430f': '\xe4\x8c\x8f',
'ModifierLetterChineseToneYangShang': '\xea\x9c\x83',
'UnicodeCharacter0xf729': '\xef\x9c\xa9',
'CjkUnifiedIdeograph796d': '\xe7\xa5\xad',
'CjkUnifiedIdeograph796e': '\xe7\xa5\xae',
'CjkUnifiedIdeograph796f': '\xe7\xa5\xaf',
'UnicodeCharacter0xf728': '\xef\x9c\xa8',
'CjkUnifiedIdeograph796a': '\xe7\xa5\xaa',
'CjkUnifiedIdeograph796b': '\xe7\xa5\xab',
'CjkUnifiedIdeograph796c': '\xe7\xa5\xac',
'HebrewLetterTetWithDagesh': '\xef\xac\xb8',
'HangulSyllableJenh': '\xec\xa0\xa2',
'HangulSyllableJenj': '\xec\xa0\xa1',
'GeorgianCapitalLetterJhan': '\xe1\x82\xbf',
'TopSquareBracket': '\xe2\x8e\xb4',
'UnicodeCharacter0xfe75': '\xef\xb9\xb5',
'NewTaiLueLetterLowKa': '\xe1\xa6\x85',
'HangulSyllableJeng': '\xec\xa0\xb1',
'HangulSyllableJjwalt': '\xec\xab\xa5',
'HangulSyllableJjwals': '\xec\xab\xa4',
'HangulSyllableJjwalp': '\xec\xab\xa6',
'UnicodeCharacter0xea71': '\xee\xa9\xb1',
'UnicodeCharacter0xebad': '\xee\xae\xad',
'HangulSyllableJjwalg': '\xec\xab\xa1',
'UnicodeCharacter0xdfbe': '\xed\xbe\xbe',
'UnicodeCharacter0xdfbf': '\xed\xbe\xbf',
'HangulSyllableJjwalb': '\xec\xab\xa3',
'UnicodeCharacter0xdfbb': '\xed\xbe\xbb',
'CombiningAcutegraveacute': '\xe1\xb7\x89',
'HangulSyllableJjwalm': '\xec\xab\xa2',
'HangulSyllableJjwalh': '\xec\xab\xa7',
'UnicodeCharacter0xfbb6': '\xef\xae\xb6',
'UnicodeCharacter0xfbb7': '\xef\xae\xb7',
'LatinSmallLetterLongSWithDotAbove': '\xe1\xba\x9b',
'sho': '\xcf\xb8',
'UnicodeCharacter0xfbb4': '\xef\xae\xb4',
'UnicodeCharacter0xfbb5': '\xef\xae\xb5',
'KangxiRadicalDeath': '\xe2\xbd\x8d',
'RightParenthesisExtension': '\xe2\x8e\x9f',
'Anchor': '\xe2\x9a\x93',
'UnicodeCharacter0xfbb3': '\xef\xae\xb3',
'UnicodeCharacter0xdfb4': '\xed\xbe\xb4',
'UnicodeCharacter0xdfb5': '\xed\xbe\xb5',
'UnicodeCharacter0xdfb6': '\xed\xbe\xb6',
'ArmenianCapitalLetterEt': '\xd4\xb8',
'UnicodeCharacter0xdfb0': '\xed\xbe\xb0',
'UnicodeCharacter0xdfb1': '\xed\xbe\xb1',
'UnicodeCharacter0xdfb2': '\xed\xbe\xb2',
'UnicodeCharacter0xdfb3': '\xed\xbe\xb3',
'KatakanaLetterSmallMu': '\xe3\x87\xba',
'UnicodeCharacter0xdfb9': '\xed\xbe\xb9',
'LepchaLetterPha': '\xe1\xb0\x90',
'CjkUnifiedIdeograph9d78': '\xe9\xb5\xb8',
'UnicodeCharacter0xab3a': '\xea\xac\xba',
'ArmenianCapitalLetterEh': '\xd4\xb7',
'DingbatNegativeCircledDigitFive': '\xe2\x9d\xba',
'CjkUnifiedIdeograph9260': '\xe9\x89\xa0',
'CjkUnifiedIdeograph9263': '\xe9\x89\xa3',
'CjkUnifiedIdeograph9262': '\xe9\x89\xa2',
'CjkUnifiedIdeograph9265': '\xe9\x89\xa5',
'CjkUnifiedIdeograph9264': '\xe9\x89\xa4',
'CjkUnifiedIdeograph9267': '\xe9\x89\xa7',
'CjkUnifiedIdeograph9266': '\xe9\x89\xa6',
'CjkUnifiedIdeograph9269': '\xe9\x89\xa9',
'CjkUnifiedIdeograph9268': '\xe9\x89\xa8',
'HangulSyllableAeng': '\xec\x95\xb5',
'HangulSyllableGgilp': '\xeb\x82\x8a',
'HangulSyllableBolm': '\xeb\xb3\xbe',
'HangulSyllableGgils': '\xeb\x82\x88',
'ParenthesizedIdeographEnterprise': '\xe3\x88\xbd',
'UnicodeCharacter0xab3b': '\xea\xac\xbb',
'UnicodeCharacter0xfbb9': '\xef\xae\xb9',
'KhmerDigitThree': '\xe1\x9f\xa3',
'HangulSyllableSswess': '\xec\x92\xb0',
'CjkUnifiedIdeograph8bf9': '\xe8\xaf\xb9',
'HangulSyllableHenj': '\xed\x97\xa9',
'CjkUnifiedIdeograph9d73': '\xe9\xb5\xb3',
'DimensionOrigin': '\xe2\x8c\xb1',
'HangulSyllableGgilh': '\xeb\x82\x8b',
'VaiSyllableNggo': '\xea\x97\x9c',
'UnicodeCharacter0xebae': '\xee\xae\xae',
'VaiSyllableNggi': '\xea\x95\x84',
'UnicodeCharacter0xf72f': '\xef\x9c\xaf',
'VaiSyllableNgge': '\xea\x98\x84',
'UnicodeCharacter0xf72e': '\xef\x9c\xae',
'VaiSyllableNgga': '\xea\x95\xac',
'CjkUnifiedIdeograph926a': '\xe9\x89\xaa',
'CjkUnifiedIdeograph926c': '\xe9\x89\xac',
'CjkUnifiedIdeograph926b': '\xe9\x89\xab',
'CjkUnifiedIdeograph926e': '\xe9\x89\xae',
'CjkUnifiedIdeograph926d': '\xe9\x89\xad',
'CjkUnifiedIdeograph926f': '\xe9\x89\xaf',
'UnicodeCharacter0xf72c': '\xef\x9c\xac',
'VaiSyllableNggu': '\xea\x96\xb5',
'HangulSyllableGgilg': '\xeb\x82\x85',
'CjkUnifiedIdeograph8803': '\xe8\xa0\x83',
'UnicodeCharacter0xf72a': '\xef\x9c\xaa',
'UnicodeCharacter0xab3e': '\xea\xac\xbe',
'ArmenianSmallLetterIni': '\xd5\xab',
'HangulSyllableGgilb': '\xeb\x82\x87',
'HangulSyllableCebs': '\xec\xb3\x86',
'sube': '\xe2\x82\x91',
'HangulSyllablePoegg': '\xed\x91\x82',
'VaiSyllableKpa': '\xea\x95\x94',
'HangulSyllablePoegs': '\xed\x91\x83',
'UnicodeCharacter0xe1fa': '\xee\x87\xba',
'UnicodeCharacter0xe1fb': '\xee\x87\xbb',
'UnicodeCharacter0xe1fc': '\xee\x87\xbc',
'UnicodeCharacter0xe1fd': '\xee\x87\xbd',
'UnicodeCharacter0xe1fe': '\xee\x87\xbe',
'UnicodeCharacter0xe1ff': '\xee\x87\xbf',
'UnicodeCharacter0x1738': '\xe1\x9c\xb8',
'CjkUnifiedIdeograph78cb': '\xe7\xa3\x8b',
'CjkUnifiedIdeograph78cc': '\xe7\xa3\x8c',
'CjkUnifiedIdeograph78ca': '\xe7\xa3\x8a',
'CjkUnifiedIdeograph78cf': '\xe7\xa3\x8f',
'CjkUnifiedIdeograph78cd': '\xe7\xa3\x8d',
'CjkUnifiedIdeograph78ce': '\xe7\xa3\x8e',
'UnicodeCharacter0xec80': '\xee\xb2\x80',
'CjkUnifiedIdeograph3708': '\xe3\x9c\x88',
'UnicodeCharacter0xec82': '\xee\xb2\x82',
'UnicodeCharacter0xec83': '\xee\xb2\x83',
'UnicodeCharacter0xec84': '\xee\xb2\x84',
'UnicodeCharacter0xec85': '\xee\xb2\x85',
'UnicodeCharacter0xec86': '\xee\xb2\x86',
'UnicodeCharacter0xe804': '\xee\xa0\x84',
'UnicodeCharacter0xec88': '\xee\xb2\x88',
'UnicodeCharacter0xec89': '\xee\xb2\x89',
'HangulSyllableJjyoj': '\xec\xad\x82',
'ArabicLetterRehWithInvertedV': '\xdb\xaf',
'ArabicLigatureNoonWithHehInitialForm': '\xef\xb3\x96',
'UnicodeCharacter0x865': '\xe0\xa1\xa5',
'LatinCapitalLetterHWithDotAbove': '\xe1\xb8\xa2',
'UnicodeCharacter0x867': '\xe0\xa1\xa7',
'UnicodeCharacter0x866': '\xe0\xa1\xa6',
'UnicodeCharacter0x861': '\xe0\xa1\xa1',
'UnicodeCharacter0x860': '\xe0\xa1\xa0',
'UnicodeCharacter0x863': '\xe0\xa1\xa3',
'UnicodeCharacter0x862': '\xe0\xa1\xa2',
'CjkUnifiedIdeograph7146': '\xe7\x85\x86',
'CjkUnifiedIdeograph7147': '\xe7\x85\x87',
'CjkUnifiedIdeograph7144': '\xe7\x85\x84',
'CjkUnifiedIdeograph7145': '\xe7\x85\x85',
'UnicodeCharacter0x869': '\xe0\xa1\xa9',
'UnicodeCharacter0x868': '\xe0\xa1\xa8',
'CjkUnifiedIdeograph7140': '\xe7\x85\x80',
'CjkUnifiedIdeograph7141': '\xe7\x85\x81',
'HangulSyllableSseunj': '\xec\x93\xb5',
'UnicodeCharacter0xe1f1': '\xee\x87\xb1',
'CjkUnifiedIdeograph78c8': '\xe7\xa3\x88',
'CjkUnifiedIdeograph78c9': '\xe7\xa3\x89',
'UnicodeCharacter0xe1f4': '\xee\x87\xb4',
'UnicodeCharacter0xe1f5': '\xee\x87\xb5',
'UnicodeCharacter0xe1f6': '\xee\x87\xb6',
'KangxiRadicalBody': '\xe2\xbe\x9d',
'BoxDrawingsDownLightAndRightUpHeavy': '\xe2\x94\xa1',
'CjkUnifiedIdeograph78c3': '\xe7\xa3\x83',
'CjkUnifiedIdeograph78c0': '\xe7\xa3\x80',
'CjkUnifiedIdeograph78c1': '\xe7\xa3\x81',
'MongolianLetterSibeU': '\xe1\xa1\xa1',
'CjkUnifiedIdeograph78c7': '\xe7\xa3\x87',
'ParenthesizedHangulTikeutA': '\xe3\x88\x90',
'CjkUnifiedIdeograph78c5': '\xe7\xa3\x85',
'MongolianLetterSibeI': '\xe1\xa1\x9e',
'HangulChoseongThieuth': '\xe1\x84\x90',
'CjkUnifiedIdeograph65c7': '\xe6\x97\x87',
'MongolianLetterSibeE': '\xe1\xa1\x9d',
'UnicodeCharacter0x86e': '\xe0\xa1\xae',
'UnicodeCharacter0x86d': '\xe0\xa1\xad',
'UnicodeCharacter0xef58': '\xee\xbd\x98',
'UnicodeCharacter0x86f': '\xe0\xa1\xaf',
'UnicodeCharacter0x86a': '\xe0\xa1\xaa',
'MongolianLetterAliGaliInvertedUbadama': '\xe1\xa2\x84',
'UnicodeCharacter0x86b': '\xe0\xa1\xab',
'CjkUnifiedIdeograph714f': '\xe7\x85\x8f',
'UnicodeCharacter0xef53': '\xee\xbd\x93',
'CjkUnifiedIdeograph714d': '\xe7\x85\x8d',
'CjkUnifiedIdeograph714e': '\xe7\x85\x8e',
'CjkUnifiedIdeograph714b': '\xe7\x85\x8b',
'CjkUnifiedIdeograph714c': '\xe7\x85\x8c',
'UnicodeCharacter0xef54': '\xee\xbd\x94',
'CjkUnifiedIdeograph714a': '\xe7\x85\x8a',
'UnicodeCharacter0xec8a': '\xee\xb2\x8a',
'UnicodeCharacter0xec8b': '\xee\xb2\x8b',
'UnicodeCharacter0xec8c': '\xee\xb2\x8c',
'BlackBowtie': '\xe2\xa7\x93',
'UnicodeCharacter0xec8e': '\xee\xb2\x8e',
'UnicodeCharacter0xec8f': '\xee\xb2\x8f',
'NkoLetterWa': '\xdf\xa5',
'HangulSyllableBobs': '\xeb\xb4\x86',
'UnicodeCharacter0xeecb': '\xee\xbb\x8b',
'CjkUnifiedIdeograph8d48': '\xe8\xb5\x88',
'UnicodeCharacter0xe492': '\xee\x92\x92',
'ModifierLetterMidToneBar': '\xcb\xa7',
'MyanmarLetterSha': '\xe1\x81\x90',
'BengaliRupeeMark': '\xe0\xa7\xb2',
'BengaliDigitSix': '\xe0\xa7\xac',
'HangulSyllableSweolh': '\xec\x89\x83',
'ArabicLetterKehehInitialForm': '\xef\xae\x90',
'DoubleNestedLessthan': '\xe2\xaa\xa1',
'UnicodeCharacter0xe493': '\xee\x92\x93',
'BlackStar': '\xe2\x98\x85',
'CjkUnifiedIdeograph679d': '\xe6\x9e\x9d',
'CjkUnifiedIdeograph679e': '\xe6\x9e\x9e',
'CjkUnifiedIdeograph679f': '\xe6\x9e\x9f',
'CjkUnifiedIdeograph679a': '\xe6\x9e\x9a',
'CjkUnifiedIdeograph679b': '\xe6\x9e\x9b',
'CjkUnifiedIdeograph679c': '\xe6\x9e\x9c',
'HangulSyllableDek': '\xeb\x8e\x88',
'UnicodeCharacter0xe494': '\xee\x92\x94',
'CjkUnifiedIdeograph7380': '\xe7\x8e\x80',
'HangulSyllableJweon': '\xec\xa4\x9c',
'ArabicLigatureJeemWithHahWithAlefMaksuraFinalForm': '\xef\xb6\xa6',
'WhiteMediumSmallSquare': '\xe2\x97\xbd',
'KayahLiTonePlophu': '\xea\xa4\xab',
'CjkUnifiedIdeograph6794': '\xe6\x9e\x94',
'CjkUnifiedIdeograph6795': '\xe6\x9e\x95',
'CjkUnifiedIdeograph6796': '\xe6\x9e\x96',
'CjkUnifiedIdeograph6797': '\xe6\x9e\x97',
'CjkUnifiedIdeograph6790': '\xe6\x9e\x90',
'CjkUnifiedIdeograph6791': '\xe6\x9e\x91',
'CjkUnifiedIdeograph6792': '\xe6\x9e\x92',
'CjkUnifiedIdeograph6793': '\xe6\x9e\x93',
'CjkUnifiedIdeograph6798': '\xe6\x9e\x98',
'CjkUnifiedIdeograph6799': '\xe6\x9e\x99',
'CjkUnifiedIdeograph7386': '\xe7\x8e\x86',
'AplFunctionalSymbolStileTilde': '\xe2\x8d\xad',
'UnicodeCharacter0xdf31': '\xed\xbc\xb1',
'HangulSyllableSoelg': '\xec\x87\xa9',
'CjkUnifiedIdeograph6a49': '\xe6\xa9\x89',
'CjkUnifiedIdeograph6a48': '\xe6\xa9\x88',
'CyrillicCapitalLetterAWithDiaeresis': '\xd3\x92',
'HangulSyllableSoelb': '\xec\x87\xab',
'CjkUnifiedIdeograph6a45': '\xe6\xa9\x85',
'CjkUnifiedIdeograph6a44': '\xe6\xa9\x84',
'CjkUnifiedIdeograph6a47': '\xe6\xa9\x87',
'CjkUnifiedIdeograph6a46': '\xe6\xa9\x86',
'CjkUnifiedIdeograph6a41': '\xe6\xa9\x81',
'CjkUnifiedIdeograph6a40': '\xe6\xa9\x80',
'CjkUnifiedIdeograph6a43': '\xe6\xa9\x83',
'CjkUnifiedIdeograph6a42': '\xe6\xa9\x82',
'ArabicindicDigitZero': '\xd9\xa0',
'CjkUnifiedIdeograph3a3f': '\xe3\xa8\xbf',
'CjkUnifiedIdeograph3a3e': '\xe3\xa8\xbe',
'CjkUnifiedIdeograph3a3d': '\xe3\xa8\xbd',
'CjkUnifiedIdeograph3a3c': '\xe3\xa8\xbc',
'CjkUnifiedIdeograph3a3b': '\xe3\xa8\xbb',
'CjkUnifiedIdeograph3a3a': '\xe3\xa8\xba',
'UnicodeCharacter0xfbbf': '\xef\xae\xbf',
'UnicodeCharacter0xe497': '\xee\x92\x97',
'UnicodeCharacter0xfae0': '\xef\xab\xa0',
'CjkUnifiedIdeograph5dc5': '\xe5\xb7\x85',
'CanadianSyllabicsWestcreeKwii': '\xe1\x91\xb9',
'UnicodeCharacter0xfbbd': '\xef\xae\xbd',
'GreekSmallLetterUpsilonWithTonos': '\xcf\x8d',
'UnicodeCharacter0xfbbe': '\xef\xae\xbe',
'ArabicLigatureLamWithMeemWithYehFinalForm': '\xef\xb6\xad',
'HangulSyllableSwigg': '\xec\x89\xae',
'CjkUnifiedIdeograph5dc1': '\xe5\xb7\x81',
'CjkUnifiedIdeograph3a37': '\xe3\xa8\xb7',
'CjkUnifiedIdeograph3a36': '\xe3\xa8\xb6',
'CjkUnifiedIdeograph3a35': '\xe3\xa8\xb5',
'CjkUnifiedIdeograph3a34': '\xe3\xa8\xb4',
'CjkUnifiedIdeograph3a33': '\xe3\xa8\xb3',
'CjkUnifiedIdeograph3a32': '\xe3\xa8\xb2',
'CjkUnifiedIdeograph3a31': '\xe3\xa8\xb1',
'CjkUnifiedIdeograph3a30': '\xe3\xa8\xb0',
'CjkUnifiedIdeograph5dc3': '\xe5\xb7\x83',
'CjkUnifiedIdeograph3a39': '\xe3\xa8\xb9',
'CjkUnifiedIdeograph3a38': '\xe3\xa8\xb8',
'ModifierLetterSmallEng': '\xe1\xb5\x91',
'CjkUnifiedIdeograph6a4e': '\xe6\xa9\x8e',
'CjkUnifiedIdeograph6a4d': '\xe6\xa9\x8d',
'CjkUnifiedIdeograph6a4f': '\xe6\xa9\x8f',
'CjkUnifiedIdeograph6a4a': '\xe6\xa9\x8a',
'MyanmarLetterDdha': '\xe1\x80\x8e',
'CjkUnifiedIdeograph6a4c': '\xe6\xa9\x8c',
'CjkUnifiedIdeograph6a4b': '\xe6\xa9\x8b',
'UnicodeCharacter0xd812': '\xed\xa0\x92',
'CjkUnifiedIdeograph5dc8': '\xe5\xb7\x88',
'CjkUnifiedIdeograph5dc9': '\xe5\xb7\x89',
'CopticCapitalLetterLshapedHa': '\xe2\xb3\x90',
'HangulSyllableJalg': '\xec\x9e\x99',
'CjkUnifiedIdeograph89b9': '\xe8\xa6\xb9',
'CjkUnifiedIdeograph89b8': '\xe8\xa6\xb8',
'HangulSyllableCyaeg': '\xec\xb1\xbd',
'HangulSyllableSeolh': '\xec\x84\xab',
'HangulSyllableCyaeb': '\xec\xb2\x8d',
'HangulSyllableCyaec': '\xec\xb2\x93',
'CjkUnifiedIdeograph89b1': '\xe8\xa6\xb1',
'CjkUnifiedIdeograph89b0': '\xe8\xa6\xb0',
'CjkUnifiedIdeograph89b3': '\xe8\xa6\xb3',
'SinhalaLetterDantajaSayanna': '\xe0\xb7\x83',
'CjkUnifiedIdeograph89b5': '\xe8\xa6\xb5',
'CjkUnifiedIdeograph89b4': '\xe8\xa6\xb4',
'CjkUnifiedIdeograph89b7': '\xe8\xa6\xb7',
'CjkUnifiedIdeograph89b6': '\xe8\xa6\xb6',
'HangulSyllableCyaet': '\xec\xb2\x95',
'HangulSyllableCyaep': '\xec\xb2\x96',
'HangulSyllableCyanh': '\xec\xb1\xa6',
'HangulSyllableSeolt': '\xec\x84\xa9',
'HangulSyllableSeolp': '\xec\x84\xaa',
'HangulSyllableSeols': '\xec\x84\xa8',
'CjkUnifiedIdeograph7692': '\xe7\x9a\x92',
'TibetanSignRdelDkarRdelNag': '\xe0\xbc\x9f',
'UnicodeCharacter0xd7bd': '\xed\x9e\xbd',
'HangulSyllableYeogs': '\xec\x97\xaf',
'MalayalamVowelSignIi': '\xe0\xb5\x80',
'HangulSyllableMyaelt': '\xeb\xa8\xa9',
'CjkUnifiedIdeograph7690': '\xe7\x9a\x90',
'LepchaLetterHla': '\xe1\xb0\x9e',
'CjkUnifiedIdeograph7696': '\xe7\x9a\x96',
'UnicodeCharacter0xfae1': '\xef\xab\xa1',
'CjkUnifiedIdeograph7697': '\xe7\x9a\x97',
'**=': '\xe2\x81\xbc',
'CjkUnifiedIdeograph3706': '\xe3\x9c\x86',
'CjkUnifiedIdeograph7695': '\xe7\x9a\x95',
'LatinSmallLetterMWithDotBelow': '\xe1\xb9\x83',
'CjkUnifiedIdeograph89ba': '\xe8\xa6\xba',
'EthiopicSyllablePoa': '\xe2\xb6\x92',
'CjkUnifiedIdeograph89bb': '\xe8\xa6\xbb',
'CjkUnifiedIdeograph89be': '\xe8\xa6\xbe',
'CjkUnifiedIdeograph89bd': '\xe8\xa6\xbd',
'CjkUnifiedIdeograph6d98': '\xe6\xb6\x98',
'OriyaLetterDha': '\xe0\xac\xa7',
'YiSyllableHlix': '\xea\x86\x98',
'HangulSyllableSyeoc': '\xec\x85\xab',
'YiSyllableHlit': '\xea\x86\x97',
'HangulSyllableTwing': '\xed\x8a\x95',
'YiSyllableHlip': '\xea\x86\x9a',
'VerticalLineWithCircleBelow': '\xe2\xab\xb0',
'HangulSyllableJjeubs': '\xec\xaf\xa6',
'LatinSmallLetterFengDigraph': '\xca\xa9',
'HangulJongseongTikeutrieul': '\xe1\x87\x8b',
'CjkCompatibilityIdeographfa46': '\xef\xa9\x86',
'UnicodeCharacter0xd811': '\xed\xa0\x91',
'clockwiseintegral': '\xe2\x88\xb1',
'ArabicLigatureNoonWithAlefMaksuraFinalForm': '\xef\xb2\x8e',
'YiSyllableHlyx': '\xea\x86\xb2',
'HangulSyllableTaenh': '\xed\x83\xa2',
'ArabicLetterSeenWithDotBelowAndDotAbove': '\xda\x9a',
'AnticlockwiseClosedCircleArrow': '\xe2\xa5\x80',
'CjkUnifiedIdeograph4c76': '\xe4\xb1\xb6',
'CjkUnifiedIdeograph41be': '\xe4\x86\xbe',
'CjkUnifiedIdeograph41bd': '\xe4\x86\xbd',
'MyanmarLetterShanTha': '\xe1\x82\x80',
'CjkUnifiedIdeograph41bf': '\xe4\x86\xbf',
'CjkUnifiedIdeograph41ba': '\xe4\x86\xba',
'CjkUnifiedIdeograph41bc': '\xe4\x86\xbc',
'CjkUnifiedIdeograph41bb': '\xe4\x86\xbb',
'TeluguSignAnusvara': '\xe0\xb0\x82',
'HangulSyllableMyaelm': '\xeb\xa8\xa6',
'Multiset': '\xe2\x8a\x8c',
'ArabicLigatureBehWithJeemInitialForm': '\xef\xb2\x9c',
'LatinCapitalLetterUWithHorn': '\xc6\xaf',
'HangulSyllableBbyeong': '\xeb\xbc\x9d',
'CjkUnifiedIdeograph3be8': '\xe3\xaf\xa8',
'HangulSyllableBbyeonj': '\xeb\xbc\x8d',
'CjkUnifiedIdeograph3be9': '\xe3\xaf\xa9',
'HebrewLetterQofWithDagesh': '\xef\xad\x87',
'LatinSmallLetterUWithHornAndGrave': '\xe1\xbb\xab',
'CjkUnifiedIdeograph6d94': '\xe6\xb6\x94',
'**8': '\xe2\x81\xb8',
'CjkUnifiedIdeograph3be4': '\xe3\xaf\xa4',
'YiSyllableNryrx': '\xea\x8e\xa9',
'CjkUnifiedIdeograph3be5': '\xe3\xaf\xa5',
'HangulSyllableTeunj': '\xed\x8a\xbd',
'HangulSyllableTeunh': '\xed\x8a\xbe',
'LatinEpigraphicLetterReversedP': '\xea\x9f\xbc',
'CjkUnifiedIdeograph41b9': '\xe4\x86\xb9',
'CjkUnifiedIdeograph41b8': '\xe4\x86\xb8',
'CjkUnifiedIdeograph41b5': '\xe4\x86\xb5',
'CjkUnifiedIdeograph41b4': '\xe4\x86\xb4',
'CjkUnifiedIdeograph41b7': '\xe4\x86\xb7',
'CjkUnifiedIdeograph41b6': '\xe4\x86\xb6',
'CjkUnifiedIdeograph41b1': '\xe4\x86\xb1',
'CjkUnifiedIdeograph41b0': '\xe4\x86\xb0',
'CjkUnifiedIdeograph41b3': '\xe4\x86\xb3',
'CjkUnifiedIdeograph41b2': '\xe4\x86\xb2',
'CjkUnifiedIdeograph9d9b': '\xe9\xb6\x9b',
'CjkUnifiedIdeograph9d9c': '\xe9\xb6\x9c',
'MalayalamVowelSignE': '\xe0\xb5\x86',
'CjkUnifiedIdeograph9d9f': '\xe9\xb6\x9f',
'CjkUnifiedIdeograph9d9d': '\xe9\xb6\x9d',
'CjkUnifiedIdeograph9d9e': '\xe9\xb6\x9e',
'MalayalamVowelSignO': '\xe0\xb5\x8a',
'CjkUnifiedIdeograph9cb8': '\xe9\xb2\xb8',
'HangulSyllableIlm': '\xec\x9d\xbe',
'MalayalamVowelSignI': '\xe0\xb4\xbf',
'HangulSyllableIlp': '\xec\x9e\x82',
'CjkUnifiedIdeograph5818': '\xe5\xa0\x98',
'MalayalamVowelSignU': '\xe0\xb5\x81',
'CjkUnifiedIdeograph8828': '\xe8\xa0\xa8',
'CjkUnifiedIdeograph8829': '\xe8\xa0\xa9',
'GreaterthanBesideLessthan': '\xe2\xaa\xa5',
'CjkUnifiedIdeograph5812': '\xe5\xa0\x92',
'CjkUnifiedIdeograph5813': '\xe5\xa0\x93',
'CjkUnifiedIdeograph5810': '\xe5\xa0\x90',
'CjkUnifiedIdeograph5811': '\xe5\xa0\x91',
'CjkUnifiedIdeograph5816': '\xe5\xa0\x96',
'CjkUnifiedIdeograph5817': '\xe5\xa0\x97',
'CjkUnifiedIdeograph5814': '\xe5\xa0\x94',
'CjkUnifiedIdeograph5815': '\xe5\xa0\x95',
'CjkUnifiedIdeograph52f5': '\xe5\x8b\xb5',
'CjkUnifiedIdeograph52f4': '\xe5\x8b\xb4',
'CjkUnifiedIdeograph52f7': '\xe5\x8b\xb7',
'CjkUnifiedIdeograph52f6': '\xe5\x8b\xb6',
'CjkUnifiedIdeograph52f1': '\xe5\x8b\xb1',
'CjkUnifiedIdeograph52f0': '\xe5\x8b\xb0',
'CjkUnifiedIdeograph52f3': '\xe5\x8b\xb3',
'CjkUnifiedIdeograph52f2': '\xe5\x8b\xb2',
'CjkUnifiedIdeograph65ab': '\xe6\x96\xab',
'CjkUnifiedIdeograph65ac': '\xe6\x96\xac',
'UnicodeCharacter0xf707': '\xef\x9c\x87',
'CjkUnifiedIdeograph65aa': '\xe6\x96\xaa',
'CjkUnifiedIdeograph52f9': '\xe5\x8b\xb9',
'CjkUnifiedIdeograph52f8': '\xe5\x8b\xb8',
'CopticCapitalLetterSampi': '\xe2\xb3\x80',
'CjkUnifiedIdeograph65ae': '\xe6\x96\xae',
'UnicodeCharacter0xf44b': '\xef\x91\x8b',
'CjkUnifiedIdeograph5c79': '\xe5\xb1\xb9',
'CjkUnifiedIdeograph5c78': '\xe5\xb1\xb8',
'UnicodeCharacter0xf44c': '\xef\x91\x8c',
'CjkUnifiedIdeograph5c73': '\xe5\xb1\xb3',
'CjkUnifiedIdeograph5c72': '\xe5\xb1\xb2',
'CjkUnifiedIdeograph5c71': '\xe5\xb1\xb1',
'CjkUnifiedIdeograph5c70': '\xe5\xb1\xb0',
'CjkUnifiedIdeograph5c77': '\xe5\xb1\xb7',
'CjkUnifiedIdeograph5c76': '\xe5\xb1\xb6',
'CjkUnifiedIdeograph5c75': '\xe5\xb1\xb5',
'CjkUnifiedIdeograph5c74': '\xe5\xb1\xb4',
'CjkUnifiedIdeograph769b': '\xe7\x9a\x9b',
'HiraganaLetterKo': '\xe3\x81\x93',
'HiraganaLetterKi': '\xe3\x81\x8d',
'CjkUnifiedIdeograph769c': '\xe7\x9a\x9c',
'CjkUnifiedIdeograph581b': '\xe5\xa0\x9b',
'HiraganaLetterKe': '\xe3\x81\x91',
'CjkUnifiedIdeograph882f': '\xe8\xa0\xaf',
'CjkUnifiedIdeograph581a': '\xe5\xa0\x9a',
'CjkUnifiedIdeograph581f': '\xe5\xa0\x9f',
'HiraganaLetterKa': '\xe3\x81\x8b',
'CjkUnifiedIdeograph581d': '\xe5\xa0\x9d',
'CjkUnifiedIdeograph581e': '\xe5\xa0\x9e',
'CjkUnifiedIdeograph9d92': '\xe9\xb6\x92',
'CjkUnifiedIdeograph9d93': '\xe9\xb6\x93',
'CjkUnifiedIdeograph9d90': '\xe9\xb6\x90',
'CjkUnifiedIdeograph769a': '\xe7\x9a\x9a',
'CjkUnifiedIdeograph9d96': '\xe9\xb6\x96',
'CjkUnifiedIdeograph9d97': '\xe9\xb6\x97',
'CjkUnifiedIdeograph9d94': '\xe9\xb6\x94',
'CjkUnifiedIdeograph9d95': '\xe9\xb6\x95',
'HiraganaLetterKu': '\xe3\x81\x8f',
'CjkUnifiedIdeograph9d98': '\xe9\xb6\x98',
'CjkUnifiedIdeograph9d99': '\xe9\xb6\x99',
'DingbatCircledSansserifDigitSeven': '\xe2\x9e\x86',
'CjkUnifiedIdeograph769d': '\xe7\x9a\x9d',
'CjkUnifiedIdeograph5c7c': '\xe5\xb1\xbc',
'CjkUnifiedIdeograph5c7b': '\xe5\xb1\xbb',
'CjkUnifiedIdeograph5c7a': '\xe5\xb1\xba',
'CjkUnifiedIdeograph769e': '\xe7\x9a\x9e',
'CyrillicCapitalLetterYeruWithDiaeresis': '\xd3\xb8',
'MongolianDigitTwo': '\xe1\xa0\x92',
'CjkUnifiedIdeograph5c7d': '\xe5\xb1\xbd',
'CjkUnifiedIdeograph52fe': '\xe5\x8b\xbe',
'CjkUnifiedIdeograph52fd': '\xe5\x8b\xbd',
'CjkUnifiedIdeograph65a8': '\xe6\x96\xa8',
'CjkUnifiedIdeograph52ff': '\xe5\x8b\xbf',
'CjkUnifiedIdeograph52fa': '\xe5\x8b\xba',
'HangulSyllableJobs': '\xec\xa2\x82',
'CjkUnifiedIdeograph52fc': '\xe5\x8b\xbc',
'CjkUnifiedIdeograph52fb': '\xe5\x8b\xbb',
'CjkUnifiedIdeograph65a2': '\xe6\x96\xa2',
'CjkUnifiedIdeograph65a3': '\xe6\x96\xa3',
'CjkUnifiedIdeograph65a0': '\xe6\x96\xa0',
'CjkUnifiedIdeograph65a1': '\xe6\x96\xa1',
'CjkUnifiedIdeograph65a6': '\xe6\x96\xa6',
'CjkUnifiedIdeograph65a7': '\xe6\x96\xa7',
'CjkUnifiedIdeograph65a4': '\xe6\x96\xa4',
'CjkUnifiedIdeograph65a5': '\xe6\x96\xa5',
'UnicodeCharacter0x10cf': '\xe1\x83\x8f',
'UnicodeCharacter0x10cd': '\xe1\x83\x8d',
'UnicodeCharacter0x10ce': '\xe1\x83\x8e',
'UnicodeCharacter0x10cb': '\xe1\x83\x8b',
'UnicodeCharacter0x10cc': '\xe1\x83\x8c',
'CircledIdeographTwo': '\xe3\x8a\x81',
'UnicodeCharacter0x10ca': '\xe1\x83\x8a',
'NkoLetterNa': '\xdf\xa3',
'**2': '\xc2\xb2',
'BraillePatternDots134578': '\xe2\xa3\x9d',
'CjkUnifiedIdeograph402a': '\xe4\x80\xaa',
'CjkUnifiedIdeograph402b': '\xe4\x80\xab',
'CjkUnifiedIdeograph402c': '\xe4\x80\xac',
'CjkUnifiedIdeograph402d': '\xe4\x80\xad',
'CjkUnifiedIdeograph402e': '\xe4\x80\xae',
'CjkUnifiedIdeograph402f': '\xe4\x80\xaf',
'YiSyllableNzyrx': '\xea\x8c\x87',
'TibetanSubjoinedLetterRa': '\xe0\xbe\xb2',
'CjkUnifiedIdeograph8796': '\xe8\x9e\x96',
'CjkUnifiedIdeograph8797': '\xe8\x9e\x97',
'CjkUnifiedIdeograph8794': '\xe8\x9e\x94',
'ArabicLetterGueh': '\xda\xb3',
'CjkUnifiedIdeograph8792': '\xe8\x9e\x92',
'CjkUnifiedIdeograph8793': '\xe8\x9e\x93',
'CjkUnifiedIdeograph8790': '\xe8\x9e\x90',
'CjkUnifiedIdeograph8791': '\xe8\x9e\x91',
'CyrillicKavyka': '\xea\x99\xbe',
'HangulSyllablePeulb': '\xed\x94\x8f',
'HangulSyllableSeog': '\xec\x84\x9d',
'CjkUnifiedIdeograph8798': '\xe8\x9e\x98',
'CjkUnifiedIdeograph8799': '\xe8\x9e\x99',
'HangulSyllableGyugg': '\xea\xb7\x9e',
'HangulSyllableDdweolt': '\xeb\x9b\x85',
'HangulSyllableDdweols': '\xeb\x9b\x84',
'CjkUnifiedIdeograph586a': '\xe5\xa1\xaa',
'HangulSyllableDdweolp': '\xeb\x9b\x86',
'EthiopicSyllableFee': '\xe1\x8d\x8c',
'HangulSyllableNweoc': '\xeb\x88\xb7',
'CjkUnifiedIdeograph4020': '\xe4\x80\xa0',
'CjkUnifiedIdeograph4021': '\xe4\x80\xa1',
'CjkUnifiedIdeograph4022': '\xe4\x80\xa2',
'CjkUnifiedIdeograph4023': '\xe4\x80\xa3',
'CjkUnifiedIdeograph4024': '\xe4\x80\xa4',
'CjkUnifiedIdeograph4025': '\xe4\x80\xa5',
'CjkUnifiedIdeograph4026': '\xe4\x80\xa6',
'CjkUnifiedIdeograph4027': '\xe4\x80\xa7',
'CjkUnifiedIdeograph4028': '\xe4\x80\xa8',
'CjkUnifiedIdeograph4029': '\xe4\x80\xa9',
'UnicodeCharacter0x10c6': '\xe1\x83\x86',
'UnicodeCharacter0x10c7': '\xe1\x83\x87',
'LatinLetterSmallCapitalRum': '\xea\x9d\xb6',
'UnicodeCharacter0x10c8': '\xe1\x83\x88',
'UnicodeCharacter0x10c9': '\xe1\x83\x89',
'LatinCapitalLetterOWithInvertedBreve': '\xc8\x8e',
'ErrorbarredBlackDiamond': '\xe2\xa7\xb1',
'LeftWhiteCornerBracket': '\xe3\x80\x8e',
'NkoLetterLa': '\xdf\x9f',
'HangulSyllableGyaebs': '\xea\xb1\xa6',
'CjkUnifiedIdeograph8ea4': '\xe8\xba\xa4',
'CjkUnifiedIdeograph879f': '\xe8\x9e\x9f',
'CjkUnifiedIdeograph879d': '\xe8\x9e\x9d',
'CjkUnifiedIdeograph879e': '\xe8\x9e\x9e',
'CjkUnifiedIdeograph879b': '\xe8\x9e\x9b',
'CjkUnifiedIdeograph879c': '\xe8\x9e\x9c',
'GujaratiLetterTtha': '\xe0\xaa\xa0',
'LatinCapitalLetterGWithAcute': '\xc7\xb4',
'KangxiRadicalSeal': '\xe2\xbc\x99',
'CjkUnifiedIdeograph4eb1': '\xe4\xba\xb1',
'CjkUnifiedIdeograph4eb0': '\xe4\xba\xb0',
'CjkUnifiedIdeograph4eb3': '\xe4\xba\xb3',
'CjkUnifiedIdeograph4eb2': '\xe4\xba\xb2',
'CjkUnifiedIdeograph4eb5': '\xe4\xba\xb5',
'CjkUnifiedIdeograph4eb4': '\xe4\xba\xb4',
'CjkUnifiedIdeograph4eb7': '\xe4\xba\xb7',
'CjkUnifiedIdeograph4eb6': '\xe4\xba\xb6',
'CjkUnifiedIdeograph4eb9': '\xe4\xba\xb9',
'CjkUnifiedIdeograph4eb8': '\xe4\xba\xb8',
'CjkUnifiedIdeograph4790': '\xe4\x9e\x90',
'CjkUnifiedIdeograph4791': '\xe4\x9e\x91',
'CjkUnifiedIdeograph4796': '\xe4\x9e\x96',
'CjkUnifiedIdeograph4797': '\xe4\x9e\x97',
'CjkUnifiedIdeograph4794': '\xe4\x9e\x94',
'CjkUnifiedIdeograph4795': '\xe4\x9e\x95',
'UnicodeCharacter0xe404': '\xee\x90\x84',
'HangulSyllableCyelb': '\xec\xb3\xb7',
'CjkUnifiedIdeograph946f': '\xe9\x91\xaf',
'CjkUnifiedIdeograph946e': '\xe9\x91\xae',
'CjkUnifiedIdeograph946d': '\xe9\x91\xad',
'CjkUnifiedIdeograph946c': '\xe9\x91\xac',
'CjkUnifiedIdeograph946b': '\xe9\x91\xab',
'CjkUnifiedIdeograph946a': '\xe9\x91\xaa',
'UnicodeCharacter0xe402': '\xee\x90\x82',
'UnicodeCharacter0xe2b9': '\xee\x8a\xb9',
'UnicodeCharacter0xe2b8': '\xee\x8a\xb8',
'Juno': '\xe2\x9a\xb5',
'UnicodeCharacter0xe2b3': '\xee\x8a\xb3',
'CjkUnifiedIdeograph577f': '\xe5\x9d\xbf',
'CjkUnifiedIdeograph577e': '\xe5\x9d\xbe',
'CjkUnifiedIdeograph577d': '\xe5\x9d\xbd',
'CjkUnifiedIdeograph577c': '\xe5\x9d\xbc',
'CjkUnifiedIdeograph577b': '\xe5\x9d\xbb',
'CjkUnifiedIdeograph577a': '\xe5\x9d\xba',
'UnicodeCharacter0xe2b4': '\xee\x8a\xb4',
'UnicodeCharacter0xe429': '\xee\x90\xa9',
'UnicodeCharacter0xe428': '\xee\x90\xa8',
'CjkUnifiedIdeograph9cb2': '\xe9\xb2\xb2',
'UnicodeCharacter0xe425': '\xee\x90\xa5',
'UnicodeCharacter0xdea6': '\xed\xba\xa6',
'UnicodeCharacter0xe427': '\xee\x90\xa7',
'HangulSyllableDyobs': '\xeb\x91\x86',
'UnicodeCharacter0xe421': '\xee\x90\xa1',
'UnicodeCharacter0xe420': '\xee\x90\xa0',
'UnicodeCharacter0xe423': '\xee\x90\xa3',
'UnicodeCharacter0xe422': '\xee\x90\xa2',
'CjkUnifiedIdeograph7a64': '\xe7\xa9\xa4',
'HangulSyllableSeol': '\xec\x84\xa4',
'CjkUnifiedIdeograph9469': '\xe9\x91\xa9',
'CjkUnifiedIdeograph6d16': '\xe6\xb4\x96',
'CjkUnifiedIdeograph9467': '\xe9\x91\xa7',
'PrecedesOrEqualTo': '\xe2\x89\xbc',
'CjkUnifiedIdeograph9465': '\xe9\x91\xa5',
'CjkUnifiedIdeograph9464': '\xe9\x91\xa4',
'CjkUnifiedIdeograph9463': '\xe9\x91\xa3',
'CjkUnifiedIdeograph7a66': '\xe7\xa9\xa6',
'CjkUnifiedIdeograph9461': '\xe9\x91\xa1',
'CjkUnifiedIdeograph4ed6': '\xe4\xbb\x96',
'CjkUnifiedIdeograph4eba': '\xe4\xba\xba',
'HangulSyllableNweon': '\xeb\x88\xa4',
'CjkUnifiedIdeograph4ebc': '\xe4\xba\xbc',
'CjkUnifiedIdeograph4ebb': '\xe4\xba\xbb',
'CjkUnifiedIdeograph4ebe': '\xe4\xba\xbe',
'CjkUnifiedIdeograph4ebd': '\xe4\xba\xbd',
'CjkUnifiedIdeograph4ebf': '\xe4\xba\xbf',
'CjkUnifiedIdeograph479b': '\xe4\x9e\x9b',
'CjkUnifiedIdeograph479c': '\xe4\x9e\x9c',
'CjkUnifiedIdeograph479a': '\xe4\x9e\x9a',
'CjkUnifiedIdeograph479f': '\xe4\x9e\x9f',
'CjkUnifiedIdeograph479d': '\xe4\x9e\x9d',
'CjkUnifiedIdeograph479e': '\xe4\x9e\x9e',
'**+': '\xe2\x81\xba',
'HangulSyllableSeom': '\xec\x84\xac',
'UnicodeCharacter0xec6a': '\xee\xb1\xaa',
'CjkUnifiedIdeograph7a62': '\xe7\xa9\xa2',
'UnicodeCharacter0xe42e': '\xee\x90\xae',
'CircledIdeographNine': '\xe3\x8a\x88',
'CjkUnifiedIdeograph7a63': '\xe7\xa9\xa3',
'UnicodeCharacter0xe42a': '\xee\x90\xaa',
'UnicodeCharacter0xe42c': '\xee\x90\xac',
'HangulSyllableKyig': '\xed\x82\x89',
'CjkUnifiedIdeograph9cb4': '\xe9\xb2\xb4',
'CjkUnifiedIdeograph5779': '\xe5\x9d\xb9',
'CjkUnifiedIdeograph5778': '\xe5\x9d\xb8',
'CjkUnifiedIdeograph5777': '\xe5\x9d\xb7',
'CjkUnifiedIdeograph5776': '\xe5\x9d\xb6',
'CjkUnifiedIdeograph5775': '\xe5\x9d\xb5',
'CjkUnifiedIdeograph5774': '\xe5\x9d\xb4',
'CjkUnifiedIdeograph5773': '\xe5\x9d\xb3',
'CjkUnifiedIdeograph5772': '\xe5\x9d\xb2',
'CjkUnifiedIdeograph5771': '\xe5\x9d\xb1',
'CjkUnifiedIdeograph5770': '\xe5\x9d\xb0',
'UnicodeCharacter0xec6f': '\xee\xb1\xaf',
'UnicodeCharacter0xa797': '\xea\x9e\x97',
'Xor': '\xe2\x8a\xbb',
'UnicodeCharacter0xa796': '\xea\x9e\x96',
'HangulSyllableJjus': '\xec\xad\x9b',
'HangulSyllableJjup': '\xec\xad\xa2',
'HangulSyllableHyup': '\xed\x9d\x8e',
'UnicodeCharacter0xa795': '\xea\x9e\x95',
'VaiSyllableHi': '\xea\x94\xa6',
'CopticCapitalLetterOldCopticHat': '\xe2\xb3\x94',
'UnicodeCharacter0xaa73': '\xea\xa9\xb3',
'LeftRightWaveArrow': '\xe2\x86\xad',
'UnicodeCharacter0xaa71': '\xea\xa9\xb1',
'UnicodeCharacter0xa794': '\xea\x9e\x94',
'UnicodeCharacter0xaa77': '\xea\xa9\xb7',
'UnicodeCharacter0xaa76': '\xea\xa9\xb6',
'UnicodeCharacter0xaa75': '\xea\xa9\xb5',
'UnicodeCharacter0xaa74': '\xea\xa9\xb4',
'HangulSyllableJjub': '\xec\xad\x99',
'BlackDraughtsMan': '\xe2\x9b\x82',
'UnicodeCharacter0xaa79': '\xea\xa9\xb9',
'UnicodeCharacter0xaa78': '\xea\xa9\xb8',
'CjkUnifiedIdeograph944c': '\xe9\x91\x8c',
'OpencircuitoutputLtypeSymbol': '\xe2\x8e\x90',
'UnicodeCharacter0xa792': '\xea\x9e\x92',
'CjkUnifiedIdeograph8897': '\xe8\xa2\x97',
'UnicodeCharacter0xe316': '\xee\x8c\x96',
'CjkUnifiedIdeograph8896': '\xe8\xa2\x96',
'UnicodeCharacter0xd82d': '\xed\xa0\xad',
'HangulSyllableCyeobs': '\xec\xb3\xa2',
'KangxiRadicalSalt': '\xe2\xbf\x84',
'CjkUnifiedIdeograph8899': '\xe8\xa2\x99',
'ArabicLigatureYehWithHamzaAboveWithUIsolatedForm': '\xef\xaf\xb0',
'CjkUnifiedIdeograph8898': '\xe8\xa2\x98',
'EthiopicSyllablePhee': '\xe1\x8c\xb4',
'UnicodeCharacter0xec6d': '\xee\xb1\xad',
'ThaiCharacterChoChing': '\xe0\xb8\x89',
'UnicodeCharacter0xaa7c': '\xea\xa9\xbc',
'UnicodeCharacter0xaa7b': '\xea\xa9\xbb',
'UnicodeCharacter0xaa7a': '\xea\xa9\xba',
'CjkUnifiedIdeograph5864': '\xe5\xa1\xa4',
'SaurashtraLetterYa': '\xea\xa2\xab',
'UnicodeCharacter0xaa7f': '\xea\xa9\xbf',
'UnicodeCharacter0xaa7e': '\xea\xa9\xbe',
'UnicodeCharacter0xaa7d': '\xea\xa9\xbd',
'HangulSyllableBuc': '\xeb\xb6\x97',
'HangulSyllableBub': '\xeb\xb6\x91',
'ArabicLetterFarsiYehWithExtendedArabicindicDigitFourBelow': '\xdd\xb7',
'CjkUnifiedIdeograph9cb7': '\xe9\xb2\xb7',
'HangulSyllableBug': '\xeb\xb6\x81',
'HangulSyllableJjyigs': '\xec\xaf\xb3',
'CjkUnifiedIdeograph59ad': '\xe5\xa6\xad',
'UnicodeCharacter0xa799': '\xea\x9e\x99',
'HangulSyllableBup': '\xeb\xb6\x9a',
'HangulSyllableBus': '\xeb\xb6\x93',
'CjkUnifiedIdeograph5860': '\xe5\xa1\xa0',
'HangulSyllableBut': '\xeb\xb6\x99',
'HangulSyllableTweog': '\xed\x89\x89',
'TamilVowelSignOo': '\xe0\xaf\x8b',
'HangulSyllableBwass': '\xeb\xb4\xa4',
'KatakanaMiddleDot': '\xe3\x83\xbb',
'HangulSyllableTweod': '\xed\x89\x8f',
'UnicodeCharacter0xe5d3': '\xee\x97\x93',
'HangulSyllableSeoj': '\xec\x84\xb2',
'CanadianSyllabicsToo': '\xe1\x91\x91',
'CjkUnifiedIdeograph480b': '\xe4\xa0\x8b',
'CjkUnifiedIdeograph480c': '\xe4\xa0\x8c',
'LatinSmallLetterRWithMiddleTilde': '\xe1\xb5\xb2',
'CjkUnifiedIdeograph480a': '\xe4\xa0\x8a',
'CjkUnifiedIdeograph480f': '\xe4\xa0\x8f',
'CjkUnifiedIdeograph480d': '\xe4\xa0\x8d',
'CjkUnifiedIdeograph480e': '\xe4\xa0\x8e',
'SyriacHarkleanMetobelus': '\xdc\x8c',
'OriyaIsshar': '\xe0\xad\xb0',
'UnicodeCharacter0xf442': '\xef\x91\x82',
'HangulSyllableSeok': '\xec\x84\xb4',
'GlagoliticSmallLetterLatinateMyslite': '\xe2\xb1\x9e',
'CjkUnifiedIdeograph8ea6': '\xe8\xba\xa6',
'HangulSyllableTweom': '\xed\x89\x98',
'UnicodeCharacter0xdb5a': '\xed\xad\x9a',
'UnicodeCharacter0xed19': '\xee\xb4\x99',
'UnicodeCharacter0xdb5c': '\xed\xad\x9c',
'HangulSyllableTweol': '\xed\x89\x90',
'UnicodeCharacter0xdb5e': '\xed\xad\x9e',
'UnicodeCharacter0xdb5d': '\xed\xad\x9d',
'UnicodeCharacter0xdb5f': '\xed\xad\x9f',
'UnicodeCharacter0xed10': '\xee\xb4\x90',
'HangulSyllableTweok': '\xed\x89\xa0',
'UnicodeCharacter0xed12': '\xee\xb4\x92',
'CircledIdeographWater': '\xe3\x8a\x8c',
'UnicodeCharacter0xed14': '\xee\xb4\x94',
'UnicodeCharacter0xed15': '\xee\xb4\x95',
'UnicodeCharacter0xed16': '\xee\xb4\x96',
'HangulSyllableTweoj': '\xed\x89\x9e',
'CjkUnifiedIdeograph6e33': '\xe6\xb8\xb3',
'UnicodeCharacter0xf444': '\xef\x91\x84',
'CombiningLeftwardsHarpoonWithBarbDownwards': '\xe2\x83\xad',
'CjkRadicalSeal': '\xe2\xba\x8b',
'HangulSyllableTweoh': '\xed\x89\xa3',
'UnicodeCharacter0xecb5': '\xee\xb2\xb5',
'HangulSyllableBbyess': '\xeb\xbc\xb8',
'YiSyllableFur': '\xea\x83\x9d',
'YiSyllableFup': '\xea\x83\x9b',
'YiSyllableFut': '\xea\x83\x98',
'UnicodeCharacter0xdeac': '\xed\xba\xac',
'YiSyllableFux': '\xea\x83\x99',
'UnicodeCharacter0xe5d6': '\xee\x97\x96',
'CjkUnifiedIdeograph4802': '\xe4\xa0\x82',
'CjkUnifiedIdeograph4803': '\xe4\xa0\x83',
'CjkUnifiedIdeograph4800': '\xe4\xa0\x80',
'CjkUnifiedIdeograph4801': '\xe4\xa0\x81',
'CjkUnifiedIdeograph4806': '\xe4\xa0\x86',
'CjkUnifiedIdeograph4807': '\xe4\xa0\x87',
'CjkUnifiedIdeograph4804': '\xe4\xa0\x84',
'CjkUnifiedIdeograph4805': '\xe4\xa0\x85',
'CjkCompatibilityIdeographfa1e': '\xef\xa8\x9e',
'MongolianLetterTodoKa': '\xe1\xa1\x97',
'CjkUnifiedIdeograph4808': '\xe4\xa0\x88',
'CjkUnifiedIdeograph4809': '\xe4\xa0\x89',
'UnicodeCharacter0xe310': '\xee\x8c\x90',
'BalineseMusicalSymbolLefthandClosedPlak': '\xe1\xad\xba',
'CjkUnifiedIdeograph8118': '\xe8\x84\x98',
'UnicodeCharacter0xd814': '\xed\xa0\x94',
'UnicodeCharacter0xe5d7': '\xee\x97\x97',
'UnicodeCharacter0xdb51': '\xed\xad\x91',
'UnicodeCharacter0xdb50': '\xed\xad\x90',
'UnicodeCharacter0xdb53': '\xed\xad\x93',
'UnicodeCharacter0xdb52': '\xed\xad\x92',
'UnicodeCharacter0xdb55': '\xed\xad\x95',
'UnicodeCharacter0xdb54': '\xed\xad\x94',
'UnicodeCharacter0xdb57': '\xed\xad\x97',
'UnicodeCharacter0xa82d': '\xea\xa0\xad',
'UnicodeCharacter0xdb59': '\xed\xad\x99',
'UnicodeCharacter0xdb58': '\xed\xad\x98',
'UnicodeCharacter0xe7fc': '\xee\x9f\xbc',
'KangxiRadicalSpoon': '\xe2\xbc\x94',
'UnicodeCharacter0xed1d': '\xee\xb4\x9d',
'BraillePatternDots23': '\xe2\xa0\x86',
'UnicodeCharacter0xed1f': '\xee\xb4\x9f',
'CircledHangulMieumA': '\xe3\x89\xb2',
'UnicodeCharacter0xf8f0': '\xef\xa3\xb0',
'HangulSyllableTeom': '\xed\x85\x80',
'HangulSyllableNoegg': '\xeb\x87\x8e',
'UnicodeCharacter0xa82f': '\xea\xa0\xaf',
'UnicodeCharacter0xf8f4': '\xef\xa3\xb4',
'UnicodeCharacter0xf8f5': '\xef\xa3\xb5',
'CjkUnifiedIdeograph83e8': '\xe8\x8f\xa8',
'CjkUnifiedIdeograph7938': '\xe7\xa4\xb8',
'CjkUnifiedIdeograph83e6': '\xe8\x8f\xa6',
'ThaanaLetterLhaviyani': '\xde\x85',
'CjkUnifiedIdeograph83e4': '\xe8\x8f\xa4',
'CjkUnifiedIdeograph83e5': '\xe8\x8f\xa5',
'CjkUnifiedIdeograph83e2': '\xe8\x8f\xa2',
'GreekSmallLetterAlphaWithPerispomeniAndYpogegrammeni': '\xe1\xbe\xb7',
'CjkUnifiedIdeograph83e0': '\xe8\x8f\xa0',
'CjkUnifiedIdeograph83e1': '\xe8\x8f\xa1',
'UnicodeCharacter0xef1a': '\xee\xbc\x9a',
'HangulSyllableJyenj': '\xec\xa1\x99',
'HangulSyllableNoegs': '\xeb\x87\x8f',
'HangulSyllableTeoh': '\xed\x85\x8b',
'HangulSyllableKalm': '\xec\xb9\xbe',
'HangulSyllableBbonj': '\xeb\xbd\x85',
'HangulSyllableBbonh': '\xeb\xbd\x86',
'HangulSyllableTeod': '\xed\x84\xb7',
'HebrewAccentTevir': '\xd6\x9b',
'HangulSyllableBbong': '\xeb\xbd\x95',
'HangulSyllableTeog': '\xed\x84\xb1',
'CjkUnifiedIdeograph9442': '\xe9\x91\x82',
'LatinCapitalLetterEWithCircumflexAndDotBelow': '\xe1\xbb\x86',
'cepsilon': '\xce\xb5',
'CjkUnifiedIdeograph9c95': '\xe9\xb2\x95',
'UnicodeCharacter0xf8fb': '\xef\xa3\xbb',
'HangulSyllableCwass': '\xec\xb4\xb8',
'UnicodeCharacter0xf8fd': '\xef\xa3\xbd',
'HangulSyllableTeoc': '\xed\x85\x87',
'UnicodeCharacter0xf8ff': '\xef\xa3\xbf',
'CjkUnifiedIdeograph9c94': '\xe9\xb2\x94',
'CjkUnifiedIdeograph83ef': '\xe8\x8f\xaf',
'CjkUnifiedIdeograph83ed': '\xe8\x8f\xad',
'CjkUnifiedIdeograph83ee': '\xe8\x8f\xae',
'CjkUnifiedIdeograph83eb': '\xe8\x8f\xab',
'CjkUnifiedIdeograph83ec': '\xe8\x8f\xac',
'CjkUnifiedIdeograph83ea': '\xe8\x8f\xaa',
'BugineseVowelSignU': '\xe1\xa8\x98',
'ArabicLigatureMeemWithJeemInitialForm': '\xef\xb3\x8e',
'UnicodeCharacter0xe40f': '\xee\x90\x8f',
'UnicodeCharacter0xe10c': '\xee\x84\x8c',
'KannadaDigitFive': '\xe0\xb3\xab',
'CyrillicSmallLetterTe': '\xd1\x82',
'CyrillicCapitalLetterIotifiedA': '\xea\x99\x96',
'UnicodeCharacter0xe40b': '\xee\x90\x8b',
'CanadianSyllabicsRii': '\xe1\x95\x87',
'ArabicLetterGuehMedialForm': '\xef\xae\x99',
'HangulSyllableHwalg': '\xed\x99\x9d',
'DoesNotSucceed': '\xe2\x8a\x81',
'HalfwidthKatakanaLetterRa': '\xef\xbe\x97',
'GurmukhiDigitFour': '\xe0\xa9\xaa',
'SummationTop': '\xe2\x8e\xb2',
'MyanmarLetterShanNya': '\xe1\x81\xba',
'UnicodeCharacter0xe5d9': '\xee\x97\x99',
'LowerRightQuadrantCircularArc': '\xe2\x97\x9e',
'!<-': '\xe2\x86\x9a',
'NkoHighToneApostrophe': '\xdf\xb4',
'SubsetWithMultiplicationSignBelow': '\xe2\xab\x81',
'HebrewAccentPashta': '\xd6\x99',
'SyriacLetterSuperscriptAlaph': '\xdc\x91',
'HangulSyllableDeobs': '\xeb\x8d\xa6',
'UpwardsHarpoonWithBarbRightToBar': '\xe2\xa5\x94',
'CjkUnifiedIdeograph811f': '\xe8\x84\x9f',
'CjkUnifiedIdeograph9004': '\xe9\x80\x84',
'HalfwidthKatakanaLetterRu': '\xef\xbe\x99',
'UnicodeCharacter0xeb09': '\xee\xac\x89',
'UnicodeCharacter0xf760': '\xef\x9d\xa0',
'MongolianLetterSibeTsa': '\xe1\xa1\xae',
'HangulSyllableRess': '\xeb\xa0\x9c',
'GreekLunateEpsilonSymbol': '\xcf\xb5',
'KangxiRadicalSay': '\xe2\xbd\x88',
'UnicodeCharacter0xf767': '\xef\x9d\xa7',
'UnicodeCharacter0xf4a0': '\xef\x92\xa0',
'ParenthesizedDigitThree': '\xe2\x91\xb6',
'CjkCompatibilityIdeographf9ff': '\xef\xa7\xbf',
'BugineseLetterBa': '\xe1\xa8\x85',
'CjkUnifiedIdeograph5e55': '\xe5\xb9\x95',
'LatinSmallLetterEzhWithCurl': '\xca\x93',
'KangxiRadicalOld': '\xe2\xbd\xbc',
'CjkUnifiedIdeograph59a7': '\xe5\xa6\xa7',
'CanadianSyllabicsCarrierJa': '\xe1\x98\x9b',
'CanadianSyllabicsCarrierJe': '\xe1\x98\x97',
'CjkUnifiedIdeograph59a6': '\xe5\xa6\xa6',
'CanadianSyllabicsCarrierJi': '\xe1\x98\x99',
'CanadianSyllabicsCarrierJo': '\xe1\x98\x96',
'ArabicLigatureLamWithMeemIsolatedForm': '\xef\xb1\x82',
'CanadianSyllabicsCarrierJu': '\xe1\x98\x94',
'CjkUnifiedIdeograph59a3': '\xe5\xa6\xa3',
'UnicodeCharacter0xda61': '\xed\xa9\xa1',
'UnicodeCharacter0xf4a2': '\xef\x92\xa2',
'HangulSyllableDdolm': '\xeb\x98\x9a',
'CjkUnifiedIdeograph59a1': '\xe5\xa6\xa1',
'CjkCompatibilityIdeographf92b': '\xef\xa4\xab',
'ArabicSukunIsolatedForm': '\xef\xb9\xbe',
'YiSyllableNzox': '\xea\x8b\xba',
'ThaiCharacterDoDek': '\xe0\xb8\x94',
'HangulSyllableReong': '\xeb\xa0\x81',
'HangulSyllableReonj': '\xeb\x9f\xb1',
'CjkUnifiedIdeograph891d': '\xe8\xa4\x9d',
'HangulSyllableReonh': '\xeb\x9f\xb2',
'CjkUnifiedIdeograph5e58': '\xe5\xb9\x98',
'HangulSyllableWaebs': '\xec\x99\xae',
'OghamLetterNion': '\xe1\x9a\x85',
'CjkUnifiedIdeograph89ca': '\xe8\xa7\x8a',
'UnicodeCharacter0xf689': '\xef\x9a\x89',
'CopticSmallLetterSou': '\xe2\xb2\x8b',
'YiSyllableNjyrx': '\xea\x91\x88',
'RomanNumeralOneHundredThousand': '\xe2\x86\x88',
'CjkUnifiedIdeograph59a9': '\xe5\xa6\xa9',
'CjkUnifiedIdeograph6801': '\xe6\xa0\x81',
'CjkUnifiedIdeograph59a8': '\xe5\xa6\xa8',
'HangulSyllableDdolg': '\xeb\x98\x99',
'EthiopicTonalMarkChiret': '\xe1\x8e\x96',
'CjkUnifiedIdeograph65f4': '\xe6\x97\xb4',
'VaiSyllableGben': '\xea\x97\xb0',
'YiSyllableSie': '\xea\x8c\x8e',
'ArabicindicDigitNine': '\xd9\xa9',
'ThaiCharacterKoKai': '\xe0\xb8\x81',
'ArabicDammatan': '\xd9\x8c',
'UnicodeCharacter0xf371': '\xef\x8d\xb1',
'HangulSyllableNyeolp': '\xeb\x85\x8e',
'YiSyllableXiex': '\xea\x91\xa2',
'HangulSyllableNyeols': '\xeb\x85\x8c',
'HangulSyllableNyeolt': '\xeb\x85\x8d',
'CjkUnifiedIdeograph642f': '\xe6\x90\xaf',
'CjkUnifiedIdeograph642d': '\xe6\x90\xad',
'CjkUnifiedIdeograph642e': '\xe6\x90\xae',
'GreekSmallLetterOmicronWithDasia': '\xe1\xbd\x81',
'CjkUnifiedIdeograph642c': '\xe6\x90\xac',
'LatinSmallLetterIWithAcute': '\xc3\xad',
'CjkUnifiedIdeograph642a': '\xe6\x90\xaa',
'HangulSyllableNyeolb': '\xeb\x85\x8b',
'HangulSyllableNyeolg': '\xeb\x85\x89',
'HangulSyllableNyeolh': '\xeb\x85\x8f',
'TibetanMarkTshegShad': '\xe0\xbc\x8f',
'HangulSyllableNyeolm': '\xeb\x85\x8a',
'UnicodeCharacter0xf56a': '\xef\x95\xaa',
'UnicodeCharacter0xe1f9': '\xee\x87\xb9',
'UnicodeCharacter0xe696': '\xee\x9a\x96',
'CjkUnifiedIdeograph5a6a': '\xe5\xa9\xaa',
'RightWhiteCornerBracket': '\xe3\x80\x8f',
'PhagspaLetterAspiratedFa': '\xea\xa1\xb0',
'CjkUnifiedIdeograph5f0f': '\xe5\xbc\x8f',
'GlagoliticSmallLetterVede': '\xe2\xb0\xb2',
'UnicodeCharacter0xf4a9': '\xef\x92\xa9',
'GeorgianLetterKan': '\xe1\x83\x99',
'UnicodeCharacter0xe697': '\xee\x9a\x97',
'HangulSyllableWilh': '\xec\x9c\x93',
'UnicodeCharacter0xe23c': '\xee\x88\xbc',
'HangulSyllableWilm': '\xec\x9c\x8e',
'TagalogVowelSignU': '\xe1\x9c\x93',
'HangulSyllableWilb': '\xec\x9c\x8f',
'HangulSyllableWilg': '\xec\x9c\x8d',
'ArabicLetterJehFinalForm': '\xef\xae\x8b',
'RingAbove': '\xcb\x9a',
'CjkUnifiedIdeograph637f': '\xe6\x8d\xbf',
'HangulSyllableNaegg': '\xeb\x82\xb6',
'CjkUnifiedIdeograph9990': '\xe9\xa6\x90',
'Jupiter': '\xe2\x99\x83',
'CjkUnifiedIdeograph6428': '\xe6\x90\xa8',
'CjkUnifiedIdeograph6429': '\xe6\x90\xa9',
'CjkUnifiedIdeograph6426': '\xe6\x90\xa6',
'HangulJongseongRieulsios': '\xe1\x86\xb3',
'CjkUnifiedIdeograph6424': '\xe6\x90\xa4',
'CjkUnifiedIdeograph6425': '\xe6\x90\xa5',
'CjkUnifiedIdeograph6422': '\xe6\x90\xa2',
'CjkUnifiedIdeograph6423': '\xe6\x90\xa3',
'CjkUnifiedIdeograph6420': '\xe6\x90\xa0',
'CjkUnifiedIdeograph6421': '\xe6\x90\xa1',
'UnicodeCharacter0xf608': '\xef\x98\x88',
'CjkUnifiedIdeograph8e62': '\xe8\xb9\xa2',
'BalineseLetterLaLengaTedung': '\xe1\xac\x8e',
'CombiningCyrillicLetterKa': '\xe2\xb7\xa6',
'UnicodeCharacter0xf687': '\xef\x9a\x87',
'UnicodeCharacter0xaa94': '\xea\xaa\x94',
'UnicodeCharacter0xf56b': '\xef\x95\xab',
'UnicodeCharacter0xb65': '\xe0\xad\xa5',
'UnicodeCharacter0xb64': '\xe0\xad\xa4',
'UnicodeCharacter0xaa97': '\xea\xaa\x97',
'UnicodeCharacter0xe054': '\xee\x81\x94',
'HangulSyllableGelt': '\xea\xb2\x99',
'CjkUnifiedIdeograph7a67': '\xe7\xa9\xa7',
'UnicodeCharacter0xaa96': '\xea\xaa\x96',
'HangulSyllableGelp': '\xea\xb2\x9a',
'HangulSyllableGels': '\xea\xb2\x98',
'HangulSyllableJjwebs': '\xec\xae\x92',
'HangulSyllableNyub': '\xeb\x8a\x85',
'HangulSyllableGelg': '\xea\xb2\x95',
'LatinSmallLetterEzhWithCaron': '\xc7\xaf',
'HangulSyllableGelb': '\xea\xb2\x97',
'HangulSyllableGelm': '\xea\xb2\x96',
'UnicodeCharacter0xaa92': '\xea\xaa\x92',
'HangulSyllableGelh': '\xea\xb2\x9b',
'GeorgianLetterHae': '\xe1\x83\xb0',
'UnicodeCharacter0xf098': '\xef\x82\x98',
'CjkRadicalClothes': '\xe2\xbb\x82',
'EthiopicTonalMarkYizet': '\xe1\x8e\x90',
'HangulSyllableMugg': '\xeb\xac\xb6',
'UnicodeCharacter0xf56d': '\xef\x95\xad',
'HangulSyllableCwaegg': '\xec\xb5\x82',
'HangulJongseongKhieukh': '\xe1\x86\xbf',
'CjkUnifiedIdeograph8e67': '\xe8\xb9\xa7',
'TibetanLetterDa': '\xe0\xbd\x91',
'BengaliCurrencyNumeratorFour': '\xe0\xa7\xb7',
'LessthanAboveGreaterthanAboveDoublelineEqual': '\xe2\xaa\x91',
'UnicodeCharacter0xf682': '\xef\x9a\x82',
'LatinCapitalLetterLWithCircumflexBelow': '\xe1\xb8\xbc',
'HangulSyllableJwelg': '\xec\xa4\xbd',
'LepchaDigitNine': '\xe1\xb1\x89',
'VaiFullStop': '\xea\x98\x8e',
'LatinSmallLetterThorn': '\xc3\xbe',
'UnicodeCharacter0xf56f': '\xef\x95\xaf',
'TibetanLetterTsa': '\xe0\xbd\x99',
'HalfwidthUpwardsArrow': '\xef\xbf\xaa',
'ThaiCharacterNoNen': '\xe0\xb8\x93',
'CjkUnifiedIdeograph9bd7': '\xe9\xaf\x97',
'CjkUnifiedIdeograph9bd6': '\xe9\xaf\x96',
'CjkUnifiedIdeograph9bd5': '\xe9\xaf\x95',
'CjkUnifiedIdeograph9bd4': '\xe9\xaf\x94',
'CjkUnifiedIdeograph9bd3': '\xe9\xaf\x93',
'CjkUnifiedIdeograph9bd2': '\xe9\xaf\x92',
'CjkUnifiedIdeograph9bd1': '\xe9\xaf\x91',
'TeluguDigitSix': '\xe0\xb1\xac',
'YiSyllableQox': '\xea\x90\x89',
'CjkUnifiedIdeograph9bd9': '\xe9\xaf\x99',
'CjkUnifiedIdeograph876f': '\xe8\x9d\xaf',
'CjkUnifiedIdeograph9c2a': '\xe9\xb0\xaa',
'LatinCapitalLetterWWithDiaeresis': '\xe1\xba\x84',
'HangulSyllableDdyis': '\xeb\x9d\x97',
'HexagramForObstruction': '\xe4\xb7\xa6',
'HangulSyllableKyaenj': '\xec\xbb\x8d',
'HangulSyllableKyaenh': '\xec\xbb\x8e',
'CjkUnifiedIdeograph9c2f': '\xe9\xb0\xaf',
'UnicodeCharacter0xd7a8': '\xed\x9e\xa8',
'LatinSmallLetterMum': '\xea\x9d\xb3',
'CjkUnifiedIdeograph92cb': '\xe9\x8b\x8b',
'CjkUnifiedIdeograph92cc': '\xe9\x8b\x8c',
'HangulSyllableSsalt': '\xec\x8c\x85',
'ParenthesizedNumberFifteen': '\xe2\x92\x82',
'CjkUnifiedIdeograph7342': '\xe7\x8d\x82',
'EthiopicDigitTwo': '\xe1\x8d\xaa',
'SyriacHbasaBelow': '\xdc\xbb',
'CjkUnifiedIdeograph9562': '\xe9\x95\xa2',
'CjkUnifiedIdeograph92ca': '\xe9\x8b\x8a',
'CjkUnifiedIdeograph9bdc': '\xe9\xaf\x9c',
'CjkUnifiedIdeograph9bdb': '\xe9\xaf\x9b',
'CjkUnifiedIdeograph9bda': '\xe9\xaf\x9a',
'UnicodeCharacter0xda1a': '\xed\xa8\x9a',
'CjkUnifiedIdeograph92cf': '\xe9\x8b\x8f',
'HalfwidthHangulLetterHieuh': '\xef\xbe\xbe',
'RomanNumeralOneThousand': '\xe2\x85\xaf',
'CjkUnifiedIdeograph37f8': '\xe3\x9f\xb8',
'KhmerVowelSignOe': '\xe1\x9e\xbe',
'CjkUnifiedIdeograph37f9': '\xe3\x9f\xb9',
'MarriageSymbol': '\xe2\x9a\xad',
'OriyaLetterRa': '\xe0\xac\xb0',
'HangulSyllableRenh': '\xeb\xa0\x8e',
'CjkUnifiedIdeograph3c85': '\xe3\xb2\x85',
'UnicodeCharacter0xe069': '\xee\x81\xa9',
'HangulSyllableRenj': '\xeb\xa0\x8d',
'D': '\xce\x94',
'HangulSyllableBwi': '\xeb\xb7\x94',
'CjkUnifiedIdeograph7b0a': '\xe7\xac\x8a',
'CjkUnifiedIdeograph7b0c': '\xe7\xac\x8c',
'CjkUnifiedIdeograph7b0b': '\xe7\xac\x8b',
'CjkUnifiedIdeograph7b0e': '\xe7\xac\x8e',
'CjkUnifiedIdeograph7b0d': '\xe7\xac\x8d',
'CjkUnifiedIdeograph9560': '\xe9\x95\xa0',
'CjkUnifiedIdeograph7b0f': '\xe7\xac\x8f',
'UnicodeCharacter0xaa9e': '\xea\xaa\x9e',
'CjkCompatibilityIdeographf920': '\xef\xa4\xa0',
'CjkUnifiedIdeograph37f0': '\xe3\x9f\xb0',
'CjkUnifiedIdeograph3c86': '\xe3\xb2\x86',
'KannadaLetterUu': '\xe0\xb2\x8a',
'HangulSyllableBbeogg': '\xeb\xbb\x92',
'HangulSyllableSsyeong': '\xec\x8e\xb5',
'SquareLm': '\xe3\x8f\x90',
'UnicodeCharacter0xf230': '\xef\x88\xb0',
'SquareLn': '\xe3\x8f\x91',
'UnicodeCharacter0xf44a': '\xef\x91\x8a',
'HangulSyllableBbeogs': '\xeb\xbb\x93',
'GreekSmallLetterEtaWithDasia': '\xe1\xbc\xa1',
'HangulSyllableBwa': '\xeb\xb4\x90',
'SquareKmSquared': '\xe3\x8e\xa2',
'SquareLx': '\xe3\x8f\x93',
'ArabicTatweelWithFathatanAbove': '\xef\xb9\xb1',
'HangulSyllableNyut': '\xeb\x8a\x8d',
'YiSyllableCie': '\xea\x8a\xb4',
'CjkCompatibilityIdeographf928': '\xef\xa4\xa8',
'YiSyllableSu': '\xea\x8c\xa0',
'CjkUnifiedIdeograph9566': '\xe9\x95\xa6',
'UnicodeCharacter0xf233': '\xef\x88\xb3',
'UnicodeCharacter0xf44f': '\xef\x91\x8f',
'CjkUnifiedIdeograph65fb': '\xe6\x97\xbb',
'CjkUnifiedIdeograph7b09': '\xe7\xac\x89',
'KhmerLetterLo': '\xe1\x9e\x9b',
'YiSyllableSa': '\xea\x8c\x92',
'YiSyllableSe': '\xea\x8c\x9c',
'CjkUnifiedIdeograph7b01': '\xe7\xac\x81',
'CjkUnifiedIdeograph7b00': '\xe7\xac\x80',
'CjkUnifiedIdeograph7b03': '\xe7\xac\x83',
'CjkUnifiedIdeograph7b02': '\xe7\xac\x82',
'CjkUnifiedIdeograph7b05': '\xe7\xac\x85',
'CjkUnifiedIdeograph7b04': '\xe7\xac\x84',
'CanadianSyllabicsCarrierNee': '\xe1\x98\x85',
'KhmerLetterLa': '\xe1\x9e\xa1',
'CjkUnifiedIdeograph91d8': '\xe9\x87\x98',
'DevanagariLetterHa': '\xe0\xa4\xb9',
'CjkUnifiedIdeograph91d4': '\xe9\x87\x94',
'CjkUnifiedIdeograph91d5': '\xe9\x87\x95',
'CjkUnifiedIdeograph91d6': '\xe9\x87\x96',
'CjkUnifiedIdeograph91d7': '\xe9\x87\x97',
'CjkUnifiedIdeograph91d0': '\xe9\x87\x90',
'CjkUnifiedIdeograph91d1': '\xe9\x87\x91',
'HangulChoseongCeongchieumssangsios': '\xe1\x84\xbf',
'CjkUnifiedIdeograph91d3': '\xe9\x87\x93',
'CjkUnifiedIdeograph71ea': '\xe7\x87\xaa',
'CjkUnifiedIdeograph71ec': '\xe7\x87\xac',
'CjkUnifiedIdeograph71eb': '\xe7\x87\xab',
'CjkUnifiedIdeograph71ee': '\xe7\x87\xae',
'CjkUnifiedIdeograph71ed': '\xe7\x87\xad',
'CjkUnifiedIdeograph71ef': '\xe7\x87\xaf',
'PlusSignInTriangle': '\xe2\xa8\xb9',
'CjkUnifiedIdeograph7ba6': '\xe7\xae\xa6',
'HangulSyllableSsyeonh': '\xec\x8e\xa6',
'GlagoliticSmallLetterPe': '\xe2\xb1\x8a',
'HangulSyllableByaed': '\xeb\xb1\xaf',
'ModifierLetterSmallSidewaysU': '\xe1\xb5\x99',
'HangulSyllableBbyalg': '\xeb\xba\xa1',
'YiSyllableHxox': '\xea\x89\x97',
'UnicodeCharacter0xdf4e': '\xed\xbd\x8e',
'UnicodeCharacter0xdf4b': '\xed\xbd\x8b',
'LepchaLetterBa': '\xe1\xb0\x93',
'UnicodeCharacter0xdf4c': '\xed\xbd\x8c',
'CjkUnifiedIdeograph71e1': '\xe7\x87\xa1',
'CjkUnifiedIdeograph71e0': '\xe7\x87\xa0',
'CjkUnifiedIdeograph71e3': '\xe7\x87\xa3',
'LepchaLetterTha': '\xe1\xb0\x8b',
'CjkUnifiedIdeograph71e5': '\xe7\x87\xa5',
'CjkUnifiedIdeograph71e4': '\xe7\x87\xa4',
'CjkUnifiedIdeograph71e7': '\xe7\x87\xa7',
'CjkUnifiedIdeograph71e6': '\xe7\x87\xa6',
'CjkUnifiedIdeograph71e9': '\xe7\x87\xa9',
'CjkUnifiedIdeograph71e8': '\xe7\x87\xa8',
'HangulSyllableBbyalb': '\xeb\xba\xa3',
'BalineseAdegAdeg': '\xe1\xad\x84',
'GreekCapitalLunateSigmaSymbol': '\xcf\xb9',
'CjkUnifiedIdeograph7103': '\xe7\x84\x83',
'CjkUnifiedIdeograph91de': '\xe9\x87\x9e',
'CjkUnifiedIdeograph91df': '\xe9\x87\x9f',
'KangxiRadicalDish': '\xe2\xbd\xab',
'CjkUnifiedIdeograph91da': '\xe9\x87\x9a',
'LepchaLetterMla': '\xe1\xb0\x96',
'CjkUnifiedIdeograph91dc': '\xe9\x87\x9c',
'HangulSyllableGgeh': '\xea\xbb\xb3',
'HangulSyllableGgej': '\xea\xbb\xae',
'HangulSyllableGgek': '\xea\xbb\xb0',
'HangulSyllableGgel': '\xea\xbb\xa0',
'HangulSyllableGgem': '\xea\xbb\xa8',
'HangulSyllableGgen': '\xea\xbb\x9c',
'HangulSyllableGgeo': '\xea\xba\xbc',
'HangulSyllableDdyaess': '\xeb\x96\x98',
'HangulSyllableGgeb': '\xea\xbb\xa9',
'HangulSyllableGgec': '\xea\xbb\xaf',
'HangulSyllableGged': '\xea\xbb\x9f',
'HangulSyllableGgeg': '\xea\xbb\x99',
'HangulSyllableByaek': '\xeb\xb2\x80',
'HangulSyllableGgep': '\xea\xbb\xb2',
'CherokeeLetterTle': '\xe1\x8f\x9e',
'HangulSyllableGges': '\xea\xbb\xab',
'HangulSyllableGget': '\xea\xbb\xb1',
'HangulSyllableGgeu': '\xeb\x81\x84',
'CjkUnifiedIdeograph76a7': '\xe7\x9a\xa7',
'CjkUnifiedIdeograph92c3': '\xe9\x8b\x83',
'UnicodeCharacter0xe2e9': '\xee\x8b\xa9',
'HangulJongseongIeungssangkiyeok': '\xe1\x87\xad',
'CjkUnifiedIdeograph92c0': '\xe9\x8b\x80',
'CombiningRingAbove': '\xcc\x8a',
'CjkUnifiedIdeograph92c1': '\xe9\x8b\x81',
'MalayalamLetterVocalicRr': '\xe0\xb5\xa0',
'UnicodeCharacter0xe24e': '\xee\x89\x8e',
'CjkUnifiedIdeograph92c6': '\xe9\x8b\x86',
'UnicodeCharacter0xe24f': '\xee\x89\x8f',
'UnicodeCharacter0xe24a': '\xee\x89\x8a',
'UnicodeCharacter0xe24c': '\xee\x89\x8c',
'CjkUnifiedIdeograph92c7': '\xe9\x8b\x87',
'UnicodeCharacter0xecaf': '\xee\xb2\xaf',
'UnicodeCharacter0xecae': '\xee\xb2\xae',
'UnicodeCharacter0xecad': '\xee\xb2\xad',
'UnicodeCharacter0xecac': '\xee\xb2\xac',
'CjkUnifiedIdeograph92c4': '\xe9\x8b\x84',
'UnicodeCharacter0xecaa': '\xee\xb2\xaa',
'HangulSyllableDaet': '\xeb\x8c\x99',
'CjkUnifiedIdeograph92c5': '\xe9\x8b\x85',
'UnicodeCharacter0xe2e3': '\xee\x8b\xa3',
'UnicodeCharacter0xe2e0': '\xee\x8b\xa0',
'TwoIntersectingLogicalOr': '\xe2\xa9\x96',
'CjkUnifiedIdeograph678a': '\xe6\x9e\x8a',
'ParenthesizedNumberThirteen': '\xe2\x92\x80',
'GlagoliticCapitalLetterOtu': '\xe2\xb0\x99',
'UnicodeCharacter0xe249': '\xee\x89\x89',
'UnicodeCharacter0xe248': '\xee\x89\x88',
'UnicodeCharacter0xe245': '\xee\x89\x85',
'CjkUnifiedIdeograph4239': '\xe4\x88\xb9',
'UnicodeCharacter0xe247': '\xee\x89\x87',
'UnicodeCharacter0xe246': '\xee\x89\x86',
'UnicodeCharacter0xe241': '\xee\x89\x81',
'CjkUnifiedIdeograph40dd': '\xe4\x83\x9d',
'UnicodeCharacter0xe243': '\xee\x89\x83',
'CjkUnifiedIdeograph37fc': '\xe3\x9f\xbc',
'HangulJungseongIaraea': '\xe1\x86\x9d',
'ArmenianCapitalLetterKen': '\xd4\xbf',
'ArmenianCapitalLetterKeh': '\xd5\x94',
'HangulSyllableGyol': '\xea\xb5\x98',
'HangulSyllableGyom': '\xea\xb5\xa0',
'CombiningLeftAngleAbove': '\xcc\x9a',
'HangulSyllableSwiss': '\xec\x8a\x80',
'HangulSyllableGyoh': '\xea\xb5\xab',
'KatakanaLetterZu': '\xe3\x82\xba',
'YiSyllableRep': '\xea\x8f\x93',
'HangulSyllableGyok': '\xea\xb5\xa8',
'HangulSyllableGyod': '\xea\xb5\x97',
'HangulSyllableTwaed': '\xed\x87\x9f',
'WhiteSquareWithLowerLeftQuadrant': '\xe2\x97\xb1',
'HangulSyllableGyog': '\xea\xb5\x91',
'DevanagariVowelSignIi': '\xe0\xa5\x80',
'HangulSyllableGyob': '\xea\xb5\xa1',
'HangulSyllableGyoc': '\xea\xb5\xa7',
'UnicodeCharacter0xeca7': '\xee\xb2\xa7',
'KatakanaLetterZa': '\xe3\x82\xb6',
'UnicodeCharacter0xeca5': '\xee\xb2\xa5',
'UnicodeCharacter0xeca4': '\xee\xb2\xa4',
'UnicodeCharacter0xeca3': '\xee\xb2\xa3',
'KatakanaLetterZe': '\xe3\x82\xbc',
'HangulSyllableJwilp': '\xec\xa5\x9e',
'UnicodeCharacter0xeca0': '\xee\xb2\xa0',
'HangulSyllableGyot': '\xea\xb5\xa9',
'KatakanaLetterZi': '\xe3\x82\xb8',
'CjkUnifiedIdeograph8273': '\xe8\x89\xb3',
'HangulSyllableTwaep': '\xed\x87\xb2',
'HangulSyllableTwaes': '\xed\x87\xab',
'KatakanaLetterZo': '\xe3\x82\xbe',
'LatinSmallLetterEWithCedillaAndBreve': '\xe1\xb8\x9d',
'GreekCapitalLetterEtaWithTonos': '\xce\x89',
'BoxDrawingsLightDownAndRight': '\xe2\x94\x8c',
'UnicodeCharacter0xa0d': '\xe0\xa8\x8d',
'UnicodeCharacter0xa0e': '\xe0\xa8\x8e',
'UnicodeCharacter0xa0b': '\xe0\xa8\x8b',
'UnicodeCharacter0xa0c': '\xe0\xa8\x8c',
'CjkUnifiedIdeograph861c': '\xe8\x98\x9c',
'CjkUnifiedIdeograph861b': '\xe8\x98\x9b',
'CjkUnifiedIdeograph861a': '\xe8\x98\x9a',
'UnicodeCharacter0xf571': '\xef\x95\xb1',
'UnicodeCharacter0xf576': '\xef\x95\xb6',
'CjkUnifiedIdeograph861f': '\xe8\x98\x9f',
'CjkUnifiedIdeograph861e': '\xe8\x98\x9e',
'CjkUnifiedIdeograph861d': '\xe8\x98\x9d',
'HangulSyllableGwilt': '\xea\xb7\x8d',
'MyanmarLetterWesternPwoKarenPwa': '\xe1\x81\xa6',
'BopomofoLetterInnn': '\xe3\x86\xb3',
'LatinSmallLetterYWithCircumflex': '\xc5\xb7',
'CanadianSyllabicsThii': '\xe1\x95\xa2',
'UnicodeCharacter0xe49a': '\xee\x92\x9a',
'UnicodeCharacter0xe49b': '\xee\x92\x9b',
'UnicodeCharacter0xe49c': '\xee\x92\x9c',
'HangulSyllableDyeng': '\xeb\x8e\xbd',
'SinhalaLetterMahaapraanaTayanna': '\xe0\xb6\xae',
'UnicodeCharacter0xe49e': '\xee\x92\x9e',
'CjkUnifiedIdeograph7baf': '\xe7\xae\xaf',
'UnicodeCharacter0xa04': '\xe0\xa8\x84',
'CjkUnifiedIdeograph8618': '\xe8\x98\x98',
'CjkUnifiedIdeograph94b4': '\xe9\x92\xb4',
'UnicodeCharacter0xa00': '\xe0\xa8\x80',
'UnicodeCharacter0xdf47': '\xed\xbd\x87',
'CjkUnifiedIdeograph8613': '\xe8\x98\x93',
'CjkUnifiedIdeograph8612': '\xe8\x98\x92',
'CjkUnifiedIdeograph8611': '\xe8\x98\x91',
'CjkUnifiedIdeograph8610': '\xe8\x98\x90',
'CjkUnifiedIdeograph8617': '\xe8\x98\x97',
'CjkUnifiedIdeograph8616': '\xe8\x98\x96',
'CjkUnifiedIdeograph8615': '\xe8\x98\x95',
'CjkUnifiedIdeograph8614': '\xe8\x98\x94',
'LatinCapitalLetterDWithSmallLetterZ': '\xc7\xb2',
'UnicodeCharacter0xdf45': '\xed\xbd\x85',
'BengaliLetterRaWithLowerDiagonal': '\xe0\xa7\xb1',
'UnicodeCharacter0xdf42': '\xed\xbd\x82',
'MalayalamDigitFive': '\xe0\xb5\xab',
'CjkUnifiedIdeograph7bac': '\xe7\xae\xac',
'UnicodeCharacter0xe31e': '\xee\x8c\x9e',
'CjkUnifiedIdeograph4099': '\xe4\x82\x99',
'CjkUnifiedIdeograph74e0': '\xe7\x93\xa0',
'SinhalaLetterIruuyanna': '\xe0\xb6\x8e',
'HangulDoubleDotToneMark': '\xe3\x80\xaf',
'ArabicSignSallallahouAlayheWassallam': '\xd8\x90',
'HangulSyllableYibs': '\xec\x9d\xaa',
'CjkUnifiedIdeograph74e1': '\xe7\x93\xa1',
'HangulSyllablePeunj': '\xed\x94\x89',
'ArabicLetterKirghizYu': '\xdb\x89',
'ArabicLetterFarsiYehWithThreeDotsAbove': '\xd8\xbf',
'CombiningCyrillicLetterVe': '\xe2\xb7\xa1',
'CjkUnifiedIdeograph67b9': '\xe6\x9e\xb9',
'CjkUnifiedIdeograph67b8': '\xe6\x9e\xb8',
'EstimatedSymbol': '\xe2\x84\xae',
'CjkUnifiedIdeograph74e2': '\xe7\x93\xa2',
'CjkUnifiedIdeograph67b1': '\xe6\x9e\xb1',
'FigureSpace': '\xe2\x80\x87',
'CjkUnifiedIdeograph67b3': '\xe6\x9e\xb3',
'CjkUnifiedIdeograph67b2': '\xe6\x9e\xb2',
'CjkUnifiedIdeograph67b5': '\xe6\x9e\xb5',
'CjkUnifiedIdeograph67b4': '\xe6\x9e\xb4',
'CjkUnifiedIdeograph67b7': '\xe6\x9e\xb7',
'CjkUnifiedIdeograph67b6': '\xe6\x9e\xb6',
'UnicodeCharacter0xde18': '\xed\xb8\x98',
'SmallExclamationMark': '\xef\xb9\x97',
'HangulSyllablePoelb': '\xed\x91\x8b',
'searrow': '\xe2\x86\x98',
'UnicodeCharacter0xe31b': '\xee\x8c\x9b',
'UnicodeCharacter0xe2eb': '\xee\x8b\xab',
'OpenSubset': '\xe2\x9f\x83',
'UnicodeCharacter0xe31a': '\xee\x8c\x9a',
'HangulJongseongSsangrieul': '\xe1\x87\x90',
'HangulSyllableHebs': '\xed\x97\xb6',
'OriyaLetterTta': '\xe0\xac\x9f',
'HangulSyllableJjoep': '\xec\xac\xaa',
'HangulSyllableJjoet': '\xec\xac\xa9',
'HangulSyllableJjoej': '\xec\xac\xa6',
'HangulSyllableJjoek': '\xec\xac\xa8',
'HangulSyllableJjoeh': '\xec\xac\xab',
'HangulSyllableJjoen': '\xec\xac\x94',
'HangulSyllableJjoel': '\xec\xac\x98',
'HangulSyllableJjoem': '\xec\xac\xa0',
'CjkUnifiedIdeograph67ba': '\xe6\x9e\xba',
'HangulSyllableYun': '\xec\x9c\xa4',
'CjkUnifiedIdeograph67bc': '\xe6\x9e\xbc',
'CjkUnifiedIdeograph67bb': '\xe6\x9e\xbb',
'CjkUnifiedIdeograph67be': '\xe6\x9e\xbe',
'CjkUnifiedIdeograph67bd': '\xe6\x9e\xbd',
'HangulSyllableJjoed': '\xec\xac\x97',
'CjkUnifiedIdeograph67bf': '\xe6\x9e\xbf',
'HangulSyllableYul': '\xec\x9c\xa8',
'HangulSyllableYum': '\xec\x9c\xb0',
'UnicodeCharacter0xe8c0': '\xee\xa3\x80',
'LaoNiggahita': '\xe0\xbb\x8d',
'UnicodeCharacter0xdb7f': '\xed\xad\xbf',
'UnicodeCharacter0xdb7e': '\xed\xad\xbe',
'UnicodeCharacter0xe8c6': '\xee\xa3\x86',
'UnicodeCharacter0xdb7d': '\xed\xad\xbd',
'UnicodeCharacter0xe8e8': '\xee\xa3\xa8',
'HangulSyllableKagg': '\xec\xb9\xb6',
'CjkUnifiedIdeograph48fd': '\xe4\xa3\xbd',
'CjkUnifiedIdeograph48fe': '\xe4\xa3\xbe',
'CjkUnifiedIdeograph48ff': '\xe4\xa3\xbf',
'CjkUnifiedIdeograph9059': '\xe9\x81\x99',
'CjkUnifiedIdeograph48fa': '\xe4\xa3\xba',
'CjkUnifiedIdeograph48fb': '\xe4\xa3\xbb',
'CjkUnifiedIdeograph48fc': '\xe4\xa3\xbc',
'CjkUnifiedIdeograph9052': '\xe9\x81\x92',
'CjkUnifiedIdeograph8f0a': '\xe8\xbc\x8a',
'CjkUnifiedIdeograph8f0b': '\xe8\xbc\x8b',
'CjkUnifiedIdeograph8f0c': '\xe8\xbc\x8c',
'ThaanaLetterDaadhu': '\xde\x9f',
'CjkUnifiedIdeograph8f0e': '\xe8\xbc\x8e',
'CjkUnifiedIdeograph8f0f': '\xe8\xbc\x8f',
'CjkUnifiedIdeograph9055': '\xe9\x81\x95',
'CircledIdeographSociety': '\xe3\x8a\x93',
'CjkUnifiedIdeograph66d9': '\xe6\x9b\x99',
'CjkUnifiedIdeograph8a64': '\xe8\xa9\xa4',
'GreekSmallLetterAlphaWithTonos': '\xce\xac',
'CjkUnifiedIdeograph4618': '\xe4\x98\x98',
'CjkUnifiedIdeograph4617': '\xe4\x98\x97',
'CjkUnifiedIdeograph4616': '\xe4\x98\x96',
'CjkUnifiedIdeograph4615': '\xe4\x98\x95',
'CjkUnifiedIdeograph4614': '\xe4\x98\x94',
'CjkUnifiedIdeograph4613': '\xe4\x98\x93',
'CjkUnifiedIdeograph4612': '\xe4\x98\x92',
'CjkUnifiedIdeograph4611': '\xe4\x98\x91',
'CjkUnifiedIdeograph4610': '\xe4\x98\x90',
'UnicodeCharacter0xe490': '\xee\x92\x90',
'HangulSyllableNic': '\xeb\x8b\x9f',
'HangulSyllableNib': '\xeb\x8b\x99',
'HangulSyllablePaes': '\xed\x8c\xbb',
'HangulSyllableNid': '\xeb\x8b\x8f',
'HangulSyllableNig': '\xeb\x8b\x89',
'HangulSyllablePaep': '\xed\x8d\x82',
'HangulSyllableNih': '\xeb\x8b\xa3',
'HangulSyllableNik': '\xeb\x8b\xa0',
'HangulSyllableNij': '\xeb\x8b\x9e',
'HangulSyllableGwibs': '\xea\xb7\x92',
'MongolianLetterOe': '\xe1\xa0\xa5',
'HangulSyllableNin': '\xeb\x8b\x8c',
'HangulSyllablePaeg': '\xed\x8c\xa9',
'HangulSyllableNip': '\xeb\x8b\xa2',
'HangulSyllableNis': '\xeb\x8b\x9b',
'CjkUnifiedIdeograph9389': '\xe9\x8e\x89',
'HangulSyllableJjaeng': '\xec\xa8\x8d',
'HangulSyllableNit': '\xeb\x8b\xa1',
'UnicodeCharacter0xa4e1': '\xea\x93\xa1',
'SquareDmSquared': '\xe3\x8d\xb8',
'HangulSyllableJjaenj': '\xec\xa7\xbd',
'HangulSyllablePaem': '\xed\x8c\xb8',
'HangulSyllableJjaenh': '\xec\xa7\xbe',
'HangulSyllablePaek': '\xed\x8d\x80',
'UnicodeCharacter0xa4e2': '\xea\x93\xa2',
'HangulSyllablePaeh': '\xed\x8d\x83',
'UnicodeCharacter0xa4e3': '\xea\x93\xa3',
'UnicodeCharacter0xe496': '\xee\x92\x96',
'CjkUnifiedIdeograph66db': '\xe6\x9b\x9b',
'CjkUnifiedIdeograph461f': '\xe4\x98\x9f',
'CjkUnifiedIdeograph461e': '\xe4\x98\x9e',
'CjkUnifiedIdeograph461d': '\xe4\x98\x9d',
'CjkUnifiedIdeograph461c': '\xe4\x98\x9c',
'CjkUnifiedIdeograph461b': '\xe4\x98\x9b',
'CjkUnifiedIdeograph461a': '\xe4\x98\x9a',
'CjkUnifiedIdeograph66de': '\xe6\x9b\x9e',
'CjkUnifiedIdeograph48f4': '\xe4\xa3\xb4',
'CjkUnifiedIdeograph48f5': '\xe4\xa3\xb5',
'CjkUnifiedIdeograph48f6': '\xe4\xa3\xb6',
'CjkUnifiedIdeograph48f7': '\xe4\xa3\xb7',
'CjkUnifiedIdeograph48f0': '\xe4\xa3\xb0',
'CjkUnifiedIdeograph48f1': '\xe4\xa3\xb1',
'CjkUnifiedIdeograph48f2': '\xe4\xa3\xb2',
'CjkUnifiedIdeograph48f3': '\xe4\xa3\xb3',
'CjkUnifiedIdeograph8f00': '\xe8\xbc\x80',
'CjkUnifiedIdeograph8f01': '\xe8\xbc\x81',
'SupersetWithPlusSignBelow': '\xe2\xab\x80',
'CjkUnifiedIdeograph8f03': '\xe8\xbc\x83',
'CjkUnifiedIdeograph48f8': '\xe4\xa3\xb8',
'CjkUnifiedIdeograph48f9': '\xe4\xa3\xb9',
'CjkUnifiedIdeograph8f06': '\xe8\xbc\x86',
'CjkUnifiedIdeograph8f07': '\xe8\xbc\x87',
'CjkUnifiedIdeograph9381': '\xe9\x8e\x81',
'CjkCompatibilityIdeographfa4e': '\xef\xa9\x8e',
'CjkUnifiedIdeograph348c': '\xe3\x92\x8c',
'CjkUnifiedIdeograph348b': '\xe3\x92\x8b',
'CjkUnifiedIdeograph348a': '\xe3\x92\x8a',
'CjkUnifiedIdeograph81eb': '\xe8\x87\xab',
'CjkUnifiedIdeograph348f': '\xe3\x92\x8f',
'CjkUnifiedIdeograph348e': '\xe3\x92\x8e',
'CjkUnifiedIdeograph348d': '\xe3\x92\x8d',
'UnicodeCharacter0xe269': '\xee\x89\xa9',
'CombiningDoubleRingBelow': '\xcd\x9a',
'UnicodeCharacter0xe3ec': '\xee\x8f\xac',
'CjkUnifiedIdeograph525a': '\xe5\x89\x9a',
'CjkUnifiedIdeograph525b': '\xe5\x89\x9b',
'CjkUnifiedIdeograph525c': '\xe5\x89\x9c',
'CjkUnifiedIdeograph525d': '\xe5\x89\x9d',
'CjkUnifiedIdeograph525e': '\xe5\x89\x9e',
'CjkUnifiedIdeograph525f': '\xe5\x89\x9f',
'UnicodeCharacter0xe3ed': '\xee\x8f\xad',
'BalineseLetterCaLaca': '\xe1\xac\x99',
'CjkUnifiedIdeograph554e': '\xe5\x95\x8e',
'CjkUnifiedIdeograph3489': '\xe3\x92\x89',
'CjkUnifiedIdeograph3488': '\xe3\x92\x88',
'CjkUnifiedIdeograph3483': '\xe3\x92\x83',
'CjkUnifiedIdeograph3482': '\xe3\x92\x82',
'CjkUnifiedIdeograph3481': '\xe3\x92\x81',
'CjkUnifiedIdeograph3480': '\xe3\x92\x80',
'CjkUnifiedIdeograph3487': '\xe3\x92\x87',
'CjkUnifiedIdeograph3486': '\xe3\x92\x86',
'CjkUnifiedIdeograph3485': '\xe3\x92\x85',
'CjkUnifiedIdeograph3484': '\xe3\x92\x84',
'CjkUnifiedIdeograph5258': '\xe5\x89\x98',
'CjkUnifiedIdeograph5259': '\xe5\x89\x99',
'UnicodeCharacter0xe3e9': '\xee\x8f\xa9',
'UnicodeCharacter0xe3e8': '\xee\x8f\xa8',
'BalineseVowelSignPepet': '\xe1\xad\x82',
'CjkUnifiedIdeograph5250': '\xe5\x89\x90',
'CjkUnifiedIdeograph5251': '\xe5\x89\x91',
'CjkUnifiedIdeograph5252': '\xe5\x89\x92',
'CjkUnifiedIdeograph5253': '\xe5\x89\x93',
'CjkUnifiedIdeograph5254': '\xe5\x89\x94',
'CjkUnifiedIdeograph5255': '\xe5\x89\x95',
'CjkUnifiedIdeograph5256': '\xe5\x89\x96',
'CjkUnifiedIdeograph5257': '\xe5\x89\x97',
'CjkUnifiedIdeograph80cc': '\xe8\x83\x8c',
'HangulSyllableCoegs': '\xec\xb5\x9f',
'HangulSyllableSswebs': '\xec\x92\xae',
'HalfwidthKatakanaLetterNu': '\xef\xbe\x87',
'CjkUnifiedIdeograph6509': '\xe6\x94\x89',
'CjkUnifiedIdeograph6508': '\xe6\x94\x88',
'CjkUnifiedIdeograph6505': '\xe6\x94\x85',
'CjkUnifiedIdeograph6504': '\xe6\x94\x84',
'CjkUnifiedIdeograph6507': '\xe6\x94\x87',
'CjkUnifiedIdeograph6506': '\xe6\x94\x86',
'CjkUnifiedIdeograph6501': '\xe6\x94\x81',
'CjkUnifiedIdeograph6500': '\xe6\x94\x80',
'CjkUnifiedIdeograph6503': '\xe6\x94\x83',
'CjkUnifiedIdeograph6502': '\xe6\x94\x82',
'HangulSyllableMyalh': '\xeb\xa8\x8f',
'FullwidthLatinCapitalLetterD': '\xef\xbc\xa4',
'UnicodeCharacter0xdadf': '\xed\xab\x9f',
'UnicodeCharacter0xa7ec': '\xea\x9f\xac',
'EthiopicSyllableJee': '\xe1\x8c\x84',
'UnicodeCharacter0xa7eb': '\xea\x9f\xab',
'LepchaLetterYa': '\xe1\xb0\x9a',
'LatinSmallLigatureFf': '\xef\xac\x80',
'YiSyllableRrot': '\xea\x8e\x80',
'suba': '\xe2\x82\x90',
'LatinSmallLigatureFl': '\xef\xac\x82',
'UnicodeCharacter0xe635': '\xee\x98\xb5',
'UnicodeCharacter0xdc5e': '\xed\xb1\x9e',
'UnicodeCharacter0xed7b': '\xee\xb5\xbb',
'BengaliLetterDda': '\xe0\xa6\xa1',
'UnicodeCharacter0xa7ee': '\xea\x9f\xae',
'UnicodeCharacter0xed7c': '\xee\xb5\xbc',
'UnicodeCharacter0xeb27': '\xee\xac\xa7',
'UnicodeCharacter0xe50a': '\xee\x94\x8a',
'UnicodeCharacter0xa7ed': '\xea\x9f\xad',
'CjkUnifiedIdeograph650e': '\xe6\x94\x8e',
'CjkUnifiedIdeograph650d': '\xe6\x94\x8d',
'CopticCapitalLetterVida': '\xe2\xb2\x82',
'CjkUnifiedIdeograph650a': '\xe6\x94\x8a',
'CjkUnifiedIdeograph650c': '\xe6\x94\x8c',
'ArabicLetterNoonGhunna': '\xda\xba',
'LatinCapitalLetterWWithCircumflex': '\xc5\xb4',
'HangulSyllableGe': '\xea\xb2\x8c',
'ArabicLetterAlefWithMaddaAboveIsolatedForm': '\xef\xba\x81',
'UnicodeCharacter0xed7e': '\xee\xb5\xbe',
'ArabicLigatureSadWithMeemInitialForm': '\xef\xb2\xb3',
'BalineseLetterSaSaga': '\xe1\xac\xb0',
'ArabicLetterThalFinalForm': '\xef\xba\xac',
'GeorgianCapitalLetterShin': '\xe1\x82\xb8',
'WhiteCircleWithTwoDots': '\xe2\x9a\x87',
'UnicodeCharacter0xa4ea': '\xea\x93\xaa',
'UnicodeCharacter0xa4eb': '\xea\x93\xab',
'UnicodeCharacter0xa4ec': '\xea\x93\xac',
'UnicodeCharacter0xa4ed': '\xea\x93\xad',
'HangulSyllableSsyanh': '\xec\x8c\xb6',
'UnicodeCharacter0xa4ee': '\xea\x93\xae',
'LatinSmallLetterThornWithStrokeThroughDescender': '\xea\x9d\xa7',
'UnicodeCharacter0xa4ef': '\xea\x93\xaf',
'HangulSyllableSswan': '\xec\x8f\xb8',
'HangulSyllablePubs': '\xed\x92\x8a',
'LatinCapitalLetterDz': '\xc7\xb1',
'HangulSyllableDilm': '\xeb\x94\x9e',
'HangulSyllableDilh': '\xeb\x94\xa3',
'UnicodeCharacter0xe5e3': '\xee\x97\xa3',
'HangulSyllableDilg': '\xeb\x94\x9d',
'EthiopicSyllableLwa': '\xe1\x88\x8f',
'CjkUnifiedIdeograph3c4a': '\xe3\xb1\x8a',
'CjkUnifiedIdeograph3c4b': '\xe3\xb1\x8b',
'CjkUnifiedIdeograph3c4c': '\xe3\xb1\x8c',
'CjkUnifiedIdeograph3c4d': '\xe3\xb1\x8d',
'CjkUnifiedIdeograph3c4e': '\xe3\xb1\x8e',
'CjkUnifiedIdeograph3c4f': '\xe3\xb1\x8f',
'CjkUnifiedIdeograph7674': '\xe7\x99\xb4',
'CjkUnifiedIdeograph7675': '\xe7\x99\xb5',
'CjkUnifiedIdeograph7676': '\xe7\x99\xb6',
'CjkUnifiedIdeograph7677': '\xe7\x99\xb7',
'CjkUnifiedIdeograph7670': '\xe7\x99\xb0',
'CjkUnifiedIdeograph7671': '\xe7\x99\xb1',
'CjkUnifiedIdeograph7672': '\xe7\x99\xb2',
'CjkUnifiedIdeograph7673': '\xe7\x99\xb3',
'GreaterthanOrEqualTo': '\xe2\x89\xa5',
'FullwidthLessthanSign': '\xef\xbc\x9c',
'CjkUnifiedIdeograph8e6a': '\xe8\xb9\xaa',
'HangulSyllableGgyap': '\xea\xba\x9e',
'HangulSyllableTyubs': '\xed\x8a\xae',
'HangulSyllableDdoss': '\xeb\x98\xa4',
'CjkUnifiedIdeograph3c40': '\xe3\xb1\x80',
'CjkUnifiedIdeograph3c41': '\xe3\xb1\x81',
'CjkUnifiedIdeograph3c42': '\xe3\xb1\x82',
'CjkUnifiedIdeograph3c43': '\xe3\xb1\x83',
'CjkUnifiedIdeograph3c44': '\xe3\xb1\x84',
'CjkUnifiedIdeograph3c45': '\xe3\xb1\x85',
'CjkUnifiedIdeograph3c46': '\xe3\xb1\x86',
'ArabicSmallHighYeh': '\xdb\xa7',
'CjkUnifiedIdeograph3c48': '\xe3\xb1\x88',
'CjkUnifiedIdeograph3c49': '\xe3\xb1\x89',
'CjkUnifiedIdeograph767f': '\xe7\x99\xbf',
'CjkUnifiedIdeograph62ec': '\xe6\x8b\xac',
'CjkUnifiedIdeograph767b': '\xe7\x99\xbb',
'CjkUnifiedIdeograph767c': '\xe7\x99\xbc',
'SubscriptSix': '\xe2\x82\x86',
'UnicodeCharacter0xe8ca': '\xee\xa3\x8a',
'CjkUnifiedIdeograph8053': '\xe8\x81\x93',
'UnicodeCharacter0xe8cf': '\xee\xa3\x8f',
'UnicodeCharacter0xf68e': '\xef\x9a\x8e',
'ArabicLigatureFehWithMeemInitialForm': '\xef\xb3\x81',
'FullwidthWonSign': '\xef\xbf\xa6',
'ModifierLetterSmallGamma': '\xcb\xa0',
'ArabicLigatureTehWithHehMedialForm': '\xef\xb3\xa4',
'BengaliVowelSignE': '\xe0\xa7\x87',
'HangulSyllableBbyaes': '\xeb\xbb\x87',
'BengaliVowelSignI': '\xe0\xa6\xbf',
'UnicodeCharacter0xe8ce': '\xee\xa3\x8e',
'BengaliVowelSignO': '\xe0\xa7\x8b',
'HangulSyllableBbyaep': '\xeb\xbb\x8e',
'HangulSyllableJjyinj': '\xec\xaf\xb5',
'BengaliVowelSignU': '\xe0\xa7\x81',
'SingleHighreversed9QuotationMark': '\xe2\x80\x9b',
'UnicodeCharacter0xf68f': '\xef\x9a\x8f',
'CjkUnifiedIdeograph996a': '\xe9\xa5\xaa',
'TagalogLetterGa': '\xe1\x9c\x84',
'HangulSyllableHwab': '\xed\x99\xa5',
'GujaratiLetterCa': '\xe0\xaa\x9a',
'HangulSyllableByolt': '\xeb\xb5\xb1',
'CjkUnifiedIdeograph6e18': '\xe6\xb8\x98',
'CjkUnifiedIdeograph6e19': '\xe6\xb8\x99',
'CjkUnifiedIdeograph6e12': '\xe6\xb8\x92',
'CjkUnifiedIdeograph6e13': '\xe6\xb8\x93',
'CjkUnifiedIdeograph6e10': '\xe6\xb8\x90',
'CjkUnifiedIdeograph6e11': '\xe6\xb8\x91',
'CjkUnifiedIdeograph6e16': '\xe6\xb8\x96',
'CjkUnifiedIdeograph6e17': '\xe6\xb8\x97',
'CjkUnifiedIdeograph6e14': '\xe6\xb8\x94',
'CjkUnifiedIdeograph6e15': '\xe6\xb8\x95',
'UnicodeCharacter0xa7e3': '\xea\x9f\xa3',
'CjkUnifiedIdeograph83c3': '\xe8\x8f\x83',
'UnicodeCharacter0xa7e2': '\xea\x9f\xa2',
'HangulSyllableTyin': '\xed\x8b\x98',
'HangulSyllableByolp': '\xeb\xb5\xb2',
'UnicodeCharacter0xa7e1': '\xea\x9f\xa1',
'Assertion': '\xe2\x8a\xa6',
'KangxiRadicalValley': '\xe2\xbe\x95',
'CjkUnifiedIdeograph74ba': '\xe7\x92\xba',
'LeftwardsArrowtail': '\xe2\xa4\x99',
'HangulSyllableTweh': '\xed\x89\xbf',
'UnicodeCharacter0xa7e7': '\xea\x9f\xa7',
'HangulSyllableByols': '\xeb\xb5\xb0',
'CanadianSyllabicsCarrierInitialZ': '\xe1\x99\x87',
'CjkUnifiedIdeograph3f57': '\xe3\xbd\x97',
'SquareKoopo': '\xe3\x8c\x9e',
'CjkUnifiedIdeograph74bb': '\xe7\x92\xbb',
'LatinCapitalLetterGlottalStop': '\xc9\x81',
'UnicodeCharacter0xa7e5': '\xea\x9f\xa5',
'SylotiNagriLetterPho': '\xea\xa0\x9a',
'HangulSyllableByolm': '\xeb\xb5\xae',
'CjkUnifiedIdeograph6e1b': '\xe6\xb8\x9b',
'CjkUnifiedIdeograph6e1c': '\xe6\xb8\x9c',
'ArabicLetterHahWithTwoDotsVerticalAbove': '\xda\x82',
'CjkUnifiedIdeograph6e1f': '\xe6\xb8\x9f',
'CjkUnifiedIdeograph6e1d': '\xe6\xb8\x9d',
'CjkUnifiedIdeograph6e1e': '\xe6\xb8\x9e',
'HangulSyllableTweb': '\xed\x89\xb5',
'FullwidthComma': '\xef\xbc\x8c',
'HangulSyllableNyae': '\xeb\x83\xac',
'CjkUnifiedIdeograph74bf': '\xe7\x92\xbf',
'CjkUnifiedIdeograph9d24': '\xe9\xb4\xa4',
'UnicodeCharacter0xa7e9': '\xea\x9f\xa9',
'HangulSyllableBbyaeg': '\xeb\xba\xb5',
'UnicodeCharacter0xa7e8': '\xea\x9f\xa8',
'HangulSyllableTyil': '\xed\x8b\x9c',
'HangulSyllableDdyobs': '\xeb\x9a\x92',
'HangulSyllableMwang': '\xeb\xab\x99',
'UnicodeCharacter0xe46e': '\xee\x91\xae',
'UnicodeCharacter0xf68b': '\xef\x9a\x8b',
'HangulSyllableKyeolh': '\xec\xbc\xab',
'CjkUnifiedIdeograph4a29': '\xe4\xa8\xa9',
'CjkUnifiedIdeograph4a28': '\xe4\xa8\xa8',
'HangulSyllableBbyaek': '\xeb\xbb\x8c',
'CjkUnifiedIdeograph4a25': '\xe4\xa8\xa5',
'CjkUnifiedIdeograph4a24': '\xe4\xa8\xa4',
'CjkUnifiedIdeograph4a27': '\xe4\xa8\xa7',
'CjkUnifiedIdeograph4a26': '\xe4\xa8\xa6',
'CjkUnifiedIdeograph4a21': '\xe4\xa8\xa1',
'CjkUnifiedIdeograph4a20': '\xe4\xa8\xa0',
'CjkUnifiedIdeograph4a23': '\xe4\xa8\xa3',
'CjkUnifiedIdeograph4a22': '\xe4\xa8\xa2',
'HangulSyllableCyis': '\xec\xb9\x8f',
'HangulSyllableCyip': '\xec\xb9\x96',
'HangulSyllableByolg': '\xeb\xb5\xad',
'HangulSyllableKyeolm': '\xec\xbc\xa6',
'HangulSyllableCyit': '\xec\xb9\x95',
'UnicodeCharacter0xedbe': '\xee\xb6\xbe',
'HangulSyllableBbyaen': '\xeb\xba\xb8',
'UnicodeCharacter0xedbf': '\xee\xb6\xbf',
'UnicodeCharacter0xedba': '\xee\xb6\xba',
'UnicodeCharacter0xedbc': '\xee\xb6\xbc',
'UnicodeCharacter0xedbb': '\xee\xb6\xbb',
'HangulSyllableCyib': '\xec\xb9\x8d',
'HangulSyllableCyic': '\xec\xb9\x93',
'DoublePrime': '\xe2\x80\xb3',
'HangulSyllableBbyael': '\xeb\xba\xbc',
'HangulSyllableCyid': '\xec\xb9\x83',
'CjkUnifiedIdeograph9cbc': '\xe9\xb2\xbc',
'HangulSyllableCyij': '\xec\xb9\x92',
'HangulSyllableCyik': '\xec\xb9\x94',
'HangulSyllableCyih': '\xec\xb9\x97',
'HangulSyllableBbyaem': '\xeb\xbb\x84',
'HangulSyllableCyin': '\xec\xb9\x80',
'HangulSyllableGgyaed': '\xea\xba\xa7',
'HangulSyllableBbwigg': '\xec\x80\xa2',
'ParenthesizedNumberTen': '\xe2\x91\xbd',
'CjkUnifiedIdeograph8ef9': '\xe8\xbb\xb9',
'YiSyllableVyt': '\xea\x83\xba',
'YiSyllableVyr': '\xea\x83\xbf',
'YiSyllableVyp': '\xea\x83\xbd',
'CjkUnifiedIdeograph8ef8': '\xe8\xbb\xb8',
'CjkUnifiedIdeograph4a2e': '\xe4\xa8\xae',
'CjkUnifiedIdeograph4a2d': '\xe4\xa8\xad',
'CjkUnifiedIdeograph4a2f': '\xe4\xa8\xaf',
'CjkUnifiedIdeograph4a2a': '\xe4\xa8\xaa',
'CjkUnifiedIdeograph4a2c': '\xe4\xa8\xac',
'CjkUnifiedIdeograph4a2b': '\xe4\xa8\xab',
'CjkUnifiedIdeograph9cbd': '\xe9\xb2\xbd',
'LightRightTortoiseShellBracketOrnament': '\xe2\x9d\xb3',
'HangulSyllableSsyiss': '\xec\x94\xa0',
'BlackfeatheredNorthEastArrow': '\xe2\x9e\xb6',
'HangulSyllableHyeod': '\xed\x98\x87',
'KhmerSignAhsda': '\xe1\x9f\x8f',
'HangulSyllableByalt': '\xeb\xb1\x99',
'HangulSyllableMod': '\xeb\xaa\xaf',
'CjkUnifiedIdeograph9cbe': '\xe9\xb2\xbe',
'UnicodeCharacter0xedb9': '\xee\xb6\xb9',
'UnicodeCharacter0xedb8': '\xee\xb6\xb8',
'CjkUnifiedIdeograph8ef0': '\xe8\xbb\xb0',
'UnicodeCharacter0xedb5': '\xee\xb6\xb5',
'UnicodeCharacter0xedb4': '\xee\xb6\xb4',
'UnicodeCharacter0xedb7': '\xee\xb6\xb7',
'UnicodeCharacter0xedb6': '\xee\xb6\xb6',
'UnicodeCharacter0xedb1': '\xee\xb6\xb1',
'UnicodeCharacter0xedb0': '\xee\xb6\xb0',
'HangulSyllableCoeb': '\xec\xb5\xad',
'UnicodeCharacter0xedb2': '\xee\xb6\xb2',
'LineIntegrationWithSemicircularPathAroundPole': '\xe2\xa8\x93',
'CjkUnifiedIdeograph8b0d': '\xe8\xac\x8d',
'ParenthesizedHangulCieuc': '\xe3\x88\x88',
'HangulSyllableMog': '\xeb\xaa\xa9',
'UnicodeCharacter0xdbd6': '\xed\xaf\x96',
'UnicodeCharacter0xdbd7': '\xed\xaf\x97',
'UnicodeCharacter0xdbd4': '\xed\xaf\x94',
'CjkUnifiedIdeograph931d': '\xe9\x8c\x9d',
'HangulSyllableNyeng': '\xeb\x85\xb1',
'UnicodeCharacter0xdbd8': '\xed\xaf\x98',
'UnicodeCharacter0xdbd9': '\xed\xaf\x99',
'YiSyllableJyx': '\xea\x8f\xb7',
'CjkUnifiedIdeograph9cbf': '\xe9\xb2\xbf',
'HangulSyllableMob': '\xeb\xaa\xb9',
'LatinEpigraphicLetterInvertedM': '\xea\x9f\xbd',
'HangulSyllableMoc': '\xeb\xaa\xbf',
'CjkUnifiedIdeograph8b0e': '\xe8\xac\x8e',
'UnicodeCharacter0xe46d': '\xee\x91\xad',
'CjkUnifiedIdeograph931c': '\xe9\x8c\x9c',
'CjkUnifiedIdeograph931b': '\xe9\x8c\x9b',
'UnicodeCharacter0x208f': '\xe2\x82\x8f',
'CjkUnifiedIdeograph956c': '\xe9\x95\xac',
'UnicodeCharacter0xf862': '\xef\xa1\xa2',
'LatinSmallLetterUWithMacronAndDiaeresis': '\xe1\xb9\xbb',
'HangulSyllablePyuj': '\xed\x93\xbe',
'HangulSyllableSwih': '\xec\x8a\x87',
'HangulSyllableGgaels': '\xea\xb9\xb4',
'HangulSyllableSwij': '\xec\x8a\x82',
'HangulSyllableSwik': '\xec\x8a\x84',
'HangulSyllableSwil': '\xec\x89\xb4',
'HangulSyllableSwim': '\xec\x89\xbc',
'HangulSyllableSwin': '\xec\x89\xb0',
'UnicodeCharacter0xe3fa': '\xee\x8f\xba',
'HangulSyllableSwib': '\xec\x89\xbd',
'HangulSyllableSwic': '\xec\x8a\x83',
'HangulSyllableSwid': '\xec\x89\xb3',
'HangulSyllableCyed': '\xec\xb3\xb3',
'HangulSyllableSwig': '\xec\x89\xad',
'UnicodeCharacter0xdbdb': '\xed\xaf\x9b',
'HangulSyllablePyug': '\xed\x93\xa9',
'UnicodeCharacter0xdbda': '\xed\xaf\x9a',
'UnicodeCharacter0xdbdf': '\xed\xaf\x9f',
'UnicodeCharacter0xdbdd': '\xed\xaf\x9d',
'UnicodeCharacter0xdbde': '\xed\xaf\x9e',
'BalineseMusicalSymbolDing': '\xe1\xad\xa6',
'HangulSyllableSwis': '\xec\x89\xbf',
'HangulSyllableSwit': '\xec\x8a\x85',
'HangulSyllablePyud': '\xed\x93\xaf',
'HangulSyllablePyuc': '\xed\x93\xbf',
'HangulSyllableGgaelt': '\xea\xb9\xb5',
'CjkUnifiedIdeograph956a': '\xe9\x95\xaa',
'HangulSyllablePyub': '\xed\x93\xb9',
'CjkUnifiedIdeograph74b1': '\xe7\x92\xb1',
'HangulSyllableGgyaeb': '\xea\xba\xb1',
'CjkUnifiedIdeograph74b0': '\xe7\x92\xb0',
'UnicodeCharacter0xf41b': '\xef\x90\x9b',
'CjkUnifiedIdeograph74b3': '\xe7\x92\xb3',
'CjkUnifiedIdeograph8459': '\xe8\x91\x99',
'CjkUnifiedIdeograph8458': '\xe8\x91\x98',
'UnicodeCharacter0xeb52': '\xee\xad\x92',
'CjkUnifiedIdeograph74b2': '\xe7\x92\xb2',
'UnicodeCharacter0xf021': '\xef\x80\xa1',
'CjkUnifiedIdeograph8451': '\xe8\x91\x91',
'LatinCapitalLetterZWithLineBelow': '\xe1\xba\x94',
'CjkUnifiedIdeograph8453': '\xe8\x91\x93',
'CjkUnifiedIdeograph8452': '\xe8\x91\x92',
'CjkUnifiedIdeograph8455': '\xe8\x91\x95',
'CjkUnifiedIdeograph8454': '\xe8\x91\x94',
'CjkUnifiedIdeograph8457': '\xe8\x91\x97',
'CjkUnifiedIdeograph74b4': '\xe7\x92\xb4',
'HangulSyllableGgyaec': '\xea\xba\xb7',
'CjkUnifiedIdeograph74b7': '\xe7\x92\xb7',
'LogicalAndWithDoubleOverbar': '\xe2\xa9\x9e',
'CjkUnifiedIdeograph74b6': '\xe7\x92\xb6',
'ArabicLetterLamIsolatedForm': '\xef\xbb\x9d',
'HangulSyllableCyec': '\xec\xb4\x83',
'UnicodeCharacter0x2b9c': '\xe2\xae\x9c',
'UnicodeCharacter0x2b9b': '\xe2\xae\x9b',
'UnicodeCharacter0x2b9a': '\xe2\xae\x9a',
'UnicodeCharacter0x2b9f': '\xe2\xae\x9f',
'UnicodeCharacter0x2b9e': '\xe2\xae\x9e',
'UnicodeCharacter0x2b9d': '\xe2\xae\x9d',
'HangulSyllableRyang': '\xeb\x9f\x89',
'DevanagariLetterDdha': '\xe0\xa4\xa2',
'UnicodeCharacter0xdf6a': '\xed\xbd\xaa',
'LatinCapitalLetterLWithMiddleDot': '\xc4\xbf',
'CjkUnifiedIdeograph956d': '\xe9\x95\xad',
'ArabicLigatureLamWithHehInitialForm': '\xef\xb3\x8d',
'CjkUnifiedIdeograph845a': '\xe8\x91\x9a',
'SyriacPthahaDotted': '\xdc\xb2',
'CjkUnifiedIdeograph845c': '\xe8\x91\x9c',
'CjkUnifiedIdeograph845b': '\xe8\x91\x9b',
'CjkUnifiedIdeograph845e': '\xe8\x91\x9e',
'CjkUnifiedIdeograph845d': '\xe8\x91\x9d',
'CjkUnifiedIdeograph845f': '\xe8\x91\x9f',
'UnicodeCharacter0x2b93': '\xe2\xae\x93',
'UnicodeCharacter0x2b92': '\xe2\xae\x92',
'UnicodeCharacter0x2b91': '\xe2\xae\x91',
'UnicodeCharacter0x2b90': '\xe2\xae\x90',
'UnicodeCharacter0x2b97': '\xe2\xae\x97',
'UnicodeCharacter0x2b96': '\xe2\xae\x96',
'UnicodeCharacter0x2b95': '\xe2\xae\x95',
'UnicodeCharacter0x2b94': '\xe2\xae\x94',
'CopticSmallLetterDialectpAlef': '\xe2\xb2\xb3',
'UnicodeCharacter0x2b98': '\xe2\xae\x98',
'SundaneseLetterZa': '\xe1\xae\x90',
'CjkCompatibilityIdeographf996': '\xef\xa6\x96',
'HangulSyllableGgaelm': '\xea\xb9\xb2',
'UnicodeCharacter0x12d7': '\xe1\x8b\x97',
'KangxiRadicalLine': '\xe2\xbc\x81',
'UnicodeCharacter0xa7bf': '\xea\x9e\xbf',
'CjkUnifiedIdeograph3f2f': '\xe3\xbc\xaf',
'CjkUnifiedIdeograph3f2e': '\xe3\xbc\xae',
'CjkUnifiedIdeograph3f2d': '\xe3\xbc\xad',
'CjkUnifiedIdeograph3f2c': '\xe3\xbc\xac',
'CjkUnifiedIdeograph3f2b': '\xe3\xbc\xab',
'CjkUnifiedIdeograph3f2a': '\xe3\xbc\xaa',
'UnicodeCharacter0xa7ba': '\xea\x9e\xba',
'GreekSmallLetterIotaWithTonos': '\xce\xaf',
'ArabicLigatureDadWithHahInitialForm': '\xef\xb2\xb5',
'HangulSyllableDeoj': '\xeb\x8d\xaa',
'HangulSyllableNe': '\xeb\x84\xa4',
'CjkCompatibilityIdeographf993': '\xef\xa6\x93',
'HangulSyllableDeod': '\xeb\x8d\x9b',
'HangulSyllableNu': '\xeb\x88\x84',
'HangulSyllableDeog': '\xeb\x8d\x95',
'UnicodeCharacter0xdf6d': '\xed\xbd\xad',
'HangulSyllableCyeo': '\xec\xb3\x90',
'CanadianSyllabicsCarrierGee': '\xe1\x97\xb2',
'HangulSyllableBbwib': '\xec\x80\xb1',
'UnicodeCharacter0x1c3a': '\xe1\xb0\xba',
'CjkUnifiedIdeograph847b': '\xe8\x91\xbb',
'SaurashtraVowelSignOo': '\xea\xa3\x82',
'HangulSyllableDeoc': '\xeb\x8d\xab',
'ArabicFatha': '\xd9\x8e',
'VaiSyllableUn': '\xea\x96\x96',
'HangulSyllableCyel': '\xec\xb3\xb4',
'H': '\xce\x97',
'CyrillicCapitalLetterAbkhasianHa': '\xd2\xa8',
'CjkUnifiedIdeograph5bc3': '\xe5\xaf\x83',
'CjkUnifiedIdeograph3f27': '\xe3\xbc\xa7',
'CjkUnifiedIdeograph3f26': '\xe3\xbc\xa6',
'CjkUnifiedIdeograph3f25': '\xe3\xbc\xa5',
'CjkUnifiedIdeograph3f24': '\xe3\xbc\xa4',
'CjkUnifiedIdeograph3f23': '\xe3\xbc\xa3',
'CjkUnifiedIdeograph3f22': '\xe3\xbc\xa2',
'CjkUnifiedIdeograph3f21': '\xe3\xbc\xa1',
'CjkUnifiedIdeograph3f20': '\xe3\xbc\xa0',
'Interrobang': '\xe2\x80\xbd',
'HangulSyllableCyem': '\xec\xb3\xbc',
'CjkUnifiedIdeograph3f29': '\xe3\xbc\xa9',
'CjkUnifiedIdeograph3f28': '\xe3\xbc\xa8',
'TeluguLetterJa': '\xe0\xb0\x9c',
'LatinSmallLetterClosedOpenE': '\xca\x9a',
'YiSyllableHmiex': '\xea\x82\x95',
'YiSyllableHmiep': '\xea\x82\x97',
'CjkUnifiedIdeograph7c74': '\xe7\xb1\xb4',
'UnicodeCharacter0x1c39': '\xe1\xb0\xb9',
'UnicodeCharacter0x1c38': '\xe1\xb0\xb8',
'CjkUnifiedIdeograph91ce': '\xe9\x87\x8e',
'HangulSyllableYej': '\xec\x98\x9e',
'DoubleNestedLessthanWithUnderbar': '\xe2\xaa\xa3',
'HangulSyllableYeh': '\xec\x98\xa3',
'LatinSmallLetterDum': '\xea\x9d\xb1',
'HangulSyllableBwenj': '\xeb\xb6\xbd',
'ShortDownTackWithOverbar': '\xe2\xab\xa7',
'HangulSyllableBwenh': '\xeb\xb6\xbe',
'HangulSyllableYem': '\xec\x98\x98',
'HangulSyllableNyugs': '\xeb\x89\xb7',
'HangulSyllableBweng': '\xeb\xb7\x8d',
'EthiopicSyllableRwa': '\xe1\x88\xaf',
'HangulSyllableYeg': '\xec\x98\x89',
'HangulSyllableYed': '\xec\x98\x8f',
'UnicodeCharacter0xed87': '\xee\xb6\x87',
'CjkUnifiedIdeograph665d': '\xe6\x99\x9d',
'HangulSyllableYes': '\xec\x98\x9b',
'HangulSyllableYep': '\xec\x98\xa2',
'GeorgianCapitalLetterChar': '\xe1\x82\xbd',
'CjkUnifiedIdeograph91cd': '\xe9\x87\x8d',
'HangulSyllableCyaes': '\xec\xb2\x8f',
'HangulSyllableGgyaet': '\xea\xba\xb9',
'UnicodeCharacter0xe858': '\xee\xa1\x98',
'UnicodeCharacter0xe859': '\xee\xa1\x99',
'UnicodeCharacter0xed84': '\xee\xb6\x84',
'CjkUnifiedIdeograph7c97': '\xe7\xb2\x97',
'UnicodeCharacter0xe850': '\xee\xa1\x90',
'HangulSyllableCweolm': '\xec\xb6\xba',
'HangulSyllableJweonh': '\xec\xa4\x9e',
'UnicodeCharacter0xe853': '\xee\xa1\x93',
'UnicodeCharacter0xe854': '\xee\xa1\x94',
'UnicodeCharacter0xe855': '\xee\xa1\x95',
'UnicodeCharacter0xe856': '\xee\xa1\x96',
'UnicodeCharacter0xe857': '\xee\xa1\x97',
'CjkUnifiedIdeograph6b5a': '\xe6\xad\x9a',
'HangulSyllableBweolb': '\xeb\xb6\xa7',
'CjkUnifiedIdeograph6b5c': '\xe6\xad\x9c',
'ArabicLetterLamWithBar': '\xdd\xaa',
'CjkUnifiedIdeograph6b5e': '\xe6\xad\x9e',
'CjkUnifiedIdeograph6b5d': '\xe6\xad\x9d',
'CjkUnifiedIdeograph6b5f': '\xe6\xad\x9f',
'UnicodeCharacter0xed80': '\xee\xb6\x80',
'T': '\xce\xa4',
'LimbuSubjoinedLetterYa': '\xe1\xa4\xa9',
'UnicodeCharacter0x12b1': '\xe1\x8a\xb1',
'YiSyllableWap': '\xea\x8a\x83',
'CjkUnifiedIdeograph79af': '\xe7\xa6\xaf',
'subgamma': '\xe1\xb5\xa7',
'UnicodeCharacter0xda0e': '\xed\xa8\x8e',
'CjkUnifiedIdeograph6b51': '\xe6\xad\x91',
'CjkUnifiedIdeograph6b50': '\xe6\xad\x90',
'CjkUnifiedIdeograph6b53': '\xe6\xad\x93',
'CjkUnifiedIdeograph6b52': '\xe6\xad\x92',
'CjkUnifiedIdeograph6b55': '\xe6\xad\x95',
'CjkUnifiedIdeograph6b54': '\xe6\xad\x94',
'CjkUnifiedIdeograph6b57': '\xe6\xad\x97',
'CjkUnifiedIdeograph6b56': '\xe6\xad\x96',
'CjkUnifiedIdeograph6b59': '\xe6\xad\x99',
'CjkUnifiedIdeograph6b58': '\xe6\xad\x98',
'HangulSyllableJyaep': '\xec\x9f\xbe',
'CjkUnifiedIdeograph6161': '\xe6\x85\xa1',
'HangulSyllablePyebs': '\xed\x8f\xa2',
'CjkUnifiedIdeograph9316': '\xe9\x8c\x96',
'CjkUnifiedIdeograph86c9': '\xe8\x9b\x89',
'CjkUnifiedIdeograph91ca': '\xe9\x87\x8a',
'ArabicLigatureNoonWithMeemInitialForm': '\xef\xb3\x95',
'CjkUnifiedIdeograph9311': '\xe9\x8c\x91',
'UnicodeCharacter0xe85b': '\xee\xa1\x9b',
'UnicodeCharacter0xe85c': '\xee\xa1\x9c',
'HangulSyllableJeugg': '\xec\xa6\x8a',
'UnicodeCharacter0xda0d': '\xed\xa8\x8d',
'UnicodeCharacter0xe85f': '\xee\xa1\x9f',
'CanadianSyllabicsCarrierGu': '\xe1\x97\xaf',
'CjkUnifiedIdeograph9313': '\xe9\x8c\x93',
'CjkUnifiedIdeograph89c3': '\xe8\xa7\x83',
'YiSyllableNziep': '\xea\x8b\xb3',
'HangulSyllablePwaegs': '\xed\x90\xa7',
'HangulSyllableDdwat': '\xeb\x99\x85',
'LatinCapitalLetterUWithDiaeresisAndAcute': '\xc7\x97',
'HangulSyllableDdwas': '\xeb\x98\xbf',
'HangulSyllableDdwap': '\xeb\x99\x86',
'YiSyllableNziex': '\xea\x8b\xb1',
'CjkUnifiedIdeograph4b4d': '\xe4\xad\x8d',
'CjkUnifiedIdeograph4b4e': '\xe4\xad\x8e',
'CjkUnifiedIdeograph4b4f': '\xe4\xad\x8f',
'HangulSyllableDdwam': '\xeb\x98\xbc',
'HangulSyllableDdwaj': '\xeb\x99\x82',
'CjkUnifiedIdeograph4b4a': '\xe4\xad\x8a',
'CjkUnifiedIdeograph4b4b': '\xe4\xad\x8b',
'CjkUnifiedIdeograph4b4c': '\xe4\xad\x8c',
'HangulSyllableDdwag': '\xeb\x98\xad',
'HangulSyllableDdwad': '\xeb\x98\xb3',
'HangulSyllableDdwae': '\xeb\x99\x88',
'KangxiRadicalLame': '\xe2\xbc\xaa',
'CombiningSuspensionMark': '\xe1\xb7\x83',
'CjkUnifiedIdeograph82b6': '\xe8\x8a\xb6',
'MalayalamLetterLa': '\xe0\xb4\xb2',
'CjkUnifiedIdeograph82b4': '\xe8\x8a\xb4',
'CjkUnifiedIdeograph82b5': '\xe8\x8a\xb5',
'CjkUnifiedIdeograph82b2': '\xe8\x8a\xb2',
'CjkUnifiedIdeograph82b3': '\xe8\x8a\xb3',
'CjkUnifiedIdeograph82b0': '\xe8\x8a\xb0',
'CjkUnifiedIdeograph82b1': '\xe8\x8a\xb1',
'CjkUnifiedIdeograph93cc': '\xe9\x8f\x8c',
'CjkUnifiedIdeograph93cb': '\xe9\x8f\x8b',
'CjkUnifiedIdeograph93ca': '\xe9\x8f\x8a',
'CjkUnifiedIdeograph93cf': '\xe9\x8f\x8f',
'ArabicLetterWawWithHamzaAbove': '\xd8\xa4',
'CjkUnifiedIdeograph82b9': '\xe8\x8a\xb9',
'UnicodeCharacter0xd81c': '\xed\xa0\x9c',
'HangulSyllableGilh': '\xea\xb8\xbf',
'UnicodeCharacter0xd81a': '\xed\xa0\x9a',
'UnicodeCharacter0xf144': '\xef\x85\x84',
'HebrewLetterSamekh': '\xd7\xa1',
'UnicodeCharacter0xd81f': '\xed\xa0\x9f',
'UnicodeCharacter0xd81e': '\xed\xa0\x9e',
'UnicodeCharacter0xd81d': '\xed\xa0\x9d',
'RunicLetterLaukazLaguLogrL': '\xe1\x9b\x9a',
'UnicodeCharacter0xda0f': '\xed\xa8\x8f',
'HangulSyllableGilb': '\xea\xb8\xbb',
'YiRadicalJjy': '\xea\x92\xb5',
'HangulSyllableGilg': '\xea\xb8\xb9',
'CjkUnifiedIdeograph4b44': '\xe4\xad\x84',
'CjkUnifiedIdeograph4b45': '\xe4\xad\x85',
'CjkUnifiedIdeograph4b46': '\xe4\xad\x86',
'CjkUnifiedIdeograph4b47': '\xe4\xad\x87',
'CjkUnifiedIdeograph4b40': '\xe4\xad\x80',
'CjkUnifiedIdeograph4b41': '\xe4\xad\x81',
'CjkUnifiedIdeograph4b42': '\xe4\xad\x82',
'CjkUnifiedIdeograph4b43': '\xe4\xad\x83',
'ParenthesizedHangulSiosA': '\xe3\x88\x94',
'CjkUnifiedIdeograph4b48': '\xe4\xad\x88',
'CjkUnifiedIdeograph4b49': '\xe4\xad\x89',
'UnicodeCharacter0xaae1': '\xea\xab\xa1',
'UnicodeCharacter0xaae0': '\xea\xab\xa0',
'UnicodeCharacter0xaae3': '\xea\xab\xa3',
'BengaliLetterNga': '\xe0\xa6\x99',
'HangulSyllableRyaed': '\xeb\x9f\x97',
'HangulSyllableRyaeg': '\xeb\x9f\x91',
'UnicodeCharacter0xaae2': '\xea\xab\xa2',
'UnicodeCharacter0xd817': '\xed\xa0\x97',
'UnicodeCharacter0xd816': '\xed\xa0\x96',
'HangulSyllableRyaec': '\xeb\x9f\xa7',
'HangulSyllableRyaeb': '\xeb\x9f\xa1',
'HangulSyllableRyaem': '\xeb\x9f\xa0',
'UnicodeCharacter0xaae5': '\xea\xab\xa5',
'HangulSyllableGgeut': '\xeb\x81\x9d',
'HangulSyllableRyaen': '\xeb\x9f\x94',
'HangulSyllableGgeus': '\xeb\x81\x97',
'HangulSyllableGgeup': '\xeb\x81\x9e',
'UnicodeCharacter0xaae4': '\xea\xab\xa4',
'CjkUnifiedIdeograph82bf': '\xe8\x8a\xbf',
'HangulSyllableRyaet': '\xeb\x9f\xa9',
'CjkUnifiedIdeograph82bd': '\xe8\x8a\xbd',
'CjkUnifiedIdeograph82be': '\xe8\x8a\xbe',
'CjkUnifiedIdeograph82bb': '\xe8\x8a\xbb',
'CjkUnifiedIdeograph82bc': '\xe8\x8a\xbc',
'HangulSyllableGgeuh': '\xeb\x81\x9f',
'CjkUnifiedIdeograph82ba': '\xe8\x8a\xba',
'CjkUnifiedIdeograph93c3': '\xe9\x8f\x83',
'CjkUnifiedIdeograph93c2': '\xe9\x8f\x82',
'CjkUnifiedIdeograph93c1': '\xe9\x8f\x81',
'CjkUnifiedIdeograph93c0': '\xe9\x8f\x80',
'CjkUnifiedIdeograph93c7': '\xe9\x8f\x87',
'CjkUnifiedIdeograph7c7c': '\xe7\xb1\xbc',
'CjkUnifiedIdeograph93c5': '\xe9\x8f\x85',
'CjkUnifiedIdeograph93c4': '\xe9\x8f\x84',
'HangulSyllableJyaes': '\xec\x9f\xb7',
'GreekCapitalLetterEtaWithOxia': '\xe1\xbf\x8b',
'CjkUnifiedIdeograph5d68': '\xe5\xb5\xa8',
'CombiningOverline': '\xcc\x85',
'CjkUnifiedIdeograph5d63': '\xe5\xb5\xa3',
'CjkUnifiedIdeograph5d62': '\xe5\xb5\xa2',
'CjkUnifiedIdeograph5d61': '\xe5\xb5\xa1',
'CjkUnifiedIdeograph5d60': '\xe5\xb5\xa0',
'CjkUnifiedIdeograph5d67': '\xe5\xb5\xa7',
'CjkUnifiedIdeograph5d66': '\xe5\xb5\xa6',
'CjkUnifiedIdeograph5d65': '\xe5\xb5\xa5',
'CjkUnifiedIdeograph5d64': '\xe5\xb5\xa4',
'UnicodeCharacter0x1af4': '\xe1\xab\xb4',
'UnicodeCharacter0x1af5': '\xe1\xab\xb5',
'UnicodeCharacter0x1af6': '\xe1\xab\xb6',
'UnicodeCharacter0x1af7': '\xe1\xab\xb7',
'UnicodeCharacter0x1af0': '\xe1\xab\xb0',
'BlackTelephone': '\xe2\x98\x8e',
'UnicodeCharacter0x1af2': '\xe1\xab\xb2',
'UnicodeCharacter0x1af3': '\xe1\xab\xb3',
'CjkUnifiedIdeograph90cd': '\xe9\x83\x8d',
'CjkUnifiedIdeograph90ce': '\xe9\x83\x8e',
'CjkUnifiedIdeograph90cf': '\xe9\x83\x8f',
'TagbanwaLetterGa': '\xe1\x9d\xa4',
'UnicodeCharacter0x1af8': '\xe1\xab\xb8',
'UnicodeCharacter0x1af9': '\xe1\xab\xb9',
'CjkUnifiedIdeograph90cb': '\xe9\x83\x8b',
'CjkUnifiedIdeograph6658': '\xe6\x99\x98',
'UnicodeCharacter0xde89': '\xed\xba\x89',
'PhagspaLetterHa': '\xea\xa1\x9c',
'ArmenianCapitalLetterIni': '\xd4\xbb',
'LeftwardsHarpoonWithBarbUpAboveLeftwardsHarpoonWithBarbDown': '\xe2\xa5\xa2',
'ContainsAsMember': '\xe2\x88\x8b',
'CjkUnifiedIdeograph5b86': '\xe5\xae\x86',
'ArabicLetterYehWithHamzaAboveInitialForm': '\xef\xba\x8b',
'CjkUnifiedIdeograph5b85': '\xe5\xae\x85',
'UnicodeCharacter0xf560': '\xef\x95\xa0',
'UnicodeCharacter0xde88': '\xed\xba\x88',
'UnicodeCharacter0x1afd': '\xe1\xab\xbd',
'UnicodeCharacter0x1afe': '\xe1\xab\xbe',
'UnicodeCharacter0x1aff': '\xe1\xab\xbf',
'CjkUnifiedIdeograph5b84': '\xe5\xae\x84',
'CjkUnifiedIdeograph90c8': '\xe9\x83\x88',
'ReverseSolidusWithHorizontalStroke': '\xe2\xa7\xb7',
'UnicodeCharacter0x1afb': '\xe1\xab\xbb',
'UnicodeCharacter0x1afc': '\xe1\xab\xbc',
'CjkUnifiedIdeograph90c4': '\xe9\x83\x84',
'CjkUnifiedIdeograph6651': '\xe6\x99\x91',
'CjkUnifiedIdeograph90c6': '\xe9\x83\x86',
'CjkUnifiedIdeograph90c7': '\xe9\x83\x87',
'CjkUnifiedIdeograph90c0': '\xe9\x83\x80',
'CjkUnifiedIdeograph90c1': '\xe9\x83\x81',
'CjkUnifiedIdeograph90c2': '\xe9\x83\x82',
'CjkUnifiedIdeograph90c3': '\xe9\x83\x83',
'SyriacSublinearColon': '\xdc\x84',
'CjkUnifiedIdeograph5d6c': '\xe5\xb5\xac',
'CjkUnifiedIdeograph5d6b': '\xe5\xb5\xab',
'CjkUnifiedIdeograph5d6a': '\xe5\xb5\xaa',
'UnicodeCharacter0xf563': '\xef\x95\xa3',
'CjkUnifiedIdeograph5d6f': '\xe5\xb5\xaf',
'CjkUnifiedIdeograph5d6e': '\xe5\xb5\xae',
'CjkUnifiedIdeograph5d6d': '\xe5\xb5\xad',
'UnicodeCharacter0xe69d': '\xee\x9a\x9d',
'LatinCapitalLetterSWithCedilla': '\xc5\x9e',
'CjkUnifiedIdeograph7a53': '\xe7\xa9\x93',
'CjkUnifiedIdeograph7cb7': '\xe7\xb2\xb7',
'UnicodeCharacter0xf562': '\xef\x95\xa2',
'CjkCompatibilityIdeographf99a': '\xef\xa6\x9a',
'EthiopicSyllableTee': '\xe1\x89\xb4',
'UnicodeCharacter0xe69e': '\xee\x9a\x9e',
'HangulSyllableNils': '\xeb\x8b\x94',
'CjkUnifiedIdeograph4389': '\xe4\x8e\x89',
'CjkUnifiedIdeograph4388': '\xe4\x8e\x88',
'CjkUnifiedIdeograph4381': '\xe4\x8e\x81',
'CjkUnifiedIdeograph4380': '\xe4\x8e\x80',
'CjkUnifiedIdeograph4383': '\xe4\x8e\x83',
'CjkUnifiedIdeograph4382': '\xe4\x8e\x82',
'CjkUnifiedIdeograph4385': '\xe4\x8e\x85',
'CjkUnifiedIdeograph4384': '\xe4\x8e\x84',
'CjkUnifiedIdeograph4387': '\xe4\x8e\x87',
'CjkUnifiedIdeograph4386': '\xe4\x8e\x86',
'ArabicLetterOeIsolatedForm': '\xef\xaf\x99',
'CjkUnifiedIdeograph45d9': '\xe4\x97\x99',
'CjkUnifiedIdeograph45d8': '\xe4\x97\x98',
'HangulSyllableRulh': '\xeb\xa3\xb7',
'CjkUnifiedIdeograph45d3': '\xe4\x97\x93',
'CjkUnifiedIdeograph45d2': '\xe4\x97\x92',
'CjkUnifiedIdeograph45d1': '\xe4\x97\x91',
'CjkUnifiedIdeograph45d0': '\xe4\x97\x90',
'CjkUnifiedIdeograph45d7': '\xe4\x97\x97',
'CjkUnifiedIdeograph45d6': '\xe4\x97\x96',
'CjkUnifiedIdeograph45d5': '\xe4\x97\x95',
'CjkUnifiedIdeograph45d4': '\xe4\x97\x94',
'UnicodeCharacter0xeab4': '\xee\xaa\xb4',
'BraillePatternDots12478': '\xe2\xa3\x8b',
'UnicodeCharacter0xeab6': '\xee\xaa\xb6',
'UnicodeCharacter0xeab7': '\xee\xaa\xb7',
'UnicodeCharacter0xeab0': '\xee\xaa\xb0',
'HangulJungseongAraeai': '\xe1\x86\xa1',
'UnicodeCharacter0xeab2': '\xee\xaa\xb2',
'SaurashtraVowelSignAa': '\xea\xa2\xb5',
'UnicodeCharacter0xeab8': '\xee\xaa\xb8',
'UnicodeCharacter0xeab9': '\xee\xaa\xb9',
'UnicodeCharacter0xe038': '\xee\x80\xb8',
'UnicodeCharacter0xe039': '\xee\x80\xb9',
'ModifierLetterMacron': '\xcb\x89',
'UnicodeCharacter0xf564': '\xef\x95\xa4',
'UnicodeCharacter0xe032': '\xee\x80\xb2',
'HangulJungseongAraeau': '\xe1\x86\xa0',
'UnicodeCharacter0xe030': '\xee\x80\xb0',
'UnicodeCharacter0xe031': '\xee\x80\xb1',
'UnicodeCharacter0xe036': '\xee\x80\xb6',
'UnicodeCharacter0xe037': '\xee\x80\xb7',
'UnicodeCharacter0xe034': '\xee\x80\xb4',
'UnicodeCharacter0xe035': '\xee\x80\xb5',
'UnicodeCharacter0xf449': '\xef\x91\x89',
'CyrillicLetterSmallCapitalEl': '\xe1\xb4\xab',
'CjkUnifiedIdeograph7d66': '\xe7\xb5\xa6',
'CjkUnifiedIdeograph45dc': '\xe4\x97\x9c',
'CjkUnifiedIdeograph45db': '\xe4\x97\x9b',
'CjkUnifiedIdeograph45da': '\xe4\x97\x9a',
'CjkUnifiedIdeograph45df': '\xe4\x97\x9f',
'CjkUnifiedIdeograph45de': '\xe4\x97\x9e',
'CjkUnifiedIdeograph45dd': '\xe4\x97\x9d',
'UnicodeCharacter0xf567': '\xef\x95\xa7',
'ArabicLigatureLamWithHahIsolatedForm': '\xef\xb1\x80',
'CjkUnifiedIdeograph438a': '\xe4\x8e\x8a',
'CjkUnifiedIdeograph438c': '\xe4\x8e\x8c',
'CjkUnifiedIdeograph438b': '\xe4\x8e\x8b',
'CjkUnifiedIdeograph438e': '\xe4\x8e\x8e',
'CjkUnifiedIdeograph438d': '\xe4\x8e\x8d',
'CjkUnifiedIdeograph438f': '\xe4\x8e\x8f',
'HangulSyllableDwelm': '\xeb\x92\x92',
'HangulSyllableDwelh': '\xeb\x92\x97',
'UnicodeCharacter0xe03b': '\xee\x80\xbb',
'HangulSyllableDwelg': '\xeb\x92\x91',
'RejangLetterMba': '\xea\xa5\x82',
'HangulSyllableDwelb': '\xeb\x92\x93',
'UnicodeCharacter0xe03d': '\xee\x80\xbd',
'HangulJongseongYesieungsios': '\xe1\x87\xb1',
'UnicodeCharacter0xeabd': '\xee\xaa\xbd',
'UnicodeCharacter0xeabe': '\xee\xaa\xbe',
'UnicodeCharacter0xeabf': '\xee\xaa\xbf',
'UnicodeCharacter0xeaba': '\xee\xaa\xba',
'SquarePiasutoru': '\xe3\x8c\xae',
'UnicodeCharacter0xeabc': '\xee\xaa\xbc',
'HangulSyllableDwelt': '\xeb\x92\x95',
'UnicodeCharacter0xaaec': '\xea\xab\xac',
'HangulSyllableDwelp': '\xeb\x92\x96',
'KangxiRadicalTannedLeather': '\xe2\xbe\xb1',
'CjkUnifiedIdeograph4d6c': '\xe4\xb5\xac',
'BraillePatternDots13678': '\xe2\xa3\xa5',
'CjkUnifiedIdeograph915f': '\xe9\x85\x9f',
'CjkUnifiedIdeograph915e': '\xe9\x85\x9e',
'CjkUnifiedIdeograph915d': '\xe9\x85\x9d',
'CjkUnifiedIdeograph915c': '\xe9\x85\x9c',
'CjkUnifiedIdeograph915b': '\xe9\x85\x9b',
'CjkUnifiedIdeograph915a': '\xe9\x85\x9a',
'UnicodeCharacter0xaaed': '\xea\xab\xad',
'UnicodeCharacter0xde81': '\xed\xba\x81',
'CombiningLeftArrowheadAbove': '\xe1\xb7\xbe',
'UnicodeCharacter0xf23e': '\xef\x88\xbe',
'UnicodeCharacter0xaaef': '\xea\xab\xaf',
'HangulSyllableJeugs': '\xec\xa6\x8b',
'YiSyllableHuo': '\xea\x89\xb7',
'FractionNumeratorOne': '\xe2\x85\x9f',
'CombiningDoubleRightwardsArrowBelow': '\xcd\xa2',
'LatinLetterSmallCapitalTurnedR': '\xe1\xb4\x9a',
'AlmostEqualToWithCircumflexAccent': '\xe2\xa9\xaf',
'UnicodeCharacter0xde80': '\xed\xba\x80',
'CjkUnifiedIdeograph5478': '\xe5\x91\xb8',
'CjkUnifiedIdeograph5479': '\xe5\x91\xb9',
'LatinLetterSmallCapitalTurnedE': '\xe2\xb1\xbb',
'UnicodeCharacter0xf23d': '\xef\x88\xbd',
'CjkUnifiedIdeograph5470': '\xe5\x91\xb0',
'CjkUnifiedIdeograph5471': '\xe5\x91\xb1',
'CjkUnifiedIdeograph5472': '\xe5\x91\xb2',
'CjkUnifiedIdeograph5473': '\xe5\x91\xb3',
'CjkUnifiedIdeograph5474': '\xe5\x91\xb4',
'CjkUnifiedIdeograph5475': '\xe5\x91\xb5',
'CjkUnifiedIdeograph5476': '\xe5\x91\xb6',
'CjkUnifiedIdeograph5477': '\xe5\x91\xb7',
'HangulSyllableNweogs': '\xeb\x88\xa3',
'UnicodeCharacter0xeb83': '\xee\xae\x83',
'HangulSyllableNweogg': '\xeb\x88\xa2',
'CjkUnifiedIdeograph9159': '\xe9\x85\x99',
'CjkUnifiedIdeograph9158': '\xe9\x85\x98',
'CjkUnifiedIdeograph9157': '\xe9\x85\x97',
'CjkUnifiedIdeograph9156': '\xe9\x85\x96',
'CjkUnifiedIdeograph9155': '\xe9\x85\x95',
'CjkUnifiedIdeograph9154': '\xe9\x85\x94',
'CjkUnifiedIdeograph9153': '\xe9\x85\x93',
'CjkUnifiedIdeograph9152': '\xe9\x85\x92',
'CjkUnifiedIdeograph9151': '\xe9\x85\x91',
'CjkUnifiedIdeograph9150': '\xe9\x85\x90',
'UnicodeCharacter0xa6e9': '\xea\x9b\xa9',
'ThreeRightwardsArrows': '\xe2\x87\xb6',
'CjkUnifiedIdeograph547a': '\xe5\x91\xba',
'CjkUnifiedIdeograph547b': '\xe5\x91\xbb',
'CjkUnifiedIdeograph547c': '\xe5\x91\xbc',
'CjkUnifiedIdeograph547d': '\xe5\x91\xbd',
'CjkUnifiedIdeograph547e': '\xe5\x91\xbe',
'CjkUnifiedIdeograph547f': '\xe5\x91\xbf',
'ArabicDammaIsolatedForm': '\xef\xb9\xb8',
'UnicodeCharacter0xf23f': '\xef\x88\xbf',
'UnicodeCharacter0xf443': '\xef\x91\x83',
'HangulSyllableMwed': '\xeb\xad\xb3',
'UnicodeCharacter0xeb85': '\xee\xae\x85',
'HangulSyllableDyop': '\xeb\x91\x8e',
'HangulSyllableDyos': '\xeb\x91\x87',
'CjkUnifiedIdeograph8fa9': '\xe8\xbe\xa9',
'HangulSyllableDyoh': '\xeb\x91\x8f',
'HangulSyllableDyok': '\xeb\x91\x8c',
'CjkUnifiedIdeograph8fa8': '\xe8\xbe\xa8',
'HangulSyllableDyom': '\xeb\x91\x84',
'HangulSyllableDyol': '\xeb\x90\xbc',
'HangulSyllableDyon': '\xeb\x90\xb8',
'CjkUnifiedIdeograph3c8f': '\xe3\xb2\x8f',
'HangulSyllableDyoc': '\xeb\x91\x8b',
'HangulSyllableDyob': '\xeb\x91\x85',
'UnicodeCharacter0xe6de': '\xee\x9b\x9e',
'HangulSyllableDyod': '\xeb\x90\xbb',
'LatinSmallLetterYWithAcute': '\xc3\xbd',
'WhiteMediumStar': '\xe2\xad\x90',
'SquarePondo': '\xe3\x8d\x80',
'GeorgianSmallLetterEn': '\xe2\xb4\x84',
'Infinity': '\xe2\x88\x9e',
'UnicodeCharacter0xeb84': '\xee\xae\x84',
'HangulSyllableMeuss': '\xeb\xaf\x94',
'LatinSmallLetterYWithTilde': '\xe1\xbb\xb9',
'!~~': '\xe2\x89\x89',
'HangulSyllableBbogg': '\xeb\xbd\x82',
'UnicodeCharacter0xf441': '\xef\x91\x81',
'CanadianSyllabicsNaskapiWaa': '\xe1\x90\x9b',
'CjkUnifiedIdeograph440f': '\xe4\x90\x8f',
'CjkUnifiedIdeograph440d': '\xe4\x90\x8d',
'CjkUnifiedIdeograph440e': '\xe4\x90\x8e',
'CjkUnifiedIdeograph440b': '\xe4\x90\x8b',
'CjkUnifiedIdeograph440c': '\xe4\x90\x8c',
'CjkUnifiedIdeograph440a': '\xe4\x90\x8a',
'UnicodeCharacter0xe6d9': '\xee\x9b\x99',
'UnicodeCharacter0xe6d8': '\xee\x9b\x98',
'UnicodeCharacter0xeb87': '\xee\xae\x87',
'UnicodeCharacter0xde87': '\xed\xba\x87',
'HangulSyllableReut': '\xeb\xa6\x8d',
'UnicodeCharacter0xe6d1': '\xee\x9b\x91',
'UnicodeCharacter0xe6d0': '\xee\x9b\x90',
'UnicodeCharacter0xe6d3': '\xee\x9b\x93',
'UnicodeCharacter0xe6d2': '\xee\x9b\x92',
'UnicodeCharacter0xe6d5': '\xee\x9b\x95',
'UnicodeCharacter0xe6d4': '\xee\x9b\x94',
'UnicodeCharacter0xe6d7': '\xee\x9b\x97',
'UnicodeCharacter0xe6d6': '\xee\x9b\x96',
'UnicodeCharacter0xf446': '\xef\x91\x86',
'UnicodeCharacter0xd884': '\xed\xa2\x84',
'CjkUnifiedIdeograph8fa5': '\xe8\xbe\xa5',
'ArabicLigatureSadWithAlefMaksuraFinalForm': '\xef\xb4\xa1',
'VaiSyllableNgon': '\xea\x96\xbc',
'CjkUnifiedIdeograph8fa4': '\xe8\xbe\xa4',
'RomanNumeralEleven': '\xe2\x85\xaa',
'VaiSyllableNjee': '\xea\x94\x9c',
'UnicodeCharacter0xeb86': '\xee\xae\x86',
'LimbuExclamationMark': '\xe1\xa5\x84',
'CjkUnifiedIdeograph4408': '\xe4\x90\x88',
'CjkUnifiedIdeograph4409': '\xe4\x90\x89',
'CjkUnifiedIdeograph4406': '\xe4\x90\x86',
'CjkUnifiedIdeograph4407': '\xe4\x90\x87',
'CjkUnifiedIdeograph4404': '\xe4\x90\x84',
'CjkUnifiedIdeograph4405': '\xe4\x90\x85',
'CjkUnifiedIdeograph4402': '\xe4\x90\x82',
'CjkUnifiedIdeograph4403': '\xe4\x90\x83',
'CjkUnifiedIdeograph4400': '\xe4\x90\x80',
'CjkUnifiedIdeograph4401': '\xe4\x90\x81',
'HangulSyllableRi': '\xeb\xa6\xac',
'LowerHalfCircle': '\xe2\x97\xa1',
'CjkUnifiedIdeograph3836': '\xe3\xa0\xb6',
'CjkUnifiedIdeograph3837': '\xe3\xa0\xb7',
'CjkUnifiedIdeograph55a9': '\xe5\x96\xa9',
'CjkUnifiedIdeograph55a8': '\xe5\x96\xa8',
'CjkUnifiedIdeograph3832': '\xe3\xa0\xb2',
'CjkUnifiedIdeograph3833': '\xe3\xa0\xb3',
'CjkUnifiedIdeograph3830': '\xe3\xa0\xb0',
'CjkUnifiedIdeograph3831': '\xe3\xa0\xb1',
'CjkUnifiedIdeograph55a3': '\xe5\x96\xa3',
'CjkUnifiedIdeograph55a2': '\xe5\x96\xa2',
'CjkUnifiedIdeograph55a1': '\xe5\x96\xa1',
'CjkUnifiedIdeograph55a0': '\xe5\x96\xa0',
'CjkUnifiedIdeograph55a7': '\xe5\x96\xa7',
'CjkUnifiedIdeograph55a6': '\xe5\x96\xa6',
'CjkUnifiedIdeograph55a5': '\xe5\x96\xa5',
'CjkUnifiedIdeograph3839': '\xe3\xa0\xb9',
'HangulSyllableCaels': '\xec\xb1\x90',
'EmQuad': '\xe2\x80\x81',
'UnicodeCharacter0xec24': '\xee\xb0\xa4',
'HangulSyllableCaelp': '\xec\xb1\x92',
'HangulSyllableCaelt': '\xec\xb1\x91',
'ArabicLigatureHahWithMeemWithYehFinalForm': '\xef\xb5\x9a',
'TibetanSubjoinedLetterDha': '\xe0\xbe\xa2',
'CombiningDoubleLowLine': '\xcc\xb3',
'HangulSyllableTyaes': '\xed\x84\xa7',
'UnicodeCharacter0xf445': '\xef\x91\x85',
'MuchLessthan': '\xe2\x89\xaa',
'PresentationFormForVerticalExclamationMark': '\xef\xb8\x95',
'UnicodeCharacter0xec08': '\xee\xb0\x88',
'UnicodeCharacter0xec09': '\xee\xb0\x89',
'cTheta': '\xcf\xb4',
'HangulSyllableHonh': '\xed\x98\xbe',
'HangulSyllableOenj': '\xec\x99\xbd',
'LatinSmallLetterPWithSquirrelTail': '\xea\x9d\x95',
'UnicodeCharacter0xec01': '\xee\xb0\x81',
'ArabicLetterYehBarreeWithExtendedArabicindicDigitThreeAbove': '\xdd\xbb',
'UnicodeCharacter0xe8e5': '\xee\xa3\xa5',
'UnicodeCharacter0xec04': '\xee\xb0\x84',
'CjkUnifiedIdeograph77e1': '\xe7\x9f\xa1',
'UnicodeCharacter0xec06': '\xee\xb0\x86',
'UnicodeCharacter0xec07': '\xee\xb0\x87',
'CircledLatinSmallLetterY': '\xe2\x93\xa8',
'CircledLatinSmallLetterX': '\xe2\x93\xa7',
'CircledLatinSmallLetterZ': '\xe2\x93\xa9',
'UnicodeCharacter0xef73': '\xee\xbd\xb3',
'CircledLatinSmallLetterQ': '\xe2\x93\xa0',
'CircledLatinSmallLetterP': '\xe2\x93\x9f',
'CircledLatinSmallLetterS': '\xe2\x93\xa2',
'CircledLatinSmallLetterR': '\xe2\x93\xa1',
'CircledLatinSmallLetterU': '\xe2\x93\xa4',
'CircledLatinSmallLetterT': '\xe2\x93\xa3',
'CircledLatinSmallLetterW': '\xe2\x93\xa6',
'CircledLatinSmallLetterV': '\xe2\x93\xa5',
'CjkUnifiedIdeograph383f': '\xe3\xa0\xbf',
'CircledLatinSmallLetterH': '\xe2\x93\x97',
'CjkUnifiedIdeograph383d': '\xe3\xa0\xbd',
'CjkUnifiedIdeograph383e': '\xe3\xa0\xbe',
'CjkUnifiedIdeograph383b': '\xe3\xa0\xbb',
'CircledLatinSmallLetterL': '\xe2\x93\x9b',
'CircledLatinSmallLetterO': '\xe2\x93\x9e',
'CircledLatinSmallLetterN': '\xe2\x93\x9d',
'CjkUnifiedIdeograph55ac': '\xe5\x96\xac',
'CjkUnifiedIdeograph55ab': '\xe5\x96\xab',
'CircledLatinSmallLetterC': '\xe2\x93\x92',
'CircledLatinSmallLetterB': '\xe2\x93\x91',
'CircledLatinSmallLetterE': '\xe2\x93\x94',
'CjkUnifiedIdeograph55af': '\xe5\x96\xaf',
'CjkUnifiedIdeograph55ae': '\xe5\x96\xae',
'CjkUnifiedIdeograph55ad': '\xe5\x96\xad',
'HangulSyllableHyabs': '\xed\x96\xa2',
'TibetanSymbolNorBuGsumKhyil': '\xe0\xbf\x8b',
'CyrillicSmallLetterEnWithMiddleHook': '\xd4\xa3',
'HangulSyllableGgyuh': '\xeb\x81\x83',
'UnicodeCharacter0xec0a': '\xee\xb0\x8a',
'CjkUnifiedIdeograph722b': '\xe7\x88\xab',
'UnicodeCharacter0xec0c': '\xee\xb0\x8c',
'UnicodeCharacter0xd844': '\xed\xa1\x84',
'UnicodeCharacter0xec0e': '\xee\xb0\x8e',
'HangulSyllableTagg': '\xed\x83\x82',
'CanadianSyllabicsNaskapiSpwa': '\xe1\x94\x8c',
'HangulSyllableTags': '\xed\x83\x83',
'HangulSyllableReuj': '\xeb\xa6\x8a',
'HangulSyllableNyolp': '\xeb\x87\xb6',
'HangulSyllableNyols': '\xeb\x87\xb4',
'HangulSyllableNyolt': '\xeb\x87\xb5',
'KhmerSymbolLekAttakPram': '\xe1\x9f\xb5',
'LimbuSmallLetterKa': '\xe1\xa4\xb0',
'BalineseSignUluCandra': '\xe1\xac\x81',
'OghamLetterEabhadh': '\xe1\x9a\x95',
'ArabicLigatureTehWithAlefMaksuraIsolatedForm': '\xef\xb0\x8f',
'HangulSyllableNyolg': '\xeb\x87\xb1',
'UnicodeCharacter0xdbe': '\xe0\xb6\xbe',
'UnicodeCharacter0xdbf': '\xe0\xb6\xbf',
'HangulSyllableNyolm': '\xeb\x87\xb2',
'CyrillicSmallLetterOmegaWithTitlo': '\xd1\xbd',
'CjkCompatibilityIdeographf9fe': '\xef\xa7\xbe',
'CjkUnifiedIdeograph671d': '\xe6\x9c\x9d',
'CjkUnifiedIdeograph671e': '\xe6\x9c\x9e',
'CjkUnifiedIdeograph671f': '\xe6\x9c\x9f',
'CjkUnifiedIdeograph671a': '\xe6\x9c\x9a',
'CjkUnifiedIdeograph671b': '\xe6\x9c\x9b',
'CjkUnifiedIdeograph671c': '\xe6\x9c\x9c',
'UnicodeCharacter0xf1fa': '\xef\x87\xba',
'ClockwiseOpenCircleArrow': '\xe2\x86\xbb',
'YiSyllableSsyrx': '\xea\x8d\x84',
'HangulSyllablePyus': '\xed\x93\xbb',
'UnicodeCharacter0xa6fd': '\xea\x9b\xbd',
'HangulSyllableBwelh': '\xeb\xb7\x87',
'CjkUnifiedIdeograph51d7': '\xe5\x87\x97',
'UnicodeCharacter0xdb2': '\xe0\xb6\xb2',
'BlackLargeCircle': '\xe2\xac\xa4',
'UnicodeCharacter0xe058': '\xee\x81\x98',
'RightwardsDoubleDashArrow': '\xe2\xa4\x8d',
'YiSyllableDax': '\xea\x84\x88',
'UnicodeCharacter0xefa6': '\xee\xbe\xa6',
'HangulSyllableJeols': '\xec\xa0\x8c',
'HangulSyllableJeolt': '\xec\xa0\x8d',
'HangulSyllableSseom': '\xec\x8d\xb8',
'YiSyllableDap': '\xea\x84\x8a',
'HangulSyllableJjyaelp': '\xec\xa8\xbe',
'YiSyllableDat': '\xea\x84\x87',
'UnicodeCharacter0xefa4': '\xee\xbe\xa4',
'HangulSyllableJjyaelt': '\xec\xa8\xbd',
'HangulSyllableJeolb': '\xec\xa0\x8b',
'HangulSyllableJjyaelh': '\xec\xa8\xbf',
'CjkUnifiedIdeograph6718': '\xe6\x9c\x98',
'CjkUnifiedIdeograph6719': '\xe6\x9c\x99',
'HangulSyllableJjyaelm': '\xec\xa8\xba',
'HangulSyllableJeolg': '\xec\xa0\x89',
'GurmukhiSignBindi': '\xe0\xa8\x82',
'CjkUnifiedIdeograph6715': '\xe6\x9c\x95',
'CjkUnifiedIdeograph6716': '\xe6\x9c\x96',
'CjkUnifiedIdeograph6717': '\xe6\x9c\x97',
'CjkUnifiedIdeograph6710': '\xe6\x9c\x90',
'CjkUnifiedIdeograph6711': '\xe6\x9c\x91',
'CjkUnifiedIdeograph6712': '\xe6\x9c\x92',
'CjkUnifiedIdeograph6713': '\xe6\x9c\x93',
'UnicodeCharacter0xefa0': '\xee\xbe\xa0',
'UnicodeCharacter0xee71': '\xee\xb9\xb1',
'UnicodeCharacter0xefa1': '\xee\xbe\xa1',
'LaoDigitOne': '\xe0\xbb\x91',
'HangulSyllableNyet': '\xeb\x85\xb5',
'UnicodeCharacter0xd8f8': '\xed\xa3\xb8',
'HangulSyllableYeoj': '\xec\x98\x82',
'HangulSyllableJeuk': '\xec\xa6\xa0',
'HangulSyllableYeoh': '\xec\x98\x87',
'HangulSyllableDdolb': '\xeb\x98\x9b',
'UnicodeCharacter0xefa8': '\xee\xbe\xa8',
'CjkCompatibilityIdeographf9fd': '\xef\xa7\xbd',
'UnicodeCharacter0xefa9': '\xee\xbe\xa9',
'HangulSyllablePwenj': '\xed\x92\xb5',
'CjkUnifiedIdeograph855c': '\xe8\x95\x9c',
'EthiopicSyllableQoa': '\xe1\x89\x87',
'HangulSyllableGgyeobs': '\xea\xbc\x86',
'GeorgianSmallLetterRae': '\xe2\xb4\x90',
'CjkUnifiedIdeograph66c8': '\xe6\x9b\x88',
'LepchaDigitFour': '\xe1\xb1\x84',
'HangulSyllableHolb': '\xed\x99\x83',
'ArabicLigatureKafWithLamMedialForm': '\xef\xb3\xab',
'HangulSyllableHolm': '\xed\x99\x82',
'HangulSyllableJyinj': '\xec\xa6\xa9',
'HangulSyllableHolh': '\xed\x99\x87',
'UnicodeCharacter0xe876': '\xee\xa1\xb6',
'HangulSyllableHolt': '\xed\x99\x85',
'HangulSyllableKwess': '\xed\x80\xac',
'HangulSyllableHolp': '\xed\x99\x86',
'UnicodeCharacter0xe652': '\xee\x99\x92',
'CanadianSyllabicsCarrierLo': '\xe1\x98\xa3',
'ArabicLetterKirghizYuIsolatedForm': '\xef\xaf\xa2',
'CjkUnifiedIdeograph5b87': '\xe5\xae\x87',
'CanadianSyllabicsCarrierLi': '\xe1\x98\xa6',
'UnicodeCharacter0xe870': '\xee\xa1\xb0',
'HangulSyllableNibs': '\xeb\x8b\x9a',
'HangulSyllableJeug': '\xec\xa6\x89',
'HangulSyllableJjeuss': '\xec\xaf\xa8',
'HangulSyllableYeot': '\xec\x98\x85',
'CjkUnifiedIdeograph7f5f': '\xe7\xbd\x9f',
'UnicodeCharacter0xe7ef': '\xee\x9f\xaf',
'CjkUnifiedIdeograph7f5d': '\xe7\xbd\x9d',
'CjkUnifiedIdeograph7f5e': '\xe7\xbd\x9e',
'CjkUnifiedIdeograph7f5b': '\xe7\xbd\x9b',
'HexagramForApproach': '\xe4\xb7\x92',
'UnicodeCharacter0xe7ea': '\xee\x9f\xaa',
'CjkUnifiedIdeograph7f5a': '\xe7\xbd\x9a',
'ArabicLetterKafFinalForm': '\xef\xbb\x9a',
'HangulSyllableDulb': '\xeb\x91\x9b',
'HangulSyllableYeop': '\xec\x98\x86',
'HangulSyllableDulg': '\xeb\x91\x99',
'CjkUnifiedIdeograph73bb': '\xe7\x8e\xbb',
'CjkUnifiedIdeograph73bc': '\xe7\x8e\xbc',
'CjkUnifiedIdeograph73ba': '\xe7\x8e\xba',
'CjkUnifiedIdeograph73bf': '\xe7\x8e\xbf',
'HangulSyllableDulm': '\xeb\x91\x9a',
'CjkUnifiedIdeograph73bd': '\xe7\x8e\xbd',
'CjkUnifiedIdeograph6b7b': '\xe6\xad\xbb',
'HangulSyllableSsyilg': '\xec\x94\x95',
'HangulSyllableKeg': '\xec\xbc\x81',
'HangulSyllableSsyilb': '\xec\x94\x97',
'BalineseLetterRa': '\xe1\xac\xad',
'HangulSyllableKec': '\xec\xbc\x97',
'HangulSyllableKel': '\xec\xbc\x88',
'HangulSyllableKem': '\xec\xbc\x90',
'HangulSyllableSsyilm': '\xec\x94\x96',
'VectorOrCrossProduct': '\xe2\xa8\xaf',
'HangulSyllableKeh': '\xec\xbc\x9b',
'CjkUnifiedIdeograph51db': '\xe5\x87\x9b',
'HangulSyllableKej': '\xec\xbc\x96',
'HangulSyllableSsyilh': '\xec\x94\x9b',
'Overline': '\xe2\x80\xbe',
'HangulSyllableKeu': '\xed\x81\xac',
'LimbuLetterSa': '\xe1\xa4\x9b',
'HangulSyllableSsyilt': '\xec\x94\x99',
'HangulSyllableSsyils': '\xec\x94\x98',
'CjkUnifiedIdeograph8de0': '\xe8\xb7\xa0',
'UnicodeCharacter0xf62e': '\xef\x98\xae',
'CjkUnifiedIdeograph51de': '\xe5\x87\x9e',
'CjkUnifiedIdeograph4e84': '\xe4\xba\x84',
'CjkUnifiedIdeograph73b8': '\xe7\x8e\xb8',
'CjkUnifiedIdeograph73b9': '\xe7\x8e\xb9',
'HangulSyllableNyem': '\xeb\x85\xac',
'CjkUnifiedIdeograph73b2': '\xe7\x8e\xb2',
'CjkUnifiedIdeograph73b3': '\xe7\x8e\xb3',
'CjkUnifiedIdeograph73b0': '\xe7\x8e\xb0',
'CjkUnifiedIdeograph73b1': '\xe7\x8e\xb1',
'CjkUnifiedIdeograph73b6': '\xe7\x8e\xb6',
'CjkUnifiedIdeograph73b7': '\xe7\x8e\xb7',
'CjkUnifiedIdeograph73b4': '\xe7\x8e\xb4',
'CjkUnifiedIdeograph73b5': '\xe7\x8e\xb5',
'CjkUnifiedIdeograph7f56': '\xe7\xbd\x96',
'KhmerSymbolBuonKoet': '\xe1\xa7\xa4',
'CjkUnifiedIdeograph7f54': '\xe7\xbd\x94',
'CjkUnifiedIdeograph7f55': '\xe7\xbd\x95',
'CjkUnifiedIdeograph7f52': '\xe7\xbd\x92',
'MyanmarConsonantSignMonMedialLa': '\xe1\x81\xa0',
'CjkUnifiedIdeograph7f50': '\xe7\xbd\x90',
'CjkUnifiedIdeograph7f51': '\xe7\xbd\x91',
'CjkUnifiedIdeograph7f58': '\xe7\xbd\x98',
'ArmenianSmallLetterCa': '\xd5\xae',
'YiSyllableHna': '\xea\x85\xb2',
'BraillePatternDots123567': '\xe2\xa1\xb7',
'YiSyllableHne': '\xea\x85\xba',
'UnicodeCharacter0xdf62': '\xed\xbd\xa2',
'YiSyllableHni': '\xea\x85\xaa',
'BraillePatternDots123568': '\xe2\xa2\xb7',
'ArabicLigatureNoonWithHahInitialForm': '\xef\xb3\x93',
'HexagramForSmallTaming': '\xe4\xb7\x88',
'GreekSmallLetterUpsilonWithPsiliAndPerispomeni': '\xe1\xbd\x96',
'CjkUnifiedIdeograph5f6a': '\xe5\xbd\xaa',
'CjkUnifiedIdeograph5f6c': '\xe5\xbd\xac',
'CjkUnifiedIdeograph5f6b': '\xe5\xbd\xab',
'CjkUnifiedIdeograph5f6e': '\xe5\xbd\xae',
'CjkUnifiedIdeograph5f6d': '\xe5\xbd\xad',
'CjkUnifiedIdeograph5f6f': '\xe5\xbd\xaf',
'UnicodeCharacter0xdf63': '\xed\xbd\xa3',
'HangulSyllablePoels': '\xed\x91\x8c',
'HangulSyllableRyal': '\xeb\x9e\xbc',
'HangulSyllableNyolh': '\xeb\x87\xb7',
'WhiteScissors': '\xe2\x9c\x84',
'CyrillicSmallLetterCche': '\xea\x9a\x87',
'HangulSyllableRwaelm': '\xeb\xa2\x9e',
'CjkUnifiedIdeograph5b81': '\xe5\xae\x81',
'CjkUnifiedIdeograph5f69': '\xe5\xbd\xa9',
'CjkUnifiedIdeograph5f68': '\xe5\xbd\xa8',
'HangulSyllableBbulb': '\xeb\xbf\x97',
'CjkUnifiedIdeograph5f61': '\xe5\xbd\xa1',
'CjkUnifiedIdeograph5f60': '\xe5\xbd\xa0',
'DevanagariGraveAccent': '\xe0\xa5\x93',
'CjkUnifiedIdeograph5f62': '\xe5\xbd\xa2',
'CjkUnifiedIdeograph5f65': '\xe5\xbd\xa5',
'CjkUnifiedIdeograph5f64': '\xe5\xbd\xa4',
'CjkUnifiedIdeograph5f67': '\xe5\xbd\xa7',
'LatinSmallLetterTurnedHWithFishhookAndTail': '\xca\xaf',
'CjkUnifiedIdeograph5d95': '\xe5\xb6\x95',
'CjkRadicalHead': '\xe2\xbb\xa1',
'UnicodeCharacter0xdf65': '\xed\xbd\xa5',
'UnicodeCharacter0xece9': '\xee\xb3\xa9',
'HangulSyllableYaegs': '\xec\x96\x9b',
'CjkCompatibilityIdeographf9fb': '\xef\xa7\xbb',
'OghamLetterBeith': '\xe1\x9a\x81',
'CjkUnifiedIdeograph5d91': '\xe5\xb6\x91',
'HangulSyllableBboebs': '\xeb\xbe\xa6',
'IntegralWithTimesSign': '\xe2\xa8\x98',
'CjkUnifiedIdeograph7522': '\xe7\x94\xa2',
'HangulSyllableYaegg': '\xec\x96\x9a',
'UnicodeCharacter0xea66': '\xee\xa9\xa6',
'UnicodeCharacter0xdf66': '\xed\xbd\xa6',
'ChamLetterKha': '\xea\xa8\x87',
'HangulSyllablePyup': '\xed\x94\x82',
'CjkUnifiedIdeograph5cdb': '\xe5\xb3\x9b',
'CjkUnifiedIdeograph9a3f': '\xe9\xa8\xbf',
'UnicodeCharacter0xa6f9': '\xea\x9b\xb9',
'LatinSmallLetterTurnedAe': '\xe1\xb4\x82',
'HangulSyllableGgyis': '\xeb\x81\xb3',
'LargeDownTack': '\xe2\x9f\x99',
'CjkUnifiedIdeograph9a3c': '\xe9\xa8\xbc',
'UnicodeCharacter0xea67': '\xee\xa9\xa7',
'HangulSyllableNyek': '\xeb\x85\xb4',
'CjkUnifiedIdeograph9a3b': '\xe9\xa8\xbb',
'HangulSyllableDyoss': '\xeb\x91\x88',
'UnicodeCharacter0xecc2': '\xee\xb3\x82',
'UnicodeCharacter0xa6f4': '\xea\x9b\xb4',
'VaiSyllableJu': '\xea\x96\xb1',
'IdeographicTelegraphSymbolForHourOne': '\xe3\x8d\x99',
'UnicodeCharacter0xfadd': '\xef\xab\x9d',
'UnicodeCharacter0xa6f1': '\xea\x9b\xb1',
'UnicodeCharacter0xea68': '\xee\xa9\xa8',
'HebrewAccentTipeha': '\xd6\x96',
'UnicodeCharacter0xa6f0': '\xea\x9b\xb0',
'ArabicLigatureFehWithKhahIsolatedForm': '\xef\xb0\xaf',
'HangulSyllableGgyul': '\xeb\x80\xb0',
'SinhalaLetterAlpapraanaPayanna': '\xe0\xb6\xb4',
'ChamLetterFinalNg': '\xea\xa9\x82',
'HangulSyllableRwaelb': '\xeb\xa2\x9f',
'UnicodeCharacter0xdd35': '\xed\xb4\xb5',
'UnicodeCharacter0xea69': '\xee\xa9\xa9',
'HangulSyllableGgyig': '\xeb\x81\xa1',
'ArabicLigatureDadWithRehIsolatedForm': '\xef\xb4\x90',
'HangulSyllableBeud': '\xeb\xb8\x93',
'HangulSyllableBeug': '\xeb\xb8\x8d',
'UnicodeCharacter0xdd36': '\xed\xb4\xb6',
'WhiteUpPointingIndex': '\xe2\x98\x9d',
'HangulSyllableBeub': '\xeb\xb8\x9d',
'HangulSyllableBeum': '\xeb\xb8\x9c',
'HangulSyllableBeul': '\xeb\xb8\x94',
'HangulSyllableBeun': '\xeb\xb8\x90',
'VaiSyllableJa': '\xea\x95\xa7',
'HangulSyllableBeuk': '\xeb\xb8\xa4',
'HangulSyllableBeuj': '\xeb\xb8\xa2',
'HangulSyllableBeut': '\xeb\xb8\xa5',
'TibetanSubjoinedLetterA': '\xe0\xbe\xb8',
'CjkUnifiedIdeograph7dec': '\xe7\xb7\xac',
'HangulSyllableBeup': '\xeb\xb8\xa6',
'HangulSyllableBeus': '\xeb\xb8\x9f',
'UnicodeCharacter0xe55e': '\xee\x95\x9e',
'UnicodeCharacter0xdd32': '\xed\xb4\xb2',
'LatinSmallLetterMiddlewelshLl': '\xe1\xbb\xbb',
'UnicodeCharacter0xe55d': '\xee\x95\x9d',
'UnicodeCharacter0xec21': '\xee\xb0\xa1',
'HangulSyllableJjej': '\xec\xa9\xbe',
'TibetanSubjoinedLetterZa': '\xe0\xbe\xaf',
'VaiSyllableJe': '\xea\x98\x80',
'HangulSyllableJjek': '\xec\xaa\x80',
'UnicodeCharacter0xf7ee': '\xef\x9f\xae',
'HangulSyllableNyeg': '\xeb\x85\x9d',
'BalineseLetterAkara': '\xe1\xac\x85',
'CjkUnifiedIdeograph9410': '\xe9\x90\x90',
'CjkUnifiedIdeograph6b78': '\xe6\xad\xb8',
'UnicodeCharacter0xf727': '\xef\x9c\xa7',
'HangulSyllableJjeo': '\xec\xa9\x8c',
'CjkUnifiedIdeograph9d18': '\xe9\xb4\x98',
'NkoDigitSeven': '\xdf\x87',
'HangulSyllableJjel': '\xec\xa9\xb0',
'CjkUnifiedIdeograph9d12': '\xe9\xb4\x92',
'CjkUnifiedIdeograph9d13': '\xe9\xb4\x93',
'CjkUnifiedIdeograph9d10': '\xe9\xb4\x90',
'CjkUnifiedIdeograph9d11': '\xe9\xb4\x91',
'CjkUnifiedIdeograph9d16': '\xe9\xb4\x96',
'CjkUnifiedIdeograph9d17': '\xe9\xb4\x97',
'CjkUnifiedIdeograph9d14': '\xe9\xb4\x94',
'CjkUnifiedIdeograph9d15': '\xe9\xb4\x95',
'CjkUnifiedIdeograph8cab': '\xe8\xb2\xab',
'CjkUnifiedIdeograph8cac': '\xe8\xb2\xac',
'ArabicLetterTcheheh': '\xda\x87',
'CjkUnifiedIdeograph8caa': '\xe8\xb2\xaa',
'CjkUnifiedIdeograph8caf': '\xe8\xb2\xaf',
'HalfwidthKatakanaLetterSe': '\xef\xbd\xbe',
'CjkUnifiedIdeograph8cad': '\xe8\xb2\xad',
'CjkUnifiedIdeograph8cae': '\xe8\xb2\xae',
'TibetanDigitHalfThree': '\xe0\xbc\xac',
'CanadianSyllabicsNgii': '\xe1\x96\x90',
'UnicodeCharacter0xda07': '\xed\xa8\x87',
'UnicodeCharacter0xe87f': '\xee\xa1\xbf',
'ArabicLigatureSheenWithAlefMaksuraFinalForm': '\xef\xb4\x99',
'UnicodeCharacter0xda06': '\xed\xa8\x86',
'HangulSyllableDdwilt': '\xeb\x9b\xbd',
'HangulSyllableSsoem': '\xec\x90\xbc',
'CjkUnifiedIdeograph8ca2': '\xe8\xb2\xa2',
'CjkUnifiedIdeograph8ca3': '\xe8\xb2\xa3',
'CjkUnifiedIdeograph8ca0': '\xe8\xb2\xa0',
'CjkUnifiedIdeograph8ca1': '\xe8\xb2\xa1',
'CjkUnifiedIdeograph8ca6': '\xe8\xb2\xa6',
'CjkUnifiedIdeograph8ca7': '\xe8\xb2\xa7',
'CjkUnifiedIdeograph8ca4': '\xe8\xb2\xa4',
'CjkUnifiedIdeograph8ca5': '\xe8\xb2\xa5',
'UnicodeCharacter0xe87b': '\xee\xa1\xbb',
'CjkUnifiedIdeograph8ca8': '\xe8\xb2\xa8',
'CjkUnifiedIdeograph8ca9': '\xe8\xb2\xa9',
'CjkUnifiedIdeograph99a9': '\xe9\xa6\xa9',
'NkoLetterDa': '\xdf\x98',
'CjkUnifiedIdeograph9d1b': '\xe9\xb4\x9b',
'CjkUnifiedIdeograph9d1c': '\xe9\xb4\x9c',
'CjkUnifiedIdeograph9d1a': '\xe9\xb4\x9a',
'CjkUnifiedIdeograph9d1f': '\xe9\xb4\x9f',
'CjkUnifiedIdeograph9d1d': '\xe9\xb4\x9d',
'CjkUnifiedIdeograph9d1e': '\xe9\xb4\x9e',
'CjkUnifiedIdeograph8172': '\xe8\x85\xb2',
'CjkUnifiedIdeograph8173': '\xe8\x85\xb3',
'CjkUnifiedIdeograph8170': '\xe8\x85\xb0',
'CjkUnifiedIdeograph8171': '\xe8\x85\xb1',
'CjkUnifiedIdeograph8176': '\xe8\x85\xb6',
'CjkUnifiedIdeograph8177': '\xe8\x85\xb7',
'CjkUnifiedIdeograph8174': '\xe8\x85\xb4',
'CjkUnifiedIdeograph8175': '\xe8\x85\xb5',
'HangulSyllableSsoed': '\xec\x90\xb3',
'CjkUnifiedIdeograph8178': '\xe8\x85\xb8',
'CjkUnifiedIdeograph8179': '\xe8\x85\xb9',
'HangulSyllableRyos': '\xeb\xa3\x9f',
'HangulSyllableJjes': '\xec\xa9\xbb',
'HangulSyllableSsoeg': '\xec\x90\xad',
'DoesNotPrecedeOrEqual': '\xe2\x8b\xa0',
'HangulSyllableJjep': '\xec\xaa\x82',
'TeluguLetterVocalicLl': '\xe0\xb1\xa1',
'CjkUnifiedIdeograph8bec': '\xe8\xaf\xac',
'CjkUnifiedIdeograph8374': '\xe8\x8d\xb4',
'CjkUnifiedIdeograph8bea': '\xe8\xaf\xaa',
'UnicodeCharacter0xea45': '\xee\xa9\x85',
'CjkUnifiedIdeograph8bef': '\xe8\xaf\xaf',
'CjkUnifiedIdeograph8bee': '\xe8\xaf\xae',
'CjkUnifiedIdeograph8375': '\xe8\x8d\xb5',
'Models': '\xe2\x8a\xa7',
'TeluguFractionDigitZeroForOddPowersOfFour': '\xe0\xb1\xb8',
'UnicodeCharacter0xf627': '\xef\x98\xa7',
'HangulSyllableJjet': '\xec\xaa\x81',
'CjkUnifiedIdeograph8370': '\xe8\x8d\xb0',
'HangulSyllableRyot': '\xeb\xa3\xa5',
'CjkUnifiedIdeograph8371': '\xe8\x8d\xb1',
'UnicodeCharacter0xf2b3': '\xef\x8a\xb3',
'UnicodeCharacter0xece5': '\xee\xb3\xa5',
'UnicodeCharacter0xf076': '\xef\x81\xb6',
'DoubleNestedGreaterthan': '\xe2\xaa\xa2',
'CjkUnifiedIdeograph817b': '\xe8\x85\xbb',
'CjkUnifiedIdeograph75e5': '\xe7\x97\xa5',
'QuadrantUpperLeftAndUpperRightAndLowerLeft': '\xe2\x96\x9b',
'CjkUnifiedIdeograph817f': '\xe8\x85\xbf',
'CjkUnifiedIdeograph817d': '\xe8\x85\xbd',
'CjkUnifiedIdeograph75e4': '\xe7\x97\xa4',
'VaiSymbolFaa': '\xea\x98\x98',
'HangulSyllableJyenh': '\xec\xa1\x9a',
'CjkUnifiedIdeograph75e7': '\xe7\x97\xa7',
'YiSyllableZzie': '\xea\x8b\x96',
'LatinCapitalLetterOWithCircumflexAndHookAbove': '\xe1\xbb\x94',
'HangulSyllableBbult': '\xeb\xbf\x99',
'HangulSyllableGgweolg': '\xea\xbf\x9d',
'CjkUnifiedIdeograph6fa5': '\xe6\xbe\xa5',
'HangulSyllableSswaep': '\xec\x90\xaa',
'EthiopicSyllableGlottalU': '\xe1\x8a\xa1',
'HangulSyllableSswaet': '\xec\x90\xa9',
'CjkUnifiedIdeograph75e0': '\xe7\x97\xa0',
'CjkUnifiedIdeograph8be3': '\xe8\xaf\xa3',
'EthiopicSyllableGlottalO': '\xe1\x8a\xa6',
'CjkUnifiedIdeograph8be1': '\xe8\xaf\xa1',
'BoxDrawingsVerticalDoubleAndLeftSingle': '\xe2\x95\xa2',
'CjkUnifiedIdeograph8be7': '\xe8\xaf\xa7',
'CjkUnifiedIdeograph75e3': '\xe7\x97\xa3',
'CjkUnifiedIdeograph8be5': '\xe8\xaf\xa5',
'EthiopicSyllableGlottalI': '\xe1\x8a\xa2',
'HangulSyllableSswaeb': '\xec\x90\xa1',
'HangulSyllableSswaec': '\xec\x90\xa7',
'CjkUnifiedIdeograph8be9': '\xe8\xaf\xa9',
'EthiopicSyllableGlottalE': '\xe1\x8a\xa5',
'HangulSyllableSswaeg': '\xec\x90\x91',
'HangulSyllableSswaed': '\xec\x90\x97',
'EthiopicSyllableGlottalA': '\xe1\x8a\xa0',
'CjkUnifiedIdeograph98b7': '\xe9\xa2\xb7',
'UnicodeCharacter0xe551': '\xee\x95\x91',
'UnicodeCharacter0xdd3f': '\xed\xb4\xbf',
'UnicodeCharacter0xda3c': '\xed\xa8\xbc',
'UnicodeCharacter0xe550': '\xee\x95\x90',
'HangzhouNumeralTwo': '\xe3\x80\xa2',
'HangulSyllableHyuc': '\xed\x9d\x8b',
'UnicodeCharacter0xe553': '\xee\x95\x93',
'HangulSyllableJjyeop': '\xec\xaa\x9e',
'UnicodeCharacter0xd831': '\xed\xa0\xb1',
'UnicodeCharacter0xe552': '\xee\x95\x92',
'UnicodeCharacter0xdd3c': '\xed\xb4\xbc',
'LatinSmallLetterAWithDotBelow': '\xe1\xba\xa1',
'UnicodeCharacter0xdd3b': '\xed\xb4\xbb',
'UnicodeCharacter0xd833': '\xed\xa0\xb3',
'UnicodeCharacter0xe554': '\xee\x95\x94',
'UnicodeCharacter0xab41': '\xea\xad\x81',
'UnicodeCharacter0xab40': '\xea\xad\x80',
'UnicodeCharacter0xab43': '\xea\xad\x83',
'UnicodeCharacter0xab42': '\xea\xad\x82',
'UnicodeCharacter0xab45': '\xea\xad\x85',
'NkoCombiningShortHighTone': '\xdf\xab',
'UnicodeCharacter0xab47': '\xea\xad\x87',
'UnicodeCharacter0xab46': '\xea\xad\x86',
'UnicodeCharacter0xab49': '\xea\xad\x89',
'UnicodeCharacter0xab48': '\xea\xad\x88',
'UnicodeCharacter0xe556': '\xee\x95\x96',
'UnicodeCharacter0xe559': '\xee\x95\x99',
'HangulSyllableHwibs': '\xed\x9c\xaa',
'CrossedSwords': '\xe2\x9a\x94',
'UnicodeCharacter0xe4b6': '\xee\x92\xb6',
'UnicodeCharacter0xe558': '\xee\x95\x98',
'CjkUnifiedIdeograph9a39': '\xe9\xa8\xb9',
'CjkUnifiedIdeograph9a38': '\xe9\xa8\xb8',
'HangulSyllableMyass': '\xeb\xa8\x94',
'UnicodeCharacter0xf042': '\xef\x81\x82',
'CjkUnifiedIdeograph9a34': '\xe9\xa8\xb4',
'HangulSyllableGibs': '\xea\xb9\x82',
'CjkUnifiedIdeograph5da4': '\xe5\xb6\xa4',
'HangulSyllableBbulp': '\xeb\xbf\x9a',
'SaurashtraLetterVocalicLl': '\xea\xa2\x8b',
'UnicodeCharacter0xa48f': '\xea\x92\x8f',
'HangulSyllableSels': '\xec\x85\x84',
'ZNotationSchemaPiping': '\xe2\xa8\xa0',
'CjkUnifiedIdeograph9312': '\xe9\x8c\x92',
'UnicodeCharacter0xda09': '\xed\xa8\x89',
'UnicodeCharacter0xab4a': '\xea\xad\x8a',
'UnicodeCharacter0xf043': '\xef\x81\x83',
'UnicodeCharacter0xab4c': '\xea\xad\x8c',
'UnicodeCharacter0xab4b': '\xea\xad\x8b',
'UnicodeCharacter0xab4e': '\xea\xad\x8e',
'CjkUnifiedIdeograph9a33': '\xe9\xa8\xb3',
'CircleWithRightHalfBlack': '\xe2\x97\x91',
'MediumRightpointingAngleBracketOrnament': '\xe2\x9d\xad',
'HangulSyllableSelh': '\xec\x85\x87',
'HangulSyllableSelm': '\xec\x85\x82',
'EightPointedBlackStar': '\xe2\x9c\xb4',
'ArabicLetterNoonWithRing': '\xda\xbc',
'UnicodeCharacter0xd7f5': '\xed\x9f\xb5',
'CjkUnifiedIdeograph644c': '\xe6\x91\x8c',
'IdeographicTelegraphSymbolForDayTwentyone': '\xe3\x8f\xb4',
'UnicodeCharacter0xd7f4': '\xed\x9f\xb4',
'HangulSyllableBboelb': '\xeb\xbe\x9f',
'CombiningCyrillicLetterBigYus': '\xe2\xb7\xbe',
'HangulSyllableBboelg': '\xeb\xbe\x9d',
'UnicodeCharacter0xd7f7': '\xed\x9f\xb7',
'CjkUnifiedIdeograph9087': '\xe9\x82\x87',
'UnicodeCharacter0xf040': '\xef\x81\x80',
'CjkUnifiedIdeograph80e5': '\xe8\x83\xa5',
'CjkUnifiedIdeograph80e4': '\xe8\x83\xa4',
'CjkUnifiedIdeograph80e7': '\xe8\x83\xa7',
'CjkUnifiedIdeograph80e6': '\xe8\x83\xa6',
'CjkUnifiedIdeograph80e1': '\xe8\x83\xa1',
'CjkUnifiedIdeograph80e0': '\xe8\x83\xa0',
'CjkUnifiedIdeograph80e3': '\xe8\x83\xa3',
'CjkUnifiedIdeograph80e2': '\xe8\x83\xa2',
'HangulSyllableBboels': '\xeb\xbe\xa0',
'LatinSmallLetterReversedCWithDot': '\xea\x9c\xbf',
'HangulSyllableBboelp': '\xeb\xbe\xa2',
'CjkUnifiedIdeograph80e9': '\xe8\x83\xa9',
'LatinSmallLetterHWithHook': '\xc9\xa6',
'UnicodeCharacter0xd998': '\xed\xa6\x98',
'HangulSyllableBboelt': '\xeb\xbe\xa1',
'HangulSyllablePwibs': '\xed\x93\x9e',
'UnicodeCharacter0xd7f3': '\xed\x9f\xb3',
'UnicodeCharacter0xf041': '\xef\x81\x81',
'LatinSmallLetterZWithRetroflexHook': '\xca\x90',
'CjkUnifiedIdeograph80ee': '\xe8\x83\xae',
'WhiteVerticalEllipse': '\xe2\xac\xaf',
'CjkUnifiedIdeograph8e74': '\xe8\xb9\xb4',
'CjkUnifiedIdeograph80ef': '\xe8\x83\xaf',
'CjkUnifiedIdeograph80ea': '\xe8\x83\xaa',
'CjkUnifiedIdeograph8e73': '\xe8\xb9\xb3',
'CjkUnifiedIdeograph80ec': '\xe8\x83\xac',
'CjkUnifiedIdeograph80eb': '\xe8\x83\xab',
'UnicodeCharacter0xd7f9': '\xed\x9f\xb9',
'CjkUnifiedIdeograph9089': '\xe9\x82\x89',
'CjkUnifiedIdeograph8e78': '\xe8\xb9\xb8',
'CjkUnifiedIdeograph8e79': '\xe8\xb9\xb9',
'CyrillicCapitalLetterYn': '\xea\x99\x9e',
'BalineseLetterMa': '\xe1\xac\xab',
'HanunooSignPamudpod': '\xe1\x9c\xb4',
'HangulSyllableJwalg': '\xec\xa2\x95',
'HangulSyllableJjyeoss': '\xec\xaa\x98',
'ArabicLetterGhainFinalForm': '\xef\xbb\x8e',
'CjkUnifiedIdeograph7d6b': '\xe7\xb5\xab',
'CjkUnifiedIdeograph6195': '\xe6\x86\x95',
'YiSyllableNji': '\xea\x90\xb3',
'PresentationFormForVerticalLeftSquareBracket': '\xef\xb9\x87',
'OriyaVowelSignU': '\xe0\xad\x81',
'ArabicLetterLamWithDotAbove': '\xda\xb6',
'OriyaVowelSignI': '\xe0\xac\xbf',
'CjkUnifiedIdeograph6faa': '\xe6\xbe\xaa',
'UnicodeCharacter0xfbd': '\xe0\xbe\xbd',
'OriyaVowelSignO': '\xe0\xad\x8b',
'UnicodeCharacter0xda08': '\xed\xa8\x88',
'OriyaVowelSignE': '\xe0\xad\x87',
'PhagspaLetterKha': '\xea\xa1\x81',
'CjkUnifiedIdeograph4075': '\xe4\x81\xb5',
'HangulSyllableByong': '\xeb\xb5\xb9',
'CjkUnifiedIdeograph37ef': '\xe3\x9f\xaf',
'CjkUnifiedIdeograph37ee': '\xe3\x9f\xae',
'CjkUnifiedIdeograph37ed': '\xe3\x9f\xad',
'CjkUnifiedIdeograph37ec': '\xe3\x9f\xac',
'CjkUnifiedIdeograph37eb': '\xe3\x9f\xab',
'CjkUnifiedIdeograph37ea': '\xe3\x9f\xaa',
'CjkUnifiedIdeograph75ea': '\xe7\x97\xaa',
'UnicodeCharacter0x1259': '\xe1\x89\x99',
'HangulSyllableGgwaenh': '\xea\xbd\xaa',
'HangulSyllableGwi': '\xea\xb7\x80',
'CanadianSyllabicsCwoo': '\xe1\x92\x9a',
'CjkUnifiedIdeograph75ec': '\xe7\x97\xac',
'HangulSyllableDdyabs': '\xeb\x95\xba',
'ChamDigitThree': '\xea\xa9\x93',
'CjkUnifiedIdeograph75eb': '\xe7\x97\xab',
'CjkUnifiedIdeograph54e8': '\xe5\x93\xa8',
'HangulSyllableGwe': '\xea\xb6\xa4',
'CjkUnifiedIdeograph4888': '\xe4\xa2\x88',
'CjkUnifiedIdeograph4889': '\xe4\xa2\x89',
'CjkUnifiedIdeograph4882': '\xe4\xa2\x82',
'CjkUnifiedIdeograph4883': '\xe4\xa2\x83',
'CjkUnifiedIdeograph4880': '\xe4\xa2\x80',
'CjkUnifiedIdeograph4881': '\xe4\xa2\x81',
'CjkUnifiedIdeograph4886': '\xe4\xa2\x86',
'CjkUnifiedIdeograph4887': '\xe4\xa2\x87',
'CjkUnifiedIdeograph4884': '\xe4\xa2\x84',
'CjkUnifiedIdeograph4885': '\xe4\xa2\x85',
'GreekSmallLetterStigma': '\xcf\x9b',
'CanadianSyllabicsMoosecreeSk': '\xe1\x94\x89',
'UnicodeCharacter0xf26b': '\xef\x89\xab',
'UnicodeCharacter0xf26c': '\xef\x89\xac',
'UnicodeCharacter0xf26a': '\xef\x89\xaa',
'UnicodeCharacter0xf26f': '\xef\x89\xaf',
'UnicodeCharacter0xf26d': '\xef\x89\xad',
'HangulSyllableJjyuss': '\xec\xaf\x8c',
'CjkUnifiedIdeograph3dea': '\xe3\xb7\xaa',
'CjkUnifiedIdeograph695b': '\xe6\xa5\x9b',
'MongolianLetterSibeRaa': '\xe1\xa1\xb0',
'CjkUnifiedIdeograph695c': '\xe6\xa5\x9c',
'omicron': '\xce\xbf',
'CjkUnifiedIdeograph37e7': '\xe3\x9f\xa7',
'CjkUnifiedIdeograph37e6': '\xe3\x9f\xa6',
'CjkUnifiedIdeograph37e5': '\xe3\x9f\xa5',
'CjkUnifiedIdeograph37e4': '\xe3\x9f\xa4',
'CjkUnifiedIdeograph37e3': '\xe3\x9f\xa3',
'CjkUnifiedIdeograph37e2': '\xe3\x9f\xa2',
'CjkUnifiedIdeograph37e1': '\xe3\x9f\xa1',
'CjkUnifiedIdeograph37e0': '\xe3\x9f\xa0',
'CanadianSyllabicsNng': '\xe1\x96\x96',
'CjkUnifiedIdeograph695f': '\xe6\xa5\x9f',
'CjkUnifiedIdeograph6f40': '\xe6\xbd\x80',
'CjkUnifiedIdeograph37e9': '\xe3\x9f\xa9',
'CjkUnifiedIdeograph37e8': '\xe3\x9f\xa8',
'UnicodeCharacter0xf268': '\xef\x89\xa8',
'HangulSyllableDeogg': '\xeb\x8d\x96',
'CjkUnifiedIdeograph695d': '\xe6\xa5\x9d',
'UnicodeCharacter0xe3bb': '\xee\x8e\xbb',
'GujaratiLetterNna': '\xe0\xaa\xa3',
'UnicodeCharacter0xf262': '\xef\x89\xa2',
'HangulSyllableBbyolg': '\xeb\xbe\xb9',
'UnicodeCharacter0xf260': '\xef\x89\xa0',
'CjkUnifiedIdeograph695e': '\xe6\xa5\x9e',
'UnicodeCharacter0xf266': '\xef\x89\xa6',
'UnicodeCharacter0xf267': '\xef\x89\xa7',
'UnicodeCharacter0xf264': '\xef\x89\xa4',
'UnicodeCharacter0xf265': '\xef\x89\xa5',
'CjkUnifiedIdeograph488b': '\xe4\xa2\x8b',
'CjkUnifiedIdeograph488c': '\xe4\xa2\x8c',
'CjkUnifiedIdeograph488a': '\xe4\xa2\x8a',
'CjkUnifiedIdeograph488f': '\xe4\xa2\x8f',
'CjkUnifiedIdeograph488d': '\xe4\xa2\x8d',
'CjkUnifiedIdeograph488e': '\xe4\xa2\x8e',
'HangulSyllablePaed': '\xed\x8c\xaf',
'HangulSyllableSsaeg': '\xec\x8c\x95',
'UnicodeCharacter0xdaeb': '\xed\xab\xab',
'UnicodeCharacter0xf073': '\xef\x81\xb3',
'CopticSymbolKhiRo': '\xe2\xb3\xa9',
'CjkUnifiedIdeograph9c50': '\xe9\xb1\x90',
'CjkUnifiedIdeograph9c53': '\xe9\xb1\x93',
'CjkUnifiedIdeograph9c52': '\xe9\xb1\x92',
'CjkUnifiedIdeograph9c55': '\xe9\xb1\x95',
'CjkUnifiedIdeograph9c54': '\xe9\xb1\x94',
'CjkUnifiedIdeograph9c57': '\xe9\xb1\x97',
'CjkUnifiedIdeograph9c56': '\xe9\xb1\x96',
'CjkUnifiedIdeograph9c59': '\xe9\xb1\x99',
'CjkUnifiedIdeograph9c58': '\xe9\xb1\x98',
'CjkUnifiedIdeograph4738': '\xe4\x9c\xb8',
'HangulSyllablePaec': '\xed\x8c\xbf',
'HangulSyllableGgat': '\xea\xb9\xa5',
'CyrillicSmallLetterAbkhasianHa': '\xd2\xa9',
'CjkUnifiedIdeograph98b9': '\xe9\xa2\xb9',
'ArabicLigatureGhainWithMeemWithYehFinalForm': '\xef\xb5\xba',
'UnicodeCharacter0xe8fe': '\xee\xa3\xbe',
'UnicodeCharacter0xe8fd': '\xee\xa3\xbd',
'UnicodeCharacter0xe8ff': '\xee\xa3\xbf',
'UnicodeCharacter0xe8fa': '\xee\xa3\xba',
'UnicodeCharacter0xe8fc': '\xee\xa3\xbc',
'UnicodeCharacter0xe8fb': '\xee\xa3\xbb',
'YiSyllableNyox': '\xea\x91\x96',
'CjkUnifiedIdeograph4268': '\xe4\x89\xa8',
'CjkUnifiedIdeograph4269': '\xe4\x89\xa9',
'CjkUnifiedIdeograph4266': '\xe4\x89\xa6',
'CjkUnifiedIdeograph4267': '\xe4\x89\xa7',
'CjkUnifiedIdeograph4264': '\xe4\x89\xa4',
'CjkUnifiedIdeograph4265': '\xe4\x89\xa5',
'CjkUnifiedIdeograph4262': '\xe4\x89\xa2',
'CjkUnifiedIdeograph4263': '\xe4\x89\xa3',
'CjkUnifiedIdeograph4260': '\xe4\x89\xa0',
'CjkUnifiedIdeograph4261': '\xe4\x89\xa1',
'CjkUnifiedIdeograph9c5a': '\xe9\xb1\x9a',
'CjkUnifiedIdeograph9a37': '\xe9\xa8\xb7',
'CjkUnifiedIdeograph9c5c': '\xe9\xb1\x9c',
'CjkUnifiedIdeograph9c5b': '\xe9\xb1\x9b',
'CjkUnifiedIdeograph9c5e': '\xe9\xb1\x9e',
'CjkUnifiedIdeograph9c5d': '\xe9\xb1\x9d',
'CjkUnifiedIdeograph9c5f': '\xe9\xb1\x9f',
'HangulSyllableDwaegg': '\xeb\x8f\xbe',
'BalineseLetterBaKembang': '\xe1\xac\xaa',
'HangulSyllableDwaegs': '\xeb\x8f\xbf',
'LatinSmallLetterAWithCircumflexAndHookAbove': '\xe1\xba\xa9',
'CjkUnifiedIdeograph426f': '\xe4\x89\xaf',
'CjkUnifiedIdeograph426d': '\xe4\x89\xad',
'CjkUnifiedIdeograph426e': '\xe4\x89\xae',
'CjkUnifiedIdeograph426b': '\xe4\x89\xab',
'CjkUnifiedIdeograph426c': '\xe4\x89\xac',
'CjkUnifiedIdeograph426a': '\xe4\x89\xaa',
'UnicodeCharacter0xe8f9': '\xee\xa3\xb9',
'MuchGreaterthan': '\xe2\x89\xab',
'LimbuLetterNa': '\xe1\xa4\x8f',
'UnicodeCharacter0xe8f5': '\xee\xa3\xb5',
'CjkUnifiedIdeograph9777': '\xe9\x9d\xb7',
'UnicodeCharacter0xe8f7': '\xee\xa3\xb7',
'UnicodeCharacter0xe8f6': '\xee\xa3\xb6',
'UnicodeCharacter0xe8f1': '\xee\xa3\xb1',
'UnicodeCharacter0xe8f0': '\xee\xa3\xb0',
'UnicodeCharacter0xe8f3': '\xee\xa3\xb3',
'UnicodeCharacter0xe8f2': '\xee\xa3\xb2',
'HangulSyllableGgeonh': '\xea\xbb\x82',
'CjkUnifiedIdeograph91c8': '\xe9\x87\x88',
'ArabicLigatureBehWithAlefMaksuraFinalForm': '\xef\xb1\xae',
'TibetanSubjoinedLetterGa': '\xe0\xbe\x92',
'HangulSyllableBbyuss': '\xec\x81\x90',
'LeftDoubleWigglyFence': '\xe2\xa7\x9a',
'HangulSyllableRyags': '\xeb\x9e\xb7',
'SaurashtraLetterTa': '\xea\xa2\xa1',
'HangulSyllableEunh': '\xec\x9d\x82',
'Chiron': '\xe2\x9a\xb7',
'HangulSyllableNwaet': '\xeb\x87\x89',
'CjkUnifiedIdeograph9996': '\xe9\xa6\x96',
'LepchaConsonantSignKang': '\xe1\xb0\xb5',
'HangulSyllableNwaen': '\xeb\x86\xb4',
'GreaterthanOverEqualTo': '\xe2\x89\xa7',
'HangulSyllableNwael': '\xeb\x86\xb8',
'HangulSyllableNwaek': '\xeb\x87\x88',
'HangulSyllableNwaej': '\xeb\x87\x86',
'HangulSyllablePaen': '\xed\x8c\xac',
'HangulSyllableNwaeh': '\xeb\x87\x8b',
'HangulSyllableNwaeg': '\xeb\x86\xb1',
'UnicodeCharacter0xd8fb': '\xed\xa3\xbb',
'HangulSyllableNwaed': '\xeb\x86\xb7',
'HangulSyllableNwaec': '\xeb\x87\x87',
'CjkUnifiedIdeograph96c5': '\xe9\x9b\x85',
'UnicodeCharacter0xf7d5': '\xef\x9f\x95',
'ModifierLetterSmallReversedOpenE': '\xe1\xb6\x9f',
'ArabicLigatureKafWithMeemWithMeemFinalForm': '\xef\xb6\xbb',
'GurmukhiLetterDa': '\xe0\xa8\xa6',
'HangulSyllableByonj': '\xeb\xb5\xa9',
'HangulSyllableRyeobs': '\xeb\xa0\xb6',
'CombiningConjoiningMacron': '\xef\xb8\xa6',
'LimbuLetterSha': '\xe1\xa4\x99',
'HangulSyllableJe': '\xec\xa0\x9c',
'UnicodeCharacter0xf4f4': '\xef\x93\xb4',
'UnicodeCharacter0xe234': '\xee\x88\xb4',
'UnicodeCharacter0xd9e3': '\xed\xa7\xa3',
'HangulSyllableBbagg': '\xeb\xb9\xa2',
'UnicodeCharacter0xd9db': '\xed\xa7\x9b',
'HangulSyllableEung': '\xec\x9d\x91',
'UnicodeCharacter0xd9d1': '\xed\xa7\x91',
'BalineseVowelSignTalingRepaTedung': '\xe1\xad\x81',
'GeorgianCapitalLetterVin': '\xe1\x82\xa5',
'ModifierLetterLowCircumflexAccent': '\xea\x9e\x88',
'CjkUnifiedIdeograph91c4': '\xe9\x87\x84',
'UnicodeCharacter0xd9dd': '\xed\xa7\x9d',
'UnicodeCharacter0xf0dd': '\xef\x83\x9d',
'CjkUnifiedIdeograph6138': '\xe6\x84\xb8',
'CjkUnifiedIdeograph6139': '\xe6\x84\xb9',
'CjkUnifiedIdeograph9245': '\xe9\x89\x85',
'CjkUnifiedIdeograph9349': '\xe9\x8d\x89',
'CjkUnifiedIdeograph6134': '\xe6\x84\xb4',
'CjkUnifiedIdeograph6135': '\xe6\x84\xb5',
'CjkUnifiedIdeograph6136': '\xe6\x84\xb6',
'CjkUnifiedIdeograph6137': '\xe6\x84\xb7',
'MultiplicationSignInTriangle': '\xe2\xa8\xbb',
'CjkUnifiedIdeograph5d03': '\xe5\xb4\x83',
'CjkUnifiedIdeograph6132': '\xe6\x84\xb2',
'CjkUnifiedIdeograph6133': '\xe6\x84\xb3',
'CjkUnifiedIdeograph5d02': '\xe5\xb4\x82',
'CyrillicSmallLetterZeWithDiaeresis': '\xd3\x9f',
'HangulJongseongSiospieup': '\xe1\x87\xaa',
'TripleSolidusBinaryRelation': '\xe2\xab\xbb',
'CopticSmallLetterOldCopticOou': '\xe2\xb2\xbf',
'YiSyllableFu': '\xea\x83\x9a',
'BraillePatternDots15678': '\xe2\xa3\xb1',
'HangulSyllablePaej': '\xed\x8c\xbe',
'OcrBowTie': '\xe2\x91\x85',
'CjkUnifiedIdeograph7ca3': '\xe7\xb2\xa3',
'CjkUnifiedIdeograph5d09': '\xe5\xb4\x89',
'CyrillicCapitalLetterKa': '\xd0\x9a',
'CjkUnifiedIdeograph7ca0': '\xe7\xb2\xa0',
'CjkUnifiedIdeograph7ca7': '\xe7\xb2\xa7',
'CjkUnifiedIdeograph7ca6': '\xe7\xb2\xa6',
'CjkUnifiedIdeograph7ca5': '\xe7\xb2\xa5',
'CjkUnifiedIdeograph5d08': '\xe5\xb4\x88',
'LatinCapitalLetterCWithCaron': '\xc4\x8c',
'CjkUnifiedIdeograph7ca8': '\xe7\xb2\xa8',
'CjkUnifiedIdeograph9a8b': '\xe9\xaa\x8b',
'UnicodeCharacter0xf148': '\xef\x85\x88',
'CjkUnifiedIdeograph9a8c': '\xe9\xaa\x8c',
'CjkUnifiedIdeograph569e': '\xe5\x9a\x9e',
'HangulSyllableJa': '\xec\x9e\x90',
'CjkUnifiedIdeograph9a8e': '\xe9\xaa\x8e',
'UnicodeCharacter0xd9d2': '\xed\xa7\x92',
'HangulSyllableSyos': '\xec\x88\x8f',
'CjkUnifiedIdeograph9a8f': '\xe9\xaa\x8f',
'CjkUnifiedIdeograph613d': '\xe6\x84\xbd',
'CjkUnifiedIdeograph613e': '\xe6\x84\xbe',
'CjkUnifiedIdeograph613f': '\xe6\x84\xbf',
'BengaliLetterKhandaTa': '\xe0\xa7\x8e',
'CjkUnifiedIdeograph934b': '\xe9\x8d\x8b',
'CjkUnifiedIdeograph613a': '\xe6\x84\xba',
'CjkUnifiedIdeograph613b': '\xe6\x84\xbb',
'CjkUnifiedIdeograph613c': '\xe6\x84\xbc',
'CjkUnifiedIdeograph7cac': '\xe7\xb2\xac',
'CjkUnifiedIdeograph7cab': '\xe7\xb2\xab',
'CjkUnifiedIdeograph7caa': '\xe7\xb2\xaa',
'BraillePatternDots12378': '\xe2\xa3\x87',
'CjkUnifiedIdeograph7caf': '\xe7\xb2\xaf',
'CjkUnifiedIdeograph7cae': '\xe7\xb2\xae',
'CjkUnifiedIdeograph7cad': '\xe7\xb2\xad',
'HangulSyllableSsaes': '\xec\x8c\xa7',
'UnicodeCharacter0xd9d5': '\xed\xa7\x95',
'HangulSyllableDdyinh': '\xeb\x9d\x8a',
'TeluguLetterRra': '\xe0\xb0\xb1',
'HangulSyllableCyalh': '\xec\xb1\xaf',
'HangulSyllableRweogg': '\xeb\xa4\x86',
'EthiopicSyllableSzaa': '\xe1\x88\xa3',
'HalfwidthFormsLightVertical': '\xef\xbf\xa8',
'EthiopicSyllableXee': '\xe1\x8a\x84',
'VaiSyllableJee': '\xea\x94\x9b',
'MyanmarLetterRumaiPalaungFa': '\xe1\x82\x8e',
'CjkUnifiedIdeograph9241': '\xe9\x89\x81',
'ArabicLigatureWasallamIsolatedForm': '\xef\xb7\xb8',
'CircledWhiteBullet': '\xe2\xa6\xbe',
'CjkUnifiedIdeograph777e': '\xe7\x9d\xbe',
'CjkUnifiedIdeograph777d': '\xe7\x9d\xbd',
'CjkUnifiedIdeograph777f': '\xe7\x9d\xbf',
'CjkUnifiedIdeograph777a': '\xe7\x9d\xba',
'CjkUnifiedIdeograph777c': '\xe7\x9d\xbc',
'CjkUnifiedIdeograph777b': '\xe7\x9d\xbb',
'HangulSyllableGeugg': '\xea\xb7\xba',
'CjkUnifiedIdeograph364e': '\xe3\x99\x8e',
'CjkUnifiedIdeograph364d': '\xe3\x99\x8d',
'CjkUnifiedIdeograph364f': '\xe3\x99\x8f',
'CjkUnifiedIdeograph364a': '\xe3\x99\x8a',
'CjkUnifiedIdeograph364c': '\xe3\x99\x8c',
'CjkUnifiedIdeograph364b': '\xe3\x99\x8b',
'CjkUnifiedIdeograph9eb6': '\xe9\xba\xb6',
'CjkUnifiedIdeograph9eb7': '\xe9\xba\xb7',
'CjkUnifiedIdeograph9eb4': '\xe9\xba\xb4',
'CjkUnifiedIdeograph9eb5': '\xe9\xba\xb5',
'LatinSmallLetterVWithCurl': '\xe2\xb1\xb4',
'CjkUnifiedIdeograph6950': '\xe6\xa5\x90',
'CjkUnifiedIdeograph9eb0': '\xe9\xba\xb0',
'CjkUnifiedIdeograph9eb1': '\xe9\xba\xb1',
'HangulSyllableGgeunh': '\xeb\x81\x8a',
'UnicodeCharacter0x984': '\xe0\xa6\x84',
'CjkUnifiedIdeograph60fe': '\xe6\x83\xbe',
'UnicodeCharacter0x980': '\xe0\xa6\x80',
'UnicodeCharacter0xde1b': '\xed\xb8\x9b',
'UnicodeCharacter0xea1a': '\xee\xa8\x9a',
'GeorgianLetterHie': '\xe1\x83\xb2',
'UnicodeCharacter0xea1c': '\xee\xa8\x9c',
'UnicodeCharacter0xea1b': '\xee\xa8\x9b',
'UnicodeCharacter0xea1e': '\xee\xa8\x9e',
'UnicodeCharacter0xea1d': '\xee\xa8\x9d',
'CjkUnifiedIdeograph6957': '\xe6\xa5\x97',
'CjkUnifiedIdeograph7775': '\xe7\x9d\xb5',
'CjkUnifiedIdeograph7774': '\xe7\x9d\xb4',
'CjkUnifiedIdeograph7777': '\xe7\x9d\xb7',
'CjkUnifiedIdeograph7776': '\xe7\x9d\xb6',
'CjkUnifiedIdeograph7771': '\xe7\x9d\xb1',
'CjkUnifiedIdeograph7770': '\xe7\x9d\xb0',
'CjkUnifiedIdeograph7773': '\xe7\x9d\xb3',
'CjkUnifiedIdeograph7772': '\xe7\x9d\xb2',
'TibetanLetterLa': '\xe0\xbd\xa3',
'CjkUnifiedIdeograph6955': '\xe6\xa5\x95',
'CjkUnifiedIdeograph7779': '\xe7\x9d\xb9',
'CjkUnifiedIdeograph7778': '\xe7\x9d\xb8',
'CjkCompatibilityIdeographf974': '\xef\xa5\xb4',
'HangulSyllableRong': '\xeb\xa1\xb1',
'WhiteChessPawn': '\xe2\x99\x99',
'RightArcGreaterthanBracket': '\xe2\xa6\x94',
'BugineseLetterNgka': '\xe1\xa8\x83',
'HangulSyllableSyoj': '\xec\x88\x92',
'UnicodeCharacter0xea19': '\xee\xa8\x99',
'ChamLetterNhja': '\xea\xa8\x92',
'UnicodeCharacter0x98d': '\xe0\xa6\x8d',
'UnicodeCharacter0x98e': '\xe0\xa6\x8e',
'HanunooLetterRa': '\xe1\x9c\xad',
'HangulSyllableGeugs': '\xea\xb7\xbb',
'UnicodeCharacter0xea11': '\xee\xa8\x91',
'HangulSyllableSyol': '\xec\x88\x84',
'UnicodeCharacter0xea13': '\xee\xa8\x93',
'HangulSyllablePya': '\xed\x8d\x84',
'UnicodeCharacter0xea15': '\xee\xa8\x95',
'UnicodeCharacter0xea14': '\xee\xa8\x94',
'UnicodeCharacter0xea17': '\xee\xa8\x97',
'UnicodeCharacter0xea16': '\xee\xa8\x96',
'CjkUnifiedIdeograph3645': '\xe3\x99\x85',
'CjkUnifiedIdeograph3644': '\xe3\x99\x84',
'CjkUnifiedIdeograph3647': '\xe3\x99\x87',
'CjkUnifiedIdeograph3646': '\xe3\x99\x86',
'CjkUnifiedIdeograph3641': '\xe3\x99\x81',
'CjkUnifiedIdeograph3640': '\xe3\x99\x80',
'CjkUnifiedIdeograph3643': '\xe3\x99\x83',
'CjkUnifiedIdeograph3642': '\xe3\x99\x82',
'CjkUnifiedIdeograph9ebf': '\xe9\xba\xbf',
'CjkUnifiedIdeograph9ebd': '\xe9\xba\xbd',
'CjkUnifiedIdeograph9ebe': '\xe9\xba\xbe',
'CjkUnifiedIdeograph3649': '\xe3\x99\x89',
'CjkUnifiedIdeograph3648': '\xe3\x99\x88',
'gamma': '\xce\xb3',
'CjkUnifiedIdeograph9eba': '\xe9\xba\xba',
'YiSyllableXit': '\xea\x91\x9d',
'UnicodeCharacter0xe1c0': '\xee\x87\x80',
'ModifierLetterSmallTurnedAe': '\xe1\xb5\x86',
'YiSyllableXip': '\xea\x91\xa0',
'UnicodeCharacter0xfdee': '\xef\xb7\xae',
'YiSyllableXix': '\xea\x91\x9e',
'UnicodeCharacter0xfded': '\xef\xb7\xad',
'YiSyllableXie': '\xea\x91\xa3',
'ArabicSignSindhiAmpersand': '\xdb\xbd',
'CjkUnifiedIdeograph60e9': '\xe6\x83\xa9',
'KatakanaLetterNu': '\xe3\x83\x8c',
'CjkUnifiedIdeograph54fc': '\xe5\x93\xbc',
'CjkUnifiedIdeograph54fb': '\xe5\x93\xbb',
'CjkUnifiedIdeograph54fa': '\xe5\x93\xba',
'CjkUnifiedIdeograph60e4': '\xe6\x83\xa4',
'CjkUnifiedIdeograph60e3': '\xe6\x83\xa3',
'CyrillicCapitalLetterIWithGrave': '\xd0\x8d',
'CjkUnifiedIdeograph54fe': '\xe5\x93\xbe',
'CjkUnifiedIdeograph54fd': '\xe5\x93\xbd',
'CherokeeLetterKa': '\xe1\x8e\xa7',
'UnicodeCharacter0xa9ec': '\xea\xa7\xac',
'UnicodeCharacter0xa9eb': '\xea\xa7\xab',
'UnicodeCharacter0xa9ee': '\xea\xa7\xae',
'UnicodeCharacter0xa9ed': '\xea\xa7\xad',
'UnicodeCharacter0xa9ef': '\xea\xa7\xaf',
'HairSpace': '\xe2\x80\x8a',
'CjkUnifiedIdeograph669a': '\xe6\x9a\x9a',
'CjkUnifiedIdeograph669c': '\xe6\x9a\x9c',
'ParenthesizedNumberEleven': '\xe2\x91\xbe',
'CjkUnifiedIdeograph669e': '\xe6\x9a\x9e',
'CjkUnifiedIdeograph669d': '\xe6\x9a\x9d',
'UnicodeCharacter0xda98': '\xed\xaa\x98',
'CjkUnifiedIdeograph669f': '\xe6\x9a\x9f',
'UnicodeCharacter0xda96': '\xed\xaa\x96',
'UnicodeCharacter0xda97': '\xed\xaa\x97',
'UnicodeCharacter0xda94': '\xed\xaa\x94',
'UnicodeCharacter0xda95': '\xed\xaa\x95',
'UnicodeCharacter0xda92': '\xed\xaa\x92',
'HangulSyllableJi': '\xec\xa7\x80',
'UnicodeCharacter0xda90': '\xed\xaa\x90',
'HangulSyllableGenj': '\xea\xb2\x91',
'LeftwardsArrowWithDoubleVerticalStroke': '\xe2\x87\xba',
'HangulSyllableUh': '\xec\x9b\x8b',
'CjkUnifiedIdeograph54f9': '\xe5\x93\xb9',
'CjkUnifiedIdeograph54f8': '\xe5\x93\xb8',
'CjkUnifiedIdeograph54f3': '\xe5\x93\xb3',
'CjkUnifiedIdeograph54f2': '\xe5\x93\xb2',
'CjkUnifiedIdeograph54f1': '\xe5\x93\xb1',
'CjkUnifiedIdeograph54f0': '\xe5\x93\xb0',
'CjkUnifiedIdeograph54f7': '\xe5\x93\xb7',
'CjkUnifiedIdeograph54f6': '\xe5\x93\xb6',
'CjkUnifiedIdeograph54f5': '\xe5\x93\xb5',
'CjkUnifiedIdeograph54f4': '\xe5\x93\xb4',
'MalayalamDigitOne': '\xe0\xb5\xa7',
'HangulSyllableUk': '\xec\x9b\x88',
'delta': '\xce\xb4',
'GreekCapitalLetterOmegaWithDasiaAndOxia': '\xe1\xbd\xad',
'HangulSyllableUm': '\xec\x9b\x80',
'CjkUnifiedIdeograph6691': '\xe6\x9a\x91',
'CjkUnifiedIdeograph6690': '\xe6\x9a\x90',
'CjkUnifiedIdeograph6693': '\xe6\x9a\x93',
'CjkUnifiedIdeograph6692': '\xe6\x9a\x92',
'CombiningDoubleGraveAccent': '\xcc\x8f',
'CjkUnifiedIdeograph6694': '\xe6\x9a\x94',
'CjkUnifiedIdeograph6697': '\xe6\x9a\x97',
'CjkUnifiedIdeograph6696': '\xe6\x9a\x96',
'CjkUnifiedIdeograph6699': '\xe6\x9a\x99',
'CjkUnifiedIdeograph6698': '\xe6\x9a\x98',
'UnicodeCharacter0xda9d': '\xed\xaa\x9d',
'CjkUnifiedIdeograph7a5b': '\xe7\xa9\x9b',
'UnicodeCharacter0xda9b': '\xed\xaa\x9b',
'UnicodeCharacter0xda9c': '\xed\xaa\x9c',
'UnicodeCharacter0xda9a': '\xed\xaa\x9a',
'UnicodeCharacter0xa9e1': '\xea\xa7\xa1',
'CombiningCyrillicLetterIotifiedA': '\xe2\xb7\xbc',
'UnicodeCharacter0xa9e3': '\xea\xa7\xa3',
'UnicodeCharacter0xa9e2': '\xea\xa7\xa2',
'UnicodeCharacter0xa9e5': '\xea\xa7\xa5',
'UnicodeCharacter0xa9e4': '\xea\xa7\xa4',
'UnicodeCharacter0xa9e7': '\xea\xa7\xa7',
'UnicodeCharacter0xa9e6': '\xea\xa7\xa6',
'UnicodeCharacter0xa9e9': '\xea\xa7\xa9',
'UnicodeCharacter0xa9e8': '\xea\xa7\xa8',
'imaginaryI': '\xe2\x85\x88',
'jj': '\xe2\x85\x89',
'ArabicLetterAlefWithWavyHamzaBelow': '\xd9\xb3',
'KhmerSymbolDapbeiKoet': '\xe1\xa7\xad',
'CjkUnifiedIdeograph3b7d': '\xe3\xad\xbd',
'CjkUnifiedIdeograph3b7e': '\xe3\xad\xbe',
'CjkUnifiedIdeograph3b7f': '\xe3\xad\xbf',
'CjkUnifiedIdeograph3b7a': '\xe3\xad\xba',
'CjkUnifiedIdeograph3b7b': '\xe3\xad\xbb',
'CjkUnifiedIdeograph3b7c': '\xe3\xad\xbc',
'GlagoliticCapitalLetterPe': '\xe2\xb0\x9a',
'CjkUnifiedIdeograph70ab': '\xe7\x82\xab',
'CjkUnifiedIdeograph70ac': '\xe7\x82\xac',
'HangulJungseongEoeu': '\xe1\x85\xbc',
'CjkUnifiedIdeograph70ae': '\xe7\x82\xae',
'CjkUnifiedIdeograph70af': '\xe7\x82\xaf',
'CjkUnifiedIdeograph6969': '\xe6\xa5\xa9',
'GurmukhiLetterNya': '\xe0\xa8\x9e',
'SquareDm': '\xe3\x8d\xb7',
'SquareDl': '\xe3\x8e\x97',
'SquareDa': '\xe3\x8d\xb2',
'CjkUnifiedIdeograph6962': '\xe6\xa5\xa2',
'CjkUnifiedIdeograph6961': '\xe6\xa5\xa1',
'SquareDb': '\xe3\x8f\x88',
'CjkUnifiedIdeograph6967': '\xe6\xa5\xa7',
'CjkUnifiedIdeograph6966': '\xe6\xa5\xa6',
'CjkUnifiedIdeograph6965': '\xe6\xa5\xa5',
'CjkUnifiedIdeograph6964': '\xe6\xa5\xa4',
'KannadaLetterMa': '\xe0\xb2\xae',
'GeorgianSmallLetterUn': '\xe2\xb4\x93',
'CjkUnifiedIdeograph3b74': '\xe3\xad\xb4',
'CjkUnifiedIdeograph3b75': '\xe3\xad\xb5',
'CjkUnifiedIdeograph3b76': '\xe3\xad\xb6',
'CjkUnifiedIdeograph3b77': '\xe3\xad\xb7',
'CjkUnifiedIdeograph3b70': '\xe3\xad\xb0',
'CjkUnifiedIdeograph3b71': '\xe3\xad\xb1',
'CjkUnifiedIdeograph3b72': '\xe3\xad\xb2',
'CjkUnifiedIdeograph3b73': '\xe3\xad\xb3',
'CjkUnifiedIdeograph70a0': '\xe7\x82\xa0',
'EthiopicSyllableGyo': '\xe2\xb7\x9e',
'CjkUnifiedIdeograph70a2': '\xe7\x82\xa2',
'CjkUnifiedIdeograph70a3': '\xe7\x82\xa3',
'CjkUnifiedIdeograph3b78': '\xe3\xad\xb8',
'CjkUnifiedIdeograph3b79': '\xe3\xad\xb9',
'CjkUnifiedIdeograph70a6': '\xe7\x82\xa6',
'EthiopicSyllableGyi': '\xe2\xb7\x9a',
'EthiopicSyllableGyu': '\xe2\xb7\x99',
'HangulSyllableMwels': '\xeb\xad\xb8',
'HangulSyllableMwelp': '\xeb\xad\xba',
'HangulSyllableJjweolt': '\xec\xad\xb1',
'CjkUnifiedIdeograph9a88': '\xe9\xaa\x88',
'HangulSyllablePwad': '\xed\x90\x8f',
'UnicodeCharacter0xeb8a': '\xee\xae\x8a',
'CyrillicSmallLetterIo': '\xd1\x91',
'CjkUnifiedIdeograph696c': '\xe6\xa5\xac',
'CjkUnifiedIdeograph696b': '\xe6\xa5\xab',
'CjkUnifiedIdeograph696a': '\xe6\xa5\xaa',
'CjkUnifiedIdeograph696f': '\xe6\xa5\xaf',
'CjkUnifiedIdeograph696e': '\xe6\xa5\xae',
'CyrillicSmallLetterIe': '\xd0\xb5',
'CjkUnifiedIdeograph941b': '\xe9\x90\x9b',
'HangulSyllableAess': '\xec\x95\xb4',
'HangulSyllableDwec': '\xeb\x92\x9f',
'BraillePatternDots2358': '\xe2\xa2\x96',
'HangulSyllableDwed': '\xeb\x92\x8f',
'HangulSyllableDweg': '\xeb\x92\x89',
'HangulSyllableDweh': '\xeb\x92\xa3',
'HangulSyllableDwek': '\xeb\x92\xa0',
'HangulSyllableDwej': '\xeb\x92\x9e',
'BraillePatternDots2357': '\xe2\xa1\x96',
'BraillePatternDots2356': '\xe2\xa0\xb6',
'HangulSyllableDweo': '\xeb\x91\xac',
'ModifierLetterExtralowDottedToneBar': '\xea\x9c\x8c',
'CjkUnifiedIdeograph9a80': '\xe9\xaa\x80',
'HangulSyllableDwes': '\xeb\x92\x9b',
'HangulSyllableDwet': '\xeb\x92\xa1',
'CjkUnifiedIdeograph6f88': '\xe6\xbe\x88',
'CjkUnifiedIdeograph6f89': '\xe6\xbe\x89',
'CjkUnifiedIdeograph6f86': '\xe6\xbe\x86',
'CjkUnifiedIdeograph6f87': '\xe6\xbe\x87',
'CjkUnifiedIdeograph6f84': '\xe6\xbe\x84',
'CjkUnifiedIdeograph6f85': '\xe6\xbe\x85',
'CjkUnifiedIdeograph6f82': '\xe6\xbe\x82',
'CjkUnifiedIdeograph6f83': '\xe6\xbe\x83',
'CjkUnifiedIdeograph6f80': '\xe6\xbe\x80',
'CjkUnifiedIdeograph6f81': '\xe6\xbe\x81',
'CjkUnifiedIdeograph9a83': '\xe9\xaa\x83',
'LatinSmallLetterOWithOgonekAndMacron': '\xc7\xad',
'FullwidthRightParenthesis': '\xef\xbc\x89',
'CjkUnifiedIdeograph9a84': '\xe9\xaa\x84',
'UnicodeCharacter0xeb8c': '\xee\xae\x8c',
'HangulSyllableJwig': '\xec\xa5\x91',
'CjkUnifiedIdeograph9a86': '\xe9\xaa\x86',
'BoxDrawingsHeavyVerticalAndLeft': '\xe2\x94\xab',
'CjkUnifiedIdeograph9a87': '\xe9\xaa\x87',
'HangulSyllableHyuk': '\xed\x9d\x8c',
'CjkUnifiedIdeograph3627': '\xe3\x98\xa7',
'CjkUnifiedIdeograph6f8f': '\xe6\xbe\x8f',
'ZNotationLeftBindingBracket': '\xe2\xa6\x89',
'CjkUnifiedIdeograph6f8d': '\xe6\xbe\x8d',
'CjkUnifiedIdeograph6f8e': '\xe6\xbe\x8e',
'CjkUnifiedIdeograph6f8b': '\xe6\xbe\x8b',
'CjkUnifiedIdeograph6f8c': '\xe6\xbe\x8c',
'CjkUnifiedIdeograph6f8a': '\xe6\xbe\x8a',
'HangulJongseongRieulmieumkiyeok': '\xe1\x87\x91',
'HangulSyllableMogg': '\xeb\xaa\xaa',
'DiameterSign': '\xe2\x8c\x80',
'ArabicLetterYehBarreeWithHamzaAboveFinalForm': '\xef\xae\xb1',
'RejangLetterWa': '\xea\xa5\x80',
'UnicodeCharacter0xee2a': '\xee\xb8\xaa',
'CjkUnifiedIdeograph8ea7': '\xe8\xba\xa7',
'HangulSyllableBbiss': '\xec\x82\xa4',
'UnicodeCharacter0xeb8e': '\xee\xae\x8e',
'SquareWithUpperRightDiagonalHalfBlack': '\xe2\xac\x94',
'ArabicLigatureSadWithMeemWithMeemInitialForm': '\xef\xb7\x85',
'DevanagariDigitSix': '\xe0\xa5\xac',
'HangulSyllableGos': '\xea\xb3\xb3',
'ArabicRay': '\xd8\x88',
'UnicodeCharacter0xf2b8': '\xef\x8a\xb8',
'CanadianSyllabicsCarrierYu': '\xe1\x98\x8e',
'CanadianSyllabicsCarrierYi': '\xe1\x98\x92',
'CanadianSyllabicsCarrierYo': '\xe1\x98\x8f',
'HangulSyllableDeb': '\xeb\x8e\x81',
'GreekSmallLetterOmicronWithPsili': '\xe1\xbd\x80',
'CanadianSyllabicsCarrierYe': '\xe1\x98\x90',
'HangulSyllableJwit': '\xec\xa5\xa9',
'HangulSyllableJulg': '\xec\xa4\x85',
'HangulSyllableKyaeng': '\xec\xbb\x9d',
'CjkUnifiedIdeograph7714': '\xe7\x9c\x94',
'UnicodeCharacter0xdc57': '\xed\xb1\x97',
'HangulSyllableJwip': '\xec\xa5\xaa',
'LatinSmallLetterZWithDescender': '\xe2\xb1\xac',
'RupeeSign': '\xe2\x82\xa8',
'HangulSyllableCyalt': '\xec\xb1\xad',
'CanadianSyllabicsKoo': '\xe1\x91\xb0',
'UnicodeCharacter0xfde1': '\xef\xb7\xa1',
'CjkUnifiedIdeograph85bd': '\xe8\x96\xbd',
'HangulSyllableGgeolg': '\xea\xbb\x85',
'TifinaghLetterAhaggarYazh': '\xe2\xb5\x8b',
'LatinCapitalLetterAfricanD': '\xc6\x89',
'NkoCombiningNasalizationMark': '\xdf\xb2',
'CjkUnifiedIdeograph38ec': '\xe3\xa3\xac',
'KatakanaLetterRi': '\xe3\x83\xaa',
'UnicodeCharacter0xfde2': '\xef\xb7\xa2',
'Nonforking': '\xe2\xab\x9d',
'KatakanaLetterRa': '\xe3\x83\xa9',
'KatakanaLetterRe': '\xe3\x83\xac',
'UnicodeCharacter0xfde4': '\xef\xb7\xa4',
'MyanmarLetterSa': '\xe1\x80\x9e',
'UnicodeCharacter0xfde7': '\xef\xb7\xa7',
'CjkUnifiedIdeograph85bb': '\xe8\x96\xbb',
'KatakanaLetterRu': '\xe3\x83\xab',
'HangulSyllableYel': '\xec\x98\x90',
'HangulSyllableJulb': '\xec\xa4\x87',
'HebrewPointSinDot': '\xd7\x82',
'UnicodeCharacter0xa8e': '\xe0\xaa\x8e',
'HangulSyllableGoj': '\xea\xb3\xb6',
'HangulSyllableGaegs': '\xea\xb0\x9f',
'CjkCompatibilityIdeographfa69': '\xef\xa9\xa9',
'HangulSyllableGok': '\xea\xb3\xb8',
'CjkCompatibilityIdeographfa67': '\xef\xa9\xa7',
'CjkCompatibilityIdeographfa66': '\xef\xa9\xa6',
'CjkCompatibilityIdeographfa65': '\xef\xa9\xa5',
'CjkCompatibilityIdeographfa64': '\xef\xa9\xa4',
'CjkCompatibilityIdeographfa63': '\xef\xa9\xa3',
'HangulSyllableGoh': '\xea\xb3\xbb',
'CjkCompatibilityIdeographfa61': '\xef\xa9\xa1',
'CjkCompatibilityIdeographfa60': '\xef\xa9\xa0',
'UnicodeCharacter0xf068': '\xef\x81\xa8',
'HangulLetterPieupsioskiyeok': '\xe3\x85\xb4',
'HangulSyllableByul': '\xeb\xb7\xb8',
'UnicodeCharacter0xf060': '\xef\x81\xa0',
'UnicodeCharacter0xf061': '\xef\x81\xa1',
'UnicodeCharacter0xf062': '\xef\x81\xa2',
'UnicodeCharacter0xf063': '\xef\x81\xa3',
'UnicodeCharacter0xf064': '\xef\x81\xa4',
'UnicodeCharacter0xf065': '\xef\x81\xa5',
'UnicodeCharacter0xf066': '\xef\x81\xa6',
'UnicodeCharacter0xf067': '\xef\x81\xa7',
'CjkUnifiedIdeograph4e49': '\xe4\xb9\x89',
'CjkUnifiedIdeograph4e48': '\xe4\xb9\x88',
'CjkUnifiedIdeograph3b2e': '\xe3\xac\xae',
'CjkUnifiedIdeograph4e43': '\xe4\xb9\x83',
'CjkUnifiedIdeograph4e42': '\xe4\xb9\x82',
'CjkUnifiedIdeograph4e41': '\xe4\xb9\x81',
'CjkUnifiedIdeograph4e40': '\xe4\xb9\x80',
'CjkUnifiedIdeograph4e47': '\xe4\xb9\x87',
'CjkUnifiedIdeograph4e46': '\xe4\xb9\x86',
'CjkUnifiedIdeograph4e45': '\xe4\xb9\x85',
'CjkUnifiedIdeograph4e44': '\xe4\xb9\x84',
'UnicodeCharacter0xea8f': '\xee\xaa\x8f',
'WaveDash': '\xe3\x80\x9c',
'HangulSyllableBbulm': '\xeb\xbf\x96',
'CjkCompatibilityIdeographfa6a': '\xef\xa9\xaa',
'YiSyllableGip': '\xea\x87\x9d',
'UnicodeCharacter0xf7a1': '\xef\x9e\xa1',
'UnicodeCharacter0xda37': '\xed\xa8\xb7',
'UnicodeCharacter0xa84': '\xe0\xaa\x84',
'CopticSmallLetterOldCopticHei': '\xe2\xb3\x93',
'UnicodeCharacter0xa80': '\xe0\xaa\x80',
'UnicodeCharacter0xd953': '\xed\xa5\x93',
'CjkUnifiedIdeograph57e9': '\xe5\x9f\xa9',
'CjkUnifiedIdeograph57e8': '\xe5\x9f\xa8',
'UnicodeCharacter0xe349': '\xee\x8d\x89',
'CjkUnifiedIdeograph4e4c': '\xe4\xb9\x8c',
'CjkUnifiedIdeograph4e4b': '\xe4\xb9\x8b',
'CjkUnifiedIdeograph4e4a': '\xe4\xb9\x8a',
'CjkUnifiedIdeograph57e2': '\xe5\x9f\xa2',
'CjkUnifiedIdeograph57e5': '\xe5\x9f\xa5',
'CjkUnifiedIdeograph4e4f': '\xe4\xb9\x8f',
'CjkUnifiedIdeograph4e4e': '\xe4\xb9\x8e',
'CjkUnifiedIdeograph4e4d': '\xe4\xb9\x8d',
'HangulSyllableBbulg': '\xeb\xbf\x95',
'CjkUnifiedIdeograph3de9': '\xe3\xb7\xa9',
'UnicodeCharacter0xf1a5': '\xef\x86\xa5',
'HangulSyllableKwaess': '\xec\xbe\xa0',
'UnicodeCharacter0xf06a': '\xef\x81\xaa',
'UnicodeCharacter0xf06b': '\xef\x81\xab',
'UnicodeCharacter0xf06c': '\xef\x81\xac',
'UnicodeCharacter0xf06d': '\xef\x81\xad',
'UnicodeCharacter0xf06e': '\xef\x81\xae',
'CopticCapitalLetterShei': '\xcf\xa2',
'CjkUnifiedIdeograph8aba': '\xe8\xaa\xba',
'CjkUnifiedIdeograph8abc': '\xe8\xaa\xbc',
'CjkUnifiedIdeograph8abb': '\xe8\xaa\xbb',
'CjkUnifiedIdeograph8abe': '\xe8\xaa\xbe',
'CjkUnifiedIdeograph45b7': '\xe4\x96\xb7',
'CjkUnifiedIdeograph8abf': '\xe8\xaa\xbf',
'AplFunctionalSymbolQuadDiamond': '\xe2\x8c\xba',
'CjkUnifiedIdeograph9f29': '\xe9\xbc\xa9',
'CjkUnifiedIdeograph9f28': '\xe9\xbc\xa8',
'CjkUnifiedIdeograph9f21': '\xe9\xbc\xa1',
'CjkUnifiedIdeograph9f20': '\xe9\xbc\xa0',
'CjkUnifiedIdeograph9f23': '\xe9\xbc\xa3',
'CjkUnifiedIdeograph9f22': '\xe9\xbc\xa2',
'CjkUnifiedIdeograph9f25': '\xe9\xbc\xa5',
'RingPoint': '\xe2\xb8\xb0',
'CjkUnifiedIdeograph965a': '\xe9\x99\x9a',
'CjkUnifiedIdeograph9f26': '\xe9\xbc\xa6',
'CjkUnifiedIdeograph8693': '\xe8\x9a\x93',
'CjkUnifiedIdeograph8692': '\xe8\x9a\x92',
'CjkUnifiedIdeograph8691': '\xe8\x9a\x91',
'CjkUnifiedIdeograph8690': '\xe8\x9a\x90',
'CjkUnifiedIdeograph8697': '\xe8\x9a\x97',
'CjkUnifiedIdeograph8696': '\xe8\x9a\x96',
'CjkUnifiedIdeograph8695': '\xe8\x9a\x95',
'CjkUnifiedIdeograph8694': '\xe8\x9a\x94',
'CjkUnifiedIdeograph8699': '\xe8\x9a\x99',
'CjkUnifiedIdeograph8698': '\xe8\x9a\x98',
'LatinCapitalLetterHWithCaron': '\xc8\x9e',
'YiSyllableRex': '\xea\x8f\x91',
'UnicodeCharacter0xea1f': '\xee\xa8\x9f',
'HangulSyllableOeh': '\xec\x9a\x93',
'UnicodeCharacter0xda30': '\xed\xa8\xb0',
'HangulSyllableOej': '\xec\x9a\x8e',
'HangulSyllableOek': '\xec\x9a\x90',
'HangulSyllableOel': '\xec\x9a\x80',
'HangulSyllableOem': '\xec\x9a\x88',
'LeftpointingAngleBracket': '\xe2\x8c\xa9',
'UnicodeCharacter0xda31': '\xed\xa8\xb1',
'CjkUnifiedIdeograph9f2a': '\xe9\xbc\xaa',
'CjkUnifiedIdeograph9f2c': '\xe9\xbc\xac',
'CjkUnifiedIdeograph9f2b': '\xe9\xbc\xab',
'CjkUnifiedIdeograph9f2e': '\xe9\xbc\xae',
'CjkUnifiedIdeograph9f2d': '\xe9\xbc\xad',
'ModifierLetterDownTack': '\xcb\x95',
'CjkUnifiedIdeograph9f2f': '\xe9\xbc\xaf',
'CjkUnifiedIdeograph8ab1': '\xe8\xaa\xb1',
'CjkUnifiedIdeograph4147': '\xe4\x85\x87',
'CjkUnifiedIdeograph8ab3': '\xe8\xaa\xb3',
'CjkUnifiedIdeograph8ab2': '\xe8\xaa\xb2',
'CjkUnifiedIdeograph8ab5': '\xe8\xaa\xb5',
'CjkUnifiedIdeograph8ab4': '\xe8\xaa\xb4',
'CjkUnifiedIdeograph8ab7': '\xe8\xaa\xb7',
'CjkUnifiedIdeograph8ab6': '\xe8\xaa\xb6',
'CjkUnifiedIdeograph8ab9': '\xe8\xaa\xb9',
'CjkUnifiedIdeograph8ab8': '\xe8\xaa\xb8',
'HangulSyllableOes': '\xec\x9a\x8b',
'HangulSyllableOet': '\xec\x9a\x91',
'UnicodeCharacter0xda35': '\xed\xa8\xb5',
'CircledLatinCapitalLetterT': '\xe2\x93\x89',
'CircledLatinCapitalLetterU': '\xe2\x93\x8a',
'CircledLatinCapitalLetterV': '\xe2\x93\x8b',
'CircledLatinCapitalLetterW': '\xe2\x93\x8c',
'CircledLatinCapitalLetterP': '\xe2\x93\x85',
'CircledLatinCapitalLetterQ': '\xe2\x93\x86',
'CircledLatinCapitalLetterR': '\xe2\x93\x87',
'CircledLatinCapitalLetterS': '\xe2\x93\x88',
'HangulSyllableBbuls': '\xeb\xbf\x98',
'VaiSyllableDhi': '\xea\x94\xb5',
'CircledLatinCapitalLetterX': '\xe2\x93\x8d',
'CircledLatinCapitalLetterY': '\xe2\x93\x8e',
'CircledLatinCapitalLetterZ': '\xe2\x93\x8f',
'CircledLatinCapitalLetterD': '\xe2\x92\xb9',
'CircledLatinCapitalLetterE': '\xe2\x92\xba',
'CircledLatinCapitalLetterF': '\xe2\x92\xbb',
'CircledLatinCapitalLetterG': '\xe2\x92\xbc',
'CircledLatinCapitalLetterA': '\xe2\x92\xb6',
'CircledLatinCapitalLetterB': '\xe2\x92\xb7',
'CircledLatinCapitalLetterC': '\xe2\x92\xb8',
'CircledLatinCapitalLetterL': '\xe2\x93\x81',
'CircledLatinCapitalLetterM': '\xe2\x93\x82',
'CircledLatinCapitalLetterN': '\xe2\x93\x83',
'CircledLatinCapitalLetterO': '\xe2\x93\x84',
'CircledLatinCapitalLetterH': '\xe2\x92\xbd',
'CircledLatinCapitalLetterI': '\xe2\x92\xbe',
'CircledLatinCapitalLetterJ': '\xe2\x92\xbf',
'GreaterthanOrSlantedEqualToWithDotInside': '\xe2\xaa\x80',
'UnicodeCharacter0x187b': '\xe1\xa1\xbb',
'UnicodeCharacter0x187c': '\xe1\xa1\xbc',
'CjkUnifiedIdeograph8f8b': '\xe8\xbe\x8b',
'UnicodeCharacter0x187a': '\xe1\xa1\xba',
'UnicodeCharacter0x187f': '\xe1\xa1\xbf',
'CjkUnifiedIdeograph8f8e': '\xe8\xbe\x8e',
'UnicodeCharacter0x187d': '\xe1\xa1\xbd',
'UnicodeCharacter0x187e': '\xe1\xa1\xbe',
'ZNotationRangeAntirestriction': '\xe2\xa9\xa5',
'UnicodeCharacter0xe051': '\xee\x81\x91',
'AtomSymbol': '\xe2\x9a\x9b',
'SquareWithBottomHalfBlack': '\xe2\xac\x93',
'MongolianLetterGa': '\xe1\xa0\xad',
'CjkUnifiedIdeograph9b3c': '\xe9\xac\xbc',
'HangulSyllableNap': '\xeb\x82\xb2',
'HangulSyllableNas': '\xeb\x82\xab',
'HangulSyllableNat': '\xeb\x82\xb1',
'UnicodeCharacter0xe052': '\xee\x81\x92',
'CjkUnifiedIdeograph7fd9': '\xe7\xbf\x99',
'CjkUnifiedIdeograph7fd8': '\xe7\xbf\x98',
'CjkUnifiedIdeograph8035': '\xe8\x80\xb5',
'CjkUnifiedIdeograph8034': '\xe8\x80\xb4',
'CjkUnifiedIdeograph8033': '\xe8\x80\xb3',
'DoubleCircledDigitSix': '\xe2\x93\xba',
'CjkUnifiedIdeograph8031': '\xe8\x80\xb1',
'CjkUnifiedIdeograph8030': '\xe8\x80\xb0',
'CjkUnifiedIdeograph7fd1': '\xe7\xbf\x91',
'CjkUnifiedIdeograph7fd0': '\xe7\xbf\x90',
'CjkUnifiedIdeograph7fd3': '\xe7\xbf\x93',
'CjkUnifiedIdeograph7fd2': '\xe7\xbf\x92',
'CjkUnifiedIdeograph7fd5': '\xe7\xbf\x95',
'CjkUnifiedIdeograph7fd4': '\xe7\xbf\x94',
'CjkUnifiedIdeograph7fd7': '\xe7\xbf\x97',
'CjkUnifiedIdeograph7fd6': '\xe7\xbf\x96',
'UnicodeCharacter0xdafe': '\xed\xab\xbe',
'HangulSyllableBbwelb': '\xec\x80\x8f',
'UnicodeCharacter0xdafd': '\xed\xab\xbd',
'UnicodeCharacter0xe053': '\xee\x81\x93',
'HangulSyllableNals': '\xeb\x82\xa4',
'CjkUnifiedIdeograph8f80': '\xe8\xbe\x80',
'CjkUnifiedIdeograph8f81': '\xe8\xbe\x81',
'CjkUnifiedIdeograph8f82': '\xe8\xbe\x82',
'CjkUnifiedIdeograph8f83': '\xe8\xbe\x83',
'CjkUnifiedIdeograph8f84': '\xe8\xbe\x84',
'CjkUnifiedIdeograph8f85': '\xe8\xbe\x85',
'CjkUnifiedIdeograph8f86': '\xe8\xbe\x86',
'CjkUnifiedIdeograph8f87': '\xe8\xbe\x87',
'CjkUnifiedIdeograph8f88': '\xe8\xbe\x88',
'CjkUnifiedIdeograph8f89': '\xe8\xbe\x89',
'UnicodeCharacter0x1878': '\xe1\xa1\xb8',
'UnicodeCharacter0x1879': '\xe1\xa1\xb9',
'TifinaghLetterYadh': '\xe2\xb4\xb8',
'CjkUnifiedIdeograph803f': '\xe8\x80\xbf',
'CjkUnifiedIdeograph803e': '\xe8\x80\xbe',
'SinhalaVowelSignKettiIspilla': '\xe0\xb7\x92',
'CjkUnifiedIdeograph803c': '\xe8\x80\xbc',
'CjkUnifiedIdeograph803b': '\xe8\x80\xbb',
'CjkUnifiedIdeograph803a': '\xe8\x80\xba',
'CjkUnifiedIdeograph9db2': '\xe9\xb6\xb2',
'CombiningGraveToneMark': '\xcd\x80',
'UnicodeCharacter0xe74a': '\xee\x9d\x8a',
'CjkUnifiedIdeograph7fdc': '\xe7\xbf\x9c',
'DevanagariLetterSha': '\xe0\xa4\xb6',
'TifinaghLetterYadd': '\xe2\xb4\xb9',
'CjkUnifiedIdeograph7fdd': '\xe7\xbf\x9d',
'SinhalaPunctuationKunddaliya': '\xe0\xb7\xb4',
'CjkUnifiedIdeograph7fdf': '\xe7\xbf\x9f',
'CjkUnifiedIdeograph9db0': '\xe9\xb6\xb0',
'CjkUnifiedIdeograph7904': '\xe7\xa4\x84',
'CjkUnifiedIdeograph9db7': '\xe9\xb6\xb7',
'CjkUnifiedIdeograph9db6': '\xe9\xb6\xb6',
'CjkUnifiedIdeograph5126': '\xe5\x84\xa6',
'CjkUnifiedIdeograph5127': '\xe5\x84\xa7',
'CjkUnifiedIdeograph5124': '\xe5\x84\xa4',
'CjkUnifiedIdeograph5125': '\xe5\x84\xa5',
'CjkUnifiedIdeograph5122': '\xe5\x84\xa2',
'CjkUnifiedIdeograph5123': '\xe5\x84\xa3',
'CjkUnifiedIdeograph5120': '\xe5\x84\xa0',
'CjkUnifiedIdeograph5121': '\xe5\x84\xa1',
'UnicodeCharacter0xe055': '\xee\x81\x95',
'TibetanAstrologicalSignSgraGcanCharRtags': '\xe0\xbc\x97',
'CjkUnifiedIdeograph5128': '\xe5\x84\xa8',
'CjkUnifiedIdeograph5129': '\xe5\x84\xa9',
'CjkUnifiedIdeograph9636': '\xe9\x98\xb6',
'CjkUnifiedIdeograph9637': '\xe9\x98\xb7',
'CjkUnifiedIdeograph9634': '\xe9\x98\xb4',
'CjkUnifiedIdeograph9635': '\xe9\x98\xb5',
'CjkUnifiedIdeograph5fe8': '\xe5\xbf\xa8',
'CjkUnifiedIdeograph5fe9': '\xe5\xbf\xa9',
'CjkUnifiedIdeograph4968': '\xe4\xa5\xa8',
'CjkUnifiedIdeograph9631': '\xe9\x98\xb1',
'CjkUnifiedIdeograph5fe4': '\xe5\xbf\xa4',
'CjkUnifiedIdeograph5fe5': '\xe5\xbf\xa5',
'CjkUnifiedIdeograph5fe6': '\xe5\xbf\xa6',
'CjkUnifiedIdeograph5fe7': '\xe5\xbf\xa7',
'CjkUnifiedIdeograph5fe0': '\xe5\xbf\xa0',
'CjkUnifiedIdeograph5fe1': '\xe5\xbf\xa1',
'CjkUnifiedIdeograph5fe2': '\xe5\xbf\xa2',
'CjkUnifiedIdeograph5fe3': '\xe5\xbf\xa3',
'GeorgianSmallLetterDon': '\xe2\xb4\x83',
'UnicodeCharacter0xe056': '\xee\x81\x96',
'HangulSyllableTwaeg': '\xed\x87\x99',
'HangulSyllableTeung': '\xed\x8b\x8d',
'UnicodeCharacter0xf808': '\xef\xa0\x88',
'HangulSyllableBbwelp': '\xec\x80\x92',
'LatinCapitalLetterJWithStroke': '\xc9\x88',
'ArabicLetterMeemMedialForm': '\xef\xbb\xa4',
'KangxiRadicalSweet': '\xe2\xbd\xa2',
'DoubleRightArcLessthanBracket': '\xe2\xa6\x96',
'UnicodeCharacter0xf800': '\xef\xa0\x80',
'UnicodeCharacter0xf801': '\xef\xa0\x81',
'UnicodeCharacter0xf806': '\xef\xa0\x86',
'HangulSyllableKyaelg': '\xec\xbb\x91',
'UnicodeCharacter0xf804': '\xef\xa0\x84',
'UnicodeCharacter0xf805': '\xef\xa0\x85',
'CjkUnifiedIdeograph963f': '\xe9\x98\xbf',
'BraillePatternDots345': '\xe2\xa0\x9c',
'CjkUnifiedIdeograph963e': '\xe9\x98\xbe',
'CjkUnifiedIdeograph963b': '\xe9\x98\xbb',
'CjkUnifiedIdeograph963c': '\xe9\x98\xbc',
'LatinSmallLetterCWithHook': '\xc6\x88',
'CjkUnifiedIdeograph963a': '\xe9\x98\xba',
'CjkUnifiedIdeograph5fed': '\xe5\xbf\xad',
'CjkRadicalSimplifiedHorn': '\xe2\xbb\x86',
'CjkUnifiedIdeograph5fef': '\xe5\xbf\xaf',
'YiSyllableDa': '\xea\x84\x89',
'CjkUnifiedIdeograph5fea': '\xe5\xbf\xaa',
'CjkUnifiedIdeograph5feb': '\xe5\xbf\xab',
'CjkUnifiedIdeograph5fec': '\xe5\xbf\xac',
'CjkUnifiedIdeograph512f': '\xe5\x84\xaf',
'CjkUnifiedIdeograph512d': '\xe5\x84\xad',
'CjkUnifiedIdeograph512e': '\xe5\x84\xae',
'CjkUnifiedIdeograph512b': '\xe5\x84\xab',
'CjkUnifiedIdeograph512c': '\xe5\x84\xac',
'DoubleSucceeds': '\xe2\xaa\xbc',
'CjkUnifiedIdeograph512a': '\xe5\x84\xaa',
'HangulSyllableKyaelb': '\xec\xbb\x93',
'UnicodeCharacter0xee66': '\xee\xb9\xa6',
'YiSyllableDu': '\xea\x84\x96',
'UnicodeCharacter0xf84a': '\xef\xa1\x8a',
'LatinSmallLetterReversedOpenE': '\xc9\x9c',
'HangulLetterYoya': '\xe3\x86\x87',
'HangulSyllableBaelh': '\xeb\xb0\xbf',
'UnicodeCharacter0xf80b': '\xef\xa0\x8b',
'HangulSyllableBaelg': '\xeb\xb0\xb9',
'HangulSyllableKeoss': '\xec\xbb\xb8',
'UnicodeCharacter0xf80a': '\xef\xa0\x8a',
'HangulSyllableBaelb': '\xeb\xb0\xbb',
'UnicodeCharacter0xf80d': '\xef\xa0\x8d',
'UnicodeCharacter0xf80e': '\xef\xa0\x8e',
'UnicodeCharacter0xee62': '\xee\xb9\xa2',
'HangulSyllableBaelt': '\xeb\xb0\xbd',
'UnicodeCharacter0xee63': '\xee\xb9\xa3',
'YiSyllableNry': '\xea\x8e\xa7',
'HangulSyllableBaelp': '\xeb\xb0\xbe',
'UnicodeCharacter0xe059': '\xee\x81\x99',
'CjkUnifiedIdeograph978e': '\xe9\x9e\x8e',
'CopticCapitalLetterKhei': '\xcf\xa6',
'HangulSyllableMyaes': '\xeb\xa8\xaf',
'CjkUnifiedIdeograph978b': '\xe9\x9e\x8b',
'HangulSyllableTyibs': '\xed\x8b\xa6',
'HangulSyllableGeunh': '\xea\xb7\xbe',
'UnicodeCharacter0xebaa': '\xee\xae\xaa',
'PhagspaLetterEe': '\xea\xa1\xa6',
'HangulSyllableKyaelt': '\xec\xbb\x95',
'GeorgianSmallLetterZen': '\xe2\xb4\x86',
'AplFunctionalSymbolCommaBar': '\xe2\x8d\xaa',
'CjkUnifiedIdeograph978a': '\xe9\x9e\x8a',
'CombiningLowLine': '\xcc\xb2',
'UpperRightPencil': '\xe2\x9c\x90',
'VaiDigitSix': '\xea\x98\xa6',
'GeorgianLetterUn': '\xe1\x83\xa3',
'HangulSyllableMyaet': '\xeb\xa8\xb5',
'UnicodeCharacter0xdc5d': '\xed\xb1\x9d',
'HangulSyllableDyanh': '\xeb\x8c\xa2',
'UnicodeCharacter0xdc5f': '\xed\xb1\x9f',
'UnicodeCharacter0xe155': '\xee\x85\x95',
'DownwardsArrowWithCornerLeftwards': '\xe2\x86\xb5',
'UnicodeCharacter0xdc5a': '\xed\xb1\x9a',
'UnicodeCharacter0xdc5b': '\xed\xb1\x9b',
'HangulSyllableCing': '\xec\xb9\xad',
'CjkUnifiedIdeograph3403': '\xe3\x90\x83',
'CjkUnifiedIdeograph3402': '\xe3\x90\x82',
'CjkUnifiedIdeograph3401': '\xe3\x90\x81',
'CjkUnifiedIdeograph3400': '\xe3\x90\x80',
'CjkUnifiedIdeograph3407': '\xe3\x90\x87',
'CjkUnifiedIdeograph3406': '\xe3\x90\x86',
'CjkUnifiedIdeograph3405': '\xe3\x90\x85',
'CjkUnifiedIdeograph3404': '\xe3\x90\x84',
'CjkUnifiedIdeograph3409': '\xe3\x90\x89',
'CjkUnifiedIdeograph3408': '\xe3\x90\x88',
'SumSum': '\xe2\x85\x80',
'UnicodeCharacter0xf6cc': '\xef\x9b\x8c',
'YiSyllableHnox': '\xea\x85\xb7',
'HangulSyllableGgweonj': '\xea\xbf\x99',
'ElementOfWithUnderbar': '\xe2\x8b\xb8',
'UnicodeCharacter0xdaf2': '\xed\xab\xb2',
'UnicodeCharacter0xe154': '\xee\x85\x94',
'HangulSyllableGgweong': '\xea\xbf\xa9',
'DevanagariLetterGga': '\xe0\xa5\xbb',
'GreekSmallLetterUpsilonWithVaria': '\xe1\xbd\xba',
'HangulSyllableHyanj': '\xed\x96\x95',
'CjkUnifiedIdeograph76da': '\xe7\x9b\x9a',
'CjkUnifiedIdeograph9b31': '\xe9\xac\xb1',
'VaiSyllableThoo': '\xea\x96\x80',
'UnicodeCharacter0xdaf7': '\xed\xab\xb7',
'CjkUnifiedIdeograph76dc': '\xe7\x9b\x9c',
'UnicodeCharacter0xdaf6': '\xed\xab\xb6',
'CjkUnifiedIdeograph340c': '\xe3\x90\x8c',
'CjkUnifiedIdeograph340b': '\xe3\x90\x8b',
'CjkUnifiedIdeograph340a': '\xe3\x90\x8a',
'CjkUnifiedIdeograph76db': '\xe7\x9b\x9b',
'CjkUnifiedIdeograph340f': '\xe3\x90\x8f',
'CjkUnifiedIdeograph340e': '\xe3\x90\x8e',
'CjkUnifiedIdeograph340d': '\xe3\x90\x8d',
'HangulSyllablePyeos': '\xed\x8f\x87',
'CjkUnifiedIdeograph76de': '\xe7\x9b\x9e',
'UnicodeCharacter0xdaf4': '\xed\xab\xb4',
'OriyaLetterGa': '\xe0\xac\x97',
'CjkUnifiedIdeograph790b': '\xe7\xa4\x8b',
'YiSyllableChyx': '\xea\x8d\xb7',
'HangulSyllableGegs': '\xea\xb2\x8f',
'HangulSyllablePeolb': '\xed\x8e\x87',
'UnicodeCharacter0xdc59': '\xed\xb1\x99',
'YiSyllableChyr': '\xea\x8d\xbb',
'UnicodeCharacter0xdc55': '\xed\xb1\x95',
'YiSyllableChyp': '\xea\x8d\xb9',
'CjkUnifiedIdeograph76df': '\xe7\x9b\x9f',
'UnicodeCharacter0xdc50': '\xed\xb1\x90',
'HangulSyllableGgaeng': '\xea\xb9\xbd',
'YiSyllableChyt': '\xea\x8d\xb6',
'UnicodeCharacter0xdc53': '\xed\xb1\x93',
'CjkUnifiedIdeograph81fe': '\xe8\x87\xbe',
'CjkUnifiedIdeograph81fd': '\xe8\x87\xbd',
'CjkUnifiedIdeograph81ff': '\xe8\x87\xbf',
'CjkUnifiedIdeograph81fa': '\xe8\x87\xba',
'UnicodeCharacter0xe156': '\xee\x85\x96',
'CjkUnifiedIdeograph81fc': '\xe8\x87\xbc',
'CjkUnifiedIdeograph81fb': '\xe8\x87\xbb',
'LeftwardsTwoheadedArrowWithDoubleVerticalStroke': '\xe2\xac\xb5',
'HangulSyllableDwenh': '\xeb\x92\x8e',
'RunicLetterLongbranchyr': '\xe1\x9b\xa6',
'CjkUnifiedIdeograph9dba': '\xe9\xb6\xba',
'MalayalamDigitSeven': '\xe0\xb5\xad',
'CjkUnifiedIdeograph97a8': '\xe9\x9e\xa8',
'CjkUnifiedIdeograph98b4': '\xe9\xa2\xb4',
'WhiteVerticalBar': '\xe2\xab\xbe',
'HangulSyllableDwenj': '\xeb\x92\x8d',
'LimbuVowelcarrierLetter': '\xe1\xa4\x80',
'CjkUnifiedIdeograph97a1': '\xe9\x9e\xa1',
'CjkUnifiedIdeograph97a0': '\xe9\x9e\xa0',
'CjkUnifiedIdeograph97a3': '\xe9\x9e\xa3',
'CjkUnifiedIdeograph97a2': '\xe9\x9e\xa2',
'CjkUnifiedIdeograph97a5': '\xe9\x9e\xa5',
'CjkUnifiedIdeograph97a4': '\xe9\x9e\xa4',
'CjkUnifiedIdeograph97a7': '\xe9\x9e\xa7',
'CjkUnifiedIdeograph97a6': '\xe9\x9e\xa6',
'NeitherLessthanNorEquivalentTo': '\xe2\x89\xb4',
'UnicodeCharacter0xe151': '\xee\x85\x91',
'HangulSyllableSyeolg': '\xec\x85\x9d',
'CjkUnifiedIdeograph9dbe': '\xe9\xb6\xbe',
'KangxiRadicalLegs': '\xe2\xbc\x89',
'HangulSyllableSyugs': '\xec\x8a\x8b',
'HangulSyllableJyeogg': '\xec\xa0\xba',
'CjkUnifiedIdeograph9dbd': '\xe9\xb6\xbd',
'UnicodeCharacter0xda3b': '\xed\xa8\xbb',
'HangulSyllableSyeolm': '\xec\x85\x9e',
'CjkUnifiedIdeograph8c05': '\xe8\xb0\x85',
'CjkUnifiedIdeograph8c04': '\xe8\xb0\x84',
'CjkUnifiedIdeograph8c07': '\xe8\xb0\x87',
'CjkUnifiedIdeograph8c06': '\xe8\xb0\x86',
'CjkUnifiedIdeograph8c01': '\xe8\xb0\x81',
'CjkUnifiedIdeograph8c00': '\xe8\xb0\x80',
'CjkUnifiedIdeograph8c03': '\xe8\xb0\x83',
'CjkUnifiedIdeograph8c02': '\xe8\xb0\x82',
'KangxiRadicalBean': '\xe2\xbe\x96',
'HangulSyllableMyaem': '\xeb\xa8\xac',
'CjkUnifiedIdeograph8c09': '\xe8\xb0\x89',
'CjkUnifiedIdeograph8c08': '\xe8\xb0\x88',
'HangulSyllableJyeogs': '\xec\xa0\xbb',
'UnicodeCharacter0xda3e': '\xed\xa8\xbe',
'UnicodeCharacter0xe150': '\xee\x85\x90',
'HangulSyllablePeuh': '\xed\x94\x9f',
'CjkUnifiedIdeograph97aa': '\xe9\x9e\xaa',
'CjkUnifiedIdeograph97ac': '\xe9\x9e\xac',
'CjkUnifiedIdeograph97ab': '\xe9\x9e\xab',
'CjkUnifiedIdeograph97ae': '\xe9\x9e\xae',
'CjkUnifiedIdeograph97ad': '\xe9\x9e\xad',
'CjkUnifiedIdeograph97af': '\xe9\x9e\xaf',
'CjkUnifiedIdeograph81f5': '\xe8\x87\xb5',
'CjkUnifiedIdeograph81f4': '\xe8\x87\xb4',
'CjkUnifiedIdeograph81f7': '\xe8\x87\xb7',
'CjkUnifiedIdeograph81f6': '\xe8\x87\xb6',
'CjkUnifiedIdeograph81f1': '\xe8\x87\xb1',
'CjkUnifiedIdeograph81f0': '\xe8\x87\xb0',
'CjkUnifiedIdeograph81f3': '\xe8\x87\xb3',
'CjkUnifiedIdeograph81f2': '\xe8\x87\xb2',
'CjkUnifiedIdeograph98b8': '\xe9\xa2\xb8',
'CyrillicSmallLetterKaWithHook': '\xd3\x84',
'CjkUnifiedIdeograph81f8': '\xe8\x87\xb8',
'HangulSyllablePeuss': '\xed\x94\x98',
'UnicodeCharacter0xe153': '\xee\x85\x93',
'CjkUnifiedIdeograph8c0e': '\xe8\xb0\x8e',
'CjkUnifiedIdeograph8c0d': '\xe8\xb0\x8d',
'CjkUnifiedIdeograph8c0f': '\xe8\xb0\x8f',
'CjkUnifiedIdeograph8c0a': '\xe8\xb0\x8a',
'CjkUnifiedIdeograph8c0c': '\xe8\xb0\x8c',
'CjkUnifiedIdeograph8c0b': '\xe8\xb0\x8b',
'HangulSyllableJju': '\xec\xad\x88',
'HalfwidthHangulLetterYae': '\xef\xbf\x85',
'UnicodeCharacter0xea40': '\xee\xa9\x80',
'CjkUnifiedIdeograph66d3': '\xe6\x9b\x93',
'HangulSyllableSswil': '\xec\x93\x80',
'GreekSmallLetterOmegaWithDasia': '\xe1\xbd\xa1',
'HangulSyllableJjo': '\xec\xaa\xbc',
'HangulSyllableJji': '\xec\xb0\x8c',
'HangulSyllableSswim': '\xec\x93\x88',
'HangulSyllableJje': '\xec\xa9\xa8',
'HangulSyllableSswin': '\xec\x92\xbc',
'CjkUnifiedIdeograph35d6': '\xe3\x97\x96',
'CjkUnifiedIdeograph35d7': '\xe3\x97\x97',
'CjkUnifiedIdeograph35d4': '\xe3\x97\x94',
'CjkUnifiedIdeograph35d5': '\xe3\x97\x95',
'CjkUnifiedIdeograph35d2': '\xe3\x97\x92',
'CjkUnifiedIdeograph35d3': '\xe3\x97\x93',
'CjkUnifiedIdeograph35d0': '\xe3\x97\x90',
'CjkUnifiedIdeograph35d1': '\xe3\x97\x91',
'CjkUnifiedIdeograph5940': '\xe5\xa5\x80',
'CjkUnifiedIdeograph5941': '\xe5\xa5\x81',
'CjkUnifiedIdeograph5942': '\xe5\xa5\x82',
'CjkUnifiedIdeograph5943': '\xe5\xa5\x83',
'CjkUnifiedIdeograph5944': '\xe5\xa5\x84',
'CjkUnifiedIdeograph5945': '\xe5\xa5\x85',
'CjkUnifiedIdeograph35d8': '\xe3\x97\x98',
'CjkUnifiedIdeograph35d9': '\xe3\x97\x99',
'CjkUnifiedIdeograph7898': '\xe7\xa2\x98',
'CjkUnifiedIdeograph7899': '\xe7\xa2\x99',
'UnicodeCharacter0xeeda': '\xee\xbb\x9a',
'UnicodeCharacter0xeedf': '\xee\xbb\x9f',
'HangulSyllableSswij': '\xec\x93\x8e',
'UnicodeCharacter0xeedd': '\xee\xbb\x9d',
'ArabicLetterBehWithSmallV': '\xdd\x96',
'CjkUnifiedIdeograph7890': '\xe7\xa2\x90',
'CjkUnifiedIdeograph7891': '\xe7\xa2\x91',
'CjkUnifiedIdeograph7892': '\xe7\xa2\x92',
'CjkUnifiedIdeograph7893': '\xe7\xa2\x93',
'CjkUnifiedIdeograph7894': '\xe7\xa2\x94',
'CjkUnifiedIdeograph7895': '\xe7\xa2\x95',
'CjkUnifiedIdeograph7896': '\xe7\xa2\x96',
'CjkUnifiedIdeograph7897': '\xe7\xa2\x97',
'MediumRightCurlyBracketOrnament': '\xe2\x9d\xb5',
'CjkCompatibilityIdeographfa19': '\xef\xa8\x99',
'GreekCapitalLetterSan': '\xcf\xba',
'HangulSyllableSyi': '\xec\x8b\x80',
'BoxDrawingsRightLightAndLeftDownHeavy': '\xe2\x94\xb1',
'HangulSyllableGgyolb': '\xea\xbe\xa7',
'CjkUnifiedIdeograph94ab': '\xe9\x92\xab',
'CjkUnifiedIdeograph94ac': '\xe9\x92\xac',
'CjkUnifiedIdeograph94aa': '\xe9\x92\xaa',
'CjkUnifiedIdeograph94af': '\xe9\x92\xaf',
'CjkUnifiedIdeograph94ad': '\xe9\x92\xad',
'CjkUnifiedIdeograph94ae': '\xe9\x92\xae',
'UnicodeCharacter0xeed2': '\xee\xbb\x92',
'CjkUnifiedIdeograph76d9': '\xe7\x9b\x99',
'UnicodeCharacter0xeed0': '\xee\xbb\x90',
'UnicodeCharacter0xeed1': '\xee\xbb\x91',
'UnicodeCharacter0xeed6': '\xee\xbb\x96',
'UnicodeCharacter0xeed7': '\xee\xbb\x97',
'UnicodeCharacter0xeed4': '\xee\xbb\x94',
'UnicodeCharacter0xeed5': '\xee\xbb\x95',
'CopticOldNubianFullStop': '\xe2\xb3\xb9',
'CjkUnifiedIdeograph789a': '\xe7\xa2\x9a',
'CjkUnifiedIdeograph789b': '\xe7\xa2\x9b',
'CjkUnifiedIdeograph789c': '\xe7\xa2\x9c',
'CjkUnifiedIdeograph789d': '\xe7\xa2\x9d',
'PropertyLine': '\xe2\x85\x8a',
'CjkUnifiedIdeograph789f': '\xe7\xa2\x9f',
'CjkUnifiedIdeograph35df': '\xe3\x97\x9f',
'CjkUnifiedIdeograph35dd': '\xe3\x97\x9d',
'CjkUnifiedIdeograph35de': '\xe3\x97\x9e',
'CjkUnifiedIdeograph35db': '\xe3\x97\x9b',
'CjkUnifiedIdeograph35dc': '\xe3\x97\x9c',
'CjkUnifiedIdeograph35da': '\xe3\x97\x9a',
'SyriacZqaphaDotted': '\xdc\xb5',
'CjkUnifiedIdeograph594a': '\xe5\xa5\x8a',
'CjkUnifiedIdeograph594b': '\xe5\xa5\x8b',
'CjkUnifiedIdeograph594c': '\xe5\xa5\x8c',
'CjkUnifiedIdeograph594d': '\xe5\xa5\x8d',
'CjkUnifiedIdeograph594e': '\xe5\xa5\x8e',
'CjkUnifiedIdeograph594f': '\xe5\xa5\x8f',
'UnicodeCharacter0xe5a8': '\xee\x96\xa8',
'HangulSyllableJjyeh': '\xec\xaa\xbb',
'CjkUnifiedIdeograph94a8': '\xe9\x92\xa8',
'CjkUnifiedIdeograph94a9': '\xe9\x92\xa9',
'CjkUnifiedIdeograph94a2': '\xe9\x92\xa2',
'CjkUnifiedIdeograph94a3': '\xe9\x92\xa3',
'CjkUnifiedIdeograph94a0': '\xe9\x92\xa0',
'CjkUnifiedIdeograph94a1': '\xe9\x92\xa1',
'CjkUnifiedIdeograph94a6': '\xe9\x92\xa6',
'CjkUnifiedIdeograph94a7': '\xe9\x92\xa7',
'CjkUnifiedIdeograph94a4': '\xe9\x92\xa4',
'CjkUnifiedIdeograph94a5': '\xe9\x92\xa5',
'CjkUnifiedIdeograph76d1': '\xe7\x9b\x91',
'TibetanMarkMnyamYigGiMgoRgyan': '\xe0\xbf\x91',
'UnicodeCharacter0xe5a4': '\xee\x96\xa4',
'UnicodeCharacter0xaabc': '\xea\xaa\xbc',
'CjkUnifiedIdeograph76d3': '\xe7\x9b\x93',
'CjkUnifiedIdeograph7b6f': '\xe7\xad\xaf',
'CjkUnifiedIdeograph76d2': '\xe7\x9b\x92',
'HangulSyllableJjyen': '\xec\xaa\xa4',
'CjkUnifiedIdeograph989c': '\xe9\xa2\x9c',
'!~==': '\xe2\x89\x87',
'CjkUnifiedIdeograph76d5': '\xe7\x9b\x95',
'CjkRadicalSecondOne': '\xe2\xba\x82',
'CjkUnifiedIdeograph76d4': '\xe7\x9b\x94',
'CjkUnifiedIdeograph36b7': '\xe3\x9a\xb7',
'CjkUnifiedIdeograph36b6': '\xe3\x9a\xb6',
'CjkUnifiedIdeograph36b5': '\xe3\x9a\xb5',
'CjkUnifiedIdeograph36b4': '\xe3\x9a\xb4',
'CjkUnifiedIdeograph36b3': '\xe3\x9a\xb3',
'CjkUnifiedIdeograph36b2': '\xe3\x9a\xb2',
'CjkUnifiedIdeograph36b1': '\xe3\x9a\xb1',
'CjkUnifiedIdeograph36b0': '\xe3\x9a\xb0',
'CjkUnifiedIdeograph76d6': '\xe7\x9b\x96',
'UnicodeCharacter0xe5a2': '\xee\x96\xa2',
'CjkUnifiedIdeograph36b9': '\xe3\x9a\xb9',
'CjkUnifiedIdeograph36b8': '\xe3\x9a\xb8',
'HangulSyllableCyaj': '\xec\xb1\xb6',
'ArabicLetterHehWithYehAbove': '\xdb\x80',
'HangulSyllableCyah': '\xec\xb1\xbb',
'HangulSyllableCyan': '\xec\xb1\xa4',
'YiSyllableVax': '\xea\x83\xab',
'HangulSyllableCyam': '\xec\xb1\xb0',
'HangulSyllableCyab': '\xec\xb1\xb1',
'TifinaghLetterTawellemetYaz': '\xe2\xb5\xa4',
'OriyaLetterSha': '\xe0\xac\xb6',
'HangulSyllableCyag': '\xec\xb1\xa1',
'NotSquareOriginalOfOrEqualTo': '\xe2\x8b\xa3',
'CjkUnifiedIdeograph69a3': '\xe6\xa6\xa3',
'UnicodeCharacter0xe158': '\xee\x85\x98',
'UnicodeCharacter0xef0b': '\xee\xbc\x8b',
'HangulSyllableCyas': '\xec\xb1\xb3',
'HangulSyllableCyap': '\xec\xb1\xba',
'CjkUnifiedIdeograph69a1': '\xe6\xa6\xa1',
'UnicodeCharacter0xf04f': '\xef\x81\x8f',
'HangulSyllableCyat': '\xec\xb1\xb9',
'CjkCompatibilityIdeographfa16': '\xef\xa8\x96',
'CjkUnifiedIdeograph36bf': '\xe3\x9a\xbf',
'CjkUnifiedIdeograph36be': '\xe3\x9a\xbe',
'CjkUnifiedIdeograph36bd': '\xe3\x9a\xbd',
'CjkUnifiedIdeograph36bc': '\xe3\x9a\xbc',
'CjkUnifiedIdeograph36bb': '\xe3\x9a\xbb',
'CjkUnifiedIdeograph36ba': '\xe3\x9a\xba',
'HangulSyllableDwiss': '\xeb\x92\xb8',
'CjkUnifiedIdeograph87ce': '\xe8\x9f\x8e',
'HangulSyllablePeug': '\xed\x94\x85',
'CjkUnifiedIdeograph9784': '\xe9\x9e\x84',
'CyrillicCapitalLetterUWithMacron': '\xd3\xae',
'HangulSyllableToelg': '\xed\x87\xbd',
'CjkUnifiedIdeograph9782': '\xe9\x9e\x82',
'nabla': '\xe2\x88\x87',
'HangulSyllableHyus': '\xed\x9d\x87',
'CjkUnifiedIdeograph7f68': '\xe7\xbd\xa8',
'AplFunctionalSymbolUpTackOverbar': '\xe2\x8d\x91',
'ChamLetterBa': '\xea\xa8\x9d',
'HangulSyllableSsolm': '\xec\x8f\xa2',
'UnicodeCharacter0xe613': '\xee\x98\x93',
'CjkCompatibilityIdeographf9c6': '\xef\xa7\x86',
'YiSyllableLuot': '\xea\x87\x83',
'CjkCompatibilityIdeographfa9e': '\xef\xaa\x9e',
'YiSyllableLuop': '\xea\x87\x86',
'UnicodeCharacter0xf04e': '\xef\x81\x8e',
'GreekSmallLetterIotaWithDialytikaAndOxia': '\xe1\xbf\x93',
'YiSyllableLuox': '\xea\x87\x84',
'CjkCompatibilityIdeographf9c7': '\xef\xa7\x87',
'LeftwardsDoubleDashArrow': '\xe2\xa4\x8c',
'CjkUnifiedIdeograph630c': '\xe6\x8c\x8c',
'CjkUnifiedIdeograph630b': '\xe6\x8c\x8b',
'CjkUnifiedIdeograph630a': '\xe6\x8c\x8a',
'CjkUnifiedIdeograph630f': '\xe6\x8c\x8f',
'PinwheelStar': '\xe2\x9c\xaf',
'CjkUnifiedIdeograph630d': '\xe6\x8c\x8d',
'UnicodeCharacter0xe611': '\xee\x98\x91',
'UnicodeCharacter0x26dd': '\xe2\x9b\x9d',
'UnicodeCharacter0x26de': '\xe2\x9b\x9e',
'UnicodeCharacter0x26df': '\xe2\x9b\x9f',
'UnicodeCharacter0x26da': '\xe2\x9b\x9a',
'UnicodeCharacter0x26db': '\xe2\x9b\x9b',
'UnicodeCharacter0x26dc': '\xe2\x9b\x9c',
'HangulSyllableHoelh': '\xed\x9a\x9b',
'CjkCompatibilityIdeographf9c0': '\xef\xa7\x80',
'HangulSyllableHoelm': '\xed\x9a\x96',
'HangulSyllableHoelb': '\xed\x9a\x97',
'HangulSyllableBbyos': '\xeb\xbf\x83',
'HangulSyllableBbyop': '\xeb\xbf\x8a',
'HangulSyllableHoelg': '\xed\x9a\x95',
'HangulSyllableBbyot': '\xeb\xbf\x89',
'HangulSyllableBbyoj': '\xeb\xbf\x86',
'HangulSyllableBbyok': '\xeb\xbf\x88',
'HangulSyllableBbyoh': '\xeb\xbf\x8b',
'UnicodeCharacter0xf04c': '\xef\x81\x8c',
'HangulSyllableBbyon': '\xeb\xbe\xb4',
'ThaanaLetterDaviyani': '\xde\x91',
'HangulSyllableBbyom': '\xeb\xbf\x80',
'HangulSyllableBbyob': '\xeb\xbf\x81',
'HangulSyllableBbyoc': '\xeb\xbf\x87',
'HangulSyllableHoelp': '\xed\x9a\x9a',
'HangulSyllableBbyog': '\xeb\xbe\xb1',
'HangulSyllableBbyod': '\xeb\xbe\xb7',
'CubeRoot': '\xe2\x88\x9b',
'CjkCompatibilityIdeographf9c1': '\xef\xa7\x81',
'UnicodeCharacter0x26d8': '\xe2\x9b\x98',
'UnicodeCharacter0x26d9': '\xe2\x9b\x99',
'UnicodeCharacter0x26d4': '\xe2\x9b\x94',
'UnicodeCharacter0x26d5': '\xe2\x9b\x95',
'UnicodeCharacter0x26d6': '\xe2\x9b\x96',
'UnicodeCharacter0x26d7': '\xe2\x9b\x97',
'UnicodeCharacter0x26d0': '\xe2\x9b\x90',
'UnicodeCharacter0x26d1': '\xe2\x9b\x91',
'UnicodeCharacter0x26d2': '\xe2\x9b\x92',
'UnicodeCharacter0x26d3': '\xe2\x9b\x93',
'CjkUnifiedIdeograph6309': '\xe6\x8c\x89',
'CjkUnifiedIdeograph6308': '\xe6\x8c\x88',
'CjkUnifiedIdeograph6303': '\xe6\x8c\x83',
'CjkUnifiedIdeograph6302': '\xe6\x8c\x82',
'CjkUnifiedIdeograph6301': '\xe6\x8c\x81',
'CjkUnifiedIdeograph6300': '\xe6\x8c\x80',
'CjkUnifiedIdeograph6307': '\xe6\x8c\x87',
'CjkUnifiedIdeograph6306': '\xe6\x8c\x86',
'CombiningDoubleMacron': '\xcd\x9e',
'CjkUnifiedIdeograph6304': '\xe6\x8c\x84',
'HangulSyllableJaegg': '\xec\x9e\xae',
'YiSyllableQi': '\xea\x8f\xbe',
'LatinSmallLetterFWithDotAbove': '\xe1\xb8\x9f',
'UnicodeCharacter0xef99': '\xee\xbe\x99',
'CanadianSyllabicsCaai': '\xe1\x92\x8a',
'CjkUnifiedIdeograph4e7e': '\xe4\xb9\xbe',
'UnicodeCharacter0xf04a': '\xef\x81\x8a',
'CjkUnifiedIdeograph5ece': '\xe5\xbb\x8e',
'UnicodeCharacter0xc49': '\xe0\xb1\x89',
'CjkUnifiedIdeograph5ecf': '\xe5\xbb\x8f',
'CjkUnifiedIdeograph5eca': '\xe5\xbb\x8a',
'CjkUnifiedIdeograph5ecc': '\xe5\xbb\x8c',
'CjkUnifiedIdeograph5ecb': '\xe5\xbb\x8b',
'HangulSyllableGgoebs': '\xea\xbe\x92',
'UnicodeCharacter0xc45': '\xe0\xb1\x85',
'CjkUnifiedIdeograph7b66': '\xe7\xad\xa6',
'CjkUnifiedIdeograph7bfc': '\xe7\xaf\xbc',
'CjkUnifiedIdeograph7bfb': '\xe7\xaf\xbb',
'CjkUnifiedIdeograph7bfa': '\xe7\xaf\xba',
'IntersectionAboveUnion': '\xe2\xa9\x87',
'CjkUnifiedIdeograph7bfe': '\xe7\xaf\xbe',
'CjkUnifiedIdeograph7bfd': '\xe7\xaf\xbd',
'UnicodeCharacter0xdbd5': '\xed\xaf\x95',
'CjkUnifiedIdeograph7b64': '\xe7\xad\xa4',
'YiSyllableNjy': '\xea\x91\x86',
'UnicodeCharacter0xdae8': '\xed\xab\xa8',
'HangulSyllableKeuss': '\xed\x82\x80',
'CjkUnifiedIdeograph5ec5': '\xe5\xbb\x85',
'CjkUnifiedIdeograph5ec4': '\xe5\xbb\x84',
'CjkUnifiedIdeograph5ec7': '\xe5\xbb\x87',
'CjkUnifiedIdeograph5ec6': '\xe5\xbb\x86',
'CjkUnifiedIdeograph5ec1': '\xe5\xbb\x81',
'OghamLetterColl': '\xe1\x9a\x89',
'SmallRomanNumeralSix': '\xe2\x85\xb5',
'CjkUnifiedIdeograph5ec2': '\xe5\xbb\x82',
'CombiningRightArrowAbove': '\xe2\x83\x97',
'ArabicFathaWithTwoDots': '\xd9\x9e',
'UnicodeCharacter0xc4e': '\xe0\xb1\x8e',
'UnicodeCharacter0xc4f': '\xe0\xb1\x8f',
'UnicodeCharacter0xf01b': '\xef\x80\x9b',
'YiSyllableZup': '\xea\x8a\xa5',
'KangxiRadicalLife': '\xe2\xbd\xa3',
'UnicodeCharacter0xfae3': '\xef\xab\xa3',
'HangulSyllableDdwaegg': '\xeb\x99\x8a',
'HangulSyllableGeogg': '\xea\xb1\xb2',
'HangulSyllableGgeobs': '\xea\xbb\x8e',
'HangulSyllableCalg': '\xec\xb0\xb1',
'YiSyllableQuop': '\xea\x90\x87',
'YiSyllableQuot': '\xea\x90\x84',
'HangulSyllableCalm': '\xec\xb0\xb2',
'YiSyllableQuox': '\xea\x90\x85',
'HangulSyllableCalh': '\xec\xb0\xb7',
'CombiningMacronbreve': '\xe1\xb7\x8c',
'CjkUnifiedIdeograph7bf3': '\xe7\xaf\xb3',
'CjkUnifiedIdeograph7bf2': '\xe7\xaf\xb2',
'CjkUnifiedIdeograph7bf1': '\xe7\xaf\xb1',
'CjkUnifiedIdeograph7bf0': '\xe7\xaf\xb0',
'CjkUnifiedIdeograph7bf7': '\xe7\xaf\xb7',
'CjkUnifiedIdeograph7bf6': '\xe7\xaf\xb6',
'CjkUnifiedIdeograph7bf5': '\xe7\xaf\xb5',
'CjkUnifiedIdeograph7bf4': '\xe7\xaf\xb4',
'CjkUnifiedIdeograph7bf9': '\xe7\xaf\xb9',
'CjkUnifiedIdeograph7bf8': '\xe7\xaf\xb8',
'MyanmarLetterShanNa': '\xe1\x81\xbc',
'GurmukhiSignVisarga': '\xe0\xa8\x83',
'HangulSyllableSung': '\xec\x88\xad',
'UnicodeCharacter0xe3dc': '\xee\x8f\x9c',
'LatinSmallLetterHWithBreveBelow': '\xe1\xb8\xab',
'HangulSyllableSsigs': '\xec\x94\xab',
'BallotBoxWithX': '\xe2\x98\x92',
'CjkUnifiedIdeograph62df': '\xe6\x8b\x9f',
'CjkUnifiedIdeograph62dd': '\xe6\x8b\x9d',
'CjkUnifiedIdeograph62de': '\xe6\x8b\x9e',
'CjkUnifiedIdeograph62db': '\xe6\x8b\x9b',
'CjkUnifiedIdeograph62dc': '\xe6\x8b\x9c',
'CjkUnifiedIdeograph62da': '\xe6\x8b\x9a',
'UnicodeCharacter0xdf55': '\xed\xbd\x95',
'HangulSyllableSaegs': '\xec\x83\x8b',
'ArabicLetterKehehWithThreeDotsPointingUpwardsBelow': '\xdd\xa4',
'UnicodeCharacter0xdf56': '\xed\xbd\x96',
'UnicodeCharacter0xdf51': '\xed\xbd\x91',
'GreekSmallLetterAlphaWithDasia': '\xe1\xbc\x81',
'UnicodeCharacter0xdf53': '\xed\xbd\x93',
'UnicodeCharacter0xdf52': '\xed\xbd\x92',
'LatinSmallLigatureSt': '\xef\xac\x86',
'HeavyHeartExclamationMarkOrnament': '\xe2\x9d\xa3',
'RightwardsWhiteArrow': '\xe2\x87\xa8',
'EthiopicSyllableQhwi': '\xe1\x89\x9a',
'HangulSyllableSaegg': '\xec\x83\x8a',
'UnicodeCharacter0xe587': '\xee\x96\x87',
'EthiopicSyllableQhwe': '\xe1\x89\x9d',
'EthiopicSyllableQhwa': '\xe1\x89\x98',
'EulerConstant': '\xe2\x84\x87',
'HangulSyllableKyab': '\xec\xba\xbd',
'HangulSyllableKyac': '\xec\xbb\x83',
'HangulSyllableKyag': '\xec\xba\xad',
'CjkUnifiedIdeograph62d8': '\xe6\x8b\x98',
'CjkUnifiedIdeograph62d9': '\xe6\x8b\x99',
'CjkUnifiedIdeograph62d6': '\xe6\x8b\x96',
'CjkUnifiedIdeograph62d7': '\xe6\x8b\x97',
'CjkUnifiedIdeograph62d4': '\xe6\x8b\x94',
'CjkUnifiedIdeograph62d5': '\xe6\x8b\x95',
'CjkUnifiedIdeograph62d2': '\xe6\x8b\x92',
'CjkUnifiedIdeograph62d3': '\xe6\x8b\x93',
'CjkUnifiedIdeograph62d0': '\xe6\x8b\x90',
'CjkUnifiedIdeograph62d1': '\xe6\x8b\x91',
'HangulSyllableKyas': '\xec\xba\xbf',
'HangulSyllableKyap': '\xec\xbb\x86',
'HangulSyllableKyat': '\xec\xbb\x85',
'CjkUnifiedIdeograph9871': '\xe9\xa1\xb1',
'KannadaVowelSignAu': '\xe0\xb3\x8c',
'HangulSyllableSseubs': '\xec\x94\x82',
'EraseToTheRight': '\xe2\x8c\xa6',
'HangulLetterSiostikeut': '\xe3\x85\xbc',
'CjkUnifiedIdeograph9873': '\xe9\xa1\xb3',
'UnicodeCharacter0xf0d2': '\xef\x83\x92',
'HangulSyllableBbyaebs': '\xeb\xbb\x86',
'HangulSyllableNyags': '\xeb\x83\x93',
'CjkUnifiedIdeograph9875': '\xe9\xa1\xb5',
'UnicodeCharacter0xdf5e': '\xed\xbd\x9e',
'UnicodeCharacter0xdf5d': '\xed\xbd\x9d',
'HangulSyllableCyebs': '\xec\xb3\xbe',
'UnicodeCharacter0xdf5f': '\xed\xbd\x9f',
'UnicodeCharacter0xdf5a': '\xed\xbd\x9a',
'CjkUnifiedIdeograph9876': '\xe9\xa1\xb6',
'UnicodeCharacter0xdf5c': '\xed\xbd\x9c',
'UnicodeCharacter0xdf5b': '\xed\xbd\x9b',
'HangulSyllableNyagg': '\xeb\x83\x92',
'UnicodeCharacter0xf0d1': '\xef\x83\x91',
'CjkUnifiedIdeograph541d': '\xe5\x90\x9d',
'UnicodeCharacter0xeff1': '\xee\xbf\xb1',
'HangulSyllableSwaelp': '\xec\x87\x92',
'NotParallelTo': '\xe2\x88\xa6',
'CjkUnifiedIdeograph61be': '\xe6\x86\xbe',
'CjkUnifiedIdeograph61bd': '\xe6\x86\xbd',
'CjkUnifiedIdeograph61bc': '\xe6\x86\xbc',
'CjkUnifiedIdeograph61bb': '\xe6\x86\xbb',
'CjkUnifiedIdeograph61ba': '\xe6\x86\xba',
'CjkUnifiedIdeograph7c6f': '\xe7\xb1\xaf',
'CjkUnifiedIdeograph7c6d': '\xe7\xb1\xad',
'CjkUnifiedIdeograph7c6e': '\xe7\xb1\xae',
'CjkUnifiedIdeograph7c6b': '\xe7\xb1\xab',
'CjkUnifiedIdeograph7c6c': '\xe7\xb1\xac',
'HebrewPointQamatsQatan': '\xd7\x87',
'star': '\xe2\x8b\x86',
'HangulSyllableSyeoss': '\xec\x85\xa8',
'CjkUnifiedIdeograph4e74': '\xe4\xb9\xb4',
'HangulSyllableSseolt': '\xec\x8d\xb5',
'TibetanDigitHalfEight': '\xe0\xbc\xb1',
'HangulSyllableSseolp': '\xec\x8d\xb6',
'HangulSyllableSseols': '\xec\x8d\xb4',
'HangulSyllableSseolm': '\xec\x8d\xb2',
'CombiningCyrillicLetterTse': '\xe2\xb7\xb0',
'HangulSyllableSseolh': '\xec\x8d\xb7',
'HangulSyllableSseolg': '\xec\x8d\xb1',
'HangulSyllableSseolb': '\xec\x8d\xb3',
'CjkUnifiedIdeograph7c66': '\xe7\xb1\xa6',
'CjkUnifiedIdeograph7c67': '\xe7\xb1\xa7',
'CjkUnifiedIdeograph7c64': '\xe7\xb1\xa4',
'CjkUnifiedIdeograph7c65': '\xe7\xb1\xa5',
'CjkUnifiedIdeograph7c62': '\xe7\xb1\xa2',
'CjkUnifiedIdeograph7c63': '\xe7\xb1\xa3',
'CjkUnifiedIdeograph7c60': '\xe7\xb1\xa0',
'CjkUnifiedIdeograph7c61': '\xe7\xb1\xa1',
'YiSyllableZzo': '\xea\x8b\x9d',
'UnicodeCharacter0xe584': '\xee\x96\x84',
'CjkUnifiedIdeograph7c68': '\xe7\xb1\xa8',
'CjkUnifiedIdeograph7c69': '\xe7\xb1\xa9',
'YiSyllableZzu': '\xea\x8b\xa3',
'UnicodeCharacter0xf60a': '\xef\x98\x8a',
'CjkUnifiedIdeograph61b9': '\xe6\x86\xb9',
'CopticCapitalLetterZata': '\xe2\xb2\x8c',
'CjkUnifiedIdeograph61b7': '\xe6\x86\xb7',
'CjkUnifiedIdeograph61b6': '\xe6\x86\xb6',
'CjkUnifiedIdeograph61b5': '\xe6\x86\xb5',
'CjkUnifiedIdeograph61b4': '\xe6\x86\xb4',
'CjkUnifiedIdeograph61b3': '\xe6\x86\xb3',
'CjkUnifiedIdeograph61b2': '\xe6\x86\xb2',
'CjkUnifiedIdeograph61b1': '\xe6\x86\xb1',
'CjkUnifiedIdeograph61b0': '\xe6\x86\xb0',
'CjkUnifiedIdeograph4aa6': '\xe4\xaa\xa6',
'CjkCompatibilityIdeographfa94': '\xef\xaa\x94',
'CjkCompatibilityIdeographfa95': '\xef\xaa\x95',
'EthiopicSyllableSwa': '\xe1\x88\xb7',
'HangulSyllableJjeolh': '\xec\xa9\x9b',
'HangulSyllableGyeogs': '\xea\xb2\xab',
'HangulSyllableHunh': '\xed\x9b\x8a',
'CjkCompatibilityIdeographfa93': '\xef\xaa\x93',
'UnicodeCharacter0xdae7': '\xed\xab\xa7',
'CjkCompatibilityIdeographfa90': '\xef\xaa\x90',
'UnicodeCharacter0xf5dd': '\xef\x97\x9d',
'HangulSyllableJjeolg': '\xec\xa9\x95',
'HangulSyllableByej': '\xeb\xb3\xae',
'CanadianSyllabicsCarrierZi': '\xe1\x99\x84',
'HangulSyllableTolb': '\xed\x86\xab',
'YiRadicalKe': '\xea\x93\x86',
'HangulSyllableTolg': '\xed\x86\xa9',
'HangulSyllableGgwelh': '\xea\xbf\xbf',
'UnicodeCharacter0xd89c': '\xed\xa2\x9c',
'HangulSyllableYaeh': '\xec\x96\xb3',
'GujaratiLetterVa': '\xe0\xaa\xb5',
'HangulSyllableDyap': '\xeb\x8c\xb6',
'UnicodeCharacter0xaba': '\xe0\xaa\xba',
'LatinCapitalLetterSWithDotBelowAndDotAbove': '\xe1\xb9\xa8',
'HangulSyllableTolm': '\xed\x86\xaa',
'HangulSyllableTols': '\xed\x86\xac',
'GreekCapitalLetterAlphaWithPsiliAndOxiaAndProsgegrammeni': '\xe1\xbe\x8c',
'HangulSyllableRwaelp': '\xeb\xa2\xa2',
'HangulSyllableGgwelm': '\xea\xbf\xba',
'CjkCompatibilityIdeographfa99': '\xef\xaa\x99',
'DingbatCircledSansserifDigitEight': '\xe2\x9e\x87',
'CyrillicCapitalLetterZheWithDescender': '\xd2\x96',
'EnSpace': '\xe2\x80\x82',
'UnicodeCharacter0xe1f7': '\xee\x87\xb7',
'LatinCapitalLetterOWithDiaeresis': '\xc3\x96',
'DevanagariLetterNa': '\xe0\xa4\xa8',
'SquareMiribaaru': '\xe3\x8d\x8a',
'CjkUnifiedIdeograph91f6': '\xe9\x87\xb6',
'HangulSyllableGgwelb': '\xea\xbf\xbb',
'EthiopicSyllableRoa': '\xe2\xb6\x82',
'CjkUnifiedIdeograph91f7': '\xe9\x87\xb7',
'UnicodeCharacter0xe39f': '\xee\x8e\x9f',
'UnicodeCharacter0xf0df': '\xef\x83\x9f',
'UnicodeCharacter0xd899': '\xed\xa2\x99',
'CjkUnifiedIdeograph91f5': '\xe9\x87\xb5',
'UnicodeCharacter0xdb11': '\xed\xac\x91',
'HangulSyllableGgwelg': '\xea\xbf\xb9',
'YenSign': '\xc2\xa5',
'CjkUnifiedIdeograph91f2': '\xe9\x87\xb2',
'UnicodeCharacter0xd891': '\xed\xa2\x91',
'UnicodeCharacter0xd890': '\xed\xa2\x90',
'UnicodeCharacter0xd897': '\xed\xa2\x97',
'UnicodeCharacter0xab1': '\xe0\xaa\xb1',
'UnicodeCharacter0xd895': '\xed\xa2\x95',
'GeorgianCapitalLetterChin': '\xe1\x82\xb9',
'CjkUnifiedIdeograph77d8': '\xe7\x9f\x98',
'CjkUnifiedIdeograph50ff': '\xe5\x83\xbf',
'CjkUnifiedIdeograph50fe': '\xe5\x83\xbe',
'CjkUnifiedIdeograph50fd': '\xe5\x83\xbd',
'CjkUnifiedIdeograph50fc': '\xe5\x83\xbc',
'CjkUnifiedIdeograph50fb': '\xe5\x83\xbb',
'CjkUnifiedIdeograph50fa': '\xe5\x83\xba',
'CjkUnifiedIdeograph77d0': '\xe7\x9f\x90',
'CjkUnifiedIdeograph77d1': '\xe7\x9f\x91',
'CjkUnifiedIdeograph77d2': '\xe7\x9f\x92',
'CjkUnifiedIdeograph77d3': '\xe7\x9f\x93',
'CjkUnifiedIdeograph77d4': '\xe7\x9f\x94',
'TaiLeLetterVa': '\xe1\xa5\x9d',
'CjkUnifiedIdeograph77d6': '\xe7\x9f\x96',
'CjkUnifiedIdeograph77d7': '\xe7\x9f\x97',
'CjkUnifiedIdeograph7abc': '\xe7\xaa\xbc',
'HangulSyllableDyan': '\xeb\x8c\xa0',
'BuhidLetterNa': '\xe1\x9d\x88',
'UnicodeCharacter0xf0da': '\xef\x83\x9a',
'HangulSyllableDyam': '\xeb\x8c\xac',
'VaiSymbolTaa': '\xea\x98\x99',
'CjkUnifiedIdeograph39fa': '\xe3\xa7\xba',
'CjkUnifiedIdeograph39fb': '\xe3\xa7\xbb',
'CjkUnifiedIdeograph39fc': '\xe3\xa7\xbc',
'CjkUnifiedIdeograph39fd': '\xe3\xa7\xbd',
'CjkUnifiedIdeograph39fe': '\xe3\xa7\xbe',
'CjkUnifiedIdeograph39ff': '\xe3\xa7\xbf',
'HangulSyllableNulm': '\xeb\x88\x8e',
'HangulSyllableNulb': '\xeb\x88\x8f',
'KhmerVowelSignU': '\xe1\x9e\xbb',
'HangulSyllableJjeolb': '\xec\xa9\x97',
'UnicodeCharacter0xdeae': '\xed\xba\xae',
'HangulSyllableNulg': '\xeb\x88\x8d',
'CanadianSyllabicsRcreeRe': '\xe1\x95\x83',
'MeasuredAngle': '\xe2\x88\xa1',
'YiSyllableZhyx': '\xea\x8d\x9c',
'LatinCapitalLetterUWithHornAndHookAbove': '\xe1\xbb\xac',
'UnicodeCharacter0xe05a': '\xee\x81\x9a',
'UnicodeCharacter0xe8b3': '\xee\xa2\xb3',
'KhmerVowelSignI': '\xe1\x9e\xb7',
'KhmerVowelSignE': '\xe1\x9f\x81',
'HangulSyllableNulp': '\xeb\x88\x92',
'CjkUnifiedIdeograph71cd': '\xe7\x87\x8d',
'HangulSyllableBbyubs': '\xec\x81\x8e',
'YiSyllableZhyt': '\xea\x8d\x9b',
'HangulSyllableNult': '\xeb\x88\x91',
'CjkUnifiedIdeograph6959': '\xe6\xa5\x99',
'CjkUnifiedIdeograph9cae': '\xe9\xb2\xae',
'UnicodeCharacter0xf017': '\xef\x80\x97',
'CjkUnifiedIdeograph9cad': '\xe9\xb2\xad',
'HangulSyllableHah': '\xed\x95\xb3',
'CjkUnifiedIdeograph50f7': '\xe5\x83\xb7',
'CjkUnifiedIdeograph50f6': '\xe5\x83\xb6',
'CjkUnifiedIdeograph50f5': '\xe5\x83\xb5',
'CjkUnifiedIdeograph50f4': '\xe5\x83\xb4',
'CjkUnifiedIdeograph50f3': '\xe5\x83\xb3',
'CjkUnifiedIdeograph50f2': '\xe5\x83\xb2',
'CjkUnifiedIdeograph50f1': '\xe5\x83\xb1',
'CjkUnifiedIdeograph50f0': '\xe5\x83\xb0',
'CjkUnifiedIdeograph77da': '\xe7\x9f\x9a',
'CjkUnifiedIdeograph77db': '\xe7\x9f\x9b',
'CjkUnifiedIdeograph77dc': '\xe7\x9f\x9c',
'CjkUnifiedIdeograph77dd': '\xe7\x9f\x9d',
'CjkUnifiedIdeograph77de': '\xe7\x9f\x9e',
'CjkUnifiedIdeograph50f9': '\xe5\x83\xb9',
'UnicodeCharacter0x1a56': '\xe1\xa9\x96',
'CjkUnifiedIdeograph924a': '\xe9\x89\x8a',
'GreekCapitalLetterEpsilonWithPsiliAndVaria': '\xe1\xbc\x9a',
'EthiopicSyllableGwaa': '\xe1\x8c\x93',
'DingbatCircledSansserifDigitNine': '\xe2\x9e\x88',
'UnicodeCharacter0xe4e0': '\xee\x93\xa0',
'UnicodeCharacter0xe05c': '\xee\x81\x9c',
'CjkUnifiedIdeograph39f0': '\xe3\xa7\xb0',
'CjkUnifiedIdeograph39f1': '\xe3\xa7\xb1',
'CjkUnifiedIdeograph39f2': '\xe3\xa7\xb2',
'CjkUnifiedIdeograph39f3': '\xe3\xa7\xb3',
'CjkUnifiedIdeograph39f4': '\xe3\xa7\xb4',
'CjkUnifiedIdeograph39f5': '\xe3\xa7\xb5',
'CjkUnifiedIdeograph39f6': '\xe3\xa7\xb6',
'CjkUnifiedIdeograph39f7': '\xe3\xa7\xb7',
'CjkUnifiedIdeograph39f8': '\xe3\xa7\xb8',
'CjkUnifiedIdeograph39f9': '\xe3\xa7\xb9',
'YiSyllableGget': '\xea\x88\xa6',
'YiSyllableGgep': '\xea\x88\xa9',
'SquareHoon': '\xe3\x8d\x82',
'ParenthesizedHangulKhieukh': '\xe3\x88\x8a',
'YiSyllableGgex': '\xea\x88\xa7',
'MalayalamLetterTha': '\xe0\xb4\xa5',
'UnicodeCharacter0xf638': '\xef\x98\xb8',
'UnicodeCharacter0xf6f': '\xe0\xbd\xaf',
'UnicodeCharacter0xf6e': '\xe0\xbd\xae',
'UnicodeCharacter0xf6d': '\xe0\xbd\xad',
'UnicodeCharacter0xf631': '\xef\x98\xb1',
'DongSign': '\xe2\x82\xab',
'UnicodeCharacter0xf633': '\xef\x98\xb3',
'UnicodeCharacter0xf632': '\xef\x98\xb2',
'UnicodeCharacter0xf635': '\xef\x98\xb5',
'UnicodeCharacter0xf634': '\xef\x98\xb4',
'UnicodeCharacter0xf637': '\xef\x98\xb7',
'UnicodeCharacter0xf636': '\xef\x98\xb6',
'CjkUnifiedIdeograph5b0f': '\xe5\xac\x8f',
'CjkUnifiedIdeograph5b0e': '\xe5\xac\x8e',
'CjkUnifiedIdeograph5b0d': '\xe5\xac\x8d',
'CjkUnifiedIdeograph5b0c': '\xe5\xac\x8c',
'CjkUnifiedIdeograph5b0b': '\xe5\xac\x8b',
'CjkUnifiedIdeograph5b0a': '\xe5\xac\x8a',
'CjkUnifiedIdeograph9abb': '\xe9\xaa\xbb',
'DoublelineSlantedEqualToOrGreaterthan': '\xe2\xaa\x9c',
'HangulSyllableDdaegs': '\xeb\x95\x8f',
'UnicodeCharacter0xe8b7': '\xee\xa2\xb7',
'EthiopicSyllableThaa': '\xe1\x8c\xa3',
'CjkUnifiedIdeograph9a28': '\xe9\xa8\xa8',
'CjkUnifiedIdeograph9a29': '\xe9\xa8\xa9',
'PhilippineDoublePunctuation': '\xe1\x9c\xb6',
'LatinSmallLetterCon': '\xea\x9d\xaf',
'CjkUnifiedIdeograph9a22': '\xe9\xa8\xa2',
'CjkUnifiedIdeograph9a23': '\xe9\xa8\xa3',
'CjkUnifiedIdeograph9a20': '\xe9\xa8\xa0',
'CjkUnifiedIdeograph9a21': '\xe9\xa8\xa1',
'CjkUnifiedIdeograph9a26': '\xe9\xa8\xa6',
'CjkUnifiedIdeograph9a27': '\xe9\xa8\xa7',
'CjkUnifiedIdeograph9a24': '\xe9\xa8\xa4',
'CjkUnifiedIdeograph9a25': '\xe9\xa8\xa5',
'ArabicLetterKafWithTwoDotsAbove': '\xdd\xbf',
'UnicodeCharacter0x20b8': '\xe2\x82\xb8',
'CjkUnifiedIdeograph7994': '\xe7\xa6\x94',
'UnicodeCharacter0xf63a': '\xef\x98\xba',
'UnicodeCharacter0xe05f': '\xee\x81\x9f',
'UnicodeCharacter0xe8b6': '\xee\xa2\xb6',
'UnicodeCharacter0xf63b': '\xef\x98\xbb',
'UnicodeCharacter0xf63e': '\xef\x98\xbe',
'UnicodeCharacter0xf63d': '\xef\x98\xbd',
'MongolianLetterTodoGaa': '\xe1\xa1\x98',
'UnicodeCharacter0xf873': '\xef\xa1\xb3',
'HangulLetterRieul': '\xe3\x84\xb9',
'HangulSyllablePing': '\xed\x95\x91',
'CombiningDoubleInvertedBreve': '\xcd\xa1',
'HangulSyllableKaet': '\xec\xba\xa9',
'UnicodeCharacter0xde8': '\xe0\xb7\xa8',
'+/-': '\xc2\xb1',
'CjkUnifiedIdeograph9a2b': '\xe9\xa8\xab',
'CjkUnifiedIdeograph9a2c': '\xe9\xa8\xac',
'LargeLeftTriangleOperator': '\xe2\xa8\x9e',
'CjkUnifiedIdeograph9a2f': '\xe9\xa8\xaf',
'CjkUnifiedIdeograph9a2d': '\xe9\xa8\xad',
'CjkUnifiedIdeograph9a2e': '\xe9\xa8\xae',
'CjkUnifiedIdeograph5b07': '\xe5\xac\x87',
'CjkUnifiedIdeograph5b06': '\xe5\xac\x86',
'CjkUnifiedIdeograph5b05': '\xe5\xac\x85',
'CjkUnifiedIdeograph5b04': '\xe5\xac\x84',
'CjkUnifiedIdeograph5b03': '\xe5\xac\x83',
'CjkUnifiedIdeograph5b02': '\xe5\xac\x82',
'CjkUnifiedIdeograph5b01': '\xe5\xac\x81',
'RightwardsArrowWithTailWithVerticalStroke': '\xe2\xa4\x94',
'HangulSyllableHab': '\xed\x95\xa9',
'CjkUnifiedIdeograph5b09': '\xe5\xac\x89',
'CjkUnifiedIdeograph5b08': '\xe5\xac\x88',
'CjkUnifiedIdeograph87e2': '\xe8\x9f\xa2',
'HangulSyllableRwigg': '\xeb\xa4\xbe',
'HangulSyllableMyulb': '\xeb\xae\xaf',
'HangulSyllableMyulg': '\xeb\xae\xad',
'HangulSyllableMyulh': '\xeb\xae\xb3',
'SesameDot': '\xef\xb9\x85',
'LeftWhiteSquareBracket': '\xe3\x80\x9a',
'HangulSyllableMyulm': '\xeb\xae\xae',
'CjkUnifiedIdeograph87e0': '\xe8\x9f\xa0',
'UnicodeCharacter0xe583': '\xee\x96\x83',
'CjkUnifiedIdeograph9d57': '\xe9\xb5\x97',
'UnicodeCharacter0xde0': '\xe0\xb7\xa0',
'HangulSyllableMyuls': '\xeb\xae\xb0',
'CjkUnifiedIdeograph87e1': '\xe8\x9f\xa1',
'HangulSyllableMyult': '\xeb\xae\xb1',
'HangulSyllableRwigs': '\xeb\xa4\xbf',
'CjkUnifiedIdeograph91fb': '\xe9\x87\xbb',
'SyriacObliqueLineAbove': '\xdd\x87',
'YiSyllableJjox': '\xea\x90\xa4',
'BugineseEndOfSection': '\xe1\xa8\x9f',
'CjkUnifiedIdeograph91fc': '\xe9\x87\xbc',
'UnicodeCharacter0x2e50': '\xe2\xb9\x90',
'UnicodeCharacter0x2e51': '\xe2\xb9\x91',
'UnicodeCharacter0x2e52': '\xe2\xb9\x92',
'UnicodeCharacter0x2e53': '\xe2\xb9\x93',
'UnicodeCharacter0x2e54': '\xe2\xb9\x94',
'UnicodeCharacter0x2e55': '\xe2\xb9\x95',
'UnicodeCharacter0x2e56': '\xe2\xb9\x96',
'UnicodeCharacter0x2e57': '\xe2\xb9\x97',
'UnicodeCharacter0x2e58': '\xe2\xb9\x98',
'UnicodeCharacter0x2e59': '\xe2\xb9\x99',
'UnicodeCharacter0xea96': '\xee\xaa\x96',
'CjkUnifiedIdeograph91fa': '\xe9\x87\xba',
'UnicodeCharacter0xe6b7': '\xee\x9a\xb7',
'UnicodeCharacter0xe6b6': '\xee\x9a\xb6',
'HangulSyllableKaes': '\xec\xba\xa3',
'CjkUnifiedIdeograph71c1': '\xe7\x87\x81',
'MongolianLetterTodoAliGaliTa': '\xe1\xa2\x98',
'CjkUnifiedIdeograph87e9': '\xe8\x9f\xa9',
'SinhalaLetterAlpapraanaCayanna': '\xe0\xb6\xa0',
'UnicodeCharacter0xe6b3': '\xee\x9a\xb3',
'HangulSyllablePyess': '\xed\x8f\xa4',
'NewTaiLueLetterHighTsa': '\xe1\xa6\x88',
'HangulLetterRieulhieuh': '\xe3\x85\x80',
'HangulSyllableBbweobs': '\xeb\xbf\xba',
'UnicodeCharacter0xe6b2': '\xee\x9a\xb2',
'HangulSyllableTwigg': '\xed\x8a\x82',
'UnicodeCharacter0xe6b1': '\xee\x9a\xb1',
'HangulSyllableKaep': '\xec\xba\xaa',
'CjkUnifiedIdeograph71c4': '\xe7\x87\x84',
'HangulChoseongPansios': '\xe1\x85\x80',
'CjkUnifiedIdeograph7d87': '\xe7\xb6\x87',
'CjkUnifiedIdeograph7d86': '\xe7\xb6\x86',
'HangulSyllableSyoss': '\xec\x88\x90',
'pi': '\xcf\x80',
'HangulSyllableWags': '\xec\x99\x83',
'UnicodeCharacter0x2e5a': '\xe2\xb9\x9a',
'UnicodeCharacter0x2e5b': '\xe2\xb9\x9b',
'UnicodeCharacter0x2e5c': '\xe2\xb9\x9c',
'UnicodeCharacter0x2e5d': '\xe2\xb9\x9d',
'UnicodeCharacter0x2e5e': '\xe2\xb9\x9e',
'UnicodeCharacter0x2e5f': '\xe2\xb9\x9f',
'HangulSyllableWagg': '\xec\x99\x82',
'CjkUnifiedIdeograph7d82': '\xe7\xb6\x82',
'EthiopicSyllableYo': '\xe1\x8b\xae',
'VaiSyllableBi': '\xea\x94\xac',
'CanadianSyllabicsWestcreeLa': '\xe1\x95\x8d',
'GurmukhiDigitEight': '\xe0\xa9\xae',
'ArabicLetterNgFinalForm': '\xef\xaf\x94',
'EthiopicSyllableYi': '\xe1\x8b\xaa',
'DevanagariLetterDda': '\xe0\xa4\xa1',
'VaiSyllableBa': '\xea\x95\x92',
'EthiopicSyllableYe': '\xe1\x8b\xad',
'CanadianSyllabicsWestcreeLo': '\xe1\x95\x8a',
'VaiSyllableBe': '\xea\x97\xaa',
'EthiopicSyllableYa': '\xe1\x8b\xa8',
'CjkUnifiedIdeograph448f': '\xe4\x92\x8f',
'CjkUnifiedIdeograph448d': '\xe4\x92\x8d',
'CjkUnifiedIdeograph448e': '\xe4\x92\x8e',
'CjkUnifiedIdeograph448b': '\xe4\x92\x8b',
'CjkUnifiedIdeograph448c': '\xe4\x92\x8c',
'CjkUnifiedIdeograph448a': '\xe4\x92\x8a',
'UnicodeCharacter0x884': '\xe0\xa2\x84',
'HangulSyllableGgyuk': '\xeb\x81\x80',
'MusicFlatSign': '\xe2\x99\xad',
'EthiopicSyllableYu': '\xe1\x8b\xa9',
'VaiSyllableBu': '\xea\x96\x9d',
'HangulSyllableCeulm': '\xec\xb8\xaa',
'LessthanClosedByCurve': '\xe2\xaa\xa6',
'Sagittarius': '\xe2\x99\x90',
'CjkUnifiedIdeograph9d54': '\xe9\xb5\x94',
'CjkUnifiedIdeograph57b1': '\xe5\x9e\xb1',
'HangulSyllableCeulg': '\xec\xb8\xa9',
'SubscriptPlusSign': '\xe2\x82\x8a',
'HangulSyllableCeulb': '\xec\xb8\xab',
'HangulSyllableHad': '\xed\x95\x9f',
'CjkUnifiedIdeograph4158': '\xe4\x85\x98',
'CjkUnifiedIdeograph4159': '\xe4\x85\x99',
'CjkUnifiedIdeograph84fe': '\xe8\x93\xbe',
'CjkUnifiedIdeograph4154': '\xe4\x85\x94',
'GreekSmallLetterUpsilonWithDasia': '\xe1\xbd\x91',
'CjkUnifiedIdeograph4156': '\xe4\x85\x96',
'CjkUnifiedIdeograph4157': '\xe4\x85\x97',
'CjkUnifiedIdeograph4150': '\xe4\x85\x90',
'CjkUnifiedIdeograph4151': '\xe4\x85\x91',
'CjkUnifiedIdeograph4152': '\xe4\x85\x92',
'CjkUnifiedIdeograph4153': '\xe4\x85\x93',
'CjkUnifiedIdeograph4486': '\xe4\x92\x86',
'CjkUnifiedIdeograph4487': '\xe4\x92\x87',
'CjkUnifiedIdeograph4484': '\xe4\x92\x84',
'CjkUnifiedIdeograph4485': '\xe4\x92\x85',
'CjkUnifiedIdeograph4482': '\xe4\x92\x82',
'CjkUnifiedIdeograph4483': '\xe4\x92\x83',
'CjkUnifiedIdeograph4480': '\xe4\x92\x80',
'CjkUnifiedIdeograph4481': '\xe4\x92\x81',
'CjkUnifiedIdeograph4488': '\xe4\x92\x88',
'CjkUnifiedIdeograph4489': '\xe4\x92\x89',
'UpwardsHarpoonWithBarbLeftBesideUpwardsHarpoonWithBarbRight': '\xe2\xa5\xa3',
'HangulSyllableSsyaelp': '\xec\x8d\x9a',
'HangulSyllableSsyaels': '\xec\x8d\x98',
'YiSyllableNurx': '\xea\x86\x95',
'HangulSyllableSsyaelt': '\xec\x8d\x99',
'CjkUnifiedIdeograph415d': '\xe4\x85\x9d',
'CjkUnifiedIdeograph415e': '\xe4\x85\x9e',
'CjkUnifiedIdeograph415f': '\xe4\x85\x9f',
'UnicodeCharacter0xeff4': '\xee\xbf\xb4',
'CjkUnifiedIdeograph415a': '\xe4\x85\x9a',
'KangxiRadicalDoNot': '\xe2\xbd\x8f',
'CjkUnifiedIdeograph415c': '\xe4\x85\x9c',
'HangulSyllableSsyaelb': '\xec\x8d\x97',
'HangulSyllableSsyaelg': '\xec\x8d\x95',
'HangulSyllableSsyaelh': '\xec\x8d\x9b',
'HangulSyllableSsyaelm': '\xec\x8d\x96',
'FullwidthYenSign': '\xef\xbf\xa5',
'CjkUnifiedIdeograph4961': '\xe4\xa5\xa1',
'HangulSyllableGgugs': '\xea\xbe\xbb',
'SquareGanma': '\xe3\x8c\x8f',
'HangulSyllableCweonh': '\xec\xb6\xb6',
'CjkUnifiedIdeograph984e': '\xe9\xa1\x8e',
'CjkUnifiedIdeograph984d': '\xe9\xa1\x8d',
'CanadianSyllabicsLwi': '\xe1\x93\x9e',
'CjkUnifiedIdeograph984a': '\xe9\xa1\x8a',
'CjkUnifiedIdeograph984c': '\xe9\xa1\x8c',
'CjkUnifiedIdeograph984b': '\xe9\xa1\x8b',
'ArabicLetterHehGoalMedialForm': '\xef\xae\xa9',
'CjkUnifiedIdeograph82f8': '\xe8\x8b\xb8',
'CanadianSyllabicsLwo': '\xe1\x93\xa2',
'CjkUnifiedIdeograph535e': '\xe5\x8d\x9e',
'CjkUnifiedIdeograph535d': '\xe5\x8d\x9d',
'SinhalaVowelSignKombuva': '\xe0\xb7\x99',
'CjkUnifiedIdeograph535f': '\xe5\x8d\x9f',
'CjkUnifiedIdeograph535a': '\xe5\x8d\x9a',
'CjkUnifiedIdeograph535c': '\xe5\x8d\x9c',
'CjkUnifiedIdeograph535b': '\xe5\x8d\x9b',
'CjkUnifiedIdeograph5683': '\xe5\x9a\x83',
'CjkUnifiedIdeograph5682': '\xe5\x9a\x82',
'CjkUnifiedIdeograph5681': '\xe5\x9a\x81',
'CjkUnifiedIdeograph5680': '\xe5\x9a\x80',
'CjkUnifiedIdeograph5687': '\xe5\x9a\x87',
'LatinCapitalLetterUWithAcute': '\xc3\x9a',
'CjkUnifiedIdeograph5685': '\xe5\x9a\x85',
'CjkUnifiedIdeograph5684': '\xe5\x9a\x84',
'CjkUnifiedIdeograph76e6': '\xe7\x9b\xa6',
'CjkUnifiedIdeograph76e7': '\xe7\x9b\xa7',
'CjkUnifiedIdeograph76e4': '\xe7\x9b\xa4',
'CjkUnifiedIdeograph76e5': '\xe7\x9b\xa5',
'CjkUnifiedIdeograph76e2': '\xe7\x9b\xa2',
'CjkUnifiedIdeograph76e3': '\xe7\x9b\xa3',
'CjkUnifiedIdeograph76e0': '\xe7\x9b\xa0',
'GurmukhiUra': '\xe0\xa9\xb3',
'HangulSyllableBwaelb': '\xeb\xb4\xb7',
'HangulSyllablePoen': '\xed\x91\x84',
'CjkUnifiedIdeograph76e8': '\xe7\x9b\xa8',
'CjkUnifiedIdeograph76e9': '\xe7\x9b\xa9',
'CjkUnifiedIdeograph9845': '\xe9\xa1\x85',
'CjkUnifiedIdeograph9844': '\xe9\xa1\x84',
'CjkUnifiedIdeograph9847': '\xe9\xa1\x87',
'DingbatCircledSansserifNumberTen': '\xe2\x9e\x89',
'CjkUnifiedIdeograph9841': '\xe9\xa1\x81',
'CjkUnifiedIdeograph9840': '\xe9\xa1\x80',
'CjkUnifiedIdeograph9843': '\xe9\xa1\x83',
'CjkUnifiedIdeograph9842': '\xe9\xa1\x82',
'GreekSmallLetterUpsilon': '\xcf\x85',
'CjkUnifiedIdeograph9849': '\xe9\xa1\x89',
'CjkUnifiedIdeograph9848': '\xe9\xa1\x88',
'DevanagariVowelSignVocalicRr': '\xe0\xa5\x84',
'HangulSyllableMeonj': '\xeb\xa8\xbd',
'CjkUnifiedIdeograph76ef': '\xe7\x9b\xaf',
'CjkUnifiedIdeograph76ed': '\xe7\x9b\xad',
'CjkUnifiedIdeograph76ee': '\xe7\x9b\xae',
'CjkUnifiedIdeograph76eb': '\xe7\x9b\xab',
'CjkUnifiedIdeograph76ec': '\xe7\x9b\xac',
'CjkUnifiedIdeograph76ea': '\xe7\x9b\xaa',
'YiSyllableHa': '\xea\x89\xb3',
'CjkUnifiedIdeograph5355': '\xe5\x8d\x95',
'CjkUnifiedIdeograph5354': '\xe5\x8d\x94',
'CjkUnifiedIdeograph5357': '\xe5\x8d\x97',
'CjkUnifiedIdeograph5356': '\xe5\x8d\x96',
'CjkUnifiedIdeograph5351': '\xe5\x8d\x91',
'CjkUnifiedIdeograph5350': '\xe5\x8d\x90',
'CjkUnifiedIdeograph5353': '\xe5\x8d\x93',
'CjkUnifiedIdeograph5352': '\xe5\x8d\x92',
'CjkUnifiedIdeograph568c': '\xe5\x9a\x8c',
'CjkUnifiedIdeograph568b': '\xe5\x9a\x8b',
'CjkUnifiedIdeograph568a': '\xe5\x9a\x8a',
'UnicodeCharacter0xf0e6': '\xef\x83\xa6',
'CjkUnifiedIdeograph5359': '\xe5\x8d\x99',
'CjkUnifiedIdeograph5358': '\xe5\x8d\x98',
'CjkUnifiedIdeograph568e': '\xe5\x9a\x8e',
'CjkUnifiedIdeograph568d': '\xe5\x9a\x8d',
'ArabicSmallHighDotlessHeadOfKhah': '\xdb\xa1',
'LeftpointingCurvedAngleBracket': '\xe2\xa7\xbc',
'HangulSyllableJwas': '\xec\xa2\x9f',
'HangulSyllableJoelb': '\xec\xa3\x8f',
'HangulSyllableJwap': '\xec\xa2\xa6',
'HangulSyllableJoelm': '\xec\xa3\x8e',
'LatinSmallLetterAWithHookAbove': '\xe1\xba\xa3',
'HangulSyllableJoelh': '\xec\xa3\x93',
'HangulSyllableJwag': '\xec\xa2\x8d',
'CjkUnifiedIdeograph695a': '\xe6\xa5\x9a',
'HangulSyllableJwae': '\xec\xa2\xa8',
'HangulSyllableJwad': '\xec\xa2\x93',
'HangulSyllableJwac': '\xec\xa2\xa3',
'HangulSyllableJwab': '\xec\xa2\x9d',
'HangulSyllableJoels': '\xec\xa3\x90',
'HangulSyllableJwan': '\xec\xa2\x90',
'HangulSyllableJwam': '\xec\xa2\x9c',
'HangulSyllableJwal': '\xec\xa2\x94',
'HangulSyllableJwak': '\xec\xa2\xa4',
'CyrillicCapitalLetterCheWithDiaeresis': '\xd3\xb4',
'HangulSyllableJwah': '\xec\xa2\xa7',
'HangulSyllableMyid': '\xeb\xaf\xa3',
'HangulSyllableDyolb': '\xeb\x90\xbf',
'HangulSyllableBwess': '\xeb\xb7\x8c',
'HangulSyllableMyib': '\xeb\xaf\xad',
'HangulSyllableMyic': '\xeb\xaf\xb3',
'HangulSyllableMyil': '\xeb\xaf\xa4',
'HangulSyllableMyim': '\xeb\xaf\xac',
'HangulSyllableMyin': '\xeb\xaf\xa0',
'HangulSyllableMyih': '\xeb\xaf\xb7',
'HangulSyllableMyij': '\xeb\xaf\xb2',
'UnicodeCharacter0xabce': '\xea\xaf\x8e',
'HangulSyllableMyit': '\xeb\xaf\xb5',
'LatinSmallLetterGWithDotAbove': '\xc4\xa1',
'HangulSyllableMyip': '\xeb\xaf\xb6',
'UnicodeCharacter0xead8': '\xee\xab\x98',
'HangulSyllableMyis': '\xeb\xaf\xaf',
'GreekCapitalLetterOmicron': '\xce\x9f',
'HangulSyllableRyult': '\xeb\xa5\xa5',
'YiSyllableDie': '\xea\x84\x85',
'HangulSyllableRyulp': '\xeb\xa5\xa6',
'CjkUnifiedIdeograph85ca': '\xe8\x97\x8a',
'HangulSyllableCoess': '\xec\xb5\xb0',
'YiSyllableDip': '\xea\x84\x83',
'HangulSyllableRyulg': '\xeb\xa5\xa1',
'CjkUnifiedIdeograph57bc': '\xe5\x9e\xbc',
'YiSyllableDit': '\xea\x84\x80',
'CircledTimes': '\xe2\x8a\x97',
'YiSyllableDix': '\xea\x84\x81',
'HangulSyllableRyulm': '\xeb\xa5\xa2',
'VaiSyllablePoo': '\xea\x95\xb6',
'HangulSyllableRyulh': '\xeb\xa5\xa7',
'UnicodeCharacter0xe96c': '\xee\xa5\xac',
'SquareYaado': '\xe3\x8d\x8e',
'ArabicSignAlayheAssallam': '\xd8\x91',
'HangulSyllableHaegs': '\xed\x95\xb7',
'HangulSyllableKaeb': '\xec\xba\xa1',
'CjkUnifiedIdeograph4978': '\xe4\xa5\xb8',
'CjkUnifiedIdeograph4979': '\xe4\xa5\xb9',
'CjkUnifiedIdeograph4976': '\xe4\xa5\xb6',
'CjkUnifiedIdeograph4977': '\xe4\xa5\xb7',
'CjkUnifiedIdeograph4974': '\xe4\xa5\xb4',
'CjkUnifiedIdeograph4975': '\xe4\xa5\xb5',
'CjkUnifiedIdeograph4972': '\xe4\xa5\xb2',
'CjkUnifiedIdeograph4973': '\xe4\xa5\xb3',
'CjkUnifiedIdeograph4970': '\xe4\xa5\xb0',
'BuhidLetterSa': '\xe1\x9d\x90',
'CjkUnifiedIdeograph744c': '\xe7\x91\x8c',
'CjkUnifiedIdeograph744b': '\xe7\x91\x8b',
'CjkUnifiedIdeograph744a': '\xe7\x91\x8a',
'CjkUnifiedIdeograph744f': '\xe7\x91\x8f',
'CjkUnifiedIdeograph744e': '\xe7\x91\x8e',
'CjkUnifiedIdeograph744d': '\xe7\x91\x8d',
'HangulSyllableKaec': '\xec\xba\xa7',
'CjkRadicalCity': '\xe2\xbb\x8f',
'HangulSyllableHyaebs': '\xed\x96\xbe',
'CjkCompatibilityIdeographf9cf': '\xef\xa7\x8f',
'CjkUnifiedIdeograph8a1d': '\xe8\xa8\x9d',
'CjkUnifiedIdeograph8a1e': '\xe8\xa8\x9e',
'LaoLetterBo': '\xe0\xba\x9a',
'CjkUnifiedIdeograph8a1a': '\xe8\xa8\x9a',
'CjkUnifiedIdeograph8a1b': '\xe8\xa8\x9b',
'CjkUnifiedIdeograph8a1c': '\xe8\xa8\x9c',
'CjkUnifiedIdeograph3d2e': '\xe3\xb4\xae',
'CjkUnifiedIdeograph3d2d': '\xe3\xb4\xad',
'CjkUnifiedIdeograph3d2f': '\xe3\xb4\xaf',
'CjkUnifiedIdeograph3d2a': '\xe3\xb4\xaa',
'CjkUnifiedIdeograph3d2c': '\xe3\xb4\xac',
'CjkUnifiedIdeograph3d2b': '\xe3\xb4\xab',
'CjkUnifiedIdeograph7443': '\xe7\x91\x83',
'MalayalamLetterChilluRr': '\xe0\xb5\xbc',
'CjkUnifiedIdeograph7441': '\xe7\x91\x81',
'CjkUnifiedIdeograph7440': '\xe7\x91\x80',
'CjkUnifiedIdeograph7447': '\xe7\x91\x87',
'CjkUnifiedIdeograph7446': '\xe7\x91\x86',
'CjkUnifiedIdeograph7445': '\xe7\x91\x85',
'CjkUnifiedIdeograph7444': '\xe7\x91\x84',
'HangulSyllableSswad': '\xec\x8f\xbb',
'HangulSyllableSswae': '\xec\x90\x90',
'CjkUnifiedIdeograph7449': '\xe7\x91\x89',
'CjkUnifiedIdeograph7448': '\xe7\x91\x88',
'HangulSyllableBbinh': '\xec\x82\x96',
'HangulSyllableSswab': '\xec\x90\x85',
'HangulSyllableBbinj': '\xec\x82\x95',
'BopomofoLetterCh': '\xe3\x84\x94',
'CjkUnifiedIdeograph497f': '\xe4\xa5\xbf',
'CjkUnifiedIdeograph497d': '\xe4\xa5\xbd',
'CjkUnifiedIdeograph497e': '\xe4\xa5\xbe',
'CjkUnifiedIdeograph497b': '\xe4\xa5\xbb',
'CjkUnifiedIdeograph497c': '\xe4\xa5\xbc',
'CjkUnifiedIdeograph497a': '\xe4\xa5\xba',
'CjkUnifiedIdeograph8a14': '\xe8\xa8\x94',
'CjkUnifiedIdeograph8a15': '\xe8\xa8\x95',
'CjkUnifiedIdeograph8a16': '\xe8\xa8\x96',
'CautionSign': '\xe2\x98\xa1',
'CjkUnifiedIdeograph3d29': '\xe3\xb4\xa9',
'CjkUnifiedIdeograph3d28': '\xe3\xb4\xa8',
'CjkUnifiedIdeograph8a12': '\xe8\xa8\x92',
'HangzhouNumeralFive': '\xe3\x80\xa5',
'CjkUnifiedIdeograph3d25': '\xe3\xb4\xa5',
'CjkUnifiedIdeograph3d24': '\xe3\xb4\xa4',
'CjkUnifiedIdeograph3d27': '\xe3\xb4\xa7',
'CjkUnifiedIdeograph3d26': '\xe3\xb4\xa6',
'CjkUnifiedIdeograph3d21': '\xe3\xb4\xa1',
'CjkUnifiedIdeograph3d20': '\xe3\xb4\xa0',
'CjkUnifiedIdeograph3d23': '\xe3\xb4\xa3',
'CjkUnifiedIdeograph3d22': '\xe3\xb4\xa2',
'KhmerSymbolDapKoet': '\xe1\xa7\xaa',
'UnicodeCharacter0xf1d7': '\xef\x87\x97',
'FlowerPunctuationMark': '\xe2\x81\x95',
'CjkUnifiedIdeograph7436': '\xe7\x90\xb6',
'CjkUnifiedIdeograph96ba': '\xe9\x9a\xba',
'CjkUnifiedIdeograph96bc': '\xe9\x9a\xbc',
'CjkUnifiedIdeograph96bb': '\xe9\x9a\xbb',
'GreaterthanOverlappingLessthan': '\xe2\xaa\xa4',
'TeluguLetterSha': '\xe0\xb0\xb6',
'CjkUnifiedIdeograph96bf': '\xe9\x9a\xbf',
'CjkUnifiedIdeograph84f4': '\xe8\x93\xb4',
'CjkUnifiedIdeograph99a8': '\xe9\xa6\xa8',
'UnicodeCharacter0xf1d6': '\xef\x87\x96',
'CjkUnifiedIdeograph84f6': '\xe8\x93\xb6',
'CjkUnifiedIdeograph7433': '\xe7\x90\xb3',
'CjkUnifiedIdeograph84f0': '\xe8\x93\xb0',
'CjkUnifiedIdeograph7431': '\xe7\x90\xb1',
'CjkCompatibilityIdeographf9e8': '\xef\xa7\xa8',
'CjkUnifiedIdeograph84f1': '\xe8\x93\xb1',
'RightArrowWithCircledPlus': '\xe2\x9f\xb4',
'CjkCompatibilityIdeographf9e9': '\xef\xa7\xa9',
'UnicodeCharacter0xec26': '\xee\xb0\xa6',
'UnicodeCharacter0xe720': '\xee\x9c\xa0',
'CjkUnifiedIdeograph84f3': '\xe8\x93\xb3',
'CjkUnifiedIdeograph99a1': '\xe9\xa6\xa1',
'CjkCompatibilityIdeographf9cc': '\xef\xa7\x8c',
'CjkUnifiedIdeograph99a0': '\xe9\xa6\xa0',
'UnicodeCharacter0xec28': '\xee\xb0\xa8',
'NkoCombiningLongRisingTone': '\xdf\xb1',
'CjkUnifiedIdeograph99a7': '\xe9\xa6\xa7',
'UnicodeCharacter0xec29': '\xee\xb0\xa9',
'HangulSyllableKaem': '\xec\xba\xa0',
'CjkUnifiedIdeograph99a6': '\xe9\xa6\xa6',
'CjkUnifiedIdeograph96b9': '\xe9\x9a\xb9',
'CjkUnifiedIdeograph96b8': '\xe9\x9a\xb8',
'UnicodeCharacter0xf1d4': '\xef\x87\x94',
'CjkUnifiedIdeograph7438': '\xe7\x90\xb8',
'UnicodeCharacter0xec27': '\xee\xb0\xa7',
'CjkUnifiedIdeograph96b1': '\xe9\x9a\xb1',
'CjkUnifiedIdeograph96b0': '\xe9\x9a\xb0',
'CjkUnifiedIdeograph96b3': '\xe9\x9a\xb3',
'CjkUnifiedIdeograph96b2': '\xe9\x9a\xb2',
'CjkUnifiedIdeograph96b5': '\xe9\x9a\xb5',
'SyriacDottedZlamaAngular': '\xdc\xb9',
'CjkUnifiedIdeograph96b7': '\xe9\x9a\xb7',
'CjkUnifiedIdeograph96b6': '\xe9\x9a\xb6',
'HangulSyllableHyelh': '\xed\x98\xab',
'UnicodeCharacter0xe73a': '\xee\x9c\xba',
'UnicodeCharacter0xec25': '\xee\xb0\xa5',
'HangulSyllableNeh': '\xeb\x84\xbf',
'CjkUnifiedIdeograph8ae9': '\xe8\xab\xa9',
'UnicodeCharacter0xec22': '\xee\xb0\xa2',
'HangulSyllableHyelb': '\xed\x98\xa7',
'UnicodeCharacter0xe73c': '\xee\x9c\xbc',
'BlackLeftPointingIndex': '\xe2\x98\x9a',
'HangulSyllableHyelg': '\xed\x98\xa5',
'WhiteHorizontalEllipse': '\xe2\xac\xad',
'UnicodeCharacter0xe726': '\xee\x9c\xa6',
'UnicodeCharacter0xe73b': '\xee\x9c\xbb',
'UnicodeCharacter0xec20': '\xee\xb0\xa0',
'HangulSyllableHyelp': '\xed\x98\xaa',
'UnicodeCharacter0xe73e': '\xee\x9c\xbe',
'HangulSyllableHyels': '\xed\x98\xa8',
'HangulSyllableHyelt': '\xed\x98\xa9',
'UnicodeCharacter0xe962': '\xee\xa5\xa2',
'CjkUnifiedIdeograph9d50': '\xe9\xb5\x90',
'UnicodeCharacter0xe73d': '\xee\x9c\xbd',
'HangulSyllableSswaeng': '\xec\x90\xa5',
'UnicodeCharacter0xe963': '\xee\xa5\xa3',
'UnicodeCharacter0xf1cd': '\xef\x87\x8d',
'NewTaiLueLetterLowXva': '\xe1\xa6\xa9',
'LeftwardsTripleDashArrow': '\xe2\xa4\x8e',
'UnicodeCharacter0xe965': '\xee\xa5\xa5',
'UnicodeCharacter0xe727': '\xee\x9c\xa7',
'CjkUnifiedIdeograph8ae0': '\xe8\xab\xa0',
'UnicodeCharacter0xe966': '\xee\xa5\xa6',
'CjkUnifiedIdeograph8ae1': '\xe8\xab\xa1',
'CjkUnifiedIdeograph8ae3': '\xe8\xab\xa3',
'LogicalAndWithHorizontalDash': '\xe2\xa9\x9c',
'UnicodeCharacter0xf1d1': '\xef\x87\x91',
'CjkUnifiedIdeograph8ae5': '\xe8\xab\xa5',
'HangulSyllableNuls': '\xeb\x88\x90',
'CjkUnifiedIdeograph8ae6': '\xe8\xab\xa6',
'HangulSyllableMiss': '\xeb\xb0\x8c',
'CjkUnifiedIdeograph8ae7': '\xe8\xab\xa7',
'UnicodeCharacter0xea54': '\xee\xa9\x94',
'CjkUnifiedIdeograph8a07': '\xe8\xa8\x87',
'SmallLeftCurlyBracket': '\xef\xb9\x9b',
'CjkUnifiedIdeograph6bfc': '\xe6\xaf\xbc',
'CjkUnifiedIdeograph8a06': '\xe8\xa8\x86',
'UnicodeCharacter0xf1d0': '\xef\x87\x90',
'CjkUnifiedIdeograph8a05': '\xe8\xa8\x85',
'HangulSyllableMyelg': '\xeb\xaa\x95',
'CjkUnifiedIdeograph8a04': '\xe8\xa8\x84',
'CjkUnifiedIdeograph8a03': '\xe8\xa8\x83',
'UnicodeCharacter0xf2b6': '\xef\x8a\xb6',
'CjkUnifiedIdeograph8a02': '\xe8\xa8\x82',
'UnicodeCharacter0xe267': '\xee\x89\xa7',
'CjkUnifiedIdeograph8a01': '\xe8\xa8\x81',
'DevanagariVowelSignVocalicLl': '\xe0\xa5\xa3',
'CjkUnifiedIdeograph8a00': '\xe8\xa8\x80',
'ArabicLigatureAinWithAlefMaksuraIsolatedForm': '\xef\xb3\xb7',
'DoubleDagger': '\xe2\x80\xa1',
'CjkUnifiedIdeograph9f58': '\xe9\xbd\x98',
'UnicodeCharacter0xf089': '\xef\x82\x89',
'HalfwidthWhiteCircle': '\xef\xbf\xae',
'WhiteConcavesidedDiamondWithLeftwardsTick': '\xe2\x9f\xa2',
'YiSyllableViep': '\xea\x83\xa9',
'UnicodeCharacter0xdae3': '\xed\xab\xa3',
'UnicodeCharacter0xf2b7': '\xef\x8a\xb7',
'UnicodeCharacter0xe262': '\xee\x89\xa2',
'HangulSyllableGgyup': '\xeb\x81\x82',
'Union': '\xe2\x88\xaa',
'UnicodeCharacter0xf082': '\xef\x82\x82',
'HangulSyllablePoej': '\xed\x91\x96',
'CjkUnifiedIdeograph90cc': '\xe9\x83\x8c',
'UnicodeCharacter0xf763': '\xef\x9d\xa3',
'CjkUnifiedIdeograph8a08': '\xe8\xa8\x88',
'UnicodeCharacter0xf080': '\xef\x82\x80',
'HangulSyllableDyols': '\xeb\x91\x80',
'UnicodeCharacter0xf081': '\xef\x82\x81',
'VaiSyllableNju': '\xea\x96\xb2',
'UnicodeCharacter0xeb63': '\xee\xad\xa3',
'UnicodeCharacter0xf086': '\xef\x82\x86',
'HangulSyllableGoegg': '\xea\xb4\xb6',
'UnicodeCharacter0xf087': '\xef\x82\x87',
'UnicodeCharacter0x9fff': '\xe9\xbf\xbf',
'UnicodeCharacter0x9ffe': '\xe9\xbf\xbe',
'UnicodeCharacter0x9ffd': '\xe9\xbf\xbd',
'UnicodeCharacter0x9ffc': '\xe9\xbf\xbc',
'UnicodeCharacter0x9ffb': '\xe9\xbf\xbb',
'UnicodeCharacter0x9ffa': '\xe9\xbf\xba',
'UnicodeCharacter0xf2b5': '\xef\x8a\xb5',
'UnicodeCharacter0xf085': '\xef\x82\x85',
'HangulSyllableGoegs': '\xea\xb4\xb7',
'AnticlockwiseIntegration': '\xe2\xa8\x91',
'HangulSyllableMeug': '\xeb\xaf\x81',
'UniversalRecyclingSymbol': '\xe2\x99\xb2',
'UnicodeCharacter0xeba0': '\xee\xae\xa0',
'X': '\xce\x9e',
'ThaiCharacterPoPla': '\xe0\xb8\x9b',
'DingbatNegativeCircledDigitFour': '\xe2\x9d\xb9',
'UnicodeCharacter0x9ff6': '\xe9\xbf\xb6',
'UnicodeCharacter0x9ff5': '\xe9\xbf\xb5',
'UnicodeCharacter0x9ff4': '\xe9\xbf\xb4',
'UnicodeCharacter0x9ff3': '\xe9\xbf\xb3',
'UnicodeCharacter0x9ff2': '\xe9\xbf\xb2',
'UnicodeCharacter0x9ff1': '\xe9\xbf\xb1',
'UnicodeCharacter0x9ff0': '\xe9\xbf\xb0',
'YiSyllableNyi': '\xea\x91\x8c',
'UnicodeCharacter0x9ff9': '\xe9\xbf\xb9',
'UnicodeCharacter0x9ff8': '\xe9\xbf\xb8',
'UnicodeCharacter0xec2f': '\xee\xb0\xaf',
'coproduct': '\xe2\x88\x90',
'UnicodeCharacter0xeba1': '\xee\xae\xa1',
'KangxiRadicalTwo': '\xe2\xbc\x86',
'CjkCompatibilityIdeographf9ec': '\xef\xa7\xac',
'CjkUnifiedIdeograph8eab': '\xe8\xba\xab',
'UnicodeCharacter0xec2d': '\xee\xb0\xad',
'CjkUnifiedIdeograph824a': '\xe8\x89\x8a',
'CjkUnifiedIdeograph824b': '\xe8\x89\x8b',
'CjkUnifiedIdeograph824c': '\xe8\x89\x8c',
'CjkUnifiedIdeograph824d': '\xe8\x89\x8d',
'CjkUnifiedIdeograph824e': '\xe8\x89\x8e',
'CjkUnifiedIdeograph824f': '\xe8\x89\x8f',
'UnicodeCharacter0xe730': '\xee\x9c\xb0',
'CjkCompatibilityIdeographf9ef': '\xef\xa7\xaf',
'CjkUnifiedIdeograph7fc2': '\xe7\xbf\x82',
'UnicodeCharacter0xec2c': '\xee\xb0\xac',
'UnicodeCharacter0xeba6': '\xee\xae\xa6',
'CjkUnifiedIdeograph6bc9': '\xe6\xaf\x89',
'CjkUnifiedIdeograph6bc8': '\xe6\xaf\x88',
'CjkCompatibilityIdeographf9ed': '\xef\xa7\xad',
'SinhalaLetterIiyanna': '\xe0\xb6\x8a',
'CjkUnifiedIdeograph6bc3': '\xe6\xaf\x83',
'CjkUnifiedIdeograph6bc2': '\xe6\xaf\x82',
'CjkUnifiedIdeograph6bc1': '\xe6\xaf\x81',
'CjkUnifiedIdeograph6bc0': '\xe6\xaf\x80',
'CjkUnifiedIdeograph6bc7': '\xe6\xaf\x87',
'CjkUnifiedIdeograph6bc6': '\xe6\xaf\x86',
'CjkUnifiedIdeograph6bc5': '\xe6\xaf\x85',
'CjkUnifiedIdeograph6bc4': '\xe6\xaf\x84',
'HangulSyllableSoej': '\xec\x87\xb6',
'ThaiDigitFive': '\xe0\xb9\x95',
'ChamDigitOne': '\xea\xa9\x91',
'UnicodeCharacter0xe737': '\xee\x9c\xb7',
'LatinCapitalLetterEWithCaron': '\xc4\x9a',
'LatinCapitalLetterQWithDiagonalStroke': '\xea\x9d\x98',
'UnicodeCharacter0xe736': '\xee\x9c\xb6',
'UnicodeCharacter0xf2b1': '\xef\x8a\xb1',
'UnicodeCharacter0xe739': '\xee\x9c\xb9',
'GurmukhiDigitZero': '\xe0\xa9\xa6',
'CjkUnifiedIdeograph8aea': '\xe8\xab\xaa',
'UnicodeCharacter0xf1c8': '\xef\x87\x88',
'UnicodeCharacter0xf1c9': '\xef\x87\x89',
'HangulSyllableJyass': '\xec\x9f\x9c',
'CjkUnifiedIdeograph6bcc': '\xe6\xaf\x8c',
'CjkUnifiedIdeograph6bcb': '\xe6\xaf\x8b',
'CjkUnifiedIdeograph6bca': '\xe6\xaf\x8a',
'ArabicLigatureJeemWithHahWithYehFinalForm': '\xef\xb6\xbe',
'CjkUnifiedIdeograph6bcf': '\xe6\xaf\x8f',
'CjkUnifiedIdeograph6bce': '\xe6\xaf\x8e',
'CjkUnifiedIdeograph6bcd': '\xe6\xaf\x8d',
'CjkUnifiedIdeograph8240': '\xe8\x89\x80',
'CjkUnifiedIdeograph8241': '\xe8\x89\x81',
'CjkUnifiedIdeograph8242': '\xe8\x89\x82',
'CjkUnifiedIdeograph8243': '\xe8\x89\x83',
'CjkUnifiedIdeograph8244': '\xe8\x89\x84',
'CjkUnifiedIdeograph8245': '\xe8\x89\x85',
'CjkUnifiedIdeograph8246': '\xe8\x89\x86',
'CjkUnifiedIdeograph8247': '\xe8\x89\x87',
'CjkUnifiedIdeograph8248': '\xe8\x89\x88',
'CjkUnifiedIdeograph8249': '\xe8\x89\x89',
'HangulSyllableMying': '\xeb\xaf\xb1',
'CjkUnifiedIdeograph7b98': '\xe7\xae\x98',
'UnicodeCharacter0xf0b8': '\xef\x82\xb8',
'LatinCapitalLetterLWithSmallLetterJ': '\xc7\x88',
'ArabicLetterHahWithExtendedArabicindicDigitFourBelow': '\xdd\xbc',
'CjkUnifiedIdeograph7b99': '\xe7\xae\x99',
'CjkUnifiedIdeograph7b96': '\xe7\xae\x96',
'HangulSyllablePult': '\xed\x92\x85',
'CjkUnifiedIdeograph7b97': '\xe7\xae\x97',
'GreekCapitalLetterOmegaWithDasiaAndPerispomeni': '\xe1\xbd\xaf',
'ArabicLetterFarsiYehMedialForm': '\xef\xaf\xbf',
'UnicodeCharacter0xe400': '\xee\x90\x80',
'HangulSyllableBeong': '\xeb\xb2\x99',
'HangulSyllableJoess': '\xec\xa3\x98',
'HangulSyllableBeonj': '\xeb\xb2\x89',
'CyrillicCapitalLetterEnWithTail': '\xd3\x89',
'RecyclingSymbolForType3Plastics': '\xe2\x99\xb5',
'ArabicLigatureBehWithAlefMaksuraIsolatedForm': '\xef\xb0\x89',
'BraillePatternDots1368': '\xe2\xa2\xa5',
'BraillePatternDots1367': '\xe2\xa1\xa5',
'CjkCompatibilityIdeographfa09': '\xef\xa8\x89',
'diff': '\xe2\x88\x82',
'CjkUnifiedIdeograph99aa': '\xe9\xa6\xaa',
'UnicodeCharacter0xf3c4': '\xef\x8f\x84',
'UnicodeCharacter0xf3c5': '\xef\x8f\x85',
'UnicodeCharacter0xf3c6': '\xef\x8f\x86',
'UnicodeCharacter0xf3c7': '\xef\x8f\x87',
'UnicodeCharacter0xf3c0': '\xef\x8f\x80',
'UnicodeCharacter0xf3c1': '\xef\x8f\x81',
'UnicodeCharacter0xf3c2': '\xef\x8f\x82',
'UnicodeCharacter0xf3c3': '\xef\x8f\x83',
'CjkUnifiedIdeograph4bff': '\xe4\xaf\xbf',
'LeftwardsArrowWithHook': '\xe2\x86\xa9',
'CjkUnifiedIdeograph4bfd': '\xe4\xaf\xbd',
'CjkUnifiedIdeograph4bfe': '\xe4\xaf\xbe',
'CjkUnifiedIdeograph4bfb': '\xe4\xaf\xbb',
'CjkUnifiedIdeograph4bfc': '\xe4\xaf\xbc',
'CjkUnifiedIdeograph4bfa': '\xe4\xaf\xba',
'CjkUnifiedIdeograph99ad': '\xe9\xa6\xad',
'HangulSyllableRwaeng': '\xeb\xa2\xa9',
'GreekSmallLetterIotaWithPerispomeni': '\xe1\xbf\x96',
'CjkUnifiedIdeograph84c2': '\xe8\x93\x82',
'CyrillicSmallLetterCheWithVerticalStroke': '\xd2\xb9',
'HangulSyllableMwigg': '\xeb\xae\x8a',
'LatinCapitalLetterRRotunda': '\xea\x9d\x9a',
'BengaliLetterUu': '\xe0\xa6\x8a',
'YiSyllableZzat': '\xea\x8b\x98',
'UnicodeCharacter0xef2b': '\xee\xbc\xab',
'YiSyllableZzap': '\xea\x8b\x9b',
'HangulSyllableMyals': '\xeb\xa8\x8c',
'TibetanDigitZero': '\xe0\xbc\xa0',
'UnicodeCharacter0xf3ce': '\xef\x8f\x8e',
'UnicodeCharacter0xf3cf': '\xef\x8f\x8f',
'UnicodeCharacter0xf3ca': '\xef\x8f\x8a',
'CjkUnifiedIdeograph4bf8': '\xe4\xaf\xb8',
'CjkUnifiedIdeograph4bf9': '\xe4\xaf\xb9',
'CjkUnifiedIdeograph4bf6': '\xe4\xaf\xb6',
'CjkUnifiedIdeograph4bf7': '\xe4\xaf\xb7',
'CjkUnifiedIdeograph4bf4': '\xe4\xaf\xb4',
'CjkUnifiedIdeograph4bf5': '\xe4\xaf\xb5',
'CjkUnifiedIdeograph4bf2': '\xe4\xaf\xb2',
'CjkUnifiedIdeograph4bf3': '\xe4\xaf\xb3',
'CjkUnifiedIdeograph4bf0': '\xe4\xaf\xb0',
'CjkUnifiedIdeograph4bf1': '\xe4\xaf\xb1',
'CjkUnifiedIdeograph5dd5': '\xe5\xb7\x95',
'CjkUnifiedIdeograph5dd4': '\xe5\xb7\x94',
'CjkUnifiedIdeograph5dd7': '\xe5\xb7\x97',
'CjkUnifiedIdeograph5dd6': '\xe5\xb7\x96',
'CjkUnifiedIdeograph5dd1': '\xe5\xb7\x91',
'CjkUnifiedIdeograph5dd0': '\xe5\xb7\x90',
'UnicodeCharacter0x1a88': '\xe1\xaa\x88',
'UnicodeCharacter0x1a89': '\xe1\xaa\x89',
'UnicodeCharacter0x1a86': '\xe1\xaa\x86',
'UnicodeCharacter0x1a87': '\xe1\xaa\x87',
'UnicodeCharacter0xbad': '\xe0\xae\xad',
'UnicodeCharacter0x1a85': '\xe1\xaa\x85',
'UnicodeCharacter0xbab': '\xe0\xae\xab',
'UnicodeCharacter0xbac': '\xe0\xae\xac',
'UnicodeCharacter0x1a80': '\xe1\xaa\x80',
'UnicodeCharacter0x1a81': '\xe1\xaa\x81',
'HangulSyllableBeh': '\xeb\xb2\xbb',
'HangulSyllableBek': '\xeb\xb2\xb8',
'HangulSyllableBej': '\xeb\xb2\xb6',
'CjkUnifiedIdeograph92ce': '\xe9\x8b\x8e',
'KhmerSymbolPrammuoyRoc': '\xe1\xa7\xb6',
'HangulSyllableBeo': '\xeb\xb2\x84',
'HangulSyllableBen': '\xeb\xb2\xa4',
'HangulSyllableBec': '\xeb\xb2\xb7',
'HangulSyllableBeb': '\xeb\xb2\xb1',
'HangulSyllableBed': '\xeb\xb2\xa7',
'HangulSyllableBeg': '\xeb\xb2\xa1',
'CjkUnifiedIdeograph8a0f': '\xe8\xa8\x8f',
'LatinSmallLetterOWithHookAbove': '\xe1\xbb\x8f',
'CjkUnifiedIdeograph3d1b': '\xe3\xb4\x9b',
'UnicodeCharacter0xe9d7': '\xee\xa7\x97',
'CjkUnifiedIdeograph8a0d': '\xe8\xa8\x8d',
'LatinSmallLetterHWithDiaeresis': '\xe1\xb8\xa7',
'CjkUnifiedIdeograph6a9f': '\xe6\xaa\x9f',
'CjkUnifiedIdeograph8a0c': '\xe8\xa8\x8c',
'HangulJongseongRieulnieun': '\xe1\x87\x8d',
'CjkUnifiedIdeograph6a9e': '\xe6\xaa\x9e',
'CjkUnifiedIdeograph6a9b': '\xe6\xaa\x9b',
'CjkUnifiedIdeograph6a9c': '\xe6\xaa\x9c',
'CjkUnifiedIdeograph6a9a': '\xe6\xaa\x9a',
'HangulSyllableSswiss': '\xec\x93\x8c',
'CjkUnifiedIdeograph8a0a': '\xe8\xa8\x8a',
'HangulSyllableJyup': '\xec\xa6\x86',
'CjkUnifiedIdeograph5dde': '\xe5\xb7\x9e',
'CjkUnifiedIdeograph5ddd': '\xe5\xb7\x9d',
'CjkUnifiedIdeograph5ddf': '\xe5\xb7\x9f',
'CjkUnifiedIdeograph5dda': '\xe5\xb7\x9a',
'CjkUnifiedIdeograph5ddc': '\xe5\xb7\x9c',
'CjkUnifiedIdeograph5ddb': '\xe5\xb7\x9b',
'UnicodeCharacter0xba6': '\xe0\xae\xa6',
'UnicodeCharacter0xba7': '\xe0\xae\xa7',
'UnicodeCharacter0x1a8d': '\xe1\xaa\x8d',
'UnicodeCharacter0xba5': '\xe0\xae\xa5',
'UnicodeCharacter0xba2': '\xe0\xae\xa2',
'UnicodeCharacter0x1a8c': '\xe1\xaa\x8c',
'UnicodeCharacter0xba0': '\xe0\xae\xa0',
'UnicodeCharacter0xba1': '\xe0\xae\xa1',
'CjkUnifiedIdeograph6a98': '\xe6\xaa\x98',
'CjkUnifiedIdeograph6a99': '\xe6\xaa\x99',
'CjkUnifiedIdeograph6a96': '\xe6\xaa\x96',
'CjkUnifiedIdeograph6a97': '\xe6\xaa\x97',
'CjkUnifiedIdeograph6a94': '\xe6\xaa\x94',
'CjkUnifiedIdeograph6a95': '\xe6\xaa\x95',
'CjkUnifiedIdeograph6a92': '\xe6\xaa\x92',
'CjkUnifiedIdeograph6a93': '\xe6\xaa\x93',
'CjkUnifiedIdeograph6a90': '\xe6\xaa\x90',
'CjkUnifiedIdeograph6a91': '\xe6\xaa\x91',
'UnicodeCharacter0xe408': '\xee\x90\x88',
'HangulSyllableDwe': '\xeb\x92\x88',
'HangulSyllableMyeom': '\xeb\xaa\x80',
'HangulSyllableMyeon': '\xeb\xa9\xb4',
'HangulSyllableDwa': '\xeb\x8f\xa0',
'HangulSyllableMyeoj': '\xeb\xaa\x86',
'HangulSyllableMyeok': '\xeb\xaa\x88',
'HangulSyllableMyeod': '\xeb\xa9\xb7',
'CjkUnifiedIdeograph8aa8': '\xe8\xaa\xa8',
'HangulSyllableMyeog': '\xeb\xa9\xb1',
'HangulSyllableDwi': '\xeb\x92\xa4',
'HangulSyllableMyeob': '\xeb\xaa\x81',
'HangulSyllableMyeoc': '\xeb\xaa\x87',
'LatinSmallLetterLWithBelt': '\xc9\xac',
'SolidusWithOverbar': '\xe2\xa7\xb6',
'HangulSyllableMyeot': '\xeb\xaa\x89',
'BraillePatternDots268': '\xe2\xa2\xa2',
'HangulSyllableMyeos': '\xeb\xaa\x83',
'CjkUnifiedIdeograph6d48': '\xe6\xb5\x88',
'CjkUnifiedIdeograph6d49': '\xe6\xb5\x89',
'HangulSyllableDaeg': '\xeb\x8c\x81',
'UnicodeCharacter0xf2cc': '\xef\x8b\x8c',
'HangulSyllableDaed': '\xeb\x8c\x87',
'CjkUnifiedIdeograph6d40': '\xe6\xb5\x80',
'CjkUnifiedIdeograph6d41': '\xe6\xb5\x81',
'CjkUnifiedIdeograph6d42': '\xe6\xb5\x82',
'CjkUnifiedIdeograph6d43': '\xe6\xb5\x83',
'CjkUnifiedIdeograph6d44': '\xe6\xb5\x84',
'CjkUnifiedIdeograph6d45': '\xe6\xb5\x85',
'CjkUnifiedIdeograph6d46': '\xe6\xb5\x86',
'CjkUnifiedIdeograph6d47': '\xe6\xb5\x87',
'CjkUnifiedIdeograph7a15': '\xe7\xa8\x95',
'CjkUnifiedIdeograph7a14': '\xe7\xa8\x94',
'CjkUnifiedIdeograph7a17': '\xe7\xa8\x97',
'CjkUnifiedIdeograph7a16': '\xe7\xa8\x96',
'CjkUnifiedIdeograph7a11': '\xe7\xa8\x91',
'CjkUnifiedIdeograph7a10': '\xe7\xa8\x90',
'CjkUnifiedIdeograph7a13': '\xe7\xa8\x93',
'CjkUnifiedIdeograph7a12': '\xe7\xa8\x92',
'UnicodeCharacter0xd91f': '\xed\xa4\x9f',
'CjkUnifiedIdeograph829a': '\xe8\x8a\x9a',
'UnicodeCharacter0xd91d': '\xed\xa4\x9d',
'UnicodeCharacter0xd91e': '\xed\xa4\x9e',
'CjkUnifiedIdeograph7a19': '\xe7\xa8\x99',
'CjkUnifiedIdeograph7a18': '\xe7\xa8\x98',
'UnicodeCharacter0xd91a': '\xed\xa4\x9a',
'CjkUnifiedIdeograph6cc2': '\xe6\xb3\x82',
'HangulSyllableWeogg': '\xec\x9b\x8e',
'UnicodeCharacter0xf7bd': '\xef\x9e\xbd',
'UnicodeCharacter0xf55b': '\xef\x95\x9b',
'LatinCapitalLetterReversedCWithDot': '\xea\x9c\xbe',
'YiSyllableCo': '\xea\x8a\xbf',
'HangulSyllableJjyibs': '\xec\xb0\x82',
'UnicodeCharacter0xf55c': '\xef\x95\x9c',
'HangulSyllableJjyalg': '\xec\xa8\x9d',
'HangulSyllableMyaess': '\xeb\xa8\xb0',
'UnicodeCharacter0xaab7': '\xea\xaa\xb7',
'HangulSyllableWeogs': '\xec\x9b\x8f',
'CjkUnifiedIdeograph84c9': '\xe8\x93\x89',
'HangulSyllableSeubs': '\xec\x8a\xb6',
'CjkUnifiedIdeograph7a1e': '\xe7\xa8\x9e',
'CjkUnifiedIdeograph7a1d': '\xe7\xa8\x9d',
'CjkUnifiedIdeograph7a1f': '\xe7\xa8\x9f',
'CjkUnifiedIdeograph7a1a': '\xe7\xa8\x9a',
'CjkUnifiedIdeograph7a1c': '\xe7\xa8\x9c',
'CjkUnifiedIdeograph7a1b': '\xe7\xa8\x9b',
'UnicodeCharacter0xd916': '\xed\xa4\x96',
'HangulSyllableJyum': '\xec\xa5\xbc',
'UnicodeCharacter0xd914': '\xed\xa4\x94',
'HangulSyllableJjyalb': '\xec\xa8\x9f',
'UnicodeCharacter0xd912': '\xed\xa4\x92',
'HangulSyllableGgwaegs': '\xea\xbd\xa7',
'UnicodeCharacter0xd910': '\xed\xa4\x90',
'HangulSyllableRwen': '\xeb\xa4\xa4',
'DevanagariLetterCandraE': '\xe0\xa4\x8d',
'CjkUnifiedIdeograph7869': '\xe7\xa1\xa9',
'UnicodeCharacter0xdd3d': '\xed\xb4\xbd',
'DevanagariLetterCandraA': '\xe0\xa5\xb2',
'ArabicLigatureAlefMaksuraWithSuperscriptAlefFinalForm': '\xef\xb2\x90',
'DevanagariLetterCandraO': '\xe0\xa4\x91',
'CjkUnifiedIdeograph6d4b': '\xe6\xb5\x8b',
'CjkUnifiedIdeograph6d4c': '\xe6\xb5\x8c',
'CjkUnifiedIdeograph6d4d': '\xe6\xb5\x8d',
'CjkUnifiedIdeograph6d4e': '\xe6\xb5\x8e',
'CjkUnifiedIdeograph6d4f': '\xe6\xb5\x8f',
'CjkUnifiedIdeograph96a9': '\xe9\x9a\xa9',
'CjkUnifiedIdeograph5e08': '\xe5\xb8\x88',
'CjkUnifiedIdeograph5e09': '\xe5\xb8\x89',
'CjkUnifiedIdeograph95f0': '\xe9\x97\xb0',
'CjkUnifiedIdeograph95f1': '\xe9\x97\xb1',
'CjkUnifiedIdeograph95f6': '\xe9\x97\xb6',
'CjkUnifiedIdeograph95f7': '\xe9\x97\xb7',
'CjkUnifiedIdeograph95f4': '\xe9\x97\xb4',
'CjkUnifiedIdeograph95f5': '\xe9\x97\xb5',
'CjkUnifiedIdeograph5e00': '\xe5\xb8\x80',
'UnicodeCharacter0x1dea': '\xe1\xb7\xaa',
'UnicodeCharacter0x1deb': '\xe1\xb7\xab',
'UnicodeCharacter0x1dec': '\xe1\xb7\xac',
'UnicodeCharacter0x1ded': '\xe1\xb7\xad',
'UnicodeCharacter0x1dee': '\xe1\xb7\xae',
'UnicodeCharacter0x1def': '\xe1\xb7\xaf',
'CjkUnifiedIdeograph5e07': '\xe5\xb8\x87',
'CyrillicSmallLetterDoubleMonocularO': '\xea\x99\xad',
'HangulJungseongWe': '\xe1\x85\xb0',
'UnicodeCharacter0xe19d': '\xee\x86\x9d',
'RunicLetterStan': '\xe1\x9b\xa5',
'UnicodeCharacter0xfae6': '\xef\xab\xa6',
'GreekCapitalLetterAlphaWithPsiliAndPerispomeni': '\xe1\xbc\x8e',
'HangulSyllableJjyalt': '\xec\xa8\xa1',
'UnicodeCharacter0xe498': '\xee\x92\x98',
'CjkUnifiedIdeograph517a': '\xe5\x85\xba',
'UnicodeCharacter0xce5': '\xe0\xb3\xa5',
'UnicodeCharacter0xce4': '\xe0\xb3\xa4',
'CyrillicCapitalLetterZheWithDiaeresis': '\xd3\x9c',
'CjkUnifiedIdeograph96a1': '\xe9\x9a\xa1',
'HangulSyllableBbwac': '\xeb\xbd\xb3',
'HangulSyllableSsaegg': '\xec\x8c\x96',
'HangulSyllableSsabs': '\xec\x8c\x8a',
'LeftBarbDownRightBarbUpHarpoon': '\xe2\xa5\x8b',
'LatinSmallLetterUWithTilde': '\xc5\xa9',
'CjkUnifiedIdeograph6c6d': '\xe6\xb1\xad',
'HangulLetterRieulsios': '\xe3\x84\xbd',
'UnicodeCharacter0xe936': '\xee\xa4\xb6',
'CjkUnifiedIdeograph87c2': '\xe8\x9f\x82',
'CircledKatakanaHu': '\xe3\x8b\xab',
'UnicodeCharacter0x1de8': '\xe1\xb7\xa8',
'EthiopicSyllableMaa': '\xe1\x88\x9b',
'CircledKatakanaHi': '\xe3\x8b\xaa',
'CjkUnifiedIdeograph93ce': '\xe9\x8f\x8e',
'CircledKatakanaHo': '\xe3\x8b\xad',
'CjkUnifiedIdeograph95fd': '\xe9\x97\xbd',
'CjkUnifiedIdeograph95fe': '\xe9\x97\xbe',
'SquareKmCapital': '\xe3\x8f\x8e',
'LatinSmallLetterPWithStroke': '\xe1\xb5\xbd',
'CjkUnifiedIdeograph5e0c': '\xe5\xb8\x8c',
'CjkUnifiedIdeograph5e0d': '\xe5\xb8\x8d',
'CjkUnifiedIdeograph5e0e': '\xe5\xb8\x8e',
'CircledKatakanaHe': '\xe3\x8b\xac',
'UnicodeCharacter0x1de7': '\xe1\xb7\xa7',
'KannadaLetterDha': '\xe0\xb2\xa7',
'UnicodeCharacter0xe9d9': '\xee\xa7\x99',
'UnicodeCharacter0xdfef': '\xed\xbf\xaf',
'UnicodeCharacter0xdae4': '\xed\xab\xa4',
'FullwidthDigitZero': '\xef\xbc\x90',
'o': '\xce\xbf',
'CjkUnifiedIdeograph93cd': '\xe9\x8f\x8d',
'UnicodeCharacter0xdda9': '\xed\xb6\xa9',
'TwoAsterisksAlignedVertically': '\xe2\x81\x91',
'UnicodeCharacter0xf40f': '\xef\x90\x8f',
'UnicodeCharacter0xf40d': '\xef\x90\x8d',
'CjkUnifiedIdeograph7f6b': '\xe7\xbd\xab',
'UnicodeCharacter0xf40b': '\xef\x90\x8b',
'UnicodeCharacter0xf40c': '\xef\x90\x8c',
'UnicodeCharacter0xf40a': '\xef\x90\x8a',
'CjkUnifiedIdeograph4c36': '\xe4\xb0\xb6',
'UnicodeCharacter0xe66c': '\xee\x99\xac',
'UnicodeCharacter0xe66b': '\xee\x99\xab',
'UnicodeCharacter0xe66a': '\xee\x99\xaa',
'UnicodeCharacter0x1cb9': '\xe1\xb2\xb9',
'GreekCapitalLetterEtaWithDasiaAndPerispomeni': '\xe1\xbc\xaf',
'UnicodeCharacter0xe66f': '\xee\x99\xaf',
'DoubleSquareUnion': '\xe2\xa9\x8f',
'UnicodeCharacter0xe66d': '\xee\x99\xad',
'CjkUnifiedIdeograph3feb': '\xe3\xbf\xab',
'BlankSymbol': '\xe2\x90\xa2',
'CjkUnifiedIdeograph3fea': '\xe3\xbf\xaa',
'CjkUnifiedIdeograph3fef': '\xe3\xbf\xaf',
'CjkUnifiedIdeograph3fed': '\xe3\xbf\xad',
'CjkUnifiedIdeograph3fee': '\xe3\xbf\xae',
'UnicodeCharacter0xdee6': '\xed\xbb\xa6',
'UnicodeCharacter0xdee7': '\xed\xbb\xa7',
'UnicodeCharacter0xdee4': '\xed\xbb\xa4',
'UnicodeCharacter0xdee5': '\xed\xbb\xa5',
'UnicodeCharacter0xdee2': '\xed\xbb\xa2',
'UnicodeCharacter0xdee3': '\xed\xbb\xa3',
'UnicodeCharacter0xdee0': '\xed\xbb\xa0',
'UnicodeCharacter0xdee1': '\xed\xbb\xa1',
'HangulSyllableBbwibs': '\xec\x80\xb2',
'Fuse': '\xe2\x8f\x9b',
'HangulSyllableSoep': '\xec\x87\xba',
'SyriacThreeDotsBelow': '\xdd\x86',
'CjkCompatibilityIdeographfa0b': '\xef\xa8\x8b',
'UnicodeCharacter0x1cb0': '\xe1\xb2\xb0',
'DevanagariDigitEight': '\xe0\xa5\xae',
'UnicodeCharacter0xe663': '\xee\x99\xa3',
'UnicodeCharacter0xe662': '\xee\x99\xa2',
'UnicodeCharacter0xe661': '\xee\x99\xa1',
'UnicodeCharacter0xe660': '\xee\x99\xa0',
'UnicodeCharacter0xe667': '\xee\x99\xa7',
'UnicodeCharacter0xe666': '\xee\x99\xa6',
'UnicodeCharacter0xe665': '\xee\x99\xa5',
'UnicodeCharacter0xe664': '\xee\x99\xa4',
'MongolianLetterAliGaliCa': '\xe1\xa2\x8b',
'UnicodeCharacter0xe669': '\xee\x99\xa9',
'UnicodeCharacter0xe668': '\xee\x99\xa8',
'UnicodeCharacter0xf556': '\xef\x95\x96',
'UnicodeCharacter0x1cb7': '\xe1\xb2\xb7',
'UnicodeCharacter0xf406': '\xef\x90\x86',
'UnicodeCharacter0xf407': '\xef\x90\x87',
'UnicodeCharacter0xf404': '\xef\x90\x84',
'UnicodeCharacter0xf405': '\xef\x90\x85',
'CopticCapitalLetterOldCopticShima': '\xe2\xb3\x9a',
'UnicodeCharacter0xf403': '\xef\x90\x83',
'UnicodeCharacter0xf400': '\xef\x90\x80',
'UnicodeCharacter0xf401': '\xef\x90\x81',
'UnicodeCharacter0xf408': '\xef\x90\x88',
'UnicodeCharacter0xf409': '\xef\x90\x89',
'LogicalOrWithDoubleUnderbar': '\xe2\xa9\xa3',
'LatinSmallLetterJWithStroke': '\xc9\x89',
'CjkUnifiedIdeograph3fe2': '\xe3\xbf\xa2',
'CjkUnifiedIdeograph3fe3': '\xe3\xbf\xa3',
'CjkUnifiedIdeograph3fe0': '\xe3\xbf\xa0',
'CjkUnifiedIdeograph3fe1': '\xe3\xbf\xa1',
'CjkUnifiedIdeograph3fe6': '\xe3\xbf\xa6',
'CjkUnifiedIdeograph3fe7': '\xe3\xbf\xa7',
'CjkUnifiedIdeograph3fe4': '\xe3\xbf\xa4',
'CjkUnifiedIdeograph3fe5': '\xe3\xbf\xa5',
'UnicodeCharacter0xdeef': '\xed\xbb\xaf',
'MalayalamAuLengthMark': '\xe0\xb5\x97',
'CjkUnifiedIdeograph3fe8': '\xe3\xbf\xa8',
'CjkUnifiedIdeograph3fe9': '\xe3\xbf\xa9',
'UnicodeCharacter0xdeeb': '\xed\xbb\xab',
'UnicodeCharacter0xdeec': '\xed\xbb\xac',
'HangulSyllableBebs': '\xeb\xb2\xb2',
'CjkUnifiedIdeograph5535': '\xe5\x94\xb5',
'CjkUnifiedIdeograph5534': '\xe5\x94\xb4',
'CjkUnifiedIdeograph5537': '\xe5\x94\xb7',
'CjkUnifiedIdeograph5536': '\xe5\x94\xb6',
'CjkUnifiedIdeograph5531': '\xe5\x94\xb1',
'CjkUnifiedIdeograph5530': '\xe5\x94\xb0',
'CjkUnifiedIdeograph5533': '\xe5\x94\xb3',
'CjkUnifiedIdeograph5532': '\xe5\x94\xb2',
'CjkUnifiedIdeograph5539': '\xe5\x94\xb9',
'CjkUnifiedIdeograph5538': '\xe5\x94\xb8',
'CjkUnifiedIdeograph8729': '\xe8\x9c\xa9',
'CjkUnifiedIdeograph8728': '\xe8\x9c\xa8',
'CjkUnifiedIdeograph53c7': '\xe5\x8f\x87',
'CjkUnifiedIdeograph53c6': '\xe5\x8f\x86',
'CjkUnifiedIdeograph53c5': '\xe5\x8f\x85',
'CjkUnifiedIdeograph53c4': '\xe5\x8f\x84',
'CjkUnifiedIdeograph53c3': '\xe5\x8f\x83',
'CjkUnifiedIdeograph53c2': '\xe5\x8f\x82',
'CjkUnifiedIdeograph53c1': '\xe5\x8f\x81',
'CjkUnifiedIdeograph53c0': '\xe5\x8f\x80',
'UnicodeCharacter0xf837': '\xef\xa0\xb7',
'CjkUnifiedIdeograph53c9': '\xe5\x8f\x89',
'CjkUnifiedIdeograph53c8': '\xe5\x8f\x88',
'ArabicLigatureTahWithMeemWithYehFinalForm': '\xef\xb5\xb4',
'SouthEastArrow': '\xe2\x86\x98',
'YiSyllableKax': '\xea\x88\x80',
'CjkUnifiedIdeograph38e0': '\xe3\xa3\xa0',
'CjkUnifiedIdeograph38e1': '\xe3\xa3\xa1',
'CjkUnifiedIdeograph38e2': '\xe3\xa3\xa2',
'CjkUnifiedIdeograph38e3': '\xe3\xa3\xa3',
'CjkUnifiedIdeograph38e4': '\xe3\xa3\xa4',
'CjkUnifiedIdeograph38e5': '\xe3\xa3\xa5',
'CjkUnifiedIdeograph38e6': '\xe3\xa3\xa6',
'CjkUnifiedIdeograph38e7': '\xe3\xa3\xa7',
'CjkUnifiedIdeograph38e8': '\xe3\xa3\xa8',
'CjkUnifiedIdeograph38e9': '\xe3\xa3\xa9',
'UnicodeCharacter0xf839': '\xef\xa0\xb9',
'LatinSmallLetterTWithCurl': '\xc8\xb6',
'GurmukhiLetterLa': '\xe0\xa8\xb2',
'CjkUnifiedIdeograph53cf': '\xe5\x8f\x8f',
'CjkUnifiedIdeograph53ce': '\xe5\x8f\x8e',
'CjkUnifiedIdeograph53cd': '\xe5\x8f\x8d',
'CjkUnifiedIdeograph53cc': '\xe5\x8f\x8c',
'CjkUnifiedIdeograph53cb': '\xe5\x8f\x8b',
'CjkUnifiedIdeograph53ca': '\xe5\x8f\x8a',
'YiSyllableVurx': '\xea\x83\xb8',
'GlagoliticSmallLetterFita': '\xe2\xb1\x9a',
'CjkUnifiedIdeograph717e': '\xe7\x85\xbe',
'CjkUnifiedIdeograph553e': '\xe5\x94\xbe',
'CjkUnifiedIdeograph553d': '\xe5\x94\xbd',
'CjkUnifiedIdeograph872e': '\xe8\x9c\xae',
'CjkUnifiedIdeograph553f': '\xe5\x94\xbf',
'CjkUnifiedIdeograph553a': '\xe5\x94\xba',
'CjkUnifiedIdeograph872b': '\xe8\x9c\xab',
'CjkUnifiedIdeograph553c': '\xe5\x94\xbc',
'CjkUnifiedIdeograph553b': '\xe5\x94\xbb',
'YiSyllableKap': '\xea\x88\x82',
'KangxiRadicalSquare': '\xe2\xbd\x85',
'HangulSyllableJjyaeng': '\xec\xa9\x85',
'CjkUnifiedIdeograph717b': '\xe7\x85\xbb',
'CjkUnifiedIdeograph38ea': '\xe3\xa3\xaa',
'CjkUnifiedIdeograph38eb': '\xe3\xa3\xab',
'CircledWhiteStar': '\xe2\x9c\xaa',
'CjkUnifiedIdeograph38ed': '\xe3\xa3\xad',
'CjkUnifiedIdeograph38ee': '\xe3\xa3\xae',
'CjkUnifiedIdeograph38ef': '\xe3\xa3\xaf',
'YiSyllableSsap': '\xea\x8c\xb4',
'UnicodeCharacter0xf2c7': '\xef\x8b\x87',
'KhmerSymbolDapRoc': '\xe1\xa7\xba',
'GreekSmallLetterAlphaWithPsiliAndVaria': '\xe1\xbc\x82',
'HangulLetterYa': '\xe3\x85\x91',
'HangulLetterYo': '\xe3\x85\x9b',
'HangulLetterYi': '\xe3\x85\xa2',
'UnicodeCharacter0xefd': '\xe0\xbb\xbd',
':(': '\xe2\x8c\xa2',
':)': '\xe2\x8c\xa3',
'UnicodeCharacter0xefe': '\xe0\xbb\xbe',
'ArabicLetterTehehInitialForm': '\xef\xad\xa4',
'UnicodeCharacter0xf2c2': '\xef\x8b\x82',
'UnicodeCharacter0xeff': '\xe0\xbb\xbf',
'LatinSmallLetterClosedOmega': '\xc9\xb7',
':=': '\xe2\x89\x94',
'CjkUnifiedIdeograph3d88': '\xe3\xb6\x88',
'YiSyllableSsax': '\xea\x8c\xb2',
'PhagspaLetterTsha': '\xea\xa1\x91',
'UnicodeCharacter0xefb': '\xe0\xbb\xbb',
'CyrillicCapitalLetterDwe': '\xea\x9a\x80',
'AplFunctionalSymbolUpwardsVane': '\xe2\x8d\x8f',
'UnicodeCharacter0xf2c9': '\xef\x8b\x89',
'CanadianSyllabicsLhoo': '\xe1\x96\xa3',
'UnicodeCharacter0xefc': '\xe0\xbb\xbc',
'HangulSyllableNyelh': '\xeb\x85\xab',
'UnicodeCharacter0xf2c8': '\xef\x8b\x88',
'HangulSyllableTyalb': '\xed\x84\x83',
'ArabicLigatureTehWithYehIsolatedForm': '\xef\xb0\x90',
'RejangLetterPa': '\xea\xa4\xb6',
'CjkUnifiedIdeograph472c': '\xe4\x9c\xac',
'CjkUnifiedIdeograph472b': '\xe4\x9c\xab',
'CjkUnifiedIdeograph472a': '\xe4\x9c\xaa',
'ArabicLetterFarsiYehWithExtendedArabicindicDigitThreeAbove': '\xdd\xb6',
'CjkUnifiedIdeograph472f': '\xe4\x9c\xaf',
'CjkUnifiedIdeograph472e': '\xe4\x9c\xae',
'CjkUnifiedIdeograph472d': '\xe4\x9c\xad',
'HebrewLetterShinWithDagesh': '\xef\xad\x89',
'YiSyllableBbiet': '\xea\x81\x9a',
'HangulSyllableHun': '\xed\x9b\x88',
'CjkUnifiedIdeograph4729': '\xe4\x9c\xa9',
'CjkUnifiedIdeograph4728': '\xe4\x9c\xa8',
'HangulSyllableHuk': '\xed\x9b\x9c',
'HangulSyllableHuj': '\xed\x9b\x9a',
'HangulSyllableHuh': '\xed\x9b\x9f',
'CjkUnifiedIdeograph4723': '\xe4\x9c\xa3',
'CjkUnifiedIdeograph4722': '\xe4\x9c\xa2',
'CjkUnifiedIdeograph4721': '\xe4\x9c\xa1',
'CjkUnifiedIdeograph4720': '\xe4\x9c\xa0',
'CjkUnifiedIdeograph4727': '\xe4\x9c\xa7',
'CjkUnifiedIdeograph4726': '\xe4\x9c\xa6',
'CjkUnifiedIdeograph4725': '\xe4\x9c\xa5',
'CjkUnifiedIdeograph4724': '\xe4\x9c\xa4',
'CopticCapitalLetterKapa': '\xe2\xb2\x94',
'VaiSyllableNdoleMa': '\xea\x98\xaa',
'Keyboard': '\xe2\x8c\xa8',
'HangulSyllableHut': '\xed\x9b\x9d',
'HangulSyllableHus': '\xed\x9b\x97',
'VaiSyllableWon': '\xea\x97\x80',
'VaiSyllableWoo': '\xea\x95\xb4',
'CjkUnifiedIdeograph3973': '\xe3\xa5\xb3',
'CjkUnifiedIdeograph3972': '\xe3\xa5\xb2',
'ArabicInvertedDamma': '\xd9\x97',
'CjkUnifiedIdeograph3970': '\xe3\xa5\xb0',
'CjkUnifiedIdeograph3977': '\xe3\xa5\xb7',
'CjkUnifiedIdeograph3976': '\xe3\xa5\xb6',
'CjkUnifiedIdeograph3975': '\xe3\xa5\xb5',
'ArmenianCapitalLetterSha': '\xd5\x87',
'CjkUnifiedIdeograph3dd7': '\xe3\xb7\x97',
'CjkUnifiedIdeograph3dd6': '\xe3\xb7\x96',
'CjkUnifiedIdeograph3979': '\xe3\xa5\xb9',
'CjkUnifiedIdeograph3978': '\xe3\xa5\xb8',
'CjkUnifiedIdeograph3dd3': '\xe3\xb7\x93',
'CjkUnifiedIdeograph3dd2': '\xe3\xb7\x92',
'UnicodeCharacter0x900': '\xe0\xa4\x80',
'CjkUnifiedIdeograph3dd0': '\xe3\xb7\x90',
'CjkUnifiedIdeograph7263': '\xe7\x89\xa3',
'CjkUnifiedIdeograph7262': '\xe7\x89\xa2',
'CjkUnifiedIdeograph7261': '\xe7\x89\xa1',
'CjkUnifiedIdeograph7260': '\xe7\x89\xa0',
'CjkUnifiedIdeograph7267': '\xe7\x89\xa7',
'CjkUnifiedIdeograph7266': '\xe7\x89\xa6',
'CjkUnifiedIdeograph7265': '\xe7\x89\xa5',
'CjkUnifiedIdeograph7264': '\xe7\x89\xa4',
'UnicodeCharacter0xdd4d': '\xed\xb5\x8d',
'UnicodeCharacter0xdd4e': '\xed\xb5\x8e',
'CjkUnifiedIdeograph7269': '\xe7\x89\xa9',
'CjkUnifiedIdeograph7268': '\xe7\x89\xa8',
'UnicodeCharacter0xdd4a': '\xed\xb5\x8a',
'UnicodeCharacter0xdd4b': '\xed\xb5\x8b',
'UnicodeCharacter0xdd4c': '\xed\xb5\x8c',
'ArabicLigatureDadWithKhahIsolatedForm': '\xef\xb0\xa4',
'CjkUnifiedIdeograph4cd2': '\xe4\xb3\x92',
'HangulSyllableGgyolh': '\xea\xbe\xab',
'UnicodeCharacter0xdba9': '\xed\xae\xa9',
'LatinSmallLetterWWithCircumflex': '\xc5\xb5',
'BlackSunWithRays': '\xe2\x98\x80',
'HangulSyllableRuh': '\xeb\xa4\x83',
'CjkUnifiedIdeograph726c': '\xe7\x89\xac',
'CjkUnifiedIdeograph726b': '\xe7\x89\xab',
'CjkUnifiedIdeograph726a': '\xe7\x89\xaa',
'UnicodeCharacter0xea92': '\xee\xaa\x92',
'UnicodeCharacter0xdd48': '\xed\xb5\x88',
'CjkUnifiedIdeograph726f': '\xe7\x89\xaf',
'CjkUnifiedIdeograph726e': '\xe7\x89\xae',
'CjkUnifiedIdeograph726d': '\xe7\x89\xad',
'UnicodeCharacter0xdd44': '\xed\xb5\x84',
'UnicodeCharacter0xdd45': '\xed\xb5\x85',
'UnicodeCharacter0xdd46': '\xed\xb5\x86',
'UnicodeCharacter0xdd47': '\xed\xb5\x87',
'UnicodeCharacter0xdd40': '\xed\xb5\x80',
'UnicodeCharacter0xdd41': '\xed\xb5\x81',
'UnicodeCharacter0xdd42': '\xed\xb5\x82',
'CjkUnifiedIdeograph4cd8': '\xe4\xb3\x98',
'CjkUnifiedIdeograph397c': '\xe3\xa5\xbc',
'TibetanLetterTa': '\xe0\xbd\x8f',
'CjkUnifiedIdeograph397a': '\xe3\xa5\xba',
'UnicodeCharacter0xeb6d': '\xee\xad\xad',
'UnicodeCharacter0xeb6c': '\xee\xad\xac',
'CjkUnifiedIdeograph397f': '\xe3\xa5\xbf',
'CjkUnifiedIdeograph397e': '\xe3\xa5\xbe',
'CjkUnifiedIdeograph397d': '\xe3\xa5\xbd',
'CjkUnifiedIdeograph3ddf': '\xe3\xb7\x9f',
'CjkUnifiedIdeograph3dde': '\xe3\xb7\x9e',
'CjkUnifiedIdeograph3ddd': '\xe3\xb7\x9d',
'CjkUnifiedIdeograph3ddc': '\xe3\xb7\x9c',
'RightwardsArrowFromBarToBlackDiamond': '\xe2\xa4\xa0',
'CjkUnifiedIdeograph3dda': '\xe3\xb7\x9a',
'UnicodeCharacter0xf28a': '\xef\x8a\x8a',
'OrWithDotInside': '\xe2\x9f\x87',
'YiSyllableMurx': '\xea\x83\x87',
'HangulSyllableWelb': '\xec\x9b\xb3',
'CjkUnifiedIdeograph4c3f': '\xe4\xb0\xbf',
'ArabicLigatureBehWithYehFinalForm': '\xef\xb1\xaf',
'HangulSyllableWelm': '\xec\x9b\xb2',
'UnicodeCharacter0x2d76': '\xe2\xb5\xb6',
'HangulSyllableRuj': '\xeb\xa3\xbe',
'HangulSyllablePess': '\xed\x8e\xac',
'CjkUnifiedIdeograph4c3a': '\xe4\xb0\xba',
'UnicodeCharacter0xd7ad': '\xed\x9e\xad',
'UnicodeCharacter0xd7ae': '\xed\x9e\xae',
'CjkUnifiedIdeograph90c5': '\xe9\x83\x85',
'ModifierLetterExtralowToneBar': '\xcb\xa9',
'UnicodeCharacter0xd7aa': '\xed\x9e\xaa',
'UnicodeCharacter0xd7ab': '\xed\x9e\xab',
'HangulSyllableCenj': '\xec\xb2\xb9',
'KangxiRadicalDot': '\xe2\xbc\x82',
'RightDoubleQuotationMark': '\xe2\x80\x9d',
'HangulSyllableWelh': '\xec\x9b\xb7',
'HangulSyllableSsweonj': '\xec\x92\x85',
'HexagramForSmallPreponderance': '\xe4\xb7\xbd',
'HangulSyllableSsweonh': '\xec\x92\x86',
'HangulSyllableRum': '\xeb\xa3\xb8',
'HangulSyllableSsweong': '\xec\x92\x95',
'UnicodeCharacter0xf28c': '\xef\x8a\x8c',
'UnicodeCharacter0xf6b8': '\xef\x9a\xb8',
'UnicodeCharacter0xf6b9': '\xef\x9a\xb9',
'HangulSyllableSeogs': '\xec\x84\x9f',
'CombiningRightHalfRingBelow': '\xcc\xb9',
'CjkUnifiedIdeograph661a': '\xe6\x98\x9a',
'UnicodeCharacter0xf6b3': '\xef\x9a\xb3',
'CjkUnifiedIdeograph661c': '\xe6\x98\x9c',
'CjkUnifiedIdeograph661b': '\xe6\x98\x9b',
'CjkUnifiedIdeograph661e': '\xe6\x98\x9e',
'CjkUnifiedIdeograph661d': '\xe6\x98\x9d',
'UnicodeCharacter0xf6b4': '\xef\x9a\xb4',
'NorthEastArrow': '\xe2\x86\x97',
'UnicodeCharacter0xf7bc': '\xef\x9e\xbc',
'HangulSyllableRul': '\xeb\xa3\xb0',
'ArabicLetterKafWithRing': '\xda\xab',
'HangulSyllableWelp': '\xec\x9b\xb6',
'HangulSyllableGaelm': '\xea\xb0\xa6',
'HangulSyllableWels': '\xec\x9b\xb4',
'SquareMega': '\xe3\x8d\x8b',
'HangulSyllableGaelg': '\xea\xb0\xa5',
'HangulSyllableGaelb': '\xea\xb0\xa7',
'KannadaDigitFour': '\xe0\xb3\xaa',
'UnicodeCharacter0xd7a4': '\xed\x9e\xa4',
'UnicodeCharacter0xd7a5': '\xed\x9e\xa5',
'UnicodeCharacter0xd7a6': '\xed\x9e\xa6',
'UnicodeCharacter0xd7a7': '\xed\x9e\xa7',
'HangulSyllableBils': '\xeb\xb9\x90',
'EthiopicSyllableLu': '\xe1\x88\x89',
'HangulSyllableGaelt': '\xea\xb0\xa9',
'HangulSyllableGaels': '\xea\xb0\xa8',
'YiRadicalVep': '\xea\x92\xb1',
'HangulSyllableGaelp': '\xea\xb0\xaa',
'ArmenianApostrophe': '\xd5\x9a',
'CjkUnifiedIdeograph67d8': '\xe6\x9f\x98',
'HangulJungseongEoo': '\xe1\x85\xba',
'ArabicLigatureDadWithHahWithAlefMaksuraFinalForm': '\xef\xb5\xae',
'KhmerSymbolTuteyasat': '\xe1\xa7\xb0',
'HangulSyllableJjyobs': '\xec\xac\xbe',
'CjkUnifiedIdeograph6619': '\xe6\x98\x99',
'CjkUnifiedIdeograph6618': '\xe6\x98\x98',
'CjkUnifiedIdeograph8de2': '\xe8\xb7\xa2',
'CjkUnifiedIdeograph6611': '\xe6\x98\x91',
'CjkUnifiedIdeograph6610': '\xe6\x98\x90',
'CjkUnifiedIdeograph6613': '\xe6\x98\x93',
'CjkUnifiedIdeograph6612': '\xe6\x98\x92',
'HangulJungseongEou': '\xe1\x85\xbb',
'CjkUnifiedIdeograph6614': '\xe6\x98\x94',
'CjkUnifiedIdeograph6617': '\xe6\x98\x97',
'CjkUnifiedIdeograph6616': '\xe6\x98\x96',
'CjkUnifiedIdeograph8fe7': '\xe8\xbf\xa7',
'CjkUnifiedIdeograph8fe6': '\xe8\xbf\xa6',
'CjkUnifiedIdeograph8fe5': '\xe8\xbf\xa5',
'CjkUnifiedIdeograph8fe4': '\xe8\xbf\xa4',
'CjkUnifiedIdeograph8fe3': '\xe8\xbf\xa3',
'CjkUnifiedIdeograph8fe2': '\xe8\xbf\xa2',
'CjkUnifiedIdeograph8fe1': '\xe8\xbf\xa1',
'CjkUnifiedIdeograph8fe0': '\xe8\xbf\xa0',
'CjkCompatibilityIdeographfa1c': '\xef\xa8\x9c',
'CjkUnifiedIdeograph8fe9': '\xe8\xbf\xa9',
'CjkUnifiedIdeograph8fe8': '\xe8\xbf\xa8',
'CjkUnifiedIdeograph3e82': '\xe3\xba\x82',
'GreekSmallLetterOmegaWithDasiaAndYpogegrammeni': '\xe1\xbe\xa1',
'CjkUnifiedIdeograph3e80': '\xe3\xba\x80',
'GlagoliticCapitalLetterYestu': '\xe2\xb0\x85',
'CjkUnifiedIdeograph3e86': '\xe3\xba\x86',
'CjkUnifiedIdeograph3e87': '\xe3\xba\x87',
'CjkUnifiedIdeograph3e84': '\xe3\xba\x84',
'CjkUnifiedIdeograph3e85': '\xe3\xba\x85',
'CjkUnifiedIdeograph3e88': '\xe3\xba\x88',
'CjkUnifiedIdeograph3e89': '\xe3\xba\x89',
'KannadaLetterEe': '\xe0\xb2\x8f',
'CjkUnifiedIdeograph7177': '\xe7\x85\xb7',
'HangulSyllableGguc': '\xea\xbf\x8f',
'DevanagariStressSignUdatta': '\xe0\xa5\x91',
'HangulSyllableBbaess': '\xeb\xba\x90',
'PhagspaSuperfixedLetterRa': '\xea\xa1\xb2',
'CjkUnifiedIdeograph7173': '\xe7\x85\xb3',
'BalinesePanti': '\xe1\xad\x9a',
'CjkUnifiedIdeograph7172': '\xe7\x85\xb2',
'CjkUnifiedIdeograph3e8b': '\xe3\xba\x8b',
'CjkUnifiedIdeograph3e8c': '\xe3\xba\x8c',
'CjkUnifiedIdeograph3e8a': '\xe3\xba\x8a',
'CjkUnifiedIdeograph3e8f': '\xe3\xba\x8f',
'HangulSyllableDdeulp': '\xeb\x9c\xb6',
'CjkUnifiedIdeograph3e8d': '\xe3\xba\x8d',
'CjkUnifiedIdeograph3e8e': '\xe3\xba\x8e',
'CyrillicSmallLetterZhe': '\xd0\xb6',
'HangulSyllableNyong': '\xeb\x87\xbd',
'CjkUnifiedIdeograph8fef': '\xe8\xbf\xaf',
'CjkUnifiedIdeograph8fee': '\xe8\xbf\xae',
'CjkUnifiedIdeograph8fed': '\xe8\xbf\xad',
'CjkUnifiedIdeograph8fec': '\xe8\xbf\xac',
'CjkUnifiedIdeograph8feb': '\xe8\xbf\xab',
'CjkUnifiedIdeograph8fea': '\xe8\xbf\xaa',
'HangulSyllableDdeulb': '\xeb\x9c\xb3',
'HangulSyllableDdeulm': '\xeb\x9c\xb2',
'CentrelineLowLine': '\xef\xb9\x8e',
'HangulSyllableDdeulh': '\xeb\x9c\xb7',
'UnicodeCharacter0xef4': '\xe0\xbb\xb4',
'HangulSyllableCwaelm': '\xec\xb5\x8a',
'UnicodeCharacter0xef5': '\xe0\xbb\xb5',
'ArabicSignRadiAllahouAnhu': '\xd8\x93',
'HangulSyllableRub': '\xeb\xa3\xb9',
'UnicodeCharacter0xef6': '\xe0\xbb\xb6',
'GujaratiLetterNya': '\xe0\xaa\x9e',
'UnicodeCharacter0xa6ba': '\xea\x9a\xba',
'GreekLetterDigamma': '\xcf\x9c',
'UnicodeCharacter0xef7': '\xe0\xbb\xb7',
'CjkUnifiedIdeograph7179': '\xe7\x85\xb9',
'HangulSyllableJubs': '\xec\xa4\x8e',
'UnicodeCharacter0xef0': '\xe0\xbb\xb0',
'UnicodeCharacter0xef1': '\xe0\xbb\xb1',
'HangulLetterAraea': '\xe3\x86\x8d',
'ArabicLigatureYehWithYehIsolatedForm': '\xef\xb1\x9a',
'UnicodeCharacter0xef2': '\xe0\xbb\xb2',
'CyrillicCapitalLetterHaWithHook': '\xd3\xbc',
'UnicodeCharacter0xef3': '\xe0\xbb\xb3',
'CjkUnifiedIdeograph8de5': '\xe8\xb7\xa5',
'CjkUnifiedIdeograph950e': '\xe9\x94\x8e',
'CherokeeLetterTso': '\xe1\x8f\xa6',
'TibetanSubjoinedLetterTta': '\xe0\xbe\x9a',
'UnicodeCharacter0xf2bc': '\xef\x8a\xbc',
'ArabicLigatureMeemWithKhahInitialForm': '\xef\xb3\x90',
'CjkUnifiedIdeograph3d8a': '\xe3\xb6\x8a',
'CjkUnifiedIdeograph6f0f': '\xe6\xbc\x8f',
'CjkUnifiedIdeograph6f0d': '\xe6\xbc\x8d',
'CjkUnifiedIdeograph6f0e': '\xe6\xbc\x8e',
'CjkUnifiedIdeograph6f0b': '\xe6\xbc\x8b',
'CjkUnifiedIdeograph6f0c': '\xe6\xbc\x8c',
'CjkUnifiedIdeograph6f0a': '\xe6\xbc\x8a',
'UnicodeCharacter0xef8': '\xe0\xbb\xb8',
'HangulSyllableCyen': '\xec\xb3\xb0',
'ThereExists': '\xe2\x88\x83',
'CjkUnifiedIdeograph3d8f': '\xe3\xb6\x8f',
'CherokeeLetterTse': '\xe1\x8f\xa4',
'HangulSyllableMed': '\xeb\xa9\x9b',
'ShortDownTack': '\xe2\xab\x9f',
'UnicodeCharacter0xed3d': '\xee\xb4\xbd',
'HangulSyllableByanh': '\xeb\xb1\x92',
'UnicodeCharacter0xa6bf': '\xea\x9a\xbf',
'YiSyllableHmuox': '\xea\x82\x9c',
'YiSyllableHat': '\xea\x89\xb1',
'HangulSyllablePwinh': '\xed\x93\x92',
'HangulSyllablePwinj': '\xed\x93\x91',
'CjkUnifiedIdeograph34f0': '\xe3\x93\xb0',
'LatinCapitalLetterQWithStrokeThroughDescender': '\xea\x9d\x96',
'HangulSyllablePwing': '\xed\x93\xa1',
'CjkUnifiedIdeograph92c2': '\xe9\x8b\x82',
'YiSyllableRrop': '\xea\x8e\x83',
'YiSyllableHap': '\xea\x89\xb4',
'LatinSmallLetterOWithBreve': '\xc5\x8f',
'CjkUnifiedIdeograph6f06': '\xe6\xbc\x86',
'CjkUnifiedIdeograph6f07': '\xe6\xbc\x87',
'CjkUnifiedIdeograph6f04': '\xe6\xbc\x84',
'CjkUnifiedIdeograph6f05': '\xe6\xbc\x85',
'CjkUnifiedIdeograph6f02': '\xe6\xbc\x82',
'CjkUnifiedIdeograph6f03': '\xe6\xbc\x83',
'CjkUnifiedIdeograph6f00': '\xe6\xbc\x80',
'CjkUnifiedIdeograph6f01': '\xe6\xbc\x81',
'UnicodeCharacter0xed3b': '\xee\xb4\xbb',
'KannadaLetterNga': '\xe0\xb2\x99',
'LatinCapitalLetterTWithLineBelow': '\xe1\xb9\xae',
'TifinaghLetterYaddh': '\xe2\xb4\xba',
'HangulSyllableSses': '\xec\x8e\x97',
'DevanagariLetterLlla': '\xe0\xa4\xb4',
'MongolianNirugu': '\xe1\xa0\x8a',
'HangulSyllableCwaelg': '\xec\xb5\x89',
'YiSyllableRrox': '\xea\x8e\x81',
'CjkUnifiedIdeograph4f3a': '\xe4\xbc\xba',
'CjkUnifiedIdeograph4f3c': '\xe4\xbc\xbc',
'CjkUnifiedIdeograph4f3b': '\xe4\xbc\xbb',
'CjkUnifiedIdeograph4f3e': '\xe4\xbc\xbe',
'CjkUnifiedIdeograph4f3d': '\xe4\xbc\xbd',
'ArabicLigatureGhainWithYehFinalForm': '\xef\xb4\x96',
'CjkUnifiedIdeograph4f3f': '\xe4\xbc\xbf',
'CjkUnifiedIdeograph56aa': '\xe5\x9a\xaa',
'CjkUnifiedIdeograph56ab': '\xe5\x9a\xab',
'MongolianLetterSibePa': '\xe1\xa1\xa6',
'CjkUnifiedIdeograph56ad': '\xe5\x9a\xad',
'CjkUnifiedIdeograph56ae': '\xe5\x9a\xae',
'CjkUnifiedIdeograph56af': '\xe5\x9a\xaf',
'HangulSyllableJjyaej': '\xec\xa9\x86',
'CjkUnifiedIdeograph7d69': '\xe7\xb5\xa9',
'CjkUnifiedIdeograph7d68': '\xe7\xb5\xa8',
'CjkUnifiedIdeograph7d65': '\xe7\xb5\xa5',
'CjkUnifiedIdeograph7d64': '\xe7\xb5\xa4',
'CjkUnifiedIdeograph7d67': '\xe7\xb5\xa7',
'Saturn': '\xe2\x99\x84',
'CjkUnifiedIdeograph7d61': '\xe7\xb5\xa1',
'CjkUnifiedIdeograph7d60': '\xe7\xb5\xa0',
'CjkUnifiedIdeograph7d63': '\xe7\xb5\xa3',
'CjkUnifiedIdeograph7d62': '\xe7\xb5\xa2',
'CjkUnifiedIdeograph4f39': '\xe4\xbc\xb9',
'CjkUnifiedIdeograph4f38': '\xe4\xbc\xb8',
'HangulSyllableSweos': '\xec\x89\x87',
'CjkUnifiedIdeograph4f31': '\xe4\xbc\xb1',
'CjkUnifiedIdeograph4f30': '\xe4\xbc\xb0',
'CjkUnifiedIdeograph4f33': '\xe4\xbc\xb3',
'CjkUnifiedIdeograph4f32': '\xe4\xbc\xb2',
'CjkUnifiedIdeograph4f35': '\xe4\xbc\xb5',
'CjkUnifiedIdeograph4f34': '\xe4\xbc\xb4',
'CjkUnifiedIdeograph4f37': '\xe4\xbc\xb7',
'CjkUnifiedIdeograph4f36': '\xe4\xbc\xb6',
'YiSyllableTut': '\xea\x84\xb0',
'UnicodeCharacter0xd9c0': '\xed\xa7\x80',
'YiSyllableTup': '\xea\x84\xb3',
'YiSyllableTur': '\xea\x84\xb5',
'SucceedsAboveSinglelineEqualsSign': '\xe2\xaa\xb0',
'CombiningLatinSmallLetterRRotunda': '\xe1\xb7\xa3',
'HangulSyllableJjyaem': '\xec\xa9\x80',
'YiSyllableTux': '\xea\x84\xb1',
'HangulSyllableSseb': '\xec\x8e\x95',
'HangulSyllableSsec': '\xec\x8e\x9b',
'HangulSyllableJwags': '\xec\xa2\x8f',
'HeavyTeardropshankedRightwardsArrow': '\xe2\x9e\xbb',
'BengaliLetterSha': '\xe0\xa6\xb6',
'CjkUnifiedIdeograph7d6d': '\xe7\xb5\xad',
'HangulSyllableRweos': '\xeb\xa4\x97',
'CjkUnifiedIdeograph7d6f': '\xe7\xb5\xaf',
'CjkUnifiedIdeograph7d6a': '\xe7\xb5\xaa',
'HangulSyllableRweot': '\xeb\xa4\x9d',
'CjkUnifiedIdeograph7d6c': '\xe7\xb5\xac',
'CopticSmallLetterCrossedShei': '\xe2\xb3\x83',
'CjkUnifiedIdeograph56a0': '\xe5\x9a\xa0',
'CjkUnifiedIdeograph56a1': '\xe5\x9a\xa1',
'CjkUnifiedIdeograph56a2': '\xe5\x9a\xa2',
'CjkUnifiedIdeograph56a3': '\xe5\x9a\xa3',
'CjkUnifiedIdeograph56a4': '\xe5\x9a\xa4',
'CjkUnifiedIdeograph56a5': '\xe5\x9a\xa5',
'CjkUnifiedIdeograph56a6': '\xe5\x9a\xa6',
'CjkUnifiedIdeograph56a7': '\xe5\x9a\xa7',
'CjkUnifiedIdeograph56a8': '\xe5\x9a\xa8',
'CjkUnifiedIdeograph56a9': '\xe5\x9a\xa9',
'HangulSyllableRweoc': '\xeb\xa4\x9b',
'HangulSyllableRweob': '\xeb\xa4\x95',
'HangulSyllableRweod': '\xeb\xa4\x8b',
'HangulSyllableRweog': '\xeb\xa4\x85',
'ParenthesizedHangulHieuh': '\xe3\x88\x8d',
'SmallContainsWithVerticalBarAtEndOfHorizontalStroke': '\xe2\x8b\xbc',
'UnicodeCharacter0xabfc': '\xea\xaf\xbc',
'UnicodeCharacter0xabfb': '\xea\xaf\xbb',
'UnicodeCharacter0xabfa': '\xea\xaf\xba',
'UnicodeCharacter0xe528': '\xee\x94\xa8',
'ThreeperemSpace': '\xe2\x80\x84',
'CjkUnifiedIdeograph9ab8': '\xe9\xaa\xb8',
'UnicodeCharacter0xabfe': '\xea\xaf\xbe',
'UnicodeCharacter0xabfd': '\xea\xaf\xbd',
'KayahLiLetterPha': '\xea\xa4\x96',
'CjkUnifiedIdeograph6952': '\xe6\xa5\x92',
'HangulSyllableSweob': '\xec\x89\x85',
'HangulSyllableSweoc': '\xec\x89\x8b',
'UnicodeCharacter0xe0c1': '\xee\x83\x81',
'CjkUnifiedIdeograph6eda': '\xe6\xbb\x9a',
'UnicodeCharacter0xe529': '\xee\x94\xa9',
'CjkUnifiedIdeograph6edc': '\xe6\xbb\x9c',
'CjkUnifiedIdeograph6edb': '\xe6\xbb\x9b',
'OpenCentreCross': '\xe2\x9c\x9b',
'CjkUnifiedIdeograph6edd': '\xe6\xbb\x9d',
'CjkUnifiedIdeograph6edf': '\xe6\xbb\x9f',
'HangulSyllableMwebs': '\xeb\xad\xbe',
'CyrillicCapitalLetterDje': '\xd0\x82',
'UnicodeCharacter0xd5a': '\xe0\xb5\x9a',
'UnicodeCharacter0xd5b': '\xe0\xb5\x9b',
'UnicodeCharacter0xd5c': '\xe0\xb5\x9c',
'UnicodeCharacter0xd5d': '\xe0\xb5\x9d',
'UnicodeCharacter0xd5e': '\xe0\xb5\x9e',
'UnicodeCharacter0xd5f': '\xe0\xb5\x9f',
'HangulSyllableByaeng': '\xeb\xb1\xbd',
'UnicodeCharacter0xabf9': '\xea\xaf\xb9',
'UnicodeCharacter0xabf8': '\xea\xaf\xb8',
'UnicodeCharacter0xe3c8': '\xee\x8f\x88',
'UnicodeCharacter0xabf3': '\xea\xaf\xb3',
'UnicodeCharacter0xabf2': '\xea\xaf\xb2',
'UnicodeCharacter0xabf1': '\xea\xaf\xb1',
'UnicodeCharacter0xabf0': '\xea\xaf\xb0',
'UnicodeCharacter0xabf7': '\xea\xaf\xb7',
'UnicodeCharacter0xabf6': '\xea\xaf\xb6',
'UnicodeCharacter0xabf5': '\xea\xaf\xb5',
'UnicodeCharacter0xabf4': '\xea\xaf\xb4',
'UnicodeCharacter0xd58': '\xe0\xb5\x98',
'UnicodeCharacter0xd59': '\xe0\xb5\x99',
'CjkUnifiedIdeograph63b0': '\xe6\x8e\xb0',
'UnicodeCharacter0xd50': '\xe0\xb5\x90',
'UnicodeCharacter0xd51': '\xe0\xb5\x91',
'UnicodeCharacter0xd52': '\xe0\xb5\x92',
'UnicodeCharacter0xd53': '\xe0\xb5\x93',
'UnicodeCharacter0xd54': '\xe0\xb5\x94',
'UnicodeCharacter0xd55': '\xe0\xb5\x95',
'UnicodeCharacter0xd56': '\xe0\xb5\x96',
'CjkUnifiedIdeograph6ed9': '\xe6\xbb\x99',
'CjkUnifiedIdeograph6ed8': '\xe6\xbb\x98',
'CjkUnifiedIdeograph6951': '\xe6\xa5\x91',
'CjkUnifiedIdeograph6ed1': '\xe6\xbb\x91',
'CjkUnifiedIdeograph6ed0': '\xe6\xbb\x90',
'CjkUnifiedIdeograph6ed3': '\xe6\xbb\x93',
'CjkUnifiedIdeograph6ed2': '\xe6\xbb\x92',
'CjkUnifiedIdeograph6ed5': '\xe6\xbb\x95',
'CjkUnifiedIdeograph6ed4': '\xe6\xbb\x94',
'CjkUnifiedIdeograph6ed7': '\xe6\xbb\x97',
'CjkUnifiedIdeograph6ed6': '\xe6\xbb\x96',
'CjkUnifiedIdeograph7ef0': '\xe7\xbb\xb0',
'CjkUnifiedIdeograph7ef1': '\xe7\xbb\xb1',
'CjkUnifiedIdeograph7ef2': '\xe7\xbb\xb2',
'CjkUnifiedIdeograph7ef3': '\xe7\xbb\xb3',
'CjkUnifiedIdeograph7ef4': '\xe7\xbb\xb4',
'CjkUnifiedIdeograph7ef5': '\xe7\xbb\xb5',
'CjkUnifiedIdeograph7ef6': '\xe7\xbb\xb6',
'CjkUnifiedIdeograph7ef7': '\xe7\xbb\xb7',
'CjkUnifiedIdeograph4aed': '\xe4\xab\xad',
'CjkUnifiedIdeograph4aee': '\xe4\xab\xae',
'CjkUnifiedIdeograph4aef': '\xe4\xab\xaf',
'UnicodeCharacter0xddab': '\xed\xb6\xab',
'TwoDotLeader': '\xe2\x80\xa5',
'CjkUnifiedIdeograph4aea': '\xe4\xab\xaa',
'CjkUnifiedIdeograph4aeb': '\xe4\xab\xab',
'CjkUnifiedIdeograph4aec': '\xe4\xab\xac',
'CjkUnifiedIdeograph92c8': '\xe9\x8b\x88',
'CjkUnifiedIdeograph6956': '\xe6\xa5\x96',
'RighthandInteriorProduct': '\xe2\xa8\xbd',
'HangulSyllableBying': '\xeb\xb8\xbd',
'HangulSyllableByinh': '\xeb\xb8\xae',
'HebrewAccentZaqefQatan': '\xd6\x94',
'HangulSyllableByinj': '\xeb\xb8\xad',
'HangulSyllableWim': '\xec\x9c\x94',
'CjkUnifiedIdeograph9a8a': '\xe9\xaa\x8a',
'UnicodeCharacter0xdf21': '\xed\xbc\xa1',
'CjkUnifiedIdeograph92c9': '\xe9\x8b\x89',
'LatinSmallLetterPWithStrokeThroughDescender': '\xea\x9d\x91',
'ArabicLigatureYehWithHamzaAboveWithHahInitialForm': '\xef\xb2\x98',
'ArabicLetterQafWithThreeDotsAbove': '\xda\xa8',
'KangxiRadicalFish': '\xe2\xbf\x82',
'HangulSyllableSswe': '\xec\x92\x9c',
'ArabicLigatureYehWithHamzaAboveWithJeemIsolatedForm': '\xef\xb0\x80',
'HangulSyllableSswa': '\xec\x8f\xb4',
'HangulSyllableTyess': '\xed\x86\x98',
'HangulSyllableSswi': '\xec\x92\xb8',
'CjkUnifiedIdeograph7efa': '\xe7\xbb\xba',
'CjkUnifiedIdeograph7efb': '\xe7\xbb\xbb',
'CjkUnifiedIdeograph7efc': '\xe7\xbb\xbc',
'CjkUnifiedIdeograph4ae8': '\xe4\xab\xa8',
'CjkUnifiedIdeograph4ae9': '\xe4\xab\xa9',
'CjkUnifiedIdeograph7eff': '\xe7\xbb\xbf',
'LeftwardsArrowAboveShortRightwardsArrow': '\xe2\xa5\x83',
'CjkUnifiedIdeograph4ae4': '\xe4\xab\xa4',
'CjkUnifiedIdeograph4ae5': '\xe4\xab\xa5',
'CjkUnifiedIdeograph4ae6': '\xe4\xab\xa6',
'CjkUnifiedIdeograph4ae7': '\xe4\xab\xa7',
'CjkUnifiedIdeograph4ae0': '\xe4\xab\xa0',
'CjkUnifiedIdeograph4ae1': '\xe4\xab\xa1',
'CjkUnifiedIdeograph4ae2': '\xe4\xab\xa2',
'CjkUnifiedIdeograph4ae3': '\xe4\xab\xa3',
'HangulSyllableHyalm': '\xed\x96\x9a',
'CjkUnifiedIdeograph8ba4': '\xe8\xae\xa4',
'HangulSyllableHyalh': '\xed\x96\x9f',
'UnicodeCharacter0xe71c': '\xee\x9c\x9c',
'HeavyConcavepointedBlackRightwardsArrow': '\xe2\x9e\xa8',
'HangulSyllableHyalb': '\xed\x96\x9b',
'IdeographicDescriptionCharacterSurroundFromUpperLeft': '\xe2\xbf\xb8',
'LatinCapitalLetterSWithAcute': '\xc5\x9a',
'SyriacFeminineDot': '\xdd\x80',
'HangulSyllableHyalt': '\xed\x96\x9d',
'HangulSyllableNwess': '\xeb\x89\x90',
'UnicodeCharacter0xe520': '\xee\x94\xa0',
'HangulSyllableHyalp': '\xed\x96\x9e',
'LatinSmallLetterMWithPalatalHook': '\xe1\xb6\x86',
'HangulSyllableGeonj': '\xea\xb1\xb5',
'CyrillicSmallLetterAWithBreve': '\xd3\x91',
'CjkUnifiedIdeograph8ba3': '\xe8\xae\xa3',
'ArabicLigatureSeenWithYehIsolatedForm': '\xef\xb3\xbc',
'HangulSyllableBbeuss': '\xec\x81\xac',
'LimbuDigitEight': '\xe1\xa5\x8e',
'UnicodeCharacter0xf768': '\xef\x9d\xa8',
'HangulSyllableGeong': '\xea\xb2\x85',
'HangulSyllableDyaenh': '\xeb\x8c\xbe',
'UnicodeCharacter0xe521': '\xee\x94\xa1',
'HangulSyllableReup': '\xeb\xa6\x8e',
'ArabicLigatureKafWithAlefIsolatedForm': '\xef\xb0\xb7',
'UnicodeCharacter0xe33f': '\xee\x8c\xbf',
'UnicodeCharacter0xf7bb': '\xef\x9e\xbb',
'CyrillicCapitalLetterKaWithVerticalStroke': '\xd2\x9c',
'YiSyllableMuot': '\xea\x82\xb9',
'YiSyllableMuop': '\xea\x82\xbc',
'HangulSyllableKyeg': '\xec\xbc\xb9',
'LongRightwardsArrowFromBar': '\xe2\x9f\xbc',
'HangulSyllableByoss': '\xeb\xb5\xb8',
'CircledNumberFifty': '\xe3\x8a\xbf',
'CjkUnifiedIdeograph5aec': '\xe5\xab\xac',
'CjkUnifiedIdeograph5aeb': '\xe5\xab\xab',
'CjkUnifiedIdeograph5aea': '\xe5\xab\xaa',
'HangulSyllableEonj': '\xec\x96\xb9',
'CjkUnifiedIdeograph5aef': '\xe5\xab\xaf',
'CjkUnifiedIdeograph5aee': '\xe5\xab\xae',
'CjkUnifiedIdeograph5aed': '\xe5\xab\xad',
'HangulJungseongEu': '\xe1\x85\xb3',
'BoxDrawingsLightHorizontal': '\xe2\x94\x80',
'ArabicLigatureKafWithLamInitialForm': '\xef\xb3\x87',
'HangulJungseongEo': '\xe1\x85\xa5',
'UnicodeCharacter0xe72b': '\xee\x9c\xab',
'CjkUnifiedIdeograph7f7f': '\xe7\xbd\xbf',
'CjkUnifiedIdeograph5ae3': '\xe5\xab\xa3',
'CjkUnifiedIdeograph5ae2': '\xe5\xab\xa2',
'CjkUnifiedIdeograph5ae1': '\xe5\xab\xa1',
'CjkUnifiedIdeograph5ae0': '\xe5\xab\xa0',
'CjkUnifiedIdeograph5ae7': '\xe5\xab\xa7',
'CjkUnifiedIdeograph5ae6': '\xe5\xab\xa6',
'CjkUnifiedIdeograph5ae5': '\xe5\xab\xa5',
'CjkUnifiedIdeograph5ae4': '\xe5\xab\xa4',
'HangulSyllablePyang': '\xed\x8d\x99',
'CjkUnifiedIdeograph5ae9': '\xe5\xab\xa9',
'CjkUnifiedIdeograph5ae8': '\xe5\xab\xa8',
'CjkUnifiedIdeograph7f7a': '\xe7\xbd\xba',
'UnicodeCharacter0xe524': '\xee\x94\xa4',
'UnicodeCharacter0xe72c': '\xee\x9c\xac',
'ArabicLetterHahWithSmallArabicLetterTahBelow': '\xdd\xae',
'ThaiCharacterMaiHanakat': '\xe0\xb8\xb1',
'HebrewLetterAlefWithMapiq': '\xef\xac\xb0',
'HangulSyllableTek': '\xed\x85\xa4',
'CjkUnifiedIdeograph9919': '\xe9\xa4\x99',
'HangulSyllableTyaenj': '\xed\x84\x99',
'HangulSyllableJwenj': '\xec\xa4\xb9',
'CjkUnifiedIdeograph8f23': '\xe8\xbc\xa3',
'HangulSyllableTyaeng': '\xed\x84\xa9',
'UnicodeCharacter0xf14b': '\xef\x85\x8b',
'UnicodeCharacter0xf883': '\xef\xa2\x83',
'CjkUnifiedIdeograph75e6': '\xe7\x97\xa6',
'UnicodeCharacter0xf881': '\xef\xa2\x81',
'UnicodeCharacter0xf886': '\xef\xa2\x86',
'UnicodeCharacter0xf887': '\xef\xa2\x87',
'UnicodeCharacter0xf884': '\xef\xa2\x84',
'DingbatCircledSansserifDigitOne': '\xe2\x9e\x80',
'UnicodeCharacter0xf1de': '\xef\x87\x9e',
'HangulSyllableTeo': '\xed\x84\xb0',
'UnicodeCharacter0xf888': '\xef\xa2\x88',
'UnicodeCharacter0xf889': '\xef\xa2\x89',
'MyanmarLetterEasternPwoKarenGhwa': '\xe1\x81\xb0',
'ModifierLetterDotHorizontalBar': '\xea\x9c\x99',
'csigma': '\xcf\x82',
'CjkUnifiedIdeograph8f24': '\xe8\xbc\xa4',
'UnicodeCharacter0xef0d': '\xee\xbc\x8d',
'UnicodeCharacter0xdeee': '\xed\xbb\xae',
'UnicodeCharacter0xe108': '\xee\x84\x88',
'CjkCompatibilityIdeographfac7': '\xef\xab\x87',
'UnicodeCharacter0xf2dd': '\xef\x8b\x9d',
'UnicodeCharacter0xf2de': '\xef\x8b\x9e',
'UnicodeCharacter0xf2df': '\xef\x8b\x9f',
'LimbuDigitZero': '\xe1\xa5\x86',
'UnicodeCharacter0xf2da': '\xef\x8b\x9a',
'UnicodeCharacter0xf1dd': '\xef\x87\x9d',
'HangulSyllableTeb': '\xed\x85\x9d',
'CjkUnifiedIdeograph949a': '\xe9\x92\x9a',
'HangulSyllableHum': '\xed\x9b\x94',
'HangulSyllableJjaels': '\xec\xa8\x84',
'UnicodeCharacter0xf88b': '\xef\xa2\x8b',
'UnicodeCharacter0xf88c': '\xef\xa2\x8c',
'HangulSyllableJwenh': '\xec\xa4\xba',
'UnicodeCharacter0xf88a': '\xef\xa2\x8a',
'UnicodeCharacter0xf88f': '\xef\xa2\x8f',
'RightDoubleParenthesis': '\xe2\xb8\xa9',
'UnicodeCharacter0xf88d': '\xef\xa2\x8d',
'YiSyllableLu': '\xea\x87\x90',
'YiSyllableLy': '\xea\x87\x96',
'UnicodeCharacter0xdad8': '\xed\xab\x98',
'HangulSyllableCij': '\xec\xb9\xae',
'HangulSyllableJjaelt': '\xec\xa8\x85',
'CjkCompatibilityIdeographf953': '\xef\xa5\x93',
'YiSyllableLa': '\xea\x87\x81',
'GreekCapitalLetterAlphaWithOxia': '\xe1\xbe\xbb',
'GreekCapitalLetterSho': '\xcf\xb7',
'YiSyllableLe': '\xea\x87\x8c',
'YiSyllableLi': '\xea\x86\xb9',
'YiSyllableLo': '\xea\x87\x89',
'YiSyllableJjiet': '\xea\x90\x9c',
'UnicodeCharacter0xf2d4': '\xef\x8b\x94',
'UnicodeCharacter0xf2d5': '\xef\x8b\x95',
'UnicodeCharacter0xf2d6': '\xef\x8b\x96',
'UnicodeCharacter0xf2d7': '\xef\x8b\x97',
'UnicodeCharacter0xf2d0': '\xef\x8b\x90',
'HangulSyllableJwa': '\xec\xa2\x8c',
'UnicodeCharacter0xf2d2': '\xef\x8b\x92',
'UnicodeCharacter0xf2d3': '\xef\x8b\x93',
'UnicodeCharacter0xe106': '\xee\x84\x86',
'LatinSmallLetterZWithMiddleTilde': '\xe1\xb5\xb6',
'UnicodeCharacter0xf2d8': '\xef\x8b\x98',
'UnicodeCharacter0xf2d9': '\xef\x8b\x99',
'UnicodeCharacter0xe107': '\xee\x84\x87',
'UnicodeCharacter0xf1db': '\xef\x87\x9b',
'ArmenianCapitalLetterFeh': '\xd5\x96',
'LatinCapitalLetterOWithDoubleGrave': '\xc8\x8c',
'ThaiCharacterNgoNgu': '\xe0\xb8\x87',
'RightwardsTwoheadedArrowWithTailWithVerticalStroke': '\xe2\xa4\x97',
'CjkUnifiedIdeograph950b': '\xe9\x94\x8b',
'HangulSyllableBbyaelh': '\xeb\xbb\x83',
'HangulSyllableSye': '\xec\x85\xb0',
'CircledKatakanaU': '\xe3\x8b\x92',
'HangulSyllableCyuls': '\xec\xb8\x90',
'ThaiCharacterMaiChattawa': '\xe0\xb9\x8b',
'HangulSyllableCyulp': '\xec\xb8\x92',
'DashedLowLine': '\xef\xb9\x8d',
'HangulSyllableCyult': '\xec\xb8\x91',
'BottomRightHalfBracket': '\xe2\xb8\xa5',
'CjkUnifiedIdeograph8d3b': '\xe8\xb4\xbb',
'CjkUnifiedIdeograph8d3a': '\xe8\xb4\xba',
'CircledKatakanaE': '\xe3\x8b\x93',
'CanadianSyllabicsFinalShortHorizontalStroke': '\xe1\x90\xa8',
'CjkUnifiedIdeograph8b94': '\xe8\xae\x94',
'CjkUnifiedIdeograph8d3e': '\xe8\xb4\xbe',
'CircledKatakanaA': '\xe3\x8b\x90',
'HangulSyllableNwaenj': '\xeb\x86\xb5',
'CircledKatakanaO': '\xe3\x8b\x94',
'HangulSyllableNwaenh': '\xeb\x86\xb6',
'HangulSyllableCyulg': '\xec\xb8\x8d',
'CircledKatakanaI': '\xe3\x8b\x91',
'HermitianConjugateMatrix': '\xe2\x8a\xb9',
'HangulSyllableJjaelb': '\xec\xa8\x83',
'HangulSyllableGwaek': '\xea\xb4\xb0',
'CjkCompatibilityIdeographf956': '\xef\xa5\x96',
'YiSyllableNrurx': '\xea\x8e\xa3',
'SuperscriptSeven': '\xe2\x81\xb7',
'HangulSyllableJjaelg': '\xec\xa8\x81',
'HangulSyllableRyaebs': '\xeb\x9f\xa2',
'CjkUnifiedIdeograph8d33': '\xe8\xb4\xb3',
'WhiteSquareWithLeftwardsTick': '\xe2\x9f\xa4',
'CjkUnifiedIdeograph8d31': '\xe8\xb4\xb1',
'CjkUnifiedIdeograph8d30': '\xe8\xb4\xb0',
'CjkUnifiedIdeograph8d37': '\xe8\xb4\xb7',
'CjkUnifiedIdeograph8d36': '\xe8\xb4\xb6',
'CjkUnifiedIdeograph8d35': '\xe8\xb4\xb5',
'CjkUnifiedIdeograph8d34': '\xe8\xb4\xb4',
'HangulSyllableCwaelp': '\xec\xb5\x8e',
'CjkUnifiedIdeograph8d39': '\xe8\xb4\xb9',
'CjkUnifiedIdeograph8d38': '\xe8\xb4\xb8',
'UnicodeCharacter0xdbea': '\xed\xaf\xaa',
'WhiteRightpointingTriangle': '\xe2\x96\xb7',
'UnicodeCharacter0xe6c9': '\xee\x9b\x89',
'BengaliLetterRaWithMiddleDiagonal': '\xe0\xa7\xb0',
'YiSyllableChap': '\xea\x8d\xa4',
'EthiopicSyllableTzaa': '\xe1\x8d\x83',
'YiSyllableChat': '\xea\x8d\xa1',
'ArabicLetterYehBarreeWithHamzaAboveIsolatedForm': '\xef\xae\xb0',
'ArabicLigatureYehWithHamzaAboveWithEIsolatedForm': '\xef\xaf\xb6',
'YiSyllableChax': '\xea\x8d\xa2',
'LatinSmallLetterHv': '\xc6\x95',
'WhiteSmallSquare': '\xe2\x96\xab',
'ArabicSignRahmatullahAlayhe': '\xd8\x92',
'UnicodeCharacter0x1cf6': '\xe1\xb3\xb6',
'UnicodeCharacter0x1cf7': '\xe1\xb3\xb7',
'UnicodeCharacter0x1cf4': '\xe1\xb3\xb4',
'UnicodeCharacter0x1cf5': '\xe1\xb3\xb5',
'UnicodeCharacter0x1cf2': '\xe1\xb3\xb2',
'UnicodeCharacter0x1cf3': '\xe1\xb3\xb3',
'UnicodeCharacter0x1cf0': '\xe1\xb3\xb0',
'UnicodeCharacter0x1cf1': '\xe1\xb3\xb1',
'CjkUnifiedIdeograph8c85': '\xe8\xb2\x85',
'CjkUnifiedIdeograph8c84': '\xe8\xb2\x84',
'CjkUnifiedIdeograph8c87': '\xe8\xb2\x87',
'CjkUnifiedIdeograph8c86': '\xe8\xb2\x86',
'CjkUnifiedIdeograph8c81': '\xe8\xb2\x81',
'CjkUnifiedIdeograph8c80': '\xe8\xb2\x80',
'UnicodeCharacter0x1cf8': '\xe1\xb3\xb8',
'UnicodeCharacter0x1cf9': '\xe1\xb3\xb9',
'CyrillicCapitalLetterUWithDiaeresis': '\xd3\xb0',
'SmallGreaterthanSign': '\xef\xb9\xa5',
'CjkUnifiedIdeograph950c': '\xe9\x94\x8c',
'OpenoutlinedRightwardsArrow': '\xe2\x9e\xbe',
'LatinCapitalLetterAWithCircumflexAndDotBelow': '\xe1\xba\xac',
'UnicodeCharacter0xf5d2': '\xef\x97\x92',
'LatinCapitalLetterDWithHook': '\xc6\x8a',
'YiSyllableSox': '\xea\x8c\x98',
'BoxDrawingsHeavyDownAndHorizontal': '\xe2\x94\xb3',
'UnicodeCharacter0xdbeb': '\xed\xaf\xab',
'YiSyllableSot': '\xea\x8c\x97',
'YiSyllableSop': '\xea\x8c\x9a',
'BengaliVowelSignUu': '\xe0\xa7\x82',
'UnicodeCharacter0xf30a': '\xef\x8c\x8a',
'AplFunctionalSymbolQuadLessthan': '\xe2\x8d\x83',
'UnicodeCharacter0xf30b': '\xef\x8c\x8b',
'UnicodeCharacter0xf30e': '\xef\x8c\x8e',
'UnicodeCharacter0xf30d': '\xef\x8c\x8d',
'UnicodeCharacter0xf30f': '\xef\x8c\x8f',
'HangulSyllableBbyus': '\xec\x81\x8f',
'ThaiDigitTwo': '\xe0\xb9\x92',
'ck': '\xcf\xb0',
'RomanNumeralFive': '\xe2\x85\xa4',
'UnicodeCharacter0x1cff': '\xe1\xb3\xbf',
'UnicodeCharacter0x1cfd': '\xe1\xb3\xbd',
'UnicodeCharacter0x1cfe': '\xe1\xb3\xbe',
'UnicodeCharacter0x1cfb': '\xe1\xb3\xbb',
'UnicodeCharacter0x1cfc': '\xe1\xb3\xbc',
'UnicodeCharacter0x1cfa': '\xe1\xb3\xba',
'CjkUnifiedIdeograph8c8e': '\xe8\xb2\x8e',
'CjkUnifiedIdeograph8c8d': '\xe8\xb2\x8d',
'CjkUnifiedIdeograph8c8f': '\xe8\xb2\x8f',
'CjkUnifiedIdeograph8c8a': '\xe8\xb2\x8a',
'HangulSyllableRwaegs': '\xeb\xa2\x97',
'CjkUnifiedIdeograph8c8c': '\xe8\xb2\x8c',
'CjkUnifiedIdeograph8c8b': '\xe8\xb2\x8b',
'UnicodeCharacter0xf309': '\xef\x8c\x89',
'UnicodeCharacter0xf308': '\xef\x8c\x88',
'HangulSyllableJwelt': '\xec\xa5\x81',
'ArabicLetterVehInitialForm': '\xef\xad\xac',
'UnicodeCharacter0xf301': '\xef\x8c\x81',
'UnicodeCharacter0xf300': '\xef\x8c\x80',
'UnicodeCharacter0xf303': '\xef\x8c\x83',
'UnicodeCharacter0xf302': '\xef\x8c\x82',
'UnicodeCharacter0xfd8': '\xe0\xbf\x98',
'UnicodeCharacter0xf304': '\xef\x8c\x84',
'UnicodeCharacter0xf307': '\xef\x8c\x87',
'CjkUnifiedIdeograph46f7': '\xe4\x9b\xb7',
'UnicodeCharacter0xebd5': '\xee\xaf\x95',
'UnicodeCharacter0xec39': '\xee\xb0\xb9',
'CjkUnifiedIdeograph46f4': '\xe4\x9b\xb4',
'CombiningDiaeresisBelow': '\xcc\xa4',
'HangulSyllableSat': '\xec\x83\x85',
'UnicodeCharacter0xebd7': '\xee\xaf\x97',
'CombiningCommaBelow': '\xcc\xa6',
'UnicodeCharacter0xebd6': '\xee\xaf\x96',
'CjkUnifiedIdeograph7b6d': '\xe7\xad\xad',
'CjkUnifiedIdeograph9914': '\xe9\xa4\x94',
'CjkUnifiedIdeograph9e4b': '\xe9\xb9\x8b',
'LatinSmallLetterUWithCaron': '\xc7\x94',
'CjkUnifiedIdeograph9e4d': '\xe9\xb9\x8d',
'CjkUnifiedIdeograph9e4e': '\xe9\xb9\x8e',
'CjkUnifiedIdeograph9e4f': '\xe9\xb9\x8f',
'CjkUnifiedIdeograph9e81': '\xe9\xba\x81',
'SquareKiro': '\xe3\x8c\x94',
'UnicodeCharacter0xebd3': '\xee\xaf\x93',
'UnicodeCharacter0xebd2': '\xee\xaf\x92',
'ArabicShaddaMedialForm': '\xef\xb9\xbd',
'UnicodeCharacter0xe14f': '\xee\x85\x8f',
'CjkUnifiedIdeograph781a': '\xe7\xa0\x9a',
'CjkUnifiedIdeograph781b': '\xe7\xa0\x9b',
'CjkUnifiedIdeograph781c': '\xe7\xa0\x9c',
'CjkUnifiedIdeograph781d': '\xe7\xa0\x9d',
'CjkUnifiedIdeograph781e': '\xe7\xa0\x9e',
'CjkUnifiedIdeograph781f': '\xe7\xa0\x9f',
'TibetanDigitThree': '\xe0\xbc\xa3',
'HangulSyllableCyaen': '\xec\xb2\x80',
'ArabicLigatureHehWithJeemIsolatedForm': '\xef\xb1\x91',
'CyrillicSmallLetterHaWithDescender': '\xd2\xb3',
'HangulLetterSsangcieuc': '\xe3\x85\x89',
'NumberSixteenFullStop': '\xe2\x92\x97',
'YiSyllableNdox': '\xea\x85\x9c',
'YiSyllableNdot': '\xea\x85\x9b',
'YiSyllableNdop': '\xea\x85\x9e',
'CjkUnifiedIdeograph9e48': '\xe9\xb9\x88',
'CjkUnifiedIdeograph9e49': '\xe9\xb9\x89',
'CanadianSyllabicsThcreeThoo': '\xe1\x96\xab',
'CjkUnifiedIdeograph9e40': '\xe9\xb9\x80',
'CjkUnifiedIdeograph9e41': '\xe9\xb9\x81',
'CjkUnifiedIdeograph9e42': '\xe9\xb9\x82',
'CjkUnifiedIdeograph9e43': '\xe9\xb9\x83',
'CjkUnifiedIdeograph9e44': '\xe9\xb9\x84',
'CjkUnifiedIdeograph9e45': '\xe9\xb9\x85',
'CjkUnifiedIdeograph9e46': '\xe9\xb9\x86',
'CjkUnifiedIdeograph9e47': '\xe9\xb9\x87',
'ChamPunctuationTripleDanda': '\xea\xa9\x9f',
'PhagspaLetterPa': '\xea\xa1\x8c',
'YiSyllableBbu': '\xea\x81\xae',
'CjkUnifiedIdeograph7810': '\xe7\xa0\x90',
'WhiteRightPointingIndex': '\xe2\x98\x9e',
'CjkUnifiedIdeograph7812': '\xe7\xa0\x92',
'CjkUnifiedIdeograph7813': '\xe7\xa0\x93',
'CjkUnifiedIdeograph7814': '\xe7\xa0\x94',
'CjkUnifiedIdeograph7815': '\xe7\xa0\x95',
'CjkUnifiedIdeograph7816': '\xe7\xa0\x96',
'CjkUnifiedIdeograph7817': '\xe7\xa0\x97',
'LatinSmallLetterLezh': '\xc9\xae',
'CjkUnifiedIdeograph7819': '\xe7\xa0\x99',
'UnicodeCharacter0xe148': '\xee\x85\x88',
'UnicodeCharacter0xe149': '\xee\x85\x89',
'KannadaVowelSignVocalicLl': '\xe0\xb3\xa3',
'CanadianSyllabicsFo': '\xe1\x95\x97',
'HalfwidthHangulLetterNieuncieuc': '\xef\xbe\xa5',
'CanadianSyllabicsFi': '\xe1\x95\x95',
'CanadianSyllabicsFe': '\xe1\x95\x93',
'CanadianSyllabicsFa': '\xe1\x95\x99',
'LatinSmallLetterSWithCircumflex': '\xc5\x9d',
'BraillePatternDots345678': '\xe2\xa3\xbc',
'OghamLetterTinne': '\xe1\x9a\x88',
'YiSyllableVix': '\xea\x83\xa3',
'HangulSyllableHyob': '\xed\x9a\xb9',
'CjkUnifiedIdeograph98a4': '\xe9\xa2\xa4',
'SquareMSquared': '\xe3\x8e\xa1',
'CircledIdeographHave': '\xe3\x8a\x92',
'UnicodeCharacter0xddbf': '\xed\xb6\xbf',
'UnicodeCharacter0xddbd': '\xed\xb6\xbd',
'UnicodeCharacter0xddbe': '\xed\xb6\xbe',
'UnicodeCharacter0xddbb': '\xed\xb6\xbb',
'UnicodeCharacter0xddbc': '\xed\xb6\xbc',
'CyrillicSmallLetterEnWithDescender': '\xd2\xa3',
'LatinSmallLetterOWithStroke': '\xc3\xb8',
'HangulSyllableCyaek': '\xec\xb2\x94',
'HangulSyllableHeop': '\xed\x97\xa2',
'HangulSyllableHeos': '\xed\x97\x9b',
'HangulSyllableHeot': '\xed\x97\xa1',
'ArabicLetterDalWithThreeDotsAboveDownwards': '\xda\x8f',
'CjkUnifiedIdeograph9815': '\xe9\xa0\x95',
'CjkUnifiedIdeograph93c9': '\xe9\x8f\x89',
'LatinLetterGlottalStop': '\xca\x94',
'DoubleVerticalLine': '\xe2\x80\x96',
'HangulSyllableBbyul': '\xec\x81\x84',
'ArabicLetterDdal': '\xda\x88',
'DingbatNegativeCircledDigitTwo': '\xe2\x9d\xb7',
'TibetanKuRuKhaBzhiMigCan': '\xe0\xbe\xbf',
'LeftRightBlackArrow': '\xe2\xac\x8c',
'HangulSyllableHeoh': '\xed\x97\xa3',
'ArmenianSmallLigatureEchYiwn': '\xd6\x87',
'HangulSyllableHeoj': '\xed\x97\x9e',
'ParenthesizedHangulKiyeok': '\xe3\x88\x80',
'CjkUnifiedIdeograph6f9e': '\xe6\xbe\x9e',
'HangulSyllableHeon': '\xed\x97\x8c',
'UnicodeCharacter0xe65a': '\xee\x99\x9a',
'MyanmarLetterNga': '\xe1\x80\x84',
'UnicodeCharacter0xddb8': '\xed\xb6\xb8',
'UnicodeCharacter0xdcab': '\xed\xb2\xab',
'UnicodeCharacter0xddb6': '\xed\xb6\xb6',
'UnicodeCharacter0xddb7': '\xed\xb6\xb7',
'UnicodeCharacter0xddb4': '\xed\xb6\xb4',
'UnicodeCharacter0xddb5': '\xed\xb6\xb5',
'UnicodeCharacter0xddb2': '\xed\xb6\xb2',
'UnicodeCharacter0xddb3': '\xed\xb6\xb3',
'UnicodeCharacter0xddb0': '\xed\xb6\xb0',
'UnicodeCharacter0xddb1': '\xed\xb6\xb1',
'HangulSyllableNwenj': '\xeb\x89\x81',
'HangulSyllableNwenh': '\xeb\x89\x82',
'CanadianSyllabicsNcreeThi': '\xe1\x95\xa1',
'CanadianSyllabicsNcreeThe': '\xe1\x95\x9f',
'HangulSyllableNweng': '\xeb\x89\x91',
'CjkUnifiedIdeograph6079': '\xe6\x81\xb9',
'CjkUnifiedIdeograph6078': '\xe6\x81\xb8',
'CjkUnifiedIdeograph638a': '\xe6\x8e\x8a',
'CjkUnifiedIdeograph957f': '\xe9\x95\xbf',
'CjkUnifiedIdeograph957a': '\xe9\x95\xba',
'CjkUnifiedIdeograph638f': '\xe6\x8e\x8f',
'CjkUnifiedIdeograph638e': '\xe6\x8e\x8e',
'CjkUnifiedIdeograph638d': '\xe6\x8e\x8d',
'CjkUnifiedIdeograph6071': '\xe6\x81\xb1',
'CjkUnifiedIdeograph6070': '\xe6\x81\xb0',
'CjkUnifiedIdeograph6073': '\xe6\x81\xb3',
'CjkUnifiedIdeograph6072': '\xe6\x81\xb2',
'CjkUnifiedIdeograph6075': '\xe6\x81\xb5',
'CjkUnifiedIdeograph6074': '\xe6\x81\xb4',
'CjkUnifiedIdeograph6077': '\xe6\x81\xb7',
'CjkUnifiedIdeograph6076': '\xe6\x81\xb6',
'MyanmarVowelSignKayahOe': '\xe1\x81\xb2',
'CjkUnifiedIdeograph79cb': '\xe7\xa7\x8b',
'CjkUnifiedIdeograph79ca': '\xe7\xa7\x8a',
'MalayalamLetterChilluK': '\xe0\xb5\xbf',
'CjkUnifiedIdeograph79cf': '\xe7\xa7\x8f',
'CjkUnifiedIdeograph79ce': '\xe7\xa7\x8e',
'CjkUnifiedIdeograph79cd': '\xe7\xa7\x8d',
'BengaliLetterTa': '\xe0\xa6\xa4',
'HangulSyllablePyok': '\xed\x91\xb4',
'OriyaLetterTtha': '\xe0\xac\xa0',
'MongolianLetterTa': '\xe1\xa0\xb2',
'Thunderstorm': '\xe2\x98\x88',
'PresentationFormForVerticalLeftAngleBracket': '\xef\xb8\xbf',
'CjkUnifiedIdeograph6af3': '\xe6\xab\xb3',
'CjkUnifiedIdeograph6af2': '\xe6\xab\xb2',
'CjkUnifiedIdeograph6af1': '\xe6\xab\xb1',
'CjkUnifiedIdeograph6af0': '\xe6\xab\xb0',
'CjkUnifiedIdeograph6af7': '\xe6\xab\xb7',
'CjkUnifiedIdeograph6af6': '\xe6\xab\xb6',
'CjkUnifiedIdeograph6af5': '\xe6\xab\xb5',
'CjkUnifiedIdeograph599f': '\xe5\xa6\x9f',
'HangulSyllableJjoegg': '\xec\xac\x92',
'CjkUnifiedIdeograph6af9': '\xe6\xab\xb9',
'CjkUnifiedIdeograph6af8': '\xe6\xab\xb8',
'CjkUnifiedIdeograph79c3': '\xe7\xa7\x83',
'CjkUnifiedIdeograph79c2': '\xe7\xa7\x82',
'CjkUnifiedIdeograph79c1': '\xe7\xa7\x81',
'CjkUnifiedIdeograph79c0': '\xe7\xa7\x80',
'CjkUnifiedIdeograph79c7': '\xe7\xa7\x87',
'CjkUnifiedIdeograph79c6': '\xe7\xa7\x86',
'CjkUnifiedIdeograph79c5': '\xe7\xa7\x85',
'CjkUnifiedIdeograph79c4': '\xe7\xa7\x84',
'MultiplicationSign': '\xc3\x97',
'CjkUnifiedIdeograph79c9': '\xe7\xa7\x89',
'CjkUnifiedIdeograph79c8': '\xe7\xa7\x88',
'UnicodeCharacter0xaa8d': '\xea\xaa\x8d',
'LatinLetterSmallCapitalEth': '\xe1\xb4\x86',
'CjkUnifiedIdeograph6382': '\xe6\x8e\x82',
'ArabicLetterWawWithRing': '\xdb\x84',
'CjkUnifiedIdeograph6380': '\xe6\x8e\x80',
'CjkUnifiedIdeograph6387': '\xe6\x8e\x87',
'CjkUnifiedIdeograph6386': '\xe6\x8e\x86',
'CjkUnifiedIdeograph6385': '\xe6\x8e\x85',
'CjkUnifiedIdeograph6384': '\xe6\x8e\x84',
'CjkUnifiedIdeograph607a': '\xe6\x81\xba',
'CjkUnifiedIdeograph607c': '\xe6\x81\xbc',
'CjkUnifiedIdeograph607b': '\xe6\x81\xbb',
'CjkUnifiedIdeograph607e': '\xe6\x81\xbe',
'CjkUnifiedIdeograph607d': '\xe6\x81\xbd',
'CjkUnifiedIdeograph607f': '\xe6\x81\xbf',
'CjkUnifiedIdeograph6afc': '\xe6\xab\xbc',
'CjkUnifiedIdeograph6afb': '\xe6\xab\xbb',
'CjkUnifiedIdeograph6afa': '\xe6\xab\xba',
'CjkUnifiedIdeograph6aff': '\xe6\xab\xbf',
'CjkUnifiedIdeograph6afe': '\xe6\xab\xbe',
'CjkUnifiedIdeograph6afd': '\xe6\xab\xbd',
'UnicodeCharacter0xaa8e': '\xea\xaa\x8e',
'RunicLetterHaeglH': '\xe1\x9a\xbb',
'CjkUnifiedIdeograph8257': '\xe8\x89\x97',
'HangulSyllableJjyolt': '\xec\xac\xb9',
'ModifierLetterSmallCapitalIWithStroke': '\xe1\xb6\xa7',
'CjkUnifiedIdeograph8256': '\xe8\x89\x96',
'ModifierLetterLowLeftArrowhead': '\xcb\xb1',
'LatinSmallLetterUWithDiaeresisBelow': '\xe1\xb9\xb3',
'HangulSyllableTyaess': '\xed\x84\xa8',
'CjkUnifiedIdeograph703b': '\xe7\x80\xbb',
'CjkUnifiedIdeograph703c': '\xe7\x80\xbc',
'CjkUnifiedIdeograph703a': '\xe7\x80\xba',
'CjkUnifiedIdeograph703f': '\xe7\x80\xbf',
'CjkUnifiedIdeograph703d': '\xe7\x80\xbd',
'CjkUnifiedIdeograph703e': '\xe7\x80\xbe',
'ThaanaLetterNoonu': '\xde\x82',
'CjkUnifiedIdeograph372b': '\xe3\x9c\xab',
'CjkUnifiedIdeograph372c': '\xe3\x9c\xac',
'CjkUnifiedIdeograph372a': '\xe3\x9c\xaa',
'CjkUnifiedIdeograph372f': '\xe3\x9c\xaf',
'CjkUnifiedIdeograph372d': '\xe3\x9c\xad',
'CjkUnifiedIdeograph372e': '\xe3\x9c\xae',
'CjkUnifiedIdeograph3bef': '\xe3\xaf\xaf',
'CjkUnifiedIdeograph3bed': '\xe3\xaf\xad',
'CjkUnifiedIdeograph3bee': '\xe3\xaf\xae',
'CjkUnifiedIdeograph3beb': '\xe3\xaf\xab',
'CjkUnifiedIdeograph3bec': '\xe3\xaf\xac',
'UnicodeCharacter0xda1d': '\xed\xa8\x9d',
'CjkUnifiedIdeograph3bea': '\xe3\xaf\xaa',
'CjkUnifiedIdeograph7388': '\xe7\x8e\x88',
'CjkUnifiedIdeograph7389': '\xe7\x8e\x89',
'RightwardsTwoheadedArrowWithDoubleVerticalStroke': '\xe2\xa4\x81',
'CjkUnifiedIdeograph7381': '\xe7\x8e\x81',
'CjkUnifiedIdeograph7382': '\xe7\x8e\x82',
'BalineseMusicalSymbolCombiningJegogan': '\xe1\xad\xaf',
'CjkUnifiedIdeograph7384': '\xe7\x8e\x84',
'CjkUnifiedIdeograph7385': '\xe7\x8e\x85',
'SundaneseLetterJa': '\xe1\xae\x8f',
'CjkUnifiedIdeograph7387': '\xe7\x8e\x87',
'CjkUnifiedIdeograph7032': '\xe7\x80\xb2',
'CjkUnifiedIdeograph7033': '\xe7\x80\xb3',
'CjkUnifiedIdeograph7030': '\xe7\x80\xb0',
'CjkUnifiedIdeograph7031': '\xe7\x80\xb1',
'CjkUnifiedIdeograph7036': '\xe7\x80\xb6',
'CjkUnifiedIdeograph7037': '\xe7\x80\xb7',
'CjkUnifiedIdeograph7034': '\xe7\x80\xb4',
'CjkUnifiedIdeograph7035': '\xe7\x80\xb5',
'NewTaiLueSignLae': '\xe1\xa7\x9e',
'CjkUnifiedIdeograph7038': '\xe7\x80\xb8',
'SquareMCubed': '\xe3\x8e\xa5',
'UnicodeCharacter0xf354': '\xef\x8d\x94',
'UnicodeCharacter0xebde': '\xee\xaf\x9e',
'UnicodeCharacter0xf355': '\xef\x8d\x95',
'MyanmarLetterShanFa': '\xe1\x81\xbe',
'CjkUnifiedIdeograph9e8e': '\xe9\xba\x8e',
'UnicodeCharacter0xf352': '\xef\x8d\x92',
'CjkUnifiedIdeograph5edd': '\xe5\xbb\x9d',
'UnicodeCharacter0xf8a0': '\xef\xa2\xa0',
'UnicodeCharacter0xf353': '\xef\x8d\x93',
'UnicodeCharacter0xddf9': '\xed\xb7\xb9',
'UnicodeCharacter0xeb3a': '\xee\xac\xba',
'UnicodeCharacter0xf350': '\xef\x8d\x90',
'CjkUnifiedIdeograph738a': '\xe7\x8e\x8a',
'CjkUnifiedIdeograph738b': '\xe7\x8e\x8b',
'CjkUnifiedIdeograph738c': '\xe7\x8e\x8c',
'CjkUnifiedIdeograph738d': '\xe7\x8e\x8d',
'CjkUnifiedIdeograph738e': '\xe7\x8e\x8e',
'CjkUnifiedIdeograph738f': '\xe7\x8e\x8f',
'CjkUnifiedIdeograph9e8a': '\xe9\xba\x8a',
'CjkUnifiedIdeograph3722': '\xe3\x9c\xa2',
'CjkUnifiedIdeograph3723': '\xe3\x9c\xa3',
'CjkUnifiedIdeograph3720': '\xe3\x9c\xa0',
'CjkUnifiedIdeograph3721': '\xe3\x9c\xa1',
'CjkUnifiedIdeograph3726': '\xe3\x9c\xa6',
'CjkUnifiedIdeograph3727': '\xe3\x9c\xa7',
'CjkUnifiedIdeograph3724': '\xe3\x9c\xa4',
'CjkUnifiedIdeograph3725': '\xe3\x9c\xa5',
'CjkUnifiedIdeograph3be6': '\xe3\xaf\xa6',
'CjkUnifiedIdeograph3be7': '\xe3\xaf\xa7',
'CjkUnifiedIdeograph3728': '\xe3\x9c\xa8',
'CjkUnifiedIdeograph3729': '\xe3\x9c\xa9',
'CjkUnifiedIdeograph3be2': '\xe3\xaf\xa2',
'CjkUnifiedIdeograph3be3': '\xe3\xaf\xa3',
'CjkUnifiedIdeograph3be0': '\xe3\xaf\xa0',
'CjkUnifiedIdeograph3be1': '\xe3\xaf\xa1',
'HangulJungseongYoo': '\xe1\x86\x87',
'HangulJungseongYoi': '\xe1\x86\x88',
'HangulSyllableMwagg': '\xeb\xab\x86',
'HangulSyllableBwigs': '\xeb\xb7\x97',
'PerpendicularWithS': '\xe2\xab\xa1',
'DevanagariLetterGhha': '\xe0\xa5\x9a',
'HangulSyllablePyoc': '\xed\x91\xb3',
'HangulSyllableMwags': '\xeb\xab\x87',
'CanadianSyllabicsWestcreeYwa': '\xe1\x94\xba',
'CanadianSyllabicsWestcreeYwe': '\xe1\x94\xb0',
'partial': '\xe2\x88\x82',
'CanadianSyllabicsWestcreeYwi': '\xe1\x94\xb2',
'CanadianSyllabicsWestcreeYwo': '\xe1\x94\xb6',
'GeorgianLetterFi': '\xe1\x83\xb6',
'CjkUnifiedIdeograph7a98': '\xe7\xaa\x98',
'UnicodeCharacter0xeb3c': '\xee\xac\xbc',
'HangulSyllableKyis': '\xed\x82\x9b',
'HangulSyllableKyip': '\xed\x82\xa2',
'UnicodeCharacter0xe6fe': '\xee\x9b\xbe',
'HangulSyllableKyit': '\xed\x82\xa1',
'HangulSyllableKyij': '\xed\x82\x9e',
'HangulSyllableKyik': '\xed\x82\xa0',
'HangulSyllableKyih': '\xed\x82\xa3',
'HangulSyllableKyin': '\xed\x82\x8c',
'HangulSyllableKyil': '\xed\x82\x90',
'HangulSyllableKyim': '\xed\x82\x98',
'HangulSyllableKyib': '\xed\x82\x99',
'HangulSyllableKyic': '\xed\x82\x9f',
'ArabicLigatureSheenWithRehFinalForm': '\xef\xb4\xa9',
'CombiningReverseSolidusOverlay': '\xe2\x83\xa5',
'HangulSyllableKyid': '\xed\x82\x8f',
'BlackDraughtsKing': '\xe2\x9b\x83',
'UnicodeCharacter0xdb92': '\xed\xae\x92',
'ModifierLetterVerticalLine': '\xcb\x88',
'CjkUnifiedIdeograph7a91': '\xe7\xaa\x91',
'GreekPhiSymbol': '\xcf\x95',
'CjkUnifiedIdeograph7a90': '\xe7\xaa\x90',
'KhmerLetterCho': '\xe1\x9e\x88',
'KhmerLetterCha': '\xe1\x9e\x86',
'NkoDigitOne': '\xdf\x81',
'BraillePatternDots28': '\xe2\xa2\x82',
'ArabicLetterHahFinalForm': '\xef\xba\xa2',
'HangulSyllableJyeng': '\xec\xa1\xa9',
'DoubleAcuteAccent': '\xcb\x9d',
'BraillePatternDots25': '\xe2\xa0\x92',
'BraillePatternDots24': '\xe2\xa0\x8a',
'BraillePatternDots27': '\xe2\xa1\x82',
'BraillePatternDots26': '\xe2\xa0\xa2',
'HangulSyllableHwanh': '\xed\x99\x9a',
'HangulSyllableHwanj': '\xed\x99\x99',
'CjkUnifiedIdeograph6039': '\xe6\x80\xb9',
'HangulSyllableBbyass': '\xeb\xba\xac',
'LatinCapitalLetterTresillo': '\xea\x9c\xaa',
'CircledNumberTen': '\xe2\x91\xa9',
'HangulSyllableYeong': '\xec\x98\x81',
'HangulSyllableHyeoh': '\xed\x98\x9b',
'CjkUnifiedIdeograph70d6': '\xe7\x83\x96',
'RightwardsArrowToBlackDiamond': '\xe2\xa4\x9e',
'HangulSyllableYeonh': '\xec\x97\xb2',
'HangulSyllableYeonj': '\xec\x97\xb1',
'CjkUnifiedIdeograph9027': '\xe9\x80\xa7',
'UnicodeCharacter0xf8a6': '\xef\xa2\xa6',
'CjkUnifiedIdeograph9026': '\xe9\x80\xa6',
'UnicodeCharacter0x8ed': '\xe0\xa3\xad',
'UnicodeCharacter0x8ee': '\xe0\xa3\xae',
'UnicodeCharacter0x8ef': '\xe0\xa3\xaf',
'UnicodeCharacter0x8ea': '\xe0\xa3\xaa',
'UnicodeCharacter0x8eb': '\xe0\xa3\xab',
'UnicodeCharacter0x8ec': '\xe0\xa3\xac',
'CjkUnifiedIdeograph6808': '\xe6\xa0\x88',
'CjkUnifiedIdeograph6809': '\xe6\xa0\x89',
'CjkCompatibilityIdeographf92a': '\xef\xa4\xaa',
'CjkUnifiedIdeograph9024': '\xe9\x80\xa4',
'HangulSyllableRass': '\xeb\x9e\x90',
'HangulSyllableDdolh': '\xeb\x98\x9f',
'CjkCompatibilityIdeographf92e': '\xef\xa4\xae',
'HangulSyllableCwaeg': '\xec\xb5\x81',
'CjkUnifiedIdeograph6800': '\xe6\xa0\x80',
'LatinCapitalLetterIota': '\xc6\x96',
'CjkUnifiedIdeograph6802': '\xe6\xa0\x82',
'CjkUnifiedIdeograph6803': '\xe6\xa0\x83',
'CjkUnifiedIdeograph6804': '\xe6\xa0\x84',
'CjkUnifiedIdeograph6805': '\xe6\xa0\x85',
'CjkUnifiedIdeograph6806': '\xe6\xa0\x86',
'CjkUnifiedIdeograph6807': '\xe6\xa0\x87',
'CjkUnifiedIdeograph9021': '\xe9\x80\xa1',
'HangulSyllableDdolt': '\xeb\x98\x9d',
'ArabicLigatureSadWithYehIsolatedForm': '\xef\xb4\x86',
'CjkUnifiedIdeograph9020': '\xe9\x80\xa0',
'HangulSyllableDdolp': '\xeb\x98\x9e',
'HangulSyllableDdols': '\xeb\x98\x9c',
'GujaratiLetterPha': '\xe0\xaa\xab',
'CjkUnifiedIdeograph3aaa': '\xe3\xaa\xaa',
'UnicodeCharacter0xe7ac': '\xee\x9e\xac',
'VaiSyllableSoo': '\xea\x96\x87',
'UnicodeCharacter0x8e8': '\xe0\xa3\xa8',
'UnicodeCharacter0x8e9': '\xe0\xa3\xa9',
'UnicodeCharacter0x8e4': '\xe0\xa3\xa4',
'UnicodeCharacter0x8e5': '\xe0\xa3\xa5',
'UnicodeCharacter0x8e6': '\xe0\xa3\xa6',
'UnicodeCharacter0x8e7': '\xe0\xa3\xa7',
'UnicodeCharacter0x8e0': '\xe0\xa3\xa0',
'UnicodeCharacter0x8e1': '\xe0\xa3\xa1',
'UnicodeCharacter0x8e2': '\xe0\xa3\xa2',
'UnicodeCharacter0x8e3': '\xe0\xa3\xa3',
'BraillePatternDots1467': '\xe2\xa1\xa9',
'HangulSyllableMugs': '\xeb\xac\xb7',
'BraillePatternDots1468': '\xe2\xa2\xa9',
'HalfwidthKatakanaLetterHu': '\xef\xbe\x8c',
'YiSyllableCix': '\xea\x8a\xaf',
'YiSyllableCit': '\xea\x8a\xae',
'HangulSyllableBbwaeb': '\xeb\xbe\x89',
'EthiopicSyllableSoa': '\xe2\xb6\x83',
'VaiSyllableDhhee': '\xea\x94\x91',
'YiSyllableCip': '\xea\x8a\xb1',
'CjkCompatibilityIdeographf923': '\xef\xa4\xa3',
'CjkCompatibilityIdeographf922': '\xef\xa4\xa2',
'CjkCompatibilityIdeographf921': '\xef\xa4\xa1',
'GlagoliticSmallLetterSpideryHa': '\xe2\xb1\x92',
'CjkCompatibilityIdeographf927': '\xef\xa4\xa7',
'CjkCompatibilityIdeographf926': '\xef\xa4\xa6',
'CjkCompatibilityIdeographf925': '\xef\xa4\xa5',
'CjkCompatibilityIdeographf924': '\xef\xa4\xa4',
'CjkUnifiedIdeograph680a': '\xe6\xa0\x8a',
'CjkUnifiedIdeograph680b': '\xe6\xa0\x8b',
'CjkUnifiedIdeograph680c': '\xe6\xa0\x8c',
'CjkUnifiedIdeograph680d': '\xe6\xa0\x8d',
'CjkUnifiedIdeograph680e': '\xe6\xa0\x8e',
'CjkUnifiedIdeograph680f': '\xe6\xa0\x8f',
'GujaratiLetterNa': '\xe0\xaa\xa8',
'CircledLessthan': '\xe2\xa7\x80',
'UnicodeCharacter0xe2de': '\xee\x8b\x9e',
'CjkUnifiedIdeograph5998': '\xe5\xa6\x98',
'UnicodeCharacter0xdace': '\xed\xab\x8e',
'emptyset': '\xe2\x88\x85',
'HangulSyllableHyeog': '\xed\x98\x81',
'CjkUnifiedIdeograph40df': '\xe4\x83\x9f',
'NkoLetterCha': '\xdf\x97',
'CjkUnifiedIdeograph40de': '\xe4\x83\x9e',
'CjkUnifiedIdeograph40db': '\xe4\x83\x9b',
'CjkUnifiedIdeograph40dc': '\xe4\x83\x9c',
'doubleintegral': '\xe2\x88\xac',
'CjkUnifiedIdeograph40da': '\xe4\x83\x9a',
'TibetanVowelSignReversedIi': '\xe0\xbe\x81',
'HangulSyllableDiss': '\xeb\x94\xa8',
'CjkUnifiedIdeograph5c90': '\xe5\xb2\x90',
'UnicodeCharacter0xf288': '\xef\x8a\x88',
'CjkUnifiedIdeograph70d0': '\xe7\x83\x90',
'HangulSyllableJjibs': '\xec\xb0\x9e',
'UnicodeCharacter0xefa7': '\xee\xbe\xa7',
'RunicLetterDagazDaegD': '\xe1\x9b\x9e',
'HangulSyllableYus': '\xec\x9c\xb3',
'HangulSyllableYup': '\xec\x9c\xba',
'HangulSyllableYut': '\xec\x9c\xb9',
'HangulSyllableYuj': '\xec\x9c\xb6',
'HangulSyllableYuk': '\xec\x9c\xb8',
'HangulSyllableYuh': '\xec\x9c\xbb',
'UnicodeCharacter0x9a9': '\xe0\xa6\xa9',
'SaurashtraSignAnusvara': '\xea\xa2\x80',
'CjkRadicalCsimplifiedSilk': '\xe2\xba\xb0',
'HangulSyllableYub': '\xec\x9c\xb1',
'HangulSyllableYuc': '\xec\x9c\xb7',
'HangulSyllableYug': '\xec\x9c\xa1',
'HangulSyllableYud': '\xec\x9c\xa7',
'HangulSyllableRwaenj': '\xeb\xa2\x99',
'CjkUnifiedIdeograph40d6': '\xe4\x83\x96',
'CjkUnifiedIdeograph40d7': '\xe4\x83\x97',
'CjkUnifiedIdeograph40d4': '\xe4\x83\x94',
'CjkUnifiedIdeograph40d5': '\xe4\x83\x95',
'CjkUnifiedIdeograph40d2': '\xe4\x83\x92',
'CjkUnifiedIdeograph40d3': '\xe4\x83\x93',
'CjkUnifiedIdeograph40d0': '\xe4\x83\x90',
'CjkUnifiedIdeograph40d1': '\xe4\x83\x91',
'HangulSyllableDdoelh': '\xeb\x99\xb3',
'LepchaPunctuationNyetThyoomTarol': '\xe1\xb0\xbc',
'CjkUnifiedIdeograph40d8': '\xe4\x83\x98',
'CjkUnifiedIdeograph40d9': '\xe4\x83\x99',
'BraillePatternDots3567': '\xe2\xa1\xb4',
'UnicodeCharacter0x109c': '\xe1\x82\x9c',
'LatinCapitalLetterLWithCaron': '\xc4\xbd',
'BoxDrawingsUpLightAndRightHeavy': '\xe2\x94\x95',
'BraillePatternDots3568': '\xe2\xa2\xb4',
'HangulSyllableHyeoc': '\xed\x98\x97',
'PhagspaLetterXa': '\xea\xa1\xa3',
'TaiLeLetterNa': '\xe1\xa5\xa2',
'UnicodeCharacter0xefa2': '\xee\xbe\xa2',
'YiSyllableZhap': '\xea\x8d\x89',
'YiSyllableZhat': '\xea\x8d\x86',
'YiSyllableZhax': '\xea\x8d\x87',
'UnicodeCharacter0xe3cf': '\xee\x8f\x8f',
'HangulSyllableSyubs': '\xec\x8a\x9a',
'BlackRightwardsBullet': '\xe2\x81\x8d',
'CjkUnifiedIdeograph603e': '\xe6\x80\xbe',
'CjkUnifiedIdeograph97fc': '\xe9\x9f\xbc',
'UnicodeCharacter0xec43': '\xee\xb1\x83',
'CjkUnifiedIdeograph902f': '\xe9\x80\xaf',
'UnicodeCharacter0xf35b': '\xef\x8d\x9b',
'KangxiRadicalFlute': '\xe2\xbf\x95',
'UnicodeCharacter0xe0d5': '\xee\x83\x95',
'UnicodeCharacter0xf35c': '\xef\x8d\x9c',
'HangulSyllableBbyolp': '\xeb\xbe\xbe',
'CjkUnifiedIdeograph902d': '\xe9\x80\xad',
'UnicodeCharacter0x1fb5': '\xe1\xbe\xb5',
'CjkUnifiedIdeograph902c': '\xe9\x80\xac',
'UnicodeCharacter0xf35a': '\xef\x8d\x9a',
'ArabicLetterHamza': '\xd8\xa1',
'CjkUnifiedIdeograph902b': '\xe9\x80\xab',
'HangulSyllablePyop': '\xed\x91\xb6',
'HangulSyllableDdoelm': '\xeb\x99\xae',
'CjkUnifiedIdeograph95c6': '\xe9\x97\x86',
'CjkUnifiedIdeograph902a': '\xe9\x80\xaa',
'LatinSmallLetterIWithCaron': '\xc7\x90',
'HangulSyllableKyeols': '\xec\xbc\xa8',
'UnicodeCharacter0xec23': '\xee\xb0\xa3',
'HangulSyllableKyeolp': '\xec\xbc\xaa',
'HangulSyllableKyeolt': '\xec\xbc\xa9',
'UpwardsTripleArrow': '\xe2\xa4\x8a',
'HangulSyllablePyos': '\xed\x91\xaf',
'TamilLetterOo': '\xe0\xae\x93',
'notelementof': '\xe2\x88\x89',
'UnicodeCharacter0xe52c': '\xee\x94\xac',
'NewTaiLueLetterHighNa': '\xe1\xa6\x90',
'ArabicLigatureSheenWithAlefMaksuraIsolatedForm': '\xef\xb3\xbd',
'HangulSyllableCweolp': '\xec\xb6\xbe',
'HangulSyllableCweols': '\xec\xb6\xbc',
'GreekDialytikaTonos': '\xce\x85',
'HangulSyllableCweolt': '\xec\xb6\xbd',
'HangulSyllableCweolb': '\xec\xb6\xbb',
'CyrillicSmallLetterYeru': '\xd1\x8b',
'CjkUnifiedIdeograph7a9d': '\xe7\xaa\x9d',
'CjkUnifiedIdeograph5b8f': '\xe5\xae\x8f',
'CjkUnifiedIdeograph5b8e': '\xe5\xae\x8e',
'CjkUnifiedIdeograph5b8d': '\xe5\xae\x8d',
'CjkUnifiedIdeograph5b8c': '\xe5\xae\x8c',
'CjkUnifiedIdeograph5b8b': '\xe5\xae\x8b',
'CjkUnifiedIdeograph5b8a': '\xe5\xae\x8a',
'UnicodeCharacter0xdd8d': '\xed\xb6\x8d',
'UnicodeCharacter0xf280': '\xef\x8a\x80',
'CjkUnifiedIdeograph70d8': '\xe7\x83\x98',
'CjkUnifiedIdeograph4d68': '\xe4\xb5\xa8',
'CjkUnifiedIdeograph4d69': '\xe4\xb5\xa9',
'UnicodeCharacter0xf145': '\xef\x85\x85',
'UnicodeCharacter0xe52e': '\xee\x94\xae',
'UnicodeCharacter0xf143': '\xef\x85\x83',
'UnicodeCharacter0xf142': '\xef\x85\x82',
'CjkUnifiedIdeograph7c05': '\xe7\xb0\x85',
'UnicodeCharacter0xf140': '\xef\x85\x80',
'CjkUnifiedIdeograph4d60': '\xe4\xb5\xa0',
'CjkUnifiedIdeograph4d61': '\xe4\xb5\xa1',
'CjkUnifiedIdeograph4d62': '\xe4\xb5\xa2',
'CjkUnifiedIdeograph4d63': '\xe4\xb5\xa3',
'CjkUnifiedIdeograph4d64': '\xe4\xb5\xa4',
'CjkUnifiedIdeograph4d65': '\xe4\xb5\xa5',
'CjkUnifiedIdeograph4d66': '\xe4\xb5\xa6',
'CjkUnifiedIdeograph4d67': '\xe4\xb5\xa7',
'UnicodeCharacter0xdf1b': '\xed\xbc\x9b',
'HangulSyllableBboss': '\xeb\xbd\x94',
'CjkUnifiedIdeograph5b89': '\xe5\xae\x89',
'CjkUnifiedIdeograph5b88': '\xe5\xae\x88',
'UnicodeCharacter0xe5e': '\xe0\xb9\x9e',
'UnicodeCharacter0xe5d': '\xe0\xb9\x9d',
'CanadianSyllabicsCarrierJee': '\xe1\x98\x98',
'UnicodeCharacter0xe5f': '\xe0\xb9\x9f',
'CjkUnifiedIdeograph5b83': '\xe5\xae\x83',
'CjkUnifiedIdeograph5b82': '\xe5\xae\x82',
'UnicodeCharacter0xe5c': '\xe0\xb9\x9c',
'CjkUnifiedIdeograph5b80': '\xe5\xae\x80',
'CanadianSyllabicsWestcreePwoo': '\xe1\x91\x83',
'CjkUnifiedIdeograph7c06': '\xe7\xb0\x86',
'UnicodeCharacter0xf762': '\xef\x9d\xa2',
'ChamLetterGa': '\xea\xa8\x88',
'UnicodeCharacter0xf14f': '\xef\x85\x8f',
'SyriacLetterReversedPe': '\xdc\xa7',
'UnicodeCharacter0xf14d': '\xef\x85\x8d',
'UnicodeCharacter0xf14c': '\xef\x85\x8c',
'CjkUnifiedIdeograph66c9': '\xe6\x9b\x89',
'UnicodeCharacter0xf14a': '\xef\x85\x8a',
'UnicodeCharacter0xaaea': '\xea\xab\xaa',
'CjkUnifiedIdeograph4d6a': '\xe4\xb5\xaa',
'CjkUnifiedIdeograph4d6b': '\xe4\xb5\xab',
'TagbanwaLetterWa': '\xe1\x9d\xaf',
'CjkUnifiedIdeograph4d6d': '\xe4\xb5\xad',
'CjkUnifiedIdeograph4d6e': '\xe4\xb5\xae',
'CjkUnifiedIdeograph4d6f': '\xe4\xb5\xaf',
'IdeographicAnnotationTwoMark': '\xe3\x86\x93',
'UnicodeCharacter0xf765': '\xef\x9d\xa5',
'UnicodeCharacter0xdc3a': '\xed\xb0\xba',
'CjkUnifiedIdeograph9e12': '\xe9\xb8\x92',
'UnicodeCharacter0xf283': '\xef\x8a\x83',
'GurmukhiLetterLla': '\xe0\xa8\xb3',
'CyrillicSmallLetterByelorussianukrainianI': '\xd1\x96',
'YiSyllableSyt': '\xea\x8c\xa4',
'CjkUnifiedIdeograph66c2': '\xe6\x9b\x82',
'UnicodeCharacter0xf234': '\xef\x88\xb4',
'CjkUnifiedIdeograph8f40': '\xe8\xbd\x80',
'ArabicLigatureShaddaWithFathaIsolatedForm': '\xef\xb1\xa0',
'ArabicindicDigitFive': '\xd9\xa5',
'CjkUnifiedIdeograph8950': '\xe8\xa5\x90',
'CjkUnifiedIdeograph8951': '\xe8\xa5\x91',
'CjkUnifiedIdeograph8952': '\xe8\xa5\x92',
'CjkUnifiedIdeograph8953': '\xe8\xa5\x93',
'CjkUnifiedIdeograph8954': '\xe8\xa5\x94',
'CjkUnifiedIdeograph8955': '\xe8\xa5\x95',
'CjkUnifiedIdeograph8956': '\xe8\xa5\x96',
'CjkUnifiedIdeograph8957': '\xe8\xa5\x97',
'CjkUnifiedIdeograph8958': '\xe8\xa5\x98',
'CjkUnifiedIdeograph8959': '\xe8\xa5\x99',
'HangulSyllableRoess': '\xeb\xa3\x84',
'CjkUnifiedIdeograph51d0': '\xe5\x87\x90',
'CjkUnifiedIdeograph51d1': '\xe5\x87\x91',
'CjkUnifiedIdeograph51d2': '\xe5\x87\x92',
'CjkUnifiedIdeograph51d3': '\xe5\x87\x93',
'CjkUnifiedIdeograph51d4': '\xe5\x87\x94',
'CjkUnifiedIdeograph51d5': '\xe5\x87\x95',
'CjkUnifiedIdeograph51d6': '\xe5\x87\x96',
'LepchaLetterDda': '\xe1\xb1\x8f',
'CjkUnifiedIdeograph51d8': '\xe5\x87\x98',
'CjkUnifiedIdeograph51d9': '\xe5\x87\x99',
'UnicodeCharacter0xf285': '\xef\x8a\x85',
'HangulSyllableYeon': '\xec\x97\xb0',
'HangulSyllableYeol': '\xec\x97\xb4',
'HangulSyllableYeom': '\xec\x97\xbc',
'CjkUnifiedIdeograph83c5': '\xe8\x8f\x85',
'HangulSyllableYeok': '\xec\x98\x84',
'YiSyllableSyr': '\xea\x8c\xa9',
'HangulSyllablePweng': '\xed\x93\x85',
'HangulSyllablePwenh': '\xed\x92\xb6',
'HangulSyllableYeog': '\xec\x97\xad',
'HangulSyllableYeod': '\xec\x97\xb3',
'HangulSyllableYeob': '\xec\x97\xbd',
'HangulSyllableYeoc': '\xec\x98\x83',
'HangulSyllableDyeoss': '\xeb\x8e\xa0',
'DoubleCircledDigitFour': '\xe2\x93\xb8',
'CyrillicCapitalLetterTsse': '\xea\x9a\x90',
'HangulSyllableYeos': '\xec\x97\xbf',
'MongolianLetterManchuRa': '\xe1\xa1\xb5',
'CjkUnifiedIdeograph51da': '\xe5\x87\x9a',
'VerticalZigzagLine': '\xe2\xa6\x9a',
'CjkUnifiedIdeograph51dc': '\xe5\x87\x9c',
'CjkUnifiedIdeograph51dd': '\xe5\x87\x9d',
'CyrillicSmallLetterKomiSje': '\xd4\x8d',
'CjkUnifiedIdeograph51df': '\xe5\x87\x9f',
'CjkUnifiedIdeograph895a': '\xe8\xa5\x9a',
'CjkUnifiedIdeograph895b': '\xe8\xa5\x9b',
'CjkUnifiedIdeograph895c': '\xe8\xa5\x9c',
'CjkUnifiedIdeograph895d': '\xe8\xa5\x9d',
'CjkUnifiedIdeograph895e': '\xe8\xa5\x9e',
'CjkUnifiedIdeograph895f': '\xe8\xa5\x9f',
'EuroSign': '\xe2\x82\xac',
'UnicodeCharacter0xf287': '\xef\x8a\x87',
'LongLeftwardsDoubleArrow': '\xe2\x9f\xb8',
'CjkUnifiedIdeograph5cdc': '\xe5\xb3\x9c',
'SyriacDottedZlamaHorizontal': '\xdc\xb8',
'CjkUnifiedIdeograph5cdf': '\xe5\xb3\x9f',
'CjkUnifiedIdeograph5cdd': '\xe5\xb3\x9d',
'CjkUnifiedIdeograph5cde': '\xe5\xb3\x9e',
'UnicodeCharacter0xe55a': '\xee\x95\x9a',
'CjkUnifiedIdeograph7dea': '\xe7\xb7\xaa',
'CjkUnifiedIdeograph7deb': '\xe7\xb7\xab',
'EthiopicSyllableCaa': '\xe1\x89\xbb',
'CjkUnifiedIdeograph7ded': '\xe7\xb7\xad',
'CjkUnifiedIdeograph7dee': '\xe7\xb7\xae',
'CjkUnifiedIdeograph7def': '\xe7\xb7\xaf',
'UnicodeCharacter0xe55f': '\xee\x95\x9f',
'VaiSyllableJi': '\xea\x95\x80',
'CjkUnifiedIdeograph7840': '\xe7\xa1\x80',
'VaiSyllableJo': '\xea\x97\x98',
'ThaiCharacterThoThung': '\xe0\xb8\x96',
'ArabicLigatureYehWithRehFinalForm': '\xef\xb2\x91',
'GreekCapitalLetterLamda': '\xce\x9b',
'UnicodeCharacter0xf237': '\xef\x88\xb7',
'CjkUnifiedIdeograph6fa9': '\xe6\xbe\xa9',
'CjkUnifiedIdeograph6fa8': '\xe6\xbe\xa8',
'CjkUnifiedIdeograph8376': '\xe8\x8d\xb6',
'CjkUnifiedIdeograph8377': '\xe8\x8d\xb7',
'CjkUnifiedIdeograph75e9': '\xe7\x97\xa9',
'CjkUnifiedIdeograph75e8': '\xe7\x97\xa8',
'CjkUnifiedIdeograph8372': '\xe8\x8d\xb2',
'CjkUnifiedIdeograph8373': '\xe8\x8d\xb3',
'CjkUnifiedIdeograph6fa1': '\xe6\xbe\xa1',
'CjkUnifiedIdeograph6fa0': '\xe6\xbe\xa0',
'CjkUnifiedIdeograph6fa3': '\xe6\xbe\xa3',
'CjkUnifiedIdeograph6fa2': '\xe6\xbe\xa2',
'SinhalaLetterVayanna': '\xe0\xb7\x80',
'CjkUnifiedIdeograph6fa4': '\xe6\xbe\xa4',
'CjkUnifiedIdeograph6fa7': '\xe6\xbe\xa7',
'CjkUnifiedIdeograph6fa6': '\xe6\xbe\xa6',
'CjkUnifiedIdeograph7de0': '\xe7\xb7\xa0',
'CjkUnifiedIdeograph7de1': '\xe7\xb7\xa1',
'CjkUnifiedIdeograph7de2': '\xe7\xb7\xa2',
'CjkUnifiedIdeograph7de3': '\xe7\xb7\xa3',
'NewTaiLueLetterLowSa': '\xe1\xa6\x8c',
'CjkUnifiedIdeograph7de5': '\xe7\xb7\xa5',
'CjkUnifiedIdeograph7de6': '\xe7\xb7\xa6',
'CjkUnifiedIdeograph7de7': '\xe7\xb7\xa7',
'CjkUnifiedIdeograph7de8': '\xe7\xb7\xa8',
'CjkUnifiedIdeograph7de9': '\xe7\xb7\xa9',
'UnicodeCharacter0xf7e8': '\xef\x9f\xa8',
'UnicodeCharacter0xf7e9': '\xef\x9f\xa9',
'CjkUnifiedIdeograph8848': '\xe8\xa1\x88',
'IdeographicTelegraphSymbolForHourTwelve': '\xe3\x8d\xa4',
'CjkUnifiedIdeograph7341': '\xe7\x8d\x81',
'CjkUnifiedIdeograph5cd2': '\xe5\xb3\x92',
'CjkUnifiedIdeograph5cd3': '\xe5\xb3\x93',
'CjkUnifiedIdeograph5cd0': '\xe5\xb3\x90',
'CjkUnifiedIdeograph5cd1': '\xe5\xb3\x91',
'CjkUnifiedIdeograph5cd6': '\xe5\xb3\x96',
'CjkUnifiedIdeograph5cd7': '\xe5\xb3\x97',
'CjkUnifiedIdeograph5cd4': '\xe5\xb3\x94',
'CjkUnifiedIdeograph5cd5': '\xe5\xb3\x95',
'CjkUnifiedIdeograph5cd8': '\xe5\xb3\x98',
'CjkUnifiedIdeograph5cd9': '\xe5\xb3\x99',
'CjkUnifiedIdeograph837d': '\xe8\x8d\xbd',
'CjkUnifiedIdeograph837e': '\xe8\x8d\xbe',
'CjkUnifiedIdeograph837f': '\xe8\x8d\xbf',
'CjkUnifiedIdeograph837a': '\xe8\x8d\xba',
'CjkUnifiedIdeograph837b': '\xe8\x8d\xbb',
'CjkUnifiedIdeograph837c': '\xe8\x8d\xbc',
'CanadianSyllabicsYaai': '\xe1\x94\xa7',
'CjkUnifiedIdeograph75ed': '\xe7\x97\xad',
'CjkUnifiedIdeograph6fac': '\xe6\xbe\xac',
'CjkUnifiedIdeograph6fab': '\xe6\xbe\xab',
'CjkUnifiedIdeograph6fae': '\xe6\xbe\xae',
'CjkUnifiedIdeograph6fad': '\xe6\xbe\xad',
'LatinCapitalLetterIWithGrave': '\xc3\x8c',
'CjkUnifiedIdeograph6faf': '\xe6\xbe\xaf',
'UnicodeCharacter0xf76c': '\xef\x9d\xac',
'UnicodeCharacter0xde6d': '\xed\xb9\xad',
'UnicodeCharacter0xf76b': '\xef\x9d\xab',
'UnicodeCharacter0xede3': '\xee\xb7\xa3',
'UnicodeCharacter0xde6c': '\xed\xb9\xac',
'ArabicLigatureTahWithYehFinalForm': '\xef\xb4\x92',
'CjkUnifiedIdeograph8840': '\xe8\xa1\x80',
'CjkUnifiedIdeograph5603': '\xe5\x98\x83',
'CjkUnifiedIdeograph5602': '\xe5\x98\x82',
'CjkUnifiedIdeograph5601': '\xe5\x98\x81',
'CjkUnifiedIdeograph5600': '\xe5\x98\x80',
'CjkUnifiedIdeograph5607': '\xe5\x98\x87',
'CjkUnifiedIdeograph5606': '\xe5\x98\x86',
'CjkUnifiedIdeograph5605': '\xe5\x98\x85',
'CjkUnifiedIdeograph5604': '\xe5\x98\x84',
'ArabicLigatureNoonWithMeemIsolatedForm': '\xef\xb1\x8e',
'CjkUnifiedIdeograph5609': '\xe5\x98\x89',
'CjkUnifiedIdeograph5608': '\xe5\x98\x88',
'UnicodeCharacter0x1a71': '\xe1\xa9\xb1',
'UnicodeCharacter0xf76f': '\xef\x9d\xaf',
'CjkUnifiedIdeograph9773': '\xe9\x9d\xb3',
'CjkUnifiedIdeograph9772': '\xe9\x9d\xb2',
'CjkUnifiedIdeograph9771': '\xe9\x9d\xb1',
'CjkUnifiedIdeograph9770': '\xe9\x9d\xb0',
'BraillePatternDots145678': '\xe2\xa3\xb9',
'CjkUnifiedIdeograph9776': '\xe9\x9d\xb6',
'CjkUnifiedIdeograph9775': '\xe9\x9d\xb5',
'CjkUnifiedIdeograph9774': '\xe9\x9d\xb4',
'CjkUnifiedIdeograph9779': '\xe9\x9d\xb9',
'CjkUnifiedIdeograph9778': '\xe9\x9d\xb8',
'GeorgianCapitalLetterCan': '\xe1\x82\xba',
'HangulSyllableNaet': '\xeb\x83\x8d',
'FullwidthLeftParenthesis': '\xef\xbc\x88',
'HangulSyllableNaep': '\xeb\x83\x8e',
'HangulSyllableNaes': '\xeb\x83\x87',
'CjkUnifiedIdeograph8f49': '\xe8\xbd\x89',
'HangulSyllableNaed': '\xeb\x82\xbb',
'HangulSyllableNaeg': '\xeb\x82\xb5',
'UnicodeCharacter0xf5df': '\xef\x97\x9f',
'HangulSyllableNaec': '\xeb\x83\x8b',
'HangulSyllableNaeb': '\xeb\x83\x85',
'HangulSyllableNaem': '\xeb\x83\x84',
'HangulSyllableNael': '\xeb\x82\xbc',
'HangulSyllableNaen': '\xeb\x82\xb8',
'HangulSyllableNaeh': '\xeb\x83\x8f',
'HangulSyllableNaek': '\xeb\x83\x8c',
'HangulSyllableNaej': '\xeb\x83\x8a',
'CjkUnifiedIdeograph977c': '\xe9\x9d\xbc',
'CjkUnifiedIdeograph977b': '\xe9\x9d\xbb',
'CjkUnifiedIdeograph977a': '\xe9\x9d\xba',
'TriangularBullet': '\xe2\x80\xa3',
'CjkUnifiedIdeograph977f': '\xe9\x9d\xbf',
'CjkUnifiedIdeograph977e': '\xe9\x9d\xbe',
'CjkUnifiedIdeograph977d': '\xe9\x9d\xbd',
'HangulSyllableKaelh': '\xec\xba\x9f',
'UnicodeCharacter0xeb8b': '\xee\xae\x8b',
'CjkUnifiedIdeograph560c': '\xe5\x98\x8c',
'CjkUnifiedIdeograph560b': '\xe5\x98\x8b',
'CjkUnifiedIdeograph560a': '\xe5\x98\x8a',
'CjkUnifiedIdeograph560f': '\xe5\x98\x8f',
'CjkUnifiedIdeograph560e': '\xe5\x98\x8e',
'CjkUnifiedIdeograph560d': '\xe5\x98\x8d',
'HangzhouNumeralThirty': '\xe3\x80\xba',
'HangulSyllableCoelp': '\xec\xb5\xaa',
'SquaredPlus': '\xe2\x8a\x9e',
'HangulSyllableUj': '\xec\x9b\x86',
'EthiopicSyllableQe': '\xe1\x89\x85',
'HangulSyllableUl': '\xec\x9a\xb8',
'CjkUnifiedIdeograph4538': '\xe4\x94\xb8',
'HangulSyllableUn': '\xec\x9a\xb4',
'EthiopicSyllableQa': '\xe1\x89\x80',
'EthiopicSyllableQo': '\xe1\x89\x86',
'HangulSyllableUb': '\xec\x9b\x81',
'HangulSyllableUc': '\xec\x9b\x87',
'HangulSyllableUd': '\xec\x9a\xb7',
'EthiopicSyllableQi': '\xe1\x89\x82',
'CjkUnifiedIdeograph4536': '\xe4\x94\xb6',
'EthiopicSyllableQu': '\xe1\x89\x81',
'HangulSyllableUp': '\xec\x9b\x8a',
'HangulSyllableUs': '\xec\x9b\x83',
'HangulSyllableUt': '\xec\x9b\x89',
'SundaneseLetterNa': '\xe1\xae\x94',
'MalayalamLetterGa': '\xe0\xb4\x97',
'HangulSyllableJwin': '\xec\xa5\x94',
'HangulSyllableJwim': '\xec\xa5\xa0',
'HangulSyllableJwil': '\xec\xa5\x98',
'HangulSyllableJwik': '\xec\xa5\xa8',
'HangulSyllableJwij': '\xec\xa5\xa6',
'CombiningLeftAngleBelow': '\xcd\x89',
'HangulSyllableJwih': '\xec\xa5\xab',
'HangulSyllableMyat': '\xeb\xa8\x99',
'CjkUnifiedIdeograph4532': '\xe4\x94\xb2',
'HangulSyllableJwid': '\xec\xa5\x97',
'HangulSyllableMyap': '\xeb\xa8\x9a',
'HangulSyllableJwib': '\xec\xa5\xa1',
'HangulSyllableMyas': '\xeb\xa8\x93',
'HangulSyllableMyal': '\xeb\xa8\x88',
'HangulSyllableMyam': '\xeb\xa8\x90',
'HangulSyllableMyan': '\xeb\xa8\x84',
'CjkUnifiedIdeograph7655': '\xe7\x99\x95',
'HangulSyllableMyah': '\xeb\xa8\x9b',
'HangulSyllableMyaj': '\xeb\xa8\x96',
'HangulSyllableMyak': '\xeb\xa8\x98',
'HangulSyllableMyad': '\xeb\xa8\x87',
'HangulSyllableMyae': '\xeb\xa8\x9c',
'HangulSyllableMyag': '\xeb\xa8\x81',
'HangulSyllableJwis': '\xec\xa5\xa3',
'HangulSyllableMyab': '\xeb\xa8\x91',
'HangulSyllableMyac': '\xeb\xa8\x97',
'HangulLetterKapyeounphieuph': '\xe3\x86\x84',
'UnicodeCharacter0xee6d': '\xee\xb9\xad',
'UnicodeCharacter0xee6e': '\xee\xb9\xae',
'MyanmarLetterVocalicRr': '\xe1\x81\x93',
'UnicodeCharacter0xee6a': '\xee\xb9\xaa',
'UnicodeCharacter0xee6b': '\xee\xb9\xab',
'UnicodeCharacter0xee6c': '\xee\xb9\xac',
'CjkUnifiedIdeograph3520': '\xe3\x94\xa0',
'CjkUnifiedIdeograph3521': '\xe3\x94\xa1',
'CjkUnifiedIdeograph3522': '\xe3\x94\xa2',
'CjkUnifiedIdeograph3523': '\xe3\x94\xa3',
'CjkUnifiedIdeograph3524': '\xe3\x94\xa4',
'CjkUnifiedIdeograph3525': '\xe3\x94\xa5',
'CjkUnifiedIdeograph3526': '\xe3\x94\xa6',
'CjkUnifiedIdeograph3527': '\xe3\x94\xa7',
'CjkUnifiedIdeograph3528': '\xe3\x94\xa8',
'CjkUnifiedIdeograph3529': '\xe3\x94\xa9',
'LaoLetterSoTam': '\xe0\xba\x8a',
'HangulSyllableOng': '\xec\x98\xb9',
'YiSyllableHuox': '\xea\x89\xb6',
'YiSyllableHuop': '\xea\x89\xb8',
'HangulSyllableCoelb': '\xec\xb5\xa7',
'YiSyllableHuot': '\xea\x89\xb5',
'YiSyllableFurx': '\xea\x83\x9c',
'HalfwidthKatakanaLetterSmallTu': '\xef\xbd\xaf',
'FullwidthDigitEight': '\xef\xbc\x98',
'HangulSyllableYals': '\xec\x96\x88',
'CjkUnifiedIdeograph352a': '\xe3\x94\xaa',
'CjkUnifiedIdeograph352b': '\xe3\x94\xab',
'CjkUnifiedIdeograph352c': '\xe3\x94\xac',
'CjkUnifiedIdeograph352d': '\xe3\x94\xad',
'CjkUnifiedIdeograph352e': '\xe3\x94\xae',
'CjkUnifiedIdeograph352f': '\xe3\x94\xaf',
'OpenSuperset': '\xe2\x9f\x84',
'CjkUnifiedIdeograph4c4f': '\xe4\xb1\x8f',
'HangulSyllableYalb': '\xec\x96\x87',
'CjkUnifiedIdeograph91fd': '\xe9\x87\xbd',
'DingbatCircledSansserifDigitFive': '\xe2\x9e\x84',
'UnicodeCharacter0xee69': '\xee\xb9\xa9',
'UnicodeCharacter0xee64': '\xee\xb9\xa4',
'UnicodeCharacter0xee65': '\xee\xb9\xa5',
'UnicodeCharacter0xde38': '\xed\xb8\xb8',
'HangulSyllableYalh': '\xec\x96\x8b',
'UnicodeCharacter0xee60': '\xee\xb9\xa0',
'UnicodeCharacter0xee61': '\xee\xb9\xa1',
'HangulSyllableYalm': '\xec\x96\x86',
'PrecedesOrEquivalentTo': '\xe2\x89\xbe',
'CanadianSyllabicsQaa': '\xe1\x96\x84',
'CjkUnifiedIdeograph3ba2': '\xe3\xae\xa2',
'HangulSyllableMyaep': '\xeb\xa8\xb6',
'HebrewLetterMem': '\xd7\x9e',
'LatinSmallLetterTurnedDelta': '\xc6\x8d',
'CjkUnifiedIdeograph3ba3': '\xe3\xae\xa3',
'CanadianSyllabicsQai': '\xe1\x99\xaf',
'LatinSmallLetterGWithHook': '\xc9\xa0',
'KatakanaDigraphKoto': '\xe3\x83\xbf',
'HangulSyllableByelt': '\xeb\xb3\xa5',
'HangulSyllableMyaeb': '\xeb\xa8\xad',
'HangulSyllableMyaec': '\xeb\xa8\xb3',
'UnicodeCharacter0xde64': '\xed\xb9\xa4',
'HangulSyllableMyaeg': '\xeb\xa8\x9d',
'HangulSyllableMyaed': '\xeb\xa8\xa3',
'CjkUnifiedIdeograph8cc1': '\xe8\xb3\x81',
'HangulSyllableMyaej': '\xeb\xa8\xb2',
'HangulSyllableMyaek': '\xeb\xa8\xb4',
'HangulSyllableMyaeh': '\xeb\xa8\xb7',
'HangulSyllableMyaen': '\xeb\xa8\xa0',
'HangulSyllableMyael': '\xeb\xa8\xa4',
'CjkUnifiedIdeograph3ba7': '\xe3\xae\xa7',
'HangulSyllableSswid': '\xec\x92\xbf',
'HangulSyllableGgyolp': '\xea\xbe\xaa',
'HangulSyllableGgyols': '\xea\xbe\xa8',
'HangulSyllableSswig': '\xec\x92\xb9',
'HangulSyllableGgyolt': '\xea\xbe\xa9',
'HangulSyllableSswib': '\xec\x93\x89',
'HangulSyllableSswic': '\xec\x93\x8f',
'CjkUnifiedIdeograph8a9d': '\xe8\xaa\x9d',
'CjkUnifiedIdeograph8a9e': '\xe8\xaa\x9e',
'CjkUnifiedIdeograph8a9f': '\xe8\xaa\x9f',
'UnicodeCharacter0xde60': '\xed\xb9\xa0',
'HangulSyllableSswih': '\xec\x93\x93',
'CjkUnifiedIdeograph8a9a': '\xe8\xaa\x9a',
'CjkUnifiedIdeograph8a9b': '\xe8\xaa\x9b',
'CjkUnifiedIdeograph8a9c': '\xe8\xaa\x9c',
'HangulSyllableSswit': '\xec\x93\x91',
'TradeMarkSign': '\xe2\x84\xa2',
'HangulSyllableSswip': '\xec\x93\x92',
'HangulSyllableGgyolg': '\xea\xbe\xa5',
'HangulSyllableSswis': '\xec\x93\x8b',
'SylotiNagriLetterGho': '\xea\xa0\x8a',
'HangulSyllableAgs': '\xec\x95\x87',
'HangulSyllableGgyolm': '\xea\xbe\xa6',
'CherokeeLetterU': '\xe1\x8e\xa4',
'UnicodeCharacter0xe20c': '\xee\x88\x8c',
'CjkUnifiedIdeograph9d31': '\xe9\xb4\xb1',
'DownwardsBlackArrow': '\xe2\xac\x87',
'ArabicLigatureSheenWithMeemWithMeemInitialForm': '\xef\xb5\xad',
'EthiopicSyllableThu': '\xe1\x8c\xa1',
'UnicodeCharacter0xfe6f': '\xef\xb9\xaf',
'EthiopicSyllableThi': '\xe1\x8c\xa2',
'EthiopicSyllableTho': '\xe1\x8c\xa6',
'EthiopicPrefaceColon': '\xe1\x8d\xa6',
'EthiopicSyllableTha': '\xe1\x8c\xa0',
'HangulSyllableCyilm': '\xec\xb9\x86',
'ArabicLigatureYehWithHehMedialForm': '\xef\xb3\xb1',
'EthiopicSyllableThe': '\xe1\x8c\xa5',
'HangulSyllableHwilt': '\xed\x9c\xa5',
'HangulSyllablePweobs': '\xed\x92\xa6',
'UpwardsWhiteDoubleArrowOnPedestal': '\xe2\x87\xaf',
'HangulSyllableMaec': '\xeb\xa7\xbb',
'CjkUnifiedIdeograph8a98': '\xe8\xaa\x98',
'CjkUnifiedIdeograph8a99': '\xe8\xaa\x99',
'CjkUnifiedIdeograph8a94': '\xe8\xaa\x94',
'CjkUnifiedIdeograph8a95': '\xe8\xaa\x95',
'CjkUnifiedIdeograph8a96': '\xe8\xaa\x96',
'CjkUnifiedIdeograph8a97': '\xe8\xaa\x97',
'CjkUnifiedIdeograph8a90': '\xe8\xaa\x90',
'CjkUnifiedIdeograph8a91': '\xe8\xaa\x91',
'CjkUnifiedIdeograph8a92': '\xe8\xaa\x92',
'CjkUnifiedIdeograph8a93': '\xe8\xaa\x93',
'HangulSyllableKut': '\xec\xbf\xb9',
'HangulSyllableKup': '\xec\xbf\xba',
'UnicodeCharacter0xe3b8': '\xee\x8e\xb8',
'HangulSyllableKus': '\xec\xbf\xb3',
'UnicodeCharacter0xeb35': '\xee\xac\xb5',
'HangulSyllableKud': '\xec\xbf\xa7',
'UnicodeCharacter0xde4a': '\xed\xb9\x8a',
'HangulSyllableKug': '\xec\xbf\xa1',
'ModifierLetterChineseToneYinQu': '\xea\x9c\x84',
'HangulSyllableKub': '\xec\xbf\xb1',
'HangulSyllableKuc': '\xec\xbf\xb7',
'HangulSyllableKul': '\xec\xbf\xa8',
'HangulSyllableKum': '\xec\xbf\xb0',
'HangulSyllableKun': '\xec\xbf\xa4',
'HexagramForTheWanderer': '\xe4\xb7\xb7',
'HangulSyllableKuh': '\xec\xbf\xbb',
'CjkUnifiedIdeograph734f': '\xe7\x8d\x8f',
'HangulSyllableKuj': '\xec\xbf\xb6',
'HangulSyllableKuk': '\xec\xbf\xb8',
'CjkUnifiedIdeograph8d3d': '\xe8\xb4\xbd',
'UnicodeCharacter0xe3b5': '\xee\x8e\xb5',
'HangulSyllableHung': '\xed\x9b\x99',
'HangulSyllableHunj': '\xed\x9b\x89',
'HangulSyllableGgwelp': '\xea\xbf\xbe',
'HangulSyllableGgwels': '\xea\xbf\xbc',
'CyrillicCapitalLetterIotifiedLittleYus': '\xd1\xa8',
'UnicodeCharacter0xeb36': '\xee\xac\xb6',
'HangulSyllableGgwelt': '\xea\xbf\xbd',
'UnicodeCharacter0xde36': '\xed\xb8\xb6',
'MultiplicationSignWithDotAbove': '\xe2\xa8\xb0',
'CjkUnifiedIdeograph799e': '\xe7\xa6\x9e',
'CjkUnifiedIdeograph799d': '\xe7\xa6\x9d',
'CjkUnifiedIdeograph799f': '\xe7\xa6\x9f',
'CjkUnifiedIdeograph799a': '\xe7\xa6\x9a',
'CjkUnifiedIdeograph734b': '\xe7\x8d\x8b',
'CjkUnifiedIdeograph799c': '\xe7\xa6\x9c',
'CjkUnifiedIdeograph799b': '\xe7\xa6\x9b',
'HangulSyllableSswibs': '\xec\x93\x8a',
'UnicodeCharacter0x20ba': '\xe2\x82\xba',
'UnicodeCharacter0x20bb': '\xe2\x82\xbb',
'UnicodeCharacter0x20bc': '\xe2\x82\xbc',
'UnicodeCharacter0x20bd': '\xe2\x82\xbd',
'UnicodeCharacter0x20be': '\xe2\x82\xbe',
'UnicodeCharacter0x20bf': '\xe2\x82\xbf',
'ArabicLigatureSheenWithMeemFinalForm': '\xef\xb4\xa8',
'YiSyllableNryx': '\xea\x8e\xa6',
'HangulSyllablePyilm': '\xed\x94\xaa',
'HangulSyllableDwegg': '\xeb\x92\x8a',
'YiSyllableNryr': '\xea\x8e\xaa',
'YiSyllableNryp': '\xea\x8e\xa8',
'UnicodeCharacter0xde37': '\xed\xb8\xb7',
'YiSyllableNryt': '\xea\x8e\xa5',
'HangulSyllableMyegg': '\xeb\xaa\x8e',
'HangulSyllableDwegs': '\xeb\x92\x8b',
'HangulSyllableJjeunj': '\xec\xaf\x99',
'EightPetalledOutlinedBlackFlorette': '\xe2\x9d\x81',
'UnicodeCharacter0x20b9': '\xe2\x82\xb9',
'CjkUnifiedIdeograph7997': '\xe7\xa6\x97',
'CjkUnifiedIdeograph7996': '\xe7\xa6\x96',
'CjkUnifiedIdeograph7991': '\xe7\xa6\x91',
'CjkUnifiedIdeograph7990': '\xe7\xa6\x90',
'CjkUnifiedIdeograph7993': '\xe7\xa6\x93',
'CjkUnifiedIdeograph7992': '\xe7\xa6\x92',
'UnicodeCharacter0xde30': '\xed\xb8\xb0',
'HangulSyllableJjeunh': '\xec\xaf\x9a',
'CjkUnifiedIdeograph7999': '\xe7\xa6\x99',
'CjkUnifiedIdeograph7998': '\xe7\xa6\x98',
'UnicodeCharacter0x20b6': '\xe2\x82\xb6',
'UnicodeCharacter0x20b7': '\xe2\x82\xb7',
'GeorgianSmallLetterGhan': '\xe2\xb4\x96',
'HangulSyllableJjeung': '\xec\xaf\xa9',
'CyrillicSmallLetterNje': '\xd1\x9a',
'HangulSyllableDdyel': '\xeb\x97\xbc',
'HangulSyllableDdyem': '\xeb\x98\x84',
'UnicodeCharacter0xde1d': '\xed\xb8\x9d',
'UnicodeCharacter0xde31': '\xed\xb8\xb1',
'<=': '\xe2\x89\xa4',
'HangulSyllableDdwaeng': '\xeb\x99\x9d',
'<-': '\xe2\x86\x90',
'HangulSyllableDdwaenj': '\xeb\x99\x8d',
'IdeographicTelegraphSymbolForDayNineteen': '\xe3\x8f\xb2',
'HangulSyllableDdwaenh': '\xeb\x99\x8e',
'HangulSyllableMeonh': '\xeb\xa8\xbe',
'YiSyllableRuo': '\xea\x8f\x8b',
'KatakanaLetterSmallHu': '\xe3\x87\xb7',
'UnicodeCharacter0xeb32': '\xee\xac\xb2',
'HangulSyllableJyolb': '\xec\xa3\xab',
'UnicodeCharacter0xeaad': '\xee\xaa\xad',
'CjkUnifiedIdeograph9169': '\xe9\x85\xa9',
'HangulSyllableMeong': '\xeb\xa9\x8d',
'ArabicLigatureThehWithYehFinalForm': '\xef\xb1\xbb',
'KatakanaLetterSmallHe': '\xe3\x87\xb8',
'CircledIdeographCopy': '\xe3\x8a\xa2',
'KatakanaLetterSmallHa': '\xe3\x87\xb5',
'KatakanaLetterSmallHo': '\xe3\x87\xb9',
'UnicodeCharacter0xe36b': '\xee\x8d\xab',
'KatakanaLetterSmallHi': '\xe3\x87\xb6',
'CjkUnifiedIdeograph8e7d': '\xe8\xb9\xbd',
'UnicodeCharacter0xde33': '\xed\xb8\xb3',
'<~': '\xe2\x86\x9c',
'UnicodeCharacter0xe6ca': '\xee\x9b\x8a',
'KhmerSignBariyoosan': '\xe1\x9f\x95',
'ArabicLetterNgIsolatedForm': '\xef\xaf\x93',
'GlagoliticCapitalLetterPokoji': '\xe2\xb0\x92',
'CjkUnifiedIdeograph453d': '\xe4\x94\xbd',
'SymbolForShiftOut': '\xe2\x90\x8e',
'HangulSyllableJyolt': '\xec\xa3\xad',
'GreekSubscriptSmallLetterRho': '\xe1\xb5\xa8',
'CanadianSyllabicsCarrierTtsee': '\xe1\x99\xaa',
'CyrillicCapitalLetterMonocularO': '\xea\x99\xa8',
'CjkUnifiedIdeograph869c': '\xe8\x9a\x9c',
'HangulSyllableSsong': '\xec\x8f\xad',
'HangulSyllableJyolp': '\xec\xa3\xae',
'CjkUnifiedIdeograph453a': '\xe4\x94\xba',
'UnicodeCharacter0xe5ea': '\xee\x97\xaa',
'HangulSyllableJyols': '\xec\xa3\xac',
'UnicodeCharacter0xdd76': '\xed\xb5\xb6',
'CjkUnifiedIdeograph582a': '\xe5\xa0\xaa',
'ChamDigitNine': '\xea\xa9\x99',
'UnicodeCharacter0xf232': '\xef\x88\xb2',
'CjkUnifiedIdeograph582e': '\xe5\xa0\xae',
'CjkUnifiedIdeograph622a': '\xe6\x88\xaa',
'CjkUnifiedIdeograph622b': '\xe6\x88\xab',
'CjkUnifiedIdeograph622c': '\xe6\x88\xac',
'CjkUnifiedIdeograph622d': '\xe6\x88\xad',
'CjkUnifiedIdeograph622e': '\xe6\x88\xae',
'CjkUnifiedIdeograph622f': '\xe6\x88\xaf',
'CjkUnifiedIdeograph97d9': '\xe9\x9f\x99',
'CjkUnifiedIdeograph6ccb': '\xe6\xb3\x8b',
'CjkUnifiedIdeograph6ccc': '\xe6\xb3\x8c',
'UnicodeCharacter0xe36f': '\xee\x8d\xaf',
'CjkUnifiedIdeograph6cca': '\xe6\xb3\x8a',
'CjkUnifiedIdeograph6ccf': '\xe6\xb3\x8f',
'CjkUnifiedIdeograph6ccd': '\xe6\xb3\x8d',
'CjkUnifiedIdeograph6cce': '\xe6\xb3\x8e',
'HangulSyllableTaegg': '\xed\x83\x9e',
'HangulSyllableRwilp': '\xeb\xa5\x8a',
'UnicodeCharacter0xde4c': '\xed\xb9\x8c',
'CircledPerpendicular': '\xe2\xa6\xb9',
'HangulLetterWeo': '\xe3\x85\x9d',
'UnicodeCharacter0xec78': '\xee\xb1\xb8',
'SinhalaVowelSignKombuDeka': '\xe0\xb7\x9b',
'VaiSymbolDang': '\xea\x98\x9a',
'SylotiNagriLetterHo': '\xea\xa0\xa2',
'NewTaiLueLetterLowHa': '\xe1\xa6\xa3',
'CjkUnifiedIdeograph7a2a': '\xe7\xa8\xaa',
'YiSyllableWu': '\xea\x80\x95',
'CjkUnifiedIdeograph7a2b': '\xe7\xa8\xab',
'BlackPentagon': '\xe2\xac\x9f',
'CjkUnifiedIdeograph6cc3': '\xe6\xb3\x83',
'CjkUnifiedIdeograph6cc0': '\xe6\xb3\x80',
'CjkUnifiedIdeograph6cc1': '\xe6\xb3\x81',
'CjkUnifiedIdeograph6cc6': '\xe6\xb3\x86',
'CjkUnifiedIdeograph6cc7': '\xe6\xb3\x87',
'CjkUnifiedIdeograph6cc4': '\xe6\xb3\x84',
'CjkUnifiedIdeograph6cc5': '\xe6\xb3\x85',
'HangulSyllableJjyalm': '\xec\xa8\x9e',
'CjkUnifiedIdeograph7a2d': '\xe7\xa8\xad',
'CjkUnifiedIdeograph6cc8': '\xe6\xb3\x88',
'CjkUnifiedIdeograph6cc9': '\xe6\xb3\x89',
'HangulSyllableJjyalh': '\xec\xa8\xa3',
'TibetanLetterDza': '\xe0\xbd\x9b',
'CjkUnifiedIdeograph7a2e': '\xe7\xa8\xae',
'CjkUnifiedIdeograph6228': '\xe6\x88\xa8',
'CjkUnifiedIdeograph6229': '\xe6\x88\xa9',
'UnicodeCharacter0xe372': '\xee\x8d\xb2',
'CjkUnifiedIdeograph7a2f': '\xe7\xa8\xaf',
'HangulSyllableJjyals': '\xec\xa8\xa0',
'CjkUnifiedIdeograph6220': '\xe6\x88\xa0',
'CjkUnifiedIdeograph6221': '\xe6\x88\xa1',
'CjkUnifiedIdeograph6222': '\xe6\x88\xa2',
'CjkUnifiedIdeograph6223': '\xe6\x88\xa3',
'CjkUnifiedIdeograph6224': '\xe6\x88\xa4',
'CjkUnifiedIdeograph6225': '\xe6\x88\xa5',
'CjkUnifiedIdeograph6226': '\xe6\x88\xa6',
'CjkUnifiedIdeograph6227': '\xe6\x88\xa7',
'KayahLiToneCalya': '\xea\xa4\xac',
'CjkUnifiedIdeograph52c8': '\xe5\x8b\x88',
'UnicodeCharacter0xe376': '\xee\x8d\xb6',
'UnicodeCharacter0xe36e': '\xee\x8d\xae',
'ArabicLetterHahWithSmallArabicLetterTahAndTwoDots': '\xdd\xaf',
'CjkUnifiedIdeograph4cde': '\xe4\xb3\x9e',
'CjkUnifiedIdeograph4cdd': '\xe4\xb3\x9d',
'CjkUnifiedIdeograph4cdf': '\xe4\xb3\x9f',
'CjkUnifiedIdeograph4cda': '\xe4\xb3\x9a',
'CjkUnifiedIdeograph4cdc': '\xe4\xb3\x9c',
'CjkUnifiedIdeograph4cdb': '\xe4\xb3\x9b',
'HangulSyllableSseun': '\xec\x93\xb4',
'CjkUnifiedIdeograph717f': '\xe7\x85\xbf',
'NkoLetterTa': '\xdf\x95',
'LimbuVowelSignI': '\xe1\xa4\xa1',
'CjkUnifiedIdeograph717c': '\xe7\x85\xbc',
'LimbuVowelSignO': '\xe1\xa4\xa8',
'CjkUnifiedIdeograph717a': '\xe7\x85\xba',
'YiSyllableYur': '\xea\x92\x86',
'EightPointedPinwheelStar': '\xe2\x9c\xb5',
'CjkUnifiedIdeograph869e': '\xe8\x9a\x9e',
'HangulSyllableNyelp': '\xeb\x85\xaa',
'LimbuVowelSignA': '\xe1\xa4\xa0',
'YiSyllableYux': '\xea\x92\x82',
'HangulSyllableNyelt': '\xeb\x85\xa9',
'LimbuVowelSignE': '\xe1\xa4\xa7',
'CjkUnifiedIdeograph388f': '\xe3\xa2\x8f',
'CjkUnifiedIdeograph388e': '\xe3\xa2\x8e',
'CjkUnifiedIdeograph388d': '\xe3\xa2\x8d',
'CjkUnifiedIdeograph388c': '\xe3\xa2\x8c',
'CjkUnifiedIdeograph388b': '\xe3\xa2\x8b',
'CjkUnifiedIdeograph388a': '\xe3\xa2\x8a',
'HangulSyllableNyelm': '\xeb\x85\xa6',
'HangulSyllableNyelb': '\xeb\x85\xa7',
'YiSyllableYuo': '\xea\x91\xbb',
'HangulSyllableNyelg': '\xeb\x85\xa5',
'LimbuVowelSignU': '\xe1\xa4\xa2',
'CjkUnifiedIdeograph4cd5': '\xe4\xb3\x95',
'CjkUnifiedIdeograph4cd4': '\xe4\xb3\x94',
'CjkUnifiedIdeograph4cd7': '\xe4\xb3\x97',
'CjkUnifiedIdeograph4cd6': '\xe4\xb3\x96',
'CjkUnifiedIdeograph4cd1': '\xe4\xb3\x91',
'CjkUnifiedIdeograph4cd0': '\xe4\xb3\x90',
'CjkUnifiedIdeograph4cd3': '\xe4\xb3\x93',
'SymbolForDelete': '\xe2\x90\xa1',
'HangulSyllableDdwij': '\xeb\x9c\x86',
'CjkUnifiedIdeograph4cd9': '\xe4\xb3\x99',
'TibetanAstrologicalSignSdongTshugs': '\xe0\xbc\x99',
'ArabicLigatureYehWithHahWithYehFinalForm': '\xef\xb6\xae',
'contourintegral': '\xe2\x88\xae',
'TibetanLetterGha': '\xe0\xbd\x83',
'CjkUnifiedIdeograph3887': '\xe3\xa2\x87',
'UpwardsArrowFromBar': '\xe2\x86\xa5',
'CjkUnifiedIdeograph3885': '\xe3\xa2\x85',
'CjkUnifiedIdeograph3884': '\xe3\xa2\x84',
'CjkUnifiedIdeograph3883': '\xe3\xa2\x83',
'CjkUnifiedIdeograph3882': '\xe3\xa2\x82',
'CjkUnifiedIdeograph3881': '\xe3\xa2\x81',
'CjkUnifiedIdeograph3880': '\xe3\xa2\x80',
'EqualToByDefinition': '\xe2\x89\x9d',
'CjkUnifiedIdeograph659e': '\xe6\x96\x9e',
'CjkUnifiedIdeograph3889': '\xe3\xa2\x89',
'CjkUnifiedIdeograph3888': '\xe3\xa2\x88',
'EthiopicSyllableLa': '\xe1\x88\x88',
'CjkUnifiedIdeograph7176': '\xe7\x85\xb6',
'CjkUnifiedIdeograph7175': '\xe7\x85\xb5',
'CjkUnifiedIdeograph7174': '\xe7\x85\xb4',
'EthiopicSyllableLe': '\xe1\x88\x8d',
'DownwardsPairedArrows': '\xe2\x87\x8a',
'CjkUnifiedIdeograph7171': '\xe7\x85\xb1',
'CjkUnifiedIdeograph7170': '\xe7\x85\xb0',
'EthiopicSyllableLi': '\xe1\x88\x8a',
'ArabicLigatureHahWithJeemInitialForm': '\xef\xb2\xa9',
'HangulSyllableJjweonh': '\xec\xad\xaa',
'HangulSyllableJogs': '\xec\xa1\xb3',
'EthiopicSyllableLo': '\xe1\x88\x8e',
'CjkUnifiedIdeograph7178': '\xe7\x85\xb8',
'GreekCapitalLetterRhoWithDasia': '\xe1\xbf\xac',
'CherokeeLetterTsi': '\xe1\x8f\xa5',
'UnicodeCharacter0x1b7d': '\xe1\xad\xbd',
'UnicodeCharacter0x1b7e': '\xe1\xad\xbe',
'UnicodeCharacter0x1b7f': '\xe1\xad\xbf',
'CherokeeLetterTsa': '\xe1\x8f\xa3',
'RightTriangleAboveLeftTriangle': '\xe2\xa7\x8e',
'ArabicLigatureYehWithNoonFinalForm': '\xef\xb2\x94',
'CjkUnifiedIdeograph9239': '\xe9\x88\xb9',
'HangulSyllableSseuj': '\xec\x94\x86',
'CherokeeLetterTsu': '\xe1\x8f\xa7',
'CherokeeLetterTsv': '\xe1\x8f\xa8',
'UnicodeCharacter0xdca8': '\xed\xb2\xa8',
'UnicodeCharacter0xef05': '\xee\xbc\x85',
'UnicodeCharacter0xef04': '\xee\xbc\x84',
'UnicodeCharacter0xef07': '\xee\xbc\x87',
'UnicodeCharacter0xef06': '\xee\xbc\x86',
'UnicodeCharacter0xef01': '\xee\xbc\x81',
'UnicodeCharacter0xef00': '\xee\xbc\x80',
'UnicodeCharacter0xef03': '\xee\xbc\x83',
'UnicodeCharacter0xef02': '\xee\xbc\x82',
'UnicodeCharacter0xf865': '\xef\xa1\xa5',
'BengaliLetterMa': '\xe0\xa6\xae',
'UnicodeCharacter0xef09': '\xee\xbc\x89',
'UnicodeCharacter0xef08': '\xee\xbc\x88',
'ModifierLetterRaisedColon': '\xcb\xb8',
'UnicodeCharacter0xe442': '\xee\x91\x82',
'<-<': '\xe2\x86\xa2',
'GreekSmallLetterKoppa': '\xcf\x9f',
'UnicodeCharacter0xdca9': '\xed\xb2\xa9',
'HangulSyllableHwilp': '\xed\x9c\xa6',
'YiSyllableZzyt': '\xea\x8b\xa7',
'YiSyllableZzyr': '\xea\x8b\xac',
'YiSyllableZzyp': '\xea\x8b\xaa',
'UnicodeCharacter0xe447': '\xee\x91\x87',
'LatinSmallLetterOWithGrave': '\xc3\xb2',
'ParenthesizedDigitEight': '\xe2\x91\xbb',
'HangulSyllableGeonh': '\xea\xb1\xb6',
'LatinSmallLetterPWithFlourish': '\xea\x9d\x93',
'UnicodeCharacter0xe444': '\xee\x91\x84',
'LatinSmallLetterUWithDiaeresisAndCaron': '\xc7\x9a',
'UnicodeCharacter0xefac': '\xee\xbe\xac',
'TibetanSubjoinedLetterSha': '\xe0\xbe\xb4',
'ArabicLigatureSheenWithHehMedialForm': '\xef\xb3\xaa',
'AplFunctionalSymbolDownCaretTilde': '\xe2\x8d\xb1',
'ArabicLigatureLamWithJeemWithYehFinalForm': '\xef\xb6\xac',
'UnicodeCharacter0xf4cb': '\xef\x93\x8b',
'UnicodeCharacter0xef0e': '\xee\xbc\x8e',
'CjkUnifiedIdeograph7a20': '\xe7\xa8\xa0',
'UnicodeCharacter0xef0f': '\xee\xbc\x8f',
'UnicodeCharacter0xef0a': '\xee\xbc\x8a',
'UnicodeCharacter0xef0c': '\xee\xbc\x8c',
'CjkUnifiedIdeograph7a21': '\xe7\xa8\xa1',
'GreekSmallLetterSigma': '\xcf\x83',
'CjkUnifiedIdeograph7a22': '\xe7\xa8\xa2',
'GreaterthanOrSlantedEqualToWithDotAbove': '\xe2\xaa\x82',
'HangulSyllableJilh': '\xec\xa7\x8f',
'GreekSmallLetterIotaWithDialytika': '\xcf\x8a',
'HangulSyllableDac': '\xeb\x8b\xbb',
'VaiSyllableWan': '\xea\x95\x8f',
'HangulSyllableJilm': '\xec\xa7\x8a',
'HangulSyllableJilb': '\xec\xa7\x8b',
'CjkUnifiedIdeograph7a24': '\xe7\xa8\xa4',
'HangulSyllableJilg': '\xec\xa7\x89',
'CjkUnifiedIdeograph7a25': '\xe7\xa8\xa5',
'CjkUnifiedIdeograph6a1f': '\xe6\xa8\x9f',
'HalfwidthKatakanaLetterU': '\xef\xbd\xb3',
'CjkUnifiedIdeograph6a1d': '\xe6\xa8\x9d',
'CjkUnifiedIdeograph6a1e': '\xe6\xa8\x9e',
'CjkUnifiedIdeograph6a1b': '\xe6\xa8\x9b',
'CjkUnifiedIdeograph6a1c': '\xe6\xa8\x9c',
'CircledIdeographWood': '\xe3\x8a\x8d',
'CjkUnifiedIdeograph6a1a': '\xe6\xa8\x9a',
'HangulSyllableJils': '\xec\xa7\x8c',
'HangulSyllableJilp': '\xec\xa7\x8e',
'CjkUnifiedIdeograph7a27': '\xe7\xa8\xa7',
'HangulSyllableJilt': '\xec\xa7\x8d',
'HangulSyllableKeolp': '\xec\xbb\xb2',
'CjkUnifiedIdeograph6a8a': '\xe6\xaa\x8a',
'HangulSyllableKeols': '\xec\xbb\xb0',
'UnicodeCharacter0xe37f': '\xee\x8d\xbf',
'NewTaiLueSignLaev': '\xe1\xa7\x9f',
'ArabicLigatureNoonWithAlefMaksuraIsolatedForm': '\xef\xb1\x8f',
'HangulSyllableSseug': '\xec\x93\xb1',
'LatinCapitalLetterLWithMiddleTilde': '\xe2\xb1\xa2',
'UnicodeCharacter0xeba5': '\xee\xae\xa5',
'CjkUnifiedIdeograph6a16': '\xe6\xa8\x96',
'CjkUnifiedIdeograph6a17': '\xe6\xa8\x97',
'DoublelineSlantedGreaterthanOrEqualTo': '\xe2\xab\xba',
'CjkUnifiedIdeograph6a15': '\xe6\xa8\x95',
'CjkUnifiedIdeograph6a12': '\xe6\xa8\x92',
'CjkUnifiedIdeograph6a13': '\xe6\xa8\x93',
'CjkUnifiedIdeograph6a10': '\xe6\xa8\x90',
'CjkUnifiedIdeograph6a11': '\xe6\xa8\x91',
'QuadrantUpperRight': '\xe2\x96\x9d',
'HangulSyllableSseud': '\xec\x93\xb7',
'CjkUnifiedIdeograph6a18': '\xe6\xa8\x98',
'CjkUnifiedIdeograph6a19': '\xe6\xa8\x99',
'EthiopicSyllableHoa': '\xe1\x88\x87',
'LineIntegrationWithRectangularPathAroundPole': '\xe2\xa8\x92',
'HangulSyllableSwilh': '\xec\x89\xbb',
'HangulSyllableSsug': '\xec\x91\xa5',
'CyrillicCapitalLetterKomiDzje': '\xd4\x86',
'HangulJongseongPhieuph': '\xe1\x87\x81',
'PresentationFormForVerticalLowLine': '\xef\xb8\xb3',
'HangulSyllableSwilm': '\xec\x89\xb6',
'LimbuSmallLetterTa': '\xe1\xa4\xb3',
'HangulSyllableSae': '\xec\x83\x88',
'CjkUnifiedIdeograph59ec': '\xe5\xa7\xac',
'dot': '\xe2\x8b\x85',
'CjkUnifiedIdeograph59eb': '\xe5\xa7\xab',
'CjkUnifiedIdeograph5827': '\xe5\xa0\xa7',
'ParenthesizedIdeographMoon': '\xe3\x88\xaa',
'HangulSyllableYanh': '\xec\x96\x82',
'Mercury': '\xe2\x98\xbf',
'CjkUnifiedIdeograph6332': '\xe6\x8c\xb2',
'EqualAndParallelTo': '\xe2\x8b\x95',
'CjkUnifiedIdeograph7a99': '\xe7\xaa\x99',
'LatinSmallLetterDWithPalatalHook': '\xe1\xb6\x81',
'CjkUnifiedIdeograph6333': '\xe6\x8c\xb3',
'CjkUnifiedIdeograph7a95': '\xe7\xaa\x95',
'CjkUnifiedIdeograph7a94': '\xe7\xaa\x94',
'CjkUnifiedIdeograph7a97': '\xe7\xaa\x97',
'CjkUnifiedIdeograph7a96': '\xe7\xaa\x96',
'WaveArrowPointingDirectlyLeft': '\xe2\xac\xbf',
'CjkUnifiedIdeograph6330': '\xe6\x8c\xb0',
'CjkUnifiedIdeograph7a93': '\xe7\xaa\x93',
'CjkUnifiedIdeograph7a92': '\xe7\xaa\x92',
'UnicodeCharacter0xee27': '\xee\xb8\xa7',
'CjkUnifiedIdeograph6331': '\xe6\x8c\xb1',
'PesetaSign': '\xe2\x82\xa7',
'CjkUnifiedIdeograph6336': '\xe6\x8c\xb6',
'CjkUnifiedIdeograph6337': '\xe6\x8c\xb7',
'ExtendedArabicindicDigitThree': '\xdb\xb3',
'CjkUnifiedIdeograph6334': '\xe6\x8c\xb4',
'HangulSyllableRyogs': '\xeb\xa3\x8f',
'CjkUnifiedIdeograph6335': '\xe6\x8c\xb5',
'HangulSyllableMwelg': '\xeb\xad\xb5',
'HangulSyllableGgwalh': '\xea\xbd\x97',
'SquarePaatu': '\xe3\x8c\xac',
'CanadianSyllabicsWestcreeWe': '\xe1\x90\x8d',
'LeftwardsHarpoonOverRightwardsHarpoon': '\xe2\x87\x8b',
'elementof': '\xe2\x88\x88',
'ArmenianSmallLetterTo': '\xd5\xa9',
'CjkUnifiedIdeograph8bca': '\xe8\xaf\x8a',
'EthiopicSyllableKwi': '\xe1\x8a\xb2',
'CjkUnifiedIdeograph7a9e': '\xe7\xaa\x9e',
'EthiopicSyllableKwe': '\xe1\x8a\xb5',
'CjkUnifiedIdeograph7a9f': '\xe7\xaa\x9f',
'CjkUnifiedIdeograph7a9a': '\xe7\xaa\x9a',
'EthiopicSyllableKwa': '\xe1\x8a\xb0',
'CjkUnifiedIdeograph7a9c': '\xe7\xaa\x9c',
'CjkUnifiedIdeograph7a9b': '\xe7\xaa\x9b',
'AustralSign': '\xe2\x82\xb3',
'CjkUnifiedIdeograph8bcb': '\xe8\xaf\x8b',
'GreekSmallLetterEpsilonWithDasiaAndOxia': '\xe1\xbc\x95',
'CjkUnifiedIdeograph8bce': '\xe8\xaf\x8e',
'YiSyllableBie': '\xea\x80\x9c',
'CjkUnifiedIdeograph8bcd': '\xe8\xaf\x8d',
'UnicodeCharacter0x2bcb': '\xe2\xaf\x8b',
'UnicodeCharacter0xaa86': '\xea\xaa\x86',
'CjkUnifiedIdeograph8bcf': '\xe8\xaf\x8f',
'LatinCapitalLetterUWithDoubleGrave': '\xc8\x94',
'GreekSmallLetterEtaWithDasiaAndPerispomeni': '\xe1\xbc\xa7',
'HangulSyllableNyonj': '\xeb\x87\xad',
'UnicodeCharacter0x2bcf': '\xe2\xaf\x8f',
'SmallComma': '\xef\xb9\x90',
'UnicodeCharacter0xaa87': '\xea\xaa\x87',
'HangulSyllableNyels': '\xeb\x85\xa8',
'TibetanMarkDelimiterTshegBstar': '\xe0\xbc\x8c',
'HeavyRightwardsArrow': '\xe2\x9e\x99',
'CjkUnifiedIdeograph5e1b': '\xe5\xb8\x9b',
'UnicodeCharacter0xf4c0': '\xef\x93\x80',
'HangulSyllableNyaelt': '\xeb\x83\xb9',
'TagbanwaLetterBa': '\xe1\x9d\xaa',
'HangulSyllableJjyulm': '\xec\xaf\x82',
'RightOneEighthBlock': '\xe2\x96\x95',
'HangulSyllableMeoss': '\xeb\xa9\x8c',
'YiSyllableBip': '\xea\x80\x99',
'HangulSyllableNyaels': '\xeb\x83\xb8',
'UnicodeCharacter0xaa84': '\xea\xaa\x84',
'CjkUnifiedIdeograph5e8a': '\xe5\xba\x8a',
'CjkUnifiedIdeograph5e8b': '\xe5\xba\x8b',
'CjkUnifiedIdeograph5e8c': '\xe5\xba\x8c',
'CjkUnifiedIdeograph5e8d': '\xe5\xba\x8d',
'CjkUnifiedIdeograph5e8e': '\xe5\xba\x8e',
'CjkUnifiedIdeograph5e8f': '\xe5\xba\x8f',
'CjkUnifiedIdeograph6596': '\xe6\x96\x96',
'CjkUnifiedIdeograph5e1d': '\xe5\xb8\x9d',
'HangulSyllableGgess': '\xea\xbb\xac',
'UnicodeCharacter0xdca6': '\xed\xb2\xa6',
'UnicodeCharacter0xaae6': '\xea\xab\xa6',
'CjkUnifiedIdeograph5716': '\xe5\x9c\x96',
'YiSyllableBit': '\xea\x80\x96',
'HangulSyllableNyonh': '\xeb\x87\xae',
'ApproachesTheLimit': '\xe2\x89\x90',
'HangulSyllableCyabs': '\xec\xb1\xb2',
'HangulSyllableDdwaet': '\xeb\x99\xa1',
'LatinCapitalLetterUWithHornAndDotBelow': '\xe1\xbb\xb0',
'HangulSyllableBbyelt': '\xeb\xbc\xb1',
'CjkUnifiedIdeograph5e80': '\xe5\xba\x80',
'CjkUnifiedIdeograph5e81': '\xe5\xba\x81',
'CjkUnifiedIdeograph5e82': '\xe5\xba\x82',
'CjkUnifiedIdeograph5e83': '\xe5\xba\x83',
'CjkUnifiedIdeograph5e84': '\xe5\xba\x84',
'CjkUnifiedIdeograph5e85': '\xe5\xba\x85',
'CjkUnifiedIdeograph5e86': '\xe5\xba\x86',
'CjkUnifiedIdeograph5e87': '\xe5\xba\x87',
'CjkUnifiedIdeograph5e88': '\xe5\xba\x88',
'CjkUnifiedIdeograph5e89': '\xe5\xba\x89',
'CjkUnifiedIdeograph5c29': '\xe5\xb0\xa9',
'CyrillicCapitalLetterAWithBreve': '\xd3\x90',
'HangulSyllablePaebs': '\xed\x8c\xba',
'HangulSyllableMyegs': '\xeb\xaa\x8f',
'OlChikiDigitNine': '\xe1\xb1\x99',
'CjkUnifiedIdeograph582c': '\xe5\xa0\xac',
'CjkUnifiedIdeograph582b': '\xe5\xa0\xab',
'HiraganaLetterNa': '\xe3\x81\xaa',
'CjkUnifiedIdeograph582f': '\xe5\xa0\xaf',
'HiraganaLetterNe': '\xe3\x81\xad',
'CjkUnifiedIdeograph582d': '\xe5\xa0\xad',
'HiraganaLetterNi': '\xe3\x81\xab',
'HiraganaLetterNo': '\xe3\x81\xae',
'UnicodeCharacter0xe371': '\xee\x8d\xb1',
'UnicodeCharacter0xe370': '\xee\x8d\xb0',
'UnicodeCharacter0xe373': '\xee\x8d\xb3',
'HangulSyllableBbog': '\xeb\xbd\x81',
'UnicodeCharacter0xe375': '\xee\x8d\xb5',
'UnicodeCharacter0xe374': '\xee\x8d\xb4',
'HiraganaLetterNu': '\xe3\x81\xac',
'CjkUnifiedIdeograph52c9': '\xe5\x8b\x89',
'CjkUnifiedIdeograph52c6': '\xe5\x8b\x86',
'CjkUnifiedIdeograph52c7': '\xe5\x8b\x87',
'CjkUnifiedIdeograph52c4': '\xe5\x8b\x84',
'CjkUnifiedIdeograph52c5': '\xe5\x8b\x85',
'CjkUnifiedIdeograph52c2': '\xe5\x8b\x82',
'CjkUnifiedIdeograph52c3': '\xe5\x8b\x83',
'CjkUnifiedIdeograph52c0': '\xe5\x8b\x80',
'CjkUnifiedIdeograph52c1': '\xe5\x8b\x81',
'CjkUnifiedIdeograph5c2d': '\xe5\xb0\xad',
'CjkUnifiedIdeograph5c2e': '\xe5\xb0\xae',
'CjkUnifiedIdeograph5c2f': '\xe5\xb0\xaf',
'CjkUnifiedIdeograph5c2a': '\xe5\xb0\xaa',
'CjkUnifiedIdeograph5c2b': '\xe5\xb0\xab',
'CjkUnifiedIdeograph5c2c': '\xe5\xb0\xac',
'UnicodeCharacter0xaa80': '\xea\xaa\x80',
'SubsetWithDot': '\xe2\xaa\xbd',
'ArabicLigatureGhainWithYehIsolatedForm': '\xef\xb3\xba',
'SinhalaVowelSignKombuvaHaaDigaAelapilla': '\xe0\xb7\x9d',
'UnicodeCharacter0xe44f': '\xee\x91\x8f',
'CjkUnifiedIdeograph768e': '\xe7\x9a\x8e',
'MongolianLetterAliGaliKa': '\xe1\xa2\x89',
'HangulSyllableBbyelb': '\xeb\xbc\xaf',
'UnicodeCharacter0xe44e': '\xee\x91\x8e',
'CjkUnifiedIdeograph9402': '\xe9\x90\x82',
'HangulSyllableJogg': '\xec\xa1\xb2',
'ArabicLigatureYehWithHamzaAboveWithHehInitialForm': '\xef\xb2\x9b',
'UnicodeCharacter0xe37a': '\xee\x8d\xba',
'HangulSyllableKeolt': '\xec\xbb\xb1',
'UnicodeCharacter0xe37c': '\xee\x8d\xbc',
'UnicodeCharacter0xe37b': '\xee\x8d\xbb',
'UnicodeCharacter0xe37e': '\xee\x8d\xbe',
'HangulSyllableBbyelg': '\xeb\xbc\xad',
'UnicodeCharacter0xaa81': '\xea\xaa\x81',
'SquareImageOfOrNotEqualTo': '\xe2\x8b\xa4',
'CjkUnifiedIdeograph52cf': '\xe5\x8b\x8f',
'DoubleIntegral': '\xe2\x88\xac',
'CjkUnifiedIdeograph52cd': '\xe5\x8b\x8d',
'CjkUnifiedIdeograph52ce': '\xe5\x8b\x8e',
'CjkUnifiedIdeograph52cb': '\xe5\x8b\x8b',
'CjkUnifiedIdeograph52cc': '\xe5\x8b\x8c',
'CjkUnifiedIdeograph52ca': '\xe5\x8b\x8a',
'CjkUnifiedIdeograph5823': '\xe5\xa0\xa3',
'CjkUnifiedIdeograph5822': '\xe5\xa0\xa2',
'CjkUnifiedIdeograph5821': '\xe5\xa0\xa1',
'CjkUnifiedIdeograph5820': '\xe5\xa0\xa0',
'IdeographicIterationMark': '\xe3\x80\x85',
'CjkUnifiedIdeograph5826': '\xe5\xa0\xa6',
'CjkUnifiedIdeograph50d3': '\xe5\x83\x93',
'CjkUnifiedIdeograph5824': '\xe5\xa0\xa4',
'HangulSyllableKeolm': '\xec\xbb\xae',
'CjkUnifiedIdeograph5829': '\xe5\xa0\xa9',
'CjkUnifiedIdeograph5828': '\xe5\xa0\xa8',
'HangulSyllableKeolh': '\xec\xbb\xb3',
'HangulSyllableDdwael': '\xeb\x99\x90',
'HangulSyllableDdwaem': '\xeb\x99\x98',
'HangulSyllableDdwaen': '\xeb\x99\x8c',
'HangulSyllableDdwaeh': '\xeb\x99\xa3',
'HangulSyllableDdwaej': '\xeb\x99\x9e',
'HangulSyllableDdwaek': '\xeb\x99\xa0',
'HangulSyllableDdwaed': '\xeb\x99\x8f',
'HangulSyllableDdwaeg': '\xeb\x99\x89',
'HangulSyllableDdwaeb': '\xeb\x99\x99',
'HangulSyllableDdwaec': '\xeb\x99\x9f',
'CjkUnifiedIdeograph5c24': '\xe5\xb0\xa4',
'CjkUnifiedIdeograph5c25': '\xe5\xb0\xa5',
'CjkUnifiedIdeograph5c26': '\xe5\xb0\xa6',
'CjkUnifiedIdeograph5c27': '\xe5\xb0\xa7',
'CjkUnifiedIdeograph5c20': '\xe5\xb0\xa0',
'CjkUnifiedIdeograph5c21': '\xe5\xb0\xa1',
'CjkUnifiedIdeograph5c22': '\xe5\xb0\xa2',
'CjkUnifiedIdeograph5c23': '\xe5\xb0\xa3',
'CombiningMacronacute': '\xe1\xb7\x84',
'CjkCompatibilityIdeographf96c': '\xef\xa5\xac',
'ArabicLetterRehWithTwoDotsAbove': '\xda\x97',
'CjkUnifiedIdeograph5c28': '\xe5\xb0\xa8',
'LatinSmallLetterLWithDotBelowAndMacron': '\xe1\xb8\xb9',
'HangulSyllableDdwaes': '\xeb\x99\x9b',
'CjkUnifiedIdeograph815c': '\xe8\x85\x9c',
'NkoDigitSix': '\xdf\x86',
'MalayalamVowelSignEe': '\xe0\xb5\x87',
'CjkUnifiedIdeograph8bc5': '\xe8\xaf\x85',
'ArabicLigatureHahWithYehIsolatedForm': '\xef\xb4\x80',
'CjkUnifiedIdeograph5e19': '\xe5\xb8\x99',
'CjkUnifiedIdeograph8bc4': '\xe8\xaf\x84',
'CjkUnifiedIdeograph5e18': '\xe5\xb8\x98',
'CjkUnifiedIdeograph8bc7': '\xe8\xaf\x87',
'HangulSyllableSswags': '\xec\x8f\xb7',
'HangulSyllableBit': '\xeb\xb9\x9d',
'LatinSmallLetterRWithPalatalHook': '\xe1\xb6\x89',
'CjkUnifiedIdeograph99b9': '\xe9\xa6\xb9',
'UnicodeCharacter0xf488': '\xef\x92\x88',
'UnicodeCharacter0xf489': '\xef\x92\x89',
'UnicodeCharacter0xf486': '\xef\x92\x86',
'UnicodeCharacter0xf487': '\xef\x92\x87',
'UnicodeCharacter0xf484': '\xef\x92\x84',
'HangulSyllableMyenh': '\xeb\xaa\x92',
'HangulSyllableSseut': '\xec\x94\x89',
'UnicodeCharacter0xf483': '\xef\x92\x83',
'UnicodeCharacter0xf480': '\xef\x92\x80',
'UnicodeCharacter0xf481': '\xef\x92\x81',
'CjkUnifiedIdeograph65b3': '\xe6\x96\xb3',
'CjkUnifiedIdeograph5e13': '\xe5\xb8\x93',
'CjkUnifiedIdeograph65b1': '\xe6\x96\xb1',
'CjkUnifiedIdeograph65b0': '\xe6\x96\xb0',
'CjkUnifiedIdeograph65b7': '\xe6\x96\xb7',
'CjkUnifiedIdeograph65b6': '\xe6\x96\xb6',
'CjkUnifiedIdeograph65b5': '\xe6\x96\xb5',
'CjkUnifiedIdeograph5e12': '\xe5\xb8\x92',
'CjkUnifiedIdeograph65b9': '\xe6\x96\xb9',
'CjkUnifiedIdeograph65b8': '\xe6\x96\xb8',
'LeftwardsArrowWithTipUpwards': '\xe2\xac\x91',
'TeluguFractionDigitOneForOddPowersOfFour': '\xe0\xb1\xb9',
'HangulSyllableMyiss': '\xeb\xaf\xb0',
'Q': '\xce\x98',
'CjkUnifiedIdeograph59e2': '\xe5\xa7\xa2',
'HangulLetterAe': '\xe3\x85\x90',
'CjkUnifiedIdeograph65bc': '\xe6\x96\xbc',
'CjkUnifiedIdeograph633b': '\xe6\x8c\xbb',
'CjkUnifiedIdeograph65ba': '\xe6\x96\xba',
'HangulSyllableBol': '\xeb\xb3\xbc',
'CjkUnifiedIdeograph65bf': '\xe6\x96\xbf',
'CjkUnifiedIdeograph65be': '\xe6\x96\xbe',
'CjkUnifiedIdeograph633c': '\xe6\x8c\xbc',
'YiRadicalCuop': '\xea\x92\xbd',
'CjkUnifiedIdeograph59e7': '\xe5\xa7\xa7',
'CjkUnifiedIdeograph950f': '\xe9\x94\x8f',
'HangulSyllableBaels': '\xeb\xb0\xbc',
'or': '\xe2\x88\xa8',
'CjkUnifiedIdeograph633a': '\xe6\x8c\xba',
'LatinSmallLetterNWithCedilla': '\xc5\x86',
'UnicodeCharacter0xf48f': '\xef\x92\x8f',
'CjkUnifiedIdeograph633f': '\xe6\x8c\xbf',
'UnicodeCharacter0xf48d': '\xef\x92\x8d',
'UnicodeCharacter0xf48e': '\xef\x92\x8e',
'UnicodeCharacter0xf48b': '\xef\x92\x8b',
'UnicodeCharacter0xf48c': '\xef\x92\x8c',
'CjkUnifiedIdeograph950a': '\xe9\x94\x8a',
'UnicodeCharacter0xf485': '\xef\x92\x85',
'HangulSyllableNyinh': '\xeb\x8a\xb2',
'NkoComma': '\xdf\xb8',
'HangulSyllableNyinj': '\xeb\x8a\xb1',
'CjkUnifiedIdeograph70de': '\xe7\x83\x9e',
'UnicodeCharacter0xf482': '\xef\x92\x82',
'CjkUnifiedIdeograph633e': '\xe6\x8c\xbe',
'UnicodeCharacter0xfe67': '\xef\xb9\xa7',
'negation': '\xc2\xac',
'CjkUnifiedIdeograph3cb6': '\xe3\xb2\xb6',
'CjkUnifiedIdeograph3cb7': '\xe3\xb2\xb7',
'CjkUnifiedIdeograph3cb4': '\xe3\xb2\xb4',
'CjkUnifiedIdeograph3cb5': '\xe3\xb2\xb5',
'CjkUnifiedIdeograph3cb2': '\xe3\xb2\xb2',
'CjkUnifiedIdeograph3cb3': '\xe3\xb2\xb3',
'CjkUnifiedIdeograph3cb0': '\xe3\xb2\xb0',
'CjkUnifiedIdeograph3cb1': '\xe3\xb2\xb1',
'CombiningEqualsSignBelow': '\xcd\x87',
'CjkUnifiedIdeograph3cb8': '\xe3\xb2\xb8',
'CjkUnifiedIdeograph3cb9': '\xe3\xb2\xb9',
'HangulSyllableSseup': '\xec\x94\x8a',
'CjkUnifiedIdeograph65b2': '\xe6\x96\xb2',
'YiSyllableVut': '\xea\x83\xb4',
'HangulSyllableToelm': '\xed\x87\xbe',
'CjkUnifiedIdeograph4059': '\xe4\x81\x99',
'CjkUnifiedIdeograph4058': '\xe4\x81\x98',
'UnicodeCharacter0xe2aa': '\xee\x8a\xaa',
'UnicodeCharacter0xe2af': '\xee\x8a\xaf',
'UnicodeCharacter0xe2ad': '\xee\x8a\xad',
'UnicodeCharacter0xe2ae': '\xee\x8a\xae',
'CjkUnifiedIdeograph4051': '\xe4\x81\x91',
'CjkUnifiedIdeograph4050': '\xe4\x81\x90',
'CjkUnifiedIdeograph4053': '\xe4\x81\x93',
'CjkUnifiedIdeograph4052': '\xe4\x81\x92',
'CjkUnifiedIdeograph4055': '\xe4\x81\x95',
'CjkUnifiedIdeograph4054': '\xe4\x81\x94',
'CjkUnifiedIdeograph4057': '\xe4\x81\x97',
'CjkUnifiedIdeograph4056': '\xe4\x81\x96',
'CjkUnifiedIdeograph3cbf': '\xe3\xb2\xbf',
'CjkUnifiedIdeograph3cbd': '\xe3\xb2\xbd',
'CjkUnifiedIdeograph3cbe': '\xe3\xb2\xbe',
'CjkUnifiedIdeograph3cbb': '\xe3\xb2\xbb',
'CjkUnifiedIdeograph3cbc': '\xe3\xb2\xbc',
'CjkUnifiedIdeograph3cba': '\xe3\xb2\xba',
'CjkUnifiedIdeograph65b4': '\xe6\x96\xb4',
'RunicLetterRaidoRadReidR': '\xe1\x9a\xb1',
'UnicodeCharacter0xd95c': '\xed\xa5\x9c',
'TibetanSubjoinedLetterWa': '\xe0\xbe\xad',
'UnicodeCharacter0xe2a3': '\xee\x8a\xa3',
'CjkUnifiedIdeograph70db': '\xe7\x83\x9b',
'UnicodeCharacter0xe2a1': '\xee\x8a\xa1',
'UnicodeCharacter0xe2a6': '\xee\x8a\xa6',
'UnicodeCharacter0xe2a7': '\xee\x8a\xa7',
'UnicodeCharacter0xe2a4': '\xee\x8a\xa4',
'UnicodeCharacter0xe2a5': '\xee\x8a\xa5',
'CjkUnifiedIdeograph405a': '\xe4\x81\x9a',
'CjkUnifiedIdeograph405c': '\xe4\x81\x9c',
'CjkUnifiedIdeograph405b': '\xe4\x81\x9b',
'CjkUnifiedIdeograph405e': '\xe4\x81\x9e',
'CjkUnifiedIdeograph405d': '\xe4\x81\x9d',
'CjkUnifiedIdeograph405f': '\xe4\x81\x9f',
'BalineseMusicalSymbolDaeng': '\xe1\xad\xa7',
'EthiopicNumberFifty': '\xe1\x8d\xb6',
'ArabicLetterLamMedialForm': '\xef\xbb\xa0',
'UnicodeCharacter0xe2b6': '\xee\x8a\xb6',
'YiSyllableLyrx': '\xea\x87\x98',
'CjkCompatibilityIdeographf966': '\xef\xa5\xa6',
'TurnedAngle': '\xe2\xa6\xa2',
'CjkCompatibilityIdeographf965': '\xef\xa5\xa5',
'CjkUnifiedIdeograph992b': '\xe9\xa4\xab',
'CjkUnifiedIdeograph992c': '\xe9\xa4\xac',
'CjkUnifiedIdeograph992a': '\xe9\xa4\xaa',
'CjkUnifiedIdeograph992f': '\xe9\xa4\xaf',
'CjkUnifiedIdeograph992d': '\xe9\xa4\xad',
'CjkUnifiedIdeograph992e': '\xe9\xa4\xae',
'UnicodeCharacter0xd95a': '\xed\xa5\x9a',
'HangulJongseongNieunsios': '\xe1\x87\x87',
'UnicodeCharacter0xaa4f': '\xea\xa9\x8f',
'SucceedsAboveAlmostEqualTo': '\xe2\xaa\xb8',
'UnicodeCharacter0xaa4e': '\xea\xa9\x8e',
'CjkUnifiedIdeograph501b': '\xe5\x80\x9b',
'CjkUnifiedIdeograph501c': '\xe5\x80\x9c',
'CjkUnifiedIdeograph501a': '\xe5\x80\x9a',
'CjkUnifiedIdeograph501f': '\xe5\x80\x9f',
'VaiSyllableVoo': '\xea\x95\xbe',
'CjkUnifiedIdeograph501d': '\xe5\x80\x9d',
'CjkUnifiedIdeograph501e': '\xe5\x80\x9e',
'CjkUnifiedIdeograph74fe': '\xe7\x93\xbe',
'CjkUnifiedIdeograph74fd': '\xe7\x93\xbd',
'CjkUnifiedIdeograph74ff': '\xe7\x93\xbf',
'CjkUnifiedIdeograph74fa': '\xe7\x93\xba',
'LatinSmallLetterAvWithHorizontalBar': '\xea\x9c\xbb',
'CjkUnifiedIdeograph74fc': '\xe7\x93\xbc',
'CjkUnifiedIdeograph74fb': '\xe7\x93\xbb',
'UnicodeCharacter0xf5d3': '\xef\x97\x93',
'EthiopicComma': '\xe1\x8d\xa3',
'UnicodeCharacter0xf5d1': '\xef\x97\x91',
'UnicodeCharacter0xf5d0': '\xef\x97\x90',
'UnicodeCharacter0xf5d7': '\xef\x97\x97',
'UnicodeCharacter0xf5d6': '\xef\x97\x96',
'UnicodeCharacter0xf5d5': '\xef\x97\x95',
'UnicodeCharacter0xf5d4': '\xef\x97\x94',
'UnicodeCharacter0xf5d9': '\xef\x97\x99',
'UnicodeCharacter0xf5d8': '\xef\x97\x98',
'CjkUnifiedIdeograph9922': '\xe9\xa4\xa2',
'CjkUnifiedIdeograph9923': '\xe9\xa4\xa3',
'CjkUnifiedIdeograph9920': '\xe9\xa4\xa0',
'CjkUnifiedIdeograph9921': '\xe9\xa4\xa1',
'CjkUnifiedIdeograph9926': '\xe9\xa4\xa6',
'CjkUnifiedIdeograph9927': '\xe9\xa4\xa7',
'CjkUnifiedIdeograph9924': '\xe9\xa4\xa4',
'CjkUnifiedIdeograph9925': '\xe9\xa4\xa5',
'CjkUnifiedIdeograph9928': '\xe9\xa4\xa8',
'CjkUnifiedIdeograph9929': '\xe9\xa4\xa9',
'LaoLetterSoSung': '\xe0\xba\xaa',
'UnicodeCharacter0xfd40': '\xef\xb5\x80',
'UnicodeCharacter0xf5dc': '\xef\x97\x9c',
'UnicodeCharacter0xf5db': '\xef\x97\x9b',
'UnicodeCharacter0xf5da': '\xef\x97\x9a',
'UnicodeCharacter0xab21': '\xea\xac\xa1',
'UnicodeCharacter0xf5de': '\xef\x97\x9e',
'UnicodeCharacter0x8a1': '\xe0\xa2\xa1',
'HangulSyllableWess': '\xec\x9b\xbc',
'UnicodeCharacter0xfd41': '\xef\xb5\x81',
'HangulSyllableJec': '\xec\xa0\xb3',
'CjkUnifiedIdeograph5012': '\xe5\x80\x92',
'CjkUnifiedIdeograph5013': '\xe5\x80\x93',
'CjkUnifiedIdeograph5010': '\xe5\x80\x90',
'CjkUnifiedIdeograph5011': '\xe5\x80\x91',
'CjkUnifiedIdeograph5016': '\xe5\x80\x96',
'CjkUnifiedIdeograph5017': '\xe5\x80\x97',
'CjkUnifiedIdeograph5014': '\xe5\x80\x94',
'CjkUnifiedIdeograph5015': '\xe5\x80\x95',
'CjkUnifiedIdeograph74f5': '\xe7\x93\xb5',
'CjkUnifiedIdeograph74f4': '\xe7\x93\xb4',
'CjkUnifiedIdeograph5018': '\xe5\x80\x98',
'CjkUnifiedIdeograph5019': '\xe5\x80\x99',
'CjkUnifiedIdeograph74f1': '\xe7\x93\xb1',
'ArabicLetterSuperscriptAlef': '\xd9\xb0',
'CjkUnifiedIdeograph74f3': '\xe7\x93\xb3',
'CjkUnifiedIdeograph74f2': '\xe7\x93\xb2',
'HangulSyllableJeg': '\xec\xa0\x9d',
'RejangSectionMark': '\xea\xa5\x9f',
'UnicodeCharacter0xdedc': '\xed\xbb\x9c',
'UnicodeCharacter0xf294': '\xef\x8a\x94',
'SyriacLetterLamadh': '\xdc\xa0',
'HangulSyllableJeh': '\xec\xa0\xb7',
'UnicodeCharacter0xdc52': '\xed\xb1\x92',
'SinhalaLetterAuyanna': '\xe0\xb6\x96',
'UnicodeCharacter0xf292': '\xef\x8a\x92',
'UnicodeCharacter0xfe1b': '\xef\xb8\x9b',
'HangulSyllableTyoh': '\xed\x88\xab',
'HangulSyllableTyok': '\xed\x88\xa8',
'HangulSyllableTyoj': '\xed\x88\xa6',
'HangulSyllableTyom': '\xed\x88\xa0',
'HangulSyllableTyol': '\xed\x88\x98',
'HangulSyllableTyon': '\xed\x88\x94',
'UnicodeCharacter0xfd47': '\xef\xb5\x87',
'HangulJongseongRieulhieuh': '\xe1\x86\xb6',
'HangulSyllableTyob': '\xed\x88\xa1',
'HangulSyllableTyod': '\xed\x88\x97',
'HangulSyllableTyog': '\xed\x88\x91',
'HangulSyllableJel': '\xec\xa0\xa4',
'HangulSyllableJeo': '\xec\xa0\x80',
'HangulSyllableTyop': '\xed\x88\xaa',
'HangulSyllableTyos': '\xed\x88\xa3',
'HangulSyllableJonh': '\xec\xa1\xb6',
'CjkUnifiedIdeograph8e77': '\xe8\xb9\xb7',
'CjkUnifiedIdeograph65bb': '\xe6\x96\xbb',
'UnicodeCharacter0xf298': '\xef\x8a\x98',
'HangulSyllableByugs': '\xeb\xb7\xb3',
'YiSyllableMguop': '\xea\x88\xb8',
'CjkUnifiedIdeograph65bd': '\xe6\x96\xbd',
'HangulSyllableByugg': '\xeb\xb7\xb2',
'CyrillicCapitalLetterHardSign': '\xd0\xaa',
'LatinCapitalLetterEWithHookAbove': '\xe1\xba\xba',
'CjkUnifiedIdeograph4839': '\xe4\xa0\xb9',
'CjkUnifiedIdeograph4838': '\xe4\xa0\xb8',
'HangulSyllableGgyagg': '\xea\xba\x86',
'CjkUnifiedIdeograph4833': '\xe4\xa0\xb3',
'CjkUnifiedIdeograph4832': '\xe4\xa0\xb2',
'CjkUnifiedIdeograph4831': '\xe4\xa0\xb1',
'CjkUnifiedIdeograph4830': '\xe4\xa0\xb0',
'CjkUnifiedIdeograph4837': '\xe4\xa0\xb7',
'CjkUnifiedIdeograph4836': '\xe4\xa0\xb6',
'CjkUnifiedIdeograph4835': '\xe4\xa0\xb5',
'CjkUnifiedIdeograph4834': '\xe4\xa0\xb4',
'CjkUnifiedIdeograph756a': '\xe7\x95\xaa',
'CjkUnifiedIdeograph756b': '\xe7\x95\xab',
'CjkUnifiedIdeograph756c': '\xe7\x95\xac',
'CjkUnifiedIdeograph756d': '\xe7\x95\xad',
'CjkUnifiedIdeograph756e': '\xe7\x95\xae',
'CjkUnifiedIdeograph756f': '\xe7\x95\xaf',
'UnicodeCharacter0x1df4': '\xe1\xb7\xb4',
'CjkUnifiedIdeograph9932': '\xe9\xa4\xb2',
'MediumLeftpointingAngleBracketOrnament': '\xe2\x9d\xac',
'UnicodeCharacter0xe933': '\xee\xa4\xb3',
'TamilMonthSign': '\xe0\xaf\xb4',
'HangulSyllableByels': '\xeb\xb3\xa4',
'UnicodeCharacter0xdb20': '\xed\xac\xa0',
'LimbuLetterWa': '\xe1\xa4\x98',
'UnicodeCharacter0xdb22': '\xed\xac\xa2',
'UnicodeCharacter0xdb23': '\xed\xac\xa3',
'UnicodeCharacter0xdb24': '\xed\xac\xa4',
'UnicodeCharacter0xdb25': '\xed\xac\xa5',
'UnicodeCharacter0xdb26': '\xed\xac\xa6',
'UnicodeCharacter0xdb27': '\xed\xac\xa7',
'UnicodeCharacter0x128e': '\xe1\x8a\x8e',
'UnicodeCharacter0x128f': '\xe1\x8a\x8f',
'CjkUnifiedIdeograph3e0b': '\xe3\xb8\x8b',
'CjkUnifiedIdeograph3e0c': '\xe3\xb8\x8c',
'CjkUnifiedIdeograph3e0a': '\xe3\xb8\x8a',
'CjkUnifiedIdeograph3e0f': '\xe3\xb8\x8f',
'UnicodeCharacter0xe932': '\xee\xa4\xb2',
'CjkUnifiedIdeograph3e0d': '\xe3\xb8\x8d',
'CjkUnifiedIdeograph3e0e': '\xe3\xb8\x8e',
'CjkUnifiedIdeograph7560': '\xe7\x95\xa0',
'CjkUnifiedIdeograph7561': '\xe7\x95\xa1',
'CjkUnifiedIdeograph7562': '\xe7\x95\xa2',
'CyrillicSmallLetterYi': '\xd1\x97',
'CjkUnifiedIdeograph7564': '\xe7\x95\xa4',
'CjkUnifiedIdeograph7565': '\xe7\x95\xa5',
'CjkUnifiedIdeograph7566': '\xe7\x95\xa6',
'CjkUnifiedIdeograph7567': '\xe7\x95\xa7',
'CjkUnifiedIdeograph7568': '\xe7\x95\xa8',
'CjkUnifiedIdeograph7569': '\xe7\x95\xa9',
'CyrillicSmallLetterYa': '\xd1\x8f',
'CjkUnifiedIdeograph83f9': '\xe8\x8f\xb9',
'CjkUnifiedIdeograph83f8': '\xe8\x8f\xb8',
'HangulSyllableGgoegs': '\xea\xbe\x83',
'CjkUnifiedIdeograph483c': '\xe4\xa0\xbc',
'CjkUnifiedIdeograph483b': '\xe4\xa0\xbb',
'CjkUnifiedIdeograph483a': '\xe4\xa0\xba',
'UnicodeCharacter0xe378': '\xee\x8d\xb8',
'CjkUnifiedIdeograph483f': '\xe4\xa0\xbf',
'CjkUnifiedIdeograph483e': '\xe4\xa0\xbe',
'CyrillicSmallLetterYu': '\xd1\x8e',
'CjkUnifiedIdeograph3e08': '\xe3\xb8\x88',
'CjkUnifiedIdeograph3e09': '\xe3\xb8\x89',
'YiSyllableHit': '\xea\x89\xae',
'FullwidthLeftCurlyBracket': '\xef\xbd\x9b',
'CjkUnifiedIdeograph3e02': '\xe3\xb8\x82',
'CjkUnifiedIdeograph3e03': '\xe3\xb8\x83',
'CjkUnifiedIdeograph3e00': '\xe3\xb8\x80',
'LatinSmallLetterCWithDotAbove': '\xc4\x8b',
'UnicodeCharacter0x1289': '\xe1\x8a\x89',
'CjkUnifiedIdeograph3e07': '\xe3\xb8\x87',
'CjkUnifiedIdeograph3e04': '\xe3\xb8\x84',
'CjkUnifiedIdeograph3e05': '\xe3\xb8\x85',
'CjkUnifiedIdeograph9eef': '\xe9\xbb\xaf',
'HexagramForOppression': '\xe4\xb7\xae',
'CjkUnifiedIdeograph4789': '\xe4\x9e\x89',
'HangulSyllableBboess': '\xeb\xbe\xa8',
'CjkUnifiedIdeograph8e71': '\xe8\xb9\xb1',
'UnicodeCharacter0xdb2c': '\xed\xac\xac',
'UnicodeCharacter0xdb2d': '\xed\xac\xad',
'UnicodeCharacter0xdb2e': '\xed\xac\xae',
'UnicodeCharacter0xdb2f': '\xed\xac\xaf',
'UnicodeCharacter0xe407': '\xee\x90\x87',
'CjkUnifiedIdeograph4788': '\xe4\x9e\x88',
'UnicodeCharacter0xe937': '\xee\xa4\xb7',
'HangulSyllableCeobs': '\xec\xb2\xaa',
'ArabicLetterKafInitialForm': '\xef\xbb\x9b',
'TamilDigitOne': '\xe0\xaf\xa7',
'LeftwardsTwoheadedArrowWithVerticalStroke': '\xe2\xac\xb4',
'CombiningGraphemeJoiner': '\xcd\x8f',
'CanadianSyllabicsMii': '\xe1\x92\xa6',
'CjkUnifiedIdeograph6851': '\xe6\xa1\x91',
'UnicodeCharacter0xedee': '\xee\xb7\xae',
'HangulSyllableHyeob': '\xed\x98\x91',
'DownwardsHarpoonWithBarbRightToBar': '\xe2\xa5\x95',
'CjkUnifiedIdeograph7be8': '\xe7\xaf\xa8',
'UnicodeCharacter0xe68e': '\xee\x9a\x8e',
'CjkUnifiedIdeograph7be9': '\xe7\xaf\xa9',
'UnicodeCharacter0xe488': '\xee\x92\x88',
'FallingDiagonalCrossingRisingDiagonal': '\xe2\xa4\xac',
'CanadianSyllabicsSwaa': '\xe1\x94\x82',
'SymbolForNewline': '\xe2\x90\xa4',
'HangulSyllableNying': '\xeb\x8b\x81',
'HangulSyllableGgoegg': '\xea\xbe\x82',
'=:': '\xe2\x89\x95',
'CjkUnifiedIdeograph80ca': '\xe8\x83\x8a',
'CjkUnifiedIdeograph7be2': '\xe7\xaf\xa2',
'UnicodeCharacter0xe935': '\xee\xa4\xb5',
'CjkUnifiedIdeograph7be3': '\xe7\xaf\xa3',
'UnicodeCharacter0x274e': '\xe2\x9d\x8e',
'CjkUnifiedIdeograph4b23': '\xe4\xac\xa3',
'UnicodeCharacter0x274c': '\xe2\x9d\x8c',
'ArabicLetterNgInitialForm': '\xef\xaf\x95',
'InterlinearAnnotationAnchor': '\xef\xbf\xb9',
'UnicodeCharacter0xdf0a': '\xed\xbc\x8a',
'UnicodeCharacter0xdf0f': '\xed\xbc\x8f',
'CjkCompatibilityIdeographf9ee': '\xef\xa7\xae',
'HangulChoseongChitueumsios': '\xe1\x84\xbc',
'UnicodeCharacter0xe934': '\xee\xa4\xb4',
'UnicodeCharacter0xdf0d': '\xed\xbc\x8d',
'RialSign': '\xef\xb7\xbc',
'UnicodeCharacter0xdf0e': '\xed\xbc\x8e',
'SaurashtraLetterAa': '\xea\xa2\x83',
'SaurashtraLetterAi': '\xea\xa2\x8e',
'SaurashtraLetterAu': '\xea\xa2\x91',
'UnicodeCharacter0xedea': '\xee\xb7\xaa',
'TibetanSignRnamBcad': '\xe0\xbd\xbf',
'CombiningAnticlockwiseRingOverlay': '\xe2\x83\x9a',
'UnicodeCharacter0xe68a': '\xee\x9a\x8a',
'KatakanaLetterBu': '\xe3\x83\x96',
'VaiDigitEight': '\xea\x98\xa8',
'UnicodeCharacter0xfa6c': '\xef\xa9\xac',
'KatakanaLetterBi': '\xe3\x83\x93',
'HangulSyllableJang': '\xec\x9e\xa5',
'UnicodeCharacter0xdcb9': '\xed\xb2\xb9',
'UnicodeCharacter0xdcb8': '\xed\xb2\xb8',
'CopticCapitalLetterDei': '\xcf\xae',
'KatakanaLetterBo': '\xe3\x83\x9c',
'UnicodeCharacter0xdcb5': '\xed\xb2\xb5',
'KatakanaLetterBa': '\xe3\x83\x90',
'UnicodeCharacter0xdcb7': '\xed\xb2\xb7',
'SinhalaVowelSignKombuvaHaaAelapilla': '\xe0\xb7\x9c',
'HangulSyllableJanh': '\xec\x9e\x96',
'CircledBullet': '\xe2\xa6\xbf',
'HangulSyllableJanj': '\xec\x9e\x95',
'UnicodeCharacter0x530': '\xd4\xb0',
'CjkUnifiedIdeograph59c2': '\xe5\xa7\x82',
'YiSyllableDdep': '\xea\x85\x8a',
'YiSyllableDdex': '\xea\x85\x88',
'YiSyllableNgox': '\xea\x89\xa8',
'UnicodeCharacter0xfe1e': '\xef\xb8\x9e',
'HangulSyllableHeulg': '\xed\x9d\x99',
'HangulSyllableHeulb': '\xed\x9d\x9b',
'CjkUnifiedIdeograph7122': '\xe7\x84\xa2',
'HangulSyllableHeulm': '\xed\x9d\x9a',
'GreekCapitalLetterAlphaWithDasiaAndPerispomeni': '\xe1\xbc\x8f',
'UnicodeCharacter0xdcbf': '\xed\xb2\xbf',
'YiSyllableNgot': '\xea\x89\xa7',
'UnicodeCharacter0xf29e': '\xef\x8a\x9e',
'UnicodeCharacter0xdcbc': '\xed\xb2\xbc',
'UnicodeCharacter0xdcbb': '\xed\xb2\xbb',
'HangulSyllableHeult': '\xed\x9d\x9d',
'eta': '\xce\xb7',
'HangulSyllableHeulp': '\xed\x9d\x9e',
'HangulSyllableHeuls': '\xed\x9d\x9c',
'forall': '\xe2\x88\x80',
'ArabicVowelSignDotBelow': '\xd9\x9c',
'GreekLowerNumeralSign': '\xcd\xb5',
'CjkUnifiedIdeograph4b25': '\xe4\xac\xa5',
'UnicodeCharacter0xf36e': '\xef\x8d\xae',
'HangulSyllableTyat': '\xed\x84\x91',
'WhiteMediumSquare': '\xe2\x97\xbb',
'UnicodeCharacter0xe2ab': '\xee\x8a\xab',
'HangulSyllableJjeot': '\xec\xa9\xa5',
'HangulSyllableJjeos': '\xec\xa9\x9f',
'HangulSyllableJjeop': '\xec\xa9\xa6',
'UnicodeCharacter0xe2ac': '\xee\x8a\xac',
'UnicodeCharacter0xf6fd': '\xef\x9b\xbd',
'HangulSyllableRyong': '\xeb\xa3\xa1',
'HangulSyllableJjeod': '\xec\xa9\x93',
'HangulSyllableJjeob': '\xec\xa9\x9d',
'HangulSyllableJjeoc': '\xec\xa9\xa3',
'UnicodeCharacter0xdbf5': '\xed\xaf\xb5',
'HangulSyllableJjeon': '\xec\xa9\x90',
'KangxiRadicalEight': '\xe2\xbc\x8b',
'HangulSyllableJjeol': '\xec\xa9\x94',
'HangulSyllableJjeom': '\xec\xa9\x9c',
'HangulSyllableRyonj': '\xeb\xa3\x91',
'HangulSyllableJjeok': '\xec\xa9\xa4',
'HangulSyllableRyonh': '\xeb\xa3\x92',
'BugineseLetterGa': '\xe1\xa8\x81',
'HangulSyllableSsaeb': '\xec\x8c\xa5',
'UnicodeCharacter0xdbf7': '\xed\xaf\xb7',
'GlagoliticSmallLetterYeri': '\xe2\xb1\x90',
'HangulSyllableSsaec': '\xec\x8c\xab',
'HangulSyllableDdit': '\xeb\x9d\xb9',
'HangulSyllableDdis': '\xeb\x9d\xb3',
'HangulSyllableDdip': '\xeb\x9d\xba',
'CjkUnifiedIdeograph3737': '\xe3\x9c\xb7',
'HangulSyllableDdin': '\xeb\x9d\xa4',
'HangulSyllableDdil': '\xeb\x9d\xa8',
'HangulSyllableDdim': '\xeb\x9d\xb0',
'HangulSyllableDdij': '\xeb\x9d\xb6',
'HangulSyllableDdik': '\xeb\x9d\xb8',
'HangulSyllableDdih': '\xeb\x9d\xbb',
'GlagoliticSmallLetterYeru': '\xe2\xb1\x8f',
'HangulSyllableDdig': '\xeb\x9d\xa1',
'HangulSyllableDdid': '\xeb\x9d\xa7',
'HangulSyllableDdib': '\xeb\x9d\xb1',
'HangulSyllableDdic': '\xeb\x9d\xb7',
'CjkUnifiedIdeograph706d': '\xe7\x81\xad',
'CjkUnifiedIdeograph7af4': '\xe7\xab\xb4',
'HangulSyllableSsaed': '\xec\x8c\x9b',
'CjkUnifiedIdeograph37b5': '\xe3\x9e\xb5',
'CjkUnifiedIdeograph7af6': '\xe7\xab\xb6',
'UnicodeCharacter0xdf02': '\xed\xbc\x82',
'UnicodeCharacter0xdf03': '\xed\xbc\x83',
'UnicodeCharacter0xfe1f': '\xef\xb8\x9f',
'HangulSyllableSsaeh': '\xec\x8c\xaf',
'UnicodeCharacter0xdf01': '\xed\xbc\x81',
'UnicodeCharacter0xb3a': '\xe0\xac\xba',
'UnicodeCharacter0xb3b': '\xe0\xac\xbb',
'CjkUnifiedIdeograph852e': '\xe8\x94\xae',
'CjkUnifiedIdeograph852d': '\xe8\x94\xad',
'CjkUnifiedIdeograph3731': '\xe3\x9c\xb1',
'CjkUnifiedIdeograph852a': '\xe8\x94\xaa',
'CjkUnifiedIdeograph852c': '\xe8\x94\xac',
'CjkUnifiedIdeograph852b': '\xe8\x94\xab',
'LatinSmallLetterEWithNotch': '\xe2\xb1\xb8',
'UnicodeCharacter0xdf04': '\xed\xbc\x84',
'mp': '\xe2\x88\x93',
'UnicodeCharacter0xdf05': '\xed\xbc\x85',
'HangulSyllableSsyeogs': '\xec\x8e\xa3',
'CjkUnifiedIdeograph6f2e': '\xe6\xbc\xae',
'TibetanLetterCa': '\xe0\xbd\x85',
'RightwardsHarpoonWithBarbUpToBar': '\xe2\xa5\x93',
'CjkUnifiedIdeograph6417': '\xe6\x90\x97',
'CjkUnifiedIdeograph6416': '\xe6\x90\x96',
'CjkUnifiedIdeograph6415': '\xe6\x90\x95',
'CjkUnifiedIdeograph6414': '\xe6\x90\x94',
'CjkUnifiedIdeograph6413': '\xe6\x90\x93',
'CjkUnifiedIdeograph6412': '\xe6\x90\x92',
'CjkUnifiedIdeograph6411': '\xe6\x90\x91',
'CjkUnifiedIdeograph6410': '\xe6\x90\x90',
'ParenthesizedIdeographFire': '\xe3\x88\xab',
'ArabicLetterHehIsolatedForm': '\xef\xbb\xa9',
'CjkUnifiedIdeograph6419': '\xe6\x90\x99',
'CjkUnifiedIdeograph6418': '\xe6\x90\x98',
'CircledIdeographSix': '\xe3\x8a\x85',
'HanunooLetterGa': '\xe1\x9c\xa4',
'HangulSyllableDwabs': '\xeb\x8f\xb2',
'UnicodeCharacter0xb34': '\xe0\xac\xb4',
'CjkUnifiedIdeograph8529': '\xe8\x94\xa9',
'UnicodeCharacter0xb31': '\xe0\xac\xb1',
'HangulSyllableByagg': '\xeb\xb1\x8e',
'CjkUnifiedIdeograph8525': '\xe8\x94\xa5',
'CjkUnifiedIdeograph8524': '\xe8\x94\xa4',
'CjkUnifiedIdeograph8527': '\xe8\x94\xa7',
'CjkUnifiedIdeograph8526': '\xe8\x94\xa6',
'CjkUnifiedIdeograph8521': '\xe8\x94\xa1',
'CjkUnifiedIdeograph8520': '\xe8\x94\xa0',
'CjkUnifiedIdeograph8523': '\xe8\x94\xa3',
'CjkUnifiedIdeograph8522': '\xe8\x94\xa2',
'CjkUnifiedIdeograph641f': '\xe6\x90\x9f',
'CjkUnifiedIdeograph641e': '\xe6\x90\x9e',
'CjkUnifiedIdeograph641d': '\xe6\x90\x9d',
'CjkUnifiedIdeograph641c': '\xe6\x90\x9c',
'CjkUnifiedIdeograph641b': '\xe6\x90\x9b',
'CjkUnifiedIdeograph641a': '\xe6\x90\x9a',
'UnicodeCharacter0xe7a3': '\xee\x9e\xa3',
'ArabicKasratan': '\xd9\x8d',
'CjkUnifiedIdeograph8e5d': '\xe8\xb9\x9d',
'HangulSyllableMwelm': '\xeb\xad\xb6',
'HangulSyllableSsweoss': '\xec\x92\x94',
'CjkUnifiedIdeograph8e5b': '\xe8\xb9\x9b',
'CjkUnifiedIdeograph8e5c': '\xe8\xb9\x9c',
'CjkUnifiedIdeograph6df2': '\xe6\xb7\xb2',
'CjkUnifiedIdeograph6df3': '\xe6\xb7\xb3',
'CjkUnifiedIdeograph6df0': '\xe6\xb7\xb0',
'CjkUnifiedIdeograph6df1': '\xe6\xb7\xb1',
'CjkUnifiedIdeograph6df6': '\xe6\xb7\xb6',
'CjkUnifiedIdeograph6df7': '\xe6\xb7\xb7',
'CjkUnifiedIdeograph6df4': '\xe6\xb7\xb4',
'CjkUnifiedIdeograph6df5': '\xe6\xb7\xb5',
'CjkUnifiedIdeograph6df8': '\xe6\xb7\xb8',
'CjkUnifiedIdeograph6df9': '\xe6\xb7\xb9',
'GreekCapitalLetterUpsilonWithDasiaAndVaria': '\xe1\xbd\x9b',
'AplFunctionalSymbolUpTackDiaeresis': '\xe2\x8d\xa1',
'ErrorbarredBlackSquare': '\xe2\xa7\xaf',
'BalineseMusicalSymbolCombiningGong': '\xe1\xad\xb3',
'TopRightCrop': '\xe2\x8c\x8e',
'UnicodeCharacter0xf43f': '\xef\x90\xbf',
'UnicodeCharacter0xe2a2': '\xee\x8a\xa2',
'HangulSyllableTaep': '\xed\x83\xb6',
'UnicodeCharacter0xe7a4': '\xee\x9e\xa4',
'UnicodeCharacter0xe2a0': '\xee\x8a\xa0',
'UnicodeCharacter0xde3d': '\xed\xb8\xbd',
'San': '\xcf\xba',
'CjkUnifiedIdeograph6e69': '\xe6\xb9\xa9',
'HangulSyllableWabs': '\xec\x99\x92',
'CyrillicCapitalLetterTche': '\xea\x9a\x92',
'CjkUnifiedIdeograph6dfb': '\xe6\xb7\xbb',
'MongolianLetterSibeCha': '\xe1\xa1\xb1',
'CjkUnifiedIdeograph6dfa': '\xe6\xb7\xba',
'CjkUnifiedIdeograph6dff': '\xe6\xb7\xbf',
'CjkUnifiedIdeograph6dfd': '\xe6\xb7\xbd',
'CjkUnifiedIdeograph6dfe': '\xe6\xb7\xbe',
'ArmenianCapitalLetterGim': '\xd4\xb3',
'UnicodeCharacter0xe204': '\xee\x88\x84',
'UnicodeCharacter0xe2a8': '\xee\x8a\xa8',
'HangulSyllableBbanh': '\xeb\xb9\xa6',
'GurmukhiLetterNna': '\xe0\xa8\xa3',
'CjkUnifiedIdeograph6e64': '\xe6\xb9\xa4',
'HangulSyllableMulm': '\xeb\xac\xbe',
'HangulSyllableJwaebs': '\xec\xa2\xba',
'HangulSyllableDwess': '\xeb\x92\x9c',
'GurmukhiLetterRra': '\xe0\xa9\x9c',
'UnicodeCharacter0xde4d': '\xed\xb9\x8d',
'RomanNumeralOneHundred': '\xe2\x85\xad',
'HangulSyllableGgeogs': '\xea\xba\xbf',
'KannadaLetterRa': '\xe0\xb2\xb0',
'HangulSyllableGgeogg': '\xea\xba\xbe',
'UnicodeCharacter0xdd7a': '\xed\xb5\xba',
'SquareOv': '\xe3\x8d\xb5',
'CjkUnifiedIdeograph7afa': '\xe7\xab\xba',
'KhmerLetterSa': '\xe1\x9e\x9f',
'UnicodeCharacter0xe92d': '\xee\xa4\xad',
'HangulSyllableMulg': '\xeb\xac\xbd',
'HangulSyllableNwigs': '\xeb\x89\x9b',
'CanadianSyllabicsCarrierShu': '\xe1\x99\x94',
'UnicodeCharacter0xe92a': '\xee\xa4\xaa',
'UnicodeCharacter0xe92b': '\xee\xa4\xab',
'YiSyllableTu': '\xea\x84\xb2',
'CanadianSyllabicsCarrierShi': '\xe1\x99\x98',
'YiSyllableTi': '\xea\x84\x9c',
'YiSyllableTo': '\xea\x84\xab',
'CanadianSyllabicsCarrierSho': '\xe1\x99\x95',
'CanadianSyllabicsCarrierSha': '\xe1\x99\x99',
'YiSyllableTa': '\xea\x84\xa3',
'CanadianSyllabicsCarrierShe': '\xe1\x99\x96',
'HangulSyllableNwigg': '\xeb\x89\x9a',
'YiSyllableTe': '\xea\x84\xae',
'NobreakSpace': '\xc2\xa0',
'YiSyllableCiex': '\xea\x8a\xb3',
'YiSyllableCiep': '\xea\x8a\xb5',
'YiSyllableCiet': '\xea\x8a\xb2',
'UnicodeCharacter0xaaa8': '\xea\xaa\xa8',
'TeluguDigitOne': '\xe0\xb1\xa7',
'CjkUnifiedIdeograph7921': '\xe7\xa4\xa1',
'HangulSyllableKwigs': '\xed\x80\xb7',
'UnicodeCharacter0xe928': '\xee\xa4\xa8',
'UnicodeCharacter0xe929': '\xee\xa4\xa9',
'UnicodeCharacter0xdd7b': '\xed\xb5\xbb',
'UnicodeCharacter0xe924': '\xee\xa4\xa4',
'CjkUnifiedIdeograph94ce': '\xe9\x93\x8e',
'UnicodeCharacter0xe926': '\xee\xa4\xa6',
'HangulSyllableKwigg': '\xed\x80\xb6',
'UnicodeCharacter0xe920': '\xee\xa4\xa0',
'SupersetBesideAndJoinedByDashWithSubset': '\xe2\xab\x98',
'UnicodeCharacter0xe922': '\xee\xa4\xa2',
'UnicodeCharacter0xe923': '\xee\xa4\xa3',
'HangulSyllableCa': '\xec\xb0\xa8',
'SymbolForDataLinkEscape': '\xe2\x90\x90',
'CjkStrokeSp': '\xe3\x87\x93',
'EthiopicSyllableGgwa': '\xe2\xb6\x93',
'CjkUnifiedIdeograph84b2': '\xe8\x92\xb2',
'CjkUnifiedIdeograph84b3': '\xe8\x92\xb3',
'CjkStrokeSt': '\xe3\x87\x99',
'EthiopicSyllableGgwe': '\xe2\xb6\x96',
'CjkUnifiedIdeograph84b6': '\xe8\x92\xb6',
'CjkStrokeSw': '\xe3\x87\x84',
'CjkUnifiedIdeograph84b8': '\xe8\x92\xb8',
'EthiopicSyllableGgwi': '\xe2\xb6\x94',
'CjkStrokeSz': '\xe3\x87\x97',
'VaiSyllableSho': '\xea\x97\x94',
'CjkUnifiedIdeograph6c25': '\xe6\xb0\xa5',
'CjkUnifiedIdeograph6c24': '\xe6\xb0\xa4',
'CjkUnifiedIdeograph6c27': '\xe6\xb0\xa7',
'CjkUnifiedIdeograph685d': '\xe6\xa1\x9d',
'CjkUnifiedIdeograph6c21': '\xe6\xb0\xa1',
'CjkUnifiedIdeograph6c20': '\xe6\xb0\xa0',
'CjkUnifiedIdeograph6c23': '\xe6\xb0\xa3',
'CjkStrokeSg': '\xe3\x87\x9a',
'VaiSyllableShi': '\xea\x94\xbc',
'UnicodeCharacter0xdbfc': '\xed\xaf\xbc',
'CjkUnifiedIdeograph6c29': '\xe6\xb0\xa9',
'CjkUnifiedIdeograph6c28': '\xe6\xb0\xa8',
'CjkUnifiedIdeograph685b': '\xe6\xa1\x9b',
'SymbolForAcknowledge': '\xe2\x90\x86',
'LaoLetterPhoSung': '\xe0\xba\x9c',
'GreekCapitalLetterOmegaWithPsiliAndOxia': '\xe1\xbd\xac',
'UnicodeCharacter0xe360': '\xee\x8d\xa0',
'UnicodeCharacter0xf6b1': '\xef\x9a\xb1',
'UnicodeCharacter0xeeaa': '\xee\xba\xaa',
'OriyaLetterWa': '\xe0\xad\xb1',
'CjkUnifiedIdeograph6c2e': '\xe6\xb0\xae',
'CjkUnifiedIdeograph6c2d': '\xe6\xb0\xad',
'CjkUnifiedIdeograph6c2f': '\xe6\xb0\xaf',
'CjkUnifiedIdeograph6c2a': '\xe6\xb0\xaa',
'CjkUnifiedIdeograph6c2c': '\xe6\xb0\xac',
'CjkUnifiedIdeograph6c2b': '\xe6\xb0\xab',
'UnicodeCharacter0xe5e1': '\xee\x97\xa1',
'UnicodeCharacter0xf1ab': '\xef\x86\xab',
'OcrChair': '\xe2\x91\x81',
'CjkUnifiedIdeograph84ba': '\xe8\x92\xba',
'CjkUnifiedIdeograph84bb': '\xe8\x92\xbb',
'CjkUnifiedIdeograph84bc': '\xe8\x92\xbc',
'CjkUnifiedIdeograph84bd': '\xe8\x92\xbd',
'CjkUnifiedIdeograph84be': '\xe8\x92\xbe',
'CjkUnifiedIdeograph84bf': '\xe8\x92\xbf',
'EthiopicSyllableWee': '\xe1\x8b\x8c',
'UnicodeCharacter0xe09a': '\xee\x82\x9a',
'CjkUnifiedIdeograph4c78': '\xe4\xb1\xb8',
'CjkUnifiedIdeograph4c79': '\xe4\xb1\xb9',
'HangulSyllableHwed': '\xed\x9c\x83',
'CjkUnifiedIdeograph4c70': '\xe4\xb1\xb0',
'CjkUnifiedIdeograph4c71': '\xe4\xb1\xb1',
'CjkUnifiedIdeograph4c72': '\xe4\xb1\xb2',
'CjkUnifiedIdeograph4c73': '\xe4\xb1\xb3',
'CjkUnifiedIdeograph4c74': '\xe4\xb1\xb4',
'CjkUnifiedIdeograph4c75': '\xe4\xb1\xb5',
'ClosedUnionWithSerifs': '\xe2\xa9\x8c',
'CjkUnifiedIdeograph4c77': '\xe4\xb1\xb7',
'CjkUnifiedIdeograph44b4': '\xe4\x92\xb4',
'CjkUnifiedIdeograph44b5': '\xe4\x92\xb5',
'CjkUnifiedIdeograph44b6': '\xe4\x92\xb6',
'CjkUnifiedIdeograph44b7': '\xe4\x92\xb7',
'CjkUnifiedIdeograph44b0': '\xe4\x92\xb0',
'CjkUnifiedIdeograph44b1': '\xe4\x92\xb1',
'CjkUnifiedIdeograph44b2': '\xe4\x92\xb2',
'CjkUnifiedIdeograph44b3': '\xe4\x92\xb3',
'CjkUnifiedIdeograph44b8': '\xe4\x92\xb8',
'CjkUnifiedIdeograph44b9': '\xe4\x92\xb9',
'GreekCapitalLetterAlphaWithTonos': '\xce\x86',
'MongolianLetterSibeGa': '\xe1\xa1\xa4',
'HangulSyllableCyogg': '\xec\xb5\xba',
'WhiteSunWithRays': '\xe2\x98\xbc',
'BraillePatternDots2678': '\xe2\xa3\xa2',
'UnicodeCharacter0x170d': '\xe1\x9c\x8d',
'CjkUnifiedIdeograph7928': '\xe7\xa4\xa8',
'CjkUnifiedIdeograph44bd': '\xe4\x92\xbd',
'CjkUnifiedIdeograph44be': '\xe4\x92\xbe',
'CjkUnifiedIdeograph44bf': '\xe4\x92\xbf',
'HangulSyllableHwen': '\xed\x9c\x80',
'UnicodeCharacter0xf385': '\xef\x8e\x85',
'CjkUnifiedIdeograph44ba': '\xe4\x92\xba',
'CjkUnifiedIdeograph44bb': '\xe4\x92\xbb',
'CjkUnifiedIdeograph44bc': '\xe4\x92\xbc',
'HangulSyllableRyess': '\xeb\xa1\x94',
'UnicodeCharacter0xf388': '\xef\x8e\x88',
'DevanagariLetterUu': '\xe0\xa4\x8a',
'HangulSyllableHweh': '\xed\x9c\x97',
'UnicodeCharacter0xe364': '\xee\x8d\xa4',
'UnicodeCharacter0xf6b5': '\xef\x9a\xb5',
'MyanmarLetterNa': '\xe1\x80\x94',
'CjkUnifiedIdeograph4c7a': '\xe4\xb1\xba',
'CjkUnifiedIdeograph4c7b': '\xe4\xb1\xbb',
'CjkUnifiedIdeograph4c7c': '\xe4\xb1\xbc',
'SquareMbSmall': '\xe3\x8f\x94',
'CjkUnifiedIdeograph4c7e': '\xe4\xb1\xbe',
'CjkUnifiedIdeograph4c7f': '\xe4\xb1\xbf',
'ArabicLigatureAlefWithFathatanIsolatedForm': '\xef\xb4\xbd',
'EthiopicSyllableVaa': '\xe1\x89\xab',
'HangulSyllableHeuss': '\xed\x9d\xa4',
'UnicodeCharacter0xe67e': '\xee\x99\xbe',
'UnicodeCharacter0xeecd': '\xee\xbb\x8d',
'UnicodeCharacter0xe365': '\xee\x8d\xa5',
'HalfwidthKatakanaLetterRo': '\xef\xbe\x9b',
'HangulSyllableJyol': '\xec\xa3\xa8',
'UnicodeCharacter0xe93c': '\xee\xa4\xbc',
'HangulSyllableHwep': '\xed\x9c\x96',
'UnicodeCharacter0xf52e': '\xef\x94\xae',
'UnicodeCharacter0xf52d': '\xef\x94\xad',
'UnicodeCharacter0xf52f': '\xef\x94\xaf',
'UnicodeCharacter0xf52a': '\xef\x94\xaa',
'UnicodeCharacter0xe67b': '\xee\x99\xbb',
'UnicodeCharacter0xf52c': '\xef\x94\xac',
'UnicodeCharacter0xf52b': '\xef\x94\xab',
'UnicodeCharacter0xe67c': '\xee\x99\xbc',
'BlackRightpointingPointer': '\xe2\x96\xba',
'UnicodeCharacter0xe5e7': '\xee\x97\xa7',
'UnicodeCharacter0xed94': '\xee\xb6\x94',
'CjkUnifiedIdeograph7e06': '\xe7\xb8\x86',
'CjkCompatibilityIdeographf92f': '\xef\xa4\xaf',
'UnicodeCharacter0xecb8': '\xee\xb2\xb8',
'UnicodeCharacter0xecb9': '\xee\xb2\xb9',
'UnicodeCharacter0xecb6': '\xee\xb2\xb6',
'UnicodeCharacter0xecb7': '\xee\xb2\xb7',
'RightwardsSquiggleArrow': '\xe2\x87\x9d',
'HangulSyllableGwess': '\xea\xb6\xb8',
'UnicodeCharacter0xecb2': '\xee\xb2\xb2',
'CjkUnifiedIdeograph7e04': '\xe7\xb8\x84',
'UnicodeCharacter0xecb0': '\xee\xb2\xb0',
'UnicodeCharacter0xecb1': '\xee\xb2\xb1',
'UnicodeCharacter0x1bda': '\xe1\xaf\x9a',
'UnicodeCharacter0x1bdc': '\xe1\xaf\x9c',
'UnicodeCharacter0x1bdb': '\xe1\xaf\x9b',
'UnicodeCharacter0x1bde': '\xe1\xaf\x9e',
'UnicodeCharacter0x1bdd': '\xe1\xaf\x9d',
'UnicodeCharacter0x1bdf': '\xe1\xaf\x9f',
'CjkCompatibilityIdeographf9f3': '\xef\xa7\xb3',
'UnicodeCharacter0xe5e6': '\xee\x97\xa6',
'CjkUnifiedIdeograph3fdd': '\xe3\xbf\x9d',
'GlagoliticSmallLetterBigYus': '\xe2\xb1\x98',
'HangulSyllableTwap': '\xed\x87\x96',
'UnicodeCharacter0xf525': '\xef\x94\xa5',
'UnicodeCharacter0xf524': '\xef\x94\xa4',
'UnicodeCharacter0xf527': '\xef\x94\xa7',
'UnicodeCharacter0xf526': '\xef\x94\xa6',
'UnicodeCharacter0xf521': '\xef\x94\xa1',
'HangulSyllableNyes': '\xeb\x85\xaf',
'CyrillicSmallLetterUkrainianIe': '\xd1\x94',
'UnicodeCharacter0xf522': '\xef\x94\xa2',
'UnicodeCharacter0xe5e9': '\xee\x97\xa9',
'UnicodeCharacter0xf529': '\xef\x94\xa9',
'HangulSyllablePyanh': '\xed\x8d\x8a',
'UnicodeCharacter0x1bd1': '\xe1\xaf\x91',
'UnicodeCharacter0x1bd0': '\xe1\xaf\x90',
'UnicodeCharacter0x1bd3': '\xe1\xaf\x93',
'UnicodeCharacter0x1bd2': '\xe1\xaf\x92',
'UnicodeCharacter0x1bd5': '\xe1\xaf\x95',
'UnicodeCharacter0x1bd4': '\xe1\xaf\x94',
'UnicodeCharacter0x1bd7': '\xe1\xaf\x97',
'UnicodeCharacter0x1bd6': '\xe1\xaf\x96',
'UnicodeCharacter0x1bd9': '\xe1\xaf\x99',
'UnicodeCharacter0x1bd8': '\xe1\xaf\x98',
'CjkUnifiedIdeograph3fdb': '\xe3\xbf\x9b',
'HangulSyllableNyeo': '\xeb\x85\x80',
'HangulSyllableNyen': '\xeb\x85\xa0',
'CjkUnifiedIdeograph7e08': '\xe7\xb8\x88',
'UnicodeCharacter0xe5e8': '\xee\x97\xa8',
'ThaiDigitEight': '\xe0\xb9\x98',
'UnicodeCharacter0xecbd': '\xee\xb2\xbd',
'UnicodeCharacter0xecbe': '\xee\xb2\xbe',
'UnicodeCharacter0xecbb': '\xee\xb2\xbb',
'UnicodeCharacter0xecbc': '\xee\xb2\xbc',
'HangulSyllableNyeh': '\xeb\x85\xb7',
'CjkUnifiedIdeograph8644': '\xe8\x99\x84',
'CjkUnifiedIdeograph8645': '\xe8\x99\x85',
'CjkUnifiedIdeograph8646': '\xe8\x99\x86',
'CjkUnifiedIdeograph8647': '\xe8\x99\x87',
'CjkUnifiedIdeograph8640': '\xe8\x99\x80',
'CjkUnifiedIdeograph8641': '\xe8\x99\x81',
'CjkUnifiedIdeograph8642': '\xe8\x99\x82',
'CjkUnifiedIdeograph8643': '\xe8\x99\x83',
'BraillePatternDots234678': '\xe2\xa3\xae',
'HangulSyllableNyej': '\xeb\x85\xb2',
'CjkUnifiedIdeograph8648': '\xe8\x99\x88',
'CjkUnifiedIdeograph8649': '\xe8\x99\x89',
'RightwardsArrowWithTipUpwards': '\xe2\xac\x8f',
'HangulSyllableTwad': '\xed\x87\x83',
'CjkUnifiedIdeograph8439': '\xe8\x90\xb9',
'CjkUnifiedIdeograph5508': '\xe5\x94\x88',
'CjkUnifiedIdeograph8438': '\xe8\x90\xb8',
'CjkUnifiedIdeograph38d2': '\xe3\xa3\x92',
'NkoCombiningShortRisingTone': '\xdf\xad',
'HangulSyllableYalg': '\xec\x96\x85',
'LatinCapitalLetterOWithMiddleTilde': '\xc6\x9f',
'RejangLetterNda': '\xea\xa5\x84',
'AplFunctionalSymbolDownShoeStile': '\xe2\x8d\xa6',
'HangulSyllableNyec': '\xeb\x85\xb3',
'OriyaSignAvagraha': '\xe0\xac\xbd',
'LatinSmallLetterGWithAcute': '\xc7\xb5',
'HangulSyllableTwab': '\xed\x87\x8d',
'HangulSyllableTyeng': '\xed\x86\x99',
'CircledNumberEleven': '\xe2\x91\xaa',
'MultiplicationSignWithUnderbar': '\xe2\xa8\xb1',
'HangulSyllableTyenj': '\xed\x86\x89',
'HangulSyllableTyenh': '\xed\x86\x8a',
'LatinCapitalLetterNWithCircumflexBelow': '\xe1\xb9\x8a',
'CjkUnifiedIdeograph864d': '\xe8\x99\x8d',
'CjkUnifiedIdeograph864e': '\xe8\x99\x8e',
'CjkUnifiedIdeograph864f': '\xe8\x99\x8f',
'HangulSyllablePyih': '\xed\x94\xbb',
'CjkUnifiedIdeograph864a': '\xe8\x99\x8a',
'CjkUnifiedIdeograph864b': '\xe8\x99\x8b',
'CjkUnifiedIdeograph5501': '\xe5\x94\x81',
'HangulSyllableHegs': '\xed\x97\xa7',
'CjkUnifiedIdeograph5502': '\xe5\x94\x82',
'HangulSyllablePyin': '\xed\x94\xa4',
'GeorgianCapitalLetterMan': '\xe1\x82\xab',
'CjkUnifiedIdeograph5503': '\xe5\x94\x83',
'HangulSyllablePyim': '\xed\x94\xb0',
'MyanmarSignSection': '\xe1\x81\x8b',
'HangulSyllableSsoss': '\xec\x8f\xac',
'MongolianLetterLa': '\xe1\xa0\xaf',
'HangulSyllableGwigs': '\xea\xb7\x83',
'CjkUnifiedIdeograph3aee': '\xe3\xab\xae',
'CjkUnifiedIdeograph3aed': '\xe3\xab\xad',
'CjkUnifiedIdeograph3aef': '\xe3\xab\xaf',
'CjkUnifiedIdeograph3aea': '\xe3\xab\xaa',
'UnicodeCharacter0xf4b5': '\xef\x92\xb5',
'CjkUnifiedIdeograph3aec': '\xe3\xab\xac',
'CjkUnifiedIdeograph3aeb': '\xe3\xab\xab',
'GeorgianCapitalLetterZhar': '\xe1\x82\xaf',
'UnicodeCharacter0xfaf9': '\xef\xab\xb9',
'CjkUnifiedIdeograph8e90': '\xe8\xba\x90',
'CjkUnifiedIdeograph924c': '\xe9\x89\x8c',
'HangulSyllableJjweng': '\xec\xae\x95',
'UnicodeCharacter0xfaf8': '\xef\xab\xb8',
'BopomofoLetterAnn': '\xe3\x86\xa9',
'UnicodeCharacter0xe7d2': '\xee\x9f\x92',
'YiSyllableDdut': '\xea\x85\x8b',
'BopomofoLetterAng': '\xe3\x84\xa4',
'HangulSyllableRwaebs': '\xeb\xa2\xa6',
'UnicodeCharacter0xaa67': '\xea\xa9\xa7',
'CjkUnifiedIdeograph906c': '\xe9\x81\xac',
'CjkUnifiedIdeograph906b': '\xe9\x81\xab',
'CjkUnifiedIdeograph906a': '\xe9\x81\xaa',
'CjkUnifiedIdeograph906f': '\xe9\x81\xaf',
'CjkUnifiedIdeograph906e': '\xe9\x81\xae',
'CjkUnifiedIdeograph7c34': '\xe7\xb0\xb4',
'NewTaiLueVowelSignVowelShortener': '\xe1\xa6\xb0',
'CombiningLigatureLeftHalf': '\xef\xb8\xa0',
'UnicodeCharacter0xfaf7': '\xef\xab\xb7',
'UnicodeCharacter0xf20e': '\xef\x88\x8e',
'CjkUnifiedIdeograph3ae5': '\xe3\xab\xa5',
'CjkUnifiedIdeograph3ae4': '\xe3\xab\xa4',
'CjkUnifiedIdeograph3ae7': '\xe3\xab\xa7',
'CjkUnifiedIdeograph3ae6': '\xe3\xab\xa6',
'CjkUnifiedIdeograph3ae1': '\xe3\xab\xa1',
'CjkUnifiedIdeograph3ae0': '\xe3\xab\xa0',
'CjkUnifiedIdeograph3ae3': '\xe3\xab\xa3',
'CjkUnifiedIdeograph3ae2': '\xe3\xab\xa2',
'UnicodeCharacter0xfaf1': '\xef\xab\xb1',
'DevanagariLetterSsa': '\xe0\xa4\xb7',
'CjkUnifiedIdeograph3ae9': '\xe3\xab\xa9',
'CjkUnifiedIdeograph3ae8': '\xe3\xab\xa8',
'WhiteChessKnight': '\xe2\x99\x98',
'UnicodeCharacter0xfaf0': '\xef\xab\xb0',
'HangulSyllablePis': '\xed\x95\x8f',
'CjkUnifiedIdeograph8eb5': '\xe8\xba\xb5',
'UnicodeCharacter0xfaf3': '\xef\xab\xb3',
'CjkUnifiedIdeograph8eb4': '\xe8\xba\xb4',
'CjkUnifiedIdeograph7e0b': '\xe7\xb8\x8b',
'CjkUnifiedIdeograph9f85': '\xe9\xbe\x85',
'UnicodeCharacter0xf05d': '\xef\x81\x9d',
'UnicodeCharacter0xe689': '\xee\x9a\x89',
'UnicodeCharacter0xe93b': '\xee\xa4\xbb',
'CjkUnifiedIdeograph9063': '\xe9\x81\xa3',
'CanadianSyllabicsNe': '\xe1\x93\x80',
'CjkUnifiedIdeograph9061': '\xe9\x81\xa1',
'CanadianSyllabicsNg': '\xe1\x96\x95',
'CjkUnifiedIdeograph9067': '\xe9\x81\xa7',
'ArabicPercentSign': '\xd9\xaa',
'CjkUnifiedIdeograph9065': '\xe9\x81\xa5',
'CjkUnifiedIdeograph9064': '\xe9\x81\xa4',
'CjkUnifiedIdeograph8eb1': '\xe8\xba\xb1',
'CjkUnifiedIdeograph9069': '\xe9\x81\xa9',
'CanadianSyllabicsNo': '\xe1\x93\x84',
'CanadianSyllabicsNh': '\xe1\x93\x92',
'CanadianSyllabicsNi': '\xe1\x93\x82',
'RaisedDottedInterpolationMarker': '\xe2\xb8\x87',
'HangulSyllableHwess': '\xed\x9c\x90',
'CjkUnifiedIdeograph8eb3': '\xe8\xba\xb3',
'UnicodeCharacter0xff00': '\xef\xbc\x80',
'UnicodeCharacter0xe688': '\xee\x9a\x88',
'CjkUnifiedIdeograph8eb2': '\xe8\xba\xb2',
'UnicodeCharacter0xa9ea': '\xea\xa7\xaa',
'CjkUnifiedIdeograph843c': '\xe8\x90\xbc',
'HangulSyllableDeum': '\xeb\x93\xac',
'HangulSyllableDeul': '\xeb\x93\xa4',
'HangulSyllableDeuk': '\xeb\x93\xb4',
'HangulSyllableDeuj': '\xeb\x93\xb2',
'HangulSyllableDeuh': '\xeb\x93\xb7',
'HangulSyllableDeug': '\xeb\x93\x9d',
'HangulSyllableSsibs': '\xec\x94\xba',
'HangulSyllableDeud': '\xeb\x93\xa3',
'HangulSyllableDeuc': '\xeb\x93\xb3',
'HangulSyllableDeub': '\xeb\x93\xad',
'ArabicLigatureGhainWithMeemIsolatedForm': '\xef\xb0\xac',
'HebrewAccentOle': '\xd6\xab',
'CjkUnifiedIdeograph7301': '\xe7\x8c\x81',
'CjkUnifiedIdeograph7302': '\xe7\x8c\x82',
'CjkUnifiedIdeograph7303': '\xe7\x8c\x83',
'CjkUnifiedIdeograph7304': '\xe7\x8c\x84',
'CjkUnifiedIdeograph7305': '\xe7\x8c\x85',
'CjkUnifiedIdeograph7306': '\xe7\x8c\x86',
'CjkUnifiedIdeograph7307': '\xe7\x8c\x87',
'CjkUnifiedIdeograph7308': '\xe7\x8c\x88',
'CjkUnifiedIdeograph7309': '\xe7\x8c\x89',
'UnicodeCharacter0xde2a': '\xed\xb8\xaa',
'HangulSyllableDeut': '\xeb\x93\xb5',
'HangulSyllableDeus': '\xeb\x93\xaf',
'UnicodeCharacter0xde2f': '\xed\xb8\xaf',
'UnicodeCharacter0xde2e': '\xed\xb8\xae',
'HangulSyllableDeup': '\xeb\x93\xb6',
'CjkUnifiedIdeograph968f': '\xe9\x9a\x8f',
'CjkUnifiedIdeograph968e': '\xe9\x9a\x8e',
'CjkUnifiedIdeograph968d': '\xe9\x9a\x8d',
'CjkUnifiedIdeograph968c': '\xe9\x9a\x8c',
'KangxiRadicalCart': '\xe2\xbe\x9e',
'CjkUnifiedIdeograph968a': '\xe9\x9a\x8a',
'UnicodeCharacter0x171f': '\xe1\x9c\x9f',
'UnicodeCharacter0x171e': '\xe1\x9c\x9e',
'UnicodeCharacter0x171d': '\xe1\x9c\x9d',
'UnicodeCharacter0x171c': '\xe1\x9c\x9c',
'UnicodeCharacter0x171b': '\xe1\x9c\x9b',
'UnicodeCharacter0x171a': '\xe1\x9c\x9a',
'CjkUnifiedIdeograph4572': '\xe4\x95\xb2',
'CjkUnifiedIdeograph519f': '\xe5\x86\x9f',
'CjkUnifiedIdeograph4570': '\xe4\x95\xb0',
'CjkUnifiedIdeograph519d': '\xe5\x86\x9d',
'CjkUnifiedIdeograph4576': '\xe4\x95\xb6',
'CjkUnifiedIdeograph519b': '\xe5\x86\x9b',
'CjkUnifiedIdeograph4574': '\xe4\x95\xb4',
'CjkUnifiedIdeograph4575': '\xe4\x95\xb5',
'CjkUnifiedIdeograph8f79': '\xe8\xbd\xb9',
'CjkUnifiedIdeograph74f9': '\xe7\x93\xb9',
'CjkUnifiedIdeograph4578': '\xe4\x95\xb8',
'CjkUnifiedIdeograph4579': '\xe4\x95\xb9',
'HalfwidthKatakanaLetterMo': '\xef\xbe\x93',
'CjkUnifiedIdeograph74f8': '\xe7\x93\xb8',
'HangulSyllableKabs': '\xec\xba\x86',
'CjkUnifiedIdeograph730a': '\xe7\x8c\x8a',
'CjkUnifiedIdeograph730b': '\xe7\x8c\x8b',
'CjkUnifiedIdeograph730c': '\xe7\x8c\x8c',
'CjkUnifiedIdeograph730d': '\xe7\x8c\x8d',
'CjkUnifiedIdeograph730e': '\xe7\x8c\x8e',
'CanadianSyllabicsCaa': '\xe1\x92\x91',
'HangulSyllableBaegg': '\xeb\xb0\xb2',
'UnicodeCharacter0xde22': '\xed\xb8\xa2',
'UnicodeCharacter0xde21': '\xed\xb8\xa1',
'UnicodeCharacter0xde20': '\xed\xb8\xa0',
'UnicodeCharacter0xde27': '\xed\xb8\xa7',
'UnicodeCharacter0xde26': '\xed\xb8\xa6',
'UnicodeCharacter0xde25': '\xed\xb8\xa5',
'UnicodeCharacter0xde24': '\xed\xb8\xa4',
'TildeOperatorAboveRightwardsArrow': '\xe2\xa5\xb2',
'ArabicLetterGhain': '\xd8\xba',
'HangulSyllableBibs': '\xeb\xb9\x96',
'HangulSyllableBaegs': '\xeb\xb0\xb3',
'CjkUnifiedIdeograph74f7': '\xe7\x93\xb7',
'CjkCompatibilityIdeographfaca': '\xef\xab\x8a',
'CjkUnifiedIdeograph457b': '\xe4\x95\xbb',
'CjkUnifiedIdeograph457c': '\xe4\x95\xbc',
'CjkUnifiedIdeograph5195': '\xe5\x86\x95',
'CjkUnifiedIdeograph457a': '\xe4\x95\xba',
'LatinCapitalLetterIWithCaron': '\xc7\x8f',
'CjkUnifiedIdeograph5192': '\xe5\x86\x92',
'CjkUnifiedIdeograph5191': '\xe5\x86\x91',
'CjkUnifiedIdeograph5190': '\xe5\x86\x90',
'HangulSyllableBbess': '\xeb\xbc\x80',
'UnicodeCharacter0xdf96': '\xed\xbe\x96',
'CjkUnifiedIdeograph5199': '\xe5\x86\x99',
'CjkUnifiedIdeograph5198': '\xe5\x86\x98',
'CjkUnifiedIdeograph9687': '\xe9\x9a\x87',
'CjkUnifiedIdeograph9686': '\xe9\x9a\x86',
'CjkUnifiedIdeograph9685': '\xe9\x9a\x85',
'CjkUnifiedIdeograph9684': '\xe9\x9a\x84',
'CjkUnifiedIdeograph9683': '\xe9\x9a\x83',
'CjkUnifiedIdeograph9682': '\xe9\x9a\x82',
'UnicodeCharacter0x1719': '\xe1\x9c\x99',
'UnicodeCharacter0x1718': '\xe1\x9c\x98',
'UnicodeCharacter0x1717': '\xe1\x9c\x97',
'UnicodeCharacter0x1716': '\xe1\x9c\x96',
'UnicodeCharacter0x1715': '\xe1\x9c\x95',
'CjkCompatibilityIdeographf9f0': '\xef\xa7\xb0',
'CjkUnifiedIdeograph9689': '\xe9\x9a\x89',
'CjkUnifiedIdeograph9688': '\xe9\x9a\x88',
'HangulSyllableGgeult': '\xeb\x81\x91',
'HangulSyllableGgeuls': '\xeb\x81\x90',
'**)': '\xe2\x81\xbe',
'HangulSyllableGgeulp': '\xeb\x81\x92',
'UnicodeCharacter0xdc8f': '\xed\xb2\x8f',
'UnicodeCharacter0xdc8e': '\xed\xb2\x8e',
'UnicodeCharacter0xdc8d': '\xed\xb2\x8d',
'UnicodeCharacter0xdc8c': '\xed\xb2\x8c',
'UnicodeCharacter0xdc8b': '\xed\xb2\x8b',
'UnicodeCharacter0xdc8a': '\xed\xb2\x8a',
'CjkUnifiedIdeograph655f': '\xe6\x95\x9f',
'CjkUnifiedIdeograph655d': '\xe6\x95\x9d',
'CjkUnifiedIdeograph655e': '\xe6\x95\x9e',
'CjkUnifiedIdeograph655b': '\xe6\x95\x9b',
'CjkUnifiedIdeograph655c': '\xe6\x95\x9c',
'CjkUnifiedIdeograph655a': '\xe6\x95\x9a',
'HangulSyllableGgeulm': '\xeb\x81\x8e',
'HangulSyllableGgeulh': '\xeb\x81\x93',
'CjkUnifiedIdeograph5221': '\xe5\x88\xa1',
'CjkUnifiedIdeograph5220': '\xe5\x88\xa0',
'CjkUnifiedIdeograph5223': '\xe5\x88\xa3',
'CjkUnifiedIdeograph5222': '\xe5\x88\xa2',
'CjkUnifiedIdeograph5225': '\xe5\x88\xa5',
'CjkUnifiedIdeograph5224': '\xe5\x88\xa4',
'CjkUnifiedIdeograph5227': '\xe5\x88\xa7',
'CjkUnifiedIdeograph5226': '\xe5\x88\xa6',
'CjkUnifiedIdeograph5229': '\xe5\x88\xa9',
'CjkUnifiedIdeograph5228': '\xe5\x88\xa8',
'UnicodeCharacter0xe3f8': '\xee\x8f\xb8',
'HangulSyllableConj': '\xec\xb4\x8d',
'LatinCapitalLetterAWithCaron': '\xc7\x8d',
'UnicodeCharacter0xe686': '\xee\x9a\x86',
'HangulLetterMieumpieup': '\xe3\x85\xae',
'GreekDialytikaAndPerispomeni': '\xe1\xbf\x81',
'CjkUnifiedIdeograph58ed': '\xe5\xa3\xad',
'CjkUnifiedIdeograph58ee': '\xe5\xa3\xae',
'CjkUnifiedIdeograph58eb': '\xe5\xa3\xab',
'CjkUnifiedIdeograph58ec': '\xe5\xa3\xac',
'CjkUnifiedIdeograph58ea': '\xe5\xa3\xaa',
'ModifierLetterSmallLWithPalatalHook': '\xe1\xb6\xaa',
'CjkUnifiedIdeograph6556': '\xe6\x95\x96',
'CjkUnifiedIdeograph6557': '\xe6\x95\x97',
'CjkUnifiedIdeograph6554': '\xe6\x95\x94',
'CjkUnifiedIdeograph6555': '\xe6\x95\x95',
'CjkUnifiedIdeograph6552': '\xe6\x95\x92',
'CjkUnifiedIdeograph6553': '\xe6\x95\x93',
'CjkUnifiedIdeograph6550': '\xe6\x95\x90',
'CjkUnifiedIdeograph6551': '\xe6\x95\x91',
'CjkUnifiedIdeograph5a35': '\xe5\xa8\xb5',
'VaiSyllableRoo': '\xea\x96\x84',
'ArabicLigatureLamWithKhahWithMeemFinalForm': '\xef\xb6\x85',
'CjkUnifiedIdeograph6558': '\xe6\x95\x98',
'CjkUnifiedIdeograph6559': '\xe6\x95\x99',
'UnicodeCharacter0xf203': '\xef\x88\x83',
'LatinSmallLetterIWithInvertedBreve': '\xc8\x8b',
'UnicodeCharacter0xdc89': '\xed\xb2\x89',
'UnicodeCharacter0xdc88': '\xed\xb2\x88',
'UnicodeCharacter0xdc87': '\xed\xb2\x87',
'UnicodeCharacter0xdc86': '\xed\xb2\x86',
'UnicodeCharacter0xdc85': '\xed\xb2\x85',
'UnicodeCharacter0xdc84': '\xed\xb2\x84',
'UnicodeCharacter0xdc83': '\xed\xb2\x83',
'UnicodeCharacter0xdc82': '\xed\xb2\x82',
'UnicodeCharacter0xdc81': '\xed\xb2\x81',
'UnicodeCharacter0xdc80': '\xed\xb2\x80',
'CjkUnifiedIdeograph58e6': '\xe5\xa3\xa6',
'CjkUnifiedIdeograph58e7': '\xe5\xa3\xa7',
'CjkUnifiedIdeograph58e4': '\xe5\xa3\xa4',
'CjkUnifiedIdeograph58e5': '\xe5\xa3\xa5',
'CjkUnifiedIdeograph58e2': '\xe5\xa3\xa2',
'CjkUnifiedIdeograph58e3': '\xe5\xa3\xa3',
'CjkUnifiedIdeograph58e0': '\xe5\xa3\xa0',
'CjkUnifiedIdeograph58e1': '\xe5\xa3\xa1',
'ArabicLigatureRasoulIsolatedForm': '\xef\xb7\xb6',
'LatinSmallLetterOWithDotAbove': '\xc8\xaf',
'CjkUnifiedIdeograph97ce': '\xe9\x9f\x8e',
'CjkUnifiedIdeograph58e8': '\xe5\xa3\xa8',
'CjkUnifiedIdeograph58e9': '\xe5\xa3\xa9',
'CjkUnifiedIdeograph522a': '\xe5\x88\xaa',
'CjkUnifiedIdeograph8ebe': '\xe8\xba\xbe',
'CjkUnifiedIdeograph522c': '\xe5\x88\xac',
'CjkUnifiedIdeograph522b': '\xe5\x88\xab',
'CjkUnifiedIdeograph522e': '\xe5\x88\xae',
'CjkUnifiedIdeograph522d': '\xe5\x88\xad',
'UnicodeCharacter0xe3fd': '\xee\x8f\xbd',
'CjkUnifiedIdeograph522f': '\xe5\x88\xaf',
'SouthEastArrowWithHook': '\xe2\xa4\xa5',
'HangulSyllableJjyeot': '\xec\xaa\x9d',
'CjkUnifiedIdeograph6880': '\xe6\xa2\x80',
'CjkUnifiedIdeograph6881': '\xe6\xa2\x81',
'CjkUnifiedIdeograph6882': '\xe6\xa2\x82',
'CjkUnifiedIdeograph6883': '\xe6\xa2\x83',
'CjkUnifiedIdeograph6884': '\xe6\xa2\x84',
'CjkUnifiedIdeograph6885': '\xe6\xa2\x85',
'CjkUnifiedIdeograph6886': '\xe6\xa2\x86',
'CjkUnifiedIdeograph6887': '\xe6\xa2\x87',
'CjkUnifiedIdeograph3c3a': '\xe3\xb0\xba',
'CjkUnifiedIdeograph6889': '\xe6\xa2\x89',
'CjkUnifiedIdeograph3c3c': '\xe3\xb0\xbc',
'CjkUnifiedIdeograph3c3b': '\xe3\xb0\xbb',
'CjkUnifiedIdeograph3c3e': '\xe3\xb0\xbe',
'CjkUnifiedIdeograph3c3d': '\xe3\xb0\xbd',
'CjkUnifiedIdeograph3c3f': '\xe3\xb0\xbf',
'CjkUnifiedIdeograph88d8': '\xe8\xa3\x98',
'CjkUnifiedIdeograph88d9': '\xe8\xa3\x99',
'CjkUnifiedIdeograph8ebc': '\xe8\xba\xbc',
'CjkUnifiedIdeograph88d2': '\xe8\xa3\x92',
'CjkUnifiedIdeograph88d3': '\xe8\xa3\x93',
'CjkUnifiedIdeograph88d0': '\xe8\xa3\x90',
'CjkUnifiedIdeograph88d1': '\xe8\xa3\x91',
'CjkUnifiedIdeograph88d6': '\xe8\xa3\x96',
'CjkUnifiedIdeograph88d7': '\xe8\xa3\x97',
'CjkUnifiedIdeograph88d4': '\xe8\xa3\x94',
'CjkUnifiedIdeograph88d5': '\xe8\xa3\x95',
'ParenthesizedLatinSmallLetterM': '\xe2\x92\xa8',
'ParenthesizedLatinSmallLetterL': '\xe2\x92\xa7',
'ParenthesizedLatinSmallLetterO': '\xe2\x92\xaa',
'ParenthesizedLatinSmallLetterN': '\xe2\x92\xa9',
'ParenthesizedLatinSmallLetterI': '\xe2\x92\xa4',
'ParenthesizedLatinSmallLetterH': '\xe2\x92\xa3',
'ParenthesizedLatinSmallLetterK': '\xe2\x92\xa6',
'ParenthesizedLatinSmallLetterJ': '\xe2\x92\xa5',
'ParenthesizedLatinSmallLetterE': '\xe2\x92\xa0',
'ParenthesizedLatinSmallLetterD': '\xe2\x92\x9f',
'ParenthesizedLatinSmallLetterG': '\xe2\x92\xa2',
'ParenthesizedLatinSmallLetterF': '\xe2\x92\xa1',
'ParenthesizedLatinSmallLetterA': '\xe2\x92\x9c',
'ParenthesizedLatinSmallLetterC': '\xe2\x92\x9e',
'ParenthesizedLatinSmallLetterB': '\xe2\x92\x9d',
'AplFunctionalSymbolDownTackJot': '\xe2\x8d\x8e',
'ParenthesizedLatinSmallLetterY': '\xe2\x92\xb4',
'ParenthesizedLatinSmallLetterX': '\xe2\x92\xb3',
'UnicodeCharacter0xe637': '\xee\x98\xb7',
'ParenthesizedLatinSmallLetterZ': '\xe2\x92\xb5',
'ParenthesizedLatinSmallLetterU': '\xe2\x92\xb0',
'ParenthesizedLatinSmallLetterT': '\xe2\x92\xaf',
'ParenthesizedLatinSmallLetterW': '\xe2\x92\xb2',
'ParenthesizedLatinSmallLetterV': '\xe2\x92\xb1',
'ParenthesizedLatinSmallLetterQ': '\xe2\x92\xac',
'ParenthesizedLatinSmallLetterP': '\xe2\x92\xab',
'ParenthesizedLatinSmallLetterS': '\xe2\x92\xae',
'ParenthesizedLatinSmallLetterR': '\xe2\x92\xad',
'CjkUnifiedIdeograph97cb': '\xe9\x9f\x8b',
'CjkUnifiedIdeograph88db': '\xe8\xa3\x9b',
'CjkUnifiedIdeograph88dc': '\xe8\xa3\x9c',
'CjkUnifiedIdeograph88da': '\xe8\xa3\x9a',
'CyrillicSmallLetterAbkhasianDze': '\xd3\xa1',
'HangulSyllableJap': '\xec\x9e\xaa',
'CjkUnifiedIdeograph88dd': '\xe8\xa3\x9d',
'CjkUnifiedIdeograph88de': '\xe8\xa3\x9e',
'CjkUnifiedIdeograph3c39': '\xe3\xb0\xb9',
'CjkUnifiedIdeograph3c38': '\xe3\xb0\xb8',
'CjkUnifiedIdeograph688b': '\xe6\xa2\x8b',
'CjkUnifiedIdeograph688c': '\xe6\xa2\x8c',
'CjkUnifiedIdeograph688d': '\xe6\xa2\x8d',
'CjkUnifiedIdeograph688e': '\xe6\xa2\x8e',
'CjkUnifiedIdeograph688f': '\xe6\xa2\x8f',
'YiSyllableCap': '\xea\x8a\xb9',
'CjkUnifiedIdeograph3c31': '\xe3\xb0\xb1',
'CjkUnifiedIdeograph3c30': '\xe3\xb0\xb0',
'CjkUnifiedIdeograph3c33': '\xe3\xb0\xb3',
'CjkUnifiedIdeograph3c32': '\xe3\xb0\xb2',
'CjkUnifiedIdeograph3c35': '\xe3\xb0\xb5',
'CjkUnifiedIdeograph3c34': '\xe3\xb0\xb4',
'CjkUnifiedIdeograph3c37': '\xe3\xb0\xb7',
'CjkUnifiedIdeograph3c36': '\xe3\xb0\xb6',
'HangulSyllableGyult': '\xea\xb7\xa9',
'UnicodeCharacter0xe56b': '\xee\x95\xab',
'CjkUnifiedIdeograph9e8c': '\xe9\xba\x8c',
'CjkUnifiedIdeograph7685': '\xe7\x9a\x85',
'CjkUnifiedIdeograph7684': '\xe7\x9a\x84',
'CjkUnifiedIdeograph7687': '\xe7\x9a\x87',
'CjkUnifiedIdeograph7686': '\xe7\x9a\x86',
'CjkUnifiedIdeograph7681': '\xe7\x9a\x81',
'CjkUnifiedIdeograph7680': '\xe7\x9a\x80',
'CjkUnifiedIdeograph7683': '\xe7\x9a\x83',
'CjkUnifiedIdeograph7682': '\xe7\x9a\x82',
'CjkUnifiedIdeograph7689': '\xe7\x9a\x89',
'CjkUnifiedIdeograph7688': '\xe7\x9a\x88',
'HangulSyllableHo': '\xed\x98\xb8',
'>=': '\xe2\x89\xa5',
'UnicodeCharacter0xfafe': '\xef\xab\xbe',
'LatinCapitalLigatureIj': '\xc4\xb2',
'CjkUnifiedIdeograph3f0b': '\xe3\xbc\x8b',
'UnicodeCharacter0xfafd': '\xef\xab\xbd',
'CjkUnifiedIdeograph6e2c': '\xe6\xb8\xac',
'CjkUnifiedIdeograph6e2b': '\xe6\xb8\xab',
'CjkUnifiedIdeograph6e2a': '\xe6\xb8\xaa',
'UnicodeCharacter0xfaff': '\xef\xab\xbf',
'CjkUnifiedIdeograph6e2f': '\xe6\xb8\xaf',
'CjkUnifiedIdeograph6e2e': '\xe6\xb8\xae',
'CjkUnifiedIdeograph6e2d': '\xe6\xb8\xad',
'UnicodeCharacter0xfafa': '\xef\xab\xba',
'VaiSyllableWo': '\xea\x96\xbf',
'HangulSyllablePic': '\xed\x95\x93',
'UnicodeCharacter0xfe6e': '\xef\xb9\xae',
'HangulSyllableKigg': '\xed\x82\xa6',
'UnicodeCharacter0xfafb': '\xef\xab\xbb',
'UnicodeCharacter0xd932': '\xed\xa4\xb2',
'HangulSyllablePugg': '\xed\x91\xba',
'CjkUnifiedIdeograph61a7': '\xe6\x86\xa7',
'HangulSyllablePib': '\xed\x95\x8d',
'CopticSmallLetterLshapedHa': '\xe2\xb3\x91',
'HangulSyllablePugs': '\xed\x91\xbb',
'UnicodeCharacter0xe3db': '\xee\x8f\x9b',
'ArabicLigatureThehWithMeemIsolatedForm': '\xef\xb0\x92',
'LatinCapitalLetterIs': '\xea\x9d\xac',
'UnicodeCharacter0xf100': '\xef\x84\x80',
'YiSyllableGa': '\xea\x87\xa4',
'CjkUnifiedIdeograph61a1': '\xe6\x86\xa1',
'HangulSyllableTyot': '\xed\x88\xa9',
'CjkUnifiedIdeograph6e23': '\xe6\xb8\xa3',
'CjkUnifiedIdeograph6e22': '\xe6\xb8\xa2',
'CjkUnifiedIdeograph6e21': '\xe6\xb8\xa1',
'CjkUnifiedIdeograph6e20': '\xe6\xb8\xa0',
'CjkUnifiedIdeograph6e27': '\xe6\xb8\xa7',
'CjkUnifiedIdeograph6e26': '\xe6\xb8\xa6',
'CjkUnifiedIdeograph6e25': '\xe6\xb8\xa5',
'CjkUnifiedIdeograph6e24': '\xe6\xb8\xa4',
'CjkUnifiedIdeograph6e29': '\xe6\xb8\xa9',
'CjkUnifiedIdeograph6e28': '\xe6\xb8\xa8',
'CjkUnifiedIdeograph4a7f': '\xe4\xa9\xbf',
'CircleWithLowerHalfBlack': '\xe2\x97\x92',
'CjkUnifiedIdeograph4a7d': '\xe4\xa9\xbd',
'CjkUnifiedIdeograph4a7e': '\xe4\xa9\xbe',
'CjkUnifiedIdeograph4a7b': '\xe4\xa9\xbb',
'CjkUnifiedIdeograph4a7c': '\xe4\xa9\xbc',
'CjkUnifiedIdeograph4a7a': '\xe4\xa9\xba',
'OriyaLetterSsa': '\xe0\xac\xb7',
'CjkUnifiedIdeograph8ff3': '\xe8\xbf\xb3',
'LatinCapitalLetterWWithDotAbove': '\xe1\xba\x86',
'LatinSmallLetterOWithMacronAndGrave': '\xe1\xb9\x91',
'CjkCompatibilityIdeographfac4': '\xef\xab\x84',
'HangulSyllableBbwaet': '\xeb\xbe\x91',
'HangulSyllableMaelp': '\xeb\xa7\xb2',
'HangulSyllableMaels': '\xeb\xa7\xb0',
'HangulSyllableBbwaep': '\xeb\xbe\x92',
'HangulSyllableMaelt': '\xeb\xa7\xb1',
'VaiSyllableWu': '\xea\x96\x99',
'HangulSyllableBbwael': '\xeb\xbe\x80',
'HangulSyllableMaelh': '\xeb\xa7\xb3',
'CjkUnifiedIdeograph7e48': '\xe7\xb9\x88',
'CjkUnifiedIdeograph7e49': '\xe7\xb9\x89',
'HangulSyllableMaelm': '\xeb\xa7\xae',
'HangulSyllableBbwaej': '\xeb\xbe\x8e',
'HangulSyllableBbwaek': '\xeb\xbe\x90',
'CjkUnifiedIdeograph7e42': '\xe7\xb9\x82',
'CjkUnifiedIdeograph7e43': '\xe7\xb9\x83',
'CjkUnifiedIdeograph7e40': '\xe7\xb9\x80',
'CjkUnifiedIdeograph7e41': '\xe7\xb9\x81',
'CjkUnifiedIdeograph7e46': '\xe7\xb9\x86',
'CjkUnifiedIdeograph7e47': '\xe7\xb9\x87',
'CjkUnifiedIdeograph7e44': '\xe7\xb9\x84',
'CjkUnifiedIdeograph7e45': '\xe7\xb9\x85',
'CjkUnifiedIdeograph7236': '\xe7\x88\xb6',
'LatinCapitalLetterAWithCircumflexAndAcute': '\xe1\xba\xa4',
'CanadianSyllabicsWestcreeShwii': '\xe1\x94\x9c',
'CombiningCommaAboveRight': '\xcc\x95',
'NewTaiLueLetterHighFa': '\xe1\xa6\x9a',
'LatinSmallLetterYWithMacron': '\xc8\xb3',
'CjkUnifiedIdeograph4a74': '\xe4\xa9\xb4',
'CjkUnifiedIdeograph4a75': '\xe4\xa9\xb5',
'CjkUnifiedIdeograph4a72': '\xe4\xa9\xb2',
'LatinSmallLetterSWithCommaBelow': '\xc8\x99',
'CjkUnifiedIdeograph4a70': '\xe4\xa9\xb0',
'CjkUnifiedIdeograph4a71': '\xe4\xa9\xb1',
'ArabicLetterSeenInitialForm': '\xef\xba\xb3',
'CjkUnifiedIdeograph4a78': '\xe4\xa9\xb8',
'CjkUnifiedIdeograph4a79': '\xe4\xa9\xb9',
'HangulSyllableSsinj': '\xec\x94\xad',
'HangulSyllableSsinh': '\xec\x94\xae',
'CjkUnifiedIdeograph7e4b': '\xe7\xb9\x8b',
'CjkUnifiedIdeograph7e4c': '\xe7\xb9\x8c',
'CjkUnifiedIdeograph7e4a': '\xe7\xb9\x8a',
'CjkUnifiedIdeograph7e4f': '\xe7\xb9\x8f',
'CjkUnifiedIdeograph7e4d': '\xe7\xb9\x8d',
'CjkUnifiedIdeograph7e4e': '\xe7\xb9\x8e',
'UnicodeCharacter0xfd4b': '\xef\xb5\x8b',
'YiRadicalHxop': '\xea\x92\xbf',
'BraillePatternDots2478': '\xe2\xa3\x8a',
'CanadianSyllabicsWestcreeLwoo': '\xe1\x93\xa5',
'RejangConsonantSignNg': '\xea\xa5\x8f',
'CyrillicSmallLetterPalochka': '\xd3\x8f',
'UnicodeCharacter0xf44e': '\xef\x91\x8e',
'HangulSyllableTyalp': '\xed\x84\x86',
'KayahLiLetterNa': '\xea\xa4\x94',
'HangulSyllableTyalt': '\xed\x84\x85',
'HangulSyllableTyalh': '\xed\x84\x87',
'UnicodeCharacter0xfd4c': '\xef\xb5\x8c',
'CjkUnifiedIdeograph5a79': '\xe5\xa9\xb9',
'CjkUnifiedIdeograph5a78': '\xe5\xa9\xb8',
'CjkUnifiedIdeograph5a75': '\xe5\xa9\xb5',
'CjkUnifiedIdeograph5a74': '\xe5\xa9\xb4',
'CjkUnifiedIdeograph5a77': '\xe5\xa9\xb7',
'CjkUnifiedIdeograph5a76': '\xe5\xa9\xb6',
'CjkUnifiedIdeograph5a71': '\xe5\xa9\xb1',
'CjkUnifiedIdeograph5a70': '\xe5\xa9\xb0',
'CjkUnifiedIdeograph5a73': '\xe5\xa9\xb3',
'CjkUnifiedIdeograph5a72': '\xe5\xa9\xb2',
'CjkUnifiedIdeograph92f1': '\xe9\x8b\xb1',
'CjkUnifiedIdeograph92f0': '\xe9\x8b\xb0',
'UnicodeCharacter0xdbae': '\xed\xae\xae',
'UnicodeCharacter0xdbad': '\xed\xae\xad',
'CjkUnifiedIdeograph721a': '\xe7\x88\x9a',
'UnicodeCharacter0xdbaa': '\xed\xae\xaa',
'CjkUnifiedIdeograph92f3': '\xe9\x8b\xb3',
'UnicodeCharacter0xdbac': '\xed\xae\xac',
'UnicodeCharacter0xdbab': '\xed\xae\xab',
'CjkUnifiedIdeograph9595': '\xe9\x96\x95',
'CjkUnifiedIdeograph92f2': '\xe9\x8b\xb2',
'BlackLowerLeftTriangle': '\xe2\x97\xa3',
'CjkUnifiedIdeograph92f5': '\xe9\x8b\xb5',
'CjkUnifiedIdeograph7c8a': '\xe7\xb2\x8a',
'CjkUnifiedIdeograph92f4': '\xe9\x8b\xb4',
'CjkUnifiedIdeograph92f7': '\xe9\x8b\xb7',
'CjkUnifiedIdeograph5a7e': '\xe5\xa9\xbe',
'CjkUnifiedIdeograph5a7d': '\xe5\xa9\xbd',
'CjkUnifiedIdeograph5a7f': '\xe5\xa9\xbf',
'CjkUnifiedIdeograph5a7a': '\xe5\xa9\xba',
'FourTeardropspokedAsterisk': '\xe2\x9c\xa2',
'CjkUnifiedIdeograph5a7c': '\xe5\xa9\xbc',
'CjkUnifiedIdeograph5a7b': '\xe5\xa9\xbb',
'MyanmarLetterDda': '\xe1\x80\x8d',
'HangulSyllableMwinj': '\xeb\xae\x8d',
'CjkUnifiedIdeograph7c8b': '\xe7\xb2\x8b',
'ArabicLigatureKafWithHahInitialForm': '\xef\xb3\x85',
'LatinSmallLetterUWithTildeAndAcute': '\xe1\xb9\xb9',
'UnicodeCharacter0xfd4f': '\xef\xb5\x8f',
'CjkUnifiedIdeograph7e93': '\xe7\xba\x93',
'CanadianSyllabicsQa': '\xe1\x96\x83',
'UnicodeCharacter0xdba8': '\xed\xae\xa8',
'CjkUnifiedIdeograph704c': '\xe7\x81\x8c',
'UnicodeCharacter0xdba5': '\xed\xae\xa5',
'UnicodeCharacter0xdba4': '\xed\xae\xa4',
'CanadianSyllabicsQo': '\xe1\x96\x81',
'UnicodeCharacter0xdba6': '\xed\xae\xa6',
'CanadianSyllabicsQi': '\xe1\x95\xbf',
'TaiLeLetterFa': '\xe1\xa5\x9c',
'UnicodeCharacter0xdba3': '\xed\xae\xa3',
'UnicodeCharacter0xdba2': '\xed\xae\xa2',
'CjkUnifiedIdeograph6aec': '\xe6\xab\xac',
'ArabicLigatureJeemWithYehIsolatedForm': '\xef\xb4\x82',
'NegativeCircledNumberEighteen': '\xe2\x93\xb2',
'CjkUnifiedIdeograph7e96': '\xe7\xba\x96',
'ChamLetterFinalSs': '\xea\xa9\x8b',
'CjkUnifiedIdeograph848b': '\xe8\x92\x8b',
'CjkUnifiedIdeograph848c': '\xe8\x92\x8c',
'CjkUnifiedIdeograph848a': '\xe8\x92\x8a',
'CjkUnifiedIdeograph848f': '\xe8\x92\x8f',
'CjkUnifiedIdeograph848d': '\xe8\x92\x8d',
'CjkUnifiedIdeograph848e': '\xe8\x92\x8e',
'YiSyllableNyit': '\xea\x91\x8a',
'VaiSyllableNyee': '\xea\x94\xa3',
'HangulSyllableYegg': '\xec\x98\x8a',
'LatinCapitalLetterIWithDoubleGrave': '\xc8\x88',
'ThaiCharacterSaraAiMaimuan': '\xe0\xb9\x83',
'HangulSyllableSswaegs': '\xec\x90\x93',
'HangulSyllableDdeong': '\xeb\x96\xb5',
'CjkUnifiedIdeograph9ec6': '\xe9\xbb\x86',
'CjkUnifiedIdeograph8eed': '\xe8\xbb\xad',
'CjkUnifiedIdeograph8eee': '\xe8\xbb\xae',
'CjkUnifiedIdeograph8eef': '\xe8\xbb\xaf',
'CjkUnifiedIdeograph8eea': '\xe8\xbb\xaa',
'CjkUnifiedIdeograph8eeb': '\xe8\xbb\xab',
'CjkUnifiedIdeograph8eec': '\xe8\xbb\xac',
'CjkUnifiedIdeograph8482': '\xe8\x92\x82',
'CjkUnifiedIdeograph8483': '\xe8\x92\x83',
'CjkUnifiedIdeograph8480': '\xe8\x92\x80',
'CjkUnifiedIdeograph8481': '\xe8\x92\x81',
'CjkUnifiedIdeograph8486': '\xe8\x92\x86',
'CjkUnifiedIdeograph8487': '\xe8\x92\x87',
'CjkUnifiedIdeograph8484': '\xe8\x92\x84',
'CjkUnifiedIdeograph8485': '\xe8\x92\x85',
'CjkUnifiedIdeograph8488': '\xe8\x92\x88',
'CjkUnifiedIdeograph8489': '\xe8\x92\x89',
'YiSyllableGgut': '\xea\x88\xaa',
'YiSyllableGu': '\xea\x87\xb4',
'CjkUnifiedIdeograph7c8f': '\xe7\xb2\x8f',
'YiSyllableGgup': '\xea\x88\xad',
'HangulSyllableGalg': '\xea\xb0\x89',
'YiSyllableGgur': '\xea\x88\xaf',
'HangulSyllableGalh': '\xea\xb0\x8f',
'HangulSyllableGalm': '\xea\xb0\x8a',
'YiSyllableGgux': '\xea\x88\xab',
'HangulSyllableMyong': '\xeb\xac\xad',
'CjkUnifiedIdeograph8ee8': '\xe8\xbb\xa8',
'CjkUnifiedIdeograph8ee9': '\xe8\xbb\xa9',
'SquareOriginalOf': '\xe2\x8a\x90',
'CjkUnifiedIdeograph8ee4': '\xe8\xbb\xa4',
'CjkUnifiedIdeograph8ee5': '\xe8\xbb\xa5',
'CjkUnifiedIdeograph8ee6': '\xe8\xbb\xa6',
'CjkUnifiedIdeograph8ee7': '\xe8\xbb\xa7',
'CjkUnifiedIdeograph8ee0': '\xe8\xbb\xa0',
'CjkUnifiedIdeograph8ee1': '\xe8\xbb\xa1',
'CjkUnifiedIdeograph8ee2': '\xe8\xbb\xa2',
'CjkUnifiedIdeograph8ee3': '\xe8\xbb\xa3',
'HangulSyllableCim': '\xec\xb9\xa8',
'CjkCompatibilityIdeographf9af': '\xef\xa6\xaf',
'UnicodeCharacter0xeb7a': '\xee\xad\xba',
'CyrillicCapitalLetterGheWithStrokeAndHook': '\xd3\xba',
'CopticCapitalLetterCrossedShei': '\xe2\xb3\x82',
'UnicodeCharacter0xeb7b': '\xee\xad\xbb',
'HangulSyllableSaels': '\xec\x83\x94',
'LatinCapitalLetterIWithInvertedBreve': '\xc8\x8a',
'LatinCapitalLetterEWithTildeBelow': '\xe1\xb8\x9a',
'HangulSyllableSaelt': '\xec\x83\x95',
'UnicodeCharacter0xeb7d': '\xee\xad\xbd',
'UnicodeCharacter0xeb7e': '\xee\xad\xbe',
'HangulSyllableSaelb': '\xec\x83\x93',
'HangulSyllableSaelg': '\xec\x83\x91',
'CjkUnifiedIdeograph4750': '\xe4\x9d\x90',
'VaiSyllableKpen': '\xea\x97\xad',
'HangulSyllableSaelh': '\xec\x83\x97',
'HangulSyllableSaelm': '\xec\x83\x92',
'HalfwidthHangulLetterRieulmieum': '\xef\xbe\xab',
'CjkUnifiedIdeograph8dae': '\xe8\xb6\xae',
'UnicodeCharacter0x12c1': '\xe1\x8b\x81',
'TeluguLetterGa': '\xe0\xb0\x97',
'CjkUnifiedIdeograph8daf': '\xe8\xb6\xaf',
'CjkUnifiedIdeograph8daa': '\xe8\xb6\xaa',
'UnicodeCharacter0x2dbf': '\xe2\xb6\xbf',
'UnicodeCharacter0x12c6': '\xe1\x8b\x86',
'UnicodeCharacter0x12c7': '\xe1\x8b\x87',
'TifinaghModifierLetterLabializationMark': '\xe2\xb5\xaf',
'ArabicLigatureMeemWithMeemFinalForm': '\xef\xb2\x89',
'CjkUnifiedIdeograph92fa': '\xe9\x8b\xba',
'HangulSyllableJweos': '\xec\xa4\xab',
'HangulSyllableJweot': '\xec\xa4\xb1',
'LaoLetterThoSung': '\xe0\xba\x96',
'CjkUnifiedIdeograph9ec9': '\xe9\xbb\x89',
'HangulSyllableKyagg': '\xec\xba\xae',
'GreaterthanAboveDoublelineEqualAboveLessthan': '\xe2\xaa\x8c',
'HangulSyllableJweoc': '\xec\xa4\xaf',
'HangulSyllableJweob': '\xec\xa4\xa9',
'HangulSyllableJweod': '\xec\xa4\x9f',
'CjkUnifiedIdeograph7aaf': '\xe7\xaa\xaf',
'HangulSyllableJweoh': '\xec\xa4\xb3',
'HangulSyllableJweok': '\xec\xa4\xb0',
'HangulSyllableJweoj': '\xec\xa4\xae',
'HangulSyllableJweom': '\xec\xa4\xa8',
'HangulSyllableJweol': '\xec\xa4\xa0',
'CjkUnifiedIdeograph92fd': '\xe9\x8b\xbd',
'HangulChoseongIeungkiyeok': '\xe1\x85\x81',
'CjkUnifiedIdeograph424b': '\xe4\x89\x8b',
'CjkUnifiedIdeograph83ff': '\xe8\x8f\xbf',
'HeavyBlackCurvedDownwardsAndRightwardsArrow': '\xe2\x9e\xa5',
'CjkUnifiedIdeograph92ff': '\xe9\x8b\xbf',
'CjkUnifiedIdeograph83fe': '\xe8\x8f\xbe',
'CjkUnifiedIdeograph83fd': '\xe8\x8f\xbd',
'CjkUnifiedIdeograph8da9': '\xe8\xb6\xa9',
'CjkUnifiedIdeograph8da8': '\xe8\xb6\xa8',
'OriyaSignVirama': '\xe0\xad\x8d',
'CjkUnifiedIdeograph83fc': '\xe8\x8f\xbc',
'CjkUnifiedIdeograph8da7': '\xe8\xb6\xa7',
'CjkUnifiedIdeograph8da6': '\xe8\xb6\xa6',
'CjkUnifiedIdeograph8da1': '\xe8\xb6\xa1',
'CjkUnifiedIdeograph8da0': '\xe8\xb6\xa0',
'CjkUnifiedIdeograph8da3': '\xe8\xb6\xa3',
'CjkUnifiedIdeograph83fb': '\xe8\x8f\xbb',
'CjkUnifiedIdeograph83fa': '\xe8\x8f\xba',
'ArmenianCapitalLetterMen': '\xd5\x84',
'BoxDrawingsVerticalSingleAndHorizontalDouble': '\xe2\x95\xaa',
'infty': '\xe2\x88\x9e',
'HangulSyllableReng': '\xeb\xa0\x9d',
'GeorgianCapitalLetterCil': '\xe1\x82\xbc',
'leftarrow': '\xe2\x86\x90',
'UnicodeCharacter0xe88c': '\xee\xa2\x8c',
'UnicodeCharacter0xe88b': '\xee\xa2\x8b',
'UnicodeCharacter0xe88a': '\xee\xa2\x8a',
'UnicodeCharacter0xe88f': '\xee\xa2\x8f',
'UnicodeCharacter0xe88e': '\xee\xa2\x8e',
'UnicodeCharacter0xe88d': '\xee\xa2\x8d',
'MyanmarVowelSignSgawKarenEu': '\xe1\x81\xa2',
'CanadianSyllabicsTye': '\xe1\x95\xb0',
'HangulSyllableMa': '\xeb\xa7\x88',
'HangulSyllableMe': '\xeb\xa9\x94',
'UnicodeCharacter0xdb29': '\xed\xac\xa9',
'CjkRadicalBamboo': '\xe2\xba\xae',
'HangulSyllableMi': '\xeb\xaf\xb8',
'HangulSyllableMo': '\xeb\xaa\xa8',
'UnicodeCharacter0xf866': '\xef\xa1\xa6',
'HangulSyllableKweoh': '\xed\x80\x97',
'HangulSyllableMu': '\xeb\xac\xb4',
'BlackFlorette': '\xe2\x9c\xbf',
'UnicodeCharacter0xe883': '\xee\xa2\x83',
'UnicodeCharacter0x1c4a': '\xe1\xb1\x8a',
'UnicodeCharacter0x1c4b': '\xe1\xb1\x8b',
'UnicodeCharacter0x1c4c': '\xe1\xb1\x8c',
'UnicodeCharacter0xe887': '\xee\xa2\x87',
'UnicodeCharacter0xe886': '\xee\xa2\x86',
'UnicodeCharacter0xe885': '\xee\xa2\x85',
'UnicodeCharacter0xe884': '\xee\xa2\x84',
'HangulSyllableGgoep': '\xea\xbe\x9a',
'HangulSyllableKweol': '\xed\x80\x84',
'UnicodeCharacter0xe889': '\xee\xa2\x89',
'HangulSyllableGgoes': '\xea\xbe\x93',
'HangulSyllableGgoet': '\xea\xbe\x99',
'VaiSyllableRu': '\xea\x96\xa9',
'HangulSyllableGgoeh': '\xea\xbe\x9b',
'VaiSyllableRi': '\xea\x94\xb8',
'HangulSyllableGgoek': '\xea\xbe\x98',
'VaiSyllableRo': '\xea\x97\x90',
'HangulSyllableGgoem': '\xea\xbe\x90',
'HangulSyllableGgoen': '\xea\xbe\x84',
'UnicodeCharacter0xdb21': '\xed\xac\xa1',
'VaiSyllableRa': '\xea\x95\x9f',
'CanadianSyllabicsTyo': '\xe1\x95\xb2',
'HangulSyllableGgoed': '\xea\xbe\x87',
'HangulSyllableKwaelm': '\xec\xbe\x96',
'VaiSyllableRe': '\xea\x97\xb8',
'CanadianSyllabicsCarrierDzee': '\xe1\x99\x8b',
'SaurashtraDigitFour': '\xea\xa3\x94',
'UnicodeCharacter0xd82b': '\xed\xa0\xab',
'UnicodeCharacter0xd82c': '\xed\xa0\xac',
'UnicodeCharacter0xd82a': '\xed\xa0\xaa',
'UnicodeCharacter0xd82f': '\xed\xa0\xaf',
'HangulSyllableJwic': '\xec\xa5\xa7',
'UnicodeCharacter0xd82e': '\xed\xa0\xae',
'CjkUnifiedIdeograph82a9': '\xe8\x8a\xa9',
'CjkUnifiedIdeograph82a8': '\xe8\x8a\xa8',
'CjkUnifiedIdeograph82a7': '\xe8\x8a\xa7',
'CjkUnifiedIdeograph82a6': '\xe8\x8a\xa6',
'CjkUnifiedIdeograph82a5': '\xe8\x8a\xa5',
'CjkUnifiedIdeograph82a4': '\xe8\x8a\xa4',
'CjkUnifiedIdeograph82a3': '\xe8\x8a\xa3',
'CjkUnifiedIdeograph82a2': '\xe8\x8a\xa2',
'CjkUnifiedIdeograph82a1': '\xe8\x8a\xa1',
'CjkUnifiedIdeograph82a0': '\xe8\x8a\xa0',
'HangulSyllableJunj': '\xec\xa4\x81',
'PhagspaLetterZha': '\xea\xa1\x94',
'Asterism': '\xe2\x81\x82',
'UnicodeCharacter0xf3c9': '\xef\x8f\x89',
'TeluguVowelSignVocalicR': '\xe0\xb1\x83',
'MongolianFreeVariationSelectorThree': '\xe1\xa0\x8d',
'TeluguVowelSignVocalicL': '\xe0\xb1\xa2',
'HangulSyllableKwaelt': '\xec\xbe\x99',
'MalayalamDateMark': '\xe0\xb5\xb9',
'QuadrantLowerLeft': '\xe2\x96\x96',
'CjkUnifiedIdeograph74f6': '\xe7\x93\xb6',
'ThaiCharacterRoRua': '\xe0\xb8\xa3',
'CjkUnifiedIdeograph82af': '\xe8\x8a\xaf',
'OriyaSignNukta': '\xe0\xac\xbc',
'CjkUnifiedIdeograph82ad': '\xe8\x8a\xad',
'CjkUnifiedIdeograph82ac': '\xe8\x8a\xac',
'HangulLetterPieup': '\xe3\x85\x82',
'CjkUnifiedIdeograph7039': '\xe7\x80\xb9',
'HangulSyllableNweoss': '\xeb\x88\xb4',
'UnicodeCharacter0xd828': '\xed\xa0\xa8',
'UnicodeCharacter0x2db7': '\xe2\xb6\xb7',
'UnicodeCharacter0xd822': '\xed\xa0\xa2',
'UnicodeCharacter0xd823': '\xed\xa0\xa3',
'UnicodeCharacter0xd820': '\xed\xa0\xa0',
'CjkRadicalSnoutTwo': '\xe2\xba\x95',
'UnicodeCharacter0xd826': '\xed\xa0\xa6',
'UnicodeCharacter0xd827': '\xed\xa0\xa7',
'UnicodeCharacter0xd824': '\xed\xa0\xa4',
'UnicodeCharacter0xd825': '\xed\xa0\xa5',
'HebrewLetterFinalNun': '\xd7\x9f',
'RejangVowelSignO': '\xea\xa5\x8b',
'HangulSyllableJwinj': '\xec\xa5\x95',
'CjkUnifiedIdeograph83f7': '\xe8\x8f\xb7',
'CjkUnifiedIdeograph8b68': '\xe8\xad\xa8',
'CjkUnifiedIdeograph74f0': '\xe7\x93\xb0',
'CjkRadicalSpiritOne': '\xe2\xba\xac',
'CjkUnifiedIdeograph83f6': '\xe8\x8f\xb6',
'CjkUnifiedIdeograph8b62': '\xe8\xad\xa2',
'CjkUnifiedIdeograph8b63': '\xe8\xad\xa3',
'CjkUnifiedIdeograph8b60': '\xe8\xad\xa0',
'CjkUnifiedIdeograph8b61': '\xe8\xad\xa1',
'CjkUnifiedIdeograph8b66': '\xe8\xad\xa6',
'CjkUnifiedIdeograph83f5': '\xe8\x8f\xb5',
'Tiny': '\xe2\xa7\xbe',
'CjkUnifiedIdeograph8b65': '\xe8\xad\xa5',
'CjkUnifiedIdeograph5d3d': '\xe5\xb4\xbd',
'CjkUnifiedIdeograph5d3e': '\xe5\xb4\xbe',
'CjkUnifiedIdeograph5d3f': '\xe5\xb4\xbf',
'CjkUnifiedIdeograph7563': '\xe7\x95\xa3',
'CjkUnifiedIdeograph5d3a': '\xe5\xb4\xba',
'CjkUnifiedIdeograph5d3b': '\xe5\xb4\xbb',
'CjkUnifiedIdeograph5d3c': '\xe5\xb4\xbc',
'FarsiSymbol': '\xe2\x98\xab',
'UnicodeCharacter0x1acf': '\xe1\xab\x8f',
'UnicodeCharacter0x1ace': '\xe1\xab\x8e',
'UnicodeCharacter0x1acd': '\xe1\xab\x8d',
'UnicodeCharacter0x1acc': '\xe1\xab\x8c',
'UnicodeCharacter0x1acb': '\xe1\xab\x8b',
'UnicodeCharacter0x1aca': '\xe1\xab\x8a',
'CjkUnifiedIdeograph83f2': '\xe8\x8f\xb2',
'UnicodeCharacter0xeb78': '\xee\xad\xb8',
'BopomofoLetterSh': '\xe3\x84\x95',
'CjkUnifiedIdeograph83f1': '\xe8\x8f\xb1',
'LatinSmallLetterOWithTilde': '\xc3\xb5',
'CjkUnifiedIdeograph83f0': '\xe8\x8f\xb0',
'UnicodeCharacter0xefe7': '\xee\xbf\xa7',
'CjkUnifiedIdeograph93f4': '\xe9\x8f\xb4',
'CjkUnifiedIdeograph85aa': '\xe8\x96\xaa',
'CjkUnifiedIdeograph85ab': '\xe8\x96\xab',
'CjkUnifiedIdeograph85ac': '\xe8\x96\xac',
'CjkUnifiedIdeograph85ad': '\xe8\x96\xad',
'CjkUnifiedIdeograph85ae': '\xe8\x96\xae',
'CjkUnifiedIdeograph85af': '\xe8\x96\xaf',
'CjkUnifiedIdeograph93f3': '\xe9\x8f\xb3',
'CjkUnifiedIdeograph93f8': '\xe9\x8f\xb8',
'CjkUnifiedIdeograph93f9': '\xe9\x8f\xb9',
'CjkUnifiedIdeograph5d34': '\xe5\xb4\xb4',
'CjkUnifiedIdeograph5d35': '\xe5\xb4\xb5',
'CjkUnifiedIdeograph5d36': '\xe5\xb4\xb6',
'CjkUnifiedIdeograph5d37': '\xe5\xb4\xb7',
'CjkUnifiedIdeograph5d30': '\xe5\xb4\xb0',
'CjkUnifiedIdeograph5d31': '\xe5\xb4\xb1',
'UnicodeCharacter0x1ac9': '\xe1\xab\x89',
'UnicodeCharacter0x1ac8': '\xe1\xab\x88',
'UnicodeCharacter0x1ac7': '\xe1\xab\x87',
'UnicodeCharacter0x1ac6': '\xe1\xab\x86',
'UnicodeCharacter0x1ac5': '\xe1\xab\x85',
'UnicodeCharacter0x1ac4': '\xe1\xab\x84',
'UnicodeCharacter0x1ac3': '\xe1\xab\x83',
'UnicodeCharacter0x1ac2': '\xe1\xab\x82',
'UnicodeCharacter0x1ac1': '\xe1\xab\x81',
'UnicodeCharacter0x1ac0': '\xe1\xab\x80',
'UnicodeCharacter0xeb70': '\xee\xad\xb0',
'SaurashtraLetterVa': '\xea\xa2\xae',
'LatinCapitalLetterEzhReversed': '\xc6\xb8',
'KangxiRadicalBrush': '\xe2\xbe\x80',
'CjkUnifiedIdeograph8b6c': '\xe8\xad\xac',
'CjkUnifiedIdeograph8b6a': '\xe8\xad\xaa',
'CjkUnifiedIdeograph8b6f': '\xe8\xad\xaf',
'UnicodeCharacter0xeb72': '\xee\xad\xb2',
'CjkUnifiedIdeograph8b6d': '\xe8\xad\xad',
'CjkUnifiedIdeograph8b6e': '\xe8\xad\xae',
'CjkUnifiedIdeograph85a0': '\xe8\x96\xa0',
'CjkUnifiedIdeograph85a1': '\xe8\x96\xa1',
'CjkUnifiedIdeograph85a2': '\xe8\x96\xa2',
'CjkUnifiedIdeograph85a3': '\xe8\x96\xa3',
'CjkUnifiedIdeograph85a4': '\xe8\x96\xa4',
'CjkUnifiedIdeograph85a5': '\xe8\x96\xa5',
'CjkUnifiedIdeograph85a6': '\xe8\x96\xa6',
'CjkUnifiedIdeograph85a7': '\xe8\x96\xa7',
'CjkUnifiedIdeograph85a8': '\xe8\x96\xa8',
'CjkUnifiedIdeograph85a9': '\xe8\x96\xa9',
'BlackSquare': '\xe2\x96\xa0',
'MalayalamLetterOo': '\xe0\xb4\x93',
'UnicodeCharacter0xeb77': '\xee\xad\xb7',
'CyrillicSmallLetterAWithDiaeresis': '\xd3\x93',
'CjkUnifiedIdeograph45ad': '\xe4\x96\xad',
'CjkUnifiedIdeograph45ae': '\xe4\x96\xae',
'CjkUnifiedIdeograph45af': '\xe4\x96\xaf',
'CjkUnifiedIdeograph45aa': '\xe4\x96\xaa',
'CjkUnifiedIdeograph45ab': '\xe4\x96\xab',
'CjkUnifiedIdeograph45ac': '\xe4\x96\xac',
'HangulSyllableDdoels': '\xeb\x99\xb0',
'CjkUnifiedIdeograph96c3': '\xe9\x9b\x83',
'UnicodeCharacter0xea80': '\xee\xaa\x80',
'HangulSyllableDdoelt': '\xeb\x99\xb1',
'CjkUnifiedIdeograph435b': '\xe4\x8d\x9b',
'VerticalKanaRepeatMarkUpperHalf': '\xe3\x80\xb3',
'CjkUnifiedIdeograph9a99': '\xe9\xaa\x99',
'CjkUnifiedIdeograph435a': '\xe4\x8d\x9a',
'CjkUnifiedIdeograph435f': '\xe4\x8d\x9f',
'ChamLetterNga': '\xea\xa8\x8b',
'CjkUnifiedIdeograph435d': '\xe4\x8d\x9d',
'CjkUnifiedIdeograph435e': '\xe4\x8d\x9e',
'CjkUnifiedIdeograph9a93': '\xe9\xaa\x93',
'CjkUnifiedIdeograph9a92': '\xe9\xaa\x92',
'CjkUnifiedIdeograph9a91': '\xe9\xaa\x91',
'CjkUnifiedIdeograph9a90': '\xe9\xaa\x90',
'CjkUnifiedIdeograph9a97': '\xe9\xaa\x97',
'CjkUnifiedIdeograph9a96': '\xe9\xaa\x96',
'CjkUnifiedIdeograph9a95': '\xe9\xaa\x95',
'CjkUnifiedIdeograph9a94': '\xe9\xaa\x94',
'HangulSyllableDweog': '\xeb\x91\xad',
'UnicodeCharacter0xe06d': '\xee\x81\xad',
'HangulSyllableDweod': '\xeb\x91\xb3',
'HangulSyllableDweoc': '\xeb\x92\x83',
'HangulSyllableDweob': '\xeb\x91\xbd',
'UnicodeCharacter0xe06c': '\xee\x81\xac',
'UnicodeCharacter0xe06b': '\xee\x81\xab',
'CjkUnifiedIdeograph791e': '\xe7\xa4\x9e',
'CjkUnifiedIdeograph791d': '\xe7\xa4\x9d',
'HangulSyllableDweom': '\xeb\x91\xbc',
'CjkUnifiedIdeograph791f': '\xe7\xa4\x9f',
'CjkUnifiedIdeograph791a': '\xe7\xa4\x9a',
'HangulSyllableDweoj': '\xeb\x92\x82',
'CjkUnifiedIdeograph791c': '\xe7\xa4\x9c',
'HangulChoseongTikeut': '\xe1\x84\x83',
'UnicodeCharacter0xa99e': '\xea\xa6\x9e',
'UnicodeCharacter0xa99d': '\xea\xa6\x9d',
'UnicodeCharacter0xfdd8': '\xef\xb7\x98',
'UnicodeCharacter0xa99f': '\xea\xa6\x9f',
'UnicodeCharacter0xa99a': '\xea\xa6\x9a',
'YiRadicalNzup': '\xea\x92\xb4',
'UnicodeCharacter0xa99c': '\xea\xa6\x9c',
'UnicodeCharacter0xa99b': '\xea\xa6\x9b',
'UnicodeCharacter0xfdd2': '\xef\xb7\x92',
'UnicodeCharacter0xfdd3': '\xef\xb7\x93',
'UnicodeCharacter0xfdd0': '\xef\xb7\x90',
'CjkUnifiedIdeograph483d': '\xe4\xa0\xbd',
'UnicodeCharacter0xfdd6': '\xef\xb7\x96',
'UnicodeCharacter0xfdd7': '\xef\xb7\x97',
'UnicodeCharacter0xfdd4': '\xef\xb7\x94',
'UnicodeCharacter0xe837': '\xee\xa0\xb7',
'CjkUnifiedIdeograph4352': '\xe4\x8d\x92',
'CjkUnifiedIdeograph4353': '\xe4\x8d\x93',
'CjkUnifiedIdeograph4350': '\xe4\x8d\x90',
'CjkUnifiedIdeograph4351': '\xe4\x8d\x91',
'CjkUnifiedIdeograph4356': '\xe4\x8d\x96',
'CjkUnifiedIdeograph4357': '\xe4\x8d\x97',
'CjkUnifiedIdeograph4354': '\xe4\x8d\x94',
'CjkUnifiedIdeograph4355': '\xe4\x8d\x95',
'CjkUnifiedIdeograph9a9c': '\xe9\xaa\x9c',
'CjkUnifiedIdeograph9a9b': '\xe9\xaa\x9b',
'LatinCapitalLetterOWithHornAndTilde': '\xe1\xbb\xa0',
'CjkUnifiedIdeograph4359': '\xe4\x8d\x99',
'CjkUnifiedIdeograph9a9f': '\xe9\xaa\x9f',
'CjkUnifiedIdeograph9a9e': '\xe9\xaa\x9e',
'CjkUnifiedIdeograph9a9d': '\xe9\xaa\x9d',
'TibetanSymbolRdoRjeRgyaGram': '\xe0\xbf\x87',
'CjkUnifiedIdeograph45a5': '\xe4\x96\xa5',
'CjkUnifiedIdeograph45a6': '\xe4\x96\xa6',
'CjkUnifiedIdeograph45a7': '\xe4\x96\xa7',
'CjkUnifiedIdeograph45a0': '\xe4\x96\xa0',
'CjkUnifiedIdeograph45a1': '\xe4\x96\xa1',
'CjkUnifiedIdeograph45a2': '\xe4\x96\xa2',
'CjkUnifiedIdeograph45a3': '\xe4\x96\xa3',
'ModifierLetterCircumflexAccent': '\xcb\x86',
'CjkUnifiedIdeograph45a8': '\xe4\x96\xa8',
'CjkUnifiedIdeograph45a9': '\xe4\x96\xa9',
'UnicodeCharacter0xa995': '\xea\xa6\x95',
'UnicodeCharacter0xa994': '\xea\xa6\x94',
'UnicodeCharacter0xa997': '\xea\xa6\x97',
'YiSyllableNzix': '\xea\x8b\xae',
'UnicodeCharacter0xa991': '\xea\xa6\x91',
'UnicodeCharacter0xa990': '\xea\xa6\x90',
'UnicodeCharacter0xa993': '\xea\xa6\x93',
'UnicodeCharacter0xa992': '\xea\xa6\x92',
'UnicodeCharacter0xfddb': '\xef\xb7\x9b',
'UnicodeCharacter0xfddc': '\xef\xb7\x9c',
'UnicodeCharacter0xfdda': '\xef\xb7\x9a',
'UnicodeCharacter0xa999': '\xea\xa6\x99',
'UnicodeCharacter0xa998': '\xea\xa6\x98',
'UnicodeCharacter0xfddd': '\xef\xb7\x9d',
'CjkUnifiedIdeograph7c8e': '\xe7\xb2\x8e',
'CanadianSyllabicsQii': '\xe1\x96\x80',
'UnicodeCharacter0xe064': '\xee\x81\xa4',
'BoxDrawingsLightVerticalAndHorizontal': '\xe2\x94\xbc',
'UnicodeCharacter0xe066': '\xee\x81\xa6',
'CjkUnifiedIdeograph7919': '\xe7\xa4\x99',
'CjkUnifiedIdeograph7918': '\xe7\xa4\x98',
'UnicodeCharacter0xe063': '\xee\x81\xa3',
'UnicodeCharacter0xe062': '\xee\x81\xa2',
'CjkUnifiedIdeograph7915': '\xe7\xa4\x95',
'CjkUnifiedIdeograph7914': '\xe7\xa4\x94',
'CjkUnifiedIdeograph7917': '\xe7\xa4\x97',
'CjkUnifiedIdeograph7916': '\xe7\xa4\x96',
'CjkUnifiedIdeograph7911': '\xe7\xa4\x91',
'CjkUnifiedIdeograph7910': '\xe7\xa4\x90',
'CjkUnifiedIdeograph7913': '\xe7\xa4\x93',
'CjkUnifiedIdeograph7912': '\xe7\xa4\x92',
'CjkUnifiedIdeograph3e01': '\xe3\xb8\x81',
'CjkUnifiedIdeograph50e8': '\xe5\x83\xa8',
'CjkUnifiedIdeograph9113': '\xe9\x84\x93',
'CjkUnifiedIdeograph3e06': '\xe3\xb8\x86',
'InvertedExclamationMark': '\xc2\xa1',
'CopticSmallLetterRo': '\xe2\xb2\xa3',
'UnicodeCharacter0xa97c': '\xea\xa5\xbc',
'UnicodeCharacter0xa97b': '\xea\xa5\xbb',
'VaiSyllableBhu': '\xea\x96\x9c',
'UnicodeCharacter0xa97a': '\xea\xa5\xba',
'CjkUnifiedIdeograph540a': '\xe5\x90\x8a',
'CjkUnifiedIdeograph9b24': '\xe9\xac\xa4',
'CjkUnifiedIdeograph540c': '\xe5\x90\x8c',
'CjkUnifiedIdeograph540b': '\xe5\x90\x8b',
'IdeographicAnnotationThirdMark': '\xe3\x86\x9b',
'CjkUnifiedIdeograph540d': '\xe5\x90\x8d',
'CjkUnifiedIdeograph9b23': '\xe9\xac\xa3',
'BoxDrawingsHeavyHorizontal': '\xe2\x94\x81',
'CjkUnifiedIdeograph9b29': '\xe9\xac\xa9',
'CjkUnifiedIdeograph9b28': '\xe9\xac\xa8',
'HangulSyllableRoenh': '\xeb\xa2\xb6',
'UnicodeCharacter0xa97f': '\xea\xa5\xbf',
'HangulSyllableNabs': '\xeb\x82\xaa',
'UnicodeCharacter0xa97e': '\xea\xa5\xbe',
'LatinSmallLetterDWithLineBelow': '\xe1\xb8\x8f',
'UnicodeCharacter0xa97d': '\xea\xa5\xbd',
'HangulSyllableGgyuj': '\xeb\x80\xbe',
'CjkUnifiedIdeograph5a8e': '\xe5\xaa\x8e',
'CjkUnifiedIdeograph5a8f': '\xe5\xaa\x8f',
'UnicodeCharacter0xe4ed': '\xee\x93\xad',
'HangulSyllableGgwaenj': '\xea\xbd\xa9',
'UnicodeCharacter0xdb2a': '\xed\xac\xaa',
'HangulSyllableGgyun': '\xeb\x80\xac',
'UnicodeCharacter0xefee': '\xee\xbf\xae',
'omega': '\xcf\x89',
'UnicodeCharacter0xdb2b': '\xed\xac\xab',
'CjkUnifiedIdeograph5a8a': '\xe5\xaa\x8a',
'CjkUnifiedIdeograph9110': '\xe9\x84\x90',
'CjkUnifiedIdeograph5a8b': '\xe5\xaa\x8b',
'HangulSyllableGgyum': '\xeb\x80\xb8',
'CombiningHookAbove': '\xcc\x89',
'Tricolon': '\xe2\x81\x9d',
'CjkUnifiedIdeograph5400': '\xe5\x90\x80',
'CjkUnifiedIdeograph5403': '\xe5\x90\x83',
'CjkUnifiedIdeograph5402': '\xe5\x90\x82',
'CjkUnifiedIdeograph5405': '\xe5\x90\x85',
'CjkUnifiedIdeograph5404': '\xe5\x90\x84',
'CjkUnifiedIdeograph5407': '\xe5\x90\x87',
'CjkUnifiedIdeograph5406': '\xe5\x90\x86',
'CjkUnifiedIdeograph5409': '\xe5\x90\x89',
'CjkUnifiedIdeograph5408': '\xe5\x90\x88',
'CjkUnifiedIdeograph8048': '\xe8\x81\x88',
'CjkUnifiedIdeograph9189': '\xe9\x86\x89',
'CjkUnifiedIdeograph923a': '\xe9\x88\xba',
'CjkUnifiedIdeograph923f': '\xe9\x88\xbf',
'CjkUnifiedIdeograph923d': '\xe9\x88\xbd',
'CjkUnifiedIdeograph923e': '\xe9\x88\xbe',
'CjkUnifiedIdeograph9180': '\xe9\x86\x80',
'CjkUnifiedIdeograph9181': '\xe9\x86\x81',
'CjkUnifiedIdeograph9182': '\xe9\x86\x82',
'CjkUnifiedIdeograph9183': '\xe9\x86\x83',
'CjkUnifiedIdeograph9184': '\xe9\x86\x84',
'CjkUnifiedIdeograph9185': '\xe9\x86\x85',
'CjkUnifiedIdeograph9186': '\xe9\x86\x86',
'CjkUnifiedIdeograph9187': '\xe9\x86\x87',
'ArabicLetterSadMedialForm': '\xef\xba\xbc',
'HangulSyllableNyaen': '\xeb\x83\xb0',
'BugineseLetterNga': '\xe1\xa8\x82',
'HangulSyllableGgyip': '\xeb\x81\xba',
'HangulSyllableGyinh': '\xea\xb8\x9a',
'HangulSyllableGyinj': '\xea\xb8\x99',
'NarySquareIntersectionOperator': '\xe2\xa8\x85',
'HangulSyllableYolt': '\xec\x9a\xa1',
'HangulSyllableGying': '\xea\xb8\xa9',
'HangulSyllableYolp': '\xec\x9a\xa2',
'HangulSyllableYols': '\xec\x9a\xa0',
'CyrillicCapitalLetterKaWithHook': '\xd3\x83',
'HangulSyllableYolm': '\xec\x9a\x9e',
'HangulSyllableJjae': '\xec\xa7\xb8',
'HangulSyllableYolh': '\xec\x9a\xa3',
'DentistrySymbolLightDownAndHorizontalWithTriangle': '\xe2\x8f\x84',
'HangulSyllableYolg': '\xec\x9a\x9d',
'CanadianSyllabicsYcreeCoo': '\xe1\x92\x8f',
'HangulSyllableYolb': '\xec\x9a\x9f',
'UnicodeCharacter0xe142': '\xee\x85\x82',
'UnicodeCharacter0xe83c': '\xee\xa0\xbc',
'LeftTriangleBesideVerticalBar': '\xe2\xa7\x8f',
'CjkUnifiedIdeograph9232': '\xe9\x88\xb2',
'CjkUnifiedIdeograph9233': '\xe9\x88\xb3',
'CjkUnifiedIdeograph9230': '\xe9\x88\xb0',
'CjkUnifiedIdeograph9231': '\xe9\x88\xb1',
'CjkUnifiedIdeograph9236': '\xe9\x88\xb6',
'CjkUnifiedIdeograph9237': '\xe9\x88\xb7',
'CjkUnifiedIdeograph9234': '\xe9\x88\xb4',
'CjkUnifiedIdeograph9235': '\xe9\x88\xb5',
'HangulSyllableJess': '\xec\xa0\xb0',
'CjkUnifiedIdeograph9114': '\xe9\x84\x94',
'CjkUnifiedIdeograph918b': '\xe9\x86\x8b',
'CjkUnifiedIdeograph918c': '\xe9\x86\x8c',
'CjkUnifiedIdeograph918d': '\xe9\x86\x8d',
'CjkUnifiedIdeograph918e': '\xe9\x86\x8e',
'CjkUnifiedIdeograph918f': '\xe9\x86\x8f',
'UnicodeCharacter0xf6bb': '\xef\x9a\xbb',
'UnicodeCharacter0xf6ed': '\xef\x9b\xad',
'UnicodeCharacter0xf6ec': '\xef\x9b\xac',
'ModifierLetterExtrahighDottedLeftstemToneBar': '\xea\x9c\x8d',
'HangulSyllableBbaeg': '\xeb\xb9\xbd',
'VaiSyllableYoo': '\xea\x96\x8e',
'UnicodeCharacter0xf6eb': '\xef\x9b\xab',
'EthiopicSyllableMwee': '\xe1\x8e\x82',
'GurmukhiLetterCha': '\xe0\xa8\x9b',
'CjkUnifiedIdeograph7c81': '\xe7\xb2\x81',
'CjkUnifiedIdeograph7c82': '\xe7\xb2\x82',
'UnicodeCharacter0xf6bc': '\xef\x9a\xbc',
'UnicodeCharacter0xe6c0': '\xee\x9b\x80',
'UnicodeCharacter0xe6c1': '\xee\x9b\x81',
'UnicodeCharacter0xe6c2': '\xee\x9b\x82',
'CjkUnifiedIdeograph7c83': '\xe7\xb2\x83',
'UnicodeCharacter0xe6c4': '\xee\x9b\x84',
'UnicodeCharacter0xe6c5': '\xee\x9b\x85',
'UnicodeCharacter0xe6c6': '\xee\x9b\x86',
'UnicodeCharacter0xe6c7': '\xee\x9b\x87',
'UnicodeCharacter0xe6c8': '\xee\x9b\x88',
'CjkUnifiedIdeograph6b37': '\xe6\xac\xb7',
'CjkRadicalRap': '\xe2\xba\x99',
'DD': '\xe2\x85\x85',
'CjkUnifiedIdeograph7c85': '\xe7\xb2\x85',
'CjkUnifiedIdeograph55dd': '\xe5\x97\x9d',
'CjkRadicalRam': '\xe2\xba\xb7',
'CjkUnifiedIdeograph55df': '\xe5\x97\x9f',
'HangulSyllableBbyugs': '\xec\x80\xbf',
'CjkUnifiedIdeograph55da': '\xe5\x97\x9a',
'CjkUnifiedIdeograph55db': '\xe5\x97\x9b',
'CjkUnifiedIdeograph55dc': '\xe5\x97\x9c',
'MusicNaturalSign': '\xe2\x99\xae',
'HebrewLetterDalet': '\xd7\x93',
'CjkUnifiedIdeograph4437': '\xe4\x90\xb7',
'CjkUnifiedIdeograph4436': '\xe4\x90\xb6',
'CjkUnifiedIdeograph4435': '\xe4\x90\xb5',
'CjkUnifiedIdeograph4434': '\xe4\x90\xb4',
'CjkUnifiedIdeograph4433': '\xe4\x90\xb3',
'CjkUnifiedIdeograph4432': '\xe4\x90\xb2',
'CjkUnifiedIdeograph4431': '\xe4\x90\xb1',
'CjkUnifiedIdeograph4430': '\xe4\x90\xb0',
'UnicodeCharacter0x83b': '\xe0\xa0\xbb',
'CjkUnifiedIdeograph380f': '\xe3\xa0\x8f',
'CjkUnifiedIdeograph380e': '\xe3\xa0\x8e',
'UnicodeCharacter0x83a': '\xe0\xa0\xba',
'UnicodeCharacter0x83f': '\xe0\xa0\xbf',
'CjkUnifiedIdeograph380b': '\xe3\xa0\x8b',
'CjkUnifiedIdeograph4439': '\xe4\x90\xb9',
'UnicodeCharacter0x83e': '\xe0\xa0\xbe',
'CircledDivisionSign': '\xe2\xa8\xb8',
'UnicodeCharacter0xa972': '\xea\xa5\xb2',
'UnicodeCharacter0xf6ba': '\xef\x9a\xba',
'UnicodeCharacter0xa971': '\xea\xa5\xb1',
'CjkUnifiedIdeograph9016': '\xe9\x80\x96',
'CjkUnifiedIdeograph55d4': '\xe5\x97\x94',
'CjkUnifiedIdeograph55d5': '\xe5\x97\x95',
'CjkUnifiedIdeograph55d6': '\xe5\x97\x96',
'CjkUnifiedIdeograph55d7': '\xe5\x97\x97',
'ParenthesizedHangulRieulA': '\xe3\x88\x91',
'CjkUnifiedIdeograph55d1': '\xe5\x97\x91',
'CjkUnifiedIdeograph55d2': '\xe5\x97\x92',
'CjkUnifiedIdeograph55d3': '\xe5\x97\x93',
'UnicodeCharacter0xa977': '\xea\xa5\xb7',
'CjkUnifiedIdeograph55d8': '\xe5\x97\x98',
'CjkUnifiedIdeograph55d9': '\xe5\x97\x99',
'UnicodeCharacter0xa976': '\xea\xa5\xb6',
'ModifierLetterUpArrowhead': '\xcb\x84',
'CjkUnifiedIdeograph9118': '\xe9\x84\x98',
'UnicodeCharacter0xe6cb': '\xee\x9b\x8b',
'UnicodeCharacter0xe6cc': '\xee\x9b\x8c',
'HangulSyllableBbyaed': '\xeb\xba\xbb',
'UnicodeCharacter0xe6ce': '\xee\x9b\x8e',
'UnicodeCharacter0xe6cf': '\xee\x9b\x8f',
'UnicodeCharacter0xf6bf': '\xef\x9a\xbf',
'UnicodeCharacter0xa974': '\xea\xa5\xb4',
'CjkUnifiedIdeograph9017': '\xe9\x80\x97',
'PlusSignWithSmallCircleAbove': '\xe2\xa8\xa2',
'EthiopicSyllableDi': '\xe1\x8b\xb2',
'ModifierLetterCapitalP': '\xe1\xb4\xbe',
'CanadianSyllabicsWestcreeWa': '\xe1\x90\x98',
'ModifierLetterCapitalR': '\xe1\xb4\xbf',
'ModifierLetterCapitalU': '\xe1\xb5\x81',
'ModifierLetterCapitalT': '\xe1\xb5\x80',
'ModifierLetterCapitalW': '\xe1\xb5\x82',
'ModifierLetterCapitalV': '\xe2\xb1\xbd',
'EthiopicSyllableDa': '\xe1\x8b\xb0',
'CanadianSyllabicsWestcreeWi': '\xe1\x90\x8f',
'EthiopicSyllableDe': '\xe1\x8b\xb5',
'UnicodeCharacter0xe089': '\xee\x82\x89',
'ModifierLetterCapitalA': '\xe1\xb4\xac',
'CjkUnifiedIdeograph443f': '\xe4\x90\xbf',
'UnicodeCharacter0x838': '\xe0\xa0\xb8',
'UnicodeCharacter0x839': '\xe0\xa0\xb9',
'CjkUnifiedIdeograph443c': '\xe4\x90\xbc',
'ModifierLetterCapitalD': '\xe1\xb4\xb0',
'CjkUnifiedIdeograph443a': '\xe4\x90\xba',
'CjkUnifiedIdeograph3808': '\xe3\xa0\x88',
'UnicodeCharacter0x832': '\xe0\xa0\xb2',
'UnicodeCharacter0x833': '\xe0\xa0\xb3',
'UnicodeCharacter0x830': '\xe0\xa0\xb0',
'UnicodeCharacter0x831': '\xe0\xa0\xb1',
'UnicodeCharacter0x836': '\xe0\xa0\xb6',
'ModifierLetterCapitalL': '\xe1\xb4\xb8',
'UnicodeCharacter0x834': '\xe0\xa0\xb4',
'GeorgianSmallLetterHe': '\xe2\xb4\xa1',
'UnicodeCharacter0xea8e': '\xee\xaa\x8e',
'CurlyBracketExtension': '\xe2\x8e\xaa',
'UnicodeCharacter0xe1c3': '\xee\x87\x83',
'UnicodeCharacter0xe1c2': '\xee\x87\x82',
'UnicodeCharacter0xe1c1': '\xee\x87\x81',
'YiSyllableBbyt': '\xea\x81\xb2',
'UnicodeCharacter0xe1c7': '\xee\x87\x87',
'UnicodeCharacter0xe1c6': '\xee\x87\x86',
'HebrewLetterAyin': '\xd7\xa2',
'UnicodeCharacter0xe082': '\xee\x82\x82',
'HangulSyllableBbwaebs': '\xeb\xbe\x8a',
'CjkUnifiedIdeograph9015': '\xe9\x80\x95',
'UnicodeCharacter0xe1c9': '\xee\x87\x89',
'UnicodeCharacter0xe1c8': '\xee\x87\x88',
'HebrewLetterPe': '\xd7\xa4',
'LatinSmallLetterCWithCircumflex': '\xc4\x89',
'CjkUnifiedIdeograph8338': '\xe8\x8c\xb8',
'HangulJungseongAraea': '\xe1\x86\x9e',
'UnicodeCharacter0xe087': '\xee\x82\x87',
'YiSyllableBbyp': '\xea\x81\xb5',
'UnicodeCharacter0xe086': '\xee\x82\x86',
'UnicodeCharacter0xe748': '\xee\x9d\x88',
'KayahLiLetterVa': '\xea\xa4\xa0',
'UnicodeCharacter0xef89': '\xee\xbe\x89',
'UnicodeCharacter0xef88': '\xee\xbe\x88',
'MyanmarVowelSignGebaKarenI': '\xe1\x81\xb1',
'ArabicLigatureQalaUsedAsKoranicStopSignIsolatedForm': '\xef\xb7\xb1',
'UnicodeCharacter0xef85': '\xee\xbe\x85',
'UnicodeCharacter0xef84': '\xee\xbe\x84',
'UnicodeCharacter0xef87': '\xee\xbe\x87',
'ModifierLetterEndHighTone': '\xcb\xba',
'CjkUnifiedIdeograph86b3': '\xe8\x9a\xb3',
'UnicodeCharacter0xef80': '\xee\xbe\x80',
'UnicodeCharacter0xef83': '\xee\xbe\x83',
'UnicodeCharacter0xef82': '\xee\xbe\x82',
'HangulSyllableToek': '\xed\x88\x8c',
'UnicodeCharacter0xe1cb': '\xee\x87\x8b',
'UnicodeCharacter0xe1ca': '\xee\x87\x8a',
'UnicodeCharacter0xe1cf': '\xee\x87\x8f',
'UnicodeCharacter0xe1ce': '\xee\x87\x8e',
'UnicodeCharacter0xe1cd': '\xee\x87\x8d',
'TibetanMarkNgasBzungNyiZla': '\xe0\xbc\xb5',
'CjkUnifiedIdeograph8dea': '\xe8\xb7\xaa',
'HangulChoseongSios': '\xe1\x84\x89',
'CjkUnifiedIdeograph86b0': '\xe8\x9a\xb0',
'YiSyllableBbyx': '\xea\x81\xb3',
'PrecedesUnderRelation': '\xe2\x8a\xb0',
'CjkUnifiedIdeograph8dec': '\xe8\xb7\xac',
'HangulSyllablePoebs': '\xed\x91\x92',
'CyrillicSmallLetterHwe': '\xea\x9a\x95',
'CjkUnifiedIdeograph8deb': '\xe8\xb7\xab',
'UnicodeCharacter0xf6e9': '\xef\x9b\xa9',
'UnicodeCharacter0xef8e': '\xee\xbe\x8e',
'CjkUnifiedIdeograph8dee': '\xe8\xb7\xae',
'UnicodeCharacter0xef8f': '\xee\xbe\x8f',
'UnicodeCharacter0xef8a': '\xee\xbe\x8a',
'UnicodeCharacter0xef8c': '\xee\xbe\x8c',
'CjkUnifiedIdeograph8ded': '\xe8\xb7\xad',
'CjkUnifiedIdeograph9010': '\xe9\x80\x90',
'HangulSyllableKelg': '\xec\xbc\x89',
'CjkUnifiedIdeograph8def': '\xe8\xb7\xaf',
'HangulSyllableJjwing': '\xec\xae\xb1',
'SucceedsButNotEquivalentTo': '\xe2\x8b\xa9',
'CjkUnifiedIdeograph93ab': '\xe9\x8e\xab',
'->': '\xe2\x86\x92',
'HangulSyllableJjwinh': '\xec\xae\xa2',
'HangulSyllableJjwinj': '\xec\xae\xa1',
'LaoLetterWo': '\xe0\xba\xa7',
'BopomofoLetterEng': '\xe3\x84\xa5',
'CjkUnifiedIdeograph86b6': '\xe8\x9a\xb6',
'BopomofoLetterEnn': '\xe3\x86\xa5',
'HangulSyllableUgg': '\xec\x9a\xb2',
'CjkUnifiedIdeograph5a84': '\xe5\xaa\x84',
'CjkUnifiedIdeograph5a85': '\xe5\xaa\x85',
'CjkUnifiedIdeograph9aa4': '\xe9\xaa\xa4',
'CjkUnifiedIdeograph5a86': '\xe5\xaa\x86',
'LatinSmallLetterVWithHook': '\xca\x8b',
'HangulSyllableJjut': '\xec\xad\xa1',
'CjkUnifiedIdeograph5a80': '\xe5\xaa\x80',
'BopomofoLetterNg': '\xe3\x84\xab',
'CjkUnifiedIdeograph5a81': '\xe5\xaa\x81',
'CjkUnifiedIdeograph7c80': '\xe7\xb2\x80',
'YiSyllableShop': '\xea\x8e\xb5',
'LeftRightDoubleArrowWithStroke': '\xe2\x87\x8e',
'ArabicLetterHahInitialForm': '\xef\xba\xa3',
'UnicodeCharacter0xa6b2': '\xea\x9a\xb2',
'UnicodeCharacter0xe749': '\xee\x9d\x89',
'CjkUnifiedIdeograph5a83': '\xe5\xaa\x83',
'UnicodeCharacter0xa6b1': '\xea\x9a\xb1',
'UnicodeCharacter0xa6b0': '\xea\x9a\xb0',
'HangulSyllableToen': '\xed\x87\xb8',
'UnicodeCharacter0xa6b7': '\xea\x9a\xb7',
'CjkUnifiedIdeograph998e': '\xe9\xa6\x8e',
'UnicodeCharacter0xa6b6': '\xea\x9a\xb6',
'ArabicLigatureKafWithMeemWithMeemInitialForm': '\xef\xb7\x83',
'YiSyllableWuo': '\xea\x8a\x85',
'UnicodeCharacter0xdfb7': '\xed\xbe\xb7',
'UnicodeCharacter0xa6b5': '\xea\x9a\xb5',
'UnicodeCharacter0xf0c6': '\xef\x83\x86',
'CjkUnifiedIdeograph82d7': '\xe8\x8b\x97',
'UpperHalfCircle': '\xe2\x97\xa0',
'LatinSmallLetterDWithTopbar': '\xc6\x8c',
'HangulSyllableJjuj': '\xec\xad\x9e',
'HangulJungseongUeoeu': '\xe1\x86\x8b',
'LepchaSignRan': '\xe1\xb0\xb6',
'CjkUnifiedIdeograph7884': '\xe7\xa2\x84',
'HangulSyllableGinj': '\xea\xb8\xb5',
'HangulSyllableKyebs': '\xec\xbd\x8a',
'CjkUnifiedIdeograph4d38': '\xe4\xb4\xb8',
'varphi': '\xcf\x86',
'CjkUnifiedIdeograph3a68': '\xe3\xa9\xa8',
'CjkUnifiedIdeograph3a69': '\xe3\xa9\xa9',
'HangulSyllableGinh': '\xea\xb8\xb6',
'UnicodeCharacter0xd7b8': '\xed\x9e\xb8',
'CjkUnifiedIdeograph3a60': '\xe3\xa9\xa0',
'CjkUnifiedIdeograph3a61': '\xe3\xa9\xa1',
'CjkUnifiedIdeograph3a62': '\xe3\xa9\xa2',
'CjkUnifiedIdeograph3a63': '\xe3\xa9\xa3',
'CjkUnifiedIdeograph3a64': '\xe3\xa9\xa4',
'CjkUnifiedIdeograph3a65': '\xe3\xa9\xa5',
'CjkUnifiedIdeograph3a66': '\xe3\xa9\xa6',
'CjkUnifiedIdeograph3a67': '\xe3\xa9\xa7',
'HangulSyllableJjuk': '\xec\xad\xa0',
'ArabicLigatureNoonWithHahIsolatedForm': '\xef\xb1\x8c',
'subu': '\xe1\xb5\xa4',
'CjkUnifiedIdeograph3e9a': '\xe3\xba\x9a',
'FigureDash': '\xe2\x80\x92',
'CjkUnifiedIdeograph7494': '\xe7\x92\x94',
'CjkUnifiedIdeograph7495': '\xe7\x92\x95',
'CjkUnifiedIdeograph7496': '\xe7\x92\x96',
'BalineseDigitZero': '\xe1\xad\x90',
'CjkUnifiedIdeograph7490': '\xe7\x92\x90',
'CjkUnifiedIdeograph7491': '\xe7\x92\x91',
'CjkUnifiedIdeograph7492': '\xe7\x92\x92',
'CjkUnifiedIdeograph7493': '\xe7\x92\x93',
'CjkUnifiedIdeograph8378': '\xe8\x8d\xb8',
'HangulSyllablePyelg': '\xed\x8f\x99',
'CjkUnifiedIdeograph7498': '\xe7\x92\x98',
'Cylindricity': '\xe2\x8c\xad',
'HangulSyllablePyelh': '\xed\x8f\x9f',
'HangulSyllableKwa': '\xec\xbd\xb0',
'CjkUnifiedIdeograph7c84': '\xe7\xb2\x84',
'YiSyllableShot': '\xea\x8e\xb2',
'UnicodeCharacter0xd7b3': '\xed\x9e\xb3',
'HangulSyllableHeom': '\xed\x97\x98',
'UnicodeCharacter0xea7f': '\xee\xa9\xbf',
'LatinCapitalLetterAvWithHorizontalBar': '\xea\x9c\xba',
'HangulSyllablePyelm': '\xed\x8f\x9a',
'UnicodeCharacter0xe6e6': '\xee\x9b\xa6',
'subx': '\xe2\x82\x93',
'HangulSyllableDoess': '\xeb\x90\xac',
'CjkUnifiedIdeograph3a6a': '\xe3\xa9\xaa',
'CjkUnifiedIdeograph3a6b': '\xe3\xa9\xab',
'CjkUnifiedIdeograph3a6c': '\xe3\xa9\xac',
'CjkUnifiedIdeograph3a6d': '\xe3\xa9\xad',
'CjkUnifiedIdeograph3a6e': '\xe3\xa9\xae',
'CjkUnifiedIdeograph3a6f': '\xe3\xa9\xaf',
'UnicodeCharacter0xf7da': '\xef\x9f\x9a',
'UnicodeCharacter0xdb95': '\xed\xae\x95',
'YiSyllableCyrx': '\xea\x8b\x8e',
'HangulSyllableJjun': '\xec\xad\x8c',
'UnicodeCharacter0xdb97': '\xed\xae\x97',
'SquareGal': '\xe3\x8f\xbf',
'CjkUnifiedIdeograph749d': '\xe7\x92\x9d',
'CjkUnifiedIdeograph749e': '\xe7\x92\x9e',
'CjkUnifiedIdeograph749f': '\xe7\x92\x9f',
'UnicodeCharacter0xdb96': '\xed\xae\x96',
'CjkUnifiedIdeograph749a': '\xe7\x92\x9a',
'CjkUnifiedIdeograph749b': '\xe7\x92\x9b',
'CjkUnifiedIdeograph749c': '\xe7\x92\x9c',
'HangulSyllablePyelt': '\xed\x8f\x9d',
'CjkUnifiedIdeograph89d7': '\xe8\xa7\x97',
'CjkUnifiedIdeograph8de3': '\xe8\xb7\xa3',
'GreekUpsilonWithDiaeresisAndHookSymbol': '\xcf\x94',
'HangulSyllableDeulb': '\xeb\x93\xa7',
'CjkUnifiedIdeograph833c': '\xe8\x8c\xbc',
'HangulSyllableDeulg': '\xeb\x93\xa5',
'HangulSyllableDeulh': '\xeb\x93\xab',
'LatinCapitalLetterAWithAcute': '\xc3\x81',
'HangulSyllableToed': '\xed\x87\xbb',
'MongolianColon': '\xe1\xa0\x84',
'ArabicLigatureDadWithKhahWithMeemFinalForm': '\xef\xb5\xaf',
'HangulSyllableDeulm': '\xeb\x93\xa6',
'CjkUnifiedIdeograph8de4': '\xe8\xb7\xa4',
'HangulSyllableDeulp': '\xeb\x93\xaa',
'HangulSyllableDyeonh': '\xeb\x8e\x92',
'CjkUnifiedIdeograph7c87': '\xe7\xb2\x87',
'HangulSyllableDeult': '\xeb\x93\xa9',
'CombiningGreekDialytikaTonos': '\xcd\x84',
'subo': '\xe2\x82\x92',
'CyrillicCapitalLetterBarredO': '\xd3\xa8',
'HangulSyllableDyeonj': '\xeb\x8e\x91',
'CjkUnifiedIdeograph73a3': '\xe7\x8e\xa3',
'CjkUnifiedIdeograph73a2': '\xe7\x8e\xa2',
'CjkUnifiedIdeograph73a1': '\xe7\x8e\xa1',
'CjkUnifiedIdeograph73a0': '\xe7\x8e\xa0',
'CjkUnifiedIdeograph73a7': '\xe7\x8e\xa7',
'CjkUnifiedIdeograph73a6': '\xe7\x8e\xa6',
'CjkUnifiedIdeograph73a5': '\xe7\x8e\xa5',
'CjkUnifiedIdeograph73a4': '\xe7\x8e\xa4',
'CjkUnifiedIdeograph7f67': '\xe7\xbd\xa7',
'CjkUnifiedIdeograph7f66': '\xe7\xbd\xa6',
'CjkUnifiedIdeograph73a9': '\xe7\x8e\xa9',
'CjkUnifiedIdeograph73a8': '\xe7\x8e\xa8',
'CjkUnifiedIdeograph7f63': '\xe7\xbd\xa3',
'CjkUnifiedIdeograph7f62': '\xe7\xbd\xa2',
'CjkUnifiedIdeograph7f61': '\xe7\xbd\xa1',
'CjkUnifiedIdeograph7f60': '\xe7\xbd\xa0',
'CjkUnifiedIdeograph6484': '\xe6\x92\x84',
'HorizontalEllipsis': '\xe2\x80\xa6',
'subi': '\xe1\xb5\xa2',
'DevanagariLetterNya': '\xe0\xa4\x9e',
'KangxiRadicalBlood': '\xe2\xbe\x8e',
'CjkUnifiedIdeograph93a9': '\xe9\x8e\xa9',
'HangulSyllablePyeolh': '\xed\x8f\x83',
'CjkUnifiedIdeograph93a8': '\xe9\x8e\xa8',
'ZNotationSpot': '\xe2\xa6\x81',
'TeluguLetterJha': '\xe0\xb0\x9d',
'CjkUnifiedIdeograph73ac': '\xe7\x8e\xac',
'CjkUnifiedIdeograph73ab': '\xe7\x8e\xab',
'CjkUnifiedIdeograph73aa': '\xe7\x8e\xaa',
'CjkUnifiedIdeograph73af': '\xe7\x8e\xaf',
'CjkUnifiedIdeograph73ae': '\xe7\x8e\xae',
'CjkUnifiedIdeograph73ad': '\xe7\x8e\xad',
'UpperHalfBlock': '\xe2\x96\x80',
'CjkUnifiedIdeograph7f6f': '\xe7\xbd\xaf',
'SquareImageOf': '\xe2\x8a\x8f',
'CjkUnifiedIdeograph7f6d': '\xe7\xbd\xad',
'CjkUnifiedIdeograph7f6c': '\xe7\xbd\xac',
'SquareKuroone': '\xe3\x8c\x9b',
'CjkUnifiedIdeograph7f6a': '\xe7\xbd\xaa',
'UnicodeCharacter0xe7fa': '\xee\x9f\xba',
'HangulSyllableJjuc': '\xec\xad\x9f',
'HangulSyllableGgyaes': '\xea\xba\xb3',
'UnicodeCharacter0xef3e': '\xee\xbc\xbe',
'UnicodeCharacter0xe5b1': '\xee\x96\xb1',
'CjkUnifiedIdeograph60b6': '\xe6\x82\xb6',
'ArabicLetterSeenWithThreeDotsBelowAndThreeDotsAbove': '\xda\x9c',
'GujaratiLetterGha': '\xe0\xaa\x98',
'HangulSyllableTus': '\xed\x88\xbf',
'HangulSyllableTeugg': '\xed\x8a\xba',
'UnicodeCharacter0xda48': '\xed\xa9\x88',
'UnicodeCharacter0xef3f': '\xee\xbc\xbf',
'HangulSyllableSsyung': '\xec\x93\xa9',
'UnicodeCharacter0xf77d': '\xef\x9d\xbd',
'HangulSyllableSsyunh': '\xec\x93\x9a',
'UnicodeCharacter0xf77f': '\xef\x9d\xbf',
'HangulSyllableSsyunj': '\xec\x93\x99',
'HangulSyllableBbyaeh': '\xeb\xbb\x8f',
'UnicodeCharacter0xf77a': '\xef\x9d\xba',
'UnicodeCharacter0xf77b': '\xef\x9d\xbb',
'UnicodeCharacter0xf77c': '\xef\x9d\xbc',
'LatinCapitalLetterNWithDotAbove': '\xe1\xb9\x84',
'ArabicLetterHighHamzaAlef': '\xd9\xb5',
'ArabicLigatureHehWithJeemInitialForm': '\xef\xb3\x97',
'HebrewPointShinDot': '\xd7\x81',
'LatinSmallLetterTurnedAlpha': '\xc9\x92',
'HangulSyllableRugg': '\xeb\xa3\xaa',
'FullwidthDigitSeven': '\xef\xbc\x97',
'CjkUnifiedIdeograph5b37': '\xe5\xac\xb7',
'HangulSyllableDdeus': '\xeb\x9c\xbb',
'CjkUnifiedIdeograph5b34': '\xe5\xac\xb4',
'CjkUnifiedIdeograph5b35': '\xe5\xac\xb5',
'HangulSyllableMobs': '\xeb\xaa\xba',
'CjkUnifiedIdeograph5b32': '\xe5\xac\xb2',
'OghamFeatherMark': '\xe1\x9a\x9b',
'FullwidthVerticalLine': '\xef\xbd\x9c',
'UnicodeCharacter0xd8ff': '\xed\xa3\xbf',
'CjkUnifiedIdeograph5b33': '\xe5\xac\xb3',
'UnicodeCharacter0xf778': '\xef\x9d\xb8',
'CjkUnifiedIdeograph5b30': '\xe5\xac\xb0',
'UnicodeCharacter0xf774': '\xef\x9d\xb4',
'UnicodeCharacter0xf775': '\xef\x9d\xb5',
'UnicodeCharacter0xf776': '\xef\x9d\xb6',
'UnicodeCharacter0xf777': '\xef\x9d\xb7',
'UnicodeCharacter0xf770': '\xef\x9d\xb0',
'UnicodeCharacter0xf771': '\xef\x9d\xb1',
'UnicodeCharacter0xf772': '\xef\x9d\xb2',
'UnicodeCharacter0xf773': '\xef\x9d\xb3',
'CjkUnifiedIdeograph6996': '\xe6\xa6\x96',
'HangulSyllableDdanh': '\xeb\x94\xb6',
'CjkUnifiedIdeograph6997': '\xe6\xa6\x97',
'CjkUnifiedIdeograph8cbc': '\xe8\xb2\xbc',
'CjkUnifiedIdeograph8cbb': '\xe8\xb2\xbb',
'CjkUnifiedIdeograph8cba': '\xe8\xb2\xba',
'CjkUnifiedIdeograph8cbf': '\xe8\xb2\xbf',
'CjkUnifiedIdeograph8cbe': '\xe8\xb2\xbe',
'CjkUnifiedIdeograph8cbd': '\xe8\xb2\xbd',
'UnicodeCharacter0xab3d': '\xea\xac\xbd',
'YiSyllableSurx': '\xea\x8c\xa2',
'CjkUnifiedIdeograph6995': '\xe6\xa6\x95',
'LimbuVowelSignEe': '\xe1\xa4\xa3',
'HangulSyllableJjud': '\xec\xad\x8f',
'BalineseLetterRaRepa': '\xe1\xac\x8b',
'UnicodeCharacter0xdb9a': '\xed\xae\x9a',
'LatinSmallLetterZWithStroke': '\xc6\xb6',
'ArabicLetterYehIsolatedForm': '\xef\xbb\xb1',
'TwoJoinedSquares': '\xe2\xa7\x89',
'DoublePrecedes': '\xe2\xaa\xbb',
'TamilSignVisarga': '\xe0\xae\x83',
'UnicodeCharacter0xa6bb': '\xea\x9a\xbb',
'CjkUnifiedIdeograph8855': '\xe8\xa1\x95',
'CjkUnifiedIdeograph8854': '\xe8\xa1\x94',
'CjkUnifiedIdeograph8857': '\xe8\xa1\x97',
'CjkUnifiedIdeograph8856': '\xe8\xa1\x96',
'CjkUnifiedIdeograph8851': '\xe8\xa1\x91',
'CjkUnifiedIdeograph8850': '\xe8\xa1\x90',
'CjkUnifiedIdeograph8853': '\xe8\xa1\x93',
'CjkUnifiedIdeograph8852': '\xe8\xa1\x92',
'ArabicLetterWawWithExtendedArabicindicDigitTwoAbove': '\xdd\xb8',
'CjkUnifiedIdeograph9d2b': '\xe9\xb4\xab',
'CjkUnifiedIdeograph9d2a': '\xe9\xb4\xaa',
'CjkUnifiedIdeograph82dc': '\xe8\x8b\x9c',
'CjkUnifiedIdeograph9d2f': '\xe9\xb4\xaf',
'CjkUnifiedIdeograph9d2e': '\xe9\xb4\xae',
'CjkUnifiedIdeograph9d2d': '\xe9\xb4\xad',
'CjkUnifiedIdeograph3cdb': '\xe3\xb3\x9b',
'UnicodeCharacter0xf88e': '\xef\xa2\x8e',
'HebrewAccentYetiv': '\xd6\x9a',
'HangulSyllableDdyuss': '\xeb\x9c\xa0',
'UnicodeCharacter0xf05a': '\xef\x81\x9a',
'UnicodeCharacter0xa6be': '\xea\x9a\xbe',
'UnicodeCharacter0xd8fe': '\xed\xa3\xbe',
'HangulJongseongMieumpansios': '\xe1\x87\x9f',
'KhmerSignYuukaleapintu': '\xe1\x9f\x88',
'CjkUnifiedIdeograph8cb9': '\xe8\xb2\xb9',
'CjkUnifiedIdeograph8cb8': '\xe8\xb2\xb8',
'UnicodeCharacter0xd8bc': '\xed\xa2\xbc',
'DoubleCircledDigitSeven': '\xe2\x93\xbb',
'CjkUnifiedIdeograph8cb2': '\xe8\xb2\xb2',
'CjkUnifiedIdeograph8cb1': '\xe8\xb2\xb1',
'CjkUnifiedIdeograph8cb0': '\xe8\xb2\xb0',
'CjkUnifiedIdeograph8cb7': '\xe8\xb2\xb7',
'CjkUnifiedIdeograph8cb6': '\xe8\xb2\xb6',
'CjkUnifiedIdeograph8cb5': '\xe8\xb2\xb5',
'CjkUnifiedIdeograph8cb4': '\xe8\xb2\xb4',
'CjkUnifiedIdeograph9d23': '\xe9\xb4\xa3',
'CjkUnifiedIdeograph9d22': '\xe9\xb4\xa2',
'CjkUnifiedIdeograph9d21': '\xe9\xb4\xa1',
'CjkUnifiedIdeograph9d20': '\xe9\xb4\xa0',
'CjkUnifiedIdeograph9d27': '\xe9\xb4\xa7',
'CjkUnifiedIdeograph9d26': '\xe9\xb4\xa6',
'CjkUnifiedIdeograph9d25': '\xe9\xb4\xa5',
'MyanmarVowelSignE': '\xe1\x80\xb1',
'HangulSyllableSwaen': '\xec\x87\x88',
'HangulSyllableNeolm': '\xeb\x84\x92',
'OriyaLetterVocalicR': '\xe0\xac\x8b',
'CjkUnifiedIdeograph9d28': '\xe9\xb4\xa8',
'HangulSyllableNeolh': '\xeb\x84\x97',
'TeluguLetterDha': '\xe0\xb0\xa7',
'HangulSyllableSwaeh': '\xec\x87\x9f',
'BengaliCurrencyNumeratorOneLessThanTheDenominator': '\xe0\xa7\xb8',
'IdeographicDescriptionCharacterLeftToMiddleAndRight': '\xe2\xbf\xb2',
'HangulSyllableSwaet': '\xec\x87\x9d',
'OriyaLetterVocalicL': '\xe0\xac\x8c',
'CjkUnifiedIdeograph7a48': '\xe7\xa9\x88',
'HangulSyllableSwaep': '\xec\x87\x9e',
'HangulSyllableNeols': '\xeb\x84\x94',
'CjkUnifiedIdeograph885e': '\xe8\xa1\x9e',
'CjkUnifiedIdeograph885d': '\xe8\xa1\x9d',
'CjkUnifiedIdeograph885f': '\xe8\xa1\x9f',
'CjkUnifiedIdeograph885a': '\xe8\xa1\x9a',
'CjkUnifiedIdeograph885c': '\xe8\xa1\x9c',
'CjkUnifiedIdeograph885b': '\xe8\xa1\x9b',
'EthiopicSyllableShwa': '\xe1\x88\xbf',
'HangulSyllableGwaegg': '\xea\xb4\x9a',
'CjkRadicalFoot': '\xe2\xbb\x8a',
'UnicodeCharacter0xe45d': '\xee\x91\x9d',
'UnicodeCharacter0xe45e': '\xee\x91\x9e',
'UnicodeCharacter0xe45f': '\xee\x91\x9f',
'UnicodeCharacter0xe45a': '\xee\x91\x9a',
'CjkUnifiedIdeograph5a51': '\xe5\xa9\x91',
'UnicodeCharacter0xe45c': '\xee\x91\x9c',
'KayahLiLetterMa': '\xea\xa4\x97',
'HangulSyllableHwels': '\xed\x9c\x88',
'CjkUnifiedIdeograph814c': '\xe8\x85\x8c',
'CjkUnifiedIdeograph814b': '\xe8\x85\x8b',
'CjkUnifiedIdeograph814a': '\xe8\x85\x8a',
'CjkUnifiedIdeograph90aa': '\xe9\x82\xaa',
'HangulSyllableHes': '\xed\x97\xb7',
'CjkUnifiedIdeograph814f': '\xe8\x85\x8f',
'CjkUnifiedIdeograph814e': '\xe8\x85\x8e',
'MyanmarVowelSignI': '\xe1\x80\xad',
'HangulSyllableHeo': '\xed\x97\x88',
'HangulSyllableHen': '\xed\x97\xa8',
'HangulSyllableHem': '\xed\x97\xb4',
'HangulSyllableSsi': '\xec\x94\xa8',
'HangulSyllableHek': '\xed\x97\xbc',
'HangulSyllableSso': '\xec\x8f\x98',
'HangulSyllableHeh': '\xed\x97\xbf',
'HangulSyllableHeg': '\xed\x97\xa5',
'CjkUnifiedIdeograph4fdd': '\xe4\xbf\x9d',
'CjkUnifiedIdeograph909f': '\xe9\x82\x9f',
'HangulSyllableSsa': '\xec\x8b\xb8',
'CjkUnifiedIdeograph5a50': '\xe5\xa9\x90',
'HangulSyllableHeb': '\xed\x97\xb5',
'HangulSyllableSse': '\xec\x8e\x84',
'GeorgianLetterHar': '\xe1\x83\xb4',
'UnicodeCharacter0xe458': '\xee\x91\x98',
'HangulSyllableBeulg': '\xeb\xb8\x95',
'CopticCapitalLetterOldCopticGangia': '\xe2\xb3\x96',
'SquareBussyeru': '\xe3\x8c\xb4',
'UnicodeCharacter0xe454': '\xee\x91\x94',
'UnicodeCharacter0xe455': '\xee\x91\x95',
'HangulSyllableBeulh': '\xeb\xb8\x9b',
'UnicodeCharacter0xe457': '\xee\x91\x97',
'UnicodeCharacter0x10fe': '\xe1\x83\xbe',
'UnicodeCharacter0x10fd': '\xe1\x83\xbd',
'UnicodeCharacter0xe452': '\xee\x91\x92',
'BraillePatternDots135678': '\xe2\xa3\xb5',
'HangulSyllableBeuls': '\xeb\xb8\x98',
'HangulSyllableBeulp': '\xeb\xb8\x9a',
'|\\/': '\xe2\x86\x93',
'HangulSyllableBeult': '\xeb\xb8\x99',
'UnicodeCharacter0xdcb4': '\xed\xb2\xb4',
'HangulSyllableSsyeogg': '\xec\x8e\xa2',
'CjkUnifiedIdeograph5a57': '\xe5\xa9\x97',
'HangulSyllableGyulg': '\xea\xb7\xa5',
'UnicodeCharacter0xdcb6': '\xed\xb2\xb6',
'HangulSyllableGyulb': '\xea\xb7\xa7',
'CjkUnifiedIdeograph5d44': '\xe5\xb5\x84',
'GreekCapitalLetterOmegaWithPsiliAndPerispomeniAndProsgegrammeni': '\xe1\xbe\xae',
'HangulSyllableReuss': '\xeb\xa6\x88',
'HangulSyllableGyulm': '\xea\xb7\xa6',
'KatakanaLetterBe': '\xe3\x83\x99',
'CjkUnifiedIdeograph5b3f': '\xe5\xac\xbf',
'CjkUnifiedIdeograph8149': '\xe8\x85\x89',
'CjkUnifiedIdeograph8148': '\xe8\x85\x88',
'HangulSyllableGyuls': '\xea\xb7\xa8',
'UnicodeCharacter0xdcb3': '\xed\xb2\xb3',
'CjkUnifiedIdeograph909d': '\xe9\x82\x9d',
'CjkUnifiedIdeograph662e': '\xe6\x98\xae',
'CjkUnifiedIdeograph5a56': '\xe5\xa9\x96',
'CjkUnifiedIdeograph8142': '\xe8\x85\x82',
'CjkUnifiedIdeograph8141': '\xe8\x85\x81',
'CjkUnifiedIdeograph8140': '\xe8\x85\x80',
'CjkUnifiedIdeograph8147': '\xe8\x85\x87',
'CjkUnifiedIdeograph5b3d': '\xe5\xac\xbd',
'CjkUnifiedIdeograph8145': '\xe8\x85\x85',
'CjkUnifiedIdeograph8144': '\xe8\x85\x84',
'CjkUnifiedIdeograph5720': '\xe5\x9c\xa0',
'CjkUnifiedIdeograph5721': '\xe5\x9c\xa1',
'CjkUnifiedIdeograph5722': '\xe5\x9c\xa2',
'OriyaLetterYya': '\xe0\xad\x9f',
'LatinSmallLetterSidewaysOpenO': '\xe1\xb4\x92',
'CjkUnifiedIdeograph5725': '\xe5\x9c\xa5',
'CjkUnifiedIdeograph5726': '\xe5\x9c\xa6',
'CjkUnifiedIdeograph5727': '\xe5\x9c\xa7',
'GlagoliticCapitalLetterDjervi': '\xe2\xb0\x8c',
'CjkUnifiedIdeograph5729': '\xe5\x9c\xa9',
'HangulSyllablePwaej': '\xed\x90\xba',
'SuperscriptTwo': '\xc2\xb2',
'CjkUnifiedIdeograph5b3c': '\xe5\xac\xbc',
'CjkUnifiedIdeograph9430': '\xe9\x90\xb0',
'CjkUnifiedIdeograph9431': '\xe9\x90\xb1',
'CjkUnifiedIdeograph9432': '\xe9\x90\xb2',
'CjkUnifiedIdeograph9433': '\xe9\x90\xb3',
'CjkUnifiedIdeograph9434': '\xe9\x90\xb4',
'CjkUnifiedIdeograph9435': '\xe9\x90\xb5',
'CjkUnifiedIdeograph9436': '\xe9\x90\xb6',
'CjkUnifiedIdeograph9437': '\xe9\x90\xb7',
'CjkUnifiedIdeograph509b': '\xe5\x82\x9b',
'CjkUnifiedIdeograph509c': '\xe5\x82\x9c',
'UnicodeCharacter0xab32': '\xea\xac\xb2',
'CjkUnifiedIdeograph509a': '\xe5\x82\x9a',
'CjkUnifiedIdeograph509f': '\xe5\x82\x9f',
'UnicodeCharacter0xab35': '\xea\xac\xb5',
'CjkUnifiedIdeograph509d': '\xe5\x82\x9d',
'CjkUnifiedIdeograph509e': '\xe5\x82\x9e',
'HangulSyllableBwanj': '\xeb\xb4\x95',
'HangulSyllableBwanh': '\xeb\xb4\x96',
'UnicodeCharacter0xf453': '\xef\x91\x93',
'CjkUnifiedIdeograph5d49': '\xe5\xb5\x89',
'HangulSyllableBwang': '\xeb\xb4\xa5',
'CjkUnifiedIdeograph5d48': '\xe5\xb5\x88',
'CjkUnifiedIdeograph59b8': '\xe5\xa6\xb8',
'CjkUnifiedIdeograph59b9': '\xe5\xa6\xb9',
'CjkUnifiedIdeograph59b6': '\xe5\xa6\xb6',
'CjkUnifiedIdeograph59b7': '\xe5\xa6\xb7',
'CjkUnifiedIdeograph59b4': '\xe5\xa6\xb4',
'CjkUnifiedIdeograph59b5': '\xe5\xa6\xb5',
'CjkUnifiedIdeograph59b2': '\xe5\xa6\xb2',
'CjkUnifiedIdeograph59b3': '\xe5\xa6\xb3',
'KannadaSignNukta': '\xe0\xb2\xbc',
'CjkUnifiedIdeograph59b1': '\xe5\xa6\xb1',
'TifinaghLetterYak': '\xe2\xb4\xbd',
'TifinaghLetterYaj': '\xe2\xb4\xb6',
'ModifierLetterLowToneBar': '\xcb\xa8',
'CjkUnifiedIdeograph5099': '\xe5\x82\x99',
'CjkUnifiedIdeograph943d': '\xe9\x90\xbd',
'TifinaghLetterYan': '\xe2\xb5\x8f',
'TifinaghLetterYam': '\xe2\xb5\x8e',
'TifinaghLetterYal': '\xe2\xb5\x8d',
'CjkUnifiedIdeograph5092': '\xe5\x82\x92',
'CjkUnifiedIdeograph5093': '\xe5\x82\x93',
'TifinaghLetterYaa': '\xe2\xb5\x84',
'CjkUnifiedIdeograph5091': '\xe5\x82\x91',
'TifinaghLetterYag': '\xe2\xb4\xb3',
'TifinaghLetterYaf': '\xe2\xb4\xbc',
'CjkUnifiedIdeograph5094': '\xe5\x82\x94',
'CjkUnifiedIdeograph5095': '\xe5\x82\x95',
'HangulSyllableElt': '\xec\x97\x9d',
'TifinaghLetterYaz': '\xe2\xb5\xa3',
'TifinaghLetterYay': '\xe2\xb5\xa2',
'CjkUnifiedIdeograph572c': '\xe5\x9c\xac',
'BengaliVowelSignVocalicLl': '\xe0\xa7\xa3',
'CjkUnifiedIdeograph572e': '\xe5\x9c\xae',
'CjkUnifiedIdeograph572f': '\xe5\x9c\xaf',
'TifinaghLetterYas': '\xe2\xb5\x99',
'TifinaghLetterYar': '\xe2\xb5\x94',
'TifinaghLetterYaq': '\xe2\xb5\x87',
'TifinaghLetterYap': '\xe2\xb5\x92',
'TifinaghLetterYaw': '\xe2\xb5\xa1',
'TifinaghLetterYav': '\xe2\xb5\xa0',
'TifinaghLetterYat': '\xe2\xb5\x9c',
'UnicodeCharacter0xf455': '\xef\x91\x95',
'HangulSyllableJjyeoc': '\xec\xaa\x9b',
'HangulSyllableCweon': '\xec\xb6\xb4',
'CjkUnifiedIdeograph59bf': '\xe5\xa6\xbf',
'HangulSyllableTebs': '\xed\x85\x9e',
'CjkUnifiedIdeograph59bd': '\xe5\xa6\xbd',
'CjkUnifiedIdeograph59be': '\xe5\xa6\xbe',
'CjkUnifiedIdeograph59bb': '\xe5\xa6\xbb',
'CjkUnifiedIdeograph59bc': '\xe5\xa6\xbc',
'GlagoliticSmallLetterRitsi': '\xe2\xb1\x83',
'CjkUnifiedIdeograph775c': '\xe7\x9d\x9c',
'CjkUnifiedIdeograph775b': '\xe7\x9d\x9b',
'WestSyriacCross': '\xe2\x99\xb0',
'SquareDesi': '\xe3\x8c\xa5',
'UnicodeCharacter0xf454': '\xef\x91\x94',
'HangulSyllableTyu': '\xed\x8a\x9c',
'HangulSyllableDang': '\xeb\x8b\xb9',
'UnicodeCharacter0xf61a': '\xef\x98\x9a',
'MongolianLetterTodoPa': '\xe1\xa1\x8c',
'HangulSyllableDanh': '\xeb\x8b\xaa',
'CjkUnifiedIdeograph909a': '\xe9\x82\x9a',
'ArmenianCapitalLetterCheh': '\xd5\x83',
'HangulSyllableTye': '\xed\x86\x84',
'HangulSyllableTya': '\xed\x83\xb8',
'HangulSyllableTyo': '\xed\x88\x90',
'HangulSyllableCweog': '\xec\xb6\xb1',
'EthiopicSyllableDaa': '\xe1\x8b\xb3',
'HangulSyllableTyi': '\xed\x8b\x94',
'UnicodeCharacter0x8c': '\xc2\x8c',
'UnicodeCharacter0xed4c': '\xee\xb5\x8c',
'UnicodeCharacter0xed4b': '\xee\xb5\x8b',
'UnicodeCharacter0xed4a': '\xee\xb5\x8a',
'UnicodeCharacter0xed4f': '\xee\xb5\x8f',
'UnicodeCharacter0xed4e': '\xee\xb5\x8e',
'YiRadicalVur': '\xea\x92\xaf',
'Beta': '\xce\x92',
'MonogramForYin': '\xe2\x9a\x8b',
'UnicodeCharacter0xd98e': '\xed\xa6\x8e',
'AplFunctionalSymbolSemicolonUnderbar': '\xe2\x8d\xae',
'VaiSyllableNe': '\xea\x98\x89',
'CjkUnifiedIdeograph7289': '\xe7\x8a\x89',
'HangulSyllableSsweoc': '\xec\x92\x97',
'CjkUnifiedIdeograph7288': '\xe7\x8a\x88',
'HangulSyllableDdeul': '\xeb\x9c\xb0',
'HangulSyllableSsweod': '\xec\x92\x87',
'UnicodeCharacter0xea7e': '\xee\xa9\xbe',
'UnicodeCharacter0xe5b5': '\xee\x96\xb5',
'MongolianLetterAliGaliSsa': '\xe1\xa2\x94',
'HangulSyllableCweos': '\xec\xb7\x83',
'VaiSyllableNa': '\xea\x95\xaf',
'LatinCapitalLetterAWithBreveAndHookAbove': '\xe1\xba\xb2',
'CjkUnifiedIdeograph3ed7': '\xe3\xbb\x97',
'HangulSyllableSsweog': '\xec\x92\x81',
'CjkUnifiedIdeograph3ed4': '\xe3\xbb\x94',
'UnicodeCharacter0xea7b': '\xee\xa9\xbb',
'YiSyllableNgop': '\xea\x89\xaa',
'UnicodeCharacter0xed49': '\xee\xb5\x89',
'UnicodeCharacter0xed48': '\xee\xb5\x88',
'UnicodeCharacter0xea7a': '\xee\xa9\xba',
'DownwardsHarpoonWithBarbLeftFromBar': '\xe2\xa5\xa1',
'HangulSyllableKaelb': '\xec\xba\x9b',
'UnicodeCharacter0xed43': '\xee\xb5\x83',
'UnicodeCharacter0xed42': '\xee\xb5\x82',
'UnicodeCharacter0xed41': '\xee\xb5\x81',
'UnicodeCharacter0xed40': '\xee\xb5\x80',
'UnicodeCharacter0xed47': '\xee\xb5\x87',
'UnicodeCharacter0xed46': '\xee\xb5\x86',
'NextPage': '\xe2\x8e\x98',
'CjkUnifiedIdeograph3ed3': '\xe3\xbb\x93',
'UnicodeCharacter0xf8cc': '\xef\xa3\x8c',
'UnicodeCharacter0xf8cb': '\xef\xa3\x8b',
'HangulSyllableDwalh': '\xeb\x8f\xaf',
'CjkUnifiedIdeograph3ed0': '\xe3\xbb\x90',
'UnicodeCharacter0xf8ce': '\xef\xa3\x8e',
'CjkUnifiedIdeograph5d4d': '\xe5\xb5\x8d',
'EthiopicSyllableNwa': '\xe1\x8a\x97',
'HangulSyllableKaelg': '\xec\xba\x99',
'CjkUnifiedIdeograph3ed1': '\xe3\xbb\x91',
'HangulSyllableDwalg': '\xeb\x8f\xa9',
'EthiopicSyllableGaa': '\xe1\x8c\x8b',
'CjkUnifiedIdeograph657c': '\xe6\x95\xbc',
'HangulSyllableDdeun': '\xeb\x9c\xac',
'HangulSyllableSsweon': '\xec\x92\x84',
'CanadianSyllabicsNaa': '\xe1\x93\x88',
'HangulSyllableDwals': '\xeb\x8f\xac',
'HangulSyllableDwalp': '\xeb\x8f\xae',
'HangulSyllableDwalt': '\xeb\x8f\xad',
'CyrillicSmallLetterQa': '\xd4\x9b',
'WhitePentagon': '\xe2\xac\xa0',
'SquareKarorii': '\xe3\x8c\x8d',
'BraillePatternDots167': '\xe2\xa1\xa1',
'HangulChoseongChitueumchieuch': '\xe1\x85\x94',
'CjkUnifiedIdeograph8e49': '\xe8\xb9\x89',
'CjkUnifiedIdeograph8e48': '\xe8\xb9\x88',
'CjkUnifiedIdeograph8e47': '\xe8\xb9\x87',
'CjkUnifiedIdeograph8e46': '\xe8\xb9\x86',
'CjkUnifiedIdeograph8e45': '\xe8\xb9\x85',
'BraillePatternDots168': '\xe2\xa2\xa1',
'CjkUnifiedIdeograph8e43': '\xe8\xb9\x83',
'CjkUnifiedIdeograph8e42': '\xe8\xb9\x82',
'CircledHangulPhieuph': '\xe3\x89\xac',
'CjkUnifiedIdeograph8e40': '\xe8\xb9\x80',
'DoubleVerticalBarLeftTurnstile': '\xe2\xab\xa3',
'HangulSyllableGgyaebs': '\xea\xba\xb2',
'UnicodeCharacter0xf8c2': '\xef\xa3\x82',
'UnicodeCharacter0xf8c1': '\xef\xa3\x81',
'ArabicLetterKehehWithThreeDotsBelow': '\xd8\xbc',
'UnicodeCharacter0xf8c7': '\xef\xa3\x87',
'UnicodeCharacter0xf8c6': '\xef\xa3\x86',
'UnicodeCharacter0xf8c5': '\xef\xa3\x85',
'UnicodeCharacter0xf8c4': '\xef\xa3\x84',
'UnicodeCharacter0xf6b2': '\xef\x9a\xb2',
'UnicodeCharacter0xf8c8': '\xef\xa3\x88',
'UnionAboveBarAboveIntersection': '\xe2\xa9\x88',
'ArabicLetterDahalIsolatedForm': '\xef\xae\x84',
'CjkUnifiedIdeograph90a7': '\xe9\x82\xa7',
'CjkUnifiedIdeograph8e4f': '\xe8\xb9\x8f',
'CjkUnifiedIdeograph8e4e': '\xe8\xb9\x8e',
'CjkUnifiedIdeograph8e4d': '\xe8\xb9\x8d',
'CjkUnifiedIdeograph8e4c': '\xe8\xb9\x8c',
'CjkUnifiedIdeograph8e4b': '\xe8\xb9\x8b',
'CjkUnifiedIdeograph8e4a': '\xe8\xb9\x8a',
'UnicodeCharacter0xe5b6': '\xee\x96\xb6',
'UnicodeCharacter0xd857': '\xed\xa1\x97',
'BalineseVowelSignLaLenga': '\xe1\xac\xbc',
'UnicodeCharacter0x20c2': '\xe2\x83\x82',
'DoubleLogicalAnd': '\xe2\xa9\x93',
'UnicodeCharacter0xd86f': '\xed\xa1\xaf',
'CanadianSyllabicsCarrierHwu': '\xe1\x97\x96',
'UnicodeCharacter0xf55d': '\xef\x95\x9d',
'CanadianSyllabicsMaa': '\xe1\x92\xab',
'HangulSyllableKeulm': '\xed\x81\xb6',
'EthiopicSyllableZhaa': '\xe1\x8b\xa3',
'UnicodeCharacter0xd86d': '\xed\xa1\xad',
'CanadianSyllabicsCarrierHwe': '\xe1\x97\x98',
'HangulSyllableMyek': '\xeb\xaa\xa4',
'UnicodeCharacter0xd856': '\xed\xa1\x96',
'CanadianSyllabicsCarrierHwa': '\xe1\x97\x9b',
'CanadianSyllabicsSwii': '\xe1\x93\xba',
'UnicodeCharacter0xd86b': '\xed\xa1\xab',
'CanadianSyllabicsCarrierHwi': '\xe1\x97\x9a',
'UnicodeCharacter0xd86c': '\xed\xa1\xac',
'CjkUnifiedIdeograph92ed': '\xe9\x8b\xad',
'CjkUnifiedIdeograph425f': '\xe4\x89\x9f',
'CjkUnifiedIdeograph425e': '\xe4\x89\x9e',
'CjkUnifiedIdeograph425d': '\xe4\x89\x9d',
'CjkUnifiedIdeograph425c': '\xe4\x89\x9c',
'CjkUnifiedIdeograph425b': '\xe4\x89\x9b',
'CjkUnifiedIdeograph425a': '\xe4\x89\x9a',
'CjkUnifiedIdeograph92ec': '\xe9\x8b\xac',
'HangulSyllableMyeo': '\xeb\xa9\xb0',
'UnicodeCharacter0xf239': '\xef\x88\xb9',
'UnicodeCharacter0xf238': '\xef\x88\xb8',
'HangulSyllableBbaelt': '\xeb\xba\x89',
'HangulSyllableJjwil': '\xec\xae\xa4',
'UnicodeCharacter0xd855': '\xed\xa1\x95',
'HangulSyllableSswaenj': '\xec\x90\x95',
'UnicodeCharacter0xf55f': '\xef\x95\x9f',
'HangulSyllableDdyaeb': '\xeb\x96\x95',
'HangulSyllableDdyaec': '\xeb\x96\x9b',
'HangulSyllableDdyaeg': '\xeb\x96\x85',
'HangulSyllableDdyaed': '\xeb\x96\x8b',
'HangulSyllableDdyaej': '\xeb\x96\x9a',
'HangulSyllableDdyaek': '\xeb\x96\x9c',
'HangulSyllableDdyaeh': '\xeb\x96\x9f',
'HangulSyllableDdyaen': '\xeb\x96\x88',
'CjkUnifiedIdeograph728c': '\xe7\x8a\x8c',
'HangulSyllableDdyael': '\xeb\x96\x8c',
'HangulSyllableDdyaem': '\xeb\x96\x94',
'HangulSyllablePyeolb': '\xed\x8e\xbf',
'CjkUnifiedIdeograph75bd': '\xe7\x96\xbd',
'HangulSyllableDdyaep': '\xeb\x96\x9e',
'CjkUnifiedIdeograph728b': '\xe7\x8a\x8b',
'HangulSyllableDdyaet': '\xeb\x96\x9d',
'KannadaDigitThree': '\xe0\xb3\xa9',
'HangulSyllableKwass': '\xec\xbe\x84',
'NaryLogicalOr': '\xe2\x8b\x81',
'HangulSyllableMeo': '\xeb\xa8\xb8',
'HangulSyllableDdalp': '\xeb\x94\xbe',
'UnicodeCharacter0xea78': '\xee\xa9\xb8',
'ArabicLigatureAinWithMeemWithMeemInitialForm': '\xef\xb5\xb7',
'LatinSmallLetterOWithCircumflexAndAcute': '\xe1\xbb\x91',
'HangulSyllableSyeogg': '\xec\x85\x96',
'CjkUnifiedIdeograph4257': '\xe4\x89\x97',
'CjkUnifiedIdeograph4256': '\xe4\x89\x96',
'CjkUnifiedIdeograph4255': '\xe4\x89\x95',
'CjkUnifiedIdeograph4254': '\xe4\x89\x94',
'CjkUnifiedIdeograph4253': '\xe4\x89\x93',
'CjkUnifiedIdeograph4252': '\xe4\x89\x92',
'CjkUnifiedIdeograph4251': '\xe4\x89\x91',
'CjkUnifiedIdeograph4250': '\xe4\x89\x90',
'CjkUnifiedIdeograph92e8': '\xe9\x8b\xa8',
'CjkUnifiedIdeograph92e9': '\xe9\x8b\xa9',
'CjkUnifiedIdeograph4259': '\xe4\x89\x99',
'CjkUnifiedIdeograph4258': '\xe4\x89\x98',
'CanadianSyllabicsKwi': '\xe1\x91\xb6',
'CjkUnifiedIdeograph3edf': '\xe3\xbb\x9f',
'UnicodeCharacter0xea74': '\xee\xa9\xb4',
'CanadianSyllabicsKwo': '\xe1\x91\xba',
'HangulSyllableMac': '\xeb\xa7\x9f',
'CanadianSyllabicsKwa': '\xe1\x91\xbe',
'UnicodeCharacter0xea73': '\xee\xa9\xb3',
'UnicodeCharacter0xefbe': '\xee\xbe\xbe',
'UnicodeCharacter0xefbd': '\xee\xbe\xbd',
'CanadianSyllabicsKwe': '\xe1\x91\xb4',
'CjkUnifiedIdeograph3edd': '\xe3\xbb\x9d',
'UnicodeCharacter0xefba': '\xee\xbe\xba',
'HangulSyllableRip': '\xeb\xa7\x86',
'cs': '\xcf\x82',
'HangulSyllableRyeogg': '\xeb\xa0\xa6',
'CjkUnifiedIdeograph3ede': '\xe3\xbb\x9e',
'HebrewLetterBet': '\xd7\x91',
'cr': '\xcf\xb1',
'UnicodeCharacter0xf4cf': '\xef\x93\x8f',
'CjkUnifiedIdeograph3edb': '\xe3\xbb\x9b',
'OghamLetterGort': '\xe1\x9a\x8c',
'GurmukhiLetterYa': '\xe0\xa8\xaf',
'CjkUnifiedIdeograph3edc': '\xe3\xbb\x9c',
'EquiangularTo': '\xe2\x89\x9a',
'HangulSyllableRim': '\xeb\xa6\xbc',
'cp': '\xcf\x96',
'YiSyllableTep': '\xea\x84\xaf',
'HangulSyllableMad': '\xeb\xa7\x8f',
'UnicodeCharacter0xea44': '\xee\xa9\x84',
'BraillePatternDots24578': '\xe2\xa3\x9a',
'UnicodeCharacter0xd851': '\xed\xa1\x91',
'CjkUnifiedIdeograph3eda': '\xe3\xbb\x9a',
'YiSyllableTex': '\xea\x84\xad',
'CjkUnifiedIdeograph7759': '\xe7\x9d\x99',
'CyrillicCapitalLetterShortIWithTail': '\xd2\x8a',
'SaurashtraDanda': '\xea\xa3\x8e',
'KhmerDigitFour': '\xe1\x9f\xa4',
'HangulSyllableMah': '\xeb\xa7\xa3',
'HangulSyllableRih': '\xeb\xa7\x87',
'VaiSyllableDhha': '\xea\x95\x9d',
'SyriacLetterE': '\xdc\xa5',
'VaiSyllableDhhe': '\xea\x97\xb6',
'UnicodeCharacter0xd850': '\xed\xa1\x90',
'KangxiRadicalNot': '\xe2\xbd\x86',
'VaiSyllableDhhi': '\xea\x94\xb6',
'HangulSyllableRij': '\xeb\xa7\x82',
'VaiSyllableDhho': '\xea\x97\x8e',
'SmallNumberSign': '\xef\xb9\x9f',
'VaiSyllableDhhu': '\xea\x96\xa7',
'HangulSyllableRid': '\xeb\xa6\xb3',
'UnicodeCharacter0xefb7': '\xee\xbe\xb7',
'UnicodeCharacter0xefb6': '\xee\xbe\xb6',
'UnicodeCharacter0xefb5': '\xee\xbe\xb5',
'UnicodeCharacter0xefb4': '\xee\xbe\xb4',
'UnicodeCharacter0xefb3': '\xee\xbe\xb3',
'HangulSyllableRig': '\xeb\xa6\xad',
'UnicodeCharacter0xefb1': '\xee\xbe\xb1',
'UnicodeCharacter0xefb0': '\xee\xbe\xb0',
'HangulSyllableDdalt': '\xeb\x94\xbd',
'SinhalaSignAllakuna': '\xe0\xb7\x8a',
'UnicodeCharacter0xe9be': '\xee\xa6\xbe',
'HangulSyllableNeoss': '\xeb\x84\x9c',
'YiRadicalGgop': '\xea\x92\x96',
'NewTaiLueDigitOne': '\xe1\xa7\x91',
'HangulSyllableRic': '\xeb\xa7\x83',
'HangulSyllableRib': '\xeb\xa6\xbd',
'UnicodeCharacter0xd98f': '\xed\xa6\x8f',
'YiSyllableRruox': '\xea\x8d\xbe',
'CyrillicCapitalLetterDze': '\xd0\x85',
'GurmukhiLetterU': '\xe0\xa8\x89',
'UnicodeCharacter0xe016': '\xee\x80\x96',
'GurmukhiLetterI': '\xe0\xa8\x87',
'CjkUnifiedIdeograph937f': '\xe9\x8d\xbf',
'CjkUnifiedIdeograph937e': '\xe9\x8d\xbe',
'CjkUnifiedIdeograph937d': '\xe9\x8d\xbd',
'CjkUnifiedIdeograph937c': '\xe9\x8d\xbc',
'CjkUnifiedIdeograph937b': '\xe9\x8d\xbb',
'CjkUnifiedIdeograph937a': '\xe9\x8d\xba',
'GurmukhiLetterA': '\xe0\xa8\x85',
'ArabicLigatureTahWithMeemInitialForm': '\xef\xb4\xb3',
'UnicodeCharacter0xe011': '\xee\x80\x91',
'HangulSyllableDdat': '\xeb\x95\x89',
'ArabicLetterFarsiYehInitialForm': '\xef\xaf\xbe',
'HangulSyllableDdas': '\xeb\x95\x83',
'HangulSyllableDdap': '\xeb\x95\x8a',
'HangulSyllableJjeog': '\xec\xa9\x8d',
'HangulSyllableJjegg': '\xec\xa9\xaa',
'UnicodeCharacter0xefff': '\xee\xbf\xbf',
'UnicodeCharacter0xe013': '\xee\x80\x93',
'CjkUnifiedIdeograph99de': '\xe9\xa7\x9e',
'HangulSyllableDdag': '\xeb\x94\xb1',
'BraillePatternDots13457': '\xe2\xa1\x9d',
'BraillePatternDots13456': '\xe2\xa0\xbd',
'HangulSyllableDdab': '\xeb\x95\x81',
'HangulSyllableDdac': '\xeb\x95\x87',
'LatinSmallLetterUWithHornAndHookAbove': '\xe1\xbb\xad',
'CjkUnifiedIdeograph7cdd': '\xe7\xb3\x9d',
'CjkUnifiedIdeograph7cde': '\xe7\xb3\x9e',
'CjkUnifiedIdeograph7cdf': '\xe7\xb3\x9f',
'HangulSyllableDdam': '\xeb\x95\x80',
'HangulSyllableDdaj': '\xeb\x95\x86',
'BraillePatternDots13458': '\xe2\xa2\x9d',
'CjkUnifiedIdeograph7cdb': '\xe7\xb3\x9b',
'CjkUnifiedIdeograph7cdc': '\xe7\xb3\x9c',
'CjkUnifiedIdeograph9377': '\xe9\x8d\xb7',
'CjkUnifiedIdeograph9376': '\xe9\x8d\xb6',
'CjkUnifiedIdeograph9375': '\xe9\x8d\xb5',
'CjkUnifiedIdeograph9374': '\xe9\x8d\xb4',
'CjkUnifiedIdeograph9373': '\xe9\x8d\xb3',
'CjkUnifiedIdeograph9372': '\xe9\x8d\xb2',
'CjkUnifiedIdeograph9371': '\xe9\x8d\xb1',
'CjkUnifiedIdeograph9370': '\xe9\x8d\xb0',
'CjkUnifiedIdeograph9768': '\xe9\x9d\xa8',
'J': '\xce\xa6',
'CjkUnifiedIdeograph951d': '\xe9\x94\x9d',
'HangulSyllableJwelb': '\xec\xa4\xbf',
'CjkUnifiedIdeograph9379': '\xe9\x8d\xb9',
'CjkUnifiedIdeograph9378': '\xe9\x8d\xb8',
'HangulSyllableSsebs': '\xec\x8e\x96',
'TibetanSubjoinedLetterDa': '\xe0\xbe\xa1',
'ModifierLetterBeginHighTone': '\xcb\xb9',
'CjkUnifiedIdeograph7cd8': '\xe7\xb3\x98',
'CjkUnifiedIdeograph7cd9': '\xe7\xb3\x99',
'UnicodeCharacter0xd8e9': '\xed\xa3\xa9',
'UnicodeCharacter0xd8e8': '\xed\xa3\xa8',
'CjkUnifiedIdeograph7cd4': '\xe7\xb3\x94',
'CjkUnifiedIdeograph7cd5': '\xe7\xb3\x95',
'HangulJongseongMieumsios': '\xe1\x87\x9d',
'CjkUnifiedIdeograph7cd7': '\xe7\xb3\x97',
'CjkUnifiedIdeograph7cd0': '\xe7\xb3\x90',
'CjkUnifiedIdeograph7cd1': '\xe7\xb3\x91',
'CjkUnifiedIdeograph7cd2': '\xe7\xb3\x92',
'CjkUnifiedIdeograph7cd3': '\xe7\xb3\x93',
'HangulSyllableJjeoh': '\xec\xa9\xa7',
'SaurashtraLetterIi': '\xea\xa2\x85',
'HangulSyllableWaegg': '\xec\x99\x9e',
'CopticCapitalLetterOldNubianShima': '\xe2\xb3\x9c',
'LatinSmallLetterEWithCircumflexAndAcute': '\xe1\xba\xbf',
'SylotiNagriLetterTtho': '\xea\xa0\x91',
'CjkUnifiedIdeograph6499': '\xe6\x92\x99',
'CjkUnifiedIdeograph6498': '\xe6\x92\x98',
'CjkUnifiedIdeograph6497': '\xe6\x92\x97',
'LatinSmallLetterHWithDotBelow': '\xe1\xb8\xa5',
'CjkUnifiedIdeograph6495': '\xe6\x92\x95',
'CjkUnifiedIdeograph6494': '\xe6\x92\x94',
'CjkUnifiedIdeograph6493': '\xe6\x92\x93',
'CjkUnifiedIdeograph6492': '\xe6\x92\x92',
'CjkUnifiedIdeograph6491': '\xe6\x92\x91',
'CjkUnifiedIdeograph6490': '\xe6\x92\x90',
'HangulSyllableCwaebs': '\xec\xb5\x92',
'HangulSyllableGgyung': '\xeb\x80\xbd',
'HangulSyllableGgyunh': '\xeb\x80\xae',
'HangulSyllableGgyunj': '\xeb\x80\xad',
'HangulSyllableKwilb': '\xed\x80\xbf',
'UnicodeCharacter0xe743': '\xee\x9d\x83',
'IntegralWithUnderbar': '\xe2\xa8\x9c',
'UnicodeCharacter0xe04b': '\xee\x81\x8b',
'TibetanLetterKa': '\xe0\xbd\x80',
'CjkUnifiedIdeograph9eaf': '\xe9\xba\xaf',
'CjkUnifiedIdeograph9eae': '\xe9\xba\xae',
'CjkUnifiedIdeograph9ead': '\xe9\xba\xad',
'CjkUnifiedIdeograph9eac': '\xe9\xba\xac',
'CjkUnifiedIdeograph9eab': '\xe9\xba\xab',
'CjkUnifiedIdeograph9eaa': '\xe9\xba\xaa',
'YiSyllableGgox': '\xea\x88\xa3',
'CjkUnifiedIdeograph8c7a': '\xe8\xb1\xba',
'HangulSyllableDyaelp': '\xeb\x8d\x86',
'GurmukhiIri': '\xe0\xa9\xb2',
'HangulSyllablePelh': '\xed\x8e\xa7',
'UnicodeCharacter0xea77': '\xee\xa9\xb7',
'HangulSyllablePelg': '\xed\x8e\xa1',
'WhiteMediumLozenge': '\xe2\xac\xa8',
'HangulSyllablePelt': '\xed\x8e\xa5',
'CjkUnifiedIdeograph649f': '\xe6\x92\x9f',
'CjkUnifiedIdeograph649e': '\xe6\x92\x9e',
'CjkUnifiedIdeograph649d': '\xe6\x92\x9d',
'CjkUnifiedIdeograph649c': '\xe6\x92\x9c',
'CjkUnifiedIdeograph649b': '\xe6\x92\x9b',
'CjkUnifiedIdeograph649a': '\xe6\x92\x9a',
'UnicodeCharacter0xa9cb': '\xea\xa7\x8b',
'CjkUnifiedIdeograph89da': '\xe8\xa7\x9a',
'CjkUnifiedIdeograph9ea6': '\xe9\xba\xa6',
'CjkUnifiedIdeograph9ea5': '\xe9\xba\xa5',
'CjkUnifiedIdeograph9ea4': '\xe9\xba\xa4',
'CjkUnifiedIdeograph9ea3': '\xe9\xba\xa3',
'CjkUnifiedIdeograph9ea2': '\xe9\xba\xa2',
'HanunooVowelSignU': '\xe1\x9c\xb3',
'CjkUnifiedIdeograph9ea0': '\xe9\xba\xa0',
'CjkUnifiedIdeograph9ea9': '\xe9\xba\xa9',
'CjkUnifiedIdeograph9ea8': '\xe9\xba\xa8',
'CjkUnifiedIdeograph9199': '\xe9\x86\x99',
'UnicodeCharacter0xd9af': '\xed\xa6\xaf',
'UnicodeCharacter0xd865': '\xed\xa1\xa5',
'KannadaVowelSignE': '\xe0\xb3\x86',
'HanunooVowelSignI': '\xe1\x9c\xb2',
'YiSyllableFox': '\xea\x83\x95',
'UnicodeCharacter0xd9ad': '\xed\xa6\xad',
'UnicodeCharacter0xd863': '\xed\xa1\xa3',
'HangulSyllableSyib': '\xec\x8b\x91',
'HangulSyllableSyic': '\xec\x8b\x97',
'UnicodeCharacter0xd9ae': '\xed\xa6\xae',
'HangulSyllableNuk': '\xeb\x88\x9c',
'HangulSyllableSyig': '\xec\x8b\x81',
'HangulSyllableSyid': '\xec\x8b\x87',
'ArabicLetterNoonInitialForm': '\xef\xbb\xa7',
'HangulSyllableSyij': '\xec\x8b\x96',
'HangulSyllableSyik': '\xec\x8b\x98',
'GreekSmallLetterEtaWithDasiaAndOxia': '\xe1\xbc\xa5',
'HangulSyllableSyin': '\xec\x8b\x84',
'HangulSyllableSyil': '\xec\x8b\x88',
'DentistrySymbolLightUpAndHorizontalWithTriangle': '\xe2\x8f\x85',
'HangulSyllableSyis': '\xec\x8b\x93',
'HangulSyllableSyip': '\xec\x8b\x9a',
'HangulSyllableDdwebs': '\xeb\x9b\xa6',
'HangulSyllableSyit': '\xec\x8b\x99',
'UnicodeCharacter0xf71f': '\xef\x9c\x9f',
'UnicodeCharacter0xd9aa': '\xed\xa6\xaa',
'UnicodeCharacter0xea60': '\xee\xa9\xa0',
'CjkUnifiedIdeograph9bb5': '\xe9\xae\xb5',
'UnicodeCharacter0xea62': '\xee\xa9\xa2',
'UnicodeCharacter0xea63': '\xee\xa9\xa3',
'UnicodeCharacter0xea64': '\xee\xa9\xa4',
'UnicodeCharacter0xea65': '\xee\xa9\xa5',
'CjkUnifiedIdeograph3618': '\xe3\x98\x98',
'CjkUnifiedIdeograph3619': '\xe3\x98\x99',
'CjkUnifiedIdeograph3616': '\xe3\x98\x96',
'CjkUnifiedIdeograph3617': '\xe3\x98\x97',
'HangulLetterSsanghieuh': '\xe3\x86\x85',
'CjkUnifiedIdeograph3615': '\xe3\x98\x95',
'CjkUnifiedIdeograph3612': '\xe3\x98\x92',
'CjkUnifiedIdeograph3613': '\xe3\x98\x93',
'CjkUnifiedIdeograph3610': '\xe3\x98\x90',
'CjkUnifiedIdeograph3611': '\xe3\x98\x91',
'HangulJungseongUu': '\xe1\x86\x8d',
'UnicodeCharacter0xda77': '\xed\xa9\xb7',
'UnicodeCharacter0xd868': '\xed\xa1\xa8',
'CjkUnifiedIdeograph7728': '\xe7\x9c\xa8',
'SinhalaLetterAayanna': '\xe0\xb6\x86',
'CjkUnifiedIdeograph7726': '\xe7\x9c\xa6',
'CjkUnifiedIdeograph7727': '\xe7\x9c\xa7',
'UpperRightQuadrantCircularArc': '\xe2\x97\x9d',
'CjkUnifiedIdeograph7725': '\xe7\x9c\xa5',
'CjkUnifiedIdeograph7722': '\xe7\x9c\xa2',
'CjkUnifiedIdeograph7723': '\xe7\x9c\xa3',
'CjkUnifiedIdeograph7720': '\xe7\x9c\xa0',
'CjkUnifiedIdeograph7721': '\xe7\x9c\xa1',
'HangulSyllableJjot': '\xec\xab\x95',
'YiSyllableJjur': '\xea\x90\xac',
'YiSyllableJjup': '\xea\x90\xaa',
'HangulSyllableJjop': '\xec\xab\x96',
'YiSyllableTiep': '\xea\x84\xa0',
'YiSyllableJjut': '\xea\x90\xa7',
'YiSyllableJjux': '\xea\x90\xa8',
'UnicodeCharacter0xef32': '\xee\xbc\xb2',
'UnicodeCharacter0xda32': '\xed\xa8\xb2',
'HangulSyllableJjod': '\xec\xab\x83',
'CircledDigitEight': '\xe2\x91\xa7',
'HangulSyllableJjog': '\xec\xaa\xbd',
'HangulSyllableJjob': '\xec\xab\x8d',
'HangulSyllableJjoc': '\xec\xab\x93',
'HangulSyllableJjol': '\xec\xab\x84',
'HangulSyllableJjom': '\xec\xab\x8c',
'HangulSyllableJjon': '\xec\xab\x80',
'ModifierLetterExtrahighToneBar': '\xcb\xa5',
'HangulSyllableJjoj': '\xec\xab\x92',
'HangulSyllableJjok': '\xec\xab\x94',
'CjkUnifiedIdeograph9ba8': '\xe9\xae\xa8',
'CjkUnifiedIdeograph9ba9': '\xe9\xae\xa9',
'CjkUnifiedIdeograph772f': '\xe7\x9c\xaf',
'CjkUnifiedIdeograph9ba1': '\xe9\xae\xa1',
'CjkUnifiedIdeograph772d': '\xe7\x9c\xad',
'CjkUnifiedIdeograph772e': '\xe7\x9c\xae',
'CjkUnifiedIdeograph772b': '\xe7\x9c\xab',
'CjkUnifiedIdeograph772c': '\xe7\x9c\xac',
'CjkUnifiedIdeograph9ba6': '\xe9\xae\xa6',
'CjkUnifiedIdeograph772a': '\xe7\x9c\xaa',
'HangulSyllableSabs': '\xec\x82\xbe',
'UnicodeCharacter0xea6b': '\xee\xa9\xab',
'UnicodeCharacter0xea6c': '\xee\xa9\xac',
'UnicodeCharacter0xea6d': '\xee\xa9\xad',
'UnicodeCharacter0xea6e': '\xee\xa9\xae',
'CjkUnifiedIdeograph89dd': '\xe8\xa7\x9d',
'HangulSyllableGess': '\xea\xb2\xa0',
'CjkUnifiedIdeograph361f': '\xe3\x98\x9f',
'UnicodeCharacter0xdd9f': '\xed\xb6\x9f',
'CjkUnifiedIdeograph361d': '\xe3\x98\x9d',
'CjkUnifiedIdeograph361e': '\xe3\x98\x9e',
'CjkUnifiedIdeograph361b': '\xe3\x98\x9b',
'CjkUnifiedIdeograph361c': '\xe3\x98\x9c',
'UnicodeCharacter0xdd9a': '\xed\xb6\x9a',
'CjkUnifiedIdeograph361a': '\xe3\x98\x9a',
'KhmerLetterKo': '\xe1\x9e\x82',
'CjkUnifiedIdeograph54c5': '\xe5\x93\x85',
'CjkUnifiedIdeograph54c6': '\xe5\x93\x86',
'CjkUnifiedIdeograph54c7': '\xe5\x93\x87',
'CjkUnifiedIdeograph54c0': '\xe5\x93\x80',
'GreekSmallLetterOmegaWithPsili': '\xe1\xbd\xa0',
'CjkUnifiedIdeograph54c2': '\xe5\x93\x82',
'CjkUnifiedIdeograph54c3': '\xe5\x93\x83',
'SquareGy': '\xe3\x8f\x89',
'HangulSyllableNyalt': '\xeb\x83\x9d',
'CjkUnifiedIdeograph54c8': '\xe5\x93\x88',
'CjkUnifiedIdeograph54c9': '\xe5\x93\x89',
'KhmerLetterKa': '\xe1\x9e\x80',
'HangulSyllableRoec': '\xeb\xa3\x87',
'SquareGb': '\xe3\x8e\x87',
'HangulSyllableRoeb': '\xeb\xa3\x81',
'UnicodeCharacter0xea70': '\xee\xa9\xb0',
'CjkUnifiedIdeograph98fa': '\xe9\xa3\xba',
'UnicodeCharacter0xfde': '\xe0\xbf\x9e',
'UnicodeCharacter0xfdd': '\xe0\xbf\x9d',
'HangulSyllablePyolh': '\xed\x91\xab',
'UnicodeCharacter0xfdf': '\xe0\xbf\x9f',
'UnicodeCharacter0xfda': '\xe0\xbf\x9a',
'UnicodeCharacter0xfdc': '\xe0\xbf\x9c',
'UnicodeCharacter0xfdb': '\xe0\xbf\x9b',
'UnicodeCharacter0xa9b0': '\xea\xa6\xb0',
'UnicodeCharacter0xa9b1': '\xea\xa6\xb1',
'UnicodeCharacter0xa9b2': '\xea\xa6\xb2',
'UnicodeCharacter0xa9b3': '\xea\xa6\xb3',
'UnicodeCharacter0xa9b4': '\xea\xa6\xb4',
'UnicodeCharacter0xa9b5': '\xea\xa6\xb5',
'UnicodeCharacter0xa9b6': '\xea\xa6\xb6',
'UnicodeCharacter0xa9b7': '\xea\xa6\xb7',
'CjkUnifiedIdeograph693d': '\xe6\xa4\xbd',
'CjkUnifiedIdeograph693e': '\xe6\xa4\xbe',
'CjkUnifiedIdeograph693f': '\xe6\xa4\xbf',
'HangulSyllableDdeugg': '\xeb\x9c\xaa',
'CjkUnifiedIdeograph6399': '\xe6\x8e\x99',
'CjkUnifiedIdeograph693b': '\xe6\xa4\xbb',
'CjkUnifiedIdeograph693c': '\xe6\xa4\xbc',
'HangulSyllablePyols': '\xed\x91\xa8',
'HangulSyllablePyolp': '\xed\x91\xaa',
'CjkUnifiedIdeograph98fd': '\xe9\xa3\xbd',
'HangulSyllablePyolt': '\xed\x91\xa9',
'UnicodeCharacter0xd98c': '\xed\xa6\x8c',
'HangulSyllableRwelt': '\xeb\xa4\xad',
'UnicodeCharacter0xef36': '\xee\xbc\xb6',
'CjkUnifiedIdeograph6f18': '\xe6\xbc\x98',
'UnicodeCharacter0xe5ec': '\xee\x97\xac',
'CjkUnifiedIdeograph54cd': '\xe5\x93\x8d',
'CjkUnifiedIdeograph54ce': '\xe5\x93\x8e',
'CjkUnifiedIdeograph54cf': '\xe5\x93\x8f',
'UnicodeCharacter0xe5eb': '\xee\x97\xab',
'CjkUnifiedIdeograph54ca': '\xe5\x93\x8a',
'CjkUnifiedIdeograph54cb': '\xe5\x93\x8b',
'CjkUnifiedIdeograph54cc': '\xe5\x93\x8c',
'HangulSyllableRwelp': '\xeb\xa4\xae',
'CjkUnifiedIdeograph8dad': '\xe8\xb6\xad',
'CjkUnifiedIdeograph68b5': '\xe6\xa2\xb5',
'UnicodeCharacter0xe5ed': '\xee\x97\xad',
'CjkUnifiedIdeograph6934': '\xe6\xa4\xb4',
'CjkUnifiedIdeograph6935': '\xe6\xa4\xb5',
'CjkUnifiedIdeograph6936': '\xe6\xa4\xb6',
'CjkUnifiedIdeograph6937': '\xe6\xa4\xb7',
'CjkUnifiedIdeograph6930': '\xe6\xa4\xb0',
'CjkUnifiedIdeograph6931': '\xe6\xa4\xb1',
'CjkUnifiedIdeograph6932': '\xe6\xa4\xb2',
'CjkUnifiedIdeograph6933': '\xe6\xa4\xb3',
'CjkUnifiedIdeograph911c': '\xe9\x84\x9c',
'UnicodeCharacter0xe5ef': '\xee\x97\xaf',
'CjkUnifiedIdeograph6938': '\xe6\xa4\xb8',
'CherokeeLetterHu': '\xe1\x8e\xb1',
'CherokeeLetterHv': '\xe1\x8e\xb2',
'UnicodeCharacter0xfd5': '\xe0\xbf\x95',
'CherokeeLetterHi': '\xe1\x8e\xaf',
'UnicodeCharacter0xfd7': '\xe0\xbf\x97',
'UnicodeCharacter0xfd6': '\xe0\xbf\x96',
'CherokeeLetterHo': '\xe1\x8e\xb0',
'CherokeeLetterHa': '\xe1\x8e\xad',
'GlagoliticCapitalLetterTvrido': '\xe2\xb0\x95',
'UnicodeCharacter0xa9bc': '\xea\xa6\xbc',
'UnicodeCharacter0xfd9': '\xe0\xbf\x99',
'CherokeeLetterHe': '\xe1\x8e\xae',
'UnicodeCharacter0xa9bf': '\xea\xa6\xbf',
'ArabicLetterUFinalForm': '\xef\xaf\x98',
'CjkUnifiedIdeograph70f1': '\xe7\x83\xb1',
'CjkUnifiedIdeograph70f0': '\xe7\x83\xb0',
'CjkUnifiedIdeograph70f3': '\xe7\x83\xb3',
'CjkUnifiedIdeograph70f2': '\xe7\x83\xb2',
'NotTrue': '\xe2\x8a\xad',
'CjkUnifiedIdeograph70f4': '\xe7\x83\xb4',
'CjkUnifiedIdeograph70f7': '\xe7\x83\xb7',
'CjkUnifiedIdeograph70f6': '\xe7\x83\xb6',
'CjkUnifiedIdeograph70f9': '\xe7\x83\xb9',
'CjkUnifiedIdeograph70f8': '\xe7\x83\xb8',
'CjkUnifiedIdeograph7b50': '\xe7\xad\x90',
'CjkUnifiedIdeograph7b51': '\xe7\xad\x91',
'CjkUnifiedIdeograph7b56': '\xe7\xad\x96',
'CjkUnifiedIdeograph7b57': '\xe7\xad\x97',
'CjkUnifiedIdeograph7b54': '\xe7\xad\x94',
'CjkUnifiedIdeograph7b55': '\xe7\xad\x95',
'UnicodeCharacter0xf00f': '\xef\x80\x8f',
'LepchaLetterTsa': '\xe1\xb0\x97',
'HangulSyllableJjyeonj': '\xec\xaa\x89',
'HangulSyllableMyeogs': '\xeb\xa9\xb3',
'GreekSmallLetterEtaWithOxiaAndYpogegrammeni': '\xe1\xbf\x84',
'CjkUnifiedIdeograph88be': '\xe8\xa2\xbe',
'UnicodeCharacter0xef39': '\xee\xbc\xb9',
'UnicodeCharacter0xda39': '\xed\xa8\xb9',
'HangulSyllableDdwis': '\xeb\x9c\x83',
'UnicodeCharacter0xf057': '\xef\x81\x97',
'GreekSmallLetterIotaWithPsiliAndPerispomeni': '\xe1\xbc\xb6',
'HangulSyllableRwelg': '\xeb\xa4\xa9',
'CjkCompatibilityIdeographf9dc': '\xef\xa7\x9c',
'UnicodeCharacter0xda7d': '\xed\xa9\xbd',
'CjkUnifiedIdeograph852f': '\xe8\x94\xaf',
'BraillePatternDots2367': '\xe2\xa1\xa6',
'HebrewPointSegol': '\xd6\xb6',
'BraillePatternDots2368': '\xe2\xa2\xa6',
'CjkUnifiedIdeograph70fa': '\xe7\x83\xba',
'UnicodeCharacter0xe506': '\xee\x94\x86',
'CjkUnifiedIdeograph70fc': '\xe7\x83\xbc',
'CjkUnifiedIdeograph70fb': '\xe7\x83\xbb',
'CjkUnifiedIdeograph70fe': '\xe7\x83\xbe',
'CjkUnifiedIdeograph68b9': '\xe6\xa2\xb9',
'CjkUnifiedIdeograph70ff': '\xe7\x83\xbf',
'CjkUnifiedIdeograph7b5b': '\xe7\xad\x9b',
'LaoVowelSignAa': '\xe0\xba\xb2',
'CjkUnifiedIdeograph91c7': '\xe9\x87\x87',
'CjkUnifiedIdeograph7b5a': '\xe7\xad\x9a',
'ModifierLetterSmallChi': '\xe1\xb5\xa1',
'CjkUnifiedIdeograph91c0': '\xe9\x87\x80',
'CjkUnifiedIdeograph7b5d': '\xe7\xad\x9d',
'CjkUnifiedIdeograph7b5e': '\xe7\xad\x9e',
'YiSyllableGga': '\xea\x88\x9c',
'HangulSyllableBbeobs': '\xeb\xbb\xa2',
'HangulSyllableMyeogg': '\xeb\xa9\xb2',
'YiSyllableGge': '\xea\x88\xa8',
'CjkUnifiedIdeograph9bbc': '\xe9\xae\xbc',
'YiSyllableGgi': '\xea\x88\x96',
'UnicodeCharacter0xa8db': '\xea\xa3\x9b',
'UnicodeCharacter0xa8da': '\xea\xa3\x9a',
'CjkUnifiedIdeograph9bbb': '\xe9\xae\xbb',
'LatinCapitalLetterIWithCircumflex': '\xc3\x8e',
'UnicodeCharacter0xa8df': '\xea\xa3\x9f',
'YiSyllableGgo': '\xea\x88\xa4',
'UnicodeCharacter0xa8dd': '\xea\xa3\x9d',
'IdeographicDescriptionCharacterAboveToMiddleAndBelow': '\xe2\xbf\xb3',
'YiSyllableGgu': '\xea\x88\xac',
'HangulSyllableJyunj': '\xec\xa5\xb1',
'HangzhouNumeralNine': '\xe3\x80\xa9',
'KannadaLetterJa': '\xe0\xb2\x9c',
'HangulSyllableBaec': '\xeb\xb1\x87',
'HangulSyllableBaeb': '\xeb\xb1\x81',
'CjkUnifiedIdeograph5a5c': '\xe5\xa9\x9c',
'HangulSyllableBaed': '\xeb\xb0\xb7',
'HangulSyllableBaeg': '\xeb\xb0\xb1',
'LaoVowelSignAi': '\xe0\xbb\x84',
'HangulSyllableBaek': '\xeb\xb1\x88',
'HangulSyllableBaej': '\xeb\xb1\x86',
'HangulSyllableBaem': '\xeb\xb1\x80',
'HangulSyllableBael': '\xeb\xb0\xb8',
'HangulSyllableBaen': '\xeb\xb0\xb4',
'HangulSyllableBaep': '\xeb\xb1\x8a',
'HangulSyllableBaes': '\xeb\xb1\x83',
'UnicodeCharacter0xea41': '\xee\xa9\x81',
'HangulSyllableBaet': '\xeb\xb1\x89',
'CjkUnifiedIdeograph911e': '\xe9\x84\x9e',
'CjkUnifiedIdeograph5a5b': '\xe5\xa9\x9b',
'NkoLetterGba': '\xdf\x9c',
'LeftwardsHarpoonWithBarbDownToBar': '\xe2\xa5\x96',
'GreekSmallLetterEtaWithTonos': '\xce\xae',
'BraillePatternDots1235678': '\xe2\xa3\xb7',
'CjkUnifiedIdeograph98f3': '\xe9\xa3\xb3',
'CjkUnifiedIdeograph85e2': '\xe8\x97\xa2',
'DevanagariLetterMa': '\xe0\xa4\xae',
'WhiteFlorette': '\xe2\x9d\x80',
'ArmenianSmallLigatureMenIni': '\xef\xac\x95',
'CjkUnifiedIdeograph911d': '\xe9\x84\x9d',
'CopticSmallLetterDialectpHori': '\xe2\xb3\x8b',
'UnicodeCharacter0xf052': '\xef\x81\x92',
'HangulSyllableSsaelh': '\xec\x8c\xa3',
'BengaliLetterSsa': '\xe0\xa6\xb7',
'ArabicLigatureYehWithMeemWithMeemInitialForm': '\xef\xb6\x9d',
'CjkCompatibilityIdeographf9df': '\xef\xa7\x9f',
'CjkUnifiedIdeograph9a4d': '\xe9\xa9\x8d',
'HeavyDoubleCommaQuotationMarkOrnament': '\xe2\x9d\x9e',
'HangulSyllableGgass': '\xea\xb9\xa0',
'PositionIndicator': '\xe2\x8c\x96',
'CjkUnifiedIdeograph98f4': '\xe9\xa3\xb4',
'CjkUnifiedIdeograph9096': '\xe9\x82\x96',
'UnicodeCharacter0xd9a4': '\xed\xa6\xa4',
'a': '\xce\xb1',
'UnicodeCharacter0xd9a5': '\xed\xa6\xa5',
'LaoVowelSignAy': '\xe0\xbb\x83',
'UnicodeCharacter0xd9a3': '\xed\xa6\xa3',
'LatinSmallLetterEWithMacronAndAcute': '\xe1\xb8\x97',
'UnicodeCharacter0xd9a0': '\xed\xa6\xa0',
'CjkUnifiedIdeograph9097': '\xe9\x82\x97',
'UnicodeCharacter0xa9c8': '\xea\xa7\x88',
'UnicodeCharacter0xe747': '\xee\x9d\x87',
'HangulSyllableSsaebs': '\xec\x8c\xa6',
'EmptySet': '\xe2\x88\x85',
'CanadianSyllabicsFinalAcute': '\xe1\x90\x9f',
'notequal': '\xe2\x89\xa0',
'UnicodeCharacter0xf45a': '\xef\x91\x9a',
'HangulSyllableKaelm': '\xec\xba\x9a',
'CjkCompatibilityIdeographfa3d': '\xef\xa8\xbd',
'CjkCompatibilityIdeographfa3e': '\xef\xa8\xbe',
'CjkCompatibilityIdeographfa3f': '\xef\xa8\xbf',
'BlackUniversalRecyclingSymbol': '\xe2\x99\xbb',
'UnicodeCharacter0xe238': '\xee\x88\xb8',
'UnicodeCharacter0xe239': '\xee\x88\xb9',
'LessthanAboveSimilarAboveGreaterthan': '\xe2\xaa\x8f',
'CjkCompatibilityIdeographfa3c': '\xef\xa8\xbc',
'LatinSmallLetterIWithTilde': '\xc4\xa9',
'UnicodeCharacter0xe235': '\xee\x88\xb5',
'UnicodeCharacter0xe236': '\xee\x88\xb6',
'UnicodeCharacter0xe237': '\xee\x88\xb7',
'UnicodeCharacter0xe230': '\xee\x88\xb0',
'UnicodeCharacter0xe231': '\xee\x88\xb1',
'UnicodeCharacter0xe232': '\xee\x88\xb2',
'UnicodeCharacter0xe233': '\xee\x88\xb3',
'CjkUnifiedIdeograph4e1d': '\xe4\xb8\x9d',
'CjkUnifiedIdeograph4e1e': '\xe4\xb8\x9e',
'CjkUnifiedIdeograph4e1f': '\xe4\xb8\x9f',
'CjkUnifiedIdeograph4e1a': '\xe4\xb8\x9a',
'CjkUnifiedIdeograph4e1b': '\xe4\xb8\x9b',
'HangulLetterSiosnieun': '\xe3\x85\xbb',
'CombiningAlmostEqualToAbove': '\xcd\x8c',
'HangulSyllableJjyal': '\xec\xa8\x9c',
'UnicodeCharacter0xe4cf': '\xee\x93\x8f',
'UnicodeCharacter0xe4cd': '\xee\x93\x8d',
'UnicodeCharacter0xe4ce': '\xee\x93\x8e',
'UnicodeCharacter0xe4cb': '\xee\x93\x8b',
'CjkUnifiedIdeograph917c': '\xe9\x85\xbc',
'ParenthesizedHangulNieunA': '\xe3\x88\x8f',
'UnicodeCharacter0xe4ca': '\xee\x93\x8a',
'CjkCompatibilityIdeographfa34': '\xef\xa8\xb4',
'YiRadicalMi': '\xea\x92\x98',
'YiSyllableNzat': '\xea\x8b\xb4',
'CjkUnifiedIdeograph917b': '\xe9\x85\xbb',
'CjkCompatibilityIdeographfa30': '\xef\xa8\xb0',
'CjkCompatibilityIdeographfa31': '\xef\xa8\xb1',
'CjkCompatibilityIdeographfa32': '\xef\xa8\xb2',
'CjkCompatibilityIdeographfa33': '\xef\xa8\xb3',
'UnicodeCharacter0xe23d': '\xee\x88\xbd',
'BugineseLetterTa': '\xe1\xa8\x88',
'UnicodeCharacter0xe23f': '\xee\x88\xbf',
'CjkCompatibilityIdeographfa38': '\xef\xa8\xb8',
'UnicodeCharacter0xe23a': '\xee\x88\xba',
'UnicodeCharacter0xe23b': '\xee\x88\xbb',
'CjkUnifiedIdeograph917d': '\xe9\x85\xbd',
'GreekSmallLetterAlphaWithPsiliAndPerispomeniAndYpogegrammeni': '\xe1\xbe\x86',
'BengaliRupeeSign': '\xe0\xa7\xb3',
'LefttorightOverride': '\xe2\x80\xad',
'HangulSyllableJjyat': '\xec\xa8\xad',
'GlagoliticCapitalLetterIotatedSmallYus': '\xe2\xb0\xa7',
'UnicodeCharacter0xe4c8': '\xee\x93\x88',
'UnicodeCharacter0xe4c9': '\xee\x93\x89',
'UnicodeCharacter0xe4c6': '\xee\x93\x86',
'UnicodeCharacter0xe4c7': '\xee\x93\x87',
'UnicodeCharacter0xe4c4': '\xee\x93\x84',
'UnicodeCharacter0xe4c5': '\xee\x93\x85',
'UnicodeCharacter0xe4c2': '\xee\x93\x82',
'UnicodeCharacter0xe4c3': '\xee\x93\x83',
'UnicodeCharacter0xe4c0': '\xee\x93\x80',
'UnicodeCharacter0xe4c1': '\xee\x93\x81',
'CjkUnifiedIdeograph4e14': '\xe4\xb8\x94',
'CjkUnifiedIdeograph4e15': '\xe4\xb8\x95',
'CjkUnifiedIdeograph4e16': '\xe4\xb8\x96',
'CjkUnifiedIdeograph4e17': '\xe4\xb8\x97',
'CjkUnifiedIdeograph4e10': '\xe4\xb8\x90',
'CjkUnifiedIdeograph4e11': '\xe4\xb8\x91',
'CjkUnifiedIdeograph4e12': '\xe4\xb8\x92',
'NotTilde': '\xe2\x89\x81',
'CjkUnifiedIdeograph3cd8': '\xe3\xb3\x98',
'BlackDiamondMinusWhiteX': '\xe2\x9d\x96',
'CjkUnifiedIdeograph4e18': '\xe4\xb8\x98',
'CjkUnifiedIdeograph4e19': '\xe4\xb8\x99',
'BraillePatternDots578': '\xe2\xa3\x90',
'HangulSyllableBbwit': '\xec\x80\xb9',
'HangulSyllableBbwis': '\xec\x80\xb3',
'HangulSyllableBbwip': '\xec\x80\xba',
'phi': '\xcf\x95',
'HangulSyllableSsyulm': '\xec\x93\x9e',
'UnicodeCharacter0xa77': '\xe0\xa9\xb7',
'UnicodeCharacter0xa76': '\xe0\xa9\xb6',
'HangulSyllableBbwid': '\xec\x80\xa7',
'ChamConsonantSignYa': '\xea\xa8\xb3',
'HangulSyllableBbwic': '\xec\x80\xb7',
'HangulSyllableBbwin': '\xec\x80\xa4',
'CjkUnifiedIdeograph901b': '\xe9\x80\x9b',
'HangulSyllableBbwil': '\xec\x80\xa8',
'HangulSyllableBbwim': '\xec\x80\xb0',
'HangulSyllableBbwij': '\xec\x80\xb6',
'HangulSyllableBbwik': '\xec\x80\xb8',
'UnicodeCharacter0xa79': '\xe0\xa9\xb9',
'UnicodeCharacter0xa78': '\xe0\xa9\xb8',
'UnicodeCharacter0x2d2d': '\xe2\xb4\xad',
'UnicodeCharacter0x2d2e': '\xe2\xb4\xae',
'UnicodeCharacter0x2d2f': '\xe2\xb4\xaf',
'HangulSyllableJjonh': '\xec\xab\x82',
'EthiopicSyllableSebatbeitPwa': '\xe1\x8e\x8c',
'RaisedSquare': '\xe2\xb8\x8b',
'UnicodeCharacter0x2d2b': '\xe2\xb4\xab',
'UnicodeCharacter0x2d2c': '\xe2\xb4\xac',
'HangulSyllableSyolp': '\xec\x88\x8a',
'HangulSyllableSyols': '\xec\x88\x88',
'HangulSyllableJjong': '\xec\xab\x91',
'HangulSyllableSyolt': '\xec\x88\x89',
'HangulJongseongPieup': '\xe1\x86\xb8',
'HangulSyllableSyolh': '\xec\x88\x8b',
'UnicodeCharacter0xe5e2': '\xee\x97\xa2',
'HangulSyllableSyolm': '\xec\x88\x86',
'UnicodeCharacter0xeeb2': '\xee\xba\xb2',
'CjkUnifiedIdeograph901c': '\xe9\x80\x9c',
'UnicodeCharacter0xe5e5': '\xee\x97\xa5',
'HangulSyllableSyolb': '\xec\x88\x87',
'HangulSyllableSyolg': '\xec\x88\x85',
'CjkUnifiedIdeograph7d55': '\xe7\xb5\x95',
'UnicodeCharacter0xa7f': '\xe0\xa9\xbf',
'UnicodeCharacter0xa7e': '\xe0\xa9\xbe',
'UnicodeCharacter0xa7d': '\xe0\xa9\xbd',
'UnicodeCharacter0xa7c': '\xe0\xa9\xbc',
'UnicodeCharacter0xa7b': '\xe0\xa9\xbb',
'UnicodeCharacter0xa7a': '\xe0\xa9\xba',
'UnicodeCharacter0xf610': '\xef\x98\x90',
'HangulSyllableDeong': '\xeb\x8d\xa9',
'UnicodeCharacter0xf66d': '\xef\x99\xad',
'ArabicLigatureMeemWithHahIsolatedForm': '\xef\xb1\x86',
'CjkUnifiedIdeograph7d58': '\xe7\xb5\x98',
'HangulSyllableKyolb': '\xec\xbf\x8f',
'HangulSyllableCwinj': '\xec\xb7\xad',
'HangulSyllableKyolg': '\xec\xbf\x8d',
'HangulSyllableCwinh': '\xec\xb7\xae',
'CjkUnifiedIdeograph9357': '\xe9\x8d\x97',
'HangulSyllableKyolh': '\xec\xbf\x93',
'HangulSyllableKyolm': '\xec\xbf\x8e',
'HangulSyllableDeonj': '\xeb\x8d\x99',
'HangulSyllableHyass': '\xed\x96\xa4',
'LatinSmallLetterInsularD': '\xea\x9d\xba',
'_beta': '\xe1\xb5\xa6',
'ArabicLigatureSeenWithAlefMaksuraIsolatedForm': '\xef\xb3\xbb',
'UnicodeCharacter0xf617': '\xef\x98\x97',
'HangulSyllableSsyulb': '\xec\x93\x9f',
'LatinSmallLetterInsularF': '\xea\x9d\xbc',
'HangulSyllablePyeolg': '\xed\x8e\xbd',
'UnicodeCharacter0x2d26': '\xe2\xb4\xa6',
'UnicodeCharacter0x2d27': '\xe2\xb4\xa7',
'CjkUnifiedIdeograph9356': '\xe9\x8d\x96',
'HangulSyllableTolh': '\xed\x86\xaf',
'UnicodeCharacter0x2d28': '\xe2\xb4\xa8',
'UnicodeCharacter0x2d29': '\xe2\xb4\xa9',
'CjkUnifiedIdeograph9d5a': '\xe9\xb5\x9a',
'CjkUnifiedIdeograph8528': '\xe8\x94\xa8',
'HangulSyllablePang': '\xed\x8c\xa1',
'UnicodeCharacter0xf616': '\xef\x98\x96',
'SundaneseLetterNga': '\xe1\xae\x8d',
'HangulSyllableBiss': '\xeb\xb9\x98',
'ArabicLetterRnoonMedialForm': '\xef\xae\xa3',
'HangulSyllablePanj': '\xed\x8c\x91',
'HangulSyllablePanh': '\xed\x8c\x92',
'CjkUnifiedIdeograph464a': '\xe4\x99\x8a',
'CjkUnifiedIdeograph464b': '\xe4\x99\x8b',
'CjkUnifiedIdeograph464c': '\xe4\x99\x8c',
'CjkUnifiedIdeograph464d': '\xe4\x99\x8d',
'ArabicLetterTehMarbuta': '\xd8\xa9',
'CjkUnifiedIdeograph464f': '\xe4\x99\x8f',
'LatinSmallLetterTopHalfO': '\xe1\xb4\x96',
'HangulSyllablePeolg': '\xed\x8e\x85',
'UnicodeCharacter0xe79c': '\xee\x9e\x9c',
'UnicodeCharacter0xe79b': '\xee\x9e\x9b',
'UnicodeCharacter0xe79a': '\xee\x9e\x9a',
'UnicodeCharacter0xe79f': '\xee\x9e\x9f',
'UnicodeCharacter0xe79e': '\xee\x9e\x9e',
'UnicodeCharacter0xe79d': '\xee\x9e\x9d',
'LatinSmallLetterInsularS': '\xea\x9e\x85',
'CjkUnifiedIdeograph49e4': '\xe4\xa7\xa4',
'CjkUnifiedIdeograph49e5': '\xe4\xa7\xa5',
'CjkUnifiedIdeograph49e6': '\xe4\xa7\xa6',
'CjkUnifiedIdeograph49e7': '\xe4\xa7\xa7',
'CjkUnifiedIdeograph49e0': '\xe4\xa7\xa0',
'CjkUnifiedIdeograph49e1': '\xe4\xa7\xa1',
'CjkUnifiedIdeograph49e2': '\xe4\xa7\xa2',
'CjkUnifiedIdeograph49e3': '\xe4\xa7\xa3',
'CjkUnifiedIdeograph49e8': '\xe4\xa7\xa8',
'CjkUnifiedIdeograph49e9': '\xe4\xa7\xa9',
'CjkUnifiedIdeograph4640': '\xe4\x99\x80',
'CjkUnifiedIdeograph4641': '\xe4\x99\x81',
'CjkUnifiedIdeograph4642': '\xe4\x99\x82',
'CjkUnifiedIdeograph4643': '\xe4\x99\x83',
'CjkUnifiedIdeograph4644': '\xe4\x99\x84',
'CjkUnifiedIdeograph4645': '\xe4\x99\x85',
'CjkUnifiedIdeograph4646': '\xe4\x99\x86',
'CjkUnifiedIdeograph4647': '\xe4\x99\x87',
'CjkUnifiedIdeograph4648': '\xe4\x99\x88',
'CjkUnifiedIdeograph4649': '\xe4\x99\x89',
'LeftBarbDownRightBarbDownHarpoon': '\xe2\xa5\x90',
'UnicodeCharacter0xfe1': '\xe0\xbf\xa1',
'LeftwardsArrowToBarOverRightwardsArrowToBar': '\xe2\x86\xb9',
'HangulSyllablePyaeb': '\xed\x8d\xb1',
'UnicodeCharacter0x1f7e': '\xe1\xbd\xbe',
'UnicodeCharacter0x1f7f': '\xe1\xbd\xbf',
'HangulSyllablePyaet': '\xed\x8d\xb9',
'CjkUnifiedIdeograph9254': '\xe9\x89\x94',
'HangulSyllablePyaep': '\xed\x8d\xba',
'CjkUnifiedIdeograph49ed': '\xe4\xa7\xad',
'CjkUnifiedIdeograph49ee': '\xe4\xa7\xae',
'CjkUnifiedIdeograph49ef': '\xe4\xa7\xaf',
'HangulSyllableDdyiss': '\xeb\x9d\x98',
'CjkUnifiedIdeograph49ea': '\xe4\xa7\xaa',
'CjkUnifiedIdeograph49eb': '\xe4\xa7\xab',
'CjkUnifiedIdeograph49ec': '\xe4\xa7\xac',
'MyanmarLetterGha': '\xe1\x80\x83',
'GreekCapitalLetterEtaWithProsgegrammeni': '\xe1\xbf\x8c',
'ArabicLetterTchehehFinalForm': '\xef\xad\xbf',
'UnicodeCharacter0xe793': '\xee\x9e\x93',
'RightwardsTwoHeadedArrow': '\xe2\x86\xa0',
'UnicodeCharacter0xe791': '\xee\x9e\x91',
'UnicodeCharacter0xe790': '\xee\x9e\x90',
'UnicodeCharacter0xe797': '\xee\x9e\x97',
'UnicodeCharacter0xe796': '\xee\x9e\x96',
'UnicodeCharacter0xe795': '\xee\x9e\x95',
'UnicodeCharacter0xe794': '\xee\x9e\x94',
'CjkUnifiedIdeograph9255': '\xe9\x89\x95',
'UnicodeCharacter0xe798': '\xee\x9e\x98',
'ArabicLigatureBehWithHehInitialForm': '\xef\xb2\xa0',
'CjkUnifiedIdeograph48a5': '\xe4\xa2\xa5',
'CjkUnifiedIdeograph48a4': '\xe4\xa2\xa4',
'CjkUnifiedIdeograph48a7': '\xe4\xa2\xa7',
'CjkUnifiedIdeograph48a6': '\xe4\xa2\xa6',
'CjkUnifiedIdeograph48a1': '\xe4\xa2\xa1',
'CjkUnifiedIdeograph48a0': '\xe4\xa2\xa0',
'CjkUnifiedIdeograph48a3': '\xe4\xa2\xa3',
'CjkUnifiedIdeograph48a2': '\xe4\xa2\xa2',
'CjkUnifiedIdeograph48a9': '\xe4\xa2\xa9',
'CjkUnifiedIdeograph48a8': '\xe4\xa2\xa8',
'CjkUnifiedIdeograph960f': '\xe9\x98\x8f',
'CjkUnifiedIdeograph960e': '\xe9\x98\x8e',
'ArabicLetterFarsiYehWithExtendedArabicindicDigitTwoAbove': '\xdd\xb5',
'CjkUnifiedIdeograph960c': '\xe9\x98\x8c',
'CjkUnifiedIdeograph960b': '\xe9\x98\x8b',
'CjkUnifiedIdeograph960a': '\xe9\x98\x8a',
'HangulSyllableDyalh': '\xeb\x8c\xab',
'UnicodeCharacter0xf85d': '\xef\xa1\x9d',
'UnicodeCharacter0xf85f': '\xef\xa1\x9f',
'UnicodeCharacter0xf85a': '\xef\xa1\x9a',
'HangulSyllableDyalm': '\xeb\x8c\xa6',
'UnicodeCharacter0xf85c': '\xef\xa1\x9c',
'UnicodeCharacter0xf85b': '\xef\xa1\x9b',
'YiSyllableQut': '\xea\x90\x8c',
'CjkUnifiedIdeograph511f': '\xe5\x84\x9f',
'CjkUnifiedIdeograph511e': '\xe5\x84\x9e',
'CjkUnifiedIdeograph511d': '\xe5\x84\x9d',
'CjkUnifiedIdeograph511c': '\xe5\x84\x9c',
'CjkUnifiedIdeograph511b': '\xe5\x84\x9b',
'CjkUnifiedIdeograph511a': '\xe5\x84\x9a',
'HangulSyllableDyalg': '\xeb\x8c\xa5',
'HangulJongseongIeung': '\xe1\x86\xbc',
'CanadianSyllabicsCii': '\xe1\x92\x8c',
'HangulSyllableDyalp': '\xeb\x8c\xaa',
'HangulSyllableDyals': '\xeb\x8c\xa8',
'HangulSyllableDyalt': '\xeb\x8c\xa9',
'LatinCapitalLetterTurnedInsularG': '\xea\x9d\xbe',
'HangulSyllableJjeolm': '\xec\xa9\x96',
'UnicodeCharacter0xf555': '\xef\x95\x95',
'HangulSyllablePoh': '\xed\x90\x87',
'CjkUnifiedIdeograph9609': '\xe9\x98\x89',
'CjkUnifiedIdeograph9608': '\xe9\x98\x88',
'CjkUnifiedIdeograph9607': '\xe9\x98\x87',
'CjkUnifiedIdeograph9606': '\xe9\x98\x86',
'CjkUnifiedIdeograph9605': '\xe9\x98\x85',
'CjkUnifiedIdeograph9604': '\xe9\x98\x84',
'LatinSmallLetterDWithDotAbove': '\xe1\xb8\x8b',
'CjkUnifiedIdeograph9602': '\xe9\x98\x82',
'CjkUnifiedIdeograph9601': '\xe9\x98\x81',
'CjkUnifiedIdeograph9600': '\xe9\x98\x80',
'CjkUnifiedIdeograph48ae': '\xe4\xa2\xae',
'CjkUnifiedIdeograph48ad': '\xe4\xa2\xad',
'CjkUnifiedIdeograph48af': '\xe4\xa2\xaf',
'CjkUnifiedIdeograph48aa': '\xe4\xa2\xaa',
'CjkUnifiedIdeograph717d': '\xe7\x85\xbd',
'CjkUnifiedIdeograph48ac': '\xe4\xa2\xac',
'CjkUnifiedIdeograph48ab': '\xe4\xa2\xab',
'HangulSyllableTyonh': '\xed\x88\x96',
'HangulSyllableJjeolt': '\xec\xa9\x99',
'HangulSyllableJjeols': '\xec\xa9\x98',
'HangulSyllableJjeolp': '\xec\xa9\x9a',
'CjkUnifiedIdeograph9179': '\xe9\x85\xb9',
'CjkUnifiedIdeograph9178': '\xe9\x85\xb8',
'BottomSquareBracketOverTopSquareBracket': '\xe2\x8e\xb6',
'UnicodeCharacter0xf855': '\xef\xa1\x95',
'UnicodeCharacter0xf854': '\xef\xa1\x94',
'CjkUnifiedIdeograph9251': '\xe9\x89\x91',
'BlackVerySmallSquare': '\xe2\xac\x9d',
'UnicodeCharacter0xf851': '\xef\xa1\x91',
'UnicodeCharacter0xf850': '\xef\xa1\x90',
'CjkUnifiedIdeograph5119': '\xe5\x84\x99',
'CjkUnifiedIdeograph5118': '\xe5\x84\x98',
'CjkUnifiedIdeograph5117': '\xe5\x84\x97',
'CjkUnifiedIdeograph5116': '\xe5\x84\x96',
'CjkUnifiedIdeograph5115': '\xe5\x84\x95',
'CjkUnifiedIdeograph5114': '\xe5\x84\x94',
'CjkUnifiedIdeograph5113': '\xe5\x84\x93',
'CjkUnifiedIdeograph5112': '\xe5\x84\x92',
'CjkUnifiedIdeograph5111': '\xe5\x84\x91',
'CjkUnifiedIdeograph5110': '\xe5\x84\x90',
'HangulSyllableTyong': '\xed\x88\xa5',
'Ratio': '\xe2\x88\xb6',
'UnicodeCharacter0xdaca': '\xed\xab\x8a',
'HebrewLetterFinalKaf': '\xd7\x9a',
'UnicodeCharacter0xdacb': '\xed\xab\x8b',
'UnicodeCharacter0xd85c': '\xed\xa1\x9c',
'DoubleSuperset': '\xe2\x8b\x91',
'UnicodeCharacter0xdacc': '\xed\xab\x8c',
'LatinSubscriptSmallLetterSchwa': '\xe2\x82\x94',
'UnicodeCharacter0xdacd': '\xed\xab\x8d',
'UnicodeCharacter0xeb89': '\xee\xae\x89',
'CyrillicSmallLetterKaWithDescender': '\xd2\x9b',
'HangulSyllableKye': '\xec\xbc\xb8',
'IdeographicTelegraphSymbolForDayTwentyfour': '\xe3\x8f\xb7',
'CjkCompatibilityIdeographfa02': '\xef\xa8\x82',
'KhmerIndependentVowelQuu': '\xe1\x9e\xa9',
'UnicodeCharacter0xd85b': '\xed\xa1\x9b',
'ThaanaLetterTtaa': '\xde\x98',
'KhmerIndependentVowelQuk': '\xe1\x9e\xa8',
'SquareFm': '\xe3\x8e\x99',
'CjkUnifiedIdeograph345d': '\xe3\x91\x9d',
'CjkUnifiedIdeograph345e': '\xe3\x91\x9e',
'CjkUnifiedIdeograph345f': '\xe3\x91\x9f',
'SaurashtraDigitSeven': '\xea\xa3\x97',
'CjkUnifiedIdeograph345a': '\xe3\x91\x9a',
'CjkUnifiedIdeograph345b': '\xe3\x91\x9b',
'CjkUnifiedIdeograph345c': '\xe3\x91\x9c',
'CjkUnifiedIdeograph69e3': '\xe6\xa7\xa3',
'UnicodeCharacter0xd85a': '\xed\xa1\x9a',
'CjkUnifiedIdeograph8575': '\xe8\x95\xb5',
'UnicodeCharacter0xfee': '\xe0\xbf\xae',
'CombiningLatinSmallLetterFlattenedOpenAAbove': '\xe1\xb7\x93',
'UnicodeCharacter0xe881': '\xee\xa2\x81',
'CjkUnifiedIdeograph3458': '\xe3\x91\x98',
'CjkUnifiedIdeograph3459': '\xe3\x91\x99',
'ArabicLigatureTehWithKhahWithAlefMaksuraFinalForm': '\xef\xb6\xa2',
'CjkUnifiedIdeograph3454': '\xe3\x91\x94',
'CjkUnifiedIdeograph3455': '\xe3\x91\x95',
'CjkUnifiedIdeograph3456': '\xe3\x91\x96',
'CjkUnifiedIdeograph3457': '\xe3\x91\x97',
'RightTack': '\xe2\x8a\xa2',
'CjkUnifiedIdeograph3451': '\xe3\x91\x91',
'CjkUnifiedIdeograph3452': '\xe3\x91\x92',
'CjkUnifiedIdeograph3453': '\xe3\x91\x93',
'x': '\xce\xbe',
'HangulSyllableSswinj': '\xec\x92\xbd',
'BalineseVowelSignLaLengaTedung': '\xe1\xac\xbd',
'HebrewPointQubuts': '\xd6\xbb',
'CjkUnifiedIdeograph9517': '\xe9\x94\x97',
'CjkUnifiedIdeograph760e': '\xe7\x98\x8e',
'CjkUnifiedIdeograph760d': '\xe7\x98\x8d',
'CjkUnifiedIdeograph760f': '\xe7\x98\x8f',
'CjkUnifiedIdeograph760a': '\xe7\x98\x8a',
'CjkUnifiedIdeograph760c': '\xe7\x98\x8c',
'CjkUnifiedIdeograph760b': '\xe7\x98\x8b',
'HangulSyllableJit': '\xec\xa7\x99',
'CyrillicCapitalLetterTeWithDescender': '\xd2\xac',
'HangulSyllableJip': '\xec\xa7\x9a',
'HangulSyllableJis': '\xec\xa7\x93',
'UnicodeCharacter0xdc07': '\xed\xb0\x87',
'HangulSyllableJid': '\xec\xa7\x87',
'HangulSyllableJig': '\xec\xa7\x81',
'HangulSyllableCwe': '\xec\xb7\x8c',
'UnicodeCharacter0xdc03': '\xed\xb0\x83',
'UnicodeCharacter0xdc02': '\xed\xb0\x82',
'HangulSyllableJic': '\xec\xa7\x97',
'YiSyllableCyx': '\xea\x8b\x8b',
'CjkUnifiedIdeograph8c5f': '\xe8\xb1\x9f',
'HangulSyllableJil': '\xec\xa7\x88',
'DownwardsArrow': '\xe2\x86\x93',
'CjkUnifiedIdeograph8c5e': '\xe8\xb1\x9e',
'CjkUnifiedIdeograph8c5b': '\xe8\xb1\x9b',
'CjkUnifiedIdeograph8c5c': '\xe8\xb1\x9c',
'HangulSyllableJik': '\xec\xa7\x98',
'CjkUnifiedIdeograph8c5a': '\xe8\xb1\x9a',
'HangulSyllableGgalt': '\xea\xb9\x99',
'FullwidthDigitThree': '\xef\xbc\x93',
'CjkUnifiedIdeograph7609': '\xe7\x98\x89',
'CjkUnifiedIdeograph7608': '\xe7\x98\x88',
'LatinSmallLetterKra': '\xc4\xb8',
'TagbanwaLetterA': '\xe1\x9d\xa0',
'CjkUnifiedIdeograph7605': '\xe7\x98\x85',
'VulgarFractionFourFifths': '\xe2\x85\x98',
'CjkUnifiedIdeograph7607': '\xe7\x98\x87',
'CjkUnifiedIdeograph690f': '\xe6\xa4\x8f',
'NegativeCircledNumberThirteen': '\xe2\x93\xad',
'CjkUnifiedIdeograph7600': '\xe7\x98\x80',
'CjkUnifiedIdeograph7603': '\xe7\x98\x83',
'TagbanwaLetterI': '\xe1\x9d\xa1',
'WhiteSquareWithVerticalBisectingLine': '\xe2\x97\xab',
'CjkUnifiedIdeograph76f9': '\xe7\x9b\xb9',
'HangulSyllableGgalg': '\xea\xb9\x95',
'TagbanwaLetterU': '\xe1\x9d\xa2',
'HangulSyllableRyenh': '\xeb\xa1\x86',
'HangulSyllableRyenj': '\xeb\xa1\x85',
'HangulSyllableGgalb': '\xea\xb9\x97',
'HangulSyllableGgalm': '\xea\xb9\x96',
'ArabicLetterAlefWithHamzaAboveFinalForm': '\xef\xba\x84',
'HangulSyllableRyeng': '\xeb\xa1\x95',
'HangulSyllableGgalh': '\xea\xb9\x9b',
'UnicodeCharacter0xdc0f': '\xed\xb0\x8f',
'UnicodeCharacter0xdc0e': '\xed\xb0\x8e',
'UnicodeCharacter0xdc0d': '\xed\xb0\x8d',
'UnicodeCharacter0xdc0c': '\xed\xb0\x8c',
'UnicodeCharacter0xdc0b': '\xed\xb0\x8b',
'CjkUnifiedIdeograph8c58': '\xe8\xb1\x98',
'CjkUnifiedIdeograph8c59': '\xe8\xb1\x99',
'CjkUnifiedIdeograph8c56': '\xe8\xb1\x96',
'CjkUnifiedIdeograph8c57': '\xe8\xb1\x97',
'CjkUnifiedIdeograph8c54': '\xe8\xb1\x94',
'CjkUnifiedIdeograph8c55': '\xe8\xb1\x95',
'CjkUnifiedIdeograph8c52': '\xe8\xb1\x92',
'CjkUnifiedIdeograph8c53': '\xe8\xb1\x93',
'CjkUnifiedIdeograph8c50': '\xe8\xb1\x90',
'CjkUnifiedIdeograph8c51': '\xe8\xb1\x91',
'CjkUnifiedIdeograph431f': '\xe4\x8c\x9f',
'UnicodeCharacter0xf2c0': '\xef\x8b\x80',
'UnicodeCharacter0xe880': '\xee\xa2\x80',
'HangulSyllableDdeng': '\xeb\x97\x91',
'HangulSyllableDdenh': '\xeb\x97\x82',
'HangulSyllableDdenj': '\xeb\x97\x81',
'UnicodeCharacter0x20ff': '\xe2\x83\xbf',
'HangulSyllableSsoelp': '\xec\x90\xba',
'HangulSyllableSsoels': '\xec\x90\xb8',
'HangulSyllableHegg': '\xed\x97\xa6',
'LatinSmallLetterNWithCurl': '\xc8\xb5',
'HangulSyllableSsoelt': '\xec\x90\xb9',
'CjkUnifiedIdeograph81c8': '\xe8\x87\x88',
'CjkUnifiedIdeograph81c9': '\xe8\x87\x89',
'CjkUnifiedIdeograph81c6': '\xe8\x87\x86',
'CjkUnifiedIdeograph81c7': '\xe8\x87\x87',
'CjkUnifiedIdeograph81c4': '\xe8\x87\x84',
'CjkUnifiedIdeograph81c5': '\xe8\x87\x85',
'CjkUnifiedIdeograph81c2': '\xe8\x87\x82',
'CjkUnifiedIdeograph81c3': '\xe8\x87\x83',
'CjkUnifiedIdeograph81c0': '\xe8\x87\x80',
'CjkUnifiedIdeograph431c': '\xe4\x8c\x9c',
'CjkUnifiedIdeograph593a': '\xe5\xa4\xba',
'CjkUnifiedIdeograph593c': '\xe5\xa4\xbc',
'CjkUnifiedIdeograph593b': '\xe5\xa4\xbb',
'CjkUnifiedIdeograph593e': '\xe5\xa4\xbe',
'CjkUnifiedIdeograph593d': '\xe5\xa4\xbd',
'HangulSyllableSsoelg': '\xec\x90\xb5',
'CjkUnifiedIdeograph4f0e': '\xe4\xbc\x8e',
'HangulSyllableSsoelh': '\xec\x90\xbb',
'UnicodeCharacter0x20fc': '\xe2\x83\xbc',
'HangulSyllableSsoelm': '\xec\x90\xb6',
'UnicodeCharacter0xe6ff': '\xee\x9b\xbf',
'CjkUnifiedIdeograph8c77': '\xe8\xb1\xb7',
'LatinSmallLetterDzWithCaron': '\xc7\x86',
'HangulSyllableGwelg': '\xea\xb6\xad',
'ArabicLigatureYehWithHamzaAboveWithOeFinalForm': '\xef\xaf\xb3',
'HangulSyllableGwelh': '\xea\xb6\xb3',
'HangulSyllableGwelm': '\xea\xb6\xae',
'HangulSyllableGwelp': '\xea\xb6\xb2',
'HangulSyllableGwels': '\xea\xb6\xb0',
'HangulSyllableGwelt': '\xea\xb6\xb1',
'ArabicLigatureNoonWithMeemFinalForm': '\xef\xb2\x8c',
'UnicodeCharacter0xf242': '\xef\x89\x82',
'OlChikiDigitEight': '\xe1\xb1\x98',
'CjkUnifiedIdeograph5931': '\xe5\xa4\xb1',
'CjkUnifiedIdeograph5930': '\xe5\xa4\xb0',
'CjkUnifiedIdeograph5933': '\xe5\xa4\xb3',
'CjkUnifiedIdeograph5932': '\xe5\xa4\xb2',
'CjkUnifiedIdeograph5935': '\xe5\xa4\xb5',
'CjkUnifiedIdeograph5934': '\xe5\xa4\xb4',
'CjkUnifiedIdeograph5937': '\xe5\xa4\xb7',
'CjkUnifiedIdeograph5936': '\xe5\xa4\xb6',
'CjkUnifiedIdeograph5939': '\xe5\xa4\xb9',
'CjkUnifiedIdeograph5938': '\xe5\xa4\xb8',
'imag': '\xe2\x85\x88',
'HangulSyllableReulp': '\xeb\xa6\x82',
'MediumSmallWhiteCircle': '\xe2\x9a\xac',
'CjkUnifiedIdeograph81cf': '\xe8\x87\x8f',
'CjkUnifiedIdeograph81cd': '\xe8\x87\x8d',
'CjkUnifiedIdeograph81ce': '\xe8\x87\x8e',
'CjkUnifiedIdeograph81cb': '\xe8\x87\x8b',
'CjkUnifiedIdeograph81cc': '\xe8\x87\x8c',
'CjkUnifiedIdeograph81ca': '\xe8\x87\x8a',
'YiSyllableNrut': '\xea\x8e\x9f',
'HangulSyllableReult': '\xeb\xa6\x81',
'LatinCapitalLetterAo': '\xea\x9c\xb4',
'EthiopicSyllableCi': '\xe1\x89\xba',
'LatinCapitalLetterAa': '\xea\x9c\xb2',
'LatinCapitalLetterUWithBreve': '\xc5\xac',
'LatinCapitalLetterAe': '\xc3\x86',
'YiSyllableNrur': '\xea\x8e\xa4',
'LatinCapitalLetterAy': '\xea\x9c\xbc',
'YiSyllableShyr': '\xea\x8f\x85',
'CanadianSyllabicsWestcreeRo': '\xe1\x96\x8c',
'YiSyllableNrup': '\xea\x8e\xa2',
'LatinCapitalLetterAu': '\xea\x9c\xb6',
'YiSyllableShyp': '\xea\x8f\x83',
'LatinCapitalLetterAv': '\xea\x9c\xb8',
'CjkUnifiedIdeograph3574': '\xe3\x95\xb4',
'CjkUnifiedIdeograph836e': '\xe8\x8d\xae',
'ModifierLetterLeftHalfRing': '\xca\xbf',
'HangulSyllableJyugg': '\xec\xa5\xae',
'HangulSyllableNeulp': '\xeb\x8a\x9e',
'YiSyllableShyt': '\xea\x8f\x80',
'UnicodeCharacter0xfb19': '\xef\xac\x99',
'CanadianSyllabicsWestcreeRe': '\xe1\x96\x8a',
'UnicodeCharacter0xeffc': '\xee\xbf\xbc',
'UnicodeCharacter0xfb12': '\xef\xac\x92',
'UnicodeCharacter0xfb11': '\xef\xac\x91',
'UnicodeCharacter0xfb10': '\xef\xac\x90',
'CjkUnifiedIdeograph94b3': '\xe9\x92\xb3',
'CjkUnifiedIdeograph94b2': '\xe9\x92\xb2',
'CjkUnifiedIdeograph94b1': '\xe9\x92\xb1',
'CjkUnifiedIdeograph94b0': '\xe9\x92\xb0',
'CjkUnifiedIdeograph94b7': '\xe9\x92\xb7',
'CjkUnifiedIdeograph94b6': '\xe9\x92\xb6',
'CjkUnifiedIdeograph94b5': '\xe9\x92\xb5',
'CombiningDoubleAcuteAccent': '\xcc\x8b',
'CjkUnifiedIdeograph94b9': '\xe9\x92\xb9',
'CjkUnifiedIdeograph94b8': '\xe9\x92\xb8',
'LessthanOrGreaterthan': '\xe2\x89\xb6',
'CanadianSyllabicsWestcreeRa': '\xe1\x96\x8d',
'UnicodeCharacter0xeea9': '\xee\xba\xa9',
'ChamLetterAi': '\xea\xa8\x84',
'CanadianSyllabicsWestcreeShwaa': '\xe1\x94\xa4',
'UnicodeCharacter0xeea5': '\xee\xba\xa5',
'UnicodeCharacter0xeea4': '\xee\xba\xa4',
'UnicodeCharacter0xeea7': '\xee\xba\xa7',
'CjkUnifiedIdeograph91b5': '\xe9\x86\xb5',
'UnicodeCharacter0xeea1': '\xee\xba\xa1',
'UnicodeCharacter0xeea0': '\xee\xba\xa0',
'UnicodeCharacter0xeea3': '\xee\xba\xa3',
'HangulSyllableReulg': '\xeb\xa5\xbd',
'CjkUnifiedIdeograph836c': '\xe8\x8d\xac',
'CopticCapitalLetterPsi': '\xe2\xb2\xae',
'UnicodeCharacter0xfb1b': '\xef\xac\x9b',
'UnicodeCharacter0xfb1a': '\xef\xac\x9a',
'TagalogLetterBa': '\xe1\x9c\x8a',
'UnicodeCharacter0xf557': '\xef\x95\x97',
'HangulSyllableBals': '\xeb\xb0\xa0',
'HangulChoseongSiosthieuth': '\xe1\x84\xb9',
'UnicodeCharacter0xeeae': '\xee\xba\xae',
'UnicodeCharacter0xeead': '\xee\xba\xad',
'UnicodeCharacter0xeeaf': '\xee\xba\xaf',
'DownFishTail': '\xe2\xa5\xbf',
'CjkUnifiedIdeograph9b63': '\xe9\xad\xa3',
'UnicodeCharacter0xeeac': '\xee\xba\xac',
'UnicodeCharacter0xeeab': '\xee\xba\xab',
'CjkUnifiedIdeograph94bc': '\xe9\x92\xbc',
'CjkUnifiedIdeograph94bb': '\xe9\x92\xbb',
'CjkUnifiedIdeograph94ba': '\xe9\x92\xba',
'UnicodeCharacter0xdac3': '\xed\xab\x83',
'CjkUnifiedIdeograph94bf': '\xe9\x92\xbf',
'CjkUnifiedIdeograph94be': '\xe9\x92\xbe',
'CjkUnifiedIdeograph94bd': '\xe9\x92\xbd',
'MongolianLetterManchuAliGaliTa': '\xe1\xa2\xa0',
'UnicodeCharacter0xdac4': '\xed\xab\x84',
'CjkCompatibilityIdeographf94a': '\xef\xa5\x8a',
'UnicodeCharacter0xdac5': '\xed\xab\x85',
'CanadianSyllabicsCarrierGhee': '\xe1\x97\x87',
'HangulSyllablePass': '\xed\x8c\xa0',
'UnicodeCharacter0xede8': '\xee\xb7\xa8',
'UnicodeCharacter0xdac6': '\xed\xab\x86',
'CjkUnifiedIdeograph492d': '\xe4\xa4\xad',
'CjkUnifiedIdeograph7acc': '\xe7\xab\x8c',
'CjkUnifiedIdeograph7acb': '\xe7\xab\x8b',
'CjkUnifiedIdeograph7aca': '\xe7\xab\x8a',
'UnicodeCharacter0xdac7': '\xed\xab\x87',
'UnicodeCharacter0xede0': '\xee\xb7\xa0',
'CjkUnifiedIdeograph7acf': '\xe7\xab\x8f',
'CjkUnifiedIdeograph7ace': '\xe7\xab\x8e',
'CjkUnifiedIdeograph7acd': '\xe7\xab\x8d',
'UnicodeCharacter0xdac8': '\xed\xab\x88',
'UnicodeCharacter0xdac9': '\xed\xab\x89',
'HangulJongseongMieumpieup': '\xe1\x87\x9c',
'HangulSyllableDdyoj': '\xeb\x9a\x96',
'HangulSyllableDdyok': '\xeb\x9a\x98',
'HangulSyllableDdyoh': '\xeb\x9a\x9b',
'HangulSyllableMuk': '\xeb\xad\x8c',
'HangulSyllableDdyon': '\xeb\x9a\x84',
'HangulSyllableDdyol': '\xeb\x9a\x88',
'HangulSyllableDdyom': '\xeb\x9a\x90',
'TifinaghLetterYazz': '\xe2\xb5\xa5',
'HangulSyllableDdyoc': '\xeb\x9a\x97',
'CjkUnifiedIdeograph492b': '\xe4\xa4\xab',
'HangulSyllableDdyog': '\xeb\x9a\x81',
'HangulSyllableDdyod': '\xeb\x9a\x87',
'HangulSyllableHeuj': '\xed\x9d\xa6',
'IdeographicFullStop': '\xe3\x80\x82',
'TifinaghLetterYazh': '\xe2\xb5\x8a',
'HangulSyllableRyek': '\xeb\xa1\x98',
'HangulSyllableDdyot': '\xeb\x9a\x99',
'CjkUnifiedIdeograph928f': '\xe9\x8a\x8f',
'CjkUnifiedIdeograph7959': '\xe7\xa5\x99',
'LatinSmallLetterZWithLineBelow': '\xe1\xba\x95',
'ArabicLetterHehFinalForm': '\xef\xbb\xaa',
'CjkUnifiedIdeograph7ac9': '\xe7\xab\x89',
'CjkUnifiedIdeograph7ac8': '\xe7\xab\x88',
'CjkUnifiedIdeograph7ac3': '\xe7\xab\x83',
'CjkUnifiedIdeograph7ac2': '\xe7\xab\x82',
'CjkUnifiedIdeograph7ac1': '\xe7\xab\x81',
'CjkRadicalSun': '\xe2\xba\x9c',
'CjkUnifiedIdeograph7ac7': '\xe7\xab\x87',
'CjkUnifiedIdeograph7ac6': '\xe7\xab\x86',
'GreekSmallLetterEpsilonWithPsili': '\xe1\xbc\x90',
'CjkUnifiedIdeograph7ac4': '\xe7\xab\x84',
'CanadianSyllabicsYo': '\xe1\x94\xaa',
'CanadianSyllabicsYi': '\xe1\x94\xa8',
'CanadianSyllabicsYe': '\xe1\x94\xa6',
'CanadianSyllabicsYa': '\xe1\x94\xad',
'HangulSyllableJjabs': '\xec\xa7\xae',
'CjkUnifiedIdeograph8eb6': '\xe8\xba\xb6',
'HangulSyllableHeup': '\xed\x9d\xaa',
'LaoLetterPhoTam': '\xe0\xba\x9e',
'HangulSyllableRyem': '\xeb\xa1\x90',
'LatinCapitalLetterEzh': '\xc6\xb7',
'ArabicLigatureTehWithMeemFinalForm': '\xef\xb1\xb2',
'HangulSyllablePyeogg': '\xed\x8e\xb6',
'CjkUnifiedIdeograph840b': '\xe8\x90\x8b',
'CjkUnifiedIdeograph840c': '\xe8\x90\x8c',
'CjkUnifiedIdeograph840a': '\xe8\x90\x8a',
'CjkUnifiedIdeograph840f': '\xe8\x90\x8f',
'CjkUnifiedIdeograph7e3d': '\xe7\xb8\xbd',
'CjkUnifiedIdeograph840d': '\xe8\x90\x8d',
'CjkUnifiedIdeograph840e': '\xe8\x90\x8e',
'CombiningLeftHalfRingAbove': '\xcd\x91',
'HangulSyllableBbwing': '\xec\x80\xb5',
'UnicodeCharacter0xded0': '\xed\xbb\x90',
'CjkUnifiedIdeograph6dfc': '\xe6\xb7\xbc',
'CyrillicCapitalLetterIota': '\xea\x99\x86',
'HangulJungseongYeoo': '\xe1\x85\xbd',
'UpperHalfInverseWhiteCircle': '\xe2\x97\x9a',
'HangulSyllableDaelh': '\xeb\x8c\x8f',
'HangulJungseongYeou': '\xe1\x85\xbe',
'CjkUnifiedIdeograph6355': '\xe6\x8d\x95',
'CjkUnifiedIdeograph6356': '\xe6\x8d\x96',
'CjkUnifiedIdeograph6357': '\xe6\x8d\x97',
'CjkUnifiedIdeograph6350': '\xe6\x8d\x90',
'CjkUnifiedIdeograph6351': '\xe6\x8d\x91',
'CjkUnifiedIdeograph6352': '\xe6\x8d\x92',
'CjkUnifiedIdeograph6353': '\xe6\x8d\x93',
'UnicodeCharacter0xf6a8': '\xef\x9a\xa8',
'CjkUnifiedIdeograph6358': '\xe6\x8d\x98',
'CjkUnifiedIdeograph6359': '\xe6\x8d\x99',
'CjkUnifiedIdeograph91ba': '\xe9\x86\xba',
'SubscriptFour': '\xe2\x82\x84',
'HangulSyllableRyugs': '\xeb\xa5\x9b',
'ReverseTildeOperatorAboveLeftwardsArrow': '\xe2\xad\x81',
'UnicodeCharacter0xc71': '\xe0\xb1\xb1',
'UnicodeCharacter0xc70': '\xe0\xb1\xb0',
'UnicodeCharacter0xc73': '\xe0\xb1\xb3',
'UnicodeCharacter0xc72': '\xe0\xb1\xb2',
'UnicodeCharacter0xc75': '\xe0\xb1\xb5',
'UnicodeCharacter0xc74': '\xe0\xb1\xb4',
'UnicodeCharacter0xc77': '\xe0\xb1\xb7',
'UnicodeCharacter0xc76': '\xe0\xb1\xb6',
'CjkUnifiedIdeograph8402': '\xe8\x90\x82',
'CjkUnifiedIdeograph8403': '\xe8\x90\x83',
'TibetanSyllableOm': '\xe0\xbc\x80',
'CjkUnifiedIdeograph8401': '\xe8\x90\x81',
'CjkUnifiedIdeograph8406': '\xe8\x90\x86',
'EthiopicWordspace': '\xe1\x8d\xa1',
'CjkUnifiedIdeograph8404': '\xe8\x90\x84',
'CjkUnifiedIdeograph8405': '\xe8\x90\x85',
'CjkUnifiedIdeograph635d': '\xe6\x8d\x9d',
'CjkUnifiedIdeograph635e': '\xe6\x8d\x9e',
'CjkUnifiedIdeograph635f': '\xe6\x8d\x9f',
'CjkUnifiedIdeograph4315': '\xe4\x8c\x95',
'CjkUnifiedIdeograph635a': '\xe6\x8d\x9a',
'CjkUnifiedIdeograph635b': '\xe6\x8d\x9b',
'CjkUnifiedIdeograph635c': '\xe6\x8d\x9c',
'GreekDialytikaAndOxia': '\xe1\xbf\xae',
'UnicodeCharacter0x20f1': '\xe2\x83\xb1',
'UnicodeCharacter0xe6f0': '\xee\x9b\xb0',
'UnicodeCharacter0x20f2': '\xe2\x83\xb2',
'HangulSyllablePyeogs': '\xed\x8e\xb7',
'CjkUnifiedIdeograph4311': '\xe4\x8c\x91',
'HangulSyllableEub': '\xec\x9d\x8d',
'UnicodeCharacter0xee16': '\xee\xb8\x96',
'HangulSyllableMoel': '\xeb\xac\x84',
'HalfwidthHangulLetterWae': '\xef\xbf\x8e',
'TibetanDigitSix': '\xe0\xbc\xa6',
'MalayalamLetterDdha': '\xe0\xb4\xa2',
'LimbuLetterNga': '\xe1\xa4\x85',
'HangulSyllableCass': '\xec\xb0\xbc',
'HangulSyllableJjyim': '\xec\xb0\x80',
'LatinSmallLetterUWithInvertedBreve': '\xc8\x97',
'UnicodeCharacter0xa7a1': '\xea\x9e\xa1',
'HangulSyllableNyabs': '\xeb\x83\xa2',
'CjkUnifiedIdeograph7bc8': '\xe7\xaf\x88',
'CjkUnifiedIdeograph7bc9': '\xe7\xaf\x89',
'NewTaiLueLetterHighKva': '\xe1\xa6\xa6',
'CjkUnifiedIdeograph7bc5': '\xe7\xaf\x85',
'CjkUnifiedIdeograph7bc6': '\xe7\xaf\x86',
'CjkUnifiedIdeograph7bc7': '\xe7\xaf\x87',
'CjkUnifiedIdeograph7bc0': '\xe7\xaf\x80',
'CjkUnifiedIdeograph7bc1': '\xe7\xaf\x81',
'CjkUnifiedIdeograph7bc2': '\xe7\xaf\x82',
'CjkUnifiedIdeograph7bc3': '\xe7\xaf\x83',
'TamilLetterRra': '\xe0\xae\xb1',
'HangulSyllableBbyaegg': '\xeb\xba\xb6',
'CjkUnifiedIdeograph5ef8': '\xe5\xbb\xb8',
'CjkUnifiedIdeograph5ef9': '\xe5\xbb\xb9',
'NegativeCircledNumberEleven': '\xe2\x93\xab',
'CjkUnifiedIdeograph5ef7': '\xe5\xbb\xb7',
'CjkUnifiedIdeograph5ef4': '\xe5\xbb\xb4',
'CjkUnifiedIdeograph5ef5': '\xe5\xbb\xb5',
'TeluguLetterOo': '\xe0\xb0\x93',
'CjkUnifiedIdeograph5ef3': '\xe5\xbb\xb3',
'CjkUnifiedIdeograph5ef0': '\xe5\xbb\xb0',
'CjkUnifiedIdeograph5ef1': '\xe5\xbb\xb1',
'CjkUnifiedIdeograph9283': '\xe9\x8a\x83',
'LatinSmallLetterCWithCurl': '\xc9\x95',
'CjkUnifiedIdeograph9282': '\xe9\x8a\x82',
'HangulSyllableBbyang': '\xeb\xba\xad',
'HangulSyllableJaebs': '\xec\x9e\xbe',
'ArabicLetterHahIsolatedForm': '\xef\xba\xa1',
'HangulSyllableJwegg': '\xec\xa4\xb6',
'YiRadicalZup': '\xea\x92\xa2',
'CjkUnifiedIdeograph9287': '\xe9\x8a\x87',
'HangulSyllableJwegs': '\xec\xa4\xb7',
'HangulSyllablePwim': '\xed\x93\x9c',
'CjkUnifiedIdeograph9286': '\xe9\x8a\x86',
'ArabicLetterYehFinalForm': '\xef\xbb\xb2',
'LatinSmallLetterIotaWithStroke': '\xe1\xb5\xbc',
'HangulSyllableSis': '\xec\x8b\xaf',
'CjkUnifiedIdeograph5eff': '\xe5\xbb\xbf',
'CjkUnifiedIdeograph5efd': '\xe5\xbb\xbd',
'CjkUnifiedIdeograph5efe': '\xe5\xbb\xbe',
'CjkUnifiedIdeograph5efb': '\xe5\xbb\xbb',
'GreekCapitalLetterAlphaWithPsiliAndVariaAndProsgegrammeni': '\xe1\xbe\x8a',
'CjkUnifiedIdeograph5efa': '\xe5\xbb\xba',
'Perspective': '\xe2\x8c\x86',
'ArabicLigatureSheenWithKhahMedialForm': '\xef\xb4\xb9',
'CjkUnifiedIdeograph7bcd': '\xe7\xaf\x8d',
'CjkUnifiedIdeograph7bce': '\xe7\xaf\x8e',
'CjkUnifiedIdeograph7bcf': '\xe7\xaf\x8f',
'CjkUnifiedIdeograph7bca': '\xe7\xaf\x8a',
'CjkUnifiedIdeograph7bcb': '\xe7\xaf\x8b',
'CjkUnifiedIdeograph7bcc': '\xe7\xaf\x8c',
'CjkUnifiedIdeograph9288': '\xe9\x8a\x88',
'HangulSyllableSit': '\xec\x8b\xb5',
'LatinCapitalLetterEWithCircumflexAndTilde': '\xe1\xbb\x84',
'ArabicLetterSeen': '\xd8\xb3',
'CjkUnifiedIdeograph3f78': '\xe3\xbd\xb8',
'CjkUnifiedIdeograph3f79': '\xe3\xbd\xb9',
'CircledIdeographSuitable': '\xe3\x8a\x9c',
'CjkUnifiedIdeograph3f70': '\xe3\xbd\xb0',
'CjkUnifiedIdeograph3f71': '\xe3\xbd\xb1',
'CjkUnifiedIdeograph3f72': '\xe3\xbd\xb2',
'CjkUnifiedIdeograph3f73': '\xe3\xbd\xb3',
'CjkUnifiedIdeograph3f74': '\xe3\xbd\xb4',
'CjkUnifiedIdeograph3f75': '\xe3\xbd\xb5',
'CjkUnifiedIdeograph3f76': '\xe3\xbd\xb6',
'LatinCapitalLetterGWithHook': '\xc6\x93',
'BraillePatternDots1257': '\xe2\xa1\x93',
'BraillePatternDots1256': '\xe2\xa0\xb3',
'UnicodeCharacter0xf7e4': '\xef\x9f\xa4',
'HangulSyllableEp': '\xec\x97\xaa',
'UnicodeCharacter0xe80b': '\xee\xa0\x8b',
'UnicodeCharacter0xe80a': '\xee\xa0\x8a',
'BraillePatternDots1258': '\xe2\xa2\x93',
'HangulSyllableEt': '\xec\x97\xa9',
'HangulSyllableEu': '\xec\x9c\xbc',
'UnicodeCharacter0xe80e': '\xee\xa0\x8e',
'UnicodeCharacter0xe80d': '\xee\xa0\x8d',
'HangulSyllableEh': '\xec\x97\xab',
'HangulSyllableEj': '\xec\x97\xa6',
'HangulSyllableEk': '\xec\x97\xa8',
'HangulSyllableEl': '\xec\x97\x98',
'HangulSyllableEm': '\xec\x97\xa0',
'HangulSyllableEn': '\xec\x97\x94',
'HangulSyllableEo': '\xec\x96\xb4',
'HangulSyllableEb': '\xec\x97\xa1',
'HangulSyllableEc': '\xec\x97\xa7',
'HangulSyllableEd': '\xec\x97\x97',
'HangulSyllableDdae': '\xeb\x95\x8c',
'HangulSyllableEg': '\xec\x97\x91',
'VaiSyllableZa': '\xea\x95\xa4',
'DiamondWithBottomHalfBlack': '\xe2\xac\x99',
'VaiSyllableZe': '\xea\x97\xbd',
'CjkUnifiedIdeograph3f7a': '\xe3\xbd\xba',
'CjkUnifiedIdeograph3f7b': '\xe3\xbd\xbb',
'CjkUnifiedIdeograph3f7c': '\xe3\xbd\xbc',
'CjkUnifiedIdeograph3f7d': '\xe3\xbd\xbd',
'CjkUnifiedIdeograph3f7e': '\xe3\xbd\xbe',
'CjkUnifiedIdeograph3f7f': '\xe3\xbd\xbf',
'VaiSyllableZu': '\xea\x96\xae',
'HangulSyllableMwalb': '\xeb\xab\x8f',
'HangulSyllableMwalg': '\xeb\xab\x8d',
'HangulSyllableMwalh': '\xeb\xab\x93',
'HangulSyllableBbwinh': '\xec\x80\xa6',
'HangulSyllableMwalm': '\xeb\xab\x8e',
'HangulSyllableMwals': '\xeb\xab\x90',
'UnicodeCharacter0xe809': '\xee\xa0\x89',
'HangulSyllableMwalp': '\xeb\xab\x92',
'HangulSyllableMwalt': '\xeb\xab\x91',
'UnicodeCharacter0xe803': '\xee\xa0\x83',
'UnicodeCharacter0xe802': '\xee\xa0\x82',
'UnicodeCharacter0xe801': '\xee\xa0\x81',
'UnicodeCharacter0xe800': '\xee\xa0\x80',
'UnicodeCharacter0xe807': '\xee\xa0\x87',
'UnicodeCharacter0xe806': '\xee\xa0\x86',
'UnicodeCharacter0xe805': '\xee\xa0\x85',
'CjkUnifiedIdeograph4920': '\xe4\xa4\xa0',
'UnicodeCharacter0xec62': '\xee\xb1\xa2',
'YiSyllableHlyt': '\xea\x86\xb1',
'UnicodeCharacter0xec63': '\xee\xb1\xa3',
'ContainsAsNormalSubgroup': '\xe2\x8a\xb3',
'HangulSyllableRep': '\xeb\xa0\xa2',
'UnicodeCharacter0xec61': '\xee\xb1\xa1',
'PresentationFormForVerticalRightCurlyBracket': '\xef\xb8\xb8',
'UnicodeCharacter0xec66': '\xee\xb1\xa6',
'UnicodeCharacter0xe92c': '\xee\xa4\xac',
'UnicodeCharacter0xec67': '\xee\xb1\xa7',
'HangulSyllableGgenj': '\xea\xbb\x9d',
'HangulSyllableGgenh': '\xea\xbb\x9e',
'HangulSyllableGgeng': '\xea\xbb\xad',
'UnicodeCharacter0xec65': '\xee\xb1\xa5',
'UnicodeCharacter0xf2cd': '\xef\x8b\x8d',
'HangulSyllableRes': '\xeb\xa0\x9b',
'HangulSyllableTyugg': '\xed\x8a\x9e',
'CjkUnifiedIdeograph6b02': '\xe6\xac\x82',
'CjkUnifiedIdeograph6b03': '\xe6\xac\x83',
'CjkUnifiedIdeograph6b00': '\xe6\xac\x80',
'CjkUnifiedIdeograph6b01': '\xe6\xac\x81',
'CjkUnifiedIdeograph6b06': '\xe6\xac\x86',
'CjkUnifiedIdeograph6b07': '\xe6\xac\x87',
'CjkUnifiedIdeograph6b04': '\xe6\xac\x84',
'CjkUnifiedIdeograph6b05': '\xe6\xac\x85',
'CjkUnifiedIdeograph7c57': '\xe7\xb1\x97',
'CjkUnifiedIdeograph7c56': '\xe7\xb1\x96',
'CjkUnifiedIdeograph6b08': '\xe6\xac\x88',
'CjkUnifiedIdeograph6b09': '\xe6\xac\x89',
'CjkUnifiedIdeograph7c53': '\xe7\xb1\x93',
'CjkUnifiedIdeograph7c52': '\xe7\xb1\x92',
'CjkUnifiedIdeograph7c51': '\xe7\xb1\x91',
'CjkUnifiedIdeograph7c50': '\xe7\xb1\x90',
'FullwidthDigitTwo': '\xef\xbc\x92',
'UnicodeCharacter0xe22f': '\xee\x88\xaf',
'BraillePatternDots358': '\xe2\xa2\x94',
'HangulSyllableDweonh': '\xeb\x91\xb2',
'HangulSyllableSanj': '\xec\x82\xb1',
'HangulSyllableNyubs': '\xeb\x8a\x86',
'HangulSyllableSang': '\xec\x83\x81',
'ParenthesizedIdeographResource': '\xe3\x88\xbe',
'HangulSyllableCaenj': '\xec\xb1\x89',
'RecyclingSymbolForType5Plastics': '\xe2\x99\xb7',
'HangulSyllableDweong': '\xeb\x92\x81',
'BraillePatternDots356': '\xe2\xa0\xb4',
'BraillePatternDots357': '\xe2\xa1\x94',
'CjkUnifiedIdeograph6b0b': '\xe6\xac\x8b',
'CjkUnifiedIdeograph6b0c': '\xe6\xac\x8c',
'CjkUnifiedIdeograph6b0a': '\xe6\xac\x8a',
'CjkUnifiedIdeograph6b0f': '\xe6\xac\x8f',
'CjkUnifiedIdeograph6b0d': '\xe6\xac\x8d',
'CjkUnifiedIdeograph6b0e': '\xe6\xac\x8e',
'CjkUnifiedIdeograph7c5f': '\xe7\xb1\x9f',
'CjkUnifiedIdeograph7c5e': '\xe7\xb1\x9e',
'CjkUnifiedIdeograph7c5d': '\xe7\xb1\x9d',
'CjkUnifiedIdeograph7c5c': '\xe7\xb1\x9c',
'CjkUnifiedIdeograph7c5b': '\xe7\xb1\x9b',
'CjkUnifiedIdeograph7c5a': '\xe7\xb1\x9a',
'LimbuSubjoinedLetterRa': '\xe1\xa4\xaa',
'SquareSaikuru': '\xe3\x8c\x9f',
'BraillePatternDots34567': '\xe2\xa1\xbc',
'BoxDrawingsHeavyDownAndLeft': '\xe2\x94\x93',
'TamilDigitSeven': '\xe0\xaf\xad',
'LatinSmallLetterTurnedMWithLongLeg': '\xc9\xb0',
'BraillePatternDots34568': '\xe2\xa2\xbc',
'CjkUnifiedIdeograph55c0': '\xe5\x97\x80',
'UnicodeCharacter0xe012': '\xee\x80\x92',
'IntegralWithOverbar': '\xe2\xa8\x9b',
'HangulSyllableRet': '\xeb\xa0\xa1',
'CjkUnifiedIdeograph4b35': '\xe4\xac\xb5',
'UnicodeCharacter0x2dd7': '\xe2\xb7\x97',
'CjkUnifiedIdeograph4b37': '\xe4\xac\xb7',
'CjkUnifiedIdeograph4b36': '\xe4\xac\xb6',
'CjkUnifiedIdeograph4b31': '\xe4\xac\xb1',
'CjkUnifiedIdeograph4b30': '\xe4\xac\xb0',
'CjkUnifiedIdeograph4b33': '\xe4\xac\xb3',
'CjkUnifiedIdeograph4b32': '\xe4\xac\xb2',
'CjkUnifiedIdeograph4b39': '\xe4\xac\xb9',
'CjkUnifiedIdeograph4b38': '\xe4\xac\xb8',
'WavyOverline': '\xef\xb9\x8b',
'HangulSyllableSsigg': '\xec\x94\xaa',
'YiSyllableGgix': '\xea\x88\x95',
'EmptySetWithRightArrowAbove': '\xe2\xa6\xb3',
'HangulSyllableKyeoss': '\xec\xbc\xb0',
'UnicodeCharacter0xaa9': '\xe0\xaa\xa9',
'ArabicLigatureTehWithHahWithJeemFinalForm': '\xef\xb5\x91',
'UnicodeCharacter0x2ddf': '\xe2\xb7\x9f',
'CjkUnifiedIdeograph4b3d': '\xe4\xac\xbd',
'CjkUnifiedIdeograph77c3': '\xe7\x9f\x83',
'CjkUnifiedIdeograph4b3f': '\xe4\xac\xbf',
'CjkUnifiedIdeograph4b3a': '\xe4\xac\xba',
'CjkUnifiedIdeograph77c4': '\xe7\x9f\x84',
'CjkUnifiedIdeograph4b3c': '\xe4\xac\xbc',
'CjkUnifiedIdeograph4b3b': '\xe4\xac\xbb',
'CjkUnifiedIdeograph77c9': '\xe7\x9f\x89',
'CjkUnifiedIdeograph77c8': '\xe7\x9f\x88',
'UnicodeCharacter0xeb81': '\xee\xae\x81',
'HangulSyllableSwaenh': '\xec\x87\x8a',
'GurmukhiEkOnkar': '\xe0\xa9\xb4',
'HangulSyllableSwaenj': '\xec\x87\x89',
'UnicodeCharacter0xe64d': '\xee\x99\x8d',
'HangulSyllableSyibs': '\xec\x8b\x92',
'CjkRadicalBlue': '\xe2\xbb\x98',
'GreaterthanAndNotApproximate': '\xe2\xaa\x8a',
'GujaratiLetterYa': '\xe0\xaa\xaf',
'HebrewLetterKafWithDagesh': '\xef\xac\xbb',
'HangulLetterKapyeounmieum': '\xe3\x85\xb1',
'UnicodeCharacter0xe162': '\xee\x85\xa2',
'LepchaDigitEight': '\xe1\xb1\x88',
'UnicodeCharacter0xf64a': '\xef\x99\x8a',
'UnicodeCharacter0xf64b': '\xef\x99\x8b',
'UnicodeCharacter0xf64c': '\xef\x99\x8c',
'UnicodeCharacter0xf64d': '\xef\x99\x8d',
'UnicodeCharacter0xf64e': '\xef\x99\x8e',
'UnicodeCharacter0xf64f': '\xef\x99\x8f',
'TibetanMarkInitialBrdaRnyingYigMgoMdunMa': '\xe0\xbf\x93',
'HangulSyllableTyass': '\xed\x84\x8c',
'CjkUnifiedIdeograph90d9': '\xe9\x83\x99',
'CjkUnifiedIdeograph90d8': '\xe9\x83\x98',
'HangulSyllableSsyeols': '\xec\x8e\xac',
'CjkUnifiedIdeograph90d5': '\xe9\x83\x95',
'CjkUnifiedIdeograph90d4': '\xe9\x83\x94',
'CjkUnifiedIdeograph90d7': '\xe9\x83\x97',
'CjkUnifiedIdeograph90d6': '\xe9\x83\x96',
'CjkUnifiedIdeograph90d1': '\xe9\x83\x91',
'CjkUnifiedIdeograph90d0': '\xe9\x83\x90',
'CjkUnifiedIdeograph90d3': '\xe9\x83\x93',
'CjkUnifiedIdeograph90d2': '\xe9\x83\x92',
'CjkUnifiedIdeograph39a9': '\xe3\xa6\xa9',
'CjkUnifiedIdeograph39a8': '\xe3\xa6\xa8',
'CjkUnifiedIdeograph9a1a': '\xe9\xa8\x9a',
'CjkUnifiedIdeograph9a1f': '\xe9\xa8\x9f',
'CjkUnifiedIdeograph9a1e': '\xe9\xa8\x9e',
'CjkUnifiedIdeograph9a1d': '\xe9\xa8\x9d',
'CjkUnifiedIdeograph39a1': '\xe3\xa6\xa1',
'UpArrowThroughCircle': '\xe2\xa6\xbd',
'CjkUnifiedIdeograph39a3': '\xe3\xa6\xa3',
'CjkUnifiedIdeograph39a2': '\xe3\xa6\xa2',
'CjkUnifiedIdeograph39a5': '\xe3\xa6\xa5',
'CjkUnifiedIdeograph39a4': '\xe3\xa6\xa4',
'CjkUnifiedIdeograph39a7': '\xe3\xa6\xa7',
'CjkUnifiedIdeograph39a6': '\xe3\xa6\xa6',
'HangulSyllablePam': '\xed\x8c\x9c',
'UnicodeCharacter0xf1a6': '\xef\x86\xa6',
'HangulSyllableHyugg': '\xed\x9c\xb6',
'BuhidLetterKa': '\xe1\x9d\x83',
'CjkUnifiedIdeograph90de': '\xe9\x83\x9e',
'CjkUnifiedIdeograph90dd': '\xe9\x83\x9d',
'CjkUnifiedIdeograph90df': '\xe9\x83\x9f',
'CjkUnifiedIdeograph90da': '\xe9\x83\x9a',
'CjkUnifiedIdeograph90dc': '\xe9\x83\x9c',
'CjkUnifiedIdeograph90db': '\xe9\x83\x9b',
'UnicodeCharacter0xf640': '\xef\x99\x80',
'UnicodeCharacter0xf641': '\xef\x99\x81',
'UnicodeCharacter0xf642': '\xef\x99\x82',
'UnicodeCharacter0xf643': '\xef\x99\x83',
'UnicodeCharacter0xf644': '\xef\x99\x84',
'UnicodeCharacter0xf645': '\xef\x99\x85',
'HangulSyllableHyugs': '\xed\x9c\xb7',
'UnicodeCharacter0xf647': '\xef\x99\x87',
'HangulSyllableTyelt': '\xed\x86\x91',
'HangulSyllableDdwaep': '\xeb\x99\xa2',
'RunicLetterShorttwigmadrM': '\xe1\x9b\x99',
'CjkUnifiedIdeograph8abd': '\xe8\xaa\xbd',
'HexagramForGreatPower': '\xe4\xb7\xa1',
'CjkCompatibilityIdeographf9d9': '\xef\xa7\x99',
'KangxiRadicalWest': '\xe2\xbe\x91',
'TaiLeLetterUe': '\xe1\xa5\xaa',
'CjkUnifiedIdeograph9a12': '\xe9\xa8\x92',
'CjkUnifiedIdeograph9a11': '\xe9\xa8\x91',
'CjkUnifiedIdeograph804a': '\xe8\x81\x8a',
'CjkUnifiedIdeograph9a17': '\xe9\xa8\x97',
'CjkUnifiedIdeograph9a16': '\xe9\xa8\x96',
'CjkUnifiedIdeograph9a15': '\xe9\xa8\x95',
'CjkUnifiedIdeograph9a14': '\xe9\xa8\x94',
'CjkUnifiedIdeograph39aa': '\xe3\xa6\xaa',
'CjkUnifiedIdeograph39ac': '\xe3\xa6\xac',
'CjkUnifiedIdeograph39ab': '\xe3\xa6\xab',
'MongolianLetterSha': '\xe1\xa0\xb1',
'CjkUnifiedIdeograph39ad': '\xe3\xa6\xad',
'CjkUnifiedIdeograph39af': '\xe3\xa6\xaf',
'UnicodeCharacter0xd9b7': '\xed\xa6\xb7',
'HangulSyllableNwas': '\xeb\x86\xa7',
'UnicodeCharacter0xd9b5': '\xed\xa6\xb5',
'UnicodeCharacter0xd9b4': '\xed\xa6\xb4',
'UnicodeCharacter0xd9b3': '\xed\xa6\xb3',
'CjkUnifiedIdeograph804d': '\xe8\x81\x8d',
'UnicodeCharacter0xd9b1': '\xed\xa6\xb1',
'HangulSyllableCyolh': '\xec\xb6\x87',
'CjkUnifiedIdeograph804e': '\xe8\x81\x8e',
'UnicodeCharacter0xd9b9': '\xed\xa6\xb9',
'UnicodeCharacter0xd9b8': '\xed\xa6\xb8',
'CombiningOgonek': '\xcc\xa8',
'HangulSyllableDdeoss': '\xeb\x96\xb4',
'HangulSyllableKwae': '\xec\xbe\x8c',
'CjkUnifiedIdeograph4ad6': '\xe4\xab\x96',
'CjkUnifiedIdeograph548a': '\xe5\x92\x8a',
'HangulSyllableCyolm': '\xec\xb6\x82',
'CjkUnifiedIdeograph548c': '\xe5\x92\x8c',
'Paragraphos': '\xe2\xb8\x8f',
'LatinCapitalLetterKWithDescender': '\xe2\xb1\xa9',
'CjkUnifiedIdeograph548d': '\xe5\x92\x8d',
'ArmenianCapitalLetterHo': '\xd5\x80',
'CopticSmallLetterIauda': '\xe2\xb2\x93',
'BoxDrawingsVerticalDoubleAndHorizontalSingle': '\xe2\x95\xab',
'HangulSyllableTaet': '\xed\x83\xb5',
'UnicodeCharacter0xe168': '\xee\x85\xa8',
'CjkUnifiedIdeograph5489': '\xe5\x92\x89',
'CjkUnifiedIdeograph5488': '\xe5\x92\x88',
'CjkUnifiedIdeograph68ba': '\xe6\xa2\xba',
'CjkUnifiedIdeograph5481': '\xe5\x92\x81',
'CjkUnifiedIdeograph5480': '\xe5\x92\x80',
'CjkUnifiedIdeograph5483': '\xe5\x92\x83',
'CjkUnifiedIdeograph5482': '\xe5\x92\x82',
'CjkUnifiedIdeograph5485': '\xe5\x92\x85',
'CjkUnifiedIdeograph5484': '\xe5\x92\x84',
'CjkUnifiedIdeograph5487': '\xe5\x92\x87',
'CjkUnifiedIdeograph5486': '\xe5\x92\x86',
'UnicodeCharacter0xd9bf': '\xed\xa6\xbf',
'UnicodeCharacter0xd9be': '\xed\xa6\xbe',
'UnicodeCharacter0xd9bd': '\xed\xa6\xbd',
'UnicodeCharacter0xd9bc': '\xed\xa6\xbc',
'HangulSyllableCyolg': '\xec\xb6\x81',
'UnicodeCharacter0xd9ba': '\xed\xa6\xba',
'UnicodeCharacter0xe925': '\xee\xa4\xa5',
'ModifierLetterLowRing': '\xcb\xb3',
'HangulSyllableDdwegg': '\xeb\x9b\x96',
'UnicodeCharacter0xe927': '\xee\xa4\xa7',
'HangulJungseongYueo': '\xe1\x86\x8f',
'CjkUnifiedIdeograph68bf': '\xe6\xa2\xbf',
'HangulSyllableNwab': '\xeb\x86\xa5',
'UnicodeCharacter0xda3d': '\xed\xa8\xbd',
'IdeographicNumberZero': '\xe3\x80\x87',
'ModifierLetterHighLeftstemToneBar': '\xea\x9c\x93',
'natexp': '\xe2\x85\x87',
'HangulSyllableDdwegs': '\xeb\x9b\x97',
'HangulSyllableNwag': '\xeb\x86\x95',
'UnicodeCharacter0xddff': '\xed\xb7\xbf',
'BoxDrawingsDownLightAndRightHeavy': '\xe2\x94\x8d',
'HangulSyllableWebs': '\xec\x9b\xba',
'HangulSyllableNwae': '\xeb\x86\xb0',
'UnicodeCharacter0xec6e': '\xee\xb1\xae',
'UnicodeCharacter0xe223': '\xee\x88\xa3',
'ArabicLetterNyehFinalForm': '\xef\xad\xb7',
'HangulSyllableCyolp': '\xec\xb6\x86',
'YiSyllableNdurx': '\xea\x85\xa6',
'UnicodeCharacter0xe221': '\xee\x88\xa1',
'CjkUnifiedIdeograph68bd': '\xe6\xa2\xbd',
'TeluguLetterTa': '\xe0\xb0\xa4',
'NewTaiLueLetterLowXa': '\xe1\xa6\x86',
'GujaratiOm': '\xe0\xab\x90',
'HangulSyllableCyolt': '\xec\xb6\x85',
'FunctionApplication': '\xe2\x81\xa1',
'CjkUnifiedIdeograph910b': '\xe9\x84\x8b',
'CjkUnifiedIdeograph910c': '\xe9\x84\x8c',
'CjkUnifiedIdeograph910d': '\xe9\x84\x8d',
'CjkUnifiedIdeograph910e': '\xe9\x84\x8e',
'CjkUnifiedIdeograph910f': '\xe9\x84\x8f',
'UnicodeCharacter0xe224': '\xee\x88\xa4',
'CjkUnifiedIdeograph68be': '\xe6\xa2\xbe',
'HalfwidthHangulLetterE': '\xef\xbf\x87',
'KhmerSymbolPrambuonRoc': '\xe1\xa7\xb9',
'HalfwidthHangulLetterA': '\xef\xbf\x82',
'UnicodeCharacter0xf088': '\xef\x82\x88',
'HalfwidthHangulLetterO': '\xef\xbf\x8c',
'HalfwidthHangulLetterI': '\xef\xbf\x9c',
'UnicodeCharacter0xe606': '\xee\x98\x86',
'HalfwidthHangulLetterU': '\xef\xbf\x93',
'ArabicLigatureFehWithAlefMaksuraIsolatedForm': '\xef\xb0\xb1',
'CjkCompatibilityIdeographf9d1': '\xef\xa7\x91',
'CjkUnifiedIdeograph9100': '\xe9\x84\x80',
'CjkUnifiedIdeograph9101': '\xe9\x84\x81',
'CjkUnifiedIdeograph9102': '\xe9\x84\x82',
'CjkUnifiedIdeograph9103': '\xe9\x84\x83',
'CjkUnifiedIdeograph9104': '\xe9\x84\x84',
'CjkUnifiedIdeograph9105': '\xe9\x84\x85',
'CjkUnifiedIdeograph9106': '\xe9\x84\x86',
'CjkUnifiedIdeograph9107': '\xe9\x84\x87',
'CjkUnifiedIdeograph9108': '\xe9\x84\x88',
'CjkUnifiedIdeograph9109': '\xe9\x84\x89',
'UnicodeCharacter0xf1a0': '\xef\x86\xa0',
'HangulSyllableJjoec': '\xec\xac\xa7',
'HalfwidthBlackSquare': '\xef\xbf\xad',
'CjkUnifiedIdeograph8597': '\xe8\x96\x97',
'GreekCapitalLetterEtaWithPsiliAndPerispomeni': '\xe1\xbc\xae',
'HangulSyllablePabs': '\xed\x8c\x9e',
'Semisextile': '\xe2\x9a\xba',
'UnicodeCharacter0xeb82': '\xee\xae\x82',
'CjkUnifiedIdeograph8040': '\xe8\x81\x80',
'HangulSyllableReubs': '\xeb\xa6\x86',
'HangulSyllableToenj': '\xed\x87\xb9',
'CjkUnifiedIdeograph98ce': '\xe9\xa3\x8e',
'CjkUnifiedIdeograph8041': '\xe8\x81\x81',
'UnicodeCharacter0xec5c': '\xee\xb1\x9c',
'UnicodeCharacter0xec5b': '\xee\xb1\x9b',
'UnicodeCharacter0xec5a': '\xee\xb1\x9a',
'CjkUnifiedIdeograph927d': '\xe9\x89\xbd',
'UnicodeCharacter0xec5e': '\xee\xb1\x9e',
'LatinCapitalLetterTWithStroke': '\xc5\xa6',
'HangulSyllableGeulb': '\xea\xb8\x83',
'HangulSyllableGeulg': '\xea\xb8\x81',
'UnicodeCharacter0xe603': '\xee\x98\x83',
'HangulSyllableGeulh': '\xea\xb8\x87',
'HangulSyllableGeulm': '\xea\xb8\x82',
'CjkUnifiedIdeograph927c': '\xe9\x89\xbc',
'HangulSyllableGeuls': '\xea\xb8\x84',
'CircledIdeographThree': '\xe3\x8a\x82',
'HangulSyllableGeulp': '\xea\xb8\x86',
'HangulSyllableGeult': '\xea\xb8\x85',
'CjkUnifiedIdeograph76ff': '\xe7\x9b\xbf',
'CjkUnifiedIdeograph76fe': '\xe7\x9b\xbe',
'CjkUnifiedIdeograph76fd': '\xe7\x9b\xbd',
'CjkUnifiedIdeograph76fc': '\xe7\x9b\xbc',
'CjkUnifiedIdeograph76fb': '\xe7\x9b\xbb',
'CjkUnifiedIdeograph76fa': '\xe7\x9b\xba',
'CjkUnifiedIdeograph84b0': '\xe8\x92\xb0',
'UnicodeCharacter0xec59': '\xee\xb1\x99',
'CjkUnifiedIdeograph8049': '\xe8\x81\x89',
'CjkUnifiedIdeograph84b1': '\xe8\x92\xb1',
'UnicodeCharacter0xec53': '\xee\xb1\x93',
'UnicodeCharacter0xec52': '\xee\xb1\x92',
'UnicodeCharacter0xec51': '\xee\xb1\x91',
'UnicodeCharacter0xec50': '\xee\xb1\x90',
'UnicodeCharacter0xec57': '\xee\xb1\x97',
'LatinCapitalLetterTWithDotAbove': '\xe1\xb9\xaa',
'UnicodeCharacter0xec55': '\xee\xb1\x95',
'UnicodeCharacter0xec54': '\xee\xb1\x94',
'SmallReverseSolidus': '\xef\xb9\xa8',
'CjkCompatibilityIdeographf9d5': '\xef\xa7\x95',
'UnicodeCharacter0xead6': '\xee\xab\x96',
'CjkUnifiedIdeograph84b4': '\xe8\x92\xb4',
'GreekCapitalLetterUpsilonWithMacron': '\xe1\xbf\xa9',
'CjkUnifiedIdeograph84b5': '\xe8\x92\xb5',
'HangulSyllableMeulg': '\xeb\xaf\x89',
'SmallRomanNumeralOne': '\xe2\x85\xb0',
'CjkUnifiedIdeograph76f8': '\xe7\x9b\xb8',
'CjkUnifiedIdeograph76f7': '\xe7\x9b\xb7',
'CjkUnifiedIdeograph76f6': '\xe7\x9b\xb6',
'CjkUnifiedIdeograph76f5': '\xe7\x9b\xb5',
'CjkUnifiedIdeograph76f4': '\xe7\x9b\xb4',
'CjkUnifiedIdeograph76f3': '\xe7\x9b\xb3',
'CjkUnifiedIdeograph76f2': '\xe7\x9b\xb2',
'CjkUnifiedIdeograph76f1': '\xe7\x9b\xb1',
'CjkUnifiedIdeograph76f0': '\xe7\x9b\xb0',
'DoesNotDivide': '\xe2\x88\xa4',
'HangulSyllableMeult': '\xeb\xaf\x8d',
'CjkCompatibilityIdeographf9d4': '\xef\xa7\x94',
'HangulSyllableBbyeols': '\xeb\xbc\x94',
'HangulSyllableMeulp': '\xeb\xaf\x8e',
'HangulSyllableMeuls': '\xeb\xaf\x8c',
'CjkUnifiedIdeograph84b9': '\xe8\x92\xb9',
'CjkUnifiedIdeograph9828': '\xe9\xa0\xa8',
'CjkUnifiedIdeograph591b': '\xe5\xa4\x9b',
'HangulChoseongKapyeounmieum': '\xe1\x84\x9d',
'CjkCompatibilityIdeographfa40': '\xef\xa9\x80',
'UnicodeCharacter0xf1cc': '\xef\x87\x8c',
'UnicodeCharacter0x8f3': '\xe0\xa3\xb3',
'CjkCompatibilityIdeographfa43': '\xef\xa9\x83',
'HangulSyllablePyubs': '\xed\x93\xba',
'ArabicindicDigitSix': '\xd9\xa6',
'HangulSyllableMyaegs': '\xeb\xa8\x9f',
'HangulSyllableBbyej': '\xeb\xbc\xba',
'CjkCompatibilityIdeographfa45': '\xef\xa9\x85',
'UnicodeCharacter0xa6b3': '\xea\x9a\xb3',
'CjkCompatibilityIdeographfa44': '\xef\xa9\x84',
'SquarePointo': '\xe3\x8c\xbd',
'CjkUnifiedIdeograph5308': '\xe5\x8c\x88',
'CjkUnifiedIdeograph5309': '\xe5\x8c\x89',
'CjkUnifiedIdeograph5306': '\xe5\x8c\x86',
'CjkUnifiedIdeograph5307': '\xe5\x8c\x87',
'CjkUnifiedIdeograph5304': '\xe5\x8c\x84',
'CjkUnifiedIdeograph5305': '\xe5\x8c\x85',
'CjkUnifiedIdeograph5302': '\xe5\x8c\x82',
'CjkUnifiedIdeograph5303': '\xe5\x8c\x83',
'CjkUnifiedIdeograph5300': '\xe5\x8c\x80',
'CjkUnifiedIdeograph5301': '\xe5\x8c\x81',
'KangxiRadicalShortThread': '\xe2\xbc\xb3',
'CjkUnifiedIdeograph6c26': '\xe6\xb0\xa6',
'CjkUnifiedIdeograph9818': '\xe9\xa0\x98',
'CjkUnifiedIdeograph9819': '\xe9\xa0\x99',
'CjkUnifiedIdeograph9816': '\xe9\xa0\x96',
'CjkUnifiedIdeograph9817': '\xe9\xa0\x97',
'CjkUnifiedIdeograph9814': '\xe9\xa0\x94',
'CjkUnifiedIdeograph747e': '\xe7\x91\xbe',
'CjkUnifiedIdeograph9812': '\xe9\xa0\x92',
'CjkUnifiedIdeograph9813': '\xe9\xa0\x93',
'BengaliLetterNya': '\xe0\xa6\x9e',
'CjkUnifiedIdeograph9811': '\xe9\xa0\x91',
'HangulSyllableYags': '\xec\x95\xbf',
'UnicodeCharacter0xf796': '\xef\x9e\x96',
'UnicodeCharacter0xf1ca': '\xef\x87\x8a',
'GurmukhiVowelSignOo': '\xe0\xa9\x8b',
'KangxiRadicalLongStride': '\xe2\xbc\xb5',
'CjkUnifiedIdeograph6c22': '\xe6\xb0\xa2',
'HangulSyllableYagg': '\xec\x95\xbe',
'LatinLetterBilabialPercussive': '\xca\xac',
'UnicodeCharacter0xe609': '\xee\x98\x89',
'YiRadicalPyt': '\xea\x92\xac',
'UnicodeCharacter0xf797': '\xef\x9e\x97',
'CjkUnifiedIdeograph673f': '\xe6\x9c\xbf',
'DevanagariLetterRha': '\xe0\xa5\x9d',
'Dagger': '\xe2\x80\xa0',
'HangulSyllableKwalt': '\xec\xbd\xbd',
'CjkUnifiedIdeograph981f': '\xe9\xa0\x9f',
'CjkUnifiedIdeograph981d': '\xe9\xa0\x9d',
'CjkUnifiedIdeograph981e': '\xe9\xa0\x9e',
'CjkUnifiedIdeograph981b': '\xe9\xa0\x9b',
'CjkUnifiedIdeograph981c': '\xe9\xa0\x9c',
'LatinSmallLetterScriptG': '\xc9\xa1',
'CjkUnifiedIdeograph981a': '\xe9\xa0\x9a',
'CombiningInvertedBridgeBelow': '\xcc\xba',
'HangulSyllableBbyem': '\xeb\xbc\xb4',
'CjkUnifiedIdeograph673e': '\xe6\x9c\xbe',
'HangulSyllableToels': '\xed\x88\x80',
'HangulSyllableKwalg': '\xec\xbd\xb9',
'CjkUnifiedIdeograph530f': '\xe5\x8c\x8f',
'CjkUnifiedIdeograph673b': '\xe6\x9c\xbb',
'CjkUnifiedIdeograph530d': '\xe5\x8c\x8d',
'CjkUnifiedIdeograph530e': '\xe5\x8c\x8e',
'CjkUnifiedIdeograph530b': '\xe5\x8c\x8b',
'OcrAmountOfCheck': '\xe2\x91\x87',
'HangulSyllableDaenj': '\xeb\x8c\x85',
'CjkUnifiedIdeograph530a': '\xe5\x8c\x8a',
'SinhalaLetterAeeyanna': '\xe0\xb6\x88',
'ParenthesizedHangulCieucU': '\xe3\x88\x9c',
'HangulSyllableOlt': '\xec\x98\xb1',
'CjkUnifiedIdeograph6769': '\xe6\x9d\xa9',
'CjkUnifiedIdeograph6768': '\xe6\x9d\xa8',
'CjkUnifiedIdeograph6765': '\xe6\x9d\xa5',
'CjkUnifiedIdeograph6764': '\xe6\x9d\xa4',
'CjkUnifiedIdeograph6767': '\xe6\x9d\xa7',
'CjkUnifiedIdeograph6766': '\xe6\x9d\xa6',
'CjkUnifiedIdeograph6761': '\xe6\x9d\xa1',
'CjkUnifiedIdeograph6760': '\xe6\x9d\xa0',
'CjkUnifiedIdeograph6763': '\xe6\x9d\xa3',
'CjkUnifiedIdeograph6762': '\xe6\x9d\xa2',
'HangulSyllableBbyen': '\xeb\xbc\xa8',
'CjkUnifiedIdeograph494f': '\xe4\xa5\x8f',
'CjkUnifiedIdeograph494e': '\xe4\xa5\x8e',
'CjkUnifiedIdeograph494d': '\xe4\xa5\x8d',
'CjkUnifiedIdeograph494c': '\xe4\xa5\x8c',
'CjkUnifiedIdeograph494b': '\xe4\xa5\x8b',
'ParenthesizedHangulCieucA': '\xe3\x88\x96',
'UnicodeCharacter0xf795': '\xef\x9e\x95',
'HangulSyllableHyaegs': '\xed\x96\xaf',
'HangulSyllableWaels': '\xec\x99\xa8',
'ModifierLetterCentredLeftHalfRing': '\xcb\x93',
'UnicodeCharacter0xe585': '\xee\x96\x85',
'CyrillicCapitalLetterBlendedYus': '\xea\x99\x9a',
'SylotiNagriLetterDo': '\xea\xa0\x96',
'LatinCapitalLetterEWithInvertedBreve': '\xc8\x86',
'LatinSmallLetterAWithInvertedBreve': '\xc8\x83',
'UnicodeCharacter0xdedb': '\xed\xbb\x9b',
'CjkUnifiedIdeograph7e38': '\xe7\xb8\xb8',
'UnicodeCharacter0xf792': '\xef\x9e\x92',
'CanadianSyllabicsFinalTopHalfRing': '\xe1\x90\xa2',
'CjkUnifiedIdeograph4947': '\xe4\xa5\x87',
'ThaiCharacterSaraUe': '\xe0\xb8\xb6',
'CjkUnifiedIdeograph4945': '\xe4\xa5\x85',
'CjkUnifiedIdeograph4944': '\xe4\xa5\x84',
'CjkUnifiedIdeograph4943': '\xe4\xa5\x83',
'CjkUnifiedIdeograph4942': '\xe4\xa5\x82',
'CjkUnifiedIdeograph4941': '\xe4\xa5\x81',
'CjkUnifiedIdeograph4940': '\xe4\xa5\x80',
'HangulSyllableJeos': '\xec\xa0\x93',
'SmallRightCurlyBracket': '\xef\xb9\x9c',
'BoxDrawingsUpSingleAndHorizontalDouble': '\xe2\x95\xa7',
'CjkUnifiedIdeograph4949': '\xe4\xa5\x89',
'CjkUnifiedIdeograph4948': '\xe4\xa5\x88',
'BoxDrawingsDownHeavyAndRightLight': '\xe2\x94\x8e',
'CjkUnifiedIdeograph8ebd': '\xe8\xba\xbd',
'CjkUnifiedIdeograph676e': '\xe6\x9d\xae',
'CjkUnifiedIdeograph676d': '\xe6\x9d\xad',
'LatinCapitalLetterKWithDotBelow': '\xe1\xb8\xb2',
'CjkUnifiedIdeograph676f': '\xe6\x9d\xaf',
'CjkUnifiedIdeograph676a': '\xe6\x9d\xaa',
'CjkUnifiedIdeograph8a4a': '\xe8\xa9\x8a',
'CjkUnifiedIdeograph676c': '\xe6\x9d\xac',
'CjkUnifiedIdeograph676b': '\xe6\x9d\xab',
'YiSyllableGgiep': '\xea\x88\x99',
'HangulSyllableEnh': '\xec\x97\x96',
'YiSyllableGgiex': '\xea\x88\x97',
'ArabicLetterThal': '\xd8\xb0',
'UnicodeCharacter0xf046': '\xef\x81\x86',
'UnicodeCharacter0xf790': '\xef\x9e\x90',
'HangulSyllableMweogs': '\xeb\xad\x93',
'HangulSyllableWaelp': '\xec\x99\xaa',
'UnicodeCharacter0xe067': '\xee\x81\xa7',
'UnicodeCharacter0xf044': '\xef\x81\x84',
'MalayalamLetterRa': '\xe0\xb4\xb0',
'HangulSyllableRwat': '\xeb\xa2\x91',
'HangulSyllableTeoss': '\xed\x85\x84',
'UnicodeCharacter0x18cf': '\xe1\xa3\x8f',
'UnicodeCharacter0xe4b1': '\xee\x92\xb1',
'UnicodeCharacter0x18cd': '\xe1\xa3\x8d',
'UnicodeCharacter0x18ce': '\xe1\xa3\x8e',
'UnicodeCharacter0x18cb': '\xe1\xa3\x8b',
'UnicodeCharacter0x18cc': '\xe1\xa3\x8c',
'MalayalamDigitNine': '\xe0\xb5\xaf',
'UnicodeCharacter0x18ca': '\xe1\xa3\x8a',
'HangulSyllableJeog': '\xec\xa0\x81',
'CjkUnifiedIdeograph8ebf': '\xe8\xba\xbf',
'HangulSyllableDaeng': '\xeb\x8c\x95',
'UnicodeCharacter0xe4b3': '\xee\x92\xb3',
'CjkUnifiedIdeograph68f7': '\xe6\xa3\xb7',
'UnicodeCharacter0xe4b2': '\xee\x92\xb2',
'HangulSyllableGgwegg': '\xea\xbf\xb2',
'CjkUnifiedIdeograph3d76': '\xe3\xb5\xb6',
'CjkUnifiedIdeograph3d77': '\xe3\xb5\xb7',
'CjkUnifiedIdeograph3d74': '\xe3\xb5\xb4',
'CjkUnifiedIdeograph3d75': '\xe3\xb5\xb5',
'CjkUnifiedIdeograph3d72': '\xe3\xb5\xb2',
'CjkUnifiedIdeograph3d73': '\xe3\xb5\xb3',
'CjkUnifiedIdeograph3d70': '\xe3\xb5\xb0',
'CjkUnifiedIdeograph3d71': '\xe3\xb5\xb1',
'Bowtie': '\xe2\x8b\x88',
'CjkUnifiedIdeograph72e3': '\xe7\x8b\xa3',
'CjkUnifiedIdeograph72e0': '\xe7\x8b\xa0',
'CjkUnifiedIdeograph72e1': '\xe7\x8b\xa1',
'CjkUnifiedIdeograph72e6': '\xe7\x8b\xa6',
'CjkUnifiedIdeograph72e7': '\xe7\x8b\xa7',
'CjkUnifiedIdeograph3d78': '\xe3\xb5\xb8',
'CjkUnifiedIdeograph3d79': '\xe3\xb5\xb9',
'CjkUnifiedIdeograph8eba': '\xe8\xba\xba',
'HangulSyllableRwal': '\xeb\xa2\x80',
'LaoVowelSignYy': '\xe0\xba\xb7',
'CjkUnifiedIdeograph7419': '\xe7\x90\x99',
'CjkUnifiedIdeograph7414': '\xe7\x90\x94',
'CjkUnifiedIdeograph7415': '\xe7\x90\x95',
'CjkUnifiedIdeograph7416': '\xe7\x90\x96',
'CjkUnifiedIdeograph7417': '\xe7\x90\x97',
'CjkUnifiedIdeograph7410': '\xe7\x90\x90',
'CjkUnifiedIdeograph7411': '\xe7\x90\x91',
'CjkUnifiedIdeograph7412': '\xe7\x90\x92',
'CjkUnifiedIdeograph7413': '\xe7\x90\x93',
'HangulSyllableBbyed': '\xeb\xbc\xab',
'UnicodeCharacter0xe99e': '\xee\xa6\x9e',
'HangulSyllableRwah': '\xeb\xa2\x93',
'YiSyllablePyp': '\xea\x81\x93',
'CombiningInvertedBreve': '\xcc\x91',
'HangulSyllableRwag': '\xeb\xa1\xb9',
'HangulSyllableWaelm': '\xec\x99\xa6',
'UnicodeCharacter0x18c8': '\xe1\xa3\x88',
'UnicodeCharacter0x18c9': '\xe1\xa3\x89',
'UnicodeCharacter0x18c6': '\xe1\xa3\x86',
'UnicodeCharacter0x18c7': '\xe1\xa3\x87',
'BoxDrawingsLightQuadrupleDashVertical': '\xe2\x94\x8a',
'UnicodeCharacter0x18c5': '\xe1\xa3\x85',
'UnicodeCharacter0x18c2': '\xe1\xa3\x82',
'UnicodeCharacter0x18c3': '\xe1\xa3\x83',
'UnicodeCharacter0x18c0': '\xe1\xa3\x80',
'UnicodeCharacter0x18c1': '\xe1\xa3\x81',
'HangulSyllableSsyej': '\xec\x8f\x92',
'UnicodeCharacter0xf59c': '\xef\x96\x9c',
'LatinLetterBilabialClick': '\xca\x98',
'CjkUnifiedIdeograph741e': '\xe7\x90\x9e',
'CjkUnifiedIdeograph741f': '\xe7\x90\x9f',
'CjkUnifiedIdeograph7e32': '\xe7\xb8\xb2',
'CjkUnifiedIdeograph741a': '\xe7\x90\x9a',
'CjkUnifiedIdeograph741b': '\xe7\x90\x9b',
'CjkUnifiedIdeograph741c': '\xe7\x90\x9c',
'CjkUnifiedIdeograph3d7f': '\xe3\xb5\xbf',
'CjkUnifiedIdeograph3d7d': '\xe3\xb5\xbd',
'CjkUnifiedIdeograph3d7e': '\xe3\xb5\xbe',
'CjkUnifiedIdeograph3d7b': '\xe3\xb5\xbb',
'CjkUnifiedIdeograph3d7c': '\xe3\xb5\xbc',
'HangulSyllableRwaem': '\xeb\xa2\xa4',
'CjkUnifiedIdeograph3d7a': '\xe3\xb5\xba',
'CjkUnifiedIdeograph72eb': '\xe7\x8b\xab',
'CjkUnifiedIdeograph72ec': '\xe7\x8b\xac',
'HangulSyllableBbyilg': '\xec\x81\xbd',
'CjkUnifiedIdeograph72ea': '\xe7\x8b\xaa',
'CjkUnifiedIdeograph72ef': '\xe7\x8b\xaf',
'CjkUnifiedIdeograph72ed': '\xe7\x8b\xad',
'CjkUnifiedIdeograph72ee': '\xe7\x8b\xae',
'UnicodeCharacter0xee8f': '\xee\xba\x8f',
'UnicodeCharacter0xa7ac': '\xea\x9e\xac',
'YiSyllablePyr': '\xea\x81\x95',
'CjkUnifiedIdeograph8ebb': '\xe8\xba\xbb',
'UnicodeCharacter0xee8d': '\xee\xba\x8d',
'YiSyllableHnex': '\xea\x85\xb9',
'CanadianSyllabicsSayisiHa': '\xe1\x97\x83',
'CanadianSyllabicsSayisiHe': '\xe1\x97\x80',
'EthiopicSyllableMya': '\xe1\x8d\x99',
'CanadianSyllabicsSayisiHi': '\xe1\x97\x81',
'CanadianSyllabicsSayisiHo': '\xe1\x97\x82',
'HangulSyllableBbyeg': '\xeb\xbc\xa5',
'UnicodeCharacter0xee8c': '\xee\xba\x8c',
'HangulSyllableKols': '\xec\xbd\xa0',
'HangulSyllableDdyolb': '\xeb\x9a\x8b',
'HangulSyllableKolp': '\xec\xbd\xa2',
'HangulSyllableNigs': '\xeb\x8b\x8b',
'RunicLetterOn': '\xe1\x9a\xb0',
'UnicodeCharacter0xa7ab': '\xea\x9e\xab',
'HangulSyllableKolt': '\xec\xbd\xa1',
'HangulSyllableCwalb': '\xec\xb4\xaf',
'RunicLetterOe': '\xe1\x9a\xaf',
'HangulSyllableNigg': '\xeb\x8b\x8a',
'HangulSyllableKolb': '\xec\xbd\x9f',
'UnicodeCharacter0xdad7': '\xed\xab\x97',
'HangulSyllableKolg': '\xec\xbd\x9d',
'SquareGirudaa': '\xe3\x8c\x93',
'HangulSyllableCwalp': '\xec\xb4\xb2',
'HangulSyllableCwals': '\xec\xb4\xb0',
'HangulSyllableKolh': '\xec\xbd\xa3',
'HangulSyllableKeut': '\xed\x82\x85',
'HangulSyllableCwalt': '\xec\xb4\xb1',
'HangulSyllableKolm': '\xec\xbd\x9e',
'UnicodeCharacter0xeafd': '\xee\xab\xbd',
'TaiLeLetterAue': '\xe1\xa5\xac',
'YiSyllableRyr': '\xea\x8f\x9f',
'SymbolForDeviceControlTwo': '\xe2\x90\x92',
'YiSyllableRyp': '\xea\x8f\x9d',
'YiSyllableRyt': '\xea\x8f\x9a',
'YiSyllableRyx': '\xea\x8f\x9b',
'ArabicLetterVeFinalForm': '\xef\xaf\x9f',
'CjkCompatibilityIdeographfa4a': '\xef\xa9\x8a',
'UnicodeCharacter0xf798': '\xef\x9e\x98',
'CanadianSyllabicsNgaai': '\xe1\x96\x8e',
'CjkUnifiedIdeograph5ab1': '\xe5\xaa\xb1',
'CjkCompatibilityIdeographfa4c': '\xef\xa9\x8c',
'CjkCompatibilityIdeographfa4b': '\xef\xa9\x8b',
'YiSyllableNgiep': '\xea\x89\x9f',
'CjkCompatibilityIdeographfa4d': '\xef\xa9\x8d',
'OlChikiLetterOtt': '\xe1\xb1\xb4',
'YiSyllableHniep': '\xea\x85\xaf',
'HangulSyllableCelb': '\xec\xb2\xbf',
'YiSyllableHniet': '\xea\x85\xac',
'HangulSyllableCelg': '\xec\xb2\xbd',
'UnicodeCharacter0x9fef': '\xe9\xbf\xaf',
'YiSyllableHniex': '\xea\x85\xad',
'UnicodeCharacter0x9fed': '\xe9\xbf\xad',
'UnicodeCharacter0x9fee': '\xe9\xbf\xae',
'UnicodeCharacter0x9feb': '\xe9\xbf\xab',
'UnicodeCharacter0x9fec': '\xe9\xbf\xac',
'UnicodeCharacter0x9fea': '\xe9\xbf\xaa',
'HangulSyllableCelp': '\xec\xb3\x82',
'CopticCapitalLetterOldCopticEsh': '\xe2\xb3\x86',
'CjkUnifiedIdeograph5f39': '\xe5\xbc\xb9',
'HangulSyllableCelt': '\xec\xb3\x81',
'MathematicalRightFlattenedParenthesis': '\xe2\x9f\xaf',
'CjkUnifiedIdeograph5f33': '\xe5\xbc\xb3',
'CjkUnifiedIdeograph5f30': '\xe5\xbc\xb0',
'CjkUnifiedIdeograph5f31': '\xe5\xbc\xb1',
'CjkUnifiedIdeograph5f36': '\xe5\xbc\xb6',
'CjkUnifiedIdeograph5f37': '\xe5\xbc\xb7',
'CjkUnifiedIdeograph5f34': '\xe5\xbc\xb4',
'CjkUnifiedIdeograph5f35': '\xe5\xbc\xb5',
'CjkUnifiedIdeograph54c1': '\xe5\x93\x81',
'YiSyllableLix': '\xea\x86\xb8',
'EthiopicSyllableKxwi': '\xe1\x8b\x82',
'EthiopicSyllableKxwe': '\xe1\x8b\x85',
'EthiopicSyllableKxwa': '\xe1\x8b\x80',
'CjkUnifiedIdeograph34d9': '\xe3\x93\x99',
'CjkUnifiedIdeograph34d8': '\xe3\x93\x98',
'CjkUnifiedIdeograph34d7': '\xe3\x93\x97',
'CjkUnifiedIdeograph34d6': '\xe3\x93\x96',
'CjkUnifiedIdeograph34d5': '\xe3\x93\x95',
'CjkUnifiedIdeograph34d4': '\xe3\x93\x94',
'CjkUnifiedIdeograph34d3': '\xe3\x93\x93',
'CjkUnifiedIdeograph34d2': '\xe3\x93\x92',
'CjkUnifiedIdeograph34d1': '\xe3\x93\x91',
'CjkUnifiedIdeograph34d0': '\xe3\x93\x90',
'HangulSyllableNeonj': '\xeb\x84\x8d',
'HangulSyllableJjyagg': '\xec\xa8\x96',
'HangulChoseongSioskhieukh': '\xe1\x84\xb8',
'HangulSyllableBbeng': '\xeb\xbc\x81',
'CombiningLatinSmallLetterZ': '\xe1\xb7\xa6',
'CombiningLatinSmallLetterU': '\xcd\xa7',
'CombiningLatinSmallLetterT': '\xcd\xad',
'HangulSyllableRyinj': '\xeb\xa6\x95',
'CombiningLatinSmallLetterV': '\xcd\xae',
'CjkUnifiedIdeograph5f3f': '\xe5\xbc\xbf',
'c': '\xcf\x87',
'TibetanSignMarTshes': '\xe0\xbc\xbf',
'CombiningLatinSmallLetterR': '\xcd\xac',
'CombiningLatinSmallLetterM': '\xcd\xab',
'CombiningLatinSmallLetterL': '\xe1\xb7\x9d',
'CombiningLatinSmallLetterO': '\xcd\xa6',
'CombiningLatinSmallLetterN': '\xe1\xb7\xa0',
'CombiningLatinSmallLetterI': '\xcd\xa5',
'CombiningLatinSmallLetterH': '\xcd\xaa',
'CombiningLatinSmallLetterK': '\xe1\xb7\x9c',
'UnicodeCharacter0x9fe9': '\xe9\xbf\xa9',
'CombiningLatinSmallLetterE': '\xcd\xa4',
'CombiningLatinSmallLetterD': '\xcd\xa9',
'CombiningLatinSmallLetterG': '\xe1\xb7\x9a',
'UnicodeCharacter0x9fe5': '\xe9\xbf\xa5',
'CombiningLatinSmallLetterA': '\xcd\xa3',
'UnicodeCharacter0x9fe3': '\xe9\xbf\xa3',
'CombiningLatinSmallLetterC': '\xcd\xa8',
'UnicodeCharacter0x9fe1': '\xe9\xbf\xa1',
'UnicodeCharacter0xf38a': '\xef\x8e\x8a',
'HangulSyllableDdwaess': '\xeb\x99\x9c',
'CjkUnifiedIdeograph34df': '\xe3\x93\x9f',
'CjkUnifiedIdeograph34de': '\xe3\x93\x9e',
'CjkUnifiedIdeograph34dd': '\xe3\x93\x9d',
'CjkUnifiedIdeograph34dc': '\xe3\x93\x9c',
'CjkUnifiedIdeograph34db': '\xe3\x93\x9b',
'CjkUnifiedIdeograph34da': '\xe3\x93\x9a',
'UnicodeCharacter0xf38b': '\xef\x8e\x8b',
'CjkUnifiedIdeograph8284': '\xe8\x8a\x84',
'UnicodeCharacter0xf38e': '\xef\x8e\x8e',
'SymbolForLineFeed': '\xe2\x90\x8a',
'UnicodeCharacter0xf592': '\xef\x96\x92',
'HangulSyllableHweogg': '\xed\x9b\xa2',
'UnicodeCharacter0xf38d': '\xef\x8e\x8d',
'CjkUnifiedIdeograph6bfd': '\xe6\xaf\xbd',
'CjkUnifiedIdeograph6bfe': '\xe6\xaf\xbe',
'CjkUnifiedIdeograph6bff': '\xe6\xaf\xbf',
'CanadianSyllabicsCarrierTtsa': '\xe1\x99\xac',
'CjkUnifiedIdeograph6bfa': '\xe6\xaf\xba',
'CjkUnifiedIdeograph6bfb': '\xe6\xaf\xbb',
'CanadianSyllabicsCarrierTtse': '\xe1\x99\xa9',
'UnicodeCharacter0xe9bb': '\xee\xa6\xbb',
'ThaiCharacterSoSala': '\xe0\xb8\xa8',
'CanadianSyllabicsCarrierTtso': '\xe1\x99\xa8',
'UnicodeCharacter0xf1d3': '\xef\x87\x93',
'UnicodeCharacter0xe53e': '\xee\x94\xbe',
'CanadianSyllabicsCarrierTtsu': '\xe1\x99\xa7',
'Yot': '\xcf\xb3',
'HangulSyllableGwalg': '\xea\xb4\x85',
'HangulSyllableGwalb': '\xea\xb4\x87',
'HangulSyllableGwalm': '\xea\xb4\x86',
'HangulSyllableGwalh': '\xea\xb4\x8b',
'HangulSyllableGwalt': '\xea\xb4\x89',
'UnicodeCharacter0xe53d': '\xee\x94\xbd',
'HangulSyllableGwalp': '\xea\xb4\x8a',
'HangulSyllableGwals': '\xea\xb4\x88',
'RunicLetterGeboGyfuG': '\xe1\x9a\xb7',
'UnicodeCharacter0xfb0a': '\xef\xac\x8a',
'UnicodeCharacter0xe060': '\xee\x81\xa0',
'CjkUnifiedIdeograph9cf4': '\xe9\xb3\xb4',
'HangulSyllableDyus': '\xeb\x93\x93',
'HangulSyllableDyup': '\xeb\x93\x9a',
'CjkUnifiedIdeograph9cf6': '\xe9\xb3\xb6',
'HangulSyllableDyut': '\xeb\x93\x99',
'CjkUnifiedIdeograph6bf4': '\xe6\xaf\xb4',
'CjkUnifiedIdeograph6bf5': '\xe6\xaf\xb5',
'CjkUnifiedIdeograph6bf6': '\xe6\xaf\xb6',
'OriyaLetterRha': '\xe0\xad\x9d',
'CjkUnifiedIdeograph6bf0': '\xe6\xaf\xb0',
'CjkUnifiedIdeograph6bf1': '\xe6\xaf\xb1',
'CjkUnifiedIdeograph6bf2': '\xe6\xaf\xb2',
'CjkUnifiedIdeograph6bf3': '\xe6\xaf\xb3',
'HangulSyllableDyuc': '\xeb\x93\x97',
'UnicodeCharacter0x87b': '\xe0\xa1\xbb',
'CjkUnifiedIdeograph6bf8': '\xe6\xaf\xb8',
'CjkUnifiedIdeograph6bf9': '\xe6\xaf\xb9',
'CjkUnifiedIdeograph9cf1': '\xe9\xb3\xb1',
'UnicodeCharacter0xe999': '\xee\xa6\x99',
'UnicodeCharacter0xf1f5': '\xef\x87\xb5',
'CombiningLeftRightArrowBelow': '\xcd\x8d',
'UnicodeCharacter0xf04b': '\xef\x81\x8b',
'CjkUnifiedIdeograph9cf3': '\xe9\xb3\xb3',
'HangulSyllableJeong': '\xec\xa0\x95',
'UnicodeCharacter0xe53b': '\xee\x94\xbb',
'YiSyllableShox': '\xea\x8e\xb3',
'UnicodeCharacter0xe91b': '\xee\xa4\x9b',
'UnicodeCharacter0xe8a4': '\xee\xa2\xa4',
'UnicodeCharacter0xe2b1': '\xee\x8a\xb1',
'BlackChessPawn': '\xe2\x99\x9f',
'CjkUnifiedIdeograph8239': '\xe8\x88\xb9',
'CjkUnifiedIdeograph8238': '\xe8\x88\xb8',
'UnicodeCharacter0xe341': '\xee\x8d\x81',
'OriyaDigitSeven': '\xe0\xad\xad',
'CjkUnifiedIdeograph8231': '\xe8\x88\xb1',
'CjkUnifiedIdeograph8230': '\xe8\x88\xb0',
'CjkUnifiedIdeograph8233': '\xe8\x88\xb3',
'CjkUnifiedIdeograph8232': '\xe8\x88\xb2',
'ArabicLetterRehWithTwoDotsVerticallyAbove': '\xdd\xab',
'CjkUnifiedIdeograph8234': '\xe8\x88\xb4',
'CjkUnifiedIdeograph8237': '\xe8\x88\xb7',
'CjkUnifiedIdeograph8236': '\xe8\x88\xb6',
'ModifierLetterReversedComma': '\xca\xbd',
'UnicodeCharacter0xf3f9': '\xef\x8f\xb9',
'UnicodeCharacter0xf3f8': '\xef\x8f\xb8',
'UnicodeCharacter0xf3f7': '\xef\x8f\xb7',
'UnicodeCharacter0xe2b0': '\xee\x8a\xb0',
'UnicodeCharacter0xf3f5': '\xef\x8f\xb5',
'UnicodeCharacter0xf3f4': '\xef\x8f\xb4',
'UnicodeCharacter0xf3f3': '\xef\x8f\xb3',
'UnicodeCharacter0xf3f2': '\xef\x8f\xb2',
'UnicodeCharacter0xf3f1': '\xef\x8f\xb1',
'UnicodeCharacter0xf3f0': '\xef\x8f\xb0',
'HangulSyllableBbyet': '\xeb\xbc\xbd',
'RightwardsTripleDashArrow': '\xe2\xa4\x8f',
'ArabicLetterTahIsolatedForm': '\xef\xbb\x81',
'RomanNumeralSixLateForm': '\xe2\x86\x85',
'UnicodeCharacter0xe2b7': '\xee\x8a\xb7',
'ArabicLetterDyehMedialForm': '\xef\xad\xb5',
'UnicodeCharacter0xed1a': '\xee\xb4\x9a',
'CjkUnifiedIdeograph849f': '\xe8\x92\x9f',
'UnicodeCharacter0xf3ff': '\xef\x8f\xbf',
'UnicodeCharacter0xf3fe': '\xef\x8f\xbe',
'UnicodeCharacter0xf3fd': '\xef\x8f\xbd',
'UnicodeCharacter0xf3fc': '\xef\x8f\xbc',
'HangulSyllableMwaess': '\xeb\xab\xb4',
'RightNormalFactorSemidirectProduct': '\xe2\x8b\x8a',
'UnicodeCharacter0xebe': '\xe0\xba\xbe',
'BraillePatternDots1356': '\xe2\xa0\xb5',
'BraillePatternDots1357': '\xe2\xa1\x95',
'HangulSyllableSswigg': '\xec\x92\xba',
'UnicodeCharacter0xe8a7': '\xee\xa2\xa7',
'UnicodeCharacter0xdc24': '\xed\xb0\xa4',
'HangulSyllableJjyaenj': '\xec\xa8\xb5',
'CjkUnifiedIdeograph823a': '\xe8\x88\xba',
'CjkUnifiedIdeograph823c': '\xe8\x88\xbc',
'CjkUnifiedIdeograph823b': '\xe8\x88\xbb',
'CjkUnifiedIdeograph823e': '\xe8\x88\xbe',
'CjkUnifiedIdeograph823d': '\xe8\x88\xbd',
'BraillePatternDots1358': '\xe2\xa2\x95',
'CjkUnifiedIdeograph823f': '\xe8\x88\xbf',
'HangulSyllableKyiss': '\xed\x82\x9c',
'HangulSyllableTalh': '\xed\x83\x8f',
'CjkUnifiedIdeograph96e7': '\xe9\x9b\xa7',
'UnicodeCharacter0xec9b': '\xee\xb2\x9b',
'UnicodeCharacter0xe8a0': '\xee\xa2\xa0',
'UnicodeCharacter0xe2b5': '\xee\x8a\xb5',
'UnicodeCharacter0xec9a': '\xee\xb2\x9a',
'CircledNumberTwentyOne': '\xe3\x89\x91',
'HangulSyllableCyogs': '\xec\xb5\xbb',
'HangulSyllableHwaebs': '\xed\x9a\x82',
'UnicodeCharacter0xf381': '\xef\x8e\x81',
'GreekPsiliAndOxia': '\xe1\xbf\x8e',
'UnicodeCharacter0xf380': '\xef\x8e\x80',
'RomanNumeralFiveThousand': '\xe2\x86\x81',
'TightTrifoliateSnowflake': '\xe2\x9d\x85',
'HangulSyllableBbyags': '\xeb\xba\x9b',
'UnicodeCharacter0xf383': '\xef\x8e\x83',
'UnicodeCharacter0xe582': '\xee\x96\x82',
'HangulLetterSsangkiyeok': '\xe3\x84\xb2',
'UnicodeCharacter0xf382': '\xef\x8e\x82',
'CjkUnifiedIdeograph9f06': '\xe9\xbc\x86',
'ModifierLetterRaisedExclamationMark': '\xea\x9c\x9d',
'UnicodeCharacter0xf384': '\xef\x8e\x84',
'CjkUnifiedIdeograph5abf': '\xe5\xaa\xbf',
'UnicodeCharacter0xf387': '\xef\x8e\x87',
'UnicodeCharacter0x1a5a': '\xe1\xa9\x9a',
'HangulJungseongOeo': '\xe1\x85\xbf',
'UnicodeCharacter0x1a5b': '\xe1\xa9\x9b',
'UnicodeCharacter0x1a5e': '\xe1\xa9\x9e',
'UnicodeCharacter0x1a5d': '\xe1\xa9\x9d',
'HebrewMarkMasoraCircle': '\xd6\xaf',
'CjkUnifiedIdeograph4be7': '\xe4\xaf\xa7',
'CanadianSyllabicsWestcreeTwo': '\xe1\x91\x9e',
'CjkUnifiedIdeograph4be5': '\xe4\xaf\xa5',
'CjkUnifiedIdeograph4be4': '\xe4\xaf\xa4',
'CjkUnifiedIdeograph4be3': '\xe4\xaf\xa3',
'CjkUnifiedIdeograph4be2': '\xe4\xaf\xa2',
'CjkUnifiedIdeograph4be1': '\xe4\xaf\xa1',
'CanadianSyllabicsWestcreeTwi': '\xe1\x91\x9a',
'CanadianSyllabicsWestcreeTwe': '\xe1\x91\x98',
'HangulSyllableDdyolp': '\xeb\x9a\x8e',
'CjkUnifiedIdeograph4be9': '\xe4\xaf\xa9',
'CjkUnifiedIdeograph4be8': '\xe4\xaf\xa8',
'ps': '\xcf\x88',
'CjkUnifiedIdeograph8366': '\xe8\x8d\xa6',
'UnicodeCharacter0xe8a3': '\xee\xa2\xa3',
'CurlyLogicalOr': '\xe2\x8b\x8e',
'ArmenianSmallLetterMen': '\xd5\xb4',
'ArabicLigatureKafWithMeemFinalForm': '\xef\xb2\x82',
'CjkUnifiedIdeograph4bef': '\xe4\xaf\xaf',
'CjkUnifiedIdeograph4bee': '\xe4\xaf\xae',
'CjkUnifiedIdeograph4bed': '\xe4\xaf\xad',
'CjkUnifiedIdeograph4bec': '\xe4\xaf\xac',
'CjkUnifiedIdeograph4beb': '\xe4\xaf\xab',
'CjkUnifiedIdeograph4bea': '\xe4\xaf\xaa',
'CjkUnifiedIdeograph8365': '\xe8\x8d\xa5',
'LatinSmallLetterLWithCircumflexBelow': '\xe1\xb8\xbd',
'UnicodeCharacter0x1a59': '\xe1\xa9\x99',
'UnicodeCharacter0x1a58': '\xe1\xa9\x98',
'CjkUnifiedIdeograph5da8': '\xe5\xb6\xa8',
'CjkUnifiedIdeograph5da9': '\xe5\xb6\xa9',
'UnicodeCharacter0x1a51': '\xe1\xa9\x91',
'UnicodeCharacter0x1a50': '\xe1\xa9\x90',
'UnicodeCharacter0x1a53': '\xe1\xa9\x93',
'UnicodeCharacter0x1a52': '\xe1\xa9\x92',
'UnicodeCharacter0x1a55': '\xe1\xa9\x95',
'UnicodeCharacter0x1a54': '\xe1\xa9\x94',
'UnicodeCharacter0x1a57': '\xe1\xa9\x97',
'LatinSmallLetterOWithMacron': '\xc5\x8d',
'BalineseLetterHa': '\xe1\xac\xb3',
'GeorgianSmallLetterTan': '\xe2\xb4\x87',
'CjkUnifiedIdeograph8364': '\xe8\x8d\xa4',
'HangulSyllableSulb': '\xec\x88\xa3',
'UnicodeCharacter0xf1f7': '\xef\x87\xb7',
'CjkUnifiedIdeograph63df': '\xe6\x8f\x9f',
'CjkUnifiedIdeograph63de': '\xe6\x8f\x9e',
'CjkUnifiedIdeograph63dd': '\xe6\x8f\x9d',
'CjkUnifiedIdeograph63dc': '\xe6\x8f\x9c',
'CjkUnifiedIdeograph63db': '\xe6\x8f\x9b',
'CjkUnifiedIdeograph63da': '\xe6\x8f\x9a',
'CjkUnifiedIdeograph6cfe': '\xe6\xb3\xbe',
'GeorgianSmallLetterTar': '\xe2\xb4\x92',
'UnicodeCharacter0xf6e8': '\xef\x9b\xa8',
'HangulSyllableMyaeng': '\xeb\xa8\xb1',
'CjkUnifiedIdeograph34b8': '\xe3\x92\xb8',
'SubsetOfAboveTildeOperator': '\xe2\xab\x87',
'CjkUnifiedIdeograph8363': '\xe8\x8d\xa3',
'CjkUnifiedIdeograph4c7d': '\xe4\xb1\xbd',
'CjkUnifiedIdeograph6d3a': '\xe6\xb4\xba',
'HangulSyllableGgil': '\xeb\x82\x84',
'DoubleIntersection': '\xe2\x8b\x92',
'CjkUnifiedIdeograph6d3b': '\xe6\xb4\xbb',
'CjkUnifiedIdeograph6d3e': '\xe6\xb4\xbe',
'CopticCapitalLetterPi': '\xe2\xb2\xa0',
'UnicodeCharacter0xa6b4': '\xea\x9a\xb4',
'CjkUnifiedIdeograph6d3f': '\xe6\xb4\xbf',
'CjkUnifiedIdeograph7a4f': '\xe7\xa9\x8f',
'CjkUnifiedIdeograph7a4d': '\xe7\xa9\x8d',
'CjkUnifiedIdeograph7a4e': '\xe7\xa9\x8e',
'CjkUnifiedIdeograph7a4b': '\xe7\xa9\x8b',
'CjkUnifiedIdeograph7a4c': '\xe7\xa9\x8c',
'CjkUnifiedIdeograph7a4a': '\xe7\xa9\x8a',
'CjkUnifiedIdeograph63d7': '\xe6\x8f\x97',
'CjkUnifiedIdeograph63d6': '\xe6\x8f\x96',
'CjkUnifiedIdeograph63d5': '\xe6\x8f\x95',
'CanadianSyllabicsWestcreeShwo': '\xe1\x94\x9e',
'CjkUnifiedIdeograph63d3': '\xe6\x8f\x93',
'BraillePatternDots258': '\xe2\xa2\x92',
'CjkUnifiedIdeograph63d1': '\xe6\x8f\x91',
'CjkUnifiedIdeograph63d0': '\xe6\x8f\x90',
'BraillePatternDots257': '\xe2\xa1\x92',
'BraillePatternDots256': '\xe2\xa0\xb2',
'CjkUnifiedIdeograph63d9': '\xe6\x8f\x99',
'CanadianSyllabicsWestcreeShwi': '\xe1\x94\x9a',
'UnicodeCharacter0xf3cd': '\xef\x8f\x8d',
'CircledIdeographReligion': '\xe3\x8a\xaa',
'HangulSyllableNyalp': '\xeb\x83\x9e',
'HangulSyllableSsweogg': '\xec\x92\x82',
'CjkUnifiedIdeograph6d31': '\xe6\xb4\xb1',
'CjkUnifiedIdeograph6d30': '\xe6\xb4\xb0',
'CjkUnifiedIdeograph6d33': '\xe6\xb4\xb3',
'CjkUnifiedIdeograph6d32': '\xe6\xb4\xb2',
'CjkUnifiedIdeograph6d35': '\xe6\xb4\xb5',
'CjkUnifiedIdeograph6d34': '\xe6\xb4\xb4',
'CjkUnifiedIdeograph6d37': '\xe6\xb4\xb7',
'CjkUnifiedIdeograph6d36': '\xe6\xb4\xb6',
'CjkUnifiedIdeograph6d39': '\xe6\xb4\xb9',
'CjkUnifiedIdeograph6d38': '\xe6\xb4\xb8',
'OpenCentreTeardropspokedAsterisk': '\xe2\x9c\xbc',
'CjkUnifiedIdeograph7a45': '\xe7\xa9\x85',
'CjkUnifiedIdeograph7a42': '\xe7\xa9\x82',
'CjkUnifiedIdeograph7a43': '\xe7\xa9\x83',
'CjkUnifiedIdeograph7a40': '\xe7\xa9\x80',
'CjkUnifiedIdeograph7a41': '\xe7\xa9\x81',
'CjkUnifiedIdeograph7d88': '\xe7\xb6\x88',
'LatinSmallLetterMWithAcute': '\xe1\xb8\xbf',
'CjkUnifiedIdeograph8360': '\xe8\x8d\xa0',
'UnicodeCharacter0xec94': '\xee\xb2\x94',
'CircledKatakanaMu': '\xe3\x8b\xb0',
'CanadianSyllabicsNii': '\xe1\x93\x83',
'HangulSyllableNyaenh': '\xeb\x83\xb2',
'HangulSyllableNyaenj': '\xeb\x83\xb1',
'HangulSyllableDdugs': '\xeb\x9a\x9f',
'RomanNumeralNine': '\xe2\x85\xa8',
'CjkUnifiedIdeograph5e7a': '\xe5\xb9\xba',
'UnicodeCharacter0x2bef': '\xe2\xaf\xaf',
'UnicodeCharacter0x2bee': '\xe2\xaf\xae',
'UnicodeCharacter0x2bed': '\xe2\xaf\xad',
'UnicodeCharacter0x2bec': '\xe2\xaf\xac',
'UnicodeCharacter0x2beb': '\xe2\xaf\xab',
'UnicodeCharacter0x2bea': '\xe2\xaf\xaa',
'SymbolForRecordSeparator': '\xe2\x90\x9e',
'CircledKatakanaMi': '\xe3\x8b\xaf',
'UnicodeCharacter0xec90': '\xee\xb2\x90',
'CircledKatakanaMo': '\xe3\x8b\xb2',
'HangulSyllableBwec': '\xeb\xb7\x8f',
'GurmukhiVowelSignU': '\xe0\xa9\x81',
'BraillePatternDots1234568': '\xe2\xa2\xbf',
'HangulSyllableBweg': '\xeb\xb6\xb9',
'CjkUnifiedIdeograph9cfe': '\xe9\xb3\xbe',
'CjkUnifiedIdeograph95ec': '\xe9\x97\xac',
'CjkUnifiedIdeograph95eb': '\xe9\x97\xab',
'CjkUnifiedIdeograph95ea': '\xe9\x97\xaa',
'HangulSyllableBweh': '\xeb\xb7\x93',
'BraillePatternDots1234567': '\xe2\xa1\xbf',
'UnicodeCharacter0x874': '\xe0\xa1\xb4',
'CjkUnifiedIdeograph95ee': '\xe9\x97\xae',
'CjkUnifiedIdeograph95ed': '\xe9\x97\xad',
'HangulSyllableBwes': '\xeb\xb7\x8b',
'CjkUnifiedIdeograph7138': '\xe7\x84\xb8',
'HangulSyllableBwet': '\xeb\xb7\x91',
'UnicodeCharacter0x872': '\xe0\xa1\xb2',
'GreaterthanAboveSimilarAboveLessthan': '\xe2\xaa\x90',
'GurmukhiVowelSignI': '\xe0\xa8\xbf',
'CjkUnifiedIdeograph9cfa': '\xe9\xb3\xba',
'UnicodeCharacter0x2be7': '\xe2\xaf\xa7',
'UnicodeCharacter0x2be6': '\xe2\xaf\xa6',
'SinhalaVowelSignDigaGaettapilla': '\xe0\xb7\xb2',
'UnicodeCharacter0x2be4': '\xe2\xaf\xa4',
'UnicodeCharacter0x2be3': '\xe2\xaf\xa3',
'UnicodeCharacter0x2be2': '\xe2\xaf\xa2',
'UnicodeCharacter0x2be1': '\xe2\xaf\xa1',
'UnicodeCharacter0x2be0': '\xe2\xaf\xa0',
'CjkUnifiedIdeograph5e79': '\xe5\xb9\xb9',
'CjkUnifiedIdeograph5e78': '\xe5\xb9\xb8',
'IdeographicEnteringToneMark': '\xe3\x80\xad',
'HangulSyllableBbyeng': '\xeb\xbc\xb9',
'ArabicLetterNoonWithSmallV': '\xdd\xa9',
'UnicodeCharacter0x2be9': '\xe2\xaf\xa9',
'UnicodeCharacter0x2be8': '\xe2\xaf\xa8',
'CjkUnifiedIdeograph7133': '\xe7\x84\xb3',
'BoxDrawingsDownLightAndUpHorizontalHeavy': '\xe2\x95\x87',
'EthiopicSyllableZho': '\xe1\x8b\xa6',
'LatinCapitalLetterUWithHornAndAcute': '\xe1\xbb\xa8',
'KhmerSymbolDapbeiRoc': '\xe1\xa7\xbd',
'HangulSyllableOgs': '\xec\x98\xa7',
'CjkUnifiedIdeograph7137': '\xe7\x84\xb7',
'HangulSyllableNelp': '\xeb\x84\xb2',
'CjkUnifiedIdeograph95e9': '\xe9\x97\xa9',
'CjkUnifiedIdeograph95e8': '\xe9\x97\xa8',
'CjkUnifiedIdeograph7135': '\xe7\x84\xb5',
'HangulSyllableJjyem': '\xec\xaa\xb0',
'CjkUnifiedIdeograph95e3': '\xe9\x97\xa3',
'CjkUnifiedIdeograph95e2': '\xe9\x97\xa2',
'CjkUnifiedIdeograph95e1': '\xe9\x97\xa1',
'CjkUnifiedIdeograph7134': '\xe7\x84\xb4',
'CjkUnifiedIdeograph95e7': '\xe9\x97\xa7',
'CjkUnifiedIdeograph8228': '\xe8\x88\xa8',
'CjkUnifiedIdeograph95e5': '\xe9\x97\xa5',
'CjkUnifiedIdeograph95e4': '\xe9\x97\xa4',
'HangulSyllableBbwaeng': '\xeb\xbe\x8d',
'ArabicLigatureTehWithHahWithJeemInitialForm': '\xef\xb5\x92',
'UnicodeCharacter0xded9': '\xed\xbb\x99',
'CjkUnifiedIdeograph9094': '\xe9\x82\x94',
'ClosedSuperset': '\xe2\xab\x90',
'cth': '\xcf\x91',
'YiSyllableWex': '\xea\x8a\x8a',
'EthiopicSyllableZhi': '\xe1\x8b\xa2',
'HangulSyllableBweonh': '\xeb\xb6\xa2',
'YiSyllableWep': '\xea\x8a\x8c',
'HangulSyllableBeog': '\xeb\xb2\x85',
'UnicodeCharacter0xe38a': '\xee\x8e\x8a',
'UnicodeCharacter0xe38b': '\xee\x8e\x8b',
'HangulSyllableBeod': '\xeb\xb2\x8b',
'HangulSyllableBeoc': '\xeb\xb2\x9b',
'HangulSyllableBeob': '\xeb\xb2\x95',
'UnicodeCharacter0xe38f': '\xee\x8e\x8f',
'HangulSyllableBeon': '\xeb\xb2\x88',
'HangulSyllableBeom': '\xeb\xb2\x94',
'HangulSyllableBeol': '\xeb\xb2\x8c',
'HangulSyllableBeok': '\xeb\xb2\x9c',
'HangulSyllableBeoj': '\xeb\xb2\x9a',
'HangulSyllableBeoh': '\xeb\xb2\x9f',
'HangulSyllableBeot': '\xeb\xb2\x9d',
'HangulSyllableBeos': '\xeb\xb2\x97',
'UnicodeCharacter0xe9e8': '\xee\xa7\xa8',
'HangulSyllableBeop': '\xeb\xb2\x9e',
'UnicodeCharacter0xe9e5': '\xee\xa7\xa5',
'UnicodeCharacter0xe9e4': '\xee\xa7\xa4',
'YiSyllableZzurx': '\xea\x8b\xa5',
'ParenthesizedIdeographMetal': '\xe3\x88\xae',
'UnicodeCharacter0xe9e1': '\xee\xa7\xa1',
'UnicodeCharacter0xe9e0': '\xee\xa7\xa0',
'UnicodeCharacter0xe9e3': '\xee\xa7\xa3',
'UnicodeCharacter0xe9e2': '\xee\xa7\xa2',
'HangulSyllableJjeogs': '\xec\xa9\x8f',
'CjkUnifiedIdeograph72f3': '\xe7\x8b\xb3',
'CjkUnifiedIdeograph72f2': '\xe7\x8b\xb2',
'UnicodeCharacter0xf7ca': '\xef\x9f\x8a',
'HangulSyllableHonj': '\xed\x98\xbd',
'CjkUnifiedIdeograph8369': '\xe8\x8d\xa9',
'GreekSmallLetterAlphaWithPsiliAndOxiaAndYpogegrammeni': '\xe1\xbe\x84',
'HangulSyllableMweolh': '\xeb\xad\x9f',
'ArabicLigatureAlayheIsolatedForm': '\xef\xb7\xb7',
'CjkUnifiedIdeograph72f6': '\xe7\x8b\xb6',
'RightWhiteLenticularBracket': '\xe3\x80\x97',
'CjkUnifiedIdeograph72f5': '\xe7\x8b\xb5',
'UnicodeCharacter0xe9ee': '\xee\xa7\xae',
'HangulSyllableDyen': '\xeb\x8e\xac',
'HangulSyllableSsags': '\xec\x8b\xbb',
'UnicodeCharacter0xe9ea': '\xee\xa7\xaa',
'CyrillicSmallLetterShcha': '\xd1\x89',
'UnicodeCharacter0xe9ec': '\xee\xa7\xac',
'CjkUnifiedIdeograph8368': '\xe8\x8d\xa8',
'UnicodeCharacter0xe380': '\xee\x8e\x80',
'UnicodeCharacter0xe381': '\xee\x8e\x81',
'UnicodeCharacter0xe382': '\xee\x8e\x82',
'UnicodeCharacter0xe383': '\xee\x8e\x83',
'UnicodeCharacter0xe384': '\xee\x8e\x84',
'UnicodeCharacter0xe385': '\xee\x8e\x85',
'UnicodeCharacter0xe386': '\xee\x8e\x86',
'UnicodeCharacter0xe387': '\xee\x8e\x87',
'UnicodeCharacter0xe388': '\xee\x8e\x88',
'TibetanMarkShad': '\xe0\xbc\x8d',
'UnicodeCharacter0xf1d2': '\xef\x87\x92',
'HangulSyllableSsagg': '\xec\x8b\xba',
'UnicodeCharacter0xe8a8': '\xee\xa2\xa8',
'UnicodeCharacter0xe8a9': '\xee\xa2\xa9',
'HangulSyllableBbals': '\xeb\xb9\xac',
'UnicodeCharacter0xe8a5': '\xee\xa2\xa5',
'UnicodeCharacter0xe8a6': '\xee\xa2\xa6',
'HangulSyllableBbalp': '\xeb\xb9\xae',
'UnicodeCharacter0xe731': '\xee\x9c\xb1',
'UnicodeCharacter0xe8a1': '\xee\xa2\xa1',
'UnicodeCharacter0xe8a2': '\xee\xa2\xa2',
'HangulSyllableBbalt': '\xeb\xb9\xad',
'ArabicLigatureSeenWithKhahWithAlefMaksuraFinalForm': '\xef\xb6\xa8',
'LatinCapitalLetterCon': '\xea\x9d\xae',
'HangulSyllableBbalm': '\xeb\xb9\xaa',
'GreaterthanAboveSimilarOrEqual': '\xe2\xaa\x8e',
'HangulSyllableBbalb': '\xeb\xb9\xab',
'HangulSyllableHong': '\xed\x99\x8d',
'HangulSyllableBbalg': '\xeb\xb9\xa9',
'CjkUnifiedIdeograph9c02': '\xe9\xb0\x82',
'CanadianSyllabicsTwoo': '\xe1\x91\x9f',
'CjkUnifiedIdeograph3ff9': '\xe3\xbf\xb9',
'CjkUnifiedIdeograph3ff8': '\xe3\xbf\xb8',
'CjkUnifiedIdeograph9c06': '\xe9\xb0\x86',
'CjkUnifiedIdeograph9c07': '\xe9\xb0\x87',
'CjkUnifiedIdeograph9c04': '\xe9\xb0\x84',
'CjkUnifiedIdeograph9c05': '\xe9\xb0\x85',
'CjkUnifiedIdeograph3ff3': '\xe3\xbf\xb3',
'CjkUnifiedIdeograph3ff2': '\xe3\xbf\xb2',
'CjkUnifiedIdeograph556d': '\xe5\x95\xad',
'CjkUnifiedIdeograph3ff0': '\xe3\xbf\xb0',
'CjkUnifiedIdeograph3ff7': '\xe3\xbf\xb7',
'CjkUnifiedIdeograph556c': '\xe5\x95\xac',
'CjkUnifiedIdeograph3ff5': '\xe3\xbf\xb5',
'CjkUnifiedIdeograph3ff4': '\xe3\xbf\xb4',
'UnicodeCharacter0xe658': '\xee\x99\x98',
'UnicodeCharacter0xe659': '\xee\x99\x99',
'CjkUnifiedIdeograph38ba': '\xe3\xa2\xba',
'HangulSyllableJeus': '\xec\xa6\x9b',
'CjkUnifiedIdeograph38bc': '\xe3\xa2\xbc',
'CjkUnifiedIdeograph38bb': '\xe3\xa2\xbb',
'CjkUnifiedIdeograph38be': '\xe3\xa2\xbe',
'CjkUnifiedIdeograph38bd': '\xe3\xa2\xbd',
'UnicodeCharacter0xe654': '\xee\x99\x94',
'CjkUnifiedIdeograph38bf': '\xe3\xa2\xbf',
'UnicodeCharacter0xf7ce': '\xef\x9f\x8e',
'GreekSmallLetterIotaWithOxia': '\xe1\xbd\xb7',
'HangulSyllableHalt': '\xed\x95\xa5',
'greaterthanequal': '\xe2\x89\xa5',
'UnicodeCharacter0xf1f0': '\xef\x87\xb0',
'UnicodeCharacter0xe8ad': '\xee\xa2\xad',
'UnicodeCharacter0xe8ae': '\xee\xa2\xae',
'UnicodeCharacter0xe8af': '\xee\xa2\xaf',
'HangulSyllableKwad': '\xec\xbd\xb7',
'UnicodeCharacter0xe8aa': '\xee\xa2\xaa',
'UnicodeCharacter0xe8ab': '\xee\xa2\xab',
'UnicodeCharacter0xe8ac': '\xee\xa2\xac',
'CjkUnifiedIdeograph38b9': '\xe3\xa2\xb9',
'LatinLetterSmallCapitalLWithStroke': '\xe1\xb4\x8c',
'YiSyllableZex': '\xea\x8a\x9f',
'HiraganaLetterYa': '\xe3\x82\x84',
'UnicodeCharacter0xf79f': '\xef\x9e\x9f',
'GreekSmallLetterOmegaWithVaria': '\xe1\xbd\xbc',
'UnicodeCharacter0xe732': '\xee\x9c\xb2',
'BlackCentreWhiteStar': '\xe2\x9c\xac',
'CjkUnifiedIdeograph38b1': '\xe3\xa2\xb1',
'CjkUnifiedIdeograph38b0': '\xe3\xa2\xb0',
'CjkUnifiedIdeograph38b3': '\xe3\xa2\xb3',
'CjkUnifiedIdeograph38b2': '\xe3\xa2\xb2',
'CjkUnifiedIdeograph38b5': '\xe3\xa2\xb5',
'CjkUnifiedIdeograph38b4': '\xe3\xa2\xb4',
'MyanmarConsonantSignMedialHa': '\xe1\x80\xbe',
'CjkUnifiedIdeograph38b6': '\xe3\xa2\xb6',
'CjkUnifiedIdeograph9c0b': '\xe9\xb0\x8b',
'CjkUnifiedIdeograph9c0c': '\xe9\xb0\x8c',
'CjkUnifiedIdeograph9c0a': '\xe9\xb0\x8a',
'GreaterthanAndSinglelineNotEqualTo': '\xe2\xaa\x88',
'CjkUnifiedIdeograph5568': '\xe5\x95\xa8',
'CjkUnifiedIdeograph5569': '\xe5\x95\xa9',
'CjkUnifiedIdeograph3ffc': '\xe3\xbf\xbc',
'CjkUnifiedIdeograph3ffb': '\xe3\xbf\xbb',
'CjkUnifiedIdeograph3ffa': '\xe3\xbf\xba',
'CjkUnifiedIdeograph5565': '\xe5\x95\xa5',
'CjkUnifiedIdeograph5562': '\xe5\x95\xa2',
'CjkUnifiedIdeograph5563': '\xe5\x95\xa3',
'CjkUnifiedIdeograph5560': '\xe5\x95\xa0',
'CjkUnifiedIdeograph3ffd': '\xe3\xbf\xbd',
'BengaliLetterAu': '\xe0\xa6\x94',
'UnicodeCharacter0xe735': '\xee\x9c\xb5',
'CjkUnifiedIdeograph614e': '\xe6\x85\x8e',
'CjkUnifiedIdeograph614d': '\xe6\x85\x8d',
'CjkUnifiedIdeograph614f': '\xe6\x85\x8f',
'CjkUnifiedIdeograph614a': '\xe6\x85\x8a',
'CjkUnifiedIdeograph614c': '\xe6\x85\x8c',
'CjkUnifiedIdeograph614b': '\xe6\x85\x8b',
'CjkUnifiedIdeograph6cf9': '\xe6\xb3\xb9',
'CjkUnifiedIdeograph6cf8': '\xe6\xb3\xb8',
'UnicodeCharacter0xf79d': '\xef\x9e\x9d',
'SyriacSublinearColonSkewedRight': '\xdc\x89',
'UnicodeCharacter0xe734': '\xee\x9c\xb4',
'HangulSyllableDdyi': '\xeb\x9d\x84',
'UnicodeCharacter0xe8c8': '\xee\xa3\x88',
'HangulSyllableDdye': '\xeb\x97\xb4',
'HangulSyllableDdya': '\xeb\x95\xa8',
'UnicodeCharacter0xf179': '\xef\x85\xb9',
'UnicodeCharacter0xded3': '\xed\xbb\x93',
'HangulSyllableDdyu': '\xeb\x9c\x8c',
'SinhalaLetterDantajaNayanna': '\xe0\xb6\xb1',
'UnicodeCharacter0xf79e': '\xef\x9e\x9e',
'UnicodeCharacter0xf1c4': '\xef\x87\x84',
'TibetanSubjoinedLetterLa': '\xe0\xbe\xb3',
'FullwidthLatinCapitalLetterT': '\xef\xbc\xb4',
'HangulSyllableBweonj': '\xeb\xb6\xa1',
'OlChikiRelaa': '\xe1\xb1\xbb',
'GeorgianLetterTan': '\xe1\x83\x97',
'GeorgianLetterTar': '\xe1\x83\xa2',
'ArabicLetterWawWithExtendedArabicindicDigitThreeAbove': '\xdd\xb9',
'CjkUnifiedIdeograph6148': '\xe6\x85\x88',
'CjkUnifiedIdeograph6145': '\xe6\x85\x85',
'CjkUnifiedIdeograph6144': '\xe6\x85\x84',
'CjkUnifiedIdeograph6147': '\xe6\x85\x87',
'CjkUnifiedIdeograph6146': '\xe6\x85\x86',
'CjkUnifiedIdeograph6141': '\xe6\x85\x81',
'CjkUnifiedIdeograph6140': '\xe6\x85\x80',
'CjkUnifiedIdeograph6143': '\xe6\x85\x83',
'CjkUnifiedIdeograph6142': '\xe6\x85\x82',
'GurmukhiLetterAu': '\xe0\xa8\x94',
'UnicodeCharacter0xdc49': '\xed\xb1\x89',
'UnicodeCharacter0xf1f3': '\xef\x87\xb3',
'FullwidthLatinCapitalLetterR': '\xef\xbc\xb2',
'DoesNotSucceedOrEqual': '\xe2\x8b\xa1',
'CombiningAcutemacron': '\xe1\xb7\x87',
'GurmukhiLetterAa': '\xe0\xa8\x86',
'UnicodeCharacter0xf79c': '\xef\x9e\x9c',
'KayahLiLetterRa': '\xea\xa4\x9a',
'GurmukhiLetterAi': '\xe0\xa8\x90',
'LatinSmallLetterTWithLineBelow': '\xe1\xb9\xaf',
'BlackHourglass': '\xe2\xa7\x97',
'ArabicLigatureFehWithJeemInitialForm': '\xef\xb2\xbe',
'TibetanLetterSa': '\xe0\xbd\xa6',
'CjkUnifiedIdeograph723e': '\xe7\x88\xbe',
'CjkUnifiedIdeograph723f': '\xe7\x88\xbf',
'FullwidthLatinCapitalLetterZ': '\xef\xbc\xba',
'CjkUnifiedIdeograph723a': '\xe7\x88\xba',
'CjkUnifiedIdeograph723b': '\xe7\x88\xbb',
'CjkUnifiedIdeograph723c': '\xe7\x88\xbc',
'CjkUnifiedIdeograph4774': '\xe4\x9d\xb4',
'CjkUnifiedIdeograph4775': '\xe4\x9d\xb5',
'CjkUnifiedIdeograph4776': '\xe4\x9d\xb6',
'CjkUnifiedIdeograph4777': '\xe4\x9d\xb7',
'CjkUnifiedIdeograph4770': '\xe4\x9d\xb0',
'CjkUnifiedIdeograph4771': '\xe4\x9d\xb1',
'CjkUnifiedIdeograph4772': '\xe4\x9d\xb2',
'CjkUnifiedIdeograph4773': '\xe4\x9d\xb3',
'CjkUnifiedIdeograph392d': '\xe3\xa4\xad',
'CjkUnifiedIdeograph392e': '\xe3\xa4\xae',
'CjkUnifiedIdeograph392f': '\xe3\xa4\xaf',
'UnicodeCharacter0xeb5e': '\xee\xad\x9e',
'CjkUnifiedIdeograph4778': '\xe4\x9d\xb8',
'CjkUnifiedIdeograph4779': '\xe4\x9d\xb9',
'CjkUnifiedIdeograph392b': '\xe3\xa4\xab',
'CjkUnifiedIdeograph392c': '\xe3\xa4\xac',
'HangulChoseongPieupsioskiyeok': '\xe1\x84\xa2',
'FullwidthLatinCapitalLetterF': '\xef\xbc\xa6',
'HanunooLetterWa': '\xe1\x9c\xaf',
'HangulSyllableGgaegg': '\xea\xb9\xaa',
'CyrillicCapitalLetterTse': '\xd0\xa6',
'CjkUnifiedIdeograph7238': '\xe7\x88\xb8',
'CjkUnifiedIdeograph7239': '\xe7\x88\xb9',
'CjkUnifiedIdeograph7f3a': '\xe7\xbc\xba',
'CjkUnifiedIdeograph7234': '\xe7\x88\xb4',
'CjkUnifiedIdeograph7235': '\xe7\x88\xb5',
'GreekQuestionMark': '\xcd\xbe',
'CjkUnifiedIdeograph7237': '\xe7\x88\xb7',
'CjkUnifiedIdeograph7230': '\xe7\x88\xb0',
'CjkUnifiedIdeograph7231': '\xe7\x88\xb1',
'CjkUnifiedIdeograph7232': '\xe7\x88\xb2',
'CjkUnifiedIdeograph7233': '\xe7\x88\xb3',
'UnicodeCharacter0xe7cb': '\xee\x9f\x8b',
'HangulSyllableGgaegs': '\xea\xb9\xab',
'KatakanahiraganaProlongedSoundMark': '\xe3\x83\xbc',
'UnicodeCharacter0xe7ce': '\xee\x9f\x8e',
'UnicodeCharacter0xe7cd': '\xee\x9f\x8d',
'CyrillicCapitalLetterVe': '\xd0\x92',
'RunicLetterEth': '\xe1\x9a\xa7',
'HangulSyllableWiss': '\xec\x9c\x98',
'BraillePatternDots14578': '\xe2\xa3\x99',
'UnicodeCharacter0xf2c1': '\xef\x8b\x81',
'UnicodeCharacter0xe7cf': '\xee\x9f\x8f',
'FullwidthLatinCapitalLetterH': '\xef\xbc\xa8',
'CjkUnifiedIdeograph477d': '\xe4\x9d\xbd',
'CjkUnifiedIdeograph477e': '\xe4\x9d\xbe',
'CjkUnifiedIdeograph477f': '\xe4\x9d\xbf',
'UnicodeCharacter0xecb4': '\xee\xb2\xb4',
'CjkUnifiedIdeograph3928': '\xe3\xa4\xa8',
'CjkUnifiedIdeograph477a': '\xe4\x9d\xba',
'CjkUnifiedIdeograph477b': '\xe4\x9d\xbb',
'CjkUnifiedIdeograph477c': '\xe4\x9d\xbc',
'CjkUnifiedIdeograph3924': '\xe3\xa4\xa4',
'CjkUnifiedIdeograph3925': '\xe3\xa4\xa5',
'CjkUnifiedIdeograph3926': '\xe3\xa4\xa6',
'CjkUnifiedIdeograph3927': '\xe3\xa4\xa7',
'CjkUnifiedIdeograph3920': '\xe3\xa4\xa0',
'CjkUnifiedIdeograph3921': '\xe3\xa4\xa1',
'CjkUnifiedIdeograph3922': '\xe3\xa4\xa2',
'CjkUnifiedIdeograph3923': '\xe3\xa4\xa3',
'CjkUnifiedIdeograph3696': '\xe3\x9a\x96',
'CjkUnifiedIdeograph3697': '\xe3\x9a\x97',
'CjkUnifiedIdeograph3694': '\xe3\x9a\x94',
'CjkUnifiedIdeograph3695': '\xe3\x9a\x95',
'CjkUnifiedIdeograph3692': '\xe3\x9a\x92',
'CjkUnifiedIdeograph3693': '\xe3\x9a\x93',
'CjkUnifiedIdeograph3690': '\xe3\x9a\x90',
'CjkUnifiedIdeograph3691': '\xe3\x9a\x91',
'KhmerCurrencySymbolRiel': '\xe1\x9f\x9b',
'CjkUnifiedIdeograph3698': '\xe3\x9a\x98',
'CjkUnifiedIdeograph3699': '\xe3\x9a\x99',
'HangulSyllableRyuss': '\xeb\xa5\xac',
'KhmerSignNikahit': '\xe1\x9f\x86',
'HangulSyllableGeolg': '\xea\xb1\xb9',
'CjkUnifiedIdeograph8d43': '\xe8\xb5\x83',
'LatinCapitalLetterOWithDoubleAcute': '\xc5\x90',
'SummationWithIntegral': '\xe2\xa8\x8b',
'HangulSyllableKoenh': '\xec\xbe\xae',
'HangulSyllableGgwap': '\xea\xbd\xa2',
'HangulSyllableGgwas': '\xea\xbd\x9b',
'HangulSyllableGgwal': '\xea\xbd\x90',
'HangulSyllableGgwam': '\xea\xbd\x98',
'LargerThan': '\xe2\xaa\xab',
'HangulSyllableDdyenh': '\xeb\x97\xba',
'HangulSyllableGgwah': '\xea\xbd\xa3',
'HangulSyllableGgwaj': '\xea\xbd\x9e',
'HangulSyllableGgwak': '\xea\xbd\xa0',
'HangulSyllableGgwad': '\xea\xbd\x8f',
'HangulSyllableGgwae': '\xea\xbd\xa4',
'HangulSyllableGgwag': '\xea\xbd\x89',
'HangulSyllableGgwab': '\xea\xbd\x99',
'CanadianSyllabicsShwoo': '\xe1\x94\x9f',
'HangulSyllableRyim': '\xeb\xa6\xa0',
'HangulSyllableRyil': '\xeb\xa6\x98',
'HangulSyllableRyin': '\xeb\xa6\x94',
'ArmenianCapitalLetterSeh': '\xd5\x8d',
'HangulSyllableRyik': '\xeb\xa6\xa8',
'HangulSyllableRyij': '\xeb\xa6\xa6',
'HangulSyllableRyid': '\xeb\xa6\x97',
'HangulSyllableRyig': '\xeb\xa6\x91',
'HangulSyllableNod': '\xeb\x85\xbf',
'UnicodeCharacter0xecb3': '\xee\xb2\xb3',
'SundaneseVowelSignPanaelaeng': '\xe1\xae\xa6',
'HangulSyllableRyib': '\xeb\xa6\xa1',
'CjkUnifiedIdeograph369f': '\xe3\x9a\x9f',
'HangulSyllableNok': '\xeb\x86\x90',
'CjkUnifiedIdeograph369d': '\xe3\x9a\x9d',
'CjkUnifiedIdeograph369e': '\xe3\x9a\x9e',
'CjkUnifiedIdeograph369b': '\xe3\x9a\x9b',
'CjkUnifiedIdeograph369c': '\xe3\x9a\x9c',
'UnicodeCharacter0xdd1a': '\xed\xb4\x9a',
'CjkUnifiedIdeograph369a': '\xe3\x9a\x9a',
'CyrillicCapitalLetterKomiLje': '\xd4\x88',
'HangulSyllableRyit': '\xeb\xa6\xa9',
'MongolianLetterAliGaliTta': '\xe1\xa2\x8c',
'HangulSyllableRyip': '\xeb\xa6\xaa',
'HangulSyllableRyis': '\xeb\xa6\xa3',
'CanadianSyllabicsBlackfootA': '\xe1\x96\xb3',
'HangulSyllableNoh': '\xeb\x86\x93',
'CanadianSyllabicsBlackfootE': '\xe1\x96\xb0',
'HangulSyllableDdwigg': '\xeb\x9b\xb2',
'ArabicLigatureKafWithAlefFinalForm': '\xef\xb2\x80',
'CanadianSyllabicsBlackfootI': '\xe1\x96\xb1',
'UnicodeCharacter0xe762': '\xee\x9d\xa2',
'EthiopicSyllablePhwa': '\xe1\x8c\xb7',
'HangulSyllableHalb': '\xed\x95\xa3',
'HangulSyllableHyi': '\xed\x9d\xac',
'CanadianSyllabicsBlackfootO': '\xe1\x96\xb2',
'HangulSyllableNon': '\xeb\x85\xbc',
'MeasuredAngleWithOpenArmEndingInArrowPointingLeftAndDown': '\xe2\xa6\xaf',
'CanadianSyllabicsBlackfootS': '\xe1\x94\x88',
'TibetanMarkBkaShogYigMgo': '\xe0\xbc\x8a',
'VaiSyllableToo': '\xea\x95\xbf',
'CjkUnifiedIdeograph7ff0': '\xe7\xbf\xb0',
'HangulSyllableNyaess': '\xeb\x84\x80',
'UnicodeCharacter0xd7d9': '\xed\x9f\x99',
'ShoulderedOpenBox': '\xe2\x8d\xbd',
'HangulSyllableSswaes': '\xec\x90\xa3',
'UnicodeCharacter0xd7d6': '\xed\x9f\x96',
'LBBarSymbol': '\xe2\x84\x94',
'UnicodeCharacter0xd7d4': '\xed\x9f\x94',
'UnicodeCharacter0xd7d3': '\xed\x9f\x93',
'LessthanOrSlantedEqualToWithDotInside': '\xe2\xa9\xbf',
'UnicodeCharacter0xd7d1': '\xed\x9f\x91',
'UnicodeCharacter0xd7d0': '\xed\x9f\x90',
'HangulSyllablePwab': '\xed\x90\x99',
'KhmerLetterCa': '\xe1\x9e\x85',
'KhmerLetterCo': '\xe1\x9e\x87',
'HangulSyllablePwae': '\xed\x90\xa4',
'HangulSyllableBeu': '\xeb\xb8\x8c',
'LatinSmallLetterOWithLoop': '\xea\x9d\x8d',
'HangulSyllableDoeb': '\xeb\x90\xa9',
'HangulSyllableKwas': '\xec\xbe\x83',
'HangulSyllableSsyinh': '\xec\x94\x92',
'YiSyllableByrx': '\xea\x80\xb6',
'UnicodeCharacter0xe163': '\xee\x85\xa3',
'HangulSyllablePwah': '\xed\x90\xa3',
'HangulSyllableBet': '\xeb\xb2\xb9',
'UnicodeCharacter0xe0ba': '\xee\x82\xba',
'HangulSyllableKwat': '\xec\xbe\x89',
'HangulSyllablePwak': '\xed\x90\xa0',
'SinhalaLetterYayanna': '\xe0\xb6\xba',
'HangulSyllablePwaj': '\xed\x90\x9e',
'HangulSyllablePwam': '\xed\x90\x98',
'ArabicSmallHighRoundedZero': '\xdb\x9f',
'UnicodeCharacter0xecbf': '\xee\xb2\xbf',
'HangulSyllableHols': '\xed\x99\x84',
'MongolianLetterZhi': '\xe1\xa1\x81',
'GurmukhiAddak': '\xe0\xa9\xb1',
'LatinLetterGlottalStopWithStroke': '\xca\xa1',
'BalineseDigitTwo': '\xe1\xad\x92',
'HangulSyllablePwan': '\xed\x90\x8c',
'UnicodeCharacter0xd7dc': '\xed\x9f\x9c',
'UnicodeCharacter0xd7db': '\xed\x9f\x9b',
'UnicodeCharacter0xd7da': '\xed\x9f\x9a',
'HangulSyllableCab': '\xec\xb0\xb9',
'UnicodeCharacter0xef4b': '\xee\xbd\x8b',
'HangulSyllablePwap': '\xed\x90\xa2',
'HangulSyllableSyas': '\xec\x83\xb7',
'HangulSyllableSyap': '\xec\x83\xbe',
'HangulSyllableSyat': '\xec\x83\xbd',
'HangulSyllableSyaj': '\xec\x83\xba',
'HangulSyllableSyak': '\xec\x83\xbc',
'HangulSyllableSyah': '\xec\x83\xbf',
'HangulSyllableSyan': '\xec\x83\xa8',
'HangulSyllableSyal': '\xec\x83\xac',
'HangulSyllableSyam': '\xec\x83\xb4',
'HangulSyllableSyab': '\xec\x83\xb5',
'HangulSyllableSyac': '\xec\x83\xbb',
'UnicodeCharacter0xecba': '\xee\xb2\xba',
'UnicodeCharacter0xe537': '\xee\x94\xb7',
'HangulSyllableSyag': '\xec\x83\xa5',
'HangulSyllableSyad': '\xec\x83\xab',
'HangulSyllableSyae': '\xec\x84\x80',
'HangulSyllableHalg': '\xed\x95\xa1',
'UnicodeCharacter0x9fc7': '\xe9\xbf\x87',
'CjkUnifiedIdeograph4a92': '\xe4\xaa\x92',
'UnicodeCharacter0xf0c2': '\xef\x83\x82',
'CjkUnifiedIdeograph3b05': '\xe3\xac\x85',
'CjkUnifiedIdeograph3b04': '\xe3\xac\x84',
'CjkUnifiedIdeograph3b07': '\xe3\xac\x87',
'CjkUnifiedIdeograph3b06': '\xe3\xac\x86',
'CjkUnifiedIdeograph3b01': '\xe3\xac\x81',
'CjkUnifiedIdeograph3b00': '\xe3\xac\x80',
'CjkUnifiedIdeograph3b03': '\xe3\xac\x83',
'CjkUnifiedIdeograph3b02': '\xe3\xac\x82',
'UnicodeCharacter0xe536': '\xee\x94\xb6',
'CjkUnifiedIdeograph3b09': '\xe3\xac\x89',
'CjkUnifiedIdeograph3b08': '\xe3\xac\x88',
'ModifierLetterDownArrowhead': '\xcb\x85',
'CjkUnifiedIdeograph7591': '\xe7\x96\x91',
'CjkUnifiedIdeograph7590': '\xe7\x96\x90',
'CjkUnifiedIdeograph7593': '\xe7\x96\x93',
'CjkUnifiedIdeograph7592': '\xe7\x96\x92',
'CjkUnifiedIdeograph7595': '\xe7\x96\x95',
'CjkUnifiedIdeograph7594': '\xe7\x96\x94',
'CjkUnifiedIdeograph7597': '\xe7\x96\x97',
'CjkUnifiedIdeograph7596': '\xe7\x96\x96',
'CjkRadicalJade': '\xe2\xba\xa9',
'CjkUnifiedIdeograph7598': '\xe7\x96\x98',
'UnicodeCharacter0xf80c': '\xef\xa0\x8c',
'UnicodeCharacter0xf0c1': '\xef\x83\x81',
'LatinCapitalLetterCWithCircumflex': '\xc4\x88',
'CjkUnifiedIdeograph6274': '\xe6\x89\xb4',
'HangulSyllableGwils': '\xea\xb7\x8c',
'HangulSyllableDdyulg': '\xeb\x9c\x95',
'UnicodeCharacter0xe535': '\xee\x94\xb5',
'HangulSyllableDdyulh': '\xeb\x9c\x9b',
'HangulSyllableDdyulm': '\xeb\x9c\x96',
'KannadaLetterBa': '\xe0\xb2\xac',
'CjkUnifiedIdeograph3b0d': '\xe3\xac\x8d',
'CjkUnifiedIdeograph3b0f': '\xe3\xac\x8f',
'CjkUnifiedIdeograph3b0a': '\xe3\xac\x8a',
'CjkUnifiedIdeograph3b0c': '\xe3\xac\x8c',
'CjkUnifiedIdeograph3b0b': '\xe3\xac\x8b',
'UnicodeCharacter0xf0c0': '\xef\x83\x80',
'BrokenCircleWithNorthwestArrow': '\xe2\x8e\x8b',
'AplFunctionalSymbolDownwardsVane': '\xe2\x8d\x96',
'UnicodeCharacter0xe0bd': '\xee\x82\xbd',
'YiSyllableGox': '\xea\x87\xab',
'ModifierLetterSmallNWithLeftHook': '\xe1\xb6\xae',
'UnicodeCharacter0xe534': '\xee\x94\xb4',
'YiSyllableGop': '\xea\x87\xad',
'HangulSyllableTeolb': '\xed\x84\xbb',
'YiSyllableGot': '\xea\x87\xaa',
'UnicodeCharacter0xde46': '\xed\xb9\x86',
'CjkUnifiedIdeograph759a': '\xe7\x96\x9a',
'CjkUnifiedIdeograph759c': '\xe7\x96\x9c',
'CjkUnifiedIdeograph759b': '\xe7\x96\x9b',
'CjkUnifiedIdeograph759e': '\xe7\x96\x9e',
'CjkUnifiedIdeograph759d': '\xe7\x96\x9d',
'CjkUnifiedIdeograph759f': '\xe7\x96\x9f',
'HangulSyllableCag': '\xec\xb0\xa9',
'UnicodeCharacter0xddf7': '\xed\xb7\xb7',
'ArabicLetterThehInitialForm': '\xef\xba\x9b',
'CjkUnifiedIdeograph7ffe': '\xe7\xbf\xbe',
'HangulSyllableGeols': '\xea\xb1\xbc',
'HangulSyllableRogs': '\xeb\xa1\x9f',
'UnicodeCharacter0xe76e': '\xee\x9d\xae',
'UnicodeCharacter0x16f9': '\xe1\x9b\xb9',
'UnicodeCharacter0x16f8': '\xe1\x9b\xb8',
'UnicodeCharacter0x16f7': '\xe1\x9b\xb7',
'UnicodeCharacter0x16f6': '\xe1\x9b\xb6',
'UnicodeCharacter0x16f5': '\xe1\x9b\xb5',
'UnicodeCharacter0x16f4': '\xe1\x9b\xb4',
'UnicodeCharacter0x16f3': '\xe1\x9b\xb3',
'UnicodeCharacter0x16f2': '\xe1\x9b\xb2',
'UnicodeCharacter0x16f1': '\xe1\x9b\xb1',
'UnicodeCharacter0xe76c': '\xee\x9d\xac',
'CjkUnifiedIdeograph6a5b': '\xe6\xa9\x9b',
'KatakanaLetterWe': '\xe3\x83\xb1',
'HangulSyllableKigs': '\xed\x82\xa7',
'KatakanaLetterWa': '\xe3\x83\xaf',
'CjkUnifiedIdeograph6a5c': '\xe6\xa9\x9c',
'KatakanaLetterWo': '\xe3\x83\xb2',
'MyanmarConsonantSignMonMedialNa': '\xe1\x81\x9e',
'KatakanaLetterWi': '\xe3\x83\xb0',
'RunicLetterShorttwignaudN': '\xe1\x9a\xbf',
'CjkUnifiedIdeograph7d3f': '\xe7\xb4\xbf',
'UnicodeCharacter0xe5cf': '\xee\x97\x8f',
'CjkUnifiedIdeograph7d3d': '\xe7\xb4\xbd',
'CjkUnifiedIdeograph6a5a': '\xe6\xa9\x9a',
'CjkUnifiedIdeograph7d3b': '\xe7\xb4\xbb',
'CjkUnifiedIdeograph7d3c': '\xe7\xb4\xbc',
'CjkUnifiedIdeograph6f39': '\xe6\xbc\xb9',
'CjkUnifiedIdeograph6f38': '\xe6\xbc\xb8',
'CjkUnifiedIdeograph6f37': '\xe6\xbc\xb7',
'BoxDrawingsVerticalLightAndRightHeavy': '\xe2\x94\x9d',
'CjkUnifiedIdeograph6f35': '\xe6\xbc\xb5',
'CjkUnifiedIdeograph6f34': '\xe6\xbc\xb4',
'CjkUnifiedIdeograph6f33': '\xe6\xbc\xb3',
'CjkUnifiedIdeograph6f32': '\xe6\xbc\xb2',
'CjkUnifiedIdeograph6f31': '\xe6\xbc\xb1',
'CjkUnifiedIdeograph6f30': '\xe6\xbc\xb0',
'UnicodeCharacter0xe531': '\xee\x94\xb1',
'HangulSyllableHeolm': '\xed\x97\x92',
'UnicodeCharacter0xe7c2': '\xee\x9f\x82',
'UnicodeCharacter0xddf5': '\xed\xb7\xb5',
'HangulSyllableSoebs': '\xec\x87\xb2',
'CjkUnifiedIdeograph6a5e': '\xe6\xa9\x9e',
'CjkUnifiedIdeograph7f35': '\xe7\xbc\xb5',
'HangulSyllableUlm': '\xec\x9a\xba',
'BraillePatternDots3458': '\xe2\xa2\x9c',
'UnicodeCharacter0xe7c7': '\xee\x9f\x87',
'BraillePatternDots123478': '\xe2\xa3\x8f',
'UnicodeCharacter0xe7c6': '\xee\x9f\x86',
'BraillePatternDots3457': '\xe2\xa1\x9c',
'BraillePatternDots3456': '\xe2\xa0\xbc',
'UnicodeCharacter0xe530': '\xee\x94\xb0',
'CircledIdeographPrint': '\xe3\x8a\x9e',
'UnicodeCharacter0xe7c9': '\xee\x9f\x89',
'CjkUnifiedIdeograph9ef4': '\xe9\xbb\xb4',
'CanadianSyllabicsCarrierMee': '\xe1\x98\x8b',
'UnicodeCharacter0xe7c8': '\xee\x9f\x88',
'UnicodeCharacter0x16ff': '\xe1\x9b\xbf',
'UnicodeCharacter0x16fe': '\xe1\x9b\xbe',
'UnicodeCharacter0x16fd': '\xe1\x9b\xbd',
'UnicodeCharacter0x16fc': '\xe1\x9b\xbc',
'UnicodeCharacter0x16fb': '\xe1\x9b\xbb',
'UnicodeCharacter0x16fa': '\xe1\x9b\xba',
'HangulSyllableUlh': '\xec\x9a\xbf',
'UnicodeCharacter0x244f': '\xe2\x91\x8f',
'UnicodeCharacter0x244d': '\xe2\x91\x8d',
'UnicodeCharacter0x244e': '\xe2\x91\x8e',
'UnicodeCharacter0x244b': '\xe2\x91\x8b',
'UnicodeCharacter0x244c': '\xe2\x91\x8c',
'HangulSyllableCwak': '\xec\xb4\xbc',
'CjkUnifiedIdeograph7d36': '\xe7\xb4\xb6',
'CjkUnifiedIdeograph7d37': '\xe7\xb4\xb7',
'CjkUnifiedIdeograph7d34': '\xe7\xb4\xb4',
'CjkUnifiedIdeograph7d35': '\xe7\xb4\xb5',
'CjkUnifiedIdeograph7d32': '\xe7\xb4\xb2',
'CjkUnifiedIdeograph7d33': '\xe7\xb4\xb3',
'CjkUnifiedIdeograph7d30': '\xe7\xb4\xb0',
'CjkUnifiedIdeograph7d31': '\xe7\xb4\xb1',
'CjkUnifiedIdeograph6f3f': '\xe6\xbc\xbf',
'CjkUnifiedIdeograph6f3e': '\xe6\xbc\xbe',
'CjkUnifiedIdeograph6f3d': '\xe6\xbc\xbd',
'CjkUnifiedIdeograph6f3c': '\xe6\xbc\xbc',
'CjkUnifiedIdeograph6f3b': '\xe6\xbc\xbb',
'CjkUnifiedIdeograph6f3a': '\xe6\xbc\xba',
'CjkUnifiedIdeograph7d39': '\xe7\xb4\xb9',
'Caret': '\xe2\x80\xb8',
'HangulSyllableCwal': '\xec\xb4\xac',
'HangulSyllableBbyis': '\xec\x82\x87',
'UnicodeCharacter0xddf8': '\xed\xb7\xb8',
'CjkUnifiedIdeograph4e9d': '\xe4\xba\x9d',
'CjkUnifiedIdeograph4e9e': '\xe4\xba\x9e',
'CjkUnifiedIdeograph4e9f': '\xe4\xba\x9f',
'CjkUnifiedIdeograph4e9a': '\xe4\xba\x9a',
'CjkUnifiedIdeograph4e9b': '\xe4\xba\x9b',
'CjkUnifiedIdeograph4e9c': '\xe4\xba\x9c',
'CjkUnifiedIdeograph56f9': '\xe5\x9b\xb9',
'BugineseLetterLa': '\xe1\xa8\x92',
'YiSyllableHly': '\xea\x86\xb3',
'UnicodeCharacter0xe152': '\xee\x85\x92',
'CjkUnifiedIdeograph56f1': '\xe5\x9b\xb1',
'CjkUnifiedIdeograph56f0': '\xe5\x9b\xb0',
'CjkUnifiedIdeograph56f3': '\xe5\x9b\xb3',
'CjkUnifiedIdeograph56f2': '\xe5\x9b\xb2',
'CjkUnifiedIdeograph56f5': '\xe5\x9b\xb5',
'CjkUnifiedIdeograph56f4': '\xe5\x9b\xb4',
'CjkUnifiedIdeograph56f7': '\xe5\x9b\xb7',
'CjkUnifiedIdeograph56f6': '\xe5\x9b\xb6',
'HangulSyllableBbwegg': '\xec\x80\x86',
'CjkUnifiedIdeograph9616': '\xe9\x98\x96',
'HangulSyllablePeong': '\xed\x8e\x91',
'CjkUnifiedIdeograph4e98': '\xe4\xba\x98',
'CjkUnifiedIdeograph4e99': '\xe4\xba\x99',
'CjkUnifiedIdeograph4e94': '\xe4\xba\x94',
'CjkUnifiedIdeograph4e95': '\xe4\xba\x95',
'CjkUnifiedIdeograph4e96': '\xe4\xba\x96',
'CjkUnifiedIdeograph4e97': '\xe4\xba\x97',
'CjkUnifiedIdeograph4e90': '\xe4\xba\x90',
'CjkUnifiedIdeograph4e91': '\xe4\xba\x91',
'CjkUnifiedIdeograph4e92': '\xe4\xba\x92',
'CjkUnifiedIdeograph4e93': '\xe4\xba\x93',
'UnicodeCharacter0xdc48': '\xed\xb1\x88',
'NegatedDoubleVerticalBarDoubleRightTurnstile': '\xe2\x8a\xaf',
'HangulSyllableDyiss': '\xeb\x94\x8c',
'MyanmarLetterBha': '\xe1\x80\x98',
'SquareHuiito': '\xe3\x8c\xb3',
'UnicodeCharacter0xe580': '\xee\x96\x80',
'UnicodeCharacter0xf23c': '\xef\x88\xbc',
'UnicodeCharacter0xf039': '\xef\x80\xb9',
'UnicodeCharacter0xf038': '\xef\x80\xb8',
'CjkUnifiedIdeograph56fa': '\xe5\x9b\xba',
'UnicodeCharacter0xf032': '\xef\x80\xb2',
'CjkUnifiedIdeograph56fc': '\xe5\x9b\xbc',
'CjkUnifiedIdeograph56fb': '\xe5\x9b\xbb',
'ParallelWithTildeOperator': '\xe2\xab\xb3',
'CjkUnifiedIdeograph56fd': '\xe5\x9b\xbd',
'UnicodeCharacter0xf035': '\xef\x80\xb5',
'IdeographicClosingMark': '\xe3\x80\x86',
'HangulSyllableBbwan': '\xeb\xbd\xa0',
'HangulSyllableHoem': '\xed\x9a\x9c',
'HangulSyllableBbwal': '\xeb\xbd\xa4',
'HangulSyllableBbwam': '\xeb\xbd\xac',
'HangulSyllableBbwaj': '\xeb\xbd\xb2',
'HangulSyllableBbwak': '\xeb\xbd\xb4',
'HangulSyllableBbwah': '\xeb\xbd\xb7',
'HangulSyllableHoel': '\xed\x9a\x94',
'HangulSyllableBbwag': '\xeb\xbd\x9d',
'HangulSyllableBbwad': '\xeb\xbd\xa3',
'HangulSyllableBbwae': '\xeb\xbd\xb8',
'HangulSyllableBbwab': '\xeb\xbd\xad',
'CjkUnifiedIdeograph86c7': '\xe8\x9b\x87',
'UnicodeCharacter0xabe2': '\xea\xaf\xa2',
'UnicodeCharacter0xabe3': '\xea\xaf\xa3',
'UnicodeCharacter0xabe0': '\xea\xaf\xa0',
'CjkUnifiedIdeograph86c6': '\xe8\x9b\x86',
'UnicodeCharacter0xabe6': '\xea\xaf\xa6',
'UnicodeCharacter0xabe7': '\xea\xaf\xa7',
'UnicodeCharacter0xabe4': '\xea\xaf\xa4',
'SupersetBesideSubset': '\xe2\xab\x97',
'UnicodeCharacter0xabe8': '\xea\xaf\xa8',
'UnicodeCharacter0xabe9': '\xea\xaf\xa9',
'HangulSyllableBbwas': '\xeb\xbd\xaf',
'HangulSyllableBbwap': '\xeb\xbd\xb6',
'HangulSyllableHoeh': '\xed\x9a\xa7',
'UnicodeCharacter0xe539': '\xee\x94\xb9',
'NorthWestDoubleArrow': '\xe2\x87\x96',
'UnicodeCharacter0xd65': '\xe0\xb5\xa5',
'UnicodeCharacter0xd64': '\xe0\xb5\xa4',
'HanunooLetterHa': '\xe1\x9c\xb1',
'ArabicDammatanIsolatedForm': '\xef\xb9\xb2',
'CjkUnifiedIdeograph7eac': '\xe7\xba\xac',
'CjkUnifiedIdeograph7eab': '\xe7\xba\xab',
'CjkUnifiedIdeograph7eae': '\xe7\xba\xae',
'CjkUnifiedIdeograph7ead': '\xe7\xba\xad',
'EthiopicSyllableBee': '\xe1\x89\xa4',
'CjkUnifiedIdeograph7eaf': '\xe7\xba\xaf',
'VaiSyllableDee': '\xea\x94\x94',
'HangulSyllableHoed': '\xed\x9a\x93',
'HangulSyllableHoeg': '\xed\x9a\x8d',
'UnicodeCharacter0xe538': '\xee\x94\xb8',
'UnicodeCharacter0xabeb': '\xea\xaf\xab',
'UnicodeCharacter0xabec': '\xea\xaf\xac',
'C': '\xce\xa7',
'UnicodeCharacter0xabea': '\xea\xaf\xaa',
'UnicodeCharacter0xabef': '\xea\xaf\xaf',
'LatinSmallLetterYWithDiaeresis': '\xc3\xbf',
'UnicodeCharacter0xabed': '\xea\xaf\xad',
'UnicodeCharacter0xabee': '\xea\xaf\xae',
'BoxDrawingsLightLeft': '\xe2\x95\xb4',
'CjkUnifiedIdeograph86c8': '\xe8\x9b\x88',
'UnicodeCharacter0x9fcd': '\xe9\xbf\x8d',
'SinhalaLetterEyanna': '\xe0\xb6\x91',
'HangulSyllableCwas': '\xec\xb4\xb7',
'HangulSyllableDuj': '\xeb\x91\xa6',
'CjkUnifiedIdeograph868c': '\xe8\x9a\x8c',
'HangulSyllableCas': '\xec\xb0\xbb',
'YiSyllableVyrx': '\xea\x83\xbe',
'UnicodeCharacter0xf4a4': '\xef\x92\xa4',
'CjkUnifiedIdeograph7ea9': '\xe7\xba\xa9',
'CjkUnifiedIdeograph7ea8': '\xe7\xba\xa8',
'CjkUnifiedIdeograph7ea1': '\xe7\xba\xa1',
'CjkUnifiedIdeograph7ea0': '\xe7\xba\xa0',
'TopArcAnticlockwiseArrowWithPlus': '\xe2\xa4\xbd',
'CjkUnifiedIdeograph7ea2': '\xe7\xba\xa2',
'CjkUnifiedIdeograph7ea5': '\xe7\xba\xa5',
'CjkUnifiedIdeograph7ea4': '\xe7\xba\xa4',
'CjkUnifiedIdeograph7ea7': '\xe7\xba\xa7',
'CjkUnifiedIdeograph7ea6': '\xe7\xba\xa6',
'UnicodeCharacter0xf23b': '\xef\x88\xbb',
'LessthanEqualToOrGreaterthan': '\xe2\x8b\x9a',
'FullwidthLeftWhiteParenthesis': '\xef\xbd\x9f',
'TamilDigitThree': '\xe0\xaf\xa9',
'HangulSyllableByaelt': '\xeb\xb1\xb5',
'HangulSyllableHoet': '\xed\x9a\xa5',
'CjkUnifiedIdeograph7fab': '\xe7\xbe\xab',
'CjkUnifiedIdeograph7fac': '\xe7\xbe\xac',
'UnicodeCharacter0xe71a': '\xee\x9c\x9a',
'CjkUnifiedIdeograph7faa': '\xe7\xbe\xaa',
'CjkUnifiedIdeograph7faf': '\xe7\xbe\xaf',
'UnicodeCharacter0xe71f': '\xee\x9c\x9f',
'CjkUnifiedIdeograph7fad': '\xe7\xbe\xad',
'CjkUnifiedIdeograph7fae': '\xe7\xbe\xae',
'UnicodeCharacter0xf860': '\xef\xa1\xa0',
'CjkUnifiedIdeograph9f78': '\xe9\xbd\xb8',
'CjkUnifiedIdeograph9f79': '\xe9\xbd\xb9',
'CombiningCyrillicLetterYu': '\xe2\xb7\xbb',
'UnicodeCharacter0xde43': '\xed\xb9\x83',
'CjkUnifiedIdeograph9f72': '\xe9\xbd\xb2',
'CjkUnifiedIdeograph806a': '\xe8\x81\xaa',
'CjkUnifiedIdeograph806b': '\xe8\x81\xab',
'ThaiCharacterSaraUee': '\xe0\xb8\xb7',
'CjkUnifiedIdeograph806d': '\xe8\x81\xad',
'CjkUnifiedIdeograph806e': '\xe8\x81\xae',
'CjkUnifiedIdeograph806f': '\xe8\x81\xaf',
'CjkUnifiedIdeograph9f75': '\xe9\xbd\xb5',
'UnicodeCharacter0xb84': '\xe0\xae\x84',
'UnicodeCharacter0xb81': '\xe0\xae\x81',
'UnicodeCharacter0xb80': '\xe0\xae\x80',
'CombiningCyrillicLetterSha': '\xe2\xb7\xb2',
'UnicodeCharacter0xf864': '\xef\xa1\xa4',
'UnicodeCharacter0x11fa': '\xe1\x87\xba',
'iota': '\xce\xb9',
'CjkUnifiedIdeograph7fa8': '\xe7\xbe\xa8',
'CjkUnifiedIdeograph7fa9': '\xe7\xbe\xa9',
'HangulJungseongYayo': '\xe1\x85\xb9',
'UnicodeCharacter0xf1a9': '\xef\x86\xa9',
'CjkUnifiedIdeograph7fa2': '\xe7\xbe\xa2',
'CjkUnifiedIdeograph7fa3': '\xe7\xbe\xa3',
'OlChikiDigitZero': '\xe1\xb1\x90',
'CjkUnifiedIdeograph7fa1': '\xe7\xbe\xa1',
'CjkUnifiedIdeograph7fa6': '\xe7\xbe\xa6',
'CjkUnifiedIdeograph7fa7': '\xe7\xbe\xa7',
'CjkUnifiedIdeograph7fa4': '\xe7\xbe\xa4',
'CjkUnifiedIdeograph7fa5': '\xe7\xbe\xa5',
'UnicodeCharacter0xf868': '\xef\xa1\xa8',
'HangulSyllableGeub': '\xea\xb8\x89',
'UnicodeCharacter0xf869': '\xef\xa1\xa9',
'HangulSyllablePon': '\xed\x8f\xb0',
'HangulSyllableDdad': '\xeb\x94\xb7',
'UnicodeCharacter0xb8d': '\xe0\xae\x8d',
'Ch': '\xce\xa7',
'UnicodeCharacter0xb8c': '\xe0\xae\x8c',
'UnicodeCharacter0xb8b': '\xe0\xae\x8b',
'HangulSyllableSwebs': '\xec\x89\xa2',
'HangulSyllableJan': '\xec\x9e\x94',
'VerticalKanaRepeatWithVoicedSoundMark': '\xe3\x80\xb2',
'CjkUnifiedIdeograph8068': '\xe8\x81\xa8',
'CjkUnifiedIdeograph8069': '\xe8\x81\xa9',
'HangulSyllableGeuc': '\xea\xb8\x8f',
'GreekCapitalLetterIotaWithPsili': '\xe1\xbc\xb8',
'CjkUnifiedIdeograph8060': '\xe8\x81\xa0',
'CjkUnifiedIdeograph8061': '\xe8\x81\xa1',
'CjkUnifiedIdeograph8062': '\xe8\x81\xa2',
'CjkUnifiedIdeograph8063': '\xe8\x81\xa3',
'CjkUnifiedIdeograph8064': '\xe8\x81\xa4',
'CjkUnifiedIdeograph8065': '\xe8\x81\xa5',
'CjkUnifiedIdeograph8066': '\xe8\x81\xa6',
'CjkUnifiedIdeograph8067': '\xe8\x81\xa7',
'YiSyllableAx': '\xea\x80\x89',
'UnicodeCharacter0xe6bd': '\xee\x9a\xbd',
'YiSyllableAp': '\xea\x80\x8b',
'UnicodeCharacter0xf2ae': '\xef\x8a\xae',
'UnicodeCharacter0xf2ad': '\xef\x8a\xad',
'GujaratiLetterKha': '\xe0\xaa\x96',
'YiSyllableAt': '\xea\x80\x88',
'UnicodeCharacter0xf2aa': '\xef\x8a\xaa',
'SaurashtraLetterGha': '\xea\xa2\x95',
't': '\xcf\x84',
'LatinSmallLetterKWithPalatalHook': '\xe1\xb6\x84',
'HangulSyllableHwebs': '\xed\x9c\x8e',
'ArabicLigatureSheenWithHehInitialForm': '\xef\xb4\xb2',
'ArabicLigatureLamWithAlefWithHamzaAboveIsolatedForm': '\xef\xbb\xb7',
'UnicodeCharacter0xd9f7': '\xed\xa7\xb7',
'UnicodeCharacter0xe6bb': '\xee\x9a\xbb',
'CanadianSyllabicsCarrierTlhe': '\xe1\x98\xb6',
'CanadianSyllabicsCarrierTlha': '\xe1\x98\xb9',
'CanadianSyllabicsCarrierTlho': '\xe1\x98\xb5',
'MyanmarShanDigitNine': '\xe1\x82\x99',
'CanadianSyllabicsCarrierTlhi': '\xe1\x98\xb8',
'CanadianSyllabicsCarrierTlhu': '\xe1\x98\xb4',
'UnicodeCharacter0xf319': '\xef\x8c\x99',
'UnicodeCharacter0xf2a9': '\xef\x8a\xa9',
'UnicodeCharacter0xe768': '\xee\x9d\xa8',
'UnicodeCharacter0xf2a7': '\xef\x8a\xa7',
'UnicodeCharacter0xf2a6': '\xef\x8a\xa6',
'UnicodeCharacter0xf2a5': '\xef\x8a\xa5',
'UnicodeCharacter0xf2a4': '\xef\x8a\xa4',
'UnicodeCharacter0xf2a3': '\xef\x8a\xa3',
'CjkUnifiedIdeograph5c12': '\xe5\xb0\x92',
'UnicodeCharacter0xe6ba': '\xee\x9a\xba',
'UnicodeCharacter0xf2a0': '\xef\x8a\xa0',
'UnicodeCharacter0xffc0': '\xef\xbf\x80',
'GreekCapitalLetterOmegaWithVaria': '\xe1\xbf\xba',
'UnicodeCharacter0xf312': '\xef\x8c\x92',
'ArabicLigatureHehWithYehIsolatedForm': '\xef\xb1\x94',
'PhilippineSinglePunctuation': '\xe1\x9c\xb5',
'CjkUnifiedIdeograph89cf': '\xe8\xa7\x8f',
'UnicodeCharacter0xf311': '\xef\x8c\x91',
'CjkUnifiedIdeograph89cd': '\xe8\xa7\x8d',
'CjkUnifiedIdeograph89ce': '\xe8\xa7\x8e',
'NegativeCircledNumberNineteen': '\xe2\x93\xb3',
'CjkUnifiedIdeograph961e': '\xe9\x98\x9e',
'CjkUnifiedIdeograph89cb': '\xe8\xa7\x8b',
'LatinSmallLetterSWithCaron': '\xc5\xa1',
'CjkUnifiedIdeograph89cc': '\xe8\xa7\x8c',
'HangulSyllablePeogg': '\xed\x8d\xbe',
'NegativeCircledNumberSeventeen': '\xe2\x93\xb1',
'CjkUnifiedIdeograph5fb9': '\xe5\xbe\xb9',
'CanadianSyllabicsMwaa': '\xe1\x92\xb8',
'CjkUnifiedIdeograph5fb5': '\xe5\xbe\xb5',
'CjkUnifiedIdeograph5fb4': '\xe5\xbe\xb4',
'CjkUnifiedIdeograph5fb7': '\xe5\xbe\xb7',
'CjkUnifiedIdeograph5fb6': '\xe5\xbe\xb6',
'CjkUnifiedIdeograph5fb1': '\xe5\xbe\xb1',
'CjkUnifiedIdeograph5fb0': '\xe5\xbe\xb0',
'CjkUnifiedIdeograph5fb3': '\xe5\xbe\xb3',
'CjkUnifiedIdeograph5fb2': '\xe5\xbe\xb2',
'YiSyllableBut': '\xea\x80\xac',
'UnicodeCharacter0xfff5': '\xef\xbf\xb5',
'YiSyllableBur': '\xea\x80\xb1',
'YiSyllableBup': '\xea\x80\xaf',
'YiSyllableZzip': '\xea\x8b\x93',
'GujaratiVowelSignUu': '\xe0\xab\x82',
'SquareWithOrthogonalCrosshatchFill': '\xe2\x96\xa6',
'HangulSyllableJjyuh': '\xec\xaf\x93',
'NewTaiLueLetterFinalNg': '\xe1\xa7\x82',
'MongolianFullStop': '\xe1\xa0\x83',
'HangulSyllableJjyuk': '\xec\xaf\x90',
'HangulSyllableJjyul': '\xec\xaf\x80',
'HangulSyllableJjyum': '\xec\xaf\x88',
'HangulSyllableJjyun': '\xec\xae\xbc',
'UnicodeCharacter0xf86a': '\xef\xa1\xaa',
'CjkUnifiedIdeograph73ec': '\xe7\x8f\xac',
'HangulSyllableJjyub': '\xec\xaf\x89',
'HangulSyllableJjyuc': '\xec\xaf\x8f',
'HangulSyllableJjyud': '\xec\xae\xbf',
'CjkUnifiedIdeograph73eb': '\xe7\x8f\xab',
'UnicodeCharacter0xd9f6': '\xed\xa7\xb6',
'HangulSyllableBaeng': '\xeb\xb1\x85',
'KhmerSignMuusikatoan': '\xe1\x9f\x89',
'HangulSyllableJjyup': '\xec\xaf\x92',
'HangulSyllableJjyus': '\xec\xaf\x8b',
'HangulSyllableJjyut': '\xec\xaf\x91',
'UnicodeCharacter0xf86e': '\xef\xa1\xae',
'HangulSyllableJjwenh': '\xec\xae\x86',
'AnticlockwiseGappedCircleArrow': '\xe2\x9f\xb2',
'HangulSyllableColp': '\xec\xb4\x96',
'BraillePatternDots35678': '\xe2\xa3\xb4',
'LatinCapitalLetterRWithCedilla': '\xc5\x96',
'HangulSyllableJjwenj': '\xec\xae\x85',
'CjkUnifiedIdeograph5fbe': '\xe5\xbe\xbe',
'CjkUnifiedIdeograph5fbd': '\xe5\xbe\xbd',
'HebrewLetterWideTav': '\xef\xac\xa8',
'CjkUnifiedIdeograph5fbf': '\xe5\xbe\xbf',
'CjkUnifiedIdeograph5fba': '\xe5\xbe\xba',
'CjkUnifiedIdeograph5fbc': '\xe5\xbe\xbc',
'CjkUnifiedIdeograph5fbb': '\xe5\xbe\xbb',
'CjkUnifiedIdeograph85da': '\xe8\x97\x9a',
'QuestionExclamationMark': '\xe2\x81\x88',
'NumberEighteenFullStop': '\xe2\x92\x99',
'CjkUnifiedIdeograph5c18': '\xe5\xb0\x98',
'KhmerSymbolPramKoet': '\xe1\xa7\xa5',
'BoxDrawingsDoubleUpAndRight': '\xe2\x95\x9a',
'HangulSyllablePeunh': '\xed\x94\x8a',
'HangulSyllableCon': '\xec\xb4\x8c',
'ArabicLetterThehIsolatedForm': '\xef\xba\x99',
'HangulSyllableCol': '\xec\xb4\x90',
'HangulSyllableCom': '\xec\xb4\x98',
'HangulSyllableCoj': '\xec\xb4\x9e',
'HangulSyllableDdak': '\xeb\x95\x88',
'HangulSyllableCoh': '\xec\xb4\xa3',
'HangulSyllableCog': '\xec\xb4\x89',
'HangulSyllableCod': '\xec\xb4\x8f',
'HangulSyllableCoe': '\xec\xb5\x9c',
'HangulSyllableCob': '\xec\xb4\x99',
'HangulSyllableCoc': '\xec\xb4\x9f',
'HangulSyllableCyuss': '\xec\xb8\x98',
'CjkUnifiedIdeograph8d68': '\xe8\xb5\xa8',
'CjkUnifiedIdeograph8d69': '\xe8\xb5\xa9',
'CjkUnifiedIdeograph8d64': '\xe8\xb5\xa4',
'CjkUnifiedIdeograph8d65': '\xe8\xb5\xa5',
'CjkUnifiedIdeograph8d66': '\xe8\xb5\xa6',
'CjkUnifiedIdeograph8d67': '\xe8\xb5\xa7',
'CjkUnifiedIdeograph8d60': '\xe8\xb5\xa0',
'CopticCapitalLetterMi': '\xe2\xb2\x98',
'CjkUnifiedIdeograph8d62': '\xe8\xb5\xa2',
'CjkUnifiedIdeograph8d63': '\xe8\xb5\xa3',
'HangulSyllableDdah': '\xeb\x95\x8b',
'LepchaLetterTa': '\xe1\xb0\x8a',
'GreekCapitalLetterIotaWithPsiliAndPerispomeni': '\xe1\xbc\xbe',
'HangulSyllableGwaeng': '\xea\xb4\xad',
'CjkUnifiedIdeograph82b8': '\xe8\x8a\xb8',
'LongLeftwardsArrowFromBar': '\xe2\x9f\xbb',
'HangulSyllableRyaegg': '\xeb\x9f\x92',
'CjkUnifiedIdeograph8d6d': '\xe8\xb5\xad',
'CjkUnifiedIdeograph8d6e': '\xe8\xb5\xae',
'CjkUnifiedIdeograph8d6f': '\xe8\xb5\xaf',
'ArabicLigatureYehWithHamzaAboveWithJeemInitialForm': '\xef\xb2\x97',
'BoxDrawingsVerticalLightAndLeftHeavy': '\xe2\x94\xa5',
'CjkUnifiedIdeograph8d6b': '\xe8\xb5\xab',
'CjkUnifiedIdeograph8d6c': '\xe8\xb5\xac',
'GreekCapitalLetterUpsilonWithOxia': '\xe1\xbf\xab',
'CjkUnifiedIdeograph7baa': '\xe7\xae\xaa',
'HangulSyllableGeut': '\xea\xb8\x91',
'CjkUnifiedIdeograph9343': '\xe9\x8d\x83',
'UnicodeCharacter0xe159': '\xee\x85\x99',
'HangulSyllableKwij': '\xed\x81\x8a',
'UnicodeCharacter0xf370': '\xef\x8d\xb0',
'UnicodeCharacter0xe2bc': '\xee\x8a\xbc',
'CjkUnifiedIdeograph97d8': '\xe9\x9f\x98',
'LatinSmallLetterEth': '\xc3\xb0',
'UnicodeCharacter0xf374': '\xef\x8d\xb4',
'WavyLine': '\xe2\x8c\x87',
'UnicodeCharacter0xf376': '\xef\x8d\xb6',
'UnicodeCharacter0xf377': '\xef\x8d\xb7',
'CjkUnifiedIdeograph97d2': '\xe9\x9f\x92',
'CjkUnifiedIdeograph97d3': '\xe9\x9f\x93',
'CjkUnifiedIdeograph97d0': '\xe9\x9f\x90',
'CjkUnifiedIdeograph97d1': '\xe9\x9f\x91',
'CjkUnifiedIdeograph97d6': '\xe9\x9f\x96',
'CjkUnifiedIdeograph97d7': '\xe9\x9f\x97',
'CjkUnifiedIdeograph97d4': '\xe9\x9f\x94',
'LatinCapitalLetterNWithGrave': '\xc7\xb8',
'LatinSmallLetterAWithTilde': '\xc3\xa3',
'KangxiRadicalGrass': '\xe2\xbe\x8b',
'HangulSyllableSswaek': '\xec\x90\xa8',
'LatinCapitalLetterUWithDiaeresisAndCaron': '\xc7\x99',
'GreekCapitalLetterUpsilonWithDialytika': '\xce\xab',
'UnicodeCharacter0xe2bb': '\xee\x8a\xbb',
'HangulSyllableJyeolt': '\xec\xa1\x85',
'CjkUnifiedIdeograph906d': '\xe9\x81\xad',
'HangulSyllableJyeolp': '\xec\xa1\x86',
'HangulSyllableJyeols': '\xec\xa1\x84',
'HangulSyllableJyeolm': '\xec\xa1\x82',
'HangulSyllableGeus': '\xea\xb8\x8b',
'HangulSyllableJyeolh': '\xec\xa1\x87',
'CjkRadicalSimplifiedWalk': '\xe2\xbb\x8c',
'UnicodeCharacter0xe581': '\xee\x96\x81',
'HangulSyllableJyeolg': '\xec\xa1\x81',
'UnicodeCharacter0xe2ba': '\xee\x8a\xba',
'HangulSyllableJyeolb': '\xec\xa1\x83',
'HangulSyllableSyulh': '\xec\x8a\x97',
'BengaliDigitSeven': '\xe0\xa7\xad',
'HangulSyllableSyulm': '\xec\x8a\x92',
'HangulSyllableSyulb': '\xec\x8a\x93',
'Estimates': '\xe2\x89\x99',
'UnicodeCharacter0xe882': '\xee\xa2\x82',
'UnicodeCharacter0xf37a': '\xef\x8d\xba',
'UnicodeCharacter0xf37b': '\xef\x8d\xbb',
'GlagoliticSmallLetterSlovo': '\xe2\xb1\x84',
'UnicodeCharacter0xf37d': '\xef\x8d\xbd',
'ModifierLetterSmallTurnedA': '\xe1\xb5\x84',
'UnicodeCharacter0xf37f': '\xef\x8d\xbf',
'HangulSyllableBung': '\xeb\xb6\x95',
'CjkUnifiedIdeograph97db': '\xe9\x9f\x9b',
'CjkUnifiedIdeograph97dc': '\xe9\x9f\x9c',
'HangulSyllableBunj': '\xeb\xb6\x85',
'CjkUnifiedIdeograph97da': '\xe9\x9f\x9a',
'CjkUnifiedIdeograph97df': '\xe9\x9f\x9f',
'CjkUnifiedIdeograph97dd': '\xe9\x9f\x9d',
'CjkUnifiedIdeograph97de': '\xe9\x9f\x9e',
'CjkRadicalDeath': '\xe2\xba\x9e',
'CjkUnifiedIdeograph8fb4': '\xe8\xbe\xb4',
'CjkUnifiedIdeograph9e31': '\xe9\xb8\xb1',
'CjkUnifiedIdeograph9e30': '\xe9\xb8\xb0',
'CjkUnifiedIdeograph9e33': '\xe9\xb8\xb3',
'CjkUnifiedIdeograph9e32': '\xe9\xb8\xb2',
'CjkUnifiedIdeograph9e35': '\xe9\xb8\xb5',
'CjkUnifiedIdeograph9e34': '\xe9\xb8\xb4',
'CjkUnifiedIdeograph9e37': '\xe9\xb8\xb7',
'CjkUnifiedIdeograph9e36': '\xe9\xb8\xb6',
'CjkUnifiedIdeograph9e39': '\xe9\xb8\xb9',
'CjkUnifiedIdeograph9e38': '\xe9\xb8\xb8',
'UnicodeCharacter0xf047': '\xef\x81\x87',
'UnicodeCharacter0xe5fc': '\xee\x97\xbc',
'EquivalentWithFourDotsAbove': '\xe2\xa9\xb8',
'CjkUnifiedIdeograph7868': '\xe7\xa1\xa8',
'CjkUnifiedIdeograph7861': '\xe7\xa1\xa1',
'CjkUnifiedIdeograph7860': '\xe7\xa1\xa0',
'SupersetWithMultiplicationSignBelow': '\xe2\xab\x82',
'CjkUnifiedIdeograph7862': '\xe7\xa1\xa2',
'Envelope': '\xe2\x9c\x89',
'CjkUnifiedIdeograph7864': '\xe7\xa1\xa4',
'CjkUnifiedIdeograph7867': '\xe7\xa1\xa7',
'CjkUnifiedIdeograph7866': '\xe7\xa1\xa6',
'HangulSyllableInj': '\xec\x9d\xb9',
'HangulSyllableMol': '\xeb\xaa\xb0',
'GeorgianCapitalLetterGhan': '\xe1\x82\xb6',
'CjkUnifiedIdeograph9e3a': '\xe9\xb8\xba',
'UnicodeCharacter0xdc7b': '\xed\xb1\xbb',
'LatinCapitalLetterOWithMacron': '\xc5\x8c',
'CjkUnifiedIdeograph9e3b': '\xe9\xb8\xbb',
'CjkUnifiedIdeograph9e3e': '\xe9\xb8\xbe',
'CjkUnifiedIdeograph9e3d': '\xe9\xb8\xbd',
'CjkUnifiedIdeograph9e3f': '\xe9\xb8\xbf',
'SucceedsAboveSinglelineNotEqualTo': '\xe2\xaa\xb2',
'HangulSyllableDdeot': '\xeb\x96\xb9',
'UnicodeCharacter0xfb07': '\xef\xac\x87',
'ArabicLetterSheenWithDotBelow': '\xdb\xba',
'CjkUnifiedIdeograph89c6': '\xe8\xa7\x86',
'EthiopicTonalMarkKenat': '\xe1\x8e\x95',
'CjkUnifiedIdeograph89c7': '\xe8\xa7\x87',
'sigma': '\xcf\x83',
'HangulSyllableInh': '\xec\x9d\xba',
'CjkUnifiedIdeograph89c4': '\xe8\xa7\x84',
'HangulSyllableCyinh': '\xec\xb9\x82',
'HangulSyllableCyinj': '\xec\xb9\x81',
'CopticCapitalLetterDialectpKapa': '\xe2\xb2\xb8',
'CjkUnifiedIdeograph786a': '\xe7\xa1\xaa',
'KannadaLetterCha': '\xe0\xb2\x9b',
'CjkUnifiedIdeograph786b': '\xe7\xa1\xab',
'BalloonspokedAsterisk': '\xe2\x9d\x89',
'CjkUnifiedIdeograph786d': '\xe7\xa1\xad',
'NegativeCircledNumberTwelve': '\xe2\x93\xac',
'CjkUnifiedIdeograph786f': '\xe7\xa1\xaf',
'NotNormalSubgroupOfOrEqualTo': '\xe2\x8b\xac',
'UmbrellaWithRainDrops': '\xe2\x98\x94',
'CjkUnifiedIdeograph73e9': '\xe7\x8f\xa9',
'ArabicLetterSadWithThreeDotsAbove': '\xda\x9e',
'BraillePatternDots123678': '\xe2\xa3\xa7',
'CjkUnifiedIdeograph73e8': '\xe7\x8f\xa8',
'UnicodeCharacter0xa6e7': '\xea\x9b\xa7',
'HangulSyllablePweolg': '\xed\x92\x9d',
'GreekCapitalLetterIotaWithDialytika': '\xce\xaa',
'LatinSmallLetterXWithDotAbove': '\xe1\xba\x8b',
'HangulSyllableGeo': '\xea\xb1\xb0',
'BlackScissors': '\xe2\x9c\x82',
'TeluguVowelSignI': '\xe0\xb0\xbf',
'TeluguVowelSignO': '\xe0\xb1\x8a',
'SquareNotto': '\xe3\x8c\xa9',
'HangulSyllableHess': '\xed\x97\xb8',
'TeluguVowelSignE': '\xe0\xb1\x86',
'UnicodeCharacter0xa6e4': '\xea\x9b\xa4',
'HangulSyllablePweolh': '\xed\x92\xa3',
'CjkUnifiedIdeograph89c8': '\xe8\xa7\x88',
'TifinaghLetterYarr': '\xe2\xb5\x95',
'CjkUnifiedIdeograph6ace': '\xe6\xab\x8e',
'CjkUnifiedIdeograph6acf': '\xe6\xab\x8f',
'CjkUnifiedIdeograph89c9': '\xe8\xa7\x89',
'TeluguVowelSignU': '\xe0\xb1\x81',
'LeftwardsDoubleArrowFromBar': '\xe2\xa4\x86',
'ArabicLetterKirghizOe': '\xdb\x85',
'HangulLetterYae': '\xe3\x85\x92',
'CanadianSyllabicsAa': '\xe1\x90\x8b',
'CjkUnifiedIdeograph9062': '\xe9\x81\xa2',
'NeitherApproximatelyNorActuallyEqualTo': '\xe2\x89\x87',
'CanadianSyllabicsAi': '\xe1\x90\x9c',
'UnicodeCharacter0xdc2c': '\xed\xb0\xac',
'CjkUnifiedIdeograph8dd0': '\xe8\xb7\x90',
'CjkUnifiedIdeograph9060': '\xe9\x81\xa0',
'UnicodeCharacter0xf8b2': '\xef\xa2\xb2',
'UnicodeCharacter0xe0b1': '\xee\x82\xb1',
'CopticSymbolShimaSima': '\xe2\xb3\xaa',
'CanadianSyllabicsNa': '\xe1\x93\x87',
'CjkUnifiedIdeograph6ac8': '\xe6\xab\x88',
'CjkUnifiedIdeograph6ac9': '\xe6\xab\x89',
'CjkUnifiedIdeograph6ac4': '\xe6\xab\x84',
'LimbuDigitSix': '\xe1\xa5\x8c',
'CjkUnifiedIdeograph6ac6': '\xe6\xab\x86',
'CjkUnifiedIdeograph6ac7': '\xe6\xab\x87',
'CjkUnifiedIdeograph6ac0': '\xe6\xab\x80',
'CjkUnifiedIdeograph6ac1': '\xe6\xab\x81',
'CjkUnifiedIdeograph6ac2': '\xe6\xab\x82',
'CjkUnifiedIdeograph6ac3': '\xe6\xab\x83',
'CjkUnifiedIdeograph70ee': '\xe7\x83\xae',
'div': '\xc3\xb7',
'LatinCapitalLetterIWithTilde': '\xc4\xa8',
'AcuteAccent': '\xc2\xb4',
'CjkUnifiedIdeograph9068': '\xe9\x81\xa8',
'CjkUnifiedIdeograph9528': '\xe9\x94\xa8',
'CjkUnifiedIdeograph9529': '\xe9\x94\xa9',
'CjkUnifiedIdeograph9526': '\xe9\x94\xa6',
'CjkUnifiedIdeograph9527': '\xe9\x94\xa7',
'CjkUnifiedIdeograph9524': '\xe9\x94\xa4',
'CjkUnifiedIdeograph9525': '\xe9\x94\xa5',
'CjkUnifiedIdeograph9522': '\xe9\x94\xa2',
'CjkUnifiedIdeograph9523': '\xe9\x94\xa3',
'CjkUnifiedIdeograph9520': '\xe9\x94\xa0',
'CjkUnifiedIdeograph9521': '\xe9\x94\xa1',
'HangulSyllableCyaebs': '\xec\xb2\x8e',
'CjkUnifiedIdeograph9e52': '\xe9\xb9\x92',
'UnicodeCharacter0xe0b3': '\xee\x82\xb3',
'HangulSyllableHenh': '\xed\x97\xaa',
'CjkRadicalJsimplifiedDragon': '\xe2\xbb\xaf',
'FullwidthLowLine': '\xef\xbc\xbf',
'HangulSyllableRoels': '\xeb\xa2\xbc',
'HangulSyllableMang': '\xeb\xa7\x9d',
'HangulSyllableBoelh': '\xeb\xb5\x97',
'UnicodeCharacter0xa6e0': '\xea\x9b\xa0',
'HangulSyllableBoelg': '\xeb\xb5\x91',
'HangulSyllableManh': '\xeb\xa7\x8e',
'HangulSyllableBoelb': '\xeb\xb5\x93',
'AccountOf': '\xe2\x84\x80',
'CjkUnifiedIdeograph79f8': '\xe7\xa7\xb8',
'CjkUnifiedIdeograph79f9': '\xe7\xa7\xb9',
'UnicodeCharacter0xe0b2': '\xee\x82\xb2',
'CjkUnifiedIdeograph79f4': '\xe7\xa7\xb4',
'CjkUnifiedIdeograph79f5': '\xe7\xa7\xb5',
'CjkUnifiedIdeograph79f6': '\xe7\xa7\xb6',
'CjkUnifiedIdeograph79f7': '\xe7\xa7\xb7',
'CjkUnifiedIdeograph79f0': '\xe7\xa7\xb0',
'CjkUnifiedIdeograph79f1': '\xe7\xa7\xb1',
'CjkUnifiedIdeograph79f2': '\xe7\xa7\xb2',
'CjkUnifiedIdeograph79f3': '\xe7\xa7\xb3',
'TibetanDigitOne': '\xe0\xbc\xa1',
'CanadianSyllabicsRwaa': '\xe1\x95\x8e',
'HangulSyllablePweolm': '\xed\x92\x9e',
'SinhalaSignVisargaya': '\xe0\xb6\x83',
'HangulSyllableNweot': '\xeb\x88\xb9',
'LatinCapitalLetterUWithDiaeresisBelow': '\xe1\xb9\xb2',
'HebrewPointSheva': '\xd6\xb0',
'DaletSymbol': '\xe2\x84\xb8',
'HangulSyllableNeud': '\xeb\x8a\x97',
'UnicodeCharacter0xe0b5': '\xee\x82\xb5',
'GreekSmallDottedLunateSigmaSymbol': '\xcd\xbc',
'HangulLetterPieupcieuc': '\xe3\x85\xb6',
'CjkUnifiedIdeograph952f': '\xe9\x94\xaf',
'TamilDaySign': '\xe0\xaf\xb3',
'CjkUnifiedIdeograph952e': '\xe9\x94\xae',
'CjkUnifiedIdeograph952b': '\xe9\x94\xab',
'CjkUnifiedIdeograph952c': '\xe9\x94\xac',
'CjkUnifiedIdeograph952a': '\xe9\x94\xaa',
'UnicodeCharacter0xee85': '\xee\xba\x85',
'HangulSyllableJjoebs': '\xec\xac\xa2',
'CjkUnifiedIdeograph6fb7': '\xe6\xbe\xb7',
'CjkUnifiedIdeograph79fe': '\xe7\xa7\xbe',
'CjkUnifiedIdeograph79ff': '\xe7\xa7\xbf',
'CjkUnifiedIdeograph79fa': '\xe7\xa7\xba',
'CjkUnifiedIdeograph79fb': '\xe7\xa7\xbb',
'CjkUnifiedIdeograph79fc': '\xe7\xa7\xbc',
'UnicodeCharacter0xe0b4': '\xee\x82\xb4',
'HangulSyllableHoegg': '\xed\x9a\x8e',
'BlackSpadeSuit': '\xe2\x99\xa0',
'MongolianLetterYa': '\xe1\xa0\xb6',
'HangulSyllableCyeos': '\xec\xb3\xa3',
'CyrillicCapitalLetterLha': '\xd4\x94',
'HangulSyllableKwebs': '\xed\x80\xaa',
'HangulSyllableCyeot': '\xec\xb3\xa9',
'HangulSyllableDeun': '\xeb\x93\xa0',
'ArabicLetterHehGoalWithHamzaAbove': '\xdb\x82',
'CjkUnifiedIdeograph70eb': '\xe7\x83\xab',
'CjkUnifiedIdeograph42b2': '\xe4\x8a\xb2',
'CjkUnifiedIdeograph42b3': '\xe4\x8a\xb3',
'CjkUnifiedIdeograph42b0': '\xe4\x8a\xb0',
'CjkUnifiedIdeograph42b1': '\xe4\x8a\xb1',
'CjkUnifiedIdeograph42b6': '\xe4\x8a\xb6',
'CjkUnifiedIdeograph42b7': '\xe4\x8a\xb7',
'CjkUnifiedIdeograph42b4': '\xe4\x8a\xb4',
'CjkUnifiedIdeograph42b5': '\xe4\x8a\xb5',
'HangulSyllableCyeoj': '\xec\xb3\xa6',
'HangulSyllableCyeok': '\xec\xb3\xa8',
'CjkUnifiedIdeograph42b8': '\xe4\x8a\xb8',
'CjkUnifiedIdeograph42b9': '\xe4\x8a\xb9',
'HangulSyllableCyeon': '\xec\xb3\x94',
'HangulJongseongSsangkiyeok': '\xe1\x86\xa9',
'HangulSyllableCyeol': '\xec\xb3\x98',
'HangulSyllableCyeom': '\xec\xb3\xa0',
'UnicodeCharacter0xdf2d': '\xed\xbc\xad',
'UnicodeCharacter0xdf2e': '\xed\xbc\xae',
'CjkUnifiedIdeograph3719': '\xe3\x9c\x99',
'CjkUnifiedIdeograph3718': '\xe3\x9c\x98',
'UnicodeCharacter0xdf2a': '\xed\xbc\xaa',
'UnicodeCharacter0xdf2b': '\xed\xbc\xab',
'HangulSyllableNweod': '\xeb\x88\xa7',
'CjkUnifiedIdeograph3713': '\xe3\x9c\x93',
'CjkUnifiedIdeograph3712': '\xe3\x9c\x92',
'CjkUnifiedIdeograph3711': '\xe3\x9c\x91',
'CjkUnifiedIdeograph3710': '\xe3\x9c\x90',
'CjkUnifiedIdeograph3717': '\xe3\x9c\x97',
'CjkUnifiedIdeograph3716': '\xe3\x9c\x96',
'CjkUnifiedIdeograph3715': '\xe3\x9c\x95',
'CjkUnifiedIdeograph3714': '\xe3\x9c\x94',
'CjkUnifiedIdeograph3bf7': '\xe3\xaf\xb7',
'CjkUnifiedIdeograph3bf6': '\xe3\xaf\xb6',
'CjkUnifiedIdeograph3bf5': '\xe3\xaf\xb5',
'CjkUnifiedIdeograph3bf4': '\xe3\xaf\xb4',
'CjkUnifiedIdeograph3bf3': '\xe3\xaf\xb3',
'CjkUnifiedIdeograph3bf2': '\xe3\xaf\xb2',
'CjkUnifiedIdeograph3bf1': '\xe3\xaf\xb1',
'CjkUnifiedIdeograph3bf0': '\xe3\xaf\xb0',
'CjkUnifiedIdeograph7003': '\xe7\x80\x83',
'CjkUnifiedIdeograph7002': '\xe7\x80\x82',
'CjkUnifiedIdeograph7001': '\xe7\x80\x81',
'CjkUnifiedIdeograph7000': '\xe7\x80\x80',
'CjkUnifiedIdeograph7007': '\xe7\x80\x87',
'CjkUnifiedIdeograph7006': '\xe7\x80\x86',
'CjkUnifiedIdeograph3bf9': '\xe3\xaf\xb9',
'CjkUnifiedIdeograph3bf8': '\xe3\xaf\xb8',
'CjkUnifiedIdeograph42bb': '\xe4\x8a\xbb',
'CjkUnifiedIdeograph42bc': '\xe4\x8a\xbc',
'CjkUnifiedIdeograph42ba': '\xe4\x8a\xba',
'CjkUnifiedIdeograph42bf': '\xe4\x8a\xbf',
'DevanagariLetterVocalicR': '\xe0\xa4\x8b',
'CjkUnifiedIdeograph42bd': '\xe4\x8a\xbd',
'CjkUnifiedIdeograph42be': '\xe4\x8a\xbe',
'HangulSyllableNweob': '\xeb\x88\xb1',
'ChamDigitEight': '\xea\xa9\x98',
'DevanagariLetterVocalicL': '\xe0\xa4\x8c',
'UnicodeCharacter0xee81': '\xee\xba\x81',
'CjkRadicalWestTwo': '\xe2\xbb\x84',
'CjkUnifiedIdeograph3bff': '\xe3\xaf\xbf',
'CjkUnifiedIdeograph3bfe': '\xe3\xaf\xbe',
'CjkUnifiedIdeograph3bfd': '\xe3\xaf\xbd',
'CjkUnifiedIdeograph3bfc': '\xe3\xaf\xbc',
'CjkUnifiedIdeograph3bfb': '\xe3\xaf\xbb',
'CjkUnifiedIdeograph3bfa': '\xe3\xaf\xba',
'HangulSyllablePad': '\xed\x8c\x93',
'CjkUnifiedIdeograph700c': '\xe7\x80\x8c',
'CjkUnifiedIdeograph700b': '\xe7\x80\x8b',
'CjkUnifiedIdeograph700a': '\xe7\x80\x8a',
'HangulSyllablePah': '\xed\x8c\xa7',
'CjkUnifiedIdeograph700f': '\xe7\x80\x8f',
'CjkUnifiedIdeograph700e': '\xe7\x80\x8e',
'CjkUnifiedIdeograph700d': '\xe7\x80\x8d',
'HangulSyllablePas': '\xed\x8c\x9f',
'UnicodeCharacter0xdf25': '\xed\xbc\xa5',
'UnicodeCharacter0xdf26': '\xed\xbc\xa6',
'HangulSyllablePap': '\xed\x8c\xa6',
'RunicLetterSigelLongbranchsolS': '\xe1\x9b\x8b',
'SundaneseLetterGa': '\xe1\xae\x8c',
'UnicodeCharacter0xdf22': '\xed\xbc\xa2',
'HangulSyllablePat': '\xed\x8c\xa5',
'CjkUnifiedIdeograph371c': '\xe3\x9c\x9c',
'CjkUnifiedIdeograph371b': '\xe3\x9c\x9b',
'CjkUnifiedIdeograph371a': '\xe3\x9c\x9a',
'HangulSyllableNweoj': '\xeb\x88\xb6',
'UnicodeCharacter0xde9f': '\xed\xba\x9f',
'CjkUnifiedIdeograph371f': '\xe3\x9c\x9f',
'CjkUnifiedIdeograph371e': '\xe3\x9c\x9e',
'CjkUnifiedIdeograph371d': '\xe3\x9c\x9d',
'LatinCapitalLetterLWithHighStroke': '\xea\x9d\x88',
'CjkUnifiedIdeograph44cb': '\xe4\x93\x8b',
'HangulSyllableCugg': '\xec\xb6\x96',
'HangulSyllableRye': '\xeb\xa1\x80',
'VaiSyllableZhee': '\xea\x94\x99',
'GlagoliticSmallLetterIotatedBigYus': '\xe2\xb1\x99',
'UnicodeCharacter0xf4c8': '\xef\x93\x88',
'UnicodeCharacter0xde2b': '\xed\xb8\xab',
'HangulSyllableTyim': '\xed\x8b\xa4',
'ArabicLetterTehFinalForm': '\xef\xba\x96',
'HangulSyllableCugs': '\xec\xb6\x97',
'EthiopicSyllableShe': '\xe1\x88\xbd',
'BraillePatternDots12': '\xe2\xa0\x83',
'BraillePatternDots13': '\xe2\xa0\x85',
'BraillePatternDots14': '\xe2\xa0\x89',
'BraillePatternDots15': '\xe2\xa0\x91',
'BraillePatternDots16': '\xe2\xa0\xa1',
'BraillePatternDots17': '\xe2\xa1\x81',
'BraillePatternDots18': '\xe2\xa2\x81',
'EthiopicSyllableSho': '\xe1\x88\xbe',
'EthiopicSyllableShi': '\xe1\x88\xba',
'CjkUnifiedIdeograph48dd': '\xe4\xa3\x9d',
'EthiopicSyllableShu': '\xe1\x88\xb9',
'CjkUnifiedIdeograph687c': '\xe6\xa1\xbc',
'CjkUnifiedIdeograph687b': '\xe6\xa1\xbb',
'CjkUnifiedIdeograph687e': '\xe6\xa1\xbe',
'CjkUnifiedIdeograph687d': '\xe6\xa1\xbd',
'SaurashtraLetterSsa': '\xea\xa2\xb0',
'CjkUnifiedIdeograph687f': '\xe6\xa1\xbf',
'GreekSubscriptSmallLetterChi': '\xe1\xb5\xaa',
'UnicodeCharacter0xde2d': '\xed\xb8\xad',
'YiSyllableJuop': '\xea\x8f\xab',
'UnicodeCharacter0xd9c1': '\xed\xa7\x81',
'CjkCompatibilityIdeographf963': '\xef\xa5\xa3',
'EthiopicSyllableJwa': '\xe1\x8c\x87',
'GeorgianLetterTurnedGan': '\xe1\x83\xb9',
'PrescriptionTake': '\xe2\x84\x9e',
'CjkUnifiedIdeograph7eaa': '\xe7\xba\xaa',
'UnicodeCharacter0xdc78': '\xed\xb1\xb8',
'CjkUnifiedIdeograph968b': '\xe9\x9a\x8b',
'CjkUnifiedIdeograph6871': '\xe6\xa1\xb1',
'DownwardsDashedArrow': '\xe2\x87\xa3',
'CjkUnifiedIdeograph6873': '\xe6\xa1\xb3',
'CjkUnifiedIdeograph6872': '\xe6\xa1\xb2',
'CjkUnifiedIdeograph6875': '\xe6\xa1\xb5',
'CjkUnifiedIdeograph6874': '\xe6\xa1\xb4',
'CjkUnifiedIdeograph6877': '\xe6\xa1\xb7',
'CjkUnifiedIdeograph6876': '\xe6\xa1\xb6',
'CjkUnifiedIdeograph6879': '\xe6\xa1\xb9',
'CjkUnifiedIdeograph6878': '\xe6\xa1\xb8',
'CjkCompatibilityIdeographf962': '\xef\xa5\xa2',
'q': '\xce\xb8',
'HangulChoseongTikeutkiyeok': '\xe1\x84\x97',
'CjkUnifiedIdeograph9909': '\xe9\xa4\x89',
'HangulSyllableNenh': '\xeb\x84\xaa',
'ArabicLetterHehDoachashmeeFinalForm': '\xef\xae\xab',
'CjkUnifiedIdeograph346f': '\xe3\x91\xaf',
'UnicodeCharacter0x8f5': '\xe0\xa3\xb5',
'UnicodeCharacter0x8f4': '\xe0\xa3\xb4',
'UnicodeCharacter0x8f7': '\xe0\xa3\xb7',
'UnicodeCharacter0x8f6': '\xe0\xa3\xb6',
'UnicodeCharacter0x8f1': '\xe0\xa3\xb1',
'UnicodeCharacter0x8f0': '\xe0\xa3\xb0',
'DownpointingTriangleWithRightHalfBlack': '\xe2\xa7\xa9',
'UnicodeCharacter0x8f2': '\xe0\xa3\xb2',
'LatinSmallLetterOWithOgonek': '\xc7\xab',
'CjkUnifiedIdeograph6b83': '\xe6\xae\x83',
'CjkUnifiedIdeograph6b80': '\xe6\xae\x80',
'CjkUnifiedIdeograph6b81': '\xe6\xae\x81',
'UnicodeCharacter0x8f9': '\xe0\xa3\xb9',
'UnicodeCharacter0x8f8': '\xe0\xa3\xb8',
'CjkUnifiedIdeograph6b84': '\xe6\xae\x84',
'CjkUnifiedIdeograph6b85': '\xe6\xae\x85',
'UnicodeCharacter0xf3ee': '\xef\x8f\xae',
'LaoLetterFoSung': '\xe0\xba\x9f',
'CjkUnifiedIdeograph8f71': '\xe8\xbd\xb1',
'UnicodeCharacter0xf7c5': '\xef\x9f\x85',
'CjkUnifiedIdeograph4573': '\xe4\x95\xb3',
'UnicodeCharacter0xf3ea': '\xef\x8f\xaa',
'CjkUnifiedIdeograph519e': '\xe5\x86\x9e',
'CjkUnifiedIdeograph4571': '\xe4\x95\xb1',
'CjkUnifiedIdeograph519c': '\xe5\x86\x9c',
'UnicodeCharacter0xaacb': '\xea\xab\x8b',
'CjkUnifiedIdeograph4577': '\xe4\x95\xb7',
'HangulSyllableTaelt': '\xed\x83\xa9',
'CjkCompatibilityIdeographfad2': '\xef\xab\x92',
'UnicodeCharacter0xfade': '\xef\xab\x9e',
'CjkUnifiedIdeograph7611': '\xe7\x98\x91',
'EthiopicSyllableLee': '\xe1\x88\x8c',
'CjkUnifiedIdeograph519a': '\xe5\x86\x9a',
'UnicodeCharacter0xfada': '\xef\xab\x9a',
'HangulChoseongPieupsios': '\xe1\x84\xa1',
'ArabicLigatureHahWithMeemWithAlefMaksuraFinalForm': '\xef\xb5\x9b',
'CjkUnifiedIdeograph8f76': '\xe8\xbd\xb6',
'HangulSyllableYeoss': '\xec\x98\x80',
'UnicodeCharacter0x8fe': '\xe0\xa3\xbe',
'UnicodeCharacter0x8fd': '\xe0\xa3\xbd',
'UnicodeCharacter0x8ff': '\xe0\xa3\xbf',
'UnicodeCharacter0x8fa': '\xe0\xa3\xba',
'UnicodeCharacter0x8fc': '\xe0\xa3\xbc',
'UnicodeCharacter0x8fb': '\xe0\xa3\xbb',
'CjkUnifiedIdeograph6b8b': '\xe6\xae\x8b',
'CjkUnifiedIdeograph6b8c': '\xe6\xae\x8c',
'CjkUnifiedIdeograph6b8a': '\xe6\xae\x8a',
'CjkUnifiedIdeograph6b8f': '\xe6\xae\x8f',
'CjkUnifiedIdeograph6b8d': '\xe6\xae\x8d',
'CjkUnifiedIdeograph6b8e': '\xe6\xae\x8e',
'HangulSyllableKyalg': '\xec\xba\xb5',
'GreekSmallLetterAlphaWithDasiaAndPerispomeniAndYpogegrammeni': '\xe1\xbe\x87',
'HangulSyllableKyalb': '\xec\xba\xb7',
'HangulSyllableKyalm': '\xec\xba\xb6',
'HangulSyllableSseogg': '\xec\x8d\xaa',
'HangulSyllableKyalh': '\xec\xba\xbb',
'HangulSyllableKyalt': '\xec\xba\xb9',
'HangulSyllableKyals': '\xec\xba\xb8',
'CyrillicCapitalLetterKje': '\xd0\x8c',
'UnicodeCharacter0xeb4b': '\xee\xad\x8b',
'CjkUnifiedIdeograph8683': '\xe8\x9a\x83',
'RejangLetterA': '\xea\xa5\x86',
'CjkUnifiedIdeograph97c8': '\xe9\x9f\x88',
'UnicodeCharacter0xde29': '\xed\xb8\xa9',
'CjkUnifiedIdeograph97c7': '\xe9\x9f\x87',
'UnicodeCharacter0xde28': '\xed\xb8\xa8',
'LatinCapitalLetterAWithCircumflex': '\xc3\x82',
'CjkUnifiedIdeograph97c6': '\xe9\x9f\x86',
'CjkUnifiedIdeograph9fad': '\xe9\xbe\xad',
'CjkUnifiedIdeograph9fae': '\xe9\xbe\xae',
'CjkUnifiedIdeograph9faf': '\xe9\xbe\xaf',
'CjkUnifiedIdeograph97c5': '\xe9\x9f\x85',
'CjkUnifiedIdeograph9fab': '\xe9\xbe\xab',
'CjkUnifiedIdeograph9fac': '\xe9\xbe\xac',
'CjkUnifiedIdeograph730f': '\xe7\x8c\x8f',
'CjkUnifiedIdeograph6fb2': '\xe6\xbe\xb2',
'TamilLetterRa': '\xe0\xae\xb0',
'CjkUnifiedIdeograph97c2': '\xe9\x9f\x82',
'UnicodeCharacter0xde23': '\xed\xb8\xa3',
'CjkUnifiedIdeograph97c1': '\xe9\x9f\x81',
'NewTaiLueLetterHighKa': '\xe1\xa6\x82',
'CjkUnifiedIdeograph97c0': '\xe9\x9f\x80',
'HangulSyllableSswaem': '\xec\x90\xa0',
'CjkUnifiedIdeograph9fa8': '\xe9\xbe\xa8',
'CjkUnifiedIdeograph9fa9': '\xe9\xbe\xa9',
'CjkUnifiedIdeograph9fa4': '\xe9\xbe\xa4',
'CjkUnifiedIdeograph9fa5': '\xe9\xbe\xa5',
'CjkUnifiedIdeograph9fa6': '\xe9\xbe\xa6',
'CjkUnifiedIdeograph9fa7': '\xe9\xbe\xa7',
'CombiningLatinSmallLetterLongS': '\xe1\xb7\xa5',
'CjkUnifiedIdeograph9fa1': '\xe9\xbe\xa1',
'CjkUnifiedIdeograph9fa2': '\xe9\xbe\xa2',
'CjkUnifiedIdeograph9fa3': '\xe9\xbe\xa3',
'LatinSmallLetterNWithCircumflexBelow': '\xe1\xb9\x8b',
'UnicodeCharacter0xe266': '\xee\x89\xa6',
'UnicodeCharacter0xe53c': '\xee\x94\xbc',
'Sigma': '\xce\xa3',
'union': '\xe2\x88\xaa',
'FullwidthExclamationMark': '\xef\xbc\x81',
'YiSyllablePuo': '\xea\x81\x84',
'UnicodeCharacter0xf373': '\xef\x8d\xb3',
'YiSyllablePux': '\xea\x81\x8b',
'CjkCompatibilityIdeographfad8': '\xef\xab\x98',
'YiSyllablePup': '\xea\x81\x8d',
'YiSyllablePur': '\xea\x81\x8f',
'LatinSmallLetterEWithMacronAndGrave': '\xe1\xb8\x95',
'YiSyllablePut': '\xea\x81\x8a',
'ArabicLetterTahWithThreeDotsAbove': '\xda\x9f',
'HangulSyllableDdyad': '\xeb\x95\xaf',
'YiSyllableNrap': '\xea\x8e\x97',
'CjkUnifiedIdeograph9902': '\xe9\xa4\x82',
'YiSyllableNrat': '\xea\x8e\x94',
'YiSyllableNrax': '\xea\x8e\x95',
'CjkUnifiedIdeograph50c8': '\xe5\x83\x88',
'CjkUnifiedIdeograph50c9': '\xe5\x83\x89',
'YiSyllableHlat': '\xea\x86\x9e',
'CjkUnifiedIdeograph50c0': '\xe5\x83\x80',
'CjkUnifiedIdeograph50c1': '\xe5\x83\x81',
'CjkUnifiedIdeograph50c2': '\xe5\x83\x82',
'CjkUnifiedIdeograph50c3': '\xe5\x83\x83',
'CjkUnifiedIdeograph50c4': '\xe5\x83\x84',
'CjkUnifiedIdeograph50c5': '\xe5\x83\x85',
'CjkUnifiedIdeograph50c6': '\xe5\x83\x86',
'CjkUnifiedIdeograph50c7': '\xe5\x83\x87',
'CjkUnifiedIdeograph4d1a': '\xe4\xb4\x9a',
'UnicodeCharacter0xaaba': '\xea\xaa\xba',
'CjkUnifiedIdeograph4d1c': '\xe4\xb4\x9c',
'CjkUnifiedIdeograph4d1b': '\xe4\xb4\x9b',
'CjkUnifiedIdeograph4d1e': '\xe4\xb4\x9e',
'CjkUnifiedIdeograph4d1d': '\xe4\xb4\x9d',
'UnicodeCharacter0xaabf': '\xea\xaa\xbf',
'CjkUnifiedIdeograph4d1f': '\xe4\xb4\x9f',
'YiSyllableHlap': '\xea\x86\xa1',
'SinhalaVowelSignKettiAedapilla': '\xe0\xb7\x90',
'GlagoliticSmallLetterHeru': '\xe2\xb1\x88',
'UnicodeCharacter0xf17f': '\xef\x85\xbf',
'UnicodeCharacter0xf17d': '\xef\x85\xbd',
'UnicodeCharacter0xf17e': '\xef\x85\xbe',
'UnicodeCharacter0xf17b': '\xef\x85\xbb',
'UnicodeCharacter0xf17c': '\xef\x85\xbc',
'UnicodeCharacter0xf17a': '\xef\x85\xba',
'CjkUnifiedIdeograph5197': '\xe5\x86\x97',
'UnicodeCharacter0xe89': '\xe0\xba\x89',
'CircledNumberFortyOne': '\xe3\x8a\xb6',
'CjkUnifiedIdeograph5196': '\xe5\x86\x96',
'UnicodeCharacter0xf8b3': '\xef\xa2\xb3',
'CjkUnifiedIdeograph50ca': '\xe5\x83\x8a',
'CjkUnifiedIdeograph50cb': '\xe5\x83\x8b',
'CjkUnifiedIdeograph50cc': '\xe5\x83\x8c',
'CjkUnifiedIdeograph50cd': '\xe5\x83\x8d',
'CjkUnifiedIdeograph50ce': '\xe5\x83\x8e',
'CjkUnifiedIdeograph50cf': '\xe5\x83\x8f',
'UnicodeCharacter0xe85': '\xe0\xba\x85',
'CjkUnifiedIdeograph5194': '\xe5\x86\x94',
'HangulSyllableJjweogg': '\xec\xad\xa6',
'CjkUnifiedIdeograph457f': '\xe4\x95\xbf',
'IdeographicAnnotationEarthMark': '\xe3\x86\x9e',
'TaiLeLetterMa': '\xe1\xa5\x9b',
'CjkUnifiedIdeograph8f7d': '\xe8\xbd\xbd',
'HangulSyllableKeum': '\xed\x81\xbc',
'CjkUnifiedIdeograph457d': '\xe4\x95\xbd',
'UnicodeCharacter0xf178': '\xef\x85\xb8',
'UnicodeCharacter0xe265': '\xee\x89\xa5',
'UnicodeCharacter0xf176': '\xef\x85\xb6',
'UnicodeCharacter0xf177': '\xef\x85\xb7',
'UnicodeCharacter0xf174': '\xef\x85\xb4',
'CjkUnifiedIdeograph457e': '\xe4\x95\xbe',
'UnicodeCharacter0xf172': '\xef\x85\xb2',
'UnicodeCharacter0xf173': '\xef\x85\xb3',
'UnicodeCharacter0xf170': '\xef\x85\xb0',
'UnicodeCharacter0xf171': '\xef\x85\xb1',
'CjkUnifiedIdeograph4d11': '\xe4\xb4\x91',
'CjkUnifiedIdeograph4d10': '\xe4\xb4\x90',
'CjkUnifiedIdeograph4d13': '\xe4\xb4\x93',
'CjkUnifiedIdeograph4d12': '\xe4\xb4\x92',
'CjkUnifiedIdeograph4d15': '\xe4\xb4\x95',
'CjkUnifiedIdeograph4d14': '\xe4\xb4\x94',
'CjkUnifiedIdeograph4d17': '\xe4\xb4\x97',
'CjkUnifiedIdeograph4d16': '\xe4\xb4\x96',
'CjkUnifiedIdeograph4d19': '\xe4\xb4\x99',
'CjkUnifiedIdeograph4d18': '\xe4\xb4\x98',
'GreekSmallLetterEtaWithDasiaAndVariaAndYpogegrammeni': '\xe1\xbe\x93',
'HangulSyllablePyenj': '\xed\x8f\x95',
'DevanagariStressSignAnudatta': '\xe0\xa5\x92',
'HangulSyllablePyenh': '\xed\x8f\x96',
'DownArrowhead': '\xe2\x8c\x84',
'HangulSyllableBbweogg': '\xeb\xbf\xaa',
'WhiteStar': '\xe2\x98\x86',
'HangulSyllableDyeols': '\xeb\x8e\x98',
'HangulSyllableDyeolp': '\xeb\x8e\x9a',
'HangulSyllableTwibs': '\xed\x8a\x92',
'CjkUnifiedIdeograph8929': '\xe8\xa4\xa9',
'CjkUnifiedIdeograph8928': '\xe8\xa4\xa8',
'NewTaiLueVowelSignOy': '\xe1\xa6\xbd',
'HangulSyllableDyeolm': '\xeb\x8e\x96',
'HangulSyllableDyeolh': '\xeb\x8e\x9b',
'stigma': '\xcf\x9b',
'CjkUnifiedIdeograph8921': '\xe8\xa4\xa1',
'CjkUnifiedIdeograph8920': '\xe8\xa4\xa0',
'CjkUnifiedIdeograph8923': '\xe8\xa4\xa3',
'AplFunctionalSymbolLeftShoeStile': '\xe2\x8d\xa7',
'CjkUnifiedIdeograph8925': '\xe8\xa4\xa5',
'CjkUnifiedIdeograph8924': '\xe8\xa4\xa4',
'CjkUnifiedIdeograph8927': '\xe8\xa4\xa7',
'CjkUnifiedIdeograph8926': '\xe8\xa4\xa6',
'GreaterthanClosedByCurve': '\xe2\xaa\xa7',
'UnicodeCharacter0xe1b2': '\xee\x86\xb2',
'CjkUnifiedIdeograph909c': '\xe9\x82\x9c',
'CjkUnifiedIdeograph5b58': '\xe5\xad\x98',
'CjkUnifiedIdeograph5b59': '\xe5\xad\x99',
'UnicodeCharacter0xe766': '\xee\x9d\xa6',
'RightwardsTwoheadedTripleDashArrow': '\xe2\xa4\x90',
'CjkUnifiedIdeograph51ca': '\xe5\x87\x8a',
'CjkUnifiedIdeograph5b51': '\xe5\xad\x91',
'CjkUnifiedIdeograph51cc': '\xe5\x87\x8c',
'CjkUnifiedIdeograph51cb': '\xe5\x87\x8b',
'CjkUnifiedIdeograph51ce': '\xe5\x87\x8e',
'CjkUnifiedIdeograph51cd': '\xe5\x87\x8d',
'CjkUnifiedIdeograph5b56': '\xe5\xad\x96',
'CjkUnifiedIdeograph51cf': '\xe5\x87\x8f',
'CjkUnifiedIdeograph6aaf': '\xe6\xaa\xaf',
'CjkUnifiedIdeograph97cd': '\xe9\x9f\x8d',
'CjkUnifiedIdeograph9681': '\xe9\x9a\x81',
'CjkUnifiedIdeograph892a': '\xe8\xa4\xaa',
'CjkUnifiedIdeograph97cc': '\xe9\x9f\x8c',
'CjkUnifiedIdeograph892c': '\xe8\xa4\xac',
'CjkUnifiedIdeograph892b': '\xe8\xa4\xab',
'CjkUnifiedIdeograph892e': '\xe8\xa4\xae',
'CjkUnifiedIdeograph6aad': '\xe6\xaa\xad',
'HangulSyllableDdweop': '\xeb\x9b\x92',
'CjkUnifiedIdeograph892f': '\xe8\xa4\xaf',
'HangulSyllableDdweon': '\xeb\x9a\xbc',
'HangulSyllableDdweol': '\xeb\x9b\x80',
'CjkUnifiedIdeograph6aae': '\xe6\xaa\xae',
'HangulSyllableDdweoj': '\xeb\x9b\x8e',
'CjkUnifiedIdeograph97ca': '\xe9\x9f\x8a',
'HangulSyllableDdweoh': '\xeb\x9b\x93',
'HebrewLetterYodWithHiriq': '\xef\xac\x9d',
'CjkUnifiedIdeograph6aab': '\xe6\xaa\xab',
'HangulSyllableDdweod': '\xeb\x9a\xbf',
'YiSyllableSyp': '\xea\x8c\xa7',
'HangulSyllableDdweob': '\xeb\x9b\x89',
'HangulSyllableDdweoc': '\xeb\x9b\x8f',
'CjkUnifiedIdeograph6aac': '\xe6\xaa\xac',
'CjkUnifiedIdeograph51c9': '\xe5\x87\x89',
'BengaliLetterRha': '\xe0\xa7\x9d',
'BlackChessQueen': '\xe2\x99\x9b',
'CjkUnifiedIdeograph51c1': '\xe5\x87\x81',
'CjkUnifiedIdeograph51c0': '\xe5\x87\x80',
'CjkUnifiedIdeograph51c3': '\xe5\x87\x83',
'CjkUnifiedIdeograph51c2': '\xe5\x87\x82',
'CjkUnifiedIdeograph51c5': '\xe5\x87\x85',
'CjkUnifiedIdeograph51c4': '\xe5\x87\x84',
'CjkUnifiedIdeograph51c7': '\xe5\x87\x87',
'CjkUnifiedIdeograph51c6': '\xe5\x87\x86',
'UnicodeCharacter0xf3e5': '\xef\x8f\xa5',
'HexagramForComingToMeet': '\xe4\xb7\xab',
'CjkUnifiedIdeograph6d9f': '\xe6\xb6\x9f',
'HalfwidthKatakanaMiddleDot': '\xef\xbd\xa5',
'LatinSmallLetterOWithCircumflexAndHookAbove': '\xe1\xbb\x95',
'UnicodeCharacter0xf3e3': '\xef\x8f\xa3',
'HangulSyllableKegg': '\xec\xbc\x82',
'SylotiNagriLetterPo': '\xea\xa0\x99',
'CjkUnifiedIdeograph3eb0': '\xe3\xba\xb0',
'CjkUnifiedIdeograph3eb1': '\xe3\xba\xb1',
'CjkUnifiedIdeograph3eb2': '\xe3\xba\xb2',
'CjkUnifiedIdeograph3eb3': '\xe3\xba\xb3',
'CjkUnifiedIdeograph3eb4': '\xe3\xba\xb4',
'CjkUnifiedIdeograph3eb5': '\xe3\xba\xb5',
'CjkUnifiedIdeograph3eb6': '\xe3\xba\xb6',
'CjkUnifiedIdeograph3eb7': '\xe3\xba\xb7',
'CjkUnifiedIdeograph412e': '\xe4\x84\xae',
'CjkUnifiedIdeograph412d': '\xe4\x84\xad',
'CjkUnifiedIdeograph412f': '\xe4\x84\xaf',
'CjkUnifiedIdeograph412a': '\xe4\x84\xaa',
'NewTaiLueLetterLowPa': '\xe1\xa6\x97',
'CjkUnifiedIdeograph412c': '\xe4\x84\xac',
'CjkUnifiedIdeograph412b': '\xe4\x84\xab',
'CanadianSyllabicsBlackfootNa': '\xe1\x96\xbb',
'HangulSyllableCuss': '\xec\xb6\xa8',
'CanadianSyllabicsBlackfootNe': '\xe1\x96\xb8',
'CanadianSyllabicsBlackfootNi': '\xe1\x96\xb9',
'OlChikiDigitFive': '\xe1\xb1\x95',
'HangulSyllableGgwaebs': '\xea\xbd\xb6',
'CanadianSyllabicsBlackfootNo': '\xe1\x96\xba',
'UnicodeCharacter0xf3e9': '\xef\x8f\xa9',
'**i': '\xe2\x81\xb1',
'CjkUnifiedIdeograph3eba': '\xe3\xba\xba',
'CjkUnifiedIdeograph3ebb': '\xe3\xba\xbb',
'CjkUnifiedIdeograph3ebc': '\xe3\xba\xbc',
'CjkUnifiedIdeograph4129': '\xe4\x84\xa9',
'CjkUnifiedIdeograph3ebe': '\xe3\xba\xbe',
'CjkUnifiedIdeograph3ebf': '\xe3\xba\xbf',
'CjkUnifiedIdeograph4125': '\xe4\x84\xa5',
'CjkUnifiedIdeograph4124': '\xe4\x84\xa4',
'CjkUnifiedIdeograph4127': '\xe4\x84\xa7',
'CjkUnifiedIdeograph4126': '\xe4\x84\xa6',
'CjkUnifiedIdeograph4121': '\xe4\x84\xa1',
'CjkUnifiedIdeograph4120': '\xe4\x84\xa0',
'CjkUnifiedIdeograph4123': '\xe4\x84\xa3',
'CjkUnifiedIdeograph4122': '\xe4\x84\xa2',
'OlChikiLetterAl': '\xe1\xb1\x9e',
'GreekCapitalLetterOmegaWithDasia': '\xe1\xbd\xa9',
'OlChikiLetterAg': '\xe1\xb1\x9c',
'HangulLetterRieulmieum': '\xe3\x84\xbb',
'SyriacThreeDotsAbove': '\xdd\x85',
'HangulSyllableGgeulg': '\xeb\x81\x8d',
'HangulSyllableBbaelb': '\xeb\xba\x87',
'UnicodeCharacter0xe74d': '\xee\x9d\x8d',
'BoxDrawingsVerticalDoubleAndRightSingle': '\xe2\x95\x9f',
'HangulSyllablePwac': '\xed\x90\x9f',
'GreekSmallLetterZeta': '\xce\xb6',
'ArabicLetterBeheh': '\xda\x80',
'LatinSmallLetterTsDigraph': '\xca\xa6',
'HangulSyllableGgeulb': '\xeb\x81\x8f',
'DingbatNegativeCircledDigitNine': '\xe2\x9d\xbe',
'UnicodeCharacter0x169f': '\xe1\x9a\x9f',
'UnicodeCharacter0x169d': '\xe1\x9a\x9d',
'UnicodeCharacter0x169e': '\xe1\x9a\x9e',
'HangulSyllablePyaegs': '\xed\x8d\xa3',
'ArabicLigatureHehWithMeemIsolatedForm': '\xef\xb1\x92',
'CjkUnifiedIdeograph972d': '\xe9\x9c\xad',
'CjkUnifiedIdeograph972e': '\xe9\x9c\xae',
'CjkUnifiedIdeograph972f': '\xe9\x9c\xaf',
'ArabicLetterDulIsolatedForm': '\xef\xae\x86',
'LatinSmallLetterOWithAcute': '\xc3\xb3',
'CjkUnifiedIdeograph972a': '\xe9\x9c\xaa',
'CjkUnifiedIdeograph972b': '\xe9\x9c\xab',
'CjkUnifiedIdeograph6049': '\xe6\x81\x89',
'CanadianSyllabicsWestcreeCwoo': '\xe1\x92\x9b',
'CjkUnifiedIdeograph565d': '\xe5\x99\x9d',
'CjkUnifiedIdeograph565e': '\xe5\x99\x9e',
'CjkUnifiedIdeograph565f': '\xe5\x99\x9f',
'EthiopicSyllableTu': '\xe1\x89\xb1',
'CjkUnifiedIdeograph565a': '\xe5\x99\x9a',
'CjkUnifiedIdeograph565b': '\xe5\x99\x9b',
'CjkUnifiedIdeograph565c': '\xe5\x99\x9c',
'EthiopicSyllableTi': '\xe1\x89\xb2',
'UpperOneEighthBlock': '\xe2\x96\x94',
'SubscriptTwo': '\xe2\x82\x82',
'EthiopicSyllableTo': '\xe1\x89\xb6',
'CjkUnifiedIdeograph6045': '\xe6\x81\x85',
'EthiopicSyllableTa': '\xe1\x89\xb0',
'CjkUnifiedIdeograph7db0': '\xe7\xb6\xb0',
'CjkUnifiedIdeograph7db3': '\xe7\xb6\xb3',
'CjkUnifiedIdeograph7db2': '\xe7\xb6\xb2',
'EthiopicSyllableTe': '\xe1\x89\xb5',
'CjkUnifiedIdeograph7db4': '\xe7\xb6\xb4',
'CjkUnifiedIdeograph7db7': '\xe7\xb6\xb7',
'CjkUnifiedIdeograph7db6': '\xe7\xb6\xb6',
'UnicodeCharacter0xe3f2': '\xee\x8f\xb2',
'CjkUnifiedIdeograph9095': '\xe9\x82\x95',
'CjkUnifiedIdeograph9728': '\xe9\x9c\xa8',
'CjkUnifiedIdeograph9729': '\xe9\x9c\xa9',
'UnicodeCharacter0xe3f3': '\xee\x8f\xb3',
'CjkUnifiedIdeograph9724': '\xe9\x9c\xa4',
'CjkUnifiedIdeograph9092': '\xe9\x82\x92',
'CjkUnifiedIdeograph9726': '\xe9\x9c\xa6',
'CjkUnifiedIdeograph9727': '\xe9\x9c\xa7',
'CjkUnifiedIdeograph9720': '\xe9\x9c\xa0',
'CjkUnifiedIdeograph9721': '\xe9\x9c\xa1',
'CjkUnifiedIdeograph9722': '\xe9\x9c\xa2',
'CjkUnifiedIdeograph9093': '\xe9\x82\x93',
'CjkUnifiedIdeograph8c2c': '\xe8\xb0\xac',
'UnicodeCharacter0xe3f1': '\xee\x8f\xb1',
'CjkUnifiedIdeograph9090': '\xe9\x82\x90',
'VaiSyllableGo': '\xea\x97\x9d',
'VaiSyllableGa': '\xea\x95\xad',
'CjkUnifiedIdeograph9091': '\xe9\x82\x91',
'CyrillicCapitalLetterRha': '\xd4\x96',
'VaiSyllableGe': '\xea\x98\x86',
'UnicodeCharacter0xe3f7': '\xee\x8f\xb7',
'CjkUnifiedIdeograph8a0b': '\xe8\xa8\x8b',
'UnicodeCharacter0xde2c': '\xed\xb8\xac',
'HangulSyllableCong': '\xec\xb4\x9d',
'CjkUnifiedIdeograph7dba': '\xe7\xb6\xba',
'UnicodeCharacter0xe52a': '\xee\x94\xaa',
'CjkUnifiedIdeograph7dbc': '\xe7\xb6\xbc',
'CjkUnifiedIdeograph7dbb': '\xe7\xb6\xbb',
'CjkUnifiedIdeograph7dbe': '\xe7\xb6\xbe',
'CjkUnifiedIdeograph7dbd': '\xe7\xb6\xbd',
'UnicodeCharacter0xe52f': '\xee\x94\xaf',
'CjkUnifiedIdeograph7dbf': '\xe7\xb6\xbf',
'CjkUnifiedIdeograph5658': '\xe5\x99\x98',
'CjkUnifiedIdeograph5659': '\xe5\x99\x99',
'HangulSyllableConh': '\xec\xb4\x8e',
'CjkUnifiedIdeograph5654': '\xe5\x99\x94',
'CjkUnifiedIdeograph5655': '\xe5\x99\x95',
'CjkUnifiedIdeograph5656': '\xe5\x99\x96',
'CjkUnifiedIdeograph5657': '\xe5\x99\x97',
'CjkUnifiedIdeograph5650': '\xe5\x99\x90',
'CjkUnifiedIdeograph5651': '\xe5\x99\x91',
'CjkUnifiedIdeograph5652': '\xe5\x99\x92',
'CjkUnifiedIdeograph5653': '\xe5\x99\x93',
'CjkUnifiedIdeograph5386': '\xe5\x8e\x86',
'CjkUnifiedIdeograph5387': '\xe5\x8e\x87',
'CjkUnifiedIdeograph5384': '\xe5\x8e\x84',
'CjkUnifiedIdeograph5385': '\xe5\x8e\x85',
'CjkUnifiedIdeograph5382': '\xe5\x8e\x82',
'CjkUnifiedIdeograph5383': '\xe5\x8e\x83',
'CjkUnifiedIdeograph5380': '\xe5\x8e\x80',
'CjkUnifiedIdeograph5381': '\xe5\x8e\x81',
'UnicodeCharacter0xf0b4': '\xef\x82\xb4',
'UnicodeCharacter0xf0b5': '\xef\x82\xb5',
'UnicodeCharacter0xf0b6': '\xef\x82\xb6',
'CjkUnifiedIdeograph63b2': '\xe6\x8e\xb2',
'HangulSyllableRyass': '\xeb\x9f\x88',
'UnicodeCharacter0xf0b1': '\xef\x82\xb1',
'CjkUnifiedIdeograph5388': '\xe5\x8e\x88',
'CjkUnifiedIdeograph5389': '\xe5\x8e\x89',
'CjkUnifiedIdeograph9896': '\xe9\xa2\x96',
'CjkUnifiedIdeograph9897': '\xe9\xa2\x97',
'CjkUnifiedIdeograph9894': '\xe9\xa2\x94',
'CjkUnifiedIdeograph9895': '\xe9\xa2\x95',
'CjkUnifiedIdeograph9892': '\xe9\xa2\x92',
'CjkUnifiedIdeograph9893': '\xe9\xa2\x93',
'CjkUnifiedIdeograph9890': '\xe9\xa2\x90',
'CjkUnifiedIdeograph9891': '\xe9\xa2\x91',
'CjkUnifiedIdeograph9898': '\xe9\xa2\x98',
'CjkUnifiedIdeograph9899': '\xe9\xa2\x99',
'CjkUnifiedIdeograph58ef': '\xe5\xa3\xaf',
'FullwidthPoundSign': '\xef\xbf\xa1',
'HangulSyllableJigg': '\xec\xa7\x82',
'CjkUnifiedIdeograph41c8': '\xe4\x87\x88',
'HangulSyllableDyo': '\xeb\x90\xb4',
'CjkUnifiedIdeograph8195': '\xe8\x86\x95',
'KatakanahiraganaSemivoicedSoundMark': '\xe3\x82\x9c',
'GurmukhiLetterTha': '\xe0\xa8\xa5',
'CjkUnifiedIdeograph355b': '\xe3\x95\x9b',
'CjkUnifiedIdeograph355e': '\xe3\x95\x9e',
'CjkUnifiedIdeograph355d': '\xe3\x95\x9d',
'varnothing': '\xe2\x88\x85',
'CjkUnifiedIdeograph355f': '\xe3\x95\x9f',
'CjkUnifiedIdeograph989f': '\xe9\xa2\x9f',
'CjkUnifiedIdeograph989d': '\xe9\xa2\x9d',
'CjkUnifiedIdeograph989e': '\xe9\xa2\x9e',
'CjkUnifiedIdeograph989b': '\xe9\xa2\x9b',
'CjkUnifiedIdeograph56e4': '\xe5\x9b\xa4',
'CjkUnifiedIdeograph989a': '\xe9\xa2\x9a',
'HangulSyllableMyeonj': '\xeb\xa9\xb5',
'CjkUnifiedIdeograph538f': '\xe5\x8e\x8f',
'CjkUnifiedIdeograph41c2': '\xe4\x87\x82',
'CjkUnifiedIdeograph538d': '\xe5\x8e\x8d',
'CjkUnifiedIdeograph538e': '\xe5\x8e\x8e',
'CjkUnifiedIdeograph538b': '\xe5\x8e\x8b',
'CjkUnifiedIdeograph538c': '\xe5\x8e\x8c',
'CjkUnifiedIdeograph538a': '\xe5\x8e\x8a',
'UnicodeCharacter0xf0bd': '\xef\x82\xbd',
'UnicodeCharacter0xf0be': '\xef\x82\xbe',
'UnicodeCharacter0xf0bf': '\xef\x82\xbf',
'CjkUnifiedIdeograph41c0': '\xe4\x87\x80',
'UnicodeCharacter0xf0bb': '\xef\x82\xbb',
'UnicodeCharacter0xf0bc': '\xef\x82\xbc',
'CjkUnifiedIdeograph3559': '\xe3\x95\x99',
'CjkUnifiedIdeograph3558': '\xe3\x95\x98',
'BraillePatternDots5': '\xe2\xa0\x90',
'SquareAaru': '\xe3\x8c\x83',
'CjkUnifiedIdeograph3551': '\xe3\x95\x91',
'CjkUnifiedIdeograph41c6': '\xe4\x87\x86',
'CjkUnifiedIdeograph3553': '\xe3\x95\x93',
'CjkUnifiedIdeograph3552': '\xe3\x95\x92',
'CjkUnifiedIdeograph3555': '\xe3\x95\x95',
'MathematicalRightDoubleAngleBracket': '\xe2\x9f\xab',
'CjkUnifiedIdeograph3557': '\xe3\x95\x97',
'CjkUnifiedIdeograph3556': '\xe3\x95\x96',
'UnicodeCharacter0xf73f': '\xef\x9c\xbf',
'CjkCompatibilityIdeographf949': '\xef\xa5\x89',
'SundaneseDigitSix': '\xe1\xae\xb6',
'NumeroSign': '\xe2\x84\x96',
'CjkUnifiedIdeograph41c5': '\xe4\x87\x85',
'UnicodeCharacter0xdff0': '\xed\xbf\xb0',
'ChamPunctuationDanda': '\xea\xa9\x9d',
'HangulSyllableJjalm': '\xec\xa7\xa6',
'HangulSyllableMyeonh': '\xeb\xa9\xb6',
'HangulSyllableJjalb': '\xec\xa7\xa7',
'LatinSmallLetterUWithTildeBelow': '\xe1\xb9\xb5',
'HangulSyllableJjalg': '\xec\xa7\xa5',
'CopticCapitalLetterHate': '\xe2\xb2\x8e',
'UnicodeCharacter0xe74f': '\xee\x9d\x8f',
'OriyaLetterNga': '\xe0\xac\x99',
'UnicodeCharacter0xa6ef': '\xea\x9b\xaf',
'HangulSyllableJjals': '\xec\xa7\xa8',
'HangulSyllableJjalp': '\xec\xa7\xaa',
'CjkUnifiedIdeograph672f': '\xe6\x9c\xaf',
'CjkUnifiedIdeograph6aa6': '\xe6\xaa\xa6',
'CjkUnifiedIdeograph36ec': '\xe3\x9b\xac',
'CyrillicSmallLetterSha': '\xd1\x88',
'UnicodeCharacter0xeebb': '\xee\xba\xbb',
'UnicodeCharacter0xee37': '\xee\xb8\xb7',
'UnicodeCharacter0xee36': '\xee\xb8\xb6',
'UnicodeCharacter0xee35': '\xee\xb8\xb5',
'CjkUnifiedIdeograph6aa5': '\xe6\xaa\xa5',
'UnicodeCharacter0xee33': '\xee\xb8\xb3',
'UnicodeCharacter0xee32': '\xee\xb8\xb2',
'UnicodeCharacter0xee31': '\xee\xb8\xb1',
'UnicodeCharacter0xee30': '\xee\xb8\xb0',
'CjkUnifiedIdeograph6aa2': '\xe6\xaa\xa2',
'Xi': '\xce\x9e',
'RightwardsTwoheadedArrowWithVerticalStroke': '\xe2\xa4\x80',
'UnicodeCharacter0xee39': '\xee\xb8\xb9',
'UnicodeCharacter0xee38': '\xee\xb8\xb8',
'CjkUnifiedIdeograph6aa0': '\xe6\xaa\xa0',
'HangulSyllableGyebs': '\xea\xb3\x96',
'HangulSyllableDult': '\xeb\x91\x9d',
'CyrillicSmallLetterUWithMacron': '\xd3\xaf',
'CjkUnifiedIdeograph6aa1': '\xe6\xaa\xa1',
'UnicodeCharacter0xeebc': '\xee\xba\xbc',
'UnicodeCharacter0xee3f': '\xee\xb8\xbf',
'UnicodeCharacter0xee3e': '\xee\xb8\xbe',
'ThaiCharacterPaiyannoi': '\xe0\xb8\xaf',
'UnicodeCharacter0xee3c': '\xee\xb8\xbc',
'UnicodeCharacter0xee3b': '\xee\xb8\xbb',
'UnicodeCharacter0xee3a': '\xee\xb8\xba',
'HangulSyllableDyi': '\xeb\x93\xb8',
'ArabicLetterHighHamzaYeh': '\xd9\xb8',
'ArabicLetterRehWithDotBelowAndDotAbove': '\xda\x96',
'HangulJongseongRieul': '\xe1\x86\xaf',
'CircledNumberTwentyFive': '\xe3\x89\x95',
'LatinSmallLetterNWithCaron': '\xc5\x88',
'CjkUnifiedIdeograph9548': '\xe9\x95\x88',
'HangulSyllableKeolg': '\xec\xbb\xad',
'EthiopicSyllableKoa': '\xe1\x8a\xaf',
'CjkUnifiedIdeograph9549': '\xe9\x95\x89',
'UnicodeCharacter0xa6eb': '\xea\x9b\xab',
'GreekSmallLetterAlphaWithVrachy': '\xe1\xbe\xb0',
'CyrillicSmallLetterKomiDe': '\xd4\x81',
'UnicodeCharacter0xf45c': '\xef\x91\x9c',
'LeftSevenEighthsBlock': '\xe2\x96\x89',
'CjkUnifiedIdeograph79d5': '\xe7\xa7\x95',
'UnicodeCharacter0xeebf': '\xee\xba\xbf',
'UnicodeCharacter0xe7d6': '\xee\x9f\x96',
'CjkUnifiedIdeograph8812': '\xe8\xa0\x92',
'UnicodeCharacter0xf1c0': '\xef\x87\x80',
'CjkUnifiedIdeograph8814': '\xe8\xa0\x94',
'CjkUnifiedIdeograph8a65': '\xe8\xa9\xa5',
'CjkUnifiedIdeograph43f3': '\xe4\x8f\xb3',
'CjkUnifiedIdeograph8a67': '\xe8\xa9\xa7',
'CopticCapitalLetterEie': '\xe2\xb2\x88',
'CjkUnifiedIdeograph8a61': '\xe8\xa9\xa1',
'CjkUnifiedIdeograph8a60': '\xe8\xa9\xa0',
'CjkUnifiedIdeograph8a63': '\xe8\xa9\xa3',
'CjkUnifiedIdeograph43f2': '\xe4\x8f\xb2',
'CjkUnifiedIdeograph94cd': '\xe9\x93\x8d',
'CopticSmallLetterAkhmimicKhei': '\xe2\xb3\x89',
'CjkUnifiedIdeograph8a69': '\xe8\xa9\xa9',
'CjkUnifiedIdeograph8a68': '\xe8\xa9\xa8',
'OlChikiDigitOne': '\xe1\xb1\x91',
'CjkUnifiedIdeograph9543': '\xe9\x95\x83',
'HangulSyllableTegs': '\xed\x85\x8f',
'CjkUnifiedIdeograph9544': '\xe9\x95\x84',
'HangulSyllableGolt': '\xea\xb3\xad',
'HangulSyllableGols': '\xea\xb3\xac',
'PhagspaLetterFa': '\xea\xa1\xa4',
'KangxiRadicalCover': '\xe2\xbc\x8d',
'HangulSyllableGolm': '\xea\xb3\xaa',
'CjkUnifiedIdeograph79d8': '\xe7\xa7\x98',
'CjkUnifiedIdeograph70e7': '\xe7\x83\xa7',
'TeluguLengthMark': '\xe0\xb1\x95',
'HangulSyllableGolg': '\xea\xb3\xa9',
'CjkUnifiedIdeograph79d9': '\xe7\xa7\x99',
'UnicodeCharacter0xebcf': '\xee\xaf\x8f',
'HangulSyllableGolb': '\xea\xb3\xab',
'BuhidLetterTa': '\xe1\x9d\x86',
'CjkUnifiedIdeograph8a6e': '\xe8\xa9\xae',
'CjkUnifiedIdeograph8a6d': '\xe8\xa9\xad',
'CjkUnifiedIdeograph8a6f': '\xe8\xa9\xaf',
'CjkUnifiedIdeograph8a6a': '\xe8\xa9\xaa',
'YiSyllableBep': '\xea\x80\xab',
'CjkUnifiedIdeograph8a6c': '\xe8\xa9\xac',
'CjkUnifiedIdeograph8a6b': '\xe8\xa9\xab',
'UnicodeCharacter0x19cf': '\xe1\xa7\x8f',
'UnicodeCharacter0x19ce': '\xe1\xa7\x8e',
'UnicodeCharacter0x19cd': '\xe1\xa7\x8d',
'UnicodeCharacter0x19cc': '\xe1\xa7\x8c',
'UnicodeCharacter0x19cb': '\xe1\xa7\x8b',
'UnicodeCharacter0x19ca': '\xe1\xa7\x8a',
'UnicodeCharacter0xf731': '\xef\x9c\xb1',
'KangxiRadicalDragon': '\xe2\xbf\x93',
'SinhalaLetterUuyanna': '\xe0\xb6\x8c',
'HangulSyllableNyilt': '\xeb\x8a\xb9',
'UnicodeCharacter0xf8b6': '\xef\xa2\xb6',
'HangulSyllableJjyass': '\xec\xa8\xa8',
'UnicodeCharacter0xee8e': '\xee\xba\x8e',
'UnicodeCharacter0xf734': '\xef\x9c\xb4',
'HangulLetterNieuncieuc': '\xe3\x84\xb5',
'HangulSyllableJweogs': '\xec\xa4\x9b',
'YiSyllableBex': '\xea\x80\xa9',
'BraillePatternDots23678': '\xe2\xa3\xa6',
'GreekSmallLetterIotaWithDasiaAndVaria': '\xe1\xbc\xb3',
'CjkUnifiedIdeograph7de4': '\xe7\xb7\xa4',
'UnicodeCharacter0xf737': '\xef\x9c\xb7',
'UnicodeCharacter0xe350': '\xee\x8d\x90',
'UnicodeCharacter0xe3fb': '\xee\x8f\xbb',
'UnicodeCharacter0xee8b': '\xee\xba\x8b',
'LatinCapitalLetterKWithStroke': '\xea\x9d\x80',
'UnicodeCharacter0xe3fc': '\xee\x8f\xbc',
'CjkUnifiedIdeograph7409': '\xe7\x90\x89',
'HangulSyllablePwat': '\xed\x90\xa1',
'HangulSyllableRibs': '\xeb\xa6\xbe',
'HangulSyllableJjyugs': '\xec\xae\xbb',
'YiSyllableSsox': '\xea\x8c\xb6',
'UnicodeCharacter0xe3ff': '\xee\x8f\xbf',
'YiSyllableSsot': '\xea\x8c\xb5',
'YiSyllableSsop': '\xea\x8c\xb8',
'HangulSyllableHilp': '\xed\x9e\x96',
'UnicodeCharacter0xe3fe': '\xee\x8f\xbe',
'CjkUnifiedIdeograph70e3': '\xe7\x83\xa3',
'RighttoleftEmbedding': '\xe2\x80\xab',
'HangulSyllableGgweop': '\xea\xbf\xae',
'HangulSyllableGgweos': '\xea\xbf\xa7',
'HangulSyllableGgweot': '\xea\xbf\xad',
'HangulSyllableKoelh': '\xec\xbe\xb7',
'HangulSyllableHuss': '\xed\x9b\x98',
'LatinLetterRetroflexClick': '\xc7\x83',
'HangulSyllableKoelg': '\xec\xbe\xb1',
'MyanmarSignVisarga': '\xe1\x80\xb8',
'HangulSyllableMalg': '\xeb\xa7\x91',
'HangulSyllableGgweob': '\xea\xbf\xa5',
'HangulSyllableGgweoc': '\xea\xbf\xab',
'HangulSyllableGgweod': '\xea\xbf\x9b',
'ArabicLetterTteheh': '\xd9\xba',
'HangulSyllableGgweog': '\xea\xbf\x95',
'HangulSyllableGgweoh': '\xea\xbf\xaf',
'CjkUnifiedIdeograph84c3': '\xe8\x93\x83',
'HangulSyllableGgweoj': '\xea\xbf\xaa',
'HangulSyllableGgweok': '\xea\xbf\xac',
'HangulSyllableGgweol': '\xea\xbf\x9c',
'HangulSyllableGgweom': '\xea\xbf\xa4',
'HangulSyllableGgweon': '\xea\xbf\x98',
'HangulSyllableCwaec': '\xec\xb5\x97',
'CopticSymbolStauros': '\xe2\xb3\xa7',
'ThaiDigitOne': '\xe0\xb9\x91',
'ArabicLigatureSeenWithKhahWithYehFinalForm': '\xef\xb7\x86',
'HangulSyllableMalb': '\xeb\xa7\x93',
'UnicodeCharacter0xdfd6': '\xed\xbf\x96',
'UnicodeCharacter0xdfd7': '\xed\xbf\x97',
'UnicodeCharacter0xdfd4': '\xed\xbf\x94',
'UnicodeCharacter0xdfd5': '\xed\xbf\x95',
'UnicodeCharacter0xdfd2': '\xed\xbf\x92',
'UnicodeCharacter0xdfd3': '\xed\xbf\x93',
'UnicodeCharacter0xdfd0': '\xed\xbf\x90',
'YiSyllableHluox': '\xea\x86\xa2',
'HangulSyllableHils': '\xed\x9e\x94',
'UnicodeCharacter0xdfd8': '\xed\xbf\x98',
'UnicodeCharacter0xdfd9': '\xed\xbf\x99',
'UnicodeCharacter0xd925': '\xed\xa4\xa5',
'UnicodeCharacter0xf713': '\xef\x9c\x93',
'HalfwidthHangulLetterRieulsios': '\xef\xbe\xad',
'WhiteUppointingTriangleWithDot': '\xe2\x97\xac',
'CjkUnifiedIdeograph819b': '\xe8\x86\x9b',
'RightWhiteParenthesis': '\xe2\xa6\x86',
'HangulSyllableHilt': '\xed\x9e\x95',
'RejangLetterLa': '\xea\xa4\xbe',
'UnicodeCharacter0xf258': '\xef\x89\x98',
'CjkUnifiedIdeograph7404': '\xe7\x90\x84',
'UnicodeCharacter0xdfdf': '\xed\xbf\x9f',
'CjkUnifiedIdeograph94c0': '\xe9\x93\x80',
'UnicodeCharacter0xdfdd': '\xed\xbf\x9d',
'UnicodeCharacter0xdfde': '\xed\xbf\x9e',
'UnicodeCharacter0xdfdb': '\xed\xbf\x9b',
'UnicodeCharacter0xdfdc': '\xed\xbf\x9c',
'CjkUnifiedIdeograph94c1': '\xe9\x93\x81',
'UnicodeCharacter0xd859': '\xed\xa1\x99',
'HangulSyllableTyogg': '\xed\x88\x92',
'UnicodeCharacter0xda34': '\xed\xa8\xb4',
'YiSyllableNdex': '\xea\x85\x9f',
'ArabicLetterAlefWithHamzaAbove': '\xd8\xa3',
'SyriacBarrekh': '\xdd\x8a',
'TriangleWithSerifsAtBottom': '\xe2\xa7\x8d',
'HangulSyllableJug': '\xec\xa3\xbd',
'CjkUnifiedIdeograph94c8': '\xe9\x93\x88',
'HangulSyllableYunh': '\xec\x9c\xa6',
'BoxDrawingsUpDoubleAndLeftSingle': '\xe2\x95\x9c',
'CjkUnifiedIdeograph94c9': '\xe9\x93\x89',
'HangulSyllableYung': '\xec\x9c\xb5',
'CombiningDoubleBreveBelow': '\xcd\x9c',
'UnicodeCharacter0xef42': '\xee\xbd\x82',
'ModifierLetterDoublePrime': '\xca\xba',
'ArabicLetterDdahalIsolatedForm': '\xef\xae\x82',
'HangulSyllableJyang': '\xec\x9f\x9d',
'HangulSyllableJyanh': '\xec\x9f\x8e',
'HangulSyllableJyanj': '\xec\x9f\x8d',
'RunicLetterIng': '\xe1\x9b\x9d',
'NkoLetterYa': '\xdf\xa6',
'CjkUnifiedIdeograph9d66': '\xe9\xb5\xa6',
'HangulSyllablePuls': '\xed\x92\x84',
'CjkUnifiedIdeograph92cd': '\xe9\x8b\x8d',
'UnicodeCharacter0xeeb9': '\xee\xba\xb9',
'CjkUnifiedIdeograph6251': '\xe6\x89\x91',
'CjkUnifiedIdeograph6250': '\xe6\x89\x90',
'CjkUnifiedIdeograph6253': '\xe6\x89\x93',
'CjkUnifiedIdeograph6252': '\xe6\x89\x92',
'CjkUnifiedIdeograph6255': '\xe6\x89\x95',
'CjkUnifiedIdeograph6254': '\xe6\x89\x94',
'CjkUnifiedIdeograph6257': '\xe6\x89\x97',
'CjkUnifiedIdeograph6256': '\xe6\x89\x96',
'CjkUnifiedIdeograph6259': '\xe6\x89\x99',
'CjkUnifiedIdeograph6258': '\xe6\x89\x98',
'LatinSmallLetterAWithDotAbove': '\xc8\xa7',
'ArabicLigatureSheenWithJeemMedialForm': '\xef\xb4\xb7',
'CjkUnifiedIdeograph85d2': '\xe8\x97\x92',
'CjkUnifiedIdeograph768d': '\xe7\x9a\x8d',
'CjkUnifiedIdeograph600a': '\xe6\x80\x8a',
'CjkUnifiedIdeograph8f70': '\xe8\xbd\xb0',
'CjkUnifiedIdeograph768f': '\xe7\x9a\x8f',
'ArabicLetterKehehMedialForm': '\xef\xae\x91',
'CjkUnifiedIdeograph768a': '\xe7\x9a\x8a',
'LatinSmallLetterDzDigraphWithCurl': '\xca\xa5',
'CjkUnifiedIdeograph95b5': '\xe9\x96\xb5',
'CjkUnifiedIdeograph625a': '\xe6\x89\x9a',
'CjkUnifiedIdeograph625c': '\xe6\x89\x9c',
'CjkUnifiedIdeograph625b': '\xe6\x89\x9b',
'CjkUnifiedIdeograph625e': '\xe6\x89\x9e',
'CjkUnifiedIdeograph625d': '\xe6\x89\x9d',
'CjkUnifiedIdeograph625f': '\xe6\x89\x9f',
'CjkUnifiedIdeograph768b': '\xe7\x9a\x8b',
'CjkUnifiedIdeograph8f73': '\xe8\xbd\xb3',
'HangulSyllableDyaegg': '\xeb\x8c\xba',
'UnicodeCharacter0xa4e5': '\xea\x93\xa5',
'UnicodeCharacter0xdf8a': '\xed\xbe\x8a',
'UnicodeCharacter0xeeb6': '\xee\xba\xb6',
'CjkUnifiedIdeograph95b2': '\xe9\x96\xb2',
'HangulSyllableJuj': '\xec\xa4\x92',
'UnicodeCharacter0x819': '\xe0\xa0\x99',
'DentistrySymbolLightVerticalAndWave': '\xe2\x8f\x86',
'HangulSyllableCyeols': '\xec\xb3\x9c',
'UnicodeCharacter0xd900': '\xed\xa4\x80',
'CircledIdeographFemale': '\xe3\x8a\x9b',
'ParenthesizedHangulSios': '\xe3\x88\x86',
'CjkUnifiedIdeograph8f72': '\xe8\xbd\xb2',
'BengaliLetterNa': '\xe0\xa6\xa8',
'HangulSyllableCyeolp': '\xec\xb3\x9e',
'CjkUnifiedIdeograph954a': '\xe9\x95\x8a',
'CjkUnifiedIdeograph95b3': '\xe9\x96\xb3',
'GreekSmallLetterOmegaWithYpogegrammeni': '\xe1\xbf\xb3',
'CjkUnifiedIdeograph954b': '\xe9\x95\x8b',
'MongolianLetterAliGaliUbadama': '\xe1\xa2\x83',
'HangulSyllableGgwinj': '\xeb\x80\x91',
'HangulSyllableGgwinh': '\xeb\x80\x92',
'CjkUnifiedIdeograph78ed': '\xe7\xa3\xad',
'CjkUnifiedIdeograph78ee': '\xe7\xa3\xae',
'CjkUnifiedIdeograph78ef': '\xe7\xa3\xaf',
'CjkUnifiedIdeograph8f75': '\xe8\xbd\xb5',
'CjkUnifiedIdeograph78ea': '\xe7\xa3\xaa',
'CjkUnifiedIdeograph78eb': '\xe7\xa3\xab',
'CjkUnifiedIdeograph78ec': '\xe7\xa3\xac',
'CjkUnifiedIdeograph43fe': '\xe4\x8f\xbe',
'CjkUnifiedIdeograph4ca8': '\xe4\xb2\xa8',
'CjkUnifiedIdeograph4ca9': '\xe4\xb2\xa9',
'CjkUnifiedIdeograph4ca6': '\xe4\xb2\xa6',
'CjkUnifiedIdeograph4ca7': '\xe4\xb2\xa7',
'CjkUnifiedIdeograph4ca4': '\xe4\xb2\xa4',
'CjkUnifiedIdeograph4ca5': '\xe4\xb2\xa5',
'CjkUnifiedIdeograph4ca2': '\xe4\xb2\xa2',
'CjkUnifiedIdeograph4ca3': '\xe4\xb2\xa3',
'CjkUnifiedIdeograph4ca0': '\xe4\xb2\xa0',
'CjkUnifiedIdeograph4ca1': '\xe4\xb2\xa1',
'HangulSyllableTeolt': '\xed\x84\xbd',
'UnicodeCharacter0xe93d': '\xee\xa4\xbd',
'LepchaSubjoinedLetterYa': '\xe1\xb0\xa4',
'UnicodeCharacter0xdf8d': '\xed\xbe\x8d',
'CjkUnifiedIdeograph9191': '\xe9\x86\x91',
'CjkUnifiedIdeograph95b1': '\xe9\x96\xb1',
'kappa': '\xce\xba',
'cuberoot': '\xe2\x88\x9b',
'HangulSyllableNyeos': '\xeb\x85\x93',
'HangulSyllableNyeop': '\xeb\x85\x9a',
'CjkUnifiedIdeograph8f77': '\xe8\xbd\xb7',
'CjkUnifiedIdeograph4caf': '\xe4\xb2\xaf',
'CjkUnifiedIdeograph4cad': '\xe4\xb2\xad',
'CjkUnifiedIdeograph4cae': '\xe4\xb2\xae',
'CjkUnifiedIdeograph4cab': '\xe4\xb2\xab',
'CjkUnifiedIdeograph4cac': '\xe4\xb2\xac',
'CjkUnifiedIdeograph4caa': '\xe4\xb2\xaa',
'HangulSyllableNyeoc': '\xeb\x85\x97',
'HangulSyllableNyeob': '\xeb\x85\x91',
'TamilLetterNga': '\xe0\xae\x99',
'CjkUnifiedIdeograph78e8': '\xe7\xa3\xa8',
'LatinSmallLetterIWithDiaeresis': '\xc3\xaf',
'HangulSyllableNyeod': '\xeb\x85\x87',
'CjkUnifiedIdeograph78e4': '\xe7\xa3\xa4',
'CjkUnifiedIdeograph78e5': '\xe7\xa3\xa5',
'CjkUnifiedIdeograph78e6': '\xe7\xa3\xa6',
'CjkUnifiedIdeograph78e7': '\xe7\xa3\xa7',
'CjkUnifiedIdeograph78e0': '\xe7\xa3\xa0',
'CjkUnifiedIdeograph78e1': '\xe7\xa3\xa1',
'CjkUnifiedIdeograph78e2': '\xe7\xa3\xa2',
'CjkUnifiedIdeograph78e3': '\xe7\xa3\xa3',
'CjkUnifiedIdeograph6a71': '\xe6\xa9\xb1',
'CjkUnifiedIdeograph7b71': '\xe7\xad\xb1',
'UnicodeCharacter0xeddb': '\xee\xb7\x9b',
'CjkUnifiedIdeograph64bf': '\xe6\x92\xbf',
'CjkUnifiedIdeograph64bd': '\xe6\x92\xbd',
'CjkUnifiedIdeograph64be': '\xe6\x92\xbe',
'CjkUnifiedIdeograph64bb': '\xe6\x92\xbb',
'CjkUnifiedIdeograph64bc': '\xe6\x92\xbc',
'CjkUnifiedIdeograph64ba': '\xe6\x92\xba',
'MyanmarLetterTha': '\xe1\x80\x91',
'HangulSyllableCyeolh': '\xec\xb3\x9f',
'HangulSyllableGyong': '\xea\xb5\xa5',
'CjkUnifiedIdeograph7b72': '\xe7\xad\xb2',
'HangulSyllableGyonj': '\xea\xb5\x95',
'UnicodeCharacter0xec02': '\xee\xb0\x82',
'CjkUnifiedIdeograph8f41': '\xe8\xbd\x81',
'HangulSyllableTwaenh': '\xed\x87\x9e',
'LatinCapitalLetterDWithDotBelow': '\xe1\xb8\x8c',
'HangulSyllableTwaenj': '\xed\x87\x9d',
'HangulSyllableJup': '\xec\xa4\x96',
'UnicodeCharacter0xdaf0': '\xed\xab\xb0',
'UnicodeCharacter0xedde': '\xee\xb7\x9e',
'HangulSyllableJat': '\xec\x9e\xa9',
'RightwardsDashedArrow': '\xe2\x87\xa2',
'UnicodeCharacter0xeddd': '\xee\xb7\x9d',
'HangulSyllableKyeoj': '\xec\xbc\xb2',
'CjkUnifiedIdeograph8f78': '\xe8\xbd\xb8',
'HangulSyllableKyeoh': '\xec\xbc\xb7',
'CjkUnifiedIdeograph7b73': '\xe7\xad\xb3',
'CjkUnifiedIdeograph64b8': '\xe6\x92\xb8',
'CjkUnifiedIdeograph64b9': '\xe6\x92\xb9',
'CjkUnifiedIdeograph64b6': '\xe6\x92\xb6',
'PoundSign': '\xc2\xa3',
'CjkUnifiedIdeograph64b4': '\xe6\x92\xb4',
'CjkUnifiedIdeograph64b5': '\xe6\x92\xb5',
'CjkUnifiedIdeograph64b2': '\xe6\x92\xb2',
'CjkUnifiedIdeograph64b3': '\xe6\x92\xb3',
'CjkUnifiedIdeograph64b0': '\xe6\x92\xb0',
'CjkUnifiedIdeograph64b1': '\xe6\x92\xb1',
'HangulSyllableBod': '\xeb\xb3\xbb',
'HangulSyllableJas': '\xec\x9e\xa3',
'CjkUnifiedIdeograph7392': '\xe7\x8e\x92',
'CjkUnifiedIdeograph77e7': '\xe7\x9f\xa7',
'HangulSyllableKyeos': '\xec\xbc\xaf',
'HangulSyllableKyeop': '\xec\xbc\xb6',
'HangulSyllableKyeot': '\xec\xbc\xb5',
'LatinSmallLetterSidewaysO': '\xe1\xb4\x91',
'HangulSyllableJam': '\xec\x9e\xa0',
'CjkUnifiedIdeograph7b74': '\xe7\xad\xb4',
'GlagoliticSmallLetterInitialIzhe': '\xe2\xb0\xba',
'CjkUnifiedIdeograph688a': '\xe6\xa2\x8a',
'VaiSyllableHee': '\xea\x94\x82',
'HangulSyllableHaebs': '\xed\x96\x86',
'HangulSyllableJjwilp': '\xec\xae\xaa',
'YiSyllableCat': '\xea\x8a\xb6',
'HangulSyllableBon': '\xeb\xb3\xb8',
'EthiopicSyllableDdee': '\xe1\x8b\xbc',
'LatinSmallLetterSidewaysU': '\xe1\xb4\x9d',
'HangulSyllableBom': '\xeb\xb4\x84',
'HangulSyllableJah': '\xec\x9e\xab',
'CjkUnifiedIdeograph7b75': '\xe7\xad\xb5',
'UnicodeCharacter0xab60': '\xea\xad\xa0',
'HangulSyllableJak': '\xec\x9e\xa8',
'YiSyllableWox': '\xea\x8a\x87',
'HangulSyllableBok': '\xeb\xb4\x8c',
'SaurashtraLetterSha': '\xea\xa2\xaf',
'UnicodeCharacter0xe9c9': '\xee\xa7\x89',
'HangulSyllableBoj': '\xeb\xb4\x8a',
'CjkUnifiedIdeograph8882': '\xe8\xa2\x82',
'RightAngle': '\xe2\x88\x9f',
'UnicodeCharacter0xf78f': '\xef\x9e\x8f',
'HangulSyllableJad': '\xec\x9e\x97',
'HangulSyllableBoh': '\xeb\xb4\x8f',
'HangulSyllableSsyeog': '\xec\x8e\xa1',
'CjkUnifiedIdeograph7b76': '\xe7\xad\xb6',
'UpwardsArrowLeftwardsOfDownwardsArrow': '\xe2\x87\x85',
'CjkUnifiedIdeograph5db7': '\xe5\xb6\xb7',
'HangulSyllableJaj': '\xec\x9e\xa6',
'UnicodeCharacter0xdd2b': '\xed\xb4\xab',
'CjkUnifiedIdeograph5db6': '\xe5\xb6\xb6',
'CjkUnifiedIdeograph9db5': '\xe9\xb6\xb5',
'CjkUnifiedIdeograph5db5': '\xe5\xb6\xb5',
'CyrillicCapitalLetterDzelo': '\xea\x99\x82',
'CjkUnifiedIdeograph5db4': '\xe5\xb6\xb4',
'HangulSyllableJac': '\xec\x9e\xa7',
'CjkUnifiedIdeograph6a29': '\xe6\xa8\xa9',
'CjkUnifiedIdeograph6a28': '\xe6\xa8\xa8',
'CjkUnifiedIdeograph6a27': '\xe6\xa8\xa7',
'UnicodeCharacter0xbff': '\xe0\xaf\xbf',
'UnicodeCharacter0xbfe': '\xe0\xaf\xbe',
'UnicodeCharacter0xbfd': '\xe0\xaf\xbd',
'UnicodeCharacter0xbfc': '\xe0\xaf\xbc',
'UnicodeCharacter0xbfb': '\xe0\xaf\xbb',
'HorizontalLineExtension': '\xe2\x8e\xaf',
'ArmenianSmallLetterBen': '\xd5\xa2',
'UnicodeCharacter0xe9c0': '\xee\xa7\x80',
'CjkUnifiedIdeograph5db1': '\xe5\xb6\xb1',
'CircledHangulHieuhA': '\xe3\x89\xbb',
'UnicodeCharacter0xe9c7': '\xee\xa7\x87',
'CjkUnifiedIdeograph5db0': '\xe5\xb6\xb0',
'CjkUnifiedIdeograph380d': '\xe3\xa0\x8d',
'UnicodeCharacter0xe9c4': '\xee\xa7\x84',
'CyrillicCapitalLetterBigYus': '\xd1\xaa',
'EthiopicSyllableQyu': '\xe2\xb7\x81',
'CjkUnifiedIdeograph6a2f': '\xe6\xa8\xaf',
'CjkUnifiedIdeograph6a2e': '\xe6\xa8\xae',
'CjkUnifiedIdeograph6a2d': '\xe6\xa8\xad',
'CjkUnifiedIdeograph6a2c': '\xe6\xa8\xac',
'CjkUnifiedIdeograph6a2b': '\xe6\xa8\xab',
'CjkUnifiedIdeograph6a2a': '\xe6\xa8\xaa',
'HangulSyllableOnh': '\xec\x98\xaa',
'EthiopicSyllableQye': '\xe2\xb7\x85',
'UnicodeCharacter0xdbf4': '\xed\xaf\xb4',
'CjkUnifiedIdeograph380c': '\xe3\xa0\x8c',
'LatinSmallLetterIWithCircumflex': '\xc3\xae',
'EthiopicSyllableQyo': '\xe2\xb7\x86',
'GreekThetaSymbol': '\xcf\x91',
'EthiopicSyllableQyi': '\xe2\xb7\x82',
'ArabicLigatureFehWithMeemWithYehFinalForm': '\xef\xb7\x81',
'HangulSyllableSeobs': '\xec\x84\xae',
'UnicodeCharacter0xe764': '\xee\x9d\xa4',
'UnicodeCharacter0xf3cc': '\xef\x8f\x8c',
'EqualToOrLessthan': '\xe2\x8b\x9c',
'DingbatCircledSansserifDigitSix': '\xe2\x9e\x85',
'HangulSyllableOnj': '\xec\x98\xa9',
'UnicodeCharacter0xf39a': '\xef\x8e\x9a',
'CjkUnifiedIdeograph380a': '\xe3\xa0\x8a',
'UnicodeCharacter0xd941': '\xed\xa5\x81',
'UnicodeCharacter0xd940': '\xed\xa5\x80',
'UnicodeCharacter0xd943': '\xed\xa5\x83',
'UnicodeCharacter0xd942': '\xed\xa5\x82',
'UnicodeCharacter0xd945': '\xed\xa5\x85',
'UnicodeCharacter0xd944': '\xed\xa5\x84',
'UnicodeCharacter0xd947': '\xed\xa5\x87',
'UnicodeCharacter0xd946': '\xed\xa5\x86',
'UnicodeCharacter0xd949': '\xed\xa5\x89',
'UnicodeCharacter0xd948': '\xed\xa5\x88',
'UnicodeCharacter0xe32a': '\xee\x8c\xaa',
'KangxiRadicalMoon': '\xe2\xbd\x89',
'SaurashtraLetterNa': '\xea\xa2\xa5',
'FullwidthBrokenBar': '\xef\xbf\xa4',
'UnicodeCharacter0xeb29': '\xee\xac\xa9',
'UnicodeCharacter0xe7de': '\xee\x9f\x9e',
'UnicodeCharacter0xe313': '\xee\x8c\x93',
'UnicodeCharacter0xdefd': '\xed\xbb\xbd',
'UnicodeCharacter0xdefc': '\xed\xbb\xbc',
'BalineseCarikPamungkah': '\xe1\xad\x9d',
'EthiopicSyllablePee': '\xe1\x8d\x94',
'UnicodeCharacter0xd94c': '\xed\xa5\x8c',
'HangulSyllableTid': '\xed\x8b\xb7',
'HangulSyllableTic': '\xed\x8c\x87',
'HangulSyllableTib': '\xed\x8c\x81',
'HangulSyllableBwaej': '\xeb\xb5\x82',
'ArabicLetterDadWithDotBelow': '\xdb\xbb',
'HangulSyllableTin': '\xed\x8b\xb4',
'HangulSyllableTim': '\xed\x8c\x80',
'HangulSyllableTil': '\xed\x8b\xb8',
'HangulSyllableTik': '\xed\x8c\x88',
'HangulSyllableSeugs': '\xec\x8a\xa7',
'HangulSyllableTih': '\xed\x8c\x8b',
'HangulSyllableTit': '\xed\x8c\x89',
'HangulSyllableTis': '\xed\x8c\x83',
'HangulSyllableTip': '\xed\x8c\x8a',
'HangulSyllableSeugg': '\xec\x8a\xa6',
'HangulSyllableMyaenh': '\xeb\xa8\xa2',
'CjkUnifiedIdeograph5d5a': '\xe5\xb5\x9a',
'SquareCal': '\xe3\x8e\x88',
'HangulSyllableBwael': '\xeb\xb4\xb4',
'HangulSyllableBegs': '\xeb\xb2\xa3',
'SuperscriptOne': '\xc2\xb9',
'HangulSyllableTweng': '\xed\x89\xb9',
'SquareCap': '\xe2\x8a\x93',
'HangulSyllableBegg': '\xeb\xb2\xa2',
'HangulSyllableTenj': '\xed\x85\x91',
'UnicodeCharacter0xe30b': '\xee\x8c\x8b',
'GlagoliticCapitalLetterDzelo': '\xe2\xb0\x87',
'HangulSyllableTwenh': '\xed\x89\xaa',
'UnicodeCharacter0xe30e': '\xee\x8c\x8e',
'HangulSyllableTwenj': '\xed\x89\xa9',
'CjkUnifiedIdeograph3bd1': '\xe3\xaf\x91',
'surfaceintegral': '\xe2\x88\xaf',
'HangulSyllableMwaelb': '\xeb\xab\xab',
'HangulSyllableMwaelg': '\xeb\xab\xa9',
'CjkUnifiedIdeograph3bd0': '\xe3\xaf\x90',
'HangulSyllableMwaelh': '\xeb\xab\xaf',
'HangulSyllableMwaelm': '\xeb\xab\xaa',
'GlagoliticCapitalLetterOnu': '\xe2\xb0\x91',
'HangulSyllableMwaelp': '\xeb\xab\xae',
'HangulSyllableMwaels': '\xeb\xab\xac',
'CjkUnifiedIdeograph3736': '\xe3\x9c\xb6',
'HangulSyllableMwaelt': '\xeb\xab\xad',
'HangulSyllableTeng': '\xed\x85\xa1',
'HangulSyllableMyaenj': '\xeb\xa8\xa1',
'CjkUnifiedIdeograph41df': '\xe4\x87\x9f',
'CjkUnifiedIdeograph41de': '\xe4\x87\x9e',
'CjkUnifiedIdeograph41dd': '\xe4\x87\x9d',
'CjkUnifiedIdeograph41dc': '\xe4\x87\x9c',
'CjkUnifiedIdeograph41db': '\xe4\x87\x9b',
'CjkUnifiedIdeograph41da': '\xe4\x87\x9a',
'CjkUnifiedIdeograph3730': '\xe3\x9c\xb0',
'UnicodeCharacter0xe308': '\xee\x8c\x88',
'HangulSyllableJjyunj': '\xec\xae\xbd',
'UnionWithLogicalOr': '\xe2\xa9\x85',
'CjkUnifiedIdeograph3733': '\xe3\x9c\xb3',
'CjkUnifiedIdeograph5dbf': '\xe5\xb6\xbf',
'UnicodeCharacter0xe300': '\xee\x8c\x80',
'UnicodeCharacter0xe301': '\xee\x8c\x81',
'UnicodeCharacter0xe302': '\xee\x8c\x82',
'CjkUnifiedIdeograph3732': '\xe3\x9c\xb2',
'TibetanMarkGugRtagsGyon': '\xe0\xbc\xba',
'CjkUnifiedIdeograph5dbe': '\xe5\xb6\xbe',
'UnicodeCharacter0xe306': '\xee\x8c\x86',
'UnicodeCharacter0xe307': '\xee\x8c\x87',
'CjkUnifiedIdeograph3bd9': '\xe3\xaf\x99',
'CjkUnifiedIdeograph5dbd': '\xe5\xb6\xbd',
'CjkUnifiedIdeograph5dbc': '\xe5\xb6\xbc',
'CjkUnifiedIdeograph99bc': '\xe9\xa6\xbc',
'IdeographicAnnotationReverseMark': '\xe3\x86\x91',
'AplFunctionalSymbolQuoteQuad': '\xe2\x8d\x9e',
'CjkUnifiedIdeograph5dba': '\xe5\xb6\xba',
'CjkUnifiedIdeograph41d9': '\xe4\x87\x99',
'CjkUnifiedIdeograph41d8': '\xe4\x87\x98',
'CjkUnifiedIdeograph41d7': '\xe4\x87\x97',
'CjkUnifiedIdeograph41d6': '\xe4\x87\x96',
'CjkUnifiedIdeograph41d5': '\xe4\x87\x95',
'CjkUnifiedIdeograph41d4': '\xe4\x87\x94',
'CjkUnifiedIdeograph41d3': '\xe4\x87\x93',
'CjkUnifiedIdeograph41d2': '\xe4\x87\x92',
'CjkUnifiedIdeograph41d1': '\xe4\x87\x91',
'CjkUnifiedIdeograph41d0': '\xe4\x87\x90',
'HangulSyllableSsyolp': '\xec\x91\x96',
'HangulSyllableSsyols': '\xec\x91\x94',
'ChamVowelSignO': '\xea\xa8\xaf',
'notexists': '\xe2\x88\x84',
'ArabicSmallHighZain': '\xd8\x97',
'PresentationFormForVerticalHorizontalEllipsis': '\xef\xb8\x99',
'SundaneseVowelSignPamepet': '\xe1\xae\xa8',
'LatinSmallLetterNWithLeftHook': '\xc9\xb2',
'HangulFiller': '\xe3\x85\xa4',
'CjkUnifiedIdeograph8888': '\xe8\xa2\x88',
'CjkUnifiedIdeograph8889': '\xe8\xa2\x89',
'CjkUnifiedIdeograph5874': '\xe5\xa1\xb4',
'CjkUnifiedIdeograph53fa': '\xe5\x8f\xba',
'CjkUnifiedIdeograph53fb': '\xe5\x8f\xbb',
'HangulChoseongSiosphieuph': '\xe1\x84\xba',
'CjkUnifiedIdeograph53fd': '\xe5\x8f\xbd',
'CjkUnifiedIdeograph53fe': '\xe5\x8f\xbe',
'CjkUnifiedIdeograph53ff': '\xe5\x8f\xbf',
'CjkUnifiedIdeograph5873': '\xe5\xa1\xb3',
'UnicodeCharacter0xf459': '\xef\x91\x99',
'CjkUnifiedIdeograph3cf2': '\xe3\xb3\xb2',
'ArabicHamzaBelow': '\xd9\x95',
'MetricalTetraseme': '\xe2\x8f\x98',
'UnicodeCharacter0xf451': '\xef\x91\x91',
'CjkUnifiedIdeograph877a': '\xe8\x9d\xba',
'CjkUnifiedIdeograph877b': '\xe8\x9d\xbb',
'CjkUnifiedIdeograph877c': '\xe8\x9d\xbc',
'CjkUnifiedIdeograph877d': '\xe8\x9d\xbd',
'CjkUnifiedIdeograph3cf0': '\xe3\xb3\xb0',
'CjkUnifiedIdeograph877f': '\xe8\x9d\xbf',
'UnicodeCharacter0xf456': '\xef\x91\x96',
'UnicodeCharacter0xaae9': '\xea\xab\xa9',
'CjkUnifiedIdeograph4ece': '\xe4\xbb\x8e',
'HangulSyllableUg': '\xec\x9a\xb1',
'CjkUnifiedIdeograph4ecb': '\xe4\xbb\x8b',
'IdeographicTelegraphSymbolForHourSeventeen': '\xe3\x8d\xa9',
'CjkUnifiedIdeograph4ecc': '\xe4\xbb\x8c',
'CjkUnifiedIdeograph9c88': '\xe9\xb2\x88',
'CjkUnifiedIdeograph9c89': '\xe9\xb2\x89',
'HangulSyllableHwass': '\xed\x99\xa8',
'UnicodeCharacter0xe327': '\xee\x8c\xa7',
'CjkUnifiedIdeograph9c82': '\xe9\xb2\x82',
'CjkUnifiedIdeograph9c83': '\xe9\xb2\x83',
'CjkUnifiedIdeograph9c80': '\xe9\xb2\x80',
'CjkUnifiedIdeograph4eca': '\xe4\xbb\x8a',
'CjkUnifiedIdeograph9c86': '\xe9\xb2\x86',
'CjkUnifiedIdeograph9c87': '\xe9\xb2\x87',
'CjkUnifiedIdeograph9c84': '\xe9\xb2\x84',
'CjkUnifiedIdeograph9c85': '\xe9\xb2\x85',
'CjkUnifiedIdeograph8778': '\xe8\x9d\xb8',
'CjkUnifiedIdeograph8779': '\xe8\x9d\xb9',
'CjkUnifiedIdeograph8770': '\xe8\x9d\xb0',
'CjkUnifiedIdeograph8771': '\xe8\x9d\xb1',
'CjkUnifiedIdeograph8772': '\xe8\x9d\xb2',
'CjkUnifiedIdeograph8773': '\xe8\x9d\xb3',
'CjkUnifiedIdeograph8774': '\xe8\x9d\xb4',
'CjkUnifiedIdeograph8775': '\xe8\x9d\xb5',
'CjkUnifiedIdeograph8776': '\xe8\x9d\xb6',
'CjkUnifiedIdeograph8777': '\xe8\x9d\xb7',
'CjkUnifiedIdeograph53f8': '\xe5\x8f\xb8',
'CjkUnifiedIdeograph53f9': '\xe5\x8f\xb9',
'HangulSyllableHip': '\xed\x9e\xa2',
'UnicodeCharacter0xe320': '\xee\x8c\xa0',
'CjkUnifiedIdeograph53f0': '\xe5\x8f\xb0',
'CjkUnifiedIdeograph53f1': '\xe5\x8f\xb1',
'CjkUnifiedIdeograph53f2': '\xe5\x8f\xb2',
'CjkUnifiedIdeograph53f3': '\xe5\x8f\xb3',
'LepchaLetterCha': '\xe1\xb0\x87',
'CjkUnifiedIdeograph53f5': '\xe5\x8f\xb5',
'CjkUnifiedIdeograph53f6': '\xe5\x8f\xb6',
'CjkUnifiedIdeograph53f7': '\xe5\x8f\xb7',
'SinhalaLetterSanyakaDayanna': '\xe0\xb6\xb3',
'CjkUnifiedIdeograph9c8b': '\xe9\xb2\x8b',
'CjkUnifiedIdeograph9c8c': '\xe9\xb2\x8c',
'CjkUnifiedIdeograph9c8a': '\xe9\xb2\x8a',
'TibetanSignRdelDkarGsum': '\xe0\xbc\x9c',
'LatinCapitalLetterBWithTopbar': '\xc6\x82',
'CjkUnifiedIdeograph9c8d': '\xe9\xb2\x8d',
'CjkUnifiedIdeograph9c8e': '\xe9\xb2\x8e',
'HangulSyllableDdeulg': '\xeb\x9c\xb1',
'CjkUnifiedIdeograph702f': '\xe7\x80\xaf',
'UnicodeCharacter0xab6e': '\xea\xad\xae',
'Et': '\xce\x97',
'UnicodeCharacter0xab6d': '\xea\xad\xad',
'GreekSmallLetterLamda': '\xce\xbb',
'HangulSyllableSswass': '\xec\x90\x88',
'HangulSyllablePeolp': '\xed\x8e\x8a',
'UnicodeCharacter0xe2dd': '\xee\x8b\x9d',
'UnicodeCharacter0xe2df': '\xee\x8b\x9f',
'UnicodeCharacter0xe2da': '\xee\x8b\x9a',
'UnicodeCharacter0xe2dc': '\xee\x8b\x9c',
'UnicodeCharacter0xe2db': '\xee\x8b\x9b',
'CjkUnifiedIdeograph99be': '\xe9\xa6\xbe',
'HangulSyllablePong': '\xed\x90\x81',
'HangulSyllablePonh': '\xed\x8f\xb2',
'CjkUnifiedIdeograph3caf': '\xe3\xb2\xaf',
'CjkUnifiedIdeograph3cae': '\xe3\xb2\xae',
'CjkUnifiedIdeograph3cad': '\xe3\xb2\xad',
'CjkUnifiedIdeograph3cac': '\xe3\xb2\xac',
'CjkUnifiedIdeograph3cab': '\xe3\xb2\xab',
'CjkUnifiedIdeograph3caa': '\xe3\xb2\xaa',
'HangulSyllableMyogg': '\xeb\xac\x9a',
'CjkUnifiedIdeograph408b': '\xe4\x82\x8b',
'CjkUnifiedIdeograph408c': '\xe4\x82\x8c',
'CjkUnifiedIdeograph408a': '\xe4\x82\x8a',
'CjkUnifiedIdeograph408f': '\xe4\x82\x8f',
'IdeographicAnnotationLinkingMark': '\xe3\x86\x90',
'CjkUnifiedIdeograph408d': '\xe4\x82\x8d',
'CjkUnifiedIdeograph408e': '\xe4\x82\x8e',
'TibetanSubjoinedLetterTa': '\xe0\xbe\x9f',
'UnicodeCharacter0xf788': '\xef\x9e\x88',
'HangulSyllableBeugs': '\xeb\xb8\x8f',
'CjkUnifiedIdeograph3bda': '\xe3\xaf\x9a',
'HangulSyllableGagg': '\xea\xb0\x82',
'CjkUnifiedIdeograph3ca9': '\xe3\xb2\xa9',
'CjkUnifiedIdeograph3ca8': '\xe3\xb2\xa8',
'CjkUnifiedIdeograph3ca7': '\xe3\xb2\xa7',
'CjkUnifiedIdeograph3ca6': '\xe3\xb2\xa6',
'CjkUnifiedIdeograph3ca5': '\xe3\xb2\xa5',
'CjkUnifiedIdeograph3ca4': '\xe3\xb2\xa4',
'CjkUnifiedIdeograph3ca3': '\xe3\xb2\xa3',
'CjkUnifiedIdeograph3ca2': '\xe3\xb2\xa2',
'CjkUnifiedIdeograph3ca1': '\xe3\xb2\xa1',
'CjkUnifiedIdeograph3ca0': '\xe3\xb2\xa0',
'LeftwardsArrowOverRightwardsArrow': '\xe2\x87\x86',
'KangxiRadicalCity': '\xe2\xbe\xa2',
'CjkUnifiedIdeograph3bdb': '\xe3\xaf\x9b',
'UnicodeCharacter0xe2d9': '\xee\x8b\x99',
'HangulSyllableTiss': '\xed\x8c\x84',
'HangulSyllableHij': '\xed\x9e\x9e',
'UnicodeCharacter0xe2d5': '\xee\x8b\x95',
'CjkUnifiedIdeograph3bde': '\xe3\xaf\x9e',
'UnicodeCharacter0xe2d7': '\xee\x8b\x97',
'UnicodeCharacter0xe2d6': '\xee\x8b\x96',
'UnicodeCharacter0xe2d1': '\xee\x8b\x91',
'UnicodeCharacter0xe2d0': '\xee\x8b\x90',
'UnicodeCharacter0xe2d3': '\xee\x8b\x93',
'HangulSyllablePol': '\xed\x8f\xb4',
'HangulSyllableKeung': '\xed\x82\x81',
'GurmukhiLetterIi': '\xe0\xa8\x88',
'ArabicLetterHehDoachashmeeMedialForm': '\xef\xae\xad',
'HangulSyllableMyogs': '\xeb\xac\x9b',
'CjkUnifiedIdeograph3bdf': '\xe3\xaf\x9f',
'CjkUnifiedIdeograph4082': '\xe4\x82\x82',
'CjkUnifiedIdeograph4083': '\xe4\x82\x83',
'CjkUnifiedIdeograph4080': '\xe4\x82\x80',
'CjkUnifiedIdeograph4081': '\xe4\x82\x81',
'CjkUnifiedIdeograph4086': '\xe4\x82\x86',
'CjkUnifiedIdeograph4087': '\xe4\x82\x87',
'CjkUnifiedIdeograph4084': '\xe4\x82\x84',
'CjkUnifiedIdeograph4085': '\xe4\x82\x85',
'HangulSyllableJjyop': '\xec\xad\x86',
'CjkUnifiedIdeograph4088': '\xe4\x82\x88',
'CjkUnifiedIdeograph4089': '\xe4\x82\x89',
'UnicodeCharacter0xe833': '\xee\xa0\xb3',
'LargeTripleVerticalBarOperator': '\xe2\xab\xbc',
'CjkUnifiedIdeograph3daa': '\xe3\xb6\xaa',
'CjkUnifiedIdeograph3dab': '\xe3\xb6\xab',
'CjkUnifiedIdeograph3dac': '\xe3\xb6\xac',
'CjkUnifiedIdeograph3dad': '\xe3\xb6\xad',
'CjkUnifiedIdeograph3dae': '\xe3\xb6\xae',
'CjkUnifiedIdeograph3daf': '\xe3\xb6\xaf',
'HangulSyllableSseul': '\xec\x93\xb8',
'UnicodeCharacter0x97a': '\xe0\xa5\xba',
'HangulSyllableSseum': '\xec\x94\x80',
'CjkUnifiedIdeograph8737': '\xe8\x9c\xb7',
'UnicodeCharacter0xaad2': '\xea\xab\x92',
'NotAnElementOf': '\xe2\x88\x89',
'HangulSyllableSseuk': '\xec\x94\x88',
'HangulSyllableSseuh': '\xec\x94\x8b',
'UnicodeCharacter0xe489': '\xee\x92\x89',
'ModifierLetterUs': '\xea\x9d\xb0',
'UnicodeCharacter0xe487': '\xee\x92\x87',
'UnicodeCharacter0xe486': '\xee\x92\x86',
'UnicodeCharacter0xe485': '\xee\x92\x85',
'HangulSyllableGgyilp': '\xeb\x81\xae',
'UnicodeCharacter0xe483': '\xee\x92\x83',
'UnicodeCharacter0xe482': '\xee\x92\x82',
'UnicodeCharacter0xe481': '\xee\x92\x81',
'HangulSyllableCwaek': '\xec\xb5\x98',
'CjkUnifiedIdeograph4ec6': '\xe4\xbb\x86',
'UnicodeCharacter0xdef4': '\xed\xbb\xb4',
'CjkUnifiedIdeograph4ec7': '\xe4\xbb\x87',
'UnicodeCharacter0xdef3': '\xed\xbb\xb3',
'CjkUnifiedIdeograph8604': '\xe8\x98\x84',
'HangulSyllableGags': '\xea\xb0\x83',
'UnicodeCharacter0x5ec': '\xd7\xac',
'HangulSyllableJyulh': '\xec\xa5\xbb',
'HangulSyllableNeup': '\xeb\x8a\xaa',
'HangulSyllableNeus': '\xeb\x8a\xa3',
'LatinCapitalLetterLWithDotBelowAndMacron': '\xe1\xb8\xb8',
'HangulSyllableNeut': '\xeb\x8a\xa9',
'CjkUnifiedIdeograph3cff': '\xe3\xb3\xbf',
'UnicodeCharacter0xdef0': '\xed\xbb\xb0',
'HangulSyllableSseuc': '\xec\x94\x87',
'CjkUnifiedIdeograph3da0': '\xe3\xb6\xa0',
'CjkUnifiedIdeograph3da1': '\xe3\xb6\xa1',
'CjkUnifiedIdeograph3da2': '\xe3\xb6\xa2',
'CjkUnifiedIdeograph3da3': '\xe3\xb6\xa3',
'CjkUnifiedIdeograph3da4': '\xe3\xb6\xa4',
'CjkUnifiedIdeograph3cfd': '\xe3\xb3\xbd',
'UnicodeCharacter0x979': '\xe0\xa5\xb9',
'UnicodeCharacter0x978': '\xe0\xa5\xb8',
'UnicodeCharacter0x977': '\xe0\xa5\xb7',
'UnicodeCharacter0x976': '\xe0\xa5\xb6',
'UnicodeCharacter0x975': '\xe0\xa5\xb5',
'UnicodeCharacter0x974': '\xe0\xa5\xb4',
'UnicodeCharacter0x973': '\xe0\xa5\xb3',
'HangulSyllableNeul': '\xeb\x8a\x98',
'HangulSyllableNeun': '\xeb\x8a\x94',
'ChamConsonantSignFinalNg': '\xea\xa9\x83',
'CjkUnifiedIdeograph797b': '\xe7\xa5\xbb',
'CanadianSyllabicsNaskapiStwa': '\xe1\x94\x8d',
'HangulSyllablePoet': '\xed\x91\x99',
'HangulSyllableSsyong': '\xec\x91\x9d',
'SmallElementOfWithVerticalBarAtEndOfHorizontalStroke': '\xe2\x8b\xb4',
'UnicodeCharacter0xdef9': '\xed\xbb\xb9',
'UnicodeCharacter0xec46': '\xee\xb1\x86',
'UnicodeCharacter0xe48f': '\xee\x92\x8f',
'UnicodeCharacter0xe48e': '\xee\x92\x8e',
'UnicodeCharacter0xdef8': '\xed\xbb\xb8',
'UnicodeCharacter0xe48c': '\xee\x92\x8c',
'UnicodeCharacter0xe48b': '\xee\x92\x8b',
'UnicodeCharacter0xe48a': '\xee\x92\x8a',
'HangulSyllablePoep': '\xed\x91\x9a',
'TeluguDigitEight': '\xe0\xb1\xae',
'CjkUnifiedIdeograph4db5': '\xe4\xb6\xb5',
'UnicodeCharacter0xf8be': '\xef\xa2\xbe',
'CjkUnifiedIdeograph4db0': '\xe4\xb6\xb0',
'CjkUnifiedIdeograph4db1': '\xe4\xb6\xb1',
'CjkUnifiedIdeograph4db2': '\xe4\xb6\xb2',
'CjkUnifiedIdeograph4db3': '\xe4\xb6\xb3',
'LatinSmallLetterOpenOWithRetroflexHook': '\xe1\xb6\x97',
'HangulSyllableSsilg': '\xec\x94\xb1',
'NkoCombiningDoubleDotAbove': '\xdf\xb3',
'HangulSyllableGgwid': '\xeb\x80\x93',
'YiSyllableYup': '\xea\x92\x84',
'CjkUnifiedIdeograph9d56': '\xe9\xb5\x96',
'HangulSyllableGgwib': '\xeb\x80\x9d',
'HangulSyllableGgwic': '\xeb\x80\xa3',
'CjkUnifiedIdeograph5bdc': '\xe5\xaf\x9c',
'CjkUnifiedIdeograph5bdb': '\xe5\xaf\x9b',
'CjkUnifiedIdeograph5bda': '\xe5\xaf\x9a',
'HangulSyllableGgwih': '\xeb\x80\xa7',
'CjkUnifiedIdeograph5bdf': '\xe5\xaf\x9f',
'CjkUnifiedIdeograph5bde': '\xe5\xaf\x9e',
'CjkUnifiedIdeograph5bdd': '\xe5\xaf\x9d',
'HangulSyllableRyae': '\xeb\x9f\x90',
'HangulSyllableRyad': '\xeb\x9e\xbb',
'CjkUnifiedIdeograph5029': '\xe5\x80\xa9',
'CjkUnifiedIdeograph5028': '\xe5\x80\xa8',
'HangulSyllableTeolh': '\xed\x84\xbf',
'HangulSyllableRyac': '\xeb\x9f\x8b',
'ArabicindicPerTenThousandSign': '\xd8\x8a',
'CjkUnifiedIdeograph5023': '\xe5\x80\xa3',
'CjkUnifiedIdeograph5022': '\xe5\x80\xa2',
'CjkUnifiedIdeograph5021': '\xe5\x80\xa1',
'CjkUnifiedIdeograph5020': '\xe5\x80\xa0',
'CjkUnifiedIdeograph5027': '\xe5\x80\xa7',
'CjkUnifiedIdeograph5026': '\xe5\x80\xa6',
'CjkUnifiedIdeograph5025': '\xe5\x80\xa5',
'CjkUnifiedIdeograph5024': '\xe5\x80\xa4',
'CjkUnifiedIdeograph74c6': '\xe7\x93\x86',
'CjkUnifiedIdeograph74c7': '\xe7\x93\x87',
'CjkUnifiedIdeograph74c4': '\xe7\x93\x84',
'CjkUnifiedIdeograph74c5': '\xe7\x93\x85',
'CjkUnifiedIdeograph74c2': '\xe7\x93\x82',
'CjkUnifiedIdeograph74c3': '\xe7\x93\x83',
'CjkUnifiedIdeograph74c0': '\xe7\x93\x80',
'CjkUnifiedIdeograph74c1': '\xe7\x93\x81',
'CjkUnifiedIdeograph9913': '\xe9\xa4\x93',
'Quincunx': '\xe2\x9a\xbb',
'CjkUnifiedIdeograph9911': '\xe9\xa4\x91',
'CjkUnifiedIdeograph9910': '\xe9\xa4\x90',
'CjkUnifiedIdeograph9917': '\xe9\xa4\x97',
'CjkUnifiedIdeograph9916': '\xe9\xa4\x96',
'CjkUnifiedIdeograph74c8': '\xe7\x93\x88',
'CjkUnifiedIdeograph74c9': '\xe7\x93\x89',
'HangulSyllableByals': '\xeb\xb1\x98',
'CjkUnifiedIdeograph5bd9': '\xe5\xaf\x99',
'EthiopicSyllablePhoa': '\xe2\xb6\x91',
'HalfwidthKatakanaLetterSmallU': '\xef\xbd\xa9',
'UnicodeCharacter0xf73c': '\xef\x9c\xbc',
'CjkUnifiedIdeograph5bd3': '\xe5\xaf\x93',
'CjkUnifiedIdeograph5bd2': '\xe5\xaf\x92',
'CjkUnifiedIdeograph5bd1': '\xe5\xaf\x91',
'CjkUnifiedIdeograph5bd0': '\xe5\xaf\x90',
'CjkUnifiedIdeograph5bd7': '\xe5\xaf\x97',
'CjkUnifiedIdeograph5bd6': '\xe5\xaf\x96',
'DoesNotContainAsNormalSubgroup': '\xe2\x8b\xab',
'CjkUnifiedIdeograph5bd4': '\xe5\xaf\x94',
'HalfwidthKatakanaLetterSmallA': '\xef\xbd\xa7',
'HalfwidthKatakanaLetterSmallE': '\xef\xbd\xaa',
'HalfwidthKatakanaLetterSmallI': '\xef\xbd\xa8',
'HangulSyllableNogg': '\xeb\x85\xba',
'HangulSyllableByubs': '\xeb\xb8\x82',
'UnicodeCharacter0xde9d': '\xed\xba\x9d',
'HalfwidthKatakanaLetterSmallO': '\xef\xbd\xab',
'CjkUnifiedIdeograph74cf': '\xe7\x93\x8f',
'CjkUnifiedIdeograph74cd': '\xe7\x93\x8d',
'CjkUnifiedIdeograph74ce': '\xe7\x93\x8e',
'CjkUnifiedIdeograph74cb': '\xe7\x93\x8b',
'CjkUnifiedIdeograph74cc': '\xe7\x93\x8c',
'KannadaLetterSsa': '\xe0\xb2\xb7',
'CjkUnifiedIdeograph991c': '\xe9\xa4\x9c',
'CjkUnifiedIdeograph991b': '\xe9\xa4\x9b',
'CjkUnifiedIdeograph991a': '\xe9\xa4\x9a',
'HangulSyllableDdyes': '\xeb\x98\x87',
'CjkUnifiedIdeograph991f': '\xe9\xa4\x9f',
'CjkUnifiedIdeograph991e': '\xe9\xa4\x9e',
'CjkUnifiedIdeograph991d': '\xe9\xa4\x9d',
'UnicodeCharacter0xeb20': '\xee\xac\xa0',
'CjkUnifiedIdeograph45c8': '\xe4\x97\x88',
'CjkUnifiedIdeograph502b': '\xe5\x80\xab',
'CjkUnifiedIdeograph502a': '\xe5\x80\xaa',
'LargeCircle': '\xe2\x97\xaf',
'CjkUnifiedIdeograph502f': '\xe5\x80\xaf',
'CjkUnifiedIdeograph502e': '\xe5\x80\xae',
'CjkUnifiedIdeograph502d': '\xe5\x80\xad',
'CjkUnifiedIdeograph6648': '\xe6\x99\x88',
'CjkUnifiedIdeograph6649': '\xe6\x99\x89',
'HangulSyllableJugs': '\xec\xa3\xbf',
'ThaanaLetterSheenu': '\xde\x9d',
'CjkUnifiedIdeograph6642': '\xe6\x99\x82',
'CjkUnifiedIdeograph6643': '\xe6\x99\x83',
'CjkUnifiedIdeograph6640': '\xe6\x99\x80',
'CjkUnifiedIdeograph6641': '\xe6\x99\x81',
'CjkUnifiedIdeograph6646': '\xe6\x99\x86',
'CjkUnifiedIdeograph6647': '\xe6\x99\x87',
'CjkUnifiedIdeograph6644': '\xe6\x99\x84',
'CjkUnifiedIdeograph6645': '\xe6\x99\x85',
'CjkUnifiedIdeograph486d': '\xe4\xa1\xad',
'CjkUnifiedIdeograph486e': '\xe4\xa1\xae',
'CjkUnifiedIdeograph486f': '\xe4\xa1\xaf',
'CjkUnifiedIdeograph486a': '\xe4\xa1\xaa',
'CjkUnifiedIdeograph486b': '\xe4\xa1\xab',
'CjkUnifiedIdeograph486c': '\xe4\xa1\xac',
'CyrillicSmallLetterAbkhasianCheWithDescender': '\xd2\xbf',
'UnicodeCharacter0xf435': '\xef\x90\xb5',
'BraillePatternDots135': '\xe2\xa0\x95',
'UnicodeCharacter0xe418': '\xee\x90\x98',
'UnicodeCharacter0xf434': '\xef\x90\xb4',
'CjkUnifiedIdeograph8e16': '\xe8\xb8\x96',
'UnicodeCharacter0xe419': '\xee\x90\x99',
'UnicodeCharacter0xf433': '\xef\x90\xb3',
'EthiopicSyllableDdi': '\xe1\x8b\xba',
'ArabicLetterRnoonIsolatedForm': '\xef\xae\xa0',
'UnicodeCharacter0xf432': '\xef\x90\xb2',
'CanadianSyllabicsCarrierYee': '\xe1\x98\x91',
'<-|': '\xe2\x86\xa4',
'UnicodeCharacter0xf8bc': '\xef\xa2\xbc',
'HangulSyllableJwagg': '\xec\xa2\x8e',
'HangulSyllableTeolm': '\xed\x84\xba',
'UnicodeCharacter0xf430': '\xef\x90\xb0',
'HangulSyllableHoenj': '\xed\x9a\x91',
'CjkUnifiedIdeograph4864': '\xe4\xa1\xa4',
'CjkUnifiedIdeograph4865': '\xe4\xa1\xa5',
'CjkUnifiedIdeograph4866': '\xe4\xa1\xa6',
'CjkUnifiedIdeograph4867': '\xe4\xa1\xa7',
'CjkUnifiedIdeograph4860': '\xe4\xa1\xa0',
'CjkUnifiedIdeograph4861': '\xe4\xa1\xa1',
'GreekSmallLetterUpsilonWithPerispomeni': '\xe1\xbf\xa6',
'CjkUnifiedIdeograph4863': '\xe4\xa1\xa3',
'MaleWithStrokeAndMaleAndFemaleSign': '\xe2\x9a\xa7',
'CjkUnifiedIdeograph4868': '\xe4\xa1\xa8',
'CjkUnifiedIdeograph4869': '\xe4\xa1\xa9',
'AplFunctionalSymbolDelDiaeresis': '\xe2\x8d\xa2',
'YiSyllableFop': '\xea\x83\x97',
'CjkUnifiedIdeograph7abf': '\xe7\xaa\xbf',
'UnicodeCharacter0xe411': '\xee\x90\x91',
'LatinCapitalLetterCWithStroke': '\xc8\xbb',
'CjkUnifiedIdeograph664c': '\xe6\x99\x8c',
'CjkUnifiedIdeograph664a': '\xe6\x99\x8a',
'CjkUnifiedIdeograph664f': '\xe6\x99\x8f',
'CjkUnifiedIdeograph664d': '\xe6\x99\x8d',
'CjkUnifiedIdeograph664e': '\xe6\x99\x8e',
'UnicodeCharacter0xe413': '\xee\x90\x93',
'CjkUnifiedIdeograph9d51': '\xe9\xb5\x91',
'UnicodeCharacter0xe414': '\xee\x90\x94',
'NkoExclamationMark': '\xdf\xb9',
'ChamDigitSeven': '\xea\xa9\x97',
'MongolianLetterTodoMa': '\xe1\xa1\x8f',
'LatinCapitalLetterCWithCedilla': '\xc3\x87',
'cubed': '\xc2\xb3',
'UnicodeCharacter0xe416': '\xee\x90\x96',
'YiSyllableHxit': '\xea\x89\x86',
'ArabicLetterAlefWithMaddaAboveFinalForm': '\xef\xba\x82',
'UnicodeCharacter0xe417': '\xee\x90\x97',
'HangulSyllableKwib': '\xed\x81\x85',
'HangulSyllableSeulg': '\xec\x8a\xad',
'GreekLetterArchaicKoppa': '\xcf\x98',
'ArabicEndOfAyah': '\xdb\x9d',
'CjkUnifiedIdeograph3e33': '\xe3\xb8\xb3',
'CjkUnifiedIdeograph3e32': '\xe3\xb8\xb2',
'CjkUnifiedIdeograph3e31': '\xe3\xb8\xb1',
'CjkUnifiedIdeograph3e30': '\xe3\xb8\xb0',
'CjkUnifiedIdeograph3e37': '\xe3\xb8\xb7',
'CjkUnifiedIdeograph3e36': '\xe3\xb8\xb6',
'CjkUnifiedIdeograph3e35': '\xe3\xb8\xb5',
'CjkUnifiedIdeograph3e34': '\xe3\xb8\xb4',
'CjkUnifiedIdeograph3e39': '\xe3\xb8\xb9',
'CjkUnifiedIdeograph3e38': '\xe3\xb8\xb8',
'CjkUnifiedIdeograph6eee': '\xe6\xbb\xae',
'UnicodeCharacter0xab5b': '\xea\xad\x9b',
'CjkUnifiedIdeograph7519': '\xe7\x94\x99',
'CjkUnifiedIdeograph7518': '\xe7\x94\x98',
'HangulSyllableGeb': '\xea\xb2\x9d',
'HangulSyllableGec': '\xea\xb2\xa3',
'HangulSyllableGed': '\xea\xb2\x93',
'HangulSyllableGeg': '\xea\xb2\x8d',
'CjkUnifiedIdeograph3b8e': '\xe3\xae\x8e',
'CjkUnifiedIdeograph3b8d': '\xe3\xae\x8d',
'CjkUnifiedIdeograph7513': '\xe7\x94\x93',
'CjkUnifiedIdeograph3b8f': '\xe3\xae\x8f',
'CjkUnifiedIdeograph3b8a': '\xe3\xae\x8a',
'CjkUnifiedIdeograph7514': '\xe7\x94\x94',
'CjkUnifiedIdeograph3b8c': '\xe3\xae\x8c',
'CjkUnifiedIdeograph3b8b': '\xe3\xae\x8b',
'UnicodeCharacter0x17de': '\xe1\x9f\x9e',
'SquareWb': '\xe3\x8f\x9d',
'CjkUnifiedIdeograph9477': '\xe9\x91\xb7',
'UnicodeCharacter0xaa8b': '\xea\xaa\x8b',
'SyriacAbbreviationMark': '\xdc\x8f',
'UnicodeCharacter0xab5c': '\xea\xad\x9c',
'HangulSyllableGgwals': '\xea\xbd\x94',
'UnicodeCharacter0xaa8c': '\xea\xaa\x8c',
'HangulSyllableGgwalm': '\xea\xbd\x92',
'ArabicLigatureNoonWithYehFinalForm': '\xef\xb2\x8f',
'GreekSmallLetterUpsilonWithDialytikaAndPerispomeni': '\xe1\xbf\xa7',
'UnicodeCharacter0xea83': '\xee\xaa\x83',
'HangulSyllableGgwalg': '\xea\xbd\x91',
'UnicodeCharacter0xaa8a': '\xea\xaa\x8a',
'YiRadicalDdur': '\xea\x92\xa4',
'HangulSyllableGgwalb': '\xea\xbd\x93',
'HangulSyllableDoeh': '\xeb\x90\xb3',
'HangulSyllableDoek': '\xeb\x90\xb0',
'HangulSyllableDoej': '\xeb\x90\xae',
'CjkUnifiedIdeograph3b89': '\xe3\xae\x89',
'CjkUnifiedIdeograph3b88': '\xe3\xae\x88',
'HangulSyllableDoen': '\xeb\x90\x9c',
'CjkUnifiedIdeograph3b85': '\xe3\xae\x85',
'CjkUnifiedIdeograph3b84': '\xe3\xae\x84',
'CjkUnifiedIdeograph3b87': '\xe3\xae\x87',
'CjkUnifiedIdeograph3b86': '\xe3\xae\x86',
'CjkUnifiedIdeograph3b81': '\xe3\xae\x81',
'CjkUnifiedIdeograph3b80': '\xe3\xae\x80',
'CjkUnifiedIdeograph3b83': '\xe3\xae\x83',
'CjkUnifiedIdeograph3b82': '\xe3\xae\x82',
'CjkUnifiedIdeograph3e3c': '\xe3\xb8\xbc',
'CjkUnifiedIdeograph3e3b': '\xe3\xb8\xbb',
'CjkUnifiedIdeograph3e3a': '\xe3\xb8\xba',
'CjkUnifiedIdeograph3e3f': '\xe3\xb8\xbf',
'CjkUnifiedIdeograph3e3e': '\xe3\xb8\xbe',
'CjkUnifiedIdeograph3e3d': '\xe3\xb8\xbd',
'HangulSyllableDoep': '\xeb\x90\xb2',
'HangulSyllableDoes': '\xeb\x90\xab',
'HangulSyllableDoet': '\xeb\x90\xb1',
'HangulSyllableDis': '\xeb\x94\xa7',
'UnicodeCharacter0xab5a': '\xea\xad\x9a',
'HangulSyllableHelt': '\xed\x97\xb1',
'UnicodeCharacter0xaa99': '\xea\xaa\x99',
'CjkUnifiedIdeograph8f7c': '\xe8\xbd\xbc',
'UnicodeCharacter0xea81': '\xee\xaa\x81',
'UnicodeCharacter0xdf81': '\xed\xbe\x81',
'CjkUnifiedIdeograph95bb': '\xe9\x96\xbb',
'HangulSyllableGgyelh': '\xea\xbc\x9f',
'ArabicLigatureHehWithAlefMaksuraIsolatedForm': '\xef\xb1\x93',
'HangulSyllableNalp': '\xeb\x82\xa6',
'KangxiRadicalArrow': '\xe2\xbd\xae',
'HangulSyllableSyang': '\xec\x83\xb9',
'UnicodeCharacter0xab5f': '\xea\xad\x9f',
'UnicodeCharacter0xaa98': '\xea\xaa\x98',
'CjkUnifiedIdeograph8f7b': '\xe8\xbd\xbb',
'UnicodeCharacter0xea86': '\xee\xaa\x86',
'HangulSyllableJwabs': '\xec\xa2\x9e',
'HangulSyllableSyanh': '\xec\x83\xaa',
'CjkUnifiedIdeograph7a7c': '\xe7\xa9\xbc',
'UnicodeCharacter0xf43e': '\xef\x90\xbe',
'CjkUnifiedIdeograph7a7b': '\xe7\xa9\xbb',
'HangulSyllableSyanj': '\xec\x83\xa9',
'HangulSyllableSeulp': '\xec\x8a\xb2',
'HangulSyllableCeogg': '\xec\xb2\x9a',
'UnicodeCharacter0xf43c': '\xef\x90\xbc',
'CjkUnifiedIdeograph8f7e': '\xe8\xbd\xbe',
'UnicodeCharacter0xf43b': '\xef\x90\xbb',
'UnicodeCharacter0xea87': '\xee\xaa\x87',
'CjkUnifiedIdeograph6369': '\xe6\x8d\xa9',
'HangulSyllableCeogs': '\xec\xb2\x9b',
'UnicodeCharacter0xf43a': '\xef\x90\xba',
'CjkUnifiedIdeograph6368': '\xe6\x8d\xa8',
'CjkUnifiedIdeograph8711': '\xe8\x9c\x91',
'LimbuDigitSeven': '\xe1\xa5\x8d',
'ArabicLigatureSheenWithJeemWithYehFinalForm': '\xef\xb5\xa9',
'ArabicLigatureBehWithMeemInitialForm': '\xef\xb2\x9f',
'CjkUnifiedIdeograph589c': '\xe5\xa2\x9c',
'HangulSyllableDdyaels': '\xeb\x96\x90',
'HangulSyllableDdyaelp': '\xeb\x96\x92',
'UnicodeCharacter0xe0fe': '\xee\x83\xbe',
'UnicodeCharacter0xea84': '\xee\xaa\x84',
'HangulSyllableDdyaelt': '\xeb\x96\x91',
'HangulSyllableDdyaelh': '\xeb\x96\x93',
'UnicodeCharacter0x560': '\xd5\xa0',
'HangulSyllableJyaet': '\xec\x9f\xbd',
'HangulSyllableDdyaelm': '\xeb\x96\x8e',
'HangulSyllableGgyelm': '\xea\xbc\x9a',
'HangulSyllableDdyaelg': '\xeb\x96\x8d',
'UnicodeCharacter0xab5e': '\xea\xad\x9e',
'HangulSyllableJjelm': '\xec\xa9\xb2',
'HangulSyllableJjelh': '\xec\xa9\xb7',
'HangulSyllableJjelg': '\xec\xa9\xb1',
'SundaneseSignPangwisad': '\xe1\xae\x82',
'UnicodeCharacter0xea85': '\xee\xaa\x85',
'HangulSyllableJjelb': '\xec\xa9\xb3',
'VaiSyllableCee': '\xea\x94\x9a',
'CjkUnifiedIdeograph4f68': '\xe4\xbd\xa8',
'CjkUnifiedIdeograph4f69': '\xe4\xbd\xa9',
'HangulSyllableSyenh': '\xec\x85\xb6',
'HangulSyllableSyenj': '\xec\x85\xb5',
'CjkUnifiedIdeograph4f62': '\xe4\xbd\xa2',
'BugineseLetterDa': '\xe1\xa8\x89',
'CjkUnifiedIdeograph4f60': '\xe4\xbd\xa0',
'CjkUnifiedIdeograph4f61': '\xe4\xbd\xa1',
'CjkUnifiedIdeograph4f66': '\xe4\xbd\xa6',
'CjkUnifiedIdeograph4f67': '\xe4\xbd\xa7',
'CjkUnifiedIdeograph4f64': '\xe4\xbd\xa4',
'CjkUnifiedIdeograph4f65': '\xe4\xbd\xa5',
'HangulSyllableBbweon': '\xeb\xbf\xac',
'HangulSyllableBbweol': '\xeb\xbf\xb0',
'HangulSyllableBbweom': '\xeb\xbf\xb8',
'HangulSyllableBbweoj': '\xeb\xbf\xbe',
'HangulSyllableBbweok': '\xec\x80\x80',
'HangulSyllableBbweoh': '\xec\x80\x83',
'CjkUnifiedIdeograph8f7f': '\xe8\xbd\xbf',
'HangulSyllableBbweog': '\xeb\xbf\xa9',
'HangulSyllableBbweod': '\xeb\xbf\xaf',
'HangulSyllableBbweob': '\xeb\xbf\xb9',
'HangulSyllableBbwaen': '\xeb\xbd\xbc',
'HangulSyllableGuh': '\xea\xb6\x87',
'HangulSyllableBbwaeh': '\xeb\xbe\x93',
'HangulSyllableBbweot': '\xec\x80\x81',
'HangulSyllableNyic': '\xeb\x8b\x83',
'HangulSyllableBbweop': '\xec\x80\x82',
'CjkCompatibilityIdeographf946': '\xef\xa5\x86',
'HangulSyllableGuk': '\xea\xb6\x84',
'CjkUnifiedIdeograph4f6b': '\xe4\xbd\xab',
'CjkUnifiedIdeograph4f6c': '\xe4\xbd\xac',
'HangulSyllableKwim': '\xed\x81\x84',
'CjkUnifiedIdeograph4f6a': '\xe4\xbd\xaa',
'CjkUnifiedIdeograph4f6f': '\xe4\xbd\xaf',
'HangulSyllableRobs': '\xeb\xa1\xae',
'CjkUnifiedIdeograph4f6d': '\xe4\xbd\xad',
'CjkUnifiedIdeograph4f6e': '\xe4\xbd\xae',
'HangulSyllableBbwaed': '\xeb\xbd\xbf',
'DownwardsArrowWithHorizontalStroke': '\xe2\xa4\x88',
'HangulSyllableNyig': '\xeb\x8a\xad',
'HangulSyllableWaelh': '\xec\x99\xab',
'VaiSymbolKung': '\xea\x98\x9c',
'SquareKirowatto': '\xe3\x8c\x97',
'CjkUnifiedIdeograph5766': '\xe5\x9d\xa6',
'YiSyllableNzex': '\xea\x8b\xbc',
'HangulSyllableRyeolt': '\xeb\xa0\xb1',
'CjkUnifiedIdeograph5767': '\xe5\x9d\xa7',
'HangulSyllableRyeolp': '\xeb\xa0\xb2',
'HangulSyllableGgoelp': '\xea\xbe\x8e',
'UnicodeCharacter0xdf88': '\xed\xbe\x88',
'HangulSyllableGgoels': '\xea\xbe\x8c',
'ArabicLigatureTehWithJeemIsolatedForm': '\xef\xb0\x8b',
'CjkRadicalWaterTwo': '\xe2\xba\xa2',
'HangulSyllableRwa': '\xeb\xa1\xb8',
'HangulSyllableRyeolb': '\xeb\xa0\xaf',
'HangulSyllableRyeolm': '\xeb\xa0\xae',
'HangulSyllableBbwaec': '\xeb\xbe\x8f',
'HangulSyllableRyeolh': '\xeb\xa0\xb3',
'GurmukhiLetterRa': '\xe0\xa8\xb0',
'HangulSyllableGgoelt': '\xea\xbe\x8d',
'HangulChoseongIeungsios': '\xe1\x85\x85',
'CjkUnifiedIdeograph6ea8': '\xe6\xba\xa8',
'CjkUnifiedIdeograph6ea9': '\xe6\xba\xa9',
'UnicodeCharacter0xc64': '\xe0\xb1\xa4',
'CircledHangulPieupA': '\xe3\x89\xb3',
'CjkUnifiedIdeograph6ea2': '\xe6\xba\xa2',
'CjkUnifiedIdeograph6ea3': '\xe6\xba\xa3',
'CjkUnifiedIdeograph6ea0': '\xe6\xba\xa0',
'CjkUnifiedIdeograph6ea1': '\xe6\xba\xa1',
'NormalSubgroupOfOrEqualTo': '\xe2\x8a\xb4',
'CjkUnifiedIdeograph6ea7': '\xe6\xba\xa7',
'CjkUnifiedIdeograph6ea4': '\xe6\xba\xa4',
'CjkUnifiedIdeograph6ea5': '\xe6\xba\xa5',
'CjkUnifiedIdeograph71e2': '\xe7\x87\xa2',
'GujaratiLetterI': '\xe0\xaa\x87',
'HangulSyllableJjwabs': '\xec\xab\xaa',
'HangulSyllableNyis': '\xeb\x8a\xbf',
'LeftwardsHarpoonWithBarbDownBelowLongDash': '\xe2\xa5\xab',
'HeavySingleCommaQuotationMarkOrnament': '\xe2\x9d\x9c',
'UnicodeCharacter0xf0d5': '\xef\x83\x95',
'LatinSmallLetterQWithHook': '\xca\xa0',
'UnicodeCharacter0xe4e4': '\xee\x93\xa4',
'BoxDrawingsVerticalSingleAndRightDouble': '\xe2\x95\x9e',
'ArabicLigatureNoonWithKhahIsolatedForm': '\xef\xb1\x8d',
'HangulSyllableJyaem': '\xec\x9f\xb4',
'UnicodeCharacter0xeb24': '\xee\xac\xa4',
'ArabicLigatureQafWithMeemWithHahFinalForm': '\xef\xb5\xbe',
'CjkUnifiedIdeograph3e27': '\xe3\xb8\xa7',
'CjkUnifiedIdeograph6eab': '\xe6\xba\xab',
'CjkUnifiedIdeograph6eac': '\xe6\xba\xac',
'CjkUnifiedIdeograph6eaa': '\xe6\xba\xaa',
'CjkUnifiedIdeograph6eaf': '\xe6\xba\xaf',
'CjkUnifiedIdeograph6ead': '\xe6\xba\xad',
'CjkUnifiedIdeograph6eae': '\xe6\xba\xae',
'LatinCapitalLetterEWithCircumflexAndAcute': '\xe1\xba\xbe',
'HangulChoseongPieup': '\xe1\x84\x87',
'CjkUnifiedIdeograph4a77': '\xe4\xa9\xb7',
'UnicodeCharacter0xe1dd': '\xee\x87\x9d',
'KangxiRadicalWalk': '\xe2\xbe\xa1',
'CjkUnifiedIdeograph5eaa': '\xe5\xba\xaa',
'CjkUnifiedIdeograph4a73': '\xe4\xa9\xb3',
'CjkUnifiedIdeograph3c84': '\xe3\xb2\x84',
'BraillePatternDots1458': '\xe2\xa2\x99',
'UnicodeCharacter0xe1de': '\xee\x87\x9e',
'UnicodeCharacter0xb04': '\xe0\xac\x84',
'CjkUnifiedIdeograph5dd9': '\xe5\xb7\x99',
'UnicodeCharacter0xb00': '\xe0\xac\x80',
'CjkUnifiedIdeograph8576': '\xe8\x95\xb6',
'CjkUnifiedIdeograph8577': '\xe8\x95\xb7',
'CjkUnifiedIdeograph6de9': '\xe6\xb7\xa9',
'CjkUnifiedIdeograph3c87': '\xe3\xb2\x87',
'CjkUnifiedIdeograph8572': '\xe8\x95\xb2',
'ThaanaLetterSaadhu': '\xde\x9e',
'CjkUnifiedIdeograph8570': '\xe8\x95\xb0',
'CjkUnifiedIdeograph8571': '\xe8\x95\xb1',
'CjkUnifiedIdeograph6de3': '\xe6\xb7\xa3',
'CjkUnifiedIdeograph3c80': '\xe3\xb2\x80',
'CjkUnifiedIdeograph6de1': '\xe6\xb7\xa1',
'CjkUnifiedIdeograph6de0': '\xe6\xb7\xa0',
'CjkUnifiedIdeograph6de7': '\xe6\xb7\xa7',
'CjkUnifiedIdeograph6de6': '\xe6\xb7\xa6',
'CjkUnifiedIdeograph6de5': '\xe6\xb7\xa5',
'CjkUnifiedIdeograph3c81': '\xe3\xb2\x81',
'HangulSyllablePwalm': '\xed\x90\x92',
'CjkUnifiedIdeograph3c82': '\xe3\xb2\x82',
'HangulSyllablePwalh': '\xed\x90\x97',
'HangulChoseongSioshieuh': '\xe1\x84\xbb',
'HangulSyllablePwalg': '\xed\x90\x91',
'CjkUnifiedIdeograph3c83': '\xe3\xb2\x83',
'HangulSyllablePwalb': '\xed\x90\x93',
'CjkUnifiedIdeograph44af': '\xe4\x92\xaf',
'UnicodeCharacter0xaa83': '\xea\xaa\x83',
'CjkUnifiedIdeograph947b': '\xe9\x91\xbb',
'HeavyExclamationMarkOrnament': '\xe2\x9d\xa2',
'CjkUnifiedIdeograph4abe': '\xe4\xaa\xbe',
'CjkUnifiedIdeograph4abd': '\xe4\xaa\xbd',
'HangulSyllablePwalt': '\xed\x90\x95',
'CjkUnifiedIdeograph4abf': '\xe4\xaa\xbf',
'CjkUnifiedIdeograph4aba': '\xe4\xaa\xba',
'HangulSyllablePwals': '\xed\x90\x94',
'CjkUnifiedIdeograph4abc': '\xe4\xaa\xbc',
'CjkUnifiedIdeograph4abb': '\xe4\xaa\xbb',
'CjkUnifiedIdeograph857f': '\xe8\x95\xbf',
'EmptySetWithSmallCircleAbove': '\xe2\xa6\xb2',
'LimbuDigitNine': '\xe1\xa5\x8f',
'HangulLetterYeorinhieuh': '\xe3\x86\x86',
'CjkUnifiedIdeograph857b': '\xe8\x95\xbb',
'CjkUnifiedIdeograph857c': '\xe8\x95\xbc',
'HangulSyllableJjem': '\xec\xa9\xb8',
'CjkUnifiedIdeograph857a': '\xe8\x95\xba',
'CjkUnifiedIdeograph6dec': '\xe6\xb7\xac',
'CjkUnifiedIdeograph6deb': '\xe6\xb7\xab',
'CjkUnifiedIdeograph6dea': '\xe6\xb7\xaa',
'HexagramForDecrease': '\xe4\xb7\xa8',
'CjkUnifiedIdeograph6dee': '\xe6\xb7\xae',
'CjkUnifiedIdeograph6ded': '\xe6\xb7\xad',
'ArabicLigatureMeemWithJeemWithYehFinalForm': '\xef\xb7\x80',
'ThaiDigitZero': '\xe0\xb9\x90',
'UnicodeCharacter0xb0e': '\xe0\xac\x8e',
'UnicodeCharacter0xb0d': '\xe0\xac\x8d',
'CjkUnifiedIdeograph4ab9': '\xe4\xaa\xb9',
'CjkUnifiedIdeograph4ab8': '\xe4\xaa\xb8',
'CjkUnifiedIdeograph4ab5': '\xe4\xaa\xb5',
'CjkUnifiedIdeograph4ab4': '\xe4\xaa\xb4',
'CjkUnifiedIdeograph4ab7': '\xe4\xaa\xb7',
'CjkUnifiedIdeograph4ab6': '\xe4\xaa\xb6',
'CjkUnifiedIdeograph4ab1': '\xe4\xaa\xb1',
'CjkUnifiedIdeograph4ab0': '\xe4\xaa\xb0',
'CjkUnifiedIdeograph4ab3': '\xe4\xaa\xb3',
'CjkUnifiedIdeograph4ab2': '\xe4\xaa\xb2',
'HangulSyllableJjec': '\xec\xa9\xbf',
'HangulSyllableSsing': '\xec\x94\xbd',
'UnicodeCharacter0xd813': '\xed\xa0\x93',
'UnicodeCharacter0xf5cc': '\xef\x97\x8c',
'HangulSyllableKung': '\xec\xbf\xb5',
'ArabicLigatureQafWithHahInitialForm': '\xef\xb3\x82',
'HangulSyllableKunj': '\xec\xbf\xa5',
'VaiSyllableBhoo': '\xea\x95\xb7',
'ModifierLetterLowRightArrowhead': '\xcb\xb2',
'UnicodeCharacter0xeeb7': '\xee\xba\xb7',
'Omicron': '\xce\x9f',
'CjkUnifiedIdeograph74be': '\xe7\x92\xbe',
'VaiSymbolDoo': '\xea\x98\x9e',
'ArabicLigatureMeemWithHahWithYehFinalForm': '\xef\xb6\x8b',
'PiPi': '\xe2\x84\xbf',
'UnicodeCharacter0xf6cf': '\xef\x9b\x8f',
'HangulSyllableHelg': '\xed\x97\xad',
'EthiopicSyllableCcee': '\xe2\xb6\xac',
'MyanmarVowelSignVocalicLl': '\xe1\x81\x99',
'UnicodeCharacter0xe4e6': '\xee\x93\xa6',
'CjkUnifiedIdeograph4ec5': '\xe4\xbb\x85',
'LeftpointingDoubleAngleQuotationMark': '\xc2\xab',
'OriyaLetterHa': '\xe0\xac\xb9',
'GreekSmallLetterIotaWithDialytikaAndPerispomeni': '\xe1\xbf\x97',
'CjkUnifiedIdeograph4ec2': '\xe4\xbb\x82',
'CjkUnifiedIdeograph84e9': '\xe8\x93\xa9',
'CjkUnifiedIdeograph84e8': '\xe8\x93\xa8',
'CjkUnifiedIdeograph84e1': '\xe8\x93\xa1',
'CjkUnifiedIdeograph84e0': '\xe8\x93\xa0',
'CjkUnifiedIdeograph84e3': '\xe8\x93\xa3',
'CjkUnifiedIdeograph84e2': '\xe8\x93\xa2',
'CyrillicCapitalLetterStraightUWithStroke': '\xd2\xb0',
'KangxiRadicalSmall': '\xe2\xbc\xa9',
'CjkUnifiedIdeograph84e7': '\xe8\x93\xa7',
'CjkUnifiedIdeograph84e6': '\xe8\x93\xa6',
'LatinSmallLetterOi': '\xc6\xa3',
'HangulSyllableSsyeok': '\xec\x8e\xb8',
'LatinSmallLetterOo': '\xea\x9d\x8f',
'HangulChoseongIeung': '\xe1\x84\x8b',
'FullwidthFullStop': '\xef\xbc\x8e',
'TibetanLetterNga': '\xe0\xbd\x84',
'UnicodeCharacter0xe977': '\xee\xa5\xb7',
'UnicodeCharacter0xe976': '\xee\xa5\xb6',
'UnicodeCharacter0xe975': '\xee\xa5\xb5',
'UnicodeCharacter0xe974': '\xee\xa5\xb4',
'UnicodeCharacter0xe973': '\xee\xa5\xb3',
'TaiLeLetterNga': '\xe1\xa5\x92',
'UnicodeCharacter0xe971': '\xee\xa5\xb1',
'UnicodeCharacter0xe970': '\xee\xa5\xb0',
'CjkUnifiedIdeograph6c7f': '\xe6\xb1\xbf',
'CjkUnifiedIdeograph6c7d': '\xe6\xb1\xbd',
'CjkUnifiedIdeograph6c7e': '\xe6\xb1\xbe',
'CjkUnifiedIdeograph6c7b': '\xe6\xb1\xbb',
'LatinSmallLetterOu': '\xc8\xa3',
'UnicodeCharacter0xe979': '\xee\xa5\xb9',
'CjkUnifiedIdeograph6c7a': '\xe6\xb1\xba',
'HangulSyllableTyals': '\xed\x84\x84',
'CjkUnifiedIdeograph4ec0': '\xe4\xbb\x80',
'CyrillicCapitalLetterFita': '\xd1\xb2',
'CjkUnifiedIdeograph84ea': '\xe8\x93\xaa',
'CjkUnifiedIdeograph84ec': '\xe8\x93\xac',
'CjkUnifiedIdeograph84eb': '\xe8\x93\xab',
'CjkUnifiedIdeograph84ee': '\xe8\x93\xae',
'CjkUnifiedIdeograph84ed': '\xe8\x93\xad',
'CjkUnifiedIdeograph84ef': '\xe8\x93\xaf',
'CjkUnifiedIdeograph61eb': '\xe6\x87\xab',
'HangulSyllableGgyaegs': '\xea\xba\xa3',
'LatinCapitalLetterSWithDotAbove': '\xe1\xb9\xa0',
'CjkUnifiedIdeograph61ec': '\xe6\x87\xac',
'HalfwidthKatakanaLetterSo': '\xef\xbd\xbf',
'LatinCapitalLetterKWithCedilla': '\xc4\xb6',
'HangulSyllableNwibs': '\xeb\x89\xaa',
'CjkUnifiedIdeograph4ec1': '\xe4\xbb\x81',
'KhmerLetterVo': '\xe1\x9e\x9c',
'YiSyllableIe': '\xea\x80\x86',
'UnicodeCharacter0xe97b': '\xee\xa5\xbb',
'CjkUnifiedIdeograph6c78': '\xe6\xb1\xb8',
'CjkUnifiedIdeograph6c79': '\xe6\xb1\xb9',
'CjkUnifiedIdeograph6c76': '\xe6\xb1\xb6',
'CjkUnifiedIdeograph6c77': '\xe6\xb1\xb7',
'CjkUnifiedIdeograph6c74': '\xe6\xb1\xb4',
'TibetanVowelSignU': '\xe0\xbd\xb4',
'CjkUnifiedIdeograph6c72': '\xe6\xb1\xb2',
'CjkUnifiedIdeograph6c73': '\xe6\xb1\xb3',
'CjkUnifiedIdeograph6c70': '\xe6\xb1\xb0',
'CjkUnifiedIdeograph6c71': '\xe6\xb1\xb1',
'TibetanVowelSignO': '\xe0\xbd\xbc',
'LatinCapitalLetterVWithDotBelow': '\xe1\xb9\xbe',
'YiSyllableIt': '\xea\x80\x80',
'TibetanVowelSignI': '\xe0\xbd\xb2',
'HangulSyllableHelm': '\xed\x97\xae',
'YiSyllableIx': '\xea\x80\x81',
'TibetanVowelSignE': '\xe0\xbd\xba',
'HangulSyllableTyalm': '\xed\x84\x82',
'HangulSyllableJye': '\xec\xa1\x94',
'HangulSyllableJya': '\xec\x9f\x88',
'MyanmarLetterAu': '\xe1\x80\xaa',
'UnicodeCharacter0x2bfb': '\xe2\xaf\xbb',
'HangulSyllableJyo': '\xec\xa3\xa0',
'HangulSyllableJyi': '\xec\xa6\xa4',
'CjkUnifiedIdeograph4c0a': '\xe4\xb0\x8a',
'CjkUnifiedIdeograph4c0c': '\xe4\xb0\x8c',
'CjkUnifiedIdeograph4c0b': '\xe4\xb0\x8b',
'CjkUnifiedIdeograph4c0e': '\xe4\xb0\x8e',
'CjkUnifiedIdeograph4c0d': '\xe4\xb0\x8d',
'AplFunctionalSymbolZilde': '\xe2\x8d\xac',
'CjkUnifiedIdeograph4c0f': '\xe4\xb0\x8f',
'UnicodeCharacter0x1cca': '\xe1\xb3\x8a',
'HangulSyllableGgaenh': '\xea\xb9\xae',
'UnicodeCharacter0x1ccc': '\xe1\xb3\x8c',
'UnicodeCharacter0x1ccb': '\xe1\xb3\x8b',
'UnicodeCharacter0x1cce': '\xe1\xb3\x8e',
'UnicodeCharacter0x1ccd': '\xe1\xb3\x8d',
'UnicodeCharacter0x1ccf': '\xe1\xb3\x8f',
'RingOperator': '\xe2\x88\x98',
'UnicodeCharacter0xf245': '\xef\x89\x85',
'KhmerSignAtthacan': '\xe1\x9f\x9d',
'CopticCapitalLetterUa': '\xe2\xb2\xa8',
'LeftOneQuarterBlock': '\xe2\x96\x8e',
'CjkUnifiedIdeograph5ea8': '\xe5\xba\xa8',
'CjkUnifiedIdeograph8e03': '\xe8\xb8\x83',
'HangulSyllableTyalg': '\xed\x84\x81',
'UnicodeCharacter0x1cc9': '\xe1\xb3\x89',
'UnicodeCharacter0x1cc8': '\xe1\xb3\x88',
'CjkUnifiedIdeograph4c03': '\xe4\xb0\x83',
'CjkUnifiedIdeograph4c02': '\xe4\xb0\x82',
'CjkUnifiedIdeograph4c05': '\xe4\xb0\x85',
'CjkUnifiedIdeograph4c04': '\xe4\xb0\x84',
'CjkUnifiedIdeograph4c07': '\xe4\xb0\x87',
'CjkUnifiedIdeograph4c06': '\xe4\xb0\x86',
'CjkUnifiedIdeograph4c09': '\xe4\xb0\x89',
'CjkUnifiedIdeograph4c08': '\xe4\xb0\x88',
'UnicodeCharacter0x1cc3': '\xe1\xb3\x83',
'UnicodeCharacter0x1cc2': '\xe1\xb3\x82',
'UnicodeCharacter0x1cc5': '\xe1\xb3\x85',
'UnicodeCharacter0x1cc4': '\xe1\xb3\x84',
'UnicodeCharacter0x1cc7': '\xe1\xb3\x87',
'UnicodeCharacter0x1cc6': '\xe1\xb3\x86',
'LepchaLetterLa': '\xe1\xb0\x9c',
'HangulSyllableMaeh': '\xeb\xa7\xbf',
'HangulSyllableHelh': '\xed\x97\xb3',
'AplFunctionalSymbolTildeDiaeresis': '\xe2\x8d\xa8',
'CjkUnifiedIdeograph8e02': '\xe8\xb8\x82',
'UnicodeCharacter0xe0fd': '\xee\x83\xbd',
'HangulSyllableMaej': '\xeb\xa7\xba',
'UnicodeCharacter0xed0b': '\xee\xb4\x8b',
'CjkUnifiedIdeograph3c8d': '\xe3\xb2\x8d',
'CjkUnifiedIdeograph5ea0': '\xe5\xba\xa0',
'CjkUnifiedIdeograph3c8e': '\xe3\xb2\x8e',
'HangulSyllablePyeoss': '\xed\x8f\x88',
'SyriacLetterBeth': '\xdc\x92',
'LatinCapitalLetterBWithStroke': '\xc9\x83',
'LeftwardsDoubleArrowWithVerticalStroke': '\xe2\xa4\x82',
'HangulSyllableMaem': '\xeb\xa7\xb4',
'HangulSyllableMaen': '\xeb\xa7\xa8',
'GreekCapitalLetterOmicronWithDasiaAndOxia': '\xe1\xbd\x8d',
'HangulSyllableJyinh': '\xec\xa6\xaa',
'DoubleCircledDigitNine': '\xe2\x93\xbd',
'CjkUnifiedIdeograph3c8a': '\xe3\xb2\x8a',
'UnicodeCharacter0xdbaf': '\xed\xae\xaf',
'LeftwardsPairedArrows': '\xe2\x87\x87',
'CjkUnifiedIdeograph3c8b': '\xe3\xb2\x8b',
'LatinCapitalLetterThorn': '\xc3\x9e',
'tau': '\xcf\x84',
'CjkUnifiedIdeograph47ef': '\xe4\x9f\xaf',
'IdeographicRisingToneMark': '\xe3\x80\xab',
'CjkUnifiedIdeograph47ed': '\xe4\x9f\xad',
'CjkUnifiedIdeograph47ee': '\xe4\x9f\xae',
'CjkUnifiedIdeograph47eb': '\xe4\x9f\xab',
'CjkUnifiedIdeograph47ec': '\xe4\x9f\xac',
'UnicodeCharacter0x1bcf': '\xe1\xaf\x8f',
'CjkUnifiedIdeograph47ea': '\xe4\x9f\xaa',
'ModifierLetterCapitalBarredB': '\xe1\xb4\xaf',
'ArabicLigatureAllahIsolatedForm': '\xef\xb7\xb2',
'CjkUnifiedIdeograph8e00': '\xe8\xb8\x80',
'HangulSyllableMyaebs': '\xeb\xa8\xae',
'UnicodeCharacter0xe3ce': '\xee\x8f\x8e',
'LatinCapitalLetterZWithDotAbove': '\xc5\xbb',
'HangulSyllableSsyags': '\xec\x8c\xb3',
'YiSyllableChuop': '\xea\x8d\xa8',
'CjkUnifiedIdeograph92b3': '\xe9\x8a\xb3',
'HangulSyllableTyeong': '\xed\x85\xbd',
'HangulSyllableTyeonh': '\xed\x85\xae',
'HangulSyllableTyeonj': '\xed\x85\xad',
'PresentationFormForVerticalRightParenthesis': '\xef\xb8\xb6',
'RightwardsHarpoonWithBarbUpFromBar': '\xe2\xa5\x9b',
'ArabicLetterPehehIsolatedForm': '\xef\xad\xae',
'CherokeeLetterDla': '\xe1\x8f\x9c',
'ParenthesizedKoreanCharacterOjeon': '\xe3\x88\x9d',
'CjkCompatibilityIdeographf91d': '\xef\xa4\x9d',
'HangulSyllableAenh': '\xec\x95\xa6',
'UnicodeCharacter0x1bc8': '\xe1\xaf\x88',
'UnicodeCharacter0x1bc9': '\xe1\xaf\x89',
'HangulSyllableKyanj': '\xec\xba\xb1',
'CjkUnifiedIdeograph9c3f': '\xe9\xb0\xbf',
'CjkUnifiedIdeograph47e8': '\xe4\x9f\xa8',
'CjkUnifiedIdeograph47e9': '\xe4\x9f\xa9',
'CjkUnifiedIdeograph47e6': '\xe4\x9f\xa6',
'CjkUnifiedIdeograph47e7': '\xe4\x9f\xa7',
'CjkUnifiedIdeograph47e4': '\xe4\x9f\xa4',
'CjkUnifiedIdeograph47e5': '\xe4\x9f\xa5',
'CjkUnifiedIdeograph47e2': '\xe4\x9f\xa2',
'CjkUnifiedIdeograph47e3': '\xe4\x9f\xa3',
'CjkUnifiedIdeograph47e0': '\xe4\x9f\xa0',
'CjkUnifiedIdeograph47e1': '\xe4\x9f\xa1',
'MongolianLetterSibeJa': '\xe1\xa1\xaa',
'ArabicLigatureQafWithHahIsolatedForm': '\xef\xb0\xb3',
'UnicodeCharacter0xe8df': '\xee\xa3\x9f',
'CjkUnifiedIdeograph4799': '\xe4\x9e\x99',
'HangulSyllableGyat': '\xea\xb1\x91',
'UnicodeCharacter0xe7c5': '\xee\x9f\x85',
'HangulSyllableGyas': '\xea\xb1\x8b',
'HangulSyllableGyap': '\xea\xb1\x92',
'SinhalaLetterAlpapraanaKayanna': '\xe0\xb6\x9a',
'HangulSyllableGyal': '\xea\xb1\x80',
'HangulSyllableGyam': '\xea\xb1\x88',
'HangulSyllableGyaj': '\xea\xb1\x8e',
'HangulSyllableGyak': '\xea\xb1\x90',
'HangulSyllableGyah': '\xea\xb1\x93',
'HangulSyllableGyag': '\xea\xb0\xb9',
'HangulSyllableGyad': '\xea\xb0\xbf',
'SyriacRbasaBelow': '\xdc\xb7',
'HangulSyllableGyab': '\xea\xb1\x89',
'HangulSyllableGyac': '\xea\xb1\x8f',
'UnicodeCharacter0xda80': '\xed\xaa\x80',
'CjkUnifiedIdeograph61e1': '\xe6\x87\xa1',
'RunicLetterShorttwigarA': '\xe1\x9b\x86',
'HangulSyllableWeh': '\xec\x9c\x83',
'HangulSyllableKyang': '\xec\xbb\x81',
'RejangLetterNga': '\xea\xa4\xb2',
'YiSyllableChuot': '\xea\x8d\xa5',
'CjkUnifiedIdeograph61e5': '\xe6\x87\xa5',
'LatinSmallLetterZWithCurl': '\xca\x91',
'HangulSyllableJjyik': '\xec\xb0\x88',
'UnicodeCharacter0xe119': '\xee\x84\x99',
'UnicodeCharacter0xe118': '\xee\x84\x98',
'CanadianSyllabicsThoo': '\xe1\x95\xa5',
'UnicodeCharacter0xe111': '\xee\x84\x91',
'UnicodeCharacter0xe110': '\xee\x84\x90',
'UnicodeCharacter0xe113': '\xee\x84\x93',
'UnicodeCharacter0xe112': '\xee\x84\x92',
'UnicodeCharacter0xe115': '\xee\x84\x95',
'UnicodeCharacter0xe114': '\xee\x84\x94',
'UnicodeCharacter0xe117': '\xee\x84\x97',
'HangulSyllableJjyid': '\xec\xaf\xb7',
'UnicodeCharacter0xe60e': '\xee\x98\x8e',
'MongolianLetterO': '\xe1\xa0\xa3',
'MongolianLetterI': '\xe1\xa0\xa2',
'UnicodeCharacter0xe60d': '\xee\x98\x8d',
'MongolianLetterE': '\xe1\xa0\xa1',
'MongolianLetterA': '\xe1\xa0\xa0',
'UnicodeCharacter0xe60f': '\xee\x98\x8f',
'MongolianLetterU': '\xe1\xa0\xa4',
'HangulSyllableDdyeoss': '\xeb\x97\xac',
'VaiSyllableZoo': '\xea\x96\x89',
'UnicodeCharacter0xe11a': '\xee\x84\x9a',
'PlusSignWithBlackTriangle': '\xe2\xa8\xa8',
'CjkUnifiedIdeograph7b7a': '\xe7\xad\xba',
'CjkUnifiedIdeograph8d51': '\xe8\xb5\x91',
'UnicodeCharacter0xe11e': '\xee\x84\x9e',
'LatinSmallLetterHalfH': '\xe2\xb1\xb6',
'UnicodeCharacter0xe11f': '\xee\x84\x9f',
'Shamrock': '\xe2\x98\x98',
'LeftwardsDoubleArrowtail': '\xe2\xa4\x9b',
'ArabicLigatureYehWithHamzaAboveWithYuIsolatedForm': '\xef\xaf\xb4',
'KhmerSymbolBeiKoet': '\xe1\xa7\xa3',
'ThaiCharacterLoChula': '\xe0\xb8\xac',
'CjkUnifiedIdeograph602b': '\xe6\x80\xab',
'CjkUnifiedIdeograph602c': '\xe6\x80\xac',
'CjkUnifiedIdeograph903f': '\xe9\x80\xbf',
'RunicLetterCalc': '\xe1\x9b\xa3',
'CjkUnifiedIdeograph602f': '\xe6\x80\xaf',
'CjkUnifiedIdeograph903a': '\xe9\x80\xba',
'NumberSeventeenFullStop': '\xe2\x92\x98',
'CjkUnifiedIdeograph602e': '\xe6\x80\xae',
'CjkUnifiedIdeograph8b27': '\xe8\xac\xa7',
'ArabicLetterTeheh': '\xd9\xbf',
'YiSyllableChuox': '\xea\x8d\xa6',
'MyanmarLetterPha': '\xe1\x80\x96',
'GurmukhiLetterGha': '\xe0\xa8\x98',
'LeftwardsWaveArrow': '\xe2\x86\x9c',
'CjkUnifiedIdeograph3fcd': '\xe3\xbf\x8d',
'ArabicLigatureJeemWithAlefMaksuraIsolatedForm': '\xef\xb4\x81',
'CjkUnifiedIdeograph3fce': '\xe3\xbf\x8e',
'BoxDrawingsDoubleVerticalAndLeft': '\xe2\x95\xa3',
'HangulSyllableKyigs': '\xed\x82\x8b',
'HangulSyllableCik': '\xec\xb9\xb0',
'CanadianSyllabicsIn': '\xe1\x90\xac',
'CanadianSyllabicsIi': '\xe1\x90\x84',
'HangulSyllableKyigg': '\xed\x82\x8a',
'HangulSyllableTinh': '\xed\x8b\xb6',
'ArabicSignSafha': '\xd8\x83',
'CjkUnifiedIdeograph7b7d': '\xe7\xad\xbd',
'HangulSyllableOed': '\xec\x99\xbf',
'MyanmarConsonantSignMedialWa': '\xe1\x80\xbd',
'UnicodeCharacter0xf89f': '\xef\xa2\x9f',
'CjkUnifiedIdeograph6028': '\xe6\x80\xa8',
'CjkUnifiedIdeograph6029': '\xe6\x80\xa9',
'CjkUnifiedIdeograph9038': '\xe9\x80\xb8',
'CjkUnifiedIdeograph9039': '\xe9\x80\xb9',
'BoxDrawingsRightUpHeavyAndLeftDownLight': '\xe2\x95\x84',
'DevanagariDigitTwo': '\xe0\xa5\xa8',
'CjkUnifiedIdeograph6023': '\xe6\x80\xa3',
'CjkUnifiedIdeograph6020': '\xe6\x80\xa0',
'CjkUnifiedIdeograph6021': '\xe6\x80\xa1',
'CjkUnifiedIdeograph6026': '\xe6\x80\xa6',
'CjkUnifiedIdeograph6027': '\xe6\x80\xa7',
'CjkUnifiedIdeograph6024': '\xe6\x80\xa4',
'CjkUnifiedIdeograph6025': '\xe6\x80\xa5',
'GeorgianSmallLetterXan': '\xe2\xb4\x9e',
'ModifierLetterUpTack': '\xcb\x94',
'HangulSyllableCyaelm': '\xec\xb2\x86',
'HangulSyllableOss': '\xec\x98\xb8',
'HangulSyllableCyaelt': '\xec\xb2\x89',
'HangulSyllableCyaelp': '\xec\xb2\x8a',
'HangulSyllableCyaels': '\xec\xb2\x88',
'GreekCapitalLetterNu': '\xce\x9d',
'CjkUnifiedIdeograph7b7f': '\xe7\xad\xbf',
'CjkUnifiedIdeograph62f4': '\xe6\x8b\xb4',
'UnicodeCharacter0xe0c8': '\xee\x83\x88',
'UnicodeCharacter0xe0c9': '\xee\x83\x89',
'HangulSyllableBigg': '\xeb\xb9\x86',
'GreekSmallLetterGamma': '\xce\xb3',
'UnicodeCharacter0xe0c2': '\xee\x83\x82',
'UnicodeCharacter0xe0c3': '\xee\x83\x83',
'UnicodeCharacter0xe0c0': '\xee\x83\x80',
'HangulSyllableCin': '\xec\xb9\x9c',
'UnicodeCharacter0xe0c6': '\xee\x83\x86',
'UnicodeCharacter0xe0c7': '\xee\x83\x87',
'UnicodeCharacter0xe0c4': '\xee\x83\x84',
'UnicodeCharacter0xe0c5': '\xee\x83\x85',
'HangulSyllableBigs': '\xeb\xb9\x87',
'HangulSyllableJwebs': '\xec\xa5\x86',
'LongLeftTack': '\xe2\x9f\x9e',
'HangulSyllableBaebs': '\xeb\xb1\x82',
'GreekUpsilonWithAcuteAndHookSymbol': '\xcf\x93',
'HangulSyllableJweong': '\xec\xa4\xad',
'CjkUnifiedIdeograph737a': '\xe7\x8d\xba',
'CjkUnifiedIdeograph3805': '\xe3\xa0\x85',
'CjkUnifiedIdeograph737c': '\xe7\x8d\xbc',
'CjkUnifiedIdeograph737b': '\xe7\x8d\xbb',
'CjkUnifiedIdeograph737e': '\xe7\x8d\xbe',
'CjkUnifiedIdeograph737d': '\xe7\x8d\xbd',
'CjkUnifiedIdeograph737f': '\xe7\x8d\xbf',
'UnicodeCharacter0xed07': '\xee\xb4\x87',
'CjkUnifiedIdeograph394a': '\xe3\xa5\x8a',
'CjkUnifiedIdeograph66b8': '\xe6\x9a\xb8',
'CjkUnifiedIdeograph66b9': '\xe6\x9a\xb9',
'UnicodeCharacter0xed06': '\xee\xb4\x86',
'CjkUnifiedIdeograph66b4': '\xe6\x9a\xb4',
'CjkUnifiedIdeograph66b5': '\xe6\x9a\xb5',
'CjkUnifiedIdeograph66b6': '\xe6\x9a\xb6',
'CjkUnifiedIdeograph66b7': '\xe6\x9a\xb7',
'CjkUnifiedIdeograph66b0': '\xe6\x9a\xb0',
'CjkUnifiedIdeograph66b1': '\xe6\x9a\xb1',
'CjkUnifiedIdeograph66b2': '\xe6\x9a\xb2',
'CjkUnifiedIdeograph66b3': '\xe6\x9a\xb3',
'UnicodeCharacter0xeb3b': '\xee\xac\xbb',
'CjkUnifiedIdeograph454c': '\xe4\x95\x8c',
'CjkUnifiedIdeograph454b': '\xe4\x95\x8b',
'CjkUnifiedIdeograph454a': '\xe4\x95\x8a',
'GeorgianLetterLas': '\xe1\x83\x9a',
'AlmostEqualOrEqualTo': '\xe2\x89\x8a',
'CjkUnifiedIdeograph454f': '\xe4\x95\x8f',
'CjkUnifiedIdeograph454e': '\xe4\x95\x8e',
'CjkUnifiedIdeograph454d': '\xe4\x95\x8d',
'CjkUnifiedIdeograph7379': '\xe7\x8d\xb9',
'CjkUnifiedIdeograph40e9': '\xe4\x83\xa9',
'ModifierLetterSmallTurnedOpenE': '\xe1\xb5\x8c',
'LatinSmallLetterNum': '\xea\x9d\xb4',
'CjkUnifiedIdeograph40e8': '\xe4\x83\xa8',
'CjkUnifiedIdeograph7371': '\xe7\x8d\xb1',
'CjkUnifiedIdeograph7370': '\xe7\x8d\xb0',
'CjkUnifiedIdeograph7373': '\xe7\x8d\xb3',
'CjkUnifiedIdeograph7372': '\xe7\x8d\xb2',
'CjkUnifiedIdeograph7375': '\xe7\x8d\xb5',
'CjkUnifiedIdeograph7374': '\xe7\x8d\xb4',
'CjkUnifiedIdeograph7377': '\xe7\x8d\xb7',
'CjkUnifiedIdeograph62f9': '\xe6\x8b\xb9',
'UnicodeCharacter0xe8d9': '\xee\xa3\x99',
'MongolianLetterQa': '\xe1\xa0\xac',
'UnicodeCharacter0xe0cb': '\xee\x83\x8b',
'UnicodeCharacter0xe0cc': '\xee\x83\x8c',
'UnicodeCharacter0xe0ca': '\xee\x83\x8a',
'UnicodeCharacter0xe0cf': '\xee\x83\x8f',
'UnicodeCharacter0xdba7': '\xed\xae\xa7',
'UnicodeCharacter0xe0cd': '\xee\x83\x8d',
'UnicodeCharacter0xe0ce': '\xee\x83\x8e',
'CjkUnifiedIdeograph4549': '\xe4\x95\x89',
'CjkUnifiedIdeograph4548': '\xe4\x95\x88',
'ThaiCharacterFoFa': '\xe0\xb8\x9d',
'CjkUnifiedIdeograph4543': '\xe4\x95\x83',
'CjkUnifiedIdeograph4542': '\xe4\x95\x82',
'CjkUnifiedIdeograph4541': '\xe4\x95\x81',
'CjkUnifiedIdeograph4540': '\xe4\x95\x80',
'CjkUnifiedIdeograph4547': '\xe4\x95\x87',
'CjkUnifiedIdeograph4546': '\xe4\x95\x86',
'CjkUnifiedIdeograph4545': '\xe4\x95\x85',
'CjkUnifiedIdeograph4544': '\xe4\x95\x84',
'CjkUnifiedIdeograph3802': '\xe3\xa0\x82',
'HangulJongseongThieuth': '\xe1\x87\x80',
'HangulSyllableNonh': '\xeb\x85\xbe',
'Flatness': '\xe2\x8f\xa5',
'CjkUnifiedIdeograph66be': '\xe6\x9a\xbe',
'CjkUnifiedIdeograph66bf': '\xe6\x9a\xbf',
'UnicodeCharacter0xed08': '\xee\xb4\x88',
'CjkUnifiedIdeograph66ba': '\xe6\x9a\xba',
'CjkUnifiedIdeograph66bb': '\xe6\x9a\xbb',
'CjkUnifiedIdeograph66bc': '\xe6\x9a\xbc',
'CjkUnifiedIdeograph3793': '\xe3\x9e\x93',
'CjkUnifiedIdeograph3792': '\xe3\x9e\x92',
'CjkUnifiedIdeograph3791': '\xe3\x9e\x91',
'CjkUnifiedIdeograph3790': '\xe3\x9e\x90',
'CjkUnifiedIdeograph3797': '\xe3\x9e\x97',
'CjkUnifiedIdeograph3796': '\xe3\x9e\x96',
'CjkUnifiedIdeograph3795': '\xe3\x9e\x95',
'CjkUnifiedIdeograph3794': '\xe3\x9e\x94',
'CjkUnifiedIdeograph58ff': '\xe5\xa3\xbf',
'CjkUnifiedIdeograph3799': '\xe3\x9e\x99',
'CjkUnifiedIdeograph3798': '\xe3\x9e\x98',
'CjkUnifiedIdeograph58fc': '\xe5\xa3\xbc',
'CjkUnifiedIdeograph58fb': '\xe5\xa3\xbb',
'CjkUnifiedIdeograph58fa': '\xe5\xa3\xba',
'UpwardsTwoHeadedArrow': '\xe2\x86\x9f',
'CjkUnifiedIdeograph7083': '\xe7\x82\x83',
'CjkUnifiedIdeograph7082': '\xe7\x82\x82',
'CjkUnifiedIdeograph7081': '\xe7\x82\x81',
'LatinCapitalLetterDWithCaron': '\xc4\x8e',
'CjkUnifiedIdeograph7087': '\xe7\x82\x87',
'IdeographicDescriptionCharacterAboveToBelow': '\xe2\xbf\xb1',
'CjkUnifiedIdeograph7085': '\xe7\x82\x85',
'TeluguLetterNga': '\xe0\xb0\x99',
'UnicodeCharacter0xe3c5': '\xee\x8f\x85',
'ModifierLetterSmallDotlessJWithStroke': '\xe1\xb6\xa1',
'CjkUnifiedIdeograph7089': '\xe7\x82\x89',
'CjkUnifiedIdeograph7088': '\xe7\x82\x88',
'UnicodeCharacter0xe3c1': '\xee\x8f\x81',
'UnicodeCharacter0xe3c0': '\xee\x8f\x80',
'UnicodeCharacter0xe3c3': '\xee\x8f\x83',
'UnicodeCharacter0xe3c2': '\xee\x8f\x82',
'HangulSyllableGgwael': '\xea\xbd\xac',
'UnicodeCharacter0xf5aa': '\xef\x96\xaa',
'SquarePr': '\xe3\x8f\x9a',
'UnicodeCharacter0xf5ab': '\xef\x96\xab',
'CjkUnifiedIdeograph3800': '\xe3\xa0\x80',
'HangulSyllableGgwaeb': '\xea\xbd\xb5',
'LatinSmallLetterEWithStroke': '\xc9\x87',
'CjkUnifiedIdeograph9ad9': '\xe9\xab\x99',
'UnicodeCharacter0xf5ad': '\xef\x96\xad',
'SinhalaLetterMahaapraanaBayanna': '\xe0\xb6\xb7',
'ArabicLetterKhahMedialForm': '\xef\xba\xa8',
'UnicodeCharacter0xf5ae': '\xef\x96\xae',
'CjkUnifiedIdeograph708c': '\xe7\x82\x8c',
'CombiningCyrillicLetterO': '\xe2\xb7\xaa',
'CjkUnifiedIdeograph708a': '\xe7\x82\x8a',
'CjkUnifiedIdeograph708f': '\xe7\x82\x8f',
'CjkUnifiedIdeograph708e': '\xe7\x82\x8e',
'CjkRadicalMoon': '\xe2\xba\x9d',
'BalineseLetterTaMurdaMahaprana': '\xe1\xac\x9e',
'UnicodeCharacter0xab56': '\xea\xad\x96',
'HangulSyllableSyilp': '\xec\x8b\x8e',
'UnicodeCharacter0xe3ca': '\xee\x8f\x8a',
'UnicodeCharacter0xe3cc': '\xee\x8f\x8c',
'HangulSyllablePit': '\xed\x95\x95',
'CjkUnifiedIdeograph379c': '\xe3\x9e\x9c',
'CjkUnifiedIdeograph379b': '\xe3\x9e\x9b',
'CjkUnifiedIdeograph379a': '\xe3\x9e\x9a',
'HangulSyllablePih': '\xed\x95\x97',
'UnicodeCharacter0xde1f': '\xed\xb8\x9f',
'CjkUnifiedIdeograph379f': '\xe3\x9e\x9f',
'CjkUnifiedIdeograph379e': '\xe3\x9e\x9e',
'CjkUnifiedIdeograph379d': '\xe3\x9e\x9d',
'CjkUnifiedIdeograph58f7': '\xe5\xa3\xb7',
'CjkUnifiedIdeograph58f6': '\xe5\xa3\xb6',
'CjkUnifiedIdeograph58f5': '\xe5\xa3\xb5',
'CjkUnifiedIdeograph58f4': '\xe5\xa3\xb4',
'LatinCapitalLetterNWithCedilla': '\xc5\x85',
'CjkUnifiedIdeograph58f2': '\xe5\xa3\xb2',
'CjkUnifiedIdeograph58f1': '\xe5\xa3\xb1',
'CjkUnifiedIdeograph58f0': '\xe5\xa3\xb0',
'GujaratiVowelSignCandraE': '\xe0\xab\x85',
'ArabicLetterFehInitialForm': '\xef\xbb\x93',
'GujaratiVowelSignCandraO': '\xe0\xab\x89',
'HangulSyllableRyet': '\xeb\xa1\x99',
'UnicodeCharacter0xeb75': '\xee\xad\xb5',
'CopticSmallLetterCryptogrammicEie': '\xe2\xb2\xb7',
'SquareRira': '\xe3\x8d\x92',
'SquarePa': '\xe3\x8e\xa9',
'HangulSyllableBelt': '\xeb\xb2\xad',
'UpFishTail': '\xe2\xa5\xbe',
'ThaanaLetterGhainu': '\xde\xa3',
'HangulSyllableDenh': '\xeb\x8d\xb6',
'HangulSyllableTing': '\xed\x8c\x85',
'CjkUnifiedIdeograph3fc8': '\xe3\xbf\x88',
'ArabicLigatureSadWithHahWithHahFinalForm': '\xef\xb5\xa4',
'ReversedDoubleStrokeNotSign': '\xe2\xab\xad',
'CjkUnifiedIdeograph420e': '\xe4\x88\x8e',
'CjkUnifiedIdeograph83bc': '\xe8\x8e\xbc',
'BalineseLetterAsyuraSasak': '\xe1\xad\x8b',
'LaoLetterLoLing': '\xe0\xba\xa3',
'CombiningCyrillicLetterA': '\xe2\xb7\xb6',
'LimbuLetterGha': '\xe1\xa4\x84',
'HangulSyllableGgwaep': '\xea\xbd\xbe',
'HangulSyllableCaed': '\xec\xb1\x8b',
'UnicodeCharacter0xe605': '\xee\x98\x85',
'EditorialCoronis': '\xe2\xb8\x8e',
'HangulSyllableTyils': '\xed\x8b\xa0',
'HangulSyllableTyilt': '\xed\x8b\xa1',
'UnicodeCharacter0xf4d2': '\xef\x93\x92',
'UnicodeCharacter0xf4d3': '\xef\x93\x93',
'UnicodeCharacter0xf4d0': '\xef\x93\x90',
'UnicodeCharacter0xf4d1': '\xef\x93\x91',
'UnicodeCharacter0xf4d6': '\xef\x93\x96',
'UnicodeCharacter0xf4d7': '\xef\x93\x97',
'UnicodeCharacter0xf4d4': '\xef\x93\x94',
'CjkUnifiedIdeograph83bf': '\xe8\x8e\xbf',
'MinusTilde': '\xe2\x89\x82',
'UnicodeCharacter0xeb34': '\xee\xac\xb4',
'HangulSyllableTyilb': '\xed\x8b\x9f',
'HangulSyllableBelm': '\xeb\xb2\xaa',
'HangulSyllableGgwaet': '\xea\xbd\xbd',
'CjkUnifiedIdeograph6569': '\xe6\x95\xa9',
'CjkUnifiedIdeograph6568': '\xe6\x95\xa8',
'CjkUnifiedIdeograph6567': '\xe6\x95\xa7',
'WhiteDownPointingIndex': '\xe2\x98\x9f',
'CjkUnifiedIdeograph6565': '\xe6\x95\xa5',
'CjkUnifiedIdeograph6564': '\xe6\x95\xa4',
'CjkUnifiedIdeograph6563': '\xe6\x95\xa3',
'CjkUnifiedIdeograph6562': '\xe6\x95\xa2',
'CjkUnifiedIdeograph6561': '\xe6\x95\xa1',
'CjkUnifiedIdeograph6560': '\xe6\x95\xa0',
'HangulSyllableDwaen': '\xeb\x90\x80',
'UnicodeCharacter0xeb37': '\xee\xac\xb7',
'HangulSyllableBelh': '\xeb\xb2\xaf',
'KangxiRadicalTurtle': '\xe2\xbf\x94',
'UnicodeCharacter0xeb30': '\xee\xac\xb0',
'CyrillicSmallLetterSchwa': '\xd3\x99',
'UnicodeCharacter0xf1ff': '\xef\x87\xbf',
'UnicodeCharacter0xeb31': '\xee\xac\xb1',
'CjkRadicalCsimplifiedCart': '\xe2\xbb\x8b',
'UnicodeCharacter0xeb33': '\xee\xac\xb3',
'SundaneseLetterWa': '\xe1\xae\x9d',
'ArabicQuestionMark': '\xd8\x9f',
'CjkRadicalPerson': '\xe2\xba\x85',
'CanadianSyllabicsShwi': '\xe1\x94\x99',
'CjkUnifiedIdeograph656f': '\xe6\x95\xaf',
'CjkUnifiedIdeograph656e': '\xe6\x95\xae',
'CjkUnifiedIdeograph656d': '\xe6\x95\xad',
'CjkUnifiedIdeograph656c': '\xe6\x95\xac',
'CjkUnifiedIdeograph656b': '\xe6\x95\xab',
'CjkUnifiedIdeograph656a': '\xe6\x95\xaa',
'LatinSmallLetterLambdaWithStroke': '\xc6\x9b',
'CircledNumberTwentyTwo': '\xe3\x89\x92',
'UnicodeCharacter0xf4db': '\xef\x93\x9b',
'UnicodeCharacter0xf4dc': '\xef\x93\x9c',
'UnicodeCharacter0xf4da': '\xef\x93\x9a',
'UnicodeCharacter0xf4df': '\xef\x93\x9f',
'CjkUnifiedIdeograph8e81': '\xe8\xba\x81',
'UnicodeCharacter0xf4dd': '\xef\x93\x9d',
'UnicodeCharacter0xf4de': '\xef\x93\x9e',
'CjkCompatibilityIdeographf970': '\xef\xa5\xb0',
'CjkCompatibilityIdeographf971': '\xef\xa5\xb1',
'CjkCompatibilityIdeographf972': '\xef\xa5\xb2',
'CjkCompatibilityIdeographf973': '\xef\xa5\xb3',
'HangulSyllableEng': '\xec\x97\xa5',
'CjkCompatibilityIdeographf975': '\xef\xa5\xb5',
'CjkCompatibilityIdeographf976': '\xef\xa5\xb6',
'CjkCompatibilityIdeographf977': '\xef\xa5\xb7',
'CjkCompatibilityIdeographf978': '\xef\xa5\xb8',
'HangulSyllableEnj': '\xec\x97\x95',
'HangulSyllableRwagg': '\xeb\xa1\xba',
'CyrillicCapitalLetterPe': '\xd0\x9f',
'CjkUnifiedIdeograph8e86': '\xe8\xba\x86',
'HangulSyllableJjyip': '\xec\xb0\x8a',
'ArabicLigatureLamWithJeemInitialForm': '\xef\xb3\x89',
'CjkUnifiedIdeograph7250': '\xe7\x89\x90',
'HangulSyllableRwags': '\xeb\xa1\xbb',
'HangulSyllableTyeoss': '\xed\x85\xbc',
'CjkUnifiedIdeograph9ad1': '\xe9\xab\x91',
'HangulSyllableDibs': '\xeb\x94\xa6',
'MongolianLetterTodoHaa': '\xe1\xa1\x99',
'SquareRentogen': '\xe3\x8d\x96',
'UnicodeCharacter0xf5a0': '\xef\x96\xa0',
'BoxDrawingsHeavyUpAndLightDown': '\xe2\x95\xbf',
'UnicodeCharacter0xf5a1': '\xef\x96\xa1',
'CjkUnifiedIdeograph93d7': '\xe9\x8f\x97',
'HangulSyllablePyigg': '\xed\x94\xa2',
'UnicodeCharacter0xe1d6': '\xee\x87\x96',
'UnicodeCharacter0xdb64': '\xed\xad\xa4',
'CjkCompatibilityIdeographf97b': '\xef\xa5\xbb',
'UnicodeCharacter0xf5a3': '\xef\x96\xa3',
'CjkCompatibilityIdeographf97d': '\xef\xa5\xbd',
'CjkCompatibilityIdeographf97e': '\xef\xa5\xbe',
'CjkCompatibilityIdeographf97f': '\xef\xa5\xbf',
'UnicodeCharacter0xdb65': '\xed\xad\xa5',
'UnicodeCharacter0xf5a4': '\xef\x96\xa4',
'HangulSyllablePyigs': '\xed\x94\xa3',
'LatinSmallLetterOWithCircumflex': '\xc3\xb4',
'UnicodeCharacter0xf5a5': '\xef\x96\xa5',
'ClockwiseContourIntegral': '\xe2\x88\xb2',
'GreekLetterStigma': '\xcf\x9a',
'YiSyllableZox': '\xea\x8a\x9c',
'UnicodeCharacter0xf5a6': '\xef\x96\xa6',
'HangulSyllableMubs': '\xeb\xad\x86',
'LatinCapitalLetterOWithCircumflexAndGrave': '\xe1\xbb\x92',
'UnicodeCharacter0xf5a7': '\xef\x96\xa7',
'YiSyllableZop': '\xea\x8a\x9e',
'HangulSyllableDdeonh': '\xeb\x96\xa6',
'HangulSyllableJjyis': '\xec\xb0\x83',
'UnicodeCharacter0xdb62': '\xed\xad\xa2',
'CanadianSyllabicsCarrierLhee': '\xe1\x98\xb1',
'HangulSyllableDdeonj': '\xeb\x96\xa5',
'ChamLetterTa': '\xea\xa8\x93',
'exists': '\xe2\x88\x83',
'HangulSyllableGalp': '\xea\xb0\x8e',
'HangulSyllableSols': '\xec\x86\x98',
'ArabicLigatureAinWithMeemInitialForm': '\xef\xb2\xbb',
'YiSyllableNjiep': '\xea\x90\xb8',
'LeftDoubleParenthesis': '\xe2\xb8\xa8',
'HangulSyllableSolt': '\xec\x86\x99',
'HangulSyllableSolh': '\xec\x86\x9b',
'ArabicLetterPeheh': '\xda\xa6',
'HangulSyllableSolm': '\xec\x86\x96',
'HangulSyllableSolb': '\xec\x86\x97',
'HangulSyllableTyulh': '\xed\x8a\xab',
'HangulSyllableSolg': '\xec\x86\x95',
'HangulSyllableGalt': '\xea\xb0\x8d',
'CjkUnifiedIdeograph7e7c': '\xe7\xb9\xbc',
'CjkUnifiedIdeograph7e7b': '\xe7\xb9\xbb',
'CjkUnifiedIdeograph7e7a': '\xe7\xb9\xba',
'HangulSyllableGugs': '\xea\xb5\xaf',
'CombiningRightArrowheadAndDownArrowheadBelow': '\xe1\xb7\xbf',
'CjkUnifiedIdeograph6e79': '\xe6\xb9\xb9',
'CjkUnifiedIdeograph7e7e': '\xe7\xb9\xbe',
'CjkUnifiedIdeograph7e7d': '\xe7\xb9\xbd',
'CjkUnifiedIdeograph6e74': '\xe6\xb9\xb4',
'CjkUnifiedIdeograph6e75': '\xe6\xb9\xb5',
'CjkUnifiedIdeograph6e76': '\xe6\xb9\xb6',
'CjkUnifiedIdeograph6e77': '\xe6\xb9\xb7',
'LatinLetterStretchedC': '\xca\x97',
'CjkUnifiedIdeograph6e71': '\xe6\xb9\xb1',
'CjkUnifiedIdeograph6e72': '\xe6\xb9\xb2',
'Pluto': '\xe2\x99\x87',
'UnicodeCharacter0x9db': '\xe0\xa7\x9b',
'TamilLetterJa': '\xe0\xae\x9c',
'CjkRadicalNetThree': '\xe2\xba\xb3',
'CircledHangulMieum': '\xe3\x89\xa4',
'UnicodeCharacter0x9de': '\xe0\xa7\x9e',
'GujaratiLetterIi': '\xe0\xaa\x88',
'UnicodeCharacter0xd984': '\xed\xa6\x84',
'CjkUnifiedIdeograph83b1': '\xe8\x8e\xb1',
'UnicodeCharacter0xa7d4': '\xea\x9f\x94',
'HangulSyllableGgyeolm': '\xea\xbb\xbe',
'UnicodeCharacter0xeee8': '\xee\xbb\xa8',
'UnicodeCharacter0xa7d5': '\xea\x9f\x95',
'HangulSyllableGgyeolg': '\xea\xbb\xbd',
'UnicodeCharacter0xa4e4': '\xea\x93\xa4',
'UnicodeCharacter0xa7d6': '\xea\x9f\x96',
'UnicodeCharacter0xe44b': '\xee\x91\x8b',
'S': '\xce\xa3',
'UnicodeCharacter0xa7d7': '\xea\x9f\x97',
'SquareWithTopHalfBlack': '\xe2\xac\x92',
'UnicodeCharacter0xa7d0': '\xea\x9f\x90',
'ArabicLetterBeh': '\xd8\xa8',
'HangulSyllableGgyeolt': '\xea\xbc\x81',
'HangulSyllableGgyeols': '\xea\xbc\x80',
'UnicodeCharacter0xa7d1': '\xea\x9f\x91',
'UnicodeCharacter0x9d2': '\xe0\xa7\x92',
'UnicodeCharacter0x9d3': '\xe0\xa7\x93',
'UnicodeCharacter0x9d0': '\xe0\xa7\x90',
'UnicodeCharacter0x9d1': '\xe0\xa7\x91',
'UnicodeCharacter0x9d6': '\xe0\xa7\x96',
'UnicodeCharacter0xa7d2': '\xea\x9f\x92',
'UnicodeCharacter0x9d4': '\xe0\xa7\x94',
'UnicodeCharacter0x9d5': '\xe0\xa7\x95',
'HangulJungseongYoyae': '\xe1\x86\x85',
'UnicodeCharacter0x9d8': '\xe0\xa7\x98',
'UnicodeCharacter0x9d9': '\xe0\xa7\x99',
'TripleHorizontalBarWithDoubleVerticalStroke': '\xe2\xa9\xa8',
'CjkUnifiedIdeograph7e73': '\xe7\xb9\xb3',
'CjkUnifiedIdeograph7e72': '\xe7\xb9\xb2',
'CjkUnifiedIdeograph7e71': '\xe7\xb9\xb1',
'CjkUnifiedIdeograph7e70': '\xe7\xb9\xb0',
'CjkUnifiedIdeograph7e77': '\xe7\xb9\xb7',
'NeitherASupersetOfNorEqualTo': '\xe2\x8a\x89',
'CjkUnifiedIdeograph7e75': '\xe7\xb9\xb5',
'CjkUnifiedIdeograph7e74': '\xe7\xb9\xb4',
'CjkUnifiedIdeograph6e7d': '\xe6\xb9\xbd',
'CjkUnifiedIdeograph6e7e': '\xe6\xb9\xbe',
'CjkUnifiedIdeograph6e7f': '\xe6\xb9\xbf',
'CjkUnifiedIdeograph7e78': '\xe7\xb9\xb8',
'NationalDigitShapes': '\xe2\x81\xae',
'CircledIdeographMedicine': '\xe3\x8a\xa9',
'CjkUnifiedIdeograph6e7b': '\xe6\xb9\xbb',
'CjkUnifiedIdeograph6e7c': '\xe6\xb9\xbc',
'CjkUnifiedIdeograph5a2f': '\xe5\xa8\xaf',
'HangulSyllableCis': '\xec\xb9\xab',
'CjkUnifiedIdeograph5a2d': '\xe5\xa8\xad',
'CjkUnifiedIdeograph5a2e': '\xe5\xa8\xae',
'CjkUnifiedIdeograph5a2b': '\xe5\xa8\xab',
'CjkUnifiedIdeograph5a2c': '\xe5\xa8\xac',
'CjkUnifiedIdeograph4a49': '\xe4\xa9\x89',
'CjkUnifiedIdeograph4a48': '\xe4\xa9\x88',
'CjkUnifiedIdeograph4a47': '\xe4\xa9\x87',
'CjkUnifiedIdeograph4a46': '\xe4\xa9\x86',
'CjkUnifiedIdeograph4a45': '\xe4\xa9\x85',
'CjkUnifiedIdeograph4a44': '\xe4\xa9\x84',
'CjkUnifiedIdeograph4a43': '\xe4\xa9\x83',
'CjkUnifiedIdeograph4a42': '\xe4\xa9\x82',
'CjkUnifiedIdeograph4a41': '\xe4\xa9\x81',
'CjkUnifiedIdeograph4a40': '\xe4\xa9\x80',
'GreekCapitalLetterIotaWithVrachy': '\xe1\xbf\x98',
'CjkUnifiedIdeograph4d9a': '\xe4\xb6\x9a',
'CjkUnifiedIdeograph4d9c': '\xe4\xb6\x9c',
'CjkUnifiedIdeograph4d9b': '\xe4\xb6\x9b',
'CjkUnifiedIdeograph4d9e': '\xe4\xb6\x9e',
'CjkUnifiedIdeograph4d9d': '\xe4\xb6\x9d',
'CjkUnifiedIdeograph4d9f': '\xe4\xb6\x9f',
'TaiLeLetterEe': '\xe1\xa5\xa5',
'UnicodeCharacter0xe1d1': '\xee\x87\x91',
'TaiLeLetterEh': '\xe1\xa5\xa6',
'BoxDrawingsLightDoubleDashHorizontal': '\xe2\x95\x8c',
'HangulSyllableBbyeot': '\xeb\xbc\xa1',
'UnicodeCharacter0xec4e': '\xee\xb1\x8e',
'CjkUnifiedIdeograph4d99': '\xe4\xb6\x99',
'CjkUnifiedIdeograph4d98': '\xe4\xb6\x98',
'CombiningAcuteToneMark': '\xcd\x81',
'HangulSyllableHwilh': '\xed\x9c\xa7',
'CjkUnifiedIdeograph4d91': '\xe4\xb6\x91',
'CjkUnifiedIdeograph4d90': '\xe4\xb6\x90',
'CjkUnifiedIdeograph4d93': '\xe4\xb6\x93',
'CjkUnifiedIdeograph4d92': '\xe4\xb6\x92',
'CjkUnifiedIdeograph4d95': '\xe4\xb6\x95',
'CjkUnifiedIdeograph4d94': '\xe4\xb6\x94',
'CjkUnifiedIdeograph4d97': '\xe4\xb6\x97',
'CjkUnifiedIdeograph4d96': '\xe4\xb6\x96',
'CjkUnifiedIdeograph5a26': '\xe5\xa8\xa6',
'CjkUnifiedIdeograph5a27': '\xe5\xa8\xa7',
'CjkUnifiedIdeograph5a24': '\xe5\xa8\xa4',
'CjkUnifiedIdeograph5a25': '\xe5\xa8\xa5',
'CjkUnifiedIdeograph5a22': '\xe5\xa8\xa2',
'CjkUnifiedIdeograph5a23': '\xe5\xa8\xa3',
'CjkUnifiedIdeograph5a20': '\xe5\xa8\xa0',
'CjkUnifiedIdeograph5a21': '\xe5\xa8\xa1',
'CjkUnifiedIdeograph4a4f': '\xe4\xa9\x8f',
'CjkUnifiedIdeograph4a4e': '\xe4\xa9\x8e',
'CjkUnifiedIdeograph4a4d': '\xe4\xa9\x8d',
'CjkUnifiedIdeograph4a4c': '\xe4\xa9\x8c',
'CjkUnifiedIdeograph4a4b': '\xe4\xa9\x8b',
'CjkUnifiedIdeograph4a4a': '\xe4\xa9\x8a',
'CjkUnifiedIdeograph5a29': '\xe5\xa8\xa9',
'UnicodeCharacter0xe377': '\xee\x8d\xb7',
'HangulSyllableBbye': '\xeb\xbc\xa4',
'HangulSyllableHilg': '\xed\x9e\x91',
'ArabicLigatureThehWithHehMedialForm': '\xef\xb3\xa6',
'UnicodeCharacter0xf5c1': '\xef\x97\x81',
'UnicodeCharacter0xdb6d': '\xed\xad\xad',
'UnicodeCharacter0xe5f8': '\xee\x97\xb8',
'ArabicLigatureNoonWithJeemWithAlefMaksuraFinalForm': '\xef\xb6\x99',
'CjkUnifiedIdeograph9d5f': '\xe9\xb5\x9f',
'FullwidthColon': '\xef\xbc\x9a',
'HangulSyllableSseonh': '\xec\x8d\xae',
'UnicodeCharacter0xdb6f': '\xed\xad\xaf',
'ArabicLigatureHahWithYehFinalForm': '\xef\xb4\x9c',
'RightwardsHarpoonWithBarbDownAboveLeftwardsHarpoonWithBarbDown': '\xe2\xa5\xa9',
'HangulSyllableMyonh': '\xeb\xac\x9e',
'UnicodeCharacter0xf1fc': '\xef\x87\xbc',
'HangulSyllableKyaeh': '\xec\xbb\xa3',
'HangulSyllableMyonj': '\xeb\xac\x9d',
'HangulSyllableGyeoss': '\xea\xb2\xbc',
'UnicodeCharacter0xe379': '\xee\x8d\xb9',
'UnicodeCharacter0xdbc0': '\xed\xaf\x80',
'UnicodeCharacter0xe00': '\xe0\xb8\x80',
'ArabicLigatureHahWithAlefMaksuraIsolatedForm': '\xef\xb3\xbf',
'GeorgianCapitalLetterHie': '\xe1\x83\x82',
'HangulSyllableMyoss': '\xeb\xac\xac',
'OcrInvertedFork': '\xe2\x91\x83',
'UnicodeCharacter0xf6ac': '\xef\x9a\xac',
'HangulSyllableCweogg': '\xec\xb6\xb2',
'LeftRightArrowWithVerticalStroke': '\xe2\x87\xb9',
'HangulSyllableHabs': '\xed\x95\xaa',
'UnicodeCharacter0xf6aa': '\xef\x9a\xaa',
'HangulSyllableCweogs': '\xec\xb6\xb3',
'KayahLiLetterHta': '\xea\xa4\x93',
'CjkUnifiedIdeograph74a8': '\xe7\x92\xa8',
'CircledHangulChieuch': '\xe3\x89\xa9',
'CjkUnifiedIdeograph74a9': '\xe7\x92\xa9',
'HalfwidthKatakanaLetterMu': '\xef\xbe\x91',
'AplFunctionalSymbolOmegaUnderbar': '\xe2\x8d\xb9',
'UnicodeCharacter0xf6ae': '\xef\x9a\xae',
'UnicodeCharacter0xd986': '\xed\xa6\x86',
'UnicodeCharacter0xe5f4': '\xee\x97\xb4',
'CjkUnifiedIdeograph4d7c': '\xe4\xb5\xbc',
'CjkUnifiedIdeograph8e8e': '\xe8\xba\x8e',
'CjkUnifiedIdeograph74a5': '\xe7\x92\xa5',
'CjkUnifiedIdeograph8e8d': '\xe8\xba\x8d',
'KayahLiLetterYa': '\xea\xa4\x9b',
'CjkUnifiedIdeograph9028': '\xe9\x80\xa8',
'CanadianSyllabicsCarrierThee': '\xe1\x97\x9f',
'LatinSubscriptSmallLetterE': '\xe2\x82\x91',
'UnicodeCharacter0xe5f5': '\xee\x97\xb5',
'HangulSyllableDwaelp': '\xeb\x90\x8a',
'CjkUnifiedIdeograph830e': '\xe8\x8c\x8e',
'CjkUnifiedIdeograph830d': '\xe8\x8c\x8d',
'TeluguLetterDa': '\xe0\xb0\xa6',
'CjkUnifiedIdeograph830a': '\xe8\x8c\x8a',
'UnicodeCharacter0xaadd': '\xea\xab\x9d',
'CjkUnifiedIdeograph830c': '\xe8\x8c\x8c',
'CjkUnifiedIdeograph830b': '\xe8\x8c\x8b',
'MalayalamDigitZero': '\xe0\xb5\xa6',
'CjkUnifiedIdeograph6fdc': '\xe6\xbf\x9c',
'CjkUnifiedIdeograph6fda': '\xe6\xbf\x9a',
'CjkUnifiedIdeograph6fdf': '\xe6\xbf\x9f',
'CjkUnifiedIdeograph6fdd': '\xe6\xbf\x9d',
'CjkUnifiedIdeograph6fde': '\xe6\xbf\x9e',
'CanadianSyllabicsCarrierHwee': '\xe1\x97\x99',
'UnicodeCharacter0xe5f6': '\xee\x97\xb6',
'HangulSyllableKyaep': '\xec\xbb\xa2',
'OlChikiLetterIh': '\xe1\xb1\xa6',
'OlChikiLetterIs': '\xe1\xb1\xa5',
'OlChikiLetterIr': '\xe1\xb1\xa8',
'UpwardsHarpoonWithBarbLeftwards': '\xe2\x86\xbf',
'UnicodeCharacter0xf793': '\xef\x9e\x93',
'HangulSyllableJjwass': '\xec\xab\xac',
'CjkUnifiedIdeograph8305': '\xe8\x8c\x85',
'CjkUnifiedIdeograph8304': '\xe8\x8c\x84',
'CjkUnifiedIdeograph6fd8': '\xe6\xbf\x98',
'CjkUnifiedIdeograph6fd9': '\xe6\xbf\x99',
'CjkUnifiedIdeograph8301': '\xe8\x8c\x81',
'CjkUnifiedIdeograph8300': '\xe8\x8c\x80',
'CjkUnifiedIdeograph8303': '\xe8\x8c\x83',
'CjkUnifiedIdeograph8302': '\xe8\x8c\x82',
'CjkUnifiedIdeograph6fd2': '\xe6\xbf\x92',
'CjkUnifiedIdeograph6fd3': '\xe6\xbf\x93',
'CjkUnifiedIdeograph6fd0': '\xe6\xbf\x90',
'SquareSenti': '\xe3\x8c\xa2',
'CjkUnifiedIdeograph6fd6': '\xe6\xbf\x96',
'SquareSento': '\xe3\x8c\xa3',
'CjkUnifiedIdeograph6fd4': '\xe6\xbf\x94',
'CjkUnifiedIdeograph6fd5': '\xe6\xbf\x95',
'UnicodeCharacter0xed13': '\xee\xb4\x93',
'ArabicLetterFarsiYehIsolatedForm': '\xef\xaf\xbc',
'CyrillicLetterMultiocularO': '\xea\x99\xae',
'UnicodeCharacter0xe5c7': '\xee\x97\x87',
'UnicodeCharacter0xe5f0': '\xee\x97\xb0',
'HangulSyllableGgwaegg': '\xea\xbd\xa6',
'HangulSyllableYebs': '\xec\x98\x9a',
'CjkUnifiedIdeograph89eb': '\xe8\xa7\xab',
'HangulChoseongIeungtikeut': '\xe1\x85\x82',
'UnicodeCharacter0xaaeb': '\xea\xab\xab',
'SundaneseLetterTa': '\xe1\xae\x92',
'VaiSyllableTee': '\xea\x94\x8e',
'HangulSyllablePu': '\xed\x91\xb8',
'CjkCompatibilityIdeographf986': '\xef\xa6\x86',
'HangulSyllableByulg': '\xeb\xb7\xb9',
'CjkCompatibilityIdeographf980': '\xef\xa6\x80',
'HangulSyllablePe': '\xed\x8e\x98',
'ArabicLetterDulFinalForm': '\xef\xae\x87',
'CjkCompatibilityIdeographf983': '\xef\xa6\x83',
'HangulSyllablePa': '\xed\x8c\x8c',
'HangulSyllablePo': '\xed\x8f\xac',
'HangulSyllableByulb': '\xeb\xb7\xbb',
'UnicodeCharacter0xea8b': '\xee\xaa\x8b',
'HangulSyllablePi': '\xed\x94\xbc',
'UnicodeCharacter0xe5f2': '\xee\x97\xb2',
'ArabicLetterHah': '\xd8\xad',
'HangulSyllableKeok': '\xec\xbb\xbc',
'CjkUnifiedIdeograph93d3': '\xe9\x8f\x93',
'UnicodeCharacter0xa7de': '\xea\x9f\x9e',
'ModifierLetterExtralowDottedLeftstemToneBar': '\xea\x9c\x91',
'CjkUnifiedIdeograph89ed': '\xe8\xa7\xad',
'HangulSyllableSswenh': '\xec\x92\xa2',
'HangulSyllableDi': '\xeb\x94\x94',
'VaiSyllableOn': '\xea\x96\xbb',
'VaiSyllableOo': '\xea\x95\xb1',
'CjkUnifiedIdeograph8dd6': '\xe8\xb7\x96',
'CjkUnifiedIdeograph8dd7': '\xe8\xb7\x97',
'CjkUnifiedIdeograph8dd4': '\xe8\xb7\x94',
'LatinSmallLetterUWithDiaeresisAndMacron': '\xc7\x96',
'CjkUnifiedIdeograph8dd2': '\xe8\xb7\x92',
'CjkUnifiedIdeograph8dd3': '\xe8\xb7\x93',
'GlagoliticCapitalLetterI': '\xe2\xb0\x8b',
'CjkUnifiedIdeograph8dd1': '\xe8\xb7\x91',
'CjkUnifiedIdeograph7c48': '\xe7\xb1\x88',
'CjkUnifiedIdeograph901f': '\xe9\x80\x9f',
'CjkUnifiedIdeograph8dd8': '\xe8\xb7\x98',
'CjkUnifiedIdeograph8dd9': '\xe8\xb7\x99',
'UnicodeCharacter0xa7db': '\xea\x9f\x9b',
'SaurashtraVowelSignEe': '\xea\xa2\xbf',
'UnicodeCharacter0xa7dc': '\xea\x9f\x9c',
'RightAngleBracket': '\xe3\x80\x89',
'UnicodeCharacter0xe9ac': '\xee\xa6\xac',
'LepchaLetterNya': '\xe1\xb0\x89',
'HangulSyllableDa': '\xeb\x8b\xa4',
'CjkUnifiedIdeograph8ddf': '\xe8\xb7\x9f',
'CjkUnifiedIdeograph8ddd': '\xe8\xb7\x9d',
'CjkUnifiedIdeograph8dde': '\xe8\xb7\x9e',
'CjkUnifiedIdeograph8ddb': '\xe8\xb7\x9b',
'CjkUnifiedIdeograph8ddc': '\xe8\xb7\x9c',
'CjkUnifiedIdeograph89ef': '\xe8\xa7\xaf',
'CjkUnifiedIdeograph8dda': '\xe8\xb7\x9a',
'HangulSyllableKeot': '\xec\xbb\xbd',
'HangulSyllableKeop': '\xec\xbb\xbe',
'UnicodeCharacter0xed92': '\xee\xb6\x92',
'HangulSyllableKeos': '\xec\xbb\xb7',
'HangulSyllableRyej': '\xeb\xa1\x96',
'HangulSyllableKeom': '\xec\xbb\xb4',
'HangulSyllableGgyeolb': '\xea\xbb\xbf',
'HangulSyllableKeoh': '\xec\xbb\xbf',
'HangulSyllableDe': '\xeb\x8d\xb0',
'HangulSyllableKeoj': '\xec\xbb\xba',
'AplFunctionalSymbolGreaterthanDiaeresis': '\xe2\x8d\xa9',
'HangulSyllableKeod': '\xec\xbb\xab',
'TaiLeLetterOo': '\xe1\xa5\xa8',
'HangulSyllableKeog': '\xec\xbb\xa5',
'CjkCompatibilityIdeographf9e4': '\xef\xa7\xa4',
'HangulSyllableKeob': '\xec\xbb\xb5',
'HangulSyllableKeoc': '\xec\xbb\xbb',
'UnicodeCharacter0xed96': '\xee\xb6\x96',
'YiSyllableYurx': '\xea\x92\x85',
'HangulSyllableYop': '\xec\x9a\xae',
'HangulSyllableYos': '\xec\x9a\xa7',
'HangulSyllableYot': '\xec\x9a\xad',
'GeorgianSmallLetterJil': '\xe2\xb4\x9b',
'HangulSyllableYoh': '\xec\x9a\xaf',
'UnicodeCharacter0xed98': '\xee\xb6\x98',
'HangulSyllableYoj': '\xec\x9a\xaa',
'HangulSyllableYok': '\xec\x9a\xac',
'HangulSyllableYol': '\xec\x9a\x9c',
'CjkUnifiedIdeograph8dac': '\xe8\xb6\xac',
'HangulSyllableYon': '\xec\x9a\x98',
'HangulSyllableYob': '\xec\x9a\xa5',
'CjkUnifiedIdeograph8dab': '\xe8\xb6\xab',
'HangulSyllableYod': '\xec\x9a\x9b',
'EthiopicSyllableRya': '\xe1\x8d\x98',
'HangulSyllableYog': '\xec\x9a\x95',
'HangulSyllableSaet': '\xec\x83\xa1',
'HangulSyllableSaes': '\xec\x83\x9b',
'HangulSyllableSaep': '\xec\x83\xa2',
'HangulSyllableSaen': '\xec\x83\x8c',
'TamilDigitSix': '\xe0\xaf\xac',
'HangulSyllableSael': '\xec\x83\x90',
'HangulSyllableSaem': '\xec\x83\x98',
'HangulSyllableSaej': '\xec\x83\x9e',
'HangulSyllableSaek': '\xec\x83\xa0',
'HangulSyllableSaeh': '\xec\x83\xa3',
'HangulSyllableSaeg': '\xec\x83\x89',
'HangulSyllableSaed': '\xec\x83\x8f',
'HangulSyllableSaeb': '\xec\x83\x99',
'HangulSyllableSaec': '\xec\x83\x9f',
'HangulSyllableGweobs': '\xea\xb6\x9a',
'HangulSyllableBbalh': '\xeb\xb9\xaf',
'HangulSyllableDwaed': '\xeb\x90\x83',
'UnicodeCharacter0xf159': '\xef\x85\x99',
'DeltaEqualTo': '\xe2\x89\x9c',
'HebrewLetterWideKaf': '\xef\xac\xa4',
'LimitedLiabilitySign': '\xe3\x8b\x8f',
'KhmerVowelSignAe': '\xe1\x9f\x82',
'HangulSyllableGgaebs': '\xea\xb9\xba',
'ReversedSemicolon': '\xe2\x81\x8f',
'HangulSyllableRaep': '\xeb\x9e\xb2',
'HangulSyllableRaes': '\xeb\x9e\xab',
'HangulSyllableRaet': '\xeb\x9e\xb1',
'UnicodeCharacter0xf791': '\xef\x9e\x91',
'KhmerVowelSignAa': '\xe1\x9e\xb6',
'HangulSyllableRaec': '\xeb\x9e\xaf',
'HangulSyllableRaeb': '\xeb\x9e\xa9',
'HangulSyllableRaed': '\xeb\x9e\x9f',
'HangulSyllableRaeg': '\xeb\x9e\x99',
'HangulSyllableRaeh': '\xeb\x9e\xb3',
'HangulSyllableRaek': '\xeb\x9e\xb0',
'HangulSyllableRaej': '\xeb\x9e\xae',
'HangulSyllableRaem': '\xeb\x9e\xa8',
'HangulSyllableRael': '\xeb\x9e\xa0',
'HangulSyllableRaen': '\xeb\x9e\x9c',
'UnicodeCharacter0xf153': '\xef\x85\x93',
'HangulSyllableJweop': '\xec\xa4\xb2',
'CjkUnifiedIdeograph85fa': '\xe8\x97\xba',
'CjkUnifiedIdeograph85fc': '\xe8\x97\xbc',
'CjkUnifiedIdeograph85fb': '\xe8\x97\xbb',
'CjkUnifiedIdeograph85fe': '\xe8\x97\xbe',
'CjkUnifiedIdeograph85fd': '\xe8\x97\xbd',
'|/\\': '\xe2\x86\x91',
'CjkUnifiedIdeograph85ff': '\xe8\x97\xbf',
'CjkUnifiedIdeograph8b5c': '\xe8\xad\x9c',
'CjkUnifiedIdeograph8b5b': '\xe8\xad\x9b',
'CjkUnifiedIdeograph8b5a': '\xe8\xad\x9a',
'UnicodeCharacter0xd876': '\xed\xa1\xb6',
'UnicodeCharacter0xd871': '\xed\xa1\xb1',
'CjkUnifiedIdeograph8b5f': '\xe8\xad\x9f',
'CjkUnifiedIdeograph8b5e': '\xe8\xad\x9e',
'CjkUnifiedIdeograph8b5d': '\xe8\xad\x9d',
'UnicodeCharacter0xf157': '\xef\x85\x97',
'UnicodeCharacter0xd879': '\xed\xa1\xb9',
'UnicodeCharacter0xd878': '\xed\xa1\xb8',
'UnicodeCharacter0xf6a3': '\xef\x9a\xa3',
'HangulSyllableSsyeolt': '\xec\x8e\xad',
'UnicodeCharacter0xd983': '\xed\xa6\x83',
'UnicodeCharacter0xe567': '\xee\x95\xa7',
'UnicodeCharacter0xaad8': '\xea\xab\x98',
'HangulLetterPieupsiostikeut': '\xe3\x85\xb5',
'CjkUnifiedIdeograph4d78': '\xe4\xb5\xb8',
'ArabicLigatureMohammadIsolatedForm': '\xef\xb7\xb4',
'UnicodeCharacter0xf6a7': '\xef\x9a\xa7',
'UnicodeCharacter0xe15c': '\xee\x85\x9c',
'CjkUnifiedIdeograph6119': '\xe6\x84\x99',
'UnicodeCharacter0xeee3': '\xee\xbb\xa3',
'UnicodeCharacter0xf6a6': '\xef\x9a\xa6',
'GreekDasiaAndPerispomeni': '\xe1\xbf\x9f',
'WhiteCircleWithLowerRightQuadrant': '\xe2\x97\xb6',
'CjkUnifiedIdeograph8b51': '\xe8\xad\x91',
'CjkUnifiedIdeograph8b50': '\xe8\xad\x90',
'CjkUnifiedIdeograph8b57': '\xe8\xad\x97',
'CjkUnifiedIdeograph8b56': '\xe8\xad\x96',
'EthiopicSyllableXwaa': '\xe1\x8a\x8b',
'CjkUnifiedIdeograph8b54': '\xe8\xad\x94',
'CjkUnifiedIdeograph8b59': '\xe8\xad\x99',
'CjkUnifiedIdeograph8b58': '\xe8\xad\x98',
'GurmukhiLetterKhha': '\xe0\xa9\x99',
'BraillePatternDots678': '\xe2\xa3\xa0',
'CjkUnifiedIdeograph85f9': '\xe8\x97\xb9',
'CjkUnifiedIdeograph85f8': '\xe8\x97\xb8',
'CanadianSyllabicsYcreeKoo': '\xe1\x91\xb1',
'HebrewLetterHe': '\xd7\x94',
'UnicodeCharacter0xaad3': '\xea\xab\x93',
'CjkUnifiedIdeograph85f1': '\xe8\x97\xb1',
'CjkUnifiedIdeograph85f0': '\xe8\x97\xb0',
'CjkUnifiedIdeograph85f3': '\xe8\x97\xb3',
'CombiningCircumflexAccent': '\xcc\x82',
'CjkUnifiedIdeograph85f5': '\xe8\x97\xb5',
'CjkUnifiedIdeograph85f4': '\xe8\x97\xb4',
'CjkUnifiedIdeograph85f7': '\xe8\x97\xb7',
'CjkUnifiedIdeograph7484': '\xe7\x92\x84',
'UnicodeCharacter0xaad1': '\xea\xab\x91',
'CjkUnifiedIdeograph8e6d': '\xe8\xb9\xad',
'CjkUnifiedIdeograph4d77': '\xe4\xb5\xb7',
'UnicodeCharacter0xaad7': '\xea\xab\x97',
'HebrewMarkLowerDot': '\xd7\x85',
'DevanagariDigitFour': '\xe0\xa5\xaa',
'UnicodeCharacter0xaad5': '\xea\xab\x95',
'SquareKhz': '\xe3\x8e\x91',
'UnicodeCharacter0xeadf': '\xee\xab\x9f',
'HangulSyllablePweoc': '\xed\x92\xab',
'UnicodeCharacter0xeadd': '\xee\xab\x9d',
'UnicodeCharacter0xeade': '\xee\xab\x9e',
'UnicodeCharacter0xeadb': '\xee\xab\x9b',
'HangulSyllableJweog': '\xec\xa4\x99',
'HangulSyllableDdeogs': '\xeb\x96\xa3',
'MyanmarShanDigitSeven': '\xe1\x82\x97',
'ArabicLetterFehFinalForm': '\xef\xbb\x92',
'LaoVowelSignIi': '\xe0\xba\xb5',
'HangulSyllablePweog': '\xed\x92\x95',
'HangulSyllableSsyeolp': '\xec\x8e\xae',
'HangulSyllableCwegs': '\xec\xb7\x8f',
'CombiningDoubleVerticalLineAbove': '\xcc\x8e',
'HangulSyllablePweod': '\xed\x92\x9b',
'CjkUnifiedIdeograph6668': '\xe6\x99\xa8',
'HangulSyllableCwegg': '\xec\xb7\x8e',
'HangulSyllableSyaess': '\xec\x84\x94',
'ModifierLetterSmallTopHalfO': '\xe1\xb5\x94',
'HangulSyllableTulb': '\xed\x88\xb7',
'HangulSyllablePweoh': '\xed\x92\xaf',
'ArabicLetterU': '\xdb\x87',
'KangxiRadicalFur': '\xe2\xbd\x91',
'HangulSyllableTulg': '\xed\x88\xb5',
'ForkedParagraphos': '\xe2\xb8\x90',
'HangulSyllableAlp': '\xec\x95\x92',
'HangulSyllableAls': '\xec\x95\x90',
'HangulSyllableAlt': '\xec\x95\x91',
'SubscriptMinus': '\xe2\x82\x8b',
'HangulJongseongPhieuphpieup': '\xe1\x87\xb3',
'CjkUnifiedIdeograph7cc5': '\xe7\xb3\x85',
'UnicodeCharacter0xe44d': '\xee\x91\x8d',
'HangulSyllableTuls': '\xed\x88\xb8',
'ArabicLetterE': '\xdb\x90',
'HangulSyllablePweom': '\xed\x92\xa4',
'PhagspaLetterNa': '\xea\xa1\x8b',
'CjkUnifiedIdeograph7cc4': '\xe7\xb3\x84',
'LatinSmallLetterDWithDotBelow': '\xe1\xb8\x8d',
'CjkUnifiedIdeograph3e5f': '\xe3\xb9\x9f',
'CjkUnifiedIdeograph7cc7': '\xe7\xb3\x87',
'HangulSyllableAlg': '\xec\x95\x8d',
'UnicodeCharacter0xead9': '\xee\xab\x99',
'[9]': '\xe2\x82\x89',
'GeorgianLetterCan': '\xe1\x83\xaa',
'HangulSyllableHeub': '\xed\x9d\xa1',
'HangulSyllableGgyogs': '\xea\xbe\x9f',
'HangulSyllableSsyeolm': '\xec\x8e\xaa',
'ClearScreenSymbol': '\xe2\x8e\x9a',
'HangulSyllableJwinh': '\xec\xa5\x96',
'UnicodeCharacter0xe3da': '\xee\x8f\x9a',
'UnicodeCharacter0xea4a': '\xee\xa9\x8a',
'CjkUnifiedIdeograph901e': '\xe9\x80\x9e',
'SquareRupii': '\xe3\x8d\x93',
'BraillePatternDots56': '\xe2\xa0\xb0',
'HangulSyllableGyeonh': '\xea\xb2\xae',
'CjkUnifiedIdeograph9b78': '\xe9\xad\xb8',
'CjkUnifiedIdeograph9b79': '\xe9\xad\xb9',
'CjkUnifiedIdeograph9b76': '\xe9\xad\xb6',
'CjkUnifiedIdeograph9b77': '\xe9\xad\xb7',
'CjkUnifiedIdeograph9b74': '\xe9\xad\xb4',
'CjkUnifiedIdeograph9b75': '\xe9\xad\xb5',
'CjkUnifiedIdeograph9b72': '\xe9\xad\xb2',
'CjkUnifiedIdeograph9b73': '\xe9\xad\xb3',
'CjkUnifiedIdeograph9b70': '\xe9\xad\xb0',
'CjkUnifiedIdeograph9b71': '\xe9\xad\xb1',
'UnicodeCharacter0xdef6': '\xed\xbb\xb6',
'UnicodeCharacter0xec05': '\xee\xb0\x85',
'CjkUnifiedIdeograph8ffc': '\xe8\xbf\xbc',
'CjkUnifiedIdeograph7946': '\xe7\xa5\x86',
'CjkUnifiedIdeograph7947': '\xe7\xa5\x87',
'CjkUnifiedIdeograph4369': '\xe4\x8d\xa9',
'CjkUnifiedIdeograph4368': '\xe4\x8d\xa8',
'CjkUnifiedIdeograph7942': '\xe7\xa5\x82',
'CjkUnifiedIdeograph7943': '\xe7\xa5\x83',
'CjkUnifiedIdeograph7940': '\xe7\xa5\x80',
'AscendingNode': '\xe2\x98\x8a',
'CjkUnifiedIdeograph4363': '\xe4\x8d\xa3',
'CjkUnifiedIdeograph4362': '\xe4\x8d\xa2',
'CjkUnifiedIdeograph4361': '\xe4\x8d\xa1',
'CjkUnifiedIdeograph4360': '\xe4\x8d\xa0',
'CjkUnifiedIdeograph4367': '\xe4\x8d\xa7',
'CjkUnifiedIdeograph4366': '\xe4\x8d\xa6',
'CjkUnifiedIdeograph4365': '\xe4\x8d\xa5',
'CjkUnifiedIdeograph4364': '\xe4\x8d\xa4',
'YiSyllableNbux': '\xea\x82\x86',
'CjkUnifiedIdeograph9b7f': '\xe9\xad\xbf',
'UnicodeCharacter0xdabf': '\xed\xaa\xbf',
'CjkUnifiedIdeograph9b7d': '\xe9\xad\xbd',
'CjkUnifiedIdeograph9b7e': '\xe9\xad\xbe',
'CjkUnifiedIdeograph9b7b': '\xe9\xad\xbb',
'CjkUnifiedIdeograph9b7c': '\xe9\xad\xbc',
'UnicodeCharacter0xdaba': '\xed\xaa\xba',
'CjkUnifiedIdeograph9b7a': '\xe9\xad\xba',
'CjkUnifiedIdeograph8da5': '\xe8\xb6\xa5',
'LatinSmallLetterVisigothicZ': '\xea\x9d\xa3',
'CjkUnifiedIdeograph8da4': '\xe8\xb6\xa4',
'LatinCapitalLetterHwair': '\xc7\xb6',
'HangulSyllableByeobs': '\xeb\xb3\x8e',
'CjkUnifiedIdeograph794f': '\xe7\xa5\x8f',
'CjkUnifiedIdeograph794d': '\xe7\xa5\x8d',
'CjkUnifiedIdeograph794e': '\xe7\xa5\x8e',
'CjkUnifiedIdeograph794b': '\xe7\xa5\x8b',
'CjkUnifiedIdeograph794c': '\xe7\xa5\x8c',
'HexagramForDevelopment': '\xe4\xb7\xb4',
'CjkUnifiedIdeograph436c': '\xe4\x8d\xac',
'CjkUnifiedIdeograph436b': '\xe4\x8d\xab',
'CjkUnifiedIdeograph436a': '\xe4\x8d\xaa',
'CjkUnifiedIdeograph7367': '\xe7\x8d\xa7',
'CjkUnifiedIdeograph436f': '\xe4\x8d\xaf',
'CjkUnifiedIdeograph436e': '\xe4\x8d\xae',
'CjkUnifiedIdeograph436d': '\xe4\x8d\xad',
'CjkUnifiedIdeograph8da2': '\xe8\xb6\xa2',
'BopomofoLetterB': '\xe3\x84\x85',
'BopomofoLetterC': '\xe3\x84\x98',
'SylotiNagriLetterGo': '\xea\xa0\x89',
'BopomofoLetterA': '\xe3\x84\x9a',
'BopomofoLetterF': '\xe3\x84\x88',
'BopomofoLetterG': '\xe3\x84\x8d',
'BopomofoLetterD': '\xe3\x84\x89',
'BopomofoLetterE': '\xe3\x84\x9c',
'BopomofoLetterJ': '\xe3\x84\x90',
'BopomofoLetterK': '\xe3\x84\x8e',
'BopomofoLetterH': '\xe3\x84\x8f',
'BopomofoLetterI': '\xe3\x84\xa7',
'BopomofoLetterN': '\xe3\x84\x8b',
'BopomofoLetterO': '\xe3\x84\x9b',
'BopomofoLetterL': '\xe3\x84\x8c',
'BopomofoLetterM': '\xe3\x84\x87',
'BopomofoLetterR': '\xe3\x84\x96',
'BopomofoLetterS': '\xe3\x84\x99',
'BopomofoLetterP': '\xe3\x84\x86',
'BopomofoLetterQ': '\xe3\x84\x91',
'BopomofoLetterV': '\xe3\x84\xaa',
'Sho': '\xcf\xb7',
'BopomofoLetterT': '\xe3\x84\x8a',
'BopomofoLetterU': '\xe3\x84\xa8',
'BopomofoLetterZ': '\xe3\x84\x97',
'KannadaDigitZero': '\xe0\xb3\xa6',
'BopomofoLetterX': '\xe3\x84\x92',
'KayahLiDigitSeven': '\xea\xa4\x87',
'HangulSyllablePyolm': '\xed\x91\xa6',
'UnicodeCharacter0xdcf4': '\xed\xb3\xb4',
'HangulSyllableHeud': '\xed\x9d\x97',
'BraillePatternDots124678': '\xe2\xa3\xab',
'CyrillicSmallLetterIotifiedYat': '\xea\x99\x93',
'HangulSyllableSsyeolg': '\xec\x8e\xa9',
'HangulSyllableBwaebs': '\xeb\xb4\xbe',
'ArabicLigatureLamWithAlefWithHamzaBelowFinalForm': '\xef\xbb\xba',
'CjkUnifiedIdeograph7200': '\xe7\x88\x80',
'HangulSyllableJjwanh': '\xec\xab\x9e',
'CombiningLeftTackBelow': '\xcc\x98',
'HangulSyllableJjwang': '\xec\xab\xad',
'HangulSyllableHwabs': '\xed\x99\xa6',
'HangulChoseongKapyeounphieuph': '\xe1\x85\x97',
'UnicodeCharacter0xe405': '\xee\x90\x85',
'CjkUnifiedIdeograph9ade': '\xe9\xab\x9e',
'ModifierLetterSmallTurnedAlpha': '\xe1\xb6\x9b',
'CjkUnifiedIdeograph7363': '\xe7\x8d\xa3',
'UnicodeCharacter0xe51f': '\xee\x94\x9f',
'HangulSyllableSip': '\xec\x8b\xb6',
'UnicodeCharacter0xf129': '\xef\x84\xa9',
'CjkUnifiedIdeograph9209': '\xe9\x88\x89',
'CjkUnifiedIdeograph9208': '\xe9\x88\x88',
'HangulSyllableGad': '\xea\xb0\x87',
'CjkUnifiedIdeograph9203': '\xe9\x88\x83',
'CjkUnifiedIdeograph9202': '\xe9\x88\x82',
'CjkUnifiedIdeograph9201': '\xe9\x88\x81',
'CjkUnifiedIdeograph9200': '\xe9\x88\x80',
'UnicodeCharacter0x2150': '\xe2\x85\x90',
'UnicodeCharacter0x2151': '\xe2\x85\x91',
'UnicodeCharacter0x2152': '\xe2\x85\x92',
'CjkUnifiedIdeograph9204': '\xe9\x88\x84',
'CjkUnifiedIdeograph68a9': '\xe6\xa2\xa9',
'CjkUnifiedIdeograph68a8': '\xe6\xa2\xa8',
'->>': '\xe2\x86\xa0',
'CjkUnifiedIdeograph68a3': '\xe6\xa2\xa3',
'CjkUnifiedIdeograph68a2': '\xe6\xa2\xa2',
'CjkUnifiedIdeograph68a1': '\xe6\xa2\xa1',
'CjkUnifiedIdeograph68a0': '\xe6\xa2\xa0',
'CjkUnifiedIdeograph68a7': '\xe6\xa2\xa7',
'CjkUnifiedIdeograph68a6': '\xe6\xa2\xa6',
'CjkUnifiedIdeograph68a5': '\xe6\xa2\xa5',
'CjkUnifiedIdeograph68a4': '\xe6\xa2\xa4',
'HangulSyllableDyek': '\xeb\x8f\x80',
'EthiopicSyllableKa': '\xe1\x8a\xa8',
'HangulSyllableDyeh': '\xeb\x8f\x83',
'HangulSyllableDyeo': '\xeb\x8e\x8c',
'EthiopicSyllableKe': '\xe1\x8a\xad',
'HangulSyllableDyem': '\xeb\x8e\xb8',
'HangulSyllableDyel': '\xeb\x8e\xb0',
'HangulSyllableDyec': '\xeb\x8e\xbf',
'EthiopicSyllableKi': '\xe1\x8a\xaa',
'CjkUnifiedIdeograph8ff9': '\xe8\xbf\xb9',
'HangulSyllableDyeg': '\xeb\x8e\xa9',
'HangulSyllableSij': '\xec\x8b\xb2',
'EthiopicSyllableKo': '\xe1\x8a\xae',
'CjkUnifiedIdeograph9cea': '\xe9\xb3\xaa',
'CjkUnifiedIdeograph8ff6': '\xe8\xbf\xb6',
'CjkUnifiedIdeograph9cec': '\xe9\xb3\xac',
'CjkUnifiedIdeograph9ceb': '\xe9\xb3\xab',
'CjkUnifiedIdeograph9cee': '\xe9\xb3\xae',
'EthiopicSyllableKu': '\xe1\x8a\xa9',
'CjkUnifiedIdeograph9cef': '\xe9\xb3\xaf',
'HangulSyllableDyes': '\xeb\x8e\xbb',
'HangulSyllableDyep': '\xeb\x8f\x82',
'GeorgianSmallLetterOn': '\xe2\xb4\x8d',
'HangulSyllableMoebs': '\xeb\xac\x8e',
'LatinSmallLetterPWithHook': '\xc6\xa5',
'HangulSyllableJwing': '\xec\xa5\xa5',
'CjkUnifiedIdeograph9586': '\xe9\x96\x86',
'UnicodeCharacter0xf213': '\xef\x88\x93',
'VaiSyllableNgen': '\xea\x97\xa3',
'CjkUnifiedIdeograph68ac': '\xe6\xa2\xac',
'CjkUnifiedIdeograph68ab': '\xe6\xa2\xab',
'CjkUnifiedIdeograph68aa': '\xe6\xa2\xaa',
'CjkUnifiedIdeograph68af': '\xe6\xa2\xaf',
'CjkUnifiedIdeograph68ae': '\xe6\xa2\xae',
'CjkUnifiedIdeograph68ad': '\xe6\xa2\xad',
'HangulSyllableGgeot': '\xea\xbb\x95',
'CircledNumberFourteen': '\xe2\x91\xad',
'HangulSyllableDos': '\xeb\x8f\x97',
'CjkUnifiedIdeograph920c': '\xe9\x88\x8c',
'CjkUnifiedIdeograph920b': '\xe9\x88\x8b',
'CjkUnifiedIdeograph920a': '\xe9\x88\x8a',
'CjkUnifiedIdeograph920f': '\xe9\x88\x8f',
'CjkUnifiedIdeograph920e': '\xe9\x88\x8e',
'CjkUnifiedIdeograph920d': '\xe9\x88\x8d',
'CjkUnifiedIdeograph9587': '\xe9\x96\x87',
'CjkUnifiedIdeograph3e59': '\xe3\xb9\x99',
'CjkUnifiedIdeograph9ce3': '\xe9\xb3\xa3',
'CjkUnifiedIdeograph9ce2': '\xe9\xb3\xa2',
'CjkUnifiedIdeograph9ce5': '\xe9\xb3\xa5',
'CjkUnifiedIdeograph9ce4': '\xe9\xb3\xa4',
'CjkUnifiedIdeograph9ce7': '\xe9\xb3\xa7',
'CjkUnifiedIdeograph666a': '\xe6\x99\xaa',
'CjkUnifiedIdeograph9ce9': '\xe9\xb3\xa9',
'CjkUnifiedIdeograph9ce8': '\xe9\xb3\xa8',
'HangulLetterRieulphieuph': '\xe3\x84\xbf',
'YiRadicalGot': '\xea\x92\xb6',
'HangulSyllableDon': '\xeb\x8f\x88',
'MyanmarVowelSignKayahU': '\xe1\x81\xb3',
'HangulSyllableDdyec': '\xeb\x98\x8b',
'HangulSyllableDoh': '\xeb\x8f\x9f',
'HangulSyllableDok': '\xeb\x8f\x9c',
'ArabicLigatureJeemWithMeemWithHahFinalForm': '\xef\xb5\x98',
'LowerLeftQuadrantCircularArc': '\xe2\x97\x9f',
'HangulSyllableDoe': '\xeb\x90\x98',
'UnicodeCharacter0xf128': '\xef\x84\xa8',
'HangulSyllableDod': '\xeb\x8f\x8b',
'HangulSyllableJjyaebs': '\xec\xa9\x82',
'HangulSyllableDog': '\xeb\x8f\x85',
'UnicodeCharacter0xef7d': '\xee\xbd\xbd',
'UnicodeCharacter0xef7e': '\xee\xbd\xbe',
'UnicodeCharacter0xef7f': '\xee\xbd\xbf',
'KangxiRadicalTwoHands': '\xe2\xbc\xb6',
'CjkCompatibilityIdeographf9a2': '\xef\xa6\xa2',
'UnicodeCharacter0xef7a': '\xee\xbd\xba',
'UnicodeCharacter0xe23e': '\xee\x88\xbe',
'UnicodeCharacter0xef7c': '\xee\xbd\xbc',
'CjkCompatibilityIdeographf9a8': '\xef\xa6\xa8',
'CjkCompatibilityIdeographf9a9': '\xef\xa6\xa9',
'UnicodeCharacter0x843': '\xe0\xa1\x83',
'UnicodeCharacter0x842': '\xe0\xa1\x82',
'UnicodeCharacter0x841': '\xe0\xa1\x81',
'UnicodeCharacter0x840': '\xe0\xa1\x80',
'UnicodeCharacter0x847': '\xe0\xa1\x87',
'UnicodeCharacter0x846': '\xe0\xa1\x86',
'UnicodeCharacter0x845': '\xe0\xa1\x85',
'UnicodeCharacter0x844': '\xe0\xa1\x84',
'CjkUnifiedIdeograph7120': '\xe7\x84\xa0',
'CjkUnifiedIdeograph7121': '\xe7\x84\xa1',
'UnicodeCharacter0x849': '\xe0\xa1\x89',
'UnicodeCharacter0x848': '\xe0\xa1\x88',
'CjkUnifiedIdeograph7124': '\xe7\x84\xa4',
'CjkUnifiedIdeograph7125': '\xe7\x84\xa5',
'ArabicMarkNoonGhunna': '\xd9\x98',
'CjkUnifiedIdeograph7127': '\xe7\x84\xa7',
'HangulSyllableSwed': '\xec\x89\x97',
'CjkUnifiedIdeograph90ea': '\xe9\x83\xaa',
'HangulSyllableTyaeb': '\xed\x84\xa5',
'CjkUnifiedIdeograph90ef': '\xe9\x83\xaf',
'RadicalSymbolBottom': '\xe2\x8e\xb7',
'CjkUnifiedIdeograph8940': '\xe8\xa5\x80',
'CjkUnifiedIdeograph90ed': '\xe9\x83\xad',
'HangulSyllableTyaeg': '\xed\x84\x95',
'CjkUnifiedIdeograph90ee': '\xe9\x83\xae',
'YiSyllableSsix': '\xea\x8c\xab',
'UnicodeCharacter0x84c': '\xe0\xa1\x8c',
'UnicodeCharacter0x84b': '\xe0\xa1\x8b',
'UnicodeCharacter0x84a': '\xe0\xa1\x8a',
'UnicodeCharacter0x84f': '\xe0\xa1\x8f',
'UnicodeCharacter0x84e': '\xe0\xa1\x8e',
'UnicodeCharacter0x84d': '\xe0\xa1\x8d',
'CjkUnifiedIdeograph712a': '\xe7\x84\xaa',
'CjkUnifiedIdeograph712b': '\xe7\x84\xab',
'CjkUnifiedIdeograph712c': '\xe7\x84\xac',
'CjkUnifiedIdeograph712d': '\xe7\x84\xad',
'CjkUnifiedIdeograph712e': '\xe7\x84\xae',
'CjkUnifiedIdeograph712f': '\xe7\x84\xaf',
'UnicodeCharacter0xef74': '\xee\xbd\xb4',
'HangulSyllableTyaek': '\xed\x84\xac',
'UnicodeCharacter0xef76': '\xee\xbd\xb6',
'UnicodeCharacter0xef77': '\xee\xbd\xb7',
'UnicodeCharacter0xef70': '\xee\xbd\xb0',
'UnicodeCharacter0xef71': '\xee\xbd\xb1',
'UnicodeCharacter0xef72': '\xee\xbd\xb2',
'HangulSyllableTyaej': '\xed\x84\xaa',
'HangulSyllableNyegs': '\xeb\x85\x9f',
'UnicodeCharacter0x96': '\xc2\x96',
'UnicodeCharacter0xef78': '\xee\xbd\xb8',
'HangulSyllableSwen': '\xec\x89\x94',
'HangulSyllableSweo': '\xec\x88\xb4',
'ArabicLigatureKafWithLamIsolatedForm': '\xef\xb0\xbb',
'HangulSyllableRyolp': '\xeb\xa3\x9a',
'HangulSyllableSwej': '\xec\x89\xa6',
'MediumFlattenedRightParenthesisOrnament': '\xe2\x9d\xab',
'TibetanSubjoinedLetterKssa': '\xe0\xbe\xb9',
'CanadianSyllabicsWestcreeMwo': '\xe1\x92\xb3',
'PresentationFormForVerticalRightWhiteCornerBracket': '\xef\xb9\x84',
'HangulSyllableKyegg': '\xec\xbc\xba',
'BlackUpperLeftTriangle': '\xe2\x97\xa4',
'HangulSyllableKyegs': '\xec\xbc\xbb',
'HangulSyllableTyaep': '\xed\x84\xae',
'GreekSmallLetterTheta': '\xce\xb8',
'HangulSyllableSwep': '\xec\x89\xaa',
'HangulSyllableRulb': '\xeb\xa3\xb3',
'UnicodeCharacter0xea3c': '\xee\xa8\xbc',
'HangulSyllableBbyogs': '\xeb\xbe\xb3',
'CjkUnifiedIdeograph51e5': '\xe5\x87\xa5',
'UnicodeCharacter0xea3b': '\xee\xa8\xbb',
'HangulSyllableSwes': '\xec\x89\xa3',
'LongLeftwardsDoubleArrowFromBar': '\xe2\x9f\xbd',
'LatinSmallLetterNWithLongRightLeg': '\xc6\x9e',
'CjkUnifiedIdeograph901a': '\xe9\x80\x9a',
'CjkUnifiedIdeograph8574': '\xe8\x95\xb4',
'HangulSyllableRyuj': '\xeb\xa5\xae',
'CjkUnifiedIdeograph51e9': '\xe5\x87\xa9',
'HangulSyllableJjweolb': '\xec\xad\xaf',
'UnicodeCharacter0xe37d': '\xee\x8d\xbd',
'CjkUnifiedIdeograph77c1': '\xe7\x9f\x81',
'UnicodeCharacter0xdd2f': '\xed\xb4\xaf',
'PhagspaLetterNna': '\xea\xa1\xac',
'MyanmarLetterVocalicLl': '\xe1\x81\x95',
'UnicodeCharacter0xeb80': '\xee\xae\x80',
'GujaratiDigitOne': '\xe0\xab\xa7',
'LaoLetterHoSung': '\xe0\xba\xab',
'UnicodeCharacter0xdd2d': '\xed\xb4\xad',
'UnicodeCharacter0xdcfd': '\xed\xb3\xbd',
'RunicLetterLongbranchmadrM': '\xe1\x9b\x98',
'SyriacRukkakha': '\xdd\x82',
'BraillePatternDots1578': '\xe2\xa3\x91',
'CjkUnifiedIdeograph86b1': '\xe8\x9a\xb1',
'EthiopicSyllableKxi': '\xe1\x8a\xba',
'CjkUnifiedIdeograph86b7': '\xe8\x9a\xb7',
'CjkUnifiedIdeograph86b4': '\xe8\x9a\xb4',
'CjkUnifiedIdeograph86b5': '\xe8\x9a\xb5',
'EthiopicSyllableKxe': '\xe1\x8a\xbd',
'CjkUnifiedIdeograph9ed4': '\xe9\xbb\x94',
'CjkUnifiedIdeograph86b8': '\xe8\x9a\xb8',
'CjkUnifiedIdeograph86b9': '\xe8\x9a\xb9',
'EthiopicSyllableKxa': '\xe1\x8a\xb8',
'HangulSyllableTilb': '\xed\x8b\xbb',
'UnicodeCharacter0xdd2c': '\xed\xb4\xac',
'CjkUnifiedIdeograph3a1a': '\xe3\xa8\x9a',
'CjkUnifiedIdeograph3a1c': '\xe3\xa8\x9c',
'CjkUnifiedIdeograph3a1b': '\xe3\xa8\x9b',
'CjkUnifiedIdeograph3a1e': '\xe3\xa8\x9e',
'CjkUnifiedIdeograph3a1d': '\xe3\xa8\x9d',
'CjkUnifiedIdeograph3a1f': '\xe3\xa8\x9f',
'EthiopicSyllableKxu': '\xe1\x8a\xb9',
'ArabicLetterDdalIsolatedForm': '\xef\xae\x88',
'VulgarFractionTwoThirds': '\xe2\x85\x94',
'OcrDash': '\xe2\x91\x88',
'HangulSyllableDyess': '\xeb\x8e\xbc',
'ArabicLigatureQafWithYehFinalForm': '\xef\xb1\xbf',
'YiSyllableHliep': '\xea\x86\x9d',
'MalayalamLetterDda': '\xe0\xb4\xa1',
'UnicodeCharacter0xe0ff': '\xee\x83\xbf',
'YiSyllableHliex': '\xea\x86\x9b',
'HebrewPointHiriq': '\xd6\xb4',
'HangulSyllableBbyuls': '\xec\x81\x88',
'HangulSyllableNubs': '\xeb\x88\x96',
'HangulSyllableJjweols': '\xec\xad\xb0',
'CjkUnifiedIdeograph3a11': '\xe3\xa8\x91',
'CjkUnifiedIdeograph3a10': '\xe3\xa8\x90',
'CjkUnifiedIdeograph3a13': '\xe3\xa8\x93',
'CjkUnifiedIdeograph3a12': '\xe3\xa8\x92',
'CjkUnifiedIdeograph3a15': '\xe3\xa8\x95',
'CjkUnifiedIdeograph3a14': '\xe3\xa8\x94',
'CjkUnifiedIdeograph3a17': '\xe3\xa8\x97',
'CjkUnifiedIdeograph3a16': '\xe3\xa8\x96',
'CjkUnifiedIdeograph3a19': '\xe3\xa8\x99',
'CjkUnifiedIdeograph3a18': '\xe3\xa8\x98',
'HangulSyllableJjweolp': '\xec\xad\xb2',
'HalfwidthHangulLetterSsangkiyeok': '\xef\xbe\xa2',
'YiSyllableSsip': '\xea\x8c\xad',
'RunicLetterGer': '\xe1\x9b\x84',
'CjkUnifiedIdeograph86bb': '\xe8\x9a\xbb',
'CjkUnifiedIdeograph86bc': '\xe8\x9a\xbc',
'CjkUnifiedIdeograph86ba': '\xe8\x9a\xba',
'CjkUnifiedIdeograph86bf': '\xe8\x9a\xbf',
'HangulSyllableCanh': '\xec\xb0\xae',
'CjkUnifiedIdeograph86bd': '\xe8\x9a\xbd',
'CjkUnifiedIdeograph86be': '\xe8\x9a\xbe',
'KhmerSymbolLekAttakPii': '\xe1\x9f\xb2',
'BalineseDigitEight': '\xe1\xad\x98',
'HebrewAccentQadma': '\xd6\xa8',
'EthiopicSyllableQaa': '\xe1\x89\x83',
'OriyaLetterGha': '\xe0\xac\x98',
'CjkRadicalCsimplifiedSee': '\xe2\xbb\x85',
'GreekSmallLetterEpsilonWithOxia': '\xe1\xbd\xb3',
'QuadrantUpperLeftAndUpperRightAndLowerRight': '\xe2\x96\x9c',
'HangulSyllableKogs': '\xec\xbd\x97',
'UnicodeCharacter0xe888': '\xee\xa2\x88',
'TildeWithRingAbove': '\xe2\xb8\x9b',
'MyanmarVowelSignTallAa': '\xe1\x80\xab',
'HangulLetterYuye': '\xe3\x86\x8b',
'CjkUnifiedIdeograph89d3': '\xe8\xa7\x93',
'CjkUnifiedIdeograph89d2': '\xe8\xa7\x92',
'CjkUnifiedIdeograph89d1': '\xe8\xa7\x91',
'CjkUnifiedIdeograph89d0': '\xe8\xa7\x90',
'CjkUnifiedIdeograph73d8': '\xe7\x8f\x98',
'CjkUnifiedIdeograph73d9': '\xe7\x8f\x99',
'CjkUnifiedIdeograph89d5': '\xe8\xa7\x95',
'CjkUnifiedIdeograph89d4': '\xe8\xa7\x94',
'CjkUnifiedIdeograph73d4': '\xe7\x8f\x94',
'CjkUnifiedIdeograph73d5': '\xe7\x8f\x95',
'CjkUnifiedIdeograph73d6': '\xe7\x8f\x96',
'CjkUnifiedIdeograph73d7': '\xe7\x8f\x97',
'CjkUnifiedIdeograph73d0': '\xe7\x8f\x90',
'CjkUnifiedIdeograph73d1': '\xe7\x8f\x91',
'CjkUnifiedIdeograph73d2': '\xe7\x8f\x92',
'CjkUnifiedIdeograph73d3': '\xe7\x8f\x93',
'CjkUnifiedIdeograph46bf': '\xe4\x9a\xbf',
'CjkUnifiedIdeograph46bd': '\xe4\x9a\xbd',
'CjkUnifiedIdeograph46be': '\xe4\x9a\xbe',
'CjkUnifiedIdeograph46bb': '\xe4\x9a\xbb',
'CjkUnifiedIdeograph46bc': '\xe4\x9a\xbc',
'CjkUnifiedIdeograph46ba': '\xe4\x9a\xba',
'UnicodeCharacter0xe5fe': '\xee\x97\xbe',
'HangulSyllableGgoej': '\xea\xbe\x96',
'VaiSyllableFi': '\xea\x94\xb1',
'HangulSyllableGgoel': '\xea\xbe\x88',
'HangulChoseongSsangcieuc': '\xe1\x84\x8d',
'CjkUnifiedIdeograph89dc': '\xe8\xa7\x9c',
'CjkUnifiedIdeograph89db': '\xe8\xa7\x9b',
'RightSemidirectProduct': '\xe2\x8b\x8c',
'CjkUnifiedIdeograph89df': '\xe8\xa7\x9f',
'CjkUnifiedIdeograph89de': '\xe8\xa7\x9e',
'CjkUnifiedIdeograph836f': '\xe8\x8d\xaf',
'HexagramForEnthusiasm': '\xe4\xb7\x8f',
'CjkUnifiedIdeograph73de': '\xe7\x8f\x9e',
'CjkUnifiedIdeograph73df': '\xe7\x8f\x9f',
'HangulSyllableRyub': '\xeb\xa5\xa9',
'HangulSyllableTas': '\xed\x83\x93',
'CjkUnifiedIdeograph73da': '\xe7\x8f\x9a',
'CjkUnifiedIdeograph73db': '\xe7\x8f\x9b',
'CjkUnifiedIdeograph73dc': '\xe7\x8f\x9c',
'HangulSyllableCweok': '\xec\xb7\x88',
'HangulSyllableTam': '\xed\x83\x90',
'CjkUnifiedIdeograph836d': '\xe8\x8d\xad',
'HangulSyllableTak': '\xed\x83\x98',
'HangulSyllableTaj': '\xed\x83\x96',
'UnicodeCharacter0xf882': '\xef\xa2\x82',
'HangulSyllableTah': '\xed\x83\x9b',
'HangulSyllableTag': '\xed\x83\x81',
'ArabicLetterYeh': '\xd9\x8a',
'HangulSyllableTae': '\xed\x83\x9c',
'HangulSyllableTad': '\xed\x83\x87',
'CjkUnifiedIdeograph66c3': '\xe6\x9b\x83',
'HangulSyllableGgoeb': '\xea\xbe\x91',
'CjkUnifiedIdeograph836b': '\xe8\x8d\xab',
'YiSyllableHlox': '\xea\x86\xa5',
'HangulSyllableGgoec': '\xea\xbe\x97',
'CjkUnifiedIdeograph836a': '\xe8\x8d\xaa',
'HangulSyllableGgyels': '\xea\xbc\x9c',
'YiSyllableHlop': '\xea\x86\xa7',
'UnicodeCharacter0xf898': '\xef\xa2\x98',
'HangulSyllableGgyelp': '\xea\xbc\x9e',
'RunicLetterShorttwigossO': '\xe1\x9a\xad',
'MyanmarDigitThree': '\xe1\x81\x83',
'HangulSyllableGgyelt': '\xea\xbc\x9d',
'CjkUnifiedIdeograph46b6': '\xe4\x9a\xb6',
'CjkUnifiedIdeograph46b7': '\xe4\x9a\xb7',
'CjkUnifiedIdeograph46b4': '\xe4\x9a\xb4',
'CjkUnifiedIdeograph46b5': '\xe4\x9a\xb5',
'CjkUnifiedIdeograph46b2': '\xe4\x9a\xb2',
'CjkUnifiedIdeograph46b3': '\xe4\x9a\xb3',
'CjkUnifiedIdeograph46b0': '\xe4\x9a\xb0',
'CjkUnifiedIdeograph46b1': '\xe4\x9a\xb1',
'GreekSmallLetterPsi': '\xcf\x88',
'HangulSyllableGgyelb': '\xea\xbc\x9b',
'HangulSyllableGgoeg': '\xea\xbe\x81',
'HangulSyllableGgyelg': '\xea\xbc\x99',
'CjkUnifiedIdeograph46b8': '\xe4\x9a\xb8',
'CjkUnifiedIdeograph46b9': '\xe4\x9a\xb9',
'ThaiCharacterThoThahan': '\xe0\xb8\x97',
'HangulSyllableRwils': '\xeb\xa5\x88',
'HangulSyllableRyud': '\xeb\xa5\x9f',
'HangulSyllableRwilt': '\xeb\xa5\x89',
'EthiopicSyllablePharyngealEe': '\xe1\x8b\x94',
'<===': '\xe2\x87\x9a',
'GujaratiDigitSix': '\xe0\xab\xac',
'CjkUnifiedIdeograph90e2': '\xe9\x83\xa2',
'YiSyllableUop': '\xea\x80\x8e',
'YiSyllableMguox': '\xea\x88\xb6',
'CjkUnifiedIdeograph6fbc': '\xe6\xbe\xbc',
'HangulSyllableByass': '\xeb\xb1\xa0',
'CjkUnifiedIdeograph90e0': '\xe9\x83\xa0',
'KangxiRadicalTen': '\xe2\xbc\x97',
'CjkUnifiedIdeograph90e1': '\xe9\x83\xa1',
'HangulSyllableNom': '\xeb\x86\x88',
'CjkUnifiedIdeograph5f8c': '\xe5\xbe\x8c',
'CjkUnifiedIdeograph5f8b': '\xe5\xbe\x8b',
'CjkUnifiedIdeograph5f8a': '\xe5\xbe\x8a',
'CjkUnifiedIdeograph5f8f': '\xe5\xbe\x8f',
'CjkUnifiedIdeograph5f8e': '\xe5\xbe\x8e',
'CjkUnifiedIdeograph5f8d': '\xe5\xbe\x8d',
'HangulSyllableTwess': '\xed\x89\xb8',
'ArabicLigatureNoonWithJeemWithYehFinalForm': '\xef\xb7\x87',
'HangulSyllableMyugg': '\xeb\xae\xa6',
'LatinSmallLetterOWithHornAndHookAbove': '\xe1\xbb\x9f',
'TaiLeLetterTha': '\xe1\xa5\x97',
'CjkUnifiedIdeograph77ba': '\xe7\x9e\xba',
'LatinSmallLetterZWithCaron': '\xc5\xbe',
'HangulSyllableRwilm': '\xeb\xa5\x86',
'UnicodeCharacter0xe5fb': '\xee\x97\xbb',
'MyanmarVowelSignWesternPwoKarenUe': '\xe1\x81\xa8',
'HangulSyllableHweolp': '\xed\x9b\xae',
'HangulSyllableDeuss': '\xeb\x93\xb0',
'HangulSyllableHweols': '\xed\x9b\xac',
'HangulSyllableHweolt': '\xed\x9b\xad',
'HangulSyllableCyaed': '\xec\xb2\x83',
'HangulSyllableHweolh': '\xed\x9b\xaf',
'HangulSyllableHweolm': '\xed\x9b\xaa',
'CjkUnifiedIdeograph7536': '\xe7\x94\xb6',
'EthiopicNumberTen': '\xe1\x8d\xb2',
'HangulSyllableHweolb': '\xed\x9b\xab',
'HebrewLetterHeWithMapiq': '\xef\xac\xb4',
'CjkUnifiedIdeograph7535': '\xe7\x94\xb5',
'HangulSyllableNeuss': '\xeb\x8a\xa4',
'HangulSyllableHweolg': '\xed\x9b\xa9',
'CjkUnifiedIdeograph8579': '\xe8\x95\xb9',
'UnicodeCharacter0xf015': '\xef\x80\x95',
'HangulSyllablePweogs': '\xed\x92\x97',
'HangulSyllableDag': '\xeb\x8b\xa5',
'SmallRomanNumeralOneHundred': '\xe2\x85\xbd',
'CjkUnifiedIdeograph5f89': '\xe5\xbe\x89',
'CjkUnifiedIdeograph5f88': '\xe5\xbe\x88',
'HangulSyllablePweogg': '\xed\x92\x96',
'CjkUnifiedIdeograph5f83': '\xe5\xbe\x83',
'CjkUnifiedIdeograph5f82': '\xe5\xbe\x82',
'CjkUnifiedIdeograph5f81': '\xe5\xbe\x81',
'CjkUnifiedIdeograph5f80': '\xe5\xbe\x80',
'CjkUnifiedIdeograph5f87': '\xe5\xbe\x87',
'CjkUnifiedIdeograph5f86': '\xe5\xbe\x86',
'CjkUnifiedIdeograph5f85': '\xe5\xbe\x85',
'CjkUnifiedIdeograph5f84': '\xe5\xbe\x84',
'CjkUnifiedIdeograph5c55': '\xe5\xb1\x95',
'CjkUnifiedIdeograph5c54': '\xe5\xb1\x94',
'CjkUnifiedIdeograph5c57': '\xe5\xb1\x97',
'CjkUnifiedIdeograph5c56': '\xe5\xb1\x96',
'CjkUnifiedIdeograph5c51': '\xe5\xb1\x91',
'CjkUnifiedIdeograph5c50': '\xe5\xb1\x90',
'CjkUnifiedIdeograph5c53': '\xe5\xb1\x93',
'CjkUnifiedIdeograph5c52': '\xe5\xb1\x92',
'CjkUnifiedIdeograph9d7d': '\xe9\xb5\xbd',
'CjkUnifiedIdeograph9d7e': '\xe9\xb5\xbe',
'CjkUnifiedIdeograph9d7f': '\xe9\xb5\xbf',
'CjkUnifiedIdeograph5c59': '\xe5\xb1\x99',
'CjkUnifiedIdeograph5c58': '\xe5\xb1\x98',
'CjkUnifiedIdeograph9d7b': '\xe9\xb5\xbb',
'CjkUnifiedIdeograph9d7c': '\xe9\xb5\xbc',
'HangulSyllableTaeh': '\xed\x83\xb7',
'HangulLetterPieupsios': '\xe3\x85\x84',
'CombiningDiaeresis': '\xcc\x88',
'AnticlockwiseOpenCircleArrow': '\xe2\x86\xba',
'HangulSyllableDae': '\xeb\x8c\x80',
'HangulSyllableKyu': '\xed\x81\x90',
'LatinSmallLetterDWithStroke': '\xc4\x91',
'UnicodeCharacter0xf5bf': '\xef\x96\xbf',
'ArabicLigatureYehWithMeemMedialForm': '\xef\xb3\xb0',
'CjkUnifiedIdeograph880f': '\xe8\xa0\x8f',
'UnicodeCharacter0xf726': '\xef\x9c\xa6',
'CjkUnifiedIdeograph880d': '\xe8\xa0\x8d',
'CjkUnifiedIdeograph880e': '\xe8\xa0\x8e',
'CjkUnifiedIdeograph880b': '\xe8\xa0\x8b',
'CjkUnifiedIdeograph880c': '\xe8\xa0\x8c',
'UnicodeCharacter0xf721': '\xef\x9c\xa1',
'CjkUnifiedIdeograph880a': '\xe8\xa0\x8a',
'YiSyllableMguo': '\xea\x88\xb7',
'WhiteDraughtsKing': '\xe2\x9b\x81',
'CjkUnifiedIdeograph98b2': '\xe9\xa2\xb2',
'UnicodeCharacter0xea36': '\xee\xa8\xb6',
'HangulSyllableGga': '\xea\xb9\x8c',
'UnicodeCharacter0xea35': '\xee\xa8\xb5',
'UnicodeCharacter0xea34': '\xee\xa8\xb4',
'HangulSyllableMec': '\xeb\xa9\xab',
'CjkUnifiedIdeograph5c5e': '\xe5\xb1\x9e',
'CjkUnifiedIdeograph5c5d': '\xe5\xb1\x9d',
'CjkUnifiedIdeograph5c5f': '\xe5\xb1\x9f',
'CjkUnifiedIdeograph5c5a': '\xe5\xb1\x9a',
'CjkUnifiedIdeograph9d79': '\xe9\xb5\xb9',
'CjkUnifiedIdeograph5c5c': '\xe5\xb1\x9c',
'CjkUnifiedIdeograph5c5b': '\xe5\xb1\x9b',
'CjkUnifiedIdeograph9d74': '\xe9\xb5\xb4',
'CjkUnifiedIdeograph9d75': '\xe9\xb5\xb5',
'CjkUnifiedIdeograph9d76': '\xe9\xb5\xb6',
'CjkUnifiedIdeograph9d77': '\xe9\xb5\xb7',
'CjkUnifiedIdeograph9d70': '\xe9\xb5\xb0',
'CjkUnifiedIdeograph9d71': '\xe9\xb5\xb1',
'CjkUnifiedIdeograph9d72': '\xe9\xb5\xb2',
'CjkUnifiedIdeograph8a81': '\xe8\xaa\x81',
'CjkUnifiedIdeograph8806': '\xe8\xa0\x86',
'CjkUnifiedIdeograph8807': '\xe8\xa0\x87',
'CjkUnifiedIdeograph8804': '\xe8\xa0\x84',
'CjkUnifiedIdeograph8805': '\xe8\xa0\x85',
'CjkUnifiedIdeograph8802': '\xe8\xa0\x82',
'TamilLetterNnna': '\xe0\xae\xa9',
'CjkUnifiedIdeograph8800': '\xe8\xa0\x80',
'CjkUnifiedIdeograph8801': '\xe8\xa0\x81',
'HalfwidthKatakanaLetterMa': '\xef\xbe\x8f',
'HangulSyllableReu': '\xeb\xa5\xb4',
'CjkUnifiedIdeograph8808': '\xe8\xa0\x88',
'CjkUnifiedIdeograph8809': '\xe8\xa0\x89',
'CanadianSyllabicsNaskapiTwaa': '\xe1\x91\xa5',
'LimbuLetterYa': '\xe1\xa4\x95',
'HangulSyllableDab': '\xeb\x8b\xb5',
'UnicodeCharacter0xeb8f': '\xee\xae\x8f',
'UnicodeCharacter0xf23a': '\xef\x88\xba',
'TibetanSubjoinedLetterNna': '\xe0\xbe\x9e',
'LatinSmallLetterUWithDoubleAcute': '\xc5\xb1',
'WhiteSquareWithRightwardsTick': '\xe2\x9f\xa5',
'UnicodeCharacter0x17fc': '\xe1\x9f\xbc',
'CjkUnifiedIdeograph400b': '\xe4\x80\x8b',
'CjkUnifiedIdeograph400c': '\xe4\x80\x8c',
'CjkUnifiedIdeograph400a': '\xe4\x80\x8a',
'CjkUnifiedIdeograph400f': '\xe4\x80\x8f',
'HangulSyllableMet': '\xeb\xa9\xad',
'CjkUnifiedIdeograph400d': '\xe4\x80\x8d',
'CjkUnifiedIdeograph400e': '\xe4\x80\x8e',
'YiSyllableXot': '\xea\x91\xa7',
'Complement': '\xe2\x88\x81',
'HangulSyllableKyobs': '\xec\xbf\x96',
'CircledRingOperator': '\xe2\x8a\x9a',
'HangulSyllableMep': '\xeb\xa9\xae',
'HangulSyllableCalp': '\xec\xb0\xb6',
'HangulSyllableGgwanh': '\xea\xbd\x8e',
'HangulSyllableDdweobs': '\xeb\x9b\x8a',
'ArabicLetterMeemInitialForm': '\xef\xbb\xa3',
'CjkUnifiedIdeograph4008': '\xe4\x80\x88',
'CjkUnifiedIdeograph4009': '\xe4\x80\x89',
'CjkUnifiedIdeograph4002': '\xe4\x80\x82',
'CjkUnifiedIdeograph4003': '\xe4\x80\x83',
'CjkUnifiedIdeograph4000': '\xe4\x80\x80',
'CjkUnifiedIdeograph4001': '\xe4\x80\x81',
'CjkUnifiedIdeograph4006': '\xe4\x80\x86',
'CjkUnifiedIdeograph4007': '\xe4\x80\x87',
'CjkUnifiedIdeograph4004': '\xe4\x80\x84',
'CjkUnifiedIdeograph4005': '\xe4\x80\x85',
'HangulSyllableBwigg': '\xeb\xb7\x96',
'GreekSmallLetterOmicronWithTonos': '\xcf\x8c',
'CjkRadicalHeartTwo': '\xe2\xba\x97',
'HangulLetterRieultikeut': '\xe3\x85\xaa',
'KhmerDigitEight': '\xe1\x9f\xa8',
'UnicodeCharacter0xd9ea': '\xed\xa7\xaa',
'LatinSmallLetterGlottalStop': '\xc9\x82',
'PresentationFormForVerticalRightTortoiseShellBracket': '\xef\xb8\xba',
'MyanmarShanDigitZero': '\xe1\x82\x90',
'UnicodeCharacter0xd872': '\xed\xa1\xb2',
'ArabicLetterGafWithThreeDotsAbove': '\xda\xb4',
'UnicodeCharacter0xe406': '\xee\x90\x86',
'CjkUnifiedIdeograph4ed9': '\xe4\xbb\x99',
'CjkUnifiedIdeograph4ed8': '\xe4\xbb\x98',
'UnicodeCharacter0xe403': '\xee\x90\x83',
'UnicodeCharacter0xd9ed': '\xed\xa7\xad',
'UnicodeCharacter0xe401': '\xee\x90\x81',
'HangulSyllableGgigg': '\xeb\x81\xbe',
'CjkUnifiedIdeograph4ed3': '\xe4\xbb\x93',
'CjkUnifiedIdeograph4ed2': '\xe4\xbb\x92',
'CjkUnifiedIdeograph4ed1': '\xe4\xbb\x91',
'CjkUnifiedIdeograph4ed0': '\xe4\xbb\x90',
'CjkUnifiedIdeograph4ed7': '\xe4\xbb\x97',
'HangulLetterRieulpieup': '\xe3\x84\xbc',
'CjkUnifiedIdeograph4ed5': '\xe4\xbb\x95',
'CjkUnifiedIdeograph4ed4': '\xe4\xbb\x94',
'UnicodeCharacter0xf657': '\xef\x99\x97',
'HangulSyllableGgigs': '\xeb\x81\xbf',
'SyriacLetterSemkath': '\xdc\xa3',
'CjkUnifiedIdeograph944a': '\xe9\x91\x8a',
'HangulSyllableRweolb': '\xeb\xa4\x8f',
'NotASupersetOf': '\xe2\x8a\x85',
'CjkUnifiedIdeograph944b': '\xe9\x91\x8b',
'CjkUnifiedIdeograph944e': '\xe9\x91\x8e',
'CjkUnifiedIdeograph944d': '\xe9\x91\x8d',
'CjkUnifiedIdeograph944f': '\xe9\x91\x8f',
'CjkUnifiedIdeograph59af': '\xe5\xa6\xaf',
'CjkUnifiedIdeograph59ae': '\xe5\xa6\xae',
'KannadaDigitSeven': '\xe0\xb3\xad',
'CjkUnifiedIdeograph59ac': '\xe5\xa6\xac',
'CjkUnifiedIdeograph59ab': '\xe5\xa6\xab',
'CjkUnifiedIdeograph59aa': '\xe5\xa6\xaa',
'CjkUnifiedIdeograph75f1': '\xe7\x97\xb1',
'CjkUnifiedIdeograph575a': '\xe5\x9d\x9a',
'CjkUnifiedIdeograph575c': '\xe5\x9d\x9c',
'CjkUnifiedIdeograph575b': '\xe5\x9d\x9b',
'CjkUnifiedIdeograph575e': '\xe5\x9d\x9e',
'CjkUnifiedIdeograph575d': '\xe5\x9d\x9d',
'CjkUnifiedIdeograph575f': '\xe5\x9d\x9f',
'CjkUnifiedIdeograph8114': '\xe8\x84\x94',
'CjkUnifiedIdeograph8115': '\xe8\x84\x95',
'CjkUnifiedIdeograph8116': '\xe8\x84\x96',
'CjkUnifiedIdeograph75f3': '\xe7\x97\xb3',
'CjkUnifiedIdeograph8110': '\xe8\x84\x90',
'CjkUnifiedIdeograph8111': '\xe8\x84\x91',
'CjkUnifiedIdeograph8112': '\xe8\x84\x92',
'CjkUnifiedIdeograph8113': '\xe8\x84\x93',
'UnicodeCharacter0x7bb': '\xde\xbb',
'UnicodeCharacter0x7bc': '\xde\xbc',
'UnicodeCharacter0x7ba': '\xde\xba',
'UnicodeCharacter0x7bf': '\xde\xbf',
'CjkUnifiedIdeograph8119': '\xe8\x84\x99',
'UnicodeCharacter0x7bd': '\xde\xbd',
'UnicodeCharacter0x7be': '\xde\xbe',
'CjkUnifiedIdeograph9449': '\xe9\x91\x89',
'CjkUnifiedIdeograph9448': '\xe9\x91\x88',
'FullwidthGraveAccent': '\xef\xbd\x80',
'HangulSyllableTeon': '\xed\x84\xb4',
'CjkUnifiedIdeograph8361': '\xe8\x8d\xa1',
'HangulSyllableTeok': '\xed\x85\x88',
'HangulSyllableTeoj': '\xed\x85\x86',
'CjkUnifiedIdeograph9441': '\xe9\x91\x81',
'CjkUnifiedIdeograph9440': '\xe9\x91\x80',
'CjkUnifiedIdeograph9443': '\xe9\x91\x83',
'CjkRadicalMortar': '\xe2\xba\xbd',
'CjkUnifiedIdeograph9445': '\xe9\x91\x85',
'CjkUnifiedIdeograph9444': '\xe9\x91\x84',
'CjkUnifiedIdeograph9447': '\xe9\x91\x87',
'CjkUnifiedIdeograph82ab': '\xe8\x8a\xab',
'CjkUnifiedIdeograph75f8': '\xe7\x97\xb8',
'UnicodeCharacter0xe40e': '\xee\x90\x8e',
'UnicodeCharacter0xe40d': '\xee\x90\x8d',
'UnicodeCharacter0xe40c': '\xee\x90\x8c',
'CjkUnifiedIdeograph82aa': '\xe8\x8a\xaa',
'UnicodeCharacter0xe40a': '\xee\x90\x8a',
'CjkUnifiedIdeograph75f9': '\xe7\x97\xb9',
'CjkUnifiedIdeograph4edc': '\xe4\xbb\x9c',
'CjkUnifiedIdeograph4edb': '\xe4\xbb\x9b',
'CjkUnifiedIdeograph4eda': '\xe4\xbb\x9a',
'CjkUnifiedIdeograph4edf': '\xe4\xbb\x9f',
'CjkUnifiedIdeograph4ede': '\xe4\xbb\x9e',
'CjkUnifiedIdeograph4edd': '\xe4\xbb\x9d',
'CjkUnifiedIdeograph811d': '\xe8\x84\x9d',
'CjkUnifiedIdeograph811e': '\xe8\x84\x9e',
'UnicodeCharacter0x7b8': '\xde\xb8',
'UnicodeCharacter0x7b9': '\xde\xb9',
'CjkUnifiedIdeograph811a': '\xe8\x84\x9a',
'CjkUnifiedIdeograph811b': '\xe8\x84\x9b',
'CjkUnifiedIdeograph811c': '\xe8\x84\x9c',
'UnicodeCharacter0x7b2': '\xde\xb2',
'UnicodeCharacter0x7b3': '\xde\xb3',
'UnicodeCharacter0x7b6': '\xde\xb6',
'UnicodeCharacter0x7b7': '\xde\xb7',
'UnicodeCharacter0x7b4': '\xde\xb4',
'UnicodeCharacter0x7b5': '\xde\xb5',
'CjkUnifiedIdeograph5759': '\xe5\x9d\x99',
'CjkUnifiedIdeograph5758': '\xe5\x9d\x98',
'CjkUnifiedIdeograph59a5': '\xe5\xa6\xa5',
'CjkUnifiedIdeograph59a4': '\xe5\xa6\xa4',
'LatinCapitalLetterYWithDotBelow': '\xe1\xbb\xb4',
'CjkUnifiedIdeograph59a2': '\xe5\xa6\xa2',
'CanadianSyllabicsTha': '\xe1\x95\xa6',
'CjkUnifiedIdeograph59a0': '\xe5\xa6\xa0',
'CjkUnifiedIdeograph5751': '\xe5\x9d\x91',
'CjkUnifiedIdeograph5750': '\xe5\x9d\x90',
'CjkUnifiedIdeograph5753': '\xe5\x9d\x93',
'CjkUnifiedIdeograph5752': '\xe5\x9d\x92',
'CjkUnifiedIdeograph5755': '\xe5\x9d\x95',
'CjkUnifiedIdeograph5754': '\xe5\x9d\x94',
'CjkUnifiedIdeograph5757': '\xe5\x9d\x97',
'CjkUnifiedIdeograph5756': '\xe5\x9d\x96',
'CjkUnifiedIdeograph970f': '\xe9\x9c\x8f',
'YiRadicalTu': '\xea\x92\xa8',
'UnicodeCharacter0xe543': '\xee\x95\x83',
'VaiSyllableGbee': '\xea\x94\x8b',
'SquareHooru': '\xe3\x8d\x81',
'CjkUnifiedIdeograph9993': '\xe9\xa6\x93',
'CjkUnifiedIdeograph9992': '\xe9\xa6\x92',
'CjkUnifiedIdeograph9991': '\xe9\xa6\x91',
'CjkUnifiedIdeograph7dd0': '\xe7\xb7\x90',
'CjkUnifiedIdeograph9997': '\xe9\xa6\x97',
'CjkUnifiedIdeograph96bd': '\xe9\x9a\xbd',
'CjkUnifiedIdeograph9995': '\xe9\xa6\x95',
'CjkUnifiedIdeograph9994': '\xe9\xa6\x94',
'UnicodeCharacter0xaa95': '\xea\xaa\x95',
'CjkUnifiedIdeograph970b': '\xe9\x9c\x8b',
'CjkUnifiedIdeograph9999': '\xe9\xa6\x99',
'CjkUnifiedIdeograph9998': '\xe9\xa6\x98',
'UnicodeCharacter0xaa91': '\xea\xaa\x91',
'UnicodeCharacter0xaa90': '\xea\xaa\x90',
'UnicodeCharacter0xaa93': '\xea\xaa\x93',
'CjkUnifiedIdeograph970c': '\xe9\x9c\x8c',
'UnicodeCharacter0xd821': '\xed\xa0\xa1',
'UnicodeCharacter0xe544': '\xee\x95\x84',
'GreekSmallLetterOmegaWithPerispomeni': '\xe1\xbf\xb6',
'CjkUnifiedIdeograph970a': '\xe9\x9c\x8a',
'CjkUnifiedIdeograph9a02': '\xe9\xa8\x82',
'OriyaSignAnusvara': '\xe0\xac\x82',
'CjkUnifiedIdeograph6d3d': '\xe6\xb4\xbd',
'CjkUnifiedIdeograph7dd9': '\xe7\xb7\x99',
'CjkUnifiedIdeograph999c': '\xe9\xa6\x9c',
'CjkUnifiedIdeograph999b': '\xe9\xa6\x9b',
'CjkUnifiedIdeograph999a': '\xe9\xa6\x9a',
'CjkUnifiedIdeograph7dd8': '\xe7\xb7\x98',
'CjkUnifiedIdeograph999f': '\xe9\xa6\x9f',
'CjkUnifiedIdeograph999e': '\xe9\xa6\x9e',
'CjkUnifiedIdeograph999d': '\xe9\xa6\x9d',
'ElementOfWithLongHorizontalStroke': '\xe2\x8b\xb2',
'UnicodeCharacter0xaa9d': '\xea\xaa\x9d',
'HangulSyllableBwe': '\xeb\xb6\xb8',
'UnicodeCharacter0xaa9f': '\xea\xaa\x9f',
'UnicodeCharacter0xaa9a': '\xea\xaa\x9a',
'VerticalBarBesideRightTriangle': '\xe2\xa7\x90',
'UnicodeCharacter0xaa9c': '\xea\xaa\x9c',
'UnicodeCharacter0xaa9b': '\xea\xaa\x9b',
'YiSyllableNju': '\xea\x91\x80',
'HangulSyllableDdoegg': '\xeb\x99\xa6',
'UnicodeCharacter0xf537': '\xef\x94\xb7',
'HangulChoseongIeungpieup': '\xe1\x85\x84',
'HangulSyllableKalh': '\xec\xba\x83',
'CjkUnifiedIdeograph5ce5': '\xe5\xb3\xa5',
'TamilVowelSignIi': '\xe0\xaf\x80',
'CjkUnifiedIdeograph5ce7': '\xe5\xb3\xa7',
'CjkUnifiedIdeograph736f': '\xe7\x8d\xaf',
'BoxDrawingsLightVerticalAndLeft': '\xe2\x94\xa4',
'leftrightarrow': '\xe2\x86\x94',
'CjkUnifiedIdeograph5ce1': '\xe5\xb3\xa1',
'HebrewAccentZinor': '\xd6\xae',
'VeryMuchLessthan': '\xe2\x8b\x98',
'CjkUnifiedIdeograph5ce0': '\xe5\xb3\xa0',
'CjkUnifiedIdeograph80c7': '\xe8\x83\x87',
'CjkUnifiedIdeograph80c6': '\xe8\x83\x86',
'CjkUnifiedIdeograph80c5': '\xe8\x83\x85',
'CjkUnifiedIdeograph80c4': '\xe8\x83\x84',
'CjkUnifiedIdeograph80c3': '\xe8\x83\x83',
'CjkUnifiedIdeograph80c2': '\xe8\x83\x82',
'CjkUnifiedIdeograph80c1': '\xe8\x83\x81',
'CjkUnifiedIdeograph80c0': '\xe8\x83\x80',
'KatakanaLetterSmallU': '\xe3\x82\xa5',
'CjkUnifiedIdeograph5ce2': '\xe5\xb3\xa2',
'CjkUnifiedIdeograph80c9': '\xe8\x83\x89',
'CjkUnifiedIdeograph80c8': '\xe8\x83\x88',
'KatakanaLetterSmallO': '\xe3\x82\xa9',
'KatakanaLetterSmallI': '\xe3\x82\xa3',
'KatakanaLetterSmallE': '\xe3\x82\xa7',
'KatakanaLetterSmallA': '\xe3\x82\xa1',
'UnicodeCharacter0xd9e7': '\xed\xa7\xa7',
'CircledPlus': '\xe2\x8a\x95',
'HangulSyllableDes': '\xeb\x8e\x83',
'UnicodeCharacter0xdb7b': '\xed\xad\xbb',
'UnicodeCharacter0xdb7a': '\xed\xad\xba',
'HangulSyllableDep': '\xeb\x8e\x8a',
'CjkUnifiedIdeograph736d': '\xe7\x8d\xad',
'TopRightHalfBracket': '\xe2\xb8\xa3',
'HangulSyllableDeu': '\xeb\x93\x9c',
'HangulSyllableDet': '\xeb\x8e\x89',
'UnicodeCharacter0xa4e8': '\xea\x93\xa8',
'UnicodeCharacter0xa4e9': '\xea\x93\xa9',
'UnicodeCharacter0xed70': '\xee\xb5\xb0',
'CjkUnifiedIdeograph6182': '\xe6\x86\x82',
'HangulSyllableDeo': '\xeb\x8d\x94',
'HangulSyllableDen': '\xeb\x8d\xb4',
'HangulSyllableDem': '\xeb\x8e\x80',
'HangulSyllableDel': '\xeb\x8d\xb8',
'UnicodeCharacter0xa4e0': '\xea\x93\xa0',
'CjkUnifiedIdeograph8e9a': '\xe8\xba\x9a',
'CjkUnifiedIdeograph8e9b': '\xe8\xba\x9b',
'CjkUnifiedIdeograph8e9c': '\xe8\xba\x9c',
'CjkUnifiedIdeograph8e9d': '\xe8\xba\x9d',
'CjkUnifiedIdeograph8e9e': '\xe8\xba\x9e',
'CjkUnifiedIdeograph8e9f': '\xe8\xba\x9f',
'UnicodeCharacter0xa4e7': '\xea\x93\xa7',
'MongolianLetterTodoUe': '\xe1\xa1\x89',
'HangulSyllableDass': '\xeb\x8b\xb8',
'UnicodeCharacter0xf65b': '\xef\x99\x9b',
'CjkUnifiedIdeograph80cf': '\xe8\x83\x8f',
'CjkUnifiedIdeograph80ce': '\xe8\x83\x8e',
'CjkUnifiedIdeograph80cd': '\xe8\x83\x8d',
'UpperLeftQuadrantCircularArc': '\xe2\x97\x9c',
'CjkUnifiedIdeograph80cb': '\xe8\x83\x8b',
'CanadianSyllabicsWestcreeWoo': '\xe1\x90\x95',
'CjkUnifiedIdeograph7b70': '\xe7\xad\xb0',
'CjkUnifiedIdeograph8e98': '\xe8\xba\x98',
'CjkUnifiedIdeograph8e99': '\xe8\xba\x99',
'UnicodeCharacter0xed7a': '\xee\xb5\xba',
'UnicodeCharacter0xed7f': '\xee\xb5\xbf',
'UnicodeCharacter0xed7d': '\xee\xb5\xbd',
'CjkUnifiedIdeograph9066': '\xe9\x81\xa6',
'CjkUnifiedIdeograph736b': '\xe7\x8d\xab',
'CjkUnifiedIdeograph8e91': '\xe8\xba\x91',
'CjkUnifiedIdeograph8e92': '\xe8\xba\x92',
'CjkUnifiedIdeograph8e93': '\xe8\xba\x93',
'CjkUnifiedIdeograph8e94': '\xe8\xba\x94',
'CjkUnifiedIdeograph8e95': '\xe8\xba\x95',
'CjkUnifiedIdeograph8e96': '\xe8\xba\x96',
'CjkUnifiedIdeograph8e97': '\xe8\xba\x97',
'UnicodeCharacter0xdb79': '\xed\xad\xb9',
'UnicodeCharacter0xdb78': '\xed\xad\xb8',
'HangulSyllableKalg': '\xec\xb9\xbd',
'UnicodeCharacter0xdb73': '\xed\xad\xb3',
'UnicodeCharacter0xdb72': '\xed\xad\xb2',
'UnicodeCharacter0xdb71': '\xed\xad\xb1',
'UnicodeCharacter0xdb70': '\xed\xad\xb0',
'UnicodeCharacter0xdb77': '\xed\xad\xb7',
'UnicodeCharacter0xdb76': '\xed\xad\xb6',
'UnicodeCharacter0xdb75': '\xed\xad\xb5',
'UnicodeCharacter0xdb74': '\xed\xad\xb4',
'CjkUnifiedIdeograph83c8': '\xe8\x8f\x88',
'CjkUnifiedIdeograph83c9': '\xe8\x8f\x89',
'HangulSyllableTwes': '\xed\x89\xb7',
'CjkUnifiedIdeograph92dd': '\xe9\x8b\x9d',
'HangulSyllableTwet': '\xed\x89\xbd',
'GlagoliticCapitalLetterFritu': '\xe2\xb0\x97',
'CjkUnifiedIdeograph83c0': '\xe8\x8f\x80',
'CjkUnifiedIdeograph83c1': '\xe8\x8f\x81',
'CjkUnifiedIdeograph83c2': '\xe8\x8f\x82',
'SmallContainsWithOverbar': '\xe2\x8b\xbe',
'CjkUnifiedIdeograph83c4': '\xe8\x8f\x84',
'RightwardsArrowThroughGreaterthan': '\xe2\xad\x83',
'CjkUnifiedIdeograph83c6': '\xe8\x8f\x86',
'CjkUnifiedIdeograph83c7': '\xe8\x8f\x87',
'HangulSyllableTwec': '\xed\x89\xbb',
'CjkUnifiedIdeograph8b69': '\xe8\xad\xa9',
'HangulSyllableTwed': '\xed\x89\xab',
'HangulSyllableTweg': '\xed\x89\xa5',
'HangulSyllableBbyogg': '\xeb\xbe\xb2',
'HangulSyllableTwek': '\xed\x89\xbc',
'HangulSyllableTwej': '\xed\x89\xba',
'HangulSyllableTwem': '\xed\x89\xb4',
'HangulSyllableTwel': '\xed\x89\xac',
'HangulSyllableTweo': '\xed\x89\x88',
'HangulSyllableTwen': '\xed\x89\xa8',
'HangulSyllableOeng': '\xec\x9a\x8d',
'HangulSyllableGgyabs': '\xea\xba\x96',
'CanadianSyllabicsCwaa': '\xe1\x92\x9e',
'HangulSyllableBbolt': '\xeb\xbd\x8d',
'SquareWithLowerRightDiagonalHalfBlack': '\xe2\x97\xaa',
'HangulSyllableBbolp': '\xeb\xbd\x8e',
'HangulSyllableBbols': '\xeb\xbd\x8c',
'UnicodeCharacter0xd94b': '\xed\xa5\x8b',
'HangulSyllableBbolm': '\xeb\xbd\x8a',
'HangulSyllableBbolh': '\xeb\xbd\x8f',
'HangulSyllableMyol': '\xeb\xac\xa0',
'HangulSyllableBbolg': '\xeb\xbd\x89',
'HangulSyllableBbolb': '\xeb\xbd\x8b',
'HalfwidthKatakanaLetterMe': '\xef\xbe\x92',
'CjkUnifiedIdeograph958e': '\xe9\x96\x8e',
'CjkUnifiedIdeograph8b67': '\xe8\xad\xa7',
'WhiteSquare': '\xe2\x96\xa1',
'UpDownDoubleArrow': '\xe2\x87\x95',
'HangulSyllableHyeong': '\xed\x98\x95',
'ArabicLetterHehGoal': '\xdb\x81',
'NaryUnionOperatorWithDot': '\xe2\xa8\x83',
'LatinCapitalLetterRWithDoubleGrave': '\xc8\x90',
'CjkUnifiedIdeograph83ca': '\xe8\x8f\x8a',
'CjkUnifiedIdeograph83cb': '\xe8\x8f\x8b',
'CjkUnifiedIdeograph83cc': '\xe8\x8f\x8c',
'CjkUnifiedIdeograph83cd': '\xe8\x8f\x8d',
'CjkUnifiedIdeograph83ce': '\xe8\x8f\x8e',
'CjkUnifiedIdeograph83cf': '\xe8\x8f\x8f',
'CjkUnifiedIdeograph958f': '\xe9\x96\x8f',
'UnicodeCharacter0x32ff': '\xe3\x8b\xbf',
'HangulSyllableDdult': '\xeb\x9a\xa9',
'HangulSyllableDduls': '\xeb\x9a\xa8',
'HangulSyllableDdulp': '\xeb\x9a\xaa',
'FullwidthDigitSix': '\xef\xbc\x96',
'HangulSyllableDdulm': '\xeb\x9a\xa6',
'HangulSyllableDdulh': '\xeb\x9a\xab',
'HangulSyllableDdulg': '\xeb\x9a\xa5',
'HangulSyllableOenh': '\xec\x99\xbe',
'HangulSyllableDdulb': '\xeb\x9a\xa7',
'CyrillicSmallLetterVe': '\xd0\xb2',
'CjkUnifiedIdeograph5cee': '\xe5\xb3\xae',
'CjkUnifiedIdeograph5ced': '\xe5\xb3\xad',
'UnicodeCharacter0xf535': '\xef\x94\xb5',
'OpenBox': '\xe2\x90\xa3',
'CjkUnifiedIdeograph5cef': '\xe5\xb3\xaf',
'LatinSmallLetterToneSix': '\xc6\x85',
'UnicodeCharacter0xdced': '\xed\xb3\xad',
'CombiningEnclosingScreen': '\xe2\x83\xa2',
'UnicodeCharacter0xdcef': '\xed\xb3\xaf',
'UnicodeCharacter0xdcea': '\xed\xb3\xaa',
'UnicodeCharacter0xdceb': '\xed\xb3\xab',
'UnicodeCharacter0xdcec': '\xed\xb3\xac',
'CjkUnifiedIdeograph5cec': '\xe5\xb3\xac',
'CjkUnifiedIdeograph5ceb': '\xe5\xb3\xab',
'ArabicSmallYeh': '\xdb\xa6',
'MongolianLetterAliGaliHalfYa': '\xe1\xa2\xa7',
'RejangLetterYa': '\xea\xa4\xbf',
'HangulSyllableCalt': '\xec\xb0\xb5',
'UnicodeCharacter0xdf0b': '\xed\xbc\x8b',
'HangulSyllableGgolh': '\xea\xbc\xbb',
'UnicodeCharacter0xdce8': '\xed\xb3\xa8',
'UnicodeCharacter0xdce9': '\xed\xb3\xa9',
'UnicodeCharacter0xdce4': '\xed\xb3\xa4',
'UnicodeCharacter0xdce5': '\xed\xb3\xa5',
'UnicodeCharacter0xdce6': '\xed\xb3\xa6',
'UnicodeCharacter0xdce7': '\xed\xb3\xa7',
'UnicodeCharacter0xdce0': '\xed\xb3\xa0',
'UnicodeCharacter0xdce1': '\xed\xb3\xa1',
'UnicodeCharacter0xdce2': '\xed\xb3\xa2',
'UnicodeCharacter0xdce3': '\xed\xb3\xa3',
'HangulSyllableGgolm': '\xea\xbc\xb6',
'UnicodeCharacter0xdf0c': '\xed\xbc\x8c',
'CjkUnifiedIdeograph958b': '\xe9\x96\x8b',
'HangulSyllableGgyags': '\xea\xba\x87',
'CjkUnifiedIdeograph9706': '\xe9\x9c\x86',
'LimbuLetterDa': '\xe1\xa4\x8d',
'HangulSyllableReus': '\xeb\xa6\x87',
'CjkUnifiedIdeograph9707': '\xe9\x9c\x87',
'CjkUnifiedIdeograph9704': '\xe9\x9c\x84',
'HangulSyllableReud': '\xeb\xa5\xbb',
'HangulSyllableReug': '\xeb\xa5\xb5',
'HangulSyllableGgolp': '\xea\xbc\xba',
'HangulSyllableReuc': '\xeb\xa6\x8b',
'HangulSyllableReub': '\xeb\xa6\x85',
'HangulSyllableReum': '\xeb\xa6\x84',
'HangulSyllableReul': '\xeb\xa5\xbc',
'CjkUnifiedIdeograph958c': '\xe9\x96\x8c',
'HangulSyllableReun': '\xeb\xa5\xb8',
'HangulSyllableReuh': '\xeb\xa6\x8f',
'HangulSyllableReuk': '\xeb\xa6\x8c',
'CjkUnifiedIdeograph9703': '\xe9\x9c\x83',
'YiSyllableZzyx': '\xea\x8b\xa8',
'HangulSyllableTyeoh': '\xed\x86\x83',
'CjkUnifiedIdeograph7dde': '\xe7\xb7\x9e',
'AmalgamationOrCoproduct': '\xe2\xa8\xbf',
'HangulSyllableSseot': '\xec\x8e\x81',
'CjkUnifiedIdeograph7ddd': '\xe7\xb7\x9d',
'HangulSyllableSseop': '\xec\x8e\x82',
'HangulSyllableSseos': '\xec\x8d\xbb',
'HangulSyllableSseol': '\xec\x8d\xb0',
'VaiSyllableIn': '\xea\x94\xa5',
'HangulSyllableSseon': '\xec\x8d\xac',
'UnicodeCharacter0xefa5': '\xee\xbe\xa5',
'HangulSyllableSseoh': '\xec\x8e\x83',
'UnicodeCharacter0xefa3': '\xee\xbe\xa3',
'HangulSyllableSseoj': '\xec\x8d\xbe',
'HangulSyllableSseok': '\xec\x8e\x80',
'HangulSyllableSseod': '\xec\x8d\xaf',
'HangulSyllableSseog': '\xec\x8d\xa9',
'HangulSyllableSseob': '\xec\x8d\xb9',
'HangulSyllableSseoc': '\xec\x8d\xbf',
'CanadianSyllabicsCarrierLe': '\xe1\x98\xa4',
'CanadianSyllabicsCarrierLa': '\xe1\x98\xa7',
'CjkUnifiedIdeograph93f5': '\xe9\x8f\xb5',
'SquareGaron': '\xe3\x8c\x8e',
'HangulJungseongIya': '\xe1\x86\x99',
'CjkRadicalLameOne': '\xe2\xba\x8e',
'CjkUnifiedIdeograph93f6': '\xe9\x8f\xb6',
'CanadianSyllabicsCarrierLu': '\xe1\x98\xa2',
'CjkUnifiedIdeograph93f7': '\xe9\x8f\xb7',
'CjkUnifiedIdeograph4fa1': '\xe4\xbe\xa1',
'SaurashtraLetterJa': '\xea\xa2\x99',
'CjkUnifiedIdeograph93f0': '\xe9\x8f\xb0',
'BoxDrawingsVerticalLightAndHorizontalHeavy': '\xe2\x94\xbf',
'CjkUnifiedIdeograph9709': '\xe9\x9c\x89',
'CyrillicSmallLetterIota': '\xea\x99\x87',
'CjkUnifiedIdeograph93f1': '\xe9\x8f\xb1',
'UnicodeCharacter0xefaf': '\xee\xbe\xaf',
'UnicodeCharacter0xefad': '\xee\xbe\xad',
'UnicodeCharacter0xefae': '\xee\xbe\xae',
'UnicodeCharacter0xefab': '\xee\xbe\xab',
'CjkUnifiedIdeograph93f2': '\xe9\x8f\xb2',
'YiRadicalChe': '\xea\x93\x83',
'CanadianSyllabicsNaskapiNwaa': '\xe1\x93\x8f',
'RecyclingSymbolForType1Plastics': '\xe2\x99\xb3',
'UnicodeCharacter0xe491': '\xee\x92\x91',
'IdeographicAnnotationManMark': '\xe3\x86\x9f',
'UnicodeCharacter0x312f': '\xe3\x84\xaf',
'GurmukhiLetterZa': '\xe0\xa9\x9b',
'UnicodeCharacter0x312e': '\xe3\x84\xae',
'KatakanaLetterGe': '\xe3\x82\xb2',
'HangulSyllableMyob': '\xeb\xac\xa9',
'KatakanaLetterGa': '\xe3\x82\xac',
'TibetanMarkYigMgoTshegShadMa': '\xe0\xbc\x87',
'KatakanaLetterGo': '\xe3\x82\xb4',
'CjkUnifiedIdeograph6457': '\xe6\x91\x97',
'KatakanaLetterGi': '\xe3\x82\xae',
'KatakanaLetterGu': '\xe3\x82\xb0',
'TeluguLetterTsa': '\xe0\xb1\x98',
'RightwardsHarpoonWithBarbUpwards': '\xe2\x87\x80',
'HangulSyllableGgon': '\xea\xbc\xb0',
'CanadianSyllabicsRoo': '\xe1\x95\x89',
'SlopingLargeOr': '\xe2\xa9\x97',
'IdeographicTelegraphSymbolForDayFourteen': '\xe3\x8f\xad',
'HangulSyllableMyunh': '\xeb\xae\xaa',
'MyanmarLetterVocalicR': '\xe1\x81\x92',
'MyanmarLetterVocalicL': '\xe1\x81\x94',
'YiSyllableBbep': '\xea\x81\xab',
'YiSyllableBbex': '\xea\x81\xa9',
'UnicodeCharacter0xd8f6': '\xed\xa3\xb6',
'UnicodeCharacter0xd8f7': '\xed\xa3\xb7',
'UnicodeCharacter0xd8f4': '\xed\xa3\xb4',
'UnicodeCharacter0xd8f5': '\xed\xa3\xb5',
'UnicodeCharacter0xd8f2': '\xed\xa3\xb2',
'CjkUnifiedIdeograph5d32': '\xe5\xb4\xb2',
'UnicodeCharacter0xd8f0': '\xed\xa3\xb0',
'UnicodeCharacter0xd8f1': '\xed\xa3\xb1',
'CjkUnifiedIdeograph644a': '\xe6\x91\x8a',
'CjkUnifiedIdeograph644b': '\xe6\x91\x8b',
'CjkUnifiedIdeograph5d33': '\xe5\xb4\xb3',
'CjkUnifiedIdeograph644d': '\xe6\x91\x8d',
'CjkUnifiedIdeograph644e': '\xe6\x91\x8e',
'CjkUnifiedIdeograph644f': '\xe6\x91\x8f',
'UnicodeCharacter0xd8f9': '\xed\xa3\xb9',
'HangulSyllableNyeonj': '\xeb\x85\x85',
'CjkUnifiedIdeograph6198': '\xe6\x86\x98',
'CjkUnifiedIdeograph6199': '\xe6\x86\x99',
'CjkUnifiedIdeograph6196': '\xe6\x86\x96',
'CjkUnifiedIdeograph6197': '\xe6\x86\x97',
'CjkUnifiedIdeograph6194': '\xe6\x86\x94',
'MalayalamLetterSha': '\xe0\xb4\xb6',
'CjkUnifiedIdeograph6192': '\xe6\x86\x92',
'CjkUnifiedIdeograph6193': '\xe6\x86\x93',
'CjkUnifiedIdeograph6190': '\xe6\x86\x90',
'CjkUnifiedIdeograph6191': '\xe6\x86\x91',
'EthiopicNumberSixty': '\xe1\x8d\xb7',
'CjkUnifiedIdeograph5d38': '\xe5\xb4\xb8',
'SyriacColonSkewedLeft': '\xdc\x86',
'TibetanLetterKha': '\xe0\xbd\x81',
'HangulSyllableWing': '\xec\x9c\x99',
'CyrillicSmallLetterTwe': '\xea\x9a\x8d',
'GreekYpogegrammeni': '\xcd\xba',
'HangulSyllableWinj': '\xec\x9c\x89',
'YiSyllableRrurx': '\xea\x8e\x8c',
'HangulSyllableWinh': '\xec\x9c\x8a',
'ArabicLetterBehIsolatedForm': '\xef\xba\x8f',
'KhmerSymbolPrammuoyKoet': '\xe1\xa7\xa6',
'HangulSyllableJab': '\xec\x9e\xa1',
'CjkUnifiedIdeograph619f': '\xe6\x86\x9f',
'CjkUnifiedIdeograph619d': '\xe6\x86\x9d',
'CjkUnifiedIdeograph619e': '\xe6\x86\x9e',
'CjkUnifiedIdeograph619b': '\xe6\x86\x9b',
'CjkUnifiedIdeograph619c': '\xe6\x86\x9c',
'CjkUnifiedIdeograph619a': '\xe6\x86\x9a',
'CjkUnifiedIdeograph6448': '\xe6\x91\x88',
'CjkUnifiedIdeograph6449': '\xe6\x91\x89',
'UnicodeCharacter0xd8fd': '\xed\xa3\xbd',
'ChamLetterNha': '\xea\xa8\x91',
'HangulSyllableTyeop': '\xed\x86\x82',
'UnicodeCharacter0xd8fc': '\xed\xa3\xbc',
'UnicodeCharacter0xd8fa': '\xed\xa3\xba',
'CjkUnifiedIdeograph6440': '\xe6\x91\x80',
'CjkUnifiedIdeograph6441': '\xe6\x91\x81',
'CjkUnifiedIdeograph6442': '\xe6\x91\x82',
'CjkUnifiedIdeograph6443': '\xe6\x91\x83',
'CjkUnifiedIdeograph6444': '\xe6\x91\x84',
'CjkUnifiedIdeograph6445': '\xe6\x91\x85',
'CjkUnifiedIdeograph6446': '\xe6\x91\x86',
'CjkUnifiedIdeograph6447': '\xe6\x91\x87',
'LL': '\xe2\x85\x82',
'CombiningCyrillicLetterIe': '\xe2\xb7\xb7',
'UnicodeCharacter0xf742': '\xef\x9d\x82',
'CjkUnifiedIdeograph8b6b': '\xe8\xad\xab',
'HangulSyllableTyeos': '\xed\x85\xbb',
'Aries': '\xe2\x99\x88',
'CjkUnifiedIdeograph771f': '\xe7\x9c\x9f',
'CjkUnifiedIdeograph771e': '\xe7\x9c\x9e',
'CjkUnifiedIdeograph771d': '\xe7\x9c\x9d',
'CopticCapitalLetterGangia': '\xcf\xaa',
'CjkUnifiedIdeograph771b': '\xe7\x9c\x9b',
'CjkUnifiedIdeograph771a': '\xe7\x9c\x9a',
'HangulSyllableSswik': '\xec\x93\x90',
'CjkUnifiedIdeograph362f': '\xe3\x98\xaf',
'CjkUnifiedIdeograph362e': '\xe3\x98\xae',
'CjkUnifiedIdeograph362d': '\xe3\x98\xad',
'CjkUnifiedIdeograph362c': '\xe3\x98\xac',
'CjkUnifiedIdeograph362b': '\xe3\x98\xab',
'CjkUnifiedIdeograph362a': '\xe3\x98\xaa',
'HangulSyllableGenh': '\xea\xb2\x92',
'HangulSyllableGeng': '\xea\xb2\xa1',
'CjkUnifiedIdeograph6941': '\xe6\xa5\x81',
'KangxiRadicalStopping': '\xe2\xbe\x89',
'KangxiRadicalWork': '\xe2\xbc\xaf',
'CjkUnifiedIdeograph93fd': '\xe9\x8f\xbd',
'HangulSyllableTyobs': '\xed\x88\xa2',
'CjkUnifiedIdeograph93fe': '\xe9\x8f\xbe',
'UnicodeCharacter0xf530': '\xef\x94\xb0',
'CjkUnifiedIdeograph93ff': '\xe9\x8f\xbf',
'GeorgianLetterHoe': '\xe1\x83\xb5',
'UnicodeCharacter0xeaf0': '\xee\xab\xb0',
'CjkUnifiedIdeograph93fa': '\xe9\x8f\xba',
'LatinSmallLetterPWithMiddleTilde': '\xe1\xb5\xb1',
'CjkUnifiedIdeograph3626': '\xe3\x98\xa6',
'CjkUnifiedIdeograph3625': '\xe3\x98\xa5',
'CjkUnifiedIdeograph3624': '\xe3\x98\xa4',
'CjkUnifiedIdeograph3623': '\xe3\x98\xa3',
'CjkUnifiedIdeograph3622': '\xe3\x98\xa2',
'CjkUnifiedIdeograph3621': '\xe3\x98\xa1',
'CjkUnifiedIdeograph3620': '\xe3\x98\xa0',
'CjkUnifiedIdeograph5f27': '\xe5\xbc\xa7',
'CjkUnifiedIdeograph6949': '\xe6\xa5\x89',
'HelmSymbol': '\xe2\x8e\x88',
'CjkUnifiedIdeograph93fc': '\xe9\x8f\xbc',
'HangulSyllableTyeot': '\xed\x86\x81',
'CjkUnifiedIdeograph3629': '\xe3\x98\xa9',
'CjkUnifiedIdeograph3628': '\xe3\x98\xa8',
'CjkUnifiedIdeograph7717': '\xe7\x9c\x97',
'CjkUnifiedIdeograph7716': '\xe7\x9c\x96',
'CjkUnifiedIdeograph7715': '\xe7\x9c\x95',
'CjkUnifiedIdeograph4f43': '\xe4\xbd\x83',
'CjkUnifiedIdeograph7713': '\xe7\x9c\x93',
'CjkUnifiedIdeograph7712': '\xe7\x9c\x92',
'CjkUnifiedIdeograph7711': '\xe7\x9c\x91',
'CjkUnifiedIdeograph7710': '\xe7\x9c\x90',
'CjkUnifiedIdeograph7719': '\xe7\x9c\x99',
'CjkUnifiedIdeograph7718': '\xe7\x9c\x98',
'HangulSyllableSyelm': '\xec\x85\xba',
'HangulSyllableMwenh': '\xeb\xad\xb2',
'RightBlackLenticularBracket': '\xe3\x80\x91',
'IdeographicTelegraphSymbolForHourTwentythree': '\xe3\x8d\xaf',
'YiSyllableQie': '\xea\x90\x82',
'CircledHangulNieunA': '\xe3\x89\xaf',
'HangulSyllableJyon': '\xec\xa3\xa4',
'YiSyllableQix': '\xea\x8f\xbd',
'HangulSyllableBbweolt': '\xeb\xbf\xb5',
'HangulSyllableJyom': '\xec\xa3\xb0',
'LatinSmallLetterToneFive': '\xc6\xbd',
'HalfwidthKatakanaLetterMi': '\xef\xbe\x90',
'YiSyllableQip': '\xea\x8f\xbf',
'HangulSyllableBels': '\xeb\xb2\xac',
'YiSyllableQit': '\xea\x8f\xbc',
'UnicodeCharacter0xa9f6': '\xea\xa7\xb6',
'CjkUnifiedIdeograph9bf1': '\xe9\xaf\xb1',
'CjkUnifiedIdeograph9bf0': '\xe9\xaf\xb0',
'CjkUnifiedIdeograph9bf3': '\xe9\xaf\xb3',
'CjkUnifiedIdeograph9bf2': '\xe9\xaf\xb2',
'CjkUnifiedIdeograph9bf5': '\xe9\xaf\xb5',
'CjkUnifiedIdeograph9bf4': '\xe9\xaf\xb4',
'CjkUnifiedIdeograph9bf7': '\xe9\xaf\xb7',
'CjkUnifiedIdeograph9bf6': '\xe9\xaf\xb6',
'CjkUnifiedIdeograph9bf9': '\xe9\xaf\xb9',
'CjkUnifiedIdeograph9bf8': '\xe9\xaf\xb8',
'HangulSyllableGgod': '\xea\xbc\xb3',
'HangulSyllableBbweolp': '\xeb\xbf\xb6',
'YiSyllableYy': '\xea\x92\x89',
'HangulSyllableGoc': '\xea\xb3\xb7',
'BengaliLetterVocalicRr': '\xe0\xa7\xa0',
'EthiopicSyllableGwa': '\xe1\x8c\x90',
'HangulSyllableBelp': '\xeb\xb2\xae',
'EthiopicSyllableNaa': '\xe1\x8a\x93',
'HangulSyllableKyaelm': '\xec\xbb\x92',
'BoxDrawingsDownSingleAndRightDouble': '\xe2\x95\x92',
'HangulSyllableKyaelh': '\xec\xbb\x97',
'CjkUnifiedIdeograph353b': '\xe3\x94\xbb',
'HangulSyllableCwaegs': '\xec\xb5\x83',
'EthiopicSyllableGwe': '\xe1\x8c\x95',
'HangulSyllableMweng': '\xeb\xae\x81',
'CjkUnifiedIdeograph9b9c': '\xe9\xae\x9c',
'HangulSyllableKyaelp': '\xec\xbb\x96',
'HangulSyllableKyaels': '\xec\xbb\x94',
'YiSyllableZurx': '\xea\x8a\xa6',
'LatinSmallLetterOWithTildeAndMacron': '\xc8\xad',
'CjkUnifiedIdeograph70c3': '\xe7\x83\x83',
'CjkUnifiedIdeograph9bfa': '\xe9\xaf\xba',
'UnicodeCharacter0xda3a': '\xed\xa8\xba',
'CjkUnifiedIdeograph9bfc': '\xe9\xaf\xbc',
'CjkUnifiedIdeograph9bfb': '\xe9\xaf\xbb',
'CjkUnifiedIdeograph9bfe': '\xe9\xaf\xbe',
'CjkUnifiedIdeograph9bfd': '\xe9\xaf\xbd',
'UnicodeCharacter0xda3f': '\xed\xa8\xbf',
'CjkUnifiedIdeograph9bff': '\xe9\xaf\xbf',
'HangulSyllableSwaeb': '\xec\x87\x95',
'CanadianSyllabicsNwa': '\xe1\x93\x8b',
'WhiteBullet': '\xe2\x97\xa6',
'HangulSyllableJjyep': '\xec\xaa\xba',
'HangulLetterYesieungpansios': '\xe3\x86\x83',
'HangulSyllableJjyes': '\xec\xaa\xb3',
'HangulSyllableJjyet': '\xec\xaa\xb9',
'CjkUnifiedIdeograph7b6c': '\xe7\xad\xac',
'CjkUnifiedIdeograph7b6b': '\xe7\xad\xab',
'CjkUnifiedIdeograph7b6a': '\xe7\xad\xaa',
'HangulSyllableJjyek': '\xec\xaa\xb8',
'HangulSyllableJjyel': '\xec\xaa\xa8',
'CanadianSyllabicsCarrierRu': '\xe1\x97\x8a',
'CjkUnifiedIdeograph7b6e': '\xe7\xad\xae',
'SinhalaLetterOoyanna': '\xe0\xb6\x95',
'HangulSyllableJjyeb': '\xec\xaa\xb1',
'HangulSyllableJjyec': '\xec\xaa\xb7',
'HangulSyllableJjyed': '\xec\xaa\xa7',
'CjkUnifiedIdeograph7a30': '\xe7\xa8\xb0',
'HangulLetterIeung': '\xe3\x85\x87',
'HangulSyllableSyus': '\xec\x8a\x9b',
'TripleNestedLessthan': '\xe2\xab\xb7',
'Frown': '\xe2\x8c\xa2',
'HangulSyllablePyogg': '\xed\x91\x9e',
'HangulSyllableSyeolp': '\xec\x85\xa2',
'CopticCapitalLetterAlfa': '\xe2\xb2\x80',
'CjkUnifiedIdeograph9b9f': '\xe9\xae\x9f',
'UnicodeCharacter0xf744': '\xef\x9d\x84',
'ArabicLigatureBehWithZainFinalForm': '\xef\xb1\xab',
'BraillePatternDots2568': '\xe2\xa2\xb2',
'CjkUnifiedIdeograph7b63': '\xe7\xad\xa3',
'CjkUnifiedIdeograph7b62': '\xe7\xad\xa2',
'CjkUnifiedIdeograph7b61': '\xe7\xad\xa1',
'CjkUnifiedIdeograph7b60': '\xe7\xad\xa0',
'CjkUnifiedIdeograph7b67': '\xe7\xad\xa7',
'KangxiRadicalSlave': '\xe2\xbe\xaa',
'CjkUnifiedIdeograph7b65': '\xe7\xad\xa5',
'CjkUnifiedIdeograph694d': '\xe6\xa5\x8d',
'CjkUnifiedIdeograph7b69': '\xe7\xad\xa9',
'CjkUnifiedIdeograph7b68': '\xe7\xad\xa8',
'CjkUnifiedIdeograph9563': '\xe9\x95\xa3',
'YiSyllableQy': '\xea\x90\x94',
'subrho': '\xe1\xb5\xa8',
'CjkUnifiedIdeograph7a36': '\xe7\xa8\xb6',
'YiSyllableQu': '\xea\x90\x8e',
'HalfwidthHangulLetterOe': '\xef\xbf\x8f',
'HebrewAccentGereshMuqdam': '\xd6\x9d',
'HangulSyllableGyaels': '\xea\xb1\xa0',
'CjkUnifiedIdeograph435c': '\xe4\x8d\x9c',
'GreekSmallLetterEpsilonWithDasia': '\xe1\xbc\x91',
'HangulSyllableDyaegs': '\xeb\x8c\xbb',
'DownwardsArrowFromBar': '\xe2\x86\xa7',
'CanadianSyllabicsCarrierSee': '\xe1\x99\x91',
'CjkUnifiedIdeograph9a98': '\xe9\xaa\x98',
'OutlinedWhiteStar': '\xe2\x9a\x9d',
'ArabicLigatureKafWithYehFinalForm': '\xef\xb2\x84',
'KangxiRadicalMortar': '\xe2\xbe\x85',
'HangulSyllableWaes': '\xec\x99\xaf',
'HangulSyllableSyaelm': '\xec\x84\x8a',
'KhmerLetterNo': '\xe1\x9e\x93',
'HangulSyllableWaep': '\xec\x99\xb6',
'ArabicLigatureAlefWithFathatanFinalForm': '\xef\xb4\xbc',
'CjkUnifiedIdeograph7d8f': '\xe7\xb6\x8f',
'CjkUnifiedIdeograph7d8e': '\xe7\xb6\x8e',
'CjkUnifiedIdeograph7d8d': '\xe7\xb6\x8d',
'CjkUnifiedIdeograph7d8c': '\xe7\xb6\x8c',
'CjkUnifiedIdeograph7d8b': '\xe7\xb6\x8b',
'CjkUnifiedIdeograph7d8a': '\xe7\xb6\x8a',
'CjkUnifiedIdeograph4c8a': '\xe4\xb2\x8a',
'CjkUnifiedIdeograph4c8c': '\xe4\xb2\x8c',
'CjkUnifiedIdeograph4c8b': '\xe4\xb2\x8b',
'CjkUnifiedIdeograph4c8e': '\xe4\xb2\x8e',
'CjkUnifiedIdeograph4c8d': '\xe4\xb2\x8d',
'HangulSyllableBbunh': '\xeb\xbf\x92',
'CjkUnifiedIdeograph4c8f': '\xe4\xb2\x8f',
'CjkUnifiedIdeograph87eb': '\xe8\x9f\xab',
'CjkUnifiedIdeograph87ec': '\xe8\x9f\xac',
'CjkUnifiedIdeograph91f4': '\xe9\x87\xb4',
'CjkUnifiedIdeograph87ea': '\xe8\x9f\xaa',
'CjkUnifiedIdeograph87ef': '\xe8\x9f\xaf',
'CjkUnifiedIdeograph91f3': '\xe9\x87\xb3',
'CjkUnifiedIdeograph87ed': '\xe8\x9f\xad',
'CjkUnifiedIdeograph87ee': '\xe8\x9f\xae',
'CjkUnifiedIdeograph71cc': '\xe7\x87\x8c',
'CjkUnifiedIdeograph71cb': '\xe7\x87\x8b',
'CjkUnifiedIdeograph71ca': '\xe7\x87\x8a',
'CjkUnifiedIdeograph71cf': '\xe7\x87\x8f',
'CjkUnifiedIdeograph71ce': '\xe7\x87\x8e',
'CjkUnifiedIdeograph4f4e': '\xe4\xbd\x8e',
'BalineseMusicalSymbolDang': '\xe1\xad\xa4',
'CombiningZigzagBelow': '\xe1\xb7\x8f',
'UnicodeCharacter0xeb8d': '\xee\xae\x8d',
'UnicodeCharacter0xe495': '\xee\x92\x95',
'HangulSyllableDdoelg': '\xeb\x99\xad',
'TagbanwaVowelSignI': '\xe1\x9d\xb2',
'SaurashtraLetterE': '\xea\xa2\x8c',
'TagbanwaVowelSignU': '\xe1\x9d\xb3',
'HangulSyllablePinj': '\xed\x95\x81',
'HangulSyllablePinh': '\xed\x95\x82',
'LepchaLetterDa': '\xe1\xb0\x8c',
'HangulSyllableWaen': '\xec\x99\xa0',
'UnicodeCharacter0xaab3': '\xea\xaa\xb3',
'HangulSyllableKoess': '\xec\xbe\xbc',
'YiSyllableHxap': '\xea\x89\x91',
'GreekCapitalLetterEtaWithDasiaAndVaria': '\xe1\xbc\xab',
'CjkUnifiedIdeograph87e3': '\xe8\x9f\xa3',
'CjkUnifiedIdeograph71c9': '\xe7\x87\x89',
'CjkUnifiedIdeograph71c8': '\xe7\x87\x88',
'CjkUnifiedIdeograph87e6': '\xe8\x9f\xa6',
'CjkUnifiedIdeograph87e7': '\xe8\x9f\xa7',
'CjkUnifiedIdeograph87e4': '\xe8\x9f\xa4',
'CjkUnifiedIdeograph87e5': '\xe8\x9f\xa5',
'CjkUnifiedIdeograph71c3': '\xe7\x87\x83',
'CjkUnifiedIdeograph71c2': '\xe7\x87\x82',
'ModifierLetterYangDepartingToneMark': '\xcb\xab',
'CjkUnifiedIdeograph71c0': '\xe7\x87\x80',
'CjkUnifiedIdeograph71c7': '\xe7\x87\x87',
'CjkUnifiedIdeograph71c6': '\xe7\x87\x86',
'CjkUnifiedIdeograph71c5': '\xe7\x87\x85',
'CjkStrokeTn': '\xe3\x87\x9d',
'CjkUnifiedIdeograph4c89': '\xe4\xb2\x89',
'CjkUnifiedIdeograph4c88': '\xe4\xb2\x88',
'CjkUnifiedIdeograph7d85': '\xe7\xb6\x85',
'CjkUnifiedIdeograph7d84': '\xe7\xb6\x84',
'CjkUnifiedIdeograph7d83': '\xe7\xb6\x83',
'CanadianSyllabicsWestcreeTwaa': '\xe1\x91\xa4',
'CjkUnifiedIdeograph7d81': '\xe7\xb6\x81',
'CjkUnifiedIdeograph7d80': '\xe7\xb6\x80',
'CjkUnifiedIdeograph4c81': '\xe4\xb2\x81',
'CjkUnifiedIdeograph4c80': '\xe4\xb2\x80',
'CjkUnifiedIdeograph4c83': '\xe4\xb2\x83',
'CjkUnifiedIdeograph4c82': '\xe4\xb2\x82',
'CjkUnifiedIdeograph4c85': '\xe4\xb2\x85',
'CjkUnifiedIdeograph4c84': '\xe4\xb2\x84',
'CjkUnifiedIdeograph4c87': '\xe4\xb2\x87',
'CjkUnifiedIdeograph4c86': '\xe4\xb2\x86',
'UpDownArrow': '\xe2\x86\x95',
'HebrewLetterPeWithRafe': '\xef\xad\x8e',
'ModifierLetterGraveAccent': '\xcb\x8b',
'CyrillicCapitalLetterKomiNje': '\xd4\x8a',
'SineWave': '\xe2\x88\xbf',
'OriyaLetterPa': '\xe0\xac\xaa',
'KhmerSignBeyyal': '\xe1\x9f\x98',
'UnicodeCharacter0xd829': '\xed\xa0\xa9',
'HangulSyllableWaed': '\xec\x99\xa3',
'HangulSyllableDweon': '\xeb\x91\xb0',
'CjkUnifiedIdeograph7b77': '\xe7\xad\xb7',
'HangulSyllableMyeolm': '\xeb\xa9\xba',
'UnicodeCharacter0xe26e': '\xee\x89\xae',
'HangulSyllableDweol': '\xeb\x91\xb4',
'UnicodeCharacter0xe26c': '\xee\x89\xac',
'UnicodeCharacter0xe26b': '\xee\x89\xab',
'UnicodeCharacter0xe26a': '\xee\x89\xaa',
'UnicodeCharacter0xf08b': '\xef\x82\x8b',
'HangulSyllableDweok': '\xeb\x92\x84',
'UnicodeCharacter0xf08a': '\xef\x82\x8a',
'UnicodeCharacter0xf08f': '\xef\x82\x8f',
'HangulSyllableDyim': '\xeb\x94\x88',
'UnicodeCharacter0xf08d': '\xef\x82\x8d',
'UnicodeCharacter0xf08e': '\xef\x82\x8e',
'HangulSyllableSyess': '\xec\x86\x84',
'HangulSyllableTyoss': '\xed\x88\xa4',
'FourDotMark': '\xe2\x81\x9b',
'WhiteNib': '\xe2\x9c\x91',
'CjkUnifiedIdeograph3886': '\xe3\xa2\x86',
'BugineseLetterYa': '\xe1\xa8\x90',
'HangulSyllableMyeolh': '\xeb\xa9\xbf',
'CjkUnifiedIdeograph44e5': '\xe4\x93\xa5',
'CjkUnifiedIdeograph44e4': '\xe4\x93\xa4',
'CjkUnifiedIdeograph44e7': '\xe4\x93\xa7',
'CjkUnifiedIdeograph44e6': '\xe4\x93\xa6',
'CjkUnifiedIdeograph44e1': '\xe4\x93\xa1',
'CjkUnifiedIdeograph44e0': '\xe4\x93\xa0',
'CjkUnifiedIdeograph44e3': '\xe4\x93\xa3',
'CjkUnifiedIdeograph44e2': '\xe4\x93\xa2',
'CjkUnifiedIdeograph44e9': '\xe4\x93\xa9',
'CjkUnifiedIdeograph44e8': '\xe4\x93\xa8',
'InhibitSymmetricSwapping': '\xe2\x81\xaa',
'HangulLetterRieulthieuth': '\xe3\x84\xbe',
'UnicodeCharacter0xf548': '\xef\x95\x88',
'HangulSyllableDweos': '\xeb\x91\xbf',
'HangulSyllableGyit': '\xea\xb8\xad',
'HangulSyllableGyis': '\xea\xb8\xa7',
'HangulSyllableGyip': '\xea\xb8\xae',
'CjkRadicalNetOne': '\xe2\xba\xb1',
'HangulSyllableDyic': '\xeb\x94\x8f',
'YiSyllableGguot': '\xea\x88\x9e',
'CircledAnticlockwiserotatedDivisionSign': '\xe2\xa6\xbc',
'HangulSyllableGyig': '\xea\xb8\x95',
'HangulSyllableGyid': '\xea\xb8\x9b',
'UnicodeCharacter0xe264': '\xee\x89\xa4',
'HangulSyllableGyib': '\xea\xb8\xa5',
'HangulSyllableGyic': '\xea\xb8\xab',
'UnicodeCharacter0xe261': '\xee\x89\xa1',
'UnicodeCharacter0xe260': '\xee\x89\xa0',
'HangulSyllableGyin': '\xea\xb8\x98',
'UnicodeCharacter0xf083': '\xef\x82\x83',
'HangulSyllableGyil': '\xea\xb8\x9c',
'HangulSyllableGyim': '\xea\xb8\xa4',
'HangulSyllableGyij': '\xea\xb8\xaa',
'HangulSyllableGyik': '\xea\xb8\xac',
'CanadianSyllabicsKii': '\xe1\x91\xae',
'UnicodeCharacter0xe268': '\xee\x89\xa8',
'ArabicLetterSeenWithTwoDotsVerticallyAbove': '\xdd\xad',
'ArabicLigatureKhahWithAlefMaksuraFinalForm': '\xef\xb4\x9f',
'MyanmarLetterIi': '\xe1\x80\xa4',
'CjkUnifiedIdeograph44ee': '\xe4\x93\xae',
'CjkUnifiedIdeograph44ed': '\xe4\x93\xad',
'CjkUnifiedIdeograph44ef': '\xe4\x93\xaf',
'CjkUnifiedIdeograph44ea': '\xe4\x93\xaa',
'CjkUnifiedIdeograph44ec': '\xe4\x93\xac',
'CjkUnifiedIdeograph44eb': '\xe4\x93\xab',
'UnicodeCharacter0xfdd5': '\xef\xb7\x95',
'CjkUnifiedIdeograph863e': '\xe8\x98\xbe',
'CjkUnifiedIdeograph863d': '\xe8\x98\xbd',
'CjkUnifiedIdeograph863f': '\xe8\x98\xbf',
'CjkUnifiedIdeograph863a': '\xe8\x98\xba',
'CjkCompatibilityIdeographfa08': '\xef\xa8\x88',
'CjkUnifiedIdeograph863c': '\xe8\x98\xbc',
'CjkUnifiedIdeograph863b': '\xe8\x98\xbb',
'CjkCompatibilityIdeographfa05': '\xef\xa8\x85',
'CjkCompatibilityIdeographfa04': '\xef\xa8\x84',
'CjkCompatibilityIdeographfa07': '\xef\xa8\x87',
'CjkCompatibilityIdeographfa06': '\xef\xa8\x86',
'CjkCompatibilityIdeographfa01': '\xef\xa8\x81',
'ArabicLetterDalWithTwoDotsVerticallyBelowAndSmallTah': '\xdd\x99',
'CjkCompatibilityIdeographfa03': '\xef\xa8\x83',
'HebrewPointPatah': '\xd6\xb7',
'UnicodeCharacter0xe191': '\xee\x86\x91',
'UnicodeCharacter0xe190': '\xee\x86\x90',
'UnicodeCharacter0xe193': '\xee\x86\x93',
'UnicodeCharacter0xe192': '\xee\x86\x92',
'UnicodeCharacter0xe195': '\xee\x86\x95',
'UnicodeCharacter0xe194': '\xee\x86\x94',
'UnicodeCharacter0xe197': '\xee\x86\x97',
'UnicodeCharacter0xe196': '\xee\x86\x96',
'UnicodeCharacter0xe199': '\xee\x86\x99',
'HangulSyllableMwigs': '\xeb\xae\x8b',
'KangxiRadicalSickness': '\xe2\xbd\xa7',
'HangulSyllableRwes': '\xeb\xa4\xb3',
'CyrillicSmallLetterReversedYu': '\xea\x99\x95',
'HangulSyllableJyus': '\xec\xa5\xbf',
'HangulSyllableRwep': '\xeb\xa4\xba',
'HangulSyllableJyut': '\xec\xa6\x85',
'HangulSyllableRwet': '\xeb\xa4\xb9',
'LatinCapitalLetterTWithHook': '\xc6\xac',
'UnicodeCharacter0xeaf4': '\xee\xab\xb4',
'HangulSyllableRwec': '\xeb\xa4\xb7',
'HangulSyllableRweb': '\xeb\xa4\xb1',
'HangulSyllableJyuc': '\xec\xa6\x83',
'HangulSyllableJyub': '\xec\xa5\xbd',
'CopticSymbolTauRo': '\xe2\xb3\xa8',
'HangulSyllableJyud': '\xec\xa5\xb3',
'HangulSyllableJyug': '\xec\xa5\xad',
'HangulSyllableRwed': '\xeb\xa4\xa7',
'HangulSyllableRwek': '\xeb\xa4\xb8',
'HangulSyllableRwej': '\xeb\xa4\xb6',
'HangulSyllableJyuk': '\xec\xa6\x84',
'HangulSyllableRweh': '\xeb\xa4\xbb',
'HangulSyllableRweo': '\xeb\xa4\x84',
'CjkUnifiedIdeograph4358': '\xe4\x8d\x98',
'KayahLiDigitFour': '\xea\xa4\x84',
'HangulSyllableRwel': '\xeb\xa4\xa8',
'UnicodeCharacter0xe19a': '\xee\x86\x9a',
'UnicodeCharacter0xe19c': '\xee\x86\x9c',
'UnicodeCharacter0xe19b': '\xee\x86\x9b',
'UnicodeCharacter0xe19e': '\xee\x86\x9e',
'UnicodeCharacter0xa29': '\xe0\xa8\xa9',
'UnicodeCharacter0xe19f': '\xee\x86\x9f',
'CjkUnifiedIdeograph8635': '\xe8\x98\xb5',
'CjkUnifiedIdeograph8634': '\xe8\x98\xb4',
'CjkUnifiedIdeograph8637': '\xe8\x98\xb7',
'CjkUnifiedIdeograph8636': '\xe8\x98\xb6',
'CjkUnifiedIdeograph8631': '\xe8\x98\xb1',
'CjkUnifiedIdeograph8630': '\xe8\x98\xb0',
'CjkUnifiedIdeograph8633': '\xe8\x98\xb3',
'CjkUnifiedIdeograph8632': '\xe8\x98\xb2',
'CjkUnifiedIdeograph5e53': '\xe5\xb9\x93',
'CjkCompatibilityIdeographfa0d': '\xef\xa8\x8d',
'CjkCompatibilityIdeographfa0f': '\xef\xa8\x8f',
'CjkUnifiedIdeograph8639': '\xe8\x98\xb9',
'CjkUnifiedIdeograph8638': '\xe8\x98\xb8',
'CjkCompatibilityIdeographfa0c': '\xef\xa8\x8c',
'CjkUnifiedIdeograph9138': '\xe9\x84\xb8',
'UnicodeCharacter0xf554': '\xef\x95\x94',
'CjkUnifiedIdeograph45a4': '\xe4\x96\xa4',
'LatinCapitalLetterAWithDiaeresis': '\xc3\x84',
'YiSyllableRrex': '\xea\x8e\x85',
'UnicodeCharacter0xf550': '\xef\x95\x90',
'UnicodeCharacter0xf551': '\xef\x95\x91',
'UnicodeCharacter0xf552': '\xef\x95\x92',
'UnicodeCharacter0xf553': '\xef\x95\x93',
'UnicodeCharacter0xf558': '\xef\x95\x98',
'UnicodeCharacter0xf559': '\xef\x95\x99',
'HangulSyllableSweoss': '\xec\x89\x88',
'CjkUnifiedIdeograph9135': '\xe9\x84\xb5',
'CjkUnifiedIdeograph7101': '\xe7\x84\x81',
'HangulSyllableByelp': '\xeb\xb3\xa6',
'TaiLeLetterTone2': '\xe1\xa5\xb0',
'HangulSyllableRwaenh': '\xeb\xa2\x9a',
'CircledIdeographMale': '\xe3\x8a\x9a',
'CombiningCyrillicLetterTe': '\xe2\xb7\xae',
'KangxiRadicalHead': '\xe2\xbe\xb8',
'HangulSyllableWelg': '\xec\x9b\xb1',
'UnicodeCharacter0xdbb4': '\xed\xae\xb4',
'UnicodeCharacter0x2d79': '\xe2\xb5\xb9',
'UnicodeCharacter0x2d78': '\xe2\xb5\xb8',
'UnicodeCharacter0x2d77': '\xe2\xb5\xb7',
'KangxiRadicalSacrificialWine': '\xe2\xbe\xbf',
'UnicodeCharacter0x2d75': '\xe2\xb5\xb5',
'UnicodeCharacter0x2d74': '\xe2\xb5\xb4',
'UnicodeCharacter0x2d73': '\xe2\xb5\xb3',
'UnicodeCharacter0x2d72': '\xe2\xb5\xb2',
'UnicodeCharacter0x2d71': '\xe2\xb5\xb1',
'UnicodeCharacter0x2d70': '\xe2\xb5\xb0',
'CjkUnifiedIdeograph67d3': '\xe6\x9f\x93',
'CjkUnifiedIdeograph67d2': '\xe6\x9f\x92',
'CjkUnifiedIdeograph67d1': '\xe6\x9f\x91',
'CjkUnifiedIdeograph67d0': '\xe6\x9f\x90',
'CjkUnifiedIdeograph67d7': '\xe6\x9f\x97',
'CjkUnifiedIdeograph67d6': '\xe6\x9f\x96',
'CjkUnifiedIdeograph67d5': '\xe6\x9f\x95',
'CjkUnifiedIdeograph67d4': '\xe6\x9f\x94',
'UnicodeCharacter0xe31f': '\xee\x8c\x9f',
'CjkUnifiedIdeograph67d9': '\xe6\x9f\x99',
'CjkUnifiedIdeograph66a0': '\xe6\x9a\xa0',
'HangulSyllableHeugg': '\xed\x9d\x92',
'UnicodeCharacter0xda05': '\xed\xa8\x85',
'HangulSyllableSsyat': '\xec\x8d\x89',
'UnicodeCharacter0xeaf5': '\xee\xab\xb5',
'HangulSyllableSsyas': '\xec\x8d\x83',
'HangulSyllableSsyap': '\xec\x8d\x8a',
'HangulSyllableRyuh': '\xeb\xa5\xb3',
'HangulSyllableRyuk': '\xeb\xa5\xb0',
'HangulSyllableDwinh': '\xeb\x92\xaa',
'HangulSyllableSsyag': '\xec\x8c\xb1',
'HangulSyllableDwinj': '\xeb\x92\xa9',
'UnicodeCharacter0xa996': '\xea\xa6\x96',
'HangulSyllableSsyab': '\xec\x8d\x81',
'HangulSyllableSsyac': '\xec\x8d\x87',
'HangulSyllableSsyan': '\xec\x8c\xb4',
'HangulSyllableRyum': '\xeb\xa5\xa8',
'HangulSyllableSsyal': '\xec\x8c\xb8',
'HangulSyllableSsyam': '\xec\x8d\x80',
'HangulSyllableSsyaj': '\xec\x8d\x86',
'HangulSyllableSsyak': '\xec\x8d\x88',
'HangulSyllableSsyah': '\xec\x8d\x8b',
'HangulSyllableDwing': '\xeb\x92\xb9',
'CjkUnifiedIdeograph67dc': '\xe6\x9f\x9c',
'CjkUnifiedIdeograph67db': '\xe6\x9f\x9b',
'CjkUnifiedIdeograph67da': '\xe6\x9f\x9a',
'CjkUnifiedIdeograph67df': '\xe6\x9f\x9f',
'CjkUnifiedIdeograph67de': '\xe6\x9f\x9e',
'CjkUnifiedIdeograph67dd': '\xe6\x9f\x9d',
'HangulSyllableSsep': '\xec\x8e\x9e',
'GreekCapitalLetterAlphaWithDasiaAndVaria': '\xe1\xbc\x8b',
'HangulSyllableRyun': '\xeb\xa5\x9c',
'HangulSyllableSset': '\xec\x8e\x9d',
'HangulSyllableSseu': '\xec\x93\xb0',
'HangulSyllableSseh': '\xec\x8e\x9f',
'HangulSyllableGeop': '\xea\xb2\x8a',
'HangulSyllableSsej': '\xec\x8e\x9a',
'HangulSyllableSsek': '\xec\x8e\x9c',
'HangulSyllableSsel': '\xec\x8e\x8c',
'HangulSyllableSsem': '\xec\x8e\x94',
'HangulSyllableSsen': '\xec\x8e\x88',
'HangulSyllableSseo': '\xec\x8d\xa8',
'HeavyBlackfeatheredRightwardsArrow': '\xe2\x9e\xb8',
'UnicodeCharacter0x2d7f': '\xe2\xb5\xbf',
'UnicodeCharacter0x2d7e': '\xe2\xb5\xbe',
'UnicodeCharacter0x2d7d': '\xe2\xb5\xbd',
'UnicodeCharacter0x2d7c': '\xe2\xb5\xbc',
'UnicodeCharacter0x2d7b': '\xe2\xb5\xbb',
'UnicodeCharacter0x2d7a': '\xe2\xb5\xba',
'HangulSyllableGilm': '\xea\xb8\xba',
'naturalExponent': '\xe2\x85\x87',
'SundaneseDigitTwo': '\xe1\xae\xb2',
'CjkUnifiedIdeograph9ab9': '\xe9\xaa\xb9',
'HangulSyllableByaenh': '\xeb\xb1\xae',
'HangulSyllableByaenj': '\xeb\xb1\xad',
'CjkUnifiedIdeograph9ab2': '\xe9\xaa\xb2',
'CjkUnifiedIdeograph9ab3': '\xe9\xaa\xb3',
'CjkUnifiedIdeograph9ab0': '\xe9\xaa\xb0',
'CjkUnifiedIdeograph9ab1': '\xe9\xaa\xb1',
'CjkUnifiedIdeograph9ab6': '\xe9\xaa\xb6',
'CjkUnifiedIdeograph9ab7': '\xe9\xaa\xb7',
'CjkUnifiedIdeograph9ab4': '\xe9\xaa\xb4',
'CjkUnifiedIdeograph9ab5': '\xe9\xaa\xb5',
'CjkUnifiedIdeograph8f2b': '\xe8\xbc\xab',
'CjkUnifiedIdeograph8f2c': '\xe8\xbc\xac',
'CjkUnifiedIdeograph8f2a': '\xe8\xbc\xaa',
'CjkUnifiedIdeograph8f2f': '\xe8\xbc\xaf',
'CjkUnifiedIdeograph8f2d': '\xe8\xbc\xad',
'CjkUnifiedIdeograph8f2e': '\xe8\xbc\xae',
'LimbuLetterBha': '\xe1\xa4\x93',
'PhagspaLetterMa': '\xea\xa1\x8f',
'EthiopicParagraphSeparator': '\xe1\x8d\xa8',
'UnicodeCharacter0xe065': '\xee\x81\xa5',
'UnicodeCharacter0xea08': '\xee\xa8\x88',
'CjkUnifiedIdeograph48df': '\xe4\xa3\x9f',
'UnicodeCharacter0xdf08': '\xed\xbc\x88',
'CheckMark': '\xe2\x9c\x93',
'CjkUnifiedIdeograph48de': '\xe4\xa3\x9e',
'CjkUnifiedIdeograph48db': '\xe4\xa3\x9b',
'CjkUnifiedIdeograph48dc': '\xe4\xa3\x9c',
'HangulSyllableDyuh': '\xeb\x93\x9b',
'CjkUnifiedIdeograph48da': '\xe4\xa3\x9a',
'CjkUnifiedIdeograph4639': '\xe4\x98\xb9',
'CjkUnifiedIdeograph4638': '\xe4\x98\xb8',
'CjkUnifiedIdeograph4631': '\xe4\x98\xb1',
'CjkUnifiedIdeograph4630': '\xe4\x98\xb0',
'CjkUnifiedIdeograph4633': '\xe4\x98\xb3',
'CjkUnifiedIdeograph4632': '\xe4\x98\xb2',
'CjkUnifiedIdeograph4635': '\xe4\x98\xb5',
'CjkUnifiedIdeograph4634': '\xe4\x98\xb4',
'CjkUnifiedIdeograph4637': '\xe4\x98\xb7',
'CjkUnifiedIdeograph4636': '\xe4\x98\xb6',
'CjkUnifiedIdeograph8f22': '\xe8\xbc\xa2',
'TibetanVowelSignVocalicLl': '\xe0\xbd\xb9',
'CjkUnifiedIdeograph8f20': '\xe8\xbc\xa0',
'CjkUnifiedIdeograph8f21': '\xe8\xbc\xa1',
'CjkUnifiedIdeograph8f26': '\xe8\xbc\xa6',
'CjkUnifiedIdeograph8f27': '\xe8\xbc\xa7',
'MalayalamDigitEight': '\xe0\xb5\xae',
'CjkUnifiedIdeograph8f25': '\xe8\xbc\xa5',
'HangulSyllableJjaelp': '\xec\xa8\x86',
'CjkUnifiedIdeograph8f28': '\xe8\xbc\xa8',
'CjkUnifiedIdeograph8f29': '\xe8\xbc\xa9',
'KangxiRadicalTrack': '\xe2\xbd\xb1',
'UnicodeCharacter0xe7f8': '\xee\x9f\xb8',
'HangulSyllableJjaelh': '\xec\xa8\x87',
'GreekCapitalLetterOmegaWithTonos': '\xce\x8f',
'HangulSyllableJjaelm': '\xec\xa8\x82',
'CopticSmallLetterKhi': '\xe2\xb2\xad',
'VulgarFractionFiveSixths': '\xe2\x85\x9a',
'CjkUnifiedIdeograph9abc': '\xe9\xaa\xbc',
'CjkUnifiedIdeograph9aba': '\xe9\xaa\xba',
'CjkUnifiedIdeograph9abf': '\xe9\xaa\xbf',
'HangulSyllableRyus': '\xeb\xa5\xab',
'CjkUnifiedIdeograph9abd': '\xe9\xaa\xbd',
'CjkUnifiedIdeograph9abe': '\xe9\xaa\xbe',
'HangulSyllableGeoc': '\xea\xb2\x87',
'CjkUnifiedIdeograph463a': '\xe4\x98\xba',
'RighttoleftOverride': '\xe2\x80\xae',
'CjkUnifiedIdeograph463c': '\xe4\x98\xbc',
'CjkUnifiedIdeograph463b': '\xe4\x98\xbb',
'CjkUnifiedIdeograph463e': '\xe4\x98\xbe',
'CjkUnifiedIdeograph463d': '\xe4\x98\xbd',
'HangulSyllableRwaegg': '\xeb\xa2\x96',
'CjkUnifiedIdeograph463f': '\xe4\x98\xbf',
'TifinaghLetterYabh': '\xe2\xb4\xb2',
'CjkUnifiedIdeograph9de5': '\xe9\xb7\xa5',
'GlagoliticCapitalLetterShtapic': '\xe2\xb0\xac',
'CjkUnifiedIdeograph48d8': '\xe4\xa3\x98',
'CjkUnifiedIdeograph48d9': '\xe4\xa3\x99',
'CjkUnifiedIdeograph48d6': '\xe4\xa3\x96',
'CjkUnifiedIdeograph48d7': '\xe4\xa3\x97',
'CjkUnifiedIdeograph48d4': '\xe4\xa3\x94',
'CjkUnifiedIdeograph48d5': '\xe4\xa3\x95',
'CjkUnifiedIdeograph48d2': '\xe4\xa3\x92',
'CjkUnifiedIdeograph48d3': '\xe4\xa3\x93',
'CjkUnifiedIdeograph48d0': '\xe4\xa3\x90',
'CjkUnifiedIdeograph48d1': '\xe4\xa3\x91',
'UnicodeCharacter0xe499': '\xee\x92\x99',
'YiSyllableBbe': '\xea\x81\xaa',
'YiSyllableBba': '\xea\x81\xa0',
'MyanmarToneMarkSgawKarenKePho': '\xe1\x81\xa4',
'YiSyllableBbo': '\xea\x81\xa7',
'ModifierLetterSmallSWithHook': '\xe1\xb6\xb3',
'UnicodeCharacter0xe7f6': '\xee\x9f\xb6',
'YiSyllableBbi': '\xea\x81\x98',
'MyanmarLetterMonJha': '\xe1\x81\x9b',
'ThaanaLetterQaafu': '\xde\xa4',
'HangulSyllableGyaelh': '\xea\xb1\xa3',
'HangulSyllableMom': '\xeb\xaa\xb8',
'YiSyllableBby': '\xea\x81\xb4',
'CjkUnifiedIdeograph527b': '\xe5\x89\xbb',
'CjkUnifiedIdeograph527c': '\xe5\x89\xbc',
'CjkUnifiedIdeograph527a': '\xe5\x89\xba',
'CjkUnifiedIdeograph527f': '\xe5\x89\xbf',
'CjkUnifiedIdeograph527d': '\xe5\x89\xbd',
'CjkUnifiedIdeograph527e': '\xe5\x89\xbe',
'HangulSyllableJjyolb': '\xec\xac\xb7',
'HangulSyllableJjyolg': '\xec\xac\xb5',
'UnicodeCharacter0xea04': '\xee\xa8\x84',
'ThaanaLetterDhaalu': '\xde\x8b',
'LatinCapitalLetterYWithHook': '\xc6\xb3',
'CjkUnifiedIdeograph9de8': '\xe9\xb7\xa8',
'Excess': '\xe2\x88\xb9',
'HangulSyllableJjyols': '\xec\xac\xb8',
'HangulSyllableJjyolp': '\xec\xac\xba',
'YiSyllableBbot': '\xea\x81\xa5',
'UnicodeCharacter0xe7f4': '\xee\x9f\xb4',
'Breve': '\xcb\x98',
'AlefSymbol': '\xe2\x84\xb5',
'UnicodeCharacter0xee58': '\xee\xb9\x98',
'ArabicLetterKafWithThreeDotsBelow': '\xda\xae',
'YiSyllablePa': '\xea\x81\x81',
'HangulSyllableSim': '\xec\x8b\xac',
'HalfwidthKatakanaLetterHo': '\xef\xbe\x8e',
'CjkRadicalTurtle': '\xe2\xbb\xb1',
'YiSyllableZux': '\xea\x8a\xa3',
'UnicodeCharacter0xe7f5': '\xee\x9f\xb5',
'HalfwidthKatakanaLetterHi': '\xef\xbe\x8b',
'HalfwidthKatakanaLetterHe': '\xef\xbe\x8d',
'HalfwidthKatakanaLetterHa': '\xef\xbe\x8a',
'CjkUnifiedIdeograph5272': '\xe5\x89\xb2',
'CjkUnifiedIdeograph5273': '\xe5\x89\xb3',
'CjkUnifiedIdeograph5270': '\xe5\x89\xb0',
'CjkUnifiedIdeograph5271': '\xe5\x89\xb1',
'CjkUnifiedIdeograph5276': '\xe5\x89\xb6',
'CjkUnifiedIdeograph5277': '\xe5\x89\xb7',
'CjkUnifiedIdeograph5274': '\xe5\x89\xb4',
'CjkUnifiedIdeograph5275': '\xe5\x89\xb5',
'UnicodeCharacter0xea03': '\xee\xa8\x83',
'CjkUnifiedIdeograph5278': '\xe5\x89\xb8',
'CjkUnifiedIdeograph5279': '\xe5\x89\xb9',
'ArabicLetterTchehMedialForm': '\xef\xad\xbd',
'TaiLeLetterPha': '\xe1\xa5\x9a',
'HangulSyllableSsunj': '\xec\x91\xa9',
'HangulSyllableSsunh': '\xec\x91\xaa',
'HangulSyllableSsung': '\xec\x91\xb9',
'j': '\xcf\x86',
'Sparkle': '\xe2\x9d\x87',
'CopticCapitalLetterFi': '\xe2\xb2\xaa',
'KangxiRadicalPlow': '\xe2\xbd\xbe',
'HangulSyllableByelb': '\xeb\xb3\xa3',
'CjkUnifiedIdeograph776f': '\xe7\x9d\xaf',
'UnicodeCharacter0xde3a': '\xed\xb8\xba',
'UnicodeCharacter0xf069': '\xef\x81\xa9',
'UnicodeCharacter0xea01': '\xee\xa8\x81',
'UnicodeCharacter0xdc3f': '\xed\xb0\xbf',
'CanadianSyllabicsBlackfootWo': '\xe1\x96\xb6',
'UnicodeCharacter0xdc3d': '\xed\xb0\xbd',
'UnicodeCharacter0xdc3e': '\xed\xb0\xbe',
'UnicodeCharacter0xdc3b': '\xed\xb0\xbb',
'UnicodeCharacter0xdc3c': '\xed\xb0\xbc',
'HangulSyllableJjeu': '\xec\xaf\x94',
'CjkUnifiedIdeograph9b25': '\xe9\xac\xa5',
'SingleRightpointingAngleQuotationMark': '\xe2\x80\xba',
'CombiningLatinSmallLetterRBelow': '\xe1\xb7\x8a',
'CjkUnifiedIdeograph9b27': '\xe9\xac\xa7',
'CjkUnifiedIdeograph9b26': '\xe9\xac\xa6',
'UnicodeCharacter0xdf06': '\xed\xbc\x86',
'CjkUnifiedIdeograph540e': '\xe5\x90\x8e',
'CjkUnifiedIdeograph9b20': '\xe9\xac\xa0',
'UnicodeCharacter0xdc38': '\xed\xb0\xb8',
'UnicodeCharacter0xdc39': '\xed\xb0\xb9',
'UnicodeCharacter0xdc36': '\xed\xb0\xb6',
'UnicodeCharacter0xdc37': '\xed\xb0\xb7',
'UnicodeCharacter0xdc34': '\xed\xb0\xb4',
'CjkUnifiedIdeograph540f': '\xe5\x90\x8f',
'UnicodeCharacter0xdc32': '\xed\xb0\xb2',
'UnicodeCharacter0xdc33': '\xed\xb0\xb3',
'UnicodeCharacter0xdc30': '\xed\xb0\xb0',
'UnicodeCharacter0xdc31': '\xed\xb0\xb1',
'CjkUnifiedIdeograph7995': '\xe7\xa6\x95',
'UnicodeCharacter0xed89': '\xee\xb6\x89',
'CjkUnifiedIdeograph913c': '\xe9\x84\xbc',
'SaurashtraConsonantSignHaaru': '\xea\xa2\xb4',
'CjkUnifiedIdeograph7656': '\xe7\x99\x96',
'CjkUnifiedIdeograph7657': '\xe7\x99\x97',
'CjkUnifiedIdeograph7654': '\xe7\x99\x94',
'NumberTwelveFullStop': '\xe2\x92\x93',
'CjkUnifiedIdeograph7652': '\xe7\x99\x92',
'CjkUnifiedIdeograph7653': '\xe7\x99\x93',
'CjkUnifiedIdeograph7650': '\xe7\x99\x90',
'CjkUnifiedIdeograph7651': '\xe7\x99\x91',
'HangulSyllableRoenj': '\xeb\xa2\xb5',
'HangulSyllableMoe': '\xeb\xab\xbc',
'CjkUnifiedIdeograph7658': '\xe7\x99\x98',
'CjkUnifiedIdeograph7659': '\xe7\x99\x99',
'YiSyllableZur': '\xea\x8a\xa7',
'varTheta': '\xcf\xb4',
'CjkUnifiedIdeograph8c67': '\xe8\xb1\xa7',
'CjkUnifiedIdeograph8c66': '\xe8\xb1\xa6',
'CjkUnifiedIdeograph8c65': '\xe8\xb1\xa5',
'CjkUnifiedIdeograph8c64': '\xe8\xb1\xa4',
'CjkUnifiedIdeograph8c63': '\xe8\xb1\xa3',
'CjkUnifiedIdeograph8c62': '\xe8\xb1\xa2',
'CjkUnifiedIdeograph8c61': '\xe8\xb1\xa1',
'CjkUnifiedIdeograph8c60': '\xe8\xb1\xa0',
'YiSyllableRruo': '\xea\x8d\xbf',
'UnicodeCharacter0xea05': '\xee\xa8\x85',
'CjkUnifiedIdeograph8c69': '\xe8\xb1\xa9',
'CjkUnifiedIdeograph8c68': '\xe8\xb1\xa8',
'ParenthesizedNumberEighteen': '\xe2\x92\x85',
'QuestionedEqualTo': '\xe2\x89\x9f',
'CjkUnifiedIdeograph3c6b': '\xe3\xb1\xab',
'CjkUnifiedIdeograph3c6c': '\xe3\xb1\xac',
'CjkUnifiedIdeograph3c6a': '\xe3\xb1\xaa',
'CjkUnifiedIdeograph3c6f': '\xe3\xb1\xaf',
'CjkUnifiedIdeograph3c6d': '\xe3\xb1\xad',
'CjkUnifiedIdeograph3c6e': '\xe3\xb1\xae',
'HangulSyllableBuss': '\xeb\xb6\x94',
'HangulSyllableJyolg': '\xec\xa3\xa9',
'MongolianManchuFullStop': '\xe1\xa0\x89',
'HangulSyllableJyolm': '\xec\xa3\xaa',
'HangulSyllableJyolh': '\xec\xa3\xaf',
'KangxiRadicalLeaf': '\xe2\xbe\xb4',
'CjkUnifiedIdeograph765f': '\xe7\x99\x9f',
'CjkUnifiedIdeograph765d': '\xe7\x99\x9d',
'CjkUnifiedIdeograph765e': '\xe7\x99\x9e',
'CjkUnifiedIdeograph765b': '\xe7\x99\x9b',
'CjkUnifiedIdeograph765c': '\xe7\x99\x9c',
'CjkUnifiedIdeograph765a': '\xe7\x99\x9a',
'UnicodeCharacter0xda29': '\xed\xa8\xa9',
'ThaiCharacterNoNu': '\xe0\xb8\x99',
'CjkUnifiedIdeograph3c68': '\xe3\xb1\xa8',
'CjkUnifiedIdeograph3c69': '\xe3\xb1\xa9',
'SmallIdeographicComma': '\xef\xb9\x91',
'CjkUnifiedIdeograph3c62': '\xe3\xb1\xa2',
'CjkUnifiedIdeograph3c63': '\xe3\xb1\xa3',
'CjkUnifiedIdeograph3c60': '\xe3\xb1\xa0',
'CjkUnifiedIdeograph3c61': '\xe3\xb1\xa1',
'CjkUnifiedIdeograph3c66': '\xe3\xb1\xa6',
'CjkUnifiedIdeograph3c67': '\xe3\xb1\xa7',
'CjkUnifiedIdeograph3c64': '\xe3\xb1\xa4',
'CjkUnifiedIdeograph3c65': '\xe3\xb1\xa5',
'CjkUnifiedIdeograph8c6f': '\xe8\xb1\xaf',
'CjkUnifiedIdeograph8c6e': '\xe8\xb1\xae',
'CjkUnifiedIdeograph8c6d': '\xe8\xb1\xad',
'CjkUnifiedIdeograph8c6c': '\xe8\xb1\xac',
'CjkUnifiedIdeograph8c6b': '\xe8\xb1\xab',
'CjkUnifiedIdeograph8c6a': '\xe8\xb1\xaa',
'HangulSyllableDdwinj': '\xeb\x9b\xb5',
'ArabicLetterKirghizOeFinalForm': '\xef\xaf\xa1',
'CombiningDoubleTildeRightHalf': '\xef\xb8\xa3',
'SundaneseSignPanyecek': '\xe1\xae\x80',
'UnicodeCharacter0xef27': '\xee\xbc\xa7',
'UnicodeCharacter0x886': '\xe0\xa2\x86',
'UnicodeCharacter0xeaf9': '\xee\xab\xb9',
'BoxDrawingsVerticalHeavyAndLeftLight': '\xe2\x94\xa8',
'GujaratiLetterAi': '\xe0\xaa\x90',
'GujaratiLetterAa': '\xe0\xaa\x86',
'KangxiRadicalSpear': '\xe2\xbd\xad',
'HangulSyllableKweoss': '\xed\x80\x90',
'SubscriptRightParenthesis': '\xe2\x82\x8e',
'GujaratiLetterAu': '\xe0\xaa\x94',
'ThaanaLetterThaalu': '\xde\x9b',
'ErrorbarredBlackCircle': '\xe2\xa7\xb3',
'CjkUnifiedIdeograph9011': '\xe9\x80\x91',
'TibetanVowelSignIi': '\xe0\xbd\xb3',
'RightSquareBracketExtension': '\xe2\x8e\xa5',
'HangulSyllableByelh': '\xeb\xb3\xa7',
'UnicodeCharacter0xde3c': '\xed\xb8\xbc',
'HangulSyllableGgeolm': '\xea\xbb\x86',
'LatinSmallLetterFWithMiddleTilde': '\xe1\xb5\xae',
'UnicodeCharacter0xf141': '\xef\x85\x81',
'CjkUnifiedIdeograph986b': '\xe9\xa1\xab',
'CjkUnifiedIdeograph7d03': '\xe7\xb4\x83',
'UnicodeCharacter0xe83b': '\xee\xa0\xbb',
'CjkUnifiedIdeograph8558': '\xe8\x95\x98',
'_chi': '\xe1\xb5\xaa',
'HangulSyllableDdeuss': '\xeb\x9c\xbc',
'CjkUnifiedIdeograph817c': '\xe8\x85\xbc',
'CjkUnifiedIdeograph8559': '\xe8\x95\x99',
'CanadianSyllabicsTe': '\xe1\x91\x8c',
'CanadianSyllabicsTa': '\xe1\x91\x95',
'GeorgianCapitalLetterKan': '\xe1\x82\xa9',
'CanadianSyllabicsTo': '\xe1\x91\x90',
'CanadianSyllabicsTh': '\xe1\x95\xaa',
'EthiopicSyllableKwee': '\xe1\x8a\xb4',
'VariationSelector6': '\xef\xb8\x85',
'UnicodeCharacter0xf709': '\xef\x9c\x89',
'CombiningVerticalLineBelow': '\xcc\xa9',
'CjkUnifiedIdeograph9796': '\xe9\x9e\x96',
'CjkUnifiedIdeograph9799': '\xe9\x9e\x99',
'CjkUnifiedIdeograph8552': '\xe8\x95\x92',
'CjkUnifiedIdeograph9798': '\xe9\x9e\x98',
'UnicodeCharacter0xf708': '\xef\x9c\x88',
'GreekCapitalLetterOmegaWithOxia': '\xe1\xbf\xbb',
'HangulJongseongRieulpieup': '\xe1\x86\xb2',
'HangulSyllableCwelp': '\xec\xb7\x9a',
'SinhalaLetterAlpapraanaGayanna': '\xe0\xb6\x9c',
'UnicodeCharacter0x9c': '\xc2\x9c',
'UnicodeCharacter0xe879': '\xee\xa1\xb9',
'CanadianSyllabicsFii': '\xe1\x95\x96',
'UnicodeCharacter0xdbf8': '\xed\xaf\xb8',
'UnicodeCharacter0xdbf9': '\xed\xaf\xb9',
'GeorgianCapitalLetterHar': '\xe1\x83\x84',
'CircledIdeographName': '\xe3\x8a\x94',
'UnicodeCharacter0xd996': '\xed\xa6\x96',
'UnicodeCharacter0xdbf6': '\xed\xaf\xb6',
'KannadaLetterJha': '\xe0\xb2\x9d',
'UnicodeCharacter0xdbf0': '\xed\xaf\xb0',
'UnicodeCharacter0xdbf1': '\xed\xaf\xb1',
'UnicodeCharacter0xdbf2': '\xed\xaf\xb2',
'UnicodeCharacter0xdbf3': '\xed\xaf\xb3',
'GeorgianCapitalLetterHae': '\xe1\x83\x80',
'CjkUnifiedIdeograph7af5': '\xe7\xab\xb5',
'NorthWestArrowCrossingNorthEastArrow': '\xe2\xa4\xb2',
'CjkUnifiedIdeograph7af7': '\xe7\xab\xb7',
'CjkUnifiedIdeograph7af0': '\xe7\xab\xb0',
'CjkUnifiedIdeograph7af1': '\xe7\xab\xb1',
'CjkUnifiedIdeograph7af2': '\xe7\xab\xb2',
'CjkUnifiedIdeograph7af3': '\xe7\xab\xb3',
'UnicodeCharacter0xa9fa': '\xea\xa7\xba',
'CjkUnifiedIdeograph7af8': '\xe7\xab\xb8',
'CjkUnifiedIdeograph7af9': '\xe7\xab\xb9',
'HangulSyllableBoess': '\xeb\xb5\x9c',
'CjkUnifiedIdeograph5401': '\xe5\x90\x81',
'CjkUnifiedIdeograph9b2d': '\xe9\xac\xad',
'HangulSyllableBuh': '\xeb\xb6\x9b',
'CjkUnifiedIdeograph9b2f': '\xe9\xac\xaf',
'UnicodeCharacter0xf705': '\xef\x9c\x85',
'CjkUnifiedIdeograph9b2a': '\xe9\xac\xaa',
'CjkUnifiedIdeograph7afd': '\xe7\xab\xbd',
'CjkUnifiedIdeograph7afe': '\xe7\xab\xbe',
'CjkUnifiedIdeograph7aff': '\xe7\xab\xbf',
'QuadrantUpperRightAndLowerLeftAndLowerRight': '\xe2\x96\x9f',
'CjkUnifiedIdeograph7afb': '\xe7\xab\xbb',
'CjkUnifiedIdeograph7afc': '\xe7\xab\xbc',
'UnicodeCharacter0x26bf': '\xe2\x9a\xbf',
'UnicodeCharacter0x26bd': '\xe2\x9a\xbd',
'UnicodeCharacter0x26be': '\xe2\x9a\xbe',
'HangulSyllableTwegg': '\xed\x89\xa6',
'HangulSyllablePwang': '\xed\x90\x9d',
'BoxDrawingsUpLightAndLeftHeavy': '\xe2\x94\x99',
'UnicodeCharacter0xf704': '\xef\x9c\x84',
'LeftSingleQuotationMark': '\xe2\x80\x98',
'UnicodeCharacter0xdbfd': '\xed\xaf\xbd',
'UnicodeCharacter0xdbfe': '\xed\xaf\xbe',
'UnicodeCharacter0xdbff': '\xed\xaf\xbf',
'YiSyllableNbiep': '\xea\x81\xbc',
'KhmerSymbolPrampiiKoet': '\xe1\xa7\xa7',
'UnicodeCharacter0xdbfa': '\xed\xaf\xba',
'UnicodeCharacter0xdbfb': '\xed\xaf\xbb',
'BlackLeftpointingPointer': '\xe2\x97\x84',
'HangulSyllableTup': '\xed\x89\x86',
'YiSyllableSsut': '\xea\x8c\xbc',
'BetSymbol': '\xe2\x84\xb6',
'LatinCapitalLetterOWithCircumflexAndAcute': '\xe1\xbb\x90',
'CyrillicCapitalLetterPsi': '\xd1\xb0',
'CyrillicSmallLetterAleutKa': '\xd4\x9f',
'UnicodeCharacter0xdc00': '\xed\xb0\x80',
'CjkUnifiedIdeograph9188': '\xe9\x86\x88',
'UnicodeCharacter0xc29': '\xe0\xb0\xa9',
'CjkUnifiedIdeograph838e': '\xe8\x8e\x8e',
'CjkUnifiedIdeograph838d': '\xe8\x8e\x8d',
'CjkUnifiedIdeograph838f': '\xe8\x8e\x8f',
'CjkUnifiedIdeograph838a': '\xe8\x8e\x8a',
'HangulSyllableNyep': '\xeb\x85\xb6',
'CjkUnifiedIdeograph838c': '\xe8\x8e\x8c',
'CjkUnifiedIdeograph838b': '\xe8\x8e\x8b',
'CjkUnifiedIdeograph8433': '\xe8\x90\xb3',
'CjkUnifiedIdeograph8432': '\xe8\x90\xb2',
'CjkUnifiedIdeograph8431': '\xe8\x90\xb1',
'CjkUnifiedIdeograph8430': '\xe8\x90\xb0',
'CjkUnifiedIdeograph8437': '\xe8\x90\xb7',
'CjkUnifiedIdeograph8436': '\xe8\x90\xb6',
'CjkUnifiedIdeograph8435': '\xe8\x90\xb5',
'CjkUnifiedIdeograph8434': '\xe8\x90\xb4',
'HangulSyllableNyed': '\xeb\x85\xa3',
'NewTaiLueLetterLowMa': '\xe1\xa6\x99',
'WreathProduct': '\xe2\x89\x80',
'UpwardsWhiteArrowOnPedestalWithHorizontalBar': '\xe2\x87\xac',
'HangulSyllableNyeb': '\xeb\x85\xad',
'HangulSyllablePyik': '\xed\x94\xb8',
'CjkUnifiedIdeograph5eef': '\xe5\xbb\xaf',
'CjkUnifiedIdeograph5eee': '\xe5\xbb\xae',
'CjkUnifiedIdeograph5eed': '\xe5\xbb\xad',
'CjkUnifiedIdeograph5eec': '\xe5\xbb\xac',
'CjkUnifiedIdeograph5eeb': '\xe5\xbb\xab',
'CjkUnifiedIdeograph5eea': '\xe5\xbb\xaa',
'HangulSyllablePyil': '\xed\x94\xa8',
'HangulSyllablePyic': '\xed\x94\xb7',
'HangulSyllablePyib': '\xed\x94\xb1',
'CjkUnifiedIdeograph9561': '\xe9\x95\xa1',
'HangulSyllablePyig': '\xed\x94\xa1',
'LatinSmallLetterDWithCircumflexBelow': '\xe1\xb8\x93',
'HangulSyllablePyid': '\xed\x94\xa7',
'CjkUnifiedIdeograph8eb9': '\xe8\xba\xb9',
'CjkUnifiedIdeograph8eb8': '\xe8\xba\xb8',
'ArabicLigatureKhahWithYehIsolatedForm': '\xef\xb4\x84',
'UnicodeCharacter0x2b7e': '\xe2\xad\xbe',
'UnicodeCharacter0x2b7d': '\xe2\xad\xbd',
'CjkUnifiedIdeograph8eb7': '\xe8\xba\xb7',
'UnicodeCharacter0x2b7f': '\xe2\xad\xbf',
'UnicodeCharacter0x2b7a': '\xe2\xad\xba',
'CjkUnifiedIdeograph8eb0': '\xe8\xba\xb0',
'UnicodeCharacter0x2b7c': '\xe2\xad\xbc',
'UnicodeCharacter0x2b7b': '\xe2\xad\xbb',
'CanadianSyllabicsWestcreeSwe': '\xe1\x93\xb7',
'CjkUnifiedIdeograph843b': '\xe8\x90\xbb',
'CjkUnifiedIdeograph843a': '\xe8\x90\xba',
'CanadianSyllabicsWestcreeSwa': '\xe1\x94\x81',
'CjkUnifiedIdeograph843f': '\xe8\x90\xbf',
'CjkUnifiedIdeograph843e': '\xe8\x90\xbe',
'CjkUnifiedIdeograph843d': '\xe8\x90\xbd',
'HangulSyllableKwaegs': '\xec\xbe\x8f',
'CanadianSyllabicsWestcreeSwo': '\xe1\x93\xbd',
'CjkUnifiedIdeograph8a88': '\xe8\xaa\x88',
'CanadianSyllabicsWestcreeSwi': '\xe1\x93\xb9',
'CjkUnifiedIdeograph979c': '\xe9\x9e\x9c',
'HangulSyllableTweolt': '\xed\x89\x95',
'CjkUnifiedIdeograph979b': '\xe9\x9e\x9b',
'CjkUnifiedIdeograph8389': '\xe8\x8e\x89',
'CjkUnifiedIdeograph8388': '\xe8\x8e\x88',
'CjkUnifiedIdeograph8385': '\xe8\x8e\x85',
'CjkUnifiedIdeograph8384': '\xe8\x8e\x84',
'CjkUnifiedIdeograph8387': '\xe8\x8e\x87',
'CjkUnifiedIdeograph8386': '\xe8\x8e\x86',
'CjkUnifiedIdeograph8381': '\xe8\x8e\x81',
'CjkUnifiedIdeograph8380': '\xe8\x8e\x80',
'CjkUnifiedIdeograph8383': '\xe8\x8e\x83',
'CjkUnifiedIdeograph8382': '\xe8\x8e\x82',
'UnicodeCharacter0x2b79': '\xe2\xad\xb9',
'UnicodeCharacter0x2b78': '\xe2\xad\xb8',
'UnicodeCharacter0x2b75': '\xe2\xad\xb5',
'UnicodeCharacter0x2b74': '\xe2\xad\xb4',
'UnicodeCharacter0x2b77': '\xe2\xad\xb7',
'UnicodeCharacter0x2b76': '\xe2\xad\xb6',
'UnicodeCharacter0x2b71': '\xe2\xad\xb1',
'UnicodeCharacter0x2b70': '\xe2\xad\xb0',
'UnicodeCharacter0x2b73': '\xe2\xad\xb3',
'UnicodeCharacter0x2b72': '\xe2\xad\xb2',
'SinhalaLetterTaalujaSanyoogaNaaksikyaya': '\xe0\xb6\xa5',
'CjkUnifiedIdeograph5ee6': '\xe5\xbb\xa6',
'CjkUnifiedIdeograph5ee5': '\xe5\xbb\xa5',
'CjkUnifiedIdeograph5ee4': '\xe5\xbb\xa4',
'CjkUnifiedIdeograph5ee3': '\xe5\xbb\xa3',
'CjkUnifiedIdeograph5ee2': '\xe5\xbb\xa2',
'CjkUnifiedIdeograph5ee1': '\xe5\xbb\xa1',
'CjkUnifiedIdeograph5ee0': '\xe5\xbb\xa0',
'GlagoliticSmallLetterZhivete': '\xe2\xb0\xb6',
'HangulSyllableTweols': '\xed\x89\x94',
'ArabicLigatureKafWithJeemIsolatedForm': '\xef\xb0\xb8',
'CjkUnifiedIdeograph5ee9': '\xe5\xbb\xa9',
'CjkUnifiedIdeograph5ee8': '\xe5\xbb\xa8',
'CjkUnifiedIdeograph3f0a': '\xe3\xbc\x8a',
'VaiSyllableWi': '\xea\x94\xa8',
'CjkUnifiedIdeograph3f0c': '\xe3\xbc\x8c',
'Ruledelayed': '\xe2\xa7\xb4',
'CjkUnifiedIdeograph3f0e': '\xe3\xbc\x8e',
'CjkUnifiedIdeograph3f0d': '\xe3\xbc\x8d',
'HangulSyllableHi': '\xed\x9e\x88',
'CjkUnifiedIdeograph3f0f': '\xe3\xbc\x8f',
'VaiSyllableWa': '\xea\x95\x8e',
'HangulSyllableHe': '\xed\x97\xa4',
'VaiSyllableWe': '\xea\x97\xa6',
'HangulSyllableHa': '\xed\x95\x98',
'BraillePatternDots1268': '\xe2\xa2\xa3',
'HangulSyllableHu': '\xed\x9b\x84',
'UnicodeCharacter0x92': '\xc2\x92',
'BraillePatternDots1267': '\xe2\xa1\xa3',
'ArabicLetterRrehFinalForm': '\xef\xae\x8d',
'HebrewPointJudeospanishVarika': '\xef\xac\x9e',
'CjkUnifiedIdeograph9346': '\xe9\x8d\x86',
'PhagspaLetterDa': '\xea\xa1\x8a',
'ArabicLigatureYehWithHamzaAboveWithWawIsolatedForm': '\xef\xaf\xae',
'BalineseCarikSiki': '\xe1\xad\x9e',
'HangulSyllableGgobs': '\xea\xbc\xbe',
'HangulSyllableBbeolp': '\xeb\xbb\x9e',
'ArmenianComma': '\xd5\x9d',
'CjkUnifiedIdeograph9347': '\xe9\x8d\x87',
'CjkUnifiedIdeograph3f01': '\xe3\xbc\x81',
'CjkUnifiedIdeograph3f00': '\xe3\xbc\x80',
'CjkUnifiedIdeograph3f03': '\xe3\xbc\x83',
'CjkUnifiedIdeograph3f02': '\xe3\xbc\x82',
'CjkUnifiedIdeograph3f05': '\xe3\xbc\x85',
'CjkUnifiedIdeograph3f04': '\xe3\xbc\x84',
'CjkUnifiedIdeograph3f07': '\xe3\xbc\x87',
'CjkUnifiedIdeograph3f06': '\xe3\xbc\x86',
'CjkUnifiedIdeograph3f09': '\xe3\xbc\x89',
'CjkUnifiedIdeograph3f08': '\xe3\xbc\x88',
'CjkUnifiedIdeograph9dd9': '\xe9\xb7\x99',
'CjkUnifiedIdeograph9861': '\xe9\xa1\xa1',
'HangulSyllableKweop': '\xed\x80\x96',
'CjkUnifiedIdeograph7d0d': '\xe7\xb4\x8d',
'HangulSyllableKweot': '\xed\x80\x95',
'HangulSyllablePwaess': '\xed\x90\xb8',
'ArmenianCapitalLetterBen': '\xd4\xb2',
'HangulSyllableKweob': '\xed\x80\x8d',
'HangulSyllableKweoc': '\xed\x80\x93',
'HangulSyllableKweod': '\xed\x80\x83',
'HangulSyllableKweog': '\xec\xbf\xbd',
'CjkUnifiedIdeograph9dd8': '\xe9\xb7\x98',
'HangulSyllableKweoj': '\xed\x80\x92',
'TeluguLetterLa': '\xe0\xb0\xb2',
'MyanmarSymbolShanExclamation': '\xe1\x82\x9f',
'HangulSyllableKweom': '\xed\x80\x8c',
'TeluguSignTuumu': '\xe0\xb1\xbf',
'UnicodeCharacter0xf66e': '\xef\x99\xae',
'UnicodeCharacter0xf4a8': '\xef\x92\xa8',
'LightShade': '\xe2\x96\x91',
'SyriacMusic': '\xdd\x89',
'CjkUnifiedIdeograph9342': '\xe9\x8d\x82',
'HangulSyllableBong': '\xeb\xb4\x89',
'SyriacLetterAlaph': '\xdc\x90',
'UnicodeCharacter0xe832': '\xee\xa0\xb2',
'HangulSyllableSwi': '\xec\x89\xac',
'UnicodeCharacter0xe830': '\xee\xa0\xb0',
'UnicodeCharacter0xe831': '\xee\xa0\xb1',
'UnicodeCharacter0xe836': '\xee\xa0\xb6',
'CjkUnifiedIdeograph855b': '\xe8\x95\x9b',
'UnicodeCharacter0xe834': '\xee\xa0\xb4',
'UnicodeCharacter0xe835': '\xee\xa0\xb5',
'CjkUnifiedIdeograph6b3c': '\xe6\xac\xbc',
'CjkUnifiedIdeograph6b3b': '\xe6\xac\xbb',
'CjkUnifiedIdeograph6b3a': '\xe6\xac\xba',
'CjkUnifiedIdeograph7c8c': '\xe7\xb2\x8c',
'CjkUnifiedIdeograph7c8d': '\xe7\xb2\x8d',
'CjkUnifiedIdeograph6b3f': '\xe6\xac\xbf',
'CjkUnifiedIdeograph6b3e': '\xe6\xac\xbe',
'CjkUnifiedIdeograph6b3d': '\xe6\xac\xbd',
'UnicodeCharacter0xf66c': '\xef\x99\xac',
'SaurashtraLetterVocalicR': '\xea\xa2\x88',
'TibetanMarkInitialYigMgoMdunMa': '\xe0\xbc\x84',
'UnicodeCharacter0xe67a': '\xee\x99\xba',
'LatinCapitalLetterOWithMacronAndGrave': '\xe1\xb9\x90',
'UnicodeCharacter0xf542': '\xef\x95\x82',
'UnicodeCharacter0xd9f9': '\xed\xa7\xb9',
'TibetanMarkCaretDzudRtagsMeLongCan': '\xe0\xbc\x93',
'UnicodeCharacter0xda0a': '\xed\xa8\x8a',
'SaurashtraLetterVocalicL': '\xea\xa2\x8a',
'CjkUnifiedIdeograph7c88': '\xe7\xb2\x88',
'CjkUnifiedIdeograph7c89': '\xe7\xb2\x89',
'CjkUnifiedIdeograph6b39': '\xe6\xac\xb9',
'CjkUnifiedIdeograph6b38': '\xe6\xac\xb8',
'UnicodeCharacter0xe83f': '\xee\xa0\xbf',
'UnicodeCharacter0xe83d': '\xee\xa0\xbd',
'UnicodeCharacter0xe83e': '\xee\xa0\xbe',
'CjkUnifiedIdeograph6b33': '\xe6\xac\xb3',
'CjkUnifiedIdeograph6b32': '\xe6\xac\xb2',
'CjkUnifiedIdeograph6b31': '\xe6\xac\xb1',
'CjkUnifiedIdeograph6b30': '\xe6\xac\xb0',
'ArabicLetterHeh': '\xd9\x87',
'CjkUnifiedIdeograph6b36': '\xe6\xac\xb6',
'CjkUnifiedIdeograph6b35': '\xe6\xac\xb5',
'CjkUnifiedIdeograph6b34': '\xe6\xac\xb4',
'HangulSyllableDo': '\xeb\x8f\x84',
'UnicodeCharacter0xe05d': '\xee\x81\x9d',
'ExtendedArabicindicDigitEight': '\xdb\xb8',
'LatinEpigraphicLetterILonga': '\xea\x9f\xbe',
'BalineseMusicalSymbolCombiningKempulWithJegogan': '\xe1\xad\xb0',
'UnicodeCharacter0xe179': '\xee\x85\xb9',
'CjkCompatibilityIdeographfa80': '\xef\xaa\x80',
'CjkUnifiedIdeograph93ae': '\xe9\x8e\xae',
'CjkUnifiedIdeograph93ad': '\xe9\x8e\xad',
'SymbolForEndOfMedium': '\xe2\x90\x99',
'CjkUnifiedIdeograph93af': '\xe9\x8e\xaf',
'CjkUnifiedIdeograph93aa': '\xe9\x8e\xaa',
'BigSolidus': '\xe2\xa7\xb8',
'BraillePatternDots367': '\xe2\xa1\xa4',
'KhmerDigitOne': '\xe1\x9f\xa1',
'CjkUnifiedIdeograph4b6f': '\xe4\xad\xaf',
'BraillePatternDots368': '\xe2\xa2\xa4',
'CjkUnifiedIdeograph4b6d': '\xe4\xad\xad',
'CjkUnifiedIdeograph4b6e': '\xe4\xad\xae',
'CjkUnifiedIdeograph4b6b': '\xe4\xad\xab',
'CjkUnifiedIdeograph4b6c': '\xe4\xad\xac',
'CjkUnifiedIdeograph4b6a': '\xe4\xad\xaa',
'CjkUnifiedIdeograph82d0': '\xe8\x8b\x90',
'CjkUnifiedIdeograph82d1': '\xe8\x8b\x91',
'CjkUnifiedIdeograph82d2': '\xe8\x8b\x92',
'CjkUnifiedIdeograph82d3': '\xe8\x8b\x93',
'CjkUnifiedIdeograph82d4': '\xe8\x8b\x94',
'CjkUnifiedIdeograph82d5': '\xe8\x8b\x95',
'CjkUnifiedIdeograph82d6': '\xe8\x8b\x96',
'MalayalamLetterJa': '\xe0\xb4\x9c',
'CjkUnifiedIdeograph82d8': '\xe8\x8b\x98',
'CjkUnifiedIdeograph82d9': '\xe8\x8b\x99',
'UnicodeCharacter0xa6b9': '\xea\x9a\xb9',
'UnicodeCharacter0xa6b8': '\xea\x9a\xb8',
'ArabicLetterDdahalFinalForm': '\xef\xae\x83',
'UnicodeCharacter0xf547': '\xef\x95\x87',
'ArabicLigatureTehWithJeemWithMeemInitialForm': '\xef\xb5\x90',
'HangulSyllableKoenj': '\xec\xbe\xad',
'HangulSyllableBbaebs': '\xeb\xba\x8e',
'CjkUnifiedIdeograph93a5': '\xe9\x8e\xa5',
'CjkUnifiedIdeograph93a4': '\xe9\x8e\xa4',
'CjkUnifiedIdeograph93a7': '\xe9\x8e\xa7',
'CjkUnifiedIdeograph93a6': '\xe9\x8e\xa6',
'CjkUnifiedIdeograph93a1': '\xe9\x8e\xa1',
'CjkUnifiedIdeograph93a0': '\xe9\x8e\xa0',
'CjkUnifiedIdeograph4b68': '\xe4\xad\xa8',
'CjkUnifiedIdeograph4b69': '\xe4\xad\xa9',
'CjkUnifiedIdeograph4b66': '\xe4\xad\xa6',
'CjkUnifiedIdeograph4b67': '\xe4\xad\xa7',
'CjkUnifiedIdeograph4b64': '\xe4\xad\xa4',
'CjkUnifiedIdeograph4b65': '\xe4\xad\xa5',
'CjkUnifiedIdeograph4b62': '\xe4\xad\xa2',
'CjkUnifiedIdeograph4b63': '\xe4\xad\xa3',
'CjkUnifiedIdeograph4b60': '\xe4\xad\xa0',
'CjkUnifiedIdeograph4b61': '\xe4\xad\xa1',
'GreekCapitalLetterGamma': '\xce\x93',
'DevanagariDigitFive': '\xe0\xa5\xab',
'ArabicLigatureYehWithHamzaAboveWithAeFinalForm': '\xef\xaf\xad',
'GreekCapitalLetterAlphaWithDasiaAndProsgegrammeni': '\xe1\xbe\x89',
'HangulSyllableHeoss': '\xed\x97\x9c',
'SmallContainsAsMember': '\xe2\x88\x8d',
'ThaiCharacterToPatak': '\xe0\xb8\x8f',
'UnicodeCharacter0xa6bc': '\xea\x9a\xbc',
'CjkUnifiedIdeograph82da': '\xe8\x8b\x9a',
'CjkUnifiedIdeograph82db': '\xe8\x8b\x9b',
'CjkUnifiedIdeograph7b1e': '\xe7\xac\x9e',
'CjkUnifiedIdeograph82dd': '\xe8\x8b\x9d',
'CjkUnifiedIdeograph82de': '\xe8\x8b\x9e',
'CjkUnifiedIdeograph82df': '\xe8\x8b\x9f',
'UnicodeCharacter0xa6bd': '\xea\x9a\xbd',
'HangulSyllableJeoss': '\xec\xa0\x94',
'CjkUnifiedIdeograph9b2b': '\xe9\xac\xab',
'CjkUnifiedIdeograph90af': '\xe9\x82\xaf',
'TagbanwaLetterYa': '\xe1\x9d\xac',
'CjkUnifiedIdeograph90ad': '\xe9\x82\xad',
'CjkUnifiedIdeograph90ae': '\xe9\x82\xae',
'CjkUnifiedIdeograph90ab': '\xe9\x82\xab',
'CjkUnifiedIdeograph90ac': '\xe9\x82\xac',
'RightwardsArrowWithLoop': '\xe2\x86\xac',
'UnicodeCharacter0xf544': '\xef\x95\x84',
'UnicodeCharacter0xd9f3': '\xed\xa7\xb3',
'BuhidLetterDa': '\xe1\x9d\x87',
'CjkUnifiedIdeograph5d45': '\xe5\xb5\x85',
'TopParenthesis': '\xe2\x8f\x9c',
'CjkUnifiedIdeograph5d47': '\xe5\xb5\x87',
'CjkUnifiedIdeograph5d46': '\xe5\xb5\x86',
'CjkUnifiedIdeograph5d41': '\xe5\xb5\x81',
'CjkUnifiedIdeograph5d40': '\xe5\xb5\x80',
'CjkUnifiedIdeograph5d43': '\xe5\xb5\x83',
'CjkUnifiedIdeograph5d42': '\xe5\xb5\x82',
'LatinSmallLetterTWithHook': '\xc6\xad',
'SInTriangle': '\xe2\xa7\x8c',
'HangulSyllableCweoh': '\xec\xb7\x8b',
'UnicodeCharacter0xf002': '\xef\x80\x82',
'HangulSyllableCweoj': '\xec\xb7\x86',
'TaiLeLetterXa': '\xe1\xa5\x91',
'HangulSyllableCweol': '\xec\xb6\xb8',
'HangulSyllableCweom': '\xec\xb7\x80',
'CjkUnifiedIdeograph9504': '\xe9\x94\x84',
'UnicodeCharacter0xf003': '\xef\x80\x83',
'HangulSyllableCweob': '\xec\xb7\x81',
'HangulSyllableCweoc': '\xec\xb7\x87',
'HangulSyllableCweod': '\xec\xb6\xb7',
'UnicodeCharacter0xf000': '\xef\x80\x80',
'MultiplicationSignInDoubleCircle': '\xe2\xa8\xb7',
'UnicodeCharacter0xf001': '\xef\x80\x81',
'CjkUnifiedIdeograph7b7e': '\xe7\xad\xbe',
'Opposition': '\xe2\x98\x8d',
'HangulSyllableCweop': '\xec\xb7\x8a',
'UnicodeCharacter0xf006': '\xef\x80\x86',
'MalayalamLetterGha': '\xe0\xb4\x98',
'HangulSyllableCweot': '\xec\xb7\x89',
'UnicodeCharacter0xf007': '\xef\x80\x87',
'CjkUnifiedIdeograph5d4e': '\xe5\xb5\x8e',
'KannadaLetterPha': '\xe0\xb2\xab',
'LeftFiveEighthsBlock': '\xe2\x96\x8b',
'CjkUnifiedIdeograph5d4f': '\xe5\xb5\x8f',
'CjkUnifiedIdeograph5d4a': '\xe5\xb5\x8a',
'UnicodeCharacter0xf004': '\xef\x80\x84',
'CjkUnifiedIdeograph5d4c': '\xe5\xb5\x8c',
'CjkUnifiedIdeograph5d4b': '\xe5\xb5\x8b',
'UnicodeCharacter0xf005': '\xef\x80\x85',
'L': '\xce\x9b',
'Lightning': '\xe2\x98\x87',
'CjkUnifiedIdeograph90a6': '\xe9\x82\xa6',
'LatinSmallLetterCWithCedilla': '\xc3\xa7',
'CjkUnifiedIdeograph90a4': '\xe9\x82\xa4',
'CjkUnifiedIdeograph90a5': '\xe9\x82\xa5',
'CjkUnifiedIdeograph90a2': '\xe9\x82\xa2',
'CjkUnifiedIdeograph90a3': '\xe9\x82\xa3',
'CjkUnifiedIdeograph90a0': '\xe9\x82\xa0',
'CjkUnifiedIdeograph90a1': '\xe9\x82\xa1',
'HangulSyllableSswaenh': '\xec\x90\x96',
'CjkUnifiedIdeograph90a8': '\xe9\x82\xa8',
'CjkUnifiedIdeograph90a9': '\xe9\x82\xa9',
'EthiopicCombiningGeminationMark': '\xe1\x8d\x9f',
'HebrewLetterTav': '\xd7\xaa',
'CjkUnifiedIdeograph9506': '\xe9\x94\x86',
'LatinCapitalLetterPWithStroke': '\xe2\xb1\xa3',
'UnicodeCharacter0xd9f0': '\xed\xa7\xb0',
'HangulSyllableTyem': '\xed\x86\x94',
'HangulSyllableGgeols': '\xea\xbb\x88',
'CyrillicCapitalLetterEnWithDescender': '\xd2\xa2',
'LatinCapitalLetterCuatrillo': '\xea\x9c\xac',
'CjkUnifiedIdeograph9507': '\xe9\x94\x87',
'UnicodeCharacter0xe015': '\xee\x80\x95',
'YiSyllableRax': '\xea\x8f\x87',
'UnicodeCharacter0xe017': '\xee\x80\x97',
'UnicodeCharacter0xe010': '\xee\x80\x90',
'CjkRadicalNetTwo': '\xe2\xba\xb2',
'LatinSmallLetterXWithDiaeresis': '\xe1\xba\x8d',
'HangulSyllableRung': '\xeb\xa3\xbd',
'HangulSyllableRunh': '\xeb\xa3\xae',
'UnicodeCharacter0xda0c': '\xed\xa8\x8c',
'YiSyllableRap': '\xea\x8f\x89',
'UnicodeCharacter0xe018': '\xee\x80\x98',
'UnicodeCharacter0xe019': '\xee\x80\x99',
'YiSyllableRat': '\xea\x8f\x86',
'YiSyllableGgot': '\xea\x88\xa2',
'ArabicLetterAlefWasla': '\xd9\xb1',
'YiSyllableGgop': '\xea\x88\xa5',
'CjkCompatibilityIdeographfa8a': '\xef\xaa\x8a',
'PrecedesAboveEqualsSign': '\xe2\xaa\xb3',
'CjkUnifiedIdeograph9500': '\xe9\x94\x80',
'EastSyriacCross': '\xe2\x99\xb1',
'ZNotationDomainAntirestriction': '\xe2\xa9\xa4',
'HangulSyllableDyaels': '\xeb\x8d\x84',
'CjkUnifiedIdeograph9a48': '\xe9\xa9\x88',
'CjkUnifiedIdeograph9a49': '\xe9\xa9\x89',
'CyrillicSmallLetterDzze': '\xea\x9a\x89',
'CjkUnifiedIdeograph9a44': '\xe9\xa9\x84',
'CjkUnifiedIdeograph9a45': '\xe9\xa9\x85',
'CjkUnifiedIdeograph9a46': '\xe9\xa9\x86',
'CjkUnifiedIdeograph9a47': '\xe9\xa9\x87',
'CjkUnifiedIdeograph9a40': '\xe9\xa9\x80',
'CjkUnifiedIdeograph9a41': '\xe9\xa9\x81',
'CjkUnifiedIdeograph9a42': '\xe9\xa9\x82',
'CjkUnifiedIdeograph9a43': '\xe9\xa9\x83',
'CjkUnifiedIdeograph45fe': '\xe4\x97\xbe',
'CjkUnifiedIdeograph45fd': '\xe4\x97\xbd',
'HangulSyllableDyaelb': '\xeb\x8d\x83',
'CjkUnifiedIdeograph45ff': '\xe4\x97\xbf',
'CjkUnifiedIdeograph45fa': '\xe4\x97\xba',
'CjkUnifiedIdeograph45fc': '\xe4\x97\xbc',
'CjkUnifiedIdeograph45fb': '\xe4\x97\xbb',
'HangulSyllableDyaelh': '\xeb\x8d\x87',
'CjkUnifiedIdeograph9501': '\xe9\x94\x81',
'HangulSyllableDyaelm': '\xeb\x8d\x82',
'YiSyllableTiex': '\xea\x84\x9e',
'HangulSyllableKwibs': '\xed\x81\x86',
'LowerHalfInverseWhiteCircle': '\xe2\x97\x9b',
'KangxiRadicalScript': '\xe2\xbd\x82',
'HangulSyllableNyot': '\xeb\x88\x81',
'CjkUnifiedIdeograph4559': '\xe4\x95\x99',
'UnicodeCharacter0xe01d': '\xee\x80\x9d',
'UnicodeCharacter0xe01e': '\xee\x80\x9e',
'UnicodeCharacter0xe01f': '\xee\x80\x9f',
'UnicodeCharacter0xe01a': '\xee\x80\x9a',
'UnicodeCharacter0xe01b': '\xee\x80\x9b',
'UnicodeCharacter0xe01c': '\xee\x80\x9c',
'HangulSyllableKulm': '\xec\xbf\xaa',
'CjkUnifiedIdeograph9502': '\xe9\x94\x82',
'UnicodeCharacter0xf0de': '\xef\x83\x9e',
'CjkUnifiedIdeograph45f5': '\xe4\x97\xb5',
'CjkUnifiedIdeograph45f4': '\xe4\x97\xb4',
'CjkUnifiedIdeograph45f7': '\xe4\x97\xb7',
'CjkUnifiedIdeograph45f6': '\xe4\x97\xb6',
'CjkUnifiedIdeograph45f1': '\xe4\x97\xb1',
'CjkUnifiedIdeograph45f0': '\xe4\x97\xb0',
'CjkUnifiedIdeograph45f3': '\xe4\x97\xb3',
'CjkUnifiedIdeograph45f2': '\xe4\x97\xb2',
'LaoVowelSignAm': '\xe0\xba\xb3',
'CjkUnifiedIdeograph45f9': '\xe4\x97\xb9',
'CjkUnifiedIdeograph45f8': '\xe4\x97\xb8',
'CjkUnifiedIdeograph9c96': '\xe9\xb2\x96',
'UnicodeCharacter0xd9a8': '\xed\xa6\xa8',
'UnicodeCharacter0xd9a9': '\xed\xa6\xa9',
'CjkUnifiedIdeograph9503': '\xe9\x94\x83',
'CjkUnifiedIdeograph9a4e': '\xe9\xa9\x8e',
'CjkUnifiedIdeograph9a4f': '\xe9\xa9\x8f',
'RunicLetterShorttwigtyrT': '\xe1\x9b\x90',
'DevanagariLetterVocalicLl': '\xe0\xa5\xa1',
'CjkUnifiedIdeograph9a4a': '\xe9\xa9\x8a',
'CjkUnifiedIdeograph9a4b': '\xe9\xa9\x8b',
'CjkUnifiedIdeograph9a4c': '\xe9\xa9\x8c',
'CjkUnifiedIdeograph5452': '\xe5\x91\x92',
'CjkUnifiedIdeograph5453': '\xe5\x91\x93',
'CjkUnifiedIdeograph5450': '\xe5\x91\x90',
'CjkUnifiedIdeograph5451': '\xe5\x91\x91',
'CjkUnifiedIdeograph5456': '\xe5\x91\x96',
'CjkUnifiedIdeograph5457': '\xe5\x91\x97',
'CjkUnifiedIdeograph5454': '\xe5\x91\x94',
'CjkUnifiedIdeograph5455': '\xe5\x91\x95',
'CjkUnifiedIdeograph917a': '\xe9\x85\xba',
'YiSyllableLep': '\xea\x87\x8d',
'CjkUnifiedIdeograph5458': '\xe5\x91\x98',
'CjkUnifiedIdeograph5459': '\xe5\x91\x99',
'CjkUnifiedIdeograph917e': '\xe9\x85\xbe',
'CjkUnifiedIdeograph87d7': '\xe8\x9f\x97',
'CjkUnifiedIdeograph917f': '\xe9\x85\xbf',
'UnicodeCharacter0xe6c3': '\xee\x9b\x83',
'HangulSyllableKulg': '\xec\xbf\xa9',
'ArabicLetterThehFinalForm': '\xef\xba\x9a',
'HangulSyllableReoh': '\xeb\xa0\x87',
'BalineseMusicalSymbolDeng': '\xe1\xad\xa2',
'CjkUnifiedIdeograph87d0': '\xe8\x9f\x90',
'LatinEpigraphicLetterReversedF': '\xea\x9f\xbb',
'CjkUnifiedIdeograph87d3': '\xe8\x9f\x93',
'LatinSmallLetterInsularG': '\xe1\xb5\xb9',
'HexagramForAbundance': '\xe4\xb7\xb6',
'CjkUnifiedIdeograph87d2': '\xe8\x9f\x92',
'CjkUnifiedIdeograph69eb': '\xe6\xa7\xab',
'CjkUnifiedIdeograph69ec': '\xe6\xa7\xac',
'KhmerSymbolBeiRoc': '\xe1\xa7\xb3',
'CjkUnifiedIdeograph69ea': '\xe6\xa7\xaa',
'CjkUnifiedIdeograph69ef': '\xe6\xa7\xaf',
'LatinSmallLetterInsularT': '\xea\x9e\x87',
'ArabicLetterDalWithInvertedV': '\xdb\xae',
'CjkUnifiedIdeograph69ee': '\xe6\xa7\xae',
'ArabicLigatureBehWithHahIsolatedForm': '\xef\xb0\x86',
'UnicodeCharacter0xeef5': '\xee\xbb\xb5',
'CjkUnifiedIdeograph71d4': '\xe7\x87\x94',
'KayahLiDigitThree': '\xea\xa4\x83',
'UnicodeCharacter0xf00b': '\xef\x80\x8b',
'SquareOriginalOfOrNotEqualTo': '\xe2\x8b\xa5',
'CjkUnifiedIdeograph87d9': '\xe8\x9f\x99',
'LaoLetterFoTam': '\xe0\xba\x9d',
'UnicodeCharacter0xa7f8': '\xea\x9f\xb8',
'UnicodeCharacter0xe4fc': '\xee\x93\xbc',
'CjkUnifiedIdeograph87d8': '\xe8\x9f\x98',
'CjkUnifiedIdeograph545b': '\xe5\x91\x9b',
'CjkUnifiedIdeograph545c': '\xe5\x91\x9c',
'CjkUnifiedIdeograph545a': '\xe5\x91\x9a',
'CjkUnifiedIdeograph545f': '\xe5\x91\x9f',
'HangulSyllablePeugg': '\xed\x94\x86',
'CjkUnifiedIdeograph545d': '\xe5\x91\x9d',
'CjkUnifiedIdeograph545e': '\xe5\x91\x9e',
'CjkUnifiedIdeograph9171': '\xe9\x85\xb1',
'CjkUnifiedIdeograph9170': '\xe9\x85\xb0',
'CjkUnifiedIdeograph9173': '\xe9\x85\xb3',
'CjkUnifiedIdeograph71d1': '\xe7\x87\x91',
'CjkUnifiedIdeograph9175': '\xe9\x85\xb5',
'CjkUnifiedIdeograph9174': '\xe9\x85\xb4',
'CjkUnifiedIdeograph9177': '\xe9\x85\xb7',
'CjkUnifiedIdeograph9176': '\xe9\x85\xb6',
'CjkUnifiedIdeograph69e2': '\xe6\xa7\xa2',
'ArmenianCapitalLetterOh': '\xd5\x95',
'CjkUnifiedIdeograph69e0': '\xe6\xa7\xa0',
'CjkUnifiedIdeograph69e1': '\xe6\xa7\xa1',
'CjkUnifiedIdeograph69e6': '\xe6\xa7\xa6',
'CjkUnifiedIdeograph69e7': '\xe6\xa7\xa7',
'CjkUnifiedIdeograph69e4': '\xe6\xa7\xa4',
'CjkUnifiedIdeograph55de': '\xe5\x97\x9e',
'CjkUnifiedIdeograph69e8': '\xe6\xa7\xa8',
'CjkUnifiedIdeograph69e9': '\xe6\xa7\xa9',
'oiint': '\xe2\x88\xaf',
'PrecedesAboveNotEqualTo': '\xe2\xaa\xb5',
'UnicodeCharacter0xd993': '\xed\xa6\x93',
'CjkUnifiedIdeograph4454': '\xe4\x91\x94',
'KannadaVowelSignEe': '\xe0\xb3\x87',
'UnicodeCharacter0xe6fc': '\xee\x9b\xbc',
'UnicodeCharacter0xe6fb': '\xee\x9b\xbb',
'HangulSyllableSsyeot': '\xec\x8e\xb9',
'HangulSyllableSsyeos': '\xec\x8e\xb3',
'HangulSyllableSsyeop': '\xec\x8e\xba',
'UnicodeCharacter0xe6fd': '\xee\x9b\xbd',
'EthiopicSyllableCu': '\xe1\x89\xb9',
'CjkUnifiedIdeograph9508': '\xe9\x94\x88',
'UnicodeCharacter0xd8be': '\xed\xa2\xbe',
'CjkRadicalRepeat': '\xe2\xba\x80',
'HangulSyllableSsyeod': '\xec\x8e\xa7',
'LongLeftwardsArrow': '\xe2\x9f\xb5',
'HangulSyllableSsyeob': '\xec\x8e\xb1',
'CanadianSyllabicsWestcreeRi': '\xe1\x96\x8b',
'EthiopicSyllableCo': '\xe1\x89\xbe',
'HangulSyllableSsyeon': '\xec\x8e\xa4',
'EthiopicSyllableCa': '\xe1\x89\xb8',
'HangulSyllableSsyeol': '\xec\x8e\xa8',
'HangulSyllableSsyeom': '\xec\x8e\xb0',
'HangulSyllableSsyeoj': '\xec\x8e\xb6',
'EthiopicSyllableCe': '\xe1\x89\xbd',
'HangulSyllableSsyeoh': '\xec\x8e\xbb',
'Bullet': '\xe2\x80\xa2',
'SaurashtraLetterNga': '\xea\xa2\x96',
'CjkUnifiedIdeograph928c': '\xe9\x8a\x8c',
'CjkUnifiedIdeograph928b': '\xe9\x8a\x8b',
'CjkUnifiedIdeograph928a': '\xe9\x8a\x8a',
'BengaliVowelSignAu': '\xe0\xa7\x8c',
'CjkUnifiedIdeograph928e': '\xe9\x8a\x8e',
'CjkUnifiedIdeograph928d': '\xe9\x8a\x8d',
'CjkUnifiedIdeograph446a': '\xe4\x91\xaa',
'CjkUnifiedIdeograph446b': '\xe4\x91\xab',
'CjkUnifiedIdeograph446c': '\xe4\x91\xac',
'CjkUnifiedIdeograph446d': '\xe4\x91\xad',
'CjkUnifiedIdeograph446e': '\xe4\x91\xae',
'CjkUnifiedIdeograph446f': '\xe4\x91\xaf',
'CjkUnifiedIdeograph68b2': '\xe6\xa2\xb2',
'CjkUnifiedIdeograph68b3': '\xe6\xa2\xb3',
'ZNotationBagMembership': '\xe2\x8b\xbf',
'GreekSmallLetterKappa': '\xce\xba',
'CjkUnifiedIdeograph68b0': '\xe6\xa2\xb0',
'UnicodeCharacter0xe6f3': '\xee\x9b\xb3',
'UnicodeCharacter0xe6f2': '\xee\x9b\xb2',
'UnicodeCharacter0xe6f1': '\xee\x9b\xb1',
'CjkUnifiedIdeograph68b1': '\xe6\xa2\xb1',
'UnicodeCharacter0xe6f7': '\xee\x9b\xb7',
'UnicodeCharacter0xe6f6': '\xee\x9b\xb6',
'UnicodeCharacter0xe6f5': '\xee\x9b\xb5',
'UnicodeCharacter0xe6f4': '\xee\x9b\xb4',
'CjkUnifiedIdeograph68b6': '\xe6\xa2\xb6',
'UnicodeCharacter0xe6f9': '\xee\x9b\xb9',
'UnicodeCharacter0xe6f8': '\xee\x9b\xb8',
'CjkUnifiedIdeograph68b7': '\xe6\xa2\xb7',
'CjkUnifiedIdeograph4468': '\xe4\x91\xa8',
'CjkUnifiedIdeograph4469': '\xe4\x91\xa9',
'CjkUnifiedIdeograph9281': '\xe9\x8a\x81',
'CjkUnifiedIdeograph9280': '\xe9\x8a\x80',
'CopticSmallLetterLaula': '\xe2\xb2\x97',
'CjkUnifiedIdeograph68b4': '\xe6\xa2\xb4',
'CjkUnifiedIdeograph9285': '\xe9\x8a\x85',
'CjkUnifiedIdeograph9284': '\xe9\x8a\x84',
'CjkUnifiedIdeograph4460': '\xe4\x91\xa0',
'CjkUnifiedIdeograph4461': '\xe4\x91\xa1',
'CjkUnifiedIdeograph4462': '\xe4\x91\xa2',
'CjkUnifiedIdeograph4463': '\xe4\x91\xa3',
'CjkUnifiedIdeograph4464': '\xe4\x91\xa4',
'CjkUnifiedIdeograph4465': '\xe4\x91\xa5',
'CjkUnifiedIdeograph4466': '\xe4\x91\xa6',
'CjkUnifiedIdeograph4467': '\xe4\x91\xa7',
'CjkUnifiedIdeograph9cb0': '\xe9\xb2\xb0',
'UnicodeCharacter0x83c': '\xe0\xa0\xbc',
'HexagramForBitingThrough': '\xe4\xb7\x94',
'LatinCross': '\xe2\x9c\x9d',
'CjkUnifiedIdeograph68b8': '\xe6\xa2\xb8',
'CjkUnifiedIdeograph3850': '\xe3\xa1\x90',
'CjkUnifiedIdeograph3851': '\xe3\xa1\x91',
'CjkUnifiedIdeograph3852': '\xe3\xa1\x92',
'CjkUnifiedIdeograph3853': '\xe3\xa1\x93',
'CjkUnifiedIdeograph3854': '\xe3\xa1\x94',
'CjkUnifiedIdeograph3855': '\xe3\xa1\x95',
'CjkUnifiedIdeograph3856': '\xe3\xa1\x96',
'CjkUnifiedIdeograph3857': '\xe3\xa1\x97',
'CjkUnifiedIdeograph3858': '\xe3\xa1\x98',
'CjkUnifiedIdeograph3859': '\xe3\xa1\x99',
'UnicodeCharacter0xec68': '\xee\xb1\xa8',
'UnicodeCharacter0xec69': '\xee\xb1\xa9',
'HangulSyllableCaenh': '\xec\xb1\x8a',
'RunicSinglePunctuation': '\xe1\x9b\xab',
'CjkUnifiedIdeograph55c5': '\xe5\x97\x85',
'CjkUnifiedIdeograph55c4': '\xe5\x97\x84',
'CjkUnifiedIdeograph55c7': '\xe5\x97\x87',
'CjkUnifiedIdeograph55c6': '\xe5\x97\x86',
'CjkUnifiedIdeograph55c1': '\xe5\x97\x81',
'UnicodeCharacter0x83d': '\xe0\xa0\xbd',
'CjkUnifiedIdeograph55c3': '\xe5\x97\x83',
'CjkUnifiedIdeograph55c2': '\xe5\x97\x82',
'CjkUnifiedIdeograph4438': '\xe4\x90\xb8',
'CjkUnifiedIdeograph55c9': '\xe5\x97\x89',
'CjkUnifiedIdeograph55c8': '\xe5\x97\x88',
'CjkUnifiedIdeograph982f': '\xe9\xa0\xaf',
'CjkUnifiedIdeograph982e': '\xe9\xa0\xae',
'CjkUnifiedIdeograph982d': '\xe9\xa0\xad',
'CjkUnifiedIdeograph982c': '\xe9\xa0\xac',
'CjkUnifiedIdeograph982b': '\xe9\xa0\xab',
'CjkUnifiedIdeograph982a': '\xe9\xa0\xaa',
'VaiSyllableNdee': '\xea\x94\x95',
'YiSyllableXyrx': '\xea\x91\xaf',
'CjkUnifiedIdeograph533f': '\xe5\x8c\xbf',
'CjkUnifiedIdeograph533e': '\xe5\x8c\xbe',
'CjkUnifiedIdeograph533d': '\xe5\x8c\xbd',
'CjkUnifiedIdeograph533c': '\xe5\x8c\xbc',
'CjkUnifiedIdeograph533b': '\xe5\x8c\xbb',
'CjkUnifiedIdeograph533a': '\xe5\x8c\xba',
'KangxiRadicalBone': '\xe2\xbe\xbb',
'CjkUnifiedIdeograph55ce': '\xe5\x97\x8e',
'CjkUnifiedIdeograph55cd': '\xe5\x97\x8d',
'CjkUnifiedIdeograph55cf': '\xe5\x97\x8f',
'CjkUnifiedIdeograph55ca': '\xe5\x97\x8a',
'CjkUnifiedIdeograph55cc': '\xe5\x97\x8c',
'CjkUnifiedIdeograph55cb': '\xe5\x97\x8b',
'LaoVowelSignY': '\xe0\xba\xb6',
'TildeWithDotAbove': '\xe2\xb8\x9e',
'ObjectReplacementCharacter': '\xef\xbf\xbc',
'HangulSyllableTaec': '\xed\x83\xb3',
'CjkUnifiedIdeograph385a': '\xe3\xa1\x9a',
'CjkUnifiedIdeograph385b': '\xe3\xa1\x9b',
'CjkUnifiedIdeograph385c': '\xe3\xa1\x9c',
'CjkUnifiedIdeograph385d': '\xe3\xa1\x9d',
'CjkUnifiedIdeograph385e': '\xe3\xa1\x9e',
'CjkUnifiedIdeograph385f': '\xe3\xa1\x9f',
'HangulSyllableTaed': '\xed\x83\xa3',
'HangulSyllableTaek': '\xed\x83\xb4',
'HangulSyllableTaej': '\xed\x83\xb2',
'UnicodeCharacter0xa7f1': '\xea\x9f\xb1',
'AplFunctionalSymbolCircleStile': '\xe2\x8c\xbd',
'HangulSyllableTaen': '\xed\x83\xa0',
'HangulSyllableDdyogs': '\xeb\x9a\x83',
'HangulSyllableTael': '\xed\x83\xa4',
'CjkUnifiedIdeograph5337': '\xe5\x8c\xb7',
'CjkUnifiedIdeograph5336': '\xe5\x8c\xb6',
'CjkUnifiedIdeograph5335': '\xe5\x8c\xb5',
'CjkUnifiedIdeograph5334': '\xe5\x8c\xb4',
'CjkUnifiedIdeograph5333': '\xe5\x8c\xb3',
'CjkUnifiedIdeograph5332': '\xe5\x8c\xb2',
'CjkUnifiedIdeograph5331': '\xe5\x8c\xb1',
'CjkUnifiedIdeograph5330': '\xe5\x8c\xb0',
'HangulSyllableGgyeogs': '\xea\xbb\xb7',
'CjkUnifiedIdeograph5339': '\xe5\x8c\xb9',
'CjkUnifiedIdeograph5338': '\xe5\x8c\xb8',
'CjkUnifiedIdeograph9827': '\xe9\xa0\xa7',
'CjkUnifiedIdeograph9826': '\xe9\xa0\xa6',
'CjkUnifiedIdeograph9825': '\xe9\xa0\xa5',
'CjkUnifiedIdeograph9824': '\xe9\xa0\xa4',
'CjkUnifiedIdeograph9823': '\xe9\xa0\xa3',
'CjkUnifiedIdeograph9822': '\xe9\xa0\xa2',
'CjkUnifiedIdeograph9821': '\xe9\xa0\xa1',
'CjkUnifiedIdeograph9820': '\xe9\xa0\xa0',
'LaoVowelSignA': '\xe0\xba\xb0',
'CjkUnifiedIdeograph9829': '\xe9\xa0\xa9',
'MongolianLetterAliGaliDamaru': '\xe1\xa2\x82',
'ArabicLetterBehFinalForm': '\xef\xba\x90',
'TamilOm': '\xe0\xaf\x90',
'HangulSyllableByiss': '\xeb\xb8\xbc',
'UnicodeCharacter0xde6f': '\xed\xb9\xaf',
'MeasuredAngleOpeningLeft': '\xe2\xa6\x9b',
'UnicodeCharacter0xee87': '\xee\xba\x87',
'CjkUnifiedIdeograph673d': '\xe6\x9c\xbd',
'KangxiRadicalHair': '\xe2\xbe\xbd',
'GlagoliticCapitalLetterZhivete': '\xe2\xb0\x86',
'CjkUnifiedIdeograph673c': '\xe6\x9c\xbc',
'UnicodeCharacter0xa7f7': '\xea\x9f\xb7',
'CjkUnifiedIdeograph673a': '\xe6\x9c\xba',
'CjkUnifiedIdeograph55d0': '\xe5\x97\x90',
'HangulSyllableDdyunj': '\xeb\x9c\x91',
'CjkUnifiedIdeograph99fd': '\xe9\xa7\xbd',
'UnicodeCharacter0xee88': '\xee\xba\x88',
'UnicodeCharacter0xee89': '\xee\xba\x89',
'MalayalamLetterUu': '\xe0\xb4\x8a',
'CjkUnifiedIdeograph99fb': '\xe9\xa7\xbb',
'HangulSyllableBbwaegs': '\xeb\xbd\xbb',
'LatinCapitalLetterEWithOgonek': '\xc4\x98',
'UnicodeCharacter0xa7f4': '\xea\x9f\xb4',
'HangulSyllableJyeot': '\xec\xa1\x91',
'KangxiRadicalFeather': '\xe2\xbd\xbb',
'LessthanAboveSimilarOrEqual': '\xe2\xaa\x8d',
'HangulSyllableBbwaegg': '\xeb\xbd\xba',
'CjkUnifiedIdeograph6736': '\xe6\x9c\xb6',
'CjkUnifiedIdeograph6737': '\xe6\x9c\xb7',
'CjkUnifiedIdeograph6734': '\xe6\x9c\xb4',
'CjkUnifiedIdeograph6735': '\xe6\x9c\xb5',
'CjkUnifiedIdeograph6732': '\xe6\x9c\xb2',
'CjkUnifiedIdeograph6733': '\xe6\x9c\xb3',
'CjkUnifiedIdeograph6730': '\xe6\x9c\xb0',
'CjkUnifiedIdeograph6731': '\xe6\x9c\xb1',
'KangxiRadicalWheat': '\xe2\xbf\x86',
'ArmenianSmallLetterZhe': '\xd5\xaa',
'CjkUnifiedIdeograph6738': '\xe6\x9c\xb8',
'CjkUnifiedIdeograph6739': '\xe6\x9c\xb9',
'YiSyllableDot': '\xea\x84\x8d',
'YiSyllableDop': '\xea\x84\x90',
'CjkUnifiedIdeograph87dd': '\xe8\x9f\x9d',
'HangulSyllableSseugs': '\xec\x93\xb3',
'HebrewLetterDaletWithDagesh': '\xef\xac\xb3',
'FiveDotMark': '\xe2\xb8\xad',
'YiSyllableDox': '\xea\x84\x8e',
'HangulSyllableJeonj': '\xec\xa0\x85',
'HangulSyllableJeonh': '\xec\xa0\x86',
'MediumWhiteCircle': '\xe2\x9a\xaa',
'UnicodeCharacter0xe6cd': '\xee\x9b\x8d',
'HangulSyllableSseugg': '\xec\x93\xb2',
'CjkUnifiedIdeograph3a9a': '\xe3\xaa\x9a',
'UnicodeCharacter0xeba': '\xe0\xba\xba',
'CjkUnifiedIdeograph3a9c': '\xe3\xaa\x9c',
'CjkUnifiedIdeograph3a9b': '\xe3\xaa\x9b',
'CjkUnifiedIdeograph3a9e': '\xe3\xaa\x9e',
'CjkUnifiedIdeograph3a9d': '\xe3\xaa\x9d',
'UnicodeCharacter0xebf': '\xe0\xba\xbf',
'CjkUnifiedIdeograph3a9f': '\xe3\xaa\x9f',
'CjkUnifiedIdeograph3d49': '\xe3\xb5\x89',
'CjkUnifiedIdeograph3d48': '\xe3\xb5\x88',
'CjkUnifiedIdeograph3d47': '\xe3\xb5\x87',
'CjkUnifiedIdeograph3d46': '\xe3\xb5\x86',
'CjkUnifiedIdeograph3d45': '\xe3\xb5\x85',
'CjkUnifiedIdeograph3d44': '\xe3\xb5\x84',
'CjkUnifiedIdeograph3d43': '\xe3\xb5\x83',
'CjkUnifiedIdeograph3d42': '\xe3\xb5\x82',
'CjkUnifiedIdeograph3d41': '\xe3\xb5\x81',
'CjkUnifiedIdeograph3d40': '\xe3\xb5\x80',
'GujaratiVowelSignVocalicLl': '\xe0\xab\xa3',
'ArabicLigatureSadWithRehIsolatedForm': '\xef\xb4\x8f',
'CjkUnifiedIdeograph71db': '\xe7\x87\x9b',
'HangulSyllableGogs': '\xea\xb3\xa3',
'LatinSmallLetterKWithDotBelow': '\xe1\xb8\xb3',
'CjkUnifiedIdeograph72fc': '\xe7\x8b\xbc',
'CjkUnifiedIdeograph72fb': '\xe7\x8b\xbb',
'CjkUnifiedIdeograph72fa': '\xe7\x8b\xba',
'CjkUnifiedIdeograph72ff': '\xe7\x8b\xbf',
'CjkUnifiedIdeograph72fe': '\xe7\x8b\xbe',
'CjkUnifiedIdeograph72fd': '\xe7\x8b\xbd',
'HangulSyllableGogg': '\xea\xb3\xa2',
'KhmerSymbolPrambuonKoet': '\xe1\xa7\xa9',
'HangulSyllableOelb': '\xec\x9a\x83',
'AplFunctionalSymbolQuadLeftwardsArrow': '\xe2\x8d\x87',
'CjkUnifiedIdeograph3d4f': '\xe3\xb5\x8f',
'CjkUnifiedIdeograph3d4e': '\xe3\xb5\x8e',
'CjkUnifiedIdeograph3d4d': '\xe3\xb5\x8d',
'CjkUnifiedIdeograph3d4c': '\xe3\xb5\x8c',
'CjkUnifiedIdeograph3d4b': '\xe3\xb5\x8b',
'CjkUnifiedIdeograph3d4a': '\xe3\xb5\x8a',
'CjkUnifiedIdeograph3a99': '\xe3\xaa\x99',
'CjkUnifiedIdeograph3a98': '\xe3\xaa\x98',
'RunicLetterAnsuzA': '\xe1\x9a\xa8',
'CjkUnifiedIdeograph3a91': '\xe3\xaa\x91',
'CjkUnifiedIdeograph3a90': '\xe3\xaa\x90',
'CjkUnifiedIdeograph3a93': '\xe3\xaa\x93',
'CjkUnifiedIdeograph3a92': '\xe3\xaa\x92',
'CjkUnifiedIdeograph3a95': '\xe3\xaa\x95',
'CjkUnifiedIdeograph3a94': '\xe3\xaa\x94',
'CjkUnifiedIdeograph3a97': '\xe3\xaa\x97',
'CjkUnifiedIdeograph3a96': '\xe3\xaa\x96',
'BopomofoLetterEh': '\xe3\x84\x9d',
'BopomofoLetterEi': '\xe3\x84\x9f',
'CjkUnifiedIdeograph72f1': '\xe7\x8b\xb1',
'CjkUnifiedIdeograph72f0': '\xe7\x8b\xb0',
'CjkUnifiedIdeograph72f7': '\xe7\x8b\xb7',
'EthiopicSyllableDo': '\xe1\x8b\xb6',
'BopomofoLetterEn': '\xe3\x84\xa3',
'CjkUnifiedIdeograph72f4': '\xe7\x8b\xb4',
'CjkUnifiedIdeograph72f9': '\xe7\x8b\xb9',
'CjkUnifiedIdeograph72f8': '\xe7\x8b\xb8',
'BopomofoLetterEe': '\xe3\x86\xa4',
'HangulSyllableJung': '\xec\xa4\x91',
'PhagspaLetterPha': '\xea\xa1\x8d',
'AplFunctionalSymbolDeltaUnderbar': '\xe2\x8d\x99',
'NkoLetterNaWoloso': '\xdf\xa0',
'BopomofoLetterEr': '\xe3\x84\xa6',
'HangulSyllableSess': '\xec\x85\x8c',
'ArabicLigatureTehWithHahWithMeemInitialForm': '\xef\xb5\x93',
'SundaneseVowelSignPaneuleung': '\xe1\xae\xa9',
'UnicodeCharacter0xdff5': '\xed\xbf\xb5',
'HangulSyllableSsyin': '\xec\x94\x90',
'UnicodeCharacter0xdff6': '\xed\xbf\xb6',
'CircledIdeographCorrect': '\xe3\x8a\xa3',
'HangulSyllableBbilb': '\xec\x82\x9b',
'UnicodeCharacter0xd89f': '\xed\xa2\x9f',
'CanadianSyllabicsWestcreeWo': '\xe1\x90\x93',
'HangulSyllableSsyil': '\xec\x94\x94',
'HalfwidthIdeographicFullStop': '\xef\xbd\xa1',
'HangulSyllableSsyim': '\xec\x94\x9c',
'HangulSyllableSsyij': '\xec\x94\xa2',
'HangulSyllableSsyik': '\xec\x94\xa4',
'UnicodeCharacter0xe7ca': '\xee\x9f\x8a',
'CjkUnifiedIdeograph68bb': '\xe6\xa2\xbb',
'CjkUnifiedIdeograph7f3b': '\xe7\xbc\xbb',
'CjkUnifiedIdeograph7f3c': '\xe7\xbc\xbc',
'CjkUnifiedIdeograph7f3d': '\xe7\xbc\xbd',
'CjkUnifiedIdeograph7f3e': '\xe7\xbc\xbe',
'CjkUnifiedIdeograph7f3f': '\xe7\xbc\xbf',
'CjkUnifiedIdeograph68bc': '\xe6\xa2\xbc',
'GlagoliticCapitalLetterNashi': '\xe2\xb0\x90',
'HangulSyllableDubs': '\xeb\x91\xa2',
'VulgarFractionOneThird': '\xe2\x85\x93',
'HangulSyllableDdyenj': '\xeb\x97\xb9',
'ModifierLetterCapitalB': '\xe1\xb4\xae',
'ModifierLetterCapitalE': '\xe1\xb4\xb1',
'HangulSyllableDdyeng': '\xeb\x98\x89',
'CjkUnifiedIdeograph443b': '\xe4\x90\xbb',
'HangulSyllableTun': '\xed\x88\xb0',
'HangulSyllableBbilh': '\xec\x82\x9f',
'HangulSyllableGgwebs': '\xeb\x80\x82',
'CjkUnifiedIdeograph3809': '\xe3\xa0\x89',
'HangulSyllablePwag': '\xed\x90\x89',
'HangulSyllableSsyinj': '\xec\x94\x91',
'HangulSyllableSsyib': '\xec\x94\x9d',
'HangulSyllableSsying': '\xec\x94\xa1',
'CopticCapitalLetterOldCopticHori': '\xe2\xb3\x8c',
'BalineseLetterPa': '\xe1\xac\xa7',
'HangulSyllablePwal': '\xed\x90\x90',
'HangulSyllableSsyic': '\xec\x94\xa3',
'CjkUnifiedIdeograph3807': '\xe3\xa0\x87',
'HangulSyllablePwas': '\xed\x90\x9b',
'CjkUnifiedIdeograph99f0': '\xe9\xa7\xb0',
'CjkUnifiedIdeograph3806': '\xe3\xa0\x86',
'CjkUnifiedIdeograph6402': '\xe6\x90\x82',
'HangulSyllableNoeh': '\xeb\x87\xa7',
'ModifierLetterCapitalK': '\xe1\xb4\xb7',
'HangulSyllableBbils': '\xec\x82\x9c',
'CjkUnifiedIdeograph3804': '\xe3\xa0\x84',
'CjkUnifiedIdeograph3803': '\xe3\xa0\x83',
'HangulSyllableJjaeb': '\xec\xa8\x89',
'UnicodeCharacter0x837': '\xe0\xa0\xb7',
'CjkUnifiedIdeograph7f30': '\xe7\xbc\xb0',
'CjkUnifiedIdeograph7f31': '\xe7\xbc\xb1',
'CjkUnifiedIdeograph7f32': '\xe7\xbc\xb2',
'CjkUnifiedIdeograph7f33': '\xe7\xbc\xb3',
'CjkUnifiedIdeograph7f34': '\xe7\xbc\xb4',
'CjkUnifiedIdeograph3801': '\xe3\xa0\x81',
'CjkUnifiedIdeograph7f36': '\xe7\xbc\xb6',
'CjkUnifiedIdeograph7f37': '\xe7\xbc\xb7',
'CjkUnifiedIdeograph7f38': '\xe7\xbc\xb8',
'CjkUnifiedIdeograph7f39': '\xe7\xbc\xb9',
'PresentationFormForVerticalLeftCornerBracket': '\xef\xb9\x81',
'UnicodeCharacter0x835': '\xe0\xa0\xb5',
'SupersetPrecedingSolidus': '\xe2\x9f\x89',
'CjkUnifiedIdeograph5f0c': '\xe5\xbc\x8c',
'CjkUnifiedIdeograph5f0b': '\xe5\xbc\x8b',
'CjkUnifiedIdeograph5f0a': '\xe5\xbc\x8a',
'TibetanLetterDha': '\xe0\xbd\x92',
'PeaceSymbol': '\xe2\x98\xae',
'CjkUnifiedIdeograph5f0e': '\xe5\xbc\x8e',
'CjkUnifiedIdeograph5f0d': '\xe5\xbc\x8d',
'UnicodeCharacter0xe951': '\xee\xa5\x91',
'YiSyllableHlu': '\xea\x86\xad',
'HangulSyllableBbilt': '\xec\x82\x9d',
'YiSyllableHlo': '\xea\x86\xa6',
'HangulSyllableJjaeh': '\xec\xa8\x93',
'YiSyllableHli': '\xea\x86\x99',
'YiSyllableHle': '\xea\x86\xa9',
'FourBalloonspokedAsterisk': '\xe2\x9c\xa3',
'HangulSyllableSsyit': '\xec\x94\xa5',
'YiSyllableHla': '\xea\x86\xa0',
'CjkUnifiedIdeograph7b8a': '\xe7\xae\x8a',
'UnicodeCharacter0xe033': '\xee\x80\xb3',
'CjkUnifiedIdeograph6c5e': '\xe6\xb1\x9e',
'ArabicLetterHighHamza': '\xd9\xb4',
'CjkUnifiedIdeograph7b8c': '\xe7\xae\x8c',
'HangulSyllableSsyis': '\xec\x94\x9f',
'UnicodeCharacter0xe270': '\xee\x89\xb0',
'HangulSyllableSsyip': '\xec\x94\xa6',
'UnicodeCharacter0xf56c': '\xef\x95\xac',
'CjkUnifiedIdeograph7b8d': '\xe7\xae\x8d',
'PresentationFormForVerticalQuestionMark': '\xef\xb8\x96',
'ShadowedWhiteStar': '\xe2\x9c\xb0',
'BellSymbol': '\xe2\x8d\xbe',
'UnicodeCharacter0xf56e': '\xef\x95\xae',
'CjkUnifiedIdeograph7b8f': '\xe7\xae\x8f',
'UnicodeCharacter0xe275': '\xee\x89\xb5',
'CjkUnifiedIdeograph5f03': '\xe5\xbc\x83',
'CjkUnifiedIdeograph5f02': '\xe5\xbc\x82',
'CjkUnifiedIdeograph5f01': '\xe5\xbc\x81',
'CjkUnifiedIdeograph5f00': '\xe5\xbc\x80',
'CjkUnifiedIdeograph5f07': '\xe5\xbc\x87',
'CjkUnifiedIdeograph5f06': '\xe5\xbc\x86',
'CjkUnifiedIdeograph5f05': '\xe5\xbc\x85',
'CjkUnifiedIdeograph5f04': '\xe5\xbc\x84',
'CjkUnifiedIdeograph5f09': '\xe5\xbc\x89',
'CjkUnifiedIdeograph5f08': '\xe5\xbc\x88',
'GreekCapitalLetterAlphaWithDasiaAndOxiaAndProsgegrammeni': '\xe1\xbe\x8d',
'HangulLetterYesieungsios': '\xe3\x86\x82',
'LatinCapitalLetterAWithBreveAndGrave': '\xe1\xba\xb0',
'UnicodeCharacter0xd9fd': '\xed\xa7\xbd',
'UnicodeCharacter0xe7ff': '\xee\x9f\xbf',
'UnicodeCharacter0xa69a': '\xea\x9a\x9a',
'UnicodeCharacter0xe1c5': '\xee\x87\x85',
'ArabicLetterRehWithSmallArabicLetterTahAndTwoDots': '\xdd\xb1',
'UnicodeCharacter0xe1c4': '\xee\x87\x84',
'HangulSyllableSwang': '\xec\x86\xbd',
'UnicodeCharacter0xdffc': '\xed\xbf\xbc',
'KayahLiDigitFive': '\xea\xa4\x85',
'CombiningMacron': '\xcc\x84',
'HangulSyllableSwanj': '\xec\x86\xad',
'HangulSyllableSwanh': '\xec\x86\xae',
'HangulSyllablePwiss': '\xed\x93\xa0',
'UnicodeCharacter0xe7fd': '\xee\x9f\xbd',
'HangulSyllableNal': '\xeb\x82\xa0',
'HangulSyllableTwilm': '\xed\x8a\x8a',
'ArabicLetterAlefWaslaFinalForm': '\xef\xad\x91',
'prod': '\xe2\x88\x8f',
'HangulSyllableSsup': '\xec\x91\xbe',
'nu': '\xce\xbd',
'HangulSyllableCess': '\xec\xb3\x88',
'UnicodeCharacter0xea0c': '\xee\xa8\x8c',
'HangulSyllableDyult': '\xeb\x93\x8d',
'PhagspaLetterCandrabindu': '\xea\xa1\xb3',
'UnicodeCharacter0xe7fb': '\xee\x9f\xbb',
'SubsetAboveSubset': '\xe2\xab\x95',
'ArabicLetterPehInitialForm': '\xef\xad\x98',
'CircledIdeographCongratulation': '\xe3\x8a\x97',
'UnicodeCharacter0xf3ef': '\xef\x8f\xaf',
'UnicodeCharacter0xe722': '\xee\x9c\xa2',
'UnicodeCharacter0xf3ed': '\xef\x8f\xad',
'ArabicindicCubeRoot': '\xd8\x86',
'UnicodeCharacter0xf3eb': '\xef\x8f\xab',
'UnicodeCharacter0xf3ec': '\xef\x8f\xac',
'UnicodeCharacter0xe723': '\xee\x9c\xa3',
'HangulSyllableGgyigg': '\xeb\x81\xa2',
'UnicodeCharacter0xe721': '\xee\x9c\xa1',
'imagj': '\xe2\x85\x89',
'UnicodeCharacter0xea0a': '\xee\xa8\x8a',
'HangulSyllableGgyigs': '\xeb\x81\xa3',
'YiSyllableRop': '\xea\x8f\x90',
'UnicodeCharacter0xe724': '\xee\x9c\xa4',
'UnicodeCharacter0xe725': '\xee\x9c\xa5',
'UnicodeCharacter0xecf1': '\xee\xb3\xb1',
'UnicodeCharacter0xea0f': '\xee\xa8\x8f',
'ArabicLigatureNoonWithYehIsolatedForm': '\xef\xb1\x90',
'UnicodeCharacter0xe728': '\xee\x9c\xa8',
'HangulSyllableDit': '\xeb\x94\xad',
'CjkUnifiedIdeograph7b81': '\xe7\xae\x81',
'UnicodeCharacter0xe729': '\xee\x9c\xa9',
'YiSyllableSuox': '\xea\x8c\x94',
'CjkUnifiedIdeograph7b80': '\xe7\xae\x80',
'LatinCapitalLetterOWithLongStrokeOverlay': '\xea\x9d\x8a',
'ArabicLigatureYehWithHehInitialForm': '\xef\xb3\x9e',
'CjkUnifiedIdeograph7b83': '\xe7\xae\x83',
'UnicodeCharacter0xf3e6': '\xef\x8f\xa6',
'UnicodeCharacter0xf3e7': '\xef\x8f\xa7',
'UnicodeCharacter0xf3e4': '\xef\x8f\xa4',
'CjkUnifiedIdeograph7b82': '\xe7\xae\x82',
'UnicodeCharacter0xf3e2': '\xef\x8f\xa2',
'HangulSyllableDyulp': '\xeb\x93\x8e',
'UnicodeCharacter0xf3e0': '\xef\x8f\xa0',
'UnicodeCharacter0xf3e1': '\xef\x8f\xa1',
'CjkUnifiedIdeograph7b85': '\xe7\xae\x85',
'RunicLetterShorttwighagallH': '\xe1\x9a\xbd',
'UnicodeCharacter0xe176': '\xee\x85\xb6',
'UnicodeCharacter0xf3e8': '\xef\x8f\xa8',
'CjkUnifiedIdeograph7b84': '\xe7\xae\x84',
'HangulSyllableBbeolm': '\xeb\xbb\x9a',
'CanadianSyllabicsWoodscreeTh': '\xe1\x96\x9f',
'CjkUnifiedIdeograph7b87': '\xe7\xae\x87',
'UnicodeCharacter0xef81': '\xee\xbe\x81',
'CjkUnifiedIdeograph7b86': '\xe7\xae\x86',
'UnicodeCharacter0xea0d': '\xee\xa8\x8d',
'EqualToOrGreaterthan': '\xe2\x8b\x9d',
'NkoLetterFa': '\xdf\x9d',
'ClosedSubset': '\xe2\xab\x8f',
'z': '\xce\xb6',
'LepchaVowelSignU': '\xe1\xb0\xaa',
'UnicodeCharacter0xe1cc': '\xee\x87\x8c',
'CjkUnifiedIdeograph8198': '\xe8\x86\x98',
'CjkUnifiedIdeograph8199': '\xe8\x86\x99',
'UnicodeCharacter0xe630': '\xee\x98\xb0',
'CjkUnifiedIdeograph8194': '\xe8\x86\x94',
'CjkUnifiedIdeograph6c58': '\xe6\xb1\x98',
'CjkUnifiedIdeograph8196': '\xe8\x86\x96',
'CjkUnifiedIdeograph8197': '\xe8\x86\x97',
'CjkUnifiedIdeograph8190': '\xe8\x86\x90',
'CjkUnifiedIdeograph8191': '\xe8\x86\x91',
'CjkUnifiedIdeograph8192': '\xe8\x86\x92',
'CjkUnifiedIdeograph8193': '\xe8\x86\x93',
'SuperscriptThree': '\xc2\xb3',
'UnicodeCharacter0xf8cd': '\xef\xa3\x8d',
'HangulSyllableTegg': '\xed\x85\x8e',
'CjkUnifiedIdeograph918a': '\xe9\x86\x8a',
'LightLeftTortoiseShellBracketOrnament': '\xe2\x9d\xb2',
'BraillePatternDots7': '\xe2\xa1\x80',
'UnicodeCharacter0xe172': '\xee\x85\xb2',
'UnicodeCharacter0xdb1a': '\xed\xac\x9a',
'CjkUnifiedIdeograph819d': '\xe8\x86\x9d',
'CjkUnifiedIdeograph819e': '\xe8\x86\x9e',
'CjkUnifiedIdeograph819f': '\xe8\x86\x9f',
'~==': '\xe2\x89\x85',
'CjkUnifiedIdeograph819a': '\xe8\x86\x9a',
'RightWhiteSquareBracket': '\xe3\x80\x9b',
'CjkUnifiedIdeograph819c': '\xe8\x86\x9c',
'CjkRadicalWaterOne': '\xe2\xba\xa1',
'UnicodeCharacter0xf568': '\xef\x95\xa8',
'UnicodeCharacter0xdf2f': '\xed\xbc\xaf',
'HangulSyllableDyulm': '\xeb\x93\x8a',
'UnicodeCharacter0xe180': '\xee\x86\x80',
'HangulSyllableGgeolp': '\xea\xbb\x8a',
'YiSyllableZzop': '\xea\x8b\x9e',
'BengaliLetterSa': '\xe0\xa6\xb8',
'YiSyllableZzox': '\xea\x8b\x9c',
'DevanagariSignAvagraha': '\xe0\xa4\xbd',
'HangulSyllableGyaeh': '\xea\xb1\xaf',
'HangulSyllableGyaej': '\xea\xb1\xaa',
'HangulSyllableGyaek': '\xea\xb1\xac',
'HangulSyllableGyael': '\xea\xb1\x9c',
'HangulSyllableGyaem': '\xea\xb1\xa4',
'HangulSyllableGyaen': '\xea\xb1\x98',
'HangulSyllableCyeolt': '\xec\xb3\x9d',
'HangulSyllableGyaeb': '\xea\xb1\xa5',
'HangulSyllableGyaec': '\xea\xb1\xab',
'HangulSyllableGyaed': '\xea\xb1\x9b',
'CjkUnifiedIdeograph8d08': '\xe8\xb4\x88',
'HangulSyllableGyaeg': '\xea\xb1\x95',
'HangulSyllableCyeolb': '\xec\xb3\x9b',
'HangulSyllableCac': '\xec\xb0\xbf',
'HangulSyllableCyeolg': '\xec\xb3\x99',
'UnicodeCharacter0xe27a': '\xee\x89\xba',
'HangulSyllableGyaep': '\xea\xb1\xae',
'HangulSyllableGyaes': '\xea\xb1\xa7',
'HangulSyllableGyaet': '\xea\xb1\xad',
'UnicodeCharacter0xe27b': '\xee\x89\xbb',
'HangulSyllableCyeolm': '\xec\xb3\x9a',
'HangulSyllableCae': '\xec\xb1\x84',
'HangulSyllableBog': '\xeb\xb3\xb5',
'UnicodeCharacter0xab69': '\xea\xad\xa9',
'UnicodeCharacter0xab68': '\xea\xad\xa8',
'HangulSyllableBoc': '\xeb\xb4\x8b',
'HangulSyllableBob': '\xeb\xb4\x85',
'UnicodeCharacter0xab63': '\xea\xad\xa3',
'UnicodeCharacter0xab62': '\xea\xad\xa2',
'UnicodeCharacter0xab61': '\xea\xad\xa1',
'LatinSmallLetterAWithCircumflexAndDotBelow': '\xe1\xba\xad',
'UnicodeCharacter0xab67': '\xea\xad\xa7',
'UnicodeCharacter0xab66': '\xea\xad\xa6',
'UnicodeCharacter0xab65': '\xea\xad\xa5',
'UnicodeCharacter0xab64': '\xea\xad\xa4',
'UnicodeCharacter0x1a20': '\xe1\xa8\xa0',
'UnicodeCharacter0x1a21': '\xe1\xa8\xa1',
'UnicodeCharacter0x1a22': '\xe1\xa8\xa2',
'UnicodeCharacter0x1a23': '\xe1\xa8\xa3',
'UnicodeCharacter0x1a24': '\xe1\xa8\xa4',
'UnicodeCharacter0x1a25': '\xe1\xa8\xa5',
'UnicodeCharacter0x1a26': '\xe1\xa8\xa6',
'UnicodeCharacter0x1a27': '\xe1\xa8\xa7',
'UnicodeCharacter0x1a28': '\xe1\xa8\xa8',
'UnicodeCharacter0x1a29': '\xe1\xa8\xa9',
'HangulSyllableCaj': '\xec\xb0\xbe',
'CjkUnifiedIdeograph5db9': '\xe5\xb6\xb9',
'MongolianSibeSyllableBoundaryMarker': '\xe1\xa0\x87',
'HangulSyllableGip': '\xea\xb9\x8a',
'HangulSyllableCak': '\xec\xb1\x80',
'HangulSyllableSsuj': '\xec\x91\xba',
'TamilVowelSignAa': '\xe0\xae\xbe',
'HangulSyllableBwaes': '\xeb\xb4\xbf',
'HangulSyllableCal': '\xec\xb0\xb0',
'HangulSyllableBwaep': '\xeb\xb5\x86',
'TamilVowelSignAi': '\xe0\xaf\x88',
'CjkUnifiedIdeograph8d07': '\xe8\xb4\x87',
'HangulSyllableBwaek': '\xeb\xb5\x84',
'TamilVowelSignAu': '\xe0\xaf\x8c',
'HangulSyllableBwaeh': '\xeb\xb5\x87',
'HangulSyllableBwaen': '\xeb\xb4\xb0',
'HangulSyllableBwaem': '\xeb\xb4\xbc',
'ClockwiseClosedCircleArrow': '\xe2\xa5\x81',
'HangulSyllableBwaec': '\xeb\xb5\x83',
'HangulSyllableBwaeb': '\xeb\xb4\xbd',
'HangulSyllableBwaeg': '\xeb\xb4\xad',
'HangulSyllableBwaed': '\xeb\xb4\xb3',
'UnicodeCharacter0x1a2a': '\xe1\xa8\xaa',
'UnicodeCharacter0x1a2b': '\xe1\xa8\xab',
'UnicodeCharacter0x1a2c': '\xe1\xa8\xac',
'UnicodeCharacter0x1a2d': '\xe1\xa8\xad',
'UnicodeCharacter0x1a2e': '\xe1\xa8\xae',
'UnicodeCharacter0x1a2f': '\xe1\xa8\xaf',
'CjkCompatibilityIdeographf9e6': '\xef\xa7\xa6',
'HangulSyllableHoss': '\xed\x99\x8c',
'HangulSyllableKoj': '\xec\xbd\xaa',
'CjkUnifiedIdeograph7f43': '\xe7\xbd\x83',
'HangulSyllableSenj': '\xec\x84\xbd',
'HangulSyllableSenh': '\xec\x84\xbe',
'UnicodeCharacter0xef8d': '\xee\xbe\x8d',
'UnicodeCharacter0xab6c': '\xea\xad\xac',
'UnicodeCharacter0xab6b': '\xea\xad\xab',
'TibetanMarkNgasBzungSgorRtags': '\xe0\xbc\xb7',
'HangulSyllableSeng': '\xec\x85\x8d',
'UnicodeCharacter0xab6f': '\xea\xad\xaf',
'CircleWithUpperRightQuadrantBlack': '\xe2\x97\x94',
'HeavyNorthEastArrow': '\xe2\x9e\x9a',
'AnticlockwiseTopSemicircleArrow': '\xe2\x86\xb6',
'CjkCompatibilityIdeographf9e7': '\xef\xa7\xa7',
'HangulSyllableDyeop': '\xeb\x8e\xa6',
'UnicodeCharacter0xcce': '\xe0\xb3\x8e',
'UnicodeCharacter0xccf': '\xe0\xb3\x8f',
'HangulSyllablePweo': '\xed\x92\x94',
'HangulSyllableBbyuh': '\xec\x81\x97',
'EthiopicSyllableShaa': '\xe1\x88\xbb',
'CjkUnifiedIdeograph8e1a': '\xe8\xb8\x9a',
'CjkUnifiedIdeograph8e1b': '\xe8\xb8\x9b',
'CjkUnifiedIdeograph8e1c': '\xe8\xb8\x9c',
'CjkUnifiedIdeograph8e1d': '\xe8\xb8\x9d',
'CjkUnifiedIdeograph8e1e': '\xe8\xb8\x9e',
'CjkUnifiedIdeograph8e1f': '\xe8\xb8\x9f',
'UnicodeCharacter0x1cef': '\xe1\xb3\xaf',
'CyrillicCapitalLetterOWithDiaeresis': '\xd3\xa6',
'CjkUnifiedIdeograph8f3f': '\xe8\xbc\xbf',
'HeavyOutlinedBlackStar': '\xe2\x9c\xae',
'UnicodeCharacter0xef8b': '\xee\xbe\x8b',
'HangulSyllablePweh': '\xed\x93\x8b',
'CjkUnifiedIdeograph7a77': '\xe7\xa9\xb7',
'CjkUnifiedIdeograph7a76': '\xe7\xa9\xb6',
'CjkUnifiedIdeograph7a75': '\xe7\xa9\xb5',
'CjkUnifiedIdeograph7a74': '\xe7\xa9\xb4',
'CjkUnifiedIdeograph7a73': '\xe7\xa9\xb3',
'CjkUnifiedIdeograph7a72': '\xe7\xa9\xb2',
'CjkUnifiedIdeograph7a71': '\xe7\xa9\xb1',
'CjkUnifiedIdeograph7a70': '\xe7\xa9\xb0',
'ArabicLetterPehehFinalForm': '\xef\xad\xaf',
'CjkUnifiedIdeograph7a79': '\xe7\xa9\xb9',
'CjkUnifiedIdeograph7a78': '\xe7\xa9\xb8',
'UnicodeCharacter0x1cea': '\xe1\xb3\xaa',
'HangulSyllablePwed': '\xed\x92\xb7',
'CjkUnifiedIdeograph8e10': '\xe8\xb8\x90',
'EthiopicSyllableDdo': '\xe1\x8b\xbe',
'CjkUnifiedIdeograph8e12': '\xe8\xb8\x92',
'CjkUnifiedIdeograph8e13': '\xe8\xb8\x93',
'BraillePatternDots134': '\xe2\xa0\x8d',
'KangxiRadicalSheep': '\xe2\xbd\xba',
'BraillePatternDots136': '\xe2\xa0\xa5',
'BraillePatternDots137': '\xe2\xa1\x85',
'BraillePatternDots138': '\xe2\xa2\x85',
'CjkUnifiedIdeograph8e19': '\xe8\xb8\x99',
'EthiopicSyllableDde': '\xe1\x8b\xbd',
'HangulSyllableSeuls': '\xec\x8a\xb0',
'EthiopicSyllableDda': '\xe1\x8b\xb8',
'CorrespondsTo': '\xe2\x89\x98',
'HangulSyllableSeulm': '\xec\x8a\xae',
'UnicodeCharacter0xcc9': '\xe0\xb3\x89',
'UnicodeCharacter0x18bc': '\xe1\xa2\xbc',
'HangulSyllableSeulh': '\xec\x8a\xb3',
'UnicodeCharacter0xcc5': '\xe0\xb3\x85',
'CjkUnifiedIdeograph6cb1': '\xe6\xb2\xb1',
'EthiopicSyllableDdu': '\xe1\x8b\xb9',
'LatinCapitalLetterWWithGrave': '\xe1\xba\x80',
'BalineseLetterKa': '\xe1\xac\x93',
'HangulSyllablePyah': '\xed\x8d\x9f',
'CjkUnifiedIdeograph7f47': '\xe7\xbd\x87',
'HangulSyllableGwaegs': '\xea\xb4\x9b',
'CjkUnifiedIdeograph7fbb': '\xe7\xbe\xbb',
'HangulSyllableSsyals': '\xec\x8c\xbc',
'CjkUnifiedIdeograph6cb2': '\xe6\xb2\xb2',
'CjkUnifiedIdeograph7fba': '\xe7\xbe\xba',
'UnicodeCharacter0xf188': '\xef\x86\x88',
'CjkUnifiedIdeograph7a7f': '\xe7\xa9\xbf',
'CjkUnifiedIdeograph7a7e': '\xe7\xa9\xbe',
'CjkUnifiedIdeograph7a7d': '\xe7\xa9\xbd',
'ArmenianSmallLetterZa': '\xd5\xa6',
'GurmukhiTippi': '\xe0\xa9\xb0',
'CjkUnifiedIdeograph7a7a': '\xe7\xa9\xba',
'UnicodeCharacter0xf1dc': '\xef\x87\x9c',
'HangulSyllableDyeod': '\xeb\x8e\x93',
'HangulSyllableGub': '\xea\xb5\xbd',
'HangulSyllableGuc': '\xea\xb6\x83',
'HangulSyllableGud': '\xea\xb5\xb3',
'GujaratiLetterU': '\xe0\xaa\x89',
'HangulSyllableGug': '\xea\xb5\xad',
'CanadianSyllabicsCwii': '\xe1\x92\x96',
'HangulSyllableMoels': '\xeb\xac\x88',
'HangulSyllableGuj': '\xea\xb6\x82',
'CjkUnifiedIdeograph7fbd': '\xe7\xbe\xbd',
'HangulSyllableGul': '\xea\xb5\xb4',
'HangulSyllableGum': '\xea\xb5\xbc',
'HangulSyllableGun': '\xea\xb5\xb0',
'HangulSyllableGup': '\xea\xb6\x86',
'GujaratiLetterA': '\xe0\xaa\x85',
'HangulSyllableGus': '\xea\xb5\xbf',
'HangulSyllableGut': '\xea\xb6\x85',
'GujaratiLetterE': '\xe0\xaa\x8f',
'HangulSyllableDyeoh': '\xeb\x8e\xa7',
'CircledIdeographFive': '\xe3\x8a\x84',
'GujaratiLetterO': '\xe0\xaa\x93',
'UnicodeCharacter0xf24d': '\xef\x89\x8d',
'UnicodeCharacter0xf24e': '\xef\x89\x8e',
'UnicodeCharacter0xf24f': '\xef\x89\x8f',
'UnicodeCharacter0xf24a': '\xef\x89\x8a',
'UnicodeCharacter0xf24b': '\xef\x89\x8b',
'UnicodeCharacter0xf24c': '\xef\x89\x8c',
'HangulSyllableDwags': '\xeb\x8f\xa3',
'HangulSyllableJjeonj': '\xec\xa9\x91',
'CjkUnifiedIdeograph60b0': '\xe6\x82\xb0',
'HangulSyllableMoelm': '\xeb\xac\x86',
'UnicodeCharacter0xf786': '\xef\x9e\x86',
'HangulSyllablePwep': '\xed\x93\x8a',
'CjkUnifiedIdeograph6238': '\xe6\x88\xb8',
'UnicodeCharacter0x2bf8': '\xe2\xaf\xb8',
'UnicodeCharacter0x2bf9': '\xe2\xaf\xb9',
'UnicodeCharacter0x2bf6': '\xe2\xaf\xb6',
'UnicodeCharacter0x2bf7': '\xe2\xaf\xb7',
'UnicodeCharacter0x2bf4': '\xe2\xaf\xb4',
'UnicodeCharacter0x2bf5': '\xe2\xaf\xb5',
'UnicodeCharacter0x2bf2': '\xe2\xaf\xb2',
'UnicodeCharacter0x2bf3': '\xe2\xaf\xb3',
'UnicodeCharacter0x2bf0': '\xe2\xaf\xb0',
'UnicodeCharacter0x2bf1': '\xe2\xaf\xb1',
'NewTaiLueVowelSignE': '\xe1\xa6\xb5',
'EthiopicSyllableMe': '\xe1\x88\x9d',
'PhagspaLetterVoicelessSha': '\xea\xa1\xae',
'HangulChoseongChitueumcieuc': '\xe1\x85\x8e',
'HangulSyllableMoelg': '\xeb\xac\x85',
'HangulLetterKapyeounssangpieup': '\xe3\x85\xb9',
'ArabicLigatureSheenWithHahWithYehFinalForm': '\xef\xb6\xaa',
'HangulSyllableDib': '\xeb\x94\xa5',
'HangulSyllableRoc': '\xeb\xa1\xb3',
'CircledKatakanaNi': '\xe3\x8b\xa5',
'CjkUnifiedIdeograph6230': '\xe6\x88\xb0',
'CircledKatakanaNo': '\xe3\x8b\xa8',
'CircledKatakanaNa': '\xe3\x8b\xa4',
'UnicodeCharacter0x2bfd': '\xe2\xaf\xbd',
'UnicodeCharacter0x2bfe': '\xe2\xaf\xbe',
'CircledKatakanaNe': '\xe3\x8b\xa7',
'UnicodeCharacter0x2bfc': '\xe2\xaf\xbc',
'UnicodeCharacter0x2bfa': '\xe2\xaf\xba',
'UnicodeCharacter0xf244': '\xef\x89\x84',
'CombiningIsBelow': '\xe1\xb7\x90',
'UnicodeCharacter0xf246': '\xef\x89\x86',
'UnicodeCharacter0xf247': '\xef\x89\x87',
'UnicodeCharacter0xf240': '\xef\x89\x80',
'UnicodeCharacter0xf241': '\xef\x89\x81',
'HangulSyllableBboeng': '\xeb\xbe\xa9',
'HangulSyllableGgyaegg': '\xea\xba\xa2',
'HangulSyllableBboenh': '\xeb\xbe\x9a',
'HangulSyllableBboenj': '\xeb\xbe\x99',
'CircledKatakanaNu': '\xe3\x8b\xa6',
'UnicodeCharacter0xf249': '\xef\x89\x89',
'CjkUnifiedIdeograph92b5': '\xe9\x8a\xb5',
'CjkUnifiedIdeograph92b4': '\xe9\x8a\xb4',
'CjkUnifiedIdeograph92b7': '\xe9\x8a\xb7',
'CjkUnifiedIdeograph92b6': '\xe9\x8a\xb6',
'CjkUnifiedIdeograph92b1': '\xe9\x8a\xb1',
'CjkUnifiedIdeograph92b0': '\xe9\x8a\xb0',
'GurmukhiLetterDda': '\xe0\xa8\xa1',
'CjkUnifiedIdeograph92b2': '\xe9\x8a\xb2',
'CjkUnifiedIdeograph92b9': '\xe9\x8a\xb9',
'CjkUnifiedIdeograph92b8': '\xe9\x8a\xb8',
'HangulSyllableAenj': '\xec\x95\xa5',
'VaiSyllableDhee': '\xea\x94\x90',
'CjkUnifiedIdeograph9c39': '\xe9\xb0\xb9',
'CjkUnifiedIdeograph9c38': '\xe9\xb0\xb8',
'UnicodeCharacter0xe8dc': '\xee\xa3\x9c',
'UnicodeCharacter0xe8db': '\xee\xa3\x9b',
'UnicodeCharacter0xe8da': '\xee\xa3\x9a',
'LatinCapitalLetterIWithTildeBelow': '\xe1\xb8\xac',
'CjkUnifiedIdeograph9c33': '\xe9\xb0\xb3',
'CjkUnifiedIdeograph9c32': '\xe9\xb0\xb2',
'CjkUnifiedIdeograph9c31': '\xe9\xb0\xb1',
'CjkUnifiedIdeograph9c30': '\xe9\xb0\xb0',
'CjkUnifiedIdeograph9c37': '\xe9\xb0\xb7',
'CjkUnifiedIdeograph9c36': '\xe9\xb0\xb6',
'CjkUnifiedIdeograph9c35': '\xe9\xb0\xb5',
'CjkUnifiedIdeograph9c34': '\xe9\xb0\xb4',
'SyriacLetterPersianGhamal': '\xdc\xae',
'HangulSyllableGep': '\xea\xb2\xa6',
'DevanagariLetterKha': '\xe0\xa4\x96',
'UnicodeCharacter0xe60a': '\xee\x98\x8a',
'UnicodeCharacter0x1ce9': '\xe1\xb3\xa9',
'UnicodeCharacter0xe60c': '\xee\x98\x8c',
'UnicodeCharacter0xe60b': '\xee\x98\x8b',
'EighthNote': '\xe2\x99\xaa',
'WaveArrowPointingDirectlyRight': '\xe2\xa4\xb3',
'CjkUnifiedIdeograph4c20': '\xe4\xb0\xa0',
'UnicodeCharacter0xd810': '\xed\xa0\x90',
'CjkUnifiedIdeograph4208': '\xe4\x88\x88',
'CjkUnifiedIdeograph4209': '\xe4\x88\x89',
'CjkUnifiedIdeograph3fcf': '\xe3\xbf\x8f',
'CjkUnifiedIdeograph3fca': '\xe3\xbf\x8a',
'CjkUnifiedIdeograph3fcb': '\xe3\xbf\x8b',
'CjkUnifiedIdeograph3fcc': '\xe3\xbf\x8c',
'CjkUnifiedIdeograph4200': '\xe4\x88\x80',
'CjkUnifiedIdeograph4201': '\xe4\x88\x81',
'CjkUnifiedIdeograph4202': '\xe4\x88\x82',
'CjkUnifiedIdeograph4203': '\xe4\x88\x83',
'CjkUnifiedIdeograph4204': '\xe4\x88\x84',
'CjkUnifiedIdeograph4205': '\xe4\x88\x85',
'CjkUnifiedIdeograph4206': '\xe4\x88\x86',
'CjkUnifiedIdeograph4207': '\xe4\x88\x87',
'UnicodeCharacter0xe8d7': '\xee\xa3\x97',
'UnicodeCharacter0xe8d6': '\xee\xa3\x96',
'UnicodeCharacter0xe8d5': '\xee\xa3\x95',
'UnicodeCharacter0xe8d4': '\xee\xa3\x94',
'UnicodeCharacter0xe8d3': '\xee\xa3\x93',
'UnicodeCharacter0xe8d2': '\xee\xa3\x92',
'UnicodeCharacter0xe8d1': '\xee\xa3\x91',
'UnicodeCharacter0xe8d0': '\xee\xa3\x90',
'CjkUnifiedIdeograph9c3c': '\xe9\xb0\xbc',
'CjkUnifiedIdeograph9c3b': '\xe9\xb0\xbb',
'CjkUnifiedIdeograph9c3a': '\xe9\xb0\xba',
'LimbuLetterLa': '\xe1\xa4\x97',
'CjkUnifiedIdeograph9c3e': '\xe9\xb0\xbe',
'CjkUnifiedIdeograph9c3d': '\xe9\xb0\xbd',
'CjkUnifiedIdeograph92be': '\xe9\x8a\xbe',
'CjkUnifiedIdeograph92bd': '\xe9\x8a\xbd',
'CjkUnifiedIdeograph92bf': '\xe9\x8a\xbf',
'CjkUnifiedIdeograph92ba': '\xe9\x8a\xba',
'UnicodeCharacter0x1ce1': '\xe1\xb3\xa1',
'CjkUnifiedIdeograph92bc': '\xe9\x8a\xbc',
'CjkUnifiedIdeograph92bb': '\xe9\x8a\xbb',
'SubsetOf': '\xe2\x8a\x82',
'BoxDrawingsHeavyUp': '\xe2\x95\xb9',
'UnicodeCharacter0x1ce0': '\xe1\xb3\xa0',
'LimbuLetterTha': '\xe1\xa4\x8c',
'CjkUnifiedIdeograph67cc': '\xe6\x9f\x8c',
'CjkUnifiedIdeograph3fc4': '\xe3\xbf\x84',
'CjkUnifiedIdeograph3fc5': '\xe3\xbf\x85',
'CjkUnifiedIdeograph3fc6': '\xe3\xbf\x86',
'CjkUnifiedIdeograph3fc7': '\xe3\xbf\x87',
'CjkUnifiedIdeograph3fc0': '\xe3\xbf\x80',
'CjkUnifiedIdeograph3fc1': '\xe3\xbf\x81',
'CjkUnifiedIdeograph3fc2': '\xe3\xbf\x82',
'CjkUnifiedIdeograph3fc3': '\xe3\xbf\x83',
'CjkUnifiedIdeograph420a': '\xe4\x88\x8a',
'CjkUnifiedIdeograph420b': '\xe4\x88\x8b',
'CjkUnifiedIdeograph420c': '\xe4\x88\x8c',
'CjkUnifiedIdeograph420d': '\xe4\x88\x8d',
'CjkUnifiedIdeograph3fc9': '\xe3\xbf\x89',
'CjkUnifiedIdeograph420f': '\xe4\x88\x8f',
'HangulSyllableDim': '\xeb\x94\xa4',
'UnicodeCharacter0xe604': '\xee\x98\x84',
'UnicodeCharacter0xe607': '\xee\x98\x87',
'HangulSyllableRob': '\xeb\xa1\xad',
'UnicodeCharacter0xe601': '\xee\x98\x81',
'UnicodeCharacter0xe600': '\xee\x98\x80',
'CanadianSyllabicsSoo': '\xe1\x93\xb2',
'UnicodeCharacter0xe602': '\xee\x98\x82',
'CjkRadicalCsimplifiedFly': '\xe2\xbb\x9c',
'HangulSyllableBelg': '\xeb\xb2\xa9',
'HangulSyllableBelb': '\xeb\xb2\xab',
'UnicodeCharacter0xe608': '\xee\x98\x88',
'UnicodeCharacter0xf824': '\xef\xa0\xa4',
'CanadianSyllabicsCarrierDi': '\xe1\x91\x94',
'UnicodeCharacter0xf825': '\xef\xa0\xa5',
'HangulSyllableBbeod': '\xeb\xbb\x97',
'YiSyllableNjiet': '\xea\x90\xb5',
'CjkCompatibilityIdeographf9da': '\xef\xa7\x9a',
'CjkUnifiedIdeograph8d0c': '\xe8\xb4\x8c',
'HangulSyllableBbeob': '\xeb\xbb\xa1',
'CjkUnifiedIdeograph7e22': '\xe7\xb8\xa2',
'YiSyllableNjiex': '\xea\x90\xb6',
'LatinCapitalLetterHeng': '\xea\x9c\xa6',
'YiSyllableNzur': '\xea\x8c\x82',
'HangulSyllableNwagg': '\xeb\x86\x96',
'YiSyllableNzup': '\xea\x8c\x80',
'GreekCapitalLetterUpsilonWithDasiaAndOxia': '\xe1\xbd\x9d',
'UnicodeCharacter0xf823': '\xef\xa0\xa3',
'HangulSyllableDik': '\xeb\x94\xac',
'YiSyllableNzux': '\xea\x8b\xbe',
'CjkUnifiedIdeograph5557': '\xe5\x95\x97',
'CjkUnifiedIdeograph5556': '\xe5\x95\x96',
'CjkUnifiedIdeograph5555': '\xe5\x95\x95',
'CjkUnifiedIdeograph5554': '\xe5\x95\x94',
'CjkUnifiedIdeograph5553': '\xe5\x95\x93',
'CjkUnifiedIdeograph5552': '\xe5\x95\x92',
'CjkUnifiedIdeograph5551': '\xe5\x95\x91',
'CjkUnifiedIdeograph5550': '\xe5\x95\x90',
'YiSyllableNzuo': '\xea\x8b\xb9',
'CjkUnifiedIdeograph3592': '\xe3\x96\x92',
'CjkUnifiedIdeograph5559': '\xe5\x95\x99',
'CjkUnifiedIdeograph5558': '\xe5\x95\x98',
'HangulChoseongNieuntikeut': '\xe1\x84\x95',
'GurmukhiLetterBa': '\xe0\xa8\xac',
'ArabicLetterHehWithYehAboveFinalForm': '\xef\xae\xa5',
'EthiopicSyllableTsee': '\xe1\x8c\xbc',
'UnicodeCharacter0xeac3': '\xee\xab\x83',
'CopticCapitalLetterDialectpHori': '\xe2\xb3\x8a',
'EthiopicNumberThirty': '\xe1\x8d\xb4',
'CjkUnifiedIdeograph555f': '\xe5\x95\x9f',
'CjkUnifiedIdeograph555e': '\xe5\x95\x9e',
'CjkUnifiedIdeograph555d': '\xe5\x95\x9d',
'CjkUnifiedIdeograph555c': '\xe5\x95\x9c',
'CjkUnifiedIdeograph555b': '\xe5\x95\x9b',
'CjkUnifiedIdeograph555a': '\xe5\x95\x9a',
'HangulSyllableSwenj': '\xec\x89\x95',
'BraillePatternDots25678': '\xe2\xa3\xb2',
'UnicodeCharacter0xf2b4': '\xef\x8a\xb4',
'HangulSyllableSwenh': '\xec\x89\x96',
'InvisibleTimes': '\xe2\x81\xa2',
'UnicodeCharacter0xf2b2': '\xef\x8a\xb2',
'HangulSyllableGyaess': '\xea\xb1\xa8',
'CjkUnifiedIdeograph9328': '\xe9\x8c\xa8',
'CjkUnifiedIdeograph9329': '\xe9\x8c\xa9',
'RejangVowelSignAu': '\xea\xa5\x8c',
'UnicodeCharacter0xf2b0': '\xef\x8a\xb0',
'CjkUnifiedIdeograph9320': '\xe9\x8c\xa0',
'CjkUnifiedIdeograph9321': '\xe9\x8c\xa1',
'CjkUnifiedIdeograph9322': '\xe9\x8c\xa2',
'CjkUnifiedIdeograph9323': '\xe9\x8c\xa3',
'CjkUnifiedIdeograph9324': '\xe9\x8c\xa4',
'CjkUnifiedIdeograph9325': '\xe9\x8c\xa5',
'CjkUnifiedIdeograph9326': '\xe9\x8c\xa6',
'CjkUnifiedIdeograph9327': '\xe9\x8c\xa7',
'CjkUnifiedIdeograph6116': '\xe6\x84\x96',
'CjkUnifiedIdeograph6117': '\xe6\x84\x97',
'CjkUnifiedIdeograph6114': '\xe6\x84\x94',
'CjkUnifiedIdeograph6115': '\xe6\x84\x95',
'CjkUnifiedIdeograph6112': '\xe6\x84\x92',
'CjkUnifiedIdeograph6113': '\xe6\x84\x93',
'CjkUnifiedIdeograph6110': '\xe6\x84\x90',
'CjkUnifiedIdeograph6111': '\xe6\x84\x91',
'RejangVowelSignAi': '\xea\xa5\x8a',
'CjkUnifiedIdeograph6118': '\xe6\x84\x98',
'LimbuSmallLetterLa': '\xe1\xa4\xb8',
'UnicodeCharacter0xf79b': '\xef\x9e\x9b',
'SimilarAboveLessthanAboveEqualsSign': '\xe2\xaa\x9f',
'CjkRadicalMoundTwo': '\xe2\xbb\x96',
'BoxDrawingsRightHeavyAndLeftVerticalLight': '\xe2\x94\xbe',
'KhmerSignKakabat': '\xe1\x9f\x8e',
'CyrillicCapitalLetterIo': '\xd0\x81',
'CjkUnifiedIdeograph7cc9': '\xe7\xb3\x89',
'CjkUnifiedIdeograph7cc8': '\xe7\xb3\x88',
'ArabicLigatureSheenWithJeemIsolatedForm': '\xef\xb4\x89',
'CyrillicCapitalLetterIe': '\xd0\x95',
'CjkUnifiedIdeograph6cba': '\xe6\xb2\xba',
'GreekLunateSigmaSymbol': '\xcf\xb2',
'CjkUnifiedIdeograph7cc6': '\xe7\xb3\x86',
'CjkUnifiedIdeograph7cc1': '\xe7\xb3\x81',
'CjkUnifiedIdeograph7cc0': '\xe7\xb3\x80',
'LeftRightWhiteArrow': '\xe2\xac\x84',
'CjkUnifiedIdeograph7cc2': '\xe7\xb3\x82',
'CjkUnifiedIdeograph611f': '\xe6\x84\x9f',
'CjkUnifiedIdeograph611d': '\xe6\x84\x9d',
'CjkUnifiedIdeograph611e': '\xe6\x84\x9e',
'CjkUnifiedIdeograph611b': '\xe6\x84\x9b',
'CjkUnifiedIdeograph611c': '\xe6\x84\x9c',
'CjkUnifiedIdeograph611a': '\xe6\x84\x9a',
'UnicodeCharacter0xf54a': '\xef\x95\x8a',
'DownwardsZigzagArrow': '\xe2\x86\xaf',
'CjkUnifiedIdeograph932a': '\xe9\x8c\xaa',
'CjkUnifiedIdeograph932b': '\xe9\x8c\xab',
'CjkUnifiedIdeograph932c': '\xe9\x8c\xac',
'CjkUnifiedIdeograph932d': '\xe9\x8c\xad',
'CjkUnifiedIdeograph932e': '\xe9\x8c\xae',
'CjkUnifiedIdeograph932f': '\xe9\x8c\xaf',
'HangulSyllablePoss': '\xed\x90\x80',
'HangulSyllableSsegs': '\xec\x8e\x87',
'CjkUnifiedIdeograph7cce': '\xe7\xb3\x8e',
'CjkUnifiedIdeograph7ccd': '\xe7\xb3\x8d',
'LatinLetterSmallCapitalK': '\xe1\xb4\x8b',
'CjkUnifiedIdeograph7ccf': '\xe7\xb3\x8f',
'CjkUnifiedIdeograph7cca': '\xe7\xb3\x8a',
'LatinSmallLetterDWithHookAndTail': '\xe1\xb6\x91',
'CjkUnifiedIdeograph7ccc': '\xe7\xb3\x8c',
'CjkUnifiedIdeograph7ccb': '\xe7\xb3\x8b',
'HangulSyllableSih': '\xec\x8b\xb7',
'BraillePatternDots12357': '\xe2\xa1\x97',
'BraillePatternDots12356': '\xe2\xa0\xb7',
'HangulSyllableSil': '\xec\x8b\xa4',
'GurmukhiSignVirama': '\xe0\xa9\x8d',
'HangulSyllableSin': '\xec\x8b\xa0',
'UnicodeCharacter0xe7bd': '\xee\x9e\xbd',
'HangulSyllableSib': '\xec\x8b\xad',
'HangulSyllableSic': '\xec\x8b\xb3',
'HangulSyllableSid': '\xec\x8b\xa3',
'BraillePatternDots12358': '\xe2\xa2\x97',
'HangulSyllableSig': '\xec\x8b\x9d',
'EthiopicSyllableGyee': '\xe2\xb7\x9c',
'HangulSyllableGgwilt': '\xeb\x80\x99',
'HangulSyllableDeugs': '\xeb\x93\x9f',
'CjkUnifiedIdeograph779f': '\xe7\x9e\x9f',
'CjkUnifiedIdeograph779e': '\xe7\x9e\x9e',
'CjkUnifiedIdeograph779d': '\xe7\x9e\x9d',
'CjkUnifiedIdeograph779c': '\xe7\x9e\x9c',
'CjkUnifiedIdeograph779b': '\xe7\x9e\x9b',
'CjkUnifiedIdeograph779a': '\xe7\x9e\x9a',
'CjkUnifiedIdeograph9ed8': '\xe9\xbb\x98',
'CjkUnifiedIdeograph9ed9': '\xe9\xbb\x99',
'UnicodeCharacter0xea3a': '\xee\xa8\xba',
'UnicodeCharacter0xea3f': '\xee\xa8\xbf',
'UnicodeCharacter0xea3e': '\xee\xa8\xbe',
'UnicodeCharacter0xea3d': '\xee\xa8\xbd',
'CjkUnifiedIdeograph9ed0': '\xe9\xbb\x90',
'CjkUnifiedIdeograph9ed1': '\xe9\xbb\x91',
'CjkUnifiedIdeograph9ed2': '\xe9\xbb\x92',
'CjkUnifiedIdeograph9ed3': '\xe9\xbb\x93',
'CjkUnifiedIdeograph9b19': '\xe9\xac\x99',
'CjkUnifiedIdeograph9ed5': '\xe9\xbb\x95',
'CjkUnifiedIdeograph9ed6': '\xe9\xbb\x96',
'CjkUnifiedIdeograph9ed7': '\xe9\xbb\x97',
'UnicodeCharacter0xfbb8': '\xef\xae\xb8',
'IdeographicLevelToneMark': '\xe3\x80\xaa',
'EnterSymbol': '\xe2\x8e\x86',
'LatinSmallLetterDelta': '\xe1\xba\x9f',
'TibetanLetterNa': '\xe0\xbd\x93',
'UnicodeCharacter0xf02b': '\xef\x80\xab',
'CjkUnifiedIdeograph7799': '\xe7\x9e\x99',
'CjkUnifiedIdeograph7798': '\xe7\x9e\x98',
'CjkUnifiedIdeograph7797': '\xe7\x9e\x97',
'CjkUnifiedIdeograph7796': '\xe7\x9e\x96',
'CjkUnifiedIdeograph7795': '\xe7\x9e\x95',
'CjkUnifiedIdeograph7794': '\xe7\x9e\x94',
'CjkUnifiedIdeograph7793': '\xe7\x9e\x93',
'CjkUnifiedIdeograph7792': '\xe7\x9e\x92',
'CjkUnifiedIdeograph7791': '\xe7\x9e\x91',
'CjkUnifiedIdeograph7790': '\xe7\x9e\x90',
'HangulSyllableSsut': '\xec\x91\xbd',
'UnicodeCharacter0xf812': '\xef\xa0\x92',
'HanunooLetterPa': '\xe1\x9c\xa9',
'HangulSyllableReh': '\xeb\xa0\xa3',
'HangulSyllableRek': '\xeb\xa0\xa0',
'HangulSyllableRej': '\xeb\xa0\x9e',
'HangulSyllableRem': '\xeb\xa0\x98',
'HangulSyllableRel': '\xeb\xa0\x90',
'HangulSyllableReo': '\xeb\x9f\xac',
'HangulSyllableRen': '\xeb\xa0\x8c',
'HangulSyllableRec': '\xeb\xa0\x9f',
'HangulSyllableReb': '\xeb\xa0\x99',
'HangulSyllableRed': '\xeb\xa0\x8f',
'HangulSyllableReg': '\xeb\xa0\x89',
'UnicodeCharacter0xea33': '\xee\xa8\xb3',
'UnicodeCharacter0xea32': '\xee\xa8\xb2',
'UnicodeCharacter0xea31': '\xee\xa8\xb1',
'UnicodeCharacter0xea30': '\xee\xa8\xb0',
'UnicodeCharacter0xea37': '\xee\xa8\xb7',
'CjkUnifiedIdeograph5c0a': '\xe5\xb0\x8a',
'UnicodeCharacter0xdd28': '\xed\xb4\xa8',
'UnicodeCharacter0xdd29': '\xed\xb4\xa9',
'UnicodeCharacter0xdd26': '\xed\xb4\xa6',
'CjkUnifiedIdeograph9eda': '\xe9\xbb\x9a',
'CjkUnifiedIdeograph9edb': '\xe9\xbb\x9b',
'CjkUnifiedIdeograph9edc': '\xe9\xbb\x9c',
'CjkUnifiedIdeograph9edd': '\xe9\xbb\x9d',
'CjkUnifiedIdeograph9ede': '\xe9\xbb\x9e',
'CjkUnifiedIdeograph9edf': '\xe9\xbb\x9f',
'UnicodeCharacter0xdd21': '\xed\xb4\xa1',
'CjkUnifiedIdeograph54de': '\xe5\x93\x9e',
'CjkUnifiedIdeograph54dd': '\xe5\x93\x9d',
'YiSyllableXop': '\xea\x91\xaa',
'CjkUnifiedIdeograph54df': '\xe5\x93\x9f',
'CjkUnifiedIdeograph54da': '\xe5\x93\x9a',
'CjkUnifiedIdeograph54dc': '\xe5\x93\x9c',
'CjkUnifiedIdeograph54db': '\xe5\x93\x9b',
'CjkUnifiedIdeograph950d': '\xe9\x94\x8d',
'YiSyllableXox': '\xea\x91\xa8',
'OriyaVowelSignUu': '\xe0\xad\x82',
'UnicodeCharacter0xf2bd': '\xef\x8a\xbd',
'HangulSyllableJeb': '\xec\xa0\xad',
'UnicodeCharacter0xde95': '\xed\xba\x95',
'ArabicLigatureSeenWithHehInitialForm': '\xef\xb4\xb1',
'UnicodeCharacter0xf2be': '\xef\x8a\xbe',
'UnicodeCharacter0xf2bb': '\xef\x8a\xbb',
'YiSyllableBe': '\xea\x80\xaa',
'CherokeeLetterMe': '\xe1\x8e\xba',
'CherokeeLetterMa': '\xe1\x8e\xb9',
'CherokeeLetterMo': '\xe1\x8e\xbc',
'SymbolForStartOfText': '\xe2\x90\x82',
'CherokeeLetterMi': '\xe1\x8e\xbb',
'CherokeeLetterMu': '\xe1\x8e\xbd',
'HangulSyllableMwess': '\xeb\xae\x80',
'UnicodeCharacter0xf5eb': '\xef\x97\xab',
'HangulSyllableSsyaet': '\xec\x8d\xa5',
'CjkUnifiedIdeograph9ddd': '\xe9\xb7\x9d',
'UnicodeCharacter0xe03c': '\xee\x80\xbc',
'HangulSyllableJed': '\xec\xa0\xa3',
'HangulSyllableTyoc': '\xed\x88\xa7',
'CjkUnifiedIdeograph54d5': '\xe5\x93\x95',
'CjkUnifiedIdeograph54d4': '\xe5\x93\x94',
'CjkUnifiedIdeograph54d7': '\xe5\x93\x97',
'CjkUnifiedIdeograph54d6': '\xe5\x93\x96',
'CjkUnifiedIdeograph54d1': '\xe5\x93\x91',
'CjkUnifiedIdeograph54d0': '\xe5\x93\x90',
'CjkUnifiedIdeograph54d3': '\xe5\x93\x93',
'CjkUnifiedIdeograph54d2': '\xe5\x93\x92',
'LatinSmallLetterWWithHook': '\xe2\xb1\xb3',
'YiSyllableBo': '\xea\x80\xa7',
'CjkUnifiedIdeograph54d9': '\xe5\x93\x99',
'CjkUnifiedIdeograph54d8': '\xe5\x93\x98',
'CircledIdeographEnterprise': '\xe3\x8a\xad',
'HangulSyllableCyoc': '\xec\xb6\x8f',
'CjkUnifiedIdeograph9565': '\xe9\x95\xa5',
'CanadianSyllabicsCarrierPee': '\xe1\x97\xab',
'UnicodeCharacter0xed11': '\xee\xb4\x91',
'UnicodeCharacter0xf82e': '\xef\xa0\xae',
'HangulSyllableSsyaes': '\xec\x8d\x9f',
'HangulSyllableHyeonh': '\xed\x98\x86',
'UnicodeCharacter0xf82f': '\xef\xa0\xaf',
'HangulSyllableHyeonj': '\xed\x98\x85',
'Earth': '\xe2\x99\x81',
'YiSyllableBu': '\xea\x80\xae',
'CjkUnifiedIdeograph70cb': '\xe7\x83\x8b',
'CjkUnifiedIdeograph70cc': '\xe7\x83\x8c',
'CjkUnifiedIdeograph70ca': '\xe7\x83\x8a',
'CjkUnifiedIdeograph70cf': '\xe7\x83\x8f',
'KannadaLetterOo': '\xe0\xb2\x93',
'CjkUnifiedIdeograph70cd': '\xe7\x83\x8d',
'CjkUnifiedIdeograph70ce': '\xe7\x83\x8e',
'UnicodeCharacter0xf82c': '\xef\xa0\xac',
'CjkUnifiedIdeograph5230': '\xe5\x88\xb0',
'CjkUnifiedIdeograph3b5f': '\xe3\xad\x9f',
'CjkUnifiedIdeograph3b5d': '\xe3\xad\x9d',
'CjkUnifiedIdeograph3b5e': '\xe3\xad\x9e',
'CjkUnifiedIdeograph3b5b': '\xe3\xad\x9b',
'CjkUnifiedIdeograph3b5c': '\xe3\xad\x9c',
'CjkUnifiedIdeograph3b5a': '\xe3\xad\x9a',
'UnicodeCharacter0xe03f': '\xee\x80\xbf',
'UnicodeCharacter0xda71': '\xed\xa9\xb1',
'ArabicLetterKhahIsolatedForm': '\xef\xba\xa5',
'NaryTimesOperator': '\xe2\xa8\x89',
'LeftArrowWithCircledPlus': '\xe2\xac\xb2',
'CjkUnifiedIdeograph6945': '\xe6\xa5\x85',
'CjkUnifiedIdeograph6944': '\xe6\xa5\x84',
'CjkUnifiedIdeograph6947': '\xe6\xa5\x87',
'CjkUnifiedIdeograph6946': '\xe6\xa5\x86',
'TeluguFractionDigitOneForEvenPowersOfFour': '\xe0\xb1\xbc',
'CjkUnifiedIdeograph6940': '\xe6\xa5\x80',
'CjkUnifiedIdeograph6943': '\xe6\xa5\x83',
'CjkUnifiedIdeograph6942': '\xe6\xa5\x82',
'CjkUnifiedIdeograph43ba': '\xe4\x8e\xba',
'PopDirectionalFormatting': '\xe2\x80\xac',
'SquareBq': '\xe3\x8f\x83',
'LatinSmallLetterCWithAcute': '\xc4\x87',
'EthiopicSyllableGwi': '\xe1\x8c\x92',
'HangulSyllableMwenj': '\xeb\xad\xb1',
'YiSyllableYu': '\xea\x92\x83',
'HangulSyllableSsyaeb': '\xec\x8d\x9d',
'CjkUnifiedIdeograph3b58': '\xe3\xad\x98',
'CjkUnifiedIdeograph3b59': '\xe3\xad\x99',
'CjkUnifiedIdeograph3b56': '\xe3\xad\x96',
'CjkUnifiedIdeograph3b57': '\xe3\xad\x97',
'CjkUnifiedIdeograph3b54': '\xe3\xad\x94',
'CjkUnifiedIdeograph3b55': '\xe3\xad\x95',
'CjkUnifiedIdeograph3b52': '\xe3\xad\x92',
'CjkUnifiedIdeograph3b53': '\xe3\xad\x93',
'CjkUnifiedIdeograph3b50': '\xe3\xad\x90',
'CjkUnifiedIdeograph3b51': '\xe3\xad\x91',
'CjkUnifiedIdeograph70c2': '\xe7\x83\x82',
'CanadianSyllabicsNwe': '\xe1\x93\x89',
'CjkUnifiedIdeograph70c0': '\xe7\x83\x80',
'CjkUnifiedIdeograph70c1': '\xe7\x83\x81',
'CjkUnifiedIdeograph70c6': '\xe7\x83\x86',
'EthiopicSyllablePharyngealA': '\xe1\x8b\x90',
'CjkUnifiedIdeograph70c4': '\xe7\x83\x84',
'CjkUnifiedIdeograph70c5': '\xe7\x83\x85',
'YiSyllableYi': '\xea\x91\xb3',
'CjkUnifiedIdeograph70c8': '\xe7\x83\x88',
'CjkUnifiedIdeograph70c9': '\xe7\x83\x89',
'HangulSyllableGwaelg': '\xea\xb4\xa1',
'YiSyllableYo': '\xea\x91\xbf',
'CjkUnifiedIdeograph694e': '\xe6\xa5\x8e',
'CyrillicSmallLetterKa': '\xd0\xba',
'CjkUnifiedIdeograph694f': '\xe6\xa5\x8f',
'CjkUnifiedIdeograph694a': '\xe6\xa5\x8a',
'CjkUnifiedIdeograph694c': '\xe6\xa5\x8c',
'CjkUnifiedIdeograph694b': '\xe6\xa5\x8b',
'HangulSyllableDdeubs': '\xeb\x9c\xba',
'ArabicLigatureYehWithMeemWithYehFinalForm': '\xef\xb6\xb0',
'HangulSyllableSsyaej': '\xec\x8d\xa2',
'LatinSmallLetterRWithCaron': '\xc5\x99',
'UnicodeCharacter0xf8af': '\xef\xa2\xaf',
'CjkUnifiedIdeograph9def': '\xe9\xb7\xaf',
'CjkUnifiedIdeograph9ded': '\xe9\xb7\xad',
'CjkUnifiedIdeograph9dee': '\xe9\xb7\xae',
'CjkUnifiedIdeograph9deb': '\xe9\xb7\xab',
'CjkUnifiedIdeograph9dec': '\xe9\xb7\xac',
'CjkUnifiedIdeograph9dea': '\xe9\xb7\xaa',
'ArabicLigatureThehWithRehFinalForm': '\xef\xb1\xb6',
'HangulSyllableMyeolg': '\xeb\xa9\xb9',
'SinhalaLetterMahaapraanaDayanna': '\xe0\xb6\xb0',
'HangulSyllableNud': '\xeb\x88\x8b',
'HangulSyllableMyeolb': '\xeb\xa9\xbb',
'HangulSyllableKags': '\xec\xb9\xb7',
'LeftwardsArrowWithTailWithDoubleVerticalStroke': '\xe2\xac\xba',
'UnicodeCharacter0xdfe3': '\xed\xbf\xa3',
'!->': '\xe2\x86\x9b',
'HangulSyllableMyeolt': '\xeb\xa9\xbd',
'HangulSyllableMyeolp': '\xeb\xa9\xbe',
'HangulSyllableMyeols': '\xeb\xa9\xbc',
'UnicodeCharacter0xf46c': '\xef\x91\xac',
'superi': '\xe2\x81\xb1',
'HangulSyllableKwet': '\xed\x80\xb1',
'CircledNumberTwelve': '\xe2\x91\xab',
'CanadianSyllabicsWestcreeTwii': '\xe1\x91\x9c',
'YiRadicalGguo': '\xea\x92\xa6',
'theta': '\xce\xb8',
'BoxDrawingsVerticalHeavyAndRightLight': '\xe2\x94\xa0',
'CjkUnifiedIdeograph4fec': '\xe4\xbf\xac',
'ArabicLigatureTahWithMeemWithHahInitialForm': '\xef\xb5\xb2',
'StressOutlinedWhiteStar': '\xe2\x9c\xa9',
'CjkUnifiedIdeograph9de6': '\xe9\xb7\xa6',
'CjkUnifiedIdeograph9de7': '\xe9\xb7\xa7',
'CjkUnifiedIdeograph9de4': '\xe9\xb7\xa4',
'CjkUnifiedIdeograph7497': '\xe7\x92\x97',
'CjkUnifiedIdeograph9de2': '\xe9\xb7\xa2',
'CjkUnifiedIdeograph9de3': '\xe9\xb7\xa3',
'CjkUnifiedIdeograph9de0': '\xe9\xb7\xa0',
'CjkUnifiedIdeograph9de1': '\xe9\xb7\xa1',
'HangulSyllableKwes': '\xed\x80\xab',
'LatinSmallLetterUWithHorn': '\xc6\xb0',
'CjkUnifiedIdeograph9de9': '\xe9\xb7\xa9',
'GreekPsili': '\xe1\xbe\xbf',
'HexagramForTheKeepingStillMountain': '\xe4\xb7\xb3',
'LatinSmallLetterYWithDotBelow': '\xe1\xbb\xb5',
'YiRadicalLie': '\xea\x92\xbb',
'UnicodeCharacter0xe07e': '\xee\x81\xbe',
'ModifierLetterSmallTurnedRWithHook': '\xca\xb5',
'HangulSyllableSsass': '\xec\x8c\x8c',
'BraillePatternDots3467': '\xe2\xa1\xac',
'HangulSyllableWaej': '\xec\x99\xb2',
'BraillePatternDots3468': '\xe2\xa2\xac',
'CjkUnifiedIdeograph7499': '\xe7\x92\x99',
'CombiningCyrillicMillionsSign': '\xd2\x89',
'HebrewLetterGimelWithDagesh': '\xef\xac\xb2',
'YiRadicalJo': '\xea\x92\xb3',
'LatinSmallLetterQWithStrokeThroughDescender': '\xea\x9d\x97',
'TriangleWithDotAbove': '\xe2\xa7\x8a',
'HangulSyllableBbwebs': '\xec\x80\x96',
'HangulSyllableKal': '\xec\xb9\xbc',
'HangulSyllableBabs': '\xeb\xb0\xa6',
'CjkUnifiedIdeograph7d09': '\xe7\xb4\x89',
'CjkUnifiedIdeograph7d08': '\xe7\xb4\x88',
'CjkUnifiedIdeograph7d07': '\xe7\xb4\x87',
'CjkUnifiedIdeograph7d06': '\xe7\xb4\x86',
'CjkUnifiedIdeograph7d05': '\xe7\xb4\x85',
'CjkUnifiedIdeograph7d04': '\xe7\xb4\x84',
'GreekCapitalLetterOmicronWithPsiliAndVaria': '\xe1\xbd\x8a',
'CjkUnifiedIdeograph7d02': '\xe7\xb4\x82',
'CjkUnifiedIdeograph7d01': '\xe7\xb4\x81',
'CjkUnifiedIdeograph7d00': '\xe7\xb4\x80',
'MongolianLetterSibeZa': '\xe1\xa1\xaf',
'UnicodeCharacter0xf857': '\xef\xa1\x97',
'UnicodeCharacter0x9e': '\xc2\x9e',
'UnicodeCharacter0x9d': '\xc2\x9d',
'UnicodeCharacter0x9f': '\xc2\x9f',
'UnicodeCharacter0x9a': '\xc2\x9a',
'ArmenianCapitalLetterPeh': '\xd5\x8a',
'UnicodeCharacter0x9b': '\xc2\x9b',
'CjkCompatibilityIdeographf9e3': '\xef\xa7\xa3',
'SmallAmpersand': '\xef\xb9\xa0',
'MyanmarVowelSignEAbove': '\xe1\x80\xb5',
'UnicodeCharacter0xeac2': '\xee\xab\x82',
'UnicodeCharacter0xe63a': '\xee\x98\xba',
'HangulSyllableKibs': '\xed\x82\xb6',
'HangulSyllableNwet': '\xeb\x89\x95',
'HangulSyllableNwes': '\xeb\x89\x8f',
'CanadianSyllabicsKaa': '\xe1\x91\xb3',
'CombiningEnclosingUpwardPointingTriangle': '\xe2\x83\xa4',
'HangulSyllableNweo': '\xeb\x88\xa0',
'HangulSyllableNwen': '\xeb\x89\x80',
'HangulSyllableNwem': '\xeb\x89\x8c',
'HangulSyllableNwel': '\xeb\x89\x84',
'HangulSyllableNwek': '\xeb\x89\x94',
'HangulSyllableNwej': '\xeb\x89\x92',
'HangulSyllableNweh': '\xeb\x89\x97',
'HangulSyllableNweg': '\xeb\x88\xbd',
'UnicodeCharacter0xe17c': '\xee\x85\xbc',
'HangulSyllableNwed': '\xeb\x89\x83',
'HangulSyllableNwec': '\xeb\x89\x93',
'HangulSyllableNweb': '\xeb\x89\x8d',
'UnicodeCharacter0xef13': '\xee\xbc\x93',
'KatakanaLetterPo': '\xe3\x83\x9d',
'UnicodeCharacter0x99': '\xc2\x99',
'UnicodeCharacter0x98': '\xc2\x98',
'KatakanaLetterPi': '\xe3\x83\x94',
'UnicodeCharacter0x95': '\xc2\x95',
'UnicodeCharacter0x94': '\xc2\x94',
'UnicodeCharacter0x97': '\xc2\x97',
'KatakanaLetterPe': '\xe3\x83\x9a',
'UnicodeCharacter0x91': '\xc2\x91',
'UnicodeCharacter0x90': '\xc2\x90',
'UnicodeCharacter0x93': '\xc2\x93',
'KatakanaLetterPa': '\xe3\x83\x91',
'HangulSyllableHaeb': '\xed\x96\x85',
'WhiteHeartSuit': '\xe2\x99\xa1',
'CjkUnifiedIdeograph7d0f': '\xe7\xb4\x8f',
'CjkUnifiedIdeograph7d0e': '\xe7\xb4\x8e',
'KatakanaLetterPu': '\xe3\x83\x97',
'CjkUnifiedIdeograph7d0c': '\xe7\xb4\x8c',
'CjkUnifiedIdeograph7d0b': '\xe7\xb4\x8b',
'CjkUnifiedIdeograph7d0a': '\xe7\xb4\x8a',
'CjkUnifiedIdeograph4e65': '\xe4\xb9\xa5',
'CjkUnifiedIdeograph4e64': '\xe4\xb9\xa4',
'RomanNumeralOne': '\xe2\x85\xa0',
'CjkUnifiedIdeograph4e66': '\xe4\xb9\xa6',
'CjkUnifiedIdeograph4e61': '\xe4\xb9\xa1',
'CjkUnifiedIdeograph4e60': '\xe4\xb9\xa0',
'CjkUnifiedIdeograph4e63': '\xe4\xb9\xa3',
'CjkUnifiedIdeograph4e62': '\xe4\xb9\xa2',
'CjkUnifiedIdeograph4e69': '\xe4\xb9\xa9',
'CjkUnifiedIdeograph4e68': '\xe4\xb9\xa8',
'CjkCompatibilityIdeographfa85': '\xef\xaa\x85',
'CjkCompatibilityIdeographf9eb': '\xef\xa7\xab',
'CjkCompatibilityIdeographfa87': '\xef\xaa\x87',
'CjkCompatibilityIdeographfa86': '\xef\xaa\x86',
'CjkCompatibilityIdeographfa81': '\xef\xaa\x81',
'CjkUnifiedIdeograph8036': '\xe8\x80\xb6',
'CjkCompatibilityIdeographfa83': '\xef\xaa\x83',
'CjkCompatibilityIdeographfa82': '\xef\xaa\x82',
'IdeographicDescriptionCharacterFullSurround': '\xe2\xbf\xb4',
'UnicodeCharacter0xef19': '\xee\xbc\x99',
'CjkCompatibilityIdeographfa89': '\xef\xaa\x89',
'CjkCompatibilityIdeographfa88': '\xef\xaa\x88',
'CjkUnifiedIdeograph57cc': '\xe5\x9f\x8c',
'KannadaLetterLla': '\xe0\xb2\xb3',
'CjkUnifiedIdeograph57ca': '\xe5\x9f\x8a',
'TeluguVowelSignUu': '\xe0\xb1\x82',
'CjkUnifiedIdeograph57cf': '\xe5\x9f\x8f',
'CjkUnifiedIdeograph57ce': '\xe5\x9f\x8e',
'CjkUnifiedIdeograph57cd': '\xe5\x9f\x8d',
'SuperscriptPlusSign': '\xe2\x81\xba',
'IncreasesAs': '\xe2\xa7\xa1',
'UnicodeCharacter0xe4f1': '\xee\x93\xb1',
'UnicodeCharacter0xe4f0': '\xee\x93\xb0',
'UnicodeCharacter0xe4f3': '\xee\x93\xb3',
'UnicodeCharacter0xe4f2': '\xee\x93\xb2',
'UnicodeCharacter0xe4f5': '\xee\x93\xb5',
'UnicodeCharacter0xe4f4': '\xee\x93\xb4',
'UnicodeCharacter0xe4f7': '\xee\x93\xb7',
'UnicodeCharacter0xe4f6': '\xee\x93\xb6',
'UnicodeCharacter0xe4f9': '\xee\x93\xb9',
'UnicodeCharacter0xe4f8': '\xee\x93\xb8',
'UnicodeCharacter0xf008': '\xef\x80\x88',
'UnicodeCharacter0xf009': '\xef\x80\x89',
'HangulSyllableWael': '\xec\x99\xa4',
'OghamLetterIfin': '\xe1\x9a\x98',
'CjkCompatibilityIdeographfa8e': '\xef\xaa\x8e',
'HangulSyllableCyos': '\xec\xb6\x8b',
'CjkCompatibilityIdeographfa8f': '\xef\xaa\x8f',
'OlChikiGaahlaaTtuddaag': '\xe1\xb1\xb9',
'CjkUnifiedIdeograph8379': '\xe8\x8d\xb9',
'CjkCompatibilityIdeographfa8c': '\xef\xaa\x8c',
'CjkUnifiedIdeograph8d9d': '\xe8\xb6\x9d',
'CjkUnifiedIdeograph9ac1': '\xe9\xab\x81',
'UnicodeCharacter0xd3b': '\xe0\xb4\xbb',
'UnicodeCharacter0xd3c': '\xe0\xb4\xbc',
'UnicodeCharacter0xd3a': '\xe0\xb4\xba',
'CjkUnifiedIdeograph4e6e': '\xe4\xb9\xae',
'CjkUnifiedIdeograph4e6d': '\xe4\xb9\xad',
'CjkUnifiedIdeograph4e6f': '\xe4\xb9\xaf',
'CjkUnifiedIdeograph4e6a': '\xe4\xb9\xaa',
'CjkUnifiedIdeograph4e6c': '\xe4\xb9\xac',
'CjkUnifiedIdeograph4e6b': '\xe4\xb9\xab',
'YiSyllableHnie': '\xea\x85\xae',
'HangulSyllableWaem': '\xec\x99\xac',
'HangulSyllableCwalh': '\xec\xb4\xb3',
'UnicodeCharacter0xe4fa': '\xee\x93\xba',
'UnicodeCharacter0xf00c': '\xef\x80\x8c',
'CjkUnifiedIdeograph68eb': '\xe6\xa3\xab',
'UnicodeCharacter0xe4fb': '\xee\x93\xbb',
'ModifierLetterSmallAlpha': '\xe1\xb5\x85',
'UnicodeCharacter0xe4fd': '\xee\x93\xbd',
'UnicodeCharacter0xf00d': '\xef\x80\x8d',
'UnicodeCharacter0xe4ff': '\xee\x93\xbf',
'MalayalamLetterVocalicLl': '\xe0\xb5\xa1',
'CjkCompatibilityIdeographf9ea': '\xef\xa7\xaa',
'ParenthesizedIdeographNine': '\xe3\x88\xa8',
'CjkUnifiedIdeograph57c3': '\xe5\x9f\x83',
'CjkUnifiedIdeograph57c2': '\xe5\x9f\x82',
'CjkUnifiedIdeograph57c1': '\xe5\x9f\x81',
'CjkUnifiedIdeograph57c0': '\xe5\x9f\x80',
'CjkUnifiedIdeograph57c7': '\xe5\x9f\x87',
'CjkUnifiedIdeograph57c6': '\xe5\x9f\x86',
'CjkUnifiedIdeograph57c5': '\xe5\x9f\x85',
'CjkUnifiedIdeograph57c4': '\xe5\x9f\x84',
'HangulSyllableWaeb': '\xec\x99\xad',
'CjkUnifiedIdeograph57c9': '\xe5\x9f\x89',
'CjkUnifiedIdeograph57c8': '\xe5\x9f\x88',
'CjkUnifiedIdeograph8adc': '\xe8\xab\x9c',
'CjkUnifiedIdeograph8adb': '\xe8\xab\x9b',
'CjkUnifiedIdeograph8ada': '\xe8\xab\x9a',
'VaiSyllableMbe': '\xea\x97\xab',
'CjkUnifiedIdeograph8adf': '\xe8\xab\x9f',
'CjkUnifiedIdeograph8ade': '\xe8\xab\x9e',
'CjkUnifiedIdeograph8add': '\xe8\xab\x9d',
'CjkUnifiedIdeograph49be': '\xe4\xa6\xbe',
'CjkUnifiedIdeograph49bd': '\xe4\xa6\xbd',
'CjkUnifiedIdeograph49bf': '\xe4\xa6\xbf',
'CjkUnifiedIdeograph49ba': '\xe4\xa6\xba',
'IdeographicTelegraphSymbolForDaySix': '\xe3\x8f\xa5',
'CjkUnifiedIdeograph49bc': '\xe4\xa6\xbc',
'CjkUnifiedIdeograph49bb': '\xe4\xa6\xbb',
'HangulSyllablePwags': '\xed\x90\x8b',
'HangulSyllableDdying': '\xeb\x9d\x99',
'HangulSyllableWegg': '\xec\x9b\xaa',
'CjkUnifiedIdeograph62bb': '\xe6\x8a\xbb',
'YiSyllableHnip': '\xea\x85\xab',
'HangulSyllableHyunh': '\xed\x9c\xba',
'HangulSyllableDdyinj': '\xeb\x9d\x89',
'HangulSyllableGyass': '\xea\xb1\x8c',
'CjkUnifiedIdeograph9f4c': '\xe9\xbd\x8c',
'CjkUnifiedIdeograph9f4b': '\xe9\xbd\x8b',
'CjkUnifiedIdeograph9f4a': '\xe9\xbd\x8a',
'CjkUnifiedIdeograph9f4f': '\xe9\xbd\x8f',
'CjkUnifiedIdeograph9f4e': '\xe9\xbd\x8e',
'CjkUnifiedIdeograph9f4d': '\xe9\xbd\x8d',
'HangulSyllableJwaet': '\xec\xa3\x81',
'LatinCapitalLetterOWithCircumflexAndTilde': '\xe1\xbb\x96',
'VaiDigitThree': '\xea\x98\xa3',
'YiSyllableHnit': '\xea\x85\xa8',
'CjkUnifiedIdeograph49b5': '\xe4\xa6\xb5',
'CjkUnifiedIdeograph49b4': '\xe4\xa6\xb4',
'CjkUnifiedIdeograph49b7': '\xe4\xa6\xb7',
'CjkUnifiedIdeograph49b6': '\xe4\xa6\xb6',
'CjkUnifiedIdeograph49b1': '\xe4\xa6\xb1',
'CjkUnifiedIdeograph49b0': '\xe4\xa6\xb0',
'CjkUnifiedIdeograph49b3': '\xe4\xa6\xb3',
'CjkUnifiedIdeograph49b2': '\xe4\xa6\xb2',
'HangulSyllableSsyig': '\xec\x94\x8d',
'HangulSyllableSsyid': '\xec\x94\x93',
'CjkUnifiedIdeograph49b9': '\xe4\xa6\xb9',
'CjkUnifiedIdeograph49b8': '\xe4\xa6\xb8',
'HangulSyllableSsos': '\xec\x8f\xab',
'CjkUnifiedIdeograph8ad9': '\xe8\xab\x99',
'CjkUnifiedIdeograph8ad8': '\xe8\xab\x98',
'GreekSmallLetterTau': '\xcf\x84',
'YiSyllableHnix': '\xea\x85\xa9',
'CjkUnifiedIdeograph8ad3': '\xe8\xab\x93',
'CjkUnifiedIdeograph8ad2': '\xe8\xab\x92',
'CjkUnifiedIdeograph8ad1': '\xe8\xab\x91',
'CjkUnifiedIdeograph8ad0': '\xe8\xab\x90',
'CjkUnifiedIdeograph8ad7': '\xe8\xab\x97',
'CjkUnifiedIdeograph8ad6': '\xe8\xab\x96',
'CjkUnifiedIdeograph8ad5': '\xe8\xab\x95',
'CjkUnifiedIdeograph8ad4': '\xe8\xab\x94',
'CjkUnifiedIdeograph9f43': '\xe9\xbd\x83',
'CjkUnifiedIdeograph9f42': '\xe9\xbd\x82',
'CjkUnifiedIdeograph9f41': '\xe9\xbd\x81',
'CjkUnifiedIdeograph9f40': '\xe9\xbd\x80',
'CjkUnifiedIdeograph9f47': '\xe9\xbd\x87',
'CjkUnifiedIdeograph9f46': '\xe9\xbd\x86',
'CjkUnifiedIdeograph9f45': '\xe9\xbd\x85',
'CjkUnifiedIdeograph9f44': '\xe9\xbd\x84',
'CjkUnifiedIdeograph8a0e': '\xe8\xa8\x8e',
'CjkUnifiedIdeograph9f49': '\xe9\xbd\x89',
'CjkUnifiedIdeograph9f48': '\xe9\xbd\x88',
'HangulSyllableSsom': '\xec\x8f\xa8',
'LatinSmallLetterTum': '\xea\x9d\xb7',
'GreekDialytikaAndVaria': '\xe1\xbf\xad',
'HangulSyllableCyul': '\xec\xb8\x8c',
'HangulSyllableSsok': '\xec\x8f\xb0',
'CjkUnifiedIdeograph7fb3': '\xe7\xbe\xb3',
'CjkUnifiedIdeograph7fb2': '\xe7\xbe\xb2',
'CjkUnifiedIdeograph7fb1': '\xe7\xbe\xb1',
'CjkUnifiedIdeograph7fb0': '\xe7\xbe\xb0',
'CjkUnifiedIdeograph7fb7': '\xe7\xbe\xb7',
'CjkUnifiedIdeograph7fb6': '\xe7\xbe\xb6',
'CjkUnifiedIdeograph7fb5': '\xe7\xbe\xb5',
'CjkUnifiedIdeograph7fb4': '\xe7\xbe\xb4',
'CjkUnifiedIdeograph7fb9': '\xe7\xbe\xb9',
'CjkUnifiedIdeograph7fb8': '\xe7\xbe\xb8',
'UnicodeCharacter0xf1d9': '\xef\x87\x99',
'UnicodeCharacter0xf1d8': '\xef\x87\x98',
'HangulSyllableSsog': '\xec\x8f\x99',
'HangulSyllablePyult': '\xed\x93\xb5',
'MyanmarSignAnusvara': '\xe1\x80\xb6',
'BlackMoonLilith': '\xe2\x9a\xb8',
'EthiopicSyllableCcho': '\xe2\xb6\xbe',
'EthiopicSyllableCchi': '\xe2\xb6\xba',
'EthiopicSyllableCche': '\xe2\xb6\xbd',
'EthiopicSyllableCcha': '\xe2\xb6\xb8',
'CjkUnifiedIdeograph8011': '\xe8\x80\x91',
'CjkUnifiedIdeograph8010': '\xe8\x80\x90',
'CjkUnifiedIdeograph8013': '\xe8\x80\x93',
'CjkUnifiedIdeograph8012': '\xe8\x80\x92',
'CjkUnifiedIdeograph8015': '\xe8\x80\x95',
'CjkUnifiedIdeograph64df': '\xe6\x93\x9f',
'CjkUnifiedIdeograph8017': '\xe8\x80\x97',
'CjkUnifiedIdeograph8016': '\xe8\x80\x96',
'EthiopicSyllableCchu': '\xe2\xb6\xb9',
'CjkUnifiedIdeograph8018': '\xe8\x80\x98',
'HangulSyllableSswam': '\xec\x90\x84',
'HangulSyllableCyus': '\xec\xb8\x97',
'UpwardsHarpoonWithBarbRightwards': '\xe2\x86\xbe',
'IdeographicTelegraphSymbolForDayTwenty': '\xe3\x8f\xb3',
'LatinCapitalLetterYWithLoop': '\xe1\xbb\xbe',
'LatinSmallLetterTWithDiagonalStroke': '\xe2\xb1\xa6',
'HangulSyllableCeoh': '\xec\xb2\xb3',
'GlagoliticCapitalLetterGlagoli': '\xe2\xb0\x83',
'CyrillicSmallLetterU': '\xd1\x83',
'CjkUnifiedIdeograph7fbc': '\xe7\xbe\xbc',
'CyrillicSmallLetterI': '\xd0\xb8',
'CombiningCyrillicLetterShcha': '\xe2\xb7\xb3',
'UnicodeCharacter0xe72a': '\xee\x9c\xaa',
'UnicodeCharacter0xe72f': '\xee\x9c\xaf',
'CjkUnifiedIdeograph7fbf': '\xe7\xbe\xbf',
'CjkUnifiedIdeograph7fbe': '\xe7\xbe\xbe',
'CyrillicSmallLetterO': '\xd0\xbe',
'TrigramForMountain': '\xe2\x98\xb6',
'CyrillicSmallLetterA': '\xd0\xb0',
'HangulSyllableCeok': '\xec\xb2\xb0',
'CyrillicSmallLetterE': '\xd1\x8d',
'HangulSyllableTwaelg': '\xed\x87\xa1',
'CjkUnifiedIdeograph801a': '\xe8\x80\x9a',
'CjkUnifiedIdeograph801c': '\xe8\x80\x9c',
'CjkUnifiedIdeograph801b': '\xe8\x80\x9b',
'CjkUnifiedIdeograph801e': '\xe8\x80\x9e',
'CjkUnifiedIdeograph801d': '\xe8\x80\x9d',
'CjkUnifiedIdeograph801f': '\xe8\x80\x9f',
'LatinSmallLetterKWithDescender': '\xe2\xb1\xaa',
'SucceedsAboveNotAlmostEqualTo': '\xe2\xaa\xba',
'HangulSyllableTwaelh': '\xed\x87\xa7',
'CjkUnifiedIdeograph5140': '\xe5\x85\x80',
'CjkUnifiedIdeograph5141': '\xe5\x85\x81',
'CjkUnifiedIdeograph5142': '\xe5\x85\x82',
'CjkUnifiedIdeograph5143': '\xe5\x85\x83',
'CjkUnifiedIdeograph5144': '\xe5\x85\x84',
'CjkUnifiedIdeograph5145': '\xe5\x85\x85',
'CjkUnifiedIdeograph5146': '\xe5\x85\x86',
'CjkUnifiedIdeograph5147': '\xe5\x85\x87',
'CjkUnifiedIdeograph5148': '\xe5\x85\x88',
'CjkUnifiedIdeograph5149': '\xe5\x85\x89',
'HangulSyllablePyulg': '\xed\x93\xb1',
'UnicodeCharacter0xf828': '\xef\xa0\xa8',
'UnicodeCharacter0xf829': '\xef\xa0\xa9',
'UnicodeCharacter0xf1b9': '\xef\x86\xb9',
'HangulSyllableGyolh': '\xea\xb5\x9f',
'CjkUnifiedIdeograph9650': '\xe9\x99\x90',
'CjkUnifiedIdeograph9651': '\xe9\x99\x91',
'CjkUnifiedIdeograph9652': '\xe9\x99\x92',
'CjkUnifiedIdeograph9653': '\xe9\x99\x93',
'CjkUnifiedIdeograph9654': '\xe9\x99\x94',
'CjkUnifiedIdeograph9655': '\xe9\x99\x95',
'CjkUnifiedIdeograph9656': '\xe9\x99\x96',
'CjkUnifiedIdeograph9657': '\xe9\x99\x97',
'CjkUnifiedIdeograph9658': '\xe9\x99\x98',
'CjkUnifiedIdeograph9659': '\xe9\x99\x99',
'ArabicLetterYehBarreeWithExtendedArabicindicDigitTwoAbove': '\xdd\xba',
'HangulSyllableSweng': '\xec\x89\xa5',
'HangulSyllableKeol': '\xec\xbb\xac',
'SundaneseSignPamaaeh': '\xe1\xae\xaa',
'UnicodeCharacter0xf2b9': '\xef\x8a\xb9',
'HangulSyllablePyi': '\xed\x94\xa0',
'HangulSyllablePyo': '\xed\x91\x9c',
'CjkUnifiedIdeograph93ac': '\xe9\x8e\xac',
'HangulSyllableTwaels': '\xed\x87\xa4',
'LaoLetterNyo': '\xe0\xba\x8d',
'UnicodeCharacter0xf1b8': '\xef\x86\xb8',
'HangulSyllablePye': '\xed\x8f\x90',
'HangulSyllablePyulm': '\xed\x93\xb2',
'ArabicLigatureKhahWithAlefMaksuraIsolatedForm': '\xef\xb4\x83',
'ee': '\xe2\x85\x87',
'HangulSyllablePyu': '\xed\x93\xa8',
'UnicodeCharacter0xfdd9': '\xef\xb7\x99',
'UnicodeCharacter0xf2bf': '\xef\x8a\xbf',
'CyrillicCapitalLetterHaWithStroke': '\xd3\xbe',
'CjkUnifiedIdeograph965b': '\xe9\x99\x9b',
'CjkUnifiedIdeograph965c': '\xe9\x99\x9c',
'CjkUnifiedIdeograph965d': '\xe9\x99\x9d',
'CjkUnifiedIdeograph965e': '\xe9\x99\x9e',
'CjkUnifiedIdeograph965f': '\xe9\x99\x9f',
'UnicodeCharacter0xf2ba': '\xef\x8a\xba',
'YiSyllableBi': '\xea\x80\x98',
'UnicodeCharacter0xe178': '\xee\x85\xb8',
'KangxiRadicalDivination': '\xe2\xbc\x98',
'BlackVerticalEllipse': '\xe2\xac\xae',
'UnicodeCharacter0xf82d': '\xef\xa0\xad',
'CjkUnifiedIdeograph514a': '\xe5\x85\x8a',
'CjkUnifiedIdeograph514b': '\xe5\x85\x8b',
'CjkUnifiedIdeograph514c': '\xe5\x85\x8c',
'CjkUnifiedIdeograph514d': '\xe5\x85\x8d',
'CjkUnifiedIdeograph514e': '\xe5\x85\x8e',
'CjkUnifiedIdeograph514f': '\xe5\x85\x8f',
'LatinSmallLetterZWithHook': '\xc8\xa5',
'YiSyllableBy': '\xea\x80\xb4',
'UnicodeCharacter0xe175': '\xee\x85\xb5',
'UnicodeCharacter0xe174': '\xee\x85\xb4',
'HangulSyllableBbej': '\xeb\xbc\x82',
'HangulSyllableCeugg': '\xec\xb8\xa2',
'EthiopicDigitSeven': '\xe1\x8d\xaf',
'UnicodeCharacter0xe173': '\xee\x85\xb3',
'HangulSyllableCyoss': '\xec\xb6\x8c',
'GreekCapitalLetterOmicronWithDasiaAndVaria': '\xe1\xbd\x8b',
'LatinEpigraphicLetterArchaicM': '\xea\x9f\xbf',
'HangulSyllableDeuls': '\xeb\x93\xa8',
'UnicodeCharacter0xe171': '\xee\x85\xb1',
'UnicodeCharacter0xebea': '\xee\xaf\xaa',
'HangulSyllableNuc': '\xeb\x88\x9b',
'CjkUnifiedIdeograph64d8': '\xe6\x93\x98',
'HangulSyllableBbek': '\xeb\xbc\x84',
'CopticCapitalLetterNi': '\xe2\xb2\x9a',
'LatinCapitalLetterOWithStroke': '\xc3\x98',
'HangulSyllableBbyud': '\xec\x81\x83',
'LatinSmallLetterInsularR': '\xea\x9e\x83',
'RejangLetterKa': '\xea\xa4\xb0',
'EthiopicSyllableJoa': '\xe2\xb6\x8e',
'HangulSyllableJjyebs': '\xec\xaa\xb2',
'CjkUnifiedIdeograph8d9e': '\xe8\xb6\x9e',
'CjkUnifiedIdeograph8d95': '\xe8\xb6\x95',
'CjkUnifiedIdeograph8d9f': '\xe8\xb6\x9f',
'CjkUnifiedIdeograph8d9a': '\xe8\xb6\x9a',
'CjkUnifiedIdeograph8d9c': '\xe8\xb6\x9c',
'CjkUnifiedIdeograph8d9b': '\xe8\xb6\x9b',
'LepchaLetterSa': '\xe1\xb0\xa0',
'CjkUnifiedIdeograph3429': '\xe3\x90\xa9',
'CjkUnifiedIdeograph3428': '\xe3\x90\xa8',
'CjkUnifiedIdeograph3425': '\xe3\x90\xa5',
'CjkUnifiedIdeograph3424': '\xe3\x90\xa4',
'CjkUnifiedIdeograph3427': '\xe3\x90\xa7',
'CjkUnifiedIdeograph3426': '\xe3\x90\xa6',
'CjkUnifiedIdeograph3421': '\xe3\x90\xa1',
'CjkUnifiedIdeograph3420': '\xe3\x90\xa0',
'CjkUnifiedIdeograph3423': '\xe3\x90\xa3',
'CjkUnifiedIdeograph3422': '\xe3\x90\xa2',
'HangulSyllablePyulp': '\xed\x93\xb6',
'UnicodeCharacter0xecc1': '\xee\xb3\x81',
'HangulSyllablePyuls': '\xed\x93\xb4',
'SquareEraNameHeisei': '\xe3\x8d\xbb',
'GreekSmallLetterOmegaWithOxia': '\xe1\xbd\xbd',
'HangulSyllableJeot': '\xec\xa0\x99',
'LatinSmallLetterEshWithRetroflexHook': '\xe1\xb6\x98',
'OlChikiLetterLaa': '\xe1\xb1\x9f',
'HangulSyllablePyulb': '\xed\x93\xb3',
'UnicodeCharacter0xebc9': '\xee\xaf\x89',
'ModifierLetterSmallIWithStroke': '\xe1\xb6\xa4',
'HangulSyllablePyulh': '\xed\x93\xb7',
'EthiopicSyllableGgee': '\xe1\x8c\x9c',
'HangulSyllableBben': '\xeb\xbb\xb0',
'HalfwidthHangulLetterCieuc': '\xef\xbe\xb8',
'ArabicLigatureShaddaWithSuperscriptAlefIsolatedForm': '\xef\xb1\xa3',
'CjkUnifiedIdeograph7f69': '\xe7\xbd\xa9',
'SquaredSmallCircle': '\xe2\xa7\x87',
'CjkUnifiedIdeograph342e': '\xe3\x90\xae',
'CjkUnifiedIdeograph342d': '\xe3\x90\xad',
'CjkUnifiedIdeograph342f': '\xe3\x90\xaf',
'CjkUnifiedIdeograph342a': '\xe3\x90\xaa',
'CjkUnifiedIdeograph342c': '\xe3\x90\xac',
'CjkUnifiedIdeograph342b': '\xe3\x90\xab',
'SundaneseSignPanglayar': '\xe1\xae\x81',
'CjkUnifiedIdeograph8d94': '\xe8\xb6\x94',
'CjkUnifiedIdeograph8d97': '\xe8\xb6\x97',
'CjkUnifiedIdeograph8d96': '\xe8\xb6\x96',
'CjkUnifiedIdeograph8d91': '\xe8\xb6\x91',
'CjkUnifiedIdeograph8d90': '\xe8\xb6\x90',
'CjkUnifiedIdeograph8d93': '\xe8\xb6\x93',
'CjkUnifiedIdeograph8d92': '\xe8\xb6\x92',
'HangulSyllableBbeo': '\xeb\xbb\x90',
'CjkUnifiedIdeograph8d99': '\xe8\xb6\x99',
'CjkUnifiedIdeograph7f65': '\xe7\xbd\xa5'}

ToInterpreter = {
u'\u2083': 'SYMPYSL_SubscriptThree_',
u'\u2204': 'SYMPYSL_ThereDoesNotExist_',
u'\u2087': 'SYMPYSL_SubscriptSeven_',
u'\u2208': 'SYMPYSL_ElementOf_',
u'\u0391': 'SYMPYSL_GreekCapitalLetterAlpha_',
u'\u2210': 'SYMPYSL_NaryCoproduct_',
u'\u2093': 'SYMPYSL_LatinSubscriptSmallLetterX_',
u'\u0395': 'SYMPYSL_GreekCapitalLetterEpsilon_',
u'\u2214': 'SYMPYSL_DotPlus_',
u'\u2297': 'SYMPYSL_CircledTimes_',
u'\u0399': 'SYMPYSL_GreekCapitalLetterIota_',
u'\u2218': 'SYMPYSL_RingOperator_',
u'\u039d': 'SYMPYSL_GreekCapitalLetterNu_',
u'\u221c': 'SYMPYSL_FourthRoot_',
u'\u03a1': 'SYMPYSL_GreekCapitalLetterRho_',
u'\u03a5': 'SYMPYSL_GreekCapitalLetterUpsilon_',
u'\u03a9': 'SYMPYSL_GreekCapitalLetterOmega_',
u'\u2228': ' or ',
u'\u222c': 'SYMPYSL_DoubleIntegral_',
u'\u03b1': 'SYMPYSL_GreekSmallLetterAlpha_',
u'\u2230': 'SYMPYSL_VolumeIntegral_',
u'\xb3': '**3',
u'\u03b5': 'SYMPYSL_GreekSmallLetterEpsilon_',
u'\u03b9': 'SYMPYSL_GreekSmallLetterIota_',
u'\u2238': 'SYMPYSL_DotMinus_',
u'\u03bd': 'SYMPYSL_GreekSmallLetterNu_',
u'\u03c1': 'SYMPYSL_GreekSmallLetterRho_',
u'\u03c5': 'SYMPYSL_GreekSmallLetterUpsilon_',
u'\u2044': 'SYMPYSL_FractionSlash_',
u'\u22c7': 'SYMPYSL_DivisionTimes_',
u'\u2146': 'SYMPYSL_DoublestruckItalicSmallD_',
u'\u03c9': 'SYMPYSL_GreekSmallLetterOmega_',
u'\u03d1': 'SYMPYSL_GreekThetaSymbol_',
u'\u03d5': 'SYMPYSL_GreekPhiSymbol_',
u'\xd7': 'SYMPYSL_MultiplicationSign_',
u'\u03d9': 'SYMPYSL_GreekSmallLetterArchaicKoppa_',
u'\u03dd': 'SYMPYSL_GreekSmallLetterDigamma_',
u'\u03e1': 'SYMPYSL_GreekSmallLetterSampi_',
u'\u2260': '!=',
u'\u1d62': 'SYMPYSL_LatinSubscriptSmallLetterI_',
u'\u2264': '<=',
u'\u1d66': 'SYMPYSL_GreekSubscriptSmallLetterBeta_',
u'\u1d6a': 'SYMPYSL_GreekSubscriptSmallLetterChi_',
u'\u03f1': 'SYMPYSL_GreekRhoSymbol_',
u'\u2070': '**0',
u'\u0372': 'SYMPYSL_GreekCapitalLetterArchaicSampi_',
u'\u03f5': 'SYMPYSL_GreekLunateEpsilonSymbol_',
u'\u2074': '**4',
u'\xf7': 'SYMPYSL_DivisionSign_',
u'\u0376': 'SYMPYSL_GreekCapitalLetterPamphylianDigamma_',
u'\u03f9': 'SYMPYSL_GreekCapitalLunateSigmaSymbol_',
u'\u2078': '**8',
u'\u2c7c': 'SYMPYSL_LatinSubscriptSmallLetterJ_',
u'\u2080': 'SYMPYSL_SubscriptZero_',
u'\u2203': 'SYMPYSL_ThereExists_',
u'\u2084': 'SYMPYSL_SubscriptFour_',
u'\u2207': 'SYMPYSL_Nabla_',
u'\u2088': 'SYMPYSL_SubscriptEight_',
u'\u220f': 'SYMPYSL_NaryProduct_',
u'\u2090': 'SYMPYSL_LatinSubscriptSmallLetterA_',
u'\u0392': 'SYMPYSL_GreekCapitalLetterBeta_',
u'\u0396': 'SYMPYSL_GreekCapitalLetterZeta_',
u'\u2319': 'SYMPYSL_TurnedNotSign_',
u'\u2298': 'SYMPYSL_CircledDivisionSlash_',
u'\u221b': 'SYMPYSL_CubeRoot_',
u'\u039a': 'SYMPYSL_GreekCapitalLetterKappa_',
u'\u039e': 'SYMPYSL_GreekCapitalLetterXi_',
u'\u2227': ' and ',
u'\u03a6': 'SYMPYSL_GreekCapitalLetterPhi_',
u'\u222b': 'SYMPYSL_Integral_',
u'\xac': ' not ',
u'\u222f': 'SYMPYSL_SurfaceIntegral_',
u'\u2233': 'SYMPYSL_AnticlockwiseContourIntegral_',
u'\u03b2': 'SYMPYSL_GreekSmallLetterBeta_',
u'\u03b6': 'SYMPYSL_GreekSmallLetterZeta_',
u'\u03ba': 'SYMPYSL_GreekSmallLetterKappa_',
u'\u03be': 'SYMPYSL_GreekSmallLetterXi_',
u'\u03c2': 'SYMPYSL_GreekSmallLetterFinalSigma_',
u'\u2145': 'SYMPYSL_DoublestruckItalicCapitalD_',
u'\u03c6': 'SYMPYSL_GreekSmallLetterPhi_',
u'\u2149': 'SYMPYSL_DoublestruckItalicSmallJ_',
u'\u03d6': 'SYMPYSL_GreekPiSymbol_',
u'\u03da': 'SYMPYSL_GreekLetterStigma_',
u'\u03de': 'SYMPYSL_GreekLetterKoppa_',
u'\u1d65': 'SYMPYSL_LatinSubscriptSmallLetterV_',
u'\u1d69': 'SYMPYSL_GreekSubscriptSmallLetterPhi_',
u'\u0371': 'SYMPYSL_GreekSmallLetterHeta_',
u'\u2077': '**7',
u'\u03f6': 'SYMPYSL_GreekReversedLunateEpsilonSymbol_',
u'\u03fa': 'SYMPYSL_GreekCapitalLetterSan_',
u'\u037d': 'SYMPYSL_GreekSmallReversedDottedLunateSigmaSymbol_',
u'\u2081': 'SYMPYSL_SubscriptOne_',
u'\u2202': 'SYMPYSL_PartialDifferential_',
u'\u2085': 'SYMPYSL_SubscriptFive_',
u'\u2206': 'SYMPYSL_Increment_',
u'\u2089': 'SYMPYSL_SubscriptNine_',
u'\u2091': 'SYMPYSL_LatinSubscriptSmallLetterE_',
u'\u2310': 'SYMPYSL_ReversedNotSign_',
u'\u0393': 'SYMPYSL_GreekCapitalLetterGamma_',
u'\u2295': 'SYMPYSL_CircledPlus_',
u'\u0397': 'SYMPYSL_GreekCapitalLetterEta_',
u'\u2299': 'SYMPYSL_CircledDotOperator_',
u'\u039b': 'SYMPYSL_GreekCapitalLetterLamda_',
u'\u221a': 'SYMPYSL_SquareRoot_',
u'\u039f': 'SYMPYSL_GreekCapitalLetterOmicron_',
u'\u221e': 'SYMPYSL_Infinity_',
u'\u03a3': 'SYMPYSL_GreekCapitalLetterSigma_',
u'\u03a7': 'SYMPYSL_GreekCapitalLetterChi_',
u'\u222a': 'SYMPYSL_Union_',
u'\u222e': 'SYMPYSL_ContourIntegral_',
u'\u03b3': 'SYMPYSL_GreekSmallLetterGamma_',
u'\u2232': 'SYMPYSL_ClockwiseContourIntegral_',
u'\u03b7': 'SYMPYSL_GreekSmallLetterEta_',
u'\xb9': '**1',
u'\u03bb': 'SYMPYSL_GreekSmallLetterLamda_',
u'\u03bf': 'SYMPYSL_GreekSmallLetterOmicron_',
u'\u03c3': 'SYMPYSL_GreekSmallLetterSigma_',
u'\u22c5': 'SYMPYSL_DotOperator_',
u'\u03c7': 'SYMPYSL_GreekSmallLetterChi_',
u'\u2148': 'SYMPYSL_DoublestruckItalicSmallI_',
u'\u03cf': 'SYMPYSL_GreekCapitalKaiSymbol_',
u'\u03d7': 'SYMPYSL_GreekKaiSymbol_',
u'\u03db': 'SYMPYSL_GreekSmallLetterStigma_',
u'\u03df': 'SYMPYSL_GreekSmallLetterKoppa_',
u'\u1d64': 'SYMPYSL_LatinSubscriptSmallLetterU_',
u'\u1d68': 'SYMPYSL_GreekSubscriptSmallLetterRho_',
u'\u0370': 'SYMPYSL_GreekCapitalLetterHeta_',
u'\u03f3': 'SYMPYSL_GreekLetterYot_',
u'\u03f7': 'SYMPYSL_GreekCapitalLetterSho_',
u'\u2076': '**6',
u'\u03fb': 'SYMPYSL_GreekSmallLetterSan_',
u'\u037c': 'SYMPYSL_GreekSmallDottedLunateSigmaSymbol_',
u'\u2201': 'SYMPYSL_Complement_',
u'\u2082': 'SYMPYSL_SubscriptTwo_',
u'\u2205': 'SYMPYSL_EmptySet_',
u'\u2086': 'SYMPYSL_SubscriptSix_',
u'\u2209': 'SYMPYSL_NotAnElementOf_',
u'\u2211': 'SYMPYSL_NarySummation_',
u'\u2092': 'SYMPYSL_LatinSubscriptSmallLetterO_',
u'\u0394': 'SYMPYSL_GreekCapitalLetterDelta_',
u'\u2296': 'SYMPYSL_CircledMinus_',
u'\u2219': 'SYMPYSL_BulletOperator_',
u'\u0398': 'SYMPYSL_GreekCapitalLetterTheta_',
u'\u039c': 'SYMPYSL_GreekCapitalLetterMu_',
u'\u03a0': 'SYMPYSL_GreekCapitalLetterPi_',
u'\u03a4': 'SYMPYSL_GreekCapitalLetterTau_',
u'\u2229': 'SYMPYSL_Intersection_',
u'\u03a8': 'SYMPYSL_GreekCapitalLetterPsi_',
u'\u222d': 'SYMPYSL_TripleIntegral_',
u'\u2231': 'SYMPYSL_ClockwiseIntegral_',
u'\xb2': '**2',
u'\u03b4': 'SYMPYSL_GreekSmallLetterDelta_',
u'\u03b8': 'SYMPYSL_GreekSmallLetterTheta_',
u'\u03bc': 'SYMPYSL_GreekSmallLetterMu_',
u'\u03c0': 'SYMPYSL_GreekSmallLetterPi_',
u'\u03c4': 'SYMPYSL_GreekSmallLetterTau_',
u'\u2147': 'SYMPYSL_DoublestruckItalicSmallE_',
u'\u22c6': 'SYMPYSL_StarOperator_',
u'\u03c8': 'SYMPYSL_GreekSmallLetterPsi_',
u'\u03d0': 'SYMPYSL_GreekBetaSymbol_',
u'\u03d8': 'SYMPYSL_GreekLetterArchaicKoppa_',
u'\u03dc': 'SYMPYSL_GreekLetterDigamma_',
u'\u03e0': 'SYMPYSL_GreekLetterSampi_',
u'\u1d63': 'SYMPYSL_LatinSubscriptSmallLetterR_',
u'\u2265': '>=',
u'\u1d67': 'SYMPYSL_GreekSubscriptSmallLetterGamma_',
u'\u03f0': 'SYMPYSL_GreekKappaSymbol_',
u'\u0373': 'SYMPYSL_GreekSmallLetterArchaicSampi_',
u'\u2075': '**5',
u'\u0377': 'SYMPYSL_GreekSmallLetterPamphylianDigamma_',
u'\u2079': '**9',
u'\u03f8': 'SYMPYSL_GreekSmallLetterSho_',
u'\u037b': 'SYMPYSL_GreekSmallReversedLunateSigmaSymbol_'}

